<?php
defined('BASEPATH') OR exit('No direct script access allowed');

/*
| -------------------------------------------------------------------------
| URI ROUTING
| -------------------------------------------------------------------------
| This file lets you re-map URI requests to specific controller functions.
|
| Typically there is a one-to-one relationship between a URL string
| and its corresponding controller class/method. The segments in a
| URL normally follow this pattern:
|
|	example.com/class/method/id/
|
| In some instances, however, you may want to remap this relationship
| so that a different class/function is called than the one
| corresponding to the URL.
|
| Please see the user guide for complete details:
|
|	http://codeigniter.com/user_guide/general/routing.html
|
| -------------------------------------------------------------------------
| RESERVED ROUTES
| -------------------------------------------------------------------------
|
| There are three reserved routes:
|
|	$route['default_controller'] = 'welcome';
|
| This route indicates which controller class should be loaded if the
| URI contains no data. In the above example, the "welcome" class
| would be loaded.
|
|	$route['404_override'] = 'errors/page_missing';
|
| This route will tell the Router which controller/method to use if those
| provided in the URL cannot be matched to a valid route.
|
|	$route['translate_uri_dashes'] = FALSE;
|
| This is not exactly a route, but allows you to automatically route
| controller and method names that contain dashes. '-' isn't a valid
| class or method name character, so it requires translation.
| When you set this option to TRUE, it will replace ALL dashes in the
| controller and method URI segments.
|
| Examples:	my-controller/index	-> my_controller/index
|		my-controller/my-method	-> my_controller/my_method
*/

$route['default_controller'] = 'home';
$route['404_override'] = '';
$route['translate_uri_dashes'] = false;

// Authentication
Route::any('login', 'users/login', array('as' => 'login'));
Route::any(ADMIN_URL, 'users/restrictedZone', array('as' => 'login'));
Route::any('register', 'users/register', array('as' => 'register'));
Route::block('users/login');
Route::block('users/register');

Route::any('logout', 'users/logout');
Route::any('forgot_password', 'users/forgot_password');
Route::any('reset_password/(:any)/(:any)', 'users/reset_password/$1/$2');

// Ajax Login
Route::any('signin', 'users/ajax_login');
Route::any('forgotPassword', 'users/ajax_forgot_password');
Route::any('signup', 'users/ajax_register');
Route::any('users/ajax_login', '');
Route::any('users/ajax_forgot_password', '');

// Activation
// Route::any('activate', 'users/activate');
Route::any('activate/(:any)/(:any)', 'users/activate/$1/$2');
// Route::any('activate/(:any)', 'users/activate/$1');
Route::any('resend_activation/(:any)', 'users/resend_activation/$1');
Route::any('verify_otp/(:any)', 'users/verify_otp/$1');

// Contexts
Route::prefix(SITE_AREA, function() {
	Route::context('dashboard');
    Route::context('content', array('home' => SITE_AREA .'/content/index'));
    Route::context('financial');
    Route::context('components');
    Route::context('reports', array('home' => SITE_AREA .'/reports/index'));
    Route::context('tools');
    Route::context('settings');
    Route::context('developer');
    Route::context('help');
});

$route['pages/(:any)']	= 'pages/detail/$1';
$route['news/(:any)']	= 'news/detail/$1';
$route['contact'] = 'pages/contact';
$route['contactQuery']	= 'ajax/contact_query';
$route['categories'] = 'listings/categories';
$route['locations']	= 'listings/locations';
$route['location/(:any)/(:any)'] = 'listings/location/$1/$2';
$route['location/(:any)'] = 'listings/location/$1';
$route['tag/(:any)/(:any)'] = 'listings/tag/$1/$2';
$route['tag/(:any)'] = 'listings/tag/$1';
$route['search'] = 'listings/search';
$route['search/(:any)'] = 'listings/search/$1';
$route['detail/(:any)']	= 'listings/detail/$1';
$route['detail/(:any)/(:any)']	= 'listings/detail/$1/$2';
$route['users/detail/(:any)'] = 'users/detail/$1';
$route['requirements'] = 'leads/index';
$route['unsubscribe/(:any)/(:any)'] = 'emailer/unsubscribe/$1/$2';
$route['unsubscribe/(:any)'] = 'emailer/unsubscribe/$1';
$route['unsubscribe'] = 'emailer/unsubscribe';
$route['createPDF/(:any)'] = 'listings/create_pdf/$1';
$route['createPDF/(:any)/(:any)'] = 'listings/create_pdf/$1/$2';
//$route['sitemap'] = 'listings/sitemap';

// Ajax Queries
$route['businessQuery']	= 'ajax/business_query';
$route['claim']	= 'ajax/claim';
$route['clickToCall'] = 'ajax/clickToCall';
$route['sendSMS'] = 'ajax/sendSMS';
$route['sendEmail']	= 'ajax/sendEmail';
$route['review'] = 'ajax/review';
$route['bookmark'] = 'ajax/favorites';
$route['erroneous']	= 'ajax/erroneous';
$route['getTags'] = 'ajax/tags_search';
$route['leads'] = 'ajax/leads';
$route['starRating'] = 'ajax/star_rating';
$route['securimage'] = 'captcha/index';
$route['loadSubCategories'] = 'ajax/loadSubCategories';
$route['loadCities'] = 'ajax/loadCities';
$route['citySearch'] = 'ajax/cities_search';
$route['localitySearch'] = 'ajax/localities_search';
$route['updateBannerImpressions'] = 'ajax/updateBannerImpressions';
$route['updateBannerClicks'] = 'ajax/updateBannerClicks';
$route['subscribers'] = 'ajax/subscribers';
$route['getKeyword'] = 'ajax/ajax_search';
$route['getLocation'] = 'ajax/ajax_search_location';
$route['getQuery'] = 'ajax/ajax_module_search';
$route['ajaxSelection'] = 'ajax/ajax_selection';
$route['replyQuery'] = 'ajax/replyQuery';
$route['queryReplies'] = 'ajax/query_replies';
$route['updateOrder'] = 'ajax/updateOrder';

// Member Area
$route['members'] = 'members/index';
$route['members/pages/(:any)']	= 'members/pages/$1';
$route['members/(:num)'] = 'members/index/$1';
$route['members/add_business'] = 'members/add_business';
$route['members/edit_business']	= 'members/edit_business';
$route['members/view_business_queries']	= 'members/view_business_queries';
$route['members/view_business_queries/(:any)'] = 'members/view_business_queries/$1';
$route['members/view_business_queries/(:any)/(:any)'] = 'members/view_business_queries/$1/$2';
$route['members/posted_queries'] = 'members/posted_queries';
$route['members/posted_queries/(:any)'] = 'members/posted_queries/$1';
$route['members/posted_queries/(:any)/(:any)'] = 'members/posted_queries/$1/$2';
$route['members/view_comments']	= 'members/view_comments';
$route['members/view_comments/(:any)']	= 'members/view_comments/$1';
$route['members/view_comments/(:any)/(:any)']	= 'members/view_comments/$1/$2';
$route['members/my_comments'] = 'members/my_comments';
$route['members/my_comments/(:any)'] = 'members/my_comments/$1';
$route['members/my_comments/(:any)/(:any)'] = 'members/my_comments/$1/$2';
$route['members/orders'] = 'members/orders';
$route['members/orders/(:any)'] = 'members/orders/$1';
$route['members/invoices'] = 'members/invoices';
$route['members/invoices/(:any)'] = 'members/invoices/$1';
$route['members/pay_invoice'] = 'members/pay_invoice';
$route['members/pay_invoice/(:any)'] = 'members/pay_invoice/$1';
$route['members/view_invoice'] = 'members/view_invoice';
$route['members/view_invoice/(:any)'] = 'members/view_invoice/$1';
$route['members/download_invoice'] = 'members/download_invoice';
$route['members/download_invoice/(:any)'] = 'members/download_invoice/$1';
$route['members/favorites']	= 'members/favorites';
$route['members/favorites/(:any)']	= 'members/favorites/$1';
$route['members/favorites/(:any)/(:any)']	= 'members/favorites/$1/$2';
$route['members/affiliate']	= 'members/affiliate';
$route['members/invitation_coupons']	= 'members/invitation_coupons';
$route['members/membership'] = 'members/membership';
$route['members/packages'] = 'members/packages';
$route['members/checkout'] = 'members/checkout';
$route['orders/checkout'] = 'orders/checkout';
$route['members/renew'] = 'members/renew';
$route['members/upgrade'] = 'members/upgrade';
$route['members/logo'] = 'members/logo';
$route['membres/cover'] = 'members/cover';
$route['members/photos/(:any)/(:any)'] = 'members/photos/$1/$2';
$route['members/photos/(:any)'] = 'members/photos/$1';
$route['members/photos'] = 'members/photos';
$route['members/business_hours'] = 'members/business_hours';
$route['members/wallets/(:any)'] = 'members/wallets/$1';
$route['members/wallets'] = 'members/wallets';
$route['members/points/(:any)'] = 'members/points/$1';
$route['members/points'] = 'members/points';
$route['users/profile'] = 'users/profile';
$route['members/view'] = 'members/view';
$route['members/add'] = 'members/add';
$route['members/edit'] = 'members/edit';
$route['members/category'] = 'members/category';
$route['members/ordered_items/(:any)/(:any)']	= 'members/ordered_items/$1/$2';
$route['members/ordered_items/(:any)']	= 'members/ordered_items/$1';
$route['members/ordered_items'] = 'members/ordered_items';
$route['members/shipments'] = 'members/shipments';
$route['members/track'] = 'members/track';
$route['members/wishlists'] = 'members/wishlists';
$route['members/notifications'] = 'members/notifications';
$route['members/payouts'] = 'members/payouts';
$route['members/variants/(:any)/(:any)'] = 'members/variants/$1/$2';
$route['members/my_requirements'] = 'members/my_requirements';
$route['members/documents/(:any)/(:any)']	= 'members/documents/$1/$2';

// Bookings Module
$route['booking_form/(:any)/(:any)'] = 'bookings/booking_form/$1/$2';
$route['booking_form/(:any)'] = 'bookings/booking_form/$1';
$route['bookings/forms'] = 'bookings/forms';
$route['bookings/fields'] = 'bookings/fields';
$route['bookings/options'] = 'bookings/options';
$route['bookings/packages'] = 'bookings/packages';
$route['bookings/settings'] = 'bookings/settings';
$route['bookings/view/(:any)/(:any)/(:any)'] = 'bookings/view/$1/$2/$3';
$route['bookings/view/(:any)/(:any)'] = 'bookings/view/$1/$2';
$route['bookings/view/(:any)'] = 'bookings/view/$1';
$route['bookings/view'] = 'bookings/view';
$route['bookings/export/(:any)'] = 'bookings/export/$1';
$route['bookings/cancellation/(:any)'] = 'bookings/cancellation/$1';
$route['bookings/settings/(:any)'] = 'bookings/settings/$1';

// Leads Module
$route['leads/view/(:any)/(:any)'] = 'leads/view/$1/$2';
$route['leads/view/(:any)'] = 'leads/view/$1';
$route['leads/view'] = 'leads/view';
// Ads Module
$route['ads/view/(:any)/(:any)'] = 'ads/view/$1/$2';
$route['ads/view/(:any)'] = 'ads/view/$1';
$route['ads/packages'] = 'ads/packages';
$route['ads/edit'] = 'ads/edit';
$route['advertise'] = 'ads/advertise';
$route['ads/click'] = 'ads/click';
$route['ads/(:any)'] = 'ads/index/$1';
$route['ads'] = 'ads/index';
$route['forums'] = 'forums';

// Emailer
$route['logo'] = 'emailer/logo';
$route['visit'] = 'emailer/visit';;

// Modules
$route['blogs/category/(:any)'] = 'listings/view/blogs/categories/$1';
$route['blogs/location/(:any)'] = 'listings/view/blogs/location/$1';
$route['blogs/detail/(:any)'] = 'listings/describe/blogs/$1';
$route['blogs'] = 'listings/view/blogs/search';

$route['deals/category/(:any)'] = 'listings/view/deals/categories/$1';
$route['deals/location/(:any)'] = 'listings/view/deals/location/$1';
$route['deals/detail/(:any)'] = 'listings/describe/deals/$1';
$route['deals'] = 'listings/view/deals/search';

$route['classifieds/category/(:any)'] = 'listings/view/classifieds/categories/$1';
$route['classifieds/location/(:any)'] = 'listings/view/classifieds/location/$1';
$route['classifieds/detail/(:any)'] = 'listings/describe/classifieds/$1';
$route['classifieds'] = 'listings/view/classifieds/search';

$route['events/category/(:any)'] = 'listings/view/events/categories/$1';
$route['events/location/(:any)'] = 'listings/view/events/location/$1';
$route['events/detail/(:any)'] = 'listings/describe/events/$1';
$route['events'] = 'listings/view/events/search';

$route['jobs/category/(:any)'] = 'listings/view/jobs/categories/$1';
$route['jobs/location/(:any)'] = 'listings/view/jobs/location/$1';
$route['jobs/detail/(:any)'] = 'listings/describe/jobs/$1';
$route['jobs'] = 'listings/view/jobs/search';

$route['products/category/(:any)'] = 'listings/view/products/categories/$1';
$route['products/location/(:any)'] = 'listings/view/products/location/$1';
$route['products/detail/(:any)'] = 'listings/describe/products/$1';
$route['products'] = 'listings/view/products/search';

$route['properties/category/(:any)'] = 'listings/view/properties/categories/$1';
$route['properties/location/(:any)'] = 'listings/view/properties/location/$1';
$route['properties/detail/(:any)'] = 'listings/describe/properties/$1';
$route['properties'] = 'listings/view/properties/search';

$route['videos/category/(:any)'] = 'listings/view/videos/categories/$1';
$route['videos/location/(:any)'] = 'listings/view/videos/location/$1';
$route['videos/detail/(:any)'] = 'listings/describe/videos/$1';
$route['videos'] = 'listings/view/videos/search';

// Affiliates Module
$route['ref/detail/(:any)'] = 'affiliates/cloaking/$1'; // Affiliate Link Cloaking
$route['ref/(:any)'] = 'affiliates/cloaking/$1'; // Affiliate Link Cloaking
$route['affiliates/view_invoice']	= 'affiliates/view_invoice';
$route['affiliates']	= 'affiliates';
$route['country'] = 'listings/change_country';
$route['countries'] = 'listings/countries_list';
$route['countries/(:any)'] = 'listings/countries_list/$1';

// Categories
$route['category/(:any)/(:any)/(:any)/(:any)/(:any)/(:any)/(:any)'] = 'listings/category/$1/$2/$3/$4/$5/$6/$7';
$route['category/(:any)/(:any)/(:any)/(:any)/(:any)/(:any)'] = 'listings/category/$1/$2/$3/$4/$5/$6';
$route['category/(:any)/(:any)/(:any)/(:any)/(:any)'] = 'listings/category/$1/$2/$3/$4/$5';
$route['category/(:any)/(:any)/(:any)/(:any)'] = 'listings/category/$1/$2/$3/$4';
$route['category/(:any)/(:any)/(:any)'] = 'listings/category/$1/$2/$3';
$route['category/(:any)/(:any)'] = 'listings/category/$1/$2';
$route['category/(:any)'] = 'listings/category/$1';

// Ads
$route['space'] = 'ads/space';

// Property Investment
$route['property-investment/(:any)/(:any)'] = 'projects/detail/$1/$2';
$route['property-investment'] = 'projects';

$route = Route::map($route);