<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/**
 * This controller is created for all jquery functions
 * jQuery requests call Front-Controller.php because of extends method
 * which leads in unnecessary execution of database queries
 * 
 * @author GoClixy
 *
 */
class Ajax extends Base_Controller {
	protected $z_cache;
	public function __construct() {
		parent::__construct();		
		
		$this->load->model('listings/listings_model');
		$this->load->model('listings/ajax_model');
		$this->load->library('parser');
		$this->load->library('form_validation');
		$this->load->helper('security');
		$this->load->helper('gc');
		$this->load->library('gc');
		$this->load->library('goclixy/gcache');
		$this->load->library('goclixy/gemailer');
		$this->load->library('translations');
		$this->load->library('users/auth');
		$this->lang->load('routes', 'english');
		$this->lang->load('application', $this->session->userdata('language'));
		$this->lang->load('goclixy', $this->session->userdata('language'));
		$this->lang->load('listings/listings', $this->session->userdata('language'));
				
		// Load Zend Caching
		if(settings_item('site.enable_cache') == 1) {			
			$this->load->library('gzf_cache', array('lifetime' => 900));
			$this->z_cache = $this->gzf_cache->get_instance();
			$this->load->library('slug');
		}
	}
	
	/**
	 * Allow User to claim a listing
	 * Get amount from the package_id (members/checkout method)
	 */
	public function claim() {
		$this->form_validation->set_rules('listing_id', 'lang:label_listing_id', 'xss_clean|required');
		$this->form_validation->set_rules('price', 'lang:label_price', 'xss_clean');
		if ($this->form_validation->run() === FALSE) {
			$validation_errors = validation_errors();
			if ($validation_errors) {
				$validation_error = $validation_errors;
			}
			echo json_encode(array('result' => 0, 'message' => $validation_error, 'type' => 'error'));
		} else {
			$sql = "SELECT COUNT(*) AS count FROM " . $this->db->dbprefix . "order_items OT WHERE OT.user_id = " . $this->auth->user_id() . " AND OT.module_id = " . gc_module_id('packages') . " AND OT.status_id = 2 AND (OT.expires_on IS NULL OR OT.expires_on >= '". date('Y-m-d') ."')";
			$result = $this->db->query($sql)->row();
			if($result->count > 0) {
			    $this->load->model('orders/orders_model');
			    $listing_id = $this->input->post('listing_id');
			    $this->orders_model->cart($listing_id, 0, 1, 1, 'claims', true); // Claims
				// $this->session->set_userdata('order_type', 'claims');
				// $this->session->set_userdata('claimed_listing_id', $this->input->post('listing_id'));
				echo json_encode(array('result' => 1, 'message' => lang('msg_checkout_redirection'), 'type' => 'success', 'url' => gc_url_format('members/checkout')));
			} else if($result->count == 0) {
				echo json_encode(array('result' => 0, 'message' => lang('msg_active_membership_claim'), 'type' => 'error'));
			}
		}
	}
	
	/**
	 * Submit Contact Query
	 */
	public function contact_query() {
	    $this->load->model('contact_queries/contact_queries_model');
	    echo json_encode($this->contact_queries_model->contact_query($this->auth->user_id()));
	}	
	
	/**
	 * BUSINESS ENQUIRY FORM
	 * Send enquiry to business owner
	 *
	 */
	public function business_query() {
	    $this->ajax_model->send_query($this->auth->user_id());
	}
	
	/**
	 * LEADS FORM
	 * Save customer needs form data to database
	 *
	 */
	public function leads() {
		$this->load->model('leads/leads_model');
		$result = $this->leads_model->save_leads($this->auth->user_id());
		echo json_encode($result);
	}
	
	/**
	 * Generate Random Value
	 */
	public function random() {
	    echo base64_encode(rand(8888,9999));
	}
	
	/**
	 * COMMENT FORM
	 * Allow users to post comment on listing
	 */
	public function review() {
	    $this->load->model('comments/comments_model');
	    echo json_encode($this->comments_model->save_review($this->auth->user_id()));
	}
	
	/**
	 * Erroneous FORM
	 * Allow users to claim or send report for the incorrect information
	 * Called by AJAX
	 */
	public function erroneous($is_mobile = false, $rest_extra = array()) {
	    $this->load->model('erroneous/erroneous_model');
	    echo json_encode($this->erroneous_model->report($this->auth->user_id()));
	}
	
	/**
	 * Display phone number to users on click to call
	 */
	public function clickToCall() {
		$data['device'] = $this->input->post("device");
		$data['listing_id'] = $this->input->post("listing_id");
		$data['ip_address'] = $this->input->ip_address();
		$data['created_on'] = date('Y-m-d H:i:s');
		$this->load->model('click_to_call/click_to_call_model');
		if($this->session->userdata('user_id')) {
			$data['user_id'] = $this->session->userdata('user_id');
		} else {
			$data['user_id'] = 0;
		}
		$sql = $this->db->insert_string($this->db->dbprefix .'click_to_call', $data) . ' ON DUPLICATE KEY UPDATE counts = counts + 1';
		$this->db->query($sql);
		$this->db->insert_id();
	}
	
	/**
	 * Allow users to send SMS to themselves or friend
	 * @return string
	 *
	 */
	public function sendSMS() {
		$this->ajax_model->send_sms();
	}
	
	/**
	 * Admin Panel - Custom SMS Messages
	 * @return string
	 */
	public function sms() {
		$this->bulk_send('sms');
	}
	
	/**
	 * Admin Panel - Custom/Bulk Email
	 * @return string
	 */
	public function email() {
		$this->bulk_send('email');	
	}
	
	/**
	 * Admin Panel Bulk Send Information
	 */
	private function bulk_send($action = 'email') {
		$this->form_validation->set_rules('module', 'lang:label_module', 'required|xss_clean');
		$this->form_validation->set_rules('row_ids', 'lang:label_id', 'required|xss_clean');
		if($action == 'email') {
		    $this->form_validation->set_rules('email_subject', 'lang:label_subject', 'required|xss_clean');
		    $this->form_validation->set_rules('email_message', 'lang:label_message', 'required|xss_clean');
		} else {		    
		    $this->form_validation->set_rules('sms_message', 'lang:label_message', 'required|xss_clean');
		}
		
		if ($this->form_validation->run() === FALSE) {
			$validation_errors = validation_errors();
			if ($validation_errors) {
				$validation_error = $validation_errors;
			}
			echo json_encode(array('result' => 0, 'message' => $validation_error, 'type' => 'danger'));
		} else {
			$this->load->model('emailer/emailer_model');
			$module = escape($this->input->post('module'));
			$row_ids = escape($this->input->post('row_ids'));
			$subject = $this->input->post('email_subject');
			$message = $action == 'email' ? $this->input->post('email_message') : $this->input->post('sms_message');
			$result = $this->emailer_model->bulk_send($module, $row_ids, 0, $subject, $message, false, null, $action, false);
			$success = $result['success'];
			if($success) {
				echo json_encode(array('result' => 1, 'message' => lang('success_message'), 'type' => 'success'));
			} else {
			    echo json_encode(array('result' => 0, 'message' => lang($action . '_error'), 'type' => 'danger'));
			}
		}
	}
	
	/**
	 * Allow users to send Email
	 * @return string
	 */
	public function sendEmail() {
		$this->form_validation->set_rules('email_to', 'lang:label_friends_email', 'required|valid_email|xss_clean');
		$this->form_validation->set_rules('email_from', 'lang:label_your_email', 'required|valid_email|xss_clean');
		$this->form_validation->set_rules('email_from_name', 'lang:label_name', 'required|xss_clean');
		$this->form_validation->set_rules('email_captcha_code', 'lang:label_captcha', 'xss_clean|required|valid_captcha');
	
		if ($this->form_validation->run() === FALSE) {
			$validation_errors = validation_errors();
			if ($validation_errors) {
				$validation_error = $validation_errors;
			}
			echo json_encode(array('result' => 0, 'message' => $validation_error, 'type' => 'danger'));
		} else {
			$module = ($this->input->post('module') ? escape($this->input->post('module')) : 'listings');
			$id = (int) $this->input->post('row_id');
			// Send Module Email
			$from = escape($this->input->post('email_from_name'));
			$email_to = $this->input->post('email_to');
			$this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'module_email_templates', 'module_table' => $module, 'action_type' => 'share', 'with_default' => 1, 'id' => $id, 'email_to' => $email_to));	
			echo json_encode(array('result' => 1, 'message' => lang('success_message'), 'type' => 'success'));
		}
	}
	
	/**
	 * Allow users to rate listing
	 * @param int $listing_id
	 * @return string
	 *
	 */
	public function star_rating() {
		if (settings_item('lst.loggedin_rating_only') == 1 && $this->auth->is_logged_in()) { // Only Loggedin Users can Rate a Business
			echo json_encode ( $this->rate_business() );
		} else if (settings_item('lst.loggedin_rating_only') == 1 && !$this->auth->is_logged_in()) {
			echo json_encode ( array ("label" => 'danger', "result" => lang('error_loggedin_rating_only') . ' <br /><a class="text-white" href="' . gc_url_format('login') .'">'.lang('msg_click_here_login').'</a>') );
		} else if (settings_item('lst.loggedin_rating_only') == 0) {
			echo json_encode ( $this->rate_business() );
		}
	}
	
	/**
	 * Rate Business
	 * @return string
	 */
	private function rate_business() {
		$module = escape($this->input->post('module'));
		$row_id = (int) $this->input->post('row_id');
		header('Content-Type: application/x-json; charset=utf-8');
		$ip_address = $this->input->ip_address();
		$new_rating = $this->input->post('ratings', true);
		if(empty($row_id)) {
		    return false;
		}
		$data = array (
				'row_id' => $row_id,
				'module_id' => gc_module_id($module),
				'ip_address' => $ip_address,
				'new_rating' => $new_rating
		);
		$result = $this->gc->star_rating($data, $this->auth->user_id());
			
		$message = array();
		switch($result) {
			case 0:
				break;
			case 1:
			case 4:
				$message = array(
    				'label' => 'success',
    				'result' => lang('success_rating')
				);
				break;
			case 2:
			case 5:
				$message = array(
					'label' => 'danger',
					'result' => sprintf(lang('error_rating'), lang('singular_' . $module))
				);
				break;
			case 3:
				$message = array(
					'label' => 'danger',
					'result' => sprintf(lang('failure_rating'), lang('singular_' . $module))
				);
				break;
		}
		return $message;
	}
	
	/**
	 * Allow user's to add module to favorites
	 * jQuery CallBack
	 */
	public function favorites() {
		$row_id = (int)$this->input->post('row_id');
		$user_id = escape($this->auth->user_id());
		$module = escape($this->input->post('module'));
		$isFavorite = escape($this->input->post('isFavorite'));
		$this->load->model('favorites/favorites_model');
		if($isFavorite == 0) {
			$data = array(
				'user_id' => $user_id,
				'row_id' => $row_id,
				'module_id' => gc_module_id($module)
			);
			$count = $this->favorites_model->count_by($data);
			if($count <= 0) {
				$data['created_ip'] = $this->input->ip_address();
				$this->favorites_model->insert($data);
			}			
			echo json_encode(array('result' => 1, 'type' => 'success', 'message' => lang('favorite_added')));
		} else {
			$this->db->where(array('user_id' => $user_id, 'row_id' => $row_id, 'module_id' => gc_module_id($module)))->delete('favorites');
			echo json_encode(array('result' => 0, 'type' => 'success', 'message' => lang('favorite_removed')));
		}
	}
	
	/**
	 * Save Subscribers
	 */
	public function subscribe() {
	    $this->load->model('subscribers/subscribers_model');
	    $this->subscribers_model->save_subscriber();
	}
	
	/**
	 * BANNER CLICKS
	 * Update banner hits on user click
	 * Used by AJAX
	 */
	public function updateBannerClicks() {
		$id = $this->input->post('id');
		$this->db->where('id', $id);
		$this->db->set('clicks', 'clicks + 1', FALSE);
		$this->db->update('banners');
	}
	
	/**
	 * BANNER IMPRESSIONS
	 * Update banner impressions on mouse over
	 * Used by AJAX
	 */
	public function updateBannerImpressions() {
		$id = (int) $this->input->post('id');
		/* The below code is also slow
		 * $banner = $this->db->select('impressions')->where('id', $id)->get('banners')->row();
		if($banner) {
		    $this->db->where('id', $id)->update('banners', array('impressions' => $banner->impressions + 1));
		}*/
		$this->db->where('id', $id);
		$this->db->set('impressions', 'impressions + 1', FALSE);
		$this->db->update('banners');
	}

	/**
	 * Ajax function to loadCities on user click
	 */
	public function loadCities() {
		$state_id = $this->input->post('id');
		$module_id = intval($this->input->post('md'));
		$module = gc_module_name($module_id);
		if(gc_module_settings($module, 'locations') == 0) {
			$module_id = 1;
			$module = 'listings';
		}
		$this->load->model('locations/cities_model');		
		$where = array('state_id' => $state_id, 'active' => 1);
		if(!empty(settings_item('lst.hide_counts_zero_locations'))) {
		    $where['counts > '] = 0;
		}
		$rows = $this->cities_model->select('id, name, slug')->order_by('name', 'asc')->find_all_by($where);
		$result = array();
		if($rows) {
			foreach($rows as $row) {
				$result[] = array(
					'id' => $row->id,
					'name' => $row->name,
					'url' => gc_location_url($module, $row->slug, $row->id)
				);
			}
		}
		$output = json_decode(json_encode($result),true);
		echo json_encode($output);
	}
	
	/**
	 * Ajax function to loadSubCategories on user click
	 */
	public function loadSubCategories() {
		$this->load->model('categories/categories_model');
		$parent_id = intval($this->input->post('id'));
		$module_id = intval($this->input->post('md'));
		$module = gc_module_name($module_id);
		if(gc_module_settings($module) == 0) {
			$module_id = 1;
			$module = 'listings';
		}
		$rows = $this->categories_model->load_categories($module_id, $parent_id);
		$result = array();
		if($rows) {
			foreach($rows as $row) {			    
			    if(settings_item('site.category_slug_uris') == 1) {
			        $row->slug = $row->slug_uris;
			    }
				$info = array(
					'id' => $row->id,
					'name' => $row->name,
					'url' => gc_category_url(gc_module_name($module_id), $row->slug, $row->id)
				);
				if(settings_item('lst.category_listings_count') == 1) {
				    $info['counts'] = $row->counts;
				}
				$result[] = $info;
			}
		}
		$output = json_decode(json_encode($result),true);
		echo json_encode($output);
	}
	
	/**
	 * Display city on user keyup
	 * @return string
	 */
	public function ajax_search_location () {
		$module = escape($this->input->get('module'));
		$keyword = escape($this->input->get('search'));
		$this->load->library('slug');
		
		// location keyword set in the listings_model category_tag_location_search method
		// adding state name with the city 
		$location_keyword = $this->session->userdata('location_keyword');
		if(!empty($location_keyword) && $location_keyword == $this->input->get('search')) {
			$result[] = array(
					'name' => $location_keyword,
					'type' => 1
			);
		} else {
			$keyword = $this->location_handler($keyword);
			if(settings_item('site.enable_cache') == 1) {
				$cacheID = str_replace('-', '_', $this->slug->create_slug($keyword)). '_' . $this->session->userdata('search_country'); // Cache giving errors when invalid url with spaces and special characters
				if(!$listings = $this->z_cache->load($cacheID)) {
					$listings = $listings = $this->listings_model->ajax_search_location($keyword, $module);
					$this->z_cache->save($listings, $cacheID);
				}
			} else {
				$listings = $this->listings_model->ajax_search_location($keyword);
			}
		
			$result = array();
			foreach($listings as $listing) {
				$result[] = array(
						//'id' => $listing->id,
						'name' => $listing->name,
						'type' => $listing->type
				);
			}
		}
		$output = json_decode(json_encode($result),true);
		echo json_encode($output);
		//return $listings;
	}
	
	/**
	 * Grab and display result on user keyup
	 * @return string
	 */
	public function ajax_search () {
		$module = escape($this->input->get('module')) ;
		if(gc_module_id($module) == 0) {
			$module = 'listings';
		}
		$keyword = escape($this->input->get('search'));
		$locality_id = intval($this->session->userdata('search_locality'));
		$search_location = str_replace("_"," ", $this->session->userdata('search_location') ?? '');
		$category_id = 0;//$this->session->userdata('search_category');
		$country_iso = $this->session->userdata('search_country');
		$this->load->model('listings/listings_model');
		$result = $this->listings_model->ajax_results($module, $keyword, $country_iso, $search_location, $locality_id, $category_id);
		$output = json_decode(json_encode($result),true);
		echo json_encode($output);
	}
	
	/**
	 * Store category/locality id on user selection
	 */
	public function ajax_selection () {
		if(!empty($_POST['category'])) {
			$this->category_handler(escape($this->input->post('category')));
		}
	
		if(!empty($_POST['locality'])) {
			$this->locality_handler(escape($this->input->post('locality')));
		}
	}
	
	/**
	 * Storing location information
	 * @param string $search_location
	 * @return string
	 */
	public function location_handler($search_location) {
		if($search_location) {
			$this->session->set_userdata('search_location', $search_location);
			return $search_location;
		} elseif($this->session->userdata('search_location')) {
			$search_location = $this->session->userdata('search_location');
			return $search_location;
			$this->session->unset_userdata('search_location');
		} else {
			$search_location ="";
			return $search_location;
		}
	}
	
	/**
	 * Storing category information
	 * @param string $search_category
	 * @return string
	 */
	public function category_handler($search_category) {
		if($search_category) {
			$this->session->set_userdata('search_category', $search_category);
			return $search_category;
		} elseif($this->session->userdata('search_category')) {
			$search_category = $this->session->userdata('search_category');
			return $search_category;
			$this->session->unset_userdata('search_category');
		} else {
			$search_category ="";
			return $search_category;
		}
	}
	
	/**
	 * Business Owner Reply to the User
	 * @return string
	 */
	public function replyQuery() {
		$this->lang->load('members/members', $this->session->userdata('language'));
		$this->form_validation->set_rules('query_id', 'lang:members_business_queries', 'trim|required');
		$this->form_validation->set_rules('reply_subject', 'lang:label_subject', 'trim|required');
		$this->form_validation->set_rules('reply_message', 'lang:label_message', 'trim|required');
		$this->form_validation->set_rules('captcha_code', 'lang:label_captcha', 'xss_clean|required|valid_captcha');
	
		if ($this->form_validation->run() === FALSE) {
			$validation_errors = validation_errors();
			if ($validation_errors) {
				$validation_error = $validation_errors;
			}
			echo json_encode(array('result' => 0, 'message' => $validation_error, 'type' => 'error'));
		} else {
			$query_id = (int) $this->input->post('query_id');
			// Retrieve user email address from database
			$save_query = escape($this->input->post('save_query'));
			$this->load->model('business_queries/business_queries_model');
			$query = $this->business_queries_model->select('user_id, email, module_id, row_id')->find($query_id);
			if($query) {
				// Retrieve user email address from database
				$email = $this->business_queries_model->get_owner_email($query);
				if(!$email) {
					echo json_encode(array('result' => 0, 'message' => lang('error_message'), 'type' => 'error'));
					return false;
				}
			
				$subject = escape($this->input->post('reply_subject'));
				// email is sent to the business owner from user via posted_queries page
				$message = '';
				if($save_query && $save_query != 0) {
					$email_to = $email;
					$message .= sprintf(lang('reply_email'), $query->email) . "<br /><br />";
				} else { // business owner replied to the user
					$email_to = $query->email;
				}
				$message .= escape($this->input->post('reply_message')) .'<hr><br />' .settings_item('site.title') .'<br />' .site_url();
				// Send Email
				$this->load->library('emailer/emailer');
				$data = array(
						// 'from'		=> $query->email_from,
						'to'		=> $email_to,
						'subject'	=> $subject,
						'message'	=> $message
				);
					
				$success = $this->emailer->send($data, FALSE);
				if($success) {
					if(!empty($query->user_id)) {
						// save query reply for future reference
						$reply = array(
								'query_id' => $query_id,
								'user_id' => $query->user_id,
								'replied_by' => $this->auth->user_id(),
								'subject' => $subject,
								'message' => escape($this->input->post('reply_message')),
								'created_on' => date('Y-m-d H:i:s'),
								'created_ip' => $this->input->ip_address()
						);
						$this->db->insert('query_replies', $reply);
					}
					// Update only if the business owner replies
					if(!$save_query) {
						$this->db->update($this->db->dbprefix .'business_queries', array('replied' => 1), array('id' => $query_id));
					} else {
						$this->db->update($this->db->dbprefix .'business_queries', array('replied' => 0), array('id' => $query_id));
					}
					echo json_encode(array('result' => 1, 'message' => lang('success_message'), 'type' => 'success'));
				}
			} else {
				echo json_encode(array('result' => 0, 'message' => lang('error_message'), 'type' => 'error'));
			}
		}
	}
	
	/**
	 * Display query replies to the business owner on their dashboard
	 */
	public function query_replies() {
		header('Content-Type: application/x-json; charset=utf-8');
		Assets::add_js('query-replies.js');
		$query_id = $this->input->post('id');
		$this->load->model('business_queries/business_queries_model');
		$replies = $this->business_queries_model->query_replies($query_id);
		echo json_encode($replies);
	}
	
	/**
	 * Ajax Tags Search - OnType Tag Selection - Post Your Requirements
	 */
	public function tags_search() {
		$query = escape($this->input->get('search'));
		$this->load->model('tags/tags_model');
		$result = $this->tags_model->ajax_tags_search($query);
		$output = json_decode(json_encode($result),true);
		echo json_encode($output);
	}

	/**
	 * Ajax Cities Search - OnType City Selection
	 */
	public function cities_search() {
		$query = escape($this->input->get('search'));
		$this->load->model('locations/cities_model');
		$result = $this->cities_model->ajax_cities_search($query, 15);
		$output = json_decode(json_encode($result),true);
		echo json_encode($output);
	}
	
	/**
	 * Ajax Localities Search - OnType Locality Selection
	 */
	public function localities_search() {
		$city_id = escape($this->input->get('city_id'));
		$query = escape($this->input->get('search'));
		if(!empty($city_id)) {
			$this->load->model('locations/localities_model');
			$this->db->like('name', $query);
			$result = $this->localities_model->select('id, name')->order_by('name', 'asc')->limit(10)->find_all_by(array('city_id' => $city_id, 'active' => 1));
			$output = json_decode(json_encode($result),true);
			echo json_encode($output);
		}
	}
	
	/**
	 * Provide security to external scripts like Responsive File Manager
	 * Only Admin can access the Responsive File Manager for security reasons
	 * $id - Cookie SESSION ID (bf_session)
	 */
	public function check_permission() {
		// IP Address cannot be used in the query as filemanager requests from the server URL.
	    $id = isset($_GET['id']) ? $_GET['id'] : null;
	    // IP Address cannot be used in the query as filemanager requests from the server URL.
	    $this->load->library('session');
	    if($this->session->userdata('role_id') == 1) {
	        echo true;
	    } else if(!empty($id)) {
			$result = $this->db->select('data')->get_where('ci3_sessions', array('id' => escape($id)))->row();
			if($result) {
				// read BLOB data and get the user's role_id
				$session_data = $result->data ;  // your BLOB data who are a String
				$return_data = array();  // array where you put your "BLOB" resolved data
	
				$offset = 0;
				while ($offset < strlen($session_data))  {
					if (!strstr(substr($session_data, $offset), "|")) {
						throw new Exception("invalid data, remaining: " . substr($session_data, $offset));
					}
					$pos = strpos($session_data, "|", $offset);
					$num = $pos - $offset;
					$varname = substr($session_data, $offset, $num);
					$offset += $num + 1;
					$data = unserialize(substr($session_data, $offset));
					$return_data[$varname] = $data;
					$offset += strlen(serialize($data));
				}
				if(!empty($return_data['role_id']) && $return_data['role_id'] == 1) {
					echo true;
				}
			}
		}
	}
	
	/**
	 * Admin Reply to the User
	 * @return string
	 */
	public function admin_email() {
		$this->auth->restrict();
		$this->form_validation->set_rules('query_id', 'lang:label_id', 'required|xss_clean');
		$this->form_validation->set_rules('request_uri', 'lang:label_website', 'required|xss_clean');
		$this->form_validation->set_rules('reply_subject', 'lang:label_subject', 'required|xss_clean');
		$this->form_validation->set_rules('reply_message', 'lang:label_message', 'required|xss_clean');
	
		if ($this->form_validation->run() === FALSE) {
			$validation_errors = validation_errors();
			if ($validation_errors) {
				$validation_error = $validation_errors;
			}
			echo json_encode(array('result' => 0, 'message' => $validation_error, 'type' => 'error'));
		} else {
			$query_id = escape($this->input->post('query_id'));
			$request_uri = escape($this->input->post('request_uri'));
			// Retrieve user email address from database
			switch($request_uri) {
				case 'contact_queries':
					$this->load->model('contact_queries/contact_queries_model');
					$query = $this->contact_queries_model->find($query_id);
					break;
				case 'business_queries':
					$this->load->model('business_queries/business_queries_model');
					$query = $this->business_queries_model->find($query_id);
					break;
				case 'leads':
					$this->load->model('leads/leads_model');
					$query = $this->leads_model->find($query_id);
					break;
				case 'payout_users':
					$this->load->model('payouts/payouts_model');
					$query = $this->payouts_model->get_user($query_id);
					break;
			}
			if($query) {
				// Send Email
				$this->load->library('emailer/emailer');
				$data = array(
						'from'		=> settings_item('sender_email'),
						'to'		=> $query->email,
						'subject'	=> escape($this->input->post('reply_subject')),
						'message'	=> $this->input->post('reply_message')
				);
				$success = $this->emailer->send($data, FALSE);
				if($success) {
					switch($request_uri) {
					    case 'contact_queries':
					        $this->db->update($request_uri, array('replied_on' => date('Y-m-d H:i:s'), 'status' => 1), array('id' => $query_id));
						    break;
						case 'leads':
							$this->db->update($request_uri, array('replied_on' => date('Y-m-d H:i:s')), array('id' => $query_id));
							break;
						case 'business_queries':
							$this->db->update($request_uri, array('replied' => 1), array('id' => $query_id));
							// save query reply for future reference
							$reply = array(
									'query_id' => $query_id,
									'user_id' => $query->user_id,
									'replied_by' => $this->auth->user_id(),
									'subject' => escape($this->input->post('reply_subject')),
									'message' => $this->input->post('reply_message'),
									'created_on' => date('Y-m-d H:i:s'),
									'created_ip' => $this->input->ip_address()
							);
							$this->db->insert('query_replies', $reply);
							break;
					}
						
					echo json_encode(array('result' => 1, 'message' => lang('success_message'), 'type' => 'success'));
				} else {
					echo json_encode(array('result' => 0, 'message' => lang('email_error'), 'type' => 'error'));
				}
			} else {
				echo json_encode(array('result' => 0, 'message' => lang('email_error'), 'type' => 'error'));
			}
		}
	}
	
	/**
	 * Checkout page - Ajax function to apply coupon code
	 */
	public function updateOrder() {
	    $coupon = escape($this->input->post('coupon'));
	    $custom_cart = $this->input->post('custom_cart') ? true: false;
	    $user_id = $this->auth->user_id();
	    $this->load->model('orders/orders_model');
	    // $data = $this->orders_model->order_update($user_id, $coupon, $action);
	    if(empty($coupon)) {	        
	        $this->orders_model->checkout_params($user_id, false, array('clear' => 'coupon'));
	        $data = $this->orders_model->update_cart_items();	        
	    } else {
    	    $this->load->model('coupons/coupons_model');
    	    $data = $this->coupons_model->apply_coupon($coupon, $user_id, false, $custom_cart);
	    }
	    // custom cart page
	    if($custom_cart) {
	        $data = $this->cart_update($user_id, null, array(), $custom_cart);
	    }
	    header('Content-Type: application/x-json; charset=utf-8');
	    echo json_encode($data);
	}
	
	/**
	 * FANCY TREE PLUGIN - Display Categories Tree to the User
	 * @return string
	 */
	public function getCategoriesTree() {
		$this->load->model('categories/categories_model');
		/**
		 * Retrieve listing and category parent id from GET
		 * Get categories information from database based on parent id
		 * Enable lazy load only if parent has subcategories
		 * Join with Listing Categories table for Edit Listing
		*/
		$parent_id = 0;
		$package_id = (int) $this->input->get('package_id');
		$row_id = escape($this->input->get('row_id'));
		$module = escape($this->input->get('module'));
	
		// prevent direct access of this function
		if(empty($module) || gc_module_id($module) == 0) {
			Template::redirect(site_url());
		}
	
		if($this->input->get('parent')) {
			$parent_id = $this->input->get('parent');
		}
	
		//POST BACK HANDLING
		$post_categories_array = array();
		$post_categories = $this->session->userdata($module . '_categories');	
		if(settings_item('site.enable_cache') == 1) {
			$cacheID = 'categories_tree_' . $parent_id . '_' . $this->auth->user_id() . '_' . $module . '_' . $post_categories;
			$cacheID = str_replace('-', '_', $this->slug->create_slug($cacheID));
			if(!$data = $this->z_cache->load($cacheID)) {
				if($post_categories) {
					$post_categories_array = str_replace('/', ',', $post_categories);
					$post_categories_array = explode(',', $post_categories_array);
				}
				$data = $this->categories_model->get_categories_ajax($parent_id, $module, $post_categories_array, $package_id);
				$this->z_cache->save($data, $cacheID);
			}
		} else {
			if($post_categories) {
				$post_categories_array = str_replace('/', ',', $post_categories);
				$post_categories_array = explode(',', $post_categories_array);
			}
			$data = $this->categories_model->get_categories_ajax($parent_id, $module, $post_categories_array, $package_id);
		}
		echo json_encode($data);
	}
	
	/**
	 * Selectize search - Users/Categories/States/Cities/Tags/Listings
	 * @param string $module - Name of the module
	 * @param string $q - Search Keyword
	 */
	public function selectizeSearch($module, $q = '') {
	    header('Content-Type: application/x-json; charset=utf-8');
	    $select = '';
	    $role_id = $this->auth->role_id();
	    $user_id = $this->auth->user_id();
		$primary_field = 'id';
		$description_field = '';
		$id = 0;
		$relation_id = 0;
		$relation_field = null;
		if(!empty($_GET)) {
		    $q = escape(urldecode($this->input->get('q')));
		    $id = (array) $this->input->get('id');
		    $relation_id = (array) $this->input->get('relation_id'); // relation field ids
		    $relation_field = escape($this->input->get('relation_field')); // relation field name
		} else {
		    $q = escape(urldecode($q));
		}		
		$where = array();
		if($module == 'countries') {
			$primary_field = 'iso';
		}
		switch($module) {	
		    case 'non_affiliates':
		        // do not display email to frontend users for security reasons
		        $field = $this->session->userdata('user_display_field') ?  $this->session->userdata('user_display_field') : 'display_name';
		        if($this->session->userdata('role_id')) {
		            $description_field = 'email';
		        }
		        // display non affiliate users only while creating a new affiliate through the Admin Panel
		        if($module == 'non_affiliates') {
		            $field = "CONCAT(id, ' - ', display_name)";
		            $description_field = 'email';
		            $where = 'id NOT IN (SELECT user_id FROM ' . $this->db->dbprefix . 'affiliate_users)';
		            if(!empty($q)) {
		                $where .= " AND (email LIKE '%" . $q . "%' OR display_name LIKE '%" . $q . "%')";
		                $q = null;
		            }
		        } else if(!empty($q)) { // the user can also be searched by email
		            $where = "(email LIKE '%" . $q . "%' OR display_name LIKE '%" . $q . "%')";
		            $q = null;
		        }
		        $module = 'users';
		        break;
		    case 'users':
		        $field = "CONCAT(id, ' - ', display_name)";
		        $description_field = 'email';
		        break;
		    case 'sales_users':
		        $field = "CONCAT(id, ' - ', display_name)";
		        $description_field = 'phone_number';
		        $where[] = "role_id = 9";
		        $module = 'users';
		        break;
		    case 'delivery_users':
		        $field = "CONCAT(id, ' - ', display_name)";
		        $description_field = 'phone_number';
		        $where[] = "role_id = 8";
		        $module = 'users';
		        break;
		    case 'franchise_users':
		        $field = "CONCAT(id, ' - ', display_name)";
		        $description_field = 'phone_number';
		        $where[] = "role_id = 7";
		        $module = 'users';
		        break;
		    case 'business_users':
		        $select = "M.id, CONCAT(M.id, ' - ', M.company_name) AS name";
		        $field = 'M.company_name';
		        $description_field = 'phone_number';
		        $where[] = "role_id = 2";
		        $module = 'users';
		        break;
		    case 'cities':
		        $field = 'M.name';
		        $select = "M.id, CONCAT(M.name, ', ', S.name, ', ', M.country_iso) AS name";
		        break;
			case 'categories':
			case 'states':
			case 'tags':
			case 'countries':
			case 'payment_gateways':
			case 'faq_groups':
			case 'subscriber_groups':
			case 'mobile_link_groups':
			case 'mobile_slider_groups':
			case 'training_groups':
			case 'coupons':
			case 'email_template_groups':
			case 'subscriber_users':
			case 'banner_groups':
			case 'languages':
				$field = 'name';
				break;
			case 'country_currencies':
			    $field = 'code';
			    $module = 'currencies';
			    break;
			default:
				$field = 'title';
		}
		if(!empty($q)) {
		    if($module == 'users') {
		        $where[] = " (phone_number LIKE '%" . $q . "%' OR email LIKE '%" . $q . "%' OR display_name LIKE '%" . $q . "%')";
		    } else {
		        $where[] = $field . " LIKE '%" . $q . "%'";
		    }
		}
		// display user specific listings, products etc. to the user
		// while adding products, jobs and other modules information
		if($role_id == 2 && in_array($module, gc_modules())) {
		    $where[] = 'M.user_id = ' . $user_id;
		}
		$this->db->from($module . ' M');
		if(empty($select)) {
		    $select = $primary_field . ' AS id,' . $field . ' AS name';
		} else if($module != 'users') {
		    $this->db->join('states S', 'S.id = M.state_id', 'INNER');
		}		
		if(!empty($relation_id)) { // Field relations
		    if($module == 'states') {
		        $where[] = "FIND_IN_SET(country_iso, '". implode(',', $relation_id) ."')";
		    } else {
		        $where[] = $relation_field . ' IN (' . implode(',', $relation_id) . ')';
		    }
		}
		if(!empty($where)) {
		    $this->db->where(is_string($where) ? $where : implode(' AND ', $where));
		}
		if(!empty($description_field)) {
		    $select .= ', ' . $description_field . ' as description';
		}
		$result = $this->db->select($select)->get()->result();
		echo json_encode($result);
	}
	
	/**
	 * Get Subcategories using AJAX
	 * @param number $parent_id
	 */
	public function loadCategories($id) {
	    header('Content-Type: application/x-json; charset=utf-8');
	    $this->load->model('categories/categories_model');
	    $parent_id = (int) $id;
	    if(!empty($parent_id)) {
	        $categories = $this->categories_model->get_translated('M.active = 1 AND M.deleted = 0 AND M.parent_id = ' . $parent_id, true);
	    } else {
	        $categories = '';
	    }
	    echo json_encode($categories);
	}
	
	/**
	 * Location Dropdown
	 * @param string $table
	 * @param number $parent_id
	 */
	public function load($table, $parent_id) {
	    header('Content-Type: application/x-json; charset=utf-8');
	    $this->load->model('locations/cities_model');
	    $result = $this->cities_model->load($table, $parent_id);
	    $output = json_decode(json_encode($result),true);
	    echo json_encode($output);
	}
	
	/**
	 * Image CRUD - Upload, List and Delete images through AJAX instead of query segments
	 * @param string $action - upload_file, list, ajax_list, delete_file etc.
	 */
	public function image_crud($action) {
	    $this->load->library('image_handling');
	    $this->image_handling->perform($action);
	}
	
	/**
	 * START OF ECOMMERCE
	 *
	 * Add Item to Cart
	 */
	public function cart() {
	    $data = array();
	    $checkout_page = 0;
	    $relation_item_id = (int) $this->input->post('relation_item_id'); // Optional Service
	    $this->load->model('orders/orders_model');
	    $update_cart = (int) $this->input->post('update_cart'); // Add/Removal of optional services on the checkout page
	    $qty = (int) $this->input->post('qty');
	    $qty = !empty($qty) ? $qty : 1;
	    $module_id = !empty($_GET['module']) ? (int) $_GET['module'] : (int) $this->input->post('module');
	    $module_name = gc_module_name($module_id);
	    $module = !empty($module_name) ? $module_name : 'products';
	    $user_id = $this->auth->user_id();	    
	    if(isset($_GET['code']) || isset($_GET['item'])) { // Remove Item from Cart
	        $id = isset($_GET['code']) ? $_GET['code'] : $_GET['item'];
	        $update_cart = (int) $this->input->get('update_cart');
	        $checkout_page = (int) $this->input->get('checkout');
	        // variants - different variants of technical support and other items
	        // radio change - remove previous item and add new selected to the cart
	        $is_variant = (int) $this->input->get('is_variant');
	        if($is_variant == 1) {
	            $this->add_remove_variant_cart($id, $qty);
	        } else {
	            $this->orders_model->cart($id, 1, $qty, 0, $module, ($update_cart == 1 || $checkout_page == 1 ? false : true), array('relation_item_id' => $relation_item_id), $user_id);
	        }
	    } else if(isset($_POST['action'])) {
	        $action = (int) $_POST['action'];
	        switch($action) {
	            case 0:
	                if(isset($_POST['item'])) {
	                    $id = $_POST['item'];
	                    $is_variant = (int) $this->input->post('is_variant');
	                    if($is_variant == 1) {
	                        $this->add_remove_variant_cart($id, $qty);
	                    } else {
	                        // if the user is not logged in, save the login_redirect_url to the checkout page
	                        // redirect user to the checkout page
	                        if(!$this->auth->is_logged_in()) {
	                            $this->session->set_userdata('login_redirect_url', gc_url_format('members/checkout'));
	                        }
	                        $extra = array('relation_item_id' => $relation_item_id);
	                        if(strpos($id, '-') !== false) {
	                            $product_variant = explode('-', $id);
	                            if(!empty($product_variant[1])) {
	                                $extra['variant_id'] = $product_variant[1];
	                            }
	                        }
	                        $this->orders_model->cart((int) $id, $action, $qty, 0, $module, ($update_cart == 1 ? false : true), $extra, $user_id);
	                    }
	                }
	                break;
	            case 2:
	                $this->orders_model->cart_load();
	                break;
	        }
	    }
	    $data = $this->orders_model->update_cart_items($user_id);
	    header('Content-Type: application/x-json; charset=utf-8');
	    echo json_encode($data);
	}
	
	/**
	 * Update Cart Items
	 */
	public function update_cart() {
	    header('Content-Type: application/x-json; charset=utf-8');
	    $user_id = $this->auth->user_id();	  
	    $custom_cart = $this->input->post('custom_cart') ? true: false;
	    $this->load->model('orders/orders_model');
	    if(!empty($_POST['items'])) {
	       $items = json_decode($_POST['items'], true);
	       $checkout_params = $this->orders_model->checkout_params($user_id, 'all');
	       $cart_items = $checkout_params['items'];
	       foreach($items as $item) {
	           $cart_items[$item['id']]['qty'] = $item['qty'];
	       }
	       $this->orders_model->checkout_params($user_id, null, array('items' => $cart_items)); 
	    }
	    $coupon = $_POST['coupon'];
	    if(empty($coupon)) {
	        $this->orders_model->checkout_params($user_id, false, array('clear' => 'coupon'));
	    }
	    echo json_encode($this->cart_update($user_id, $items, array(), $custom_cart));
	    return false;
	}
	
	/**
	 * Cart Update
	 * @param number $user_id
	 * @param array $items
	 * @param boolean $custom_cart - Custom cart page with manual coding
	 */
	private function cart_update($user_id, $items, $data = array(), $custom_cart = false) {	 
	    if(empty($data)) {
	       $data = $this->orders_model->update_cart_items($user_id, false, $custom_cart);
	    }
	    if(empty($items)) {
    	    $cart_items = $this->orders_model->checkout_params($user_id);
    	    if($cart_items) {
    	        foreach($cart_items as $key => $cart_item) {
    	            $items[] = array(
    	                'id' => $key,
    	                'qty' => $cart_item['qty']
    	            );
    	        }
    	    }
	    }
	    $total = 0;
	    $subtotal = 0;
	    if(!empty($data['items'])) {
	        $cart = $data['items'];
	        foreach($items as $key => $item) {
	            if(!empty($cart[$item['id']])) {
	                $cart_item = $cart[$item['id']];
	                $items[$key]['qty'] = $cart_item['qty'];
	                $items[$key]['total'] = currency_format($cart_item['total']);
	                $total += $cart_item['total'];
	                $subtotal += $cart_item['subtotal'];
	            }
	        }
	    }
	    return array('result' => 1, 'items' => $items, 'subtotal' => currency_format($subtotal),'total' => currency_format($total));
	}
	
	/**
	 * Clear and reload cart
	 */
	public function clear_cart($user_id = 0) {
	    $this->load->model('orders/orders_model');
	    $this->orders_model->checkout_params($user_id, false, array(), true);
	    header('Content-Type: application/x-json; charset=utf-8');
	    echo json_encode(array('checkout' => gc_uri_string('checkout'), 'cart' => gc_uri_string('cart')));
	    return false;
	}
	
	/**
	 * Add and Remove variant to/from the cart
	 * @param array $variant_info
	 */
	private function add_remove_variant_cart($id, $qty = 1) {
	    $this->load->model('orders/orders_model');
	    $user_id = 0;
	    $variant_info = explode('-', $id);
	    $item_removal_id = '';
	    $relation_item_id = !empty($variant_info[2]) ? $variant_info[2] : 0;
	    $checkout_cart_items = $this->orders_model->checkout_params($user_id);
	    if(!empty($checkout_cart_items)) {
	        foreach($checkout_cart_items as $item_id => $checkout_cart_item) {
	            if($checkout_cart_item['item']['id'] == $variant_info[0] && $checkout_cart_item['relation_item_id'] == $relation_item_id) {
	                $item_removal_id = $item_id;
	                $relation_item_id = $variant_info[2];
	            }
	        }
	    }
	    if(!empty($item_removal_id)) {
	        $this->orders_model->cart($item_removal_id, 1, $qty, 0, 'products', false, array('relation_item_id' => $relation_item_id));
	    }
	    if(!empty($variant_info[1])) {
	        $this->orders_model->cart((int)$variant_info[0], 0, $qty, 0, 'products', false, array('relation_item_id' => $relation_item_id, 'variant_id' => $variant_info[1]));
	    }
	}
	
	/**
	 * Update Qty
	 */
	public function qty() {
	    header('Content-Type: application/x-json; charset=utf-8');
	    $item = $_POST['item'];
	    $qty = (int) $_POST['qty'];
	    $message = lang('cart_qty_zero');
	    $success = 0;
	    if($qty == 0) {
	        echo json_encode(array('success' => $success, 'message' => $message));
	        return false;
	    }
	    $this->load->model('orders/orders_model');
	    $result = $this->orders_model->item($item);
	    if(!$result || $result->stock < $qty) {
	        echo json_encode(array('success' => $success, 'message' => $message));
	        return false;
	    }
	    // variants - different variants of technical support and other items
	    // radio change - remove previous item and add new selected to the cart
	    $this->orders_model->cart($item, 1, $qty, 0, 'products', false, array());
	    
	    /*$cart = (array) $this->session->userdata('cart');
	    if(empty($cart[$item])) {
	        echo json_encode(array('success' => $success, 'message' => $message));
	        return false;
	    }
	    $cart[$item]['qty'] = $qty;
	    $this->session->set_userdata('cart', $cart);
	    */
	    $data = $this->orders_model->update_cart_items();
	    // echo json_encode(array('success' => 1, 'message' => $message));
	    echo json_encode($data);
	    return false;
	}
	
	/**
	 * Save Wishlist Item
	 * @return boolean
	 */
	public function wishlist() {
	    // header('Content-Type: application/x-json; charset=utf-8'); js error fix in the gc.js file
	    $success = 0;
	    $label_text = lang('label_wishlist');
	    $message = sprintf(lang('msg_item_exists'), $label_text);
	    if($this->auth->is_logged_in()) {
	        $item = (int) (!empty($_POST['row_id']) ? $_POST['row_id'] : $_POST['item']);
	        if(empty($item)) {
	            return false;
	        }
	        $item = explode('-', $item);
	        $product_id = (int) $item[0];
	        $variant_id = 0;
	        if(isset($item[1])) {
	            $variant_id = (int) $item[1];
	        }
	        if(!empty($product_id)) {
	            $user_id = $this->auth->user_id();
	            $where = array('user_id' => $user_id, 'product_id' => $product_id, 'variant_id' => $variant_id);
	            // Check existence
	            $count = $this->db->where('id', $product_id)->count_all_results('products');
	            if($count > 0) {
	                $this->load->model('ecommerce/ecommerce_model');
	                // check existence
	                $existence = $this->db->where($where)->count_all_results('product_wishlists');
	                if($existence == 0) {
	                    $this->ecommerce_model->save_wishlist_item($user_id, $product_id, $variant_id);
	                    $success = 1;
	                    $message = sprintf(lang('msg_item_success'), $label_text);
	                } else {
	                    if($this->db->where($where)->delete('product_wishlists')) {
	                        $success = 2;
	                        $message = sprintf(lang('msg_item_removed'), $label_text);
	                    }
	                }
	            }
	        }
	    } else {
	        $message = sprintf(lang('msg_not_logged_in'), $label_text);
	    }
	    echo json_encode(array('success' => $success, 'message' => $message));
	}
	
	/**
	 * Save Notify Me Item
	 * @return boolean
	 */
	public function notify_me() {
	    header('Content-Type: application/x-json; charset=utf-8');
	    $this->form_validation->set_rules('notify_email', 'lang:label_email', 'required|xss_clean|valid_email');
	    if ($this->form_validation->run() === FALSE) {
	        $validation_errors = validation_errors();
	        if ($validation_errors) {
	            $validation_error = $validation_errors;
	        }
	        echo json_encode(array('result' => 0, 'message' => $validation_error));
	    } else {
	        $result = 0;
	        $label_text = lang('label_notify_list');
	        $message = sprintf(lang('msg_item_exists'), $label_text);
	        $item = $_POST['item'];
	        if(empty($item)) {
	            return false;
	        }
	        $item = explode('-', $item);
	        $product_id = (int) $item[0];
	        $variant_id = 0;
	        if(isset($item[1])) {
	            $variant_id = (int) $item[1];
	        }
	        if(!empty($product_id)) {
	            $user_id = (int) $this->auth->user_id();
	            $email_id = $this->input->post('notify_email');
	            $where = array('email' => $email_id, 'product_id' => $product_id, 'variant_id' => $variant_id, 'deleted' => 0);
	            // Check existence
	            $count = $this->db->where('id', $product_id)->count_all_results('products');
	            if($count > 0) {
	                $this->load->model('ecommerce/ecommerce_model');
	                // check existence
	                $existence = $this->db->where($where)->count_all_results('product_notifications');
	                if($existence == 0) {
	                    $this->db->insert('product_notifications', array(
	                        'user_id' => $user_id,
	                        'product_id' => $product_id,
	                        'variant_id' => $variant_id,
	                        'email' => $email_id,
	                        'created_on' => date('Y-m-d H:i:s'),
	                        'created_ip' => $this->input->ip_address()
	                    ));
	                    $result = 1;
	                    $message = sprintf(lang('msg_item_success'), $label_text);
	                }
	            }
	        }
	        echo json_encode(array('result' => $result, 'message' => $message));
	    }
	}
	
	/**
	 * Common function to Order rows using AJAX nestedSortable
	 */
	public function prioritize() {
	    $this->load->library('template');
	    $this->load->library('assets');
	    $info = $this->session->userdata('prioritize_rows');
	    $table = $info['table'];
	    $where = $info['where'];
	    // Save order from ajax call
	    if (isset($_POST['sortable'])) {
	        $rows = $_POST['sortable'];
	        if (count($rows)) {
	            foreach ($rows as $order => $row) {
	                if (isset($row['id']) && $row['id'] != '') {
	                    $data = array('priority' => $order);
	                    if($table == 'field_modules') {
	                        $update_where = $where;
	                        $update_where['field_id'] = $row['id'];
	                    } else {
	                        $update_where = array('id' => $row['id']);
	                    }
	                    $this->db->where($update_where)->update($table, $data);
	                }
	            }
	        }
	    }
	    // Load view
	    if(!empty($where)) {
	        $this->db->where($where);
	    }
	    if($table == 'field_modules') {
	        $this->db->join('fields F', 'F.id = ' . $table . '.field_id', 'INNER');
	        $result = $this->db->select('F.id, F.name')->order_by('priority')->get($table)->result_array();
	    } else {
	        $result = $this->db->order_by('priority')->get($table)->result_array();
	    }
	    Template::set('result', $result);
	    Template::set('maxLevels', $info['max_levels']);
	    Template::set('toolbar_title', lang('order_fields'));
	    Template::set_view('common/order_ajax');
	    Template::render();
	}
	
	/**
	 * Leaflet Map Markers on the View Page
	 */
	public function get_markers() {
	    header('Content-Type: application/x-json; charset=utf-8');
	    $result = array('result' => 0);
	    $map_markers = $this->session->userdata('map_markers');
	    if(!empty($map_markers)) {
	        $result['result'] = 1;
	        $result['markers'] = $map_markers;
	    }
	    echo json_encode($result);
	}
	
	/**
	 * Save Callback Requests
	 */
	public function callbacks() {
	    $this->ajax_model->feedbacks_callbacks();
	}
	
	/**
	 * Save Feedbacks
	 */
	public function feedbacks() {
	    $this->ajax_model->feedbacks_callbacks('feedbacks');
	}
	
	/**
	 * Site Offline Form Email Collection
	 */
	public function offline() {
	    $this->form_validation->set_rules('name', 'lang:label_name', 'trim|sanitize');
	    $this->form_validation->set_rules('email', 'lang:label_email', 'trim|required|valid_email');
	    $this->form_validation->set_rules('phone_number', 'lang:label_phone_number', 'trim|phone_check');
	    if ($this->form_validation->run() === FALSE) {
	        $validation_errors = validation_errors();
	        if ($validation_errors) {
	            $validation_error = $validation_errors;
	        }
	        echo json_encode(array('result' => 0, 'message' => $validation_error, 'type' => 'danger'));
	    } else {
	        $data = array(
	            'name' => $this->input->post('name'),
	            'email' => $this->input->post('email'),
	            'phone_number' => $this->input->post('phone_number'),
	        );
	    }
	}
	
	/**
	 * Display Help to the User
	 * @param string $module
	 * @return boolean
	 */
	public function help($context, $module) {
	    $host = 'https://www.goclixy.com/manuals/training_material?id=1&context=' . $context . '&module=' . $module;
	    $ch = curl_init();
	    curl_setopt($ch, CURLOPT_URL, $host);
	    curl_setopt($ch, CURLOPT_RETURNTRANSFER,1);
	    curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
	    // curl_setopt($ch, CURLOPT_USERPWD, "$login:$password");
	    $result = curl_exec($ch);
	    curl_close($ch);
	    if(!empty($result)) {
	        echo $result;
	    } else {
	        echo '<div class="alert alert-danger">' . lang('error_help_product') . '</div>';
	    }
	    return false;
	}
	
	/**
	 * Upload File
	 */
	public function uploader() {
	    $this->load->library('goclixy/gfile');
	    $this->gfile->uploader();
	}
	
	/**
	 * Download File
	 */
	public function download() {
	    $this->load->library('goclixy/gfile');
	    $this->gfile->manage();
	}
	
	/**
	 * Download File
	 */
	public function view() {
	    $this->load->library('goclixy/gfile');
	    $this->gfile->manage(false);
	}
	
	/**
	 * Remove File
	 */
	public function remove() {
	    $this->load->library('goclixy/gfile');
	    $_GET['action'] = 1;
	    $_GET['hash'] = isset($_GET['hash']) && $_GET['hash'] == 2 ? 0 : 1;
	    $this->gfile->manage();
	}
	
	/**
	 * File - Update Title/Priority
	 */
	public function update_file() {
	    $this->load->library('goclixy/gfile');
	    $this->gfile->update_file();
	}
	
	/**
	 * Package Information
	 * @param number $package_id
	 * @return boolean
	 */
	public function package_info($package_id) {
	    $this->auth->restrict();
	    $this->lang->load('packages/packages');
	    $this->load->model('packages/packages_model');
	    echo $this->packages_model->package_info($package_id, true);
	    return false;
	}
	
	/**
	 * Update Thread/Post Vote counter
	 */
	public function forum_vote() {
	    $this->load->model('forums/forums_model');
	    $this->forums_model->votes();
	}
	
	/**
	 * Signup Form
	 */
	public function signup() {
	    if($this->auth->is_logged_in()) {
	        echo json_encode(array('result' => false, 'message' => lang('us_signup_logged_in'), 'type' => 'danger'));
	        return false;
	    }
	    $this->load->model('users/user_model');
	    $_POST['register'] = true;
	    $result = $this->user_model->register(true);
	    echo json_encode($result);
	    return false;
	}
	
	/**
	 * Get Products
	 */
	public function get() {
	    $this->load->model('listings/listings_model');
	    $this->listings_model->ajax();
	}
	
	/**
	 * Clear Limits
	 */
	public function clear_limits() {
	    $this->load->library('gcrud');
	    $result = $this->gcrud->clear_limits();
	    echo $result ? 1 : 0;
	}
	
	/**
	 * Get Graph
	 * @param string $module
	 */
	public function update($module = 'commissions') {
	    $this->auth->restrict('Site.Chart.View');
	    $result = array();
	    switch($module) {
	        case 'commissions':
	        case 'payouts':
	            $data = (array) $this->session->userdata('chart_query_params');
	            $data['from'] = escape($this->input->post('from'));
	            $data['to'] = escape($this->input->post('to'));
	            $data['table'] = $module;
	            $result = $this->gcrud->chart('between', $data);
	            break;
	        case 'order_items':
	            $where = (array) $this->session->userdata('chart_query_params');
	            $where['status_id'] = 2;
	            $where['prefix'] = 'M';
	            $where['table'] = 'order_items';
	            $where['field'] = 'total';
	            $where['from'] = escape($this->input->post('from'));
	            $where['to'] = escape($this->input->post('to'));
	            $result = $this->gcrud->chart('between', $where);
	            $labels = $result['labels'];
	            $data = $result['data'];
	            break;
	        case 'orders':
	            $where = (array) $this->session->userdata('chart_query_params');
	            $where['status_id'] = 2;
	            $where['prefix'] = 'M';
	            $where['table'] = 'orders';
	            $where['field'] = 'order_total';
	            $where['from'] = escape($this->input->post('from'));
	            $where['to'] = escape($this->input->post('to'));
	            $result = $this->gcrud->chart('between', $where);
	            break;
	        case 'seo_backlinks':
	        case 'affiliate_referrals':
	            $where = (array) $this->session->userdata('chart_query_params');
	            $where['prefix'] = 'M';
	            $where['table'] = $module;
	            $where['field'] = 'counter';
	            $where['from'] = escape($this->input->post('from'));
	            $where['to'] = escape($this->input->post('to'));
	            $result = $this->gcrud->chart('between', $where);
	            break;
	        default: // Modules Summary           
	            $where = (array) $this->session->userdata('chart_query_params');
	            $where['prefix'] = 'M';
	            $where['table'] = $module;
	            $where['field'] = 'counter';
	            $where['from'] = escape($this->input->post('from'));
	            $where['to'] = escape($this->input->post('to'));
	            $result = $this->gcrud->chart('between', $where);
	            break;
	    }
	    header('Content-Type: application/x-json; charset=utf-8');
	    $result['success'] = 1;
	    echo json_encode($result);
	}
	
	/**
	 * Email Templates - Display Fields
	 */
	public function template_fields($prefix, $id, $preview = 0) {
	    $preview = (int) $preview == 1 ? true : false;
	    $this->auth->restrict('Site.Dashboard.View');
	    if($prefix == 'email') { // predefined email templates
	        $prefix = '';
	    }
	    $this->gemailer->fields_preview($id, $prefix, $preview);
	}
	
	/**
	 * Load Template
	 */
	public function load_template() {
	    $this->auth->restrict();
	    $template_id = (int) $this->input->post('id');
	    $module = $this->input->post('module');
	    if($template_id > 0) {
	        $table = $module == 'messages' ? 'message_templates' : 'email_templates';
	        $template = $this->db->where('id', $template_id)->get($table)->row();
	        if($template) {
	            $message = array('result' => 1, 'message' => $template->message, 'type' => 'success');
	            if($table == 'email_templates') {
	                $message['subject'] = $template->subject;
	            }
	            echo json_encode($message);
	        } else {
	            echo json_encode(array('result' => 0, 'subject' => '', 'message' => '', 'type' => 'danger'));
	        }
	    } else {
	        echo json_encode(array('result' => 0, 'subject' => '', 'message' => '', 'type' => 'danger'));
	    }
	}
	
	/**
	 * Return all language strings to be updated online
	 * @param string $origLang
	 * @return boolean
	 */
	public function lang_strings($origLang = 'english') {
	    $this->load->helper('translate/languages');
	    $this->load->library('translate/translate_lib');
	    // Load all language codes from the config file
	    $codes = $this->translations->get_languages(false);
	    if(!empty($codes[strtolower($origLang)])) {
	        $code = $codes[strtolower($origLang)];
    	    $allLangFiles = list_lang_files();
    	    $coreLangFiles = $allLangFiles['core'];
    	    // remove routes_lang.php from the language list
    	    if (in_array('routes_lang.php', $coreLangFiles)) {
    	        unset($coreLangFiles[array_search('routes_lang.php', $coreLangFiles)]);
    	    }
    	    sort($coreLangFiles);
    	    if (isset($allLangFiles['custom'])) {
    	        $moduleLangFiles = $allLangFiles['custom'];
    	        sort($moduleLangFiles);
    	        $coreLangFiles = array_merge($coreLangFiles, $moduleLangFiles);
    	    }
    	    $all_text_strings = array();
    	    foreach($coreLangFiles as $langFile) {
    	        $lang_strings = load_lang_file($langFile, $origLang);
    	        if(!empty($lang_strings)) {
    	            foreach($lang_strings as $key => $value) {
    	                $all_text_strings[strtolower($key)] = $value;
    	            }
    	        }
    	    }
    	    echo json_encode(array('code' => $code, 'strings' => $all_text_strings));
    	    return false;
	    }
	}
	
	/**
	 * Selectize Options
	 * @param string $module - Name of the module
	 * @param string $q - Search Keyword
	 */
	public function options($type_id) {
	    header('Content-Type: application/x-json; charset=utf-8');
	    $q = escape(urldecode($this->input->get('q')));
	    $primary_field = 'id';
	    $field = 'name';
	    if(!empty($q)) {
	        $this->db->or_like($field, $q);
	    }
	    $where = '';
	    $type_id = (int) $type_id;
	    $parent_id = (array) urldecode($this->input->get('relation_id'));
	    if(!empty($type_id)) {
	        $where .= 'type_id = ' . $type_id;
	    }
	    if(!empty($parent_id[0])) {
	        $where .= ' AND parent_id IN (' . implode(',', $parent_id) .')';
	    }
	    if(!empty($where)) {
	        $this->db->where($where);
	    }
	    $result = array();
	    $records = $this->db->select('id, name_lang, name')->get('options')->result();
	    
	    if($records) {
	        foreach($records as $record) {
	            $result[] = array(
	                'id' => $record->id,
	                'name' => $record->name_lang == 1 ? lang($record->name) : $record->name
	            );
	        }
	    }
	    echo json_encode($result);
	}
	
	/**
	 * Load countries from a central location and display list to the user
	 */
	public function change_country() {
	    $modal_data = $this->load->view('common/_modal_country_change', null, true);
	    echo $modal_data;
	}
	
	/**
	 * Get Payout Amount
	 * Payouts Module - Pay Method
	 */ 
	public function payout_amount() {
	    header('Content-Type: application/x-json; charset=utf-8');
	    if($this->auth->has_permission('Payouts.Financial.View')) {
    	    $id_user = (int) $this->input->post('user_id');
    	    $item_id = (int) $this->input->post('item_id');
    	    $item_status = (int) $this->input->post('status');
    	    
    	    $payouts_user = $this->session->userdata('payouts_user');
    	    if(empty($payouts_user[$id_user])) {
    	        echo 0;
    	        return false;
    	    }
     	    $payout_user = $payouts_user[$id_user];
     	    $payout_module = $payout_user['module'];
     	    
     	    // get items from the payout_checked_items session variable
     	    $checked_key = $payout_module . '_' . $id_user;
    	    $payout_checked_items = (array) $this->session->userdata('payout_checked_items');
    	    $items = !empty($payout_checked_items[$checked_key]) ? $payout_checked_items[$checked_key] : array();
    	    if($item_status == 1) {
    	        $items[] = $item_id;
    	    } else if(!empty($items)) {
    	        $item_key = array_search ($item_id, $items);
    	        unset($items[$item_key]);
    	    }
    	    if(!empty($items)) { // session is used because of datatable
    	        $payout_checked_items[$checked_key] = $items;
    	        $this->session->set_userdata('payout_checked_items', $payout_checked_items);
    	    }
    	    $payout_amount = 0;
    	    if(!empty($items) && $payouts_user && $id_user > 0 && !empty($payouts_user[$id_user])) {
     	        $user_id = $payout_user['user_id'];
     	        $where = $payout_user['where'];
    	        if($payout_module == 'commissions') {
     	            $where .= " AND C.id IN (" . implode(',', $items) . ")";
     	            $this->load->model('commissions/commissions_model');
    	            $commission_amount = $this->commissions_model->payout_user_commission_amount($user_id, 2, $where);
    	            $payout_amount = $commission_amount->commission_amount;
    	        } else {
     	            $where .= " AND id IN (" . implode(',', $items) . ")";
    	            $this->db->from('order_items PM');
    			    $result_payout = $this->db->select('SUM(PM.payout) AS payout_amount')->where($where)->get()->row();
    			    $payout_amount = $result_payout->payout_amount;
    	        }
    	    }
    	    echo decimal_convert($payout_amount);
    	    return true;
	    } else {
	        echo 0;
	        return false;
	    }
	}
}