<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
 || ##################################################################### ||
 || # GoClixy - Business Directory Script							   # ||
 || # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
 || # ----------------------------------------------------------------  # ||
 || # This file is owned and copyrighted by GoClixy.com. Your license   # ||
 || # confers  no  title  or ownership in the Script and is not a sale  # ||
 || # of any rights to the Software. This file may not be redistributed # ||
 || # in whole or significant part.									   # ||
 || # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
 || # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
 || ##################################################################### ||
 \*=======================================================================*/
// reference the Dompdf namespace
class Background extends CI_Controller {
    /**
     * Execute command in background
     * Note: Data is passed via file_put_contents and retrieved via file_get_contents
     * The file path is in the application/tmp folder.
     * 
     * NOTE: Do not forget to update the public_html/index.php file. The vendor path must be set to real path e.g. /home/fyond/vendor/autoload.php
     * Also update the application/config.php file.
     */
    public function __construct() {
        parent::__construct();
        $this->hooks->enabled = false;
        $this->load->library('settings/settings_lib');
    }
    
    /**
     * Send Email/SMS in Background
     */
    public function emailer() {
        $records = $this->get_task(1);
        if($records) {
            $this->load->library('goclixy/gemailer');
            $this->load->helper('gc');
            foreach($records as $record) {
                $data = json_decode($record->value, true);
                $this->delete_task($record->id);
                $type = $data['gemailer_type'];
                unset($data['gemailer_type']);
                $this->gemailer->_sms_email_pdf($type, $data);
            }
        }
    }
    
    /**
     * Cloudinary Image Save in Background
     */
    public function cloudinary() {
        $records = $this->get_task(2);
        if($records) {
            foreach($records as $record) {
                $data = json_decode($record->value, true);
                $this->delete_task($record->id);
                $this->load->model('images/images_model');
                $this->images_model->save_cloudinary($data['public_id'], $data['file'], $data['action']);
            }
        }
    }
    
    /**
     * Send Socket Notification to the user
     * @return boolean
     */
    public function socket() {
        $records = $this->get_task(3);
        if($records) {
            foreach($records as $record) {
                $data = json_decode($record->value, true);
                $this->delete_task($record->id);
                // send notification to the user
                $this->load->library('socketio');
                $this->socketio->send($data);
            }
        }
    }
    
    /**
     * Resend Socket Notification
     * @param number $id - ID stored in the background notifications table
     */
    public function socket_resend($id) {
        $record = $this->db->where('id', $id)->get('background_notifications')->row();
        if($record) {
            $data = json_decode($record->value, true);
            $data['id'] = $record->id;
            // send notification to the user
            $this->load->library('socketio');
            $this->socketio->send($data, 'resend-notification');
        }
    }
    
    /**
     * Send Onesignal Notification to the user
     * @return boolean
     */
    public function onesignal() {
        $records = $this->get_task(4);
        if($records) {
            foreach($records as $record) {
                $data = json_decode($record->value, true);
                $this->delete_task($record->id);
                // send notification to the user
                $this->load->model('mobiles/mobiles_model');
                $this->mobiles_model->onesignal_notification($data);
            }
        }
    }
    
    /**
     * Send FCM Notification to the user
     * @return boolean
     */
    public function fcm() {
        $records = $this->get_task(5);
        if($records) {
            foreach($records as $record) {
                $data = json_decode($record->value, true);
                $this->delete_task($record->id);
                // send notification to the user
                $this->load->model('mobiles/mobiles_model');
                $this->mobiles_model->fcm_notification($data);
            }
        }
    }
    
    /**
     * Get Task
     * @param number $type_id
     * @return object
     */
    private function get_task($type_id) {
        return $this->db->where('type_id', $type_id)->order_by('id', 'DESC')->get('background_tasks')->result();
    }
    
    /**
     * Delete Background Tasks using ID
     * @param number $id
     */
    private function delete_task($id) {
        $this->db->where('id', $id)->delete('background_tasks');
    }
}