<?php defined('BASEPATH') || exit('No direct script access allowed');
/**
 * Demo Data Collection controller
 *
 * The base controller which displays the homepage of the Bonfire site.
 *
 * @package    Bonfire
 * @subpackage Controllers
 * @category   Controllers
 * @author     Bonfire Dev Team
 * @link       http://guides.cibonfire.com/helpers/file_helpers.html
 *
 */
class Demo extends CI_Controller {
    public function __construct() {
        parent::__construct();
        $ci =& get_instance();
        $ci->hooks->enabled = false;
        $this->load->library('template');
        $this->load->library('assets');
        $this->load->library('form_validation');
        $this->load->library('session');
        $this->load->helper('form');
        $this->load->helper('application');
        $this->lang->load('application');
        $this->lang->load('goclixy');
        $this->load->library('settings/settings_lib');
    }
    
    //--------------------------------------------------------------------
    
    /**
     * Displays the homepage of the Bonfire app
     *
     * @return void
     */
    public function index() {
        if (isset($_POST['save'])) {
            $this->form_validation->set_rules('name', lang('label_name'), 'trim|required');
            $this->form_validation->set_rules('email', lang('label_email'), 'trim|required|valid_email');
            $this->form_validation->set_rules('phone_number', lang('label_phone_number'), 'trim|required|phone_check');
            if($this->form_validation->run() == TRUE) {
                $email = $this->input->post('email');
                $data = array(
                    'name' => $this->input->post('name'),
                    'email' => $email,
                    'phone_number' => $this->input->post('phone_number')
                );
                $result = $this->curl($data);
                if(!empty($result['success'])) {
                    // save data in the session variable
                    $this->session->set_userdata('demo_subscriber', $email);
                    $this->load->helper('cookie');
                    $cookie = array(
                        'name'   => 'demo_subscriber',
                        'value'  => $this->input->ip_address(),
                        'expire' => '86400', // 1 day into seconds
                        'secure' => TRUE
                    );
                    set_cookie($cookie);
                    $message = 'Thank you for your interest in our demo! An email has just been sent to your inbox.<br /><strong>Note: Please do not close your web browser during the demo period, the login information is only valid for the current browser session. You are being redirected to the demo page.</strong>';
                    Template::set_message($message, 'success');
                    if($this->session->userdata('demo_redirect_url')) {
                        $url = $this->session->userdata('demo_redirect_url');
                        header( "refresh:5;url=" . $url );
                    } else {
                        Template::redirect(site_url());
                    }
                } else { // display warning message
                    $message = !empty($result['message']) ? $result['message'] : 'An error occurred, please try again later.';
                    Template::set_message($message, 'danger');
                }
            }
        }
        $this->load->view('demo');
    }
    
    /**
     * Save subscriber data in to database using CURL
     * @param array $data
     * @return mixed
     */
    private function curl($data) {
        $url = 'https://www.goclixy.com/rest/subscribe';
        $data['group_id'] = 3; // change group id for each product
        $data['ip_address'] = $this->input->ip_address();
        $postdata = json_encode($data);
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $postdata);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: application/json'));
        $result = curl_exec($ch);
        curl_close($ch);
        return json_decode($result, true);
    }
    
    /**
     * Display Themes to the User
     */
    public function themes() {
        Assets::add_js('general.js');
        $this->load->model('themes/themes_model');
        $themes = $this->themes_model->all_themes();
        Template::set(array(
            'records' => $themes
        ));
        Template::set_view('themes/demo');
        Template::render('blank');
    }
}