<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Install extends CI_Controller {

	/** @var string The minimum PHP version required to use Bonfire. */
	protected $minVersionPhp = '5.6';
	
    public function __construct()
    {
        parent::__construct();
        
        // Load the basics since Base_Controller is not used here.
		$this->lang->load('routes', 'english');	
        $this->lang->load('application');
		$this->lang->load('goclixy');
        
        // Make sure the template library doesn't try to use sessions.
        $this->load->library('template');
        Template::setSessionUse(false);
        
        $this->load->library('assets');
        $this->load->library('events');
        $this->load->helper('security');
        
        $this->load->helper('application');
        
        // Disable hooks, since they may rely on an installed environment.
        get_instance()->hooks->enabled = false;
        
        // Load the Installer library.
        $this->lang->load('install');
        $this->load->library('installer_lib');
        $this->load->library('form_validation');
        $this->load->library('crypt');
        
        // Set session driver to files because database is not yet created. Also update session files path
        $this->config->set_item('sess_driver', 'files');
        $this->config->set_item('sess_save_path', FCPATH .'assets/cache/ci3_sessions');
        $this->load->library('session');
    }

    //--------------------------------------------------------------------

    public function index() {    	
        error_reporting(0); //Setting this to E_ALL showed that that cause of not redirecting were few blank lines added in some php files.
        
        if (!$this->installer_lib->is_installed()) {

            $data = array();

            // PHP Version Check
            $data['php_min_version']    = '5.6';
            $data['php_acceptable']     = $this->installer_lib->php_acceptable($data['php_min_version']);
            $data['php_version']        = $this->installer_lib->php_version;

            // Curl Enabled?
            $data['curl_enabled']       = $this->installer_lib->cURL_enabled();

            // Files/Folders writeable?
            $data['folders']            = $this->installer_lib->check_folders();
            $data['files']              = $this->installer_lib->check_files();
            
            if(isset($_POST['save'])) {
            	$this->load->library('form_validation');
            	$this->form_validation->set_rules('site_title', lang('in_site_title'), 'required|trim|xss_clean');
            	$this->form_validation->set_rules('site_email', lang('in_site_email'), 'required|trim|xss_clean');
            	$this->form_validation->set_rules('site_description', lang('in_site_description'), 'trim|xss_clean');
            	$this->form_validation->set_rules('db_hostname', lang('in_database'). ' ' .lang('in_hostname'), 'required|xss_clean');
            	$this->form_validation->set_rules('db_username', lang('in_database'). ' ' .lang('in_username'), 'required|trim|xss_clean');
            	$this->form_validation->set_rules('db_password', lang('in_database'). ' ' .lang('in_password'), 'required|trim|xss_clean');
            	$this->form_validation->set_rules('db_name', lang('in_db_name'), 'required|trim|xss_clean');
            
            	if ($this->form_validation->run() === TRUE) {
	            	// First create the database, then create tables, then write config file
	    			if($this->installer_lib->create_database() == false) {
	    				Template::set_message(lang('in_create_database'), 'danger');
	    				$message = 'DB ERROR';
	    			} else if ($this->installer_lib->create_tables() == false) {
	    				Template::set_message(lang('in_create_tables'), 'danger');
	    				$message = 'TABLE NOT CREATED';
	    			} else if ($this->installer_lib->write_config() == false) {
	    				Template::set_message(lang('in_write_config'), 'danger');
	    				$message = 'CONFIG FILE ERROR';
	    			}
	    			 
	    			// If no errors, redirect to registration page
	    			if(!isset($message)) {
	    				$redir = ((isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == "on") ? "https" : "http");
	    				$redir .= "://".$_SERVER['HTTP_HOST'];
	    				$redir .= str_replace(basename($_SERVER['SCRIPT_NAME']),"",$_SERVER['SCRIPT_NAME']);
	    				$redir = str_replace('install/','',$redir);
						
						// Save site email and username to session
	    				$this->session->set_userdata('site_title', $this->input->post('site_title'));
	    				$this->session->set_userdata('site_email', $this->input->post('site_email'));
	    				$this->session->set_userdata('site_description', $this->input->post('site_description'));
						
	    				/**
	    				 * After installation store the timestamp in a variable
	    				 * Pass to do_install function using encrypt
	    				 * Create installed.txt file only if timestamp exists
	    				 * Else redirect user to homepage
	    				 */	    				
	    				$id = $this->crypt->encode(time());
						
	    				// After installation redirect user to do_install method
						redirect(site_url('install/do_install?id='.$id));
	    			}
            	}
       		}
            Template::set($data);
        } else {
            Template::redirect(site_url());
        }        
        Template::set('db_config_path', '../application/config/database.php');
        Template::render('install');
    }
    
    /**
     * Allow user to enter database credentials
     */
    public function database() {
    	$this->load->helper('form');
    	 
    	if ($this->installer_lib->is_installed()) {
    		show_error(lang('in_already_installed'));
    	}
    	 
    	// Does the database table even exist?
    	if ($this->installer_lib->db_settings_exist === FALSE) {
    		show_error( lang('in_need_db_settings') );
    	}
    
    	error_reporting(0); //Setting this to E_ALL showed that that cause of not redirecting were few blank lines added in some php files.
    
    	if($_POST) {
    
    		// Validate the post data
    		if($this->installer_lib->validate_post($_POST) == true)
    		{
    
    			// First create the database, then create tables, then write config file
    			if($this->installer_lib->create_database() == false) {
    				Template::set_message(lang('in_create_database'), 'danger');
    				$message = 'DB ERROR';
    			} else if ($this->installer_lib->create_tables() == false) {
    				Template::set_message(lang('in_create_tables'), 'danger');
    				$message = 'TABLE NOT CREATED';
    			} else if ($this->installer_lib->write_config() == false) {
    				Template::set_message(lang('in_write_config'), 'danger');
    				$message = 'CONFIG FILE ERROR';
    			}
    			 
    			// If no errors, redirect to registration page
    			if(!isset($message)) {
    				$redir = ((isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == "on") ? "https" : "http");
    				$redir .= "://".$_SERVER['HTTP_HOST'];
    				$redir .= str_replace(basename($_SERVER['SCRIPT_NAME']),"",$_SERVER['SCRIPT_NAME']);
    				$redir = str_replace('install/','',$redir);
    				// Save site email and username to session
    				$this->session->set_userdata('site_email', $this->input->post('site_email'));
    				/**
    				 * After installation store the timestamp in a variable
    				 * Pass to do_install function using encrypt
    				 * Create installed.txt file only if timestamp exists
    				 * Else redirect user to homepage
    				 */
    				$id = $this->crypt->encode(time());
    				// After installation redirect user to do_install method
    				redirect(site_url('install/do_install?id='.$id));
    				//header( 'Location: ' . $redir . '' ) ;
    			}
    		}
    		else {
    			Template::set_message(lang('in_validation_errors'), 'danger');
    		}
    	}
    	Template::set('db_config_path', '../application/config/database.php');
    	Template::set_view('install/database_config');
    	Template::render();
    }

    //--------------------------------------------------------------------

    /**
     * Handles the basic installation of the migrations into the database
     * if available, and displays the current status.
     *
     * @return void
     */
    public function do_install() {     
		$this->load->library('crypt');
    	$id = intval($this->crypt->decode($this->input->get('id')));
    	if($id == 0) {
    		Template::redirect(site_url());
    	} 
		
		if ($this->installer_lib->is_installed()) {
	    	show_error(lang('in_already_installed'));
	    } else {
	    	$this->write_installation_file();
	    }
	    				
	    // Does the database table even exist?
	    if ($this->installer_lib->db_settings_exist === FALSE) {
	    	show_error( lang('in_need_db_settings') );
	    }
	    $post_data = array (
			'site_email' => $this->session->userdata('site_email'),
			'site_title' => $this->session->userdata('site_title'),
			'site_description' => $this->session->userdata('site_description')
		);					
	    $this->installer_lib->setup($post_data); // Save site title, description, email and user
	    $this->load->library('statistics');
	    $this->statistics->info();
	    
		$this->session->unset_userdata('site_title');
    	$this->session->unset_userdata('site_description');
    	$this->session->unset_userdata('site_email');
		// Extend home.php controller to Front_Controller instead of CI_Controller
    	file_put_contents('../application/controllers/Home.php', str_replace("extends CI_Controller", "extends Front_Controller", file_get_contents('../application/controllers/Home.php')));
        Template::render();
    }
    
    /**
     * Write a file to /public/install/installed.txt as a simpler
     * check whether it's installed, so developing doesn't require
     * us to remove the install folder.
     * We made it to the end, so we're good to go!
     */
    private function write_installation_file() {
    	$filename = APPPATH .'config/installed.txt';
    	 
    	$msg = 'Installed On: '. date('r') ."\n";
    	$this->load->helper('file');
    	write_file($filename, $msg);
    }

    //--------------------------------------------------------------------

    /**
     * Sends anonymous stastics back to cibonfire.com. These are only used
     * for seeing environments we should be targeting for development.
     *
     * @return [type] [description]
     */
    public function statistics()
    {
        $this->load->library('installer_lib');
        $db = $this->load->database('default', true);

        $data = array(
            'bonfire_version'   => BONFIRE_VERSION,
            'php_version'       => phpversion(),
            'server'            => $this->input->server('SERVER_SOFTWARE'),
            'dbdriver'          => $db->dbdriver,
            'dbserver'          => @mysqli_get_server_info($db->conn_id),
            'dbclient'          => preg_replace('/[^0-9\.]/','', mysqli_get_client_info()),
            'curl'              => $this->installer_lib->cURL_enabled(),
            'server_hash'       => md5($this->input->server('SERVER_NAME').$this->input->server('SERVER_ADDR').$this->input->server('SERVER_SIGNATURE'))
        );

        $data_string = '';

        foreach($data as $key=>$value)
        {
            $data_string .= $key.'='.$value.'&';
        }
        rtrim($data_string, '&');

        $url = 'http://cibonfire.com/stats/collect';

        $ch = curl_init();

        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_POST, count($data));
        curl_setopt($ch, CURLOPT_POSTFIELDS, $data_string);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);

        $result = curl_exec($ch);

        curl_close($ch);
    }

    /**
     * Overwrite files in case of manual installation
     */
    public function manual($write_installation_file = true) {
    	// Extend home.php controller to Front_Controller instead of CI_Controller
    	file_put_contents(APPPATH . 'controllers/Home.php', str_replace("extends CI_Controller", "extends Front_Controller", file_get_contents(APPPATH . 'controllers/Home.php')));
    	// Autoload database library
    	file_put_contents(APPPATH . 'config/autoload.php', str_replace("autoload['libraries'] = array();", "autoload['libraries'] = array('database');", file_get_contents(APPPATH . 'config/autoload.php')));
    	if($write_installation_file) {
    		// write installed.txt file
    		$this->write_installation_file();
    		Template::redirect(site_url());
    	}
    }
    //--------------------------------------------------------------------
    
    /**
     * Import Database
     * @param string $folder_name
     * @param string $file_name
     */
    private function import_dump($folder_name = null , $file_name) {
    	$folder_name = 'dumps';
    	$path = 'assets/backup_db/'; // Codeigniter application /assets
    	$file_restore = $this->load->file($path . $folder_name . '/' . $file_name, true);
    	$file_array = explode(';', $file_restore);
    	foreach ($file_array as $query)
    	{
    		$this->db->query("SET FOREIGN_KEY_CHECKS = 0");
    		$this->db->query($query);
    		$this->db->query("SET FOREIGN_KEY_CHECKS = 1");
    	}
    }

}