<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
 || ##################################################################### ||
 || # GoClixy - Business Directory Script							   # ||
 || # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
 || # ----------------------------------------------------------------  # ||
 || # This file is owned and copyrighted by GoClixy.com. Your license   # ||
 || # confers  no  title  or ownership in the Script and is not a sale  # ||
 || # of any rights to the Software. This file may not be redistributed # ||
 || # in whole or significant part.									   # ||
 || # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
 || # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
 || ##################################################################### ||
 \*=======================================================================*/
class Update extends CI_Controller
{
    /**
     * ORDERS table commission_type replaced with commission_module
     * PAYOUTS and PAYOUT_USERS table type replaced with module_id
     * PRODUCTS table - field added offer_price and
     * ALTER TABLE `rs0d_products` ADD `stock` INT NOT NULL DEFAULT '0' AFTER `offer_price`, ADD `stock_notification` INT NOT NULL DEFAULT '0' AFTER `stock`, ADD `tax_percent` FLOAT NOT NULL DEFAULT '0' AFTER `stock_threshold`, ADD `tax_inclusive` TINYINT(1) NOT NULL DEFAULT '0' AFTER `tax_percent`, ADD `variants` LONGTEXT NULL AFTER `tax_inclusive`;
     * PACKAGES - Commissions field added to the packages table
     * ALTER TABLE `rs0d_packages` ADD `commissions` VARCHAR(450) NULL AFTER `multisale`;
     * Packages form fields settings to be replaced.
     *
     * ORDERS table - item to be changed to an array - Function written in the orders_model.php file order_item_to_array $this->orders_model->order_item_to_array();
     */
    /**
     * CHANGE THE PHP EXECUTION TIME FROM 120 TO A HIGHER VALUE FOR A LARGER DATABASE
     */
    // remove sanitize from the validation list
    // change the v3 with the pasted folder name
    // in case of windows, replace the backslash (/) with forwardslash (\\)
    private $new_version = '/v5/public_html';
    
    // existing website application path
    private $old_apppath = '';
    
    // existing website public_html folder
    private $old_fcpath  = '';
    
    // new version where images will be copied
    private $new_fcpath = '';
    
    // existing website favicon location
    private $old_faviconpath = '';
    
    // site version
    private $version = '5.10.1';
    
    // old version
    private $old_version = '3.5';
    
    // old db information
    private $olddb = '';
    private $olddb_name = '';
    
    public function __construct() {
        parent::__construct();
        // Assumption - directory name must be v4 and it must be inside the public_html folder
        // explode will remove the /v5/public_html from the current FCPATH
        // that is the path of the old website
        $result = explode($this->new_version, FCPATH);
        
        // in case of windows, replace the backslash (/) with forwardslash (\\)
        $this->old_apppath = substr($result[0], 0, strrpos( $result[0], '/')) . '/application/';
        $this->old_fcpath = $result[0] . '/assets/images/';
        $this->new_fcpath = FCPATH .'assets/images/';
        $this->old_faviconpath = $result[0];
        
        $this->load->helper('form');
        $this->load->library('session');
        $this->load->helper('directory');
        $this->load->library('form_validation');
        $this->load->helper('gc');
        $this->load->helper('translate/languages');
        $this->load->library('gc');
        $this->load->model('listings/listings_model');
        $this->load->model('images/images_model');
        $this->load->helper('application');
        
        // Disable hooks, since they may rely on an installed environment.
        get_instance()->hooks->enabled = false;
        $this->olddb = $this->load->database('olddb', TRUE);
        $this->olddb_name = $this->olddb->database;
    }
    
    /**
     * Drop all tables without _bak at the end
     * Reverse tables, remove _bak from the table nameDrop
     *
     */
    public function revert_tables() {
        $this->load->dbforge();
        $tables = $this->db->list_tables();
        foreach ($tables as $table) {
            $table_without_prefix = str_replace($this->db->dbprefix, '', $table);
            // Delete any table with _bak at the end
            if(substr($table_without_prefix, -4) != '_bak') {
                if($table != $this->db->dbprefix . 'ci3_sessions') {
                    $this->dbforge->drop_table($table_without_prefix, TRUE);
                }
            }
        }
        foreach ($tables as $table) {
            $table_without_prefix = str_replace($this->db->dbprefix, '', $table);
            // Delete any table with _bak at the end
            if(substr($table_without_prefix, -4) == '_bak') {
                if($table != $this->db->dbprefix . 'ci3_sessions') {
                    $new_table_name = str_replace('_bak', '', $table_without_prefix);
                    $this->dbforge->rename_table($table_without_prefix, $new_table_name);
                }
            }
        }
        // drop ci3_sessions table with _bak prefix
        $this->dbforge->drop_table('ci3_sessions_bak', TRUE);
    }
    
    /**
     * Perform Migration
     */
    public function index() {
        $version = $this->version;
        $this->lang->load('routes', 'english');
        $this->lang->load('application');
        $this->lang->load('goclixy');
        $this->lang->load('install');
        $is_updated = false;
        $data = array();
        
        // If 'config/update_<VERSION>.txt' exists, the app is already updated
        if (is_file(APPPATH . 'config/update_' . $version .'.txt')) {
            $data['is_updated'] = $is_updated;
        }
        
        if(isset($_POST['submit'])) {
            if (is_file(APPPATH . 'config/update_' . $version .'.txt')) {
                $is_updated = true;
                redirect(site_url());
            }
            if(!$is_updated) {
                $this->_migrate_db();
                
                // Copy all images
                $this->copy_images();
                
                // Copy language files
                // $this->copy_lang_files(); // NOT RECOMMENDED: this will overwrite existing language strings with new one
                
				// Copy logo image
				$logo = $this->db->select('value')->where('name', 'site.logo')->get('settings')->row();
				if(!empty($logo)) {
					copy($this->old_fcpath . $logo->value, $this->new_fcpath . $logo->value);
				}
				
				// Copy logo_contrast image
				$logo_contrast = $this->db->select('value')->where('name', 'site.logo_contrast')->get('settings')->row();
				if(!empty($logo_contrast) && file_exists($this->old_fcpath . $logo_contrast->value)) {
					copy($this->old_fcpath . $logo_contrast->value, $this->new_fcpath . $logo_contrast->value);
				}
				
				// Copy PDF image
				$logo_pdf = $this->db->select('value')->where('name', 'site.logo_pdf')->get('settings')->row();
				if(!empty($logo_pdf) && file_exists($this->old_fcpath . $logo_pdf->value)) {
					copy($this->old_fcpath . $logo_pdf->value, $this->new_fcpath . $logo_pdf->value);
				}
				
				// Copy Analytics Files
				$analytics = $this->db->select('value')->where('name', 'seo.analytics_key_file')->get('settings')->row();
				if(!empty($analytics->value)) {
					if(file_exists($this->old_apppath . 'third_party/Google/' . $analytics->value)) {
						copy($this->old_apppath . 'third_party/Google/' . $analytics->value, APPPATH . 'third_party/Google/' . $analytics->value);
					}
				}
				
                // Display success message to the user
                $data['update_success'] = lang('update_success') . $version;
                
                // Write a file to /config/update_<VERSION>.txt as a simple check
                $this->load->helper('file');
                $filename = APPPATH . 'config/update_' . $version .'.txt';
                $msg = 'GoClixy v' . $version . ' Updated On: ' . date('r') . "\n";
                write_file($filename, $msg);
                
                // Regenerate the Global File
                $this->load->library('themes/widgets');
                $this->widgets->compile_global();
                
                // Regenerate Thumbnails
                $config_modules = gc_modules();
                $config_modules[] = 'users';
                $config_modules[] = 'categories';
                foreach($config_modules as $config_module) {
                    $this->images_model->thumbnail_generate($config_module);
                }
				
                // Tweak logic to fasten search queries
                // Insert query column in the table itself
                $this->gc->_tweak_fields();
            }
        }        
        $data['meta_title'] = lang('update_title') . $version;
        $data['version']	= $version;
        $this->load->view('upgrade/copy', $data);
    }
	
	/**
	 * Migrate DB
	 */
	private function _migrate_db() {
		// To fasten database operation
		// https://stackoverflow.com/questions/2167522/innodb-takes-over-an-hour-to-import-600mb-file-myisam-in-a-few-minutes/2167641#2167641
		$this->db->simple_query('SET unique_checks = 0;');
		$this->db->simple_query('SET foreign_key_checks = 0;');
		$db_tables = (array) $this->db->list_tables();
		$old_db_tables = (array) $this->olddb->list_tables();
		$old_dates_table = array('tags', 'videos', 'search_queries', 'business_queries', 'listing_erroneous', 'click_to_call', 'categories', 'cities');
		foreach($old_dates_table as $old_date_table) {
			if(in_array($this->olddb->dbprefix . $old_date_table, $old_db_tables)) {
				$old_table_fields = (array) $this->olddb->list_fields($old_date_table);
				if(in_array('created_on', $old_table_fields)) {
					$this->olddb->where("created_on = '0000-00-00 00:00:00'")->update($old_date_table, array('created_on' => date('Y-m-d H:i:s')));
				}
				if(in_array('modified_on', $old_table_fields)) {
					$this->olddb->where("modified_on = '0000-00-00 00:00:00'")->update($old_date_table, array('modified_on' => null));
				}
			}
		}
		
		// migrate users
		$this->olddb->where("display_name_changed = '0000-00-00'")->update('users', array('display_name_changed' => null));
		$this->olddb->where("created_on = '0000-00-00 00:00:00'")->update('users', array('created_on' => date('Y-m-d H:i:s')));
		$users_table_fields = (array) $this->olddb->list_fields('users');
		if(in_array('otp_expires_on', $users_table_fields)) {
			$this->olddb->where("otp_expires_on = '0000-00-00 00:00:00'")->update('users', array('otp_expires_on' => null));
		}
		// $this->olddb->where("tfa_retry_on = '0000-00-00 00:00:00'")->update('users', array('tfa_retry_on' => null));
		
		// migrate listings
		$this->olddb->where("expires_on = '0000-00-00'")->update('listings', array('expires_on' => null));
		
		// migrate banners
		$this->olddb->where("start_date = '0000-00-00'")->update('banners', array('start_date' => null));
		$this->olddb->where("end_date = '0000-00-00'")->update('banners', array('end_date' => null));
		
		// migrate subscribers
		$subscribers_fields = (array) $this->db->list_fields('subscriber_users');
		if(!in_array('user_id', $subscribers_fields)) { // add user_id column if does not exist
			$this->db->query('ALTER TABLE ' . $this->olddb->dbprefix . 'subscriber_users ADD user_id BIGINT(20) DEFAULT 0 AFTER status_id;');
		}
		if(in_array($this->olddb->dbprefix . 'subscribers', $old_db_tables)) {
			$subscribers_fields = (array) $this->olddb->list_fields('subscribers');
			if(!in_array('group_ids', $subscribers_fields)) {
				$this->olddb->query('ALTER TABLE ' . $this->olddb->dbprefix . 'subscribers ADD group_ids VARCHAR(455) DEFAULT 1 AFTER id;');
			}
			if(!in_array('created_ip', $subscribers_fields)) {
				$this->olddb->query('ALTER TABLE ' . $this->olddb->dbprefix . 'subscribers CHANGE ip created_ip VARCHAR(50);');
			}
			if(!in_array('created_on', $subscribers_fields)) {
				$this->olddb->query('ALTER TABLE ' . $this->olddb->dbprefix . 'subscribers CHANGE subscribed_on created_on DATETIME;');
			}
			if(!in_array('country_iso', $subscribers_fields)) {
				$this->olddb->query('ALTER TABLE ' . $this->olddb->dbprefix . 'subscribers ADD country_iso CHAR(2) DEFAULT \'IN\' AFTER email;');
			}
			$this->_copy_tables('subscriber_users', 'subscribers');
		}
		
		// migrate ratings and other tables
		$this->olddb->where("listing_expiry = '0000-00-00 00:00:00'")->update('listing_claims', array('listing_expiry' => null));
		
		$db_fields = (array) $this->olddb->list_fields('business_queries');
		if(!in_array('phone_number', $db_fields)) {
			$this->olddb->query('ALTER TABLE ' . $this->olddb->dbprefix . 'business_queries CHANGE phone phone_number VARCHAR(20);');
		}
		if(!in_array('created_on', $db_fields)) {
			$this->olddb->query('ALTER TABLE ' . $this->olddb->dbprefix . 'business_queries CHANGE posted_on created_on DATETIME;');
		}
		if(!in_array('created_ip', $db_fields)) {
			$this->olddb->query('ALTER TABLE ' . $this->olddb->dbprefix . 'business_queries CHANGE ip created_ip VARCHAR(50);');
		}
		if(!in_array('status_id', $db_fields)) {
			$this->olddb->query('ALTER TABLE ' . $this->olddb->dbprefix . 'business_queries CHANGE status status_id TINYINT(1);');
		}
		$this->olddb->update('business_queries', array('status_id' => 2));
		
		$db_fields = (array) $this->olddb->list_fields('contact_queries');
		if(!in_array('phone_number', $db_fields)) {
			$this->olddb->query('ALTER TABLE ' . $this->olddb->dbprefix . 'contact_queries CHANGE phone phone_number VARCHAR(20);');
		}
		if(!in_array('created_on', $db_fields)) {
			$this->olddb->query('ALTER TABLE ' . $this->olddb->dbprefix . 'contact_queries CHANGE posted_on created_on DATETIME;');
		}
		if(!in_array('ip_address', $db_fields)) {
			$this->olddb->query('ALTER TABLE ' . $this->olddb->dbprefix . 'contact_queries CHANGE ip ip_address VARCHAR(50);');
		}
		
		$db_fields = (array) $this->olddb->list_fields('listing_erroneous');
		if(!in_array('phone_number', $db_fields)) {
			$this->olddb->query('ALTER TABLE ' . $this->olddb->dbprefix . 'listing_erroneous CHANGE phone phone_number VARCHAR(20);');
		}
		if(!in_array('created_on', $db_fields)) {
			$this->olddb->query('ALTER TABLE ' . $this->olddb->dbprefix . 'listing_erroneous CHANGE reported_on created_on DATETIME;');
		}
		if(!in_array('created_ip', $db_fields)) {
			$this->olddb->query('ALTER TABLE ' . $this->olddb->dbprefix . 'listing_erroneous CHANGE ip created_ip VARCHAR(50);');
		}
		
		// leads table data
		$db_fields = (array) $this->olddb->list_fields('leads');
		if(in_array('replied_on', $db_fields)) {
			$this->olddb->where("replied_on = '0000-00-00 00:00:00'")->update('leads', array('replied_on' => null));
		}
		if(in_array('deleted_by', $db_fields)) {
			$this->olddb->where("deleted_by IS NULL")->update('leads', array('deleted_by' => 0));
		}
		
		// payouts table
		$db_fields = (array) $this->olddb->list_fields('payouts');
		if(in_array('deleted_by', $db_fields)) {
			$this->olddb->where("deleted_by IS NULL")->update('payouts', array('deleted_by' => 0));
		}
		
		// order histories status information
		$db_fields = (array) $this->olddb->list_fields('order_histories');
		if(!in_array('new_order_status_id', $db_fields)) {
			$this->olddb->query('ALTER TABLE ' . $this->olddb->dbprefix . 'order_histories ADD new_order_status_id SMALLINT(5) DEFAULT 0 AFTER order_status;');
			$this->olddb->query('ALTER TABLE ' . $this->olddb->dbprefix . 'order_histories CHANGE order_status old_order_status_id SMALLINT(5);');
			$this->olddb->query('UPDATE ' . $this->olddb->dbprefix . 'order_histories SET new_order_status_id = old_order_status_id + 1');
		}
		
		// product variants
		$db_fields = (array) $this->olddb->list_fields('product_variants');
		if(!in_array('sku', $db_fields)) {
			$this->olddb->query('ALTER TABLE ' . $this->olddb->dbprefix . 'product_variants ADD sku VARCHAR(50) DEFAULT 0 AFTER value_id;');
		}
		
		// migrate social login settings
		$this->_copyrows('hybrid_auth_settings');
		$skip_tables = array(
			'activities', 'ad_email_templates', 'affiliate_email_templates', 'background_tasks', 'banner_api', 'callback_email_templates',
			'ci3_sessions', 'countries', 'contact_queries_email_templates', 'cronjobs', 'cron_translations', 'currencies', 'currency_rates',
			'feedback_email_templates', 'forum_email_templates', 'franchise_email_templates', 'google_listings', 'groups', 'languages', 'lead_email_templates',
			'fields', 'field_modules', 'field_options', 'field_positions', 'field_position_groups', 'field_relations',
			'links', 'link_menus', 'link_relations', 'login_attempts', 'marketplace', 'migrations', 'modules', 'module_email_templates',
			'options', 'option_hierarchies', 'option_relations', 'option_types', // comments these in the next version
			'order_email_templates', 'payment_gateways', 'payout_email_templates', 'permissions', 'query_email_templates',
			'roles', 'role_permissions', 'schema_version', 'sessions', 'settings', 'sms_gateways',
			'themes', 'theme_blocks', 'theme_demos', 'theme_folders', 'theme_settings', 'theme_views', 'user_email_templates',
			'hybrid_auth_settings' // key field is creating issue
		);        
		$live_tables = array();
		foreach($db_tables as $db_table) {
			$live_tables[] = str_replace($this->db->dbprefix, '', $db_table);
		}
		// migrate tables data
		$old_categories_relation = false;
		foreach($old_db_tables as $old_db_table) {
			$old_db_table = str_replace($this->olddb->dbprefix, '', $old_db_table);
			if(!in_array($old_db_table, $skip_tables) && in_array($old_db_table, $live_tables)) {
				switch($old_db_table) {
					case 'categories_relation':
						$categories_relation_fields = (array) $this->olddb->list_fields('categories_relation');
						if(in_array('sub_id', $categories_relation_fields)) {
							$old_categories_relation = true;
						} else {
							$this->_copy_tables($old_db_table, $old_db_table);
						}
						break;
					default:
						$this->_copy_tables($old_db_table, $old_db_table);                                
				}
			}
		}
		
		// old categories relation table
		if($old_categories_relation) {                    
			// categories handling
			$this->db->truncate('categories_hierarchy');
			$this->db->truncate('categories_relation');
			$this->load->library('goclixy/gclosure');
			$gclosure = new Gclosure('categories', 'categories_hierarchy');
			$records = $this->db->get('categories')->result();
			if($records) {
				foreach($records as $record) {
					// save hierarchy
					$gclosure->add($record->id, ($record->parent_id == 0 ? $record->id : $record->parent_id));
				}
			}
			
			// insert subsub categories first
			$sql = "INSERT INTO " . $this->db->dbprefix . "categories_relation (module_id, row_id, category_id) (SELECT module_id, row_id, subsub_id AS category_id FROM " . $this->olddb_name . "." . $this->olddb->dbprefix . "categories_relation WHERE subsub_id != 0)";
			$this->db->query($sql);
			
			// insert sub categories second
			$sql = "INSERT INTO " . $this->db->dbprefix . "categories_relation (module_id, row_id, category_id) (SELECT module_id, row_id, sub_id AS category_id FROM " . $this->olddb_name . "." . $this->olddb->dbprefix . "categories_relation WHERE subsub_id = 0 AND sub_id != 0)";
			$this->db->query($sql);
			
			// insert primary categories last
			$sql = "INSERT IGNORE INTO " . $this->db->dbprefix . "categories_relation (module_id, row_id, category_id) (SELECT module_id, row_id, parent_id AS category_id FROM " . $this->olddb_name . "." . $this->olddb->dbprefix . "categories_relation WHERE subsub_id = 0 AND sub_id = 0 AND parent_id != 0)";
			$this->db->query($sql);
			
			// update modules and icons
			$this->update_categories();
		}
		
		// update locations tables
		$sql = "UPDATE " . $this->db->dbprefix . "states SET country_id = (SELECT id FROM " . $this->db->dbprefix . "countries WHERE iso = " . $this->db->dbprefix . "states.country_iso)";
		$this->db->query($sql);
		$sql = "UPDATE " . $this->db->dbprefix . "cities SET country_iso = (SELECT country_iso FROM " . $this->db->dbprefix . "states WHERE id = " . $this->db->dbprefix . "cities.state_id)";
		$this->db->query($sql);
		$sql = "UPDATE " . $this->db->dbprefix . "cities SET country_id = (SELECT id FROM " . $this->db->dbprefix . "countries WHERE iso = " . $this->db->dbprefix . "cities.country_iso), country = (SELECT printable_name FROM " . $this->db->dbprefix . "countries WHERE iso = " . $this->db->dbprefix . "cities.country_iso), state = (SELECT name FROM " . $this->db->dbprefix . "states WHERE id = " . $this->db->dbprefix . "cities.state_id)";
		$this->db->query($sql);
		$sql = "UPDATE " . $this->db->dbprefix . "cities INNER JOIN " . $this->db->dbprefix . "states ON " . $this->db->dbprefix . "states.id = " . $this->db->dbprefix . "cities.state_id SET " . $this->db->dbprefix . "cities.state = " . $this->db->dbprefix . "states.name";
		$this->db->query($sql);
		$sql = "UPDATE " . $this->db->dbprefix . "localities INNER JOIN " . $this->db->dbprefix . "cities ON " . $this->db->dbprefix . "localities.city_id = " . $this->db->dbprefix . "cities.id SET " . $this->db->dbprefix . "localities.country_iso = " . $this->db->dbprefix . "cities.country_iso, " . $this->db->dbprefix . "localities.state_id = " . $this->db->dbprefix . "cities.state_id";
		$this->db->query($sql);
		
		// settings
		$this->_copyrows('settings');
		
		// update banner images url
		// $sql = "UPDATE " . $this->db->dbprefix . "banners SET image = CONCAT('banners/', image) WHERE image IS NOT NULL;";
		// $this->db->query($sql);
		
		// Re-enable Unique and Foreign Key Checks
		$this->db->simple_query('SET unique_checks = 1;');
		$this->db->simple_query('SET foreign_key_checks = 1;');
	}
    
    /**
     * Update icons and module_relations information
     */
    private function update_categories() {
        $records = $this->db->where('module_relations IS NULL')->get('categories')->result();
        if($records) {
            $this->load->library('slug');
            $this->load->model('categories/categories_model');
            $selected_modules = gc_modules();
            $selected_modules[] = 'videos';
            $flipped_modules = array();
            foreach($selected_modules as $selected_module) {
                $flipped_modules[$selected_module] = gc_module_id($selected_module);
            }
            $modules_selected = array();
            if(!empty($selected_modules)) {
                foreach($selected_modules as $selected_module) {
                    $modules_selected[] = gc_module_id($selected_module);
                }
            }
            foreach($records as $record) {
                if(!empty($record->name)) {
                    $data = array();
                    $data['module_relations']   = implode(',', $modules_selected);
                    if(empty($record->slug)) {
                        $data['slug'] = $this->slug->create_slug($record->name);
                    }
                    if(empty($record->icon)) {
                        $data['icon'] = $this->categories_model->random_icon();
                    }
                    $this->db->where('id', $record->id)->update('categories', $data);
                }
            }
        }
    }
    
    /**
     * Copy Tables using MySQL command - Links and Fields
     * @param string $table_source - Source Table Name
     * @param string $table_bak - Backup/Old Table Name
     * @param boolean $insert_only - TRUE - Perform insert only do not update
     */
    private function _copy_tables($table_source, $table_bak, $insert_only = false) {
		/*if($table_source != 'comments') {
			$this->db->truncate($table_source);
		}*/
		$this->db->truncate($table_source);
        $existing_column_names = $this->olddb->list_fields($table_bak);
        $new_column_names = $this->db->list_fields($table_source);
        $columns = array_intersect($existing_column_names, $new_column_names);
        if(!empty($columns)) {
            $columns_comma_separated = implode(',', $columns);
            $sql = "INSERT";
            if($insert_only) {
                $sql .= " IGNORE";
            }
            $sql .= " INTO " . $this->db->dbprefix . $table_source . " (" . $columns_comma_separated . ")";
            $sql .= " SELECT " . $columns_comma_separated . " FROM " . $this->olddb_name . '.' . $this->olddb->dbprefix . $table_bak;
            if(!$insert_only) {
                $duplicate_columns = array();
                foreach($columns as $column) {
                    $duplicate_columns[] = $column . '= VALUES(' . $column . ')';
                }
                $sql .= " ON DUPLICATE KEY UPDATE " . implode(',', $duplicate_columns);
            }
            $this->db->query($sql);
        }
    }
    
    /**
     * Migrate rows
     * Copy data from existing tables to new database
     */
    private function _copyrows($table, $singledb = true) {
        $NEWDB = $this->load->database('default', TRUE);
        $OLDDB = $this->load->database('olddb', TRUE);
        
        // check if table exists in old database
        if (!$OLDDB->table_exists($table) ) {
            return false;
        }
        // get fields from both tables
        $NEWDB_columns = $NEWDB->list_fields($table);
        $OLDDB_columns = $OLDDB->list_fields($table);
        
        // get common fields
        $common_fields = array_intersect($NEWDB_columns, $OLDDB_columns);
        if(!empty($common_fields)) {
            switch($table) {
                case 'users':
                    // get mobile_number as phone_number from the users table
                    if(in_array('mobile_number', $OLDDB_columns)) {
                        array_push($common_fields, "mobile_number AS phone_number");
                    }
                    break;
                case 'jobs':
                    if(in_array('location', $OLDDB_columns)) {
                        array_push($common_fields, "location AS address");
                    }
                    break;
                case 'comments':
                case 'ratings':
                case 'business_queries':
                    // Comments, Ratings, Business Queries
                    if(in_array('listing_id', $OLDDB_columns)) {
                        array_push($common_fields, "1 AS module_id, listing_id AS row_id");
                    }
                    break;
                    /*case 'custom_fields': // Table Deleted
                     // Custom Fields
                     if(!in_array('module_id', $OLDDB_columns)) {
                     array_push($common_fields, "1 AS module_id");
                     }
                     break;*/
                case 'packages':
                    // qrcode has no default value, this was failing packages insert
                    array_push($common_fields, "1 AS qrcode");
                    break;
                case 'currency_rates':
                    if(in_array('from', $OLDDB_columns)) {
                        array_push($common_fields, "from AS currency_from, to AS currency_to");
                    }
                    break;
            }
            if($table == 'orders' && $this->old_version < 3.5) {
                $fields = array();
                foreach($common_fields as $field) {
                    // remove order_type as this field has integer value
                    // as per new changes
                    if($field != 'order_type') {
                        $fields[]= 'O.' . $field;
                    }
                }
                $sql = "SELECT " . implode(',', $fields);
                $sql .= ", CASE O.order_type WHEN 'new' THEN 1 WHEN 'claims' THEN 2 END AS order_type";
                $sql .= ", CASE O.order_type WHEN 'new' THEN O.package_id WHEN 'claims' THEN L.package_id END AS package_id";
                $sql .= ", 1 AS subscription_type";
                $sql .= " FROM " . $this->db->dbprefix . "orders_bak O";
                $sql .= " LEFT JOIN " . $this->db->dbprefix . "listing_claims_bak LC ON LC.order_id = O.order_id";
                $sql .= " LEFT JOIN " . $this->db->dbprefix . "listings_bak L ON L.id = LC.listing_id";
                $result = $this->db->query($sql)->result();
            } else {
                $result = $OLDDB->select(implode(',', $common_fields))->get($table)->result();
            }
            
            if($result) {
                // do not truncate in case of payment and sms gateways
                switch($table) {
                    case 'email_templates':
                    case 'payment_gateways':
                    case 'sms_gateways':
                        $this->db->update_batch($table, $result, 'id');
                        break;
                    case 'comments':
                        $this->db->truncate('comments_hierarchy');
                        $this->db->empty_table($table);
                        $this->db->insert_batch($table, (array) $result);
                        // Add comments hierarchy only if the existing comments_hierarchy is empty
                        if (!$this->db->table_exists('comments_hierarchy_bak') ) {
                            // $hierarchy = $this->db->count_all_results('comments_hierarchy_bak');
                            // if($hierarchy == 0) {
                            $this->load->model('comments/comments_model');
                            $comments = $this->comments_model->select('id')->find_all();
                            if($comments) {
                                foreach($comments as $comment) {
                                    // save comment hierarchy
                                    $this->comments_model->add($comment->id, $comment->id);
                                }
                            }
                            //}
                        }
                        break;
                    case 'orders':
                        $this->db->truncate('orders');
                        if(!empty($result)) {
                            if($this->old_version < 3.5) {
                                $order_data = array();
                                foreach($result as $orders) {
                                    $order = array();
                                    $item = array();
                                    foreach($orders as $key => $value) {
                                        switch($key) {
                                            case 'title':
                                            case 'description':
                                                $item[$key] = $value;
                                                break;
                                            default:
                                                $order[$key] = $value;
                                        }
                                    }
                                    
                                    if($order['order_type'] == 1 && !empty($item['description'])) {
                                        $description = str_replace('<li class="list-group-item">', '<li class="package-list-item">', $item['description']);
                                        $item['description'] = '<ul class="package-list">' . $description . '</ul>';
                                    } else if($order['order_type'] == 2 && !empty($item['title'])) {
                                        $item['title'] = 'Claim: ' . $item['title'];
                                    }
                                    $order['item'] = json_encode($item);
                                }
                                if(!empty($order_data)) {
                                    $this->db->insert_batch($table, $order_data);
                                }
                            } else {
                                $this->db->insert_batch($table, (array) $result);
                            }
                        }
                        break;
                    case 'settings':
                        $this->db->update_batch($table, $result, 'name');
                        // GoClixy-f0118f1f4ed0.p12
                        $analytics = $this->db->select('value')->where('name', 'seo.analytics_key_file')->get($table)->row();
                        if(!empty($analytics->value) && $analytics->value == 'GoClixy-f0118f1f4ed0.p12') {
                            $this->db->where('name', 'seo.analytics_key_file')->update($table, array('value' => ''));
                        }
                        // Update site version
                        $this->db->where('name', 'site.version')->update($table, array('value' => $this->version));
                        break;
                    case 'links':
                    case 'link_locations':
                        if($table == 'link_locations') {
                            // create index first
                            $columns = array('link_id', 'location_id');
                            $this->check_index($table, $columns, true, true);
                        }
                        // copy new links
                        $this->copy_tables($table, $table . '_bak', true);
                        break;
                    case 'tags':
                    case 'users':
                        $this->db->truncate($table);
                        if(!empty($result)) {
                            foreach($result as $key => $data) {
                                $insert_query = $this->db->insert_string($table, $data);
                                $insert_query = str_replace('INSERT INTO','INSERT IGNORE INTO', $insert_query);
                                $this->db->query($insert_query);
                            }
                        }
                        break;
                    default:
                        $this->db->truncate($table);
                        if(!empty($result)) {
                            $this->db->insert_batch($table, (array) $result);
                        }
                }
            }
        }
    }
    
    // UPDATE CATEGORIES LEVEL
    // CHANGE VERSION IN THE SETTINGS
    /**
     * UPDATE Considerations: YouTube URL
     */
    public function index_() {
        $version = $this->version;
        $this->lang->load('routes', 'english');
        $this->lang->load('application');
        $this->lang->load('goclixy');
        $this->lang->load('install');
        $is_updated = false;
        $data = array();
        
        // If 'config/update_<VERSION>.txt' exists, the app is already updated
        if (is_file(APPPATH . 'config/update_' . $version .'.txt')) {
            $data['is_updated'] = $is_updated;
        }
        
        if(isset($_POST['submit'])) {
            if (is_file(APPPATH . 'config/update_' . $version .'.txt')) {
                $is_updated = true;
                redirect(site_url());
            }
            if(!$is_updated) {
                // To fasten database operation
                // https://stackoverflow.com/questions/2167522/innodb-takes-over-an-hour-to-import-600mb-file-myisam-in-a-few-minutes/2167641#2167641
                $this->db->simple_query('SET unique_checks = 0;');
                $this->db->simple_query('SET foreign_key_checks = 0;');
                // Important - Drop Comments Hiearchy Foreign Keys
                $this->db->simple_query('ALTER TABLE '. $this->db->dbprefix .'comments_hierarchy DROP FOREIGN KEY `comments_hierarchy_ibfk_1`');
                $this->db->simple_query('ALTER TABLE '. $this->db->dbprefix .'comments_hierarchy DROP FOREIGN KEY `comments_hierarchy_ibfk_2`');
                $this->rename_existing_tables();
                if($this->old_version < '3.5') {                    
                    // Copy Images Information before copying any further information
                    // Truncate new images table first
                    $this->db->truncate('images');
                    $this->module_images();
                }
                
                // Tables added in version 3.5 and higher
                $this->copyrows('images');
                $this->copyrows('ads');
                $this->copyrows('ad_costs');
                $this->copyrows('ad_keywords');
                $this->copyrows('ad_packages');
                $this->copyrows('ad_views');
                $this->copyrows('affiliate_coupons');
                $this->copyrows('attributes');
                $this->copyrows('attribute_values');
                $this->copyrows('bookings');
                $this->copyrows('booking_allotments');
                $this->copyrows('booking_allotment_fields');
                $this->copyrows('booking_fields');
                $this->copyrows('booking_field_relations');
                $this->copyrows('booking_forms');
                $this->copyrows('booking_hierarchy');
                $this->copyrows('booking_metas');
                $this->copyrows('booking_rules');
                $this->copyrows('cancellations');
                $this->copyrows('commissions');
               // $this->copyrows('fields'); // New fields are added to the new version
               // $this->copyrows('field_modules');
               // $this->copyrows('field_relations');
               // $this->copyrows('groups');
                $this->copyrows('languages');
                $this->copyrows('leads');
                $this->copyrows('lead_emails');
                $this->copyrows('links');
                $this->copyrows('link_locations');
                $this->copyrows('locale');
                $this->copyrows('metas');
                $this->copyrows('order_items');
                $this->copyrows('order_shipments');
                $this->copyrows('product_variants');
                $this->copyrows('translations');
                
                // Older Tables
                $this->copyrows('banners');
                $this->copyrows('banner_categories');
                $this->copyrows('banner_locations');
                $this->copyrows('banner_types');
                $this->copyrows('blogs');
                $this->copyrows('branches');
                $this->copyrows('business_hours');
                $this->copyrows('business_queries');
                $this->copyrows('categories');
                $this->copyrows('cities');
                $this->copyrows('classifieds');
                $this->copyrows('click_to_call');
                $this->copyrows('comments');
                $this->copyrows('contact_queries');
                $this->copyrows('countries');
                $this->copyrows('coupons');
                $this->copyrows('currencies');
                $this->copyrows('currency_rates');
                $this->copyrows('deals');
                $this->copyrows('email_queue');
                $this->copyrows('module_email_templates'); // Do not overwrite email templates as they are modified a lot in the latest version 3.5
                $this->copyrows('events');
                $this->copyrows('favorites');
                $this->copyrows('hybrid_auth_settings');
                $this->copyrows('invoices');
                $this->copyrows('jobs');
                
                // Copy Images Only After Listings table insert
                $this->copyrows('listings');
                
                // Update QRCodes path
                $this->qrcodes();
                
                $this->copyrows('listing_claims');
                $this->copyrows('listing_erroneous');
                
                // A category can belong to multiple modules
                // insert relation from the categories table to the module relations table
                if($this->old_version < '3.5') { 
                    $this->module_categories_relation();
                    
                    // Move categories relation in to the categories_relation table
                    $this->module_categories();
                    
                    // Move Tags from the modules table in to the tag_relations table
                    $this->module_tags();
                }                
                
                // update youtube videos URL from id to a URL save id in the youtube_id field and complete URL in the url field.
                $this->copyrows('localities');
                $this->copyrows('mobile_devices');
                $this->copyrows('mobile_notifications');
                $this->copyrows('mobile_slider');
                $this->copyrows('module_relations');
                $this->copyrows('news');
                $this->copyrows('product_notifications');
                $this->copyrows('orders');
                $this->copyrows('order_coupons');
                $this->copyrows('order_customers');
                $this->copyrows('order_histories');
                $this->copyrows('packages');
                $this->copyrows('pages');
                $this->copyrows('payment_gateways');
                $this->copyrows('payouts');
                // $this->copyrows('payout_histories');
                $this->copyrows('payout_users');
                $this->copyrows('products');
                $this->copyrows('properties');
                $this->copyrows('query_replies');                
                $this->copyrows('ratings');
                $this->copyrows('rating_users');
                // $this->copyrows('recently_viewed'); // Do not copy recently viewed items
                $this->copyrows('search_queries');
                $this->copyrows('settings');
                $this->copyrows('sliders');
                $this->copyrows('sms_gateways');
                $this->copyrows('states');
                $this->copyrows('subscribers');
                $this->copyrows('tags');
                $this->copyrows('tag_locations');
                $this->copyrows('tag_relations');
                $this->copyrows('tax_rates');
                $this->copyrows('testimonials');
                // $this->copyrows('themes');
                // $this->copyrows('theme_settings');
                // $this->copyrows('theme_views');
                $this->copyrows('unsubscribers');
                $this->copyrows('users');
                $this->copyrows('product_variants');
                $this->copyrows('videos');
                $this->copyrows('wallets_points');
                $this->copyrows('product_wishlists');
                
                // Tweak logic to fasten search queries
                // Insert query column in the table itself
                $this->gc->_tweak_fields();
                
                // Copy logo also
                if ($this->db->table_exists('settings_bak') ) {
                    $logo = $this->db->select('value')->where('name', 'site.logo')->get('settings_bak')->row();
                    if(!empty($logo)) {
                        copy($this->old_fcpath . $logo->value, $this->new_fcpath . $logo->value);
                    }
                    // Copy Analytics Files
                    $analytics = $this->db->select('value')->where('name', 'seo.analytics_key_file')->get('settings_bak')->row();
                    if(!empty($analytics->value)) {
                        if(file_exists($this->old_apppath . 'third_party/Google/' . $analytics->value)) {
                            copy($this->old_apppath . 'third_party/Google/' . $analytics->value, APPPATH . 'third_party/Google/' . $analytics->value);
                        }
                    }
                }
				
                // packages trusted seal also
                if ($this->db->table_exists('packages_bak') ) {
                    $packages = $this->db->select('trusted_seal')->get('packages_bak')->result();
                    if(!empty($packages)) {
                        foreach($packages as $package) {
                            if(!empty($package->trusted_seal)) {
                                if(file_exists($this->old_fcpath . $package->trusted_seal)) {
                                    copy($this->old_fcpath . $package->trusted_seal, $this->new_fcpath . $package->trusted_seal);
                                }
                            }
                        }
                    }
                }
                
                // make v4 database changes
                $this->v4_changes();
                
                // Drop backup tables only after copying images
                // Logo and trusted_seal images must also be copied
                $this->drop_backup_tables();
                
                // Re-enable Unique and Foreign Key Checks
                $this->db->simple_query('SET unique_checks = 1;');
                $this->db->simple_query('SET foreign_key_checks = 1;');
                
                // Copy all images
                $this->copy_images();
                
                // Copy language files
                $this->copy_lang_files();
                
                // Handle application_lang.php split in the goclixy_lang.php file
                $this->split_application_lang('goclixy_lang.php', 'application_lang.php');
                $this->split_application_lang('application_lang.php', 'application_lang.php');
                
                // Display success message to the user
                $data['update_success'] = lang('update_success') . $version;
                
                // Write a file to /config/update_<VERSION>.txt as a simple check
                $this->load->helper('file');
                $filename = APPPATH . 'config/update_' . $version .'.txt';
                $msg = 'GoClixy v' . $version . ' Updated On: ' . date('r') . "\n";
                write_file($filename, $msg);
                
                // Regenerate the Global File
                $this->load->library('themes/widgets');
                $this->widgets->compile_global();
                
                // Regenerate Thumbnails
                $config_modules = gc_modules();
                $config_modules[] = 'users';
                $config_modules[] = 'categories';
                foreach($config_modules as $config_module) {
                    $this->images_model->thumbnail_generate($config_module);
                }
            }
        }
        $data['meta_title'] = lang('update_title') . $version;
        $data['version']	= $version;
        $this->load->view('upgrade/copy', $data);
    }
    
    /**
     * Migrate rows
     * Copy data from existing tables to new database
     */
    private function copyrows($table, $singledb = true) {
        if($singledb) {
            /*$NEWDB = $this->load->database('default', TRUE);
             $OLDDB = $this->load->database('olddb', TRUE);
             
             // check if table exists in old database
             if (!$OLDDB->table_exists($table) ) {
             return false;
             }
             // get fields from both tables
             $NEWDB_columns = $NEWDB->list_fields($table);
             $OLDDB_columns = $OLDDB->list_fields($table);*/
        } else {
            
        }
        
        // do not proceed further, if a table does not exist
        if (!$this->db->table_exists($table . '_bak') ) {
            return false;
        }
        
        // get fields from both tables
        $NEWDB_columns = $this->db->list_fields($table);
        $OLDDB_columns = $this->db->list_fields($table . '_bak');
        
        // get common fields
        $common_fields = array_intersect($NEWDB_columns, $OLDDB_columns);
        if(!empty($common_fields)) {
            switch($table) {
                case 'users':
                    // get mobile_number as phone_number from the users table
                    if(in_array('mobile_number', $OLDDB_columns)) {
                        array_push($common_fields, "mobile_number AS phone_number");
                    }
                    break;
                case 'jobs':
                    if(in_array('location', $OLDDB_columns)) {
                        array_push($common_fields, "location AS address");
                    }
                    break;
                case 'comments':
                case 'ratings':
                case 'business_queries':
                    // Comments, Ratings, Business Queries
                    if(in_array('listing_id', $OLDDB_columns)) {
                        array_push($common_fields, "1 AS module_id, listing_id AS row_id");
                    }
                    break;
                    /*case 'custom_fields': // Table Deleted
                     // Custom Fields
                     if(!in_array('module_id', $OLDDB_columns)) {
                     array_push($common_fields, "1 AS module_id");
                     }
                     break;*/
                case 'packages':
                    // qrcode has no default value, this was failing packages insert
                    array_push($common_fields, "1 AS qrcode");
                    break;
                case 'currency_rates':
                    if(in_array('from', $OLDDB_columns)) {
                        array_push($common_fields, "from AS currency_from, to AS currency_to");
                    }
                    break;
            }
            if($table == 'orders' && $this->old_version < 3.5) {
                $fields = array();
                foreach($common_fields as $field) {
                    // remove order_type as this field has integer value
                    // as per new changes
                    if($field != 'order_type') {
                        $fields[]= 'O.' . $field;
                    }
                }
                $sql = "SELECT " . implode(',', $fields);
                $sql .= ", CASE O.order_type WHEN 'new' THEN 1 WHEN 'claims' THEN 2 END AS order_type";
                $sql .= ", CASE O.order_type WHEN 'new' THEN O.package_id WHEN 'claims' THEN L.package_id END AS package_id";
                $sql .= ", 1 AS subscription_type";
                $sql .= " FROM " . $this->db->dbprefix . "orders_bak O";
                $sql .= " LEFT JOIN " . $this->db->dbprefix . "listing_claims_bak LC ON LC.order_id = O.order_id";
                $sql .= " LEFT JOIN " . $this->db->dbprefix . "listings_bak L ON L.id = LC.listing_id";
                $result = $this->db->query($sql)->result();
            } else {
                $result = $this->db->select(implode(',', $common_fields))->get($table . '_bak')->result();
            }
            
            if($result) {
                // do not truncate in case of payment and sms gateways
                switch($table) {
                    case 'email_templates':
                    case 'payment_gateways':
                    case 'sms_gateways':
                        $this->db->update_batch($table, $result, 'id');
                        break;
                    case 'comments':
                        $this->db->truncate('comments_hierarchy');
                        $this->db->empty_table($table);
                        $this->db->insert_batch($table, (array) $result);
                        // Add comments hierarchy only if the existing comments_hierarchy is empty
                        if (!$this->db->table_exists('comments_hierarchy_bak') ) {
                            // $hierarchy = $this->db->count_all_results('comments_hierarchy_bak');
                            // if($hierarchy == 0) {
                            $this->load->model('comments/comments_model');
                            $comments = $this->comments_model->select('id')->find_all();
                            if($comments) {
                                foreach($comments as $comment) {
                                    // save comment hierarchy
                                    $this->comments_model->add($comment->id, $comment->id);
                                }
                            }
                            //}
                        }
                        break;
                    case 'orders':
                        $this->db->truncate('orders');
                        if(!empty($result)) {
                            if($this->old_version < 3.5) {                            
                                $order_data = array();
                                foreach($result as $orders) {                                
                                    $order = array();
                                    $item = array();
                                    foreach($orders as $key => $value) {
                                        switch($key) {
                                            case 'title':
                                            case 'description':
                                                $item[$key] = $value;
                                                break;
                                            default:
                                                $order[$key] = $value;
                                        }
                                    }                                    
                                    
                                    if($order['order_type'] == 1 && !empty($item['description'])) {
                                        $description = str_replace('<li class="list-group-item">', '<li class="package-list-item">', $item['description']);
                                        $item['description'] = '<ul class="package-list">' . $description . '</ul>';
                                    } else if($order['order_type'] == 2 && !empty($item['title'])) {
                                        $item['title'] = 'Claim: ' . $item['title'];
                                    }
                                    $order['item'] = json_encode($item);
                                }
                                if(!empty($order_data)) {
                                    $this->db->insert_batch($table, $order_data);
                                }                           
                            } else {                            
                                $this->db->insert_batch($table, (array) $result);
                            }
                        }
                        break;
                    case 'settings':
                        $this->db->update_batch($table, $result, 'name');
                        // GoClixy-f0118f1f4ed0.p12
                        $analytics = $this->db->select('value')->where('name', 'seo.analytics_key_file')->get($table)->row();
                        if(!empty($analytics->value) && $analytics->value == 'GoClixy-f0118f1f4ed0.p12') {
                            $this->db->where('name', 'seo.analytics_key_file')->update($table, array('value' => ''));
                        }
                        // Update site version
                        $this->db->where('name', 'site.version')->update($table, array('value' => $this->version));
                        break;
                    case 'links':
                    case 'link_locations':
                        if($table == 'link_locations') {
                            // create index first
                            $columns = array('link_id', 'location_id');
                            $this->check_index($table, $columns, true, true);
                        }
                        // copy new links
                        $this->copy_tables($table, $table . '_bak', true);
                        break;
                    case 'tags':
                    case 'users':
                        $this->db->truncate($table);
                        if(!empty($result)) {
                            foreach($result as $key => $data) {
                                $insert_query = $this->db->insert_string($table, $data);
                                $insert_query = str_replace('INSERT INTO','INSERT IGNORE INTO', $insert_query);
                                $this->db->query($insert_query);
                            }
                        }
                        break;
                    default:
                        $this->db->truncate($table);
                        if(!empty($result)) {
                            $this->db->insert_batch($table, (array) $result);
                        }
                }
            }
        }
    }
    
    /**
     * Restore images from the old website
     */
    private function copy_images() {
        $oldpath = $this->old_fcpath;
        $newpath = $this->new_fcpath;
        $this->copy_files('banners', $oldpath, $newpath);
        $this->copy_files('blogs', $oldpath, $newpath);
        $this->copy_files('categories', $oldpath, $newpath);
        $this->copy_files('classifieds', $oldpath, $newpath);
        $this->copy_files('comments', $oldpath, $newpath);
        $this->copy_files('deals', $oldpath, $newpath);
        $this->copy_files('events', $oldpath, $newpath);
        $this->copy_files('jobs', $oldpath, $newpath);
        $this->copy_files('listings', $oldpath, $newpath);
        $this->copy_files('payment-gateways', $oldpath, $newpath);
        $this->copy_files('products', $oldpath, $newpath);
        $this->copy_files('properties', $oldpath, $newpath);
        $this->copy_files('qrcodes', $oldpath, $newpath);
        $this->copy_files('sliders', $oldpath, $newpath);
        $this->copy_files('sms-gateways', $oldpath, $newpath);
        $this->copy_files('themes', $oldpath, $newpath);
        $this->copy_files('users', $oldpath, $newpath);
        $this->copy_files('variants', $oldpath, $newpath);
        $this->copy_files('videos', $oldpath, $newpath);
        
        // extra files to be copied
        $extra_files = array(
            'no-image.png' => $oldpath . 'no-image.png',
            'no-logo.png' => $oldpath . 'no-logo.png'
        );
        // copy favicon file
        if($this->old_version < 3.5) {
            $extra_files['favicon.ico'] = $this->old_faviconpath . '/' . 'favicon.ico';
        }
        foreach($extra_files as $filename => $old_file_path) {            
            if(file_exists($old_file_path)) { // copy files only if they exist                
                copy($old_file_path, $newpath . $filename);
            }
        }
    }
    
    /**
     * Generate Thumbnails
     * @param string $module
     */
    private function copy_files($module, $oldpath, $newpath) {
        $oldpath = $oldpath . $module . '/';
        if($module == 'logo') {
            $newpath .= 'listings/logos/';
        } else {
            $newpath .= ($module == 'photos' ? 'listings' : $module) . '/';
        }
        
        /**
         * Sub-folders contained within the directory will be mapped as well.
         * If you wish to control the recursion depth, you can do so using the second parameter (integer).
         * A depth of 1 will only map the top level directory
         * By default, hidden files will not be included in the returned array.
         * To override this behavior, you may set a third parameter to true (boolean):
         */
        //$files = directory_map('./assets/images/' . $module, 1);
        $files = directory_map('../../assets/images/' . $module);
        if(!$files) {
            return false;
        }
        asort($files);
        /*foreach($files as $file) {
            // get only image files based on their extensions
            $info = pathinfo($file);
            if(isset($info['extension'])) {
                switch(strtolower($info['extension'])) {
                    case "jpg":
                    case "jpeg":
                    case "png":
                    case "gif":
                        if(file_exists($oldpath . $info['basename'])){
                            copy($oldpath . $info['basename'], $newpath . $info['basename']);
                        }
                        break;
                }
            }
        }*/
        
        // copy files recursively - inside folders and subfolders
        $this->recursive_copy($files, $oldpath, $newpath);
        
        // Regenerate thumbnails for each module
        // $this->images_model->thumbnail_generate($module);
    }
    
    /**
     * Recursively copy files inside a subdirectory
     * @param array $files
     * @param string $oldpath
     * @param string $newpath
     */
    private function recursive_copy($files, $oldpath, $newpath) {
        if (!file_exists($newpath)) { // if newpath does not exist, create it
            mkdir($newpath, 0777, true);
        }
        foreach($files as $file_folder => $file) {
            // get only image files based on their extensions
            if(is_string($file)) {
                $info = pathinfo($file);
                if(isset($info['extension'])) {
                    switch(strtolower($info['extension'])) {
                        case "jpg":
                        case "jpeg":
                        case "png":
                        case "gif":
                            if(file_exists($oldpath . $info['basename'])) {
                                copy($oldpath . $info['basename'], $newpath . $info['basename']);
                            }
                            break;
                    }
                }
            } else {
                $this->recursive_copy($file, $oldpath . $file_folder, $newpath . $file_folder);
            }
        }
    }
    
    /**
     * Rename existing tables with _bak extension
     * Create new tables using the backup sql file
     */
    private function rename_existing_tables() {
        if (!$data = file_get_contents(APPPATH . "/db/goclixy.sql")) {
            $error = error_get_last();
            echo "HTTP request failed. Error was: " . $error['message'];
        } else {
            $this->load->dbforge();
            $tables = $this->db->list_tables();
            foreach ($tables as $table) {
                $table_without_prefix = str_replace($this->db->dbprefix, '', $table);
                // Delete any table with _bak at the end
                if(substr($table_without_prefix, -4) == '_bak') {
                    $this->dbforge->drop_table($table_without_prefix, TRUE);
                } //else {
                $table_bak = $table_without_prefix . '_bak';
                // Produces: DROP TABLE IF EXISTS table_name
                $this->dbforge->drop_table($table_bak, TRUE);
                $this->dbforge->rename_table($table_without_prefix, $table_bak);
                //}
            }
            // To fasten import operation
            // https://stackoverflow.com/questions/2167522/innodb-takes-over-an-hour-to-import-600mb-file-myisam-in-a-few-minutes/2167641#2167641
            $this->db->simple_query('SET autocommit=0;');
            // Name of the file
            $filename = APPPATH . "/db/goclixy.sql";
            // Temporary variable, used to store current query
            $templine = '';
            // Read in entire file
            $lines = file($filename);
            // Loop through each line
            foreach ($lines as $line)
            {
                // Skip it if it's a comment
                if (substr($line, 0, 2) == '--' || $line == '')
                    continue;
                    
                    // Add this line to the current segment
                    $templine .= $line;
                    // If it has a semicolon at the end, it's the end of the query
                    if (substr(trim($line), -1, 1) == ';') {
                        // Perform the query
                        //mysql_query($templine) or print('Error performing query \'<strong>' . $templine . '\': ' . mysql_error() . '<br /><br />');
                        $this->db->query($templine);
                        // Reset temp variable to empty
                        $templine = '';
                    }
            }
            $this->db->simple_query('COMMIT;');
        }
    }
    
    /**
     * Drop all tables with _bak at the end
     */
    private function drop_backup_tables() {
        $this->load->dbforge();
        $tables = $this->db->list_tables();
        foreach ($tables as $table) {
            $table_without_prefix = str_replace($this->db->dbprefix, '', $table);
            // Delete any table with _bak at the end
            if(substr($table_without_prefix, -4) == '_bak') {
                // Produces: DROP TABLE IF EXISTS table_name
                $this->dbforge->drop_table($table_without_prefix, TRUE);
            }
        }
    }
    
    /**
     * Copy Language Files
     */
    private function copy_lang_files() {
        // get all languages information from the existing website
        $languages = directory_map($this->old_apppath . 'language/', 1);
        if(!$languages) {
            return false;
        }
        foreach($languages as $language) {
            $lang = substr($language, 0, -1);
            if($lang != 'english') {
                // Application folder languages
                $app_files = directory_map($this->old_apppath . 'language/' . $lang, 1);
                if($app_files) {
                    $app_path = 'language/' . $lang;
                    if (!is_dir(APPPATH . $app_path)) {
                        mkdir(APPPATH . $app_path, 0777, TRUE);
                    }
                    foreach($app_files as $app_file) {
                        $app_filepath = $app_path . '/' . $app_file;
                        copy($this->old_apppath . $app_filepath, APPPATH . $app_filepath);
                    }
                }
                // Module languages
                $folders = directory_map($this->old_apppath . 'modules/', 1);
                if($folders) {
                    foreach($folders as $folder) {
                        $module = substr($folder, 0, -1);
                        $loadLanguage = directory_map($this->old_apppath . 'modules/' . $module . '/language/' . $lang . '/', 1);
                        if($loadLanguage) {
                            $path = 'modules/'. $module . '/language/' . $lang;
                            if (!is_dir(APPPATH . $path)) {
                                mkdir(APPPATH . $path, 0777, TRUE);
                            }
                            $filepath = $path . '/' . $module . '_lang.php';
                            if(file_exists($this->old_apppath . $filepath)) {
                                copy($this->old_apppath . $filepath, APPPATH . $filepath);
                            }
                        }
                    }
                }
            }
        }
    }
    
    /**
     * Handle application_lang.php file split into goclixy_lang.php file
     *
     * firstly copy the application_lang.php file from the existing website to the language folder
     *
     * @param string $orig_file
     * @param string $lang_file
     * @param string $overwrite_english
     */
    private function split_application_lang($orig_file, $lang_file, $do_not_overwrite_english = true) {
        // get all keys from the latest application_lang.php file
        // Do it first for goclixy_lang.php file else application_lang.php file will be overwritten
        // and only english text will be available
        $keys = load_lang_file($orig_file, 'english');
        
        // search each language file in the application folder
        $languages = list_languages();
        
        // do not overwrite english language
        if($do_not_overwrite_english) {
            // remove english from the languages array
            if (in_array('english', $languages)) {
                unset($languages[array_search('english',$languages)]);
            }
        }
        foreach($languages as $language) {
            // load each language file
            $lang_array = load_lang_file($lang_file, $language);
            if($lang_array) {
                $lang_keys[$language] = $keys;
                // search key in the language file
                foreach($keys as $key => $value) {
                    $search = array_key_exists ($key, $lang_array);
                    if($search) {
                        // key exists, use the old/existing value
                        $lang_keys[$language][$key] = $lang_array[$key];
                    }
                }
                // save language file
                save_lang_file($orig_file, $language, $lang_keys[$language]);
            }
        }
    }
    
    /**
     * Copy images from existing tables to images table
     * @param string $module
     * @param string $table - e.g. property_images
     * @param string $field_name - e.g. property_id
     */
    public function copy_existing_images($module, $table, $field_name) {
        $module_id = gc_module_id($module);
        switch($module) {
            case 'blogs':
            case 'deals':
            case 'events':
            case 'jobs':
            case 'logo':
                if($module == 'logo') {
                    $module = 'listings';
                    $module_id = gc_module_id($module);
                    $type_id = 2;
                } else {
                    $type_id = 0;
                }
                $result = $this->db->get($module)->result();
                $data = array();
                foreach($result as $row) {
                    // check existence in the images table
                    $count = $this->db->where(array('module_id' => $module_id, 'row_id' => $row->id))->count_all_results('images');
                    if($count == 0) {
                        $data[] = "(" . $module_id . "," . $row->id . "," . $type_id . ",'" . $row->$field_name . "','" . $row->title . "',1)";
                    }
                }
                if(!empty($data)) {
                    $sql = "INSERT INTO " . $this->db->dbprefix . "images (module_id, row_id, type_id, url, title, status) VALUES " . implode(',', $data);
                    $this->db->query($sql);
                }
                break;
            default:
                $sql = "INSERT INTO " . $this->db->dbprefix . "images (module_id, row_id, title, url, priority, status)";
                $sql .= " SELECT $module_id AS module_id, $field_name AS row_id, title, url, priority, status";
                $sql .= " FROM " . $this->db->dbprefix . $table;
                $this->db->query($sql);
        }
    }
    
    /**
     * Insert/Update - Module Relations Table
     * Take information from the categories table and based on the selected module values
     * update the module_relations table
     *
     * Insert relation from the categories table to the module relations table
     */
    public function module_categories_relation($module = 'categories') {
        $categories = $this->db->get($module . '_bak')->result();
        $batch_delete = array();
        $data = array();
        if($categories) {
            foreach($categories as $category) {
                if(!empty($category->listings)) {
                    $data[] = array(
                        'row_id' => $category->id,
                        'module_id' => gc_module_id($module),
                        'toggle_module_id' => gc_module_id('listings')
                    );
                }
                if(!empty($category->blogs)) {
                    $data[] = array(
                        'row_id' => $category->id,
                        'module_id' => gc_module_id($module),
                        'toggle_module_id' => gc_module_id('blogs')
                    );
                }
                if(!empty($category->classifieds)) {
                    $data[] = array(
                        'row_id' => $category->id,
                        'module_id' => gc_module_id($module),
                        'toggle_module_id' => gc_module_id('classifieds')
                    );
                }
                if(!empty($category->deals)) {
                    $data[] = array(
                        'row_id' => $category->id,
                        'module_id' => gc_module_id($module),
                        'toggle_module_id' => gc_module_id('deals')
                    );
                }
                if(!empty($category->events)) {
                    $data[] = array(
                        'row_id' => $category->id,
                        'module_id' => gc_module_id($module),
                        'toggle_module_id' => gc_module_id('events')
                    );
                }
                if(!empty($category->jobs)) {
                    $data[] = array(
                        'row_id' => $category->id,
                        'module_id' => gc_module_id($module),
                        'toggle_module_id' => gc_module_id('jobs')
                    );
                }
                if(!empty($category->products)) {
                    $data[] = array(
                        'row_id' => $category->id,
                        'module_id' => gc_module_id($module),
                        'toggle_module_id' => gc_module_id('products')
                    );
                }
                if(!empty($category->properties)) {
                    $data[] = array(
                        'row_id' => $category->id,
                        'module_id' => gc_module_id($module),
                        'toggle_module_id' => gc_module_id('properties')
                    );
                }
                if(!empty($category->videos)) {
                    $data[] = array(
                        'row_id' => $category->id,
                        'module_id' => gc_module_id($module),
                        'toggle_module_id' => gc_module_id('videos')
                    );
                }
                $batch_delete[] = "(" . $category->id . "," . gc_module_id($module) .")";
            }
        }
        $this->db->query("DELETE FROM " . $this->db->dbprefix . "module_relations WHERE (row_id,module_id) IN (" . implode(',', $batch_delete). ")");
        if(!empty($data)) {
            $this->db->insert_batch('module_relations', $data);
        }
    }
    
    /**
     * Get images from existing modules and save them in the images table
     * @param string $module
     * @param string $field - image/logo_url
     * @param string $images_table
     * @param string $field_id
     */
    public function module_images() {
        $config_modules = gc_modules();
        foreach($config_modules as $config_module) {
            switch($config_module) {
                case 'listings':
                    // Copy logo
                    // Copy images from the images table
                    $this->db_images_copy($config_module, 'logo_url');
                    $this->db_images_copy($config_module, null, 'images', 'listing_id');
                    break;
                case 'blogs':
                case 'deals':
                case 'events':
                case 'jobs':
                    // Copy Image
                    $this->db_images_copy($config_module, 'image');
                    break;
                case 'classifieds':
                    $this->db_images_copy($config_module, null, 'classified_images', 'classified_id');
                    break;
                case 'properties':
                    $this->db_images_copy($config_module, null, 'property_images', 'property_id');
                    break;
                case 'products':
                    $this->db_images_copy($config_module, null, 'product_images', 'product_id');
                    break;
            }
        }
    }
    
    /**
     * Copy images information from modules into the images table
     * @param string $module
     * @param string $field
     * @param string $images_table
     * @param string $field_id
     */
    private function db_images_copy($module, $field = 'logo_url', $images_table = null, $field_id = null) {
        $module_id = gc_module_id($module);
        $table = (!empty($images_table) ? $images_table : $module) . '_bak';
        $records = $this->db->get($table)->result();
        if($records) {
            $path = 'assets/images/' . $module;
            foreach($records as $record) {
                // Module has only one image
                if(empty($images_table) && !empty($field) && !empty($record->{$field})) {
                    $type_id = ($field == 'logo_url' ? 2 : 1); // Set as the Cover Image
                    $filename = $record->{$field};
                    $row_id = $record->id;
                    $status = 1;
                } else if(!empty($images_table)) {
                    $type_id = 0;
                    $row_id = $record->{$field_id};
                    $filename = $record->url;
                    $status = (isset($record->status) ? $record->status : (isset($record->active) ? $record->active : 1));
                }
                // Perform operation only if there is an image
                if(!empty($filename)) {
                    $data = array(
                        'module_id' => $module_id,
                        'row_id' => $row_id,
                        'type_id' => $type_id,
                        'title' => $record->title,
                        'url' => $path . '/' . $filename,
                        'path' => $path,
                        'filename' => $filename,
                        'status' => (isset($status) ? $status : 1)
                    );
                    // check image existence in the images table
                    // Duplicate avoidance while re running the update process
                    $existence = $this->db->get_where('images', array('module_id' => $module_id, 'row_id' => $row_id, 'type_id' => $type_id))->row();
                    if(!$existence) { // Insert table
                        $this->db->insert('images', $data);
                    }
                }
            }
        }
    }
    
    /**
     * Copy QRCodes after copying the listings information
     */
    private function qrcodes() {
        $qrcodes = array();
        $module = 'listings';
        $records = $this->db->get($module)->result();
        if($records) {
            foreach($records as $record) {
                if(!empty($record->qrfile)) {
                    $qrcodes[] = array(
                        'id' => $record->id,
                        'qrfile' => 'assets/images/qrcodes/' . $record->qrfile
                    );
                }
            }
            if(!empty($qrcodes)) {
                $this->db->update_batch($module, $qrcodes, 'id');
            }
        }
    }
    
    /**
     * Move Listings and other modules categories information in to the categories_relation table
     */
    public function module_categories() {
        // Listing Categories
        $sql = "INSERT INTO " . $this->db->dbprefix . "categories_relation (module_id, row_id, parent_id, sub_id, subsub_id)";
        $sql .= " SELECT 1 AS module_id, listing_id AS row_id, parent_id, sub_id, subsub_id FROM " . $this->db->dbprefix . "listing_categories_bak";
        $this->db->simple_query($sql);
        
        // Module Categories
        $msql = "INSERT INTO " . $this->db->dbprefix . "categories_relation (module_id, row_id, parent_id, sub_id, subsub_id)";
        $msql .= " SELECT module_id, row_id, parent_id, sub_id, subsub_id FROM " . $this->db->dbprefix . "module_categories_bak";
        $this->db->simple_query($msql);
    }
    
    /**
     * Get tags from the module table and insert into the tag_relations table
     */
    public function module_tags() {
        $this->load->model('tags/tags_model');
        // Listing Tags
        $sql = "INSERT INTO " . $this->db->dbprefix . "tag_relations (module_id, row_id, tag_id)";
        $sql .= " SELECT 1 AS module_id, listing_id AS row_id, tag_id FROM " . $this->db->dbprefix . "listing_tags_bak";
        $this->db->simple_query($sql);
        
        // Get all existing tags and insert relation in the module_relations table
        // Initially all tags belongs to listings
        $result = $this->db->get('tags_bak')->result();
        $toggle_module_id = gc_module_id('listings');
        $tags_module_id = gc_module_id('tags');
        if($result) {
            $data = array();
            foreach($result as $tag) {
                $data[] = array(
                    'module_id' => $tags_module_id,
                    'row_id' => $tag->id,
                    'toggle_module_id' => $toggle_module_id
                );
            }
            if(!empty($data)) {
                $this->db->on_duplicate('module_relations', $data);
            }
        }
        
        // Module Tags
        $config_modules = gc_modules_only();
        foreach($config_modules as $config_module) {
            $module_id = gc_module_id($config_module);
            $records = $this->db->get($config_module)->result();
            if($records) {
                foreach($records as $record) {
                    if(!empty($record->keywords)) {
                        $this->tags_model->save_module_tags($record->keywords, $record->id, $config_module);
                    }
                }
            }
        }
    }
    
    /**
     * GoClixy v4 Changes
     */
    public function v4_changes() {
        // Set packages fields in the field_relations table
        /*$fields = array(19,20,22,23,24,25,26,27,28,29,37,80);
        foreach($fields as $field) {
            $this->db->where(array('module_id' => 1, 'field_id' => $field))->update('field_relations', array('field_package' => 1));
        }*/
        
        // Pacakges table form fields in an array
        $packages = $this->db->select('id, form_fields')->get('packages')->result();
        if($packages) {
            foreach($packages as $package) {
                if(!empty($package->form_fields)) {
                    $form_fields = json_decode($package->form_fields, true);
                    if(isset($form_fields['show_address'])) { // deal with reupdate
                        $form_fields = json_encode(array('listings' => $form_fields));
                        $this->db->where('id', $package->id)->update('packages', array('form_fields' => $form_fields));
                    }
                }
            }
        }
        
        // Orders table items in an array
        $orders = $this->db->select('id, item')->get('orders')->result();
        if($orders) {
            $order_data = array();
            foreach($orders as $order) {
                if(!empty($order->item)) {
                    $item = json_decode($order->item, true);
                    if(!$this->isMultiArray($item)) { // deal with reupdate
                        $order_data[] = array('id' => $order->id, 'item' => json_encode(array($item)));
                    }
                }
            }
            if(!empty($order_data)) {
                $this->db->update_batch('orders', $order_data, 'id');
            }
        }
    }
    
    /**
     * Check whether an array is multidimensional or not
     * @param array $a
     * @return boolean
     */
    private function isMultiArray($a){
        foreach($a as $v) if(is_array($v)) return TRUE;
        return FALSE;
    }
    
    /**
     * Check index on a table
     * @param string $table - Name of the table
     * @param array $columns - Name of the columns in array
     * @param boolean $index_unique - TRUE - index must be unique
     * @param boolean $create_index - TRUE - create index if not exist
     * @return boolean
     */
    private function check_index($table, $columns, $index_unique = false, $create_index = false) {
        $index_exist = false;
        $indexes = $this->db->query('SHOW INDEX FROM ' . $this->db->dbprefix . $table)->result_array();
        if($indexes) {
            foreach($indexes as $index) {
                if(in_array($index['Column_name'], $columns)) {
                    if($index_unique && $index['Non_unique'] == 0) {
                        $index_exist = true;
                    }
                    /*$key = array_search($index['Column_name'], $columns);
                     if (false !== $key) {
                     unset($columns[$key]);
                     }*/
                }
            }
        }
        if($create_index && !$index_exist) {
            $index_name = $index_unique ? 'unique_' . $table : 'index_' . $table;
            // Unique index
            $unique_index = "ALTER TABLE " . $this->db->dbprefix . $table . " ADD ". ($index_unique ? 'UNIQUE' : '') ." INDEX `". $index_name . "` (" . implode(', ',$columns) . ")";
            $this->db->query ($unique_index);
            $index_exist = true;
        }
        return $index_exist;
    }
    
    /**
     * Copy Tables using MySQL command - Links and Fields
     * @param string $table_source - Source Table Name
     * @param string $table_bak - Backup/Old Table Name
     * @param boolean $insert_only - TRUE - Perform insert only do not update
     */
    private function copy_tables($table_source, $table_bak, $insert_only = false) {
        if($table_source != 'comments') {
            $this->db->truncate($table_source);
        }
        $existing_column_names = $this->db->list_fields($table_bak);
        $new_column_names = $this->db->list_fields($table_source);
        $columns = array_intersect($existing_column_names, $new_column_names);
        if(!empty($columns)) {
            $columns_comma_separated = implode(',', $columns);
            $sql = "INSERT";
            if($insert_only) {
                $sql .= " IGNORE";
            }
            $sql .= " INTO " . $this->db->dbprefix . $table_source . " (" . $columns_comma_separated . ")";
            $sql .= " SELECT " . $columns_comma_separated . " FROM " . $this->db->dbprefix . $table_bak;
            if(!$insert_only) {
                $duplicate_columns = array();
                foreach($columns as $column) {
                    $duplicate_columns[] = $column . '= VALUES(' . $column . ')';
                }
                $sql .= " ON DUPLICATE KEY UPDATE " . implode(',', $duplicate_columns);
            }
            $this->db->query($sql);
        }
    }
    
    /**
     * Update QRPath
     */
    public function update_qrpath() {
        $records = $this->db->get('listings')->result();
        $data= array();
        if($records) {
            foreach($records as $record) {
                if (strpos($record->qrfile, 'assets/images/qrcodes/assets/images/qrcodes/') !== false) {
                    $data[] = array(
                        'id' => $record->id,
                        'qrfile' => str_replace('assets/images/qrcodes/assets/images/qrcodes/', 'assets/images/qrcodes/', $record->qrfile),
                    );
                }
            }
            if(!empty($data)) {
                $this->db->update_batch('listings', $data, 'id');
            }
        }
    }
}