<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/**
 * Bonfire
 *
 * An open source project to allow developers get a jumpstart their development of CodeIgniter applications
 *
 * @package   Bonfire
 * @author    Bonfire Dev Team
 * @copyright Copyright (c) 2011 - 2013, Bonfire Dev Team
 * @license   http://guides.cibonfire.com/license.html
 * @link      http://cibonfire.com
 * @since     Version 1.0
 * @filesource
 */

// ------------------------------------------------------------------------

/**
 * Content context controller
 *
 * The controller which displays the homepage of the Content context in Bonfire site.
 *
 * @package    Bonfire
 * @subpackage Controllers
 * @category   Controllers
 * @author     Bonfire Dev Team
 * @link       http://guides.cibonfire.com/helpers/file_helpers.html
 *
 */
class Content extends Admin_Controller
{


	/**
	 * Controller constructor sets the Title and Permissions
	 *
	 */
	public function __construct()
	{
		parent::__construct();

		Template::set('toolbar_title', lang('bf_context_content'));

		$this->auth->restrict('Site.Content.View');
	}//end __construct()

	//--------------------------------------------------------------------

	/**
	 * Displays the initial page of the Content context
	 *
	 * @return void
	 */
	public function index() {
		$links = array(
			array(
				'name' => lang('gc_menu_categories'),
				'icon' => 'fas fa-list',
				'link' => '/content/categories',
				'color'=> 'primary'
			),
			array(
				'name' => lang('gc_menu_comments'),
				'icon' => 'fas fa-comments',
				'link' => '/content/comments',
				'color'=> 'warning'
			),
			array(
				'name' => lang('gc_menu_images'),
				'icon' => 'fas fa-image',
				'link' => '/content/images',
				'color'=> 'info'
			),
			array(
				'name' => lang('gc_menu_listings'),
				'icon' => 'fas fa-th-list',
				'link' => '/content/listings',
				'color'=> 'success'
			),
			array(
				'name' => lang('gc_menu_news'),
				'icon' => 'fas fa-newspaper',
				'link' => '/content/news',
				'color'=> 'danger'
			),
			array(
				'name' => lang('gc_menu_pages'),
				'icon' => 'fas fa-pen-square',
				'link' => '/content/pages',
				'color'=> 'info'
			),
			array(
				'name' => lang('gc_menu_ratings'),
				'icon' => 'fas fa-star',
				'link' => '/content/ratings',
				'color'=> 'success'
			),
			array(
				'name' => lang('gc_menu_tags'),
				'icon' => 'fas fa-tags',
				'link' => '/content/tags',
				'color'=> 'inverse'
			)			
		);
		$colors = array(
			1 => 'warning',
			2 => 'primary',
			3 => 'success',
			4 => 'info',
			5 => 'danger',
			6 => 'inverse',
			7 => 'warning'
		);
		$i = 1;
		$config_modules = gc_modules_only();
		foreach($config_modules as $config_module) {
			array_push($links, array(
				'name' => lang('gc_menu_' . $config_module),
				'icon' => 'fas fa-' . gc_module_icon($config_module),
				'link' => '/content/listings/view/' . $config_module,
			    'color'=> isset($colors[$i]) ? $colors[$i] : 'success'
			));
			++$i;
		}
		array_push($links, array(
			'name' => lang('gc_menu_videos'),
			'icon' => 'fas fa-video',
			'link' => '/content/listings/videos',
			'color'=> 'danger'
		));
		Template::set('links', $links);
		Template::set_view('context');
		Template::render();
	}//end index()

	//--------------------------------------------------------------------


}//end class