<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/**
 * Bonfire
 *
 * An open source project to allow developers get a jumpstart their development of CodeIgniter applications
 *
 * @package   Bonfire
 * @author    Bonfire Dev Team
 * @copyright Copyright (c) 2011 - 2013, Bonfire Dev Team
 * @license   http://guides.cibonfire.com/license.html
 * @link      http://cibonfire.com
 * @since     Version 1.0
 * @filesource
 */

//--------------------------------------------------------------------

/**
 * Form Helpers
 *
 * Creates HTML5 extensions for the standard CodeIgniter form helper.
 *
 * These functions also wrap the form elements as necessary to create
 * the styling that the Bootstrap-inspired admin theme requires to
 * make it as simple as possible for a developer to maintain styling
 * with the core. Also makes changing the core a snap.
 *
 * All methods (including overridden versions of the originals) now
 * support passing a final 'label' attribute that will create the
 * label along with the field.
 *
 * @package    Bonfire
 * @subpackage Helpers
 * @category   Helpers
 * @author     Bonfire Dev Team
 * @link       http://guides.cibonfire.com/helpers/array_helpers.html
 *
 */

if ( ! function_exists('_form_common'))
{
	/**
	 * Used by many of the new functions to wrap the input in the correct
	 * tags so that the styling is automatic.
	 *
	 * @access private
	 *
	 * @param string $type    A string with the name of the element type.
	 * @param string $data    Either a string with the element name, or an array of key/value pairs of all attributes.
	 * @param string $value   Either a string with the value, or blank if an array is passed to the $data param.
	 * @param string $label   A string with the label of the element.
	 * @param string $extra   A string with any additional items to include, like Javascript.
	 * @param string $tooltip A string for inline help or a tooltip icon
	 *
	 * @return string A string with the formatted input element, label tag and wrapping divs.
	 */
	function _form_common($type='text', $data='', $value='', $label='', $extra='', $tooltip = '')
	{
		$defaults = array('type' => $type, 'name' => (( ! is_array($data)) ? $data : ''), 'value' => $value);

		// If name is empty at this point, try to grab it from the $data array
		if (empty($defaults['name']) && is_array($data) && isset($data['name']))
		{
			$defaults['name'] = $data['name'];
			unset($data['name']);
		}

		// If label is empty at this point, try to grab it from the $data array
		if (empty($label) && is_array($data) && isset($data['label']))
		{
			$label = $data['label'];
			unset($data['label']);
		}

		// If tooltip is empty at this point, try to grab it from the $data array
		if (empty($tooltip) && is_array($data) && isset($data['tooltip']))
		{
			$tooltip = $data['tooltip'];
			unset($data['tooltip']);
		}

		$error = '';

		if (function_exists('form_error'))
		{
			if (form_error($defaults['name']))
			{
				$error   = ' error';
				$tooltip = '<span class="help-inline">' . form_error($defaults['name']) . '</span>' . PHP_EOL;
			}
		}

		$output = _parse_form_attributes($data, $defaults);

		$output = <<<EOL

<div class="form-group row {$error}">
	<label class="col-sm-3 col-form-label" for="{$defaults['name']}">{$label}</label>
	<div class="col-sm-6">
		 <input class="form-control" {$output} {$extra} />
		{$tooltip}
	</div>
</div>

EOL;

		return $output;

	}//end _form_common()
}

if ( ! function_exists('gc_form_common'))
{
	/**
	 * Used by many of the new functions to wrap the input in the correct
	 * tags so that the styling is automatic.
	 *
	 * @access private
	 *
	 * @param string $type    A string with the name of the element type.
	 * @param string $data    Either a string with the element name, or an array of key/value pairs of all attributes.
	 * @param string $value   Either a string with the value, or blank if an array is passed to the $data param.
	 * @param string $label   A string with the label of the element.
	 * @param string $extra   A string with any additional items to include, like Javascript.
	 *
	 * @return string A string with the formatted input element, label tag and wrapping divs.
	 */
	function gc_form_common($type='text', $data='', $value='', $label='', $extra='')
	{
		// If name is empty at this point, try to grab it from the $data array
		if (empty($defaults['name']) && is_array($data) && isset($data['name']))
		{
			$defaults['name'] = $data['name'];
			$field_name = $data['name'];
			unset($data['name']);
		} else {
			$field_name = $data;
		}
		
		// If label is empty at this point, try to grab it from the $data array
		if (empty($label) && is_array($data) && isset($data['label']))
		{
			$label = $data['label'];
			unset($data['label']);
		}
		
		// process extra parameters
		if(!empty($extra) && is_array($extra)) {
			$field_extra = array();
			foreach($extra as $key => $extra_value) {
				$field_extra[] = $key . '=' . '"' . $extra_value . '"';
			}
			$extra = implode(' ' , $field_extra);
		}
		
		$defaults = array('type' => $type, 'name' => (( ! is_array($data)) ? $data : ''), 'value' => set_value($field_name, $value));
		$output = _parse_form_attributes($data, $defaults);

		$output = <<<EOL
		 <input {$output} {$extra} />
EOL;

		return $output;

	}//end _form_common()
}

if ( ! function_exists('gc_form_element'))
{
    /**
     * Used by many of the new functions to wrap the input in the correct
     * tags so that the styling is automatic.
     *
     * @access private
     *
     * @param string $type    		A string with the name of the element type.
     * @param string $data    		Either a string with the element name, or an array of key/value pairs of all attributes.
     * @param string $value   		Either a string with the value, or blank if an array is passed to the $data param.
     * @param string $label   		A string with the label of the element.
     * @param string $size   		A string with the size of the element, another class can also be added
     *
     * A field can show a help in three blocks
     * help_tooltip - Display help in a tooltip with label
     * help_block - Display help below the field
     * help_inline - Display help with the field
     *
     * @return string A string with the formatted input element, label tag and wrapping divs.
     */
    function gc_form_element($type='text', $data='', $value='', $label='', $size='')
    {
        switch($type) {
            case 'actions':
                return form_actions($data);
                break;
            case 'form_city':
                return form_city($data);
                break;
            case 'selectize':
                // fixed classes, no need to pass them as a parameter each time
                $data['field_class'] = 'selectized';
                $data['size'] = 'selectize-input-control ' . (!empty($size) ? $size : 'col-sm-6');
                break;
            case 'filemanager':
                $filemanager_field_name = isset($data['name']) ? $data['name'] : 'image_url';
                $filemanager_field_id = isset($data['field_id']) ? $data['field_id'] : $filemanager_field_name;
                $fldr = isset($data['fldr']) ? $data['fldr'] : '';
                $filemanager_url = site_url() . 'assets/filemanager/dialog.php?type=2&field_id=' . $filemanager_field_id . '&fldr=' . $fldr . '&relative_url=1&lang=' . get_language_code(config_item('language')) . '&akey=' . md5(settings_item('site.filemanager_akey'));
                $data['element_container'] = 'div.input-group';
                $data['options'] = array(
                    array('type' => 'text', 'name' => $filemanager_field_name, 'value' => $value, 'extra' => array('id' => $filemanager_field_id, 'class' => 'form-control', 'placeholder' => lang('placeholder_filemanager'))),
                    array('type' => 'link', 'url' => $filemanager_url, 'title' => lang('bf_action_browse'), 'value' => '', 'field_container' => 'span.input-group-btn', 'extra' => array('class' => 'btn btn-secondary iframe-btn'))
                );
                break;
        }
        
        // Below given parameters to be always passed in the $data array
        // As they are not used always
        $field_name = '';
        $extra = array(); 		// array $extra   			A string with any additional items to include, like Javascript.
        $help = '';				// string $help				A string for toggle_tooltip help with label
        $tooltip = ''; 			// string $tooltip 			A string to display help text with the field
        $help_class = '';		// string $help_class		A string to display help inline with the field, by default help is shown as a block below the field
        $control_class = '';	// string $control_class 	Additional classes to be added to form-group to perform js operations
        $control_style = '';	// string $control_style	Additional inline styles to show or a hide field
        $field_class = '';		// string $field_class		Additional classes to be added to the input element to perform js operations
        $options = array();		// string $options			Multiple fields in the same form-group
        $placeholder = '';		// string $placeholder   	A string with the placeholder of the element.
        $default_value = '';	// string $default_value   	A string with the default value of the element.
        $field_only = FALSE;	// string $field_only		A boolean to return field only from the _prepare_element
        $element_container = '';// string $element_container A string to wrap fields with a container e.g. form-inline div
        $no_label = FALSE;		// boolean $no_label		Do not show a label if it is disabled, introduced mainly for labels/checkboxes
        $form_elements = array(); // array $form_elements	Pass elements externally and return with wrapper
        $label_width = 'col-sm-3';// string $label_width		Width of the Label, default is col-sm-3
        
        // If name is empty at this point, try to grab it from the $data array
        if (empty($defaults['name']) && is_array($data) && isset($data['name']))
        {
            $field_name = $data['name'];
            unset($data['name']);
        } else if(is_string($data)) {
            $field_name = $data;
        }
        
        // If label is empty at this point, try to grab it from the $data array
        if (empty($label) && is_array($data) && isset($data['label']))
        {
            $label = $data['label'];
            unset($data['label']);
        }
        
        // If placeholder is empty at this point, try to grab it from the $data array
        if (is_array($data) && isset($data['placeholder']))
        {
            $placeholder = $data['placeholder'];
            unset($data['placeholder']);
        }
        
        // If default_value is empty at this point, try to grab it from the $data array
        if (is_array($data) && isset($data['default_value']))
        {
            $default_value = $data['default_value'];
            unset($data['default_value']);
        }
        
        // If size is empty at this point, try to grab it from the $data array
        if (empty($size) && is_array($data) && isset($data['size']))
        {
            $size = $data['size'];
            unset($data['size']);
        }
        
        // Try to grab the width of the label from the $data array
        if (is_array($data) && isset($data['label_width']))
        {
            $label_width = $data['label_width'];
            unset($data['label_width']);
        }
        
        // try to grab the extra from the $data array
        if (is_array($data) && isset($data['extra']))
        {
            $extra = $data['extra'];
            unset($data['extra']);
        }
        
        // try to grab the tooltip from the $data array
        if (is_array($data) && isset($data['tooltip'])) {
            if(!empty($data['help_class'])) {
                $help_class = $data['help_class'];
                unset($data['help_class']);
            } else {
                $help_class = 'help-block';
            }
            if($help_class == 'help-tooltip') {
                $data['help'] = $data['tooltip'];
                unset($data['tooltip']);
            } else {
                $tooltip = '<span class="' . $help_class . '"><small>' . $data['tooltip'] . '</small></span>';
                unset($data['tooltip']);
            }
        }
        
        // try to grab the toggle_tooltip from the $data array
        if (is_array($data) && isset($data['help'])) {
            $help = $data['help'];
            unset($data['help']);
        }
        
        // try to grab the control class from the $data array
        if (is_array($data) && !empty($data['control_class'])) {
            $control_class = $data['control_class'];
            unset($data['control_class']);
        }
        
        // try to grab the control class from the $data array
        if (is_array($data) && !empty($data['control_style'])) {
            $control_style = ' style="' . $data['control_style'] . '"';
            unset($data['control_style']);
        }
        
        // try to grab the field_class from the $data array
        if (is_array($data) && isset($data['field_class'])) {
            $field_class = $data['field_class'];
            unset($data['field_class']);
        }
        
        // try to grab the options from the $data array
        if (is_array($data) && isset($data['options'])) {
            $options = $data['options'];
            unset($data['options']);
        }
        
        // try to grab the field_only from the $data array
        if (is_array($data) && isset($data['field_only'])) {
            $field_only = $data['field_only'];
            unset($data['field_only']);
        }
        
        // try to grab the field_only from the $data array
        if (is_array($data) && isset($data['no_label'])) {
            $no_label = $data['no_label'];
            unset($data['no_label']);
        }
        
        // try to grab the field_only from the $data array
        if (is_array($data) && isset($data['form_elements'])) {
            $form_elements = $data['form_elements'];
            unset($data['form_elements']);
        }
        
        $error = '';
        
        if (function_exists('form_error'))
        {
            if (form_error($field_name))
            {
                $field_error = (is_array($data) && isset($data['name_prefix']) ? $data['name_prefix'] . $field_name : $field_name);
                $error   = ' error';
                $tooltip = '<span class="badge badge-danger">' . strip_tags(form_error($field_error)) . '</span>' . PHP_EOL;
            }
        }
        
        // add additional classes to form-group to perform js operations
        $control_class = (!empty($control_class) ? 'form-group form-row ' . $control_class : 'form-group form-row');
        
        // set field size and other class
        $size = (!empty($size) ? $size : "col-sm-6");
        
        // set field class
        if($type == 'dropdown') {
            $field_class = $field_class == 'selectpicker' ? '' : (!empty($field_class) ? 'custom-select ' . $field_class : 'custom-select');
            // $field_class = (!empty($field_class) ? 'custom-select ' . $field_class : 'custom-select');
        } else {
            $field_class = (!empty($field_class) ? 'form-control ' . $field_class : 'form-control');
        }
        
        // toggle help tooltip
        $toggle_tooltip = '';
        if(!empty($help)) {
            $toggle_tooltip = '<a role="button" data-toggle="tooltip" data-placement="bottom" data-original-title="' . $help . '" class="help-label"><i class="fas fa-question"></i></a>';
        }
        
        // Display number of counts with field label - PROFILE FILTER FIELDS
        if(isset($data['template']) && $data['template'] == 'filter') { // template variable is used to change the layout
            $CI =& get_instance();
            $data['filter_counts'] = $CI->session->userdata('filter_counts');
        }
        
        if(empty($form_elements)) {
            if($type != 'dropdown' && $type != 'multiselect' && !empty($options)) {
                $element_type = ($type == 'dropdown-group' ? 'dropdown' : $type);
                // for multiple fields
                $data['value'] = $value;
                $data['default_value'] = $default_value;
                $data['extra'] = $extra;
                $data['class'] = $field_class; // For Input Text Boxes
                $form_elements = array();
                foreach($options as $option) {
                    $form_elements[] = _prepare_element($element_type, $option, $data);
                }
                $form_elements = implode("\n", $form_elements);
            } else {
                $form_data = array(
                    'name' => $field_name,
                    'value' => $value,
                    'label' => $label,
                    'default_value' => $default_value,
                    'options' => $options
                );
                if(is_array($data)) {
                    $form_data = array_merge($form_data, $data);
                }
                switch($type) {
                    case 'text':
                    case 'textarea':
                        if(!empty($placeholder)) {
                            $extra['placeholder'] = $placeholder;
                        }
                        $extra['class'] = $field_class;
                        break;
                    case 'checkbox':
                    case 'radio':
                        break;
                    default:
                        $extra['class'] = $field_class;
                        break;
                }
                $form_data['extra'] = $extra;
                $form_elements = _prepare_element($type, $form_data);
            }
        } else {
            $form_elements = implode("\n", $form_elements);
        }
        
        // try to grab the element_container from the $data array
        if (is_array($data) && isset($data['element_container'])) {
            $element_container = $data['element_container'];
            unset($data['element_container']);
            $form_elements = emmet($element_container, $form_elements);
        }
        if(!empty($tooltip)) {
            $form_elements = $form_elements . PHP_EOL . $tooltip;
        }
        // do not display a label if it disabled for a form element
        if($no_label) {
            $label = '';
            $toggle_tooltip = '';
        }
        
        if($field_only) {
            return $form_elements;
        } else {
            $output = '';
            if(isset($data['template'])) { // template variable is used to change the layout
                switch($data['template']) {
                    case 'filter':
                    case 'filter_session':
                        $counts = '';
                        if(!empty($data['filter_counts']) && isset($data['filter_counts']->{$field_name})) {
                            $counts = ' (' . $data['filter_counts']->{$field_name} . ')';
                        }
                        $accordion_id = $field_name;
                        $accordion_data = $form_elements;
                        if(empty($field_name)) {
                            $accordion_id = $type . rand(1, 999);
                            if($type == 'form_elements') {
                                $accordion_data = <<<EOL
                    <div class="row">
                        {$form_elements}
                    </div>
EOL;
                            }
                        }
                        $icon = isset($data['icon']) ? $data['icon'] : 'fas fa-chevron-right';
                        $btn_text = lang('gc_action_submit');
                        if($data['template'] == 'filter') {
                            $output = <<<EOL
        <div class="panel-group" id="accordion{$field_name}">
            <div class="card card-info card-filter border-0 mb-0">
                <div class="card-header parent-heading-1 bg-transparent">
                    <h5 class="card-title m-0 h6">
        				<i class="{$icon} fa-fw mr-1"></i>
        				<a href="#" title="{$label}">{$label}{$counts}</a>
        				<span class="float-right">
        					<a data-toggle="collapse" data-parent="#accordion{$accordion_id}" href="#collapse-{$accordion_id}" aria-expanded="true">
        						<i class="fas fa-plus"></i>
        					</a>
        				</span>
        			</h5>
                </div>
                <div id="collapse-{$accordion_id}" class="panel-collapse collapse m-3" style="">
                        {$accordion_data}
                    <button class="btn btn-warning btn-theme-secondary btn-xs" type="submit" name="submit">{$btn_text}</button>
                </div>
            </div>
        </div>
EOL;
                        } else {
                            $CI =& get_instance();
                            $fields_collapse_data = (array) $CI->session->userdata('fields_collapse_data');
                            $fields_collapse_data[$field_name] = array(
                                'id' => $accordion_id,
                                'name' => $field_name,
                                'label' => $label,
                                'icon' => $icon,
                                'counts' => $counts,
                                'data' => $accordion_data
                            );
                            $CI->session->set_userdata('fields_collapse_data', $fields_collapse_data);
                        }
                        break;
                    case 'check-inline':
                        $output = $form_elements;
                        break;
                    case 'form-group':
                        if(!empty($data['emmet_container'])) {
                            $form_elements = emmet($data['emmet_container'], $form_elements);
                        }
                        $output = <<<EOL
<div class="form-group">
    <label for="{$field_name}">{$label} {$toggle_tooltip}</label>
    {$form_elements}
</div>
EOL;
    break;
                }
            } else {
                $output = <<<EOL
                
<div class="{$control_class}{$error}"{$control_style}>
	<label class="{$label_width} col-form-label" for="{$field_name}">{$label} {$toggle_tooltip}</label>
	<div class="{$size}">
		{$form_elements}
	</div>
</div>

EOL;
            }
            return $output;
        }
        
    }//end _form_common()
}

if ( ! function_exists('_prepare_element')) {
    /**
     * Prepare input field data
     * @param string $type    		A string with the name of the element type.
     * @param string $data    		Either a string with the element name, or an array of key/value pairs of all attributes.
     */
    function _prepare_element($type='text', $data = array(), $common_data = array()) {
        // Allow to process multiple type inputs e.g. input-group with textbox and button
        $type = is_array($data) && isset($data['type']) ? $data['type'] : $type;
        $field_name = is_array($data) && isset($data['name']) ? $data['name'] : $data;
        $value = is_array($data) && isset($data['value']) ? $data['value'] : (isset($common_data['value']) ? $common_data['value'] : '');
        $field_label = is_array($data) && isset($data['label']) ? $data['label'] : (isset($common_data['label_prefix']) ? lang($common_data['label_prefix'] . $field_name) : '');
        $field_extra = is_array($data) && isset($data['extra']) ? $data['extra'] : (isset($common_data['extra']) ? $common_data['extra'] : array());
        $default_value = is_array($data) && isset($data['default_value']) ? $data['default_value'] : (isset($common_data['default_value']) ? $common_data['default_value'] : '');
        
        // Multi Language - Settings Module Fields
        // @param string value_prefix e.g. site.
        if(isset($data['value_prefix'])) {
            $value_prefix = $data['value_prefix'];
            $value_field = $value_prefix . $field_name;
        }
        
        // A boolean to find whether the field is a translate field
        $field_translate = false;
        if(settings_item('site.multi_language') == 1) {
            $field_multi_language = is_array($data) && !empty($data['lang']) ? true : (!empty($common_data['lang']) ? true : false);
        }
        if(isset($field_multi_language) && $field_multi_language) {
            $field_translate = true;
        }
        
        // to deal with array fields, POST value is passed as a json parameter
        if(isset($data['field_json']) && isset($data['json_field'])) {
            $field_json = $data['field_json'];
            $json_field = $data['json_field'];
            // use of TRIM in validation rules will result in JSON data
            $value = isset($_POST[$field_json]) ? (is_array($_POST[$field_json]) ? $_POST[$field_json] : json_decode($_POST[$field_json])) : $value;
            $value = isset($value->$json_field) ? $value->$json_field : '';
        }
        
        // prepare field value
        if(is_object($value) && isset($value->$field_name)) {
            $field_value = $value->$field_name;
        } else if(is_array($value) && isset($value[$field_name])) { // value is an array
            $field_value = $value[$field_name];
        } else if(is_array($value) && isset($value_field) && isset($value[$value_field])) { // value is an array
            $field_value = $value[$value_field];
        } else if (!is_array($value) && !is_object($value) && isset($value)) { // value is a string
            $field_value = $value;
        } else {
            $field_value = $default_value;
        }
        
        // A field name can also have a prefix e.g. package_address
        // package_ is the prefix and address is the field name
        // the value above must look for the field name only
        $field_name = isset($common_data['name_prefix']) ? $common_data['name_prefix'] . $field_name : (isset($data['name_prefix']) ? $data['name_prefix'] . $field_name : $field_name);
        
        // A field name can also be an array e.g. modules[listings]
        // Pass the is_array parameter with the name prefix to enclose the field in an array
        /*if(isset($common_data['is_array'])) {
            $field_name = $common_data['is_array'] . '[' . $field_name . ']';
        }*/        
        
        // A field name can also be an array e.g. modules[listings]
        // Pass the is_array parameter with the name prefix to enclose the field in an array
        if(!empty($common_data['is_array'])) {
            $field_id = $common_data['is_array'] . '_' . $field_name;
            $field_name = $common_data['is_array'] . '[' . $field_name . ']';
            if(empty($field_extra['id'])) {
                $field_extra['id'] = $field_id;
            }
        } else if(!empty($data['is_array'])) {
            $field_id = $data['is_array'] . '_' . $field_name;
            $field_name = $data['is_array'] . '[' . $field_name . ']';
            if(empty($field_extra['id'])) {
                $field_extra['id'] = $field_id;
            }
        }
        
        switch($type) {
            case 'button':
                $field_extra['class'] = str_replace('form-control ', '', $field_extra['class']);
                $field = form_button($field_name, $field_label, $field_extra);
                break;
            case 'radio':
                /*if(!isset($field_extra['id'])) { // do not set id, it will create issue with multiple controls on the same form
					$field_extra['id'] = $field_name;
				}*/
                // unable to set the radio button checked after post if the radio default value is 0
                // Fields Module CORE/CUSTOM Radio
                if($default_value == 0 && is_bool($field_value) && $field_value) {
                    $field_extra['checked'] = TRUE;
                }
                $label_extra = isset($data['label_extra']) ? $data['label_extra'] : array();
                $field = form_label(form_radio(array('name' => $field_name, 'checked' => (is_bool($field_value) && $field_value ? TRUE : FALSE)), $default_value, set_radio($field_name, $field_value, (is_bool($field_value) && $field_value ? TRUE : FALSE)), $field_extra) . $field_label, $field_name, $label_extra);
                break;
            case 'checkbox':
                if(!isset($field_extra['id'])) {
                    $field_extra['id'] = $field_name;
                }
                $field_value = $field_value == 1 ? TRUE : FALSE;
                // for li checkboxes
                $icon = '';
                if(is_array($data) && !empty($data['icon'])) {
                    $icon = '<img src="' . $data['icon'] . '" /> ';
                }
                $field = form_label(form_checkbox($field_name, 1, set_checkbox($field_name, 1, (is_bool($field_value) && $field_value ? TRUE : FALSE)), $field_extra) . $icon . $field_label, $field_name);
                break;
            case 'text':
                $field_extra['id'] = (isset($field_extra['id']) ? $field_extra['id'] : $field_name);
                // if multiple fields are wrapped in a container e.g. form-inline
                // also add placeholder to the textbox with the help of a prefix
                if(isset($common_data['placeholder_prefix'])) {
                    $field_extra['placeholder'] = lang($common_data['placeholder_prefix'] . $field_name);
                }
                if(!isset($field_extra['class'])) {
                    $field_extra['class'] = $common_data['class'];
                }
                // Field Translate
                if($field_translate) {
                    $field = field_translate($type, $field_name, $field_extra, $data);
                } else {
                    $field = form_input($field_name, set_value($field_name, $field_value, FALSE), $field_extra);
                }
                break;
            case 'file':
                $field_id = $field_name; // Append ID with the field name to allow user to have different upload fields in the same form
                $field = '<input id="' . $field_id . '" type="file" name="' . $field_name . '" class="form-control" />';
                if((!empty($_POST[$field_id . '_uploaded_file']) || !empty($field_value))) { // && !isset($field_extra['member_area'])
                    $field_value = !empty($_POST[$field_id . '_uploaded_file']) ? $_POST[$field_id . '_uploaded_file'] : $field_value;
                    $field .= '		<input type="hidden" name="' . $field_name . '_uploaded_file" id="' . $field_id . '_uploaded_file" value="' . $field_value . '" />' . PHP_EOL;
                } else {
                    $field .= '		<input type="hidden" name="' . $field_name . '_uploaded_file" id="' . $field_id . '_uploaded_file" />' . PHP_EOL;
                }
                if(!empty($field_value)) { // Display file to the user
                    if(!empty($field_extra['file_path'])) {
                        $file_src = $field_extra['file_path'] . '/' . $field_value;
                    } else {
                        $file_src = site_url('assets/files/' . $field_value);
                    }
                    $field .= '<a href="' . $file_src . '" title="' . $field_value . '" id="' . $field_name . '_link">' . $field_value . '</a>';
                }
                break;                
            case 'fileuploader':
                $uploader_counter = isset($field_extra['counter']) ? $field_extra['counter'] : 1; // Number of fields to be added to the Form
                $show_counter = isset($field_extra['show_counter']) ? $field_extra['show_counter'] : false; // Show numbering with each field
                $uploader_text = isset($field_extra['uploader_text']) ? $field_extra['uploader_text'] : ''; // Display Text with the Uploader Field
                $is_multi = isset($field_extra['is_multi']) ? true : false; // Field for multi files upload
                // Language Strings
                $file_click_to_add = lang('file_click_to_add');
                $file_browse = lang('file_browse');
                $file_uploaded = lang('file_uploaded');
                $file_select_single = lang('file_select_single');
                $file_status = lang('label_status');
                $file_waiting = lang('file_waiting');
                // Uploaded Files
                $CI =& get_instance();
                $uploaded_files = $CI->session->flashdata('uploaded_files');
                
                // a form can have multiple fileuploader fields, restore their values to null on all files deletion
                $fileuploader_fields = (array) $CI->session->userdata('fileuploader_fields');
                $fileuploader_fields[] = $field_name;
                $CI->session->set_userdata('fileuploader_fields', $fileuploader_fields);
                
                $field = '';
                $file_template = '';
                $field_class = $is_multi ? 'drop-zone' : 'drop-area'; // Single and Multi both are maintained separately with the help of different class names
                if($is_multi) {
                    $file_template = <<<EOL
	            <script type="text/html" id="files-template">
	                 <li><span class="file-name"><strong>%%filename%%</strong></span> <span class="file-action-text text-muted">{$file_waiting}</span></li>
	            </script>
EOL;
                }
                for($i = 1; $i <= $uploader_counter; $i++) {
                    if(empty($uploader_text)) {
                        $uploader_label = $show_counter ? '<span>' . $i . '</span>' : '';
                    } else {
                        $uploader_label = $uploader_text;
                    }
                    $uploader_html = '';
                    $field_id = $field_name . '-' . $i;
                    if($uploader_label) {
                        $uploader_html = '<span class="mr-2 h5 d-inline">' . $uploader_label . '</span>';
                    }
                    $uploaded_file = isset($uploaded_files[$field_name . '-' . $i]) ? $uploaded_files[$field_name . '-' . $i] : false;
                    if($is_multi) {
                        $template_heading = '';
                        $uploaded_file_info = '';
                        if(!empty($uploaded_file) && is_array($uploaded_file)) {
                            foreach($uploaded_file as $file_uploaded_info) {
                                $uploaded_file_info .= '<li>' . $file_uploaded_info . '</li>' . PHP_EOL;
                            }
                        } /*else {
                        $uploaded_file_info = '<li class="text-muted text-center empty">' . lang('file_not_uploaded') . '</li>';
                        }*/
                        $lang_title = lang('label_title');
                        $lang_priority = lang('label_priority');
                        $lang_file = lang('label_file');
                        $template_heading = <<<EOL
        <div class="form-row mt-2">
            <div class="col-5 small font-weight-bold">{$lang_title}</div>
            <div class="col-1 small font-weight-bold">{$lang_priority}</div>
            <div class="col-2 small font-weight-bold">{$lang_file}</div>
        </div>
EOL;
                        $file_info = <<<EOL
        <small class="status text-muted">{$file_select_single}</small>
        {$template_heading}
        <ul class="list-unstyled mt-1 file-list">
            {$uploaded_file_info}
        </ul>
EOL;
            
                    } else {
                        // Display File Name
                        if($uploaded_file[0]) {
                            $file_info = <<<EOL
                    <small class="status text-success">{$file_uploaded}</small>
              		<small class="file-url">{$uploaded_file[0]}</small>
EOL;
                        } else {
                            $file_info = <<<EOL
                    <small class="status text-muted">{$file_select_single}</small>
                    <small class="file-url"></small>
EOL;
                        }
                    }
                    $field_disabled = !$is_multi && $uploaded_file ? ' disabled' : '';
                    $field .= <<<EOL
                <div class="dm-uploader {$field_class}" data-field="{$field_id}">
                      <div class="form-row mb-2">
                          <div class="col"> {$uploader_html}
                            <div class="progress mb-2" style="display: none;">
                            	<div class="progress-bar progress-bar-striped progress-bar-animated bg-primary" role="progressbar" style="width: 0%;" aria-valuenow="0" aria-valuemin="0" aria-valuemax="0">0%</div>
                            </div>
                            <div role="button" class="btn btn-primary mr-2">
                              	<i class="far fa-folder fa-fw"></i> {$file_browse}
                              	<input type="file" title="{$file_click_to_add}"{$field_disabled}>
                            </div>
                            {$file_info}
                          </div>
                  	</div>
                </div>
EOL;
                }
                $field .= $file_template;
                break;
            case 'fileuploader_old':
                $uploader_counter = isset($field_extra['counter']) ? $field_extra['counter'] : 1; // Number of fields to be added to the Form
                $show_counter = isset($field_extra['show_counter']) ? $field_extra['show_counter'] : false; // Show numbering with each field
                $uploader_text = isset($field_extra['uploader_text']) ? $field_extra['uploader_text'] : ''; // Display Text with the Uploader Field
                $is_multi = isset($field_extra['is_multi']) ? true : false; // Field for multi files upload
                // Language Strings
                $file_click_to_add = lang('file_click_to_add');
                $file_browse = lang('file_browse');
                $file_uploaded = lang('file_uploaded');
                $file_select_single = lang('file_select_single');
                $file_status = lang('label_status');
                $file_waiting = lang('file_waiting');
                // Uploaded Files
                $CI =& get_instance();
                $uploaded_files = $CI->session->flashdata('uploaded_files');
                $field = '';
                $file_template = '';
                $field_class = $is_multi ? 'drop-zone' : 'drop-area'; // Single and Multi both are maintained separately with the help of different class names
                if($is_multi) {
                    $file_template = <<<EOL
	            <script type="text/html" id="files-template">
	                 <li><span class="file-name"><strong>%%filename%%</strong></span> <span class="file-action-text text-muted">{$file_waiting}</span></li>
	            </script>
EOL;
                }
                for($i = 1; $i <= $uploader_counter; $i++) {
                    if(empty($uploader_text)) {
                        $uploader_label = $show_counter ? '<span>' . $i . '</span>' : '';
                    } else {
                        $uploader_label = $uploader_text;
                    }
                    $uploader_html = '';
                    $field_id = $field_name . '-' . $i;
                    if($uploader_label) {
                        $uploader_html = '<span class="mr-2 h5 d-inline">' . $uploader_label . '</span>';
                    }
                    $uploaded_file = isset($uploaded_files[$field_name . '-' . $i]) ? $uploaded_files[$field_name . '-' . $i] : false;
                    if($is_multi) {
                        $uploaded_file_info = '';
                        if(!empty($uploaded_file) && is_array($uploaded_file)) {
                            foreach($uploaded_file as $file_uploaded_info) {
                                $uploaded_file_info .= '<li>' . $file_uploaded_info . '</li>' . PHP_EOL;
                            }
                        } /*else {
                        $uploaded_file_info = '<li class="text-muted text-center empty">' . lang('file_not_uploaded') . '</li>';
                        }*/
                        $file_info = <<<EOL
        <small class="status text-muted">{$file_select_single}</small>
        <ul class="list-unstyled mt-1 file-list">
            {$uploaded_file_info}
        </ul>
EOL;
            
                    } else {
                        // Display File Name
                        if($uploaded_file[0]) {
                            $file_info = <<<EOL
                    <small class="status text-success">{$file_uploaded}</small>
              		<small class="file-url">{$uploaded_file[0]}</small>
EOL;
                        } else {
                            $file_info = <<<EOL
                    <small class="status text-muted">{$file_select_single}</small>
                    <small class="file-url"></small>
EOL;
                        }
                    }
                    $field_disabled = !$is_multi && $uploaded_file ? ' disabled' : '';
                    $field .= <<<EOL
                <div class="dm-uploader {$field_class}" data-field="{$field_id}">
                      <div class="form-row mb-2">
                          <div class="col"> {$uploader_html}
                            <div class="progress mb-2" style="display: none;">
                            	<div class="progress-bar progress-bar-striped progress-bar-animated bg-primary" role="progressbar" style="width: 0%;" aria-valuenow="0" aria-valuemin="0" aria-valuemax="0">0%</div>
                            </div>
                            <div role="button" class="btn btn-primary mr-2">
                              	<i class="far fa-folder fa-fw"></i> {$file_browse}
                              	<input type="file" title="{$file_click_to_add}"{$field_disabled}>
                            </div>
                            {$file_info}
                          </div>
                  	</div>
                </div>
EOL;
                }
                $field .= $file_template;
                break;
            case 'datetimepicker':
                $field = '<div class="input-group" id="'. $field_name . '">' . PHP_EOL;
                $field .= form_input($field_name, set_value($field_name, $field_value, FALSE), $field_extra);
                $field .= '		<div class="input-group-append input-group-addon"><span class="input-group-text"><i class="fas fa-calendar"></i></span></div>' . PHP_EOL;
                $field .= '	</div>';
                break;
            case 'iconpicker':
                $field = '<div class="input-group iconpicker-container">' . PHP_EOL;
                $field .= form_input($field_name, set_value($field_name, $field_value, FALSE), $field_extra);
                $field .= ' <div class="input-group-append input-group-addon">' . PHP_EOL;
                $field .= '		<span class="input-group-text"></span>' . PHP_EOL;
                $field .= ' </div>' . PHP_EOL;
                $field .= '	</div>';
                // $field = '<div data-search="true" data-search-text="' . lang('search_placeholder') . '" role="iconpicker" data-header="false" data-footer="false"></div>';
                // $field = '<button class="btn btn-warning" data-icon="fab fa-stack-overflow" role="iconpicker"></button>';
                break;
            case 'colorpicker':
                $field = '<div class="input-group gc-colorpicker">' . PHP_EOL;
                $field .= form_input($field_name, set_value($field_name, $field_value, FALSE), $field_extra);
                $field .= ' <div class="input-group-append">' . PHP_EOL;
                $field .= '      <span class="input-group-text input-group-addon"><i></i></span>' . PHP_EOL;
                $field .= ' </div>' . PHP_EOL;
                $field .= '</div>';
                break;
            case 'image':
                $field = '<input id="' . $field_name . '" type="file" name="' . $field_name . '" class="form-control" />' . PHP_EOL;
                if(!empty($field_extra['help_image'])) {
                    $field .= '		<span class="help-block"><small>' . $field_extra['help_image'] . '</small></span>' . PHP_EOL;
                }
                
                if((!empty($_POST['uploaded_image']) || !empty($field_value)) && !isset($field_extra['member_area'])) {
                    $image = !empty($_POST['uploaded_image']) ? $_POST['uploaded_image'] : $field_value;
                    $field .= '		<input type="hidden" name="uploaded_image" id="uploaded_image" class="form-control readonly-field" readonly="readonly" value="' . $field_value . '" />' . PHP_EOL;
                } else {
                    $field .= '		<input type="hidden" name="uploaded_image" id="uploaded_image" />' . PHP_EOL;
                }
                $field .= '		<input type="hidden" name="image_' . $field_name . '" id="img-clear-' . $field_name . '" value="' . $field_value . '" />' . PHP_EOL;
                if(!empty($field_value)) {
                    if(!empty($field_extra['image_path'])) {
                        if(valid_url($field_value)) {
                            $image_src = $field_value;
                            $full_image_src = $field_value;
                        } else {
                            $image_src = $field_extra['image_path'] . '/' . $field_value;
                            $full_image_src = $image_src;
                        }
                    } else {
                        $image_src = gc_image_url($field_value);
                        $full_image_src = gc_image_url($field_value, false);
                    }
                    $field .= '		<div id="image_container" class="fileupload-preview fileupload-exists thumbnail module-img-thumb mt-1 img-clear-' . $field_name . '">' . PHP_EOL;
                    $field .= '			<a href="' . $full_image_src . '" class="gc-fancybox gc-fancybox">' . PHP_EOL;
                    $field .= '				<img src="' . $image_src . '" class="module-img img-thumbnail">' . PHP_EOL;
                    $field .= '			</a>' . PHP_EOL;                    
                    if(isset($field_extra['clear_image'])) {
                        $field .= form_button('clear_image_' . $field_name, '<i class="fas fa-times fa-fw mr-1"></i>' . $field_extra['clear_image'], array('id' => 'clear-' . $field_name, 'class' => 'btn btn-danger btn-sm ml-2 img-clear'));
                        $field .= form_hidden('no_image', set_value('no_image', 0));
                    }
                    $field .= '		</div>';
                }
                break;
            case 'dropdown':
                $options = is_array($data) && isset($data['options']) ? $data['options'] : array();
                $field_extra['id'] = isset($field_extra['id']) ? $field_extra['id'] : $field_name; // set dropdown field ID
                $field = form_dropdown($field_name, $options, set_value($field_name, $field_value), $field_extra);
                break;
            case 'multiselect':
                $options = is_array($data) && isset($data['options']) ? $data['options'] : array();
                $selected = is_array($data) && isset($data['selected']) ? $data['selected'] : array();
                $field = form_multiselect($field_name, $options, $selected, $field_extra);
                break;
            case 'textarea':
                $field_extra['id'] = $field_name;
                $field_extra['rows'] = isset($field_extra['rows']) ? $field_extra['rows'] : 5;
                $field_extra['cols'] = isset($field_extra['cols']) ? $field_extra['cols'] : 10;
                // Field Translate
                if($field_translate) {
                    $field = field_translate($type, $field_name, $field_extra, $data);
                } else {
                    $field = form_textarea($field_name, set_value($field_name, $field_value, FALSE), $field_extra);
                }
                break;
            case 'link':
                $url = isset($data['url']) ? $data['url'] : site_url();
                $field_title = isset($data['title']) ? $data['title'] : $label;
                $field_extra['class'] = isset($field_extra['class']) ? str_replace('form-control', '', $field_extra['class']) : '';
                $field = anchor($url, $field_title, $field_extra);
                break;
            case 'filemanager':
                break;
            case 'map':
                $field = '';
                if(settings_item('lst.map_leaflet') == 1) {
                    $value = (object) $value;
                    if(isset($value->latitude) && isset($value->longitude)) {
                        $map_params = ' data-latitude="' . $value->latitude . '" data-longitude="' . $value->longitude . '"';
                    } else {
                        $CI = &get_instance();
                        $latlng = $CI->gc->get_latlng(); // get latitude and longitude using MaxMind
                        $map_params = ' data-latitude="' . $latlng['latitude'] . '" data-longitude="' . $latlng['longitude'] . '"';
                    }
                    // $value = data-latitude="" data-longitude=""
                    $field .= '<div class="position-relative">';
                    $field .= '<button class="leaflet-current-location btn btn-success btn-xs" type="button" style="position: absolute; top: 0; right: 0; z-index: 10000;"><i class="fas fa-sync fa-fw mr-1"></i>' . lang('current_location') . '</button>';
                    $field .= '<div id="mapLeafletForm" class="map-leaflet"' . $map_params .'></div>';
                    $field .= '</div>';
                } else {
                    $field = '<script type="text/javascript">var centreGot = false;</script>';
                    if(settings_item('lst.google_places_api') == 1) {
                        $field .= '<script type="text/javascript" src="https://maps.googleapis.com/maps/api/js?libraries=places&key=' . settings_item('site.google_api_key') . '"></script>';
                    }
                    $field .= $value['js'];
                    $field .= $value['html'];
                }
                break;
            case 'selectize':
                /**
                 * Common function to add selectize Search
                 * js/search-selectize.js file must be added in controller to use this file
                 * @param $max_items - Maximum items to be selected
                 * @param $load_url - Selectize search url
                 */
                $CI =& get_instance();
                $CI->load->model('fields/fields_model');
                // Load value on POST validation error or from the Saved values
                if(empty($data['selectize_value']) && (isset($_POST[$field_name]) || !empty($field_value))) {
                    if(isset($_POST[$field_name])) {
                        $field_value = $_POST[$field_name];
                    }
                    $module = isset($data['module']) ? $data['module'] : 'users';
                    $data['selectize_value'] = $CI->fields_model->selectize_result($module, $field_name, $field_value);
                }
                
                // pass extra parameters e.g. options etc.
                // params:relation#city_id,root#country_iso
                $extra_params = '';
                if(!empty($data['field_options']['params'])) {
                    // parameters are in comma separated format e.g. options, max_items etc.
                    $extra_params = $data['field_options']['params'];
                }
                
                // option field type_id value
                $selectize_options = '';
                if(!empty($data['field_options']['relation_field']) && !empty($data['field_options']['relation_value'])) {
                    $selectize_table = isset($data['module']) ? $data['module'] : 'options';
                    // load parent specific child only
                    $selectize_where = '';
                    if(!empty($data['field_options']['params'])) {
                        $field_params = explode('#', $data['field_options']['params']);
                        if(!empty($field_params[0]) && $field_params[0] == 'root') {
                            $field_root = explode(',', $field_params[1]);
                            $selectize_record = (array) $value;
                            if(!empty($_POST[$field_root[0]])) {
                                $selectize_where = $_POST[$field_root[0]];
                            } else if(!empty($selectize_record[$field_root[0]])) {
                                $selectize_where = $selectize_record[$field_root[0]];
                            }
                        }
                    }
                    if(empty($data['field_options']['no_ajax']) && empty($selectize_where)) {
                        $selectize_options = '';
                    } else {
                        $selectize_options = $CI->fields_model->load_selectize_options($selectize_table, $data['field_options']['relation_field'], $data['field_options']['relation_value'], $selectize_where);
                    }
                }
                // No Ajax (no_ajax:1)  - Do not send data to Ajax Method, show local options only
                $no_ajax = 0;
                if(!empty($data['field_options']['no_ajax'])) {
                    $no_ajax = 1;
                    if(empty($selectize_options)) {
                        $selectize_options = json_encode($CI->fields_model->database_options(array('table' => $data['field_options']['load'])));
                    }
                }
                $field_extra['id'] = $field_name;
                $field = '<input id="' . $field_name . '" name="' . $field_name . '" class="form-control selectized" placeholder="' . (isset($field_extra['placeholder']) ? $field_extra['placeholder'] : '') . '" value=""' . (!empty($data['selectize_value']) ? " data-selectize-value='" . json_encode($data['selectize_value']) ."'" : '') . (isset($field_extra['required']) ? ' required="true"' : '') . ' />';
                $max_items = isset($data['max_items']) ? $data['max_items'] : 1;
                $load_url = isset($data['load_url']) ? $data['load_url'] : site_url('ajax/selectizeSearch/users');
                $field .= '<input type="hidden" class="selectize-search-input selectize-' . $field_name. '" value="' . $field_name . '" maxItems="' . $max_items . '" loadUrl="' . $load_url . '" data-extra-params="' . $extra_params . '" data-options=\'' . $selectize_options . '\' data-no-ajax="' . $no_ajax . '" />';
                break;
            case 'selectize_old':
                /**
                 * Common function to add selectize Search
                 * js/search-selectize.js file must be added in controller to use this file
                 * @param $max_items - Maximum items to be selected
                 * @param $load_url - Selectize search url
                 */
                if(!isset($field_extra['id'])) {
                    $field_extra['id'] = $field_name;
                }
                if(isset($data['create'])) { // simple selectize without any Ajax operations
                    if(isset($_POST[$field_name])) {
                        $field_value = $_POST[$field_name];
                    }
                    $field = '<input id="' . $field_extra['id'] . '" name="' . $field_name . '" class="form-control selectized" placeholder="' . (isset($field_extra['placeholder']) ? $field_extra['placeholder'] : '') . '" value="' . $field_value . '"' . (isset($field_extra['required']) ? ' required="true"' : '') . ' />';
                } else {
                    // Load value on POST validation error or from the Saved values
                    if(!isset($data['selectize_value']) && (isset($_POST[$field_name]) || !empty($field_value))) {
                        if(isset($_POST[$field_name])) {
                            $field_value = $_POST[$field_name];
                        }
                        $CI =& get_instance();
                        $module = isset($data['module']) ? $data['module'] : 'users';
                        $CI->load->model('fields/fields_model');
                        $data['selectize_value'] = $CI->fields_model->selectize_result($module, $field_name, $field_value);
                    }
                    $field = '<input id="' . $field_extra['id'] . '" name="' . $field_name . '" class="form-control selectized" placeholder="' . (isset($field_extra['placeholder']) ? $field_extra['placeholder'] : '') . '" value=""' . (!empty($data['selectize_value']) ? " data-selectize-value='" . json_encode($data['selectize_value']) ."'" : '') . (isset($field_extra['required']) ? ' required="true"' : '') . ' />';
                    $max_items = isset($data['max_items']) ? $data['max_items'] : 1;
                    $load_url = isset($data['load_url']) ? $data['load_url'] : site_url('ajax/selectizeSearch/users');
                    $field .= '<input type="hidden" class="selectize-search-input" value="' . $field_extra['id'] . '" maxItems="' . $max_items . '" loadUrl="' . $load_url . '" />';
                }
                break;
            case 'rating':
                $CI = &get_instance();
                $field = '<input id="input-ratings" type="number" name="' . $field_name . '" value="' . round($field_value, 2) . '" class="rating" min=0 max=5 step=0.5 data-show-clear="false" data-size="xs"' . ($CI->session->userdata('is_rtl') ? ' data-rtl="true"' : '') . (isset($field_extra['readonly']) && $field_extra['readonly'] ? ' readonly="true"' : '') . '/>' . PHP_EOL;
                $field .= '<div id="rating_feedback"></div>';
                break;
            case 'submit':
                $field_extra['class'] = isset($field_extra['class']) ? str_replace('form-control ', '', $field_extra['class']) : '';
                $field = form_submit($field_name, $field_label, $field_extra);
                break;
            case 'custom':
                $field = fields_custom($data);
                break;
            case 'captcha_code':
                $field_id = isset($field_extra['id']) ? $field_extra['id'] : $field_name;
                $field_placeholder = lang('placeholder_captcha');
                $field_error = sprintf(lang('required'), lang('label_captcha'));
                $field_refresh_url = site_url("securimage");
                $captcha_refresh_text = lang('captcha_refresh');
                $field_required = isset($field_extra['required']) ? ' required data-error="' . $field_error . '"' : false;
                $input_field = '<input type="text" class="form-control input-sm" name="' . $field_name . '" id="' . $field_id . '" value="' . set_value($field_name) . '" placeholder="' . $field_placeholder . '"' . $field_required . ' />';
                $captcha_image_id = $field_id . '_image';
                $field = <<<EOL
                <div class="input-group">
                    {$input_field}
                    <div class="input-group-append">
                        <span class="input-group-text p-0"><img class="img-fluid refresh-captcha" id="{$captcha_image_id}" src="{$field_refresh_url}" alt="CAPTCHA Image" /></span>
                        <span class="input-group-text">
    					   <a id="update" href="#" onclick="document.getElementById('{$captcha_image_id}').src = '{$field_refresh_url}?' + Math.random(); return false" data-toggle="tooltip" data-placement="bottom" title="{$captcha_refresh_text}">
    						  <i class="fas fa-sync"></i>
    					   </a>
    				    </span>
                    </div>
                </div>
EOL;
                    break;
            case 'google_captcha':
                $CI = &get_instance();
                $CI->load->library('recaptcha');
                $field = $CI->recaptcha->getWidget();
                $field .= $CI->recaptcha->getScriptTag();
                break;
            case 'terms_conditions':
                $check_input = '<input type="checkbox" name="' . $field_name . '" id="chk_terms" value="1" ' . set_checkbox($field_name, 1) . '>';
                $lang_display = sprintf(lang('label_conditions_checkbox'), '<a href="#" data-toggle="modal" data-target="#terms_' . $field_name . '" class="captcha-text">' . lang('terms_conditions').'</a>');
                $lang_close = lang('form_close');
                $CI =& get_instance();
                // pass page slug as a field extra attribute
                $page_slug = isset($field_extra['terms_page']) ? $field_extra['terms_page'] : settings_item('fin.terms_and_conditions_page');
                if(is_numeric($page_slug)) {
                    $CI->db->where('id', $page_slug);
                } else {
                    $CI->db->where('slug', $page_slug);
                }
                $page = $CI->db->get('pages')->row();
                if($page) {
                    $field = <<<EOL
			    <div class="checkbox mt-2">
			        <label>
			             {$check_input}
				         {$lang_display}
   					</label>
				</div>
				<div id="terms_{$field_name}" class="modal fade" tabindex="-1" role="dialog" aria-labelledby="TermsAndConditions" aria-hidden="true" class="z-index: 1080 !important;">
					<div class="modal-dialog">
						<div class="modal-content">
							<div class="modal-header">
								<h4 class="modal-title">{$page->title}</h4>
								<button type="button" class="close" aria-label="Close" onclick="$('#terms_{$field_name}').modal('toggle');">
								  <span aria-hidden="true">&times;</span>
								</button>
							</div>
							<div class="modal-body"><div class="terms-block text-left">{$page->body}</div></div>
							<div class="modal-footer">
								<button type="button" class="btn btn-secondary" data-dismiss="modal">{$lang_close}</button>
							</div>
						</div>
					</div>
				</div>
EOL;
                } else {
                    $field = '';
                }
                break;
            case 'password':
                $field_extra['id'] = (isset($field_extra['id']) ? $field_extra['id'] : $field_name);
                // if multiple fields are wrapped in a container e.g. form-inline
                // also add placeholder to the textbox with the help of a prefix
                if(isset($common_data['placeholder_prefix'])) {
                    $field_extra['placeholder'] = lang($common_data['placeholder_prefix'] . $field_name);
                }
                if(!isset($field_extra['class'])) {
                    $field_extra['class'] = $common_data['class'];
                }
                $field_extra['data-toggle'] = 'password';
                $field_html = gc_form_common($type, $field_name, $field_value, $field_label, $field_extra);
                $field = <<<EOL
<div class="input-group">
  {$field_html}
  <div class="input-group-append">
    <span class="input-group-text input-password-hide" style="cursor: pointer;"><i class="fas fa-eye"></i></span>
  </div>
</div>
EOL;
                break;
            default:
                $field_extra['id'] = (isset($field_extra['id']) ? $field_extra['id'] : $field_name);
                $field = gc_form_common($type, $field_name, $field_value, $field_label, $field_extra);
                break;
        }
        
        if($type == 'radio' || $type == 'checkbox') {
            // do not wrap Radio/Checkbox in <div> container if no_wrap attribute is set
            $wrap = is_array($data) && isset($data['no_wrap']) ? FALSE : TRUE;
            
            // Ad Package Create - Hide Checkbox Fields by adding the style and ID in the DIV tab
            $wrap_extra = isset($data['wrap_extra']) ? $data['wrap_extra'] : '';
            
            if($wrap) {
                $field = PHP_EOL . '<div class="' . $type . '"' . $wrap_extra . '>'  . PHP_EOL . $field;
            }
            if(is_array($data) && !empty($data['tooltip'])) {
                $field .= ' <span class="help-inline"><small>' . $data['tooltip'] . '</small></span>';
            }
            if($wrap) {
                $field .= PHP_EOL . '</div>';
            }
        } else if(!$field_translate && is_array($data) && isset($data['tooltip'])) {
            $field .= ' <span class="help-block"><small>' . $data['tooltip'] . '</small></span>';
        }
        if(!$field_translate) {
            // Wrap field within a container e.g. <div>, <li> etc.
            // Code e.g. li.col-sm-3
            if(is_array($data) && !empty($data['field_container'])) {
                $field = emmet($data['field_container'], $field);
                unset($data['field_container']);
            }
            if(is_array($data) && isset($data['field_append'])) {
                // Append text or inline container after a field
                // e.g. span.margin-class{X} - X is the name of the element
                $field = emmet($data['field_append']) . $field;
                unset($data['field_append']);
            }
        }
        return $field;
    }
}

if(! function_exists('field_translate')) {
    /**
     * Multi Language Field - Input Text and Textarea with Languages Dropdown Menu
     * @param string $field_name
     * @param string $field_extra
     * @param array $data - Stored Values and Other Settings
     */
    function field_translate($field_type, $field_name, $field_extra, $data) {
        $separator = '_trlng_';
        $CI =& get_instance();
        // Session library is always loaded, so there is no need to load it on each form field
        $CI->session->unset_userdata('field_languages');
        if(!$CI->session->userdata('field_languages')) {
            $CI->load->library('translations');
            $field_languages = $CI->translations->field_languages();
        } else {
            $field_languages = $CI->session->userdata('field_languages');
        }
        $default_language_iso = $field_languages['default'];
        $languages = $field_languages['languages'];
        $dropdown_menu = $field_languages['menu'];
        $output = array();
        
        // Also change the ID of the field
        $field_id = $field_extra['id'];
        foreach($languages as $language) {
            $key = $language[0];
            $value = $language[1];
            $key_name = !empty($language[2]) ? '<i class="famfamfam-flag-' . $language[2] . '"></i> ' . strtoupper($language[2]) : ucwords($key);
            
            // hide all fields except the default language one
            $style = ($key == $default_language_iso ? '' : ' style="display: none;"');
            $name_field = $field_name . $separator . $key;
            if(!empty($data['value'])) {
                $data['value'] = (array) $data['value']; // typecast to array to avoid any object error
            }
            // Multi Language - Settings Module Fields
            // @param string value_prefix e.g. site.
            if(isset($data['value_prefix'])) {
                $value_prefix = $data['value_prefix'];
                $value_field = $value_prefix . $name_field;
            } else {
                $value_field = $name_field;
            }
            $value_field =  isset($data['value'][$value_field]) ? $data['value'][$value_field] : '';
            
            // Also change the ID of the field
            if(!empty($field_id)) {
                $field_extra['id'] = $field_id . $separator . $key;
            }
            
            if($field_type == 'text') {
                $field = form_input($name_field, set_value($name_field, $value_field, FALSE), $field_extra);
            } else {
                $field = form_textarea($name_field, set_value($name_field, $value_field, FALSE), $field_extra);
            }
            
            // Append additional information with each field
            if(is_array($data) && isset($data['tooltip'])) {
                $field .= ' <span class="help-block"><small>' . $data['tooltip'] . '</small></span>';
            }
            // Wrap field within a container e.g. <div>, <li> etc.
            // Code e.g. li.col-sm-3
            if(is_array($data) && isset($data['field_container'])) {
                $field = emmet($data['field_container'], $field);
            }
            if(is_array($data) && isset($data['field_append'])) {
                // Append text or inline container after a field
                // e.g. span.margin-class{X} - X is the name of the element
                $field = emmet($data['field_append']) . $field;
            }
            
            $output[] = <<<EOL
            
<div class="translate-field lang-{$key}"{$style}>
	<div class="input-group">
        {$field}
	   <div class="input-group-append">
           <div class="dropdown show">
		      <a type="button" class="btn btn-secondary text-white dropdown-toggle" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">
			     {$key_name}
		      </a>
		      {$dropdown_menu}
	       </div>
       </div>
    </div>
</div>

EOL;
        }
        return implode('', $output);
    }
}

if(! function_exists('field_with_options')) {
    /**
     * Custom fields having multiple options e.g. INPUT and CHECKBOXES
     */
    function field_with_options($field_label, $field, $record, $type = 'text') {
        $checkbox_array = array(); // Checkbox array data
        $field_name = $field['name'];
        $json_field = FALSE;
        $form_inline_elements = array(); // display fields inline e.g. Latitude and Longitude
        // if the validation string has the is_array rule then the field type is an array
        $record = (object) $record;
        if(strpos($field['rules'], 'is_array') !== FALSE) {
            $option_value = isset($record->{$field_name}) ? json_decode($record->{$field_name}) : array();
            $json_field = TRUE;
        } else {
            $option_value = $record; // isset($record->$field_name) ? json_decode($record->$field_name) : '';
        }
        
        // Options parameters that are passed
        foreach($field['options'] as $option) {
            $option_extra = array();
            // field name
            $option_name = (!empty($option['name']) ? $option['name'] : $option);
            // size of the input field
            $option_size = (!empty($option['class']) ? $option['class'] : ($field['class'] == 0 ? 'col-sm-6' : 'col-sm-' . $field['class']));
            // specific to checkbox
            if($type == 'checkbox') {
                if(!empty($option['id'])) { // options table
                    $option_label = $option['name'];
                } else {
                    $option_label = isset($option['label']) ? $option['label'] : $option_name;
                    $option_label = $field['options_lang'] == 1 ? lang('label_' . $option_label) : $option_label;
                }
                $default_value = (!empty($option['value']) ? $option['value'] : 1);
            } else {
                // label of the field
                $option_label = (!empty($option['label']) ? $option['label'] : ($field['options_lang'] == 1 ? lang('label_' . $option_name) : $option_name));
                // input field placeholder
                if(!empty($option['placeholder'])) {
                    $option_extra['placeholder'] = ($field['options_lang'] == 1 ? lang($option['placeholder']) : $option['placeholder']);
                }
                // input field required attribute,  if not added in options, get it from the validation rules
                $option_required = isset($option['required']) || $field['required'] ? TRUE : FALSE;
                if($option_required) {
                    // if option is required, append * to the field label
                    $option_label = $option_label . lang('bf_form_label_required');
                    //$option_extra['required'] = TRUE;
                }
                
                // field default value
                $default_value = (!empty($option['value']) ? $option['value'] : (isset($field['value']) ? $field['value'] : ''));
            }
            // tooltip
            $option_tooltip = (!empty($option['tooltip']) ? $option['tooltip'] : (!empty($field['tooltip']) ? $field['tooltip'] : ''));
            
            $input_data = array();
            if($json_field) {
                // pass json field name to the form_helper for values
                $input_data['field_json'] = $field_name;
                if($type == 'checkbox' && !empty($option['id'])) { // options table
                    $input_data['json_field'] = $option['id'];
                    $option_field_name = $field_name . '[' . $option['id'] . ']';
                } else {
                    $input_data['json_field'] = $option_name;
                    $option_field_name = $field_name . '[' . $option_name . ']';
                }
            } else {
                $option_field_name = $option_name;
            }
            
            $input_data['name'] = $option_field_name;
            $input_data['label_width'] = isset($field['label_width']) ? $field['label_width'] : 'col-sm-3';
            $input_data['extra'] = $option_extra;
            $input_data['help_class'] = $field['tooltip_position'];
            $input_data['tooltip'] = $option_tooltip;
            $input_data['element_container'] = (!empty($field['element_container']) ? $field['element_container'] : '');
            $input_data['field_container'] = (!empty($field['field_container']) ? $field['field_container'] : '');
            // add field to the form
            if($type == 'text') {
                $input_data['default_value'] = $default_value;
                // Display Latitude and Longitude field in one line
                if($field['control_class'] == 'form-inline') {
                    $input_data['field_only'] =  TRUE;
                    $form_inline_elements[] = gc_form_element($type, $input_data, $option_value, $option_label, $option_size);
                } else {
                    echo gc_form_element($type, $input_data, $option_value, $option_label, $option_size);
                }
            } else {
                $input_data['label'] = $option_label; // display label with checkbox
                if(!empty($option['icon'])) {
                    $input_data['icon'] = base_url() . 'assets/images/icons/ico_' . $option['icon'];
                }
                if(!empty($option['image'])) {
                    $input_data['icon'] = $option['image']['original'];
                }
                $checkbox_input = $input_data;
                unset($checkbox_input['tooltip']);
                $checkbox_array[] = $checkbox_input;
            }
        }
        $size = !empty($field['class']) ? 'col-sm-' . $field['class'] : 'col-sm-6';
        $is_array = !empty($field['is_array']) ? $field['is_array'] : '';
        if(!empty($form_inline_elements)) {
            return gc_form_element('form_elements', array('name' => '', 'is_array' => $is_array, 'label_width' => 'col-sm-3', 'control_class' => $field['control_class'], 'form_elements' => $form_inline_elements, 'tooltip' => $input_data['tooltip'], 'help_class' => $input_data['help_class']), array(), $field_label, $size);
        }
        if(!empty($checkbox_array)) {
            // if you want to display label with a checkbox, enter it in the placeholder field
            $checkbox_label = (isset($field['placeholder_lang']) && $field['placeholder_lang'] == 1 ? lang($field['placeholder']) : $field['placeholder']);
            if(!empty($field['required'])) {
                $checkbox_label .= lang('bf_form_label_required');
            }
            return gc_form_element('checkbox', array('name' => '', 'is_array' => $is_array, 'label_width' => 'col-sm-3', 'options' => $checkbox_array, 'element_container' => $input_data['element_container'], 'tooltip' => $input_data['tooltip'], 'help_class' => $input_data['help_class']), $option_value, $checkbox_label, $size);
        }
    }
}

if ( ! function_exists('emmet')) {
	/**
	 * https://packagist.org/packages/artem_c/emmet
	 * https://emmet.io/
	 * https://github.com/artemBilik/emmet
	 * ADDED THROUGH COMPOSER IN THE VENDORS FOLDER
	 * # is used to allot ID
	 * . is used to allot class
	 *
	 * For multiple classes use: ul.class1.class2.class3
	 *
	 * ul#abc will make it <ul id="abc"></ul>
	 * ul.class1.class2.class3 will make it <ul class="class1 class2 class3"></ul>
	 * ul#abc.class1.class2.class3 will make it <ul id="abc" class="class1 class2 class3"></ul>
	 * $separator_string = '{####}'
	 * @param string $syntax - String to be parsed
	 * @param string $separator_string - separator_string/value between container
	 * Default Value -> split start and ending tag and return the value
	 * Passed Value -> Do not split, return the complete HTML code
	 *
	 * Note: Do not pass $form_elements directly as a separator else it will result in Garbage output
	 * $form_elements has HTML string which result in invalid HTML generation
	 */
    function emmet($syntax, $html_elements = NULL) {
		if(!empty($html_elements)) {
			$emmet = new \artem_c\emmet\Emmet($syntax. '{`value`}');
			$string = $emmet->create([ 'value' => PHP_EOL . $html_elements ]);
		} else { // to parse custom string e.g. span.margin-class{X}
			$emmet = new \artem_c\emmet\Emmet($syntax);
			$string = $emmet->create();
		}
		return $string;
	}
}

if ( ! function_exists('form_actions')) {
    /**
     * Prepare input field data
     * @param string $type    		A string with the name of the element type.
     * @param string $data    		Either a string with the element name, or an array of key/value pairs of all attributes.
     */
    function form_actions($data = array()) {
        $form_elements = array();
        $size = isset($data['size']) ? $data['size'] : 'col-sm-9 offset-sm-3';
        $name = (isset($data['name']) ? $data['name'] : 'save');
        $label = (isset($data['label']) ? '<i class="fas fa-paper-plane fa-fw mr-1"></i>' . $data['label'] : '<i class="fas fa-paper-plane fa-fw mr-1"></i>' . lang('save_exit'));
        $submit_extra = array(
            'name' => $name,
            'content' => $label,
            'value' => true,
            'type' => 'submit',
            'class' => 'btn btn-primary btn-theme-primary btn-submit' // btn-submit class is added to perform jquery related operation like changing package in the Listings Form
        );
        $form_elements[] = form_button($submit_extra);
        
        // hidden button to avoid multiple clicks in a fraction of second
        $submit_extra['name'] = 'save_hidden';
        $save_only['type'] = 'button';
        $submit_extra['hidden'] = true;
        $submit_extra['disabled'] = true;
        $submit_extra['class'] = 'btn btn-primary btn-theme-primary btn-form-hidden';
        $form_elements[] = form_button($submit_extra);
        
        if(isset($data['save_only'])) {
            // Allow user to save the data but do not redirect to another page
            $save_only_label = '<i class="fas fa-save fa-fw mr-1"></i>' . lang('save');
            $save_only = array(
                'name' => 'save_only',
                'content' => $save_only_label,
                'value' => true,
                'type' => 'submit',
                'class' => 'btn btn-info btn-theme-primary btn-submit' // btn-submit class is added to perform jquery related operation like changing package in the Listings Form
            );
            $form_elements[] = lang('bf_or') . ' ' . form_button($save_only);
            
            // hidden button to avoid multiple clicks in a fraction of second
            $save_only['name'] = 'save_only_hidden';
            $save_only['type'] = 'button';
            $save_only['hidden'] = true;
            $save_only['disabled'] = true;
            $save_only['class'] = 'btn btn-primary btn-theme-primary btn-form-hidden';
            $form_elements[] = form_button($save_only);
        }
        
        $cancel = isset($data['cancel']) ? FALSE : TRUE;
        $delete = isset($data['delete']) ? FALSE : TRUE;
        if($cancel) {
            $cancel_extra = array(
                'class' => 'btn btn-warning btn-theme-secondary'
            );
            $url = isset($data['url']) ? $data['url'] : site_url();
            $form_elements[] = lang('bf_or') . ' ' . anchor($url, '<i class="fas fa-undo fa-fw mr-1"></i>' . lang('cancel'), $cancel_extra);
        }
        if($delete) {
            $delete_extra = array(
                'id' => 'delete-me',
                'name' => 'delete',
                'type' => 'submit',
                'value' => true,
                'content' => '<i class="fas fa-times fa-fw mr-1"></i>' . lang('bf_action_delete'),
                'class' => 'btn btn-danger',
                'onclick' => 'return confirm(\'' . js_escape(lang('delete_confirm')) . '\')'
            );
            $form_elements[] = lang('bf_or') . ' ' . form_button($delete_extra);
        }
        // Additional Buttons
        if(!empty($data['extra_actions'])) {
            foreach($data['extra_actions'] as $extra_action) {
                $form_elements[] = lang('bf_or') . ' ' . $extra_action;
            }
        }
        $form_elements = implode("\n", $form_elements);
        $output = <<<EOL
        
<div class="form-row card card-body bg-light">
    <div class="{$size}">
		{$form_elements}
	</div>
</div>

EOL;
		
		return $output;
    }
}

if ( ! function_exists('form_validations')) {
	/**
	 * Common method to display validation messages on all forms
	 */
	function form_validations() {
		$validation_errors = validation_errors();
		if(!$validation_errors) {
			return false;
		}
		$heading = lang('validations_error');
		$output = <<<EOL

<div class="alert alert-block alert-danger">
	<a class="close" data-dismiss="alert">&times;</a>
	<h4 class="alert-heading">{$heading}</h4>
	{$validation_errors}
</div>

EOL;

		return $output;
	}
}

if( ! function_exists('form_city')) {
	/**
	 * Google Places API/Custom City selection
	 * @param $hasLocality
	 * @param $city_only
	 * @param $withoutMap - Load jQuery and other files if without Map
	 */
	function form_city($data) {
		$hasLocality = !isset($data['hasLocality']) || (isset($data['hasLocality']) && $data['hasLocality']) ? true : false;
		$city_only = isset($data['city_only']) && $data['city_only'] ? true : false;
		$required = isset($data['required']) && !$data['required'] ? false : true;
		$field_label = ($required ? lang('label_city') . lang('bf_form_label_required') : lang('label_city'));

		// Saved values are also displayed using the POST variable
		$selected_city = !empty($_POST['selected_city']) ? $_POST['selected_city'] : '';
		$selected_locality = !empty($_POST['selected_locality']) ? $_POST['selected_locality'] : '';
		$city_id = !empty($_POST['city_id']) ? $_POST['city_id'] : 0;
		$hasLocalities = !empty($_POST['hasLocalities']) ? TRUE : FALSE;
		$locality_id = !empty($_POST['locality_id']) ? $_POST['locality_id'] : 0;
		$label_width = isset($data['label_width']) ? $data['label_width'] : 'col-sm-3';
		$size = isset($data['size']) ? $data['size'] : 'col-sm-6';
		$field_only = isset($data['field_only']) && $data['field_only'] ? true : false;
		
		$form_elements = array();
		if(settings_item('lst.google_places_api') == 0) {
			$city_input_elements = array(
				gc_form_element('text', array('name' => 'city', 'field_only' => TRUE, 'field_class' => 'search-typeahead', 'extra' => array('id' => 'cities', 'autocomplete' => 'off', 'placeholder' => lang('help_city'))), (isset($selected_city) ? $selected_city : ''), $field_label),
				gc_form_element('button', array('name' => '', 'field_only' => TRUE, 'field_container' => 'div.input-group-append.search-input-group-btn', 'field_class' => 'city-spinner btn ' . (!empty($selected_city) ?  'btn-success' : 'btn-warning')), array(), '<i class="fas ' . (!empty($selected_city) ? 'fa-check-circle' : 'fa-info-circle'). '"></i>')
			);
			$form_elements[] = gc_form_element('form_elements', array('name' => '', 'element_container' => 'div.input-group', 'label_width' => $label_width, 'form_elements' => $city_input_elements, 'field_only' => $field_only), array(), $field_label, $size);

			$form_elements[] = form_hidden ( 'city_id', isset($city_id) ? $city_id : '');
			if($hasLocality) {
				$form_elements[] = form_hidden ( 'hasLocalities', isset($hasLocalities) ? $hasLocalities : 0);
				$form_elements[] = form_hidden ( 'locality_id', isset($locality_id) ? $locality_id : '');
			
				$locality_input_elements = array(
					gc_form_element('text', array('name' => 'locality', 'field_only' => TRUE, 'field_class' => 'search-typeahead', 'extra' => array('id' => 'localities', 'autocomplete' => 'off', 'placeholder' => lang('help_locality'))), $selected_locality, lang('select_locality')),
					gc_form_element('button', array('name' => '', 'field_only' => TRUE, 'field_container' => 'div.input-group-append.search-input-group-btn', 'field_class' => 'locality-spinner btn ' . (!empty($selected_locality) ?  'btn-success' : 'btn-warning')), array(), '<i class="fas ' . (!empty($selected_locality) ? 'fa-check-circle' : 'fa-info-circle'). '"></i>')
				);
				$control_style = isset($selected_city) && isset($hasLocalities) && $hasLocalities > 0 ? '' : 'display: none;';
				$form_elements[] = gc_form_element('form_elements', array('name' => '', 'control_class' => 'locality-selection', 'control_style' => $control_style, 'element_container' => 'div.input-group', 'label_width' => $label_width, 'form_elements' => $locality_input_elements, 'field_only' => $field_only), array(), lang('select_locality'), $size);
			}
		} else {
			$city_name = !empty($_POST['city_name']) ? $_POST['city_name'] : '';			
			$state_abbrev = !empty($_POST['state_abbrev']) ? $_POST['state_abbrev'] : '';
			$state_name = !empty($_POST['state_name']) ? $_POST['state_name'] : '';
			$country_name = !empty($_POST['country_name']) ? $_POST['country_name'] : '';
			$country_iso = !empty($_POST['country_iso']) ? $_POST['country_iso'] : '';
			
			$form_elements[] = gc_form_element('text', array('name' => 'txtPlaces', 'label_width' => $label_width, 'extra' => array('placeholder' => lang('help_city')), 'field_only' => $field_only), (isset($selected_city) ? $selected_city : ''), $field_label, $size);
			$form_elements[] = form_hidden ( 'city', $city_name);
			$form_elements[] = form_hidden ( 'state_name', $state_name);
			$form_elements[] = form_hidden ( 'state_abbrev', $state_abbrev);	
			$form_elements[] = form_hidden ( 'country_name', $country_name);
			$form_elements[] = form_hidden ( 'country_iso', $country_iso);
			
			$withoutMap = FALSE;
			if($city_only) {
				$withoutMap = TRUE;
				// Load Google Map API
				$form_elements[] = '<script type="text/javascript" src="https://maps.googleapis.com/maps/api/js?libraries=places&key=' . settings_item('site.google_api_key') . '"></script>';

				//Assets::add_js('https://maps.googleapis.com/maps/api/js?libraries=places&key=' . settings_item('site.google_api_key'), 'external', TRUE);
			}
			$CI = &get_instance();
			Assets::add_js($CI->load->view('common/google_places_js', array('withoutMap' => $withoutMap), TRUE), 'inline');
		}
		// set city_only to true if the page has no Google Map e.g. checkout and user profile page
		/**/
		
		// Wrap field within a container e.g. <div>, <li> etc.
		// Code e.g. li.col-sm-3
		$form_elements = implode(PHP_EOL, $form_elements);
		if(is_array($data) && !empty($data['field_container'])) {
		    $form_elements = emmet($data['field_container'], $form_elements);
		}
		return $form_elements;
	}
}

if( ! function_exists('location_selectize')) {
	/**
	 * Country, State and City selection using selectize
	 * Added for Orders->Reports method
	 */
	function location_selectize($data = array(), $countries = array(), $selected_country = '', $states = array(), $selected_state = '', $cities = array(), $selected_city = '') {
		$form_elements = '';
		$size = isset($data['size']) ? $data['size'] : 'col-sm-4';
		$required = isset($data['required']) && $data['required'] ? lang('bf_form_label_required') : '';
		$countries_option = array('' => lang('placeholder_country'));
		if(!empty($countries)):
			foreach($countries as $country):
				$countries_option[$country->iso] = ucwords(strtolower($country->name)); 
			endforeach;
		endif;
		$form_elements .= gc_form_element('dropdown', array('name' => 'country_iso', 'options' => $countries_option), $selected_country, lang('label_select_country'). $required, $size) . PHP_EOL;
		
		$states_option = array('' => lang('placeholder_state'));
		if(!empty($states)):
			foreach($states as $state):
				$states_option[$state->id] = $state->name;
			endforeach;
		endif;
		$form_elements .= gc_form_element('dropdown', array('name' => 'state_id', 'options' => $states_option), $selected_state, lang('label_select_state'). $required, $size) . PHP_EOL;
		
		$cities_option = array('' => lang('placeholder_city'));
		if(!empty($cities)):
			foreach($cities as $city):
				$cities_option[$city->id] = $city->name;
			endforeach;
		endif;
		$form_elements .= gc_form_element('dropdown', array('name' => 'city_id', 'options' => $cities_option), $selected_city, lang('label_select_city'). $required, $size);
		return $form_elements;
	}
}

if( ! function_exists('field_location')) {
    /**
     * Country, State and City selection using selectize
     * Added for Orders->Reports method
     * Dependencies:
     Assets::add_css('selectize.bootstrap3.css');
     Assets::add_js('selectize.min.js');
     Assets::add_js('selectize-search.js');
     Assets::add_js('location-selection.js');
     */
    function field_location($data = array(), $record = array()) {
        $CI =& get_instance();
        $record = (object) $record;
        $form_elements = '';
        $field_id = '';
        $field_name = '';
        if(isset($data['name']) && $data['name'] != 'location' && $data['name'] != 'city') {
            $field_name = str_replace('city_id', '', $data['name']);
            $field_id = $field_name;
        } else {
            $field_id = 'location_';
        }
        $country_column = $field_name . 'country_iso';
        $state_column = $field_name . 'state_id';
        $city_column = $field_name . 'city_id';
        $label_width = isset($data['label_width']) ? $data['label_width'] : 'col-sm-3';
        $size = isset($data['size']) ? $data['size'] : 'col-sm-4';
        $required = isset($data['required']) && $data['required'] ? lang('bf_form_label_required') : '';
        $countries_option = array('' => lang('placeholder_select'));
        if(empty($countries)) {
            $countries = $CI->db->select('iso, name')->order_by('name', 'ASC')->get('countries')->result();
        }
        // saved/selection locations handling
        $states_option = array('' => lang('placeholder_select'));
        $selected_country = !empty($_POST[$country_column]) ? $_POST[$country_column] : (!empty($record->{$country_column}) ? $record->{$country_column} : '');
        if(!empty($selected_country)) {
            $states = $CI->db->where(array('country_iso' => $selected_country))->order_by('name', 'ASC')->get('states')->result();
        }
        $selected_state = !empty($_POST[$state_column]) ? $_POST[$state_column] : (!empty($record->{$state_column}) ? $record->{$state_column} : '');
        if(!empty($selected_state)) {
            $cities = $CI->db->where(array('state_id' => $selected_state))->order_by('name', 'ASC')->get('cities')->result();
        }
        if(!empty($countries)) {
            foreach($countries as $country) {
                $countries_option[$country->iso] = ucwords(strtolower($country->name));
            }
        }
        $form_elements .= gc_form_element('dropdown', array('name' => $country_column, 'label_width' => $label_width, 'options' => $countries_option, 'field_class' => 'location-country-iso', 'extra' => array('id' => $field_id . 'country_iso')), $record, lang('label_country'). $required, $size) . PHP_EOL;
        if(!empty($states)) {
            foreach($states as $state) {
                $states_option[$state->id] = $state->name;
            }
        }
        $form_elements .= gc_form_element('dropdown', array('name' => $state_column, 'label_width' => $label_width, 'options' => $states_option, 'field_class' => 'location-state-id', 'extra' => array('id' => $field_id . 'state_id')), $selected_state, lang('label_state'). $required, $size) . PHP_EOL;
        $selected_city = !empty($_POST[$city_column]) ? $_POST[$city_column] : (!empty($record->{$city_column}) ? $record->{$city_column} : '');
        $cities_option = array('' => lang('placeholder_select'));
        if(!empty($cities)) {
            foreach($cities as $city) {
                $cities_option[$city->id] = $city->name;
            }
        }
        $form_elements .= gc_form_element('dropdown', array('name' => $city_column, 'label_width' => $label_width, 'options' => $cities_option, 'field_class' => 'location-city-id', 'extra' => array('id' => $field_id . 'city_id')), $selected_city, lang('label_city'). $required, $size);
        return $form_elements;
    }
}

if( ! function_exists('search_admin')) {
    /**
     * Admin Panel Index Page Search Block
     * @param $placeholder
     * @param $searchterm
     */
    function search_admin($placeholder, $size = 'col-sm-6', $margins = 'mt-3 mb-3') {
        $btn_value = lang('label_admin_search');
        $value = set_value('search', (isset($_POST['searchterm']) ? $_POST['searchterm'] : ''));
        $output = <<<EOL
        
<div class="input-group {$margins} {$size} float-right">
  <input name="search" type="text" class="form-control" placeholder="{$placeholder}" id="search" aria-label="{$placeholder}" aria-describedby="" value="{$value}" />
  <div class="input-group-prepend">
    <button class="btn btn-outline-secondary" type="submit">{$btn_value}</button>
  </div>
</div>

EOL;
        
        return $output;
    }
}

if( ! function_exists('form_filter')) {
	/**
	 * Admin Panel Index Page Search Block
	 * @param $placeholder
	 * @param $searchterm
	 */
	function form_filter($field, $btn_value, $tooltip = '') {
		$icon = (isset($_POST['sort_column']) && $_POST['sort_column'] == $field ? '<i class="fas fa-sort-'. ($_POST['sort_order'] == 'desc' ? 'up' : 'down') .'"></i>' : '');
		if(!empty($tooltip)) {
			$tooltip = ' data-toggle="tooltip" data-placement="bottom" data-original-title="' . $tooltip . '"';
		}
		$output = <<<EOL

<button type="submit" name="sort" value="{$field}" class="btn-sort"{$tooltip}>
	{$btn_value} {$icon}
</button>

EOL;

		return $output;
	}
}

//--------------------------------------------------------------------

if ( ! function_exists('form_email'))
{
	/**
	 * Returns a properly templated email input field.
	 *
	 * @param string $data    Either a string with the element name, or an array of key/value pairs of all attributes.
	 * @param string $value   Either a string with the value, or blank if an array is passed to the $data param.
	 * @param string $label   A string with the label of the element.
	 * @param string $extra   A string with any additional items to include, like Javascript.
	 * @param string $tooltip A string for inline help or a tooltip icon
	 *
	 * @return string A string with the formatted input element, label tag and wrapping divs.
	 */
	function form_email($data='', $value='', $label='', $extra='', $tooltip = '')
	{
		return _form_common('email', $data, $value, $label, $extra, $tooltip);

	}//end form_email()
}

//--------------------------------------------------------------------

if ( ! function_exists('form_password'))
{
	/**
	 * Returns a properly templated password input field.
	 *
	 * @param string $data    Either a string with the element name, or an array of key/value pairs of all attributes.
	 * @param string $value   Either a string with the value, or blank if an array is passed to the $data param.
	 * @param string $label   A string with the label of the element.
	 * @param string $extra   A string with any additional items to include, like Javascript.
	 * @param string $tooltip A string for inline help or a tooltip icon
	 *
	 * @return string A string with the formatted input element, label tag and wrapping divs.
	 */
	function form_password($data='', $value='', $label='', $extra='', $tooltip = '')
	{
		return _form_common('password', $data, $value, $label, $extra, $tooltip);

	}//end form_password()
}

//--------------------------------------------------------------------

if ( ! function_exists('form_url'))
{
	/**
	 * Returns a properly templated URL input field.
	 *
	 * @param string $data    Either a string with the element name, or an array of key/value pairs of all attributes.
	 * @param string $value   Either a string with the value, or blank if an array is passed to the $data param.
	 * @param string $label   A string with the label of the element.
	 * @param string $extra   A string with any additional items to include, like Javascript.
	 * @param string $tooltip A string for inline help or a tooltip icon
	 *
	 * @return string A string with the formatted input element, label tag and wrapping divs.
	 */
	function form_url($data='', $value='', $label='', $extra='', $tooltip = '')
	{
		return _form_common('url', $data, $value, $label, $extra, $tooltip);

	}//end form_url()
}

//--------------------------------------------------------------------

if ( ! function_exists('form_telephone'))
{
	/**
	 * Returns a properly templated Telephone input field.
	 *
	 * @param string $data    Either a string with the element name, or an array of key/value pairs of all attributes.
	 * @param string $value   Either a string with the value, or blank if an array is passed to the $data param.
	 * @param string $label   A string with the label of the element.
	 * @param string $extra   A string with any additional items to include, like Javascript.
	 * @param string $tooltip A string for inline help or a tooltip icon
	 *
	 * @return string A string with the formatted input element, label tag and wrapping divs.
	 */
	function form_telephone($data='', $value='', $label='', $extra='', $tooltip = '')
	{
		return _form_common('tel', $data, $value, $label, $extra, $tooltip);

	}//end form_telephone()
}

//--------------------------------------------------------------------

if ( ! function_exists('form_number'))
{
	/**
	 * Returns a properly templated number input field.
	 *
	 * @param string $data    Either a string with the element name, or an array of key/value pairs of all attributes.
	 * @param string $value   Either a string with the value, or blank if an array is passed to the $data param.
	 * @param string $label   A string with the label of the element.
	 * @param string $extra   A string with any additional items to include, like Javascript.
	 * @param string $tooltip A string for inline help or a tooltip icon
	 *
	 * @return string A string with the formatted input element, label tag and wrapping divs.
	 */
	function form_number($data='', $value='', $label='', $extra='', $tooltip = '')
	{
		return _form_common('number', $data, $value, $label, $extra, $tooltip);

	}//end form_number()
}

//--------------------------------------------------------------------

if ( ! function_exists('form_range'))
{
	/**
	 * Returns a properly templated range input field.
	 *
	 * NOTE: The $data value should be an array and should contain both
	 * 'min' and 'max' values. If they are not present, then they will
	 * default to 1 and 10, respectively.
	 *
	 * @param string $data    Either a string with the element name, or an array of key/value pairs of all attributes.
	 * @param string $value   Either a string with the value, or blank if an array is passed to the $data param.
	 * @param string $label   A string with the label of the element.
	 * @param string $extra   A string with any additional items to include, like Javascript.
	 * @param string $tooltip A string for inline help or a tooltip icon
	 *
	 * @return string A string with the formatted input element, label tag and wrapping divs.
	 */
	function form_range($data='', $value='', $label='', $extra='', $tooltip = '')
	{
		if (is_string($data))
		{
			$data = (array)$data;
		}

		$data['min'] = isset($opts['min']) ? $opts['min'] : 1;
		$data['max'] = isset($opts['max']) ? $opts['max'] : 10;

		return _form_common('range', $data, $value, $label, $extra, $tooltip);

	}//end form_color()
}

//--------------------------------------------------------------------

if ( ! function_exists('form_color'))
{
	/**
	 * Returns a properly templated color input field.
	 *
	 * @param string $data    Either a string with the element name, or an array of key/value pairs of all attributes.
	 * @param string $value   Either a string with the value, or blank if an array is passed to the $data param.
	 * @param string $label   A string with the label of the element.
	 * @param string $extra   A string with any additional items to include, like Javascript.
	 * @param string $tooltip A string for inline help or a tooltip icon
	 *
	 * @return string A string with the formatted input element, label tag and wrapping divs.
	 */
	function form_color($data='', $value='', $label='', $extra='', $tooltip = '')
	{
		return _form_common('color', $data, $value, $label, $extra, $tooltip);

	}//end form_color()
}

//--------------------------------------------------------------------

if ( ! function_exists('form_search'))
{
	/**
	 * Returns a properly templated search input field.
	 *
	 * @param string $data    Either a string with the element name, or an array of key/value pairs of all attributes.
	 * @param string $value   Either a string with the value, or blank if an array is passed to the $data param.
	 * @param string $label   A string with the label of the element.
	 * @param string $extra   A string with any additional items to include, like Javascript.
	 * @param string $tooltip A string for inline help or a tooltip icon
	 *
	 * @return string A string with the formatted input element, label tag and wrapping divs.
	 */
	function form_search($data='', $value='', $label='', $extra='', $tooltip = '')
	{
		return _form_common('search', $data, $value, $label, $extra, $tooltip);

	}//end form_search()
}

//--------------------------------------------------------------------

if ( ! function_exists('form_date'))
{
	/**
	 * Returns a properly templated date input field.
	 *
	 * @param string $data    Either a string with the element name, or an array of key/value pairs of all attributes.
	 * @param string $value   Either a string with the value, or blank if an array is passed to the $data param.
	 * @param string $label   A string with the label of the element.
	 * @param string $extra   A string with any additional items to include, like Javascript.
	 * @param string $tooltip A string for inline help or a tooltip icon
	 *
	 * @return string A string with the formatted input element, label tag and wrapping divs.
	 */
	function form_date($data='', $value='', $label='', $extra='', $tooltip = '')
	{
		return _form_common('date', $data, $value, $label, $extra, $tooltip);

	}//end form_date()
}

//--------------------------------------------------------------------

if ( ! function_exists('form_datetime'))
{
	/**
	 * Returns a properly templated date input field.
	 *
	 * @param string $data    Either a string with the element name, or an array of key/value pairs of all attributes.
	 * @param string $value   Either a string with the value, or blank if an array is passed to the $data param.
	 * @param string $label   A string with the label of the element.
	 * @param string $extra   A string with any additional items to include, like Javascript.
	 * @param string $tooltip A string for inline help or a tooltip icon
	 *
	 * @return string A string with the formatted input element, label tag and wrapping divs.
	 */
	function form_datetime($data='', $value='', $label='', $extra='', $tooltip = '')
	{
		return _form_common('datetime', $data, $value, $label, $extra, $tooltip);

	}//end form_date()
}

//--------------------------------------------------------------------

if ( ! function_exists('form_month'))
{
	/**
	 * Returns a properly templated month input field.
	 *
	 * @param string $data    Either a string with the element name, or an array of key/value pairs of all attributes.
	 * @param string $value   Either a string with the value, or blank if an array is passed to the $data param.
	 * @param string $label   A string with the label of the element.
	 * @param string $extra   A string with any additional items to include, like Javascript.
	 * @param string $tooltip A string for inline help or a tooltip icon
	 *
	 * @return string A string with the formatted input element, label tag and wrapping divs.
	 */
	function form_month($data='', $value='', $label='', $extra='', $tooltip = '')
	{
		return _form_common('month', $data, $value, $label, $extra, $tooltip);

	}//end form_date()
}

//--------------------------------------------------------------------
//EXTEND FORM_HIDDEN FOR ID
if ( ! function_exists('form_hidden'))
{
	function form_hidden($name, $value = '', $id = false)
	{
		if ( ! is_array($name))
		{
			return '<input type="hidden" id="'.($id ? $id : $name).'" name="'.$name.'" value="'.form_prep($value).'" />';
		}

		$form = '';

		foreach ($name as $name => $value)
		{
			$form .= "\n";
			$form .= '<input type="hidden"  id="'.($id ? $id : $name).'" name="'.$name.'" value="'.form_prep($value).'" />';
		}

		return $form;
	}
}


if ( ! function_exists('form_textarea'))
{
	/**
	 * Textarea field
	 *
	 * @param	mixed	$data
	 * @param	string	$value
	 * @param	mixed	$extra
	 * @return	string
	 */
	function form_textarea($data = '', $value = '', $extra = '')
	{
		$defaults = array(
			'name' => is_array($data) ? '' : $data,
			'cols' => (is_array($extra) && isset($extra['cols']) ? $extra['cols'] : '10'),
			'rows' => (is_array($extra) && isset($extra['rows']) ? $extra['rows'] : '5')
		);
		// unset rows and cols as they have alredy been passed above
		unset($extra['cols']);
		unset($extra['rows']);
		if ( ! is_array($data) OR ! isset($data['value'])) {
			$val = $value;
		} else {
			$val = $data['value'];
			unset($data['value']); // textareas don't use the value attribute
		}

		return '<textarea '._parse_form_attributes($data, $defaults)._attributes_to_string($extra).'>'
			.html_escape($val)
			."</textarea>\n";
	}
}

if ( ! function_exists('field_search')) {
    /**
     * Common search fields
     * @param array $data
     * @return string
     */
    function field_search($data = array()) {
        $output = '';
        $field = $data['field'];
        $field_class = isset($data['class']) ? ' ' . $data['class'] : '';
        switch($field) {
            case 'city':
                $deafult_country = isset($data['country']) ? $data['country'] : 'US';
                $default_state = isset($data['state']) ? $data['state'] : 0;
                $default_city = isset($data['city']) ? $data['city'] : 0;
                $location = isset($data['location']) ? $data['location'] : 0;
                $placeholder = lang('header_select_city');
                $type_location = isset($data['type_location']) ? $data['type_location'] : 1;
                $cities = array();
                if(!empty($data['cities'])) {
                    foreach($data['cities'] as $city) {
                        $selected = $default_city == $city->id || $location == $city->id ? ' selected': '';
                        $cities[] = <<<EOL
            <option value="{$city->id}"{$selected}>{$city->name}</option>
EOL;
                    }
                }
                $options = implode(PHP_EOL, $cities);
                $output = <<<EOL
    <input type="hidden" name="search-country" id="search-country" value="{$deafult_country}" />
    <input type="hidden" name="search-state" id="search-state" value="{$default_state}" />
    <input type="hidden" name="tl" id="tl" value="{$type_location}">
    <select id="select-city" name="l" placeholder="{$placeholder}" class="form-control{$field_class}">
        <option value="">{$placeholder}</option>
        {$options}
    </select>
EOL;
                break;
            case 'locality':
                $placeholder = lang('header_select_locality');
                $locality = isset($data['locality']) ? $data['locality'] : 0;
                $selected = $locality == 0 ? ' selected': '';
                $localities = array();
                if(!empty($data['localities'])) {
                    foreach($data['localities'] as $area) {
                        $selected = $locality == $area->id ? ' selected': '';
                        $localities[] = <<<EOL
        <option value="{$area->id}"{$selected}>{$area->name}</option>
EOL;
                    }
                }
                $options = implode(PHP_EOL, $localities);
                $output = <<<EOL
    <select id="select-locality" name="lt" placeholder="{$placeholder}" class="form-control{$field_class}">
		<option value="0"{$selected}>{$placeholder}</option>
		{$options}
    </select>
EOL;
                break;
            case 'location':
                $placeholder = lang('header_location_term');
                $location = isset($data['location']) ? $data['location'] : '';
                $type_location = isset($data['type_location']) ? $data['type_location'] : 1;
                $output = <<<EOL
                                <input class="form-control typeahead-location{$field_class}" name="l" autocomplete="off" type="text" value="{$location}" placeholder="{$placeholder}">
                                <input type="hidden" name="tl" id="tl" value="{$type_location}">
EOL;
                break;
            case 'category':
                $placeholder = lang('header_select_category');
                $selected_category = isset($data['category']) ? $data['category'] : 0;
                $selected = $selected_category == 0 ? ' selected': '';
                $categories = '';
                if(!empty($data['categories'])) {
                    foreach($data['categories'] as $category) {
                        $selected = $selected_category == $category->id ? ' selected': '';
                        $categories .= <<<EOL

                                    <option value="{$category->id}"{$selected}>{$category->name}</option>
EOL;
                    }
                }
                $options = $categories;
                // c is a reserved keyword in Codeigniter pagination library
                // remove it from the goclixy/ci3/libraries/Pagination.php file at line number 432 to use it.
                $output = <<<EOL
<select id="select-category" name="cg" placeholder="{$placeholder}" class="form-control{$field_class}">
                                    <option value="0"{$selected}>{$placeholder}</option>{$options}    		      
                                </select>

EOL;
                break;
            case 'query':
                $placeholder = lang('header_search_term');
                $keyword = isset($data['keyword']) ? $data['keyword'] : '';
                $field = <<<EOL

                                <input class="typeahead form-control typeahead-search{$field_class}" name="q" autocomplete="off" type="text" value="{$keyword}" placeholder="{$placeholder}">
EOL;
                if(isset($data['group'])) {
                    $output = <<<EOL
        <div class="input-group input-group-btn">
            {$field}
			<div class="input-group-append">
				<button type="submit" class="btn btn-info btn-theme-secondary" name="search">
					<i class="fas fa-search"></i>
				</button>
			</div>
		</div>
EOL;
                } else {
                    $output = $field;
                }
                break;
            case 'submit':
                $output = <<<EOL

                                <button type="submit" class="btn btn-info btn-theme-secondary{$field_class}" name="search">
                					<i class="fas fa-search"></i>
                				</button>
EOL;
                break;
                
        }
        return $output;
    }
}

if ( ! function_exists('theme_block')) {
    /**
     * Common Theme Blocks with Links
     * @param array $data
     * @return string
     */
    function theme_block($data = array()) {
        $output = '';
        $element = isset($data['block']) ? $data['block'] : '';
        switch($element) {
            case 'social':
                $links = array('linkedin', 'facebook', 'twitter', 'googleplus', 'youtube', 'instagram');
                $social_links = '';
                foreach($links as $link) {                    
                    $name = $link . '_url';
                    if(settings_item('site.' . $name)) {
                        $value = settings_item('site.' . $name);
                        $title = lang('label_' . $name);
                        $icon = $link;
                        if($link == 'googleplus') {
                            $icon = 'google-plus';
                            $link = 'gplus';
                        }
                        if($link == 'youtube') {
                            $link = 'ytube';
                        }
                        $social_links .= <<<EOL

                    <li class="{$link}">
                        <a href="{$value}" target="_blank" data-toggle="tooltip" data-placement="bottom" title="{$title}">
                            <i class="fab fa-{$icon}"></i>
                        </a>
                    </li>
EOL;
                    }
                }
                $output = <<<EOL
        <ul class="social nav-link">{$social_links}
                </ul>

EOL;
                break;
            case 'language':
                if(settings_item('site.multi_language') == 1) {
                    $CI =& get_instance();
                    $csrf_token_name = $CI->security->get_csrf_token_name();
                    $csrf_hash = $CI->security->get_csrf_hash();
                    if(!empty($data['languages'])) {
                        $lang = isset($data['language']) ? $data['language'] : '';
                        $languages = '';
                        $form_class = '';
                        if(isset($data['dropdown'])) {
                            $form_class = ' mt-1 mb-1 mr-2';
                            $languages .= '<div class="form-group m-0">' . PHP_EOL;
                            $languages .= '<select name="site_languages" id="site_languages">' . PHP_EOL;
                            foreach($data['languages'] as $language) {
                                $languages .= '<option value="' . $language->label . '~' . $language->flag . '"' . (isset($lang) && ($lang == $language->label) ? ' selected': '') . '><i class="famfamfam-flag-' . $language->flag . '"></i> ' . $language->name . '</option>' . PHP_EOL;
                            }
                            $languages .= '</select>'  . PHP_EOL . '</div>';
                        } else {
                            $languages .= '<div class="input-btn-group language-selector">' . PHP_EOL;
                            foreach($data['languages'] as $language) {
                                $link_text = $language->flag;
                                if(isset($data['btn'])) { // Modero Theme
                                    $class = ($lang == $language->label ? ' btn-theme-primary' : ' btn-theme-secondary');
                                } else {
                                    $class = ($lang == $language->label ? ' active' : '');
                                    $link_text = '<i class="famfamfam-flag-' . $link_text . '"></i>';
                                }
                                $languages .= <<<EOL
                                
                <a class="btn{$class}" id="{$language->label}">
                    {$link_text}
                </a>
EOL;
                            }
                            $languages .= '</div>';
                        }
                        $output = <<<EOL
        <form class="form-horizontal{$form_class}" id="language-change" role="form" accept-charset="utf-8" method="post">
            <input type="hidden" id="csrf_token" name="{$csrf_token_name}" value="{$csrf_hash}" />
				{$languages}
            <input type="hidden" name="site_language" id="site_language" value="" />
        </form>
        
EOL;
                    }
                }
                break;
            case 'brand':               
                $CI =& get_instance();
                $site_url = site_url();
                $country_data = settings_item('site.default_country');
                $country_info = json_decode($country_data, true);
                if(!empty($country_info)) {
                    $data_parser = $country_info;
                } else {
                    $data_parser = array('country' => $country_data);
                }
                $keywords = $CI->parser->parse_string(settings_item('site.meta_keywords'), $data_parser, true);
                $title = $CI->parser->parse_string(settings_item('site.meta_title'), $data_parser, true);
                $site_title = $CI->parser->parse_string(settings_item('site.title'), $data_parser, true);
                $data['logo'] = site_logo(null, !empty($data['key']) ? $data['key'] : 'logo');
                $brand_style = !empty($data['logo']) ? ' style="background: rgba(0, 0, 0, 0) url(\'' . $data['logo'] . '\') no-repeat scroll 0 0;"' : '';
                if(settings_item('seo.use_h1') == 1) {
                    $link = <<<EOL
    <h1><a href="{$site_url}" title="{$keywords}">{$title}</a></h1>
EOL;
                } else {
                    $link = <<<EOL
    <a href="{$site_url}" title="{$keywords}">{$title}</a>
EOL;
                }
                if(settings_item('site.logo_with_title')) { // display title below the logo
                    $output = <<<EOL
<div id="brand" data-content="{$site_title}" class="navbar-brand logo-with-title"{$brand_style}>
    {$link}
    </div>
    
EOL;
                } else {
                $output = <<<EOL
<div id="brand" class="navbar-brand"{$brand_style}>
    {$link}
    </div>

EOL;
                }
                break;                
            case 'loggedin_user':
                $CI =& get_instance();
                if($CI->auth->is_logged_in() === TRUE) {
                    $user_info = (array) $CI->session->userdata('user_info');
                    if(!empty($user_info['email'])) {
                        $class = isset($data['class']) ? ' class="' . $data['class'] . '"' : '';
                        $icon = !empty($data['icon']) ? '<i class="fas ' . $data['icon'] . ' fa-fr"></i>' : '';
                        $output = '<a href="' . gc_url_format('members') . '"' . $class . ' title="' . lang('header_my_account') . '">' . $icon . $user_info['display_name'] . '</a>';
                    }
                }
                break;
            case 'site_user':
                $CI =& get_instance();
                $class = isset($data['class']) ? ' class="' . $data['class'] . '"' : '';
                $icon = !empty($data['icon']) ? '<i class="fas ' . $data['icon'] . ' fa-fr"></i>' : '';
                if($CI->auth->is_logged_in() === TRUE && $CI->uri->segment(1) == gc_uri_string('members')) {
                    $output = '<a href="' . site_url() . '"' . $class . '>' . lang('label_take_me_home') . '</a>';
                } else if($CI->auth->is_logged_in()) {
                    $output = '<a href="' . gc_url_format('members') . '"' . $class . ' title="' . lang('header_my_account') . '">' . $icon . lang('header_my_account') . '</a>';
                } else {
                    if(isset($data['modal'])) {
                        $output = '<a role="button"' . $class . ' data-toggle="modal" data-target="#signinModal">' . lang('header_my_account') . '</a>';
                    } else {
                        $output = '<a href="' . gc_url_format('login') . '"' . $class . ' title="' . lang('header_my_account') . '">' . $icon . lang('header_my_account') . '</a>';
                    }                    
                }
                break;
            case 'location':
                $display_link = false;
                if(settings_item('adv.search_location') == 1 || (settings_item('adv.search_blocks') == 1 && settings_item('adv.search_location') == 3)) {
                    $display_link = true;
                }
                if($display_link) {
                    $CI =& get_instance();
                    $title = lang('header_change_location');
                    if(settings_item('adv.search_location') == 3 && settings_item('adv.search_blocks') == 1 && $CI->session->userdata('pre_state')) {
                        $title = $CI->session->userdata('pre_state');
                    } else if($CI->session->userdata('country_info')) {
                        $location = $CI->session->userdata('country_info');
                        if(isset($location['name'])) {
                            $title = $location['name'];
                        }
                    }
                    $class = isset($data['class']) ? ' class="' . $data['class'] . '"' : '';
                    $icon = isset($data['icon']) ? '<i class="fas ' . $data['icon'] . ' fa-fl"></i>' : '<i class="fas fa-chevron-right fa-fl"></i>';               
                    $output = <<<EOL
        <a {$class} role="button" id="modal-launcher" data-toggle="modal" data-target="#selectLocation">
            {$title}{$icon}
        </a>
EOL;
                }
                break;
            case 'quick_links':
                $category_url = gc_url_format('categories');
                $category_title = lang('home_category_browse');
                $location_url = gc_url_format('locations');
                $location_title = lang('home_location_browse');
                $requirement_url = gc_url_format('requirements');
                $requirement_title = lang('post_your_requirements');
                $icon = !empty($data['icon']) ? '<i class="fas ' . $data['icon'] . '"></i>' : '';
                $countries_list_link = '';
                if(settings_item('site.international')):
                    $countries_list_link = '<li><a href="' . site_url('countries') . '" title="' . lang('live_sites') . '">' . $icon . lang('live_sites') . '</a></li>';
                endif;
                $header = '<h5>' . lang('footer_quick_links') . '</h5>';
                $output = <<<EOL
        {$header}
        <ul class="list-unstyled footer-menu">						
		     <li><a href="{$category_url}" title="{$category_title}">{$icon}{$category_title}</a></li>
			 <li><a href="{$location_url}" title="{$location_title}">{$icon}{$location_title}</a></li>
			 <li><a href="{$requirement_url}" title="{$requirement_title}">{$icon}{$requirement_title}</a></li>{$countries_list_link}
		</ul>
EOL;
                break;
            case 'module_links':
                $config_modules = gc_modules_only();
                $config_modules[] = 'videos';
                $links = '';
                $icon = isset($data['icon']) ? '<i class="fas ' . $data['icon'] . '"></i>' : '<i class="fas fa-angle-double-right fa-fr"></i>';
                $header = '<h5>' . lang('footer_useful_links') . '</h5>';
                foreach($config_modules as $config_module) {
                    if(gc_module_settings($config_module) == 1) {
                        $url = gc_url_format($config_module);
                        $title = lang($config_module);
                        $links .= <<<EOL

                <li><a href="{$url}" title="{$title}">{$icon}{$title}</a></li>
EOL;
                    }
                }
                $output = <<<EOL
        {$header}
        <ul class="list-unstyled footer-menu">
            {$links}
		</ul>
EOL;
                break;
            case 'site_email':
                if(settings_item('site.display_email') == 1) {
                    $email = settings_item('site.system_email');
                    $output = <<<EOL
                <a href="mailto:{$email}" class="nav-link">
                    <i class="fas fa-envelope fa-fr"></i>
                    <strong>{$email}</strong>
                </a>
EOL;
                }
                break;
            case 'site_phone':
                if(settings_item('site.phone_number')) {
                    $phone = settings_item('site.phone_number');
                    $title = lang('header_call_us');
                    $output = <<<EOL
            <a class="nav-link" role="button">
                <i class="fas fa-mobile-alt mr-1"></i>
                <span class="d-none d-sm-inline-block">{$title}: </span>
                <strong>{$phone}</strong>
            </a>
EOL;
                }
                break;
            case 'copyright':
                if(settings_item('site.copyright_info') == 1) {
                    $output = <<<EOL
        <h5>Coded with <span class="fas fa-heart"></span>by GoClixy</h5>
        <p>
            <a href="https://www.goclixy.com" target="_blank" title="GoClixy Business Directory Script">
				GoClixy Business Directory Script
			</a>
        </p>

EOL;
                }
                break;
    		
        }
        if(!empty($data['container'])) {
            $output = emmet($data['container'], $output);
        }
        unset($data);
        return $output;
    }
}

if ( ! function_exists('fields_custom'))
{
    /**
     * Dynamic Fields with Options
     *
     * @param string $field_name - Name of the Field
     * @param array $field_options - Field Options
     * @param array $attributes - Saved Attributes
     * @param string $separator - will be replaced with the field Index to simplify the save operations
     * @return string
     */
    function fields_custom($data = array()) {
        // $field_name = null, $field_options = array(), $attributes = array(), $saved_options = array(), $separator = '!~~~!'
        $field_name = $data['name']; // Name of the Field
        $input_fields = $data['fields']; // Field Options
        $attributes = isset($data['tree']) ? $data['tree'] : array(); // All attributes with values
        $saved_options = isset($data['value']) ? $data['value'] : array(); // Saved Attributes
        $unique_select = isset($data['unique']) && $data['unique'] ? true : 0; // Allow user to select only one option
        $limit = isset($data['limit']) ? $data['limit'] : 50;
        $separator = '!~~~!'; // the separator will be replaced with the field Index to simplify the save operations
        $separator_id = '_' . $separator . '_';
        
        // Prepare fields
        $field_options = array();
        $CI =& get_instance();
        if(!empty($input_fields)) {
            foreach($input_fields as $input_field) {
                $input_field_id = $field_name . $separator_id . $input_field['name'];
                $extra_field_params = $input_field['type'] == 'textarea' ? array('id' => $input_field_id, 'rows' => 2) : array('id' => $input_field_id);
                $field_content = array(
                    'name' => $input_field['name'],
                    'type' => $input_field['type'],
                    'label' => !empty($input_field['label_lang']) ? lang($input_field['label']) : $input_field['label'],
                    'field' => array(
                        'name' =>  $field_name . '[' . $separator . '][' . $input_field['name'] . ']',
                        'field_only' => true,
                        'field_class' => isset($input_field['class']) ? $input_field['class'] : '',
                        'extra' => $extra_field_params,
                        'default_value' => isset($input_field['default_value']) ? $input_field['default_value'] : '',
                        'label' => $input_field['type'] == 'checkbox' ? '' : $input_field['label']
                    )
                );
                if(isset($input_field['options'])) {
                    $field_content['field']['options'] = $input_field['options'];
                } else if(!empty($input_field['options_table'])) {
                    $select = '';
                    $sort_by = 'name';
                    $where = 'status = 1 AND deleted = 0';
                    switch($input_field['options_table']) {
                        case 'currencies':
                            $select = 'id, code AS name';
                            $sort_by = 'code';
                            break;
                        case 'options':
                            if(!empty($input_field['relation_value'])) {
                                $where .= ' AND type_id = ' . (int) $input_field['relation_value'];
                            }
                            $sort_by = 'priority';
                            break;
                    }
                    $options_data = $CI->gcache->get($input_field['options_table'], array('select' => $select, 'where' => $where, 'order_by' => $sort_by, 'result' => 1));
                    $data_options = array('' => lang('placeholder_select'));
                    if(!empty($options_data)) {
                        foreach($options_data as $option_data) {
                            $data_options[$option_data->id] = !empty($option_data->name_lang) ? lang($option_data->name) : $option_data->name;
                        }
                    }
                    $field_content['field']['options'] = $data_options;
                }
                if(isset($input_field['field_class'])) {
                    $field_content['field']['field_class'] = $input_field['field_class'];
                }
                if(isset($input_field['readonly'])) {
                    $field_content['field']['extra']['readonly'] = $input_field['readonly'];
                }
                $field_options[] = $field_content;
            }
        }
        
        // Post Handling
        if(isset($_POST[$field_name])) {
            $saved_options = $_POST[$field_name];
        }
        
        // Give each row input fields a unique index to simplify save operation
        $fi = 1;
        
        // Prepare fields for jQuery function
        $fields_table = '<tr>' . PHP_EOL;
        foreach($field_options as $field_option) {
            $field = gc_form_element($field_option['type'], $field_option['field'], (isset($field_option['field']['default_value']) ? $field_option['field']['default_value'] : ''), $field_option['label']); // set default value
            $fields_table .= <<<EOL
            
                <td>
                    {$field}                </td>
EOL;
        }
        $fields_table .= <<<EOL
        
                <td><a href="#" class="remove_field"><i class="fas fa-minus"></i></a></td>
            </tr>
EOL;
        
        $field_counts = count($field_options) + 1;
        $btn_add = lang('add');
        $btn_action = lang('label_action');
        
        // Table Headings
        $th = '';
        foreach($field_options as $field_option) {
            $field_label = $field_option['label'];
            if($field_option['type'] == 'hidden') { // hidden field
                $th .= "<td></td>";
            } else {
                $th .= <<<EOL
                
                <td>{$field_label}</td>
EOL;
            }
        }
        
        // Table Body with input fields
        $tbody = '';
        if(!empty($saved_options)) {
            if(is_string($saved_options)) {
                $saved_options = json_decode($saved_options);
            }
            foreach($saved_options as $saved_option) {
                $saved_option = (array) $saved_option;
                $fi++;
                $tbody .= '<tr>' . PHP_EOL;
                foreach($field_options as $field_option) {
                    $field_saved_value = !empty($saved_option[$field_option['name']]) ? $saved_option[$field_option['name']] : (isset($field_option['field']['default_value']) ? $field_option['field']['default_value'] : '');
                    
                    // Attribute values are hardcoded
                    // Retreive selected name attribute values
                    if($field_option['name'] == 'value' && isset($field_option['field']['options'])) {
                        $field_option['field']['options'] = $attributes[$saved_option['name']];
                    }
                    
                    $tbody .= '<td>' . str_replace($separator, $fi, gc_form_element($field_option['type'], $field_option['field'], $field_saved_value)) . '</td>' . PHP_EOL;
                }
                $tbody .= '<td><a href="#" class="remove_field"><i class="fas fa-minus"></i></a></td>' . PHP_EOL;
                $tbody .= '</tr>';
            }
        } else {
            $tbody .= str_replace($separator, $fi, $fields_table);
        }
        
        $json_fields = json_encode($fields_table);
        $json_attributes = json_encode($attributes);
        $no_more_attribute = lang('no_more_attributes');
        $field_id = $field_name;
        $output = <<<EOL
    <table class="table table-striped table-hover table-options" id="{$field_id}">
        <tbody id="tablevalues{$field_id}">
            <tr>
                <td colspan="{$field_counts}">
                    <button class="btn btn-primary add-option" id="add{$field_id}">{$btn_add}</button>
		         </td>
	       </tr>
           <tr>{$th}
                <td>{$btn_action}</td>
           </tr>
           {$tbody}
        </tbody>
    </table>
<script src="https://cdnjs.cloudflare.com/ajax/libs/jquery/3.3.1/jquery.slim.min.js"></script>
<script type="text/javascript">
$( document ).ready(function() {
    var tableId         = '{$field_id}';
    var wrapper         = $('#' + tableId); //Fields wrapper
    var add_button      = $("#add" + tableId); //Add button ID
	var field_index		= {$fi};
	var fields{$field_id} = {$json_fields};
    var max_fields      = {$limit}; //maximum input boxes allowed
    var x = 1; //initlal text box count
	add_button.click(function(e) { //on add input button click
        e.preventDefault();
        if(x < max_fields) { //max input box allowed
            x++; //text box increment
            field_index++;
            var newFields = fields{$field_id}.replace(/{$separator}/g, field_index);
            
            // Allow user to select only one variant at a time
            var uniqueSelect = {$unique_select};
            if(uniqueSelect) {
                // Value of the previous selected attribute fields
                var selectedValues = [];
                
                // All attributes
                var attributes = {$json_attributes};
                
                var compAttr = [];
                $.each(attributes, function( key, value ) {
                    compAttr.push(key);
                });
                
                // Go through each field having class attr
                $(".attr").each(function() {
                
                    // Get value of the selected options
                    selectedValues.push($(this).val());
                    compAttr.splice($.inArray($(this).val(), compAttr), 1);
                });
                
                // if it is the last attribute do not append field
                if(compAttr.length == 0) {
                    alert('{$no_more_attribute}'); // Display message to the user
                    return false;
                }
            }
            
            // fields.replace(field_separator/g, field_index)
            $(wrapper).append(newFields); //add input box
            
            // Allow user to select only one variant at a time
            if(uniqueSelect) {
                // Newly added field ID
                var currentField = 'variants_' + field_index;
                
                // Remove all previous attributes from the last added field
                $.each( selectedValues, function( key, value ) {
                    $('#' + currentField + '_name').find('option[value=' + value + ']').remove();
                });
                
                // Selected attribute
                var selectedValue = $('#' + currentField + '_name').val();
                
                // Also reset the value field options
                // get new options from the attributes field
                $('#' + currentField + '_value').html('');
                $.each(attributes, function( key, value ) {
                    if(key == selectedValue) {
                        $.each(value, function( option_id, option_value ) {
                            $('#' + currentField + '_value').append($('<option>', {
                                value: option_id,
                                text : option_value
                            }));
                        });
                    }
                });
            }
        }
    });
    
    wrapper.on("click",".remove_field", function(e){ //user click on remove text
        e.preventDefault();
        $(this).closest('tr').remove();x--;
    })
    
    // on dropdown change, populate corresponding values
    /*$(".attr").each(function(index) {
    
    });*/
    
    // Event binding to elements that are not in the DOM on initial page load will not work.
    // Therefore, the events are binded to the parent table
    wrapper.on("change", '.attr', function() {
        var attrId = $(this).val();
    	var data = {$json_attributes};
    	var options = '';
    	$.each(data, function (index, item) {
    		if(attrId == index) {
    			$.each(item, function (id, value) {
    				options += '<option value="' + id + '">' + value + '</option>';
    			});
    		}
    	});
    	$(this).closest("td").next().find(":input").html(options);
    });
});
</script>
EOL;
           return $output;
    }
}

/************************************
 * START OF ECOMMERCE SECTION
 ************************************/
if ( ! function_exists('cart_load')) {
    /**
     * Products Cart Load
     * @param array $data
     * @return string
     */
    function cart_load($data = array(), $mobile = false, $link_class = "btn btn-primary btn-theme-secondary rounded-circle") {
        $CI =& get_instance();
        $output = '';
        $CI->lang->load('orders/orders');
        $CI->load->model('orders/orders_model');
        $cart_view = lang('cart_view');
        $cart_shopping = lang('cart_shopping');
        $close = lang('form_close');
        $count = count($CI->orders_model->checkout_params(0));
        if($mobile) {
            $checkout_url = gc_url_format('members/checkout');
            $output = <<<EOL
<a href="{$checkout_url}" class="{$link_class}" title="{$cart_view}"><span class="cart-items-count">{$count}</span> <i class="fas fa-cart-plus"></i></a>
EOL;
        } else {
            $output = <<<EOL
<a href="#" class="cart-box {$link_class}" title="{$cart_view}"><span class="cart-items-count">{$count}</span> <i class="fas fa-cart-plus"></i></a>
<div class="shopping-cart-box">
<a href="#" class="close-shopping-cart-box">{$close}</a>
<h3>{$cart_shopping}</h3>
    <div class="shopping-cart-results"></div>
</div>
EOL;
        }
        return $output;
    }
}

if(!function_exists('cart_html_tpl')) {
    /**
     * Wrap contents within HTML tags
     * @param string $content
     * @return string
     */
    function cart_html_tpl($content, $has_header = true) {   
        $header = '';
        if($has_header) {
            // $logo = gc_image_url(settings_item('site.logo'), false);
			$logo_site = settings_item('site.logo');
			$regexp =  "/((?:https?\:\/\/|www\.)(?:[-a-z0-9]+\.)*[-a-z0-9]+.*)/i";
			if(preg_match($regexp, $logo_site)){
				$logo = $logo_site;
			} else {
				$logo = file_get_contents(FCPATH . 'assets/images/'. $logo_site);
				$logo = base64_encode($logo);
			}
            $alt = settings_item('site.title');
            $header = <<<EOL
<header><img src="{$logo}" alt="{$alt}"/><hr></header>
<footer><hr></footer>
EOL;
        }
        $html_content = <<<EOL
<html>
<head>
  <style>
    @page { margin: 100px 25px; }
    header { position: fixed; top: -60px; left: 0px; right: 0px; height: 50px; }
    footer { position: fixed; bottom: -60px; left: 0px; right: 0px; height: 50px; }
  </style>
</head>
<body>{$header}
  <main>
    {$content}
  </main>
</body>
</html>
EOL;
        return $html_content;
    }
}

if(!function_exists('cart_item')) {
    /**
     * Admin/PDF/View/Checkout - Cart Item Layout
     * @return string
     */
    function cart_item($item, $return = null, $module = 'products') {
        $output = '';
        $item_info = json_decode($item);
        if(!empty($item_info)) {
            $item_url = site_url();
            if(!empty($item_info->slug)) {
                $item_url = gc_url_format($module, $item_info->slug, $item_info->id) . (!empty($item_info->variant_id) ? '?sku=' . $item_info->variant_id : '');
            }
            $output = '';
            $item_title = strip_tags(isset($item_info->title) ? $item_info->title : $item_info->name);
            $item_image = !empty($item_info->cover) ? gc_image_url($item_info->cover, false) : gc_site_image('no-image');
            $item_attributes = !empty($item_info->attributes) ? ' (<small>' . implode('|', (array) $item_info->attributes) . '</small>)' : '';
            if(empty($return)) {
                $output = <<<EOL
    <div class="row">
        <div class="col-sm-4">
      		<a href="{$item_url}" title="{$item_title}" target="_blank">
      			<img src="{$item_image}" class="img-thumbnail" alt="{$item_title}" />
      		</a>
      	</div>
      	<div class="col-sm-8">
      		<a href="{$item_url}" title="{$item_title}" target="_blank">
                {$item_title}{$item_attributes}
      		</a>
      	</div>
    </div>
EOL;
            }
        }
        return $output;
    }
}

if(!function_exists('cart_layout')) {
    /**
     * Admin/PDF/View/Checkout - Cart Items Layout
     * @param array $data -> items|order_type|is_cart|amount|currency|pyg_module
     * @return string
     */
    function cart_layout($data = array()) {
        $output = '';
        // to display the amount in purchased currency
        $currency = isset($data['currency']) ? array('currency' => $data['currency']) : array();
        $role = isset($data['role']) ? $data['role'] : 'checkout';
        
        // PDF and EMAIL Styling
        $table_style = '';
        $th_style = '';
        $td_style = '';
        $line_through = '';
        $image_style = '';
        if(isset($data['inline_style'])) {
            $table_style = ' style="width: 100%; border: 1px solid #444; border-collapse: collapse; padding: 10px;"';
            $th_style = ' style="font-size: 14px; background-color: #ddd; color: #444; padding: 5px; border: 1px solid #444; text-align: center;"';
            $td_style = ' style="border: 1px solid #444; padding: 10px; text-align: center;"';
            $line_through = ' style="text-decoration-line: line-through; -webkit-text-decoration-line: line-through;"';
            $image_style = ' style="max-width: 250px; height: auto;"';
        }
        
        $items = $data['items'];
        $style_width = ' style="width: 150px;"';
        if(empty($items)) {
            return $output;
        }
        // Variables
        $label_item = lang('label_item');
        $label_image = lang('label_image');
        $label_qty = lang('label_qty');
        $label_price = lang('label_price');
        $header = <<<EOL
        
<div class="table-responsive">
<table class="table table-hover table-bordered shopping-cart"{$table_style}>
    <thead class="text-muted thead-light">
        <tr>
            <th scope="col"{$th_style} class="w-25">{$label_image}</th>
            <th scope="col"{$th_style}>{$label_item}</th>
            <th scope="col"{$th_style}>{$label_qty}</th>
            <th scope="col" class="text-center"{$th_style}>{$label_price}</th>
        </tr>
    </thead>
    <tbody>
EOL;
        // Admin Panel - Display shipment information with each item
        $shipments = isset($data['shipments']) ? $data['shipments'] : array();
        $ordered_items = isset($data['ordered_items']) ? $data['ordered_items'] : array();
        $cart_item = '';
        
        // Optional Purchased Items
        $purchased_items = array();
        foreach($items as $purchased_item_id => $purchased_item) {
            $purchased_item = (object) $purchased_item;
            $purchased_item_id = $role != 'checkout' && isset($purchased_item->row_id) ? $purchased_item->row_id . '-' . $purchased_item->variant_id . '-' . $purchased_item->relation_item_id : $purchased_item_id;
            $purchased_items[$purchased_item_id] = array(
                'relation_item_id' => isset($purchased_item->relation_item_id) ? $purchased_item->relation_item_id : 0,
                'qty' => empty($purchased_item->qty) ? 1 : $purchased_item->qty
            );
        }
        
        foreach($items as $item_id => $item) {
            $item = (array) $item;
            $total_savings = 0;
            // the user might have added items from the member area
            // display relatiion_item name with the optional item
            if($role != 'admin' && !empty($item['relation_item_id'])) {
                continue;
            }
            $relation_item = -1;
            if(!empty($item['relation_item_id'])) {
                $relation_item = $item['relation_item_id'] . '-' . (!empty($item['relation_item_variant_id']) ? $item['relation_item_variant_id'] : 0) . '-0';
            }
            if(!empty($items[$relation_item])) {
                continue;
            }
            $item['item'] = is_string($item['item']) ? json_decode($item['item'], true) : $item['item']; // saved data is in json format
            $price = $item['price'];
            if(!empty($item['offer_price'])) {
                $price = $item['offer_price'];
            }
            $item_stock = isset($item['stock']) ? $item['stock'] : 1;
            $item_stock = $item_stock > 10 ? 10 : $item_stock;
            $module_id = $item['module_id'];
            $item_cover = !empty($item['item']['cover']) ? gc_image_url($item['item']['cover'], false) : gc_site_image('no-image');
            $item_title = !empty($item['item']['name']) ? $item['item']['name'] : (!empty($item['item']['title']) ? $item['item']['title'] : '');
            if($module_id != 2) { // do not display module description in case of products
                $item_description = $item['item']['description'];
            } else {
                $item_description = '';
            }
            $item_saving = '';
            if(!empty($item['savings'])) {
                $additional_discount = 0;
                $discount_info = '';
                $savings = $item['savings'];
                $total_savings += $savings;
                if(!empty($item['item']['additional_discount'])) {
                    $total_savings += $item['item']['additional_discount'];
                    // $savings = $savings - $item['item']['additional_discount']; // creating issues with pricing rules
                    $additional_discount = '<span class="text-secondary small d-block additional_discount_' . $item_id . '">' . lang('additional_discount_' . $item['item']['additional_discount_type']) . ' <span id="additional_discount_' . $item_id . '">' . currency_format($item['item']['additional_discount'], $currency) . '</span></span>';
                }
                
            }
            // savings and discounts
            $item_saving .= '<span class="text-secondary small checkout-savings' . (!empty($savings) ? ' d-block' : '') . '"' . (empty($savings) ? ' style="display:none;"' : '') . '>' . lang('cart_saving_offer') . ' <span id="savings_' . $item_id . '">' . (!empty($savings) ? currency_format($savings, $currency) : 0) . '</span></span>';
            $item_saving .= !empty($additional_discount) ? $additional_discount : '';
            if(!empty($item['item']['rules_discount'])) {
                $item_saving .= '<span class="text-primary small rules-discount' . (!empty($item['item']['rules_discount']) ? ' d-block' : '') . '"' . (empty($item['item']['rules_discount']) ? ' style="display: none;"' : '') . '>' . lang('rules_discount') . ' <span id="rules_discount_' . $item_id . '">' . (!empty($item['item']['rules_discount']) ? currency_format($item['item']['rules_discount'], $currency) : currency_format(0, $currency)) . '</span></span>';
            }
            if(!empty($item['item']['rules_discount'])) {
                $total_savings += $item['item']['rules_discount'];
            }
            if(!empty($item['discount'])) {
                $total_savings += $item['discount'];
            }
            $item_discount = '<span class="item-discount text-info small item-discount_' . $item_id. '"' . ($item['discount'] == 0 ? ' style="display:none;"' : ' style="d-block"') . '>' . lang('cart_discount_item') . ' <span id="discount_' . $item_id . '">' . currency_format($item['discount'], $currency) . '</span></span>';
            // Available Items in stock
            $item_limited_stock = '';
            if(($module_id == 8 || $module_id == 39) && $item_stock <= 5 && $role == 'checkout') { // is_view - Admin Panel View Order
                $item_limited_stock = '<span class="text-danger small font-weight-bold">' . sprintf(lang('cart_limited_stock'), $item_stock) . '</span>';
            }
            
            // Quantity and Price calculations
            $item_qty = isset($item['qty']) ? $item['qty'] : 1;
            $item_qty_price = $item_qty * $price;
            $item_price = currency_format($item_qty * $price, $currency);
            $item_subtotal = '<span id="subtotal_' . $item_id . '">' . currency_format($item['subtotal'], $currency) . '</span>';
            
            // Item Savings
            $layout_savings = '';
            $label_item_price = sprintf(lang('cart_item_price'), '<span id="unit_price_' . $item_id . '">' . currency_format($price, $currency) . '</span>');
            if(!empty($item['savings']) || !empty($item['item']['rules_discount'])) {
                // isset($item['item_qty_price']) ? $item['item_qty_price'] : 
                $item['item_qty_price'] = $item['price'] * $item_qty; // admin/member view page
                $item_actual_price = '<span id="item_qty_price_' . $item_id . '">' . currency_format($item['item_qty_price'], $currency) . '</span>';
                
                // savings item
                $savings_on_item = $item['savings'];
                if(!empty($item['item']['rules_discount'])) {
                    $savings_on_item += $item['item']['rules_discount'];
                }
                
                $savings_in_percentage = decimal_convert(($savings_on_item * 100)/($item_qty *  $item['price'])) . '%';
                $item_off = sprintf(lang('cart_item_off'), $savings_in_percentage);
                $layout_savings = <<<EOL
                    <span id="item_qty_price_{$item_id}" class="text-muted line-through"{$line_through}><strike>{$item_actual_price}</strike></span>
    			    <span class="text-success font-weight-bold d-block">{$item_off}</span>
                    <small class="text-muted d-block small">({$label_item_price})</small>
EOL;
            } else if($module_id == 1) {
                $layout_savings = <<<EOL
                    <small class="text-muted d-block small">({$label_item_price})</small>
EOL;
            }
            
            // Shipment information
            $shipment_info = '';
            if(!empty($ordered_items)) {
                foreach($ordered_items as $ordered_item) {
                    if($ordered_item->row_id == $product['id'] && $ordered_item->variant_id == $product['variant_id']) {
                        if(!empty($shipments)) {
                            foreach($data['shipment_status'] as $key => $value) {
                                $shipment_exists = false;
                                foreach($shipments as $shipment) {
                                    $tooltip_text = '';
                                    $tooltip = '';
                                    if($shipment->order_item_id == $ordered_item->id) {
                                        if($shipment->status == $key) {
                                            $class = 'bg-success';
                                            $tooltip_text = '<em>' . gc_date_format($shipment->created_on) . '</em> <br>' . $shipment->description;
                                            $tooltip = ' data-toggle="tooltip" data-placement="bottom" data-html="true" title="' . $tooltip_text . '"';
                                            $shipment_exists = true;
                                        }
                                    }
                                }
                                if(!$shipment_exists) {
                                    $class = 'bg-secondary';
                                }
                                $shipment_info .= '<div class="progress-bar ' . $class . '" ' . $tooltip . ' style="width: 25%;">' . $value . '</div>' . PHP_EOL;
                            }
                        }
                    }
                }
            }
            if(!empty($shipment_info)) {
                $shipment_info = <<<EOL
                
       <div class="progress">
            {$shipment_info}        </div>
EOL;
            }
            
            // Item Tax
            $tax = !empty($item['tax']) ? $item['tax'] : 0;
            $tax_percent = !empty($item['tax_percent']) ? $item['tax_percent'] : 0;
            $tax_inclusive = !empty($item['tax_inclusive']) ? $item['tax_inclusive'] : 0;
            $item_tax = '';
            if(!empty($tax)) {
                $item_tax = '<small class="text-warning d-block">' . sprintf(lang('cart_item_tax'), '<span id="tax_' . $item_id . '">' . currency_format($tax, $currency). '</span><span class="ml-1">(<span id="tax_percent_' . $item_id . '">' . $tax_percent . '</span>% ' . lang('label_extra') . ')</span></small>');
            }
            // Item Attributes
            $item_attributes = '';
            if(!empty($item['item']['attributes'])) {
                $item['item']['attributes'] = (array) $item['item']['attributes'];
                $item_attributes = '<span class="d-block small">(' . implode(' | ', $item['item']['attributes']) . ')</span>';
            }
            
            $item_action = '';
            if($role != 'checkout') {
                $max_qty = $item_qty + 1;
            } else {
                $max_qty = !empty($item['max_qty']) ? $item['max_qty'] : 1;
            }
            $item_qty = $item_qty > $max_qty ? $max_qty : $item_qty; // The user can buy max specified items only
            if((!empty($item['max_qty']) && $item['max_qty'] > 1) || $item_qty > 1) {
                $options_qty = '';
                if($role == 'checkout') {
                    $options_limit = $max_qty <= $item_stock ? $max_qty : $item_stock;
                    for($i = 1; $i <= $options_limit; $i++) {
                        $option_selected = $item_qty == $i ? ' selected' : '';
                        $options_qty .= <<<EOL
                    <option value="{$i}"{$option_selected}>{$i}</option>
                    
EOL;
                    }
                    $item_qty = <<<EOL
<select data-id="{$item_id}" class="custom-select item-qty">
{$options_qty}                </select>
EOL;
                }
            }
            
            // wisthlist and remove buttons
            if($role == 'checkout') {
                $wishlist_btn = '';
                if($module_id == 1) {
                    $btn_class = isset($item['wishlist']) && $item['wishlist'] == 1 ? 'btn-success text-white' : 'btn-outline-success';
                    $label_wishlist = lang('save_to_wishlist');
                    $wishlist_btn = '<a role="button" class="btn ' . $btn_class . ' item-wishlist" data-toggle="tooltip" data-original-title="' . $label_wishlist . '"> <i class="fas fa-heart"></i></a>';
                }
                $label_remove = lang('gc_action_remove');
                $item_action = <<<EOL
            <div class="mt-2">
                	{$wishlist_btn}
                	<a role="button" class="btn btn-outline-danger item-remove" data-toggle="tooltip" data-original-title="{$label_remove}" data-code="{$item_id}" data-checkout="1"> <i class="fas fa-times"></i></a>
            	</div>
EOL;
            }
            $item_url = null;
            if(!empty($item['item']['slug'])) { // Saved products in database
                $item_url = gc_url_format(gc_module_name($module_id), $item['item']['slug'], $item['item']['id']);
                if(!empty($item['variant_id'])) {
                    $item_url .= '?sku=' . $item['variant_id'];
                }
            } else if(!empty($item['url'])) { // URL in the session variable
                $item_url =  $item['url'];
            }
            // style attribute is added for email responsiveness
            $item_img_url = !empty($item_url) ? '<a href="' . $item_url . '" class="img-box align-top" title="' . $item_title . '" target="_blank"><img src="' . $item_cover . '" class="rounded mx-auto d-block img-fluid"' . $image_style . '></a>' : '<div class="img-box align-top"><img src="' . $item_cover . '" class="rounded mx-auto d-block img-fluid"' . $image_style . ' /></div>';
            $item_title_url = !empty($item_url) ? '<a href="' . $item_url . '" title="' . $item_title . '" target="_blank">' . $item_title . $item_attributes . '</a>' : $item_title . $item_attributes;
            $item_status = isset($item['status']) && $role == 'checkout' ? $item['status'] : '';
            // $item_id = $item['row_id']; // Checkout Page - Unable to remove item
            // admin panel display item status with each item
            if(!empty($data['statuses']) && !empty($data['statuses'][$item['status_id']])) {
                $item_status_info = $data['statuses'][$item['status_id']];
                $item_status = !empty($item_status_info->name_lang) ? lang($item_status_info->name) : $item_status_info->name;
                $item_status = '<span class="badge badge-' . ($item['status_id'] == 2 ? 'success' : 'danger') . '">' . $item_status . '</span>';
            }
            
            // checkout type - Repeat Order/Renewal/Upgrade/Invoice Pay
            $checkout_type = '';
            if($role == 'admin') {
                if(!empty($item['ru_order_item_id'])) {
                    $checkout_type = anchor(site_url(SITE_AREA . '/financial/orders/view/' . $item['ru_order_id'] . '/0/1'), sprintf(lang('order_subscription_' . $item['subscription_type']), $item['ru_order_id'], $item['ru_order_item_id']), 'class="text-secondary" target="_blank"');
                }
            }
            // display purchases item id on the invoice for tracking purposes
            $order_settings = json_decode(settings_item('order.settings'), true);
            if($role != 'checkout' && !empty($order_settings['item_id_invoice'])) {
                $item_title_url .= '<span class="text-muted d-block small"><span class="font-weight-bold">' . lang('item_purchase_id') . ':</span> ' . $item['id'] . '</span>';
            }
            // Display Relation Item information
            if(!empty($item['item']['relation_item_info'])) {
                $display_relation_item_info = true; // display relation item info only if the parent is not present in the order
                if($role == 'admin') { // Admin Panel
                    $relation_item_key = array_search($item['relation_item_id'], array_column($items, 'row_id'));
                    if(!empty($relation_item_key)) {
                        $display_relation_item_info = false;
                    }
                } else {
                    if(isset($items[$item['relation_item_id'] . '-0-0'])) {
                        $display_relation_item_info = false;
                    }
                }
                if($display_relation_item_info) {
                    $item_title_url .= '<br /><span class="small badge badge-warning text-white">' . $item['item']['relation_item_info'] . '</span>';
                }
            }
            
            // total savings on this item
            $savings_total = '';
            if(!empty($total_savings)) {
                $savings_total = '<hr class="mt-1 mb-1" /><span class="text-success small d-block">' . lang('cart_saving_item') . ' ' . currency_format($total_savings, $currency) . '</span>';
            }
            
            // allow admin to edit the order item
            $edit_item_link = '';
            $CI =& get_instance();
            if($role == 'admin' && $CI->auth->has_permission('Order_Items.Financial.Edit') && !empty($item['id'])) {
                $edit_item_link = anchor(SITE_AREA . '/financial/orders/form/items/edit/' . $item['id'], '<i class="fas fa-edit fa-fw mr-1"></i>' . lang('edit'), 'class="btn btn-primary btn-xs"');
            }
            
            // Optional Items
            $optional_items = !empty($item['item']['optional_items']) ? theme_view('widgets/others/_checkout_optional_items_01', array('optional_items' => $item['item']['optional_items'], 'is_view' => ($role != 'checkout' ? true : false), 'item_id' => $item['item']['id'], 'purchased_items' => $purchased_items), true) : '';
            $cart_item .= <<<EOL
        <tr id="item_{$item_id}" data-id="{$item_id}" data-module="{$module_id}">
            <td{$td_style}>{$item_img_url}</td>
            <td{$td_style}>
                <div class="text-truncate">{$item_title_url}</div>
                {$item_description}
                {$optional_items}
        		{$item_saving}
        		{$item_discount}
        		{$savings_total}
                {$item_limited_stock}
                {$item_status}
                {$shipment_info}
                {$checkout_type}
                {$edit_item_link}
            </td>
            <td{$td_style}>
                {$item_qty}
            </td>
            <td class="text-center"{$td_style}>
                <div class="price-wrap">
                    <var class="price font-weight-bold h4">{$item_subtotal}</var>
    {$layout_savings}
                    {$item_tax}
                </div>
    {$item_action}
            </td>
        </tr>
EOL;
        }
        $output = <<<EOL
{$header}
{$cart_item}
    </tbody>
</table>
</div>

EOL;
return $output;
    }
}

if(!function_exists('cart_summary')) {
    /**
     * Admin/PDF/View/Checkout - Cart Summary Layout
     * @param array $data -> items|currency
     * Tax is product specific, it is calculated individually for each product
     * @return string
     */
    function cart_summary($data = array()) {
        $currency = isset($data['currency']) ? array('currency' => $data['currency']) : array();
        // PDF and EMAIL Styling
        $table_style = '';
        $th_style = '';
        $td_style = '';
        if(isset($data['inline_style'])) {
            $table_style = ' style="width: 100%; border: 1px solid black; border-collapse: collapse; padding: 10px;"';
            $th_style = ' style="font-size: 14px; background-color: #ddd; color: #444; padding-right: 10px; text-align: right; font-weight: 600;"';
            $td_style = ' style="border: 1px solid #444; padding: 5px; text-align: center;"';
        }
        $style_width = ' style="width: 150px;"';
        $pdf = isset($data['pdf']) ? true : false;
        $label_subtotal = lang('invoice_subtotal');
        $label_total = lang('invoice_total');
        $currency_code = isset($data['currency_code']) ? $data['currency_code'] : settings_item('site.currency_code');
        $rate_usd = isset($data['currency_value']) ? $data['currency_value'] : 1;
        $subtotal = 0;
        $total = 0;
        $tax = 0;
        $discount = 0;
        if(!empty($data['items'])) {
            foreach($data['items'] as $item) {
                $item = (array) $item;
                $subtotal += $item['subtotal'];
                $total += $item['total'];
                $tax += $item['tax'];
                $discount += $item['discount'];
            }
        }
        $subtotal = '<span id="order_subtotal">' . currency_format($subtotal, $currency) . '</span>';
        $total = '<span id="order_total">' . currency_format($total, $currency) . '</span>';
        $order_tax = '';
        if($tax > 0) {
            $label_taxrate = lang('label_taxrate');
            $tax_amount = '<span id="order_tax">' . currency_format(append_zero($tax), $currency) . '</span>';
            $order_tax = <<<EOL
    <tr class="tax_row">
		<td{$th_style}>{$label_taxrate}<span class="fas fa-plus-circle float-right"></span></td>
		<td{$td_style}>
			{$tax_amount}
		</td>
	</tr>
EOL;
        }
        
        // Do not check $discount variable else the discount table row will not be added on the checkout page.
        // Row must be added to show it Coupon Apply
        $order_discount = '';
        
        if(!empty($discount)) {
            $style_discount = '';
        } else {
            $style_discount = ' style="display: none;"';
        }
        $discount_type = isset($data['coupon_value']) ? $data['coupon_value'] : 0;
        $label_discount = lang('label_discount');
        $discount_order = '<span id="order_discount">' . currency_format($discount, $currency) . '</span>';
        $order_discount = <<<EOL
    <tr class="discount_row"{$style_discount}>
		<td{$th_style}>{$label_discount}<span class='help-inline discount-type'> ({$discount_type}%)</span><span class="fas fa-minus-circle float-right"></span></td>
		<td{$td_style}>
			{$discount_order}
		</td>
	</tr>
EOL;
			
			$output = <<<EOL
			
<div class="table-responsive">
<table class="table table-bordered table-summary"{$table_style}>
    <tr>
        <td{$th_style}>{$label_subtotal}</td>
        <td{$td_style}>{$subtotal}</td>
    </tr>
    {$order_discount}
    {$order_tax}
    <tr>
		<td{$th_style}>{$label_total}</td>
		<td{$td_style}>{$total}</td>
	</tr>
</table>
</div>

EOL;
    return $output;
    }
}

/************************************
 * END OF ECOMMERCE SECTION
 ************************************/
if(!function_exists('booking_details')) {
    /**
     * Admin/PDF/View/Checkout - Booking Details
     * @return string
     */
    function booking_details($data = array()) {
        $id = isset($data['id']) ? $data['id'] : 0;
        $title = isset($data['title']) ? $data['title'] : lang('gc_menu_bookings');
        $table = '';
        if(!empty($data['data'])) {
            $form_data = json_decode($data['data']);
            if(!empty($form_data)) {
                $tr = '';
                foreach($form_data as $detail) {
                    $tr .= '<tr>' . PHP_EOL;
                    $tr .= '<td>' . $detail->label . '</td>' . PHP_EOL;
                    $tr .= '<td>';
                    if($detail->type == 'image' && !empty($detail->value)) {
                        $tr .= '<a rel="gallery" href="' . gc_image_url($detail->value, false) .'" class="gc-fancybox">' . PHP_EOL;
                        $tr .= '<img src="' . gc_image_url($detail->value) . '" alt="' . $title .'" title="' . $title .'" class="img-thumbnail d-block mx-auto"/>';
				        $tr .= '</a>';
                    } else {
                        $tr .= $detail->value;
                    }
                    $tr .= '</td>' . PHP_EOL;
                    $tr .= '</tr>';
                }
                $table = <<<EOL
<table class="table table-hover table-sm">
{$tr}
</table>
EOL;
            }
        }
        $detail = isset($data['detail']) ? json_decode($data['detail']) : array();
        $close = lang('form_close');
        $output = <<<EOL
<!-- Button trigger modal -->
<button type="button" class="btn btn-primary btn-xs" data-toggle="modal" data-target="#form_{$id}">
  <i class="fas fa-eye"></i>
</button>
<!-- Modal -->
<div class="modal fade" id="form_{$id}" tabindex="-1" role="dialog" aria-labelledby="form_{$id}" aria-hidden="true">
  <div class="modal-dialog" role="document">
    <div class="modal-content">
      <div class="modal-header">
        <h5 class="modal-title">{$title}</h5>
        <button type="button" class="close" data-dismiss="modal" aria-label="Close">
          <span aria-hidden="true">&times;</span>
        </button>
      </div>
      <div class="modal-body">
      	{$table}
      </div>
      <div class="modal-footer">
        <button type="button" class="btn btn-secondary" data-dismiss="modal">{$close}</button>
      </div>
    </div>
  </div>
</div>
EOL;
        return $output;
    }
}
/**
* Filter input based on a whitelist. This filter strips out all characters that
* are NOT:
* - letters
* - numbers
* - Textile Markup special characters.
*
* Textile markup special characters are:
* _-.*#;:|!"+%{}@
*
* This filter will also pass cyrillic characters, and characters like � and �.
*
* Typical usage:
* $string = '_ - . * # ; : | ! " + % { } @ abcdefgABCDEFG12345 ��???????????????? ' . "\nAnd another line!";
* echo textile_sanitize($string);
*
* @param string $string
* @return string The sanitized string
* @author Joost van Veen
*/
function textile_sanitize($string) {
	$whitelist = '/[^a-zA-Z0-9?-??-?��????????????????' . settings_item('site.whitelist') . ' \.\*\+\\n|#;:!"%@{} _-]/';
	return preg_replace($whitelist, '', $string ?? '');
}
function escape($string) {
	return textile_sanitize($string);
}

/**
* Remove extra spaces from a string
*/
function remove_extra_spaces($str) {
		/* "/\s+/", "/\s([?.!])/" to remove extra spaces used, 
		"/\,+/", "/\,([?.!])/" to remove extra commas if any between each keyword
		"/\s,+/", "/\,([?.!])/" to remove space between word and comma*/
		//Get keywords with minimum length of four including space
		preg_match_all('/([\s\-a\-z0\-9]{4,})/i', $str, $matches, PREG_PATTERN_ORDER); 
		// Retrieve only first array from multidimensional array
		$keywords_with_minimum_length = implode(',', $matches[0]);
		// Match below given pattern and replace specified information
		$patterns = array("/\s+/", "/\s([?.!])/", "/\,+/", "/\,([?.!])/", "/\s,+/", "/\,([?.!])/");
		$replacer = array(" ","$1", ",","$1");
		// Convert keywords to lower character
		return strtolower(trim(preg_replace( $patterns, $replacer, $keywords_with_minimum_length))); //REMOVE WHITE SPACES BEFORE AND AFTER COMMAS
	}
//--------------------------------------------------------------------

