<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/**
 * GoClixy - Business Directory Script
 *
 * @package   GoClixy
 * @author    GoClixy Team
 * @copyright Copyright (c) 2015 - 2017, GoClixy Team
 * @link      https://www.goclixy.com
 * @filesource
 */

if ( ! function_exists('append_zero')) {
    /**
     * Append Zero to Decimal Value
     * @param float $value
     * @return float
     */
    function append_zero($value) {
        // add zero if length of first integer is one
        $first_digit = explode('.', $value);
        if((strlen((string) $first_digit[0]) == 1)) {
            $result = '0' .$value;
        } else {
            $result = $value;
        }
        return $result;
    } //end append_zero()
}

if ( ! function_exists('currency_format')) {
    /**
     * European countries display EURO symbol after the price
     * This function will allow the site owner to change the currency symbol position
     *
     * @param float $price
     * @param string $emmet_string - Enclose price within HTML elements
     * @return string
     */
    function currency_format($price, $data = array()) {
        $CI =& get_instance();
        // the currency value must be saved in the currencies table
        if(settings_item('site.currency_restrict') == 1) { // restrict user to default currency
            $currency = $CI->session->userdata('default_currency');
        } else {
            $currency = isset($data['currency']) ? $data['currency'] : $CI->session->userdata('site_currency');
        }
        if(empty($currency)) { // session does not work in Mobile App
            $CI->load->model('currencies/currencies_model');
            $currency = $CI->currencies_model->get_currency(settings_item('site.currency_code'));
        }
        $currency = (object) $currency;
        if(settings_item('site.currency_restrict') == 1) {
            $currency->rate_usd = 1; // restrict the usd rate to 1
        }
        $price = (float) $price;
        if(!empty($data['reverse_conversion'])) {
            // price to USD
            $price = decimal_convert($price / $currency->rate_usd, $currency->decimal_format);
            return $price;
        }
        $price = decimal_convert($price * $currency->rate_usd, $currency->decimal_format);
        // $price = number_format($price * $currency->rate_usd, 0); // number format without decimal
        if(!empty($data['price_only'])) {
            // price filter without currency
            return $price;
        }
        if(!empty($data['emmet_string'])) {
            $price = emmet($data['emmet_string'], $price);
        }
        $display_format = str_replace('%d', $price, $currency->display_format);
        return sprintf($display_format, $currency->symbol);
    } // end currency_format()
}

if ( ! function_exists('gc_date_format')) {
    
    /**
     * Allow Admin to set the date and time format for the entire website
     *
     * @param $created_on datetime
     * @param boolean $time - TRUE to display TIME
     *
     * @return date_format.
     */
    function gc_date_format($created_on, $time = true) {
        if(empty($created_on) || $created_on == '0000-00-00 00:00:00' || $created_on == '0000-00-00') {
            return $created_on;
        }
        // Unable to parse date having slash, so return such date and time without any parsing
        if(strpos($created_on, '/')) {
            return $created_on;
        }
        // $posted = new DateTime($created_on); // Throwing Error
        
        // Added on March 14, 2018
        $datetime = new DateTime();
        $split_date_time = explode(' ', $created_on);
        $parse_string = isset($split_date_time[1]) ? 'Y-m-d H:i:s' : 'Y-m-d';
        $posted = $datetime->createFromFormat($parse_string, $created_on);
        if(!$posted) {
            return $created_on;
        }
        
        // Convert into the specified format
        if($time == TRUE) {
            return $posted->format(settings_item('site.date_format') . ' ' .settings_item('site.time_format'));
        } else {
            return $posted->format(settings_item('site.date_format'));
        }
    } //end gc_date_format()
}

if ( ! function_exists('gc_address_format')) {
    /**
     * Common function for address format
     * @param string $address
     * @param string $locality
     * @param string $city
     * @param string $postcode
     * @param string $state
     * @param string $country
     * @param integer $template -> 1 -> multiline, 2 -> single line
     */
    function gc_address_format($address, $locality, $city, $state, $postcode, $country, $template = 1, $block = 'right') {
        if($template == 2 && $block == 'right') {
            $address = $address . ' ' . (!empty($locality) ? ', ' . $locality . ' ' : '') . $city . ', ' . $state . ' ' . $postcode . ', ' .ucwords(strtolower($country ?? ''));
            return $address;
        }
        $address = $address . (!empty($locality) ? ', ' . $locality : '') . '<br />' . $city .', '. $state . ' ' . $postcode . '<br />' . ucwords(strtolower($country ?? ''));
        return $address;
    } //end gc_address_format()
}

if ( ! function_exists('gc_listing_search_url')) {
    /**
     * Search URL
     * @param string $keyword
     * @param string $location
     * @param int $city_id
     * @param int $category_id
     * @param int $locality_id
     * @param string $type
     * @return string
     */
    function gc_listing_search_url ($keyword, $location, $city_id, $category_id, $locality_id, $type = 'locality') {
        if(settings_item('adv.search_blocks') == 1) {
            $url = gc_url_format('search') . '?q=' . urlencode($keyword) . '&l=' . $city_id . (!empty($category_id) ? '&c=' . $category_id : '') .'&tl=1&lt=' . $locality_id;
        } else {
            $url = gc_url_format('search') . '?q=' . urlencode($keyword) . '&l=' . urlencode($location) . (!empty($category_id) ? '&c=' . $category_id : '') .'&tl=1&lt=' . $locality_id;
        }
        return $url;
    } //end gc_address_format()
}

if ( ! function_exists('gc_tags_url')) {
    /**
     * Tags URL handling
     * @param string $module
     * @param array $tags
     * @param string $location
     * @param int $location_id - only in case of default search block
     * @param int $category_id
     * @param int $type_location
     * @return multitype:string
     */
    function gc_tags_url($module, $tags, $city, $city_id, $params) {
        $tag_with_urls = array();
        if(!empty($tags) && !empty($params)) {
            $location = !empty($params['location']) ? $params['location'] : $city;
            if(settings_item('adv.search_blocks') == 0) {
                $location = !empty($params['location_id']) ? (int) $params['location_id'] : (int) $location_id;
            }
            $category_id = (int) $params['category_id'];
            $type_location = (int) $params['type_location'];
            $keywords = explode(",", $tags);
            $i = 1;
            foreach($keywords as $keyword) {
                $tag_id = substr( $keyword, strrpos( $keyword, '-' ) + 1);
                // remove everything after the last hyphen
                $tag = substr($keyword, 0, strrpos($keyword, '-'));
                if($params['tag_id'] == $tag_id) {
                    $tag_with_urls[0] = '<strong>' . $tag . '</strong>';
                } else {
                    if($module == 'listings') {
                        $tag_with_urls[$i] = anchor(gc_url_format('search') . '?q=' . urlencode($tag) . '&l=' .urlencode($location) . ($category_id != 0 ? '&c=' . $category_id : '') . ($type_location != 0 ? '&tl=' . $type_location : '&tl=1') . '&t=' . $tag_id, $tag, 'title="'. $tag .'"');
                    } else {
                        $tag_with_urls[$i] = '<a href="'. gc_url_format($module) .'?q=' . urlencode($tag) . (!empty($location) ? '&l=' . urlencode($location) : '') . ($type_location != 0 ? '&mtl=' . $type_location : '&mtl=1') . '&t=' . $tag_id . '" title="' . $tag .'">' . $tag .'</a>';
                    }
                }
                $i++;
            }
        }
        ksort($tag_with_urls); // sort array based on keys
        return implode(', ', $tag_with_urls);
    } //end gc_tags_url()
}

if ( ! function_exists('gc_detail_tags')) {
    /**
     * Module Tags Handling View and Detail Page
     * @param string $module - Module Name
     * @param string $all_tags
     * @param string $city
     * @return string
     */
    function gc_detail_tags($module, $tags, $city) {
        $links = array();
        if($module == 'listings') {
            foreach($tags as $tag) {
                $links[] = '<a href="' . gc_url_format('search') . '?q=' .urlencode($tag->name). '&l=' . urlencode($city) .'&tl=1&t='. $tag->id .'" title="' . $tag->name .'">' . $tag->name . '</a>';
            }
        } else {
            foreach($tags as $tag) {
                $links[] = '<a href="'. gc_url_format($module) .'?q=' . urlencode($tag->name) . (!empty($city) ? '&l=' . urlencode($city) : '') . '" title="' . $tag->name .'">' . $tag->name .'</a>';
            }
        }
        return implode(', ', $links);
    }
}

if ( ! function_exists('gc_category_url')) {
    /**
     * Module Category URL format
     * @param string $slug
     * @param int $id
     * @param string $city
     */
    function gc_category_url($module, $slug, $id = NULL) {
        if(settings_item('lst.category_city_filter') == 1) {
            $slug = gc_slug_city($module, $slug);
        }
        if(settings_item('site.category_slug_uris') == 1) {
            $url = site_url(($module != 'listings' ? gc_uri_string($module) : '') . '/' . gc_uri_string('category') . '/' . $slug);
        } else {
            $url = site_url(($module != 'listings' ? gc_uri_string($module) : '') . '/' . gc_uri_string('category') . '/' . $slug .'-' . $id);
        }
        return $url;
    } //end gc_category_url()
}

if ( ! function_exists('gc_location_url')) {
    /**
     * Module Location URL format
     * @param string $slug
     * @param int $id
     * @param string $city
     */
    function gc_location_url($module, $slug, $id = NULL) {
        if($module == 'listings') {
            return gc_url_format('cities', $slug, $id);
        } else {
            return site_url(gc_uri_string($module) . '/' . gc_uri_string('location') . '/' . $slug .'-' . $id);
        }
    } //end gc_location_url()
}

if ( ! function_exists('gc_locality_url')) {
    /**
     * Module Locality URL format
     * @param string $slug
     * @param int $id
     * @param string $city
     */
    function gc_locality_url($module, $slug, $id = NULL) {
        return site_url(gc_uri_string($module) . '/' . gc_uri_string('locality') . '/' . $slug .'-' . $id);
    } //end gc_locality_url()
}

if ( ! function_exists('gc_slug_city')) {
    /**
     * Append city name with slug for SEO
     * @param string $module
     * @param string $slug
     */
    function gc_slug_city($module, $slug) {
        // append location name with category slug for SEO purposes
        if($module == 'listings' || gc_module_settings($module, 'locations') == 1) {
            $CI =& get_instance();
            $location = json_decode($CI->session->userdata('tl_location') ?? '');
            if(!empty($location->name)) {
                $CI->load->library('slug');
                $slug .= '-'  . $CI->slug->create_slug($location->name);
            }
        }
        return $slug;
    }
}

if ( ! function_exists('gc_url_format')) {
    /**
     * Common URL format For Entire Website
     * @param string $module
     * @param string $type - Detail/View
     * @param array $extra - Additional Parameters
     * @param string $slug
     * @param int $id
     * @param string $city
     */
    function gc_url_format($module = 'listings', $slug = NULL, $id = 0, $city = NULL) {
        $url = '';
        if(!empty($slug)) {
            switch($module) {
                case 'listings':
                    if(settings_item('route.listings') == 1) {
                        // set value to 1 for the primary route without detail in the URL
                        // don't forget to update the config/routes.php file: $route['(:any)']	= 'listings/detail/$1'; remove $route['detail/(:any)']	= 'listings/detail/$1';
                        $url = site_url(gc_listing_url($slug, $id, $city));
                    } else {
                        $url = site_url(gc_uri_string('detail') . '/' . gc_listing_url($slug, $id, $city));
                    }
                    break;
                case 'pages':
                    $url = site_url(gc_uri_string('pages') .'/' . $slug);
                    break;
                case 'news':
                    $url = site_url(gc_uri_string('news') . '/' . $slug);
                    break;
                case 'cities':
                    $url = site_url(gc_uri_string('location') . '/' . gc_module_url($slug, $id));
                    break;
                case 'categories':
                    if(settings_item('site.category_slug_uris') == 1) {
                        $url = site_url(gc_uri_string('category') . '/' . $slug);
                    } else {
                        $url = site_url(gc_uri_string('category') . '/' . gc_module_url($slug, $id));
                    }
                    break;
                case 'tags':
                    $url = site_url(gc_uri_string('tag') . '/' . gc_module_url($slug, $id));
                    break;
                case 'forum_categories':
                case 'forums/category':
                    $url = site_url(gc_uri_string('forums') .'/' . gc_uri_string('category') . '/' . $slug);
                    break;
                case 'forum_threads':
                case 'forums/thread':
                    $url = site_url(gc_uri_string('forums') .'/' . gc_uri_string('thread') . '/' . $slug);
                    break;
                case 'forum_posts':
                case 'forums/post':
                    $url = site_url(gc_uri_string('forums') .'/' . gc_uri_string('post') . '/' . $slug);
                    break;
                case 'members/pages':
                    $url = site_url(gc_uri_string('members') . '/' . gc_uri_string('pages') .'/' . $slug);
                    break;
                case 'ref':
                    $url = site_url(gc_uri_string('ref') . '/' . $slug);
                    break;
                default:
                    $url = site_url(gc_uri_string($module) . '/' . gc_uri_string('detail') . '/' . gc_module_url($slug, $id));
                    break;
            }
        } else {
            if( strpos( $module, '/' ) !== false ) { // Member Area URLs
                $split_url = explode('/', $module);
                $member_area = $split_url[0]; // 'members'
                $member_area = gc_uri_string($member_area);
                switch($split_url[1]) {
                    case 'contact':
                        $url = site_url(gc_uri_string('pages') . '/' . gc_uri_string('contact'));
                        break;
                    case 'profile': // User's Controller
                        $url = site_url(gc_uri_string('users') . '/' . gc_uri_string('profile'));
                        break;
                    case 'advertise':
                        $url = site_url(gc_uri_string('advertise'));
                        break;
                    case 'verify_otp':
                        $url = site_url(gc_uri_string('verify_otp'));
                        break;
                    case 'forgot_password':
                        $url = site_url(gc_uri_string('forgot_password'));
                        break;
                    case 'reset_password':
                        $url = site_url(gc_uri_string('reset_password'));
                        break;
                    case 'resend_activation':
                        $url = site_url(gc_uri_string('resend_activation'));
                        break;
                    default:                        
                        if(isset($split_url[2])) {
                            $url = site_url(gc_uri_string($split_url[0]) . '/' . gc_uri_string($split_url[1]) . '/' . $split_url[2]);
                        } else {
                            $url = site_url(gc_uri_string($split_url[0]) . '/' . gc_uri_string($split_url[1]));
                        }
                        if(!empty($slug)) {
                            $url .= '/' . $slug;
                        }
                        if(!empty($id)) {
                            $url .= $id;
                        }
                        // $url = site_url($member_area . '/' . gc_uri_string($split_url[1]));
                }
            } else {
                switch($module) {
                    case 'login':
                    case 'register':
                        /**
                         * A theme can also have an ajax login. This type of login can be handled in two ways:
                         *
                         * METHOD 1:
                         * Use a session variable name ajax_login and set its value to TRUE in the WIDGET itself to redirect user to the AJAX box.
                         * This variable is unset on theme change.
                         *
                         * METHOD 2 (NOT USED):
                         * Create a new variable theme.login in the settings table
                         * If the value of the theme.login is 1, it means, this theme uses an Ajax Login
                         * Value of this variable is changed while changing the theme default.
                         */
                        // $CI =& get_instance();
                        $url = site_url(gc_uri_string($module)); // ($CI->session->userdata('ajax_login') ? '#' : '') .  gc_uri_string($module)
                        break;
                    default:
                        $url = site_url(gc_uri_string($module));
                        break;
                }
            }
        }
        return $url;
    } //end gc_date_format()
}

if ( ! function_exists('gc_listing_url')) {
    /**
     * Listing Detail Page URL Format
     * @param string $slug
     * @param string $id
     * @param string $city
     * @return string
     */
    function gc_listing_url($slug, $id = NULL, $city = NULL) {
        $CI =& get_instance();
        $CI->load->library('slug');
        if(settings_item('lst.slug_url') == 1) {
            $url = $slug;
        } else if(settings_item('lst.city_url') == 1) {
            $url = $slug . '-' . $CI->slug->create_slug($city) . '-' . $id;
        } else {
            $url = $slug . '-' . $id;
        }
        return $url;
    }//end gc_listing_url()
}

if ( ! function_exists('gc_module_url')) {
    /**
     * Modules Detail Page URL Format
     * @param string $slug
     * @param string $id
     * @return string
     */
    function gc_module_url($slug, $id) {
        $url = $slug .'-' . $id;
        return $url;
    } //end gc_module_url()
}


if ( ! function_exists('gc_uri_string')) {
    
    /**
     * Allow Admin to Change URLs in any language
     * @param string $module
     * @return string
     */
    function gc_uri_string($module) {
        $gc_uri_string = lang('route_' . $module);
        return $gc_uri_string;
    } //end gc_uri_string()
}

if ( ! function_exists('number_shorten')) {
    /**
     * Shortens a number and attaches K, M, B, etc. accordingly to display number of hits
     * @param number $number
     * @param number $precision
     * @param string $divisors
     * @return string
     */
    
    function number_shorten($number, $precision = 3, $divisors = null) {
        // Setup default $divisors if not provided
        if (!isset($divisors)) {
            $divisors = array(
                pow(1000, 0) => '', // 1000^0 == 1
                pow(1000, 1) => 'K', // Thousand
                pow(1000, 2) => 'M', // Million
                pow(1000, 3) => 'B', // Billion
                pow(1000, 4) => 'T', // Trillion
                pow(1000, 5) => 'Qa', // Quadrillion
                pow(1000, 6) => 'Qi', // Quintillion
            );
        }
        
        // Loop through each $divisor and find the
        // lowest amount that matches
        foreach ($divisors as $divisor => $shorthand) {
            if (abs($number) < ($divisor * 1000)) {
                // We found a match!
                break;
            }
        }
        
        // We found our match, or there were no matches.
        // Either way, use the last defined value for $divisor.
        return number_format($number / $divisor, $precision) . $shorthand;
    } //end number_shorten()
}

if ( ! function_exists('decimal_convert')) {
    /**
     * Convert value upto two decimal places
     * @param float $value
     * @return float
     */
    function decimal_convert($value, $decimal_format = null) {
        $decimal_format = '.';
        return number_format($value, 2, $decimal_format, '');
        // return number_format($value, 2, '.','');
    } //end decimal_convert()
}

if ( ! function_exists('gc_languages')) {
    /**
     * This function will return language name from language code e.g. en - english
     * @param string $name
     * @return string
     */
    function gc_languages($code = null, $name = null) {
        $CI =& get_instance();
        $CI->load->library('translations');
        return $CI->translations->get_languages(true, $code, $name);
    }
}

if ( ! function_exists('get_language_code')) {
    /**
     * This function will return language code from language name e.g. english - en
     * @param string $name
     * @return string
     */
    function get_language_code($name){
        return gc_languages(null, $name);
    } //end get_language_code()
}

if ( ! function_exists('truncate_text')) {
     /**
     * Common function to truncate description text
     * @param string $body
     * @param string $maxlength
     * @return string
     */
    function truncate_text($body, $maxlength, $strip_tags = true){
        if (strlen($body)<$maxlength) return $body;
        $body = substr($body, 0, $maxlength);
        $rpos = strrpos($body,' ');
        if ($rpos > 0) $body = substr($body, 0, $rpos) . '...';
        if($strip_tags) {
            return strip_tags($body);
        } else {
            return $body;
        }
    } //end truncate_text()
}

if ( ! function_exists('convert_date_time')) {
    /**
     * Home Page News to convert datetime to minutes, hours, days, weeks and years
     * @param string $time
     * @param boolean $is_future - Datetime is future
     * @param boolean $display_ago - Add Ago
     * @return string
     */// convert_date_time
    function convert_date_time ($datetime, $is_future = false, $display_ago = true) {
        $time = strtotime($datetime);
        $time = $is_future ? $time - time() : time() - $time; // to get the time since that moment
        $tokens = array (
            31536000 => lang('label_years'),
            2592000 => lang('label_months'),
            604800 => lang('label_weeks'),
            86400 => lang('label_days'),
            3600 => lang('label_hours'),
            60 => lang('label_minutes'),
            1 => lang('label_seconds')
        );
        foreach ($tokens as $unit => $text) {
            if ($time < $unit) continue;
            $numberOfUnits = floor($time / $unit);
            return $numberOfUnits . ' ' . $text . ' ' . ($display_ago ? lang('label_ago') : '');
            //return $numberOfUnits.' '.$text.(($numberOfUnits>1)?'s':'');
        }
        return gc_date_format($datetime, true); // Return default datetime
    } //end convert_date_time()
}

if ( ! function_exists('gc_module_id')) {
    /**
     * Get Module ID - Comments, Favorites, Module Catgories and Ratings
     *
     * Note: Module IDs are hard coded in JS files like Ads module, therefore, do not change them
     *
     * @param string $module
     * @return number
     */
    function gc_module_id($module) {
        switch($module) {
            case 'users': // Custom Fields For User's Profile
                $module_id = 10;
                break;
            case 'pages': // Mobile APP Slider Redirect
                $module_id = 11;
                break;
            case 'categories': // Mobile APP Slider Redirect
                $module_id = 12;
                break;
            case 'cities':
                $module_id = 13;
                break;
            case 'news':
                $module_id = 14;
                break;
            case 'banners': // Advertisement Module
                $module_id = 15;
                break;
            case 'tags': // Advertisement Module
                $module_id = 16;
                break;
            case 'ads': // Advertisement Module
                $module_id = 17;
                break;
            case 'leads':
                $module_id = 18;
                break;
            case 'subscribers':
                $module_id = 19;
                break;
            case 'packages':
                $module_id = 20;
                break;
            case 'wallets':
                $module_id = 21;
                break;
            case 'coupons':
                $module_id = 22;
                break;
            case 'fields':
                $module_id = 23;
                break;
            case 'images':
                $module_id = 24;
                break;
            case 'points':
                $module_id = 25;
                break;
            case 'links': // 26 is given to branch offices
                $module_id = 27;
                break;
            case 'groups':
                $module_id = 28;
                break;
            case 'bookings':
                $module_id = 29;
                break;
            case 'orders':
                $module_id = 30;
                break;
            case 'business_queries':
                $module_id = 31;
                break;
            case 'testimonials':
                $module_id = 32;
                break;
            case 'email_templates':
                $module_id = 33;
                break;
            case 'ad_packages':
                $module_id = 34;
                break;
            case 'settings':
                $module_id = 35;
                break;
            case 'registrations':
                $module_id = 36;
                break;
            case 'comments':
                $module_id = 37;
                break;
            case 'affiliates':
                $module_id = 38;
                break;
            case 'product_variants':
                $module_id = 39;
                break;
            case 'invitations':
                $module_id = 40;
                break;
            case 'payouts':
                $module_id = 41;
                break;
            case 'states':
                $module_id = 42;
                break;
            case 'countries':
                $module_id = 43;
                break;
            case 'feedbacks':
                $module_id = 44;
                break;
            case 'callbacks':
                $module_id = 45;
                break;
            case 'contact_queries':
                $module_id = 46;
                break;
            case 'subscriber_users':
                $module_id = 47;
                break;
            case 'solutions':
                $module_id = 51;
                break;
            case 'percents': // Business Completion Percents
                $module_id = 52;
                break;
            case 'pyg': // Package - Pay As You Go
                $module_id = 53;
                break;
            case 'claims': // Listing Claims
                $module_id = 54;
                break;
            case 'affiliate_users':
                $module_id = 55;
                break;
            case 'commission_hierarchies': // affiliate tier commissions
                $module_id = 56;
                break;
            case 'commission_cpc': // cost per click commissions
                $module_id = 57;
                break;
            case 'commission_standard': // standard commissions
                $module_id = 58;
                break;
            case 'commission_levels': // commission_levels pricing
                $module_id = 59;
                break;
                // Important do not forget to add the new module in the modules table
            case 'order_items':
                $module_id = 60;
                break;
            case 'posts':
                $module_id = 61;
                break;
            case 'post_types':
                $module_id = 62;
                break;
            case 'commissions': // payouts and affiliate commissions
                $module_id = 63;
                break;
            default:
                $module_id = !empty($module) ? gc_module_key('id', $module) : 0;
        }
        return $module_id;
    } //end gc_module_id()
}

if ( ! function_exists('gc_module_name')) {
    /**
     * Get Module Name from Module ID - Comments, Favorites, Module Catgories and Ratings
     *
     * Note: Module name also serve as the table name for queries in some modules like Ad
     * i.e. always use the database table name for the module name
     *
     * @param string $module
     * @return number
     */
    function gc_module_name($module_id) {
        switch($module_id) {
            case 10:
                $module = 'users';
                break;
            case 11:
                $module = 'pages';
                break;
            case 12:
                $module = 'categories';
                break;
            case 13:
                $module = 'cities';
                break;
            case 14:
                $module = 'news';
                break;
            case 15:
                $module = 'banners';
                break;
            case 16:
                $module = 'tags';
                break;
            case 17:
                $module = 'ads';
                break;
            case 18:
                $module = 'leads';
                break;
            case 19:
                $module = 'subscribers';
                break;
            case 20:
                $module = 'packages';
                break;
            case 21:
                $module = 'wallets';
                break;
            case 22:
                $module = 'coupons';
                break;
            case 23:
                $module = 'fields';
                break;
            case 24:
                $module = 'images';
                break;
            case 25:
                $module = 'points';
                break;
            case 27: // 26 is assigned to the branch offices
                $module = 'links';
                break;
            case 28:
                $module = 'groups';
                break;
            case 29:
                $module = 'bookings';
                break;
            case 30:
                $module = 'orders';
                break;
            case 31:
                $module = 'business_queries';
                break;
            case 32:
                $module = 'testimonials';
                break;
            case 33:
                $module = 'email_templates';
                break;
            case 34:
                $module = 'ad_packages';
                break;
            case 35:
                $module = 'settings';
                break;
            case 36:
                $module = 'registrations';
                break;
            case 37:
                $module = 'comments';
                break;
            case 38:
                $module = 'affiliates';
                break;
            case 39:
                $module = 'product_variants';
                break;
            case 40:
                $module = 'invitations';
                break;
            case 41:
                $module = 'payouts';
                break;
            case 42:
                $module = 'states';
                break;
            case 43:
                $module = 'countries';
                break;
            case 44:
                $module = 'feedbacks';
                break;
            case 45:
                $module = 'callbacks';
                break;
            case 46:
                $module = 'contact_queries';
                break;
            case 47:
                $module = 'subscriber_users';
                break;
            case 51:
                $module = 'solutions';
                break;
            case 52:
                $module = 'percents';
                break;
            case 53: // Package - Pay As You Go
                $module = 'pyg';
                break;
            case 54: // Listing Claims
                $module = 'claims';
                break;
            case 55:
                $module = 'affiliate_users';
                break;
            case 56: // affiliate tier commissions
                $module = 'commission_hierarchies';
                break;
            case 57: // cost per click commissions
                $module = 'commission_cpc';
                break;
            case 58: // standard commissions
                $module = 'commission_standard';
                break;
            case 59: // commission_levels pricing
                $module = 'commission_levels';
                break;
            case 60: // Payouts Module
                $module = 'order_items';
                break;
            case 61:
                $module = 'posts';
                break;
            case 62:
                $module = 'post_types';
                break;
            case 63:
                $module = 'commissions';
                break;
            default:
                $module = gc_module_key('module', $module_id, '', 'id');
        }
        return $module;
    } //end gc_module_name()
}

if ( ! function_exists('gc_module_icon')) {
    /**
     * Each module has their own fa icon
     * This function helps to get all fa name from one location
     * @param string $module - name of the module
     * @return string
     */
    function gc_module_icon($module, $mobile = FALSE) {
        switch ($module) {
            case 'all':
                $icon = 'home';
                $ion = 'home';
                break;
            default:
                $icon = gc_module_key('icon', $module, '');
                $ion = gc_module_key('ion', $module, '');
        }
        if($mobile) {
            return $ion;
        } else {
            return $icon;
        }
    } //end gc_module_icon()
}

if ( ! function_exists('gc_module_translation')) {
    /**
     * Translate module text for multi languages
     *
     * Also update the information in the Modules table
     *
     * @param string $module
     * @return string
     */
    function gc_module_translation($module) {
        $language = '';
        switch ($module) {
            case 'users':
                $language = lang('users');
                break;
            case 'pages':
                $language = lang('bf_pages');
                break;
            case 'categories':
                $language = lang('categories');
                break;
            case 'cities':
                $language = lang('cities');
                break;
            case 'news':
                $language = lang('news');
                break;
            case 'banners':
                $language = lang('label_banners');
                break;
            case 'tags':
                $language = lang('label_tags');
                break;
            case 'ads':
                $language = lang('gc_menu_ads');
                break;
            case 'leads':
                $language = lang('gc_menu_leads');
                break;
            case 'subscribers':
                $language = lang('gc_menu_subscribers');
                break;
            case 'packages':
                $language = lang('gc_menu_packages');
                break;
            case 'wallets':
                $language = lang('gc_menu_wallets');
                break;
            case 'points':
                $language = lang('gc_menu_points');
                break;
            case 'coupons':
                $language = lang('gc_menu_coupons');
                break;
            case 'fields':
                $language = lang('gc_menu_fields');
                break;
            case 'comments':
                $language = lang('gc_menu_comments');
                break;
            case 'variants':
                $language = lang('gc_menu_variants');
                break;
            case 'bookings':
                $language = lang('gc_menu_bookings');
                break;
            case 'invitations':
                $language = lang('invitations');
                break;
            case 'affiliates':
                $language = lang('gc_menu_affiliates');
                break;
            case 'payouts':
                $language = lang('gc_menu_payouts');
                break;
            case 'orders':
                $language = lang('gc_menu_orders');
                break;
            default:
                $language = gc_module_key('language', $module, '');
                if(!empty($language) && is_string($language)) {
                    $language = lang($language);
                } else {
                    $language = '';
                }
        }
        return $language;
    } //end gc_module_translation()
}

if ( ! function_exists('get_video_title')) {
    /**
     * Get Youtube Video Title
     * @return string title
     */
    function get_video_title($video_id) {
        /*$content = file_get_contents("https://youtube.com/get_video_info?video_id=" .$video_id);
         parse_str($content, $ytarr);
         if(!empty($ytarr['title'])) {
         return $ytarr['title'];
         } else {
         return null;
         }*/
        // $url = "http://www.youtube.com/watch?v=".$video_id;
        $tmp = file_get_contents("http://youtube.com/watch?v=" . $video_id);
        $tmp2 = substr($tmp, (strpos($tmp, "<title>") + 7), 220);
        $tmp = substr($tmp2, 0, strpos($tmp2, "</title>") - 9);
        $title = htmlspecialchars_decode($tmp);
        return trim($title);
    } //end get_video_title()
}

if ( ! function_exists('get_video_id')) {
    /**
     * Get youtube video ID
     * @param string $url
     * @return string
     */
    function get_video_id($url) {
        // http://stackoverflow.com/questions/3452546/javascript-regex-how-to-get-youtube-video-id-from-url
        $regex = '/^.*(?:(?:youtu\.be\/|v\/|vi\/|u\/\w\/|embed\/)|(?:(?:watch)?\?v(?:i)?=|\&v(?:i)?=))([^#\&\?]*).*/';
        if(preg_match($regex, $url, $match)) {
            return $match[1];
        } else {
            return false;
        }
    } //end get_video_id()
}

if ( ! function_exists('get_video_duration')) {
    /**
     * Get youtube video duration
     * @param string $videoID
     * @return string
     */
    function get_video_duration($videoID) {
        // https://stackoverflow.com/questions/15596753/how-do-i-get-video-durations-with-youtube-api-version-3/15605838
        $apikey = settings_item('site.google_api_key'); // Like this AIcvSyBsLA8znZn-i-aPLWFrsPOlWMkEyVaXAcv
        $duration = @file_get_contents("https://www.googleapis.com/youtube/v3/videos?part=contentDetails&id=$videoID&key=$apikey");
        if(!empty($duration)) {
            $VidDuration =json_decode($duration, true);
            foreach ($VidDuration['items'] as $vidTime) {
                $VidDuration= $vidTime['contentDetails']['duration'];
            }
            preg_match_all('/(\d+)/',$VidDuration,$parts);
            return append_zero($parts[0][0]) . ":" .
                append_zero($parts[0][1]) .
                (!empty($parts[0][2]) ? ":". $parts[0][2] : ''); // Return 1:11:46 (i.e.) HH:MM:SS
        } else {
            return '';
        }
    } //end get_video_duration()
}

if ( ! function_exists('gc_order_status')) {
    /**
     * Return order status
     * @param int $status
     * @param boolean $with_badge - True - Enclose status in SPAN
     * @return string
     */
    function gc_order_status($status_value, $with_badge = false) {
        $badge = '';
        switch ($status_value) {
            case '0':
                $status = lang('tab_active');
                $badge = 'info';
                break;
            case '1':
                $status = lang('tab_confirmed');
                $badge = 'success';
                break;
            case '2':
                $status = lang('tab_pending');
                $badge = 'warning';
                break;
            case '3':
                $status = lang('tab_failed');
                $badge = 'danger';
                break;
            case '4':
                $status = lang('tab_cancelled');
                $badge = 'secondary';
                break;
            case '5':
                $status = lang('tab_upgraded');
                $badge = 'primary';
                break;
            case '6':
                $status = lang('tab_renewed');
                $badge = 'primary';
                break;
            case '7':
                $status = lang('tab_claimed');
                $badge = 'primary';
                break;
            case '8':
                $status = lang('tab_expired');
                $badge = 'danger';
                break;
            case '9':
                $status = lang('tab_refunded');
                $badge = 'info';
                break;
            case '10':
                $status = lang('tab_spammed');
                $badge = 'secondary';
                break;
        }
        if($with_badge) {
            $status = '<span class="badge badge-' . $badge . '">' . $status . '</span>';
        }
        return $status;
    } //end gc_order_status()
}

if ( ! function_exists('gc_invoice_status')) {
    /**
     * Return Invoice status
     * @param int $status
     * @return string
     */
    function gc_invoice_status($status) {
        $message = '';
        switch($status) {
            case 0:
                $message = lang('pending');
                break;
            case 1:
                $message = lang('paid');
                break;
            case 2:
                $message = lang('cancelled');
                break;
            case 3:
                $message = lang('status_expired');
                break;
        }
        return $message;
    } //end gc_invoice_status()
}

if ( ! function_exists('gc_comment_status')) {
    /**
     * Return comment message based on its status
     * @param int $status
     * @return string
     */
    function gc_comment_status($status) {
        $message = '';
        switch($status) {
            case 0:
                $message = lang('comments_inactive');
                break;
            case 1:
                $message = lang('comments_active');
                break;
            case 2:
                $message = lang('comments_flag');
                break;
            case 3:
                $message = lang('comments_spam');
                break;
            case 4:
                $message = lang('comments_reject');
                break;
        }
        return $message;
    } //end gc_comment_status()
}

if ( ! function_exists('gc_listing_status')) {
    /**
     * Return listing status
     * @param int $status
     * @return string
     */
    function gc_listing_status($status_value) {
        switch ($status_value) {
            case '1':
                $status = lang('us_active');
                break;
            case '2':
                $status = lang('status_expired');
                break;
            case '3':
                $status = lang('status_spam');
                break;
            default:
                $status = lang('us_inactive');
                break;
        }
        return $status;
    } //end gc_listing_status()
}

if ( ! function_exists('get_property_purpose')) {
    /**
     * Return property purpose - Sale/Rent
     * @param int $purpose
     */
    function get_property_purpose($purpose) {
        $property_purpose = '';
        switch($purpose) {
            case 1:
                $property_purpose = lang('label_purpose_sale');
                break;
            case 2:
                $property_purpose = lang('label_purpose_rent');
                break;
            case 3:
                $property_purpose = lang('label_purpose_rent_sale');
                break;
        }
        return $property_purpose;
    } //end get_property_purpose()
}

if ( ! function_exists('get_property_ownership')) {
    /**
     * Return property ownership - Agent/Owner/Builder
     * @param int $ownership
     */
    function get_property_ownership($ownership) {
        $property_ownership = '';
        switch($ownership) {
            case 1:
                $property_ownership = lang('label_ownership_agent');
                break;
            case 2:
                $property_ownership = lang('label_ownership_owner');
                break;
            case 3:
                $property_ownership = lang('label_ownership_builder');
                break;
        }
        return $property_ownership;
    } //end get_property_ownership()()
}

if ( ! function_exists('get_property_type')) {
    /**
     * Return property type
     * @param int $type
     */
    function get_property_type($type) {
        switch($type) {
            case 1:
                $property_type = lang('label_type_apartment');
                break;
            case 2:
                $property_type = lang('label_type_house');
                break;
            case 3:
                $property_type = lang('label_type_land');
                break;
            case 3:
                $property_type = lang('label_type_commercial');
                break;
        }
        return $property_type;
    } //end get_property_type()()
}

if ( ! function_exists('get_job_type')) {
    /**
     * Return job type
     * @param int $type
     */
    function get_job_type($type) {
        switch($type) {
            case 1:
                $job_type = lang('job_type_fulltime');
                break;
            case 2:
                $job_type = lang('job_type_parttime');
                break;
            case 3:
                $job_type = lang('job_type_freelancer');
                break;
            case 4:
                $job_type = lang('job_type_internship');
                break;
            case 5:
                $job_type = lang('job_type_contract');
                break;
        }
        return $job_type;
    } //end get_job_type()()
}

if ( ! function_exists('gc_image_url')) {
    /**
     * Return the image url from the image path
     *
     * @param string $image_path - image with upload directory path
     * @param string $url - e.g. assets/images/listings/logos/watsonIT.png
     * @param string $thumb - Return thumb image if TRUE e.g. assets/images/listings/logos/thumbs/thumb__watsonIT.png
     * @param boolean $no_image - if image does not exist or empty then return the no-logo.png file path
     * @param string $module - Name of the module
     * @param boolean $relative - Display Filemanager and other images (themes, images etc.) using the relative path saved in database
     * @param boolean $base64_encode - DOMPDF live server image issue
     */
    function gc_image_url($image_path, $thumb = true, $no_image = true, $module = null, $relative = false, $base64_encode = false) {
        $url = $image_path;
        if(!empty($image_path) && is_string($image_path)) {
            $regexp =  "/((?:https?\:\/\/|www\.)(?:[-a-z0-9]+\.)*[-a-z0-9]+.*)/i";
            if(preg_match($regexp, $image_path) || substr($image_path, 0, 8) == "https://") {
                // url does not start with http or https protocol
                if(substr($image_path, 0, 4) == "www.") {
                    // append http to the image url
                    $url = 'http://'. $image_path;
                }
            } else {
                $image_folder_prefix = '';
                if(strpos($image_path, 'assets/images') === false) {
                    $image_folder_prefix = 'assets/images/';
                }
                // get image thumb
                if(!empty($module)) {
                    if(is_array($module) && !empty($module['module'])) {
                        $module = $module['module'];
                    }
                    $image_url = $module . '/';
                    if(!$thumb) {
                        switch($module) {
                            case 'cities':
                            case 'states':
                            case 'countries':
                            case 'location':
                            case 'categories':
                                $image_url = '';
                                break;
                        }
                        $url = base_url() . $image_folder_prefix . $image_url . $image_path;
                    } else {
                        switch($module) {
                            case 'cities':
                            case 'states':
                            case 'countries':
                            case 'location':
                            case 'categories':
                                $image_path = str_replace($image_url, '', $image_path);
                                break;
                        }
                        $url = base_url() . $image_folder_prefix . $image_url . 'thumbs/thumb__' . $image_path;
                    }
                } else if($relative) {
                    // Display Filemanager and other images using the relative path saved in database
                    $url = base_url() . $image_folder_prefix . $image_path;
                } else {
                    $image_path = $image_folder_prefix . $image_path;
                    if($thumb) {
                        // explode the image path to an array
                        $path = array_map('trim', explode('/', $image_path));
                        // remove the filename from the array and store it in a variable
                        $filename = array_pop($path);
                        $image_path = implode('/', $path) . '/thumbs/thumb__' . $filename;
                    }
                    if($base64_encode) {
                        $path = FCPATH . $image_path;
                        $type = pathinfo($path, PATHINFO_EXTENSION);
                        $data = file_get_contents($path);
                        $url = 'data:image/' . $type . ';base64,' . base64_encode($data);
                    } else {
                        $url = base_url() . $image_path;
                    }
                }
            }
        } else if($no_image) {
            $url = gc_site_image('no-image');
        }
        return $url;
    }
}

if ( ! function_exists('getStringBetween')) {
    /**
     * Extract text between two characters from a string
     * @param string $str
     * @param string $from
     * @param string $to
     * @param string $withFromAndTo
     * @return string
     */
    function getStringBetween($str, $from, $to, $withFromAndTo = false) {
        $sub = substr($str, strpos($str,$from)+strlen($from),strlen($str));
        if ($withFromAndTo)
            return $from . substr($sub,0, strrpos($sub,$to)) . $to;
            else
                return substr($sub, 0, stripos($sub, $to));
    }
}

if ( ! function_exists('gc_unsubscribe')) {
    /**
     * Return unsubscribe message with url
     * @param string $module
     * @param string $str
     * @return string
     */
    function gc_unsubscribe($row_id, $module, $message = NULL) {
        if(filter_var($row_id, FILTER_VALIDATE_EMAIL)) {
            return $message;
        }
        $url = '<small>' .sprintf(lang('em_msg_unsubscribe_text'), settings_item('site.title'), anchor(site_url('unsubscribe?id=' . urlencode($row_id) . '&md=' . $module), lang('click_here'), 'title="' . settings_item('site.title') . ' ' . lang('em_unsubscribe_link') . '"')) . '</small>';
        if(!empty($message)) {
            $message = $message . '<br /><br />' . $url;
        } else {
            $message = $url;
        }
        return $message;
    }
}

if ( ! function_exists('gc_redirect_uri')) {
    /**
     * Exploder redirect URI and return parameters
     * $separator = "!~!";
     * @param string $redirect_url
     * @param int $module_id - returned through mysql query
     * @param string $return - requested return
     * @param string $extend_url - in case of comments #comments-box is added to the URL
     * @return array
     */
    function gc_redirect_uri($redirect_url, $module_id, $return = NULL, $extend_url = '') {
        $result = array();
        if(!empty($redirect_url)) {
            $uri_array = explode('!~!', $redirect_url);
            $id = isset($uri_array[0]) ? (int) $uri_array[0] : 0;
            $title = isset($uri_array[1]) ? $uri_array[1] : NULL;
            $slug = isset($uri_array[2]) ? $uri_array[2] : NULL;
            $status = isset($uri_array[3]) ? (int) $uri_array[3] : NULL;
            $city = isset($uri_array[4]) ? $uri_array[4] : NULL;
            $module = gc_module_name($module_id);
            if($module == 'variants') {
                $module = 'products';
            }
            $result = array(
                'id' => $id,
                'title' => $title,
                'slug' => $slug,
                'status' => $status,
                'city' => $city,
                'url' => gc_url_format($module, $slug, $id, $city),
                'link' => ($status == 1 ? '<a href="' . gc_url_format($module, $slug, $id, $city) . $extend_url . '" title="' . $title . '" target="_blank">' . $id . ' - ' . $title .' <i class="fa fa-external-link"></i></a>' : $id . ' - ' . $title)
            );
            if(!empty($return)) {
                return $result[$return];
            } else {
                return $result;
            }
        }
        return null;
    }
}

if ( ! function_exists('gc_modules')) {
    /**
     * Get all modules from the Application Config
     * Except a specific module
     * @param string $key - Site Modules key name
     * @param boolean $skip_video_branches - Do not skip videos and branches - Packages
     * @return array
     */
    function gc_modules($key = 'module', $skip_video_branches = true) {
        $config_modules = gc_site_modules();
        $modules = array_column($config_modules, $key);
        if($skip_video_branches) {
            $video_key = array_keys($modules, 'videos');
            if(!empty($video_key)) {
                unset($modules[$video_key[0]]);
            }
            $branch_key = array_keys($modules, 'branches');
            if(!empty($branch_key)) {
                unset($modules[$branch_key[0]]);
            }
        }
        return $modules;
    }
}

if ( ! function_exists('gc_site_modules')) {
    /**
     * Get only those modules from the Database table where site_module = 1
     * Do not grab all the modules, this may result in errors in other part of application
     *
     * Core modules like Users, Links, Fields etc. should be deal as it is.
     * Core modules information is also stored in the Modules Table, but this is only for reference, it is not used anywhere.
     *
     * @return array
     */
    function gc_site_modules() {
        // Load Zend Caching
        $CI =& get_instance();
        $CI->load->library('session');
        // $CI->session->unset_userdata('gc_site_modules');
        if(!$CI->session->userdata('gc_site_modules')) {
            $CI->load->library('gzf_cache', array('lifetime' => 604800));
            $gc_cache = $CI->gzf_cache->get_instance();
            $cacheID = 'gc_site_modules';
            if(!$gc_site_modules = $gc_cache->load($cacheID)) {
                $site_modules = $CI->db->order_by('priority')->get_where('modules', array('site_module' => 1, 'status' => 1))->result();
                // Create an Array
                $gc_site_modules = array();
                foreach($site_modules as $site_module) {
                    $gc_site_modules[] = array(
                        'module' => $site_module->name,
                        'id' => $site_module->id,
                        'language' => $site_module->name,
                        'icon' => $site_module->icon,
                        'ion' => $site_module->ion,
                        'location' => $site_module->location,
                        'review' => $site_module->review,
                        'status' => $site_module->status,
                    );
                }
                $gc_cache->save($gc_site_modules, $cacheID, array('site_modules'));
            }
            $CI->session->set_userdata('gc_site_modules', $gc_site_modules);
        }
        return $CI->session->userdata('gc_site_modules');
    }
}

if ( ! function_exists('gc_module_key')) {
    /**
     * Get specific key value of one or all modules from the Application Config
     * Except a specific module
     * @param string $key - Name of the Key
     * @param string/integer $module - Module Name or Module ID
     * @param string $key_type - Value to be searched, in case of module_id this will be id
     * @param string/integer $return - Default return if a value is not found
     * @return array
     */
    function gc_module_key($key = 'status', $module = null, $return = 0, $key_type = 'module') {
        $site_modules = array();
        $config_modules = gc_site_modules();
        $modules = array_column($config_modules, $key_type);
        if(!empty($module)) {
            // array_keys is faster
            $key_value = array_keys($modules, $module);
            if(!empty($key_value)) {
                return $config_modules[$key_value[0]][$key];
            }
            return $return;
        } else {
            foreach($modules as $module_key => $module_value) {
                $site_modules[$module_value] = $config_modules[$module_key][$key];
            }
            return $site_modules;
        }
    }
}

if ( ! function_exists('gc_modules_only')) {
    /**
     * Get all modules from the Application Config except a specific module
     * Except a specific module
     * @return array
     */
    function gc_modules_only($module = 'listings', $modules = array()) {
        if(empty($modules)) {
            $modules = gc_modules();
        }
        $key = array_search($module, $modules);
        if (false !== $key) {
            unset($modules[$key]);
        }
        return $modules;
    }
}

if( ! function_exists('gc_site_module')) {
    /**
     * Check if the module is a site module or not
     * @param string $module
     * @param string $key
     * @return boolean
     */
    function gc_site_module($module = 'listings', $key = 'site_module') {
        return gc_module_settings($module, $key) == 1;
    }
}

if ( ! function_exists('gc_field_value')) {
    /**
     * Get module field value e.g. module_limit, module_status etc.
     * @param string $module
     * @param string $field
     * @param string $return - 0 for checkboxes like auto approve
     */
    function gc_field_value($module, $field, $return = 0) {
        $result = (array)json_decode($field);
        $value = in_array($module, $result) || array_key_exists($module, $result) ? $result[$module] : $return;
        return $value;
    }
}

if ( ! function_exists('gc_module_settings')) {
    /**
     * Get module settings from the settings table
     * @param string $module
     * @param string $type - Null/Reviews/Images/Locations
     * @param string $return - 0 for checkboxes like auto approve
     */
    function gc_module_settings($module, $type = NULL, $return = 0) {
        if($type == NULL) {
            $key = 'status';
        } else {
            $key = ($type == 'locations' ? 'location' : 'review');
        }
        return gc_module_key($key, $module);
    }
}

if ( ! function_exists('gc_form_fields')) {
    /**
     * Extract text between two characters from a string
     * @param string $str
     * @param string $from
     * @param string $to
     * @param string $withFromAndTo
     * @return string
     */
    function gc_form_fields($fields, $search, $match = NULL) {
        $result = array();
        foreach($fields as $key => $field) {
            if(isset($field[$search]) && $field[$search] == $match) {
                $result[] = $field;
            }
        }
        return $result;
    }
}

if ( ! function_exists('clear_cache')) {
    /**
     * Clear Cache
     * @param string $cache_id
     */
    function clear_cache($cache_id = null, $purge_assets = false) {
        // Load Zend Caching
        $CI =& get_instance();
        $CI->load->library('gzf_cache', array('lifetime' => 900));
        $gc_cache = $CI->gzf_cache->get_instance();
        
        // Clean Zend Caching
        if(!empty($cache_id)) {
            /**
             * Tags are a way to categorize cache records. When you save a cache with the save() method,
             * you can set an array of tags to apply for this record.
             * Then you will be able to clean all cache records tagged with a given tag (or tags):
             * $cache->save($huge_data, 'myUniqueID', array('tagA', 'tagB', 'tagC'));
             * Note: note than the save() method accepts an optional fourth argument: $specificLifetime
             * (if != FALSE, it sets a specific lifetime for this particular cache record)
             */
            // https://framework.zend.com/manual/1.12/en/zend.cache.theory.html#zend.cache.clean
            $gc_cache->clean(Zend_Cache::CLEANING_MODE_MATCHING_ANY_TAG , array($cache_id));
            // Remove a specific Cache ID file
            //$gc_cache->remove('popular');
        } else {
            $gc_cache->clean(Zend_Cache::CLEANING_MODE_ALL);
        }
        
        // Purge all files under the assets/cache folder
        if($purge_assets) {
            Assets::clear_cache();
        }
    }
}

if ( ! function_exists('duration_conversion')) {
    /**
     * Theme Packages - Convert Duration to Months/Days
     * @param integer $duration
     * @param boolean $days_only - TRUE - Return days only
     * FALSE - Convert days to Months, Years using current date format
     * @param boolean $decimal_conversion - TRUE - Return in Decimals
     */
    function duration_conversion($duration, $convert_days = false, $decimal_conversion = false) {
        $result = $duration . ' ' . lang('bf_days');
        if($duration == 0) {
            $result = lang('label_lifetime');
        }
        if($convert_days) {
            $start_date = new DateTime(date("Y/m/d"));
            $end_date = new DateTime(date("Y/m/d", strtotime("+" . $duration . " days")));
            $dd = date_diff($start_date,$end_date);
            if($decimal_conversion) { // DECIMAL CONVERSION
                if($duration < 365 && $duration > 31) {
                    /*if($dd->d != 0) {
                     $result = $dd->m . ' ' . lang('gc_months');
                     }*/
                    $result = $dd->m . '.' . $dd->d . ' ' . lang('gc_months');
                } else if($duration >= 365) {
                    /*if($dd->m != 0 && $dd->y == 1) {
                     // $result = $dd->y . ' ' . lang('label_year');
                     $result = $dd->y . '.' . $dd->m . ' ' . lang('gc_years');
                     } else if($dd->m != 0) {
                     $result = $dd->y . ' ' . lang('gc_years');
                     }*/
                    $result = $dd->y . '.' . $dd->m . ' ' . lang('gc_years');
                }
            } else {
                $duration_format = array();
                if($dd->y != 0) {
                    if($dd->y > 1) {
                        $duration_format[] = $dd->y . ' ' . lang('gc_years');
                    } else {
                        $duration_format[] = $dd->y . ' ' . lang('label_year');
                    }
                }
                if($dd->m != 0) {
                    if($dd->m > 1) {
                        $duration_format[] = $dd->m . ' ' . lang('gc_months');
                    } else {
                        $duration_format[] = $dd->m . ' ' . lang('gc_month');
                    }
                }
                if($dd->d != 0) {
                    $duration_format[] = $dd->d . ' ' . lang('bf_days');
                }
                $result = implode(', ', $duration_format);
            }
        }
        return $result;
    }
}

if ( ! function_exists('split_datetime')) {
    /**
     * Events/News  - Get Months and Day Information from the Datetime Field
     * @param string $date
     * @return multitype:string
     */
    function split_datetime($date, $return = null) {
        // get the datetime from from the database and convert it accordingly        
        // $dateformat = explode('/', str_replace(' ', '/', $record->start_date));
        // $date = $dateformat[2] . '-' . $dateformat[1] . '-' . $dateformat[0];
        $day = date('d', strtotime($date));
        $month = date('M', strtotime($date));//date("F", mktime(0, 0, 0, date("m", strtotime($date)), 10));
        switch($return) {
            case 'day':
                return $day;
            case 'month':
                return $month;
            default:
                return array('day' => $day, 'month' => $month);
        }
    }
}

if ( ! function_exists('replace_querystring')) {
    /**
     * Replace/Remove a single variable value pair from querystring
     * @param string $url - Current URL with querystring
     * @param array $params - Replace single variable value pair from querystring
     * @param string $remove_param - Remove specific parameter from URL while preserving other parameters
     * @return multitype:string
     */
    function replace_querystring($params = array(), $remove_param = null, $url = null) {
        if(empty($url)) {
            $url = current_url();
            if(isset($_SERVER['QUERY_STRING'])) {
                $url .= '?' . $_SERVER['QUERY_STRING'];
            }
        }
        $query = parse_url($url, PHP_URL_QUERY);
        parse_str($query, $existing_params);
        if (empty($existing_params)) {
            return rtrim($url, '?') . '?' . http_build_query($params);
        } else if(!empty($remove_param)) {
            unset($existing_params[$remove_param]);
        }
        $params = array_merge($existing_params, $params);
        return preg_replace('#\?.*#', '?' . http_build_query($params), $url);
    }
}

if ( ! function_exists('object_sort')) {
    /**
     * Sort Object by a specific fields
     * @param object $object
     * @param string $sortBy - Field Name
     * @param string $direction - ASC, DESC
     * @return multitype:unknown
     */
    function object_sort($object, $sortBy, $direction='asc') {
        $sortedArray=array();
        $tmpArray=array();
        foreach($object as $obj) {
            $tmpArray[]=$obj->$sortBy;
        }
        if($direction=='asc') {
            asort($tmpArray);
        } else {
            arsort($tmpArray);
        }
        foreach($tmpArray as $k=>$tmp) {
            $sortedArray[]=$object[$k];
        }
        return $sortedArray;
    }
}

if ( ! function_exists('recursive_replace_all_between')) {
    /**
     * Delete all between certain character(s) in string
     *
     * This method will search given $content for $start and $end and,
     * if such has been found, clear $content from substring between these two characters,
     * including $start and $end itself.
     *
     * @param string $start - e.g {widget}
     * @param string $end - - e.g {/widget}
     * @param string $content
     * @param string $replacement
     * @return mixed
     */
    function recursive_replace_all_between($start, $end, $content, $replacement) {
        $str = getStringBetween($content, $start, $end);
        while ($str) {
            $content = str_replace($start . $str . $end, $replacement, $content);
            // Check for the next text if exists in content
            $str = getStringBetween($content, $start, $end);
        }
        return $content;
    }
}

if ( ! function_exists('valid_url')) {
    /**
     * Check whether a URL is valid or not
     * @param string $url
     * @return boolean
     */
    function valid_url($url) {
        $regexp =  "/((?:https?\:\/\/|www\.)(?:[-a-z0-9]+\.)*[-a-z0-9]+.*)/i";
        if(preg_match($regexp, $url)) {
            return true;
        }
        return false;
    }
}

if ( ! function_exists('slug_redirect')) {
    /**
     * Redirect user to the correct slug
     * @param string $module
     * @param string $slug
     * @return boolean
     */
    function slug_redirect($module, $record, $mobile = false, $segment = 2) {
        if($mobile) { // Do not redirect in case of Mobile App
            return true;
        }
        $city = (isset($record->city) ? $record->city : null);
        $actual_slug = gc_url_format($module, $record->slug, $record->id, $city);
        if($record && ($actual_slug != current_url())) {
            Template::redirect(gc_url_format($module, $record->slug, $record->id, $city));
        }
    }
}

/**********************************************************************
 * IMPORT/EXPORT - CRONJOBS & THEMES
 **********************************************************************/
if ( ! function_exists('read_directory')) {
    /**
     * Read extracted directory
     * @param string $path
     */
    function read_directory($path, $delimiter){
        $dirHandle = opendir($path);
        while($item = readdir($dirHandle)) {
            $newPath = $path . "/" . $item;
            if(is_dir($newPath) && $item != '.' && $item != '..') {
                read_directory($newPath, $delimiter); // Folder
            } else if(!empty($item) && $item != '.' && $item != '..') {
                // To avoid duplication of public_html folder, the php_version/folder name is also added with delimiter
                replace_file($newPath, $delimiter . '/application/');
                replace_file($newPath, $delimiter . '/goclixy/');
                replace_file($newPath, $delimiter . '/public_html/');
            }
        }
    }
}

if ( ! function_exists('replace_file')) {
    /**
     * Replace a file
     * @param string $new_file
     * @param string $delimiter - /application/, /public_html/
     */
    function replace_file($new_file, $delimiter) {
        if(strrpos($new_file, $delimiter) !== false) {
            // Get the string after a string from a string
            // https://stackoverflow.com/questions/4221117/get-the-string-after-a-string-from-a-string
            $pos = strpos($new_file, $delimiter);
            $substring = substr($new_file, $pos+strlen($delimiter), strlen($new_file)-1);
            $existing_file = FCPATH; // public_html folder
            if(strrpos($delimiter, '/application/') !== false) { // application folder
                $existing_file = APPPATH;
            } else if(strrpos($delimiter, '/goclixy/') !== false) { // goclixy folder
                $existing_file = BFPATH;
            }
            $existing_file .= $substring;
            // New files will also be copied.
            // Both file paths should not be same
            // Check whether the file exists on the server
            // if(strcmp($existing_file, $new_file) != 0 && file_exists($existing_file) && file_exists($new_file)) {
            if(strcmp($existing_file, $new_file) != 0 && file_exists($new_file)) {
                // Check whether the destination path exist or not
                // Copying language files must also create folders
                $existing_dir = dirname($existing_file);
                // if path does not exist, create folders recursively
                if (!file_exists($existing_dir)) {
                    mkdir($existing_dir, 0777, true);
                }
                // replace file
                rename($new_file, $existing_file);
            }
        }
    }
}

if ( ! function_exists('rrmdir')) {
    /**
     * Recursively delete a directory and all of it's contents - e.g.the equivalent of `rm -r` on the command-line.
     * Consistent with `rmdir()` and `unlink()`, an E_WARNING level error will be generated on failure.
     *
     * @param string $source absolute path to directory or file to delete.
     * @param bool   $removeOnlyChildren set to true will only remove content inside directory.
     *
     * @return bool true on success; false on failure
     */
    function rrmdir($dirPath) {
        if (is_dir($dirPath)) {
            $handle = opendir($dirPath); // Important - trying to remove a directory that php is still using.
            $dirPath = str_replace('\\', '/', $dirPath);
            $objects = scandir($dirPath);
            foreach ($objects as $object) {
                // $object = stripslashes($object); // Important for windows
                if ($object != "." && $object !="..") {
                    if (filetype($dirPath . DIRECTORY_SEPARATOR . $object) == "dir") {
                        rrmdir($dirPath . DIRECTORY_SEPARATOR . $object);
                    } else {
                        unlink($dirPath . DIRECTORY_SEPARATOR . $object);
                    }
                }
            }
            reset($objects);
            // avoid error messages on windows
            /*$remaining_objects = scandir($dirPath); // again scan the directory
             $search_var = array('.', '..');
             foreach($search_var as $blank_folder) {
             if (($key = array_search($blank_folder, $remaining_objects)) !== false) {
             unset($remaining_objects[$key]); // unlink . and ..
             }
             }
             if(empty($objects)) { // do not delete a directory if there are still contents
             rmdir($dirPath);
             }*/
            closedir($handle); // Important - Delete the directory only after completion of the inner folders and files removal
            rmdir($dirPath);
        }
    }
}

if( ! function_exists('file_copy')) {
    /**
     * Copy file if destination does not exist
     * @param string $source
     * @param string $destination
     */
    function file_copy($source, $destination) {
        $path = pathinfo($destination);
        if (!file_exists($path['dirname'])) {
            mkdir($path['dirname'], 0777, true);
        }
        if (!copy($source,$destination)) {
            echo "copy failed \n";
        }
    }
}

if( ! function_exists('xcopy')) {
    /**
     * Copy a file, or recursively copy a folder and its contents
     * @author      Aidan Lister <aidan@php.net>
     * @version     1.0.1
     * @link        http://aidanlister.com/2004/04/recursively-copying-directories-in-php/
     * @param       string   $source    Source path
     * @param       string   $dest      Destination path
     * @param       int      $permissions New folder creation permissions
     * @return      bool     Returns true on success, false on failure
     */
    function xcopy($source, $dest, $permissions = 0755) {
        // Check for symlinks
        if (is_link($source)) {
            return symlink(readlink($source), $dest);
        }
        
        // Simple copy for a file
        if (is_file($source)) {
            return copy($source, $dest);
        }
        
        // Make destination directory
        if (!is_dir($dest)) {
            mkdir($dest, $permissions);
        }
        
        // Loop through the folder
        $dir = dir($source);
        while (false !== $entry = $dir->read()) {
            // Skip pointers
            if ($entry == '.' || $entry == '..') {
                continue;
            }
            
            // Deep copy directories
            xcopy("$source/$entry", "$dest/$entry", $permissions);
        }
        
        // Clean up
        $dir->close();
        return true;
    }
}

if( ! function_exists('create_zip')) {
    /**
     * Create a Zip Archieve of a folder
     * @param string $source
     * @param string $destination
     * @return boolean
     */
    function create_zip($the_folder, $zip_file_name) {
        $za = new FlxZipArchive;
        $res = $za->open($zip_file_name, ZipArchive::CREATE);
        if($res === TRUE) {
            $za->addDir($the_folder, basename($the_folder));
            $za->close();
        } else {
            echo 'Could not create a zip archive';
        }
    }
}

if( ! function_exists('extract_zip')) {
    /**
     * Extract zip file in a folder
     * @param string $zipFile
     * @param string $delimiter - Separator between files e.g. themes/public_html/uploads/imports, themes is the separator
     */
    function extract_zip($zipFile, $delimiter = '') {
        /* Open the Zip file */
        $zip = new ZipArchive;
        $extractPath = FCPATH . time() . rand(10, 10000); // The name of the directory to extract files.
        
        // Check if the directory already exists.
        if(!is_dir($extractPath)) {
            //Directory does not exist, so lets create it.
            mkdir($extractPath, 0777);
        }
        if($zip->open($zipFile) != "true") {
            Template::set_message(lang('uvf_error_zip'), 'danger');
            Template::redirect(current_url());
        }
        
        /* Extract Zip File */
        $zip->extractTo($extractPath);
        $zip->close();
        
        // Get all directories and files information within the extracted folder
        // and use the rename command to replace them
        read_directory($extractPath, $delimiter);
        
        // Unlink zip file
        unlink($zipFile);
        
        // Delete extracted files
        rrmdir($extractPath);
    }
}

if(!function_exists('ordered_item_info')) {
    /**
     * Ordered Item Information
     *
     * Data is stored in the json format in the orders table
     *
     * @param string $item
     */
    function ordered_item_info($item, $order_type = 0, $return = 'title') {
        $item = json_decode($item);
        $html = '';
        if(!empty($item) && $order_type == 8) {
            $item_titles = array();
            // multiple items
            foreach($item as $item_info) {
                $item_titles[] = $item_info->title;
            }
            $html = implode(', ', $item_titles);
        } else if(!empty($item)) {
            if($return == 'title') {
                $html = $item->name;
            } else {
                $html = preg_replace('/(<\/[^>]+?>)(<[^>\/][^>]*?>)/', '$1 $2', $item->description);
            }
        }
        return $html;
    }
}

if(!function_exists('getHeaders')) {
    /**
     * http://www.w3bees.com/2013/09/download-file-from-remote-server-with.html
     * This function gets all header information before downloading a file.
     * This function will help you to know file is exist or not and filesize.
     * @param str #url
     * @return array
     */
    function getHeaders($url) {
        $ch = curl_init($url);
        curl_setopt( $ch, CURLOPT_NOBODY, true );
        curl_setopt( $ch, CURLOPT_RETURNTRANSFER, false );
        curl_setopt( $ch, CURLOPT_HEADER, false );
        curl_setopt( $ch, CURLOPT_FOLLOWLOCATION, true );
        curl_setopt( $ch, CURLOPT_MAXREDIRS, 3 );
        curl_exec( $ch );
        $headers = curl_getinfo( $ch );
        curl_close( $ch );
        
        return $headers;
    }
}

if(!function_exists('curl_download')) {
    /**
     * Download a file from the remote server with PHP
     * @param string $url, $path
     * @return bool || void
     */
    function curl_download($url, $path) {
        # open file to write
        $fp = fopen ($path, 'w+');
        # start curl
        $ch = curl_init();
        curl_setopt( $ch, CURLOPT_URL, $url );
        # set return transfer to false
        curl_setopt( $ch, CURLOPT_RETURNTRANSFER, false );
        curl_setopt( $ch, CURLOPT_BINARYTRANSFER, true );
        curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, true );
        # increase timeout to download big file
        curl_setopt( $ch, CURLOPT_CONNECTTIMEOUT, 10 );
        # write data to local file
        curl_setopt( $ch, CURLOPT_FILE, $fp );
        # https redirect issue
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
        # execute curl
        curl_exec( $ch );
        # close curl
        curl_close( $ch );
        # close local file
        fclose( $fp );
        
        if (filesize($path) > 0) return true;
    }
}

if(!function_exists('curl_marketplace')) {
    /**
     * Read a URL from the marketplace with PHP
     * @param string $uri_string - URL
     * @param boolean $url - Use Another URL
     * @return bool || void
     */
    function curl_marketplace($uri_string, $postfields = array(), $url = null) {
        switch($uri_string) {
            case 'revision':
            case 'messages':
                // $download_id = settings_item('site.download_id');
                $url = 'https://gc-licensing-default-rtdb.firebaseio.com/' . $uri_string . '.json'; // ?orderBy=%22$key%22&equalTo=%22' . $download_id . '%22
                $ch = curl_init($url);
                curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                $response = curl_exec($ch);
                // Check for cURL errors
                /*if (curl_errno($ch)) {
                    echo 'Curl error: ' . curl_error($ch);
                }*/
                curl_close($ch);
                if($response) {
                    $result = json_decode($response, true);
                    if(empty($result['error'])) {
                        return (object) $result;
                    } else {
                        return false;
                    }
                    /*$result = json_decode($response, true);
                    if(!empty($result[$download_id])) {
                        $url = rtrim(site_url(), "/");
                        $url = preg_replace('#^https?://#', '', $url); // Remove "http://" or "https://"
                        $url = preg_replace('#^www\.#', '', $url); // Remove "www."
                        $parsedUrl = parse_url($url); // Extract the domain using parse_url
                        // Check if the 'host' key exists in the parsed URL
                        if (isset($parsedUrl['host'])) {
                            $url = $parsedUrl['host'];
                        }
                        if(in_array($url, $result[$download_id])) {
                            return true;
                        }
                    }*/
                }
                return false;
                break;
            default:
                $product_id = 1;
                $user = 'marketplace@goclixy.com';
                $phrase = 'mkt2018!@#';
                if(empty($url)) {
                    $url = 'https://www.goclixy.com/rest/' . $uri_string;
                }
                if(empty($url) && empty($postfields)) {
                    $url .= '?id=' . $product_id;
                }
                $ch = curl_init();
                curl_setopt($ch, CURLOPT_URL, $url);
                curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
                if(!empty($postfields)) {
                    curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
                    if(empty($url)) {
                        curl_setopt($ch, CURLOPT_USERPWD, "$user:$phrase");
                    }
                    curl_setopt($ch, CURLOPT_POSTFIELDS, $postfields);
                }
                $result = curl_exec($ch);
                curl_close($ch);
                if($result && is_string($result)) {
                    $result = json_decode($result, true);
                }
                return $result;
        }
    }
}

if(!function_exists('process_value')) {
    /**
     * Process Meta Field Checkbox or other Array Field Values
     * @param string $value
     */
    function process_value($value) {
        $values_array = json_decode($value, true);
        if(!empty($values_array) && is_array($values_array)) {
            $saved_values = array();
            foreach($values_array as $key => $value):
            // $saved_values[] = '<li>' . lang('label_' . $key) . '</li>';
            $saved_values[] = '<li>' . $key . '</li>';
            endforeach;
            return '<ul>' . implode(PHP_EOL, $saved_values) . '</ul>';
        } else {
            return $value;
        }
    }
}

if(!function_exists('ip_in_range')) {
    /**
     * Check if a given ip is in a network
     * @param  string $ip    IP to check in IPV4 format eg. 127.0.0.1
     * @param  string $range IP/CIDR netmask eg. 127.0.0.0/24, also 127.0.0.1 is accepted and /32 assumed
     * @return boolean true if the ip is in this range / false if not.
     */
    function ip_in_range( $ip, $range ) {
        if ( strpos( $range, '/' ) == false ) {
            $range .= '/32';
        }
        // $range is in IP/CIDR format eg 127.0.0.1/24
        list( $range, $netmask ) = explode( '/', $range, 2 );
        $range_decimal = ip2long( $range );
        $ip_decimal = ip2long( $ip );
        $wildcard_decimal = pow( 2, ( 32 - $netmask ) ) - 1;
        $netmask_decimal = ~ $wildcard_decimal;
        return ( ( $ip_decimal & $netmask_decimal ) == ( $range_decimal & $netmask_decimal ) );
    }
}

if(!function_exists('valid_link')) {
    /**
     * Append HTTP/HTTPS with Website Redirect Link
     * @param string $link
     * @return string
     */
    function valid_link($link) {
        if(substr($link, 0, 4) == "www.") {
            return 'http://' . $link;
        } else {
            return $link;
        }
    }
}

if(!function_exists('gc_click_to_call')) {
    /**
     * Listing Detail Page - Click to Call Setup
     * @param number $phone_number
     * @param number $mobile_number
     * @param string $phone_icon
     * @param string $mobile_icon
     * @return string
     */
    function gc_click_to_call($phone_number, $mobile_number, $phone_icon = '', $mobile_icon = '') {
        $html_output = '';
        if(!empty($phone_number) && !empty($mobile_number)) {
            $html_output = $mobile_icon . $mobile_number . ', ' . $phone_icon . '<span itemprop="telephone">' . $phone_number .'</span>';
        } elseif (!empty($mobile_number)) {
            $html_output = $mobile_icon . '<span itemprop="telephone">' . $mobile_number .'</span>';
        } else {
            $html_output = $phone_icon . '<span itemprop="telephone">' . $phone_number .'</span>';
        }
        return $html_output;
    }
}

if(!function_exists('calculate_savings')) {
    /**
     * Calculate Savings
     * @param float $price
     * @param float $offer_price
     * @return number[]
     */
    function calculate_savings($price, $offer_price, $class = 'package-savings') {
        $result = array('html' => '');
        if($price > $offer_price && !empty($offer_price)) {
            $savings = $price - $offer_price;
            $savings_in_percentage = round(($savings * 100)/($price));
            $result['price'] = currency_format($price);
            $result['savings'] = currency_format($savings);
            $result['savings_in_percentage'] = $savings_in_percentage;
            $result['html'] = '<span class="' . $class . '"><del class="ml-1 mr-1 small">' . currency_format($price) . '</del><span class="small">' . $savings_in_percentage . '% ' . lang('label_off') . '</span></span>';
        }
        return $result;
    }
}

if( ! function_exists('phone_codes')) {
    /**
     * Get Phone Codes
     * @param string $iso - Get country code of a specific country
     * e.g. phone_codes(visitor_info()) - Get Visitors Country Code
     */
    function phone_codes($iso = null, $is_optional = false) {
        $CI =& get_instance();
        $CI->load->library('gzf_cache', array('lifetime' => 20000));
        $gc_cache = $CI->gzf_cache->get_instance();
        $cacheID = 'country_phone_codes';
        // if(!$result = $gc_cache->load($cacheID)) {
        $phone_codes = array();
        $country_phone_codes = array(); // for country specific loading
        $records = $CI->db->select('iso, phone_code')->where('phone_code != 0')->order_by('phone_code')->get('countries')->result();
        if($records) {
            foreach($records as $record) {
                $phone_codes[$record->phone_code] = '+' . $record->phone_code;
                $country_phone_codes[$record->iso] = $record->phone_code;
            }
        }
        $result = array('phone_codes' => $phone_codes, 'country_phone_codes' => $country_phone_codes);
        $gc_cache->save($result, $cacheID, array('countries'));
        //}
        if(!empty($iso)) {
            return  isset($result['country_phone_codes'][$iso]) ? $result['country_phone_codes'][$iso] : 0;
        } else {
            $phone_codes = $result['phone_codes'];
            if($is_optional) {
                $phone_codes[0] = lang('placeholder_select');
                ksort($phone_codes);
            }
            return $phone_codes;
        }
    }
}

if( ! function_exists('visitor_info')) {
    /**
     * Get Visitors Info e.g. country code
     * @param string $field
     */
    function visitor_info($field = 'country_abbrev') {
        $CI =& get_instance();
        $visitor_info = $CI->session->userdata('visitor_info');
        if(!empty($visitor_info[$field])) {
            return $visitor_info[$field];
        } else {
            return null;
        }
    }
}

if( ! function_exists('previousUrl')) {
    /**
     * Store/Retrieve the previously viewed page's complete URL.
     * @param boolean $is_save
     * @return string
     */
    function previousUrl($is_save = false) {
        $CI =& get_instance();
        if (! class_exists('CI_Session', false)) {
            $CI->load->library('session');
        }
        if($is_save) {
            $CI->session->set_userdata('previous_url', current_url() . (isset($_SERVER['QUERY_STRING']) ? $_SERVER['QUERY_STRING'] : ''));
        } else {
            return $CI->session->userdata('previous_url') ? $CI->session->userdata('previous_url') : current_url();
        }
    }
}

if ( ! function_exists('gc_sub_categories')) {
    /**
     * Explode sub_categories redirect URI and return parameters
     * $separator = "!~!";
     * @param string $redirect_url
     * @param string $return - requested return
     * @return array
     */
    function gc_sub_categories($redirect_url, $params = array(), $module = 'listings') {
        $result = array();
        if(!empty($redirect_url)) {
            $sub_categories = explode('###', $redirect_url);
            foreach($sub_categories as $sub_category) {
                $uri_array = explode('!~!', $sub_category);
                $id = isset($uri_array[0]) ? (int) $uri_array[0] : 0;
                $name = isset($uri_array[1]) ? $uri_array[1] : NULL;
                $slug = isset($uri_array[2]) ? $uri_array[2] : NULL;
                $counts = isset($uri_array[3]) ? (int) $uri_array[3] : NULL;
                $status = isset($uri_array[4]) ? $uri_array[4] : NULL;
                $result[] = array(
                    'id' => $id,
                    'name' => $name,
                    'slug' => $slug,
                    'status' => $status,
                    'counts' => $counts,
                    'url' => gc_category_url($module, $slug, $id),
                    'link' => ($status == 1 ? '<a href="' . gc_category_url($module, $slug, $id) . '" title="' . $name . '>' . $name . ' <span class="text-muted">(' . $counts .  ')</span></a>' : $name)
                );
            }
        }
        return $result;
    }
}

if(! function_exists('user_fields')) {
    /**
     * Fields take from the users table - Forums, Blogs, Tickets and other Modules
     * @return string
     */
    function user_fields($prefix = 'U') {
        $prefix = $prefix . '.';
        $sql = $prefix . "display_name, " . $prefix . "username, " . $prefix . "email AS user_email, " . $prefix . "profile_image, " . $prefix . "slug AS user_slug, " . $prefix . "is_online, " . $prefix . "created_on AS join_date, CONCAT(" . $prefix . "id, '!~!', " . $prefix . "username, '!~!', " . $prefix . "email, '!~!', " . $prefix . "display_name, '!~!', " . $prefix . "created_on) AS profile_user";
        return $sql;
    }
}

if(! function_exists('listing_fields')) {
    /**
     * Fields take from the listings table
     * @return string
     */
    function listing_fields($prefix = 'L') {
        $prefix = $prefix . '.';
        $sql = $prefix . "title AS business_title, " . $prefix . "slug AS business_slug, " . $prefix . "city AS business_city, " . $prefix . "phone_number AS business_phone_number, " . $prefix . "email AS business_email, " . $prefix . "logo AS business_logo, " . $prefix . "cover AS business_cover, " . $prefix . "ratings AS business_rating";
        return $sql;
    }
}

if ( ! function_exists('gc_json_settings')) {
    /**
     * Get settings saved in json format.
     * @param string $name - Name of the setting e.g. blog.settings
     * @param string $field
     * @param string $return - 0 for checkboxes like auto approve
     */
    function gc_json_settings($name, $key = null, $return = 0) {
        $result = json_decode(settings_item($name), true);
        if(!empty($key)) {
            return !empty($result[$key]) ? $result[$key] : $return;
        } else {
            return $result;
        }
    }
}

if(!function_exists('product_fields')) {
    /**
     * Fields taken from the products table - Releases and Licenses
     * @return string
     */
    function product_fields($prefix = 'P') {
        $prefix = $prefix . '.';
        $sql = $prefix . "title AS product_name, CONCAT(" . $prefix . "id, '!~!', " . $prefix . "title, '!~!', " . $prefix . "slug, '!~!', " . $prefix . "status, '!~!products') AS redirect_uri";
        return $sql;
    }
}


if (!function_exists('exec_background')) {
    /**
     * Execute command in background
     * Note: Data is passed via file_put_contents and retrieved via file_get_contents
     * The file path is in the application/tmp folder.
     * Do not forget to update the public_html/index.php file. The vendor path must be set to real path e.g. /home/fyond/vendor/autoload.php
     * @param string $cmd
     *
     * @param string $php_path: /usr/bin/nohup php: spawn a background process to perform firebase calls asynchronously.
     * Please notice the use of nohup - to run a command in Unix without getting killed as soon as the function completes running
     * If everything is in right place, this implementation will now create a new thread in the background to send curl request to firebase, without waiting for the HTTP response.
     */
    function exec_background($method, $php_path = 'php', $nohup = false, $path_windows = 'e:\\xampp\\php\\') {
        if (substr(php_uname(), 0, 7) == "Windows") {
            $cmd = $path_windows . $php_path . ' ' . FCPATH . 'index.php background ' . $method;
            pclose(popen("start /B ". $cmd, "r"));
        } else{
            if($nohup) {
                $php_path = '/usr/bin/nohup php';
            }
            $cmd = $php_path . ' ' . FCPATH . 'index.php background ' . $method;
            exec($cmd . " > /dev/null &");
        }
    }
}


if(!function_exists('query_fields')) {
    /**
     * Fields taken from the blogs/forums and other tables
     * @return string
     */
    function query_fields($module, $prefix = 'M') {
        $prefix = $prefix . '.';
        switch($module) {
            case 'products':
                $sql = "CONCAT(" . $prefix . "id, '!~!', " . $prefix . "name, '!~!', " . $prefix . "slug, '!~!', " . $prefix . "status_id, '!~!" . $module . "') AS redirect_uri";
                break;
            case 'blogs':
                $sql = "CONCAT(" . $prefix . "id, '!~!', " . $prefix . "title, '!~!', " . $prefix . "slug, '!~!', " . $prefix . "status_id, '!~!" . $module . "') AS redirect_uri";
                break;
            case 'subscribers':
                $sql = "CONCAT(" . $prefix . "id, '!~!', " . $prefix . "name, '!~!', " . $prefix . "email, '!~!', " . $prefix . "status_id, '!~!" . $module . "') AS redirect_uri";
                break;
            case 'threads':
            case 'posts':
                $sql = "CONCAT(" . $prefix . "id, '!~!', " . $prefix . "subject, '!~!', " . $prefix . "slug, '!~!', " . $prefix . "status_id, '!~!" . $module . "') AS redirect_uri";
                break;
        }
        return $sql;
    }
}

if( ! function_exists('explode_group_concat')) {
    /**
     * Explode MySQL Group Concatenated Data
     * E.g. display categories with listings
     * @param string $column_data
     */
    function explode_group_concat($data) {
        $result = array();
        $records = explode('~~~', $data);
        if(!empty($records)) {
            foreach($records as $record) {
                $row = explode('!~!', $record);
                $result[] = array(
                    'id' => $row[0],
                    'title' => $row[1],
                    'slug' => !empty($row[2]) ? $row[2] : null,
                    'icon' => !empty($row[3]) ? $row[3] : null
                );
            }
        }
        return $result;
    }
}

if(! function_exists('prepare_urls')) {
    /**
     * Multiple Links e.g. product_ids, online_users_info
     * Columns are concatenated in MySQL queries
     * ~~~ is the field separator
     * e.g. 1!~!Testing Product!~!testing-product
     * id - 0, name - 1, slug - 2, status - 3, module_name - 4
     * module name is added because module_id field is different for a specific row
     * @param string $url = e.g. https://www.goclixy.com/admin/components/products/form/product_brands/%s
     * @return string
     */
    function prepare_urls($column_data, $url = null, $separator = '<br />', $emmet_string = null, $role = 'admin', $extra = array()) {
        if(!empty($column_data)) {
            $urls = array();
            // multiple links separator
            $fields_array = explode('~~~', $column_data);
            if(!empty($fields_array)) {
                foreach($fields_array as $field_array) {
                    $column_values = explode('!~!', $field_array); // column value separator
                    $url_title = $column_values[0]; // row_id
                    if(isset($column_values[1])) { // id - 0, name - 1, slug - 2
                        if(!empty($extra['redirect_icon'])) {
                            $url_title .= ' - ' . $column_values[1] . '<i class="fas fa-external-link-alt fa-fw ml-1"></i>'; // row title/name
                        } else {
                            $url_title .= ' - ' . $column_values[1]; // row title/name
                        }
                    }
                    $module = isset($column_values[4]) ? $column_values[4] : null;
                    if(empty($url) && ($module == 'products' || $module == 'blogs')) {
                        $url = gc_url_format($module, '%s', '%s');
                    }
                    $cover = !empty($column_values[5]) ? gc_image_url($column_values[5]) : null;
                    switch($module) {
                        case 'variants':
                            $url = gc_url_format('products', '%s', '%s');
                            $link_url = !empty($column_values[3]) && $column_values[3] == 1 ? anchor(sprintf($url, $column_values[2], $column_values[0]), $url_title, 'target="_blank"') . '<span class="small d-block">' . $column_values[6] . ' - ' . $column_values[7] . '</span>' : $url_title;
                            break;
                        case 'products':
                            $url = gc_url_format($module, '%s', '%s');
                            $link_url = !empty($column_values[3]) && $column_values[3] == 2 ? anchor(sprintf($url, $column_values[2], $column_values[0]), $url_title, 'target="_blank"') : $url_title;
                            break;
                        case 'blogs':
                            $url = gc_url_format('blogs/detail', '%s');
                            $link_url = !empty($column_values[3]) && $column_values[3] == 2 ? anchor(sprintf($url, $column_values[2], $column_values[0]), $url_title, 'target="_blank"') : $url_title;
                            break;
                        case 'threads':
                        case 'posts':
                            // summary
                            $url = gc_url_format('forums/' . ($module == 'threads' ? 'thread' : 'post'), '%s');
                            $link_url = !empty($column_values[3]) && $column_values[3] == 2 ? anchor(sprintf($url, $column_values[2], $column_values[0]), $url_title, 'target="_blank"') : $url_title;
                            break;
                        case 'subscribers':
                            $link_url = anchor(site_url(SITE_AREA . '/components/subscribers/form/users/edit/' . $column_values[0]), $url_title, 'target="_blank"');
                            break;
                        case 'subscriber_messages':
                            $link_url = anchor(site_url(SITE_AREA . '/components/emailer/sbl/templates/all/0/' . $column_values[0]), $url_title, 'target="_blank"');
                            break;
                        default:
                            $link_url = anchor(sprintf($url, $column_values[2], $column_values[0]), $url_title, 'target="_blank"');
                    }
                    if(!empty($emmet_string)) {
                        $link_url = emmet($emmet_string, $link_url);
                    }
                    $urls[] = $link_url;
                }
                if(!empty($urls)) {
                    return implode($separator ?? '', $urls);
                }
            }
        }
        return null;
    }
}

if(!function_exists('progress_bar')) {
    /**
     * Display Profile Completion Progress Bar in different colors
     * @param number $value
     */
    function progress_bar($module, $id, $value, $class_only = false, $role = 'admin') {
        switch (true) {
            case  ($value < 20):
                $class = 'danger';
                break;
            case  ($value < 40):
                $class = 'warning text-secondary';
                break;
            case  ($value < 60):
                $class = 'info';
                break;
            case  ($value < 80):
                $class = 'primary';
                break;
            default:
                $class = 'success';
        }
        if($class_only) {
            return $class;
        } else {
            $lang = lang('percent_completion');
            if($role == 'admin') {
                $output = <<<EOL
<a href="javascript();">
<div class="progress percent-completion mt-2" data-tooltip="tooltip" data-title="{$lang}" data-id="{$id}" data-toggle="modal" data-target="#profileCompletion">
  <div class="progress-bar bg-{$class}" style="width:{$value}%">{$value}%</div>
</div>
</a>
<div id="percentCompletion" class="modal fade" role="dialog">
  <div class="modal-dialog">
    <div class="modal-content">
      <div class="modal-header">
        <h4 class="modal-title completion-heading"></h4>
        <button type="button" class="close" data-dismiss="modal">&times;</button>
      </div>
      <div class="modal-body">
        <div class="completion-content"></div>
      </div>
    </div>
  </div>
</div>
EOL;
            } else {
                $CI =& get_instance();
                $CI->load->model('groups/groups_model');
                $result = $CI->groups_model->percent_completion($module, $id, $role, true);
                $heading = $result['heading'];
                $data = $result['output'];
                $output = <<<EOL
<div class="progress" data-toggle="modal" data-target="#percentCompletion">
  <div class="progress-bar bg-{$class}" style="width:{$value}%">{$value}%</div>
</div>
<div id="moduleCompletion" class="modal fade" role="dialog">
  <div class="modal-dialog">
    <div class="modal-content">
      <div class="modal-header">
        <h4 class="modal-title">{$lang}</h4>
        <button type="button" class="close" data-dismiss="modal">&times;</button>
      </div>
      <div class="modal-body">
        {$data}
      </div>
    </div>
  </div>
</div>
EOL;
            }
        }
        return $output;
    }
}

if ( ! function_exists('parse_bbcode')) {
    /**
     * parse_bbcode
     *
     * Converts BBCode style tags into basic HTML
     *
     * @access public
     * @param string unparsed string
     * @param int max image width
     * @return string
     */
    function parse_bbcode($str = '', $max_images = 0){
        // Max image size eh? Better shrink that pic!
        if($max_images > 0):
        $str_max = "style=\"max-width:".$max_images."px; width: [removed]this.width > ".$max_images." ? ".$max_images.": true);\"";
        endif;
        $find = array(
            "'\[b\](.*?)\[/b\]'is",
            "'\[i\](.*?)\[/i\]'is",
            "'\[u\](.*?)\[/u\]'is",
            "'\[s\](.*?)\[/s\]'is",
            "'\[img\](.*?)\[/img\]'i",
            "'\[url\](.*?)\[/url\]'i",
            "'\[url=(.*?)\](.*?)\[/url\]'i",
            "'\[link\](.*?)\[/link\]'i",
            "'\[link=(.*?)\](.*?)\[/link\]'i"
        );
        $replace = array(
            '<strong>\\1</strong>',
            '<em>\\1</em>',
            '<u>\\1</u>',
            '<s>\\1</s>',
            '<img src="\\1" alt="" />',
            '<a href="\\1">\\1</a>',
            '<a href="\\1">\\2</a>',
            '<a href="\\1">\\1</a>',
            '<a href="\\1">\\2</a>'
        );
        return preg_replace($find, $replace, $str);
    }
}

if( ! function_exists('theme_settings')) {
    /**
     * Get Theme Settings
     * @param boolean $clear_session - if true, clear the session variable
     * @param string $theme - Theme to be processed, default value is null
     * Pass this value while resetting a theme
     */
    function theme_settings($theme = null, $clear_session = false) {
        $CI =& get_instance();
        if($CI->session->userdata('theme_demo') && empty($theme) && settings_item('site.demo_mode') == 1) {
            // load theme specific headers and footers
            $theme = $CI->session->userdata('theme_demo')['theme'];
        }
        $CI->load->model('themes/themes_model');
        $theme_settings = $CI->themes_model->get_theme_options(null, $theme);
        $default_theme_settings = json_decode(settings_item('theme.settings'), true);
        if(empty($theme_settings['navbar'])) { // get default navbar
            $theme_settings['navbar'] = $default_theme_settings['navbar'];
        }
        if(empty($theme_settings['footer'])) { // get default footer
            $theme_settings['footer'] = $default_theme_settings['footer'];
        }
        return $theme_settings;
    }
}

if( ! function_exists('generateRandomString')) {
    /**
     * PHP Random String Generator
     * @param number $length
     * @return string
     */
    function generateRandomString($length = 10) {
        $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $charactersLength = strlen($characters);
        $randomString = '';
        for ($i = 0; $i < $length; $i++) {
            $randomString .= $characters[rand(0, $charactersLength - 1)];
        }
        return $randomString;
    }
}

if ( ! function_exists('gc_demo_mode')) {
    /**
     * Check Demo Mode
     * Redirect user to a specific page after a specific action
     * @param
     */
    function gc_demo_mode($data = array()) {
        if(settings_item('site.demo_live') == 0) {
            return true;
        }
        if(empty($data['no_message'])) {
            Template::set_message(lang('site_demo_live'), 'danger');
        }
        if(empty($data['no_redirect'])) {
            if(!empty($data['redirect_url'])) {
                $redirect_url = $data['redirect_url'];
            } else if($data['dashboard_redirect']) {
                $redirect_url = site_url(SITE_AREA . '/dashboard/summary');
            } else {
                if(!empty($data['redirect'])) {
                    switch($data['redirect']) {
                        case 'home':
                            $redirect_url = site_url();
                            break;
                        case 'dashboard':
                            $redirect_url = site_url(SITE_AREA . '/dashboard/summary');
                            break;
                        case 'emailer':
                            $redirect_url = site_url(SITE_AREA . '/components/emailer/summary');
                            break;
                        case 'builder':
                            $redirect_url = site_url(SITE_AREA . '/developer/builder');
                            break;
                        case 'roles':
                            $redirect_url = site_url(SITE_AREA . '/settings/roles');
                            break;
                        case 'permissions':
                            $redirect_url = site_url(SITE_AREA . '/settings/permissions');
                            break;
                        case 'users':
                            $redirect_url = site_url(SITE_AREA . '/settings/users');
                            break;
                        case 'database':
                            $redirect_url = site_url(SITE_AREA . '/developer/database');
                            break;
                        case 'logs':
                            $redirect_url = site_url(SITE_AREA . '/developer/logs');
                            break;
                        case 'fields':
                            $redirect_url = site_url(SITE_AREA . '/developer/fields');
                            break;
                        case 'current':
                            $redirect_url = current_url();
                            break;
                    }
                } else {
                    $redirect_url = current_url();
                }
            }
            Template::redirect($redirect_url);
        }
        // render a demo layout to display message to the user
        // e.g. form - image uploader
        if(!empty($data['render'])) {
            Template::set('toolbar_title', lang('title_demo_mode'));
            Template::set_view('common/_demo_mode');
            Template::render();
        }
        return false;
    }
}

if ( ! function_exists('site_logo')) {
    /**
     * Demo specific logo
     * @param string $css: Not NULL, Display Image
     * @param string $logo: logo/logo_contrast
     * @return boolean|string
     */
    function site_logo($css = null, $key = 'logo') {
        $logo = settings_item('site.' . $key);
        if(settings_item('site.demo_mode') == 1) {
            $CI =& get_instance();
            $theme_demo = $CI->session->userdata('theme_demo');
            $CI->load->model('themes/themes_model');
            $theme_logo = $CI->themes_model->get_theme_options($key, !empty($theme_demo['theme']) ? $theme_demo['theme'] : null);
            if(!empty($theme_logo)) {
                $logo = $theme_logo;
            }
        }
        $logo = gc_image_url($logo, false);
        if(empty($css)) {
            return $logo;
        } else {
            $logo_image = '<img src="' . $logo . '" class="' . $css . '" alt="' . settings_item('site.title') . '">';
            return $logo_image;
        }
    }
}

if( ! function_exists('gc_rand')) {
    /**
     * Generate and verify hash
     * @param string $hash
     * @param string $str
     */
    function gc_rand($hash, $str) {
        return hash($hash, $str);
    }
}


if( ! function_exists('gc_process_params')) {
    /**
     * Process Params Column Data
     * @param array $record
     * @param string $key
     * @return array
     */
    function gc_process_params($record, $key = 'params') {
        $params = array();
        $saved_params = !empty($record->{$key}) ? json_decode($record->{$key}, true) : array();
        if(!empty($saved_params)) {
            foreach($saved_params as $key => $saved_param) {
                if(array_key_exists('key', $saved_param)) {
                    if(!empty($saved_param['key'])) {
                        $params[$saved_param['key']] = $saved_param['value'];
                    }
                } else {
                    $params[$key] = $saved_param;
                }
            }
        }
        return $params;
    }
}

if ( ! function_exists('api_goclixy')) {
    /**
     * Get data from Goclixy API
     * @param string $params
     */
    function api_goclixy($params) {
        $url = 'https://api.goclixy.com/rest/' . $params;
        $user = base64_decode('YXBpMjAxOUBnb2NsaXh5LmNvbQ==');
        $phrase = base64_decode('MTlGZWIxOUdAQ2whQCN4eQ==');
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL,$url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER,1);
        curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
        curl_setopt($ch, CURLOPT_USERPWD, "$user:$phrase");
        $records = curl_exec($ch);
        curl_close($ch);
        return $records;
    }
}

if ( ! function_exists('process_images')) {
    /**
     * Process Images
     * @param string $images
     * @param string $title - Default Title
     * @param number $type_id
     */
    function process_images($images, $title, $type_id = 0) {
        $result = array();
        $images = is_string($images) ? json_decode($images, true) : $images;
        if(!empty($images)) {
            foreach($images as $image) {
                $image_type_id = $image['type_id'];
                $image_title = !empty($image['title']) ? $image['title'] : $title;
                if(!empty($image['status'])) {
                    switch($type_id) {
                        case 4: // return gallery and cover images
                            if($image_type_id == 0 || $image_type_id == 1) {
                                $result[] = array(
                                    'thumbnail' => gc_image_url($image['url']),
                                    'original' => gc_image_url($image['url'], false),
                                    'title' => $image_title
                                );
                            }
                            break;
                        case 3: // return gallery images only
                            if($image_type_id == 0) {
                                $result[] = array(
                                    'thumbnail' => gc_image_url($image['url']),
                                    'original' => gc_image_url($image['url'], false),
                                    'title' => $image_title
                                );
                            }
                            break;
                        case 2: // return logos only
                            if($image_type_id == 2) {
                                $result[] = array(
                                    'thumbnail' => gc_image_url($image['url']),
                                    'original' => gc_image_url($image['url'], false),
                                    'title' => $image_title
                                );
                            }
                            break;
                        case 1: // return cover images only
                            if($image_type_id == 1) {
                                $result[] = array(
                                    'thumbnail' => gc_image_url($image['url']),
                                    'original' => gc_image_url($image['url'], false),
                                    'title' => $image_title
                                );
                            }
                            break;
                        default:
                            $result[] = array(
                                'thumbnail' => gc_image_url($image['url']),
                                'original' => gc_image_url($image['url'], false),
                                'title' => $image_title
                            );
                    }
                }
            }
        }
        return $result;
    }
}

if( ! function_exists('gc_sort_array')) {
    /**
     * Sort Array by Priority
     * Usage: uasort($fields, "gc_sort_array");
     * @return string
     */
    function gc_sort_array($a, $b) {
        return $a['priority'] <=> $b['priority'];
        /*if(!empty($a['priority']) && !empty($b['priority'])) {
            return ($a["priority"] <= $b["priority"]) ? -1 : 1;
        }*/
    }
}

if( ! function_exists('gc_social_sharing')) {
    /**
     * URL sharing on social sites parameters set
     * @return string
     */
    function gc_social_sharing($record, $params = array()) {
        if(!empty($record->cover)) {
            $og_image = gc_image_url($record->cover, false);
        } else if(!empty($record->logo_url)) {
            $og_image = gc_image_url($record->logo_url, false);
        } else if(!empty($record->image)) {
            $og_image = gc_image_url($record->image, false);
        } else {
            $og_image = gc_image_url(settings_item('site.logo'), false);
        }
        $result = array(
            'image' => $og_image
        );
        return $result;
    }
}

if( ! function_exists('gc_display_field')) {
    /**
     * GoClixy Package Display Field
     * @return string
     */
    function gc_display_field($record, $field, $package, $params = array()) {
        $result = theme_view('widgets/modules/tpl/_display_field', array('record' => $record, 'field' => $field, 'package' => $package, 'params' => $params), true);
        return $result;
    }
}

if( ! function_exists('gc_cache_id')) {
    /**
     * Compute a checksum of a lengthy array to prepare cache id
     * @param string $data
     * @return string
     */
    function gc_cache_id($data = array()) {
        return md5(json_encode($data));
    }
}

if( ! function_exists('gc_site_image')) {
    /**
     * GoClixy hardcoded images, they can also be replaced with live url
     * @param string $slug
     * @return string
     */
    function gc_site_image($slug) {
        $items = array(
            'accepted-card' => 'accepted-card.png',
            'safe-checkout' => 'safe-checkout.png',
            'bookings' => 'bookings.jpg',
            'wallet-payment' => 'wallet-payment.png',
            'leads' => 'leads.jpg',
            'ajax-loader' => 'ajax-loader.gif',
            'affiliates' => 'affiliates.png',
            'forum' => 'forum.png',
            'forum-bg' => 'forum-bg.png',
            'no-image' => 'no-image.png',
            'no-logo' => 'no-logo.png',
            'no-user' => 'no-user.png',
            'business-query' => 'business-query.png',
            'admin-area-wallpaper' => 'admin-area-wallpaper.jpg',
            'favicon' => 'favicon.ico',
            'pyg' => 'pyg.png'
        );
        if(!empty($items[$slug])) {
            if(settings_item('site.images_server')) {
                $items[$slug] = settings_item('site.images_server') . $items[$slug];
            }
            return gc_image_url($items[$slug], false);
        } else {
            return base_url('assets/images/no-image.png');
        }
    }
}

if( ! function_exists('str_contains_word')) {
    /**
     * Check if a string contains specific words
     * @param string $str
     * @param array $arr
     * @return boolean
     */
    function str_contains_word($str, array $arr) {
        if(empty($arr)) {
            $arr = array('escorts service', 'call girls', 'sex toys');
        }
        $str = strtolower($str);
        foreach ($arr as $word) {
            // Works in Hebrew and any other unicode characters
            // Thanks https://medium.com/@shiba1014/regex-word-boundaries-with-unicode-207794f6e7ed
            // Thanks https://www.phpliveregex.com/
            if (preg_match('/(?<=[\s,.:;"\']|^)' . $word . '(?=[\s,.:;"\']|$)/', $str)) return true;
        }
        return false;
    }
}

if( ! function_exists('gc_sanitize_text')) {
    /**
	 * Strip tags and words from html
	 * https://stackoverflow.com/questions/5517255/remove-style-attribute-from-html-tags
	 * @param string $text
	 * @param string $allowed_tags
	 * @return mixed
	 */
    function gc_sanitize_text($text, $allowed_tags = '<ul><li><b><i><sup><sub><em><strong><u><br><br/><br /><p><h2><h3><h4><h5><h6>') { // <a><ul><li><b><i><sup><sub><em><strong><u><br><br/><br /><p><h2><h3><h4><h5><h6>
        mb_regex_encoding('UTF-8');
        $original_text = $text;
        //replace MS special characters first
        $search = array('/&lsquo;/u', '/&rsquo;/u', '/&ldquo;/u', '/&rdquo;/u', '/&mdash;/u');
        $replace = array('\'', '\'', '"', '"', '-');
        $text = preg_replace($search, $replace, $text);
        //make sure _all_ html entities are converted to the plain ascii equivalents - it appears
        //in some MS headers, some html entities are encoded and some aren't
        //$text = html_entity_decode($text, ENT_QUOTES, 'UTF-8');
        //try to strip out any C style comments first, since these, embedded in html comments, seem to
        //prevent strip_tags from removing html comments (MS Word introduced combination)
        if(mb_stripos($text, '/*') !== FALSE){
            $text = mb_eregi_replace('#/\*.*?\*/#s', '', $text, 'm');
        }
        //introduce a space into any arithmetic expressions that could be caught by strip_tags so that they won't be
        //'<1' becomes '< 1'(note: somewhat application specific)
        $text = preg_replace(array('/<([0-9]+)/'), array('< $1'), $text);
        $text = strip_tags($text, $allowed_tags);
        //eliminate extraneous whitespace from start and end of line, or anywhere there are two or more spaces, convert it to one
        $text = preg_replace(array('/^\s\s+/', '/\s\s+$/', '/\s\s+/u'), array('', '', ' '), $text);
        //strip out inline css and simplify style tags
        $search = array('#<(strong|b)[^>]*>(.*?)</(strong|b)>#isu', '#<(em|i)[^>]*>(.*?)</(em|i)>#isu', '#<u[^>]*>(.*?)</u>#isu');
        $replace = array('<b>$2</b>', '<i>$2</i>', '<u>$1</u>');
        $text = preg_replace($search, $replace, $text);
        //on some of the ?newer MS Word exports, where you get conditionals of the form 'if gte mso 9', etc., it appears
        //that whatever is in one of the html comments prevents strip_tags from eradicating the html comment that contains
        //some MS Style Definitions - this last bit gets rid of any leftover comments */
        $num_matches = preg_match_all("/\<!--/u", $text, $matches);
        if($num_matches){
            $text = preg_replace('/\<!--(.)*--\>/isu', '', $text);
        }
        $text = preg_replace('/(<[^>]+) style=".*?"/i', '$1', $text);
        
        // remove plain text links
        $pattern = "/[a-zA-Z]*[:\/\/]*[A-Za-z0-9\-_]+\.+[A-Za-z0-9\.\/%&=\?\-_]+/i";
        $replacement = "";
        $text = preg_replace($pattern, $replacement, $text);
        
        // text is empty, strip tags
        if(empty($text)) {
            $text = strip_tags($original_text);
        }
        return $text;
    }
}

if( ! function_exists('loadPopups')) {
    /**
     * Load Popups on page load
     * @param number $length
     * @return string
     */
    function loadPopups() {
        $modals = array();
        $js = array();
        $CI =& get_instance();
        $CI->load->library('gzf_cache', array('lifetime' => 604800)); // seven days
        $gc_cache = $CI->gzf_cache->get_instance();
        $CI->load->library('slug');
        $today_date = date('Y-m-d');
        $current_url = current_url();
        $cacheID = 'popups_' . str_replace('-', '_', $CI->slug->create_slug($current_url . '_' . $today_date));
        if(!$result = $gc_cache->load($cacheID)) {
            $frequencies = array(); // the number of times a popup is displayed to visitors
            // get all popups
            $records = $CI->gcrud->get_all_crud(0, 'M.status = 1 AND M.deleted = 0', 'M.timeout ASC', 10000, 0, array('module' => 'popups', 'prefix' => 'page', 'view' => true, 'cache' => 'pages'));
            if($records) {
                foreach($records as $record) {
                    if((empty($record->date_start) || $today_date >= $record->date_start)
                        && (empty($record->date_end) || $today_date <= $record->date_end)) {
                            $has_popup = false;
                            if(!empty($record->link_url)) {
                                $links = (array) json_decode($record->link_url, true);
                                if(!empty($links)) {
                                    foreach($links as $link) {
                                        if(!empty($link['url'])) {
                                            $url_link = $link['url'];
                                            $link_url = valid_url($url_link) ? $url_link : site_url($url_link);
                                            if($link_url == $current_url) {
                                                $has_popup = true;
                                            }
                                        }
                                    }
                                } else {
                                    $has_popup = true;
                                }
                            } else {
                                $has_popup = true;
                            }
                            if($has_popup) {
                                $content = $record->page_description;
                                // delete cookie if the duration is 0
                                if(empty($record->frequency_duration)) {
                                    delete_cookie('popup_' . $record->id);
                                }
                                $modals[$record->id] = $CI->load->view('pages/_tpl_popups', array('id' => $record->id, 'content' => $content), true);
                                $frequencies[$record->id][$current_url] = array('frequency' => $record->frequency, 'duration' => $record->frequency_duration);
                                $js[$record->id] = <<<EOL
setTimeout(function() {
    $('#popup{$record->id}').modal();
}, {$record->timeout});
EOL;
                            }
                        }
                }
            }
            // $result = array('js' => implode(PHP_EOL, $js), 'modals' => implode(PHP_EOL, $modals), 'frequencies' => $frequencies);
            $result = array('js' => $js, 'modals' => $modals, 'frequencies' => $frequencies);
            $gc_cache->save($result, $cacheID, array('pages'));
        }
        $js = $result['js'];
        $modals = $result['modals'];
        // limit the number of popups views on a page
        if(!empty($result['frequencies'])) {
            $frequencies = $result['frequencies'];
            foreach($frequencies as $fkey => $fvalue) {
                $cookie_name = 'popup_' . $fkey;
                $cookie_duration = (int) $fvalue[$current_url]['duration'];
                if(!empty($fvalue[$current_url]['frequency']) && !empty($fvalue[$current_url]['duration'])) { // the popup has a frequency
                    if(!$CI->input->cookie($cookie_name)) {
                        unset($fvalue[$current_url]['duration']);
                        $fvalue[$current_url]['frequency'] = $fvalue[$current_url]['frequency'] - 1;
                        $cookie = array (
                            'name' => $cookie_name,
                            'value' => base64_encode(json_encode($fvalue)),
                            'expire' => time() + $cookie_duration*24*60*60*1000 // store cookie for the next 7 days:: (days/1) (hours/day) (minutes/hour) (seconds/minute) (miliseconds/second)
                        );
                        $CI->input->set_cookie($cookie);
                    } else { // get the cookie
                        $cookie = json_decode(base64_decode($CI->input->cookie($cookie_name)), true);
                        if(!empty($cookie[$current_url]['frequency'])) {
                            $cookie[$current_url]['frequency'] = $cookie[$current_url]['frequency'] - 1;
                        } else {
                            unset($js[$fkey]);
                            unset($modals[$fkey]);
                            $cookie[$current_url]['frequency'] = 0;
                        }
                        if(isset($cookie['duration'])) {
                            unset($cookie['duration']);
                        }
                        // reset the cookie for the next duration
                        $cookie = array (
                            'name' => $cookie_name,
                            'value' => base64_encode(json_encode($cookie)),
                            'expire' => time() + $cookie_duration*24*60*60*1000 // store cookie for the next 7 days:: (days/1) (hours/day) (minutes/hour) (seconds/minute) (miliseconds/second)
                        );
                        $CI->input->set_cookie($cookie);
                    }
                }
            }
        }
        $result = array('js' => implode(PHP_EOL, $js), 'modals' => implode(PHP_EOL, $modals));
        return $result;
    }
}