<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
 || ##################################################################### ||
 || # GoClixy - Online Software Solutions							   # ||
 || # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
 || # ----------------------------------------------------------------  # ||
 || # This file is owned and copyrighted by GoClixy.com. Your license   # ||
 || # confers  no  title  or ownership in the Script and is not a sale  # ||
 || # of any rights to the Software. This file may not be redistributed # ||
 || # in whole or significant part.									   # ||
 || # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
 || # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
 || ##################################################################### ||
 \*=======================================================================*/
use Google\Analytics\Data\V1beta\BetaAnalyticsDataClient;
use Google\Analytics\Data\V1beta\DateRange;
use Google\Analytics\Data\V1beta\Dimension;
use Google\Analytics\Data\V1beta\Metric;
use Google\Analytics\Data\V1beta\OrderBy;
use Google\Analytics\Data\V1beta\OrderBy\DimensionOrderBy;
class Analytics {
    /**
     * Stores the CodeIgniter core object.
     *
     * @access public
     *
     * @var object
     */
    protected $CI;
    protected $az_cache;
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        $this->CI =& get_instance();
        $this->CI->load->library('gzf_cache', array('lifetime' => 900));
        $this->az_cache = $this->CI->gzf_cache->get_instance();
    }

    /**
     * Display analytics data to the user
     */
    public function data() {
        $cacheID = 'gc_analytics';
        if(!$data = $this->az_cache->load($cacheID)) {
            $data = $this->_data();
            $this->az_cache->save($data, $cacheID, array('gc_analytics'));
        }
        return $data;
    }

    /**
     * Get analytics data
     */
    private function _data() {
        $result = array();
        try {
            $client = new BetaAnalyticsDataClient(['credentials' => APPPATH . 'third_party/Google/' . settings_item('seo.analytics_key_file')]); // https://github.com/googleapis/php-analytics-data#installation

            $property = 'properties/' . settings_item('seo.analytics_property_id'); // property id
            
            // Calculate the date range for the last 7 days
            $endDate = new DateTime();
            $startDate = clone $endDate;
            $startDate->modify('-' . (!empty(settings_item('seo.analytics_rows_limit')) ? settings_item('seo.analytics_rows_limit') : 7). ' days');
            
            $dateRange = new DateRange();
            $dateRange->setStartDate($startDate->format('Y-m-d'));
            $dateRange->setEndDate($endDate->format('Y-m-d'));
            
            $dimension = new Dimension();
            $dimension->setName("date");

            // order by date
            $orderBy = new OrderBy([
                'dimension' => new DimensionOrderBy([
                    'dimension_name' => 'date',
                    'order_type' => DimensionOrderBy\OrderType::ALPHANUMERIC
                ])
            ]);
            
            $metric = new Metric();
            $metric->setName("activeUsers");
            
            $response = $client->runReport([
                'property' => $property,
                'dateRanges' => [$dateRange],
                'dimensions' => [$dimension],
                'metrics' => [$metric],
                'orderBys' => [$orderBy]
            ]);
            if($response) {
                foreach ($response->getRows() as $row) {
                    $dimensionValue = $row->getDimensionValues()[0]->getValue(); // '20230810'
                    $metricValue = $row->getMetricValues()[0]->getValue();

                    // format date
                    $num_array = str_split($dimensionValue);
                    $formatted_date = $num_array[0] . $num_array[1] . $num_array[2] . $num_array[3] . '-' . $num_array[4] . $num_array[5] . '-' . $num_array[6] . $num_array[7];
                    
                    // prepare result array
                    $result[] = array(
                        'date' => $formatted_date,
                        1 => $metricValue
                    );
                }
            }
            return $result;
        }  catch (Exception $e) {
            return array('message' => $e->getMessage());
        }
    }
}
?>