<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
 || ##################################################################### ||
 || # GoClixy - Business Directory Script							   # ||
 || # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
 || # ----------------------------------------------------------------  # ||
 || # This file is owned and copyrighted by GoClixy.com. Your license   # ||
 || # confers  no  title  or ownership in the Script and is not a sale  # ||
 || # of any rights to the Software. This file may not be redistributed # ||
 || # in whole or significant part.									   # ||
 || # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
 || # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
 || ##################################################################### ||
 \*=======================================================================*/
class Crud {
    /**
     * Stores the CodeIgniter core object.
     *
     * @access public
     *
     * @var object
     */
    public $CI;
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct($config = array()) {
        $this->CI =& get_instance();
        $this->CI->load->library('gc');
        $this->CI->load->model('fields/fields_model');
    }
    
    /**
     * Index Method
     * filter - Use the switch statement in the controller itself and pass it using the filter parameter
     * uri_segment - Segment of the URL for pagination and search redirection
     * redirect_uri - The url to be redirected, in case of admin it is without SITE_AREA
     */
    public function index($module, $offset = 0, $role = 'admin', $params = array()) {// Do we have any actions?        
        // Status Change
        if (isset($_POST['status_change'])) {
            $this->CI->gc->status_change($module, $_POST['status_change'], 'status');
        }
        $context = 'Content';
        if(isset($params['context'])) {
            $context = ucwords($params['context']);
        }          
        $edit_url = '';
        if($role == 'admin') {
            $edit_url = site_url(SITE_AREA . '/' . strtolower($context) . '/' . $module . '/edit/');
            if(!isset($params['redirect_uri'])) {
                $params['redirect_uri'] = '/' . strtolower($context) . '/' . $module . '/index';
            }
        }
        $redirect_uri = isset($params['redirect_uri']) ? $params['redirect_uri'] : 'members';
        $redirect_url = $role == 'admin' ? site_url(SITE_AREA . $redirect_uri) : gc_url_format($redirect_uri);
        $custom_model = isset($params['model']) ? $params['model'] : $module;
        $module_model = $this->CI->load->model($custom_model .'_model');
        $module_model = $this->CI->{$custom_model . '_model'};
        
        /*********************************
         * VARIABLES
         ********************************/
        $uri_segment = isset($params['uri_segment']) ? $params['uri_segment'] : 5;
        $dynamic_fields = !empty($params['dynamic_fields']) ? true : false; // true - load fields from the database
        $fields_method = isset($params['fields_method']) ? $params['fields_method'] : 'fields';
        $fields = $this->fields($fields_method, $module, $module_model, $role, 'index', $dynamic_fields); // module fields
        $search_fields = !empty($params['search_fields']) ? $params['search_fields'] : array(); // search fields
        $soft_delete = !empty($module_model->soft_deletes) ? true : false; // change the method from protected to public in the Model
        $table_prefix = isset($params['prefix']) ? $params['prefix'] . '.' : '';
        $footer_actions = !empty($params['footer_actions']) ? $params['footer_actions'] : array(); // Footer actions
        $actions = !empty($params['actions']) ? $params['actions'] : array(); // Other actions to be performed e.g. copy_row, view etc.
        $can_delete = $role == 'admin' ? $this->CI->auth->has_permission($module . '.' . $context . '.Delete') : (!empty($params['can_delete']) ? true : false);
        $can_edit	= $role == 'admin' ? $this->CI->auth->has_permission($module . '.' . $context . '.Edit') : (!empty($params['can_edit']) ? true : false);
        $can_create	= $role == 'admin' ? $this->CI->auth->has_permission($module . '.' . $context . '.Create') : (!empty($params['can_create']) ? true : false);
        $edit_url = !empty($params['edit_url']) ? $params['edit_url'] : ($can_edit ? $edit_url : ''); // Edit page url
        $create_url = !empty($params['create_url']) ? $params['create_url'] : ''; // Create page url
        $filter_tabs = !empty($params['filter_tabs']) ? $params['filter_tabs'] : array(); // Filter tabs
        $filter_letters = !empty($params['filter_letters']) ? TRUE : FALSE; // Filter Letters
        $filter_dropdown = !empty($params['filter_dropdown']) ? TRUE : FALSE; // Filter Dropdown
		$render = !empty($params['render']) ? $params['render'] : ''; // Layout to be Rendered
        /*********************************
         * END OF VARIABLES
         ********************************/
        
        /*********************************
         * START OF ACTIONS
         ********************************/
		// Allow Admin to change status
		if(isset($_POST['change_status'])) {
		    $id = $_POST['change_status'];
		    $new_status = $_POST['status_' . $id];
		    $this->update_status($module, $id, $new_status);
		}
        if (isset($_POST['submit'])) {
            $action = $this->CI->input->post('submit');
            $checked = $this->CI->input->post('checked');
            if($checked) {
                $module_name = count($checked) == 1 ? lang('singular_' . $module) : lang($module);
                switch(strtolower($action)) {
                    case 'activate':
                        $this->CI->gc->update_status($module, $checked, 'status', 1);
                        Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_activated')), 'success');
                        break;
                        
                    case 'deactivate':
                        $this->CI->gc->update_status($module, $checked, 'status', 0);
                        Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_deactivated')), 'success');
                        break;
                        
                    case 'spam':
                        $this->CI->gc->update_status($module, $checked, 'status', 2);
                        Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_spammed')), 'success');
                        break;
                        
                    case 'delete':
                        if(isset($params['delete_method'])) { // custom delete method specified in the module model
                            $module_model->{$params['delete_method']}($checked);
                        } else {
                            if($soft_delete) {
                                $this->CI->gc->update_status($module, $checked, 'deleted', 1);                            
                            } else {
                                $this->CI->gc->purge_records($module, $checked);
                            }
                        }
                        Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_deleted')), 'success');
                        break;
                        
                    case 'purge':
                        $this->CI->gc->purge_records($module, $checked);
                        Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_deleted')), 'success');
                        break;
                        
                    case 'restore':
                        $this->CI->gc->update_status($module, $checked, 'deleted', 0);
                        Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_restored')), 'success');
                        break;
                        
                    default:
                        if(strpos($action, '__') !== false) {
                            $action_array = explode('__', $action); // __ is the separator used to identify field and its value
                            $column_name = $action_array[0];
                            $column_value = $action_array[1];
                            $data = array(
                                $column_name => $column_value,
                                'modified_on' => date('Y-m-d H:i:s'),
                                'modified_ip' => $this->CI->input->ip_address()
                            );
                            if($column_name == 'status_id') {
                                if(strpos($module, 'ticket') !== false && $column_value == 3) {
                                    $data['closed_by'] = $this->CI->auth->user_id();
                                }
                            }
                            foreach($checked as $checked_id) {
                                if($module == 'orders') {
                                    $this->CI->load->model('orders/orders_model');
                                    $order = $this->CI->db->where('id', $checked_id)->get('orders')->row();
                                    if($order) {
                                        $this->CI->orders_model->update_status($checked_id, $order->order_id, $order, $column_value, (int) $this->CI->input->post('notify_customer_' . $checked_id), true);
                                    }
                                } else {
                                    $this->CI->db->where('id', $checked_id)->update($module, $data);
                                }
                            }
                            Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_updated')), 'success');
                            break;
                        }
                        break;
                }
            }
        }
        
        // Field value update
        if (isset($_POST['update']) || isset($_POST['change'])) { // change - dropdown update
            $field = isset($_POST['update']) ? $_POST['update'] : (isset($_POST['change']) ? $_POST['change'] : null);
            if(!empty($field)) {
                $field_key = key($field); // get the key name, Note: the existing value is appended with the field_name using the underscore separator (_).
                $row_id = $field[$field_key]; // get the row id
                
                // get the existing value of the row
                $pos = strrpos($field_key, '_');
                $row_value = $pos === false ? $field_key : substr($field_key, $pos + 1);
                
                // get the field name by removing the existing value from the string
                $field_name = substr($field_key, 0, strrpos($field_key, "_"));
                
                // new value of the row
                if(isset($_POST['change'])) {
                    $dropdown_field = $_POST['dropdown_field'];
                    $new_value = $dropdown_field[$field_key];
                } else {
                    $new_value = $row_value == 0 ? 1 : 0;
                }
                
                // perform operation
                $this->CI->gc->update_status($module, array($row_id), $field_name, $new_value);
                
                // display message to the user
                Template::set_message(sprintf(lang('msg_action_success'), lang('singular_' . $module), lang('label_updated')), 'success');
                Template::redirect(current_url());
            }
        }
                
        // Create duplicate row
        if (!empty ( $_POST ['copy'] )) {
            $copy_id = $_POST ['copy'];
            $this->copy_row($copy_id, $module);
        }        
        /*********************************
         * END OF ACTIONS
         ********************************/        
        
        /*********************************
         * START OF FILTER
         ********************************/
        $filter = isset($params['filter']) ? $params['filter'] : '';
        $where = '';
        if(empty($where) && $soft_delete) {
            $where = $table_prefix . 'deleted = 0';
        }
        if(isset($filter)) {
            // Filters
            if (preg_match('{first_letter-([A-Z])}', $filter, $matches)) {
                $filter_type = 'first_letter';
                $first_letter = $matches[1];
            } else {
                $filter_type = $filter;
            }
            if(!empty($where)) {
                $where .= ' AND ';
            }
            switch($filter_type) {
            	case 'active':
            		$where .= $table_prefix . 'status = 1';
            		break;
            
            	case 'inactive':
            		$where .= $table_prefix . 'status = 0';
            		break;
            
            	case 'deleted':
            		$where = $table_prefix . 'deleted = 1';
            		break;
            
            	case 'first_letter':
            		$where .= "SUBSTRING( LOWER(" .$table_prefix . "name), 1, 1)= '". $first_letter ."'";
            		break;
            		
            	default:
            		if($filter_type != 'all' && !empty($params['filter_field'])) {
            			if(in_array($filter_type, $filter_tabs)) {
            				$val = array_search ($filter_type, $filter_tabs);
            				$where .= $table_prefix . $params['filter_field'] . ' = ' . $val;
            			}
            		} else {
            		    if(strpos($filter_type, '__') !== false) {
            		        $filter_array = explode('__', $filter_type);
            		        $where .= $table_prefix . $filter_array[0] . " = '" . $filter_array[1] . "'";
            		    } else if(strpos($filter_type, '--') !== false) {
            		        $filter_array = explode('--', $filter_type);
            		        $where .= $table_prefix . $filter_array[0] . ' = ' . $filter_array[1];
            		    } else if($filter_type != 'all' && !empty($params['filter_field'])) {
            		        if(in_array($filter_type, $filter_tabs)) {
            		            $val = array_search ($filter_type, $filter_tabs);
            		            $where .= $table_prefix . $params['filter_field'] . ' = ' . $val;
            		        }
            		    }
            		}
            }
            Template::set('index_url', $redirect_url .'/');
            Template::set('filter_type', $filter_type);
        }        
        /*********************************
         * END OF FILTER
         ********************************/
        
        /*********************************
         * START OF QUERIES
         ********************************/
        if(!empty($params['where'])) {
            $where .= !empty($where) ? ' AND ' . $params['where'] : $params['where'];            
        }
        $like_array = array();
        $searchterm = $this->CI->gc->search($module, $this->CI->uri->segment($uri_segment), $redirect_uri);              
        $limit = ($role == 'admin' ? settings_item('site.list_limit') : settings_item('site.front_list_limit'));

        if(!empty($params['query_method'])) { // get data using a method specified in the Module Model
            $table_alias = isset($params['table_alias']) ? $params['table_alias'] : '';
            if(!empty($searchterm)) {
                // This is the basic search filter, you could add more fields to the search
                if(!empty($search_fields)) {                    
                    foreach($search_fields as $search_field) {
                        $like_array[] = $table_alias . $search_field . " LIKE '%" . $searchterm . "%'";
                    }
                    $where .= !empty($where) ? ' AND (' . implode(' OR ', $like_array) . ')' : implode(' OR ', $like_array);
                }
            }
			$where .= isset($params['query_where']) ? (!empty($where) ? ' AND ' . $params['query_where'] : $params['query_where']) : '';
            $sort_by = 'id DESC';
            $order_by = $this->CI->gc->sort($module);
            if(!empty($order_by)) {
                $sort_by = $order_by;
            }            
            $query_extra = isset($params['query_extra']) ? $params['query_extra'] : array(); // Extra parameters to be passed to the query
            if($params['query_method'] == 'get_all_crud') { // the get_rows method is in the library itself
                $total_rows = $this->{$params['query_method']}(1, $where, $sort_by, $limit, $offset, $query_extra);
                $records = $this->{$params['query_method']}(0, $where, $sort_by, $limit, $offset, $query_extra); // Get all records
            } else {
                $total_rows = $module_model->{$params['query_method']}(1, $where, $sort_by, $limit, $offset, $query_extra);
                $records = $module_model->{$params['query_method']}(0, $where, $sort_by, $limit, $offset, $query_extra); // Get all records
            }
        } else {
            if(!empty($searchterm)) {
                // This is the basic search filter, you could add more fields to the search
                if(!empty($search_fields)) {
                    foreach($search_fields as $search_field) {
                        $like_array[$search_field] = $searchterm;
                    }
                }
            } 
            // Count
            if(!empty($like_array)) {
                $module_model->or_like($like_array);
            }
            if(!empty($where)) {
                $module_model->where($where);
            }
            $total_rows = $module_model->count_all();
            
            // Records
            $module_model->limit($limit, $offset);
            $order_by = $this->CI->gc->sort($module);
            if(!empty($order_by)) {
                $module_model->order_by($this->CI->session->userdata($module . '_sort')['column'], $this->CI->session->userdata($module . '_sort')['order']);
            } else {
                $module_model->order_by('id', 'DESC');
            }
            if(!empty($like_array)) {
                $module_model->or_like($like_array);
            }
            if(!empty($where)) {
                $module_model->where($where);
            }
            $records = $module_model->find_all(); // Get all records
        }
        /*********************************
         * END OF QUERIES
         ********************************/
        
        /*********************************
         * START OF PAGINATION
         ********************************/
        $pagerUriSegment = $uri_segment;
        $pagerBaseUrl = $redirect_url . '/' . (!empty($filter) ? $filter . '/' : '');
        
        $this->CI->load->library('pagination');
        $pager['base_url']    = $pagerBaseUrl;
        $pager['total_rows']  = $total_rows;
        $pager['per_page']    = $limit;
        $pager['uri_segment'] = $pagerUriSegment;
        
        $this->CI->pagination->initialize($pager);
        /*********************************
         * END OF PAGINATION
         ********************************/ 
        if($role != 'mobile') {
            // Display form within the index page
            if(!empty($params['index_form'])) {
                $edit_url = $redirect_url . (!empty($filter) ? '/' . $filter : '') . '/' . $offset . '/';
                $params['id'] = $this->CI->uri->segment($params['index_form']) ? $this->CI->uri->segment($params['index_form']) : 0;
                if(!empty($params['id'])) {
                    $params['redirect_url'] = $edit_url . $params['id'];
                }
                $this->form($module, ($params['id'] == 0 ? 'create' : 'edit'), $role, $params);
            }
            Template::set(array(
                'can_create' => $can_create,
                'create_url' => $create_url,
                'can_delete' => $can_delete,
                'can_edit' => $can_edit,
                'edit_url' => $edit_url . '%s',
                'fields' => $fields,
                'filter_letters' => $filter_letters, // filter by letters
                'filter_dropdown' => $filter_dropdown, // show filters in a dropdown
                'filter_tabs' => $filter_tabs, // filter tabs
                'footer_actions' => $footer_actions,
                'actions' => $actions,
                'context' => $context, // for permissions check on the index page
                'role' => $role,
                'module' => $module,
                'records' => $records,
                'total_rows' => $total_rows,
                'toolbar_title' => lang('manage_' . $module)
            ));
            Template::set_view('fields/_index');
            Template::render($render);
        } else {
            return array('records' => $records, 'total_rows' => $total_rows);
        }
    }
    
    /**
     * Get Theme Contents/Groups etc.
     * @param number $count
     * @param string $where
     * @param string $sort_by
     * @param number $limit
     * @param number $offset
     * @param array $extra - cache - name of the string to be used during clear cache
     * @return array|number
     */
    public function get_all_crud($count = 0, $where = null, $sort_by = 'M.id DESC', $limit = 15, $offset = 0, $extra = array()) {
        $cache_variable = null;
        if(isset($extra['cache'])) {
            $cache_variable =  $extra['cache'] ? $extra['cache'] : null;
            unset($extra['cache']);
        }
        if(!empty($cache_variable)) {
            // Load Zend Caching
            $gcrud_cache = new stdClass();
            $this->CI->load->library('slug');
            $this->CI->load->library('gzf_cache', array('lifetime' => 604800));
            if(method_exists($this->CI->gzf_cache, 'get_instance')) {
                $gcrud_cache = $this->CI->gzf_cache->get_instance();
            }
            $cacheID = 'gcrud_' . $count . '_' . implode('_', (array) $where) . '_' . $sort_by . '_' . $limit . '_' . $offset . '_' . implode('_', $extra);
            $cacheID = str_replace('-', '_', $this->CI->slug->create_slug($cacheID));
            if(!$result = $gcrud_cache->load($cacheID)) {
                $result = $this->_get_all_crud($count, $where, $sort_by, $limit, $offset, $extra);
                $gcrud_cache->save($result, $cacheID, array($cache_variable));
            }
        } else {
            $result = $this->_get_all_crud($count, $where, $sort_by, $limit, $offset, $extra);
        }
        return $result;
    }
    
    /**
     * Get Theme Contents/Groups etc.
     * @param number $count
     * @param string $where
     * @param string $sort_by
     * @param number $limit
     * @param number $offset
     * @return array|number
     */
    private function _get_all_crud($count = 0, $where = null, $sort_by = 'M.id DESC', $limit = 15, $offset = 0, $extra = array()) {
        if(!isset($extra['module'])) {
            return false;
        }
        $module = $extra['module'];
        $prefix = !empty($extra['prefix']) ? $extra['prefix'] : '';
        $table = (!empty($extra['prefix']) ? $extra['prefix'] . '_' : '') . $extra['module'];
        if($count == 1) {
            $sql = "SELECT COUNT(*) AS total_count";
        } else {
            switch($module) {
                case 'theme_contents':
                    $sql = "SELECT M.*";
                    $sql .= ", (SELECT GROUP_CONCAT(CONCAT(TCG.id, '!~!', TCG.name) SEPARATOR '~~~') FROM " . $this->CI->db->dbprefix . "theme_content_groups TCG WHERE FIND_IN_SET(TCG.id, M.group_ids)) AS groups";
                    break;
                case 'image_comments':
                    $sql = "SELECT M.*, S.color AS status_color, S.name AS status, S.name_lang AS status_lang, " . $this->user_fields() . PHP_EOL;
                    $sql .= ", I.url AS image_url";
                    /*if($prefix == 'product') {
                        $sql .= ", (SELECT CONCAT(P.id, '!~!', P.name, '!~!', P.slug, '!~!', P.status_id, '!~!products') FROM " . $this->CI->db->dbprefix . "products P WHERE P.id = M.product_id) AS redirect_uri";
                    } else {
                        $sql .= ", (SELECT CONCAT(B.id, '!~!', B.title, '!~!', B.slug, '!~!', B.status_id, '!~!blogs') FROM " . $this->CI->db->dbprefix . "blogs B WHERE B.id = M.blog_id) AS redirect_uri";
                    }*/
                    break;
                case 'image_likes':
                case 'image_favorites':
                    $sql = "SELECT M.*, U.display_name, U.email AS user_email, I.url AS image_url";
                    break;
                case 'incorrects':
                case 'image_incorrects':
                    $sql = "SELECT M.*, U.display_name, U.email AS user_email, P.name AS profile_name, P.profile_id AS id_profile, I.url AS image_url";
                    $sql .= ", S.color AS status_color, S.name AS status, S.name_lang AS status_lang" . PHP_EOL;
                    break;
                default:
                    $sql = "SELECT M.*";
                    
            }
        }
        $sql .= " FROM " . $this->CI->db->dbprefix . $table . " M";
        if($count == 0) {
            switch($module) {
                case 'image_comments':
                    $sql .= " LEFT JOIN " . $this->CI->db->dbprefix . "users U ON U.id = M.user_id";
                    $sql .= " LEFT JOIN " . $this->CI->db->dbprefix . "comment_statuses S ON S.id = M.status_id";
                    $sql .= " LEFT JOIN " . $this->CI->db->dbprefix . "images I ON I.id = M.image_id";
                    break;
                case 'image_likes':
                case 'image_favorites':
                    $sql .= " LEFT JOIN " . $this->CI->db->dbprefix . "users U ON U.id = M.user_id";
                    $sql .= " LEFT JOIN " . $this->CI->db->dbprefix . "images I ON I.id = M.image_id";
                    break;
                case 'incorrects':
                case 'image_incorrects':
                    $sql .= " LEFT JOIN " . $this->CI->db->dbprefix . "users U ON U.id = M.user_id";
                    $sql .= " LEFT JOIN " . $this->CI->db->dbprefix . "profiles P ON P.id = M.profile_id";
                    $sql .= " LEFT JOIN " . $this->CI->db->dbprefix . "images I ON I.id = M.image_id";
                    $sql .= " LEFT JOIN " . $this->CI->db->dbprefix . "incorrect_types S ON S.id = M.type_id";
                    break;
            }
        }        
        $sql .= " WHERE M.deleted = 0";
        if(!empty($where)) {
            $sql .= ' AND '. $where;
        }
        if ($count == 0) {
            $sql .= " GROUP BY M.id ORDER BY " . $sort_by;
            if ($limit != 0) {
                $sql .= " LIMIT " . $offset . ", " . $limit;
            } else {
                $sql .= " LIMIT $limit";
            }
            $query = $this->CI->db->query($sql);
            $records = $query->result();
            return $records;
        } else {
            $result = $this->CI->db->query($sql)->row();
            return $result ? $result->total_count : 0;
        }
    }
                    
    
    /**
     * Allows editing of module data.
     *
     * @param array $params
     * 1. context - Name of the context for the admin redirection
     * 2. uri_segment - Segment ID in case of edit form (ADMIN AREA), default is 5
     * 
     * @return void
     */
    public function form($module, $type = 'create', $role = 'admin', $params = array()) {
        
        // Index page can also be with fields, e.g. states, cities and localities
        $action_type = ($type == 'create' ? 'insert' : 'update');
        $index_form = !empty($params['index_form']) ? true : false;
        $custom_model = isset($params['model']) ? $params['model'] : $module;
        $module_model = $this->CI->load->model($custom_model .'_model');
        $module_model = $this->CI->{$custom_model . '_model'};
        $user_id = (isset($params['user_id']) ? $params['user_id'] : $this->CI->auth->user_id());
        $label = lang('singular_' . $module);
        $id = isset($params['id']) ? $params['id'] : 0;
        $view_params = isset($params['view_params']) ? $params['view_params'] : array();
        $dynamic_fields = !empty($params['dynamic_fields']) ? true : false; // true - load fields from the database
        $fields_method = isset($params['fields_method']) ? $params['fields_method'] : 'fields';
        $fields = $this->fields($fields_method, $module, $module_model, $role, 'form', $dynamic_fields, $type, $view_params); // module fields
        $mobile = $role == 'mobile' ? true : false;
        
        // Display fields in a group
        $groups = array();
        if(!empty($param['groups'])) {
            $groups = $params['groups'];
        } else if($dynamic_fields) {
        	$this->CI->lang->load('groups/groups');
        	$groups = $this->CI->fields_model->field_groups($fields, array());
        }
        
        // Form Layout
        /*$tab_layout = null;
        if(isset($params['layout'])) {
            $layout = $params['layout'];
            if(isset($params['tab_layout'])) {
                $tab_layout = $params['tab_layout'];
            }
        } else {
            // Load layout from the settings table
            $field_settings = json_decode(settings_item('field.settings'), true);
            $field_layouts = json_decode(settings_item('field.layouts'), true);
            if(isset($field_settings[$role][$module])) {
                $layout = $field_settings[$role][$module];
            } else {
                $layout = 'none';
            }
            if(isset($field_layouts[$role][$module])) {
                $tab_layout = $field_layouts[$role][$module];
            }
        }
        ,
           // 'layout' => $layout,
           // 'tab_layout' => $tab_layout
        */
        if($role == 'admin') {
            $context = isset($params['context']) ? $params['context'] : 'Content';
            $permission = $module . '.' . $context . '.';
            if($type == 'edit') {
                $uri_segment = isset($params['uri_segment']) ? $params['uri_segment'] : 5;
                if(empty($id)) {
                    $id = $this->CI->uri->segment($uri_segment);
                }
            }            
            if(!isset($params['redirect_uri'])) {
                $params['redirect_uri'] = '/' . strtolower($context) . '/' . $module . '/index';
            }
        }
        if(empty($params['redirect_url'])) {
            $redirect_uri = isset($params['redirect_uri']) ? $params['redirect_uri'] : 'members';
            $redirect_url = $role == 'admin' ? site_url(SITE_AREA . $redirect_uri) : gc_url_format($redirect_uri);
        } else {
            $redirect_url = $params['redirect_url'];
        }
        if ($type == 'edit' && empty($id)) {
            Template::set_message(lang('invalid_id'), 'danger');
            redirect($redirect_url);
        }
        $record = !empty($id) ? $this->CI->db->where('id', $id)->get($module)->row() : array();
        if($type == 'edit') {
            if(!$record) {
                Template::set_message(lang('invalid_id'), 'danger');
                redirect($redirect_url);
            }
            if($module == 'profiles') { // Restrict Agent/Franchisee to their specific records
                $this->CI->auth->restrict_agent($record);
            }
        }
        $view_data = array( // data to be passed to the view file
            'form_fields' => $fields,
            'field_groups' => $groups
        );
        // Load CSS and JS files
        $field_type = ($role != 'admin' ? 'frontend' : 'admin_only');
        $form_fields = $this->CI->fields_model->load_fields($module, array(), $field_type, $record, $id, $mobile, $fields);
        // $this->CI->fields_model->load_js_methods($fields, $module, $field_type, array(), $id);
        // Add/Edit page
        if (isset($_POST['save'])) {
            if($role == 'admin') {
                $this->CI->auth->restrict($permission . ucwords($type));
            }
            if ($success = $this->CI->fields_model->save_module($fields, $module, $user_id, $role, $id, $action_type)) {
                if($id == 0) { // insert operation performed
                    $id = $success;
                }
                if(isset($params['method'])) {
                    $module_model->{$params['method']}($id, $action_type, $record);
                }
                log_activity($this->CI->auth->user_id(), lang('act_edit_record') . ': ' . $id . ' : ' . $this->CI->input->ip_address(), $module);
                Template::set_message(sprintf(lang('msg_save_success'), $label), 'success');
                Template::redirect($redirect_url);
            }
            // Not validation error
            if ( ! empty($module_model->error)) {
                Template::set_message(sprintf(lang('msg_edit_failure'), $label) . $module_model->error, 'danger');
            }
        } elseif (isset($_POST['delete'])) {
            if($role == 'admin') {
                $this->CI->auth->restrict($permission . 'Delete');
            }
            if ($module_model->delete($id)) {
                log_activity($this->CI->auth->user_id(), lang('act_delete_record') . ': ' . $id . ' : ' . $this->CI->input->ip_address(), $module);
                Template::set_message(sprintf(lang('msg_delete_success'), $label), 'success');
                redirect($redirect_url);
            }
            Template::set_message(sprintf(lang('msg_delete_failure'), $label) . $module_model->error, 'danger');
        }
        if($index_form) {    
            $view_data['form_record'] = $record;
            Template::set($view_data);
        } else {
            $view_data['cancel_url'] = $redirect_url;
            $view_data['module'] = $module;
            $view_data['record'] = $record;
            $view_data['toolbar_title'] = lang($type . '_' . $module);
            $render = isset($params['render']) ? $params['render'] : '';
			$disable_cancel_btn = isset($params['cancel_btn']) && $params['cancel_btn'] == 1 ? false : true;
			if($disable_cancel_btn) {
				Template::set('disable_cancel_btn', $disable_cancel_btn);
			}
			if($render == 'member') {
                Assets::add_css('member');
                Assets::add_js('member');                
                Template::set(array(
                    'elements_container' => 'div.card.card-default.card-body'                    
                ));
            }
            Template::set($view_data);
            Template::set_view('fields/_form');
            Template::render($render);
        }
    }    
    
    /**
     * Allow Admin to Copy A Row
     * @param string $module
     */
    private function copy_row($clone_id, $module, $actions = array()) {
        $el = $this->CI->db->get_where($module, array('id' => $clone_id))->row();
        if($el) {
            $clone_data = array();
            foreach($el as $key => $value) {
                switch($key) {
                    case 'id':
                    case 'created_by':
                    case 'modified_by':
                    case 'modified_on':
                    case 'modified_ip':
                        break;
                    case 'created_on':
                        $clone_data[$key] = date('Y-m-d H:i:s');
                        break;
                    case 'created_ip':
                        $clone_data[$key] = $this->input->ip_address();
                        break;
                    case 'name':
                        $clone_data[$key] = $value .' '.lang('label_copy');
                        break;
                    default:
                        $clone_data[$key] = $value;
                }
            }
            $this->CI->db->insert($module, $clone_data);
            $inserted_id = $this->CI->db->insert_id();
            if (is_numeric ( $inserted_id )) {
                if(!empty($actions)) {
                    foreach($actions as $action) {
                        // perform other actions
                    }
                }
                Template::set_message(lang('msg_clone_success'), 'success');
                Template::redirect(current_url());
            } else {
                Template::set_message(lang('msg_clone_error'), 'danger');
            }
        }
    }
    
    /**
     * Return module fields
     * @param string $fields_method - Using Single Model for multiple forms
     * @param string $module
     * @param object $module_model
     * @param string $role
     * @param string $page
     * @param boolean $dynamic_fields - allow user to switch a module to fields easily without any changes to CRUD library
     * @param string $form_type - show/hide fields from the create and edit form separately
     * @param array $view_params - Extra parameters sent to the view from the controller
     * @return array
     */
    private function fields($fields_method, $module, $module_model, $role = 'admin', $page = 'index', $dynamic_fields = false, $form_type = 'insert', $view_params = array()) {
        if($page == 'index') {
            $field_type = ($role != 'admin' ? 'frontend_index' : 'admin_index');
        } else {
            $field_type = ($role != 'admin' ? 'frontend' : 'admin_only');
        }
        if($dynamic_fields) {
        	$fields = $this->CI->fields_model->get_fields($module, $field_type);
            return $fields;
        } else {
            $fields = $module_model->{$fields_method}();
            return $this->CI->fields_model->crud_fields($fields, $field_type, $form_type, $view_params);
        }
    }
    
    /**
     * Frontend Index Method - Displays a list of all rows.
     *
     * @return void
     */
    public function frontend_index($module, $limit, $offset = 0, $params = array()) {
        $where = array('status' => 1);
		$this->CI->db->limit($limit, $offset);
        $records = $this->CI->db->where($where)->order_by('id', 'DESC')->get($module)->result();
       // $records = $this->CI->layouts->widget_modules_view($module, $records, array());
        $total_rows = $this->CI->db->where($where)->count_all_results($module);
        // Pagination
        $this->CI->load->library('pagination');
        $pager['base_url']     = gc_url_format($module) . '/index';
        $pager['total_rows']   = $total_rows;
        $pager['per_page']     = $limit;
        $pager['uri_segment']  = 3;
        $this->CI->pagination->initialize($pager);
        $pagination = $this->CI->pagination->create_links();
        
        Template::set(array(
			'toolbar_title' => lang('gc_menu_' . $module),
            'module' => $module,
            'pagination' => $pagination,
            'records' => $records
        ));
        $layout = $this->CI->gc->load_assets('modules/' . $module . '/view');
        Template::set_view($layout['view']);
        Template::render($layout['layout']);
    }
    
    /**
     * Display news information to the user
     * @param string $module
     * @param array $params
     */
    public function frontend_detail($module, $params = array()) {
        if(empty($params['where']))	{
            Template::redirect(site_url());
        } else {
            $record = $this->CI->db->where($params['where'])->get($module)->row();
            if(!$record) {
                Template::redirect(site_url());
            }
            // get translations if exists
            $record = $this->CI->translations->get_translations($module, $record->id, $record, 1);
            $page_title = $record->meta_title;
			$description = $record->description;
            $last_space = strrpos( substr( $description, 0, 200 ), ' ' );
            $page_description = strip_tags(substr( $description, 0, $last_space ) . '...');
            $page_keywords = $record->keywords;
            Template::set(array(
				'module' => $module,
                'title' => $record->title,
                'record'=> $record,
                'page_title' => $record->meta_title,
                'page_keywords' => $record->meta_keywords,
                'page_description' => $record->meta_description
            ));
            
            // update module hits
            $this->CI->gc->update_hits($record->id, $module);
            
            // add/update recently viewed status
            $this->CI->gc->add_recently_viewed($record->id, $record);
            $layout = $this->CI->gc->load_assets('modules/' . $module . '/detail');
			Template::set_view($layout['view']);
            Template::render($layout['layout']);
        }
    }
    
    /**
     * Fields take from the users table - Forums, Blogs, Tickets and other Modules
     * @return string
     */
    private function user_fields($prefix = 'U') {
        $prefix = $prefix . '.';
        $sql = $prefix . "display_name, " . $prefix . "username, " . $prefix . "email AS user_email, " . $prefix . "profile_image, " . $prefix . "is_online, " . $prefix . "created_on AS join_date, CONCAT(" . $prefix . "id, '!~!', " . $prefix . "username, '!~!', " . $prefix . "email, '!~!', " . $prefix . "display_name, '!~!', " . $prefix . "created_on) AS profile_user";
        return $sql;
    }
    
    
    /**
     * Update Status
     * @param number $id
     * @param number $status - New Status
     * @param array $extra
     *
     * REQUIRED PARAMTERES
     * record - Record data
     *
     * OPTIONAL PARAMETERS
     * template_id - Notify User - Email Template ID
     * is_redirect - Not set, redirect user
     *
     * @param boolean $is_redirect
     */
    public function update_status($module, $id, $status, $extra = array()) {
        if($module == 'orders') {
            $this->CI->load->model('orders/orders_model');
            $order = $this->CI->db->where('id', $id)->get('orders')->row();
            if($order) {
                $this->CI->orders_model->update_status($id, $order->order_id, $order, $status, (int) $this->CI->input->post('notify_customer_' . $id), true);
                $message = sprintf(lang('msg_action_success'), lang('singular_' . $module), lang('label_updated')) . ' ';
                $type = 'success';
            } else {
                $message = lang('msg_nothing_to_update');
                $type = 'danger';
            }
        } else {
            $table = (!empty($prefix) ? $prefix . '_' : '') .  $module;
            if(!isset($extra['record'])) {
                $record = $this->CI->db->where('id', $id)->get($table)->row();
            } else {
                $record = $extra['record'];
            }
            if(!$record) {
                return false;
            }
            $message = '';
            if($record->status_id != $status) {
                $data = array(
                    'status_id' => $status,
                    'modified_on' => date('Y-m-d H:i:s'),
                    'modified_ip' => $this->CI->input->ip_address()
                );
                $this->CI->db->where('id', $id)->update($table, $data);
                if($table == 'commissions') {
                    // also update status in the commission_affiliates table
                    // get commission_affiliates status first, do not update it, if it is already paid
                    unset($data['modified_ip']);
                    $this->CI->load->model('commissions/commissions_model');
                    $this->CI->commissions_model->update_commission_affiliates($id, $data);
                }
                $message = sprintf(lang('msg_action_success'), lang('singular_' . $module), lang('label_updated')) . ' ';
                $type = 'success';
            } else {
                $message = lang('msg_nothing_to_update');
                $type = 'danger';
            }
            if(isset($_POST['notify_customer_' . $id]) && !empty($extra['template_id'])) {
                $this->send_email($module, $extra['template_id'], $record->id, array('record' => $record));
                $message .= lang('msg_customer_notified');
                $type = 'success';
            }
        }
        if(!isset($extra['is_redirect'])) {
            Template::set_message($message, $type);
            Template::redirect(current_url());
        }
    }
    
    /**
     * The statuses table fields are common for all modules
     * Use this method to get counts
     * Get Filter
     * @param string $module
     * $data attributes
     * @param string table - Name of the table to be queried
     * @param string prefix - Name of the table prefix
     * @param string user_id - ID of the user
     * @param string field - Name of the column to be used for filteration
     * @return NULL[]
     */
    public function get_filter($module, $data = array()) {
        $column_name = '';
        switch($module) {
            case 'comment_statuses':
            case 'incorrect_types':
            case 'statuses':
                $column_name = 'status_id';
                break;
            case 'groups':
                $column_name = 'group_id';
                break;
        }
        $table_prefix = (!empty($data['prefix']) ? $data['prefix'] . '_' : '');
        $module =  $table_prefix. $module;
        $result = array();
        $records = $this->CI->db->get($module)->result();
        if($records) {
            $where = array();
            if(!empty($data['user_id'])) {
                $field_name = isset($data['field']) ? $data['field'] : 'user_id';
                $where[$field_name] = $data['user_id'];
            }
            $count_table = isset($data['table']) ? $data['table'] : $module;
            foreach($records as $record) {
                $where[$column_name] = $record->id;
                $result[$record->id] = array(
                    'id' => $record->id,
                    'name' => !empty($record->name_lang) ? lang($record->name) : $record->name,
                    'counts' => $this->CI->db->where($where)->count_all_results($count_table)
                );
            }
        }
        return $result;
    }
}