<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed'); 

/*
Alessandro Minoccheri
V 1.1.0
09-04-2014
https://github.com/AlessandroMinoccheri
*/

class CurrencyConverter {
    /**
     * Stores the CodeIgniter core object.
     *
     * @access public
     *
     * @var object
     */
    public $CI;
    public $cz_cache;
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        $this->CI =& get_instance();
        $this->CI->load->library('gzf_cache', array('lifetime' => 43200)); // 12 hours storage
        $this->CI->load->library('slug');
        if(method_exists($this->CI->gzf_cache, 'get_instance')) {
            $this->cz_cache = $this->CI->gzf_cache->get_instance();
        }
    }
    
    /**
     * Convert Currency
     * @param string $fromCurrency
     * @param string $toCurrency
     * @param float $amount
     * @param number $saveIntoDb
     * @param number $hourDifference
     * @return string
     */
    public function convert($fromCurrency, $toCurrency, $amount, $saveIntoDb = 1, $hourDifference = 1) {
        if($fromCurrency != $toCurrency) {
            $rate = 0;

            if ($fromCurrency=="PDS")
                $fromCurrency = "GBP";
            
            if($saveIntoDb == 1){
                $this->_checkIfExistTable();

                $this->CI->db->select('*');
                $this->CI->db->from('currency_rates');
                $this->CI->db->where('currency_from', $fromCurrency);
                $this->CI->db->where('currency_to', $toCurrency);
                $query = $this->CI->db->get();
                $find = 0;

                foreach ($query->result() as $row) {
                    $find = 1;
                    $last_updated = $row->modified;
                    $now = date('Y-m-d H:i:s');
                    $dStart = new DateTime($now);
                    $dEnd = new DateTime($last_updated);
                    $diff = $dStart->diff($dEnd);

                    if(((int)$diff->y >= 1) || ((int)$diff->m >= 1) || ((int)$diff->d >= 1) || ((int)$diff->h >= $hourDifference) || ((double)$row->rates == 0)){
                        $rate = $this->_getRates($fromCurrency, $toCurrency);

                        $data = array(
                            'currency_from'  => $fromCurrency,
                            'currency_to' => $toCurrency,
                            'rates' => $rate,
                            'modified' => date('Y-m-d H:i:s'),
                         );

                         $this->CI->db->where('id', $row->id);
                         $this->CI->db->update('currency_rates',$data);     
                    }
                    else{
                        $rate = $row->rates;
                    }
                }

                if($find == 0){
                    $rate = $this->_getRates($fromCurrency, $toCurrency);

                    $data = array(
                        'currency_from'  => $fromCurrency,
                        'currency_to' => $toCurrency,
                        'rates' => $rate,
                        'created' => date('Y-m-d H:i:s'),
                        'modified' => date('Y-m-d H:i:s'),
                    );

                    $this->CI->db->insert('currency_rates',$data); 
                }

                $value = (double)$rate * (double)$amount;

                return number_format((double)$value, 2, '.', '');
            } else {
                $rate = $this->_getRates($fromCurrency, $toCurrency);
                $value = (double)$rate * (double)$amount;

                return number_format((double)$value, 2, '.', '');
            }
        } else {
            return number_format((double)$amount, 2, '.', '');
        }
    }

    /**
     * Get Rates
     * @param string $fromCurrency
     * @param string $toCurrency
     */
    public function _getRates($fromCurrency, $toCurrency){        
        $conversion = $fromCurrency . '_' . $toCurrency;
        $url = 'https://api.goclixy.com/rest/currency_rate?from=' . $fromCurrency . '&to='. $toCurrency;
        // https://github.com/fawazahmed0/currency-api#readme
        // https://cdn.jsdelivr.net/gh/fawazahmed0/currency-api@1/2022-05-26/currencies/usd.json        
        // Get API key online
        // $url = 'http://free.currencyconverterapi.com/api/v3/convert?q='. $conversion .'&compact=ultra&key=' . $key;
        // Yahoo finance API Discontinued 2017-11-06
        // $url = 'http://finance.yahoo.com/d/quotes.csv?e=.csv&f=sl1d1t1&s='. $fromCurrency . $toCurrency .'=X';
        $handle = @fopen($url, 'r');
         
        if ($handle) {
            $result = fgets($handle, 4096);
            fclose($handle);
        }

        if(!empty($result)){
            // Yahoo Finance API
            // $allData = explode(',',$result); /* Get all the contents to an array */
            // $rate = $allData[1];
            // FREE CURRENCY CONVERTER API
            $allData = json_decode($result);
            $rate = (float) $allData->{$conversion};
        }
        else{
            $rate = 0;
        }
        return $rate;
    }
    
    /**
     * Convert amount to a specific currency
     * @param string $to
     * @param float $amount
     * @param string $from
     * @param boolean $return_rate - TRUE - Return USD Rate
     * @param float $rate_usd - Saved USD Rate - View Order/Invoice
     * @return float|number
     */
    public function amount_conversion($to, $amount, $from = null, $return_rate = false, $rate_usd = 0, $record_to = array(), $record_from = array()) {
        $cacheID = $to . '_' . $amount . '_' . $from . '_' . $return_rate . '_' . $rate_usd;
        $cacheID = str_replace('-', '_', $this->CI->slug->create_slug($cacheID));
        if(!$result = $this->cz_cache->load($cacheID)) {
            $this->CI->load->model('currencies/currencies_model');
            $symbol = settings_item('site.currency_code');
            $amount_usd = $amount;
            $rate = 1;
            if(empty($from)) {
                $from = settings_item('site.currency_code');
            }
            if($to == $from) { // both currencies are same, return the amount as it is
                return $amount;
            }
            if(empty($record_to)) {
                $record_to = (array) $this->CI->currencies_model->get_currency($to);
            }
            if(!empty($record_to)) {
                // get default currency rates
                if(empty($record_from)) {
                    $record_from = (array) $this->CI->currencies_model->get_currency($from);
                }
                if(empty($record_from)) {
                    return $amount;
                }
                // convert amount to USD
                if($record_from['rate_usd'] != 1) {
                    $amount_usd = $amount / round($record_from['rate_usd'], 4);
                }
                if($record_to['rate_usd'] != 1) {
                    // use USD amount to convert to the final amount
                    $amount = decimal_convert($amount_usd * round($record_to['rate_usd'], 4));
                    $rate = $record_to['rate_usd'];
                    $symbol = $record_to['symbol']; // to be displayed with amount
                }
            }
            if($return_rate) {
                $result = array('amount' => $amount, 'rate' => $rate, 'symbol' => $symbol);
            } else {
                $result = $amount;
            }
            $this->cz_cache->save($result, $cacheID, array('currencies'));
        }        
        return $result;
    }

    /**
     * Create Table if not exists
     * @return boolean
     */
    private function _checkIfExistTable(){
        if ($this->CI->db->table_exists('currency_rates') ){
            return(true);
        }
        else{
            $this->CI->load->dbforge();
            $this->CI->dbforge->add_field(array(
                'id' => array(
                    'type' => 'INT',
                    'constraint' => 11,
                    'unsigned' => TRUE,
                    'auto_increment' => TRUE
                ),
                'currency_from' => array(
                    'type' => 'VARCHAR',
                    'constraint' => '5',
                    'null' => FALSE
                ),
                'currency_to' => array(
                    'type' => 'VARCHAR',
                    'constraint' => '5',
                    'null' => FALSE
                ),
                'rates' => array(
                    'type' => 'VARCHAR',
                    'constraint' => '10',
                    'null' => FALSE
                ),
                'created' => array(
                    'type' => 'DATETIME'
                ),
                'modified' => array(
                    'type' => 'DATETIME'
                )
            ));

            $this->CI->dbforge->add_key('id', TRUE);
            $this->CI->dbforge->create_table('currency_rates',TRUE);
        } 
    }
}
?>