<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
 || ##################################################################### ||
 || # GoClixy - Business Directory Script							   # ||
 || # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
 || # ----------------------------------------------------------------  # ||
 || # This file is owned and copyrighted by GoClixy.com. Your license   # ||
 || # confers  no  title  or ownership in the Script and is not a sale  # ||
 || # of any rights to the Software. This file may not be redistributed # ||
 || # in whole or significant part.									   # ||
 || # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
 || # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
 || ##################################################################### ||
 \*=======================================================================*/
class Ecommerce {
    /**
     * Stores the CodeIgniter core object.
     *
     * @access public
     *
     * @var object
     */
    public $CI;
    public $ez_cache;
    
    protected $table_name	            = 'product_attributes';
    protected $table_attributes	        = 'product_attributes';
    protected $table_attribute_values   = 'product_attribute_values';
    protected $table_variants	        = 'product_variants';
    protected $table_variant_values	    = 'product_variant_values';
    protected $table_notifications      = 'product_notifications';
    protected $table_wishlists          = 'product_wishlists';
    protected $table_categories         = 'product_categories';
    protected $table_options            = 'product_options';
    protected $table_tags               = 'product_tags';
    protected $table_products           = 'products';
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct($config = array()) {
        $this->CI =& get_instance();
        $this->CI->load->library('gzf_cache', array('lifetime' => 900));
        $this->CI->load->library('slug');
        if(method_exists($this->CI->gzf_cache, 'get_instance')) {
            $this->ez_cache = $this->CI->gzf_cache->get_instance();
        }		
    }    
    
    /**
     * Detail Page - Get a specific product
     * @param number $id
     * @return array
     */
    public function get_product($id, $get_brand = false, $active_only = true, $with_package = false) {
        $sql = "SELECT M.*, " . user_fields() . PHP_EOL;
        if($get_brand) {
            $sql .= ", PM.name AS product_brand";
            $sql .= ", (SELECT GROUP_CONCAT(CONCAT(id, '!~!', name, '!~!', slug, '!~!', status, '!~!product_categories') ORDER BY id ASC SEPARATOR '~~~') FROM " . $this->CI->db->dbprefix . "product_categories WHERE FIND_IN_SET(id, M.category_ids)) AS categories" . PHP_EOL;
        }
        if($with_package) {
            $sql .= ", PKG.commissions";
        }
        $sql .= " FROM " . $this->CI->db->dbprefix . "products M" . PHP_EOL;
        if($get_brand) {
            $sql .= " LEFT JOIN " . $this->CI->db->dbprefix . "listings PM On PM.id = M.brand_id";
        }
        if($with_package) {
            $sql .= " LEFT JOIN " . $this->CI->db->dbprefix . "packages PKG On PKG.id = M.package_id";
        }
        $sql .= " LEFT JOIN " . $this->CI->db->dbprefix . "users U ON U.id = M.listing_id" . PHP_EOL;
        $sql .= " WHERE M.id = " . $id;
        if($active_only) {
            $sql .= " AND M.status = 1 AND M.deleted = 0";
        }
        return $this->CI->db->query($sql)->row();
    }
    
    /**
     * Get Product Upgrade
     * @param number $item_id
     * @param number $from_product_id
     * @param number $to_product_id
     * @return object
     */
    public function get_product_upgrade($item_id = 0, $from_product_id = 0, $to_product_id = 0) {
        if(!empty($item_id)) {
            $record = $this->CI->db->where('id', $item_id)->get($this->table_upgrades)->row();
        } else {
            $record = $this->CI->db->where(array('from_product_id' => $from_product_id, 'to_product_id' => $to_product_id))->get($this->table_upgrades)->row();
        }
        return $record;
    }
    
    /**
     * Upgrade Price Calculations - Member Area/Checkout Page
     * @param array $from_product
     * @param array $upgrade_item - Item from the product_upgrades table
     */
    public function upgrade_price($from_product, $upgrade_item) {
        if(empty($upgrade_item)) {
            return $upgrade_item;
        }
        $offer_price = !empty($from_product->offer_price) ? $from_product->offer_price : $from_product->price;
        $savings = 0;
        $savings_in_percentage = 0;
        if($upgrade_item->value_type == 'F') {
            $price = $upgrade_item->value;
            // Note: Here is an assumption that the value is lesser than the product price
            if($offer_price > $price) {
                $savings = $offer_price - $price;
            }
        } else {
            $savings = (($offer_price * $upgrade_item->value) / 100);
            $price = $offer_price - $savings;
            $savings_in_percentage = $upgrade_item->value;
        }
        $upgrade_item->savings = $savings;
        $upgrade_item->savings_in_percentage = $savings_in_percentage;
        $upgrade_item->price = $price;
        return $upgrade_item;
    }
    
    /**
     * Get Variant
     * @param number $id
     * @return object
     */
    public function get_variant($id) {
        return $this->CI->db->where('id', $id)->get($this->table_variants)->row();
    }
    
    /**
     * Update Item on Variant Select
     * @param integer $product_id
     * @param array $values
     * @param integer $variant_id = Display specific variant to a user
     * @return array|array|NULL[]
     */
    public function update_item($product_id, $values = array(), $variant_id = 0) {
        $result = array();
        if(!empty($variant_id)) {
            $stock = $this->get_variant($variant_id);
        } else {
            if (empty($values)) {
                return $result;
            }
            foreach ($values as $key => $value) {
                $value = (int) $value;
                if ($value == 0) {
                    unset($values[$key]);
                }
            }
            if (empty($values)) {
                return $result;
            }
            $where = "PV.product_id = " . $product_id . " AND ";
            if (count($values) == 1) { // The user selected only one value
                // get the first value and display it to the user
                $where .= " PV.value_id = " . reset($values); // First value of the array regardless of KEY
            } else {
                $i = 1;
                $where .= "(";
                foreach ($values as $value) {
                    $prefix = "PV" . $i;
                    if($i != 1) {
                        $where .= "AND";
                    }
                    $where .= PHP_EOL;
                    $where .= " EXISTS (SELECT * FROM " . $this->db->dbprefix . $this->table_variant_values . " AS " . $prefix . " WHERE " . $prefix . ".variant_id = PV.variant_id AND " . $prefix . ".value_id  = " . $value . " AND " . $prefix . ".product_id = " . $product_id . ")" . PHP_EOL;
                    $i++;
                }
                $where .= ")";
            }
            $stock = $this->get_variant_with_stock($where);
        }
        if(empty($stock)) {
            // Variant does not exist
            $selected_variants = $this->CI->session->userdata('selected_variants');
            
            // Get the user selected variant
            $selected_option = array_diff($values, $selected_variants);
            if(!empty($selected_option)) {
                $where = "PV.product_id = " . $product_id . " AND PV.value_id = " . reset($selected_option);
                $stock = $this->get_variant_with_stock($where);
            }
        }
        if ($stock) {
            $result = array(
                'variant_id' => $stock->id,
                'sku' => $stock->sku,
                'price' => $stock->price,
                'stock' => $stock->stock
            );
            // get other variants of this particular unit
            $other_variants = $this->CI->db->get_where($this->table_variant_values, array(
                'variant_id' => $stock->id
            ))->result();
            if ($other_variants) {
                // To display Attributes Name and Values on Load Cart
                $attributes = $this->get_attributes(true);
                $attribute_values = $this->get_values(0, true);
                foreach ($other_variants as $other_variant) {
                    $result['values'][$other_variant->attribute_id] = $other_variant->value_id;
                    if(isset($attributes[$other_variant->attribute_id]) && isset($attribute_values[$other_variant->value_id])) {
                        $result['attributes'][$attributes[$other_variant->attribute_id]] = $attribute_values[$other_variant->value_id];
                    }
                }
            }
        }
        
        // Unset Selected Variants
        $this->CI->session->unset_userdata('selected_variants');
        return $result;
    }
    
    /**
     * Checkout Page - Display Relation Item information in the summary page to the Admin and Member
     * @param number $product_id
     * @param number $variant_id
     * @return object
     */
    public function get_product_or_variant($product_id, $variant_id = 0) {
        $cacheID = 'checkout_product_relation_item_' . $product_id . '_' . $variant_id;
        if(!$result = $this->ez_cache->load($cacheID)) {
            $sql = "SELECT P.title AS name";
            if(!empty($variant_id)) {
                $sql .= ", V.name as variant, V.sku";
            }
            $sql .= " FROM " . $this->CI->db->dbprefix . $this->table_products . " P";
            if(!empty($variant_id)) {
                $sql .= " LEFT JOIN " . $this->CI->db->dbprefix . $this->table_variants . " V ON V.product_id = P.id";
            }
            $sql .= " WHERE P.id = " . $product_id;
            if(!empty($variant_id)) {
                $sql .= " AND V.id = $variant_id";
            }
            $result = $this->CI->db->query($sql)->row();
            $this->ez_cache->save($result, $cacheID, array('products'));
        }
        return $result;
    }
    
    /**
     * Get Attributes
     */
    public function get_attributes($options = false) {
        $result = $this->CI->db->get($this->table_attributes)->result_array();
        if($options) {
            return $this->to_options($result);
        } else {
            return $result;
        }
    }
    
    /**
     * Get Values
     * @param number $attribute_id
     */
    public function get_values($attribute_id = 0, $options = false) {
        if($attribute_id != 0) {
            $result = $this->CI->db->get_where($this->table_attribute_values, array('attribute_id' => $attribute_id))->result_array();
        } else {
            $result = $this->CI->db->get($this->table_attribute_values)->result_array();
        }
        if($options) {
            return $this->to_options($result, 'value');
        } else {
            return $result;
        }
    }
    
    /**
     * Convert records in to options array e.g. key => value
     * @param array $records
     * @return array
     */
    private function to_options($records, $field = 'name') {
        if(!empty($records)) {
            $result = array();
            foreach($records as $record) {
                $result[$record['id']] = $record[$field];
            }
            return $result;
        } else {
            return $records;
        }
    }
    
    /**
     * Get Variant with Stock
     * @param string $where
     * @return array
     */
    private function get_variant_with_stock($where = null) {
        $sql = "SELECT PU.*";
        $sql .= " FROM " . $this->CI->db->dbprefix . $this->table_variants . " PU" . PHP_EOL;
        $sql .= " INNER JOIN " . $this->CI->db->dbprefix . $this->table_variant_values . " PV ON PV.variant_id = PU.id" . PHP_EOL;
        $sql .= " WHERE PU.stock > 0";
        if (! empty($where)) {
            $sql .= " AND " . $where;
        }
        return $this->CI->db->query($sql)->row();
    }
}