<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');
/**
 *
 * CI / MySQL Closure Table Model
 *
 * @link  http://www.slideshare.net/billkarwin/models-for-hierarchical-data
 * @TODO  improve
 *
 * sql schema:
 * CREATE TABLE `closures` (
 *   `id` int(11) NOT NULL AUTO_INCREMENT,
 *   `ancestor` int(11) NOT NULL,
 *   `descendant` int(11) NOT NULL,
 *   `depth` int(11) NOT NULL,
 *   PRIMARY KEY (`id`)
 * );
 */
class Gc_closure {
    
    // To store instance of codeigniter
    public $CI;
    
    // root table
    public $table;
    
    // default closures table
    public $closure_table = 'closures';
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct($table_name = NULL, $closure_table = NULL) {
        $this->CI =& get_instance();
        $this->table = $table_name;
        if ($closure_table !== NULL) {
            $this->closure_table = $this->CI->db->dbprefix . $closure_table;
        }
    }
    
    /**
     * Fetch children.
     *
     * Example to generate nested tree:
     *
     *   $data = $this->get_children(1, TRUE, FALSE, TRUE);
     *   print_r($data);
     *
     * If level/depth specified then self will be ignore.
     *
     * @param  int      node id
     * @param  boolean  include self
     * @param  mixed    node level/depth (e.g direct children = 1)
     * @param  boolean  nestify the result
     * @return mixed    array if query true
     */
    public function get_children($node_id = 1, $self = FALSE, $level = FALSE, $nested = FALSE) {        
        $this->CI->db->select('t.*, c2.ancestor as parent, c1.depth as level');
        $this->CI->db->from($this->closure_table." c1");
        $this->CI->db->join($this->table.' t','t.id = c1.descendant');
        $this->CI->db->join($this->closure_table.' c2', 'c2.depth IN(1) AND c2.descendant = c1.descendant','LEFT'); // ugh backticking INTs in joins @TODO
        $this->CI->db->where('c1.ancestor',$node_id);
        if ( ! $self) {
            $this->CI->db->where('c1.descendant <>', $node_id);
        }
        if ($level) {
            $this->CI->db->where('c1.depth = ', $level);
        }
        $query = $this->CI->db->get();
        if ( ! $query->num_rows()) {
            return FALSE;
        }
        $result = $query->result_array();
        if ($nested AND ! $level) {
            $trees = array();
            $root = null;
            foreach ($result as $row) {
                $trees[$row['id']] = $row;
            }
            foreach ($trees as $key => $row) {
                if( ! $root) {
                    $root = $row['parent'];
                }
                $trees[$row['parent']]['children'][$key] =& $trees[$key];
            }
            $result = $trees[$root];
            if ( ! $self) {
                return $result['children'];
            }
            return isset($result['id']) ? $result : array_shift($result['children']);
        }
        return $result;
        
    }
    
    /**
     * Add a node (as last child).
     *
     * @param  int    node id
     * @param  int    target id
     * @return boolean
     */
    public function add($node_id, $target_id = 0) {
        /*$sql = 'SELECT ancestor, ' . $node_id . ', depth+1
				FROM ' . $this->closure_table . '
				WHERE descendant = '.$target_id.'
				UNION
				SELECT ' . $node_id . ',' . $node_id . ',0';
        $query = 'INSERT INTO ' . $this->closure_table . ' (ancestor, descendant,depth) (' . $sql . ')';*/
        $sql = 'SELECT ancestor, ' . $node_id . ', depth+1
				FROM '. $this->closure_table.'
				WHERE descendant = ' . $target_id . '
				UNION
				SELECT ' . $node_id . ',' . $node_id . ',0';
        $query = 'INSERT INTO ' . $this->closure_table . ' (ancestor, descendant, depth) ' . $sql;
        // $query = 'INSERT INTO ' . $this->closure_table . ' (ancestor, descendant, depth) (' . $sql . ')';
        $result = $this->CI->db->query($query);
        return $result;
    }
    
    /**
     * Check if current node has children.
     *
     * @param   int       node id
     * @return  boolean
     */
    public function has_children($node_id) {
        $this->CI->db->select('descendant')
        ->from($this->closure_table)
        ->where('ancestor', $node_id);
        $descendants = $this->CI->db->get()->result_array();
        foreach ($descendants as $k => $v) {
            $descendants[$k] = $v['descendant'];
        }
        
        $query = $this->CI->db->select('COUNT(*) as total')
        ->from($this->closure_table)
        ->where('ancestor IN (' . implode(',', $descendants) . ')')
        ->where('descendant <>',$node_id)
        ->get();
        return (bool) $query->row()->total;
        
    }
    
    /**
     * Get parent(s) of current node.
     *
     * @param  int    current node id
     * @param  mixed  level up (e.g direct parent = 1)
     * @return mixed  array if succeed
     */
    public function get_parent($node_id, $level = NULL)
    {
        
        $this->CI->db->select('t.*')
        ->from($this->table.' t')
        ->join($this->closure_table.' c','t.id = c.ancestor')
        ->where('c.descendant',$node_id)
        ->where('c.ancestor <>',$node_id);
        if ($level) {
            $this->CI->db->where('c.depth', $level);
        }
        $this->CI->db->order_by('t.id');
        $query = $this->CI->db->get();
        if ($query->num_rows()) {
            if ($level) {
                return $query->row();
            }
            return $query->result();
        }
        return FALSE;
    }
    
    /**
     * TODO: optional recursion
     *
     * Delete node.
     *
     * @param  int      node id
     * @param  boolean  if TRUE, it will also delete from reference table
     * @return mixed
     */
    public function delete($node_id, $delete_reference = TRUE) {        
        $operand = 'select descendant as id from '.$this->closure_table.' where ancestor = '.$node_id;
        $query = 'select id, descendant from '.$this->closure_table.' where descendant IN ('.$operand.')';
        $result_start = $this->CI->db->query($query);
        if ( $result_start->num_rows() > 0 ) {
            $descendants = array_column($result_start->result(), 'id');
            if(!empty($descendants)) {
                $result_delete = $this->CI->db->where('id IN (' . implode(',', $descendants) . ')')->delete($this->closure_table);
                if ($delete_reference) {
                    $descendants = array_column($result_start->result(), 'descendant');
                    $delete_refs = $this->CI->db->where('id IN (' . implode(',', $descendants) . ')')->delete($this->table);
                }
                return $result_delete;
            }
        }
        return FALSE;
    }
    
    /**
     * Move node with its children to another node.
     *
     * @link  http://www.mysqlperformanceblog.com/2011/02/14/moving-subtrees-in-closure-table/
     *
     * @param  int  node to be moved
     * @param  int  target node
     * @return void
     */
    public function move($node_id, $target_id) {
        // MySQLs multi-table DELETE
        $query1 = 'DELETE a FROM '.$this->closure_table.' AS a ';
        $query1 .= 'JOIN '.$this->closure_table.' AS d ON a.descendant = d.descendant ';
        $query1 .= 'LEFT JOIN '.$this->closure_table.' AS x ';
        $query1 .= 'ON x.ancestor = d.ancestor AND x.descendant = a.ancestor ';
        $query1 .= 'WHERE d.ancestor = '.$node_id.'  AND x.ancestor IS NULL';
        $res1 = $this->CI->db->query($query1);
        $query2 = 'INSERT INTO '.$this->closure_table.' (ancestor, descendant, depth) ';
        $query2 .= 'SELECT a.ancestor, b.descendant, a.depth+b.depth+1 ';
        $query2 .= 'FROM '.$this->closure_table.' AS a JOIN '.$this->closure_table.' AS b ';
        $query2 .= 'WHERE b.ancestor = '.$node_id.' AND a.descendant = '.$target_id;
        $res2 = $this->CI->db->query($query2);
        return $res1 AND $res2;
    }
    
    /**
     * Get (all) root nodes.
     */
    public function get_root() {
        $this->CI->db->select('r.descendant');
        $this->CI->db->from($this->closure_table." r");
        $this->CI->db->join($this->closure_table." p","r.descendant = p.descendant AND p.ancestor <> p.descendant","LEFT");
        $this->CI->db->where('p.descendant IS NULL',NULL);
        $result = $this->CI->db->get();
        if ($result) {
            return array_column($result->result(),'descendant');
        } else {
            return false;
        }
    }
}