<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
 || ##################################################################### ||
 || # GoClixy - Business Directory Script							   # ||
 || # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
 || # ----------------------------------------------------------------  # ||
 || # This file is owned and copyrighted by GoClixy.com. Your license   # ||
 || # confers  no  title  or ownership in the Script and is not a sale  # ||
 || # of any rights to the Software. This file may not be redistributed # ||
 || # in whole or significant part.									   # ||
 || # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
 || # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
 || ##################################################################### ||
 \*=======================================================================*/
class Image_handling {
    /**
     * Store images in a Temporary Table
     *
     * @access public
     *
     * @var object
     */
    public $CI;
    public $url = null;
    public $image_path = 'assets/images/temp';
    
    /** @var string $table - Name of the Table */
    protected $table = 'tmp_images';
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct($config = array()) {
        $this->CI =& get_instance();
        $this->CI->load->library('gc');
        $this->CI->load->library('goclixy/gcache');
    }
    
    /**
     * Allow user to upload photo
     */
    public function upload_photos($module) {
        if(empty(settings_item('site.minimal')) && !$this->CI->db->table_exists($this->table)) {
            $this->CI->load->dbforge();
            $fields = array(
                'id' => array(
                    'type' => 'INT',
                    'constraint' => 11,
                    'unsigned' => TRUE,
                    'auto_increment' => TRUE
                ),
                'module_id' => array(
                    'type' => 'TINYINT',
                    'constraint' => '1',
                    'default' => 0,
                ),
                'row_id' => array(
                    'type' => 'VARCHAR',
                    'constraint' => '50',
                    'default' => null,
                ),
                'title' => array(
                    'type' => 'VARCHAR',
                    'constraint' => '255',
                    'unique' => TRUE,
                    'default' => null
                ),
                'url' => array(
                    'type' => 'VARCHAR',
                    'constraint' => '255',
                    'unique' => TRUE,
                ),
                'priority' => array(
                    'type' =>'INT',
                    'constraint' => '11',
                    'default' => 0,
                ),
                'type_id' => array(
                    'type' => 'TINYINT',
                    'constraint' => '1',
                    'default' => 0,
                ),
                'status' => array(
                    'type' => 'TINYINT',
                    'constraint' => '1',
                    'default' => 0,
                )
            );
            // LISTINGS TABLE
            $this->CI->dbforge->add_field($fields);
            $this->CI->dbforge->add_key('id', true);
            $this->CI->dbforge->create_table($this->table);
        }
        // Generate a New Temp ID on each listing
        $this->CI->session->unset_userdata('tmp_id');
        $this->tmp_id();
        
        $user_limit = 3;
        $output = $this->perform('view', $module);
        $help_image = sprintf(lang('help_msg_images'), $user_limit, lang('label_review')) . '<br />';
        $help_image .= sprintf(lang('image_help'), settings_item('lst.image_file_size'), settings_item('lst.image_width'), settings_item('lst.image_height'));
        
        $help = <<<EOL
<div class="alert alert-warning alert-success fade show col-sm-12">
	<a class="close" data-dismiss="alert">&times;</a>
	{$help_image}
</div>

EOL;
	$output = $this->perform('view', $module);;
	Assets::add_css($output->css_files);
	Assets::add_js($output->js_files);
	$hidden_field = '<input type="hidden" name="tmp_id" class="tmp_id" value="' . $this->CI->session->userdata('tmp_id') . '" />';
	$output->output = $help . PHP_EOL . $hidden_field . PHP_EOL . '<script type="text/javascript" src="//ajax.googleapis.com/ajax/libs/jquery/2.0.0/jquery.min.js"></script>' . PHP_EOL . $output->output;
	Template::set('output', $output->output);
    }
    
    /**
     * Generate a Temporary ID
     */
    private function tmp_id() {
        // Unset this session id on form submit so that the user can add a new listing with new session
        // Before redirecting to the activation page
        // Empty tmp_images table on submit and also delete all temporary images
        if(!$this->CI->session->userdata('tmp_id')) {
            $tmp_id = time() . rand(1,20) . ip2long($this->CI->input->ip_address());
            $this->CI->session->set_userdata('tmp_id', $tmp_id);
        }
    }
    
    /**
     * Perform Action
     * @param string $action
     * @param string $module
     * @return StdClass
     */
    public function perform($action = 'view', $module = 'comments') {
        $module_id = gc_module_id($module);
        $module_id = $module_id == 0 ? 1 : $module_id;
        $user_limit = 3;
        $row_id = $this->CI->session->userdata('tmp_id');
        $images_count = $this->CI->gcache->get($this->table, array('where' => 'row_id = ' . $row_id, 'counts' => 1));// $this->CI->db->where('row_id', $row_id)->count_all_results($this->table);
        //Load image_crud config and libraries file for upload_images function
        /* --------------------------------- */
        $this->CI->load->config('image_crud');
        $this->CI->load->library('image_crudd');
        $this->CI->load->library('image_moo');
        /* --------------------------------- */
        $image_crud = new image_CRUDD();
        $image_crud->set_primary_key_field('id');
        $image_crud->set_url_field('url');
        $image_crud->set_title_field('title');
        $extra_fields = array(
            'module_id' => $module_id,
            'row_id' => $row_id,
            'status' => 0,
            'limit' => $user_limit,
            'count' => $images_count,
            'role' => 4
        );
        $image_crud->set_extras_field($extra_fields);
        $image_path = 'assets/images/temp';
        
        $image_crud->set_table($this->table)->set_relation_field('row_id')->set_ordering_field('priority')->set_image_path($image_path); // CHANGE THIS FOR PATH
        switch($action) {
            case 'upload':
            case 'list':
            case 'ajax_list':
            case 'delete_file':
                $image_crud->render();
                break;
            default:
                return $image_crud->render();
        }
    }
    
    /**
     * Move images to the correct folder
     * @param string $module
     * @param integer $row_id
     * @param integer $tmp_id
     */
    public function save_images($module, $row_id, $tmp_id, $status = 0) {
        $module_id = gc_module_id($module);
        $images = $this->CI->db->get_where($this->table, array('row_id' => $tmp_id, 'module_id' => $module_id))->result();
        if($images) {
            $this->CI->load->model('images/images_model');
            foreach($images as $image) {
                $this->CI->images_model->save_image($module, $row_id, $image->url, $status, 0, 50); // 50 - The number of images can be stored for a particular user
                // Purge images and thumbnails from the temp directory
                $file_url = FCPATH . $this->image_path . '/' . $image->url;
                $thumbnail_url = FCPATH . $this->image_path . '/thumb__' . $image->url;
                if(file_exists($file_url)) {
                    unlink($file_url);
                }
                if(file_exists($thumbnail_url)) {
                    unlink($thumbnail_url);
                }
            }
            // Delete all the uploaded images information from the tmp_images table
            $this->CI->db->where('row_id', $tmp_id)->delete($this->table);
        }
    }
    
    public function purge_images() {
        // Empty the temp table and delete all the images with thumbs in the directory
        // Delete images only using cronjob
    }
    
    /*-----------------------------------------------------*/
    /* METHODS TO BE DELETED
     *-----------------------------------------------------*/
    /**
     * Store image after success of the form submission
     * @param int $row_id
     * @param int $module_id
     * @param int $row_id
     */
    public function store_image($module_id, $row_id) {
        $row_id = $this->CI->session->userdata('row_id');
        $module = gc_module_name($module_id);
        $path = 'assets/images/' . $module;
        $images = $this->CI->db->order_by('priority', 'asc')->get_where($this->table, array('row_id' => $row_id))->result();
        if($images) {
            $i = 0;
            foreach($images as $image) {
                if(file_exists(realpath(FCPATH.'/assets/images/temp/' . $image->url))) {
                    $filename = $this->check_file_existence($module, $image->url);
                    $data = array(
                        'module_id' => $module_id,
                        'row_id' => $row_id,
                        'type_id' => 0,
                        'title' => $image->title,
                        'url' => $path . '/' . $filename,
                        'path' => $path,
                        'filename' => $filename,
                        'status' => 0
                    );
                    $this->CI->db->insert('images', $data);
                    $this->resize_image($module, $image->url);
                    // Unlink images
                    unlink(realpath(FCPATH.'/assets/images/temp/' . $image->url));
                }
                $i++;
            }
            
            // delete all images related to this session
            $this->CI->db->delete($this->table, array('row_id' => $row_id));
        }
    }
    
    /**
     * Check File Existence
     * @param string $module
     * @param string $url
     * @return string
     */
    private function check_file_existence($module, $url) {
        $image = $url;
        // Check whether the file exists in the destination folder
        if(file_exists(realpath(FCPATH.'/assets/images/' . $module . '/' . $url))) {
            $image = $url . '-' . time() . rand(1,1000);
            $tmp_image = FCPATH.'/assets/images/temp/' . $url;
            $path = FCPATH.'/assets/images/' . $module . '/' . $image;
            
            // Rename the file
            rename($tmp_image, $path);
        } else {
            $tmp_image = FCPATH.'/assets/images/temp/' . $url;
            $path = FCPATH.'/assets/images/' . $module . '/' . $url;
            copy($tmp_image, $path);
        }
        return $image;
    }
    
    /**
     * Create thumbnails of uploaded image
     * @param string $path
     * @param string $upload_path
     * @param string $file
     */
    private function resize_image($module, $file){
        if(substr((string)$file, 0, 8) == 'thumbs__') {
            // delete existing thumbs file in the same folder
            unlink(realpath(FCPATH.'/assets/images/' . $module . '/' . $file));
        } else {
            $this->CI->gc->resize_image($module, $file);
        }
    }
}