<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class Layouts {
	/**
	 * Stores the CodeIgniter core object.
	 *
	 * @access public
	 *
	 * @var object
	 */
	protected $CI;	
	protected $z_cache;
	
	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct($config = array()) {
		$this->CI =& get_instance();
		$this->CI->load->helper('gc');
		$this->CI->load->library('gc');
		$this->CI->load->library('translations');
		$this->CI->load->library('gzf_cache', array('lifetime' => 900));
		$this->CI->load->library('slug');
		$this->z_cache = $this->CI->gzf_cache->get_instance();
	}
	
	/**
	 * Featured, Popular, Recently Added and Viewed
	 * @param string $module
	 * @param string $block
	 * @param number $limit
	 * @param array $records
	 * @return array|<multitype:, object>
	 */
	public function widget_fprv($module, $block, $limit = 2, $records = array()) {
		if(!empty($records)) {
			return $records;
		}
		$result = array();
		$this->CI->load->model('listings/listings_model');
		switch($block) {
			case 'home_featured':
			case 'popular':
			    $result = $this->CI->listings_model->get_fprlv($module, $block, $limit);
				break;
			case 'featured':
				// View Page specific right featured
				// records is passed as a boolean value to this block.
				if(!is_bool($records)) {
				    $result = $this->CI->listings_model->get_fprlv($module, 'featured', $limit);
				}
				break;
			case 'recently_added':
			case 'recently':
			    $result = $this->CI->listings_model->get_fprlv($module, 'recently', $limit);
				break;
			case 'last_viewed':
				$result = $this->CI->gc->get_recently_viewed($module, $limit);
				break;
		}
		return $result;
	}
	
	/**
	 * Widget Packages
	 * @param array $packages
	 * @param string $type - 0 to display Subscription Packages, 1 to display Advertising Packages.
	 * @param string $ul_replace
	 * @param string $li_replace
	 */
	public function widget_packages($packages, $type_id = 0, $ul_replace = null, $li_replace = null) {
		$data = array();
		// Show packages to the user on home page
		$this->CI->load->model('members/members_model');
		$this->CI->load->model('packages/packages_model');
		$this->CI->load->library('crypt');
		if(empty($packages)) {
			if($type_id == 0) {
				$packages = $this->CI->members_model->get_packages();
				$this->CI->session->set_userdata('order_type', 'packages');
			} else if($type_id == 1) {
			    $this->CI->lang->load('ads/ads');
			    $packages = $this->CI->translations->get_translated_rows('ad_packages', array('id', 'title', 'description', 'icon', 'pricing_type', 'price', 'duration', 'highlight'), 'status = 1', 'display_order ASC', 50);
			    // redirect user to the checkout page
			    $this->CI->session->set_userdata('order_type', 'ads');
			} else {
			    $this->CI->lang->load('commissions/commissions');
			    $packages = $this->CI->translations->get_translated_rows('commission_levels', array('id', 'name', 'description', 'icon', 'price', 'duration', 'highlight'), 'price != 0 AND status = 1', 'priority ASC', 50, 'name AS title');
				// redirect user to the checkout page
				$this->CI->session->set_userdata('order_type', 'commission_levels');
			}
			$this->CI->session->set_userdata('subscription_type', 1);
		}
		if($type_id != 1) {
			$checkout_url = $this->CI->auth->is_logged_in() ? 'members/checkout' : 'orders/checkout';
		} else {
			$checkout_url = 'advertise';
		}
		$has_packages = isset($packages) && is_array($packages) && count($packages);
		if ($has_packages) {
			$success = true;
			foreach ($packages as $package) {
				$package = (object) $package;
				$description = $package->description;
				$extra = array('html' => '');
				if(!empty($package->offer_price)) {
				    $extra = calculate_savings($package->price, $package->offer_price);	
				    $price = currency_format($package->offer_price) . $extra['html'];			    
				} else if(!empty($package->price)) {
				    $price = currency_format($package->price);
				} else if($type_id == 0) {
				    $price = lang('label_free');
				} else {
				    $price = lang('label_campaign');
				}
				$package_info = array (
						'id' => $package->id,
						'title' => $package->title,
				        'description' => $description, // $type_id == 0 ? $this->CI->packages_model->description($package->id, $package) : $description, // Auto Fill Description
						'icon' => $package->icon,
						'currency' => settings_item('site.currency_symbol'),
				        'price_without_currency' => $package->price,
				        'price' => $price,
				        'duration' => duration_conversion($package->duration),
						'highlight' => $package->highlight,
						'checkout_url' => $checkout_url,
				        'extra' => $extra
				);
				if(isset($package->expires_on)) { // Change Package
					$package_info['order_id'] = $package->order_id;
					$package_info['expires_on'] = $package->expires_on;
					$package_info['days_before_next_invoice'] = $package->days_before_next_invoice;
				}
				$data[] = (object) $package_info;
			}
		}
		return $data;
	}
	
	/**
	 * Widget Sliders - Can be used with Modules having Commong Fields e.g. Title, Description, Hits, Created On etc.
	 * @param array $records - Existing Records passed via Controller
	 * @param string $module - Name of the module
	 * @param string $block  - Name of the Block (Featured, Popular, Recently)
	 * @param string $limit  - Number of records to be displayed
	 */
	public function widget_sliders($data = array()) {
	    $module = !empty($data['module']) ? $data['module'] : 'listings';
	    $block = !empty($data['block']) ? $data['block'] : 'recently_added';
	    $limit = !empty($data['limit']) ? (int) $data['limit'] : 10;
	    $truncate_length = !empty($data['length']) ? $data['length'] : 200;
	    $address_template = !empty($data['address_tpl']) ? $data['address_tpl'] : 2;
	    $records = !empty($data['records']) ? $data['records'] : array();
	    $keyword_wrap = !empty($data['keyword_wrap']) ? $data['keyword_wrap'] : '';
	    $thumb = empty($data['thumb']) ? false : true;
	    
		$this->CI->lang->load('listings/listings');
		// Grab Common Fields
		// Title, Image, Description, URL, Keywords, Hits, Comments, Created On (Month & Date), Author Image, Author Name
		$fields = array('title, image, description');
		$success = false;
		$data = array();
		$module_id = gc_module_id($module);
		if($module_id == 0) {
			return false;
		}
		// Get records to display to the user
		if($module == 'news') {
		    $result = $this->CI->translations->get_translated_rows('news', NULL, 'status = 1', 'created_on DESC', $limit);
		} else {
			$result = $this->widget_fprv($module, $block, $limit, $records);
		}
		if(!empty($result)) {
			if($module != 'news') {
				$this->CI->load->model('fields/fields_model');
				$result = $this->CI->fields_model->process_records($module, 'frontend', $result, array(), true);
			}
			foreach($result as $row) {
				$row = (object) $row;
				$item = array();
				
				if(isset($row->keywords)) {
					$keywords = array();
					$tags = array_map('trim', explode(', ', $row->keywords));
					foreach($tags as $tag) {
						if(!empty($tag)) {
							$tag = '<a href="' . ($module == 'listings' ? gc_url_format('search') : gc_url_format($module)) . '?q=' . urlencode($tag) . '" title="' . $tag . '">' . $tag . '</a>';
							if(!empty($keyword_wrap)) {
								$keywords[] = emmet($keyword_wrap, $tag);
							} else {
								$keywords[] = $tag;
							}
						}
					}
					$item['keywords'] = implode('', $keywords);
				}
				
				if(isset($row->comments)) {
					$item['comments'] = $row->comments;
				}
				$item['redirect_url'] = gc_url_format($module, $row->slug, $row->id, (isset($row->city) ? $row->city : null));
				$common_fields = $this->common_fields($module, $row, $truncate_length, $thumb, $address_template);
				
				$data[] = (object) array_merge($item, $common_fields);
			}
		}
		return $data;
	}
	
	/**
	 * Display Testimonial Widget to Frontend Users
	 * @param number $limit
	 * @param string $module
	 * @return multitype:
	 */
	public function widget_testimonials($limit = 10, $module = 'testimonials', $truncate_length = 200) {	
		$module = 'testimonials'; // Hardcoded, sometimes view takes value from other modules
		$data = array();
		$result = array();
		if($module == 'testimonials') {
		    $select_extra = "U.designation AS author_post, U.display_name AS author_name, U.profile_image AS author_image, U.id as author_id, U.slug AS author_slug, U.city AS author_city";
		    $join_extra = " LEFT JOIN " . $this->CI->db->dbprefix . "users U ON U.id = M.user_id";		    
		    $result = $this->CI->translations->get_translated_rows('testimonials', NULL, 'status = 1', 'priority ASC', $limit, $select_extra, $join_extra);		    
		}
		if(!empty($result)) {
			foreach($result as $row) {
				$item = array();
				if(isset($row->description)) {
				    $item['description'] = empty($truncate_length) ? $row->description : truncate_text(strip_tags($row->description), intval($truncate_length));
				}
				if(isset($row->author_name)) {
					$item['author_name'] = $row->author_name;
				}
				if(isset($row->author_image)) {
					$item['author_image'] = gc_image_url($row->author_image, true, true, 'users');
				}
				if(isset($row->author_post)) {
					$item['author_post'] = $row->author_post;
				}
				if(isset($row->author_slug)) {
					$item['author_url'] = gc_url_format('users', $row->author_slug, $row->author_id);
				}
				$data[] = (object) $item;			
			}
		}
		return $data;
	}
	
	/**
	 * Modules - Search/Category/Location View
	 * @param string $module
	 * @param array $records
	 */
	public function widget_modules_view($module, $records, $tag_url_params, $truncate_length = 400, $address_template = 2, $thumb = true) {
		$data = array();
		if(!empty($records)) {
			$this->CI->load->model('fields/fields_model');
			$records = $this->CI->fields_model->process_records($module, 'frontend', $records, array(), true);
			foreach($records as $record) {
				$item = array();
				$item['redirect_url'] = !empty($record->ad_url) ? gc_url_format('ads/click') . '?id=' . $record->ad_url : gc_url_format($module, $record->slug, $record->id, (isset($record->city) ? $record->city : null));
				if(!empty($record->tags)) {
				    // new tags format, tag_ids are saved in the modules table
				    $keywords = !empty($record->keywords) ? explode(',', $record->keywords) : array();
				    $tag_ids = !empty($record->tags) ? explode(',', $record->tags) : array();
				    $tags = array();
				    if(!empty($keywords[0])) {
    				    foreach($keywords as $k => $v) {
    				        if(isset($tag_ids[$k])) {
    				            $tag = $tag_ids[$k];
    				            $tags[] = trim($v) . '-' . $tag;
    				        }
    				    }
				    }
				    if(!empty($tags)) {
    				    $record->tags = implode(', ', $tags);
    					$item['tags'] = gc_tags_url($module, $record->tags, (!empty($record->city) ? $record->city : ''), (!empty($record->city_id) ? $record->city_id : 0), $tag_url_params);
				    }
				}	
				$common_fields = $this->common_fields($module, $record, $truncate_length, $thumb, $address_template);
				$data[] = (object) array_merge($item, $common_fields);
			}
		}
		return $data;
	}
	
	/**
	 * Module Common Fields to Display
	 * @param string $module
	 * @param object $record
	 * @param integer $truncate_length
	 * @return multitype:string date_format. currency NULL Ambigous <string, string>
	 */
	private function common_fields($module, $record, $truncate_length, $thumb = true, $address_template = true) {
		$item = array();
		$item['id'] = $record->id;
		$package_display_fields = $module == 'listings' && !empty($record->display_fields) ? json_decode($record->display_fields)->listings : null;
		if(!empty($record->cover)) {
			$item['image'] = gc_image_url($record->cover, $thumb);
		} else if(!empty($record->logo)) {
			$item['image'] = gc_image_url($record->logo, $thumb);
		} else if(!empty($record->youtube_id)) {
		    $item['youtube_id'] = $record->youtube_id;
			$item['image'] = 'https://img.youtube.com/vi/' . $record->youtube_id . '/0.jpg';
		} else if(!empty($record->image)) {
		    if($module == 'news') {
		        $item['image'] = gc_image_url($record->image); // site_url('assets/images/' . $record->image);
		    } else {
			    $item['image'] = gc_image_url($record->image, $thumb);
		    }
		} else {
		    $item['image'] = gc_site_image('no-image');
		}
		if(!empty($record->images)) {
		    $item['images'] = $record->images;
		}
		if(isset($record->title)) {
			$item['title'] = $record->title;
		}
		if(isset($record->slug)) {
			$item['slug'] = $record->slug;
		}
		// Open/Closed Now
		$item['open_status'] = 0;
		if(!empty($record->open_time) && !empty($record->close_time)) {
		    $current_time = date('H:i');
		    if($record->open_time < $current_time && $record->close_time > $current_time) {
		        $item['open_status'] = 1;
		    } else {
		        $item['open_status'] = 2;
		    }
		}
		if(settings_item('site.categories_modules') == 1 && !empty($record->category_id) && !empty($record->metas)) {
		    $metas = $this->CI->fields_model->category_metas_format($module, $record);
		    $item['metas'] = $metas;
		}
		$item['description'] = '';
		if(isset($record->description)) {
			$item['description'] = !empty($truncate_length) ? truncate_text(strip_tags($record->description), intval($truncate_length)) : $record->description;
		}
		if(gc_module_settings($module, 'locations') == 1) {
			if(isset($record->address)) {
			    if(!empty($package_display_fields->address) && !empty($record->show_address)) { // Listings Specific
					$item['address'] = $record->address;
				} else if($module != 'listings') {
					$item['address'] = $record->address;
				}
				if(!empty($address_template) && isset($item['address'])) {
					$item['address'] = gc_address_format($record->address, (isset($record->locality) ? $record->locality : ''), $record->city, $record->state, (isset($record->postcode) ? $record->postcode : ''), $record->country, $address_template);
				}
			}
			if(isset($record->locality_id)) {
				$item['locality_id'] = $record->locality_id;
			}
			if(isset($record->locality)) {
				$item['locality'] = $record->locality;
			}
			if(isset($record->city_id)) {
				$item['city_id'] = $record->city_id;
			}
			if(isset($record->city)) {
				$item['city'] = $record->city;
			}
			if(isset($record->state_id)) {
				$item['state_id'] = $record->state_id;
			}
			if(isset($record->state)) {
				$item['state'] = $record->state;
			}
			if(isset($record->country)) {
				$item['country'] = $record->country;
			}
			if(isset($record->postcode)) {
				$item['postcode'] = $record->postcode;
			}
			if(!empty($record->latitude)) {
				$item['latitude'] = $record->latitude;
			}
			if(!empty($record->longitude)) {
				$item['longitude'] = $record->longitude;
			}
		}
		// City, State and Country Information
		if(!empty($record->email)) {
			if($module == 'listings') {
			    if(isset($record->show_email) && $record->show_email == 1 && !empty($package_display_fields->email)) {
					$item['email'] = $record->email;
				}
			} else {
				$item['email'] = $record->email;
			}
		}
		if(!empty($record->saved_categories)) {
		    $item['saved_categories'] = is_string($record->saved_categories) ? explode_group_concat($record->saved_categories) : $record->saved_categories;
		}
		if(!empty($record->categories_set)) {
		    $item['category_ids'] = $record->categories_set;
		}
		if(!empty($record->claimed)) {
		    $item['claimed'] = $record->claimed;
		}
		if(!empty($record->website) && !empty($package_display_fields->website)) {
			$item['website'] = $record->website;
		}
		if(!empty($record->color_scheme)) {
			$item['color_scheme'] = $record->color_scheme;
		}
		if(!empty($record->border_color)) {
			$item['border_color'] = $record->border_color;
		}
		if(!empty($record->trusted_seal)) {
			$item['trusted_seal'] = site_url('assets/images/' . $record->trusted_seal);
		}
		if(!empty($record->featured)) {
		    $item['featured'] = $record->featured;
		}
		if(!empty($record->phone_number) || !empty($record->mobile_number)) {
		    if(!empty($package_display_fields->phone_number) && isset($record->show_phone) && $record->show_phone == 1) {
				$item['phone_number'] = $record->phone_number;
				$item['mobile_number'] = $record->mobile_number;
			}
		}
		if(isset($record->hits)) {
			$item['hits'] = $record->hits;
		}
		if(isset($record->isFavorite)) {
			$item['isFavorite'] = $record->isFavorite > 0 ? true : false;
		}
		if(isset($record->average_rating)) {
			$item['average_rating'] = $record->average_rating;
		}
		if(isset($record->total_users)) {
		    $item['total_users'] = $record->total_users;
		}
		if(isset($record->created_on)) {
			if(!empty($record->created_on) && $record->created_on != '0000-00-00 00:00:00') {
			    if($module == 'news') {
			        $item['created_on'] = $record->published_on;
			    } else {
			        $item['created_on'] = gc_date_format($record->created_on, true);
			    }
				// For News Slider
				$split_date = split_datetime($record->created_on);
				$item['created_day'] = $split_date['day'];
				$item['created_month'] = $split_date['month'];
			}
		}
		if(!empty($record->start_date)) {
			$item['start_date'] = gc_date_format($record->start_date, true);
			$split_date = split_datetime($record->start_date);
			$item['start_day'] = $split_date['day'];
			$item['start_month'] = $split_date['month'];
		}
		// Module Specific - Additional Fields
		if(isset($record->salary)) {
			$item['salary'] = $record->salary;
		}
		if(isset($record->code)) { // Coupon Expires On
			$item['code'] = $record->code;
		}
		if(isset($record->expires_on) && $record->expires_on > date('Y-m-d')) {
			$item['expires_on'] = gc_date_format($record->expires_on);
		}
		if(isset($record->squarefeet)) {
			$item['squarefeet'] = $record->squarefeet;
		}
		if(isset($record->bedrooms)) {
			$item['bedrooms'] = $record->bedrooms;
		}
		if(isset($record->bathrooms)) {
			$item['bathrooms'] = $record->bathrooms;
		}
		if(isset($record->floors)) {
			$item['floors'] = $record->floors;
		}
		if(isset($record->rooms)) {
			$item['rooms'] = $record->rooms;
		}
		if(isset($record->rent_price)) {
			$item['rent_price'] = $record->rent_price;
		}
		if(!empty($record->price)) {
			$item['price'] = currency_format((float) $record->price);
			if(!empty($record->offer_price) && empty($record->savings)) {
			    $item['savings'] = calculate_savings($record->price, $record->offer_price, 'gc-view-savings');
			    $item['offer_price'] = currency_format((float) $record->offer_price);
			}
		}
		if(isset($record->type)) {
			$item['type'] = $record->type;
		}
		if(isset($record->discount)) {
		    if(!empty($record->type_id) && $record->type_id == 'P') {
		        $item['discount'] = $record->discount . '%';
		    } else {
		        $item['discount'] = $record->discount;
		    }
		    $item['type'] = lang('label_off');
		}
		if(isset($record->venue)) { // Coupon Expires On
			$item['venue'] = $record->venue;
		}
		if(!empty($record->external_url)) {
		    $item['external_url'] = $record->external_url;
		}
		
		// display ratings to the user
		if(!empty($record->ratings)) {
		    $item['ratings'] = number_format($record->ratings, 1, '.', '');
		}
		if(!empty($record->rating_users)) {
		    $item['rating_users'] = $record->rating_users;
		}
		if(!empty($record->facilities)) {
		    $item['facilities'] = is_string($record->facilities) ? json_decode($record->facilities, true) : $record->facilities;
		}
		
		// ECommerce		
		if(!empty($record->offer_price) && $record->price > $record->offer_price && is_float($record->offer_price)) {
		    $item['offer_price'] = currency_format($record->offer_price);
		    $item['savings_in_percentage'] = decimal_convert((($record->price - $record->offer_price) * 100) / $record->price) . '%';
		}
		
		// Author Information
		if(!empty($record->listing_slug) && $module != 'listings') { // Display listing detail as the Author Information
			$item['author_name'] = $record->listing_title;
			$item['author_url'] = gc_url_format('listings', $record->listing_slug, $record->listing_id, $record->listing_city);
		} else if(!empty($record->author_slug)) { // Display User Detail as the Author Information
			$item['author_name'] = $record->author_name;
			$item['author_url'] = gc_url_format('users', $record->author_slug, $record->author_id, $record->author_city);
		}
		if(isset($record->author_image) && !empty($record->listing_logo)) {
			$item['author_image'] = gc_image_url($record->listing_logo);
		} else if(isset($record->author_image)) {
			$item['author_image'] = gc_image_url($record->author_image, true, true, 'users');
		}
		return $item;
	}
	
	/**
	 * Display Banner in multiple columns to the user
	 * @param object $banner
	 * @return string
	 */
	public function widget_banner($banner) {
		$this->CI->load->model('ads/ads_model');
		$this->CI->ads_model->click_impression_counter($banner->ad_url); // update impressions counter
		switch ($banner->type) {
			case 'image':
				$url = '<a href="' . gc_url_format('ads/click') . '?id=' . $banner->ad_url. '" rel="nofollow" target="' . $banner->target .'">' . PHP_EOL;
				$url .= '<img id="' . $banner->id . '" class="img-fluid banner d-block mx-auto" src="' . site_url('assets/images/banners/' . $banner->image) . '" alt="' . $banner->title . '" />' . PHP_EOL;
				$url .= '</a>';
				return $url;
				break;
			case 'html':
				return $banner->html_text;
				break;
			case 'text':
				return htmlspecialchars($banner->html_text);
				break;
		}
	}
	
	/**
	 * Display Random Images to the user
	 */
	public function widget_slider_folder_scan() {
		$slider_folder = "assets/images/sliders/";
		$path = FCPATH . $slider_folder;
		$images = scandir($path, 1);
		if(!empty($images)) {
			foreach($images as $key => $image) {
				if(substr($image, -4) != '.jpg' && substr($image, -4) != '.png' && substr($image, -5) != '.jpeg') {
					unset($images[$key]);
				}
			}
		}
		if(!empty($images)) {
			$random_image = $images[array_rand($images, 1)];
		}
		return site_url($slider_folder . $random_image);
	}
	
	/**
	 * Display Random Images to the user
	 */
	public function widget_slider_old($limit = 1) {
	    $record = $this->CI->db->order_by('RAND()')->limit($limit)->where(array('status' => 1))->get('banner_sliders')->result();
	    $result = array();
	    if($record) {
	        foreach($record as $slider) {
	            $result[] = array(
	                'title' => $slider->title,
	                'url' => $slider->primary_url,
	                'image' => site_url('assets/images/' . $slider->image),
	                'html_text' => $slider->html_text
	            );
	        }
	    }
	    return $limit == 1 ? (isset($result[0]) ? $result[0] : array()) : $result;
	}
	
	/**
	 * Display Random Images to the user
	 */
	public function widget_slider($data = array()) {
	    $module = 'sliders';
	    $query_where = 'M.status = 1';
	    $sort_by = !empty($data['sort_by']) ? $data['sort_by'] : 'M.priority ASC';
	    $where = array();
	    if(!empty($data['group_id'])) {
	        if(is_string($data['group_id'])) {
	            $data['group_id'] = $this->CI->gc->get_groups('banner_groups', $data['group_id']);
	        }
	        $where[] = "FIND_IN_SET(" . $data['group_id'] . ", group_ids)";
	    }
	    if(!empty($data['ids'])) {
	        if(is_string($data['ids'])) {
	            $where[] = "slug IN ('" . $data['ids'] . "')";
	        } else {
	            $where[] = "id IN (" . $data['ids'] . ")";
	        }
	    }
	    if(!empty($where)) {
	        $query_where .= ' AND ' . implode(' AND ', $where);
	    }
	    // $query_where .= ' AND M.status = 1';
	    $limit = isset($data['limit']) ? $data['limit'] : 1;
	    $records = $this->CI->gc->get_all_crud(0, $query_where, $sort_by, $limit, 0, array('module' => $module, 'prefix' => 'banner', 'cache' => !empty($data['sort_by']) ? null : 'banners'));
	    // get multi language values
	    if(settings_item('site.multi_language') == 1) {
    	    $language_code = 'en'; 
    	    $languages = $this->CI->session->userdata('site_languages');
    	    foreach($languages as $language) {
    	        if($language->label == $this->CI->session->userdata('language')) {
    	            $language_code = $language->code;
    	        }
    	    }
	    }
	    $module_id = gc_module_id('banner_' . $module);
	    if($records) { // numeric value
	        foreach($records as $key => $record) {
	            // get multi language values
	            if(settings_item('site.multi_language') == 1) {
    	            $translations = $this->CI->gc->get('translations', array('where' => "language_code = '" . $language_code . "' AND module_id = " . $module_id . " AND row_id = " . $record->id, 'result' => 1));
    	            if($translations) {
    	                foreach($translations as $translation) {
    	                    $record->{$translation->name} = $translation->value;
    	                }
    	            }
	            }
	            // get demo sliders from ecomsta api
	            if(settings_item('site.demo_mode') == 1 && $this->CI->session->userdata('theme_demo') && !empty($record->extras)) { // load theme with demo content
	                $theme_demo = $this->CI->session->userdata('theme_demo');
	                if(!empty($theme_demo['content'])) {
	                    $extras = json_decode($record->extras, true);
	                    if(!empty($extras[$theme_demo['content']])) {
	                        $slider_record = (object) $extras[$theme_demo['content']];
	                        $records[$key]->title = !empty($slider_record->title) ? $slider_record->title : $record->title;
	                        $records[$key]->html_text = !empty($slider_record->html_text) ? $slider_record->html_text : $record->html_text;
	                        $record->params = !empty($slider_record->params) ? json_encode($slider_record->params) : $record->params;
	                        $record->image = !empty($slider_record->image) ? $slider_record->image : $record->image;
	                    }
	                }
	            }
	            $records[$key]->image_url = '';
	            if(!empty($record->image)) {
	                $records[$key]->image_url = valid_url($record->image) ? $record->image : base_url('assets/images/' . $record->image);
	                $records[$key]->image = $records[$key]->image_url;
	            }
	            if(!empty($record->params)) {
	                $params = array();
	                $saved_params = json_decode($record->params, true);
	                if(!empty($saved_params)) {
	                    foreach($saved_params as $param) {
	                        switch($param['key']) {
	                            case 'background_image':
	                                $params[$param['key']] = valid_url($param['value']) ? $param['value'] : base_url('assets/images/' . $param['value']);
	                                break;
	                            default:
	                                $params[$param['key']] = $param['value'];
	                        }
	                    }
	                }
	                $records[$key]->params = $params;
	            }
	        }
	    }
	    return $limit == 1 && $records ? $records[0] : $records;
	}
	
	
	/**
	 * Display Categories to the User
	 * @param string $module - Name of the Module
	 * @param array $categories - Already passed values in the Controller file
	 * @param integer $parent_id - ID of the Parent Category to load subcategories
	 * @param number $limit - Number of records to be visible at a time
	 * @return array
	 */
	public function widget_categories($module, $categories = array(), $parent_id = 0, $limit = 50, $extra_params = array()) {
	    if(empty($categories) && gc_module_id($module) > 0) {
	        $this->CI->load->model('categories/categories_model');
	        $categories = $this->CI->categories_model->categories(0, $parent_id, $module, null, $limit, 0, $extra_params);
	    }
	    return $categories;
	}
	
	/**
	 * Display Locations to the User
	 * @param string $table - Name of the table
	 * @param string $module - Name of the module, whose locations to be displayed
	 * @param number $limit - Number of rows to be displayed
	 * @param boolean $list_only - Display for lists without any extra join
	 * @param boolean $having_counts - Display only those row which have records
	 * @return object
	 */
	public function widget_locations($table = 'countries', $module = 'listings', $limit = 10, $list_only = false, $having_counts = false) {
	    $field = 'id';
	    $select = "SELECT name, image, counts, ";
	    switch($table) {
	        case 'countries':
	            $field = 'iso';
	            $compare_field = 'country_iso';
	            $select .= 'NULL AS slug,';
	            break;
	        case 'states':
	            $compare_field = 'state_id';
	            $select .= 'slug,';
	            break;
	        default:
	            $compare_field = 'city_id';
	            $select .= 'slug,';
	            
	    }
	    $select .= ' ' . $field;
	    $sql = $select;
	    $where = $having_counts ? ' AND counts > 0' : '';
	    if($table == 'states' && settings_item('lst.country_filter') == 1) {
	        $where .= " AND country_iso = '" . $this->CI->session->userdata('search_country') . "'";
	    }
	    if($list_only) {
	        $sql .= " FROM " . $this->CI->db->dbprefix . $table . " T WHERE active = 1 " . $where . " ORDER BY T.name DESC LIMIT " . $limit;
	    } else {
    	    $sql .= ", (SELECT COUNT(*) FROM " . $this->CI->db->dbprefix . "listings L WHERE L." . $compare_field . " = T." . $field . ") As listings,";
    	    $sql .= " (SELECT COUNT(*) FROM " . $this->CI->db->dbprefix . "users U WHERE U." . $compare_field . " = T." . $field . ") As users";
    	    if($table == 'countries') {
    	        $sql .= ", (SELECT COUNT(*) FROM " . $this->CI->db->dbprefix . "cities CT";
    	        $sql .= " INNER JOIN " . $this->CI->db->dbprefix . "states S ON S.id = CT.state_id WHERE S.country_iso = T.iso) AS cities";
    	    } else if($table == 'states') {
    	        $sql .= ", (SELECT COUNT(*) FROM " . $this->CI->db->dbprefix . "cities CT WHERE CT.state_id = T.id) As cities";
    	    }
    	    $sql .= " FROM " . $this->CI->db->dbprefix . $table . " T WHERE active = 1 " . $where . " ORDER BY listings DESC LIMIT " . $limit;
	    }
	    $result = $this->CI->db->query($sql)->result();
	    if($result) {
	        foreach($result as $key => $row) {
	            foreach($row as $key_row => $value) {
	                if($key_row == 'image') {
	                    $result[$key]->$key_row = gc_image_url($value, false);
	                }
	            }
	            if($table == 'countries') {
	                $result[$key]->redirect_url = site_url(gc_uri_string('search') . '?iso=' . $row->{$field});
	            } else if($table == 'states') {
	                $result[$key]->redirect_url = site_url(gc_uri_string('search') . '?l=' . $row->{$field} . '&tl=2');
	            } else {
	                $result[$key]->redirect_url = gc_location_url($module == 'banner_sliders' ? 'listings' : $module, $row->slug, $row->{$field});
	            }
	        }
	    }
	    return $result;
	}	
	
	/**
	 * Display counts on the Home Page
	 * @return string[][]|NULL[][]
	 */
	public function widget_counter() {
	    $data = array(
	        'listings' => array(
	            'count' => $this->CI->db->where(array('deleted' => 0, 'status' => 1))->count_all_results('listings'),
	            'icon' => 'fas fa-' . gc_module_icon('listings')
	        ),
	        'users' => array(
	            'count' => $this->CI->db->where(array('deleted' => 0, 'active' => 1))->count_all_results('users'),
	            'icon' => 'fas fa-users'
	        ),
	        'categories' => array(
	            'count' => $this->CI->db->where(array('active' => 1))->count_all_results('categories'),
	            'icon' => 'fas fa-list'
	        ),
	        'cities' => array(
	            'count' => $this->CI->db->where(array('active' => 1))->count_all_results('cities'),
	            'icon' => 'fas fa-map-marker-alt'
	        )
	    );
	    return $data;
	}
	
	/**
	 * Prepare business links
	 * @param object $record
	 */
	public function widget_member_links($module, $record, $data = array()) {
	    $this->CI->load->library('crypt');
	    $id = $record->id;
	    $encrypted_id = $this->CI->crypt->encode($record->id);
	    $pyg = json_decode($record->pyg);
	    $links = array();
	    if(!empty($data['with_view']) && $record->status == 1) {
	        $links[] = array(
	            'name' => 'view',
	            'url' => gc_url_format($module, $record->slug, $record->id, (!empty($record->city) ? $record->city : null)),
	            'params' => '',
	            'label' => lang('label_view'),
	            'icon' => 'fas fa-external-link-square-alt',
	            'class' => 'success',
	            'mobile_icon' => 'open-outline',
	            'mobile_url' => '/tabs/member/modules/detail/' . $module . '/' . $record->id
	        );
	    }
	    if(!empty($data['with_edit'])) {
	        $links[] = array(
	            'name' => 'edit',
	            'url' => gc_url_format('members/edit_business'),
	            'params' => '?id=' . $encrypted_id,
	            'label' => lang('edit'),
	            'icon' => 'fas fa-pencil-alt',
	            'class' => 'primary',
	            'mobile_icon' => 'create-outline',
	            'mobile_url' => '/tabs/member/edit-module/' . $module . '/' . $record->id
	        );
	    }
	    if($module == 'listings' && empty($data['mobile'])) {
	        $links[] = array(
	            'name' => 'logo',
	            'url' => gc_url_format('members/logo'),
	            'params' => '?id=' . $encrypted_id,
	            'label' => lang('listings_action_logo'),
	            'icon' => 'fas fa-camera',
	            'class' => 'info',
	            'mobile_icon' => 'camera-outline',
	            'mobile_url' => '/image-crud/' . $module . '/' . $record->id . '/2'
	        );
	    }
	    $links[] = array(
	        'name' => 'cover',
	        'url' => gc_url_format('members/cover'),
	        'params' => '?id=' . $encrypted_id,
	        'label' => lang('label_cover_image'),
	        'icon' => 'fas fa-camera-retro',
	        'class' => 'secondary',
	        'mobile_icon' => 'image-outline',
	        'mobile_url' => '/image-crud/' . $module . '/' . $record->id . '/1'
	    );
	    if(empty($data['mobile'])) { // mobile app specific links only
	        $links[] = array(
	            'name' => 'documents',
	            'url' => gc_url_format('members/documents'),
	            'params' => '/listings/' . $id,
	            'label' => lang('file_documents'),
	            'icon' => 'fas fa-file-pdf',
	            'class' => 'danger',
	            'mobile_icon' => 'document-outline',
	            'mobile_url' => '/crud/files/0/' . $module . '/' . $record->id
	        );
	        $links[] = array(
	            'name' => 'category_fields',
	            'url' => gc_url_format('members/category'),
	            'params' => '?id=' . $encrypted_id . '&md=' . $this->CI->crypt->encode($module, true),
	            'label' => lang('additional_info'),
	            'icon' => 'fas fa-address-card',
	            'class' => 'danger',
	            'mobile_icon' => 'contract-outline',
	            'mobile_url' => '/crud/additional/' . $module . '/' . $record->id
	        );
	    }
	    $links[] = array(
	        'name' => 'photos',
	        'url' => gc_url_format('members/photos'),
	        'params' => '/listings/' . $id,
	        'label' => lang('photos'),
	        'icon' => 'far fa-image',
	        'class' => 'warning',
	        'mobile_icon' => 'images-outline',
	        // 'mobile_url' => '/image-crud/' . $module . '/' . $record->id . '/0'
	        'mobile_url' => '/crud/gallery/0/' . $module . '/' . $record->id
	    );
	    if($module == 'listings') {
	        $config_modules = gc_modules('module', false);
	        $listings_key = array_keys($config_modules, 'listings');
	        unset($config_modules[$listings_key[0]]);
	        foreach($config_modules as $config_module) {
	            if(($config_module == 'branches' || gc_module_settings($config_module) == 1) && (gc_field_value($config_module, $record->module_limit)!= 999  || $pyg->{$config_module} > 0)) {
	                $links[] = array(
	                    'name' => $config_module,
	                    'url' => gc_url_format('members/view'),
	                    'params' => '?id=' . $encrypted_id . '&md=' . $this->CI->crypt->encode($config_module, true) .  ($config_module == 'videos' ? '&tm=' . $this->CI->crypt->encode($module, true) : ''),
	                    'label' => lang($config_module),
	                    'icon' => 'fas fa-' . gc_module_icon($config_module),
	                    'mobile_icon' => gc_module_icon($config_module, true),
	                    'mobile_url' => '/modules/' . $config_module . '/search/' . $record->id . '/all/1'
	                );
	            }
	        }
	        $links[] = array(
	            'name' => 'business_hours',
	            'url' => gc_url_format('members/business_hours'),
	            'params' => '?id=' . $encrypted_id,
	            'label' => lang('business_hours'),
	            'icon' => 'far fa-clock',
	            'class' => 'primary',
	            'mobile_icon' => 'time-outline',
	            'mobile_url' => '/crud/business_hours/0/' . $module . '/' . $record->id
	        );
	        $links[] = array(
	            'name' => 'package_info',
	            'url' => '',
	            'params' => '',
	            'label' => lang('package_info'),
	            'icon' => 'fas fa-box-open',
	            'class' => 'secondary',
	            'extra_css' => 'preview-modal',
	            'extra' => ' data-href="' . site_url('ajax/package_info/' . $record->package_id) . '"',
	            'mobile_icon' => 'cube-outline',
	            'mobile_url' => 'packageInfo'
	        );
	        $links[] = array(
	            'name' => 'change_package',
	            'url' => gc_url_format('members/packages'),
	            'params' => '?id=' . $encrypted_id,
	            'label' => lang('listings_action_change_package'),
	            'icon' => 'fas fa-exchange-alt',
	            'class' => 'info',
	            'mobile_icon' => 'cube-outline',
	            'mobile_url' => '/crud/change_package/' . $module . '/' . $record->id . '/1'
	        );
	    }
	    if($module == 'products') {
	        $links[] = array(
	            'name' => 'variants',
	            'url' => site_url('members/variants/') . $record->id,
	            'params' => '',//'?id=' . $encrypted_id,
	            'label' => lang('gc_menu_variants'),
	            'icon' => 'fas fa-cash-register',
	            'class' => 'info',
	            'mobile_icon' => 'ellipsis-horizontal-circle-outline',
	            'mobile_url' => '/member/info-pages/variants/all/' . $record->id
	        );
	    }
	    if(!empty($data['booking_forms'])) {
	        $links[] = array(
	            'type' => 'button',
	            'name' => 'booking',
	            'value' => $id,
	            'url' => '',
	            'params' => '',
	            'label' => lang('label_booking_forms'),
	            'icon' => 'fas fa-calendar-alt',
	            'class' => 'btn-theme-secondary',
	            'extra' => ''
	        );
	    }
	    if(!empty($data['with_delete'])) {
	        $links[] = array(
	            'type' => 'button',
	            'name' => 'delete',
	            'value' => $id,
	            'url' => '',
	            'params' => '',
	            'label' => lang('bf_action_delete'),
	            'icon' => 'fas fa-trash-alt',
	            'class' => 'btn-danger',
	            'extra' => ' onclick="$(\'#trash\').val(' . $id . '); return confirm(\'' . js_escape(lang('delete_confirm')) . '\');"',
	            'mobile_icon' => 'trash-outline',
	            'mobile_url' => '/tabs/member/crud/removal/0/listings/' . $id
	        );
	    }
	    return $links;
	}
	
	/**
	 * Load Widget Subscribers Form
	 * @param number $group_id
	 * @param array $data -
	 * btn_label - Label of the Button
	 * no_submit - Without Submit Button
	 * field_only - true to display Field Only
	 * icon - fas fa-home
	 * fields_container - e.g. div.mt-2.d-block
	 * btn_container - e.g. div.mt-2.d-block
	 * @return string
	 */
	public function widget_subscriber($group_id, $data = array()) {
	    // Given phone-number class to the field via Field Class input field
	    // only one field can be the phone number field because of the database column issue.
	    // e.g. phone_number_1 must have phone_code_1 column in the database.
	    Assets::add_js('
            var fieldSelector = "#sf_phone_number";//"input[name=phone_number]"; // conflict with add_business form
            if($(fieldSelector).length == 1) {
                var input = document.querySelector(fieldSelector);
                var iti = window.intlTelInput(input, {
                    hiddenInput: "phone_code",
                    initialCountry: \'' . visitor_info() . '\',
                    initialDialCode: true,
                    separateDialCode: true,
                    formatOnDisplay: true,
                    nationalMode: true,
                    utilsScript: \'' . base_url() . 'assets/js/utils.js\'});
                $(".iti").css("width", "100%");
	        
                // get initial country code
                $(\'input[name=phone_code]\').val(iti.getSelectedCountryData().dialCode);
	        
                var input = $(fieldSelector);
                // listen to the telephone input for changes
                input.on(\'countrychange\', function(e) {
                    var isValid = iti.isValidNumber(); // check whether the input number is valid or not
                    // change the hidden input value to the selected country code
                    $(\'input[name=phone_code]\').val(iti.getSelectedCountryData().dialCode);
                });
        }
        ', 'inline');
	    $this->CI->load->model('subscribers/subscribers_model');
	    $form_data = $this->CI->subscribers_model->load_form($group_id, $data);
	    return $form_data;
	}
	
	/**
     * Display ads to the user
     * @param string $module - Name of the module listings/products/jobs/events etc
     * @param array $params
	 */
	public function widget_ads($module, $params = array()) {
	    $module_id = gc_module_id($module);
	    $prefix = $this->CI->db->dbprefix;
	    $extra_where = array();
	    if(!empty($params['category_id'])) { 
	        $extra_where[] = "(AK.row_id = " . $params['category_id'] . " AND AK.module_id = 12)"; // categories filter
	    }
	    if(!empty($params['city_id'])) {
	        $extra_where[] = "(AK.row_id = " . $params['city_id'] . " AND AK.module_id = 13)"; // cities filter
	    }
	    if(!empty($params['tag_id'])) {
	        $extra_where[] = "(AK.row_id = " . $params['tag_id'] . " AND AK.module_id = 16)"; // tags filter
	    }
	    $sql = "SELECT M.id AS ad_id, T.id, T.title, T.slug, T.address, T.city, T.cover, T.logo FROM";
	    $sql .= " (SELECT MD.id, MD.package_id, MD.row_id FROM " . $prefix . "ads MD";
	    $sql .= " INNER JOIN " . $prefix . "ad_keywords AK ON AK.ad_id = MD.id";
	    $sql .= " WHERE MD.status = 1 AND MD.deleted = 0 AND MD.module_id = " . $module_id;
	    if(!empty($extra_where)) {
	        $sql .= " AND (" . implode(" OR ", $extra_where) . ")";
	    }
	    $sql .= " GROUP BY MD.id) M";
	    $sql .= " INNER JOIN " . $prefix . $module . " T ON T.id = M.row_id";
	    $records = $this->CI->db->query($sql)->result();
	    if($records) {
	        foreach($records as $key => $record) {
	            if(!empty($record->cover)) {
	                $records[$key]->image = gc_image_url($record->cover);
	            } else {
	                $records[$key]->image = gc_image_url($record->logo);
	            }
	            $records[$key]->redirect_url = gc_url_format($module, $record->slug, $record->id, (isset($record->city) ? $record->city : null));
	        }
	    }
	    return $records;
	}
	
	/**
	 * Widget Footer Trending Items
	 * @param string $module
	 */
	public function widget_footer_tags($module, $limit, $order_by = 'hits', $where = null, $select = 'id, name, slug') {
	    $cacheID = 'footer_tags_' . $module . '_' . $limit;
	    if(!$records = $this->z_cache->load($cacheID)) {
	        if(!empty($where)) {
	            $this->CI->db->where($where);
	        }
	        if(!empty($order_by)) {
	            $this->CI->db->order_by('hits', 'DESC');
	        }
	        $records = $this->CI->db->select($select)->limit($limit)->get($module)->result();
	        $this->z_cache->save($records, $cacheID, array($module));
	    }
	    return $records;
	}
	
	/**
	 * Widget Link Description
	 */
	public function widget_link_description($page_url = null) {
	    $page_url = !empty($page_url) ? $page_url : current_url();
	    $cacheID = 'link_' . str_replace('-', '_', $this->CI->slug->create_slug($page_url));
	    if(!$link = $this->z_cache->load($cacheID)) {
	        $this->CI->load->model('links/links_model');
	        $link = $this->CI->links_model->get_all_links($page_url);
	        $this->z_cache->save(array('info' => $link), $cacheID, array('links'));
	    }
	    return empty($link['info']['description']) ? null : $link['info']['description']; // lang('heading_link_text')
	}
	
	/**
	 * Training Videos Groups
	 * @param number $group_id
	 */
	public function widget_training_videos($group_id = 0) {
	    $cacheID = 'training_videos_' . $group_id;
	    if(!$result = $this->z_cache->load($cacheID)) {
    	    $where = 'status = 1 AND deleted = 0';
    	    if(!empty($group_id)) {
    	       $where .= " AND FIND_IN_SET(" . $group_id . ", group_ids)";
    	    }
    	    $records = $this->CI->db->where($where)->get('training_videos')->result();
    	    $filter_groups = array();
    	    if(!empty($records)) {
        	    $group_ids = array();
        	    foreach($records as $record) {
        	       $ids_group = explode(',', $record->group_ids);
        	       if(!empty($ids_group[0])) {
        	           foreach($ids_group as $id_group) {
        	               $group_ids[$id_group] = $id_group;
        	           }
        	       }
        	    }
        	    if(!empty($group_ids)) {
        	       $filter_groups = $this->CI->db->where('id IN (' . implode(',', $group_ids) . ')')->get('training_groups')->result();
        	    }
    	    }
    	    $result = array('records' => $records, 'filter_groups' => $filter_groups);
    	    $this->z_cache->save($result, $cacheID, array('training_videos'));
	    }
	    return $result;
	}
}
?>