<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
 || ##################################################################### ||
 || # GoClixy - Online Software Solutions							   # ||
 || # Copyright (C) 2021 GoClixy. All Rights Reserved.				   # ||
 || # ----------------------------------------------------------------  # ||
 || # This file is owned and copyrighted by GoClixy.com. Your license   # ||
 || # confers  no  title  or ownership in the Script and is not a sale  # ||
 || # of any rights to the Software. This file may not be redistributed # ||
 || # in whole or significant part.									   # ||
 || # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
 || # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
 || ##################################################################### ||
 \*=======================================================================*/
class Gcache {
    /**
     * Stores the object for a longer period.
     * To be used in helper files
     *
     * @access public
     *
     * @var object
     */
    protected $CI;
    protected $obj_gcache;
    protected $dbprefix;
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct($config = array()) {
        $this->CI =& get_instance();
        $this->CI->load->library('gzf_cache', array('lifetime' => 604800)); // one week time
        $this->CI->load->library('slug');
        if(method_exists($this->CI->gzf_cache, 'get_instance')) {
            $this->obj_gcache = $this->CI->gzf_cache->get_instance();
        }
        $this->dbprefix = $this->CI->db->dbprefix;
    }
    
    /**
     * Get routes from the routes_lang.php files for gc_uri_string function
     * @param string $key
     * @return string
     */
    public function routes($key = null) {
        $cacheID = 'gc_routes';        
        if(!$result = $this->obj_gcache->load($cacheID)) {
            $this->CI->load->helper('translate/languages');
            $result = load_lang_file('routes_lang.php', 'english');
            $this->obj_gcache->save($result, $cacheID, array('gc_routes'));
        }
        return !empty($result['route_' . $key]) ? $result['route_' . $key] : '';
    }
    
    /**
     * List all tables in the database
     * @param string $table_name
     * @return boolean|mixed[]
     */
    public function list_tables($table_name = null) {
        $cacheID = 'gc_tables';
        if(!$result = $this->obj_gcache->load($cacheID)) {
            $result = array();
            $tables = (array) $this->CI->db->list_tables();
            if($tables) {
                foreach($tables as $table) {
                    $table = str_replace($this->dbprefix, '', $table);
                    $result[$table] = $table;
                }
            }
            $this->obj_gcache->save($result, $cacheID, array($cacheID));
        }
        if(!empty($table_name)) {
            if(!empty($result[$table_name])) {
                return true;
            } else {
                return false;
            }
        } else {
            return $result;
        }
    }
    
    /**
     * Get curl data
     * @param string $uri
     * @param array $postfields
     * @param string $url - External URL to be called
     * @param boolean $global_curl - Data is from a global URL, where result is different
     * @return boolean|array
     */
    public function curl_data($uri, $postfields = array(), $url = null, $global_curl = false) {
        $cacheID = 'gc_curl_' . $uri . $url . (!empty($postfields) ? gc_cache($postfields) : '');
        $cacheID = str_replace('-', '_', $this->CI->slug->create_slug($cacheID));
        if(!$result = $this->obj_gcache->load($cacheID)) {
            $this->CI->load->helper('gc');
            if(empty($url)) {
                $url = 'https://data.goclixy.com/rest/' . $uri;
            }
            $result = curl_marketplace(null, $postfields, $url);
            $this->obj_gcache->save($result, $cacheID, array($cacheID));
        }
        if(!$global_curl) {
            if(!empty($result) && $result['result'] == 1) {
                return $result['records'];
            } else {
                return false;
            }
        } else {
            return $result;
        }
    }
    
    /**
     * Get information from a specific table
     * @param array $params
     * @return object
     */
    public function get($table, $params = array()) {
        $cacheID = $table . '_' . gc_cache_id($params);
        $cacheID = str_replace('-', '_', $this->CI->slug->create_slug($cacheID));
        if(!$result = $this->obj_gcache->load($cacheID)) {
            if(!empty($params['select'])) {
                $this->CI->db->select($params['select']);
            }
            if(!empty($params['where'])) {
                $this->CI->db->where($params['where']);
            }
            if(!empty($params['order_by'])) {
                $this->CI->db->order_by($params['order_by']);
            }
            if(!empty($params['result'])) {
                if(!empty($params['limit'])) {
                    $this->CI->db->limit($params['limit']);
                }
                $result = $this->CI->db->get($table)->result();
            } else if(!empty($params['counts'])) {
                $result = $this->CI->db->count_all_results($table);
            } else {
                $result = $this->CI->db->get($table)->row();
            }
            $this->obj_gcache->save($result, $cacheID, array($table));
        }
        return $result;
    }
    
    /**
     * Other site images like wallets, leads etc.
     * Do not hardcode images, get them through database so that their path can be changed
     * @param string $slug
     * @return string
     */
    public function site_image($slug) {
        $table = 'site_images';
        $cacheID = $table . '_' . $slug;
        $cacheID = str_replace('-', '_', $this->CI->slug->create_slug($cacheID));
        if(!$result = $this->obj_gcache->load($cacheID)) {
            $record = $this->CI->db->where('slug', $slug)->get($table)->row();
            if($record) {
                $result = gc_image_url($record->url, false);
            } else {
                $result = gc_site_image('no-image');
            }
            $this->obj_gcache->save($result, $cacheID, array($table));
        }
        return $result;
    }
}