<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');
class Gcategories {
    
    // To store instance of codeigniter
    protected $CI;
    protected $z_cache;
    protected $tree_html;
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        $this->CI =& get_instance();        
        
        // Load Zend Caching
        // if(settings_item('site.enable_cache') == 1) {
        $this->CI->load->library('gzf_cache', array('lifetime' => 900));
        $this->z_cache = $this->CI->gzf_cache->get_instance();
        $this->CI->load->library('slug');
        // }
    }
    
    /**
     * FANCY TREE PLUGIN - Display Categories Tree to the User
     * @return string
     */
    public function load_tree() {
        /**
         * Retrieve listing and category parent id from GET
         * Get categories information from database based on parent id
         * Enable lazy load only if parent has subcategories
         * Join with Listing Categories table for Edit Listing
         */
        $parent_id = 0;
        $package_id = (int) $this->CI->input->get('package_id');
        $row_id = escape($this->CI->input->get('row_id'));
        $module = escape($this->CI->input->get('module'));
        
        if($this->CI->input->get('parent')) {
            $parent_id = $this->CI->input->get('parent');
        }
        
        // POST BACK HANDLING
        $post_categories_array = array();
        $post_categories = $this->CI->session->userdata($module . '_categories');
        if(settings_item('site.enable_cache') == 1) {
            $cacheID = 'categories_tree_' . $parent_id . '_' . $this->auth->user_id() . '_' . $module . '_' . $post_categories;
            $cacheID = str_replace('-', '_', $this->slug->create_slug($cacheID));
            if(!$data = $this->z_cache->load($cacheID)) {
                if($post_categories) {
                    $post_categories_array = str_replace('/', ',', $post_categories);
                    $post_categories_array = explode(',', $post_categories_array);
                }
                $data = $this->get_categories_ajax($parent_id, $module, $post_categories_array);
                $this->z_cache->save($data, $cacheID);
            }
        } else {
            if($post_categories) {
                $post_categories_array = str_replace('/', ',', $post_categories);
                $post_categories_array = explode(',', $post_categories_array);
            }
            $data = $this->get_categories_ajax($parent_id, $module, $post_categories_array);
        }
        echo json_encode($data);
    }
    
    /**
     * Query to Get Categories Tree
     * @param integer $parent_id
     * @param string $module
     * @param integer $package_id
     * @return array
     */
    private function get_categories_for_tree($parent_id, $module) {
        $prefix = $module == 'products' ? 'product_' : '';
        $sql = "SELECT M.id, M.parent_id, M.name, M.status_id, CC.counts";
        $sql .= " FROM ". $this->CI->db->dbprefix . $prefix. "categories M" . PHP_EOL;
        $sql .= " LEFT OUTER JOIN (";
        $sql .= " SELECT CS.parent_id, COUNT(*) AS counts FROM ". $this->CI->db->dbprefix . $prefix . "categories CS" . PHP_EOL;
        $sql .= " GROUP BY CS.parent_id) CC ON M.id = CC.parent_id" . PHP_EOL;
        $sql .= " WHERE M.parent_id = $parent_id AND M.status_id = 2";
        $sql .= " ORDER BY M.name";
        $result = $this->CI->db->query($sql)->result();
        return $result;
    }
    
    /**
     * FANCY TREE - Display Categories Tree to the User
     * @param number $parent_id
     * @param string $module
     * @param array $post_categories
     * @return multitype:multitype:boolean NULL
     */
    public function get_categories_ajax($parent_id, $module, $post_categories) {
        $result = $this->get_categories_for_tree($parent_id, $module);
        $data = array();
        if($result) {
            foreach($result as $category) {
                if(!empty($post_categories) && in_array($category->id, $post_categories)) {
                    // Category has no children, this is the root category
                    if($category->counts == 0) {
                        $data[] = array(
                            'title' => $category->name,
                            'folder' => TRUE,
                            'key' => $category->id,
                            'selected' => TRUE
                        );
                    } else { // Category has children, the child category will be selected
                        $data[] = array(
                            'title' => $category->name,
                            'folder' => TRUE,
                            'key' => $category->id,
                            'children' => $this->get_categories_ajax($category->id, $module, $post_categories)
                        );
                    }
                } else {
                    if($category->counts == 0) {
                        $data[] = array(
                            'title' => $category->name,
                            'folder' => TRUE,
                            'key' => $category->id
                        );
                    } else {
                        $data[] = array(
                            'title' => $category->name,
                            'folder' => TRUE,
                            'key' => $category->id,
                            'lazy' => TRUE
                        );
                    }
                }
            }
        }
        return $data;
    }
    
    /**
     * FancyTree Categories Post Handling
     * @param string $module
     * @param number $id
     */
    public function fancytree_categories($module, $id = 0) {
        if($id == 0) { // Create Page
            if(empty($_POST['categories'])) {
                $this->CI->session->unset_userdata($module . '_categories');
            } else {
                $this->CI->session->set_userdata($module . '_categories', $_POST['categories']);
            }
        } else { // Edit Page
            $fancytree_format = $this->CI->db->select('categories')->where('id', $id)->get($module)->row();
            if($fancytree_format) {
                $this->CI->session->set_userdata($module . '_categories', $fancytree_format->categories);
            }
        }
    }
    
    /**
     * Get Unique Categories
     * @param string $categories e.g. '/4,/2,/2/1';
     * @return string
     */
    public function save_categories($id, $module) {
        $category_ids = array();
        $categories = $this->CI->input->post('categories');
        $array_values = explode(',', $categories);
        if($array_values) {
            foreach($array_values as $array_value) {
                $category_ids_array = explode('/', $array_value);
                foreach($category_ids_array as $category_id) {
                    if(!empty($category_id)) {
                        $category_ids[$category_id] = $category_id;
                    }
                }
            }
        }
        $this->CI->db->where('id', $id)->update($module, array('categories' => $categories, 'categories_set' => implode(',', $category_ids)));
    }
    
    /**
     * Update Category Counts
     * @param string $module - name of the module
     */
    public function update_counts($module) {
        $table = $module == 'blogs' ? 'blog_categories' : 'product_categories';
        $records = $this->CI->db->get($table)->result();
        if($records) {
            $data = array();
            foreach($records as $record) {
                $sql = "SELECT COUNT(*) AS total_counts FROM " . $this->CI->db->dbprefix . $module . " WHERE FIND_IN_SET(" . $record->id . ", category_ids)";
                $result = $this->CI->db->query($sql)->row();
                if($result->total_counts > 0) {
                    $data[] = array(
                        'id' => $record->id,
                        'counts' => $result->total_counts
                    );
                }
            }            
            if(!empty($data)) {
                $this->CI->db->update_batch($table, $data, 'id');
            }
        }
    }
    
    /**
     * Display All Categories
     */
    public function get($count = 0, $parent_id = 0, $module = 'blogs', $where = null, $limit = 150, $offset = 0) {
        $extra = settings_item('lst.hide_counts_zero_categories') == 1 ? 'M.counts != 0' : null;
        if(settings_item('site.enable_cache') == 1) {
            $cacheID = 'categories_' . $module . '_' . $count . '_' . $parent_id . '_' . $module . '_' . implode('_', (array) $where) . '_' . $limit . '_' . $offset . '_' . $extra;
            $cacheID = str_replace('-', '_', $this->CI->slug->create_slug($cacheID));
            if(!$categories = $this->z_cache->load($cacheID)) {
                $categories = array('categories' => $this->_categories($count, $parent_id, $module, $where, $limit, $offset, $extra));
                $this->z_cache->save($categories, $cacheID, array($module));
            }
        } else {
            $categories = array('categories' => $this->_categories($count, $parent_id, $module, $where, $limit, $offset, $extra));
        }
        return $categories['categories'];
    }
    
    /**
     * Display All Categories
     * @param number $count
     * @param number $parent_id
     * @param string $module
     * @param array $where
     * @param number $limit
     * @param number $offset
     * @return object|number
     */
    private function _categories($count = 0, $parent_id = 0, $module = 'blogs', $where = null, $limit = 150, $offset = 0, $extra = null) {
        $table = $module == 'blogs' ? 'blog_categories' : 'product_categories';
        if($count == 0) {
            $table_fields = array('id', 'parent_id', 'name', 'description', 'icon', 'slug', 'counts', 'image');
            $query = $this->CI->translations->build_query($table, $table_fields);
            $sql = $query['select'] . ", SUBSTRING_INDEX(GROUP_CONCAT(S.name SEPARATOR ', '), ',', 3) AS description, COUNT(S.id) AS has_children";
        } else {
            $sql = "SELECT COUNT(*) AS total_count";
        }
        $sql .= " FROM " . $this->CI->db->dbprefix . $table . " M";
        if($count == 0) {
            $sql .= $this->CI->translations->common_joins($table) . " S ON S.parent_id = M.id AND S.status_id = 2";
        }         
        if($count == 0 && !empty($query['join'])) {
            $sql .= $query['join'];
        }
        $sql .= " WHERE M.parent_id = $parent_id AND M.status_id = 2";
        if(!empty($where)) {
            $sql .= " AND M.name LIKE '%" . $where . "%'";
        }
        if(!empty($extra)) {
            $sql .= " AND " . $extra;
        }
        if($count == 0) {
            $sql .= " GROUP BY M.id ORDER BY M.priority, name";
            if ($limit != 0) {
                $sql .= " LIMIT " . $offset . ", " . $limit;
            } else {
                $sql .= " LIMIT $limit";
            }
            return $this->CI->db->query ( $sql )->result();
        } else {
            $result = $this->CI->db->query ( $sql )->row();
            return $result ? $result->total_count : 0;
        }
    }
    
    /**
     * Get Categories for Toolbar Title
     * @param number $id
     * @param string $module
     */
    public function get_category($id, $module = 'blogs', $select = null) {
        $table = $module == 'blogs' ? 'blog_categories' : 'product_categories';
        if(!empty($select)) { // marketplace rest api
            $this->CI->db->select($select);
        }
        $record = $this->CI->db->where('id', $id)->get($table)->row();
        return $record;
    }
    
    /**
     * To display all categories to the admin
     * @return multitype:multitype:unknown
     */
    public function get_order($module, $parent_id = 0) {
        $table = $module == 'blogs' ? 'blog_categories' : 'product_categories';
        $where = array();
        if($parent_id != 0) {
            $where['parent_id'] = $parent_id;
        }
        $this->CI->db->from($table);
        $result = $this->CI->db->select('id, parent_id, name')->where($where)->group_by('id')->order_by('priority')->get()->result_array();
        return $this->prepareList($result);
    }
    
    /**
     * Prepare list to get parents and its children
     * @param array $items
     * @param number $pid
     * @return multitype:multitype:unknown
     */
    private function prepareList(array $items, $pid = 0) {
        $output = array();
        
        if(!empty($items)) {
            # loop through the items
            foreach ($items as $item) {
                # Whether the parent_id of the item matches the current $pid
                if((int) $item['parent_id'] == $pid) {
                    
                    # Call the function recursively, use the item's id as the parent's id
                    # The function returns the list of children or an empty array()
                    if ($children = $this->prepareList($items, $item['id'])) {
                        # Store all children of the current item
                        $item['children'] = $children;
                    }
                    
                    # Fill the output
                    $output[] = $item;
                }
            }
        }
        return $output;
    }
    
    /**
     * Save order only when user clicks on save button
     * @param array $links
     */
    public function save_order($module, $records) {
        $table = $module == 'blogs' ? 'blog_categories' : 'product_categories';
        $relations_table = $module == 'blogs' ? 'blog_category_relations' : 'product_category_relations';
        $this->CI->load->library('goclixy/gclosure');
        $gclosure = new Gclosure($table, $relations_table);
        if (count($records)) {
            foreach ($records as $order => $record) {
                $where = array('id' => (!empty($record['id']) ? $record['id'] : 0));
                if (!empty($record['id'])) {
                    $parent_id = (int) $record['parent_id'];
                    $data = array('priority' => $order, 'parent_id' => $parent_id);
                    $this->CI->db->where($where)->update($table, $data);
                    /**
                     * Save Category Relations in the category_relations table for the multilevel menus
                     * @var array $record
                     */
                    $id = $record['id'];
                    $record = $this->CI->db->where('id', $id)->get($table)->row();
                    if($record) {
                        $parent_id = $record->parent_id;
                        $gclosure->delete($id, false); // do not delete from the categories table
                        // save hierarchy
                        $gclosure->add($id, ($parent_id == 0 ? $id : $parent_id));
                    }
                }
            }
        }
    }
    
    /**
     * Category Links Tree for Marketplace and Other Blocks
     * @param array $records
     * @param array $extra
     * @return string
     */
    public function category_links_tree($records, $extra = array('tpl' => 1)) {
        if(!empty($records)) {
            $this->tree_html = '';
            $tpl = isset($extra['tpl']) ? (int) $extra['tpl'] : 0;
            switch($tpl) {
                case 3:
                    $this->tree_html .= '<div class="navbar-nav">';
                    $this->print_tree($records, false, null, $tpl, $extra);
                    $this->tree_html .= '</div>';
                    break;
                case 2:
                    $this->tree_html .= '<div id="dropdownMenu' . $tpl . $parent_id . '">';
                    $this->tree_html .= '<div class="list-group panel">';
                    $this->print_tree($records, false, null, $tpl, $extra);
                    $this->tree_html .= '</div>';
                    $this->tree_html .= '</div>';
                    break;
                case 1:
                    if(isset($extra['with_heading'])) {
                        $this->tree_html .= '<h5>' . $result['heading'] . '</h5>';
                    }
                    $this->tree_html .= '<ul' . (isset($extra['parent_class']) ? ' class="' . $extra['parent_class'] . '"' : '') . '>';
                    $this->print_tree($records, false, null, $tpl, $extra);
                    $this->tree_html .= '</ul>';
                    break;
                default:
                    $this->tree_html .= '<ul class="navbar-nav mr-auto">';
                    $this->print_tree($records, false, null, $tpl, $extra);
                    $this->tree_html .= '</ul>';
            }
        }
        return $this->tree_html;
    }
    
    /**
     * Category has children hierarchy
     * The first element of the array is not a multidimensional array
     * @param array $tree
     */
    public function print_tree($records, $has_children = false, $ul_extra = '', $tpl = 1, $extra = array()) {
        if($has_children) {
            switch($tpl) {
                case 1:
                    $this->tree_html .= PHP_EOL . '<ul' . $ul_extra . '>';
                    break;
                case 2:
                    $this->tree_html .= '<div class="collapse"' . $ul_extra . '>';
                    break;
                case 3:
                    break;
                default:
                    $this->tree_html .= PHP_EOL . '<ul ' . $ul_extra . 'class="dropdown-menu border-0 shadow">';
            }
        }
        foreach($records as $record) {
            $this->tree_html($record, $has_children, $tpl, $extra);
        }
        if($has_children) {
            switch($tpl) {
                case 2:
                    $this->tree_html .= '</div>';
                    break;
                case 3:
                    break;
                default:
                    $this->tree_html .= '</ul>';
            }
        }
    }
    
    /**
     * Display Tree View
     * @param array $affiliate
     */
    private function tree_html($record, $has_children = false, $tpl = 0, $extra = array()) {
        $this->tree_html .= PHP_EOL;
        $icon = !empty($extra['with_icon']) ? '<i class="' . (!empty($record['icon']) ? $record['icon'] : 'fas fa-check-circle') . ' fa-fw mr-1"></i>' : '';
        $target = !empty($record['target']) && $record['target'] == '_blank' ? ' target="' . $record['target'] . '"' : '';
        $label = $record['name'];
        $redirect_url = gc_category_url('listings', $record['slug'], $record['id']);
        $parent_id = $record['parent_id'];
        switch($tpl) {
            case 1: // Member Area - Accordion
                $link_badge = !empty($record['badge']) ? '<span class="nva01__label">' . $record['badge'] . '</span>' : '';
                $this->tree_html .= '<li class="active-' . $record['id'] . '">';
                if(!empty($record['children'])) {
                    $this->tree_html .= '<button class="nva-link nva-link-' . $record['id'] . ' btn has-arrow text-truncate" aria-expanded="false" type="button" name="category_filter" value="' . $record['id'] . '">' . $icon . $label . '</button>' . $link_badge;
                    // $this->tree_html .= '<a class="nva-link has-arrow" href="javascript:void(0);" title="'. $label .'"' . $target . '>' . $icon . $label . '</a>' . $link_badge;
                    $ul_class = ' class="submenu"';
                    $this->print_tree($record['children'], true, $ul_class, $tpl, $extra);
                } else {
                    $this->tree_html .= '<button class="nva-link nva-link-' . $record['id'] . ' btn text-truncate" aria-expanded="false" type="submit" name="category_filter" value="' . $record['id'] . '">' . $icon . $label . '</button>' . $link_badge;
                    // $this->tree_html .= '<a class="nva-link" href="' . $redirect_url . '" title="'. $label .'"' . $target . '>' . $icon . $label . '</a>' . $link_badge;
                }
                $this->tree_html .= '</li>';
                break;
            case 2: // Vertical Accordion Menu
                $active = (!empty($record['active']) ? ' active' : '');
                $badge = !empty($record['badge']) ? '<span class="badge badge-danger badge-pill float-right">' . $record['badge'] . '</span>' : '';
                $link_class = isset($extra['link_class']) ? $extra['link_class'] : 'list-group-item-success';
                if(empty($record['children'])) {
                    if($record['parent_id'] == 0) {
                        $this->tree_html .= '<a href="' . $redirect_url . '" class="list-group-item ' . $link_class . $active . '" data-parent="#dropdownMenu' . $tpl . $parent_id . '">' . $icon . $label . $badge . '</a>' . PHP_EOL;
                    } else {
                        $this->tree_html .= '<a href="' . $redirect_url . '" class="list-group-item ' . $link_class . $active . '" data-parent="#dropdownMenu' . $tpl . $parent_id . $record['parent_id'] . '">' . $icon . $label . $badge . '</a>' . PHP_EOL;
                    }
                } else {
                    if($record['parent_id'] == 0) {
                        $this->tree_html .= '<a href="#dropdownMenu' . $tpl . $parent_id . $record['id'] . '" class="list-group-item ' . $link_class . $active . '" data-toggle="collapse" data-parent="#dropdownMenu' . $tpl . $parent_id . '">' . $icon . $label . $badge . ' <i class="fas fa-caret-down"></i></a>' . PHP_EOL;
                    } else {
                        $this->tree_html .= '<a href="#dropdownMenu' . $tpl . $parent_id . $record['id'] . '" class="list-group-item" data-toggle="collapse" data-parent="#dropdownMenu' . $tpl . $parent_id . $record['parent_id'] . '">' . $icon . $label . $badge . ' <i class="fas fa-caret-down"></i></a>' . PHP_EOL;
                    }
                    $ul_class = ' class="collapse list-group-submenu" id="dropdownMenu' . $tpl . $parent_id . $record['id'] . '"';
                    $this->print_tree($record['children'], true, $ul_class, $tpl, $extra);
                }
                break;
            case 3:
                $active = (!empty($record['active']) ? ' active' : '');
                $this->tree_html .= '<a class="nav-item nav-link btn-info btn-tertiary' . $active . '" href="' . $redirect_url . '" title="'. $label .'"' . $target . '>' . $icon . $label . '</a>';
                break;
            default: // Navbar Dropdown Menu
                if(!empty($record['children'])) {
                    if($record['parent_id'] == 0) {
                        $this->tree_html .= '<li class="nav-item dropdown">' . PHP_EOL;
                        $this->tree_html .= '<a id="dropdownMenu' . $tpl . $parent_id . $record['id'] . '" href="#" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false" class="nav-link dropdown-toggle" title="'. $label .'"' . $target . '>' . $icon . $label . '</a>' . PHP_EOL;
                        $ul_id = '';
                    } else {
                        $this->tree_html .= '<li class="dropdown-submenu">' . PHP_EOL;
                        $this->tree_html .= '<a id="dropdownMenu' . $tpl . $parent_id . $record['id'] . '" href="#" role="button" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false" class="dropdown-item dropdown-toggle" title="'. $label .'"' . $target . '>' . $icon . $label . '</a>' . PHP_EOL;
                        $ul_id = 'aria-labelledby="dropdownMenu' . $tpl . $parent_id . $record['id'] . '" ';
                    }
                    $this->print_tree($record['children'], true, $ul_id, $tpl, $extra);
                } else {
                    if($record['parent_id'] == 0) {
                        $this->tree_html .= '<li class="nav-item">' . PHP_EOL;
                        $this->tree_html .= '<a href="' . $redirect_url . '" class="nav-link" title="'. $label .'"' . $target . '>' . $icon . $label . '</a>' . PHP_EOL;
                    } else {
                        $this->tree_html .= '<li>' . PHP_EOL;
                        $this->tree_html .= '<a href="' . $redirect_url . '" class="dropdown-item" title="'. $label .'"' . $target . '>' . $icon . $label . '</a>' . PHP_EOL;
                    }
                }
                $this->tree_html .= '</li>';
        }
    }
    
    /**
     * Layout - Get categories based on the where parameters
     * @param array $where
     */
    public function get_categories($where, $table = 'categories', $select = 'id, name, slug, counts') {
        $cacheID = 'categories_' . $table . '_' . str_replace('-', '_', $this->CI->slug->create_slug($where));
        // if(!$records = $this->z_cache->load($cacheID)) {
        $records = $this->CI->db->select($select)->where($where)->get($table)->result();
        $this->z_cache->save($records, $cacheID);
        //}
        return $records;
    }
}