<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');
/**
 *
 * CI / MySQL Closure Table Model
 *
 * @link  http://www.slideshare.net/billkarwin/models-for-hierarchical-data
 * @TODO  improve
 *
 * sql schema:
 * CREATE TABLE `closures` (
 *   `id` int(11) NOT NULL AUTO_INCREMENT,
 *   `ancestor` int(11) NOT NULL,
 *   `descendant` int(11) NOT NULL,
 *   `depth` int(11) NOT NULL,
 *   PRIMARY KEY (`id`)
 * );
 * 
$this->load->library('goclixy/gclosure');
$gclosure = new Gclosure('categories', 'categories_hierarchy');
 */
class Gclosure {
    
    // To store instance of codeigniter
    public $CI;
    
    // root table
    public $table;
    
    // default closures table
    public $closure_table = 'closures';
    
    // other global variables
    protected $gclr_cache;
    protected $search_tree;
    protected $tree_key_ids;
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct($table_name = NULL, $closure_table = NULL) {
        $this->CI =& get_instance();
        $this->table = $table_name;
        if ($closure_table !== NULL) {
            $this->closure_table = $this->CI->db->dbprefix . $closure_table;
        }
        $this->CI->load->library('gzf_cache', array('lifetime' => 900));
        $this->gclr_cache = $this->CI->gzf_cache->get_instance();
        $this->CI->load->library('slug');
    }
    
    /**
     * Fetch children.
     *
     * Example to generate nested tree:
     *
     *   $data = $this->get_children(1, TRUE, FALSE, TRUE);
     *   print_r($data);
     *
     * If level/depth specified then self will be ignore.
     *
     * @param  int      node id
     * @param  boolean  include self
     * @param  mixed    node level/depth (e.g direct children = 1)
     * @param  boolean  nestify the result
     * @return mixed    array if query true
     */
    public function get_children($node_id = 1, $self = FALSE, $level = FALSE, $nested = FALSE, $extra = array()) {        
        $select = isset($extra['select']) ? $extra['select'] : 't.*';
        $select .= ', c2.ancestor as parent, c1.depth as level';
        $this->CI->db->select($select);
        $this->CI->db->from($this->closure_table." c1");
        $this->CI->db->join($this->table.' t','t.id = c1.descendant');
        if(isset($extra['joins'])) {
            $joins = $extra['joins'];
            foreach($joins as $join) {
                // $extra['join'] = array('table' => 'manual_products', 'column' => 't.product_id', 'prefix' => 'p');
                $this->CI->db->join($join['table'] . ' ' . $join['prefix'], $join['prefix'] . '.id = ' . $join['column']);
            }
        }
        $this->CI->db->join($this->closure_table.' c2', 'c2.depth IN(1) AND c2.descendant = c1.descendant','LEFT'); // ugh backticking INTs in joins @TODO
        $this->CI->db->where('c1.ancestor',$node_id);
        if(isset($extra['where'])) {
            $this->CI->db->where($extra['where']);
        }
        if ( ! $self) {
            $this->CI->db->where('c1.descendant <>', $node_id);
        }
        if ($level) {
            $this->CI->db->where('c1.depth = ', $level);
        }
        if(!empty($extra['sort_by'])) {
            $this->CI->db->order_by($extra['sort_by']);
        }
        $query = $this->CI->db->get();
        if ( ! $query->num_rows()) {
            return FALSE;
        }
        $result = $query->result_array();
        if ($nested AND ! $level) {
            $trees = array();
            $root = null;
            foreach ($result as $row) {
                if($this->table == 'links') {
                    $row = $this->CI->links_model->process_link($row);
                }
                $trees[$row['id']] = $row;
            }
            foreach ($trees as $key => $row) {
                if( ! $root) {
                    $root = $self ? $node_id : $row['parent']; // modified on July 13, 2021
                }
                $trees[$row['parent']]['children'][$key] =& $trees[$key];
            }
            $result = $trees[$root];
            if ( ! $self) {
                return $result['children'];
            }
            return isset($result['id']) ? $result : array_shift($result['children']);
        }
        return $result;
        
    }
    
    /**
     * Add a node (as last child).
     *
     * @param  int    node id
     * @param  int    target id
     * @return boolean
     */
    public function add($node_id, $target_id = 0) {
        /*$sql = 'SELECT ancestor, ' . $node_id . ', depth+1
				FROM ' . $this->closure_table . '
				WHERE descendant = '.$target_id.'
				UNION
				SELECT ' . $node_id . ',' . $node_id . ',0';
        $query = 'INSERT INTO ' . $this->closure_table . ' (ancestor, descendant,depth) (' . $sql . ')';*/
        $sql = 'SELECT ancestor, ' . $node_id . ', depth+1
				FROM '. $this->closure_table.'
				WHERE descendant = ' . (empty($target_id) ? 0 : $target_id) . '
				UNION
				SELECT ' . $node_id . ',' . $node_id . ',0';
        $query = 'INSERT INTO ' . $this->closure_table . ' (ancestor, descendant, depth) ' . $sql;
        // $query = 'INSERT INTO ' . $this->closure_table . ' (ancestor, descendant, depth) (' . $sql . ')';
        $result = $this->CI->db->query($query);
        return $result;
    }
    
    /**
     * Check if current node has children.
     *
     * @param   int       node id
     * @return  boolean
     */
    public function has_children($node_id) {
        $this->CI->db->select('descendant')
        ->from($this->closure_table)
        ->where('ancestor', $node_id);
        $descendants = $this->CI->db->get()->result_array();
        foreach ($descendants as $k => $v) {
            $descendants[$k] = $v['descendant'];
        }
        
        $query = $this->CI->db->select('COUNT(*) as total')
        ->from($this->closure_table)
        ->where('ancestor IN (' . implode(',', $descendants) . ')')
        ->where('descendant <>',$node_id)
        ->get();
        return (bool) $query->row()->total;
        
    }
    
    /**
     * Get parent(s) of current node.
     *
     * @param  int    current node id
     * @param  mixed  level up (e.g direct parent = 1)
     * @param string $sort_by
     * @return mixed  array if succeed
     */
    public function get_parent($node_id, $level = NULL, $sort_by = 'ASC', $self = false) {        
        $this->CI->db->select('t.*, c.depth')
        ->from($this->table.' t')
        ->join($this->closure_table.' c','t.id = c.ancestor')
        ->where('c.descendant',$node_id);
        if(!$self) {
            $this->CI->db->where('c.ancestor <>',$node_id);
        }
        if ($level) {
            $this->CI->db->where('c.depth', $level);
        }
        // $this->CI->db->order_by('t.id', $sort_by);
        $this->CI->db->order_by('c.depth', $sort_by);
        $query = $this->CI->db->get();
        if ($query->num_rows()) {
            if ($level) {
                return $query->row();
            }
            $result = $query->result();
            return $result;
        }
        return FALSE;
    }
    
    /**
     * TODO: optional recursion
     *
     * Delete node.
     *
     * @param  int      node id
     * @param  boolean  if TRUE, it will also delete from reference table
     * @return mixed
     */
    public function delete($node_id, $delete_reference = TRUE) {        
        $operand = 'select descendant as id from '.$this->closure_table.' where ancestor = '.$node_id;
        $query = 'select id, descendant from '.$this->closure_table.' where descendant IN ('.$operand.')';
        $result_start = $this->CI->db->query($query);
        if ( $result_start->num_rows() > 0 ) {
            $descendants = array_column($result_start->result(), 'id');
            if(!empty($descendants)) {
                $result_delete = $this->CI->db->where('id IN (' . implode(',', $descendants) . ')')->delete($this->closure_table);
                if ($delete_reference) {
                    $descendants = array_column($result_start->result(), 'descendant');
                    $delete_refs = $this->CI->db->where('id IN (' . implode(',', $descendants) . ')')->delete($this->table);
                }
                return $result_delete;
            }
        }
        return FALSE;
    }
    
    /**
     * Move node with its children to another node.
     *
     * @link  http://www.mysqlperformanceblog.com/2011/02/14/moving-subtrees-in-closure-table/
     *
     * @param  int  node to be moved
     * @param  int  target node
     * @return void
     */
    public function move($node_id, $target_id) {
        // MySQL�s multi-table DELETE
        $query1 = 'DELETE a FROM '.$this->closure_table.' AS a ';
        $query1 .= 'JOIN '.$this->closure_table.' AS d ON a.descendant = d.descendant ';
        $query1 .= 'LEFT JOIN '.$this->closure_table.' AS x ';
        $query1 .= 'ON x.ancestor = d.ancestor AND x.descendant = a.ancestor ';
        $query1 .= 'WHERE d.ancestor = '.$node_id.'  AND x.ancestor IS NULL';
        $res1 = $this->CI->db->query($query1);
        $query2 = 'INSERT INTO '.$this->closure_table.' (ancestor, descendant, depth) ';
        $query2 .= 'SELECT a.ancestor, b.descendant, a.depth+b.depth+1 ';
        $query2 .= 'FROM '.$this->closure_table.' AS a JOIN '.$this->closure_table.' AS b ';
        $query2 .= 'WHERE b.ancestor = '.$node_id.' AND a.descendant = '.$target_id;
        $res2 = $this->CI->db->query($query2);
        return $res1 AND $res2;
    }
    
    /**
     * Get (all) root nodes.
     */
    public function get_root() {
        $this->CI->db->select('r.descendant');
        $this->CI->db->from($this->closure_table." r");
        $this->CI->db->join($this->closure_table." p","r.descendant = p.descendant AND p.ancestor <> p.descendant","LEFT");
        $this->CI->db->where('p.descendant IS NULL',NULL);
        $result = $this->CI->db->get();
        if ($result) {
            return array_column($result->result(),'descendant');
        } else {
            return false;
        }
    }
    
    /**
     * Print tree - View Page
     * @param array $where
     * @return string|boolean
     */
    public function tree($where = array(), $extra = array(), $self = true, $level = false, $nested = true, $reference_field = 'parent_id') {
        if(!empty($where)) {
            $this->CI->db->where($where);
        }
        $parents = $this->CI->db->select('id')->order_by('priority', 'ASC')->get($this->table)->result();
        $data = array();
        if($parents) {
            foreach($parents as $parent) {
                $result = $this->single_node($parent->id, $extra, $self, $level, $nested);
                if(!empty($result)) {
                    $data[] = $result;
                }
            }
        }
        return $data;
    }
    
    /**
     * Single node tree
     * @param int $comment_id
     * @return string
     */
    public function single_node($id, $extra = array(), $self = true, $level = true, $nested = true) {
        $tree = $this->get_children($id, $self, $level, $nested, $extra);
        return $tree;
    }
    
    /**
     * Print Tree
     * @param array $records
     * @param array $extra
     * @return string
     * 
     * Usage:
     * 
     * $this->load->library('gclosure');
     * $gclosure = new Gclosure('product_categories', 'product_category_relations');
     * $records = $gclosure->tree('parent_id = 0', array(), true, false, true);
     * echo $gclosure->hierarchy_tree($records, array('tpl' => 2));
     */
    public function hierarchy_tree($records, $extra = array('tpl' => 1)) {
        if(!empty($records)) {
            $this->tree_html = '';
            $tpl = isset($extra['tpl']) ? (int) $extra['tpl'] : 0;
            switch($tpl) {
                case 3:
                    $this->tree_html .= '<div class="navbar-nav">';
                    $this->print_tree($records, false, null, $tpl, $extra);
                    $this->tree_html .= '</div>';
                    break;
                case 2:
                    $this->tree_html .= '<div id="dropdownMenu' . $tpl . '0">';
                    $this->tree_html .= '<div class="list-group panel">';
                    $this->print_tree($records, false, null, $tpl, $extra);
                    $this->tree_html .= '</div>';
                    $this->tree_html .= '</div>';
                    break;
                case 1:
                    if(isset($extra['with_heading'])) {
                        $this->tree_html .= '<h5>' . $result['heading'] . '</h5>';
                    }
                    $this->tree_html .= '<ul' . (isset($extra['parent_class']) ? ' class="' . $extra['parent_class'] . '"' : '') . '>';
                    $this->print_tree($records, false, null, $tpl, $extra);
                    $this->tree_html .= '</ul>';
                    break;
                default:
                    $this->tree_html .= '<ul class="navbar-nav mr-auto">';
                    $this->print_tree($records, false, null, $tpl, $extra);
                    $this->tree_html .= '</ul>';
            }
        }
        return $this->tree_html;
    }
    
    /**
     * Category has children hierarchy
     * The first element of the array is not a multidimensional array
     * @param array $tree
     */
    public function print_tree($records, $has_children = false, $ul_extra = '', $tpl = 1, $extra = array()) {
        if($has_children) {
            switch($tpl) {
                case 1:
                    $this->tree_html .= PHP_EOL . '<ul' . $ul_extra . '>';
                    break;
                case 2:
                    $this->tree_html .= '<div class="collapse"' . $ul_extra . '>';
                    break;
                case 3:
                    break;
                default:
                    $this->tree_html .= PHP_EOL . '<ul ' . $ul_extra . 'class="dropdown-menu border-0 shadow">';
            }
        }
        foreach($records as $record) {
            $this->tree_html($record, $has_children, $tpl, $extra);
        }
        if($has_children) {
            switch($tpl) {
                case 2:
                    $this->tree_html .= '</div>';
                    break;
                case 3:
                    break;
                default:
                    $this->tree_html .= '</ul>';
            }
        }
    }
    
    /**
     * Display Tree View
     * @param array $affiliate
     */
    private function tree_html($record, $has_children = false, $tpl = 0, $extra = array()) {
        $this->tree_html .= PHP_EOL;
        $icon = !empty($extra['with_icon']) ? '<i class="' . (!empty($record['icon']) ? $record['icon'] : 'fas fa-check-circle') . ' fa-fw mr-1"></i>' : '';
        $target = !empty($record['target']) && $record['target'] == '_blank' ? ' target="' . $record['target'] . '"' : '';
        $label = $record['name'];
        $redirect_url = gc_category_url('listings', $record['slug'], $record['id']);
        $parent_id = $record['parent_id'];
        switch($tpl) {
            case 1: // Member Area - Accordion
                $link_badge = !empty($record['badge']) ? '<span class="nva01__label">' . $record['badge'] . '</span>' : '';
                $this->tree_html .= '<li class="active-' . $record['id'] . '">';
                if(!empty($record['children'])) {
                    $this->tree_html .= '<button class="nva-link nva-link-' . $record['id'] . ' btn has-arrow text-truncate" aria-expanded="false" type="button" name="category_filter" value="' . $record['id'] . '">' . $icon . $label . '</button>' . $link_badge;
                    // $this->tree_html .= '<a class="nva-link has-arrow" href="javascript:void(0);" title="'. $label .'"' . $target . '>' . $icon . $label . '</a>' . $link_badge;
                    $ul_class = ' class="submenu"';
                    $this->print_tree($record['children'], true, $ul_class, $tpl, $extra);
                } else {
                    $this->tree_html .= '<button class="nva-link nva-link-' . $record['id'] . ' btn text-truncate" aria-expanded="false" type="submit" name="category_filter" value="' . $record['id'] . '">' . $icon . $label . '</button>' . $link_badge;
                    // $this->tree_html .= '<a class="nva-link" href="' . $redirect_url . '" title="'. $label .'"' . $target . '>' . $icon . $label . '</a>' . $link_badge;
                }
                $this->tree_html .= '</li>';
                break;
            case 2: // Vertical Accordion Menu
                $active = (!empty($record['active']) ? ' active' : '');
                $badge = !empty($record['badge']) ? '<span class="badge badge-danger badge-pill float-right">' . $record['badge'] . '</span>' : '';
                $link_class = isset($extra['link_class']) ? $extra['link_class'] : 'list-group-item-success';
                if(empty($record['children'])) {
                    if($record['parent_id'] == 0) {
                        $this->tree_html .= '<a href="' . $redirect_url . '" class="list-group-item ' . $link_class . $active . '" data-parent="#dropdownMenu' . $tpl . $parent_id . '">' . $icon . $label . $badge . '</a>' . PHP_EOL;
                    } else {
                        $this->tree_html .= '<a href="' . $redirect_url . '" class="list-group-item ' . $link_class . $active . '" data-parent="#dropdownMenu' . $tpl . 0 . $record['parent_id'] . '">' . $icon . $label . $badge . '</a>' . PHP_EOL;
                    }
                } else {
                    if($record['parent_id'] == 0) {
                        $this->tree_html .= '<a href="#dropdownMenu' . $tpl . $parent_id . $record['id'] . '" class="list-group-item ' . $link_class . $active . '" data-toggle="collapse" data-parent="#dropdownMenu' . $tpl . $parent_id . '">' . $icon . $label . $badge . ' <i class="fas fa-caret-down"></i></a>' . PHP_EOL;
                    } else {
                        $this->tree_html .= '<a href="#dropdownMenu' . $tpl . $parent_id . $record['id'] . '" class="list-group-item" data-toggle="collapse" data-parent="#dropdownMenu' . $tpl . 0 . $record['parent_id'] . '">' . $icon . $label . $badge . ' <i class="fas fa-caret-down"></i></a>' . PHP_EOL;
                    }
                    $ul_class = ' class="collapse list-group-submenu" id="dropdownMenu' . $tpl . $parent_id . $record['id'] . '"';
                    $this->print_tree($record['children'], true, $ul_class, $tpl, $extra);
                }
                break;
            case 3:
                $active = (!empty($record['active']) ? ' active' : '');
                $this->tree_html .= '<a class="nav-item nav-link btn-info btn-tertiary' . $active . '" href="' . $redirect_url . '" title="'. $label .'"' . $target . '>' . $icon . $label . '</a>';
                break;
            default: // Navbar Dropdown Menu
                if(!empty($record['children'])) {
                    if($record['parent_id'] == 0) {
                        $this->tree_html .= '<li class="nav-item dropdown">' . PHP_EOL;
                        $this->tree_html .= '<a id="dropdownMenu' . $tpl . $parent_id . $record['id'] . '" href="#" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false" class="nav-link dropdown-toggle" title="'. $label .'"' . $target . '>' . $icon . $label . '</a>' . PHP_EOL;
                        $ul_id = '';
                    } else {
                        $this->tree_html .= '<li class="dropdown-submenu">' . PHP_EOL;
                        $this->tree_html .= '<a id="dropdownMenu' . $tpl . $parent_id . $record['id'] . '" href="#" role="button" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false" class="dropdown-item dropdown-toggle" title="'. $label .'"' . $target . '>' . $icon . $label . '</a>' . PHP_EOL;
                        $ul_id = 'aria-labelledby="dropdownMenu' . $tpl . $parent_id . $record['id'] . '" ';
                    }
                    $this->print_tree($record['children'], true, $ul_id, $tpl, $extra);
                } else {
                    if($record['parent_id'] == 0) {
                        $this->tree_html .= '<li class="nav-item">' . PHP_EOL;
                        $this->tree_html .= '<a href="' . $redirect_url . '" class="nav-link" title="'. $label .'"' . $target . '>' . $icon . $label . '</a>' . PHP_EOL;
                    } else {
                        $this->tree_html .= '<li>' . PHP_EOL;
                        $this->tree_html .= '<a href="' . $redirect_url . '" class="dropdown-item" title="'. $label .'"' . $target . '>' . $icon . $label . '</a>' . PHP_EOL;
                    }
                }
                $this->tree_html .= '</li>';
        }
    }
    
    /**
     * Categories modules form header search
     */
    public function prepare_search_tree() {  
        $cacheID = 'categories_modules_search';
        if(!$result = $this->gclr_cache->load($cacheID)) {
            $categories_tree = array();
            $parents = $this->CI->db->select('id')->where('active = 1 AND deleted = 0')->order_by('name', 'ASC')->get($this->table)->result();
            if($parents) {
                foreach($parents as $parent) {
                    $result = $this->single_node($parent->id, array(), true, false, true);
                    if(!empty($result)) {
                        $categories_tree[] = $result;
                    }
                }
            }
            $this->child_last_level($categories_tree);
            $result = array('tree' => $this->search_tree, 'tree_key_ids' => $this->tree_key_ids);
            $this->gclr_cache->save($result, $cacheID, array('categories'));
        }
        return $result;
    }
    
    /**
     * Categories modules form header search
     * @param array $categories_tree
     * @param object $gclosure
     */    
    private function child_last_level($categories_tree) {
        if(!empty($categories_tree)) {
            foreach($categories_tree as $category_tree) {
                if(!empty($category_tree['children'])) {
                    $this->child_last_level($category_tree['children']);
                } else {
                    $this->search_tree[] = array(
                        'id' => $category_tree['id'],
                        'name' => str_replace("'", "", $category_tree['name']),
                        'description' => $this->parents_description($category_tree['id'])
                    );
                    $this->tree_key_ids[$category_tree['id']] = array(
                        'id' => $category_tree['id'],
                        'name' => $category_tree['name'],
                        'description' => $this->parents_description($category_tree['id'])
                    );
                }
            }
        }
    }
    
    /**
     * Use parents name as description
     * @param number $category_id
     * @return string
     */
    private function parents_description($category_id) {
        $parents = $this->get_parent($category_id, NULL, 'desc');
        $description = array();
        if($parents) {
            foreach($parents as $parent) {
                $description[] = str_replace("'", "", $parent->name);
            }
        }
        if(!empty($description)) {
            $description = implode(' -> ',$description);
            return $description;
        } else {
            return '';
        }
    }
}