<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
 || ##################################################################### ||
 || # GoClixy - Business Directory Script							   # ||
 || # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
 || # ----------------------------------------------------------------  # ||
 || # This file is owned and copyrighted by GoClixy.com. Your license   # ||
 || # confers  no  title  or ownership in the Script and is not a sale  # ||
 || # of any rights to the Software. This file may not be redistributed # ||
 || # in whole or significant part.									   # ||
 || # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
 || # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
 || ##################################################################### ||
 \*=======================================================================*/
class Gdb_migrate {
    /**
     * A common function to migrate tables fields and rows to an existing database
     * 
     * Stores the CodeIgniter core object.
     *
     * @access public
     *
     * @var object
     */
    protected $CI;
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct($config = array()) {
        $this->CI =& get_instance();
    }    
    
    /**
     * Export Tables Data
     * Call this function to create a JSON file of database
     * @return boolean
     */
    public function export() {
        $structure = array();
        $tables = $this->CI->db->list_tables();
        foreach($tables as $table) {
            $table = str_replace($this->CI->db->dbprefix, '', $table);
            $fields = array();
            $field_data = $this->CI->db->field_data($table);
            if($field_data) {
                foreach($field_data as $data_field) {
                    $fields[$data_field->name] = $data_field;
                }
            }
            // $sql = "SELECT DISTINCT INDEX_NAME FROM INFORMATION_SCHEMA.STATISTICS WHERE TABLE_NAME='" . $table . "'";
            $sql = "SHOW INDEX FROM ". $this->CI->db->dbprefix . $table;
            $indexes = $this->CI->db->query($sql)->result();
            
            $structure[$table] = array(
                'fields' => $fields,
                'indexes' => $indexes
            );
        }
        
        // rows data
        $rows = array();
        $tables = array('permissions', 'roles', 'role_permissions', 'modules', 'settings');
        foreach($tables as $table) {
            $records = $this->CI->db->get($table)->result();
            $rows[$table] = $records;
        }
        $data = array(
            'structure' => $structure,
            'rows' => $rows
        );
        $content = json_encode($data);
        // Write the changes out...
        if (! function_exists('write_file')) {
            $this->CI->load->helper('file');
        }
        
        // save file
        $file = FCPATH . 'uploads/db.json';
        if (write_file($file, $content)) {
            return true;
        }
    }
    
    /**
     * Import Structure and Indexes
     * Call this function to view the missing tables and indexes SQL
     * @return boolean
     */
    public function import() {
        $file = FCPATH . 'uploads/db.json';
        $file = file($file);
        if(empty($file[0])) {
            return false;
        }
        $content = json_decode($file[0]);
        if(empty($content->structure)) {
            return false;
        }
        $json_content = $content->structure;
        $fields_order = array();
        foreach($json_content as $jkey => $jvalue) {
            foreach($jvalue->fields as $jfield) {
                $fields_order[$jkey][] = $jfield->name;
            }
        }
        $tables = array();
        $tables_data = $this->CI->db->list_tables();
        if($tables_data) {
            foreach($tables_data as $table_data) {
                $tables[] = str_replace($this->CI->db->dbprefix, '', $table_data);
            }
        }
        $queries = array();
        foreach($json_content as $table => $structure) {
            if(in_array($table, $tables)) {
                // table exists in the database, compare fields
                $new_fields = $structure->fields;
                if(!empty($new_fields)) {
                    // get existing fields
                    $existing_fields = array();
                    $field_data = $this->CI->db->field_data($table);
                    foreach($field_data as $data_field) {
                        $existing_fields[$data_field->name] = $data_field;
                    }
                    foreach($new_fields as $new_field_name => $new_field_structure) {
                        if(array_key_exists($new_field_name, $existing_fields)) {
                            $diff = array_diff((array) $new_field_structure, (array) $existing_fields[$new_field_name]);
                            if(!empty($diff) && empty($diff['primary_key']) && (!array_key_exists('default', $diff) || $diff['default'] != 'current_timestamp()')) {
                                // field modified
                                $field_format = $this->format_field($new_field_structure);
                                $sql = "ALTER TABLE " . $this->CI->db->dbprefix . $table . " CHANGE " . $new_field_name . " " . $new_field_name . " " . $field_format['type'] . " CHARACTER SET utf8 COLLATE utf8_general_ci " . $field_format['default'] . ";";
                                // $this->CI->db->query($sql);
                                $queries[] = $sql;
                            }
                        } else {
                            // new field add it to table
                            $field_position = array_search($new_field_name, $fields_order[$table]);
                            if($field_position == 0) {
                                $after_field = 'id';
                            } else {
                                $after_field = $fields_order[$table][$field_position - 1];
                            }
                            $field_format = $this->format_field($new_field_structure);
                            $sql = "ALTER TABLE " . $this->CI->db->dbprefix . $table . " ADD " . $new_field_name . " " . $field_format['type'] . " " . $field_format['default'] . " AFTER " . $after_field . ";";
                            // $this->CI->db->query($sql);
                            $queries[] = $sql;
                        }
                    }
                }
            } else {
                // new table import it directly
                $sql = "CREATE TABLE " . $this->CI->db->dbprefix . $table;
                $table_fields = array();
                $new_fields = $structure->fields;
                if(!empty($new_fields)) {
                    foreach($new_fields as $new_field_name => $new_field_structure) {
                        $field_format = $this->format_field($new_field_structure);
                        $table_fields[] = $new_field_name . " " . $field_format['type'] . " " . $field_format['default'];
                    }
                }
                $sql .= "(" . PHP_EOL . implode(',' . PHP_EOL, $table_fields) . PHP_EOL . ");";
                $queries[] = $sql;
            }
            if(in_array($table, $tables)) {
                $queries = array_merge($queries, $this->table_indexes($table, $structure->indexes));
            }
        }
        // import rows data
        $rows = $this->_import_rows($file);
        $content = array(
            'queries' => $queries,
            'rows' => $rows,
            'content' => $content
        );
        // save file
        $file = FCPATH . 'uploads/imports/process_db.json';
        // Write the changes out...
        if (! function_exists('write_file')) {
            $this->CI->load->helper('file');
        }
        if (write_file($file, json_encode($content))) {
            $result = implode('<br />', $queries);
            $rows = json_encode($rows);
            $output = <<<EOL
<precode>{$result}</precode>
<br />
$rows
EOL;
            echo $output;
            return true;
        }
    }
    
    /**
     * Add table indexes
     * @param string $table
     * @param object $indexes
     * @return string[]
     */
    private function table_indexes($table, $indexes) {
        $table_indexes = array();
        if(!empty($indexes)) {
            // get table current indexes
            $sql = "SHOW INDEX FROM ". $this->CI->db->dbprefix . $table;
            $current_indexes = $this->CI->db->query($sql)->result();
            $indexes_current = array();
            if($current_indexes) {
                foreach($current_indexes as $current_index) {
                    if(array_key_exists($current_index->Key_name, $indexes_current)) {
                        $indexes_current[$current_index->Key_name]['columns'][] = $current_index->Column_name;
                    } else {
                        $current_index->columns = array($current_index->Column_name);
                        $indexes_current[$current_index->Key_name] = (array) $current_index;
                    }
                }
            }
            // format new indexes, add distinct indexes only
            $indexes_table = array();
            foreach($indexes as $table_index) {
                if(array_key_exists($table_index->Key_name, $indexes_table)) {
                    $indexes_table[$table_index->Key_name]['columns'][] = $table_index->Column_name;
                } else {
                    $table_index->columns = array($table_index->Column_name);
                    $indexes_table[$table_index->Key_name] = (array) $table_index;
                }
            }
            foreach($indexes_table as $index) {
                $proceed = true;
                if(array_key_exists($index['Key_name'], $indexes_current)) {
                    if(!empty($indexes_current[$index['Key_name']])) {
                        if(empty(array_diff($indexes_current[$index['Key_name']]['columns'], $index['columns']))) {
                            $proceed = false;
                        }
                    }
                }
                // ALTER TABLE `rs0d_listings__` CHANGE `id` `id` BIGINT(20) NOT NULL AUTO_INCREMENT;
                if($proceed) {
                    switch($index['Key_name']) {
                        case 'PRIMARY':
                            $table_indexes[] = "ALTER TABLE " . $this->CI->db->dbprefix . $table . " ADD PRIMARY KEY (`" . $index['Column_name'] . "`);";
                            break;
                        default:
                            $index_columns = array();
                            foreach($index['columns'] as $column) {
                                $index_columns[] = "`" . $column . "`";
                            }
                            $index_columns = implode(',', $index_columns);
                            if($index['Index_type'] == 'BTREE') {
                                $table_indexes[] = "ALTER TABLE " . $this->CI->db->dbprefix . $table . " ADD INDEX " . $index['Key_name'] . " (" . $index_columns . ");";
                            } else if($index['Index_type'] == 'FULLTEXT') {
                                $table_indexes[] = "ALTER TABLE " . $this->CI->db->dbprefix . $table . " ADD FULLTEXT " . $index['Key_name'] . " (" . $index_columns . ");";
                            }
                    }
                }
            }
        }
        return $table_indexes;
    }
    
    /**
     * Format Field
     * @param object $new_field_structure
     * @return string[]
     */
    private function format_field($new_field_structure) {
        $field_type = strtoupper($new_field_structure->type);
        if(is_numeric($new_field_structure->default) && $new_field_structure->default == 0) {
            $field_default = 'NOT NULL DEFAULT ' . $new_field_structure->default;
        } else {
            $field_default = empty($new_field_structure->default) ? 'NULL' : 'NOT NULL DEFAULT ' . (is_numeric($new_field_structure->default) ? $new_field_structure->default : "'" . $new_field_structure->default . "'");
        }
        if(!empty($new_field_structure->max_length)) {
            $field_type .= '(' . $new_field_structure->max_length . ')';
        }
        if($new_field_structure->name == 'id') {
            $field_default = 'NOT NULL';
        }
        return array('type' => $field_type, 'default' => $field_default);
    }
    
    
    /**
     * Imports Rows Data Only
     * @param array $content
     */
    private function _import_rows($content) {
        // read file
        $result = array();
        if(!empty($content[0])) {
            $tables = array('permissions', 'roles', 'modules', 'settings');
            $content = json_decode($content[0], true);
            if(!empty($content['rows'])) {
                foreach($tables as $table) {
                    switch($table) {
                        case 'settings':
                        case 'permissions':
                        case 'modules':
                            $result = array_merge($result, $this->_handle_rows($content, $table, 'name'));
                            break;
                        case 'roles':
                            $result = array_merge($result, $this->_handle_rows($content, $table, 'role_name'));
                            break;
                    }
                }
            }
        }
        return $result;
    }
    
    /**
     * Handle Rows
     * @param array $content
     * @param string $table
     * @param string $field
     * @return array[][]
     */
    private function _handle_rows($content, $table, $field = 'name') {
        $result = array();
        $existing_rows = array();
        $existing_records = $this->CI->db->get($table)->result();
        if($existing_records) {
            foreach($existing_records as $existing_record) {
                $existing_rows[] = $existing_record->{$field};
            }
        }
        $table_content = $content['rows'][$table];
        if(!empty($table_content)) {
            foreach($table_content as $content_table) {
                if(!in_array($content_table[$field], $existing_rows)) {
                    /*if($table == 'permissions') {
                     unset($content_table['permission_id']);
                     }*/
                    $result[$table]['insert'][] = $content_table;
                }
            }
        }
        return $result;
    }
    
    /**
     * Process Import
     */
    private function process_import() {
        $file = FCPATH . 'uploads/imports/process_db.json';
        $file = file($file);
        if(empty($file[0])) {
            return false;
        }
        $content = json_decode($file[0], true);
        $queries = !empty($content['queries']) ? $content['queries'] : array();
        $rows = !empty($content['rows']) ? $content['rows'] : array();
        if(!empty($queries)) {
            foreach($queries as $query) {
                $this->CI->db->query($query);
            }
        }
        if(!empty($rows)) {
            $permission_roles = array();
            if(!empty($content['rows']['role_permissions'])) {
                $role_permissions = $content['rows']['role_permissions'];
                if(!empty($role_permissions)) {
                    foreach($role_permissions as $role_permission) {
                        $permission_roles[$role_permission['permission_id']][] = $role_permission['role_id'];
                    }
                }
            }
            foreach($rows as $tbl => $data) {
                switch($tbl) {
                    case 'permissions':
                        foreach($data['insert'] as $permission) {
                            $permission_id = $permission['permission_id'];
                            unset($permission['permission_id']);
                            $role_permission = array();
                            if(!empty($permission_roles[$permission_id])) {
                                $role_permission = $permission_roles[$permission_id];
                            }
                            // insert permission
                            $this->CI->db->insert($tbl, $permission);
                            $permission_id = $this->CI->db->insert_id();
                            if($permission_id > 0 && !empty($role_permission)) {
                                // insert role_permissions
                                $role_permission_data = array();
                                foreach($role_permission as $role_id) {
                                    $role_permission_data[] = array('role_id' => $role_id, 'permission_id' => $permission_id);
                                }
                                $this->CI->db->insert_batch('role_permissions', $role_permission_data);
                            }
                        }
                        break;
                    default:
                        $this->CI->db->insert_batch($tbl, $data['insert']);
                }
            }
        }
    }
}