<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');
/**
 * Emailer/SMS library to send Email and SMS throughout the application
 * 
 * E.g Usages
 * Notify Referrer - $this->gemailer->sms_email_pdf('email', array('template_table' => 'affiliate_email_templates', 'module_table' => 'affiliate_users', 'action_type' => 'notify_referrer', 'id' => 2));
 * Add Affiliate - $this->gemailer->sms_email_pdf('email', array('template_table' => 'affiliate_email_templates', 'module_table' => 'affiliate_users', 'action_type' => 'add', 'id' => 2));
 * @author goclixy
 */
class Gemailer {
    
    // To store instance of codeigniter
    public $CI;
    protected $gemailer_cache;
    protected $template_table = 'email_templates';
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        $this->CI =& get_instance();
        $this->CI->load->library('gcrud');
        $this->CI->load->library('slug');
        /*$this->CI->load->library('gzf_cache', array('lifetime' => 900));
        $this->gemailer_cache = $this->gzf_cache->get_instance();
        
        */
    }
    
    /**
     * Send Email and SMS in background
     * @param string $type
     * @param array $data
     */
    public function sms_email_pdf($type = 'email', $data = array()) {
        if($type == 'pdf' || $type == 'view') {
            return $this->_sms_email_pdf($type, $data);
        } else {
            // run task in background
            if(settings_item('site.background_tasks') == 1) {
                $data['gemailer_type'] = $type;            
                $this->CI->gc->background_task('emailer', json_encode($data));
                exec_background('emailer');
                return array('emailer_success' => true, 'sms_success' => true, 'emailer_error_message' => null);
            } else {
                return $this->_sms_email_pdf($type, $data);
            }
        }
    }

    /**
     * Common function to send SMS/Email and PDF
     * @param number $type - sms_email|email|sms|pdf|view
     * view - return formatted data
     * @param array $data
     * @return boolean
     */
    public function _sms_email_pdf($type = 'email', $data = array()) {
        $this->CI->load->library('parser');
        $this->CI->load->library('emailer/emailer');
        // Email Template Parameters
        $where = '';
        $email_template_table = isset($data['template_table']) ? $data['template_table'] : '';
        if(!empty($data['where'])) {
            $where = $data['where'];
        } else if(!empty($data['template_id'])) {
            $where = array('id' => $data['template_id']);
        } else if(!empty($data['module_table']) && $email_template_table != 'email_templates') {
            $where = array('module_table' => $data['module_table']);            
            if(!empty($data['action_type'])) {
                $where['action_type'] = $data['action_type'];
            }
        }    
        if(!empty($data['user_type'])) { // send email to a specific user type
            $where['user_type'] = $data['user_type'];
        }        
        if(!empty($data['group_id'])) { // subscribers - send email to a specific group
            $where['group_id'] = $data['group_id'];
        }
        // do not check the status field value in case of template preview
        if(empty($data['email_template_fields'])) {
            $where['status'] = 1;
        }
        if(empty($where) && empty($email_template_table)) {
            return false;
        }
        // do not proceed further if no template exists
        if(!empty($where)) {
            $this->CI->db->where($where);
        }
        $email_templates = $this->CI->db->get($email_template_table)->result();
        if(!$email_templates && empty($data['with_default'])) {
            return false;
        }
        
        // other parameters
        $send_email = $type == 'sms_email' || $type == 'email' ? true : false;
        $send_sms = $type == 'sms_email' || $type == 'sms' ? true : false;
        // $queueOverride If true, forces the email to be queued. If false, forces the email to be sent immediately.
        $queue_override = !empty($data['queue_override']) ? true : false;
        if($type != 'pdf' && $type != 'view' && !$send_email && !$send_sms) {
            return false;
        }
        $record = $this->get_email_fields($data);        
        if(empty($email_templates) && !empty($data['with_default']) && !empty($record['record']['default_template'])) {
            $email_templates = $this->_tpl($data['module_table'], $data['action_type'], $record['record']['default_template']);
        }
        
        // send information and message to a specific user
        $from = !empty($data['from']) ? $data['from'] : null;
        $email_to = isset($data['email_to']) ? $data['email_to'] : $record['email_to'];
        $message_to = isset($data['message_to']) ? $data['message_to'] : $record['message_to'];
        $message_params = $record['record'];
        $message_params['value_from'] = $from;
        
        // Get Email Template
        $default_email_data = array(
            'from'		=> $from,
            'to'		=> $email_to
        );
        // default headers
        if(!empty($data['headers'])) {
            $default_email_data['headers'] = $data['headers'];
        } else if(!empty($message_params['headers'])) {
            $default_email_data['headers'] = $message_params['headers'];
        }
        
        $emails_data = array();
        $default_sms_data = array(
            'to' => !empty($message_to) ? $message_to : '',
            'message' => null
        );
        $messages_data = array();
        foreach($email_templates as $email_template) {
            $template_email_data = $default_email_data;
            // additional parameters like to, cc, bcc can be added here in a specific format
            $params = $this->email_param_values($email_template->params);
            if(!empty($params['cc'])) {
                $template_email_data['headers']['cc'] = $params['cc'];
            }
            if(!empty($params['bcc'])) {
                $template_email_data['headers']['bcc'] = $params['bcc'];
            }
            if(!empty($data['attachments'])) { // send files as an attachments
                $template_email_data['attachments'] = $data['attachments'];
            }
            if(!empty($params['queue_override'])) { // override the queue
                $queue_override = true;
            }
            $template_email_data['subject'] = $this->CI->parser->parse_string($email_template->subject, $message_params, TRUE);
            $template_email_data['message'] = $this->CI->parser->parse_string($email_template->message, $message_params, TRUE);
            switch($email_template->user_type) {
                case 'admin': // administrator
                    $template_email_data['to'] = !empty($params['to']) ? $params['to'] : settings_item('site.system_email');
                    $default_sms_data['to'] = !empty($params['phone_number']) ? $params['phone_number'] : null;
                    // send email with reply_name and reply_email header
                    switch($data['module_table']) {
                        case 'contact_queries':
                            $template_email_data['headers'] = array(
                                'reply_name' => $message_params['value_name'],
                                'reply_email' => $message_params['value_email']
                            );
                            break;
                        case 'forum_threads':
                        case 'forum_posts':                            
                            $template_email_data['headers'] = array(
                                'reply_email' => $message_params['value_user_email'],
                                'reply_name' => $message_params['value_display_name']
                            );
                            break;
                    }
                    break;
                case 'user':
                    if($data['module_table'] == 'subscriber_users') {
                        $this->CI->load->library('crypt');
                        $template_email_data['message'] = gc_unsubscribe($this->CI->crypt->encode(isset($data['id']) ? $data['id'] : 0), 'subscriber_users', $template_email_data['message']);
                    }
                    $template_email_data['to'] = $email_to;
                    $default_sms_data['to'] = $message_to;
                    if(!empty($record['headers'])) {
                        $template_email_data['headers'] = $record['headers'];
                    } 
                    break;
                case 'seller':
                    $template_email_data['to'] = !empty($message_params['value_seller_email']) ? $message_params['value_seller_email'] : '';
                    $default_sms_data['to'] = !empty($message_params['value_seller_phone_number']) ? $message_params['value_seller_phone_number'] : '';
                    break;
                case 'assigned_user': // Tickets - Assigned User
                    $default_sms_data['to'] = $message_params['value_assigned_phone_number'];
                    $template_email_data['to'] = $message_params['value_assigned_user_email'];
                    $template_email_data['headers'] = array(
                        'reply_email' => $message_params['value_user_email'],
                        'reply_name' => $message_params['value_display_name']
                    );
                    break;
                case 'guest':
                    $template_email_data['to'] = !empty($data['email_to']) ? $data['email_to'] : null;
                    $default_sms_data['to'] = !empty($data['message_to']) ? $data['message_to'] : null;
                    if(empty($template_email_data['to']) && ($email_template->action_type == 'erroneous' || $email_template_table == 'query_email_templates')) {
                        $template_email_data['to'] = $message_params['value_email'];
                        $default_sms_data['to'] = $message_params['value_phone_number'];
                    }
                    break;
                default:
                    $template_email_data['to'] = $email_to;
                    $default_sms_data['to'] = $message_to;
            }
            $emails_data[] = $template_email_data;
            
            // prepare sms data
            if(!empty($email_template->sms_text)) {
                $default_sms_data['message'] = $this->CI->parser->parse_string($email_template->sms_text, $message_params, TRUE);
                $messages_data[] = $default_sms_data;
            }
        }
        
        // return processed content
        if($type == 'view' && !empty($emails_data[0])) {
            $view_data = $emails_data[0];
            $view_data['sms_text'] = !empty($messages_data[0]) ? $messages_data[0]['message'] : '';
            return $view_data;
        }
        
        // generate PDF
        if($type == 'pdf' && !empty($emails_data[0]['message'])) {
            if(!empty($message_params['value_title'])) {
                $filename = $message_params['value_title'];
            } else {
                $filename = !empty($data['filename']) ? $data['filename'] : date('Y-m-d H:i:s');
                $filename = $this->CI->slug->create_slug($filename);
            }
            $content = $emails_data[0]['message'];
            $content = $this->CI->load->view('orders/_tpl_pdf', array('content' => $content), true);
            $this->CI->gc->create_pdf($filename, $content, true, array('download' => true));
        }
        
        // send email
        $email_success = false;
        if(!empty($emails_data) && $send_email) {
            foreach($emails_data as $data) {
                if(!empty($data['to'])) {
                    $email_success = $this->CI->emailer->send($data, $queue_override);
                }
            }
        }
        
        // send sms
        $sms_success = false;
        if(!empty($messages_data) && $send_sms) {
            foreach($messages_data as $message_data) {
                if(!empty($message_data['to']) && !empty($message_data['message'])) {
                    $sms_success = $this->CI->gc->send_sms($message_data['message'], $message_data['to'], $queue_override);
                }
            }
        }
        return array('emailer_success' => $email_success, 'sms_success' => $sms_success, 'emailer_error_message' => ($send_email && !$email_success ? $this->CI->emailer->error . ", " . $this->CI->emailer->debug_message : ''));
    }
    
    /**
     * Get Email Fields
     * All email data must be retrieved here, so that it can be displayed in available fields list
     * 
     * To avoid complexity it is better to have main code in the modules itself, 
     * call the function here
     * 
     * @param array $data
     * @return boolean|array
     */
    private function get_email_fields($data) {
        $row_id = isset($data['id']) ? $data['id'] : 0;
        $module = $data['module_table'];
        $action_type = isset($data['action_type']) ? $data['action_type'] : null;
        $email_to = '';
        $message_to = '';
        $headers = array();
        // passed only in preview or view of template fields
        if(isset($data['email_template_fields'])) {
            return array('record' => $data['email_template_fields'], 'email_to' => $email_to, 'message_to' => $message_to, 'headers' => $headers);
        }
        switch($action_type) {
            case 'notify_referrer': // Affiliates Module - Notify Referrer
                $this->CI->load->model('affiliates/affiliates_model');
                $affiliate = $this->CI->affiliates_model->get_affiliate_user($row_id, true, 'id');
                // meta data is not yet saved in the affiliate_users table, so get it from the database
                $record = $this->CI->gcrud->find_meta_for('affiliate_users', $row_id, $affiliate);
                if(empty($data['record'])) {
                    $referrer = array();
                    if(!empty($record->ref_by)) {
                        $referrer = $this->CI->affiliates_model->get_affiliate_user($record->ref_by, true);
                    }
                } else {
                    $referrer = $data['record'];
                }
                if(!empty($referrer)) {
                    $email_to = $referrer->user_email;
                    $message_to = $referrer->phone_number;
                    $record->referrer_name = $referrer->name; 
                }
                break;
            case 'reminders': // order reminders
                if(!empty($data['record'])) {
                    $record = (array) $data['record'];
                } else { // Template Preview Only
                    $this->CI->load->model('orders/orders_model');
                    $record = $this->CI->orders_model->notifications(0, 1, false);
                    if($record) {
                        $record = $record[0];
                        if(empty($record->sequence_no)) {
                            $record->sequence_no = '00';
                        }
                    }
                }
                break;
            case 'bulk': // listings bulk users generate
                if(!empty($data['record'])) {
                    $record = (array) $data['record'];
                } else { // Template Preview Only
                    $this->CI->load->model('listings/listings_model');
                    $record = $this->CI->listings_model->bulk_notify($row_id, array(), !empty($data['user_type']) ? $data['user_type'] : null);
                }
                break;
            case 'bid_notification':
                $this->CI->load->model('ads/ads_model');
                $record = $this->CI->ads_model->higher_bid_notification(!empty($data['record']) ? $data['record'] : null);
                break;
            case 'erroneous': // incorrect reported
                $this->CI->load->model('erroneous/erroneous_model');
                $record = $this->CI->erroneous_model->get_erroneous($row_id);
                if($record) {
                    $record->listing_url = gc_url_format('listings', $record->listing_slug, $record->listing_id, urlencode($record->listing_city));
                    $record->listing_url = anchor($record->listing_url, $record->listing_url);
                    $record->incorrect_info = $this->CI->erroneous_model->incorrect_options($record->type_id);
                    // listing owner notification                   
                    $email_to = $record->listing_email;
                    $message_to = $record->listing_phone_number;                    
                    $headers = array(
                        'reply_email' => $record->email,
                        'reply_name' => $record->name
                    );
                }
                break;
            case 'email_templates':
                $this->CI->load->model('emailer/emailer_model');
                $record = $this->CI->emailer_model->template_fields();
                break;
            default:
                $record = array();
                // get and format field values
                if(!empty($data['record'])) {
                    $record = (array) $data['record'];
                } else if(!empty($module)) {
                    switch($module) {
                        case 'affiliate_users':
                            $this->CI->load->model('affiliates/affiliates_model');
                            $record = $this->CI->affiliates_model->get_affiliate_user($row_id, true, 'id');                            
                            if(!empty($record)) {
                                $email_to = $record->user_email;
                                $message_to = $record->phone_number;
                            }
                            break;
                        case 'order_items':
                            $this->CI->load->model('orders/orders_model');
                            $record = $this->CI->orders_model->order_item_emailer($row_id);
                            break;
                        case 'subscriber_users':
                            $this->CI->load->model('subscribers/subscribers_model');
                            $record = $this->CI->subscribers_model->get_info((!empty($data['group_id']) ? $data['group_id'] : 0), $row_id);
                            $data['processed'] = 1;
                            if(!empty($record['value_email'])) {
                                $email_to = $record['value_email'];
                            }
                            if(!empty($record['value_phone_number'])) {
                                $message_to = $record['value_phone_number'];
                            }
                            break;
                        case 'leads':
                            $this->CI->load->model('leads/leads_model');
                            $record = $this->CI->leads_model->get_info((!empty($data['group_id']) ? $data['group_id'] : 0), $row_id);
                            $data['processed'] = 1;
                            if(!empty($record['value_email'])) {
                                $email_to = $record['value_email'];
                            }
                            if(!empty($record['value_phone_number'])) {
                                $message_to = $record['value_phone_number'];
                            }
                            break;
                        case 'payouts':
                            $this->CI->load->model('payouts/payouts_model');
                            $record = $this->CI->payouts_model->info($module, $row_id);
                            $data['processed'] = 1;
                            if(!empty($record['value_email'])) {
                                $email_to = $record['value_email'];
                            }
                            if(!empty($record['value_phone_number'])) {
                                $message_to = $record['value_phone_number'];
                            }
                            break;
                        case 'wallets':
                        case 'points':
                            $this->CI->load->model('wallets/wallets_points_model');
                            $record = $this->CI->wallets_points_model->info($module, $row_id);
                            $data['processed'] = 1;
                            if(!empty($record['value_email'])) {
                                $email_to = $record['value_email'];
                            }
                            if(!empty($record['value_phone_number'])) {
                                $message_to = $record['value_phone_number'];
                            }
                            break;
                        case 'ads':
                            $this->CI->load->model('ads/ads_model');
                            $record = $this->CI->ads_model->get_ad($row_id);
                            break;
                        case 'listing_claims':
                            $this->CI->load->model('listings/claims_model');
                            $record = $this->CI->claims_model->claim_email($row_id);
                            break;
                        default:        
                            if(in_array($module, gc_modules())) {
                                if($data['template_table'] == 'query_email_templates') {
                                    $this->CI->load->model('business_queries/business_queries_model');
                                    $record = $this->CI->business_queries_model->get_email_data($row_id);
                                    if($record) {
                                        if(!empty($record->author_email)) {
                                            $email_to = $record->author_email;
                                        }
                                        if(!empty($record->author_phone_number)) {
                                            $message_to = $record->author_phone_number;
                                        }
                                    }
                                } else {
                                    $record = $this->CI->gc->module_data($module, $row_id);
                                    $data['processed'] = 1;
                                    if(!empty($record['value_email'])) {
                                        $email_to = $record['value_email'];                                    
                                    } else if(!empty($record['value_author_email'])) {
                                        $email_to = $record['value_author_email'];                                        
                                    }
                                    if(!empty($record['value_phone_number'])) {
                                        $message_to = $record['value_phone_number'];
                                    } else if(!empty($record['value_author_phone_number'])) {
                                        $message_to = $record['value_author_phone_number'];
                                    }
                                }
                            } else {
                                $records = $this->CI->gcrud->get_all_crud(0, 'M.id = ' . $row_id, 'M.id ASC', 1, 0, array('module' => $module, 'view' => true, 'cache' => $module));
                                if(!empty($records[0])) {
                                    $record = $records[0];
                                }
                            }
                    }
                }
        }
        if(!$record) {
            Template::set_message(lang('invalid_id'), 'danger');
            return array('record' => array(), 'email_to' => $email_to, 'message_to' => $message_to, 'headers' => $headers);
        }
        $record = (array) $record;
        $message_params = array();
        // the data has already been processed, the prefix value_, label_ already adding in the module files
        if(empty($data['processed']) && !empty($record)) {
            foreach($record as $key => $value) {
                switch($key) {
                    case 'cover':
                        $message_params['value_' . $key] = !empty($value) ? '<img src="' . base_url('assets/images/' . $value) . '" class="img-fluid">' : '';
                        break;
                    case 'profile_user': // user's profile link
                        $message_params['value_profile_user'] = prepare_urls($value, gc_url_format('user') . '?id=' . $record['user_id']);
                        break;
                    case 'redirect_uri': // product detail page url
                        $message_params['value_product_url'] = prepare_urls($value);
                        break;
                    case 'created_on':
                    case 'modified_on':
                    case 'reported_on':
                        $message_params['value_' . $key] = gc_date_format($value);
                        break;
                    case 'status':
                        switch($module) {
                            default:
                                $message_params['value_' . $key] = lang($value);                                
                        }
                        break;
                    default:
                        $message_params['value_' . $key] = $value;
                }
            }
        } else if(!empty($record)) {
            $message_params = $record;
        }
        // set label parameters
        if($module == 'orders') { 
                foreach($record as $rkey => $rval) {
                    $rlabel = lang('label_' . $rkey);
                    if(strpos($rlabel, 'FIXME') === false) {
                        $message_params['label_' . $rkey] = $rlabel;
                }
            }          
            $additional_keys = array('invoice_no', 'order_amount', 'order_status', 'order_summary', 'customer_information', 'order_query');
            foreach($additional_keys as $aval) {
                $message_params['label_' . $aval] = lang('label_' . $aval);
            }
        }
        // additional fields
        /*switch($module) {
            case 'blogs':
                $blog_url = gc_url_format('blogs/detail', $record['slug']);
                $message_params['value_blog_url'] = anchor($blog_url, $blog_url);
                break;
            case 'products':
                $product_url = gc_url_format('products', $record['slug'], $record['id']);
                $message_params['value_product_url'] = anchor($product_url, $product_url);
                break;
        }*/
        $message_params['value_site_logo'] = '<img src="' . gc_image_url(settings_item('site.logo'), false) . '" />';
        $message_params['value_site_title'] = settings_item('site.title');;
        $message_params['value_site_email'] = settings_item('site.system_email');
        $message_params['value_site_url'] = anchor(site_url(), site_url());
        $message_params['value_site_address'] = settings_item('site.address');
        $message_params['value_site_phone'] = settings_item('site.phone_number');
        
        // users/forums table
        if(empty($email_to)) {
            if(!empty($record['email'])) {
                $email_to = $record['email'];
            } else if(!empty($record['user_email'])) {
                $email_to = $record['user_email'];               
            }
        }
        if(empty($email_to)) {
            if(!empty($record['phone_number'])) {
                $message_to = $record['phone_number'];
            } else if(!empty($record['user_phone_number'])) {
                $message_to = $record['user_phone_number'];
            }
        }
        
        if(!empty($message_params['value_link'])) {
            $message_params['value_link'] = anchor($message_params['value_link'], $message_params['value_link']);
        }
        return array('record' => $message_params, 'email_to' => $email_to, 'message_to' => $message_to, 'headers' => $headers);
    }
    
    /**
     * The email can be sent to a specific email id, CC and BCC to a specific Email
     * @param string $params
     * @return string[]|mixed[]|NULL
     */
    private function email_param_values($params) {
        if(!empty($params)) {
            $param_values = array();
            // get all params
            $params_array = array_map('trim', explode(';', $params));
            if($params_array) {
                foreach($params_array as $param_field) {
                    $param_value = array_map('trim', explode(':', $param_field));
                    $param_values[$param_value[0]] = (isset($param_value[1]) ? $param_value[1] : '');
                }
            }
            return $param_values;
        }
        return null;
    }
    
    /**
     * Template Preview and Available Fields
     * @param number $id - ID of the Template
     * @param string $table_prefix - Template Table Prefix e.g. form, ticket, blog etc.
     * @param boolean $preview - True - Display Email Preview
     * @return boolean
     */
    public function fields_preview($id = 0, $table_prefix = '', $preview = true) {
        $this->CI->lang->load('admin');
        $content = '';
        $heading = '';
        if($table_prefix != 'subscriber') {
            $table_prefix = !empty($table_prefix) && $table_prefix != 'email_templates' ? $table_prefix . '_' : '';
            $template_table = $table_prefix . 'email_templates';
        } else {
            $template_table = $table_prefix . '_templates';
        }
        $record = $this->CI->db->where('id', $id)->get($template_table)->row();
        if($record) {
            $heading = $record->name;
            $data = array();
            if($template_table == 'email_templates') { // predefined email templates
                $record->module_table = 'email_templates';
                $record->user_type = '';
                $record->action_type = 'email_templates';
            }
            $module = $record->module_table;
            // get random row
            if($record->action_type == 'erroneous') {
                $table = 'listing_erroneous';
            } 
            if($template_table == 'query_email_templates') {
                $table = 'business_queries';
            } else if($template_table == 'wallets_email_templates') {
                $table = 'wallets_history';
            } else {
                $table = $module;
            }
            $random_row = $this->CI->db->order_by('rand()')->get($table)->row(); // order_by('rand()')->
            if($random_row) {
                $data['id'] = $random_row->id;
            } else {
                $data['id'] = 1;
            }
            $data['template_table'] = $template_table;
            $data['module_table'] = $module;
            $data['user_type'] = $record->user_type;
            $data['action_type'] = $record->action_type;
            // subscribers templates
            if($template_table == 'subscriber_templates') {
                $data['group_id'] = $record->group_id;                
            }
            // ticket replied message id
            if($record->module_table == 'tickets' && $record->action_type == 'reply') {
                // get message id
                $ticket_message = $this->CI->db->where('ticket_id', $data['id'])->get($module)->row();
                if($ticket_message) {
                    $data['extra'] = array('ticket_message_id' => $ticket_message->id);
                }
            }
            // listings bulk users generate
            if($record->action_type == 'bulk' && $record->user_type == 'admin') {
                $data['user_type'] = $record->user_type;
            }
            if($record->action_type == 'expiry_notification') {
                $this->CI->load->library('cronjobs/cronjobs_tasks');
                $data['record'] = $this->CI->cronjobs_tasks->_expired_email($data['module_table'], null);
            }
            if($record->module_table == 'orders') { //  && $record->action_type == 'status_update'
                $this->CI->load->model('orders/orders_model');
                $order = $this->CI->orders_model->get_order(null);
                if($order) {
                    $data['record'] = $this->CI->orders_model->email_print($order->order_id, 'preview', array('order' => $order));
                }                
            }
            if($record->module_table == 'products' && $template_table == 'product_email_templates') { // ecommerce
                $this->CI->load->model('ecommerce/ecommerce_model');
                $data['record'] = $this->CI->ecommerce_model->template_fields($record->action_type);
            }
            if($record->module_table == 'payouts' && empty($record->action_type)) {
                // affilate payment receipt
                $data['processed'] = true;
                $this->CI->load->model('payouts/payouts_model');
                $this->CI->lang->load('affiliates/affiliates');
                $this->CI->lang->load('commissions/commissions');
                $this->CI->lang->load('payouts/payouts');
                $this->CI->lang->load('orders/orders');
                $row = $this->CI->payouts_model->invoice_template_params($data['id']);                
                $row['value_site_title'] = settings_item('site.title');;
                $row['value_site_email'] = settings_item('site.system_email');
                $row['value_site_url'] = site_url();
                $row['value_site_address'] = settings_item('site.address');
                $row['value_site_phone'] = settings_item('site.phone_number');
            } else {
                // get template fields
                $row = $this->get_email_fields($data);
                if($row) {
                    $row = $row['record'];
                    // additional avaialable fields
                    $additional_avaialble_fields = array();
                    switch($record->action_type) {
                        case 'account_activated':
                        case 'activation_code':
                        case 'forgot_password':
                        case 'back_in_stock':
                            $row['value_customer_name'] = 'Testing';
                            $row['value_link'] = site_url();
                            break;
                    }
                    // additional fields to be passed
                    switch($module) {
                        case 'blogs':
                            $row['from'] = 'Testing';
                            break;
                    }
                }
            }
            if($preview) {
                $template_params = array('template_table' => $template_table, 'module_table' => $module, 'action_type' => $record->action_type, 'user_type' => $record->user_type, 'id' => $data['id'], 'email_template_fields' => $row);  
                // additional fields to be passed
                switch($module) {
                    case 'blogs':
                        $template_params['from'] = 'Testing';
                        break;
                }
                $template = $this->sms_email_pdf('view', $template_params);
                if(!empty($template['message'])) {
                    $sms_text = !empty($template['sms_text']) ? '<h5>' . lang('sms_text') . '</h5>' . $template['sms_text'] : '';
                    $lang_preview = lang('em_template_preview');
                    $content = <<<EOL
    <div class="alert alert-info">{$lang_preview}</div>
    <h4>{$template['subject']}</h4>
    {$template['message']}
    {$sms_text}
EOL;
                }
            } else {
                $content_data = array();
                if(!empty($row)) {
                    foreach($row as $key => $value) {
                        switch($key) {
                            case 'value_created_by':
                            case 'value_created_ip':
                            case 'value_modified_by':
                            case 'value_modified_ip':
                            case 'value_status_id':
                            case 'value_deleted':
                            case 'value_deleted_by':
                            case 'value_profile_user':
                            case 'value_status_lang':
                            case 'value_password_hash':
                            case 'value_reset_hash':
                            case 'value_display_name_changed':
                            case 'value_banned':
                            case 'value_product_uri':
                            case 'value_keywords':
                            case 'value_license_settings':
                            case 'value_type_id';
                            case 'value_recurring':
                            case 'value_renewable':
                            case 'value_ticket_support':
                            case 'value_reminder_notifications':
                            case 'value_days_before_next_invoice':
                            case 'value_filename':
                            case 'value_params':
                            case 'default_template':
                                break;
                            default:
                                $has_reserved_keyword = false;
                                $keywords = array('otp', 'tfa', 'hash', 'password', 'meta_', 'invoice_');
                                foreach($keywords as $keyword) {
                                    if(strpos($key, $keyword) !== false) {
                                        $has_reserved_keyword = true;
                                    }
                                }       
                                if(!$has_reserved_keyword) {
                                    $content_data[] = '<tr><td>' . $key . '</td><td class="text-truncate">' . $value . '</td></tr>';
                                }
                                break;
                        }                    
                    }
                }
                $content_data = implode(PHP_EOL, $content_data);
                $lang_name = lang('label_name');
                $lang_value = lang('em_sample_value');
                $help_instructions = lang('help_instructions') . '<br />' . lang('help_label_usage') . '<br />' . lang('help_value_usage');
                $content = <<<EOL
{$help_instructions}
<style>
.img-fluid {
    max-width: 200px;
}
</style>
<div class="table-responsive">
    <table class="table table-striped table-sm">
        <thead>
            <tr>
                <th>{$lang_name}</th>
                <th>{$lang_value}</th>
            </tr>
        </thead>
        <tbody>
            {$content_data}
        </tbody>
    </table>
</div>
EOL;
            }
        }
        $close = lang('form_close');
        $output = <<<EOL
<div class="modal" id="previewModal">
  <div class="modal-dialog mw-100 w-75">
    <div class="modal-content">
      <div class="modal-header bg-primary text-white">
        <h4 class="modal-title">{$heading}</h4>
        <button type="button" class="close" data-dismiss="modal">&times;</button>
      </div>
      <div class="modal-body">
        {$content}
      </div>
      <div class="modal-footer">
        <button type="button" class="btn btn-danger" data-dismiss="modal">{$close}</button>
      </div>
    </div>
  </div>
</div>
EOL;
        echo $output;
        return false;
    }
    
    /**
     * Default Template for Blogs, Products, Properties etc.
     * @param string $module
     * @param string $action_type
     * @param array $data
     * @return stdClass[]
     */
    private function _tpl($module, $action_type, $data) {
        $this->CI->lang->load('emailer/emailer');
        $user_types = array('user'); 
        if($action_type == 'add') {
            $user_types[] = 'admin';
        } else if($action_type == 'share') {
            $user_types = array('guest');            
        }
        $email_templates = array();
        foreach($user_types as $user_type) {
            $subject = '';
            $content = '';
            switch($action_type) {
                case 'share':
                    $subject = sprintf(lang('subject_share'), '{value_author_name}');          
                    break;
                case 'add': // Add Module Notification to the User
                    if($user_type == 'admin') {                    
                        $subject = sprintf(lang('subjet_notify'), lang('singular_' . $module), '{value_title}', '{value_site_title}');
                        $content .= sprintf(lang('content_notify_admin'), lang('singular_' . $module), '{value_author_name}') . '<br />' . PHP_EOL;
                    } else {                    
                        $subject = sprintf(lang('subject_add'), '{value_title}', '{value_site_title}');
                        $content .= sprintf(lang('content_add_module'), lang('singular_' . $module), '{value_title}') . '<br />' . PHP_EOL;
                        $content .= sprintf(lang('content_add_module_1'), '{value_url}');
                    }
                    break;
                case 'status_update': // Edit Module Notification - Display Status information to the User
                    $subject = lang('label_dear') . ' {value_author_name}, ' . sprintf(lang('subject_edit'), lang('singular_' . $module), '{value_title}', '{value_status}');
                    $content .= sprintf(lang('content_edit_module'), lang('singular_' . $module), '{value_status}') . '<br />' . PHP_EOL;
                    break;
            }
            // Load the Default Template file in case of a new module
            // Create a Demo Template
            $content .= '<table style="width: 100%;">';
            if(!empty($data['fields'])) {
                foreach($data['fields'] as $field) {
                    $content .= '<tr><td>{label_' . $field . '}</td><td>{value_' . $field . '}</td></tr>' . PHP_EOL;
                }
            }
            $content .= '</table>' . PHP_EOL;
            if($action_type == 'share') {
                $content .= '<h2>' . lang('author_details') . '</h2>' . PHP_EOL;
                $content .= '<table style="width: 100%;">' . PHP_EOL;
                if(!empty($data['author'])) {
                    foreach($data['author'] as $author_field) {
                        $content .= '<tr><td>{label_' . $author_field . '}</td><td>{value_' . $author_field . '}</td></tr>' . PHP_EOL;
                    }
                }
                $content .= '</table>' . PHP_EOL;
            }
            
            $content .= lang('best_regards') . '<br />' . PHP_EOL;
            if($action_type == 'edit') {
                $content .= lang('more_info') . '<br />{value_url}<br />' . PHP_EOL;
            }
            $content .= '{value_site_title}<br />{value_site_url}';
            $email_template = new stdClass();
            $email_template->subject = $subject;
            $email_template->message = $content;
            $email_template->user_type = $user_type;
            $email_template->sms_text = '';
            $email_template->params = '';
            $email_templates[] = $email_template;
        }
        return $email_templates;
    }
}