<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
 || ##################################################################### ||
 || # GoClixy - Business Directory Script							   # ||
 || # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
 || # ----------------------------------------------------------------  # ||
 || # This file is owned and copyrighted by GoClixy.com. Your license   # ||
 || # confers  no  title  or ownership in the Script and is not a sale  # ||
 || # of any rights to the Software. This file may not be redistributed # ||
 || # in whole or significant part.									   # ||
 || # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
 || # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
 || ##################################################################### ||
 \*=======================================================================*/
class Gfile {
    /**
     * Stores the CodeIgniter core object.
     * 
     * Do not use database to store temporary files
     * Instead use a session variable and store the files in it.
     * On Save, check this session variable value and move files accordingly with new names
     *
     * A column upload_token is created in the tmp_files and images table. 
     * All files removal and download will be done using this token variable.
     * 
     * @access public
     *
     * @var object
     */
    protected $CI;
    protected $file_cache;
    protected $table = 'files';
    protected $table_tmp = 'tmp_files';
    protected $upload_dir = 'assets/files/';
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct($config = array()) {
        $this->CI =& get_instance();
        $this->CI->load->library('slug');
        $this->CI->load->library('gzf_cache', array('lifetime' => 7200)); // 2 hours
        $this->file_cache = $this->CI->gzf_cache->get_instance();
        // name of the table where files to be saved
        if($this->CI->session->userdata('gc_files_table')) {
            $this->table = $this->CI->session->userdata('gc_files_table');
        }
    }
    
    /**
     * Generate a Temporary ID
     * NOTE: Do not forget to unset this id on successfull save
     */
    public function tmp_id() {
        // Unset this session id on form submit so that the user can add a new listing with new session
        // Before redirecting to the activation page
        // Empty tmp_images table on submit and also delete all temporary images
        if(!$this->CI->session->userdata('tmp_files_id')) {
            $tmp_id = time() . rand(1,9999); // . ip2long($this->CI->input->ip_address());
            $this->CI->session->set_userdata('tmp_files_id', $tmp_id);
        }
    }
    
    /**
     * Do not use database to store temporary files
     * Instead use a session variable and store the files in it.
     * On Ticket Save, check this session variable value and move files accordingly with new names
     */
    public function uploader($ajax = true) {
        // check limit first
        $module = $this->CI->input->post('module');
        $module_id = gc_module_id($module);
        $row_id = (int) $this->CI->input->post('row_id');
        $files_limit = (int) $this->CI->session->userdata('files_limit_' . $module . '_' . $row_id);
        $tmp_row_id = $this->CI->session->userdata('tmp_files_id');
        $saved_files_count = $this->CI->db->where(array('row_id' => $row_id, 'module_id' => $module_id))->count_all_results($this->table);
        $temp_files_count = $this->CI->db->where(array('row_id' => $tmp_row_id, 'module_id' => $module_id))->count_all_results($this->table_tmp);
        $total_files = $saved_files_count + $temp_files_count;
        if($total_files >= $files_limit) {
            header('HTTP/1.0 403 ' . sprintf(lang('file_limit'), $files_limit));
            return false;
        }
        
        // sleep for 5 seconds
        // sleep(5);
        $field_name = 'file';
        $config['upload_path'] = realpath(FCPATH . '/assets/tmp/'); // Make SURE that you chmod this directory to 777!
        // $config['allowed_types'] = 'jpg|jpeg|gif|png|txt|pdf|doc|docx|xls|xlsx|dat|zip';
        $config['allowed_types'] = 'pdf|doc|docx|xls|xlsx';
        $config['max_size'] = 1024;
        $config['encrypt_name'] = TRUE; // Encrypt file name with use of CI native option for security reasons
        $config['remove_spaces'] = TRUE; // Remove spaces from the file name
        $this->CI->load->library('upload', $config);        
	    if(!gc_demo_mode(array('no_message' => true, 'no_redirect' => true))) {
	        return false;
	    }
        if (!$this->CI->upload->do_upload('file')) { // File name is necessary
            if($ajax) {
                header('HTTP/1.0 403 ' . $this->CI->upload->display_errors());
            } else {
                $this->CI->session->set_flashdata('upload_file_error', $this->CI->upload->display_errors());
            }
            return false;
        } else {
            $data = array('upload_data' => $this->CI->upload->data());
            if($ajax) {
                // A unique token will be assigned to each uploaded file
                // this token will be used to identify files on the server
                $upload_token = substr(md5(time()), 0, 8) . mt_rand(1,1000) . ip2long($this->CI->input->ip_address());
                $original_filename = $_FILES['file']['name'];
                $uploaded_filename = $data['upload_data']['file_name'];
                $filesize = $data['upload_data']['file_size'];
                $file_extension = $data['upload_data']['file_ext'];
                $file_title = str_replace('_', ' ', $original_filename);
                $file_title = ucwords(str_replace($file_extension, '', $file_title));
                // Using Session
                /*$tmp_files = (array) $this->session->userdata('tmp_files');
                $upload_token = substr(md5(time()), 0, 8) . mt_rand(1,1000);
                $tmp_files[$upload_token] = array(
                    'filename' => $uploaded_filename,
                    'original_filename' => $original_filename
                );*/
                // Single File Upload
                // check if file already exists in the temporary table and folders
                // Using tmp_files table
                $tmp_files_data = array(
                    'module_id' => $module_id,
                    'row_id' => $tmp_row_id,
                    'field_id' => $this->CI->input->post('field_id'), // field name through which file is uploaded, used to display file with the correct FILE input field
                    'title' => $file_title,
                    'upload_token' => $upload_token,
                    'filename' => $uploaded_filename,
                    'original_filename' => $original_filename,
                    'filesize' => $filesize,
                    'file_extension' => $file_extension,
                    'created_on' => date('Y-m-d H:i:s'),
                    'created_ip' => $this->CI->input->ip_address(),
                );
                $this->CI->db->insert($this->table_tmp, $tmp_files_data);                
                echo json_encode(array('filename' => $this->file_url($upload_token, array('is_tmp' => 1, 'action' => 1, 'template' => 4))));
            }
        }
        return false;
    }
    
    /**
     * Create thumbnails of uploaded image
     * @param string $path
     * @param string $upload_path
     * @param string $file
     */
    public function resize_image($upload_path, $file) {
        // If thumbs folder does not exist, create it
        $thumbs_path = FCPATH . '/' . $upload_path . '/thumbs';
        if(!is_dir($thumbs_path)) {
            mkdir($thumbs_path, 0777, true);
        }
        $config['image_library'] = 'gd2';
        // $config['source_image'] = realpath(FCPATH.'/assets/images/'.$upload_path.'/' .$file);
        $config['source_image'] = realpath(FCPATH . '/' . $upload_path . '/' . $file);
        $config['create_thumb'] = TRUE;
        $config['maintain_ratio'] = TRUE;
        $config['overwrite'] = TRUE; // overwrite existing images
        $config['thumb_marker'] = FALSE; // don't want to add the suffix _thumb in thumbnail file name
        $config['width'] = settings_item('site.thumbnail_width');
        $config['height'] = settings_item('site.thumbnail_height');
        $config['new_image'] = './' . $upload_path .'/thumbs/thumb__' . $file;
        
        /*
         * ADD THUMB BEFORE THE NAME
         * $config['create_thumb'] = false;
         * $config['new_image'] = 'thumb_'.$data['file_name'];
         */
        $this->CI->load->library('image_lib', $config);
        /**
         * $this->image_lib->initialize($config); and $this->image_lib->clear();
         * are important else, only one thumbnail will be saved
         */
        $this->CI->image_lib->initialize($config);
        $this->CI->image_lib->resize();
        $this->CI->image_lib->clear();
        $file_thumbnail = FCPATH . '/' . $upload_path . '/thumbs/thumb__' . $file;
        if(file_exists($file_thumbnail)) {
            return true;
        } else {
            return false;
        }
    }
    
    /**
    * Do not use database to store temporary files
    * Instead use a session variable and store the files in it.
    * On Ticket Save, check this session variable value and move files accordingly with new names
    */
    /**
     * Common function to download and remove uploaded files
     * @param string file - Encoded File
     * @param integer hash - 0 - Temporary, 1 - Saved File
     * @param intenger action - 0 - Download, 1 - Remove
     */
    public function manage($is_download = true) {        
        $is_tmp = (int) (isset($_GET['hash']) ? $_GET['hash'] : 0);
        $thumb = isset($_GET['thumb']) ? true : false;
        $action = (int) (isset($_GET['action']) ? $_GET['action'] : 0);
        // assume a link like this: http://www.site.com/download/?id=filename        
        if(!isset($_GET['file'])) { // check if the file attribute is passed or not
            die(lang('file_not_exists'));
        }
        
        // get the filename from the encoded string
        $this->CI->load->library('crypt');
        $filename = escape($this->CI->crypt->decode($_GET['file']));
        if(empty($filename)) { // invalid filename
            die(lang('file_not_exists'));
        }
        
        // Remove unnecessary tags from the filename
        if (!preg_match('/^[-_a-z0-9]+$/i', $filename)) $filename = strip_tags($filename);
        if(empty($filename)) { // invalid filename
            die(lang('file_not_exists'));
        }
        
        // Make sure we can't download files above the current directory location.
        if(preg_match("/^\.\.$/", $filename)) die(lang('file_cannot_download'));
        
        // Make sure we can't download .ht control files.
        if(preg_match("/^\.ht.+$/", $filename)) die(lang('file_permission_denied'));
        
        // Get file name and location
        $table = $is_tmp == 1 ? $this->table_tmp : $this->table;
        $file_info = $this->get_file_info($table, $filename);
        if(!$file_info) {
            die(lang('file_not_exists'));
        }
        
        // Combine the download path and the filename to create the full path to the file.
        $original_filename = $file_info->original_filename;
        $upload_file_dir = $is_tmp == 1 ? 'assets/tmp/' : $this->upload_dir;
        $file = FCPATH . $upload_file_dir . (!empty($file_info->fileindex) ? $file_info->fileindex . '/' : '');
        if($thumb && $is_tmp == 0) { // temporary files have no thumb
            $file .= 'thumbs/thumb__' . $file_info->filename;
        } else {
            $file .= $file_info->filename;
        }        
        
        // Test to ensure that the file exists.
        if(!file_exists($file)) die(lang('file_not_exists'));
        
        if($action == 0) { // Download File
            // Extract the type of file which will be sent to the browser as a header
            $type = filetype($file);
            if($is_download) {
                ob_get_clean();
                header('Content-Description: File Transfer');
                header('Content-Type: application/octet-stream');
                header('Content-Disposition: attachment; filename="' . $original_filename . '"' );
                header('Expires: 0');
                header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
                header('Pragma: public');
                header('Content-Length: ' . filesize($file));
                
                // Send the file contents.
                set_time_limit(0);
                // ob_clean();
                flush();
                readfile($file);
                exit;
            } else {
                // Display image to the user
                $ntct = Array( "1" => "image/gif",
                    "2" => "image/jpeg", #Thanks to "Swiss Mister" for noting that 'jpg' mime-type is jpeg.
                    "3" => "image/png",
                    "6" => "image/bmp",
                    "17" => "image/ico");
                if(!empty(exif_imagetype($file))) {
                    ob_get_clean();
                    header('Content-type: ' . $ntct[exif_imagetype($file)]);
                    // ob_clean();
                    flush();
                    readfile($file);
                    exit;
                }
            }
        } else { // Remove File  
            // do not remove a saved file until the user clicks on the SAVE button
            if($table == $this->table) {
                $saved_deleted_files = (array) $this->CI->session->userdata('saved_deleted_files');
                $saved_deleted_files[$file_info->id] = $file;
                $this->CI->session->set_userdata('saved_deleted_files', $saved_deleted_files);
                echo json_encode(array('success' => 1));
            } else {
                if($this->CI->db->where('id', $file_info->id)->delete($table)) { // delete from database
                    unlink(realpath($file)); // remove file from server
                    echo json_encode(array('success' => 1));
                    return false;
                } else {
                    echo json_encode(array('success' => 0));
                }
            }
        }
    }
    
    /**
     * Encrypt Filename for security reasons
     * @param string $upload_token - Name of the Token generated during file upload
     * @param boolean $is_tmp - Temporary File not yet saved in database
     * @param integer $action - 0 - Download File, 1 Remove File
     * @param boolean $is_both - Return file with delete button
     * @return string
     */
    public function file_url($upload_token, $extra = array(), $record = array()) {
        $this->CI->load->library('crypt');
        $is_tmp = isset($extra['is_tmp']) ? $extra['is_tmp'] : 0;
        $table_name = $is_tmp == 1 ? $this->table_tmp : $this->table;
        $file_info = !empty($record) ? $record : $this->get_file_info($table_name, $upload_token);
        if(!$file_info) {
            return false;
        }
        $action = isset($extra['action']) ? $extra['action'] : 0;
        $with_size = isset($extra['with_size']) ? true : false;
        $with_thumbnail = isset($extra['with_thumbnail']) ? true : false;
        $template_id = isset($extra['template']) ? $extra['template'] : 1; // view templates
        // $url = $action == 1 ? 'remove' : 'download';
        $url = 'download';
        $encrypted_token = $this->CI->crypt->encode($upload_token);
        $file_url = site_url('ajax/' . $url . '?file=') . $encrypted_token;
        if($is_tmp != 0) {
            $file_url .= '&hash=' . $is_tmp;
        }
        $html_title = '<input type="text" class="dm-file-title form-control" id="' . $file_info->id . '-' . $is_tmp . '" value="' . $file_info->title . '" />';
        $html_priority = '<input type="text" class="dm-file-priority form-control" id="' . $file_info->id . '-' . $is_tmp . '" value="' . $file_info->priority . '" />';
        $html_delete = '';
        if($action == 1) { // File Remove Button
            // $file_url .= '&action=' . $action;
            $is_saved_file = $is_tmp == 0 ? 2 : 1;
            $html_delete = <<<EOL
    <button type="button" value="{$encrypted_token}" class="btn btn-danger btn-file-remove btn-xs mb-1" data-hash="{$is_saved_file}">
        <i class="fas fa-times"></i>
    </button>
EOL;
        } 
        $thumbnail = '';
        if($with_thumbnail) { // Display thumbnail with image
            $thumbnail_title = lang('file_download') . ' ' . $upload_token;
            if(!empty($file_info->fileindex) && !empty($file_info->filename)) {
                $fileindex = !empty($file_info->fileindex) ? $file_info->fileindex . '/' : '';
                if($file_info->fileindex == 'tmp') {
                    $fileindex = '';
                }
                $filepath = FCPATH . $this->upload_dir . $fileindex . $file_info->filename;
                $path_info = pathinfo($filepath, PATHINFO_EXTENSION);
                if(!empty($path_info)) {
                    switch($path_info) {
                        case 'png':
                        case 'jpg':
                        case 'jpeg':
                            $image_url = site_url('ajax/view?file=') . $encrypted_token;
                            $thumbnail_url = site_url('ajax/view?file=') . $encrypted_token . '&thumb=1';
                            if($is_tmp != 0) {
                                $image_url .= '&hash=' . $is_tmp;
                                $thumbnail_url .= '&hash=' . $is_tmp;
                            }
                            $thumbnail = <<<EOL
<a href="{$image_url}" title="{$thumbnail_title}" class="gc-fancybox img-crop-center">
    <img src="{$thumbnail_url}" alt="{$thumbnail_title}" class="img-fluid img-file">
</a>
EOL;
                            break;
                    }
                }
            }
            if(empty($thumbnail)) {
                switch($file_info->file_extension) {
                    case '.pdf':
                        $icon = '<i class="fas fa-file-pdf fa-2x img-file text-danger"></i>';
                        break;
                    case '.doc':
                    case '.docx':
                        $icon = '<i class="fas fa-file-word fa-2x img-file text-primary"></i>';
                        break;
                    case '.xls':
                    case '.xlsx':
                        $icon = '<i class="fas fa-file-excel fa-2x img-file text-success"></i>';
                        break;
                    default:
                        $icon = '<i class="fas fa-download fa-2x img-file"></i>';
                }
            $thumbnail = <<<EOL
<a href="{$file_url}" title="{$thumbnail_title}">
    {$icon}
</a>
EOL;
            }
        }
        $filesize = '<span class="help-block">, ' . $file_info->filesize . lang('file_size_kb') . '</span> ';
        $download_link = anchor($file_url, $file_info->original_filename);        
        $uploaded_by = !empty($file_info->display_name) ? '<span class="mr-2">' . lang('file_uploaded_by') . '</span><span class="badge badge-warning">' . $file_info->display_name . '</span>' : '';
        $uploaded_on = gc_date_format($file_info->created_on);
        switch($template_id) {
            case 1:
                $output = $thumbnail . anchor($file_url, (empty($file_info->title) ? $file_info->original_filename : $file_info->title)) . $filesize . $html_delete;
                break;
            case 2:
                $filesize = $file_info->filesize . ' ' . lang('file_size_kb');
                $thumbnail = !empty($thumbnail) ? '<div class="float-left mr-2">' . $thumbnail . '</div>' : '';
                $output = <<<EOL
    {$thumbnail}
    <h4>
        {$download_link}
        <span class="float-right text-right small h6">{$filesize}</span>
    </h4>
    <p>
        {$uploaded_by}
        <span class="float-right text-right">{$uploaded_on}</span>
    </p>
EOL;
                break;
            case 3:
                $filesize = $file_info->filesize . ' ' . lang('file_size_kb');
                $output = <<<EOL
    <tr>
        <td>
            <h4>{$download_link}</h4>
            <div>{$uploaded_by}</div>
        </td>
        <td>
            <div><span class="text-right small">{$filesize}</span></div>
            <div class><span class="text-right">{$uploaded_on}</span></div>
        </td>
    </tr>
EOL;
                break;
            case 4: // upload/edit
                $file = $thumbnail . anchor($file_url, $file_info->original_filename) . $filesize . $html_delete;
                $output = <<<EOL
    <div class="form-row mb-1">
        <div class="col-5">{$html_title}</div>
        <div class="col-1">{$html_priority}</div>
        <div class="col-6">{$file}</div>
    </div>
EOL;
                break;
        }
        return $output;
    }
    
    /**
     * Retrieve files on validation error and row edit
     * @param string $module
     * @param string $row_id
     * @param string $extra - Extra fields to be used for data retrievel
     */
    public function get_files($module, $row_id = 0, $extra = array()) {
        $files = array();
        // display both temporary and saved files
        $module_id = gc_module_id($module);
        $tmp_files_id = $this->CI->session->userdata('tmp_files_id');
        $template = isset($extra['template']) ? (int) $extra['template'] : 1;
        $with_thumbnail = isset($extra['with_thumbnail']) ? $extra['with_thumbnail'] : false;
        if(!empty($tmp_files_id)) {
            $tmp_files = $this->CI->db->where(array('module_id' => $module_id, 'row_id' => $tmp_files_id))->order_by('priority')->get($this->table_tmp)->result();
            if($tmp_files) {
                foreach($tmp_files as $tmp_file) {
                    $files[$tmp_file->field_id][] = $this->file_url($tmp_file->upload_token, array('is_tmp' => 1, 'action' => 1, 'template' => $template, 'with_thumbnail' => $with_thumbnail), $tmp_file);
                }
            }
        }
        if(!empty($row_id)) {
            $saved_deleted_files = (array) $this->CI->session->userdata('saved_deleted_files');
            $where = !empty($extra['where']) ? $extra['where'] : array('module_id' => $module_id, 'row_id' => $row_id);
            $saved_files = $this->CI->db->where($where)->order_by('priority')->get($this->table)->result();
            if($saved_files) {
                foreach($saved_files as $saved_file) {
                    // if(!array_key_exists($saved_file->id, $saved_deleted_files)) { // do not show deleted files on validation error
                    $files[$saved_file->field_id][] = $this->file_url($saved_file->upload_token, array('action' => 1, 'template' => $template, 'with_thumbnail' => $with_thumbnail), $saved_file);
                    // }
                }
            }
        }
        return $files;
    }
    
    /**
     * Move files to the tmp folder
     * @param string $module - Name of the Module
     * @param string $row_id - Row ID
     * @param string $extra - Extra fields to be saved in database
     */
    public function save_files($module, $row_id, $extra = array()) {
        // do not remove a saved file until the user clicks on the SAVE button
        $saved_deleted_files = (array) $this->CI->session->userdata('saved_deleted_files');
        if(!empty($saved_deleted_files)) {
            foreach($saved_deleted_files as $saved_file_id => $saved_file_name) {
                if($this->CI->db->where('id', $saved_file_id)->delete($this->table)) { // delete from database
                    unlink(realpath($saved_file_name)); // remove the file from server
                }
            }
        }
        // display both temporary and saved files
        $module_id = gc_module_id($module);
        $tmp_files_id = $this->CI->session->userdata('tmp_files_id');
        if(!empty($tmp_files_id)) {
            $tmp_files = $this->CI->db->where(array('module_id' => $module_id, 'row_id' => $tmp_files_id))->get($this->table_tmp)->result();
            $upload_folder = isset($extra['user_id']) ? $this->upload_folder($extra['user_id']) : ''; 
            $upload_dir = $this->upload_dir . (!empty($upload_folder) ? $upload_folder . '/': '');
            if($tmp_files) {
                $field_files = array();
                $tmp_files_data = array();
                foreach($tmp_files as $tmp_file) {
                    $field_id = (int)substr( $tmp_file->field_id, strrpos( $tmp_file->field_id, '-' ) + 1);
                    $field_id = explode('-' . $field_id, $tmp_file->field_id);
                    $field_id = $field_id[0];
                    $filename = $this->move_file($tmp_file->filename, $upload_dir);
                    if(!empty($filename)) {
                        $field_files[$field_id][$tmp_file->upload_token] = array(
                            'token' => $tmp_file->upload_token,
                            /*'fileindex' => $upload_folder,
                            'filename' => $tmp_file->filename,
                            'original_filename' => $tmp_file->original_filename,
                            'filesize' => $tmp_file->filesize*/
                        );
                        // Using tmp_files table
                        $data_array = array(
                            'module_id' => $tmp_file->module_id,
                            'row_id' => $row_id,
                            'field_id' => $tmp_file->field_id,
                            'title' => $tmp_file->title,
                            'upload_token' => $tmp_file->upload_token,
                            'fileindex' => $upload_folder, // Upload folder name
                            'filename' => $filename,
                            'original_filename' => $tmp_file->original_filename,
                            'filesize' => $tmp_file->filesize,
                            'file_extension' => $tmp_file->file_extension,
                            'priority' => $tmp_file->priority,
                            'created_on' => date('Y-m-d H:i:s'),
                            'created_ip' => $this->CI->input->ip_address(),
                        );
                        if(!empty($extra)) { // extra fields to be saved in a table like ticket_id
                            $data_array = array_merge($data_array, $extra);
                        }
                        $tmp_files_data[] = $data_array;
                    }
                }
                if(!empty($tmp_files_data)) {
                    $this->CI->db->on_duplicate($this->table, $tmp_files_data);
                }
            }
        }
        
        // update files information in the main table
        $this->update_references($module, $row_id, $extra);
        
        // unset temporary id
        $this->CI->session->unset_userdata('tmp_files_id');
        $this->CI->session->unset_userdata('saved_deleted_files');
        // name of the table where files to be saved
        $this->CI->session->unset_userdata('gc_files_table');
        // clear cache variable
        clear_cache('gc_files');
    }
    
    /**
     * Purge file by ID
     * @param number $id
     */
    public function purge_by_id($id) {
        $saved_file = $this->CI->db->where('id', $id)->get($this->table)->row();
        if($saved_file) {
            $file = FCPATH . 'assets/files/' . $saved_file->filename;
            if(file_exists($file)) {
                unlink($file);
            }
            $this->CI->db->where('id', $id)->delete($this->table);
            $this->update_references(gc_module_name($saved_file->module_id), $saved_file->row_id);
            return true;
        }
        return false;
    }
    
    /**
     * Update references in the main by file id
     * @param number $id
     */
    public function update_references_by_id($id) {
        // update files information in the main table
        // no need to do it for tmp table, it will be saved automatically on submit button click
        $saved_file = $this->CI->db->where('id', $id)->get($this->table)->row();
        if($saved_file) {
            $this->update_references(gc_module_name($saved_file->module_id), $saved_file->row_id);
        }
    }
    
    /**
     * Update references in the main table to avoid unnecessary table join
     * @param string $module
     * @param number $row_id
     * @param array $extra
     */
    public function update_references($module, $row_id, $extra = array()) {
        // name of the column in the database module tables
        $fileuploader_fields = $this->CI->session->userdata('fileuploader_fields') ? $this->CI->session->userdata('fileuploader_fields') : array('files');
        $module_id = gc_module_id($module);
        $where = !empty($extra['where']) ? $extra['where'] : array('module_id' => $module_id, 'row_id' => $row_id, 'status' => 1);
        $saved_files = $this->CI->db->where($where)->order_by('priority')->get($this->table)->result();
        $field_files = array();
        foreach($fileuploader_fields as $fileuploader_field) {
            $field_files[$fileuploader_field] = array();
        }
        if($saved_files) {
            foreach($saved_files as $saved_file) {                
                $field_id = explode('-', $saved_file->field_id);
                $field_id = $field_id[0];
                $field_files[$field_id][$saved_file->upload_token] = array(
                    'token' => $saved_file->upload_token,
                    'title' => $saved_file->title,
                    'extension' => $saved_file->file_extension,
                    'filesize' => $saved_file->filesize,
                    'icon' => $this->file_icon($saved_file->file_extension)
                );
            }
        }
        if(!empty($field_files)) {
            foreach($field_files as $column_name => $column_data) {
                $this->CI->db->where('id', $row_id)->update($module, array($column_name => json_encode($column_data)));
            }
        }
    }
    
    /**
     * Update Uploaded File Title/Priority
     */
    public function update_file() {
        $file_id = explode('-', $this->CI->input->post('id'));
        $id = $file_id[0];
        $is_tmp = (int) $file_id[1];
        $value = escape($this->CI->input->post('value'));
        $is_title = (int) $this->CI->input->post('is_title');
        $field_name = $is_title == 1 ? 'title' : 'priority';
        if($is_tmp == 1) {
            $table = $this->table_tmp;
        } else {
            $table = $this->table;
            // update files information in the main table
            // no need to do it for tmp table, it will be saved automatically on submit button click
            $this->update_references_by_id($id);
        }
        $this->CI->db->where('id', $id)->update($table, array($field_name => $value));
    }
    
    /**
     * Folder where files to be uploaded
     * @param number $user_id
     * @return string
     */    
    private function upload_folder($user_id) {
        $random_string = strtolower(substr(str_replace('==', '', base64_encode($user_id . '_files')), 0, 8)) . $user_id;
        return $random_string;
    }
    
    /**
     * Move file from the temporary directory to the main files folder
     * @param string $filename - Name of the file
     * @param string $upload_dir - Directory where the file is to be saved
     * @return string
     */
    private function move_file($filename, $upload_dir = 'assets/files') {
        if(!empty($filename) && !empty($upload_dir)) {
            // check if the file with the same names exists in the destination folder
            $upload_path = FCPATH . $upload_dir;
            if(!is_dir($upload_path)) {
                mkdir($upload_path, 0777, true);
            }
            $temp_path = FCPATH . 'assets/tmp/' . $filename;
            if(file_exists($upload_path . '/' . $filename)) {
                // rename file with random string
                $filename = uniqid(). '_' . $filename;
            }
            // move file only if temp file exists and destination is reachable
            $destination = $upload_path . '/' . $filename;
            if (file_exists($temp_path) && ((!file_exists($destination)) || is_writable($destination))) {
                rename($temp_path, $destination);           
                $path_info = pathinfo($destination, PATHINFO_EXTENSION);
                if(!empty($path_info)) {
                    switch($path_info) {
                        case 'png':
                        case 'jpg':
                        case 'jpeg':
                            // generate thumbnails if the uploaded file is an image
                            $this->resize_image($upload_dir, $filename);
                            break;
                    }                    
                }
                // return filename to save in database
                return $filename;
            }
        }
        return false;
    }
    
    /**
     * Get file information from database
     * @param string $table
     * @param string $token
     * @return object
     */
    private function get_file_info($table, $token) {
        $cacheID = 'gc_file_' . $table . '_' . $token;
        $cacheID = str_replace('-', '_', $this->CI->slug->create_slug($cacheID));
        if(!$file_info = $this->file_cache->load($cacheID)) {
            $file_info = $this->CI->db->where('upload_token', $token)->get($table)->row();
            $this->file_cache->save($file_info, $cacheID, array('gc_files'));
        }        
        return $file_info;
    }
    // Display files on validation error and row edit
    // Handle previously saved files and their removal
    // File upload done through field specific, save it in appropriate field
    // Multi File upload logic to be done separately
    
    /**
     * Allow user to upload documents
     * @param string $module
     * @param number $row_id
     */
    public function upload_files($module, $row_id, $extra = array()) {
        $this->CI->auth->restrict();
        $redirect_url = $this->CI->session->userdata($module . '_last_visited_page') ? $this->CI->session->userdata($module . '_last_visited_page') : gc_url_format('members');
        // check whether the user has permission to upload files
        $this->CI->db->from($module . ' M');
        $this->CI->db->join('packages P', 'P.id = M.package_id');
        $row = $this->CI->db->select('M.*, P.module_documents_limit')->where('M.id', $row_id)->get()->row();
        if(!$row || ($row->user_id != $this->CI->auth->user_id() && $this->CI->auth->role_id() != 1)) {
            Template::redirect($redirect_url);
        }
        
        // files limit
        $module_id = gc_module_id($module);
        $files_limit = !empty($row->module_documents_limit) ? json_decode($row->module_documents_limit, true) : array();
        $saved_files_count = $this->CI->db->where(array('row_id' => $row_id, 'module_id' => $module_id))->count_all_results($this->table);
        $files_limit = isset($files_limit[$module]) && $files_limit[$module] === 999 ? 0 : ($files_limit[$module] == 0 ? 10000 : $files_limit[$module]);  
        if($saved_files_count >= $files_limit) {
            // the user cannot upload more files
            // manual variable specified because of the file save message issue with flashdata
            Template::set('file_limit_over', sprintf(lang('file_limit'), $files_limit));
            // Template::redirect($redirect_url); // do not redirect, the user must be able to edit additional files
        }
        // send files limit to the uploader function
        $this->CI->session->set_userdata('files_limit_' . $module . '_' . $row_id, $files_limit);
        
        Assets::add_css('jquery.dm-uploader.min.css');
        Assets::add_js('jquery.dm-uploader.min.js');
        Assets::add_js('jquery.dm-functions.js');
        // $this->session->set_userdata('gc_files_table', 'ticket_attachments');
        $this->tmp_id();
        
        // display saved files to the user
        $files = $this->get_files($module, $row_id, array('template' => 4));
        
        // store data in a session to avoid view file changes
        $this->CI->session->set_flashdata('uploaded_files', $files);
        /**
         * A form can have multiple fileuploader fields for different purposes
         * Each field must be updated in the module's table on file removal
         * Unset the fileuploader_fields session if the POST is empty
         */
        if(empty($_POST)) {
            $this->CI->session->unset_userdata('fileuploader_fields');
        }
        
        // save the newly uploaded file
        if(isset($_POST['save'])) {
            $extra = array();
            $this->save_files($module, $row_id);
            Template::set_message ( sprintf(lang('msg_save_success'), lang('file_documents')), 'success' );
            Template::redirect(current_url());
        }
        Template::set(array(
            'toolbar_title' => lang('file_documents') . ' - ' . $row->title,
            'cancel_url' => $redirect_url,
            'module' => $module,
            'row_id' => $row_id,
            'files_limit' => $files_limit,
            'fileuploader_langs' => true
        ));
        Template::set_view('common/upload_files');
    }
    
    /**
     * Display saved files to the user
     * @param string $saved_files
     * @return string[]
     */
    public function saved_files($saved_files, $extra = array()) {
        $this->CI->load->library('crypt');
        $files = array();
        $saved_files = !empty($saved_files) ? json_decode($saved_files) : array();
        $extra['outer_emmet'] = 'ul.list-unstyled';
        $extra['inner_emmet'] = 'li';
        $html = ''; // default format
        if(!empty($saved_files)) {
            foreach($saved_files as $file) {
                $filesize = '<span class="help-block">, ' . $file->filesize . lang('file_size_kb') . '</span> ';
                $file_url = 'ajax/download?file=' . $this->CI->crypt->encode($file->token);
                $html = '<i class="' . $this->file_icon($file->extension) . ' fa-fw mr-1 fa-2x"></i>' . anchor($file_url, $file->title) . $filesize;
                $html = <<<EOL
<a href="{$file_url}" title="{$file->title}">
    {$html}
</a>
EOL;
                if(isset($extra['inner_emmet'])) {
                    $files[] = emmet($extra['inner_emmet'], $html);
                } else {
                    $files[] = $html;
                }
            }
            if(isset($extra['outer_emmet'])) {
                $html = emmet($extra['outer_emmet'], implode(PHP_EOL, $files));
            }
        }
        return array('files' => $files, 'html' => $html);
    }
    
    /**
     * Get File Icon
     * @param string $file_extension
     * @return string
     */
    public function file_icon($file_extension) {
        $icon = 'fas fa-download';
        switch($file_extension) {
            case '.pdf':
                $icon = 'fas fa-file-pdf text-danger';
                break;
            case '.doc':
            case '.docx':
                $icon = 'fas fa-file-word text-primary';
                break;
            case '.xls':
            case '.xlsx':
                $icon = 'fas fa-file-excel text-success';
                break;
            default:
                $icon = 'fas fa-download';
        }
        return $icon;
    }
    // Display files on validation error and row edit
    // Handle previously saved files and their removal
    // File upload done through field specific, save it in appropriate field
    // Multi File upload logic to be done separately
    
    /**
     * Generate Token value for new files
     * @param string $field_name - Name of the field on the Form
     * @param array $data - uploaded data
     */
    public function uploaded_file_params($field_name, $data) {        
        // A unique token will be assigned to each uploaded file
        // this token will be used to identify files on the server
        $ip_address = $this->CI->input->ip_address();
        $upload_token = substr(md5(time()), 0, 8) . mt_rand(1,1000) . (!empty($ip_address) ? ip2long($ip_address) : '');
        $original_filename = $_FILES[$field_name]['name'];
        $uploaded_filename = $data['upload_data']['file_name'];
        $filesize = $data['upload_data']['file_size'];
        $uploaded_file_params = array(
            'field_id' => $field_name,
            'upload_token' => $upload_token,
            'original_filename' => $original_filename,
            'filesize' => $filesize
        );
        // check this session variable before saving the image and save these values in the database
        $this->CI->session->set_userdata('uploaded_file_params', $uploaded_file_params);
    }
    
    /**
     * Update existing images with token and filesize
     */
    public function update_existing_images() {
        $records = $this->CI->db->where("upload_token IS NULL OR upload_token = ''")->get($this->table)->result();
        if($records) {
            foreach($records as $record) {
                $upload_token = substr(md5(time()), 0, 8) . mt_rand(999,10000);
                $filesize = filesize(FCPATH . $record->url);
                $data = array(
                    'upload_token' => $upload_token,
                    'original_filename' => $record->filename,
                    'field_id' => 'file',
                    'filesize' => round($filesize / 1024, 2)
                );
                $this->CI->db->where('id', $record->id)->update($this->table, $data);
            }
        }
    }
}