<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
 || ##################################################################### ||
 || # GoClixy - Online Software Solutions							   # ||
 || # Copyright (C) 2021 GoClixy. All Rights Reserved.				   # ||
 || # ----------------------------------------------------------------  # ||
 || # This file is owned and copyrighted by GoClixy.com. Your license   # ||
 || # confers  no  title  or ownership in the Script and is not a sale  # ||
 || # of any rights to the Software. This file may not be redistributed # ||
 || # in whole or significant part.									   # ||
 || # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
 || # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
 || ##################################################################### ||
 \*=======================================================================*/
/**
 * IMPORTANT because of use of single table for all module images
 *
 * The module parameter must be passed without any translation in the URL
 * Passing translated string will result in the image upload failure
 * The module_id is retrieved using the passed string as the 3rd segment
 *
 * type_id -> 0 means normal image, 1 means cover image, 2 means logo
 * A module can also have cover images
 *
 * @author eComsta
 *
 */
class Gimages {
    
    // To store instance of codeigniter
    public $CI;
    
    // root table
    protected $table_name   = "images";
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct($table_name = NULL) {
        $this->CI =& get_instance();
        if(!empty($table_name)) {
            $this->table_name = $this->CI->db->dbprefix . $table_name;
        }
    }
    
    /**
     * Upload image using CRUD
     * Image_CRUD 0.6 is used to upload images
     */
    public function upload_photos($module, $row_id) {
        if(!gc_demo_mode(array('no_redirect' => true, 'render' => true))) {
            return false;
        }
        // Tweak images after upload
        if(isset($_POST['tweak_images'])) {
            $this->CI->load->library('goclixy/gimages');
            $this->CI->gimages->_tweak_images_row($module, $row_id);
            Template::set_message(lang('msg_success'), 'success');
            Template::redirect(current_url());
        }
        $user_id = $this->CI->auth->user_id();
        $this->CI->load->library('crypt');
        
        // Only logged in users are having permissions to view this page
        $row_id = intval($row_id);
        $module_id = gc_module_id($module);
        
        // Product variants
        $toggle_module = null;
        $toggle_row_id = 0;
        if($module == 'variants') { // Ecommerce - Product Variant Images
            $this->lang->load('variants/variants');
            $module_redirect_url = gc_url_format('members/variants');
            // get product id and use it as a row id
            $variant_info = $this->CI->db->get_where('product_variants', array('id' => $row_id))->row();
            if(!$variant_info) {
                Template::redirect($module_redirect_url);
            }
            $module = 'products';
            $row_id = $variant_info->product_id;
            $toggle_module = 'variants';
            $toggle_row_id = $row_id;
        } else {
            $module_redirect_url = gc_url_format($module .'/view') . '?id='. $this->crypt->encode($row_id) . '&md=' . $this->crypt->encode($module, true);
        }
        if($row_id == 0 || $module_id == 0) {
            Template::redirect(gc_url_format('members'));
        }
        
        // Get Module Info
        $select = 'user_id, package_id,' . ($module == 'listings' ? 'id AS listing_id' : 'listing_id');
        $module_info = $this->CI->db->select($select)->get_where($module, array('id' => $row_id))->row();
        if(!$module_info) {
            Template::redirect(gc_url_format('members'));
        }
        if($module_info->user_id != $user_id) {
            Template::set_message(lang('msg_permission_expired_error'), 'danger');
            Template::redirect($module_redirect_url);
        }
        
        // Get images limit and status field value
        $images_limit = 100;
        $images_status = 1;
        if($images_limit == 999) {
            $images_limit = -1; // Disable image uploading if disabled via Admin Panel
            Template::set_message(lang('msg_photo_permission_error'), 'danger');
            Template::redirect($module_redirect_url);
        }
        
        // COUNT EXISTING IMAGES
        $images_count = $this->CI->db->where(array('row_id' => $row_id, 'module_id' => $module_id, 'type_id' => 0))->count_all_results($this->table_name);
        $this->image_crud($module, $images_status, $images_limit, $images_count);
        
        Template::set(array(
            'title'=> lang('label_image_uploader'),
            'page_title'=> lang('label_image_uploader'),
            'toolbar_title'=> lang('label_image_uploader'),
        ));
        Template::set_view('common/photos');
        // This method has no View in the Themes Module, i.e. load layout files manually
        $render = $this->gc->demo_custom_css_js('member');
        Template::render($render);
    }
    
    /**
     * Upload Photos Admin
     * @param string $module
     * @param int $row_id
     * @param string $redirect_url
     */
    public function upload_photos_admin($module, $row_id, $redirect_url) {
        if(!gc_demo_mode(array('no_redirect' => true, 'render' => true))) {
            return false;
        }
        // Tweak images after upload
        if(isset($_POST['tweak_images'])) {
            $this->CI->load->library('goclixy/gimages');
            $this->CI->gimages->_tweak_images_row($module, $row_id);
            Template::set_message(lang('msg_success'), 'success');
            Template::redirect(current_url());
        }
        $module_id = gc_module_id($module);
        if((int) $row_id == 0 || $module_id == 0) {
            Template::redirect($redirect_url);
        }
        $this->image_crud($module, 1, 500, 0);
        Template::set(array(
            'module' => $module,
            'row_id' => $row_id,
            'toolbar_title' => lang('upload_photos')
        ));
        Template::set_view('common/photos');
        Template::render();
    }
    
    /**
     * Common Code for uploading images
     * @param string $module
     * @param number $images_status
     * @param number $images_limit
     * @param number $images_count
     */
    private function image_crud($module, $images_status, $images_limit, $images_count) {
        $module_id = gc_module_id($module);
        /* --------------------------------- */
        $this->CI->load->config('image_crud');
        $this->CI->load->library('image_crud');
        $this->CI->load->library('image_moo');
        /* --------------------------------- */
        $image_crud = new image_CRUD();
        $image_crud->set_primary_key_field('id');
        $image_crud->set_url_field('filename');
        $image_crud->set_title_field('title');
        $image_crud->set_active_field('status');
        $image_crud->active_field_value($images_status);
        $image_crud->module_id_field_value($module_id);
        $image_crud->user_limit($images_limit);
        $image_crud->images_count($images_count);
        $image_crud->user_role($this->CI->session->userdata('role_id'));
        $image_crud->set_table($this->table_name)
        ->set_relation_field('row_id')
        ->set_module_id_field('module_id')
        ->set_ordering_field('priority')
        ->set_image_path('assets/images/' . $module); // CHANGE THIS FOR PATH
        $output = $image_crud->render();
        Assets::add_css($output->css_files);
        Assets::add_js($output->js_files);
        Template::set('output', $output->output);
        Template::set('help_image', sprintf(lang('image_help'), settings_item('site.image_file_size'), settings_item('site.image_width'), settings_item('site.image_height')));
    }
    
    /**
     * Get Images
     * @param string $module
     * @param int $row_id
     * @param string $active_only - get Active images only
     * @param number $type_id - 0 images, 1 - cover image, 2 - logo
     * @param $query - counts -> return images count, single -> return single image only
     */
    public function get_images($module, $row_id, $active_only = TRUE, $type_id = 0, $query = NULL) {
        $module_id = gc_module_id($module);
        $where = array();
        $where['module_id'] = $module_id;
        $where['row_id'] = $row_id;
        $where['type_id'] = $type_id;
        if($active_only == TRUE) {
            $where['status'] = 1;
        }
        switch($query) {
            case 'single':
                $result = $this->CI->db->where($where)->order_by('priority', 'DESC')->get($this->table_name)->row();
                break;
            case 'counts':
                $result = $this->CI->db->where($where)->count_all_results($this->table_name);
                break;
            default:
                $result = $this->CI->db->where($where)->order_by('priority', 'DESC')->get($this->table_name)->result();
        }
        return $result;
    }
    
    /**
     * Save Image - Listings Cover and other Module Image
     * @param string $module
     * @param number $row_id
     * @param string $image
     * @param number $status
     * @param number $type_id - 0 images, 1 - cover image, 2 - logo
     * @param number $limit - Overwrite image if limit is 1
     * $limit variable is added for Mobile App
     */
    public function save_image($module, $row_id, $image, $status = 1, $type_id = 0, $limit = 1) {
        // get the upload path
        $path = $this->get_upload_dir($module, true, $type_id);
        // move the image file from the temp folder and resize it
        $filename = $this->move_resize($image, $path);
        if(is_bool($filename) && !$filename) { // unable to move file, return false
            return false;
        }
        $url = $path . '/' . $filename;
        return $this->save_image_db($module, $row_id, $type_id, $path, $url, $filename, $status, $limit, FALSE, $image);
    }
    
    /**
     * Save image in the database
     *
     * Also used in Bulk Import
     *
     * @param string $module
     * @param integer $row_id
     * @param integer $type_id
     * @param string $path
     * @param string $url
     * @param string $filename
     * @param number $status
     * @param string $bulk_import
     * @return integer
     */
    public function save_image_db($module, $row_id, $type_id, $path, $url, $filename, $status = 1, $limit = 1, $bulk_import = FALSE, $original_filename = null) {
        $module_id = gc_module_id($module);
        $where = array('module_id' => $module_id, 'row_id' => $row_id, 'type_id' => $type_id);
        // Bulk save multiple images, but do not save the same image with the same url
        if($bulk_import) {
            $where['url'] = $url;
        }
        
        // Save Logo and Cover image in the Modules Table for Featured, Popular and other Search Queries
        if($type_id != 0) {
            $key = ($type_id == 1 ? 'cover' : 'logo');
            $this->CI->db->where(array('id' => $row_id))->update($module, array($key => $url));
        }
        
        $data = array('module_id' => $module_id, 'row_id' => $row_id, 'type_id' => $type_id, 'url' => $url, 'path' => $path, 'filename' => $filename, 'original_filename' => $original_filename, 'status' => $status);
        $insert = FALSE; // Allow user to insert image
        if($limit == 1) {
            $result = $this->CI->db->select('filename, path')->where($where)->order_by('id', 'DESC')->get($this->table_name)->row();
            if($result) { // Image already exists, update it
                if($result->filename != $filename) { // both images are not same, upload the new image
                    // delete previous image
                    $this->purge_image($result->filename, $result->path);
                    // update new image
                    $this->CI->db->where($where)->update($this->table_name, array('url' => $url, 'path' => $path, 'filename' => $filename, 'status' => $status));
                }
                return $row_id;
            } else {
                $insert = TRUE;
            }
        } else {
            $insert = TRUE;
        }
        if($insert) {
            $this->CI->db->insert($this->table_name, $data);
            $insert_id = $this->CI->db->insert_id();
            /*if($type_id == 0 && !$bulk_import) {
             // Earn Points
             $this->gc->earn_points('images', 0, $insert_id);
             }*/
            return $insert_id;
        }
    }
    
    /**
     * Common upload image function for all modules
     *
     * Always upload file in the temp directory and then move it to the main directory
     * Validation errors in other fields resulted in the image copy to the main folder
     *
     * @param string $field_name
     * @param string $type - image/logo/cover
     * @param string $settings - pass config parameters in an array format
     * @return multitype:NULL
     */
    public function upload_image($field_name, $type = 'image', $settings = array()) {
        if(!gc_demo_mode(array('no_message' => true, 'no_redirect' => true))) {
            return false;
        }
        $config['upload_path'] = realpath(FCPATH .'/assets/images/temp/'); //Make SURE that you chmod this directory to 777!
        $config['allowed_types'] = 'gif|jpg|jpeg|png';
        if(!empty($settings) && !empty($settings['allowed_types'])) {
            $config['allowed_types'] = $settings['allowed_types'];
        }
        switch($type) {
            case 'image':
                $max_size = settings_item('site.image_file_size');
                $max_width = settings_item('site.image_width');
                $max_height = settings_item('site.image_height');
                break;
            case 'cover':
                $max_size = settings_item('site.cover_file_size');
                $max_width = settings_item('site.cover_width');
                $max_height = settings_item('site.cover_height');
                break;
            default:
                if(!empty($settings)) {
                    $max_size = $settings['max_size'];
                    $max_width = $settings['max_width'];
                    $max_height = $settings['max_height'];
                } else {
                    return false;
                }
        }
        $config['max_size']= $max_size;
        $config['max_width']= $max_width;
        $config['max_height']= $max_height;
        $config['remove_spaces'] = TRUE; //Remove spaces from the file name
        $config['encrypt_name'] = TRUE; // convert the file name to a random encrypted string.
        $this->CI->load->library('upload', $config);
        
        if (!$this->CI->upload->do_upload($field_name)) { // Image name is necessary
            $this->CI->session->set_flashdata('upload_image_error', $this->CI->upload->display_errors());
            return false;
        } else {
            $data = array('upload_data' => $this->CI->upload->data());
        }
        return $data;
    }
    
    /**
     * Create thumbnails of uploaded image
     * @param string $path
     * @param string $upload_path
     * @param string $file
     */
    public function resize_image($upload_path, $file) {
        // If thumbs folder does not exist, create it
        $thumbs_path = FCPATH . '/' . $upload_path . '/thumbs';
        if(!is_dir($thumbs_path)) {
            mkdir($thumbs_path, 0777, true);
        }
        $config['image_library']='gd2';
        // $config['source_image']=realpath(FCPATH.'/assets/images/'.$upload_path.'/' .$file);
        $config['source_image']=realpath(FCPATH . '/' . $upload_path . '/' . $file);
        $config['create_thumb']=TRUE;
        $config['maintain_ratio']=TRUE;
        $config['overwrite'] = TRUE; // overwrite existing images
        $config['thumb_marker'] = FALSE; // don't want to add the suffix _thumb in thumbnail file name
        $config['width']=settings_item('site.thumbnail_width');
        $config['height']=settings_item('site.thumbnail_height');
        $config['new_image']='./' . $upload_path .'/thumbs/thumb__' . $file;
        /*
         * ADD THUMB BEFORE THE NAME
         * $config['create_thumb'] = false;
         * $config['new_image'] = 'thumb_'.$data['file_name'];
         */
        $this->CI->load->library('image_lib', $config);
        /**
         * $this->image_lib->initialize($config); and $this->image_lib->clear();
         * are important else, only one thumbnail will be saved
         */
        $this->CI->image_lib->initialize($config);
        $this->CI->image_lib->resize();
        $this->CI->image_lib->clear();
    }
    
    /**
     * Get module image e.g. categories/users/banners etc.
     * @param string $module
     * @param integer $row_id
     */
    public function get_module_image($filename, $module, $row_id = 0, $field = 'image') {
        // delete previous image first
        $path = 'assets/images/' . $module;
        if($row_id != 0) {
            $result = $this->CI->db->select($field)->get_where($module, array('id' => $row_id))->row();
            if($result) {
                if($this->is_new_image($module, $result->$field, $filename)) {
                    $this->purge_image($result->$field, $path);
                }
            }
        }
        return $this->move_resize($filename, $path);
    }
    
    /**
     * Check whether the uploaded image is an old or new image
     * Module Specific - categories/users/banners - Update row without changing image
     * @return boolean
     */
    public function is_new_image ($module, $old_image, $new_image) {
        // get file size of old image
        $old_img_filesize = filesize(realpath(FCPATH.'/assets/images/' . $module .'/' . $old_image));
        
        // convert size in KB and show the filesize upto 2 decimal places
        $old_img_filesize = round($old_img_filesize/1024, 2);
        
        // get file size of new image
        $new_img_filesize = filesize(realpath(FCPATH.'/assets/images/' . $module .'/' . $new_image));
        
        // convert size in KB and show the filesize upto 2 decimal places
        $new_img_filesize = round($new_img_filesize/1024, 2);
        
        // delete old image only if the file size is different
        if($new_img_filesize != $old_img_filesize) {
            return true;
        }
        return false;
    }
    
    /**
     * On module row purge, also delete the image from the specific folder
     * Module Specific - categories/users/banners
     * @param string $module
     * @param number $row_id
     * @param string $field
     */
    public function purge_module_image($module, $row_id = 0, $field = 'image') {
        $path = 'assets/images/' . $module;
        if($row_id != 0) {
            $result = $this->CI->db->select($field)->get_where($module, array('id' => $row_id))->row();
            if($result) {
                $this->purge_image($result->$field, $path);
            }
        }
    }
    
    /**
     * Move and Resize file before save
     * @param string $filename - Name of the file
     * @param string $upload_dir - Directory where the file is to be saved
     * @param boolean $is_module - e.g. categories/users/banners etc.
     * @param boolean $thumb - generate thumb if the thumb value is true
     * @return string
     */
    public function move_resize ($filename, $upload_dir, $thumb = true) {
        if(!empty($filename) && !empty($upload_dir)) {
            // check if the file with the same names exists in the destination folder
            $upload_path = FCPATH . $upload_dir;
            if(!is_dir($upload_path)) {
                mkdir($upload_path, 0777, true);
            }
            $temp_path = FCPATH . 'assets/images/temp/' . $filename;
            if(file_exists($upload_path . '/' . $filename)) {
                // rename file with random string
                $filename = uniqid(). '_' . $filename;
            }
            // move file only if temp file exists and destination is reachable
            $destination = $upload_path . '/' . $filename;
            if (file_exists($temp_path) && ((!file_exists($destination)) || is_writable($destination))) {
                rename($temp_path, $destination);
                
                // iPHONE ISSUE - read_exif_data and Adjust Image Orientation
                // get images rotated automatically via PHP when a user upload them
                $this->CI->load->library('image_autorotate', array('filepath' => $destination));
                
                // create thumbnail
                if($thumb) {
                    $this->resize_image($upload_dir, $filename);
                }
                
                // return filename to save in database
                return $filename;
            }
        }
        return false;
    }
    
    /**
     * Subdirectories under the uploads directory is the best to deal with large number of images
     * get the name of the last folder this module image is saved
     * @param string $module
     * @return boolean $path - return the path of the last uploaded directory
     * @param string $type - logo/cover
     */
    public function get_upload_dir($module, $path = false, $type_id = 0) {
        $module_id = gc_module_id($module);
        $module_path = $module;
        $where = array('module_id' => $module_id);
        $type = '';
        if($type_id != 0) {
            if($type_id == 2) {
                $type = 'logos';
            } else { // cover image
                $type = 'covers';
            }
            $module_path .= '/' . $type;
            $where['type_id'] = $type_id;
        }
        
        if($path) {
            $path = $this->image_upload_dir($module_path, $type_id);
            // image type is a logo or cover, but the path does not have any one of them
            // add TYPE to path
            if (!empty($type) && strpos($path, $type) === false) {
                $path .= '/' . $type;
                // check the folder existense
                // if not exists, create them
                $new_path = FCPATH . $path . '/';
                if(!is_dir($new_path)) {
                    mkdir($new_path, 0777, true);
                    mkdir($new_path .'/thumbs', 0777, true);
                }
            }
            return $path;
        } else {
            return 'assets/images/' . $module_path;
        }
    }
    
    /**
     * Purge specific row images
     * @param string $module
     * @param int $row_id
     * @param int $type_id - 99 Purge All Images of this module
     */
    public function purge_images($module, $row_id, $type_id = 0) {
        $module_id = gc_module_id($module);
        // Also Remove the Logo and Cover image from the Modules Table
        if($type_id != 0 && $type_id != 99) {
            $key = ($type_id == 1 ? 'cover' : 'logo');
            $this->CI->db->where(array('id' => $row_id))->update($module, array($key => null));
        }
        $where = array('module_id' => $module_id, 'row_id' => $row_id);
        if($type_id != 99) {
            $where['type_id'] = $type_id;
        }
        $images = $this->CI->db->select('id, path, filename')->get_where('images', $where)->result();
        if(!empty($images)) {
            foreach($images as $image) {
                $this->purge_image($image->filename, $image->path);
                $this->CI->db->where($where)->delete('images');
            }
        }
        
        // set auto increment value to the last id
        $alter_autoinc = "ALTER TABLE ". $this->CI->db->dbprefix . "images AUTO_INCREMENT = 1";
        $this->CI->db->simple_query($alter_autoinc);
    }
    
    /**
     * Truncate Table Images
     */
    public function truncate_table_images() {
        $modules = gc_modules();
        foreach($modules as $module) {
            $this->unlink_images($module);
        }
        $this->CI->db->truncate('images');
    }
    
    /**
     * Remove images from system on Table Truncate
     * @param string $module
     * @param string $table
     */
    public function unlink_images($module, $table = NULL) {
        $module_id = gc_module_id($module);
        $module_images = $this->CI->db->select('path, filename')->where(array('module_id' => $module_id))->get('images')->result();
        if(!empty($module_images)) {
            foreach($module_images as $module_image) {
                $this->purge_image($module_image->filename, $module_image->path);
            }
        }
        if(!empty($table)) {
            $this->CI->db->truncate($table);
        }
    }
    
    /**
     * Mobile App and Gallery Image Delete - Purge image by ID
     * @param integer $id
     * @return boolean
     */
    public function purge_image_by_id($id) {
        $result = $this->CI->db->where('id', $id)->get($this->table_name)->row();
        if($result) {
            // Remove Logo and Cover Image Tweak field from the specified module
            if($result->type_id == 1 || $result->type_id == 2) {
                $module = gc_module_name($result->module_id);
                $this->CI->db->where(array('id' => $result->row_id))->update($module, array(($result->type_id == 1 ? 'cover' : 'logo') => null));
            }
            // Delete image from the server
            $this->purge_image($result->filename, $result->path);
            // Delete image from the database
            return $this->CI->db->where('id', $id)->delete($this->table_name);
        }
        return false;
    }
    
    /**
     * Purge Image
     * @param string $filename
     * @param string $path
     */
    private function purge_image($filename, $path) {
        if(!empty($filename) && file_exists(realpath(FCPATH . $path . '/thumbs/thumb__' . $filename))) {
            unlink(realpath(FCPATH . $path . '/thumbs/thumb__' . $filename));
        }
        if(!empty($filename) && file_exists(realpath(FCPATH . $path . '/' . $filename))) {
            unlink(realpath(FCPATH . $path . '/' . $filename));
        }
    }
    
    /**
     * View Page - Return Image Type
     * @param number $type_id
     */
    public function image_type ($type_id) {
        switch($type_id) {
            case 1:
                $str = lang('label_cover_image');
                break;
            case 2:
                $str = lang('label_logo');
                break;
            default:
                $str = lang('label_image');
        }
        return $str;
    }
    
    /**
     * Generate Thumbnails
     * @param string $module
     */
    public function thumbnail_generate($module) {
        /**
         * Sub-folders contained within the directory will be mapped as well.
         * If you wish to control the recursion depth, you can do so using the second parameter (integer).
         * A depth of 1 will only map the top level directory
         * By default, hidden files will not be included in the returned array.
         * To override this behavior, you may set a third parameter to true (boolean):
         */
        /*$files = directory_map('./assets/images/' . $module); // OLD METHOD
         asort($files);*/
        $path = 'assets/images/' . $module;
        //$objects = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($path), RecursiveIteratorIterator::SELF_FIRST);
        $iterator = new RecursiveDirectoryIterator($path);
        // skip hidden files starting with dots e.g. /. or /..
        $iterator->setFlags(RecursiveDirectoryIterator::SKIP_DOTS);
        $objects = new RecursiveIteratorIterator($iterator, RecursiveIteratorIterator::SELF_FIRST);
        foreach($objects as $image => $object) {
            $filename = $object->getFilename();
            // get only image files based on their extensions
            if(strpos($filename, 'thumb__') === false) { // process only main images
                // Get file extension info
                $info = pathinfo($filename);
                if(isset($info['extension'])) {
                    switch(strtolower($info['extension'])) {
                        case "jpg":
                        case "jpeg":
                        case "png":
                        case "gif":
                            // Get the image path
                            $image_path = str_replace('\\', '/', $object->getPathname());
                            
                            // explode the image path to an array
                            $path_array = array_map('trim', explode('/', $image_path));
                            
                            // remove the filename from the array and store it in a variable
                            array_pop($path_array);
                            
                            // thumb path
                            $thumb_path = implode('/', $path_array);
                            
                            // generate thumbnail
                            // overwrite existing image if exists
                            $this->resize_image($thumb_path, $filename);
                            break;
                    }
                }
            }
        }
    }
    
    /**
     * Get image upload directory
     *
     * Subdirectories under the uploads directory is the best to deal with large number of images
     * Create new folder for images dynamically, when one directory gets 1000 images
     *
     * https://stackoverflow.com/questions/26606632/php-create-new-folder-dynamically-after-thousand-images-in-folder
     *
     * @param string $folder - image folder root location
     * e.g. listings, listings/logos, listings/covers etc.
     * @param number $type_id - Include Logo and Covers path if the $type value is > 0
     * @param string $thumb - create thumb if true
     * @return string
     */
    public function image_upload_dir($folder, $type_id = 0, $thumb = true) {
        $module_path = 'assets/images/' . $folder;
        $last_upload_dir = $module_path;
        $path = $last_upload_dir;
        // get the last saved path
        $image_upload_path = $this->read_file($folder, $type_id);
        if(!empty($image_upload_path)) {
            $last_upload_dir = $image_upload_path;//$image_upload_path->path;
        }
        $dir = FCPATH . $last_upload_dir .'/';
        if(is_dir($dir)) {
            $images = glob("$dir{*.gif,*.jpg,*.jpeg,*.JPG,*.png,*.PNG}", GLOB_BRACE); //you can add .gif or other extension as well
            if(count($images) > 1000) { // update the count to increase the folder limit
                // create new folder using month and year
                $foldername = strtolower(date('d-M-Y')) . '-' . rand(1, 1000);
                $image_path = $module_path . '/' . $foldername;
                $new_path = FCPATH . $image_path . '/';
                if(!is_dir($new_path)) {
                    mkdir($new_path, 0777, true);
                    if($thumb) { // not required in case of qrcodes
                        mkdir($new_path .'/thumbs', 0777, true);
                    }
                }
                $return = $image_path;
            } else {
                $return = $last_upload_dir;
            }
        } else {
            $return = $module_path;
        }
        $this->write_file($folder, $type_id, $return);
        return $return;
    }
    
    /**
     * Save last image upload path in the images_path.txt file
     * This file will reside in the assets/images folder
     *
     * @param string $folder
     * @param string $path
     * @return boolean
     */
    private function write_file($folder, $type_id, $path) {
        $result = $this->read_file($folder, $type_id, $path);
        if(empty($result['content']) || $result['line_found'] == 0) {
            $result['content'][] = $folder . '=' . $path;
        }
        $content = serialize($result['content']);
        // Write the changes out...
        if (! function_exists('write_file')) {
            $this->CI->load->helper('file');
        }
        
        // save file
        if (write_file(FCPATH . 'assets/images/images_path.txt', $content)) {
            return true;
        }
        return false;
    }
    
    /**
     * Read images_path.txt file to get the last saved path
     * if the $path variable is passed, it means it is requested by the write_file method
     *
     * @param string $folder
     * @param number $type_id - Exclude Logo and Covers path if the $type_id value is 0
     * Listings Module - Saving logo first and then uploading type_id 0 image was saving image in the logos folder.
     * @param string $path
     * @return string|multitype:number mixed
     */
    private function read_file($folder, $type_id = 0, $path = NULL) {
        $file = FCPATH . 'assets/images/images_path.txt';
        $image_path = '';
        $lines = array();
        $line_found = 0;
        if(file_exists($file)) {
            $file_content = file($file);
            $lines = unserialize($file_content[0]);
            foreach ($lines as $lineNumber => $line) {
                if (strpos($line, $folder) !== false) {
                    if(empty($path) || $path == NULL) {
                        $path = explode('=', $line);
                        return $path[1];
                    } else {
                        // prepare content
                        $lines[$lineNumber] = $folder . '=' . $path;
                        $line_found = 1;
                    }
                }
            }
        }
        if(empty($path)) {
            return $image_path;
        } else {
            return array('content' => $lines, 'line_found' => $line_found);
        }
    }
    
    /**
     * Remove all empty folders or subfolders recursively
     *
     * https://stackoverflow.com/questions/1833518/remove-empty-subfolders-with-php
     * @param string $path - it must be a path not the folder or module name e.g. 'assets/images/qrcodes'
     */
    public function remove_empty_dir($path) {
        if(substr($path,-1)!= DIRECTORY_SEPARATOR){
            $path .= DIRECTORY_SEPARATOR;
        }
        $d2 = array('.','..');
        $dirs = array_diff(glob($path.'*', GLOB_ONLYDIR),$d2);
        foreach($dirs as $d){
            $this->remove_empty_dir($d);
        }
        
        if(count(array_diff(glob($path.'*'),$d2))===0){
            $checkEmpSubDir = explode(DIRECTORY_SEPARATOR,$path);
            for($i=count($checkEmpSubDir)-1;$i>0;$i--){
                $path = substr(str_replace($checkEmpSubDir[$i],"",$path),0,-1);
                
                if(($files = @scandir($path)) && count($files) <= 2){
                    rmdir($path);
                }
            }
        }
    }
    
    /**
     * eComsta Version Upgrade - Insert Images information in Tweak Columns
     */
    public function _upgrade_images() {
        $config_modules = gc_modules();
        foreach($config_modules as $config_module) {
            // Get images
            $module_id = gc_module_id($config_module);
            // Get Cover Images All Modules
            $covers = $this->CI->db->where(array('module_id' => $module_id, 'type_id' => 1))->get($this->table_name)->result();
            if($covers) {
                foreach($covers as $cover) {
                    $this->CI->db->where('id', $cover->row_id)->update($config_module, array('cover' => $cover->url));
                }
            }
        }
    }
    
    /**
     * Add the Tweak information for the individual Module Row
     * @param string $module
     * @param integer $row_id
     * @param boolean $is_update - TRUE - Update instantly, FALSE - Return value for Bulk Tweaking of Multiple Rows
     */
    public function _tweak_images_row($module, $row_id, $is_update = true) {
        // Get images
        $module_id = gc_module_id($module);
        $data = array();
        // Get Cover Image of the Module
        // 'type_id' => 1,
        $images = array();
        $gallery_images = $this->CI->db->where(array('module_id' => $module_id, 'row_id' => $row_id, 'deleted' => 0))->get($this->table_name)->result();
        if($gallery_images) {
            foreach($gallery_images as $image) {
                 $image = array(
                    'url' => $image->url,
                    'title' => $image->title,
                    'type_id' => $image->type_id,
                    'status' => $image->status
                );
                $images[] = (object) $image;
            }
            if(!empty($images)) {
                $images = object_sort($images, 'type_id', 'desc'); // the cover image will always be the first image
                $data['images'] = json_encode($images);
            }
        } else {
            $this->CI->db->where(array('id' => $row_id))->update($module, array('images' => null));
        }
        if(!empty($data)) {
            if($is_update) {
                $this->CI->db->where(array('id' => $row_id))->update($module, $data);
            } else {
                return $data;
            }
        }
    }
    
    /**
     * Modules - Assign Cover from the existing Images
     */
    public function assign_cover($tweak_existing = false) {
        $modules = gc_modules_only();
        foreach($modules as $module) {
            $module_id = gc_module_id($module);
            // Get all rows
            $result = $this->CI->db->get($module)->result();
            if($result) {
                foreach($result as $row) {
                    $count = $this->CI->db->where(array('module_id' => $module_id, 'row_id' => $row->id, 'type_id' => 1))->count_all_results($this->table_name);
                    if($count <= 0) {
                        $cover = $this->CI->db->where(array('module_id' => $module_id, 'row_id' => $row->id, 'type_id' => 0))->order_by('id')->get($this->table_name)->row();
                        if($cover) {
                            $this->CI->db->where(array('id' => $cover->id))->update($this->table_name, array('type_id' => 1));
                        }
                    } else if($tweak_existing) {
                        $this->_tweak_images_row($module, $row->id);
                    }
                }
            }
        }
    }
    
    public function images_update_old($record, $module = 'products') {
        if(!$record) {
            return false;
        }
        if(empty($record->cover)) {
            // delete cover from the images table
            $this->purge_cover($record, $module);
        }
    }
    
    /**
     * Products - Update images table on cover update
     * Append/Remove cover image from the images column
     * @param object $record
     */
    public function images_update($record, $module = 'products') {
        if($record) {
            $image = array(
                'url' => $record->cover, // !empty($record->cover) ? 'assets/images/' . $record->cover : '',
                'title' => $record->name,
                'type_id' => 1, // the image type is a cover
                'status' => 1
            );
            $images = (array) json_decode($record->images, true);
            $cover_updated = false;
            if(empty($images)) {
                $images[] = $image;
                $cover_updated = true;
            } else {
                // add cover to the top of the images array
                // also save cover in the images table, if not update
                if($images[0] != $image) {
                    $images = array_merge(array($image), $images);
                    $cover_updated = true;
                }
            }
            if($cover_updated) {
                $where = array('row_id' => $record->id, 'module_id' => gc_module_id($module), 'type_id' => 1);
                $db_image = $this->_get_image($where);
                $this->_purge_cover($record, $module, $db_image); // purge cover image
                if(!empty($record->cover)) {
                    $images_folder = 'assets/images/';
                    $cover_folder = 'products/covers/';
                    $folder_path = FCPATH . $images_folder;
                    $destination_path = $folder_path . $cover_folder;
                    // check whether the image URL is live
                    $regexp =  "/((?:https?\:\/\/|www\.)(?:[-a-z0-9]+\.)*[-a-z0-9]+.*)/i";
                    
                    // check whether the image is from a live server or not
                    $live_url = false;
                    $image_path = $image['url'];
                    if(preg_match($regexp, $image_path) || substr($image_path, 0, 8) == "https://") {
                        // url does not start with http or https protocol
                        $live_url = true;
                    }
                    // get filename from the path
                    $filename = $image_path;
                    if(strpos($image_path, '/') !== false) {
                        $explode_image = explode("/", $image_path);
                        $filename = end($explode_image);
                    }
                    // $filename = strpos($image_path, '/') !== false ? substr( $image_path , strrpos( $image_path , '/' ) + 1) : $image_path; // the image is on the root folder
                    // prepare image data
                    $data = array(
                        'url' => $live_url ? $image_path : $images_folder . $cover_folder . $filename,
                        'path' => $images_folder . $cover_folder,
                        'filename' => $filename,
                        'original_filename' => $filename,
                        'status' => 1,
                        'deleted' => 0
                    );
                    if($db_image) {
                        // update the image
                        $this->CI->db->where($where)->update($this->table_name, $data);
                    } else {
                        $data = array_merge($where, $data);
                        $this->CI->db->insert($this->table_name, $data);
                    }
                    if(!$live_url) {
                        // copy and update image
                        $source_path = $folder_path . $image['url'];
                        if(file_exists($source_path)) {
                            $destination_file = $destination_path . $filename;
                            if(!file_exists($destination_file)) { // do not overwrite file if already exists
                                copy($source_path, $destination_file); // copy image
                                $this->resize_image($images_folder . $cover_folder, $filename); // resize image
                            }
                        }
                    }
                }
            }
            // Tweak Images Row
            $this->_tweak_images_row($module, $record->id);
        }
    }
    
    /**
     * Purge Cover Image
     * @param array $record
     * @param string $module
     * @return null
     */
    private function _purge_cover($record, $module, $db_image) {        
        // update images in the module folder - Update using the _tweak_images_row function
        // $this->CI->db->where('id', $record->id)->update($module, array('images' => json_encode($images)));
        // cover deleted
        if(empty($record->cover) && $db_image) {
            // Delete image from the server
            $this->purge_image($db_image->filename, $db_image->path);
            // Delete image from the database
            return $this->CI->db->where('id', $db_image->id)->delete($this->table_name);
        }
    }
    
    /**
     * Get saved image from database
     * @param array $where
     * @return array
     */
    private function _get_image($where) {        
        // check for the cover in the images table
        return $this->CI->db->where($where)->get($this->table_name)->row();
    }
    
    /**
     * Tweak all images
     * @param string $module
     */
    public function tweak_all($module = null) {
        if(!empty($module)) {
            $module_id = gc_module_id($module);
            $this->CI->db->where('module_id', $module_id);
            $this->CI->db->group_by('row_id');
        } else {
            $this->CI->db->group_by('module_id, row_id');
        }
        $records = $this->CI->db->get($this->table_name)->result();
        if(!empty($records)) {
            foreach($records as $record) {
                $this->_tweak_images_row(gc_module_name($record->module_id), $record->row_id);
            }
        }
        return true;
    }
    
    /**
     * Place all images and qrcodes on media server
     * @param string $limit -> The number of records to be processed
     * @param string $table -> listings/images
     * @param string $field -> qrfile/url
     */
    public function sync_media($limit = 1000, $table = 'listings', $field = 'qrfile') {
        $records = $this->CI->db->where($field . " LIKE '%assets/images%'")->limit($limit)->get($table)->result();
        if($records) {
            $media_url = settings_item('site.media_server');
            foreach($records as $record) {
                if(!empty($record->{$field})) {
                    $image_path = $record->{$field};
                    $regexp =  "/((?:https?\:\/\/|www\.)(?:[-a-z0-9]+\.)*[-a-z0-9]+.*)/i";
                    if(!preg_match($regexp, $image_path) && substr($image_path, 0, 8) != "https://") {
                        $post_data = array(
                            'image_url' => gc_image_url($record->{$field}, false)
                        );
                        $output = $this->curl_requests($media_url . 'uploader.php?action=download', $post_data);
                        if(!empty($output['result'])) {
                            $live_url = $media_url . '/image/' . $output['path'] . '/' . $output['name'];
                            $this->CI->db->where('id', $record->id)->update($table, array($field => $live_url));
                            if($table == 'images') {
                                $this->_tweak_images_row(gc_module_name($record->module_id), $record->row_id);
                            }
                        }
                    }
                }
            }
        }
    }
    
    /**
     * Add/Remove file to/from the media server
     * @param string $image_url
     * @param string $action
     * @return string
     */
    public function upload_media($image_url, $action = 'remove') {
        $media_url = settings_item('site.media_server');
        if(strpos($image_url, $media_url) != false && $action == 'remove') {
            $output = $this->remove_media($image_url);
            return $output;
        } else if($action != 'remove') { // upload file on the server
            $post_data = array(
                'image_url' => $image_url
            );
            $output = $this->curl_requests($media_url . 'uploader.php?action=download', $post_data);
            if(!empty($output['result'])) {
                $live_url = $media_url . 'image/' . $output['path'] . $output['name'];
                $output['live_url'] = $live_url;
                return $output;
            }
        }
        return false;
    }
    
    /**
     * Remove media from the server
     * @param string $image_url
     * @param string $media_url
     * @return mixed
     */
    public function remove_media($image_url) {
        $post_data = array(
            'image_url' => $image_url
        );
        $output = $this->curl_requests(settings_item('site.media_server') . 'uploader.php?action=remove', $post_data);
        return $output;
    }
    
    /**
     * Common function for curl operation
     */
    private function curl_requests($url, $postfields = array()) {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        if(!empty($postfields)) {
            curl_setopt($ch, CURLOPT_POST, true);
            curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($postfields));
        }
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            'Access-token: 73f229730cabdbf5120b96655a80f88d' // send security token
        ));
        $result = curl_exec($ch);
        curl_close($ch);
        if($result && is_string($result)) {
            $result = json_decode($result, true);
        }
        return $result;
    }	
}