<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');
/**
 * Usage:
 * // Display ratings to the User
 * $this->load->library('gratings');
 * $gratings = new Gratings('ratings');
 *
 * Add two columns ratings and rating_users in the modules table
 * e.g. ALTER TABLE `rs0d_products` ADD `ratings` FLOAT NOT NULL DEFAULT '0' AFTER `keywords`, ADD `rating_users` INT(11) NOT NULL DEFAULT '0' AFTER `ratings`;
 */
class Gratings {
    
    // To store instance of codeigniter
    public $CI;
    
    // root table
    protected $table_name       = 'ratings';
    protected $table_options	= 'rating_options';
    protected $table_relation	= 'rating_relations';
    protected $table_users	    = 'rating_users';
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct($table_name = 'ratings') {
        $this->CI =& get_instance();
        $this->table_name = $this->CI->db->dbprefix . $table_name;
    }
    
    
    /**
     * Display Rating history to the user
     * @param number $count
     * @param string $where
     * @param string $sort_by
     * @param number $offset
     * @param number $limit
     * @return number
     */
    public function get_history($count = 0, $rating_id = 0, $where = null, $sort_by = 'id DESC', $offset = 0, $limit = 25) {
        if($count == 0) {
            $sql = "SELECT RU.*, U.display_name, U.email";
        } else {
            $sql = "SELECT COUNT(RU.id) AS TotalCount";
        }
        $sql .= " FROM " . $this->CI->db->dbprefix . "rating_users RU";
        $sql .= " LEFT JOIN " . $this->CI->db->dbprefix . "users U ON U.id = RU.user_id";
        $sql .= " WHERE rating_id = " . $rating_id;
        if(!empty($where)) {
            $sql .= " AND $where";
        }
        
        if($count == 0) {
            $sql .= " ORDER BY " . $sort_by;
            $sql .= " LIMIT " . $offset . ", " . $limit;
            return $this->CI->db->query ( $sql )->result();
        } else {
            $result = $this->CI->db->query ( $sql )->row();
            return $result ? $result->TotalCount : 0;
        }
    }
    
    /**
     * Count ratings based on rating value.
     * @param number $module_id
     * @param number $row_id
     */
    public function ratings_count($module_id, $row_id) {
        // Logic
        // Available Ratings = Total Ratings - Higher Rating (5)
        // Rating * 100 / Available Ratings
        $total_ratings = $this->CI->db->where(array('module_id' => $module_id, 'row_id' => $row_id))->count_all_results($this->table_users);
        $rating_users = 0;
        if($total_ratings) {
            $values = array(1,2,3,4,5);
            $available_ratings = $total_ratings;
            foreach($values as $value) {
                $gt = $value - 1;
                $where = "module_id = $module_id AND row_id = $row_id AND rating";
                if($gt == 0) {
                    $where .= " < " . $value;
                } else {
                    $where .= " BETWEEN $gt AND " . $value;
                }
                $rating_bar = 0;
                $rating_count = $this->CI->db->where($where)->count_all_results($this->table_users);
                $rating_users = $rating_users + $rating_count;
                if($value == 5) {
                    $rating_bar = 100;
                    $available_ratings = $total_ratings - $rating_count;
                } else if($available_ratings != 0) {
                    $rating_bar = ($rating_count * 100) / $available_ratings;
                }
                $rating_counts[$value] = array('bar' => $rating_bar, 'count' => $rating_count);
            }
            return array('users' => $rating_users, 'counts' => $rating_counts);
        } else {
            return array();
        }
    }
    
    /**
     * Update ratings
     * @param string $module
     * @param number $row_id
     */
    public function update_rating($module, $row_id, $result = array()) {
        $module_id = gc_module_id($module);
        if(empty($result)) {
            $result = $this->CI->db->where(array('module_id' => $module_id, 'row_id' => $row_id))->get($this->table_name)->row();
        }
        if($result) {
            $this->CI->db->where('id', $row_id)->update($module, array('ratings' => $result->average_rating, 'rating_users' => $result->total_users));
        }
    }
    
    /**
     * Save Options Relation
     * When Selectize input is used to input option
     * Input Format: 1,4,5
     *
     * Use this method to allow a user to select existing options activated by the Admin and also allow them to create a new one.
     *
     * @param string $module
     * @param integer $row_id
     * @param integer $type_id - 0: Ratings, 1: Groups
     * @param string $options
     */
    public function selectize_options($module, $row_id, $type_id, $options, $status = 0) {
        $module_id = gc_module_id($module);
        // get existing options to compare them with the post submitted value
        $saved_options = $this->options_comma_separated($module, $row_id, $type_id);
        // skip below given code if nothing changed
        if($saved_options && $saved_options == $options) {
            return false;
        }
        $option_ids = array();
        
        // get existing options id to update count column
        $existing_options = $this->get_option_ids($module, $row_id, $type_id);
        if($existing_options) {
            foreach($existing_options as $existing_option) {
                $option_ids[] = $existing_option->option_id;
            }
        }
        
        // save options information
        $data = array();
        if(!empty($options)) {
            // convert string to an array and trim the whitespace
            $options = array_map('trim', explode(',', $options));
            // save only unique keywords
            $options = array_unique($options);
            foreach($options as $option) {
                $option = trim($option);
                // check whether the tag already exists in the Tag table
                if(is_numeric($option)) { // The user selected the existing tag or typed a numeric value
                    $option_info = $this->CI->db->select('id')->where('id', $option)->get($this->table_options)->row();
                    if(!$option_info) {
                        $option_info = $this->CI->db->select('id')->where('name', $option)->get($this->table_options)->row();
                    }
                } else { // This is a new option added by the user
                    $option = $this->CI->form_validation->rating_option_handling($option); // Remove unwanted characters from a Option
                    $option_info = $this->CI->db->select('id')->where('name', $option)->get($this->table_options)->row();
                    $option = strtoupper($option);
                }
                // Get Keyword info if an existing tag, if a new tag, save the info first and then retrieve
                $result = $this->get_option_info($option_info, $option, $module, $row_id, $type_id, $status);
                if(!empty($result['data'])) {
                    $data[] = $result['data'];
                }
            }
            if(!empty($data)) {
                // delete existing options in case user removed one
                $this->CI->db->where(array('module_id' => $module_id, 'row_id' => $row_id, 'type_id' => $type_id))->delete($this->table_relation);
                // save options information
                $this->CI->db->insert_batch($this->table_relation, $data);
            }
        } else {
            // The user may have unselected all options
            // delete existing options in case user removed all
            $this->CI->db->where(array('module_id' => $module_id, 'row_id' => $row_id))->delete($this->table_relation);
        }
    }
    
    /**
     * Get Keyword Information
     * @param array $keyword
     * @param integer/string $tag
     * @param integer $row_id
     * @param integer $module_id
     * @param integer $status - Status of the newly added keyword, can be used in bulk import
     */
    private function get_option_info($option_info, $option, $module, $row_id, $type_id, $status = 0) {
        $data = array();
        $module_id = gc_module_id($module);
        if($option_info) {
            $data = array(
                'module_id' => $module_id,
                'row_id' => $row_id,
                'option_id' => $option_info->id,
                'type_id' => $type_id
            );
        } else {
            // insert option in the options table
            $option_data = array();
            $option_data['type_id'] = $type_id;
            $option_data['name'] = $option;
            $option_data['status'] = $status;
            $option_data['created_on'] = date('Y-m-d H:i:s');
            $option_data['created_by'] = $this->CI->auth->user_id();
            $option_data['created_ip'] = $this->CI->input->ip_address();
            $this->CI->db->insert('options', $option_data);
            $option_id = $this->CI->db->insert_id();
            if($option_id) {
                $data = array(
                    'module_id' => $module_id,
                    'row_id' => $row_id,
                    'option_id' => $option_id,
                    'type_id' => $type_id
                );
            }
        }
        return array('data' => $data);
    }
    
    /**
     * Tags in comma separated format
     * Edit Module - Display both active and inactive tags
     * @param string $module
     * @param number $row_id
     * @return string|boolean
     */
    public function options_comma_separated($module, $row_id, $type_id, $active_only = FALSE) {
        $saved_options = $this->get_option_relations ($module, $row_id, $type_id, $active_only);
        $options = array();
        if($saved_options) {
            foreach ( $saved_options as $option ) {
                $options[] = $option->name;
            }
            // store them as a string in comma separated version
            $options = implode( ', ', $options);
            return $options;
        }
        return false;
    }
    
    /**
     * Get option from the relations table
     * @param string $module
     * @param number $row_id
     */
    public function get_option_relations($module, $row_id, $type_id, $active_only = FALSE) {
        $module_id = gc_module_id($module);
        $sql = "SELECT O.id, O.name FROM " . $this->CI->db->dbprefix . $this->table_options . " O";
        $sql .= " INNER JOIN " . $this->CI->db->dbprefix . $this->table_relation . " ORL ON ORL.option_id = O.id AND ORL.module_id = " . $module_id . " AND ORL.row_id = " . $row_id . " AND ORL.type_id = " . $type_id;
        if($active_only) {
            $sql .= " WHERE O.status = 1";
        }
        $sql .= " ORDER BY O.id";
        return $this->CI->db->query($sql)->result();
    }
    
    /**
     * Get option ids from the relations association table
     * @param string $module
     * @param number $row_id
     */
    public function get_option_ids($module, $row_id, $type_id) {
        $module_id = gc_module_id($module);
        $option_ids = $this->CI->db->select('option_id')->group_by('option_id')->get_where($this->table_relation, array('module_id' => $module_id, 'row_id' => $row_id, 'type_id' => $type_id))->result();
        return $option_ids;
    }
    
    /**
     * Display saved options on edit page
     * @param string $module
     * @param integer $row_id
     * @param integer $type_id
     */
    public function saved_options($module, $row_id, $type_id = 0) {
        $module_id = gc_module_id($module);
        // Display Existing Tags to the User
        $result = $this->CI->db->select('GROUP_CONCAT(option_id) AS options')->get_where($this->table_relation, array('module_id' => $module_id, 'row_id' => $row_id, 'type_id' => $type_id))->row();
        if($result) {
            return $result->options;
        }
        return null;
    }
    
    /**
     * Purge Option
     * @param integer $id
     */
    public function purge_option($id) {
        $this->CI->db->where('option_id', $id)->delete($this->table_relation);
        $this->CI->db->where('option_id', $id)->delete($this->table_name);
        return $this->CI->db->where('id', $id)->delete($this->table_options);
    }
    
    /**
     * Check whether the rating is allowed for this module or not
     * @param string $rating_module
     * @param number $row_id
     * @return boolean
     */
    public function is_allowed($rating_module, $row_id, $user_id = 0) {
        $allow_rating = false;
        $display_rating = false;
        $user_id = !empty($user_id) ? $user_id : $this->CI->auth->user_id();
        $settings = gc_json_settings($rating_module == 'products' ? 'product.settings' : 'blog.settings');
        if(!empty($settings['allow_ratings'])) {
            $display_rating = true;
            if(!empty($settings['subscribers_ratings_only'])) { // only subscribers can rate to their products
                if(user_ordered_items($rating_module, $row_id)) {
                    $allow_rating = true;
                }
            } else if(!empty($settings['loggedin_ratings_only'])) {
                if(!empty($user_id)) {
                    $allow_rating = true;
                }
            } else {
                $allow_rating = true;
            }
        }
        return array('allow' => $allow_rating, 'display' => $display_rating);
    }
    
    /**
     * Allow users to rate listing
     * @param int $row_id
     * @return string
     */
    public function save($user_id = 0, $return = false) {        
        $rating_module = escape($this->CI->input->post('module'));
        $row_id = (int) $this->CI->input->post('row_id');
        $is_allowed = $this->is_allowed($rating_module, $row_id, $user_id);
        if($is_allowed['allow']) {
            $result = $this->_rate($user_id);
        }
        if($return) {
            return $result;
        } else {
            echo json_encode ( $result );
        }
    }
    
    /**
     * Rate
     * @return string
     */
    private function _rate($user_id) {
        $module = $this->CI->input->post('module');
        $row_id = (int) $this->CI->input->post('row_id');
        header('Content-Type: application/x-json; charset=utf-8');
        $ip_address = $this->CI->input->ip_address();
        $new_rating = $this->CI->input->post('rating', true);
        $module_id = gc_module_id($module);
        $data = array (
            'module_id' => $module_id,
            'row_id' => $row_id,
            'ip_address' => $ip_address,
            'new_rating' => (int) $new_rating
        );
        $user_id = !empty($user_id) ? $user_id : $this->CI->auth->user_id();
        $result = $this->star_rating($data, $user_id);
        $message = array();
        switch($result) {
            case 0:
                break;
            case 1:
            case 4:
                $message = array(
                    'success' => 1,
                    'label' => 'success',
                    'result' => lang('success_rating')
                );
                break;
            case 2:
            case 5:
                $message = array(
                    'success' => 0,
                    'label' => 'danger',
                    'result' => sprintf(lang('error_rating'), lang('singular_' . $module))
                );
                break;
            case 3:
                $message = array(
                    'success' => 0,
                    'label' => 'danger',
                    'result' => sprintf(lang('failure_rating'), lang('singular_' . $module))
                );
                break;
        }
        return $message;
    }
    
    /**
     * Allow users to rate listing
     * @param int $listing_id
     * @return string $JsonResponse
     *
     */
    public function star_rating($rating_data, $user_id = 0) {
        $current_rating = $this->get_rating($rating_data['module_id'], $rating_data['row_id']);
        $result = 0;
        // Match user system ip address with existing records
        if ($current_rating) { // Entry exist for specified listing
            
            // check whether user already rating for this listing or not
            $ip_address = $rating_data['ip_address'];
            $rating_users = $this->CI->db->where(array('rating_id' => $current_rating->id, 'user_id' => $user_id, 'ip_address' => $ip_address))->count_all_results($this->table_users);
            if($rating_users > 0) { // User already rated for this listing
                $result = 3;
            } else {
                $data ['total_ratings']  = $current_rating->total_ratings + $rating_data['new_rating']; // Get existing ratings and increment with new rating
                $data ['total_users']    = $current_rating->total_users + 1; // Increment users by one
                $data ['average_rating'] = $data ['total_ratings'] / $data ['total_users'];
                if ($this->CI->db->where('id', $current_rating->id)->update( $this->table_name, $data)) { // Update rating information
                    // save users information with IP Address in the rating_users table
                    // to avoid further ratings from the save user
                    $rating_user_data = array(
                        'rating_id'  => $current_rating->id,
                        'user_id'    => $user_id,
                        'module_id'  => $rating_data['module_id'],
                        'row_id'     => $rating_data['row_id'],
                        'rating'     => $rating_data['new_rating'],
                        'rated_on'   => date('Y-m-d H:i:s'),
                        'ip_address' => $ip_address
                    );
                    $this->CI->db->insert($this->table_users, $rating_user_data);
                    
                    $result = 1;
                } else {
                    $result = 2;
                }
            }
        } else { // User is rating first time for this listing
            $data ['row_id']		 = $rating_data['row_id'];
            $data ['module_id']	     = $rating_data['module_id'];
            $data ['average_rating'] = $rating_data['new_rating'];
            $data ['total_ratings']  = $rating_data['new_rating'];
            $data ['total_users']    = 1;
            $this->CI->db->insert($this->table_name, $data);
            $insert_id = $this->CI->db->insert_id();
            if ($insert_id > 0) { // Update rating information
                // save users information with IP Address in the rating_users table
                // to avoid further ratings from the save user
                $rating_user_data = array(
                    'rating_id'   => $this->CI->db->insert_id(),
                    'user_id'     => (int) $user_id,
                    'module_id'   => $rating_data['module_id'],
                    'row_id'      => $rating_data['row_id'],
                    'rating'      => $rating_data['new_rating'],
                    'rated_on'    => date('Y-m-d H:i:s'),
                    'ip_address'  => $rating_data['ip_address']
                );
                $this->CI->db->insert($this->table_users, $rating_user_data);
                
                $result = 4;
            } else {
                $result = 5;
            }
        }
        // clear ratings cache
        clear_cache('ratings_' . $rating_data['module_id'] . '_' . $rating_data['row_id']);
        // update ratings in the module table
        $rating_module = gc_module_name($rating_data['module_id']);
        $this->update_rating($rating_module, $rating_data['row_id']);
        
        return $result;
    }
    
    /**
     * Get Ratings
     * @param number $module_id
     * @param number $row_id
     * @return object
     */
    public function get_rating($module_id, $row_id) {
        return $this->CI->db->where(array('row_id' => $row_id, 'module_id' => $module_id))->get($this->table_name)->row();
    }
    
    /**********************************************
     * FRONTEND
     **********************************************/    
    /**
     * Save Rating
     * @param number $field_id - Rating ID
     */
    public function save_rating($field_id) {
        $average_rating = 'average_rating_' . $field_id;
        $total_ratings = 'total_ratings_' . $field_id;
        $total_users = 'total_users_' . $field_id;
        $this->CI->form_validation->set_rules($average_rating, lang('average_rating'), 'trim|required|is_decimal');
        $this->CI->form_validation->set_rules($total_ratings, lang('total_ratings'), 'trim|required|is_decimal');
        $this->CI->form_validation->set_rules($total_users, lang('total_users'), 'trim|required|is_numeric');
        if ($this->CI->form_validation->run() == TRUE) {
            $data = array();
            $data['average_rating'] = $this->CI->input->post($average_rating);
            $data['total_ratings'] = $this->CI->input->post($total_ratings);
            $data['total_users'] = $this->CI->input->post($total_users);
            $rating = $this->CI->db->where('id', $field_id)->get($this->table_name)->row();
            if($rating) {
                // update ratings in the module table
                $rating_module = gc_module_name($rating->module_id);
                $this->update_rating($rating_module, $rating->row_id);
            }
            return $this->CI->db->where('id', $field_id)->update($this->table_name, $data);
        }
    }
    
    /**
     * Allow admin to bulk generate ratings for all listings
     */
    public function random() {
        if (isset($_POST['save'])) {
            $this->CI->form_validation->set_rules('module', lang('label_module'), 'trim');
            $this->CI->form_validation->set_rules('avg_min_value', lang('label_average_rating'), 'required|integer|min_val[1]');
            $this->CI->form_validation->set_rules('avg_max_value', lang('label_average_rating'), 'required|integer|max_val[5]|compare_fields['.$this->CI->input->post('avg_min_value').']');
            $this->CI->form_validation->set_rules('usr_min_value', lang('label_total_users'), 'required|integer|min_val[1]');
            $this->CI->form_validation->set_rules('usr_max_value', lang('label_total_users'), 'required|integer|max_val[20]|compare_fields['.$this->CI->input->post('usr_min_value').']');
            $this->CI->form_validation->set_rules('id_from', lang('label_ids'), 'required|integer|min_val[1]');
            $this->CI->form_validation->set_rules('id_to', lang('label_ids'), 'required|integer|max_val[10000000]|compare_fields['.$this->CI->input->post('id_from').']|diff_between['.$this->CI->input->post('id_from').',50000]');
            if($this->CI->form_validation->run() == TRUE) {
                $module = $this->CI->input->post('module');
                $avg_min = $this->CI->input->post('avg_min_value');
                $avg_max = $this->CI->input->post('avg_max_value');
                $user_min = $this->CI->input->post('usr_min_value');
                $user_max = $this->CI->input->post('usr_max_value');
                if($this->CI->input->post('insert_or_update') == 1) {
                    // INSERT ONLY
                    $sql = "SELECT id FROM " . $this->CI->db->dbprefix . $module ." WHERE id BETWEEN ".$this->CI->input->post('id_from')." AND ".$this->CI->input->post('id_to')." AND id NOT IN (SELECT row_id FROM " . $this->table_name . " R WHERE R.module_id = ". gc_module_id($module) .") LIMIT 50000";
                    $query = $this->CI->db->query ( $sql );
                    $get_ids = $query->result_array ();
                    if ($get_ids) {
                        $data = array ();
                        foreach ( $get_ids as $id ) {
                            $avergate_rating = mt_rand ( $avg_min * 5, $avg_max * 5 ) / 5;
                            $total_users = mt_rand ( $user_min, $user_max );
                            $data [] = array (
                                'row_id' => $id ['id'],
                                'module_id' => gc_module_id($module),
                                'average_rating' => $avergate_rating,
                                'total_ratings' => $avergate_rating * $total_users,
                                'total_users' => $total_users
                            );
                            
                            // update ratings in the module table
                            $this->update_rating($module, $id['id']);
                        }
                        $this->CI->db->insert_batch( $this->table_name, $data);
                        Template::set_message(lang('msg_success'), 'success');
                    } else {
                        Template::set_message(sprintf(lang('msg_nothing_to_update'), gc_module_translation($module)), 'danger');
                    }
                } else {
                    // INSERT AND UPDATE
                    $update_sql = "SELECT id FROM " . $this->CI->db->dbprefix . $module . " WHERE id BETWEEN ".$this->CI->input->post('id_from')." AND ".$this->CI->input->post('id_to')." LIMIT 50000";
                    $update_query = $this->CI->db->query ( $update_sql );
                    $update_ids = $update_query->result_array ();
                    if ($update_ids) {
                        foreach ( $update_ids as $id ) {
                            $avg_rating = mt_rand ( $avg_min * 5, $avg_max * 5 ) / 5;
                            $tot_users = mt_rand ( $user_min, $user_max );
                            
                            $insert_data = array (
                                'row_id' => $id ['id'],
                                'module_id' => gc_module_id($module),
                                'average_rating' => $avg_rating,
                                'total_ratings' => $avg_rating * $tot_users,
                                'total_users' => $tot_users,
                                'created_on' => date ( "Y-m-d H:i:s" )
                            );
                            
                            // update ratings in the module table
                            $this->update_rating($module, $id['id']);
                            
                            $update_data = "average_rating = " . $avg_rating . ", total_ratings = " . $avg_rating * $tot_users . ", total_users = " . $tot_users . ", modified_on = '" . date ( "Y-m-d H:i:s" ) . "'";
                            
                            $insert_update_query = $this->CI->db->insert_string ( $this->table_name, $insert_data ) . ' ON DUPLICATE KEY UPDATE ' . $update_data;
                            $this->CI->db->query ( $insert_update_query );
                            Template::set_message(lang('msg_success'), 'success');
                        }
                    } else {
                        Template::set_message(sprintf(lang('msg_insert_success'), gc_module_translation($module)), 'success');
                    }
                }
            } else {
                Template::set_message(lang('create_failure'), 'danger');
            }
        }
        Template::set('toolbar_title', lang('random_ratings'));
        Template::set_view('ratings/random');
        Template::render();
    }
    
    /**
     * Update ratings and rating_users column in the modules table
     * Done to prevent Extra JOIN
     *
     */
    public function update_modules() {
        $modules = gc_modules_only();
        $modules[] = 'listings';
        $modules[] = 'videos';
        if(isset($_POST['save'])) {
            $rating_modules = array();
            $site_modules = $this->CI->input->post('modules');
            if($site_modules != 0) {
                $rating_modules[] = gc_module_name($site_modules);
            } else {
                $rating_modules = $modules;
            }
            if(!empty($rating_modules)) {
                foreach($rating_modules as $rating_module) {
                    $module_id = gc_module_id($rating_module);
                    $sql = "SELECT id, module_id, row_id, average_rating, total_users FROM " . $this->CI->db->dbprefix . "ratings WHERE module_id = " . $module_id;
                    $records = $this->CI->db->query($sql)->result();
                    if($records) {
                        $data = array();
                        foreach($records as $record) {
                            $data[] = array(
                                'id' => $record->row_id,
                                'ratings' => $record->average_rating,
                                'rating_users' => $record->total_users
                            );
                        }
                    }
                    $this->CI->db->update_batch($rating_module, $data, 'id');
                }
                Template::set_message(lang('msg_success'), 'success');
                Template::redirect(current_url());
            } else {
                Template::set_message(lang('msg_error'), 'danger');
            }
        }
        $view_modules = array(
            0 => lang('tab_all')
        );
        foreach($modules as $module) {
            $view_modules[gc_module_id($module)] = lang($module);
        }
        asort($view_modules);
        Template::set(array(
            'toolbar_title' => lang('gc_menu_module_ratings'),
            'modules' => $view_modules
        ));
        Template::render();
    }
    
    
    /**
     * Display a list of options data.
     *
     * @return void
     */
    public function options($filter='all', $offset = 0) {
        // Status Change
        if (isset($_POST['status_change'])) {
            $this->CI->gc->status_change('rating_options', $_POST['status_change'], 'status');
        }
        // Deleting anything?
        if (isset($_POST['delete'])) {
            $this->CI->auth->restrict($this->permissionDelete);
            $checked = $this->CI->input->post('checked');
            if (is_array($checked) && count($checked)) {
                
                // If any of the deletions fail, set the result to false, so
                // failure message is set if any of the attempts fail, not just
                // the last attempt
                
                $result = true;
                foreach ($checked as $pid) {
                    $deleted = $this->purge_option($pid);
                    if ($deleted == false) {
                        $result = false;
                    }
                }
                if ($result) {
                    Template::set_message(count($checked) . ' ' . sprintf(lang('msg_action_success'), lang('label_options'), lang('label_deleted')), 'success');
                } else {
                    Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_options')), 'danger');
                }
            }
        }
        $where = '';
        // Filters
        $filter_type = $filter;
        
        switch($filter_type) {
            case 'ratings':
                $where = "type_id = 0";
                break;
                
            case 'groups':
                $where = "type_id = 1";
                break;
                
            case 'all':
                // Nothing to do
                break;
                
            default:
                show_404("ratings/options/$filter/");
        }
        
        $like_array = array();
        $searchterm = $this->CI->gc->search('rating_options', $this->uri->segment(5), '/content/ratings/options');
        if(!empty($searchterm)) {
            // This is the basic search filter, you could add more fields to the search
            $like_array = array(
                'id' => $searchterm,
                'name' => $searchterm
            );
        }
        $pagerUriSegment = 6;
        $pagerBaseUrl = site_url(SITE_AREA . '/content/ratings/options') . '/';
        
        $limit  = $this->CI->settings_lib->item('site.list_limit') ?: 15;// Sort By
        if(!empty($where)) {
            $this->CI->db->where($where);
        }
        if(!empty($like_array)) {
            $this->CI->db->or_like($like_array);
        }
        $total_rows = $this->CI->db->count_all_results('rating_options');
        $this->CI->load->library('pagination');
        $pager['base_url']    = $pagerBaseUrl;
        $pager['total_rows']  = $total_rows;
        $pager['per_page']    = $limit;
        $pager['uri_segment'] = $pagerUriSegment;
        
        $this->pagination->initialize($pager);
        $this->CI->db->limit($limit, $offset);
        
        $order_by = $this->gc->sort_by_admin('rating_options');
        if(!empty($order_by)) {
            $this->CI->db->order_by($this->session->userdata('options_sort')['column'], $this->session->userdata('options_sort')['order']);
        } else {
            $this->CI->db->order_by('id', 'DESC');
        }
        // Sort By
        if(!empty($where)) {
            $this->CI->db->where($where);
        }
        if(!empty($like_array)) {
            $this->CI->db->or_like($like_array);
        }
        $records = $this->CI->db->get('rating_options')->result();
        Template::set('records', $records);
        Template::set('index_url', site_url(SITE_AREA .'/content/ratings/options/'));
        Template::set('filter_type', $filter_type);
        Template::set('toolbar_title', lang('manage_options'));
        Template::render();
    }
    
    /**
     * Create a options object.
     *
     * @return void
     */
    public function add_option() {
        $this->CI->auth->restrict($this->permissionCreate);
        
        if (isset($_POST['save'])) {
            if ($insert_id = $this->save_options()) {
                log_activity($this->CI->auth->user_id(), lang('act_create_record') . ': ' . $insert_id . ' : ' . $this->CI->input->ip_address(), 'rating_options');
                Template::set_message(sprintf(lang('msg_save_success'), lang('label_option')), 'success');
                
                redirect(SITE_AREA . '/content/ratings');
            } else {
                Template::set_message(sprintf(lang('msg_create_failure'), lang('label_option')), 'danger');
            }
        }
        
        Template::set('toolbar_title', lang('add_option'));
        Template::set_view('content/options_form');
        Template::render();
    }
    /**
     * Allows editing of options data.
     *
     * @return void
     */
    public function edit_option() {
        $id = $this->uri->segment(5);
        if (empty($id)) {
            Template::set_message(lang('invalid_id'), 'danger');
            redirect(SITE_AREA . '/content/ratings/options');
        }
        
        if (isset($_POST['save'])) {
            $this->CI->auth->restrict($this->permissionEdit);
            
            if ($this->save_options('update', $id)) {
                log_activity($this->CI->auth->user_id(), lang('act_edit_record') . ': ' . $id . ' : ' . $this->CI->input->ip_address(), 'rating_options');
                Template::set_message(sprintf(lang('msg_save_success'), lang('label_option')), 'success');
                redirect(SITE_AREA . '/content/ratings/options');
            } else {
                Template::set_message(sprintf(lang('msg_edit_failure'), lang('label_option')), 'danger');
            }
        } elseif (isset($_POST['delete'])) {
            $this->CI->auth->restrict($this->permissionDelete);
            
            if ($this->purge_option($id)) {
                log_activity($this->CI->auth->user_id(), lang('act_delete_record') . ': ' . $id . ' : ' . $this->CI->input->ip_address(), 'rating_options');
                Template::set_message(sprintf(lang('msg_delete_success'), lang('label_option')), 'success');
                redirect(SITE_AREA . '/content/ratings/options');
            }
            Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_option')), 'danger');
        }
        $record = $this->CI->db->where('id', $id)->get('rating_options')->row();
        Template::set('record', $record);
        
        Template::set('toolbar_title', lang('edit_option'));
        Template::set_view('content/options_form');
        Template::render();
    }
    
    //--------------------------------------------------------------------------
    // !PRIVATE METHODS
    //--------------------------------------------------------------------------
    
    /**
     * Save the data.
     *
     * @param string $type Either 'insert' or 'update'.
     * @param int    $id   The ID of the record to update, ignored on inserts.
     *
     * @return boolean|integer An ID for successful inserts, true for successful
     * updates, else false.
     */
    private function save_options($type = 'insert', $id = 0) {
        if ($type == 'update') {
            $_POST['id'] = $id;
        }
        
        // Validate the data
        $this->CI->form_validation->set_rules('type_id', 'lang:label_type', 'trim|is_numeric');
        $this->CI->form_validation->set_rules('name', 'lang:label_name', 'trim|required');
        $this->CI->form_validation->set_rules('status', 'lang:label_status', 'trim');
        if ($this->CI->form_validation->run() === false) {
            return false;
        }
        $data = array(
            'type_id' => $this->CI->input->post('type_id'),
            'name' => $this->CI->input->post('name'),
            'status' => $this->CI->input->post('status') ? 1 : 0
        );
        $return = false;
        if ($type == 'insert') {
            $this->CI->db->insert('rating_options', $data);
            $id = $this->CI->db->insert_id();
            
            if (is_numeric($id)) {
                $return = $id;
            }
        } elseif ($type == 'update') {
            $return = $this->CI->db->where('id', $id)->update('rating_options', $data);
        }
        
        return $return;
    }
    
    /**
     * Get rating by divison
     * @param number $module_id
     * @param number $row_id
     * @return object
     */
    public function get_rating_division($module_id, $row_id, $table = 'rating_users') {
        $this->CI->load->library('gzf_cache', array('lifetime' => 900));
        $grating_cache = $this->CI->gzf_cache->get_instance();
        $cacheID = 'ratings_' . $module_id . '_' . $row_id;
        if(!$result = $grating_cache->load($cacheID)) {
            $dbprefix = $this->CI->db->dbprefix;
            $sql = "SELECT COUNT(*) AS five,";
            $sql .= " (SELECT COUNT(*) FROM " . $dbprefix . "rating_users WHERE module_id = " . $module_id . " AND row_id = " . $row_id . " AND (rating BETWEEN 3 AND 4)) AS four,";
            $sql .= " (SELECT COUNT(*) FROM " . $dbprefix . "rating_users WHERE module_id = " . $module_id . " AND row_id = " . $row_id . " AND (rating BETWEEN 2 AND 3)) AS three,";
            $sql .= " (SELECT COUNT(*) FROM " . $dbprefix . "rating_users WHERE module_id = " . $module_id . " AND row_id = " . $row_id . " AND (rating BETWEEN 1 AND 2)) AS two,";
            $sql .= " (SELECT COUNT(*) FROM " . $dbprefix . "rating_users WHERE module_id = " . $module_id . " AND row_id = " . $row_id . " AND (rating BETWEEN 0 AND 1)) AS one";
            $sql .= " FROM " . $dbprefix . "rating_users WHERE module_id = " . $module_id . " AND row_id = " . $row_id . " AND (rating BETWEEN 4 AND 6)";
            $result = $this->CI->db->query($sql)->row();
            $grating_cache->save($result, $cacheID, array($cacheID));
        }
        return $result;
    }
    
    /**
     * Sample Data - Generate Random Ratings
     * @param string $module
     * @param float $avg_min
     * @param float $avg_max
     * @param float $user_min
     * @param float $user_max
     */
    public function random_ratings($module, $avg_min, $avg_max, $user_min, $user_max, $limit = 25, $id_from = 0, $id_to = 25) {
        // INSERT AND UPDATE
        if($id_from > 0) {
            $update_sql = "SELECT id FROM " . $this->CI->db->dbprefix . $module . " WHERE id BETWEEN ".$this->CI->input->post('id_from')." AND ".$this->CI->input->post('id_to')." LIMIT 50000";
        } else {
            $update_sql = "SELECT id FROM " . $this->CI->db->dbprefix . $module . " LIMIT " . $limit;
        }
        $update_query = $this->CI->db->query ( $update_sql );
        $update_ids = $update_query->result_array ();
        if ($update_ids) {
            foreach ( $update_ids as $id ) {
                $avg_rating = mt_rand ( $avg_min * 5, $avg_max * 5 ) / 5;
                $tot_users = mt_rand ( $user_min, $user_max );
                
                $insert_data = array (
                    'row_id' => $id ['id'],
                    'module_id' => gc_module_id($module),
                    'average_rating' => $avg_rating,
                    'total_ratings' => $avg_rating * $tot_users,
                    'total_users' => $tot_users,
                    'created_on' => date ( "Y-m-d H:i:s" )
                );
                
                // update ratings in the module table
                $this->update_rating($module, $id['id']);
                
                $update_data = "average_rating = " . $avg_rating . ", total_ratings = " . $avg_rating * $tot_users . ", total_users = " . $tot_users . ", modified_on = '" . date ( "Y-m-d H:i:s" ) . "'";
                
                $insert_update_query = $this->CI->db->insert_string ( $this->table_name, $insert_data ) . ' ON DUPLICATE KEY UPDATE ' . $update_data;
                $this->CI->db->query ( $insert_update_query );
            }
        }
    }
}