<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
 || ##################################################################### ||
 || # GoClixy - Online Software Solutions							   # ||
 || # Copyright (C) 2021 GoClixy. All Rights Reserved.				   # ||
 || # ----------------------------------------------------------------  # ||
 || # This file is owned and copyrighted by GoClixy.com. Your license   # ||
 || # confers  no  title  or ownership in the Script and is not a sale  # ||
 || # of any rights to the Software. This file may not be redistributed # ||
 || # in whole or significant part.									   # ||
 || # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
 || # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
 || ##################################################################### ||
 \*=======================================================================*/
class Gsample {
    /**
     * Put images in a folder on the API server and query it on the server
     * Use a column name during save
     */
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        $this->CI =& get_instance();
        $this->CI->load->library('gzf_cache', array('lifetime' => 900));
        $this->CI->load->library('slug');
        $this->CI->load->library('gcrud');
        $this->CI->load->model('fields/fields_model');
        $this->z_cache = $this->CI->gzf_cache->get_instance();
    }
    
    /**
     * Import Sample Data
     */
    public function sample_data($actions = array()) {
        $limit = !empty($actions['limit']) ? $actions['limit'] : 25;
        // $this->truncate_tables(); // truncate all tables first
        if(array_key_exists('users', $actions)) {
            $this->_truncate_images('assets/images/users', true);
            $this->purge_users(false); // remove all users
            $this->random_users($limit); // generate random users
        }
        // $result = api_goclixy('ecommerce?type=listings&limit=10');
        // dump($result);
        // $this->import_module();
    }
    
    /**
     * Import Modules
     * @param number $limit
     * @param string $url_params - e.g. key=categories_id&value=217
     */
    private function import_module($module = 'listings', $limit = 20, $url_params = null) {
        $module_id = gc_module_id($module);
        $result = api_goclixy('ecommerce?type=' . $module . '&limit=' . $limit . $url_params);
        $records = json_decode($result, true);
        $images_data = array();
        $max_id = $this->CI->db->query("SELECT max(id) AS max_id from " . $this->CI->db->dbprefix . $module)->row();
        if(!empty($records['records'])) {
            // get categories and brands
            // $product_extras = array('categories' => $this->get_product_extras('product_categories'), 'brands' => $this->get_product_extras('product_brands'));
            $this->CI->load->model('locations/cities_model');
            $fields = $this->table_columns($module);
            foreach($records['records'] as $record) {
                $row = $this->CI->db->where('title', $record['title'])->get($module)->row();
                if($row) {
                    $data = array();
                    $user = (array) $this->get_random_user();
                    $user_fields = array('contact_person', 'email', 'phone_number', 'address', 'postcode');
                    if($user) {                        
                        foreach($user_fields as $user_field) {
                            if(empty($record[$user_field])) {
                                $data[$user_field] = $user_field == 'contact_person' ? $user['display_name'] : $user[$user_field];
                            } else {
                                $data[$user_field] = $record[$user_field];
                            }
                        }
                    }
                    $city = null;
                    if(!empty($record['city'])) {
                        $city = $this->CI->cities_model->find_by('name', $record['city']);
                    } else if(!empty($user['city_id'])) {
                        $city = $this->CI->cities_model->find($user['city_id']);
                    }
                    if($city) {
                        $data['city_id'] = $city->id;
                        $data['state_id'] = $city->state_id;
                        $data['country_iso'] = $city->country_iso;
                        $data['city'] = $city->name;
                        $data['state'] = $city->state;
                        $data['country'] = $city->country;
                        if(empty($record['latitude'])) {
                            $data['latitude'] = $city->latitude;
                            $data['longitude'] = $city->longitude;
                        }
                    }
                    foreach($fields as $field) {
                        $field_value = $this->process_field_value($field, $record);
                        if(!empty($field_value)) {
                            $data[$field] = $field_value;
                        }
                    }
                    foreach($record as $key => $value) {
                        switch($key) {
                            case 'title':
                                $data['title'] = $value;
                                break;
                            case 'description':
                                $data[$key] = $value;
                                break;
                            case 'meta_keywords': // to load specific demo type
                                $data[$key] = $value;
                                break;
                        }
                    }
                    // get random categories/tags
                    $categories = $this->tags_categories('categories', $module, !empty($record['categories']) ? $record['categories'] : null);
                    $categories_relation = $categories['relation'];
                    unset($categories['relation']);
                    
                    $tags = $this->tags_categories('tags', $module, !empty($record['tags']) ? $record['tags'] : null);
                    $tags_relation = $tags['relation'];
                    unset($tags['relation']);
                    
                    $data = array_merge($data, $categories, $tags);
                    
                    $row_id = 0;
                    // $this->CI->db->insert($module, $data);
                    $row_id = 0; //$this->CI->db->insert_id();
                    if($row_id) {
                        // save relation
                        if(!empty($categories_relation)) {
                            $this->save_relation('categories', $module, $row_id, $categories_relation);
                        }
                        if(!empty($tags_relation)) {
                            $this->save_relation('tags', $module, $row_id, $tags_relation);
                        }
                        
                        // save images
                        if(!empty($record['images'])) {
                            // $this->CI->load->library('goclixy/gimages');
                            $i = 1;
                            foreach($record['images'] as $url) {
                                $filename = substr( $url , strrpos( $url , '/' ) + 1);
                                // check the file existence
                                if($i == 1) {
                                    $destination_path = 'assets/images/' . $module;
                                } else {
                                    $filepath = 'assets/images/' . $module;
                                }
                                
                                $filepath = $destination_path . $filename;
                                $count = $this->CI->db->where(array('path' => $destination_path, 'original_filename' => $filename))->count_all_results('images');
                                if($count == 0) {
                                    $image_type_id = $i == 1 ? 1 : 0;
                                    $file_image = explode('.', $filename);
                                    $images_data[] = array(
                                        'module_id' => $module_id,
                                        'row_id' => $row_id,
                                        'type_id' => $image_type_id,
                                        'title' => strtoupper(str_replace('-', ' ', $file_image[0])),
                                        'url' => $url,
                                        'path' => $destination_path,
                                        'filename' => $filename,
                                        'original_filename' => $filename,
                                        'status' => 1
                                    );
                                    // $this->download_image($url, $row_id, null, null, $image_type_id, 20, 'products');
                                    $i++;
                                }
                            }
                        }
                    }
                }
            }
            
            // save images data
            if(!empty($images_data)) {
                $this->CI->db->insert_batch('images', $images_data);
                // get all images
                $where_sql = 'id > ' . (!empty($max_id->max_id) ? $max_id->max_id : 0);
                $this->CI->load->library('goclixy/gimages');
                $records = $this->CI->db->where($where_sql)->get($module)->result();
                if($records) {
                    foreach($records as $record) {
                        $this->CI->gimages->_tweak_images_row($module, $record->id);
                    }
                }
            }
        }
    }
		
	/**
	 * Import Sample Data
	 */
	public function sample_data__ld($actions = array()) {
		$limit = !empty($actions['limit']) ? $actions['limit'] : 25;
		// $this->truncate_tables(); // truncate all tables first
		if(array_key_exists('users', $actions)) {			
			$this->_truncate_images('assets/images/users', true);
			$this->purge_users(false); // remove all users
			$this->random_users($limit); // generate random users
		}
		if(array_key_exists('products', $actions)) {
			$tables = array('images', 'products', 'product_licenses', 'product_metas', 'product_ratings', 'product_rating_users', 'product_views', 'product_wishlists');
			$this->truncate_tables($tables); // truncate product tables
			$this->truncate_images(); // remove all images
			$this->import_data('categories', 'product', 1000); // import categories first
			$this->import_data('brands', 'product', 1000); // import categories first
			$url_params = null;
			if(!empty($actions['categories_id'])) {
			    $url_params .= '&key=categories_id&value=' . $actions['categories_id'];
			}
			if(!empty($actions['types_id'])) {
			    $url_params .= '&key=types_id&value=' . $actions['types_id'];
			}
			$this->import_products($limit, $url_params); // import products
			$this->CI->load->library('goclixy/gratings');
			$this->CI->gratings->random_ratings('products', 3, 5, 1, $limit, $limit); // generate random ratings
			$this->random_set('product_wishlists', 'products', 'product_id', 10); // random wishlist
			$this->CI->load->library('goclixy/gcategories'); // update categories counts
			$this->CI->gcategories->update_counts('products');			
		}
		if(array_key_exists('blogs', $actions)) {
			$this->import_data('blogs'); // import blogs
			$this->random_set('blog_favorites', 'blogs', 'blog_id', 10); // random favorites
			$this->CI->load->library('goclixy/gcategories'); // update categories counts
			$this->CI->gcategories->update_counts('products');
		}
		if(array_key_exists('subscribers', $actions)) {
			$this->import_data('users', 'subscriber', $limit, 'subscribers'); // import subscribers
		}
		if(array_key_exists('forums', $actions)) {
			$this->import_data('threads', 'forum', $limit, 'threads'); // import threads
			$this->import_data('posts', 'forum', $limit, 'posts'); // import posts
		}
		if(array_key_exists('groups', $actions)) {
			$this->import_data('groups', 'product_content', 50, 'groups'); // import groups
		}
		if(array_key_exists('features', $actions)) {
			$this->import_data('features', 'product', 50, 'features'); // import features
		}
		if(array_key_exists('faqs', $actions)) {
			$this->import_data('faqs', 'product'); // import FAQs
		}
		if(array_key_exists('testimonials', $actions)) {
			$this->import_data('testimonials', 'product'); // import testimonials
		}
		if(array_key_exists('pages', $actions)) {
			$this->import_data('pages'); // import pages
		}
		if(array_key_exists('news', $actions)) {
			$this->import_data('news'); // import news
		}
		if(array_key_exists('languages', $actions)) {
			$this->language_translations(); // copy translated text
		}
		if(array_key_exists('menus', $actions)) {
			$this->prepare_menus(); // prepare menu links
			$this->update_menus(); // update menus
		}
		if(array_key_exists('settings', $actions)) {
			$this->update_settings(); // update settings information
		}
		clear_cache(); // clear cache
	}
	
	/**
	 * Update Settings in the settings table
	 */
	private function update_settings() {
		$result = api_goclixy('ecommerce?type=settings');
        $records = json_decode($result, true);
		$data = array();
		if(!empty($records['records'])) {
			foreach($records['records'] as $record) {
				$data[] = array(
					'name' => $record['title'],
					'value' => strip_tags($record['description'])
				);
			}
			$this->CI->load->model('settings/settings_model');
            $updated = $this->CI->settings_model->update_batch($data, 'name');
		}			
	}
	
	/**
	 * Prepare menus
	 */
	private function prepare_menus() {
		$sql = "DELETE FROM " . $this->CI->db->dbprefix . "links WHERE menu_id = 1 AND parent_id != 0";
		$this->CI->db->query($sql);
		$links = $this->CI->db->where('id IN (3, 10, 57, 50, 54)')->get('links')->result();
		if($links) {
			$link_data = array();
			$modified_on = date('Y-m-d H:i:s');
			foreach($links as $link) {
				switch($link->id) {
					case 57:
						$link_data[] = array(
							'id' => $link->id,
							'name' => 'Our Products',
							'modified_on' => $modified_on
						);
						break;
					case 3:
						$link_data[] = array(
							'id' => $link->id,
							'link_url' => 'pages',
							'name_lang' => 0,
							'icon' => 'fas fa-user',
							'extra' => 'table:pages;row_id:5',
							'name' => 'About Us',
							'modified_on' => $modified_on
						);
						break;
					case 10:
						$link_data[] = array(
							'id' => $link->id,
							'link_url' => 'pages',
							'extra' => 'table:pages;row_id:3',
							'modified_on' => $modified_on
						);
						break;
					case 50:
						$link_data[] = array(
							'id' => $link->id,
							'name_lang' => 0,
							'icon' => 'fas fa-gavel',
							'link_url' => 'pages',
							'name' => 'Privacy Policy',
							'extra' => 'table:pages;row_id:1',
							'modified_on' => $modified_on
						);
						break;
					case 54:
						$link_data[] = array(
							'id' => $link->id,
							'name_lang' => 0,
							'link_url' => 'pages',
							'icon' => 'fas fa-asterisk',
							'name' => 'Terms & Conditions',
							'extra' => 'table:pages;row_id:2',
							'modified_on' => $modified_on
						);
						break;
				}
			}
			if($link_data) {
				$this->CI->db->update_batch('links', $link_data, 'id');
			}
		}
		$this->reset_id_counter('links');
	}
	
	/**
	 * Add/Edit Menus
	 */
	private function update_menus() {		
		$result = api_goclixy('ecommerce?type=menu_items');
        $records = json_decode($result, true);
		$data = array();
		if(!empty($records['records'])) {
			foreach($records['records'] as $record) {
				// check the menu existence
				$menu = $this->CI->db->where(array('menu_id' => $record['menu_id'], 'name' => $record['title']))->get('links')->row();
				$data = $record;
				$data['name'] = $record['title'];
				$data['name_lang'] = !empty($record['name_lang']) ? (int) $record['name_lang'] : 0;
				$data['layout_id'] = !empty($record['layout_id']) ? (int) $record['layout_id'] : 0;
				$data['gc_url'] = !empty($record['gc_url']) ? (int) $record['gc_url'] : 1;
				unset($data['title']);
				unset($data['description']);
				unset($data['type']);
				if($menu) {
					$data['modified_on'] = date('Y-m-d H:i:s');
					$this->CI->db->where('id', $menu->id)->update('links', $data);
				} else {
					$data['created_on'] = date('Y-m-d H:i:s');
					$this->CI->db->insert('links', $data);
					$insert_id = $this->CI->db->insert_id();
					$menu = $this->CI->db->where('id', $insert_id)->get('links')->row();
				}				
				if($menu) {
					$id = $menu->id;
					$parent_id = $menu->parent_id;
					$this->CI->load->library('gclosure');
					$gclosure = new Gclosure('links', 'link_relations');
					$gclosure->delete($id, false); // do not delete from the links table
					// save hierarchy
					$gclosure->add($id, ($parent_id == 0 ? $id : $parent_id));
				}
			}
		}			
	}
	
	/**
	 * Import Data
	 * @param string $module - Name of the module to be imported
	 * @param string $prefix - Table Prefix
	 */
	public function import_data($module = 'blogs', $prefix = '', $limit = 25, $type = null) {
		// truncate tables first
		switch($module) {
			case 'blogs':
				$tables = array('blogs', 'blog_categories', 'blog_category_relations', 'blog_comments', 'blog_comment_relations', 'blog_favorites', 'blog_tags', 'blog_tag_relations', 'blog_views');
				$this->truncate_tables($tables);
				break;
			case 'testimonials':
			case 'faqs':
			case 'features':
			case 'teams':
			case 'videos':
			case 'references':
				$this->truncate_tables(array($prefix . '_' . $module));
				break;
			case 'users':
				if($prefix == 'subscriber') {
					$tables = array('subscriber_users', 'subscriber_metas', 'subscriber_messages');
					$this->truncate_tables($tables);
				}
				break;
			case 'threads':			
				$tables = array('forum_categories', 'forum_threads', 'forum_reports', 'forum_votes');
				$this->truncate_tables($tables);
				break;
			case 'posts':
				$tables = array('forum_posts');
				$this->truncate_tables($tables);
				break;
			case 'groups':
				$tables = array('product_content_groups');
				$this->truncate_tables($tables);
				break;
			case 'pages':
				$tables = array('pages', 'page_groups', 'page_popups');
				$this->truncate_tables($tables);
				break;
			case 'categories':
			    if($prefix == 'product') {
    			    $tables = array('product_categories', 'product_category_relations');
    			    $this->truncate_tables($tables);			    
			    }
			    break;
			case 'brands':
			    if($prefix == 'product') {
			        $tables = array('product_brands');
			        $this->truncate_tables($tables);
			    }
			    break;
		}
		
		// import data
        $result = api_goclixy('ecommerce?type=' . (empty($type) ? $module : $type) . '&limit=' . $limit);
        $records = json_decode($result, true);
		if(!empty($records['records'])) {	
			$module = !empty($prefix) ? $prefix . '_' . $module : $module;
			$fields = $this->table_columns($module);
            foreach($records['records'] as $record) {
                $data = array();
				$row = false;
				switch($module) {
					case 'product_testimonials':
						break;
					case 'subscriber_users':
						$row = $this->CI->db->where('email', $record['email'])->get($module)->row();
						break;
					case 'forum_threads':
					case 'forum_posts':
						$row = $this->CI->db->where('subject', $record['title'])->get($module)->row();
						break;
					case 'product_content_groups':
					case 'product_categories':
					case 'product_brands':
						$row = $this->CI->db->where('name', $record['title'])->get($module)->row();
						break;
					default:
						$row = $this->CI->db->where('title', $record['title'])->get($module)->row();
				}
				if(!$row) {
					foreach($fields as $field) {
						$field_value = $this->process_field_value($field, $record);					
						if(!empty($field_value)) {
							switch($field) {
								case 'product_ids':								
									switch($module) {
										case 'product_features':
											break;
										default:
											$data[$field] = $field_value;								
									}
									break;
								case 'brand_ids':						
									switch($module) {
										case 'product_features':
											$data[$field] = 1;
											break;
										default:
											$data[$field] = $field_value;								
									}
									break;
								default:
									$data[$field] = $field_value;
							}
						}
					}		
					foreach($record as $key => $value) {
						switch($key) {
							case 'id':
								break;
							case 'threads_id':
								$data['thread_id'] = $this->insert_update_value('forum_' . str_replace('_id', '', $key), $value, 'subject');
								unset($data['threads__id']);
								break;
							case 'groups_id':
								$data['group_ids'] = $this->insert_update_value('product_content_' . str_replace('_id', '', $key), $value, 'name');
								unset($data['groups__id']);
								break;
							case 'products_id':
								switch($module) {
									case 'product_features':										
										break;									
								}
								break;
							case 'slug':
							    switch($module) {
							        case 'subscriber_users':	
							            break;
							        default:
							            $data[$key] = $value;
							    }
							    break;
							case 'description':
								switch($module) {
									case 'subscriber_users': // address										
										$data['address'] = $value;
										break;
									case 'forum_threads':
									case 'forum_posts':									
										$data['message'] = $value;
										break;
									case 'product_content_groups':
									case 'product_brands':
										break;
									case 'pages':
										$data['body'] = $value;
										break;
									default:
										$data[$key] = $value;
								}
								break;
							case 'hide_title':
								$data['hide_title'] = !empty($value) ? 1 : 0;
								break;
							case 'image':
								$data['cover'] = $value;
								break;
							case 'type':
							case 'types_id':
							case 'categories__id':
							case 'brands__id':
							    break;
							case 'meta_keywords':
							    switch($module) {
							        case 'product_categories':
							            $data[$key] = $value;
							            break;
							        case 'product_brands':
							            $data['extras'] = $value;
							            break;
							    }
								break;
							case 'title':
								switch($module) {
									case 'subscriber_users': // address										
										$data['name'] = $value;
										break;
									case 'product_testimonials':
										break;
									case 'forum_threads':
									case 'forum_posts':									
										$data['subject'] = $value;
										break;
									case 'product_content_groups':
									case 'product_categories':	
										$data['name'] = $value;
										break;
									case 'product_brands':
									    $data['name'] = $value;
									    break;
									default:
										$data[$key] = $value;
								}
								break;
							default:
								$data[$key] = $value;
						}
					}
					$row_id = 0;
					$this->CI->db->insert($module, $data);
					$row_id = $this->CI->db->insert_id();
				}
			}
			switch($module) {
				case 'blogs':
					$this->CI->load->library('goclixy/gcategories');
					$this->CI->gcategories->update_counts($module);
					
					// set favorites
					$this->random_set('blog_favorites', $module, 'blog_id', 10);
					break;
				case 'product_categories':
				    // save categories relation
				    $this->save_relations();
				    break;
			}
		}
	}
	
	/**
	 * Save Category Relations
	 */
	private function save_relations() {
	    $this->CI->load->library('gclosure');
	    $gclosure = new Gclosure('product_categories', 'product_category_relations');
	    $records = $this->CI->db->get('product_categories')->result();
	    if($records) {
	        foreach($records as $record) {
    	        // $gclosure->delete($record->id, false); // do not delete from the categories table
    	        // save hierarchy
    	        $gclosure->add($record->id, ($record->parent_id == 0 ? $record->id : $record->parent_id));
	        }
	    }
	}
	
	/**
	 * Set favorites and wishlists randomly
	 * @param string $table
	 * @param string $key
	 * @param number $limit
	 */
	private function random_set($table, $module, $key = 'blog_id', $limit = 10) {
		$users = $this->get_random_rows('users', $limit);
		$records = $this->get_random_rows($module, $limit);
		if($records) {
			foreach($records as $rkey => $record) {
				if(empty($users[$rkey])) {
					$user_id = $users[array_rand($users)]['id'];
				} else {
					$user_id = $users[$rkey]['id'];
				}
				$sql = "INSERT IGNORE INTO " . $this->CI->db->dbprefix . $table . " (user_id, " . $key . ", created_on, created_ip) VALUES(" . $user_id . ", " . $record['id'] . ", '" . date('Y-m-d H:i:s') . "', '" . $this->random_ip() . "')";
				$this->CI->db->simple_query($sql);
			}
		}
	}
	
	/**
	 * Generate Random IP
	 */
	private function random_ip() {		
		$ip = intval(rand()%255).'.'.intval(rand()%255).'.'.intval(rand()%255).'.'.intval(rand()%255);
		return $ip;
	}
	
	/**
	 * Get random data from a table
	 * @param string $table - Name of the table
	 * @param number $limit - Number of records to be returned
	 */
	private function get_random_rows($table, $limit = 10) {		
		$sql = "SELECT id FROM " . $this->CI->db->dbprefix . $table . " ORDER BY RAND() LIMIT " . $limit;
		$result = $this->CI->db->query($sql)->result_array();
		return $result;
	}
    
    /**
     * Generate and fill tables with sample data
     * random tags, faqs, brands, categories, testimonials, references, videos, product categories
     * tickets, subscribers, affiliates, affiliate samples
     * random blogs, description
     */
    private function _sample__data() {
		$this->truncate_tables(); // truncate all tables first
		$this->purge_users(false); // remove all users
		$this->truncate_images(); // remove all images
		$this->random_users(25); // generate random users
    }
	/**
	 * Import Products
	 * @param number $limit
	 * @param string $url_params - e.g. key=categories_id&value=217
	 */
	private function import_products($limit = 20, $url_params = null) {		
        $module = 'products';
		$module_id = gc_module_id($module);
		$result = api_goclixy('ecommerce?type=products&limit=' . $limit . $url_params);
        $records = json_decode($result, true);
		$images_data = array();
		$max_id = $this->CI->db->query("SELECT max(id) AS max_id from " . $this->CI->db->dbprefix . $module)->row();
        if(!empty($records['records'])) {
            // get categories and brands
            $product_extras = array('categories' => $this->get_product_extras('product_categories'), 'brands' => $this->get_product_extras('product_brands'));
			$this->CI->load->model('locations/cities_model');
			$fields = $this->table_columns('products');
            foreach($records['records'] as $record) {
				$product = $this->CI->db->where('name', $record['title'])->get('products')->row();
				if(!$product) {
					$data = array();
					$city = $this->CI->cities_model->find($this->get_random_city_id());	
					if($city) {
						$record['city_id'] = $city->id;
						$record['state_id'] = $city->state_id;
						$record['country_iso'] = $city->country_iso;
						$record['city'] = $city->name;
						$record['state'] = $city->state;
						$record['country'] = $city->country;
					}
					foreach($fields as $field) {
						$field_value = $this->process_field_value($field, $record);					
						if(!empty($field_value)) {
							$data[$field] = $field_value;
						}
					}
					foreach($record as $key => $value) {
						switch($key) {
							case 'title':
								$data['name'] = $value;
								break;
							case 'description':
								$data[$key] = $value;
								break;
							case 'meta_keywords': // to load specific demo type
							    $data[$key] = $value;
							    break;
							case 'price':
							case 'offer_price':
								if(!empty($value)) {
									$data[$key] = $value;
								}
								break;
							case 'highlight':
								$data[$key] = !empty($value) ? 1 : 0;
								break;
						}
					} 
					$row_id = 0;
					$this->CI->db->insert($module, $data);
					$row_id = $this->CI->db->insert_id();
					if($row_id) {
						// save images
						if(!empty($record['images'])) {
							// $this->CI->load->library('goclixy/gimages');
							$i = 1;
							foreach($record['images'] as $url) {
								$filename = substr( $url , strrpos( $url , '/' ) + 1);
								// check the file existence
								if($i == 1) {
									$destination_path = 'assets/images/' . $module;
								} else {
									$filepath = 'assets/images/' . $module;
								}
								
								$filepath = $destination_path . $filename;
								$count = $this->CI->db->where(array('path' => $destination_path, 'original_filename' => $filename))->count_all_results('images');
								if($count == 0) {
									$image_type_id = $i == 1 ? 1 : 0;
									$file_image = explode('.', $filename);
									$images_data[] = array(
										'module_id' => $module_id,
										'row_id' => $row_id,
										'type_id' => $image_type_id,
										'title' => strtoupper(str_replace('-', ' ', $file_image[0])),
										'url' => $url,
										'path' => $destination_path,
										'filename' => $filename,
										'original_filename' => $filename,
										'status' => 1
									);
									// $this->download_image($url, $row_id, null, null, $image_type_id, 20, 'products');
									$i++;
								}
							}
						}
					}
				}
            }	

			// save images data
			if(!empty($images_data)) {
				$this->CI->db->insert_batch('images', $images_data);
				// get all images
				$where_sql = 'id > ' . (!empty($max_id->max_id) ? $max_id->max_id : 0);
				$this->CI->load->library('goclixy/gimages');
				$records = $this->CI->db->where($where_sql)->get('products')->result();
				if($records) {
					foreach($records as $record) {
						$this->CI->gimages->_tweak_images_row($module, $record->id);
					}
				}
			}
        }
	}
	
	/**
	 * Get product categories, brands etc.
	 * @param string $table
	 */
	private function get_product_extras($table) {
	    $result = array();
	    $records = $this->CI->db->select('id, slug')->get($table)->result();
	    if($records) {
	        foreach($records as $record) {
	            $result[$record->slug] = $record->id;
	        }
	    }
	    return $result;
	}
	
	/**
	 * Get ID using SLUG Categories/Brands
	 * @param array $records
	 * @param string $data
	 * @return string
	 */
	private function get_id_from_slug($records, $data) {
	    $result = array();
	    $records_slug = explode(',', $data);
	    foreach($records_slug as $record_slug) {
	        if(!empty($records[$record_slug])) {
	            $result[] = $records[$record_slug];
	        }
	    }
	    return implode(',', $result);
	}
	
	/**
	 * Truncate Images
	 */
	private function truncate_images() {
		$this->_truncate_images('assets/images/products/covers', true);
		$this->_truncate_images('assets/images/products/large', true);
		$this->_truncate_images('assets/images/products', true);
	}
	
	/**
	 * Truncate Images
	 * @param string $folder - Name of the folder to be truncated
	 * @param boolean $has_thumb - True - Delete all images inside the thumbs folder
	 */
	private function _truncate_images($folder = 'assets/images/products', $has_thumb = true) {
		$files = glob(FCPATH . $folder . '/*'); //get all file names
		foreach($files as $file){
			if(is_file($file))
			unlink($file); //delete file
		}
		if($has_thumb) {
			$this->_truncate_images($folder . '/thumbs', false);
		}
	}
    
    /**
     * Insert/Update value
     * @param string $table
     * @param array $data
     * @param string $unique_key
     */
    private function insert_update_value($table, $value, $key = 'name') {
        $row = $this->CI->db->where($key, $value)->get($table)->row();
        if(!$row) {
			// $process_fields = array('slug', 'created_on', 'modified_on');
			$fields = $this->table_columns($table);
            // insert value
			$extra = array($key => $value);
			foreach($fields as $field) {
				$field_value = $this->process_field_value($field, $extra);
				if(!empty($field_value)) {
					$data[$field] = $field_value;
				}
			}
			$data[$key] = $value;
            $this->CI->db->insert($table, $data);
            return $this->CI->db->insert_id();
        }
        return $row->id;
    }
	
	/**
	 * Get Table Columns
	 * @param string $table - Name of the table
	 */
	private function table_columns($table) {	
		$columns = array();	
		$fields = $this->CI->db->field_data($table);
		if(!empty($fields)) {
			foreach($fields as $field) {
				$columns[] = $field->name;
			}
		}
		return $columns;
	}
    
    /**
     * Fill table with random values
     * @param string $table
     * @param number $limit - Number of records to be generated
     */
    private function fill_table($table, $limit = 10) {
        $fields = $this->table_columns($table);
        if(!empty($fields)) {
            $all_data = array();
            for($i = 0; $i <= $limit; $i++) {
                $data = array();
                foreach($fields as $field) {
                    $data[$field] = $this->process_field_value($field);
                }
                $all_data[] = $data;
            }
        }
        return $all_data;
    }
	
	/**
	 * Process Field Values
	 * @param string $field_name
	 */
	private function process_field_value($field_name, $extra = array()) {
		$field_value = '';
		switch($field_name) {
			case 'id':
				break;
			case 'status':
				$field_value = 1;
				break;
			case 'hits':
				$field_value = rand(50, 1000);
				break;
			case 'duration':
				$durations = array(30, 60, 90, 356);
				$field_value = $durations[array_rand($durations, 1)];
				break;
			case 'recurring':
			case 'renewable':
			case 'ticket_support':
				$field_value = 1;
				break;
			case 'user_id':
			case 'seller_id':
			case 'seller_id':
				// get random user id
				$field_value = $this->random_id('users');
				break;
			case 'brand_id':
			case 'brand_ids':
				$field_value = $this->random_id('product_brands');
				break;
			case 'product_id':
			case 'product_ids':
				$field_value = $this->random_id('products');
				break;
			case 'deleted': // skip fields
				break;
			case 'status_id': // default
				$field_value = 2;
				break;
			case 'stock':
				$field_value = rand(100, 500);
				break;
			case 'price':
			case 'offer_price':
			case 'renewal_price':
				$field_value = rand(10, 100);
				break;
			case 'duration':
				break;
			case 'created_ip':
			case 'modified_ip':
			case 'last_viewed_ip':
				$field_value = $this->CI->input->ip_address();
				break;
			case 'created_on':
			case 'modified_on':
			case 'last_viewed_on':
				$field_value = date('Y-m-d H:i:s');
				break;
			case 'city_id':
			case 'state_id':
			case 'country_iso':
				if(!empty($extra[$field_name])) {
					$field_value = $extra[$field_name];
				}
				break;
			case 'slug':
				$field_value = $this->CI->slug->create_slug(!empty($extra['name']) ? $extra['name']: $extra['title']);
				break;
			case 'icon':			
				$icons = array('check-circle', 'shopping-basket', 'palette', 'cookie', 'thumbtack', 'heart', 'star', 'thumbs-up', 'crown', 'bookmark');
				$field_value = 'fas fa-' . $icons[array_rand($icons)];
				break;
			case 'featured':
				$field_value = rand(0, 1);
				break;
		}
		return $field_value;
	}
    
    /**
     * Truncate Tables
	 * @param string $tables - Delete tables individually
     */
    private function truncate_tables($tables = array()) {
		if(empty($tables)) {
			$tables = array('activities', 'ads', 'ad_views', 
				'affiliate_referrals', 'affiliate_referrals_unique', 'affiliate_users', 'affiliate_users_hierarchies',
				'products', 'product_categories', 'product_category_relations', 'product_licenses', 'product_metas', 'product_ratings', 'product_rating_users', 'product_views', 'product_wishlists', 
				// 'product_faqs', 'product_features', 'product_references', 'product_screenshots', 'product_teams', 'product_testimonials',
				'orders', 'order_customers', 'order_coupons', 'order_histories', 'order_invoices', 'order_items',
				// 'subscriber_users', 'subscriber_metas', 'subscriber_messages',
				// 'tickets', 'ticket_attachments', 'ticket_messages', 
				'message_queue', 'message_sent', 'message_templates', 'metas',
				'email_messages', 'email_links', 'email_sent', 'email_queue', 'email_history',
				'contact_queries', 
				// 'blogs', 'blog_categories', 'blog_category_relations', 'blog_comments', 'blog_comment_relations', 'blog_favorites', 'blog_tags', 'blog_tag_relations', 'blog_views',
				// 'forum_categories', 'forum_posts', 'forum_reports', 'forum_threads', 'forum_votes',
				// 'payout_histories', 'payout_users', 'payouts',
				'manual_columns', 'manual_column_details', 'manual_groups', 'manual_links', 'manual_link_relations', 'manual_searches', 'manual_tags', 'manual_topics', 'manual_tutorials',
				'seo_backlinks', 'seo_backlink_emails', 'seo_backlink_products', 'seo_error_links', 'seo_error_link_referrers', 'seo_keywords', 'seo_links', 'seo_text',
				'images', 'tmp_files', 'tmp_images', 'translations', 'visitors', 'wallets_points');
		}
        foreach($tables as $table) {
            $this->_truncate($table);
        }
    }
    
    /**
     * Truncate table
     * @param string $table - Name of the table to truncate
     */
    private function _truncate($table) {
        $sql = 'TRUNCATE TABLE ' . $this->CI->db->dbprefix . $table;
        $this->CI->db->query($sql);
    }
    
    /**
     * Get Random ID from a table
     * @param string $table
     */
    private function random_id($table) {
       $sql = 'SELECT id FROM ' . $this->CI->db->dbprefix . $table . ' ORDER BY RAND() LIMIT 1';
       $record = $this->CI->db->query($sql)->row();
       if($record) {
           return $record->id;
       } else {
           return 0;
       }
    }
    
    /**
     * Get user information from the affiliates table
     */
    private function copy_user_info() {
        
    }
    
    /**
     * Purge Users
     */
    private function purge_users($demo_only = false) {
		$table_name = 'users';
		$where = "role_id != 1"; // AND role_id != 2 AND role_id != 8 AND email != 'user@goclixy.com'
        if($demo_only) {
            $where .= ' AND is_demo = 1';
        }
        $this->CI->db->where($where)->delete($table_name);
        // reset the id counter
        $this->reset_id_counter($table_name);
    }
    
    /**
     * Reset the ID counter to the last increment value
     */
    private function reset_id_counter($table_name) {        
        // set auto increment value to the last id
        $alter_autoinc = "ALTER TABLE " . $this->CI->db->dbprefix . $table_name." AUTO_INCREMENT = 1"; // IMPORTANT
        $this->CI->db->query($alter_autoinc);
    }
    
    /**
     * Generate Random Users
     * @param number $limit - Number of users to be generated
     * @param number $max_users - Do not generate more users if the users table already have more than $max_users in the users table
     */
    private function random_users($limit, $max_users = 50) {
        $users_count = $this->CI->db->count_all_results('users');
        if($users_count < $max_users) {
            $result = api_goclixy('randomuser?limit=' . $limit);
            $result = json_decode($result, true);
            $password = 'goclixy';
            $registration_data = array();
            $unique_images = 1;
            $this->CI->load->model('users/user_model');
            $this->CI->load->model('locations/cities_model');
            $this->CI->load->library('goclixy/gimages');
            if(isset($result['results'])) {
                foreach($result['results'] as $user_data) {
                    $display_name = ucfirst($user_data['name']['first']) . ' ' . ucfirst($user_data['name']['last']);
                    $address = ucwords($user_data['location']['street']);
                    $email = $user_data['email'];
                    // $registration_data['uuid'] = $this->uuid();
                    $registration_data['email'] = $email;
                    $registration_data['username'] = $email;
                    $registration_data['display_name'] = $display_name;
                    $registration_data['slug'] = $this->CI->slug->create_slug($display_name);
                    $registration_data['password'] = $password; //$user_data['login']['password'];
                    $registration_data['postcode'] = $user_data['location']['postcode'];
                    $registration_data['address'] = $address;
                    $registration_data['phone_number'] = $user_data['phone'];
                    $registration_data['is_demo'] = 1;
					$registration_data['profile_image'] = $user_data['picture']['large'];
                    $location = $this->CI->cities_model->find($this->get_random_city_id());
                    if($location) {
                        $registration_data['city_id'] = $location->id;
                        $registration_data['state_id'] = $location->state_id;
                        $registration_data['country_iso'] = $location->country_iso;
                        $registration_data['city'] = $location->name;
                        $registration_data['state'] = $location->state;
                        $registration_data['country'] = $location->country;
                    }
                    $registration_data['active'] = 1;
                    // check email existence first to avoid duplicates
                    $user_count = $this->CI->db->where(array('email' => $registration_data['email']))->count_all_results('users');
                    if($user_count == 0) {
                        $user_id = $this->CI->user_model->insert($registration_data);
                        /*$image_url = $user_data['picture']['large'];
                        // save image
                        if(!empty($image_url)) {
                            $image_name = substr( $image_url , strrpos( $image_url , '/' ) + 1);
                            // check image existence
                            $image_count = $unique_images == 1 ? $this->CI->db->where('profile_image', $image_name)->count_all_results('users') : 0;
                            if($image_count == 0) {
                                $this->download_image($image_url, $user_id);
                                $this->CI->db->where('id', $user_id)->update('users', array('profile_image' => $image_name));
                            }
                        }*/
                    }
                }
            }
        }
    }
    
    /**
     * Users Universally Unique Identifier
     * @return string
     */
    private function uuid() {
        $uuid = generate_uuid();
        $count = $this->CI->db->where('uuid', $uuid)->count_all_results('users');
        if($count == 0) {
            return $uuid;
        } else {
            $this->uuid();
        }
    }
    
    /**
     * Get random city id
     * @return number
     */
    private function get_random_city_id() {        
        // get random city
        // SELECT * FROM rs0d_cities ORDER BY RAND() LIMIT 1
        $random_city_id = rand(1,112001); // rand(11034,13046); // Indian Cities Only // rand(1,112001)
        return $random_city_id;
    }
    
    /**
     * Get random city id
     * @return number
     */
    private function get_random_user() {
        // get random city
        $result = $this->CI->db->query('SELECT * FROM ' . $this->CI->db->dbprefix . 'users ORDER BY RAND() LIMIT 1')->row();
        if($result) {
            return $result;
        }
        return null;
    }
    
    
    /**
     * Download Images Only
     */
    private function download_images_only() {
        $url = 'https://randomuser.me/api/portraits/';
        $people = array('men', 'women');
        $destination_path = FCPATH . 'assets/images/profiles/random';
        foreach($people as $gender) {
            for($i = 1; $i <= 99; $i++) {
                $filename = $i . '.jpg';
                $img_url = $url . $gender . '/' . $filename;
                $this->download_image($img_url, null, $filename, $destination_path . '/' . $gender . '/' . $filename);
            }
        }
    }    
    
    /**
     * Download Image
     */
    private function download_image($url, $row_id = null, $filename = null, $destination_path = null, $type_id = 2, $images_limit = 1, $module = 'users') {
        $curl = false;
        if(empty($filename)) {
            $filename = substr( $url , strrpos( $url , '/' ) + 1);
        }
        if(empty($destination_path)) {
            $destination_path = FCPATH . 'assets/images/temp/' . $filename;
        }
        if($curl) {
            $ch = curl_init($url);
            $fp = fopen($destination_path, 'wb');
            curl_setopt($ch, CURLOPT_FILE, $fp);
            curl_setopt($ch, CURLOPT_HEADER, 0);
            curl_exec($ch);
            curl_close($ch);
            fclose($fp);
        } else {
            file_put_contents($destination_path, file_get_contents($url));
        }
        if(!empty($row_id)) {
            if($module == 'users') {
                $upload_dir = 'assets/images/users';
                $this->CI->gimages->move_resize($filename, $upload_dir, true);
            } else {
                $this->CI->gimages->save_image($module, $row_id, $filename, 1, $type_id, $images_limit);
            }
        }
    }
	
	/**
	 * Translate language strings
	 */
	private function language_translations() {
		// import data
        $result = api_goclixy('ecommerce?type=languages&limit=1000');
		$records = json_decode($result, true);
		if(!empty($records['records'])) {	
			$search_language = 'english';
			$language = 'templates_lang.php';
			$search_keys = array();
			// $search_keys = array('heading_promo01' => 'abc', 'heading_sub_promo01' => 'DDD');
			foreach($records['records'] as $record) {
				$search_keys[$record['title']] = strip_tags($record['description']);
			}
			$this->CI->load->helper('translate/languages');
			$lang_array = load_lang_file($language, $search_language);
			// search in array and return the key
			if($lang_array) {
				$lang_array = array_replace($lang_array, $search_keys);
				save_lang_file($language, $search_language, $lang_array);
			}
		}
	}
	
	/**
	 * Get Sliders/Banners and insert them in the theme_options table
	 * Note: the slug must match with the online demo api data
	 * @param string $type
	 * @param string $slug - Demo Type
	 * e.g. $this->gsample->get_slider_banners('banners', 'fruits-vegetables');
	 */
	public function get_slider_banners($type = 'sliders', $slug = null) {
	    $records = $this->slider_banners($type);
	    if(!empty($slug) && !empty($records[$slug])) {
	        // update data in the theme_options table in the extras column
	        $demo_type = $this->CI->db->where('slug', $slug)->get('theme_options')->row();
	        if($demo_type) {
	            $extras = json_decode($demo_type->extras, true);
	            $extras[$type] = json_encode($records[$slug]);
	            $this->CI->db->where('id', $demo_type->id)->update('theme_options', array('extras' => json_encode($extras)));
	        }
	    }
	}
	
	/**
	 * Save Slider Banners
	 * 
	 * Data is saved in the extras column to retrieve them during demos
	 * 
	 * Run this method using Cronjob to update banners periodically after a specific period
	 * 
	 * @param string $type
	 * @param boolean $clear
	 * 
	 * Usage: $this->gsample->save_slider_banners('all', true);
	 */
	public function save_slider_banners($type = 'sliders', $clear = false) {
	    if($type == 'all') {
	        $types = array('banners', 'sliders');
	    } else {
	        $types = array($type);
	    }
	    if($clear) {
	        foreach($types as $type) {
	            $table = $type == 'sliders' ? 'banner_sliders' : 'banners';
	            $records = $this->CI->db->get($table)->result();
	            $data = array();
	            if($records) {
	                foreach($records as $record) {
	                    $data[] = array('id' => $record->id, 'extras' => null);
	                }
	            }
	            $this->CI->db->update_batch($table, $data, 'id');
	        }
	    } else {
	        foreach($types as $type) {
	            $table = $type == 'sliders' ? 'banner_sliders' : 'banners';
	            $records = $this->slider_banners($type);
	            if($records) {
	                foreach($records as $id => $record) {
	                    $this->CI->db->where('id', $id)->update($table, array('extras' => json_encode($record)));
	                }
	            }
	        }	        
	    }
	}
	
	/**
	 * Sliders/Banners
	 */
	private function slider_banners($type = 'sliders') {
	    // import data
	    $result = api_goclixy('ecommerce?type=' . $type . '&limit=1000');
	    $records = json_decode($result, true);
	    if(!empty($records['records'])) {
	        return $records['records'];
	    }
	    return false;
	}
	
	/**
	 * Use random tags and categories
	 * @param string $table - tags/categories
	 * @param string $module - listings/products/blogs
	 * @param string $name - Universities!!!!!Arts Entertainment -> Note !!!!! is the separator
	 * The user can enter tags/categories names
	 * @return string[]|NULL[]
	 */
	private function tags_categories($table, $module, $name = null) {
	    $where = 'FIND_IN_SET(' . gc_module_id($module) . ', module_relations)';
	    if(!empty($name)) {
	        $name_search = explode('!!!!!', $name);
	        $where_name = array();
	        foreach($name_search as $sname) {
	            // $where_name[] = "FIND_IN_SET('" . $sname . "', name)";
	            $where_name[] = "name = '" . $sname . "'";
	        }
	        $where .= ' AND (' . implode(' OR ', $where_name) . ')';
	    } else if($table == 'categories') {
	        $where .= ' AND parent_id != 0';
	    }
	    $sql = "SELECT * FROM " . $this->CI->db->dbprefix . $table . " WHERE " . $where . " ORDER BY rand() LIMIT 10";
	    $records = $this->CI->db->query($sql)->result();
	    $data = array();
	    if($records) {
	        $rows = array();
	        $set = array();
	        $ids = array();
	        $relation = array();
	        foreach($records as $record) {
	            $relation[] = $record->id;
	            // get parents
	            if($table == 'categories') {
	               $parents = $this->CI->gc->closure(array('where' => $record->id, 'parent' => 1, 'self' => true));
	               if($parents) {
	                   foreach($parents as $parent) {
	                       $rows[] = $parent->name;
	                       $set[] = $parent->id;
	                   }
	               }
	            } else {
	                $rows[] = $record->name;
	                $set[] = $record->id;
	            }
	        }
	        $set = array_unique(array_filter($set));
	        $data[$table . '_set'] = implode(',', $set);
	        $data['relation'] = $relation; // categories_relation table data
	        if($table == 'tags') {
	            $data['keywords'] = implode(', ', $rows);
	        } else {
	            $data['category_id'] = $records[0]->id;
	            $data['categories'] = implode(', ', $rows);
	        }
	    }
	    return $data;
	}
	
	/**
	 * Save information in the relations table
	 * @param string $table - tags/categories
	 * @param string $module
	 * @param string $row_id
	 * @param array $relation
	 */
	private function save_relation($table, $module, $row_id, $relation) {
	    $table = $table == 'tags' ? 'tag_relations' : $table . '_relation';
	    $module_id = gc_module_id($module);
	    $this->CI->db->where(array('module_id' => $module_id, 'row_id' => $row_id))->delete($table);
	    $data = array();
	    foreach($relation as $item) {
	        $key = $table == 'categories_relation' ? 'category_id' : 'tag_id';
	        $data[] = array(
	           'module_id' => $module_id,
	           'row_id' => $row_id,
	            $key => $item
	        );
	    }
	    $this->CI->db->insert_batch($table, $data);
	}
	
	/**
     * Generate random business queries, leads, ads etc.
	 */
	private function random_messages($table) {
	    // get random users
	    $records = $this->CI->db->order_by('RAND()')->get('users')->result();
	    if($records) {
	        $data = array();
	        $datetime = date('Y-m-d H:i:s');
	        $ip_address = $this->input->ip_address();
	        foreach($records as $record) {
	            $row = array(	                
	                'user_id' => $record->id,
	                'name' => $record->display_name,
	                'email' => $record->email,
	                'phone_number' => $record->phone_number,
	                'status_id' => 2,
	                'created_on' => $datetime,
	                'created_ip' => $ip_address,
	                'modified_on' => $datetime,
	                'modified_ip' => $ip_address
	            );
	            switch($table) {
	                case 'business_queries':
	                    $row['module_id'] = 1;
	                    $row['row_id'] = 1;
	                    $row['owner_id'] = 1;
	                    $row['message'] = $message;
	                    $row['resume'] = '';
	                    break;
	                case 'leads':
	                    
	                    break;
	            }
	        }
	    }
	}
}