<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Ads controller
 */
class Ads extends Front_Controller {
	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct() {
		parent::__construct();
        $this->load->model('ads/ads_model');
        $this->load->model('ads/ad_packages_model');
        $this->lang->load('members/members');
        $this->lang->load('ads');
        $this->lang->load('banners/banners');
        $this->lang->load('packages/packages');
		$this->load->library('crypt');
		// Assets::add_js('jquery.cookie.js');
		
		Assets::add_js('moment.js');
		Assets::add_css('bootstrap-datetimepicker.min.css');
		Assets::add_js('bootstrap-datetimepicker.min.js');
	}
	
	/**
	 * Ad Click Counter Handling
	 * 
	 * On Ad row click the user will be redirected to this URL.
	 */
	public function click() {
		if(!empty($_GET['id'])) {
			$this->ads_model->click_impression_counter($_GET['id'], 'click', true);
		} else {
			Template::redirect(site_url());
		}
	}
	
	/**
	 * Display Ads to the User
	 * @param number $offset
	 */
	public function index($offset = 0) {
		$this->auth->restrict('Site.Business.Financial');
		$user_id = $this->auth->user_id();
		// allow user to update the ad status
		if(isset($_POST['submit'])) {			
			$ad_id = $_POST['submit'];
			$result = $this->ads_model->select('status')->find($ad_id);
			if($result->status != 1) {
			    $ad_status = 1;
				// check wallet balance
				$balance = $this->db->select('wallets')->get_where('users', array('id' => $user_id))->row();
				if($balance && $balance->wallets <= 0) {
					Template::set_message(lang('insufficient_funds'), 'error');
					Template::redirect(current_url());
				}
			} else {
			    $ad_status = 5; // Ad paused by the user
			}
			// update status
			$sql = "UPDATE " . $this->db->dbprefix . "ads SET status = " . $ad_status . " WHERE id = " . $_POST['submit'];
			$this->db->simple_query($sql);
			Template::redirect(current_url());
		}
		if (isset($_POST['delete'])) {
			$ad_id = intval($_POST['ad_id']);
			if($ad_id != 0) {
				$this->ads_model->delete($ad_id);
				Template::redirect(current_url());
			}
		}
		$limit  = $this->limit;
		$where = "A.user_id = " . $user_id ." AND A.deleted = 0";
		$sort_by = "A.id DESC";
		$records = $this->ads_model->get_ads(0, $where, $sort_by, $offset, $limit);
		$total_records = $this->ads_model->get_ads(1, $where, $sort_by, $offset, $limit);

		$pagerUriSegment = 2;
		$pagerBaseUrl = gc_url_format('ads');
	
		$this->load->library('pagination');
		$pager['base_url']    = $pagerBaseUrl;
		$pager['total_rows']  = $total_records;
		$pager['per_page']    = $limit;
		$pager['uri_segment'] = $pagerUriSegment;
	
		$this->pagination->initialize($pager);
	
		Template::set('records', $records);

		Template::set('toolbar_title', lang('label_ads'));
		Template::set('page_title', lang('label_ads'));
		Template::set('title', lang('label_ads'));

		$layout = $this->gc->load_assets('members/ads');
		Template::set_view($layout['view']);
		Template::render($layout['layout']);
	}
	
	/**
	 * Display Ad Views to the User
	 *
	 * @param number $ad_id
	 * @param number $offset
	 */
	public function view($ad_id, $offset = 0) {	
	    $user_id = $this->auth->user_id();
	    
	    $this->ads_model->view($ad_id, $user_id, $this->limit, $offset, 'member');
	    // This method has no View in the Themes Module, i.e. load layout files manually
	    $this->gc->demo_custom_css_js('member');
	    Template::render('member');	    
	}
	
	/**
	 * Display Ad Packages to the user
	 */
	public function packages() {
		$this->auth->restrict();
		// Assets::add_module_css('ads', 'ads.css');
		// Assets::add_css('members/pricing.css');
		// Assets::add_css('members/pricing.js');
		// $ad_packages = $this->ad_packages_model->select('id, title, description, pricing_type, price, duration, highlight')->find_all_by(array('status' => 1));
		// Template::set('ad_packages', $ad_packages);
		$title = lang('label_ad') . ' ' . lang('label_packages');
		Template::set('toolbar_title', $title);
		Template::set('page_title', $title);
		Template::set('title', $title);
		
		$layout = $this->gc->load_assets('members/pricing');
		Template::set_view($layout['view']);
		Template::render($layout['layout']);
	}
	
	/**
	 * Allow user to Add a Banner
	 * @param $user_id - Allow admin to subscribe on behalf of other user
	 */
	public function advertise() {
		$this->ads_model->advertise('member');
	}
	
	/**
	 * Allow user to modify an existing banner
	 */
	public function edit() {
		$this->ads_model->edit('member');
	}
	
	/**
	 * Ajax Display Cost and Bid information to the user
	 *
	 * @param string $module
	 * @param string $q
	 */
	public function bid_info($module, $type, $q) {
	    $q = urldecode($q); // special characters of other languages e.g. Russian will not be processed without this line
		header('Content-Type: application/x-json; charset=utf-8');
		$result = $this->ads_model->bid_info($module, $q, (int) $type);
		echo json_encode($result);
	}
	
	/**
	 * Ajax Function to load keywords on title selection
	 * @param int $id
	 * @param int $module_id
	 * @param int $type - CPC/CPD
	 */
	public function load_keywords($id, $module_id, $type) {
		$result = array();
		$tags_result = array();
		$categories_result = array();
		$cities_result = array();
		$id = (int) $id;
		$type = (int) $type;
		$module_id = (int) $module_id;
		if($id == 0 && $module_id == 0 && $type > 1) {
			echo json_encode($result);
		}		
		$table = gc_module_name($module_id);
		switch($table) {
			case 'listings':
				$this->load->model('listings/listings_model');
				$cities = array();
				$tags = array();
				// get city id from the listing table
				$listing = $this->listings_model->select('city_id')->find($id);
				if($listing) {
					// get city information from the branch offices table
					$cities[0] = $listing->city_id;
					$branches = $this->db->select('city_id')->get_where('branches', array('listing_id' => $id))->result();
					if($branches) {
						foreach($branches as $branch) {
							$cities[] = $branch->city_id;
						}
					}
					if(!empty($cities)) {
						$cities_result = $this->ads_model->bid_info('cities', null, $type, implode(',',$cities));
					}
				}
				break;
			default:				
				if(gc_module_settings($table, 'locations') == 1) {
					$module_info = $this->db->select('city_id')->get_where($table, array('id' => $id))->row();
					if($module_info) {
						$cities_result = $this->ads_model->bid_info('cities', null, $type, $module_info->city_id);
					}
				}
		}
		$categories_result = $this->ads_model->module_category_info($id, 'categories_relation', $type, $table);
		$this->load->model('tags/tags_model');
		$module_tags = $this->tags_model->get_tag_ids($table, $id);
		if($module_tags) {
			foreach($module_tags as $tag) {
				$tags[] = $tag->tag_id;
			}
		}
		if(!empty($tags)) {
			$tags_result = $this->ads_model->bid_info('tags', null, $type, implode(',',$tags));
		}
		$result = array(
			'tags' => $tags_result,
			'cities' => $cities_result,
			'categories' => $categories_result
		);
		echo json_encode($result);
	}
	
	/**
	 * Ad Space
	 */
	public function space() {
	    if(!$this->auth->is_logged_in()) {
	        Template::redirect(gc_url_format('login'));
	    }
	    $redirect_url = site_url();
	    $params = !empty($_GET['buy']) ? $_GET['buy'] : null;
	    if(empty($params)) {
	        Template::redirect($redirect_url);
	    }
	    $this->load->library('crypt');
	    $params = $this->crypt->decode($params);
	    $params = json_decode($params ?? '', true);
	    if(empty($params['width']) || empty($params['height']) || empty($params['price']) || empty($params['title'])) {
	        Template::redirect($redirect_url);
	    }
	    $ad_package_title = $params['title'] . ' ' . $params['width'] . 'x' . $params['height'];
	    // check for the ad package by name
	    $ad_package = $this->db->where("title = '" . $ad_package_title . "'")->get('ad_packages')->row();
	    if(!$ad_package) {
	        // add an ad package
	        // check for a banner type
	        $where = array('width' => $params['width'], 'height' => $params['height'], 'type' => 'image');
	        $banner_type = $this->db->where($where)->get('banner_types')->row();
	        if(!$banner_type) {
	            $banner_type_data = $where;
	            $banner_type_data['title'] = $ad_package_title;
	            $banner_type_data['filesize'] = 500;
	            $banner_type_data['description'] = $banner_type_data['title'];
	            $this->db->insert('banner_types', $banner_type_data);
	            $banner_type_id = $this->db->insert_id();
	            if($banner_type_id == 0) {
	                Template::redirect($redirect_url);
	            }
	            $banner_type = $this->db->where('id', $banner_type_id)->get('banner_types')->row();
	        }
	        if($banner_type) {
	            $module_ids = array(201 => 1);
	            foreach(gc_modules() as $module_id => $module) {
	                $module_ids[gc_module_id($module)] = 1;
	            }
	            $ad_title = $params['title'];
	            $duration = 30;
	            $ad_package_description = <<<EOL
<ul>
<li>{$ad_title} Ad</li>
<li>Unlimited Impressions</li>
<li>Unlimited Clicks</li>
<li>Random Basis</li>
<li>Valid for {$duration} Days</li>
</ul>
EOL;
	            // insert data in the ad_packages table
	            $ad_package_data = array(
	                'module_id' => gc_module_id('banners'),
	                'title' => $ad_package_title,
	                'description' => $ad_package_description,
	                'icon' => $this->categories_model->random_icon(),
	                'order_status' => 1,
	                'status' => 1,
	                'pricing_type' => 'fixed',
	                'price' => $params['price'],
	                'duration' => $duration,
	                'days_before_next_invoice' => 7,
	                'invoice_expiry' => 7,
	                'invoice_first_reminder' => 1,
	                'invoice_second_reminder' => 1,
	                'invoice_third_reminder' => 1,
	                'banner_type_id' => $banner_type->id,
	                'banner_group_ids' => !empty($params['group_id']) ? $params['group_id'] : null,
	                'max_impressions' => 0,
	                'max_clicks' => 0,
	                'module_ids' => json_encode($module_ids),
	            );
	            $this->db->insert('ad_packages', $ad_package_data);
	            $ad_package_id = $this->db->insert_id();
	            if($ad_package_id == 0) {
	                Template::redirect($redirect_url);
	            }
	            Template::redirect(gc_url_format('advertise') . '?id=' . $this->crypt->encode($ad_package_id));
	        }
	    } else {
	        Template::redirect(gc_url_format('advertise') . '?id=' . $this->crypt->encode($ad_package->id));
	    }
	}
}