<?php defined('BASEPATH') || exit('No direct script access allowed');

/**
 * Financial controller
 */
class Financial extends Admin_Controller
{
    protected $permissionCreate = 'Ads.Financial.Create';
    protected $permissionDelete = 'Ads.Financial.Delete';
    protected $permissionEdit   = 'Ads.Financial.Edit';
    protected $permissionView   = 'Ads.Financial.View';

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
        
        $this->auth->restrict($this->permissionView);
        $this->lang->load('ads');
        $this->lang->load('banners/banners');
        $this->lang->load('packages/packages');
        $this->load->model('ads/ads_model');
        $this->load->model('ads/ad_packages_model');
        $this->load->model('packages/packages_model');
        $this->load->model('banners/banners_model');
        
        Assets::add_js('tinymce/tinymce.min.js');
        $this->form_validation->set_error_delimiters("<span class='error'>", "</span>");
        
        Template::set_block('sub_nav', 'financial/_sub_nav');

        Assets::add_module_js('ads', 'ads.js');
    }
    
    /**
     * Display a list of Ads data.
     * 
     * Ad will have 4 status
     * 0 - Inactive
     * 1 - Active
     * 2 - Waiting for Admin Approval
     * 3 - Expired
     * The user can only change status 0 and 1
     *
     * @return void
     */
    public function index($filter='all', $offset = 0) {
    	// Do we have any actions?
    	if (isset($_POST['submit'])) {
    	    $this->gcrud->stats(array('module' => 'ads'));
    		$action = $this->input->post('submit');
    		$checked = $this->input->post('checked');
    		if($checked) {
    			$module_name = count($checked) == 1 ? lang('label_ad') : lang('label_ads') ;
    			switch(strtolower($action)) {
    				case 'activate':
    					$this->update_ad_status($checked, 1);
    					Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_activated')), 'success');
    					break;
    						
    				case 'deactivate':
    					$this->update_ad_status($checked, 99);
    					Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_deactivated')), 'success');
    					break;
    						
    				case 'approval':
    					$this->update_ad_status($checked, 2);
    					Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('marked_approval')), 'success');
    					break;
    				
    				case 'expired':
    					$this->update_ad_status($checked, 4);
    					Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('marked_expired')), 'success');
    					break;
    						
    				case 'delete':
    					$this->update_ad_status($checked, 0);
    					$this->gc->update_status('ads', $checked, 'deleted', 1);
    					Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_deleted')), 'success');
    					break;
    						
    				case 'purge':
    					foreach($checked as $pid) { // delete Ad and its related Keywords and Views completely from the database
    						$this->ads_model->purge_ad($pid);
    					}
    					Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_deleted')), 'success');
    					break;
    						
    				case 'restore':
    					$this->gc->update_status('ads', $checked, 'deleted', 0);
    					Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_restored')), 'success');
    					break;
    			}
    		}
    	}
    	
    	// Change Status
    	if (isset($_POST['change_status']))	 {
    	    $this->gcrud->stats(array('module' => 'ads'));
    		$new_status = (int) $_POST['ad_status_' . $_POST['change_status']];
    		$new_status = ($new_status == 0 ? 99 : $new_status); 
    		$this->ads_model->update_ad_status($_POST['change_status'], $new_status);
    		Template::set_message(sprintf(lang('msg_action_success'), lang('label_ad'), lang('label_updated')), 'success');
    	}
    	
    	$where = "A.deleted = 0";
    	// Filters
    	$filter_type = $filter;
    	
    	switch($filter_type) {
    		case 'active':
    			$where .= " AND A.status = 1";
    			break;
    				
    		case 'inactive':
    			$where .= " AND A.status = 0";
    			break;
    				
    		case 'approval':
    			$where .= " AND A.status = 2";
    			break;
    				
    		case 'expired':
    			$where .= " AND A.status = 4";
    			break;
    	
    		case 'deleted':
    			$where = "A.deleted = 1";
    			break;
    	
    		case 'all':
    			// Nothing to do
    			break;
    	
    		default:
    			show_404(SITE_AREA . '/financial/ads/index/' . $filter .'/');
    	}    	
    	
    	// Search Ad
    	$searchterm = $this->gc->search('ads', $this->uri->segment(6), '/financial/ads/index/' . $filter . '/');
    	if(!empty($searchterm)) {
    		$where .= " AND A.id LIKE '%". $searchterm ."%' OR A.start_date LIKE '%". $searchterm ."%' OR A.end_date LIKE '%". $searchterm ."%' OR AP.title LIKE '%". $searchterm ."%' OR U.display_name LIKE '%". $searchterm ."%' OR U.email LIKE '%". $searchterm ."%'";
    	}
    	
    	// Order By
    	$sort_by = "A.id DESC";
    	$order_by = $this->gc->sort('ads');
    	if(!empty($order_by)) {
    		$sort_by = $order_by;
    	} 	
    	
    	$limit  = $this->settings_lib->item('site.list_limit') ?: 15;    	
    	$records = $this->ads_model->get_ads(0, $where, $sort_by, $offset, $limit);
    	$total_records = $this->ads_model->get_ads(1, $where, $sort_by, $offset, $limit);
    
    	$this->load->library('pagination');
    	$pagerUriSegment = 6;
    	$pagerBaseUrl = site_url(SITE_AREA . '/financial/ads/index/' . $filter .'/');
    	$pager['base_url']    = $pagerBaseUrl;
    	$pager['total_rows']  = $total_records;
    	$pager['per_page']    = $limit;
    	$pager['uri_segment'] = $pagerUriSegment;
    
    	$this->pagination->initialize($pager);
    	$this->ads_model->limit($limit, $offset);
    
    	Template::set('index_url', site_url(SITE_AREA .'/financial/ads/index/'));
    	Template::set('filter_type', $filter_type);
    	Template::set('records', $records);
    
    	Template::set('toolbar_title', lang('ads_manage'));
    
    	Template::render();
    }
    
    /**
     * Update Ad Status
     * @param array $checked
     * @param int $status
     */
    private function update_ad_status($checked, $status) {
    	foreach($checked as $id) {
    		$this->ads_model->update_ad_status($id, $status);
    	}
    }
    
    /**
     * Allow user to Create a new Ad
     * @param $user_id - Allow admin to subscribe on behalf of other user
     */
    public function create() {    	
    	// user search js and css dependencies
    	Assets::add_css('selectize.bootstrap3.css');
    	Assets::add_js('selectize.min.js');
    	Assets::add_js('selectize-search.js');
    	if (isset($_POST['create'])) {
    	    $this->gcrud->stats(array('module' => 'ads'));
    		$this->auth->restrict($this->permissionCreate);
    		$this->load->library('crypt');
    		$param = '&id=' . $this->crypt->encode($this->input->post('package_id'));
    		if($this->input->post('user_id')) {
    			$param .= '&user_id=' . (int) $this->input->post('user_id');
    		}
    		Template::redirect(gc_url_format('ads/advertise') . '?role=admin' . $param);
    	}
    	$packages = $this->ad_packages_model->find_all();
    	Template::set('packages', $packages);
    	Template::set('toolbar_title', lang('create_ad'));
    	Template::render();
    }
    
    /**
     * Allow user to modify an existing banner
     */
    public function edit() {
    	// Assets::add_css('bootstrap.min.css');
    	$id = $this->uri->segment(5);
    	$this->load->library('crypt');
    	if (empty($id)) {
    		Template::set_message(lang('invalid_id'), 'error');    	
    		redirect(SITE_AREA . '/financial/ads/packages');
    	}
    	Template::redirect(gc_url_format('ads/edit') . '?role=admin&id=' . $this->crypt->encode($id));
    }
    
    /**
     * Display Ad related keywords to the user
     * @param string $filter
     * @param int $ad_id
     */
    public function keywords($ad_id = 0, $filter = 'categories', $offset = 0) {
    	if(empty($ad_id)) {
    		Template::redirect(site_url(SITE_AREA .'/financial/ads/index/'));
    	}
    	
    	// Do we have any actions?
    	if (isset($_POST['submit'])) {
    	    $this->gcrud->stats(array('module' => 'ads'));
    		$action = $this->input->post('submit');
    		$checked = $this->input->post('checked');
    		if($checked) {
    			$module_name = count($checked) == 1 ? lang('label_keyword') : lang('label_keywords') ;
    			switch(strtolower($action)) {
    				case 'activate':
    					$this->gc->update_status('ad_keywords', $checked, 'status', 1);
    					Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_activated')), 'success');
    					break;
    	
    				case 'deactivate':
    					$this->gc->update_status('ad_keywords', $checked, 'status', 0);
    					Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_deactivated')), 'success');
    					break;
    				
    				case 'disapprove':
    					$this->gc->update_status('ad_keywords', $checked, 'status', 2);
    					Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_disapproved')), 'success');
    					break;
    	
    				case 'delete':
    					foreach($checked as $pid) { // delete Ad and its related Keywords and Views completely from the database
    						$this->ads_model->purge_keyword($pid);
    					}
    					Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_deleted')), 'success');
    					break;
    			}
    		}
    	}
    	 
    	// Change Status
    	if (isset($_POST['change_status']))	 {
    	    $this->gcrud->stats(array('module' => 'ads'));
    		$new_status = (int) $_POST['status_' . $_POST['change_status']];
    		$new_status = $new_status == 99 ? 0 : $new_status;
    		$this->db->where('id', $_POST['change_status'])->update('ad_keywords', array('status' => $new_status));
    		Template::set_message(sprintf(lang('msg_action_success'), lang('label_keyword'), lang('label_updated')), 'success');
    	}
    	
    	$where = "AK.ad_id = " . $ad_id;
    	// Filters
    	$filter_type = $filter;
    	 
    	switch($filter_type) {
    		case 'categories':
    			$where .= " AND module_id = " . gc_module_id('categories');
    			break;
    	
    		case 'cities':
    			$where .= " AND module_id = " . gc_module_id('cities');
    			break;
    	
    		case 'tags':
    			$where .= " AND module_id = " . gc_module_id('tags');
    			break;
    			 
    		default:
    			show_404(SITE_AREA . '/financial/ads/keywords/' . $ad_id . '/' . $filter .'/');
    	}
    	
    	// Search Keyword
    	$searchterm = $this->gc->search('ad_keywords_' . $filter, $this->uri->segment(6), '/financial/ads/keywords/' . $ad_id . '/' . $filter . '/');
    	if(!empty($searchterm)) {
    		$where .= " AND (AK.id LIKE '%". $searchterm ."%' OR M.id LIKE '%". $searchterm ."%' OR M.name LIKE '%". $searchterm ."%')";
    	}
    	 
    	// Order By
    	$sort_by = "id DESC";
    	$order_by = $this->gc->sort('ad_keywords');
    	if(!empty($order_by)) {
    		$sort_by = $order_by;
    	}
    	 
    	$limit  = $this->settings_lib->item('site.list_limit') ?: 15;
    	$records = $this->ads_model->get_saved_keywords_admin($ad_id, $filter, 0, $where, $sort_by, $offset, $limit);
    	$total_records = $this->ads_model->get_saved_keywords_admin($ad_id, $filter, 1, $where, $sort_by, $offset, $limit);

    	$this->load->library('pagination');
    	$pagerUriSegment = 7;
    	$pagerBaseUrl = site_url(SITE_AREA . '/financial/ads/keywords/' . $ad_id . '/' . $filter .'/');
    	$pager['base_url']    = $pagerBaseUrl;
    	$pager['total_rows']  = $total_records;
    	$pager['per_page']    = $limit;
    	$pager['uri_segment'] = $pagerUriSegment;
    	
    	$this->pagination->initialize($pager);
    	
    	Template::set('index_url', site_url(SITE_AREA .'/financial/ads/keywords/' . $ad_id .'/'));
    	Template::set('filter_type', $filter_type);
    	Template::set('records', $records);
    	
    	Template::set('toolbar_title', sprintf(lang('view_keywords'), $ad_id));
    	
    	Template::render();
    }

    /**
     * Display a list of Ad Packages
     *
     * @return void
     */
    public function packages($offset = 0)
    {
    	// Highlight Package
    	if (isset($_POST['highlight'])) {
    	    $this->gcrud->stats(array('module' => 'ads'));
    		$this->packages_model->highlight($_POST['highlight'], 'ad_packages', 1);
    	}
    	
        // Deleting anything?
        if (isset($_POST['delete'])) {
            $this->gcrud->stats(array('module' => 'ads'));
            $this->auth->restrict($this->permissionDelete);
            $checked = $this->input->post('checked');
            if (is_array($checked) && count($checked)) {

                // If any of the deletions fail, set the result to false, so
                // failure message is set if any of the attempts fail, not just
                // the last attempt

                $result = true;
                foreach ($checked as $pid) {
                    $deleted = $this->ads_model->purge_package($pid);
                    if ($deleted == false) {
                        $result = false;
                    }
                }
                if ($result) {
                    Template::set_message(count($checked) . ' ' . sprintf(lang('msg_action_success'), lang('label_packages'), lang('label_deleted')), 'success');
                } else {
                    Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_packages')) . $this->ad_packages_model->error, 'danger');
                }
            }
        }
        
        // Do we have any actions?
        if (isset($_POST['submit'])) {
            $this->gcrud->stats(array('module' => 'ads'));
        	$action = $this->input->post('submit');
        	$checked = $this->input->post('checked');
        	if($checked) {
        		$module_name = count($checked) == 1 ? lang('label_package') : lang('label_packages') ;
        		switch(strtolower($action)) {
        			case 'activate':
        				$this->gc->update_status('ad_packages', $checked, 'status', 1);
        				Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_activated')), 'success');
        				break;
        				 
        			case 'deactivate':
        				$this->gc->update_status('ad_packages', $checked, 'status', 0);
        				Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_deactivated')), 'success');
        				break;
        				 
        			case 'delete':
        				foreach($checked as $pid) {
        					$this->ads_model->purge_package($pid);
        				}
        				Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_deleted')), 'success');
        				break;
        		}
        	}
        }
        
        // Update Status
        if (isset($_POST['update_status'])) {
            $this->gcrud->stats(array('module' => 'ads'));
        	$this->update_status('ad_packages', $_POST['update_status']);
        	Template::set_message(sprintf(lang('msg_status_changed'), lang('label_package')), 'success');
        }        
        
        $pagerUriSegment = 5;
        $pagerBaseUrl = site_url(SITE_AREA . '/financial/ads/packages') . '/';
        
        $limit  = $this->settings_lib->item('site.list_limit') ?: 15;

        $this->load->library('pagination');
        $pager['base_url']    = $pagerBaseUrl;
        $pager['total_rows']  = $this->ad_packages_model->count_all();
        $pager['per_page']    = $limit;
        $pager['uri_segment'] = $pagerUriSegment;

        $this->pagination->initialize($pager);
        $this->ad_packages_model->limit($limit, $offset);
        
        // Sort By
        $order_by = $this->gc->sort('ad_packages');
        if(!empty($order_by)) {
        	$this->ad_packages_model->order_by($this->session->userdata('ad_packages_sort')['column'], $this->session->userdata('ad_packages_sort')['order']);
        } else {
        	$this->ad_packages_model->order_by('id', 'ASC');
        }        
        $records = $this->ad_packages_model->find_all();

        Template::set('records', $records);        
    	Template::set('toolbar_title', lang('ads_manage'));
        Template::render();
    }
    
    /**
     * Create an Ads package.
     *
     * @return void
     */
    public function create_package() {
        $this->auth->restrict($this->permissionCreate);        
        Assets::add_css('selectize.bootstrap3.css');
        Assets::add_js('selectize.min.js');
        Assets::add_js('selectize-search.js');
        // Fontawesome Icon Picker
        Assets::add_css('fontawesome-iconpicker.min.css');
        Assets::add_js('fontawesome-iconpicker.min.js');
        // Color Picker
        Assets::add_css('bootstrap-colorpicker.min.css');
        Assets::add_js('bootstrap-colorpicker.min.js');
        Assets::add_js('$(".gc-colorpicker").colorpicker();', 'inline');
        
        if (isset($_POST['save'])) {
            $this->gcrud->stats(array('module' => 'ads'));
            if ($insert_id = $this->save_ad_package()) {
                log_activity($this->auth->user_id(), sprintf(lang('msg_save_success'), lang('label_package')) . ': ' . $insert_id . ' : ' . $this->input->ip_address(), 'ads');
                Template::set_message(sprintf(lang('msg_save_success'), lang('label_package')), 'success');

                redirect(SITE_AREA . '/financial/ads/packages');
            }

            // Not validation error
            if ( ! empty($this->ad_packages_model->error)) {
                Template::set_message(sprintf(lang('msg_create_failure'), lang('label_package')) . $this->ad_packages_model->error, 'danger');
            }
        }
        $banner_types = $this->banners_model->banner_types();
        Template::set('banner_types', $banner_types);
        Template::set('modules', gc_modules());
        Template::set('toolbar_title', lang('create_package'));
        Template::set_view('package_form');

        Template::render();
    }
    /**
     * Allows editing of Ads data.
     *
     * @return void
     */
    public function edit_package() {
        Assets::add_css('selectize.bootstrap3.css');
        Assets::add_js('selectize.min.js');
        Assets::add_js('selectize-search.js');
        // Fontawesome Icon Picker
    	Assets::add_css('fontawesome-iconpicker.min.css');
    	Assets::add_js('fontawesome-iconpicker.min.js');
    	// Color Picker
    	Assets::add_css('bootstrap-colorpicker.min.css');
    	Assets::add_js('bootstrap-colorpicker.min.js');
    	Assets::add_js('$(".gc-colorpicker").colorpicker();', 'inline');
    	
        $id = $this->uri->segment(5);
        if (empty($id)) {
            Template::set_message(lang('invalid_id'), 'error');

            redirect(SITE_AREA . '/financial/ads/packages');
        }
        
        if (isset($_POST['save'])) {
            $this->gcrud->stats(array('module' => 'ads'));
            $this->auth->restrict($this->permissionEdit);

            if ($this->save_ad_package('update', $id)) {
				// Clear trusted seal
				$this->packages_model->clear_trusted_seal($id, 'ad_packages');	
				// Log the activity
                log_activity($this->auth->user_id(), sprintf(lang('msg_save_success'), lang('label_package')) . ': ' . $id . ' : ' . $this->input->ip_address(), 'ads');
                // Display message
                Template::set_message(sprintf(lang('msg_save_success'), lang('label_package')), 'success');
                // Redirect user
                Template::redirect(current_url());
            }

            // Not validation error
            if ( ! empty($this->ad_packages_model->error)) {
                Template::set_message(sprintf(lang('msg_edit_failure'), lang('label_package')) . $this->ad_packages_model->error, 'danger');
            }
        } elseif (isset($_POST['delete'])) {
            $this->gcrud->stats(array('module' => 'ads'));
            $this->auth->restrict($this->permissionDelete);

            if ($this->ads_model->purge_package($id)) {
            	// Purge all relations of this ad_package from module_relations table
            	$this->gc->purge_relation($id, 'ads');
                log_activity($this->auth->user_id(), sprintf(lang('msg_action_success'), lang('label_packages'), lang('label_deleted')) . ': ' . $id . ' : ' . $this->input->ip_address(), 'ads');
                Template::set_message(sprintf(lang('msg_action_success'), lang('label_packages'), lang('label_deleted')), 'success');

                redirect(SITE_AREA . '/financial/ads/packages');
            }

            Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_packages')) . $this->ad_packages_model->error, 'danger');
        }
        
        $banner_types = $this->banners_model->banner_types();
        Template::set('banner_types', $banner_types);
        
        $record = $this->ad_packages_model->find($id);
        
        // Multi Langauge Fields - Get translations if exists
        $record = $this->translations->get_translations('ad_packages', $id, $record);
        
        Template::set('record', $record);
        
        $relations = $this->gc->get_relations($id, 'ads');
        Template::set('relations', $relations);
        Template::set('modules', gc_modules());

        Template::set('toolbar_title', lang('edit_package'));
        Template::set_view('package_form');
        Template::render();
    }

    //--------------------------------------------------------------------------
    // !PRIVATE METHODS
    //--------------------------------------------------------------------------

    /**
     * Save the data.
     *
     * @param string $type Either 'insert' or 'update'.
     * @param int    $id   The ID of the record to update, ignored on inserts.
     *
     * @return boolean|integer An ID for successful inserts, true for successful
     * updates, else false.
     */
    private function save_ad_package($type = 'insert', $id = 0)
    {
        if ($type == 'update') {
            $_POST['id'] = $id;
        }        
		$module_id = $this->input->post('module_id');
		
		
		// validate translation fields
		$this->translations->validate_prepare_fields();
		
        // Validate the data
		$this->form_validation->set_rules('title', 'lang:label_title', 'required|trim|alpha_extra|max_length[100]');
		$this->form_validation->set_rules('description', 'lang:label_description', 'required|trim');
		$this->form_validation->set_rules('icon', 'lang:label_icon', 'trim');
		$this->form_validation->set_rules('price', 'lang:label_price', 'required|trim|is_decimal');
		$this->form_validation->set_rules('duration', 'lang:label_duration', 'trim|integer|max_length[5]');
		$this->form_validation->set_rules('tags', 'lang:label_tags', 'trim');
		$this->form_validation->set_rules('categories', 'lang:label_categories', 'trim');
		$this->form_validation->set_rules('cities', 'lang:cities', 'trim');
		$this->form_validation->set_rules('highlight', 'lang:highlight', 'trim');
		$this->form_validation->set_rules('order_status', 'lang:order_status', 'trim');		
		$this->form_validation->set_rules('display_order', 'lang:label_order', 'trim|required|is_numeric');
		$this->form_validation->set_rules('priority', 'lang:label_priority', 'trim|required|max_length[5]|integer');
		$this->form_validation->set_rules('status', 'lang:label_status', 'trim');	
		// Invoicing
		$this->form_validation->set_rules('days_before_next_invoice', 'lang:days_before_next_invoice', 'trim|is_numeric');
		$this->form_validation->set_rules('invoice_expiry', 'lang:invoice_expiry', 'trim|is_numeric');
		$this->form_validation->set_rules('invoice_first_reminder', 'lang:invoice_first_reminder', 'trim|is_numeric');
		$this->form_validation->set_rules('invoice_second_reminder', 'lang:invoice_second_reminder', 'trim|is_numeric');
		$this->form_validation->set_rules('invoice_third_reminder', 'lang:invoice_third_reminder', 'trim|is_numeric');
		// Color Schemes - Applicable only in case of Featured
		$this->form_validation->set_rules('color_scheme', 'lang:label_color_scheme', 'trim');
		$this->form_validation->set_rules('border_color', 'lang:label_border_color', 'trim');
		$this->form_validation->set_rules('clear_trusted_seal', 'lang:clear_trusted_seal', 'trim');
		
        if($module_id == 15) { // Banner
        	$this->form_validation->set_rules('banner_type_id', 'lang:label_banner_type', 'trim|integer');
        	$this->form_validation->set_rules('max_impressions', 'lang:label_max_impressions', 'required|trim|is_numeric');
        	$this->form_validation->set_rules('max_clicks', 'lang:label_max_clicks', 'required|trim|is_numeric');
        	$this->form_validation->set_rules('module_ids[]', 'lang:applicable_modules', 'required|is_array');
        	$this->form_validation->set_rules('categories_include', 'lang:bn_categories_in', 'trim|max_length[455]');
        	$this->form_validation->set_rules('categories_exclude', 'lang:bn_categories_ex', 'trim|max_length[455]');
        	$this->form_validation->set_rules('cities_include', 'lang:bn_cities_in', 'trim|max_length[455]');
        	$this->form_validation->set_rules('cities_exclude', 'lang:bn_cities_ex', 'trim|max_length[455]');
        	$this->form_validation->set_rules('links_include', 'lang:bn_links_in', 'trim|max_length[455]');
        	$this->form_validation->set_rules('links_exclude', 'lang:bn_links_ex', 'trim|max_length[455]');
        }
        
        $this->form_validation->set_rules('uploaded_image', 'lang:label_trusted_seal', 'trim');
        //Check whether the image is uploaded or not
        if(isset($_FILES['trusted_seal']) && $_FILES['trusted_seal']['size'] > 0) {        	
        	$file_data = $this->packages_model->upload_trusted_seal();
        	if(!$file_data) {
        		$_POST['file_upload'] = 'NONE';
        		$this->form_validation->set_rules('file_upload', 'label:label_trusted_seal', 'valid_file_upload');
        	}
        	// Avoid image upload process, if the form has validation issues with other fields
        	$_POST['uploaded_image'] = isset($file_data['upload_data']['file_name']) ? $file_data['upload_data']['file_name'] : '';
        }
        
        if ($this->form_validation->run() === false) {
        	return false;
        }
        
        // unset non existing translation POST fields
        $this->translations->translate_unset();
        
        // Make sure we only pass in the fields we want
        $data = $this->ad_packages_model->prep_data($this->input->post());
        
        $data['tags']				= $this->input->post('tags') ? 1 : 0;
        $data['tags_bid']			= $this->input->post('tags') && $this->input->post('tags_bid') ? 1 : 0; // save tags bid only if the tags checkbox is selected
        $data['categories']			= $this->input->post('categories') ? 1 : 0;
        $data['categories_bid']		= $this->input->post('categories') && $this->input->post('categories_bid') ? 1 : 0; // save categories bid only if the categories checkbox is selected
        $data['cities']				= $this->input->post('cities') ? 1 : 0;       
        $data['cities_bid']			= $this->input->post('cities') && $this->input->post('cities_bid') ? 1 : 0; // save cities bid only if the cities checkbox is selected
       // $data['highlight']		= $this->input->post('highlight') ? 1 : 0;
        $data['order_status']		= $this->input->post('order_status') ? 1 : 0;
        $data['status']				= $this->input->post('status') ? 1 : 0;
		$data['trusted_seal'] 		= $this->input->post('uploaded_image');
		
		// Banners
		$data['module_ids']         = json_encode($this->input->post('module_ids'));
        
        // Disable Cities for the following modules
        // tags and cities are not applicable for all modules
        // but they can also be used for all modules
        // e.g. if a user clicked the tag id 1 then the banner should display for that tag
       /* switch($module_id) {
        	case gc_module_id('videos'):
        		$data['cities'] = 0;
        		$data['cities_bid'] = 0;
        		break;
        }

        // Tags are not applicable for banners
        if($module_id == gc_module_id('banners')) {
        	$data['tags'] = 0;
        	$data['tags_bid'] = 0;
        }*/
        
        
        // Additional handling for default values should be added below,
        // or in the model's prep_data() method
        

        $return = false;
        if ($type == 'insert') {
        	$data['created_ip'] = $this->input->ip_address();
            $id = $this->ad_packages_model->insert($data);

            if (is_numeric($id)) {
                $return = $id;
            }
        } elseif ($type == 'update') {
        	$data['modified_ip'] = $this->input->ip_address();
            $return = $this->ad_packages_model->update($id, $data);
        }
        
        // Highlight package
		if($return) {
			$this->packages_model->highlight($id, 'ad_packages', ($this->input->post('highlight') ? 1 : 0));
			// $this->gc->save_module_relations($id, $this->input->post('modules'), 'ads');
		}
		
		// save translations
		$this->translations->save_translations('ad_packages', $id, $return);
		
        return $return;
    }
    
    /**
     * Save default settings
     */
    public function settings() {
		clear_cache('site_settings'); // Clear Site Settings Cache
    	$this->load->model('settings/settings_model');
    	if (isset($_POST['save'])) {
    	    $this->gcrud->stats(array('module' => 'ads'));
    		$this->form_validation->set_rules('ad_views_purge_duration', 'lang:ads_view_duration', 'trim|required|is_numeric');
    		$this->form_validation->set_rules('tags[]', 'lang:label_tags', 'trim|required|is_decimal|is_array');
    		$this->form_validation->set_rules('categories[]', 'lang:label_categories', 'trim|required|is_decimal|is_array');
    		$this->form_validation->set_rules('cities[]', 'lang:label_cites', 'trim|required|is_decimal|is_array');
    		if($this->form_validation->run() == TRUE) {
    			$settings = array(
    				'ad_views_purge_duration' => $this->input->post('ad_views_purge_duration')
    			);
    			$costs = array(
    				'tags' => $this->input->post('tags'),
    				'categories' => $this->input->post('categories'),
    				'cities' => $this->input->post('cities')    				
    			);
    			$data = array (array('name' => 'ad.costs', 'value' => json_encode($costs)), array('name' => 'ad.settings', 'value' => json_encode($settings)));
    			$updated = $this->settings_model->update_batch($data, 'name');
    			if ($updated) {
    				Template::set_message(sprintf(lang('msg_save_success'), lang('gc_menu_settings')), 'success');
    			} else {
    				Template::set_message(sprintf(lang('msg_edit_failure'), lang('gc_menu_settings')), 'error');
    			}
    		}
    	}
    	$setting_name = 'ad.costs';
    	$result = $this->db->select('value')->get_where('settings', array('name' => $setting_name))->row();
    	if($result) {
    	    $costs = json_decode($result->value ?? '');
    	} else {
    	    $costs = json_decode(settings_item($setting_name));
    	}
    	Template::set('costs', $costs);
    	Template::set('settings', json_decode(settings_item('ad.settings')));
    	Template::set('toolbar_title', lang('default_costs'));
    	Template::render();
    }
    
    /**
     * Common function to update status
     * @param string $table
     * @param int $id
     * @param number $status
     */
    private function update_status($table, $id, $status = 99) {
    	if($status != 99) { // status is passed as a parameter
    		$this->db->update($table, array('status' => $status), array('id' => $id));
    	} else {
    		$this->db->query("UPDATE " . $this->db->dbprefix . $table ." SET status = !status WHERE id = $id" );
    	}
    	Template::set_message(lang('change_status_success'), 'success');
    }
    
    /**
     * Ajax function to get package information for the Ad Form
     */
    public function package_info() {
    	$package_id = (int) $this->input->post('id');
    	$result = array();
    	if($package_id > 0) {
    		$result = $this->ad_packages_model->get_package_info($package_id);
    	}
    	echo json_encode($result);
    }
    
    /**
     * Ajax Display Cost and Bid information to the user
     *
     * @param string $module
     * @param string $q
     */
    public function bid_info($module, $type, $q) {
    	header('Content-Type: application/x-json; charset=utf-8');
    	$result = $this->ads_model->admin_keyword_search($module, $q);
    	echo json_encode($result);
    }
    
    /**
     * Display Ad related keywords to the user
     * @param string $filter
     * @param int $ad_id
     */
    public function costs($filter = 'categories', $offset = 0) {  
    	// redirect admin to the packages page if the filter is not as defined below
    	switch($filter) {
    		case 'categories':
    		case 'cities':
    		case 'tags':
    			break;
    	
    		default:
    			show_404(SITE_AREA . '/financial/ads/packages');
    	}
    	// save keyword with CPC and CPD  	
    	if(isset($_POST['save'])) {
    	    $this->gcrud->stats(array('module' => 'ads'));
    		$this->ads_model->admin_save_costs($filter);
    	}
    	Assets::add_css('selectize.bootstrap3.css');
    	Assets::add_js('selectize.min.js');
    	 
    	$where = "AC.module_id = " . gc_module_id($filter);     	
    	
    	// Search Keyword
    	$redirect_condition = $this->uri->segment(5) ? $this->uri->segment(5) : 'categories'; // Important for categories or first tab search
    	$searchterm = $this->gc->search('ad_costs_' . $filter, $redirect_condition, '/financial/ads/costs/' . $filter . '/');
    	if(!empty($searchterm)) {
    		$where .= " AND (AC.row_id LIKE '%". $searchterm ."%' OR M.name LIKE '%". $searchterm ."%')";
    	}
    
    	// Order By
    	$sort_by = "M.name ASC";
    	$order_by = $this->gc->sort('ad_costs');
    	if(!empty($order_by)) {
    		$sort_by = $order_by;
    	}
    
    	$limit  = $this->settings_lib->item('site.list_limit') ?: 15;
    	$records = $this->ads_model->get_keyword_cost(0, $filter, $where, $sort_by, $offset, $limit);
    	$total_records = $this->ads_model->get_keyword_cost(1, $filter, $where, $sort_by, $offset, $limit);
    	$this->post_handling($filter, true, $records);
    	$this->load->library('pagination');
    	$pagerUriSegment = 6;
    	$pagerBaseUrl = site_url(SITE_AREA . '/financial/ads/costs/' . $filter .'/');
    	$pager['base_url']    = $pagerBaseUrl;
    	$pager['total_rows']  = $total_records;
    	$pager['per_page']    = $limit;
    	$pager['uri_segment'] = $pagerUriSegment;
    	 
    	$this->pagination->initialize($pager);
    	 
    	Template::set('index_url', site_url(SITE_AREA .'/financial/ads/costs/'));
    	Template::set('filter_type', $filter);
    	 
    	Template::set('toolbar_title', lang('manage_cpc_cpd'));
    	Template::set_view('cost_form');
    	Template::render();
    }
    
    /**
     * Ads Tags/Categories/Cities POST Handling
     * @param string $type - tags/categories/cities
     * @param string $edit - edit page
     * @param array $params - stored values
     */
    private function post_handling($type, $edit = false, $params = array()) {
    	$variables = array();
    	if(!empty($_POST['field_' . $type])) {
    		foreach($_POST['field_' . $type]['id'] as $key => $value) {
    			$variable = array();
    			$variable['id'] = $_POST['field_' . $type]['id'][$key];
    			$variable['name'] = $_POST['field_' . $type]['name'][$key];
    			$variable['cpc'] = $_POST['field_' . $type]['cpc'][$key];
    			$variable['cpd'] = $_POST['field_' . $type]['cpd'][$key];
    			$variables[] = $variable;
    		}
    		Template::set('options', $variables);
    	} else if($edit) {
    		Template::set('options', $params);
    	} else {
    		Template::set('options', $variables);
    	}
    }
    
    /**
     * Save bid only if a user has set a higher Bid
     * Or save the id of the module row
     * @param string $module - Tags/Categories/Cities
     * @return boolean
     */
    private function save_bid($ad_id, $module) {
    	$this->form_validation->set_rules('field_options[id][]', 'lang:label_id', 'trim|is_array');
    	$this->form_validation->set_rules('field_options[name][]', 'lang:label_name', 'trim|is_array');
    	$this->form_validation->set_rules('field_options[cost][]', 'lang:label_cost', 'trim|is_array');
    	$this->form_validation->set_rules('field_options[bid][]', 'lang:label_bid', 'trim|is_array');
    	if ($this->form_validation->run() === false) {
    		return false;
    	}
    	$data = array();
    	// Save bid only if a user has set a higher Bid
    	foreach($_POST['field_options']['id'] as $key => $value) {
    		$data[] = array(
    			'ad_id' => $ad_id,
    			'row_id' => $_POST['field_options']['id'][$key],
    			'module_id' => gc_module_id($module),
    			'cost' => $_POST['field_options']['cost'][$key],
    			'bid' => $_POST['field_options']['bid'][$key],
    			'ip_address' => $this->input->ip_address(),
    			'created_on' => date('Y-m-d H:i:s')
    		);
    	}
    	// user_id, row_id and module_id are the unique keys
    	$this->db->on_duplicate('ad_bids', $data);
    }
    
    /**
     * Send Higher Bid Notification to all users
     * @param string $send_all
     */
    public function bid_notifications() {
    	if(isset($_POST['send'])) {
    	    $this->gcrud->stats(array('module' => 'ads'));
    		$send_all = $this->input->post('send_all') ? TRUE : FALSE;
    		$queue_override = $this->input->post('queue_override') ? 1 : 0;
    		$this->ads_model->notify_users_all($send_all, $queue_override);
    		Template::set_message(sprintf(lang('msg_action_success'), lang('bid_notify'), lang('label_sent')), 'success');
    	}
    	Template::set('toolbar_title', lang('bid_notifications'));
    	Template::render();
    }
    
    /**
     * Display Ad Views to the User
     * 
     * @param number $ad_id
     * @param number $offset
     */
    public function view($ad_id, $offset = 0) {
        $user_id = $this->auth->user_id();
        $redirect_url = site_url(SITE_AREA . '/financial/ads');
        $this->ads_model->view($ad_id, $user_id, $this->limit, $offset, 'admin', $redirect_url);
        Template::render();
    }
    
    /**
     * Manage Email Templates
     * @param string $module
     * @param string $filter
     * @param number $offset
     */
    public function manage($module = 'email_templates', $filter = 'all', $offset = 0) {
        $extra = array(
            'model' => 'ads',
            'module_name' => 'ads',
            'context' => 'financial',
            'fields' => $module,
            'prefix' => 'ad'
        );
        $this->gc->module_view($module, $filter, $offset, 'admin', $extra);
    }
    
    /**
     * Add/Edit Email Templates Data
     *
     * @return void
     */
    public function form($module = 'email_templates', $action = 'create') {
        $params = array(
            'selectize' => true,
            'model' => 'ads',
            'context' => 'financial',
            'module_name' => 'ads',
            'fields_method' => $module,
            'prefix' => 'ad'
        );
        $this->gcrud->form($module, $action, 'admin', $params);
    }
}