<?php defined('BASEPATH') || exit('No direct script access allowed');

class Migration_Install_ads extends Migration
{
	/**
	 * @var string The name of the database table
	 */
	private $table_name = 'ads';
	private $packages_table = 'ad_packages';
	private $costs_table = 'ad_costs';
	private $keywords_table = 'ad_keywords';
	private $views_table = 'ad_views';
	
	/**
	 * @var array The table's fields
	 */
	private $fields = array(
		'id' => array(
			'type'        => 'INT',
			'constraint'  => 11,
			'auto_increment' => true,
		),
		'user_id' => array(
			'type'       => 'BIGINT',
			'constraint' => 20,
			'null' 		 => false,
		),
		'package_id' => array(
			'type'       => 'INT',
			'constraint' => 11,
			'null' 		 => false,
		),
		'row_id' => array(
			'type'       => 'BIGINT',
			'constraint' => 20,
			'null' 		 => false,
			'default' 	 => 0
		),
		'module_id' => array(
			'type'       => 'TINYINT',
			'constraint' => 1,
			'null' 		 => false,
			'default' 	 => 0
		),
		'start_date' => array(
			'type' 		 => 'DATE',
			'null' 		 => true
		),
		'end_date' => array(
			'type'       => 'DATE',
			'null' 		 => true
		),
		'daily_budget' => array(
			'type'       => 'FLOAT',
			'null' 		 => false,
			'default' 	 => 0
		),
		'price' => array(
			'type'       => 'FLOAT',
			'null' 		 => false,
			'default' 	 => 0
		),
		'duration' => array(
			'type'       => 'SMALLINT',
			'constraint' => 5,
			'null' 		 => false,
			'default' 	 => 0
		),
		'image' => array(
			'type'       => 'VARCHAR',
			'constraint' => 255,
			'null' 		 => true
		),
		'url' => array(
			'type'       => 'VARCHAR',
			'constraint' => 255,
			'null' 		 => true
		),
		'html_text' => array(
			'type'  	 => 'TEXT',
			'null' 		 => true
		),
		'status' => array(
			'type'       => 'TINYINT',
			'constraint' => 1,
			'null' 		 => false,
			'default' 	 => 0
		),
		'deleted' => array(
			'type'       => 'TINYINT',
			'constraint' => 1,
			'null' 		 => false,
			'default' 	 => 0
		),
		'deleted_by' => array(
			'type'       => 'BIGINT',
			'constraint' => 20,
			'null'		 => false,
			'default' 	 => 0
		),
		'created_by' => array(
			'type'       => 'BIGINT',
			'constraint' => 20,
			'null' 		 => false,
			'default' 	 => 0
		),
		'modified_by' => array(
			'type'       => 'BIGINT',
			'constraint' => 20,
			'null' 		 => false,
			'default' 	 => 0
		),
		'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'modified_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'created_ip' => array(
			'type'       => 'VARCHAR',
			'constraint' => 50,
			'null'       => true,
		),
		'modified_ip' => array(
			'type'       => 'VARCHAR',
			'constraint' => 50,
			'null'       => true,
		)
	);

	/**
	 * @var array The packages table's fields
	 */
	private $packages_fields = array(
		'id' => array(
			'type'       => 'INT',
			'constraint' => 11,
			'auto_increment' => true,
		),
		'module_id' => array(
			'type'       => 'TINYINT',
			'constraint' => 1,
			'null' 		 => false,
		),
		'banner_type_id' => array(
			'type'       => 'INT',
			'constraint' => 11,
			'null' 		 => false,
			'default' 	 => 0
		),
        'title' => array(
            'type'       => 'VARCHAR',
            'constraint' => 100,
            'null'       => false,
        ),
        'description' => array(
            'type'       => 'TEXT',
            'null'       => false,
        ),
		'icon' => array(
			'type' => 'VARCHAR',
			'constraint' => 30,
			'null' => TRUE
		),
        'display_page' => array(
            'type'       => 'CHAR',
            'constraint' => 40,
            'null'       => false,
        ),
        'max_impressions' => array(
            'type'       => 'INT',
            'constraint' => 11,
            'null'       => false,
        	'default'	 => 0
        ),
        'max_clicks' => array(
            'type'       => 'INT',
            'constraint' => 11,
            'null'       => false,
        	'default'	 => 0
        ),
        'pricing_type' => array(
            'type'       => 'CHAR',
            'constraint' => '10',
            'null'       => true,
        ),
        'price' => array(
            'type'       => 'FLOAT',
            'null'       => false,
        	'default'	 => 0
        ),
        'duration' => array(
            'type'       => 'SMALLINT',
            'constraint' => 5,
            'null'       => false,
        	'default'	 => 0
        ),
		'invoicing_package' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => FALSE,
			'default'=> 0
		),
		'days_before_next_invoice' => array(
			'type' => 'SMALLINT',
			'constraint' => 5,
			'null' => FALSE,
			'default'=> 0
		),
		'invoice_expiry' => array(
			'type' => 'SMALLINT',
			'constraint' => 5,
			'null' => FALSE,
			'default'=> 0
		),
		'invoice_first_reminder' => array(
			'type' => 'SMALLINT',
			'constraint' => 5,
			'null' => FALSE,
			'default'=> 0
		),
		'invoice_second_reminder' => array(
			'type' => 'SMALLINT',
			'constraint' => 5,
			'null' => FALSE,
			'default'=> 0
		),
		'invoice_third_reminder' => array(
			'type' => 'SMALLINT',
			'constraint' => 5,
			'null' => FALSE,
			'default'=> 0
		),
        'tags' => array(
            'type'       => 'TINYINT',
            'constraint' => 1,
            'null'       => false,
        	'default'	 => 0
        ),
        'tags_bid' => array(
            'type'       => 'TINYINT',
            'constraint' => 1,
            'null'       => false,
        	'default'	 => 0
        ),
        'categories' => array(
            'type'       => 'TINYINT',
            'constraint' => 1,
            'null'       => false,
        	'default'	 => 0
        ),
        'categories_bid' => array(
            'type'       => 'TINYINT',
            'constraint' => 1,
            'null'       => false,
        	'default'	 => 0
        ),
        'cities' => array(
            'type'       => 'TINYINT',
            'constraint' => 1,
            'null'       => false,
        	'default'	 => 0
        ),
        'cities_bid' => array(
            'type'       => 'TINYINT',
            'constraint' => 1,
            'null'       => false,
        	'default'	 => 0
        ),
		'color_scheme' => array(
			'type' => 'VARCHAR',
			'constraint' => 10,
			'null' => TRUE
		),
		'border_color' => array(
			'type' => 'VARCHAR',
			'constraint' => 10,
			'null' => TRUE
		),
		'trusted_seal' => array(
			'type' => 'VARCHAR',
			'constraint' => 50,
			'null' => TRUE
		),
		'display_order' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => FALSE,
			'default'=> 0
		),
		'priority' => array(
			'type' => 'SMALLINT',
			'constraint' => 5,
			'null' => FALSE,
			'default'=> 0
		),
        'highlight' => array(
            'type'       => 'TINYINT',
            'constraint' => 1,
            'null'       => false,
        	'default'	 => 0
        ),
        'order_status' => array(
            'type'       => 'TINYINT',
            'constraint' => 1,
            'null'       => false,
        	'default'	 => 0
        ),
        'status' => array(
            'type'       => 'TINYINT',
            'constraint' => 1,
            'null'       => false,
        	'default'	 => 1
        ),
        'created_ip' => array(
            'type'       => 'VARCHAR',
            'constraint' => 50,
            'null'       => true,
        ),
        'modified_ip' => array(
            'type'       => 'VARCHAR',
            'constraint' => 50,
            'null'       => true,
        ),
		'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
        'created_by' => array(
            'type'       => 'BIGINT',
            'constraint' => 20,
            'null'       => false,
            'default'	 => 0
        ),
		'modified_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
        'modified_by' => array(
            'type'       => 'BIGINT',
            'constraint' => 20,
            'null'       => false,
            'default'	 => 0
        ),
	);
	
	/**
	 * @var array The costs table's fields
	 */
	private $costs_fields = array(
		'user_id' => array(
			'type'       => 'BIGINT',
			'constraint' => 20,
            'null'       => false,
			'default' 	 => 0
		),
		'row_id' => array(
			'type'       => 'INT',
			'constraint' => 11,
           	'null'       => false
		),
		'module_id' => array(
			'type'       => 'TINYINT',
			'constraint' => 1,
           	'null'       => false
		),
		'cpc' => array(
			'type'       => 'FLOAT',
           	'null'       => false,
			'default'	 => 0
		),
		'cpd' => array(
			'type'       => 'FLOAT',
           	'null'       => false,
			'default'	 => 0
		),
		'percent' => array(
			'type'       => 'FLOAT',
           	'null'       => false,
			'default'	 => 0
		),
		'last_updated DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'ip_address' => array(
			'type'       => 'VARCHAR',
			'constraint' => 50,
			'null'       => true,
		),
	);
	
	/**
	 * @var array The keywords table's fields
	 */
	private $keywords_fields = array(
		'id' => array(
			'type'       => 'INT',
			'constraint' => 11,
			'auto_increment' => true,
		),
		'user_id' => array(
			'type'       => 'BIGINT',
			'constraint' => 20,
            'null'       => false,
			'default' 	 => 0
		),
		'ad_id' => array(
			'type'       => 'INT',
			'constraint' => 11,
           	'null'       => false
		),
		'row_id' => array(
			'type'       => 'INT',
			'constraint' => 11,
           	'null'       => false
		),
		'module_id' => array(
			'type'       => 'TINYINT',
			'constraint' => 1,
           	'null'       => false
		),
		'cost' => array(
			'type'       => 'FLOAT',
           	'null'       => false,
			'default'	 => 0
		),
		'bid' => array(
			'type'       => 'FLOAT',
           	'null'       => false,
			'default'	 => 0
		),
		'type' => array( // CPC/CPD
			'type'       => 'TINYINT',
			'constraint' => 1,
           	'null'       => false,
			'default'	 => 0
		),
		'notify_me' => array(
			'type'       => 'TINYINT',
			'constraint' => 1,
			'null' 		 => false,
			'default' 	 => 0
		),
		'last_notify_bid' => array( // do not re-send the notification email twice to the same user
			'type'       => 'FLOAT',
           	'null'       => false,
			'default'	 => 0
		),
		'status' => array(
			'type'       => 'TINYINT',
			'constraint' => 1,
			'null' 		 => false,
			'default' 	 => 0
		),
		'ip_address' => array(
			'type'       => 'VARCHAR',
			'constraint' => 50,
			'null'       => true,
		),
		'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
	);
	
	/**
	 * @var array The views table's fields
	 */
	private $views_fields = array(
	    'id' => array(
	        'type'       => 'INT',
	        'constraint' => 11,
	        'auto_increment' => true,
	    ),
		'ad_id' => array(
			'type'       => 'INT',
			'constraint' => 11,
			'null' 		 => false,
		),
		'keyword_id' => array(
			'type'       => 'INT',
			'constraint' => 11,
			'null' 		 => false,
			'default' 	 => 0
		),
		'impression' => array(
			'type'       => 'TINYINT',
			'constraint' => 1,
			'null' 		 => false,
			'default' 	 => 0
		),
		'click' => array(
			'type'       => 'TINYINT',
			'constraint' => 1,
			'null' 		 => false,
			'default' 	 => 0
		),
		'cost' => array(
			'type'       => 'FLOAT',
			'null' 		 => false,
			'default' 	 => 0
		),
		'ip_address' => array(
			'type'       => 'VARCHAR',
			'constraint' => 50,
			'null'       => true,
		),
		'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
	);

	/**
	 * Install this version
	 *
	 * @return void
	 */
	public function up() {
		// create ads table
		$this->dbforge->add_field($this->fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('user_id', false);
		$this->dbforge->add_key('package_id', false);
		$this->dbforge->add_key('row_id', false);
		$this->dbforge->add_key('module_id', false);
		$this->dbforge->add_key('price', false);
		$this->dbforge->add_key('status', false);
		$this->dbforge->add_key('deleted', false);
		$this->dbforge->create_table($this->table_name);
		
		// create packages table
		$this->dbforge->add_field($this->packages_fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('module_id', false);
		$this->dbforge->add_key('banner_type_id', false);
		$this->dbforge->create_table($this->packages_table);
		
		// create costs table
		$this->dbforge->add_field($this->costs_fields);
		$this->dbforge->add_key('user_id', false);
		$this->dbforge->add_key('row_id', false);
		$this->dbforge->add_key('module_id', false);
		$this->dbforge->create_table($this->costs_table);
		$unique_keyword_costs = "ALTER TABLE " . $this->db->dbprefix . $this->costs_table . " ADD UNIQUE INDEX `unique_keyword` (`row_id`, `module_id`)";		
		$this->db->query($unique_keyword_costs);
		
		// create keywords table
		$this->dbforge->add_field($this->keywords_fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('user_id', false);
		$this->dbforge->add_key('ad_id', false);
		$this->dbforge->add_key('row_id', false);
		$this->dbforge->add_key('module_id', false);
		$this->dbforge->add_key('bid', false);
		$this->dbforge->create_table($this->keywords_table);
		$unique_keyword = "ALTER TABLE " . $this->db->dbprefix . $this->keywords_table . " ADD UNIQUE INDEX `unique_keyword` (`ad_id`,`row_id`, `module_id`)";		
		$this->db->query($unique_keyword);	

		// create views table
		$this->dbforge->add_field($this->views_fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('ad_id', false);
		$this->dbforge->add_key('keyword_id', false);
		$this->dbforge->add_key('impression', false);
		$this->dbforge->add_key('click', false);
		$this->dbforge->create_table($this->views_table);
		
		// Add settings row
		$this->db->insert('settings', array('name' => 'ad.settings', 'module' => 'ads', 'value' => '{"ad_views_purge_duration":"90"}'));
		$this->db->insert('settings', array('name' => 'ad.costs', 'module' => 'ads', 'value' => '{"tags":{"cpc":"5","cpd":"10"},"categories":{"cpc":"7","cpd":"14"},"cities":{"cpc":"9","cpd":"18"}}'));
	}

	/**
	 * Uninstall this version
	 *
	 * @return void
	 */
	public function down() {
		$this->dbforge->drop_table($this->table_name);
		$this->dbforge->drop_table($this->packages_table);
		$this->dbforge->drop_table($this->costs_table);
		$this->dbforge->drop_table($this->keywords_table);
		$this->dbforge->drop_table($this->views_table);
		// Remove settings row
		$settings = $this->db->select('name')->get_where('settings', array('name' => 'ad.settings'));
		if($settings) {
			$this->db->delete('settings', array('name' => 'ad.settings'));
		}
		// Remove settings costs row
		$costs = $this->db->select('name')->get_where('settings', array('name' => 'ad.costs'));
		if($costs) {
			$this->db->delete('settings', array('name' => 'ad.costs'));
		}
	}
}