<?php defined('BASEPATH') || exit('No direct script access allowed');

class Ads_model extends BF_Model
{
    protected $table_name	= 'ads';
	protected $key			= 'id';
	protected $date_format	= 'datetime';

	protected $log_user 	= true;
	protected $set_created	= true;
	protected $set_modified = true;
	protected $soft_deletes	= true;

	protected $created_field     = 'created_on';
    protected $created_by_field  = 'created_by';
	protected $modified_field    = 'modified_on';
    protected $modified_by_field = 'modified_by';
    protected $deleted_field     = 'deleted';
    protected $deleted_by_field  = 'deleted_by';

	// Customize the operations of the model without recreating the insert,
    // update, etc. methods by adding the method names to act as callbacks here.
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 	    = array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	// For performance reasons, you may require your model to NOT return the id
	// of the last inserted row as it is a bit of a slow method. This is
    // primarily helpful when running big loops over data.
	protected $return_insert_id = true;

	// The default type for returned row data.
	protected $return_type = 'object';

	// Items that are always removed from data prior to inserts or updates.
	protected $protected_attributes = array();

	// You may need to move certain rules (like required) into the
	// $insert_validation_rules array and out of the standard validation array.
	// That way it is only required during inserts, not updates which may only
	// be updating a portion of the data.
	protected $validation_rules 		= array();
	protected $insert_validation_rules  = array();
	protected $skip_validation 			= false;

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
    }
    
    /**
     * Email Template Fields
     *
     * @return array
     */
    public function email_templates($role = 'admin') {
        $fields = array(
            array(
                'type' => 'none',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'dropdown',
                'name' => 'module_table',
                'label' => 'label_module',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:gc_menu_ads;value:ads',
                'options_detailed' => 1,
                'options_lang' => 1,
                'class' => 3,
                'rules' => 'trim|unique_email_template[ad]',
            ),
            array(
                'type' => 'dropdown',
                'name' => 'user_type',
                'label' => 'bf_user',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:placeholder_select;value:|name:us_admin;value:admin|name:bf_user;value:user',
                'options_detailed' => 1,
                'options_lang' => 1,
                'class' => 3,
                'rules' => 'trim',
            ),
            array(
                'type' => 'dropdown',
                'name' => 'action_type',
                'label' => 'label_action',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:placeholder_select;value:|name:add;value:add|name:edit;value:edit|name:delete;value:delete|name:em_status_update;value:status_update|name:ad_bid_notification;value:bid_notification|name:em_expiry_notification;value:expiry_notification',
                'options_detailed' => 1,
                'options_lang' => 1,
                'class' => 3,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'name',
                'label' => 'label_name',
                'placeholder' => 'placeholder_name',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
                'rules' => 'required|trim|max_length[50]'
            ),
            array(
                'type' => 'text',
                'name' => 'subject',
                'label' => 'label_subject',
                'placeholder' => 'placeholder_subject',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'required|trim|max_length[255]'
            ),
            array(
                'type' => 'textarea',
                'name' => 'message',
                'label' => 'label_message',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'custom_rules' => 'editor',
                'field_class' => 'editor',
                'class' => 9,
                'rules' => 'trim|required',
            ),
            array(
                'type' => 'textarea',
                'name' => 'sms_text',
                'label' => 'sms_text',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'class' => 9,
                'field_extra' => 'rows:3',
                'rules' => 'trim',
            ),
            array(
                'type' => 'textarea',
                'name' => 'params',
                'label' => 'em_params',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'class' => 9,
                'tooltip' => lang('em_params_help'),
                'field_extra' => 'rows:3',
                'rules' => 'trim',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'status',
                'label' => 'label_status',
                'frontend' => false,
                'admin_only' => false,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            )
        );
        return $fields;
    }
    
    /**
     * Display all advertising campaign to the User
     * @param number $count
     * @param string $where - Pass user ID in the where clause for member area
     * @param string $sort_by
     * @param number $offset
     * @param number $limit
     * @return number
     */
    public function get_ads($count = 0, $where = null, $sort_by = 'id DESC', $offset = 0, $limit = 25) {
    	if($count == 0) {
    		$sql = "SELECT A.*, AP.title, AP.description, AP.pricing_type, U.display_name, U.email, BT.type AS banner_type";
    		$sql .= ", (SELECT SUM(AV.click) FROM " . $this->db->dbprefix . "ad_views AV WHERE AV.ad_id = A.id) AS clicks";
    		$sql .= ", (SELECT SUM(AV.impression) FROM " . $this->db->dbprefix . "ad_views AV WHERE AV.ad_id = A.id) AS impressions";
    		$sql .= ", (SELECT COUNT(AK.id) FROM " . $this->db->dbprefix . "ad_keywords AK WHERE AK.ad_id = A.id) AS hasKeywords";
    		//$sql .= ", SUM(AV.click) AS clicks, SUM(AV.impression) AS impressions, COUNT(AK.id) AS hasKeywords";
    		// display module title to the user
    		/*$sql .= ", CASE WHEN A.module_id = 1 THEN ";
    		$sql .= "(SELECT title FROM " . $this->db->dbprefix. "listings WHERE id = A.row_id) END AS row_title";*/
    	} else {
    		$sql = "SELECT COUNT(A.id) AS TotalCount";
    	}
    	$sql .= " FROM " . $this->db->dbprefix . "ads A";
    	$sql .= " INNER JOIN " . $this->db->dbprefix . "ad_packages AP ON AP.id = A.package_id";
    	$sql .= " LEFT JOIN " . $this->db->dbprefix . "banner_types BT ON BT.id = AP.banner_type_id";
    	/*if($count == 0) {
    		$sql .= " LEFT JOIN " . $this->db->dbprefix . "ad_keywords AK ON AK.ad_id = A.id";
    		$sql .= " LEFT JOIN " . $this->db->dbprefix . "ad_views AV ON AV.ad_id = A.id";    		
    	}*/
    	$sql .= " LEFT JOIN " . $this->db->dbprefix . "users U ON U.id = A.user_id";
    	if(!empty($where)) {
    		$sql .= " WHERE $where";
    	}
    	if($count == 0) {
    		$sql .= " GROUP BY A.id";
    		$sql .= " ORDER BY " . $sort_by;    		
    		$sql .= " LIMIT " . $offset . ", " . $limit;
    		return $this->db->query ( $sql )->result();
    	} else {
    		$result = $this->db->query ( $sql )->row();
    		return $result ? $result->TotalCount : 0;
    	}
    }
    
    /**
     * Display Row Title to the User with clickable link
     * @param int $row_id
     * @param int $module_id
     */
    public function get_row_info($row_id, $module_id) {
    	if(!empty($module_id) && $module_id != 15) {
	    	$table = gc_module_name($module_id);
	    	if(empty($table )) {
	    	    return false;
	    	}
	    	$sql = "SELECT M.title, M.slug, M.status";
	    	if($table == 'listings') {
	    		$sql .= ", C.name AS city";
	    	}
	    	$sql .= " FROM " . $this->db->dbprefix . $table . " M";
	    	if($table == 'listings') {
	    		$sql .= " LEFT JOIN " . $this->db->dbprefix . "cities C ON C.id = M.city_id";
	    	}
	    	$sql .= " WHERE M.id = " . $row_id;
	    	return $this->db->query($sql)->row();
    	} else {
    		return false;
    	}
    }
    
    /**
     * Create a new Ad.
     *
     * @return void
     */
    public function advertise($role = 'admin') {
    	// set the redirect URL to the Admin Panel in case of Admin Editing
    	$role = !empty($_GET['role']) && $_GET['role'] == 'admin' ? 'admin' : $role;
    	$redirect_url = ($role == 'admin' ? SITE_AREA . '/financial/ads' : gc_url_format('ads'));    	
    	$this->load->model('ads/ad_packages_model');
    	$package_id = (int) $this->crypt->decode($this->input->get('id'));
    	$user_id = (int) $this->input->get('user_id');
    	$user_id = ($user_id == 0 ? $this->auth->user_id() : $user_id);
    	if((int)$package_id == 0) {
    		redirect ( $redirect_url );
    	}
    	if($role == 'member') {
    		$this->auth->restrict();
    	}
    	
    	// get package information
    	$ad_package_info = $this->ad_packages_model->get_package_info($package_id);
    	if(!$ad_package_info) {
    		redirect($redirect_url);
    	}
    	
    	Assets::add_module_css('ads', 'ads.css');
    	Assets::add_module_js('ads', 'ads.js');
    	if (isset($_POST['save'])) {
    		if ($insert_id = $this->save_ad($role, $ad_package_info, $user_id)) {
    		    // Notify Admin/User
    		    // $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'ad_email_templates', 'module_table' => 'ads', 'action_type' => 'add', 'id' => $insert_id));
    			log_activity($user_id, sprintf(lang('msg_save_success'), lang('label_ad')) . ': ' . $insert_id . ' : ' . $this->input->ip_address(), 'ads');
    			if($this->input->post('checkout_price') && $this->input->post('checkout_price') > 0) {
    				// redirect user to the checkout page
    				// $this->session->set_userdata('order_type', 'ads');
    				// $this->session->set_userdata('checkout_redirect', $redirect_url);
    				// $checkout_param = '?id=' . $this->crypt->encode($insert_id);
    				// if($role == 'admin') {
    			    //		$checkout_param .= '&user_id=' . $this->crypt->encode($user_id);
    				// }
					$this->load->model('orders/orders_model');
					$this->orders_model->cart($insert_id, 0, 1, 1, 'ads', false);
    				Template::redirect(gc_url_format('members/checkout'));
    			} else {
    				Template::set_message(sprintf(lang('msg_save_success'), lang('label_ad')), 'success');
    				redirect($redirect_url);
    			}
    		}
    
    		// Not validation error
    		if ( ! empty($this->ads_model->error)) {
    			Template::set_message(sprintf(lang('msg_create_failure'), lang('label_ad')) . $this->ads_model->error, 'danger');
    		}
    	}
    	
    	if($role == 'admin') {
    		$packages = $this->ad_packages_model->select('id,title')->find_all();
    		Template::set('packages', $packages);
    		Template::set('toolbar_title', lang('create_ad'));
    	}
    	// pricing post handling
    	if($ad_package_info->pricing_type == 'campaign') {
    		if($ad_package_info->tags == 1) {
    			$this->post_handling('tags');
    		}
    		if($ad_package_info->categories == 1) {
    			$this->post_handling('categories');
    		}
    		if($ad_package_info->cities == 1) {
    			$this->post_handling('cities');
    		}
    	}
    	
    	// Display module title to the user based on user id
    	if($ad_package_info->module_id != gc_module_id('banners')) {
    	    // get module table information
    	    $table = gc_module_name($ad_package_info->module_id);
    	    switch($table) {
    	        case 'listings':
    	            $records = $this->db->select('id,title')->order_by('title', 'ASC')->limit(100)->get_where($table, array('user_id' => $user_id, 'deleted' => 0, 'status' => 1))->result();
    	            break;
    	        default: // other modules
    	            // $this->db->join('listings L', 'L.id = ' . $table . '.id');
    	            // $records = $this->db->select("$table.id, $table.title")->order_by('title', 'ASC')->limit(100)->get_where($table, array('L.user_id' => $user_id, 'L.deleted' => 0, 'L.status' => 1, $table .'.deleted' => 0, $table .'.status' => 1))->result();
    	            $records = $this->db->select("$table.id, $table.title")->where(array('user_id' => $user_id, 'status' => 1))->order_by('title', 'ASC')->limit(100)->get($table)->result();
    	    }
    	    if(empty($records)) {
    	        Template::set_message(sprintf(lang('advertise_error'), lang($table)), 'danger');
    	        Template::redirect($redirect_url);
    	    }
    	    Template::set('records', $records);
    	}
    	
    	Template::set(array(
    		'ad_package' => $ad_package_info,
    		'role' => $role,
    		'redirect_url' => $redirect_url
    	));
    	Template::set_view('ad_form');
    	$this->gc->demo_custom_css_js('member');
    	Template::render('member');
    }
    
    /**
     * Allows editing of Ads data.
     *
     * @return void
     */
    public function edit($role = 'admin') {
    	// set the redirect URL to the Admin Panel in case of Admin Editing
    	$role = !empty($_GET['role']) && $_GET['role'] == 'admin' ? 'admin' : $role;
    	$redirect_url = ($role == 'admin' ? SITE_AREA . '/financial/ads' : site_url('ads'));
    	$id = intval($this->crypt->decode($this->input->get('id')));
    	if($id == 0) {
    		redirect ($redirect_url);
    	}
    	if($role == 'member') {
    		$this->auth->restrict();		
    	}
    	// load css and js files
    	$this->gc->demo_custom_css_js('member');
    	Assets::add_module_css('ads', 'ads.css');
    	Assets::add_module_js('ads', 'ads.js');
    	
		// allow the admin to edit the ad on their behalf
    	$user_id = (int) $this->input->get('user_id');
    	$user_id = ($user_id == 0 ? $this->auth->user_id() : $user_id);
    	
    	// get ad information
    	$record = $this->ads_model->find($id);
    	
    	// restrict member to edit their own Ad
    	if($role == 'member' && $user_id != $record->user_id) {
    		Template::set_message ( lang ( 'unauthorized_deleted' ), 'error' );
    		redirect ( $redirect_url );
    	}
    	 
    	$packages = $this->ad_packages_model->select('id,title')->find_all();
    	$ad_package_info = $this->ad_packages_model->get_package_info($record->package_id);
    	if(!$ad_package_info) {
    		redirect($redirect_url);
    	}
    	
    	if (isset($_POST['save'])) {
    	    $this->gcrud->stats(array('module' => 'ads'));
    		if ($this->save_ad($role, $ad_package_info, $user_id, 'update', $id)) {
    			log_activity($user_id, sprintf(lang('msg_save_success'), lang('label_ad')) . ': ' . $id . ' : ' . $this->input->ip_address(), 'ads');
    			Template::set_message(sprintf(lang('msg_save_success'), lang('label_ad')), 'success');
    			redirect($redirect_url);
    		}
    
    		// Not validation error
    		if ( ! empty($this->ads_model->error)) {
    			Template::set_message(sprintf(lang('msg_edit_failure'), lang('label_ad')) . $this->ads_model->error, 'danger');
    		}
    	} elseif (isset($_POST['delete'])) {
    	    $this->gcrud->stats(array('module' => 'ads'));
    		if ($this->ads_model->delete($id)) {
    			log_activity($user_id, sprintf(lang('msg_action_success'), lang('label_ads'), lang('label_deleted')) . ': ' . $id . ' : ' . $this->input->ip_address(), 'ads');
    			Template::set_message(sprintf(lang('msg_action_success'), lang('label_ads'), lang('label_deleted')), 'success');
    
    			redirect($redirect_url);
    		}
    
    		Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_ads')) . $this->ads_model->error, 'danger');
    	}
    	
    	Template::set(array(
    		'record' => $record, 
    		'ad_package' => $ad_package_info,
    		'packages' => $packages
    	));
    	
    	// pricing post handling only if the duration is 0
    	// do not show tags, categories and cities to a user if the purchased Ad is a fixed duration Ad
    	if($ad_package_info->pricing_type == 'campaign') {
    		if($ad_package_info->tags == 1) {
    			$this->post_handling('tags', true, $this->get_saved_keywords($id, 'tags'));
    		}
    		if($ad_package_info->categories == 1) {
    			$this->post_handling('categories', true, $this->get_saved_keywords($id, 'categories'));
    		}
    		if($ad_package_info->cities == 1) {
    			$this->post_handling('cities', true, $this->get_saved_keywords($id, 'cities'));
    		}
    	
    		// Display module title to the user based on user id
    		if($record->module_id != gc_module_id('banners')) {
    			// get module table information
    			$table = gc_module_name($ad_package_info->module_id);
    			if(!empty($record->row_id) && !empty($record->end_date) && $record->end_date != '0000-00-00') {
    				$row_info = $this->db->select('id,title')->get_where($table, array('id' => $record->row_id))->row();
    				Template::set('row_info', $row_info);
    			} else {
	    			switch($table) {
	    				case 'listings':	    					
	    					if($role == 'admin') {
	    						$search_where = array();
	    					} else {
	    						$search_where = array('user_id' => $user_id, 'deleted' => 0, 'status' => 1);
	    					}
	    					$records = $this->db->select('id,title')->order_by('title', 'ASC')->limit(100)->get_where($table, $search_where)->result();
	    					break;
	    				default: // other modules
	    					if($role == 'admin') {
	    						$search_where = array();
	    					} else {
	    						$search_where = array('L.user_id' => $user_id, 'L.deleted' => 0, 'L.status' => 1, $table .'.deleted' => 0, $table .'.status' => 1);
	    					}
	    					$this->db->join('listings L', 'L.id = ' . $table . '.id');
	    					$records = $this->db->select("$table.id,$table.title")->order_by('title', 'ASC')->limit(100)->get_where($table, $search_where)->result();
	    			}
	    			Template::set('records', $records);
    			}
    		}
    	}
    	Template::set(array(
    		'role' => $role,
    		'redirect_url' => $redirect_url
    	));
    	Template::set('toolbar_title', lang('edit_ad'));
    	Template::set_view('ad_form'); 
    	Template::render('member');
    }
    
    /**
     * Display saved keywords to the user
     * @param int $ad_id
     * @param string $module
     * @return object
     */
    private function get_saved_keywords($ad_id, $module) {
    	$sql = "SELECT M.id, M.name, AK.cost, AK.bid, SUM(AV.click) AS clicks, SUM(AV.impression) AS impressions";
    	$sql .= " FROM " . $this->db->dbprefix . "ad_keywords AK";
    	$sql .= " INNER JOIN " . $this->db->dbprefix . $module . " M ON M.id = AK.row_id";
    	$sql .= " LEFT JOIN " . $this->db->dbprefix . "ad_views AV ON AV.keyword_id = AK.id";
    	$sql .= " WHERE AK.module_id = " . gc_module_id($module) . " AND AK.ad_id = " . $ad_id;  
    	$sql .= " GROUP BY AK.id";  	
    	$result = $this->db->query($sql)->result_array();
    	return $result;
    }
    
    /**
     * Display saved keywords to the user
     * @param int $ad_id
     * @param string $module
     * @return object
     */
    public function get_saved_keywords_admin($ad_id, $module, $count = 0, $where = null, $sort_by = "AK.id ASC", $offset = 0, $limit = 0) {
    	if($count == 0) {
    		$sql = "SELECT AK.id AS keyword_id, M.id, M.name, AK.cost, AK.bid, AK.status,";
    		$sql .= " (SELECT SUM(AV.click) AS clicks FROM " . $this->db->dbprefix . "ad_views AV WHERE AV.keyword_id = AK.id) AS clicks,";
    		$sql .= " (SELECT SUM(AV.impression) AS impressions FROM " . $this->db->dbprefix . "ad_views AV WHERE AV.keyword_id = AK.id) AS impressions";
    	} else {
    		$sql = "SELECT COUNT(AK.id) AS TotalCount";
    	}
    	$sql .= " FROM " . $this->db->dbprefix . "ad_keywords AK";
    	$sql .= " INNER JOIN " . $this->db->dbprefix . $module . " M ON M.id = AK.row_id";
    	if(!empty($where)) {
    		$sql .= " WHERE $where";
    	}
    	if($count == 0) {
    		$sql .= " GROUP BY AK.id";
    		$sql .= " ORDER BY " . $sort_by;
    		if($limit != 0) {
    			$sql .= " LIMIT " . $offset . ", " . $limit;
    		}
    		return $this->db->query ( $sql )->result_array();
    	} else {
    		$result = $this->db->query ( $sql )->row();
    		return $result ? $result->TotalCount : 0;
    	}
    }
    
    /**
     * Save the data.
     *
     * @param string $type Either 'insert' or 'update'.
     * @param int    $id   The ID of the record to update, ignored on inserts.
     *
     * @return boolean|integer An ID for successful inserts, true for successful
     * updates, else false.
     */
    private function save_ad($role, $ad_package, $user_id, $type = 'insert', $id = 0) {
    	if ($type == 'update') {
    		$_POST['id'] = $id;
    	}

    	$this->load->library('form_validation');
		$this->load->model('images/images_model');
    	
    	$pricing_type = $ad_package->pricing_type;
    	$banner_type = $ad_package->type;
    	// Validate the data
    	if($type == 'insert') {
    		$this->form_validation->set_rules('start_date', 'lang:label_start_date', 'required|trim|valid_datetime');
    	}
    	if($pricing_type == 'campaign') {
    		if($ad_package->duration == 0) {
    			$this->form_validation->set_rules('daily_budget', 'lang:daily_budget', 'trim|required|is_decimal|max_length[5]|min_val[1]');
    		}
    		if($ad_package->tags == 1) {
    			$this->form_validation->set_rules('field_tags[id]', 'lang:label_tags', 'trim|required|is_array');
    		}
    		if($ad_package->categories == 1) {
    			$this->form_validation->set_rules('field_categories[id]', 'lang:label_categories', 'trim|required|is_array');
    		}
    		if($ad_package->cities == 1) {
    			$this->form_validation->set_rules('field_cities[id]', 'lang:cities', 'trim|required|is_array');
    		}
    		$this->form_validation->set_rules('notify_me', 'lang:label_notify_me', 'trim');
    	}
    	
    	if($ad_package->module_id == 15) { // Banners Advertising
	    	if($banner_type == 'image' || $banner_type == 'slider') {
	    		$this->form_validation->set_rules('uploaded_image', 'lang:label_image', 'trim|required');
	    		//Check whether image is uploaded or not
	    		if(isset($_FILES['image']) && $_FILES['image']['size'] > 0){
	    			$file_data = $this->images_model->upload_image('image');
	    			if(!$file_data) {
	    				$_POST['file_upload'] = 'NONE';
	    				$this->form_validation->set_rules('file_upload', 'label:label_image', 'required|valid_file_upload');
	    			}
	    			// Avoid logo upload process, if the form has validation issues with other fields
	    			$_POST['uploaded_image'] = isset($file_data['upload_data']['file_name']) ? $file_data['upload_data']['file_name'] : '';
	    		}
	    		$this->form_validation->set_rules('url', 'lang:label_url', 'trim|required|valid_url');
	    	} else if ($banner_type == 'text' || $banner_type == 'html') {
	    		$this->form_validation->set_rules('html_text', 'lang:label_text_html', 'trim|required');
	    	}
    	} else {
    	    $module = gc_module_name($ad_package->module_id);
    	    $this->form_validation->set_rules('row_id', 'lang:singular_' . $module . '', 'trim|required|is_numeric');
    	}
    	if ($this->form_validation->run() === false) {
    		return false;
    	}
    	$data = array();
    	$data['user_id'] = $user_id;
    	$data['row_id'] = $this->input->post('row_id') ? $this->input->post('row_id') : 0;
    	$data['module_id'] = $ad_package->module_id;
    	$data['package_id'] = $ad_package->id;
    	$data['daily_budget'] = $this->input->post('daily_budget') ? $this->input->post('daily_budget') : 0;
    	if($ad_package->module_id == 15) { // Banners Advertising
	    	if($banner_type == 'image' || $banner_type == 'slider') {
				// copy image from the temp folder
				$image = $this->images_model->get_module_image($_POST['uploaded_image'], 'banners', $data['row_id']);
				if(is_string($image)) {
					$data['image'] 	= $image;
				}
	    		$data['url'] = $this->input->post('url');
	    	} else if ($banner_type == 'text' || $banner_type == 'html') {
	    		$data['html_text'] = $this->input->post('html_text');
	    	}
    	}
    	    
    	$return = false;
    	if ($type == 'insert') {
    	    $ad_status = 2; // Fixed Ad Default Status
    		$start_date = $this->input->post('start_date');
    		$data['start_date'] = $start_date;
    		if($ad_package->pricing_type == 'fixed') {
    			$data['price'] = $ad_package->price;
    			$data['duration'] = $ad_package->duration;
    			$_POST['checkout_price'] = $ad_package->price; // to redirect user to the checkout page
    		} else if($ad_package->duration > 0) {    			
    			$data['duration'] = $ad_package->duration;
    			// update price later after adding keywords to the keywords table
    		}
    		
    		// Campaign Ad Status
    		if($ad_package->pricing_type == 'campaign') {    		    
    		    $ad_status = $ad_package->order_status;
    		}
    		
    		// calculate end date based on the duration field
    		if(!empty($start_date) && $ad_package->duration > 0) {
    			$data['end_date'] = date('Y-m-d', strtotime('+' . $ad_package->duration . ' days', strtotime($start_date)));
    		}
    		
    		// waiting for admin approval
    		// Do not set the status to Approved else the Banner will not be added automatically
    		$data['status'] = $ad_status; 
    		/*if($role == 'member') {
    		} else if($role == 'admin') {
    			$data['status'] = 1; // approved 
    			$this->update_ad_status($id, 1);   			
    		}*/
    		// $data['price'] = $this->input->post('price') ? $this->input->post('price') : 0;
    		// $data['duration'] = $this->input->post('duration') ? $this->input->post('duration') : 0;
    		$data['created_ip'] = $this->input->ip_address();
    		$id = $this->ads_model->insert($data);
    
    		if (is_numeric($id)) {
    			$return = $id;
    		}
    	} elseif ($type == 'update') {
    		$data['modified_ip'] = $this->input->ip_address();
    		$return = $this->ads_model->update($id, $data);
    		if(!empty($ad_package) && $ad_package->module_id == 15 && !empty($data['html_text'])) {
    		    $ad_info = $this->db->where('id', $id)->get('ads')->row();
    		    if($ad_info && !empty($ad_info->row_id)) {
    		        $this->db->where('id', $ad_info->row_id)->update('banners', array('html_text' => $data['html_text']));
    		    }
    		}
    	}
    	
    	// save keywords
    	if($return && $ad_package->pricing_type == 'campaign') {
    		if($ad_package->tags == 1) {
    			$this->save_keywords($id, 'tags');
    		}
    		if($ad_package->categories == 1) {
    			$this->save_keywords($id, 'categories');
    		}
    		if($ad_package->cities == 1) {
    			$this->save_keywords($id, 'cities');
    		}
    		// get cost from the keywords table and update the ads price
    		// do this only if the package has a fixed duration
    		if($type == 'insert' && $ad_package->duration > 0) {
    			$keywords_cost = $this->db->select('SUM(bid) AS price')->get_where('ad_keywords', array('ad_id' => $id))->row();
    			if($keywords_cost) {
    				$ad_cost = (float) $keywords_cost->price * $ad_package->duration;
    				$_POST['checkout_price'] = $ad_cost; // to redirect user to the checkout page
    				$this->db->where('id', $id)->update('ads', array('price' => $ad_cost));
    			}
    		}
    	} else if($return) {
    	    $this->save_fixed_ad_keywords($id);
    	}
    	
    	// Update the Ad status to approved and add the banner if applicable
    	if($type == 'insert' && $ad_status == 1) {
    	    $this->update_ad_status($id, $ad_status);
    	}
    	return $return;
    }
    
    /**
     * Module Fixed Ad Keywords
     * e.g. get listings tags and keywords from the listings table itself and save information in the keywords table
     * @param number $ad_id
     */
    public function save_fixed_ad_keywords($ad_id, $ad = array()) {
        if(empty($ad)) {
            $ad = $this->db->where('id', $ad_id)->get('ads')->row();
        }
        if(!$ad) {
            return false;
        }
        if($ad->module_id == 15) {
            return false;
        }
        $user_id = $ad->user_id;
        $ip_address = $this->input->ip_address();
        $datetime = date('Y-m-d H:i:s');
        $record = $this->db->where('id', $ad->row_id)->get(gc_module_name($ad->module_id))->row();
        if($record) {
            $fields = array('tag_ids' => 't', 'category_ids' => 'c');
            foreach($fields as $field => $separator) {
                if(!empty($record->{$field})) {
                    $ids_rows = explode(',', $record->{$field});
                    $search_module_id = $field == 'tag_ids' ? gc_module_id('tags') : gc_module_id('categories');
                    foreach($ids_rows as $id_row) {
                        $id_row = (int) str_replace('id', '', rtrim($id_row, $separator));
                        $data[] = array(
                            'user_id' => $user_id,
                            'ad_id' => $ad_id,
                            'module_id' => $search_module_id,
                            'row_id' => $id_row,
                            'status' => 1,
                            'ip_address' => $ip_address,
                            'created_on' => $datetime
                        );
                    }
                }
            }
        }
        $this->db->where('ad_id', $ad_id)->delete('ad_keywords');
		if(!empty($data)) {
        	$this->db->insert_batch('ad_keywords', $data);
		}
    }
    
    /**
     * Ads Tags/Categories/Cities POST Handling
     * @param string $type - tags/categories/cities
     * @param string $edit - edit page
     * @param array $params - stored values
     */
    private function post_handling($type, $edit = false, $params = array()) {
    	$variables = array();
    	if(!empty($_POST['field_' . $type])) {
    		foreach($_POST['field_' . $type]['id'] as $key => $value) {
    			$variable = array();
    			$variable['id'] = $_POST['field_' . $type]['id'][$key];
    			$variable['name'] = $_POST['field_' . $type]['name'][$key];
    			$variable['cost'] = $_POST['field_' . $type]['cost'][$key];
    			$variable['bid'] = $_POST['field_' . $type]['bid'][$key];
    			$variables[] = $variable;
    		}
    		Template::set($type .'_options', $variables);
    	} else if($edit) {
    		Template::set($type .'_options', $params);
    	} else {
    		Template::set($type .'_options', $variables);
    	}
    }
    
    /**
     * Save keywords selected by the user for Ads
     * @param int $ad_id
     * @param string $type - Tags/Categories/Cities
     * @return boolean
     */
    private function save_keywords($ad_id, $type) {
    	/*$this->form_validation->set_rules('field_' . $type .'[id][]', 'lang:label_id', 'trim|is_array');
    	$this->form_validation->set_rules('field_' . $type .'[name][]', 'lang:label_name', 'trim|is_array');
    	$this->form_validation->set_rules('field_' . $type .'[cost][]', 'lang:label_cost', 'trim|is_array');
    	$this->form_validation->set_rules('field_' . $type .'[bid][]', 'lang:label_bid', 'trim|is_array');
    	if ($this->form_validation->run() === false) {
    		return false;
    	}*/
    	$data = array();
    	// Save bid only if a user has set a higher Bid
    	foreach($_POST['field_' . $type]['id'] as $key => $value) {
    		$data[] = array(
    			'ad_id' => $ad_id,
    			'row_id' => $_POST['field_' . $type]['id'][$key],
    			'module_id' => gc_module_id($type),
    			'cost' => $_POST['field_' . $type]['cost'][$key],
    			'bid' => $_POST['field_' . $type]['bid'][$key],
    			'notify_me' => $this->input->post('notify_me') ? 1 : 0,
    			'ip_address' => $this->input->ip_address(),
    			'created_on' => date('Y-m-d H:i:s')
    		);
    	}
    	// ad_id, row_id and module_id are the unique keys
    	$this->db->on_duplicate('ad_keywords', $data);
    }
    
    /**
     * Admin Panel - save keywords with CPC and CPD
     * @param string $type - Tags/Categories/Cities
     * @return boolean
     */
    public function admin_save_costs($type) {
    	$this->form_validation->set_rules('field_' . $type .'[id][]', 'lang:label_id', 'trim|is_array');
    	$this->form_validation->set_rules('field_' . $type .'[name][]', 'lang:label_name', 'trim|is_array');
    	$this->form_validation->set_rules('field_' . $type .'[cpc][]', 'lang:label_cpc', 'trim|is_decimal|is_array');
    	$this->form_validation->set_rules('field_' . $type .'[cpd][]', 'lang:label_cpd', 'trim|is_decimal|is_array');
    	if ($this->form_validation->run() === false) {
    		return false;
    	}
    	$data = array();
    	foreach($_POST['field_' . $type]['id'] as $key => $value) {
    		$data[] = array(
    				'user_id' => $this->auth->user_id(),
    				'row_id' => $_POST['field_' . $type]['id'][$key],
    				'module_id' => gc_module_id($type),
    				'cpc' => $_POST['field_' . $type]['cpc'][$key],
    				'cpd' => $_POST['field_' . $type]['cpd'][$key],
    				'last_updated' => date('Y-m-d H:i:s'),
    				'ip_address' => $this->input->ip_address()
    		);
    	}
    	    	
    	// delete all rows where module_id = gc_module_id($type)
    	$this->db->where('module_id', gc_module_id($type))->delete('ad_costs'); 
    	// row_id and module_id are the unique keys
    	// insert data
    	$this->db->insert_batch('ad_costs', $data);
    }
    
    /**
     * On Listing/Module title select, fill the categories keywords with the user selected categories
     * 
     * @param int $id
     * @param string $table
     * @param int $type -> 0 - CPC, 1 - CPD
     * @param string $module
     * @return array
     */
    public function module_category_info($id, $table, $type = 0, $module = null) {
    	$result = array();
    	$categories = array();
    	$where = array('row_id' => $id, 'module_id' => gc_module_id($module));
    	$records = $this->db->get_where($table, $where)->result();
    	if($records) {
	    	foreach($records as $category) {
				$categories[] = $category->category_id;
	    	}
	    	$categories = array_unique($categories);
	    	$result = $this->bid_info('categories', null, $type, implode(',', $categories));
    	}
    	return $result;
    }
    
    /**
     * Ajax Query to display cost and bid information to the user
     * @param string $table
     * @param string $q
     * @param string $ids - In case of listings, and other modules load the keywords on title select
     * @param int $type -> 0 - CPC (Cost Per Click), 1 - CPD (Cost Per Day)
     * @return object
     */
    public function bid_info($table, $q, $type = 0, $ids = null) {
    	// get default costs from the settings table
    	$default = json_decode(settings_item('ad.costs'));
    	// set default cost based on the value set by the Admin
    	$cost = ($type == 0 ? $default->$table->cpc : $default->$table->cpd);
    	
    	$sql = "SELECT M.id, ";
    	if($table == 'cities') {
    	    $sql .= "CONCAT(M.name, ', ', ST.name, ', ', ST.country_iso) AS name";
    	} else {
    	    $sql .= "M.name";
    	}
    	if($type == 0) { // Cost Per Click
    		$sql .= ", CASE WHEN AC.cpc <> '' THEN AC.cpc ELSE $cost END AS cost";
    	} else { // Cost Per Day
    		$sql .= ", CASE WHEN AC.cpd <> '' THEN AC.cpd ELSE $cost END AS cost";
    	}
    	$sql .= ", CASE WHEN AK.bid <> '' THEN AK.cost ELSE $cost END AS bid";
    	$sql .= " FROM " . $this->db->dbprefix . $table ." M";
    	$sql .= " LEFT JOIN " . $this->db->dbprefix . "ad_costs AC ON AC.row_id = M.id AND AC.module_id = " . gc_module_id($table);
    	$sql .= " LEFT JOIN " . $this->db->dbprefix . "ad_keywords AK ON AK.row_id = M.id AND AK.module_id = " . gc_module_id($table) . " AND AK.status = 1 AND AK.type = $type";
    	if($table == 'cities') {
    	    $sql .= " LEFT JOIN " . $this->db->dbprefix . "states ST ON M.state_id = ST.id";
    	}
    	$sql .= " WHERE M.active = 1";
    	if(!empty($ids)) {
    		$sql .= " AND M.id IN ($ids)";
    	} else {
    		$sql .= " AND M.name LIKE '%" . escape($q) ."%'";
    	}
    	$result = $this->db->query($sql)->result();
    	return $result;
    }
    
    /**
     * Display cpc and cpd information to the Admin
     * @param string $table
     * @param string $q
     * @return object
     */
    public function admin_keyword_search($table, $q) {
    	// get default costs from the settings table
    	$default = json_decode(settings_item('ad.costs'));
    	// set default cost based on the value set by the Admin
    	$cpc = $default->$table->cpc;
    	$cpd = $default->$table->cpd;
    	 
    	$sql = "SELECT M.id, M.name";
    	$sql .= ", CASE WHEN AC.cpc <> '' THEN AC.cpc ELSE $cpc END AS cpc";
    	$sql .= ", CASE WHEN AC.cpd <> '' THEN AC.cpd ELSE $cpd END AS cpd";
    	$sql .= " FROM " . $this->db->dbprefix . $table ." M";
    	$sql .= " LEFT JOIN " . $this->db->dbprefix . "ad_costs AC ON AC.row_id = M.id AND AC.module_id = " . gc_module_id($table);
    	if(!empty($q)) {
    		$sql .= " WHERE M.name LIKE '%" . escape($q) ."%'";
    	}
    	$result = $this->db->query($sql)->result();
    	return $result;
    }
    
    /**
     * Display saved keyword with cost to the admin
     * @param number $count
     * @param string $module
     * @param string $where
     * @param string $sort_by
     * @param number $offset
     * @param number $limit
     * @return number
     */
    public function get_keyword_cost($count, $module, $where = null, $sort_by = 'id DESC', $offset = 0, $limit = 25) {
    	if($count == 0) {
    		$sql = "SELECT AC.row_id AS id, AC.module_id, AC.cpc, AC.cpd, AC.percent, M.name";
    	} else {
    		$sql = "SELECT COUNT(AC.row_id) AS TotalCount";
    	}
    	$sql .= " FROM " . $this->db->dbprefix . "ad_costs AC";
    	$sql .= " LEFT JOIN " . $this->db->dbprefix . $module ." M ON M.id = AC.row_id";
    	if(!empty($where)) {
    		$sql .= " WHERE $where";
    	}
    	if($count == 0) {
    		$sql .= " ORDER BY " . $sort_by;
    		if($limit != 0) {
    			$sql .= " LIMIT " . $offset . ", " . $limit;
    		}
    		return $this->db->query ( $sql )->result_array();
    	} else {
    		$result = $this->db->query ( $sql )->row();
    		return $result ? $result->TotalCount : 0;
    	}
    }
    
    /**
     * Update Ad View on Ad click or impression
     * Data is inserted only because of date and time tracking
     * Delete all records longer than the 90 days
     * Admin can set this duration through the admin panel
     * 
     * @param int $ad_id - All ads tracking
     * @param int $keyword_id - if the ad is a campaign
     * @param int $type
     * 1 - click and impression both in case of ad click 
     * 0 - impression only, ad not clicked by the visitor
     */
    public function update_view($ad_id, $keyword_id, $type = 1) {
    	$data = array(
    		'ad_id' => (int) $ad_id,
    		'keyword_id' => (int) $keyword_id,
    		'impression' => 1,
    		'click' => ($type == 1 ? 1 : 0),
    		'created_on' => date('Y-m-d H:i:s'),
    		'ip_address' => $this->input->ip_address()
    	);
    	$this->db->insert('ad_views', $data);
    }
    
    /**
     * Display Ad Views to the User
     * 
     * @param number $ad_id
     * @param number $user_id
     * @param number $limit
     * @param number $offset
     * @param string $role
     * @param string $redirect_url
     */
    public function view($ad_id, $user_id, $limit = 25, $offset = 0, $role = 'admin', $redirect_url = NULL) {
        $redirect_url = empty($redirect_url) ? gc_url_format('ads') : $redirect_url;
        if(empty($ad_id)) {
            Template::redirect($redirect_url);
        }
        $ad_info = $this->ads_model->find($ad_id);
        if(!$ad_info || $ad_info && $ad_info->user_id != $user_id && $role != 'admin') {
            Template::redirect($redirect_url);
        }
        
        // DATETIME PICKER
        Assets::add_js('moment.js');
        Assets::add_css ( 'bootstrap-datetimepicker.min.css' );
        Assets::add_js ( 'bootstrap-datetimepicker.min.js' );
        Assets::add_js(
            "
if($('#from').length) {
	$('#from').datetimepicker({format: 'YYYY-MM-DD HH:mm:ss', sideBySide: true});
}
if($('#to').length) {
	$('#to').datetimepicker({format: 'YYYY-MM-DD HH:mm:ss', sideBySide: true});
}
	        ", 'inline');
        $this->auth->restrict('Site.Business.Financial');
        
        // $offset = intval($this->input->get('per_page'));
        $where = 'ad_views.ad_id = ' . $ad_id;
        if(isset($_POST['save'])) {
            $this->load->library('form_validation');
            $this->form_validation->set_rules('from', 'lang:label_from', 'trim|valid_datetime');
            $this->form_validation->set_rules('to', 'lang:label_to', 'trim|valid_datetime');
            if ($this->form_validation->run() === TRUE) {
                if($this->input->post('from') && $this->input->post('to')) {
                    $this->session->set_userdata('ads_view', array('from' => $this->input->post('from'), 'to' => $this->input->post('to')));
                } else {
                    $this->session->unset_userdata('ads_view');
                    $_POST['from'] = '';
                    $_POST['to'] = '';
                }
            }
        }
        
        // Only Admin can purge an Ad View
        if($role == 'admin' && isset($_POST['delete'])) {
            $checked = $this->input->post('checked');            
            if (is_array($checked) && count($checked)) {
                $result = TRUE;
                foreach ($checked as $pid) {
                    $deleted = $this->db->delete('ad_views', $pid);
                    if ($deleted == false) {
                        $result = FALSE;
                    }
                }                
                if ($result) {
                    Template::set_message(count($checked) .' '. lang('delete_success'), 'success');
                } else {
                    Template::set_message(lang('delete_failure') . $this->pages_model->error, 'danger');
                }
            }
        }
        
        $session_where = $this->session->userdata('ads_view');
        if(!empty($session_where['from']) && !empty($session_where['to'])) {
            $where .= " AND ( ad_views.created_on BETWEEN '" . $session_where['from'] . "' AND '" . $session_where['to'] . "')";
        }
        
        // Sort By
        $order_by = $this->gc->sort('ads_view');
        $order_column = 'created_on';
        $order_sort = 'DESC';
        if(!empty($order_by)) {
            $order_column = $this->session->userdata('ads_view_sort')['column'];
            $order_sort = $this->session->userdata('ads_view_sort')['order'];
        }
        $this->db->limit($limit, $offset);
        $this->db->join('ad_keywords AK', 'AK.id = ad_views.keyword_id', 'LEFT');
        $records = $this->db->select('ad_views.*, AK.module_id')->where($where)->order_by($order_column, $order_sort)->get('ad_views')->result();
        $total_rows = $this->db->where($where)->count_all_results('ad_views');
        
        $this->load->library('pagination');
        $pagerUriSegment = ($role == 'admin' ? 6 : 4);
        $pager['base_url']    = ($role == 'admin' ? site_url(SITE_AREA . '/financial/ads/view') : gc_url_format('ads/view')) . '/' . $ad_id;
        $pager['total_rows']  = $total_rows;
        $pager['per_page']    = $limit;
        $pager['uri_segment'] = $pagerUriSegment;
        // Pagination using GET Method
        /*$config['reuse_query_string'] = TRUE;
         $config['enable_query_strings'] = TRUE;
         $config['page_query_string'] = TRUE;*/
        $this->pagination->initialize($pager);
        Template::set(array(
            'role' => $role,
            'cancel_url' => $redirect_url,
            'date_filter' => $session_where,
            'records' => $records,
            'toolbar_title' => lang('label_ad') . '::' . $ad_id,
            'page_title' => lang('ad_views')
        ));
        Template::set_view('ads/view');
    }
    
    /**
     * Allow user to view the Ad clicks and views
     * @param number $ad_id
     * @param number $keyword_id
     * @param string $from
     * @param string $to
     * @return object
     */
    public function reporting($ad_id, $keyword_id = 0, $from = null, $to = null) {
    	$sql = "SELECT SUM(impression) AS impressions, SUM(click) AS clicks";
    	$sql .= " FROM " . $this->db->dbprefix . "ad_views AV";
    	if($keyword_id != 0) {
    		$sql .= " WHERE AV.keyword_id = $keyword_id";
    	} else {
    		$sql .= " WHERE AV.ad_id = $ad_id";
    	}
    	if(!empty($from) && !empty($to)) {
    		$sql .= " AND (DATE(created_on) BETWEEN '" . $from ."' AND '" . $to ."')";
    	}
    	$sql .= " GROUP BY " . ($keyword_id != 0 ? $keyword_id : $ad_id);
    	$sql .= " ORDER BY impressions, clicks";
    	$result = $this->db->query($sql)->result();
    	return $result;
    }
    
    /**
     * Update Ad Status
     * 
     * This method is used by Cronjob/Order Confirmation/Admin Ad Index page
     * @param int $ad_id
     * @param int $new_status
     */
    public function update_ad_status($ad_id, $new_status) {
    	$this->lang->load('ads/ads');
    	if(!empty($ad_id) && !empty($new_status)) {
    		// get ad module_id
    		$ad_info = $this->ads_model->find($ad_id);
    		// get ad_type from the package
    		$this->load->model('ads/ad_packages_model');
    		$ad_package = $this->ad_packages_model->find($ad_info->package_id);
    		if($ad_info) {
    			$new_status = $new_status == 99 ? 0 : $new_status;
    			// Ad type is a banner
	    		if($ad_info->row_id != 0 && $ad_info->module_id != 0 && $ad_info->module_id == gc_module_id('banners')) {
	    			// check the banner existence, admin might have deleted it
	    			$this->load->model('banners/banners_model');
	    			$check_existence = $this->banners_model->count_by(array('id' => $ad_info->row_id));
	    			if($check_existence > 0) {
		    			// banner has already been added for this ad
		    			// i.e. update status only
		    			if($new_status == 1) { // activate the banner
		    				$this->db->where('id', $ad_info->row_id)->update('banners', array('status' => $new_status, 'modified_on' => date('Y-m-d H:i:s')));
		    			} else { // deactivate the banner
		    				$this->db->where('id', $ad_info->row_id)->update('banners', array('status' => 0, 'modified_on' => date('Y-m-d H:i:s')));
		    			}	
	    			} else {
	    				$this->add_banner($ad_info, $ad_package);
	    			}
	    		} else if($ad_package->banner_type_id != 0 && $ad_info->module_id == gc_module_id('banners')) {
	    			// Ad Type is a banner i.e. add a new banner using the information
	    			// already filled by the user before checkout and set status accordingly
	    			$this->add_banner($ad_info, $ad_package);		
	    		}
	    		// update ad status
	    		$this->db->where('id', $ad_id)->update('ads', array('status' => $new_status));
	    		// also activate all keywords
	    		if($new_status == 1) {
	    			// A keyword can have any one of these three states - Active/Inactive/Admin Disapproved
	    			// Activate only those keywords which are not disapproved by the Admin
	    			$this->db->where(array('ad_id' => $ad_id, 'status' => 0))->update('ad_keywords', array('status' => 1));
	    			// send higer bid notification email to other users
	    			$this->notify_user($ad_id, $ad_info);
	    		} else {
	    			// Deactive all keywords
	    			// Also perform the same action on Ad expiration
	    			$this->db->where(array('ad_id' => $ad_id, 'status' => 1))->update('ad_keywords', array('status' => 0));
	    		}	
	    		// Send Status Update Email
	    		$this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'ad_email_templates', 'module_table' => 'ads', 'action_type' => 'status_update', 'id' => $ad_id));
    		}
    	}
    }
    
    /**
     * Add Banner on Ad Approval
     * @param array $ad_info
     * @param array $ad_package
     */
    private function add_banner($ad_info, $ad_package) {
    	$banner_data = array(
    		'type_id' => $ad_package->banner_type_id,
    		'ad_id' => $ad_info->id,
    		'title' => lang('label_ad') . '-' . $ad_info->id . '::' . $ad_package->title,
    		'image' => 'banners/' . $ad_info->image,
    		'url' => $ad_info->url,
    		'target' => '_blank',
    		'start_date' => $ad_info->start_date,
    		'end_date' => $ad_info->end_date,
    		'max_impressions' => $ad_package->max_impressions,
    		'max_clicks' => $ad_package->max_clicks,
    		'html_text' => $ad_info->html_text,
    		'module_ids' => $ad_package->module_ids,
    	    'categories_include' => $ad_package->categories_include,
    	    'categories_exclude' => $ad_package->categories_exclude,
    	    'cities_include' => $ad_package->cities_include,
    	    'cities_exclude' => $ad_package->cities_exclude,
    	    'tags_include' => $ad_package->tags_include,
    	    'tags_exclude' => $ad_package->tags_exclude,
    	    'links_include' => $ad_package->links_include,
    	    'links_exclude' => $ad_package->links_exclude,
    		'status' => 1,
    		'created_on' => date('Y-m-d H:i:s')
    	);
    	$this->db->insert('banners', $banner_data);
    	$banner_id = (int) $this->db->insert_id();
    	if($banner_id > 0) {
	    	// insert banner id information in the ads table
	    	$this->db->where(array('id' => $ad_info->id))->update('ads', array('row_id' => $banner_id));
	    	
	    	// Add Categories, Keywords and Locations - Campaign Banner Only
	    	// Get Keywords
	    	$keywords = $this->db->where('ad_id', $ad_info->id)->get('ad_keywords')->result();
	    	if($keywords) {
	    	    $module_id_categories = gc_module_id('categories');
	    	    $module_id_cities = gc_module_id('cities');
	    	    $module_id_tags = gc_module_id('tags');
	    	    $banner_update_data = array(
	    	        'categories_include' => array(),
	    	        'categories_exclude' => null,
	    	        'cities_include' => array(),
	    	        'cities_exclude' => null,
	    	        'tags_include' => array(),
	    	        'tags_exclude' => null
	    	    );
	    	    foreach($keywords as $keyword) {
	    	        // Prepare categories data
	    	        if($keyword->module_id == $module_id_categories) {
	    	            $banner_update_data['categories_include'][] = $keyword->row_id;
	    	        }
	    	        // Prepare categories data
	    	        if($keyword->module_id == $module_id_cities) {
	    	            $banner_update_data['cities_include'][] = $keyword->row_id;
	    	        }
	    	        // Prepare categories data
	    	        if($keyword->module_id == $module_id_tags) {
	    	            $banner_update_data['tags_include'][] = $keyword->row_id;
	    	        }
	    	    }
	    	    $banner_update_data['categories_include'] = implode(',', $banner_update_data['categories_include']);
	    	    $banner_update_data['cities_include'] = implode(',', $banner_update_data['cities_include']);
	    	    $banner_update_data['tags_include'] = implode(',', $banner_update_data['tags_include']);
	    	    $this->db->where('id', $banner_id)->update('banners', $banner_update_data);
	    	}
    	}
    }
    
    /**
     * Display Ad Status to the User
     * @param int $id
     * @return string
     */
    public function ad_status($id, $lang_string_only = false) {
    	$status = 'us_inactive';
    	switch($id) {
    		case 0:
    			$status = 'us_inactive';
    			break;
    		case 1:
    			$status = 'us_active';
    			break;
    		case 2:
    			$status = 'waiting_for_approval';
    			break;
    		case 3: // cronjob to activate only those ads which are paused for the day
    			$status = 'daily_budget_exceeded';
    			break;
    		case 4:
    			$status = 'status_expired';
    			break;
    		case 5:
    		    $status = 'label_paused';
    		    break;
    	}
    	return $lang_string_only ? $status : lang($status);
    }
    
    /**
     * Purge Package information
     * Delete all ads, keywords and views related to this package
     * @param int $package_id
     */
    public function purge_package($package_id) {
    	// delete all ads first
    	$ads = $this->db->select('id')->where('package_id', $package_id)->get('ads')->result();
    	if($ads) {
    		foreach($ads as $ad) {
    		    $this->purge_ad($ad->id);		
    		}
    	}
    	
    	// Purge Translations
    	$this->translations->purge_tanslations('ad_packages', $package_id);
    	
    	// delete package
    	$this->db->where('id', $package_id)->delete('ad_packages');

    	// Purge all relations from the module_relations table
    	$this->gc->purge_relation($package_id, 'ads');
    	
    	return true;
    }
    
    /**
     * Purge Ad information
     * Delete all keywords and views related to this Ad
     * @param int $ad_id
     */
    public function purge_ad($ad_id) {
    	// delete any banner information related to this ad
    	$ad_info = $this->ads_model->find($ad_id);
    	if($ad_info->row_id != 0 && $ad_info->module_id == gc_module_id('banners')) {
    		$this->db->where('id', $ad_info->row_id)->delete('banners');
    	}
    	// delete all keywords and views related to this Ad
    	$keywords = $this->db->select('id')->where('ad_id', $ad_id)->get('ad_keywords')->result();
    	if($keywords) {
    		foreach($keywords as $keyword) {
    			$this->purge_keyword($keyword->id);
    		}
    	}
    	// delete Ad views - Ad may not be running as a campaign
    	$this->db->where('ad_id', $ad_id)->delete('ad_views');
    	// delete Ad
    	$this->db->where('id', $ad_id)->delete('ads');
    }
    
    /**
     * Purge Keyword
     * Delete all views related to this Ad
     * @param int $ad_id
     */
    public function purge_keyword($keyword_id) {
    	// delete all views specific to this keyword
    	$this->db->where('keyword_id', $keyword_id)->delete('ad_views');
    	// delete keyword
    	$this->db->where('id', $keyword_id)->delete('ad_keywords');
    }
    
    /**
     * On Tag/City/Category delete, purge information from the Ad module
     * @param int $id
     * @param string $module
     */
    public function purge_from_ad($id, $module) {
	    // Purge information from the advertisement ad_costs table
	    $this->db->where(array('row_id' => $id, 'module_id' => gc_module_id($module)))->delete('ad_costs');
	    // Get tag information from ad_keywords table
	    $ad_keywords = $this->db->select('id')->get_where('ad_keywords', array('row_id' => $id, 'module_id' => gc_module_id($module)))->result();
	    if($ad_keywords) {
	    	foreach($ad_keywords as $keyword) {
	    		$this->purge_keyword($keyword->id);
	    	}
	    }
    }
    
    /**
     * On listing and module row purge also delete all ads related to them.
     * 
     * Note: Currently ads are not being deleted as the user may use another listing/module with the same ad.
     * 
     * @param integer $row_id
     * @param string $module
     */
    public function module_row_purge($row_id, $module) {
    	$module_id = gc_module_id($module);
    	$result = $this->ads_model->select('id')->find_all_by(array('row_id' => $row_id, 'module_id' => $module_id));
    	if($result) {
    		foreach($result as $ad) {
    			$this->purge_ad($ad->id);
    		}
    	}
    }
    
    /**
     * Purge all Ad Views longer than 90 days via cronjob
     */
    public function purge_ad_views() {
    	$ads_view = json_decode(settings_item('ad.settings'));
    	$past_date = Date('Y-m-d', strtotime("-" .  $ads_view->ad_views_purge_duration . " days"));
    	//$sql = "DELETE FROM " . $this->db->dbprefix . "ad_views WHERE DATE(created_on) > (NOW() - INTERVAL 90 DAY)";
    	$sql = "DELETE FROM " . $this->db->dbprefix . "ad_views WHERE DATE(created_on) < '" . $past_date ."'";
    	$this->db->query($sql);
    }
    
    /**
     * Common function to display Ads for all modules
     * 
     * @param string $module - Banners/Listings/Videos and other modules
     * @param number $keyword_id
     * @param string $keyword_module - Tags/Categories/Cities only
     * 
     * @param array $params - Applicable only in case of banners
     * @param int $user_id - Applicable in case of Favorites Counts View Page
     * 
     * toggle_module - Listings/Videos and other modules
     * display_page - Page on which banner to be displayed
     * e.g. array('toggle_module' => 'listings', 'display_page' => 'home')
     * 
     * @return multitype:
     */
    public function ads($module, $keyword_id = 0, $keyword_module = NULL, $params = array(), $extra = array()) {
    	// Advertisement Module Changes
    	// Get IDs of all listings advertised using featured option of Ads module
    	$ids = NULL;
    	$ad_rows = array();
    	$result = array();
    	$ads = $this->ad_row_ids($module, $keyword_id, $keyword_module);
    	if(!$ads) { // return blank array
    		return array();
    	}
    	foreach($ads as $ad) {
    		// ad_id is used as a key for impression and click counter
    		$ad_rows[$ad->ad_id] = $ad->id;
    	}
    	$field = 'id';
    	if(!empty($ad_rows)) {
    		$ids = implode(',', $ad_rows);
    		switch($module) {
    			case 'banners':
    				$this->load->model('banners/banners_model');
    				$toggle_module = !empty($params['toggle_module']) ? $params['toggle_module'] : NULL;
    				$display_page = !empty($params['display_page']) ? $params['display_page'] : 'all';
    				$rows = $this->banners_model->ads_banner($ids, $toggle_module, $display_page);
    				break;
    			default:
    				$this->load->model('listings/listings_model');
    				$rows = $this->listings_model->ads_featured($module, $ids, $extra);
    		}
    	}
    
    	if($rows) {
    		// create an ad_url and redirect the user to this URL for click counter
    		foreach($rows as $row) {
    			$ad_url = array(
    				'ad_id' => array_search($row->$field, $ad_rows),
    				'url' => ($module == 'banners' ? $row->url : gc_url_format($module, $row->slug, $row->$field, ($module == 'listings' ? $row->city : NULL))),
    				'keyword_id' => $keyword_id,
    				'keyword_module' => $keyword_module
    			);
    			if($module == 'banners') {
    				$ad_url['banner_id'] = $row->id;
    			}
    			$row->ad_url = base64_encode(json_encode($ad_url));
    			$result[] = $row;
    		}
    	}
    	return $result;
    }
    
    /**
     * Get ID of featured/banner rows based on Ad pricing type
     * Pass these IDs as a parameter to module core methods e.g. listings_search
     * 
     * @param string $module - Banners/Listings/Videos and other modules
     * @param int $id - Tag ID/City ID/ Category ID
     * @param string $type - Tags/Cities/Categories
     * @param int $limit
     */
    /*public function ads($module, $id = 0, $type = null, $limit = 10) {
    	$ads = $this->get_row_ids($module, $id, $type, $limit);
    	$ids = array();
    	foreach($ads as $ad) {
    		// ad_id is used as a key for impression and click counter
    		$ids[$ad->ad_id] = $ad->id;
    	}
    	return $ids;
    }*/
    
    public function ad_row_ids($module, $keyword_id = 0, $keyword_module = null, $limit = 10) {
    	$sql = "SELECT DISTINCT(A.row_id) AS id, A.id AS ad_id";
    	$sql .= " FROM " . $this->db->dbprefix . "ads A";
    	// get specific ads only
    	if(!empty($keyword_id) && !empty($keyword_module)) {
    		$sql .= " INNER JOIN " . $this->db->dbprefix . "ad_keywords AK ON AK.ad_id = A.id AND AK.row_id = $keyword_id AND AK.module_id = " . gc_module_id($keyword_module) . " AND AK.status = 1";
    	}
    	$sql .= " WHERE A.row_id != 0 AND A.module_id = " . gc_module_id($module); // Non Empty Row
    	$sql .= " AND A.status = 1 AND A.deleted = 0";
    	//$sql .= " ORDER BY RAND()"; // Slowest
		$sql .= " AND A.id >= (SELECT FLOOR( MAX(id) * RAND()) FROM ". $this->db->dbprefix . "ads)"; // display rows on random basis
		$sql .= " GROUP BY A.id";
		if(!empty($keyword_id) && !empty($keyword_module)) { // Campaign
			$sql .= " ORDER BY AK.bid DESC";
		} else { // Fixed Pricing Ad
			$sql .= " ORDER BY A.price, A.id DESC";
		}
		$sql .= " LIMIT " . $limit;
		$result = $this->db->query($sql)->result();
    	return $result;
    }
    
    /**
     * Common function for impression counter
     * @param array $featured_rows
     */
    public function impression_counter($featured_rows) {
    	if($featured_rows) {
    		foreach($featured_rows as $featured_row) {
    			if(!empty($featured_row->ad_url)) {
    				$this->click_impression_counter($featured_row->ad_url);
    			}
    		}
    	}
    }
    
    /**
     * Update Ad Click and Impression
     * @param string $ad_url
     * @param string $type
     * @param string $return
     */
    public function click_impression_counter($ad_url, $type = 'impression', $redirect = false) {
    	$ad = json_decode(base64_decode($ad_url));
    	if(!empty($ad)) { // validate $_GET string
    		// banners click/impression handling
    	    if(isset($ad->banner_id)) {
    	        if(settings_item('site.minimal') == 1) {
    	            if($type == 'click') {
   	                    $this->db->query("UPDATE " . $this->db->dbprefix . "banners SET " . $type . "s = " . $type ."s + 1 WHERE id = " . $ad->banner_id);
    	            }
    	        } else {
    			     $this->db->query("UPDATE " . $this->db->dbprefix . "banners SET " . $type . "s = " . $type ."s + 1 WHERE id = " . $ad->banner_id);
    	        }
    		}
	    	if(!empty($ad->ad_id)) {
	    		$ad_id = $ad->ad_id;
	    		$data = array();
		    	$data['ad_id'] = $ad_id;
		    	$data[$type] = 1;
		    	$data['created_on'] = date('Y-m-d H:i:s');
		    	$data['ip_address'] = $this->input->ip_address();
		    	if(!empty($ad->keyword_id) && !empty($ad->keyword_module)) {
			    	$keyword = $this->db->select('id, bid')->get_where('ad_keywords', array('ad_id' => $ad_id, 'row_id' => $ad->keyword_id, 'module_id' => gc_module_id($ad->keyword_module)))->row();
			    	if($keyword) {
			    	    // get keyword name to store in ad_views table
			    	    $keyword_info = $this->db->select('name')->where('id', $ad->keyword_id)->get($ad->keyword_module)->row();
			    	    if($keyword_info) {
			    	        $data['keyword'] = $keyword_info->name;
			    	    }
			    		$data['keyword_id'] = $keyword->id;
			    		// update cost only on click
			    		if($type == 'click') {
			    			$data['cost'] = $keyword->bid;
			    			$row = $this->ads_model->select('user_id, duration, price, daily_budget')->find($ad_id);
			    			if($row) {
			    				$cost = decimal_convert(floatval($row->price) + floatval($data['cost']));
			    				$update_ad = array();
			    				$update_ad['price'] = $cost;
			    				
			    				// deduct the amount from the Wallet if it is a campaign else deduct the price
			    				// Campaign Ads with fixed duration are already paid by the user using checkout.
			    				if($row->duration == 0) { // do not deduct cash if the duration is fixed, the user has already paid the amount
			    				     $this->load->model('wallets/wallets_points_model');
			    				     $balance = $this->wallets_points_model->update_cash('wallets', $row->user_id, 'ads', $keyword->bid, 'debit');
			    				}
			    				
			    				// get cost of all the clicks within the last 24 hours from the ad_views table
			    				$todays_clicks = $this->db->query("SELECT SUM(cost) AS cost FROM " . $this->db->dbprefix . "ad_views WHERE ad_id = " . $ad_id . " AND created_on > (NOW() - INTERVAL 24 HOUR)")->row();
			    				$todays_clicks_cost = (!empty($todays_clicks->cost) ? decimal_convert($todays_clicks->cost) : 0);
			    				
			    				// if the cost is greater than the daily budget, pause the AD for the day
			    				// activate the AD automatically on the next day using cronjobs
			    				// if the AD is running from the campaign and the wallet balance is zero
			    				// then disable the AD until the user has zero balance
			    				// note: negative balances are allowed for wallets and points
			    				if(($todays_clicks_cost >= decimal_convert($row->daily_budget)) || (!$balance && $balance < $keyword->bid)) {
			    					$update_ad['status'] = 3;
			    				}
			    				$this->db->where('id', $ad_id)->update('ads', $update_ad);
			    			} 			
			    		}
			    	}
		    	}
		    	// update view
		    	$this->db->insert('ad_views', $data);
		    	if($redirect) {
		    		Template::redirect($ad->url);
		    	}
	    	} else if($redirect) {
	    		Template::redirect(!empty($ad->url) ? $ad->url : site_url());
	    	}
    	} else if($redirect) { // not a valid ad_id redirect user to the home page
    		Template::redirect(site_url());
    	}
    }
    
    /**
     * Gemailer - Get Ad
     * @param number $id
     * @param boolean $preview_only
     * @return array
     */
    public function get_ad($id, $preview_only = false, $do_conversion = true) {
        $record = $this->find_ads(0, $id, $preview_only);
        if($record) {
            $record = $record[0];
            if($do_conversion) {
                $record->expires_on = gc_date_format($record->expires_on, false);
                $record->status = $this->ad_status($record->status, true);
            }
        }
        return $record;
    }

    /**
     * CronJob - Get Ads which are expiring today
     * @param integer $status
     * @return array
     */
    public function find_ads($status, $id = 0, $preview_only = false) {
    	// id, package_id, title, contact_person, email, expires_on
    	$sql = "SELECT A.id, A.package_id, A.end_date AS expires_on, A.status, AP.price, AP.title, AP.title AS package_title, AP.description AS package_description, U.display_name AS contact_person, U.email AS email";
    	$sql .= ", A.module_id, A.row_id, A.image, A.url, AP.days_before_next_invoice, AP.invoice_first_reminder, AP.invoice_second_reminder, AP.invoice_third_reminder, AP.invoice_expiry, AP.order_status";
    	$sql .= " FROM " . $this->db->dbprefix . "ads A";
    	$sql .= " INNER JOIN " . $this->db->dbprefix . "users U ON U.id = A.user_id";
    	$sql .= " INNER JOIN " . $this->db->dbprefix . "ad_packages AP ON AP.id = A.package_id";
    	if($preview_only) { // Emailer Fields/Template Preview only
    	   $sql .= " ORDER BY RAND() LIMIT 1";
    	} else if(!empty($id)) {
    	    $sql .= " WHERE A.id = " . $id;
    	} else {
    	   $sql .= " WHERE A.end_date < '" . date ( 'Y-m-d' ) . "' AND A.end_date IS NOT NULL AND A.end_date != '0000-00-00' AND A.status != " . $status;
    	}
    	$result = $this->db->query($sql)->result();
    	return $result;
    }
    
    /**
     * CronJob - Activate all Ads where the Daily Budged is exceeded
     */
    public function activate_daily_budget_ads() {
    	$result = $this->ads_model->select('id')->find_all_by(array('status' => 3, 'deleted' => 0));
    	if($result) {
    		foreach($result as $row) {
    			$this->db->where('id', $row->id)->update('ads', array('status' => 1));
    		}
    	}
    }
    
    /**
     * CronJob - Send notification to all users
     * 
     * @param string $send_all
     * @param number $queue_override
     */
    public function notify_users_all($send_all = FALSE, $queue_override = 0) {
    	$ads = $this->ads_model->select('id, user_id')->find_all();
    	if($ads) {
    		foreach($ads as $ad) {
    			$this->notify_user($ad->id, $ad, $send_all, $queue_override);
    		}
    	}
    }
    
    /**
     * Send Higher Bid notification email to the user
     *
     * The user notifications are sent based on the Ad ID instead of the Keyword status change
     * Keywords are dependent on AD.
     * Do not send notifications to the user who own the Ad
     *
     * A new field last_notify_bid is added to the keywords table
     * This field will hold the value of the last bid email sent to the user
     * The Field is added to avoid sending the notification email twice to the same user for the same bid amount.
     *
     * @param integer $ad_id
     * @param boolean $send_all - If TRUE, avoid the last_notify_bid column and
     * send notification email twice to the same user for the same bid amount.
     */
    public function notify_user($ad_id, $ad = array(), $send_all = FALSE, $queue_override = 0) {
    	$users_list = array();
    	if(empty($ad)) {
    		$ad = $this->ads_model->select('user_id')->find($ad_id);
    	}
    	if($ad && $ad->user_id != 0) {
    		$ad_user_id = $ad->user_id;
    		// send email only on ad status change from inactive to active via Admin
    		// get all the keywords using the AD ID
    		$keywords = $this->db->get_where('ad_keywords', array('ad_id' => $ad_id, 'status' => 1))->result();
    		$users_list = $this->keywords_bid($keywords, $ad_id, $ad_user_id, $send_all);
    	}
    	if(!empty($users_list)) {
    		// send email
    		// user must receive email only once for the same bid amount on the same keyword
    		foreach($users_list as $user_id => $keywords) {
    			$user_info = $this->db->select('email, username, display_name, phone_number')->get_where('users', array('id' => $user_id))->row();
    			if($user_info) {
    			    $user_info->keywords = $keywords;
    			    // Notify Admin/User on Higher Bid Notification
    			    $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'ad_email_templates', 'module_table' => 'ads', 'action_type' => 'bid_notification', 'id' => $user_id, 'record' => $user_info));
    			}
    		}
    	}
    }
    
    /**
     * Get Keyword Bids with User ID
     * @param string $keywords
     * @param int $ad_id
     * @param int $ad_user_id
     * @param boolean $send_all - If TRUE, avoid the last_notify_bid column and
     * send notification email twice to the same user for the same bid amount.
     */
    private function keywords_bid($keywords, $ad_id, $ad_user_id, $send_all) {
    	$users_list = array();
    	if(!empty($keywords)) {
    		// get all rows where
    		// 1. row_id and module_id matches
    		// 2. notify_me is checked,
    		// 3. ad_id is not the current one,
    		// 4. status is active
    		// 5. the bid is lower then the placed bid, and
    		// 6. the last notify bid is also lower then the placed bid
    		foreach($keywords as $keyword) {
    			$where = array('row_id' => $keyword->row_id, 'module_id' => $keyword->module_id, 'notify_me' => 1, 'ad_id !=' => $ad_id, 'bid < ' => $keyword->bid, 'status' => 1);
    			if(!$send_all) {
    				$where['last_notify_bid < '] = $keyword->bid;
    			}
    			$result = $this->db->get_where('ad_keywords', $where)->result(); // Active keywords only
    			if($result) {
    				foreach($result as $row) {
    					// get the user_id
    					$ad_user = $this->ads_model->select('user_id')->find($row->ad_id);
    					// do not send notification if a user changed the bid himself
    					if($ad_user && $ad_user_id != $ad_user->user_id) {
    						// get keywords name for the email content
    						$module = gc_module_name($keyword->module_id);
    						$tag = $this->db->select('name')->where('id', $keyword->row_id)->get_where($module, array())->row();
    							
    						if($tag) {
    							// save the user_id in an array if it is not there already
    							if(!array_key_exists($ad_user->user_id, $users_list)) {
    								$users_list[$ad_user->user_id] = [];
    							}
    							 
    							// get the user existing bid to display it in email
    							$existing_user_bid = lang('label_not_applicable');
    							$where_existing = array('row_id' => $keyword->row_id, 'module_id' => $keyword->module_id, 'ad_id' => $row->ad_id);
    							$bid_info = $this->db->select('bid')->get_where('ad_keywords', $where_existing)->row();
    							if($bid_info) {
    								$existing_user_bid = $bid_info->bid;
    								$this->db->where($where_existing)->update('ad_keywords', array('last_notify_bid' => $keyword->bid));
    							}
    								
    							// save them in an array
    							// the same user can have multiple keywords, therefore send all keyword list to the user via single email
    							$tag = array('name' => $tag->name, 'module' => $module, 'existing_bid' => $existing_user_bid, 'new_bid' => $keyword->bid);
    							$users_list[$ad_user->user_id][] = $tag;
    						}
    					}
    				}
    			}
    		}
    	}
    	return $users_list;
    }
    
    /**
     * Higher Bid Notification Email
     * @param array $user_info
     * @param array $keywords
     */
    public function higher_bid_notification($user_info = null) {
        $this->lang->load('ads/ads');
        if(empty($user_info)) { // template/fields preview
            $user_info = $this->db->select('email, username, display_name, phone_number')->where('role_id != 1')->order_by('RAND()')->get('users')->row();
            if($user_info) {
                $user_info->keywords = array(
                    array(
                        'name' => 'New York',
                        'module' => 'cities',
                        'existing_bid' => 10,
                        'new_bid' => 12
                    ),
                    array(
                        'name' => 'Business Professional & Services',
                        'module' => 'categories',
                        'existing_bid' => 7,
                        'new_bid' => 9
                    )
                );
            }
        }
        if(empty($user_info)) {
            return false;
        }
        $keywords = $user_info->keywords;
    	$tbody = '';
    	foreach($keywords as $keyword) {
    		$tbody .= '<tr>';
    		$tbody .= '<td>' . $keyword['name'] . '</td>';
    		$tbody .= '<td>' . lang($keyword['module']) . '</td>';
    		$tbody .= '<td>' . $keyword['existing_bid'] . '</td>';
    		$tbody .= '<td>' . $keyword['new_bid'] . '</td>';
    		$tbody .= '</tr>';
    	}    	 
    	$keywords_table = '<table style="width: 100%;">
        	<thead>
        		<tr>
        			<td>' . lang('label_keyword_name') . '</td>
        			<td>' . lang('label_keyword_type') . '</td>
        			<td>' . lang('label_existing_bid') . '</td>
        			<td>' . lang('label_new_bid') . '</td>
        		</tr>
        	</thead>
	        <tbody>' . $tbody . '
	        </tbody>
        </table>';
    	$user_info->tags = $keywords_table;
    	return $user_info;
    }
}