<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Online Software Solutions							   # ||
|| # Copyright (C) 2021 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Affiliates controller
 */
require APPPATH . "libraries/Chart.php";
class Affiliates extends Front_Controller {
	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct() {
	    parent::__construct();
		$this->lang->load('affiliates/affiliates');
		$this->lang->load('commissions/commissions');
		$this->load->model('affiliates/affiliates_model');
		$this->affiliates_model->affiliate_info();
		// load css and js files
		$this->gc->load_assets('modules/affiliates');
		if($this->auth->is_logged_in() && $this->uri->segment(2) != gc_uri_string('form') && $this->uri->segment(2) != 'get_data') {
		     $this->accept_terms_conditions(); // do not call on profile page
		}
		// clear affiliate_info session
		if(isset($_POST['refresh'])) {
		    $this->session->unset_userdata('affiliate_info');
		    Template::redirect(current_url() . (!empty($_SERVER['QUERY_STRING']) ? '?' . $_SERVER['QUERY_STRING'] : ''));
		}
	}
	
	public function index() {
	    // Assets::add_js('jquery.nice-select.min.js');
	    if(!empty($this->session->userdata('affiliate_info')) && $this->auth->is_logged_in()) {
	        $settings = gc_json_settings('payout.settings');
	        $user_id = $this->auth->user_id();
	        $this->lang->load('payouts/payouts');
	        $this->load->model('commissions/commissions_model');
	        $commission_amount = $this->commissions_model->payout_user_commission_amount($user_id, 2);
	        if(isset($_POST['payout_request'])) {
	            $this->load->model('payouts/payouts_model');
	            $payout_ref_no = $this->payouts_model->payout_request($user_id, gc_module_id('affiliate_users'), $commission_amount->commission_amount);
	            if(!$payout_ref_no) {
	                Template::set_message(lang('payout_request_error'), 'danger');
	            } else {
	                Template::set_message(sprintf(lang('payout_request_success'), $payout_ref_no), 'success');
	                Template::redirect(current_url());
	            }
	        }
	        // Display Chart
    	    Assets::add_js('moment.js');
    	    Assets::add_css('bootstrap-datetimepicker.min.css' );
    	    Assets::add_js('bootstrap-datetimepicker.min.js' );
    	    Assets::add_css('chart.css');
    	    Assets::add_js('Chart.min.js');
    	    // Copy and share affiliate code
    	    Assets::add_js('clipboard.min.js');
    	    Assets::add_js('new ClipboardJS(".copy-text");', 'inline');
    	    Assets::add_css('jssocials.css');
    	    Assets::add_js('jssocials.min.js');
    	    Assets::add_js(
    	        '$(".sharing").each(function() {
			$(this).jsSocials({
				url : $(this).data("url"),
				text: $(this).data("txt"),
				showLabel : false,
				shares: ["email", "twitter", "facebook", "googleplus", "linkedin", "pinterest", "stumbleupon", "whatsapp"]
			});
		});', 'inline'
    	        );
    	    $this->session->set_userdata('chart_query_params', array('user_id' => $user_id));
    	    $sales_by_months = $this->gcrud->chart('months', array('status_id' => 2));
    	    Assets::add_js($this->load->view('common/_chart_js', array('field_id' => 'sixMonthSales', 'labels' => $sales_by_months['labels'], 'data' => $sales_by_months['data'], 'label' => lang('commission_total'), 'with_dates' => false, 'user_id' => $user_id), true), 'inline');
    	    $result = $this->gcrud->chart('current_month', array('status_id' => 2));
    	    Assets::add_js($this->load->view('common/_chart_js', array('field_id' => 'monthlyChart', 'labels' => $result['labels'], 'data' => $result['data'], 'label' => lang('commission_total'), 'with_dates' => true, 'user_id' => $user_id), true), 'inline');
    	    $summary = $this->affiliates_model->summary_members($user_id);
    	    if($commission_amount->commission_amount && $commission_amount->commission_amount >= $settings['minimum_payout']) {
    	        $payout_request = $this->db->where(array('user_id' => $user_id, 'module_id' => gc_module_id('affiliate_users'), 'status' => 0))->get('payout_requests')->row();
    	    }
    	    Template::set(array(
    	        'page' => 'home',
    	        'record' => $summary,
    	        'commission_amount' => $commission_amount->commission_amount, // Confirmed Commissions Amount
    	        'minimum_payout' => $settings['minimum_payout'],
    	        'top_referred_urls' => $this->affiliates_model->top_referred_urls(array('user_id' => $user_id, 'distinct_field' => 'source_page')),
    	        'payout_request_number' => !empty($payout_request) ? $payout_request->request_number : null
    	    ));
	    } else {
	        Template::set(array(
	            'page' => 'home'
	        ));
	    }
	    // if the user is not logged in
	    // redirect user directly to the affiliates page after login
	    if(!$this->auth->is_logged_in()) {
	        $this->session->set_userdata('login_redirect_url', current_url());
	    }
	    
	    Assets::add_js('_affiliates.js');
	    Assets::add_css('_affiliates.css');
	    Template::render($this->affiliates_model->layout);
	}
	
	/**
	 * Display Materials to the User
	 * @param string $type
	 * @param number $is_training
	 */
	public function materials($type = 'banner', $is_training = 0) {
	    $this->is_affiliate();
	    $this->affiliates_model->affiliate_materials($this->auth->user_id(), $type, $is_training);
	    Assets::add_js('_affiliates.js');
	    Assets::add_css('_affiliates.css');
	    Template::render($this->affiliates_model->layout);
	}
	
	/**
	 * Display Referral Traffics to the User
	 * @param number $page_id
	 * 1 - affiliate_referrals
	 * 2 - affiliate_referrals_unique
	 * 3 - commissions
	 */
	public function logs($page_id = 1) {
	    $this->is_affiliate();
	    $user_id = $this->auth->user_id();
	    $this->affiliates_model->logs($page_id, $user_id);
	    Template::set(array(
	        'page' => 'referrals'
	    ));
	    Assets::add_js('_affiliates.js');
	    Assets::add_css('_affiliates.css');
	    Template::render($this->affiliates_model->layout);
	}
	
	/**
	 * Affiliates Data Tables Data
	 * Common function for commissions, referrals, referrals_unique etc.
	 */
	public function get_data() {
	    $this->auth->restrict();
	    $affiliate_datatables = $this->session->userdata('affiliate_datatable');
	    if(!empty($affiliate_datatables)) {
	        $where = '';
	        if(!empty($affiliate_datatables['where'])) {
	            $where = $affiliate_datatables['where'];
	        }
	        $this->gcrud->get_data(array_keys($affiliate_datatables['columns']), $where, $affiliate_datatables['module'], $affiliate_datatables['prefix']);
	    } else {
	       echo '';
	    }
	}
	
	/**
	 * Display Hierarchy Tree to the User
	 * @param number $affiliate_id - The admin can also view a specific user members
	 */
	public function tree() {
	    $record = $this->is_affiliate();
	    $this->affiliates_model->tree($this->auth->user_id());
	    Template::set(array(
	        'page' => 'tree'
	    ));
	    Assets::add_js('_affiliates.js');
	    Assets::add_css('_affiliates.css');
	    Template::render($this->affiliates_model->layout);
	}
	
	/**
	 * Comment has children hierarchy
	 * The first element of the array is not a multidimensional array
	 * @param array $tree
	 */
	private function print_tree($children, $has_children) {
	    $this->tree_html .= '<ul>' . PHP_EOL;
	    foreach($children as $child) {
	        $this->child_view($child, $has_children);
	    }
	    $this->tree_html .= '</ul>';
	}
	
	/**
	 * Display Tree View
	 * @param array $affiliate
	 */
	private function child_view($affiliate) {
	    $this->tree_html .= '<li>' . PHP_EOL;
	    $this->tree_html .= '<a href="' . gc_url_format('users/user') . '?id=' . $affiliate['user_id'] . '" target="_blank" title="' . $affiliate['name'] . '">' . $affiliate['name'] . '</a>';
 	    if(!empty($affiliate['children'])) {
	        $this->print_tree($affiliate['children'], true);
	    }
	    $this->tree_html .= '</li>';
	}
	
	/**
	 * Link Cloaking
	 * @param string $str e.g. https://www.goclixy.com/affiliates/cloacking/1-1-2
	 */
	public function cloaking($str) {
	    $result = $this->affiliates_model->cloaking($str);
	    // amid - affiliate material id
	    // redirect user to the appropriate page
	    if(!empty($result['redirect_url'])) {
	        $redirect_url = $result['redirect_url'];
	        $last_four_character = substr($redirect_url, -4);
	        if($last_four_character == '.pdf') {
	            $this->read_pdf_online($redirect_url);
	        } else {
	            redirect($result['redirect_url'] . '?affmid=' . $str);
	        }
	    } else {
	        redirect(site_url());
	    }
	}	
	
	/**
	 * Add/Edit Affiliates Data
	 *
	 * @return void
	 */
	public function form() {
	    $module = 'affiliate_users';
	    // The user must register through the Normal Registration Form.
	    // After registration, the user has to fill the affiliates form. 
	    // The affiliate pages will be showed only if there is an entry in the affiliate users table.
	    // Fields will be displayed from the fields module.
	    $this->auth->restrict();
	    $user_id = $this->auth->user_id();	    
	    $record = $this->db->where('user_id', $user_id)->get('affiliate_users')->row();
	    if($record) {
	        $action = 'edit';
	        $id = $record->id;
	        $page = 'profile';
	    } else {
	        $action = 'create';
	        $id = 0;
	        $page = 'register';
	    }
	    $params = array(
	        'selectize' => true,
	        'model' => 'affiliates',
	        'module_name' => 'affiliates',
	        'dynamic_fields' => true,
	        'render' => $this->affiliates_model->layout,
	        'id' => $id,
	        'layout' => 'fieldset',
	        'redirect_url' => gc_url_format('affiliates')
	    );
	    if($action == 'create') {
	        $record = $this->db->where('id', $this->auth->user_id())->get('users')->row();
	        if($record) {
	            $record->name = $record->display_name;
	        }
	        $params['record'] = $record;
	    }
	    // store custom_fields in the affiliates table,
	    // store hierarchy relation
	    // counter after post update
	    $params['method'] = 'after_affiliate_save';
	    Template::set('page', $page);
	    Assets::add_js('_affiliates.js');
	    Assets::add_css('_affiliates.css');
	    $this->gcrud->form($module, $action, 'frontend', $params);
	}
	
	/**
	 * If the terms and conditions page is updated, the member must accept it before further proceed
	 */
	public function accept_terms_conditions() {
	    $affiliate = $this->session->userdata('affiliate_info');
	    if(isset($_POST['save'])) {
	        $this->load->library('form_validation');
	        $this->form_validation->set_rules('agree', 'lang:agree_terms_of_service', 'trim|required');	        
	        if ($this->form_validation->run() === true) {
	            $this->auth->restrict();
	            $user_id = $this->auth->user_id();
	            if($user_id > 0) {
	               // update the modified_on date
	               $this->db->where('user_id', $user_id)->update('affiliate_users', array('modified_on' => date('Y-m-d H:i:s')));
	               $affiliate->terms_accepted = 1;
	               $this->session->set_userdata('affiliate_info', $affiliate);
	               Template::redirect(gc_url_format('affiliates'));
	            }
	        }
	    }
	    if(!empty($affiliate) && (!isset($affiliate->terms_accepted) || (isset($affiliate->terms_accepted) && $affiliate->terms_accepted == 0))) {
    	    $page_id = gc_json_settings('affiliate.settings', 'terms_conditions');
    	    $record = $this->db->where('id', $page_id)->get('pages')->row();
    	    if(!$record) {
    	        Template::redirect(site_url());
    	    }
    	    $record = $this->translations->get_translations('pages', $record->id, $record, 1);
    	    if($record->modified_on > $affiliate->modified_on) {
    	        Template::set(array(
    	            'row' => $record,
    	            'page' => 'home'
    	        ));
    	        // display form
    	        Assets::add_js('_affiliates.js');
    	        Assets::add_css('_affiliates.css');
    	        Template::set_view('terms_conditions');
    	        Template::render($this->affiliates_model->layout);
    	    } else {
    	        $affiliate->terms_accepted = 1;
    	        $this->session->set_userdata('affiliate_info', $affiliate);
    	        return true;
    	    }
	    } else {
	        return true;
	    }
	}
	
	/**
	 * View Affiliates
	 * @param string $filter
	 * @param number $offset
	 */
	public function view($filter = 'all', $offset = 0) {
	    $module = 'affiliates';
	    $user_id = $this->auth->user_id();
	    $extra = array(
	        'model_folder' => 'commissions',
	        'model' => 'commissions',
	        'context' => 'financial',
	        'fields' => $module,
	        'prefix' => 'commission',
	        'query_where' => 'M.ref_by = ' . $user_id,
	    );
	    $this->gc->module_view($module, $filter, $offset, 'frontend', $extra);
	}
	
	/**
	 * Display Text Ad to the User
	 * @param string $str - e.g. 6-1-11
	 */
	public function text($str) {
	    echo $this->affiliates_model->prepare_ad($str, 'text');
	}
	
	/**
	 * Display Video Ad to the User
	 * @param string $str - e.g. 6-1-11
	 */
	public function video($str) {
		header('Access-Control-Allow-Origin: *');
        header('Content-Type: application/json');
        $result = $this->affiliates_model->prepare_ad($str, 'video');
	    // echo json_encode($result); 
	    echo $result;
	    // echo $this->affiliates_model->prepare_ad($str, 'video');
	}
	
	/**
	 * Check whether the logged in user is an affiliate or not
	 * @return boolean
	 */
	private function is_affiliate() {
	    $user_id = $this->auth->user_id();
	    // only logged in users can view the affiliates information
	    $this->auth->restrict();
	    // check whether the logged in user is an affiliate or not
	    $record = $this->affiliates_model->get_affiliate($user_id);
	    if($record) {
	        return $record;
	    } else {
	        Template::redirect(gc_url_format('members'));
	    }
	}
	
	/**
	 * View/PDF Invoice
	 */
	public function view_invoice() {
	    if(empty($this->session->userdata('affiliate_info')) && $this->auth->role_id() != 1) {
	        die(lang('payout_invoice_error'));
	    }
	    $this->lang->load('payouts/payouts');
	    if(empty($_GET['id'])) {
	        die(lang('payout_invoice_error'));   
	    }
	    $payout_id = $_GET['id'];
	    $view = isset($_GET['view']) ? $_GET['view'] : 0;
	    $this->load->library('crypt');
	    $payout_id = (int) $this->crypt->decode($payout_id);
	    if(empty($payout_id)) {
	        die(lang('payout_invoice_error'));
	    }
	    $this->lang->load('orders/orders');
	    $this->load->model('payouts/payouts_model');
	    $this->load->model('commissions/commissions_model');
	    $this->payouts_model->email_print($payout_id, $view == 1 ? 'view' : 'pdf', array(), 'frontend');
	}
	
	/**
	 * Display a list of faqs data.
	 *
	 * @return void
	 */
	public function faqs($offset = 0) {
	    $records = $this->gcrud->get_all_crud($count = 0, 'status = 1 AND deleted = 0', 'M.priority ASC', 5000, 0, array('module' => 'affiliate_faqs'));
	    Template::set(array(
	        'records' => $records,
	        'page' => 'faqs'
	    ));
	    Assets::add_js('_affiliates.js');
	    Assets::add_css('_affiliates.css');
	    Template::render($this->affiliates_model->layout);
	}
	
	/**
	 * Display a list of paid commission levels data.
	 *
	 * @return void
	 */
	public function packages($offset = 0) {
	    Template::set(array(
	        'toolbar_title' => lang('gc_menu_packages'),
	        'page' => 'paid_levels'
	    ));
	    Assets::add_js('_affiliates.js');
	    Assets::add_css('_affiliates.css');
	    Assets::add_css('members/membership.css');
	    Assets::add_js('members/membership.js');
	    $layout = $this->gc->load_assets('affiliates/packages');
	    Template::set_view($layout['view']);
	    Template::render($layout['layout']);
	}
	private function _packages($offset = 0) {
	    $this->lang->load('commissions/commissions');
	    $user_id = (int) $this->auth->user_id();
	    // get already ordered commission levels from the order_items table
	    $ordered_levels = $this->db->where('module_id = ' . gc_module_id('commission_levels') . ' AND user_id = ' . $user_id . ' AND status_id = 2 AND deleted = 0')->get('order_items')->result();
	    $levels_ordered = array();
	    if($ordered_levels) {
	        foreach($ordered_levels as $ordered_level) {
	            $levels_ordered[$ordered_level->row_id] = $ordered_level->row_id;
	        }
	    }
	    // also check user level
	    $user_level = $this->db->where('user_id', $user_id)->get('affiliate_users')->row();
	    if($user_level) {
	        $levels_ordered[$user_level->level_id] = $user_level->level_id;
	    }
	    // get packages
	    $packages = $this->translations->get_translated_rows('commission_levels', array('id', 'name', 'description', 'icon', 'price', 'duration', 'highlight'), 'price != 0 AND status = 1', 'priority ASC', 50, 'name AS title');
		if($packages) {
		    // remove already ordered levels
		    foreach($packages as $key => $package) {
    		    if(in_array($package->id, $levels_ordered)) {
    		        unset($packages[$key]);
    		    }
		    }
		}
		$this->affiliates_model->affiliate_info(true); // reset the commission level
		if(empty($packages)) {
		    // redirect user to the affiliates page
		    Template::set_message(lang('msg_no_more_packages'), 'danger');
		    Template::redirect(gc_url_format('affiliates'));
		}
		// redirect user to the checkout page
		$this->session->set_userdata('order_type', 'commission_levels');
	    $records = array();
	    Template::set(array(
	        'toolbar_title' => lang('gc_menu_packages'),
	        'page' => 'paid_levels',
	        'records' => $packages
	    ));		
	    Assets::add_js('index.js');
	    Assets::add_css('index.css');
	    Assets::add_js('_affiliates.js');
	    Assets::add_css('_affiliates.css');
	    Assets::add_css('members/membership.css');
	    Assets::add_js('members/membership.js');
	    $layout = $this->gc->load_assets('affiliates/packages');
	    Template::set_view($layout['view']);
	    Template::render($layout['layout']);
	}
	
	/**
	 * Download function that you can use to download any file
	 * @param string $path
	 */ 
	private function download_file($path) {
        if (file_exists($path) && is_file($path)) {
            // file exist
            header('Content-Description: File Transfer');
            header('Content-Type: application/octet-stream');
            header('Content-Disposition: attachment; filename=' . basename($path));
            header('Content-Transfer-Encoding: binary');
            header('Expires: 0');
            header('Cache-Control: must-revalidate');
            header('Pragma: public');
            header('Content-Length: ' . filesize($path));
            set_time_limit(0);
            @readfile($path);//"@" is an error control operator to suppress errors
        } else {
            // file doesn't exist
            die('Error: The file ' . basename($path) . ' does not exist!');
        }
    }
    
    /**
	 * Function for opening pdf files in the browser online
	 * @param string $path
	 */ 
    private function read_pdf_online($path) {
        if (file_exists($path) && is_file($path)) {
            // file exist
            header("Pragma: public"); // required
            header("Expires: 0");
            header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
            header("Cache-Control: private",false); // required for certain browsers
            header("Content-Type: application/pdf");
            header("Content-Transfer-Encoding: binary");
            header("Content-Length: ".filesize($path));
            @readfile( $path );
        } else {
            // file doesn't exist
            die('File Not Found: ' . basename($path));
        }
    }
    
    /**
     *  Commission Level Checkout
     */ 
    public function checkout() {
        if(empty($_GET['id'])) {
            Template::redirect(site_url());
        }
        $id = $_GET['id'];
        $this->load->library('crypt');
        $id = (int) $this->crypt->decode($id);
        $commission_level = $this->db->where('id', $id)->get('commission_levels')->row();
        if(!$commission_level) {
            Template::redirect(site_url());
        }
        $this->load->model('orders/orders_model');
		$this->orders_model->cart($id, 0, 1, 1, 'commission_levels', false);
		Template::redirect(gc_url_format('members/checkout'));
    }
}