<?php defined('BASEPATH') || exit('No direct script access allowed');

/**
 * Financial controller
 */
class Financial extends Admin_Controller {
    
    protected $permissionView = 'Affiliates.Financial.View';
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
        
        $this->auth->restrict($this->permissionView);
        clear_cache('affiliates');
        $this->load->model('affiliates/affiliates_model');
        $this->lang->load('affiliates');
        $this->lang->load('coupons/coupons');        
        $this->form_validation->set_error_delimiters("<span class='error'>", "</span>");        
        Template::set_block('sub_nav', 'financial/_sub_nav');
        Assets::add_module_js('affiliates', 'affiliates.js');
    }
    
    /**
     * Display Forum Summary to the User
     */
    public function index() {
        
        // Display Chart
        Assets::add_js('moment.js');
        Assets::add_css('bootstrap-datetimepicker.min.css' );
        Assets::add_js('bootstrap-datetimepicker.min.js' );
        Assets::add_css('flags.css');
        Assets::add_css('chart.css');
        Assets::add_js('Chart.min.js');
        $this->session->unset_userdata('chart_query_params');
        
        // Six Months Filter
        $six_month_chart = $this->gcrud->chart('months', array('table' => 'affiliate_referrals', 'field' => 'counter'));
        Assets::add_js($this->load->view('common/_chart_js', array('table' => 'affiliate_referrals', 'field_id' => 'sixMonthChart', 'labels' => $six_month_chart['labels'], 'data' => $six_month_chart['data'], 'total' => $six_month_chart['total'], 'label' => lang('affiliate_total_referrals'), 'with_dates' => false), true), 'inline');
        
        // Monthly Filter
        $result = $this->gcrud->chart('current_month', array('table' => 'affiliate_referrals', 'field' => 'counter'));
        Assets::add_js($this->load->view('common/_chart_js', array('table' => 'affiliate_referrals', 'field_id' => 'monthlyChart', 'labels' => $result['labels'], 'data' => $result['data'], 'total' => $result['total'], 'label' => lang('affiliate_total_referrals'), 'with_dates' => true), true), 'inline');
        
        // Traffic By Devices
        $traffic_by_device = $this->affiliates_model->traffic_filter();        
        if(!empty($traffic_by_device)) {
            Assets::add_js($this->load->view('common/_chart_js', array('field_id' => 'deviceChart', 'labels' => $traffic_by_device['labels'], 'data' => $traffic_by_device['data'], 'label' => lang('affiliate_total_referrals'), 'with_dates' => false, 'type' => 'pie'), true), 'inline');
        }
        
        // Summary
        $record = $this->affiliates_model->summary();
        $tabular_blocks = array();
        // $tabular_blocks['top_visited'] = $this->forums_model->manage(0, null, 'M.hits DESC', 20, 0, array('module' => 'threads', 'prefix' => 'forum'));
        // $tabular_blocks['recently_added'] = $this->forums_model->manage(0, null, 'M.created_on DESC', 20, 0, array('module' => 'threads', 'prefix' => 'forum'));
        // $tabular_blocks['latest_posts'] = $this->forums_model->manage(0, null, 'M.created_on DESC', 20, 0, array('module' => 'posts', 'prefix' => 'forum'));
        $record->tabular_blocks = $tabular_blocks;
        //$record->recently_viewed = $this->forums_model->manage(0, null, 'M.last_viewed DESC', 20, 0, array('module' => 'views'));
        Template::set(array(
            'toolbar_title' => lang('gc_menu_affiliates') . ' - ' . lang('gc_menu_summary'),
            'record' => $record,
            'top_referrers' => $this->affiliates_model->traffic_filter('user_id', null, false),
            'traffic_by_countries' => $this->affiliates_model->traffic_filter('country_abbrev', 'country, country_abbrev', false)
        ));
        Template::render();
    }
    
    /**
     * Affiliate Detail Page
     * @param number $id - User ID
     */
    public function detail($id, $field = 'user_id') {
        $record = $this->affiliates_model->get_affiliate_user($id, true, $field);
        if(!$record) {
            Template::redirect(site_url(SITE_AREA . '/financial/affiliates'));
        }
        $this->load->model('commissions/commissions_model');        
        // DATE TIME PICKER
        Assets::add_js('moment.js');
        Assets::add_css('bootstrap-datetimepicker.min.css' );
        Assets::add_js('bootstrap-datetimepicker.min.js' );        
        Assets::add_css('chart.css');
        Assets::add_js('Chart.min.js');
        
        // Traffic By Devices
        $traffic_by_device = $this->affiliates_model->traffic_filter('user_id', null, true, array('user_id' => $record->user_id));
        if(!empty($traffic_by_device['labels'])) {
            Assets::add_js($this->load->view('common/_chart_js', array('field_id' => 'deviceChart', 'labels' => $traffic_by_device['labels'], 'data' => $traffic_by_device['data'], 'label' => lang('affiliate_total_referrals'), 'with_dates' => false, 'type' => 'pie'), true), 'inline');
        }
        // Display last months sales graph
        $sales_by_months = $this->gcrud->chart('months', array('user_id' => $record->user_id, 'status_id' => 2)); //$this->commissions_model->sales_by_months($record->user_id);
        $result = $this->gcrud->chart('current_month', array('user_id' => $record->user_id));
        $this->session->set_userdata('chart_query_params', array('user_id' => $record->user_id)); // Display user specific charts
        Assets::add_js($this->load->view('common/_chart_js', array('field_id' => 'sixMonthSales', 'labels' => $sales_by_months['labels'], 'data' => $sales_by_months['data'], 'label' => lang('commission_earned'), 'with_dates' => false), true), 'inline');
        Assets::add_js($this->load->view('common/_chart_js', array('field_id' => 'monthlyChart', 'labels' => $result['labels'], 'data' => $result['data'], 'label' => lang('commission_earned'), 'with_dates' => true), true), 'inline');
        Assets::add_module_css('affiliates', 'detail.css');
        $summary = $this->affiliates_model->summary_members($record->user_id);
        // Back or Cancel URL - Back to the Index page after viewing commissions, referral traffic etc.
        $this->session->set_userdata('referrer_page', current_url());
        Template::set(array(
            'toolbar_title' => $record->name,
            'record' => $record,
            'summary' => $summary,
            'other_info' => $this->affiliates_model->summary_members($record->user_id),
            'traffic_by_device' => $traffic_by_device,
            'top_referred_urls' => $this->affiliates_model->top_referred_urls(array('user_id' => $record->user_id)),
            'traffic_by_countries' => $this->affiliates_model->traffic_filter('country_abbrev', 'country, country_abbrev', false, array('user_id' => $record->user_id))
        ));
        Template::render();
    }
    
    /**
     * Affiliates information to be visible to the user
     * @param number $user_id
     * @param number $page_id
     * @param string $type
     * @param number $is_training
     */
    public function info($user_id, $type_id = 1, $page_id = 1, $type = 'banner') {
        switch($type_id) {
            case 0:
                break;
            case 1:
                $this->affiliates_model->affiliate_materials($user_id, $type);
                break;
            case 2:
                $this->affiliates_model->logs($page_id, $user_id);
                break;
            case 3:
                $this->affiliates_model->tree($user_id);
                break;
            default:
                Template::redirect(site_url(SITE_AREA . '/financial/affiliates'));
        }
        Template::set('role', 'admin');
        Template::render();
    }
    
    /**
     * Display Affiliate Users
     * @param string $filter
     * @param number $offset
     */
    public function index__old($filter = 'all', $offset = 0) {
        $this->manage('users', $filter, $offset);
    }
    
    /**
     * Manage Affiliate Users/Coupons/Statuses
     * @param string $module
     * @param string $filter
     * @param number $offset
     */
    public function manage($module = 'users', $filter = 'all', $offset = 0) {
        $extra = array(
            'model' => 'affiliates',
            'context' => 'financial',
            'fields' => $module,
            'prefix' => 'affiliate',
            'module_name' => 'affiliates'
        );
        if($module == 'users') {
            // affiliate_statuses table
            $extra['table_prefix'] = 'affiliate';
            // Back or Cancel URL - Back to the Index page after viewing commissions, referral traffic etc.
            $this->session->set_userdata('referrer_page', current_url());
        }
        if($module == 'commissions') {
            $extra['model_folder'] = 'commissions';
            $extra['model'] = 'commissions';
            $extra['prefix'] = '';
            $extra['table_prefix'] = 'commission';
        }        
        if($module == 'coupons') {
            $extra['actions'] = array( // Extra buttons
                array('type' => 'link', 'link' => site_url(SITE_AREA . '/financial/affiliates/commissions/%s'), 'label' => lang('label_usage'), 'class' => 'btn btn-info btn-xs'),
                array('type' => 'link', 'link' => site_url(SITE_AREA . '/financial/affiliates/commissions/0/1'), 'label' => lang('label_commission'), 'class' => 'btn btn-warning btn-xs')
            );
        }
        if($module == 'materials' || $module == 'tutorials') {
            $this->affiliates_model->filter_counter($module);
            // $extra['view'] = 'manage';
            $extra['actions'] = array( // Extra buttons
                array('type' => 'link', 'link' => site_url(SITE_AREA . '/financial/affiliates/form/' . $module . '/edit/%s'), 'label' => '<i class="fas fa-edit fa-fw mr-1"></i>' . lang('edit'), 'class' => 'btn btn-xs btn-primary'),
            );
            $extra['filter_tabs'] = $module == 'tutorials' ? $this->affiliates_model->tutorials_ad_types : $this->affiliates_model->materials_ad_types;
        }
        if($module == 'referrals') {            
            Assets::add_css('flags.css');
        }
        if($module == 'links') {            
            Template::set_message(lang('warning_links'), 'danger');
        }
        $this->gc->module_view($module, $filter, $offset, 'admin', $extra);
    }
    
    /**
     * Add/Edit Affiliates Data
     *
     * @return void
     */
    public function form($module = 'users', $action = 'create') {
        if($module == 'materials' || $module == 'tutorials') {
            $this->affiliates_model->material_form($module, $action);
        } else {
            $params = array(
                'selectize' => true,
                'model' => 'affiliates',
                'context' => 'financial',
                'module_name' => 'affiliates',
                'fields_method' => $module,
                'prefix' => 'affiliate'
            );
            if($module == 'users') {
                $params['dynamic_fields'] = true;
                $params['layout'] = 'fieldset';
                $params['method'] = 'after_affiliate_save_admin';
                Assets::add_js("
$('#user_id').change(function() {
    $.getJSON('" . site_url(SITE_AREA . '/financial/affiliates/user_info/') . "' + + $(this).val(), {}, function(data) {
         $('#name').val(data.name);
         $('#phone_number').val(data.phone_number);
         $('#address').val(data.address);
         $('#postcode').val(data.postcode);
         // set the city field value
         $('.search-typeahead').typeahead('val', data.city).blur();
         $('.search-typeahead').typeahead('val', data.location);
         $('.search-typeahead').eq(0).trigger('keyup');         
         $('#city_id').val(data.city_id);
    });
});
", 'inline');
            }
            if($module == 'email_templates') {
                Assets::add_js(array(
                    'tinymce/tinymce.min.js',
                ));
            }
            $this->gcrud->form($module, $action, 'admin', $params);
        }
    }
    
    /**
     * Auto fill the users form
     * @param number $user_id
     */
    public function user_info($user_id) {
        $this->load->model('users/user_model');
        $record = $this->user_model->get_user($user_id);
        $data = array();
        if($record) {
            $data = array(
                'name' => $record->display_name,
                'phone_number' => $record->phone_number,
                'address' => $record->address,
                'postcode' => $record->postcode,
                'city' => $record->city,
                'location' => !empty($record->city) ? $record->city . ', ' . $record->state . ', ' . $record->country_iso : '',
                'city_id' => $record->city_id
            );            
        }
        echo json_encode($data);
        return false;
    }
    
    /**
     * Save settings
     */
    public function settings() {
        $this->lang->load('commissions/commissions');
        clear_cache('site_settings'); // Clear Site Settings Cache
        $setting_name = 'affiliate.settings';
        $this->load->model('settings/settings_model');
        if (isset($_POST['save'])) {
            $this->gcrud->stats(array('module' => 'affiliates'));
            // $this->form_validation->set_rules('modules[]', 'lang:applicable_modules', 'trim|required|is_numeric|is_array');
            $this->form_validation->set_rules('commission_cpc', 'lang:commission_cpc', 'trim');
            $this->form_validation->set_rules('commission_level', 'lang:commission_level', 'trim');
            $this->form_validation->set_rules('hierarchy_setup', 'lang:hierarchy_setup', 'trim');
            $this->form_validation->set_rules('override_cookie', 'lang:override_cookie', 'trim');
            $this->form_validation->set_rules('override_referral_cookie', 'lang:override_referral_cookie', 'trim');
            $this->form_validation->set_rules('tracking_duration', 'lang:tracking_duration', 'trim|required|is_numeric');
            $this->form_validation->set_rules('terms_conditions', 'lang:terms_conditions', 'trim|required');
            if($this->form_validation->run() == TRUE) {
                $settings = array(
                    'commission_cpc' => (int) $this->input->post('commission_cpc'),
                    'commission_level' => (int) $this->input->post('commission_level'),
                    'hierarchy_setup' => (int) $this->input->post('hierarchy_setup'),
                    'override_cookie' => (int) $this->input->post('override_cookie'),
                    'override_referral_cookie' => (int) $this->input->post('override_referral_cookie'),
                    'tracking_duration' => (int) $this->input->post('tracking_duration'),
                    'terms_conditions' => $this->input->post('terms_conditions'),
                );
                $data = array (array('name' => $setting_name, 'value' => json_encode($settings)));
                $updated = $this->settings_model->update_batch($data, 'name');
                if ($updated) {
                    Template::set_message(sprintf(lang('msg_save_success'), lang('gc_menu_settings')), 'success');
                    Template::redirect(site_url(SITE_AREA . '/financial/affiliates/settings'));
                } else {
                    Template::set_message(sprintf(lang('msg_edit_failure'), lang('gc_menu_settings')), 'danger');
                }
            }
        }
        $result = $this->db->select('value')->get_where('settings', array('name' => $setting_name))->row();
        if($result) {
            $settings = json_decode($result->value ?? '');
        } else {
            $settings = json_decode(settings_item($setting_name));
        }
        Template::set('settings', $settings);
        Template::set('toolbar_title', lang('gc_menu_settings'));
        Template::render();
    }
    
    /**
     * Allow Admin to View User's Commissions
     * @param string $coupon_code
     * @param number $affiliate_id
     * @param number $offset
     */
    public function commissions($row_id, $affiliate_id = 0, $offset = 0) {
        $coupon_code = null;
        if($row_id > 0) {
            $this->db->from('affiliate_coupons AC');
            $this->db->join('coupons C', 'C.id = AC.coupon_id', 'INNER');
            $row = $this->db->select('C.code')->where('AC.id', $row_id)->get()->row();
            if($row) {
                $coupon_code = $row->code;
            }
        }
        $this->load->model('coupons/coupons_model');
        $this->coupons_model->usage('admin', $coupon_code, $affiliate_id, $offset, $this->limit);
    }
    
    /**
     * Validation Callback Function - Either select a banner image OR enter a valid URL of the image, both are not allowed. 
     * @return boolean
     */
    public function one_field_only() {
        return false;
    }
    
    /**
     * Valid Youtube Video URL
     * @param string $url
     * @return string
     */
    public function valid_youtube_url($url) {
        $ytvIDlen = 11;
        $idStarts = strpos($url, "embed/");
        if($idStarts === FALSE)
            $idStarts = strpos($url, "embed/");
            if($idStarts === FALSE) {
                // $this->form_validation->set_message('valid_youtube_url', lang('bf_form_not_valid_youtube_url'));
                return false;
            } else {
                $idStarts +=3;
                $ytvID = substr($url, $idStarts, $ytvIDlen);
                return true;
            }
    }
    
    /**
     * Check whether the file is valid or not
     * @param string $filename
     * @param string $valid_ext
     * @return boolean
     */
    public function valid_file($filename, $field) {
        if($field == 'pdf') {
            $valid_extensions = array('pdf');
        } else {
            $valid_extensions = array('gif', 'jpg', 'png');
        }
        $file_url = FCPATH . 'assets/images/' . $filename;
        if(file_exists($file_url)) {
            $ext = pathinfo($filename, PATHINFO_EXTENSION);
            if(in_array($ext, $valid_extensions)) {
                return true;
            }
        }
        $this->form_validation->set_message('valid_file', sprintf(lang('form_validation_valid_file'), $field, implode(', ', $valid_extensions)));
        return false;
    }
    
    /**
     * Allow user to change package
     * @param number $id
     */
    public function change_level($id = 0) {
        $this->lang->load('commissions/commissions');
        $id = (int) $id;
        $record = $this->db->where('id', $id)->get('affiliate_users')->row();
        if(!$record) {
            Template::set_message(lang('invalid_id'), 'danger');
            Template::redirect(site_url(SITE_AREA . '/financial/affiliates/manage/users'));
        }
        $redirect_url = $this->session->userdata('referrer_page') ? $this->session->userdata('referrer_page') : site_url(SITE_AREA . '/financial/affiliates/manage/users');
        if(isset($_POST['save'])) {
            $this->gcrud->stats(array('module' => 'affiliates'));
            $new_level_id = $this->input->post('level_id');
            if($record->level_id == $new_level_id) {
                Template::set_message(lang('msg_nothing_to_update'), 'danger');
            } else {
                $update_data = array(
                    'level_id' => $new_level_id,
                    'modified_on' => date('Y-m-d H:i:s'),
                    'modified_ip' => $this->input->ip_address(),
                    'modified_by' => $this->auth->user_id()
                );
                $this->db->where('id', $id)->update('affiliate_users', $update_data);
                Template::set_message(sprintf(lang('msg_action_success'), lang('singular_commission_levels'), lang('label_updated')), 'success');
                Template::redirect($redirect_url);
            }
        }
        $levels = array();
        $result = $this->db->get('commission_levels')->result();
        if($result) {
            foreach($result as $row) {
                $levels[$row->id] = $row->name;
            }
        }
        Template::set(array(
            'record' => $record,
            'levels' => $levels,
            'toolbar_title' => $record->id . ' (' . $record->name . ') - ' . lang('change_level'),
            'redirect_url' => $redirect_url
        ));
        Template::render();
    }
    
    /**
     * Extra Params
     */
    public function params() {
        $record = $this->db->where('id', 8)->get('affiliate_materials')->row();
        Template::set(array('material_type' => $record->type, 'params' => $record->params));
        Template::set_view('material');
        Template::render();
    }
    
    /**
     * Display Hierarchy Tree to the User
     * @param number $affiliate_id - The admin can also view a specific user members
     */
    public function tree($affiliate_id) {
        $affiliate = $this->db->where('id', $affiliate_id)->get('affiliate_users')->row();
        if(!$affiliate) {
            Template::redirect(site_url());
        }
        $this->affiliates_model->tree($affiliate->user_id, $affiliate);
        Template::set(array(
            'toolbar_title' => 'tree'
        ));
        Assets::add_js('_affiliates.js');
        Assets::add_css('_affiliates.css');
        Template::render();
    }
}