<?php defined('BASEPATH') || exit('No direct script access allowed');

class Affiliates_model extends BF_Model
{
    public $table_name	        = 'affiliate_users';
    protected $table_coupons    = 'affiliate_coupons';
    protected $table_referrals  = 'affiliate_referrals';
    protected $table_referrals_unique  = 'affiliate_referrals_unique';
    protected $table_materials  = 'affiliate_materials';
    protected $table_statuses   = 'affiliate_statuses';
    protected $table_qrcodes    = 'affiliate_qrcodes'; // For reporting purposes only
    protected $table_commissions = 'commissions';
	protected $key			= 'id';
	protected $date_format	= 'datetime';

	protected $log_user 	= true;
	protected $set_created	= true;
	protected $set_modified = true;
	protected $soft_deletes	= false;

	protected $created_field     = 'created_on';
    protected $created_by_field  = 'created_by';
	protected $modified_field    = 'modified_on';
    protected $modified_by_field = 'modified_by';

	// Customize the operations of the model without recreating the insert,
    // update, etc. methods by adding the method names to act as callbacks here.
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 	    = array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	// For performance reasons, you may require your model to NOT return the id
	// of the last inserted row as it is a bit of a slow method. This is
    // primarily helpful when running big loops over data.
	protected $return_insert_id = true;

	// The default type for returned row data.
	protected $return_type = 'object';

	// Items that are always removed from data prior to inserts or updates.
	protected $protected_attributes = array();

	// You may need to move certain rules (like required) into the
	// $insert_validation_rules array and out of the standard validation array.
	// That way it is only required during inserts, not updates which may only
	// be updating a portion of the data.
	protected $validation_rules 		= array();
	protected $insert_validation_rules  = array();
	protected $skip_validation 			= false;
	protected $context			        = 'financial';
	protected $moduleName               = 'affiliates';
	protected $gc_affiliates_cache;
	protected $tree_html; // variable to store comments HTML
	protected $materials_fields = array(
	    'banner_alt', 'banner_description', 'banner_image', 'banner_url',
	    'video_title', 'video_description', 'video_url',
	    'text_title', 'text_description',
	    'html_title', 'html_description',
	    'pdf_file', 'pdf_description'
	);
	
	public $layout = '_affiliates';
	public $materials_ad_types = array('banner' => 'ad_type__banner', 'video' => 'ad_type__video', 'text' => 'ad_type__text', 'html' => 'ad_type__html', 'pdf' => 'ad_type__pdf');
	public $tutorials_ad_types = array('pdf' => 'ad_type__pdf', 'video' => 'ad_type__video');
   
	/**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
        // Load Zend Caching
        $this->load->library('slug');
        $this->load->library('gzf_cache', array('lifetime' => 900));
        if(method_exists($this->gzf_cache, 'get_instance')) {
            $this->gc_affiliates_cache = $this->gzf_cache->get_instance();
        }
    }
    
    /**
     * Affiliate Users Fields
     *
     * @return array
     */
    /*public function users() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'dropdown',
                'name' => 'level_id',
                'label' => 'singular_commission_levels',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:commission_levels;value:database',
                'options_detailed' => 1,
                'options_lang' => 1,
                'class' => 3,
                'rules' => 'trim|is_numeric|required',
            ),
            array(
                'type' => 'selectize',
                'name' => 'ref_by',
                'label' => 'affiliate_referred_by',
                'placeholder' => 'placeholder_selectize',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => true,
                'options' => 'load:users;module:users;max_items:1',
                'options_detailed' => 1,
                'rules' => 'trim'
            ),
            array(
                'type' => 'selectize',
                'name' => 'user_id',
                'label' => 'bf_user',
                'placeholder' => 'placeholder_selectize',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => true,
                'options' => 'load:users;module:users;max_items:1',
                'options_detailed' => 1,
                'rules' => 'trim|required',
            ),
            array(
                'type' => 'dropdown',
                'name' => 'status_id',
                'label' => 'label_status',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:affiliate_statuses;value:database',
                'options_detailed' => 1,
                'options_lang' => 1,
                'class' => 3,
                'rules' => 'trim|is_numeric|required',
            ),
            array(
                'type' => 'text',
                'name' => 'tracking_duration',
                'label' => 'affiliate_tracking_duration',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'class' => 2,
                'value' => 0,
                'rules' => 'trim|is_numeric|required',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_ip',
                'label' => 'created_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_ip',
                'label' => 'modified_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            
        );
        return $fields;
    }*/
    
    /**
     * Affiliate Referrals Fields
     *
     * @return array
     */
    public function referrals() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'selectize',
                'name' => 'user_id',
                'label' => 'bf_user',
                'placeholder' => 'placeholder_selectize',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => true,
                'options' => 'load:users;module:users;max_items:1',
                'options_detailed' => 1,
                'rules' => 'trim|required',
            ),
            array(
                'type' => 'text',
                'name' => 'source_page',
                'label' => 'label_source_page',
                'frontend' => false,
                'admin_only' => false,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|max_length[255]',
            ),
            array(
                'type' => 'text',
                'name' => 'landing_page',
                'label' => 'label_landing_page',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|max_length[255]',
            ),
            array(
                'type' => 'text',
                'name' => 'device_type',
                'label' => 'label_device_type',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|max_length[50]',
            ),
            array(
                'type' => 'text',
                'name' => 'web_browser',
                'label' => 'label_web_browser',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim|max_length[255]',
            ),
            array(
                'type' => 'text',
                'name' => 'country_abbrev',
                'label' => 'label_country',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'operating_system',
                'label' => 'label_action',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|max_length[255]',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_ip',
                'label' => 'created_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_ip',
                'label' => 'modified_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            
        );
        return $fields;
    }
    
    /**
     * Affiliate Faqs Fields
     *
     * @return array
     */
    public function faqs() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'text',
                'name' => 'title',
                'label' => 'label_question',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'edit' => 1,
                'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
                'rules' => 'trim|required|max_length[255]',
            ),
            array(
                'type' => 'textarea',
                'name' => 'description',
                'label' => 'label_answer',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|required',
            ),
            array(
                'type' => 'text',
                'name' => 'priority',
                'label' => 'label_priority',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'value' => 0,
                'class' => 2,
                'rules' => 'trim',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'status',
                'label' => 'label_status',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            
        );
        return $fields;
    }
    
    /**
     * Email Template Fields
     *
     * @return array
     */
    public function email_templates($role = 'admin') {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'dropdown',
                'name' => 'module_table',
                'label' => 'label_module',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:placeholder_select;value:|name:affiliate_users;value:affiliate_users|name:gc_menu_commissions;value:commissions|name:gc_menu_payouts;value:payouts',
                'options_detailed' => 1,
                'options_lang' => 1,
                'class' => 3,
                'rules' => 'trim|unique_email_template[affiliate]',
            ),
            array(
                'type' => 'dropdown',
                'name' => 'user_type',
                'label' => 'bf_user',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:placeholder_select;value:|name:label_admin;value:admin|name:bf_user;value:user',
                'options_detailed' => 1,
                'options_lang' => 1,
                'class' => 3,
                'rules' => 'trim',
            ),
            array(
                'type' => 'dropdown',
                'name' => 'action_type',
                'label' => 'label_action',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:placeholder_select;value:|name:add;value:add|name:edit;value:edit|name:delete;value:delete|name:forum_status_update;value:status_update|name:affiliate_notify_referrer;value:notify_referrer',
                'options_detailed' => 1,
                'options_lang' => 1,
                'class' => 3,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'name',
                'label' => 'label_name',
                'placeholder' => 'placeholder_name',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
                'rules' => 'required|trim|max_length[50]'
            ),
            array(
                'type' => 'text',
                'name' => 'subject',
                'label' => 'label_subject',
                'placeholder' => 'placeholder_subject',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'required|trim|max_length[255]'
            ),
            array(
                'type' => 'textarea',
                'name' => 'message',
                'label' => 'label_message',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'custom_rules' => 'editor',
                'field_class' => 'editor',
                'class' => 9,
                'rules' => 'trim|required',
            ),
            array(
                'type' => 'textarea',
                'name' => 'sms_text',
                'label' => 'sms_text',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'class' => 9,
                'field_extra' => 'rows:3',
                'rules' => 'trim',
            ),
            array(
                'type' => 'textarea',
                'name' => 'params',
                'label' => 'label_params',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'class' => 9,                
                'tooltip' => lang('emailer_params_help'),
                'field_extra' => 'rows:3',
                'rules' => 'trim',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'status',
                'label' => 'label_status',
                'frontend' => false,
                'admin_only' => false,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            )
        );
        return $fields;
    }
    
    /**
     * Affiliate Link Fields
     *
     * @return array
     */
    public function links() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'text',
                'name' => 'name',
                'label' => 'label_name',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'edit' => 1,
                'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
                'rules' => 'trim|required|max_length[255]'
            ),
            array(
                'type' => 'text',
                'name' => 'url',
                'label' => 'label_url',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|max_length[255]'
            ),
            array(
                'type' => 'checkbox',
                'name' => 'allow_qrcode',
                'label' => 'tab_ad_type__qrcode',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'status',
                'label' => 'label_status',
                'frontend' => false,
                'admin_only' => false,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            
        );
        return $fields;
    }
    
    /**
     * Affiliate Materials Fields
     *
     * @return array
     */
    public function materials() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'dropdown',
                'name' => 'link_id',
                'label' => 'singular_affiliate_links',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:affiliate_links;value:database',
                'options_detailed' => 1,
                'rules' => 'trim|required|max_length[255]'
            ),
            array(
                'type' => 'text',
                'name' => 'ad_type',
                'label' => 'label_type',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|max_length[10]',
            ),
            array(
                'type' => 'text',
                'name' => 'params',
                'label' => 'label_ad',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                // 'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'priority',
                'label' => 'label_priority',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'status',
                'label' => 'label_status',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            
        );
        return $fields;
    }
    
    /**
    * Affiliate Tutorials Fields
    *
    * @return array
    */
    public function tutorials() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'text',
                'name' => 'ad_type',
                'label' => 'label_type',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim|max_length[10]',
            ),
            array(
                'type' => 'text',
                'name' => 'params',
                'label' => 'label_ad',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
               // 'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'priority',
                'label' => 'label_priority',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'status',
                'label' => 'label_status',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            
        );
        return $fields;
    }
    
    /**
     * Affiliate Links to be displayed on the affiliates detail and the index page
     */
    public function affiliate_menus() {        
        // Materials Dropdown Menus
        $info_url = site_url(SITE_AREA . '/financial/affiliates/info/%s/');
        $materials_menus = array();
        foreach($this->materials_ad_types as $material => $material_string) {
            $materials_menus[] = array('label' => lang('affiliate_' . $material), 'url' => $info_url . '1/1/' . $material);
        }
        $materials_menus[] = array('label' => lang('gc_menu_coupons'), 'url' => $info_url . '1/1/coupon');
        $materials_menus[] = array('label' => lang('affiliate_qrcodes'), 'url' => $info_url . '1/1/qrcode');
        $materials_menus[] = array('label' => lang('affiliate_hierarchy'), 'url' => $info_url . '1/1/hierarchy');
        
        // Commissions Dropdown Menus
        $commisions_menus = array();
        $commisions_menus[] = array('label' => lang('commission_all'), 'url' => $info_url . '2/3');
        $commisions_menus[] = array('label' => lang('commission_standard'), 'url' => $info_url . '2/4');
        $commisions_menus[] = array('label' => lang('commission_coupon'), 'url' => $info_url . '2/5');
        $commisions_menus[] = array('label' => lang('commission_cpc_logs'), 'url' => $info_url . '2/6');
        $commisions_menus[] = array('label' => lang('commission_hierarchy'), 'url' => $info_url . '2/7');
        $commisions_menus[] = array('label' => lang('affiliate_tree'), 'url' => $info_url . '3');
        $commisions_menus[] = array('label' => lang('commission_payouts'), 'url' => $info_url . '2/8');
        
        // Referral Traffic
        $traffic_menus = array(
            array('label' => lang('referral_traffic_all'), 'url' => $info_url . '2/1'),
            array('label' => lang('referral_traffic_unique'), 'url' => $info_url . '2/2')
        );
        return array(
            'materials' => $materials_menus,
            'commissions' => $commisions_menus,
            'traffic' => $traffic_menus
        );
    }
    
    /**
     * Allows Adding/Editing of Affiliate Materials Data.
     *
     * @return void
     */
    public function material_form($module, $type = 'create') {
        $table = 'affiliate_' . $module;
        $redirect_url = SITE_AREA .'/financial/affiliates/manage/' . $module;
        if($type == 'edit') {
            $id = (int) $this->uri->segment(7);        
            if (empty($id)) {
                Template::set_message(lang('invalid_id'), 'danger');
                redirect($redirect_url);
            }
            $record = $this->db->where('id', $id)->get($table)->row();
            $action = 'update';
        } else {
            $record = array();
            $id = 0;
            $action = 'insert';
        }
        $module_label = lang('singular_affiliate_' . $module);
        if (isset($_POST['save']) || isset($_POST['save_only'])) {       
            $this->gcrud->stats(array('module' => $table));
            $this->auth->restrict('Affiliate_' . ucwords($module) . '.Financial.' . ucwords($type));            
            if ($this->save_material($table, $action, $id)) {
                // Log the activity
                log_activity($this->auth->user_id(), lang('act_' . $type . '_record') .': '. $id .' : '. $this->input->ip_address(), $table);
                Template::set_message(sprintf(lang('msg_save_success'), $module_label), 'success');
                if(!empty($_POST['save_only'])) {
                    $redirect_url = current_url() . (!empty($_SERVER['QUERY_STRING']) ? '?' . $_SERVER['QUERY_STRING'] : '');
                }
                redirect($redirect_url);
            } else {
                Template::set_message(sprintf(lang('msg_' . $type . '_failure'), $module_label), 'danger');
            }
        } else if (isset($_POST['delete'])) {
            $this->gcrud->stats(array('module' => $table));
            $this->auth->restrict('Affiliate_' . ucwords($module) . '.Financial.Delete');
            
            if ($this->db->where('id', $id)->delete($table)) {
                log_activity($this->auth->user_id(), lang('act_delete_record') .': '. $id .' : '. $this->input->ip_address(), $table);
                Template::set_message(sprintf(lang('msg_action_success'), $module_label, lang('label_deleted')), 'success');
                redirect($redirect_url);
            } else {
                Template::set_message(sprintf(lang('msg_delete_failure'), $module_label), 'danger');
            }
        }        
        Template::set(array(
            'toolbar_title' => lang($type . '_' . $table),
            'record' => $record
        ));
        Template::set_view('financial/' . $module);
        Template::render();
    }
    
    /**
     * Filter Counter - Materials/Tutorials module
     * @param string $module - materials/tutorials
     */
    public function filter_counter($module) {  
        $ad_types = $module == 'materials' ? $this->materials_ad_types : $this->tutorials_ad_types;
        $table = 'affiliate_' . $module;
        $sql = "SELECT COUNT(*) AS total";
        foreach($ad_types as $key => $value) {
            $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $table . " WHERE ad_type = '" . $key . "') as " . $value;
        }
        $sql .= " FROM " . $this->db->dbprefix . $table;
        $counter = (array) $this->db->query($sql)->row();
        Template::set('counter', $counter);
    }
    
    //--------------------------------------------------------------------
    // !PRIVATE METHODS
    //--------------------------------------------------------------------
    
    /**
     * Summary
     *
     * @param String $type Either "insert" or "update"
     * @param Int	 $id	The ID of the record to update, ignored on inserts
     *
     * @return Mixed    An INT id for successful inserts, TRUE for successful updates, else FALSE
     */
    private function save_material($table, $type='insert', $id = 0) {
        if ($type == 'update') {
            $_POST['id'] = $id;
        }        
        if($table == 'affiliate_materials') {
            $this->form_validation->set_rules('link_id', 'lang:singular_affiliate_links', 'trim|required');
        }
        $this->form_validation->set_rules('ad_type', 'lang:label_type', 'trim|required'); 
        $material_type = $this->input->post('ad_type');
        switch($material_type) {
            case 'banner':
                if(empty($_POST['banner_url']) && empty($_POST['banner_image'])) {
                    $this->form_validation->set_rules('banner_image', 'lang:label_image', 'trim|required'); 
                } else if(!empty($_POST['banner_url']) && !empty($_POST['banner_image'])) {
                    $this->form_validation->set_rules('banner_image', 'lang:label_image', 'trim|required|callback_one_field_only');
                } else if(!empty($_POST['banner_url']) && empty($_POST['banner_image'])) {
                    $this->form_validation->set_rules('banner_url', 'lang:affiliate_image_url', 'trim|required|valid_url');
                } else if(!empty($_POST['banner_image'])) {
                    $this->form_validation->set_rules('banner_image', 'lang:label_image', 'trim|required|callback_valid_file[image]');
                }
                break;
            case 'video':
                $this->form_validation->set_rules('video_title', 'lang:params_video_title', 'trim');
                $this->form_validation->set_rules('video_description', 'lang:label_description', 'trim'); 
                $this->form_validation->set_rules('video_url', 'lang:affiliate_placeholder_video_url', 'trim|valid_youtube_url|required'); 
                break;
            case 'text':
                $this->form_validation->set_rules('text_title', 'lang:affiliate_ad_title', 'trim|required');
                $this->form_validation->set_rules('text_description', 'lang:affiliate_ad_content', 'trim|required');
                break;
            case 'html':
                $this->form_validation->set_rules('html_title', 'lang:label_name', 'trim|required');
                $this->form_validation->set_rules('html_description', 'lang:affiliate_html_content', 'trim|required');
                break;
            case 'pdf':
                $this->form_validation->set_rules('pdf_file', 'lang:affiliate_pdf_file', 'trim|required|callback_valid_file[pdf]');
                if($table == 'affiliate_materials') {
                    $this->form_validation->set_rules('pdf_description', 'lang:label_description', 'trim|required');
                } else {
                    $this->form_validation->set_rules('pdf_description', 'lang:label_description', 'trim');
                }
                break;
            case 'qrcode':
                $this->form_validation->set_rules('qrcode_enable', 'lang:affiliate_qrcode', 'trim');
                break;
        }
        if ($this->form_validation->run() === false) {
            return false;
        }        
        // make sure we only pass in the fields we want
        $data = array();
        if($table == 'affiliate_materials') {
            $data['link_id'] = $this->input->post('link_id');
        }
        $data['ad_type']       = $this->input->post('ad_type');
        $params = array();
        foreach($_POST as $key => $param_data) {
            if(!empty($param_data) && in_array($key, $this->materials_fields) && strpos($key, $material_type) !== false) {
                $params[$key] = $param_data; 
            }
        }
        if($data['ad_type'] == 'banner') {
            if(!empty($params['banner_url'])) {
                $image_info = $this->remote_image_info($params['banner_url']);
            } else {
                $image_path = FCPATH . 'assets/images/' . $params['banner_image'];
                $image_info = getimagesize($image_path);
            }
            if(!empty($image_info[0]) && !empty($image_info[1])) {
                $params['width'] = $image_info[0];
                $params['height'] = $image_info[1];
            }
        }
        
        if($data['ad_type'] == 'video') {
            $params['video_id'] = get_video_id($params['video_url']);
            if(!empty($params['video_id'])) {
                $params['video_duration'] = get_video_duration($params['video_id']);
            }
            $params['video_title'] = !empty($params['video_title']) ? $params['video_title'] : get_video_title($params['video_id']);
            $params['video_image'] = !empty($params['video_id']) ? 'https://img.youtube.com/vi/' . $params['video_id'] . '/0.jpg' : base_url() . 'assets/images/video.png';
        }
        $data['params']     = json_encode($params);
        $data['priority']   = (int) $this->input->post('priority');
        $data['status']     = (int) $this->input->post('status');        
        if ($type == 'insert') {
            $data['created_on'] = date('Y-m-d H:i:s');
            $this->db->insert($table, $data);
            $id = $this->db->insert_id(); 
            if (is_numeric($id)) {
                $return = $id;
            } else {
                $return = FALSE;
            }
        } elseif ($type == 'update') {
            $data['modified_on'] = date('Y-m-d H:i:s');
            $return = $this->db->where('id', $id)->update($table, $data);
        }
        return $return;
    }
    
    /**
     * To avoid an extra overhead of getting banner width and height on each call
     * they are stored during banner save.
     * The banner with url are stored locally to get their width and height
     * @param string $url
     * @return array
     */
    private function remote_image_info($url = null) {
        $image_info = array();
        // check if file exists on server or not
        $headers = get_headers($url);
        $file_exists = stripos($headers[0],"200 OK") ? true : false;
        // get file information
        if($file_exists) {
            $pathinfo = pathinfo($url);
            $filename = $pathinfo['filename'].'.'.$pathinfo['extension'];
            $image_path = FCPATH . 'assets/images/temp/' . $filename;
            $ch = curl_init($url);
            $fp = fopen($image_path, 'wb');
            curl_setopt($ch, CURLOPT_FILE, $fp);
            curl_setopt($ch, CURLOPT_HEADER, 0);
            curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
            curl_exec($ch);
            curl_close($ch);
            fclose($fp);
            $image_info = getimagesize($image_path);
        }
        return $image_info;
    }
    
    //--------------------------------------------------------------------
    // SET COOKIE ON REFERRAL AND STORE VALUES
    //--------------------------------------------------------------------
    
    /**
     * Save Referrals
     * @param number $user_id
     */
    public function save_referrals_old($user_id) {        
        $this->load->model('visitors/visitors_model');
        $this->load->library('Mobile_Detect');
        $ip_address = $this->input->ip_address();
        $date_time = date('Y-m-d H:i:s');
        $data = $this->visitors_model->maxMind($ip_address);
        $detect = new Mobile_Detect;
        $source_page = "http://" . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];
        $device_type = ($detect->isMobile() ? ($detect->isTablet() ? 'tablet' : 'phone') : 'desktop'); // get device type      
        $web_browser = $detect->getUserAgent(); // browser version
        $operating_system = '';        
        $created_on = $date_time;
        $created_ip = $ip_address;
        $modified_on = $date_time;
        $modified_ip = $ip_address;
        $agent = $_SERVER['HTTP_USER_AGENT'];
        $host_name = gethostbyaddr($_SERVER['REMOTE_ADDR']);
        //ADVANCED DATA
    }
    
    /**
     * Save Referrals
     * @param number $user_id
     * @param string $landing_page
     */
    public function save_referrals($user_id, $link_id, $material_id, $landing_page = null) {
        $landing_page = empty($landing_page) ? current_url() : $landing_page;
        $this->load->model('visitors/visitors_model');
        $this->load->library('Mobile_Detect');
        $ip_address = $this->input->ip_address();
        $date_time = date('Y-m-d H:i:s');
        $location = $this->visitors_model->maxMind($ip_address);
        $detect = new Mobile_Detect;
        $user_agent = $detect->getUserAgent(); // browser version
        $source_page = isset($_SERVER['HTTP_REFERER']) ? $_SERVER['HTTP_REFERER'] : null;
        $referral_data = array(
            'user_id' => $user_id,
            'link_id' => $link_id,
            'material_id' => $material_id,
            'source_page' => $source_page,
            'landing_page' => $landing_page,
            'device_type' => ($detect->isMobile() ? ($detect->isTablet() ? 'tablet' : 'phone') : 'desktop'), // get device type
            'web_browser' => $this->getBrowser(),
            'operating_system' => $this->getOS(),
            'city' => isset($location['city']) ? $location['city'] : null,
            'region' => isset($location['region']) ? $location['region'] : null,
            'region_abbrev' => isset($location['region_abbrev']) ? $location['region_abbrev'] : null,
            'country' => isset($location['country']) ? $location['country'] : null,
            'country_abbrev' => isset($location['country_abbrev']) ? $location['country_abbrev'] : null,
            'postal_code' => isset($location['postal_code']) ? $location['postal_code'] : null,
            'latitude' => isset($location['latitude']) ? $location['latitude'] : null,
            'longitude' => isset($location['longitude']) ? $location['longitude'] : null,
            'created_on' => $date_time,
            'created_ip' => $ip_address
        );
        $this->db->insert($this->table_referrals, $referral_data);
        
        // on order confirmaion, the admin must be aware which referral link lead to the payment
        // i.e. the referral_id is also stored in the cache for reporting purposes
        // save this cookie in the commissions table on order confirmation
        $referral_id = $this->db->insert_id();        
        $tracking_duration = time() + 60*60*24*90; // expire after 90 days
        $this->load->library('crypt');
        $referral_id = $this->crypt->encode($referral_id);
        // referred user so set cookie to ref=username
        $cookie = array (
            'name' => 'referral_id',
            'value' => $referral_id,
            'expire' => $tracking_duration // saved in the affiliate_users/affiliate_settings table
        );
        $this->input->set_cookie($cookie); 
        
        // Save Unique Referral
        $unique_referral = array(
            'user_id' => $user_id,
            'link_id' => $link_id,
            'material_id' => $material_id,            
        );
        $referral = $this->db->where($unique_referral)->get($this->table_referrals_unique)->row();
        if($referral) {
            $this->db->where('id', $referral->id)->update($this->table_referrals_unique, array('counts' => $referral->counts + 1));
        } else {
            $unique_referral['source_page'] = $source_page;
            $unique_referral['landing_page'] = $landing_page;
            $this->db->insert($this->table_referrals_unique, $unique_referral);
            
            // calculate CPC commissions on unique referrals only
            $this->load->model('commissions/commissions_model');
            $this->commissions_model->calculate_cpc_commission($user_id);
        }
    }
    
    /**
     * Get Operating System
     */
    public function getOS() {
        $user_agent     =   $_SERVER['HTTP_USER_AGENT'];
        $os_platform    =   "Unknown OS Platform";
        $os_array       =   array(
            '/windows nt 10/i'     =>  'Windows 10',
            '/windows nt 6.3/i'     =>  'Windows 8.1',
            '/windows nt 6.2/i'     =>  'Windows 8',
            '/windows nt 6.1/i'     =>  'Windows 7',
            '/windows nt 6.0/i'     =>  'Windows Vista',
            '/windows nt 5.2/i'     =>  'Windows Server 2003/XP x64',
            '/windows nt 5.1/i'     =>  'Windows XP',
            '/windows xp/i'         =>  'Windows XP',
            '/windows nt 5.0/i'     =>  'Windows 2000',
            '/windows me/i'         =>  'Windows ME',
            '/win98/i'              =>  'Windows 98',
            '/win95/i'              =>  'Windows 95',
            '/win16/i'              =>  'Windows 3.11',
            '/macintosh|mac os x/i' =>  'Mac OS X',
            '/mac_powerpc/i'        =>  'Mac OS 9',
            '/linux/i'              =>  'Linux',
            '/ubuntu/i'             =>  'Ubuntu',
            '/iphone/i'             =>  'iPhone',
            '/ipod/i'               =>  'iPod',
            '/ipad/i'               =>  'iPad',
            '/android/i'            =>  'Android',
            '/blackberry/i'         =>  'BlackBerry',
            '/webos/i'              =>  'Mobile'
        );
        
        foreach ($os_array as $regex => $value) {
            if (preg_match($regex, $user_agent)) {
                $os_platform    =   $value;
            }
            
        }
        return $os_platform;
    }
    
    /**
     * Get Browser information
     * @return string
     */
    private function getBrowser() {
        $user_agent     =   $_SERVER['HTTP_USER_AGENT'];
        $browser        =   "Unknown Browser";
        $browser_array  =   array(
            '/msie/i'       =>  'Internet Explorer',
            '/firefox/i'    =>  'Firefox',
            '/safari/i'     =>  'Safari',
            '/chrome/i'     =>  'Chrome',
            '/edge/i'       =>  'Edge',
            '/opera/i'      =>  'Opera',
            '/netscape/i'   =>  'Netscape',
            '/maxthon/i'    =>  'Maxthon',
            '/konqueror/i'  =>  'Konqueror',
            '/mobile/i'     =>  'Handheld Browser'
        );        
        foreach ($browser_array as $regex => $value) {
            if (preg_match($regex, $user_agent)) {
                $browser    =   $value;
            }
            
        }
        return $browser;
    } 
    
    /**
     * Allow staff to share their affiliate id with any url
     */
    public function direct_affiliate($key = 'ref') {
        $affiliate_id = (int) $this->input->get ( $key, TRUE );
        if(!empty($affiliate_id)) {
            $settings = json_decode(settings_item('affiliate.settings'));
            $override = $settings->override_cookie;
            $set_cookie = false;
            if($override == 1) {
                $set_cookie = true;
            } else if(!$this->input->cookie($key)) {
                $set_cookie = true;
            }
            if($set_cookie) {
                $user = $this->db->where(array('user_id' => $affiliate_id, 'status_id' => 2, 'deleted' => 0))->get('affiliate_users')->row();
                if($user) {
                    // update referred counter in the affiliates table for reporting purposes
                    // data can be deleted from the referrals table because of size issue, so this field will be beneficial for reporting.
                    $this->db->where('id', $user->id)->update('affiliate_users', array('referred_counter' => $user->referred_counter + 1));
                    
                    // save referrals
                    $redirect_url = current_url();
                    $this->save_referrals($affiliate_id, 0, 0, $redirect_url); // save_referrals($user_id, $link_id, $ad_id, $redirect_url)
                    $this->_set_affiliate_cookie($user, $affiliate_id, $key);
                    // also set the ref_by cookie, this is done to have one ref link for all commissions
                    // comment the line below to have a separate link for ref_by
                    $this->_set_affiliate_cookie($user, $affiliate_id, 'ref_by');
                }
            }
            return false;
        }
    }
    
    /**
     * Store Affiliate ID in a COOKIE
     * @param array $user
     * @param string $key
     * ref_by - The user can also refer the site to other affiliates for tier commissions
     */
    public function set_affiliate($user = array(), $key = 'ref') {
        $settings = json_decode(settings_item('affiliate.settings'));
        $override = $key == 'ref_by' ? $settings->override_referral_cookie : $settings->override_cookie;
        $set_cookie = false;
        $saved_cookie = $this->input->cookie($key);
        if($override == 1) {
            $set_cookie = true;
        } else if(empty($saved_cookie)) {
            $set_cookie = true;            
        }
        if($set_cookie) {
            $affiliate_id = (int) $this->input->get ( $key, TRUE );
            // set ref_by cookie
            if($key == 'ref') {
                if($settings->hierarchy_setup == 2) {
                    $this->_set_affiliate_cookie($user, $affiliate_id, 'ref_by');
                }
            }
            $this->_set_affiliate_cookie($user, $affiliate_id, $key);
            if($key == 'ref_by') { // also set the ref cookie
                $this->_set_affiliate_cookie($user, $affiliate_id, 'ref');
            }
        }
        return false;
    }
    
    /**
     * Set Affiliate Cookie
     * @param array $user
     * @param number $affiliate_id
     * @param string $key - ref/ref_by
     * @return boolean
     */
    private function _set_affiliate_cookie($user, $affiliate_id, $key) {
        // note: do not use authenticate library here because of redirect issue in firefox
        $this->load->library('crypt');
        if(empty($user)) {
            $user = $this->db->where('user_id', $affiliate_id)->get($this->table_name)->row();
        }
        if(!empty($user)) {
            if(empty($user->tracking_duration)) {
                $affiliate_settings = json_decode(settings_item('affiliate.settings'));
                $tracking_duration = $affiliate_settings->tracking_duration;
            } else {
                $tracking_duration = $user->tracking_duration;
            }
            $tracking_duration = $tracking_duration * 86400;
            $affiliate_id = $this->crypt->encode($affiliate_id);
            // referred user so set cookie to ref=username
            $cookie = array (
                'name' => $key,
                'value' => $affiliate_id,
                'expire' => $tracking_duration // saved in the affiliate_users/affiliate_settings table
            );
            $this->input->set_cookie ( $cookie );
            return TRUE;
        } 
        return false;
    }
    //--------------------------------------------------------------------
    // END OF REFERRAL METHOD
    //--------------------------------------------------------------------
    /**
     * Get Affiliate user
     * @param number $id
     * @param boolean $with_meta
     * @return object
     */
    public function get_affiliate_user($id, $with_meta = false, $field = 'user_id') {        
        $this->db->from('affiliate_users AS A');
        $this->db->join('users U', 'U.id = A.user_id', 'INNER');
        $this->db->join('affiliate_statuses S', 'S.id = A.status_id', 'LEFT');
        $record = $this->db->select('A.*, S.name AS status, S.color AS status_color, S.name_lang AS status_lang, U.last_login, ' . user_fields())->where('A.' . $field, $id)->get()->row();
        if($record) {
            if($with_meta && !empty($record->meta_data)) {
                $meta_data = json_decode($record->meta_data);
                if($meta_data) {
                    foreach($meta_data as $key => $value) {
                        $record->{$key} = $value;
                    }
                }
            }
        }
        return $record;
    }
    
    /**
     * Get Affiliate Information
     * @param number $id
     * @param string $field
     * @return array
     */
    public function get_affiliate($id, $field = 'user_id') {
        return $this->db->where($field, $id)->get($this->table_name)->row();
    }
    
    /**
     * Method called in Gcrud library after posts insert/update
     * @param number $id
     * @param string $action_type
     * @param array $record - Old Records
     */
    public function after_affiliate_save($id, $action_type = 'insert', $record = array()) {
        $after_save_data = array();
        if($action_type == 'insert') {
            // get default status from the affiliate_statuses table
            $after_save_data['status_id'] = $this->gcrud->get_default_status('affiliate_statuses');
            $this->load->library('goclixy/gclosure');
            $gclosure = new Gclosure('affiliate_users', 'affiliate_users_hierarchies');
            if($this->input->cookie('ref_by', true)) {
                $this->load->library('crypt');
                $ref_by = $this->crypt->decode($this->input->cookie('ref_by'));
                $record = $this->get_affiliate_user($ref_by, true);
                if($record && $ref_by != $this->auth->user_id()) {
                    $after_save_data['ref_by'] = $ref_by;
                    
                    // notify referred subscriber, if template enabled in the admin panel
                    $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'affiliate_email_templates', 'module_table' => 'affiliate_users', 'action_type' => 'notify_referrer', 'id' => $id));
                    // $affiliate_settings = json_decode(settings_item('affiliate.settings'));
               
                    // save relations
                    $gclosure->add($id, $record->id);
                } else if($record) {
                    // save relations
                    $gclosure->add($id, 0);                    
                }
            } else {
                $gclosure->add($id, 0);
            }
            // notify subscriber, if template enabled in the admin panel
            $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'affiliate_email_templates', 'module_table' => 'affiliate_users', 'action_type' => ($action_type == 'insert' ? 'add' : 'edit'), 'id' => $id));
        }
        $meta_data = new stdClass();
        $meta_data = $this->gcrud->find_meta_for('affiliate_users', $id, $meta_data);
        $after_save_data['meta_data'] = json_encode($meta_data);
        $after_save_data['modified_on'] = date('Y-m-d H:i:s');
        $after_save_data['modified_ip'] = $this->input->ip_address();
        // update data in the affiliate users table
        $this->db->where('id', $id)->update($this->table_name, $after_save_data);
        // update the session table
        $this->affiliate_info(true);
    }
    
    /**
     * After Affiliate Save Admin
     * @param number $id
     * @param string $action_type
     * @param array $record
     */
    public function after_affiliate_save_admin($id, $action_type = 'insert', $record = array()) {        
        if($action_type == 'insert') {
            $this->load->library('goclixy/gclosure');
            $gclosure = new Gclosure('affiliate_users', 'affiliate_users_hierarchies');
            $gclosure->add($id, 0);
        }
        $meta_data = new stdClass();
        $meta_data = $this->gcrud->find_meta_for('affiliate_users', $id, $meta_data);
        $after_save_data['meta_data'] = json_encode($meta_data);
        if($action_type == 'insert' && $this->auth->role_id() == 1) { // automatically approve an affiliate
            $after_save_data['status_id'] = 2;
        }
        // update data in the affiliate users table
        $this->db->where('id', $id)->update($this->table_name, $after_save_data);
    }
    
    /**
     * Get logged in user affiliate information
     * @param boolean $clear_session - TRUE clear session variable
     */
    public function affiliate_info($clear_session = false) {
        // $this->auth->restrict();
        $user_id = $this->auth->user_id();
        if($clear_session) {
            $this->session->unset_userdata('affiliate_info');
        }
        if($user_id > 0 && !$this->session->userdata('affiliate_info')) {
            // store affiliate information in a session variable
            $record = $this->_affiliate_info($user_id);
            if($record) {
                $this->session->set_userdata('affiliate_info', $record);
            } else {
                $this->session->set_userdata('affiliate_info', array());
            }
        }
    }
    
    /**
     * Get Affiliate information to display on the affiliates page
     * @param number $id
     * @param string $field
     * @return object
     */
    private function _affiliate_info($id, $field = 'user_id') {
        $this->db->from('affiliate_users AS A');
        $this->db->join('commission_levels L', 'L.id = A.level_id', 'LEFT');
        $this->db->join('affiliate_statuses S', 'S.id = A.status_id', 'LEFT');
        $record = $this->db->select('A.*, L.name AS commission_level, S.name AS status, S.color AS status_color, S.name_lang AS status_lang')->where('A.' . $field, $id)->get()->row();
        if($record) {
            $record->status = !empty($record->status_lang) ? lang($record->status) : $record->status;
        }
        return $record;
    }
    
    /**
     * Display Summary
     * @return array
     */
    public function summary() {
        $sql = "SELECT COUNT(*) AS total_affiliates";
        $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->table_name . " WHERE status_id = 1) AS inactive_affiliates";
        $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->table_name . " WHERE status_id = 2) AS active_affiliates";
        $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->table_referrals . ") AS total_referrals";
        $sql .= " FROM " . $this->db->dbprefix . $this->table_name;
        $query = $this->db->query($sql); 
        $record = $query->row();
        if($record) {
            $data = array();
            foreach($record as $key => $value) {
                switch($key) {
                    case 'total_affiliates':
                        $url = site_url(SITE_AREA . '/financial/affiliates/manage/users');
                        break;
                    case 'inactive_affiliates':
                        $url = site_url(SITE_AREA . '/financial/affiliates/manage/users/status_id--1');
                        break;
                    case 'active_affiliates':
                        $url = site_url(SITE_AREA . '/financial/affiliates/manage/users/status_id--2');
                        break;
                    case 'total_referrals':
                        $url = site_url(SITE_AREA . '/financial/affiliates/manage/referrals');
                        break;
                    default:
                        $url = null;
                }
                $data[$key] = array(
                    'value' => $value,
                    'url' => $url
                );
            }
            return (object) $data;
        } else {
            return $record;
        }
    }    
    
    /**
     * Prepare Text Ad
     * @param string $str - e.g. 6-1-11
     */
    public function prepare_ad($str, $type = 'text', $document_write = false) {
        $cacheID = 'ad_' . $type . '_' . $str . '_' . $document_write;
        $cacheID = str_replace('-', '_', $this->slug->create_slug($cacheID));
        if(!$output = $this->gc_affiliates_cache->load($cacheID)) {
            $output = '';
            // the str contains affiliate_id and the link_url_id
            $params = explode('-', $str ?? '');
            if(!empty($params[2])) {
                $user_id = (int) $params[0]; // User ID
                $link_id = (int) $params[1]; // Link ID
                $ad_id = isset($params[2]) ? (int) $params[2] : 0; // Material Ad ID for Tracking Hits and Conversion Rate
                if($user_id != 0 && $link_id != 0) { // invalid parameters
                    // verify user from the affiliates table
                    $user = $this->db->where(array('user_id' => $user_id, 'status_id' => 2, 'deleted' => 0))->get('affiliate_users')->row();
                    if($user) {
                        // get link information
                        $link = $this->db->where('id', $link_id)->get('affiliate_links')->row();
                        if($ad_id != 0) {
                            $ad = $this->db->where('id', $ad_id)->get('affiliate_materials')->row();
                        }
                        if($ad && $link) {
                            if(empty($link->url)) {
                                $redirect_url = site_url(); // redirect user to the home page of the site
                            } else {
                                $redirect_url = $link->url;
                            }
                            $params = json_decode($ad->params ?? '');
                            if($type == 'text') {
                                $output = $this->text_ad_content($str, $params->text_title, $params->text_description, $redirect_url);
                            } else {
                                $output = $this->video_ad_content($str, $params->video_id, $redirect_url);
                            }
                        }
                    }
                }
            }
            if($document_write) {
                $output = preg_replace('/\v(?:[\v\h]+)/', '', $output);
                $output = "document.write('" . addslashes($output) . "')";
            }
            $this->gc_affiliates_cache->save($output, $cacheID, array('affiliates'));
        }
        return $output;
    }
    
    /**
     * Prepare Text Ad Content
     * @param string $link_id
     */
    private function text_ad_content($link_id, $title, $description, $link_url) {
        $ref_url = gc_url_format('ref') . '/' . $link_id;
        $css = $link_id;
        $js = str_replace('-', '', $link_id);
        $output = <<<EOL
<style>
        .gc-row-{$css} {
        	padding-left: 15px;
        	padding-right: 15px;	
            cursor: pointer;
        }
        .gc-row-{$css}:after {
        	content: "";
        	display: table;
        	clear: both;
        }
        .gc-column-{$css} {
        	float: left;
        	width: 50%;
        	padding: 0 10px;
        }
        .gc-card-{$css} {
        	font-family: Arial, Helvetica, sans-serif;
            position: relative;
            display: -ms-flexbox;
            display: flex;
            -ms-flex-direction: column;
            flex-direction: column;
            min-width: 0;
            word-wrap: break-word;
            background-color: #fff;
            background-clip: border-box;
            border: 1px solid rgba(0,0,0,.125);
            border-radius: .25rem;
        	-webkit-box-shadow: 0 5px 10px rgba(0, 0, 0, 0.2);
            -moz-box-shadow: 0 5px 10px rgba(0, 0, 0, 0.2);
            box-shadow: 0 5px 10px rgba(0, 0, 0, 0.2);
        }
        .gc-card-{$css} h3 {
        	font-size: 14px;
        	padding: 0px;
        	margin: 0px;
            font-weight: normal;
            line-height: 18px;
        }
        .gc-card-{$css} {
            word-wrap: break-word;
        }
        .gc-card-header-{$css} {
            padding: .75rem 1.25rem;
            margin-bottom: 0;
            background-color: rgba(0,0,0,.03);
            border-bottom: 1px solid rgba(0,0,0,.125);
        }
        .gc-card-header-{$css}:first-child {
            border-radius: calc(.25rem - 1px) calc(.25rem - 1px) 0 0;
        }
        .gc-card-body-{$css} {
            -ms-flex: 1 1 auto;
            flex: 1 1 auto;
            padding: 1.25rem;
        	font-size: 14px;
            line-height: 20px;
        }
        @media screen and (max-width: 600px) {
        	.gc-column-{$css} {
        		width: 100%;
        		display: block;
        		margin-bottom: 20px;
        	}
        }
        @media screen and (min-width: 601px) and (max-width: 1024px) {
        	.gc-column-{$css} {
        		width: 50%;
        	}
        }
        </style>
        	<div class="gc-row-{$css}" onclick="gcRedirectTextAd{$js}()">
        	  <div class="gc-column-{$css}">
        		<div class="gc-card-{$css}">
        		  <div class="gc-card-header-{$css}"><h3>{$title}</h3></div>
        		  <div class="gc-card-body-{$css}">
        			  {$description}
        			  <br />
        			  <br />
        			  <a href="{$link_url}" title="{$title}">{$link_url}</a>
        		  </div>
        		</div>
        	  </div>
        	</div>
        <script type="text/javascript">
        	function gcRedirectTextAd{$js}() {
        		var targetUrl = '{$ref_url}';
        		document.location.href = targetUrl; 
        	}
        </script>
EOL;
        return $output;
    }
    
    /**
     * Prepare Video Content
     * @param string $link_id
     * @param string $video_id
     * @param string $link_url
     * @return string
     */
    private function video_ad_content($link_id, $video_id, $link_url) {
        $ref_url = gc_url_format('ref') . '/' . $link_id;
        $link_id = str_replace('-', '', $link_id);
        $output = <<<EOL
    <div id="player{$link_id}"></div>
    <script>
      var tag{$link_id} = document.createElement('script');
      tag{$link_id}.src = "https://www.youtube.com/iframe_api";
      var firstScriptTag{$link_id} = document.getElementsByTagName('script')[0];
      firstScriptTag{$link_id}.parentNode.insertBefore(tag{$link_id}, firstScriptTag{$link_id});
      var player{$link_id};
      function onYouTubeIframeAPIReady() {
        player{$link_id} = new YT.Player('player{$link_id}', {
          height: '390',
          width: '640',
          videoId: '{$video_id}',
          events: {
            'onReady': onPlayerReady{$link_id},
            'onStateChange': onPlayerStateChange{$link_id}
          }
        });
      }
      function onPlayerReady{$link_id}(event) {
        event.target.playVideo();
      }
      var done{$link_id} = false;
      function onPlayerStateChange{$link_id}(event) {
        if (event.data == YT.PlayerState.PLAYING && !done{$link_id}) {
			setTimeout(stopVideo{$link_id}, 12000);
			done{$link_id} = true;
        }
      }
      function stopVideo{$link_id}() {
        player{$link_id}.stopVideo();		
		window.location.href="{$link_url}";
      }
    </script>
EOL;
        return $output;
    }
    
    /**
     * View/Generate QRCode for a listing
     * @param int $insert_id - Listing ID
     */
    public function qrcode($user_id, $link_id, $regenerate = true) {
        $ref_id = $user_id . '-' . $link_id;
        // check if qrfile exist
        $qrfolder = 'assets/images/affiliates/qrcodes/';
        $filename = $ref_id . '.png';
        $qrpath = FCPATH . $qrfolder . $filename;        
        if(file_exists($qrpath) && !$regenerate) {
            return base_url($qrfolder . $filename);
        } else { // Generate QRCode
            $this->load->library('ciqrcode');
            $params['data'] = gc_url_format('ref', $ref_id);
            $params['level'] = 'H';
            $params['size'] = 7;
            $params['savename'] = $qrpath;
            $this->ciqrcode->generate($params);
            // save information in database for reporting purpose
            $datetime = date('Y-m-d H:i:s');
            $ip_address = $this->input->ip_address();
            $data = array(
                'user_id' => $user_id,
                'link_id' => $link_id,
                'created_on' => $datetime,
                'created_ip' => $ip_address,
                'modified_on' => $datetime,
                'modified_ip' => $ip_address
            );
            $this->db->on_duplicate($this->table_qrcodes, $data);
            return base_url($qrfolder . $filename);
        }
        return null;
    }
    
    /***************************************************
     * MEMBER AREA
     ***************************************************/
    /**
     * Display Summary to the Member
     * @return array
     */
    public function summary_members($user_id, $where = '') {
        $sql = "SELECT COUNT(*) AS total_referrals";
        
        // total transactions till date
        $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . "commissions WHERE user_id = " . $user_id. ") AS total_transactions";
        
        // waiting for approval commissions
        $sql .= ", (" . $this->commissions_model->_summary_commission($user_id, 1, $where) . ") AS inactive_amount";
        
        // outstanding commissions
        $sql .= ", (" . $this->commissions_model->_summary_commission($user_id, 2, $where) . ") AS confirmed_amount";
        
        // paid commissions
        // $sql .= ", (" . $this->commissions_model->_summary_commission($user_id, 3, $where) . ") AS paid_amount";
        $sql .= ", (SELECT SUM(P.amount) FROM " . $this->db->dbprefix . "payouts P WHERE P.user_id = " . $user_id . " AND P.status = 1 AND P.module_id = " . gc_module_id('commissions') . ") AS paid_amount";
        
        // total commissions
        $sql .= ", (" . $this->commissions_model->_summary_commission($user_id, 0, $where . (!empty($where) ? ' AND ' : '') . '(C.status_id = 1 OR C.status_id = 2 OR C.status_id = 3)') . ") AS total_amount";
        $sql .= " FROM " . $this->db->dbprefix . $this->table_referrals . " WHERE user_id = " . $user_id;
        $result = $this->db->query($sql)->row();
        foreach($result as $key => $value) {
            switch($key) {
                case 'inactive_amount':
                case 'confirmed_amount':
                case 'paid_amount';
                case 'total_amount':
                case 'commission_total':
                    $result->{$key} = currency_format(empty($value) ? 0 : $value);
                    break;
            }
        }
        return $result;
    } 
    
    /**
     * Display Materials to the User
     * @param string $type
     * @param number $is_training
     */
    public function affiliate_materials($user_id, $type = 'banner', $is_training = 0) {
        $page = $is_training == 1 ? 'tutorials' : 'materials';
        $table = 'affiliate_' . $page;
        $type = escape($type);
        if(isset($_POST['submit'])) {
            $link_id = $this->input->post('link_id');
            $this->session->set_userdata('affiliate_link_group', $link_id);
        }
        if($this->session->userdata('affiliate_link_group')) {
            $link_id = (int) $this->session->userdata('affiliate_link_group');
        } else {
            $active_link_info = $this->gc->get('affiliate_links', array('where' => 'status = 1 AND deleted = 0', 'row' => 1));
            if($active_link_info) {
                $link_id = $active_link_info->id;
            } else {
                $link_id = 1;
            }
        }
        $records = array();
        $qrfile_url = '';
        $link_url = site_url(); // redirect/target url
        $aff_target_url = $link_url;
        $link_info = $this->gc->get('affiliate_links', array('where' => 'id = ' . $link_id, 'row' => 1));
        if($link_info && !empty($link_info->url)) {
            $aff_target_url = $link_info->url;
        }
        if($is_training == 0) { // Materials
            switch($type) {
                case 'coupon':
                    $this->lang->load('coupons/coupons');
                    Assets::add_js('clipboard.min.js');
                    Assets::add_js('new ClipboardJS(".copy-text");', 'inline');
                    Assets::add_css('jssocials.css');
                    Assets::add_js('jssocials.min.js');
                    Assets::add_js(
                        '$(".sharing").each(function() {
            			$(this).jsSocials({
            				url : $(this).data("url"),
            				text: $(this).data("txt"),
            				showLabel : false,
            				shares: ["twitter", "facebook", "googleplus", "linkedin", "pinterest", "whatsapp", "telegram"] // "stumbleupon", 
            			});
            		});', 'inline'
                        );
                    $where = "(M.user_ids = '' OR FIND_IN_SET(" . $this->auth->user_id() . ", M.user_ids)) AND M.status = 1";
                    $records = $this->gcrud->get_all_crud(0, $where, 'M.id DESC', 5000, 0, array('module' => 'coupons', 'prefix' => 'commission'));
                    break;
                case 'qrcode':
                    $link = $this->db->where('id', $link_id)->get('affiliate_links')->row();
                    if($link && $link->allow_qrcode == 1) {
                        if(!empty($link->url)) {
                            $link_url = $link->url;
                        }
                        $qrfile_url = $this->affiliates_model->qrcode($user_id, $link_id);
                    }
                    break;
                case 'hierarchy':
                    $affiliate_user = $this->db->where('user_id', $user_id)->get('affiliate_users')->row();
                    if($affiliate_user) {
                        $level = $this->db->where('id', $affiliate_user->level_id)->get('commission_levels')->row();
                        Template::set('level', $level);
                        $this->db->where('level_id', $affiliate_user->level_id);
                    }
                    $records = $this->db->get('commission_hierarchies')->result();
                    break;
                default:
                    $where = "M.ad_type = '" . $type . "' AND M.status = 1";
                    $where .= " AND M.link_id = " . $link_id;
                    // $records = $this->db->where($where)->get($table)->result();
                    $records = $this->gcrud->get_all_crud(0, $where, 'M.priority ASC', 5000, 0, array('module' => $page, 'prefix' => 'affiliate'));
                    
            }
        } else {
            $where = "M.ad_type = '" . $type . "' AND M.status = 1";
            $records = $this->gcrud->get_all_crud(0, $where, 'M.priority ASC', 5000, 0, array('module' => $page, 'prefix' => 'affiliate'));
        }
        Template::set(array(
            'toolbar_title' => lang('label_available') . ' ' . lang('affiliate_' . $type),
            'page' => $page,
            'records' => $records,
            'type' => $type,
            'is_training' => $is_training,
            'user_id' => $user_id,
            'qrfile_url' => $qrfile_url,
            'link_id' => $link_id,
            'link_url' => $link_url,
            'aff_target_url' => $aff_target_url // share url e.g. https://www.goclixy.com/pricing?affmid=3-12-2
        ));
        Template::set_view('affiliates/materials');
    }
    
    /**
     * Display Referral Traffics to the User
     * @param number $page_id
     * 1 - affiliate_referrals
     * 2 - affiliate_referrals_unique
     * 3 - commissions
     */
    public function logs($page_id = 1, $user_id = 0, $role = 'admin', $record = array(), $render = true, $is_redirect = true) {
        switch($page_id) {
            case 1:
                $toolbar_title = lang('referral_traffic_all');
                break;
            case 2:
                $toolbar_title = lang('referral_traffic_unique');
                break;
            case 3:
                $toolbar_title = lang('commission_all');
                break;
            case 4:
                $toolbar_title = lang('commission_standard');
                break;
            case 5:
                $toolbar_title = lang('commission_coupon');
                break;
            case 6:
                $toolbar_title = lang('commission_cpc');
                break;
            case 7:
                $toolbar_title = lang('commission_hierarchy');
                break;
            case 8:
                $toolbar_title = lang('commission_payouts');
                break;
            case 9: // Payouts Pay Page
                $toolbar_title = lang('commission_confirmed');
                break;
            case 10: // Payout Orders
                $toolbar_title = lang('payout_orders');
                break;
            default:
                Template::redirect(gc_url_format('affiliates'));
        }
        // Payouts Invoice PDF
        if(isset($_POST['invoice_pdf'])) {
            $this->load->library('crypt');
            Template::redirect(site_url('affiliates/payout_invoice') . '?id=' . $this->crypt->encode($_POST['invoice_pdf']));
        }
        if($page_id != 9 && $page_id != 10) {
            if(empty($record)) {
                $record = $this->get_affiliate($user_id);
            }
            if(empty($record)) {
                if($role == 'admin') {
                    Template::redirect(site_url(SITE_AREA . '/financial/affiliates'));                
                } else {
                    if($is_redirect) {
                        Template::redirect(gc_url_format('members'));
                    } else {
                        return false;
                    }
                }
            }
        }
        $where = 'M.user_id = ' . $user_id;
        $this->lang->load('orders/orders');
        $this->lang->load('commissions/commissions');
        // Load datatables - Bootstrap 4 JS and CSS are loaded
        Assets::add_js('//cdn.datatables.net/1.10.19/js/jquery.dataTables.min.js');
        Assets::add_js('//cdn.datatables.net/fixedheader/3.1.6/js/dataTables.fixedHeader.min.js');
        Assets::add_js('//cdn.datatables.net/responsive/2.2.3/js/dataTables.responsive.min.js');
        Assets::add_js('//cdn.datatables.net/1.10.19/js/dataTables.bootstrap4.min.js');
        // Assets::add_css('//cdn.datatables.net/1.10.19/css/jquery.dataTables.min.css');
        // Assets::add_css('//cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/4.1.3/css/bootstrap.css');
        Assets::add_css('//cdn.datatables.net/1.10.19/css/dataTables.bootstrap4.min.css');
        Assets::add_css('//cdn.datatables.net/fixedheader/3.1.6/css/fixedHeader.bootstrap.min.css');
        Assets::add_css('//cdn.datatables.net/responsive/2.2.3/css/responsive.bootstrap.min.css');
        Assets::add_js('
        var table = $(".table-data").DataTable( {
            "processing": true,
            "serverSide": true,
            "responsive": true,
            "ajax": {
                "url": "' . site_url('affiliates/get_data') . '"
            }
        });', 'inline');
        $affiliate_datatable = array();
        switch($page_id) {
            case 1: // Display Affiliate Referrals Logs
                $columns = array(
                    'created_on' => lang('date_time'),
                    'ad_type' => lang('ad_type'),
                    'source_page' => lang('label_source_page'),
                    'country' => lang('label_country'),
                    'created_ip' => lang('ip_address'),
                );
                $affiliate_datatable = array(
                    'columns' => $columns,
                    'module' => 'referrals',
                    'prefix' => 'affiliate',
                    'where' => $where
                );
                break;
            case 2: // Display Affiliate Referrals Unique Logs
                $columns = array(
                    'ad_type' => lang('ad_type'),
                    'source_page' => lang('label_source_page'),
                    'landing_page' => lang('label_landing_page'),
                    'counts' => lang('label_visits'),
                );
                $affiliate_datatable = array(
                    'columns' => $columns,
                    'module' => 'referrals_unique',
                    'prefix' => 'affiliate',
                    'where' => $where
                );
                break;
            case 3: // Display Commission Logs
            case 4: // Display Standard Commissions Only
            case 5: // Display Coupon Commissions
            case 6: // Display CPC Commissions
                $columns = array();
                $columns['created_on'] = lang('date_time');
                $columns['order_id'] = lang('label_order_id');
                if($page_id == 3) {
                    $columns['module_id'] = lang('label_type');
                }
                $columns['amount'] = lang('label_amount');
                $columns['order_total'] = lang('label_order_total');
                $columns['status_id'] = lang('label_status');
                $affiliate_datatable = array(
                    'columns' => $columns,
                    'module' => 'commissions',
                    'prefix' => ''
                );
                if($page_id == 4) {
                    $where .= ' AND M.referral_id = 0 AND M.module_id != 22';
                }
                if($page_id == 5) {
                    $where .= ' AND M.module_id = 22';
                }
                if($page_id == 6) {
                    $where .= ' AND M.module_id = 57';
                }
                $affiliate_datatable['where'] = $where;
                break;
            case 7: // Display Tier Commissions using the commission_affiliates table
                $columns = array(
                'created_on' => lang('date_time'),
                'order_id' => lang('label_order_id'),
                'referral_id' => lang('referred_by'),
                'depth' => lang('singular_commission_levels'),
                'amount' => lang('label_amount'),
                'status_id' => lang('label_status')
                );
                $where .= ' AND M.referral_id != 0 AND ' . 'M.referral_id != ' . $this->auth->user_id();
                $affiliate_datatable = array(
                    'columns' => $columns,
                    'module' => 'commissions',
                    'prefix' => '',
                    'where' => $where
                );
                break;
            case 8: // The user's payouts made by the admin
                $columns = array(
                    'payment_method' => lang('label_payment_method'),
                    'amount' => lang('label_amount'),
                    'transaction_id' => lang('label_transaction_id'),
                    //'comments' => lang('label_comments'),
                    'paid_on' => lang('date_time'),
                    'invoice' => lang('singular_invoices')
                    // 'created_on' => lang('label_status')
                );
                $affiliate_datatable = array(
                    'columns' => $columns,
                    'module' => 'payouts',
                    'prefix' => '',
                    'where' => $where
                );
                break;
            case 9: // Payouts Pay Commissions
            case 10:
                $columns = array(
                    'created_on' => lang('date_time'),
                    'order_id' => lang('label_order_id'),
                    'module_id' => lang('label_type'),
                    'amount' => lang('label_amount'),
                    'order_total' => lang('label_order_total'),
                    'payout_id' => lang('label_status')
                );
                if($role == 'admin') {
                    $columns = array_merge(array('check' => lang('label_item')), $columns);
                }
                if($page_id == 10) {
                    $where = 'M.seller_id = ' . $user_id;
                }
                if(!$render && $this->session->userdata('payout_filter')) { // Payouts Pay Page
                    $payout_filter = $this->session->userdata('payout_filter');
                    $where .= " AND (M.created_on BETWEEN '" . $payout_filter['from'] . "' AND '" . $payout_filter['to'] . "')";
                    if(!empty($payout_filter['module_id'])) {
                        $where .= " AND M.module_id = " . $payout_filter['module_id'];
                    }
                    $where .= " AND M.payout_id = 0"; // get only unpaid commissions
                }
                $where .= " AND M.status_id = 2";
                $affiliate_datatable = array(
                    'columns' => $columns,
                    'module' => $page_id == 10 ? 'order_items' : 'commissions',
                    'prefix' => '',
                    'where' => $where
                );
                break;
        }
        $this->session->set_userdata('affiliate_datatable', $affiliate_datatable);
        Template::set('columns', $columns);
        if($render) {
            // a session variable is used to pass the data to data tables
            Template::set(array(
                'toolbar_title' => $toolbar_title . ' - ' . $record->name,
            ));
            Template::set_view('affiliates/logs');
        }
    }
    
    /**
     * Display Hierarchy Tree to the User
     * @param number $affiliate_id - The admin can also view a specific user members
     */
    public function tree($user_id, $record = array()) {
        if(empty($record)) {
            $record = $this->get_affiliate($user_id);
        }
        if(empty($record)) {
            Template::redirect(site_url());
        }
        $tree = $this->generate_tree($user_id, $record);
        Assets::add_module_css('affiliates', 'tree.css');
        Template::set(array(
            'toolbar_title' => lang('affiliate_tree'),
            'tree_html' => $tree['html']
        ));
        Template::set_view('affiliates/tree');
    }
    
    /**
     * Generate Tree
     * @param number $user_id
     * @param object $record
     */
    public function generate_tree($user_id, $record = array(), $update = false) {
        if(empty($record)) {
            $record = $this->get_affiliate($user_id);
        }
        if(empty($record)) {
            return false;
        }
        $affiliate_id = $record->id;
        $this->load->library('goclixy/gclosure');
        $gclosure = new Gclosure('affiliate_users', 'affiliate_users_hierarchies');
        // display active referrals only
        $affiliate = $gclosure->single_node($affiliate_id, array(), true, false, true);
        if(isset($affiliate['children'])) {
            $this->tree_html .= '<ul>' . PHP_EOL;
            $this->child_view($affiliate);
            $this->tree_html .= '</ul>';
        }
        $children_counts = !empty($affiliate['children']) ? count($affiliate['children']) : 0;
        return array('html' => $this->tree_html, 'children' => $children_counts);
    }
    
    /**
     * Comment has children hierarchy
     * The first element of the array is not a multidimensional array
     * @param array $tree
     */
    private function print_tree($children, $has_children) {
        $this->tree_html .= '<ul>' . PHP_EOL;
        foreach($children as $child) {
            $this->child_view($child, $has_children);
        }
        $this->tree_html .= '</ul>';
    }
    
    /**
     * Display Tree View
     * @param array $affiliate
     */
    private function child_view($affiliate) {
        $this->tree_html .= '<li>' . PHP_EOL;
        $this->tree_html .= '<a href="' . gc_url_format('users/user') . '?id=' . $affiliate['user_id'] . '" target="_blank" title="' . $affiliate['name'] . '">' . $affiliate['name'] . '</a>';
        if(!empty($affiliate['children'])) {
            $this->print_tree($affiliate['children'], true);
        }
        $this->tree_html .= '</li>';
    }
    
    /**
     * Traffic Filter
     * @param string $group_by
     */
    public function traffic_filter($group_by = 'device_type', $select = null, $graph_format = true, $data = array()) { 
        if(empty($select)) {
            $select = $group_by;
        }
        $limit = isset($data['limit']) ? $data['limit'] : 10;
        $sql = "SELECT M.*";
        if($group_by == 'user_id') {
            $sql .= ", " . user_fields();
        }
        $sql .= " FROM (SELECT " . $select . ", COUNT(*) AS total_counts FROM " . $this->db->dbprefix . "affiliate_referrals";
        if(!empty($data['user_id'])) {
            $sql .= " WHERE user_id = " . $data['user_id'];
        }
        $sql .= " GROUP BY " . $group_by;        
        $sql .= " ORDER BY total_counts DESC) M";
        if($group_by == 'user_id') {
            $sql .= " INNER JOIN " . $this->db->dbprefix . "users U ON U.id = M.user_id";
        }
        $sql .= " ORDER BY M.total_counts DESC";
        $sql .= " LIMIT " . $limit;
        $records = $this->db->query($sql)->result();
        $result = $records;
        $labels = array();
        $data = array();
        if($records) {
            if($graph_format) {
                foreach($records as $record) {
                    switch($group_by) {
                        case 'device_type':
                            $labels[] = lang('label_' . $record->{$group_by});
                            break;
                        case 'country_abbrev':
                            $labels[] = $record->country;
                            break;
                    }
                    $data[] = $record->total_counts;
                }
                $result = array('labels' => $labels, 'data' => $data);
            }
        }
        return $result;
    }
    
    /**
     * Top Referred URLs
     * @param array $data
     */
    public function top_referred_urls($data = array()) {
        $distinct_field = 'link_id';
        if(isset($data['distinct_field'])) { // Distinct Field
            $distinct_field = $data['distinct_field'];
        }
        $sql = "SELECT DISTINCT(" . $distinct_field . "), source_page, landing_page FROM " . $this->db->dbprefix . "affiliate_referrals";
        if(!empty($data['user_id'])) {
            $sql .= " WHERE user_id = " . $data['user_id'];
        }
        $top_referred_urls = $this->db->query($sql)->result();
        return $top_referred_urls;
    }
    
    /**
     * Display affiliate name and status on the affiliates page
     * @param number $user_id
     * @param array $user
     * @param boolean $setup_session - TRUE - udpate the session variable
     */
    /*public function affiliates_session($user_id = 0, $user = array(), $setup_session = true) {
        if(empty($user_id)) {
            $user_id = $this->session->userdata('user_id');
        }
        if(empty($user)) {
            $user = (object) $this->session->userdata('user_info');
        }
        // check whether the user is an affiliate or not
        // display affiliate status to the user
        $affiliate = $this->get_affiliate_user($user_id);
        if($affiliate) {
            $user->affiliate_name = $affiliate->name;
            $user->affiliate_status = !empty($affiliate->status_lang) ? lang($affiliate->status) : $affiliate->status;
            $user->affiliate_status_color = $affiliate->status_color;
        }
        if($setup_session) {
            $this->session->set_userdata('user_info', (array) $user);            
        } else {
            return $user;
        }
    }*/
	
	/**
	 * Link Cloaking
	 * @param string $str e.g. http://localhost/mktv1/public_html/affiliates/cloacking/1-1-2
	 */
	public function cloaking($str) {
	    $redirect_url = site_url();
	    // the str contains affiliate_id and the link_url_id
	    $params = explode('-', $str ?? '');
	    if(!empty($params[0]) && $params[1]) {
	        $user_id = (int) $params[0]; // User ID
	        $link_id = (int) $params[1]; // Link ID
	        $ad_id = isset($params[2]) ? (int) $params[2] : 0; // Material Ad ID for Tracking Hits and Conversion Rate
	        if($user_id != 0 && $link_id != 0) { // invalid parameters
	            // verify user from the affiliates table
	            $user = $this->db->where(array('user_id' => $user_id, 'status_id' => 2, 'deleted' => 0))->get('affiliate_users')->row();
	            if($user) {
	                // get link information
	                $link = $this->db->where('id', $link_id)->get('affiliate_links')->row();
	                if($ad_id != 0) {
	                    $ad = $this->db->where('id', $ad_id)->get('affiliate_materials')->row();
	                    if($ad) {
	                        // update hits in the affiliate_materials table
	                        $this->db->where('id', $ad_id)->update('affiliate_materials', array('hits' => $ad->hits + 1));
	                    }
	                }
	                if($link) {
                        if(!empty($ad) && $ad->ad_type == 'pdf') {
	                        $params = json_decode($ad->params, true);
	                        if(!empty($params['pdf_file'])) {
	                            $redirect_url = FCPATH . 'assets/images/' . $params['pdf_file'];
	                        }
	                    } else if(empty($link->url)) {
	                        $redirect_url = site_url(); // redirect user to the home page of the site
	                    } else {
	                        $redirect_url = $link->url;
	                    }
	                    // update referred counter in the affiliates table for reporting purposes
	                    // data can be deleted from the referrals table because of size issue, so this field will be beneficial for reporting.
	                    $this->db->where('id', $user->id)->update('affiliate_users', array('referred_counter' => $user->referred_counter + 1));
	                    
	                    // save referrals
	                    $this->save_referrals($user_id, $link_id, $ad_id, $redirect_url);
	                    $_GET['ref'] = $user_id;
	                    $this->set_affiliate($user);
	                }
	            }
	        }
	    }	   
	    return array('redirect_url' => $redirect_url); // redirect user to the appropriate page
	}
}