<?php defined('BASEPATH') || exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Online Software Solutions							   # ||
|| # Copyright (C) 2021 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Components controller
 */
class Components extends Admin_Controller {
    protected $permissionCreate = 'SEO_Backlinks.Components.Create';
    protected $permissionDelete = 'SEO_Backlinks.Components.Delete';
    protected $permissionEdit   = 'SEO_Backlinks.Components.Edit';
    protected $permissionView   = 'SEO_Backlinks.Components.View';
    protected $context			= 'components';
    protected $moduleName       = 'backlinks';
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
        $this->auth->restrict($this->permissionView);
        clear_cache('backlinks');
        $this->load->model('seo/seo_model');
        $this->load->model('backlinks/backlinks_model');
        $this->lang->load('seo/seo');
        $this->lang->load('backlinks');
      	$this->form_validation->set_error_delimiters("<span class='error'>", "</span>");        
        Template::set_block('sub_nav', 'components/_sub_nav');        
        Assets::add_module_js('backlinks', 'backlinks.js');
    }
    
    /**
     * Display a list of Backlinks data.
     *
     * @return void
     */
    public function index($filter = 'all', $offset = 0) {
        $this->auth->restrict('Backlinks.Components.Summary');
        // DATE TIME PICKER
        Assets::add_js('moment.js');
        Assets::add_css('bootstrap-datetimepicker.min.css' );
        Assets::add_js('bootstrap-datetimepicker.min.js' );
        Assets::add_css('chart.css');
        Assets::add_js('Chart.min.js');
        $this->session->unset_userdata('chart_query_params');
        $record = $this->backlinks_model->summary();
        $tabular_blocks = array();
        $links_by_months = $this->gcrud->chart('months', array('prefix' => 'M', 'table' => 'seo_backlinks', 'field' => 'counter'));
        Assets::add_js($this->load->view('common/_chart_js', array('field_id' => 'sixMonthChart', 'labels' => $links_by_months['labels'], 'data' => $links_by_months['data'], 'total' => $links_by_months['total'], 'label' => lang('seo_total_links'), 'with_dates' => false, 'table' => 'seo_backlinks'), true), 'inline');
        $result = $this->gcrud->chart('current_month', array('prefix' => 'M', 'table' => 'seo_backlinks', 'field' => 'counter'));
        Assets::add_js($this->load->view('common/_chart_js', array('field_id' => 'monthlyChart', 'labels' => $result['labels'], 'data' => $result['data'], 'total' => $result['total'], 'label' => lang('seo_total_links'), 'with_dates' => true, 'table' => 'seo_backlinks'), true), 'inline');
        $record->tabular_blocks = $tabular_blocks;
        Template::set(array(
            'toolbar_title' => lang('gc_menu_summary'),
            'record' => $record
        ));
        Template::render();
    }
    
    /**
     * Display a list of SEO data
     * @param string $module
     * @param string $filter
     * @param number $offset
     */
    public function manage($module = 'backlinks', $filter = 'all', $offset = 0) {
        $extra = array(
            'model' => 'backlinks',
            'context' => 'components',
            'fields' => $module,
            'prefix' => 'seo'
        );
        if($module == 'backlinks') {
            $this->gcrud->date_filter_session('seo_' . $module, $filter, $offset);
            $extra['advanced_filter'] = array('backlink_product_ids', 'date_from', 'date_to');
            $extra['advanced_filter_date'] = true;
        } else if($module == 'keywords' || $module == 'text') {
            $extra['advanced_filter'] = array('backlink_product_ids');
        }
        $extra['extra_action'] = '<button type="submit" class="btn btn-success btn-sm" name="export_excel" value="export_excel" data-toggle="tooltip" data-title="' . lang('label_export') . '"><i class="fas fa-file-excel"></i></button>';
        if($module == 'words') {
            $extra['advanced_filter'] = array('group_ids');
            $extra['extra_action'] .= anchor(site_url(SITE_AREA . '/components/backlinks/build_text'), '<i class="fas fa-rocket fa-fw mr-1"></i>' . lang('seo_build_text'), 'class="btn btn-primary btn-sm ml-1"');
        }        
        $this->gc->module_view($module, $filter, $offset, 'admin', $extra);
        // $this->seo_model->view($module, $filter, $offset);
    }
    
    /**
     * Add/Edit SEO module Data
     *
     * @return void
     */
    public function form($module = 'backlinks', $action = 'create') {
        if($module == 'backlink_emails') {            
            // user search js and css dependencies
            Assets::add_css('selectize.bootstrap3.css');
            Assets::add_js('selectize.min.js');
            Assets::add_js('selectize-search.js');
            Assets::add_js('location-selection.js');
        }
        if($module == 'text') {
            Assets::add_js(array(
                'tinymce/tinymce.min.js',
            ));	
        }
        $params = array(
            'model' => 'backlinks',
            'context' => $this->context,
            'fields_method' => $module,
            'set_table' => true,
            'uri_segment' => 7,
            'redirect_url' => site_url(SITE_AREA . '/' . $this->context . '/' . $this->moduleName . '/manage/' . $module)
        );
        if($module == 'backlinks') {
            // save domain name
            $params['method'] = 'after_backlink_save';
        }
        if($module == 'backlink_emails') {
            // save password salt in an encrypted string for security reasons
            $params['method'] = 'after_backlink_email_save';
        }        
        $this->gcrud->form('seo_' . $module, $action, 'admin', $params);
    }
    
    /**
     * Build Module Text for documentation
     */
    public function build_text() {
        // Assets::add_css('selectize.bootstrap3.css');
        // Assets::add_js('selectize.min.js');
        // Assets::add_js('selectize-search.js');
        Assets::add_js('bootstrap-select.min.js');
        Assets::add_css('bootstrap-select.min.css');
        
        $field_name = 'word_type_id';
        $maxOptions = 10;
        Assets::add_js( "$('#" . $field_name . "').selectpicker({liveSearch: true, maxOptions: " . $maxOptions . ", multipleSeparator: ', ', showTick: true})" , 'inline' );
        // set selected options in case of
        Assets::add_js("var savedValues = $('#" . $field_name . "').data('value'); $('#" . $field_name . "').selectpicker('val', savedValues.toString().split(','));", 'inline');     
        
        if(isset($_POST['save'])) {
            $this->gcrud->stats(array('module' => 'backlinks'));
            $word_type_id = $this->input->post('word_type_id');
            $word_types = is_array($word_type_id) ? $word_type_id : explode(',', $word_type_id ?? '');//array(12,1,2,5,13,14);
            $with_heading = $this->input->post('with_heading') ? true: false;
            $data = array();
            $this->load->library('parser');
            $text = array();
            $additional_text = array();
            if(!empty($word_types)) {
                foreach($word_types as $word_type) {
                    $sql = "SELECT name, description FROM " . $this->db->dbprefix . "seo_words WHERE status = 1" . (!empty($word_type) ? " AND FIND_IN_SET(" . $word_type . ", group_ids)" : '') . " ORDER BY RAND()";
                    $row = $this->db->query($sql)->row();
                    if($row) {
                        $row->description=str_ireplace('<p>','',$row->description);
                        $row->description=str_ireplace('</p>','',$row->description);
                        if($with_heading) {
                            $additional_text[] = '<h3><strong>' . ucfirst($row->name) . '</strong></h3>' . $this->parser->parse_string($row->description, $data, true);
                        } else {
                            $text[] = '<li>'. $this->parser->parse_string($row->description, $data, true) . '</li>';
                        }
                    }
                }
            }
            $list = implode(PHP_EOL, $text);
            $additional_text = implode(PHP_EOL, $additional_text);
            $output = <<<EOL
<br />
{$additional_text}
<ul>
    {$list}
</ul>

EOL;
    echo $output;
    Template::set('output', $output);
        }
        Template::set(array(
            'toolbar_title' => lang('singular_seo_words')
        ));
        Template::render();
    }
}