<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
 || ##################################################################### ||
 || # GoClixy - Online Software Solutions							    # ||
 || # Copyright (C) 2021 GoClixy. All Rights Reserved.				    # ||
 || # ----------------------------------------------------------------  # ||
 || # This file is owned and copyrighted by GoClixy.com. Your license   # ||
 || # confers  no  title  or ownership in the Script and is not a sale  # ||
 || # of any rights to the Software. This file may not be redistributed # ||
 || # in whole or significant part.									    # ||
 || # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
 || # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
 || ##################################################################### ||
 \*=======================================================================*/
require_once(APPPATH . 'modules/seo/libraries/Simple_html_dom.php');
class Crawler {
    /**
     * Stores the CodeIgniter core object.
     *
     * @access public
     *
     * @var object
     */
    public $CI;
    public $z_cache;
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct($config = array()) {
        $this->CI =& get_instance();
        $this->CI->load->library('gzf_cache', array('lifetime' => 900));
        $this->CI->load->library('slug');
        $this->z_cache = $this->CI->gzf_cache->get_instance();
    }
    
    /**
     * Crawl Page
     * @param string $site_url
     */
    public function crawl($site_url) {
        $dom = new simple_html_dom();
        $html = file_get_html($site_url);
        $title = $html->find('head title', 0);
        $title = $this->clean_text($title->innertext);
        $description = $html->find('head meta[name=description]', 0);
        $description = $this->clean_text($description->content);
        $keywords = $html->find('head meta[name=\'keywords\']', 0);
        if(!empty($keywords)) {
            $keywords = $this->clean_text($keywords->content);
        }
        $headings = $html->find('h1,h2,h3,h4,h5,h6,h7');
        $all_images = $html->find('img');
        $alt_images = $html->find('img[alt]');
        $links = $html->find('a');
        $html->clear(); // clean up memory and kick off the memory issue
        
        // Number of Headings in a Page
        $headings_count = count($headings);
        
        // Images without ALT tag
        $images_without_alt = array();
        foreach($all_images as $key => $image) {
            if(isset($image->attr['src'])) {
                if(!isset($image->attr['alt'])) {
                    $images_without_alt[] = $image->attr['src'];
                }
            } else {
                unset($all_images[$key]);
            }
        }
        
        // Number of Images Count
        $images_count = count($all_images);
        $images_without_alt_count = count($images_without_alt);
        
        // Internal and External Links Counts
        $site_links = array();
        foreach($links as $link) {
            if(isset($link->attr['href'])) {
                $site_links[] = $link->attr['href'];
            }
        }
        $external_links_count = 0;
        $internal_links_count = 0;
        $site_links = array_unique($site_links);
        foreach($site_links as $site_link) {
            if($this->isInternal($site_link)) {
                $internal_links_count += 1;
            } else {
                $external_links_count += 1;
            }
        }
        
        $data = array(
            'title' => $title,
            'description' => $description,
            'keywords' => $keywords,
            'params' => array(
                'images' => $images_count,
                'images_without_alt_count' => $images_without_alt_count,
                'images_without_alt' => $images_without_alt,
                'headings' => $headings_count,
                'internal' => $internal_links_count,
                'external' => $external_links_count
             )
        );
        return $data;
        /*$this->load->library('seo/crawler');
         if($this->crawler->set_url($site_url) !== false) {
         $this->crawler->find_images();
         }*/
    }
    
    private function isInternal($url){
        if( empty( $url ) ){ return false; }
        $urlParsed = parse_url( $url );
        $host = isset($urlParsed['host']) ? $urlParsed['host'] : null;
        if( empty( $host ) ){
            /* maybe we have a relative link like: /wp-content/uploads/image.jpg */
            /* add absolute path to begin and check if file exists */
            $doc_root = $_SERVER['DOCUMENT_ROOT'];
            // Javascript and Hash are not included in the URLs
            if(strpos($url, 'javascript:') === false && strpos($url, '#') === false) {
                $maybefile = $doc_root.$url;
                /* Check if file exists */
                $fileexists = file_exists ( $maybefile );
                if( $fileexists ){
                    /* maybe you want to convert to full url? */
                    return true;
                }
            } else {
                return true;
            }
        }
        /* strip www. if exists */
        $host = str_replace('www.','',$host);
        $thishost = $_SERVER['HTTP_HOST'];
        /* strip www. if exists */
        $thishost = str_replace('www.','',$thishost);
        if( $host == $thishost ) {
            return true;
        }
        return false;
    }
    
    /**
     * Removes all HTML, special characters and extra whitespace from text
     *
     * @param 	string	Text to be cleaned
     * @return 	string	Cleaned text
     */
    private function clean_text($text){
        $preg_patterns = array(
            "/[\x80-\xFF]/", //remove special characters
            "/&nbsp/",
            "/\s+/", //remove extra whitespace
        );
        $text = strip_tags(preg_replace($preg_patterns, " ", html_entity_decode($text, ENT_QUOTES, 'UTF-8')));
        
        return $text;
    }
}
