<?php defined('BASEPATH') || exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Online Software Solutions							   # ||
|| # Copyright (C) 2021 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class Backlinks_model extends BF_Model
{
    public $table_name	                     = 'seo_links';
    protected $table_backlinks	             = 'seo_backlinks';
    protected $table_backlink_emails	     = 'seo_backlink_emails';
    protected $table_backlink_products	     = 'seo_backlink_products';
    protected $table_backlink_types	         = 'seo_backlink_types';
	protected $key			= 'id';
	protected $date_format	= 'datetime';

	protected $log_user 	= true;
	protected $set_created	= true;
	protected $set_modified = true;
	public $soft_deletes	= false;

	protected $created_field     = 'created_on';
    protected $created_by_field  = 'created_by';
	protected $modified_field    = 'modified_on';
    protected $modified_by_field = 'modified_by';

	// Customize the operations of the model without recreating the insert,
    // update, etc. methods by adding the method names to act as callbacks here.
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 	    = array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	// For performance reasons, you may require your model to NOT return the id
	// of the last inserted row as it is a bit of a slow method. This is
    // primarily helpful when running big loops over data.
	protected $return_insert_id = true;

	// The default type for returned row data.
	protected $return_type = 'object';

	// Items that are always removed from data prior to inserts or updates.
	protected $protected_attributes = array();

	// You may need to move certain rules (like required) into the
	// $insert_validation_rules array and out of the standard validation array.
	// That way it is only required during inserts, not updates which may only
	// be updating a portion of the data.
	protected $validation_rules 		= array();
	protected $insert_validation_rules  = array();
	protected $skip_validation 			= false;
	protected $context			        = 'components';
	protected $moduleName               = 'seo';
	public $recommended_limits = array(
	    'title' => 60,
	    'description' => 320,
	    'keywords' => 255,
	    'headings' => 0,
	    'images' => 10,
	    'internal' => 100,
	    'external' => 100
	);

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
    }
	
	/**
	 * Module Fields
	 *
	 * @return array
	 */
    public function backlinks() {
		$fields = array(
			array(
				'type' => 'id',
				'name' => 'id',
				'frontend_index' => true,
				'admin_index' => true
			),
		    array(
		        'type' => 'selectize',
		        'name' => 'backlink_product_ids',
		        'label' => 'gc_menu_products',
		        'frontend' => true,
		        'admin_only' => true,
		        'frontend_index' => false,
		        'admin_index' => true,
		        'options' => 'load:seo_backlink_products;module:seo_backlink_products;max_items:100',
		        'options_detailed' => 1,
		        'rules' => 'trim',
		    ),
		    array(
		        'type' => 'dropdown',
		        'name' => 'type_id',
		        'label' => 'label_type',
		        'frontend' => false,
		        'admin_only' => true,
		        'frontend_index' => true,
		        'admin_index' => true,
		        'options' => 'name:seo_backlink_types;value:database',
		        'options_detailed' => 1,
		        'class' => 3,
		        'rules' => 'trim|is_numeric',
		    ),
		    array(
		        'type' => 'text',
		        'name' => 'link_url',
		        'label' => 'label_link_url',
		        'frontend' => true,
		        'admin_only' => true,
		        'frontend_index' => true,
		        'admin_index' => true,
		        'edit' => 1,
		        'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
		        'rules' => 'trim|valid_url|max_length[255]',
		    ),
		    array(
		        'type' => 'text',
		        'name' => 'submitted_url',
		        'label' => 'label_submitted_url',
		        'frontend' => true,
		        'admin_only' => true,
		        'frontend_index' => true,
		        'admin_index' => true,
		        'rules' => 'trim|valid_url|max_length[255]',
		    ),
		    array(
		        'type' => 'text',
		        'name' => 'submitted_on',
		        'label' => 'label_submitted_on',
		        'frontend' => true,
		        'admin_only' => true,
		        'frontend_index' => true,
		        'admin_index' => true,
		        'class' => 3,
		        'rules' => 'trim|valid_datetime',
		    ),
		    array(
		        'type' => 'none',
		        'name' => 'domain',
		        'label' => 'label_domain',
		        'frontend' => false,
		        'admin_only' => false,
		        'frontend_index' => false,
		        'admin_index' => false,
		        'rules' => 'trim|max_length[255]',
		    ),
		    array(
		        'type' => 'dropdown',
		        'name' => 'page_rank',
		        'label' => 'label_page_rank',
		        'frontend' => true,
		        'admin_only' => true,
		        'frontend_index' => true,
		        'admin_index' => true,
		        'options' => 'name:0;value:0|name:1;value:1|name:2;value:2|name:3;value:3|name:4;value:4|name:5;value:5|name:6;value:6|name:7;value:7|name:8;value:8|name:9;value:9|name:10;value:10',
		        'options_detailed' => 1,
		        'class' => 1,
		        'rules' => 'trim',
		    ),
		    array(
		        'type' => 'text',
		        'name' => 'source',
		        'label' => 'label_source',
		        'frontend' => true,
		        'admin_only' => true,
		        'frontend_index' => false,
		        'admin_index' => false,
		        'class' => 4,
		        'rules' => 'trim|max_length[40]',
		    ),
		    array(
		        'type' => 'text',
		        'name' => 'anchor_text',
		        'label' => 'label_anchor_text',
		        'frontend' => true,
		        'admin_only' => true,
		        'frontend_index' => false,
		        'admin_index' => false,
		        'rules' => 'trim|max_length[100]',
		    ),
    		array(
                'type' => 'text',
                'name' => 'name',
    			'label' => 'label_name',
                'frontend' => true,
                'admin_only' => true,
    		    'frontend_index' => false,
    		    'admin_index' => false,
    			'rules' => 'trim',
    		),
		    array(
		        'type' => 'text',
		        'name' => 'email',
		        'label' => 'label_email',
		        'frontend' => true,
		        'admin_only' => true,
		        'frontend_index' => false,
		        'admin_index' => false,
		        'rules' => 'trim|valid_email',
		    ),
		    array(
		        'type' => 'text',
		        'name' => 'username',
		        'label' => 'bf_username',
		        'frontend' => true,
		        'admin_only' => true,
		        'frontend_index' => false,
		        'admin_index' => false,
		        'rules' => 'trim',
		    ),
		    array(
		        'type' => 'text',
		        'name' => 'password_salt',
		        'label' => 'bf_password',
		        'frontend' => true,
		        'admin_only' => true,
		        'frontend_index' => false,
		        'admin_index' => false,
		        'rules' => 'trim',
		    ),
		    array(
		        'type' => 'textarea',
		        'name' => 'comments',
		        'label' => 'label_submitted_text',
		        'frontend' => true,
		        'admin_only' => true,
		        'frontend_index' => false,
		        'admin_index' => false,
		        'rules' => 'trim',
		    ),
		    array(
		        'type' => 'textarea',
		        'name' => 'remarks',
		        'label' => 'label_remarks',
		        'frontend' => true,
		        'admin_only' => true,
		        'frontend_index' => false,
		        'admin_index' => false,
		        'rules' => 'trim',
		    ),
		    array(
		        'type' => 'checkbox',
		        'name' => 'status',
		        'label' => 'label_status',
		        'frontend' => false,
		        'admin_only' => true,
		        'frontend_index' => false,
		        'admin_index' => true,
		        'rules' => 'trim',
		    ),
		    array(
		        'type' => 'none',
		        'name' => 'created_on',
		        'label' => 'created_on',
		        'frontend' => true,
		        'admin_only' => true,
		        'frontend_index' => false,
		        'admin_index' => false,
		        'rules' => 'trim',
		    ),
		    array(
		        'type' => 'none',
		        'name' => 'created_by',
		        'label' => 'created_by',
		        'frontend' => true,
		        'admin_only' => true,
		        'frontend_index' => false,
		        'admin_index' => $this->auth->role_id() == 1 ? true : false,
		        'rules' => 'trim',
		    ),
		    array(
		        'type' => 'none',
		        'name' => 'created_ip',
		        'label' => 'created_ip',
		        'frontend' => true,
		        'admin_only' => true,
		        'frontend_index' => false,
		        'admin_index' => false,
		        'rules' => 'trim',
		    ),
		    array(
		        'type' => 'none',
		        'name' => 'modified_on',
		        'label' => 'modified_on',
		        'frontend' => true,
		        'admin_only' => true,
		        'frontend_index' => false,
		        'admin_index' => false,
		        'rules' => 'trim|max_length[255]',
		    ),
		    array(
		        'type' => 'none',
		        'name' => 'modified_by',
		        'label' => 'modified_by',
		        'frontend' => true,
		        'admin_only' => true,
		        'frontend_index' => false,
		        'admin_index' => false,
		        'rules' => 'trim',
		    ),
		    array(
		        'type' => 'none',
		        'name' => 'modified_ip',
		        'label' => 'modified_ip',
		        'frontend' => true,
		        'admin_only' => true,
		        'frontend_index' => false,
		        'admin_index' => false,
		        'rules' => 'trim',
		    )
		);
		return $fields;
	}
	
	/**
	 * Backlink Email Fields
	 *
	 * @return array
	 */
	public function backlink_emails() {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'first_name',
	            'label' => 'label_first_name',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'last_name',
	            'label' => 'label_last_name',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'email',
	            'label' => 'label_email',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
	            'rules' => 'trim|valid_email',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'username',
	            'label' => 'bf_username',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'password_salt',
	            'label' => 'bf_password',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'gender',
	            'label' => 'label_gender',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'options' => 'name:label_male;value:1|name:label_female;value:2',
	            'options_detailed' => true,
	            'options_lang' => true,
	            'class' => 2,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'dob',
	            'label' => 'label_dob',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'class' => 2,
	            'rules' => 'trim|valid_date',
	        ),
	        array(
	            'type' => 'textarea',
	            'name' => 'address',
	            'label' => 'label_address',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'class' => 6,
	            'rules' => 'trim|required',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'postcode',
	            'label' => 'label_postcode',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'class' => 3,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'location',
	            'name' => 'location',
	            'label' => 'label_location',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_on',
	            'label' => 'created_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_by',
	            'label' => 'created_by',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_ip',
	            'label' => 'created_ip',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_on',
	            'label' => 'modified_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim|max_length[255]',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_by',
	            'label' => 'modified_by',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_ip',
	            'label' => 'modified_ip',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        )
	    );
	    return $fields;
	}
	
	/**
	 * Backlink Products Fields
	 *
	 * @return array
	 */
	public function backlink_products($role = 'admin') {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'name',
	            'label' => 'label_name',
	            'placeholder' => 'placeholder_name',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
	            'rules' => 'required|trim|max_length[255]',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'priority',
	            'label' => 'label_priority',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'value' => 0,
	            'class' => 2,
	            'rules' => 'trim|max_length[255]'
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'status',
	            'label' => 'label_status',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	            'field_extra' => 'index_action_admin:options|icon_primary#fas fa-thumbs-up|icon_secondary#fas fa-thumbs-down'
	        )
	    );
	    return $fields;
	}
	
	/**
	 * Backlink Type Fields
	 *
	 * @return array
	 */
	public function backlink_types() {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'name',
	            'label' => 'label_name',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'edit' => 1,
	            'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
	            'rules' => 'trim|required|max_length[150]'
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'priority',
	            'label' => 'label_priority',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'value' => 0,
	            'class' => 2,
	            'rules' => 'trim|required|is_numeric',
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'status',
	            'label' => 'label_status',
	            'frontend' => false,
	            'admin_only' => false,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        )
	    );
	    return $fields;
	}
	
	/**
	 * Seo Keywords
	 *
	 * @return array
	 */
	public function keywords() {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'selectize',
	            'name' => 'backlink_product_ids',
	            'label' => 'gc_menu_products',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => true,
	            'options' => 'load:seo_backlink_products;module:seo_backlink_products;max_items:100',
	            'options_detailed' => 1,
	            'rules' => 'trim|max_length[255]',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'name',
	            'label' => 'label_name',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'admin_index' => true,
	            'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
	            'rules' => 'trim|unique[seo_keywords.name, seo_keywords.id]',
	        )
	    );
	    return $fields;
	}
	
	/**
	 * Seo Text Fields
	 *
	 * @return array
	 */
	public function text() {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'selectize',
	            'name' => 'backlink_product_ids',
	            'label' => 'gc_menu_products',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => true,
	            'options' => 'load:seo_backlink_products;module:seo_backlink_products;max_items:100',
	            'options_detailed' => 1,
	            'rules' => 'trim|max_length[255]',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'title',
	            'label' => 'label_title',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'edit' => 1,
	            'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
	            'rules' => 'trim|required|max_length[255]',
	        ),
	        array(
	            'type' => 'textarea',
	            'name' => 'description',
	            'label' => 'label_description',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'class' => 9,
	            'field_class' => 'editor',
	            'rules' => 'trim|required',
	        ),
	        array(
	            'type' => 'filemanager',
	            'name' => 'image',
	            'label' => 'label_image',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim|max_length[255]',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_on',
	            'label' => 'created_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_by',
	            'label' => 'created_by',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_ip',
	            'label' => 'created_ip',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_on',
	            'label' => 'modified_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_by',
	            'label' => 'modified_by',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_ip',
	            'label' => 'modified_ip',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        
	    );
	    return $fields;
	}
	
	/**
	 * Seo Words Fields
	 *
	 * @return array
	 */
	public function words() {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'group_ids',
	            'label' => 'label_type',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'name:seo_word_types;value:database',
	            'options_detailed' => 1,
	            'field_extra' => 'selectpicker:1;multiple:1;max_options:10',
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'name',
	            'label' => 'label_name',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'edit' => 1,
	            'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
	            'rules' => 'trim|required|max_length[255]',
	        ),
	        array(
	            'type' => 'textarea',
	            'name' => 'description',
	            'label' => 'label_description',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim|required',
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'status',
	            'label' => 'label_status',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        
	    );
	    return $fields;
	}
	
	/**
	 * After Backlink Save
	 * @param number $id
	 * @param string $action_type
	 * @param array $record
	 */
	public function after_backlink_save($id, $action_type = 'insert', $record = array()) {
	    $this->load->library('authenticate');
	    $record = $this->db->where('id', $id)->get($this->table_backlinks)->row();
	    if($record) {
	        $domain = $this->get_domain($record->link_url);
	        // $link_url = $this->authenticate->encrypt($record->link_url); // store url in encrypted format
	        // 'link_url' => $link_url, 
	        $password_salt = $this->authenticate->encrypt($record->password_salt);
	        $this->db->where('id', $id)->update($this->table_backlinks, array('domain' => $domain, 'password_salt' => $password_salt));
	    }
	}
	
	/**
	 * After Backlink Email Save
	 * @param number $id
	 * @param string $action_type
	 * @param array $record
	 */
	public function after_backlink_email_save($id, $action_type = 'insert', $record = array()) {
	    $this->load->library('authenticate');
	    $record = $this->db->where('id', $id)->get($this->table_backlink_emails)->row();
	    if($record) {
	        $password_salt = $this->authenticate->encrypt($record->password_salt);
	        $this->db->where('id', $id)->update($this->table_backlink_emails, array('password_salt' => $password_salt));
	    }
	}
	
	/**
	 * Update Domain Name
	 */
	private function update_domain() {
	    $result = $this->db->select('id, link_url')->get($this->table_backlinks)->result();
	    foreach($result as $link) {
	        $data = array(
	            'domain' => $this->get_domain($link->link_url)
	        );
	        $this->db->where('id', $link->id);
	        $this->db->update('rs0d_links', $data);
	    }
	}
	
	/**
	 * Get Domain Name From URL
	 * @param string $url
	 * @return string|boolean
	 */
	public function get_domain($url) {
	    $pieces = parse_url($url);
	    $domain = isset($pieces['host']) ? $pieces['host'] : '';
	    if (preg_match('/(?P<domain>[a-z0-9][a-z0-9\-]{1,63}\.[a-z\.]{2,6})$/i', $domain, $regs)) {
	        return $regs['domain'];
	    }
	    return false;
	}
	
	/**
	 * Replace Placeholder
	 * @param string $format
	 * @param string $args
	 * @return mixed
	 */
	public function replace_placeholders($format, $args) {
	    $i = 1;
	    foreach ($args as $arg) {
	        $arg_value = $arg->type == 'URL' ? anchor($arg->value, $arg->value, 'target="_blank"') : $arg->value;
	        $format = str_replace("\$" . $i, "$arg_value", $format);
	        $i++;
	    }
	    return $format;
	}
	
	/**
	 * Display Summary on the Home Page
	 * @return array
	 */
	public function summary_old() {	    
	    $sql = "SELECT COUNT(*) AS total_links";
	    $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->table_name . " WHERE title IS NULL) AS missing_title";
	    $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->table_name . " WHERE description IS NULL) missing_description";
	    $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->table_name . " WHERE keywords IS NULL) missing_keywords";
	    $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->table_backlinks . ") total_backlinks";
	    $sql .= " FROM " . $this->db->dbprefix . $this->table_backlinks;
	    $query = $this->db->query($sql);
	    return $query->row();
	}
	
	
	/**
	 * Display Summary
	 * @return array
	 */
	public function summary() {
	    $sql = "SELECT COUNT(*) AS total_links";
	    
	    // today links
	    $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->table_backlinks;
	    $sql .= " WHERE YEAR(created_on) = YEAR(NOW()) AND MONTH(created_on) = MONTH(NOW()) AND DAY(created_on) = DAY(NOW())) AS today_links";
	    
	    // this month links
	    $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->table_backlinks;
	    $sql .= " WHERE MONTH(created_on) = MONTH(CURDATE()) AND YEAR(created_on)  =  YEAR(CURDATE())) AS month_links";
	    
	    // yesterday links
	    $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->table_backlinks;
	    $sql .= " WHERE created_on <= NOW() AND created_on >= DATE_ADD(Now(),interval - 1 day)) AS yesterday_links";
	    $sql .= " FROM " . $this->db->dbprefix . $this->table_backlinks;
	    $query = $this->db->query($sql);
	    $record = $query->row();
	    if($record) {
	        $data = array();
	        foreach($record as $key => $value) {
	            switch($key) {
	                case 'total_links':
	                    $url = site_url(SITE_AREA . '/components/backlinks/manage/backlinks');
	                    break;
	                case 'today_links':
	                    $url = site_url(SITE_AREA . '/components/backlinks/manage/backlinks/filter_today');
	                    break;
	                case 'month_links':
	                    $url = site_url(SITE_AREA . '/components/backlinks/manage/backlinks/filter_month');
	                    break;
	                case 'yesterday_links':
	                    $url = site_url(SITE_AREA . '/components/backlinks/manage/backlinks/filter_yesterday');
	                    break;
	                default:
	                    $url = null;
	            }
	            $data[$key] = array(
	                'value' => $value,
	                'url' => $url
	            );
	        }
	        return (object) $data;
	    } else {
	        return $record;
	    }
	}
	
	/**
	 * Format Product
	 * @param string $data
	 * @return string
	 */
	public function format_product($data) {
	    $field_value = '';
	    if(!empty($data)) {
    	    $fields_array = explode('~~~', $data ?? '');
    	    if(!empty($fields_array)) {
    	        $urls = array();
    	        foreach($fields_array as $field_array) {
    	            $column_values = explode('!~!', $field_array ?? ''); // column value separator
    	            $url_title = $column_values[0]; // row_id
    	            $url_title .= ' - ' . $column_values[1]; // row title/name
    	            $urls[] = $url_title;
    	        }
    	        $field_value = implode('<br />', $urls);
    	    }
	    }
	    return $field_value;
	}
}