<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Banners controller
 */
class Settings extends Admin_Controller
{
	protected $permissionCreate = 'Banners.Settings.Create';
	protected $permissionDelete = 'Banners.Settings.Delete';
	protected $permissionEdit   = 'Banners.Settings.Edit';
	protected $permissionView   = 'Banners.Settings.View';
	
	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct() {
		parent::__construct();

		$this->auth->restrict($this->permissionView);
		clear_cache('banners');
		$this->load->model('banners_model', null, true);
		$this->lang->load('banners');	
		
		// DATE TIME PICKER
		Assets::add_js('moment.js');
		Assets::add_css('bootstrap-datetimepicker.min.css');
		Assets::add_js('bootstrap-datetimepicker.min.js');
		
		// LOAD JAVASCRIPT AND CSS FILE
		Assets::add_module_js('banners', 'banners.js');		
		Template::set_block('sub_nav', 'settings/_sub_nav');
	}
	
	/**
	 * Manage Banners
	 * @param string $filter
	 * @param number $offset
	 */
	public function index($filter = 'all', $offset = 0) {
	    $this->manage('banners', $filter, $offset);
	}
	
	/**
	 * Manage Banners/Types/Sliders
	 * @param string $module
	 * @param string $filter
	 * @param number $offset
	 */
	public function manage($module = 'banners', $filter = 'all', $offset = 0) {
	    $extra = array(
	        'model' => 'banners',
	        'context' => 'settings',
	        'fields' => $module,
	        'module_name' => 'banners',
	        'prefix' => 'banner'
	    );
	    if($module == 'banners') {
	        $extra['prefix'] = '';
	    }
	    $this->gc->module_view($module, $filter, $offset, 'admin', $extra);
	}
	
	/**
	 * Add/Edit Banners Data
	 *
	 * @return void
	 */
	public function form($module = 'banners', $action = 'create') {
	    $params = array(
	        'selectize' => true,
	        'model' => 'banners',
	        'context' => 'settings',
	        'module_name' => 'banners',
	        'fields_method' => $module,
	        'prefix' => 'banner'
	    );
	    if($module == 'banners') {
	        Template::set('layout', 'tabs');
	        $params['group_layout'] = true;
	        $params['prefix'] = '';
	        $params['method'] = 'after_banner_save';
	        $types = array();
	        $banner_types = $this->db->select('id, type')->get_where('banner_types')->result();
	        if($banner_types) {
	            foreach($banner_types as $banner_type) {
	                $types[$banner_type->id] = $banner_type->type; 
	            }
	        }
	        Assets::add_js("
	         var bannerTypes = " . json_encode($types) . ";
             // show/hide fields on POST validation errors and banner edit
             var defaultValue = $('#type_id').val(); 
             if(defaultValue != 0)  {
                var bannerType = bannerTypes[defaultValue];
                showHideFields(bannerType);
             }
	         $('#type_id').change(function () {  
                var typeId = $(this).val(); 
                var bannerType = bannerTypes[typeId];
                showHideFields(bannerType);
             });
             function showHideFields(bannerType) {
                $('.banner-field').hide();
                $('.banner-' + bannerType).show();
             }
	        ", 'inline');
	    }
	    /*if($module == 'sliders') {
	        Template::set('banner_types', $this->db->get_where('banner_types')->result());
	    }*/
	    $this->gcrud->form($module, $action, 'admin', $params);
	}
}