<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Migration_Install_banners extends Migration
{
	/**
	 * The name of the database table
	 *
	 * @var String
	 */
    private $table_name = 'banners';
    private $table_sliders = 'sliders';

	/**
	 * The table's fields
	 *
	 * @var Array
	 */
	private $fields = array(
		'id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'auto_increment' => TRUE,
		),
		'type_id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => FALSE,
		),
		'ad_id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => FALSE,
			'default' => 0
		),
		'title' => array(
			'type' => 'VARCHAR',
			'constraint' => 100,
			'null' => FALSE,
		),
		'image' => array(
			'type' => 'VARCHAR',
			'constraint' => 100,
			'null' => TRUE,
		),
		'url' => array(
			'type' => 'VARCHAR',
			'constraint' => 100,
			'null' => TRUE,
		),
		'target' => array(
			'type' => 'VARCHAR',
			'constraint' => 20,
			'null' => TRUE,
		),
		'start_date' => array(
			'type' => 'DATE',
			'null' => TRUE
		),
		'end_date' => array(
			'type' => 'DATE',
			'null' => TRUE
		),
		'max_impressions' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => TRUE,
			'default' => 0,
		),
		'impressions' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => TRUE,
			'default' => 0,
		),
		'max_clicks' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => TRUE,
			'default' => 0,
		),
		'clicks' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => TRUE,
			'default' => 0,
		),
		'slider_heading' => array(
			'type' => 'VARCHAR',
			'constraint' => 100,
			'null' => TRUE,
		),
		'html_text' => array(
			'type' => 'TEXT',
			'null' => TRUE,
		),
		'all_pages' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
		),
		'display_page' => array(
			'type' => 'VARCHAR',
			'constraint' => 20,
			'null' => TRUE,
		),
		'active' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
		),
		'priority' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => FALSE,
			'default' => '0',
		),
	    'categories' => array(
	        'type' => 'TEXT',
	        'null' => TRUE
	    ),
	    'keywords' => array(
	        'type' => 'TEXT',
	        'null' => TRUE
	    ),
		'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'modified_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
	);
	
	/**
	 * The banner_types table fields
	 *
	 * @var Array
	 */
	private $banner_types = array(
			'id' => array(
					'type' => 'TINYINT',
					'constraint' => 3,
					'auto_increment' => TRUE,
			),
			'type' => array(
					'type' => 'CHAR',
					'constraint' => 5,
					'null' => FALSE,
			),
			'title' => array(
					'type' => 'VARCHAR',
					'constraint' => 255,
					'null' => FALSE,
			),
			'width' => array(
					'type' => 'SMALLINT',
					'constraint' => 6,
					'null' => TRUE,
			),
			'height' => array(
					'type' => 'SMALLINT',
					'constraint' => 6,
					'null' => TRUE,
			),
			'filesize' => array(
					'type' => 'MEDIUMINT',
					'constraint' => 8,
					'null' => TRUE,
					'default' => '0',
			),
			'location' => array(
					'type' => 'CHAR',
					'constraint' => 10,
					'null' => TRUE,
			),
			'description' => array(
					'type' => 'MEDIUMTEXT',
					'null' => TRUE,
			),
	);
	
	/**
	 * The banner_categories table fields
	 * 
	 * @var Array
	 */
	private $banner_categories = array(
			'id' => array(
					'type' => 'INT',
					'constraint' => 11,
					'auto_increment' => TRUE,
			),
			'banner_id' => array(
					'type' => 'INT',
					'constraint' => 11,
					'null' => FALSE,
			),
			'parent_id' => array(
					'type' => 'INT',
					'constraint' => 11,
					'null' => FALSE,
			),
			'sub_id' => array(
					'type' => 'INT',
					'constraint' => 11,
					'null' => FALSE,
					'default' => 0,
			),
			'subsub_id' => array(
					'type' => 'INT',
					'constraint' => 11,
					'null' => FALSE,
					'default' => 0,
			),
	);
	
	/**
	 * The banner_locations table fields
	 *
	 * @var Array
	 */
	private $banner_locations = array(
			'id' => array(
					'type' => 'INT',
					'constraint' => 11,
					'auto_increment' => TRUE,
			),
			'banner_id' => array(
					'type' => 'INT',
					'constraint' => 11,
					'null' => FALSE,
			),
			'country_iso' => array(
					'type' => 'CHAR',
					'constraint' => 2,
					'null' => FALSE,
					'default' => 'US',
			),
			'state_id' => array(
					'type' => 'INT',
					'constraint' => 11,
					'null' => TRUE,
			),
			'city_id' => array(
					'type' => 'INT',
					'constraint' => 11,
					'null' => TRUE,
			),
	);
	
	/**
	 * The banner_sliders table fields
	 *
	 * @var Array
	 */
	private $banner_sliders = array(
	    'id' => array(
	        'type' => 'INT',
	        'constraint' => 11,
	        'auto_increment' => TRUE,
	    ),
	    'type_id' => array(
	        'type' => 'TINYINT',
	        'constraint' => 3,
	        'null' => FALSE,
	        'default' => 0
	    ),
	    'title' => array(
	        'type' => 'VARCHAR',
	        'constraint' => 255,
	        'null' => FALSE
	    ),
	    'image' => array(
	        'type' => 'VARCHAR',
	        'constraint' => 255,
	        'unique' => TRUE,
	        'null' => FALSE
	    ),
	    'html_text' => array(
	        'type' => 'VARCHAR',
	        'constraint' => 255,
	        'null' => TRUE
	    ),
	    'url' => array(
	        'type' => 'VARCHAR',
	        'constraint' => 255,
	        'null' => TRUE
	    ),
	    'target' => array(
	        'type' => 'VARCHAR',
	        'constraint' => 10,
	        'null' => TRUE
	    ),
	    'status' => array(
	        'type' => 'TINYINT',
	        'constraint' => 1,
	        'null' => FALSE,
	        'default' => 0
	    )
	);

	/**
	 * Install this migration
	 *
	 * @return void
	 */
	public function up()
	{
		// banners table fields
		$this->dbforge->add_field($this->fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('type_id', false);
		$this->dbforge->create_table($this->table_name);
		
		// banner_types table fields
		$this->dbforge->add_field($this->banner_types);
		$this->dbforge->add_key('id', true);
		$this->dbforge->create_table('banner_types');
		
		// banner_categories table fields
		$this->dbforge->add_field($this->banner_categories);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('banner_id', false);
		$this->dbforge->add_key('parent_id', false);
		$this->dbforge->add_key('sub_id', false);
		$this->dbforge->add_key('subsub_id', false);
		$this->dbforge->create_table('banner_categories');
		$unique_category_sql = "ALTER TABLE " . $this->db->dbprefix . $this->banner_categories . " ADD UNIQUE INDEX `unique_row` (`banner_id`, `parent_id`, `sub_id`, `subsub_id`)";
		$this->db->query ($unique_category_sql);
		
		// banner_locations table fields
		$this->dbforge->add_field($this->banner_locations);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('banner_id', false);
		$this->dbforge->add_key('city_id', false);
		$this->dbforge->add_key('state_id', false);
		$this->dbforge->add_key('country_iso', false);		
		$this->dbforge->create_table('banner_locations');
		$unique_location_sql = "ALTER TABLE " . $this->db->dbprefix . $this->banner_locations . " ADD UNIQUE INDEX `unique_row` (`banner_id`, `country_iso`, `state_id`, `city_id`)";
		$this->db->query ($unique_location_sql);
		
		// banner sliders fields
		$this->dbforge->add_field($this->banner_sliders);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('status', false);
		$this->dbforge->create_table($this->table_sliders);
	}

	//--------------------------------------------------------------------

	/**
	 * Uninstall this migration
	 *
	 * @return void
	 */
	public function down() {
		$this->dbforge->drop_table($this->table_name);
		$this->dbforge->drop_table('banner_types');
		$this->dbforge->drop_table('banner_categories');
		$this->dbforge->drop_table('banner_locations');
		$this->dbforge->drop_table($this->table_sliders);
	}

	//--------------------------------------------------------------------

}