<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class Banners_model extends BF_Model {

	public $table_name	= "banners";
	protected $key			= "id";
	protected $soft_deletes	= false;
	protected $date_format	= "datetime";

	protected $log_user 	= FALSE;

	protected $set_created	= true;
	protected $set_modified = true;
	protected $created_field = "created_on";
	protected $modified_field = "modified_on";

	/*
		Customize the operations of the model without recreating the insert, update,
		etc methods by adding the method names to act as callbacks here.
	 */
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 		= array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	/*
		For performance reasons, you may require your model to NOT return the
		id of the last inserted row as it is a bit of a slow method. This is
		primarily helpful when running big loops over data.
	 */
	protected $return_insert_id 	= TRUE;

	// The default type of element data is returned as.
	protected $return_type 			= "object";

	// Items that are always removed from data arrays prior to
	// any inserts or updates.
	protected $protected_attributes = array();

	/*
		You may need to move certain rules (like required) into the
		$insert_validation_rules array and out of the standard validation array.
		That way it is only required during inserts, not updates which may only
		be updating a portion of the data.
	 */
	protected $validation_rules 		= array(
		array(
			"field"		=> "type_id",
			"label"		=> "lang:label_banner_type",
			"rules"		=> "required"
		),
		array(
			"field"		=> "title",
			"label"		=> "lang:label_title",
			"rules"		=> "required|trim|max_length[100]|sanitize"
		),
	);
	protected $insert_validation_rules 	= array();
	protected $skip_validation 			= FALSE;	
	protected $z_cache;
	
	public function __construct() {
		$this->load->library('settings/Settings_lib');
		if(settings_item('site.enable_cache') == 1) {
			$this->load->library('gzf_cache', array('lifetime' => 900));
			$this->load->library('slug');
			$this->z_cache = $this->gzf_cache->get_instance();
		}
	}
	
	/**
	 * Banners Fields
	 *
	 * @return array
	 */
	public function banners() {
	    $modules = array('name:201;label:' . lang('bf_home') . ';value:201'); // 201 is a manually specified id, not
	    foreach(gc_modules() as $module_id => $module) {
	        $modules[] = 'name:' . gc_module_id($module) . ';label:' . lang($module) . ';value:' . gc_module_id($module);
	    }
	    $modules = implode('|', $modules);
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'type_id',
	            'label' => 'label_type',
	            'frontend' => false,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'name:banner_types;value:database',
	            'options_detailed' => 1,
	            'class' => 4,
	            'rules' => 'trim|is_numeric'
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'ad_id',
	            'label' => 'label_ad',
	            'frontend' => false,
	            'admin_only' => false,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'selectize',
	            'name' => 'group_ids',
	            'label' => 'gc_menu_groups',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => true,
	            'options' => 'load:banner_groups;module:groups;max_items:100',
	            'options_detailed' => 1,
	            'rules' => 'trim|max_length[255]',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'title',
	            'label' => 'label_title',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'edit' => 1,
	            'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
	            'rules' => 'trim|required|max_length[100]',
	        ),
	        array(
	            'type' => 'filemanager',
	            'name' => 'image',
	            'label' => 'label_image',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	            'control_class' => 'banner-field banner-image'
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'url',
	            'label' => 'label_url',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim|max_length[100]',
	            'control_class' => 'banner-field banner-image'
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'target',
	            'label' => 'label_target',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'options' => 'name:option_new_window;value:_blank|name:option_same_window;value:_self|name:option_top_window;value:_top|name:option_parent_window;value:_parent',
	            'options_detailed' => 1,
	            'options_lang' => 1,
	            'class' => 2,
	            'rules' => 'trim|max_length[20]',
	            'control_class' => 'banner-field banner-image'
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'start_date',
	            'label' => 'label_start_date',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'class' => 3,
	            'rules' => 'trim|valid_date',
	            'control_class' => 'banner-field banner-image'
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'end_date',
	            'label' => 'label_end_date',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'class' => 3,
	            'rules' => 'trim|valid_date',
	            'control_class' => 'banner-field banner-image'
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'max_impressions',
	            'label' => 'label_max_impressions',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'value' => 0,
	            'class' => 2,
	            'rules' => 'trim|is_numeric|required',
	            'control_class' => 'banner-field banner-image'
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'impressions',
	            'label' => 'label_impressions',
	            'frontend' => false,
	            'admin_only' => false,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'max_clicks',
	            'label' => 'label_max_clicks',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'value' => 0,
	            'class' => 2,
	            'rules' => 'trim|is_numeric|required',
	            'control_class' => 'banner-field banner-image'
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'clicks',
	            'label' => 'label_clicks',
	            'frontend' => false,
	            'admin_only' => false,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'slider_heading',
	            'label' => 'label_slider_heading',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim|max_length[100]',
	            'tooltip' => lang('help_slider_heading'),
	            'control_class' => 'banner-field banner-image'
	        ),
	        array(
	            'type' => 'textarea',
	            'name' => 'html_text',
	            'label' => 'label_text_html',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	            'tooltip' => lang('help_html_text'),
	            // 'control_class' => 'banner-field banner-text banner-html banner-image'
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'priority',
	            'label' => 'label_priority',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'value' => 0,
	            'class' => 2,
	            'rules' => 'trim|is_numeric|required',
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'status',
	            'label' => 'label_status',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'module_ids',
	            'label' => 'applicable_modules',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'options' => $modules,
	            'options_lang' => 0,
	            'options_detailed' => 1,
	            'placeholder' => 'applicable_modules',
	            'rules' => 'trim|is_array|max_length[455]',
	            'field_group' => 17
	        ),
	        array(
	            'type' => 'selectize',
	            'name' => 'categories_include',
	            'label' => 'bn_categories_in',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'options' => 'load:categories;module:categories;max_items:100',
	            'options_detailed' => 1,
	            'tooltip' => lang('bn_categories_in_help'),
	            'rules' => 'trim|max_length[455]',
	            'field_group' => 17
	        ),
	        array(
	            'type' => 'selectize',
	            'name' => 'categories_exclude',
	            'label' => 'bn_categories_ex',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'options' => 'load:categories;module:categories;max_items:100',
	            'options_detailed' => 1,
	            'tooltip' => lang('bn_categories_ex_help'),
	            'rules' => 'trim|max_length[455]',
	            'field_group' => 17
	        ),
	        array(
	            'type' => 'selectize',
	            'name' => 'cities_include',
	            'label' => 'bn_cities_in',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'options' => 'load:cities;module:cities;max_items:100',
	            'options_detailed' => 1,
	            'tooltip' => lang('bn_cities_in_help'),
	            'rules' => 'trim|max_length[455]',
	            'field_group' => 17
	        ),
	        array(
	            'type' => 'selectize',
	            'name' => 'cities_exclude',
	            'label' => 'bn_cities_ex',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'options' => 'load:cities;module:cities;max_items:100',
	            'options_detailed' => 1,
	            'tooltip' => lang('bn_categories_ex_help'),
	            'rules' => 'trim|max_length[455]',
	            'field_group' => 17
	        ),
	        array(
	            'type' => 'selectize',
	            'name' => 'tags_include',
	            'label' => 'bn_tags_in',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'options' => 'load:tags;module:tags;max_items:100',
	            'options_detailed' => 1,
	            'tooltip' => lang('bn_tags_in_help'),
	            'rules' => 'trim|max_length[455]',
	            'field_group' => 17
	        ),
	        array(
	            'type' => 'selectize',
	            'name' => 'tags_exclude',
	            'label' => 'bn_tags_ex',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'options' => 'load:tags;module:tags;max_items:100',
	            'options_detailed' => 1,
	            'tooltip' => lang('bn_tags_ex_help'),
	            'rules' => 'trim|max_length[455]',
	            'field_group' => 17
	        ),
	        array(
	            'type' => 'textarea',
	            'name' => 'links_include',
	            'label' => 'bn_links_in',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'tooltip' => lang('bn_links_in_help'),
	            'rules' => 'trim|max_length[455]',
	            'field_extra' => 'rows:3',
	            'field_group' => 17
	        ),
	        array(
	            'type' => 'textarea',
	            'name' => 'links_exclude',
	            'label' => 'bn_links_ex',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'tooltip' => lang('bn_links_ex_help'),
	            'rules' => 'trim|max_length[455]',
	            'field_extra' => 'rows:3',
	            'field_group' => 17
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_on',
	            'label' => 'created_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_on',
	            'label' => 'modified_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        
	    );
	    return $fields;
	}
	
	/**
	 * Banner Types Fields
	 *
	 * @return array
	 */
	public function types() {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'type',
	            'label' => 'label_banner_type',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'name:label_image;value:image|name:label_html;value:html|name:label_text;value:text',
	            'options_detailed' => 1,
	            'options_lang' => 1,
	            'class' => 2,
	            'rules' => 'trim|required'
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'title',
	            'label' => 'label_title',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'edit' => 1,
	            'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
	            'rules' => 'trim|required|max_length[255]',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'width',
	            'label' => 'banner_width',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'value' => 0,
	            'class' => 2,
	            'rules' => 'trim|is_numeric|required',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'height',
	            'label' => 'banner_height',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'value' => 0,
	            'class' => 2,
	            'rules' => 'trim|is_numeric|required',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'filesize',
	            'label' => 'label_filesize',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'tooltip' => lang('help_filesize'),
	            'value' => 0,
	            'class' => 2,
	            'rules' => 'trim|is_numeric|required',
	        ),/*,
	        array(
	            'type' => 'dropdown',
	            'name' => 'location',
	            'label' => 'label_location',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'name:label_top;value:top|name:label_left;value:left|name:label_right;value:right|name:label_middle;value:middle|name:label_bottom;value:bottom|name:label_front_slider;value:slider',
	            'options_detailed' => 1,
	            'options_lang' => 1,
	            'class' => 2,
	            'rules' => 'trim|required',
	        ),*/
	        array(
	            'type' => 'textarea',
	            'name' => 'description',
	            'label' => 'label_description',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        
	    );
	    return $fields;
	}
	
	/**
	 * Banner Groups Fields
	 *
	 * @return array
	 */
	public function groups() {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'name',
	            'label' => 'label_name',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'edit' => 1,
	            'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
	            'rules' => 'trim|required|max_length[150]'
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'name_lang',
	            'label' => 'name_lang',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'slug',
	            'label' => 'label_slug',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim|max_length[150]'
	        ),
	        array(
	            'type' => 'iconpicker',
	            'name' => 'icon',
	            'label' => 'label_icon',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'class' => 3,
	            'rules' => 'trim'
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'priority',
	            'label' => 'label_priority',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'value' => 0,
	            'class' => 2,
	            'rules' => 'trim|required|is_numeric',
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'status',
	            'label' => 'label_status',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        )
	    );
	    return $fields;
	}
	
	/**
	 * Banner Sliders Fields
	 *
	 * @return array
	 */
	public function sliders() {
	    $custom_fields = array(
	        array(
	            'name' => 'key',
	            'type' => 'text',
	            'label' => lang('label_key')
	        ),
	        array(
	            'name' => 'value',
	            'type' => 'text',
	            'label' => lang('label_value')
	        )
	    );
	    Template::set('custom_fields', $custom_fields);
	    $link_options = array(
	        array(
	            'name' => 'url',
	            'type' => 'text',
	            'label' => lang('label_url')
	        ),
	        array(
	            'name' => 'label',
	            'type' => 'text',
	            'label' => lang('label_label')
	        ),
	        array(
	            'name' => 'label_lang',
	            'type' => 'checkbox',
	            'label' => lang('is_lang')
	        ),
	        array(
	            'name' => 'target',
	            'type' => 'dropdown',
	            'label' => lang('label_target'),
	            'options' => array('_self' => lang('option_same_window'), '_blank' => lang('option_new_window'))
	        )
	    );
	    Template::set('link_options', $link_options);
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'group_ids',
	            'label' => 'gc_menu_groups',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => true,
	            'options' => 'name:banner_groups;value:database',
	            'options_detailed' => 1,
	            'rules' => 'trim|max_length[255]',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'title',
	            'label' => 'label_title',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'edit' => 1,
	            'multi_language' => 1,
	            'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
	            'rules' => 'trim|required|max_length[255]',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'slug',
	            'label' => 'label_slug',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim|max_length[255]'
	        ),
	        array(
	            'type' => 'filemanager',
	            'name' => 'image',
	            'label' => 'label_image',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim|max_length[255]',
	        ),
	        array(
	            'type' => 'textarea',
	            'name' => 'html_text',
	            'label' => 'label_text_html',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'multi_language' => 1,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'custom',
	            'name' => 'links',
	            'label' => 'gc_menu_links',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'field_extra' => 'options:link_options',
	            // 'tooltip' => lang('help_slider_extra'),
	            'class' => 9,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'custom',
	            'name' => 'params',
	            'label' => 'label_options',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'field_extra' => 'options:custom_fields',
	            'tooltip' => lang('help_slider_extra'),
	            'class' => 9,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'priority',
	            'label' => 'label_priority',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'value' => 0,
	            'class' => 2,
	            'rules' => 'trim|is_numeric|required',
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'status',
	            'label' => 'label_status',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        
	    );
	    return $fields;
	}

	//--------------------------------------------------------------------
	
	/**
	 * GET ALL BANNERS FOR INDEX PAGE
	 */
	public function get_all() {
		$this->table_name = 'banners'; //IMPORTANT OTHERWISE QUERY WILL RUN FOR ALL TABLES
		$this->db->select($this->db->dbprefix . 'banners.*, ' . $this->db->dbprefix . 'banner_types.type, ' . $this->db->dbprefix . 'banner_types.location');
		$this->db->join($this->db->dbprefix . 'banner_types', $this->db->dbprefix . 'banner_types.id = ' . $this->db->dbprefix . 'banners.type_id', 'inner');
		return $this->banners_model->find_all();
	}

	/**
	 * GET BANNER FOR EDIT OPERATION
	 * @param number $id for banner ID
	 */
	public function get_banner($id) {
		$this->table_name = 'banners'; //IMPORTANT OTHERWISE QUERY WILL RUN FOR ALL TABLES
		return $this->banners_model->find($id);
	}
	
	/**
	 * INSERT BANNER
	 * @param array $data
	 */
	public function insert_banner($data) {
		$this->table_name = 'banners'; //IMPORTANT OTHERWISE QUERY WILL RUN FOR ALL TABLES
		return $this->banners_model->insert($data);
	}
	
	/**
	 * UPDATE BANNER
	 * @param number $id
	 * @param array $data 
	 */
	public function update_banner($id, $data) {
		$this->table_name = 'banners'; //IMPORTANT OTHERWISE QUERY WILL RUN FOR ALL TABLES
		return $this->banners_model->update($id, $data);
	}
	
	/**
	 * DELETE BANNER
	 * @param number $id
	 */
	public function delete_banner($id) {
		$this->table_name = 'banners'; //IMPORTANT OTHERWISE QUERY WILL RUN FOR ALL TABLES
		return $this->banners_model->delete($id);
	} 
	
	/**
	 * IF BANNER TYPE IS DELETED, DELETE ALL RELATED BANNERS
	 * @param number $type_id
	 */
	public function delete_banner_types($type_id) {
		$this->table_name = 'banners'; //IMPORTANT OTHERWISE QUERY WILL RUN FOR ALL TABLES
		$where['type_id'] = $type_id;
		return $this->banners_model->delete_where($where);
	}
	
	/**
	 * GET BANNERS OF SPECIFIED TYPE ID FOR DELETION
	 * @param number $type_id
	 */
	public function get_banner_types($type_id) {
		$this->table_name = 'banners';
		$where['type_id'] = $type_id;
		return $this->banners_model->select('id')->find_all_by($where);
	}
	
	/**
	 * Return Banner Types
	 * @return array
	 */
	public function banner_types() {
	    return $this->db->select('id, title')->get('banner_types')->result();
	}
	
	/**
	 * Get Banners for pages
	 * @param string $group_id
	 * @return multitype:
	 */
	public function get_banners($group_id = 0) {
		$banners = array();
		$group_id = (int) $group_id;
		if(settings_item('site.enable_cache') == 1) {
		    $cacheID = 'banners_' . $group_id;
			$cacheID = str_replace('-', '_', $this->slug->create_slug($cacheID));
			if(!$banners = $this->z_cache->load($cacheID)) {
			    $banners = array('rows' => $this->_get_banners($group_id));
				$this->z_cache->save($banners, $cacheID);
			}
		} else {
		    $banners = array('rows' => $this->_get_banners($group_id));			
		}
		// Ads module changes
		/*$this->load->model('ads/ads_model');
		$result = $this->ads_model->ads('banners', $id, null);
		$page_banners = array_merge($result, $banners['rows']);
		*/
		return $this->ads_url($banners['rows']);
	}
	
	/**
	 * Get banner sliders - Home Page
	 * @param string $group_id
	 * @return multitype:
	 */
	public function get_banners_by_type($banner_type_id = 0) {
	    $banner_type_id = (int) $banner_type_id;
	    if(settings_item('site.enable_cache') == 1) {
	        $cacheID = 'banners_' . $banner_type_id;
	        $cacheID = str_replace('-', '_', $this->slug->create_slug($cacheID));
	        if(!$banners = $this->z_cache->load($cacheID)) {
	            $banners = array('rows' => $this->_get_banners(0, null, 0, null, $banner_type_id));
	            $this->z_cache->save($banners, $cacheID);
	        }
	    } else {
	        $banners = array('rows' => $this->_get_banners(0, null, 0, null, $banner_type_id));
	    }
	    return $this->ads_url($banners['rows']);
	}
	
	/**
	 * Get banner sliders - Home Page
	 * @param string $group_id
	 * @return multitype:
	 */
	public function get_sliders($group_id = 0) {
	    $group_id = (int) $group_id;
	    if(settings_item('site.enable_cache') == 1) {
	        $cacheID = 'banner_sliders_' . $group_id;
	        $cacheID = str_replace('-', '_', $this->slug->create_slug($cacheID));
	        if(!$banners = $this->z_cache->load($cacheID)) {
	            $banners = array('rows' => $this->_get_sliders($group_id));
	            $this->z_cache->save($banners, $cacheID);
	        }
	    } else {
	        $banners = array('rows' => $this->_get_sliders($group_id));
	    }
	}
	
	/**
	 * Get Sliders
	 * @param number $group_id
	 */
	private function _get_sliders($group_id) {
	    $where = "FIND_IN_SET($group_id, group_ids) AND status = 1";
	    return $this->db->where($where)->get('banner_sliders')->result();
	}
	
	/**
	 * Return banners
	 * @param string $group_id
	 * @return array banners
	 */
	private function _get_banners($group_id, $ids = null, $toggle_module_id = 0, $display_name = null, $banner_type_id = 0) {
		$sql = "SELECT B.*, BT.type, BT.width, BT.height";
		$sql .= " FROM " . $this->db->dbprefix . "banners B";
		$sql .= " INNER JOIN " . $this->db->dbprefix . "banner_types BT ON B.type_id = BT.id";
		$sql .= " WHERE B.status = 1 AND ((B.start_date <= '".date("Y-m-d") ."') || (B.start_date IS NULL)) AND ((B.end_date >= '".date("Y-m-d") ."') || (B.end_date IS NULL))";
		$sql .= " AND ( B.impressions <= B.max_impressions OR B.max_impressions = 0 ) AND ( B.clicks <= B.max_clicks OR B.max_clicks = 0 )";
		if(!empty($group_id)) {
		    $sql .= " AND FIND_IN_SET($group_id, B.group_ids)";
		}
		if(!empty($ids)) {
		    $sql .= " AND B.id IN (" . $ids . ")";
		}
		if(!empty($banner_type_id)) { // Sliders Home
		    $sql .= " AND B.type_id = " . $banner_type_id;
		}
		$sql .= " GROUP BY B.id ORDER BY RAND()"; // ORDER BY B.priority";
		$result = $this->db->query($sql)->result();		
		return $result;
	}
	
	/**
	 * Ad Module Changes
	 *
	 * Add Ad URL with each banner for clicks and impressions counter
	 *
	 * @param object $result
	 */
	private function ads_url($result) {
	    if($result) {
	        foreach($result as $key => $row) {
	            $ad_url = array(
	                'ad_id' => $row->ad_id,
	                'url' => $row->url,
	                'banner_id' => $row->id,
	                'keyword_id' => 0,
	                'keyword_module' => NULL
	            );
	            $row->ad_url = base64_encode(json_encode($ad_url));
	            $result[$key] = $row;
	        }
	        return $result;
	    }
	    return array();
	}
	
	/**
	 * Ads Module Banners
	 * @param string $ids - banner ids in comma separated format
	 * @param string $toggle_module - Listings/Config Modules etc.
	 * @param string $display_page - Home/View/Detail
	 */
	public function ads_banner($ids, $toggle_module = NULL, $display_page = 'all') {
	    $banners = array();
	    if(settings_item('site.enable_cache') == 1) {
	        $cacheID = 'ads_banner_' . $ids . '_' . $toggle_module . '_' . $display_page;
	        $cacheID = str_replace('-', '_', $this->slug->create_slug($cacheID));
	        if(!$banners = $this->z_cache->load($cacheID)) {
	            $banners = array('rows' => $this->_get_banners(0, $ids, $toggle_module, $display_page));
	            $this->z_cache->save($banners, $cacheID);
	        }
	    } else {
	        $banners = array('rows' => $this->_get_banners(0, $ids, $toggle_module, $display_page));
	    }
	    // Ads Module Changes
	    /* $this->load->model('ads/ads_model');
	     $result = $this->ads_model->ads('banners', 0, NULL, array('display_page' => $display_page));
	     $banners = array_merge((array)$result, $banners);*/
	    return $banners['rows'];
	}
	
	/**
	 * Check whether URL, Cities and Categories matches with the banner saved values or not
	 * @param string $current_url
	 * @param string $applicable_module
	 * @param object $banner
	 * @param integer $group_id - create group e.g. left/right/middle/bottom
	 * @return boolean
	 */
	public function is_banner_page_specific($current_url, $applicable_module, $banner, $group_id = 0) {
	    if($applicable_module == 'home') {
	        $module_id = 201; // manually specified module id
	    } else {
	        $module_id = gc_module_id($applicable_module);
	    }
	    $applicable_modules = !empty($banner->module_ids) ? json_decode($banner->module_ids, true) : '';
	    if(!empty($applicable_modules) && !array_key_exists($module_id, $applicable_modules)) {
	        return false;
	    }
	    
	    // display specific group banners e.g. left/right/middle/bottom
	    if(!empty($group_id)) {
	        $banner_group_ids = (array) explode(',', $banner->group_ids ?? '');
	        if(!in_array($group_id, $banner_group_ids)) {
	            return false;
	        }
	    }
	    
	    // do not display the same banner again on the same page
	    $banner_displayed = (array) $this->session->userdata('banner_displayed');
	    if(!empty($banner_displayed['ids']) && in_array($banner->id, $banner_displayed['ids'])) {
	        return false;
	    }
	    
	    // match links include first
	    $links_include = $this->str_matches($current_url, $banner->links_include);
	    
	    // check links exlude
	    $links_exclude = $this->str_matches($current_url, $banner->links_exclude);
	    if(!$links_include || ($links_include && !empty($banner->links_exclude) && $links_exclude)) {
	        return false;
	    }
	    
	    // match categories	    
	    // the array key must be a array
	    // set categories and cities as follows array(1,2,3,4); etc.
	    $banner_categories = $this->session->userdata('banner_categories');
	    $categories_include = $this->match_array($banner_categories, $banner->categories_include);
	    $categories_exclude = $this->match_array($banner_categories, $banner->categories_exclude);
	    // if(!$categories_include || ($categories_include && !empty($banner->categories_exclude) && $categories_exclude)) {
		if(!$categories_include || !$categories_exclude) {
	        return false;
	    }	    
	    
	    // match cities
	    // the array key must be a array
	    // set categories and cities as follows array(1,2,3,4); etc.
	    $banner_cities = $this->session->userdata('banner_cities');
	    $cities_include = $this->match_array($banner_cities, $banner->cities_include);
	    $cities_exclude = $this->match_array($banner_cities, $banner->cities_exclude);
	    // if($cities_include && !empty($banner->cities_exclude) && $cities_exclude) {
		if(!$cities_include || !$cities_exclude) {
	        return false;
	    }
	    
	    // match tags	    
	    // the array key must be a array
	    $banner_tags = $this->session->userdata('banner_tags');
	    $tags_include = $this->match_array($banner_tags, $banner->tags_include);
	    $tags_exclude = $this->match_array($banner_tags, $banner->tags_exclude);
	    if(!$tags_include || !$tags_exclude) {
	        return false;
	    }	  
		
	    // both categories and cities must match if both entered
	    if(($categories_include && !$cities_include) || (!$categories_include && $cities_include)) {
	        return false;
	    }
	    return true;
	}
	
	/**
	 * Set Display Banner Session Variable
	 * @param boolean $banner_displayed
	 */
	public function update_displayed_session($id, $banner_displayed, $current_url) {
	    if($banner_displayed) {
    	    // do not display the same banner again on the same page
    	    $banner_displayed_session = (array) $this->session->userdata('banner_displayed');
    	    if(empty($banner_displayed_session)) {
    	        $banner_displayed_session['url'] = $current_url;
    	        $banner_displayed_session['ids'] = array($id);
    	    } else {
    	        $banner_displayed_session['ids'][] = $id;
    	    }
    	    /*if(isset($banner_displayed_session['url']) && $banner_displayed_session['url'] != $current_url) {
    	        // page visited again, unset the session variable
    	        $this->session->unset_userdata('banner_displayed');
    	        $banner_displayed_session['url'] = $current_url;
    	        $banner_displayed_session['ids'] = array($banner->id);
    	    } else {
    	        $banner_displayed_session['ids'][] = $banner->id;
    	    }*/
	       $this->session->set_userdata('banner_displayed', $banner_displayed_session);
	    }
	}
	
	/**
	 * Links Include/Exclude match
	 * @param string $str - Current URL
	 * @param string $values - Saved Value
	 * @return boolean
	 */
	public function str_matches($str, $values) {
	    if(!empty($values)) {
    	    $values = array_map('trim', explode(',', $values));
    	    foreach($values as $value) {
    	        if(strpos($str, $value) !== false):
        	        return true;
    	        endif;
    	    }
    	    return false;
	    } else {
	       return true;
	    }
	}
	
	/**
	 * Match Cites/Categories with a session stored value
	 * @param array $match_with
	 * @param string $values
	 * @return boolean
	 */
	public function match_array($match_with, $values) {
	    if(!empty($values)) {
	        $values = array_map('trim', explode(',', $values));
	        $match_with = (array) $match_with;
	        if(!empty(array_intersect($match_with, $values))) {
	            return true;
	        }
	        return false;
	    } else {
	        return true;
	    }	    
	}
	
	/**
	 * Set banner specific session variables
	 * @param integer|string $ids
	 * @param string $module
	 */
	public function set_banner_session($ids, $module = 'categories') {
	    if(!empty($ids)) {	        
	        if(!is_array($ids)) {
	            if(strpos($ids, 'id') !== false) { // detail page	                
	                $ids = preg_replace("/id+/", "", $ids);
	                $ids = preg_replace("/c+/", "", $ids);
	                $ids = explode(', ', $ids);
	            } else {	                
	                $ids = array($ids);
	            }	            
	        }
	        $this->session->set_userdata('banner_' . $module, $ids);
	    } else {
	        $this->session->unset_userdata('banner_' . $module);
	    }
	}
	
	/**
	 * Method called in Gcrud library after banners insert/update
	 * @param number $id
	 * @param string $action_type
	 * @param string $record
	 */
	public function after_banner_save($id, $action_type = 'insert', $record = array()) {
	    $this->session->set_flashdata('selectize_result', null);
	}
}
