<?php
defined('BASEPATH') || exit('No direct script access allowed');
/*
 * ======================================================================= *\
 * || ##################################################################### ||
 * || # GoClixy - Business Directory Script                               # ||
 * || # Copyright (C) 2014 GoClixy. All Rights Reserved.                  # ||
 * || # ----------------------------------------------------------------  # ||
 * || # This file is owned and copyrighted by GoClixy.com. Your license   # ||
 * || # confers no title or ownership in the Script and is not a sale     # ||
 * || # of any rights to the Software. This file may not be redistributed # ||
 * || # in whole or significant part.                                     # ||
 * || # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
 * || # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
 * || ##################################################################### ||
 * \*=======================================================================
 */
/**
 * Bookings controller
 */
class Bookings extends Front_Controller {

    protected $permissionCreate = 'Bookings.Bookings.Create';
    protected $permissionDelete = 'Bookings.Bookings.Delete';
    protected $permissionEdit = 'Bookings.Bookings.Edit';
    protected $permissionView = 'Bookings.Bookings.View';

    protected $module_id = 1;
    protected $row_id = 1;

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
        
        $this->load->model('bookings/bookings_model');
        $this->load->model('links/links_model');
        $this->lang->load('bookings');
        Assets::add_module_js('bookings', 'bookings.js');
        $this->gc->demo_custom_css_js('member');
    }

    /**
     * Display a list of bookings data.
     *
     * @param number $type - 0 -> User, 1 - Member
     * @param number $offset
     * @return void
     */
    public function view($type = 0, $allotment_id = 0, $offset = 0) {
        $type = (int) $type;
        if($type > 1 || $type < 0) {
            Template::redirect(gc_url_format('members'));
        }
        $user_id = $this->auth->user_id();
        $this->bookings_model->view($user_id, $type, $allotment_id, $this->limit, $offset);
        Template::render('member');
    }

    /**
     * Allot a booking form to a user
     *
     * Display Refundable Policy dropdown and TAX settings to the user.
     *
     * @param number $form_id
     */
    public function settings($form_id = 0) {
        $this->bookings_model->settings($form_id);
        Template::render('member');
    }

    /**
     * Display All Booking Forms to the User
     *
     * @param integer $id
     *            - Update an Allotment
     */
    public function form($allotment_id = 0) {
        if ($allotment_id > 0) { // Check whether the supplied ID is correct or not
            $result = $this->db->get_where('booking_allotments', array(
                'id' => $allotment_id
            ))->row();
            if (! $result) {
                Template::redirect(gc_url_format('members'));
            }
        }
        $booking_forms = $this->db->get_where('booking_forms', array(
            'status' => 1
        ))->result();
        Template::set(array(
            'booking_forms' => $booking_forms
        ));
        Template::set_view('allotment_form');
        Template::render();
    }

    /**
     * Display all forms to the registered user for allotments
     * 
     * Module ID and Row ID information is in the session variable.
     */
    public function forms() {
        $booking_module_row = $this->session->userdata('booking_module_row');
        if(empty($booking_module_row)) {
            Template::redirect(gc_url_format('members'));
        }
        Assets::add_js(array(
            'jquery.dataTables.min',
            'dataTables.bootstrap.min'
        ));
        Assets::add_css('dataTables.bootstrap.min');
        Assets::add_js("$('#booking_forms').DataTable({\"order\": [[ 3, 'asc' ]]});", 'inline');
        /*$records = $this->db->get_where('booking_forms', array(
            'status' => 1
        ))->result();*/
        // if a form status is inactive, but the form is already configured by a user, display it to the user
        $sql = "SELECT * FROM " . $this->db->dbprefix . "booking_forms BF";
        $sql .= " WHERE BF.status = 1";
        $sql .= " OR BF.id = (SELECT form_id FROM " . $this->db->dbprefix . "booking_allotments BA";
        $sql .= " WHERE BA.module_id = " . $booking_module_row['module_id'] . " AND BA.row_id = " . $booking_module_row['row_id'];
        $sql .= " AND BA.user_id = " . $this->auth->user_id() . " LIMIT 1)";
        $records = $this->db->query($sql)->result();
        Template::set(array(
            'status' => $booking_module_row['status'],
            'bookings' => $booking_module_row['bookings'],
            'allotments' => $booking_module_row['allotments'],
            'records' => $records,
            'toolbar_title' => $booking_module_row['title'] . ' - ' . lang('available_forms')
        ));
        Template::render('member');
    }

    /**
     * Display Form Fields and Pricing packages to the User
     *
     * @param number $allotment_id
     */
    public function fields($allotment_id = 0) {        
        $this->bookings_model->fields($allotment_id);
        Template::render('member');
    }

    /**
     * Allow user to add options
     *
     * A single table is used to store options, dynamic field relations and pricing.
     * the field type_id is used to differentiate them.
     *
     * @param number $form_id
     * @param number $field_id
     * @param number $type_id
     *            = 0 -> Options, 1 -> Dynamic Fields, 2 - Pricing/Packages
     */
    public function options($allotment_id = 0, $field_id = 0, $type_id = 0) {        
        $this->bookings_model->options($allotment_id, $field_id, $type_id);
        Template::render('member');
    }

    /**
     * Allow user to add/edit pricing form
     *
     * @param number $id
     *            = 0 -> Insert, Value -> Edit
     */
    public function packages($allotment_id, $id = 0) {
        $this->bookings_model->packages($allotment_id, $id, 'member');
        Template::render('member');
    }

    /**
     * Display booking form to the Frontend User
     * 
     * @param number $allotment_id
     * @param number $id - ID of the Booking for edit purposes
     */
    public function booking_form($allotment_id, $id = 0) {
        $this->bookings_model->booking_form($allotment_id, $id, 'member');
    }
    
    /**
     * Populate pricing packages dropdown fields
     */
    public function populate_field() {
        $this->bookings_model->populate_field();
    }
    
    /**
     * Allow Member to Export Data
     * @param number $allotment_id
     */
    public function export($allotment_id) {
        $this->bookings_model->export_data($allotment_id, 0, 'member', null);
    }
    
    /**
     * Booking Cancellations
     * @param integer $booking_id
     */
    public function cancellation($booking_id) {
        $this->load->model('orders/cancellations_model');
        $this->cancellations_model->form('bookings', $booking_id, 0, 'member');
        Template::set('cancel_url', gc_url_format('bookings/view'));
        Template::render('member');
    }
}