<?php defined('BASEPATH') || exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Financial controller
 */
class Financial extends Admin_Controller
{
    protected $permissionCreate = 'Bookings.Financial.Create';
    protected $permissionDelete = 'Bookings.Financial.Delete';
    protected $permissionEdit   = 'Bookings.Financial.Edit';
    protected $permissionView   = 'Bookings.Financial.View';

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
        
        $this->auth->restrict($this->permissionView);
        $this->load->model('bookings/bookings_model');
        $this->lang->load('bookings');
        
      	$this->form_validation->set_error_delimiters("<span class='error'>", "</span>");
        
        Template::set_block('sub_nav', 'financial/_sub_nav');
    }

    /**
     * Display a list of bookings data.
     *
     * @return void
     */
    public function index($offset = 0) {
        // Deleting anything?
        if (isset($_POST['delete'])) {
            $this->gcrud->stats(array('module' => 'bookings'));
            $this->auth->restrict($this->permissionDelete);
            $checked = $this->input->post('checked');
            if (is_array($checked) && count($checked)) {

                // If any of the deletions fail, set the result to false, so
                // failure message is set if any of the attempts fail, not just
                // the last attempt

                $result = true;
                foreach ($checked as $pid) {
                    $deleted = $this->bookings_model->purge_booking($pid);
                    if ($deleted == false) {
                        $result = false;
                    }
                }
                if ($result) {
                    Template::set_message(count($checked) . ' ' . sprintf(lang('msg_action_success'), lang('label_bookings'), lang('label_deleted')), 'success');
                } else {
                    Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_bookings')) . $this->bookings_model->error, 'error');
                }
            }
        }
        // Status Change
        if (isset($_POST['status_change'])) {
            $this->gcrud->stats(array('module' => 'bookings'));
            $this->gc->status_change('bookings', $_POST['status_change'], 'status');
        }
        $this->bookings_model->view(0, 1, 0, $this->limit, $offset, 'admin');
        Template::render();
    }
    
    /**
     * Allow admin to edit bookings
     *
     * @param number $allotment_id
     * @param number $id - ID of the Booking for edit purposes
     */
    public function booking_form($allotment_id, $id = 0) {
        Assets::add_module_js('bookings', 'bookings.js');
        $this->bookings_model->booking_form($allotment_id, $id, 'admin');
    }
    
    /**
     * Create a bookings object.
     *
     * @return void
     */
    public function create() {
        $this->auth->restrict($this->permissionCreate);
        
        if (isset($_POST['save'])) {
            $this->gcrud->stats(array('module' => 'bookings'));
            if ($insert_id = $this->save_bookings()) {
                log_activity($this->auth->user_id(), lang('act_create_record') . ': ' . $insert_id . ' : ' . $this->input->ip_address(), 'bookings');
                Template::set_message(sprintf(lang('msg_save_success'), lang('label_bookings')), 'success');

                redirect(SITE_AREA . '/financial/bookings');
            }

            // Not validation error
            if ( ! empty($this->bookings_model->error)) {
                Template::set_message(sprintf(lang('msg_create_failure'), lang('label_bookings')) . $this->bookings_model->error, 'error');
            }
        }

        Template::set('toolbar_title', lang('create_bookings'));
        Template::set_view('financial/form');
        Template::render();
    }
    /**
     * Allows editing of bookings data.
     *
     * @return void
     */
    public function edit()
    {
        $id = $this->uri->segment(5);
        if (empty($id)) {
            Template::set_message(lang('invalid_id'), 'error');
            redirect(SITE_AREA . '/financial/bookings');
        }
        
        if (isset($_POST['save'])) {
            $this->gcrud->stats(array('module' => 'bookings'));
            $this->auth->restrict($this->permissionEdit);

            if ($this->save_bookings('update', $id)) {
                log_activity($this->auth->user_id(), lang('act_edit_record') . ': ' . $id . ' : ' . $this->input->ip_address(), 'bookings');
                Template::set_message(sprintf(lang('msg_save_success'), lang('label_bookings')), 'success');
                redirect(SITE_AREA . '/financial/bookings');
            }

            // Not validation error
            if ( ! empty($this->bookings_model->error)) {
                Template::set_message(sprintf(lang('msg_edit_failure'), lang('label_bookings')) . $this->bookings_model->error, 'error');
            }
        } elseif (isset($_POST['delete'])) {
            $this->gcrud->stats(array('module' => 'bookings'));
            $this->auth->restrict($this->permissionDelete);

            if ($this->bookings_model->delete($id)) {
                log_activity($this->auth->user_id(), lang('act_delete_record') . ': ' . $id . ' : ' . $this->input->ip_address(), 'bookings');
                Template::set_message(sprintf(lang('msg_delete_success'), lang('label_bookings')), 'success');
                redirect(SITE_AREA . '/financial/bookings');
            }
            Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_bookings')) . $this->bookings_model->error, 'error');
        }
        
        Template::set('record', $this->bookings_model->find($id));

        Template::set('toolbar_title', lang('edit_bookings'));        
        Template::set_view('financial/form');
        Template::render();
    }

    //--------------------------------------------------------------------------
    // !PRIVATE METHODS
    //--------------------------------------------------------------------------

    /**
     * Save the data.
     *
     * @param string $type Either 'insert' or 'update'.
     * @param int    $id   The ID of the record to update, ignored on inserts.
     *
     * @return boolean|integer An ID for successful inserts, true for successful
     * updates, else false.
     */
    private function save_bookings($type = 'insert', $id = 0) {
        if ($type == 'update') {
            $_POST['id'] = $id;
        }

        // Validate the data
        $this->form_validation->set_rules($this->bookings_model->get_validation_rules());
        if ($this->form_validation->run() === false) {
            return false;
        }

        // Make sure we only pass in the fields we want
        
        $data = $this->bookings_model->prep_data($this->input->post());

        // Additional handling for default values should be added below,
        // or in the model's prep_data() method
        

        $return = false;
        if ($type == 'insert') {
            $id = $this->bookings_model->insert($data);

            if (is_numeric($id)) {
                $return = $id;
            }
        } elseif ($type == 'update') {
            $return = $this->bookings_model->update($id, $data);
        }

        return $return;
    }
    
    /**
     * Display a list of forms data.
     *
     * @return void
     */
    public function forms($offset = 0) {
        $this->session->set_userdata('bookings_last_visited_page', current_url());
    	$table_name = 'booking_forms';
    	// Do we have any actions?
    	if (isset($_POST['submit'])) {
    	    $this->gcrud->stats(array('module' => 'bookings'));
    		$action = $this->input->post('submit');
    		$checked = $this->input->post('checked');
    		if($checked) {
    			$module_name = count($checked) == 1 ? lang('label_form') : lang('label_forms');
    			switch(strtolower($action)) {
    				case 'activate':
    					$this->gc->update_status($table_name, $checked, 'status', 1);
    					Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_activated')), 'success');
    					break;
    						
    				case 'deactivate':
    					$this->gc->update_status($table_name, $checked, 'status', 0);
    					Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_deactivated')), 'success');
    					break;
    						
    				case 'delete':
    					foreach($checked as $pid) {
    						$this->bookings_model->purge_form($pid);
    					}
    					Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_deleted')), 'success');
    					break;
    			}
    		}
    	}
    	
    	// Allow admin to assign fields
    	$this->session->unset_userdata('fields_cancel_url');
    	if (isset($_POST['assign_field'])) {
    	    $this->gcrud->stats(array('module' => 'bookings'));
    	    clear_cache('booking_fields_' . $_POST['assign_field']); // Clear Booking Fields Cache
    	    clear_cache('booking_has_availability_screen_' . $_POST['assign_field']); // Clear Availability Screen Cache    	    
    	    clear_cache('bookings_' . $_POST['assign_field']); // Clear specific cache
    	    clear_cache('booking_packages_' . $_POST['assign_field']); // Clear allotment specific packages cache
    	    
    		$this->session->set_userdata('fields_cancel_url', current_url());
    		// Template::redirect(site_url(SITE_AREA . '/developer/fields/assign/' . $_POST['assign_field'] . '/bookings'));
    		Template::redirect(site_url(SITE_AREA . '/financial/bookings/assign_fields/' . $_POST['assign_field']));
    	}
    	
    	// Status Change
    	if (isset($_POST['status_change'])) {
    	    $this->gcrud->stats(array('module' => 'bookings'));
    		$this->gc->status_change('booking_forms', $_POST['status_change'], 'status');
    	}
    	
    	// Copy form
    	if(isset($_POST['clone'])) {
    	    $this->gcrud->stats(array('module' => 'bookings'));
    	    $clone_id = $_POST ['clone'];
    	    $el = $this->db->get_where('booking_forms', array('id' => $clone_id))->row();
    	    if($el) {
    	        $clone_data = array();
    	        foreach($el as $key => $value) {
    	            switch($key) {
    	                case 'id':
    	                case 'created_by':
    	                case 'modified_by':
    	                case 'modified_on':
    	                case 'modified_ip':
    	                    break;
    	                case 'created_on':
    	                    $clone_data[$key] = date('Y-m-d H:i:s');
    	                    break;
    	                case 'created_ip':
    	                    $clone_data[$key] = $this->input->ip_address();
    	                    break;
    	                case 'title':
    	                    $clone_data[$key] = $value .' '.lang('label_copy');
    	                    break;
    	                default:
    	                    $clone_data[$key] = $value;
    	            }
    	        }
    	        $this->db->insert('booking_forms', $clone_data);
    	        $inserted_id = $this->db->insert_id();
    	        if (is_numeric ( $inserted_id ) && $inserted_id > 0) {
    	            // also clone assigned fields
    	            $assigned_fields = $this->db->get_where('field_modules', array('module_id' => gc_module_id('bookings'), 'row_id' => $clone_id))->result();
    	            if($assigned_fields) {
    	                $assigned_data = array();
    	                foreach($assigned_fields as $assigned_field) {
    	                    $assigned_data[] = array(
    	                        'field_id' => $assigned_field->field_id,
    	                        'module_id' => $assigned_field->module_id,
    	                        'row_id' => $inserted_id,
    	                        'priority' => $assigned_field->priority
    	                    );
    	                }
    	                $this->db->insert_batch('field_modules', $assigned_data);
    	            }
    	            Template::set_message(lang('msg_success'), 'success');
    	            Template::redirect(current_url());
    	        } else {
    	            Template::set_message(lang('msg_clone_error'), 'error');
    	        }
    	    }
    	}
    
    	$like_array = array();
    	$searchterm = $this->gc->search('booking_forms', $this->uri->segment(5), '/financial/bookings/forms');
    	if(!empty($searchterm)) {
    		// This is the basic search filter, you could add more fields to the search
    		$like_array = array(
    			'id' => $searchterm,
    			'title' => $searchterm,
    			'commission' => $searchterm
    		);
    	}
    	$pagerUriSegment = 5;
    	$pagerBaseUrl = site_url(SITE_AREA . '/financial/bookings/forms') . '/';
    
    	$limit  = $this->settings_lib->item('site.list_limit') ?: 15;
    	if(!empty($like_array)) {
    		$this->db->or_like($like_array);
    	}
    	$total_rows = $this->db->count_all_results('booking_forms');
    	$this->load->library('pagination');
    	$pager['base_url']    = $pagerBaseUrl;
    	$pager['total_rows']  = $total_rows;
    	$pager['per_page']    = $limit;
    	$pager['uri_segment'] = $pagerUriSegment;
    
    	$this->pagination->initialize($pager);
    	$this->bookings_model->limit($limit, $offset);
    	$order_by = $this->gc->sort('booking_forms');
    	if(!empty($order_by)) {
    		$this->db->order_by($this->session->userdata('booking_forms_sort')['column'], $this->session->userdata('booking_forms_sort')['order']);
    	} else {
    		$this->db->order_by('id', 'DESC');
    	}
    
    	if(!empty($like_array)) {
    		$this->db->or_like($like_array);
    	}
    	$records = $this->db->get('booking_forms')->result();    
    	Template::set('records', $records);    
    	Template::set('toolbar_title', lang('manage_forms'));    
    	Template::render();
    }
    
    /**
     * Create a Booking Form
     *
     * @return void
     */
    public function add_form() {
    	$this->auth->restrict($this->permissionCreate);
    
    	if (isset($_POST['save'])) {
    	    $this->gcrud->stats(array('module' => 'bookings'));
    		if ($insert_id = $this->save_form()) {
    		    // insert default template text
    		    $this->bookings_model->template_default($insert_id);
    			log_activity($this->auth->user_id(), lang('act_create_record') . ': ' . $insert_id . ' : ' . $this->input->ip_address(), 'bookings');
    			Template::set_message(sprintf(lang('msg_save_success'), lang('label_form')), 'success');
    
    			redirect(SITE_AREA . '/financial/bookings/forms');
    		}
    
    		// Not validation error
    		if ( ! empty($this->bookings_model->error)) {
    			Template::set_message(sprintf(lang('msg_create_failure'), lang('label_form')) . $this->bookings_model->error, 'error');
    		}
    	}
    
    	Template::set('toolbar_title', lang('add_form'));
    	Template::set_view('financial/form');
    	Template::render();
    }
    
    /**
     * Allows editing of Booking Form
     *
     * @return void
     */
    public function edit_form() {
    	$id = $this->uri->segment(5);
    	if (empty($id)) {
    		Template::set_message(lang('invalid_id'), 'error');
    		redirect(SITE_AREA . '/financial/bookings/forms');
    	}
    
    	if (isset($_POST['save'])) {
    	    $this->gcrud->stats(array('module' => 'bookings'));
    		$this->auth->restrict($this->permissionEdit);
    
    		if ($this->save_form('update', $id)) {
    			log_activity($this->auth->user_id(), lang('act_edit_record') . ': ' . $id . ' : ' . $this->input->ip_address(), 'booking_forms');
    			Template::set_message(sprintf(lang('msg_save_success'), lang('label_form')), 'success');
    			redirect(SITE_AREA . '/financial/bookings/forms');
    		}
    
    		// Not validation error
    		if ( ! empty($this->bookings_model->error)) {
    			Template::set_message(sprintf(lang('msg_edit_failure'), lang('label_form')) . $this->bookings_model->error, 'error');
    		}
    	} elseif (isset($_POST['delete'])) {
    	    $this->gcrud->stats(array('module' => 'bookings'));
    		$this->auth->restrict($this->permissionDelete);    
    		if ($this->bookings_model->purge_form($id)) {
    			log_activity($this->auth->user_id(), lang('act_delete_record') . ': ' . $id . ' : ' . $this->input->ip_address(), 'booking_forms');
    			Template::set_message(sprintf(lang('msg_delete_success'), lang('label_form')), 'success');
    			redirect(SITE_AREA . '/financial/bookings/forms');
    		}
    		Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_form')) . $this->bookings_model->error, 'error');
    	}
    	$record = $this->db->get_where('booking_forms', array('id' => $id))->row();
    	Template::set('record', $record);    
    	Template::set('toolbar_title', lang('edit_form'));
    	Template::set_view('financial/form');
    	Template::render();
    }
    
    //--------------------------------------------------------------------------
    // !PRIVATE METHODS
    //--------------------------------------------------------------------------
    
    /**
     * Save the data.
     *
     * @param string $type Either 'insert' or 'update'.
     * @param int    $id   The ID of the record to update, ignored on inserts.
     *
     * @return boolean|integer An ID for successful inserts, true for successful
     * updates, else false.
     */
    private function save_form($type = 'insert', $id = 0) {
    	if ($type == 'update') {
    		$_POST['id'] = $id;
    	}
    
    	// Validate the data
    	$this->form_validation->set_rules($this->bookings_model->form_validation_rules);
    	if ($this->form_validation->run() === false) {
    		return false;
    	}
    
    	// Make sure we only pass in the fields we want    
    	$data = array();
    	$data['title'] = $this->input->post('title');
    	$data['commission'] = $this->input->post('commission');
    	$data['max_packages'] = $this->input->post('max_packages');
    	$data['enable_cache'] = $this->input->post('enable_cache') ? 1 : 0;
    	// Note: Rules are hardcoded
    	// 1 - Days (1 is assigned to days)
    	$data['rules'] = json_encode($this->input->post('rules'));
    	$data['status'] = $this->input->post('status') ? 1 : 0;
    	
    	$data['form_fields'] = json_encode($this->input->post('form_fields'));
    
    	// Additional handling for default values should be added below,
    	// or in the model's prep_data() method
    	    
    	$return = false;
    	if ($type == 'insert') {
    		$data['created_on'] = date('Y-m-d H:i:s');
    		$data['created_by'] = $this->auth->user_id();
    		$data['created_ip'] = $this->input->ip_address();
    		$this->db->insert('booking_forms', $data); 
    		$id = $this->db->insert_id();
    		if (is_numeric($id)) {
    			$return = $id;
    		}
    	} elseif ($type == 'update') {
    		$data['modified_on'] = date('Y-m-d H:i:s');
    		$data['modified_by'] = $this->auth->user_id();
    		$data['modified_ip'] = $this->input->ip_address();
    		$return = $this->db->where('id', $id)->update('booking_forms', $data);
    	}
    	return $return;
    }
    
    /**
     * Display a list of Form Allotments to the User.
     *
     * @return void
     */
    public function allotments($module = 'all', $offset = 0) {
        $this->session->set_userdata('last_visited_url', current_url());
    	$table_name = 'booking_allotments';
    	// Do we have any actions?
		if (isset($_POST['submit'])) {
		    $this->gcrud->stats(array('module' => 'bookings'));
			$action = $this->input->post('submit');
			$checked = $this->input->post('checked');
			if($checked) {
				$module_name = count($checked) == 1 ? lang('label_allotment') : lang('label_allotments');
				switch(strtolower($action)) {
					case 'activate':
						$this->gc->update_status($table_name, $checked, 'status', 1);
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_activated')), 'success');
						break;
							
					case 'deactivate':
						$this->gc->update_status($table_name, $checked, 'status', 0);
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_deactivated')), 'success');
						break;
							
					case 'delete':
						foreach($checked as $pid) {
							$this->bookings_model->purge_allotment($pid);
						}
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_deleted')), 'success');
						break;
				}
			}
		}
		
		// Status Change
		if (isset($_POST['status_change'])) {
		    $this->gcrud->stats(array('module' => 'bookings'));
			$this->gc->status_change('booking_allotments', $_POST['status_change'], 'status');
		}
		
		// Actions done, now display the view
		$where = "";
		
    	// Search
    	$searchterm = $this->gc->search($table_name, $this->uri->segment(6), '/financial/bookings/allotments');
    	if(!empty($searchterm)) {
    		$where .= " (T.id LIKE '%". $searchterm ."%' OR T.user_id LIKE '%". $searchterm ."%' OR T.row_id LIKE '%". $searchterm ."%' OR T.created_on LIKE '%". $searchterm ."%'";
    		/*if($module != 'all') {
    			$where .= " OR M.title LIKE '%". $searchterm ."%'";
    		}*/
    		$where .= ")";
    	}
    	
    	// Order By
    	$sort_by = "T.id desc";
    	if(isset($_POST['sort']) && $_POST['sort'] == 'display_name') {
    		$order_by = $this->gc->sort($table_name, 'U');
    	} else {
    		$order_by = $this->gc->sort($table_name, 'T');
    	}
    	if(!empty($order_by)) {
    		$sort_by = $order_by;
    	}
    	
    	$pagerUriSegment = 6;
    	$pagerBaseUrl = site_url(SITE_AREA . '/financial/bookings/allotments') . '/' . $module;  	
    
    	$limit  = $this->settings_lib->item('site.list_limit') ?: 15;
    	$select = array("U.display_name", "U.email", "BF.title AS form");
    	$join = array(
    		"LEFT JOIN ". $this->db->dbprefix ."users U ON U.id = T.user_id",
    		"LEFT JOIN ". $this->db->dbprefix ."booking_forms BF ON BF.id = T.form_id"
    	);    	
    	if($module == 'all') {
    		$records = $this->gc->get_all_row(0, $table_name, $where, $sort_by, $offset, $this->limit, NULL, array(), $select, $join);
    	} else {
    		$records = $this->gc->get_module_rows(0, $table_name, $module, $where, $sort_by, $offset, $this->limit, NULL, array(), $select, $join);
    	}
    	
    	if($module == 'all') {
    		$total_rows = $this->gc->get_all_row(1, $table_name, $where, $sort_by, $offset, $this->limit);
    	} else {
    		$total_rows = $this->gc->get_module_rows(1, $table_name, $module, $where, $sort_by, $offset, $this->limit, NULL, array(), $select, $join);
    	}
    	$this->load->library('pagination');
    	$pager['base_url']    = $pagerBaseUrl;
    	$pager['total_rows']  = $total_rows;
    	$pager['per_page']    = $limit;
    	$pager['uri_segment'] = $pagerUriSegment;
    
    	$this->pagination->initialize($pager);
    	$this->bookings_model->limit($limit, $offset);
    	Template::set('records', $records);
    	Template::set('toolbar_title', lang('manage_allotments'));
    	Template::render();
    }
    
    /**
     * Add an Allotment
     *
     * @return void
     */
    public function add_allotment() {
    	$this->auth->restrict($this->permissionCreate);

    	// user search js and css dependencies
    	Assets::add_css('selectize.bootstrap3.css');
    	Assets::add_js('selectize.min.js');
    	Assets::add_js('selectize-search.js');
    	
    	if (isset($_POST['save'])) {
    	    $this->gcrud->stats(array('module' => 'bookings'));
    		if ($insert_id = $this->save_allotment()) {
    			log_activity($this->auth->user_id(), lang('act_create_record') . ': ' . $insert_id . ' : ' . $this->input->ip_address(), 'allotments');
    			Template::set_message(sprintf(lang('msg_save_success'), lang('label_allotment')), 'success');
    
    			Template::redirect(SITE_AREA . '/financial/bookings/allotments');
    		}
    
    		// Not validation error
    		if ( ! empty($this->bookings_model->error)) {
    			Template::set_message(sprintf(lang('msg_create_failure'), lang('label_allotment')) . $this->bookings_model->error, 'error');
    		}
    	}
    	$this->booking_forms();
    	Template::set('toolbar_title', lang('add_allotment'));
    	Template::set_view('financial/allotment_form');
    	Template::render();
    }
    
    /**
     * Allows editing of Allotment Form
     *
     * @return void
     */
    public function edit_allotment_old() {
    	$id = $this->uri->segment(5);
    	if (empty($id)) {
    		Template::set_message(lang('invalid_id'), 'error');
    		redirect(SITE_AREA . '/financial/bookings/allotments');
    	}
    	
    	// user search js and css dependencies
    	Assets::add_css('selectize.bootstrap3.css');
    	Assets::add_js('selectize.min.js');
    	Assets::add_js('selectize-search.js');
    	
    	if (isset($_POST['save'])) {
    		$this->auth->restrict($this->permissionEdit);
    
    		if ($this->save_allotment('update', $id)) {
    			log_activity($this->auth->user_id(), lang('act_edit_record') . ': ' . $id . ' : ' . $this->input->ip_address(), 'allotments');
    			Template::set_message(sprintf(lang('msg_save_success'), lang('label_allotment')), 'success');
    			redirect(SITE_AREA . '/financial/bookings/allotments');
    		}
    
    		// Not validation error
    		if ( ! empty($this->bookings_model->error)) {
    			Template::set_message(sprintf(lang('msg_edit_failure'), lang('label_allotment')) . $this->bookings_model->error, 'error');
    		}
    	} elseif (isset($_POST['delete'])) {
    		$this->auth->restrict($this->permissionDelete);
    		if ($this->bookings_model->purge_allotment($id)) {
    			log_activity($this->auth->user_id(), lang('act_delete_record') . ': ' . $id . ' : ' . $this->input->ip_address(), 'allotments');
    			Template::set_message(sprintf(lang('msg_delete_success'), lang('label_allotment')), 'success');
    			redirect(SITE_AREA . '/financial/bookings/allotments');
    		}
    		Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_allotment')) . $this->bookings_model->error, 'error');
    	}
    	$this->booking_forms();
    	$record = $this->db->get_where('booking_allotments', array('id' => $id))->row();
    	Template::set('record', $record);
    	Template::set('toolbar_title', lang('edit_allotment'));
    	Template::set_view('financial/allotment_form');
    	Template::render();
    }
    
    /**
     * Show Booking Forms to the User
     */
    private function booking_forms() {
    	$booking_forms = $this->db->select('id, title')->get('booking_forms')->result();
    	Template::set('booking_forms', $booking_forms);
    }
    
    //--------------------------------------------------------------------------
    // !PRIVATE METHODS
    //--------------------------------------------------------------------------
    
    /**
     * Save the data.
     *
     * @param string $type Either 'insert' or 'update'.
     * @param int    $id   The ID of the record to update, ignored on inserts.
     *
     * @return boolean|integer An ID for successful inserts, true for successful
     * updates, else false.
     */
    private function save_allotment($type = 'insert', $id = 0) {
    	if ($type == 'update') {
    		$_POST['id'] = $id;
    	}
    
    	// Validate the data
    	$this->form_validation->set_rules('form_id', 'lang:label_form', 'trim|required|is_numeric');
    	// $this->form_validation->set_rules('user_id', 'lang:label_user', 'trim|required|is_numeric');
    	$this->form_validation->set_rules('module_id', 'lang:label_module', 'trim|required|is_numeric');
    	$this->form_validation->set_rules('row_id', 'lang:label_row_id', 'trim|required|is_numeric|callback_valid_row_id');
    	$this->form_validation->set_rules('status', 'lang:label_status', 'trim');
    	if ($this->form_validation->run() === false) {
    		return false;
    	}
    
    	// Make sure we only pass in the fields we want
    	$data = array();
    	$data['form_id'] = $this->input->post('form_id');
    	// $data['user_id'] = $this->input->post('user_id');
    	$data['module_id'] = $this->input->post('module_id');
    	$data['row_id'] = $this->input->post('row_id');
    	$data['status'] = $this->input->post('status') ? 1 : 0;
    
    	// Additional handling for default values should be added below,
    	// or in the model's prep_data() method
    		
    	$return = false;
    	if ($type == 'insert') {
    	    // get user_id
    	    $module_info = $this->db->where('id', $data['row_id'])->get(gc_module_name($data['module_id']))->row();
    	    if(!$module_info) {
    	        return false;
    	    } else {
    	        $data['user_id'] = $module_info->user_id;
    	    }
    	    $data['created_on'] = date('Y-m-d H:i:s');
    		$data['created_ip'] = $this->input->ip_address();
    		$this->db->insert('booking_allotments', $data);
    		$id = $this->db->insert_id();
    		if (is_numeric($id)) {
    			$return = $id;
    		}
    	} elseif ($type == 'update') {
    		$data['modified_on'] = date('Y-m-d H:i:s');
    		$data['modified_ip'] = $this->input->ip_address();
    		$return = $this->db->where('id', $id)->update('booking_allotments', $data);
    	}
    
    	return $return;
    }
    
    /**
     * Allotment Form - Check Row ID validity
     * @param integer $id
     * @return boolean
     */
    public function valid_row_id($id) {
    	$module_id = $this->input->post('module_id');
    	$id = (int) $id;
    	if($module_id > 0 && $id > 0) {
    		$module = gc_module_name($module_id);
    		$count = $this->db->where('id', $id)->count_all_results($module);
    		if($count > 0) {
    			return true;
    		}
    	}
    	// set error message
    	$this->form_validation->set_message('valid_row_id', lang('msg_not_found'));
    	 
    	// return fail
    	return FALSE;
    }
    
    /**
     * Display a list of Packages defined by a User.
     * @param integer $offset
     * @param integer $allotment_id - To view a specific allotment packages
     * 
     * @return void
     */
    public function packages_old($offset = 0, $allotment_id = 0) {
    	// Do we have any actions?
    	if ($action = $this->input->post('submit')) {
    		$checked = $this->input->post('checked');
    		$module_name = count($checked) == 1 ? lang('label_package') : lang('gc_menu_packages') ;
    		if (is_array($checked) && count($checked)) {
    			switch(strtolower($action)) {
    				case 'delete':
    					$result = FALSE;
    					foreach ($checked as $pid) {
    						$deleted = $this->db->where('id', $id)->delete('booking_packages');
    					}
    					if ($result) {
    						Template::set_message(count($checked) .' '. sprintf(lang('msg_delete_success'), $module_name), 'success');
    					} else {
    						Template::set_message(lang('delete_failure') . $this->bookings_model->error, 'error');
    					}
    					break;
    				case 'save':
    					$result = FALSE;
    					foreach ($checked as $pid) {
    						$result = $this->save_package_index($pid);
    					}
    					if ($result) {
    						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_updated')), 'success');
    					} else {
    						Template::set_message(sprintf(lang('msg_edit_failure'), $module_name), 'error');
    					}
    					break;
    				case 'activate':
    					$this->gc->update_status('booking_packages', $checked, 'status', 1);
    					Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_activated')), 'success');
    					break;
    							
    				case 'deactivate':
    					$this->gc->update_status('booking_packages', $checked, 'status', 0);
    					Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_deactivated')), 'success');
    					break;
    			}
    		}
    	}
    	
    	// Status Change
    	if (isset($_POST['status_change'])) {
    		$this->gc->status_change('booking_packages', $_POST['status_change'], 'status');
    	}
    	
    	// Update Pricing Information
    	if(!empty($_POST['save'])) {
    		$field_id = $_POST['save'];
    		$this->save_package_index($field_id);
    		Template::set_message(sprintf(lang('msg_action_success'), lang('label_package'), lang('label_updated')), 'success');
    		Template::redirect(current_url());
    	}
    
    	$like_array = array();
    	$searchterm = $this->gc->search('booking_packages', $this->uri->segment(5), '/financial/bookings/packages');
    	if(!empty($searchterm)) {
    		// This is the basic search filter, you could add more fields to the search
    		$like_array = array(
    			'booking_packages.id' => $searchterm,
    			'booking_packages.title' => $searchterm,
    			'booking_packages.price' => $searchterm
    		);
    	}
    	$pagerUriSegment = 5;
    	$pagerBaseUrl = site_url(SITE_AREA . '/financial/bookings/packages') . '/';
    
    	$limit  = $this->settings_lib->item('site.list_limit') ?: 15;
    	if(!empty($like_array)) {
    		$this->db->or_like($like_array);
    	}
    	if($allotment_id > 0) {
    		$this->db->where('allotment_id', $allotment_id);
    	}
    	$total_rows = $this->db->count_all_results('booking_packages');
    	$this->load->library('pagination');
    	$pager['base_url']    = $pagerBaseUrl;
    	$pager['total_rows']  = $total_rows;
    	$pager['per_page']    = $limit;
    	$pager['uri_segment'] = $pagerUriSegment;
    
    	$this->pagination->initialize($pager);
    	$this->bookings_model->limit($limit, $offset);
    
    	$order_by = $this->gc->sort('booking_packages');
    	if(!empty($order_by)) {
    		$this->db->order_by($this->session->userdata('booking_packages_sort')['column'], $this->session->userdata('booking_packages_sort')['order']);
    	} else {
    		$this->db->order_by('id', 'DESC');
    	}
    
    	if(!empty($like_array)) {
    		$this->db->or_like($like_array);
    	}
    	if($allotment_id > 0) {
    		$this->db->where('allotment_id', $allotment_id);
    	}
    	$this->db->join('booking_allotments BA', 'BA.id = booking_packages.allotment_id', 'LEFT');
    	$this->db->join('booking_forms BF', 'BF.id = BA.form_id', 'LEFT');
    	$records = $this->db->select('booking_packages.*, BF.title AS form')->get('booking_packages')->result();
    	Template::set('records', $records);
    	Template::set('toolbar_title', lang('manage_packages'));
    	Template::render();
    }
    
    /**
     * Allow admin to have a separate email template for each form
     * 
     * if email_templates fields are empty, use the default format.
     * 
     * @param number $form_id
     */
    public function template($form_id) {
        Assets::add_js(array(
            'tinymce/tinymce.min.js',
        ));
        Assets::add_module_js('bookings', 'bookings.js');
        $this->lang->load('emailer/emailer');
        $this->lang->load('fields/fields');
        $form = $this->db->where('id', $form_id)->get('booking_forms')->row();
        $redirect_url = site_url(SITE_AREA . '/financial/bookings');
        if(!$form) {
            Template::redirect($redirect_url);
        }
        $this->load->model('fields/fields_model');
        $fields = $this->fields_model->get_booking_fields(array(
            'form_id' => $form_id
        ), false);
        if(isset($_POST['save'])) {
            $this->gcrud->stats(array('module' => 'bookings'));
            // Validate the data
            $this->form_validation->set_rules('template[email][user]', 'lang:send_user_email', 'trim');
            $this->form_validation->set_rules('template[email][member]', 'lang:send_member_email', 'trim');
            $this->form_validation->set_rules('template[email][admin]', 'lang:send_admin_email', 'trim');
            $this->form_validation->set_rules('template[sms][user]', 'lang:send_user_sms', 'trim');
            $this->form_validation->set_rules('template[sms][member]', 'lang:send_member_sms', 'trim');
            $this->form_validation->set_rules('template[sms][admin]', 'lang:send_admin_sms', 'trim');
            $this->form_validation->set_rules('template[admin][subject]', 'lang:label_email_subject', 'trim|max_length[255]');
            $this->form_validation->set_rules('template[admin][message]', 'lang:label_email_message', 'trim');
            $this->form_validation->set_rules('template[admin][sms]', 'lang:label_sms_text', 'trim');
            $this->form_validation->set_rules('template[user][subject]', 'lang:label_email_subject', 'trim|max_length[255]');
            $this->form_validation->set_rules('template[user][message]', 'lang:label_email_message', 'trim');
            $this->form_validation->set_rules('template[user][sms]', 'lang:label_sms_text', 'trim');
            if ($this->form_validation->run() === true) {
                $data = array(
                    'template' => json_encode($this->input->post('template'))
                );
                $this->db->where('id', $form_id)->update('booking_forms', $data);
                Template::set_message(sprintf(lang('msg_save_success'), lang('label_template_settings')), 'success');
                Template::redirect(current_url());
            }
        }
        if(isset($_POST['reset'])) {         
            $this->gcrud->stats(array('module' => 'bookings'));
            // reset to default template text
            $this->bookings_model->template_default($form_id, array(), $fields);
            Template::set_message(sprintf(lang('msg_save_success'), lang('label_template_settings')), 'success');
            Template::redirect(current_url());
        }
        Template::set(array(
            'record' => json_decode($form->template ?? ''),
            'fields' => $fields,
            'toolbar_title' => $form->title . ' - ' . lang('label_template_settings')
        ));
        Template::render();
    }
    
    /**
     * Allow Admin to Prioritize Fields
     * @param number $form_id
     */
    public function order($form_id) {
        Template::set('cancel_url', site_url(SITE_AREA . '/financial/bookings/forms'));
        $this->gc->order_rows('field_modules', array('module_id' => gc_module_id('bookings'), 'row_id' => $form_id));
    }
    
    /**
     * Allow Admin to Export Data
     * @param number $allotment_id
     * @param number $form_id
     */
    public function export($allotment_id = 0, $form_id = 0) {
        if($form_id == 0) {
            $redirect_url = site_url(SITE_AREA . '/financial/bookings/allotments');
        } else {
            $redirect_url = site_url(SITE_AREA . '/financial/bookings/forms');
        }            
        $this->bookings_model->export_data($allotment_id, $form_id, 'admin', $redirect_url);
    }
    
    /**
     * Allow Admin to change the allotment settings
     * @param number $allotment_id
     */
    public function edit_allotment($allotment_id) {
        $this->bookings_model->settings(0, $allotment_id, 'admin');
        Template::render();
    }
    
    /**
     * Display Form Fields and Pricing packages to the Admin
     *
     * @param number $allotment_id
     */
    public function fields($allotment_id) {
        $this->bookings_model->fields($allotment_id, 'admin');
        Template::render();
    }
    
    /**
     * Allow user to add options
     *
     * A single table is used to store options, dynamic field relations and pricing.
     * the field type_id is used to differentiate them.
     *
     * @param number $form_id
     * @param number $field_id
     * @param number $type_id = 0 -> Options, 1 -> Dynamic Fields, 2 - Pricing/Packages
     */
    public function options($allotment_id = 0, $field_id = 0, $type_id = 0) {
        $this->bookings_model->options($allotment_id, $field_id, $type_id, 'admin');
        Template::render();
    }
    
    /**
     * Allow admin to add/edit packages
     *
     * @param number $id = 0 -> Insert, Value -> Edit
     */
    public function packages($allotment_id, $id = 0) {
        $this->bookings_model->packages($allotment_id, $id, 'admin');
        Template::render();
    } 
    
    /**
     * Booking Cancellations
     * @param integer $booking_id
     */
    public function cancellation($booking_id) {
        $this->load->model('orders/cancellations_model');
        $redirect_url = site_url(SITE_AREA . '/financial/bookings');
        $this->cancellations_model->form('bookings', $booking_id, 1, 'admin', $redirect_url);
        Template::set('cancel_url', $redirect_url);
        Template::render();
    }
    
    /**
     * Assign fields to Booking Forms
     * @param number $id
     * @param string $module
     */
    public function assign_fields($id) {
        $this->load->model('fields/fields_model');
        $this->fields_model->assign($id, 'bookings');
    }
}