<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Migration_Install_bookings extends Migration {
    /**
     * The name of the database tables
     *
     * @var String
     */
    private $table_name             = 'bookings';
    private $table_allotments       = 'booking_allotments';
    private $table_allotment_fields = 'booking_allotment_fields';
    private $table_fields           = 'booking_fields';
    private $table_field_relations  = 'booking_field_relations';
    private $table_forms            = 'booking_forms';
    private $table_hierarchy        = 'booking_hierarchy';    
    private $table_metas            = 'booking_metas';
    private $table_rules            = 'booking_rules';
    
    
    /**
     * The table's fields
     *
     * @var Array
     */
    private $fields = array(
        'id' => array(
            'type' => 'INT',
            'constraint' => 11,
            'auto_increment' => TRUE
        ),
        'user_id' => array(
            'type' => 'BIGINT',
            'constraint' => 20,
            'null' => FALSE
        ),
        'form_id' => array(
            'type' => 'SMALLINT',
            'constraint' => 5,
            'null' => FALSE,
            'default' => 0
        ),
        'allotment_id' => array(
            'type' => 'INT',
            'constraint' => 11,
            'null' => FALSE,
            'default' => 0
        ),
        'allotment_booking_id' => array(
            'type' => 'INT',
            'constraint' => 11,
            'null' => FALSE,
            'default' => 0
        ),
        'package_id' => array(
            'type' => 'INT',
            'constraint' => 11,
            'null' => FALSE
        ),
        'form_data' => array(
            'type' => 'TEXT',
            'null' => TRUE
        ),
        'status' => array(
            'type' => 'TINYINT',
            'constraint' => 1,
            'null' => FALSE,
            'default' => 0
        ),
        'booking_date' => array(
            'type' => 'DATETIME',
            'null' => TRUE
        ),
        'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
        'created_ip' => array(
            'type' => 'VARCHAR',
            'constraint' => 50,            
            'null' => TRUE
        ),
        'modified_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
        'modified_ip' => array(
            'type' => 'VARCHAR',
            'constraint' => 50,
            'null' => TRUE
        )
    );
    
    /**
     * The booking_allotments table fields
     *
     * @var Array
     */
    private $booking_allotments = array(
        'id' => array(
            'type' => 'INT',
            'constraint' => 11,
            'auto_increment' => TRUE
        ),
        'module_id' => array(
            'type' => 'TINYINT',
            'constraint' => 1,
            'null' => FALSE,
            'default' => 0
        ),
        'row_id' => array(
            'type' => 'BIGINT',
            'constraint' => 20,
            'null' => FALSE,
            'default' => 0
        ),
        'user_id' => array(
            'type' => 'BIGINT',
            'constraint' => 20,
            'null' => FALSE,
            'default' => 0
        ),
        'form_id' => array(
            'type' => 'SMALLINT',
            'constraint' => 5,
            'null' => FALSE,
            'default' => 0
        ),
        'start_date' => array(
            'type' => 'DATETIME',
            'null' => TRUE
        ),
        'end_date' => array(
            'type' => 'DATETIME',
            'null' => TRUE
        ),
        'before_duration' => array(
            'type' => 'SMALLINT',
            'constraint' => 5,
            'null' => FALSE,
            'default' => 0
        ),
        'advance_duration' => array(
            'type' => 'SMALLINT',
            'constraint' => 5,
            'null' => FALSE,
            'default' => 0
        ),
        'seats' => array(
            'type' => 'SMALLINT',
            'constraint' => 5,
            'null' => FALSE,
            'default' => 0
        ),
        'seats_unit' => array(
            'type' => 'TINYINT',
            'constraint' => 1,
            'null' => FALSE,
            'default' => 0
        ),
        'seats_limit' => array(
            'type' => 'SMALLINT',
            'constraint' => 5,
            'null' => FALSE,
            'default' => 1
        ),
        'seats_available' => array(
            'type' => 'SMALLINT',
            'constraint' => 5,
            'null' => FALSE,
            'default' => 0
        ),
        'layout' => array(
            'type' => 'VARCHAR',
            'constraint' => 20,
            'null' => TRUE
        ),
        'refund_duration' => array(
            'type' => 'SMALLINT',
            'constraint' => 5,
            'null' => FALSE,
            'default' => 0
        ),
        'refund_unit' => array(
            'type' => 'TINYINT',
            'constraint' => 1,
            'null' => FALSE,
            'default' => 0
        ),
        'tax_percent' => array(
            'type' => 'FLOAT',
            'null' => FALSE,
            'default' => 0
        ),
        'tax_inclusive' => array(
            'type' => 'TINYINT',
            'constraint' => 1,
            'null' => FALSE,
            'default' => 0
        ),
        'allow_coupon' => array(
            'type' => 'TINYINT',
            'constraint' => 1,
            'null' => FALSE,
            'default' => 0
        ),
        'multisale' => array( // Allow user to purchase the same package again - 1 -> Enabled, 0 -> Not Allowed
            'type' => 'TINYINT',
            'constraint' => 1,
            'null' => FALSE,
            'default' => 1
        ),
        'status' => array(
            'type' => 'TINYINT',
            'constraint' => 1,
            'null' => FALSE,
            'default' => 0
        ),
        'created_on' => array(
            'type' => 'DATETIME',
            'null' => TRUE
        ),
        'created_ip' => array(
            'type' => 'VARCHAR',
            'constraint' => 50,
            'null' => TRUE
        ),
        'modified_on' => array(
            'type' => 'DATETIME',
            'null' => TRUE
        ),
        'modified_ip' => array(
            'type' => 'VARCHAR',
            'constraint' => 50,
            'null' => TRUE
        )
    );
    
    /**
     * The booking_allotment_fields table fields
     *
     * @var Array
     */
    private $booking_allotment_fields = array(
        'id' => array(
            'type' => 'INT',
            'constraint' => 11,
            'auto_increment' => TRUE
        ),
        'form_id' => array(
            'type' => 'SMALLINT',
            'constraint' => 5,
            'null' => FALSE,
            'default' => 0
        ),
        'allotment_id' => array(
            'type' => 'INT',
            'constraint' => 11,
            'null' => FALSE,
            'default' => 0
        ),
        'field_id' => array(
            'type' => 'SMALLINT',
            'constraint' => 5,
            'null' => FALSE,
            'default' => 0
        ),
        'is_extra' => array(
            'type' => 'TINYINT',
            'constraint' => 1,
            'null' => FALSE,
            'default' => 0
        ),
        'status' => array(
            'type' => 'TINYINT',
            'constraint' => 1,
            'null' => FALSE,
            'default' => 0
        ),
        'priority' => array(
            'type' => 'INT',
            'constraint' => 11,
            'null' => FALSE,
            'default' => 0
        )
    );
    
    /**
     * The booking_fields table fields
     *
     * @var Array
     */
    private $booking_fields = array(
        'id' => array(
            'type' => 'INT',
            'constraint' => 11,
            'auto_increment' => TRUE,
        ),
        'allotment_id' => array(
            'type' => 'INT',
            'constraint' => 11,
            'null' => FALSE,
            'default' => 0
        ),
        'form_id' => array(
            'type' => 'SMALLINT',
            'constraint' => 5,
            'null' => FALSE,
            'default' => 0
        ),
        'field_id' => array(
            'type' => 'SMALLINT',
            'constraint' => 5,
            'null' => FALSE,
            'default' => 0
        ),
        'type_id' => array(
            'type' => 'TINYINT',
            'constraint' => 1,
            'null' => FALSE,
            'default' => 0
        ),
        'dynamic_id' => array(
            'type' => 'INT',
            'constraint' => 11,
            'null' => FALSE,
            'default' => 0
        ),
        'value' => array(
            'type' => 'VARCHAR',
            'constraint' => 255,
            'null' => FALSE
        )
    );
    
    /**
     * The booking_field_relations table fields
     *
     * @var Array
     */
    private $booking_field_relations = array(
        'id' => array(
            'type' => 'INT',
            'constraint' => 11,
            'auto_increment' => TRUE,
        ),
        'allotment_id' => array(
            'type' => 'INT',
            'constraint' => 11,
            'null' => FALSE,
            'default' => 0
        ),
        'allotment_package_id' => array( // Allotment Specific Package ID
            'type' => 'INT',
            'constraint' => 11,
            'null' => FALSE,
            'default' => 0
        ),
        'form_id' => array(
            'type' => 'SMALLINT',
            'constraint' => 5,
            'null' => FALSE,
            'default' => 0
        ),
        'package_id' => array(
            'type' => 'INT',
            'constraint' => 11,
            'null' => FALSE,
            'default' => 0
        ),
        'booking_field_id' => array(
            'type' => 'INT',
            'constraint' => 11,
            'null' => FALSE,
            'default' => 0
        )
    );
    
    /**
     * The booking_forms table fields
     *
     * @var Array
     */
    private $booking_forms = array(
        'id' => array(
            'type' => 'SMALLINT',
            'constraint' => 5,
            'auto_increment' => TRUE
        ),
        'title' => array(
            'type' => 'VARCHAR',
            'constraint' => 150,
            'null' => FALSE
        ),
        'commission' => array(
            'type' => 'FLOAT',
            'null' => FALSE,
            'default' => 0
        ),
        'max_packages' => array(
            'type' => 'TINYINT',
            'constraint' => 2,
            'null' => FALSE,
            'default' => 10
        ),
        'enable_cache' => array(
            'type' => 'TINYINT',
            'constraint' => 1,
            'null' => FALSE,
            'default' => 0
        ),
        'rules' => array(
            'type' => 'VARCHAR',
            'constraint' => 255,
            'null' => TRUE
        ),
        'form_fields' => array(
            'type' => 'VARCHAR',
            'constraint' => 450,
            'null' => TRUE
        ),
        'template' => array(
            'type' => 'LONGTEXT',
            'null' => TRUE
        ),
        'status' => array(
            'type' => 'TINYINT',
            'constraint' => 1,
            'null' => FALSE,
            'default' => 1
        ),
        'created_on' => array(
            'type' => 'DATETIME',
            'null' => TRUE
        ),
        'created_by' => array(
            'type' => 'BIGINT',
            'constraint' => 20,
            'null' => FALSE,
            'default' => 0
        ),
        'created_ip' => array(
            'type' => 'VARCHAR',
            'constraint' => 50,
            'null' => TRUE
        ),
        'modified_on' => array(
            'type' => 'DATETIME',
            'null' => TRUE
        ),
        'modified_by' => array(
            'type' => 'BIGINT',
            'constraint' => 20,
            'null' => FALSE,
            'default' => 0
        ),
        'modified_ip' => array(
            'type' => 'VARCHAR',
            'constraint' => 50,
            'null' => TRUE
        )
    );
    
    /**
     * The booking_hierarchy table fields
     *
     * @var Array
     */
    private $booking_hierarchy = array(
        'id' => array(
            'type' => 'INT',
            'constraint' => 11,
            'auto_increment' => TRUE
        ),
        'ancestor' => array(
            'type' => 'INT',
            'constraint' => 11,
            'null' => FALSE
        ),
        'descendant' => array(
            'type' => 'INT',
            'constraint' => 11,
            'null' => FALSE
        ),
        'depth' => array(
            'type' => 'INT',
            'constraint' => 11,
            'null' => FALSE,
            'default' => 0
        )
    );
    
    /**
     * The booking_metas table fields
     *
     * @var Array
     */
    private $booking_metas = array(
        'id' => array(
            'type' => 'INT',
            'constraint' => 11,
            'auto_increment' => TRUE
        ),
        'booking_id' => array(
            'type' => 'INT',
            'constraint' => 11,
            'null' => FALSE
        ),
        'field_id' => array(
            'type' => 'INT',
            'constraint' => 11,
            'null' => FALSE
        ),
        'dynamic_id' => array(
            'type' => 'INT',
            'constraint' => 11,
            'null' => FALSE,
            'default' => 0
        ),        
        'field_label' => array(
            'type' => 'VARCHAR',
            'constraint' => 250,
            'null' => TRUE
        ),
        'field_value' => array(
            'type' => 'TEXT',
            'null' => TRUE
        ),
        'name' => array(
            'type' => 'VARCHAR',
            'constraint' => 100,
            'null' => FALSE
        ),
        'value' => array(
            'type' => 'TEXT',
            'null' => FALSE
        )
    );
    
    /**
     * The booking_rules table fields
     *
     * @var Array
     */
    private $booking_rules = array(
        'id' => array(
            'type' => 'INT',
            'constraint' => 11,
            'auto_increment' => TRUE
        ),
        'rule_id' => array(
            'type' => 'TINYINT',
            'constraint' => 1,
            'null' => FALSE
        ),
        'package_id' => array(
            'type' => 'INT',
            'constraint' => 11,
            'null' => FALSE
        ),
        'value' => array(
            'type' => 'TINYINT',
            'constraint' => 4,
            'null' => FALSE
        )
    );
    
    /**
     * Install this migration
     *
     * @return void
     */
    public function up() {
        // bookings table fields
        $this->dbforge->add_field($this->fields);
        $this->dbforge->add_key('id', true);
        $this->dbforge->add_key('user_id', false);
        $this->dbforge->add_key('allotment_id', false);
        $this->dbforge->add_key('form_id', false);
        $this->dbforge->add_key('status', false);
        $this->dbforge->create_table($this->table_name);
        
        // booking_allotments table fields
        $this->dbforge->add_field($this->booking_allotments);
        $this->dbforge->add_key('id', true);
        $this->dbforge->add_key('user_id', false);
        $this->dbforge->add_key('form_id', false);
        $this->dbforge->create_table($this->table_allotments);        
        $module_index_sql = "ALTER TABLE " . $this->db->dbprefix . $this->table_allotments . " ADD INDEX `module` (`module_id`, `row_id`)";
        $this->db->query ($module_index_sql);        
        
        // booking_allotment_fields table - A user can change the status of a field and display it on the availability screen
        $this->dbforge->add_field($this->booking_allotment_fields);
        $this->dbforge->add_key('id', true);
        $this->dbforge->add_key('form_id', false);
        $this->dbforge->add_key('allotment_id', false);
        $this->dbforge->add_key('field_id', false);
        $this->dbforge->add_key('is_extra', false);
        $this->dbforge->add_key('status', false);
        $this->dbforge->add_key('priority', false);
        $this->dbforge->create_table($this->table_allotment_fields);
        
        // booking_fields table fields
        $this->dbforge->add_field($this->booking_fields);
        $this->dbforge->add_key('id', true);
        $this->dbforge->add_key('allotment_id', false);
        $this->dbforge->add_key('form_id', false);
        $this->dbforge->add_key('field_id', false);
        $this->dbforge->add_key('type_id', false);
        $this->dbforge->add_key('dynamic_id', false);
        $this->dbforge->create_table($this->table_fields);
        
        // booking_field_relations table fields
        $this->dbforge->add_field($this->booking_field_relations);
        $this->dbforge->add_key('id', true);
        $this->dbforge->add_key('allotment_id', false);
        $this->dbforge->add_key('form_id', false);
        $this->dbforge->add_key('package_id', false);
        $this->dbforge->add_key('booking_field_id', false);
        $this->dbforge->create_table($this->table_field_relations);
        
        // booking_forms table
        $this->dbforge->add_field($this->booking_forms);
        $this->dbforge->add_key('id', true);
        $this->dbforge->create_table($this->table_forms);
        
        // booking_hierarchy table
        $this->dbforge->add_field($this->booking_hierarchy);
        $this->dbforge->add_key('id', true);
        $this->dbforge->create_table($this->table_hierarchy);
        
        // booking_metas table
        $this->dbforge->add_field($this->booking_metas);
        $this->dbforge->add_key('id', true);
        $this->dbforge->add_key('booking_id', false);
        $this->dbforge->add_key('field_id', false);
        $this->dbforge->add_key('dynamic_id', false);
        $this->dbforge->add_key('name', false);
        $this->dbforge->create_table($this->table_metas);
        
        // booking_rules table
        $this->dbforge->add_field($this->booking_rules);
        $this->dbforge->add_key('id', true);
        $this->dbforge->add_key('rule_id', false);
        $this->dbforge->add_key('package_id', false);
        $this->dbforge->create_table($this->table_rules);
    }
    
    //--------------------------------------------------------------------
    
    /**
     * Uninstall this migration
     *
     * @return void
     */
    public function down() {
        $this->dbforge->drop_table($this->table_name);
        $this->dbforge->drop_table($this->table_allotments);
        $this->dbforge->drop_table($this->table_allotment_fields);
        $this->dbforge->drop_table($this->table_fields);
        $this->dbforge->drop_table($this->table_field_relations);
        $this->dbforge->drop_table($this->table_forms);
        $this->dbforge->drop_table($this->table_hierarchy);
        $this->dbforge->drop_table($this->table_metas);
        $this->dbforge->drop_table($this->table_rules);
    }
}