<?php defined('BASEPATH') || exit('No direct script access allowed');
/*=======================================================================*\
 || ##################################################################### ||
 || # GoClixy - Business Directory Script							   # ||
 || # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
 || # ----------------------------------------------------------------  # ||
 || # This file is owned and copyrighted by GoClixy.com. Your license   # ||
 || # confers  no  title  or ownership in the Script and is not a sale  # ||
 || # of any rights to the Software. This file may not be redistributed # ||
 || # in whole or significant part.									   # ||
 || # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
 || # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
 || ##################################################################### ||
 \*=======================================================================*/
class Bookings_model extends BF_Model
{
    protected $table_name	          = 'bookings';
    protected $table_metas            = 'booking_metas';
    protected $table_forms	          = 'booking_forms';
    protected $table_allotments	      = 'booking_allotments';
    protected $table_fields           = 'booking_fields';
    protected $table_field_relations  = 'booking_field_relations';
    protected $table_hierarchy        = 'booking_hierarchy';
    
    protected $key			= 'id';
    protected $date_format	= 'datetime';
    
    protected $log_user 	= true;
    protected $set_created	= true;
    protected $set_modified = true;
    protected $soft_deletes	= true;
    
    protected $created_field     = 'created_on';
    protected $created_by_field  = 'created_by';
    protected $modified_field    = 'modified_on';
    protected $modified_by_field = 'modified_by';
    protected $deleted_field     = 'deleted';
    protected $deleted_by_field  = 'deleted_by';
    
    // Customize the operations of the model without recreating the insert,
    // update, etc. methods by adding the method names to act as callbacks here.
    protected $before_insert 	= array();
    protected $after_insert 	= array();
    protected $before_update 	= array();
    protected $after_update 	= array();
    protected $before_find 	    = array();
    protected $after_find 		= array();
    protected $before_delete 	= array();
    protected $after_delete 	= array();
    
    // For performance reasons, you may require your model to NOT return the id
    // of the last inserted row as it is a bit of a slow method. This is
    // primarily helpful when running big loops over data.
    protected $return_insert_id = true;
    
    // The default type for returned row data.
    protected $return_type = 'object';
    
    // Items that are always removed from data prior to inserts or updates.
    protected $protected_attributes = array();
    
    // You may need to move certain rules (like required) into the
    // $insert_validation_rules array and out of the standard validation array.
    // That way it is only required during inserts, not updates which may only
    // be updating a portion of the data.
    protected $validation_rules 		= array(
        array(
            'field' => 'form_id',
            'label' => 'lang:label_form_id',
            'rules' => 'max_length[11]',
        ),
        array(
            'field' => 'assignment_id',
            'label' => 'lang:label_assignment_id',
            'rules' => 'max_length[11]',
        ),
        array(
            'field' => 'package_id',
            'label' => 'lang:label_package_id',
            'rules' => 'max_length[11]',
        ),
        array(
            'field' => 'fields',
            'label' => 'lang:label_fields',
            'rules' => '',
        ),
        array(
            'field' => 'status',
            'label' => 'lang:label_status',
            'rules' => 'max_length[1]',
        ),
    );
    
    // Form Validation Rules
    public $form_validation_rules 		= array(
        array(
            'field' => 'title',
            'label' => 'lang:label_title',
            'rules' => 'trim|required',
        ),
        array(
            'field' => 'commission',
            'label' => 'lang:label_commission',
            'rules' => 'trim|required|is_decimal',
        ),
        array(
            'field' => 'max_packages',
            'label' => 'lang:label_max_packages',
            'rules' => 'trim|required|is_numeric',
        ),
        array(
            'field' => 'status',
            'label' => 'lang:label_status',
            'rules' => 'trim',
        ),
    );
    
    protected $insert_validation_rules  = array();
    protected $skip_validation 			= false;
    protected $closure;
    
    public $gc_booking_cache;
    
    // Allow admin to show/hide fields
    public $allotment_fields = array('start_date', 'end_date', 'seats', 'seats_limit', 'multisale'); // 'before_duration', 'advance_duration' // creating confusions
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
        // purge information from the booking_hierarchy table
        $this->load->library('Gc_closure');
        $this->closure = new Gc_closure('booking_field_relations', 'booking_hierarchy');
        $this->load->library('slug');
        // Load Zend Caching
        $this->load->library('gzf_cache', array('lifetime' => 900));
        if(method_exists($this->gzf_cache, 'get_instance')) {
            $this->gc_booking_cache = $this->gzf_cache->get_instance();
        }
    }
    
    /**
     * Save Options
     *
     * @param number $module_id
     * @param number $row_id
     * @param number $user_id
     * @param number $form_id
     * @param number $field_id
     * @param number $type_id
     * @param string $type
     * @return boolean|number
     */
    public function save_options($where) {
        $data = $where;
        $values = $this->input->post('values');
        if (empty($values)) {
            // delete all values
            return $this->db->where($where)->delete($this->table_fields);
        }
        $value_ids = array();
        foreach ($values as $value) {
            $id = $value['id'];
            $value = escape($value['value']);
            if (! empty($value)) { // save only filled values
                // check existence
                if (! empty($id)) {
                    $this->db->where('id', $id)->update($this->table_fields, array(
                        'value' => $value
                    ));
                    $value_ids[] = $id;
                } else { // insert value
                    $data = $where;
                    $data['value'] = $value;
                    $this->db->insert($this->table_fields, $data);
                    $value_ids[] = $this->db->insert_id();
                }
            }
        }
        // Delete removed rows
        if (! empty($value_ids)) {
            if ($where['type_id'] == 1) { // Also delete other records where dynamic field id is value_id
            }
            $this->db->where($where);
            return $this->db->where('id NOT IN (' . implode(',', $value_ids) . ')')->delete($this->table_fields);
        }
    }
    
    /**
     * Save Pricing
     *
     * @param array $where
     * @param array $fields
     * @param number $id
     * @param string $type
     */
    public function save_pricing($where, $fields, $id = 0) {
        // Validate each field
        $this->load->library('form_validation');
        $this->form_validation->set_rules('dummy', lang('label_dummy'), 'trim');
        foreach($fields as $field_validation) {
            $this->form_validation->set_rules($field_validation['name'], ($field_validation['label_lang'] == 1 ? lang($field_validation['label']) : $field_validation['label']), (!empty($field_validation['rules']) ? $field_validation['rules'] . '|required' : 'required'));
        }
        if($this->form_validation->run() === FALSE) { // All fields are must
            return false;
        }
        $type = ($id == 0 ? 'insert' : 'update');
        $allotment_package_id = 0;
        // $where = array('module_id' => $module_id, 'row_id' => $row_id, 'form_id' => $form_id);
        // Get highest pricing id
        if ($id == 0) {
            $package_id = $id;
            $id = $this->db->select('MAX(package_id) AS package_id')->get('booking_field_relations')->row();
            $id = $id->package_id + 1;
        } else {
            // Update
            $package_id = $id;
        }
        
        $i = 1;
        $parent_row = array();
        $not_exists = false;
        
        // do not allow user to add a pricing package with same values
        if(!$not_exists) { // run until we find a non existence row
            $fields_count = 0;
            $package_field_ids = array();
            foreach($fields as $field) {
                $exist_where = $where;
                if(isset($where['allotment_id'])) {
                    $exist_where[$this->table_fields . '.allotment_id'] = $where['allotment_id'];
                    unset($exist_where['allotment_id']);
                }
                if(isset($where['form_id'])) {
                    $exist_where[$this->table_fields . '.form_id'] = $where['form_id'];
                    unset($exist_where['form_id']);
                }
                $exist_where[$this->table_fields . '.field_id'] = $field['id'];
                $exist_where[$this->table_fields . '.dynamic_id'] = isset($field['dynamic_id']) ? $field['dynamic_id'] : 0;
                $exist_where[$this->table_fields . '.type_id'] = 2;
                $exist_where[$this->table_fields . '.value'] = $this->input->post($field['name']);
                // $this->db->join($this->table_field_relations . ' BFR', 'BFR.booking_field_id = '. $this->table_fields . '.id');
                // $exist_where['BFR.package_id'] = 'BFR.package_id';
                $existing_field = $this->db->select($this->table_fields . '.id')->where($exist_where)->get($this->table_fields)->row();
                if($existing_field) { // A non existence value found
                    $package_field_ids[] = $existing_field->id;
                }
                $fields_count++;
            }
            if(!empty($package_field_ids)) {
                $sql = "SELECT package_id FROM " . $this->db->dbprefix. "booking_field_relations";
                $sql .= " WHERE booking_field_id IN (" . implode(',', $package_field_ids) . ")";
                if(!empty($id)) {
                    $sql .= " AND package_id != " . $package_id;
                }
                $sql .= " GROUP BY package_id";
                $sql .= " HAVING COUNT(package_id) = " . $fields_count;
                $existing_package = $this->db->query($sql)->row();
                if(!$existing_package) {
                    $not_exists = true; // set non_exists variable to true
                }
            } else { // Adding package for the first time
                $not_exists = true; // set non_exists variable to true
            }
        }
        if(!$not_exists) {
            Template::set_message(lang('msg_duplicate_record'), 'danger');
            return false;
        }
        
        if(!empty($package_id)) { // Purge existing package information in case of edit
            $this->purge_pricing($package_id);
        }
        
        // Unique Package ID given to each allotment package
        $allotment_package_id = $this->db->select_max('allotment_package_id')->get_where('booking_field_relations', array('allotment_id' => $where['allotment_id']))->row();
        $allotment_package_id = (int) $allotment_package_id->allotment_package_id + 1;
        
        // save data
        foreach ($fields as $field) {
            $obj = $where;
            $field_id = explode('-', $field['id'] ?? ''); // split field on the form changes - July 22, 2021
            $obj['field_id'] = $field_id[0];
            $obj['dynamic_id'] = isset($field['dynamic_id']) ? $field['dynamic_id'] : 0;
            $obj['type_id'] = 2;
            $obj['value'] = $this->input->post($field['name']);
            $existence_where = $obj;
            $existence_where['value'] = $this->input->post($field['name']);
            
            // check value existence
            $row = $this->db->get_where($this->table_fields, $obj)->row();
            /*
             * if($row) { // do not save in the booking_fields table, but save in another table for future edits.
             *
             * }
             */
            if (! $row) {
                $this->db->insert($this->table_fields, $obj);
                $booking_field_id = $this->db->insert_id();
            } else {
                $booking_field_id = $row->id;
            }
            
            // save information in the booking_field_relations
            $this->db->insert('booking_field_relations', array(
                'allotment_id' => $where['allotment_id'],
                'allotment_package_id' => $allotment_package_id,
                'form_id' => $where['form_id'],
                'package_id' => $id,
                'booking_field_id' => $booking_field_id
            ));
            $row_id = $this->db->insert_id();
            
            // save parent_child hierarchy
            if ($row_id > 0) {
                $parent_row[$i] = $row_id;
                if ($i == 1) { // save relationship
                    $this->closure->add($row_id, $row_id);
                } else {
                    $this->closure->add($row_id, $parent_row[$i - 1]);
                }
            }
            ++ $i;
        }
        
        // save rules if not exists
        $rules = $this->input->post('rules');
        if(!empty($rules)) {
            $rule_data = array();
            foreach($rules as $rule_id => $rule_values) {
                if(!empty($rule_values)) {
                    foreach($rule_values as $rule_value_key => $rule_value) {
                        $rule_data[] = array(
                            'rule_id' => $rule_id,
                            'package_id' => $id,
                            'value' => $rule_value_key
                        );
                    }
                }
            }
            if(!empty($rule_data)) {
                foreach($rule_data as $rule_key => $rule_row) {
                    // check existence
                    $count = $this->db->where($rule_row)->count_all_results('booking_rules');
                    if($count == 0) {
                        // insert row
                        $this->db->insert('booking_rules', $rule_row);
                    }
                }
            }
        }
        return true;
    }
    
    /**
     * Count number of packages in a FORM/ALLOTMENT
     * @param number $allotment_id
     * @return number
     */
    public function packages_count($id, $field = 'allotment_id') {
        $sql = "SELECT COUNT(DISTINCT(package_id)) AS total_count FROM `" . $this->db->dbprefix . "booking_field_relations` WHERE " . $field . " = " . $id;
        $result = $this->db->query($sql)->row();
        return $result->total_count;
    }
    
    
    /**
     * Remove Booking Form
     * @param integer $id - Booking Form ID
     */
    public function purge_form($id) {
        // Delete all bookings first
        $this->purge_bookings($id);
        
        // Purge fields with relations
        $this->purge_fields($id, 'form_id');
        
        // Remove all Booking Allotments of this form
        $this->db->where('form_id', $id)->delete('booking_allotments');
        
        // Remove all Assigned Fields information
        $this->db->where(array('module_id' => gc_module_id('bookings'), 'row_id' => $id))->delete('field_modules');
        
        // Remove all Booking Allotment Fields of this form
        // it is difficult to manage allotment fields, a booking form can have thousands of allotments.
        // $this->db->where('form_id', $id)->delete('booking_allotment_fields');
        
        // At last, remove the form
        return $this->db->where('id', $id)->delete('booking_forms');
    }
    
    /**
     * Remove alloted form
     * @param integer $id
     */
    public function purge_allotment($id) {
        // Delete all bookings first
        $this->purge_bookings($id, 'allotment_id');
        
        // Purge fields with relations
        $this->purge_fields($id, 'allotment_id');
        
        // Remove all Booking Allotments of this allotment
        $this->db->where('id', $id)->delete('booking_allotments');
        
        // Remove all Booking Allotment Fields of this allotment
        // it is difficult to manage allotment fields, a booking form can have thousands of allotments.
        // $this->db->where('allotment_id', $id)->delete('booking_allotment_fields');
    }
    
    /**
     * Purge field with relations
     *
     * @param integer $id
     * @param string $field
     */
    public function purge_fields($id, $field = 'form_id') {
        $booking_field_relations = $this->db->where($field, $id)->get($this->table_field_relations)->result();
        
        $package_ids = array();
        // purge relation first
        if($booking_field_relations) {
            foreach($booking_field_relations as $booking_field_relation) {
                $package_ids[] = $booking_field_relation->package_id;
                $this->closure->delete($booking_field_relation->id, false);
            }
        }
        $package_ids = array_unique($package_ids);
        
        if(!empty($package_ids)) {
            // delete information from the booking rules table
            $this->db->where('package_id IN (' . implode(',', $package_ids) . ')')->delete('booking_rules');
            
            // delete information from the booking_field_relations table
            $this->db->where($field, $id)->delete($this->table_field_relations);
            
            // purge fields
            return $this->db->where($field, $id)->delete($this->table_fields);
        } else {
            return false;
        }
    }
    
    /**
     * Purge Bookings
     * @param integer $id
     */
    public function purge_bookings($id, $field = 'form_id') {
        // Remove field metas information first before removing the Booking
        $bookings = $this->db->where($field, $id)->get('bookings')->result();
        if($bookings) {
            foreach($bookings as $booking) {
                // purge field metas
                $this->db->where('booking_id', $booking->id)->delete($this->table_metas);
            }
        }
        // purge booking
        $this->db->where($field, $id)->delete('bookings');
    }
    
    /**
     * Purge a single booking
     * @param number $id
     */
    public function purge_booking($id) {
        // purge field metas
        $this->db->where('booking_id', $id)->delete($this->table_metas);
        // purge booking
        return $this->db->where('id', $id)->delete('bookings');
    }
    
    /**
     * Purge pricing information
     *
     * @param number $package_id
     */
    public function purge_pricing($package_id) {
        $package_ids = array();
        $records = $this->db->where('package_id', $package_id)->get($this->table_field_relations)->result();
        if ($records) {
            foreach ($records as $record) {
                $package_ids[] = $record->package_id;
                // check multiple occurances of each row
                $count = $this->db->where('booking_field_id', $record->booking_field_id)->count_all_results($this->table_field_relations);
                if ($count == 1) { // delete its information from the booking_fields table
                    $this->db->where(array('id' => $record->booking_field_id, 'type_id' => 2))->delete($this->table_fields);
                }
                // remove information from the booking_hierarchy table
                $this->closure->delete($record->id, false); // IMP - false - do not delete reference
            }
        }
        
        // get unique rows
        $package_ids = array_unique($package_ids);
        
        // delete information from the booking rules table
        if(!empty($package_ids)) {
            $this->db->where('package_id IN (' . implode(',', $package_ids) . ')')->delete('booking_rules');
        }
        
        // delete information from the booking_field_relations table
        return $this->db->where('package_id', $package_id)->delete($this->table_field_relations);
    }
    
    /**
     * Display Allotments to the Admin
     * @param number $count
     */
    public function get_allotments($count, $limit, $offset) {
        $sql = "SELECT A.*, U.display_name, U.email, F.title AS form_title FROM " . $this->db->dbprefix . $this->booking_allotments . " A";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "users U on U.id = A.user_id";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . $this->booking_forms . " F on F.id = A.form_id";
        if($count == 0) {
            $sql .= " GROUP BY A.id ORDER BY A.id DESC";
            if ($limit != 0) {
                $sql .= " LIMIT " . $offset . ", " . $limit;
            } else {
                $sql .= " LIMIT $limit";
            }
            $query = $this->db->query ( $sql );
            return $query->result();
        } else {
            $result = $this->db->query ( $sql )->row();
            return $result ? $result->total_count : 0;
        }
    }
    
    /**
     * Display booking form to the Frontend User
     *
     * @param number $allotment_id
     * @param number $id - ID of the Booking for edit purposes
     */
    public function booking_form($allotment_id, $id = 0, $role = 'admin') {
        $this->auth->restrict (); // only logged in users can make booking
        if($role == 'admin') {
            $redirect_url = site_url(SITE_AREA . '/financial/bookings');
            $render = '';
        } else {
            $redirect_url = gc_url_format('members');
            $render = 'member';
        }
        $error_message = '';
        $allotment = $this->verify_allotment($allotment_id, null);
        if($allotment->status == 0) {
            Template::set_message(lang('form_not_active'), 'danger');
            Template::redirect($redirect_url);
        }
        $this->load->model('fields/fields_model');
        $form_id = $allotment->form_id;
        $user_id = $this->auth->user_id();
        // Allow user to edit submitted information
        $type = 'insert';
        
        // display form only if the show_form variable is TRUE
        $has_availability_screen = $this->has_availability_screen($form_id);
        $availability_screen = $has_availability_screen; // Home Page Screen
        $availability_session =  $this->session->userdata('availability_screen' . $allotment_id) ? true : false;
        $availability_screen_fields = $this->session->userdata('availability_screen_fields');
        if(isset($_POST['availability_screen']) || $availability_session) {
            $availability_data = $this->session->userdata('availability_screen' . $allotment_id) ? (array) $this->session->userdata('availability_screen' . $allotment_id) : $_POST;
            $availability_booking_date = !empty($availability_data['package_booking_date']) ? $availability_data['package_booking_date'] : null;
            if(!empty($allotment->seats_unit) && !empty($availability_booking_date)) { // daily limit is set
               $total_booked_seats = (int) $this->get_booked_seats($allotment->id, 2, 0, "B.booking_date='" . $availability_booking_date . "'");
               if($allotment->seats <= $total_booked_seats) {
                   $this->session->unset_userdata('availability_screen' . $allotment_id);
                    $availability_screen = true;
                   // send user to the previous screen with a booking limit over message
                   Template::set_message(sprintf(lang('booking_day_limit_over'), $allotment->seats, gc_date_format($availability_booking_date)), 'danger');
                   Template::redirect(gc_url_format('bookings') . '/booking_form/' . $allotment->id);
               }
            }
            if(!$availability_session) {
                // User selected a package from the Check Availability list
                $availability_booking_package = $this->input->post('package_id');
                $availability_booking_date = $this->input->post('package_booking_date');
                if(!empty($availability_booking_package) && !empty($availability_booking_date)) {
                    // set post values for the package_id fields
                    $_POST['booking_date'] = $availability_booking_date;
                    // set other fields POST value
                    if(!empty($availability_screen_fields)) {
                        $booking_package_info = $this->get_booking_package($availability_booking_package);
                        foreach($availability_screen_fields as $availability_screen_field) {
                            $field_dynamic_id = isset($availability_screen_field['dynamic_id']) ? $availability_screen_field['dynamic_id'] : 0;
                            if(!empty($booking_package_info)) {
                                foreach($booking_package_info as $booking_package_field) {
                                    if($availability_screen_field['id'] == $booking_package_field->field_id && $field_dynamic_id == $booking_package_field->dynamic_id) {
                                        $_POST[$availability_screen_field['name']] = $booking_package_field->value;
                                    }
                                }
                            }
                        }
                    }
                    $this->session->set_userdata('availability_screen' . $allotment_id, $_POST);
                    $availability_screen = false;
                } else {
                    // all fields must be filled by the user.
                    $availability_screen_validations = $this->availability_screen_validations($allotment_id);
                    if($availability_screen_validations) {
                        $this->session->set_userdata('availability_screen' . $allotment_id, $_POST);
                        $availability_screen = false;
                    }
                }
            } else {
                $availability_screen = false;
            }
        }
        if(isset($_POST['unset_availability_screen'])) {
            $this->session->unset_userdata('availability_screen' . $allotment_id);
            $availability_screen = true;
        }
        
        $availability_data = $this->session->userdata('availability_screen' . $allotment_id);
        $show_form = true;
        $check_advance_duration = false;
        
        $id = (int) $id;
        $record = new stdClass();
        if($id > 0) {
            $type = 'update';
            // check permission
            $booking = $this->db->where('id', $id)->get('bookings')->row();
            if(!$booking || ($booking->user_id != $user_id && $role != 'admin')) {
                Template::redirect($redirect_url);
            }
            if($role != 'admin' && $record->status == 1) {
                // the admin can edit any form, but a user should not be able to edit submitted form
                Template::redirect($redirect_url);
            }
            
            // allow user to edit the submitted form
            $record = $this->fields_model->find_booking_meta($id);
            if(isset($record->city)) {
                $record->city_id = (int) $record->city;
            }
        }
        
        // show field values on next screen
        if(!empty($availability_data) && is_array($availability_data)) {
            foreach($availability_data as $availability_field => $availability_value) {
                $record->{$availability_field} = $availability_value;
            }
        }
        
        // check start_date and end_date
        $today = date('Y-m-d H:i:s');
        if (! empty($allotment->start_date)) {
            if ($allotment->start_date > $today) {
                // Booking has not yet started
                $show_form = false;
                $error_message = sprintf(lang('msg_start_date'), gc_date_format($allotment->start_date, true));
            }
        }
        if (! empty($allotment->end_date)) {
            if ($today > $allotment->end_date) {
                // Booking has ended
                $show_form = false;
                $error_message = sprintf(lang('msg_end_date'), gc_date_format($allotment->end_date, true));
            } else { // check time before booking
                $hourdiff = round((strtotime($allotment->end_date) - strtotime($today)) / 3600, 1);
                if (!empty($allotment->before_duration) && $allotment->before_duration < $hourdiff) {
                    // Booking time up
                    $show_form = false;
                    $error_message = sprintf(lang('msg_advanced_duration'), $allotment->before_duration, gc_date_format($allotment->end_date, true));
                }
            }
        }
        
        // count number of seats avaialble using the orders table
        if ($allotment->seats > 0) {
            if ($allotment->seats_unit == 1) { // per day limit
                if ($allotment->seats_available >= $allotment->seats) {
                    // No more seat is avaiable for the day
                    if($allotment->advance_duration == 0) {
                        $show_form = false; // advance booking is disabled by the member, do not display form
                    } else {
                        $check_advance_duration = true;
                    }
                    $error_message = sprintf(lang('msg_day_no_seats'), lang('label_today'));
                }
            } else if ($allotment->seats_available >= $allotment->seats) { // all time limit
                // No more seat is avaiable
                $show_form = false;
                $error_message = lang('msg_no_seats');
            }
        }
        
        $fields = array();
        $form_fields = array(); // orginal fields before resetting the options value
        $fields_form = array(); // fields shown on the Form
        $field_packages = array(); // fields to be passed to the check_availability method
        $has_booking_date = false; // The form has Booking Date field enabled, display check availability button on the Availability Screen
        $booking_form_fields = array();
        $fields_info = array();
        $package_field_key = 0; // index of the package field
        if ($show_form) {
            $fields = $this->fields_model->get_booking_fields(array(
                'allotment_id' => $allotment_id,
                'form_id' => $form_id
            ), false);
            $form_fields = $fields;
            // prepare for populate fields
            $i = 0;
            if (! empty($fields)) {
                foreach ($fields as $field_key => $field) {
                    if(((isset($field['is_extra']) && $field['is_extra'] == 1) || $field['name'] == 'booking_date')) { // Availability Screen
                        if(!$availability_screen) { // Display availability screen fields as Readonly for POST Handling
                            $field['group'] = 0; // Show them all in a single group
                            if($field['type'] == 'dropdown') {
                                $field['extra'] = array('disabled' => true);
                                // disable the dropdown field to disallow a new value selection
                                Assets::add_js('var select = $("#' . $field['name'] . '").selectize(); select[0].selectize.disable();', 'inline');
                                $form_fields[$field_key]['rules'] = 'trim';
                            }
                            $field['extra'] = array('readonly' => true);
                            // do not include booking date in the fields as it is handled separately
                            if($field['name'] != 'booking_date') {
                                $field_packages[] = array(
                                    'id' => $field['id'],
                                    'dynamic_id' => (isset($field['dynamic_id']) ? $field['dynamic_id'] : 0),
                                    'value' => (isset($record->{$field['name']}) ? $record->{$field['name']} : '')
                                );
                            }
                        }
                        $fields_form[$field_key] = $field;
                        if($field['name'] == 'booking_date') {
                            $has_booking_date = true;
                        } else if($availability_screen && isset($field['booking']) && $field['booking'] == 1) {
                            // populate dropdown fields on value change
                            $booking_form_fields[$field['name']] = array(
                                'id' => $field['id'],
                                'dynamic_id' => isset($field['dynamic_id']) ? $field['dynamic_id'] : 0,
                                'name' => $field['name'],
                                'options' => $field['options']
                            );
                            // add package-field class to the input field
                            $fields_form[$field_key]['field_class'] = empty($fields_form[$field_key]['field_class']) ? 'package-field' : $fields_form[$field_key]['field_class'] . ' ' . 'package-field';
                        }
                        $package_field_key = $field_key;
                    } else if(!$availability_screen && $field['is_extra'] == 0 && $field['name'] != 'booking_date') {
                        if (isset($field['booking'])) {
                            $booking_form_fields[$field['name']] = array(
                                'id' => $field['id'],
                                'dynamic_id' => isset($field['dynamic_id']) ? $field['dynamic_id'] : 0,
                                'name' => $field['name'],
                                'options' => $field['options']
                            );
                            if($i != 0) {
                                if(empty($record)) {
                                    unset($fields[$field_key]['options']);
                                    $fields[$field_key]['options'][] = array('name' => lang('label_select'), 'value' => '');
                                } else { // Display only selected option
                                    foreach($fields[$field_key]['options'] as $field_option) {
                                        if(isset($record->{$field['name']}) && $field_option['value'] == $record->{$field['name']}) {
                                            unset($fields[$field_key]['options']);
                                            $fields[$field_key]['options'][] = array('name' => $field_option['name'], 'value' => $field_option['value']);
                                        }
                                    }
                                }
                            }
                            ++$i;
                            // unset required validation rule as package id is selected by the user
                            $field_validation_rules = $form_fields[$field_key]['rules'];
                            $field_validation_rules_array = explode('|', $field_validation_rules ?? '');
                            $required_key = array_search('required', $field_validation_rules_array);
                            if (false !== $required_key) {
                                unset($field_validation_rules_array[$required_key]);
                            }
                            $form_fields[$field_key]['rules'] = implode('|', $field_validation_rules_array);
                        } else {
                            $fields_form[$field_key] = $field;
                        }
                    }
                }
            }
            
            // Check packages availability only if the Booking Form has booking date column enabled.
            // call this only after getting fields information to keep posted values
            if(isset($_POST['check_availability'])) {
                $availability_screen_validations = $this->availability_screen_validations($allotment_id);
                if($availability_screen_validations) {
                    $booking_date = $this->input->post('booking_date');
                    if(!empty($booking_date)) {
                        $available_packages = array();
                        for($i=0; $i<=15; $i++) {
                            $datetime = date('Y-m-d H:i:s', strtotime(date('Y-m-d H:i:s', strtotime($booking_date)) . " +$i day"));
                            if($datetime <= $allotment->end_date) {
                                $date = DateTime::createFromFormat('Y-m-d H:i:s', $datetime)->format('Y-m-d');
                                $available_package = $this->bookings_model->booking_packages($allotment_id, $datetime, $this->session->userdata('availability_screen_post_fields' . $allotment_id), true, $user_id, $allotment);
                                if($available_package) {
                                    $available_packages[$date] = $available_package;
                                }
                            }
                        }
                        if(empty($available_packages)) {
                            $error_message = lang('no_package');
                        }
                        Template::set('available_packages', $available_packages);
                    }
                }
            }
            
            // Handle POST validations on Availability Screen
            if($availability_screen) {
                $this->session->set_userdata('availability_screen_fields', $fields_form);
            } /*else {
            $this->session->unset_userdata('availability_screen_fields');
            } */
            
            // if(!$availability_screen && $has_availability_screen) { // not showing packages if there is no booking date
            if(!$availability_screen) {
                // get packages
                // Multisale - Allow user to buy the same package multiple times: 1 -> Allows, 0 -> Not Allowed
                $date_booking = isset($record->booking_date) && $allotment->multisale == 0 ? $record->booking_date : null; // creating problem if packages are set on alternate days like Monday, Tuesday, Wednesday. Showing all packages if multisale is set.
                $date_booking = isset($record->booking_date) ? $record->booking_date : null;
                $booking_packages = $this->booking_packages($allotment_id, $date_booking, $field_packages, false, $user_id, $allotment);
                if(empty($booking_packages)) {
                    // clear the availability session to allow user to select a date again.
                    $this->session->unset_userdata('availability_screen' . $allotment_id);
                    Template::set_message(lang('no_package'), 'danger');
                    Template::redirect(current_url());
                }
                
                $packages = array();
                $label_extra = array();
                $field_extra = array();
                $packages_available = false; // All packages are not booked for the day
                $default_selected = 0; // Package default selected
                foreach($booking_packages as $booking_package_id => $booking_package) {
                    $field_extra = array('id' => 'package_id_' . $booking_package_id);
                    if($booking_package['availability'] == 0) {
                        $field_extra['disabled'] = true;
                    }
                    $packages[] = array(
                        'name' => $booking_package['value'],
                        'value' => $booking_package_id,
                        'label_extra' => array('class' => $booking_package['availability'] == 1 ? 'badge btn-theme d-block' : 'badge badge-secondary d-block'),
                        'extra' => $field_extra
                    );
                    if(!$packages_available && $booking_package['availability'] == 1) {
                        $packages_available = true;
                        if($default_selected == 0) {
                            $default_selected = $booking_package_id;
                        }
                    }
                }
                
                if(!$packages_available) { // All packages are Booked
                    // clear the availability session to allow user to select a date again.
                    $this->session->unset_userdata('availability_screen' . $allotment_id);
                    Template::set_message(lang('no_package'), 'danger');
                    Template::redirect(current_url());
                }
                if(empty($record->package_id)) { // Do not set value in case of Edit
                    $record->package_id = $default_selected;
                }
                
                // display packages to the user
                // add packages field
                // add a new submit field to clear availablility screen values
                $field_clear_button[$package_field_key] = array(
                    'id' => 99999,
                    'type_id' => 0,
                    'type' => 'radio',
                    'name' => 'package_id',
                    'label' => 'gc_menu_packages',
                    'class' => '',
                    'placeholder' => '',
                    'tooltip' => '',
                    'tooltip_position' => '',
                    'label_lang' => 1,
                    'group' => 0,
                    'options' => $packages,
                    'options_lang' => 0
                );
                $fields_form = $this->array_insert_after($fields_form, $package_field_key, $field_clear_button);
            }
            if($user_id > 0) {
                $user_info = $this->db->select('display_name AS name, phone_number, email, address, postcode, city_id')->where('id', $user_id)->get('users')->row();
                if($user_info) {
                    foreach($user_info as $ukey => $uval) {
                        $record->{$ukey} = $uval;
                    }
                }
            }
            // Load Fields to display them to users
            $this->fields_model->load_fields('bookings', array(), 'frontend', $record, $id, FALSE, $fields_form);
        } else {
            Template::set_message($error_message, 'danger');
            if($this->session->userdata('last_visited_url')) {
                $last_visited_url = $this->session->userdata('last_visited_url');
                $this->session->unset_userdata('last_visited_url');
                Template::redirect($last_visited_url);
            } else {
                Template::redirect($redirect_url);
            }
        }
        
        if(isset($_POST['save'])) {
            $perform_save = true;
            // check advance and before duration settings
            if($check_advance_duration) {
                // if false, the form did not pass the advance duration rule validation, do not perform the save operation
                $perform_save = $this->calculate_advance_duration($allotment, $this->input->post('booking_date'));
            }
            if($perform_save) {
                $return = $this->fields_model->save_booking($form_fields, $user_id, $allotment_id, $form_id, 'member', $id, $type, $record);
                if($return) {
                    // clear the availability session to allow user to select a date again.
                    $this->session->unset_userdata('availability_screen' . $allotment_id);
                    $this->session->unset_userdata('availability_screen_fields');
                    
                    // pricing is a static field do not change it.
                    if($role != 'admin' && $this->input->post('pricing') > 0) {
                        // save tax, refund, allow_coupon and other information in a session and
                        // redirect user to the checkout page
                        $this->load->model('orders/orders_model');
                        $this->orders_model->cart($return, 0, 1, 1, 'bookings', false);
                        Template::redirect(gc_url_format('members/checkout'));
                    } else {
                        $message = lang('msg_info_saved');
                        if($role != 'admin' && $this->session->userdata('last_visited_url')) {
                            $message .= anchor($this->session->userdata('last_visited_url'), lang('bf_go_back'), 'class="btn btn-warning btn-xs ml-2"');
                        }
                        Template::set_message($message, 'success');
                        Template::redirect(current_url());
                    }
                }
            }
        }
        
        // Get Form Title
        $toolbar_title = '';
        $enable_cache = 0;
        $form = $this->db->where('id', $form_id)->get('booking_forms')->row();
        if($form) {
            $toolbar_title = $form->title;
            $enable_cache = $form->enable_cache;
        }

        // display module information in the title
        $module_name = gc_module_name($allotment->module_id);
        $module_row = $this->gc->get($module_name, array('where' => 'id = ' . $allotment->row_id, 'row' => 1));
        if($module_row) {
            $toolbar_title .= ' - ' . $module_row->title . ' (' . gc_module_translation($module_name) . ')';    
        }
        
        // store data in a session for populate fields
        $this->session->set_userdata('booking_form_fields', array(
            'allotment_id' => $allotment_id,
            'form_id' => $form_id,
            'enable_cache' => $enable_cache,
            'fields' => $booking_form_fields,
            'rules' => (array) json_decode($form->rules ?? '')
        ));
        if($this->auth->user_id()) {
            $this->load->model('users/user_model');
            $user = $this->user_model->get_user($this->auth->user_id());
            if($user) {
                $record->name = $user->display_name;
                $record->email = $user->email;
                $record->phone_number = $user->phone_number;
                $record->address = $user->address;
                $record->postcode = $user->postcode;
                $record->city_id = $user->city_id;
                $record->state_id = $user->state_id;
                $record->country_iso = $user->country_iso;
                $this->gc->location_google_places($record->city_id, true, true);
            }
        }
        Template::set(array(
            'availability_screen' => $availability_screen,
            'cancel_url' => $redirect_url,
            'allotment' => $allotment,
            'record' => $record,
            'has_booking_date' => $has_booking_date,
            'member_area' => true, // do not display image path on edit
            'has_availability_screen' => $has_availability_screen,
            'layout' => $availability_screen ? 'none' : $allotment->layout,
            'toolbar_title' => $toolbar_title,
            'error_message' => $error_message
        ));
        Template::set_view('bookings/booking_form');
        Template::render($render);
    }
    
    /**
     * Availability Screen Fields Validations
     * @return boolean
     */
    private function availability_screen_validations($allotment_id) {
        $this->load->library('form_validation');
        $availability_screen_fields = $this->session->userdata('availability_screen_fields');
        if(!empty($availability_screen_fields)) {
            $this->lang->load('fields/fields');
            $this->form_validation->set_rules('dummy', lang('label_dummy'), 'trim');
            foreach($availability_screen_fields as $availability_screen_field) {
                $this->form_validation->set_rules($availability_screen_field['name'], ($availability_screen_field['label_lang'] == 1 ? lang($availability_screen_field['label']) : $availability_screen_field['label']), (!empty($availability_screen_field['rules']) ? $availability_screen_field['rules'] . '|required' : 'required'));
            }
            if($this->form_validation->run() === TRUE) {
                $post_field_packages = array();
                foreach($availability_screen_fields as $post_field) {
                    if($post_field['name'] != 'booking_date') {
                        $post_field_packages[] = array(
                            'id' => $post_field['id'],
                            'dynamic_id' => isset($post_field['dynamic_id']) ? $post_field['dynamic_id'] : 0,
                            'value' => $this->input->post($post_field['name'])
                        );
                    }
                }
                if(!empty($post_field_packages)) {
                    $this->session->set_userdata('availability_screen_post_fields' . $allotment_id, $post_field_packages);
                } else {
                    $this->session->unset_userdata('availability_screen_post_fields' . $allotment_id);
                }
                return true;
            }
        }
        return false;
    }
    
    /**
     * Inserts any number of scalars or arrays at the point
     * in the haystack immediately after the search key ($needle) was found,
     * or at the end if the needle is not found or not supplied.
     * Modifies $haystack in place.
     * @param array &$haystack the associative array to search. This will be modified by the function
     * @param string $needle the key to search for: e.g. ''
     * @param mixed $stuff one or more arrays or scalars to be inserted into $haystack
     * @return int the index at which $needle was found
     */
    private function array_insert_after(&$haystack, $needle, $stuff){
        if (! is_array($haystack) ) return $haystack;
        
        $new_array = array();
        for ($i = 2; $i < func_num_args(); ++$i) {
            $arg = func_get_arg($i);
            if (is_array($arg)) $new_array = array_merge($new_array, $arg);
            else $new_array[] = $arg;
        }
        
        $i = 0;
        foreach($haystack as $key => $value){
            ++$i;
            if ($key == $needle) break;
        }
        
        $haystack = array_merge(array_slice($haystack, 0, $i, true), $new_array, array_slice($haystack, $i, null, true));
        return $haystack;
    }
    
    /**
     * Advance Duration Calculations
     *
     * @param object $allotment
     * @param string $booking_date
     * @return boolean
     */
    private function calculate_advance_duration($allotment, $booking_date) {
        // calculate difference between the today and the booking_date
        $today_date = date_create(date('Y-m-d'));
        $booking_date = date_create($booking_date);
        $diff = date_diff($today_date, $booking_date);
        $days_difference = 0;
        $advance_duration = $allotment->advance_duration;
        if(isset($diff->days) && $diff->invert == 0) {
            $days_difference = $diff->days;
        } else {
            Template::set_message(lang('msg_past_date'), 'danger');
            return false; // a user cannot book in a previous date
        }
        if($days_difference > 0 && $advance_duration > $days_difference) {
            // get number of seats booked in that day
            $consumed_seats = $this->get_available_seats($allotment->id, $booking_date);
            if($allotment->seats > $consumed_seats) {
                return true;
            } else {
                // Display no seat avaialble message for that day
                Template::set_message(sprintf(lang('msg_day_no_seats'), $booking_date), 'danger');
                return false;
            }
        } else {
            Template::set_message(lang('msg_invalid_date'), 'danger');
            return false;
        }
    }
    
    /**
     * Populate pricing packages dropdown fields
     */
    public function populate_field() {
        header('Content-Type: application/x-json; charset=utf-8');
        $field = $this->input->post('name');
        $value = $this->input->post('value');
        $booking_date = $this->input->post('booking_date');
        // get allotment id, form id and field information from the session variable
        $booking_form_fields = $this->session->userdata('booking_form_fields');
        $default_option = 0;
        if (empty($booking_form_fields) || !isset($booking_form_fields['fields'][$field])) {
            echo json_encode(array('success' => 0, 'default_option' => $default_option, 'message' => lang('msg_no_option')));
        }
        $allotment_id = $booking_form_fields['allotment_id'];
        $form_id = $booking_form_fields['form_id'];
        $field_id = $booking_form_fields['fields'][$field]['id'];
        $dynamic_id = $booking_form_fields['fields'][$field]['dynamic_id'];
        $enable_cache = $booking_form_fields['enable_cache'];
        $booking_rules = array();
        $rules = $booking_form_fields['rules'];
        $rule_keys = 0;
        if(!empty($rules)) {
            $rule_keys = implode('_', $rules);
        }
        
        // NOTES
        // if days selection is enabled, retrieve numeric value of day and perform join with the rules table with the value.
        // perform this operation on first dropdown selection to avoid unnecessary query.
        // get record only fulfill the condition e.g. if mon, where rules has 1 in the table.
        
        
        // Assumption - booking_date and pricing are hard-coded fields, do not change them.
        if(in_array(1, $rules) && !empty($booking_date)) {
            // 'D' modifier, for the textual representation in three letters
            // l for the full textual representation
            // w to get to number of the day (0 to 6, 0 being sunday, and 6 being saturday)
            // include only those pricing, where the value for this day in the booking_rules table is 1.
            
            // UNCOMMENT THE BELOW GIVEN CODE, IF YOU WANT TO INCLUDE BOOKING DATE IN THE DROPDOWN CHECK
            // NOTE: THIS WILL CREATE ISSUE WITH THE CHECK AVAILABILITY OPTION
            /*$booking_rules = array(
             'rule_ids' => 1,
             'values' => date('w', strtotime($booking_date))
             );*/
        }
        $cacheID = 'booking_' . $allotment_id . '_' . $form_id . '_' . $field_id . '_' . $dynamic_id . '_' . $rule_keys . '_' .  $field . '_' . $value . '_' . $booking_date;
        if($enable_cache == 1) {
            $cacheID = str_replace('-', '_', $this->slug->create_slug($cacheID));
            if(!$result = $this->gc_booking_cache->load($cacheID)) {
                $result = $this->populate_result($field, $value, $allotment_id, $form_id, $field_id, $dynamic_id, $booking_rules, $booking_form_fields);
                $this->gc_booking_cache->save($result, $cacheID, array('bookings_' . $allotment_id));
            }
        } else {
            $result = $this->populate_result($field, $value, $allotment_id, $form_id, $field_id, $dynamic_id, $booking_rules, $booking_form_fields);
        }
        if(isset($result['booking_form_fields'])) {
            $this->session->set_userdata('booking_form_fields', $result['booking_form_fields']);
        }
        echo json_encode($result);
    }
    
    /**
     * Populate Dropdown Fields on User Selection
     *
     * @param string $field
     * @param string $value
     * @param number $allotment_id
     * @param number $form_id
     * @param number $field_id
     * @param number $dynamic_id
     * @param array $booking_form_fields
     * @return number[]|string[]|array[]
     */
    private function populate_result($field, $value, $allotment_id, $form_id, $field_id, $dynamic_id, $booking_rules, $booking_form_fields) {
        // Default option is not required in case of Selectize dropdown because of placeholder
        $default_option = '<option value="">' . lang('label_select') . '</option>';
        $fields = $booking_form_fields['fields'];
        $where = array(
            'allotment_id' => $allotment_id,
            'form_id' => $form_id,
            'field_id' => $field_id,
            'dynamic_id' => $dynamic_id,
            'value' => $value
        );
        
        // store relations
        $field_hierarchy = array();
        foreach($fields as $field_relation) {
            $field_hierarchy[] = $field_relation['name'];
        }
        
        // get name of the next key
        $next_key = $this->get_next_key_array($fields, $field);
        if(!$next_key) {
            return array('success' => 0, 'default_option' => $default_option, 'message' => lang('msg_no_option'));
        }
        $next_field_id = $fields[$next_key]['id'];
        $next_field_dynamic_id = $fields[$next_key]['dynamic_id'];
        $next_field_name = $fields[$next_key]['name'];
        $next_field_options = $fields[$next_key]['options'];
        
        // get field information from the booking_fields table
        $record = $this->db->get_where($this->table_fields, $where)->row();
        $next_field_values = array();
        if ($record) { // get children
            $booking_form_fields['saved'][$field] = $record->id; // Important - save using field name as the key
            /*if (! isset($booking_form_fields['saved']) || isset($booking_form_fields['saved']) && ! in_array($record->id, $booking_form_fields['saved'])) {
            
            }*/
            // get information from the booking_field_relations table
            $relation_where = array(
                'allotment_id' => $allotment_id,
                'booking_field_id' => $record->id
            );
            if(!empty($booking_rules)) {
                // include only those pricing, where the row matches the booking_rules.
                $this->db->join('booking_rules BR', 'BR.package_id = ' . $this->table_field_relations . '.package_id AND BR.rule_id IN (' . $booking_rules['rule_ids'] .') AND BR.value IN (' . $booking_rules['values'] . ')', 'INNER');
            }
            $relations = $this->db->select($this->table_field_relations . '.id, ' . $this->table_field_relations . '.package_id')
            ->get_where($this->table_field_relations, $relation_where)
            ->result();
            $package_ids_new = array();
            if ($relations) {
                foreach ($relations as $relation) {
                    // Without Closure Table
                    /*$valid_parent = true;
                     $pricing_info = $this->db->where('package_id', $relation->package_id)->get($this->table_field_relations)->result();
                     if($pricing_info) {
                     foreach($pricing_info as $pricing_key => $pricing) {
                     if(isset($booking_form_fields['saved'][$field_hierarchy[$pricing_key]])) {
                     if($pricing->booking_field_id != $booking_form_fields['saved'][$field_hierarchy[$pricing_key]]) {
                     // Not a valid parent
                     $valid_parent = false;
                     }
                     }
                     }
                     }*/
                    
                    // With Closure Table
                    $valid_parent = true;
                    if(count($fields) > 3000) { // uncomment this code (entire IF) if having issues with packages dropdown fields
                        $parents = $this->closure->get_parent($relation->id);
                        if($parents) {
                            foreach($parents as $parent_key => $parent) {
                                if(!empty($field_hierarchy[$parent_key]) && isset($booking_form_fields['saved'][$field_hierarchy[$parent_key]]) && $parent->booking_field_id != $booking_form_fields['saved'][$field_hierarchy[$parent_key]]) {
                                    // Not a valid parent
                                    $valid_parent = false;
                                }
                            }
                        }
                    }
                    if ($valid_parent && ! in_array($relation->package_id, $package_ids_new)) {
                        $package_ids_new[] = $relation->package_id;
                    }
                }
            }
            
            if(!empty($package_ids_new)) {
                $values = $this->get_values($package_ids_new, $next_field_id, $next_field_dynamic_id);
                if (! empty($values)) {
                    foreach ($values as $next_field_row) { // // fill immediate dropdown field with the available values
                        $next_field_values[] = $next_field_row->value;
                    }
                }
            }
        }
        
        // prepare options - Normal Select Dropdown
        /*$options = array(
         '<option value="">' . lang('label_select') . '</option>'
         );*/
        $options = array();
        if (! empty($next_field_values) && ! empty($next_field_options)) {
            foreach ($next_field_options as $next_field_option) {
                if (in_array($next_field_option['value'], $next_field_values)) {
                    //$options[] = '<option value="' . $next_field_option['value'] . '">' . $next_field_option['name'] . '</option>';
                    $options[] = array(
                        'value' => $next_field_option['value'],
                        'text' => $next_field_option['name']
                    );
                }
            }
        }
        $result = array(
            'success' => 1,
            'default_option' => $default_option,
            'booking_form_fields' => $booking_form_fields
        );
        if(!empty($options)) {
            $result['field'] = $next_field_name;
            $result['options'] = $options;
        } else {
            $result['success'] = 0;
            $result['message'] = lang('msg_no_option');
        }
        return $result;
    }
    
    /**
     * Export Data
     * @param number $allotment_id
     * @param number $form_id - Allow Admin to Export Entire Form data
     * @param string $role
     */
    public function export_data($allotment_id, $form_id = 0, $role = 'admin', $redirect_url = null) {
        if(empty($redirect_url)) {
            if($role == 'admin') {
                $redirect_url = site_url(SITE_AREA . '/financial/bookings');
            } else {
                $redirect_url = gc_url_format('bookings/forms');
            }
        }
        if($form_id == 0) {
            $allotment = $this->verify_allotment($allotment_id, null, $redirect_url);
            $user_id = $this->auth->user_id();
            if($role != 'admin' && $allotment->user_id != $user_id) { // Not the owner of allotment
                Template::redirect($redirect_url);
            }
            $form_id = $allotment->form_id;
            $field_id = 'allotment_id';
            $id = $allotment_id;
        } else {
            $field_id = 'form_id';
            $id = $form_id;
        }
        $file_name = '';
        $form = $this->db->where('id', $form_id)->get('booking_forms')->row();
        if($form) {
            $file_name = $form->title;
        }
        // find allotments
        $this->db->join('users U', 'U.id = bookings.user_id', 'LEFT');
        $records = $this->db->select('U.email AS user_email, U.display_name, U.phone_number AS user_phone_number, bookings.*')->where($field_id, $id)->get('bookings')->result();
        if(!$records) {
            Template::set_message(lang('bf_model_no_data'), 'danger');
            Template::redirect($redirect_url);
        }
        $bookings = array();
        $field_names = array('id', 'user_id', 'username', 'user_email', 'user_phone_number');
        $extra_fields = false;
        foreach($records as $record) {
            if(!empty($record->form_data)) {
                $form_data = json_decode($record->form_data ?? '');
                $booking = array(
                    'id' => $record->id,
                    'user_id' => $record->user_id,
                    'username' => $record->display_name,
                    'user_email' => $record->user_email,
                    'user_phone_number' => $record->user_phone_number
                );
                foreach($form_data as $field) {
                    if($field->type == 'image') {
                        $booking[$field->label] = gc_image_url($field->value, false);
                    } else {
                        $booking[$field->label] = $field->value;
                    }
                    if(!in_array($field->label, $field_names)) {
                        $field_names[] = $field->label;
                    }
                }
                $booking['created_on'] = $record->created_on;
                $booking['created_ip'] = $record->created_ip;
                $booking['modified_on'] = $record->modified_on;
                $booking['modified_ip'] = $record->modified_ip;
                if(!$extra_fields) {
                    $field_names[] = lang('created_on');
                    $field_names[] = lang('created_ip');
                    $field_names[] = lang('modified_on');
                    $field_names[] = lang('modified_ip');
                    $extra_fields = true;
                }
                $bookings[] = $booking;
            }
        }
        if(!empty($bookings)) {
            $this->load->library('format');
            $data = array(
                'rows' => $bookings,
                'download' => true,
                'file_name' => $file_name,
                'field_names' => $field_names
            );
            $this->format->to_excel($data);
        }
    }
    
    /**
     * Verify Allotment ID
     *
     * @param number $allotment_id
     * @return number - Form ID
     */
    public function verify_allotment($allotment_id = 0, $return = 'form_id', $redirect_url = null) {
        $allotment_id = (int) $allotment_id;
        $allotment = $this->db->where('id', $allotment_id)
        ->get('booking_allotments')
        ->row();
        if (! $allotment) {
            Template::redirect(!empty($redirect_url) ? $redirect_url : gc_url_format('members'));
        }
        if($return == 'form_id') {
            return $allotment->form_id;
        } else {
            return $allotment;
        }
    }
    
    /**
     * Get next key in an array from given key in php
     *
     * @param array $array
     * @param string $key
     * @return mixed
     */
    private function get_next_key_array($array, $key) {
        $keys = array_keys($array);
        $position = array_search($key, $keys);
        if (isset($keys[$position + 1])) {
            $nextKey = $keys[$position + 1];
        }
        return isset($nextKey) ? $nextKey : false;
    }
    
    /**
     * Get Pricing Package Values
     * @param array $package_ids
     * @param number $next_field_id
     * @param number $next_field_dynamic_id
     * @return array
     */
    private function get_values($package_ids, $next_field_id, $next_field_dynamic_id) {
        $sql = "SELECT BF.value";
        $sql .= " FROM " . $this->db->dbprefix . $this->table_field_relations . " BFR";
        $sql .= " INNER JOIN " . $this->db->dbprefix . "booking_fields BF ON BF.id = BFR.booking_field_id AND BF.field_id = " . $next_field_id . " AND BF.dynamic_id = " . $next_field_dynamic_id;
        if (! empty($package_ids)) {
            $sql .= " WHERE BFR.package_id IN (" . implode(',', $package_ids) . ")";
        }
        $sql .= " GROUP BY BF.id";
        $result = $this->db->query($sql)->result();
        return $result;
    }
    
    /**
     * After successfull payment, confirm the booking and notify user.
     * @param string $order_id
     * @param array $order_info
     * @param boolean $order_updated - the user updated the booking order
     * set always false on transaction_confirmed method in the Payment_gateways_model
     * to avoid recalculations of commissions
     */
    public function confirm_order($order_id, $order_info = array(), $order_updated = false, $notify_customer = true) {
        if (empty($order_info)) {
            $order_info = $this->db->where(array('order_id' => $order_id, 'module_id' => gc_module_id('bookings')))
            ->get('order_items')
            ->row();
        }
        if (! empty($order_info)) {
            $this->session->unset_userdata('booking_id');
            
            // set booking status to active
            $this->db->where('id', $order_info->row_id)->update('bookings', array('status' => ($order_info->status_id == 2 ? 1 : 0)));
            
            // update available seats stock
            $this->update_seats_stock($order_id, $order_info, $order_updated);
            
            // send notification to the user
            if($notify_customer) {
                $this->notify($order_info->row_id);
            }
        }
    }
    
    /**
     * Update seats stock on Order Status change
     *
     * @param string $order_id
     * @param array $order_info: e.g. array()
     */
    public function update_seats_stock($order_id, $order_info, $order_updated) {
        if (empty($order_info)) {
            $order_info = $this->db->where(array('order_id' => $order_id, 'module_id' => gc_module_id('bookings')))
            ->get('order_items')
            ->row();
        }
        if (! empty($order_info)) {
            // get allotment info
            $booking = $this->db->where('id', $order_info->row_id)->get('bookings')->row();
            if(!$booking) {
                return false;
            }
            $allotment = $this->db->where('id', $booking->allotment_id)->get('booking_allotments')->row();
            $where = "allotment_id = " . $booking->allotment_id . " AND status = 1";
            if ($allotment && $allotment->seats > 0) {
                if ($allotment->seats_unit == 1) {
                    $where .= " AND DATE(created_on) = '" . date('Y-m-d') . "'";
                }
                
                // get consumed seats
                // do not count from the orders table, count from the bookings table
                // in case of daily limit is over, allow user to book for the next day.
                $consumed_seats = $this->db->select('SUM(id) AS count')->where($where)->get('bookings')->row();
                
                // update consumed seats in the booking allotments table
                if($order_info->status_id != 2 && $order_updated) { // The user cancelled the order
                    $seats_available = $allotment->seats + $consumed_seats->count;
                } else {
                    $seats_available = $allotment->seats - $consumed_seats->count;
                }
                $this->db->where('id', $order_info->row_id)->update('booking_allotments', array(
                    'seats_available' => $seats_available
                ));
            }
            /*$order_data = array(
             'seller_id' => $allotment->user_id
             );
            
             // Calculate Commission
             // commission already calculation on order confirmation
             // no need to calculate it again
             if(!$order_updated) {
             $payout_amount = $order_info->order_total; // Tax will be handled by the Seller
             $booking_form = $this->db->where('id', $booking->form_id)->get('booking_forms')->row();
             if($booking_form && $booking_form->commission > 0 && !empty($order_info->order_subtotal)) { // If Item Commission is set
             $booking_commission = $booking_form->commission;
             $commission_amount = $order_info->order_subtotal - $order_info->order_discount;
             $user_commission = ($commission_amount * $booking_commission) / 100;
             $payout_amount = ($commission_amount - $user_commission) + $order_info->order_tax;
             
             // Prepare Commissions Data
             $commission_data = array(
             'order_id' => $order_id,
             'module_id' => $order_info->module_id,
             'row_id' => $order_info->row_id,
             'user_id' => 0, // 0 - Admin
             'value' => $booking_commission,
             'amount' => $user_commission, // Currency and Status will be taken from the orders table
             'status' => $order_info->status_id == 2 ? 1 : 0 // Save commission with a status value
             );
             $this->db->insert('commissions', $commission_data);
             $order_data['order_commission'] = $user_commission;
             }
             
             // update order commission in the orders table to simplify queries
             // $this->db->where('order_id', $order_id)->update('orders', $order_data);
             /*
             if($order_info->status_id == 2) { // update payout only if an order is confirmed
             // save payouts
             $this->load->model('payouts/payouts_model');
             $this->payouts_model->update_payout($allotment->user_id, $order_info->module_id, $payout_amount, $order_info->status_id, true);
             }*/
            // }
        }
    }
    
    /**
     * Save Payouts on Order Status Change
     * @param string $order_id
     * @param array $order_info
     */
    public function save_payouts($order_id, $order_info = array()) {
        if (empty($order_info)) {
            $order_info = $this->db->where(array('order_id' => $order_id, 'module_id' => gc_module_id('bookings')))
            ->get('order_items')
            ->row();
        }
        if(!empty($order_info)) {
            // Get order commission
            $payout_amount = ($order_info->subtotal + $order_info->tax) - ($order_info->discount + $order_info->order_commission);
            $this->load->model('payouts/payouts_model');
            $this->payouts_model->update_payout($order_info);
        }
    }
    
    /**
     * Get consumed seats in a specific day
     * @param number $allotment_id
     * @param string $date
     * @return number
     */
    private function get_consumed_seats($allotment_id, $date) {
        $where = array(
            'allotment_id' => $allotment_id,
            'DATE(created_on)' => $date
        );
        // get consumed seats
        // do not count from the orders table, count from the bookings table
        // in case of daily limit is over, allow user to book for the next day.
        $consumed_seats = $this->db->select('SUM(id) AS count')->where($where)->get('bookings')->row();
        if($consumed_seats) {
            return $consumed_seats->count;
        } else {
            return 0; // no seat is booked for that day, return a zero value
        }
    }
    
    /**
     * Send notification (SMS/Email) to the User, Member and Admin
     *
     * @param number $booking_id
     * @param object $booking
     * @param string $type - null/email/sms
     */
    public function notify($booking_id, $booking = array(), $type = null, $queue_email = TRUE) {
        if(empty($booking)) {
            $this->db->join('users U', 'U.id = bookings.user_id', 'INNER');
            $booking = $this->db->select('U.display_name, U.email As user_email, U.phone_number AS user_phone_number, bookings.*')->where('bookings.id', $booking_id)->get($this->table_name)->row();
        }
        if(!empty($booking)) {
            $allotment_id = $booking->allotment_id;
            $form_id = $booking->form_id;
            
            // get email template from the booking_forms table
            $this->db->join('users U', 'U.id = booking_allotments.user_id', 'INNER');
            $this->db->join('booking_forms BF', 'BF.id = booking_allotments.form_id', 'INNER');
            $form = $this->db->select('U.display_name, U.email AS user_email, U.phone_number AS user_phone_number, booking_allotments.*, BF.title AS form_title, BF.template')->where('booking_allotments.id', $allotment_id)->get('booking_allotments')->row();
            if(!$form) {
                return false;
            }
            
            // get user email
            if(!isset($booking->user_email)) {
                $user = $this->db->select('display_name, email')->get('users')->row();
                if(!$user) {
                    return false;
                }
                if($user) {
                    $booking->user_email = $user->email;
                    $booking->display_name = $user->display_name;
                    $booking->user_phone_number = $user->user_phone_number;
                }
            }
            
            // get module information
            $form->module = '';
            $form->post_title = '';
            $form->post_url = '';
            if(!empty($form->module_id) && !empty($form->row_id)) {
                $module = gc_module_name($form->module_id);
                $form->module = $module;
                if(!empty($module)) {
                    $module_info = $this->db->where('id', $form->row_id)->get($module)->row();
                    if($module_info) {
                        $post_city = isset($module_info->city) ? $module_info->city : '';
                        $form->post_title = $module_info->title;
                        $form->post_url = gc_url_format($module, $module_info->slug, $module_info->id, $post_city);
                    }
                }
            }
            
            $this->load->library('parser');
            
            // prepare data
            $form_data = json_decode($booking->form_data ?? '');
            $content = array(
                'value_form_title' => $form->form_title,
                'value_module' => lang('singular_' . $form->module),
                'value_post_title' => $form->post_title,
                'value_post_url' => $form->post_url,
                'value_author_email' => $form->user_email,
                'value_site_title' => settings_item('site.title'),
                'value_site_url' => site_url(),
                'value_user_name' => $booking->display_name,
                'value_user_email' => $booking->user_email,
                'value_user_phone_number' => $booking->user_phone_number
            );
            if(!empty($form_data)) {
                foreach($form_data as $data) {
                    $content['label_' . $data->name] = $data->label;
                    if($data->type == 'image') {
                        $data->value = '<img src="' . gc_image_url($data->value) . '" alt="' . $data->label . '" />';
                    }
                    $content['value_' . $data->name] = $data->value;
                }
            }
            
            // get order_item details
            $order_item = $this->db->where(array('module_id' => gc_module_id('bookings'), 'row_id' => $booking->id))->get('order_items')->row();
            if($order_item) {
                foreach($order_item as $okey => $ovalue) {
                    switch($okey) {
                        case 'id':
                        case 'variant_id':
                        case 'relation_item_id':
                        case 'relation_item_variant_id':
                        case 'item':
                        case 'modified_on':
                        case 'modified_ip':
                        case 'modified_by':
                        case 'cronrun_on':
                        case 'deleted':
                            break;
                        case 'created_on':
                            $content['value_order_' . $okey] = gc_date_format($ovalue, true);
                            break;
                        default:
                            $content['value_' . $okey] = $ovalue;
                    }
                }
            }
            
            $form_data = json_decode($booking->form_data ?? '');
            $template_options = array('user', 'member', 'admin');
            $email_to = array(
                'user' => $booking->user_email,
                'member' => $form->user_email,
                'admin' => settings_item('site.system_email')
            );
            $sms_to = array(
                'user' => $booking->user_phone_number,
                'member' => $form->user_phone_number,
                'admin' => settings_item('site.phone_number')
            );
            $this->load->library('emailer/emailer');
            $template = json_decode($form->template ?? '');
            if(!empty($type)) {
                $types = array($type);
            } else {
                // Send email/sms using a single call to this method
                $types = array('email', 'sms');
            }
            
            foreach($types as $type) {
                if($type == 'email') {
                    // Send Email
                    foreach($template_options as $template_option) {
                        if(!empty($email_to[$template_option]) && !empty($template->email->{$template_option})) { // Proceed only if Email Exists
                            if($template_option == 'member') {
                                $template_subject = $template->admin->subject;
                                $template_message = $template->admin->message;
                            } else {
                                $template_subject = $template->{$template_option}->subject;
                                $template_message = $template->{$template_option}->message;
                            }
                            if($template_subject && $template_message) {
                                $subject_data = $content;
                                $subject_data['subject'] = $template_subject;
                                $subject = $this->parser->parse('email_templates/email_subject', $subject_data, TRUE);
                                $message_data = $content;
                                $message_data['message'] = $template_message;
                                $message = $this->parser->parse('email_templates/email_body', $message_data, TRUE); // TRUE is important to return string
                                $data = array(
                                    'to'		=> $email_to[$template_option],
                                    'subject'	=> $subject,
                                    'message'	=> $message
                                );
                                $this->emailer->send($data, $queue_email); // Send to email queue
                            }
                        }
                    }
                } else if($type == 'sms') { // Send SMS
                    foreach($template_options as $template_option) {
                        if(!empty($sms_to[$template_option]) && !empty($template->sms->{$template_option})) {
                            if($template_option == 'member') {
                                $template_sms = $template->admin->sms;
                            } else {
                                $template_sms = $template->{$template_option}->sms;
                            }
                            if(!empty($template_sms)) {
                                $message_data = $content;
                                $message_data['message'] = $template_sms;
                                $message = $this->parser->parse('email_templates/email_body', $message_data, TRUE);
                                $this->gc->send_sms($message, $sms_to[$template_option]);
                            }
                        }
                    }
                }
            }
        }
    }
    
    /**
     * Extra Email/SMS Template Fields
     * @return string[]
     */
    public function template_fields() {
        $fields = array(
            'form_title' => lang('form_title'),
            'module' => lang('label_module'),
            'post_title' => lang('label_post_title'),
            'post_url' => lang('label_post_url'),
            'author_email' => lang('label_author_email'),
            'site_title' => lang('label_site_title'),
            'site_url' => lang('label_site_url'),
            'user_name' => lang('label_user_name'),
            'user_email' => lang('label_user_email'),
            'user_phone_number' => lang('label_user_phone_number')
        );
        return $fields;
    }
    
    /**
     * Fill default Email/SMS templates on Email Create
     *
     * @param number $form_id
     * @param array $form
     */
    public function template_default($form_id, $form = array(), $fields = array()) {
        if(empty($form)) {
            $form = $this->db->where('id', $form_id)->get('booking_forms')->row();
        }
        if(!empty($form)) {
            // get fields
            if(empty($fields)) {
                $this->load->model('fields/fields_model');
                $fields = $this->fields_model->get_booking_fields(array(
                    'form_id' => $form_id
                ), false);
            }
            
            $user_table = '';
            $admin_table = '';
            if(!empty($fields)) {
                $admin_tr = '';
                $additional_fields = $this->template_fields();
                foreach($additional_fields as $additional_key => $additional_value) {
                    $admin_tr .= '<tr><td>' . $additional_value . '</td><td>{{value_' . $additional_key . '}}</td></tr>' . PHP_EOL;
                }
                $tr = '';
                foreach($fields as $field) {
                    $tr .= '<tr><td>{{label_' . $field['name'] . '}}</td><td>{{value_' . $field['name'] . '}}</td></tr>' . PHP_EOL;
                }
                $user_table = <<<EOL
<table>
{$tr}
</table>
EOL;

$admin_tr .= $tr;
$admin_table = <<<EOL
<table>
{$admin_tr}
</table>
EOL;
            }
            $form_title = $form->title;
            $template = array();
            $template['email']['user'] = 1;
            $template['email']['member'] = 1;
            $template['email']['admin'] = 1;
            $template['sms']['user'] = 1;
            $template['sms']['member'] = 1;
            $template['sms']['admin'] = 1;
            $template['user']['subject'] = 'Dear {{value_user_name}}, Thank you for registering with us.';
            $template['user']['message'] = <<<EOL
Dear {{value_user_name}},

<p>Thank you for registering for {{value_form_title}}. Your registration has been received, thanks!</p>

Details are as follows:<br />

{$user_table}
<br />
If you have any questions, feel free to contact at {{value_author_email}}.
<p>
Thanks,
<br /><br />
{{value_post_title}}<br />
{{value_post_url}}
</p>
EOL;
$template['user']['sms'] = <<<EOL
Dear {{value_user_name}},

Thank you for registering for {{value_form_title}}. Your registration has been received, thanks!

{{value_post_title}}
EOL;
$template['admin']['subject'] = 'A new user registered for {{value_form_title}} - {{value_post_title}}.';
$template['admin']['message'] = <<<EOL
Hello,

<p>A new user registerd for {{value_form_title}} - {{value_post_title}}.</p>

{$admin_table}
<p>
Thanks,
<br /><br />
{{value_site_title}}<br />
{{value_site_url}}
</p>
EOL;
$template['admin']['sms'] = <<<EOL
Hello,

A new user {{value_user_email}} registered for {{value_form_title}} - {{value_post_title}}.

{{value_site_title}}
EOL;
// Update Booking Form
return $this->db->where('id', $form_id)->update('booking_forms', array('template' => json_encode($template)));
        }
    }
    
    /**
     * Display Booking Form link, only if there are active bookings forms
     */
    public function booking_link() {
        $counts = $this->db->where('status', 1)->count_all_results('booking_forms');
        if($counts > 0) {
            Template::set('booking_forms', true);
        }
    }
    
    /**
     * A method to redirect user to the Booking Forms page
     *
     * This method will be called on a button click where the row_id will be passed
     *
     * The module_id, row_id will be saved in a session variable for further allotments
     *
     * Allotted forms ids will be also be saved in the session variable to display specific menu to users.
     *
     * @param string $module
     * @param number $row_id
     */
    public function view_forms($module, $row_id) {
        $row_id = (int) $row_id;
        $module_id = gc_module_id($module);
        $allotments = array();
        $bookings = array();
        $status = array();
        $where = array('module_id' => $module_id, 'row_id' => $row_id);
        // get module title to display it to the user
        $module_info = $this->db->where('id', $row_id)->get($module)->row();
        if(!$module_info) {
            return false;
        }
        $records = $this->db->where($where)->get('booking_allotments')->result();
        if($records) {
            foreach($records as $record) {
                // count bookings
                $bookings[$record->form_id]['inactive'] = $this->db->where(array('allotment_id' => $record->id, 'status' => 0))->count_all_results('bookings');
                $bookings[$record->form_id]['active'] = $this->db->where(array('allotment_id' => $record->id, 'status' => 1))->count_all_results('bookings');
                $allotments[$record->form_id] = $record->id;
                $status[$record->form_id] = $record->status; // Do not show form if the allotment is inactive
            }
        }
        $where['title'] = $module_info->title;
        $where['allotments'] = $allotments;
        $where['bookings'] = $bookings;
        $where['status'] = $status;
        $this->session->set_userdata('booking_module_row', $where);
        Template::redirect(gc_url_format('bookings/forms'));
    }
    
    /**
     * Get Form Info
     *
     * @param number $form_id
     * @param string $return - field to be returned
     */
    public function get_form_info($form_id, $return = null) {
        $result = $this->db->where('id', $form_id)->get('booking_forms')->row();
        if($result) {
            if(!empty($return)) {
                return $result->{$return};
            } else {
                return $result;
            }
        }
        return false;
    }
    
    /**
     * Save form settings
     * @param number $form_id
     * @param number $user_id
     */
    public function settings($form_id = 0, $allotment_id = 0, $role = 'member') {
        if($role == 'member') {
            $redirect_url = gc_url_format('members');
            $booking_module_row = $this->session->userdata('booking_module_row');
            $cancel_url = gc_url_format('bookings/forms');
        } else {
            $redirect_url = site_url(SITE_AREA . '/financial/bookings/allotments');
            $booking_module_row = (array) $this->db->where('id', $allotment_id)->get('booking_allotments')->row();
            $cancel_url = $redirect_url;
        }
        if(empty($booking_module_row)) {
            Template::redirect($redirect_url);
        }
        if(!empty($booking_module_row['user_id'])) {
            $user_id = $booking_module_row['user_id'];
            $form_id = $booking_module_row['form_id'];
        } else {
            $user_id = $this->auth->user_id();
        }
        $this->load->library('form_validation');
        $this->lang->load('listings/listings');
        $this->load->model('fields/fields_model');
        $this->lang->load('fields/fields');
        $form_id = (int) $form_id;
        if ($form_id == 0) {
            Template::redirect($redirect_url);
        }
        // check form existence - Valid Form ID
        $form = $this->get_form_info($form_id);
        if (! $form) {
            Template::redirect($redirect_url);
        }
        $form_fields = json_decode($form->form_fields ?? '');
        
        // DATE TIME PICKER
        if (! empty($form_fields->start_date) || ! empty($form_fields->end_date)) {
            Assets::add_js('moment.js');
            Assets::add_css('bootstrap-datetimepicker.min.css');
            Assets::add_js('bootstrap-datetimepicker.min.js');
            Assets::add_js("$('#start_date').datetimepicker({format: 'YYYY-MM-DD HH:mm:ss', sideBySide: true});", 'inline');
            Assets::add_js("$('#end_date').datetimepicker({format: 'YYYY-MM-DD HH:mm:ss', sideBySide: true});", 'inline');
        }
        
        $where = array(
            'module_id' => $booking_module_row['module_id'],
            'row_id' => $booking_module_row['row_id'],
            'user_id' => $user_id,
            'form_id' => $form_id
        );
        // save settings
        if (isset($_POST['save'])) {
            $this->gcrud->stats(array('module' => 'bookings'));
            $this->form_validation->set_rules('start_date', 'lang:label_start_date', 'trim|valid_datetime');
            $this->form_validation->set_rules('end_date', 'lang:label_end_date', 'trim|valid_datetime');
            $this->form_validation->set_rules('before_duration', 'lang:label_before_duration', 'trim|is_num');
            $this->form_validation->set_rules('advance_duration', 'lang:label_advance_duration', 'trim|is_num');
            $this->form_validation->set_rules('seats', 'lang:label_available_seats', 'trim|is_num');
            $this->form_validation->set_rules('seats_unit', 'lang:label_available_seats', 'trim');
            $this->form_validation->set_rules('seats_limit', 'lang:label_seats_limit', 'trim|is_num');
            $this->form_validation->set_rules('refund_duration', 'lang:label_refund', 'trim|is_num|required');
            $this->form_validation->set_rules('refund_unit', 'lang:label_refund', 'trim');
            $this->form_validation->set_rules('layout', 'lang:label_layout', 'trim');
            $this->form_validation->set_rules('tax_percent', 'lang:label_tax_percent', 'trim|is_num|max_val[100]|required');
            $this->form_validation->set_rules('tax_inclusive', 'lang:label_tax_inclusive', 'trim');
            $this->form_validation->set_rules('allow_coupon', 'lang:label_allow_coupon', 'trim');
            $this->form_validation->set_rules('multisale', 'lang:label_multisale', 'trim');
            $this->form_validation->set_rules('status', 'lang:label_status', 'trim');
            if ($this->form_validation->run() === true) {
                // check existence
                $data = $where;
                $data['start_date'] = $this->input->post('start_date') ? $this->input->post('start_date') : null;
                $data['end_date'] = $this->input->post('end_date') ? $this->input->post('end_date') : null;
                $data['before_duration'] = $this->input->post('before_duration') ? $this->input->post('before_duration') : 0;
                $data['advance_duration'] = $this->input->post('advance_duration') ? $this->input->post('advance_duration') : 0;
                $data['seats'] = $this->input->post('seats') ? $this->input->post('seats') : 0;
                $data['seats_unit'] = $this->input->post('seats_unit') ? $this->input->post('seats_unit') : 0;
                $data['seats_limit'] = $this->input->post('seats_limit') ? $this->input->post('seats_limit') : 0;
                $data['layout'] = $this->input->post('layout');
                $data['refund_duration'] = $this->input->post('refund_duration');
                $data['refund_unit'] = $this->input->post('refund_unit');
                $data['tax_percent'] = $this->input->post('tax_percent');
                $data['tax_inclusive'] = $this->input->post('tax_inclusive') ? 1 : 0;
                $data['allow_coupon'] = $this->input->post('allow_coupon') ? 1 : 0;
                $data['multisale'] = $this->input->post('multisale') ? 1 : (isset($form_fields->multisale) ? 0 : 1);
                $data['status'] = $this->input->post('status') ? 1 : 0;
                $row = $this->db->where($where)->get('booking_allotments')->row();
                if ($row) { // update
                    $data['modified_on'] = date('Y-m-d H:i:s');
                    $data['modified_ip'] = $this->input->ip_address();
                    $this->db->where('id', $row->id)->update('booking_allotments', $data);
                    $booking_module_row['status'][$row->id] = $data['status'];
                    $this->session->set_userdata('booking_module_row', $booking_module_row);
                } else if($role != 'admin') { // insert - Admin has a different form to perform allotment
                    $data['created_on'] = date('Y-m-d H:i:s');
                    $data['created_ip'] = $this->input->ip_address();
                    // $data['status'] = 1; // Enable by default
                    $this->db->insert('booking_allotments', $data);
                    $insert_id = $this->db->insert_id();
                    if($insert_id > 0) { // member area, new form added to allotments
                        $booking_module_row['allotments'][$form_id] = $insert_id;
                        $booking_module_row['status'][$form_id] = $data['status'];
                        $this->session->set_userdata('booking_module_row', $booking_module_row);
                    }
                }
                Template::set_message(sprintf(lang('msg_save_success'), lang('gc_menu_settings')), 'success');
                // Template::redirect(current_url());
            }
        }
        
        $record = $this->db->get_where('booking_allotments', $where)->row();
        Template::set(array(
            'role' => $role,
            'cancel_url' => $cancel_url,
            'form_fields' => $form_fields,
            'record' => $record,
            'toolbar_title' => lang('label_form') . ' - ' . $form->title
        ));
        Template::set_view('bookings/settings');
    }
    
    /**
     * Display Form Fields and Pricing packages to the User
     *
     * @param number $allotment_id
     */
    public function fields($allotment_id = 0, $role = 'member') {
        if($role == 'member') {
            $redirect_url = gc_url_format('members');
        } else {
            $redirect_url = site_url(SITE_AREA . '/financial/bookings/allotments');
        }
        $allotment = $this->bookings_model->verify_allotment($allotment_id, null);
        $form_id = $allotment->form_id;
        if ($form_id == 0) {
            Template::redirect($redirect_url);
        }
        clear_cache('booking_fields_' . $allotment_id); // Clear Booking Fields Cache
        clear_cache('booking_has_availability_screen_' . $form_id); // Clear Availability Screen Cache
        clear_cache('booking_packages_' . $allotment_id); // Clear allotment specific packages cache
        clear_cache('booking_packages_using_fields_' . $allotment_id); // Clear check availability packages
        
        $this->lang->load('fields/fields');
        $booking_module_id = gc_module_id('bookings');
        
        // Status Change in the field_modules table.
        // the user cannot change the status of the existing fields.
        // it is difficult to manage allotment fields, a booking form can have thousands of allotments.
        // change in any form field will need updation of these allotments manually.
        // is_extra and status fields settings are kept in the field_modules only.
        /*if (isset($_POST['status_change'])) {
         $sql = "UPDATE " . $this->db->dbprefix . "booking_allotment_fields SET status = NOT status WHERE allotment_id = " . $allotment_id . " AND field_id = " . $_POST['status_change'];
         $this->db->simple_query($sql);
         Template::redirect(current_url());
         }
         
         // Allow user to Add a Field on the Booking selection form
         if (isset($_POST['is_extra'])) {
         $sql = "UPDATE " . $this->db->dbprefix . "booking_allotment_fields SET is_extra = NOT is_extra WHERE allotment_id = " . $allotment_id . " AND field_id = " . $_POST['is_extra'];
         $this->db->simple_query($sql);
         Template::redirect(current_url());
         }*/
        $this->load->model('fields/fields_model');
        $records = $this->fields_model->booking_form_fields($form_id);
        Assets::add_js(array(
            'jquery.dataTables.min',
            'dataTables.bootstrap.min'
        ));
        Assets::add_css('dataTables.bootstrap.min');
        Assets::add_js("$('#booking_fields').DataTable({\"order\": [[ 1, 'desc' ]]});", 'inline');
        Assets::add_js("$('#pricing_packages').DataTable();", 'inline');
        
        // show saved pricing to the user
        $pricing_records = array();
        
        // get field names with options
        $this->load->model('fields/fields_model');
        $booking_fields = $this->fields_model->get_booking_fields(array(
            'allotment_id' => $allotment_id,
            'form_id' => $form_id
        ));
        
        // get information from the booking_field_relations table
        $relations = $this->db->where('allotment_id', $allotment_id)
        ->get('booking_field_relations')
        ->result();
        if ($relations) {
            // get all saved information from the booking_fields table
            $result = $this->db->where(array(
                'allotment_id' => $allotment_id,
                'type_id' => 2
            ))
            ->get('booking_fields')
            ->result();
            $pricing_info = array();
            if ($result) {
                foreach ($result as $row) {
                    $pricing_info[$row->id] = array(
                        'field_id' => $row->field_id,
                        'dynamic_id' => $row->dynamic_id,
                        'value' => $row->value
                    );
                }
            }
            
            // make columns
            foreach ($relations as $relation) {
                if(isset($pricing_info[$relation->booking_field_id])) {
                    // get field_id, dynamic_id and value from the pricing_info variable
                    $info = $pricing_info[$relation->booking_field_id];
                    $field_name = $info['field_id'];
                    if ($info['dynamic_id'] != 0) {
                        $field_name = $info['field_id'] . '_' . $info['dynamic_id'];
                    }
                    $pricing_records[$relation->package_id][$field_name] = $info['value'];
                    $pricing_records[$relation->package_id]['allotment_package_id'] = $relation->allotment_package_id;
                }
            }
        }
        
        // delete pricing information
        if (! empty($_POST['delete'])) {
            $this->bookings_model->purge_pricing($_POST['delete']);
            Template::set_message(sprintf(lang('msg_action_success'), lang('label_package'), lang('label_deleted')), 'success');
            Template::redirect(current_url());
        }
        $title = $this->bookings_model->get_form_info($form_id, 'title');
        Template::set(array(
            'role' => $role,
            'allotment' => $allotment,
            'allotment_id' => $allotment_id,
            'records' => $records,
            'module_links' => '<li class="nav-item mt-1">' . anchor(gc_url_format('bookings/forms'), lang('cancel'), 'class="btn btn-warning btn-xs"') . '</li>',
            'fields' => $booking_fields,
            'pricing_records' => $pricing_records,
            'toolbar_title' => $title . ' - ' .lang('gc_menu_fields')
        ));
        Template::set_view('bookings/booking_fields');
    }
    
    /**
     * Allow user to add options
     *
     * A single table is used to store options, dynamic field relations and pricing.
     * the field type_id is used to differentiate them.
     *
     * @param number $form_id
     * @param number $field_id
     * @param number $type_id
     *            = 0 -> Options, 1 -> Dynamic Fields, 2 - Pricing/Packages
     */
    public function options($allotment_id = 0, $field_id = 0, $type_id = 0, $role = 'member') {
        if($role == 'member') {
            $redirect_url = gc_url_format('members');
            $cancel_url = gc_url_format('bookings/fields') . '/' . $allotment_id;
        } else {
            $redirect_url = site_url(SITE_AREA . '/financial/bookings/fields/' . $allotment_id);
            $cancel_url = $redirect_url;
        }
        $form_id = $this->bookings_model->verify_allotment($allotment_id);
        $field_id = (int) $field_id;
        $type_id = (int) $type_id;
        if ($field_id == 0) {
            Template::redirect($redirect_url);
        }
        $this->lang->load('fields/fields');
        $booking_module_id = gc_module_id('bookings');
        $where = array(
            'allotment_id' => $allotment_id,
            'form_id' => $form_id,
            'field_id' => $field_id,
            'type_id' => $type_id
        );
        // Save field options
        if (isset($_POST['save'])) {
            $this->bookings_model->save_options($where);
            Template::set_message(sprintf(lang('msg_save_success'), lang('label_options')), 'success');
            Template::redirect(current_url());
        }
        // check field type, it must be booking and field must exist in this form
        $this->db->join('field_relations FR', 'FR.field_id = field_modules.field_id AND FR.module_id = ' . $booking_module_id, 'INNER');
        $row = $this->db->where(array(
            'field_modules.module_id' => $booking_module_id,
            'field_modules.row_id' => $form_id,
            'field_modules.field_id' => $field_id,
            'field_modules.type' == 'booking'
        ))->group_by('field_modules.field_id')->get('field_modules')->row();
        if (! $row) {
            Template::redirect($redirect_url);
        }
        $field_label = $row->label_lang == 1 ? lang($row->label) : $row->label;
        $values = $this->db->get_where('booking_fields', $where)->result();
        Template::set(array(
            'row' => $row,
            'field_label' => $field_label,
            'type_id' => $type_id,
            'values' => $values,
            'cancel_url' => $cancel_url,
            'toolbar_title' => $type_id == 0 ? sprintf(lang('manage_options'), $field_label) : lang('label_split') . ' - ' . $field_label
        ));
        Template::set_view('bookings/options');
    }
    
    /**
     * Allow user to add/edit pricing form
     *
     * @param number $id
     *            = 0 -> Insert, Value -> Edit
     */
    public function packages($allotment_id, $id = 0, $role = 'member') {
        if($role == 'member') {
            $cancel_url = gc_url_format('bookings/fields') . '/' . $allotment_id;
            $redirect_url = $cancel_url;
        } else {
            $redirect_url = site_url(SITE_AREA . '/financial/bookings/allotments');
            $cancel_url = $redirect_url;
        }
        $form_id = $this->bookings_model->verify_allotment($allotment_id);
        $id = (int) $id;
        if ($form_id == 0) {
            Template::redirect($redirect_url);
        }
        
        // check packages limit
        $this->lang->load('fields/fields');
        $form = $this->db->where('id', $form_id)->get('booking_forms')->row();
        if(!$form) {
            Template::redirect($redirect_url);
        }
        $max_packages = (int) $form->max_packages;
        $packages_count = (int) $this->bookings_model->packages_count($allotment_id);
        if(empty($id) && $packages_count >= $max_packages && empty($id)) {
            Template::set_message(sprintf(lang('msg_limit_reached'), $max_packages, $form->title), 'danger');
            Template::redirect($cancel_url);
        }
        
        $where = array(
            'allotment_id' => $allotment_id,
            'form_id' => $form_id
        );
        $this->load->model('fields/fields_model');
        $fields = $this->fields_model->get_booking_fields($where, true, false);
        // $fields = $this->get_booking_fields($where);
        if (isset($_POST['save'])) {
            if($this->bookings_model->save_pricing($where, $fields, $id)) {
                clear_cache('bookings_' . $allotment_id); // Clear specific cache
                clear_cache('booking_packages_' . $allotment_id); // Clear allotment specific packages cache
                clear_cache('booking_packages_using_fields_' . $allotment_id); // Clear check availability packages
                Template::redirect($cancel_url);
            }
        }
        
        // Display record on edit
        $record = array();
        if (! empty($id)) {
            $record['id'] = $id;
            $record_where = $where;
            foreach($record_where as $record_where_key => $record_where_value) {
                $record_where['BF.' . $record_where_key] = $record_where_value;
                unset($record_where[$record_where_key]);
            }
            $record_where['package_id'] = $id;
            $this->db->join($this->table_fields . ' BF', 'BF.id = booking_field_relations.booking_field_id AND BF.type_id = 2', 'INNER');
            $result = $this->db->where($record_where)->get('booking_field_relations')->result();
            if ($result) {
                foreach ($result as $row) {
                    foreach ($fields as $field_key => $field) {
                        // $fields[$field_key]['extra'] = array('disabled' => true);
                        if(isset($field['dynamic_id'])) {
                            if ($field['dynamic_id'] != 0 && $field['dynamic_id'] == $row->dynamic_id) {
                                $record[$field['name']] = $row->value;
                            } else if ($field['id'] == $row->field_id && $field['dynamic_id'] == 0) {
                                $record[$field['name']] = $row->value;
                            }
                        } else if ($field['id'] == $row->field_id) {
                            $record[$field['name']] = $row->value;
                        }
                    }
                }
                // get rules/days if exist
                $saved_rules = array();
                $package_saved_rules = $this->db->get_where('booking_rules', array('package_id' => $id))->result();
                if($package_saved_rules)  {
                    foreach($package_saved_rules as $package_saved_rule) {
                        $saved_rules[$package_saved_rule->rule_id][] = $package_saved_rule->value;
                    }
                    $record['rules'] = $saved_rules;
                }
            }
        }
        Template::set(array(
            'form' => $form,
            'record' => $record,
            'fields' => $fields,
            'cancel_url' => $cancel_url,
            'toolbar_title' => lang('gc_menu_packages')
        ));
        Template::set_view('bookings/packages');
    }
    
    /**
     * Get all allotment IDs with Form Title to display it to the user
     *
     * @param number $module_id
     * @param number $row_id
     */
    public function get_allotment_ids($module_id, $row_id) {
        $allotments = array();
        $this->db->join('booking_forms BF', 'BF.id = booking_allotments.form_id', 'INNER');
        $records = $this->db->select('booking_allotments.id, booking_allotments.form_id, BF.title')->where(array('module_id' => $module_id, 'row_id' => $row_id, 'booking_allotments.status' => 1, 'BF.status' => 1))->get('booking_allotments')->result();
        if($records) {
            foreach($records as $record) {
                $allotments[$record->id] = $record->title;
            }
        }
        return $allotments;
    }
    
    /**
     * Display a list of bookings data to the User, Admin and the Business Owner
     *
     * @return void
     */
    public function view($user_id, $type = 0, $allotment_id = 0, $limit = 25, $offset = 0, $role = 'member') {
        if($role == 'admin') {
            $uri_segment = 5; // offset URI segment
            $redirect_uri = '/financial/bookings/index';
            $base_url = site_url(SITE_AREA . $redirect_uri);
        } else {
            $uri_segment = 5;
            $base_url = gc_url_format('bookings/view') . '/' . $type . '/' . $allotment_id;
            $redirect_uri = $base_url;
        }
        
        $where = '';
        
        // advanced filter
        if($role == 'member') {
            $advanced_filter = array('module_id', 'date_from', 'date_to');
        } else {
            $advanced_filter = array('user_id', 'module_id', 'date_from', 'date_to');
        }
        
        // location filter
        Assets::add_css('selectize.bootstrap3.css');
        Assets::add_js('selectize.min.js');
        Assets::add_js('selectize-search.js');
        Assets::add_js('selectize-location.js');
        $extra = $this->gcrud->advanced_filter(array('advanced_filter' => $advanced_filter, 'advanced_filter_date' => true), 'bookings', 'B');
        if(!empty($extra['query_where'])) {
            $where = $extra['query_where'];
        }
        
        $searchterm = $this->gc->search('bookings', $this->uri->segment($uri_segment), $redirect_uri);
        if (! empty($searchterm)) {
            $where = "(B.id LIKE '%" . $searchterm . "%' OR B.form_data LIKE '%" . $searchterm . "%' OR B.created_on LIKE '%" . $searchterm . "%')";
        }
        $pagerUriSegment = $uri_segment;
        $pagerBaseUrl = $base_url . '/';
        
        // Order By
        $sort_by = "B.id desc";
        $order_by = $this->gc->sort('bookings');
        if(!empty($order_by)) {
            $sort_by = $order_by;
        }
        $total_rows = $this->get_bookings(1, $user_id, $type, $allotment_id, $where, $sort_by, $limit, $offset);
        $this->load->library('pagination');
        $pager['base_url'] = $pagerBaseUrl;
        $pager['total_rows'] = $total_rows;
        $pager['per_page'] = $limit;
        $pager['uri_segment'] = $pagerUriSegment;
        
        $this->pagination->initialize($pager);
        $records = $this->get_bookings(0, $user_id, $type, $allotment_id, $where, $sort_by, $limit, $offset);
        if($role != 'admin' && $type == 1) {
            Template::set('module_links', '<li class="nav-item mt-1">' . anchor(gc_url_format('bookings/forms'), lang('cancel'), 'class="btn btn-warning btn-xs"') . '</li>');
        }
        Template::set(array(
            'type' => $type,
            'role' => $role,
            'records' => $records,
            'total_rows' => $total_rows,
            'toolbar_title' => lang('gc_menu_bookings')
        ));
        Template::set_view('bookings/view');
    }
    
    /**
     * Get Bookings
     *
     * @param number $count
     * @param number $user_id
     * @param number $type - 0 - User ID, 1 - Owner ID
     * @param number $limit
     * @param number $offset
     * @return object|number
     */
    public function get_bookings($count, $user_id, $type, $allotment_id, $where, $sort_by, $limit = 0, $offset = 0) {
        $module_id = gc_module_id('bookings');
        if($count == 0) {
            $sql = "SELECT B.*, BF.title, U.display_name, U.email AS user_email, U.slug AS user_slug, U.city AS user_city, BA.module_id, BA.refund_duration, BA.refund_unit, O.qty, ";
            $sql .= $this->gc->module_query(array(), 'BA', false);
        } else {
            $sql = "SELECT COUNT(B.id) AS total_count";
        }
        $sql .= " FROM " . $this->db->dbprefix . "bookings B";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "booking_forms BF ON BF.id = B.form_id";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "users U ON U.id = B.user_id";
        $sql .= " INNER JOIN " . $this->db->dbprefix . "booking_allotments BA ON BA.id = B.allotment_id";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "order_items O ON O.row_id = B.id AND O.module_id = " . $module_id;
        $sql .= " WHERE B.deleted = 0";
        if(!empty($user_id)) { // in case of Admin, user_id will be zero
            if($type == 0) {
                $sql .= " AND B.user_id = " . $user_id;
            } else {
                $sql .= " AND BA.user_id = " . $user_id;
            }
        }
        if(!empty($where)) {
            $sql .= " AND " . $where;
        }
        if(!empty($allotment_id)) {
            $sql .= " AND B.allotment_id = " . $allotment_id;
        }
        if($count == 0) {
            $sql .= " GROUP BY B.id ORDER BY " . $sort_by;
            if ($limit != 0) {
                $sql .= " LIMIT " . $offset . ", " . $limit;
            } else {
                $sql .= " LIMIT $limit";
            }
            $query = $this->db->query ( $sql );
            return $query->result();
        } else {
            $result = $this->db->query ( $sql )->row();
            return $result ? $result->total_count : 0;
        }
    }
    
    /**
     * Get Payouts to display it to the USER or ADMIN
     *
     * @param number $count
     * @param number $allotment_id
     * @param number $form_id
     * @param number $limit
     * @param number $offset
     * @param boolean $sum - If true, return total sum
     * @return number|object
     */
    public function get_payouts($count = 0, $allotment_id = 0, $form_id = 0, $limit = 25, $offset = 0, $sum = true) {
        if($count == 0) {
            $sql = "SELECT B.id, O.order_total, O.order_subtotal, O.order_tax, O.order_discount, O.order_commission";
        } else {
            $sql = "SELECT COUNT(B.id) AS total_count";
        }
        $sql .= " FROM (SELECT id FROM " . $this->db->dbprefix . "bookings WHERE status = 1";
        if(!empty($allotment_id)) {
            $sql .= " AND allotment_id = " . $allotment_id;
        }
        if(!empty($form_id)) {
            $sql .= " AND form_id = " . $form_id;
        }
        $sql .= " ) B";
        $sql .= " INNER JOIN " . $this->db->dbprefix . "order_items OT ON OT.row_id = B.id AND OT.module_id = " . gc_module_id('bookings') . " AND OT.status_id = 1";
        $sql .= " INNER JOIN " . $this->db->dbprefix . "orders O ON O.order_id = OT.order_id";
        if($count == 0) {
            if ($limit != 0) {
                $sql .= " LIMIT " . $offset . ", " . $limit;
            } else if(!empty($limit)) {
                $sql .= " LIMIT $limit";
            }
            $result = $this->db->query ( $sql )->result();
            if($sum) {
                if($result) {
                    $payout_amount = 0;
                    $payout_commissions = 0;
                    foreach($result as $row) {
                        $payout_amount = $payout_amount + ($row->order_subtotal + $row->order_tax) - ($row->order_discount + $row->order_commission);
                        $payout_commissions = $payout_commissions + $row->order_commission;
                    }
                    return array('payouts' => $payout_amount, 'commissions' => $payout_commissions);
                } else {
                    return 0;
                }
            } else {
                return $result;
            }
        } else {
            $result = $this->db->query ( $sql )->row();
            return $result ? $result->total_count : 0;
        }
    }
    
    /***********************************************
     * START OF CHECK AVAILABILITY LOGIC
     ***********************************************/
    /**
     * Booking Pacakges/Check availability using Booking Date
     * @param number $allotment_id
     * @param array $fields
     * @param boolean $available_only - Package Available Only
     * @param object $allotment - seat specific counts
     */
    public function booking_packages($allotment_id, $booking_date, $fields = array(), $available_only = false, $user_id = 0, $allotment = null) {
        if(!$allotment) {
            $allotment = $this->verify_allotment($allotment_id, null);
        }
        // Do not delete Booking Date and Pricing fields, their name is hardcoded at different places
        // pricing value will always be used during checkout, do not allow split of the pricing and Booking Date field.
        // Show this warning in the Admin Panel.
        
        // 0 is for Sunday, 99 is used to deal with No Booking Date
        $booking_day = !empty($booking_date) ? date('w', strtotime($booking_date)) : 99;
        $cache_field_values = array();
        if(!empty($fields)) {
            foreach($fields as $cache_field_value) {
                $cache_field_values[] = $cache_field_value['value'];
            }
        }
        $cacheID = 'booking_packages_' . $allotment_id . '_' . $booking_day . '_' . implode('_', $cache_field_values) . '_' . $available_only;
        $cacheID = str_replace('-', '_', $this->slug->create_slug($cacheID));
        
        if(!$packages = $this->gc_booking_cache->load($cacheID)) {
            $packages_raw = array();
            if(empty($fields)) { 
                $packages_raw = $this->get_packages_without_fields($allotment_id, $booking_day);
            } else {
                // Search using the Input Values
                $field_values = $this->field_having_values($allotment_id, $fields);
                $field_ids = array();
                if(!empty($field_values)) {
                    foreach($field_values as $field_value) {
                        $field_ids[] = $field_value->id;
                    }
                    array_unique($field_ids);
                    // Get Package IDs
                    // Note: Dynamic ID is not considered here
                    if(!empty($field_ids)) {
                        // Get package IDs having input fields in common
                        $package_ids_using_fields = $this->get_package_ids_using_fields($allotment_id, $field_ids);
                        if(!empty($package_ids_using_fields)) {
                            $package_ids = array();
                            foreach($package_ids_using_fields as $package_row) {
                                $package_ids[] = $package_row->package_id;
                            }
                            // retrieve fields information using package ids
                            $packages_raw = $this->get_field_info_using_package_ids($package_ids, $booking_day);
                        }
                    }
                }
            }
            
            // convert packages into radio fields
            // Note: the user can select only one package at a time.
            $packages_processed = array();
            $unset_field_ids = array(); // Field to be unset from the package row
            if(!empty($fields)) {
                foreach($fields as $field_val) {
                    $unset_field_ids[] = $field_val['id'];
                }
                array_unique($unset_field_ids);
            }
            $packages = array();
            if(!empty($packages_raw)) {
                foreach($packages_raw as $package_info) {
                    if(!array_key_exists($package_info->package_id, $packages_processed)) {
                        $packages_processed[$package_info->package_id] = null;
                    }
                    //$package_label = $packages_processed[$package_info->package_id]['label'];
                    // Note: unset value using Dynamic ID
                    if(!empty($unset_field_ids) && in_array($package_info->field_id, $unset_field_ids)) {
                        // do not include this value in package preparation
                        // do nothing
                    } else {
                        if($package_info->field_name == 'pricing') {
                            $packages_processed[$package_info->package_id]['label'][$package_info->field_id] = '<span class="d-block bg-danger text-white lead p-1 mt-1">'. currency_format($package_info->value) . '</span>';
                        } else {
                            if(!empty($packages_processed[$package_info->package_id]['label'][$package_info->field_id])) {
                                $packages_processed[$package_info->package_id]['label'][$package_info->field_id] = $packages_processed[$package_info->package_id]['label'][$package_info->field_id] . ' - ' . $package_info->value;
                            } else {
                                $packages_processed[$package_info->package_id]['label'][$package_info->field_id] = $package_info->value;
                            }
                        }
                    }
                }
                foreach($packages_processed as $package_id => $package_values) {
                    $packages[$package_id]['value'] = implode('<br />', $packages_processed[$package_id]['label']);
                    $packages[$package_id]['availability'] = 1;
                }
            }
            $packages = array('packages' => $packages); // do not re-run query if empty data is returned
            $this->gc_booking_cache->save($packages, $cacheID, array('booking_packages_' . $allotment_id));
        }
        $packages = isset($packages['packages']) ? $packages['packages'] : array();
        
        // SAVE packages VARIABLE IN CACHE TO AVOID UNNECESSARY LOOPING
        // DELETE cache ON ALLOTMENT PACKAGE CHANGE
        
        // NOTE: WARN USER ON PACKAGE DELETE TO AVOID ANY AMBIGUITY
        // CACHE EXCLUDED
        // Get existing booked package of the selected booking date
        if(!empty($booking_date) && !empty($packages)) {
            $d = DateTime::createFromFormat('Y-m-d', $booking_date);
            // The Y ( 4 digits year ) returns TRUE for any integer with any number of digits so changing the comparison from == to === fixes the issue.
            if($d && $d->format('Y-m-d') === $booking_date) {
                $booking_date_only = $booking_date;
            } else {
                $booking_date_only = DateTime::createFromFormat('Y-m-d H:i:s', $booking_date)->format('Y-m-d');
            }
            // get booked seats on that specific day
            $this->get_booked_seats($allotment_id, 2, $user_id, "DATE(OT.created_on)='" . $booking_date_only . "'");
            $booked_packages = $this->get_package_ids_from_bookings($allotment_id, $booking_date_only);
            if($booked_packages) {
                foreach($booked_packages as $booked_package) {
                    if(array_key_exists($booked_package->package_id, $packages)) {
                        // Package already booked
                        // change availability to 0
                        // check booked limit today by the same user
                        $booked_seats = $this->get_booked_seats($allotment_id, 2, $user_id, "DATE(B.booking_date)='" . $booking_date_only . "'");
                        if($allotment->seats_limit != 0 && $allotment->seats_limit <= $booked_seats) {
                            if($available_only) {
                                unset($packages[$booked_package->package_id]);
                            } else {
                                $packages[$booked_package->package_id]['availability'] = 0;
                            }
                        }
                    }
                }
            }
        }
        return $packages;
    }
    
    /**
     * Get Package IDs only if the starting screen of the Booking Form has Booking Date without fields
     * @param number $allotment_id
     * @param number $day - 0 -> Sunday, 99 - No Value Specified
     * @return array
     */
    public function get_packages_without_fields($allotment_id, $day = 99) {
        $sql = "SELECT BFR.package_id, BFV.field_id, F.name AS field_name, BF.dynamic_id, BFV.value";
        $sql .= " FROM " . $this->db->dbprefix . "booking_field_relations BFR";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "booking_fields BF ON BF.id = BFR.booking_field_id";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "fields F ON F.id = BF.field_id";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "booking_fields BFV ON BFV.id = BF.value";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "booking_rules BR ON BR.package_id = BFR.package_id";
        if(!empty($allotment_id) && $day != 99) {
            $sql .= " WHERE BR.value = " . $day . " AND BFR.allotment_id = " . $allotment_id;
        } else if(!empty($allotment_id)) {
            $sql .= " WHERE BFR.allotment_id = " . $allotment_id;
        } else if($day != 99) {
            $sql .= " WHERE BR.value = " . $day;
        }
        $sql .= " GROUP BY BFR.id ORDER BY BFV.field_id DESC";
        $result = $this->db->query($sql)->result();
        return $result;
    }
    
    /**
     * Get id and value of the previously input fields
     * matching single field against multiple values
     * @param number $allotment_id
     * @param array $fields
     * @return boolean
     */
    public function field_having_values($allotment_id, $fields = array()) {
        if(empty($fields)) {
            return false;
        }
        $sql = "SELECT RBF.id, RBF.value, RBF.dynamic_id";
        $sql .= " FROM " . $this->db->dbprefix . "booking_fields RBFR";
        $sql .= " INNER JOIN " . $this->db->dbprefix . "booking_fields RBF ON RBF.value = RBFR.id";
        $sql .= " WHERE (";
        // Get array keys
        $arrayKeys = array_keys($fields);
        // Fetch last array key
        $lastArrayKey = array_pop($arrayKeys);
        //iterate array
        foreach($fields as $field_id => $field) {
            $sql .= " (RBFR.field_id = " . $field['id'] . " AND RBF.value = '" . $field['value'] . "')";
            if($field_id != $lastArrayKey) { // Not the Last Element of the array
                $sql .= " OR";
            }
        }
        $sql .= ") AND RBFR.allotment_id = " . $allotment_id;
        $sql .= " GROUP BY RBFR.field_id";
        $sql .= " HAVING COUNT(RBFR.field_id) > 0";
        $result = $this->db->query($sql)->result();
        return $result;
    }
    
    /**
     * NOT USED METHOD - Get Package IDs using field IDs using HAVING
     * @param number $allotment_id
     * @param array $fields
     * @return object
     */
    /*public function get_package_ids_using_fields_having($allotment_id, $fields = array()) {
     if(empty($fields)) {
     return false;
     }
     $sql = "SELECT DISTINCT(package_id)";
     $sql .= " FROM " . $this->db->dbprefix . "booking_field_relations BFR";
     $sql .= " WHERE (";
     // Get array keys
     $arrayKeys = array_keys($fields);
     // Fetch last array key
     $lastArrayKey = array_pop($arrayKeys);
     //iterate array
     foreach($fields as $field_key => $field_id) {
     $sql .= " BFR.booking_field_id = " . $field_id;
     if($field_key != $lastArrayKey) { // Not the Last Element of the array
     $sql .= " OR";
     }
     }
     $sql .= ") AND BFR.allotment_id = " . $allotment_id;
     $sql .= " GROUP BY BFR.booking_field_id";
     $sql .= " HAVING COUNT(BFR.booking_field_id) > 0";
     $result = $this->db->query($sql)->result();
     return $result;
     }*/
    
    /**
     * Get Package IDs using field IDs
     * @param number $allotment_id
     * @param array $fields
     * @param number $having_count - 0 -> Atleast one row must match, 1 -> Both must match
     * @return object
     */
    public function get_package_ids_using_fields($allotment_id, $fields = array()) {
        if(empty($fields)) {
            return false;
        }
        $cacheID = 'booking_packages_using_fields' . $allotment_id . '_' . implode('_', $fields);
        $cacheID = str_replace('-', '_', $this->slug->create_slug($cacheID));
        
        if(!$result = $this->gc_booking_cache->load($cacheID)) {
            // Get array keys
            $arrayKeys = array_keys($fields);
            // Fetch last array key
            $lastArrayKey = array_pop($arrayKeys);
            //iterate array
            $sql = "SELECT BFR.package_id FROM";
            $i = 1;
            foreach($fields as $field_key => $field_id) {
                if($i == 1) {
                    $sql .= " (SELECT package_id FROM " . $this->db->dbprefix . "booking_field_relations WHERE booking_field_id = $field_id) BFR";
                } else {
                    $sql .= " INNER JOIN (SELECT package_id FROM " . $this->db->dbprefix . "booking_field_relations WHERE booking_field_id = $field_id) BFR" . $i . " ON BFR" . $i . ".package_id = BFR.package_id";
                    if($field_key != $lastArrayKey) { // Not the Last Element of the array
                        $sql .= " INNER JOIN ";
                    }
                }
                $i++;
            }
            $result = $this->db->query($sql)->result();
            $result = array('result' => $result);
            $this->gc_booking_cache->save($result, $cacheID, array('booking_packages_using_fields' . $allotment_id));
        }
        return $result['result'];
    }
    
    /**
     * Get Field Info using Package IDs
     * @param array $package_ids
     * @param number $day
     * @return boolean|object
     */
    public function get_field_info_using_package_ids($package_ids, $day = 99) {
        if(empty($package_ids)) {
            return false;
        }
        $sql = "SELECT BFR.package_id, BFRR.field_id, F.name AS field_name, BF.dynamic_id, BFRR.value";
        $sql .= " FROM " . $this->db->dbprefix . "booking_field_relations BFR";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "booking_fields BF ON BF.id = BFR.booking_field_id";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "fields F ON F.id = BF.field_id";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "booking_fields BFRR ON BFRR.id = BF.value";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "booking_rules BR ON BR.package_id = BFR.package_id";
        $sql .= " WHERE BFR.package_id IN (" . implode(',', $package_ids) . ")";
        if($day != 99) {
            $sql .= " AND BR.value = " . $day;
        }
        $result = $this->db->query($sql)->result();
        return $result;
    }
    
    /**
     * Get Package IDs from the Bookings table
     * @param string $booking_date
     * @return boolean|object
     */
    public function get_package_ids_from_bookings($allotment_id, $booking_date) {
        $sql = "SELECT package_id FROM " . $this->db->dbprefix . "bookings";
        $sql .= " WHERE allotment_id = " . $allotment_id . " AND DATE(booking_date) = '" . $booking_date . "' AND status = 1";
        $result = $this->db->query($sql)->result();
        return $result;
    }
    
    /**
     * Get Package Information
     * @param number $package_id
     * @return object
     */
    public function get_booking_package($package_id) {
        $sql = "SELECT F.name, BF.value, BF.field_id, BF.dynamic_id FROM " . $this->db->dbprefix . "booking_field_relations BFR";
        $sql .= " INNER JOIN " . $this->db->dbprefix . "booking_fields BF ON BF.id = BFR.booking_field_id";
        $sql .= " INNER JOIN " . $this->db->dbprefix . "fields F ON F.id = BF.field_id";
        $sql .= " WHERE BFR.package_id = " . $package_id;
        $result = $this->db->query($sql)->result();
        return $result;
    }
    
    /**
     * Show availability screen only if the allotment has
     * Booking Date field assigned
     * OR is_extra is 1
     *
     * @param number $allotment_id
     */
    public function has_availability_screen($form_id) {
        $result = array(
            'success' => false
        );
        $cacheID = 'booking_has_availability_screen_' . $form_id;
        if(!$result = $this->gc_booking_cache->load($cacheID)) {
            $result = array();
            $success = false;
            $module_id = gc_module_id('bookings');
            $sql = "SELECT COUNT(*) AS count FROM " . $this->db->dbprefix . "field_modules BAF";
            $sql .= " INNER JOIN " . $this->db->dbprefix . "fields F ON F.id = BAF.field_id";
            $sql .= " WHERE BAF.module_id = $module_id AND BAF.row_id = $form_id AND F.name LIKE 'booking_date'";
            $booking_date = $this->db->query($sql)->row();
            if($booking_date && $booking_date->count > 0) {
                $success = true;
            }
            
            if(!$success) {
                // Count availability screen fields.
                $count = $this->db->where(array('module_id' => $module_id, 'row_id' => $form_id, 'status' => 1, 'is_extra' => 1))->count_all_results('field_modules');
                if($count > 0) {
                    $success = true;
                }
            }
            $result['success'] = $success;
            $this->gc_booking_cache->save($result, $cacheID, array('booking_has_availability_screen_' . $form_id));
        }
        return $result['success'];
    }
    
    /***********************************************
     * END OF CHECK AVAILABILITY LOGIC
     ***********************************************/
    /**
     * Format Booking Data - Sort fields
     * @param string $data
     */
    public function format_booking_data($data) {
        $fields_order = array('name', 'address', 'postcode' , 'city', 'phone_number');
        $data = json_decode($data ?? '', true);
        $final_data = array();
        $filtered_data = array();
        if(!empty($data)) {
            foreach($data as $dkey => $dvalue) {
                $filtered_data[$dvalue['name']] = $dvalue;
            }
            foreach($fields_order as $field_order) {
                if(!empty($filtered_data[$field_order])) {
                    $final_data[$field_order] = $filtered_data[$field_order];
                    unset($filtered_data[$field_order]);
                }
            }
            $final_data = array_merge($final_data, $filtered_data);
        }
        return json_encode($final_data);
    }
    
    /**
     * Get Booked Seats using Order_items table
     * A user should not be able to book a seat if all seats are already purchased
     * @param number $allotment_id
     * @param number $order_status_id
     * @return number
     */
    public function get_booked_seats($allotment_id, $order_status_id = 2, $user_id = 0, $where = null) {
        $sql = "SELECT SUM(OT.qty) AS booked_seats FROM " . $this->db->dbprefix . "order_items OT";
        $sql .= " INNER JOIN " . $this->db->dbprefix . "orders O ON O.order_id = OT.order_id";
        $sql .= " INNER JOIN " . $this->db->dbprefix . "bookings B ON OT.row_id = B.id AND OT.module_id = " . gc_module_id('bookings');
        $sql .= " WHERE B.allotment_id = " . $allotment_id . " AND OT.status_id = " . $order_status_id . " AND O.status_id = " . $order_status_id;
        if(!empty($user_id)) {
            $sql .= " AND OT.user_id = " . $user_id;
        }
        if(!empty($where)) { // get a specific date booking information
            $sql .= " AND " . $where;
        }
        $result = $this->db->query($sql)->row();
        return $result ? $result->booked_seats : 0;
    }
    
    /**
     * Member Area - All Booking Forms Bookings
     * @param number $user_id
     * @return number
     */ 
    public function member_bookings($user_id) {
        $sql = "SELECT COUNT(*) AS total_bookings FROM " . $this->db->dbprefix . "bookings M";
        $sql .= " INNER JOIN " . $this->db->dbprefix . "booking_allotments BA ON BA.id = M.allotment_id";
        $sql .= " WHERE BA.user_id = " . $user_id;
        $result = $this->db->query($sql)->row();
        return $result ? (int) $result->total_bookings : 0;
    }
}