<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * listing locations controller
 */
class branches extends Front_Controller {

	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct() {
		parent::__construct();

		$this->load->library('form_validation');
		$this->load->model('branches_model', null, true);
		$this->load->library('users/auth');
		$this->load->library('crypt');
		$this->lang->load('listings/listings');
	}
	
	/**
	 * Allow user to view all locations
	 * @param number $offset
	 */
	public function view($offset = 0) {
		$this->auth->restrict();
		
		$listing_id = intval($this->crypt->decode($this->input->get('id')));
		if($listing_id == 0) {
			redirect ( gc_url_format('members'));
		}
		if (isset($_POST['delete'])) {
			$checked = $this->input->post('checked');
			if (is_array($checked) && count($checked)) {
				$result = FALSE;
				foreach ($checked as $pid) {
					$result = $this->branches_model->delete($pid);
				}
	
				if ($result) {
					Template::set_message(count($checked) .' '. sprintf(lang ( 'msg_delete_success' ), lang('label_location')), 'success');
					redirect(site_url('branches/view?id=' . $this->crypt->encode($this->session->userdata('location_listing_id'))));
				} else {
					Template::set_message(sprintf(lang ( 'msg_delete_failure' ), lang('label_location')) . $this->branches_model->error, 'danger');
				}
			}
		}
		$listing_info = $this->listings_model->select('title, status')->find($listing_id);
		$sort_by = "id DESC";
		$records = $this->branches_model->get_branches(0, $listing_id, $this->auth->user_id(), false, $sort_by, $this->limit, $offset);
		$total_records = $this->branches_model->get_branches(1, $listing_id, $this->auth->user_id());
		
		
		$this->load->library('pagination');
		if (count($_GET) > 0) $pager['suffix'] = '?' . http_build_query($_GET, '', "&");
		$pager['base_url']    = site_url('/branches/view');
		$pager['total_rows']  = $total_records;
		$pager['per_page']    = $this->limit;
		$pager['uri_segment'] = 3;	
	
		$this->pagination->initialize($pager);
		
		Template::set(array(
			'records' => $records,
			'title' => $listing_info->title . ' - ' . lang('additional_locations'),
			'page_title' => $listing_info->title . ' - ' . lang('additional_locations'),
			'listing_status' => $listing_info->status // display eye icon only if the listing is active
		));
		
		// Listing Locations Links
		Template::set('module_links', '<li class="active"><a href="'.site_url('branches/view?id=' . $this->crypt->encode($listing_id)) .'">'.lang('manage_branches').'</a></li><li><a href="'.site_url('branches/add?id=' . $this->crypt->encode($listing_id)).'">'.lang('new_location').'</a></li>');
		
		Template::set_view ( 'members/view_locations' );
		Template::render ('member');
	}
	
	/**
	 * Allow user to add a business in multiple locations
	 */
	public function add() {
		$this->auth->restrict();		
		Assets::add_js ('google-map.js');
		$listing_id = $this->crypt->decode($this->input->get('id'));
		if((int)$listing_id == 0) {
			redirect ( gc_url_format('members'));
		}		
		$this->branches_model->add_location($listing_id, $this->auth->user_id());
	}
	
	/**
	 * Allow users to modify an existing location
	 */
	public function edit() {
		$this->auth->restrict();
		Assets::add_js ('google-map.js');
		$id = intval($this->crypt->decode($this->input->get('id')));
		if($id == 0) {
			redirect ( gc_url_format('members'));
		}
		$this->branches_model->edit_location($id, $this->auth->user_id());
	}
}