<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Listing Locations Content controller
 */
class Content extends Admin_Controller
{

	protected $permissionCreate = 'Listings.Content.Create';
	protected $permissionDelete = 'Listings.Content.Delete';
	protected $permissionEdit   = 'Listings.Content.Edit';
	protected $permissionView   = 'Listings.Content.View';
	
	//--------------------------------------------------------------------

	private $redirect_url = '/content/listings';
	
	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct()
	{
		parent::__construct();

		$this->auth->restrict('Listings.Content.View');
		$this->load->model('branches_model', null, true);
		$this->load->model('listings/listings_model');
		$this->lang->load('listings/listings');
		
		Template::set_block('sub_nav', 'content/_sub_nav');
		
	}
	
	/**
	 * Allow user to view all locations
	 * @param number $offset
	 */
	public function view($listing_id = 0, $filter='all', $offset = 0) {
		$module = 'branches';
		// Status Change
		if (isset($_POST['status_change'])) {
			$this->gc->status_change($module, $_POST['status_change'], 'status');
		}
		$table_prefix = 'B.';
		// Actions done, now display the view
		$where = $table_prefix . "deleted = 0";		
		$module_name = lang('branches');
		// Do we have any actions?
		if ($action = $this->input->post('submit')) {
			$checked = $this->input->post('checked');
		
			switch(strtolower($action)) {
				case 'activate':
					$this->gc->update_status($module, $checked, 'status', 1);
					Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_activated')), 'success');
					break;
						
				case 'deactivate':
					$this->gc->update_status($module, $checked, 'status', 0);
					Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_deactivated')), 'success');
					break;
						
				case 'featured':
				case 'unfeatured':
					$this->db->where('id IN (' . implode(',', $checked) . ')')->update($module, array('featured' => ($action == 'featured' ? 1 : 0)));
					Template::redirect(current_url());
					break;
		
				case 'expired':
					$this->gc->change_status($module, $checked, 2);
					break;
		
				case 'delete':
					$this->gc->update_status($module, $checked, 'deleted', 1);
					Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_deleted')), 'success');
					break;
						
				case 'purge':
					$this->gc->purge_records($module, $checked);
					Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_deleted')), 'success');
					break;
						
				case 'restore':
					$this->gc->update_status($module, $checked, 'deleted', 0);
					Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_restored')), 'success');
					break;
			}
			Template::redirect(current_url());
		}
		
		// Filters
		$filter_type = $filter;
		
		switch($filter) {
			case 'inactive':
				$where .= " AND " . $table_prefix . "status = 0";
				break;
		
			case 'active':
				$where .= " AND " . $table_prefix . "status = 1";
				break;
					
			case 'expired':
				$where .= " AND " . $table_prefix . "status = 2";
				break;
					
			case 'featured':
				$where .= " AND " . $table_prefix . "featured = 1";
				break;
		
			case 'deleted':
				$where = $table_prefix . "deleted = 1";
				break;
					
			case 'all':
				// Nothing to do
				break;
		
			default:
				Template::redirect(SITE_AREA . '/content/listings');
		}
		
		$toolbar_title = lang('additional_locations');
		if($listing_id != 0) {
			$listing_info = $this->listings_model->select('title')->find($listing_id);
			if($listing_info) {
				$toolbar_title = $listing_info->title . ' - ' . lang('additional_locations');
			}
		}
		// Search
		$searchterm = $this->gc->search('queries', $this->uri->segment(7), '/reports/branches');
		if(!empty($searchterm)) {
			$where .= " AND (B.contact_person LIKE '%". $searchterm ."%' OR B.phone_number LIKE '%". $searchterm ."%' OR B.city LIKE '%". $searchterm ."%' OR B.created_on LIKE '%". $searchterm ."%')";
		}
		
		$order_by = $this->gc->sort('branches');
		if(!empty($order_by)) {
			$sort_by = $order_by;
		} else {
			$sort_by = "id DESC";
		}
		$records = $this->get_branches(0, $listing_id, $where, $sort_by, $this->limit, $offset);
		$total_records = $this->get_branches(1, $listing_id, $where, $sort_by, $this->limit, $offset);
	
		$this->load->library('pagination');
		$pager['base_url']    = site_url(SITE_AREA . '/content/branches/view/' . $listing_id . '/' . $filter . '/');
		$pager['total_rows']  = $total_records;
		$pager['per_page']    = $this->limit;
		$pager['uri_segment'] = 7;
	
		$this->pagination->initialize($pager);
	
		Template::set('listing_id', $listing_id);
		Template::set('records', $records);
		Template::set('filter_type', $filter_type);
		Template::set('index_url', site_url(SITE_AREA . '/content/branches/view/' . $listing_id . '/'));
		Template::set('toolbar_title', $toolbar_title);
		//Template::set_view ( 'members/view_locations' );
		Template::render ();
	}
	
	/**
	 * Get Branches
	 * @param int $count
	 * @param int $listing_id
	 * @param string $where
	 * @param string $sort_by
	 * @param integer $limit
	 * @param integer $offset
	 * @return number
	 */
	private function get_branches($count, $listing_id, $where, $sort_by, $limit, $offset) {
		if($count == 0) {
			$sql = "SELECT B.*, L.slug AS listing_slug, L.title AS listing_title, L.city AS listing_city";
		} else {
			$sql = "SELECT COUNT(B.id) AS total_count";
		}
		$sql .= " FROM " . $this->db->dbprefix . "branches B";
		$sql .= " INNER JOIN " . $this->db->dbprefix . "listings L ON L.id = B.listing_id";	
		if(!empty($where)) {
			$sql .= " WHERE " . $where;
			$listing_id = (int) $listing_id;
			if(!empty($listing_id)) {
				$sql .= " AND B.listing_id = " . $listing_id;
			}
		}	
		if($count == 0) {
			$sql .= " ORDER BY " . $sort_by;
			$sql .= " LIMIT " . $offset . ", " . $limit ."";
			return $this->db->query($sql)->result();
		} else {
			$result = $this->db->query($sql)->row();
			return $result ? $result->total_count : 0;
		}
	}
	
	/**
	 * Creates a module row object.
	 * @return void
	 */
	public function add($listing_id) {
		$listing_id = intval($listing_id);
		if(empty($listing_id)) {
			Template::redirect(SITE_AREA .'/content/listings');
		}
		$record = $this->db->select('title')->get_where('listings', array('id' => $listing_id))->row();
		if(!$record) {
			Template::redirect(SITE_AREA .'/content/listings');
		}
		$module = 'branches';
		$redirect_url = site_url(SITE_AREA . '/content/branches/view/' . $listing_id);
		$this->gc->module('admin', $module, $this->auth->user_id(), $redirect_url, $listing_id);
		Template::set(array(
			'listing_id' => $listing_id,
			'toolbar_title' => $record->title . ' - ' . lang('add') . ' ' . lang($module)
		));
		Template::set_view('fields/form');
		Template::render();
	}
	
	/**
	 * Allows editing of module data.
	 * @return void
	 */
	public function edit() {
		$id = intval($this->uri->segment(5));
		$redirect_url = site_url(SITE_AREA . '/content/branches/view');
		if (empty($id)) {
			Template::set_message(lang('invalid_id'), 'error');
			redirect(SITE_AREA . '/content/listings');
		}
		$module = 'branches';
		$record = $this->branches_model->find($id);
		if(!$record) {
			Template::redirect(SITE_AREA .'/content/listings');
		}
		$this->gc->module('admin', $module, $this->auth->user_id(), $redirect_url, 0, $id);
		Template::set(array(
			'module' => 'branches',
			'listing_id' => $record->listing_id,
			'toolbar_title' => lang('edit') . ' ' . lang($module)
		));
		Template::set_view('fields/form');
		Template::render();
	}
	
	/**
	 * Change status of listing location from view page
	 * @param int $id
	 */
	public function update_status($id, $status, $offset = 0) {
		$this->auth->restrict($this->permissionEdit);
	
		$sql = "UPDATE " . $this->db->dbprefix . "branches SET status = ". $status ." WHERE id = " . $id;
		$this->db->simple_query($sql);
	
		Template::set_message(sprintf(lang('msg_action_success'), lang('label_location'), lang('label_updated')), 'success');
		$url = site_url(SITE_AREA .'/content/branches/view/' . $this->session->userdata('row_id'));
		if($offset != 0) {
			$url .= '/' . $offset;
		}
		redirect($url);
	}

}