<?php
if (! defined ( 'BASEPATH' ))
	exit ( 'No direct script access allowed' );
class Migration_Install_branches extends Migration {
	/**
	 * The name of the database table
	 *
	 * @var String
	 */
	private $table_name = 'branches';
	
	/**
	 * The Listing table's fields
	 *
	 * @var Array
	 */
	private $fields = array (
		'id' => array (
			'type' => 'INT',
			'constraint' => 11,
			'auto_increment' => TRUE 
		),
		'listing_id' => array (
			'type' => 'BIGINT',
			'constraint' => 20,
			'null' => FALSE,
			'default' => 0
		),
		'user_id' => array (
			'type' => 'BIGINT',
			'constraint' => 20,
			'null' => FALSE,
			'default' => 0
		),
		'package_id' => array (
			'type' => 'INT',
			'constraint' => 11,
			'null' => FALSE,
			'default' => 0 
		),
		'country_iso' => array (
			'type' => 'CHAR',
			'constraint' => 2,
			'null' => FALSE 
		),
		'state_id' => array (
			'type' => 'INT',
			'constraint' => 11,
			'null' => FALSE 
		),
		'city_id' => array (
			'type' => 'INT',
			'constraint' => 11,
			'null' => FALSE 
		),
		'locality_id' => array (
			'type' => 'INT',
			'constraint' => 11,
			'null' => TRUE 
		),
		'postcode' => array (
			'type' => 'VARCHAR',
			'constraint' => 10,
			'null' => TRUE 
		),
		'address' => array (
			'type' => 'VARCHAR',
			'constraint' => 250,
			'null' => TRUE 
		),
		'latitude' => array (
			'type' => 'VARCHAR',
			'constraint' => 30,
			'null' => FALSE 
		),
		'longitude' => array (
			'type' => 'VARCHAR',
			'constraint' => 30,
			'null' => FALSE 
		),
		'contact_person' => array (
			'type' => 'VARCHAR',
			'constraint' => 50,
			'null' => FALSE 
		),
		'phone_number' => array (
			'type' => 'VARCHAR',
			'constraint' => 20,
			'null' => FALSE 
		),
		'status' => array (
			'type' => 'TINYINT',
			'constraint' => 2,
			'null' => FALSE,
			'default' => '0' 
		),
		'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'created_by' => array (
			'type' => 'BIGINT',
			'constraint' => 20,
			'null' => FALSE,
			'default' => '0' 
		),
		'created_ip' => array (
			'type' => 'VARCHAR',
			'constraint' => 50,
			'null' => TRUE 
		),
		'modified_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'modified_by' => array (
			'type' => 'BIGINT',
			'constraint' => 20,
			'null' => FALSE,
			'default' => '0' 
		),
		'modified_ip' => array (
			'type' => 'VARCHAR',
			'constraint' => 50,
			'null' => TRUE 
		),
		'locality' => array(
			'type' => 'VARCHAR',
			'constraint' => 40,
			'null' => TRUE,
		),
		'city' => array(
			'type' => 'VARCHAR',
			'constraint' => 40,
			'null' => TRUE,
		),
		'state' => array(
			'type' => 'VARCHAR',
			'constraint' => 40,
			'null' => TRUE,
		),
		'country' => array(
			'type' => 'VARCHAR',
			'constraint' => 80,
			'null' => TRUE,
		)
	);

	/**
	 * Install this migration
	 *
	 * @return void
	 */
	public function up() {
		$this->dbforge->add_field($this->fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('listing_id',false);
		$this->dbforge->add_key('country_iso',false);
		$this->dbforge->add_key('state_id',false);
		$this->dbforge->add_key('city_id',false);
		$this->dbforge->add_key('locality_id',false);
		$this->dbforge->add_key('postcode',false);
		$this->dbforge->create_table($this->table_name);

		// ADD LOCATION INDEX
		$index_location = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " ADD FULLTEXT INDEX `search_location` (`postcode`, `locality`, `city`, `state`, `country`)";
		$this->db->query ($index_location);
	}
	
	/**
	 * Uninstall this migration
	 *
	 * @return void
	 */
	public function down() {
		if ($this->db->table_exists($this->table_name)) {
			$this->dbforge->drop_table($this->table_name);
		}
	}
}