<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class branches_model extends BF_Model {

	protected $table_name	= "branches";
	protected $key			= "id";
	protected $date_format	= "datetime";

	protected $log_user 	= TRUE;
	protected $soft_deletes	= FALSE;
	protected $set_created	= TRUE;
	protected $set_modified = TRUE;
	
	protected $created_field     = 'created_on';
	protected $created_by_field  = 'created_by';
	protected $created_ip_field  = 'created_ip';
	protected $modified_field    = 'modified_on';
	protected $modified_by_field = 'modified_by';
	protected $modified_ip_field = 'modified_ip';

	/*
		Customize the operations of the model without recreating the insert, update,
		etc methods by adding the method names to act as callbacks here.
	 */
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 		= array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	/*
		For performance reasons, you may require your model to NOT return the
		id of the last inserted row as it is a bit of a slow method. This is
		primarily helpful when running big loops over data.
	 */
	protected $return_insert_id 	= TRUE;

	// The default type of element data is returned as.
	protected $return_type 			= "object";

	// Items that are always removed from data arrays prior to
	// any inserts or updates.
	protected $protected_attributes = array();

	/*
		You may need to move certain rules (like required) into the
		$insert_validation_rules array and out of the standard validation array.
		That way it is only required during inserts, not updates which may only
		be updating a portion of the data.
	 */
	protected $validation_rules 		= array(
				
	);
	protected $insert_validation_rules 	= array();
	protected $skip_validation 			= FALSE;

	//--------------------------------------------------------------------
	
	/**
	 * Common Function to Add Branch Offices to a Listing
	 * @param int $listing_id
	 * @param int $user_id
	 * @param string $mobile - True if the request is from the Mobile Application
	 */
	public function add_location($listing_id, $user_id, $mobile = false) {
		// Check whether user has permission or not
		$this->load->model('listings/listings_model');		
		$this->load->model('activities/activity_model');
		$listing_info = $this->listings_model->select('title, user_id, package_id')->find_by(array('id' => $listing_id, 'deleted' => 0));
		if(!$listing_info && ($listing_info->user_id != $user_id)) {
			if($mobile) {
				$result = array(
					'success' => false,
					'message' => lang('msg_permission_expired_error'),
					'listing_id' => 0
						
				);
				return $result;
			} else {
				Template::set_message(lang('msg_permission_expired_error'), 'error');
				redirect(site_url('branches/view?id='.$this->crypt->encode($listing_id)));
			}
		}
		
		// GET USER LIMIT AND ACTIVE VALUE FROM PACKAGE
		$this->load->model('members/members_model');
		$package = $this->members_model->get_package_info($user_id, $listing_info->package_id, $listing_id);
		$user_limit = gc_field_value('branches', $package->module_limit);
		if($user_limit == 999) {
			if($mobile) {
				$result = array(
					'success' => false,
					'message' => sprintf(lang('msg_module_error'), $package->package_title, lang('locations')),
					'listing_id' => $listing_id
				);
				return $result;
			} else {
				Template::set_message(sprintf(lang('msg_module_error'), $package->package_title, lang('locations')), 'error');
				redirect(gc_url_format('members'));
			}
		}
	
		$status_field = (int)$package->locations_active;
		$locations_count = $this->branches_model->count_by(array('listing_id' => $listing_id));
		if(($user_limit == 0) || ($user_limit > $locations_count)) { // check whether user has permission to add listing to muliple locations or not
			if (isset ( $_POST ['save'] )) {
				if ($insert_id = $this->branches_model->save_branches (($mobile ? 'mobile' : 'member'), 'insert', $user_id, $listing_id, 0, $status_field)) {
					// Log the activity
					$this->activity_model->log_activity ( $user_id, sprintf(lang ( 'act_create_record' ), lang('label_location')) . ': ' . $insert_id . ' : ' . $this->input->ip_address (), 'branches' );
						
					if($mobile) {
						$result = array(
							'success' => true,
							'message' => sprintf(lang ( 'msg_save_success' ), lang('label_location')),
							'listing_id' => $listing_id
						);
						return $result;
					} else {
						Template::set_message ( sprintf(lang ( 'msg_save_success' ), lang('label_location')), 'success' );
						redirect ( site_url('branches/view?id=' . $this->crypt->encode($listing_id)));
					}
				} else if($mobile) {			
					return array('success' => false, 'message' => validation_errors());
				}
			}
		
			if(!$mobile) {
				if(settings_item('lst.google_places_api') == 1) {
					$this->gc->location_google_places();
				} else {
					// OnType city and locality selection
					$this->gc->ontype_city_selection();
					$this->gc->ontype_locality_selection();
				}
		
				$map = $this->gc->google_map();
				$form_fields = json_decode($package->form_fields ?? '');
		
				Template::set(array(
					'map' => $map,
				    'form_fields' => $form_fields->listings,
					'listing_id' => $listing_id,
					'title' => $listing_info->title . ' - ' . lang('new_location')
				));

				Template::set_view ( 'listing/multi_locations' );
				Template::render ('member');
			} else {
				$result = array(
					'success' => true,
					//'latitude' => settings_item('lst.latitude'),
					//'longitude' => settings_item('lst.longitude')
				);
				return $result;
			}
				
		} else {
			if($mobile) {
				$result = array(
					'success' => false,
					'message' => sprintf(lang('user_limit'), $user_limit, lang('additional_locations')),
					'listing_id' => $listing_id
				);
				return $result;
			} else {
				Template::set_message(sprintf(lang('user_limit'), $user_limit, lang('additional_locations')), 'error');
				redirect(site_url('branches/view?id='.$this->crypt->encode($listing_id)));
			}
		}
	}
	
	/**
	 * Edit Branch Office
	 * @param int $id - Branch Office ID
	 * @param int $user_id
	 * @param string $mobile - True if the request is from the mobile device
	 */
	public function edit_location($id, $user_id, $mobile = false) {
		$location = $this->branches_model->get_listing_location($id, $user_id);
		if(!$location) {
			if($mobile) {
				$result = array(
					'success' => false,
					'message' => lang('msg_permission_expired_error'),
					'listing_id' => 0
				);
				return $result;
			} else {
				redirect ( gc_url_format('members'));
			}			
		}
		$listing_id = $location->listing_id;
		if (isset ( $_POST ['save'] )) {
			if ($this->branches_model->save_branches (($mobile ? 'mobile' : 'member'), 'update', $user_id, $listing_id, $id)) {
				
				// Log the activity
				$this->load->model('activities/activity_model');
				$this->activity_model->log_activity ( $user_id, sprintf(lang ( 'msg_act_edit_record' ), lang('label_location')) . ': ' . $id . ' : ' . $this->input->ip_address (), 'branches' );
				
				if($mobile) {
					$result = array(
						'success' => true,
						'message' => sprintf(lang ( 'msg_save_success' ), lang('label_location')),
						'listing_id' => $listing_id
					);
					return $result;
				} else {					
					Template::set_message ( sprintf(lang ( 'msg_save_success' ), lang('label_location')), 'success' );
					redirect ( site_url('branches/view?id=' . $this->crypt->encode($listing_id)));
				}
			}
		} else if (!$mobile && isset($_POST['delete'])) {
			if ($this->branches_model->delete($id)) {
				// Log the activity
				log_activity($this->current_user->id, lang('act_delete_record') .': '. $id .' : '. $this->input->ip_address(), 'branches');
		
				Template::set_message(sprintf(lang('msg_delete_success'), lang('label_location')), 'success');
		
				redirect(site_url('branches/view?id=' . $this->crypt->encode($listing_id)));
			} else {
				Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_location')) . $this->branches_model->error, 'danger');
			}
		}
		
		if(!$mobile) {
			if(settings_item('lst.google_places_api') == 1) {
				$this->gc->location_google_places($location->city_id, true);
			} else {
				// OnType City and Locality Selection
				$this->gc->ontype_city_selection($location->city_id, true);
				$this->gc->ontype_locality_selection($location->locality_id, true);
			}
			
			$map = $this->gc->google_map ( $location->latitude, $location->longitude );
			
			// get form fields from the packages table
			$package = $this->packages_model->select('form_fields')->find($location->package_id);			
			$form_fields = json_decode($package->form_fields ?? '');
			
			Template::set(array(
				'title' => $location->listing_title . ' - ' . lang('modify_location'),
				'page_title' => $location->listing_title . ' - ' . lang('modify_location'),
				'map' => $map,
				'listing' => $location,
				'listing_id' => $listing_id,
			    'form_fields' => $form_fields->listings,
			));
			Template::set_view ( 'listing/multi_locations' );
			Template::render('member');
		} else {
			$this->load->model('locations/cities_model');
			$location_info = $this->cities_model->get_city_info($location->city_id);
			return array(
				'success' => true, 
				'contact_person' => $location->contact_person,
				'phone' => $location->phone_number,
				'address' => $location->address,
				'postcode' => $location->postcode,
				'country' => array('iso' => $location->country_iso, 'name' => $location_info->country),
				'state' => array('id' => $location_info->state_id, 'name' => $location_info->state),				
				'city' => array('id' => $location_info->id, 'name' => $location_info->city),	
				'latitude' => $location->latitude,
				'longitude' => $location->longitude
			);
		}
	}
	
	/**
	 * Save listing locations information
	 * @param string $role - admin/member/mobile
	 * @param string $type - 'insert'
	 * @param number $id
	 * @return number
	 */
	public function save_branches($role, $type, $user_id, $listing_id, $id = 0, $status = 0) {
		$this->load->model('listings/listings_model');
		$this->load->model('locations/cities_model');
		if ($type == 'update') {
			$_POST['id'] = $id;
		}
		if($role == 'mobile') {
			$this->form_validation->set_rules('city_id', 'lang:select_city', 'trim|required');
		} else {
			if(settings_item('lst.google_places_api') == 0) {
				$this->form_validation->set_rules('city_id', 'lang:select_city', 'trim|required');
			} else {
				$this->form_validation->set_rules('city', 'lang:select_city', 'trim|required');
			}
		}
		
		$this->form_validation->set_rules('locality_id', 'lang:select_locality', 'trim');
		$this->form_validation->set_rules('postcode', 'lang:label_postcode', 'trim|required');
		$this->form_validation->set_rules('address', 'lang:label_address', 'max_length[255]|trim|required');
		$this->form_validation->set_rules('latitude', 'lang:label_latitude', 'trim|valid_latitude');
		$this->form_validation->set_rules('longitude', 'lang:label_longitude', 'trim|valid_longitude');
		$this->form_validation->set_rules('contact_person', 'lang:label_contact_person', 'trim|max_length[50]|xss_clean');
		$this->form_validation->set_rules('phone_number', 'lang:label_phone_number', 'trim|required');
		if ($this->form_validation->run() === false) {
			return false;
		}
	
		$data['listing_id']        	= $listing_id;
		if($role == 'mobile') {
			$data['country_iso']    = $this->input->post('country_iso');
			$data['state_id']      	= $this->input->post('state_id');
			$data['city_id']      	= $this->input->post('city_id');
			$data['locality_id']    = $this->input->post('locality_id') ? $this->input->post('locality_id') : 0;
		} else {
			if(settings_item('lst.google_places_api') == 0) {
				$data['city_id']      	= $this->input->post('city_id');
				if($this->input->post('city_id')) {
					$city_info = $this->cities_model->get_city_info($this->input->post('city_id'));
					$data['state_id'] 	= $city_info->state_id;
					$data['country_iso']= $city_info->country_iso;
				}
				$data['locality_id']  	= $this->input->post('locality_id') ? $this->input->post('locality_id') : 0;
			} else { // Google Places API is used for City Database
				$city_name 				= $this->input->post('city');
				$state_abbrev 			= $this->input->post('state_abbrev');
				$state_name 			= $this->input->post('state_name');
				$country_iso 			= $this->input->post('country_iso');
				$country_name 			= $this->input->post('country_name');
				$location 				= $this->cities_model->getCityStateCountry($city_name, $state_abbrev, $state_name, $country_iso, $country_name);
				if($location && $location['city_id'] != 0) {
					$data['city_id'] 	= $location['city_id'];
					$data['state_id'] 	= $location['state_id'];
					$data['country_iso']= $location['country_iso'];
				} else if($id == 0) {
					return false;
				}
			}
		}
		$data['postcode']        	= $this->input->post('postcode') ? $this->input->post('postcode') : '';
		$data['address']        	= $this->input->post('address') ? $this->input->post('address') : '';
		$data['latitude']        	= $this->input->post('latitude') ? $this->input->post('latitude') : '';
		$data['longitude']        	= $this->input->post('longitude') ? $this->input->post('longitude') : '';
		$data['contact_person']    	= $this->input->post('contact_person') ? $this->input->post('contact_person') : '';
		$data['phone_number']      	= $this->input->post('phone_number') ? $this->input->post('phone_number') : '';
	
		if($role == 'admin') {
			$data['status'] 		= $this->input->post('status') ? 1 : 0;
		}
		
		if ($type == 'insert') {
			if($role == 'member' || $role == 'mobile') {
				$data['status'] 	= $status;
			}
			$data['created_on'] 	= date('Y-m-d H:i:s');
			$data['created_by']		= $user_id;
			$data['created_ip'] 	= $this->input->ip_address();
			$id = $this->branches_model->insert($data);
	
			if (is_numeric($id)) {
				$return = $id;
			} else {
				$return = FALSE;
			}
		} else if ($type == 'update') {
			$data['modified_by'] 	= $user_id;
			$data['modified_on'] 	= date('Y-m-d H:i:s');
			$data['modified_ip'] 	= $this->input->ip_address();
			//$this->db->where('id', $id);
			//return $this->db->update('branches', $data);
			$return = $this->branches_model->update($id, $data);
		}
		return $return;
	}
	
	/**
	 * Get Listing Locations for Member Area and Listing Detail page
	 * @param number $listing_id
	 * @param number $user_id
	 */
	public function get_branches($count, $listing_id, $user_id = 0, $isDetail = false, $sort_by = "id DESC", $limit = 25, $offset = 0) {
		if($count == 0) {
			$sql = "SELECT BR.*, BR.contact_person AS title, L.title AS listing_title, L.slug AS listing_slug, L.city AS listing_city";
			/* $sql = "SELECT BR.id, BR.listing_id, BR.address, BR.postcode, BR.contact_person, BR.phone_number, BR.country_iso, BR.state_id, BR.city_id, BR.created_on, BR.modified_on, BR.status,";
			$sql .= " L.slug AS listing_slug, L.title AS listing_title, LCT.name AS listing_city, LT.name AS locality, C.name AS country, S.name AS state, CT.name as city, BR.latitude, BR.longitude";*/
		} else {
			$sql = "SELECT COUNT(BR.id) AS total_count";
		}
		$sql .= " FROM " . $this->db->dbprefix . "branches BR";
		$sql .= " LEFT JOIN " . $this->db->dbprefix . "listings L ON L.id = BR.listing_id";
		
		/*if($count == 0) {
			$sql .= " INNER JOIN " . $this->db->dbprefix . "countries C ON C.iso = BR.country_iso";
			$sql .= " INNER JOIN " . $this->db->dbprefix . "states S ON S.id = BR.state_id";
			$sql .= " INNER JOIN " . $this->db->dbprefix . "cities CT ON CT.id = BR.city_id";
			$sql .= " LEFT JOIN " . $this->db->dbprefix . "localities LT ON LT.id = BR.locality_id";
		}*/
		
		$sql .= " WHERE BR.listing_id = " . $listing_id;
		if($user_id != 0) {
			$sql .= " AND L.user_id = " . $user_id;
		} 
		if($isDetail) { // Listing Detail Page
			$sql .= " AND BR.status = 1";
		}
		if($count == 0) {
			$sql .= " ORDER BY " . $sort_by;
			$sql .= " LIMIT " . $offset . ", " . $limit ."";
			return $this->db->query($sql)->result();
		} else {
			$result = $this->db->query($sql)->row();
			return $result ? $result->total_count : 0;
		}
	}

	/**
	 * Get listing location for edit page
	 * @param number $id
	 */
	public function get_listing_location($id, $user_id = 0) {
		$sql = "SELECT BR.*, L.title AS listing_title, L.package_id";
		$sql .= " FROM " . $this->db->dbprefix . "branches BR";
		$sql .= " INNER JOIN " . $this->db->dbprefix . "listings L ON L.id = BR.listing_id";
		$sql .= " WHERE BR.id = " . $id;
		if($user_id != 0) {
			$sql .= " AND L.user_id = " . $user_id;
		}
		$result = $this->db->query($sql)->row();
		return $result;
		/*$this->db->select($this->db->dbprefix . 'branches.*, L.title AS listing_title, L.user_id');
		$this->db->join($this->db->dbprefix . 'listings L','L.id = ' . $this->db->dbprefix . 'branches.listing_id AND L.deleted = 0 AND L.user_id = ' . $user_id .'', 'INNER');
		return $this->branches_model->find($id);*/
	}
}