<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Reports controller
 */
class Reports extends Admin_Controller {
	protected $permissionView   = 'Business_Queries.Reports.View';
	protected $permissionDelete = 'Business_Queries.Reports.Delete';
	protected $context = 'reports';
	protected $moduleName = 'business_queries';
	
	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct()
	{
		parent::__construct();

		$this->auth->restrict($this->permissionView);
		$this->load->model('business_queries_model', null, true);
		$this->lang->load('business_queries');
		Assets::add_module_js('business_queries', 'business_queries.js');
		
		//LOAD TinyMCE EDITOR
        Assets::add_js(array('tinymce/tinymce.min.js'));
		
		Template::set_block('sub_nav', 'reports/_sub_nav');
	}

	//--------------------------------------------------------------------
	
	
	/**
	 * Business Queries Summary
	 */
	public function summary($toggle_module = 'listings') {
	    // DATE TIME PICKER
	    $this->lang->load('listings/listings');
	    Assets::add_js('moment.js');
	    Assets::add_css('bootstrap-datetimepicker.min.css' );
	    Assets::add_js('bootstrap-datetimepicker.min.js' );
	    Assets::add_css('chart.css');
	    Assets::add_js('Chart.min.js');
	    $this->session->unset_userdata('chart_query_params');
	    $module = 'business_queries';
	    $toggle_module_id = gc_module_id($toggle_module);
	    $record = $this->gcrud->summary($module, 'status_id', 2, ' AND module_id = ' . $toggle_module_id);
	    $table = $module;
	    $where = 'module_id = ' . $toggle_module_id;
	    
	    // tabular blocks
	    $limit = 10;
	    $tabular_blocks = array();
	    /*$tabular_blocks['top_visited'] = $this->listings_model->_reports($module, 0, $limit, null, 'hits DESC', 'M');
	     $tabular_blocks['recently_added'] = $this->listings_model->_reports($module, 0, $limit, null, 'created_on DESC', 'M');
	     $tabular_blocks['recently_viewed'] = $this->listings_model->_reports($module, 0, $limit, null, 'last_viewed_on DESC', 'M');
	     */
	    $column = 'counter';
	    $row_by_months = $this->gcrud->chart('months', array('status' => 1, 'where' => $where,'prefix' => 'M', 'table' => $table, 'field' => $column));
	    Assets::add_js($this->load->view('common/_chart_js', array('field_id' => 'sixMonthChart', 'labels' => $row_by_months['labels'], 'data' => $row_by_months['data'], 'label' => lang('total_rows'), 'total' => $row_by_months['total'], 'with_dates' => false, 'table' => $table), true), 'inline');
	    $result = $this->gcrud->chart('current_month', array('status' => 1, 'where' => $where, 'prefix' => 'M', 'table' => $table, 'field' => $column));
	    Assets::add_js($this->load->view('common/_chart_js', array('field_id' => 'monthlyChart', 'labels' => $result['labels'], 'data' => $result['data'], 'total' => $result['total'], 'label' => lang('total_rows'), 'with_dates' => true, 'table' => $table), true), 'inline');
	    // $record->tabular_blocks = $tabular_blocks;
	    //$record->recently_viewed = $this->forums_model->manage(0, null, 'M.last_viewed DESC', 20, 0, array('module' => 'views'));
	    Template::set(array(
	        'module' => $module,
	        'toggle_module' => $toggle_module,
	        'tabular_blocks' => $tabular_blocks,
	        'toolbar_title' => lang($module) . ' - ' . lang('gc_menu_summary'),
	        'record' => $record
	    ));
	    Template::set_view('fields/summary');
	    Template::render();
	}
	
	/**
	 * Index Page
	 * @param string $filter
	 * @param number $offset
	 */
	public function index($module = 'listings', $filter='all', $offset = 0) {
	    $this->session->set_userdata('toggle_module', $module);
	    $this->manage('business_queries', $filter, $offset, $module);
	}
	
	/**
	 * Manage Business Queries
	 * @param string $module
	 * @param string $filter
	 * @param number $offset
	 */
	public function manage($module = 'business_queries', $filter = 'all', $offset = 0) {
	    $extra = array(
	        'model' => $this->moduleName,
	        'context' => $this->context,
	        'dynamic_fields' => true,
	        'fields' => null,
	        'prefix' => '',
	        'table_prefix' => 'query'
	    );
	    if($module == 'statuses' || $module == 'email_templates') {
	        $extra['fields'] = $module;
	        $extra['dynamic_fields'] = false;
	        $extra['prefix'] = 'query';
	    } else {	
	        $module_name = $this->session->userdata('toggle_module');
	        if(!in_array($module_name, gc_modules())) {
	            Template::redirect(site_url(SITE_AREA . '/reports/business_queries'));
	        }
	        $toggle_module_id = gc_module_id($module_name);
	        $extra['toggle_module'] = $module_name; 
	        $extra['query_where'] = 'M.module_id = ' . $toggle_module_id;
	        $extra['query_extra'] = array('module_id' => $toggle_module_id);
	    }
	    $this->gc->module_view($module, $filter, $offset, 'admin', $extra);
	}
	
	/**
	 * Add/Edit Blogs/Categories/Email Templates Data
	 *
	 * @return void
	 */
	public function form($module = 'business_queries', $action = 'create') {
	    Assets::add_css('selectize.bootstrap3.css');
	    Assets::add_js('selectize.min.js');
	    Assets::add_js('selectize-search.js');
	    $params = array(
	        'model' => $this->moduleName,
	        'context' => $this->context,
	        'fields_method' => null,
	        'dynamic_fields' => true,
	        'set_table' => true,
	        'uri_segment' => 7,
	        'table_prefix' => 'query',
	        'redirect_url' => site_url(SITE_AREA . '/' . $this->context . '/' . $this->moduleName . '/manage/' . $module)
	    );	    
	    if($module == 'statuses' || $module == 'email_templates') {
	        $params['fields_method'] = $module;
	        $params['dynamic_fields'] = false;
	        $params['prefix'] = 'query';
	    } else {
	        $toggle_module = 'listings';
	        if($action == 'edit') {
	            $record = $this->db->where('id', $this->uri->segment(7))->get('business_queries')->row();
	            $params['record'] = $record;
	            if($record) {
	                $toggle_module = gc_module_name($record->module_id);
	            }
	        }
	        $params['view_params'] = array('toggle_module' => $toggle_module);
	    }
	    $this->gcrud->form($module, $action, 'admin', $params);
	}	
	
	/**
	 * Assign fields to a Business Queries Form
	 * @param number $id
	 * @param string $module
	 */
	public function assign() {
	    $this->load->model('fields/fields_model');
	    $this->fields_model->assign(0, 'business_queries');
	}
	
	/**
	 * Displays a list of replies to the Admin.
	 *
	 * @return void
	 */
	public function replies($query_id, $offset=0) {
		$this->load->library('gc');
		// Deleting anything?
		if (isset($_POST['delete'])) {
			$this->auth->restrict($this->permissionDelete);
			$checked = $this->input->post('checked');
			if (is_array($checked) && count($checked)) {
	
				// If any of the deletions fail, set the result to false, so
				// failure message is set if any of the attempts fail, not just
				// the last attempt
	
				$result = true;
				foreach ($checked as $pid) {
					$deleted = $this->db->delete('query_replies', array('id' => $pid));
					if ($deleted == false) {
						$result = false;
					}
				}
				if ($result) {
					Template::set_message(count($checked) . ' ' . sprintf(lang('msg_action_success'), (count($checked) == 1 ? lang('label_reply') : lang('label_replies')), lang('label_deleted')), 'success');
				} else {
					Template::set_message(sprintf(lang('msg_delete_failure'), (count($checked) == 1 ? lang('label_reply') : lang('label_replies'))) . $this->favorites_model->error, 'danger');
				}
			}
		}
	
		$records = $this->business_queries_model->query_replies($query_id, $offset, $this->limit);
		$total_queries = $this->db->where('query_id', $query_id)->count_all_results('query_replies');
	
		Template::set('records', $records);
			
		// Pagination
		$this->load->library('pagination');
		$pager['base_url'] = site_url(SITE_AREA ."/reports/business_queries/replies/");
		$pager['total_rows'] = $total_queries;
		$pager['per_page'] = $this->limit;
		$pager['uri_segment']	= 7;
	
		$this->pagination->initialize($pager);
		Template::set('records', $records);
		Template::set('toolbar_title', lang('manage_replies') . ' - ' . $query_id);
		Template::render();
	}
	
	//--------------------------------------------------------------------


	/**
	 * Creates a Business Queries object.
	 *
	 * @return void
	 */
	private function create() {
		$this->auth->restrict ( 'Business_Queries.Reports.Create' );
		
		if (isset ( $_POST ['save'] )) {
		    $this->gcrud->stats(array('module' => 'business_queries'));
			if ($insert_id = $this->save_business_queries ()) {
				// Log the activity
				log_activity ( $this->auth->user_id(), lang ( 'act_create_record' ) . ': ' . $insert_id . ' : ' . $this->input->ip_address (), 'business_queries' );
				
				Template::set_message ( sprintf(lang('msg_save_success'), lang('business_query')), 'success' );
				redirect ( SITE_AREA . '/reports/business_queries' );
			} else {
				Template::set_message ( sprintf(lang('msg_create_failure'), lang('business_query')) . $this->business_queries_model->error, 'danger' );
			}
		}
		Assets::add_module_js ( 'business_queries', 'business_queries.js' );
		
		Template::set ( 'toolbar_title', lang ( 'create' ) . ' ' . lang('business_query') );
		Template::set_view ( 'view_query' );
		Template::render ();
	}
		
		// --------------------------------------------------------------------
	
	/**
	 * Allows editing of Business Queries data.
	 *
	 * @return void
	 */
	private function edit() {
		$id = $this->uri->segment ( 5 );
		
		if (empty ( $id )) {
			Template::set_message ( lang ( 'invalid_id' ), 'error' );
			redirect ( SITE_AREA . '/reports/business_queries' );
		}
		
		if (isset ( $_POST ['save'] )) {
		    $this->gcrud->stats(array('module' => 'business_queries'));
			$this->auth->restrict ( 'Business_Queries.Reports.Edit' );
			
			if ($this->save_business_queries ( 'update', $id )) {
				// Log the activity
				log_activity ( $this->auth->user_id(), lang ( 'act_edit_record' ) . ': ' . $id . ' : ' . $this->input->ip_address (), 'business_queries' );
				
				Template::set_message ( sprintf(lang('msg_save_success'), lang('business_query')), 'success' );
			} else {
				Template::set_message ( sprintf(lang('msg_edit_failure'), lang('business_query')) . $this->business_queries_model->error, 'danger' );
			}
		} else if (isset ( $_POST ['delete'] )) {
			$this->auth->restrict ( $this->permissionDelete );
			
			if ($this->business_queries_model->delete ( $id )) {
				// Log the activity
				log_activity ( $this->auth->user_id(), lang ( 'act_delete_record' ) . ': ' . $id . ' : ' . $this->input->ip_address (), 'business_queries' );
				
				Template::set_message ( sprintf(lang('msg_delete_success'), lang('business_query')), 'success' );
				
				redirect ( SITE_AREA . '/reports/business_queries' );
			} else {
				Template::set_message ( sprintf(lang('msg_delete_failure'), lang('business_query')) . $this->business_queries_model->error, 'danger' );
			}
		}
		Template::set ( 'business_queries', $this->business_queries_model->find ( $id ) );
		Template::set ( 'toolbar_title', lang ( 'edit' ) . ' ' . lang('business_query') );
		Template::set_view ( 'view_query' );
		Template::render ();
	}

	//--------------------------------------------------------------------

	//--------------------------------------------------------------------
	// !PRIVATE METHODS
	//--------------------------------------------------------------------

	/**
	 * Summary
	 *
	 * @param String $type Either "insert" or "update"
	 * @param Int	 $id	The ID of the record to update, ignored on inserts
	 *
	 * @return Mixed    An INT id for successful inserts, TRUE for successful updates, else FALSE
	 */
	private function save_business_queries($type = 'insert', $id = 0) {
		if ($type == 'update') {
			$_POST ['id'] = $id;
		}
		$data = array ();
		$data ['listing_id'] = $this->input->post ( 'listing' );
		$data ['user_id'] = $this->input->post ( 'user' );
		$data ['name'] = $this->input->post ( 'name' );
		$data ['email'] = $this->input->post ( 'email' );
		$data ['phone'] = $this->input->post ( 'phone' );
		$data ['message'] = $this->input->post ( 'message' );
		$data ['ip'] = $this->input->ip_address ();
		
		if ($type == 'insert') {
			$id = $this->business_queries_model->insert ( $data );
			
			if (is_numeric ( $id )) {
				$return = $id;
			} else {
				$return = FALSE;
			}
		} elseif ($type == 'update') {
			$data ['modified_ip'] = $this->input->ip_address();
			$return = $this->business_queries_model->update ( $id, $data );
		}
		
		return $return;
	}
}