<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Migration_Install_business_queries extends Migration
{
	/**
	 * The name of the database table
	 *
	 * @var String
	 */
	private $table_name = 'business_queries';
	private $replies_table_name = 'query_replies';

	/**
	 * The table's fields
	 *
	 * @var Array
	 */
	private $fields = array(
		'id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'auto_increment' => TRUE,
		),
		'module_id' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default' => 0
		),
		'row_id' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'null' => FALSE,
			'default' => 0
		),
		'listing_id' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'null' => FALSE,
			'default' => 0
		),
		'user_id' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'null' => FALSE,
			'default' => 0
		),
		'owner_id' => array( // Member Area - Display Business Queries Counts to the user
			'type' => 'BIGINT',
			'constraint' => 20,
			'null' => FALSE,
			'default' => 0
		),
		'name' => array(
			'type' => 'VARCHAR',
			'constraint' => 100,
			'null' => FALSE,
		),
		'email' => array(
			'type' => 'VARCHAR',
			'constraint' => 100,
			'null' => FALSE,
		),
		'phone' => array(
			'type' => 'VARCHAR',
			'constraint' => 20,
			'null' => TRUE,
		),
		'message' => array(
			'type' => 'TEXT',
			'null' => FALSE,
		),
		'status' => array(
			'type' => 'TINYINT',
			'constraint' => 2,
			'default' => 1,
			'null' => FALSE,
		),
		'replied' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'default' => 0,
			'null' => FALSE,
		),
		'ip' => array(
			'type' => 'VARCHAR',
			'constraint' => 50,
			'null' => FALSE,
		),
		'deleted' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'default' => 0,
			'null' => FALSE,
		),
		'posted_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'modified_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'modified_ip' => array(
			'type' => 'VARCHAR',
			'constraint' => 50,
			'null'=> TRUE
		)
	);
	
	/**
	 * The replies table's fields
	 *
	 * @var Array
	 */
	private $replies_fields = array(
		'id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'auto_increment' => TRUE
		),
		'query_id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'default' => 0
		),
		'user_id' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'default' => 0
		),
		'replied_id' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'default' => 0
		),
		'subject' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE
		),
		'message' => array(
			'type' => 'TEXT',
			'null' => FALSE
		),
		'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'created_ip' => array(
			'type' => 'VARCHAR',
			'constraint' => 50,
			'null'=> TRUE
		)
	);

	/**
	 * Install this migration
	 *
	 * @return void
	 */
	public function up()
	{
		$this->dbforge->add_field($this->fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('user_id', false);
		$this->dbforge->add_key('listing_id', false);
		$this->dbforge->create_table($this->table_name);
		$module_meta_index_sql = "ALTER TABLE " . $this->db->dbprefix . $this->module_meta_table . " ADD INDEX `module` (`module_id`, `row_id`)";
		$this->db->query ($module_meta_index_sql);
		
		// Query Replies Tabe
		$this->dbforge->add_field($this->replies_fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('query_id', false);
		$this->dbforge->add_key('user_id', false);
		$this->dbforge->add_key('replied_by', false);
		$this->dbforge->create_table($this->replies_table_name);
	}

	//--------------------------------------------------------------------

	/**
	 * Uninstall this migration
	 *
	 * @return void
	 */
	public function down()
	{
		$this->dbforge->drop_table($this->table_name);
		$this->dbforge->drop_table($this->replies_table_name);
	}

	//--------------------------------------------------------------------

}