<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class Business_queries_model extends BF_Model {

	protected $table_name	= "business_queries";
	protected $key			= "id";
	protected $soft_deletes	= true;
	protected $date_format	= "datetime";

	protected $log_user 	= FALSE;

	protected $set_created	= true;
	protected $set_modified = true;
	protected $created_field = "created_on";
	protected $modified_field = "modified_on";

	/*
		Customize the operations of the model without recreating the insert, update,
		etc methods by adding the method names to act as callbacks here.
	 */
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 		= array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	/*
		For performance reasons, you may require your model to NOT return the
		id of the last inserted row as it is a bit of a slow method. This is
		primarily helpful when running big loops over data.
	 */
	protected $return_insert_id 	= TRUE;

	// The default type of element data is returned as.
	protected $return_type 			= "object";

	// Items that are always removed from data arrays prior to
	// any inserts or updates.
	protected $protected_attributes = array();

	/*
		You may need to move certain rules (like required) into the
		$insert_validation_rules array and out of the standard validation array.
		That way it is only required during inserts, not updates which may only
		be updating a portion of the data.
	 */
	protected $insert_validation_rules 	= array();
	protected $skip_validation 			= TRUE;

		
	/**
	 * Email Template Fields
	 *
	 * @return array
	 */
	public function email_templates($role = 'admin') {
	    $modules = gc_modules();
	    $module_options = array('name:placeholder_select;value:');
	    foreach($modules as $module) {
	        $module_options[] = 'name:' . $module . ';value:' . $module;
	    }
	    $module_options = implode('|', $module_options);
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'module_table',
	            'label' => 'label_module',
	            'frontend' => false,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => $module_options,
	            'options_detailed' => 1,
	            'options_lang' => 1,
	            'class' => 3,
	            'rules' => 'trim|unique_email_template[query]'
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'user_type',
	            'label' => 'bf_user',
	            'frontend' => false,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'name:placeholder_select;value:|name:us_admin;value:admin|name:bf_user;value:user|name:us_guest;value:guest',
	            'options_detailed' => 1,
	            'options_lang' => 1,
	            'class' => 3,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'action_type',
	            'label' => 'label_action',
	            'frontend' => false,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'name:placeholder_select;value:|name:add;value:add|name:edit;value:edit|name:delete;value:delete|name:em_status_update;value:status_update',
	            'options_detailed' => 1,
	            'options_lang' => 1,
	            'class' => 3,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'name',
	            'label' => 'label_name',
	            'placeholder' => 'placeholder_name',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
	            'rules' => 'required|trim|max_length[50]'
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'subject',
	            'label' => 'label_subject',
	            'placeholder' => 'placeholder_subject',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'required|trim|max_length[255]'
	        ),
	        array(
	            'type' => 'textarea',
	            'name' => 'message',
	            'label' => 'label_message',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'custom_rules' => 'editor',
	            'field_class' => 'editor',
	            'class' => 9,
	            'rules' => 'trim|required',
	        ),
	        array(
	            'type' => 'textarea',
	            'name' => 'sms_text',
	            'label' => 'sms_text',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'class' => 9,
	            'field_extra' => 'rows:3',
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'textarea',
	            'name' => 'params',
	            'label' => 'em_params',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'class' => 9,
	            'tooltip' => lang('em_params_help'),
	            'field_extra' => 'rows:3',
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'status',
	            'label' => 'label_status',
	            'frontend' => false,
	            'admin_only' => false,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_on',
	            'label' => 'created_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_on',
	            'label' => 'modified_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        )
	    );
	    return $fields;
	}
	
	/**
	 * Activate/Deactivate  
	 */
	public function _activate_deactivate_spam($id, $status) {
		$this->db->update($this->table_name, array('status' => $status), array('id' => $id));
		if ($this->db->affected_rows() != 1) {
			return FALSE;
		}
		return TRUE;
	}
	
	/**
	 * To display total number of business queries (at left hand side) received by a particular business owner
	 * For pagination and count display
	 * @param number $user_id
	 */
	public function count_business_queries($user_id, $module = null) {
		if($user_id) {
		    $module = empty($module) ? 'listings' : $module;
			$sql = "SELECT COUNT(BQ.id) AS TotalCount";
			$sql .= " FROM " . $this->db->dbprefix . "business_queries BQ";
			$sql .= " INNER JOIN " . $this->db->dbprefix . $module . " M ON M.id = BQ.row_id AND BQ.module_id = " . gc_module_id($module);
			$sql .= " WHERE BQ.deleted = 0 AND BQ.status_id = 2";
			if(in_array($module, gc_modules())) {
				$sql .= " AND M.user_id = " . $user_id;
			}
			if($module != null) {
				$sql .= " AND BQ.module_id = " . gc_module_id($module);
			}
			$result = $this->db->query($sql)->row();
			return $result ? $result->TotalCount : 0;
		}
	}
	
	/**
	 * To display total number of posted queries (at left hand side) posted by a particular user
	 * For pagination and count display
	 * @param number $user_id
	 */
	public function count_posted_queries($user_id, $module = null) {
		if($user_id) {
			$result = $this->db->where(array('user_id' => $user_id, 'module_id' => gc_module_id($module), 'deleted' => 0))->count_all_results('business_queries');
			return $result;
		}
	}
	
	/**
	 * Get sender email address and name from database in order to send a business query reply
	 * @param int $id
	 * @return object
	 */
	public function get_email_addresses($id) {
		$this->db->select($this->db->dbprefix .'business_queries.email as email_to, '.$this->db->dbprefix .'business_queries.name, L.email as email_from, L.id as listing_id, L.title, L.slug');
		$this->db->join($this->db->dbprefix .'listings L','L.id = '. $this->db->dbprefix .'business_queries.listing_id AND L.deleted = 0', 'INNER');
		return $this->business_queries_model->find_by(array($this->db->dbprefix .'business_queries.id' => $id));
	}
	
	/**
	 * Get Business Query
	 * @param number $query_id
	 * @return object
	 */
	public function get_query($query_id) {	    
	    $sql = "SELECT M.*, U.display_name AS user";
	    $sql .= " FROM " . $this->db->dbprefix . "business_queries M";
	    $sql .= " LEFT JOIN " . $this->db->dbprefix . "users U ON U.id = M.user_id";
	    $sql .= " WHERE M.id = " . $query_id;	    
	    $result = $this->db->query ( $sql )->row();
	    return $result;
	}
	
	/**
	 * Display query replies to the Admin and Member
	 * @param int $query_id
	 * @param int $offset
	 * @param int $limit
	 */
	public function query_replies($query_id, $offset = 0, $limit = 0) {
		$sql = "SELECT QR.*, U.display_name AS user";
		$sql .= " FROM " . $this->db->dbprefix . "query_replies QR";
		$sql .= " INNER JOIN " . $this->db->dbprefix . "users U ON U.id = QR.replied_by";
		$sql .= " WHERE QR.query_id = " . (int) $query_id;
		$sql .= " ORDER BY QR.created_on DESC";
		if($limit != 0) { // Admin Area display rows with pagination
			$sql .= " LIMIT " . $offset . ", " . $limit;
		}
		$query = $this->db->query ( $sql );
		return $query->result();
	}
	
	/**
	 * Get Business Owner Email for Posted Replies
	 * @param object $query
	 * @return boolean
	 */
	public function get_owner_email($query) {
		if($query) {
			if($query->module_id == 1) {
				$this->load->model('listings/listings_model');
				$listing = $this->listings_model->select('email')->find($query->row_id);
				if($listing && !empty($listing->email)) {
					return $listing->email;
				} else {
					return false;
				}
			} else {
				$sql = "SELECT L.email FROM " . $this->db->dbprefix . gc_module_name($query->module_id) ." M";
				$sql .= " INNER JOIN " . $this->db->dbprefix . "listings L ON L.id = M.listing_id";
				$sql .= " WHERE M.id = " . $query->row_id;
				$result = $this->db->query($sql)->row();
				if($result && !empty($result->email)) {
					return $result->email;
				} else {
					return false;
				}
			}
		} else {
			return false;
		}
	}
	
	/**
	 * Get Business Query for Email
	 * @param number $id
	 */
	public function get_email_data($id) {
	    $this->db->select('business_queries.*, U.display_name');
	    $this->db->join($this->db->dbprefix . 'users U','U.id = business_queries.user_id', 'left');
	    $record = $this->business_queries_model->find($id);
	    if($record) {
	        $module = gc_module_name($record->module_id);
	        if(in_array($module, gc_modules())) {
	            if(!empty($record->row_id)) {
    	           // get module information
    	            $select = 'M.*';
    	            $this->db->from($module . ' M');
    	            if($module != 'listings') {
    	                $this->db->join('listings L', 'L.id = M.listing_id', 'LEFT');
    	                $select .= ', L.contact_person, L.email, L.phone_number';
    	            }
    	            $row = $this->db->select($select)->where('M.id', $record->row_id)->get()->row();
    	            if($row) {
    	                $record->contact_person = empty($row->contact_person) ? lang('label_sir_madam') : $row->contact_person;
    	                $record->author_email = $row->email;
    	                $record->author_phone_number = $row->phone_number;
    	                $record->module_title = $row->title;
    	                $record->module_slug = $row->slug;
    	                $record->module_city = isset($row->city) ? $row->city : null;
    	            }
    	            $module_url = gc_url_format($module, $record->module_slug, $record->row_id, $record->module_city);
    	            $record->module_url = anchor($module_url, $module_url);
	            }
	        }
	    }
	    return $record;
	}
}
