<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Contact Queries Controller
 */
class Reports extends Admin_Controller {
	protected $permissionCreate = 'Callbacks.Reports.Create';
	protected $permissionDelete = 'Callbacks.Reports.Delete';
	protected $permissionEdit   = 'Callbacks.Reports.Edit';
	protected $permissionView   = 'Callbacks.Reports.View';
	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct() {
		parent::__construct();

		$this->auth->restrict($this->permissionView);
		$this->load->model('callbacks_model', null, true);
		$this->lang->load('callbacks');
		
		//LOAD TinyMCE EDITOR
        Assets::add_js(array('tinymce/tinymce.min.js'));
        $this->form_validation->set_error_delimiters("<span class='error'>", "</span>");
        
		Template::set_block('sub_nav', 'reports/_sub_nav');

		//Assets::add_module_js('callbacks', 'callbacks.js');
	}

	//--------------------------------------------------------------------

	/**
	 * Display a list of Contact Queries data.
	 *
	 * @return void
	 */
	public function index($filter='all', $offset = 0) {
		Assets::add_js('admin-reply-email.js');
		// Do we have any actions?
		if (isset($_POST['submit'])) {
		    $this->gcrud->stats(array('module' => 'callbacks'));
			$action = $this->input->post('submit');
			$checked = $this->input->post('checked');
			if($checked) {
				$module_name = count($checked) == 1 ? lang('singular_callbacks') : lang('label_callbacks') ;
				switch(strtolower($action)) {
					case 'activate':
						$this->gc->update_status('callbacks', $checked, 'status', 1);
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_updated')), 'success');
						break;
	
					case 'deactivate':
						$this->gc->update_status('callbacks', $checked, 'status', 0);
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_updated')), 'success');
						break;
	
					case 'spam':
						$this->gc->update_status('callbacks', $checked, 'status', 2);
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_spammed')), 'success');
						break;
	
					case 'delete':
						$this->gc->update_status('callbacks', $checked, 'deleted', 1);
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_deleted')), 'success');
						break;
	
					case 'purge':
						$this->gc->purge_records('callbacks', $checked);
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_deleted')), 'success');
						break;
	
					case 'restore':
						$this->gc->update_status('callbacks', $checked, 'deleted', 0);
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_restored')), 'success');
						break;
				}
			} else {
				Template::set_message(lang('msg_no_row_selected'), 'error');
			}
		}
		 
		// Change Status
		if (isset($_POST['change_status']))	 {
		    $this->gcrud->stats(array('module' => 'callbacks'));
		    $row_id = $_POST['change_status'];
		    if(!empty($_POST['query_status_' . $row_id])) {
		        $new_status = $_POST['query_status_' . $row_id];
			    Template::set_message(sprintf(lang('msg_action_success'), lang('singular_callbacks'), lang('label_updated')), 'success');
			    $this->gc->update_status('callbacks', array($_POST['change_status']), 'status', $new_status);
			}
		}
		 
		// Actions done, now display the view
		$where = "deleted = 0";
		// Filters
		$filter_type = $filter;
		 
		switch($filter_type) {
			case 'replied':
				$where .= " AND CQ.status = 1";
				break;
	
			case 'not_replied':
				$where .= " AND CQ.status = 0";
				break;
				 
			case 'spammed':
				$where .= " AND CQ.status = 2";
				break;
				 
			case 'deleted':
				$where = "deleted = 1";
				break;
				 
			case 'all':
				// Nothing to do
				break;
				 
			default:
				show_404("callbacks/index/$filter/");
		}
		
		$searchterm = $this->gc->search('callbacks', $this->uri->segment(5), '/reports/callbacks');
		if(!empty($searchterm)) {
			//$where .= " AND (C.id LIKE '%". $searchterm ."%' OR C.username LIKE '%". $searchterm ."%' OR C.title LIKE '%". $searchterm ."%' OR C.created_on LIKE '%". $searchterm ."%')";
		}
		
		$sort_by = "CQ.id DESC";
		$order_by = $this->gc->sort('callbacks', 'CQ');
		if(!empty($order_by)) {
			$sort_by = $order_by;
		}
		 
		$records = $this->callbacks_model->get_callbacks($offset, $this->limit, $searchterm, $where, $sort_by);
		 
		$total_records = $this->callbacks_model->get_callbacks($offset, $this->limit, $searchterm, $where, $sort_by, 1);
		 
		$pagerUriSegment = 6;
		$pagerBaseUrl = site_url(SITE_AREA . '/reports/callbacks/index/' . $filter) . '/';
	
		$limit  = $this->limit;
	
		$this->load->library('pagination');
		$pager['base_url']    = $pagerBaseUrl;
		$pager['total_rows']  = $total_records;
		$pager['per_page']    = $limit;
		$pager['uri_segment'] = $pagerUriSegment;
	
		$this->pagination->initialize($pager);
	
		Template::set('index_url', site_url(SITE_AREA .'/reports/callbacks/index/'));
		Template::set('filter_type', $filter_type);
	
		Template::set('records', $records);
	
		Template::set('toolbar_title', lang('label_callbacks'));
	
		Template::render();
	}
	
	/**
	 * Allows editing of Contact Queries data.
	 *
	 * @return void
	 */
	public function view() {
		Assets::add_js('admin-reply-email.js');
		$id = $this->uri->segment(5);
		if (empty($id)) {
			Template::set_message(lang('invalid_id'), 'error');
	
			redirect(SITE_AREA . '/reports/callbacks');
		}	
		if (isset($_POST['save'])) {
		    $this->gcrud->stats(array('module' => 'callbacks'));
			$this->auth->restrict($this->permissionEdit);
	
			if ($this->save_callbacks_status($id)) {
				log_activity($this->auth->user_id(), lang('act_edit_record') . ': ' . $id . ' : ' . $this->input->ip_address(), 'callbacks');
				Template::set_message(sprintf(lang('msg_save_success'), lang('singular_callbacks')), 'success');
				redirect(SITE_AREA . '/reports/callbacks');
			}
	
			// Not validation error
			if ( ! empty($this->callbacks_model->error)) {
				Template::set_message(sprintf(lang('msg_edit_failure'), lang('singular_callbacks')) . $this->callbacks_model->error, 'danger');
			}
		} elseif (isset($_POST['delete'])) {
			$this->auth->restrict($this->permissionDelete);
	
			if ($this->callbacks_model->delete($id)) {
				log_activity($this->auth->user_id(), lang('act_delete_record') . ': ' . $id . ' : ' . $this->input->ip_address(), 'callbacks');
				Template::set_message(sprintf(lang('msg_delete_success'), lang('singular_callbacks')), 'success');
	
				redirect(SITE_AREA . '/reports/callbacks');
			}
	
			Template::set_message(sprintf(lang('msg_delete_failure'), lang('singular_callbacks')) . $this->callbacks_model->error, 'danger');
		}
	
		$record = $this->callbacks_model->get_callbacks_view($id);
		Template::set('callbacks', $record);
	
		Template::set('toolbar_title', lang('label_view') . ' ' .$record->name . ' ' . lang('singular_callbacks'));
		Template::set_view('callbacks_form');
		Template::render();
	}
	
	/**
	 * Allows editing of Contact Queries data.
	 *
	 * @return void
	 */
	public function edit() {
		$id = $this->uri->segment(5);

		if (empty($id)) {
			Template::set_message(lang('callbacks_invalid_id'), 'error');
			redirect(SITE_AREA .'/reports/callbacks');
		}

		if (isset($_POST['save'])) {
		    $this->gcrud->stats(array('module' => 'callbacks'));
			$this->auth->restrict($this->permissionEdit);

			if ($this->save_callbacks('update', $id)) {
				// Log the activity
				log_activity($this->auth->user_id(), lang('act_edit_record') .': '. $id .' : '. $this->input->ip_address(), 'callbacks');

				Template::set_message(sprintf(lang('msg_save_success'), lang('singular_callbacks')), 'success');
			} else {
				Template::set_message(sprintf(lang('msg_edit_failure'), lang('singular_callbacks')) . $this->callbacks_model->error, 'danger');
			}
		} else if (isset($_POST['delete'])) {
		    $this->gcrud->stats(array('module' => 'callbacks'));
			$this->auth->restrict($this->permissionDelete);

			if ($this->callbacks_model->delete($id)) {
				// Log the activity
				log_activity($this->auth->user_id(), lang('callbacks_act_delete_record') .': '. $id .' : '. $this->input->ip_address(), 'callbacks');

				Template::set_message(sprintf(lang('msg_delete_success'), lang('singular_callbacks')), 'success');

				redirect(SITE_AREA .'/reports/callbacks');
			} else {
				Template::set_message(sprintf(lang('msg_delete_failure'), lang('singular_callbacks')) . $this->callbacks_model->error, 'danger');
			}
		}
		Template::set('callbacks', $this->callbacks_model->find($id));
		Template::set('toolbar_title', lang('edit') .' Contact Queries');
		Template::render();
	}

	//--------------------------------------------------------------------
	// !PRIVATE METHODS
	//--------------------------------------------------------------------
	
	/**
	 * Save Status.
	 */
	private function save_callbacks_status($id) {
		$data = array();
		$data['status'] = $this->input->post('status');
		return $this->callbacks_model->update($id, $data);
	}

	/**
	 * Summary
	 *
	 * @param String $type Either "insert" or "update"
	 * @param Int	 $id	The ID of the record to update, ignored on inserts
	 *
	 * @return Mixed    An INT id for successful inserts, TRUE for successful updates, else FALSE
	 */
	private function save_callbacks($type='insert', $id=0) {
		if ($type == 'update') {
			$_POST['id'] = $id;
		}

		$data = array();
		$data['user_id']        = $this->input->post('user_id');
		$data['name']           = $this->input->post('name');
		$data['email']          = $this->input->post('email');
		$data['phone_number']   = $this->input->post('phone_number');

		if ($type == 'insert') {
		    $data['created_on']        = date('Y-m-d H:i:s');
		    $data['created_ip']        = $this->input->ip_address();
			$id = $this->callbacks_model->insert($data);

			if (is_numeric($id)) {
				$return = $id;
			} else {
				$return = FALSE;
			}
		} elseif ($type == 'update') {
		    $data['modified_on'] = date('Y-m-d H:i:s');
		    $data['modified_ip'] = $this->input->ip_address();
			$return = $this->callbacks_model->update($id, $data);
		}
		return $return;
	}	
	
	/**
	 * Manage Email Templates
	 * @param string $module
	 * @param string $filter
	 * @param number $offset
	 */
	public function manage($module = 'email_templates', $filter = 'all', $offset = 0) {
	    $extra = array(
	        'model' => 'callbacks',
	        'context' => 'reports',
	        'module_name' => 'callbacks',
	        'fields' => $module,
	        'prefix' => 'callback'
	    );
	    $this->gc->module_view($module, $filter, $offset, 'admin', $extra);
	}
	
	/**
	 * Add/Edit Email Templates Data
	 *
	 * @return void
	 */
	public function form($module = 'email_templates', $action = 'create') {
	    Assets::add_js(array(
	        'tinymce/tinymce.min.js',
	    ));
	    $params = array(
	        'selectize' => true,
	        'model' => 'callbacks',
	        'module_name' => 'callbacks',
	        'context' => 'reports',
	        'fields_method' => $module,
	        'prefix' => 'callback'
	    );
	    $this->gcrud->form($module, $action, 'admin', $params);
	}
}