<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Categories controller
 */
class Content extends Admin_Controller
{
	protected $permissionCreate = 'Categories.Content.Create';
	protected $permissionDelete = 'Categories.Content.Delete';
	protected $permissionEdit   = 'Categories.Content.Edit';
	protected $permissionView   = 'Categories.Content.View';
	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct()
	{
		parent::__construct();

		$this->auth->restrict($this->permissionView);
		clear_cache('categories');
		$this->load->model('categories_model', null, true);
		$this->lang->load('categories');		
		
		$config = array(
				'field' => 'slug',
				'title' => 'name',
				'table' => 'categories',
				'id' => 'id',
		);
		$this->load->library('slug', $config);
		
		//LOAD TinyMCE EDITOR
		Assets::add_js(array(
		      'tinymce/tinymce.min.js',
		));
				
		Assets::add_module_js('categories', 'category.js');
		Template::set_block('sub_nav', 'content/_sub_nav');
	}

	/**
	 * Displays a list of all categories to the admin.
	 *
	 * @return void
	 */
	public function index($id = 0, $offset = 0) {
		// redirect back and forth between pages
		$this->session->set_userdata('categories_last_visited_page', current_url());
		
		// Status Change
		if (isset($_POST['status_change'])) {
			$row_id = $_POST['status_change'];
			// get status for the recursive update
			$row = $this->categories_model->select('id, active')->find($row_id);
			$status = $row->active == 1 ? 0 : 1;
			$this->update_status($row_id, $status);
		}
		
		// Do we have any actions?
		if (isset($_POST['submit'])) {
			$action = $this->input->post('submit');
			$checked = $this->input->post('checked');
			if($checked) {
				$chk_module = (count($checked) == 1) ? lang('label_category') : lang('label_categories');
				switch(strtolower($action)) {					
					case 'activate':	
						$this->change_status($checked, 1);
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $chk_module, lang('label_activated')), 'success');
						if($this->uri->segment(5)) {
							Template::redirect(site_url(SITE_AREA ."/content/categories/index/" .$this->uri->segment(5)));
						} else {
							Template::redirect(site_url(SITE_AREA ."/content/categories"));
						}
						break;
							
					case 'deactivate':
						$this->change_status($checked, 0);						
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $chk_module, lang('label_deactivated')), 'success');
						if($this->uri->segment(5)) {
							Template::redirect(site_url(SITE_AREA ."/content/categories/index/" .$this->uri->segment(5)));
						} else {
							Template::redirect(site_url(SITE_AREA ."/content/categories"));
						}
						break;
				}
			}
		}
		
		// Deleting anything?
		if (isset($_POST['delete'])) {
			$checked = $this->input->post('checked');
			// To delete information from the Ads module
			$this->load->model('ads/ads_model');
			if (is_array($checked) && count($checked)) {
				$chk_module = (count($checked) == 1) ? lang('label_category') : lang('label_categories');
				$result = FALSE;
				foreach ($checked as $pid) {
					$result = $this->_purge($pid);					
				}
				if ($result) {
					Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $chk_module, lang('label_deleted')), 'success');
					Template::redirect(site_url(SITE_AREA ."/content/categories"));
				} else {
					Template::set_message(lang('delete_failure') . $this->categories_model->error, 'danger');
				}
			} else {
				Template::set_message(lang('msg_no_row_selected'), 'error');
				Template::redirect(site_url(SITE_AREA ."/content/categories"));
			}
		}
		
		$where = '';
		$searchterm = $this->gc->search('categories', $this->uri->segment(6), '/content/categories/');
		if(!empty($searchterm)) {
			$where .= "id LIKE '%" .$searchterm . "%' OR name LIKE '%" .$searchterm . "%'";
		}
		
		$sort_by = "name ASC";
		$order_by = $this->gc->sort('categories', 'C');
		if(!empty($order_by)) {
			$sort_by = $order_by;
		}
		
		$records = $this->categories_model->admin_categories(0, $id, $where, $sort_by, $offset, $this->limit);
		$total_categories = $this->categories_model->admin_categories(1, $id, $where, $sort_by, $offset, $this->limit);
		
		// Check whether categories or subcategories page called
		$parent_id = $this->uri->segment(5);
		if($parent_id) {
			$parent_name = $this->categories_model->select('name')->find($parent_id);
			Template::set('parent_name', $parent_name->name);
		}
		// Pagination
		$this->load->library('pagination');
		
		if($id != NULL) {
			$pager['base_url'] = site_url(SITE_AREA ."/content/categories/index/" .$id."/");
		} else {
			$pager['base_url'] = site_url(SITE_AREA ."/content/categories/index/0/");
		}
		$pager['total_rows'] = $total_categories;
		$pager['per_page'] = $this->limit;
		$pager['uri_segment']	= 6;
		
		$this->pagination->initialize($pager);
		// Pass parent_id as a variable
		Template::set('parent_id', $parent_id);
		Template::set('records', $records);
		// Counts redirect - Last visited URL
		$this->session->set_userdata('last_visited_url', current_url());
		if($id == NULL) {
			Template::set('toolbar_title', lang('manage_categories'));
		} else {
			Template::set('toolbar_title', lang('manage_subcategories'));
		}
		Template::render();
	}
	
	/**
	 * Purge Category
	 * @param integer $pid
	 */
	private function _purge($pid) {
		// Change category status to inactive to remove category information from the Tweak field 
		// (categories) in the modules table
		$this->update_status($pid, 0, false);
		// Purge Category Information
		return $this->categories_model->purge_category($pid);
	}
	
	/**
	 * Change Multiple Categories Status
	 * @param array $checked
	 * @param number $status
	 */
	private function change_status($checked, $status = 0) {
		//$this->gc->update_status('categories', $checked, 'active', 0);
		// 1 - perform operation on child categories
		foreach($checked as $category_id) {
			$this->recursive($category_id, $status);
		}
		// 2 - perform operation on checked categories
		$this->recursive_update_status($checked, $status, false, true);

		// 3 - Update Tweak Field (categories) in the Modules Table
		$this->gc->update_tweak_fields('categories', $category_id);
	}
	
	/**
	 * Perform recursive operation
	 * @param array $checked
	 * @param int $status - Not required in case of edit
	 */
	private function recursive($category_id, $status, $edit = false) {
		$sub_categories = $this->categories_model->select('id')->find_all_by(array('parent_id' => $category_id));
		if($sub_categories) {
			foreach($sub_categories as $sub_item) {
				$subsub_categories = $this->categories_model->select('id')->find_all_by(array('parent_id' => $sub_item->id));
				if($subsub_categories) {
					$this->recursive_update_status($subsub_categories, $status, $edit);
				}					
			}
			$this->recursive_update_status($sub_categories, $status, $edit);
		}
	}
	
	/**
	 * Update Status, Restore and Delete Records
	 * @param array $data - fields to be updated
	 */
	private function recursive_update_status($records, $status, $edit, $checked = false) {
		$data = array();
		foreach($records as $record) {
			if($checked) {
				$id = $record;
			} else {
				$id = $record->id;
			}
			if(!$edit) {
				$data[] = array(
					'id' => $id,
					'active' => $status
				);
			}
		}
		if($edit) {
			$this->gc->save_relation_batch($records, 'categories', $this->input->post('modules'));
		}
		if(!empty($data)) {
			$this->db->update_batch('categories', $data, 'id');
		}
	}

	/**
	 * Allow admin to create a new category or subcategory.
	 * @return void
	 */
	public function create() {
		Assets::add_css('fontawesome-iconpicker.min.css');
		Assets::add_js('fontawesome-iconpicker.min.js');
		$this->auth->restrict($this->permissionCreate);
		if (isset($_POST['save'])) {
			if ($insert_id = $this->save_categories()) {
				// Log the activity
				log_activity($this->current_user->id, lang('act_create_record') .': '. $insert_id .' : '. $this->input->ip_address(), 'categories');

				Template::set_message(sprintf(lang('msg_save_success'), lang('label_category')), 'success');
				Template::redirect($this->session->userdata('categories_last_visited_page') ? $this->session->userdata('categories_last_visited_page') : $this->previous_page);
			} else {
				Template::set_message(sprintf(lang('msg_create_failure'), lang('label_category')) . $this->categories_model->error, 'danger');
			}
		}
		
		//Display Parent Categories to the user
		$categories_no_parents = $this->categories_model->get_no_parents();
		Template::set('categories_no_parents', $categories_no_parents); //Pass data to view
		
		// Retrieve Subcategories on POSTBACK error
		if($this->input->post('update_parent_id')) {
			$sub_categories = $this->categories_model->select('id, name')->order_by('name', 'ASC')->find_all_by('parent_id', $this->input->post('update_parent_id'));
			Template::set(array(
				'parent_category' => $this->input->post('update_parent_id'),
				'sub_categories' => $sub_categories,
				'sub_category' => $this->input->post('subcategory_id')
			));
		}		
		Template::set('help_image', sprintf(lang('image_help'), settings_item('lst.image_file_size'), settings_item('lst.image_width'), settings_item('lst.image_height')));
		Template::set('toolbar_title', lang('add') . ' '.lang('label_category'));
		Template::set_view('content/categories_form');
		Template::render();
	}

	/**
	 * Allows editing of Categories data.
	 *
	 * @return void
	 */
	public function edit() {
		Assets::add_css('fontawesome-iconpicker.min.css');
		Assets::add_js('fontawesome-iconpicker.min.js');
		$id = $this->uri->segment(5);
		if (empty($id)) {
			Template::set_message(lang('invalid_id'), 'danger');
			redirect(SITE_AREA .'/content/categories');
		}
		if (isset($_POST['save'])) {
			$this->auth->restrict($this->permissionEdit);
			if ($this->save_categories('update', $id)) {				
				// Update Categories Relations Table on Parent Update
				$sess_cl = $this->session->userdata('l_cat_update');
				$l_parent_id = (int) $this->input->post('update_parent_id');
				$l_sub_id = (int) $this->input->post('subcategory_id');				
				$this->session->unset_userdata('l_cat_update');
				
				// Update Tweak Field (Keywords) in the Module Table
				$this->gc->update_tweak_fields('categories', $id);
				
				// Log the activity
				log_activity($this->current_user->id, lang('act_edit_record') .': '. $id .' : '. $this->input->ip_address(), 'categories');

				Template::set_message(sprintf(lang('msg_save_success'), lang('label_category')), 'success');
				Template::redirect($this->session->userdata('categories_last_visited_page') ? $this->session->userdata('categories_last_visited_page') : $this->previous_page);
			} else {
				Template::set_message(sprintf(lang('msg_edit_failure'), lang('label_category')) . $this->categories_model->error, 'danger');
			}
		} else if (isset($_POST['delete'])) {
			$this->auth->restrict($this->permissionDelete);
			$this->load->model('ads/ads_model');
			if ($this->_purge($id)) {
				// Log the activity
				log_activity($this->current_user->id, lang('act_delete_record') .': '. $id .' : '. $this->input->ip_address(), 'categories');

				Template::set_message(sprintf(lang('msg_delete_success'), lang('label_category')), 'success');

				Template::redirect($this->session->userdata('categories_last_visited_page') ? $this->session->userdata('categories_last_visited_page') : $this->previous_page);
			} else {
				Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_category')) . $this->categories_model->error, 'danger');
			}
		}
		$record = $this->categories_model->find($id);
		
		// Multi Langauge Fields - Get translations if exists
		$record = $this->translations->get_translations('categories', $id, $record);
		
		/*--------------- CATEGORY SECTION ----------------------*/
		// Display Parent Categories to the user
		$categories_no_parents = $this->categories_model->get_no_parents($id);
		$sub_category = NULL;
		$result = $this->categories_model->select('parent_id')->find($record->parent_id); //FIND CATEGORY PARENT
		if($result && $result->parent_id != 0) {
			$get_parent = $this->categories_model->select('id')->find($result->parent_id); //GET PARENT CATEGORY
			$parent_category = $get_parent->id;
			$sub_category = $record->parent_id; //AND CONSIDER LISTING CATEGORY_ID AS SUB CATEGORY
		}
		else {
			$parent_category = $record->parent_id;
		}
		//GET ALL CATEGORIES
		$categories = $this->categories_model->get_parent_categories();
		
		//GET SUBCATEGORIES ONLY IF PARENT EXIST
		if($parent_category != $record->parent_id) {
			$sub_categories = $this->categories_model->select('id, name')->where('parent_id', $parent_category)->order_by('name', 'ASC')->find_all();
		} else {
			$sub_categories = NULL;
		}
		/* ------------ END OF CATEGORY SECTION ----------------*/
		
		Template::set( array(
			'categories_no_parents' => $categories_no_parents,
			'sub_categories' 	=> $sub_categories,
			'parent_category'	=> $parent_category,
			'sub_category'		=> $sub_category,
		)); //Pass data to view
		
		// Run recursive update only if the user changed the category module
		$relations = $this->gc->get_relations($id, 'categories');
		if($relations) {
			$update_module = array();
			foreach($relations as $relation) {
				$gc_module_name = gc_module_name($relation->toggle_module_id);
				if(is_string($gc_module_name)) {
					$update_module[trim($gc_module_name)] = 1;
				}
			}
		}
		if(!empty($update_module)) {
			$this->session->set_flashdata('category_recursive_update', $update_module);	
		}
		Template::set('relations', $relations);		
		Template::set('category', $record);	
		Template::set('help_image', sprintf(lang('image_help'), settings_item('lst.image_file_size'), settings_item('lst.image_width'), settings_item('lst.image_height')));
		 
		Template::set('toolbar_title', lang('edit') .' ' .lang('label_category'));
		Template::set_view('content/categories_form');
		Template::render();
	}

	//--------------------------------------------------------------------
	// !PRIVATE METHODS
	//--------------------------------------------------------------------

	/**
	 * Summary
	 * Save Category
	 * @param String $type Either "insert" or "update"
	 * @param Int	 $id	The ID of the record to update, ignored on inserts
	 *
	 * @return Mixed    An INT id for successful inserts, TRUE for successful updates, else FALSE
	 */
	private function save_categories($type = 'insert', $id = 0) {
		$this->load->model('images/images_model');
		if ($type == 'update') {
			$_POST['id'] = $id;
		}
		
		// validate translation fields
		$this->translations->validate_prepare_fields();
		
		$this->form_validation->set_rules('parent_id', 'lang:parent_id', 'trim');
		$this->form_validation->set_rules('name', 'lang:label_name', 'trim|required|callback_category_name_check');
		$this->form_validation->set_rules('subcategory_id', 'lang:subcategory_id', 'trim');
		$this->form_validation->set_rules('uploaded_image', 'lang:label_image', 'trim');
		$this->form_validation->set_rules('mobile_icon', 'lang:label_mobile_icon', 'trim|alpha_dash');
		//Check whether image is uploaded or not
		/*if(isset($_FILES['image']) && $_FILES['image']['size'] > 0){
			$file_data = $this->images_model->upload_image('image');
			if(!$file_data) {
				$_POST['file_upload'] = 'NONE';
				$this->form_validation->set_rules('file_upload', 'label:label_image', 'valid_file_upload');
			}
			// Avoid logo upload process, if the form has validation issues with other fields
			$_POST['uploaded_image'] = isset($file_data['upload_data']['file_name']) ? $file_data['upload_data']['file_name'] : '';
		}*/
		if ($this->form_validation->run() === false) {
			return false;
		}
		
		// unset non existing translation POST fields
		$this->translations->translate_unset();
		
		$data = array();
		// In case of Category Edit, parent_id must remain same to update the categories relation
		// but the below assignment of the parent_id is resulting in the POST value update
		// therefore, the parent_id value is assigned to a new POST variable
		$_POST['update_parent_id'] = $this->input->post('parent_id');
		if($this->input->post('subcategory_id')) {
			$data['parent_id']      = $this->input->post('subcategory_id');
		} else {
			$data['parent_id']      = $this->input->post('parent_id');
		}
		$data['name']        		= $this->input->post('name');
		$data['icon']        		= $this->input->post('icon');
		$data['mobile_icon']        = $this->input->post('mobile_icon');
		// copy image from the temp folder
		/*$image = $this->images_model->get_module_image($_POST['uploaded_image'], 'categories', $id);
		if(is_string($image)) {
			$data['image'] 	= $image;
		}*/
		$data['image']        		= $this->images_model->image_filemanager($this->input->post('image'), 'categories', $id);
		$data['description']        = $this->input->post('description');
		// Check whether slug is entered or not
		if($this->input->post('slug')) {
			$data['slug'] = $this->gc->slugify('categories', $this->input->post('slug'), $id, 'name', false);
		} else {
			$data['slug'] = $this->gc->slugify('categories', $this->input->post('name'), $id, 'name', false);
		}
		$data['meta_title']        	= $this->input->post('meta_title');
		$data['meta_keywords']      = $this->input->post('meta_keywords');	
		$data['meta_description']   = $this->input->post('meta_description');		
		$data['hits']  				= (int) $this->input->post('hits');	
		$data['priority']  			= (int) $this->input->post('priority');
		$data['active']        		= $this->input->post('active') ? 1 : 0;
		$selected_modules           = $this->input->post('modules');
		$modules_selected = array();
		if(!empty($selected_modules)) {
		    foreach($selected_modules as $selected_module => $selected_value) {
		        $modules_selected[] = gc_module_id($selected_module);
		    }
		}
		$data['module_relations']   = implode(',', $modules_selected);
		if ($type == 'insert') {
			$id = $this->categories_model->insert($data);

			if (is_numeric($id)) {
				$return = $id;
			} else {
				$return = FALSE;
			}
		} elseif ($type == 'update') {			
			$return = $this->categories_model->update($id, $data);		
		}
		
		// Save Category and Module relationship
		// A Category can belong to multiple modules
		if($return) {
		    // save hierarchy
		    $this->load->library('goclixy/gclosure');
		    $gclosure = new Gclosure('categories', 'categories_hierarchy');
		    $record = $this->db->where('id', $id)->get('categories')->row();
		    if($record) {
		        $parent_id = $record->parent_id;
		        $gclosure->delete($id, false); // do not delete from the categories table		        
		        $gclosure->add($id, ($parent_id == 0 ? $id : $parent_id)); // save hierarchy
		    }
		    
			// Run recursive update only if the user changed the category module
			if($this->session->flashdata('category_recursive_update') != $this->input->post('modules')) {
				$this->recursive($id, 1, true);
			}
						
			$this->gc->save_module_relations($id, $this->input->post('modules'), 'categories');
		}
		
		// save translations
		$this->translations->save_translations('categories', $id, $return);
		
		return $return;
	}	
	
	/**
	 * Category Name Existence Check
	 * Do not allow to create same category twice in one parent category
	 * @return boolean
	 */
	public function category_name_check($str) {
	    $id = (int) $this->input->post('id');
	    $parent_id = (int) ($this->input->post('subcategory_id') ? $this->input->post('subcategory_id') : $this->input->post('parent_id'));
	    $counts = $this->db->where("id != $id AND parent_id = $parent_id AND name = '$str'")->count_all_results('categories');
	    if($counts > 0) {
	        $this->form_validation->set_message('category_name_check', lang('category_name_check'));
	        return false;
	    }
	    return true;
	}
	
	/**
	 * Allow admin to change order of categories
	 */
	public function order() {
		//LOAD NESTED SORTABLE JAVASCRIPT AND CSS FILE
		Assets::add_css('nestedSortable/admin.css');
		Assets::add_js('nestedSortable/jquery-ui.min.js');
		Assets::add_js('nestedSortable/jquery.mjs.nestedSortable.js');
		
		Assets::add_js($this->load->view('content/categories_js', null, true), 'inline');
		Template::set('sortable', TRUE);
		Template::set('toolbar_title', lang('label_order'). ' ' . lang('label_categories'));
		Template::render();
	}
	
	/**
	 * Ajax function to order categories using nestedSortable
	 */
	public function order_ajax () {
		// LOAD NESTED SORTABLE JAVASCRIPT AND CSS FILE
		Assets::add_css('nestedSortable/admin.css');
		Assets::add_js('nestedSortable/jquery-ui.min.js');
		Assets::add_js('nestedSortable/jquery.mjs.nestedSortable.js');
		Assets::add_js($this->load->view('content/categories_js', null, true), 'inline');
		// Save order from ajax call
		if (isset($_POST['sortable'])) {
			$this->categories_model->save_order($_POST['sortable']);
		}
	
		// Load view
		Template::set('categories', $this->categories_model->get_nested());
		Template::set('toolbar_title', lang('label_order'). ' ' . lang('label_categories'));
		Template::render();
	}
	
	/**
	 * Change status of category from index page
	 * @param int $id
	 */
	public function update_status($id, $status, $redirect = true) {
		$this->auth->restrict($this->permissionEdit);		
		$this->recursive($id, $status);
		$sql = "UPDATE " . $this->db->dbprefix . "categories SET active = ". $status ." WHERE id = " . $id;
		$this->db->simple_query($sql);

		// Update Tweak Field (Keywords) in the Module Table
		$this->gc->update_tweak_fields('categories', $id);
		if($redirect) {
			Template::set_message(sprintf(lang('msg_action_success'), lang('label_category'), lang('label_updated')), 'success');
			Template::redirect(current_url());
		}
	}
	
	/**
	 * Update listings count for home page
	 */
	public function counts() {
		$this->categories_model->counts();
		// $this->load->model('listings/listings_model');
		// $this->listings_model->update_counts('categories');
  		Template::set_message(lang('message_count_success'), 'success');
  		redirect(site_url(SITE_AREA .'/content/categories'));  		
	}
	
	/**
	 * Display module counts
	 */
	public function display_counts($row_id) {
	    $this->load->model('listings/listings_model');
	    $this->listings_model->display_counts($row_id);
	}
	
	/**
	 * Update Categories Slug
	 */
	public function update_slug() {
		$this->gc->update_slug('categories', 'name', false, true);
	}
	
	/**
	 * Bulk insert and update category image
	 * image must exist in the assets/images/categories/ folder with name <CATEGORY NAME SLUG>.jpg 
	 */
	public function bulkimport_images() {
		$this->load->library('slug');
		$result = $this->categories_model->select('id, name')->find_all();
		if($result) {
			$data = array();
			foreach($result as $category) {
				$file_name = $category->id .'-'. $this->slug->create_slug($category->name). '.jpg';
				if(file_exists(realpath(FCPATH.'/assets/images/categories/' . $file_name))) {
					$data[] = array(
						'id' => $category->id,
						'image' => $file_name
					);
				}
				
			}
			if(!empty($data)) {
				$this->db->update_batch('categories', $data, 'id');
			}
		}
	}
	
	/**
	 * Check existence of all images
	 */
	public function check_img_existence() {
		$module = 'categories';
		$data = array();
		$result = $this->categories_model->select('id, name, slug, image')->where('id <', 1000 )->find_all();
		if($result) {
			foreach($result as $category) {
				if(!file_exists(realpath(FCPATH.'/assets/images/' . $module . '/' .$category->image))) {
					$data[] = array(
							'id' => $category->id,
							'name' => $category->name,
							'slug' => $category->slug,
							'image' => $category->image
					);
				}
			}
		}
	}
	
	/**
	 * Assign fields to a Category
	 * @param number $id
	 * @param string $module
	 */
	public function assign_fields($id) {
	    $this->load->model('fields/fields_model');
	    $this->fields_model->assign($id, 'categories');
	}
	
	/**
     * Display categories tree
	 */
	public function tree() {	    
	    $this->load->library('goclixy/gclosure');
	    $gclosure = new Gclosure('categories', 'categories_hierarchy');
	    $content = $gclosure->hierarchy_tree($this->gc->closure());
	    Template::set(array(
	        'toolbar_title' => lang('category_tree'),
	        'content' => $content
	    ));
	    Template::render();
	}
	
	/**
	 * Update Module relations
	 */
	public function update_relations($skip_modules = array('jobs', 'properties')) {	    
	    $site_modules = array_flip(gc_modules());
	    foreach($skip_modules as $skip_module) {
	        unset($site_modules[$skip_module]);
	    }
	    $site_modules = array_flip($site_modules);
	    $records = $this->db->get('categories')->result();
	    foreach($records as $record) {
	        $this->gc->save_module_relations($record->id, $site_modules, 'categories');
	    }
	    Template::set_message('Module relations successfully updated', 'success');
	    Template::redirect(site_url(SITE_AREA . '/content/categories'));
	}
	
	/**
	 * Update Applicable Modules for all or specific categories
	 * @param number $ids
	 */
	public function update_module_relations($ids = 0, $table = 'categories', $skip_modules = array()) {
	    $applicable_modules = gc_modules();
	    $applicable_module_ids = array();
	    foreach($applicable_modules as $applicable_module) {
	        if(!in_array($applicable_module, $skip_modules)) {
	           $applicable_module_ids[] = gc_module_id($applicable_module);
	        }
	    }
	    $module_relations = implode(',', $applicable_module_ids);
	    $sql = "UPDATE " . $this->db->dbprefix . $table . " SET module_relations = '" . $module_relations . "'";
	    if(!empty($ids)) {
	       $sql .= " WHERE id IN (" . $ids . ")";
	    }
	    $this->db->query($sql);
	}
	
	/**
	 * Tweak Bulk Imported Categories
	 * This is a slow process do it in chunks, rerun the method multiple times
	 */
	public function tweak_bulk($limit = 500) {
	    if(isset($_POST['save'])) {
    	    $this->load->library('slug');
    	    $records = $this->db->where('module_relations IS NULL')->limit($limit)->get('categories')->result();
    	    if($records) {
    	        $selected_modules = gc_modules();
    	        $selected_modules[] = 'videos';
    	        $flipped_modules = array();
    	        foreach($selected_modules as $selected_module) {
    	            $flipped_modules[$selected_module] = gc_module_id($selected_module);
    	        }
    	        $modules_selected = array();
    	        if(!empty($selected_modules)) {
    	            foreach($selected_modules as $selected_module) {
    	                $modules_selected[] = gc_module_id($selected_module);
    	            }
    	        }
    	        // save hierarchy
    	        $this->load->library('goclixy/gclosure');
    	        $gclosure = new Gclosure('categories', 'categories_hierarchy');
    	        $to_be_deleted = array();
    	        foreach($records as $record) {
    	            if(!empty($record->name)) {
    	                $data = array();
    	                $data['module_relations']   = implode(',', $modules_selected);
    	                if(empty($record->slug)) {
    	                    $data['slug'] = $this->slug->create_slug($record->name);
    	                }
    	                if(empty($record->icon)) {
    	                    $data['icon'] = $this->categories_model->random_icon();
    	                }
    	                $this->db->where('id', $record->id)->update('categories', $data);
    	                
    	                // save hierarchies
    	                $gclosure->delete($record->id, false); // do not delete from the categories table
    	                $gclosure->add($record->id, ($record->parent_id == 0 ? $record->id : $record->parent_id)); // save hierarchy
    	                
    	                // save relation
    	                $this->gc->save_module_relations($record->id, $flipped_modules, 'categories');
    	            } else {
    	                $to_be_deleted[] = $record->id;
    	            }
    	        }
    	        if(!empty($to_be_deleted)) {
    	            $this->db->where('id IN (' . implode(',', $to_be_deleted) . ')')->delete('categories');
    	            // set auto increment value to the last id
    	            $alter_autoinc = "ALTER TABLE " . $this->db->dbprefix . "categories AUTO_INCREMENT = 1";
    	            $this->db->simple_query($alter_autoinc);
    	        }
    	        Template::set_message(sprintf(lang('msg_action_success'), lang('gc_menu_categories'), lang('label_updated')), 'success');
    	    } else {
    	        Template::set_message(lang('msg_nothing_to_update'), 'danger');
    	    }
	    }
	    Template::set(array('toolbar_title' => lang('category_tweak')));
	    Template::set_view('tweak');
	    Template::render();
	}
	
	/**
	 * Set Category URIs
	 */
	public function set_category_uri() {
	    $this->load->library('goclixy/gclosure');
	    $gclosure = new Gclosure('categories', 'categories_hierarchy');
	    $data = array();
	    $categories = $this->db->get('categories')->result();
	    foreach($categories as $category) {
	        $category_tree = $gclosure->get_parent($category->id, NULL, 'DESC', true);
	        if(!empty($category_tree)) {
	            $slugs = array();
	            foreach($category_tree as $tree_category) {
	                $slugs[] = $tree_category->slug;
	            }
	            $data[] = array(
	                'id' => $category->id,
	                'slug_uris' => implode('/', $slugs)
	            );
	        }
	    }
	    if(!empty($data)) {
	        $this->db->update_batch('categories', $data, 'id');
	    }
	}
	
	/**
	 * Test Slug URIs
	 */
	public function slug_uris_test() {
	    $categories = $this->db->get('categories')->result();
	    $slug_uris = array();
	    foreach($categories as $category) {
	        $url = site_url('category/' . $category->slug_uris);
	        $slug_uris[] = anchor($url, $url, 'target="_blank"');
	    }
	    dump($slug_uris);
	}
}