<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Migration_Install_categories extends Migration
{
	/**
	 * The name of the database table
	 *
	 * @var String
	 */
	private $table_name = 'categories';

	/**
	 * The table's fields
	 *
	 * @var Array
	 */
	private $fields = array(
		'id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'auto_increment' => TRUE,
		),
		'parent_id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => FALSE,
		),
		'priority' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => FALSE,
			'default'=> 0,
		),
		'name' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => FALSE,
		),
		 'slug' => array(
			'type' => 'VARCHAR',
			'constraint' => 100,
			'null' => FALSE,
		),
		'icon' => array(
			'type' => 'VARCHAR',
			'constraint' => 30,
			'null' => TRUE,
		),
		'image' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE,
		),
		'description' => array(
			'type' => 'TEXT',
			'null' => TRUE,
		),
		'meta_title' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE,
		),
		'meta_keywords' => array(
			'type' => 'TEXT',
			'null' => TRUE,
		),
		'meta_description' => array(
			'type' => 'TEXT',
			'null' => TRUE,
		),
		'counts' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => FALSE,
			'default'=> 0,
		),
		'hits' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => FALSE,
			'default'=> 0,
		),
		'active' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default'=> 1,
		),
		'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'modified_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP'
	);
	
	/**
	 * categories_level table fields
	 *
	 * @var Array
	 */
	private $categories_level_fields = array(
		'id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'auto_increment' => TRUE,
		),
		'module_id' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
		),
		'row_id' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'null' => TRUE,
			'default'=> 0
		),
		'category_id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => TRUE,
			'default'=> 0
		),
	);
	
	/**
	 * Install this migration
	 *
	 * @return void
	 */
	public function up()
	{
		$this->dbforge->add_field($this->fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('parent_id', false);
		$this->dbforge->create_table($this->table_name);
		// CHANGE ENGINE FROM INNODB TO MYISAM
		$change_engine_sql = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " ENGINE = MyISAM";
		$this->db->query ($change_engine_sql);
		// ADD FULLTEXT INDEXT
		$fulltext_sql = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " ADD FULLTEXT INDEX `name` (`name`)";
		$this->db->query ($fulltext_sql);
		// SET SLUG FIELD COLLATION TO utf8_bin
		$collation_sql = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " CHANGE `slug` `slug` VARCHAR( 255 ) CHARACTER SET utf8 COLLATE utf8_bin NOT NULL";
		$this->db->query ($collation_sql);
		
		// categories_level TABLE FIELDS
		$this->dbforge->add_field($this->categories_level_fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->create_table('categories_level');
	}

	//--------------------------------------------------------------------

	/**
	 * Uninstall this migration
	 *
	 * @return void
	 */
	public function down()
	{
		$this->dbforge->drop_table($this->table_name);
		$this->dbforge->drop_table('categories_level');
	}

	//--------------------------------------------------------------------

}