<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
 || ##################################################################### ||
 || # GoClixy - Business Directory Script							   # ||
 || # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
 || # ----------------------------------------------------------------  # ||
 || # This file is owned and copyrighted by GoClixy.com. Your license   # ||
 || # confers  no  title  or ownership in the Script and is not a sale  # ||
 || # of any rights to the Software. This file may not be redistributed # ||
 || # in whole or significant part.									   # ||
 || # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
 || # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
 || ##################################################################### ||
 \*=======================================================================*/
class Categories_model extends BF_Model {
    
    protected $table_name	= "categories";
    protected $table_relation = "categories_relation";
    protected $key			= "id";
    protected $soft_deletes	= false;
    protected $date_format	= "datetime";
    
    protected $log_user 	= FALSE;
    
    protected $set_created	= TRUE;
    protected $set_modified = TRUE;
    protected $created_field = "created_on";
    protected $modified_field = "modified_on";
    
    /*
     Customize the operations of the model without recreating the insert, update,
     etc methods by adding the method names to act as callbacks here.
     */
    protected $before_insert 	= array();
    protected $after_insert 	= array();
    protected $before_update 	= array();
    protected $after_update 	= array();
    protected $before_find 		= array();
    protected $after_find 		= array();
    protected $before_delete 	= array();
    protected $after_delete 	= array();
    
    /*
     For performance reasons, you may require your model to NOT return the
     id of the last inserted row as it is a bit of a slow method. This is
     primarily helpful when running big loops over data.
     */
    protected $return_insert_id 	= TRUE;
    
    // The default type of element data is returned as.
    protected $return_type 			= "object";
    
    // Items that are always removed from data arrays prior to
    // any inserts or updates.
    protected $protected_attributes = array();
    
    /*
     You may need to move certain rules (like required) into the
     $insert_validation_rules array and out of the standard validation array.
     That way it is only required during inserts, not updates which may only
     be updating a portion of the data.
     */
    protected $validation_rules 		= array(
        array(
            "field"		=> "name",
            "label"		=> "lang:label_name",
            "rules"		=> "required|trim|max_length[100]|sanitize"
        ),
        array(
            "field"		=> "slug",
            "label"		=> "lang:label_slug",
            "rules"		=> "trim|max_length[100]|sanitize"
        ),
        array(
            "field"		=> "hits",
            "label"		=> "lang:label_hits",
            "rules"		=> "is_numeric|required|sanitize"
        ),
        array(
            "field"		=> "priority",
            "label"		=> "lang:label_priority",
            "rules"		=> "trim|is_numeric|required|sanitize"
        ),
        array(
            "field"		=> "description",
            "label"		=> "lang:label_description",
            "rules"		=> "trim"
        ),
        array(
            "field"		=> "icon",
            "label"		=> "lang:label_category_icon",
            "rules"		=> "trim"
        ),
        array(
            "field"		=> "meta_title",
            "label"		=> "lang:label_meta_title",
            "rules"		=> "max_length[100]|sanitize"
        ),
        array(
            "field"		=> "meta_keywords",
            "label"		=> "lang:label_meta_keywords",
            "rules"		=> "trim|xss_clean|tag_handling"
        ),
        array(
            "field"		=> "description",
            "label"		=> "lang:label_description",
            "rules"		=> "xss_clean"
        ),
        array(
            "field"		=> "modules[]",
            "label"		=> "lang:applicable_modules",
            "rules"		=> "required|is_array"
        ),
        array(
            "field"		=> "active",
            "label"		=> "lang:label_status",
            "rules"		=> "trim"
        )
    );
    protected $insert_validation_rules 	= array();
    protected $skip_validation 			= FALSE;
    protected $gc_categories_cache;
    
    //--------------------------------------------------------------------
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
        // Load Zend Caching
        $this->load->library('slug');
        $this->load->library('gzf_cache', array('lifetime' => 900));
        if(method_exists($this->gzf_cache, 'get_instance')) {
            $this->gc_categories_cache = $this->gzf_cache->get_instance();
        }
    }
    
    /**
     * Display all categories to the Admin - INDEX Method
     * @param number $count
     * @param number $parent_id
     * @param array $where
     * @param string $sort_by
     * @param number $offset
     * @param number $limit
     * @return number
     */
    public function admin_categories($count, $parent_id, $where, $sort_by, $offset = 0, $limit = 25) {
        if($count == 0) {
            $sql = "SELECT *";
        } else {
            $sql = "SELECT COUNT(C.id) AS TotalCount";
        }
        $sql .= " FROM " . $this->db->dbprefix . "categories C";
        if(!empty($where)) {
            $sql .= " WHERE $where";
        } else {
            $sql .= " WHERE parent_id = " . $parent_id;
        }
        
        if($count == 0) {
            $sql .= " ORDER BY " . $sort_by;
            $sql .= " LIMIT " . $offset . ", " . $limit;
            return $this->db->query ( $sql )->result();
        } else {
            $result = $this->db->query ( $sql )->row();
            return $result ? $result->TotalCount : 0;
        }
    }
    /**
     * Get categories without parent
     * @param string $id - A category cannot be a parent of itself
     * @return multitype:string NULL
     */
    public function get_no_parents($id = 0) {
        // Fetch categories without parents
        $this->db->select('id, name');
        $this->db->where('parent_id', 0);
        if($id != 0) {
            $this->db->where('id !=', $id);
        }
        $categories = $this->categories_model->order_by('name','asc')->find_all();
        
        // Return key => value pair array
        $array = array(
            0 => lang('label_no_parent')
        );
        if (!empty($categories)) {
            foreach ($categories as $category) {
                $array[$category->id] = $category->name;
            }
        }
        return $array;
    }
    
    /**
     * Save categories only when user clicks on save button
     * @param array $categories
     */
    public function save_order ($categories) {
        $this->load->library('goclixy/gclosure');
        $gclosure = new Gclosure($this->table_name, 'categories_hierarchy');
        // Convert data into JSON before sending it via POST
        // By default PHP settings (max_input_vars) allows only 1000 post variables
        // To avoid this issue, convert them to JSON and reconvert them to array in PHP
        $categories = json_decode($categories ?? '');
        if (count($categories)) {
            foreach ($categories as $order => $category) {
                if (isset($category->id) && $category->id != 0) {
                    $id = $category->id;
                    $parent_id = (int) $category->parent_id;
                    $data = array('parent_id' => $parent_id, 'priority' => $order);
                    $this->db->set($data)->where('id', $category->id)->update($this->table_name);
                    
                    // save hierarchy
                    $id = $category->id;
                    $record = $this->db->where('id', $id)->get($this->table_name)->row();
                    if($record) {
                        $parent_id = $record->parent_id;
                        $gclosure->delete($id, false); // do not delete from the categories table
                        // save hierarchy
                        $gclosure->add($id, ($parent_id == 0 ? $id : $parent_id));
                    }
                }
            }
        }
    }
    
    /**
     * To display all categories to the admin
     * @return multitype:multitype:unknown
     */
    public function get_nested() {
        $categories = $this->db->order_by('priority')->get('categories')->result_array();
        return $this->prepareList($categories);
    }
    
    /**
     * PREPARE LIST TO GET PARENTS AND ITS CHILDREN
     * @param array $items
     * @param number $pid
     * @return multitype:multitype:unknown
     */
    private function prepareList(array $items, $pid = 0) {
        $output = array();
        
        # loop through the items
        foreach ($items as $item) {
            $item['name'] = $item['name'] . ' - ' . $item['id'];
            # Whether the parent_id of the item matches the current $pid
            if ((int) $item['parent_id'] == $pid) {
                
                # Call the function recursively, use the item's id as the parent's id
                # The function returns the list of children or an empty array()
                if ($children = $this->prepareList($items, $item['id'])) {
                    
                    # Store all children of the current item
                    $item['children'] = $children;
                }
                
                # Fill the output
                $output[] = $item;
            }
        }
        
        return $output;
    }
    
    /**
     * Get parent categories using AJAX
     * @return multitype:string NULL
     */
    public function get_parent_categories(){
        $result = $this->categories_model->select('id, name')->where(array('parent_id' => 0, 'active' => 1))->find_all();
        
        $options = array(); //FILL OPTION ARRAY
        
        $options[''] = lang('label_select_parent');
        if($result) {
            // Format for passing into form_dropdown function
            foreach ($result as $option) {
                $options[$option->id] = $option->name;
            }
            return $options;
        } else {
            return false;
        }
    }
    
    /**
     * Get sub categories using AJAX
     * @param string $parent_id
     * @return void|multitype:NULL
     */
    public function get_sub_categories($parent_id = NULL){
        if(!is_null($parent_id)){
            $results = $this->categories_model->select('id, name')->where(array('parent_id' => $parent_id, 'active' => 1))->order_by('name', 'ASC')->find_all();
            $sub_categories = array();
            // if there are sub categories
            if($results){
                // Format for passing into jQuery loop
                foreach ($results as $result) {
                    // Ajax Success function do not preserve the Database sort by feature
                    // In order to display name in alphabetically order, below format is used
                    $sub_categories[$result->name] = $result->id;
                }
                return $sub_categories;
            }
        }
        return;
    }
    
    /**
     * To display breadcrumb on detail page
     * @param int $row_id
     * @param string $module
     */
    public function get_breadcrumb($row_id, $module = 'listings') {
        $module_id = gc_module_id($module);
        $result = $this->db->where(array('row_id' => $row_id, 'module_id' => $module_id))->get($this->table_relation)->row();
        if($result) {
            $result = $this->gc->closure(array('where' => $result->category_id, 'parent' => 1, 'self' => true));
        }
        return $result;
    }
    
    /**
     * To update number of hits of visited category
     * @param int $id
     */
    public function updateHits($id) {
        $this->db->where('id', $id);
        $this->db->set('hits', 'hits + 1', FALSE);
        $this->db->update($this->db->dbprefix . 'categories');
    }
    
    /**
     * To display categories on the detail page
     * @param string $module
     * @param int $row_id
     */
    public function get_categories($row_id, $module, $default_category) {
        $sql = "SELECT * FROM " . $this->db->dbprefix . $this->table_relation ." WHERE row_id = " . $row_id ." AND module_id = ". gc_module_id($module);
        if(!empty($default_category)) {
            $sql .= " AND category_id != $default_category";
        }
        $records = $this->db->query($sql)->result();
        $result = array();
        if($records) {
            foreach($records as $record) {
                $result[] = $this->gc->closure(array('where' => $record->category_id, 'parent' => 1, 'self' => true));
            }
        }
        return $result;
    }
    
    /**
     * For meta settings
     * @param int $row_id
     * @param string $module
     */
    public function meta_categories($row_id, $module = 'listings') {
        $sql = "SELECT C.name FROM (SELECT DISTINCT(category_id) AS ID FROM " . $this->db->dbprefix . $this->table_relation . " WHERE module_id = " . gc_module_id($module) . " AND row_id = $row_id) LC";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "categories C ON C.id = LC.id";
        $query = $this->db->query ( $sql );
        $meta_keywords_categories = array();
        foreach($query->result_array() as $meta_category) {
            $meta_keywords_categories[] = strtolower($meta_category['name']);
        }
        return $meta_keywords_categories;
    }
    
    /**
     * Display All Categories
     */
    public function categories($count = 0, $parent_id = 0, $module = 'listings', $where = null, $limit = 150, $offset = 0, $extra = array()) {
        if(settings_item('lst.hide_counts_zero_categories') == 1) {
            $extra['where'] = 'M.counts != 0';
        }
        if(settings_item('site.enable_cache') == 1) {
            $cache_array = array(
                'count' => $count,
                'parent_id' => $parent_id,
                'module' => $module,
                'where' => $where,
                'limit' => $limit,
                'offset' => $offset,
                'extra' => $extra
            );
            $cacheID = gc_cache_id($cache_array);
            $cacheID = 'categories_' . str_replace('-', '_', $this->slug->create_slug($cacheID));
            $cacheID = str_replace('-', '_', $this->slug->create_slug($cacheID));
            if(!$categories = $this->gc_categories_cache->load($cacheID)) {
                $categories = array('categories' => $this->_categories($count, $parent_id, $module, $where, $limit, $offset, $extra));
                $this->gc_categories_cache->save($categories, $cacheID, array('categories'));
            }
        } else {
            $categories = array('categories' => $this->_categories($count, $parent_id, $module, $where, $limit, $offset, $extra));
        }
        return $categories['categories'];
    }
    
    /**
     * Display All Categories
     * @param number $count
     * @param number $parent_id
     * @param string $module
     * @param array $where
     * @param number $limit
     * @param number $offset
     * @return object|number
     */
    private function _categories($count = 0, $parent_id = 0, $module = 'listings', $where = null, $limit = 150, $offset = 0, $extra = array()) {
        if($count == 0) {
            $table_fields = array('id', 'parent_id', 'name', 'description', 'icon', 'mobile_icon', 'slug', 'counts', 'image', 'module_counts');
            $query = $this->translations->build_query('categories', $table_fields);
            $sql = $query['select'] . ", SUBSTRING_INDEX(GROUP_CONCAT(S.name SEPARATOR ', '), ',', 3) AS description, COUNT(S.id) AS has_children";
            $sql .= ", SUBSTRING_INDEX(GROUP_CONCAT(CONCAT(S.id, '!~!', S.name, '!~!', S.slug, '!~!', S.counts, '!~!', S.active) SEPARATOR '###'), ',', 20) AS sub_categories";
        } else {
            $sql = "SELECT COUNT(*) AS total_count";
        }
        $sql .= " FROM " . $this->db->dbprefix . "categories M";
        //$sql .= " LEFT JOIN " . $this->db->dbprefix . "categories S ON S.parent_id = M.id AND S.active = 1";
        if($count == 0) {
            $sql .= $this->translations->common_joins('categories') . " S ON S.parent_id = M.id AND S.active = 1";
        }
        
        if($count == 0 && !empty($query['join'])) {
            $sql .= $query['join'];
        }
        $sql .= " WHERE M.parent_id = " . (empty($parent_id) ? 0 : (int) $parent_id) . " AND M.active = 1";
        if(!empty($module)) {
            $sql .= " AND FIND_IN_SET(" . gc_module_id($module) . ", M.module_relations)";
        }
        if(!empty($where)) {
            $sql .= " AND M.name LIKE '%" . $where . "%'";
        }
        if(!empty($extra['where'])) {
            $sql .= " AND " . $extra['where'];
        }
        if($count == 0) {
            $sql .= " GROUP BY M.id ORDER BY " . (!empty($extra['sort_by']) ? $extra['sort_by'] : 'M.priority, name');
            if ($limit != 0) {
                $sql .= " LIMIT " . $offset . ", " . $limit;
            } else {
                $sql .= " LIMIT $limit";
            }
            return $this->db->query ( $sql )->result();
        } else {
            $result = $this->db->query ( $sql )->row();
            return $result ? $result->total_count : 0;
        }
    }
    
    /**
     * Save categories relation from the Member and Admin Area
     * @param string $module
     * @param int $row_id
     */
    public function save_categories_relation($row_id, $module) {
        $module_id = gc_module_id($module);
        $categories = $this->input->post ('categories');
        $default_category = $this->input->post('default_category');
        if(!empty($categories)) {
            /**
             * Get first selected category and convert it to an array
             * Get all selected categories
             * Merge default category with all categories to put default category on top
             */
            
            // insert default category id in the listings table
            // do this only for listings table
            if($module == 'listings' && !empty($default_category)) {
                // update default category in the listings table
                if (strpos($default_category,'/') !== false) {
                    $default_category_data['category_id'] = substr( $default_category, strrpos( $default_category, '/' ) + 1);
                } else {
                    $default_category_data['category_id'] = $default_category;
                }
                $this->db->where('id', $row_id)->update('listings', $default_category_data);
            }
            $selected_categories = explode(',',$categories ?? '');
            /**
             * If user unselected the default category, then default must be replaced with new one.
             * set the first selected_category as the default
             */
            $default_category_array = array();
            if(!in_array($default_category, $selected_categories)) {
                $default_category_array[] = $selected_categories[0];
            } else {
                $default_category_array = explode(',', $default_category ?? '');
            }
            $selected_array = array_unique(array_merge($default_category_array, $selected_categories), SORT_REGULAR);
            $all_categories = array();
            // REMOVE / FROM EACH STRING AND STORE VALUE IN AN ARRAY
            foreach($selected_array as $selected_value) {
                $all_categories[] = explode('/',substr($selected_value, 1));
            }
            
            // CONVERT ARRAY INTO INSERT BATCH DATA
            $categories_data = array();
            foreach($all_categories as $category) {
                // $category_id = end($category);
                foreach($category as $category_id) {
                    $categories_data[] = $category_id;
                }
            }
            $categories_data = array_unique(array_filter($categories_data));
            foreach($categories_data as $category_id) {
                $data[] = array (
                    'row_id' => $row_id,
                    'module_id' => $module_id,
                    'category_id' => $category_id
                );
            }
            
            if(!empty($data)) {
                // delete existing categories in case user removed one
                $this->db->where(array('row_id' => $row_id, 'module_id' => $module_id))->delete($this->table_relation);
                // save categories
                $this->db->insert_batch($this->table_relation, $data);
            }
        } else {
            // delete existing categories in case user removed all
            $this->db->where(array('row_id' => $row_id, 'module_id' => $module_id))->delete($this->table_relation);
        }
        // Save categories in comma separated format in the categories column for search queries
        $this->_tweak_categories($module_id, $row_id);
    }
    
    /**
     * Update rows count
     * @param string $module - Name of the module count to be updated
     */
    public function counts($module = 'listings') {
        // reset modules counts to null
        $this->db->update($this->table_name, array('module_counts' => null, 'counts' => 0));
        $module_counts = array();
        $all_counts = array();
        foreach(gc_modules() as $module) {
            $module_id = gc_module_id($module);
            $sql = "SELECT CR.category_id AS id, COUNT(*) AS TotalCounts FROM " . $this->db->dbprefix . "categories_relation CR";
            $sql .= " INNER JOIN " . $this->db->dbprefix . $module . " M ON M.id = CR.row_id AND M.status = 1 AND M.deleted = 0";
            $sql .= " WHERE CR.module_id = " . $module_id . " GROUP BY CR.category_id";
            $records = $this->db->query($sql)->result();
            if($records) {
                foreach($records as $record) {
                    /*$parents = $this->gc->closure(array('where' => $record->id, 'parent' => 1, 'self' => true, 'sort_by' => 'ASC'));
                     if($parents) {
                     foreach($parents as $parent) {
                     $parent_module_counts = json_decode($parent->module_counts, true);
                     $module_counts[$parent->id][$module] = (!empty($module_counts[$parent->id][$module]) ? $module_counts[$parent->id][$module] : (!empty($parent_module_counts[$module]) ? $parent_module_counts[$module] : 0)) + $record->TotalCounts;
                     $all_counts[$parent->id] = (!empty($all_counts[$parent->id]) ? $all_counts[$parent->id] : (!empty($parent->counts) ? $parent->counts : 0)) + $record->TotalCounts;
                     }
                     }
                     */
                    $module_counts[$record->id][$module] = (!empty($module_counts[$record->id][$module]) ? $module_counts[$record->id][$module] : 0) + $record->TotalCounts;
                    $all_counts[$record->id] = (!empty($all_counts[$record->id]) ? $all_counts[$record->id] : 0) + $record->TotalCounts;
                }
            }
        }
        $data = array();
        foreach($module_counts as $category_id => $module_count) {
            $data[] = array(
                'id' => $category_id,
                'module_counts' => json_encode($module_count),
                'counts' => !empty($all_counts[$category_id]) ? (int) $all_counts[$category_id] : 0
            );
        }
        if(!empty($data)) {
            $this->db->update_batch('categories', $data, 'id');
        }
    }
    
    /**
     * Get individual module counts
     * Method is also called in the Listings_model.php file.
     * @param number $module_id
     */
    public function get_counts($module_id, $return_query = false) {
        $sql = " SELECT id, sum( TotalCounts ) as TotalCounts FROM (";
        $sql .= " SELECT C.id, COUNT( DISTINCT LC.row_id ) AS TotalCounts FROM " . $this->db->dbprefix . "categories C LEFT JOIN " . $this->db->dbprefix . "categories_relation LC ON LC.category_id = C.id AND LC.module_id = " . $module_id . " GROUP BY C.id";
        $sql .= ") LSS GROUP BY id";
        if($return_query) {
            return $sql;
        } else {
            return $this->db->query($sql)->result();
        }
    }
    
    /**
     * Return categories in the fancytree format
     * @param string $module
     * @param integer $row_id
     * @return string
     */
    public function fancytree_format($module, $row_id) {
        $categories = $this->db->select('category_id')->get_where('categories_relation', array('module_id' => gc_module_id($module), 'row_id' => $row_id))->result();
        $result = array();
        $return = '';
        if($categories) {
            foreach($categories as $category) {
                $result[] = $this->format_category($category->category_id);
            }
        }
        if(!empty($result)) {
            $return = implode(',', $result);
        }
        return $return;
    }
    
    /**
     * Format Category in the Fancytree format
     * @param number $category_id
     * @return string
     */
    public function format_category($category_id) {
        $parents = $this->gc->closure(array('where' => $category_id, 'parent' => 1));
        if(empty($parents)) {
            $result = $category_id;
        } else {
            $relations = array();
            foreach($parents as $parent) {
                $relations[] = $parent->id;
            }
            $relations[] = $category_id;
            $result = implode('/', $relations);
        }
        return $result;
    }
    
    /**
     * Query to Get Categories Tree
     * @param integer $parent_id
     * @param string $module
     * @param integer $package_id
     * @return array
     */
    public function get_categories_for_tree($parent_id, $module, $package_id) {
        $categories_module_id = gc_module_id('categories');
        $toggle_module_id = gc_module_id($module);
        $table_fields = array('id', 'parent_id', 'name', 'active');
        $translated_fields = array('name');
        $query = $this->translations->build_query('categories', $table_fields, $translated_fields);
        $sql = $query['select'] . ", CC.Count as count";
        $sql .= " FROM ". $this->db->dbprefix . "categories M" . PHP_EOL;
        if(!empty($query['join'])) {
            $sql .= $query['join'];
        }
        // The Site Owner may enable specific categories for a specific module
        // In this case, the count (hasChildren) field should be blank,,
        // so also INNER JOIN the module_relations table to avoid any ambiguity
        // E.g. Enable the parent category for listings and jobs, but add the child categories only in the jobs module.
        $sql .= " LEFT OUTER JOIN (";
        $sql .= " SELECT CS.parent_id, COUNT(*) AS Count FROM ". $this->db->dbprefix . "categories CS" . PHP_EOL;
        $sql .= " WHERE CS.active = 1 AND FIND_IN_SET(" . $toggle_module_id . ", CS.module_relations) GROUP BY CS.parent_id) CC ON M.id = CC.parent_id" . PHP_EOL;
        $sql .= " WHERE M.parent_id = $parent_id AND M.active = 1 AND FIND_IN_SET(" . $toggle_module_id . ", M.module_relations)";
        if($package_id != 0) { // modify this logic to restrict a user to specific categories
            $package = $this->db->select('applicable_categories')->get_where('packages', array('id' => $package_id))->row();
            if($package && !empty($package->applicable_categories)) {
                $applicable_categories = trim($package->applicable_categories, '"');
            }
            if(!empty($applicable_categories)) {
                $sql .= " AND M.id IN (" . $applicable_categories . ")";
            }
        }
        $sql .= " ORDER BY M.name";
        $result = $this->db->query($sql)->result();
        return $result;
    }
    
    /**
     * FANCY TREE - Display Categories Tree to the User
     * @param number $parent_id
     * @param string $module
     * @param array $post_categories
     * @return multitype:multitype:boolean NULL
     */
    public function get_categories_ajax($parent_id, $module, $post_categories, $package_id = 0) {
        $result = $this->get_categories_for_tree($parent_id, $module, $package_id);
        $data = array();
        if($result) {
            foreach($result as $category) {
                if(!empty($post_categories) && in_array($category->id, $post_categories)) {
                    // Category has no children, this is the root category
                    if($category->count == 0) {
                        $data[] = array(
                            'title' => $category->name,
                            'folder' => TRUE,
                            'key' => $category->id,
                            'selected' => TRUE
                        );
                    } else { // Category has children, the child category will be selected
                        $data[] = array(
                            'title' => $category->name,
                            'folder' => TRUE,
                            'key' => $category->id,
                            'children' => $this->get_categories_ajax($category->id, $module, $post_categories, $package_id)
                        );
                    }
                } else {
                    if($category->count == 0) {
                        $data[] = array(
                            'title' => $category->name,
                            'folder' => TRUE,
                            'key' => $category->id
                        );
                    } else {
                        $data[] = array(
                            'title' => $category->name,
                            'folder' => TRUE,
                            'key' => $category->id,
                            'lazy' => TRUE
                        );
                    }
                }
            }
        }
        return $data;
    }
    
    /**
     * Purge a category with all its sub and subsub categories
     * Also purge the information from the module_relations and ads_keywords table
     * @param int $id
     */
    public function purge_category($id) {
        // Cascadding Delete Sub and Subsub categories if exists
        $sub_categories = $this->categories_model->select('id')->find_all_by(array('parent_id' => $id));
        if($sub_categories) {
            foreach($sub_categories as $sub_item) {
                $where = "parent_id = " . $sub_item->id;
                $this->categories_model->delete_where($where);
                $this->categories_model->delete($sub_item->id);
            }
        }
        
        // set auto increment value to the last id
        $alter_autoinc = "ALTER TABLE ". $this->db->dbprefix . "categories AUTO_INCREMENT = 1";
        $this->db->simple_query($alter_autoinc);
        
        // Purge all relations of this category from the module_relations table
        $this->gc->purge_relation($id, 'categories');
        
        // Purge information from the Ads module
        $this->ads_model->purge_from_ad($id, 'categories');
        
        // Purge category image
        $this->load->model('images/images_model');
        $this->images_model->purge_module_image('categories', $id);
        
        // Purge custom fields information
        $this->db->where(array('module_id' => gc_module_id('categories'), 'row_id' => $id))->delete('field_modules');
        
        // Purge Translations
        $this->translations->purge_tanslations('categories', $id);
        
        // Purge category
        $result = $this->categories_model->delete($id);
        return $result;
    }
    
    /**
     * Delete existing categories if the user saved the module without selecting any category
     * @param int $id
     * @param string $module
     */
    public function purge_categories_relation($module, $row_id) {
        return $this->db->where(array('module_id' => gc_module_id($module), 'row_id' => $row_id))->delete($this->table_relation);
    }
    
    
    /**
     * Import and return item ids
     * @param string $item = array('Arts Entertainment', 'Bars');
     * @return array
     */
    public function import_categories($item = array('Arts Entertainment', 'Bars'), $module = 'listings') {
        $this->load->library('goclixy/gclosure');
        $gclosure = new Gclosure('categories', 'categories_hierarchy');
        $module_id = gc_module_id($module);
        $output = array();
        $item_count = count($item);
        if($item_count == 1) {
            // only one item is entered and it is the parent
            $output[] = $this->verify_existence($item[0], 0, $module_id, $gclosure);
        } else {
            $parent_ids = array();
            foreach($item as $key => $value) {
                $parent_id = 0;
                if($key > 0) {
                    // get parent_id
                    // $category = $this->gc->get('categories', array('where' => array("name ='" . $item[$key-1] . "'"), 'order_by' => 'id DESC'));
                    $category = $this->db->where("name ='" . $item[$key-1] . "' AND parent_id = " . (!empty($parent_ids[$key-2]) ? $parent_ids[$key-2] : 0))->order_by('id', 'DESC')->get('categories')->row();
                    if($category) {
                        $parent_id = $category->id;
                    }
                }
                $category_id = $this->verify_existence($value, $parent_id, $module_id, $gclosure);
                $parent_ids[] = $category_id;
                $output[] = $category_id;
            }
        }
        $result = array('tree' => implode('/', $output));
        return $result;
    }
    
    /**
     * Bulk Import Categories
     * @param array $item
     * @param number $module_id
     * @return array[]
     */
    public function bulk_import($item, $module_id, $gclosure) {
        $cacheID = 'categories_' . gc_cache_id($item) . $module_id;
        if(!$result = $this->gc_categories_cache->load($cacheID)) {
            $result = array();
            $item_count = count($item);
            if($item_count == 1) {
                // only one item is entered and it is the parent
                $result[] = $this->verify_existence($item[0], 0, $module_id, $gclosure);
            } else {
                $parent_ids = array();
                foreach($item as $key => $value) {
                    $parent_id = 0;
                    if($key > 0) {
                        // get parent_id
                        // $category = $this->gc->get('categories', array('where' => array("name ='" . $item[$key-1] . "'"), 'order_by' => 'id DESC'));
                        $category = $this->db->where("name ='" . $item[$key-1] . "' AND parent_id = " . (!empty($parent_ids[$key-2]) ? $parent_ids[$key-2] : 0))->order_by('id', 'DESC')->get('categories')->row();
                        if($category) {
                            $parent_id = $category->id;
                        }
                    }
                    $category_id = $this->verify_existence($value, $parent_id, $module_id, $gclosure);
                    $parent_ids[] = $category_id;
                    $result[] = $category_id;
                }
            }
            $this->gc_categories_cache->save($result, $cacheID, array('categories'));
        }
        return $result;
    }
    
    /**
     * Verify Category Existence
     * @param string $name
     * @param number $parent_id
     */
    public function verify_existence($name, $parent_id, $module_id, $gclosure) {
        $record = $this->db->where(array('name' => $name, 'parent_id' => $parent_id))->get($this->table_name)->row();
        if($record) {
            // check for the specific module
            $module_relations = explode(',', $record->module_relations ?? '');
            if(!in_array($module_id, $module_relations)) {
                $module_relations[] = $module_id;
                $this->db->where('id', $record->id)->update($this->table_name, array('module_relations' => implode(',', $module_relations)));
            }
            return $record->id;
        } else {
            // add a new item
            $data = array(
                'parent_id' => $parent_id,
                'name' => $name,
                'slug' => $this->slug->create_slug($name),
                'icon' => $this->random_icon(),
                'module_relations' => $module_id,
                'created_on' => date('Y-m-d H:i:s')
            );
            $this->db->insert($this->table_name, $data);
            $id = (int) $this->db->insert_id();
            if($id > 0) {
                $gclosure->delete($id, false); // do not delete from the categories table
                // save hierarchy
                $gclosure->add($id, ($parent_id == 0 ? $id : $parent_id));
            }
            return $id;
        }
    }
    
    /**
     * Browse By Categories
     * @param string $module
     * @param number $parent_id
     * @param array $extra - Extra parameters from location to category redirect
     */
    public function load_categories($module_id, $parent_id = 0, $extra = array()) {
        $select = "C.id, C.name, C.slug, C.slug_uris, C.parent_id, C.icon, C.counts, C.module_counts";
        $join = '';
        $where = array();
        $where[] = "C.parent_id = " . $parent_id . " AND C.active = 1 AND FIND_IN_SET(" . $module_id .", C.module_relations)";
        if(!empty($extra['column_name']) && !empty($extra['location_id'])) {
            $module = gc_module_name($module_id);
            $select .= ", CONCAT('{\"" . $module . "\":\"', COUNT( DISTINCT(MD.id)), '\"}') AS module_counts";
            $join = " LEFT JOIN " . $this->db->dbprefix . $module . " MD ON FIND_IN_SET(C.id, MD.categories_set)";
            $where[] = "MD.deleted = 0 AND MD.status = 1 AND MD." . $extra['column_name'] . " = " . $extra['location_id'];
        }
        $sql = "SELECT " . $select;
        $sql .= " FROM " . $this->db->dbprefix . $this->table_name . " C";
        $sql .= $join;
        $sql .= " WHERE " . implode(' AND ', $where);
        $sql .= " GROUP BY C.id ORDER BY C.name ASC";
        $sql .= " LIMIT 2000";
        return $this->db->query($sql)->result();
    }
    
    /**
     * Mobile App - Display Module Categories - Module Edit Page
     * @param $module - Name of the module
     * @param $id - ID of the Row
     */
    public function get_module_categories($row_id, $module) {
        $categories = array();
        $this->load->model('categories/categories_model');
        $get_categories = $this->categories_model->get_categories($row_id, $module, NULL);
        if($get_categories) {
            foreach($get_categories as $category) {
                $parents = $this->gc->closure(array('where' => $category->category_id, 'parent' => 1));
                if(empty($parents)) {
                    $categories[] = array(
                        'parent' => '',
                        'parent_id' => 0,
                        'id' => $category['parent_id'],
                        'name' => $category['parent_category']
                    );
                } else {
                    $last_item = end($parents);
                    $item_names = array();
                    foreach($parents as $parent) {
                        $item_names[] = $parent->name;
                    }
                    $categories[] = array(
                        'parent' => implode(' -> ', $item_names),
                        'parent_id' => $category['parent_id'],
                        'id' => $last_item['id'],
                        'name' => $last_item['name']
                    );
                }
            }
        }
        return $categories;
    }
    
    /**
     * Update Tweak Field name Categories in the Module Table to fasten the Search Query
     * @param integer $module_id
     * @param integer $row_id
     * @param boolean $is_update - TRUE - Update instantly, FALSE - Return value for Bulk Tweaking of Multiple Rows
     */
    public function _tweak_categories($module_id, $row_id, $is_update = true) {
        $module = gc_module_name($module_id);
        
        $unique_ids = $this->get_category_ids($module, $row_id);
        // Get name of each category and use implode function to create a string
        if(!empty($unique_ids)) {
            // Note: the string length must be greater than 4 for FULL TEXT search
            $row = $this->db->where('id IN (' . implode(',', $unique_ids) . ') AND active = 1')->select("GROUP_CONCAT(name SEPARATOR ', ') AS categories, GROUP_CONCAT(id) AS categories_set, GROUP_CONCAT(CONCAT('id', id, 'c') SEPARATOR ',') AS category_ids, GROUP_CONCAT(CONCAT(id, '!~!', name, '!~!', COALESCE(slug,''), '!~!', COALESCE(icon,''), '!~!'" . (!empty(settings_item('site.category_slug_uris')) ? ", COALESCE(slug_uris,'')" : '') . ") SEPARATOR '~~~') AS saved_categories")->get('categories')->row();
            if(!empty($row->categories)) {
                $data = array('categories' => $row->categories, 'categories_set' => $row->categories_set, 'category_ids' => $row->category_ids, 'saved_categories' => $row->saved_categories);
                if($is_update) {
                    // Update categories information in the Module Categories column
                    $this->db->where(array('id' => $row_id))->update($module, $data);
                } else {
                    return $data;
                }
            }
        } else {
            $data = array('categories' => null, 'categories_set' => null, 'category_ids' => null, 'saved_categories' => null);
            if($module == 'listings') {
                $data['category_id'] = 0;
            }
            $this->db->where(array('id' => $row_id))->update($module, $data);
        }
    }
    
    /**
     * Return module categories in a Comma separated format for Email and PDF Flyer
     * @param string $module
     * @param integer $row_id
     * @return string
     */
    public function comma_separated($module, $row_id) {
        $module_id = gc_module_id($module);
        $categories = '';
        $ids = $this->get_category_ids($module, $row_id);
        if(!empty($ids)) {
            $row = $this->db->select('GROUP_CONCAT(name SEPARATOR ", ") AS categories')->where('id IN (' . implode(',',$ids) . ')')->get('categories')->row();
            if($row) {
                $categories = $row->categories;
            }
        }
        return $categories;
    }
    
    /**
     * Get Module Categories IDs
     * @param string $module - Name of the module
     * @param integer $row_id
     * @return <multitype:, multitype:NULL >
     */
    public function get_category_ids($module, $row_id) {
        $category_ids = array();
        $module_id = gc_module_id($module);
        // Get all the selected categories ID first
        $categories = $this->db->select('category_id')->get_where($this->table_relation, array('row_id' => $row_id, 'module_id' => $module_id))->result();
        if($categories) {
            foreach($categories as $category) {
                /*$parents = $this->gc->closure(array('where' => $category->category_id, 'parent' => 1));
                 if(!empty($parents)) {
                 foreach($parents as $parent) {
                 $category_ids[] = $parent->id;
                 }
                 }*/
                $category_ids[] = $category->category_id;
            }
            // Remove 0 from the ids list using array_filter
            $category_ids = array_unique(array_filter($category_ids));
        }
        return $category_ids;
    }
    
    /**
     * Categories FIND BY and FIND ALL - Translated Data
     * @param string $where
     * @param boolean $all_records - True - Multiple Records, False - Single Record
     */
    public function get_translated($where = null, $all_records = false) {
        $table_fields = array('parent_id', 'id', 'name');
        $translated_fields = array('name');
        $query = $this->translations->build_query('categories', $table_fields, $translated_fields);
        $sql = $query['select'];
        $sql .= " FROM " . $this->db->dbprefix . "categories M";
        if(!empty($query['join'])) {
            $sql .= $query['join'];
        }
        if(!empty($where)) {
            $sql .= " WHERE " . $where;
        }
        if($all_records) {
            $sql .= " ORDER BY M.name";
            return $this->db->query($sql)->result();
        } else {
            return $this->db->query($sql)->row();
        }
    }
    
    /**
     * Module Category Filteration
     * @param number $module_id
     * @return array
     */
    public function filter($module_id) {
        // Category Filteration
        if(isset($_POST['category_filter'])) {
            $category_id = (int) $_POST['category_id'];
            if(empty($category_id)) {
                $this->session->unset_userdata('category_filter');
            } else {
                $category_level = $this->gc->category_level_search($category_id);
                if(!empty($category_level['result'])) {
                    $level = key($category_level['result']);
                    $category_id = $category_level['result'][$level];
                    $this->session->set_userdata('category_filter',
                        array('module_id' => $module_id, 'level' => $level, 'category_id' => $category_id)
                        );
                }
            }
        }
        
        // Clear Category Filter
        if(isset($_POST['clear_category_filter'])) {
            $this->session->unset_userdata('category_filter');
            Template::redirect(current_url());
        }
        
        // POST Handling
        $category_filter = array();
        if($this->session->userdata('category_filter')) {
            $category_filter = $this->session->userdata('category_filter');
            if(!empty($category_filter['category_id'])) {
                Template::set('category_filter', $category_filter['category_id']);
            }
        }
        return $category_filter;
    }
    
    /**
     * Leads Form - Categories Dropdown
     * @param number $parent_id
     */
    public function category_dropdown($parent_id = 0, $placeholder = null) {
        $options = array();
        if(!empty($placeholder)) {
            $options = array('' => lang('placeholder_category'));
        }
        $records = $this->db->where(array('parent_id' => $parent_id, 'active' => 1))->get($this->table_name)->result();
        if($records) {
            foreach($records as $record) {
                $options[$record->id] = $record->name;
            }
        }
        return $options;
    }
    
    /**
     * Get random icon
     * @return mixed
     */
    public function random_icon() {
        $str = '
{
"name":"Font Awesome v5.0.1",
"icons":[
"fab fa-accessible-icon",
"fab fa-accusoft",
"fas fa-address-book", "far fa-address-book",
"fas fa-address-card", "far fa-address-card",
"fas fa-adjust",
"fab fa-adn",
"fab fa-adversal",
"fab fa-affiliatetheme",
"fab fa-algolia",
"fas fa-align-center",
"fas fa-align-justify",
"fas fa-align-left",
"fas fa-align-right",
"fab fa-amazon",
"fas fa-ambulance",
"fas fa-american-sign-language-interpreting",
"fab fa-amilia",
"fas fa-anchor",
"fab fa-android",
"fab fa-angellist",
"fas fa-angle-double-down",
"fas fa-angle-double-left",
"fas fa-angle-double-right",
"fas fa-angle-double-up",
"fas fa-angle-down",
"fas fa-angle-left",
"fas fa-angle-right",
"fas fa-angle-up",
"fab fa-angrycreative",
"fab fa-angular",
"fab fa-app-store",
"fab fa-app-store-ios",
"fab fa-apper",
"fab fa-apple",
"fab fa-apple-pay",
"fas fa-archive",
"fas fa-arrow-alt-circle-down", "far fa-arrow-alt-circle-down",
"fas fa-arrow-alt-circle-left", "far fa-arrow-alt-circle-left",
"fas fa-arrow-alt-circle-right", "far fa-arrow-alt-circle-right",
"fas fa-arrow-alt-circle-up", "far fa-arrow-alt-circle-up",
"fas fa-arrow-circle-down",
"fas fa-arrow-circle-left",
"fas fa-arrow-circle-right",
"fas fa-arrow-circle-up",
"fas fa-arrow-down",
"fas fa-arrow-left",
"fas fa-arrow-right",
"fas fa-arrow-up",
"fas fa-arrows-alt",
"fas fa-arrows-alt-h",
"fas fa-arrows-alt-v",
"fas fa-assistive-listening-systems",
"fas fa-asterisk",
"fab fa-asymmetrik",
"fas fa-at",
"fab fa-audible",
"fas fa-audio-description",
"fab fa-autoprefixer",
"fab fa-avianex",
"fab fa-aviato",
"fab fa-aws",
"fas fa-backward",
"fas fa-balance-scale",
"fas fa-ban",
"fab fa-bandcamp",
"fas fa-barcode",
"fas fa-bars",
"fas fa-bath",
"fas fa-battery-empty",
"fas fa-battery-full",
"fas fa-battery-half",
"fas fa-battery-quarter",
"fas fa-battery-three-quarters",
"fas fa-bed",
"fas fa-beer",
"fab fa-behance",
"fab fa-behance-square",
"fas fa-bell", "far fa-bell",
"fas fa-bell-slash", "far fa-bell-slash",
"fas fa-bicycle",
"fab fa-bimobject",
"fas fa-binoculars",
"fas fa-birthday-cake",
"fab fa-bitbucket",
"fab fa-bitcoin",
"fab fa-bity",
"fab fa-black-tie",
"fab fa-blackberry",
"fas fa-blind",
"fab fa-blogger",
"fab fa-blogger-b",
"fab fa-bluetooth",
"fab fa-bluetooth-b",
"fas fa-bold",
"fas fa-bolt",
"fas fa-bomb",
"fas fa-book",
"fas fa-bookmark", "far fa-bookmark",
"fas fa-braille",
"fas fa-briefcase",
"fab fa-btc",
"fas fa-bug",
"fas fa-building", "far fa-building",
"fas fa-bullhorn",
"fas fa-bullseye",
"fab fa-buromobelexperte",
"fas fa-bus",
"fab fa-buysellads",
"fas fa-calculator",
"fas fa-calendar", "far fa-calendar",
"fas fa-calendar-alt", "far fa-calendar-alt",
"fas fa-calendar-check", "far fa-calendar-check",
"fas fa-calendar-minus", "far fa-calendar-minus",
"fas fa-calendar-plus", "far fa-calendar-plus",
"fas fa-calendar-times", "far fa-calendar-times",
"fas fa-camera",
"fas fa-camera-retro",
"fas fa-car",
"fas fa-caret-down",
"fas fa-caret-left",
"fas fa-caret-right",
"fas fa-caret-square-down", "far fa-caret-square-down",
"fas fa-caret-square-left", "far fa-caret-square-left",
"fas fa-caret-square-right", "far fa-caret-square-right",
"fas fa-caret-square-up", "far fa-caret-square-up",
"fas fa-caret-up",
"fas fa-cart-arrow-down",
"fas fa-cart-plus",
"fab fa-cc-amex",
"fab fa-cc-apple-pay",
"fab fa-cc-diners-club",
"fab fa-cc-discover",
"fab fa-cc-jcb",
"fab fa-cc-mastercard",
"fab fa-cc-paypal",
"fab fa-cc-stripe",
"fab fa-cc-visa",
"fab fa-centercode",
"fas fa-certificate",
"fas fa-chart-area",
"fas fa-chart-bar", "far fa-chart-bar",
"fas fa-chart-line",
"fas fa-chart-pie",
"fas fa-check",
"fas fa-check-circle", "far fa-check-circle",
"fas fa-check-square", "far fa-check-square",
"fas fa-chevron-circle-down",
"fas fa-chevron-circle-left",
"fas fa-chevron-circle-right",
"fas fa-chevron-circle-up",
"fas fa-chevron-down",
"fas fa-chevron-left",
"fas fa-chevron-right",
"fas fa-chevron-up",
"fas fa-child",
"fab fa-chrome",
"fas fa-circle", "far fa-circle",
"fas fa-circle-notch",
"fas fa-clipboard", "far fa-clipboard",
"fas fa-clock", "far fa-clock",
"fas fa-clone", "far fa-clone",
"fas fa-closed-captioning", "far fa-closed-captioning",
"fas fa-cloud",
"fas fa-cloud-download-alt",
"fas fa-cloud-upload-alt",
"fab fa-cloudscale",
"fab fa-cloudsmith",
"fab fa-cloudversify",
"fas fa-code",
"fas fa-code-branch",
"fab fa-codepen",
"fab fa-codiepie",
"fas fa-coffee",
"fas fa-cog",
"fas fa-cogs",
"fas fa-columns",
"fas fa-comment", "far fa-comment",
"fas fa-comment-alt", "far fa-comment-alt",
"fas fa-comments", "far fa-comments",
"fas fa-compass", "far fa-compass",
"fas fa-compress",
"fab fa-connectdevelop",
"fab fa-contao",
"fas fa-copy", "far fa-copy",
"fas fa-copyright", "far fa-copyright",
"fab fa-cpanel",
"fab fa-creative-commons",
"fas fa-credit-card", "far fa-credit-card",
"fas fa-crop",
"fas fa-crosshairs",
"fab fa-css3",
"fab fa-css3-alt",
"fas fa-cube",
"fas fa-cubes",
"fas fa-cut",
"fab fa-cuttlefish",
"fab fa-d-and-d",
"fab fa-dashcube",
"fas fa-database",
"fas fa-deaf",
"fab fa-delicious",
"fab fa-deploydog",
"fab fa-deskpro",
"fas fa-desktop",
"fab fa-deviantart",
"fab fa-digg",
"fab fa-digital-ocean",
"fab fa-discord",
"fab fa-discourse",
"fab fa-dochub",
"fab fa-docker",
"fas fa-dollar-sign",
"fas fa-dot-circle", "far fa-dot-circle",
"fas fa-download",
"fab fa-draft2digital",
"fab fa-dribbble",
"fab fa-dribbble-square",
"fab fa-dropbox",
"fab fa-drupal",
"fab fa-dyalog",
"fab fa-earlybirds",
"fab fa-edge",
"fas fa-edit", "far fa-edit",
"fas fa-eject",
"fas fa-ellipsis-h",
"fas fa-ellipsis-v",
"fab fa-ember",
"fab fa-empire",
"fas fa-envelope", "far fa-envelope",
"fas fa-envelope-open", "far fa-envelope-open",
"fas fa-envelope-square",
"fab fa-envira",
"fas fa-eraser",
"fab fa-erlang",
"fab fa-etsy",
"fas fa-euro-sign",
"fas fa-exchange-alt",
"fas fa-exclamation",
"fas fa-exclamation-circle",
"fas fa-exclamation-triangle",
"fas fa-expand",
"fas fa-expand-arrows-alt",
"fab fa-expeditedssl",
"fas fa-external-link-alt",
"fas fa-external-link-square-alt",
"fas fa-eye",
"fas fa-eye-dropper",
"fas fa-eye-slash", "far fa-eye-slash",
"fab fa-facebook",
"fab fa-facebook-f",
"fab fa-facebook-messenger",
"fab fa-facebook-square",
"fas fa-fast-backward",
"fas fa-fast-forward",
"fas fa-fax",
"fas fa-female",
"fas fa-fighter-jet",
"fas fa-file", "far fa-file",
"fas fa-file-alt", "far fa-file-alt",
"fas fa-file-archive", "far fa-file-archive",
"fas fa-file-audio", "far fa-file-audio",
"fas fa-file-code", "far fa-file-code",
"fas fa-file-excel", "far fa-file-excel",
"fas fa-file-image", "far fa-file-image",
"fas fa-file-pdf", "far fa-file-pdf",
"fas fa-file-powerpoint", "far fa-file-powerpoint",
"fas fa-file-video", "far fa-file-video",
"fas fa-file-word", "far fa-file-word",
"fas fa-film",
"fas fa-filter",
"fas fa-fire",
"fas fa-fire-extinguisher",
"fab fa-firefox",
"fab fa-first-order",
"fab fa-firstdraft",
"fas fa-flag", "far fa-flag",
"fas fa-flag-checkered",
"fas fa-flask",
"fab fa-flickr",
"fab fa-fly",
"fas fa-folder", "far fa-folder",
"fas fa-folder-open", "far fa-folder-open",
"fas fa-font",
"fab fa-font-awesome",
"fab fa-font-awesome-alt",
"fab fa-font-awesome-flag",
"fab fa-fonticons",
"fab fa-fonticons-fi",
"fab fa-fort-awesome",
"fab fa-fort-awesome-alt",
"fab fa-forumbee",
"fas fa-forward",
"fab fa-foursquare",
"fab fa-free-code-camp",
"fab fa-freebsd",
"fas fa-frown", "far fa-frown",
"fas fa-futbol", "far fa-futbol",
"fas fa-gamepad",
"fas fa-gavel",
"fas fa-gem", "far fa-gem",
"fas fa-genderless",
"fab fa-get-pocket",
"fab fa-gg",
"fab fa-gg-circle",
"fas fa-gift",
"fab fa-git",
"fab fa-git-square",
"fab fa-github",
"fab fa-github-alt",
"fab fa-github-square",
"fab fa-gitkraken",
"fab fa-gitlab",
"fab fa-gitter",
"fas fa-glass-martini",
"fab fa-glide",
"fab fa-glide-g",
"fas fa-globe",
"fab fa-gofore",
"fab fa-goodreads",
"fab fa-goodreads-g",
"fab fa-google",
"fab fa-google-drive",
"fab fa-google-play",
"fab fa-google-plus",
"fab fa-google-plus-g",
"fab fa-google-plus-square",
"fab fa-google-wallet",
"fas fa-graduation-cap",
"fab fa-gratipay",
"fab fa-grav",
"fab fa-gripfire",
"fab fa-grunt",
"fab fa-gulp",
"fas fa-h-square",
"fab fa-hacker-news",
"fab fa-hacker-news-square",
"fas fa-hand-lizard", "far fa-hand-lizard",
"fas fa-hand-paper", "far fa-hand-paper",
"fas fa-hand-peace", "far fa-hand-peace",
"fas fa-hand-point-down", "far fa-hand-point-down",
"fas fa-hand-point-left", "far fa-hand-point-left",
"fas fa-hand-point-right", "far fa-hand-point-right",
"fas fa-hand-point-up", "far fa-hand-point-up",
"fas fa-hand-pointer", "far fa-hand-pointer",
"fas fa-hand-rock", "far fa-hand-rock",
"fas fa-hand-scissors", "far fa-hand-scissors",
"fas fa-hand-spock", "far fa-hand-spock",
"fas fa-handshake", "far fa-handshake",
"fas fa-hashtag",
"fas fa-hdd", "far fa-hdd",
"fas fa-heading",
"fas fa-headphones",
"fas fa-heart", "far fa-heart",
"fas fa-heartbeat",
"fab fa-hire-a-helper",
"fas fa-history",
"fas fa-home",
"fab fa-hooli",
"fas fa-hospital", "far fa-hospital",
"fab fa-hotjar",
"fas fa-hourglass", "far fa-hourglass",
"fas fa-hourglass-end",
"fas fa-hourglass-half",
"fas fa-hourglass-start",
"fab fa-houzz",
"fab fa-html5",
"fab fa-hubspot",
"fas fa-i-cursor",
"fas fa-id-badge", "far fa-id-badge",
"fas fa-id-card", "far fa-id-card",
"fas fa-image", "far fa-image",
"fas fa-images", "far fa-images",
"fab fa-imdb",
"fas fa-inbox",
"fas fa-indent",
"fas fa-industry",
"fas fa-info",
"fas fa-info-circle",
"fab fa-instagram",
"fab fa-internet-explorer",
"fab fa-ioxhost",
"fas fa-italic",
"fab fa-itunes",
"fab fa-itunes-note",
"fab fa-jenkins",
"fab fa-joget",
"fab fa-joomla",
"fab fa-js",
"fab fa-js-square",
"fab fa-jsfiddle",
"fas fa-key",
"fas fa-keyboard", "far fa-keyboard",
"fab fa-keycdn",
"fab fa-kickstarter",
"fab fa-kickstarter-k",
"fas fa-language",
"fas fa-laptop",
"fab fa-laravel",
"fab fa-lastfm",
"fab fa-lastfm-square",
"fas fa-leaf",
"fab fa-leanpub",
"fas fa-lemon", "far fa-lemon",
"fab fa-less",
"fas fa-level-down-alt",
"fas fa-level-up-alt",
"fas fa-life-ring", "far fa-life-ring",
"fas fa-lightbulb", "far fa-lightbulb",
"fab fa-line",
"fas fa-link",
"fab fa-linkedin",
"fab fa-linkedin-in",
"fab fa-linode",
"fab fa-linux",
"fas fa-lira-sign",
"fas fa-list",
"fas fa-list-alt", "far fa-list-alt",
"fas fa-list-ol",
"fas fa-list-ul",
"fas fa-location-arrow",
"fas fa-lock",
"fas fa-lock-open",
"fas fa-long-arrow-alt-down",
"fas fa-long-arrow-alt-left",
"fas fa-long-arrow-alt-right",
"fas fa-long-arrow-alt-up",
"fas fa-low-vision",
"fab fa-lyft",
"fab fa-magento",
"fas fa-magic",
"fas fa-magnet",
"fas fa-male",
"fas fa-map", "far fa-map",
"fas fa-map-marker",
"fas fa-map-marker-alt",
"fas fa-map-pin",
"fas fa-map-signs",
"fas fa-mars",
"fas fa-mars-double",
"fas fa-mars-stroke",
"fas fa-mars-stroke-h",
"fas fa-mars-stroke-v",
"fab fa-maxcdn",
"fab fa-medapps",
"fab fa-medium",
"fab fa-medium-m",
"fas fa-medkit",
"fab fa-medrt",
"fab fa-meetup",
"fas fa-meh", "far fa-meh",
"fas fa-mercury",
"fas fa-microchip",
"fas fa-microphone",
"fas fa-microphone-slash",
"fab fa-microsoft",
"fas fa-minus",
"fas fa-minus-circle",
"fas fa-minus-square", "far fa-minus-square",
"fab fa-mix",
"fab fa-mixcloud",
"fab fa-mizuni",
"fas fa-mobile",
"fas fa-mobile-alt",
"fab fa-modx",
"fab fa-monero",
"fas fa-money-bill-alt", "far fa-money-bill-alt",
"fas fa-moon", "far fa-moon",
"fas fa-motorcycle",
"fas fa-mouse-pointer",
"fas fa-music",
"fab fa-napster",
"fas fa-neuter",
"fas fa-newspaper", "far fa-newspaper",
"fab fa-nintendo-switch",
"fab fa-node",
"fab fa-node-js",
"fab fa-npm",
"fab fa-ns8",
"fab fa-nutritionix",
"fas fa-object-group", "far fa-object-group",
"fas fa-object-ungroup", "far fa-object-ungroup",
"fab fa-odnoklassniki",
"fab fa-odnoklassniki-square",
"fab fa-opencart",
"fab fa-openid",
"fab fa-opera",
"fab fa-optin-monster",
"fab fa-osi",
"fas fa-outdent",
"fab fa-page4",
"fab fa-pagelines",
"fas fa-paint-brush",
"fab fa-palfed",
"fas fa-paper-plane", "far fa-paper-plane",
"fas fa-paperclip",
"fas fa-paragraph",
"fas fa-paste",
"fab fa-patreon",
"fas fa-pause",
"fas fa-pause-circle", "far fa-pause-circle",
"fas fa-paw",
"fab fa-paypal",
"fas fa-pen-square",
"fas fa-pencil-alt",
"fas fa-percent",
"fab fa-periscope",
"fab fa-phabricator",
"fab fa-phoenix-framework",
"fas fa-phone",
"fas fa-phone-square",
"fas fa-phone-volume",
"fab fa-pied-piper",
"fab fa-pied-piper-alt",
"fab fa-pied-piper-pp",
"fab fa-pinterest",
"fab fa-pinterest-p",
"fab fa-pinterest-square",
"fas fa-plane",
"fas fa-play",
"fas fa-play-circle", "far fa-play-circle",
"fab fa-playstation",
"fas fa-plug",
"fas fa-plus",
"fas fa-plus-circle",
"fas fa-plus-square", "far fa-plus-square",
"fas fa-podcast",
"fas fa-pound-sign",
"fas fa-power-off",
"fas fa-print",
"fab fa-product-hunt",
"fab fa-pushed",
"fas fa-puzzle-piece",
"fab fa-python",
"fab fa-qq",
"fas fa-qrcode",
"fas fa-question",
"fas fa-question-circle", "far fa-question-circle",
"fab fa-quora",
"fas fa-quote-left",
"fas fa-quote-right",
"fas fa-random",
"fab fa-ravelry",
"fab fa-react",
"fab fa-rebel",
"fas fa-recycle",
"fab fa-red-river",
"fab fa-reddit",
"fab fa-reddit-alien",
"fab fa-reddit-square",
"fas fa-redo",
"fas fa-redo-alt",
"fas fa-registered", "far fa-registered",
"fab fa-rendact",
"fab fa-renren",
"fas fa-reply",
"fas fa-reply-all",
"fab fa-replyd",
"fab fa-resolving",
"fas fa-retweet",
"fas fa-road",
"fas fa-rocket",
"fab fa-rocketchat",
"fab fa-rockrms",
"fas fa-rss",
"fas fa-rss-square",
"fas fa-ruble-sign",
"fas fa-rupee-sign",
"fab fa-safari",
"fab fa-sass",
"fas fa-save", "far fa-save",
"fab fa-schlix",
"fab fa-scribd",
"fas fa-search",
"fas fa-search-minus",
"fas fa-search-plus",
"fab fa-searchengin",
"fab fa-sellcast",
"fab fa-sellsy",
"fas fa-server",
"fab fa-servicestack",
"fas fa-share",
"fas fa-share-alt",
"fas fa-share-alt-square",
"fas fa-share-square", "far fa-share-square",
"fas fa-shekel-sign",
"fas fa-shield-alt",
"fas fa-ship",
"fab fa-shirtsinbulk",
"fas fa-shopping-bag",
"fas fa-shopping-basket",
"fas fa-shopping-cart",
"fas fa-shower",
"fas fa-sign-in-alt",
"fas fa-sign-language",
"fas fa-sign-out-alt",
"fas fa-signal",
"fab fa-simplybuilt",
"fab fa-sistrix",
"fas fa-sitemap",
"fab fa-skyatlas",
"fab fa-skype",
"fab fa-slack",
"fab fa-slack-hash",
"fas fa-sliders-h",
"fab fa-slideshare",
"fas fa-smile", "far fa-smile",
"fab fa-snapchat",
"fab fa-snapchat-ghost",
"fab fa-snapchat-square",
"fas fa-snowflake", "far fa-snowflake",
"fas fa-sort",
"fas fa-sort-alpha-down",
"fas fa-sort-alpha-up",
"fas fa-sort-amount-down",
"fas fa-sort-amount-up",
"fas fa-sort-down",
"fas fa-sort-numeric-down",
"fas fa-sort-numeric-up",
"fas fa-sort-up",
"fab fa-soundcloud",
"fas fa-space-shuttle",
"fab fa-speakap",
"fas fa-spinner",
"fab fa-spotify",
"fas fa-square", "far fa-square",
"fab fa-stack-exchange",
"fab fa-stack-overflow",
"fas fa-star", "far fa-star",
"fas fa-star-half", "far fa-star-half",
"fab fa-staylinked",
"fab fa-steam",
"fab fa-steam-square",
"fab fa-steam-symbol",
"fas fa-step-backward",
"fas fa-step-forward",
"fas fa-stethoscope",
"fab fa-sticker-mule",
"fas fa-sticky-note", "far fa-sticky-note",
"fas fa-stop",
"fas fa-stop-circle", "far fa-stop-circle",
"fab fa-strava",
"fas fa-street-view",
"fas fa-strikethrough",
"fab fa-stripe",
"fab fa-stripe-s",
"fab fa-studiovinari",
"fab fa-stumbleupon",
"fab fa-stumbleupon-circle",
"fas fa-subscript",
"fas fa-subway",
"fas fa-suitcase",
"fas fa-sun", "far fa-sun",
"fab fa-superpowers",
"fas fa-superscript",
"fab fa-supple",
"fas fa-sync",
"fas fa-sync-alt",
"fas fa-table",
"fas fa-tablet",
"fas fa-tablet-alt",
"fas fa-tachometer-alt",
"fas fa-tag",
"fas fa-tags",
"fas fa-tasks",
"fas fa-taxi",
"fab fa-telegram",
"fab fa-telegram-plane",
"fab fa-tencent-weibo",
"fas fa-terminal",
"fas fa-text-height",
"fas fa-text-width",
"fas fa-th",
"fas fa-th-large",
"fas fa-th-list",
"fab fa-themeisle",
"fas fa-thermometer-empty",
"fas fa-thermometer-full",
"fas fa-thermometer-half",
"fas fa-thermometer-quarter",
"fas fa-thermometer-three-quarters",
"fas fa-thumbs-down", "far fa-thumbs-down",
"fas fa-thumbs-up", "far fa-thumbs-up",
"fas fa-thumbtack",
"fas fa-ticket-alt",
"fas fa-times",
"fas fa-times-circle", "far fa-times-circle",
"fas fa-tint",
"fas fa-toggle-off",
"fas fa-toggle-on",
"fas fa-trademark",
"fas fa-train",
"fas fa-transgender",
"fas fa-transgender-alt",
"fas fa-trash",
"fas fa-trash-alt", "far fa-trash-alt",
"fas fa-tree",
"fab fa-trello",
"fab fa-tripadvisor",
"fas fa-trophy",
"fas fa-truck",
"fas fa-tty",
"fab fa-tumblr",
"fab fa-tumblr-square",
"fas fa-tv",
"fab fa-twitch",
"fab fa-twitter",
"fab fa-twitter-square",
"fab fa-typo3",
"fab fa-uber",
"fab fa-uikit",
"fas fa-umbrella",
"fas fa-underline",
"fas fa-undo",
"fas fa-undo-alt",
"fab fa-uniregistry",
"fas fa-universal-access",
"fas fa-university",
"fas fa-unlink",
"fas fa-unlock",
"fas fa-unlock-alt",
"fab fa-untappd",
"fas fa-upload",
"fab fa-usb",
"fas fa-user", "far fa-user",
"fas fa-user-circle", "far fa-user-circle",
"fas fa-user-md",
"fas fa-user-plus",
"fas fa-user-secret",
"fas fa-user-times",
"fas fa-users",
"fab fa-ussunnah",
"fas fa-utensil-spoon",
"fas fa-utensils",
"fab fa-vaadin",
"fas fa-venus",
"fas fa-venus-double",
"fas fa-venus-mars",
"fab fa-viacoin",
"fab fa-viadeo",
"fab fa-viadeo-square",
"fab fa-viber",
"fas fa-video",
"fab fa-vimeo",
"fab fa-vimeo-square",
"fab fa-vimeo-v",
"fab fa-vine",
"fab fa-vk",
"fab fa-vnv",
"fas fa-volume-down",
"fas fa-volume-off",
"fas fa-volume-up",
"fab fa-vuejs",
"fab fa-weibo",
"fab fa-weixin",
"fab fa-whatsapp",
"fab fa-whatsapp-square",
"fas fa-wheelchair",
"fab fa-whmcs",
"fas fa-wifi",
"fab fa-wikipedia-w",
"fas fa-window-close", "far fa-window-close",
"fas fa-window-maximize", "far fa-window-maximize",
"fas fa-window-minimize",
"fas fa-window-restore", "far fa-window-restore",
"fab fa-windows",
"fas fa-won-sign",
"fab fa-wordpress",
"fab fa-wordpress-simple",
"fab fa-wpbeginner",
"fab fa-wpexplorer",
"fab fa-wpforms",
"fas fa-wrench",
"fab fa-xbox",
"fab fa-xing",
"fab fa-xing-square",
"fab fa-y-combinator",
"fab fa-yahoo",
"fab fa-yandex",
"fab fa-yandex-international",
"fab fa-yelp",
"fas fa-yen-sign",
"fab fa-yoast",
"fab fa-youtube"
]
}
';
        $icons = json_decode($str ?? '', true);
        $icon = $icons['icons'][array_rand($icons['icons'])];
        return $icon;
    }
    
    /**
     * Get category parents including the category itself
     * @param number $category_id
     */
    public function _get_parent($category_id) {
        $cacheID = 'category_levels_' . $category_id;
        if(!$result = $this->gc_categories_cache->load($cacheID)) {
            $this->load->library('goclixy/gclosure');
            $gclosure = new Gclosure('categories', 'categories_hierarchy');
            $result = array();
            $levels = $gclosure->get_parent($category_id, null, 'DESC', true);
            if(!empty($levels[0]->id)) {
                foreach($levels as $level) {
                    $result[] = $level->id;
                }
            }
            $this->gc_categories_cache->save($result, $cacheID, array('categories'));
        }
        return $result;
    }
}