<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Click to Call Function
 */
class Reports extends Admin_Controller
{
	protected $permissionView   = 'Click_to_Call.Reports.View';
	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct()
	{
		parent::__construct();

		$this->auth->restrict($this->permissionView);
		$this->load->model('click_to_call_model', null, true);
		$this->lang->load('click_to_call');
		
		Template::set_block('sub_nav', 'reports/_sub_nav');
		
	}

	//--------------------------------------------------------------------


	/**
	 * Displays a list of form data.
	 *
	 * @return void
	 */
	public function index($filter='all', $offset = 0) {
		// Deleting anything?
		if (isset($_POST['delete'])) {
			$checked = $this->input->post('checked');

			if (is_array($checked) && count($checked)) {
				$result = FALSE;
				foreach ($checked as $pid) {
					$result = $this->click_to_call_model->delete($pid);
				}

				if ($result) {
					Template::set_message(count($checked) .' '. sprintf(lang('msg_delete_success'), lang('label_click_to_call')), 'success');
				} else {
					Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_click_to_call')) . $this->click_to_call_model->error, 'danger');
				}
			}
		}
		
		// Actions done, now display the view
		$where = '';
		
		// Filters
		$filter_type = $filter;
		
		switch($filter_type) {
			case 'desktop':
				$where = "CTC.device = 1";
				break;
			
			case 'mobile_devices_only':
				$where = "CTC.device != 1";
				break;
				
			case 'ipad':
				$where = "CTC.device = 2";
				break;
		
			case 'iphone':
				$where = "CTC.device = 3";
				break;
					
			case 'android':
				$where = "CTC.device = 4";
				break;
		
			case 'blackberry':
				$where = "CTC.device = 5";
				break;
		
			case 'webos':
				$where = "CTC.device = 6";
				break;			
					
			case 'all':
				// Nothing to do
				break;
		
		
			default:
				show_404(SITE_AREA ."/reports/click_to_call/index/$filter/");
		}
		
		$sort_by = "id DESC";
		//$this->session->unset_userdata('click_to_call_sort');
		if(isset($_POST['sort']) && $_POST['sort'] == 'U.display_name') {
			$order_by = $this->gc->sort('click_to_call');
		} else if(isset($_POST['sort']) && $_POST['sort'] == 'L.title') {
			$order_by = $this->gc->sort('click_to_call');
		} else {
			$order_by = $this->gc->sort('click_to_call', 'CTC');
		}
		if(!empty($order_by)) {
			$sort_by = $order_by;
		}
		$records = $this->click_to_call_model->get_click_to_call_reports(0, $where, $sort_by, $offset, $this->limit);
		
		// PaginationS
		$this->load->library('pagination');
		
		$total_clicks = $this->click_to_call_model->get_click_to_call_reports(1, $where, $sort_by, $offset, $this->limit);
		
		$pager['base_url'] = site_url(SITE_AREA ."/reports/click_to_call/index/$filter/");
		$pager['total_rows'] = $total_clicks;
		$pager['per_page'] = $this->limit;
		$pager['uri_segment']	= 6;
		
		$this->pagination->initialize($pager);		

		Template::set('records', $records);
		Template::set('filter_type', $filter_type);
		Template::set('index_url', site_url(SITE_AREA .'/reports/click_to_call/index/'));
		Template::set('toolbar_title', lang('manage_click_to_call'));
		Template::render();
	}
	
	/**
	 * Advanced Search for Click To Call
	 */
	public function search($offset = 0) {
		// DATE TIME PICKER
		Assets::add_js('moment.js');
		Assets::add_css ( 'bootstrap-datetimepicker.min.css' );
		Assets::add_js ( 'bootstrap-datetimepicker.min.js' );
		Assets::add_module_js('click_to_call', 'click_to_call.js');
		
		$keyword = $this->input->post('keyword');
		$from = $this->input->post('date_from');
		$to = $this->input->post('date_to');
		$device = $this->input->post('device') ? $this->input->post('device') : 0;
		if (isset($_POST['save'])) {
		    $this->gcrud->stats(array('module' => 'click_to_call'));
			$this->form_validation->set_rules('keyword', 'lang:placeholder_ctc_search', 'trim|required');
			$this->form_validation->set_rules('date_from', 'lang:label_from', 'trim|valid_date[TRUE]|required');
			$this->form_validation->set_rules('date_to', 'lang:label_to', 'trim|valid_date[TRUE]|compare_dates[date_from]|required');
			$this->form_validation->set_rules('device', 'lang:label_device', 'trim');
			
			if ($this->form_validation->run() === true) {		
				$this->click_to_call_model->limit($this->limit, $offset);
				$records = $this->click_to_call_model->search_click_to_call($keyword,$from,$to,$device);
				if(!$records) {
					Template::set_message(lang('error_no_record_found'), 'error');
				}
						
				// Pagination
				$this->load->library('pagination');
				
				$total_clicks = $this->click_to_call_model->total_count($keyword,$from,$to,$device);
				$pager['base_url'] = site_url(SITE_AREA ."/reports/click_to_call/search/");
				$pager['total_rows'] = $total_clicks->counts;
				$pager['per_page'] = $this->limit;
				$pager['uri_segment']	= 5;
				
				$this->pagination->initialize($pager);
				Template::set('records', $records);
			}
		}
		Template::set('toolbar_title', lang('label_click_to_call') .' '.lang('bf_action_search'));
		Template::render();
		
	}
}