<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
 || ##################################################################### ||
 || # GoClixy - Business Directory Script							   # ||
 || # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
 || # ----------------------------------------------------------------  # ||
 || # This file is owned and copyrighted by GoClixy.com. Your license   # ||
 || # confers  no  title  or ownership in the Script and is not a sale  # ||
 || # of any rights to the Software. This file may not be redistributed # ||
 || # in whole or significant part.									   # ||
 || # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
 || # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
 || ##################################################################### ||
 \*=======================================================================*/
class Click_to_call_model extends BF_Model {
    
    protected $table_name	= "click_to_call";
    protected $key			= "id";
    protected $soft_deletes	= false;
    protected $date_format	= "datetime";
    
    protected $log_user 	= FALSE;
    
    protected $set_created	= true;
    protected $set_modified = true;
    protected $created_field = "created_on";
    protected $modified_field = "modified_on";
    
    /*
     Customize the operations of the model without recreating the insert, update,
     etc methods by adding the method names to act as callbacks here.
     */
    protected $before_insert 	= array();
    protected $after_insert 	= array();
    protected $before_update 	= array();
    protected $after_update 	= array();
    protected $before_find 		= array();
    protected $after_find 		= array();
    protected $before_delete 	= array();
    protected $after_delete 	= array();
    
    /*
     For performance reasons, you may require your model to NOT return the
     id of the last inserted row as it is a bit of a slow method. This is
     primarily helpful when running big loops over data.
     */
    protected $return_insert_id 	= TRUE;
    
    // The default type of element data is returned as.
    protected $return_type 			= "object";
    
    // Items that are always removed from data arrays prior to
    // any inserts or updates.
    protected $protected_attributes = array();
    
    /*
     You may need to move certain rules (like required) into the
     $insert_validation_rules array and out of the standard validation array.
     That way it is only required during inserts, not updates which may only
     be updating a portion of the data.
     */
    protected $validation_rules 		= array(
        array(
            "field"		=> "click_to_call_device",
            "label"		=> "Device",
            "rules"		=> "required|max_length[50]"
        ),
        array(
            "field"		=> "click_to_call_ip_address",
            "label"		=> "IP Address",
            "rules"		=> "required|max_length[20]"
        ),
    );
    protected $insert_validation_rules 	= array();
    protected $skip_validation 			= FALSE;
    
    //--------------------------------------------------------------------
    
    /**
     * Get Click to Call Reports for index page
     */
    public function get_click_to_call_reports($count, $where, $sort_by, $offset = 0, $limit = 25) {
        if($count == 0) {
            $sql = "SELECT CTC.*, U.display_name, U.email AS user_email, U.slug AS user_slug, U.city AS user_city, L.title AS listing_title, L.slug AS listing_slug, L.city AS listing_city";
        } else {
            $sql = "SELECT COUNT(CTC.id) AS TotalCount";
        }
        $sql .= " FROM " . $this->db->dbprefix . "click_to_call CTC";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "users U ON U.id = CTC.user_id";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "listings L ON L.id = CTC.listing_id";
        if(!empty($where)) {
            $sql .= " WHERE " . $where;
        }
        if($count == 0) {
            $sql .= " ORDER BY " . $sort_by;
            $sql .= " LIMIT " . $offset . ", " . $limit;
            return $this->db->query ( $sql )->result();
        } else {
            $result = $this->db->query ( $sql )->row();
            return $result ? $result->TotalCount : 0;
        }
    }
    
    /**
     * Search Click to Call for total counts
     * @param string $keyword
     * @param datetime $from
     * @param datetime $to
     * @param integer $device
     */
    public function search_click_to_call($keyword, $from, $to, $device = 0) {
        $sql = "SELECT L.id as listing_id, CTC.device, L.title as listing_title, COUNT(CTC.listing_id) as count FROM " . $this->db->dbprefix . "click_to_call CTC";
        $sql .= " INNER JOIN " . $this->db->dbprefix . "listings L ON L.id = CTC.listing_id AND (L.title LIKE '%$keyword%' OR L.id = '$keyword')";
        $sql .= " WHERE (CTC.created_on BETWEEN '$from' AND '$to')";
        if($device != 0) {
            $sql .= " AND device = $device";
        }
        $sql .= " GROUP BY L.title, CTC.device";
        $query = $this->db->query ( $sql );
        return $query->result_array ();
    }
    
    /**
     * Search Click to Call for total counts
     * @param string $keyword
     * @param datetime $from
     * @param datetime $to
     * @param integer $device
     */
    public function total_count($keyword, $from, $to, $device) {
        $sql = "SELECT COUNT(*) AS counts FROM (SELECT CTC.listing_id, COUNT(CTC.listing_id) as TotalCounts FROM " . $this->db->dbprefix . "click_to_call CTC";
        $sql .= " INNER JOIN " . $this->db->dbprefix . "listings L ON L.id = CTC.listing_id AND (L.title LIKE '%$keyword%' OR L.id = '$keyword')";
        $sql .= " WHERE (CTC.created_on BETWEEN '$from' AND '$to')";
        if($device != 0) {
            $sql .= " AND device = $device";
        }
        $sql .= " GROUP BY L.title, CTC.device";
        $sql .= ") CC";
        $query = $this->db->query ( $sql );
        return $query->row ();
    }
}