<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Comments controller
 */
class Content extends Admin_Controller
{
	protected $permissionCreate = 'Comments.Content.Create';
	protected $permissionDelete = 'Comments.Content.Delete';
	protected $permissionEdit   = 'Comments.Content.Edit';
	protected $permissionView   = 'Comments.Content.View';

	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct()
	{
		parent::__construct();

		$this->auth->restrict($this->permissionView);
		$this->load->model('comments_model', null, true);
		$this->lang->load('comments');
				
		//LOAD RATINGS JAVASCRIPT AND CSS FILE
		Assets::add_css('star-rating.min.css');
		Assets::add_js('star-rating.min.js');
		Assets::add_js( $this->load->view('common/ratings_js', null, true ), 'inline' );
		
		Template::set_block('sub_nav', 'content/_sub_nav');
	}

	/**
	 * Displays a list of comments to the user.
	 *
	 * @return void
	 */
	public function index($module = 'all', $filter='all', $offset=0) {
		// Do we have any actions?
		if ($action = $this->input->post('submit')) {
		    $this->gcrud->stats(array('module' => 'comments'));
			$checked = $this->input->post('checked');
		
			switch(strtolower($action)) {
				case 'approve':
					$this->change_status($checked, 1);
					break;
		
				case 'disapprove':
					$this->change_status($checked, 0);
					break;
		
				case 'flag':
					$this->change_status($checked, 2);
					break;
		
				case 'spam':
					$this->change_status($checked, 3);
					break;
		
				case 'reject':
					$this->change_status($checked, 4);
					break;
		
				case 'delete':
					$this->delete($checked);
					break;
		
				case 'purge':
					$this->purge($checked);
					break;
		
				case 'restore':
					$this->restore($checked);
					break;
			}
		}
		
		// Actions done, now display the view
		$where = "T.deleted = 0";
		
		// Filters
		$filter_type = $filter;
		
		switch($filter_type) {
			case 'inactive':
				$where .= " AND T.status = 0";
				break;
		
			case 'active':
				$where .= " AND T.status = 1";
				break;
					
			case 'flagged':
				$where .= " AND T.status = 2";
				break;
		
			case 'spammed':
				$where .= " AND T.status = 3";
				break;
		
			case 'rejected':
				$where .= " AND T.status = 4";
				break;
		
			case 'deleted':
				$where = "T.deleted = 1";
				break;
					
			case 'all':
				// Nothing to do
				break;
		
		
			default:
				show_404("comments/index/$module/$filter/");
		}
		
		$searchterm = $this->gc->search('comments', $this->uri->segment(6), '/content/comments/');
		if(!empty($searchterm)) {
			$where .= " AND (T.id LIKE '%". $searchterm ."%' OR T.username LIKE '%". $searchterm ."%' OR T.title LIKE '%". $searchterm ."%' OR T.created_on LIKE '%". $searchterm ."%'";
			if($module != 'all') {
				$where .= " OR M.title LIKE '%". $searchterm ."%'";
			}
			$where .= ")";
		}
		
		$sort_by = "id DESC";
		if(isset($_POST['sort']) && $_POST['sort'] == 'display_name') {
			$order_by = $this->gc->sort('comments', 'U');
		} else {
			$order_by = $this->gc->sort('comments', 'T');
		}
		if(!empty($order_by)) {
			$sort_by = $order_by;
		}
		
		$select = array("U.display_name AS user");
		$join = array("LEFT JOIN ". $this->db->dbprefix ."users U ON U.id = T.user_id");

		if($module == 'all') {
			$records = $this->gc->get_all_row(0, 'comments', $where, $sort_by, $offset, $this->limit, NULL, array(), $select, $join);
		} else {
			$records = $this->gc->get_module_rows(0, 'comments', $module, $where, $sort_by, $offset, $this->limit, NULL, array(), $select, $join);
		}
		
		Template::set('records', $records);
		
		// Pagination
		$this->load->library('pagination');
		if($module == 'all') {
			$total_comments = $this->gc->get_all_row(1, 'comments',$where, $sort_by, $offset, $this->limit);
		} else {
			$total_comments = $this->gc->get_module_rows(1, 'comments', $module, $where, $sort_by, $offset, $this->limit, NULL, array(), $select, $join);
		}
		
		$pager['base_url'] = site_url(SITE_AREA ."/content/comments/index/$module/$filter/");
		$pager['total_rows'] = $total_comments;
		$pager['per_page'] = $this->limit;
		$pager['uri_segment']	= 7;
		
		$this->pagination->initialize($pager);
		
		Template::set('index_url', site_url(SITE_AREA .'/content/comments/index/'. $module .'/'));
		Template::set('filter_type', $filter_type);
		Template::set('module', $module);
		
		Template::set('toolbar_title', lang('manage_comments'));
		Template::set_view('comments/content/view');
		Template::render();
	}

	/**
	 * Allow admin to change comment status
	 * @param string $comments
	 * @param number $status
	 */
	private function change_status($comments=false, $status = 1)
	{
		if (!$comments) {
			return;
		}
		$this->auth->restrict($this->permissionEdit);
		$result = false;
		foreach($comments as $id) {
			// Update overall rating
			$this->comments_model->update_overall_rating($id, ($status == 1 ? 'insert' : 'delete'));
			$result = $this->comments_model->update_hierarchy($id, $status);
		}
		if ($result) {
			$this->update_points($comments);
			Template::set_message(sprintf(lang('msg_status_changed'), lang('label_comment_s')), 'success');
			//Template::redirect(current_url());
		} else {
			Template::set_message(lang('change_status_error'). $this->comments_model->error, 'danger');
		}
	}
	
	/**
	 * Allows editing of Comments data.
	 *
	 * @return void
	 */
	public function edit() {
		$id = $this->uri->segment(5);
		if (empty($id)) {
			Template::set_message(lang('invalid_id'), 'error');
			redirect(SITE_AREA .'/content/comments');
		}
		if (isset($_POST['save'])) {
			$this->auth->restrict('Comments.Content.Edit');

			if ($this->save_comments('update', $id)) {
			    $this->gcrud->stats(array('module' => 'comments'));
				// Log the activity
				log_activity($this->current_user->id, lang('act_edit_record') .': '. $id .' : '. $this->input->ip_address(), 'comments');
				
				// Update overall rating
				$this->comments_model->update_overall_rating($id, ($this->input->post('comments_status') && $this->input->post('comments_status') == 1 ? 'insert' : 'delete'));
				
				Template::set_message(sprintf(lang('msg_save_success'), lang('label_comment')), 'success');
				redirect(site_url(SITE_AREA .'/content/comments/'));
			} else {
				Template::set_message(sprintf(lang('msg_edit_failure'), lang('label_comment')) . $this->comments_model->error, 'danger');
			}
		} else if (isset($_POST['delete'])) {
		    $this->gcrud->stats(array('module' => 'comments'));
			$this->auth->restrict($this->permissionDelete);
			// Update overall rating before comment delete
			$this->comments_model->update_overall_rating($id, 'delete');
			if ($this->comments_model->delete($id)) {
				// Log the activity
				log_activity($this->current_user->id, lang('act_delete_record') .': '. $id .' : '. $this->input->ip_address(), 'comments');

				Template::set_message(sprintf(lang('msg_delete_success'), lang('label_comment_s')), 'success');

				redirect(SITE_AREA .'/content/comments');
			} else {
				Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_comment_s')) . $this->comments_model->error, 'danger');
			}
		}
		$comment = $this->gc->get_all_row(0, 'comments', "T.id = $id", NULL, 0, 0);
		// Display uploaded images to the Admin
		$this->load->model('images/images_model');
		$images = $this->images_model->get_images('comments', $id, false);
		Template::set('images', $images);
		Template::set('comments', $comment);
		Template::set('toolbar_title', lang('edit') .' '.lang('label_comment'));
		Template::set_view('content/comments_edit.php');
		Template::render();
	}
	
	//--------------------------------------------------------------------
	// !PRIVATE METHODS
	//--------------------------------------------------------------------

	/**
	 * Summary
	 *
	 * @param String $type Either "insert" or "update"
	 * @param Int	 $id	The ID of the record to update, ignored on inserts
	 *
	 * @return Mixed    An INT id for successful inserts, TRUE for successful updates, else FALSE
	 */
	private function save_comments($type='update', $id=0)
	{
		$_POST['id'] = $id;
		$data = array();
		$data['comment']        = $this->input->post('comment');
		$data['url']        	= $this->input->post('url');
		$data['rating']         = $this->input->post('rating') ? $this->input->post('rating') : 0;
		$data['status']         = $this->input->post('status');
		$data['modified_ip']    = $this->input->ip_address();
		
		$return = $this->comments_model->update($id, $data);
		$this->update_points($id);
		return $return;
	}

	/**
	 * Delete a comment or group of comments
	 * @access private
	 * @param int $id User to delete
	 * @return void
	 */
	private function delete($comments) {
		if(!$comments) {
			return false;
		}
		$result = false;
		foreach($comments as $id) {
			// Update overall rating
			$this->comments_model->update_overall_rating($id, 'delete');
			$result = $this->comments_model->delete_hierarchy($id);			
		}
		if ($result) {
			Template::set_message(sprintf(lang('msg_delete_success'), lang('label_comment_s')), 'success');
			Template::redirect(current_url());
		} else {
			Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_comment_s')). $this->comments_model->error, 'danger');
		}
	}//end _delete()
		
	/**
	 * Purge the selected comments which are already marked as deleted
	 * @param int $id Comment to purge
	 */
	private function purge($comments) {
	    $this->load->model('images/images_model');
		if(!$comments) {
			return false;
		}
		foreach($comments as $id) {
		    $this->images_model->purge_images('comments', $id);
			$this->comments_model->delete_hierarchy($id, true);
		}
		
		Template::set_message(sprintf(lang('msg_delete_success'), lang('label_comment_s')), 'success');
		Template::redirect(current_url());
	}//end _purge()
	
	
	/**
	 * Restore the deleted comment
	 */
	private function restore($id)
	{
		if ($this->comments_model->update_comment($id, array('comments.deleted'=>0))) {
			Template::set_message(sprintf(lang('msg_restored_success'), lang('label_comment_s')), 'success');
			Template::redirect(SITE_AREA .'/content/comments');
		}
		else {
			Template::set_message(lang('restored_error'). $this->comments_model->error, 'danger');
		}	
	}//end restore()
	
	/**
	 * Update points on comments status change
	 */ 
	private function update_points($id) {
	    if(!is_array($id)) {
	        $id = array($id);
	    }
	    $this->load->library('gc');
	    foreach($id as $row_id) {
	        $row = $this->db->where('id', $row_id)->get('comments')->row();
	        if($row) {
	            $this->gc->earn_points('comments', $row->user_id, $row_id);
	        }
	    }
	}
}