<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Migration_Install_comments extends Migration
{
	/**
	 * The name of the database table
	 *
	 * @var String
	 */
	private $table_name = 'comments';
	private $closure_table = 'comments_hierarchy';
	
	/**
		CREATE TABLE IF NOT EXISTS `rs0d_comments_hierarchy` (
		  `ancestor` int(11) NOT NULL DEFAULT '0',
		  `descendant` int(11) NOT NULL DEFAULT '0',
		  `depth` tinyint(3) NOT NULL DEFAULT '0',
		  PRIMARY KEY (`ancestor`,`descendant`),
		  KEY `ancestor` (`ancestor`),
		  KEY `tree_add` (`ancestor`,`descendant`,`depth`),
		  KEY `tree_dd` (`descendant`,`depth`)
		) ENGINE=InnoDB DEFAULT CHARSET=utf8;
	 */

	/**
	 * The table's fields
	 *
	 * @var Array
	 */
	private $fields = array(
		'id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'auto_increment' => TRUE,
		),
		'parent_id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'default' => 0,
		),
		'module_id' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default' => 0
		),
		'row_id' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'null' => FALSE,
		),
		'user_id' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'null' => TRUE,
		),
		'username' => array(
			'type' => 'VARCHAR',
			'constraint' => 55,
			'null' => FALSE,
		),
		'title' => array(
			'type' => 'VARCHAR',
			'constraint' => 100,
			'null' => FALSE,
		),
        'comment' => array(
			'type' => 'VARCHAR',
			'constraint' => 2000,
			'null' => FALSE,
		),
        'url' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE,
		),
		'ip' => array(
			'type' => 'VARCHAR',
			'constraint' => 50,
			'null' => TRUE,
		),
		'status' => array(
			'type' => 'INT',
			'constraint' => 2,
			'null' => FALSE,
			'default'=> 0,
		),
		'deleted' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default'=> 0,
		),
		'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'modified_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'modified_ip' => array(
			'type' => 'VARCHAR',
			'constraint' => 50,
			'null'=> TRUE
		)
	);
	
	/**
	 * The closure's table fields
	 *
	 * @var Array
	 */
	private $closure_fields = array(
		'ancestor' => array(
			'type' => 'INT',
			'constraint' => 11,
			'default' => 0,
		),
		'descendant' => array(
			'type' => 'INT',
			'constraint' => 11,
			'default' => 0,
		),
		'depth' => array(
			'type' => 'TINYINT',
			'constraint' => 3,
			'default' => 0,
		)
	);

	/**
	 * Install this migration
	 *
	 * @return void
	 */
	public function up()
	{
		$this->dbforge->add_field($this->fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('parent_id', false);
		$this->dbforge->add_key('user_id', false);
		$this->dbforge->add_key('module_id', false);
		$this->dbforge->add_key(array('module_id', 'row_id'), false);
		$this->dbforge->create_table($this->table_name);
		
		// Closure's table
		$this->dbforge->add_field($this->closure_fields);
		$this->dbforge->add_key(array('ancestor', 'descendant'), true);
		$this->dbforge->add_key('ancestor', false);
		$this->dbforge->add_key(array('ancestor', 'descendant', 'depth'), false);
		$this->dbforge->add_key(array('descendant', 'depth'), false);
		$this->dbforge->create_table($this->closure_table);
		$fk_closure = "ALTER TABLE " . $this->db->dbprefix . $this->closure_table . "";
		$fk_closure .= " ADD CONSTRAINT `comments_hierarchy_ibfk_1` FOREIGN KEY (`ancestor`) REFERENCES `" . $this->db->dbprefix . "comments` (`id`),";
		$fk_closure .= " ADD CONSTRAINT `comments_hierarchy_ibfk_2` FOREIGN KEY (`descendant`) REFERENCES `" . $this->db->dbprefix . "comments` (`id`);";
		$this->db->query ($fk_closure);
	}

	//--------------------------------------------------------------------

	/**
	 * Uninstall this migration
	 *
	 * @return void
	 */
	public function down() {
		$this->dbforge->drop_table($this->table_name);
		$this->dbforge->drop_table($this->closure_table);
	}

	//--------------------------------------------------------------------

}