<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class Comments_model extends BF_Model {

	protected $table_name	= "comments";
	protected $closure_table = 'comments_hierarchy';
	protected $key			= "id";
	protected $soft_deletes	= true;
	protected $date_format	= "datetime";

	protected $log_user 	= FALSE;

	protected $set_created	= true;
	protected $set_modified = true;
	protected $created_field = "created_on";
	protected $modified_field = "modified_on";

	/*
		Customize the operations of the model without recreating the insert, update,
		etc methods by adding the method names to act as callbacks here.
	 */
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 		= array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	/*
		For performance reasons, you may require your model to NOT return the
		id of the last inserted row as it is a bit of a slow method. This is
		primarily helpful when running big loops over data.
	 */
	protected $return_insert_id 	= TRUE;

	// The default type of element data is returned as.
	protected $return_type 			= "object";

	// Items that are always removed from data arrays prior to
	// any inserts or updates.
	protected $protected_attributes = array();

	/*
		You may need to move certain rules (like required) into the
		$insert_validation_rules array and out of the standard validation array.
		That way it is only required during inserts, not updates which may only
		be updating a portion of the data.
	 */
	protected $validation_rules 		= array(
		array(
			"field"		=> "comments_comment",
			"label"		=> "lang:label_comment",
			"rules"		=> "required|max_length[2000]|sanitize"
		),
	);
	protected $insert_validation_rules 	= array();
	protected $skip_validation 			= TRUE;
	
	// Print comment HTML 
	protected $comment_html 		= '';

	/**
	 * Display comments on index page
	 * @param int $id
	 */
	public function get_module_comments($id, $module) {
		$sql = "SELECT C.id, C.username, C.title as comment_title, C.comment, C.status, C.created_on, M.title";
		$sql .= " FROM " . $this->db->dbprefix . "comments C";
		$sql .= " INNER JOIN " . $this->db->dbprefix . $module . " M ON M.id = C.row_id";
		$sql .= " WHERE C.deleted = 0 AND C.status = 1 AND C.module_id = " . gc_module_id($module);
		if($id != NULL) {
			$sql .= " AND C.row_id = " . $id;
		}
		return $this->db->query($sql)->result();
	}
	
	/** Get comments for view page 
	 * @param int $id
	 */
	public function get_comment($id = NULL) {
		$this->db->select('comments.*, L.title');
		$this->db->join($this->db->dbprefix . 'listings L','L.id = comments.row_id', 'left');
		return $this->comments_model->find($id);
	}
	
	/**
	 * Update comment information
	 * @param number $comment_id
	 * @param object $data
	 */
	public function update_comment($comment_id, $data) {
		//SINGLE QUERY FOR MULTIPLE ITEMS
		$where = "id IN (" . implode(",", $comment_id) . ")";
		$this->db->where($where);
		return $this->db->update('comments', $data);
	}
	
	/**
	 * Performs a standard delete, but also allows for purging of a record.
	 *
	 * @access public
	 *
	 * @param int  $id    An INT with the record ID to delete.
	 * @param bool $purge If FALSE, will perform a soft-delete. If TRUE, will permanently delete the record.
	 *
	 * @return bool TRUE/FALSE
	 */
	public function delete_comment($comment_id=0, $purge=FALSE)
	{
		if ($purge === TRUE) {
			// temporarily set the soft_deletes to TRUE.
			$this->soft_deletes = FALSE;
		}
		
		//SINGLE QUERY FOR MULTIPLE ITEMS
		$where = "id IN (" . implode(",", $comment_id) . ")";
		return $this->comments_model->delete_where($where);
	}//end delete()
	
	/**
	 * Get all comment for member view comments section
	 * @param number $user_id
	 */
	public function get_listing_comments($user_id) {
		$this->db->select($this->db->dbprefix . 'comments.*, L.title as listing_title');
		$this->db->join($this->db->dbprefix . 'listings L','L.id = ' . $this->db->dbprefix . 'comments.row_id AND L.deleted = 0', 'INNER');
		return $this->comments_model->find_all_by(array($this->db->dbprefix . 'comments.deleted' => 0, 'L.user_id'=> $user_id));
	}
	
	/**
	 * Count all comments related to a particular member
	 * @param number $user_id
	 */
	public function count_listing_comments($user_id) {
		$this->db->select($this->db->dbprefix . 'comments.*, L.title as listing_title');
		$this->db->join($this->db->dbprefix . 'listings L','L.id = ' . $this->db->dbprefix . 'comments.row_id AND L.deleted = 0', 'INNER');
		return $this->comments_model->count_by(array($this->db->dbprefix . 'comments.deleted' => 0, 'L.user_id'=> $user_id));
	}
	
	/**
	 * Display Comments with Pagination
	 * @param number $count
	 * @param integer $row_id
	 * @param number $limit
	 * @param number $offset
	 * @return array|number
	 */
	public function display_comments($count, $module, $row_id, $limit = 4, $offset = 0) {
		if($count == 0) {
			$sql = "SELECT C.username, C.title, C.comment, U.profile_image, C.created_on";
		} else {
			$sql = "SELECT COUNT(C.id) AS TotalCounts";
		}
		$sql .= " FROM " . $this->db->dbprefix . "comments C";
		if($count == 0) {
			$sql .= " LEFT JOIN " . $this->db->dbprefix . "users U ON U.id = C.user_id";
		}
		$sql .= " WHERE C.row_id = $row_id AND C.module_id = " . gc_module_id($module) . " AND C.status = 1 AND C.deleted = 0";
		if($count == 0) {
			$sql .= " ORDER BY C.created_on DESC";
			if ($limit != 0) {
				$sql .= " LIMIT " . $offset . ", " . $limit;
			} else {
				$sql .= " LIMIT $limit";
			}
			$result = $this->db->query($sql)->result();
			return $result;
		} else {
			$result = $this->db->query($sql)->row();
			return $result ? $result->TotalCounts : 0;
		}
	}
	
	/**
	 * Member Area - used in gc/member_count() method to display
	 * the business specific comments to the user 
	 * 
	 * Module wise comments are counted - A module can be in a state of deleted
	 * Delete module comments should not be counted.
	 * 
	 * @param int $user_id
	 * @return number
	 */
	public function total_view_comments($user_id) {
		$modules = gc_modules();
		$total_comments = 0;
		if(!empty($modules)) {
			foreach($modules as $module) {
				$total_comments += $this->comments_model->count_view_comments($user_id, $module);
			}
		}
		return $total_comments; 
	}
	
	/**
	 * Member Area View Comments
	 * @param int $user_id
	 * @param string $module
	 * @return number
	 */
	public function count_view_comments($user_id, $module) {
		$sql = "SELECT COUNT(C.id) AS TotalCounts FROM ". $this->db->dbprefix ."comments C";
		$sql .= " INNER JOIN ". $this->db->dbprefix . $module ." M ON M.id = C.row_id AND C.module_id = " . gc_module_id($module);
		$sql .= " WHERE M.user_id = $user_id AND M.deleted = 0 AND C.deleted = 0";
		$result = $this->db->query($sql)->row();
		return $result ? $result->TotalCounts : 0;	
	}
	
	/**
	 * Fetch children.
	 *
	 * Example to generate nested tree:
	 *
	 *   $data = $this->get_children(1, TRUE, FALSE, TRUE);
	 *   print_r($data);
	 *
	 * If level/depth specified then self will be ignore.
	 *
	 * @param  int      node id
	 * @param  boolean  include self
	 * @param  mixed    node level/depth (e.g direct children = 1)
	 * @param  boolean  nestify the result
	 * @return mixed    array if query true
	 */
	public function get_children($node_id = 1, $module_id = 0, $row_id = 0, $self = FALSE, $level = FALSE, $nested = FALSE) {	
	    $select = 't.id, t.title, t.rating, t.comment, t.username, t.profile_image, t.images, u.profile_image as user_image, u.slug as user_slug, u.id as user_id, t.url, t.created_on, c2.ancestor as parent, c1.depth as level';
	    // $select .= ", (SELECT SUBSTRING_INDEX(GROUP_CONCAT(DISTINCT(CONCAT(COALESCE(`title`,''), ':' , url)) SEPARATOR ';'), ';', 3) FROM " . $this->db->dbprefix . "images WHERE module_id = " . gc_module_id('comments') . " AND row_id = t.id AND type_id = 0 AND status = 1 GROUP BY row_id) AS images";	    
	    $this->db->select($select);
		$this->db->from($this->closure_table." c1");
		$this->db->join($this->table_name.' t','t.id = c1.descendant');
		$this->db->join('users u','u.id = t.user_id', 'LEFT');
		$this->db->join($this->closure_table.' c2', 'c2.depth IN(1) AND c2.descendant = c1.descendant','LEFT'); // ugh backticking INTs in joins @TODO
		$this->db->where(array('c1.ancestor' => $node_id, 't.module_id' => $module_id, 't.row_id' => $row_id, 't.deleted' => 0, 't.status' => 1));
		if ( ! $self) {
			$this->db->where('c1.descendant <>', $node_id);
		}
		if ($level) {
			$this->db->where('c1.depth = ', $level);
		}
		$query = $this->db->get();
		if ( ! $query->num_rows()) {
			return FALSE;
		}
		$result = $query->result_array();
		if ($nested AND ! $level) {
			$trees = array();
			$root = null;
			foreach ($result as $row) {
				$trees[$row['id']] = $row;
			}
			foreach ($trees as $key => $row) {
				if( ! $root) {
					$root = $row['parent'];
				}
				$trees[$row['parent']]['children'][$key] =& $trees[$key];
			}
			$result = $trees[$root];
			if ( ! $self) {
				return $result['children'];
			}
			return isset($result['id']) ? $result : array_shift($result['children']);
		}
		return $result;	
	}
	

	/**
	 * Add a node (as last child).
	 *
	 * @param  int    node id
	 * @param  int    target id
	 * @return boolean
	 */
	public function add($node_id, $target_id = 0) {
		/*$sql = "INSERT INTO " . $this->db->dbprefix . "comments_hierarchy";
		$sql .= " SELECT CH.ancestor, $node_id, CH.depth + 1 FROM " . $this->db->dbprefix . "comments_hierarchy CH";
		$sql .= " WHERE CH.descendant = $descendant";
		$sql .= " UNION ALL SELECT $node_id, $node_id, 0";
		$result = $this->db->query($sql);
		return $result;*/
		$sql = 'SELECT ancestor, ' . $node_id . ', depth+1
				FROM '. $this->db->dbprefix . $this->closure_table.'
				WHERE descendant = ' . $target_id . '
				UNION ALL
				SELECT '.$node_id.',' . $node_id . ',0';
		$query = 'INSERT INTO ' . $this->db->dbprefix . $this->closure_table . ' ' . $sql;
		$result = $this->db->query($query);
		return $result;
	}
	/**
	 * Check if current node has children.
	 *
	 * @param   int       node id
	 * @return  boolean
	 */
	public function has_children($node_id)
	{
		$this->db->select('descendant')
		->from($this->closure_table)
		->where('ancestor', $node_id);
		$descendants = $this->db->get()->result_array();
		foreach ($descendants as $k => $v) {
			$descendants[$k] = $v['descendant'];
		}
	
		$query = $this->db->select('COUNT(*) as total')
		->from($this->closure_table)
		->where_in('ancestor', implode(',', $descendants))
		->where('descendant <>',$node_id)
		->get();
		return (bool) $query->row()->total;
	
	}
	/**
	 * Get parent(s) of current node.
	 *
	 * @param  int    current node id
	 * @param  mixed  level up (e.g direct parent = 1)
	 * @return mixed  array if succeed
	 */
	public function get_parent($node_id, $level = NULL)
	{
	
		$this->db->select('t.*')
		->from($this->table.' t')
		->join($this->closure_table.' c','t.id = c.ancestor')
		->where('c.descendant',$node_id)
		->where('c.ancestor <>',$node_id);
		if ($level) {
			$this->db->where('c.depth', $level);
		}
		$this->db->order_by('t.id');
		$query = $this->db->get();
		if ($query->num_rows()) {
			if ($level) {
				return $query->row();
			}
			return $query->result();
		}
		return FALSE;
	}
	
	/**
	 * TODO: optional recursion
	 *
	 * Delete node.
	 *
	 * @param  int      node id
	 * @param  boolean  if TRUE, it will also delete from reference table
	 * @return mixed
	 */
	public function delete_closure($node_id, $delete_reference = TRUE) {	
		$operand = 'SELECT descendant as id FROM '. $this->db->dbprefix . $this->closure_table.' WHERE ancestor = '.$node_id;
		$query = 'SELECT id, descendant FROM '. $this->db->dbprefix . $this->closure_table.' WHERE descendant IN ('.$operand.')';
		$result_start = $this->db->query($query);
	
		if ( $result_start->num_rows() > 0 ) {
			$descendants = pluck($result_start->result(), 'id');
			$result_delete = $this->db->where_in('id', implode(',', $descendants))->delete($this->closure_table);
				
			if ($delete_reference) {
				$descendants = pluck($result_start->result(), 'descendant');
				$delete_refs = $this->db->where_in('id', implode(',', $descendants))->delete($this->table);
			}
			return $result_delete;
		}
		return FALSE;
	}
	
	/**
	 * Get all children of a specific category for DELETE (SOFT and PURGE)
	 * @param $comment_id
	 */
	private function get_all_children($comment_id) {
		$sql = "SELECT GROUP_CONCAT(descendant) AS descendants FROM " . $this->db->dbprefix . $this->closure_table;
		$sql .= " WHERE descendant IN (SELECT descendant FROM " . $this->db->dbprefix . $this->closure_table ." WHERE ancestor = $comment_id)";
		// $sql .= " AND ancestor NOT IN (SELECT descendant FROM " . $this->db->dbprefix . $this->closure_table ." WHERE ancestor = $comment_id)";
		return $this->db->query($sql)->row();
	}
	
	/**
	 * Purge a comment completely from the application
	 * @param string $module
	 * @param int $id
	 */
	public function purge_comment($module, $id) {
		$comments = $this->comments_model->find_all_by(array('module_id' => gc_module_id($module), 'row_id' => $id));
		if($comments) {
			foreach($comments as $comment) {
				$this->delete_hierarchy($comment->id, true);
			}
		}
		// set auto increment value to the last id
		$alter_autoinc = "ALTER TABLE ". $this->db->dbprefix . "comments AUTO_INCREMENT = 1";
		$this->db->simple_query($alter_autoinc);
	}
	
	/**
	 * Delete all nodes
	 * @param $comment_id
	 * @param $purge - FALSE for SOFT DELETE
	 * @return boolean
	 */
	public function delete_hierarchy($comment_id, $purge = false) {
		$descendants = $this->get_all_children($comment_id);
		if(!$descendants) {
			return false;
		}
		$descendants = $descendants->descendants;
		if($purge) {
			// remove hierarchy first
			if(!empty($descendants)) {
				$hierarchy_sql = "DELETE FROM " . $this->db->dbprefix ."comments_hierarchy WHERE descendant IN (" . $descendants . ")";
				$this->db->query($hierarchy_sql);
				// purge comments
				$sql = "DELETE FROM " . $this->db->dbprefix ."comments WHERE id IN (" . $descendants . ")";
				$this->db->query($sql);
			} else {
				// purge comment
				$sql = "DELETE FROM " . $this->db->dbprefix ."comments WHERE id = " . $comment_id;
				$this->db->query($sql);
			}
			return true;
		} else if(!empty($descendants)) {
			// request is a SOFT DELETE - update delete status
			$sql = "UPDATE " . $this->db->dbprefix ."comments SET deleted = 1 WHERE id IN (" . $descendants . ")";
			$this->db->query($sql);
			return true;
		} else {
			// request is a SOFT DELETE - update delete status
			$sql = "UPDATE " . $this->db->dbprefix ."comments SET deleted = 1 WHERE id = " . $comment_id;
			$this->db->query($sql);
			return true;
		}
	}
	
	/**
	 * Update child nodes
	 * @param $comment_id
	 * @param $purge - FALSE for SOFT DELETE
	 * @return boolean
	 */
	public function update_hierarchy($comment_id, $status) {
		$descendants = $this->get_all_children($comment_id);		
		$descendants = $descendants->descendants;
		if(!empty($descendants)) { // A comment without children will have no descendants
			$sql = "UPDATE " . $this->db->dbprefix ."comments SET status = $status WHERE id IN (" . $descendants . ")";
			$this->db->query($sql);
		} else {
			$sql = "UPDATE " . $this->db->dbprefix ."comments SET status = $status WHERE id IN (" . $comment_id . ")";
			$this->db->query($sql);
		}
		return true;
	}
	
	/**
	 * Move node with its children to another node.
	 *
	 * @link  http://www.mysqlperformanceblog.com/2011/02/14/moving-subtrees-in-closure-table/
	 *
	 * @param  int  node to be moved
	 * @param  int  target node
	 * @return void
	 */
	public function move($node_id, $target_id)
	{
		// MySQL�s multi-table DELETE
		$query1 = 'DELETE a FROM '. $this->db->dbprefix . $this->closure_table.' AS a ';
		$query1 .= 'JOIN '. $this->db->dbprefix . $this->closure_table.' AS d ON a.descendant = d.descendant ';
		$query1 .= 'LEFT JOIN '. $this->db->dbprefix . $this->closure_table.' AS x ';
		$query1 .= 'ON x.ancestor = d.ancestor AND x.descendant = a.ancestor ';
		$query1 .= 'WHERE d.ancestor = '.$node_id.'  AND x.ancestor IS NULL';
		$res1 = $this->db->query($query1);
		$query2 = 'INSERT INTO '. $this->db->dbprefix . $this->closure_table.' (ancestor, descendant, depth) ';
		$query2 .= 'SELECT a.ancestor, b.descendant, a.depth+b.depth+1 ';
		$query2 .= 'FROM '. $this->db->dbprefix . $this->closure_table.' AS a JOIN '. $this->db->dbprefix . $this->closure_table.' AS b ';
		$query2 .= 'WHERE b.ancestor = '.$node_id.' AND a.descendant = '.$target_id;
		$res2 = $this->db->query($query2);
		return $res1 AND $res2;
	}
	
	/**
	 * Get (all) root nodes.
	 */
	public function get_root() {
		$this->db->select('r.descendant');
		$this->db->from($this->closure_table." r");
		$this->db->join($this->closure_table." p","r.descendant = p.descendant AND p.ancestor <> p.descendant","LEFT");
		$this->db->where('p.descendant IS NULL',NULL);
		$result = $this->db->get();
		if ($result) {
			return pluck($result->result(),'descendant');
		} else {
			return false;
		}
	}	
	
	
	
	/**
	 * Print comment tree - View Page
	 * @param int $module_id
	 * @param int $row_id
	 * @return string|boolean
	 */
	public function comment_tree($module_id, $row_id, $tpl = '_tpl_01', $limit = 0, $params = array()) {
	    $this->comment_html = '';
	    if(!empty($limit)) {
	        $this->db->limit($limit);
	    }
	    $parent_comments = $this->comments_model->select('id')->order_by('id', 'DESC')->find_all_by(array('module_id' => $module_id, 'row_id' => $row_id, 'deleted' => 0, 'status' => 1, 'parent_id' => 0));
	    if($parent_comments) {
	        foreach($parent_comments as $comment) {
	            $this->single_comment_tree($comment->id, $module_id, $row_id, $tpl, $params);
	        }
	        return $this->comment_html;
	    }
	    return false;
	}
	
	/**
	 * Single comment tree
	 * @param int $comment_id
	 * @return string
	 */
	public function single_comment_tree($comment_id, $module_id, $row_id, $tpl = '_tpl_01', $params = array()) {
	    $tree = $this->comments_model->get_children($comment_id, $module_id, $row_id, true, false, true);
	    if($tree) { // sometimes relation does not save on restore
	        $this->comment_html .= $this->load->view('comments/' . $tpl, array('comment' => $tree, 'isChildren' => false, 'tpl' => $tpl, 'params' => $params), true);
	    }
	    return $this->comment_html;
	}
	
	/**
	 * Comment has children hierarchy
	 * The first element of the array is not a multidimensional array
	 * @param array $tree
	 */
	public function print_comment($tree, $isChildren, $tpl = '_tpl_01', $params = array()) {
	    $comment_content = '';
	    foreach($tree as $comment) {
	        $content = $this->load->view('comments/' . $tpl, array('comment' => $comment, 'isChildren' => $isChildren, 'tpl' => $tpl, 'params' => $params), true);
	        $comment_content .= $content;
	    }
	    return $comment_content;
	}
	
	/**
	 * Update overall rating on comment approval/disapproval/delete
	 * Comment rating will always overwrite the existing rating
	 * @param int $comment_id
	 * @param string $type - INSERT on Approval, DELETE on Disapproval/Delete
	 */
	public function update_overall_rating($comment_id, $type = 'insert') {
		$comment = $this->comments_model->find($comment_id);		
		if($comment && $comment->rating > 0) {
			$user_id = $comment->user_id;
			$this->load->model('ratings/ratings_model');
			$current_rating = $this->ratings_model->find_by(array('row_id' => $comment->row_id, 'module_id' => $comment->module_id));
			// Match user system ip address with existing records
			if ($current_rating) { // Entry exist for specified row			
				// check whether user already rating for this listing or not
				$ip_address = $comment->created_ip;
				$rating_users = $this->db->where(array('rating_id' => $current_rating->id, 'user_id' => $user_id, 'ip_address' => $ip_address))->count_all_results('rating_users');
				if($type == 'insert') {
					$data ['total_ratings'] = $current_rating->total_ratings + $comment->rating; // Get existing ratings and increment with new rating
					$data ['total_users'] = $current_rating->total_users + 1; // Increment users by one
				} else if($type == 'delete') {
					if($current_rating->total_users > 0) {
						$data ['total_ratings'] = $current_rating->total_ratings - $comment->rating; // Get existing ratings and increment with new rating
						$data ['total_users'] = $current_rating->total_users - 1; // Decrement users by one
					} else {
						$data ['total_ratings'] = 0;
						$data ['total_users'] = 0;
					}
				}
				if($data['total_users'] > 0) {
					$average_rating = $data ['total_ratings'] / $data ['total_users'];
					$data ['average_rating'] = number_format((float)$average_rating, 2, '.', '');
				} else {
					$data ['average_rating'] = 0;
				}
				$data ['modified_on'] = date('Y-m-d H:i:s');
				// In case of delete, the rating_users table must record
				// If rating users does not exist do not update the overall rating
				if(($rating_users > 0 && $type == 'delete') || ($rating_users == 0 && $type == 'insert')) {
					if ($this->ratings_model->update( $current_rating->id, $data)) {
						$rating_user_data = array(
							'rating_id' => $current_rating->id,
						    'module_id' => $comment->module_id,
						    'row_id' => $comment->row_id,
						    'rating' => $comment->rating,
							'user_id' => $user_id,
							'ip_address' => $ip_address
						);
						if($type == 'insert') {
							// Update rating information
							// save users information with IP Address in the rating_users table
							// avoid further ratings from the save user
							$this->db->insert('rating_users', $rating_user_data);
						} else {
							$this->db->where($rating_user_data)->delete('rating_users');
						}
					}
				}
			} else if($type == 'insert') { // User is rating first time for this listing
				$data['row_id']		= $comment->row_id;
				$data['module_id']	= $comment->module_id;
				$data ['average_rating'] = $comment->rating;
				$data ['total_ratings'] = $comment->rating;
				$data ['total_users'] = 1;
				if ($this->ratings_model->insert($data)) { // Update rating information
					// save users information with IP Address in the rating_users table
					// to avoid further ratings from the save user
					$rating_user_data = array(
						'rating_id' => $this->db->insert_id(),
						'user_id' => (int) $user_id,
					    'ip_address' => empty($ip_address) ? $this->input->ip_address() : $ip_address
					);
					$this->db->insert('rating_users', $rating_user_data);
				}
			}			
			
			// update ratings in the module table
			$rating_module = gc_module_name($comment->module_id);
			if(!empty($comment->module_id)) {
			    $this->ratings_model->update_rating($rating_module, $comment->row_id);
			}
		}
	}
	
	/**
	 * COMMENT FORM
	 * Allow users to post comment on listing
	 */
	public function save_review($user_id, $is_mobile = false, $id = 0) {
	    $prefix = $is_mobile ? '' : 'review_';
	    $this->form_validation->set_rules('user_name', 'lang:label_name', 'required|trim');
	    $this->form_validation->set_rules('parent_id', 'lang:label_parent_id', 'is_numeric|trim|sanitize');
	    $this->form_validation->set_rules('row_id', 'lang:label_id', 'required|trim|sanitize');
	    $this->form_validation->set_rules('module', 'lang:label_module', 'required|trim|sanitize');
	    $this->form_validation->set_rules($prefix . 'title', 'lang:label_title', 'required|trim|max_length[255]');
	    $this->form_validation->set_rules($prefix . 'message', 'lang:label_review', 'required|trim|max_length[2000]');
	    $this->form_validation->set_rules($prefix . 'url', 'lang:label_website', 'valid_url|trim|max_length[255]');
	    if(!$is_mobile) {
	        $this->form_validation->set_rules($prefix . 'captcha_code', 'lang:captcha_code', 'trim|required|valid_captcha');
	    }
	    if ($this->form_validation->run() === FALSE) {
	        $validation_errors = validation_errors();
	        if ($validation_errors) {
	            $validation_error = $validation_errors;
	        }
	        return array('success' => false, 'result' => 0, 'message' => $validation_error, 'type' => 'danger');
	    } else {
	        $parent_id = $this->input->post ( 'parent_id' ) ? escape($this->input->post ( 'parent_id' )) : 0;
	        $comment_title = escape($this->input->post ( $prefix . 'title' ));
	        $comment_status = (int) settings_item('lst.auto_approve_review');
	        // check for the objectionable words
	        if(settings_item('site.international') == 1 && str_contains_word($comment_title, array())) {
	            $comment_status = 3;
	        }
	        $data = array (
	            'username' => escape($this->input->post ( 'user_name' )),
	            'rating' => $this->input->post($prefix . 'rating') ? $this->input->post($prefix . 'rating') : 0,
	            'title' => $comment_title,
	            'comment' => escape($this->input->post ( $prefix . 'message' )),
	            'url' => $this->input->post ( $prefix . 'url' ),
	            'status' => $comment_status
	        );
	        $success = false;
	        if(!empty($id)) { // update comment
	            $success = $this->db->where('id', $id)->update('comments', $data);
	        } else {	            
	            $data['row_id'] = escape($this->input->post ( 'row_id' ));
	            $data['module_id'] = gc_module_id(escape($this->input->post ('module')));
	            $data['parent_id'] = $parent_id;
	            $data['created_ip']	= $this->input->ip_address();
	            $data['user_id'] = (int) $user_id;
	            if ($id = $this->comments_model->insert( $data )) {
	                $success = true;
	                // Allow user to add 3 images with a review
	                if(settings_item('lst.review_images') == 1) {
	                    // Save uploaded images in the images table
	                    $this->load->library('image_handling');
	                    $tmp_id = $this->input->post('tmp_id');
	                    $this->image_handling->save_images('comments', $id, $tmp_id, $comment_status);
	                }
	                
	                // save comment hierarchy
	                $this->comments_model->add($id, ($parent_id == 0 ? $id : $parent_id));
	                
	                // Earn Points
	                $this->gc->earn_points('comments', (isset($data['user_id']) ? $data['user_id'] : 0), $id);
	            }
	        }
	        if(!$success) {
	            $result = 0;
	            $type = 'error';
	            $message = lang('failure_message');
	        } else {	            
	            $result = 1;
	            $type = 'success';
	            $message = lang('success_message');
	        }
	        return array('success' => $result, 'result' => $result, 'message' => $message, 'type' => $type);
	    }
	}
	
	/**
	 * Get Comments for Mobile App
	 * @param number $count
	 * @param string $module
	 * @param number $row_id
	 * @param string $searchterm
	 * @param number $limit
	 * @param number $offset
	 */
	public function get_comments($count, $module, $row_id, $searchterm, $limit = 0, $offset = 0, $user_id = 0, $mobile = false) {
	    $module_id = gc_module_id($module);
	    if($count == 0) {
	        $sql = "SELECT M.id, M.user_id, M.title, M.comment, M.rating, M.created_on, U.display_name, U.profile_image, M.status";
	    } else {
	        $sql = "SELECT COUNT(M.id) AS total_count";
	    }
	    $sql .= " FROM " . $this->db->dbprefix . $this->table_name . " M";
	    $sql .= " LEFT JOIN " . $this->db->dbprefix . "users U ON U.id = M.user_id";
	    $sql .= " WHERE M.deleted = 0 AND (M.status = 1 OR (M.user_id = " . $user_id . " AND M.status = 0)) AND M.module_id = " . $module_id . " AND M.row_id = " . $row_id;
	    if(!empty($searchterm)) {
	        $sql .= " AND M.title LIKE '%" . $searchterm . "%'";
	    }
	    if($count == 0) {
	        $sql .= " ORDER BY M.id DESC";
	        if ($limit != 0) {
	            $sql .= " LIMIT " . $offset . ", " . $limit;
	        } else {
	            $sql .= " LIMIT $limit";
	        }
	        $records = $this->db->query($sql)->result();
	        if($records && $mobile = true) {
	            foreach($records as $key => $record) {
	                $records[$key]->created_on = gc_date_format($record->created_on);
	                $records[$key]->profile_image = !empty($record->profile_image) ? gc_image_url($record->profile_image, true, true, 'users') : base_url('assets/images/dp.jpg');
	            }
	        }
	        return $records;
	    } else {
	        $result = $this->db->query($sql)->row();
	        return $result ? $result->total_count : 0;
	    }
	}
}