<?php defined('BASEPATH') || exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Online Software Solutions							   # ||
|| # Copyright (C) 2021 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Financial controller
 */
class Financial extends Admin_Controller {
    protected $permissionView   = 'Commissions.Financial.View';

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
        
        $this->auth->restrict($this->permissionView);
        $this->load->model('commissions/commissions_model');
        $this->lang->load('commissions');        
      	$this->form_validation->set_error_delimiters("<span class='error'>", "</span>");        
        Template::set_block('sub_nav', 'financial/_sub_nav');
        Assets::add_module_js('commissions', 'commissions.js');
    }
    
    /**
     * Display Summary to the User
     */
    public function index() {
        // Display Chart
        Assets::add_js('moment.js');
        Assets::add_css('bootstrap-datetimepicker.min.css' );
        Assets::add_js('bootstrap-datetimepicker.min.js' );
        Assets::add_css('chart.css');
        Assets::add_js('Chart.min.js');
        $this->session->unset_userdata('chart_query_params');
        $record = $this->commissions_model->summary();
        $tabular_blocks = array();
        $sales_by_months = $this->gcrud->chart('months', array('status_id' => 2));
        Assets::add_js($this->load->view('common/_chart_js', array('table' => 'commissions', 'field_id' => 'sixMonthSales', 'labels' => $sales_by_months['labels'], 'data' => $sales_by_months['data'], 'total' => $sales_by_months['total'], 'label' => lang('commission_total'), 'with_dates' => false), true), 'inline');
        $result = $this->gcrud->chart('current_month', array('status_id' => 2));
        Assets::add_js($this->load->view('common/_chart_js', array('table' => 'commissions', 'field_id' => 'monthlyChart', 'labels' => $result['labels'], 'data' => $result['data'], 'total' => $result['total'], 'label' => lang('commission_total'), 'with_dates' => true), true), 'inline');
        Template::set(array(
            'toolbar_title' => lang('gc_menu_commissions') . ' - ' . lang('gc_menu_summary'),
            'record' => $record,
            'inactive_commissions' => $this->gcrud->get_all_crud(0, 'M.status_id = 1', 'M.created_on DESC', 10, 0, array('module' => 'commissions', 'prefix' => '')),
            'recent_commissions' => $this->gcrud->get_all_crud(0, null, 'M.created_on DESC', 10, 0, array('module' => 'commissions', 'prefix' => '')),
            'top_earners' => $this->commissions_model->commission_by_users()
        ));
        Template::render();
    }
    
    /**
     * Display a list of Commissions data
     * @param string $module
     * @param string $filter
     * @param number $offset
     */
    public function manage($module = 'commissions', $filter = 'all', $offset = 0) {
        $extra = array(
            'model' => 'commissions',
            'context' => 'financial',
            'fields' => $module
        );
        
        if($module == 'levels' && isset($_POST['hierarchies'])) {
            $this->session->set_userdata('commission_level_hierarchies', (int) $_POST['hierarchies']);
            Template::redirect(site_url(SITE_AREA . '/financial/commissions/manage/hierarchies'));
        }
        if($module == 'hierarchies') {
            if(isset($_POST['clear'])) {
                $this->session->unset_userdata('commission_level_hierarchies');
            }
            if($this->session->userdata('commission_level_hierarchies')) {
                $extra['query_where'] = 'M.level_id = ' . $this->session->userdata('commission_level_hierarchies');
                $extra_action = '<button class="btn btn-danger btn-sm ml-1" name="clear" value="clear" type="submit"><i class="fas fa-list fa-fw mr-1"></i>' . lang('commission_hierarchies_all') . '</button>';
                Template::set('extra_action', $extra_action);
            }
        }
        if($module == 'commissions') {
            // affiliate_statuses table
            $extra['table_prefix'] = 'commission';
        } else {
            $extra['prefix'] = 'commission';
            if($module == 'hierarchies' || $module == 'coupons') {                
                $extra['actions'] = array( // Extra buttons
                    array('type' => 'link', 'link' => site_url(SITE_AREA . '/financial/commissions/form/' . $module . '/edit/%s'), 'label' => '<i class="fas fa-edit fa-fw mr-1"></i>' . lang('edit'), 'class' => 'btn btn-xs btn-primary'),
                );
            }
            if($module == 'levels') {
                $extra['actions'] = array( // Extra buttons
                    array('type' => 'link', 'link' => site_url(SITE_AREA . '/financial/commissions/level/%s'), 'label' => '<i class="fas fa-money-bill fa-fw mr-1"></i>' . lang('gc_menu_commissions'), 'class' => 'btn btn-xs btn-primary'),
                    array('type' => 'button', 'name' => 'hierarchies', 'label' => '<i class="fas fa-sitemap fa-fw mr-1"></i>' . lang('commission_hierarchies'), 'class' => 'btn btn-xs btn-secondary'),
                );
            }
        }
        $this->gc->module_view($module, $filter, $offset, 'admin', $extra);
    }
    
    /**
     * Add/Edit Commissions Data
     *
     * @return void
     */
    public function form($module = 'statuses', $action = 'create') {
        $extra = array(
            'selectize' => true,
            'model' => 'commissions',
            'context' => 'financial',
            'fields_method' => $module
        );
        if($module != 'commissions') {
            $extra['prefix'] = 'commission';
            $extra['module_name'] = 'commissions';            
        }
        if($module == 'hierarchies') {
            $extra['method'] = 'after_hierarchies_save';
        }
        $this->gcrud->form($module, $action, 'admin', $extra);
    }
    
    /**
     * Level
     * @param int $id
     */
    public function level($id) {
        $level = $this->db->where('id', $id)->get('commission_levels')->row();
        if(!$level) {
            Template::set_message(lang('invalid_id'), 'danger');
            Template::redirect(site_url(SITE_AREA . '/financial/commissions/manage/levels'));
        } 
        // module search js and css dependencies
        Assets::add_css('selectize.bootstrap3.css');
        Assets::add_js('selectize.min.js');
        Assets::add_js('selectize-search.js');
        $this->lang->load('coupons/coupons');
        $applicable_modules = $this->commissions_model->applicable_modules;// array('packages', 'ads', 'leads', 'claims', 'pyg', 'products', 'bookings'); // 'wallets'
        $record = array();
        if(isset($_POST['save'])) {
            $this->gcrud->stats(array('module' => 'commissions'));
            $this->form_validation->set_rules('module', 'lang:label_module', 'trim|is_array');
            $this->form_validation->set_rules('include', 'lang:label_include', 'trim|is_array');
            $this->form_validation->set_rules('exclude', 'lang:label_exclude', 'trim|is_array');
            if($this->form_validation->run() == TRUE) {
                $module_pricing = $this->input->post('module');
                foreach($module_pricing as $pricing_module => $amount) {
                    // pricing modules
                    $pricing_module_id = gc_module_id($pricing_module);
                    $row = $this->db->where(array('level_id' => $id, 'module_id' => $pricing_module_id))->get('commission_level_modules')->row();
                    if($row) {
                        $this->db->where('id', $row->id)->update('commission_level_modules', array('amount' => $amount));
                    } else {
                        $data_module = array(
                            'level_id' => $id,
                            'module_id' => $pricing_module_id,
                            'amount' => (int) $amount,
                        );
                        $this->db->insert('commission_level_modules', $data_module);
                    }                    
                }
                $this->level_include_exclude($id, 'include');
                $this->level_include_exclude($id, 'exclude');
                Template::set_message(sprintf(lang('msg_action_success'), lang('singular_commission_levels'), lang('label_updated')), 'success');
            }
        }
        // saved data
        $pricing_modules = $this->db->where('level_id', $id)->get('commission_level_modules')->result();
        if($pricing_modules) {
            foreach($pricing_modules as $pricing) {
                if(!empty($pricing->module_id)) {
                    $record['module'][gc_module_name($pricing->module_id)] = $pricing->amount; 
                }              
            }
        }
        $exceptions = $this->db->where('level_id', $id)->get('commission_level_exceptions')->result();
        if(!empty($exceptions)) {
            foreach($exceptions as $exception) {
                if(!empty($exception->type_id)) {
                    $record['include'][gc_module_name($exception->module_id)] = $exception->row_ids;
                } else {
                    $record['exclude'][gc_module_name($exception->module_id)] = $exception->row_ids;
                }
            }
        }
        $exception_modules = array('packages', 'ads', 'products', 'categories');
        foreach($exception_modules as $exception_module) {
            $record['include'][$exception_module] = !empty($_POST['include'][$exception_module]) ? $_POST['include'][$exception_module] : (!empty($record['include'][$exception_module]) ? $record['include'][$exception_module] : '');
            $record['exclude'][$exception_module] = !empty($_POST['exclude'][$exception_module]) ? $_POST['exclude'][$exception_module] : (!empty($record['exclude'][$exception_module]) ? $record['exclude'][$exception_module] : '');
        }
        Template::set(array(
            'applicable_modules' => $applicable_modules,
            'exception_modules' => $exception_modules,
            'level' => $level,
            'record' => $record,
            'toolbar_title' => lang('singular_commission_levels') . ' - ' . $level->name
        ));
        Template::render();
    }
    
    /**
     * Level Include/Exclude
     * @param number $id
     * @param string $type
     */
    private function level_include_exclude($id, $type = 'include') {        
        $records = $this->input->post('module');
        $type_data = (array) $this->input->post($type);
        $type_id = $type == 'include' ? 1 : 0;
        foreach($records as $applicable_module => $value) {
            if(!empty($value)) {
                $row_ids = empty($type_data[$applicable_module]) ? 0 : $type_data[$applicable_module];
                $applicable_module_id = gc_module_id($applicable_module);
                $row = $this->db->where(array('type_id' => $type_id, 'level_id' => $id, 'module_id' => $applicable_module_id))->get('commission_level_exceptions')->row();
                if($row) {
                    $this->db->where('id', $row->id)->update('commission_level_exceptions', array('row_ids' => $row_ids));
                } else {
                    $data = array(
                        'type_id' => $type_id,
                        'level_id' => $id,
                        'module_id' => $applicable_module_id,
                        'row_ids' => $row_ids,
                    );
                    $this->db->insert('commission_level_exceptions', $data);
                }
            }
        }
    }
    
    /**
     * Save settings
     */
    public function settings() {
        clear_cache('site_settings'); // Clear Site Settings Cache
        $setting_name = 'commission.settings';
        $this->load->model('settings/settings_model');
        if (isset($_POST['save'])) {
            $this->gcrud->stats(array('module' => 'commissions'));
            // $this->form_validation->set_rules('modules[]', 'lang:applicable_modules', 'trim|required|is_numeric|is_array');
            $this->form_validation->set_rules('commission_level', 'lang:commission_level', 'trim');
            if($this->form_validation->run() == TRUE) {
                $settings = array(
                    'commission_level' => (int) $this->input->post('commission_level')
                );
                $data = array (array('name' => $setting_name, 'value' => json_encode($settings)));
                $updated = $this->settings_model->update_batch($data, 'name');
                if ($updated) {
                    Template::set_message(sprintf(lang('msg_save_success'), lang('gc_menu_settings')), 'success');
                    Template::redirect(site_url(SITE_AREA . '/financial/commissions/settings'));
                } else {
                    Template::set_message(sprintf(lang('msg_edit_failure'), lang('gc_menu_settings')), 'danger');
                }
            }
        }
        $result = $this->db->select('value')->get_where('settings', array('name' => $setting_name))->row();
        if($result) {
            $settings = json_decode($result->value ?? '');
        } else {
            $settings = json_decode(settings_item($setting_name) ?? '');
        }
        Template::set('settings', $settings);
        Template::set('toolbar_title', lang('gc_menu_settings'));
        Template::render();
    }
    
    public function fields() {
        $this->load->library('fields/fields');
        //$this->fields->prepare_fields('commission_levels');
        $this->fields->permissions('Commission_Products', 'Financial');
    }
    
    public function product_commissions() {
        Template::render();
    }
    
    /**
     * View Commissions
     * @param number $id - Commission Order ID
     * @return boolean
     */
    public function view($id) {
        if(isset($_POST['save'])) {
            $this->gcrud->stats(array('module' => 'commissions'));
            foreach($_POST as $key => $value) {
                if(strpos($key, 'amount_') !== false) {
                    $this->form_validation->set_rules($key, 'lang:label_amount', 'trim|is_decimal|required');
                }
            }            
            if ($this->form_validation->run() === true) {      
                $data = array();
                $date = date('Y-m-d H:i:s');
                $ip_address = $this->input->ip_address();
                foreach($_POST as $key => $value) {
                    if(strpos($key, 'amount_') !== false) {
                        $field = explode('_', $key ?? '');
                        $affiliate_id = $field[1];
                        $data[] = array(
                            'id' => $affiliate_id,
                            'amount' => $this->input->post($key),
                            /*'modified_on' => $date,
                            'modified_ip' => $ip_address*/
                        );
                    }
                }
                if(!empty($data)) {
                    $this->db->update_batch('commission_affiliates', $data, 'id');
                    $record = $this->commissions_model->get_commission_total($id);
                    // update status
                    $status_id = $this->input->post('status_id');
                    $order_data = array(
                        'amount' => $record->amount,
                        'status_id' => $status_id,
                        'modified_on' => $date,
                        'modified_ip' => $ip_address
                    );
                    $this->db->where('id', $id)->update('commissions', $order_data);
                    $this->commissions_model->update_commission_affiliates($id, array('status_id' => $status_id, 'modified_on' => $modified_on));
                    Template::set_message(sprintf(lang('msg_save_success'), lang('label_status')), 'success');
                }
                Template::redirect(current_url());
            }
        }
        $this->lang->load('orders/orders');
        $this->load->model('affiliates/affiliates_model');
        $order = $this->commissions_model->get_commission_order($id);
        if(!$order) {
            Template::set_message(lang('invalid_id'), 'danger');
            Template::redirect(site_url(SITE_AREA . '/financial/commissions'));
        }
        $records = array();
        if($order->module_id != 12) { // not a CPC commission
            $records = $this->commissions_model->get_ordered_items($order->order_id);
            if(!$records) {
                Template::set_message(lang('invalid_id'), 'danger');
                Template::redirect(site_url(SITE_AREA . '/financial/commissions'));
            }
            foreach($records as $key => $record) {
                $records[$key]->affiliates = $this->commissions_model->get_commission_affiliates($id, $record->id);
            }
        } else {
            $records = $this->commissions_model->get_commission_affiliates($id);
        }
        // get referral URL information if any
        $referral = array();
        if(!empty($order->referral_id)) {
            $referral = $this->db->where('id', $order->referral_id)->get('affiliate_referrals')->row();
        }
        Template::set(array(
            'toolbar_title' => lang('label_commission') . (!empty($order->order_id) ? ': ' . lang('label_order_id') . ' - ' . $order->order_id : ''),
            'order' => $order,
            'records' => $records,
            'referral' => $referral
        ));
        Template::render();
    }
    
    public function payouts() {
        $sql = "SELECT A.name, SUM(CA.amount) AS amount";
        $sql .= " FROM " . $this->db->dbprefix . "commission_affiliates CA";
        $sql .= " INNER JOIN " . $this->db->dbprefix . "affiliate_users A ON A.user_id = CA.user_id";
        $sql .= " WHERE CA.status_id = 2";
        $sql .= " GROUP BY CA.user_id";
        $result = $this->db->query($sql)->result();
    }
}