<?php defined('BASEPATH') || exit('No direct script access allowed');

class Migration_Install_commissions extends Migration
{
    /**
     * @var string The name of the database table
     */
    private $table_name = 'commissions';
    
    /**
     * @var array The table's fields
     */
    private $fields = array(
        'id' => array(
            'type'       => 'INT',
            'constraint' => 11,
            'auto_increment' => true,
        ),
        'order_id' => array(
            'type'       => 'VARCHAR',
            'constraint' => 20
        ),
        'user_id' => array(
            'type'       => 'BIGINT',
            'constraint' => 20,
            'default'    => 0,
        ),
        'module_id' => array(
            'type'       => 'TINYINT',
            'constraint' => 1,
            'default'    => 2
        ),
        'row_id' => array(
            'type'       => 'BIGINT',
            'constraint' => 20,
            'default'    => 0,
        ),
        'toggle_module_id' => array( // In case of Affiliate/Invitation Commissions, save the Module ID (products/packages/bookings) in this column
            'type'       => 'TINYINT',
            'constraint' => 1,
            'default'    => 0
        ),
        'toggle_row_id' => array( // In case of Affiliate/Invitation Commissions, save the Module Row ID (products/packages/bookings) in this column
            'type'       => 'BIGINT',
            'constraint' => 20,
            'default'    => 0,
        ),
        'value' => array(
            'type'       => 'FLOAT',
            'default'    => 0
        ),
        'type' => array(
            'type'       => 'CHAR',
            'constraint' => 1,
            'default'    => 'P'
        ),
        'amount' => array(
            'type'       => 'FLOAT',
            'default'    => 0
        ),
        'created_on DATETIME default CURRENT_TIMESTAMP'
    );
    
    /**
     * Install this version
     *
     * @return void
     */
    public function up() {
        $this->dbforge->add_field($this->fields);
        $this->dbforge->add_key('id', true);
        $this->dbforge->create_table($this->table_name);
        $unique_commission = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " ADD UNIQUE INDEX `unique_commission` (`order_id`, `module_id`, `row_id`)";
        $this->db->query($unique_commission);
    }
    
    /**
     * Uninstall this version
     *
     * @return void
     */
    public function down() {
        $this->dbforge->drop_table($this->table_name);
    }
}