<?php defined('BASEPATH') || exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Online Software Solutions							   # ||
|| # Copyright (C) 2021 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class Commissions_model extends BF_Model
{
    public $table_name	             = 'commissions';
    protected $table_statuses	     = 'commission_statuses';
    protected $table_levels          = 'commission_levels';
    protected $table_level_products  = 'commission_products';
    protected $table_hierarchies     = 'commission_hierarchies';
    protected $table_affiliates      = 'commission_affiliates';
	protected $key			= 'id';
	protected $date_format	= 'datetime';

	protected $log_user 	= false;
	protected $set_created	= false;
	protected $set_modified = false;
	protected $soft_deletes	= false;


	// Customize the operations of the model without recreating the insert,
    // update, etc. methods by adding the method names to act as callbacks here.
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 	    = array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	// For performance reasons, you may require your model to NOT return the id
	// of the last inserted row as it is a bit of a slow method. This is
    // primarily helpful when running big loops over data.
	protected $return_insert_id = true;

	// The default type for returned row data.
	protected $return_type = 'object';

	// Items that are always removed from data prior to inserts or updates.
	protected $protected_attributes = array();

	// You may need to move certain rules (like required) into the
	// $insert_validation_rules array and out of the standard validation array.
	// That way it is only required during inserts, not updates which may only
	// be updating a portion of the data.
	protected $validation_rules 		= array(
		array(
			'field' => 'order_id',
			'label' => 'lang:label_order_id',
			'rules' => 'max_length[50]',
		),
		array(
			'field' => 'module_id',
			'label' => 'lang:label_module_id',
			'rules' => 'max_length[1]',
		),
		array(
			'field' => 'row_id',
			'label' => 'lang:label_row_id',
			'rules' => 'max_length[20]',
		),
		array(
			'field' => 'user_id',
			'label' => 'lang:label_user_id',
			'rules' => 'max_length[20]',
		),
		array(
			'field' => 'percentage',
			'label' => 'lang:label_percentage',
			'rules' => 'max_length[0]',
		),
		array(
			'field' => 'amount',
			'label' => 'lang:label_amount',
			'rules' => 'max_length[0]',
		),
	);
	protected $insert_validation_rules  = array();
	protected $skip_validation 			= false;
	protected $context			        = 'financial';
	protected $moduleName               = 'commissions';
	public $applicable_modules          = array('packages', 'ads', 'leads', 'claims', 'pyg', 'products', 'bookings');

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
    }
    
    /**
     * Commissions Fields
     *
     * @return array
     */
    public function commissions($role = 'admin') {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'selectize',
                'name' => 'order_id',
                'label' => 'label_order_id',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'load:orders;module:orders;max_items:1',
                'options_detailed' => 1,
                'placeholder' => 'placeholder_order_id',
                'rules' => 'trim|max_length[20]',
            ),
            array(
                'type' => 'selectize',
                'name' => 'user_id',
                'label' => 'bf_user',
                'placeholder' => 'placeholder_selectize',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => true,
                'options' => 'load:users;module:users;max_items:1',
                'options_detailed' => 1,
                'rules' => 'trim|is_numeric|required',
            ),
            array(
                'type' => 'dropdown',
                'name' => 'module_id',
                'label' => 'label_type',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:gc_menu_affiliates;value:1|name:gc_menu_coupons;value:2|name:label_cpc;value:3',
                'options_detailed' => 1,
                'options_lang' => 1,
                'class' => 3,
                'rules' => 'trim|is_numeric|required',
            ),
            array(
                'type' => 'selectize',
                'name' => 'row_id',
                'label' => 'singular_coupons',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'options' => 'load:coupons;module:coupons;max_items:1',
                'options_detailed' => 1,
                'rules' => 'trim|is_numeric',
            ),
            array(
                'type' => 'text',
                'name' => 'ordered_item',
                'label' => 'label_title',
                'frontend' => false,
                'admin_only' => false,
                'frontend_index' => false,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'commissions',
                'label' => 'label_commission',
                'frontend' => false,
                'admin_only' => false,
                'frontend_index' => false,
                'admin_index' => true,
                'class' => 2,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'amount',
                'label' => 'label_amount',
                'frontend' => false,
                'admin_only' => false,
                'frontend_index' => true,
                'admin_index' => true,
                'class' => 2,
                'rules' => 'trim|float|required',
            ),
            array(
                'type' => 'dropdown',
                'name' => 'status_id',
                'label' => 'label_status',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => true,
                'options' => 'name:commission_statuses;value:database',
                'options_detailed' => 1,
                'class' => 3,
                'rules' => 'trim|required|is_numeric',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'is_affiliate',
                'label' => 'singular_affiliates',
                'frontend' => false,
                'admin_only' => false,
                'frontend_index' => false,
                'admin_index' => true,
                'class' => 3,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'date_time',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            
        );
        return $fields;
    }   
    
    /**
     * Commission Levels Fields
     *
     * @return array
     */
    public function levels() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'text',
                'name' => 'name',
                'label' => 'label_name',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
                'rules' => 'trim|required|max_length[255]',
            ),
            /*array(
                'type' => 'dropdown',
                'name' => 'type',
                'label' => 'label_type',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'options' => 'name:label_percentage;value:P|name:label_fixed;value:F',
                'options_detailed' => 1,
                'options_lang' => 1,
                'class' => 2,
                'rules' => 'trim'
            ),
            array(
                'type' => 'text',
                'name' => 'commission',
                'label' => 'label_commission',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'value' => 0,
                'class' => 2,
                'rules' => 'trim|is_decimal|required'
            ),*/
            array(
                'type' => 'text',
                'name' => 'cpc_amount',
                'label' => 'label_cpc_amount',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'class' => 2,
                'value' => 0,
                'rules' => 'trim|is_decimal|required'
            ),
            array(
                'type' => 'textarea',
                'name' => 'description',
                'label' => 'label_description',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim'
            ),
            array(
                'type' => 'textarea',
                'name' => 'hierarchy_description',
                'label' => 'hierarchy_description',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim'
            ),
            array(
                'type' => 'iconpicker',
                'name' => 'icon',
                'label' => 'label_icon',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'class' => 3,
                'rules' => 'trim'
            ),
            array(
                'type' => 'text',
                'name' => 'price',
                'label' => 'label_price',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'class' => 2,
                'value' => 0,
                'rules' => 'trim|is_decimal|required'
            ),
            array(
                'type' => 'text',
                'name' => 'offer_price',
                'label' => 'label_offer_price',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'class' => 2,
                'value' => 0,
                'rules' => 'trim|is_decimal|required'
            ),
            array(
                'type' => 'text',
                'name' => 'duration',
                'label' => 'label_duration',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'class' => 2,
                'value' => 0,
                'rules' => 'trim|is_decimal|required'
            ),
            array(
                'type' => 'text',
                'name' => 'priority',
                'label' => 'label_priority',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'value' => 0,
                'class' => 2,
                'rules' => 'trim|is_numeric|required',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'is_default',
                'label' => 'label_default',
                'frontend' => false,
                'admin_only' => false,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'highlight',
                'label' => 'label_highlight',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'status',
                'label' => 'label_status',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            
        );
        return $fields;
    }
    
    /**
     * Commission Orders Fields
     *
     * @return array
     */
    public function orders() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'text',
                'name' => 'order_id',
                'label' => 'label_order_id',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|max_length[20]',
            ),
            array(
                'type' => 'text',
                'name' => 'order_item_id',
                'label' => 'label_order_item_id',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'module_id',
                'label' => 'label_module_id',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'row_id',
                'label' => 'label_row_id',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            
        );
        return $fields;
    }
    
    /**
     * Commission Affiliates Fields
     *
     * @return array
     */
    public function affiliates() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'text',
                'name' => 'commission_id',
                'label' => 'label_commission_id',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'selectize',
                'name' => 'user_id',
                'label' => 'bf_user',
                'placeholder' => 'placeholder_selectize',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'options' => 'load:users;module:users;max_items:1',
                'options_detailed' => 1,
                'rules' => 'trim|required',
            ),
            array(
                'type' => 'text',
                'name' => 'ref_by',
                'label' => 'singular_affiliate_users',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'depth',
                'label' => 'label_depth',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'type',
                'label' => 'label_type',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'commission',
                'label' => 'label_commission',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'amount',
                'label' => 'label_amount',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            
        );
        return $fields;
    }
    
    /**
     * Affiliate Coupons Fields
     *
     * @return array
     */
    public function coupons() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'selectize',
                'name' => 'user_ids',
                'label' => 'gc_menu_users',
                'placeholder' => 'placeholder_selectize',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => true,
                'options' => 'load:users;module:users;max_items:1000',
                'options_detailed' => 1,
                'tooltip' => lang('help_user_ids'),
                'rules' => 'trim',
            ),
            array(
                'type' => 'selectize',
                'name' => 'coupon_id',
                'label' => 'label_coupon',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'load:coupons;module:coupons;max_items:1',
                'options_detailed' => 1,
                'rules' => 'trim|is_numeric|unique[commission_coupons.coupon_id, commission_coupons.id]|required',
            ),
            array(
                'type' => 'dropdown',
                'name' => 'type',
                'label' => 'label_type',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'options' => 'name:label_percentage;value:P|name:label_fixed;value:F',
                'options_detailed' => 1,
                'options_lang' => 1,
                'class' => 2,
                'rules' => 'trim'
            ),
            array(
                'type' => 'text',
                'name' => 'commission',
                'label' => 'label_commission',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'value' => 0,
                'class' => 2,
                'rules' => 'trim|is_decimal|required'
            ),
            array(
                'type' => 'text',
                'name' => 'counts',
                'label' => 'label_counts',
                'frontend' => false,
                'admin_only' => false,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
                'value' => 0,
                'rules' => 'trim|required',
            ),
            array(
                'type' => 'text',
                'name' => 'hits',
                'label' => 'label_hits',
                'frontend' => false,
                'admin_only' => false,
                'frontend_index' => true,
                'admin_index' => true,
                'value' => 0,
                'rules' => 'trim|required',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'status',
                'label' => 'label_status',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_by',
                'label' => 'created_by',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_ip',
                'label' => 'created_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_by',
                'label' => 'modified_by',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_ip',
                'label' => 'modified_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            
        );
        return $fields;
    }
    
    /**
     * Commission Hierarchies Fields
     *
     * @return array
     */
    public function hierarchies($role = 'admin', $form_type = 'create') {
        $depth = 0;
        // Depth has hardcoded values from 1 to 100
        $depths = array();
        for($i = 0; $i <= 100; $i++) {
            $depths[$i] = 'name:' . $i . ';value:' . $i;
        }
        $parent_id = 0;
        $hierarchy_options = array(
            'name:' . lang('commission_hierarchy_1') . ';value:1',
            'name:' . lang('commission_hierarchy_2') . ';value:2',
            'name:' . lang('commission_hierarchy_3') . ';value:3'
        );
        if($this->uri->segment(4) == 'form') {
            // get id of the last saved hierarchy
            /*$id = $this->uri->segment(7); // edit page
            if(!empty($id)) {
                $row = $this->db->select('id')->where('id < ', $id)->order_by('id', 'DESC')->get($this->table_hierarchies)->row();
            } else {
                $row = $this->db->select('id')->order_by('id', 'DESC')->get($this->table_hierarchies)->row();
            }
            if($row) {
                $parent_id = $row->id;
                $hierarchy_options[] = 'name:' . sprintf(lang('commission_hierarchy_4'), $row->id) . ';value:4';
            }*/            
            // get depth from the table_hierarchies table, the depth column is unique, so there is no need of DISTINCT
            $id = $this->uri->segment(7); // edit page
            if(!empty($id)) {
                $row = $this->db->select('depth')->where('id', $id)->get($this->table_hierarchies)->row();
                if($row) {
                    $depth = $row->depth;
                }
            } else {
                $this->db->where('level_id', $this->session->userdata('commission_level_hierarchies') ? $this->session->userdata('commission_level_hierarchies') : 0);
                $depth_count = $this->db->count_all_results($this->table_hierarchies);
                $depth = $depth_count + 1;
            }
            if($depth > 1) {
                $hierarchy_options[] = 'name:' . sprintf(lang('commission_hierarchy_4'), $depth - 1) . ';value:4';
            }
        }
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            /*array(
                'type' => 'hidden',
                'name' => 'parent_id',
                'value' => $parent_id,
                'frontend_index' => false,
                'admin_index' => false,
                'frontend' => true,
                'admin_only' => true,
                'rules' => 'trim',
                'field_extra' => 'fixed:1'
            ),*/            
            array(
                'type' => 'dropdown',
                'name' => 'level_id',
                'label' => 'singular_commission_levels',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:commission_levels;value:database',
                'options_detailed' => 1,
                'options_lang' => 1,
                'value' => $this->session->userdata('commission_level_hierarchies') ? $this->session->userdata('commission_level_hierarchies') : 0,
                'class' => 3,
                'rules' => 'trim|required'
            ),
            array(
                'type' => 'dropdown',
                'name' => 'type_id',
                'label' => 'label_type',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'options' => implode('|', $hierarchy_options),
                'options_detailed' => 1,
                'options_lang' => 0,
                'rules' => 'trim'
            ),
            array(
                'type' => 'text',
                'name' => 'payout_amount',
                'label' => 'payout_amount',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'value' => 0,
                'class' => 2,
                'rules' => 'trim|is_decimal|required'
            ),
            array(
                'type' => 'text',
                'name' => 'depth',
                'frontend_index' => true,
                'admin_index' => true,
                'frontend' => true,
                'admin_only' => true,
                'rules' => 'trim|required|is_numeric',
                'value' => empty($depth) ? 0 : $depth,
                // 'options' => implode('|', $depths),
                // 'options_detailed' => 1,
                'class' => 1
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            )
        );
        return $fields;
    }
    
    /**
     * Display Summary
     * @return array
     */
    public function summary() {
        $sql = "SELECT SUM(amount) AS total_amount";
        $sql .= ", (SELECT SUM(amount) FROM " . $this->db->dbprefix . $this->table_name . " WHERE status_id = 1) AS inactive_amount";
        $sql .= ", (SELECT SUM(amount) FROM " . $this->db->dbprefix . $this->table_name . " WHERE status_id = 2) AS confirmed_amount";
        $sql .= ", (SELECT SUM(amount) FROM " . $this->db->dbprefix . $this->table_name . " WHERE status_id = 3) AS paid_amount";
        $sql .= " FROM " . $this->db->dbprefix . $this->table_name;
        $query = $this->db->query($sql);
        $record = $query->row();
        if($record) {
            $data = array();
            $i = 1;
            foreach($record as $key => $value) {
                switch($key) {
                    case 'total_amount':
                        $url = null;
                        break;
                    default:
                        $url = site_url(SITE_AREA . '/financial/commissions/manage/commissions/status_id--' . $i);
                }
                $data[$key] = array(
                    'value' => currency_format(empty($value) ? 0 : $value),
                    'url' => $url
                );
                $i++;
            }
            return (object) $data;
        } else {
            return $record;
        }
    }
    
    /**
     * Commissions Summary
     * @param number $status_id
     * @param string $select
     * @return string
     */
    public function _summary_commission($user_id, $status_id = 0, $where = '', $select = 'SUM(C.amount)') {
        $sql = "SELECT $select FROM " . $this->db->dbprefix . $this->table_name . " C";
        $sql .= " WHERE C.user_id = " . $user_id;
        if(!empty($where)) {
            $sql .= " AND " . $where;
        }
        if($status_id != 0) {
            $sql .= " AND C.status_id = " . $status_id;
        }
        return $sql;
    }
    
    /**
     * Display User Commissions
     * @return array
     */
    public function commission_by_users() {
        $sql = "SELECT user_id, " . user_fields() . ", M.total_amount";
        $sql .= " FROM (SELECT user_id, SUM(amount) AS total_amount";
        $sql .= " FROM " . $this->db->dbprefix . $this->table_name;
        $sql .= " GROUP BY user_id";
        $sql .= " ORDER BY total_amount DESC LIMIT 10) M";
        $sql .= " INNER JOIN " . $this->db->dbprefix . "users U ON U.id = M.user_id";
        $sql .= " ORDER BY total_amount DESC";
        $result = $this->db->query($sql)->result();
        return $result;
    }
    
    /**
     * Save commission information in the order_items table
     * Get this information on checkout time
     * Calculate commission and save order_id and order_item_id in the commissions table
     * Module and row can also be saved in the commissions table
     * Display module specific commission
     * If order item is reveresed, change the commission status to reversed, avoid recalculation
     * Save the amount to be paid to the user and also the site commission
     * Process original amount to be paid
     * Do not allow user to change the status of already processed commissions
     * 
     * COUPONS
     * Remove the invitation coupon concept, keep affiliate_coupons only
     * Allow admin to assign a coupon to all, single or multiple users
     * This will simplify the process, find the coupon assigned to a specific affiliate and calculate commission
     * 
     * NOTE: the calculate method is item specific, whereas hierarchy_commissions is order specific
     * i.e. call calcualte_method on foreach $order_items method and hierarchy_commissions after order status change
     */
    
    /**
     * Calculate Commissions
     * @param object $order
     * @param object $order_item
     */
    public function calculate($order, $order_item) {
        $this->module_commission($order, $order_item); // Products/Booking General Commissions
        $this->affiliate_commission($order, $order_item); // Affiliate Commissions
        $this->coupon_commission($order, $order_item); // Coupon Commission
    }
    
    /**
     * Calculate Hierarchy Commissions
     * This should run once, not on each order item
     * Note: the ref is stored in the orders table during checkout
     * ref_by is stored in the affiliates table
     *
     * Affiliate Commissions Columns
     * parent_id - Commission Parent (commission->id)
     * ref_by
     * commission_type_id
     * depth - Hierarchy
     * type - P or F
     * commission - Amount
     *
     * the parent_id is stored in the commissions table, this will be used to keep a record of parent commission
     * order_id and order_item_id will be used for calculation
     * module_id will be commission_hierarchies
     *
     * @param array $product_commissions
     * @param number $affiliate_id - id of the affiliate, not user_id
     */
    public function hierarchy_commissions($order) {
        if(empty($order->affiliate_id)) {
            return false;
        }
        // check if commissions are already calculated, change status only
        $hierarchy_records = $this->db->where(array('order_id' => $order->order_id, 'module_id' => gc_module_id($this->table_hierarchies)))->get('commissions')->result();
        if($hierarchy_records) {
            foreach($hierarchy_records as $hierarchy_record) {
                $hierarchy_commission_data = array('status_id' => $order->status_id == 2 ? 2 : 1, 'modified_on' => date('Y-m-d H:i:s'), 'modified_ip' => $this->input->ip_address());
                $this->db->where('id', $hierarchy_record->id)->update($this->table_name, $hierarchy_commission_data);
            }
        } else { // do the calculations
            $affiliate_id = $order->affiliate_id;
            $affiliate = $this->db->where('user_id', $affiliate_id)->get('affiliate_users')->row();
            if(!$affiliate) {
                return false;
            }
            // $commission_where = array('order_id' => $order->order_id, 'user_id' => $order->affiliate_id);
            // do not calculate hierarchy_commissions on coupon commissions
            $commission_where = "order_id = '" . $order->order_id . "' AND user_id = " . $order->affiliate_id . " AND module_id != " . gc_module_id('coupons');
            $earned_commissions = $this->db->where($commission_where)->get('commissions')->result_array();
            if($earned_commissions) {
                // get parents commission types from the commission_hierarchies table
                // calculate affiliate references commissions
                // save commission in the commission history table
                // get affiliate references
                $this->load->library('goclixy/gclosure');
                $gclosure = new Gclosure('affiliate_users', 'affiliate_users_hierarchies');
                $parents = $gclosure->get_parent($affiliate->id, null, 'DESC');
                if($parents) {
                    $commission_hierarchies = array();
                    // get all hierarchies commissions
                    $hierarchy_commissions = $this->db->where('level_id', $affiliate->level_id)->order_by('depth')->get($this->table_hierarchies)->result();
                    if($hierarchy_commissions) {
                        foreach($hierarchy_commissions as $hierarchy_commission) {
                            $commission_hierarchies[$hierarchy_commission->depth] = array(
                                'type_id' => $hierarchy_commission->type_id,
                                'commission' => $hierarchy_commission->payout_amount
                            );
                        }
                        $affiliate_hierarchy_commissions = array();
                        foreach($parents as $parent_affiliate) {
                            // 1. percent of the commission amount
                            // 2. percent of the sale amount
                            // 3. flat amount
                            // 4. percent of the tier 1 commission amount
                            if(array_key_exists($parent_affiliate->depth, $commission_hierarchies)) {
                                $commission_params = $commission_hierarchies[$parent_affiliate->depth];
                                $commission_type = 'P';
                                $commission_value = $commission_params['commission'];
                                // each product commission is calculated individually
                                foreach($earned_commissions as $earned_commission) {
                                    switch($commission_params['type_id']) {
                                        case 1:
                                            $parent_commission = $this->calculate_commission($commission_type, $commission_value, $earned_commission['amount']);
                                            break;
                                        case 2:
                                            $parent_commission = $this->calculate_commission($commission_type, $commission_value, $earned_commission['subtotal']);
                                            break;
                                        case 3:
                                            $commission_type = 'F';
                                            $parent_commission = $this->calculate_commission($commission_type, $commission_value);
                                            break;
                                        case 4:
                                            if(isset($affiliate_hierarchy_commissions[$parent_affiliate->depth - 1])) {
                                                $commission_amount = $affiliate_hierarchy_commissions[$parent_affiliate->depth - 1]['amount'];
                                                $parent_commission = $this->calculate_commission($commission_type, $commission_value, $commission_amount);
                                            }
                                            break;
                                    }
                                    if(isset($parent_commission)) {
                                        // used for calculations
                                        $affiliate_hierarchy_commissions[$parent_affiliate->depth] = array(
                                            'parent_affiliate_id' => $parent_affiliate->user_id,
                                            'type' => $commission_type,
                                            'commission' => $commission_value,
                                            'amount' => $parent_commission
                                        );
                                        // CPC/Products/Affiliate Tier commission information is stored in the commission_affiliates table
                                        // save data in the commission affiliates table
                                        $commissions_info = array(gc_module_id('commission_hierarchies') => array('commission' => $commission_value, 'type' => $commission_type));
                                        $commission_affiliate_data = array(
                                            'parent_id' => $earned_commission['id'],
                                            'order_id' => $earned_commission['order_id'],
                                            'order_item_id' => $earned_commission['order_item_id'],
                                            'user_id' => $parent_affiliate->user_id,
                                            'referral_id' => $affiliate_id, // the primary affiliate who earned the commission
                                            'depth' => $parent_affiliate->depth
                                        );
                                        $commission_data = array(
                                            'module_id' => gc_module_id($this->table_hierarchies),
                                            // 'params' => json_encode(array('type' => $commission_type, 'commission' => $commission_value)),
                                            'commissions' => json_encode($commissions_info),
                                            'amount' => $parent_commission,
                                            'status_id' => $earned_commission['status_id']
                                        );
                                        $commission_affiliate = $this->db->where($commission_affiliate_data)->get($this->table_name)->row();
                                        if($commission_affiliate) {
                                            $commission_data['modified_on'] = date('Y-m-d H:i:s');
                                            $this->db->where('id', $commission_affiliate->id)->update($this->table_name, $commission_data);
                                        } else {
                                            $commission_data['created_on'] = date('Y-m-d H:i:s');
                                            $this->db->insert($this->table_name, array_merge($commission_affiliate_data, $commission_data));
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    
    /**
     * Calculate CPC (Cost Per Click) Commissions
     * CPC uniqueness is not calculated here, you need to do it at the references table end.
     * Note: CPC is not applicable to hierarchies commissions
     * 
     * @param number $affiliate_id
     */
    public function calculate_cpc_commission($affiliate_id) {
        // get affiliate information
        $this->db->from('affiliate_users AU');
        $this->db->join('commission_levels CL', 'CL.id = AU.level_id', 'INNER');
        $affiliate = $this->select('AU.id, AU.user_id, CL.cpc_amount')->db->where(array('AU.user_id' => $affiliate_id, 'AU.status_id' => 2))->get()->row();
        if($affiliate->cpc_amount > 0) {
            $affiliate_id = $affiliate->user_id;
            $datetime = date('Y-m-d H:i:s');
            $ip_address = $this->input->ip_address();
            $commission_order_data = array(
                'user_id' => $affiliate_id,
                'order_id' => null,
                'module_id' => gc_module_id('commission_cpc'),
                'row_id' => 0,
                'amount' => $affiliate->cpc_amount,
                'status_id' => $this->gcrud->get_default_status('commission_statuses'),
                'created_on' => $datetime,
                'created_ip' => $ip_address
            );
            $this->db->insert($this->table_name, $commission_order_data);
        }
    }
    
    /**
     * Get Payout Commissions qualifies minimum payout balance
     * 
     * @param number $count
     * @param string $where
     * @param string $sort_by
     * @param number $limit
     * @param number $offset
     * @param array $extra
     * @return array
     */
    public function payout_commissions($count = 0, $where = null, $sort_by = 'C.created_on DESC', $limit = 15, $offset = 0, $extra = array()) {
        if($count == 1) {
            $sql = "SELECT COUNT(*) AS total_count";
        } else {
            $sql = " SELECT M.*, " . user_fields();
        }
        $sql .= " FROM (SELECT COUNT(C.id) AS commission_total, C.user_id, SUM(C.amount) AS commission_amount";
        $sql .= " FROM " . $this->db->dbprefix . $this->table_name . " C";
        $sql .= " WHERE C.status_id = 2"; // Confirmed Payments Only
        $sql .= " AND C.payout_id = 0"; // not yet paid
        if(!empty($where)) {
            $sql .= " AND " . $where;
        }
        $sql .= " GROUP BY C.user_id";
        $sql .= " ORDER BY " . $sort_by;
        $sql .= " ) M";
        $sql .= " INNER JOIN " . $this->db->dbprefix . "users U ON U.id = M.user_id";
        $sql .= " WHERE commission_amount > " . $extra['minimum_payout'];
        if ($count == 0) {
            if ($limit != 0) {
                $sql .= " LIMIT " . $offset . ", " . $limit;
            } else {
                $sql .= " LIMIT $limit";
            }
            $query = $this->db->query($sql);
            return $query->result();
        } else {
            $result = $this->db->query($sql)->row();
            return $result ? $result->total_count : 0;
        }
        $result = $this->db->query($sql)->result();
        return $result;
    }
    
    /**
     * Payout User Commissions Amount
     * @param number $user_id
     * @param number $status_id
     * @param string $where
     * @return array
     */
    public function payout_user_commission_amount($user_id, $status_id = 2, $where = null) {
        $sql = $this->_summary_commission($user_id, $status_id, $where, 'SUM(C.amount) AS commission_amount');
        $result = $this->db->query($sql)->row();
        return $result;
    }
    
    /**
     * Get Commission ORDER information
     * @param number $id
     */
    public function get_commission_order($id) {
        $sql = "SELECT M.*, O.order_id, O.order_total, O.currency_value, O.payment_method, O.created_on, O.created_on, PG.display_name AS payment_method, " . user_fields();
        $sql .= ", AU.name AS affiliate_name, AU.payment_method AS affiliate_payment_method, U.last_login";
        $sql .= ", S.color AS status_color, S.name AS status, S.name_lang AS status_lang";
        $sql .= " FROM " . $this->db->dbprefix . $this->table_name . " M";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . $this->table_statuses . " S ON S.id = M.status_id";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "orders O ON O.order_id = M.order_id";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "payment_gateways PG ON PG.name = O.payment_method";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "affiliate_users AU ON AU.user_id = M.user_id";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "users U ON U.id = M.user_id";
        $sql .= " WHERE M.id = " . $id;
        $result = $this->db->query($sql)->row();
        return $result;
    }
    
    /**
     * Get Commission of a specific ID
     * @param number $id
     * @return object
     */
    public function get_ordered_items($order_id) {
        $sql = "SELECT M.*";
        $sql .= " FROM " . $this->db->dbprefix . "order_items M";
        $sql .= " WHERE M.order_id = '" . $order_id . "'";
        $result = $this->db->query($sql)->result();
        return $result;
    }
    
    /**
     * Get All Commission Affiliatess
     * @param number $commission_id
     * @return object
     */
    public function get_commission_affiliates($commission_id, $order_item_id = 0) {
        $sql = "SELECT M.*, " . user_fields();
        $sql .= " FROM " . $this->db->dbprefix . $this->table_affiliates . " M";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "affiliate_users AU ON AU.user_id = M.ref_by";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "users U ON U.id = M.ref_by";
        $sql .= " WHERE M.commission_id = " . $commission_id;
        if($order_item_id > 0) {
            $sql .= " AND M.order_item_id = " . $order_item_id;
        }
        $sql .= " ORDER BY M.depth";
        $result = $this->db->query($sql)->result();
        return $result;
    }
    
    /**
     * Get the Referral ID
     * @return number
     */
    private function referral_id() {
        $this->load->library('crypt');
        return (int) $this->crypt->decode($this->input->cookie('referral_id'));
    }
    
    /**
     * Get Affiliate Commission Amount using the commission_level_exceptions and commission_level_modules tables.
     * @param number $module_id
     * @param number $row_id
     * @param number $affiliate_id - ID of the affiliate to get the Level ID
     */
    private function affiliate_commission($order, $order_item) {
        $affiliate_id = $order->affiliate_id;
        // the affiliate should not get commission on own referred products
        if(empty($affiliate_id) || $affiliate_id == $order->user_id) {
            return false;
        }
        $module_id = $order_item->module_id;
        $row_id = $order_item->row_id;
        $result = $this->check_commission($order, $order_item);
        if(!$result) { // first time calculation
            $sql = "SELECT CLM.amount FROM " . $this->db->dbprefix . "commission_level_exceptions M";
            $sql .= " INNER JOIN " . $this->db->dbprefix . "commission_level_modules CLM ON CLM.level_id = M.level_id AND CLM.module_id = " . $module_id;
            $sql .= " WHERE M.module_id = " . $module_id; 
            $sql .= " AND ((M.row_ids = 0 OR FIND_IN_SET(" . $row_id . ", M.row_ids)) AND M.type_id = 1)"; // the item must be in the include list
            // the item must not be in the exclude list
            // $sql .= " AND ((M.row_ids = 0 OR !FIND_IN_SET(" . $row_id . ", M.row_ids)) AND M.type_id = 0)"; did not work 
            $sql .= " AND 1 = (SELECT COUNT(*) FROM " . $this->db->dbprefix . "commission_level_exceptions WHERE !FIND_IN_SET(" . $row_id . ", row_ids) AND type_id = 0 AND module_id = M.module_id AND level_id = M.level_id)";            
            $sql .= " AND M.level_id = (SELECT level_id FROM " . $this->db->dbprefix . "affiliate_users WHERE user_id = " . $affiliate_id . " AND status_id = 2 AND deleted = 0)";
            $result = $this->db->query($sql)->row();
            $commission_amount = 0;
            if($result) {
                $commission_amount = $result->amount;
            }
            $this->save($commission_amount, $order, $order_item);            
        }
    }
    
    /**
     * Products/Bookings and other modules Commission calculations
     * Note: this is not affiliate specific
     * @param object $order_item
     * @return boolean
     */
    private function module_commission($order, $order_item) {
        $admin_user_id = 1;
        if(empty($order_item->commission)) {
           return false; 
        }
        $result = $this->check_commission($order, $order_item, 'user_id', $admin_user_id);
        if(!$result) {
            $this->save($order_item->commission, $order, $order_item, 'user_id', $order_item->commission_type, $admin_user_id);
        }
    }
    
    /**
     * Calculate and save coupon commission
     * @param object $order
     */
    private function coupon_commission($order, $order_item) {
        $this->load->model('coupons/coupons_model');
        
        // check if the order has any discount coupon applied or not
        $sql = "SELECT C.*, OC.user_id FROM " . $this->db->dbprefix . "coupons C";
        $sql .= " INNER JOIN " . $this->db->dbprefix. "order_coupons OC ON OC.coupon_id = C.id";
        $sql .= " WHERE OC.order_id = '" . $order->order_id . "'";
        $record = $this->db->query($sql)->row();
        $item_order = (array) $order_item;
        if(!empty($item_order['params']) && is_string($item_order['params'])) {
            $item_order['params'] = json_decode($item_order['params'], true);
        }
        
        if($record && !empty($order->affiliate_id) && $order->affiliate_id != $record->user_id && $this->coupons_model->item_discount($item_order, $record, false)) { // the user should not get commission on own referral
            $order_item->module_id = gc_module_id('coupons');
            $order_item->row_id = $record->id;
            $order_item->affiliate_id = $order->affiliate_id;
            $result = $this->check_commission($order, $order_item, 'affiliate_id');
            if(!$result) {
                $sql = "SELECT M.commission, M.type FROM " . $this->db->dbprefix . "commission_coupons M";
                $sql .= " WHERE M.coupon_id = " . $record->id . " AND (M.user_ids = 0 OR FIND_IN_SET(" . $order->affiliate_id . ", M.user_ids))";
                $row = $this->db->query($sql)->row();
                if($row) {
                    $order_item->commission = $row->commission;
                    $order_item->commission_type = $row->type;
                    
                    // get the affiliate coupon amount
                    if($row->type == 'F') { // fixed amount
                        $affiliate_coupon_amount = $row->commission;
                    } else {
                        // get commission from the order_subtotal
                        // do not include discount and tax in the commission
                        $affiliate_coupon_amount = $order->order_subtotal * $row->commission / 100;
                    }
                    
                    // get items which comes under this coupon
                    $coupon_discount_items = array();
                    $order_items = $this->db->where('order_id', $order->order_id)->get('order_items')->result();
                    foreach($order_items as $ordered_item) {
                        $ordered_item = (array) $ordered_item;
                        if(!empty($ordered_item['params']) && is_string($ordered_item['params'])) {
                            $ordered_item['params'] = json_decode($ordered_item['params'], true);
                        }
                        $affiliate_commission_applicable = $this->coupons_model->item_discount($ordered_item, $record, false);
                        if($affiliate_commission_applicable) {
                            $coupon_discount_items[$ordered_item['id']] = true;
                        }
                    }
                    
                    // get individual items affiliate commission
                    if(!empty($coupon_discount_items)) {
                        $per_item_coupon_discount = decimal_convert($row->commission / count($coupon_discount_items));
                        foreach($coupon_discount_items as $item_id => $has_discount) {
                            $coupon_discount_items[$item_id] = sprintf('%g', $per_item_coupon_discount); // remove the trailing zero digits just before to display the percentage
                        }
                    }
                    
                    // if the discount is applicable, save it in database
                    if(!empty($coupon_discount_items[$order_item->id])) {
                        $this->save($coupon_discount_items[$order_item->id], $order, $order_item, 'affiliate_id');
                    }
                }
            }
        }
    }
    
    /**
     * Check whether the commission is already processed or not for this order item
     * if exists, update it, no need for recalculation
     * @param object $order_item
     * @param string $user_column
     * @return boolean
     */
    private function check_commission($order, $order_item, $user_column = 'affiliate_id', $user_id = 0) {        
        $module_id = $order_item->module_id;
        $row_id = $order_item->row_id;
        $datetime = date('Y-m-d H:i:s');
        $ip_address = $this->input->ip_address();
        // check existence first in the commissions table
        $where = array('order_id' => $order_item->order_id, 'order_item_id' => $order_item->id, 'module_id' => $module_id, 'row_id' => $row_id);
        // products and bookings commissions are site admin specific
        $where['user_id'] = !empty($user_id) ? $user_id : $order->{$user_column};
        $commission = $this->db->where($where)->get($this->table_name)->row();
        if($commission) {
            // update passbook
            $this->gc->update_passbook($commission->user_id, 'commissions', $commission->id, $commission->amount, ($order->status_id == 2 ? 'credit' : 'debit'));
            // activate or deactivate the commission, no need to recalculate
            return $this->db->where('id', $commission->id)->update($this->table_name, array('status_id' => $order->status_id == 2 ? 2 : 1, 'modified_on' => $datetime, 'modified_ip' => $ip_address));
        }
        return false;
    }
    
    /**
     * Save commission information in the commissions table
     * @param float $commission_amount
     * @param object $order_item
     * @param string $user_column
     */
    private function save($commission_amount, $order, $order_item, $user_column = 'affiliate_id', $type = 'P', $user_id = 0) {
        if(!empty($commission_amount)) {
            $datetime = date('Y-m-d H:i:s');
            $ip_address = $this->input->ip_address();
            $affiliate_module_id = gc_module_id('affiliates');
            $commissions = array(
                $affiliate_module_id => array('commission' => $commission_amount, 'type' => $type),
                $order_item->module_id => array('commission' => $order_item->commission, 'type' => $order_item->commission_type),
            );
            if($type == 'F') { // fixed amount
                $amount = $commission_amount;
            } else {
                // get commission from the order_subtotal
                // do not include discount and tax in the commission
                $amount = $order_item->subtotal * $commission_amount / 100;
            }
            $subtotal = $order_item->subtotal;
            if($user_column == 'user_id') { // products/bookings commission
                // $site_commission = $amount;
                $subtotal = $order_item->subtotal - $amount;
            }
            $column_user_id = !empty($user_id) ? $user_id : $order->{$user_column};
            $user_info = $this->gc->get('users', array('where' => 'id = ' . $column_user_id, 'select' => 'display_name, email, phone_number'));
            // commissions table data
            $commission_data = array(
                'order_id' => $order_item->order_id,
                'order_item_id' => $order_item->id,
                'user_id' => $column_user_id, // bookings and products commissions
                'module_id' => $order_item->module_id,
                'row_id' => $order_item->row_id,
                'total' => $order_item->total, // for tier commissions calculations
                'subtotal' => $subtotal,
                'amount' => $amount,
                'status_id' => $order->status_id == 2 ? 2 : 1,
                'is_affiliate' => $user_column == 'affiliate_id' ? 1 : 0, // to find the affiliate commissions only
                'commissions' => json_encode($commissions),
                'created_on' => $datetime,
                'created_ip' => $ip_address,
                'summary' => json_encode($user_info)
            );
            $this->db->insert($this->table_name, $commission_data);

            // update passbook
            $insert_id = (int) $this->db->insert_id();
            $commission = $this->db->where('id', $insert_id)->get($this->table_name)->row();
            if($commission) {
                $this->gc->update_passbook($commission->user_id, 'commissions', $commission->id, $commission->amount, ($order->status_id == 2 ? 'credit' : 'debit'));
            }
        }
    }
    
    /**
     * Calculate Commissions
     * @param string $type - P/F
     * @param float $commission - Amount/Percentage value
     * @param float $amount - Amount on which calculations is to be calculated
     * @return number|float
     */
    private function calculate_commission($type, $commission, $amount = 0) {
        $commission_amount = 0;
        if($type == 'P') { // percent amount
            $commission_amount = decimal_convert(($amount * $commission) / 100);
        } else if($type == 'F') {
            $commission_amount = $commission;
        }
        return $commission_amount;
    }
    
    /**
     * Confirm Affiliates commissions only, do not change the status of commisions
     * A commission can contain multiple affiliates commissions
     */
    public function update_commission_affiliates($commission_id, $data) {        
        $this->db->where('commission_id', $commission_id)->update('commission_affiliates', $data);        
    }

    /**
     * Purge a commission completely from the application
     * @param string $module
     * @param array $ids
     */
    public function purge_commission($ids) {
        foreach($ids as $id) {
            // purge data from the commission_affiliates table first
            $this->db->where('commission_id', $id)->delete($this->table_affiliates);
            // purge data from the commissions table
            $this->db->where('id', $id)->delete($this->table_name);
        }
    }
    
    /**
     * Save Depth Level on Hierarchies Save
     * @param number $id
     * @param string $action_type
     * @param array $record
     */
    public function after_hierarchies_save($id, $action_type = 'insert', $record = array()) {
        if($action_type == 'insert') {
            $record = $this->db->where('id', $id)->get($this->table_hierarchies)->row();
            if($record) {
                $this->db->where('level_id', $record->level_id);
                $depth_count = $this->db->count_all_results($this->table_hierarchies);
                $this->db->where('id', $id)->update($this->table_hierarchies, array('depth' => $depth_count));
            }
        }
    }
}