<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Contact Queries Controller
 */
class Reports extends Admin_Controller
{
	protected $permissionCreate = 'Contact_Queries.Reports.Create';
	protected $permissionDelete = 'Contact_Queries.Reports.Delete';
	protected $permissionEdit   = 'Contact_Queries.Reports.Edit';
	protected $permissionView   = 'Contact_Queries.Reports.View';
	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct() {
		parent::__construct();

		$this->auth->restrict($this->permissionView);
		$this->load->model('contact_queries_model', null, true);
		$this->lang->load('contact_queries');
		
		//LOAD TinyMCE EDITOR
        Assets::add_js(array('tinymce/tinymce.min.js'));
        $this->form_validation->set_error_delimiters("<span class='error'>", "</span>");
        
		Template::set_block('sub_nav', 'reports/_sub_nav');

		//Assets::add_module_js('contact_queries', 'contact_queries.js');
	}

	//--------------------------------------------------------------------

	/**
	 * Display a list of Contact Queries data.
	 *
	 * @return void
	 */
	public function index($filter='all', $offset = 0) {
		// Assets::add_js('admin-reply-email.js');
		// Do we have any actions?
		if (isset($_POST['submit'])) {
		    $this->gcrud->stats(array('module' => 'contact_queries'));
			$action = $this->input->post('submit');
			$checked = $this->input->post('checked');
			if($checked) {
				$module_name = count($checked) == 1 ? lang('label_contact_query') : lang('label_contact_queries') ;
				switch(strtolower($action)) {
					case 'activate':
						$this->gc->update_status('contact_queries', $checked, 'status', 1);
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_updated')), 'success');
						break;
	
					case 'deactivate':
						$this->gc->update_status('contact_queries', $checked, 'status', 0);
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_updated')), 'success');
						break;
	
					case 'spam':
						$this->gc->update_status('contact_queries', $checked, 'status', 2);
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_spammed')), 'success');
						break;
	
					case 'delete':
						$this->gc->update_status('contact_queries', $checked, 'deleted', 1);
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_deleted')), 'success');
						break;
	
					case 'purge':
						$this->gc->purge_records('contact_queries', $checked);
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_deleted')), 'success');
						break;
	
					case 'restore':
						$this->gc->update_status('contact_queries', $checked, 'deleted', 0);
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_restored')), 'success');
						break;
				}
			} else {
				Template::set_message(lang('msg_no_row_selected'), 'danger');
			}
		}
		 
		// Change Status
		if (isset($_POST['change_status']))	 {
		    $row_id = $_POST['change_status'];
		    $new_status = (int) $_POST['query_status_' . $row_id];
		    Template::set_message(sprintf(lang('msg_action_success'), lang('label_contact_query'), lang('label_updated')), 'success');
		    $this->gc->update_status('contact_queries', array($_POST['change_status']), 'status', $new_status);
		}
		 
		// Actions done, now display the view
		$where = "deleted = 0";
		// Filters
		$filter_type = $filter;
		 
		switch($filter_type) {
			case 'replied':
				$where .= " AND CQ.status = 1";
				break;
	
			case 'not_replied':
				$where .= " AND CQ.status = 0";
				break;
				 
			case 'spammed':
				$where .= " AND CQ.status = 2";
				break;
				 
			case 'deleted':
				$where = "deleted = 1";
				break;
				 
			case 'all':
				// Nothing to do
				break;
				 
			default:
				show_404("contact_queries/index/$filter/");
		}
		
		$searchterm = $this->gc->search('contact_queries', $this->uri->segment(5), '/reports/contact_queries');
		if(!empty($searchterm)) {
			//$where .= " AND (C.id LIKE '%". $searchterm ."%' OR C.username LIKE '%". $searchterm ."%' OR C.title LIKE '%". $searchterm ."%' OR C.created_on LIKE '%". $searchterm ."%')";
		}
		
		$sort_by = "CQ.id DESC";
		$order_by = $this->gc->sort('contact_queries', 'CQ');
		if(!empty($order_by)) {
			$sort_by = $order_by;
		}
		 
		$records = $this->contact_queries_model->get_contact_queries($offset, $this->limit, $searchterm, $where, $sort_by);
		 
		$total_records = $this->contact_queries_model->get_contact_queries($offset, $this->limit, $searchterm, $where, $sort_by, 1);
		 
		$pagerUriSegment = 6;
		$pagerBaseUrl = site_url(SITE_AREA . '/reports/contact_queries/index/' . $filter) . '/';
	
		$limit  = $this->limit;
	
		$this->load->library('pagination');
		$pager['base_url']    = $pagerBaseUrl;
		$pager['total_rows']  = $total_records;
		$pager['per_page']    = $limit;
		$pager['uri_segment'] = $pagerUriSegment;
	
		$this->pagination->initialize($pager);
	
		Template::set('index_url', site_url(SITE_AREA .'/reports/contact_queries/index/'));
		Template::set('filter_type', $filter_type);
	
		Template::set('records', $records);
	
		Template::set('toolbar_title', lang('label_contact_queries'));
	
		Template::render();
	}
	
	/**
	 * Allows editing of Contact Queries data.
	 *
	 * @return void
	 */
	public function view() {
		Assets::add_js('admin-reply-email.js');
		$id = $this->uri->segment(5);
		if (empty($id)) {
			Template::set_message(lang('invalid_id'), 'danger');
	
			redirect(SITE_AREA . '/reports/contact_queries');
		}
	
		if (isset($_POST['save'])) {
		    $this->gcrud->stats(array('module' => 'contact_queries'));
			$this->auth->restrict($this->permissionEdit);
	
			if ($this->save_contact_queries_status($id)) {
				log_activity($this->auth->user_id(), lang('act_edit_record') . ': ' . $id . ' : ' . $this->input->ip_address(), 'contact_queries');
				Template::set_message(sprintf(lang('msg_save_success'), lang('label_contact_query')), 'success');
				redirect(SITE_AREA . '/reports/contact_queries');
			}
	
			// Not validation error
			if ( ! empty($this->contact_queries_model->error)) {
				Template::set_message(sprintf(lang('msg_edit_failure'), lang('label_contact_query')) . $this->contact_queries_model->error, 'danger');
			}
		} elseif (isset($_POST['delete'])) {
			$this->auth->restrict($this->permissionDelete);
	
			if ($this->contact_queries_model->delete($id)) {
				log_activity($this->auth->user_id(), lang('act_delete_record') . ': ' . $id . ' : ' . $this->input->ip_address(), 'contact_queries');
				Template::set_message(sprintf(lang('msg_delete_success'), lang('label_contact_query')), 'success');
	
				redirect(SITE_AREA . '/reports/contact_queries');
			}
	
			Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_contact_query')) . $this->contact_queries_model->error, 'danger');
		}
	
		$record = $this->contact_queries_model->get_contact_queries_view($id);
		Template::set('contact_queries', $record);
	
		Template::set('toolbar_title', lang('label_view') . ' ' .$record->name . ' ' . lang('label_contact_query'));
		Template::set_view('contact_queries_form');
		Template::render();
	}
	
	/**
	 * Allows editing of Contact Queries data.
	 *
	 * @return void
	 */
	public function edit() {
		$id = $this->uri->segment(5);

		if (empty($id)) {
			Template::set_message(lang('contact_queries_invalid_id'), 'danger');
			redirect(SITE_AREA .'/reports/contact_queries');
		}

		if (isset($_POST['save'])) {
		    $this->gcrud->stats(array('module' => 'contact_queries'));
			$this->auth->restrict($this->permissionEdit);

			if ($this->save_contact_queries('update', $id)) {
				// Log the activity
				log_activity($this->auth->user_id(), lang('act_edit_record') .': '. $id .' : '. $this->input->ip_address(), 'contact_queries');

				Template::set_message(sprintf(lang('msg_save_success'), lang('label_contact_query')), 'success');
			} else {
				Template::set_message(sprintf(lang('msg_edit_failure'), lang('label_contact_query')) . $this->contact_queries_model->error, 'danger');
			}
		} else if (isset($_POST['delete'])) {
			$this->auth->restrict($this->permissionDelete);
			if ($this->contact_queries_model->delete($id)) {
				// Log the activity
				log_activity($this->auth->user_id(), lang('contact_queries_act_delete_record') .': '. $id .' : '. $this->input->ip_address(), 'contact_queries');

				Template::set_message(sprintf(lang('msg_delete_success'), lang('label_contact_query')), 'success');

				redirect(SITE_AREA .'/reports/contact_queries');
			} else {
				Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_contact_query')) . $this->contact_queries_model->error, 'danger');
			}
		}
		Template::set('contact_queries', $this->contact_queries_model->find($id));
		Template::set('toolbar_title', lang('edit') .' Contact Queries');
		Template::render();
	}

	//--------------------------------------------------------------------
	// !PRIVATE METHODS
	//--------------------------------------------------------------------
	
	/**
	 * Save Status.
	 */
	private function save_contact_queries_status($id) {
		$data = array();
		$data['status'] = $this->input->post('status');
		return $this->contact_queries_model->update($id, $data);
	}

	/**
	 * Summary
	 *
	 * @param String $type Either "insert" or "update"
	 * @param Int	 $id	The ID of the record to update, ignored on inserts
	 *
	 * @return Mixed    An INT id for successful inserts, TRUE for successful updates, else FALSE
	 */
	private function save_contact_queries($type='insert', $id=0) {
		if ($type == 'update') {
			$_POST['id'] = $id;
		}

		$data = array();
		$data['user_id']        = $this->input->post('contact_queries_user_id');
		$data['name']        = $this->input->post('contact_queries_name');
		$data['email']        = $this->input->post('contact_queries_email');
		$data['phone_number']        = $this->input->post('contact_queries_phone');
		$data['message']        = $this->input->post('contact_queries_message');
		$data['ip_address']        = $this->input->post('contact_queries_ip');
		$data['created_on']        = $this->input->post('contact_queries_posted_on') ? $this->input->post('contact_queries_posted_on') : NULL;

		if ($type == 'insert') {
			$id = $this->contact_queries_model->insert($data);

			if (is_numeric($id)) {
				$return = $id;
			} else {
				$return = FALSE;
			}
		}
		elseif ($type == 'update') {
			$return = $this->contact_queries_model->update($id, $data);
		}
		return $return;
	}
    
    /**
     * Manage Contact Queries Email Templates
     * @param string $module
     * @param string $filter
     * @param number $offset
     */
    public function manage($module = 'email_templates', $filter = 'all', $offset = 0) {
        $extra = array(
            'model' => 'contact_queries',
			'context' => 'reports',
			'module_name' => 'contact_queries',
            'fields' => $module,
            'prefix' => 'contact_queries'
        );
        $this->gc->module_view($module, $filter, $offset, 'admin', $extra);
    }
    
    /**
     * Add/Edit Email Templates Data
     *
     * @return void
     */
    public function form($module = 'email_templates', $action = 'create') {
        Assets::add_js(array(
            'tinymce/tinymce.min.js',
        ));
        $params = array(
			'selectize' => true,
			'model' => 'contact_queries',
			'module_name' => 'contact_queries',
			'context' => 'reports',
			'fields_method' => $module,
			'prefix' => 'contact_queries'
		);
		$this->gcrud->form($module, $action, 'admin', $params);
    }
}