<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Financial controller
 */
class Financial extends Admin_Controller
{
	protected $permissionCreate = 'Coupons.Financial.Create';
	protected $permissionDelete = 'Coupons.Financial.Delete';
	protected $permissionEdit   = 'Coupons.Financial.Edit';
	protected $permissionView   = 'Coupons.Financial.View';
	
	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct() {
		parent::__construct();

		$this->auth->restrict($this->permissionView);
		$this->lang->load('coupons');
		$this->load->model('coupons_model', null, true);
		Template::set_block('sub_nav', 'financial/_sub_nav');
	}

	//--------------------------------------------------------------------

	/**
	 * Manage Coupons
	 * @param string $filter
	 * @param number $offset
	 */
	public function index($filter = 'all', $offset = 0) {
	    $this->manage('coupons', $filter = 'all', $offset = 0);
	}
	
	/**
	 * Manage Coupons
	 * @param string $module
	 * @param string $filter
	 * @param number $offset
	 */
	public function manage($module = 'coupons', $filter = 'all', $offset = 0) {
	    /*if($module == 'coupons' && !empty($records)) {
	        // Set coupon to expired if date has gone
            foreach($records as $key => $coupon) {
                if($coupon->active == 1 && $coupon->valid_upto <= date('Y-m-d H:i:s')) {
                    $sql = "UPDATE " . $this->db->dbprefix . "coupons SET active = NOT active WHERE id = " .$coupon->id;
                    $this->db->simple_query($sql);
                    $records[$key]->active = 0;
                }
            }
	    }*/
	   $extra = array(
	       'model' => 'coupons',
	       'context' => 'financial',
	       'fields' => null
	   );
	   $extra['actions'] = array( // Extra buttons
	       array('type' => 'link', 'link' => site_url(SITE_AREA . '/financial/coupons/form/' . $module . '/edit/%s'), 'label' => '<i class="fas fa-edit fa-fw mr-1"></i>' . lang('edit'), 'class' => 'btn btn-xs btn-primary'),
	   );
	   if($module == 'coupons' && $this->auth->role_id() != 1) {
	       $extra['query_where'] = 'M.seller_id = ' . $this->auth->user_id();
	   }
	   $this->gc->module_view($module, $filter, $offset, 'admin', $extra);
	}
	
	/**
	 * Add/Edit Coupons Data
	 *
	 * @param string $module
	 * @param string $action
	 *
	 * @return void
	 */
	public function form($module = 'coupons', $action = 'create') {
	    $params = array(
	        'model' => $module,
	        'context' => 'financial',
	        'redirect_url' => site_url(SITE_AREA . '/financial/coupons/manage/' . $module),
	        'layout' => 'tabs'
	    );	    
	    if($module == 'coupons') {
    	    Template::set('layout', 'tabs');
    	    $params['group_layout'] = true;
    	    if($action == 'edit' && $this->auth->role_id() != 1) {
    	        $params['query_where'] = $this->auth->user_id();
    	    }
    	    if($module == 'coupons') {
    	        $params['method'] = 'after_coupon_save';
    	    }
	    } else {
	        $params['model'] = 'coupons';
	        $params['fields_method'] = 'coupons';
	    }
	    $this->gcrud->form($module, $action, 'admin', $params);
	}
	
	/**
	 * Select Affiliates
	 * @param string $isEdit
	 * @param array $users_array
	 * @param string $field_name
	 */
	public function users_selection($isEdit = false, $users_array = array(), $field_name = 'affiliate_users') {
		$selected_users = array();
		$users = $this->db->select("id, CONCAT(display_name, ' (' , email , ')') AS email")->get('users')->result_array();
		$all_users[] = array(
				'id' => 'all',
				'email' => lang('all_users')
		);
		Template::set('users', array_merge($all_users, $users));
		if($this->input->post($field_name)) { // Post handling
			Template::set($field_name, $this->selected_users($this->input->post($field_name)));
		} else if(empty($_POST) && $isEdit) {
			Template::set($field_name, $this->selected_users($users_array));
		} else if($field_name == 'applicable_users') { // set default value to ALL USERS
			Template::set($field_name, $this->selected_users('all'));
		}
	}
	
	/**
	 * Users selected via POSTS/EDIT Page
	 * @param array $users
	 * @return string
	 */
	private function selected_users($users) {
		$selected_users = array();
		$users_array = explode(',',$users ?? '');
		// User selected all users
		if(in_array('all', $users_array)) {
			$selected_users  = array(
					'id' => 'all',
					'name' => lang('all_users')
			);
		} else if(!empty($users)) { // User selected only few users
			$sql = "SELECT id, CONCAT(display_name, ' (' , email , ')') AS email FROM " . $this->db->dbprefix . "users where id in (" . $users . ")";
			$result = $this->db->query($sql)->result_array();
			if($result) {
				$selected_users = $result;
			}
		}
		return $selected_users;
	}
	
	/**
	 * Deal with Users selection search boxes
	 * @param string $type
	 * @param object $record
	 */
	private function selectize_users($type = 'affiliate_users', $record = array()) {
		$user_id = $this->input->post($type) ? $this->input->post($type) : (!empty($record->{$type}) ? $record->{$type} : array());
		$user_id = (int) $user_id;
		if(!empty($user_id)) {
			$selectize_value = $this->db->select('id,email AS name')->where('id IN (' . $user_id . ')')->get('users')->result();
			Template::set($type, $selectize_value);
		}
	}
	
	/**
	 * Store affiliates information in affiliate_coupons table
	 * @param int $coupon_id
	 * @param string $type
	 */
	private function deal_with_affiliates($coupon_id, $type = 'insert') {
	    $affiliate_commission = $this->input->post('affiliate_commission');
	    if(!empty($affiliate_commission) && $this->input->post('commission_type')) {
			$this->load->model('payouts/payouts_model');
			$data = array();
			$module_id = gc_module_id('affiliates');
			$affiliates_all = $this->input->post('affiliate_users');
			$payout_users = array(); // Also add payout_users to keep record of all payments
			// User selected ALL USERS
			if(empty($affiliates_all)) {
				$users = $this->db->select('id')->get('users')->result();
				if($users) {
					foreach($users as $user) {
						$data[] = array(
								'user_id' => $user->id,
								'coupon_id' => $coupon_id,
								'commission' => $this->input->post('affiliate_commission') ? $this->input->post('affiliate_commission') : 0,
								'type' => $this->input->post('commission_type'),
								'status' => 1,
								'created_on' => date("Y-m-d H:i:s"),
								'created_by' => $this->auth->user_id(),
								'created_ip' => $this->input->ip_address(),
								'modified_on' => date("Y-m-d H:i:s"),
								'modified_by' => $this->auth->user_id(),
								'modified_ip' => $this->input->ip_address()
						);
	
						$payout_users[] = array(
								'user_id' => $user->id,
						        'module_id' => $module_id,
								'created_on' => date("Y-m-d H:i:s"),
								'created_by' => $this->auth->user_id(),
								'created_ip' => $this->input->ip_address(),
								'modified_on' => date("Y-m-d H:i:s"),
								'modified_by' => $this->auth->user_id(),
								'modified_ip' => $this->input->ip_address()
						);
					}
				}
			} else {
				// add affiliate coupon only for the selected users
				$usersArray = explode(',',$this->input->post('affiliate_users') ?? '');
				foreach($usersArray as $user) {
					$data[] = array(
							'user_id' => $user,
							'coupon_id' => $coupon_id,
							'commission' => $this->input->post('affiliate_commission') ? $this->input->post('affiliate_commission') : 0,
							'type' => $this->input->post('commission_type'),
							'status' => 1,
							'created_on' => date("Y-m-d H:i:s"),
							'created_by' => $this->auth->user_id(),
							'created_ip' => $this->input->ip_address(),
							'modified_on' => date("Y-m-d H:i:s"),
							'modified_by' => $this->auth->user_id(),
							'modified_ip' => $this->input->ip_address()
					);
					$payout_users[] = array(
							'user_id' => $user,
					        'module_id' => $module_id,
							'created_on' => date("Y-m-d H:i:s"),
							'created_by' => $this->auth->user_id(),
							'created_ip' => $this->input->ip_address(),
							'modified_on' => date("Y-m-d H:i:s"),
							'modified_by' => $this->auth->user_id(),
							'modified_ip' => $this->input->ip_address()
					);
				}
			}
				
			/**
			 * ON_DUPLICATE method is created in the GOCLIXY/CI3/DATABASE/DB_query_builder.php FILE
			 * FOR INSERT ON DUPLICATE UPDATE - Refer to Documentation for more information
			 */
			if($type == 'insert') {
				$this->db->on_duplicate('affiliate_coupons', $data);
			} else {
				// deactivate all existing coupons
				// if a user is removed then it will be deactivated from the list
				$this->db->where('coupon_id', $coupon_id);
				$this->db->update('affiliate_coupons', array('status' => 0));
				// INSERT ON DUPLICATE UPDATE
				$this->db->on_duplicate('affiliate_coupons', $data);
			}
				
			// Also add payout_users to keep records of all payments
			$this->db->on_duplicate('payout_users', $payout_users);
		}
	}
	
	/**
	 * Change status of category from index page
	 * @param int $id
	 */
	public function update_status($id, $offset = 0)
	{
		$this->auth->restrict($this->permissionEdit);
		$sql = "UPDATE " . $this->db->dbprefix . "coupons SET active = NOT active WHERE id = " .$id;
		$this->db->simple_query($sql);
		Template::set_message(sprintf(lang('msg_action_success'), lang('label_coupon'), lang('label_updated')), 'success');
		if($offset != 0){
			redirect(site_url(SITE_AREA .'/financial/coupons/index/'.$offset));
		} else {
			redirect(site_url(SITE_AREA .'/financial/coupons/'));
		}
	}
	
	/**
	 * Track Coupon Usage with Commissions
	 * @param string $coupon_code
	 * @param number $affiliate_id
	 * @param number $offset
	 */
	public function usage($coupon_code, $affiliate_id = 0, $offset = 0) {
	    $this->coupons_model->usage('admin', $coupon_code, $affiliate_id, $offset, $this->limit);
	}
}