<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Migration_Install_coupons extends Migration
{
	/**
	 * The name of the database table
	 *
	 * @var String
	 */
	private $table_name = 'coupons';

	/**
	 * The table's fields
	 *
	 * @var Array
	 */
	private $fields = array(
		'id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'auto_increment' => TRUE,
		),
		'name' => array(
			'type' => 'VARCHAR',
			'constraint' => 150,
			'null' => FALSE,
		),
		'code' => array(
			'type' => 'VARCHAR',
			'constraint' => 150,
			'null' => FALSE,
		),
		'value' => array(
			'type' => 'DECIMAL',
			'constraint' => '12,2',
			'null' => FALSE,
		),
		'type' => array(
			'type' => 'CHAR',
			'constraint' => 1,
			'null' => FALSE,
		),
        'max_discount' => array(
			'type' => 'DECIMAL',
			'constraint' => '12,2',
			'null' => FALSE,
			'default'=> 0.00,
		),
		 'valid_from' => array(
			'type' => 'DATETIME',
			'null' => TRUE,
		),
		'valid_upto' => array(
			'type' => 'DATETIME',
			'null' => TRUE,
		),
		'max_coupon_uses' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => FALSE,
			'default'=> 0,
		),
		'max_customer_uses' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => FALSE,
			'default'=> 0,
		),
		'applicable_packages' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => FALSE,
			'default'=> 0,
		),
		'applicable_users' => array(
			'type' => 'TEXT',
			'null' => TRUE
		),
		'min_order' => array(
			'type' => 'DECIMAL',
			'constraint' => '12,2',
			'null' => FALSE,
			'default'=> 0.00,
		),
		'counts' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => FALSE,
			'default'=> 0,
		),
		'hits' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => FALSE,
			'default'=> 0,
		),
	    'invitation_user_id' => array(
	        'type' => 'BIGINT',
	        'constraint' => 20,
	        'null' => TRUE
	    ),
	    'invitation_commission' => array(
	        'type' => 'FLOAT',
	        'null' => FALSE,
	        'default'=> 0
	    ),
	    'invitation_commission_type' => array(
	        'type' => 'CHAR',
	        'constraint' => 1,
	        'null' => TRUE
	    ),
	    'affiliates_all' => array(
	        'type' => 'TINYINT',
	        'constraint' => 1,
	        'null' => FALSE,
	        'default' => 0
	    ),
		'affiliates' => array(
			'type' => 'TEXT',
			'null' => TRUE
		),
		'affiliate_commission' => array(
			'type' => 'FLOAT',
			'null' => FALSE,
			'default'=> 0
		),
		'commission_type' => array(
			'type' => 'CHAR',
			'constraint' => 1,
			'null' => TRUE
		),
		'active' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default'=> 0,
		),
		'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'created_ip' => array(
			'type' => 'VARCHAR',
			'constraint' => 50,
			'null' => TRUE
		),
		'modified_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'modified_ip' => array(
			'type' => 'VARCHAR',
			'constraint' => 50,
			'null' => TRUE
		),
	);

	/**
	 * Install this migration
	 *
	 * @return void
	 */
	public function up() {
		$this->dbforge->add_field($this->fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->create_table($this->table_name);
	}

	//--------------------------------------------------------------------

	/**
	 * Uninstall this migration
	 *
	 * @return void
	 */
	public function down() {
		$this->dbforge->drop_table($this->table_name);
	}

	//--------------------------------------------------------------------
}