<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class Coupons_model extends BF_Model {

	public $table_name	= "coupons";
	protected $key			= "id";
	protected $soft_deletes	= false;
	protected $date_format	= "datetime";

	protected $log_user 	= FALSE;

	protected $set_created	= true;
	protected $set_modified = true;
	protected $created_field = "created_on";
	protected $modified_field = "modified_on";

	/*
		Customize the operations of the model without recreating the insert, update,
		etc methods by adding the method names to act as callbacks here.
	 */
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 		= array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	/*
		For performance reasons, you may require your model to NOT return the
		id of the last inserted row as it is a bit of a slow method. This is
		primarily helpful when running big loops over data.
	 */
	protected $return_insert_id 	= TRUE;

	// The default type of element data is returned as.
	protected $return_type 			= "object";

	// Items that are always removed from data arrays prior to
	// any inserts or updates.
	protected $protected_attributes = array();

	/*
		You may need to move certain rules (like required) into the
		$insert_validation_rules array and out of the standard validation array.
		That way it is only required during inserts, not updates which may only
		be updating a portion of the data.
	 */
	protected $validation_rules 		= array();
	protected $insert_validation_rules 	= array();
	protected $skip_validation 			= FALSE;
	
	/**
	 * News Fields
	 *
	 * @return array
	 */
	public function fields() {
	    $applicable_modules = array('packages', 'ads', 'leads', 'claims', 'pyg', 'products', 'wallets');
	    foreach($applicable_modules as $module) {
	        $modules[] = 'name:' . gc_module_id($module) . ';label:' . lang($module) . ';value:' . gc_module_id($module);
	    }
	    $modules = implode('|', $modules);
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true,
	            'field_group' => 2
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'name',
	            'label' => 'label_name',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'edit' => 1,
	            'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
	            'class' => 4,
	            'rules' => 'trim|required|max_length[150]',
	            'field_group' => 2
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'code',
	            'label' => 'label_code',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'class' => 2,
	            'rules' => 'trim|required|alpha_numeric_only|max_length[150]',
	            'field_group' => 2
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'value',
	            'label' => 'label_value',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'class' => 2,
	            'value' => 0,
	            'rules' => 'trim|is_decimal|required',
	            'tooltip' => lang('help_value'),
	            'field_group' => 2
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'max_discount',
	            'label' => 'label_max_discount',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'class' => 2,
	            'value' => 0,
	            'rules' => 'trim|is_decimal|required',
	            'field_group' => 2
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'min_order',
	            'label' => 'label_min_order',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'class' => 2,
	            'value' => 0,
	            'rules' => 'trim|is_decimal|required',
	            'field_group' => 2
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'max_coupon_uses',
	            'label' => 'label_max_coupon_uses',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'class' => 2,
	            'value' => 0,
	            'rules' => 'trim|is_numeric|required',
	            'field_group' => 2
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'max_customer_uses',
	            'label' => 'label_uses_per_customer',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'class' => 2,
	            'value' => 0,
	            'rules' => 'trim|is_numeric|required',
	            'field_group' => 2
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'valid_from',
	            'label' => 'label_valid_from',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'class' => 3,
	            'rules' => 'trim|valid_datetime',
	            'field_group' => 2
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'valid_upto',
	            'label' => 'label_valid_upto',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'class' => 3,
	            'rules' => 'trim|valid_datetime',
	            'field_group' => 2
	        ),
	        array( // affiliates information
	            'type' => 'textarea',
	            'name' => 'description',
	            'label' => 'label_description',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'class' => 6,
	            'rules' => 'trim',
	            'field_group' => 2
	        ),
	        array( // affiliates visitors information
	            'type' => 'textarea',
	            'name' => 'description_visitors',
	            'label' => 'label_description_visitors',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'class' => 6,
	            'rules' => 'trim',
	            'field_group' => 2
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'applicable_modules',
	            'label' => 'applicable_modules',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'options' => $modules,
	            'options_lang' => 0,
	            'options_detailed' => 1,
	            'placeholder' => 'applicable_modules',
	            'rules' => 'trim|is_array|max_length[455]',
	            'field_group' => 6
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'counts',
	            'label' => 'label_counts',
	            'frontend' => false,
	            'admin_only' => false,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'value' => 0,
	            'class' => 2,
	            'rules' => 'trim|is_numeric|required',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'hits',
	            'label' => 'label_hits',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'value' => 0,
	            'class' => 2,
	            'rules' => 'trim|is_numeric|required',
	            'field_group' => 2
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'status',
	            'label' => 'label_status',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	            'field_group' => 2
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_on',
	            'label' => 'created_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	            'field_group' => 2
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_ip',
	            'label' => 'created_ip',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	            'field_group' => 2
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_on',
	            'label' => 'modified_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	            'field_group' => 2
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_ip',
	            'label' => 'modified_ip',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	            'field_group' => 2
	        )	        
	    );
	    $applicable_modules[] = 'users';
	    foreach($applicable_modules as $applicable_module) {
	        switch($applicable_module) {
	            case 'packages':
	            case 'ads':
	            case 'products':
	            case 'users':
	                $load_table = $applicable_module == 'ads' ? 'ad_packages' : $applicable_module;
	                $fields[] = array(
    	                'type' => 'selectize',
    	                'name' => $applicable_module . '_include',
    	                'label' => $applicable_module . '_include',
    	                'frontend' => true,
    	                'admin_only' => true,
    	                'frontend_index' => false,
    	                'admin_index' => false,
	                    'options' => 'load:' .  $load_table . ';module:' . $load_table . ';max_items:1000',
    	                'options_detailed' => 1,
    	                'rules' => 'trim',
    	                'field_group' => 6
	                );
	                $fields[] = array(
	                    'type' => 'selectize',
	                    'name' => $applicable_module . '_exclude',
	                    'label' => $applicable_module . '_exclude',
	                    'frontend' => true,
	                    'admin_only' => true,
	                    'frontend_index' => false,
	                    'admin_index' => false,
	                    'options' => 'load:' .  $load_table . ';module:' . $load_table . ';max_items:1000',
	                    'options_detailed' => 1,
	                    'rules' => 'trim',
	                    'field_group' => 6
	                );
	                break;
	        }
	    }
	    // invitations group
	    /*$fields[] = array(
	        'type' => 'selectize',
	        'name' => 'invitation_user_id',
	        'label' => 'bf_user',
	        'frontend' => true,
	        'admin_only' => true,
	        'frontend_index' => false,
	        'admin_index' => false,
	        'options' => 'load:users;module:users;max_items:1',
	        'options_detailed' => 1,
	        'rules' => 'trim|is_numeric',
	        'field_group' => 18
	    );
	    $fields[] = array(
	        'type' => 'text',
	        'name' => 'invitation_commission',
	        'label' => 'label_commission',
	        'frontend' => true,
	        'admin_only' => true,
	        'frontend_index' => false,
	        'admin_index' => false,
	        'tooltip' => lang('help_commission'),
	        'class' => 3,
	        'rules' => 'trim',
	        'field_group' => 18
	    );
	    $fields[] = array(
	        'type' => 'dropdown',
	        'name' => 'invitation_commission_type',
	        'label' => 'label_type',
	        'frontend' => true,
	        'admin_only' => true,
	        'frontend_index' => false,
	        'admin_index' => false,
	        'help' => 'help_commission',
	        'class' => 2,
	        'options' => 'name:label_fixed;value:F|name:label_percentage;value:P',
	        'options_detailed' => 1,
	        'options_lang' => 1,
	        'rules' => 'trim',
	        'field_group' => 18
	    );
	    
	    // affiliates group
	    $fields[] = array(
	        'type' => 'selectize',
	        'name' => 'affiliate_user_ids',
	        'label' => 'select_affiliates',
	        'tooltip' => lang('help_affiliates'),
	        'frontend' => true,
	        'admin_only' => true,
	        'frontend_index' => false,
	        'admin_index' => false,
	        'options' => 'load:users;module:users;max_items:100',
	        'options_detailed' => 1,
	        'rules' => 'trim',
	        'field_group' => 19
	    );
	    $fields[] = array(
	        'type' => 'text',
	        'name' => 'affiliate_commission',
	        'label' => 'label_commission',
	        'tooltip' => lang('help_affiliate_commission'),
	        'frontend' => true,
	        'admin_only' => true,
	        'frontend_index' => false,
	        'admin_index' => false,
	        'class' => 3,
	        'rules' => 'trim',
	        'field_group' => 19
	    );
	    $fields[] = array(
	        'type' => 'dropdown',
	        'name' => 'affiliate_commission_type',
	        'label' => 'label_type',
	        'frontend' => true,
	        'admin_only' => true,
	        'frontend_index' => false,
	        'admin_index' => false,
	        'help' => 'help_commission',
	        'class' => 2,
	        'options' => 'name:label_fixed;value:F|name:label_percentage;value:P',
	        'options_detailed' => 1,
	        'options_lang' => 1,
	        'rules' => 'trim',
	        'field_group' => 19
	    );*/
	    return $fields;
	}
	
	
	/**
	 * Listing Coupons Fields
	 *
	 * @return array
	 */
	public function coupons() {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'selectize',
	            'name' => 'seller_id',
	            'label' => 'us_seller',
	            'frontend' => false,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => true,
	            'options' => 'load:business_users;module:business_users;max_items:1',
	            'options_detailed' => 1,
	            'rules' => 'trim|required',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'name',
	            'label' => 'label_name',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'edit' => 1,
	            'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
	            'class' => 4,
	            'rules' => 'trim|max_length[150]',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'code',
	            'label' => 'label_code',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'class' => 4,
	            'rules' => 'trim|max_length[150]',
	        ),/*
	        array(
	        'type' => 'text',
	        'name' => 'type',
	        'label' => 'label_type',
	        'frontend' => true,
	        'admin_only' => true,
	        'frontend_index' => true,
	        'admin_index' => true,
	        'rules' => 'trim',
	        ),*/
	        array(
	            'type' => 'text',
	            'name' => 'value',
	            'label' => 'label_value',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'class' => 2,
	            'rules' => 'trim|required|is_decimal',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'max_discount',
	            'label' => 'label_max_discount',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'class' => 2,
	            'rules' => 'trim|is_decimal',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'min_order',
	            'label' => 'label_min_order',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'class' => 2,
	            'rules' => 'trim|is_decimal',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'valid_from',
	            'label' => 'label_valid_from',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'class' => 3,
	            'rules' => 'trim|valid_datetime',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'valid_upto',
	            'label' => 'label_valid_upto',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'class' => 3,
	            'rules' => 'trim|valid_datetime',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'max_coupon_uses',
	            'label' => 'label_max_coupon_uses',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'default' => 0,
	            'class' => 2,
	            'rules' => 'trim|is_numeric',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'max_customer_uses',
	            'label' => 'label_uses_per_customer',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'default' => 0,
	            'class' => 2,
	            'rules' => 'trim|is_numeric',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'products_include',
	            'label' => 'products_include',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'products_exclude',
	            'label' => 'products_exclude',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'counts',
	            'label' => 'label_counts',
	            'frontend' => false,
	            'admin_only' => false,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'value' => 0,
	            'rules' => 'trim|is_numeric',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'hits',
	            'label' => 'label_hits',
	            'frontend' => false,
	            'admin_only' => false,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'value' => 0,
	            'rules' => 'trim|is_numeric',
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'status',
	            'label' => 'label_status',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_on',
	            'label' => 'created_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_ip',
	            'label' => 'created_ip',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_on',
	            'label' => 'modified_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_ip',
	            'label' => 'modified_ip',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        
	    );
	    return $fields;
	}
	
	/**
	 * Get Coupon Code = Apply Coupon during checkout 
	 * @param string $code
	 * @param int $module_id
	 * @return array|object
	 */
	public function get_coupon($code, $toggle_module_id, $table = 'coupons') {
	    $sql = "SELECT C.* FROM " . $this->db->dbprefix . $table . " C";
		// $sql .= " INNER JOIN " . $this->db->dbprefix . "module_relations MR ON MR.row_id = C.id AND MR.module_id = " . gc_module_id('coupons') . " AND MR.toggle_module_id = $toggle_module_id"; 
		$sql .= " WHERE C.code = '" . $code ."' AND C.status = 1";		
		$result = $this->db->query ( $sql )->row();
		return $result;
	}

	/**
	 * Check whether coupon is valid or not
	 * @param string $code
	 * @return number
	 */
	public function valid_coupon_count($code, $toggle_module_id, $table = 'coupons') {
		$today = date("Y-m-d h:m:s");
		$order_coupons_table = 'order_coupons';
		$sql = "SELECT count(id) AS count FROM " . $this->db->dbprefix . $table . " C";
		// $sql .= " INNER JOIN " . $this->db->dbprefix . "module_relations MR ON MR.row_id = C.id AND MR.module_id = " . gc_module_id('coupons') . " AND MR.toggle_module_id = $toggle_module_id"; 
		$sql .= " WHERE C.code = '" . $code ."' AND C.status = 1";
		$sql .= " AND ((C.valid_from <= '". $today ."') || (C.valid_from = '0000-00-00' OR C.valid_from IS NULL))";
		$sql .= " AND ((C.valid_upto >= '". $today ."') || (C.valid_upto = '0000-00-00' OR C.valid_upto IS NULL))";
		$sql .= " AND ((C.max_coupon_uses > (SELECT COUNT(id) FROM " . $this->db->dbprefix . $order_coupons_table . " WHERE coupon_code = '". $code ."') OR (C.max_coupon_uses = 0)))";
		$result = $this->db->query ( $sql )->row();
		return $result ? (int)$result->count : 0;
	}
	
	/**
	 * Check if the user has already has applied that coupon
	 * @param string $code
	 * @param string $table - order_coupons
	 */
	public function user_limit_count($code, $user_id, $table = 'order_coupons') {
		$sql = "SELECT count(id) AS count FROM " . $this->db->dbprefix . "order_coupons";
		$sql .= " WHERE coupon_code = '" . $code ."' AND user_id = " . $user_id;
		$result = $this->db->query ( $sql )->row();
		return $result ? (int)$result->count : 0;
	}
	
	/**
	 * Purge a coupon completely from the coupons table
	 * @param int $pid - ID of the coupon to be purged
	 */
	public function purge_coupon($id) {
		// delete affiliate coupons first, before coupon delete
		$this->db->where('coupon_id', $id)->delete('affiliate_coupons');
		// Purge all relations from the module_relations table
		$this->gc->purge_relation($id, 'coupons');
		$result = $this->coupons_model->delete($id);
		return $result;
	}

	/**
	 * Coupon Usage
	 * @param string $role
	 * @param string $coupon_code - Coupon Code
	 * @param number $affiliate_id - Affiliate to be tracked
	 * @param number $offset
	 * @param number $limit
	 */
	public function usage($role = 'admin', $coupon_code = null, $affiliate_id = 0, $offset = 0, $limit = 25) {
	    if(empty($coupon_code) && empty($affiliate_id)) {
	        Template::redirect(site_url());
	    }
	    $base_url = current_url();
	    if($role == 'admin') {
	        $redirect_url = site_url(SITE_AREA ."/financial/coupons");
	        $render = '';
	    } else {
	        $redirect_url = gc_url_format('members');
	        $render = 'member';
	    }
	    // get usage from the order_coupons table
	    // Sort By
	    $order_by = 'O.id DESC';
	    $sort_by = $this->gc->sort('coupon_usage');
	    if(!empty($sort_by)) {
	        $order_by = $sort_by;
	    }
	    $records = $this->coupons_model->get_usage($coupon_code, $affiliate_id, 2, 0, $order_by, $offset, $limit);
	    // Pagination
	    $this->load->library('pagination');
	    $total_rows            = $this->coupons_model->get_usage($coupon_code, $affiliate_id, 2, 1, $order_by, $offset, $limit);
	    $pager['base_url']     = $base_url;
	    $pager['total_rows']   = $total_rows;
	    $pager['per_page']     = $limit;
	    $pager['uri_segment']  = 7;
	    $this->pagination->initialize($pager);
	    $toolbar_title = lang('coupon_usage');
	    if(!empty($coupon_code)) {
	        $toolbar_title .= ' - ' . $coupon_code;
	    } else {
	        $toolbar_title = lang('affiliate_commissions') . ' - ' . lang('label_id') . ': ' . $affiliate_id;
	    }
	    Template::set('records', $records);
	    Template::set('toolbar_title', $toolbar_title);
	    Template::set_view('coupons/usage');
	    Template::render($render);
	}
	
	/**
	 * Query to get a specific coupon usage
	 *
	 * @param string $coupon_code
	 * @param number $affiliate_id
	 * @param number $status
	 * @param number $count
	 * @param number $offset
	 * @param number $limit
	 * @return object|number
	 */
	private function get_usage($coupon_code, $affiliate_id, $status_id = 2, $count = 0, $order_by = 'O.id DESC', $offset = 0, $limit = 25) {
	    if($count == 0) {
	        $sql = "SELECT O.id, O.order_id, O.user_id, U.display_name, OC.coupon_id, OC.coupon_code, OC.amount, U.email AS user_email,";
	        $sql .= " OC.affiliate_id, OC.affiliate_commission, O.created_on, AU.display_name AS affiliate_name, AU.email AS affiliate_email";
	    } else {
	        $sql = "SELECT COUNT(O.id) AS total_count";
	    }
	    $sql .= " FROM " . $this->db->dbprefix . "orders O";
	    $sql .= " INNER JOIN " . $this->db->dbprefix . "order_coupons OC ON OC.order_id = O.order_id";
	    $sql .= " INNER JOIN " . $this->db->dbprefix . "users U ON U.id = O.user_id";
	    $sql .= " LEFT JOIN " . $this->db->dbprefix . "users AU ON AU.id = O.affiliate_id";
	    $sql .= " WHERE O.status_id = " . $status_id . " AND O.deleted = 0";
	    if(!empty($coupon_code)) {
	        $sql .= " AND OC.coupon_code = '" . $coupon_code . "'";
	    }
	    if(!empty($affiliate_id)) {
	        $sql .= " AND O.affiliate_id = " . $affiliate_id;
	    }
	    if($count == 0) {
	        $sql .= " ORDER BY " . $order_by;
	        if ($limit != 0) {
	            $sql .= " LIMIT " . $offset . ", " . $limit;
	        } else {
	            $sql .= " LIMIT $limit";
	        }
	        $result = $this->db->query($sql)->result();
	        return $result;
	    } else {
	        $result = $this->db->query ( $sql )->row();
	        return $result ? $result->total_count : 0;
	    }
	}
	
	/**
	 * Check whether coupon is valid or not
	 * @param string $code
	 * @return number
	 */
	private function _get_coupon($code) {
	    $today = date("Y-m-d h:m:s");
	    $sql = "SELECT C.* FROM " . $this->db->dbprefix . "coupons C";
	    $sql .= " WHERE C.code = '" . $code ."' AND C.status = 1";
	    $sql .= " AND (C.valid_from <= '". $today ."' || C.valid_from IS NULL)";
	    $sql .= " AND (C.valid_upto >= '". $today ."' || C.valid_upto IS NULL)";
	    $sql .= " AND ((C.max_coupon_uses > (SELECT COUNT(id) FROM " . $this->db->dbprefix . "order_coupons WHERE coupon_code = '". $code ."') OR (C.max_coupon_uses = 0)))";
	    $result = $this->db->query ( $sql )->row();
	    return $result;
	}
	
	/**
	 * Checkout Page - Apply Coupon Code
	 * @param boolean $custom_cart - TRUE - custom cart page to avoid unnecessary queries
	 * @return string
	 */
	public function apply_coupon($code, $user_id = 0, $mobile = false, $custom_cart = false) {
	    $this->lang->load('coupons/coupons');
	    $this->load->model('orders/orders_model');
	    $success = false;
	    $user_id = empty($user_id) ? $this->auth->user_id() : $user_id; // user id must be passed
	    $order_total = 0; // Order Total must be Passed through session without discount
	    $cart_items = $this->orders_model->checkout_params($user_id);
	    foreach($cart_items as $cart_item) {
	        $order_total += $cart_item['total'];
	    }
	    $record = $this->_get_coupon($code);
	    if(!$record) {
	        $message = array(
	            'success' => false,
	            'message' => lang('msg_valid_coupon')
	        );
	        return $message;
	    } else {
	        // The Counts is the number of times this coupon is actually used 
	        // The Hits is the number of times this coupon is tried by the user
	        $this->db->where('id', $record->id)->update('coupons', array('hits' => $record->hits + 1)); // Update Hits
	        $users_include = (array) explode(',', $record->users_include ?? '');
	        $users_exclude = (array) explode(',', $record->users_exclude ?? '');
	        $date = date("Y-m-d H:i:s");
	        $user_limit = $this->coupons_model->user_limit_count($code, $user_id);
	        if(($record->max_customer_uses != 0) && ($user_limit >= $record->max_customer_uses)) {
	            $message = array(
	                'success' => $success,
	                'message' => lang('msg_coupon_already_used')
	            );
	            return $message;
	        } else if(floatval($record->min_order) > floatval($order_total)) {
	            $message = array(
	                'success' => $success,
	                'message' => sprintf(lang('msg_min_order_amount'), currency_format($record->min_order))
	            );
	            return $message;
	        } else if(!empty($users_include) && !in_array($user_id, $users_include) && in_array($user_id, $users_exclude)) {
	            $message = array(
	                'success' => $success,
	                'message' => lang('msg_not_applicable')
	            );
	            return $message;
	        } else {
	            // apply coupon to items and if none of the item is valid for discount, display appropriate message to the user
	            $data = $this->calculate_discount($record, $user_id);
	            if($data['success']) {
	                $data = $this->orders_model->cart_ajax_response($user_id, $mobile, $custom_cart);
	                $data['coupon_value'] = $record->value;
	                $data['max_discount'] = $record->max_discount;
	                $data['message'] = lang('msg_coupon_applied');
	            }
	            return $data;
	        }
	    }
	}
		
	/**
	 * Apply Coupon Code
	 * @param object $coupon
	 * @param float $order_total
	 */
	public function calculate_discount($coupon, $user_id = 0) {
	    $coupon = (object) $coupon;
	    $this->load->model('orders/orders_model');
	    $cart_items = $this->orders_model->checkout_params($user_id);
	    $percentage = $coupon->value;
	    $success = false;
	    $proceed_with_calculations = true;
	    $discount_items_count = 0; // items valid for discounts
	    $discount_total = 0; // total discount for max_discount comparision
	    $discounted_items_total = 0; // used in new percentage calculation
	    if(!empty($percentage) && !empty($cart_items)) {
	        foreach($cart_items as $item_id => $cart_item) {
	            $cart_item_discount = $this->item_discount($cart_item, $coupon);
	            if($cart_item_discount !== false) {
	                $cart_items[$item_id] = $cart_item_discount;
	                $discount_total += $cart_item_discount['discount'];
	                $discounted_items_total += $cart_item_discount['subtotal'];
	                $discount_items_count += 1;
	            }
	        }
	        // The final discount must not be greater than the fixed/max_discount amount.
	        // if the max_discount is not null, get the amount in percentage.
	        // if the discount value exceeds max_discount, calculate new percentage value and apply it
	        // e.g. max_discount = 100, discount_calculated = 150, new percentage_value = $item_counts * 100 / max_discount
	        if(!empty($coupon->max_discount) && $discount_total > $coupon->max_discount) {
	            $percentage = ($coupon->max_discount * 100) / $discounted_items_total;
	            $percentage = bcdiv($percentage, 1, 2); // upto 2 decimals without round
	            $coupon->value = $percentage;
	            foreach($cart_items as $item_id => $cart_item) {
	                $cart_item_discount = $this->item_discount($cart_item, $coupon);
	                if($cart_item_discount !== false) {
	                    $cart_items[$item_id] = $cart_item_discount;
	                    $discount_total += $cart_item_discount['discount'];
	                    $discounted_items_total += $cart_item_discount['subtotal'];
	                    $discount_items_count += 1;
	                }
	            }
	        }
	    }
	    if($proceed_with_calculations) {
    	    if($discount_items_count == 0) {
    	        $message = array(
    	            'success' => $success,
    	            'message' => lang('msg_not_applicable')
    	        );
    	        return $message;
    	    } else {
    	        // $this->orders_model->cart_items();
    	        // the coupon is saved in a session variable to call it again
    	        // on coupon removal, this session is cleared and default values are stored
    	        // save coupon in a variable in case of page refresh
    	        $this->orders_model->checkout_params($user_id, false, array('items' => $cart_items, 'coupon' => $coupon));
    	        $data['success'] = true;
    	        return $data;
    	    }
	    }
	}
	
	/**
	 * Calculate Discount
	 * @param array $cart_item
	 * @param array $coupon
	 * @return number|boolean
	 */
	public function item_discount($cart_item, $coupon, $process_cart_items = true) {
	    $item_applicable_for_discount = false;
	    $applicable_modules = (array) json_decode($coupon->applicable_modules ?? '', true);
	    if(array_key_exists($cart_item['module_id'], $applicable_modules)) {
	        $module = gc_module_name($cart_item['module_id']);
	        switch($module) {
	            case 'packages':
	            case 'products':
	            case 'ads':
	                if($module == 'ads') { // discount is applicable on ad packages
	                    $row_id = isset($cart_item['params']['package_id']) ? $cart_item['params']['package_id'] : 0;
	                } else {
	                    $row_id = $cart_item['row_id'];
	                }
	                $key_include = $module . '_include';
	                $key_exclude = $module . '_exclude';
	                $include = (array) json_decode($coupon->{$key_include} ?? '', true);
	                $exclude = (array) json_decode($coupon->{$key_exclude} ?? '', true);
	                if((empty($include) || in_array($row_id, $include)) && !in_array($row_id, $exclude)) {
	                    $item_applicable_for_discount = true;
	                }
	                break;
	            default:
	                $item_applicable_for_discount = true;
	        }
	    }
	    if($item_applicable_for_discount && $process_cart_items) {
	        return $this->orders_model->process_cart_item($cart_item, $coupon->value);
	    }
	    return $item_applicable_for_discount;
	}
	
	/**
	 * Store affiliates information in affiliate_coupons table
	 * @param int $coupon_id
	 * @param string $type
	 */
	public function after_coupon_save($coupon_id, $action_type = 'insert', $record = array()) {
	    /*$record = (array) $this->db->where('id', $coupon_id)->get($this->table_name)->row();
	    if($record && !empty($record['affiliate_commission'])) {
	        $this->load->model('payouts/payouts_model');
	        $data = array();
	        $module_id = gc_module_id('affiliates');
	        $affiliates_all = $record['affiliate_user_ids'];
	        $payout_users = array(); // Also add payout_users to keep record of all payments
	        // User selected ALL USERS
	        if(empty($affiliates_all)) {
	            $users = $this->db->select('id')->get('users')->result();
	            if($users) {
	                foreach($users as $user) {
	                    $data[] = array(
	                        'user_id' => $user->id,
	                        'coupon_id' => $coupon_id,
	                        'commission' => $record['affiliate_commission'],
	                        'type' => $record['affiliate_commission_type'],
	                        'status' => 1,
	                        'created_on' => date("Y-m-d H:i:s"),
	                        'created_by' => $this->auth->user_id(),
	                        'created_ip' => $this->input->ip_address(),
	                        'modified_on' => date("Y-m-d H:i:s"),
	                        'modified_by' => $this->auth->user_id(),
	                        'modified_ip' => $this->input->ip_address()
	                    );
	                    
	                    $payout_users[] = array(
	                        'user_id' => $user->id,
	                        'module_id' => $module_id,
	                        'created_on' => date("Y-m-d H:i:s"),
	                        'created_by' => $this->auth->user_id(),
	                        'created_ip' => $this->input->ip_address(),
	                        'modified_on' => date("Y-m-d H:i:s"),
	                        'modified_by' => $this->auth->user_id(),
	                        'modified_ip' => $this->input->ip_address()
	                    );
	                }
	            }
	        } else {
	            // add affiliate coupon only for the selected users
	            $usersArray = explode(',', $record['affiliate_user_ids'] ?? '');
	            foreach($usersArray as $user) {
	                $data[] = array(
	                    'user_id' => $user,
	                    'coupon_id' => $coupon_id,
	                    'commission' => $record['affiliate_commission'],
	                    'type' => $record['affiliate_commission_type'],
	                    'status' => 1,
	                    'created_on' => date("Y-m-d H:i:s"),
	                    'created_by' => $this->auth->user_id(),
	                    'created_ip' => $this->input->ip_address(),
	                    'modified_on' => date("Y-m-d H:i:s"),
	                    'modified_by' => $this->auth->user_id(),
	                    'modified_ip' => $this->input->ip_address()
	                );
	                $payout_users[] = array(
	                    'user_id' => $user,
	                    'module_id' => $module_id,
	                    'created_on' => date("Y-m-d H:i:s"),
	                    'created_by' => $this->auth->user_id(),
	                    'created_ip' => $this->input->ip_address(),
	                    'modified_on' => date("Y-m-d H:i:s"),
	                    'modified_by' => $this->auth->user_id(),
	                    'modified_ip' => $this->input->ip_address()
	                );
	            }
	        }
	        
	        /**
	         * ON_DUPLICATE method is created in the GOCLIXY/CI3/DATABASE/DB_query_builder.php FILE
	         * FOR INSERT ON DUPLICATE UPDATE - Refer to Documentation for more information
	         */
	        /*if($action_type == 'insert') {
	            $this->db->on_duplicate('affiliate_coupons', $data);
	        } else {
	            // deactivate all existing coupons
	            // if a user is removed then it will be deactivated from the list
	            $this->db->where('coupon_id', $coupon_id);
	            $this->db->update('affiliate_coupons', array('status' => 0));
	            // INSERT ON DUPLICATE UPDATE
	            $this->db->on_duplicate('affiliate_coupons', $data);
	        }
	        
	        // Also add payout_users to keep records of all payments
	        $this->db->on_duplicate('payout_users', $payout_users);
	    }*/
	}
}