<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

/*
	Reference:
		Hours	-2 = Not Hourly
				-1 = Every Hour
		Days:	 0 = Every Day of the Week
		Months:  0 = Every Day of the Month
*/

class Settings extends Admin_Controller {
	protected $permissionView   = 'Cronjobs.Settings.View';
	
	public function __construct() {
		parent::__construct();

		$this->auth->restrict($this->permissionView);
		$this->lang->load('cronjobs');

		Template::set('toolbar_title', lang('cronjobs_manage'));
		Template::set_block('sub_nav', 'settings/sub_nav');

		$this->load->model('cronjob_model');
	}

	//--------------------------------------------------------------------

	public function index() {
		$this->load->helper('date');

		// Sort By
		$order_by = $this->gc->sort('cronjobs');
		if(!empty($order_by)) {
			$this->cronjob_model->order_by($this->session->userdata('cronjobs_sort')['column'], $this->session->userdata('cronjobs_sort')['order']);
		} else {
			$this->cronjob_model->order_by('module', 'asc');
		}
		$tasks = $this->cronjob_model->find_all();
		Template::set('tasks', $tasks);
		Template::render();
	}

	//--------------------------------------------------------------------

	public function create() {
		if ($this->input->post('submit')) {
			if ($this->save_task() !== false) {
				Template::set_message(lang('cronjob_task_completed'), 'success');
				redirect(SITE_AREA .'/settings/cronjobs');
			}
		}

		Template::set('modules',  Modules::list_modules());

		Template::set('toolbar_title', lang('cronjob_add_new_task'));
		Template::set_view('settings/task_form');
		Template::render();
	}

	//--------------------------------------------------------------------

	public function edit($id=null) {
		if ($this->input->post('submit')) {
			if ($this->save_task('update', $id) !== false) {
				Template::set_message(lang('cronjob_task_saved'), 'success');
				redirect(SITE_AREA .'/settings/cronjobs');
			}
		}

		Template::set('task', $this->cronjob_model->find($id));

		Template::set('modules', Modules::list_modules());

		Template::set_view('settings/task_form');
		Template::render();
	}

	//--------------------------------------------------------------------

	public function clear_running_flag($task_id=0) {
		if (empty($task_id)) {
			Template::set_message(lang('cronjob_no_task_id'), 'error');
			Template::redirect($this->previous_page);
		}

		// Grab it so we can give a better message
		$task = $this->cronjob_model->find($task_id);

		if (!$task) {
			Template::set_message(lang('cronjob_task_not_found') . ' ' . $task_id, 'warning');
			Template::redirect($this->previous_page);
		}

		if (!$task->running) {
			Template::set_message(lang('cronjob_task_not_running'), 'warning');
			Template::redirect($this->previous_page);
		}

		if ($this->cronjob_model->update($task_id, array('running' => 0))) {
			Template::set_message(sprintf(lang('cronjob_running_flag'), $task->id), 'success');
		} else {
			Template::set_message(lang('cronjob_database_error') . ' ' . $this->cronjob_model->error, 'danger');
		}

		Template::redirect($this->previous_page);
	}

	//--------------------------------------------------------------------


	//--------------------------------------------------------------------
	// !Private Methods
	//--------------------------------------------------------------------

	private function save_task($type='insert', $id=null) {
	    gc_demo_mode();
		$this->form_validation->set_rules('title', lang('cronjob_title'), 'required|trim|strip_tags|max_length[255]|xss_clean');
		$this->form_validation->set_rules('description', lang('cronjob_short_description'), 'trim|strip_tags|max_length[255]|xss_clean');
		$this->form_validation->set_rules('method_name', lang('cronjob_method_to_run'), 'required|trim|strip_tags|max_length[255]|alpha_dash|xss_clean');

		if ($this->form_validation->run() === false) {
			return false;
		}
		
		
		$data = array();
		
		$data['title'] = $this->input->post('title');
		$data['description'] = $this->input->post('description');
		$data['module'] = $this->input->post('module');
		$data['method_name'] = $this->input->post('method_name');
		$data['time_minutes'] = $this->input->post('time_minutes');
		$data['time_hours'] = $this->input->post('time_hours');
		$data['time_week_day'] = $this->input->post('time_week_day');
		$data['time_month_day'] = $this->input->post('time_month_day');	
		$data['time_month'] = $this->input->post('time_month');
		$data['enabled'] = $this->input->post('enabled') ? 1 : 0;

		if ($type == 'insert') {
			$id = $this->cronjob_model->insert($data);
			return $id;
		} else if ($type == 'update') {
			return $this->cronjob_model->update($id, $data);
		}

		return false;
	}	
	
	/**
	 * Update Version Files
	 * Design a special format for language files
	 * A single file for database update, run this file line by line to update database tables
	 */
	public function update() {
	    $this->load->model('cronjobs/cronjob_model');
	    $revisions = curl_marketplace('revisions');
	    $revisions = !empty($revisions['result']) ? $revisions['result'] : array();
	    if(isset($_POST['update_version'])) {
	        gc_demo_mode();
	        // perform the udpate
	        $not_updated_versions = $this->gcrud->release_update($_POST['update_version']);
	        // update version information in the settings table
	        if(empty($not_updated_versions)) {
	            clear_cache();
	            Template::set_message(lang('uvf_success_update'), 'success');
	            Template::redirect(current_url());
	        } else {
	            Template::set_message(sprintf(lang('uvf_failure_versions'), implode(', ', $not_updated_versions)), 'danger');
	        }
	    }
	    Template::set(array(
	        'revisions' => $revisions,
	        'toolbar_title' => lang('uvf_toolbar_title')
	    ));
	    Template::render();
	}
}