<?php if (! defined ( 'BASEPATH' )) exit ( 'No direct script access allowed' );
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class Cronjobs_tasks {
	/**
	 * Stores the CodeIgniter core object.
	 *
	 * @access public
	 *        
	 * @var object
	 */
    protected $CI;
    protected $dbprefix;
	
	// --------------------------------------------------------------------
	
	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct($config = array()) {
		$this->CI = & get_instance ();
		
		$this->CI->load->model ( 'cronjobs/cronjob_model', null, true );
		$this->CI->load->model ( 'orders/orders_model' );
		$this->CI->load->model ( 'tax_rates/tax_rates_model' );
		$this->CI->load->library('parser');
		$this->CI->load->helper('gc');
		$this->CI->load->library('gc');
		$this->CI->load->library('slug');
		$this->CI->load->library('goclixy/gemailer');
		$this->CI->load->library ( 'emailer/emailer' );
		$this->CI->lang->load('routes', 'english');
		$this->CI->lang->load('application');
		$this->CI->lang->load('goclixy');
		$this->dbprefix = $this->CI->db->dbprefix;
	}
	
	
	/**
	 * Handle Listing Invoicing
	 * Change status of listings expiring today
	 * Create Invoice and notify users
	 * Send reminders
	 */
	public function task_invoicing() {
	    // Get ID of all existing listings
	    $status = 2;
	    $this->CI->load->model ( 'listings/listings_model' );
	    $this->CI->load->model ( 'packages/packages_model' );
	    $this->CI->load->model ( 'orders/orders_model' );
	    $ids = array ();
	    /**
	     * SELECT * FROM `listings` WHERE expires_on <= now() AND expires_on != '0000-00-00' AND status != 2
	     */
	    $result = $this->CI->listings_model->select ( 'id, package_id, title, contact_person, email, expires_on' )->find_all_by ( array (
	        'expires_on <' => date ( 'Y-m-d' ),
	        'expires_on !=' => NULL,
	        'expires_on !=' => '0000-00-00',
	        'status !=' => $status
	    ) );
	    if ($result) {
	        $downgrade_packages = array();
	        foreach ( $result as $record ) {
	            // check whether the listing package has a downgrade package set by the Admin
	            $pkg = $this->CI->packages_model->select('downgrade_package')->find($record->package_id);
	            if($pkg && $pkg->downgrade_package != 0) {
	                $downgrade_package_id = $pkg->downgrade_package;
	                // calculate downgrade package expiry
	                $downgrade_package_info = $this->CI->packages_model->select('duration, forever, fixed_date')->find($downgrade_package_id);
	                if($downgrade_package_info) {
	                    $pkg_expires_on = $this->CI->orders_model->calculate_expires_on($downgrade_package_info, 1, 1);
	                    // update listing and other modules
	                    $this->CI->listings_model->update_recursive($record->id, 'package_id', $downgrade_package_id);
	                    // update the listing expires on date
	                    $this->CI->db->query ( "UPDATE " . $this->dbprefix . "listings SET expires_on = '" . $pkg_expires_on . "' WHERE id = " . $record->id);
	                    // send email if you want to send notification to the user
	                } else {
	                    $this->CI->listings_model->update_recursive($record->id, 'status', $status);
	                    // send email
	                    $this->_expired_email ('listings', $record );
	                }
	            } else {
	                /**
	                 * INACTIVE : set status of all modules to 0
	                 * ACTIVE: set status of only subscribed number of modules to 1
	                 */
	                $this->CI->listings_model->update_recursive($record->id, 'status', $status);
	                // send email
	                $this->_expired_email ('listings', $record );
	            }
	        }
	    }

	    /*
	     $expiring_where = "expires_on IS NOT NULL AND expires_on < '" . date ( 'Y-m-d H:i:s' ) . "' AND status_id = 2";
	     $expired_orders = $this->CI->db->where($expiring_where)->get('order_items')->result();
	     if($expired_orders) {
    	     foreach($expired_orders as $expired_order) {
        	     $this->CI->db->where('order_id', $expired_order->order_id)->update('orders', array('status_id' => 8, 'modified_on' => date('Y-m-d H:i:s')));
        	     $this->CI->db->where('order_id', $expired_order->order_id)->update('order_items', array('status_id' => 8, 'modified_on' => date('Y-m-d H:i:s')));
        	     // $this->CI->db->where('order_id', $expired_order->order_id)->update('order_invoices', array('status_id' => 8, 'modified_on' => date('Y-m-d H:i:s')));
    	     }
	     }*/
	    
	    /**
	     * Find listing which are expiring on next invoice generation date
	     * Calculate Next Invoice Date with respect to Today's date and if both are equal send invoice with email
	     * It is important to store order_id with invoice or else no invoice is generated
	     * Find first reminder date and if date is equal to today send first reminder email
	     * Find second reminder date and if date is equal to today send second reminder email
	     * Find third reminder date and if date is equal to today send reminder email
	     * Save each reminder history
	     * Do this for each listing
	     * Set reminder date option on invoice creation. if today = invoice_creation_date update reminder_dates as per settings and send email)
	     * Match each reminder date with today and notify customer
	     * Order type field is required to find whether listing is renewal of listing claimed or membership renewal
	     */
	    /**
	     * SELECT * FROM `orders` WHERE expires_on >= '2016-03-15' AND expires_on <= '2016-03-15' + INTERVAL 30 DAY
	     */
	    $tax = $this->CI->tax_rates_model->get_tax_rates();
	    $today_date = date('Y-m-d');
	    $reminder_items = $this->CI->db->where("reminders LIKE '%" . $today_date . "%'")->get('order_items')->result();
	    if($reminder_items) {
	        $this->CI->load->helper('form');
	        foreach($reminder_items as $reminder_item) {
	            // Performa Invoice Code - Get invoice generated
	            // $reminder_order_item = $this->CI->db->where(array('subscription_type' => 3, 'ru_order_id' => $reminder_item->order_id))->get('order_items')->row();
	            $reminders = json_decode($reminder_item->reminders ?? '', true);
	            if(!empty($reminders)) {
					if(!empty($reminders[0])) {
					    $reminders = json_decode($reminders[0] ?? '', true);
				    }	                
	                if(!empty($reminders)) {
	                    /*
	                     // Performa Invoice Code - Generate Invoice before order expiry
	                     if(empty($reminder_order_item) && !empty($reminders['next_invoice_date']) && $reminders['next_invoice_date'] == $today_date) {
	                        if($reminder_item->module_id == gc_module_id('packages')) {
	                            // get invoicing package
	                            $invoicing_package_id = $reminder_item->row_id;
	                            $package_info = $this->CI->gc->get('packages', array('where' => 'id = ' . $reminder_item->row_id, 'row' => 1));
	                            if(!empty($package_info->invoicing_package)) {
	                                $invoicing_package_id = $package_info->invoicing_package;
	                            }
	                            $this->CI->orders_model->generate_order($reminder_item->user_id, 'packages', $invoicing_package_id, $reminder_item->order_id);
	                        }
	                    }
	                    if(empty($reminder_order_item)) {
	                        $reminder_order_item = $this->CI->db->where(array('subscription_type' => 3, 'ru_order_id' => $reminder_item->order_id))->get('order_items')->row();
	                    }
	                    if(!empty($reminder_order_item)) {
	                        $this->send_reminders($reminder_order_item, $reminders, $today_date, 'first_reminder');
	                        $this->send_reminders($reminder_order_item, $reminders, $today_date, 'second_reminder');
	                        $this->send_reminders($reminder_order_item, $reminders, $today_date, 'third_reminder');
	                    }
	                    */
	                    // Do not send performa invoice, instead send a message of expiry
	                    $this->send_reminders($reminder_item, $reminders, $today_date, 'first_reminder');
	                    $this->send_reminders($reminder_item, $reminders, $today_date, 'second_reminder');
	                    $this->send_reminders($reminder_item, $reminders, $today_date, 'third_reminder');
	                }
	            }
	        }
	    }
	    
	    /**
	     * Changes 26/07/2016
	     * Get Next Invoice Days from all packages
	     * Calculate the Expiry based on the packages result
	     * Send Invoice Email
	     */	    
	    // Set Ad packages invoicing date
	    $this->CI->load->model('ads/ad_packages_model');
	    $ad_packages = $this->CI->ad_packages_model->select('days_before_next_invoice')->find_all();
	    $this->set_invoice_dates($ad_packages, 'ads');
	    
	    /**
	     * Handle Invoice Expiry
	     * Set the invoice to Expired if not paid even after the invoice_expiry date
	     */
	    /*$invoices_expired = $this->CI->db->select('id')->get_where('order_invoices', array(
	        // 'invoice_expiry' => date('Y-m-d'),
	        'status_id' => 1,
	        'deleted' => 0
	    ))->result();
	    if($invoices_expired) {
	        $invoice_data = array();
	        foreach($invoices_expired as $invoice_expiry) {
	            $invoice_data[] = array(
	                'id' => $invoice_expiry->id,
	                'status_id' => 8
	            );
	        }
	        $this->CI->db->update_batch('order_invoices', $invoice_data, 'id');
	    }*/
		
	    
	    // handle Ads expiry
	    $this->ad_expiry(4);
	    
	    /**
	     * Update order status which are expiring today and status is confirmed
	     * Expire only the order_items not the complete orders
		 * An order can have products, package, wallet, ad and leads that can have a different expiration date
	     */
	    $expiring_where = "expires_on IS NOT NULL AND expires_on < '" . date ( 'Y-m-d H:i:s' ) . "' AND status_id = 2";
	    $this->CI->db->where($expiring_where)->update('order_items', array('status_id' => 8, 'modified_on' => date('Y-m-d H:i:s')));
	}
	
	/**
	 * Send Reminders
	 * @param object $record
	 * @param array $reminders
	 * @param string $today_date
	 * @param string $type
	 */
	private function send_reminders($record, $reminders, $today_date, $type = 'first_reminder') {
	    if(!empty($reminders[$type]) && $reminders[$type] == $today_date) {
	        $sequence_no = '01';
	        if($type == 'second_reminder') {
	            $sequence_no = '02';
	        } else if($type == 'third_reminder') {
	            $sequence_no = '03';
	        }
	        $this->CI->orders_model->email_print($record->order_id, 'email', array('action_type' => 'reminder', 'sequence_no' => $sequence_no));
	        // save order history
	        // save_order_history($order_id, $old_status_id, $new_status_id, $comment_id = 0, $customer_notified = 0)
	        // $this->CI->orders_model->save_order_history($record->order_id, 0, 1, sprintf(lang('msg_reminder_sent'), lang('label_' . $type)));
	    }
	}
	
	/**
	 * Handle Ads Expiry
	 * @param integer $status
	 */
	private function ad_expiry($status) {
	    $this->CI->load->model('ads/ads_model');
	    $result = $this->CI->ads_model->find_ads($status);
	    if ($result) {
	        $downgrade_packages = array();
	        foreach ( $result as $record ) {
	            // Update status
	            $this->CI->db->where('id', $record->id)->update('ads', array('status' => $status));
	            // Send expiration email
	            $this->_expired_email ('ads', $record, 'ad_email_templates');
	        }
	    }
	}
	
	/**
	 * Listing/Ad Packages
	 * Common function to set invoices date
	 * @param object @packages
	 * @param string type - to specify Ad/Listing packages
	 */
	private function set_invoice_dates($packages, $type = NULL) {
		if(!empty($packages)) {
			foreach($packages as $package) {
				$next_invoice = new DateTime ( date ( 'Y-m-d' ) );
				$next_invoice->add ( new DateInterval ( 'P' . $package->days_before_next_invoice . 'D' ) );
				$next_invoice_date = $next_invoice->format ( "Y-m-d" );
				$next_invoices = $this->CI->db->select ( 'order_id, user_id, expires_on' )->where ( array (
						'expires_on' => $next_invoice_date,
						'DATE(cronrun_on) !=' => date ( 'Y-m-d' ),
						'status_id' => 2,
						'deleted' => 0
				) )->get('order_items')->result();
				if ($next_invoices) {
					/**
					 * TODOS
					 * --------- Get the Invoice Expiry date set in the packages
					 * --------- Delete any active order for the same user before adding a new order
					 * --------- If the existing order type is a subscription then check the invoicing package set in the packages.
					 * --------- Set invoicing for PayAsYouGo
					 * --------- Subscription Renewal - generate invoice only if the package is Renewable.
					 * --------- Calculate tax, subtotal and total
					 * --------- If PYG invoicing Package/Renewal is not valid
					 * --------- For renewal, update the module expiry date for PYG
					 * --------- Admin Panel create order
					 * --------- Invoice pay set featured for each module
					 * Mobile application error
					 * --------- Once cronjob run for a specific order do not update it if the job is run twice on the same day using a new column cron_run
					 * --------- Emailing to be checked
					 * --------- Change status of modules to expired for PYG module if the listing is expired.
					 */
					// Create new invoice in the invoices table and notify customer via email
					foreach ( $next_invoices as $invoice ) {
		
						// Generate order id
						$order_id = settings_item ( 'fin.order_prefix' ) . time ();
						$invoice_order_id = $invoice->order_id;
		
						// Save order information
						$tax_percent = 0.00;
						if (! empty ( $tax )) {
							$tax_percent = $tax->tax_percent;
						}
		
						// Get the current order information
						$order_info = $this->CI->orders_model->find_by ( 'order_id',  $invoice_order_id);
						$package_id = $order_info->package_id;
						if($type == 'ads') {
							$package_info = $this->CI->ad_packages_model->find($package_id);
						} else {
							$package_info = $this->CI->packages_model->find($package_id);
						}
		
						// Generate new invoice, only if the package is Renewable
						// AND order_type of existing order is not Claim
						if($type != 'ads' && $package_info->multisale == 1 && $order_info->order_type == 1) {
								
							// Get the renewal package for which the invoice will be generated
							// if the renewal package is not same as the current one get the new package information
							if($package_info->invoicing_package != $package_id) {
								$package_info = $this->CI->packages_model->find($package_info->invoicing_package);
							}
								
							// Save Order and other related information
							$this->invoicing($order_id, $invoice_order_id, $package_id, $order_info, $package_info, $tax);
								
						} else if ($order_info->order_type != 1) {
							// Save Order and other related information
							$this->invoicing($order_id, $invoice_order_id, $package_id, $order_info, $package_info, $tax);
						}
		
						/**
						 * Once cronjob run for a specific order, do not allow to rerun the job to prevent multiple invoice creation
						 * Invoice and Reminder should be sent once in a day
						 */
						$this->CI->db->update('orders', array('cronrun_on' => date('Y-m-d h:m:s')), array('order_id' => $invoice_order_id));
					}
				}
			}
		}
	}
	
	/**
	 * Save order related information
	 * @param string $order_id
	 * @param string $invoice_order_id
	 * @param int $package_id
	 * @param object $order_info
	 * @param object $package_info
	 * @param object $tax_rate
	 */
	private function invoicing($order_id, $invoice_order_id, $package_id, $order_info, $package_info, $tax_rate) {
		$this->CI->load->model('orders/orders_model');
		
		// Calculate expiry date based on package duration
		$expires_on = $this->CI->orders_model->calculate_expires_on($package_info, 3, $invoice_order_id);
		
		// Delete any existing active orders placed by the same user
		$this->CI->orders_model->delete_existing_order($order_info->user_id, $package_id, $order_info->order_type, NULL);
		
		// Save Order
		$order_data = $this->prepare_order_data($package_id, $order_info, $package_info, $expires_on, $tax_rate);
		$order_data['renewed_order_id'] = $invoice_order_id;
		$this->CI->orders_model->save_orders($order_id, settings_item ('site.currency_code'), 1, $expires_on, $order_data, 1, 0, 'insert');
			
		// Save order customer information
		$customer_data = $this->CI->db->get_where('order_customers', array('order_id' => $invoice_order_id))->row();
		$user = array(
				'id' => $customer_data->user_id,
				'display_name' => $customer_data->name,
				'email' => $customer_data->email,
				'phone_number' => $customer_data->phone_number,
				'address' => $customer_data->address,
				'postcode' => $customer_data->postcode,
				'city_id' => $customer_data->city_id
		);
		$this->CI->orders_model->save_order_customers_admin($order_id, (object) $user);
		
		// Create new invoice with the new Order ID and send to the user
		$this->CI->orders_model->save_invoices($order_id, 1, $customer_data->user_id, $expires_on, $package_info);
		/*$new_invoice = $this->CI->db->select('id')->get_where('invoices', array('order_id' => $invoice_order_id))->row();
		$invoice_list = $this->CI->cronjob_model->invoice_list($new_invoice->id);
		$this->send_invoice($invoice_list, $tax_rate->tax_percent);*/
		$this->send_invoice($order_id);
		
		// Save order history
		$this->CI->orders_model->save_order_history($order_id, 0, 1, lang('label_order_created_notified'));
	}
	
	/**
	 * Prepare Order Data for Invoicing
	 * @param int $package_id
	 * @param object $order_info
	 * @param object $package_info
	 * @param object $tax_rate
	 * @return multitype:number NULL unknown string Ambigous <number, unknown>
	 */
	private function prepare_order_data($package_id, $order_info, $package_info, $expires_on, $tax_rate) {
		$this->CI->load->model('orders/orders_model');
		// Prepare data to be saved in orders table
		$order_data = array();		
		$order_data['subscription_type'] = 3; // 3 - Renewal
		$order_data['order_discount'] = 0.00; // No Discount
			
		switch($order_info->order_type) {
			case 1: // Package
			case 6: // Ads
				$amount = $package_info->price;
				$item = array(
					'title' => $package_info->title,
					'description' => $package_info->description
				);
				break;
			case 4: // PYG - Pay As You Go
				$pyg_module = gc_module_name($order_info->module_id);
				$pyg = json_decode($package_info->pyg ?? '');
				$amount = $pyg->$pyg_module;
				$item = array(
					'title' => lang('pyg'),
					'description' => sprintf(lang('pyg_description'), $package_info->title, lang('singular_' . $pyg_module))
				);
				break;
			case 5: // Wallets
				break;
					
		}

		// Tax rates are enabled in the system
		if(!empty($tax_rate)) {
			$tax_calculation = $this->CI->orders_model->tax_calculation($amount, $tax_rate->tax_percent);
			$order_subtotal = $tax_calculation['order_subtotal'];
			$order_total = $tax_calculation['order_total'];
			$order_tax = $tax_calculation['order_tax'];
		} else {
			$order_subtotal = $amount;
			$order_total = $amount;
			$order_tax = 0.00;
		}
		
		$order_data['item_price'] = $amount;
		$order_data['order_subtotal'] = $order_subtotal;
		$order_data['order_total'] = $order_total;
		$order_data['order_tax'] = $order_tax;
		$order_data['tax_percent'] = !empty($tax_rate) ? $tax_rate->tax_percent : 0.00;
		$order_data['payment_gateways'] = NULL;
		$order_data['comments'] = NULL;
		$order_data['item'] = json_encode($item);
		$order_data['package_id'] = $package_id;
		$order_data['module_id'] = $order_info->module_id;
		$order_data['row_id'] = $order_info->row_id;
		$order_data['qty'] = $order_info->qty;
		$order_data['order_type'] = $order_info->order_type;
		$order_data['user_id'] = $order_info->user_id;
		$order_data['identity'] = $order_info->user_email;
		return $order_data;
	}
	
	
	/**
	 * Send next generated invoice to the customer
	 * @param string $order_id
	 */
	private function send_invoice($order_id) {
	    $record = $this->CI->orders_model->get_order($order_id);
	    if($record) {
	        $this->CI->gemailer->sms_email_pdf('sms_email', array('template_table' => 'order_email_templates', 'module_table' => 'orders', 'action_type' => 'reminder', 'id' => $record['id'], 'record' => $record));
	    }
	}
	
	/**
	 * Send reminders email to the customer
	 */
	private function reminders_email($sequence_no, $reminder) {	    
	    $record = $this->CI->orders_model->get_order($reminder['order_id']);
	    if($record) {
	        $record->sequence_no = $sequence_no;
	        $this->CI->gemailer->sms_email_pdf('sms_email', array('template_table' => 'order_email_templates', 'module_table' => 'orders', 'action_type' => 'reminder', 'id' => $record['id'], 'record' => $record));
	    }
	}
	
	/**
	 * Send listing expired email to the customer
	 */
	public function _expired_email($module, $record, $template_table = 'module_email_templates') {
	    if(empty($record)) { // Fields/Preview Demo Data
	        if($module == 'listings') {
	            $record = $this->CI->db->select ( 'id, package_id, title, contact_person, email, expires_on' )->get($module)->row();
	        } else if($module == 'ads') {
	            $this->CI->load->model('ads/ads_model');
	            $record = $this->CI->ads_model->find_ads (0, 0, true);
	            $record = $record ? $record[0] : array();
	        }
	        if($record) {
	            $record->expires_on = gc_date_format($record->expires_on, false);
	        }
	        return $record;
	    } else { // Notify Admin/User
	        if($record) {
	            $record->expires_on = gc_date_format($record->expires_on, false);
	            $this->CI->gemailer->sms_email_pdf('sms_email', array('template_table' => $template_table, 'module_table' => $module, 'action_type' => 'expiry_notification', 'id' => $record->id, 'record' => $record));
	        }	        
	    }
	}
	
	/**
	 * Update categories listing count
	 */
	/*
	 * public function update_listing_counts() { $sql = "UPDATE " . $this->dbprefix . "categories INNER JOIN"; $sql .= " (SELECT C.id, COUNT(DISTINCT(LC.row_id)) AS TotalCounts FROM " . $this->dbprefix . "categories C"; $sql .= " LEFT JOIN " . $this->dbprefix . "categories_level LC ON LC.parent_id = C.id OR LC.sub_id = C.id OR LC.subsub_id = C.id"; $sql .= " GROUP BY C.id) LCC USING (id) SET " . $this->dbprefix . "categories.counts = LCC.TotalCounts"; $this->CI->db->simple_query($sql); }
	 */
	
	/**
	 * Update categories and locations counts
	 */
	public function task_counts() {
		$this->CI->load->model('listings/listings_model');
		$this->CI->listings_model->update_counts();
	}
	
	/**
	 * Generate sitemap
	 */
	public function task_sitemap() {
		$this->CI->load->model ( 'listings/listings_model' );
		$this->CI->listings_model->update_counts(); // update counts first
		$this->CI->load->library ( 'sitemaps' );
		$this->empty_sitemap_folder();
		$record = $this->CI->listings_model->count_by ( array (
				'deleted' => 0,
				'status' => 1 
		) );
		if ($record > 49999) {
			$querylimit = 49999;
			$urls = array ();
			// Get lowest id from the listing table and start sitemap generation from that ID
			$min_query = "SELECT min( id ) as id, max(id) as max_id FROM " . $this->dbprefix . "listings";
			$min = $this->CI->db->query($min_query)->row();
			for($i = 0; $i < $min->max_id; $i ++) {
				$sql = "SELECT L.id, L.slug, L.city, L.created_on, L.modified_on FROM " . $this->dbprefix ."listings L";
				$sql .= " WHERE L.status = 1 AND L.deleted = 0";
				$sql .= " AND L.id > " . $i;
				$sql .= " LIMIT " . $querylimit;
				$listings = $this->CI->db->query($sql)->result();
				if(!empty($listings)) {
					/**
					 * Changefreq Guidelines
					 * NEVER: Old news stories, press releases, etc
					 * YEARLY: Contact, �About Us�, login, registration pages
					 * MONTHLY: FAQs, instructions, occasionally updated articles
					 * WEEKLY: Product info pages, website directories
					 * DAILY: Module entry index, small message board
					 * HOURLY: Major news site, weather information, forum
					 * ALWAYS: Stock market data, social bookmarking categories
					 */
					/**
					 * Priority XML Guidelines
					 * 0.8-1.0: Homepage, subdomains, product info, major features
					 * 0.4-0.7: Articles and module entries, category pages, FAQs
					 * 0.0-0.3: Outdated news, info that has become irrelevant
					 */
				    
				    $first_id = 0;
				    $last_id = 0;
				    $end_record = end($listings);
				    foreach ( $listings as $listing ) {
				        if(empty($first_id)) {
				            $first_id = $listing->id;
				        }
				        if($listing == $end_record) {
				            $last_id = $listing->id;
				        }
						if (!empty($listing->modified_on) && $listing->modified_on != '0000-00-00 00:00:00') {
							$lastmod = $listing->modified_on;
						} else if (!empty($listing->created_on) && $listing->created_on != '0000-00-00 00:00:00') {
							$lastmod = $listing->created_on;
						} else {
							$lastmod = date ( 'Y-m-d H:i:s' );
						}
						$item = array (
								"loc" => gc_url_format('listings', $listing->slug, $listing->id, $listing->city),
								"lastmod" => date ( "c", strtotime ( $lastmod ) ),
								"changefreq" => "weekly",
								"priority" => "0.8" 
						);
						
						$this->CI->sitemaps->add_item ( $item );
					}
					
					if($first_id == $last_id) {
					    $file_name = $this->CI->sitemaps->build ( "sitemap/listings.xml" );
					} else {
					    $file_name = $this->CI->sitemaps->build ( "sitemap/listings-" . $first_id . "-" . $last_id .".xml" );
					}
					$urls [] = array (
							"loc" => site_url ( $file_name ),
							"lastmod" => date ( "c", strtotime ( date ( 'Y-m-d H:i:s' ) ) ) 
					);
					$i = $last_id;
				} else {
				    $i = $min->max_id + 1;
				}
			}
		} else {
			$this->CI->config->load ( 'sitemaps' );
			$this->CI->config->set_item ( 'sitemaps_gzip', FALSE ); // disable compression for websites having few records < 49999
			
			$min_query = "SELECT min( id ) as id, max(id) as max_id FROM ".$this->dbprefix ."listings";
			$min = $this->CI->db->query($min_query)->row();
			
			$sql = "SELECT L.id, L.slug, L.created_on, L.modified_on, C.name AS city FROM " . $this->dbprefix ."listings L";
			$sql .= " LEFT JOIN " . $this->dbprefix ."cities C ON C.id = L.city_id";
			$sql .= " WHERE L.deleted = 0 AND L.status = 1";
			$listings = $this->CI->db->query($sql)->result();
			/**
			 * Changefreq Guidelines
			 * NEVER: Old news stories, press releases, etc
			 * YEARLY: Contact, �About Us�, login, registration pages
			 * MONTHLY: FAQs, instructions, occasionally updated articles
			 * WEEKLY: Product info pages, website directories
			 * DAILY: Module entry index, small message board
			 * HOURLY: Major news site, weather information, forum
			 * ALWAYS: Stock market data, social bookmarking categories
			 */
			/**
			 * Priority XML Guidelines
			 * 0.8-1.0: Homepage, subdomains, product info, major features
			 * 0.4-0.7: Articles and module entries, category pages, FAQs
			 * 0.0-0.3: Outdated news, info that has become irrelevant
			 */
			/**
			 * SELECT id, title, slug, created_on, modified_on, 'L' as type FROM rs0d_listings
			 * UNION
			 * SELECT id, title, slug, created_on, modified_on, 'N' as type FROM rs0d_news
			 * UNION
			 * SELECT id, title, slug, created_on, modified_on, 'P' as type FROM rs0d_pages
			 * UNION
			 * SELECT id, name as title, slug, created_on, modified_on, 'C' as type FROM rs0d_categories
			 */
			foreach ( $listings as $listing ) {
				if (!empty($listing->modified_on) && $listing->modified_on != '0000-00-00 00:00:00') {
					$lastmod = $listing->modified_on;
				} else if (!empty($listing->created_on) && $listing->created_on != '0000-00-00 00:00:00') {
					$lastmod = $listing->created_on;
				} else {
					$lastmod = date ( 'Y-m-d H:i:s' );
				}
				$item = array (
						"loc" => gc_url_format('listings', $listing->slug, $listing->id, $listing->city),
						"lastmod" => date ( "c", strtotime ( $lastmod ) ),
						"changefreq" => "weekly",
						"priority" => "0.8" 
				);
				
				$this->CI->sitemaps->add_item ( $item );
			}
			$file_name = $this->CI->sitemaps->build ( "sitemap/listings.xml" ); // -". $min->id .'-' . $min->max_id ."
			$urls [] = array (
					"loc" => site_url ( $file_name ),
					"lastmod" => date ( "c", strtotime ( date ( 'Y-m-d H:i:s' ) ) )
			);
		}

		// Generate Site Map for Remaining Modules
		$news = $this->module_sitemap('news');
		$pages = $this->module_sitemap('pages');
		$categories = $this->module_sitemap('categories');
		$cities = $this->module_sitemap('cities');
		
		// Modules and specific categories and cities SiteMap
		$modules_sitemap = array();
		$file_urls = array_merge((array)$urls, (array)$news, (array)$pages, (array)$categories, (array)$cities);
		$config_modules = gc_modules_only();
		$config_modules[] = 'videos';
		foreach($config_modules as $config_module) {
			$file_urls = array_merge($file_urls, (array) $this->module_sitemap($config_module));
			// categories sitemap
			$file_urls = array_merge($file_urls, (array) $this->module_category_location_sitemap($config_module));
			if(gc_module_settings($config_module, 'locations') == 1) {
				// cities sitemap
			    $file_urls = array_merge($file_urls, (array) $this->module_category_location_sitemap($config_module, 'cities'));
			}
		}
		$this->CI->sitemaps->build_index ( $file_urls, "sitemap.xml" );
	}
	
	/**
	 * Empty the sitemap folder first before generating the new sitemap
	 * IMPORTANT - Else duplicate files will be created each time on ID changes
	 * @return boolean
	 */
	private function empty_sitemap_folder() {
		$dir = FCPATH . '/sitemap';
		$di = new RecursiveDirectoryIterator($dir, FilesystemIterator::SKIP_DOTS);
		$ri = new RecursiveIteratorIterator($di, RecursiveIteratorIterator::CHILD_FIRST);
		foreach ( $ri as $file ) {
		    if($file->isDir()) {
		        rmdir($file);
		    } else if(file_exists(realpath(FCPATH . $file))) {
		        unlink($file);
		    }
		}
		return true;
	}
	
	/**
	 * Build Sitemap for Modules
	 * @param string $module - Name of the Module
	 */
	private function module_sitemap($module) {
		$this->CI->load->library ( 'sitemaps' );
		$csql = "SELECT MAX(M.id) AS max_id FROM ".$this->dbprefix .$module ." M ";
		switch ($module) {
			case 'categories':
			case 'cities':
				$csql .= "WHERE M.active = 1";
				break;
			case 'pages':
			case 'news':
				$csql .= "WHERE M.status = 1";
				break;
			default:
				$csql .= "WHERE M.status = 1 AND M.deleted = 0";
		}
		$crecord = $this->CI->db->query($csql)->row();
		if(!empty($crecord->max_id)) {
			$querylimit = 49999;
			$urls = array ();
			for($i = 0; $i <= $crecord->max_id; $i ++) {
				$sql = "SELECT M.id,";
				switch($module) {
					case 'news':
						$sql .= " title";
						break;
					case 'categories':
					    $sql .= " M.name AS title, " . (settings_item('site.category_slug_uris') == 1 ? 'M.slug_uris' : 'M.slug');
					    break;
					case 'cities':
					    $sql .= " M.name AS title";
					    break;
					default:
						$sql .= " M.title";
				}
				$sql .=", M.slug, M.created_on, M.modified_on FROM " . $this->dbprefix . $module ." M";
				switch ($module) {
					case 'categories':
					case 'cities':
						$sql .= " WHERE M.active = 1 AND M.counts != 0"; // Listings counts are saved in the counts column
						break;
					case 'pages':
					case 'news':
						$sql .= " WHERE M.status = 1";
						break;
					default:
						$sql .= " WHERE M.status = 1 AND M.deleted = 0";
				}
				$sql .= " AND M.id > $i";
                $sql .= " LIMIT " . $querylimit;
				$records = $this->CI->db->query($sql)->result();
				if(!empty($records)) {
					/**
					 * Changefreq Guidelines
					 * NEVER: Old news stories, press releases, etc
					 * YEARLY: Contact, �About Us�, login, registration pages
					 * MONTHLY: FAQs, instructions, occasionally updated articles
					 * WEEKLY: Product info pages, website directories
					 * DAILY: Blog entry index, modules, small message board
					 * HOURLY: Major news site, weather information, forum
					 * ALWAYS: Stock market data, social bookmarking categories
					 */
					/**
					 * Priority XML Guidelines
					 * 0.8-1.0: Homepage, subdomains, product info, major features
					 * 0.4-0.7: Articles and blog entries, category pages, FAQs
					 * 0.0-0.3: Outdated news, info that has become irrelevant
					 */
				    $first_id = 0;
				    $last_id = 0;
				    $end_record = end($records);
				    foreach ( $records as $record ) {
				        if(empty($first_id)) {
				            $first_id = $record->id;
				        }
				        if($record == $end_record) {
				            $last_id = $record->id;
				        }
						if (!empty($record->modified_on) && $record->modified_on != '0000-00-00 00:00:00') {
							$lastmod = $record->modified_on;
						} else if (!empty($record->created_on) && $record->created_on != '0000-00-00 00:00:00') {
							$lastmod = $record->created_on;
						} else {
							$lastmod = date ( 'Y-m-d H:i:s' );
						}
						switch($module) {
							case 'news':
							case 'pages':
								$item = array (
									"loc" => gc_url_format($module, $record->slug),
									"lastmod" => date ( "c", strtotime ( $lastmod ) ),
									"changefreq" => "weekly",
									"priority" => "0.8"
								);
								break;
							case 'categories':
								$item = array (
									"loc" => gc_category_url('listings', $record->slug, $record->id),
									"lastmod" => date ( "c", strtotime ( $lastmod ) ),
									"changefreq" => "weekly",
									"priority" => "0.8"
								);
								break;
							case 'cities':
								$city_slug = !empty($record->slug) ? $record->slug : $this->CI->slug->create_slug($record->title);
								$item = array (
									"loc" => gc_url_format('cities', $city_slug, $record->id),
									"lastmod" => date ( "c", strtotime ( $lastmod ) ),
									"changefreq" => "weekly",
									"priority" => "0.8"
								);
								break;
							default:
								$item = array (
									"loc" => gc_url_format($module, $record->slug, $record->id),
									"lastmod" => date ( "c", strtotime ( $lastmod ) ),
									"changefreq" => "weekly",
									"priority" => "0.8"
								);
						}
	
						$this->CI->sitemaps->add_item ( $item );
					}
					
					if($first_id == $last_id) {
					    $file_name = $this->CI->sitemaps->build ( "sitemap/" . $module . ".xml" );
					} else {
					    $file_name = $this->CI->sitemaps->build ( "sitemap/" . $module . ".xml" ); // "-$first_id-" . $last_id .
					}
					$urls [] = array (
							"loc" => site_url ( $file_name ),
							"lastmod" => date ( "c", strtotime ( date ( 'Y-m-d H:i:s' ) ) )
					);
					$i = $last_id;					
				} else {
				    $i = $crecord->max_id + 1;
				}
				return $urls;
			}
		}
	}
	
	/**
	 * Module specific categories and cities SiteMap
	 * 
	 * @param string $module - Name of the Module
	 * @param string $table - Name of the Table
	 */
	private function module_category_location_sitemap($module, $table = 'categories') {
		$this->CI->load->library ( 'sitemaps' );
		$urls = array();
		$querylimit = 49999;
		$sql = "SELECT MAX(row_id) AS max_id FROM " . $this->dbprefix . "module_counts WHERE module_id = " . gc_module_id($table) . " AND toggle_module_id = " . gc_module_id($module);
		$result = $this->CI->db->query($sql)->row();
		if(!empty($result->max_id)) {
		    for($i = 0; $i <= $result->max_id; $i++) {
        		$sql = "SELECT M.id, M.name AS title, M.slug, M.created_on, M.modified_on";
        		$sql .= " FROM " . $this->dbprefix . $table . " M";
        		$sql .= " INNER JOIN " . $this->dbprefix . "module_counts MC ON MC.module_id = " . gc_module_id($table) . " AND MC.row_id = M.id AND MC.toggle_module_id = " . gc_module_id($module);
        		$sql .= " WHERE M.active = 1";
        		$sql .= " AND M.id > " . $i;
        		$sql .= " LIMIT " . $querylimit;
        		$records = $this->CI->db->query($sql)->result();
        		if(!empty($records)) {
        			/**
        			 * Changefreq Guidelines
        			 * NEVER: Old news stories, press releases, etc
        			 * YEARLY: Contact, �About Us�, login, registration pages
        			 * MONTHLY: FAQs, instructions, occasionally updated articles
        			 * WEEKLY: Product info pages, website directories
        			 * DAILY: Blog entry index, small message board
        			 * HOURLY: Major news site, weather information, forum
        			 * ALWAYS: Stock market data, social bookmarking categories
        			 */
        			/**
        			 * Priority XML Guidelines
        			 * 0.8-1.0: Homepage, subdomains, product info, major features
        			 * 0.4-0.7: Articles and Module entries, category pages, FAQs
        			 * 0.0-0.3: Outdated news, info that has become irrelevant
        			 */
        		    $first_id = 0;
        		    $last_id = 0;
        		    $end_record = end($records);
        			foreach ( $records as $record ) {
        			    if(empty($first_id)) {
        			        $first_id = $record->id; 
        			    }
        			    if($record == $end_record) {
        			        $last_id = $record->id;
        			    }
        				if (!empty($record->modified_on) && $record->modified_on != '0000-00-00 00:00:00') {
        					$lastmod = $record->modified_on;
        				} else if (!empty($record->created_on) && $record->created_on != '0000-00-00 00:00:00') {
        					$lastmod = $record->created_on;
        				} else {
        					$lastmod = date ( 'Y-m-d H:i:s' );
        				}
        				switch($table) {
        					case 'categories':
        						$item = array (
        						"loc" => gc_category_url($module, $record->slug, $record->id),
        						"lastmod" => date ( "c", strtotime ( $lastmod ) ),
        						"changefreq" => "weekly",
        						"priority" => "0.8"
        								);
        								break;
        					case 'cities':
        						$city_slug = !empty($record->slug) ? $record->slug : $this->CI->slug->create_slug($record->title);
        						$item = array (
        								"loc" => gc_location_url($module, $city_slug, $record->id),
        								"lastmod" => date ( "c", strtotime ( $lastmod ) ),
        								"changefreq" => "weekly",
        								"priority" => "0.8"
        						);
        						break;
        				}
        
        				$this->CI->sitemaps->add_item ( $item );
        			}
        			if($first_id == $last_id) {
        			    $file_name = $this->CI->sitemaps->build ( "sitemap/" . $module . "-" . $table . ".xml" );
        			} else {
        			    $file_name = $this->CI->sitemaps->build ( "sitemap/" . $module . "-" . $table . ".xml" ); // "-$first_id-" . $last_id .
        			}
        			$urls[] = array (
        					"loc" => site_url ( $file_name ),
        					"lastmod" => date ( "c", strtotime ( date ( 'Y-m-d H:i:s' ) ) )
        			);
        			$i = $last_id;
        		} else {
        		    $i = $result->max_id + 1;
        		}
		    }
		}
		return $urls;
	}
	
	/**
	 * Truncate and fill listing_search table with cronjob after 24 hours
	 * All columns on which search is to be carried are taken from different tables 
	 * And inserted into listing_search table to avoid joins to make search faster.
	 */
	public function task_listing_search() {	
		return false;			
		// STEP 1: Empty listing_search table
		$truncate_listing_search = "TRUNCATE TABLE " . $this->dbprefix . "listing_search";
		$this->CI->db->simple_query($truncate_listing_search);
		
		// STEP 2: Fill listing_search table		
		$this->CI->gc->insert_update_listing_search(NULL,'insert',1);
	}
	
	/**
	 * Process the email queue in chunks.
	 *
	 * Defaults to 33 which, if processed every 5 minutes, equals 400/hour and
	 * should keep you safe with most ISPs. Always check your ISP's terms of
	 * service to verify, though.
	 */
	public function task_process_queue() {
		// Email to be processed within 5 minutes
		$limit = (int)(((int) settings_item('site.hourly_emails_limit'))/12);
		$this->CI->load->library('emailer');
		$this->CI->emailer->enable_debug(true);
		
		// Use ob to catch output designed for CRON only
		// ob_start();
		$success = $this->CI->emailer->process_queue($limit);
		// ob_end_clean();
		
		if (! $success) {
			Template::set('email_debug', $this->CI->emailer->debug_message);
		}
	}
	
	/**
	 * Purge all Ad Views greater than the X number of days
	 * (e.g. 90) specified in the Settings
	 */
	public function task_ad_views_purge() {
		$this->CI->load->model('ads/ads_model');
    	$this->CI->ads_model->purge_ad_views();
    }
    
    /**
     * Activate all Ads which exceeded their Daily Budget Limit
     * Run this job once in a Day to enable all such Ads
     */
    public function task_activate_daily_budget_ads() {
    	$this->CI->load->model('ads/ads_model');
    	$this->CI->ads_model->activate_daily_budget_ads();
    }
    
    /**
     * Delete Spam user accounts that are not associated to a listing
     */
    public function task_delete_spam_users() {
        $records = $this->CI->db->get_where('users', array('active' => 0, 'role_id != ' => 1))->result();
        if($records) {
            foreach($records as $record) {
                $count = $this->CI->db->where('user_id', $record->id)->count_all_results('listings');
                if($count == 0) {
                    // purge the user
                    $this->CI->db->where('id', $record->id)->delete('users');
                }
            }
        }
    }
    
    /**
     * Truncate Session Table automatically using Cronjob
     */
    public function task_truncate_session_table() {
        $this->CI->db->truncate('ci3_sessions');        
    }
    
    /**
     * Remove recently viewed older than 7 days
     */
    public function task_remove_recently_viewed($days = 7) {
        $date = date('Y-m-d', strtotime('-' . $days . ' days'));
        $sql = "DELETE FROM " . $this->dbprefix . "recently_viewed WHERE DATE(last_viewed) <= '" . $date . "'";
        $this->CI->db->query($sql);
    }
}
?>