<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Migration_Install_cronjobs extends Migration {
	
	/**
	 * The table's fields
	 *
	 * @var Array
	 */
	private $fields = array(
		'id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'unsigned' => TRUE,
			'auto_increment' => TRUE
		),
		'title' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => FALSE
		),
		'description' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE
		),
		'module' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE,
			'default' => 'core'
		),
		'time_minutes' => array(
			'type' => 'VARCHAR',
			'constraint' => 5,
			'null' => FALSE,
			'default' => 0
		),
        'time_hours' => array(
			'type' => 'TINYINT',
			'constraint' => 2,
			'null' => FALSE,
			'default' => 0
		),
		'time_week_day' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default' => 0
		),
		'time_month_day' => array(
			'type' => 'TINYINT',
			'constraint' => 2,
			'null' => FALSE,
			'default'=> 0
		),
		'enabled' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default'=> 0
		),
		'method_name' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => FALSE
		),
		'running' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default'=> 0
		),
		'last_start' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => TRUE,
			'default'=> 0
		),
		'last_stop' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => TRUE,
			'default'=> 0
		),
	);
	
	/**
	 * Install Cronjobs
	 *
	 * @var Array
	 */
	private $cronjobs = array (
			array(
					'title'				=> 'Optimize Database',
					'description'		=> 'Run once per day to optimize your database tables.',
					'module'			=> 'core',
					'time_minutes'		=> 0,
					'time_hours'		=> -1,
					'time_week_day'		=> 0,
					'time_month_day'	=> 0,
					'enabled'			=> 0,
					'method_name'		=> 'task_db_optimize'
			), array(
					'title'				=> 'Category Counts',
					'description'		=> 'Count and update number of listings in each category',
					'module'			=> 'core',
					'time_minutes'		=> 0,
					'time_hours'		=> -1,
					'time_week_day'		=> 0,
					'time_month_day'	=> 0,
					'enabled'			=> 1,
					'method_name'		=> 'task_category_counts'
			), array(
					'title'				=> 'Generate Sitemap',
					'description'		=> 'Generate Listings Sitemap',
					'module'			=> 'cronjobs',
					'time_minutes'		=> 0,
					'time_hours'		=> 0,
					'time_week_day'		=> 0,
					'time_month_day'	=> 0,
					'enabled'			=> 1,
					'method_name'		=> 'task_sitemap'
			), array(
					'title'				=> 'Reminder and Invoicing',
					'description'		=> 'Send reminder and new invoicing emails to the members',
					'module'			=> 'cronjobs',
					'time_minutes'		=> 0,
					'time_hours'		=> 0,
					'time_week_day'		=> 0,
					'time_month_day'	=> 0,
					'enabled'			=> 1,
					'method_name'		=> 'task_invoicing'
			), array(
					'title'				=> 'Listing Search - Advanced Method',
					'description'		=> 'All columns on which search is to be carried are taken from different tables and inserted into listing_search table to avoid joins to make search faster.',
					'module'			=> 'cronjobs',
					'time_minutes'		=> 0,
					'time_hours'		=> -1,
					'time_week_day'		=> 0,
					'time_month_day'	=> 0,
					'enabled'			=> 1,
					'method_name'		=> 'task_listing_search'
			)
	);

	public function up() 
	{
		$this->load->dbforge();
		
		if (!$this->db->table_exists('cronjobs'))
		{
		
			// Cronjobs table
			$this->dbforge->add_field($this->fields);
			$this->dbforge->add_key('id', true);
			$this->dbforge->create_table('cronjobs');
			
			// Permissions
			$this->db->insert('permissions', array(
					'name'			=> 'Cron.Settings.View',
					'description'	=> 'Allow user to view and manage cronjobs.',
					'status'		=> 'active'
				)
			);
			$this->db->insert('role_permissions', array(
					'role_id'		=> 1,
					'permission_id'	=> $this->db->insert_id()
				)
			);
	
			// Default cronjobs
			$this->load->model('cronjobs/cronjob_model');
			foreach ($this->cronjobs as $cronjob) {
				$this->cronjob_model->insert($cronjob);
			}
			
		}
	}
	
	//--------------------------------------------------------------------
	
	public function down() {
		$this->dbforge->drop_table('cronjobs');
	}
	
	//--------------------------------------------------------------------
	
}