<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Cronjob_model extends BF_Model {

    protected $table_name		= 'cronjobs';
    protected $table_background = 'background_processes';
	protected $key			= 'id';
	protected $set_created	= false;
	protected $set_modified	= false;
	protected $soft_deletes	= false;

	//---------------------------------------------------------------

	public function find_all() {
		$results = parent::find_all();
		if ($results) {
			foreach ($results as $key => &$result) {
				$result->next_run = $this->calc_next_run($result);
			}
		}		
		return $results;
	}
	
	//--------------------------------------------------------------------
	
	/**
	 * Calculate Next Run Time using Cron-Expression
	 * @param object $task
	 * @param string $run_time
	 * @return number
	 */
	public function calc_next_run($task) {		
		$cron = Cron\CronExpression::factory("$task->time_minutes $task->time_hours $task->time_month_day $task->time_month $task->time_week_day");
		$cron->isDue();
		return strtotime($cron->getNextRunDate()->format('Y-m-d H:i:s'));
	}
	
	/**
	 * Calculate PreviousRunDate Using Cron-Expression for scheduled time
	 * @param object $task
	 */
	public function calc_prev_run($task) {
		$cron = Cron\CronExpression::factory("$task->time_minutes $task->time_hours $task->time_month_day $task->time_month $task->time_week_day");
		$cron->isDue();
		return strtotime($cron->getPreviousRunDate()->format('Y-M-d H:i:s'));
	}
	
	//--------------------------------------------------------------------
	// !Private Methods
	//--------------------------------------------------------------------
	//--------------------------------------------------------------------
	
	/*
		Method: check_times()
		
		Converts cron-based times to real world times.
		
		Parameters:
			$task		- A pointer to the task object that contains the time values.
			$run_time	- A UNIX timestamp.
	*/
	private function check_times(&$task, $run_time) 
	{ 
		
	}
	
	/**
	 * Invoice Reminder List
	 * @param string $reminder_type [ first | second | third ]
	 * @return array
	 */
	public function reminder_list($reminder_type) {
		$sql = "SELECT INV.prefix as invoice_prefix, INV.id as invoice_id, INV.created_on, INV.status as invoice_status, INV.date_paid, O.order_id, O.user_email, O.order_subtotal,";
		$sql .= " OT.expires_on, OC.name as customer_name, OC.phone_number, OC.address,";
		$sql .= " OC.postcode, OC.city_id, CONCAT(C.name, ', ', S.name) AS city, OC.email FROM";
		$sql .= " (SELECT * FROM " . $this->db->dbprefix . "invoices WHERE status = 0 AND ". $reminder_type ."_reminder = '" . date('Y-m-d') . "' AND (date_paid = '0000-00-00 00:00:00' OR date_paid IS NULL)) INV";
		$sql .= " LEFT JOIN " . $this->db->dbprefix . "orders O ON O.order_id = INV.order_id";
		$sql .= " LEFT JOIN " . $this->db->dbprefix . "order_customers OC ON OC.order_id = INV.order_id";
		$sql .= " LEFT JOIN " . $this->db->dbprefix . "order_items OT ON OT.order_id = O.order_id";
		$sql .= " LEFT JOIN " . $this->db->dbprefix . "cities C ON C.id = OC.city_id";
		$sql .= " LEFT JOIN " . $this->db->dbprefix . "states S ON S.id = C.state_id";
		$query = $this->db->query ( $sql );
		$result = $query->result_array ();
		return $result;
	}
	
	/**
	 * Invoice Generated List
	 * @param string $reminder_type [ first | second | third ]
	 * @return array
	 */
	public function invoice_list($invoice_id) {
		$sql = "SELECT INV.prefix as invoice_prefix, INV.id as invoice_id, INV.created_on, INV.status as invoice_status, O.user_email, O.order_type, O.order_subtotal,";
		$sql .= " O.expires_on, P.title as package_title, P.description as package_description, OC.name as customer_name, OC.phone_number, OC.address,";
		$sql .= " OC.postcode, OC.city_id, CONCAT(C.name, ', ', S.name) AS city, OC.email FROM";
		$sql .= " (SELECT * FROM " . $this->db->dbprefix . "invoices WHERE status = 0 AND id = '" . $invoice_id . "') INV";
		$sql .= " LEFT JOIN " . $this->db->dbprefix . "orders O ON O.order_id = INV.order_id";
		$sql .= " LEFT JOIN " . $this->db->dbprefix . "order_customers OC ON OC.order_id = INV.order_id";
		$sql .= " LEFT JOIN " . $this->db->dbprefix . "packages P ON P.id = O.package_id";
		$sql .= " LEFT JOIN " . $this->db->dbprefix . "cities C ON C.id = OC.city_id";
		$sql .= " LEFT JOIN " . $this->db->dbprefix . "states S ON S.id = C.state_id";
		$query = $this->db->query ( $sql );
		$result = $query->result_array ();
		foreach($result as $invoice) {
			return $invoice;
		}
	}
	
	public function calculate_time_span($run_time, $date){
		$seconds  = abs($run_time - $date);
		return round($seconds /60);
		
		/*$months = floor($seconds / (3600*24*30));
		$day = floor($seconds / (3600*24));
		$hours = floor($seconds / 3600);
		$mins = floor(($seconds - ($hours*3600)) / 60);
		return $mins;
	
		/*$months = floor($seconds / (3600*24*30));
		$day = floor($seconds / (3600*24));
		$hours = floor($seconds / 3600);
		$mins = floor(($seconds - ($hours*3600)) / 60);
		$secs = floor($seconds % 60);
	
		if($seconds < 60)
			$time = $secs." seconds ago";
		else if($seconds < 60*60 )
			$time = $mins." min ago";
		else if($seconds < 24*60*60)
			$time = $hours." hours ago";
		else if($seconds < 24*60*60)
			$time = $day." day ago";
		else
			$time = $months." month ago";
	
		return $time;*/
	}
	
	/************************************************************************
	 * START - UPDATE VERSION FILES
	 ************************************************************************/
	
	/**
	 * Prepare files after encryption
	 * Run this method after encrypting the files
	 */
	public function prepare_files() {
	    $published_folder = $this->destination_folder . 'goclixy_bdsv' . $this->version . '/';
	    $updated_files_folder = $this->destination_folder . $this->version . '/';
	    $php_versions = array('5.6/', '7.1/');
	    $versions = $this->versions_model->find_all();
	    if($versions) {
	        foreach($versions as $version) {
	            $file_path = $version->folder_root . $version->folder . $version->file;
	            $destination_paths = array();
	            foreach($php_versions as $php_version) {
	                if($version->encrypted == 1) {
	                    $source_path = $this->destination_folder . 'encrypted/' . $php_version . $file_path;
	                    // copy to cronjobs updated tool folders
	                    $destination_paths = array($updated_files_folder . $php_version . $file_path);
	                    if($php_version == '7.1/') {
	                        $destination_paths[] = $published_folder . 'upload/' . $file_path;
	                    } else {
	                        $destination_paths[] = $published_folder . 'phpv5.6/' . $file_path;
	                    }
	                    $this->copy_files_to_folders($source_path, $destination_paths);
	                } else { // copy files directly from the ci3 directory
	                    $source_path = $this->source_folder . $file_path;
	                    // copy to cronjobs updated tool folders
	                    $destination_paths = array($updated_files_folder . $php_version . $file_path);
	                    if($php_version == '7.1/') {
	                        $destination_paths[] = $published_folder . 'upload/' . $file_path;
	                    } else if(strpos($file_path, '/language/') !== false) { // copy only language files
	                        $destination_paths[] = $published_folder . 'phpv5.6/' . $file_path;
	                    }
	                    $this->copy_files_to_folders($source_path, $destination_paths);
	                }
	            }
	        }
	        // create zip archieve of updated_files_folder
	        create_zip($updated_files_folder . '5.6/', $updated_files_folder . '5.6.zip');
	        create_zip($updated_files_folder . '7.1/', $updated_files_folder . '7.1.zip');
	        
	        // upload them to server
	        $this->upload_files($updated_files_folder . '5.6.zip', '5.6.zip');
	        $this->upload_files($updated_files_folder . '7.1.zip', '7.1.zip');
	    }
	}
	
	/**
	 * Copy Files to Folders
	 * @param string $source_path
	 * @paramm array $destination_paths
	 */
	public function copy_files_to_folders($source_path, $destination_paths) {
	    foreach($destination_paths as $destination_path) {
	        $this->file_copy($source_path, $destination_path);
	    }
	}
	
	/**
	 * Copy file if destination does not exist
	 * @param string $source
	 * @param string $destination
	 */
	public function file_copy($source, $destination) {
	    $path = pathinfo($destination);
	    if (!file_exists($path['dirname'])) {
	        mkdir($path['dirname'], 0777, true);
	    }
	    if (!copy($source,$destination)) {
	        echo "copy failed \n";
	    }
	}
	
	/**
	 * Notes: do not update a website with cronjobs if it is a customized one
	 * Best is to give user an option to update.
	 * Handle this with the help of a variable in the settings table.
	 * @param string $zipFile - Path of the Zip File
     * @param string $file_url - Online File Url
	 * @param string $delimiter - Name of the Zip File
	 * @param boolean $import_db -> TRUE - Also import DB changes, in case of Languages import, set it to FALSE
	 */
	public function update_files($zipFile, $file_url, $delimiter, $import_db = true) {
		$this->lang->load('cronjobs/cronjobs');
		$redirect_path = current_url();//site_url(SITE_AREA . '/settings/cronjobs');		
		$headers = $this->getHeaders($file_url);
		if ($headers['http_code'] === 200) {
			if ($this->download($file_url, $zipFile)) {
			    // Extract Zip File
			    extract_zip($zipFile, $delimiter);
			    
				// Make DB Changes
				if($import_db) {
				    $this->load->library('cronjobs/versions_db');
				    $this->versions_db->run();
				}
				
				Template::set_message(lang('uvf_success_update'), 'success');
				//Template::redirect($redirect_path);
			} else {
				Template::set_message(lang('msg_nothing_to_update'), 'danger');
			}
		} else {
			Template::set_message(lang('msg_nothing_to_update'), 'danger');
		}
	}
	
	/**
	 * http://www.w3bees.com/2013/09/download-file-from-remote-server-with.html
	 * This function gets all header information before downloading a file.
	 * This function will help you to know file is exist or not and filesize.
	 * @param str #url
	 * @return array
	 */
	private function getHeaders($url) {
		$ch = curl_init($url);
		curl_setopt( $ch, CURLOPT_NOBODY, true );
		curl_setopt( $ch, CURLOPT_RETURNTRANSFER, false );
		curl_setopt( $ch, CURLOPT_HEADER, false );
		curl_setopt( $ch, CURLOPT_FOLLOWLOCATION, true );
		curl_setopt( $ch, CURLOPT_MAXREDIRS, 3 );
		curl_exec( $ch );
		$headers = curl_getinfo( $ch );
		curl_close( $ch );
	
		return $headers;
	}
	
	/**
	 * Download a file from the remote server with PHP
	 * @param string $url, $path
	 * @return bool || void
	 */
	private function download($url, $path) {
		# open file to write
		$fp = fopen ($path, 'w+');
		# start curl
		$ch = curl_init();
		curl_setopt( $ch, CURLOPT_URL, $url );
		# set return transfer to false
		curl_setopt( $ch, CURLOPT_RETURNTRANSFER, false );
		curl_setopt( $ch, CURLOPT_BINARYTRANSFER, true );
		curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, true );
		# increase timeout to download big file
		curl_setopt( $ch, CURLOPT_CONNECTTIMEOUT, 10 );
		# write data to local file
		curl_setopt( $ch, CURLOPT_FILE, $fp );
		# https redirect issue
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
		# execute curl
		curl_exec( $ch );
		# close curl
		curl_close( $ch );
		# close local file
		fclose( $fp );
	
		if (filesize($path) > 0) return true;
	}
	
	//--------------------------------------------------------------------
	
	/**
	 * Process long and resource consuming queries in chunks
	 * This method will return id_from and id_to values only, you need to process them manually in the queries
	 *
	 * @param string $table - Name of the table on which action will be performed
	 * @param string $process - Action Type
	 * We can perform multiple operations on a single table, i.e. the process variable will help to keep them separately
	 *
	 * @param number $limit - Number of records to be processed at a time
	 * @param boolean $is_repeat - TRUE - Cronjob task, process from the beginning, FALSE - Task is complete, do not process further
	 * @return array[]
	 *
	 *
	 * USAGE:
	 * $this->load->model('cronjobs/cronjob_model');
    	 $table = 'categories';
    	 $process = 'category_counts';
    	 $limit = 10;
    	 // do not pass the is_repeat to TRUE if you want to make a recursive call
    	 $range = $this->cronjob_model->background_process($table, $process, $limit);
    	 // $this->cronjob_model->drop_background_processes(); // drop the background_processes table
    	 if($range['success'] == 1) {
        	 $id_from = $range['id_from'];
        	 $id_to = $range['id_to'];
        	 // perform the operation
        	 
        	 // call the background_completion method
        	 $this->cronjob_model->background_completion($table, $process, $id_from, $id_to, $range['date'], $limit);
        	 // $this->bulk(); // call until the operation is complete, bulk is the name of the method for recursive call
    	 }
	 */
	public function background_process($table, $process, $limit = 20, $is_repeat = false) {
	    // set the initial values
	    $id_from = 1;
	    $id_to = $limit;
	    $date = date('Y-m-d H:i:s');
	    $data = array('table' => $table, 'process' => $process, 'id_from' => $id_from, 'id_to' => $id_to, 'created_on' => $date);
	    // create background_process table if not exist
	    if(!$this->db->table_exists($this->table_background)) {
	        $this->load->dbforge();
	        $field_migrate_db = array(
	            'id' => array(
	                'type' => 'INT',
	                'constraint' => 11,
	                'auto_increment' => TRUE,
	            ),
	            'table' => array(
	                'type' => 'VARCHAR',
	                'constraint' => 100,
	                'null' => FALSE
	            ),
	            'process' => array(
	                'type' => 'VARCHAR',
	                'constraint' => 100,
	                'null' => FALSE
	            ),
	            'id_from' => array(
	                'type' => 'INT',
	                'constraint' => 11,
	                'null' => FALSE,
	                'default'=> '0'
	            ),
	            'id_to' => array(
	                'type' => 'INT',
	                'constraint' => 11,
	                'null' => FALSE,
	                'default' => 1
	            ),
	            'status' => array(
	                'type' => 'TINYINT',
	                'constraint' => 1,
	                'null' => FALSE,
	                'default' => 0
	            ),
	            'created_on' => array(
	                'type' => 'DATETIME',
	                'null' => TRUE
	            ),
	            'modified_on' => array(
	                'type' => 'DATETIME',
	                'null' => TRUE
	            )
	        );
	        $this->dbforge->add_field($field_migrate_db);
	        $this->dbforge->add_key('id', true);
	        $this->dbforge->add_key('created_on', false);
	        $this->dbforge->create_table($this->table_background);
	        $unique_index = "ALTER TABLE " . $this->db->dbprefix . $this->table_background . " ADD UNIQUE INDEX `idx_unique` (`table`,`process`,`id_from`,`id_to`,`created_on`)";
	        $this->db->query ($unique_index);
	        return $this->background_insert($data);
	    }
	    // check status
	    $history = $this->db->where(array('table' => $table, 'process' => $process))->order_by('created_on', 'DESC')->get($this->table_background)->row();
	    if($history) {
	        $i = 1;
	        if($history->status == 1) {
	            if(!$is_repeat) {
	                // do not proceed with this task further
	                return array('success' => 0);
	            } else { // add new row and start from the beginning
	                return $this->background_insert($data);
	            }
	        } else {
	            return array('success' => 1, 'id_from' => $history->id_from, 'id_to' => $history->id_to, 'date' => $history->created_on);
	        }
	    } else {
	        return $this->background_insert($data);
	    }
	}
	
	/**
	 * Insert information in the background_processes table.
	 * @param array $data
	 * @return array[]
	 */
	private function background_insert($data) {
	    // insert information in the background_process table
	    $this->db->insert($this->table_background, $data);
	    
	    // do not process further, return the from and to field values
	    return array('success' => 1, 'id_from' => $data['id_from'], 'id_to' => $data['id_to'], 'date' => $data['created_on']);
	}
	
	/**
	 * Set status status in the background process table
	 * Do not forget to call this method on process completion
	 */
	public function background_completion($table, $process, $id_from, $id_to, $date, $limit, $field = 'id') {
	    $data = array('table' => $table, 'process' => $process, 'id_from' => $id_from, 'id_to' => $id_to, 'created_on' => $date);
	    $record = $this->db->where($data)->get($this->table_background)->row();
	    if($record) {
	        // rows processed, set the status to 1
	        $this->db->where('id', $record->id)->update($this->table_background, array('status' => 1, 'modified_on' => date('Y-m-d H:i:s')));
	        $count = $this->db->where($field . ' >', $id_to)->count_all_results($table);
	        if($count > 0) {
	            // insert rows to be processed in future
	            $data['id_from'] = $record->id_to;
	            $data['id_to'] = $record->id_to + $limit;
	            $data['created_on'] = date('Y-m-d H:i:s');
	            $this->db->insert($this->table_background, $data);
	        }
	    }
	}
	
	/**
	 * Drop the background_processes table
	 */
	public function drop_background_processes() {
	    $this->load->dbforge();
	    $this->dbforge->drop_table($this->table_background, true);
	}
}