<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Online Software Solutions							   # ||
|| # Copyright (C) 2021 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Currency controller
 */
class Financial extends Admin_Controller
{
	protected $permissionCreate = 'Currencies.Financial.Create';
	protected $permissionDelete = 'Currencies.Financial.Delete';
	protected $permissionEdit   = 'Currencies.Financial.Edit';
	protected $permissionView   = 'Currencies.Financial.View';
	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct() {
		parent::__construct();
        clear_cache('currencies');
		$this->auth->restrict($this->permissionView);
		$this->load->model('currencies_model', null, true);
		$this->lang->load('currencies');
		Template::set_block('sub_nav', 'financial/_sub_nav');
	}
	
	/**
	 * Manage Currencies
	 * @param string $filter
	 * @param number $offset
	 */
	public function index($filter = 'all', $offset = 0) {
	    $this->manage('currencies', $filter = 'all', $offset = 0);
	}
	
	/**
	 * Manage Currencies
	 * @param string $module
	 * @param string $filter
	 * @param number $offset
	 */
	public function manage($module = 'currencies', $filter = 'all', $offset = 0) {	    
	    // Update Rate
	    if (isset($_POST['update_rate'])) {	        
	        $this->gcrud->stats(array('module' => 'currencies'));
	        $to_currency = $_POST['update_rate'];
			if(is_numeric($to_currency)) {
				$currency = $this->gc->get('currencies', array('where' => 'id = ' . $to_currency));
				if($currency) {
					$to_currency = $currency->code;
				}
			}
	        $this->load->library('CurrencyConverter');
	        $this->CurrencyConverter = new CurrencyConverter();
	        $rate = $this->CurrencyConverter->_getRates('USD', $to_currency);
	        if($rate != 0) {
	            $this->db->where('code', $to_currency)->update('currencies', array('rate_usd' => $rate, 'modified_on' => date('Y-m-d H:i:s')));
	            Template::set_message(sprintf(lang('msg_action_success'), lang('label_currency'), lang('label_updated')), 'success');
	        } else {
	            Template::set_message(sprintf(lang('msg_edit_failure'), lang('label_currency')), 'danger');
	        }
	        Template::redirect(current_url());
	    }
	    $extra = array(
	        'model' => 'currencies',
	        'context' => 'financial',
	        'fields' => null
	    );
	    $extra['actions'] = array( // Extra buttons
	        array('type' => 'link', 'link' => site_url(SITE_AREA . '/financial/currencies/form/' . $module . '/edit/%s'), 'label' => '<i class="fas fa-edit fa-fw mr-1"></i>' . lang('edit'), 'class' => 'btn btn-xs btn-primary'),
	        array('type' => 'button', 'class' => 'btn-info', 'name' => 'update_rate', 'label' => '<i class="fas fa-sync fa-fw mr-1"></i>' . lang('update_rates')),
	    );
	    $this->gc->module_view($module, $filter, $offset, 'admin', $extra);
	}
	
	/**
	 * Add/Edit Currencies Data
	 *
	 * @param string $module
	 * @param string $action
	 *
	 * @return void
	 */
	public function form($module = 'currencies', $action = 'create') {
	    $params = array(
	        'model' => $module,
	        'context' => 'financial',
	        'redirect_url' => site_url(SITE_AREA . '/financial/currencies/manage/' . $module)
	    );
	    $this->gcrud->form($module, $action, 'admin', $params);
	}
	
	/**
	 * Update Rates for all currencies
	 */
	public function update_rates() {
	    if(isset($_POST['submit'])) {
	        $this->currencies_model->update_rates();
	    }
	    Template::set('toolbar_title', lang('gc_action_update_rates'));
	    Template::render();
	}
	
	/**
	 * Convert amount to a specific currency
	 * @param string $to
	 * @param float $amount
	 * @param string $from
	 * @return float|number
	 */
	private function convert($to, $amount, $from = null, $return_rate = false) {
	    $this->load->library('CurrencyConverter');
	    echo $this->currencyconverter->amount_conversion($to, $amount, $from, $return_rate);
	}
	
	/**
	 * Update currencies information in the countries table
	 */
	private function update_currency() {
	    $this->currencies_model->country_currencies();
	}
	
	/**
	 * Update rates online
	 */
	public function update_rates_online() {
	    $result = $this->_rates();
	    if(!empty($result['usd'])) {
	        $result = $result['usd'];
    	    $records = $this->db->select('id, code, rate_usd')->get('currencies')->result();
    	    if($records) {
    	        $update_data = array();
    	        foreach($records as $record) {
    	            $code = strtolower($record->code);
    	            if(!empty($result[$code])) {
    	               $update_data[] = array(
    	                   'id' => $record->id,
    	                   'rate_usd' => $result[$code]
    	               );
    	            }
    	        }
    	        if(!empty($update_data)) {
    	            $this->db->update_batch('currencies', $update_data, 'id');
    	        }
    	    }
	    }
	}
	
	/**
	 * Get Yesterday Rates
	 * @return array
	 */
	private function _rates() {
	    $this->load->library('goclixy/gcache');
	    $date = date('Y-m-d',strtotime("-1 days"));
	    $url = 'https://cdn.jsdelivr.net/gh/fawazahmed0/currency-api@1/' . $date . '/currencies/usd.json';
	    // $result = file_get_contents($url);
	    // dump(json_decode($result));
	    $result = $this->gcache->curl_data(null, null, $url, true);
	    return $result;
	}
	
	/**
	 * Update currencies information in the countries table
	 */
	private function update_countries_currency_rates() {
	    $this->currencies_model->country_currencies();
	    $currencies = array();
	    $records = $this->db->get('currencies')->result();
	    if($records) {
	        foreach($records as $record) {
	            $currencies[$record->id] = $record;
	        }
	    }
	    $countries = $this->db->select('id, currency_code')->get('countries')->result();
	    if($countries) {
	        $countries_data = array();
	        foreach($countries as $country) {
	            if(!empty($country->currency_code) && !empty($currencies[$country->currency_code])) {
	                $currency_info = $currencies[$country->currency_code];
	                $countries_data[] = array(
	                    'id' => $country->id,
	                    'currency_info' => json_encode($currency_info)
	                );
	            }
	        }
	        if(!empty($countries_data)) {
	            $this->db->update_batch('countries', $countries_data, 'id');
	        }
	    }
	}
}