<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Migration_Install_currencies extends Migration
{
	/**
	 * The name of the database table
	 *
	 * @var String
	 */
	private $table_name = 'currencies';
	private $currency_rates_table = 'currency_rates';

	/**
	 * The table's fields
	 *
	 * @var Array
	 */
	private $fields = array(
		'id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'auto_increment' => TRUE,
		),
		'code' => array(
			'type' => 'CHAR',
			'constraint' => 3,
			'null' => FALSE,
		),
		'symbol' => array(
			'type' => 'VARCHAR',
			'constraint' => 40,
			'null' => FALSE,
		),
		'name' => array(
			'type' => 'VARCHAR',
			'constraint' => 80,
			'null' => FALSE,
		),
		'active' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
		),
	);
	
	/**
	 * The currency_rates table's fields
	 *
	 * @var Array
	 */
	private $currency_rates_fields = array(
		'id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'unsigned' => TRUE,
			'auto_increment' => TRUE,
		),
		'currency_from' => array(
			'type' => 'VARCHAR',
			'constraint' => 5,
			'null' => FALSE,
		),
		'currency_to' => array(
			'type' => 'VARCHAR',
			'constraint' => 5,
			'null' => FALSE,
		),
		'rates' => array(
			'type' => 'VARCHAR',
			'constraint' => 10,
			'null' => FALSE,
		),
		'created' => array(
			'type' => 'DATETIME',
			'null' => FALSE,
		),
		'modified' => array(
			'type' => 'DATETIME',
			'null' => FALSE,
		),
	);

	/**
	 * Install this migration
	 *
	 * @return void
	 */
	public function up()
	{
		$this->dbforge->add_field($this->fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->create_table($this->table_name);
		
		// Currency Rates Table
		$this->dbforge->add_field($this->currency_rates_fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->create_table($this->currency_rates_table);
	}

	//--------------------------------------------------------------------

	/**
	 * Uninstall this migration
	 *
	 * @return void
	 */
	public function down()
	{
		if ($this->db->table_exists($this->table_name)) {
			$this->dbforge->drop_table($this->table_name);
		}
		if ($this->db->table_exists($this->currency_rates_table)) {
			$this->dbforge->drop_table($this->currency_rates_table);
		}
	}

	//--------------------------------------------------------------------

}