<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Online Software Solutions							   # ||
|| # Copyright (C) 2021 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class Currencies_model extends BF_Model {

	public $table_name	= "currencies";
	protected $key			= "id";
	protected $soft_deletes	= false;
	protected $date_format	= "datetime";

	protected $log_user 	= FALSE;

	protected $set_created	= false;
	protected $set_modified = false;

	/*
		Customize the operations of the model without recreating the insert, update,
		etc methods by adding the method names to act as callbacks here.
	 */
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 		= array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	/*
		For performance reasons, you may require your model to NOT return the
		id of the last inserted row as it is a bit of a slow method. This is
		primarily helpful when running big loops over data.
	 */
	protected $return_insert_id 	= TRUE;

	// The default type of element data is returned as.
	protected $return_type 			= "object";

	// Items that are always removed from data arrays prior to
	// any inserts or updates.
	protected $protected_attributes = array();

	/*
		You may need to move certain rules (like required) into the
		$insert_validation_rules array and out of the standard validation array.
		That way it is only required during inserts, not updates which may only
		be updating a portion of the data.
	 */
	protected $validation_rules 		= array();
	protected $insert_validation_rules 	= array();
	protected $skip_validation 			= FALSE;
	protected $crz_cache;
	protected $CurrencyConverter;
	
	/**
	 * Constructor
	 */
	public function __construct() {	    
	    $this->load->library('gzf_cache', array('lifetime' => 43200)); // 12 hours storage
	    $this->load->library('slug');
	    if(method_exists($this->gzf_cache, 'get_instance')) {
	        $this->crz_cache = $this->gzf_cache->get_instance();
	    }
	}
	
	/**
	 * Currencies Fields
	 *
	 * @return array
	 */
	public function fields() {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'code',
	            'label' => 'label_code',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'edit' => 1,
	            'class' => 2,
	            'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
	            'rules' => 'trim|required|unique[currencies.code,currencies.id]|alpha_extra|max_length[3]',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'symbol',
	            'label' => 'label_symbol',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'class' => 2,
	            'rules' => 'trim|required|max_length[40]',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'name',
	            'label' => 'label_name',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'class' => 4,
	            'rules' => 'trim|required|max_length[80]',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'rate_usd',
	            'label' => 'label_rate_usd',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'class' => 2,
	            'rules' => 'trim|required|is_numeric',
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'decimal_format',
	            'label' => 'label_decimal_format',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'options' => 'name:label_dot;value:.|name:label_comma;value:,',
	            'options_detailed' => 1,
	            'options_lang' => 1,
	            'class' => 2,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'display_format',
	            'label' => 'label_display_format',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'options' => 'name:label_currency_price;value:%s %d|name:label_price_currency;value:%d %s',
	            'options_detailed' => 1,
	            'options_lang' => 1,
	            'class' => 2,
	            'rules' => 'trim|max_length[10]',
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'status',
	            'label' => 'label_status',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_on',
	            'label' => 'created_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_on',
	            'label' => 'modified_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        
	    );
	    return $fields;
	}
	
	/**
	 * Get and store currency info in a session variable for symbol and display format
	 * @param string $currency_code
	 * @param string $session_name - Name of the session variable
	 */
	public function currency_info($currency_code = null, $session_name = 'site_currency', $where_column = 'code') {
	    if(empty($currency_code)) {
	        $currency_code = settings_item('site.currency_code');
	    }
	    $currency = $this->get_currency($currency_code, $where_column);
	    if($currency) {
	        $this->session->set_userdata($session_name, (array) $currency);
	    } else {
	        $this->session->unset_userdata($session_name);
	    }
	}
	
	/**
	 * Get Currency Information
	 * @return string
	 */
	public function get_currency($currency_code, $where_column = 'code') {	
	    $cacheID = $currency_code . '_' . $where_column;
	    if(!$currency = $this->crz_cache->load($cacheID)) {
	       $currency = $this->db->select('code, symbol, rate_usd, decimal_format, display_format')->where($where_column, $currency_code)->get('currencies')->row();
	       $this->crz_cache->save($currency, $cacheID, array('currencies'));
	    }
	    return $currency;
	}
	
	/**
	 * Update Rates
	 * @param array $ids
	 */
	public function update_rates($ids = array()) {
	    $this->load->library('CurrencyConverter');
	    $this->CurrencyConverter = new CurrencyConverter();
	    if(!empty($ids)) {
	        $this->db->where('id IN (' . implode(',', $ids) . ')');
	    }
	    $records = $this->db->where("DATE(modified_on) != '" . date('Y-m-d') . "'")->get('currencies')->result();
	    if($records) {
	        $data = array();
	        foreach($records as $record) {
	            $rate = $this->CurrencyConverter->_getRates('USD', $record->code);
	            if($rate != 0) {
	                $data[] = array(
	                    'id' => $record->id,
	                    'rate_usd' => $rate,
	                    'modified_on' => date('Y-m-d')
	                );
	            }
	        }
	        if(!empty($data)) {
	            $this->db->update_batch('currencies', $data, 'id');
	            Template::set_message(sprintf(lang('msg_action_success'), lang('label_currency'), lang('label_updated')), 'success');
	        } else {
	            Template::set_message(lang('msg_nothing_to_update'), 'danger');
	        }
	    } else {
	        Template::set_message(lang('msg_nothing_to_update'), 'danger');
	    }
	    Template::redirect(current_url());
	}
	
	/**
	 * Country Currencies Information Update
	 */
	public function country_currencies() {
	    $currencies = array(
	        'AF' => 'AFN',
	        'AL' => 'ALL',
	        'DZ' => 'DZD',
	        'AS' => 'USD',
	        'AD' => 'EUR',
	        'AO' => 'AOA',
	        'AI' => 'XCD',
	        'AQ' => 'XCD',
	        'AG' => 'XCD',
	        'AR' => 'ARS',
	        'AM' => 'AMD',
	        'AW' => 'AWG',
	        'AU' => 'AUD',
	        'AT' => 'EUR',
	        'AZ' => 'AZN',
	        'BS' => 'BSD',
	        'BH' => 'BHD',
	        'BD' => 'BDT',
	        'BB' => 'BBD',
	        'BY' => 'BYR',
	        'BE' => 'EUR',
	        'BZ' => 'BZD',
	        'BJ' => 'XOF',
	        'BM' => 'BMD',
	        'BT' => 'BTN',
	        'BO' => 'BOB',
	        'BA' => 'BAM',
	        'BW' => 'BWP',
	        'BV' => 'NOK',
	        'BR' => 'BRL',
	        'IO' => 'USD',
	        'BN' => 'BND',
	        'BG' => 'BGN',
	        'BF' => 'XOF',
	        'BI' => 'BIF',
	        'KH' => 'KHR',
	        'CM' => 'XAF',
	        'CA' => 'CAD',
	        'CV' => 'CVE',
	        'KY' => 'KYD',
	        'CF' => 'XAF',
	        'TD' => 'XAF',
	        'CL' => 'CLP',
	        'CN' => 'CNY',
	        'HK' => 'HKD',
	        'CX' => 'AUD',
	        'CC' => 'AUD',
	        'CO' => 'COP',
	        'KM' => 'KMF',
	        'CG' => 'XAF',
	        'CD' => 'CDF',
	        'CK' => 'NZD',
	        'CR' => 'CRC',
	        'HR' => 'HRK',
	        'CU' => 'CUP',
	        'CY' => 'EUR',
	        'CZ' => 'CZK',
	        'DK' => 'DKK',
	        'DJ' => 'DJF',
	        'DM' => 'XCD',
	        'DO' => 'DOP',
	        'EC' => 'ECS',
	        'EG' => 'EGP',
	        'SV' => 'SVC',
	        'GQ' => 'XAF',
	        'ER' => 'ERN',
	        'EE' => 'EUR',
	        'ET' => 'ETB',
	        'FK' => 'FKP',
	        'FO' => 'DKK',
	        'FJ' => 'FJD',
	        'FI' => 'EUR',
	        'FR' => 'EUR',
	        'GF' => 'EUR',
	        'TF' => 'EUR',
	        'GA' => 'XAF',
	        'GM' => 'GMD',
	        'GE' => 'GEL',
	        'DE' => 'EUR',
	        'GH' => 'GHS',
	        'GI' => 'GIP',
	        'GR' => 'EUR',
	        'GL' => 'DKK',
	        'GD' => 'XCD',
	        'GP' => 'EUR',
	        'GU' => 'USD',
	        'GT' => 'QTQ',
	        'GG' => 'GGP',
	        'GN' => 'GNF',
	        'GW' => 'GWP',
	        'GY' => 'GYD',
	        'HT' => 'HTG',
	        'HM' => 'AUD',
	        'HN' => 'HNL',
	        'HU' => 'HUF',
	        'IS' => 'ISK',
	        'IN' => 'INR',
	        'ID' => 'IDR',
	        'IR' => 'IRR',
	        'IQ' => 'IQD',
	        'IE' => 'EUR',
	        'IM' => 'GBP',
	        'IL' => 'ILS',
	        'IT' => 'EUR',
	        'JM' => 'JMD',
	        'JP' => 'JPY',
	        'JE' => 'GBP',
	        'JO' => 'JOD',
	        'KZ' => 'KZT',
	        'KE' => 'KES',
	        'KI' => 'AUD',
	        'KP' => 'KPW',
	        'KR' => 'KRW',
	        'KW' => 'KWD',
	        'KG' => 'KGS',
	        'LA' => 'LAK',
	        'LV' => 'EUR',
	        'LB' => 'LBP',
	        'LS' => 'LSL',
	        'LR' => 'LRD',
	        'LY' => 'LYD',
	        'LI' => 'CHF',
	        'LT' => 'EUR',
	        'LU' => 'EUR',
	        'MK' => 'MKD',
	        'MG' => 'MGF',
	        'MW' => 'MWK',
	        'MY' => 'MYR',
	        'MV' => 'MVR',
	        'ML' => 'XOF',
	        'MT' => 'EUR',
	        'MH' => 'USD',
	        'MQ' => 'EUR',
	        'MR' => 'MRO',
	        'MU' => 'MUR',
	        'YT' => 'EUR',
	        'MX' => 'MXN',
	        'FM' => 'USD',
	        'MD' => 'MDL',
	        'MC' => 'EUR',
	        'MN' => 'MNT',
	        'ME' => 'EUR',
	        'MS' => 'XCD',
	        'MA' => 'MAD',
	        'MZ' => 'MZN',
	        'MM' => 'MMK',
	        'NA' => 'NAD',
	        'NR' => 'AUD',
	        'NP' => 'NPR',
	        'NL' => 'EUR',
	        'AN' => 'ANG',
	        'NC' => 'XPF',
	        'NZ' => 'NZD',
	        'NI' => 'NIO',
	        'NE' => 'XOF',
	        'NG' => 'NGN',
	        'NU' => 'NZD',
	        'NF' => 'AUD',
	        'MP' => 'USD',
	        'NO' => 'NOK',
	        'OM' => 'OMR',
	        'PK' => 'PKR',
	        'PW' => 'USD',
	        'PA' => 'PAB',
	        'PG' => 'PGK',
	        'PY' => 'PYG',
	        'PE' => 'PEN',
	        'PH' => 'PHP',
	        'PN' => 'NZD',
	        'PL' => 'PLN',
	        'PT' => 'EUR',
	        'PR' => 'USD',
	        'QA' => 'QAR',
	        'RE' => 'EUR',
	        'RO' => 'RON',
	        'RU' => 'RUB',
	        'RW' => 'RWF',
	        'SH' => 'SHP',
	        'KN' => 'XCD',
	        'LC' => 'XCD',
	        'PM' => 'EUR',
	        'VC' => 'XCD',
	        'WS' => 'WST',
	        'SM' => 'EUR',
	        'ST' => 'STD',
	        'SA' => 'SAR',
	        'SN' => 'XOF',
	        'RS' => 'RSD',
	        'SC' => 'SCR',
	        'SL' => 'SLL',
	        'SG' => 'SGD',
	        'SK' => 'EUR',
	        'SI' => 'EUR',
	        'SB' => 'SBD',
	        'SO' => 'SOS',
	        'ZA' => 'ZAR',
	        'GS' => 'GBP',
	        'SS' => 'SSP',
	        'ES' => 'EUR',
	        'LK' => 'LKR',
	        'SD' => 'SDG',
	        'SR' => 'SRD',
	        'SJ' => 'NOK',
	        'SZ' => 'SZL',
	        'SE' => 'SEK',
	        'CH' => 'CHF',
	        'SY' => 'SYP',
	        'TW' => 'TWD',
	        'TJ' => 'TJS',
	        'TZ' => 'TZS',
	        'TH' => 'THB',
	        'TG' => 'XOF',
	        'TK' => 'NZD',
	        'TO' => 'TOP',
	        'TT' => 'TTD',
	        'TN' => 'TND',
	        'TR' => 'TRY',
	        'TM' => 'TMT',
	        'TC' => 'USD',
	        'TV' => 'AUD',
	        'UG' => 'UGX',
	        'UA' => 'UAH',
	        'AE' => 'AED',
	        'GB' => 'GBP',
	        'US' => 'USD',
	        'UM' => 'USD',
	        'UY' => 'UYU',
	        'UZ' => 'UZS',
	        'VU' => 'VUV',
	        'VE' => 'VEF',
	        'VN' => 'VND',
	        'VI' => 'USD',
	        'WF' => 'XPF',
	        'EH' => 'MAD',
	        'YE' => 'YER',
	        'ZM' => 'ZMW',
	        'ZW' => 'ZWD',
	    );
	    foreach($currencies as $country_iso => $currency_code) {
	        $currency = $this->db->where('code', $currency_code)->get('currencies')->row();
	        if($currency) {
	            $this->db->where('iso', $country_iso)->update('countries', array('currency_code' => $currency->id));
	        }
	    }
	}
	
	/**
	 * Set Default Country/Currency on Users Login/Country Change
	 * @param string $country_iso
	 */
	public function set_country_currency($country_iso) {
	    $this->session->set_userdata('visitor_country_iso', $country_iso);
	    $this->load->model('locations/countries_model');
	    $country = $this->countries_model->_get($country_iso);
	    if(!empty($country->currency_code)) {
	        $user_currency = $this->currencies_model->get_currency($country->currency_code, 'id');
	        if($user_currency) {
	            $this->session->set_userdata('site_currency', $user_currency);
	        }
	    }
	}
	
	/**
	 * Get currencies to display them in the Navbar
	 * @return array
	 */
	public function get_currencies($active_only = 0) {	    
	    $cacheID = 'all_currencies_' . $active_only;
	    if(!$records = $this->crz_cache->load($cacheID)) {
	        if(!empty($active_only)) {
	            $this->db->where('status', 1);
	        }
	        $records = $this->db->select('name, code, symbol, rate_usd, decimal_format, display_format')->get('currencies')->result();
	        $this->crz_cache->save($records, $cacheID, array('currencies'));
	    }
	    return $records;
	}
}
