<?php defined('BASEPATH') || exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Components controller
 */
class Components extends Admin_Controller
{
    protected $permissionCreate = 'Ecommerce.Components.Create';
    protected $permissionDelete = 'Ecommerce.Components.Delete';
    protected $permissionEdit   = 'Ecommerce.Components.Edit';
    protected $permissionView   = 'Ecommerce.Components.View';
    protected $context = 'components';
    protected $moduleName = 'ecommerce';

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();        
        $this->auth->restrict($this->permissionView);
        $this->load->model('ecommerce/ecommerce_model');
        $this->lang->load('ecommerce');
      	$this->form_validation->set_error_delimiters("<span class='error'>", "</span>");        
        Template::set_block('sub_nav', 'components/_sub_nav');
        Assets::add_module_js('ecommerce', 'ecommerce.js');
    }
    
    /**
     * Display Attributes to the User
     * @param string $filter
     * @param number $offset
     */
    public function index($filter = 'all', $offset = 0) {
        $this->manage('attributes', $filter, $offset);
    }
    
    /**
     * Create a attributes object.
     *
     * @return void
     */
    public function add_attribute() {
        $this->auth->restrict($this->permissionCreate);
        
        if (isset($_POST['save'])) {            
            $this->gcrud->stats(array('module' => 'ecommerce'));
            if ($insert_id = $this->save_attributes()) {
                log_activity($this->auth->user_id(), lang('act_create_record') . ': ' . $insert_id . ' : ' . $this->input->ip_address(), 'attributes');
                Template::set_message(sprintf(lang('msg_save_success'), lang('label_attribute')), 'success');

                redirect(SITE_AREA . '/components/ecommerce/attributes');
            }

            // Not validation error
            if ( ! empty($this->ecommerce_model->error)) {
                Template::set_message(sprintf(lang('msg_create_failure'), lang('label_attribute')) . $this->ecommerce_model->error, 'error');
            }
        }

        Template::set('toolbar_title', lang('create_attributes'));
        Template::set_view('components/attributes_form');
        Template::render();
    }
    /**
     * Allows editing of attributes data.
     *
     * @return void
     */
    public function edit_attribute() {
        $id = $this->uri->segment(5);
        if (empty($id)) {
            Template::set_message(lang('invalid_id'), 'error');
            redirect(SITE_AREA . '/components/ecommerce/attributes');
        }
        
        if (isset($_POST['save'])) {
            $this->gcrud->stats(array('module' => 'ecommerce'));
            $this->auth->restrict($this->permissionEdit);

            if ($this->save_attributes('update', $id)) {
                log_activity($this->auth->user_id(), lang('act_edit_record') . ': ' . $id . ' : ' . $this->input->ip_address(), 'attributes');
                Template::set_message(sprintf(lang('msg_save_success'), lang('label_attribute')), 'success');
                Template::redirect(SITE_AREA . '/ecommerce/attributes');
            }

            // Not validation error
            if ( ! empty($this->ecommerce_model->error)) {
                Template::set_message(sprintf(lang('msg_edit_failure'), lang('label_attribute')) . $this->ecommerce_model->error, 'error');
            }
        } elseif (isset($_POST['delete'])) {
            $this->gcrud->stats(array('module' => 'ecommerce'));
            $this->auth->restrict($this->permissionDelete);

            if ($this->ecommerce_model->_purge_attributes($id)) {
                log_activity($this->auth->user_id(), lang('act_delete_record') . ': ' . $id . ' : ' . $this->input->ip_address(), 'attributes');
                Template::set_message(sprintf(lang('msg_delete_success'), lang('label_attribute')), 'success');
                redirect(SITE_AREA . '/ecommerce/attributes');
            }
            Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_attribute')) . $this->ecommerce_model->error, 'error');
        }
        
        $record = $this->ecommerce_model->get_attributes($id);
        $values = $this->db->get_where('attribute_values', array('attribute_id' => $record->id))->result();
        Template::set(array(
            'record' => $record,
            'values' => $values            
        ));
        Template::set('toolbar_title', lang('edit_attributes'));        
        Template::set_view('components/attributes_form');
        Template::render();
    }

    //--------------------------------------------------------------------------
    // !PRIVATE METHODS
    //--------------------------------------------------------------------------

    /**
     * Save the data.
     *
     * @param string $type Either 'insert' or 'update'.
     * @param int    $id   The ID of the record to update, ignored on inserts.
     *
     * @return boolean|integer An ID for successful inserts, true for successful
     * updates, else false.
     */
    private function save_attributes($type = 'insert', $id = 0) {
        if ($type == 'update') {
            $_POST['id'] = $id;
        }

        // Validate the data
        $this->form_validation->set_rules($this->attributes_model->get_validation_rules());
        if ($this->form_validation->run() === false) {
            return false;
        }
        
        // Make sure we only pass in the fields we want        
        $data = $this->attributes_model->prep_data($this->input->post());
        if(isset($data['values'])) {
            $_POST['values'] = $data['values'];        
            unset($data['values']);
        }

        // Additional handling for default values should be added below,
        // or in the model's prep_data() method
        $return = false;
        if ($type == 'insert') {
            $id = $this->db->insert('product_attributes', $data);
            if (is_numeric($id)) {
                $return = $id;
            }
        } elseif ($type == 'update') {
            $return = $this->db->where('id', $id)->update('product_attributes', $data);
        }

        // Do not delete existing options as they are saved in products table
        // if deleted also delete the sku from the product_units table
        if($id && !empty($_POST['values'])) {
            $this->ecommerce_model->save_values($id, $_POST['values']);
        }
        
        return $return;
    }
    
    /**
     * Display a list of variants data.
     *
     * @return void
     */
    public function variants($offset = 0) {
        $this->ecommerce_model->view('admin', $offset, $this->limit);
        Template::render();
    }
    
    /**
     * Create a variant object.
     *
     * @return void
     */
    public function add_variant($id = null) {
        $this->auth->restrict($this->permissionCreate);
        $this->ecommerce_model->variant($id, 'insert');
        Template::render();
    }
    
    /**
     * Allows editing of variants data.
     *
     * @return void
     */
    public function edit_variant() {
        $id = $this->uri->segment(5);
        $this->auth->restrict($this->permissionEdit);
        $this->ecommerce_model->variant($id, 'update');
        Template::render();
    }
    
    /**
     * Allow admin to upload photos
     * @param string $module - Name of the module e.g. variants
     * @param integer $id - ID of the Variant Row
     */
    public function photos($module, $id) {
        $redirect_url = SITE_AREA . '/components/ecommerce/manage/variants';
        $id = (int) $this->uri->segment(6);
        if(empty($id)) {
            Template::redirect(site_url($redirect_url));
        }
        $record = $this->ecommerce_model->get_variant($id);
        if(!$record) {
            Template::redirect(site_url($redirect_url));
        }
        $this->load->model('images/images_model');
        $this->images_model->upload_photos_admin($module, $record->id, $redirect_url);
    }
    
    /**
     * Bulk Generate Variants
     */
    public function bulk() {
        $this->ecommerce_model->bulk_generate();
    }
    
    /**
     * Variant Layout
     */
    public function variant_layout() {
        $variants = array(
            1 => array(
                'name' => 'Color',
                'id' => 1,
                'values' => array(
                    1 => 'blue',
                    2 => 'green'
                )
            )
        );
    }
    
    /**
     * Display a list of Upgrades/Combos/Attributes/Variants data.
     *
     * @return void
     */
    public function manage($module = 'products', $filter = 'all', $offset = 0) {
        $extra = array(
            'model' => 'ecommerce',
            'context' => 'components',
            'fields' => $module,
            'prefix' => 'product',
            'advanced_filter' => array('product_id')
        );
        if($module == 'email_templates') {
            unset($extra['advanced_filter']);
        }
        $this->gc->module_view($module, $filter, $offset, 'admin', $extra);
    }
    
    /**
     * Add/Edit Upgrades/Combos/Attributes/Variants Data
     *
     * @return void
     */
    public function form($module = 'upgrades', $action = 'create') {
        if($module != 'comments') { // conflicting with star-rating plugin
            Assets::add_js(array(
                'tinymce/tinymce.min.js',
            ));
        }
        if($module == 'variants') {
            if($action == 'create') {
                $this->auth->restrict('Product_variants.Components.Create');
                $product_variants_rows_filter = $this->session->userdata('product_variants_rows_filter');
                $this->ecommerce_model->variant($product_variants_rows_filter && !empty($product_variants_rows_filter['product_id']) ? $product_variants_rows_filter['product_id'] : $this->uri->segment(7), 'insert');
            } else {
                $this->auth->restrict('Product_variants.Components.Edit');
                $this->ecommerce_model->variant($this->uri->segment(7), 'update');
            }
            Template::render();
        } else {
            $prefix = 'product_';
            $params = array(
                'model' => 'ecommerce',
                'context' => $this->context,
                'fields_method' => $module,
                'set_table' => true,
                'uri_segment' => 7,
                'redirect_url' => site_url(SITE_AREA . '/' . $this->context . '/' . $this->moduleName . '/manage/' . $module)
            );
            if($action == 'create' && ($module == 'categories' || $module == 'options') && $this->session->userdata('option_user_id')) {
                // Template::set('selectize_value', array('user_id' => $this->session->userdata('option_user_id')));
                $params['record'] = array('user_id' => $this->session->userdata('option_user_id'));
            }
            $this->gcrud->form($prefix . $module, $action, 'admin', $params);
        }
    }
    
    /**
     * Unique Notification Validation Check
     */
    public function unique_notification() {
        return $this->ecommerce_model->_unique_check('notifications');
    }
    
    /**
     * Unique Wishlist Validation Check
     */
    public function unique_wishlist() {
        return $this->ecommerce_model->_unique_check('wishlists');
    }
    
    /**
     * Listings Menu
     */
    public function listings_menu() {
        $this->session->set_userdata('product_listing_id', 1);
        $this->load->model('listings/listings_model');
        $listing_id = $this->session->userdata('product_listing_id');
        $listing = $this->listings_model->get_listing_info($listing_id);
        $counts = $this->get_counts($listing->user_id);
        $menus = array(
            array(
                'label' => lang('products'),
                'icon' => 'fas fa-box',
                'color' => 'bg-primary text-white',
                'counts' => $counts['products']
            ),
            array(
                'label' => lang('gc_menu_categories'),
                'icon' => 'fas fa-certificate',
                'color' => 'bg-success text-white',
                'counts' => $counts['categories']
            ),
            array(
                'label' => lang('product_options'),
                'icon' => 'fas fa-palette',
                'color' => 'bg-warning',
                'counts' => $counts['options']
            )
        );
        $options = $this->db->get('product_options')->result();
        Template::set(array(
            'options' => $options,
            'menus' => $menus,
            'toolbar_title' => $listing->title
        ));
        Template::render();
    }
    
    /**
     * Get products, categories, and options count
     * @param number $user_id
     * @return array
     */
    private function get_counts($user_id) {
        $sql = "SELECT COUNT(*) AS products";
        $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . "product_categories WHERE user_id = " . $user_id . ") AS categories";
        $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . "product_options WHERE user_id = " . $user_id . ") AS options";
        $sql .= " FROM " . $this->db->dbprefix . "products WHERE user_id = " . $user_id;
        return (array) $this->db->query($sql)->row();
    }
    
    /**
     * Copy Products
     * Default Images
     * Auto increment price by a specific percentage
     */
}