<?php defined('BASEPATH') || exit('No direct script access allowed');
/**
 * Emailer Settings Context
 *
 * Allows the management of the Emailer. Assists in setting up the proper email
 * settings, as well as editing the template and viewing emails in the queue.
 *
 * @package Bonfire\Modules\Emailer\Controllers\Settings
 * @author  Bonfire Dev Team
 * @link    http://cibonfire.com/docs
 */
class Emailer extends Front_Controller
{
	/**
	 * Set up the permissions and load the language file.
	 *
	 * @return void
	 */
	public function __construct() {
		parent::__construct();
	
		$this->lang->load('emailer');
		$this->load->library('gc');
		$this->load->library('crypt');
		$this->load->helper('form');
	}
	
	/**
	 * Unsubscribe User
	 * @param string $visitors_email
	 */
	public function unsubscribe() {
	    $subscriber_id = $this->input->get('id');
	    if($subscriber_id == NULL) {
			Template::redirect();
		}		
		$subscriber_id = (int) $this->crypt->decode($subscriber_id); // Decode Email
		$result = $this->db->get_where('subscriber_users', array('id' => $subscriber_id))->row();
		if(!$result) {
			Template::redirect();
		}
		if(isset($_GET['submit'])) {
			if($subscriber_id > 0) {				
				if($result) {
				    // $this->db->where('id', $result->id)->delete('subscriber_users');
				    // delete all future emails from the subscribers list
				    $this->db->where(array('module_id' => gc_module_id('subscriber_users'), 'row_id' => $subscriber_id))->delete('email_queue');
					$this->db->where('id', $result->id)->update('subscriber_users', array('status_id' => 3, 'modified_on' => date('Y-m-d H:i:s'), 'modified_ip' => $this->input->ip_address()));				
					Template::set(array(
					    'hide_btn' => true,
					    'message' => sprintf(lang('msg_unsubscribe_success'), settings_item('site.title')),
					    'type' => 'success'
					));
				} else {
					Template::set_message(sprintf(lang('msg_unsubscribe_failure'), settings_item('site.title')), 'danger');
				}
			} else {
				Template::set_message(sprintf(lang('msg_unsubscribe_failure'), settings_item('site.title')), 'danger');
			}
		}		
		if($result && $result->status_id == 3) {
			Template::set(array(
			    'hide_btn' => true,
			    'message' => sprintf(lang('msg_unsubscribe_failure'), settings_item('site.title')),
			    'type' => 'danger'
			));
		}
		Template::set('id', $subscriber_id);
		$this->gc->demo_custom_css_js('index');
		Template::render();
	}
	
	/**
	 * Record user once an Email is Opened
	 */
	public function logo() {
		// (inside "record.php")
		header('Content-Type: image/gif');
		
		if(isset($_GET['read'])) {
			$sent_id = $this->crypt->decode($_GET['read']);
			$this->_tracking($sent_id);
		}
		$file = FCPATH . 'assets/images/emlogo.png'; // create a transparent emlogo.png file in the assets/images folder, you can also have your own logo
		//push out image
		if(ini_get('zlib.output_compression')) { ini_set('zlib.output_compression', 'Off'); }
		header('Pragma: public');   // required
		header('Expires: 0');       // no cache
		header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
		header('Cache-Control: private',false);
		// header('Content-Disposition: attachment; filename="' . $file . '"');
		header('Content-Transfer-Encoding: binary');
		header('Content-Length: '.filesize($file));   // provide file size
		readfile($file);      // push it out
		exit();		
	}
	
	/**
	 * Redirect user to the actual link after tracking
	 */
	public function visit() {
		$redirect_url = site_url();//'https://www.ecomsta.com';
		if(isset($_GET['id'])) {
			$sent_id = (int) $this->crypt->decode($_GET['id']);
			$link_id = 0;
			if(isset($_GET['link'])) {
				$link_id = (int) $this->crypt->decode($_GET['link']);
				$this->_tracking($sent_id, $link_id, 1);
				if($link_id > 0) {
					$link = $this->db->get_where('email_links', array('id' => $link_id))->row();
					if(!empty($link->url)) {
						$redirect_url = $link->url;
					}
				}
			}
		}
		Template::redirect($redirect_url);
	}
	
	/**
	 * Track Click
	 * @param number $sent_id
	 * @param number $link_id
	 * @param number $type_id - 0 -> Read, 1 -> Link Clicked
	 */
	private function _tracking($sent_id, $link_id = 0, $type_id = 0) {
		$sent_id = (int) $sent_id;
		$link_id = (int) $link_id;
		if($sent_id > 0) {
			$message_id = 0;
			$message = $this->db->select('message_id')->where('id', $sent_id)->get('email_sent')->row();
			if($message) {
				$message_id = $message->message_id;
			}
			$data = array(
				'sent_id' => $sent_id,
				'message_id' => $message_id,
				'type_id' => $type_id,
				'link_id' => $link_id,
				'created_on' => date('Y-m-d H:i:s'),
				'ip_address' => $this->input->ip_address()
			);
			$this->db->insert('email_history', $data);
			if($type_id == 0) { // Last Read IP Address for Reporting
				$this->db->where('id', $sent_id)->update('email_sent', array('last_ip' => $this->input->ip_address));
			}
		}
	}
}