<?php defined('BASEPATH') || exit('No direct script access allowed');

/**
 * Bonfire
 *
 * An open source project to allow developers to jumpstart their development of
 * CodeIgniter applications.
 *
 * @package   Bonfire
 * @author    Bonfire Dev Team
 * @copyright Copyright (c) 2011 - 2015, Bonfire Dev Team
 * @license   http://opensource.org/licenses/MIT The MIT License
 * @link      http://cibonfire.com
 * @since     Version 1.0
 * @filesource
 */

/**
 * Emailer Model
 *
 * @package Bonfire\Modules\Emailer\Models\Emailer_model
 * @author  Bonfire Dev Team
 * @link    http://cibonfire.com/docs
 */
class Emailer_model extends BF_Model
{
    /** @var string Name of the table. */
    public $table_name = 'email_queue';
    protected $table_history = 'email_history';

    /** @var string Name of the primary key. */
    protected $key = 'id';

    /** @var bool Whether to use soft deletes. */
    protected $soft_deletes = false;

    /** @var string The date format to use. */
    protected $date_format = 'datetime';

    /** @var bool Whether to set the created time automatically. */
    protected $set_created = false;

    /** @var bool Whether to set the modified time automatically. */
    protected $set_modified = false;
    protected $z_cache;
    
    public function __construct() {
        $this->load->model('subscribers/subscribers_model');
		$this->load->helper('gc');
        $this->load->library('gzf_cache', array('lifetime' => 900));
        $this->load->library('slug');
        $this->z_cache = $this->gzf_cache->get_instance();
    }
    
    /**
     * Email Templates Fields
     *
     * @return array
     */
    public function templates() {
        // Assets::add_js('bootstrap-select.min.js');
        // Assets::add_css('bootstrap-select.min.css');
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'dropdown', // dropdown selectpicker conflict with tinymce editor
                'name' => 'group_ids',
                'label' => 'label_type',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:email_template_groups;value:database', // 'load:email_template_groups;module:email_template_groups;max_items:100',
                'options_detailed' => 1,
                'field_extra' => 'selectpicker:1;multiple:1;max_options:10',
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'name',
                'label' => 'label_name',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'edit' => 1,
                'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
                'rules' => 'trim|required|max_length[255]',
            ),
            array(
                'type' => 'text',
                'name' => 'subject',
                'label' => 'label_subject',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|required|max_length[200]',
            ),
            array(
                'type' => 'text',
                'name' => 'short_description',
                'label' => 'emailer_short_description',
                'placeholder' => 'placeholder_short_description',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim|max_length[100]',
            ),
            array(
                'type' => 'textarea',
                'name' => 'message',
                'label' => 'label_message',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'custom_rules' => 'editor',
                'field_class' => 'editor-filemanager',
                'class' => 9,
                'rules' => 'trim|required',
            ),
            array(
                'type' => 'text',
                'name' => 'params',
                'label' => 'label_params',
                'frontend' => false,
                'admin_only' => false,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'is_php',
                'label' => 'label_is_php',
                'frontend' => false,
                'admin_only' => false,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'status',
                'label' => 'label_status',
                'frontend' => false,
                'admin_only' => false,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim'
            ),
            array(
                'type' => 'none',
                'name' => 'created_ip',
                'label' => 'created_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim'
            ),
            array(
                'type' => 'none',
                'name' => 'created_by',
                'label' => 'created_by',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => $this->auth->role_id() == 1 ? false : false,
                'rules' => 'trim'
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim'
            ),
            array(
                'type' => 'none',
                'name' => 'modified_by',
                'label' => 'modified_by',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim'
            ),
            array(
                'type' => 'none',
                'name' => 'modified_ip',
                'label' => 'modified_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim'
            )
        );
        return $fields;
    }
    
    
    /**
     * Email Template Groups Fields
     *
     * @return array
     */
    public function template_groups() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'text',
                'name' => 'name',
                'label' => 'label_name',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'edit' => 1,
                'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
                'rules' => 'trim|required|max_length[150]',
            ),
            array(
                'type' => 'text',
                'name' => 'priority',
                'label' => 'label_priority',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'value' => 0,
                'rules' => 'trim|required|is_numeric',
                'class' => 2,
            ),
            
        );
        return $fields;
    }
    
    /**
     * Email Providers Fields
     *
     * @return array
     */
    public function providers() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'text',
                'name' => 'name',
                'label' => 'label_name',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'edit' => 1,
                'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
                'rules' => 'trim|required',
            ),
            array(
                'type' => 'text',
                'name' => 'smtp_host',
                'label' => 'emailer_server_address',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|max_length[255]',
            ),
            array(
                'type' => 'dropdown',
                'name' => 'smtp_crypto',
                'label' => 'emailer_smtp_crypto',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:label_none;value:0|name:emailer_smtp_ssl;value:ssl|name:emailer_smtp_tls;value:tls',
                'options_detailed' => 1,
                'options_lang' => 1,
                'class' => 2,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'smtp_port',
                'label' => 'emailer_port',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'value' => 465,
                'class' => 2,
                'rules' => 'trim|required|is_numeric',
            ),
            array(
                'type' => 'text',
                'name' => 'smtp_timeout',
                'label' => 'emailer_timeout_secs',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'value' => 10,
                'class' => 2,
                'rules' => 'trim|required|is_numeric',
            ),
            array(
                'type' => 'text',
                'name' => 'hourly_limit',
                'label' => 'emailer_hourly_limit',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'value' => 40,
                'class' => 2,
                'rules' => 'trim|required|is_numeric',
            ),
            array(
                'type' => 'text',
                'name' => 'per_day_limit',
                'label' => 'emailer_daily_limit',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'value' => 100,
                'class' => 2,
                'rules' => 'trim|required|is_numeric',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_ip',
                'label' => 'created_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_ip',
                'label' => 'modified_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            
        );
        return $fields;
    }
    
    /**
     * Email Sender History Fields
     *
     * @return array
     */
    public function sender_history() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'text',
                'name' => 'sender_id',
                'label' => 'singular_senders',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'success_counts',
                'label' => 'emailer_success',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'failure_counts',
                'label' => 'emailer_failure',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'label_datetime',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            
        );
        return $fields;
    }
    
    /**
     * Display available variable names to the Admin
     * @param string $module
     * @return multitype:Ambigous <multitype:, multitype:string >
     */
    public function share_variables($module, $emailer = true, $separator = '<br>') {
        $this->lang->load('emailer/emailer');
        $subject = array();
        $message = array();
        switch($module) {
            case 'users':
                $subject = array(
                    'username' => lang('bf_username'),
                    'display_name'=>lang('label_display_name')
                );
                $message = array(
                    'email' => lang('label_email'),
                    'username' => lang('bf_username'),
                    'display_name' => lang('label_display_name'),
                    'profile_image' => lang('label_profile_image'),
                    'phone_number' => lang('label_phone_number'),
                    'address' => lang('label_address'),
                    'city' => lang('label_city'),
                    // 'state' => lang('label_state'),
                    'postcode' => lang('label_postcode'),
                    // 'country' =>lang('label_country')
                );
                break;
            case 'subscriber_users':
                $subject = array(
                    'email' => lang('label_email'),
                    'name'=>lang('label_name')
                );
                $message = array(
                    'email' => lang('label_email'),
                    'name' => lang('label_name'),
                    'phone_number' => lang('label_phone_number'),
                    'address' => lang('label_address'),
                    'city' => lang('label_city'),
                    // 'state' => lang('label_state'),
                    'postcode' => lang('label_postcode'),
                    // 'country' =>lang('label_country')
                );            
                break;
            default:
                $subject = array(
                    'email' => lang('label_email'),
                    'display_name'=>lang('label_display_name')
                );
                $message = array(
                    'email' => lang('label_email'),
                    'username' => lang('bf_username'),
                    'display_name' => lang('label_display_name'),
                    'profile_image' => lang('label_profile_image'),
                    'phone_number' => lang('label_phone_number'),
                    'address' => lang('label_address'),
                    'city' => lang('label_city'),
                    // 'state' => lang('label_state'),
                    'postcode' => lang('label_postcode'),
                    // 'country' =>lang('label_country')
                );            
        }
        $subject_format = array();
        if(!empty($subject)) {
            foreach($subject as $key => $value) {
                $subject_format[] = $key . ' => ' . $value;
            }
        }
        $message_format = array();
        if(!empty($message)) {
            foreach($message as $key => $value) {
                $message_format[] = $key . ' => ' . $value;
            }
        }
        $help = $emailer ? lang('em_help') . '<br />' : lang('em_help_parser') . '<br />';
        return array('subject' => $help . (implode($separator, $subject_format)), 'message' => $help . (implode($separator, $message_format)));
    }
    
    /**
     * Get available senders before processing the queue
     */
    public function get_available_senders() {        
        // get providers
        $sql = "SELECT '/usr/sbin/sendmail' AS mailpath, 'mailtype' AS mailtype, 'smtp' AS protocol, ES.name AS sender_name, ES.email AS sender_email,";
        $sql .= " ES.email AS smtp_user, ES.password AS smtp_pass, EP.smtp_host, EP.smtp_port, EP.smtp_timeout,";
        $sql .= " ES.id AS sender_id, EP.per_day_limit, EP.hourly_limit, '0' AS sent_counts";
        $sql .= " FROM " . $this->db->dbprefix . "email_senders ES";
        $sql .= " INNER JOIN " . $this->db->dbprefix . "email_providers EP ON EP.id = ES.provider_id";
        $sql .= " WHERE ES.status = 1 ORDER BY RAND()";
        // $sql .= " LEFT JOIN " . $this->db->dbprefix . "email_sender_logs ESL ON ESL.sender_id = ES.id";
        // $sql .= " GROUP BY ES.id";
        // $sql .= " ORDER BY ESL.created_on DESC";
        $senders_list = $this->db->query($sql)->result_array();
       
        // get senders used today for email sending operations
        $sender_logs = $this->sender_logs();
        // delete all senders from the senders list which exhausted their todays limit
        if($senders_list) {
            foreach($senders_list as $key => $sender) {
                if(array_key_exists($sender['sender_id'], $sender_logs)) {
                    $counts = $sender_logs[$sender['sender_id']];
                    // if limit exhausted, remove it from the list
                    if($counts['day'] >= $sender['per_day_limit'] || $counts['hour'] >= $sender['hourly_limit']) {
                        unset($senders_list[$key]);
                    } else {
                        // update sender available limit
                        $senders_list[$key]['sent_counts'] = $counts;
                    }
                }
            }
        }
        
        // return senders list
        return $senders_list;
    }
    
    /**
     * Get number of emails sent by each sender within the last hour and today
     * @return array
     */
    private function sender_logs() {
        // email sent in last hour
        // $today_date = date('Y-m-d');
        $sql = "SELECT ESL.sender_id, COUNT(*) AS todays_count";
        $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . "email_sender_logs ESLH WHERE ESLH.sender_id = ESL.sender_id AND ESLH.created_on BETWEEN (NOW() - INTERVAL 1 HOUR) AND NOW()) AS hourly_count";
        $sql .= " FROM " . $this->db->dbprefix . "email_sender_logs ESL WHERE DATE(created_on) = CURDATE() GROUP BY ESL.sender_id";
        $records = $this->db->query($sql)->result();
        $senders = array();
        if($records) {
            foreach($records as $record) {
                $senders[$record->sender_id] = array('day' => $record->todays_count, 'hour' => $record->hourly_count);
            }
        }
        return $senders;
    }

    /**
     * Pick one sender from the list
     */
    public function pick_sender() {
        $records = $this->get_available_senders();
        if(empty($records)) {
            return null;
        }
        $sender = reset($records); // get first sender
        // update the information in the email_sender_logs table
        // note: do not use the PHP datetime function for the created_on field 
        // PHP server time may be different than the MySQL server timezone
        // email_sender_logs table uses the MySQL CURRENT_TIMEZONE
        $data = array(
            'sender_id' => $sender['sender_id']
        );
        $this->db->insert('email_sender_logs', $data);
        $this->load->library('crypt');
        // decrypt password before sending email
        $sender['smtp_pass'] = $this->crypt->decode($sender['smtp_pass']);
        return $sender;
    }
    
    /**
     * After processing queue, update senders counts in the email_sender_logs table
     */
    public function update_senders() {
        $available_senders = $this->session->userdata('available_senders');
        if(!empty($available_senders)) {
            $data = array();
            foreach($available_senders as $available_sender) {
                $data[] = array(
                    'sender_id' => $available_sender['sender_id'],
                    'counts' => $available_sender['sent_counts'],
                    'date' => date('Y-m-d')
                );
            }
            $this->db->on_duplicate('email_sender_logs', $data);
        }
        // clear available senders cache
        clear_cache('available_senders');
        $this->session->unset_userdata('available_senders');
    }
    
    /**
     * Update Sender History on Email Sent for records
     * @param number $sender_id
     * @param number $sent_status
     */
    public function _update_sender($sender_id, $sent_status) {
        if(!empty($sender_id)) {
            $data = array(
                'sender_id' => $sender_id
            );
            $where = "DATE(created_on) = '" . date('Y-m-d') . "'";
            $record = $this->db->where($where)->get('email_sender_history')->row();
            if($record) {
                // update
                if($sent_status == 0) { // failure counts
                    $data['failure_counts'] = $record->failure_counts + 1;                    
                } else {
                    $data['success_counts'] = $record->success_counts + 1; 
                }
                $this->db->where('id', $record->id)->update('email_sender_history', $data);
            } else {
                if($sent_status == 0) { // failure counts
                    $data['failure_counts'] = 1;
                } else {
                    $data['success_counts'] = 1;
                }
                $data['created_on'] = date('Y-m-d H:i:s');
                $this->db->insert('email_sender_history', $data);
            }
        }
    }
    
    /**
     * Purge all logs older than the specified intervals
     * @param number $interval
     */
    public function _purge_logs($interval = 2) {
        // count and update the sent emails information in the sender_logs table        
        $sql = "DELETE FROM " . $this->db->dbprefix . "email_sender_logs WHERE created_on < now() - interval 2 DAY";
        $this->db->query($sql);
    }
    
    /**
     * Switch Template
     * @param number $template_id
     */
    public function switch_template($template_id) {
        $new_template = array();
        $template = $this->db->where('id', $template_id)->get('email_templates')->row();
        if($template && !empty($template->group_ids)) {
            $group_ids = explode(',', $template->group_ids);
            $group_where = array();
            foreach($group_ids as $group_id) {
                $group_where[] = "FIND_IN_SET(" . $group_id. ", group_ids)";
            }
            $where = "status = 1 AND (" . implode(' OR ', $group_where) . ")";
            $new_template = $this->db->where($where)->order_by('RAND()')->get('email_templates')->row();
        }
        return $new_template;
    }
    
    /**
     * Get Providers
     * @return array
     */
    private function get_providers() {        
        $cacheID = 'emailer_providers';
        if(!$providers = $this->z_cache->load($cacheID)) {
            $records = $this->db->get('email_providers')->result();
            $providers = array();
            if(!empty($records)) {
                foreach($records as $record) {
                    $providers[$record->id] = array(
                        'mailpath' => '/usr/sbin/sendmail',
                        'protocol' => 'smtp',
                        'mailtype' => 'html',
                        'smtp_host' => $record->smtp_host,
                        'smtp_port' => $record->smtp_port,
                        'smtp_timeout' => $record->smtp_timeout
                    );
                }
            }
            $this->z_cache->save($providers, $cacheID, array('emailer_providers'));
        }
        return $providers;
    }
    
    /**
     * Test sender
     * @param integer $id
     */
    public function test_sender($id, $sender = array()) {
        $data = array(
            'success' => false,
            'debug' => ''
        );
        $providers = $this->get_providers();
        if(empty($sender)) {
            $sender = $this->db->where('id', $id)->get('email_senders')->row();
        }
        if($sender) {
            $this->load->library('crypt');
            $config_settings = $providers[$sender->provider_id];
            $config_settings['smtp_user'] = $sender->email;
            $config_settings['smtp_pass'] = $this->crypt->decode($sender->password);
            $config_settings['sender_email'] = $sender->email;
            
            $email_to = settings_item('sender_email');
            $site_title = settings_item('site.title');
            $email_subject = sprintf(lang('emailer_test_subject'), $site_title);
            $email_message = sprintf(lang('emailer_test_body'), $site_title);
            $sender_email = $config_settings['sender_email'];
            $sender_name = !empty($sender->name) ? $sender->name : $site_title;
            
            $this->load->library('email');
            
            $this->email->initialize($config_settings);
            $this->email->clear(true);
            $this->email->set_newline("\r\n");
            $this->email->to($email_to);
            $this->email->from($sender_email, $sender_name);
            $this->email->subject($email_subject);
            $this->email->message($email_message);
            if ($this->email->send() === true) {
                // Email was successfully sent
                $data['success'] = true;                
            }
            $data['debug'] = $this->email->print_debugger();
        }
        return $data;
    }
    
    /**
     * Mark subscribers as spam if an email mail sent fails thrice
     * @param number $failure_counts
     * @return boolean
     */
    public function mark_spam($message_id, $failure_counts = 3) {
        // get items from email_sent table
        $records = $this->db->where(array('message_id' => $message_id, 'failure_counts >= ' => $failure_counts))->get('email_sent')->result();
        if(!$records) {            
            Template::set_message(lang('error_no_record_found'), 'danger');
            return false;
        }
        $data = array();
        foreach($records as $record) {
            $data[] = array(
              'id' => $record->row_id,
              'status' => 3  
            );
        }
        $this->db->update_batch($this->subscribers_model->table_name, $data, 'id');
        $module_name = count($checked) == 1 ? lang('singular_subscribers') : lang('em_subscribers');
        Template::set_message(count($data) .' '. sprintf(lang('msg_action_success'), $module_name, lang('marked_spam')), 'success');
        return true;
    }
    
    
    /**
     * View Page - Display Messages, Sent Items and History Information
     * @param number $count
     * @param string $type
     * @param string $where
     * @param number $offset
     * @param number $limit
     * @return object|number
     */
    public function get_view($count = 0, $type = 'messages', $where = null, $sort_by = 'T.id DESC', $offset = 0, $limit = 25) {
        $sql = " SELECT COUNT(T.id) AS total_count";
        switch($type) {
            case 'groups':
                if($count == 0) {
                    $sql = "SELECT T.*,";
                    $sql .= " (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->subscribers_model->table_name . " WHERE group_id = T.id AND status_id = 1) AS inactive,";
                    $sql .= " (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->subscribers_model->table_name . " WHERE group_id = T.id AND status_id = 2) AS active,";
                    $sql .= " (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->subscribers_model->table_name . " WHERE group_id = T.id AND status_id = 3) AS unsubscribed";
                }
                $sql .= " FROM " . $this->db->dbprefix . "email_groups T";
                break;
            case 'messages':
                if($count == 0) {
                    $sql = "SELECT T.*";
                    $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . "email_sent WHERE message_id = T.id) AS total,";
                    $sql .= " (SELECT COUNT(*) FROM " . $this->db->dbprefix . "email_sent WHERE message_id = T.id AND status = 0) AS pending,";
                    $sql .= " (SELECT COUNT(*) FROM " . $this->db->dbprefix . "email_sent WHERE message_id = T.id AND status = 1) AS sent,";
                    $sql .= " (SELECT COUNT(*) FROM " . $this->db->dbprefix . "email_sent WHERE message_id = T.id AND status = 2) AS failure,";
                    $sql .= " (SELECT COUNT(*) FROM " . $this->db->dbprefix . "email_history WHERE message_id = T.id AND type_id = 0) AS opened,";
                    $sql .= " (SELECT COUNT(*) FROM " . $this->db->dbprefix . "email_history WHERE message_id = T.id AND type_id = 1) AS clicked";
                }
                $sql .= " FROM " . $this->db->dbprefix . "subscriber_messages T";
                break;
            case 'sent':
                if($count == 0) {
                    $sql = "SELECT T.*, ESB.name, ESB.email, CONCAT(ESM.id, ' - ', ESM.name) AS message_id";
                    $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . "email_history WHERE sent_id = T.id AND type_id = 0) AS opened,";
                    $sql .= " (SELECT COUNT(*) FROM " . $this->db->dbprefix . "email_history WHERE sent_id = T.id AND type_id = 1) AS clicked";
                }
                $sql .= " FROM " . $this->db->dbprefix . "email_sent T";
                if($count == 0) {
                    $sql .= " LEFT JOIN " . $this->db->dbprefix . $this->subscribers_model->table_name . " ESB ON ESB.id = T.row_id";
                    $sql .= " LEFT JOIN " . $this->db->dbprefix . "subscriber_messages ESM ON ESM.id = T.message_id";
                }
                break;
            case 'history':
                if($count == 0) {
                    $sql = "SELECT T.*, EL.url AS link, ESM.subject AS message, CONCAT(ESM.id, ' - ', ESM.name) AS message_id";
                }
                $sql .= " FROM " . $this->db->dbprefix . "email_history T";
                if($count == 0) {
                    // $sql .= " LEFT JOIN " . $this->db->dbprefix . "email_messages EM ON EM.id = T.message_id";
                    $sql .= " LEFT JOIN " . $this->db->dbprefix . "email_links EL ON EL.id = T.link_id";
                    $sql .= " LEFT JOIN " . $this->db->dbprefix . "subscriber_messages ESM ON ESM.id = T.message_id";
                }
                break;
                
        }
        if($type == 'messages') {
            $sql .= " WHERE deleted = 0 AND type = 'email'";
            if(!empty($where)) {
                $sql .= " " . $where;
            }
        } else if(!empty($where)) {
            $sql .= " WHERE " . $where;
        }
        if($count == 0) {
            $sql .= " ORDER BY " . $sort_by;
            if ($limit != 0) {
                $sql .= " LIMIT " . $offset . ", " . $limit;
            } else {
                $sql .= " LIMIT $limit";
            }
            return $this->db->query($sql)->result();
        } else {
            $result = $this->db->query ( $sql )->row();
            return $result ? $result->total_count : 0;
        }
    }
    
    /**
     * Sent and failure summary
     * @param string $date
     * @param number $days - x days from the specified date
     */
    public function summary_by_dates($date = null, $days = 5) {
        $result = array();
        if(empty($date)) {
            $date = date('Y-m-d');
        }
        if(!empty($days)) {
            for($i = 1; $i <= $days; $i++) {
                $sent_date = date('Y-m-d', strtotime($date. ' + ' . $i . ' days'));
                $result[$sent_date] = $this->get_sent_failure_by_date($sent_date);
            }
        } else {
            $result[$date] = $this->get_sent_failure_by_date($date);
        }
        krsort($result);
        return $result;
    }
    
    /**
	 * Display Summary
	 * @return array
	 */
	public function summary() {
	    $sql = "SELECT COUNT(*) AS total_emails";
	    $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->table_name . " WHERE date_sent IS NOT NULL) AS total_sent";
	    $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->table_name . " WHERE date_sent IS NULL) AS total_pending";
	    $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->table_history . " WHERE type_id = 0) AS total_opened";
	    $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->table_history . " WHERE type_id = 1) AS total_clicked";
	    $sql .= " FROM " . $this->db->dbprefix . $this->table_name;
	    $query = $this->db->query($sql);
	    return $query->row();
	}
    
    /**
     * Get Sent and Failure count by date
     * @param string $date
     * @return NULL[]
     */
    private function get_sent_failure_by_date($date) {
        $result = array();
        $sent = 0;
        $sent_sql = "SELECT COUNT(id) AS sent FROM " . $this->db->dbprefix . "email_sent WHERE DATE(sent_on) = '" . $date . "' AND status = 1";
        $sent_result = $this->db->query($sent_sql)->row();
        if($sent_result) {
            $sent = $sent_result->sent;
        }
        $failure = 0;
        $failure_sql = "SELECT COUNT(id) AS failure FROM " . $this->db->dbprefix . "email_sent WHERE DATE(sent_on) = '" . $date . "' AND status = 2";
        $failure_result = $this->db->query($failure_sql)->row();
        if($failure_result) {
            $failure = $failure_result->failure;
        }
        $result = array(
            'sent' => $sent,
            'failure' => $failure
        );
        return $result;
    }
	
	    /**
     * Form Fields
     *
     * @return array[]
     */
    public function group_template_fields() {
		$user_id = (int) $this->auth->user_id();
		$user_profile = $this->session->userdata('user_profile');
        if(!empty($user_profile['user_id'])) {
			$user_id = $user_profile['user_id'];
		}
		$fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => false,
                'admin_index' => true
            ),
			array(
				'type' => 'dropdown',
				'name' => 'group_id',
				'label' => 'label_group',
				'frontend' => true,
				'admin_only' => true,
				'frontend_index' => false,
				'admin_index' => true,
				'value' => $user_id,
                'options' => 'load:email_groups',
                'options_detailed' => 1,
				'rules' => 'required|trim|max_length[255]',
			),
			array(
				'type' => 'dropdown',
				'name' => 'template_id',
				'label' => 'label_template',
				'frontend' => true,
				'admin_only' => true,
				'frontend_index' => true,
				'admin_index' => true,
				'edit' => 1,
                'options' => 'load:email_templates',
                'options_detailed' => 1,
				'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
				'rules' => 'required|trim|max_length[255]',
			),
			array(
				'type' => 'text',
				'name' => 'from_name',
				'label' => 'label_name',
				'frontend' => true,
				'admin_only' => true,
				'frontend_index' => true,
				'admin_index' => true,
				'edit' => 0,
				'rules' => 'required|trim',
			),
			array(
				'type' => 'text',
				'name' => 'from_email',
				'label' => 'label_email',
				'frontend' => true,
				'admin_only' => true,
				'frontend_index' => true,
				'admin_index' => true,
				'edit' => 0,
				'rules' => 'required|trim',
			),
			array(
				'type' => 'text',
				'name' => 'subject',
				'label' => 'label_subject',
				'frontend' => true,
				'admin_only' => true,
				'frontend_index' => true,
				'admin_index' => true,
				'edit' => 0,
				'rules' => 'required|trim',
			),
			array(
				'type' => 'none',
				'name' => 'created_on',
				'label' => 'created_on',
				'frontend' => true,
				'admin_only' => true,
				'frontend_index' => true,
				'admin_index' => true,
				'rules' => 'trim'
			),
			array(
				'type' => 'none',
				'name' => 'modified_on',
				'label' => 'modified_on',
				'frontend' => true,
				'admin_only' => true,
				'frontend_index' => true,
				'admin_index' => true,
				'rules' => 'trim'
			)	
        );
        return $fields;
    }
	 /**
     * Get Success Stories
     * @param number $count
     * @param string $where
     * @param string $sort_by
     * @param number $limit
     * @param number $offset
     * @return number|array[]
     */
    public function get_group_templates($count, $where, $sort_by, $limit, $offset) {
        if($count == 1) {
            $sql = 'SELECT COUNT(*) AS total_counts';
            $sql .= ' FROM ' . $this->db->dbprefix . 'email_group_templates GS';
            if(!empty($where)) {
                $sql .= " WHERE " . $where;
            }
            $result = $this->db->query($sql)->row();
            return isset($result->total_counts) ? $result->total_counts : 0;
        } else {
            $sql = "SELECT GS.*, EG.name AS group_name, ET.subject AS template_subject";
            $sql .= " FROM " . $this->db->dbprefix . "email_group_templates GS";
            $sql .= " LEFT JOIN " . $this->db->dbprefix . "email_groups EG ON EG.id = GS.group_id";
            $sql .= " LEFT JOIN " . $this->db->dbprefix . "email_templates ET ON ET.id = GS.template_id";
            if(!empty($where)) {
                $sql .= " WHERE " . $where;
            }
            $sql .= " ORDER BY GS." . $sort_by;
			$result = $this->db->query($sql)->result();
            return $result;
        }
    }
	
	/**
	 * Send Group Email on New Subscriber
	 * @param number $group_id
	 */
	public function send_group_email($group_id, $email_to = null) {		
		// get template
		$sql = "SELECT EGT.*, ET.message AS original_message, ET.is_php";
		$sql .= " FROM " . $this->db->dbprefix . "email_group_templates EGT";
		$sql .= " INNER JOIN " . $this->db->dbprefix . "email_templates ET ON ET.id = EGT.template_id";
		$sql .= " WHERE EGT.group_id = " . $group_id . " AND EGT.status = 1 AND EGT.is_default = 1";
		$email_template = $this->db->query($sql)->row();
		if($email_template) {
			if($email_template->is_php == 1) {
				// save the message in a temporary view file
				// pass the parameter
				// the template where is_php is set to 1 are the PHP templates where parameters can be passed
				$this->save_file(APPPATH . '/modules/emailer/views/tmp_view', 'php', $email_template->message);
				$message = $this->load->view('emailer/tmp_view', json_decode($email_template->template_params ?? ''), true);
			} else {
				$message = $email_template->message;
			}
			
			// Send Email
			$this->load->library('emailer/emailer');
			$email_data = array(
				'from'		=> settings_item('sender_email'),
				'to'		=> $email_to,
				'subject'	=> $email_template->subject,
				'message'	=>  $message,
				'headers' => array( // Reply to customers email address
					'reply_email' => $email_template->from_email,
					'reply_name' => $email_template->from_name
				)
			);
			$this->emailer->send($email_data, false);
		}
	}
	
    /**
     * Save file
     * @param string $url
     * @param string $extension
     * @param string $contents
     * @return boolean
     */
    private function save_file($path, $extension, $contents) {
        // Write the changes out...
        if (! function_exists('write_file')) {
            $this->load->helper('file');
        }
        
        // save file
        if (write_file($path . '.' . $extension, $contents)) {
            return true;
        }
        return false;
    }
	
	/**
	 * Send a notification email to the Admin on a new demo
	 */
	public function demo_notify_admin($data = array()) {
		$fields = array('name', 'email', 'phone_number');
		$columns = array();
		foreach($fields as $field) {
			if(!empty($data[$field])) {
				$data_col = '<tr>' . PHP_EOL;
				$data_col .= '<td style="padding: 5px;">' . lang('label_' . $field) . ':</td>' . PHP_EOL;
				$data_col .= '<td style="padding: 5px;"><strong>' . $data[$field] . '</strong></td>' . PHP_EOL;
				$data_col .= '</tr>' . PHP_EOL;
				$columns[] = $data_col;
			}
		}
		// Display location information
		if(!empty($data['location'])) {
			$location_info = json_decode($data['location'], true);
			if(!empty($location_info)) {
				foreach($location_info as $location_key => $location_value) {					
					$data_col = '<tr>' . PHP_EOL;
					$data_col .= '<td style="padding: 5px;">' . ucfirst(str_replace('_', ' ', $location_key)) . ':</td>' . PHP_EOL;
					$data_col .= '<td style="padding: 5px;"><strong>' . $location_value . '</strong></td>' . PHP_EOL;
					$data_col .= '</tr>' . PHP_EOL;
					$columns[] = $data_col;
				}
			}
		}
		$columns = implode('', $columns);
		$message = <<<EOL
		<h3>Matri Demo Viewed By:</h3><br />
		<table style="width: 100%; border-collapse: collapse;" border="1px">
			<tbody>
				{$columns}
			</tbody>
		</table>		
EOL;
		$subject = 'Matri Live Demo Login Details';
		// Send Email
		$this->load->library('emailer/emailer');
		$email = array(
			'to'		=> 'parveen@goclixy.com',
			'subject'	=> $subject,
			'message'	=> $message
		);
		return $this->emailer->send($email, false);		
	}
	
	/**
	 * User's Email Content
	 * @param string $module - Name of the module
	 * @param number $ids - ID of the rows, it can also be a comma separated variable.
	 * @param integer $featured - Get only featured records (Emailer only)
	 * @param string $subject - Subject of the Email
	 * @param string $message - Message of the Email
	 * @param boolean $template - If True, use Emailer Template
	 * @param string $scheduled_on - Date on which email is to be sent
	 * @param string $action - email/sms
	 * @param string $emailer - True means, request is from Settings->Emailer
	 * @return array
	 */
	public function bulk_send($module, $ids, $featured, $subject, $message, $template = false, $scheduled_on = null, $action = 'email', $extra = array()) {
	    $emailer = !empty($extra['emailer']) ? true : false;
	    $proceed_with_email = $action == 'email' || $action == 'sms_email' ? true : false;
	    $proceed_with_sms = $action == 'sms' || $action == 'sms_email' ? true : false;
	    $emailer = isset($extra['emailer']) && $extra['emailer'] ? true : false;
	    if($this->auth->role_id() == 20) { // franchise module
	        $franchise_params = $this->session->userdata('franchise_params');
	        // check the franchise limit
	        $this->load->library('goclixy/gfranchise');
	        switch($action) {
	            case 'sms_email':
	                if(empty($franchise_params['sms_balance'])) {
	                    $proceed_with_sms = false;
	                } else {
	                    $proceed_with_sms = true;
	                }
	                if(empty($franchise_params['email_balance'])) {
	                    $proceed_with_email = false;
	                } else {
	                    $proceed_with_email = true;
	                }
	                break;
	            case 'email':
	                if(empty($franchise_params['email_balance'])) {
	                    $proceed_with_email = false;
	                } else {
	                    $proceed_with_email = true;
	                }
	                break;
	            case 'sms':
	                if(empty($franchise_params['sms_balance'])) {
	                    $proceed_with_sms = false;
	                } else {
	                    $proceed_with_sms = true;
	                }
	                break;
	        }
	    }
	    if(!$proceed_with_sms && !$proceed_with_email) {
	        return false;
	    }	    
	    if($module == 'profiles') {
	        $records = $this->emailer_profiles($ids, 1, $featured);
	    } else {
    	    if(!empty($ids)) {
    	        $records  = $this->db->where('id IN (' . $ids . ')')->get($module)->result();
    	    } else {
    	        $records  = $this->db->get_where($module, array('active' => 1, 'deleted' => 0))->result();
    	    }
	    }
	    $subject_data = array();
	    $message_data = array();
	    $result = false;
	    $success_count = 0;
	    if($records) {
	        $this->load->library('parser');
	        $sms_text = isset($extra['sms_text']) ? $extra['sms_text'] : ($proceed_with_sms ? $message : null);
	        if(isset($extra['template_id'])) {
	            // get template from the email templates table
	            $email_template = $this->db->where('id', $extra['template_id'])->get('email_templates')->row();
	            if($email_template) {
	                $subject = $email_template->subject;
	                $message = $email_template->message;
	                $sms_text = $email_template->sms_text;
	                $template = true;
	            }
	        }
	        foreach($records as $record) {
	            if($module == 'profiles') {
	                $subject_data = array(
	                    'id' => $record->id,
	                    'name' => $record->name,
	                    'profile_id'=> $record->profile_id,
	                    'url' => gc_url_format('profiles', $record->profile_id)
	                );
	                $message_data = array(
	                    'id' => $record->id,
	                    'name' => $record->name,
	                    'profile_id'=> $record->profile_id,
	                    'hits' => $record->hits,
	                    'url' => gc_url_format('profiles', $record->profile_id)
	                );
	            } else if($module == 'subscriber_users') {
	                $subject_data = array(
	                    'id' => $record->id,
	                    'name' => $record->name,
	                    'email'=> $record->email,
	                    'phone_number'=> $record->phone_number
	                );
	                $message_data = array(
	                    'id' => $record->id,
	                    'name' => $record->name,
	                    'email'=> $record->email,
	                    'phone_number'=> $record->phone_number,
	                    'address'=> $record->address,
	                    'postcode'=> $record->postcode,
	                    'city'=> $record->city,
	                    'state'=> $record->state,
	                    'country'=> $record->country
	                );
	            } else if($module == 'users') {
	                $subject_data = array(
	                    'username' => $record->username,
	                    'display_name'=> $record->display_name
	                );
	                if(!empty($record->profile_image) && file_exists(realpath(FCPATH.'assets/images/users/'. $record->profile_image))) {
	                    $profile_image = '<img style="width:200px;height:auto;" src="' . base_url() .'assets/images/users/'. $record->profile_image .'" title="'. $record->display_name .'" alt="' . $record->display_name .'" />';
	                } else {
	                    $profile_image = '<img style="width:200px;height:auto;" src="' . gc_site_image('no-image') . '" title="'. $record->display_name .'" alt="' . $record->display_name .'" />';
	                }
	                
	                $message_data = array(
	                    'email' => $record->email,
	                    'username' => $record->username,
	                    'display_name' => $record->display_name,
	                    'profile_image' => $profile_image,
	                    'phone_number' => $record->phone_number,
	                    'address' => $record->address,
	                    'city' => $record->city,
	                    // 'state' => $record->state,
	                    'postcode' => $record->postcode,
	                    // 'country' => $record->country,
	                    'url' => $module == 'users' ? gc_url_format('user') . '?id=' . $record->id : gc_url_format($module, $record->slug, $record->id, $record->city)
	                );
	                // franchise - send password via email/SMS
	                if($this->input->post('password')) {
	                    $message_data['password'] = $this->input->post('password');
	                }
	            } else {
	                $subject_data = (array) $record;
	                $message_data = (array) $record;
	            }
	            // additional parameters
	            $subject_data['site_title'] = settings_item('site.title');
	            $subject_data['site_url'] = site_url();
	            $subject_data['site_email'] = settings_item('site.system_email');
	            $message_data['site_title'] = settings_item('site.title');
	            $message_data['site_url'] = site_url();
	            $message_data['site_email'] = settings_item('site.system_email');
	            if($template) { // Email Template Contents
	                $subject_data['subject'] = $subject;
	                $subject = $this->parser->parse('email_templates/email_subject', $subject_data, TRUE);
	                $sms_data = $message_data;
	                $message_data['message'] = $message;
	                $message = $this->parser->parse('email_templates/email_body', $message_data, TRUE);
	                // $sms_data['message'] = $sms_text;
	                // $sms_text = $this->parser->parse('email_templates/email_body', $sms_data, TRUE);
                    $sms_text = $this->parser->parse_string($sms_text, $sms_data, TRUE);
	            } else { // Custom Email Content
	                if(!empty($subject)) {
	                    $subject = $this->parser->parse_string($subject, $subject_data, true);
	                }
	                $message = $this->parser->parse_string($message, $message_data, true);
	                $sms_text = $this->parser->parse_string($sms_text, $message_data, true);
	            }
	            $email_success = false;
	            $sms_success = false;
	            
	            // send email
	            if($proceed_with_email && !empty($record->email)) {
	                $this->load->library('emailer/emailer');
	                $data = array(
	                    //'from'		=> settings_item('sender_email'),
	                    'to'		=> $record->email,
	                    'subject'	=> $subject,
	                    'message'	=> ($emailer ? gc_unsubscribe($record->email, $module, $message) : $message),
	                    'scheduled_on' => $scheduled_on
	                );
	                if($module == 'users') {
	                    $data['user_id'] = $record->id;
	                }
	                $queue = $emailer ? TRUE : FALSE; // If emailer, send all emails to queue
	                $email_success = $this->emailer->send($data, $queue, $module);
	            }
	            // send sms
	            if($proceed_with_sms && !empty($record->phone_number)) {
	                $phone_number = $record->phone_number;
	                if(!empty($phone_number)) {
	                    // Send SMS
	                    $this->load->library('gc');
	                    $sms_success = $this->gc->send_sms($sms_text, $phone_number);
	                }
	            }
	            if($email_success || $sms_success) {
	                if($email_success) {
	                    // set the status to replied
	                    switch($module) {
	                        case 'callbacks':
	                        case 'feedbacks':
	                        case 'contact_queries':
	                            if(empty($record->status)) {
	                                $this->db->where('id', $record->id)->update($module, array('status' => 1, 'modified_on' => date('Y-m-d H:i:s')));
	                            }
	                            break;
	                    }
	                }
	                if($this->auth->role_id() == 8) {
	                    // update the franchise limit
	                    if($proceed_with_email && $proceed_with_sms) {
	                        if($email_success && $sms_success) {
	                            $this->gfranchise->update_limit('sms_email');
	                        } else if($email_success) {
	                            $this->gfranchise->update_limit('email_balance');
	                        } else if($sms_success) {
	                            $this->gfranchise->update_limit('sms_balance');
	                        }
	                    } else {
	                        $this->gfranchise->update_limit($action == 'email' ? 'email_balance' : 'sms_balance');
	                    }
	                }
	                $result = true;
	                $success_count++;
	            }
	        }
	    }
	    return array('success' => $result, 'success_count' => $success_count);
	}
	
	/**
	 * Template Fields for preview
	 * @return NULL[]
	 */
	public function template_fields() {
	    $this->lang->load('emailer/emailer');
	    $record = array();
	    $links = $this->db->limit(50)->get('email_links')->result();
	    foreach($links as $link) {
	        $record['link_' . $link->id] = anchor($link->url, $link->url);
	    }
	    $record['logo_url'] = lang('emailer_logo_url');
	    return $record;
	}
	
	/**
	 * Get Messages
	 */
	public function get_messages() {
	    $user_id = $this->auth->user_id();
	    //get messages from the email queues table
	    $records = $this->db->where('user_id', $user_id)->get('emailer_queue')->result();
	    Template::set_message(array(
	        'toolbar_title' => lang('label_messages'),
	        'records' => $records
	    ));	    
	}
}
