<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Erroneous controller
 */
class Reports extends Admin_Controller
{
	protected $permissionCreate = 'Erroneous.Reports.Create';
	protected $permissionDelete = 'Erroneous.Reports.Delete';
	protected $permissionEdit   = 'Erroneous.Reports.Edit';
	protected $permissionView   = 'Erroneous.Reports.View';
	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct() {
		parent::__construct();

		$this->auth->restrict($this->permissionView);
		$this->load->model('erroneous_model', null, true);
		$this->lang->load('erroneous');
		
		Template::set_block('sub_nav', 'reports/_sub_nav');
	}

	//--------------------------------------------------------------------


	/**
	 * Displays a list of form data.
	 *
	 * @return void
	 */
	public function index($offset = 0) {
		// Deleting anything?
		if (isset($_POST['delete'])) {		    
		    $this->gcrud->stats(array('module' => 'erroneous'));
			$checked = $this->input->post('checked');

			if (is_array($checked) && count($checked)) {
				$result = FALSE;
				foreach ($checked as $pid) {
					$result = $this->erroneous_model->delete($pid);
				}

				if ($result) {
					Template::set_message(count($checked) .' '. sprintf(lang('msg_delete_success'), lang('label_erroneous')), 'success');
				} else {
					Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_erroneous')) . $this->erroneous_model->error, 'danger');
				}
			}
		}
		
		$this->erroneous_model->limit($this->limit, $offset);
		
		// Sort By
		$order_by = $this->gc->sort('erroneous');
		if(!empty($order_by)) {
			$this->erroneous_model->order_by($this->session->userdata('erroneous_sort')['column'], $this->session->userdata('erroneous_sort')['order']);
		} else {
			$this->erroneous_model->order_by('id', 'desc');
		}
		$records = $this->erroneous_model->get_erroneous_reports();
		
		// Pagination
		$this->load->library('pagination');
		
		$total_erroneous = $this->erroneous_model->count_all();
		
		$pager['base_url'] = site_url(SITE_AREA ."/reports/erroneous/index/");
		$pager['total_rows'] = $total_erroneous;
		$pager['per_page'] = $this->limit;
		$pager['uri_segment']	= 5;
		
		$this->pagination->initialize($pager);

		Template::set('records', $records);
		Template::set('toolbar_title', lang('manage_erroneous'));
		Template::render();
	}

	/**
	 * Allows viewing of Erroneous Reports data.
	 *
	 * @return void
	 */
	public function view() {
		// DATE TIME PICKER
		Assets::add_js('moment.js');
		Assets::add_css ( 'bootstrap-datetimepicker.min.css' );
		Assets::add_js ( 'bootstrap-datetimepicker.min.js' );
		
		$id = $this->uri->segment(5);

		if (empty($id)) {
			Template::set_message(lang('invalid_id'), 'error');
			redirect(SITE_AREA .'/reports/erroneous');
		}
		if (isset($_POST['save'])) {
		    $this->gcrud->stats(array('module' => 'erroneous'));
			$this->auth->restrict($this->permissionEdit);
			if ($this->save_erroneous($id)) {
				// Log the activity
				log_activity($this->current_user->id, lang('act_edit_record') .': '. $id .' : '. $this->input->ip_address(), 'erroneous');
		
				Template::set_message(sprintf(lang('msg_save_success'), lang('label_erroneous')), 'success');
				redirect(site_url(SITE_AREA .'/reports/erroneous'));
			} else {
				Template::set_message(sprintf(lang('msg_edit_failure'), lang('label_erroneous')) . $this->erroneous_model->error, 'danger');
			}
		} elseif (isset($_POST['delete'])) {
		    $this->gcrud->stats(array('module' => 'erroneous'));
			$this->auth->restrict($this->permissionDelete);

			if ($this->erroneous_model->delete($id)) {
				// Log the activity
				log_activity($this->current_user->id, lang('act_delete_record') .': '. $id .' : '. $this->input->ip_address(), 'erroneous');

				Template::set_message(sprintf(lang('msg_delete_success'), lang('label_erroneous')), 'success');

				redirect(SITE_AREA .'/reports/erroneous');
			} else {
				Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_erroneous')) . $this->erroneous_model->error, 'danger');
			}
		}
		$erroneous = $this->erroneous_model->get_erroneous($id);
		Template::set('erroneous', $erroneous);
		Template::set('toolbar_title', sprintf(lang('label_reported_erroneous'), !empty($erroneous->listing_title) ? $erroneous->listing_title : ''));
		Template::render();
	}

	//--------------------------------------------------------------------
	// !PRIVATE METHODS
	//--------------------------------------------------------------------

	/**
	 * Summary
	 *
	 * @param String $type Either "insert" or "update"
	 * @param Int	 $id	The ID of the record to update, ignored on inserts
	 *
	 * @return Mixed    An INT id for successful inserts, TRUE for successful updates, else FALSE
	 */
	private function save_erroneous($id) {
		$_POST['id'] = $id;	
		$data = array();
		$data['type_id']       = $this->input->post('type_id');
		$data['description']   = $this->input->post('description');
		$return = $this->erroneous_model->update($id, $data);
		return $return;
	}
}