<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class Erroneous_model extends BF_Model {

	protected $table_name	= "listing_erroneous";
	protected $key			= "id";
	protected $soft_deletes	= false;
	protected $date_format	= "datetime";

	protected $log_user 	= FALSE;

	protected $set_created	= true;
	protected $set_modified = true;
	protected $created_field = "reported_on";
	protected $modified_field = "modified_on";

	/*
		Customize the operations of the model without recreating the insert, update,
		etc methods by adding the method names to act as callbacks here.
	 */
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 		= array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	/*
		For performance reasons, you may require your model to NOT return the
		id of the last inserted row as it is a bit of a slow method. This is
		primarily helpful when running big loops over data.
	 */
	protected $return_insert_id 	= TRUE;

	// The default type of element data is returned as.
	protected $return_type 			= "object";

	// Items that are always removed from data arrays prior to
	// any inserts or updates.
	protected $protected_attributes = array();

	/*
		You may need to move certain rules (like required) into the
		$insert_validation_rules array and out of the standard validation array.
		That way it is only required during inserts, not updates which may only
		be updating a portion of the data.
	 */
	protected $validation_rules 		= array(
		array(
			"field"		=> "erroneous_description",
			"label"		=> "lang:label_description",
			"rules"		=> "max_length[450]|xss_clean"
		),
	);
	protected $insert_validation_rules 	= array();
	protected $skip_validation 			= FALSE;

	/**
	 * Get Claims and Erroneous for index page
	 */
	public function get_erroneous_reports() {
			$this->db->select('listing_erroneous.*, U.display_name, L.title as listing_title');
			$this->db->join($this->db->dbprefix . 'users U','U.id = listing_erroneous.user_id', 'left');
			$this->db->join($this->db->dbprefix . 'listings L','L.id = listing_erroneous.listing_id', 'left');
			return $this->erroneous_model->find_all();
	}
	
	/**
	 * Get Erroneous for view page
	 * @param number $id
	 */
	public function get_erroneous($id) {
		$this->db->select('listing_erroneous.*, U.display_name, L.title as listing_title, L.slug AS listing_slug, L.city AS listing_city, L.contact_person, L.email AS listing_email, L.phone_number AS listing_phone_number');
		$this->db->join($this->db->dbprefix . 'users U','U.id = listing_erroneous.user_id', 'left');
		$this->db->join($this->db->dbprefix . 'listings L','L.id = listing_erroneous.listing_id', 'inner');
		return $this->erroneous_model->find($id);
	}
	
	/**
	 * Incorrect Types
	 */
	public function incorrect_options($type_id, $return_all = false) {
	    $options = array (
	        1 => lang('erroneous_option_first'),
	        2 => lang('erroneous_option_second'),
	        3 => lang('erroneous_option_third'),
	        4 => lang('erroneous_option_fourth')
	    );
	    if($return_all) {
	        return $options;
	    } else {
	       return !empty($type_id) ? $options[$type_id] : '';
	    }
	}
	
	/**
	 * Report Incorrect
	 */
	public function report($user_id, $is_mobile = false) {
	    $prefix = $is_mobile ? '' : 'erroneous_';
	    $this->form_validation->set_rules($prefix . 'type', 'lang:label_type', 'trim|required');
	    $this->form_validation->set_rules($prefix . 'email', 'lang:bf_email', 'trim|valid_email');
	    $this->form_validation->set_rules($prefix . 'name', 'lang:label_name', 'trim|required|max_length[255]');
	    $this->form_validation->set_rules($prefix . 'phone', 'lang:label_phone', 'phone_check|trim');
        if(!$is_mobile) {
            $this->form_validation->set_rules($prefix . 'captcha_code', 'lang:captcha_code', 'trim|required|valid_captcha');
        }
	    if ($this->form_validation->run() === FALSE) {
	        $validation_errors = validation_errors();
	        if ($validation_errors) {
	            $validation_error = $validation_errors;
	        }	        
	        return array('success' => false, 'result' => 0, 'message' => $validation_error, 'type' => 'danger');
	    } else {        
	        // insert claim information into database
			$row_id = (int) $this->input->post ( 'listing_id' );
			$row_id = empty($row_id) ? (int) $this->input->post ( 'row_id' ) : $row_id;
	        $data = array (
	            'listing_id' => $row_id,
	            'type_id' => (int) $this->input->post ( $prefix . 'type' ),
	            'description' => escape($this->input->post ( $prefix . 'description' )),
	            'name' => escape($this->input->post ( $prefix . 'name' )),
	            'email' => escape($this->input->post ( $prefix . 'email' )),
	            'phone_number' => escape($this->input->post ( $prefix . 'phone' )),
	            'created_on' => date('Y-m-d H:i:s'),
	            'created_ip' => $this->input->ip_address()
	        );
	        $data['user_id'] = (int) $user_id;
	        $this->db->insert($this->table_name, $data );
	        $id = (int) $this->db->insert_id();
	        if ($id > 0) {
	            $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'module_email_templates', 'module_table' => 'listings', 'action_type' => 'erroneous', 'id' => $id));
	            $result = 1;
	            $type = 'success';
	            $message = lang('success_message');
	        } else {
	            $result = 0;
	            $type = 'danger';
	            $message = lang('failure_message');
	        }	        
	        return array('success' => $result, 'result' => $result, 'message' => $message, 'type' => $type);
	    }
	}
}
