<?php defined('BASEPATH') || exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Content controller
 */
class Content extends Admin_Controller {
    protected $permissionCreate = 'Faqs.Content.Create';
    protected $permissionDelete = 'Faqs.Content.Delete';
    protected $permissionEdit   = 'Faqs.Content.Edit';
    protected $permissionView   = 'Faqs.Content.View';
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
        
        $this->auth->restrict($this->permissionView);
        $this->lang->load('faqs');
        $this->lang->load('themes/themes');
        
      	$this->form_validation->set_error_delimiters("<span class='error'>", "</span>");
        
        Template::set_block('sub_nav', 'content/_sub_nav');
        Assets::add_module_js('faqs', 'faqs.js');
    }
    
    /**
     * Display a list of FAQs
     * @param number $offset
     */
    public function index($offset = 0) {
        $this->manage('faqs', $offset);
    }
    
    /**
     * Display a list of data.
     *
     * @return void
     */
    public function manage($module = 'faqs', $offset = 0) {
        $this->gcrud->index($module, $offset, 'admin', array(
            'context' => 'content',
            'model' => 'faqs',
            'fields_method' => $module,
            'query_method' => 'get_all_crud',
            'table_alias' => 'M.',
            'prefix' => 'M',
            'search_fields' => array('title', 'description'),
            'uri_segment' => 6,
            'query_extra' => array('module' => $module),
            'redirect_url' => SITE_AREA . '/content/faqs/manage/' . $module,
            'edit_url' => SITE_AREA . '/content/faqs/form/' . $module . '/edit/',
            'can_create' => true,
            'create_url' => site_url(SITE_AREA . '/content/faqs/form/' . $module)
        ));
    }
    
    /**
     * Add/Edit Object.
     *
     * @return void
     */
    public function form($module = 'faqs', $action = 'create') { 
        clear_cache('faqs');
        Assets::add_js(array(
            'tinymce/tinymce.min.js'
        ));
        $extra = array(
            'context' => 'content',
            'model' => 'faqs',
            'fields_method' => $module,
            'redirect_url' => SITE_AREA . '/content/faqs/manage/' . $module,
            'cancel_btn' => true
        );        
        if($module == 'groups') {
            $extra['method'] = 'after_group_save';
        }
        if($action == 'edit') {
            $extra['id'] = $this->uri->segment(7);
        }
        $this->gcrud->form($module, $action, 'admin', $extra);
    }
    
	/**
	 * Allow admin to change order of faq groups
	 */
	public function order() {
		//LOAD NESTED SORTABLE JAVASCRIPT AND CSS FILE
		Assets::add_css('nestedSortable/admin.css');
		Assets::add_js('nestedSortable/jquery-ui.min.js');
		Assets::add_js('nestedSortable/jquery.mjs.nestedSortable.js');
		
		Assets::add_js($this->load->view('content/order_js', null, true), 'inline');
		Template::set('sortable', TRUE);
		Template::set('toolbar_title', lang('label_order'). ' ' . lang('faq_groups'));
		Template::render();
	}
	
	/**
	 * Ajax function to order faq_groups using nestedSortable
	 */
	public function order_ajax () {
		// LOAD NESTED SORTABLE JAVASCRIPT AND CSS FILE
		Assets::add_css('nestedSortable/admin.css');
		Assets::add_js('nestedSortable/jquery-ui.min.js');
		Assets::add_js('nestedSortable/jquery.mjs.nestedSortable.js');
		Assets::add_js($this->load->view('content/order_js', null, true), 'inline');
		// Save order from ajax call
		if (isset($_POST['sortable'])) {
			$this->save_order($_POST['sortable']);
		}
	
		// Load view
		Template::set('records', $this->get_nested());
		Template::set('toolbar_title', lang('label_order'). ' ' . lang('faq_groups'));
		Template::render();
	}

    /**
     * To display all records to the admin
     * @return multitype:multitype:unknown
     */
    public function get_nested() {
        $records = $this->db->order_by('priority')->get('faq_groups')->result_array();
        return $this->prepareList($records);
    }
    
    /**
     * PREPARE LIST TO GET PARENTS AND ITS CHILDREN
     * @param array $items
     * @param number $pid
     * @return multitype:multitype:unknown
     */
    private function prepareList(array $items, $pid = 0) {
        $output = array();
        
        # loop through the items
        foreach ($items as $item) {
            $item['name'] = $item['name'] . ' - ' . $item['id'];
            # Whether the parent_id of the item matches the current $pid
            if ((int) $item['parent_id'] == $pid) {
                
                # Call the function recursively, use the item's id as the parent's id
                # The function returns the list of children or an empty array()
                if ($children = $this->prepareList($items, $item['id'])) {
                    
                    # Store all children of the current item
                    $item['children'] = $children;
                }
                
                # Fill the output
                $output[] = $item;
            }
        }
        
        return $output;
    }
	
    /**
     * Save records only when user clicks on save button
     * @param array $records
     */
    public function save_order($records) {
        $table_name = 'faq_groups';
        $this->load->library('goclixy/gclosure');
        $gclosure = new Gclosure($table_name, 'faq_group_hierarchies');
        // Convert data into JSON before sending it via POST
        // By default PHP settings (max_input_vars) allows only 1000 post variables
        // To avoid this issue, convert them to JSON and reconvert them to array in PHP
        $records = json_decode($records);
        if (count($records)) {
            foreach ($records as $order => $record) {
                if (isset($record->id) && $record->id != 0) {
                    $id = $record->id;
                    $parent_id = (int) $record->parent_id;
                    $data = array('parent_id' => $parent_id, 'priority' => $order);
                    $this->db->set($data)->where('id', $record->id)->update($table_name);
                    
                    // save hierarchy
                    $id = $record->id;
                    $record = $this->db->where('id', $id)->get($table_name)->row();
                    if($record) {
                        $parent_id = $record->parent_id;
                        $gclosure->delete($id, false); // do not delete from the categories table
                        // save hierarchy
                        $gclosure->add($id, ($parent_id == 0 ? $id : $parent_id));
                    }
                }
            }
        }
    }
}