<?php defined('BASEPATH') || exit('No direct script access allowed');
/*=======================================================================*\
 || ##################################################################### ||
 || # GoClixy - Business Directory Script							   # ||
 || # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
 || # ----------------------------------------------------------------  # ||
 || # This file is owned and copyrighted by GoClixy.com. Your license   # ||
 || # confers  no  title  or ownership in the Script and is not a sale  # ||
 || # of any rights to the Software. This file may not be redistributed # ||
 || # in whole or significant part.									   # ||
 || # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
 || # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
 || ##################################################################### ||
 \*=======================================================================*/
class Faqs_model extends BF_Model
{
    protected $table_name	= 'faqs';
    protected $table_groups = 'training_groups';
    protected $key			= 'id';
    protected $date_format	= 'datetime';
    
    protected $log_user 	= true;
    protected $set_created	= false;
    protected $set_modified = false;
    protected $soft_deletes	= false;
    
    
    // Customize the operations of the model without recreating the insert,
    // update, etc. methods by adding the method names to act as callbacks here.
    protected $before_insert 	= array();
    protected $after_insert 	= array();
    protected $before_update 	= array();
    protected $after_update 	= array();
    protected $before_find 	    = array();
    protected $after_find 		= array();
    protected $before_delete 	= array();
    protected $after_delete 	= array();
    
    // For performance reasons, you may require your model to NOT return the id
    // of the last inserted row as it is a bit of a slow method. This is
    // primarily helpful when running big loops over data.
    protected $return_insert_id = true;
    
    // The default type for returned row data.
    protected $return_type = 'object';
    
    // Items that are always removed from data prior to inserts or updates.
    protected $protected_attributes = array();
    
    // You may need to move certain rules (like required) into the
    // $insert_validation_rules array and out of the standard validation array.
    // That way it is only required during inserts, not updates which may only
    // be updating a portion of the data.
    protected $validation_rules 		= array();
    protected $insert_validation_rules  = array();
    protected $skip_validation 			= false;
    public $modules = array('fields', 'links', 'profile_percents');
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
    }
    
    /**
     * Product FAQs Fields
     *
     * @return array
     */
    public function faqs($role = 'admin') {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'selectize',
                'name' => 'group_ids',
                'label' => 'gc_menu_groups',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => true,
                'options' => 'load:faq_groups;module:faq_groups;max_items:100',
                'options_detailed' => 1,
                'rules' => 'trim|max_length[255]',
            ),
            array(
                'type' => 'text',
                'name' => 'title',
                'label' => 'label_question',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'edit' => 1,
                'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
                'rules' => 'trim|required',
            ),
            array(
                'type' => 'textarea',
                'name' => 'description',
                'label' => 'label_answer',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'custom_rules' => 'editor',
                'field_class' => 'editor',
                'rules' => 'trim|required',
            ),
            array(
                'type' => 'text',
                'name' => 'priority',
                'label' => 'label_priority',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'value' => 0,
                'class' => 2,
                'rules' => 'trim|required|is_numeric',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'status',
                'label' => 'label_status',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_ip',
                'label' => 'created_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_by',
                'label' => 'created_by',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_ip',
                'label' => 'modified_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_by',
                'label' => 'modified_by',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            )
        );
        return $fields;
    }
    
    /**
     * Group Fields
     *
     * @return array
     */
    public function faq_groups() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'text',
                'name' => 'name',
                'label' => 'label_name',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'edit' => 1,
                'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
                'rules' => 'trim|required|max_length[150]'
            ),
            array(
                'type' => 'checkbox',
                'name' => 'name_lang',
                'label' => 'name_lang',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'iconpicker',
                'name' => 'icon',
                'label' => 'label_icon',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim'
            ),
            array(
                'type' => 'text',
                'name' => 'mobile_icon',
                'label' => 'label_mobile_icon',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim|alpha_dash'
            ),
            array(
                'type' => 'text',
                'name' => 'priority',
                'label' => 'label_priority',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'value' => 0,
                'class' => 2,
                'rules' => 'trim|required|is_numeric',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'status',
                'label' => 'label_status',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            )
        );
        return $fields;
    }
    
    /**
     * Display FAQs Data to the User
     * @param string $type
     */
    public function get_info($type = 'groups', $group_id = 0, $offset = 0, $limit = 25) {
        $table = $type == 'faqs' ? $this->table_name : $this->table_groups;
        $records = array();
        $where = "status = 1";
        if(!empty($group_id)) {
            $where .= " AND FIND_IN_SET(" . $group_id . ", group_ids)";
        }
        $rows = $this->db->where($where)->order_by('priority')->limit($limit, $offset)->get($table)->result();
        if($rows) {
            if($type == 'groups') {
                foreach($rows as $row) {
                    $record = array(
                        'id' => $row->id,
                        'title' => !empty($row->name_lang) ? lang($row->name) : $row->name,
                        'description' => $row->description,
                        'image' => !empty($row->image) ? base_url('assets/images/' . $row->image) : null,
                        'url' => '/faqs/' . $row->id
                    );
                    $records[] = $record;
                }
            } else {
                foreach($rows as $record) {
                    $row = array(
                        'id' => $record->id,
                        'title' => $record->title,
                        'description' => $record->description,
                        'hits' => $record->hits,
                        'created_on' => gc_date_format($record->created_on, true),
                        'modified_on' => gc_date_format($record->modified_on, true),
                    );
                    $records[] = $row;
                }
            }
        }
        $total_rows = $this->db->where($where)->count_all_results($table);
        return array('records' => $records, 'total_rows' => $total_rows);
    }

    /**
     * Action to be performed after faq group save
     */
    public function after_group_save($id, $action_type = 'insert', $record = array()) {
        $this->load->library('goclixy/gclosure');
        $gclosure = new Gclosure('faq_groups', 'faq_group_hierarchies');
        $record = $this->db->where('id', $id)->get('faq_groups')->row();
        if($record) {
            $parent_id = $record->parent_id;
            $gclosure->delete($id, false); // do not delete from the categories table		        
            $gclosure->add($id, ($parent_id == 0 ? $id : $parent_id)); // save hierarchy
        }
    }
}