<?php defined('BASEPATH') || exit('No direct script access allowed');

/**
 * Reports controller
 */
class Reports extends Admin_Controller
{
    protected $permissionDelete = 'Favorites.Reports.Delete';
    protected $permissionCreate   = 'Favorites.Reports.Create';
    protected $permissionView   = 'Favorites.Reports.View';

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
        
        $this->auth->restrict($this->permissionView);
        $this->load->model('favorites/favorites_model');
        $this->lang->load('favorites');
        
        $this->form_validation->set_error_delimiters("<span class='error'>", "</span>");
        
        Template::set_block('sub_nav', 'reports/_sub_nav');

        Assets::add_module_js('favorites', 'favorites.js');
    }

    /**
     * Display a list of Favorites data.
     *
     * @return void
     */
    public function index($filter='listings', $offset = 0)
    {
        // Deleting anything?
        if (isset($_POST['delete'])) {
            $this->gcrud->stats(array('module' => 'favorites'));
            $this->auth->restrict($this->permissionDelete);
            $checked = $this->input->post('checked');
            if (is_array($checked) && count($checked)) {

                // If any of the deletions fail, set the result to false, so
                // failure message is set if any of the attempts fail, not just
                // the last attempt

                $result = true;
                foreach ($checked as $pid) {
                    $deleted = $this->favorites_model->delete($pid);
                    if ($deleted == false) {
                        $result = false;
                    }
                }
                if ($result) {
                    Template::set_message(count($checked) . ' ' . sprintf(lang('msg_action_success'), lang('favorites'), lang('label_deleted')), 'success');
                } else {
                    Template::set_message(sprintf(lang('msg_delete_failure'), lang('favorites')) . $this->favorites_model->error, 'danger');
                }
            }
        }
        $module = $filter;
        // Actions done, now display the view
        $where = NULL;
        // Filters
        $filter_type = $filter;
        
        $searchterm = $this->gc->search('favorites', $this->uri->segment(6), '/reports/favorites');
        if(!empty($searchterm)) {
        	// $where .= " AND (CN.id LIKE '%". $searchterm ."%' OR CN.name LIKE '%". $searchterm ."%' OR CN.email LIKE '%". $searchterm ."%' OR CN.created_on LIKE '%". $searchterm ."%')";
        }
        
        $sort_by = "T.id DESC";
        if(isset($_POST['sort']) && $_POST['sort'] == 'U.display_name') {
        	$order_by = $this->gc->sort('favorites');
        } else if(isset($_POST['sort']) && $_POST['sort'] == 'title') {
        	$order_by = $this->gc->sort('favorites');
        } else {
        	$order_by = $this->gc->sort('favorites', 'T');
        }
        if(!empty($order_by)) {
        	$sort_by = $order_by;
        }
        
        $select = array("U.display_name AS user");
        $join = array("INNER JOIN ". $this->db->dbprefix ."users U ON U.id = T.user_id");
        $records = $this->gc->get_module_rows(0, 'favorites', $module, $where, $sort_by, $offset, $this->limit, NULL, array(), $select, $join);
        $total_records = $this->gc->get_module_rows(1, 'favorites', $module, $where, $sort_by, $offset, $this->limit, NULL, array(), $select, $join);
        
        $pagerUriSegment = 6;
        $pagerBaseUrl = site_url(SITE_AREA . '/reports/favorites/index/' . $filter);
        
        $limit  = $this->limit;

        $this->load->library('pagination');
        $pager['base_url']    = $pagerBaseUrl;
        $pager['total_rows']  = $total_records;
        $pager['per_page']    = $limit;
        $pager['uri_segment'] = $pagerUriSegment;

        $this->pagination->initialize($pager);
        Template::set('module', $module);
        Template::set('index_url', site_url(SITE_AREA .'/reports/favorites/index/'));
        Template::set('filter_type', $filter_type);

        Template::set('records', $records);
        
    	Template::set('toolbar_title', lang('favorites_manage'));

        Template::render();
    }
    
    /**
     * Create a Favorites object.
     *
     * @return void
     */
    public function create()
    {
        $this->auth->restrict($this->permissionCreate);
        
        if (isset($_POST['save'])) {
            $this->gcrud->stats(array('module' => 'favorites'));
            if ($insert_id = $this->save_favorites()) {
                log_activity($this->auth->user_id(), lang('act_create_record') . ': ' . $insert_id . ' : ' . $this->input->ip_address(), 'favorites');
                Template::set_message(lang('create_success'), 'success');

                redirect(SITE_AREA . '/reports/favorites');
            }

            // Not validation error
            if ( ! empty($this->favorites_model->error)) {
                Template::set_message(lang('create_failure') . $this->favorites_model->error, 'danger');
            }
        }

        Template::set('toolbar_title', lang('action_create'));

        Template::render();
    }
    /**
     * Allows editing of Favorites data.
     *
     * @return void
     */
    public function edit()
    {
        $id = $this->uri->segment(5);
        if (empty($id)) {
            Template::set_message(lang('invalid_id'), 'error');

            redirect(SITE_AREA . '/reports/favorites');
        }
        
        if (isset($_POST['save'])) {
            $this->gcrud->stats(array('module' => 'favorites'));
            $this->auth->restrict($this->permissionEdit);

            if ($this->save_favorites('update', $id)) {
                log_activity($this->auth->user_id(), lang('act_edit_record') . ': ' . $id . ' : ' . $this->input->ip_address(), 'favorites');
                Template::set_message(lang('edit_success'), 'success');
                redirect(SITE_AREA . '/reports/favorites');
            }

            // Not validation error
            if ( ! empty($this->favorites_model->error)) {
                Template::set_message(lang('edit_failure') . $this->favorites_model->error, 'danger');
            }
        }
        
        elseif (isset($_POST['delete'])) {
            $this->auth->restrict($this->permissionDelete);

            if ($this->favorites_model->delete($id)) {
                log_activity($this->auth->user_id(), lang('act_delete_record') . ': ' . $id . ' : ' . $this->input->ip_address(), 'favorites');
                Template::set_message(lang('delete_success'), 'success');

                redirect(SITE_AREA . '/reports/favorites');
            }

            Template::set_message(lang('delete_failure') . $this->favorites_model->error, 'danger');
        }
        
        Template::set('favorites', $this->favorites_model->find($id));

        Template::set('toolbar_title', lang('edit_heading'));
        Template::render();
    }

    //--------------------------------------------------------------------------
    // !PRIVATE METHODS
    //--------------------------------------------------------------------------

    /**
     * Save the data.
     *
     * @param string $type Either 'insert' or 'update'.
     * @param int    $id   The ID of the record to update, ignored on inserts.
     *
     * @return boolean|integer An ID for successful inserts, true for successful
     * updates, else false.
     */
    private function save_favorites($type = 'insert', $id = 0)
    {
        if ($type == 'update') {
            $_POST['id'] = $id;
        }

        // Validate the data
        $this->form_validation->set_rules($this->favorites_model->get_validation_rules());
        if ($this->form_validation->run() === false) {
            return false;
        }

        // Make sure we only pass in the fields we want
        
        $data = $this->favorites_model->prep_data($this->input->post());

        // Additional handling for default values should be added below,
        // or in the model's prep_data() method
        

        $return = false;
        if ($type == 'insert') {
            $id = $this->favorites_model->insert($data);

            if (is_numeric($id)) {
                $return = $id;
            }
        } elseif ($type == 'update') {
            $return = $this->favorites_model->update($id, $data);
        }

        return $return;
    }
    
    /**
     * Allow admin to assign favorites randomly to all users
     */
    public function assign() {
    	Assets::add_css('selectize.bootstrap3.css');
    	Assets::add_js('selectize.min.js');
    	Assets::add_module_css('users', 'users.css');
    	Assets::add_module_js('users', 'users.js');
    	$this->load->model('favorites/favorites_model');
    	$this->load->model('users/user_model');
    	$this->lang->load('listings/listings');
    	$this->user_model->users_selection();
    	if (isset($_POST['save'])) {
    	    $this->gcrud->stats(array('module' => 'favorites'));
    		$this->form_validation->set_rules('module', lang('label_module'), 'trim');
    		$this->form_validation->set_rules('applicable_users', lang('label_users'), 'trim|required');
    		$this->form_validation->set_rules('id_from', lang('label_ids'), 'required|integer|min_val[1]');
    		$this->form_validation->set_rules('id_to', lang('label_ids'), 'required|integer|max_val[10000000]|compare_fields['.$this->input->post('id_from').']|diff_between['.$this->input->post('id_from').',50000]');
    		if($this->form_validation->run() == TRUE) {
    			$users_table = false;
    			$module = $this->input->post('module');
    			$usersArray = explode(',',$this->input->post('applicable_users') ?? '');
    			// User selected ALL USERS
    			if(in_array('all', $usersArray)) {
    				$users_table = true;
    				$usersArray = $this->db->select('id, email')->get_where('users', array('active' => 1, 'deleted' => 0))->result();
    			}
    
    			$sql = "SELECT id FROM " . $this->db->dbprefix . $module ." WHERE id BETWEEN ".$this->input->post('id_from')." AND ".$this->input->post('id_to')." LIMIT 50000";
    			$rows = $this->db->query ( $sql )->result();
    			if($rows && !empty($usersArray)) {
    				foreach($rows as $row) {
    					foreach($usersArray as $user) {
    						$data = array(
    								'user_id' => $users_table ? $user->id : $user,
    								'row_id' => $row->id,
    								'module_id' => gc_module_id($module),
    								'created_ip' => $this->input->ip_address(),
    								'created_on' => date('Y-m-d H:i:s')
    						);
    						$insert_query = $this->db->insert_string('favorites', $data);
    						$insert_query = str_replace('INSERT INTO','INSERT IGNORE INTO',$insert_query);
    						$this->db->query($insert_query);
    					}
    				}
    				Template::set_message(sprintf(lang('msg_save_success'), lang('favorites')), 'success');
    			} else {
    				Template::set_message(lang('create_failure') . $this->favorites_model->error, 'danger');
    			}
    		}
    	}
    	Template::set('toolbar_title', lang('gc_menu_assign_favorites'));
    	Template::render();
    }
}