<?php  if (!defined('BASEPATH')) exit('No direct script access allowed');

class Migration_Install_favorites extends Migration
{
	/**
	 * The name of the database table
	 *
	 * @var String
	 */
	private $table_name = 'favorites';
	
	/**
	 * The Favorites table's fields
	 *
	 * @var Array
	 */
	private $fields = array(
			'id' => array(
					'type' => 'BIGINT',
					'constraint' => 20,
					'auto_increment' => TRUE,
			),
			'module_id' => array(
					'type' => 'TINYINT',
					'constraint' => 1,
					'null' => FALSE,
					'default' => 0
			),
			'row_id' => array(
					'type' => 'BIGINT',
					'constraint' => 20,
					'null' => FALSE,
			),
			'user_id' => array(
					'type' => 'BIGINT',
					'constraint' => 20,
					'null' => FALSE,
			),						
			'created_ip' => array(
					'type' => 'VARCHAR',
					'constraint' => 50,
					'null' => FALSE,
			),
    	    'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
    	    'deleted' => array( // module soft delete, also set favorites and business queries status to deleted to avoid ambiguity in the member area
    	        'type' => 'TINYINT',
    	        'constraint' => 1,
    	        'null' => FALSE,
    	        'default' => 0
    	    ),
	);
	
	/**
	 * Install this migration
	 *
	 * @return void
	 */
	public function up() {
		$this->dbforge->add_field($this->fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('user_id',false);	
		$this->dbforge->create_table($this->table_name);
		
		// Add Unique Index
		$index_sql = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " ADD UNIQUE `unique_favorite` (`module_id`, `row_id`, `user_id`)";
		$this->db->query ($index_sql);
	}
	
	/**
	 * Uninstall this migration
	 *
	 * @return void
	 */
	public function down() {
		if ($this->db->table_exists($this->table_name)) {
			$this->dbforge->drop_table($this->table_name);
		}
	}
}