<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class Feedbacks_model extends BF_Model {

	protected $table_name	= "feedbacks";
	protected $key			= "id";
	protected $soft_deletes	= TRUE;
	protected $date_format	= "datetime";

	protected $log_user 	= FALSE;

	protected $set_created	= true;
	protected $set_modified = true;
	protected $created_field     = 'created_on';
	protected $modified_field    = 'modified_on';
	protected $deleted_field     = 'deleted';

	/*
		Customize the operations of the model without recreating the insert, update,
		etc methods by adding the method names to act as callbacks here.
	 */
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 		= array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	/*
		For performance reasons, you may require your model to NOT return the
		id of the last inserted row as it is a bit of a slow method. This is
		primarily helpful when running big loops over data.
	 */
	protected $return_insert_id 	= TRUE;

	// The default type of element data is returned as.
	protected $return_type 			= "object";

	// Items that are always removed from data arrays prior to
	// any inserts or updates.
	protected $protected_attributes = array();

	/*
		You may need to move certain rules (like required) into the
		$insert_validation_rules array and out of the standard validation array.
		That way it is only required during inserts, not updates which may only
		be updating a portion of the data.
	 */
	protected $validation_rules 		= array();
	protected $insert_validation_rules 	= array();
	protected $skip_validation 			= FALSE;

	//--------------------------------------------------------------------
	/**
	 * Email Template Fields
	 *
	 * @return array
	 */
	public function email_templates($role = 'admin') {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'module_table',
	            'label' => 'label_module',
	            'frontend' => false,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'name:gc_menu_feedbacks;value:feedbacks',
	            'options_detailed' => 1,
	            'options_lang' => 1,
	            'class' => 3,
	            'rules' => 'trim|unique_email_template[feedback]',
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'user_type',
	            'label' => 'bf_user',
	            'frontend' => false,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'name:placeholder_select;value:|name:us_admin;value:admin|name:bf_user;value:user',
	            'options_detailed' => 1,
	            'options_lang' => 1,
	            'class' => 3,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'action_type',
	            'label' => 'label_action',
	            'frontend' => false,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'name:placeholder_select;value:|name:add;value:add|name:edit;value:edit|name:delete;value:delete|name:em_status_update;value:status_update',
	            'options_detailed' => 1,
	            'options_lang' => 1,
	            'class' => 3,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'name',
	            'label' => 'label_name',
	            'placeholder' => 'placeholder_name',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
	            'rules' => 'required|trim|max_length[50]'
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'subject',
	            'label' => 'label_subject',
	            'placeholder' => 'placeholder_subject',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'required|trim|max_length[255]'
	        ),
	        array(
	            'type' => 'textarea',
	            'name' => 'message',
	            'label' => 'label_message',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'custom_rules' => 'editor',
	            'field_class' => 'editor',
	            'class' => 9,
	            'rules' => 'trim|required',
	        ),
	        array(
	            'type' => 'textarea',
	            'name' => 'sms_text',
	            'label' => 'sms_text',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'class' => 9,
	            'field_extra' => 'rows:3',
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'textarea',
	            'name' => 'params',
	            'label' => 'em_params',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'class' => 9,
	            'tooltip' => lang('em_params_help'),
	            'field_extra' => 'rows:3',
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'status',
	            'label' => 'label_status',
	            'frontend' => false,
	            'admin_only' => false,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_on',
	            'label' => 'created_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_on',
	            'label' => 'modified_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        )
	    );
	    return $fields;
	}
	
	/**
	 * Get All Queries
	 * @return unknown
	 */
	public function get_all_queries () {
		$this->db->select($this->db->dbprefix . 'feedbacks.*, U.display_name');
		$this->db->join($this->db->dbprefix . 'users U','U.id = ' . $this->db->dbprefix . 'feedbacks.user_id', 'left');
		return $this->feedbacks_model->find_all();
	}
	
	/**
	 * Get listing views using date in descending order
	 * @param int $offset
	 * @param int $limit
	 * @param string $searchterm
	 * @param string $where
	 * @param number $count
	 * @return number
	 */
	public function get_feedbacks($offset, $limit, $searchterm, $where, $sort_by, $count = 0) {
		if($count == 0) {
			$sql = "SELECT CQ.*, U.display_name";
		} else {
			$sql = "SELECT COUNT(CQ.id) AS TotalCount";
		}
		$sql .= " FROM " . $this->db->dbprefix . "feedbacks CQ";
		$sql .= " LEFT JOIN " . $this->db->dbprefix . "users U ON U.id = CQ.user_id";
		$sql .= " WHERE CQ." . $where ."";
		if(!empty($searchterm)) {
			$sql .= " AND (CQ.name LIKE '%" . $searchterm . "%' OR CQ.email LIKE '%" . $searchterm . "%' OR CQ.created_on LIKE '%" . $searchterm . "%')";
		}
		if($count == 0) {
			$sql .= " ORDER BY " . $sort_by;
			$sql .= " LIMIT " . $offset . ", " . $limit;
			return $this->db->query ( $sql )->result();
		} else {
			$result = $this->db->query ( $sql )->row();
			return $result ? $result->TotalCount : 0;
		}
	}
	
	/**
	 * View Requirement
	 * @param number $id
	 * @return number
	 */
	public function get_feedbacks_view($id) {
		$sql = "SELECT CQ.*, U.display_name ";
		$sql .= " FROM " . $this->db->dbprefix . "feedbacks CQ";
		$sql .= " LEFT JOIN " . $this->db->dbprefix . "users U ON U.id = CQ.user_id";
		$sql .= " WHERE CQ.id = $id";
		return $this->db->query ( $sql )->row();
	}
}
