<?php defined('BASEPATH') || exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * LISTINGS AND OTHER MODULES FORM FIELDS
 * type_id in the fields table differentiate these fields
 * 
 * 0 - Core Field
 * 1 - Custom Field
 * 2 - Category Specific
 * 
 */
class Developer extends Admin_Controller
{
    protected $permissionCreate = 'Fields.Developer.Create';
    protected $permissionDelete = 'Fields.Developer.Delete';
    protected $permissionEdit   = 'Fields.Developer.Edit';
    protected $permissionView   = 'Fields.Developer.View';
    protected $validation_rules = array('trim', 'required', 'xss_clean', 'is_array', 'sanitize');
    

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
        
        $this->auth->restrict($this->permissionView);
        $this->load->model('fields/fields_model');
        $this->load->library('slug');
        $this->lang->load('fields');
        $this->lang->load('groups/groups');
        
        $this->form_validation->set_error_delimiters("<span class='error'>", "</span>");
        
        Template::set_block('sub_nav', 'developer/_sub_nav');
        
        // Multi Language - Clear multilanguage variables stored in the cache variable
        clear_cache('module_fields_by_key');
    }
    
    /**
     * Display all fields to the Admin
     */
    public function index($filter = 'all', $group_id = -1, $offset = 0) {
        $this->session->set_userdata('fields_last_visited_page', current_url());
    	// Status Change
        if (isset($_POST['status_change'])) {
            gc_demo_mode();
            $this->gcrud->stats(array('module' => 'fields'));
    		$this->gc->status_change('fields', $_POST['status_change'], 'status');
    	}
    	
    	// Deleting anything?
    	if (isset($_POST['delete'])) {
    	    gc_demo_mode();
    		$checked = $this->input->post('checked');
    		 
    		if (is_array($checked) && count($checked)) {
    			$result = FALSE;
    			foreach ($checked as $pid) {
    				$result = $this->fields_model->purge_field($pid);
    			}
    			 
    			if ($result) {
    				$message_label = (count($checked) == 1 ? lang('label_field') : lang('label_fields'));
    				Template::set_message(count($checked) .' '. sprintf(lang('msg_delete_success'), $message_label), 'success');
    			}
    			else {
    				Template::set_message(lang('delete_failure'), 'error');
    			}
    		}
    	}
    	
    	// Copy field with a new ID
    	if (!empty ( $_POST ['clone'] )) {
    	    gc_demo_mode();
    	    $this->gcrud->stats(array('module' => 'fields'));
    		$clone_id = $_POST ['clone'];
    		$clone_row = $this->fields_model->find($clone_id);
    		$clone_data = array();
    		if($clone_row) {
    			foreach($clone_row as $key => $value) {
    				switch($key) {
    					case 'id':
    					case 'modified_on':
    						break;
    					case 'created_on':
    						$clone_data[$key] = date('Y-m-d H:i:s');
    						break;
    					case 'name':
    						$clone_data[$key] = $value . rand(1, 1000);
    						break;
    					default:
    						$clone_data[$key] = $value;
    				}
    			}
    			$insert_id = $this->fields_model->insert($clone_data);
    			if($insert_id > 0) {
    				// save module relations
    				$relations = $this->db->get_where('field_relations', array('field_id' => $clone_id))->result();
    				if($relations) {
    					foreach($relations as $relation) {
    					    $relation_data = (array) $relation;
    					    unset($relation_data['id']);
    						unset($relation_data['modified_on']);
    						$relation_data['field_id'] = $insert_id;
    						$this->db->insert('field_relations', $relation_data);
    					}
    				}
    			}
    		}
    	}
    	
    	// Actions
    	if(!empty($_POST['update_status'])) {
    	    gc_demo_mode();
    	    $this->gcrud->stats(array('module' => 'fields'));
    		$this->update_row($_POST['update_status'], $filter, 'status');
    		Template::redirect(current_url());
    	}
    	if(!empty($_POST['admin_only'])) {
    	    gc_demo_mode();
    	    $this->gcrud->stats(array('module' => 'fields'));
    		$this->update_row($_POST['admin_only'], $filter, 'admin_only');
    		Template::redirect(current_url());
    	}
    	if(!empty($_POST['frontend'])) {
    	    gc_demo_mode();
    	    $this->gcrud->stats(array('module' => 'fields'));
    		$this->update_row($_POST['frontend'], $filter, 'frontend');
    		Template::redirect(current_url());
    	}
    	if(!empty($_POST['admin_index'])) {
    	    gc_demo_mode();
    	    $this->gcrud->stats(array('module' => 'fields'));
    		$this->update_row($_POST['admin_index'], $filter, 'admin_index');
    		Template::redirect(current_url());
    	}
    	if(!empty($_POST['frontend_index'])) {
    	    gc_demo_mode();
    	    $this->gcrud->stats(array('module' => 'fields'));
    		$this->update_row($_POST['frontend_index'], $filter, 'frontend_index');
    		Template::redirect(current_url());
    	}
    	if(!empty($_POST['field_package'])) {
    	    gc_demo_mode();
    	    $this->gcrud->stats(array('module' => 'fields'));
    	    $this->update_row($_POST['field_package'], $filter, 'field_package');
    	    Template::redirect(current_url());
    	}
    	if(!empty($_POST['display_package'])) {
    	    gc_demo_mode();
    	    $this->gcrud->stats(array('module' => 'fields'));
    	    $this->update_row($_POST['display_package'], $filter, 'display_package');
    	    Template::redirect(current_url());
    	}
    	if(!empty($_POST['search_only'])) {
    	    gc_demo_mode();
    	    $this->gcrud->stats(array('module' => 'fields'));
    		$this->update_row($_POST['search_only'], $filter, 'filter_search');
    		Template::redirect(current_url());
    	}
    	if(!empty($_POST['filter'])) {
    	    gc_demo_mode();
    	    $this->gcrud->stats(array('module' => 'fields'));
    		$this->update_row($_POST['filter'], $filter, 'filter');
    		Template::redirect(current_url());
    	}
    	
    	// type_id != 2 AND
    	switch($filter) {
    	    /*case 'categories':
    	        $where = 'F.type_id = 2';
    	        break;
    	    case 'business_queries':
    	        $where = 'F.type_id = 3';
    	        break;*/
    	    default:
    	        $where = 'module_id = ' . gc_module_id($filter);
    	}
    	
    	$searchterm = $this->gc->search('fields', $this->uri->segment(6), '/developer/fields/index/' . $filter .'/');
    	if(!empty($searchterm)) {
    		$where .= " AND (name LIKE '%". $searchterm ."%' OR created_on LIKE '%". $searchterm ."%')";
    	}
    	
    	// filter fields by groups
    	if($group_id != -1) {
    	    if(!empty($where)) {
    	        $where .= " AND";
    	    }
    	    $where .= " field_group = " . (int) $group_id;
    	}
    	
    	// Sort By
    	$sort_by = "F.id DESC";
    	$order_by = $this->gc->sort('fields');
    	if(!empty($order_by)) {
    		$sort_by = $order_by;
    	}
    	
    	$records = $this->fields_model->view_fields(0, $filter, $where, $sort_by, $offset, $this->limit);
    	 
    	// Pagination
    	$this->load->library('pagination');
    	 
    	$total_fields = $this->fields_model->view_fields(1, $filter, $where, $sort_by, $offset, $this->limit);
		
    	$pager['base_url'] = site_url(SITE_AREA ."/developer/fields/index/$filter/$group_id/");
    	$pager['total_rows'] = $total_fields;
    	$pager['per_page'] = $this->limit;
    	$pager['uri_segment']	= 7;
    	 
    	$this->pagination->initialize($pager);
    	// Group Filters
    	$record_groups = $this->db->get('groups')->result();
    	$groups = array();
    	if($record_groups) {
    	    foreach($record_groups as $group) {
    	        $groups[$group->id] = !empty($group->name_lang) ? lang('group_' . $group->name) : $group->name;
    	    }
    	}
    	Template::set(array(
    	    'group_id' => $group_id,
    	    'groups' => $groups,
    	    'records' => $records,
    	    'total_rows' => $total_fields,
    	    'index_url' => site_url(SITE_AREA .'/developer/fields/index/'),
    	    'filter_type' => $filter,
    	    'toolbar_title' => lang('manage_fields')
    	));
    	Template::render();
    }
    
    /**
     * Index Page Actions
     * @param integer $field_id
     * @param string $module
     * @param string $action
     */
    private function update_row($field_id, $module, $action) {
    	//if($module != 'all') {
    	$module_id = gc_module_id($module);
    	$where = array('field_id' => $field_id, 'module_id' => $module_id);
    	$active_value = 1;
    	// get existing value
    	if($action == 'filter') {
    		$active_value = 2;
    		$action = 'filter_search';
    	}
    	$row = $this->db->select($action)->get_where('field_relations', $where)->row();
    	if($row) {
    		$value = ($row->$action == $active_value ? 0 : $active_value);
    		$this->db->where($where)->update('field_relations', array($action => $value));
    	}    		
    	//}
    }
    
    /**
     * Allow admin to add a new field
     */
    public function create() {
        Assets::add_css('fontawesome-iconpicker.min.css');
        Assets::add_js('fontawesome-iconpicker.min.js');
        Assets::add_js("$('.icp-auto').iconpicker();", 'inline');
    	Assets::add_module_js('fields', 'fields.js');
    	$this->auth->restrict($this->permissionCreate);
    	if (isset($_POST['save']) || isset($_POST['save_only'])) {
    	    gc_demo_mode();
    	    $this->gcrud->stats(array('module' => 'fields'));
    		if ($insert_id = $this->fields_model->save_fields()) {
    			// Log the activity
    			log_activity($this->current_user->id, lang('act_create_record') .': '. $insert_id .' : '. $this->input->ip_address(), 'fields');
    			Template::set_message(sprintf(lang('msg_save_success'), lang('label_field')), 'success');
    			if(isset($_POST['save'])) {
    			    $redirect_url = $this->session->userdata('fields_last_visited_page') ? $this->session->userdata('fields_last_visited_page') : SITE_AREA . '/developer/fields';
    			    Template::redirect($redirect_url);
    			} else {
    			    Template::redirect(current_url());
    			}
    		} else {
    			Template::set_message(lang('create_failure') . $this->fields_model->error, 'danger');
    		}
    	}
    	$field_groups = $this->gc->get_module_relations('groups', 'fields');
    	Template::set(array(
    		'validation_rules' => $this->validation_rules,
    	    'validation_radio_rules' => $this->fields_model->field_options(null, 'validation_rules'),
    		'field_groups' => $field_groups,
    		'toolbar_title' => lang('create_field')
    	));
    	Template::set_view('field_form');
    	Template::render();
    }
    
    /**
     * Allow admin to edit a field
     * @param number $id
     */
    public function edit() {
        Assets::add_css('fontawesome-iconpicker.min.css');
        Assets::add_js('fontawesome-iconpicker.min.js');
        Assets::add_js("$('.icp-auto').iconpicker();", 'inline');
    	Assets::add_module_js('fields', 'fields.js');
    	$id = $this->uri->segment(5);
    	 
    	if (empty($id)) {
    		Template::set_message(lang('invalid_id'), 'error');
    		redirect(SITE_AREA .'/developer/fields');
    	}
    	$record = $this->fields_model->find($id);
    	$module_data = $this->db->get_where('field_relations', array('field_id' => $id))->result();
    	$saved_modules = array();
    	if($module_data) {
    		// get saved modules from the field relations table instead of fields table
    		$saved_modules = array();
    		foreach($module_data as $module) {
    			if($module->module_id != 0) {
    				$saved_modules[gc_module_name($module->module_id)] = 1;
    			}
    		}
    		$saved_modules['general'] = 1;
    		Template::set('saved_modules', $saved_modules);
    	}
    	
    	if (isset($_POST['save']) || isset($_POST['save_only'])) {
    	    gc_demo_mode();
    	    $this->gcrud->stats(array('module' => 'fields'));
    		$this->auth->restrict($this->permissionEdit);
    		if ($this->fields_model->save_fields('update', $id, $saved_modules)) {
    			// Log the activity
    			log_activity($this->current_user->id, lang('act_edit_record') .': '. $id .' : '. $this->input->ip_address(), 'fields');
    			Template::set_message(sprintf(lang('msg_save_success'), lang('label_field')), 'success');
    			if(isset($_POST['save'])) {
    			    $redirect_url = $this->session->userdata('fields_last_visited_page') ? $this->session->userdata('fields_last_visited_page') : SITE_AREA . '/developer/fields';
    			    Template::redirect($redirect_url);
    			} else {
    			    Template::redirect(current_url());
    			}
    		} else {
    			Template::set_message(lang('edit_failure') . $this->fields_model->error, 'danger');
    		}
    	} else if (isset($_POST['delete'])) {
    	    gc_demo_mode();
    	    $this->gcrud->stats(array('module' => 'fields'));
    		$this->auth->restrict($this->permissionDelete);
    		if ($this->fields_model->purge_field($id)) {
    			// also delete field relations
    			$this->db->where('field_id', $id)->delete('field_relations');
    			// Log the activity
    			log_activity($this->current_user->id, lang('act_delete_record') .': '. $id .' : '. $this->input->ip_address(), 'fields');
    			Template::set_message(sprintf(lang('msg_delete_success'), lang('label_field')), 'success');
    			redirect(SITE_AREA .'/developer/fields');
    		} else {
    			Template::set_message(lang('delete_failure') . $this->fields_model->error, 'danger');
    		}
    	}
    	$field_groups = $this->gc->get_module_relations('groups', 'fields');
    	Template::set(array(
    		'record' => $record,
    		'validation_rules' => $this->validation_rules,
    	    'validation_radio_rules' => $this->fields_model->field_options(null, 'validation_rules'),
    		'field_groups' => $field_groups,
    		'toolbar_title' => lang('edit_field'),
    		'module_data' => $module_data
    	));
    	Template::set_view('field_form');
    	Template::render();
    }
    
    /**
     * Allow admin to change order of fields
     */
    public function order () {
    	//LOAD NESTED SORTABLE JAVASCRIPT AND CSS FILE
    	Assets::add_css('nestedSortable/admin.css');
    	Assets::add_js('nestedSortable/jquery-ui.min.js');
    	Assets::add_js('nestedSortable/jquery.mjs.nestedSortable.js');
    
    	Assets::add_js($this->load->view('developer/order_fields_js', null, true), 'inline');
    	$groups = $this->gc->get_module_relations('groups', 'fields');
    	Template::set('groups', $groups);
    	Template::set('sortable', TRUE);
    	Template::set('toolbar_title', lang('order_fields'));
    	Template::render();
    }
    
    /**
     * Order fields using AJAX nestedSortable
     */
    public function order_ajax () {
    	$module = $this->input->post('module');
    	$field_group = $this->input->post('field_group');
    	$module_id = gc_module_id($module);
    	if($module_id != 0) {
    		// Save order from ajax call
    		if (isset($_POST['sortable'])) {
    			$this->fields_model->save_order($_POST['sortable'], $module_id, $field_group);
    		}
    		// Load view
    		$fields = $this->fields_model->get_module_fields($module_id, $field_group);
    		Template::set('result', $fields);
    	}    	
    	Template::set('toolbar_title', lang('order_fields'));
    	Template::render();
    }
    
    /**
     * Save settings
     */
    public function settings() {
        clear_cache('site_settings'); // Clear Site Settings Cache
    	$layout_options = array(
    		'' => lang('label_default'),
    		'_simple_tabs' => lang('label_simple') . ' ' . lang('tabs_only'),
    		'_wizard_tabs' => lang('label_wizard') . ' ' . lang('tabs_only')
    	);
    	$this->load->model('settings/settings_model');
    	if (isset($_POST['save'])) {
    	    gc_demo_mode();
    	    $this->gcrud->stats(array('module' => 'fields'));
    		$this->form_validation->set_rules('frontend[]', 'lang:label_frontend', 'is_array');
    		$this->form_validation->set_rules('admin[]', 'lang:label_admin_only', 'is_array');
    		$this->form_validation->set_rules('mobile[]', 'lang:label_mobile', 'is_array');
    		$this->form_validation->set_rules('frontend_layouts[]', 'lang:frontend_layouts', 'is_array');
    		$this->form_validation->set_rules('admin_layouts[]', 'lang:admin_layouts', 'is_array');
    		if($this->form_validation->run() == TRUE) {
    			$settings = array(
    				'frontend' => $this->input->post('frontend'),
    				'admin' => $this->input->post('admin'),
    				'mobile' => $this->input->post('mobile')
    			);
    			$layouts = array(
    				'frontend' => $this->input->post('frontend_layouts'),
    				'admin' => $this->input->post('admin_layouts')
    			);
    			$data = array (array('name' => 'field.settings', 'value' => json_encode($settings)), array('name' => 'field.layouts', 'value' => json_encode($layouts)));
    			$updated = $this->settings_model->update_batch($data, 'name');
    			if ($updated) {
    				Template::set_message(sprintf(lang('msg_save_success'), lang('gc_menu_settings')), 'success');
    				Template::redirect(site_url(SITE_AREA . '/developer/fields/settings'));
    			} else {
    				Template::set_message(sprintf(lang('msg_edit_failure'), lang('gc_menu_settings')), 'error');
    			}
    		}
    	}
		$setting_name = 'field.settings';
    	$result = $this->db->select('value')->get_where('settings', array('name' => $setting_name))->row();
    	if($result) {
    	    $settings = json_decode($result->value ?? '');
    	} else {
    	    $settings = json_decode(settings_item($setting_name) ?? '');
    	};
		$setting_name = 'field.layouts';
    	$result = $this->db->select('value')->get_where('settings', array('name' => $setting_name))->row();
    	if($result) {
    	    $layouts = json_decode($result->value ?? '');
    	} else {
    	    $layouts = json_decode(settings_item($setting_name) ?? '');
    	};
    	Template::set(array(
    		'settings' => $settings,
    		'layouts' => $layouts,
    		'layout_options' => $layout_options
    	));
    	Template::set('toolbar_title', lang('gc_menu_settings'));
    	Template::render();
    }
    
    /**
     * Copy fields from existing custom_fields table
     */
    public function copy_fields() {
        gc_demo_mode(array('redirect' => 'dashboard'));
        $this->gcrud->stats(array('module' => 'fields', 'redirect' => 'dashboard'));
    	$result = $this->fields_model->find_all();
    	if($result) {
    		$relation_data = array();
    		foreach($result as $row) {
    			$data = (array) $row;
    			$data['field_id'] = $row->id;
    			unset($data['id']);
    			unset($data['type_id']);
    			unset($data['name']);
    			unset($data['created_on']);
    			$data['module_id'] = 0;
    			$relation_data[] = $data;
    		}
    		$this->db->insert_batch('field_relations', $relation_data);
    		
    	}
    }
    
    /**
     * Assign fields to a Module e.g. categories/bookings
     * @param number $id
     * @param string $module
     */
    public function assign($id = 0, $module = 'categories') {
    	// module search js and css dependencies
    	Assets::add_css('selectize.bootstrap3.css');
    	Assets::add_js('selectize.min.js');
    	Assets::add_js('selectize-search.js');
    	// Display fields using datatables
    	Assets::add_js( array(
	    	'jquery.dataTables.min',
	    	'dataTables.bootstrap.min',
    	));
    	Assets::add_css('dataTables.bootstrap.min');
    	Assets::add_js( "$('#fields_table').DataTable({
		   'aoColumnDefs': [{
		        'bSortable': false,
		        'aTargets': [0] /* Disable sorting on the first column, 1st one, start by the left */
		    }],
		    order: [[5, 'asc']], /* Set default sort */
		    pageLength: 50 /* Limit number of rows */
		});", 'inline' );
    	
    	// Allow admin to change row
    	if (isset($_POST['change']) && !empty($_POST['assign_module'])) {
    		Template::redirect(site_url(SITE_AREA . '/developer/fields/assign/' . $_POST['assign_module']));
    	}
    	
    	// BOOKING MODULE SPECIFIC - Allow Admin to change the status of a field module
    	if (isset($_POST['status_change'])) {
    	    gc_demo_mode();
    	    $this->gcrud->stats(array('module' => 'fields'));
    	    $sql = "UPDATE " . $this->db->dbprefix . "field_modules SET status = NOT status WHERE id = " . $_POST['status_change'];
    	    $this->db->simple_query($sql);
    	    Template::redirect(current_url());
    	}
    	
    	// BOOKING MODULE SPECIFIC - Allow user to Add a Field on the Booking selection form
    	if (isset($_POST['is_extra'])) {
    	    gc_demo_mode();
    	    $this->gcrud->stats(array('module' => 'fields'));
    	    $sql = "UPDATE " . $this->db->dbprefix . "field_modules SET is_extra = NOT is_extra WHERE id = " . $_POST['is_extra'];
    	    $this->db->simple_query($sql);
    	    Template::redirect(current_url());
    	}
    	
    	/*$where = '';
    	$searchterm = $this->gc->search('assign_fields', $this->uri->segment(3), '/developer/fields/assign/' . $id);
    	if(!empty($searchterm)) {
    		$where = " AND (F.id LIKE '%". $searchterm ."%' oR F.name LIKE '%". $searchterm ."%')";
    	}*/
    	 
    	// Sort By
    	/*$sort_by = "id DESC";
    	$order_by = $this->gc->sort('assign_fields');
    	if(!empty($order_by)) {
    		$sort_by = $order_by;
    	}*/
    	$module_id = gc_module_id($module);
    	switch($module) {
    		case 'categories':
    			$fields = $this->fields_model->get_fields_by_type(2);
    			break;
    		case 'bookings':
    			$fields = $this->fields_model->get_fields_by_type(0, $module_id, $id);
    			break;
    	}
    	
    	
    	// Save Fields
    	if (isset($_POST['save'])) {
    	    gc_demo_mode();
    	    $this->gcrud->stats(array('module' => 'fields'));
    		$checked = $this->input->post('checked');
    		// is_array($checked) && count($checked) && 
    		if ($id > 0) {
    			// delete all existing records to deal with unchecked fields
    			// $this->db->where(array('module_id' => $module_id, 'row_id' => $id))->delete('field_modules');
    			$existing_field_modules = array();
    			$field_module_rows = $this->db->where(array('module_id' => $module_id, 'row_id' => $id))->get('field_modules')->result();
    			if($field_module_rows) {
    			    foreach($field_module_rows as $field_module_row) {
    			        $existing_field_modules[$field_module_row->field_id] = $field_module_row;
    			    }
    			}
    			if(is_array($checked) && count($checked)) { // Allow user to unassign all fields
	    			$result = true;
	    			$data = array();
	    			foreach ($checked as $pid) {
	    			    if(!array_key_exists($pid, $existing_field_modules)) {
    	    				$data[] = array(
    	    					'module_id' => $module_id,
    	    					'row_id' => $id,
    	    					'field_id' => $pid
    	    				);
	    			    } else {
	    			        unset($existing_field_modules[$pid]);
	    			    }
	    			}
	    			if(!empty($data)) {
	    			    $result = $this->db->insert_batch('field_modules', $data);
	    			}
	    			if(!empty($existing_field_modules)) {
	    			    foreach($existing_field_modules as $existing_field_module) {
	    			        $this->db->where(array('module_id' => $existing_field_module->module_id, 'row_id' => $existing_field_module->row_id, 'field_id' => $existing_field_module->field_id))->delete('field_modules');
	    			    }
	    			}
	    			if($result) {
	    				$message_label = (count($checked) == 1 ? lang('label_field') : lang('label_fields'));
	    				Template::set_message(count($checked) .' '. sprintf(lang('msg_save_success'), $message_label), 'success');
	    			} else {
    					Template::set_message(lang('edit_failure'), 'danger');
    				}
    			} else {
    				Template::set_message(sprintf(lang('msg_action_success'), lang('label_fields'), lang('action_unassigned')), 'success');
    			}
    		} else {
    			Template::set_message(lang('msg_assign_error'), 'danger');
    		}
    	}
    	$records = array();
    	$saved_result = $this->db->get_where('field_modules', array('row_id' => $id, 'module_id' => $module_id))->result();
    	if($saved_result) {
    		foreach($saved_result as $row) {
    			$records[] = $row->field_id;
    		}
    	}
    	// Display name in the selectize field
    	if($id > 0) {
    		switch($module) {
    			case 'categories':
    				$this->load->model('categories/categories_model');
    				$result_data = $this->categories_model->select('id, name')->find($id);
    				break;
    			case 'bookings':
    				$result_data = $this->db->select('id, title AS name')->get_where('booking_forms', array('id' => $id))->row();
    				break;
    		}
    		if($result_data) {
    			Template::set('selectize_value', $result_data);
    		}
    	}
    	Template::set(array(
    		'module' => $module,
    		'fields' => $fields,
    		'records' => $records,
    		'toolbar_title' => lang('assign_fields')
    	));
    	Template::render();
    }    
    
    /**
     * Synchronize created fields with database
     */
    public function sync() {
        $f = array('images', 'fields');
        $newly_added_fields = array();
        if(isset($_POST['sync'])) {
            gc_demo_mode();
            $this->gcrud->stats(array('module' => 'fields'));
            $this->form_validation->set_rules('modules[]', lang('label_module'), 'trim|required');
            if ($this->form_validation->run() === true) {
                $modules = $this->input->post('modules');
                foreach($modules as $key => $module) {
                    $module_id = gc_module_id($key);
                    $fields_added = $this->fields_model->synchronize_fields($module_id);
                    if(!empty($fields_added)) {
                        $newly_added_fields[$key] = lang($key) . ': ' . implode(',', $fields_added);
                    }
                }
                if(empty($newly_added_fields)) {
                    Template::set_message(lang('msg_nothing_to_update'), 'danger');
                } else {
                    Template::set_message(lang('field_sync_success') . ': <br />'.  implode('<br />', $newly_added_fields), 'success');
                    Template::redirect(current_url());
                }
            }
        }
        Template::set(array(
            'toolbar_title' => lang('field_sync')
        ));
        Template::render();
    }    
    
    /**
     * Manage Field Pages
     * @param string $module
     * @param string $filter
     * @param number $offset
     */
    public function manage($module = 'position_groups', $filter = 'all', $offset = 0) {
        $extra = array(
            'model' => 'fields',
            'context' => 'developer',
            'prefix' => 'field',
            'fields' => $module,
        );
        $extra['actions'] = array( // Extra buttons
            array('type' => 'link', 'link' => site_url(SITE_AREA . '/developer/fields/form/' . $module . '/edit/%s'), 'label' => '<i class="fas fa-edit fa-fw mr-1"></i>' . lang('edit'), 'class' => 'btn btn-xs btn-primary'),
        );
        Template::set('extra_action', anchor(site_url(SITE_AREA . '/developer/fields/positions'), '<i class="fas fa-arrow-circle-left fa-fw mr-1"></i>' . lang('cancel'), 'class="btn btn-danger btn-sm ml-1"'));
        $this->gc->module_view($module, $filter, $offset, 'admin', $extra);
    }
    
    /**
     * Add/Edit Options Data
     *
     * @param string $module
     * @param string $action
     *
     * @return void
     */
    public function form($module = 'position_groups', $action = 'create') {
        $params = array(
            'model' => 'fields',
            'context' => 'developer',
            'prefix' => 'field',
            'fields_method' => $module,
            'redirect_url' => site_url(SITE_AREA . '/developer/fields/manage/' . $module)
        );
        $this->gcrud->form($module, $action, 'admin', $params);
    }
    
    /**
     * Field Positions
     * @param number $id
     * @param string $module
     */
    public function positions($position_group_id = 1, $module_id = 1, $toggle_module_id = 0) {
        $this->lang->load('listings/listings');
        // module search js and css dependencies
        Assets::add_css('selectize.bootstrap3.css');
        Assets::add_js('selectize.min.js');
        Assets::add_js('selectize-search.js');
        // Display fields using datatables
        Assets::add_js( array(
            'jquery.dataTables.min',
            'dataTables.bootstrap.min',
        ));
        Assets::add_css('dataTables.bootstrap.min');
        Assets::add_js( "$('#fields_table').DataTable({
		   'aoColumnDefs': [{
		        'bSortable': false,
		        'aTargets': [0] /* Disable sorting on the first column, 1st one, start by the left */
		    }],
		    order: [[6, 'desc']], /* Set default sort */
		    pageLength: 50 /* Limit number of rows */
		});
		// destroy data table before submit and it will submit all checkbox on all pages like normal
		$('form').on('submit', function (e) {
			$('#fields_table').DataTable().destroy();
		});
		", 'inline' );
        
        // Allow admin to change row
        if (isset($_POST['change']) && !empty($_POST['module_id']) && !empty($_POST['position_group_id'])) {
            Template::redirect(site_url(SITE_AREA . '/developer/fields/positions/' . $_POST['position_group_id'] . '/' . $_POST['module_id'] . '/' . $_POST['toggle_module_id']));
        }
        $module_id = (int) $module_id;
        $position_group_id = (int) $position_group_id;
        $module = gc_module_name($module_id);
        if(!empty($_POST) && (empty($module) || empty($position_group_id))) {
            Template::redirect(SITE_AREA . '/developer/fields');
        }
        $module_toggle_id = 0;
        $id_module = 0;
        $position_where = 'FR.module_id = ' . $module_id;
        if(!empty($toggle_module_id)) {
            switch($module) {
                case 'leads':
                case 'business_queries':
                    $module_toggle_id = $toggle_module_id;
                    $id_module = gc_module_id('business_queries');
                    if($module == 'leads') {
                        $id_module = gc_module_id('leads');
                        // $position_where = " FR.module_id = " . $module_id . " OR (FM.group_id = " . $module_toggle_id . ")";
                        $position_where = " FM.group_id = " . $module_toggle_id;
                    } else {
                        $position_where = " FM.module_id = " . $id_module . " AND FIND_IN_SET(" . $module_toggle_id . ", FM.module_ids)";
                    }
                    break;
            }
        }
        if(empty($position_group_id) && empty($module_id)) {
            $fields = array();
        } else {
            $fields = $this->fields_model->position_fields(array('where' => $position_where, 'module_id' => $id_module, 'toggle_module_id' => $module_toggle_id));
        }
        // Save Fields
        if (isset($_POST['save'])) {
            gc_demo_mode();
            $this->gcrud->stats(array('module' => 'fields'));
            $checked = $this->input->post('checked');
            // is_array($checked) && count($checked) &&
            if ($module_id > 0 && $position_group_id > 0) {
                // delete all existing records to deal with unchecked fields
                $this->db->where(array('position_group_id' => $position_group_id, 'module_id' => $module_id, 'toggle_module_id' => $toggle_module_id))->delete('field_positions');
                
                if(is_array($checked) && count($checked)) { // Allow user to unassign all fields
                    $result = FALSE;
                    $data = array();
                    foreach ($checked as $pid) {
                        $data[] = array(
                            'position_group_id' => $position_group_id,
                            'module_id' => $module_id,
                            'toggle_module_id' => $module == 'leads' || $module == 'business_queries' || $module == 'categories' ? (int) $toggle_module_id : 0,
                            'field_id' => $pid,
                            'priority' => $_POST['priority'][$pid],
                            'params' => $_POST['params'][$pid],
                        );
                    }
                    $result = $this->db->insert_batch('field_positions', $data);
                    if($result) {
                        $message_label = (count($checked) == 1 ? lang('label_field') : lang('label_fields'));
                        Template::set_message(count($checked) .' '. sprintf(lang('msg_save_success'), $message_label), 'success');
                    } else {
                        Template::set_message(lang('edit_failure'), 'danger');
                    }
                } else {
                    Template::set_message(sprintf(lang('msg_action_success'), lang('label_fields'), lang('action_unassigned')), 'success');
                }
            } else {
                Template::set_message(lang('msg_assign_error'), 'danger');
            }
        }
        $records = array();
        $saved_result = $this->db->get_where('field_positions', array('position_group_id' => $position_group_id, 'module_id' => $module_id, 'toggle_module_id' => $toggle_module_id))->result();
        if($saved_result) {
            foreach($saved_result as $row) {
                $records[$row->field_id] = $row;
            }
        }
        Template::set(array(
            'position_group_id' => $position_group_id,
            'module_id' => $module_id,
            'toggle_module_id' => $toggle_module_id,
            'module' => $module,
            'fields' => $fields,
            'records' => $records,
            'toolbar_title' => lang('field_position_params')
        ));
        Template::render();
    }
    
    /**
     * Export Fields
     * E.g. realestate directory, software directory etc.
     * Try to keep all language strings in the options_lang.php file
     * @param string $module
     * INSERT INTO `rs0d_field_relations` (`id`, `module_id`, `field_id`, `field_group`, `label`, `label_lang`, `type`, `options`, `options_lang`, `options_detailed`, `value`, `placeholder`, `placeholder_lang`, `tooltip`, `tooltip_lang`, `tooltip_position`, `label_width`, `class`, `control_class`, `control_condition`, `element_container`, `field_container`, `field_class`, `validation_rules`, `custom_rules`, `field_extra`, `frontend_detail`, `frontend_index`, `admin_index`, `frontend`, `admin_only`, `field_package`, `display_package`, `multi_language`, `filter_search`, `field_report`, `priority`, `status`, `modified_on`) VALUES
(3, 0, 3, 0, 'label_package', 1, 'none', '', 0, 0, '', '', 0, '', 0, 'help-inline', 3, 0, '', '', '', '', '', '[]', '', NULL, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, '2020-04-02 13:07:37');
     */
    public function export($module, $folder = 'realestate') {
        $module_id = gc_module_id($module);
        $records = $this->db->where('module_id', $module_id)->get('field_relations')->result();
        if($records) {
            $option_type_ids = array();
            $field_ids = array();
            foreach($records as $record) {
                if(!empty($record->options) && strpos($record->options, 'name:options') !== false) {
                    $options = explode(';', $record->options);
                    if(!empty($options)) {
                        foreach($options as $option) {
                            $option_params = explode(':', $option);
                            if($option_params[0] == 'relation_value') {
                                $option_type_ids[] = $option_params[1];
                            }
                        }
                    }
                }
                $field_ids[] = $record->field_id;
            }
            $field_ids = array_unique($field_ids);
            $fields = $this->db->where('id IN (' . implode(',', $field_ids) . ')')->get('fields')->result();
            $option_type_ids = array_unique($option_type_ids);
            $option_types = $this->db->where('id IN (' . implode(',', $option_type_ids) . ')')->get('option_types')->result();
            $options = $this->db->where('type_id IN (' . implode(',', $option_type_ids) . ')')->get('options')->result();
            $content = array(
                'options' => $options,
                'option_types' => $option_types,
                'fields' => $fields,
                'field_relations' => $records,
            );
            $file_location = 'db/backups/' . $folder . '/' . $module . '.json';
            $file = APPPATH . $file_location;
            // Write the changes out...
            if (! function_exists('write_file')) {
                $this->load->helper('file');
            }
            
            // save file
            if (write_file($file, json_encode($content))) {
                Template::set_message('File exported successfully at: ' . $file_location);
            }
        }
    }
    
    /**
     * Import Fields
     * E.g. realestate directory, software directory etc.
     * @param string $module
     */
    public function import($module, $folder = 'realestate') {
        $file = APPPATH . 'db/backups/' . $folder . '/' . $module . '.json';
        if(file_exists($file)) {
            $content = file_get_contents($file);
            if(empty($content)) {
                return false;
            }
            $queries = array();
            $rows = json_decode($content, true);
            if(!empty($rows)) {
                // get option_types first, do not change the positions
                $table_changed_ids = array('option_types' => array('key' => 'type_id', 'relation_table' => 'options'), 'fields' => array('key' => 'field_id', 'relation_table' => 'field_relations'));
                $column_ids = array();  
                foreach($table_changed_ids as $table_changed_id => $relation_values) {
                    if(!empty($rows[$table_changed_id])) {
                        // $this->_import_queries($rows['fields'], 'fields');
                        // unset($rows['fields']);
                        foreach($rows[$table_changed_id] as $record) {
                            $column_ids[$table_changed_id][$record['id']] = $record;
                        }
                    }
                }
                foreach($table_changed_ids as $table => $relation_values) {
                    if(!empty($column_ids[$table])) {
                        foreach($column_ids[$table] as $rdkey => $record) {
                            $row = $this->db->where('name', $record['name'])->get($table)->row();
                            if(!$row) {
                                unset($record['id']);
                                // $this->db->insert($table, $record);
                                // $row_id = (int) $this->insert_id();
                            } else {
                                $row_id = $row->id;
                            }
                            if(!empty($row_id)) {
                                $column_ids[$table][$rdkey]['updated_row_id'] = $row_id;
                                if(!empty($rows[$relation_values['relation_table']])) {
                                    foreach($rows[$relation_values['relation_table']] as $rkey => $row_info) {
                                        if($row_info[$relation_values['key']] == $record['id']) {
                                            $rows[$relation_values['relation_table']][$rkey][$relation_values['key']] = $row_id;
                                            // update options relation values
                                            if($relation_values['relation_table'] == 'field_relations' && !empty($row_info['options']) && strpos($row_info['options'], 'name:options') !== false) {
                                                $options = explode(';', $row_info['options']);
                                                if(!empty($options)) {
                                                    foreach($options as $option) {
                                                        $option_params = explode(':', $option ?? '');
                                                        if($option_params[0] == 'relation_value') {
                                                            if(!empty($column_ids['option_types'][$option_params[1]]['updated_row_id'])) {
                                                                $old_relation_value = 'relation_value:' . $option_params[1];
                                                                $new_relation_value = 'relation_value:' . $column_ids['option_types'][$option_params[1]]['updated_row_id'];
                                                                $row_info['options'] = str_replace($old_relation_value, $new_relation_value, $row_info['options']);
                                                            }
                                                        }
                                                    }
                                                }
                                            }
                                            $queries[] = $this->_import_query($row_info, $relation_values['relation_table']);
                                            // avoid reprocessing of the same key
                                            unset($rows[$relation_values['relation_table']][$rkey]);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
            // run queries in the database
            // dump($queries);
        }
    }
    
    /**
     * Prepare Import Query
     * @param array $record
     * @param string $table
     */
    private function _import_query($record, $table) {
        foreach($record as $key => $value) {
            if($key != 'id') {
                $columns[] = '`' . $key .'`';
            }
        }
        $values = array();
        foreach($record as $k => $v) {
            if($k != 'id') {
                if(is_null($v)) {
                    $values[] = null;
                } else if(is_numeric($v)) {
                    $values[] = $v;
                } else {
                    $values[] = "'" . $v . "'";
                }
            }
        }
        $query = 'INSERT IGNORE INTO `rs0d_' . $table . '` (' . implode(', ', $columns) . ') VALUES (' . implode(', ', $values) . ');';
        return $query;
    }
    
    /**
     * Initial setup
     * @param string $access_key
     * @param string $old_access_key
     */
    public function initial_setup($access_key = null, $old_access_key = 'admin', $with_rand = false) {
        $site_url = 'https://www.goclixy.com';
        $controllers_path = APPPATH . 'controllers/';
        $directories = glob($controllers_path . '*' , GLOB_ONLYDIR);
        $admin_controllers_folder = '';
        foreach($directories as $directory) {
            $folder_name = str_replace($controllers_path, '', $directory);
            if(SITE_AREA == $folder_name) {
                $admin_controllers_folder = $folder_name;
            }
        }
        if(!empty($admin_controllers_folder)) {
            $this->load->library('slug');
            $this->load->model('users/user_model');
            $this->load->library('users/auth');
            $primary_keys = array('restricted', 'admin'); // , 'Restricted', 'Admin'
            $secondary_keys = array('Access', 'Zone', 'Entry');
            $primary = $primary_keys[array_rand($primary_keys)];
            $secondary = $secondary_keys[array_rand($secondary_keys)];
            $access_key = $primary . $secondary . ($with_rand ? rand(10, 99) : '');
            $pieces = parse_url($site_url);
            $domain = isset($pieces['host']) ? $pieces['host'] : $pieces['path'];
            $domain = str_replace('www.', '', $domain);
            
            // update admin login credentials in the users table
            $email = 'info@' . $domain;
            $random_password = $this->user_model->random_password();
            $password = $this->auth->hash_password($random_password);
            $domain_explode = explode('.', $domain ?? '');
            $display_name = ucwords($domain_explode[0]);
            $user_data = array(
                'email' => $email,
                'username' => $email,
                'display_name' => $display_name,
                'slug' => $this->slug->create_slug($display_name),
                'password' => $password['hash']
            );
            // $this->db->where(array('role_id' => 1, 'active' => 1, 'deleted' => 0))->update('users', $user_data);
            
            // update email and site name in the settings table
            $settings_data = array(
                array('name' => 'site.system_email', 'value' => $email),
                array('name' => 'site.title', 'value' => $display_name),
                array('name' => 'smtp_user', 'value' => $email),
                array('name' => 'protocol', 'value' => 'mail'), 
            );
            // $this->db->update_batch('settings', $settings_data, 'name');
            
            // update folder name in the login destination column of the roles table
            $records = $this->db->where("login_destination LIKE '%" . $admin_controllers_folder . "%'")->get('roles')->result();
            if($records) {
                $login_data = array();
                foreach($records as $record) {
                    $login_data[] = array(
                        'role_id' => $record->role_id,
                        'login_destination' => str_replace($admin_controllers_folder, $access_key, $record->login_destination)
                    );
                }
                // $this->db->update_batch('roles', $login_data, 'id');
            }
            
            // update the constants.php file
            $constants_file_path = APPPATH . 'config/constants.php';
            $constants_content = file_get_contents($constants_file_path);
            $constants_content = str_replace("define('SITE_AREA', '" . $admin_controllers_folder . "');", "define('SITE_AREA', '" . $access_key . "');", $constants_content);
            // file_put_contents($constants_file_path, $constants_content);
            
            // rename the folder in the end
            // rename($controllers_path . $admin_controllers_folder, $controllers_path . $access_key);
            // Template::set_message('');
            $admin_login_url = $site_url . '/' . $access_key . '/login';
            $admin_login_url = anchor($admin_login_url, $admin_login_url, 'target="_blank"');
            $output = <<<EOL
<h2>IMPORTANT: Please note down the credentials below, and keep them in a safe location. They will not be displayed again. You will not be able to access your site's Admin Panel without them. </h2>
<h3>
<strong>Admin URL:</strong> {$admin_login_url}<br />
<strong>Username:</strong> {$email}<br />
<strong>Password:</strong> {$random_password}
</h3>
EOL;
            echo $output;
            $file = APPPATH . 'config/setup.txt';
            // Write the changes out...
            if (! function_exists('write_file')) {
                $this->load->helper('file');
            }
            $content = date('Y-m-d H:i:s');
            // save file to avoid rerunning 
            write_file($file, json_encode($content));
        }
    }
}