<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
 || ##################################################################### ||
 || # GoClixy - Online Software Solutions							   # ||
 || # Copyright (C) 2021 GoClixy. All Rights Reserved.				   # ||
 || # ----------------------------------------------------------------  # ||
 || # This file is owned and copyrighted by GoClixy.com. Your license   # ||
 || # confers  no  title  or ownership in the Script and is not a sale  # ||
 || # of any rights to the Software. This file may not be redistributed # ||
 || # in whole or significant part.									   # ||
 || # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
 || # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
 || ##################################################################### ||
 \*=======================================================================*/
class Fields {
    /**
     * Stores the CodeIgniter core object.
     *
     * @access public
     *
     * @var object
     */
    public $CI;
    
    /**
     * @var string The name of the role/permissions ref table
     */
    private $rolePermissionsTable = 'role_permissions';
    
    /**
     * @var number The role id to which the permissions will be applied
     */
    private $roleId = '1';
    
    /**
     * @var string The name of the permissions table
     */
    private $table_permissions = 'permissions';
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct($config = array()) {
        $this->CI =& get_instance();
    }
    
    /**
     * Add/Edit/Delete permissions
     * @param string $table - Name of the Permissions Table
     * @param string $action
     *
     * Usage:
     *
     $this->fields->permissions('seo_backlinks', $this->context, array(), 'insert');
     $this->fields->permissions('seo_backlinks', $this->context, array(), 'delete');
     */
    public function permissions($table, $context = 'Components', $permission_types = array(), $type = 'insert') {
        $str_array = explode('_', $table ?? '');
        $table_words = array();
        $description_words = array();
        foreach($str_array as $str) {
            $table_words[] = ucwords($str);
            $description_words[] = strtolower($str);
        }
        $table = implode('_', $table_words);
        $context = ucwords($context);
        if(empty($permission_types)) {
            $permission_types = array('View', 'Create', 'Edit', 'Delete');
        }
        $permissions_data = array();
        foreach($permission_types as $permission_type) {
            $permissions_data[] = array(
                'name' => $table . '.' . $context . '.' . ucwords($permission_type),
                'description' => 'Allow users to ' . strtolower($permission_type) . ' ' . implode(' ', $description_words),
                'status' => 'active'
            );
        }
        if($type == 'insert') {
            $rolePermissionsData = array();
            foreach ($permissions_data as $permissionValue) {
                $permission = $this->CI->db->where('name', $permissionValue['name'])->get($this->table_permissions)->row();
                if(!$permission) {
                    $this->CI->db->insert($this->table_permissions, $permissionValue);
                    $permission_id = $this->CI->db->insert_id();
                } else {
                    $permission_id = $permission->permission_id;
                }
                $rolePermissionsData[] = array(
                    'role_id'  => $this->roleId,
                    'permission_id' => $permission_id,
                );
            }
            $this->CI->db->on_duplicate($this->rolePermissionsTable, $rolePermissionsData);
        } else if($type == 'delete') { // Uninstall Permissions
            $rolePermissionsData = array();
            foreach ($permissions_data as $permissionValue) {
                $permission = $this->CI->db->where('name', $permissionValue['name'])->get($this->table_permissions)->row();
                if($permission) {
                    // delete from the role permissions table first
                    $this->CI->db->where('permission_id', $permission->permission_id)->delete($this->rolePermissionsTable);
                    $this->CI->db->where('name', $permissionValue['name'])->delete($this->table_permissions);
                }
                
            }
        }
    }
    
    /**
     * Prepare Table Fields
     * @param string $table
     * Usage: $this->fields->prepare_fields('seo_backlinks');
     */
    public function prepare_fields($table, $dump = true) {
        // Name of the module
        $str_array = explode('_', $table ?? '');
        $table_words = array();
        foreach($str_array as $str) {
            $table_words[] = ucwords($str);
        }
        $description = implode(' ', $table_words);
        $output = '';
        $fields = $this->CI->db->field_data($table);
        if($fields) {
            $field_info = '';
            foreach($fields as $field) {
                $field_name = $field->name;
                switch($field_name) {
                    case 'id':
                        $field_info .= <<<EOL
            array(
                'type' => 'id',
            	'name' => 'id',
            	'frontend_index' => true,
            	'admin_index' => true
            ),
            
EOL;
                        break;
                    case 'user_id':
                        $field_info .= <<<EOL
            array(
                'type' => 'selectize',
                'name' => 'user_id',
                'label' => 'bf_user',
                'placeholder' => 'placeholder_selectize',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => true,
                'options' => 'load:users;module:users;max_items:1',
                'options_detailed' => 1,
                'rules' => 'trim|required',
            ),
            
EOL;
                        break;
                    case 'status_id':
                    case 'product_id':
                    case 'type_id':
                    case 'group_id':
                        if($field_name == 'status_id'){
                            $field_label = 'label_status';
                            $field_table = $table . '_statuses';
                        } else if($field_name == 'product_id'){
                            $field_label = 'singular_products';
                            $field_table = $table . '_products';
                        } else if($field_name == 'group_id'){
                            $field_label = 'singular_groups';
                            $field_table = $table . '_groups';
                        } else if($field_name == 'type_id'){
                            $field_label = 'label_type';
                            $field_table = $table . '_types';
                        }
                        $field_info .= <<<EOL
            array(
                'type' => 'dropdown',
                'name' => '{$field_name}',
                'label' => '{$field_label}',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:{$field_table};value:database',
                'options_detailed' => 1,
                'class' => 3,
                'rules' => 'trim|is_numeric',
            ),
            
EOL;
                        break;
                    case 'created_on':
                    case 'created_by':
                    case 'created_ip':
                    case 'modified_on':
                    case 'modified_by':
                    case 'modified_ip':
                        $field_info .= <<<EOL
            array(
                'type' => 'none',
                'name' => '{$field_name}',
                'label' => '{$field_name}',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            
EOL;
                        break;
                    case 'deleted':
                        break;
                    default:
                        $validation_rules = 'trim';
                        if($field->type == 'varchar') {
                            $field_type = 'text';
                            $validation_rules = 'trim|max_length[' . $field->max_length . ']';
                        } else if($field->type == 'tinyint') {
                            $field_type = 'checkbox';
                        } else if($field->type == 'textarea') {
                            $field_type = 'textarea';
                        } else { // Integer Fields
                            $field_type = 'text';
                        }
                        
                        $field_info .= <<<EOL
            array(
                'type' => '{$field_type}',
                'name' => '{$field_name}',
                'label' => 'label_{$field_name}',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => '{$validation_rules}',
            ),
            
EOL;
                        
                }
            }
            if(!empty($field_info)) {
                $output = <<<EOL
                
    /**
     * {$description} Fields
     *
     * @return array
     */
    public function {$table}() {
	\$fields = array(
{$field_info}
         );
         return \$fields;
    }
    
    
EOL;
// $output .= PHP_EOL . $field_info;
            }
            if($dump) {
                dump($output);
            } else {
                return $output;
            }
        }
    }
    
    /**
     * Generate languages
     * @param string $table - name of the table
     * @param string $prefix - remove prefix if specified e.g. product_shippings remove products_ from the module name
     */
    public function generate_languages($table, $dump = true, $prefix = '') {
        $module = ucwords(str_replace('_', ' ', $table));
        $lang = array(
            "// Manage " . $module,
            "\$lang['manage_" . $table . "']    = 'Manage " . $module . "';",
            "\$lang['create_" . $table . "']    = 'Add " . $module . "';",
            "\$lang['edit_" . $table . "']      = 'Edit " . $module . "';",
            "\$lang['" . $table . "']           = '" . $module . "';",
            "\$lang['singular_" . $table . "']  = '" . $module . "';",
            "\$lang['placeholder_" . $table . "_search'] = 'Name';",
        );
        $output = '<br />' . implode('<br />', $lang) . '<br />';
        if($dump) {
            dump($output);
        }
        return $output;
    }
}