<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Migration_Install_fields extends Migration
{
	/**
	 * The name of the database table
	 *
	 * @var String
	 */
	private $table_name = 'fields';
	private $table_field_modules = 'field_modules';
	private $table_field_relations = 'field_relations';

	/**
	 * The table's fields
	 *
	 * @var Array
	 */
	private $fields = array(
		'id' => array(
			'type' => 'SMALLINT',
			'constraint' => 5,
			'auto_increment' => TRUE,
		),
		'type_id' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default' => 0
		),
		'name' => array(
			'type' => 'VARCHAR',
			'constraint' => 100,
			'null' => FALSE,
             'unique' => TRUE
		),
		'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'modified_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP'
	);
	
	/**
	 * The field modules table's fields
	 *
	 * @var Array
	 */
	private $field_modules = array(
	    'id' => array(
	        'type' => 'INT',
	        'constraint' => 11,
	        'auto_increment' => TRUE,
	    ),
		'field_id' => array(
			'type' => 'SMALLINT',
			'constraint' => 5,
			'auto_increment' => TRUE,
		),
		'module_id' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default' => 0
		),
		'row_id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => FALSE,
			'default' => 0
		),
	    'is_extra' => array( // Booking Forms - Display selected fields (value 1) on the Booking first screen.
	        'type' => 'TINYINT',
	        'constraint' => 1,
	        'null' => FALSE,
	        'default' => 0
	    ),
	    'priority' => array(
	        'type' => 'INT',
	        'constraint' => 11,
	        'null' => FALSE,
	        'default' => 0
	    ),
	    'status' => array( // Booking Forms - Allow user to activate/deactive fields.
	        'type' => 'TINYINT',
	        'constraint' => 1,
	        'null' => FALSE,
	        'default' => 1
	    )
	);
	
	/**
	 * The field relations table's fields
	 *
	 * @var Array
	 */
	private $field_relations = array(
	    'id' => array(
	        'type' => 'INT',
	        'constraint' => 11,
	        'auto_increment' => TRUE,
	    ),
		'module_id' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default' => 0
		),
		'field_id' => array(
			'type' => 'SMALLINT',
			'constraint' => 5,
			'null' => FALSE,
			'default' => 0
		),
		'field_group' => array(
			'type' => 'SMALLINT',
			'constraint' => 5,
			'null' => FALSE,
			'default' => 0
		),
		'label' => array(
			'type' => 'VARCHAR',
			'constraint' => 100,
			'null' => FALSE
		),
		'label_lang' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default' => 0
		),
		'type' => array(
			'type' => 'CHAR',
			'constraint' => 40,
			'null' => FALSE
		),
		'options' => array(
			'type' => 'MEDIUMTEXT',
			'null' => TRUE
		),
		'options_lang' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default' => 0
		),
		'options_detailed' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default' => 0
		),
		'value' => array(
			'type' => 'MEDIUMTEXT',
			'null' => TRUE
		),
		'placeholder' => array(
			'type' => 'VARCHAR',
			'constraint' => 100,
			'null' => TRUE
		),
		'placeholder_lang' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default' => 0
		),
		'tooltip' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE
		),
		'tooltip_lang' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default' => 0
		),
		'tooltip_position' => array(
			'type' => 'CHAR',
			'constraint' => 50,
			'null' => TRUE
		),
	    'label_width' => array(
	        'type' => 'TINYINT',
	        'constraint' => 1,
	        'null' => FALSE,
	        'default' => 3
	    ),
		'class' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default' => 6
		),
		'control_class' => array(
			'type' => 'VARCHAR',
			'constraint' => 150,
			'null' => TRUE
		),
		'control_condition' => array(
			'type' => 'VARCHAR',
			'constraint' => 150,
			'null' => TRUE
		),
		'element_container' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE
		),
		'field_container' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE
		),
		'validation_rules' => array(
			'type' => 'MEDIUMTEXT',
			'null' => TRUE
		),
		'custom_rules' => array(
			'type' => 'VARCHAR',
			'constraint' => 455,
			'null' => TRUE
		),
		'frontend_index' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default' => 0
		),
		'admin_index' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default' => 0
		),
		'frontend' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default' => 0
		),
		'admin_only' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default' => 0
		),
	    'field_package' => array(
	        'type' => 'TINYINT',
	        'constraint' => 1,
	        'null' => FALSE,
	        'default' => 0
	    ),
	    'multi_language' => array(
	        'type' => 'TINYINT',
	        'constraint' => 1,
	        'null' => FALSE,
	        'default' => 0
	    ),
		'filter_search' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default' => 0
		),
		'priority' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => FALSE,
			'default' => 0
		),
		'status' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default' => 0
		),
		'modified_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
	);
	
	/**
	 * Install this migration
	 *
	 * @return void
	 */
	public function up() {
		$this->dbforge->add_field($this->fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->create_table($this->table_name);
		
		// Field Modules Table
		$this->dbforge->add_field($this->field_modules);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('module_id', false);
		$this->dbforge->add_key('row_id', false);
		$this->dbforge->add_key('field_id', false);
		$this->dbforge->create_table($this->table_field_modules);
		$unique_index = "ALTER TABLE " . $this->db->dbprefix . $this->table_field_modules . " ADD UNIQUE INDEX `unique_assignment` (`field_id`, `module_id`, `row_id`)";
		$this->db->query ($unique_index);
		
		// Field Relations Table
		$this->dbforge->add_field($this->field_relations);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('field_group', false);
		$this->dbforge->create_table($this->table_field_relations);
		$unique_relation_index = "ALTER TABLE " . $this->db->dbprefix . $this->table_field_relations . " ADD UNIQUE INDEX `unique_relation` (`module_id`, `field_id`)";
		$this->db->query ($unique_relation_index);
		
		// Field settings row
		$this->db->insert('settings', array('name' => 'field.settings', 'module' => 'field', 'value' => '{"frontend":{"listings":"tabs","blogs":"fieldset","classifieds":"tabs","deals":"fieldset","events":"fieldset","jobs":"tabs","products":"fieldset","properties":"tabs","videos":"fieldset","branches":"none"},"admin":{"listings":"tabs","blogs":"fieldset","classifieds":"fieldset","deals":"fieldset","events":"fieldset","jobs":"fieldset","products":"fieldset","properties":"tabs","videos":"fieldset","branches":"none"},"mobile":{"listings":"tabs","blogs":"tabs","classifieds":"tabs","deals":"tabs","events":"tabs","jobs":"tabs","products":"tabs","properties":"tabs","videos":"tabs","branches":"tabs"}}'));
		$this->db->insert('settings', array('name' => 'field.layouts', 'module' => 'field', 'value' => '{"frontend":{"listings":"_wizard_tabs","blogs":"","classifieds":"_wizard_tabs","deals":"","events":"","jobs":"_wizard_tabs","products":"","properties":"_wizard_tabs","videos":"","branches":""},"admin":{"listings":"_simple_tabs","blogs":"","classifieds":"","deals":"","events":"","jobs":"","products":"","properties":"_simple_tabs","videos":"","branches":""}}'));
	}

	//--------------------------------------------------------------------

	/**
	 * Uninstall this migration
	 *
	 * @return void
	 */
	public function down() {
		$this->dbforge->drop_table($this->table_name);
		$this->dbforge->drop_table($this->table_field_modules);
		$this->dbforge->drop_table($this->table_field_relations);
		// Remove settings row
		$settings = $this->db->select('name')->get_where('settings', array('name' => 'field.settings'));
		if($settings) {
			$this->db->delete('settings', array('name' => 'field.settings'));
		}
		// Remove layouts row
		$layouts = $this->db->select('name')->get_where('settings', array('name' => 'field.layouts'));
		if($layouts) {
			$this->db->delete('settings', array('name' => 'field.layouts'));
		}
	}

	//--------------------------------------------------------------------

}