<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Files controller
 */
class Content extends Admin_Controller
{
	protected $permissionCreate = 'Files.Content.Create';
	protected $permissionDelete = 'Files.Content.Delete';
	protected $permissionEdit   = 'Files.Content.Edit';
	protected $permissionView   = 'Files.Content.View';
	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct()
	{
		parent::__construct();

		$this->auth->restrict($this->permissionView);
		$this->load->library('goclixy/gfile');
		$this->load->library('crypt');
		$this->load->model('files_model', null, true);
		$this->lang->load('listings/listings');
		Template::set_block('sub_nav', 'content/_sub_nav');
	}
	
	/**
	 * Display all files to the admin
	 */
	public function index($module = 'all', $filter='all', $offset = 0) {		
		// Status Change
	    if (isset($_POST['status_change'])) {
	        $this->gcrud->stats(array('module' => 'files'));
			$this->gc->status_change('files', $_POST['status_change'], 'status', 99, false);
			$this->gfile->update_references_by_id($_POST['status_change']);
			Template::redirect(current_url());
		}
		// Do we have any actions?
		if ($action = $this->input->post('submit')) {
		    $this->gcrud->stats(array('module' => 'files'));
			$checked = $this->input->post('checked');
	
			if($checked) {
				$module_name = lang('plural_image') ;
				switch(strtolower($action)) {
					case 'activate':
						$this->gc->update_status('files', $checked, 'status', 1);
						// Update Logo and Cover Tweak field in the specified module
						foreach($checked as $checked_id) {
						    $this->gc->update_tweak_fields('files', $checked_id);
						    $this->gfile->update_references_by_id($checked_id);
						}
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_updated')), 'success');
						break;
	
					case 'deactivate':
						$this->gc->update_status('files', $checked, 'status', 0);
						// Update Logo and Cover Tweak field in the specified module
						foreach($checked as $checked_id) {
						    $this->gc->update_tweak_fields('files', $checked_id);
						    $this->gfile->update_references_by_id($checked_id);
						}
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_updated')), 'success');
						break;
	
					case 'delete':
						if (is_array($checked) && count($checked)) {
							$result = FALSE;
							foreach ($checked as $pid) {
							    $result = $this->gfile->purge_by_id($pid);
							}
	
							if ($result) {
								Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_deleted')), 'success');
							} else {
								Template::set_message(lang('delete_failure') . $this->files_model->error, 'danger');
							}
						}
						break;
				}
			}
		}
	
		// Actions done, now display the view
		$where = NULL;
	
		// Filters
		$filter_type = $filter;
	
		switch($filter_type) {
			case 'inactive':
				$where = "T.status = 0";
				break;
	
			case 'active':
				$where = "T.status = 1";
				break;
					
			case 'all':
				// Nothing to do
				break;
	
			default:
				show_404("files/index/$filter/");
		}
	
		// Pagination
		$this->load->library('pagination');
	
		// Sort By
		$sort_by = "T.id DESC";
		$order_by = $this->gc->sort('files');
		if(!empty($order_by)) {
			$sort_by = $order_by;
		}
		
		// Search Files
		$searchterm = $this->gc->search('files', $this->uri->segment(7), '/content/files/');
		if(!empty($searchterm)) {
			if(!empty($where)) {
				$where .= " AND";
			}
			$where .= " (T.id LIKE '%" .$searchterm . "%' OR T.title LIKE '%" .$searchterm . "%' OR T.row_id LIKE '%" .$searchterm . "%' OR T.original_filename LIKE '%" .$searchterm . "%')";
		}
		
		if($module == 'all') {
			$records = $this->gc->get_all_row(0, 'files', $where, $sort_by, $offset, $this->limit);
		} else {
			$records = $this->gc->get_module_rows(0, 'files', $module, $where, $sort_by, $offset, $this->limit);
		}
		
		if($module == 'all') {
			$total_rows = $this->gc->get_all_row(1, 'files', $where, $sort_by, $offset, $this->limit);
		} else {
			$total_rows = $this->gc->get_module_rows(1, 'files', $module, $where, $sort_by, $offset, $this->limit);
		}
		
		$pager['base_url'] = site_url(SITE_AREA ."/content/files/index/$module/$filter");
		$pager['total_rows'] = $total_rows;
		$pager['per_page'] = $this->limit;
		$pager['uri_segment']	= 7;
	
		$this->pagination->initialize($pager);
	
		Template::set('index_url', site_url(SITE_AREA .'/content/files/index/' . $module . '/'));
		Template::set('filter_type', $filter_type);
	
		Template::set('records', $records);
		Template::set('module', $module);
		Template::set('toolbar_title', lang('gc_menu_files'));
		Template::render();
	}
}