<?php defined('BASEPATH') || exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Online Software Solutions							   # ||
|| # Copyright (C) 2021 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Components controller
 */
class Components extends Admin_Controller {
    protected $permissionCreate = 'Forums.Components.Create';
    protected $permissionDelete = 'Forums.Components.Delete';
    protected $permissionEdit   = 'Forums.Components.Edit';
    protected $permissionView   = 'Forums.Components.View';
    
    protected $context			 = 'components';
    protected $moduleName        = 'forums';
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
        
        $this->auth->restrict($this->permissionView);
        $this->load->model('forums/forums_model');
        $this->lang->load('forums');
        
      	$this->form_validation->set_error_delimiters("<span class='error'>", "</span>");        
        Template::set_block('sub_nav', 'components/_sub_nav');        
        Assets::add_module_js('forums', 'forums.js');
    }
    
    /**
     * Display Forum Summary to the User
     */
    public function index() {
        $record = $this->forums_model->summary();
        $tabular_blocks = array();
        $tabular_blocks['top_visited'] = $this->forums_model->manage(0, null, 'M.hits DESC', 20, 0, array('module' => 'threads', 'prefix' => 'forum'));
        $tabular_blocks['recently_added'] = $this->forums_model->manage(0, null, 'M.created_on DESC', 20, 0, array('module' => 'threads', 'prefix' => 'forum'));
        $tabular_blocks['latest_posts'] = $this->forums_model->manage(0, null, 'M.created_on DESC', 20, 0, array('module' => 'posts', 'prefix' => 'forum'));
        $record->tabular_blocks = $tabular_blocks;
        //$record->recently_viewed = $this->forums_model->manage(0, null, 'M.last_viewed DESC', 20, 0, array('module' => 'views'));
        Template::set(array(
            'toolbar_title' => lang('gc_menu_forums') . ' - ' . lang('gc_menu_summary'),
            'record' => $record
        ));
        Template::render();
    }
    
    /**
     * Display a list of forums data.
     *
     * @return void
     */
    public function indexd($filter = 'all', $offset = 0) {
        $this->forums_model->view('threads', $filter, $offset);
    }
    
    /**
     * Manage Modules
     * @param string $module
     * @param string $filter
     * @param number $offset
     */
    public function manage($module = 'categories', $filter = 'all', $offset = 0) {
        switch($module) {
            case 'threads':
            case 'posts':
                $columns = array('id' => lang('label_id'), 'user_id' => lang('bf_user'), 'up_count' => lang('forum_up_count'), 'down_count' => lang('forum_down_count'), 'votes' => lang('forum_votes'), 'last_updated' => lang('label_date'), 'ip_address' => lang('ip_address'));
                Template::set('columns', $columns);
                // Load datatables to display rating_users
                Assets::add_js('//cdn.datatables.net/1.10.19/js/jquery.dataTables.min.js');
                Assets::add_js('//cdn.datatables.net/fixedheader/3.1.6/js/dataTables.fixedHeader.min.js');
                Assets::add_js('//cdn.datatables.net/responsive/2.2.3/js/dataTables.responsive.min.js');
                Assets::add_js('//cdn.datatables.net/1.10.19/js/dataTables.bootstrap4.min.js');
                // Assets::add_css('//cdn.datatables.net/1.10.19/css/jquery.dataTables.min.css');
                Assets::add_css('//cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/4.1.3/css/bootstrap.css');
                Assets::add_css('//cdn.datatables.net/1.10.19/css/dataTables.bootstrap4.min.css');
                Assets::add_css('//cdn.datatables.net/fixedheader/3.1.6/css/fixedHeader.bootstrap.min.css');
                Assets::add_css('//cdn.datatables.net/responsive/2.2.3/css/responsive.bootstrap.min.css');
                Assets::add_js('
                    $(document).on("click",".btn-votes",function() {
                        $("#votesModal").modal(\'show\');
                        var rowId = $(this).val();
                        // destroy datatable
                        if ($.fn.DataTable.isDataTable(".table-data")) {
                          $(\'.table-data\').DataTable().clear().destroy();
                        }
                        var table = $(".table-data").DataTable( {
                            "processing": true,
                            "serverSide": true,
                            "responsive": true,
                            "ajax": {
                                "url": "' . site_url(SITE_AREA . '/components/forums/get_votes') . '",
                                "data": function (d) { // pass additional parameters to the jQuery DataTable ajax call
                                     d.row_id = rowId;
                                     d.module = "' . $module . '";
                                 }
                             }
                        });
                    });', 'inline');
                break;
        }
        if($module == 'email_templates') {
            $extra = array(
                'model' => 'forums',
                'context' => 'components',
                'prefix' => 'forum',
                'fields' => $module
            );
            $this->gc->module_view($module, $filter, $offset, 'admin', $extra);            
        } else {
            $this->forums_model->view($module, $filter, $offset);
        }
    }
    
    /**
     * Add/Edit Categories/Threads/Posts/Status/Email Templates Data
     *
     * @return void
     */
    public function form($module = 'threads', $action = 'create') {
        $params = array(
            'model' => 'forums',
            'context' => $this->context,
            'fields_method' => $module,
            'set_table' => true,
            'uri_segment' => 7,
            'redirect_url' => site_url(SITE_AREA . '/' . $this->context . '/' . $this->moduleName . '/manage/' . $module)
        );
        if($module == 'threads') {
            // update_forum_posts category_id after post update to avoid unnecessary join
            $params['method'] = 'after_thread_save';
        }
        if($module == 'posts') {
            // update_forum_posts counter after post update
            $params['method'] = 'after_post_save';
        }        
        if($action == 'create') {
            switch($module) {
                case 'categories':
                case 'threads':
                case 'posts':
                    $params['record'] = (object) array('user_id' => $this->auth->user_id());
                    break;
            }
        }
        $this->gcrud->form('forum_' . $module, $action, 'admin', $params);
    }  
    
    /**
     * Update Forum Posts Counter in the Users Table
     * 
     * @return void
     */
    public function update_forum_posts_counter() {
        $this->forums_model->update_forum_posts_counter();
    }
    
    /**
     * Save settings
     */
    public function settings() {
        clear_cache('site_settings'); // Clear Site Settings Cache
        $this->load->model('settings/settings_model');
        $statuses = $this->forums_model->get_filter('statuses');
        if (isset($_POST['save'])) {
            $this->gcrud->stats(array('module' => 'forums'));
            $this->form_validation->set_rules('categories_view', 'lang:forum_categories_view', 'trim');
            $this->form_validation->set_rules('subscribers_only', 'lang:forum_subscribers_only', 'trim');
            $this->form_validation->set_rules('status', 'lang:forum_statuses', 'trim');
            if($this->form_validation->run() == TRUE) {
                $settings = array(
                    'categories_view' => $this->input->post('categories_view') ? 1 : 0,
                    'subscribers_only' => $this->input->post('subscribers_only') ? 1 : 0,
                    'status' => json_encode($this->input->post('status'))
                );
                $data = array (array('name' => 'forum.settings', 'value' => json_encode($settings)));
                $updated = $this->settings_model->update_batch($data, 'name');
                if ($updated) {
                    Template::set_message(sprintf(lang('msg_save_success'), lang('gc_menu_settings')), 'success');
                    Template::redirect(current_url());
                } else {
                    Template::set_message(sprintf(lang('msg_edit_failure'), lang('gc_menu_settings')), 'danger');
                }
            }
        }
        $setting_name = 'forum.settings';
        $result = $this->db->select('value')->get_where('settings', array('name' => $setting_name))->row();
        if($result) {
            $settings = json_decode($result->value ?? '');
        } else {
            $settings = json_decode(settings_item($setting_name) ?? '');
        }
        Template::set(array(
            'settings' => $settings,
            'statuses' => $statuses
        ));
        Template::set('toolbar_title', lang('gc_menu_settings'));
        Template::render();
    }
    
    /**
     * Convert Ticket to Forum
     * @param number $id - Ticket ID
     */
    public function to_thread($id = null) {
        Assets::add_css('selectize.bootstrap3.css');
        Assets::add_js('selectize.min.js');
        Assets::add_js('selectize-search.js');
        $this->load->library('slug');
        if(isset($_POST['save'])) {
            $this->gcrud->stats(array('module' => 'forums'));
            $this->form_validation->set_rules('ticket_id', 'lang:singular_tickets', 'trim|required');
            $this->form_validation->set_rules('category_id', 'lang:singular_forum_categories', 'trim|required');
            $this->form_validation->set_rules('status_id', 'lang:singular_forum_statuses', 'trim|required');
            $this->form_validation->set_rules('tags', 'lang:singular_tags', 'trim');
            if ($this->form_validation->run() === true) {
                $ticket_id = (int) $this->input->post('ticket_id');
                $ticket = $this->db->where('id', $ticket_id)->get('tickets')->row();
                if(empty($ticket)) {
                    Template::set_message(sprintf(lang('msg_create_failure'), lang('singular_forum_threads')), 'danger');
                    Template::redirect(current_url());
                } else {
                    $thread_id = 0;
                    // find whether this ticket is already converted to a forum or not
                    $thread = $this->db->where('ticket_id', $ticket->id)->get('forum_threads')->row();
                    if($thread) {
                        $thread_id = $thread->id;
                    }
                    $date = date('Y-m-d H:i:s');
                    $ip_address = $this->input->ip_address();
                    $category_id = $this->input->post('category_id');
                    $status_id = $this->input->post('status_id');
                    // prepare thread data
                    $thread_data = array(
                        'user_id' => $ticket->user_id,
                        'category_id' => $category_id,
                        'status_id' => $status_id,
                        'ticket_id' => $ticket->id,
                        'subject' => $ticket->subject,
                        'slug' => $this->slug->create_uri($ticket->subject, $thread_id, array('table' => 'forum_threads')),
                        'message' => $ticket->message,
                        'tags' => $this->input->post('tags'),
                        'ticket_attachments' => $ticket->attachments
                    );
                    if($thread_id != 0) {
                        $thread_data['modified_on'] = $date;
                        $thread_data['modified_ip'] = $ip_address;
                        $this->db->where('id', $thread_id)->update('forum_threads', $thread_data);
                    } else {
                        $thread_data['created_on'] = $date;
                        $thread_data['created_ip'] = $ip_address;
                        $this->db->insert('forum_threads', $thread_data);
                        $thread_id = $this->db->insert_id();
                        if($thread_id == 0) {                            
                            Template::set_message(sprintf(lang('msg_create_failure'), lang('singular_forum_threads')), 'danger');
                            Template::redirect(current_url());
                        }
                        // update tickets table with the thread_id for vice-versa conversion handling
                        $this->db->where('id', $ticket_id)->update('tickets', array('thread_id' => $thread_id));
                        $thread =  $this->db->where('id', $thread_id)->get('forum_threads')->row();
                    }
                    // get posts
                    $ticket_messages = $this->db->where('ticket_id', $ticket_id)->order_by('id', 'DESC')->get('ticket_messages')->result();
                    if($ticket_messages) {
                        foreach($ticket_messages as $ticket_message) {  
                            $post_id = 0;
                            $post = $this->db->where('ticket_message_id', $ticket_message->id)->get('forum_posts')->row();
                            if($post) {
                                $post_id = $post->id;
                            }
                            $post_data = array(
                                'user_id' => $ticket_message->user_id,
                                'category_id' => $category_id,
                                'thread_id' => $thread_id,
                                'status_id' => $status_id,
                                'ticket_message_id' => $ticket_message->id,
                                'subject' => $ticket_message->subject,
                                'slug' => !empty($post->subject) ? $this->slug->create_uri($post->subject, $post_id, array('table' => 'forum_posts')) : date('Y-m-d-H-i-s'),
                                'message' => $ticket_message->message,
                                'ticket_attachments' => $ticket_message->attachments
                            );                            
                            if($post_id != 0) {
                                $post_data['modified_on'] = $date;
                                $post_data['modified_ip'] = $ip_address;
                                $this->db->where('id', $post_id)->update('forum_posts', $post_data);
                            } else {
                                $post_data['created_on'] = $date;
                                $post_data['created_ip'] = $ip_address;
                                $this->db->insert('forum_posts', $post_data);
                                $post_id = $this->db->insert_id();
                                // update ticket_messages table with the post_id for vice-versa conversion handling
                                $this->db->where('id', $ticket_message->id)->update('ticket_messages', array('post_id' => $post_id));
                            }
                        }
                    }
                    Template::set_message(lang('forum_from_ticket_success'), 'success');
                    Template::redirect(current_url());
                }
            }
        }
        $record = array();
        $ticket = array();
        if(!empty($id)) {
            $ticket = $this->db->where('id', $id)->get('tickets')->row();
        }
        $record = $this->db->where('ticket_id', $id)->get('forum_threads')->row();
        Template::set(array(
            'toolbar_title' => lang('forum_from_ticket') . (!empty($ticket->reference_id) ? ' - ' . $ticket->reference_id : ''),
            'ticket' => $ticket,
            'record' => $record,
            'redirect_url' => current_url()
        ));
        Template::render();
    }
    
    /**
     * Convert Forum To Ticket
     * @param number $id - Ticket ID
     */
    public function to_ticket($id = null) {
        Assets::add_css('selectize.bootstrap3.css');
        Assets::add_js('selectize.min.js');
        Assets::add_js('selectize-search.js');
        $this->load->library('slug');
        $this->load->model('tickets/tickets_model');
        if(isset($_POST['save'])) {
            $this->gcrud->stats(array('module' => 'forums'));
            $this->form_validation->set_rules('thread_id', 'lang:singular_forum_threads', 'trim|required');
            $this->form_validation->set_rules('product_id', 'lang:singular_ticket_products', 'trim|required');
            $this->form_validation->set_rules('department_id', 'lang:singular_ticket_departments', 'trim|required');
            $this->form_validation->set_rules('priority_id', 'lang:singular_ticket_priorities', 'trim|required');
            $this->form_validation->set_rules('status_id', 'lang:singular_forum_statuses', 'trim|required');
            if ($this->form_validation->run() === true) {
                $thread_id = (int) $this->input->post('thread_id');
                $thread = $this->db->where('id', $thread_id)->get('forum_threads')->row();
                if(empty($thread)) {
                    Template::set_message(sprintf(lang('msg_create_failure'), lang('singular_tickets')), 'danger');
                    Template::redirect(current_url());
                } else {
                    $ticket_id = 0;
                    // find whether this thread is already converted to a ticket or not
                    $ticket = $this->db->where('thread_id', $thread->id)->get('tickets')->row();
                    if($ticket) {
                        $ticket_id = $ticket->id;
                    }
                    $date = date('Y-m-d H:i:s');
                    $ip_address = $this->input->ip_address();
                    $product_id = $this->input->post('product_id');
                    $department_id = $this->input->post('department_id');
                    $priority_id = $this->input->post('priority_id');
                    $status_id = $this->input->post('status_id');
                    // prepare ticket data
                    $ticket_data = array(
                        'user_id' => $thread->user_id,
                        'product_id' => $product_id,
                        'department_id' => $department_id,
                        'priority_id' => $priority_id,
                        'status_id' => $status_id,
                        'thread_id' => $thread_id,
                        'subject' => $thread->subject,
                        'message' => $thread->message
                    );
                    if($ticket_id != 0) {
                        $ticket_data['modified_on'] = $date;
                        $ticket_data['modified_ip'] = $ip_address;
                        $this->db->where('id', $ticket_id)->update('tickets', $ticket_data);
                    } else {
                        $ticket_data['reference_id'] = $this->tickets_model->generate_reference_id();
                        $ticket_data['created_on'] = $date;
                        $ticket_data['created_ip'] = $ip_address;
                        $this->db->insert('tickets', $ticket_data);
                        $ticket_id = $this->db->insert_id();
                        if($ticket_id == 0) {
                            Template::set_message(sprintf(lang('msg_create_failure'), lang('singular_tickets')), 'danger');
                            Template::redirect(current_url());
                        }
                        $ticket =  $this->db->where('id', $ticket_id)->get('tickets')->row();
                        // update forum_threads table with the ticket_id for vice-versa conversion handling
                        $this->db->where('id', $thread_id)->update('forum_threads', array('ticket_id' => $ticket_id));
                    }
                    // get posts
                    $posts = $this->db->where('thread_id', $thread_id)->get('forum_posts')->result();
                    if($posts) {
                        foreach($posts as $post) {
                            $ticket_message_id = 0;
                            $ticket_message = $this->db->where('post_id', $post->id)->get('ticket_messages')->row();
                            if($ticket_message) {
                                $ticket_message_id = $ticket_message_id->id;
                            }
                            $ticket_message_data = array(
                                'ticket_id' => $ticket_id,
                                'user_id' => $post->user_id,
                                'post_id' => $ticket_message_id,
                                'subject' => $post->subject,
                                'message' => $post->message
                            );
                            if($ticket_message_id != 0) {
                                $ticket_message_data['modified_on'] = $date;
                                $ticket_message_data['modified_ip'] = $ip_address;
                                $this->db->where('id', $ticket_message_id)->update('ticket_messages', $ticket_message_data);
                            } else {
                                $ticket_message_data['created_on'] = $date;
                                $ticket_message_data['created_ip'] = $ip_address;
                                $this->db->insert('ticket_messages', $ticket_message_data);
                                $ticket_message_id = $this->db->insert_id();
                                // update posts table with the ticket_message_id for vice-versa conversion handling
                                $this->db->where('id', $post->id)->update('forum_posts', array('ticket_message_id' => $ticket_message_id));
                            }
                        }
                    }
                    Template::set_message(lang('forum_to_ticket_success'), 'success');
                    Template::redirect(current_url());
                }
            }
        }
        $record = array();
        $thread = array();
        if(!empty($id)) {
            $thread = $this->db->where('id', $id)->get('forum_threads')->row();
        }
        $record = $this->db->where('thread_id', $id)->get('tickets')->row();
        Template::set(array(
            'toolbar_title' => lang('forum_to_ticket') . (!empty($thread->subject) ? ' - ' . $thread->subject : ''),
            'thread' => $thread,
            'record' => $record,
            'redirect_url' => current_url()
        ));
        Template::render();
    }
    
    /**
     * Display Votes in Threads and Posts
     */
    public function get_votes() {
        $columns = array('id', 'user_id', 'up_count', 'down_count', 'votes', 'last_updated', 'ip_address');
        $where = null;
        // $row_id, $type_id = 1
        $module = isset($_GET['module']) ? $_GET['module'] : 'threads';
        $row_id = isset($_GET['row_id']) ? (int) $_GET['row_id'] : 0;
        if($module == 'threads') {
            $module = 'forum_threads';
            $field = 'thread_id';
        } else {
            $module = 'forum_posts';
            $field = 'post_id';
        }
        $where = " M." . $field . " = " . $row_id;
        $this->gcrud->get_data($columns, $where, 'votes', 'forum');
    }
}