<?php defined('BASEPATH') || exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Online Software Solutions							   # ||
|| # Copyright (C) 2021 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Forums controller
 */
class Forums extends Front_Controller {
    protected $permissionCreate = 'Forums.Forums.Create';
    protected $permissionDelete = 'Forums.Forums.Delete';
    protected $permissionEdit   = 'Forums.Forums.Edit';
    protected $permissionView   = 'Forums.Forums.View';    
    protected $context			= 'forums';
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
        $this->load->model('forums/forums_model');
        $this->lang->load('forums');
        // load css and js files
        $this->gc->load_assets('modules/forums');
        Assets::add_module_css('forums', 'forums.css');
        Assets::add_module_js('forums', 'forums.js');
    }
    
    /**
     * Forums Member Area
     * 
     * @param string $module - Name of the module
     */
    public function member($module) {
        $this->auth->restrict();
        if(empty($_GET)) {
            Template::redirect(gc_url_format('members'));
        }
        $where = array(
            'M.user_id' => $this->auth->user_id()
        );
        $this->forums_model->info($module, null, $this->forums_model->prepare_where($_GET, $where));
    }
    
    /**
     * Display GET specific records to the user
     */
    public function view($module) {
        if(empty($_GET)) {
            Template::redirect(gc_url_format('forums'));
        }
        $this->forums_model->info($module, null, $this->forums_model->prepare_where($_GET));
    }
    
    /**
     * Display all categories to the users
     */
    public function index() {
        // if the user is not logged in
        // redirect user directly to the affiliates page after login
        if(!$this->auth->is_logged_in()) {
            $this->session->set_userdata('login_redirect_url', current_url());
        }
        $settings = json_decode(settings_item('forum.settings'), true);
        if(!empty($settings['categories_view'])) { // display categories before threads
            $module = 'categories';
        } else {
            $module = 'threads';
        }
        Template::set('toolbar_title', lang('gc_menu_forums'));
        $this->forums_model->info($module, null, array('no_params' => true, 'sort_by' => 'priority'), 'frontend');
    }
    
    /**
     * Display all categories to the users
     */
    public function categories() {
        $this->forums_model->info('categories', null, array('no_params' => true), 'frontend');
    }
    
    /**
     * Display all threads to the users
     */
    public function threads() {
        if(!empty($_GET)) {
            $extra = $this->forums_model->prepare_where($_GET, array(), 'threads');
        } else {
            $extra = array('no_params' => true);
        }
        $toolbar_title = lang('gc_menu_forums') . ' ' . lang('forum_threads');
        if(isset($_GET['popular'])) {
            $meta_data = array();
            $meta_data['page'] = 'popular';
            $meta_data['value'] = lang('forum_popular_' . $_GET['popular']);
            $extra['meta_data'] = $meta_data; 
            $toolbar_title .= ' - ' . lang('forum_popular_' . $_GET['popular']);
        }
        Template::set('toolbar_title', $toolbar_title); 
        $this->forums_model->info('threads', null, $extra, 'frontend');
    }
    
    /**
     * Display all posts to the users
     */
    public function posts() {
        $this->forums_model->info('posts', null, array('no_params' => true), 'frontend');
    }
    
    /**
     * Display a specific category to the user
     * @param string $slug
     */
    public function category($slug = null) {
        $this->forums_model->info('categories', $slug, array('page' => 'category'), 'frontend');
    }
    
    /**
     * Display a specific thread to the user
     * @param string $slug
     */
    public function thread($slug = null) {
        $this->forums_model->info('threads', $slug, array('page' => 'thread'), 'frontend');
    }
    
    /**
     * Display a specific post to the user
     * @param string $slug
     */
    public function post($slug = null) {
        $this->forums_model->info('posts', $slug, array('page' => 'post'), 'frontend');
    }
    
    /**
     * Search Function
     */
    public function search() {
		$this->gc->demo_custom_css_js('index');
        Template::set(array('module' => 'search', 'toolbar_title' => lang('gc_menu_forums') . ' ' . lang('forum_search')));
        if(!empty($_GET)) {
            $meta_data = array();
            $meta_data['page'] = 'search_results';
            $where = array();
            $extra = array(
                'message_no_record_found' => lang('label_no_record_found') . '<br />' . lang('forum_search_tips') . '<br />' . lang('forum_spelling_mistake') . '<br />' . lang('forum_alternate_keyword')
            );
            $keyword = escape($this->input->get('keyword'));            
            if(!empty($keyword)) {
                $where[] = "(M.subject LIKE '%" . $keyword . "%' OR M.message LIKE '%" . $keyword . "%')";
                $meta_data['keyword'] = $keyword;
            }            
            $author = escape($this->input->get('user_id'));
            if(!empty($author)) {
                $where[] = 'M.user_id IN (' . $author . ')';
            }
            $categories = escape($this->input->get('categories'));
            if(!empty($categories)) {
                $where[] = 'M.category_id IN (' . $categories . ')';
            }
            $module = escape($this->input->get('search_in')); 
            // Posted, New, Active and Unanswered
            if(isset($_GET['type'])) {
                $this->auth->restrict();
                $logged_in_user_info = $this->session->userdata('user_info');
                $user_id = $this->auth->user_id();
                $date = new DateTime($logged_in_user_info['last_login']);
                $last_login = $date->format('Y-m-d');
                switch($_GET['type']) {
                    case 'posted':
                        $module = 'posts';
                        // $where[] = "(T.user_id = " . $user_id . " OR M.user_id = " . $user_id . ")";
                        $where[] = "M.user_id = " . $user_id;
                        $extra['message_no_record_found'] = lang('forum_type_no_posted');
                        $extra['approved_only'] = false;
                        $meta_data['page'] = 'posted';
                        Template::set('toolbar_title', lang('forum_my') . ' ' . lang('forum_posts'));
                        break;
                    case 'threads':
                        $module = 'threads';
                        $where[] = "M.user_id = " . $user_id;
                        $extra['message_no_record_found'] = lang('forum_type_no_posted');
                        $extra['approved_only'] = false;
                        $meta_data['page'] = 'posted';
                        Template::set('toolbar_title', lang('forum_my') . ' ' . lang('forum_threads'));
                        break;
                    case 'reported':
                        $module = 'reports';
                        $where[] = "M.user_id = " . $user_id;
                        $extra['message_no_record_found'] = lang('forum_type_no_reported');
                        $extra['approved_only'] = false;
                        $meta_data['page'] = 'posted';
                        Template::set('toolbar_title', lang('forum_my') . ' ' . lang('forum_reported'));
                        break;
                    case 'new':
                        $module = 'threads';
                        $where[] = "DATE(M.created_on) > '" . $last_login . "'";
                        $extra['message_no_record_found'] = lang('forum_type_no_threads');
                        Template::set('toolbar_title', lang('new') . ' ' . lang('forum_threads'));
                        break;
                    case 'active':
                        $module = 'posts';
                        $where[] = "DATE(M.created_on) > '" . $last_login . "'";
                        $extra['message_no_record_found'] = lang('forum_type_no_posts');
                        Template::set('toolbar_title', lang('forum_active') . ' ' . lang('forum_posts'));
                        break;
                    case 'unanswered':
                        $module = 'threads';
                        $where[] = "(SELECT COUNT(*) AS counts FROM " . $this->db->dbprefix . "forum_posts FP WHERE FP.thread_id = M.id AND FP.status_id = 2) = 0";
                        $extra['message_no_record_found'] = lang('forum_type_no_unanswered');
                        Template::set('toolbar_title', lang('forum_unanswered') . ' ' . lang('forum_threads'));
                        break;
                 
                }
            }
            $tag = escape($this->input->get('tag'));
            if(!empty($tag)) {
                $module = 'threads';
                $where[] = "M.tags LIKE '%" . $tag . "%'";
            }
            if(!empty($where)) {
                $extra['where'] = implode(' AND ', $where);
            }
            if($this->input->get('sort_by')) {
                $extra['sort_by'] = escape($this->input->get('sort_by')) . ' ' . escape($this->input->get('sort_order'));
            }
            if(isset($_GET['search'])) {
                $this->session->set_userdata('forum_search', $_GET); // restore on validation error or future search
            }
            // meta information
            $extra['meta_data'] = $meta_data;
            $this->forums_model->info($module, null, $extra, 'frontend');
        } else {
            $this->load->library('parser');
            $meta_data = array(
                'site_title' => settings_item('site.title')
            );
            Template::set(array(
                'page_title' => $this->parser->parse_string(lang('forum_search_meta_title'), $meta_data, TRUE),
                'meta_description' => $this->parser->parse_string(lang('forum_search_meta_description'), $meta_data, TRUE),
                'meta_keywords' => $this->parser->parse_string(lang('forum_search_meta_keywords'), $meta_data, TRUE)
            ));
            $this->session->set_userdata('user_display_field', 'display_name'); // for security reasons, do not display email to frontend users
            Assets::add_css('selectize.bootstrap3.css');
            Assets::add_js('selectize.min.js');
            Assets::add_js('selectize-search.js');
            Template::render($this->forums_model->layout);
        }
    }
    
    /**
     * Display a list of uses data.
     *
     * @return void
     */
    public function users($offset = 0) {
        $this->gc->demo_custom_css_js('index');
        // Meta Data
        $this->forums_model->meta_info('common', array(), 'users_list');
        Template::set(array(
            'date_format' => $this->forums_model->datetime_format
        ));
        $this->gcrud->index('users', $offset, 'frontend', array(
            'model' => 'user',
            'fields_method' => 'users',
            'view' => 'forums/users',
            'render' => $this->forums_model->layout,
            'redirect_uri' => 'forums/users',
            'uri_segment' => 3,
            'toolbar_title' => lang('forum_users_list'),
            'search_fields' => array('username', 'display_name', 'designation')
        ));
    }
    
    /**
     * Display user information to the visitor
     */
    public function user() {
        if(empty($_GET['id'])) {
            Template::redirect(gc_url_format('forums'));
        }
        $id = (int) $_GET['id'];
        $this->load->model('users/user_model');
        $this->user_model->profile_detail('members', null, $id);
    }
    
    /**
     * Common Form to Post Thread/Posts/Reports
     * @param string $module
     * @param number $id
     * @param number $is_report
     * @param string $type
     */
    public function form($module, $id, $is_report = 0, $type = 'create') {
        $this->forums_model->form($module, $id,  $type, $is_report);
    }
}