<?php defined('BASEPATH') || exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Online Software Solutions							   # ||
|| # Copyright (C) 2021 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class Forums_model extends BF_Model {
    /**
     * Instructions
     * 
     * All SCSS code is in the forum layouts file, add forum specific css and js to it and set it default
     * 
     * @var string
     */
    public $table_name	             = 'forum_threads';
    protected $table_posts	         = 'forum_posts';
    protected $table_categories	     = 'forum_categories';
    protected $table_statuses	     = 'forum_statuses';
    protected $table_votes	         = 'forum_votes';
    protected $table_email_templates = 'forum_email_templates';
    protected $table_negative_keywords = 'forum_negative_keywords';
    protected $table_reports         = 'forum_reports';
	protected $key			= 'id';
	protected $date_format	= 'datetime';

	protected $log_user 	= false;
	protected $set_created	= false;
	protected $set_modified = false;
	public $soft_deletes	= false;


	// Customize the operations of the model without recreating the insert,
    // update, etc. methods by adding the method names to act as callbacks here.
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 	    = array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	// For performance reasons, you may require your model to NOT return the id
	// of the last inserted row as it is a bit of a slow method. This is
    // primarily helpful when running big loops over data.
	protected $return_insert_id = true;

	// The default type for returned row data.
	protected $return_type = 'object';

	// Items that are always removed from data prior to inserts or updates.
	protected $protected_attributes = array();

	// You may need to move certain rules (like required) into the
	// $insert_validation_rules array and out of the standard validation array.
	// That way it is only required during inserts, not updates which may only
	// be updating a portion of the data.
	protected $validation_rules 		= array();
	protected $insert_validation_rules  = array();
	protected $skip_validation 			= false;	
	protected $context			 = 'components';
	protected $moduleName        = 'forums';
	public $datetime_format = 'F d, Y h:i A';
	public $layout = '_forums';

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
    }
    
    /**
     * Forum Categories Fields
     *
     * @return array
     */
    public function categories() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'selectize',
                'name' => 'user_id',
                'label' => 'bf_user',
                'placeholder' => 'placeholder_selectize',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => true,
                'options' => 'load:users;module:users;max_items:1',
                'options_detailed' => 1,
                'rules' => 'trim|is_numeric',
            ),
            /*array(
                'type' => 'dropdown',
                'name' => 'parent_id',
                'label' => 'label_parent',
                'placeholder' => 'placeholder_parent_category',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => true,
                'options' => 'name:forum_categories;value:database',
                'options_detailed' => 1,
                'class' => 3,
                'rules' => 'trim',
            ),*/
            array(
                'type' => 'text',
                'name' => 'name',
                'label' => 'label_name',
                'placeholder' => 'placeholder_name',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'edit' => 1,
                'multi_language' => 1,
                'rules' => 'trim|required|max_length[255]'
            ),
            array(
                'type' => 'text',
                'name' => 'slug',
                'label' => 'label_slug',
                'placeholder' => 'placeholder_slug',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|max_length[255]'
            ),
            array(
                'type' => 'textarea',
                'name' => 'description',
                'label' => 'label_description',
                'placeholder' => 'placeholder_description',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'custom_rules' => 'editor',
                'field_class' => 'editor',
                'class' => 6,
                'multi_language' => 1,
                'rules' => 'trim'
            ),
            array(
                'type' => 'text',
                'name' => 'priority',
                'label' => 'label_priority',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'class' => 2,
                'value' => 0,
                'rules' => 'trim|required|is_numeric'
            ),
            array(
                'type' => 'text',
                'name' => 'hits',
                'label' => 'label_hits',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'class' => 2,
                'value' => 0,
                'rules' => 'trim|required|is_numeric'
            ),
            array(
                'type' => 'dropdown',
                'name' => 'status_id',
                'label' => 'singular_forum_statuses',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:forum_statuses;value:database',
                'options_detailed' => 1,
                'class' => 3,
                'rules' => 'trim|is_numeric|required',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'allow_thread',
                'label' => 'forum_allow_thread',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|max_length[255]',
            ),
            array(
                'type' => 'none',
                'name' => 'created_ip',
                'label' => 'created_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|max_length[255]',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|max_length[255]',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_ip',
                'label' => 'modified_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|max_length[255]',
            )
        );
        return $fields;
    }
    
    /**
     * Forum Thread Fields
     *
     * @return array
     */
    public function threads() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'selectize',
                'name' => 'user_id',
                'label' => 'bf_user',
                'placeholder' => 'placeholder_selectize',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => true,
                'options' => 'load:users;module:users;max_items:1',
                'options_detailed' => 1,
                'rules' => 'trim|required',
            ),
            array(
                'type' => 'dropdown',
                'name' => 'category_id',
                'label' => 'singular_forum_categories',
                'placeholder' => 'placeholder_select',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => true,
                'options' => 'name:forum_categories;value:database',
                'options_detailed' => 1,
                'class' => 3,
                'rules' => 'trim|required'
            ),
            array(
                'type' => 'text',
                'name' => 'subject',
                'label' => 'label_subject',
                'placeholder' => 'placeholder_subject',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'edit' => 1,
                'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
                'rules' => 'trim|required|max_length[255]'
            ),
            array(
                'type' => 'text',
                'name' => 'slug',
                'label' => 'label_slug',
                'placeholder' => 'placeholder_slug',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|max_length[255]'
            ),
            array(
                'type' => 'textarea',
                'name' => 'message',
                'label' => 'label_message',
                'placeholder' => 'placeholder_message',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'custom_rules' => 'editor',
                'field_class' => 'editor',
                'rules' => 'trim'
            ),
            array(
                'type' => 'selectize',
                'name' => 'tags',
                'label' => 'label_tags',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'field_extra' => 'field_id:input-tags',
                'options' => 'max_items:15;create:true',
                'options_detailed' => 1,
                'rules' => 'trim'
            ),
            array(
                'type' => 'text',
                'name' => 'priority',
                'label' => 'label_priority',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'class' => 2,
                'value' => 0,
                'rules' => 'trim|required|is_numeric'
            ),
            array(
                'type' => 'text',
                'name' => 'hits',
                'label' => 'label_hits',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'class' => 2,
                'value' => 0,
                'rules' => 'trim|required|is_numeric'
            ),
            array(
                'type' => 'dropdown',
                'name' => 'status_id',
                'label' => 'singular_forum_statuses',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:forum_statuses;value:database',
                'options_detailed' => 1,
                'class' => 3,
                'rules' => 'trim|is_numeric|required',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|max_length[255]',
            ),
            array(
                'type' => 'none',
                'name' => 'created_ip',
                'label' => 'created_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|max_length[255]',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|max_length[255]',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_ip',
                'label' => 'modified_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|max_length[255]',
            )
        );
        return $fields;
    }
    
    /**
     * Forum Posts Fields
     *
     * @return array
     */
    public function posts() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'selectize',
                'name' => 'user_id',
                'label' => 'bf_user',
                'placeholder' => 'placeholder_selectize',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => true,
                'options' => 'load:users;module:users;max_items:1',
                'options_detailed' => 1,
                'rules' => 'trim|required',
            ),
            array(
                'type' => 'dropdown',
                'name' => 'thread_id',
                'label' => 'singular_forum_threads',
                'placeholder' => 'placeholder_select',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => true,
                'options' => 'name:forum_threads;value:database',
                'options_detailed' => 1,
                'rules' => 'trim'
            ),
            array(
                'type' => 'text',
                'name' => 'subject',
                'label' => 'label_subject',
                'placeholder' => 'placeholder_subject',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'edit' => 1,
                'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
                'rules' => 'trim|required|max_length[255]|negative_keywords'
            ),
            array(
                'type' => 'text',
                'name' => 'slug',
                'label' => 'label_slug',
                'placeholder' => 'placeholder_slug',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|max_length[255]'
            ),
            array(
                'type' => 'textarea',
                'name' => 'message',
                'label' => 'label_message',
                'placeholder' => 'placeholder_message',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'custom_rules' => 'editor',
                'field_class' => 'editor',
                'rules' => 'trim|required|negative_keywords[1]'
            ),
            array(
                'type' => 'dropdown',
                'name' => 'status_id',
                'label' => 'singular_forum_statuses',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:forum_statuses;value:database',
                'options_detailed' => 1,
                'class' => 3,
                'rules' => 'trim|is_numeric|required',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|max_length[255]',
            ),
            array(
                'type' => 'none',
                'name' => 'created_ip',
                'label' => 'created_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|max_length[255]',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|max_length[255]',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_ip',
                'label' => 'modified_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|max_length[255]',
            )
        );
        return $fields;
    }
	
	/**
	 * Forum Status Fields
	 *
	 * @return array
	 */
	public function statuses() {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'name',
	            'label' => 'label_name',
	            'placeholder' => 'placeholder_name',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'edit' => 1,
	            'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
	            'rules' => 'trim|required|max_length[150]'
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'name_lang',
	            'label' => 'name_lang',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'colorpicker',
	            'name' => 'color',
	            'label' => 'label_color',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'class' => 2,
	            'rules' => 'trim|max_length[255]',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'priority',
	            'label' => 'label_priority',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'value' => 0,
	            'class' => 2,
	            'rules' => 'trim|required|is_numeric',
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'status',
	            'label' => 'label_status',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        )
	    );
	    return $fields;
	}
	
	/**
	 * Forum Reports Fields
	 *
	 * @return array
	 */
	public function reports() {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'selectize',
	            'name' => 'user_id',
	            'label' => 'bf_user',
	            'placeholder' => 'placeholder_selectize',
	            'frontend' => false,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => true,
	            'options' => 'load:users;module:users;max_items:1',
	            'options_detailed' => 1,
	            'rules' => 'trim|required',
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'module_id',
	            'label' => 'label_module',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'name:forum_threads;value:1|name:forum_posts;value:2',
	            'options_detailed' => 1,
	            'options_lang' => 1,
	            'class' => 2,
	            'rules' => 'trim|required'
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'row_id',
	            'label' => 'label_id',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'value' => 0,
	            'class' => 3,
	            'rules' => 'trim|required|is_numeric',
	        ),
	        array(
	            'type' => 'textarea',
	            'name' => 'description',
	            'label' => 'label_description',
	            'placeholder' => 'placeholder_description',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'class' => 6,
	            'rules' => 'trim|required',
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'status_id',
	            'label' => 'singular_forum_statuses',
	            'frontend' => false,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'name:forum_statuses;value:database',
	            'options_detailed' => 1,
	            'class' => 3,
	            'rules' => 'required|trim|max_length[255]',
	        )
	    );
	    return $fields;
	}
	
	/**
	 * Email Template Fields
	 *
	 * @return array
	 */
	public function email_templates($role = 'admin') {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'module_table',
	            'label' => 'label_module',
	            'frontend' => false,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'name:placeholder_select;value:|name:singular_forum_threads;value:forum_threads|name:singular_forum_posts;value:forum_posts|name:singular_forum_categories;value:forum_categories',
	            'options_detailed' => 1,
	            'options_lang' => 1,
	            'class' => 3,
	            'rules' => 'trim|unique_email_template[forum]',
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'user_type',
	            'label' => 'bf_user',
	            'frontend' => false,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'name:placeholder_select;value:|name:label_admin;value:admin|name:bf_user;value:user',
	            'options_detailed' => 1,
	            'options_lang' => 1,
	            'class' => 3,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'action_type',
	            'label' => 'label_action',
	            'frontend' => false,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'name:placeholder_select;value:|name:add;value:add|name:edit;value:edit|name:delete;value:delete|name:forum_status_update;value:status_update',
	            'options_detailed' => 1,
	            'options_lang' => 1,
	            'class' => 3,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'name',
	            'label' => 'label_name',
	            'placeholder' => 'placeholder_name',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
	            'rules' => 'required|trim|max_length[50]'
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'subject',
	            'label' => 'label_subject',
	            'placeholder' => 'placeholder_subject',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'required|trim|max_length[255]'
	        ),
	        array(
	            'type' => 'textarea',
	            'name' => 'message',
	            'label' => 'label_message',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'custom_rules' => 'editor',
	            'field_class' => 'editor',
	            'class' => 9,
	            'rules' => 'trim|required',
	        ),
	        array(
	            'type' => 'textarea',
	            'name' => 'sms_text',
	            'label' => 'sms_text',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'class' => 9,
	            'field_extra' => 'rows:3',
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'textarea',
	            'name' => 'params',
	            'label' => 'label_params',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'class' => 9,
	            'tooltip' => lang('emailer_params_help'),
	            'field_extra' => 'rows:3',
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'status',
	            'label' => 'label_status',
	            'frontend' => false,
	            'admin_only' => false,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_on',
	            'label' => 'created_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_on',
	            'label' => 'modified_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        )
	    );
	    return $fields;
	}
	
	/**
	 * Negative Keywords Fields
	 *
	 * @return array
	 */
	public function negative_keywords($role = 'admin') {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'name',
	            'label' => 'label_name',
	            'placeholder' => 'placeholder_name',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
	            'rules' => 'trim|required|max_length[255]'
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_on',
	            'label' => 'created_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim|max_length[255]',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_on',
	            'label' => 'modified_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim|max_length[255]',
	        )
	    );
	    return $fields;
	}
	
	/**
	 * Get Filter
	 * @param string $module
	 * @return NULL[]
	 */
	public function get_filter($module, $user_id = 0, $field_name = 'user_id', $table = null) {
	    $column_name = '';
	    switch($module) {
	        case 'statuses':
	            $column_name = 'status_id';
	            break;
	        case 'categories':
	            $column_name = 'category_id';
	            break;
	    }
	    $module = 'forum_' . $module;
	    $result = array();
	    $records = $this->db->get($module)->result();
	    if($records) {
	        $where = array();
	        if(!empty($user_id)) {
	            $where[$field_name] = $user_id;
	        }
	        $count_table = empty($table) ? $this->table_name : 'forum_' . $table;
	        foreach($records as $record) {
	            $where[$column_name] = $record->id;
	            $result[$record->id] = array(
	                'id' => $record->id,
	                'name' => !empty($record->name_lang) ? lang($record->name) : $record->name,
	                'counts' => $this->db->where($where)->count_all_results($count_table)
	            );
	        }
	    }
	    return $result;
	}
	
	/**
	 * Dropdown option for forums
	 * @param string $module - Name of the module
	 * @return string[]
	 */
	public function dropdown_options($module, $prefix = 'forum_') {	    
	    $options = array();
	    $records = $this->db->get($prefix . $module)->result();
	    if($records) {
	        foreach($records as $record) {
	            $options[$record->id] = !empty($record->name_lang) ? lang($record->name) : $record->name;
	        }
	    }
	    return $options;
	}
	
	/**
	 * Get Product Upgrades/Combos/Variants/Notifications/Wishlists etc.
	 * @param number $count
	 * @param string $where
	 * @param string $sort_by
	 * @param number $limit
	 * @param number $offset
	 * @return array|number
	 */
	public function manage($count = 0, $where = null, $sort_by = 'M.id DESC', $limit = 15, $offset = 0, $extra = array()) {
	    if(!isset($extra['module'])) {
	        return false;
	    }
	    $module = $extra['module'];
	    if($count == 1) {
	        $sql = "SELECT COUNT(*) AS total_count";
	    } else {
	        switch($module) {
	            case 'threads':
	                $sql = "SELECT M.*, S.color AS status_color, S.name AS status, S.name_lang AS status_lang, C.name AS category, "  . $this->user_fields() . PHP_EOL;
	                $sql .= ", " . query_fields('threads');
	                if(isset($extra['view'])) { // To be displayed through the info function only
	                    $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . "forum_posts WHERE thread_id = M.id AND deleted = 0) AS total_posts";
	                    $sql .= ", (SELECT CONCAT(FPU.id, '!~!', FPU.username, '!~!', FPU.email, '!~!', FPU.display_name, '!~!', FP.created_on, '!~!', FP.id)";
	                    $sql .= " FROM " . $this->db->dbprefix . "forum_posts FP";
	                    $sql .= " INNER JOIN " . $this->db->dbprefix . "users FPU ON FPU.id = FP.user_id WHERE FP.thread_id = M.id AND FP.status_id = 2 ORDER BY FP.id DESC LIMIT 1) AS last_post";
	                }
	                break;
	            case 'categories':
	                $sql = "SELECT M.*, S.color AS status_color, S.name AS status, S.name_lang AS status_lang, " . $this->user_fields() . PHP_EOL;
	                if(isset($extra['view'])) { // To be displayed through the info function only
    	                $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . "forum_threads WHERE category_id = M.id AND deleted = 0) AS category_total_threads";
    	                $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . "forum_posts WHERE category_id = M.id AND deleted = 0) AS category_total_posts";
    	                $sql .= ", (SELECT CONCAT(FPU.id, '!~!', FPU.username, '!~!', FPU.email, '!~!', FPU.display_name, '!~!', FP.created_on, '!~!', FP.id, '!~!', FP.id, '!~!', FP.slug)";
    	                // $sql .= " FROM " . $this->db->dbprefix . "forum_posts FP ";
    	                // $sql .= " INNER JOIN " . $this->db->dbprefix . "forum_threads FPT ON FPT.id = FP.thread_id";
    	                $sql .= " FROM " . $this->db->dbprefix . "forum_threads FP ";
    	                $sql .= " INNER JOIN " . $this->db->dbprefix . "users FPU ON FPU.id = FP.user_id";
    	                $sql .= " WHERE FP.category_id = M.id ORDER BY FP.id DESC LIMIT 1) AS category_last_post";
	                }
	                break;
	            case 'posts':
	                $sql = "SELECT M.*, T.subject AS thread, T.slug AS thread_slug, C.name AS category, C.slug AS category_slug, S.color AS status_color, S.name AS status, S.name_lang AS status_lang, " . $this->user_fields() . PHP_EOL;
	                $sql .= ", " . query_fields('posts');
	                break;
	            case 'reports':
	                $sql = "SELECT M.*, S.color AS status_color, S.name AS status, S.name_lang AS status_lang, " . $this->user_fields() . PHP_EOL;
	                $sql .= ", CASE WHEN M.module_id = 1 THEN";
	                $sql .= " (SELECT CONCAT(FT.subject, '!~!', FT.slug) FROM " . $this->db->dbprefix . "forum_threads FT WHERE FT.id = M.row_id)";
	                $sql .= " ELSE";	                    
	                $sql .= " (SELECT CONCAT(FP.subject, '!~!', FP.slug) FROM " . $this->db->dbprefix . "forum_posts FP WHERE FP.id = M.row_id)";
	                $sql .= " END AS row_info"; //  Note: row is a reserved keyword
	                break;
	            case 'votes':
	                $sql = "SELECT M.*, " . user_fields();
	                break;
	            default:
	                $sql = "SELECT M.*";
	        }
	    }
	    $table = 'forum_' . $module;
	    $sql .= " FROM " . $this->db->dbprefix . $table . " M";
	    if($count == 0) {
	        switch($module) {
	            case 'threads':
	                $sql .= " LEFT JOIN " . $this->db->dbprefix . "users U ON U.id = M.user_id";
	                $sql .= " LEFT JOIN " . $this->db->dbprefix . $this->table_categories . " C ON C.id = M.category_id";
	                $sql .= " LEFT JOIN " . $this->db->dbprefix . $this->table_statuses . " S ON S.id = M.status_id";
	                /*if(isset($extra['search'])) { // search, posted, new, active and unanswered posts 
	                    $sql .= " LEFT JOIN " . $this->db->dbprefix . $this->table_posts . " P ON P.thread_id = M.thread_id";
	                }*/
	                break;
	            case 'categories':
	                $sql .= " LEFT JOIN " . $this->db->dbprefix . "users U ON U.id = M.user_id";
	                $sql .= " LEFT JOIN " . $this->db->dbprefix . $this->table_statuses . " S ON S.id = M.status_id";
	                break;
	            case 'posts':
	                $sql .= " LEFT JOIN " . $this->db->dbprefix . "users U ON U.id = M.user_id";
	                $sql .= " LEFT JOIN " . $this->db->dbprefix . $this->table_name . " T ON T.id = M.thread_id";
	                $sql .= " LEFT JOIN " . $this->db->dbprefix . $this->table_categories . " C ON C.id = M.category_id";
	                $sql .= " LEFT JOIN " . $this->db->dbprefix . $this->table_statuses . " S ON S.id = M.status_id";
	                break;
	            case 'reports':
	                $sql .= " LEFT JOIN " . $this->db->dbprefix . "users U ON U.id = M.user_id";
	                $sql .= " LEFT JOIN " . $this->db->dbprefix . $this->table_statuses . " S ON S.id = M.status_id";
	                break;
	            case 'votes':
	                $sql .= " INNER JOIN " . $this->db->dbprefix . "users U ON U.id = M.user_id";
	                break;
	        }
	    }
	    if(isset($extra['filter']) && $extra['filter'] == 'deleted') {
	        $sql .= " WHERE M.deleted = 1";
	    } else {
	        $sql .= " WHERE M.deleted = 0";
	    }
	    if(!empty($where)) {
	        $sql .= ' AND ' . $where;
	    }
	    if ($count == 0) {
	        $sql .= " GROUP BY M.id ORDER BY " . $sort_by;
	        if ($limit != 0) {
	            $sql .= " LIMIT " . $offset . ", " . $limit;
	        } else {
	            $sql .= " LIMIT $limit";
	        }
	        $query = $this->db->query($sql);
	        return $query->result();
	    } else {
	        $result = $this->db->query($sql)->row();
	        return $result ? $result->total_count : 0;
	    }
	}	
	
	/**
	 * Forums Summary
	 * @return object|[]
	 */
	public function summary() {
	    /*
	    // count number of visitors viewing this site using sessions table
	    // the timestamp is udpated on page refresh and view
	    // The method is not 100% accurate but it solve the purpose
	    $now = strtotime("-5 minutes"); // get last fiveminutes timestamp
	    $sql = "SELECT COUNT(*) AS guests_online FROM " . $this->db->dbprefix . "ci3_sessions WHERE timestamp > " . $now;
	    $result = $this->db->query($sql)->row();
	    return $result->guests_online;
	    */
	    $now = strtotime("-5 minutes"); // get last fiveminutes timestamp
	    $sql = "SELECT COUNT(*) AS total_categories";
	    $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . "users) AS total_users";
	    $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . "users WHERE is_online = 1) AS online_users";
	    $sql .= ", (SELECT CONCAT(id, '!~!', username, '!~!', email, '!~!', display_name, '!~!', created_on) FROM " . $this->db->dbprefix . "users ORDER BY id DESC LIMIT 1) AS last_registered_user";
	    $sql .= ", (SELECT GROUP_CONCAT(CONCAT(id, '!~!', username, '!~!', email, '!~!', display_name, '!~!', created_on) ORDER BY role_id ASC SEPARATOR '~~~') FROM " . $this->db->dbprefix . "users WHERE is_online = 1 LIMIT 10) AS online_users_info";
	    $sql .= ", (SELECT COUNT(*) AS guests_online FROM " . $this->db->dbprefix . "ci3_sessions WHERE timestamp > " . $now . " LIMIT 10) AS guests_online";
	    $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . "forum_threads) AS total_threads";
	    $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . "forum_posts) AS total_posts";
	    $sql .= " FROM " . $this->db->dbprefix . $this->table_categories;
	    $query = $this->db->query($sql); $record = $query->row();
	    if($record) {
	        $data = array();
	        $i = 1;
	        foreach($record as $key => $value) {
	            switch($key) {
	                case 'total_users':
	                    $url = site_url(SITE_AREA . '/settings/users');
	                    break;
	                case 'total_categories':
	                    $url = site_url(SITE_AREA . '/components/forums/manage/categories');
	                    break;
	                case 'total_threads':
	                    $url = site_url(SITE_AREA . '/components/forums/manage/threads');
	                    break;
	                case 'total_posts':
	                    $url = site_url(SITE_AREA . '/components/forums/manage/posts');
	                    break;
	                default:
	                    $url = null;
	            }
	            $data[$key] = array(
	                'value' => $value,
	                'url' => $url
	            );
	            $i++;
	        }
	        return (object) $data;
	    } else {
	        return $record;
	    }
	}
	
	/**
	 * Get Thread/Post/Category Row
	 * @param string $module
	 * @param number $id
	 * @param boolean $is_pdf
	 * @param string $where
	 * @param boolean $deleted - FALSE - Email notification on delete
	 * @return array|number
	 */
	public function get_row($module, $id, $is_pdf = false, $where = null, $deleted = true) {
	    $sql = "SELECT M.*, " . $this->user_fields();
	    if($module == 'threads') {
	        $sql .= ", C.name AS category, C.slug AS category_slug";
	    }
	    if($module == 'posts') {
	        $sql .= ", T.subject AS thread, T.slug AS thread_slug, T.category_id, C.name AS category, C.slug AS category_slug";
	    }
	    $sql .= ", ST.color AS status_color, ST.name_lang AS status_lang, ST.name AS status";
	    $sql .= " FROM " . $this->db->dbprefix . 'forum_' . $module . " M" . PHP_EOL;
	    if($module == 'threads') {
	        $sql .= " LEFT JOIN " . $this->db->dbprefix . $this->table_categories . " C ON C.id = M.category_id" . PHP_EOL;
	    }
	    if($module == 'posts') {
	        $sql .= " LEFT JOIN " . $this->db->dbprefix . "forum_threads T ON T.id = M.thread_id" . PHP_EOL;
	        $sql .= " LEFT JOIN " . $this->db->dbprefix . $this->table_categories . " C ON C.id = M.category_id" . PHP_EOL;
	    }
	    $sql .= " LEFT JOIN " . $this->db->dbprefix . "users U ON U.id = M.user_id" . PHP_EOL;
	    $sql .= " LEFT JOIN " . $this->db->dbprefix . $this->table_statuses .  " ST ON ST.id = M.status_id" . PHP_EOL;
	    if(!$deleted) {
	        $sql .= " WHERE M.status_id != 100"; // random value given
	    } else {
	        $sql .= " WHERE M.deleted = 0";
	    }
	    if(!empty($id)) {
	        $sql .= " AND M.id = " . $id;
	    }
	    if(!empty($where)) {
	        $sql .= " AND " . $where;
	    }
	    $result = $this->db->query($sql)->row();
	    return $result;
	}
	
	/**
	 * Display number of posts and threads on the User's profile page
	 * @param number $id
	 * @return object
	 */
	public function users_count($id) {	    
	    $sql = "SELECT COUNT(*) AS total_posts";
	    $sql .= ", (SELECT COUNT(*) AS total_threads FROM " . $this->db->dbprefix . "forum_threads WHERE user_id = $id) AS total_threads";
	    $sql .= " FROM " . $this->db->dbprefix . "forum_posts WHERE user_id = $id";
	    $result = $this->db->query($sql)->row();
	    return $result;
	}
	
	/**
	 * Fields take from the users table
	 * @return string
	 */
	private function user_fields ($prefix = 'U') {
	    $prefix = $prefix . '.';
	    $sql = $prefix . "display_name, " . $prefix . "username, " . $prefix . "email AS user_email, " . $prefix . "phone_number AS user_phone_number, " . $prefix . "profile_image, " . $prefix . "designation, " . $prefix . "signature, " . $prefix . "forum_posts, " . $prefix . "blogs_count, " . $prefix . "is_online, " . $prefix . "created_on AS join_date, CONCAT(" . $prefix . "id, '!~!', " . $prefix . "username, '!~!', " . $prefix . "email, '!~!', " . $prefix . "display_name, '!~!', " . $prefix . "created_on) AS profile_user";
	    return $sql;
	}
	
	/**
	 * Common view function for Admin, Member and Staff
	 * @param string $module: 'threads'
	 * @param string $filter
	 * @param number $offset
	 * @param string $role
	 */
	public function view_all($module, $filter, $offset, $user_id = 0, $role = 'admin', $extra = array()) {
	    $this->lang->load('forums/forums');
	    if(isset($_POST['change_status'])) {
	        $id = $_POST['change_status'];
	        $new_status = $_POST['status_' . $id];
	        $this->update_status($module, $id, $new_status);
	    }
	    if(isset($_POST['pdf'])) {
	        $this->create_pdf($_POST['pdf']);
	    }
	    $column_name = $role == 'staff' ? 'assigned_to' : 'user_id';
	    if(!empty($user_id)) {
	        $this->db->where($column_name, $user_id);
	    }
	    $total_tickets = $this->db->count_all_results($this->table_name);
	    $filter_status = $this->get_filter('statuses', $user_id, $column_name);
	    if($role != 'subscriber') {
	        Template::set(array(
	            'statuses' => $filter_status
	        ));
	    }
	    Template::set(array(
	        'total_rows' => $total_rows,
	        'filter_status' => $filter_status
	    ));
	    $params = array(
	        'query_method' => 'get_tickets',
	        'view' => 'tickets/index',
	        'table_alias' => 'T.',
	        'filter_field' => 'status', //
	        'filter' => $filter, // Display filter options
	        'search_fields' => array('reference_id', 'subject')
	    );
	    if($role == 'admin') {
	        $params['context'] = $extra['context'];
	    } else if($role == 'subscriber') {
	        $params['redirect_uri'] = 'members/tickets/' . $role;
	        $params['no_render'] = true;
	        $params['query_where'] = 'user_id = ' . $user_id;
	        $params['ticket_id'] = $extra['id']; // Ticket Detail Page
	    } else if($role == 'staff') {
	        $params['redirect_uri'] = 'members/tickets/' . $role;
	        $params['no_render'] = true;
	        $params['query_where'] = 'assigned_to = ' . $user_id;
	        $params['ticket_id'] = $extra['id']; // Ticket Detail Page
	    }
	    $this->gcrud->index('forum_' . $module, $offset, $role, $params);
	}
	
	/**
	 * Display a list of Categories/Threads/Posts/Status/Email Templates data.
	 *
	 * @return void
	 */
	public function view($module = 'categories', $filter = 'all', $offset = 0, $role = 'admin', $extra = array()) {
	    $has_status = true;
	    // Status Change
	    if (isset($_POST['status_change'])) {
	        $this->gc->status_change('forum_' . $module, $_POST['status_change'], 'status');
	    }
	    // Allow Admin to change status
	    if(isset($_POST['change_status'])) {
	        $id = $_POST['change_status'];
	        $new_status = $_POST['status_' . $id];
	        $this->update_status($module, $id, $new_status);
	    }
	    $data = array(
	        'context' => $this->context,
	        'model' => 'forums',
	        'fields_method' => $module,
	        'set_table' => true,
	        'query_method' => 'manage', // Extra Method in case of different columns
	        'query_extra' => array('module' => $module, 'filter' => $filter),
	        'table_alias' => 'M.', // Table Alias for table joins
	        'prefix' => 'M',
	        'table_shorthand' => 'M',
	        'filter' => $filter,
	        'view' => 'components/manage',
	        'redirect_uri' => '/' . $this->context . '/' . $this->moduleName . '/manage/' . $module,
	        'edit_url' => site_url(SITE_AREA . '/' . $this->context . '/' . $this->moduleName . '/form/' . $module . '/edit/')
	    );	    
	    if(isset($extra['redirect_uri'])) {
	        $data['redirect_uri'] = $extra['redirect_uri'];
	    }
	    switch($module) {
	        case 'statuses':
	            $data['search_fields'] = array('name');
	            $has_status = false;
	            break;
	        case 'categories':
	            $data['search_fields'] = array('name', 'description');
	            $data['purge_method'] = '_purge_category';
	            $data['soft_delete'] = true;
	            break;
	        case 'threads':
	            $data['search_fields'] = array('subject');
	            $data['purge_method'] = '_purge_thread';
	            $data['soft_delete'] = true;
	            break;
	        case 'posts':
	            $data['search_fields'] = array('subject', 'message');
	            $data['purge_method'] = '_purge_post';
	            $data['soft_delete'] = true;
	            break;
	        case 'votes':
	            $data['search_fields'] = array('up_count', 'down_count');
	            $has_status = false;
	            break;
	        case 'email_templates':
	            $data['search_fields'] = array('name', 'subject', 'message');
	            $has_status = false;
	            break;
	        case 'negative_keywords':
	            $data['search_fields'] = array('name');
	            $has_status = false;
	            break;
	        case 'reports':
	            $data['soft_delete'] = true;
	            $data['search_fields'] = array('description');
	            break;
	    }
	    if($has_status) {
	        $data['redirect_uri'] = '/components/forums/manage/' . $module;
	        $data['filter'] = $filter;
	        $data['filter_counter'] = true;
	        $filter_status = $this->forums_model->get_filter('statuses', 0, null, $module);
	        Template::set(array(
	            'filter_status' => $filter_status
	        ));
	    }
	    $this->gcrud->index('forum_' . $module, $offset, 'admin', $data);
	}
	
	/**
	 * Common method to send email
	 * @param array $data
	 */
	public function send_email($module_type, $action_type, $id, $data = array()) {
	    $this->load->library('parser');
	    $record = !empty($data['record']) ? $data['record'] : $this->get_row($module_type == 1 ? 'threads': 'posts', $id);
	    if(!$record) {
	        Template::set_message(lang('invalid_id'), 'danger');
	        return false;
	    }
	    $from = !empty($data['from']) ? $data['from'] : null;
	    $to = !empty($data['to']) ? $data['to'] : null;
	    $record = (array) $record;
	    $record['site_title'] = settings_item('site.title');;
	    $record['site_email'] = settings_item('site.system_email');
	    $record['site_url'] = site_url();
	    $record['thread_url'] = gc_url_format('forums/threads');
	    // Retrieve template from database
	    $email_templates = $this->db->where(array('module_type' => $module_type, 'action_type' => $action_type, 'status' => 1))->get($this->table_email_templates)->result();
	    if(!$email_templates) {
	        return false;
	    }
	    $this->load->library('emailer/emailer');
	    foreach($email_templates as $email_template) {
	        $data = array();
	        switch($email_template->user_type) {
	            case 3: // assigned user
	                $to = $record['assigned_user_email'];
	                $data['headers'] = array(
	                    'reply_email' => $record['user_email'],
	                    'reply_name' => $record['display_name']
	                );
	                break;
	            case 1: // admin
	                $to = settings_item('site.system_email');
	                $data['headers'] = array(
	                    'reply_email' => $record['user_email'],
	                    'reply_name' => $record['display_name']
	                );
	                break;
	            default:
	                $to = $record['user_email'];
	        }
    	    $subject = $this->parser->parse_string($email_template->subject, $record, TRUE);
    	    $message = $this->parser->parse_string($email_template->message, $record, TRUE);
    	    $data['to'] = $to;
    	    $data['subject'] = $subject;
    	    $data['message'] = $message;
    	    $this->emailer->send($data, FALSE);
	    }
	}
	
	/**
	 * Common Function - CREATE PDF FILE
	 * @package com.tecnick.tcpdf
	 * @abstract TCPDF - Example: Default Header and Footer
	 */
	public function create_pdf($id) {
	    $record = $this->tickets_model->get_ticket($id, true);
	    $filename = 'Ticket_' . $record->reference_id;
	    if(!$record) {
	        Template::set_message(lang('invalid_id'), 'danger');
	        return false;
	    }
	    $this->load->library('goclixy/gfile');
	    $files = $this->get_attachments($id);
	    $messages = $this->db->where('ticket_id', $id)->order_by('id', 'DESC')->get('ticket_messages')->result();
	    $data = array(
	        'role' => 'member',
	        'toolbar_title' => lang('ticket_detail'),
	        'record' => $record,
	        'files' => $files,
	        'messages' => $messages
	    );
	    $html_content = $this->load->view('tickets/tpl/_pdf', $data, true); // wrap contents within HTML tags with CSS Style
	    $this->load->library('format');
	    $this->format->to_dompdf(array('filename' => $filename, 'content' => $html_content, 'download' => true));
	}
	
	/**
	 * Update Thread/Post/Category Status
	 * @param number $id
	 * @param number $status - New Status
	 * @param boolean $is_redirect
	 */
	public function update_status($module, $id, $status, $is_redirect = true) {
	    $record = $this->get_row($module, $id, true);
	    if(!$record) {
	        return false;
	    }
	    $message = '';
	    if($record->status_id != $status) {
	        $data = array(
	            'status_id' => $status,
	            'modified_on' => date('Y-m-d H:i:s'),
	            'modified_ip' => $this->input->ip_address()
	        );
	        $this->db->where('id', $id)->update('forum_' .  $module, $data);
	        $message = sprintf(lang('msg_action_success'), lang('singular_forum_' . $module), lang('label_updated')) . ' ';
	        $type = 'success';
	    } else {
	        $message = lang('msg_nothing_to_update');
	        $type = 'danger';
	    }
	    // update reports table
	    if(isset($_POST['update_module_' . $id])) {
	        $record->status_id = $status;
	        $this->_reports_update($module, $id, $record);
	    }
	    if(isset($_POST['notify_customer_' . $id])) {
	        $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'forum_email_templates', 'module_table' => 'forum_' . $module, 'action_type' => 'status_update', 'id' => $record->id));
	        $message .= lang('msg_customer_notified');
	        $type = 'success';
	    }
	    if($is_redirect) {
	        Template::set_message($message, $type);
	        Template::redirect(current_url());
	    }
	}
	
	/**
	 * Update Threads and Posts table
	 * @param number $id
	 * @param number $status_id
	 * @param array $record
	 */
	private function _reports_update($module, $id, $record = array()) {
	    if(empty($record)) {
	        $record = $this->db->where(array('module_id' => ($module == 'threads' ? 1 : 2), 'row_id' => $id))->get($this->table_reports)->row();
	    }
	    if(!empty($record)) {
	        $status_id = $record->status_id;	        
	        $data = array(
	            'status_id' => $status_id,
	            'modified_on' => date('Y-m-d H:i:s'),
	            'modified_ip' => $this->input->ip_address()
	        );	        
	        if($record->module_id == 1) {
	            // update posts table first
	            $this->db->where('thread_id', $record->row_id)->update('forum_posts', $data);
	            
	            // update threads table
	            $this->db->where('id', $record->row_id)->update('forum_threads', $data);
	        } else {
	            // update posts table
	            $this->db->where('id', $record->row_id)->update('forum_posts', $data);
	        }
	    }
	}
	
	/**
	 * Purge Category
	 * @param number $id
	 */
	public function _purge_category($ids = array()) {
	    foreach($ids as $id) {
    	    // get all threads
    	    $records = $this->db->where('category_id', $id)->get($this->table_name)->result();
    	    if($records) {
    	        foreach($records as $record) {
    	            // purge thread and posts first
    	            $this->_purge_thread(array($record->id));
    	        }
    	    }
    	    // notify user
    	    $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'forum_email_templates', 'module_table' => $this->table_categories, 'action_type' => 'delete', 'id' => $id));
    	    // purge category
    	    $this->db->where('id', $id)->delete($this->table_categories);
	    }
	}
	
	/**
	 * Purge Thread
	 * @param number $id
	 */
	public function _purge_thread($ids = array()) {
	    foreach($ids as $id) {
    	    // get all posts
    	    $records = $this->db->where('thread_id', $id)->get($this->table_posts)->result();
    	    if($records) {
    	        foreach($records as $record) {
    	            // purge posts first
    	            $this->_purge_post(array($record->id));
    	        }
    	    }
    	    // notify user
    	    $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'forum_email_templates', 'module_table' => $this->table_name, 'action_type' => 'delete', 'id' => $id));
    	    // purge thread
    	    $this->db->where('id', $id)->delete($this->table_name);
    	    // purge reports
    	    $this->db->where(array('module_id' => 1, 'row_id' => $id))->delete($this->table_reports);
	    }
	}
	
	/**
	 * Purge Post
	 * @param number $id
	 */
	public function _purge_post($ids = array()) {
	    foreach($ids as $id) {
    	    $record = $this->db->where('id', $id)->get($this->table_posts)->row();
    	    if($record) {
    	        // decrement the forum_posts counter in the users table
    	        $this->update_forum_posts_counter(false, $record->id, $record);
    	        // notify user
    	        $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'forum_email_templates', 'module_table' => $this->table_posts, 'action_type' => 'delete', 'id' => $id));
        	    // purge post
    	        $this->db->where('id', $id)->delete($this->table_posts);
    	        // purge votes
    	        $this->db->where('post_id', $id)->delete($this->table_votes);
    	        // purge reports
    	        $this->db->where(array('module_id' => 2, 'id' => $id))->delete($this->table_reports);
    	    }
	    }
	}
	
	/**
	 * Update Forum Posts counter in the Users table to avoid unnecessary queries during thread view
	 * 
	 * @param boolean $update_all
	 * @param number $id
	 * @param object $record
	 * 
	 * @return void
	 */
	public function update_forum_posts_counter($update_all = true, $id = 0, $record = null) {
	    if(!$update_all) {
	        if(empty($record)) {
	            $record = $this->db->where('id', $id)->get($this->table_posts)->row();
	        }
	        if(!empty($record)) {
        	    $forum_posts = $this->db->where('user_id', $record->user_id)->count_all_results($this->table_posts);
        	    $this->db->where('id', $record->user_id)->update('users', array('forum_posts' => $forum_posts));
	        }
	    } else {
    	    $records = $this->db->select('DISTINCT(user_id) AS user_id')->get($this->table_posts)->result();
    	    if($records) {
    	        foreach($records as $record) {
    	            $forum_posts = $this->db->where('user_id', $record->user_id)->count_all_results($this->table_posts);
    	            $this->db->where('id', $record->user_id)->update('users', array('forum_posts' => $forum_posts));
    	        }
    	    }
	    }
	}
	
	/**
	 * Method called in Gcrud library after posts insert/update
	 * @param number $id
	 * @param string $action_type
	 */
	public function after_post_save($id, $action_type = 'insert') {
	    $record = $this->get_row('posts', $id);
	    if($action_type == 'insert') {
    	    // update the forum_posts counter in the users table
	        $this->update_forum_posts_counter(false, $id, $record);
	    }
	    if(!empty($record->category_id)) {
	       $this->db->where('id', $id)->update($this->table_posts, array('category_id' => $record->category_id));
	    }
	    $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'forum_email_templates', 'module_table' => 'forum_posts', 'action_type' => ($action_type == 'insert' ? 'add' : 'edit'), 'id' => $id, 'record' => $record));
	}
	
	/**
	 * Method called in Gcrud library after thread insert/update
	 * If category_id updated, update it also in the posts table
	 * @param number $id
	 * @param string $action_type
	 * @param array $record
	 */
	public function after_thread_save($id, $action_type = 'insert', $record = array()) {
	    $row = array();
	    if($action_type == 'update' && !empty($record->category_id)) {
	        $row = $this->get_row('threads', $id);
	        // category updated, update it in the posts table
	        if($row && $row->category_id != $record->category_id) {
	            $this->db->where('thread_id', $id)->update($this->table_posts, array('category_id' => $row->category_id));
	        }
	    }
	    $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'forum_email_templates', 'module_table' => 'forum_threads', 'action_type' => ($action_type == 'insert' ? 'add' : 'edit'), 'id' => $id, 'record' => $record));
	}
	
	/**
	 * Display User Specific Posts
	 * @param number $user_id
	 * @return array
	 */
	public function user_info($user_id) {
	    $sql = "SELECT created_on";
	    $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->table_posts . " WHERE user_id = " . $user_id . ") AS total_posts";
	    $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->table_name . " WHERE user_id = " . $user_id . ") AS total_threads";
	    $sql .= " FROM " . $this->db->dbprefix . $this->table_posts . " P";
	    $sql .= " WHERE user_id = " . $user_id;
	    $sql .= " ORDER BY P.id DESC";
	    return $this->db->query($sql)->row();
	}
	
	/**
	 * Common function for Admin/Member/Moderator/Detail page
	 * Category/Thread/Post Detail Page
	 * @param string $module
	 * @param string $slug
	 * @param array $extra - Extra parameters like WHERE
	 */
	public function info($module, $slug, $extra = array(), $role = 'admin') {
	    Assets::add_css('jssocials.css');
	    Assets::add_js('jssocials.min.js');
	    Assets::add_js(array(
	        'tinymce/tinymce.min.js',
	    ));
	    $this->load->library('goclixy/gfile');
	    
	    // delete thread
	    if(isset($_POST['delete-thread'])) {
	        $this->db->where('id', $_POST['delete-thread'])->update('forum_threads', array('deleted' => 1));
	        Template::set_message(sprintf(lang('msg_action_success'), lang('singular_forum_threads'), lang('label_deleted')), 'success');
	    }
	    
	    // delete post or thread
	    if(isset($_POST['delete'])) {
	        $delete_module = $module;
	        if($this->uri->segment(2) == gc_uri_string('category')) {
	            $delete_module = 'threads';
	        } else if($this->uri->segment(2) == gc_uri_string('thread')) {
	            $delete_module = 'posts';
	        }
	        $this->db->where('id', $_POST['delete'])->update('forum_' . $delete_module, array('deleted' => 1));
	        Template::set_message(sprintf(lang('msg_action_success'), lang('singular_forum_' . $delete_module), lang('label_deleted')), 'success');
	        // Template::redirect(current_url() . (!empty($_SERVER['QUERY_STRING']) ? '?' . $_SERVER['QUERY_STRING'] : ''));
	    }
	    
	    // variables
	    $view_data = array();
	    $this->session->set_userdata('forum_last_visited_url', current_url() . (!empty($_SERVER['QUERY_STRING']) ? '?' . $_SERVER['QUERY_STRING'] : '')); // last visited url
	    $redirect_url = gc_url_format('forums');
	    // The detail page is for specific modules
	    switch($module) {
	        case 'categories':
	            $view_data['summary'] = $this->summary();
	            break;
	        case 'threads':	// display users summary on the threads page
	            $settings = json_decode(settings_item('forum.settings'), true);
	            if(empty($settings['categories_view'])) {
	                $view_data['summary'] = $this->summary();
	            }
	            break;
	        case 'posts':
	        case 'reports':
	            break;
	        default:
	            Template::redirect($redirect_url);
	    }
	    if(empty($slug) && empty($extra) && !isset($extra['no_params'])) {
	        Template::redirect($redirect_url);
	    }
	    $has_records = true;
	    $approved_only = isset($extra['approved_only']) ? $extra['approved_only'] : true; // Display only active records only, the Admin can also view inactive records
	    if(!empty($slug)) { // view specific record (category, thread and post)
    	    $row = $this->forums_model->get_row($module, null, false, "M.slug = '" . escape($slug) . "' AND M.status_id = 2");
    	    if(!$row) {
    	        Template::redirect($redirect_url);
    	    }
    	    $view_data['row'] = $row;
    	    
    	    // update hits
    	    $this->db->where('id', $row->id)->update('forum_' . $module, array('hits' => $row->hits + 1));
	    }
	    if(!empty($slug) && $module == 'posts') {
	        $has_records = false;
	    }
	    if($has_records) {
    	    $limit = isset($_GET['limit']) ? $_GET['limit'] : settings_item('site.front_list_limit');
    	    $offset = intval($this->input->get('per_page'));
    	    $where = array();
    	    if(!empty($slug)) { // in case of category and thread, display threads and posts
    	        switch($module) {
    	            case 'categories':
    	                $where[] = 'M.category_id = ' . $row->id;
    	                $query_module = 'threads';
    	                break;
    	            case 'threads':
    	                $where[] = 'M.thread_id = ' . $row->id;
    	                $query_module = 'posts';
    	                break;    	                
    	        }
    	    } else {
    	        if(isset($extra['where'])) {
    	            $where[] = $extra['where'];
    	        }
    	        $query_module = $module;
    	    }
    	    if($approved_only) {
    	       $where[] = 'M.status_id = 2';
    	    }
    	    if(!empty($where)) {
    	        $where = implode(' AND ', $where);
    	    }
    	    $sort_by = isset($extra['sort_by']) ? $extra['sort_by'] : 'M.id DESC';
    	    $records = $this->forums_model->manage(0, $where, $sort_by, $limit, $offset, array('module' => $query_module, 'view' => true));
    	    $total_rows = $this->forums_model->manage(1, $where, $sort_by, $limit, $offset, array('module' => $query_module, 'view' => true));
    	    
    	    // Pagination
    	    $this->load->library('pagination');
    	    $this->config->load('pagination');
    	    
    	    /**
    	     * if you go back from another pagination number to click on the first(1), CI does not care anymore of your suffix config
    	     * therefore, below given code retrieve query string in case of pagination links are clicked
    	     */
    	    $base_url = current_url();
    	    /*$query_params = $_SERVER['QUERY_STRING'];
    	    if(!empty($query_params)) {
    	        if(!empty($query_params['per_page'])) {
    	           unset($query_params['per_page']);
    	        }
    	        $base_url .= '?' . $query_params;
    	    }*/
    	    $config['base_url'] = $base_url;
    	    $config['total_rows'] = $total_rows;
    	    $config['per_page'] = $limit;
    	    
    	    // Pagination using GET Method
    	    $config['reuse_query_string'] = TRUE;
    	    $config['enable_query_strings'] = TRUE;
    	    $config['page_query_string'] = TRUE;
    	    
    	    $this->pagination->initialize($config);
    	    $pagination = $this->pagination->create_links();
    	    $view_data['pagination'] = $pagination;
    	    $view_data['records'] = $records;
    	    $view_data['total_rows'] = $total_rows;
	    }
	    // next and previous page
	    if(!empty($slug)) {
	        $where_active = '';
	        if($approved_only) {
	            $where_active = 'M.status_id = 2';
	        }
	        $previous_record = array();
	        $previous_row_id = $row->id - 1;
	        if($previous_row_id != 0) {
	            $previous_record = $this->forums_model->get_row($module, $row->id - 1, false, $where_active);
	        }	        
	        $next_record = $this->forums_model->get_row($module, $row->id + 1, false, $where_active);
    	    $view_data['previous_record'] = $previous_record;
    	    $view_data['next_record'] = $next_record;
	    }
	    // Meta Data
	    if(!empty($row)) {
	        $meta_data = (array) $row;
	        $meta_data['description'] = truncate_text(isset($meta_data['description']) ? $meta_data['description'] : (!empty($meta_data['message']) ? $meta_data['message'] : ''), 160);
	        $this->meta_info($module, $meta_data, $module);
	    } else if(!empty($extra['meta_data'])) {
	        $meta_data = $extra['meta_data'];
	        $this->meta_info($meta_data['page'], $meta_data, $module);
	    } else {
	        $this->meta_info('common', array(), $module);
	    }
	    $view_data['module'] = $module;
	    $view_data['slug'] = $slug;
	    $view_data['date_format'] = $this->datetime_format;
	    $view_data['message_no_record_found'] = isset($extra['message_no_record_found']) ? $extra['message_no_record_found'] : lang('label_no_record_found');
	    if(isset($extra['page'])) {
	        $view_data['page'] = $extra['page'];
	    } else {
	        $view_data['page'] = $module;
	    }
	    // Post Reply
	    if($module == 'threads' && !empty($slug)) {
	        // posts save requires category and thread id
	        $forum_form_params = array(
	            'category_id' => isset($row->category_id) ? $row->category_id : 0,
	            'thread_id' => isset($row->id) ? $row->id : 0,
	        );
	        $this->session->set_userdata('forum_form_params', $forum_form_params);
	        
	        $this->load->library('form_validation');
	        if (isset($_POST['save'])) {
	            if ($this->save_form('posts', 'insert', $row->id)) {
	                Template::set_message(sprintf(lang('msg_save_success'), lang('singular_forum_posts')), 'success');
	                $this->session->unset_userdata('forum_form_params');
	                Template::redirect(current_url());
	            } else {
	                Template::set_message(sprintf(lang('msg_create_failure'), lang('singular_forum_posts')), 'danger');
	            }
	        }
	        $view_data['form_module'] = 'posts';
	        $view_data['toolbar_title'] = lang('forum_post_reply');
	        $view_data['is_report'] = 0;
	        $view_data['type'] = 'create';
	        $view_data['id'] = 0;
	        $view_data['form_record'] = array('subject' => 'Re: ' . $row->subject);
	    }
	    Template::set($view_data);
	    Assets::add_js('_forums.js');
	    Assets::add_css('_forums.css');
	    Template::set_view('forums/view');
	    Template::render($this->layout);
	}
	
	/**
	 * Display Meta Data
	 * @param string $page - Name of the page
	 * @param array $meta_data
	 * @param string $module
	 */
	public function meta_info($page, $meta_data, $module) {
	    $this->load->library('parser');
	    $meta_data['module'] = lang('forum_' . $module);
	    $meta_data['site_title'] = settings_item('site_title');
	    Template::set(array(
	        'page_title' => $this->parser->parse_string(lang('forum_' . $page . '_meta_title'), $meta_data, TRUE),
	        'meta_description' => $this->parser->parse_string(lang('forum_' . $page . '_meta_description'), $meta_data, TRUE),
	        'meta_keywords' => $this->parser->parse_string(lang('forum_' . $page . '_meta_keywords'), $meta_data, TRUE),
	    ));
	}
	
	/**
	 * Member Area - Common form for all modules
	 * @param string $module - Name of the module - Threads/Posts
	 * @param string $id - ID of the row/module
	 * @param string $type - create/edit
	 * @param string $is_report - True - Report Form
	 */
	public function form($module, $id = 0,  $type = 'create', $is_report = 0) {	    
	    if($type == 'create') {
	        // do not display form to the user if the subscribers only is enabled in the settings
	        // and the user is not a subscriber
    	    switch($module) {
    	        case 'threads':
    	        case 'posts':
    	           $this->has_permission();
    	           break;
    	    }
	    }
	    $this->auth->restrict();
	    Assets::add_css('selectize.bootstrap3.css');
	    Assets::add_js('selectize.min.js');
	    Assets::add_js(array(
	        'tinymce/tinymce.min.js',
	    ));
	    // Assets::add_module_js('forums', 'forums.js');
	    $this->load->library('form_validation');
	    $redirect_url = $this->session->userdata('forum_last_visited_url') ? $this->session->userdata('forum_last_visited_url') : gc_url_format('forums');
	    if($module != 'threads' && $module != 'posts') {
	        Template::redirect($redirect_url);
	    }
	    $id = (int) $id;
	    if (empty($id)) {
	        Template::set_message(lang('invalid_id'), 'danger');
	        redirect($redirect_url);
	    }
	    $action_type = ($type == 'create' ? 'insert' : 'update');
	    $user_id = $this->auth->user_id();
	    $form_table = $is_report == 1 ? 'reports' : $module;
	    // Name of the table to be queried
	    if($type == 'create') {
    	    if($is_report == 1) {
    	        $table = $module;
    	    } else {
    	        if($module == 'posts') {
    	            $table = 'threads';
    	        } else {
    	            $table = 'categories';
    	        }
    	    }
    	    $record = $this->get_row($table, $id);
	    } else {
	        $table = $module;
	        $record = $this->db->get_where('forum_' . $table, array('id' => $id))->row();	        
	        if($record->user_id != $user_id) { // not authorized to edit this page
	            Template::set_message(lang('msg_permission_denied'), 'danger');
	            redirect($redirect_url);
	        }
	    }	    
	    if(!$record) {
	        Template::set_message(lang('invalid_id'), 'danger');
	        redirect($redirect_url);
	    }
	    if($type == 'create') {
	        // check whether the user is allowed to post in this category or not
	        if($module == 'threads' && $is_report == 0 && $record->allow_thread == 0) {
	            Template::set_message(lang('forum_allow_thread_error'), 'danger');
	            redirect($redirect_url);
	        }
	        if($is_report == 1) {
	            $forum_form_params = array(
	                'module_id' => $module == 'threads' ? 1 : 2,
	                'row_id' => $id,
	            );
	        } else {
	            if($module == 'threads' && empty($record->thread_id)) {
	                $forum_form_params = array(
	                    'category_id' => $record->id,
	                    'thread_id' => 0
	                );
	            } else {
        	        $forum_form_params = array(
        	            'category_id' => isset($record->category_id) ? $record->category_id : 0,
        	            'thread_id' => isset($record->thread_id) ? $record->thread_id : 0,
        	        );	
	            }
	        }
	        $this->session->set_userdata('forum_form_params', $forum_form_params);
	    }	    
	    
	    // frontend new thread
	    if($id != 0 && $module == 'threads' && $is_report == 0) {
	        $redirect_url = gc_url_format('forums/search') . '?type=threads';
	    }
	    if (isset($_POST['save'])) {
	        $this->has_permission(false);
	        if ($this->save_form($form_table, $action_type, $id)) {
	            if($is_report == 0) {
	                Template::set_message(sprintf(lang('msg_save_success'), lang('singular_forum_' . $module)), 'success');
	            } else {
	                Template::set_message(sprintf(lang('msg_action_success'), lang('singular_forum_' . $module), lang('forum_reported')), 'success');
	            }
	            $this->session->unset_userdata('forum_form_params');
	            // frontend new thread
	            if($id != 0 && $module == 'threads' && $is_report == 0) {
	                $redirect_url = gc_url_format('forums/search') . '?type=threads';
	            }
	            Template::redirect($redirect_url);
	        } else {
	            Template::set_message(sprintf(lang('msg_' . $type . '_failure'), lang('singular_forum_' . $module)), 'danger');
	        }
	    }
	    if($type == 'create') {
	        if($table == 'categories') {
	            $url = anchor(gc_url_format('forums/category', $record->slug), $record->name);
	        } else {
	            $url = anchor(gc_url_format('forums/thread', $record->slug), $record->subject);
	        }
	        $toolbar_title = $is_report == 1 ? lang('singular_forum_reports') . ' ' . lang('singular_forum_' . $module) . '<i class="fas fa-th mr-2 ml-2 small"></i>' . $url : sprintf(lang('forum_' . $module . '_new'), $url);
	    } else {
	        $toolbar_title = lang('edit_forum_' . $module) . '<i class="fas fa-th mr-2 ml-2 small"></i>' . $record->subject;
	    }
	    Template::set(array(
	        'toolbar_title' => $toolbar_title,
	        'record' => $type == 'create' ? array() : $record,
	        'type' => $type,
	        'id' => $id,
	        'module' => $module,
	        'is_report' => $is_report
	    ));
	    Assets::add_css('_forums.css');
	    Assets::add_js('_forums.js');
	    Template::set_view('form');
	    Template::render($this->layout);
	}
	
	/**
	 * Member Area - Common Save Form for Posts/Threads and Reports
	 * @param string $type
	 * @param number $id
	 * @return boolean
	 */
	private function save_form($table, $type = 'insert', $id = 0) {
	    if($table == 'reports') {
	        $this->form_validation->set_rules('description', 'lang:forum_reason', 'trim|required');
	    } else {
	        if($table == 'threads') {
	            // $this->form_validation->set_rules('category_id', 'lang:singular_forum_categories', 'trim|required');
	            $this->form_validation->set_rules('tags', 'lang:forum_message', 'trim');
	        }
	        $this->form_validation->set_rules('subject', 'lang:label_subject', 'trim|required|negative_keywords');
	        $this->form_validation->set_rules('message', 'lang:label_message', 'trim|required|negative_keywords');
	    }	    
	    if ($this->form_validation->run() === false) {
	        return false;
	    }
	    $user_id = $this->auth->user_id();
	    $date = date('Y-m-d H:i:s');
	    $ip_address = $this->input->ip_address();
	    $forum_form_params = $this->session->userdata('forum_form_params');
	    $data = array();
	    if($table == 'reports') {
	        if($type == 'insert') {
    	        $data['module_id'] = $forum_form_params['module_id'];
    	        $data['row_id'] = $forum_form_params['row_id'];
	        }
	        $data['description'] = $this->input->post('description');
	    } else {
	        if($table == 'threads') {
	            // the user might have posted the thread from the category page
	            $data['category_id'] = $this->input->post('category_id') ? $this->input->post('category_id') : ($type == 'insert' ? $forum_form_params['category_id'] : 0);
	            if($data['category_id'] == 0 && $type == 'update') { // the form has no category field
	                unset($data['category_id']);
	            }
	            $data['tags'] = $this->input->post('tags');
	        } else if($type == 'create' || ($table == 'posts' && $type == 'insert')) { // get post data from the session variable
	            $data['category_id'] = $forum_form_params['category_id'];
	            $data['thread_id'] = $forum_form_params['thread_id'];
	        }
	        $data['subject'] = escape($this->input->post('subject'));
	        $data['message'] = $this->input->post('message');
	        $data['status_id'] = 1;
	        // get status from settings
	        switch($table) {
	            case 'threads':
	            case 'posts':
	                $settings = json_decode(settings_item('forum.settings'), true);
	                $statuses = json_decode($settings['status'] ?? '', true);
	                $data['status_id'] = (int) $statuses[$table . '_' . ($type == 'insert' ? 'create' : 'edit')];
	                break;
	        }
	    }	    
	    $table = 'forum_' . $table;
	    if($type == 'insert') {
	        if(!empty($data['subject'])) {
	            $config = array(
	                'field' => 'slug',
	                'title' => 'subject',
	                'table' => $this->db->dbprefix . $table,
	                'id' => 'id',
	            );
	            $this->load->library('slug');
	            $slugify = new Slug($config);
	            $slug = $data['subject']; // . '_' . date('Y-m-d H:i:s') . '_' . rand(1, 100);
	            $data['slug'] = $slugify->create_uri($slug);
	        }
	        $data['user_id'] = $user_id;
	        $data['created_on'] = $date;
	        $data['created_ip'] = $ip_address;
	        $this->db->insert($table, $data);
	        $id = $this->db->insert_id();
	        if(!empty($id)) {
	            $return = $id;
	        }
	    } else {
	        $data['modified_on'] = $date;
	        $data['modified_ip'] = $ip_address;
	        $return = $this->db->where('id', $id)->update($table, $data);
	    }
	    // notify admin and user
	    $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'forum_email_templates', 'module_table' => $table, 'action_type' => ($type == 'insert' ? 'add' : 'edit'), 'id' => $id));
	    return $return;
	}
	
	/**
	 * Subscriber Permission Check
	 * Permission Name: Forums.Subscriber.View
	 * @param boolean $redirect_home
	 * @return boolean
	 */
	public function has_permission($redirect_home = true, $permission = 'Forums.Subscriber.View') {
	    $settings = json_decode(settings_item('forum.settings'), true);
	    if($settings['subscribers_only'] == 0) {
	        return true;
	    }
	    if($this->session->userdata('user_ordered_items') || $this->auth->has_permission($permission)) {
	        return true;
	    } else {
	        Template::set_message(lang('forum_subscribers_only_message'), 'danger');
	        Template::redirect($redirect_home ? gc_url_format('forums') : current_url());
	    }
	}
	
	/**
	 * Update Votes
	 */
	public function votes() {
	    $thread_id = (int) $this->input->post('thread_id');
	    $post_id = (int) $this->input->post('post_id');
	    $type = (int) $this->input->post('type'); // 1 - Up Counter, 2 - Down Counter
	    if($this->auth->is_logged_in() && (!empty($thread_id) || !empty($post_id))) {	        
	        if(!empty($post_id)) {
	            $table = $this->table_posts;
	            $id = $post_id;
	        } else {
	            $table = $this->table_name;
	            $id = $thread_id;
	        }
	        $row = $this->db->select('id, votes')->where('id', $id)->get($table)->row();
	        if(!$row) {
	            return 0;
	        }
	        $user_id = (int) $this->auth->user_id();
	        $date = date('Y-m-d H:i:s');
	        $ip_address = $this->input->ip_address();
	        // check whether this user already rated for this thread/post
	        // a user can vote only once
	        $where = array('user_id' => $user_id, 'thread_id' => $thread_id, 'post_id' => $post_id);
	        $record = $this->db->where($where)->get($this->table_votes)->row();
	        $data = array();
	        $vote = 0;
	        if(!$record) {
	            $data = $where;
	            $data['last_updated'] = $date;
	            $data['ip_address'] = $ip_address;	            
	            if($type == 1) {
	                $data['up_count'] = 1;
	                $data['down_count'] = 0;
	                $vote = 1; // plus vote
	            } else {
	                $data['down_count'] = 1;
	                $data['up_count'] = 0;
	                $vote = 2; // down vote
	            }
	            $this->db->insert($this->table_votes, $data);
	        } else {	       
	            $update = true;
	            if($type == 1 && $record->up_count == 1) { // the user has already upvoted          
	                $data['up_count'] = 0;
	                $data['down_count'] = 1;
	                $vote = 2; // down vote
	            } else if($type == 2 && $record->down_count == 0) {	                
	                $data['up_count'] = 0;
	                $data['down_count'] = 1;
	                $vote = 2; // down vote
	            } else if($type == 1 && $record->up_count == 0) {	                
	                $data['up_count'] = 1;
	                $data['down_count'] = 0;
	                $vote = 1; // up vote
	            } else { // do nothing, do not change the votes
	                $update = false;
	            }
	            if($update == true) {	                
	                $data['last_updated'] = $date;
	                $data['ip_address'] = $ip_address;
	                $this->db->where('id', $record->id)->update($this->table_votes, $data);
	            }
	        }
	        $upvoted = false;
	        if($vote == 1) {
	            $votes = $row->votes + 1;
	            $upvoted = true;
	        } else if((int)$row->votes > 0 && $vote == 2) {
	            $votes = $row->votes - 1;
	        } else {
	            $votes = $row->votes;
	        }
	        $this->db->where('id', $row->id)->update($table, array('votes' => $votes));
	        if($upvoted) {
	            $message = lang('forum_upvoted');	            
	        } else {
	            $message = lang('forum_downvoted');
	        }
	        echo json_encode(array('result' => 1, 'message' => $message, 'type' => 'success', 'votes' => $votes));
	        return $votes;
	    }
	}
	
	/**
	 * Prepare where statements to filter results
	 * @param array $where
	 * @return string
	 */
	public function prepare_where($params, $where = array(), $module = 'threads') {
	    if(!empty($params)) {
	        $db_fields = $this->gc->get_db_fields('forum_' . $module);
	        foreach($params as $key => $value) {
	            $value = escape($value);
	            switch($key) {
	                case 'popular':
	                    if($value == 'week') {
	                        $value = date('Y-m-d', strtotime("-7 days"));
	                        $where[] = "(DATE(M.created_on) BETWEEN '" . $value . "' AND '" . date('Y-m-d') . "')";
	                        $sort_by = 'M.hits DESC';
	                    } else if($value == 'all_time') {
	                        $sort_by = 'M.hits DESC';
	                    }
	                    break;
	                case 'created_on':
	                    if($value == 'today') {
	                        $value = date('Y-m-d');
	                    } else if($value == 'yesterday') {
	                        $value = date('Y-m-d', strtotime("-1 days"));
	                    }
	                    $where[] = 'DATE(M.' . $key . ") = '" . $value . "'";
	                    break;
	                case 'per_page':
	                    break;
	                default:
	                    if(!empty($value) && in_array($key, $db_fields)) {
	                       $where[] = 'M.' . $key . '=' . $value;
	                    }
	            }
	        }
	        $return = array();
	        if(!empty($where)) {
	            $return['where'] = implode(' AND ', $where);
	        }
	        if(!empty($sort_by)) {
	            $return['sort_by'] = $sort_by;
	        }
	        return $return;
	    }
	    return false;
	}
}