<?php defined('BASEPATH') || exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Components controller
 */
class Components extends Admin_Controller
{
    protected $permissionCreate = 'Franchises.Components.Create';
    protected $permissionDelete = 'Franchises.Components.Delete';
    protected $permissionEdit   = 'Franchises.Components.Edit';
    protected $permissionView   = 'Franchises.Components.View';

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();        
        $this->auth->restrict($this->permissionView);
        $this->load->model('franchises/franchises_model');
        $this->lang->load('franchises');        
      	$this->form_validation->set_error_delimiters("<span class='error'>", "</span>");        
        Template::set_block('sub_nav', 'components/_sub_nav');
        Assets::add_module_js('franchises', 'franchises.js');
        clear_cache('franchises');
    }    
    
    /**
     * Franchises Summary
     */
    public function summary() {
        Assets::add_js('moment.js');
        Assets::add_css('bootstrap-datetimepicker.min.css' );
        Assets::add_js('bootstrap-datetimepicker.min.js' );
        Assets::add_css('chart.css');
        Assets::add_js('Chart.min.js');
        $this->session->unset_userdata('chart_query_params');
        $module = 'user_passbook';
        $record = $this->franchises_model->summary();
        $table = $module;
        $where = '';
        $column = 'credit';
        $param_months = array('where' => $where,'prefix' => 'M', 'table' => $table, 'field' => $column);
        if($this->auth->role_id() == 7) {
            $param_months['user_id'] = $this->auth->user_id();
        }
        $row_by_months = $this->gcrud->chart('months', $param_months);
        Assets::add_js($this->load->view('common/_chart_js', array('field_id' => 'sixMonthChart', 'labels' => $row_by_months['labels'], 'data' => $row_by_months['data'], 'label' => lang('total_rows'), 'total' => $row_by_months['total'], 'with_dates' => false, 'table' => $table), true), 'inline');
        $result = $this->gcrud->chart('current_month', $param_months);
        Assets::add_js($this->load->view('common/_chart_js', array('field_id' => 'monthlyChart', 'labels' => $result['labels'], 'data' => $result['data'], 'total' => $result['total'], 'label' => lang('total_rows'), 'with_dates' => true, 'table' => $table), true), 'inline');
        Template::set(array(
            'module' => $module,
            'toolbar_title' => lang($module) . ' - ' . lang('gc_menu_summary'),
            'record' => $record
        ));
        Template::render();
    }
    
    /**
     * Manage Franchises
     * @param string $filter
     * @param number $offset
     */
    public function index($filter = 'all', $offset = 0) {
        $this->manage('allotments', $filter = 'all', $offset = 0);
    }
    
    /**
     * Manage Franchises
     * @param string $module
     * @param string $filter
     * @param number $offset
     */
    public function manage($module = 'allotments', $filter = 'all', $offset = 0) {
        $extra = array(
            'model' => 'franchises',
            'context' => 'components',
            'fields' => $module,
            'prefix' => 'franchise',
            'table_prefix' => 'franchise'
        );
        $this->gc->module_view($module, $filter, $offset, 'admin', $extra);
    }
    
    /**
     * Add/Edit Franchises Data
     *
     * @param string $module
     * @param string $action
     *
     * @return void
     */
    public function form($module = 'allotments', $action = 'create') {
        $params = array(
            'model' => 'franchises',
            'module_name' => 'franchises',
            'context' => 'components',
            'prefix' => 'franchise',
            'table_prefix' => 'franchise',
            'fields_method' => $module
        );
        if($module == 'allotments') {
            // update summary column for search
            $params['method'] = 'after_franchise_save';
        }
        $this->gcrud->form($module, $action, 'admin', $params);
    }
    
    /**
     * View Franchises Payouts
     * @param number $offset
     */
    public function payoff($offset = 0) {
        $this->load->library('goclixy/gpayouts');
        $this->gpayouts->payoff('franchise_users', $offset, $this->limit);
    }
    
    /**
     * Pay Franchises Payouts
     * @param number $offset
     */
    public function pay($id) {
        $this->load->library('goclixy/gpayouts');
        $this->gpayouts->pay('franchise_users', $id);
    }
    
    /**
     * Display Franchises to the user
     */
    public function payouts($filter = 'all', $offset = 0) {
        $this->load->library('goclixy/gpayouts');
        $this->gpayouts->payouts('franchise_users', $filter, $offset);
    }
    
    public function test() {        
        $user_id = 67;
        $city_id = 11599; // ,11611;
        $data = array(
            'module' => 'deliveries',
            'amount' => 50
        );
        // $franchise_where = $this->franchises_model->get_allotment($user_id);
        // dump($franchise_where);
        $this->franchises_model->commission_calculations($city_id, $data);
    }
}