<?php defined('BASEPATH') || exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class Franchises_model extends BF_Model {
    protected $table_name	    = 'franchise_allotments';
    protected $table_statuses	= 'franchise_statuses';
	protected $key			= 'id';
	protected $date_format	= 'datetime';

	protected $log_user 	= false;
	protected $set_created	= false;
	protected $set_modified = false;
	protected $soft_deletes	= false;


	// Customize the operations of the model without recreating the insert,
    // update, etc. methods by adding the method names to act as callbacks here.
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 	    = array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	// For performance reasons, you may require your model to NOT return the id
	// of the last inserted row as it is a bit of a slow method. This is
    // primarily helpful when running big loops over data.
	protected $return_insert_id = true;

	// The default type for returned row data.
	protected $return_type = 'object';

	// Items that are always removed from data prior to inserts or updates.
	protected $protected_attributes = array();

	// You may need to move certain rules (like required) into the
	// $insert_validation_rules array and out of the standard validation array.
	// That way it is only required during inserts, not updates which may only
	// be updating a portion of the data.
	protected $validation_rules 		= array(
		array(
			'field' => 'name',
			'label' => 'lang:label_name',
			'rules' => 'required|max_length[20]',
		),
	);
	protected $insert_validation_rules  = array();
	protected $skip_validation 			= false;
	protected $gc_franchises_cache;

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
        // Load Zend Caching
        $this->load->library('slug');
        $this->load->library('gzf_cache', array('lifetime' => 900));
        $this->gc_franchises_cache = $this->gzf_cache->get_instance();
    }
    
    /**
     * Franchise Allotments Fields
     *
     * @return array
     */
    public function allotments() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'selectize',
                'name' => 'user_id',
                'label' => 'bf_user',
                'placeholder' => 'placeholder_selectize',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => true,
                'options' => 'load:franchise_users;module:franchise_users;max_items:1',
                'options_detailed' => 1,
                'rules' => 'trim|required',
            ),
            array(
                'type' => 'selectize',
                'name' => 'city_ids',
                'label' => 'cities',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'load:cities;module:cities;max_items:50',
                'options_detailed' => 1,
                'rules' => 'trim|required',
            ),
            array(
                'type' => 'text',
                'name' => 'commission_subscriptions',
                'label' => 'franchise_subscriptions',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'class' => 2,
                'rules' => 'trim|required|is_decimal',
            ),
            array(
                'type' => 'text',
                'name' => 'commission_ads',
                'label' => 'franchise_ads',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'class' => 2,
                'rules' => 'trim|required|is_decimal',
            ),
            array(
                'type' => 'text',
                'name' => 'commission_deliveries',
                'label' => 'franchise_deliveries',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'class' => 2,
                'rules' => 'trim|required|is_decimal',
            ),
            array(
                'type' => 'dropdown',
                'name' => 'status_id',
                'label' => 'label_status',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'options' => 'name:franchise_statuses;value:database',
                'options_detailed' => 1,
                'class' => 3,
                'rules' => 'trim|required|is_numeric',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            
        );
        return $fields;
    }
    
    /**
     * Display Summary
     * @return array
     */
    public function summary() {
        $table = 'user_passbook'; //$this->table_name;
        $role_id = $this->auth->role_id();        
        if($role_id == 7) { // display franchise specific summary
            $where = 'M.user_id = ' . $this->auth->user_id();
            $join = '';
        } else {
            $where = '';
            $join = PHP_EOL . " INNER JOIN " . $this->db->dbprefix . "users U ON U.id = M.user_id AND U.role_id = 7" . PHP_EOL;
        }
        $sql = "SELECT COUNT(*) AS total_transactions";
        
        // today rows
        $sql .= ", (SELECT SUM(credit) FROM " . $this->db->dbprefix . $table . " M";
        $sql .= $join;
        $sql .= " WHERE YEAR(M.created_on) = YEAR(NOW()) AND MONTH(M.created_on) = MONTH(NOW()) AND DAY(M.created_on) = DAY(NOW())" . $where .") AS franchise_today";
        
        // this month rows
        $sql .= ", (SELECT SUM(credit) FROM " . $this->db->dbprefix . $table . " M";
        $sql .= $join;
        $sql .= " WHERE MONTH(M.created_on) = MONTH(CURDATE()) AND YEAR(M.created_on)  =  YEAR(CURDATE())" . $where .") AS franchise_month";
        
        // yesterday rows
        $sql .= ", (SELECT SUM(credit) FROM " . $this->db->dbprefix . $table . " M";
        $sql .= $join;
        $sql .= " WHERE M.created_on <= NOW() AND M.created_on = DATE_ADD(Now(),interval - 1 day)" . $where .") AS franchise_yesterday";
        
        $sql .= " FROM " . $this->db->dbprefix . $table . " M";
        $sql .= $join;
        $sql .= " WHERE credit != 0" . $where;
        $query = $this->db->query($sql);
        $result = $query->row();
        foreach($result as $key => $value) {
            if($key == 'total_transactions') {
                $result->{$key} = $value;
            } else {
                $result->{$key} = empty($value) ? 0 : currency_format($value);
            }
        }
        return $result;
    }
    
    /**
     * Auth Library - Check whether a city is allotted to franchise or not
     * @param number $user_id
     * @return string
     */
    public function get_allotment($user_id) {
        $where = "status_id = 2 AND user_id = " . $user_id;
        $record = $this->db->where($where)->get($this->table_name)->row();
        if($record && !empty($record->city_ids)) {
            $city_ids = explode(',', $record->city_ids);
            return "city_id IN (" . implode(',', $city_ids) . ")";
        } else {
            return ''; // do not show any record to the user, logout the user and redirect to the home page
        }
    }
    
    /**
     * Franchise Commission Calculations
     * 
     * Note: Calculations are done on each transactions instead of one time or daily basis
     * The commission amount and franchise can be changed at anytime so this process will help in accuracy
     * Besides, the franchises can view the earnings in realtime
     * 
     * @param number $city_id - ID of the city where transaction is recorded
     * @param array $data - calculate commissions here and update in the user_passbook table
     * module - Subscriptions/Ads/Deliveries
     * amount - Transaction amount on which the franchise will get the commission
     * row_id - ID of the row
     * action - CREDIT/DEBIT
     */
    public function commission_calculations($city_id, $data = array()) {
        $cacheID = 'franchise_' . $city_id;
        if(!$franchise = $this->gc_franchises_cache->load($cacheID)) {
            // the user must be active, not deleted and the city must be assigned
            $where = 'status_id = 2 AND deleted = 0 AND FIND_IN_SET(' . $city_id . ', city_ids)';
            $record = (array) $this->db->where($where)->get($this->table_name)->row();
            $franchise = array('user' => $record);
            $this->gc_franchises_cache->save($franchise, $cacheID, array('franchises'));
        }
        if(!empty($franchise['user'])) {
            $user = $franchise['user'];
            if(!empty($data['module']) && !empty($data['amount']) && !empty($user['commission_' . $data['module']])) {
                // calculate commissions and update the user passbook table with the commission
                $module = $data['module'];
                $commission_percent = $user['commission_' . $module];
                $amount = ($data['amount'] * $commission_percent) / 100;
                if(!empty($amount)) {
                    // update the user_passbook table
                    $this->load->library('goclixy/gpayouts');
                    $listing_id = !empty($data['listing_id']) ? $data['listing_id'] : 0;
                    $notes = !empty($data['notes']) ? $data['notes'] : null;
                    $extra = !empty($data['extra']) ? $data['extra'] : array();
                    $this->gpayouts->update_passbook($user['user_id'], $module, $data['row_id'], $amount, $data['action'], $listing_id, $notes, $extra);
                }
            }
        }
    }
    
    /**
     * Save summary information in the summary column for search
     * @param number $id
     * @param string $action_type
     * @param array $record
     */
    public function after_franchise_save($id, $action_type = 'insert', $record = array()) {
        $this->db->from($this->table_name . ' M');
        $this->db->join('users U', 'U.id = M.user_id', 'INNER');
        $record = $this->db->select('M.*, U.display_name, U.email, U.phone_number')->where('M.id', $id)->get()->row();
        if($record) {
            if(!empty($record->city_ids)) {
                $city_ids = explode(',', $record->city_ids);
                $this->db->from('cities' . ' CT');
                $this->db->join('states S', 'S.id = CT.state_id', 'INNER');
                $cities = array();
                $records = $this->db->select('CT.id, CT.name, S.name AS state, S.country_iso')->where('CT.id IN (' . implode(',', $city_ids) . ')')->get()->result();
                if($records) {
                    foreach($records as $row) {
                        /*$cities = array(
                            'id' => $row->id,
                            'name' => $row->id . ' - ' . $row->name . ', ' . $row->state . ', ' . $row->country_iso
                        );*/
                        $cities[] = $row->id . ' - ' . $row->name . ', ' . $row->state . ', ' . $row->country_iso;
                    }
                }
            }
            $user = $record->user_id . ', ' . $record->display_name . ', ' . $record->email . ', ' . $record->phone_number;
            $this->db->where('id', $id)->update($this->table_name, array('summary' => json_encode(array('cities' => $cities, 'user' => $user))));
        }
    }
}