<?php defined('BASEPATH') || exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Developer controller
 */
class Developer extends Admin_Controller
{
    protected $permissionCreate = 'Groups.Developer.Create';
    protected $permissionDelete = 'Groups.Developer.Delete';
    protected $permissionEdit   = 'Groups.Developer.Edit';
    protected $permissionView   = 'Groups.Developer.View';

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
        
        $this->auth->restrict($this->permissionView);
        $this->load->model('groups/groups_model');
        $this->lang->load('links/links');   
        $this->lang->load('fields/fields'); 
        $this->lang->load('groups');         
                        
		Assets::add_js('moment.js');
		Assets::add_css('bootstrap-datetimepicker.min.css');
		Assets::add_js('bootstrap-datetimepicker.min.js');
      	$this->form_validation->set_error_delimiters("<span class='error'>", "</span>");
        
        Template::set_block('sub_nav', 'developer/_sub_nav');

        Assets::add_module_js('groups', 'groups.js');
    }
    
    /**
     * Allow Admin to Add field groups
     *
     * @return void
     */
    public function index($offset = 0) {
    	// Change Language String
        if (isset($_POST['status_change'])) {
            gc_demo_mode();
            $this->gcrud->stats(array('module' => 'groups'));
    		$this->gc->status_change('groups', $_POST['status_change'], 'name_lang');
    	}
    	 
    	// Deleting anything?
    	if (isset($_POST['delete'])) {
    	    gc_demo_mode();
    		$this->auth->restrict($this->permissionDelete);
    		$checked = $this->input->post('checked');
    		if (is_array($checked) && count($checked)) {
    			$result = FALSE;
    			foreach ($checked as $pid) {
    				$result = $this->groups_model->purge_group($pid);
    			}
    			$message_label = (count($checked) == 1 ? lang('label_group') : lang('label_groups'));
    			if ($result) {
    				Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $message_label, lang('label_deleted')), 'success');
    			} else {
    				Template::set_message(sprintf(lang('msg_delete_failure'), $message_label));
    			}
    		}
    	}
    
    	$searchterm = $this->gc->search('groups', $this->uri->segment(4), '/developer/groups/index/');
    	if(!empty($searchterm)) {
    		$where = "name LIKE '%". $searchterm ."%' OR created_on LIKE '%". $searchterm ."%'";
    	}
    
    	// Sort By
    	$order_by = $this->gc->sort('groups');
    	if(!empty($order_by)) {
    		$this->db->order_by($this->session->userdata('groups_sort')['column'], $this->session->userdata('groups_sort')['order']);
    	} else {
    		$this->db->order_by('id', 'desc');
    	}
    	if(isset($where)) {
    		$this->db->where($where);
    	}
    	$this->db->limit($this->limit, $offset);
    	$records = $this->db->order_by('id', 'desc')->get('groups')->result();
    	if(isset($where)) {
    		$this->db->where($where);
    	}
    	$total_records = $this->db->count_all_results('groups');
    
    	// Pagination
    	$this->load->library('pagination');
    	$pager['base_url'] = site_url(SITE_AREA ."/developer/groups/index/");
    	$pager['uri_segment']	= 5;
    	$pager['total_rows'] = $total_records;
    	$pager['per_page'] = $this->limit;
    
    	$this->pagination->initialize($pager);
    	$percent_result = $this->db->select('SUM(percent_value) AS percents')->get('groups')->row();
    	Template::set(array(
    	    'toolbar_title' => lang('manage_groups'),
    	    'records' => $records,
    	    'percents_total' => $percent_result->percents ? $percent_result->percents : 0
    	));
    	Template::render();
    }
    
    /**
     * Create a New Group
     *
     * @return void
     */
    public function create() {
    	Assets::add_css('fontawesome-iconpicker.min.css');
    	Assets::add_js('fontawesome-iconpicker.min.js');
    	Assets::add_module_js('fields', 'fields.js');
    	$this->auth->restrict($this->permissionCreate);
    	if (isset($_POST['save'])) {
    	    $this->gcrud->stats(array('module' => 'groups'));
    		$this->auth->restrict($this->permissionCreate);
    		if ($insert_id = $this->save_group()) {
    			Template::set_message(sprintf(lang('msg_save_success'), lang('label_group')), 'success');
    			Template::redirect(site_url(SITE_AREA. '/developer/groups'));
    		} else {
    			Template::set_message(sprintf(lang('msg_create_failure'), lang('label_group')) . $this->groups_model->error, 'danger');
    		}
    	}
    	Template::set('toolbar_title', lang('create_group'));
    	Template::set_view('group_form');
    	Template::render();
    }
    /**
     * Allows editing of Group data.
     *
     * @return void
     */
    public function edit() {
    	Assets::add_css('fontawesome-iconpicker.min.css');
    	Assets::add_js('fontawesome-iconpicker.min.js');
    	Assets::add_module_js('fields', 'fields.js');
    	$id = $this->uri->segment(5);
    	if (empty($id)) {
    		Template::set_message(lang('invalid_id'), 'error');
    		redirect(site_url(SITE_AREA. '/developer/groups'));
    	}
    
    	if (isset($_POST['save'])) {
    	    $this->gcrud->stats(array('module' => 'groups'));
    		$this->auth->restrict($this->permissionEdit);
    		if ($this->save_group('update', $id)) {
    			Template::set_message(sprintf(lang('msg_save_success'), lang('label_group')), 'success');
    			Template::redirect(site_url(SITE_AREA. '/developer/groups'));
    		} else {
    			Template::set_message(sprintf(lang('msg_edit_failure'), lang('label_group')), 'error');
    		}
    	}
    	$relations = $this->gc->get_relations($id, 'groups');
    	Template::set('relations', $relations);
    	
    	Template::set('record', $this->db->get_where('groups', array('id' => $id))->row());
    	Template::set('toolbar_title', lang('edit_group'));
    	Template::set_view('group_form');
    	Template::render();
    }
    
    /**
     * Save Group Information
     * @param string $type
     * @param number $id
     * @return number
     */
    private function save_group($type='insert', $id = 0) {
        gc_demo_mode();
    	if ($type == 'update') {
    		$_POST['id'] = $id;
    	}
    	$this->form_validation->set_rules('name', 'lang:label_name', 'trim|required|unique[groups.name, groups.id]');
    	// $this->form_validation->set_rules('percent_text', 'lang:percent_text', 'trim');
    	$this->form_validation->set_rules('priority', 'lang:label_priority', 'trim|required|is_numeric');
    	$this->form_validation->set_rules('icon', 'lang:label_icon', 'trim');
    	$this->form_validation->set_rules('mobile_icon', 'lang:label_mobile_icon', 'trim|alpha_dash');
    	$this->form_validation->set_rules('control_class', 'lang:label_control_class', 'trim');
    	$this->form_validation->set_rules('control_condition', 'lang:control_condition', 'trim');
    	if ($this->form_validation->run() === false) {
    		return false;
    	}
    	
    	// make sure we only pass in the form fields we want
    	$data = $this->groups_model->prep_data($this->input->post());
    	$data['name_lang'] = $this->input->post('name_lang') ? 1 : 0;
    	$data['tooltip_lang'] = $this->input->post('tooltip_lang') ? 1 : 0;    	
    	$selected_modules           = $this->input->post('modules');
    	$modules_selected = array();
    	if(!empty($selected_modules)) {
    	    foreach($selected_modules as $selected_module => $selected_value) {
    	        $modules_selected[] = gc_module_id($selected_module);
    	    }
    	}
    	$data['module_relations']   = implode(',', $modules_selected);
    	if ($type == 'insert') {
    		$data['created_on'] = date('Y-m-d H:i:s');
    		$this->db->insert('groups', $data);
    		$id = $this->db->insert_id();
    		if (is_numeric($id)) {
    			$return = $id;
    		} else {
    			$return = FALSE;
    		}
    	} elseif ($type == 'update') {
    		$data['modified_on'] = date('Y-m-d H:i:s');
    		$return = $this->db->where('id', $id)->update('groups', $data);
    	}
    	
    	// Save Group and Module relationship
    	// A Group can belong to multiple modules
    	if($return) {
    		$this->gc->save_module_relations($id, $this->input->post('modules'), 'groups');
    	}
    	return $return;
    }
    
    /**
     * Allow admin to change order of Field Groups
     */
    public function order() {
    	//LOAD NESTED SORTABLE JAVASCRIPT AND CSS FILE
    	Assets::add_css('nestedSortable/admin.css');
    	Assets::add_js('nestedSortable/jquery-ui.min.js');
    	Assets::add_js('nestedSortable/jquery.mjs.nestedSortable.js');
    
    	Assets::add_js($this->load->view('developer/order_groups_js', null, true), 'inline');
    	Template::set('sortable', TRUE);
    	Template::set('toolbar_title', lang('order_groups'));
    	Template::render();
    }
    
    /**
     * Order groups using AJAX nestedSortable
     */
    public function order_ajax () {
    	// Save order from ajax call
    	if (isset($_POST['sortable'])) {
    		$this->groups_model->save_group_order($_POST['sortable']);
    	}
    	// Load view
    	$groups = $this->db->order_by('priority')->get('groups')->result_array();
    	Template::set('result', $groups);
    	Template::set('toolbar_title', lang('order_groups'));
    	Template::set_view('common/order_ajax');
    	Template::render();
    }
    
    /**
     * Set Fields Percentage - Module Completion
     * @param integer $group_id
     */
    public function set_percent($group_id) {
        $this->lang->load('listings/listings');
        // Get group information
        $where = 'id = ' . $group_id;
        $where .= ' AND FIND_IN_SET(' . gc_module_id('percents') . ', module_relations)';
        $group = $this->db->where($where)->get('groups')->row();
        if(!$group) {
            Template::redirect(site_url(SITE_AREA . "/developer/groups"));
        }
        $table = 'module_percents';
        $module = 'listings';
        $fields = array();
        // do not allow user to set percent value greater than 100
        $consumed = $this->db->select('SUM(value) AS percents')->where(array('group_id !=' => $group->id))->get($table)->row();
        $consumed_percents = $consumed->percents ? $consumed->percents : 0;
        switch($group->name) {
            case 'gallery':
                $fields = array(
                array('name' => 'profile_image', 'label' => 'label_profile_image', 'label_lang' => 1),
                array('name' => 'profile_images', 'label' => 'label_gallery', 'label_lang' => 1)
                );
                break;
            case 'horoscope':
                $fields = array(array('name' => 'horoscope', 'label' => 'label_horoscope', 'label_lang' => 1));
                break;
            /*case 'address_phone_email':
                $fields = array(array('name' => 'horoscope', 'label' => 'label_horoscope', 'label_lang' => 1));
                break;*/
            default:
                /*$unset_fields = array('id', 'profile_id', 'user_id', 'package_id', 'email', 'hits', 'status', 'deleted', 'created_on', 'modified_on', 'created_by', 'modified_by', 'created_ip', 'modified_ip','show_photos', 'show_horoscope', 'show_phone', 'meta_title', 'meta_description', 'meta_keywords', 'expires_on', 'featured', 'home_featured', 'cover', 'package_priority', 'profile_summary', 'profile_preferences');
                 $fields = $this->db->list_fields('profiles');
                 $fields_flip = (array) array_flip($fields);
                 foreach($unset_fields as $unset_field) {
                 if(isset($fields_flip[$unset_field])) {
                 unset($fields_flip[$unset_field]);
                 }
                 }
                 $fields = array_flip($fields_flip);*/                
                $fields = $this->fields_model->get_fields($module, 'frontend', array(), 0, array('group_id' => $group->id));
                if($fields) {
                    $unset_fields = array('id', 'address_phone_email', 'user_id', 'package_id', 'hits', 'status', 'deleted', 'created_on', 'modified_on', 'created_by', 'modified_by', 'created_ip', 'modified_ip', 'expires_on', 'featured', 'home_featured', 'cover', 'package_priority');
                    foreach($fields as $key => $value) {
                        if(in_array($value['name'], $unset_fields)) {
                            unset($fields[$key]);
                        }
                    }
                } 
        }
        if(isset($_POST['save'])) {
            $this->gcrud->stats(array('module' => 'groups'));
            foreach($fields as $field) {
                $label = !empty($field['label_lang']) ? lang($field['label']) : $field['label'];
                $this->form_validation->set_rules($field['name'], $label, 'trim|is_numeric');
            }
            if ($this->form_validation->run() === true) {
                // prepare data to be saved in the percent table
                // note: a user cannot assign a field to multiple groups
                $data = array();
                $percents_total = 0;
                foreach($fields as $post_data) {
                    if(!empty($_POST[$post_data['name']])) {
                        $data[] = array(
                            'module_id' => gc_module_id($module),
                            'group_id' => $group->id,
                            'field' => $post_data['name'],
                            'value' => $_POST[$post_data['name']]
                        );
                        $percents_total += $_POST[$post_data['name']];
                    }
                }
                $percent_value = $percents_total; // Group Percents
                $percents_total = $percents_total + $consumed_percents;
                if($percents_total > 100) {
                    Template::set_message(sprintf(lang('error_percent'), $percents_total - 100), 'danger');
                } else {
                    // remove previous data first
                    // the user might have set all fields value to zero
                    $where = array('group_id' => $group->id, 'module_id' => gc_module_id($module));
                    $this->db->where($where)->delete($table);
                    // update percents in the groups table
                    $this->db->where(array('id' => $group->id))->update('groups', array('percent_value' => $percent_value));
                    if(!empty($data)) {
                        $this->db->on_duplicate($table, $data);
                    }
                    Template::set_message(sprintf(lang('msg_save_success'), lang('singular_percents')), 'success');
                    Template::redirect(site_url(SITE_AREA. '/developer/groups'));
                }
                // update percentage on value change
            }
        }
        // get saved values
        $record = new stdClass;
        $group_percents = 0;
        $rows = $this->db->where(array('group_id' => $group->id))->get($table)->result();
        if ($rows) {
            foreach ($rows as $row) {
                $key = $row->field;
                // use key for language translation
                // to display field lable on the module detail page
                $record->$key = $row->value;
                $group_percents += $row->value;
            }
        }
        Template::set(array(
            'toolbar_title' => (!empty($group->name_lang) ? lang('group_' . $group->name) : $group->name) . ' - ' . $group_percents . '%',
            'fields' => $fields,
            'consumed_percents' => $consumed_percents,
            'group_percents' => $group_percents,
            'group' => $group,
            'available_percents' => 100 - ($consumed_percents + $group_percents),
            'record' => $record
        ));
        Template::render();
    }
}