<?php defined('BASEPATH') || exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class Groups_model extends BF_Model
{
    protected $table_name	= 'groups';
    protected $table_percents = 'module_percents';
    protected $table_completion = 'module_percent_completion'; // Having specific profile completion fields
	protected $key			= 'id';
	protected $date_format	= 'datetime';

	protected $log_user 	= true;
	protected $set_created	= false;
	protected $set_modified = false;
	protected $soft_deletes	= false;


	// Customize the operations of the model without recreating the insert,
    // update, etc. methods by adding the method names to act as callbacks here.
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 	    = array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	// For performance reasons, you may require your model to NOT return the id
	// of the last inserted row as it is a bit of a slow method. This is
    // primarily helpful when running big loops over data.
	protected $return_insert_id = true;

	// The default type for returned row data.
	protected $return_type = 'object';

	// Items that are always removed from data prior to inserts or updates.
	protected $protected_attributes = array();

	// You may need to move certain rules (like required) into the
	// $insert_validation_rules array and out of the standard validation array.
	// That way it is only required during inserts, not updates which may only
	// be updating a portion of the data.
	protected $validation_rules 		= array(
		array(
			'field' => 'name',
			'label' => 'lang:label_name',
			'rules' => 'max_length[100]',
		),
		array(
			'field' => 'name_lang',
			'label' => 'lang:label_name_lang',
			'rules' => 'max_length[1]',
		),
		array(
			'field' => 'tooltip',
			'label' => 'lang:label_tooltip',
			'rules' => 'max_length[455]',
		),
		array(
			'field' => 'tooltip_lang',
			'label' => 'lang:label_tooltip_lang',
			'rules' => 'max_length[1]',
		),
		array(
			'field' => 'tooltip_class',
			'label' => 'lang:label_tooltip_class',
			'rules' => 'max_length[50]',
		),
		array(
			'field' => 'icon',
			'label' => 'lang:label_icon',
			'rules' => 'max_length[100]',
		),
		array(
			'field' => 'priority',
			'label' => 'lang:label_priority',
			'rules' => 'max_length[5]',
		),
		array(
			'field' => 'status',
			'label' => 'lang:label_status',
			'rules' => 'max_length[1]',
		),
		array(
			'field' => 'created_on',
			'label' => 'lang:label_created_on',
			'rules' => '',
		),
		array(
			'field' => 'modified_on',
			'label' => 'lang:label_modified_on',
			'rules' => '',
		),
	);
	protected $insert_validation_rules  = array();
	protected $skip_validation 			= false;
	protected $gc_group_cache;
	public $modules = array('fields', 'links', 'percents');

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
        // Load Zend Caching
        $this->load->library('slug');
        $this->load->library('gzf_cache', array('lifetime' => 900));
        if(method_exists($this->gzf_cache, 'get_instance')) {
            $this->gc_group_cache = $this->gzf_cache->get_instance();
        }
    }
    
    /**
     * Save group order only when user clicks on save button
     * @param array $fields
     */
    public function save_group_order($groups) {
    	if (count($groups)) {
    		foreach ($groups as $order => $group) {
    			if (isset($group['id']) && $group['id'] != '') {
    				$data = array('priority' => $order);
    				$this->db->where(array('id' => $group['id']))->update('groups', $data);
    			}
    		}
    	}
    }
    
    /**
     * Purge Group with its relation
     * @param integer $id
     */
    public function purge_group($id) {
    	// Purge all relations of this group from the module_relations table
    	$this->gc->purge_relation($id, 'groups');
    	// Purge Group
    	$this->groups_model->delete($id);
    }
    
    /*****************************************/
    /* START OF PROFILE COMPLETION % LOGIC	 */
    /*****************************************/
    /**
     * Module Images Percent Completion Update
     * @param string $type - insert/delete
     * @param number $profile_id
     * @param array $profile
     */
    public function gallery_completion($type, $module, $row_id, $record = array()) {
        $common_operation = false;
        if($type == 'insert' && !$this->session->userdata('gallery_completion')) {
            $common_operation = true;
            // one photo upload is taken as the profile images creteria
            // so there is no need to call the profile completion method on each image add
            $this->session->set_userdata('gallery_completion', 1);
        } else if($type == 'delete') {
            $common_operation = true;
            $this->session->unset_userdata('gallery_completion');
        }
        if($common_operation) {
            $this->load->model('images/images_model');
            $this->images_model->_tweak_images_row($module, $row_id);
            // update profile_images in the images table
            $record = $this->db->where('id', $row_id)->get($module)->row();
            if($record) {
                /*if(!empty($record) && $record['id'] == $row_id) {
                    $user_profile['profile_image'] = $record->profile_image;
                    $user_profile['profile_images'] = $record->profile_images;
                    // $this->session->set_userdata('user_profile', $user_profile);
                }*/
                // call the percent completion method
                $this->row_completion($module, $row_id, $record);
            }
        }
    }
    /**
     * Update Module Completion Percentage
     * @param number $row_id
     * @param array $profile
     * @param array $percent_fields
     */
    public function row_completion($module, $row_id, $record = array(), $percent_fields = array()) {
        // get all fields from the profile_percents table
        if(empty($percent_fields)) {
            $percent_fields = $this->db->get($this->table_percents)->result();
        }
        if(empty($record)) {
            $record = $this->db->where('id', $row_id)->get($module)->row();
        }
        $completion_percent = 0;
        $data = array();
        $deletion_fields = array();
        $module_id = gc_module_id($module);
        if(!empty($percent_fields) && !empty($record)) {
            foreach($percent_fields as $percent_field) {
                $field_name = $percent_field->field;
                if($field_name == 'latitude_longitude') {
                    $field_name = 'latitude';
                }
                $completion_field = false;
                if(!empty($record->{$field_name})) {
                    $completion_field = true;
                    $completion_percent += $percent_field->value;
                } else {
                    $deletion_fields[] = $field_name;
                }
                if($completion_field) {
                    $date = date('Y-m-d H:i:s');
                    $ip_address = $this->input->ip_address;
                    $data[] = array(
                        'module_id' => $module_id,
                        'row_id' => $row_id,
                        'field' => $field_name,
                        'value' => $percent_field->value,
                        'created_on' => $date,
                        'created_ip' => $ip_address,
                        'modified_on' => $date,
                        'modified_ip' => $ip_address
                    );
                }
            }
        }
        // remove data first
        $where = "module_id = " . $module_id . " AND row_id = " . $row_id;
        if(!empty($deletion_fields)) {
            $where .= " AND FIND_IN_SET(field, '". implode(',', $deletion_fields) ."')";
        }
        $this->db->where($where)->delete($this->table_completion);
        if(!empty($data)) {
            $this->db->on_duplicate($this->table_completion, $data);
        }
        // update completion_percentage in the profiles table
        $this->db->where('id', $row_id)->update($module, array('percent_completion' => $completion_percent));
    }
    
    /**
     * Display Profile Percentage Tree in the Member Area
     */
    public function percentage_tree($module) {
        $cacheID = $module . '_percentage_tree';
        if (! $percentage_tree = $this->gc_group_cache->load($cacheID)) {
            $this->lang->load('groups/groups');
            $form_fields = $this->fields_model->get_fields($module, 'frontend');
            $percent_fields = $this->db->get('module_percents')->result_array();
            $fields = array();
            if($form_fields) {
                foreach($percent_fields as $key => $percent_field) {
                    $fields[$percent_field['field']] = $key;
                }
            }
            $percent_fields_info = array();
            foreach($form_fields as $form_field) {
                if(array_key_exists($form_field['name'], $fields)) {
                    $percent_fields[$fields[$form_field['name']]]['label'] = !empty($form_field['label_lang']) ? lang($form_field['label']) : $form_field['label'];
                }
            }
            $field_groups = $this->fields_model->field_groups($form_fields, array(), 'group_id');
            $percentage_tree = array();
            if(!empty($field_groups)) {
                foreach($field_groups as $field_group) {
                    $percent_value = 0;
                    $group_fields = gc_form_fields($percent_fields, 'group_id', $field_group->id);
                    if(!empty($group_fields)) {
                        $group_field_values = array();
                        foreach($group_fields as $group_field) {
                            $group_field_values[] = array(
                                'id' => $group_field['id'],
                                'name' => $group_field['field'],
                                'label' => !empty($group_field['label']) ? $group_field['label'] : lang('label_' . $group_field['field']),
                                'value' => $group_field['value']
                            );
                            $percent_value += $group_field['value'];
                        }
                        $percentage_tree[] = array(
                            'id' => $field_group->id, // For edit
                            'name' => ($field_group->name_lang == 1 ?  lang('group_' . $field_group->name) : $field_group->name),
                            'mobile_icon' => $field_group->mobile_icon,
                            'fields' => $group_field_values,
                            'value' => $percent_value
                        );
                    }
                }
            }
            $this->gc_group_cache->save($percentage_tree, $cacheID, array('fields', 'groups'));
        }
        return $percentage_tree;
    }
    
    /**
     * Member Area - Business Completed in %
     * @param array $rest_extra - Mobile API Extra
     */
    public function percent_completion($module, $row_id = 0, $role = 'admin', $html = true, $rest_extra = array()) {
        $is_mobile = $role == 'mobile' ? true : false;
        $record = (array) $this->db->where('id', $row_id)->get($module)->row();
        if(!$record) {
            return false;
        }
        $percent_completion = 0;
        $percent_completion_tree = array();
        if(!empty($record)) {
            $percentage_tree = $this->percentage_tree($module);
            $percent_completion_tree = $percentage_tree;
            foreach($percentage_tree as $group_key => $group) {
                $percent_completion_tree[$group_key]['completed'] = 0;
                foreach($group['fields'] as $field_key => $field) {
                    $percentage_fields = array();
                    switch($field['name']) {
                        case 'latitude_longitude': // multiple fields
                            $percentage_fields[] = array('name' => 'latitude', 'value' => ($field['value'] / 2));
                            $percentage_fields[] = array('name' => 'longitude', 'value' => ($field['value'] / 2));
                            break;
                        default:
                            $percentage_fields[] = array('name' => $field['name'], 'value' => $field['value']);
                    }
                    foreach($percentage_fields as $percentage_field) {
                        if(array_key_exists($percentage_field['name'], $record)) {
                            if(!empty($record[$percentage_field['name']])) {
                                // the user has filled this field
                                $percent_completion_tree[$group_key]['completed'] += $percentage_field['value'];
                                $percent_completion_tree[$group_key]['fields'][$field_key]['completed'] = 1;
                                $percent_completion += $percentage_field['value'];
                            }
                        }
                    }
                }
            }
            // Update Modules Table with the new percentage
            if(isset($record['percent_completion']) && $record['percent_completion'] != $percent_completion) {
                $this->db->where('id', $record['id'])->update($module, array('percent_completion' => $percent_completion));
                if($role != 'admin') {
                    // update session variable to avoid any ambiguity
                    $record['percent_completion'] = $percent_completion;
                }
            }
            if($html) {
                $this->load->library('crypt');
                $this->load->library('parser');
                $tpl_data = array('record' => $record, 'percent_completion' => $percent_completion, 'tree' => $percent_completion_tree, 'role' => $role, 'is_mobile' => $is_mobile);
                $content = $this->load->view('groups/tpl_percent_01', $tpl_data, true);
                return array('heading' => lang('percent_completion'), 'output' => $content);
            } else {
                return array('heading' => lang('percent_completion'), 'percent' => $percent_completion, 'tree' => $percent_completion_tree);
            }
        }
    }
    /*****************************************/
    /* END OF COMPLETION % LOGIC	 */
    /*****************************************/
}