<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Images controller
 */
class Content extends Admin_Controller
{
	protected $permissionCreate = 'Images.Content.Create';
	protected $permissionDelete = 'Images.Content.Delete';
	protected $permissionEdit   = 'Images.Content.Edit';
	protected $permissionView   = 'Images.Content.View';
	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct()
	{
		parent::__construct();

		$this->auth->restrict($this->permissionView);
		$this->load->model('images_model', null, true);
		$this->lang->load('listings/listings');
		Template::set_block('sub_nav', 'content/_sub_nav');
	}
	
	/**
	 * Display all images to the admin
	 */
	public function index($module = 'all', $filter='all', $offset = 0) {		
		// Status Change
	    if (isset($_POST['status_change'])) {
	        gc_demo_mode();
	        $this->gcrud->stats(array('module' => 'images'));
			$this->gc->status_change('images', $_POST['status_change'], 'status', 99, false);
			// Update Logo and Cover Tweak field in the specified module	
			$this->gc->update_tweak_fields('images', $_POST['status_change']);
			$this->images_model->_tweak($_POST['status_change']);
			Template::redirect(current_url());
		}
		// Do we have any actions?
		if ($action = $this->input->post('submit')) {
		    gc_demo_mode();
		    $this->gcrud->stats(array('module' => 'images'));
			$checked = $this->input->post('checked');
	
			if($checked) {
				$module_name = lang('plural_image') ;
				switch(strtolower($action)) {
					case 'activate':
						$this->gc->update_status('images', $checked, 'status', 1);
						// Update Logo and Cover Tweak field in the specified module
						foreach($checked as $checked_id) {
							$this->gc->update_tweak_fields('images', $checked_id);
							$this->images_model->_tweak($checked_id);
						}
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_updated')), 'success');
						break;
	
					case 'deactivate':
						$this->gc->update_status('images', $checked, 'status', 0);
						// Update Logo and Cover Tweak field in the specified module
						foreach($checked as $checked_id) {
							$this->gc->update_tweak_fields('images', $checked_id);
							$this->images_model->_tweak($checked_id);
						}
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_updated')), 'success');
						break;
	
					case 'delete':
						if (is_array($checked) && count($checked)) {
							$result = FALSE;
							foreach ($checked as $pid) {
								$result = $this->images_model->purge_image_by_id($pid);
							}
	
							if ($result) {
								Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_deleted')), 'success');
							} else {
								Template::set_message(lang('delete_failure') . $this->images_model->error, 'danger');
							}
						}
						break;
				}
			}
		}
	
		// Actions done, now display the view
		$where = NULL;
	
		// Filters
		$filter_type = $filter;
	
		switch($filter_type) {
			case 'inactive':
				$where = "T.status = 0";
				break;
	
			case 'active':
				$where = "T.status = 1";
				break;
					
			case 'all':
				// Nothing to do
				break;
	
			default:
				show_404("images/index/$filter/");
		}
	
		// Pagination
		$this->load->library('pagination');
	
		// Sort By
		$sort_by = "T.id DESC";
		$order_by = $this->gc->sort('images');
		if(!empty($order_by)) {
			$sort_by = $order_by;
		}
		
		// Search Images
		$searchterm = $this->gc->search('images', $this->uri->segment(7), '/content/images/');
		if(!empty($searchterm)) {
			if(!empty($where)) {
				$where .= " AND";
			}
			$where .= " (T.id LIKE '%" .$searchterm . "%' OR T.title LIKE '%" .$searchterm . "%' OR T.row_id LIKE '%" .$searchterm . "%')";
		}
		
		if($module == 'all') {
			$records = $this->gc->get_all_row(0, 'images', $where, $sort_by, $offset, $this->limit);
		} else {
			$records = $this->gc->get_module_rows(0, 'images', $module, $where, $sort_by, $offset, $this->limit);
		}
		
		if($module == 'all') {
			$total_rows = $this->gc->get_all_row(1, 'images', $where, $sort_by, $offset, $this->limit);
		} else {
			$total_rows = $this->gc->get_module_rows(1, 'images', $module, $where, $sort_by, $offset, $this->limit);
		}
		
		$pager['base_url'] = site_url(SITE_AREA ."/content/images/index/$module/$filter");
		$pager['total_rows'] = $total_rows;
		$pager['per_page'] = $this->limit;
		$pager['uri_segment']	= 7;
	
		$this->pagination->initialize($pager);
	
		Template::set('index_url', site_url(SITE_AREA .'/content/images/index/' . $module . '/'));
		Template::set('filter_type', $filter_type);
	
		Template::set('records', $records);
		Template::set('module', $module);
		Template::set('toolbar_title', lang('gc_menu_images'));
		Template::render();
	}
	
	/**
	 * Generate thumbnail for all images in a particular folder
	 * @param string $module.
	 */
	public function generate_thumbnails() {
	    $this->auth->restrict('Images.Content.Thumbnails');
		$this->lang->load('listings/listings');
		$this->load->model('listings/listings_model');
		if (isset($_POST['save'])) {
		    gc_demo_mode();
		    $this->gcrud->stats(array('module' => 'images'));
			$this->form_validation->set_rules('modules[]', lang('applicable_modules'), 'trim|required|is_array');
			if($this->form_validation->run() == TRUE) {
				$modules = $this->input->post('modules');
				foreach($modules as $module => $value) {
					$this->images_model->thumbnail_generate($module);
				}
				Template::set_message(lang('thumbnail_success'), 'success');
				Template::redirect(current_url());
			}
		}
		Template::set('help_image', sprintf(lang('thumbnail_help'), settings_item('lst.thumbnail_width'), settings_item('lst.thumbnail_height')));
		Template::set('toolbar_title', lang('gc_menu_generate_thumbnails'));
		Template::set_view('modules_form');
		Template::render();
	}
	
	/**
	 * Update Images URL and PATH using module_id
	 */
	public function update_url() {
		$images = $this->images_model->find_all();
		$data = array();
		foreach($images as $image) {
			$module = gc_module_name($image->module_id);
			$path = 'assets/images/' . $module;
			if($image->type_id == 1) {
				$path .= '/covers';
			} else if($image->type_id == 2) {
				$path .= '/logos';
			}
			$url = $path . '/' . $image->filename;
			$data[] = array(
				'id' => $image->id,
				'url' => $url,
				'path' => $path
			);
		}
		$this->db->update_batch('images', $data, 'id'); 
	}
	
	/**
	 * Assign Cover Images
	 */
	public function assign_cover() {
		$this->images_model->assign_cover(true);
		Template::set_message(lang('thumbnail_success'), 'success');
		Template::redirect(site_url(SITE_AREA .'/content/images'));
	}
	
	/**
	 * Tweak Images
	 */
	public function tweak() {
	    $this->auth->restrict('Images.Content.Tweak');
	    $this->load->library('goclixy/gimages');
	    if (isset($_POST['save'])) {
	        gc_demo_mode();
	        $this->gcrud->stats(array('module' => 'images'));
	        $this->form_validation->set_rules('modules[]', lang('applicable_modules'), 'trim|required|is_array');
	        if($this->form_validation->run() == TRUE) {
	            $modules = $this->input->post('modules');
	            foreach($modules as $module => $value) {
	                $this->gimages->tweak_all($module);
	            }
	            Template::set_message(lang('msg_success'), 'success');
	            Template::redirect(current_url());
	        }
	    }
	    Template::set(array(
	        'toolbar_title' => lang('tweak_images'),
	        'action' => 'tweak'
	    ));
	    Template::set_view('modules_form');
	    Template::render();
	}
}