<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Migration_Install_images extends Migration
{
	/**
	 * The name of the database table
	 *
	 * @var String
	 */
	private $table_name = 'images';

	/**
	 * The table's fields
	 *
	 * @var Array
	 */
	private $fields = array(
		'id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'auto_increment' => TRUE,
		),
		'module_id' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
		),
		'row_id' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'null' => FALSE,
		),
		'type_id' => array( // 0 means normal image, 1 means cover image
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default' => 0
		),
		'title' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE,
		),
		'url' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => FALSE,
		),
		'path' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE,
		),
		'filename' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE,
		),
		'priority' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => TRUE,
		),
		'status' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default'=> 0,
		),
		'deleted' => array( //  added only to have a symmetry with all modules
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default'=> 0,
		)
	);
	
	/**
	 * Install this migration
	 *
	 * @return void
	 */
	public function up()
	{
		// IMAGES TABLE
		$this->dbforge->add_field($this->fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('module_id', false);
		$this->dbforge->add_key('row_id', false);
		$this->dbforge->add_key('url', false);
		$this->dbforge->add_key('status', false);
		$this->dbforge->add_key('deleted', false);
		$this->dbforge->create_table($this->table_name);
		$images_index_sql = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " ADD INDEX `module` (`module_id`, `row_id`)";
		$this->db->query ($images_index_sql);
	}

	//--------------------------------------------------------------------

	/**
	 * Uninstall this migration
	 *
	 * @return void
	 */
	public function down() {		
		if ($this->db->table_exists($this->table_name)) {
			$this->dbforge->drop_table($this->table_name);
		}
	}

	//--------------------------------------------------------------------
}