<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * IMPORTANT because of use of single table for all module images
 * 
 * The module parameter must be passed without any translation in the URL
 * Passing translated string will result in the image upload failure 
 * The module_id is retrieved using the passed string as the 3rd segment
 * 
 * type_id -> 0 means normal image, 1 means cover image, 2 means logo
 * A module can also have cover images
 * 
 * @author GoClixy
 *
 */
class Images_model extends BF_Model {

	protected $table_name	= "images";
	protected $key			= "id";
	protected $soft_deletes	= false;
	protected $date_format	= "datetime";

	protected $log_user 	= FALSE;

	protected $set_created	= false;
	protected $set_modified = false;

	/*
		Customize the operations of the model without recreating the insert, update,
		etc methods by adding the method names to act as callbacks here.
	 */
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 		= array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	/*
		For performance reasons, you may require your model to NOT return the
		id of the last inserted row as it is a bit of a slow method. This is
		primarily helpful when running big loops over data.
	 */
	protected $return_insert_id 	= TRUE;

	// The default type of element data is returned as.
	protected $return_type 			= "object";

	// Items that are always removed from data arrays prior to
	// any inserts or updates.
	protected $protected_attributes = array();

	/*
		You may need to move certain rules (like required) into the
		$insert_validation_rules array and out of the standard validation array.
		That way it is only required during inserts, not updates which may only
		be updating a portion of the data.
	 */
	protected $insert_validation_rules 	= array();
	protected $skip_validation 			= FALSE;

	//--------------------------------------------------------------------
	
	/**
	 * Upload image using CRUD
	 * Image_CRUD 0.6 is used to upload images
	 */
	public function upload_photos($module, $row_id) {
		$user_id = $this->auth->user_id();
		$this->load->model('members/members_model');
		$this->load->model('packages/packages_model');
		$this->load->library('crypt');
		$this->lang->load('members/members');
		$this->lang->load('listings/listings');
		
		// Only logged in users are having permissions to view this page
		$row_id = intval($row_id);
		$module_id = gc_module_id($module);
		
		// Product variants
		$toggle_module = null;
		$toggle_row_id = 0;
		if($module == 'variants') { // Ecommerce - Product Variant Images
		    $this->lang->load('ecommerce/ecommerce');
		    $module_redirect_url = gc_url_format('members/variants');
		    // get product id and use it as a row id
		    $variant_info = $this->db->get_where('product_variants', array('id' => $row_id))->row();
		    if(!$variant_info) {
		        Template::redirect($module_redirect_url);
		    }
		    $module = 'products';
		    $row_id = $variant_info->product_id;
		    $toggle_module = 'variants';
		    $toggle_row_id = $row_id;
		} else {
		    $module_redirect_url = gc_url_format($module .'/view') . '?id='. $this->crypt->encode($row_id) . '&md=' . $this->crypt->encode($module, true);
		}
		if($row_id == 0 || $module_id == 0) {
			Template::redirect(gc_url_format('members'));
		}
		
		// Get Module Info
		$select = 'user_id, package_id, title, ' . ($module == 'listings' ? 'id AS listing_id' : 'listing_id');
		$module_info = $this->db->select($select)->get_where($module, array('id' => $row_id))->row();
		if(!$module_info) {
			Template::redirect(gc_url_format('members'));
		}
		if($module_info->user_id != $user_id) {
			Template::set_message(lang('msg_permission_expired_error'), 'error');
			Template::redirect($module_redirect_url);
		}
		$listing_id = $module_info->listing_id;
		$package_id = $module_info->package_id;
		$package_info = $this->packages_model->find($package_id);
		if(!$package_info) {
			Template::redirect(gc_url_format('members'));
		}
		// Get images limit and status field value
		$images_limit = gc_field_value($module, $package_info->module_images_limit);
		$images_status = gc_field_value($module, $package_info->module_images_status);
		if($images_limit == 999) {
			$images_limit = -1; // Disable image uploading if disabled via Admin Panel
			Template::set_message(lang('msg_photo_permission_error'), 'error');
			Template::redirect($module_redirect_url);
		}
		
		// Limit Check
		$pyg_status = $this->members_model->pyg_check('images', $user_id, $package_id, $package_info, $row_id, $module);
		if(is_numeric($pyg_status) && $pyg_status > $images_limit) {
	        $images_limit = $pyg_status;
	    }
		if(!empty($toggle_module)) { // Ecommerce - Product Variants
		    $module = $toggle_module;
		    $row_id = $toggle_row_id;
		}
		// COUNT EXISTING IMAGES
		$images_count = $this->images_model->count_by(array('row_id' => $row_id, 'module_id' => $module_id, 'type_id' => 0));		
		$this->image_crud($module, $images_status, $images_limit, $images_count, $pyg_status);
		
		// Tweak Images
		$this->load->library('goclixy/gimages');
		$this->gimages->_tweak_images_row($module, $row_id);
		
		$toolbar_title = (!empty($module_info->title) ? $module_info->title . ' - ' . lang('photos') : lang('label_image_uploader'));
		Template::set(array(
		    'title'=> $toolbar_title,
		    'page_title'=> $toolbar_title,
		    'toolbar_title'=> $toolbar_title,
		));
		if($module == 'listings') {
			Template::set('module_links', '<li class="nav-item"><a href="'.current_url().'" class="nav-link active">'.lang('upload_photos').'</a></li><li class="nav-item"><a href="'. gc_url_format('members') . '?id=' .$this->crypt->encode($listing_id). '&md='. $this->crypt->encode($module, true) .'" class="nav-link">'.lang('gc_menu_' .$module).'</a></li>');
		} else if($module == 'variants') { // Ecommerce - Product Variants
		    Template::set('module_links', '<li class="nav-item"><a href="'. gc_url_format('members/variants') .'" title="' . lang('back_to_variants') . '" class="nav-link">' . lang('back_to_variants') . '</a></li>');
		} else {
			Template::set('module_links', '<li class="nav-item"><a href="'.current_url().'" class="nav-link active">'.lang('upload_photos').'</a></li><li class="nav-item"><a href="'. gc_url_format('members/view') . '?id=' .$this->crypt->encode($listing_id) . '&md='. $this->crypt->encode($module, true) .'" title="' . lang('gc_menu_' .$module) . '" class="nav-link">' . lang('gc_menu_' .$module) . '</a></li>');
		}
		Template::set_view('common/photos');
		// This method has no View in the Themes Module, i.e. load layout files manually
		$this->gc->demo_custom_css_js('member');
		Template::render('member');
	}
	
	/**
	 * Upload Photos Admin
	 * @param string $module
	 * @param int $row_id
	 * @param string $redirect_url
	 */
	public function upload_photos_admin($module, $row_id, $redirect_url) {	
		$module = $module == 'variants' ? 'product_variants' : $module;
		$module_id = gc_module_id($module);
		if((int) $row_id == 0 || $module_id == 0) {
			Template::redirect($redirect_url);
		}
		$this->image_crud($module, 1, 500, 0);
		
		// Tweak Images
		$this->load->library('goclixy/gimages');
		$this->gimages->_tweak_images_row($module, $row_id);
		
		$listing_id = 0;
		// Get Listing ID to go back to the module
		if($module == 'listings') {
			$listing_id = $row_id;
			$info = $this->db->select('id, title, slug, city')->get_where($module, array('id' => $row_id))->row();
		} else if($module != 'variants' && $module != 'product_variants') {
			$info = $this->db->select('listing_id, title')->get_where($module, array('id' => $row_id))->row();
			$listing_id = ($info ? $info->listing_id : 0);
		}
		Template::set(array(
			'module' => $module,
			'row_id' => $row_id,
			'listing_id' => $listing_id,
		    'toolbar_title' => (!empty($info->title) ? $info->title . ' - ' . lang('photos') : lang('upload_photos'))
		));
		Template::set_view('common/photos');
		Template::render();
	}
	
	/**
	 * Common Code for uploading images
	 * @param string $module
	 * @param number $images_status
	 * @param number $images_limit
	 * @param number $images_count
	 * @param string $pyg_status
	 */
	private function image_crud($module, $images_status, $images_limit, $images_count, $pyg_status = true) {
		$module_id = gc_module_id($module);
		/* --------------------------------- */
		$this->load->config('image_crud');
		$this->load->library('image_crud');
		$this->load->library('image_moo');
		/* --------------------------------- */
		$image_crud = new image_CRUD();
		$image_crud->set_primary_key_field('id');
		$image_crud->set_url_field('filename');
		$image_crud->set_title_field('title');
		$image_crud->set_active_field('status');
		$image_crud->active_field_value($images_status);
		$image_crud->module_id_field_value($module_id);
		$image_crud->user_limit($images_limit);
		$image_crud->images_count($images_count);
		$image_crud->pyg_status($pyg_status);
		$image_crud->user_role($this->session->userdata('role_id'));
		$image_crud->set_table('images')
			->set_relation_field('row_id')
			->set_module_id_field('module_id')
			->set_ordering_field('priority')
			->set_image_path('assets/images/' . $module); // CHANGE THIS FOR PATH
		$output = $image_crud->render();
		if(!empty($output)) {
    		Assets::add_css($output->css_files);
    		Assets::add_js($output->js_files);
		    Template::set('output', $output->output);
		}
		Template::set('help_image', sprintf(lang('image_help'), settings_item('lst.image_file_size'), settings_item('lst.image_width'), settings_item('lst.image_height')));
	}
	
	/**
	 * Get Images
	 * @param string $module
	 * @param int $row_id
	 * @param string $active_only - get Active images only
	 * @param number $type_id - 0 images, 1 - cover image, 2 - logo
	 * @param $query - counts -> return images count, single -> return single image only
	 */
	public function get_images($module, $row_id, $active_only = TRUE, $type_id = 0, $query = NULL) {
		$module_id = gc_module_id($module);
		$where = array();
		$where['module_id'] = $module_id;
		$where['row_id'] = $row_id;
		$where['type_id'] = $type_id;
		if($active_only == TRUE) {
			$where['status'] = 1;
		}
		switch($query) {
			case 'single':
				$result = $this->images_model->order_by('id', 'DESC')->find_by($where);
				break;
			case 'counts':
				$result = $this->images_model->count_by($where);
				break;
			default:
				$result = $this->images_model->order_by('id', 'DESC')->find_all_by($where);
		}
		return $result;
	}

	/**
	 * Save Image - Listings Cover and other Module Image
	 * @param string $module
	 * @param number $row_id
	 * @param string $image
	 * @param number $status
	 * @param number $type_id - 0 images, 1 - cover image, 2 - logo
	 * @param number $limit - Overwrite image if limit is 1
	 * $limit variable is added for Mobile App
	 */
	public function save_image($module, $row_id, $image, $status = 1, $type_id = 0, $limit = 1) {	
		// get the upload path
		$path = $this->get_upload_dir($module, true, $type_id);
		// move the image file from the temp folder and resize it
		$filename = $this->move_resize($image, $path);
		if(is_bool($filename) && !$filename) { // unable to move file, return false
			return false;
		}
		$url = $path . '/' . $filename;
		if($module == 'bookings') {
		    return $url;
		} else {
		    return $this->save_image_db($module, $row_id, $type_id, $path, $url, $filename, $status, $limit);
		}
	}
	
	/**
	 * Save image in the database
	 * 
	 * Also used in Bulk Import
	 * 
	 * @param string $module
	 * @param integer $row_id
	 * @param integer $type_id
	 * @param string $path
	 * @param string $url
	 * @param string $filename
	 * @param number $status
	 * @param string $bulk_import
	 * @return integer
	 */
	public function save_image_db($module, $row_id, $type_id, $path, $url, $filename, $status = 1, $limit = 1, $bulk_import = FALSE) {
		$module_id = gc_module_id($module);
		$where = array('module_id' => $module_id, 'row_id' => $row_id, 'type_id' => $type_id);
		// Bulk save multiple images, but do not save the same image with the same url
		if($bulk_import) {
			$where['url'] = $url;
		}
		
		// Save Logo and Cover image in the Modules Table for Featured, Popular and other Search Queries
		if($type_id != 0) {
			$key = ($type_id == 1 ? 'cover' : 'logo');
			$this->db->where(array('id' => $row_id))->update($module, array($key => $url));
		}
		
		$data = array('module_id' => $module_id, 'row_id' => $row_id, 'type_id' => $type_id, 'url' => $url, 'path' => $path, 'filename' => $filename, 'status' => $status);
		$insert = FALSE; // Allow user to insert image
		if($limit == 1) {
			$result = $this->images_model->select('filename, path')->order_by('id', 'DESC')->find_by($where);
			if($result) { // Image already exists, update it
				if($result->filename != $filename) { // both images are not same, upload the new image
					// delete previous image
					$this->purge_image($result->filename, $result->path);
					// update new image
					$this->db->where($where)->update($this->table_name, array('url' => $url, 'path' => $path, 'filename' => $filename, 'status' => $status));
				}
				return $row_id;
			} else {
				$insert = TRUE;
			}		
		} else {
			$insert = TRUE;
		}
		if($insert) {
			$this->db->insert($this->table_name, $data);
			$insert_id = $this->db->insert_id();
			if($type_id == 0 && !$bulk_import) {
				// Earn Points
				$this->gc->earn_points('images', 0, $insert_id);
			}
			return $insert_id;
		}
	}
	
	/**
	 * Common upload image function for all modules
	 *
	 * Always upload file in the temp directory and then move it to the main directory
	 * Validation errors in other fields resulted in the image copy to the main folder
	 *
	 * @param string $field_name
	 * @param string $type - image/logo/cover
	 * @param string $settings - pass config parameters in an array format
	 * @return multitype:NULL
	 */
	public function upload_image($field_name, $type = 'image', $settings = array()) {
	    if(!gc_demo_mode(array('no_message' => true, 'no_redirect' => true))) {
	        return false;
	    }	
		$config['upload_path'] = realpath(FCPATH .'/assets/images/temp/'); //Make SURE that you chmod this directory to 777!
		$config['allowed_types'] = 'gif|jpg|jpeg|png';
		if(!empty($settings) && !empty($settings['allowed_types'])) {
			$config['allowed_types'] = $settings['allowed_types'];
		}
		switch($type) {
			case 'image':
				$max_size = settings_item('lst.image_file_size');
				$max_width = settings_item('lst.image_width');
				$max_height = settings_item('lst.image_height');
				break;
			case 'logo':
				$max_size = settings_item('lst.logo_file_size');
				$max_width = settings_item('lst.logo_width');
				$max_height = settings_item('lst.logo_height');
				break;
			case 'cover':
				$max_size = settings_item('lst.cover_file_size');
				$max_width = settings_item('lst.cover_width');
				$max_height = settings_item('lst.cover_height');
				break;
			default:
				if(!empty($settings)) {
					$max_size = $settings['max_size'];
					$max_width = $settings['max_width'];
					$max_height = $settings['max_height'];
				} else {
					return false;
				}
		}
		$config['encrypt_name'] = TRUE; // Encrypt file name with use of CI native option for security reasons
		$config['max_size']= $max_size;
		/*$config['max_width']= $max_width;
		$config['max_height']= $max_height;		
		$new_filename = $this->rename_file(); // Rename file on upload
		if(!$file_name) {
		    $config['file_name'] = $new_filename;
		}*/
		$config['remove_spaces']=TRUE; //Remove spaces from the file name
		$this->load->library('upload', $config);
		if (!$this->upload->do_upload($field_name)) { // Image name is necessary
			$this->session->set_flashdata('upload_image_error', $this->upload->display_errors());
			return false;
		} else {
			$data = array('upload_data' => $this->upload->data());
			//RESIZE IMAGE USING resize FUNCTION
			$this->resize_image('assets/images/temp', $data['upload_data']['file_name'], $max_width, $max_height);
		}
		return $data;
	}
	
	/**
	 * Rename file for security reasons
	 */
	public function rename_file() {
	    if(!empty($_FILES['image']['name'])) {
	        $file_name = $_FILES['image']['name'];
	        $extension = str_replace('.', '', strrchr($file_name, '.'));
	        $file_array = explode('.' . $extension, $file_name ?? '');
	        
	        // get filename and extension
	        $filename = $file_array[0];
	        
	        // generate a new filename to save it in Database and Server
	        $file_template = "{UID}_" . $filename . "." . $extension;
	        
	        // the UID is unique and it is used for files downloading
	        $uid = substr(md5($file_template. time() . mt_rand(1, 999999)), 0, 8); // Name of the file on server
	        return str_replace("{UID}", $uid, $file_template);
	    } else {
	        return false;
	    }
	}
	
	/**
	 * Create thumbnails of uploaded image
	 * @param string $path
	 * @param string $upload_path
	 * @param string $file
	 * @param string $width - Width of the Image
	 * @param string $height - Height of the Image
	 * 
	 * Resize as per the information saved in the settings table
	 * 
	 */
	public function resize_image($upload_path, $file, $width = null, $height = null) {
	    $config = array();            
	    if(empty($width) || empty($height)) { // thumbnail generation
	        // If thumbs folder does not exist, create it
    		$thumbs_path = FCPATH . '/' . $upload_path . '/thumbs';
    		if(!is_dir($thumbs_path)) {
    			mkdir($thumbs_path, 0777, true);
    		}
    		$width = settings_item('lst.thumbnail_width');
    		$height = settings_item('lst.thumbnail_height');
    		$config['create_thumb'] = TRUE;
    		$config['thumb_marker'] = FALSE; // don't want to add the suffix _thumb in thumbnail file name
    		$config['new_image']='./' . $upload_path .'/thumbs/thumb__' . $file;
	    } else {
	        $config['quality'] = '70%'; // Sets the quality of the image. The higher the quality the larger the file size.
	    }
	    $config['image_library'] = 'gd2';
		// $config['source_image']=realpath(FCPATH.'/assets/images/'.$upload_path.'/' .$file);
		$config['source_image'] = realpath(FCPATH . '/' . $upload_path . '/' . $file);
		$config['maintain_ratio'] = TRUE;
		$config['overwrite'] = TRUE; // overwrite existing images
		$config['width'] = $width;
		$config['height'] = $height;
		
		/**
		 * Add Watermark to an image
		 * https://www.codeigniter.com/userguide3/libraries/image_lib.html
		 * https://stackoverflow.com/questions/2233144/codeigniter-image-manipulation-resize-watermark
		$config['wm_text'] = 'Copyright 2006 - John Doe'; // TEXT METHOD
		$config['wm_type'] = 'text'; // TEXT METHOD
		// $config['wm_type'] = 'overlay'; // OVERLAY METHOD
		// $config['wm_overlay_path'] = './images/logo.gif';// OVERLAY METHOD	
		$config['wm_vrt_alignment'] = 'middle';
		$config['wm_hor_alignment'] = 'center';
		*/
		
		$this->load->library('image_lib', $config);
		/**
		 * $this->image_lib->initialize($config); and $this->image_lib->clear();
		 * are important else, only one thumbnail will be saved
		*/
		$this->image_lib->initialize($config);
		$this->image_lib->resize();
		// $this->image_lib->watermark(); // Uncomment the above watermark code to Add Watermark
		$this->image_lib->clear();
	}
	
	/**
	 * Get module image e.g. categories/users/banners etc.
	 * @param string $module
	 * @param integer $row_id
	 */
	public function get_module_image($filename, $module, $row_id = 0, $field = 'image') {
		// delete previous image first
		$path = 'assets/images/' . $module;
		if($row_id != 0) {
			$result = $this->db->select($field)->get_where($module, array('id' => $row_id))->row();
			if($result) {
				if($this->is_new_image($module, $result->$field, $filename)) {
					$this->purge_image($result->$field, $path);
				}
			}
		}
		return $this->move_resize($filename, $path);
	}
	
	/**
	 * Check whether the uploaded image is an old or new image
	 * Module Specific - categories/users/banners - Update row without changing image
	 * @return boolean
	 */
	public function is_new_image ($module, $old_image, $new_image) {
		// get file size of old image
		$old_img_filesize = filesize(realpath(FCPATH.'/assets/images/' . $module .'/' . $old_image));
		
		// convert size in KB and show the filesize upto 2 decimal places
		$old_img_filesize = round($old_img_filesize/1024, 2);
		
		// get file size of new image
		$new_img_filesize = filesize(realpath(FCPATH.'/assets/images/' . $module .'/' . $new_image));
		
		// convert size in KB and show the filesize upto 2 decimal places
		$new_img_filesize = round($new_img_filesize/1024, 2);
		
		// delete old image only if the file size is different
		if($new_img_filesize != $old_img_filesize) {
			return true;
		}
		return false;
	}
	
	/**
	 * On module row purge, also delete the image from the specific folder
	 * Module Specific - categories/users/banners
	 * @param string $module
	 * @param number $row_id
	 * @param string $field
	 */
	public function purge_module_image($module, $row_id = 0, $field = 'image') {
		$path = 'assets/images/' . $module;
		if($row_id != 0) {
			$result = $this->db->select($field)->get_where($module, array('id' => $row_id))->row();
			if($result) {
				$this->purge_image($result->$field, $path);			
			}
		}
	}
	
	/**
	 * Move and Resize file before save
	 * @param string $filename - Name of the file
	 * @param string $upload_dir - Directory where the file is to be saved
	 * @param boolean $is_module - e.g. categories/users/banners etc.
	 * @param boolean $thumb - generate thumb if the thumb value is true
	 * @return string
	 */
	public function move_resize ($filename, $upload_dir, $thumb = true) {
		if(!empty($filename) && !empty($upload_dir)) {
			// check if the file with the same names exists in the destination folder
			$upload_path = FCPATH . $upload_dir;
			if(!is_dir($upload_path)) {
				mkdir($upload_path, 0777, true);
			}
			$temp_path = FCPATH . 'assets/images/temp/' . $filename;
			if(file_exists($upload_path . '/' . $filename)) {
				// rename file with random string
				$filename = uniqid(). '_' . $filename;
			}
			// move file only if temp file exists and destination is reachable
			$destination = $upload_path . '/' . $filename;
			if (file_exists($temp_path) && ((!file_exists($destination)) || is_writable($destination))) {
				rename($temp_path, $destination);
				
				// iPHONE ISSUE - read_exif_data and Adjust Image Orientation
				// get images rotated automatically via PHP when a user upload them
				$this->load->library('image_autorotate', array('filepath' => $destination));
				
				// create thumbnail
				if($thumb) {
					$this->resize_image($upload_dir, $filename);
				}
				
				// return filename to save in database
				return $filename;
			}
		}
		return false;		
	}
	
	/**
	 * Subdirectories under the uploads directory is the best to deal with large number of images
	 * get the name of the last folder this module image is saved
	 * @param string $module
	 * @return boolean $path - return the path of the last uploaded directory
	 * @param string $type - logo/cover
	 */
	public function get_upload_dir($module, $path = false, $type_id = 0) {
		$module_id = gc_module_id($module);	
		$module_path = $module;
		$where = array('module_id' => $module_id);
		$type = '';
		if($type_id != 0) {
			if($type_id == 2) {
				$type = 'logos';
			} else { // cover image
				$type = 'covers';
			}
			$module_path .= '/' . $type;
			$where['type_id'] = $type_id;
		}
		
		if($path) {
			$path = $this->image_upload_dir($module_path, $type_id);
			// image type is a logo or cover, but the path does not have any one of them
			// add TYPE to path
			if (!empty($type) && strpos($path, $type) === false) {
				$path .= '/' . $type;
				// check the folder existense
				// if not exists, create them
				$new_path = FCPATH . $path . '/';
				if(!is_dir($new_path)) {
					mkdir($new_path, 0777, true);
					mkdir($new_path .'/thumbs', 0777, true);
				}
			}
			return $path;
		} else {
			return 'assets/images/' . $module_path;
		}
	}
	
	/**
	 * Purge specific row images
	 * @param string $module
	 * @param int $row_id
	 * @param int $type_id - 99 Purge All Images of this module
	 */
	public function purge_images($module, $row_id, $type_id = 0) {
		$module_id = gc_module_id($module);
		// Also Remove the Logo and Cover image from the Modules Table
		if($type_id != 0 && $type_id != 99) {
			$key = ($type_id == 1 ? 'cover' : 'logo');
			$this->db->where(array('id' => $row_id))->update($module, array($key => null));
		}
		$where = array('module_id' => $module_id, 'row_id' => $row_id);
		if($type_id != 99) {
			$where['type_id'] = $type_id;
		}
		$images = $this->db->select('id, path, filename')->get_where('images', $where)->result();
		if(!empty($images)) {
			foreach($images as $image) {
				$this->purge_image($image->filename, $image->path);
				$this->db->where($where)->delete('images');
			}
		}
		
		// set auto increment value to the last id
		$alter_autoinc = "ALTER TABLE ". $this->db->dbprefix . "images AUTO_INCREMENT = 1";
		$this->db->simple_query($alter_autoinc);
	}
	
	/**
	 * Truncate Table Images
	 */
	public function truncate_table_images() {
		$modules = gc_modules();
		foreach($modules as $module) {
			$this->unlink_images($module);
		}
		$this->db->truncate('images');
	}
	
	/**
	 * Remove images from system on Table Truncate
	 * @param string $module
	 * @param string $table
	 */
	public function unlink_images($module, $table = NULL) {
		$module_id = gc_module_id($module);
		$module_images = $this->db->select('path, filename')->where(array('module_id' => $module_id))->get('images')->result();
		if(!empty($module_images)) {
			foreach($module_images as $module_image) {
				$this->purge_image($module_image->filename, $module_image->path);
			}
		}
		if(!empty($table)) {
			$this->db->truncate($table);
		}	
	}
	
	/**
	 * Mobile App and Gallery Image Delete - Purge image by ID
	 * @param integer $id
	 * @return boolean
	 */
	public function purge_image_by_id($id) {
		$result = $this->images_model->find($id);
		if($result) {
			// Remove Logo and Cover Image Tweak field from the specified module
			if($result->type_id == 1 || $result->type_id == 2) {
				$module = gc_module_name($result->module_id);
				$this->db->where(array('id' => $result->row_id))->update($module, array(($result->type_id == 1 ? 'cover' : 'logo') => null));
			}
			// Delete image from the server
			$this->purge_image($result->filename, $result->path);
			// Delete image from the database
			$this->db->where('id', $id)->delete($this->table_name);
			$this->_tweak($id, $result); // tweak images
			return true;
		}
		return false;
	}
	
	/**
	 * Purge Image
	 * @param string $filename
	 * @param string $path
	 */
	public function purge_image($filename, $path) {
		if(!empty($filename) && file_exists(realpath(FCPATH . $path . '/thumbs/thumb__' . $filename))) {
			unlink(realpath(FCPATH . $path . '/thumbs/thumb__' . $filename));
		}
		if(!empty($filename) && file_exists(realpath(FCPATH . $path . '/' . $filename))) {
			unlink(realpath(FCPATH . $path . '/' . $filename));
		}
	}
	
	/**
	 * View Page - Return Image Type
	 * @param number $type_id
	 */
	public function image_type ($type_id) {
		switch($type_id) {
			case 1:
				$str = lang('label_cover_image');
				break;
			case 2:
				$str = lang('label_logo');
				break;
			default:
				$str = lang('label_image');
		}
		return $str;
	}
	
	/**
	 * Generate Thumbnails
	 * @param string $module
	 */
	public function thumbnail_generate($module) {
		/**
		 * Sub-folders contained within the directory will be mapped as well.
		 * If you wish to control the recursion depth, you can do so using the second parameter (integer).
		 * A depth of 1 will only map the top level directory
		 * By default, hidden files will not be included in the returned array.
		 * To override this behavior, you may set a third parameter to true (boolean):
		 */
		/*$files = directory_map('./assets/images/' . $module); // OLD METHOD
		asort($files);*/
		$path = 'assets/images/' . $module;
		//$objects = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($path), RecursiveIteratorIterator::SELF_FIRST);
		$iterator = new RecursiveDirectoryIterator($path);
		// skip hidden files starting with dots e.g. /. or /..
		$iterator->setFlags(RecursiveDirectoryIterator::SKIP_DOTS);
		$objects = new RecursiveIteratorIterator($iterator, RecursiveIteratorIterator::SELF_FIRST);
		foreach($objects as $image => $object) {
			$filename = $object->getFilename();
			// get only image files based on their extensions
			if(strpos($filename, 'thumb__') === false) { // process only main images
				// Get file extension info
				$info = pathinfo($filename);
				if(isset($info['extension'])) {
					switch(strtolower($info['extension'])) {
						case "jpg":
						case "jpeg":
						case "png":
						case "gif":
							// Get the image path
							$image_path = str_replace('\\', '/', $object->getPathname());
							
							// explode the image path to an array
							$path_array = array_map('trim', explode('/', $image_path ?? ''));
							
							// remove the filename from the array and store it in a variable
							array_pop($path_array);
							
							// thumb path
							$thumb_path = implode('/', $path_array);
							
							// generate thumbnail
							// overwrite existing image if exists
							$this->resize_image($thumb_path, $filename);							
							break;
						}
				}
			}
		}
	}
	
	/**
	 * Get image upload directory
	 * 
	 * Subdirectories under the uploads directory is the best to deal with large number of images
	 * Create new folder for images dynamically, when one directory gets 1000 images
	 *
	 * https://stackoverflow.com/questions/26606632/php-create-new-folder-dynamically-after-thousand-images-in-folder
	 *
	 * @param string $folder - image folder root location
	 * e.g. listings, listings/logos, listings/covers etc. 
	 * @param number $type_id - Include Logo and Covers path if the $type value is > 0
	 * @param string $thumb - create thumb if true
	 * @return string
	 */
	public function image_upload_dir($folder, $type_id = 0, $thumb = true) {
		// $folder == 'product_variants' ? 'variants' : $folder;
		$module_path = 'assets/images/' . $folder;
		$last_upload_dir = $module_path;
		$path = $last_upload_dir;
		// get the last saved path
		$image_upload_path = $this->read_file($folder, $type_id);
		if(!empty($image_upload_path)) {
			$last_upload_dir = $image_upload_path;//$image_upload_path->path;
		}
		$dir = FCPATH . $last_upload_dir .'/';
		if(is_dir($dir)) {
			$images = glob("$dir{*.gif,*.jpg,*.jpeg,*.JPG,*.png,*.PNG}", GLOB_BRACE); //you can add .gif or other extension as well
			if(count($images) > 1000) { // update the count to increase the folder limit
				// dump($module_path .' - '. $last_upload_dir . ' - ' . count($images));
				// create new folder using month and year
				$foldername = strtolower(date('d-M-Y')) . '-' . rand(1, 1000);
				$image_path = $module_path . '/' . $foldername;
				$new_path = FCPATH . $image_path . '/';
				if(!is_dir($new_path)) {
					mkdir($new_path, 0777, true);
					if($thumb) { // not required in case of qrcodes
						mkdir($new_path .'/thumbs', 0777, true);
					}
				}
				$return = $image_path;
			} else {
				$return = $last_upload_dir;
			}
		} else {
			$return = $module_path;
		}
		$this->write_file($folder, $type_id, $return);
		return $return;
	}
	
	/**
	 * Save last image upload path in the images_path.txt file
	 * This file will reside in the assets/images folder
	 * 
	 * @param string $folder
	 * @param string $path
	 * @return boolean
	 */
	private function write_file($folder, $type_id, $path) {
		$result = $this->read_file($folder, $type_id, $path);
		if(empty($result['content']) || $result['line_found'] == 0) {
			$result['content'][] = $folder . '=' . $path;
		}
		$content = serialize($result['content']);
		// Write the changes out...
		if (! function_exists('write_file')) {
			$this->load->helper('file');
		}
		
		// save file
		if (write_file(FCPATH . 'assets/images/images_path.txt', $content)) {
			return true;
		}
		return false;
	}
	
	/**
	 * Read images_path.txt file to get the last saved path
	 * if the $path variable is passed, it means it is requested by the write_file method
	 * 
	 * @param string $folder
	 * @param number $type_id - Exclude Logo and Covers path if the $type_id value is 0
	 * Listings Module - Saving logo first and then uploading type_id 0 image was saving image in the logos folder.
	 * @param string $path
	 * @return array|string|multitype:number mixed
	 */
	private function read_file($folder, $type_id = 0, $path = NULL) {
		$file = FCPATH . 'assets/images/images_path.txt';
		$image_path = '';
		$lines = array();
		$line_found = 0;
		if(file_exists($file)) {
			$file_content = file($file);
			$lines = unserialize($file_content[0]);		
			foreach ($lines as $lineNumber => $line) {
				if (strpos($line, $folder ?? '') !== false) {
					if(empty($path) || $path == NULL) {
						$path = explode('=', $line ?? '');
						return $path[1];
					} else {
						// prepare content
						$lines[$lineNumber] = $folder . '=' . $path;
						$line_found = 1;
					}					
				}				
			}
		}
		if(empty($path)) {
			return $image_path;
		} else {
			return array('content' => $lines, 'line_found' => $line_found);
		}
	}
	
	/**
	 * Remove all empty folders or subfolders recursively
	 * 
	 * https://stackoverflow.com/questions/1833518/remove-empty-subfolders-with-php
	 * @param string $path - it must be a path not the folder or module name e.g. 'assets/images/qrcodes'
	 */
	public function remove_empty_dir($path) {
		if(substr($path,-1)!= DIRECTORY_SEPARATOR){
			$path .= DIRECTORY_SEPARATOR;
		}
		$d2 = array('.','..');
		$dirs = array_diff(glob($path.'*', GLOB_ONLYDIR),$d2);
		foreach($dirs as $d){
			$this->remove_empty_dir($d);
		}
	
		if(count(array_diff(glob($path.'*'),$d2))===0){
			$checkEmpSubDir = explode(DIRECTORY_SEPARATOR,$path ?? '');
			for($i=count($checkEmpSubDir)-1;$i>0;$i--){
				$path = substr(str_replace($checkEmpSubDir[$i],"",$path),0,-1);
	
				if(($files = @scandir($path)) && count($files) <= 2){
					rmdir($path);
				}
			}
		}
	}
	
	/**
	 * GoClixy Version Upgrade - Insert Images information in Tweak Columns
	 */
	public function _upgrade_images() {
		$config_modules = gc_modules();
		foreach($config_modules as $config_module) {
			// Get images 
			$module_id = gc_module_id($config_module);
			if($config_module == 'listings') {
				// Get logo and cover images both
				$logos = $this->images_model->find_all_by(array('module_id' => $module_id, 'type_id' => 2));
				if($logos) {
					foreach($logos as $logo) {
						$this->db->where('id', $logo->row_id)->update($config_module, array('logo' => $logo->url));
					}
				}
			}
			// Get Cover Images All Modules
			$covers = $this->images_model->find_all_by(array('module_id' => $module_id, 'type_id' => 1));
			if($covers) {
				foreach($covers as $cover) {
					$this->db->where('id', $cover->row_id)->update($config_module, array('cover' => $cover->url));
				}
			}
		}
	}
	
	/**
	 * Add the Tweak information for the individual Module Row
	 * @param string $module
	 * @param integer $row_id
	 * @param boolean $is_update - TRUE - Update instantly, FALSE - Return value for Bulk Tweaking of Multiple Rows
	 */
	public function _tweak_images_row($module, $row_id, $is_update = true) {
		// Get images
		$module_id = gc_module_id($module);
		$data = array();
		if($module == 'listings') {
			// Get logo and cover images both
			$logo = $this->images_model->find_by(array('module_id' => $module_id, 'type_id' => 2, 'row_id' => $row_id));
			if($logo) {
				$data['logo'] = $logo->url;
				$data['logo_url'] = $logo->url;
			}
		}
		// Get Cover Image of the Module
		$cover = $this->images_model->find_by(array('module_id' => $module_id, 'type_id' => 1, 'row_id' => $row_id));
		if($cover) {
			$data['cover'] = $cover->url;
		}
		if(!empty($data)) {
			if($is_update) {
				$this->db->where(array('id' => $row_id))->update($module, $data);
			} else {
				return $data;
			}
		}
	}
	
	/**
	 * Modules - Assign Cover from the existing Images
	 */
	public function assign_cover($tweak_existing = false) {
		$modules = gc_modules_only();
		foreach($modules as $module) {
			$module_id = gc_module_id($module);
			// Get all rows
			$result = $this->db->get($module)->result();
			if($result) {
				foreach($result as $row) {
					$count = $this->db->where(array('module_id' => $module_id, 'row_id' => $row->id, 'type_id' => 1))->count_all_results('images');
					if($count <= 0) {
						$cover = $this->db->where(array('module_id' => $module_id, 'row_id' => $row->id, 'type_id' => 0))->order_by('id')->get('images')->row();
						if($cover) {
							$this->db->where(array('id' => $cover->id))->update('images', array('type_id' => 1));
						}
					} else if($tweak_existing) {
						$this->_tweak_images_row($module, $row->id);
					}
				}
			}
		}
	}
	
	/**
	 * Filemanager Image handling
	 * @param string $filename - File Path without assets/images
	 * @param string $module - e.g. categories
	 * @param number $id - id of the row
	 * @param string $prefix - name to be appended with public id
	 */
	public function image_filemanager($filename, $module, $id, $field = 'image', $prefix = null) {
	    if(settings_item('site.cloudinary') == 0) { // the cloudinary is off, images are stored locally
	        return $filename;
	    } else {
	        $new_upload = false;
	        if(!empty($id)) {
	            $record = $this->db->where('id', $id)->get($module)->row();
	            if($record) {
	                $public_id = $this->filemanager_public_id($record->{$field}, $module);
	                if($public_id != $filename) { // remove old image from the server
	                    if(!empty($public_id)) {
	                        $this->cloudinary($public_id, null, 'delete');
	                    }
	                    if(!empty($filename)) { // upload new file on the server
	                        $new_upload = true;
	                    }
	                }
	            }
	        } else if(!empty($filename)) {
	            $new_upload = true;
	        }
	        if($new_upload) {
	            // upload new file on the server
	            $public_id = $this->filemanager_public_id($filename, $module);
	            if (strpos($public_id, '.') !== false) {
	                $full_path = FCPATH . 'assets/images/' . $filename;
	                if(file_exists($full_path)) {
	                    /*if(strpos(base_url(), 'localhost/') !== false) {
	                     // unable to upload via localhost URL - convert file to base64
	                     $type = pathinfo($full_path, PATHINFO_EXTENSION);
	                     $data = file_get_contents($full_path);
	                     $file = 'data:image/' . $type . ';base64,' . base64_encode($data);
	                     } else {
	                     $file = base_url('assets/images/' . $filename);
	                     }*/
	                    // send data always in base64 format
	                    $type = pathinfo($full_path, PATHINFO_EXTENSION);
	                    $data = file_get_contents($full_path);
	                    $file = 'data:image/' . $type . ';base64,' . base64_encode($data);
	                    $public_id = explode('.', $public_id ?? '');
	                    $public_id = (empty($prefix) ? $module : $prefix) . '_' . $public_id[0];
	                    $this->cloudinary($public_id, $file, 'upload');
	                }
	            }
	            return $this->public_id($public_id);
	        }
	    }
	}
	
	/**
	 * Filemanager - Get Filename
	 */
	public function filemanager_public_id($filename, $module, $check_extension = false) {
	    if (strpos($filename, '/') !== false) {
	        $public_id = explode($module . '/', $filename ?? '');
	        $public_id = $public_id[1];
	        /*if (strpos($public_id, '.') !== false) {
	         $public_id = explode('.', $public_id ?? ''); // remove extension from the file
	         $public_id = $public_id[0];
	         }*/
	    } else {
	        $public_id = $filename;
	    }
	    // dump(substr( $filename, strrpos( $filename, '/' ) + 1));
	    return $public_id;
	}
	
	/**
	 * Tweak Images Row
	 * @param number $id
	 */
	public function _tweak($id, $record = array()) {
	    if(empty($record)) {
	        $record = $this->db->where('id', $id)->get('images')->row();
	    }
	    if($record) {
	        $this->load->library('goclixy/gimages');
	        $this->gimages->_tweak_images_row(gc_module_name($record->module_id), $record->row_id);
	    }
	}
}
