<?php defined('BASEPATH') || exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Settings controller
 */
class Settings extends Admin_Controller
{
    protected $permissionCreate = 'Languages.Settings.Create';
    protected $permissionDelete = 'Languages.Settings.Delete';
    protected $permissionEdit   = 'Languages.Settings.Edit';
    protected $permissionView   = 'Languages.Settings.View';

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
        clear_cache('languages');
        $this->auth->restrict($this->permissionView);
        $this->load->model('languages/languages_model');
        $this->lang->load('languages');
        
      	$this->form_validation->set_error_delimiters("<span class='error'>", "</span>");        
        Template::set_block('sub_nav', 'settings/_sub_nav');
        Assets::add_module_js('languages', 'languages.js');
    }

    /**
     * Display a list of Languages data.
     *
     * @return void
     */
    public function index($offset = 0)
    {
        // Status Change
        if (isset($_POST['status_change'])) {
            gc_demo_mode();
            $this->gcrud->stats(array('module' => 'languages'));
            $this->gc->status_change('languages', $_POST['status_change'], 'status');
        }
        // Deleting anything?
        if (isset($_POST['delete'])) {
            gc_demo_mode();
            $this->gcrud->stats(array('module' => 'languages'));
            $this->auth->restrict($this->permissionDelete);
            $checked = $this->input->post('checked');
            if (is_array($checked) && count($checked)) {

                // If any of the deletions fail, set the result to false, so
                // failure message is set if any of the attempts fail, not just
                // the last attempt

                $result = true;
                foreach ($checked as $pid) {
                    $deleted = $this->languages_model->delete($pid);
                    if ($deleted == false) {
                        $result = false;
                    }
                }
                if ($result) {
                    Template::set_message(count($checked) . ' ' . sprintf(lang('msg_action_success'), lang('label_languages'), lang('label_deleted')), 'success');
                } else {
                    Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_languages')) . $this->languages_model->error, 'danger');
                }
            }
        }
        
	    $like_array = array();
	    $searchterm = $this->gc->search('languages', $this->uri->segment(5), '/settings/languages/index');
	    if(!empty($searchterm)) {
	    	// This is the basic search filter, you could add more fields to the search
	    	$like_array = array(
	    	    'id' => $searchterm,
	    	    'code' => $searchterm,
	    	    'name' => $searchterm
	   		);
	    }
        $pagerUriSegment = 5;
        $pagerBaseUrl = site_url(SITE_AREA . '/settings/languages/index') . '/';
        
        $limit  = $this->settings_lib->item('site.list_limit') ?: 15;
        if(!empty($like_array)) {
			$this->languages_model->or_like($like_array);
		}
		$total_rows = $this->languages_model->count_all();
        $this->load->library('pagination');
        $pager['base_url']    = $pagerBaseUrl;
        $pager['total_rows']  = $total_rows;
        $pager['per_page']    = $limit;
        $pager['uri_segment'] = $pagerUriSegment;

        $this->pagination->initialize($pager);
        $this->languages_model->limit($limit, $offset);
        
   	 	$order_by = $this->gc->sort('languages');
    	if(!empty($order_by)) {
    		$this->languages_model->order_by($this->session->userdata('languages_sort')['column'], $this->session->userdata('languages_sort')['order']);
    	} else {
    		$this->languages_model->order_by('id', 'DESC');
    	}
        
    	if(!empty($like_array)) {
			$this->languages_model->or_like($like_array);
		}
        $records = $this->languages_model->find_all();

        Template::set('records', $records);
    	Template::set('toolbar_title', lang('manage_languages'));
        Template::render();
    }
    
    /**
     * Create a Languages object.
     *
     * @return void
     */
    public function add()
    {
        $this->auth->restrict($this->permissionCreate);
        
        if (isset($_POST['save'])) {
            gc_demo_mode();
            $this->gcrud->stats(array('module' => 'languages'));
            if ($insert_id = $this->save_language()) {
                log_activity($this->auth->user_id(), lang('act_create_record') . ': ' . $insert_id . ' : ' . $this->input->ip_address(), 'languages');
                Template::set_message(sprintf(lang('msg_save_success'), lang('label_language')), 'success');

                redirect(SITE_AREA . '/settings/languages');
            }

            // Not validation error
            if ( ! empty($this->languages_model->error)) {
                Template::set_message(sprintf(lang('msg_create_failure'), lang('label_language')) . $this->languages_model->error, 'danger');
            }
        }

        Template::set('toolbar_title', lang('add_language'));
        Template::set_view('settings/languages_form');
        Template::render();
    }
    /**
     * Allows editing of Languages data.
     *
     * @return void
     */
    public function edit()
    {
        $id = $this->uri->segment(5);
        if (empty($id)) {
            Template::set_message(lang('invalid_id'), 'error');
            redirect(SITE_AREA . '/settings/languages');
        }
        
        if (isset($_POST['save'])) {
            gc_demo_mode();
            $this->gcrud->stats(array('module' => 'languages'));
            $this->auth->restrict($this->permissionEdit);

            if ($this->save_language('update', $id)) {
                log_activity($this->auth->user_id(), lang('act_edit_record') . ': ' . $id . ' : ' . $this->input->ip_address(), 'languages');
                Template::set_message(sprintf(lang('msg_save_success'), lang('label_language')), 'success');
                redirect(SITE_AREA . '/settings/languages');
            }

            // Not validation error
            if ( ! empty($this->languages_model->error)) {
                Template::set_message(sprintf(lang('msg_edit_failure'), lang('label_language')) . $this->languages_model->error, 'danger');
            }
        } elseif (isset($_POST['delete'])) {
            gc_demo_mode();
            $this->gcrud->stats(array('module' => 'languages'));
            $this->auth->restrict($this->permissionDelete);

            if ($this->languages_model->delete($id)) {
                log_activity($this->auth->user_id(), lang('act_delete_record') . ': ' . $id . ' : ' . $this->input->ip_address(), 'languages');
                Template::set_message(sprintf(lang('msg_delete_success'), lang('label_language')), 'success');
                redirect(SITE_AREA . '/settings/languages');
            }
            Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_language')) . $this->languages_model->error, 'danger');
        }
        
        Template::set('record', $this->languages_model->find($id));

        Template::set('toolbar_title', lang('edit_language'));        
        Template::set_view('settings/languages_form');
        Template::render();
    }

    //--------------------------------------------------------------------------
    // !PRIVATE METHODS
    //--------------------------------------------------------------------------

    /**
     * Save the data.
     *
     * @param string $type Either 'insert' or 'update'.
     * @param int    $id   The ID of the record to update, ignored on inserts.
     *
     * @return boolean|integer An ID for successful inserts, true for successful
     * updates, else false.
     */
    private function save_language($type = 'insert', $id = 0)
    {
        if ($type == 'update') {
            $_POST['id'] = $id;
        }

        // Validate the data
        $this->form_validation->set_rules($this->languages_model->get_validation_rules());
        if ($this->form_validation->run() === false) {
            return false;
        }

        // Make sure we only pass in the fields we want
        
        $data = $this->languages_model->prep_data($this->input->post());
        $data['is_rtl'] = $this->input->post('is_rtl') ? 1 : 0; 

        // Additional handling for default values should be added below,
        // or in the model's prep_data() method
        

        $return = false;
        if ($type == 'insert') {
            $id = $this->languages_model->insert($data);

            if (is_numeric($id)) {
                $return = $id;
            }
        } elseif ($type == 'update') {
            $return = $this->languages_model->update($id, $data);
        }

        return $return;
    }
    
    /**
     * Translate All Language Files using Google Translate and existing translated files
     * 
     * Note: place all existing language files inside the public_html/languages folder. e.g. public_html/languages/french
     */
    private function translate_all() {
        $this->load->model('languages/languages_model');
        $this->languages_model->translate_all();
    }
    
    /**
     * Replace invalid strings in all language files
     */
    private function replace_invalid_strings() {
        $this->load->model('languages/languages_model');
        $this->load->config('translate/langcodes');
        $this->load->helper('translate/languages');
        $this->load->library('translate/translate_lib');
        $config = & get_config();
        
        // Load all language codes from the config file
        $codes = $this->translations->get_languages(false);
        foreach($codes as $language => $code) {
            $this->languages_model->replace_invalid_strings($language);
        }
    }
}