<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Migration_Install_languages extends Migration
{
    /**
     * The name of the database table
     *
     * @var String
     */
    private $table_name = 'languages';
    private $table_translations = 'translations';
    
    /**
     * The table's fields
     *
     * @var Array
     */
    private $fields = array(
        'id' => array(
            'type' => 'INT',
            'constraint' => 11,
            'auto_increment' => TRUE,
        ),
        'code' => array(
            'type' => 'VARCHAR',
            'constraint' => 3,
            'null' => FALSE,
            'unique' => TRUE
        ),
        'label' => array(
            'type' => 'VARCHAR',
            'constraint' => 40,
            'null' => FALSE,
            'unique' => TRUE
        ),
        'name' => array(
            'type' => 'VARCHAR',
            'constraint' => 40,
            'null' => FALSE
        ),
        'alpha_2' => array(
            'type' => 'VARCHAR',
            'constraint' => 2,
            'null' => FALSE,
            'unique' => TRUE
        ),
        'locale' => array(
            'type' => 'VARCHAR',
            'constraint' => 5,
            'null' => TRUE
        ),
        'date_format' => array(
            'type' => 'VARCHAR',
            'constraint' => 40,
            'null' => TRUE
        ),
        'datetime_format' => array(
            'type' => 'VARCHAR',
            'constraint' => 40,
            'null' => TRUE
        ),
        'is_rtl' => array(
            'type' => 'TINYINT',
            'constraint' => 1,
            'null' => FALSE,
            'default'=> '0'
        ),
        'status' => array(
            'type' => 'TINYINT',
            'constraint' => 1,
            'null' => FALSE,
            'default'=> '1'
        )
    );
    
    /**
     * The translations table's fields
     *
     * @var Array
     */
    private $field_translations = array(
        'id' => array(
            'type' => 'BIGINT',
            'constraint' => 20,
            'auto_increment' => TRUE,
        ),
        'language_code' => array(
            'type' => 'VARCHAR',
            'constraint' => 3,
            'null' => FALSE,
            'unique' => TRUE
        ),
        'module_id' => array(
            'type' => 'TINYINT',
            'constraint' => 1,
            'null' => FALSE,
            'default'=> '0'
        ),
        'row_id' => array(
            'type' => 'BIGINT',
            'constraint' => 20,
            'default' => 0
        ),
        'name' => array(
            'type' => 'VARCHAR',
            'constraint' => 50,
            'null' => FALSE
        ),
        'value' => array(
            'type' => 'LONGTEXT',
            'null' => FALSE
        ),
    );
    
    /**
     * Install this migration
     *
     * @return void
     */
    public function up() {
        $this->dbforge->add_field($this->fields);
        $this->dbforge->add_key('id', true);
        $this->dbforge->create_table($this->table_name);
        
        // Translations Table
        $this->dbforge->add_field($this->field_translations);
        $this->dbforge->add_key('id', true);
        $this->dbforge->create_table($this->table_translations);
        
        // Indexes
        $unique_index = "ALTER TABLE " . $this->db->dbprefix . $this->table_translations . " ADD UNIQUE INDEX `unique_translation` (`language_code`, `module_id`, `row_id`, `name`)";
        $this->db->query ($unique_index);
        
        $idx_module = "ALTER TABLE " . $this->db->dbprefix . $this->table_translations . " ADD INDEX `module` (`module_id`, `row_id`)";
        $this->db->query ($idx_module);
        
        $idx_value = "ALTER TABLE " . $this->db->dbprefix . $this->table_translations . " ADD INDEX `idx_value` (`language_code`, `module_id`, `row_id`)";
        $this->db->query ($idx_value);
    }
    
    //--------------------------------------------------------------------
    
    /**
     * Uninstall this migration
     *
     * @return void
     */
    public function down() {
        $this->dbforge->drop_table($this->table_name);
        $this->dbforge->drop_table($this->table_translations);
    }
    
}