<?php defined('BASEPATH') || exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class Languages_model extends BF_Model
{
    protected $table_name	= 'languages';
	protected $key			= 'id';
	protected $date_format	= 'datetime';

	protected $log_user 	= false;
	protected $set_created	= false;
	protected $set_modified = false;
	protected $soft_deletes	= false;


	// Customize the operations of the model without recreating the insert,
    // update, etc. methods by adding the method names to act as callbacks here.
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 	    = array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	// For performance reasons, you may require your model to NOT return the id
	// of the last inserted row as it is a bit of a slow method. This is
    // primarily helpful when running big loops over data.
	protected $return_insert_id = true;

	// The default type for returned row data.
	protected $return_type = 'object';

	// Items that are always removed from data prior to inserts or updates.
	protected $protected_attributes = array();

	// You may need to move certain rules (like required) into the
	// $insert_validation_rules array and out of the standard validation array.
	// That way it is only required during inserts, not updates which may only
	// be updating a portion of the data.
	protected $validation_rules 		= array(
	    array(
	        'field' => 'code',
	        'label' => 'lang:label_code',
	        'rules' => 'trim|required|max_length[2]|unique[languages.code, languages.id]',
	    ),
	    array(
	        'field' => 'label',
	        'label' => 'lang:label_label',
	        'rules' => 'trim|required|max_length[40]|unique[languages.label, languages.id]',
	    ),
		array(
			'field' => 'name',
			'label' => 'lang:label_name',
			'rules' => 'trim|required|max_length[40]',
		),
	    array(
	        'field' => 'alpha_2',
	        'label' => 'lang:label_alpha',
	        'rules' => 'trim|required|max_length[2]|unique[languages.alpha_2, languages.id]',
	    ),
	    array(
	        'field' => 'alpha_3',
	        'label' => 'lang:label_alpha',
	        'rules' => 'trim|required|max_length[3]|unique[languages.alpha_3, languages.id]',
	    ),
		array(
			'field' => 'locale',
			'label' => 'lang:label_locale',
			'rules' => 'max_length[5]',
		),
		array(
			'field' => 'date_format',
			'label' => 'lang:label_date_format',
			'rules' => 'max_length[40]',
		),
		array(
			'field' => 'datetime_format',
			'label' => 'lang:label_datetime_format',
			'rules' => 'max_length[40]',
		),
		array(
			'field' => 'is_rtl',
			'label' => 'lang:label_is_rtl',
			'rules' => 'max_length[1]',
		),
		array(
			'field' => 'status',
			'label' => 'lang:label_status',
			'rules' => 'max_length[1]',
		),
	);
	protected $insert_validation_rules  = array();
	protected $skip_validation 			= false;
	protected $gc_translate_cache;

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();        
        // Load Zend Caching
        $this->load->library('slug');
        $this->load->library('gzf_cache', array('lifetime' => 900));
        if(method_exists($this->gzf_cache, 'get_instance')) {
            $this->gc_translate_cache = $this->gzf_cache->get_instance();
        }
    }
    
    /***********************************************************
     * BULK LANGUAGE TRANSLATE USING LOCAL AND GOOGLE TRANSLATE
     ***********************************************************/    
    /**
     * CronJob to translate all languages at once.
     * Upload these files on GoClixy Servers for users.
     *
     * Set a Cronjob in the Admin Panel Settings->CronJobs and run it after 10 minutes.
     *
     * Drop the cron_translations table manually in order to avoid further translations using CronJobs
     *
     * @param boolean $overwrite_all - 1 - Overwrite all keys, do not skip existing one
     * @param string $specific_language - e.g. swedish for swedish translations only
     */
    public function translate_all($overwrite_all = 0, $specific_language = null) {
        $table_name = 'cron_translations';
        // Create a new table as follows:
        // id, language code, filename and translated_on, translated_ip
        if (! $this->db->table_exists($table_name)) {
            // table exists
            $this->load->dbforge();
            $this->dbforge->add_field(array(
                'id' => array(
                    'type' => 'INT',
                    'constraint' => 5,
                    'unsigned' => TRUE,
                    'auto_increment' => TRUE
                ),
                'code' => array(
                    'type' => 'CHAR',
                    'constraint' => '5',
                    'null' => false
                ),
                'name' => array(
                    'type' => 'VARCHAR',
                    'constraint' => '40',
                    'null' => false
                ),
                'file' => array(
                    'type' => 'VARCHAR',
                    'constraint' => '40',
                    'null' => false
                ),
                'translated_on' => array(
                    'type' => 'DATETIME',
                    'null' => true
                ),
                'translated_ip' => array(
                    'type' => 'VARCHAR',
                    'constraint' => 50,
                    'null' => true
                )
            ));
            $this->dbforge->add_key('id', TRUE); // add `id` as primary key
            $this->dbforge->create_table($table_name);
            $unique_translation = "ALTER TABLE " . $this->db->dbprefix . $table_name . " ADD UNIQUE INDEX `unique_translation` (`code`, `name`, `file`)";
            $this->db->query($unique_translation);
        }
        $origLang = 'english';
        $origLangCode = 'en';
        
        $this->load->config('translate/langcodes');
        $this->load->helper('translate/languages');
        $this->load->library('translate/translate_lib');
        $config = & get_config();
        
        // Get specific language code
        $codes = array();
        if(!empty($specific_language)) {
            $specific_language = strtolower($specific_language);
            $two_digits_code = $this->translations->get_languages(false, $specific_language, null);
            if(!empty($two_digits_code)) {
                $codes = array($specific_language => $two_digits_code);
            }
        } else {
            // Load all language codes from the config file
            $codes = $this->translations->get_languages(false);
        }
        
        // Get all language files
        // Check whether there are custom modules with lang files.
        $allLangFiles = list_lang_files();
        $core_files = $allLangFiles['core'];
        $custom_files = $allLangFiles['custom'];
        $allLangFiles = array_merge($core_files, $custom_files);
        if(empty($allLangFiles) || empty($codes)) {
            return false;
        }
        
        foreach ($codes as $language_name => $language_code) {
            if($language_code != 'en') { // skip english language as this is the source
                foreach ($allLangFiles as $langFile) {
                    // check whether this file is already translated or not
                    $is_translated = $this->db->where(array(
                        'code' => $language_code,
                        'name' => $language_name,
                        'file' => $langFile
                    ))->count_all_results($table_name);
                    
                    // language string must be translated before save
                    $processed = false;
                    
                    if ($is_translated == 0) {
                        // Load the original language file
                        $orig = load_lang_file($langFile, $origLang);
                        if($orig) {
                            // Load the new language file
                            $new = load_lang_file($langFile, $language_name);
                            
                            // Check whether the destination file exists on not
                            // if the destination language exists translate missing keys only
                            $destination_exist = true;
                            
                            // Language file does not exist, assign original language
                            if (! $new) {
                                $new = $orig;
                                // Destination does not exist
                                // Translate all keys
                                $destination_exist = false;
                            }
                            
                            // Start the translation
                            $errcnt = 0;
                            $cnt = 0;
                            
                            // do not unset the google translate session variable
                            // $this->session->unset_userdata('stichoza');
                            
                            $i = 0;
                            // Attempt to translate each of the selected values.
                            foreach ($orig as $key => $value) {
                                if (($destination_exist && empty($new[$key])) || ! $destination_exist || $overwrite_all == 1) {
                                    if(isset($new[$key])) {
                                        $new[$key] = '* ' . $new[$key];
                                    } else {
                                        $new[$key] = '* ' . $orig[$key];
                                    }
                                    $val = '';
                                    if ($language_code) {
                                        $val = null;
                                        // try to get information from the local files translated earlier
                                        $processed_file = $this->read_lang_file($language_name, $langFile, $core_files, $custom_files);
                                        if(!empty($processed_file[$language_name][$langFile])) {
                                            if(array_key_exists($key, $processed_file[$language_name][$langFile])) {
                                                $val = $processed_file[$language_name][$langFile][$key];
                                            }
                                        }
                                        if(empty($val)) { // use Google Translate to find the translated string
                                            $google_translated = $this->file_translate($langFile, $language_name, $language_code);
                                            if(!empty($google_translated[$language_name][$langFile])) {
                                                if(array_key_exists($key, $google_translated[$language_name][$langFile])) {
                                                    $val = $google_translated[$language_name][$langFile][$key];
                                                }
                                            } else {
                                                $val = $this->translate_lib->setLang($origLangCode, $language_code)->translate($orig[$key]);
                                            }
                                        }                                        
                                    }
                                    if (!empty($val)) {
                                        $new[$key] = $val;
                                        $cnt ++;
                                    } else {
                                        // if proxy failure count is 10 remove it from the list
                                        // unset($proxies[$i]);
                                        $errcnt ++;
                                        $chkd[] = $key;
                                    }
                                    
                                    // Give up if 5 errors occur without any successful attempts,
                                    // as this may imply network or other issues.
                                    if ($errcnt >= 5 && $cnt == 0) {
                                        break;
                                    }
                                    // language string must be translated before save
                                    $processed = true;
                                }
                                $i++;
                            }
                            // Save language file
                            $language_name = strtolower($language_name);
                            if ($processed && save_lang_file($langFile, $language_name, $new)) {
                                // Save information in the database table to avoid further translation
                                $this->db->on_duplicate($table_name, array(
                                    'code' => $language_code,
                                    'name' => $language_name,
                                    'file' => $langFile,
                                    'translated_on' => date('Y-m-d H:i:s'),
                                    'translated_ip' => $this->input->ip_address()
                                ));
                            }
                        }
                    }
                }
            }
        }
    }
    
    /**
     * Get Free Proxy addresses online from spys.me
     * @return string[]
     */
    private function get_proxies() {
        $filename = FCPATH . 'proxy.txt';
        $download = curl_download('http://spys.me/proxy.txt', $filename);
        if($download && file_exists($filename)) {
            $lines = file($filename);
            $proxies = array();
            // get all variables
            foreach ($lines as $lineNumber => $line) {
                if (strpos($line, ' + ') !== false) {
                    $line_txt = str_replace(' + ', '', $line);
                    $explode_line = explode(' ', $line_txt ?? '');
                    if(isset($explode_line[0])) {
                        $proxies[] = trim($explode_line[0]);
                    }
                }
            }
            return $proxies;
        }
    }
    
    /**
     * Read language file from the public_html/languages folder
     * @param string $lang - Name of the language e.g. english, french
     * @param string $fileName - Name of the file
     */
    private function read_lang_file($lang, $fileName, $core_files, $custom_files) {
        $cacheID = 'translate_read_' . $lang . '_' . $fileName;
        $cacheID = str_replace('-', '_', $this->slug->create_slug($cacheID));      
        if(!$lang_strings = $this->gc_translate_cache->load($cacheID)) {
            $path = FCPATH . 'languages/' . $lang . '/';
            if(in_array($fileName, $core_files)) {
                $folder = 'application/language/';
            } else if(in_array($fileName, $custom_files)) {
                $module = str_replace('_lang.php', '', $fileName);
                $folder = 'application/modules/' . $module . '/language/';
            } else {
                $module = str_replace('_lang.php', '', $fileName);
                $folder = 'goclixy/' . $module . '/language/';
            }
            $file_path = $path . $folder . $lang . '/' . $fileName;
            $strings = array();
            if(file_exists($file_path)) {
                $lines = file($file_path); // read file
                foreach($lines as $line) {
                    if (strpos($line, '$lang[') !== false) {
                        $key = trim($this->getStringBetween($line,'$lang[\'','\''));
                        $value = $this->getStringBetween($line,' = \'','\';');
                        if(!empty($key) && !empty($value) && $value != '* ') {
                            $strings[$key] = $value;
                        }
                    }
                }
            }
            // save it in a session variable to avoid unnecessary computation again and again
            $lang_strings = array(
                $lang => array(
                    $fileName => $strings
                )
            );
            $this->gc_translate_cache->save($lang_strings, $cacheID, array('translate_read'));
        }
        return $lang_strings;
    }
    
    /**
     * Extract text between two characters from a string
     * @param string $str
     * @param string $from
     * @param string $to
     * @param string $withFromAndTo
     * @return string
     */
    public function getStringBetween($str,$from,$to, $withFromAndTo = false) {
        $sub = substr($str, strpos($str,$from)+strlen($from),strlen($str));
        if ($withFromAndTo)
            return $from . substr($sub,0, strrpos($sub,$to)) . $to;
            else
                return substr($sub, 0, stripos($sub, $to));
    }    
    
    /** 
     * Google has limit on API request so send more data to Google and translate complete file by reading text in iterations
     * 
     * @param string $filename  - Name of the Language File
     * @param string $target    - Name of the language to be translated
     * @param string $target_code - Code of the language to be translated
     * @param string $source    - Name of the source language
     * @param string $source_code - Code of the source language
     * @return string[][][]
     */
    public function file_translate($filename = 'application_lang.php', $target = 'french', $target_code = 'fr', $source = 'english', $source_code = 'en') {
        $cacheID = 'translate_' . $filename . '_' . $target . '_' . $target_code . '_' . $source . '_' . $source_code;
        $cacheID = str_replace('-', '_', $this->slug->create_slug($cacheID));
        if(!$google_translated = $this->gc_translate_cache->load($cacheID)) {
            $this->load->helper('translate/languages');
            $this->load->library('translate/translate_lib');
            $lang_array = load_lang_file($filename, $source);
            $translated_text = '';
            $limit = 50; // number of lines to be translated at a time
            $count_array = count($lang_array); // number of elements in the array
            $max_iterations = $count_array / $limit; // max time this loop will execute
            $max_iterations = is_float($max_iterations) ? intval($max_iterations) + 1 : $max_iterations;
            for($i = 1;$i<=$max_iterations;$i++) {
                $j = 1;
                $text = '';
                foreach($lang_array as $key => $lang_str) {
                    if($j > $limit) {
                        break;
                    }
                    $text .= $key . 'MMM ' . $lang_str . ' NNN';
                    unset($lang_array[$key]);
                    $j++;
                }
                // translate text
                $translated_text .= $this->translate_lib->setLang($source_code, $target_code)->translate($text);
            }
            $new_lang_strings = array();
            if(!empty($translated_text)) {
                // read text line by line and get the key
                $lines = explode(' NNN', $translated_text);
                foreach($lines as $line) {
                    $line = explode('MMM ', $line ?? '');
                    if(!empty($line[0]) && !empty($line[1])) {
                        $new_lang_strings[strtolower(trim($line[0]))] = trim($line[1]);
                    }
                }
            }
            // save it in session to avoid further lookup
            $google_translated = array(
                $target => array(
                    $filename => $new_lang_strings
                )
            );
            $this->gc_translate_cache->save($google_translated, $cacheID, array('translate'));
        }
        return $google_translated;
    }
    
    /**
     * Replace invalid strings in translated files e.g. multiple slashes(\\\), invalid html etc.
     */
    public function replace_invalid_strings($search_language = 'english') {
        // Strings to be found and replace
        $find_replace = array(
            '& quot;' => '"',
            '<Strong>' => '<strong>',
            '\\\'' => '\'',
            ' / ' => '',
            'https: //' => 'https://',
            'http: //' => 'http://',
            '<b> ' => '<b>',
            ' </b>' => '</b>',
            '<strong> ' => '<strong>',
            ' </strong>' => '</strong>',
            '<span> ' => '<span>',
            ' </​​span>' => '</​​span>',
            'class = ' => 'class='
        );
        $this->load->config('translate/langcodes');
        $this->load->helper('translate/languages');
        $this->load->library('translate/translate_lib');
        $languages = list_lang_files();
        $not_found = true;
        foreach($languages as $modules) {
            foreach($modules as $language) {
                $lang_array = load_lang_file($language, $search_language);
                // search in array and return the key
                if($lang_array) {
                    $lang_array = $this->iterate_find($find_replace, $lang_array);
                    if($lang_array) {
                        $not_found = false;
                        save_lang_file($language, $search_language, $lang_array);
                    }
                }
            }
        }
        if($not_found) {
            // No record found
            Template::set_message(lang('label_no_record_found'), 'error');
        }
    }
    
    /**
     * Iterate and find strings
     * @param string $find      - Keyword to be found
     * @param string $replace   - Keyword to be replaced
     * @param array $lang_array - Language Array
     * @return mixed|boolean
     */
    private function iterate_find($find_replace, $lang_array) {
        $strfind = false;
        foreach($lang_array as $key => $string) {
            if(substr($string, 0, 3) == ' %s') { // remove extra space %s from the string starts
                $strfind = true;
                $lang_array[$key] = str_replace(' %s', '%s', $lang_array[$key]);
            }
            foreach($find_replace as $find => $replace) {
                if( strpos($string, $find) !== false ) {
                    $strfind = true;
                    $lang_array[$key] = str_replace($find, $replace, $lang_array[$key]);
                }
            }
        }
        if($strfind) {
            return $lang_array;
        }
        return $strfind;
    }
    
    /**
     * Add Language String if not exist in the Language File
     * @param string $module
     * @param string $language
     */
    public function insert_lang_string($key, $value, $language = 'english', $files = array('custom_lang.php')) {
        $filenames = $files;
        $result = array();
        foreach($filenames as $filename) {
            $success = $this->find_insert_lang($filename, $key, $value);
            if(!$success) {
                // the key already exists, modify it via developer->translate
                $result[] = $key;
            }
        }
        return $result;
    }
    
    /**
     * Find Language String in the Language File, if not exists add it
     * @param string $filename
     * @param string $find - Name of the Module
     * @param string $language
     */
    private function find_insert_lang($filename, $find, $value, $language = 'english') {
        $line = array($find => $value);
        $this->load->helper('translate/languages');
        $lang_array = load_lang_file($filename, $language);
        // search in array and return the key
        if(!$lang_array) {
            $lang_array = array();
        }
        if(!isset($lang_array[$find])) {
            addLanguageLine($filename, $line, $language);
            return true;
        }
        return false;
    }
}