<?php defined('BASEPATH') || exit('No direct script access allowed');

/**
 * Financial controller
 */
class Financial extends Admin_Controller
{
    protected $permissionCreate = 'Leads.Financial.Create';
    protected $permissionDelete = 'Leads.Financial.Delete';
    protected $permissionEdit   = 'Leads.Financial.Edit';
    protected $permissionView   = 'Leads.Financial.View';
    protected $context = 'financial';
    protected $moduleName = 'leads';

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
        
        $this->auth->restrict($this->permissionView);
        $this->load->model('leads/leads_model');
        $this->lang->load('leads');
        //LOAD TinyMCE EDITOR
        Assets::add_js(array('tinymce/tinymce.min.js'));
        $this->form_validation->set_error_delimiters("<span class='error'>", "</span>");
        
        Template::set_block('sub_nav', 'financial/_sub_nav');
        //Assets::add_module_js('leads', 'leads.js');
        
    }
    
    /**
     * Leads Summary
     */
    public function summary() {
        // DATE TIME PICKER
        $this->lang->load('listings/listings');
        Assets::add_js('moment.js');
        Assets::add_css('bootstrap-datetimepicker.min.css' );
        Assets::add_js('bootstrap-datetimepicker.min.js' );
        Assets::add_css('chart.css');
        Assets::add_js('Chart.min.js');
        $this->session->unset_userdata('chart_query_params');
        $module = 'leads';
        $record = $this->leads_model->summary();
        $table = $module;
        $where = '';
        
        // tabular blocks
        $limit = 10;
        $tabular_blocks = array();
        /*$tabular_blocks['top_visited'] = $this->listings_model->_reports($module, 0, $limit, null, 'hits DESC', 'M');
        $tabular_blocks['recently_added'] = $this->listings_model->_reports($module, 0, $limit, null, 'created_on DESC', 'M');
        $tabular_blocks['recently_viewed'] = $this->listings_model->_reports($module, 0, $limit, null, 'last_viewed_on DESC', 'M');
        */
        $column = 'counter';
        $row_by_months = $this->gcrud->chart('months', array('status_id' => 2, 'where' => $where,'prefix' => 'M', 'table' => $table, 'field' => $column));
        Assets::add_js($this->load->view('common/_chart_js', array('field_id' => 'sixMonthChart', 'labels' => $row_by_months['labels'], 'data' => $row_by_months['data'], 'label' => lang('total_rows'), 'total' => $row_by_months['total'], 'with_dates' => false, 'table' => $table), true), 'inline');
        $result = $this->gcrud->chart('current_month', array('status_id' => 2, 'where' => $where, 'prefix' => 'M', 'table' => $table, 'field' => $column));
        Assets::add_js($this->load->view('common/_chart_js', array('field_id' => 'monthlyChart', 'labels' => $result['labels'], 'data' => $result['data'], 'total' => $result['total'], 'label' => lang('total_rows'), 'with_dates' => true, 'table' => $table), true), 'inline');
        // $record->tabular_blocks = $tabular_blocks;
        //$record->recently_viewed = $this->forums_model->manage(0, null, 'M.last_viewed DESC', 20, 0, array('module' => 'views'));
        Template::set(array(
            'module' => $module,
            'tabular_blocks' => $tabular_blocks,
            'toolbar_title' => lang($module) . ' - ' . lang('gc_menu_summary'),
            'record' => $record
        ));
        Template::render();
    }
    
    /**
     * Display a list of Subscriber Users.
     *
     * @return void
     */
    /*public function index($filter = 'all', $offset = 0) {
        $this->auth->restrict('Subscribers.Components.Summary');
        $record = $this->subscribers_model->summary();
        $tabular_blocks = array();
        $tabular_blocks['last_messages'] = $this->subscribers_model->manage(0, null, 'M.id DESC', 20, 0, array('module' => 'messages', 'prefix' => 'subscriber'));
        $tabular_blocks['recently_added'] = $this->subscribers_model->manage(0, null, 'M.id DESC', 20, 0, array('module' => 'users', 'prefix' => 'subscriber'));
        $tabular_blocks['coming_followups'] = $this->subscribers_model->manage(0, null, 'M.next_follow_up DESC', 20, 0, array('module' => 'followups', 'prefix' => 'subscriber'));
        $record->tabular_blocks = $tabular_blocks;
        Template::set(array(
            'toolbar_title' => lang('gc_menu_subscribers') . ' - ' . lang('gc_menu_summary'),
            'record' => $record
        ));
        Template::render();
    }*/
    
    public function index($filter = 'all', $offset = 0) {
        $this->manage('leads', $filter, $offset);
    }
    
    /**
     * Display a list of leads data
     * @param string $module
     * @param string $filter
     * @param number $offset
     */
    public function manage($module = 'leads', $filter = 'all', $offset = 0) {
        // Groups - Export Excel
        if (isset($_POST['export_excel'])) {
            $this->gcrud->stats(array('module' => 'leads'));
            $this->leads_model->export($_POST['export_excel']);
        }
        // Groups - Export PDF
        if (isset($_POST['export_pdf'])) {
            $this->gcrud->stats(array('module' => 'leads'));
            $this->leads_model->export($_POST['export_pdf'], 20000, 0, 'pdf');
        }
        // Set group_id filter
        if(isset($_POST['filter_group_id'])) {
            $this->gcrud->stats(array('module' => 'leads'));
            $rows_filter = array('group_ids' => $_POST['filter_group_id']);
            $this->session->set_userdata('leads_rows_filter', $rows_filter);
            $this->session->set_userdata('filter_group_id', $_POST['filter_group_id']);
            Template::redirect(site_url(SITE_AREA . '/' . $this->context . '/' . $this->moduleName . '/manage/leads'));
        }
        // Clear group_id filter
        if(isset($_POST['clear_group_filter'])) {
            $this->gcrud->stats(array('module' => 'leads'));
            $this->session->unset_userdata('filter_group_id');
        }
        $extra = array(
            'model' => 'leads',
            'context' => $this->context,
            'fields' => $module,
            'prefix' => 'lead',
            'table_prefix' => 'lead'
        );
        if($module == 'leads') {   
            // lead specific followups
            if(isset($_POST['followups'])) {
                $this->gcrud->stats(array('module' => 'leads'));
                $this->session->set_userdata('lead_followups_rows_filter', array('lead_id' => $_POST['followups']));
                Template::redirect(SITE_AREA . '/financial/leads/manage/followups');
            }
            // email and send sms variables
            $this->load->model('emailer/emailer_model');
            $available_variables =  $this->emailer_model->share_variables('leads', false, ', ');
            Template::set(array(
                'subject_variables' => $available_variables['subject'],
                'message_variables' => $available_variables['message']
            ));
            $extra['prefix'] = '';
            $extra['table_prefix'] = 'lead';
            $extra['fields'] = null;
            $extra['dynamic_fields'] = true;
        } else if($module == 'followups') {
            $extra['advanced_filter'] = array('lead_id', 'date_from', 'date_to');
            $extra['advanced_filter_date'] = true;
            // lead specific followups
            if($this->session->userdata('lead_followups_rows_filter')) {
                $lead_followups_rows_filter = (array) $this->session->userdata('lead_followups_rows_filter');
                if(!empty($lead_followups_rows_filter['lead_id'])) {
                    $extra['query_where'] = "M.lead_id = " . $lead_followups_rows_filter['lead_id'];
                }
            }
        }
        if($this->auth->role_id() != 1 && $module != 'templates') { // staff and other users can view only their own submit links
            $extra['query_where'] = (!empty($extra['query_where']) ? $extra['query_where'] . " AND " : '') . "M.created_by = " . $this->auth->user_id();
        }
        $this->gc->module_view($module, $filter, $offset, 'admin', $extra);
    }
    
    /**
     * Add/Edit Leads Data
     *
     * @return void
     */
    public function form($module = 'leads', $action = 'create') {
        $params = array(
            'model' => 'leads',
            'context' => $this->context,
            'fields_method' => $module,
            'set_table' => true,
            'uri_segment' => 7,
            'redirect_url' => site_url(SITE_AREA . '/' . $this->context . '/' . $this->moduleName . '/manage/' . $module)
        );
        
        // set the lead id automatically
        $lead_followups_rows_filter = (array) $this->session->userdata('lead_followups_rows_filter');
        if($module == 'followups' && $action == 'create' && !empty($lead_followups_rows_filter['lead_id'])) {
            $record = array(
                'lead_id' => $lead_followups_rows_filter['lead_id']
            );
            $params['record'] = (object) $record;
        }
        
        if($module == 'leads') { // && $this->session->userdata('filter_group_id')
            $params['dynamic_fields'] = true;
            // $params['query_method'] = 'get_subscriber';
            // $params['method'] = 'after_subscriber_save';
            // save link relations
            // $params['fields'] = $this->subscribers_model->group_fields($this->session->userdata('filter_group_id'));
        }
        /*if($module == 'templates') {
         Template::set('extra_action', $this->load->view('common/_template_fields', array('template_module' => 'subscriber_users'), true));
         if($action == 'edit') {
         Template::set('extra_actions', array(
         '<a class="template-preview btn btn-info" href="javascript:void(0)" data-href="' . site_url(SITE_AREA . '/components/subscribers/preview/' . $this->uri->segment(7)) . '"><i class="fas fa-eye fa-fw mr-1"></i>' . lang('bf_action_preview') . '</a>'
         ));
         }
         }*/
        $this->gcrud->form($module == 'leads' ? $module : 'lead_' . $module, $action, 'admin', $params);
    }
    
    /**
     * Assign fields to a Group
     * @param number $id
     * @param string $module
     */
    public function assign($id = 0, $module = 'leads') {
        $cancel_url = site_url(SITE_AREA . '/financial/leads/manage/groups');
        // Allow admin to change row
        if (isset($_POST['change']) && !empty($_POST['group_id'])) {
            Template::redirect(site_url(SITE_AREA . '/financial/leads/assign/' . $_POST['group_id']));
        }
        // module search js and css dependencies
        Assets::add_css('selectize.bootstrap3.css');
        Assets::add_js('selectize.min.js');
        Assets::add_js('selectize-search.js');
        Assets::add_module_css('leads', 'leads.css');
        // Display fields using datatables
        Assets::add_js( array(
            'jquery.dataTables.min',
            'dataTables.bootstrap.min',
        ));
        Assets::add_css('dataTables.bootstrap.min');
        Assets::add_js( "$('#fields_table').DataTable({
		   'aoColumnDefs': [{
		        'bSortable': false,
		        'aTargets': [0] /* Disable sorting on the first column, 1st one, start by the left */
		    }],
		    order: [[5, 'asc']], /* Set default sort */
		    pageLength: 50 /* Limit number of rows */
		});", 'inline' );
        $this->lang->load('fields/fields');
        $module = 'leads';
        $module_id = gc_module_id($module);
        $fields = (array) $this->fields_model->get_fields($module);
        $group = $this->db->where('id', $id)->get('lead_groups')->row();
        if(!$group) {
            Template::redirect($cancel_url);
        }
        
        // Saved Fields
        $records = array();
        $existing_fields = array(); // fields to be deleted in case of unchecked
        $saved_result = $this->db->get_where('lead_group_fields', array('group_id' => $id))->result();
        if($saved_result) {
            foreach($saved_result as $row) {
                $records[$row->field_id] = $row;
                $existing_fields[$row->field_id] = $row->id;
            }
        }
        // Save Fields
        if (isset($_POST['save'])) {
            $this->gcrud->stats(array('module' => 'leads'));
            clear_cache('lead_group_fields'); // clear cache on save
            $checked = $this->input->post('checked');
            if(is_array($checked) && count($checked)) { // Allow user to unassign all fields
                $result = FALSE;
                $data = array();
                foreach ($checked as $pid) {
                    $pid = (int) $pid;
                    $data[] = array(
                        'group_id' => $id,
                        'field_id' => $pid,
                        'priority' => (int) $this->input->post('priority_' . $pid),
                        'is_required' => (int) $this->input->post('required_' . $pid),
                        // 'status' => (int) $this->input->post('status_' . $pid)
                    );
                    // fields to be deleted
                    if(array_key_exists($pid, $existing_fields)) {
                        unset($existing_fields[$pid]);
                    }
                }
                $this->db->on_duplicate('lead_group_fields', $data);
                if(!empty($existing_fields)) {// delete unchecked fields
                    $this->db->where('id IN (' . implode(',', $existing_fields) . ')')->delete('lead_group_fields');
                }
                $message_label = (count($checked) == 1 ? lang('label_field') : lang('label_fields'));
                Template::set_message(count($checked) .' '. sprintf(lang('msg_save_success'), $message_label), 'success');
                Template::redirect(current_url());
            } else {
                if(!empty($existing_fields)) {// delete unchecked fields
                    $this->db->where('id IN (' . implode(',', $existing_fields) . ')')->delete('lead_group_fields');
                }
                Template::set_message(sprintf(lang('msg_action_success'), lang('label_fields'), lang('action_unassigned')), 'success');
                Template::redirect(current_url());
            }
        }
        Template::set(array(
            'module' => $module,
            'group' => $group,
            'fields' => $fields,
            'records' => $records,
            'cancel_url' => $cancel_url,
            'toolbar_title' => lang('field_assign') . ' - ' . (!empty($group->name_lang) ? lang($group->name) : $group->name)
        ));
        Template::render();
    }
    
    /**
     * Export Leads
     */
    public function export() {
        $this->auth->restrict('Leads.Financial.Export');
        Assets::add_js('moment.js');
        Assets::add_css('bootstrap-datetimepicker.min.css' );
        Assets::add_js('bootstrap-datetimepicker.min.js' );
        Assets::add_js("
    $('#date_from').datetimepicker({
        format: 'YYYY-MM-DD',
        widgetPositioning: {
            horizontal: 'right',
            vertical: 'bottom'
        }
    });
    $('#date_to').datetimepicker({
        format: 'YYYY-MM-DD',
        widgetPositioning: {
            horizontal: 'right',
            vertical: 'bottom'
        }
    });", 'inline');
        // Set Default
        if (isset($_POST['save'])) {
            $this->gcrud->stats(array('module' => 'leads'));
            $this->form_validation->set_rules('group_id', 'lang:singular_lead_groups', 'trim');
            $this->form_validation->set_rules('limit', 'lang:label_limit', 'trim|numeric|required');
            $this->form_validation->set_rules('offset', 'lang:label_offset', 'trim|numeric|required');
            $this->form_validation->set_rules('format', 'lang:label_format', 'trim');
            if ($this->form_validation->run() === true) {
                $group_id = (int) $this->input->post('group_id');
                $limit = (int) $this->input->post('limit');
                $offset = (int) $this->input->post('offset');
                $format = $this->input->post('format');
                $where = null;
                if($this->input->post('date_from') && $this->input->post('date_to')) {
                    $where = "(DATE(M.created_on) BETWEEN '" . $this->input->post('date_from') . "' AND '" . $this->input->post('date_from') . "')";
                } else if($this->input->post('date_from')) {
                    $where = "DATE(M.created_on) >= '" . $this->input->post('date_from') . "'";
                } else if($this->input->post('date_to')) {
                    $where = "DATE(M.created_on) <= '" . $this->input->post('date_to') . "'";
                }
                $this->leads_model->export($group_id, $limit, $offset, $format, 'M.id DESC', $where);
            }
        }
        Template::set(array(
            'toolbar_title' => lang('label_export')
        ));
        Template::render();
    }
    
    /**
     * Unique Subscriber Template
     * @return boolean
     */
    public function unique_template() {
        $id = (int) $this->input->post('id');
        $group_id = (int) $this->input->post('group_id');
        $action_type = (int) $this->input->post('action_type');
        $where = array('group_id' => $group_id, 'action_type' => $action_type);
        if($id != 0) {
            $where['id != '] = $id;
        }
        $count = $this->db->where($where)->count_all_results('lead_email_templates');
        if($count > 0) {
            $this->form_validation->set_message('unique_template', lang('form_validation_unique_template'));
            return false;
        } else {
            return true;
        }
    }

    /**
     * Display a list of Leads data.
     *
     * @return void
     */
    public function index__old($filter='all', $offset = 0) {
    	Assets::add_js('admin-reply-email.js');
    	// Do we have any actions?
    	if (isset($_POST['submit'])) {
    	    $this->gcrud->stats(array('module' => 'leads'));
    		$action = $this->input->post('submit');
    		$checked = $this->input->post('checked');
    		if($checked) {
    			switch(strtolower($action)) {
    				case 'activate':
    					foreach($checked as $lead_id) {
    						$this->leads_model->update_forward_lead($lead_id, 1);
    					}
    					Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), lang('leads'), lang('label_updated')), 'success');
    					break;
    						
    				case 'deactivate':
    					$this->gc->update_status('leads', $checked, 'status', 0);
    					Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), lang('leads'), lang('label_updated')), 'success');
    					break;
    				
    				case 'converted':
    					$this->gc->update_status('leads', $checked, 'status', 2);
    					Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), lang('leads'), lang('label_converted')), 'success');
    					break;
    						
    				case 'spam':
    					$this->gc->update_status('leads', $checked, 'status', 3);
    					Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), lang('leads'), lang('label_spammed')), 'success');
    					break;
    					
    				case 'forward':
    					$this->index_forward($checked);    					
    					break;
    					
    				case 'force_forward':
    					$this->index_forward($checked, TRUE);
    					break;
    						
    				case 'delete':
    					$this->gc->update_status('leads', $checked, 'deleted', 1);
    					Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), lang('leads'), lang('label_deleted')), 'success');
    					break;
    						
    				case 'purge':
    				    $this->load->model('fields/fields_model');
    					foreach($checked as $pid) {
    						// purge information from the lead emails table
    						$this->db->where('lead_id', $pid)->delete('lead_emails');
    						// purge meta information if any
    						$this->fields_model->purge_meta('leads', $pid);
    					}
    					$this->gc->purge_records('leads', $checked);
    					Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), lang('leads'), lang('label_deleted')), 'success');
    					break;
    						
    				case 'restore':
    					$this->gc->update_status('leads', $checked, 'deleted', 0);
    					Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), lang('leads'), lang('label_restored')), 'success');
    					break;
    			} 
    		} else {
    			Template::set_message(lang('msg_no_row_selected'), 'error');
    		}
    	}
    	// Change Status
    	if (isset($_POST['change_status']))	 {
    	    $this->gcrud->stats(array('module' => 'leads'));
    		if(!empty($_POST['status'])) {
    			$new_status = $_POST['status'] == 99 ? 0 : $_POST['status'];
    			$this->leads_model->update_forward_lead($_POST['change_status'], $new_status);
    			Template::set_message(sprintf(lang('msg_action_success'), lang('lead'), lang('label_updated')), 'success');
    		}
    	}
    	
    	// Actions done, now display the view
    	$where = "CN.deleted = 0";
    	
    	// advanced filter
    	$advanced_filter = array('date_from', 'date_to', 'location', 'category_id');
    	// location filter
    	Assets::add_css('selectize.bootstrap3.css');
    	Assets::add_js('selectize.min.js');
    	Assets::add_js('selectize-search.js');
    	Assets::add_js('selectize-location.js');
    	$extra = $this->gcrud->advanced_filter(array('advanced_filter' => $advanced_filter, 'advanced_filter_date' => true), 'leads', 'CN');
    	if(!empty($extra['query_where'])) {
    	    $where .= " AND " . $extra['query_where'];
    	}
    	
    	// Filters
    	$filter_type = $filter;
    	
    	switch($filter_type) {
    		case 'active':
    			$where .= " AND CN.status = 1";
    			break;
    				
    		case 'inactive':
    			$where .= " AND CN.status = 0";
    			break;
    		
    		case 'converted':
    			$where .= " AND CN.status = 2";
    			break;
    	
    		case 'spammed':
    			$where .= " AND CN.status = 3";
    			break;
    		
    		case 'forwarded_on':
    			$where .= " AND (CN.forwarded_on IS NOT NULL AND CN.forwarded_on != '')";
    			break;
    				
    		case 'replied_on':
    			$where .= " AND (CN.replied_on IS NOT NULL AND CN.replied_on != '')";
    			break;
    	   
    		case 'today':
    		    $where .= " AND (YEAR(CN.created_on) = YEAR(NOW()) AND MONTH(CN.created_on) = MONTH(NOW()) AND DAY(CN.created_on) = DAY(NOW()))";

    		case 'yesterday':
    		    $where .= " AND (CN.created_on <= NOW() AND CN.created_on = DATE_ADD(Now(),interval - 1 day))";
    		    break;
    		
    		case 'deleted':
    			$where = "CN.deleted = 1";
    			break;
    	
    		case 'all':
    			// Nothing to do
    			break;
    	
    		default:
    			show_404(SITE_AREA . "/financial/leads/index/$filter/");
    	}
    	
    	$searchterm = $this->gc->search('leads', $this->uri->segment(5), '/financial/leads');
    	if(!empty($searchterm)) {
    		$where .= " AND (CN.id LIKE '%". $searchterm ."%' OR CN.name LIKE '%". $searchterm ."%' OR CN.email LIKE '%". $searchterm ."%' OR CN.created_on LIKE '%". $searchterm ."%')";
    	}
    	
    	$sort_by = "CN.id DESC";
    	if(isset($_POST['sort']) && $_POST['sort'] == 'CT.name') {
    		$order_by = $this->gc->sort('leads');
    	} else if(isset($_POST['sort']) && $_POST['sort'] == 'C.name') {
    		$order_by = $this->gc->sort('leads');
    	} else {
    		$order_by = $this->gc->sort('leads', 'CN');
    	}
    	if(!empty($order_by)) {
    		$sort_by = $order_by;
    	}
    	
    	$records = $this->leads_model->get_leads($offset, $this->limit, $searchterm, $where, $sort_by);   	
    	$total_records = $this->leads_model->get_leads($offset, $this->limit, $searchterm, $where, $sort_by, 1);

        $pagerUriSegment = 6;
        $pagerBaseUrl = site_url(SITE_AREA . '/financial/leads/index/' . $filter) . '/';
        
        $limit  = $this->limit;

        $this->load->library('pagination');        
        $pager['base_url']    = $pagerBaseUrl;
        $pager['total_rows']  = $total_records;
        $pager['per_page']    = $limit;
        $pager['uri_segment'] = $pagerUriSegment;

        $this->pagination->initialize($pager);
        Template::set(array(
            'filtered_results' => $total_records
        ));
        Template::set('index_url', site_url(SITE_AREA .'/financial/leads/index/'));
        Template::set('filter_type', $filter_type);

        Template::set('records', $records);
        
    	Template::set('toolbar_title', lang('leads_manage'));

        Template::render();
    }
    
    /**
     * Index Page Footer Forward and Force Forward
     * @param array $checked
     * @param string $force_forward
     */
    private function index_forward($checked, $force_forward = FALSE) {
    	$success = false;
    	foreach($checked as $lead_id) {
    		$success = $this->leads_model->forward_lead($lead_id, 'index', $force_forward);
    	}
    	if($success) {
    		Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), lang('leads'), lang('label_forwarded')), 'success');
    	} else {
    		Template::set_message(lang('leads_forward_failure'), 'error');
    	}
    }
    
    /**
     * Create a Leads object.
     *
     * @return void
     */
    public function create()
    {
        $this->auth->restrict($this->permissionCreate);
        
        if (isset($_POST['save'])) {
            $this->gcrud->stats(array('module' => 'leads'));
            if ($insert_id = $this->save_leads()) {
                log_activity($this->auth->user_id(), lang('act_create_record') . ': ' . $insert_id . ' : ' . $this->input->ip_address(), 'leads');
                Template::set_message(sprintf(lang('msg_save_success'), lang('lead')), 'success');

                redirect(SITE_AREA . '/financial/leads');
            }

            // Not validation error
            if ( ! empty($this->leads_model->error)) {
                Template::set_message(sprintf(lang('msg_create_failure'), lang('lead')) . $this->leads_model->error, 'danger');
            }
        }

        Template::set('toolbar_title', lang('action_create'));
        Template::set_view('leads_create');
        Template::render();
    }
    /**
     * Allows editing of Leads data.
     *
     * @return void
     */
    public function view() {    	
        $id = $this->uri->segment(5);
        if (empty($id)) {
            Template::set_message(lang('invalid_id'), 'error');
            redirect(SITE_AREA . '/financial/leads');
        }

        Assets::add_js('admin-reply-email.js');
        // user search js and css dependencies
        Assets::add_css('selectize.bootstrap3.css');
        Assets::add_js('selectize.min.js');
        Assets::add_js('selectize-search.js');        
        
        $record = $this->leads_model->get_leads_view($id);        
        
        // Display Additional Fields in the Form with values
        $this->load->model('fields/fields_model');
        $module = 'leads';
        $meta_form_fields = $this->fields_model->get_fields($module, 'frontend');
        Template::set('meta_form_fields', $meta_form_fields);
        $metas = $this->fields_model->find_meta_for($module, $id);
        if($metas) {
            foreach($metas as $meta_key => $meta_value) {
                $record->{$meta_key} = $meta_value;
            }
        }
        
        if (isset($_POST['save'])) {
            $this->gcrud->stats(array('module' => 'leads'));
            $this->auth->restrict($this->permissionEdit);

            if ($this->save_leads('update', $id)) {
                // Save updated meta information
                $meta_data = $this->fields_model->field_post_values($meta_form_fields, 0, 1, 'member');
                if($meta_data) {
                    $this->fields_model->save_meta_for($module, $id, $meta_data);
                }
                
                log_activity($this->auth->user_id(), lang('act_edit_record') . ': ' . $id . ' : ' . $this->input->ip_address(), 'leads');
                Template::set_message(sprintf(lang('msg_save_success'), lang('lead')), 'success');
                redirect(SITE_AREA . '/financial/leads');
            }

            // Not validation error
            if ( ! empty($this->leads_model->error)) {
                Template::set_message(sprintf(lang('msg_edit_failure'), lang('lead')) . $this->leads_model->error, 'danger');
            }
        }  elseif (isset($_POST['delete'])) {
            $this->gcrud->stats(array('module' => 'leads'));
            $this->auth->restrict($this->permissionDelete);

            if ($this->leads_model->delete($id)) {
                log_activity($this->auth->user_id(), lang('act_delete_record') . ': ' . $id . ' : ' . $this->input->ip_address(), 'leads');
                Template::set_message(sprintf(lang('msg_delete_success'), lang('lead')), 'success');

                redirect(SITE_AREA . '/financial/leads');
            }

            Template::set_message(sprintf(lang('msg_delete_failure'), lang('lead')) . $this->leads_model->error, 'danger');
        }
        
        $user_id = $this->input->post('assigned_to') ? $this->input->post('assigned_to') : (!empty($record->assigned_to) ? $record->assigned_to : 0);
        if((int) $user_id != 0) {
        	$selectize_value = $this->db->select('id,email AS name')->get_where('users', array('id' => $user_id))->row();
        	Template::set('selectize_value', $selectize_value);
        }
        Template::set('leads', $record);

        Template::set('toolbar_title', lang('label_view') . ' ' .$record->name . ' ' . lang('lead'));
        Template::set_view('leads_form');
        Template::render();
    }
    
    /**
     * Reply to the User via Email
     * @return string
     */
    public function reply() {    	
    	$id = $this->uri->segment(5);
    	if (empty($id)) {
    		Template::set_message(lang('invalid_id'), 'error');    	
    		redirect(SITE_AREA . '/financial/leads');
    	}
    	
    	$query = $this->leads_model->find($id);
    	
    	if (isset($_POST['save'])) {
    	    $this->gcrud->stats(array('module' => 'leads'));
	    	$this->form_validation->set_rules('reply_subject', 'lang:label_subject', 'required|xss_clean');
	    	$this->form_validation->set_rules('reply_message', 'lang:label_message', 'required|xss_clean');
	    
	    	if ($this->form_validation->run() === TRUE) {
	    		
	    		// Send Email
	    		$this->load->library('emailer/emailer');
	    		$data = array(
	    				'from'		=> settings_item('sender_email'),
	    				'to'		=> $query->email,
	    				'subject'	=> escape($this->input->post('reply_subject')),
	    				'message'	=> $this->input->post('reply_message') .'<hr><br />' .settings_item('site.title') .'<br />' .site_url()
	    		);
	    		$success = $this->emailer->send($data, FALSE);
	    		if($success) {
	    			$this->db->update($this->db->dbprefix .'leads', array('status' => 1), array('id' => $query_id));
	    			Template::set_message(lang('success_message'), 'success');
	    			redirect(SITE_AREA . '/financial/leads');
	    		} else {
	    			Template::set_message(lang('email_error'), 'error');
	    			redirect(SITE_AREA . '/financial/leads');
	    		}
	    	}
    	}
    	Template::set('record', $query);
    	Template::set('toolbar_title', lang('label_reply') .' '. $query->name);
    	Template::render();
    }
    
    /**
     * Leads Reply to the User
     * @return string
     */
    public function replyQuery() {    	
    	$this->form_validation->set_rules('query_id', 'lang:label_id', 'required|xss_clean');
    	$this->form_validation->set_rules('reply_subject', 'lang:label_subject', 'required|xss_clean');
    	$this->form_validation->set_rules('reply_message', 'lang:label_message', 'required|xss_clean');

    	if ($this->form_validation->run() === FALSE) {
    		$validation_errors = validation_errors();
    		if ($validation_errors) {
    			$validation_error = $validation_errors;
    		}
    		echo json_encode(array('result' => 0, 'message' => $validation_error, 'type' => 'error'));
    	} else {
    		$query_id = escape($this->input->post('query_id'));
    		// Retrieve user email address from database
    		$query = $this->leads_model->find($query_id);
    		
    		// Send Email
    		$this->load->library('emailer/emailer');
    		$data = array(
    				'from'		=> settings_item('sender_email'),
    				'to'		=> $query->email,
    				'subject'	=> escape($this->input->post('reply_subject')),
    				'message'	=> $this->input->post('reply_message')
    		);
    		$success = $this->emailer->send($data, FALSE);
    		if($success) {
    			$this->db->update($this->db->dbprefix .'leads', array('status' => 1), array('id' => $query_id));
    			echo json_encode(array('result' => 1, 'message' => lang('success_message'), 'type' => 'success'));
    		} else {
    			echo json_encode(array('result' => 0, 'message' => lang('email_error'), 'type' => 'error'));
    		}
    	}
    }
    
    public function owner_notifications($method = 'email') {
    	$this->load->library('emailer/emailer');
    	$listings = $this->leads_model->get_owners_details(0, 1, 1, 0);
    	foreach($listings as $listing) {
    		if(!empty($listing->email) && $method == 'email') { // Send Email
    			$data = array(
    					'to'		=> $query->email,
    					'subject'	=> escape($this->input->post('reply_subject')),
    					'message'	=> $this->input->post('reply_message')
    			);
    			$success = $this->emailer->send($data, true);
    		} else if(!empty($listing->phone_number) && $method == 'sms') { // Send SMS
    			
    		}
    	}   	
    }

    //--------------------------------------------------------------------------
    // !PRIVATE METHODS
    //--------------------------------------------------------------------------
    
    /**
     * Save Status.
     */
    private function save_leads_status($id) {
    	$data = array();
    	$data['status'] = $this->input->post('status');
    	return $this->leads_model->update($id, $data);
    }

    /**
     * Save the data.
     *
     * @param string $type Either 'insert' or 'update'.
     * @param int    $id   The ID of the record to update, ignored on inserts.
     *
     * @return boolean|integer An ID for successful inserts, true for successful
     * updates, else false.
     */
    private function save_leads($type = 'insert', $id = 0)
    {
        if ($type == 'update') {
            $_POST['id'] = $id;
        }

        // Validate the data
        $this->form_validation->set_rules($this->leads_model->get_validation_rules());
        if ($this->form_validation->run() === false) {
            return false;
        }

        // Make sure we only pass in the fields we want
        
        $data = $this->leads_model->prep_data($this->input->post());
		$data['assigned_to'] = $this->input->post('assigned_to') ? $this->input->post('assigned_to') : 0;
		$data['forwarded_on'] =  $data['forwarded_on'] ? $data['forwarded_on'] : NULL;
		$data['replied_on'] =  $data['replied_on'] ? $data['replied_on'] : NULL;
		
        // Additional handling for default values should be added below,
        // or in the model's prep_data() method
        

        $return = false;
        if ($type == 'insert') {
            $id = $this->leads_model->insert($data);

            if (is_numeric($id)) {
                $return = $id;
            }
        } elseif ($type == 'update') {
            $return = $this->leads_model->update($id, $data);
        }

        return $return;
    }
    
    /**
     * Forward Lead to Business Owners
     */
    public function forward() {
    	$id = $this->uri->segment(5);
    	if (empty($id)) {
    		Template::set_message(lang('invalid_id'), 'error');
    		redirect(SITE_AREA . '/financial/leads');
    	}
    	$lead = $this->leads_model->get_leads_view($id);
    	if(!$lead) {
    		redirect(SITE_AREA . '/financial/leads');
    	} 	

    	// Load selectize jquery files
    	Assets::add_css('selectize.bootstrap3.css');
    	Assets::add_js('selectize.min.js');
    	Assets::add_js('selectize-search.js');
    	
    	// delete forwarded requests
    	if(isset($_POST['delete'])) {
    	    $this->auth->restrict($this->permissionDelete);
    	    $checked = $this->input->post('checked');
    	    if (is_array($checked) && count($checked)) {
    	        $result = FALSE;
    	        foreach ($checked as $pid) {
    	            $result = $this->db->where('lead_id = ' . $id . ' AND module_id = 1 AND row_id = ' . $pid)->delete('lead_emails');
    	        }
    	        if ($result) {
    	            // set auto increment value to the last id
    	            // $alter_autoinc = "ALTER TABLE ". $this->db->dbprefix . "lead_emails AUTO_INCREMENT = 1"; // IMPORTANT
    	            // $this->db->simple_query($alter_autoinc);
    	            Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), lang('label_forwarded') . ' ' . lang((count($checked) == 1 ? 'lead' : 'leads')), lang('label_deleted')), 'success');
    	        } else {
    	            Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_forwarded') . ' ' . lang('lead')) . $this->db->error, 'danger');
    	        }
    	    }
    	}
    	
    	if (isset($_POST['forward'])) {
    	    $this->gcrud->stats(array('module' => 'leads'));
    		$this->form_validation->set_rules('price', 'lang:label_price', 'trim|required|is_decimal');
    		$this->form_validation->set_rules('checked[]', 'lang:listings', 'trim|is_array');
    		if(!$this->input->post('checked') && $this->input->post('businesses') == '') {
    			$this->form_validation->set_rules('businesses', 'lang:listings', 'trim|required');
    		}
    		if ($this->form_validation->run() === TRUE) {
    			// update the price as set by the user
    			// change the status to forwarded
    			$this->db->where('id', $id)->update('leads', array('price' => $this->input->post('price'), 'forwarded_on' => date('Y-m-d H:i:s')));
    			
    			$ids = array();
    			// search field listings
    			if($this->input->post('businesses') != '') {
    				$ids = explode(',', $this->input->post('businesses'));
    			}
    			// listings checked by the admin
    			// merge with checked listings
    			if($this->input->post('checked')) {
    				$checked = $this->input->post('checked');
    				foreach($checked as $pid) {
    					$ids[] = $pid;	
    				}    				
    			}    			
    			$ids = array_unique($ids); // unique ids only
    			$queue_override = $this->input->post('queue_override') ? 1 : 0;
    			$this->leads_model->send_lead($id, $ids, $queue_override);
    			// $listings = $this->leads_model->search_listing($ids);
    			// send leads and save information in the lead_emails table
    			// $this->send_lead($listings);
    			// redirect user with message
    		}    		
    	}
    	
    	// selectize post handling
    	if(!empty($_POST['businesses'])) {
    	    $this->gcrud->stats(array('module' => 'leads'));
    		$ids = explode(',', $this->input->post('businesses'));
    		$ids = array_unique($ids); // unique ids only
    		$rows = $this->leads_model->search_listing($ids);
    		$selectize_value = array();
    		if($rows) {
    			foreach($rows as $row) {
    				$selectize_value[] = array(
    					'id' => $row->id,
    					'name' => $this->leads_model->title_format($row)
    				);
    			}
    		}
    		Template::set('selectize_value', $selectize_value);
    	}
    	$result = $this->leads_model->load_listings($id, 'forward', $lead);
    	Template::set(array(
    		'lead' => $lead,
    		'listings' => $result['listings']
    	));
    	
    	// Template::set('toolbar_title', lang('gc_action_forward') . ':: ' . $lead->name . (!empty($lead->category) ?  ' / ' . lang('label_category') . ': ' . $lead->category : '' ) . (!empty($lead->city) ? ' / ' . lang('label_city') .': ' . $lead->city : ''));
    	Template::set('toolbar_title', lang('gc_action_forward') . '::' . $id);
    	Template::render();
    }
    
    /**
     * Save settings
     */
    public function settings() {
		clear_cache('site_settings'); // Clear Site Settings Cache
    	$this->load->model('settings/settings_model');
    	if (isset($_POST['save'])) {
    	    $this->gcrud->stats(array('module' => 'leads'));
    		$this->form_validation->set_rules('matches[]', 'lang:label_matches', 'trim|required|is_array');
    		$this->form_validation->set_rules('price', 'lang:label_price', 'trim|required|is_decimal');
    		$this->form_validation->set_rules('limit', 'lang:label_limit', 'trim|required|min_val[1]|is_numeric');
    		$this->form_validation->set_rules('auto_approve', 'lang:auto_approve', 'trim');
    		$this->form_validation->set_rules('auto_forward', 'lang:auto_forward', 'trim');
    		if($this->form_validation->run() == TRUE) {
    			$settings = array(
    					'matches' => $this->input->post('matches'),
    					'price' => $this->input->post('price'),
    					'limit' => $this->input->post('limit'),
    					// 'auto_approve' => ($this->input->post('auto_approve') ? 1 : 0),
    					'auto_forward' => ($this->input->post('auto_forward') ? 1 : 0)
    			);
    			$data = array (array('name' => 'lead.settings', 'value' => json_encode($settings)));
    			$updated = $this->settings_model->update_batch($data, 'name');
    			if ($updated) {
    				Template::set_message(sprintf(lang('msg_save_success'), lang('gc_menu_settings')), 'success');
    				Template::redirect(site_url(SITE_AREA . '/financial/leads/settings'));
    			} else {
    				Template::set_message(sprintf(lang('msg_edit_failure'), lang('gc_menu_settings')), 'error');
    			}
    		}
    	}
    	$result = $this->db->select('value')->get_where('settings', array('name' => 'lead.settings'))->row();
    	if($result) {
    	    $settings = json_decode($result->value ?? '');
    	} else {
    	   $settings = json_decode(settings_item('lead.settings') ?? '');
    	}
    	Template::set('settings', $settings);
    	Template::set('toolbar_title', lang('gc_menu_settings'));
    	Template::render();
    }
    
    /**
     * Listings Search
     * @param string $q
     */
    public function search($q) {
    	header('Content-Type: application/x-json; charset=utf-8');
    	$q = escape(urldecode($q));
    	$result = $this->leads_model->search_listing(array(), $q, 50);
    	echo json_encode($result);
    }
    
    /**
     * Displays a list of allotments to the Admin
     *
     * @return void
     */
    public function allotments($lead_id, $status = 1, $offset=0) {
        if (isset($_POST['delete'])) {
            $checked = $this->input->post('checked');
            if (is_array($checked) && count($checked)) {
                $result = FALSE;
                foreach ($checked as $pid) {
                    $result = $this->db->where('id', $pid)->delete('lead_allotments');
                }
                if ($result) {
                    // set auto increment value to the last id
                    $alter_autoinc = "ALTER TABLE ". $this->db->dbprefix . "lead_allotments AUTO_INCREMENT = 1"; // IMPORTANT
                    $this->db->simple_query($alter_autoinc);
                    Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), lang((count($checked) == 1 ? 'singular_lead_allotments' : 'lead_allotments')), lang('label_deleted')), 'success');
                } else {
                    Template::set_message(sprintf(lang('msg_delete_failure'), lang('lead_allotments')) . $this->db->error, 'danger');
                }
            }
        }
        if(isset($_POST['assign_lead'])) {
            $this->form_validation->set_rules('assign_user_id', lang('bf_user'), 'trim|required|integer');
            if($this->form_validation->run() == TRUE) {
                $assign_user_id = (int) $this->input->post('assign_user_id');
                // check the user and lead existence
                $lead_allotted = $this->db->where('user_id = ' . $assign_user_id . ' AND lead_id = ' . $lead_id)->get('lead_allotments')->row();
                if(!$lead_allotted) {
                    $allotments_data = array(
                        'user_id' => $assign_user_id,
                        'lead_id' => $lead_id,
                        'status' => 1,
                        'created_ip' => $this->input->ip_address(),
                        'created_on' => date('Y-m-d H:i:s')
                    );
                    $this->db->insert('lead_allotments', $allotments_data);
					
					// get user information
					$user_info = $this->db->select('*, id AS user_id')->where('id', $assign_user_id)->get('users')->row();
					$this->leads_model->template($lead_id, 0, 'email', 0, array(), $user_info);
                    Template::set_message(sprintf(lang('msg_save_success'), lang('singular_lead_allotments')), 'success');
                } else {
                    Template::set_message(lang('msg_error'), 'danger');
                }
            }
        }
        $record = $this->db->where('id', $lead_id)->get('leads')->row();
        $redirect_url = $this->session->userdata('leads_last_visited_page') ? $this->session->userdata('leads_last_visited_page') : site_url(SITE_AREA . '/financials/leads');
        if(!$record) {
            Template::redirect($redirect_url);
        }
        $where = " M.lead_id = " . (int) $lead_id;
        if($status != 2) {
            $where .= " AND M.status = " . (int) $status;
        }
        
        $searchterm = $this->gc->search('leads', $this->uri->segment(6), '/financial/leads');
        if(!empty($searchterm)) {
            $where .= " AND (M.order_id LIKE '%". $searchterm ."%' OR M.user_id LIKE '%". $searchterm ."%')";
        }
        
        $sort_by = "M.id DESC";
        $order_by = $this->gc->sort('lead_allotments', 'O');
        if(!empty($order_by)) {
            $sort_by = $order_by;
        }
        
        // advanced filter
        $advanced_filter = array('date_from', 'date_to', 'user_id');
        
        // location filter
        Assets::add_css('selectize.bootstrap3.css');
        Assets::add_js('selectize.min.js');
        Assets::add_js('selectize-search.js');
        Assets::add_js('selectize-location.js');
        $this->load->library('gcrud');
        $extra = $this->gcrud->advanced_filter(array('advanced_filter' => $advanced_filter, 'advanced_filter_date' => true), 'lead_alltoments', 'M');
        if(!empty($extra['query_where'])) {
            $where .= " AND " . $extra['query_where'];
        }
        $records = $this->leads_model->get_allotments(0, $where, $sort_by, $offset, $this->limit);
        $total_queries = $this->leads_model->get_allotments(1, $where, 'M.id DESC', $offset, $this->limit);
        
        // Pagination
        $this->load->library('pagination');
        $pager['base_url'] = site_url(SITE_AREA ."/financial/leads/allotments/" . $lead_id . "/" . $status);
        $pager['total_rows'] = $total_queries;
        $pager['per_page'] = $this->limit;
        $pager['uri_segment']   = 7;
        
        $this->pagination->initialize($pager);
        Template::set(array(
            'records' => $records,
            'redirect_url' => $redirect_url,
            'toolbar_title' => lang('lead_allotments') . ':: ' . $lead_id . ' - ' . $record->name . ' (' . $record->email . ')'
        ));
        Template::render();
    }
}