<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/

class Leads extends Front_Controller {
	//--------------------------------------------------------------------
	public function __construct() {
		parent::__construct();
		$this->load->model('categories/categories_model');
		$this->load->model('listings/listings_model');
		$this->load->model('users/user_model');
		$this->load->model('leads/leads_model');
        $this->lang->load('members/members');
		$this->lang->load('leads');
	}
	
	/**
	 * Display Requirements Form to the User
	 */
	public function index() {
	    $group_id = !empty($_GET['id']) ? $_GET['id'] : 0;
	    if(empty($group_id)) {
	        // get default group
	        $group = $this->db->where(array('is_default' => 1, 'status' => 1))->get('lead_groups')->row();	        
	    } else {
	        $group = $this->db->where('id', $group_id)->get('lead_groups')->row();
	    }
	    if(!$group) {
	        // not a valid group, redirect the user to the home page
	        Template::redirect(site_url());
	    }
	    $group_id = $group->id;
	    Assets::add_module_js('leads', 'leads.js');
	    $record = array();
	    $module = 'leads';
	    $form_fields = $this->leads_model->group_fields($group_id);
	    $form_fields = $this->fields_model->load_fields($module, array(), 'frontend', $record, 0, false, $form_fields);
	    $record = $this->session->userdata('user_info');
	    Template::set(array(
	        'record' => $record
	    ));
	    $user_info = $this->session->userdata('user_info');
	    if(!empty($user_info)) {
	        if(settings_item('lst.google_places_api') == 1) {
	            $this->gc->location_google_places($user_info['city_id'], true);
	        } else {
	            $this->gc->ontype_city_selection($user_info['city_id'], true); // city selection
	        }
	    }
		Template::set(array(
		    'toolbar_title' => lang('post_your_requirements'),
			'page_title' => lang('post_your_requirements'),
			'page_description' => sprintf(lang('cn_form_help'),''),
			'title' => lang('post_your_requirements'),
		    'user' => $this->session->userdata('user_info'),
		    'group_id' => $group_id
		));
		$layout = $this->gc->load_assets('leads/form');
		Template::set_view($layout['view']);
		Template::render($layout['layout']);
	}
	
	/**
	 * Members View Leads Page
	 * @param string $filter
	 * @param number $offset
	 */
	public function view($filter='forwarded', $offset = 0) {
		$user_id = $this->auth->user_id();
		$where = "CN.deleted = 0";

		Assets::add_css('selectize.bootstrap3.css');
		Assets::add_js('selectize.min.js');
		Assets::add_js('selectize-search.js');
		Assets::add_js('moment.js');
		Assets::add_css ( 'bootstrap-datetimepicker.min.css' );
		Assets::add_js ( 'bootstrap-datetimepicker.min.js' );
		Assets::add_js("
            $('#date_from').datetimepicker({format: 'YYYY-MM-DD'});
            $('#date_to').datetimepicker({format: 'YYYY-MM-DD'})
        ", 'inline');
		
		if (isset($_POST['checkout'])) {
			// redirect user to the checkout page
			$this->load->model('orders/orders_model');
			$this->orders_model->cart($_POST['checkout'], 0, 1, 1, 'leads', false); // the user can buy multiple leads at once
			Template::redirect(gc_url_format('members/checkout'));
		}
		// Filters
		$filter_type = $filter;
			
		switch($filter_type) {
			case 'forwarded':
				// $where .= " AND (LE.user_id = " . $user_id ." OR CN.assigned_to = " . $user_id . ")"; // Specific to this user
				// $where .= " AND (CN.status_id = 2 OR CN.status_id = 3)"; // Paid and
				$where .= " AND LA.user_id = " . $user_id . " AND LA.status = 1";
				$where .= " AND CN.status_id = 2"; // show only active leads
				break;
		
			case 'search':
				$where .= " AND CN.status_id = 2";
				break;
				
			default:
				Template::redirect(gc_url_format('members'));
		}
		
		// Allow user to search based on Category and City ID
		$sess_name = 'leads_view';
		$redirect_condition = 4;
		if ($filter == 'search' && (isset($_POST['submit']) || $this->session->userdata($sess_name))) {
			if (isset($_POST['submit'])) {
				$searchterm = array(
					'category_id' => (int) $this->input->post('category_id'),
					'city_id' => (int) $this->input->post('city_id'),
				    'date_from' => escape($this->input->post('date_from')),
				    'date_to' => escape($this->input->post('date_to'))
				);
				$this->session->set_userdata($sess_name, $searchterm);
				// IF USER PERFORMED SEARCH FROM PAGINATION PAGE
				if(empty($_POST['sort']) && !empty($redirect_condition)) {
					Template::redirect(gc_url_format('leads/view') .'/' . $filter);
				}
			} else if($this->session->userdata($sess_name)) { // FOR PAGINATION
				$searchterm = $this->session->userdata($sess_name);
			}
			if(!empty($searchterm['category_id'])) {
				$where .= " AND CN.category_id = " . $searchterm['category_id'];
				$this->load->model('categories/categories_model');
				$category = $this->categories_model->select('id, name')->find($searchterm['category_id']);
				Template::set('inputCategory', $category);
			}
			if(!empty($searchterm['city_id'])) {
				$where .= " AND CN.city_id = " . (int) $searchterm['city_id'];
				$this->load->model('locations/cities_model');
				$city = $this->cities_model->select('id, name')->find($searchterm['city_id']);
				Template::set('inputCity', $city);
			}
			if(!empty($searchterm['date_from']) && !empty($searchterm['date_to'])) {
			    $where .= " AND (DATE(CN.created_on) BETWEEN '" . $searchterm['date_from'] . "' AND '" . $searchterm['date_to'] . "')";
			} else if(!empty($searchterm['date_from'])) {
			    $where .= " AND DATE(CN.created_on) >= '" . $searchterm['date_from'] . "'";
			} else if(!empty($searchterm['date_to'])) {
			    $where .= " AND DATE(CN.created_on) <= '" . $searchterm['date_to'] . "'";
			}
		}
		
		$sort_by = "CN.created_on DESC";
		$searchterm = NULL;
		$records = $this->leads_model->get_leads($offset, $this->limit, $searchterm, $where, $sort_by, 0, $filter);	
		$total_records = $this->leads_model->get_leads($offset, $this->limit, $searchterm, $where, $sort_by, 1, $filter);

		$pagerUriSegment = 4;
		$pagerBaseUrl = gc_url_format('leads/view') .'/' . $filter;
		
		$this->load->library('pagination');
		$pager['base_url']    = $pagerBaseUrl;
		$pager['total_rows']  = $total_records;
		$pager['per_page']    = $this->limit;
		$pager['uri_segment'] = $pagerUriSegment;
		
		$this->pagination->initialize($pager);
		
		Template::set('records', $records);

		Template::set('index_url', gc_url_format('leads/view') .'/');
		Template::set('filter_type', $filter_type);

		Template::set('toolbar_title', lang('view_leads'));
		Template::set('page_title', lang('view_leads'));
		Template::set('title', lang('view_leads'));
		// Lead View has no View in the Themes Module, i.e. load layout files manually
		$this->gc->demo_custom_css_js('member');
		Template::render('member');
	}
}