<?php defined('BASEPATH') || exit('No direct script access allowed');

class Migration_Install_leads extends Migration
{
	/**
	 * @var string The name of the database table
	 */
	private $table_name = 'leads';
	private $table_lead_emails = 'lead_emails';

	/**
	 * @var array The table's fields
	 */
	private $fields = array(
		'id' => array(
			'type'       => 'INT',
			'constraint' => 11,
			'auto_increment' => true,
		),
		'user_id' => array(
			'type'       => 'BIGINT',
			'constraint' => 20,
			'null'       => false,
			'default'	 => 0
		),
		'assigned_to' => array(
			'type'       => 'BIGINT',
			'constraint' => 20,
			'null'       => false,
			'default'	 => 0
		),
		'category_id' => array(
			'type'       => 'INT',
			'constraint' => 11,
			'null'       => false,
			'default'	 => 0
		),
		'tag_id' => array(
			'type'       => 'BIGINT',
			'constraint' => 20,
			'null'       => false,
			'default'	 => 0
		),
		'city_id' => array(
			'type'       => 'INT',
			'constraint' => 11,
			'null'       => false,
			'default'	 => 0
		),
        'tag' => array(
            'type'       => 'VARCHAR',
            'constraint' => 255,
            'null'       => false,
        ),
        'name' => array(
            'type'       => 'VARCHAR',
            'constraint' => 50,
            'null'       => false,
        ),
        'phone_number' => array(
            'type'       => 'VARCHAR',
            'constraint' => 20,
            'null'       => true,
        ),
        'email' => array(
            'type'       => 'VARCHAR',
            'constraint' => 100,
            'null'       => false,
        ),
        'description' => array(
            'type'       => 'TEXT',
            'null'       => false,
        ),
        'price' => array(
            'type'       => 'FLOAT',
            'null'       => false,
            'default'	 => 0,
        ),
       	'status' => array(
            'type'       => 'TINYINT',
            'constraint' => 2,
            'default'    => 1
        ),
		'hits' => array( // hits/view counter
			'type'       => 'INT',
			'constraint' => 11,
			'null'       => false,
			'default'	 => 0
		),
        'forwarded_on' => array( // save admin last forwarded date
            'type'       => 'datetime',
			'null'		 => TRUE
        ),
        'replied_on' => array( // save admin last replied date
            'type'       => 'datetime',
			'null'		 => TRUE
        ),
        'ip' => array(
        	'type'       => 'VARCHAR',
        	'constraint' => 50,
        	'null'       => false
        ),
        'deleted' => array(
            'type'       => 'TINYINT',
            'constraint' => 1,
            'default'    => '0',
        ),
        'deleted_by' => array(
            'type'       => 'BIGINT',
            'constraint' => 20,
            'null'       => true,
        ),
		'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
        'created_by' => array(
            'type'       => 'BIGINT',
            'constraint' => 20,
            'null'       => false,
        ),
		'modified_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
        'modified_by' => array(
            'type'       => 'BIGINT',
            'constraint' => 20,
            'null'       => true,
        ),
	);
	
	/**
	 * @var array The table's fields
	 */
	private $fields_lead_emails = array(
			'id' => array(
				'type'       => 'INT',
				'constraint' => 11,
				'auto_increment' => true,
			),
			'user_id' => array(
				'type'       => 'BIGINT',
				'constraint' => 20,
				'null'       => false,
				'default'	 => 0
			),
			'row_id' => array(
				'type'       => 'BIGINT',
				'constraint' => 20,
				'null'       => false,
				'default'	 => 0
			),
			'module_id' => array(
				'type'       => 'TINYINT',
				'constraint' => 1,
				'null'       => false,
				'default'	 => 1
			),
			'lead_id' => array(
				'type'       => 'INT',
				'constraint' => 11,
				'null'       => false,
				'default'	 => 0
			),
			'email' => array(
				'type'       => 'VARCHAR',
				'constraint' => 100,
				'null'       => true,
			),
			'counts' => array(
				'type'       => 'INT',
				'constraint' => 11,
				'null'       => false,
				'default'	 => 1
			),
			'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
			'created_ip' => array(
				'type'       => 'VARCHAR',
				'constraint' => 50,
				'null'       => TRUE,
			),
			'modified_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
			'modified_ip' => array(
				'type'       => 'VARCHAR',
				'constraint' => 50,
				'null'       => TRUE,
			),
	);

	/**
	 * Install this version
	 *
	 * @return void
	 */
	public function up()
	{
		$this->dbforge->add_field($this->fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('user_id', false);
		$this->dbforge->add_key('tag_id', false);
		$this->dbforge->add_key('category_id', false);
		$this->dbforge->add_key('city_id', false);
		$this->dbforge->create_table($this->table_name);
		
		// LEAD EMAILS TABLE
		$this->dbforge->add_field($this->fields_lead_emails);
		$this->dbforge->add_key('id', true);
		$this->dbforge->create_table($this->table_lead_emails);
		$unique_email = "ALTER TABLE " . $this->db->dbprefix . $this->table_lead_emails . " ADD UNIQUE INDEX `unique_email` (`row_id`, `module_id`, `lead_id`)";
		$this->db->query($unique_email);
		
		// lead settings row
		$this->db->insert('settings', array('name' => 'lead.settings', 'module' => 'lead', 'value' => '{"matches":{"both":"1","category":"1"},"price":"50","limit":"1"}'));
	}

	/**
	 * Uninstall this version
	 *
	 * @return void
	 */
	public function down()
	{
		$this->dbforge->drop_table($this->table_name);
		$this->dbforge->drop_table($this->table_lead_emails);
		// remove settings row
		$settings = $this->db->select('name')->get_where('settings', array('name' => 'lead.settings'));
		if($settings) {
			$this->db->delete('settings', array('name' => 'lead.settings'));
		}
	}
}