<?php defined('BASEPATH') || exit('No direct script access allowed');

class Leads_model extends BF_Model
{
    protected $table_name	= 'leads';
    protected $table_groups	= 'lead_groups';
    protected $table_group_fields = 'lead_group_fields';
    protected $table_statuses	= 'lead_statuses';
    protected $table_broadcasts = 'lead_broadcasts';
	protected $key			= 'id';
	protected $date_format	= 'datetime';

	protected $log_user 	= true;
	protected $set_created	= true;
	protected $set_modified = true;
	protected $soft_deletes	= true;

	protected $created_field     = 'created_on';
    protected $created_by_field  = 'created_by';
	protected $modified_field    = 'modified_on';
    protected $modified_by_field = 'modified_by';
    protected $deleted_field     = 'deleted';
    protected $deleted_by_field  = 'deleted_by';

	// Customize the operations of the model without recreating the insert,
    // update, etc. methods by adding the method names to act as callbacks here.
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 	    = array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	// For performance reasons, you may require your model to NOT return the id
	// of the last inserted row as it is a bit of a slow method. This is
    // primarily helpful when running big loops over data.
	protected $return_insert_id = true;

	// The default type for returned row data.
	protected $return_type = 'object';

	// Items that are always removed from data prior to inserts or updates.
	protected $protected_attributes = array();

	// You may need to move certain rules (like required) into the
	// $insert_validation_rules array and out of the standard validation array.
	// That way it is only required during inserts, not updates which may only
	// be updating a portion of the data.
	protected $validation_rules 		= array(
		array(
			'field' => 'name',
			'label' => 'lang:label_name',
			'rules' => 'trim',
		),
		array(
			'field' => 'phone_number',
			'label' => 'lang:label_phone_number',
			'rules' => 'trim',
		),
		array(
			'field' => 'email',
			'label' => 'lang:label_email',
			'rules' => 'valid_email|max_length[100]',
		),
		array(
			'field' => 'description',
			'label' => 'lang:label_description',
			'rules' => 'required',
		),
		array(
			'field' => 'price',
			'label' => 'lang:label_price',
			'rules' => 'trim|required|is_decimal',
		),
	);
	protected $insert_validation_rules  = array();
	protected $skip_validation 			= false;
	protected $gc_leads_cache;

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
        $this->lang->load('leads/leads');
        // Load Zend Caching
        $this->load->library('slug');
        $this->load->library('gzf_cache', array('lifetime' => 900));
        if(method_exists($this->gzf_cache, 'get_instance')) {
            $this->gc_leads_cache = $this->gzf_cache->get_instance();
        }
    }
    
    /**
     * Group Fields
     *
     * @return array
     */
    public function groups() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'text',
                'name' => 'name',
                'label' => 'label_name',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'edit' => 1,
                'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
                'rules' => 'trim|required|max_length[150]'
            ),
            array(
                'type' => 'checkbox',
                'name' => 'name_lang',
                'label' => 'name_lang',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'message',
                'label' => 'lead_success_message',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'tooltip' => lang('lead_help_success_message'),
                'edit' => 1,
                'rules' => 'trim|required|max_length[450]'
            ),
            array(
                'type' => 'checkbox',
                'name' => 'message_lang',
                'label' => 'message_lang',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'iconpicker',
                'name' => 'icon',
                'label' => 'label_icon',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'class' => 3,
                'rules' => 'trim|max_length[255]',
            ),
            array(
                'type' => 'text',
                'name' => 'priority',
                'label' => 'label_priority',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'value' => 0,
                'class' => 2,
                'rules' => 'trim|required|is_numeric',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'is_default',
                'label' => 'label_default',
                'frontend' => false,
                'admin_only' => false,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'status',
                'label' => 'label_status',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            )
        );
        return $fields;
    }
    
    /**
     * Status Fields
     *
     * @return array
     */
    public function statuses() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'text',
                'name' => 'name',
                'label' => 'label_name',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'edit' => 1,
                'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
                'rules' => 'trim|required|max_length[150]'
            ),
            array(
                'type' => 'checkbox',
                'name' => 'name_lang',
                'label' => 'name_lang',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'colorpicker',
                'name' => 'color',
                'label' => 'label_color',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'class' => 2,
                'rules' => 'trim|max_length[255]',
            ),
            array(
                'type' => 'text',
                'name' => 'priority',
                'label' => 'label_priority',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'value' => 0,
                'class' => 2,
                'rules' => 'trim|required|is_numeric',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'is_default',
                'label' => 'label_default',
                'frontend' => false,
                'admin_only' => false,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'status',
                'label' => 'label_status',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            )
        );
        return $fields;
    }
    
    /**
     * Followups Form Fields
     *
     * @return array[]
     */
    public function followups() {
        $subscriber_id = $this->uri->segment(5) ? $this->uri->segment(5) : 0;
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => false,
                'admin_index' => true
            ),
            array(
                'type' => 'selectize',
                'name' => 'subscriber_id',
                'label' => 'singular_subscribers',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'load:subscriber_users;module:subscriber_users;max_items:1',
                'options_detailed' => 1,
                'edit' => 0,
                'rules' => 'required|trim|is_numeric|max_length[255]',
            ),
            array(
                'type' => 'textarea',
                'name' => 'remarks',
                'label' => 'subscriber_remarks',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'custom_rules' => 'editor',
                'field_class' => 'editor',
                // 'edit' => 1,
                // 'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
                'rules' => 'required|trim|max_length[255]',
            ),
            array(
                'type' => 'text',
                'name' => 'next_follow_up',
                'label' => 'subscriber_next_follow_up',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'class' => 3,
                'rules' => 'trim|valid_datetime'
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim'
            ),
            array(
                'type' => 'none',
                'name' => 'created_ip',
                'label' => 'created_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim'
            ),
            array(
                'type' => 'none',
                'name' => 'created_by',
                'label' => 'created_by',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => $this->auth->role_id() == 1 ? false : false,
                'rules' => 'trim'
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => false,
                'admin_only' => false,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim'
            ),
            array(
                'type' => 'none',
                'name' => 'modified_by',
                'label' => 'modified_by',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim'
            ),
            array(
                'type' => 'none',
                'name' => 'modified_ip',
                'label' => 'modified_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim'
            )
        );
        return $fields;
    }
    
    /**
     * Email Template Fields
     *
     * @return array
     */
    public function email_templates($role = 'admin') {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'dropdown',
                'name' => 'module_table',
                'label' => 'label_module',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:gc_menu_leads;value:leads',
                'options_detailed' => 1,
                'options_lang' => 1,
                'class' => 3,
                'rules' => 'trim|unique_email_template[lead]',
            ),
            array(
                'type' => 'dropdown',
                'name' => 'group_id',
                'label' => 'singular_lead_groups',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:lead_groups;value:database',
                'options_detailed' => 1,
                'class' => 3,
                'rules' => 'trim|is_numeric|required',
            ),
            array(
                'type' => 'dropdown',
                'name' => 'user_type',
                'label' => 'bf_user',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:placeholder_select;value:|name:us_admin;value:admin|name:bf_user;value:user',
                'options_detailed' => 1,
                'options_lang' => 1,
                'class' => 3,
                'rules' => 'trim',
            ),
            array(
                'type' => 'dropdown',
                'name' => 'action_type',
                'label' => 'label_action',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:placeholder_select;value:|name:add;value:add|name:edit;value:edit|name:delete;value:delete',
                'options_detailed' => 1,
                'options_lang' => 1,
                'class' => 3,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'name',
                'label' => 'label_name',
                'placeholder' => 'placeholder_name',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
                'rules' => 'required|trim|max_length[50]'
            ),
            array(
                'type' => 'text',
                'name' => 'subject',
                'label' => 'label_subject',
                'placeholder' => 'placeholder_subject',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'required|trim|max_length[255]'
            ),
            array(
                'type' => 'textarea',
                'name' => 'message',
                'label' => 'label_message',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'custom_rules' => 'editor',
                'field_class' => 'editor',
                'class' => 9,
                'rules' => 'trim|required',
            ),
            array(
                'type' => 'textarea',
                'name' => 'sms_text',
                'label' => 'sms_text',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'class' => 9,
                'field_extra' => 'rows:3',
                'rules' => 'trim',
            ),
            array(
                'type' => 'textarea',
                'name' => 'params',
                'label' => 'em_params',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'class' => 9,
                'tooltip' => lang('em_params_help'),
                'field_extra' => 'rows:3',
                'rules' => 'trim',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'status',
                'label' => 'label_status',
                'frontend' => false,
                'admin_only' => false,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            )
        );
        return $fields;
    }
    
    /**
     * Get listing views using date in descending order
     * @param int $offset
     * @param int $limit
     * @param string $searchterm
     * @param string $where
     * @param number $count
     * @return number
     */
    public function get_leads($offset, $limit, $searchterm, $where, $sort_by, $count = 0, $filter = 'search', $user_id = 0) {
        $user_id = !empty($user_id) ? $user_id : (int) $this->auth->user_id();
    	if($count == 0) {
    		$sql = "SELECT CN.*, C.name AS category, CT.name AS city, S.name AS status, S.name_lang AS status_lang, S.color AS status_color";
    		$sql .= ", (SELECT user_id FROM " . $this->db->dbprefix . "lead_allotments WHERE lead_id = CN.id AND user_id = $user_id AND status = 1) AS assigned_to";
    		$sql .= ", (SELECT COUNT(*) FROM "  . $this->db->dbprefix . $this->table_broadcasts . " WHERE lead_id = CN.id AND (status_id = 2 || status_id = 4)) AS available_with";
    	} else {
    		$sql = "SELECT COUNT(DISTINCT(CN.id)) AS TotalCount";
    	}
    	$sql .= " FROM " . $this->db->dbprefix . "leads CN";
    	//$sql .= " LEFT JOIN " . $this->db->dbprefix . "listings L ON L.id = CN.listing_id";
    	//$sql .= " LEFT JOIN " . $this->db->dbprefix . "users U ON U.id = CN.user_id";
		if($filter == 'forwarded') {
			$sql .= " LEFT JOIN " . $this->db->dbprefix . "lead_emails LE ON LE.lead_id = CN.id";
			$sql .= " LEFT JOIN " . $this->db->dbprefix . "lead_allotments LA ON LA.lead_id = CN.id";
		}
		if($count == 0) {
		    $sql .= " LEFT JOIN " . $this->db->dbprefix . "lead_statuses S ON S.id = CN.status_id";
	    	$sql .= " LEFT JOIN " . $this->db->dbprefix . "categories C ON C.id = CN.category_id";
	    	$sql .= " LEFT JOIN " . $this->db->dbprefix . "cities CT ON CT.id = CN.city_id";
    	}
    	$sql .= " WHERE " . $where ."";
    	if(!empty($searchterm)) {
    		$sql .= " AND (CN.name LIKE '%" . $searchterm . "%' OR CN.email LIKE '%" . $searchterm . "%' OR CN.created_on LIKE '%" . $searchterm . "%')";
    	}    	
    	if($count == 0) {
    		$sql .= " GROUP BY CN.id";
    		$sql .= " ORDER BY " . $sort_by;
    		$sql .= " LIMIT " . $offset . ", " . $limit;
    		return $this->db->query ( $sql )->result();
    	} else {
    		$result = $this->db->query ( $sql )->row();
    		return $result ? $result->TotalCount : 0;
    	}
    }
    
    /**
     * View Requirement
     * @param number $id
     * @return number
     */
	public function get_leads_view($id, $with_status = false) {
	    $sql = "SELECT M.*";
	    if($with_status) {
	        $sql .= ", S.name_lang AS status_lang, S.name AS status, S.color AS status_color";
	    }
    	$sql .= " FROM " . $this->db->dbprefix . "leads M";
    	if($with_status) {
    	    $sql .= " LEFT JOIN " . $this->db->dbprefix . "lead_statuses S ON S.id = M.status_id";
    	}
    	$sql .= " WHERE M.id = $id";
    	return $this->db->query ( $sql )->row();
    }
    
    /**
     * Get Business Owners information based on Category and City ID
     * @param number $category_id
     * @param number $city_id
     * @param string $featured
     */
    public function get_owners_details($category_id = 0, $tag_id = 0, $city_id = 0, $featured = 0) {
    	/**
    	 	SELECT L.title, L.email, L.phone_number, L.contact_person FROM rs0d_listings L 
			INNER JOIN rs0d_categories_relation LC ON L.id = LC.row_id 
			INNER JOIN rs0d_packages P ON P.id = L.package_id 
			LEFT JOIN rs0d_branches LL ON LL.listing_id = L.id 
			WHERE L.status = 1 AND L.deleted = 0 AND L.featured = 1 
			AND LC.parent_id = 1 and (L.city_id = 1 OR LL.city_id = 1) 
			AND L.id >= (SELECT FLOOR( MAX(id) * Rand()) FROM rs0d_listings) 
			GROUP BY L.id ORDER BY P.priority DESC LIMIT 10
    	 */
    	$sql = "SELECT L.id, L.title, L.email, L.phone_number, L.contact_person";
    	$sql .= " FROM " . $this->db->dbprefix . "listings L";
    	if($category_id != 0) {
    		$sql .= " INNER JOIN " . $this->db->dbprefix . "categories_relation LC on L.id = LC.row_id AND LC.module_id = " . gc_module_id('listings') . " AND LC.parent_id = " . $category_id;
    	}
    	$sql .= " INNER JOIN " . $this->db->dbprefix . "packages P ON P.id = L.package_id";
    	if($tag_id != 0) {
    		$sql .= " INNER JOIN " . $this->db->dbprefix . "tag_relations LT ON LT.row_id = L.id AND LT.module_id = " . gc_module_id('listings') . " AND LT.tag_id = " . $tag_id;
    	}
    	$sql .= " LEFT JOIN " . $this->db->dbprefix . "branches LL ON LL.listing_id = L.id";
    	$sql .= " WHERE L.status = 1 AND L.deleted = 0";
    	if($featured == 1) {
    		$sql .= " AND L.featured = 1";
    	}
    	if($city_id != 0) {
    		$sql .= " AND (L.city_id = " . $city_id . " OR LL.city_id = " . $city_id . ")";
    	}
    	$sql .= " AND L.id >= (SELECT FLOOR( MAX(id) * RAND()) FROM ". $this->db->dbprefix . "listings)"; // display rows on random basis
    	$sql .= " GROUP BY L.id";
    	$sql .= " ORDER BY P.priority DESC"; // Order by packages
    	$sql .= " LIMIT 10";
    	return $this->db->query($sql)->result();
    }
    
    /**
     * Save Lead
     * @return boolean[]|string[]
     */
    public function save_leads($user_id, $mobile = false) {
        $error_message = lang('failure_message');
        $group_id = (int) $this->input->post('group_id');
        $category_id = (int) $this->input->post('category_id');
        $parent_id = (int) $this->input->post('parent_id');
        $sub_id = (int) $this->input->post('sub_id');
        $subsub_id = (int) $this->input->post('subsub_id');
        if(empty($category_id)) {
            if(!empty($subsub_id)) {
                $category_id = $subsub_id;
            } else if(!empty($sub_id)) {
                $category_id = $sub_id;
            } else {
                $category_id = $parent_id;
            }
        }
        $_POST['category_id'] = $category_id;
        $group = $this->db->where('id', $group_id)->get($this->table_groups)->row();
        if(!$group) {            
            return array('success' => false, 'result' => false, 'message' => $error_message, 'type' => 'danger');
        }
        $form_fields = $this->group_fields($group_id, 'frontend', $mobile);
        if(!$form_fields) {
            return array('success' => false, 'result' => false, 'message' => $error_message, 'type' => 'danger');
        }
        if($mobile) { // format post data to save
            $this->load->model('mobiles/mobiles_model');
            $fields_form = $this->mobiles_model->format_fields($form_fields);
            $_POST = $this->mobiles_model->format_post_data($_POST, $fields_form);
			if(!empty($_POST['city'])) {
				$_POST['city_id'] = $_POST['city'];
			}
			if(empty($_POST['parent_id'])) {
			    $_POST['parent_id'] = $category_id;
			}
		}
		$user_id = empty($user_id) ? (int) $this->auth->user_id() : $user_id;
        $role = 'frontend';
        $status_id = 1;
        // get default status
        $status = $this->db->where('is_default', 1)->get('lead_statuses')->row();
        if($status) {
            $status_id = $status->id;
        }
        // get price information from the settings table
        $lead_settings = json_decode(settings_item('lead.settings'));
        $tag = escape($this->input->post('tag'));
        $tag_id = (int) $this->input->post('tag_id');
        if(empty($tag) && !empty($tag_id)) {
            $tag_info = $this->db->where('id', $tag_id)->get('tags')->row();
            if($tag_info) {
                $tag = $tag_info->name;
            }
        }
        // get category details
        $category = $this->db->where('id', $category_id)->get('categories')->row();
        if(!$category) {
            return array('success' => false, 'result' => false, 'message' => $error_message, 'type' => 'danger');
        }
        $this->load->model('users/user_model');
        if(!empty($user_id)) {
            $user = $this->user_model->get_user($user_id);
        } 
        $location_separator = ', ';
        if(empty($user)) {
            $user = (object) $this->input->post();
            $user->display_name = $user->name;
            $user->address = '';
            $location_separator = '';
        }
        $summary = array( // to avoid unnecessary joins only
            'name' => $user->display_name,
            'email' => $user->email,
            'phone_number' => $user->phone_number,
            'address' => $user->address,
            'city_id' => $user->city_id,
            'city' => $user->city,
            'location' => $user->address . (!empty($user->city) ? $location_separator . $user->city : (!empty($_POST['city']) ? $location_separator . $_POST['city'] : '')),
            'category' => $category->name,
            'tag' => $tag,
        ); 
        // get default status and set it accordingly
        $extra_fields = array(
            'group_ids' => $group_id,
            'user_id' => $user_id,
            'city_id' => (int) $this->input->post('city_id'),
            'summary' => json_encode($summary),
            'status_id' => $status_id,
            'price' => $lead_settings->price
        );
        if ($insert_id = $this->fields_model->save_module($form_fields, 'leads', $user_id, ($mobile ? 'mobile' : $role), 0, 'insert', $extra_fields)) {
            $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'lead_email_templates', 'module_table' => 'leads', 'group_id' => $group_id, 'action_type' => 'add', 'id' => $insert_id));         
            if($status_id == 2 && $lead_settings->auto_forward == 1) {
                // Forward the lead to matched Business Owners
                $this->forward_lead($insert_id, 'forward', FALSE, true);
                // $this->forward_address_lead($insert_id, $group_id);
            }
            $message = !empty($group->message_lang) ? lang($group->message) : $group->message;
            return array('success' => true, 'result' => true, 'message' => $message, 'type' => 'success');
        } else {
            $error_message = sprintf(lang('msg_create_failure'), lang('singular_leads'));            
            $validation_errors = validation_errors();
            if ($validation_errors) {
                $error_message = $validation_errors;
            }
            return array('success' => false, 'result' => false, 'message' => $error_message, 'type' => 'danger');
        }
    }
    
    /**
     * Get All Listings to whom lead information to be forwarded
     * based on the category and city id
     * @param int $lead_id
     * @param number $limit
     * @return array|multitype:
     */
    public function get_emails($lead_id, $lead = array(), $limit = 10) {
    	if(empty($lead)) {
    		$lead = $this->leads_model->find($lead_id);
    	}    	
    	if($lead) {
    		// get all listings having lead category id and lead city id
    		/*$category_level = $this->gc->category_level_search($lead->category_id);
    		if(!$category_level) {
    			return array();
    		}*/
    		$sql = "SELECT L.id, L.title, L.contact_person, P.title AS package, L.email, L.user_id, U.display_name, U.email AS user_email, 0 AS type, LE.counts";
	    	$sql .= " FROM " . $this->db->dbprefix . "listings L";
	    	$sql .= " INNER JOIN " . $this->db->dbprefix . "packages P ON P.id = L.package_id";
	    	$sql .= " INNER JOIN " . $this->db->dbprefix . "users U ON U.id = L.user_id";
	    	$sql .= " INNER JOIN " . $this->db->dbprefix . "categories_relation LC ON LC.row_id = L.id AND LC.module_id = " . gc_module_id('listings');
	    	$sql .= " LEFT JOIN " . $this->db->dbprefix . "branches LL ON LL.listing_id = L.id"; // Branch Offices Search
	    	$sql .= " LEFT JOIN " . $this->db->dbprefix . "lead_emails LE ON LE.row_id = L.id AND LE.module_id = " . gc_module_id('listings') ." AND LE.lead_id = " . $lead_id;
	    	$sql .= " WHERE L.status = 1 AND L.deleted = 0";
	    	$sql .= " AND (L.city_id = " . $lead->city_id . " OR LL.city_id = " . $lead->city_id .")";
	    	$sql .= " AND LC.category_id = " . $lead->category_id;
	    	$sql .= " GROUP BY L.id";
	    	$sql .= " ORDER BY P.priority, L.title"; // give priority to subscribed users
	    	$sql .= " LIMIT " . $limit;
	    	$result = $this->db->query($sql)->result();
	    	return $result;
    	}
    	return array();
    }
    
    /**
     * Get Categories or Locations specific listings
     * @param array $lead
     * @param string $type
     * @param number $limit
     * @return array
     */
    public function category_location_specific($lead_id, $lead = array(), $type = "categories", $limit = 10) {
    	if(empty($lead)) {
    		$lead = $this->leads_model->find($lead_id);
    	}
    	if($type == 'categories') {
    		$filter_type = 1;
    	} else {
    		$filter_type = 2;
    	}
    	if(!empty($lead)) {    		
    		$sql = "SELECT L.id, L.title, L.contact_person, P.title AS package, L.email, L.user_id, U.display_name, U.email AS user_email, $filter_type AS type, LE.counts";
    		$sql .= " FROM " . $this->db->dbprefix . "listings L";
    		$sql .= " INNER JOIN " . $this->db->dbprefix . "packages P ON P.id = L.package_id";
    		$sql .= " INNER JOIN " . $this->db->dbprefix . "users U ON U.id = L.user_id";
			if($type == 'categories') {
				$sql .= " INNER JOIN " . $this->db->dbprefix . "categories_relation LT ON LT.row_id = L.id AND LT.module_id = " . gc_module_id('listings');
			} else {
				$sql .= " INNER JOIN " . $this->db->dbprefix . "branches LT ON LT.listing_id = L.id";
			}    		
    		$sql .= " LEFT JOIN " . $this->db->dbprefix . "lead_emails LE ON LE.row_id = L.id AND LE.module_id = " . gc_module_id('listings') ." AND LE.lead_id = " . $lead_id;
    		$sql .= " WHERE L.status = 1 AND L.deleted = 0";
    		if($type == 'categories') {
    			/*$category_level = $this->gc->category_level_search($lead->category_id);
    			if($category_level) {
    				$sql .= " AND LT." . key($category_level['result']) . " = " . $category_level['result'][key($category_level['result'])];
    			}*/
    			$sql .= " AND LT.category_id = " . $lead->category_id;
    		} else {
    			$sql .= " AND (L.city_id = " . $lead->city_id . " OR LT.city_id = " . $lead->city_id .")";
    		}
    		$sql .= " GROUP BY L.id";
    		$sql .= " ORDER BY P.priority, L.title"; // give priority to subscribed users
    		$sql .= " LIMIT " . $limit;
    		$result = $this->db->query($sql)->result();
    		return $result;
    	}
    	return array();
    }
    
    /**
     * Get forwarded leads
     * @param number $lead_id
     * @return object
     */
    public function forwarded_leads($lead_id) {
    	$sql = "SELECT L.id, L.title, L.contact_person, P.title AS package, L.email, L.user_id, U.display_name, U.email AS user_email, 3 AS type, LE.counts";
    	$sql .= " FROM " . $this->db->dbprefix . "lead_emails LE";
    	$sql .= " LEFT JOIN " . $this->db->dbprefix . "listings L ON L.id = LE.row_id AND LE.module_id = " . gc_module_id('listings');
    	$sql .= " LEFT JOIN " . $this->db->dbprefix . "packages P ON P.id = L.package_id";
    	$sql .= " LEFT JOIN " . $this->db->dbprefix . "users U ON U.id = L.user_id";
    	$sql .= " WHERE LE.lead_id = " . $lead_id;
    	$result = $this->db->query($sql)->result();
    	return $result;
    }
    
    /**
     * Listings Having Category/City/Both
     * @param int $type
     * @return string
     */
    public function type_having($type) {
    	switch($type) {
    		case 0:
    			$having = lang('both_category_location');
    			break;
    		case 1:
    			$having = lang('category_only');
    			break;
    		case 2:
    			$having = lang('city_only');
    			break;
    		default:
    			$having = lang('bf_action_search');
    	}
    	return $having;
    }
    
    /**
     * Forward Load
     * @param array $checked
     * @param string $force_forward
     */
    public function forward_lead($lead_id, $page = 'forward', $force_forward = FALSE, $ajax = false) {
    	$success = false;
    	$result = $this->load_listings($lead_id, $page, NULL, $force_forward);
    	if(!empty($result['listings'])) {
    		// update forwarded on date
    		$this->db->where('id', $lead_id)->update('leads', array('forwarded_on' => date('Y-m-d H:i:s')));
    		// forward leads
    		$this->send_lead($lead_id, 0, 0, $result['listings']);
    		$success = true;
    	}
    	return $success;
    }
    
    /**
     * Forward lead based on the users address
     * @param number $id
     * @param array $record
     * @return boolean
     */
    public function forward_address_lead($id, $group_id, $record = array()) {
        if(empty($record)) {
            $record = $this->get_leads_view($id);
        }
        if(!$record) {
            return false;
        }
        $summary = json_decode($record->summary ?? '');
        $radius = 10;
        $limit = 10;
        $listings = $this->leads_model->get_nearest($record->category_id, (!empty($summary->latitude) ? $summary->latitude : null), (!empty($summary->longitude) ? $summary->longitude : null), $id, $radius, $limit);
        if(!empty($listings)) {
            $data = array();
            $datetime = date('Y-m-d H:i:s');
            foreach($listings as $listing) {
                $business_summary = array(
                    'title' => $listing->title,
                    'address' => $listing->address,
                    'location' => !empty($listing->city) ? $listing->city . ', ' . $listing->state . (!empty($listing->postcode) ? ' - ' . $listing->postcode : '') . ', ' . ucwords(strtolower($listing->country)) : '',
                    'distance' => $listing->distance,
                    'contact_person' => $listing->contact_person,
                    'phone_number' => $listing->phone_number,
                    'cover' => $listing->cover,
                    'ratings' => $listing->ratings,
                );
                // save information in the broadcasts table               
                $data[] = array(
                    'user_id' => $record->user_id,
                    'seller_id' => $listing->user_id,
                    'listing_id' => $listing->listing_id,
                    'lead_id' => $id,
                    'business_summary' => json_encode($business_summary),
                    'status_id' => 1, // waiting
                    'created_on' => $datetime,
                    'created_ip' => $this->input->ip_address()
                );                
            }
            if(!empty($data)) {
                $this->db->insert_batch($this->table_broadcasts, $data);
                // notify sellers via Email/SMS/Notification
                // $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'lead_email_templates', 'module_table' => 'leads', 'group_id' => $group_id, 'action_type' => 'add', 'record' => $record, 'id' => $id)); 
            }
            
            // update forwarded_on column for purge removal
            $this->db->where('id', $record->id)->update($this->table_name, array('forwarded_on' => $datetime));
        }
    }
    
    /**
     * Load Listings for Forward and Index page
     * @param string $id
     * @param string $page - index/forward
     * @param string $lead
     * @param boolean $force_forward - Applicable in case of
     * Index Page to resend email to all matching businesses.
     * If a lead is already forwarded, reforward it without checking whether it is forwarded or not.
     * @return multitype:unknown
     */
    public function load_listings($id, $page = 'forward', $lead = NULL, $force_forward = FALSE) {
    	// Load Listings based on the selected settings
    	$settings = json_decode(settings_item('lead.settings'));
    	$limit = $settings->limit;
    	$matches = !empty($settings->matches) ? $settings->matches : '';
    	$matches_category_city = array(); // Listings Matches Both Category and City
    	$matches_category_only = array(); // Listings Matches Category Only
    	$matches_city_only = array(); // Listings Matches City Only
    	if($page == 'forward' || isset($matches->both)) {
    		$matches_category_city = $this->get_emails($id, $lead, $limit);
    	}
    	if($page == 'forward' || isset($matches->category)) {
    		$matches_category_only = $this->category_location_specific($id, $lead, 'categories', $limit);
    	}
    	if($page == 'forward' || isset($matches->city)) {
    		$matches_city_only = $this->category_location_specific($id, $lead, 'locations', $limit);
    	}
    	 
    	// get all listings from the lead_emails table to whom the email has already been forwarded
    	$forwarded = $this->forwarded_leads($id);
    	//$listings = array_unique(array_merge($categories_locations, $categories_specific, $locations_specific), SORT_REGULAR);
    	$listings = array_merge($matches_category_city, $matches_category_only, $matches_city_only, $forwarded);
    	$unique_listings = array();
    	 
    	// Display count of the type of the listings to which the lead is sent
    	$counts = array();
    	// initialize count
    	$counts[0] = 0;
    	$counts[1] = 0;
    	$counts[2] = 0;
    	$counts[3] = 0; // Forwarded
    	// On index page, do not forward a lead to an email ID to whom the lead has already been forwarded
    	// get all these listings using counts column
    	$index_forward = array();
    	if($listings) {
    		foreach($listings as $element) {
    			if(!isset($unique_listings[$element->id])) {
    				$unique_listings[$element->id] = $element;
    				if(empty($element->counts)) {
    					$index_forward[$element->id] = $element;
    				}
    				$counts[$element->type]++;
    			}
    		}
    	}
    	$result = array(
    			'counts' => $counts,
    			'listings' => $page == 'index' ? array_slice(($force_forward ? $unique_listings : $index_forward), 0, $limit) : $unique_listings // Limit listings based on admin preferences in lead.settings
    	);
    	return $result;
    }
    
    /**
     * Send Emails to selected listings
     * @param number $lead_id
     * @param number $listing_ids
     * @param number $queue_override - if set to 1 Override Email Queue
     * @param array $listings - To avoid multiple queries in case of Index Page forward
     * send email immediately
     */
    public function send_lead($lead_id, $listing_ids, $queue_override = 0, $listings = array(), $ajax = false) {
    	$this->lang->load('emailer/emailer');
    	$lead = $this->leads_model->get_leads_view($lead_id);
    	$failure_emails = array();
    	$success = false;
    	if(!empty($listings)) {
    		$success = true;
    		foreach($listings as $listing) {
    			// send email with content finalized by the admin
    			$result = $this->template(0, 0, 'email', $queue_override, $lead, $listing);
    			if(!is_bool($result)) {
    				$failure_emails[] = $result;
    			}
    		}
    	} else if(!empty($listing_ids)) {
    		$success = true;
    		foreach($listing_ids as $listing_id) {
    			// send email with content finalized by the admin
    			$result = $this->template($lead_id, $listing_id, 'email', $queue_override, $lead);
    			if(!is_bool($result)) {
    				$failure_emails[] = $result;
    			}
    		}	    		
	    }
	    if(!$ajax) {
		    if($success) {
		    	if(!empty($failure_emails)) {
		    		Template::set_message(sprintf(lang('em_send_failure'), implode(',', $failure_emails)) . anchor(site_url(SITE_AREA . '/components/emailer/unsubscribers'), lang('click_here'), array('target' => '_blank')), 'error');
		    	} else {
		    		if($queue_override == 0) {
		    			Template::set_message(lang('emailer_create_email_success'), 'success');
		    		} else {
		    			Template::set_message(lang('leads_forward_success'), 'success');
		    		}
		    	}
		    } else {
		    	Template::set_message(lang('leads_forward_failure'), 'error');
		    }
	    }
    }
    
    /**
     * Email Lead Template
     * @param number $lead_id
     * @param number $listing_id
     * @param string $type - email/pdf: 'email'
     * @param array $lead
     * @param array $listing
     * @return boolean
     */
    public function template($lead_id, $listing_id, $type, $queue_override, $lead = array(), $listing = array()) {
    	if(empty($lead)) {
    		$lead = $this->leads_model->get_leads_view($lead_id);
    	}
    	if(empty($listing)) {
    		$listing = $this->leads_model->listing_info($listing_id);
    	}
    	if($lead) {
    		$this->load->library('parser');
    		$this->lang->load('emailer/emailer');
    		$contact_person = !empty($listing->contact_person) ? $listing->contact_person : $listing->display_name;
    		$to = !empty($listing->email) ? $listing->email : $listing->user_email;
    		$summary = json_decode($lead->summary ?? '');
    		if(empty($summary->name)) {
    		    return false;
    		}
    		$subject = sprintf(lang('tpl_subject'), $contact_person, $summary->name, $summary->tag, $summary->category);
    		 
    		$message_data = array(
    		    'tag' => $summary->tag,
    		    'category' => $summary->category,
    		    'city' => $summary->city,
    			'description' => $lead->description,
    			'ip_address' => $lead->created_ip,
    			'created_on' => gc_date_format($lead->created_on, TRUE),
    			'site_logo' => gc_image_url(settings_item('site.logo'), false),
    			'site_title' => settings_item('site.title'),
    			'site_email' => settings_item('site.system_email'),
    			'site_url' => site_url(),
    			'ico_person' => base_url() . 'assets/images/icons/ico_person.png',
    			'ico_phone' => base_url() . 'assets/images/icons/ico_phone.png',
    			'ico_email' => base_url() . 'assets/images/icons/ico_email.png',
    			'ico_marker' => base_url() . 'assets/images/icons/ico_marker.png',
    			'ico_time' => base_url() . 'assets/images/icons/ico_time.png'
    		);
    		$message_data['header_text'] = sprintf(lang('tpl_subject'), $contact_person, '<br><br><strong>' . $summary->name . '</strong>', '"<strong>' . $summary->tag . '</strong>"', '"<strong>' . $summary->category . '</strong>"');;
    		$message_data['name'] = $summary->name;
    		$message_data['phone_number'] = $summary->phone_number;
    		$message_data['email'] = $summary->email;
    		$message_data['contact_person'] = $contact_person;
    		 
    		// show/hide buyer fields and reply button based on the below given fields value
    		// e.g. if the status is converted, do not display Reply button but display buyer information
    		// if the status is set to forwarded and the price is zero then display the buyer information
    		$message_data['price'] = $lead->price;
    		$message_data['status'] = $lead->status_id;
    		 
    		//$message_data['unsubscribe'] = gc_unsubscribe($to, 'leads');
    		//$message = $this->parser->parse('email_templates/email_body', $message_data, TRUE);
    		$message = $this->parser->parse('leads/template', $message_data, TRUE);
    		if($type == 'email') {
    			// Save in Sent Record
                if(!empty($listing_id)) { // lead is allotted not forwarded, the listing_id is null in case of allotment
    			    $this->db->on_duplicate('lead_emails', array('user_id' => $listing->user_id, 'row_id' => $listing->id, 'module_id' => gc_module_id('listings'), 'lead_id' => $lead->id, 'email' => $to, 'counts' => 'counts + 1', 'created_on' => date('Y-m-d H:i:s'), 'modified_on' => date('Y-m-d H:i:s'), 'created_ip' => $this->input->ip_address(), 'modified_ip' => $this->input->ip_address()));
                }
                
    			// Send Email
    			$this->load->library('emailer/emailer');
    			$data = array(
    					'to'		=> $to,
    					'subject'	=> $subject,
    					'message'	=> gc_unsubscribe($to, 'leads', $message)
    			);
    			$success = $this->emailer->send($data, ($queue_override == 0 ? TRUE : FALSE), 'leads');
    			if($success) {
    				return true;
    			} else {
    				return $to; // display email id to the admin
    			}    
    		} else {
    			$this->load->library('slug');
    			$file = lang('lead') . ':: ' . (!empty($summary->tag) ? $summary->tag : $summary->category) . '-' . $lead->created_on;
    			$filename = $this->slug->create_slug($file);
    			$this->gc->create_pdf($filename, $message);
    		}
    	}
    }
    
    /**
     * Listing Title Format View Page
     * @param array $listing
     * @return string
     */
    public function title_format($listing) {
    	$format = $listing->id .' - '. $listing->title . ' (' . $listing->email .', ' . lang('label_package') . '::' . $listing->package . ') ';
    	return $format;
    }
    
    /**
     * Forward - Allow Admin to Search Listings
     * 
     * @param array $ids
     * @param string $q
     * @param number $limit
     * @return object
     */
    public function search_listing($ids = array(), $q = NULL, $limit = 0) {
    	if(!empty($q)) {
    		$sql = "SELECT L.id, CONCAT(L.id, ' - ', L.title, ' (', L.email, ', " . lang('label_package') . "::', P.title, ') ') AS name";
    	} else {
    		$sql = "SELECT L.id, L.title, L.contact_person, P.title AS package, L.email, U.display_name, U.email";
    	}
    	$sql .= " FROM " . $this->db->dbprefix . "listings L";
    	$sql .= " INNER JOIN " . $this->db->dbprefix . "packages P ON P.id = L.package_id";
    	$sql .= " INNER JOIN " . $this->db->dbprefix . "users U ON U.id = L.user_id";
    	$sql .= " WHERE L.status = 1 AND L.deleted = 0";
    	if(!empty($q)) {
    		$sql .= " AND L.title LIKE '%" . $q . "%'";
    	}
    	if(!empty($ids)) {
    		$sql .= " AND L.id IN (" . implode(',', $ids) . ")";
    	} else {
    		$sql .= " ORDER BY P.priority, L.title"; // give priority to subscribed users
    	}
    	if($limit != 0) {
    		$sql .= " LIMIT " . $limit;
    	}
    	$result = $this->db->query($sql)->result();
    	return $result;
    }
    
    /**
     * Send Email Template - Get Contact Person Name and Email
     * @param int $listing_id
     */
    public function listing_info($listing_id) {
    	$sql = "SELECT L.id, L.title, L.contact_person, L.email, L.user_id, U.display_name, U.email AS user_email";
    	$sql .= " FROM " . $this->db->dbprefix . "listings L";
    	$sql .= " INNER JOIN " . $this->db->dbprefix . "users U ON U.id = L.user_id";
    	$sql .= " WHERE L.id = " . $listing_id;
    	return $this->db->query($sql)->row();
    }
    
    /**
     * Display forwarded lead counts in the Member Area
     * @param int $user_id
     * @return number
     */
    public function member_count($user_id) {
    	$sql = "SELECT COUNT(DISTINCT(LD.id)) as TotalCount";
    	$sql .= " FROM " . $this->db->dbprefix . "leads LD";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "lead_emails LE ON LE.lead_id = LD.id";
    	$sql .= " LEFT JOIN " . $this->db->dbprefix . "lead_allotments LA ON LA.lead_id = LD.id";
    	$sql .= " WHERE (LD.status_id = 1 OR LD.status_id = 2) AND (LE.user_id = " . $user_id . " OR LA.user_id = " . $user_id . ")";
    	// $sql .= " LEFT JOIN " . $this->db->dbprefix . "lead_emails LE ON LE.lead_id = LD.id";
    	// $sql .= " WHERE (LD.status_id = 2 OR LD.status_id = 3) AND (LE.user_id = " . $user_id . " OR LD.assigned_to = " . $user_id . ") AND LD.deleted = 0";
    	// $sql .= " GROUP BY LD.id";
    	$result = $this->db->query($sql)->row();
    	return $result ? $result->TotalCount : 0;
    }
    
    /**
     * Change Lead Status
     * If the Automatic Lead Forward option is enabled forward the lead to Business owners
     * 
     * @param int $id
     * @param int $status
     */
    public function update_forward_lead($id, $status) {
    	if($status == 1) {
    		$settings = json_decode(settings_item('lead.settings'));
    		if($settings->auto_forward == 1) {
	    		// Check whether the lead is already forwarded or not
	    		$lead = $this->leads_model->find($id);
	    		if($lead && (empty($lead->forwarded_on) || $lead->forwarded_on == '0000-00-00 00:00:00')) {
	    			// Forward the lead to matched Business Owners
	    			$this->forward_lead($id);
	    		}
    		}
    	}
    	$this->gc->update_status('leads', array($id), 'status', $status);    	
    }
    
    /**
     * Display Summary
     * @return array
     */
    public function summary($where = '') {
        $table = 'leads';
        $sql = "SELECT COUNT(*) AS total_active";
        
        // today rows
        $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $table;
        $sql .= " WHERE YEAR(created_on) = YEAR(NOW()) AND MONTH(created_on) = MONTH(NOW()) AND DAY(created_on) = DAY(NOW())" . $where .") AS today_rows";
        
        // this month rows
        $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $table;
        $sql .= " WHERE MONTH(created_on) = MONTH(CURDATE()) AND YEAR(created_on)  =  YEAR(CURDATE())" . $where .") AS month_rows";
        
        // yesterday rows
        $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $table;
        $sql .= " WHERE created_on <= NOW() AND created_on = DATE_ADD(Now(),interval - 1 day)" . $where .") AS yesterday_rows";
        
        $sql .= " FROM " . $this->db->dbprefix . $table;
        $sql .= " WHERE status_id = 2" . $where;
        $query = $this->db->query($sql);
        $result = $query->row();
        foreach($result as $key => $value) {
            $result->{$key} = $value;
        }
        return $result;
    }
    
    /**
     * Get Group Fields
     * @param number $id
     * @param string $type
     * @return string
     */
    public function group_fields($id, $type = 'frontend', $mobile = 0) {
        $cacheID = 'lead_group_' . $id . '_' . $type . '_' . $mobile;
        if(!$fields = $this->gc_leads_cache->load($cacheID)) {
            $field_ids = array();
            $records = $this->db->where('group_id', $id)->order_by('priority', 'ASC')->get($this->table_group_fields)->result_array();
            if($records) {
                foreach($records as $record) {
                    $field_ids[$record['field_id']] = $record;
                }
            }
            $module = 'leads';
            $field_forms = array();
            // get all fields
            $fields = $this->fields_model->get_fields($module, $type, array(), 0, array('mobile' => $mobile));
            if(!empty($field_ids)) {
                foreach($fields as $key => $field) {
                    if(!array_key_exists($field['id'], $field_ids)) {
                        unset($fields[$key]);
                    } else if($field) {
                        $rules = explode('|', $field['rules'] ?? '');
                        if($field_ids[$field['id']]['is_required'] == 1) {
                            $fields[$key]['required'] = 'required';
                            if(!in_array('required', $rules)) {
                                $rules[] = 'required';
                                $fields[$key]['rules'] = implode('|', $rules);
                            }
                        } else {
                            $fields[$key]['required'] = '';
                            if(in_array('required', $rules)) {
                                $key = array_search('required', $rules);
                                if (false !== $key) {
                                    unset($rules[$key]);
                                }
                                $fields[$key]['rules'] = implode('|', $rules);
                            }
                        }
                        $field_forms[$field['id']] = $field;
                    }
                }
            }
            if($type == 'frontend' || $type == 'admin') {
                // get saved order priority
                $form_fields = array();
                foreach($field_ids as $field_info) {
                    if(isset($field_forms[$field_info['field_id']])) {
                        $form_fields[$field_info['field_id']] = $field_forms[$field_info['field_id']];
                    }
                }
                $fields = $form_fields;
            }
            $this->gc_leads_cache->save($fields, $cacheID, array('lead_group_fields', 'fields'));
        }
        return $fields;
    }
    
    /**
     * Export leads list in an Excel/PDF format
     * @param number $group_id
     */
    public function export($group_id = 0, $limit = 20000, $offset = 0, $format = 'excel', $sort_by = 'M.id DESC', $where = null) {
        $this->load->library('format');
        $module = 'leads';
        $form_fields = $this->leads_model->group_fields($group_id, 'field_report');
        if(!empty($form_fields)) {
            $table = $module;
            $sql = "SELECT M.*, S.color AS status_color, S.name AS status, S.name_lang AS status_lang";
            $sql .= ", (SELECT GROUP_CONCAT(name SEPARATOR ', ') FROM " . $this->db->dbprefix . $this->table_groups . " WHERE FIND_IN_SET(id, M.group_ids)) AS groups_info";
            $sql .= " FROM " . $this->db->dbprefix . $table . " M";
            $sql .= " LEFT JOIN " . $this->db->dbprefix . $this->table_statuses . " S ON S.id = M.status_id";
            $query_where = null;
            if(!empty($group_id)) {
                $group = $this->db->where('id', $group_id)->get($this->table_groups)->row();
                $query_where = ' WHERE FIND_IN_SET (' . $group_id . ', M.group_ids)';
            }
            if(!empty($where)) {
                $query_where .= !empty($query_where) ? ' AND ' . $where : ' WHERE ' . $where;
            }
            $sql .= $query_where;
            $sql .= " GROUP BY M.id ORDER BY " . $sort_by;
            if ($limit != 0) {
                $sql .= " LIMIT " . $offset . ", " . $limit;
            } else {
                $sql .= " LIMIT $limit";
            }
            $records = $this->db->query($sql)->result();
            if(!empty($records)) {
                $mandatory_fields = array('name', 'status', 'created_on');
                $not_specified = lang('not_specified');
                $field_names = array(
                    'name'
                );
                $field_labels = array(lang('label_name'));
                $columns = array('name' => $not_specified);
                foreach($form_fields as $form_field) {
                    if(!in_array($form_field['name'], $mandatory_fields)) {
                        $field_names[] = $form_field['name'];
                        $field_labels[] = !empty($form_field['label_lang']) ? lang($form_field['label']) : $form_field['label'];
                        $columns[$form_field['name']] = $not_specified;
                    }
                }
                if(empty($group_id)) {
                    $field_names[] = 'groups';
                    $field_labels[] = lang('lead_groups');
                    $columns['groups'] = $not_specified;
                }
                $field_names[] = 'status';
                $field_names[] = 'created_on';
                $field_labels[] = lang('singular_lead_statuses');
                $field_labels[] = lang('created_on');
                $columns['status'] = $not_specified;
                $columns['created_on'] = $not_specified;
                $file_name = 'Leads' . (!empty($group->name) ? '_' . $group->name : '') . '_' . date('Y-m-d H:i:s');
                $file_name = $this->slug->create_slug($file_name);
                $rows = array();
                /*if(empty($group_id)) {
                 foreach($records as $key => $record) {
                 $records[$key] = $this->merge_meta($record);
                 }
                 $rows = $records;
                 }*/
                foreach($records as $key => $record) {
                    $record = (array) $this->merge_meta($record);
                    $row = $columns;
                    $row['name'] = $record['name'];
                    foreach($record as $key => $value) {
                        if(in_array($key, $field_names)) {
                            $row[$key] = $value;
                        }
                    }
                    $row['status'] = !empty($record['status_lang']) ? lang($record['status']) : $record['status'];
                    $row['created_on'] = $record['created_on'];
                    $rows[] = $row;
                }
                if($format == 'excel') {
                    $data = array('file_name' => $file_name, 'field_names' => $field_names, 'rows' => $rows);
                    $this->format->to_excel($data);
                } else { // pdf format
                    $content = $this->load->view('leads/financial/_pdf', array('field_names' => $field_labels, 'rows' => $rows), true);
                    $data = array(
                        'filename' => $file_name,
                        'content' => $content,
                        'download' => true
                    );
                    $this->gc->create_pdf($file_name, $content);
                }
            } else {
                Template::set_message(lang('lead_export_error'), 'danger');
            }
        } else {
            Template::set_message(lang('lead_export_error'), 'danger');
        }
    }
    
    /**
     * Meta information is saved in the subscribers table to avoid unnecessary joins
     * Get it and join with records
     * @param array $record
     */
    public function merge_meta($record) {
        if(!empty($record->meta_data)) {
            $metas = json_decode($record->meta_data ?? '', true);
            if(!empty($metas)) {
                foreach($metas as $meta) {
                    $record->{$meta['name']} = $meta['value'];
                }
            }
        }
        return $record;
    }
    
    /**
     * Get Lead Group Data
     * @param number $group_id
     * @param number $id
     * @return array
     */
    public function get_info($group_id, $id) {
        $record = $this->get_row('leads', $id);
        if(!$record) {
            return false;
        }
        $form_fields = $this->group_fields($group_id);
        $record = (array) $record;
        $summary = json_decode($record['summary'] ?? '', true);
        $record = array_merge($record, $summary);
        $row = $record;
        foreach($record as $key => $value) {
            if(!empty($value)) {
                switch($key) {
                    case 'assigned_to':
                    case 'forwarded_on':
                    case 'status_color':
                    case 'status_lang':
                        break;
                    case 'created_on':
                    case 'modified_on':
                        $record['value_' . $key] = gc_date_format($value);
                        break;
                    default:
                        $record['value_' . $key] = $value;
                }
            }
            unset($record[$key]);
        }
        $record['value_status'] = !empty($row['status_lang']) ? lang($row['status']) : $row['status'];
        $record['label_ip_address'] = lang('ip_address');
        $record['value_ip_address'] = !empty($record['created_ip']) ? $record['created_ip'] : lang('not_applicable');
        $record['label_location'] = lang('label_location');
        $record['value_location'] = !empty($record['geo_location']) ? $record['geo_location'] : lang('not_applicable');
        return $record;
    }
    
    /**
     * Get Lead Row
     * @param string $module
     * @param number $id
     * @param boolean $is_pdf
     * @param string $where
     * @return array|number
     */
    public function get_row($module, $id, $is_pdf = false, $where = null) {
        $sql = "SELECT M.*, CT.name AS city, CG.name AS category, T.name AS tag";
        $sql .= ", ST.color AS status_color, ST.name_lang AS status_lang, ST.name AS status";
		$sql .= ", U.display_name AS user, U.email AS user_email";
        $sql .= " FROM " . $this->db->dbprefix . $module . " M" . PHP_EOL;
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "users U ON U.id = M.user_id";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "cities CT ON CT.id = M.city_id";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "categories CG ON CG.id = M.category_id";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "tags T ON T.id = M.tag_id";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . $this->table_statuses .  " ST ON ST.id = M.status_id" . PHP_EOL;
        $sql .= " WHERE M.deleted = 0";
        if(!empty($id)) {
            $sql .= " AND M.id = " . $id;
        }
        if(!empty($where)) {
            $sql .= " AND " . $where;
        }
        $result = $this->db->query($sql)->row();
        return $result;
    }
    
    /**
     * Index Page Footer Forward and Force Forward
     * @param array $checked
     * @param string $force_forward
     */
    public function index_forward($checked, $force_forward = FALSE) {
        $success = false;
        foreach($checked as $lead_id) {
            $success = $this->forward_lead($lead_id, 'index', $force_forward);
        }
        if($success) {
            Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), lang('leads'), lang('label_forwarded')), 'success');
        } else {
            Template::set_message(lang('leads_forward_failure'), 'error');
        }
    }
    
    /**
     * Check Group ID validity
     * @param number $group_id
     * @return object
     */
    public function valid_group($group_id) {        
        if(empty($group_id)) {
            // get default group
            $group = $this->db->where(array('is_default' => 1, 'status' => 1))->get('lead_groups')->row();
        } else {
            $group = $this->db->where('id', $group_id)->get('lead_groups')->row();
        }
        return $group;
    }
    
    /**
     * Get the nearest listings based on user address latitude, longitude and radius
     * @param string $latitude
     * @param string $longitude
     * @param number $radius
     * @param number $limit
     * @param number $delivery_id - Do not include those delivery users who have alredy refused it
     * @return object
     */
    public function get_nearest($category_id, $latitude, $longitude, $lead_id = 0, $radius = 10, $limit = 10) {
        $distance_unit = settings_item('site.distance_unit') == 'km' ? 111.045 : 69.0; // use miles instead of km, the distance_unit for miles is 69.0.
        $sql = "SELECT T.user_id, T.listing_id, T.distance, T.package_priority,";
        $sql .= " T.title, T.cover, T.slug, T.address, T.city, T.postcode, T.state, T.country, T.ratings, T.contact_person, T.phone_number";
        $sql .= " FROM (";
        $sql .= "SELECT listing_id, user_id, latitude, longitude, package_priority,";
        $sql .= " title, cover, slug, address, city, postcode, state, country, ratings, contact_person, phone_number,";
        $sql .= " ROUND(distance, 2) AS distance";
        $sql .= " FROM (SELECT M.id AS listing_id, M.user_id, M.latitude, M.longitude, M.package_priority,
                 M.title, M.cover, M.slug, M.address, M.city, M.postcode, M.state, M.country, M.ratings, M.contact_person, M.phone_number,
                 P.radius, P.distance_unit
                 * DEGREES(ACOS(LEAST(1.0, COS(RADIANS(P.latpoint))
                 * COS(RADIANS(M.latitude))
                 * COS(RADIANS(P.longpoint - M.longitude))
                 + SIN(RADIANS(P.latpoint))
                 * SIN(RADIANS(M.latitude))))) AS distance";
        $sql .= " FROM " . $this->db->dbprefix . "listings AS M";
        $sql .= " JOIN ( SELECT  " . $latitude . "  AS latpoint,  " . $longitude . " AS longpoint, " . $radius . " AS radius, " . $distance_unit . " AS distance_unit) AS P ON 1=1";
        $sql .= " WHERE M.latitude BETWEEN P.latpoint  - (P.radius / P.distance_unit) AND P.latpoint  + (P.radius / P.distance_unit)";
        $sql .= " AND M.longitude BETWEEN P.longpoint - (P.radius / (P.distance_unit * COS(RADIANS(P.latpoint)))) AND P.longpoint + (P.radius / (P.distance_unit * COS(RADIANS(P.latpoint))))";
        if(!empty($lead_id)) { // do not include those listings who are already in the broadcasts table
            $sql .= " AND NOT EXISTS (SELECT * FROM " . $this->db->dbprefix . $this->table_broadcasts . " LB WHERE LB.lead_id = " . $lead_id . " AND LB.listing_id = M.id)";
        }
        // $sql .= " AND M.is_live = 1"; // get only opened listings 
        $sql .= " AND FIND_IN_SET(" . $category_id . ", M.categories_set)";
        $sql .= ") AS d WHERE distance <= radius ORDER BY distance, package_priority DESC";
        $sql .= " LIMIT " . $limit;
        $sql .= ") AS T";
        return $this->db->query($sql)->result();
    }
    
    /**
     * Purge Leads
     * @param array $ids
     */
    public function _purge($ids) {
        foreach($ids as $id) {
            $this->_purge_lead($id);
        }
        return true;
    }
    
    /**
     * Allow Business Owner to Delete Broadcast
     * @param number $id
     */
    public function _purge_broadcast($ids) {
        foreach($ids as $id) {
            $this->db->where('id', $id)->delete($this->table_broadcasts);
        }
        return true;
    }
    
    /**
     * Purge Lead
     * @param number $id
     * @param array $record
     */
    private function _purge_lead($id, $record = array()) {    
        if(empty($record)) {
            $record = $this->db->where('id', $id)->get($this->table_name)->row();
        }
        if(empty($record)) {
            return false;
        }
        // purge broadcasts first
        $this->db->where('lead_id', $id)->delete($this->table_broadcasts);
        
        // purge images
        if(!empty($record->images)) {
            $this->_purge_images($record->images);
        }
        
        // purge lead
        $this->db->where('id', $id)->delete($this->table_name);
    }
    
    /**
     * Lead Broadcast Update by Business Owner
     * @param number $id
     * @param number $status_id
     */
    public function broadcast_update($id, $user_id, $role = 'admin', $status_id = 2) {
        $user_id = empty($user_id) ? $this->auth->user_id() : $user_id;
        // get lead information
        $record = $this->db->where('id', $id)->get($this->table_broadcasts)->row();
        if(!$record || $record->seller_id != $user_id) { // only corresponding user can see and update
           return false; 
        }
        if(isset($_POST['save'])) {
            $this->form_validation->set_rules('price', 'lang:label_price', 'trim|required|is_decimal');
            if ($this->form_validation->run() === TRUE) {
                $comments = $this->input->post('comments');
                $data = array(
                    'price' => (float) $this->input->post('price'),
                    'comments' => $comments,
                    'status_id' => $status_id,
                    'modified_on' => date('Y-m-d H:i:s'),
                    'modified_ip' => $this->input->ip_address()
                );
                $result = $this->db->where('id', $id)->update($this->table_broadcasts, $data);
                if($result) {
                    // notify customer
                    
                }
                return array('success' => $result, 'message' => sprintf(lang('msg_action_success'), lang('singular_leads'), lang('label_updated')));
            } else if($role == 'mobile') {
                return array('success' => false, 'message' => validation_errors());
            }
        }
        $toolbar_title = lang('lead_update') . ' # ' . $record->lead_id;
        if($role == 'mobile') {
            $this->load->model('mobiles/mobiles_model');
            $prefix = '';
            $fields = array(
                array('name' => $prefix . 'price', 'label' => lang('label_price'), 'type' => 'text', 'rules' => array('is_decimal' => 'is_decimal')),
                array('name' => $prefix . 'comments', 'label' => lang('label_comment'), 'type' => 'textarea', 'rules' => array())                
            );
            $fields = $this->mobiles_model->rest_fields($fields);
            $data = array(
                'success' => true,
                'fields' => $fields,
                'record' => array('price' => 0),
                'extra' => array(
                    'name' => lang('lead_item_available'),
                    'icon' => 'create',
                    'help' => lang('lead_form_heading'),
                    'toolbar_title' => $toolbar_title
                )
            );
            return $data;
        } else {
            Template::set(array(
                'toolbar_title' => $toolbar_title
            ));
            Template::set_view('leads/broadcast_update');
            Template::render($role != 'admin' ? 'member' : '');
        }
    }
    
    /**
     * Convert Lead to Purchase Order
     * @param number $broadcast_id
     * @param number $user_id
     */
    public function lead_to_po($broadcast_id, $user_id, $role = 'mobile') {
        $record = $this->broadcasted_lead($broadcast_id, $user_id);
        if(!$record) {
           return array('success' => false); 
        }
        $this->load->model('mobiles/mobiles_model');
        $prefix = '';
        $fields = array(
            array('name' => $prefix . 'order_info', 'label' => lang('label_order_info'), 'type' => 'html', 'rules' => array())
        );
        $toolbar_title = lang('lead_po');
        $business_summary = json_decode($record->business_summary ?? '');
        $order_info = $business_summary->title . '<div class="font-12">' . $business_summary->address . $business_summary->location . '</div>';
        $fields = $this->mobiles_model->rest_fields($fields);
        $data = array(
            'success' => true,
            'fields' => $fields,
            'record' => array('order_info' => $order_info),
            'extra' => array(
                'name' => $toolbar_title,
                'icon' => 'create',
                'help' => lang('lead_convert_heading'),
                'toolbar_title' => lang('singular_leads') . ' # ' . $record->id
            )
        );
        return $data;
    }
    
    /**
     * Get Broadcasted Lead
     * @param number $broadcast_id
     * @param number $user_id
     * @return object
     */
    private function broadcasted_lead($broadcast_id, $user_id) {        
        // get information from the broadcasts table
        $sql = "SELECT LD.*, M.listing_id, M.price AS amount, M.business_summary";
        $sql .= " FROM " . $this->db->dbprefix . $this->table_broadcasts . " M";
        $sql .= " INNER JOIN " . $this->db->dbprefix . $this->table_name . " LD ON LD.id = M.lead_id";
        $sql .= " WHERE M.id = " . $broadcast_id . " AND M.status_id = 2 AND M.user_id = " . $user_id;
        $record = $this->db->query($sql)->row();
        return $record;
    }
    
    /**
     * Purge all leads received within a specific number of hours
     */
    public function cron_purge() {
        $hours = 48;
        // get leads
        $sql = "SELECT id, images FROM " . $this->db->dbprefix . $this->table_name . " WHERE forwarded_on > NOW() - INTERVAL " . $hours . " HOUR";
        $records = $this->db->query($sql)->result();
        if($records) {
            foreach($records as $record) {
                $this->_purge_lead($record->id, $record);
            }
        }
        /*
         * Unable to delete images
        // purge broadcasts
        $sql = "DELETE FROM " . $this->db->dbprefix . $this->table_broadcasts . " WHERE created_on > NOW() - INTERVAL " . $hours . " HOUR";
        $this->db->query($sql);
        
        // purge leads
        $sql = "DELETE FROM " . $this->db->dbprefix . $this->table_name . " WHERE forwarded_on > NOW() - INTERVAL " . $hours . " HOUR";
        $this->db->query($sql);
         */
    }
    
    /**
     * Purge Images on Lead delete
     * @param string $images
     */
    public function _purge_images($images, $folder = 'leads') {
        $images = json_decode($images ?? '', true);
        foreach($images as $image) {
            $path = FCPATH . 'assets/images/' . $folder . '/';
            $thumb_path = $path . '/thumbs/thumb__' . $image;
            if(file_exists($thumb_path)) {
                unlink($thumb_path);
            }
            $image_path = $path . '/' . $image;
            if(file_exists($image_path)) {
                unlink($image_path);
            }
        }
    }
    
    /**
     * Lead Detail or PDF
     * @param string $delivery_id
     */
    public function detail($id, $is_pdf = false, $params = array()) {
        $this->lang->load('leads/leads');
        $record = $this->get_leads_view($id, true);
        if(!$record) {
            return false;
        }
        $mobile = !empty($params['mobile']) ? true : false;
        // display images to the user
        if(!empty($record->images)) {
            $lead_images = array();
            $images = json_decode($record->images ?? '');
            foreach($images as $image) {
                $lead_images[] = array(
                    'original' => gc_image_url($image, true, true, array('module' => 'leads')),
                    'thumb' => gc_image_url($image, true, false, array('module' => 'leads'))
                );
            }
            $record->images = $lead_images;
        }
        $data = array(
            'record' => $record,
            'toolbar_title' => lang('singular_leads') . ' #' . $id
        );
        if($is_pdf) {
            $data['is_pdf'] = true;
            $content = $this->load->view('leads/detail', $data, true);
            $this->gc->create_pdf($record->$id, $content, true, array('download' => true));
        } else {
            if($mobile) {
                $users = array();
                $summary = json_decode($record->summary ?? '', true);
                $users[] = array(
                    'label' => lang('bf_user'),
                    'data' => $summary
                );
                $row = array(
                    //'amount' => currency_format($record->price),
                    'status' => !empty($record->status_lang) ? lang($record->status) : $record->status,
                    'status_color' => $record->status_color,
                    'category' => $summary['category'],
                    'description' => $record->description,
                    'summary' => $users,
                    'created_on' => gc_date_format($record->created_on),
                    'images' => $record->images
                );
                $data['record'] = $row;
                $data['extra'] = array(
                    'category' => lang('label_category'),
                    'amount' => lang('label_amount'),
                    'status' => lang('label_status'),
                    'date' => lang('created_on'),
                    'description' => lang('label_description')
                );
            }
            return $data;
        }
    }
    
    /**
     * Get Lead specific alloments
     */
    public function get_allotments($count, $where, $sort_by, $offset, $limit) {
        if($count == 0) {
            $sql = "SELECT M.*, U.email AS user_email";
        } else {
            $sql = "SELECT COUNT(*) AS TotalCount";
        }
        $sql .= " FROM " . $this->db->dbprefix . "lead_allotments M";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "users U ON U.id = M.user_id";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "orders O ON O.order_id = M.order_id";
        $sql .= " WHERE " . $where;
        if($count == 0) {
            $sql .= " ORDER BY " . $sort_by;
            $sql .= " LIMIT " . $offset . ", " . $limit;
            $result = $this->db->query ( $sql )->result();
            return $result;
        } else {
            $result = $this->db->query ( $sql )->row();
            return $result ? $result->TotalCount : 0;
        }
    }
}