<?php defined('BASEPATH') || exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # eComsta - Online eCommerce Solution							   # ||
|| # Copyright (C) 2021 eComsta. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by eComsta.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.ecomsta.com | https://www.ecomsta.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Developer controller
 */
class Developer extends Admin_Controller {
    protected $permissionView   = 'Links.Developer.View';

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();        
        $this->auth->restrict($this->permissionView);
        $this->load->model('links/links_model');
        $this->lang->load('links');
        $this->lang->load('members/members');   
      	$this->form_validation->set_error_delimiters("<span class='error'>", "</span>");        
        Template::set_block('sub_nav', 'developer/_sub_nav');
        clear_cache('links');
        $this->gc->site_cache(array('tag' => 'links'));
        Assets::add_module_js('links', 'links.js');
    }
    
    /**
     * Manage Links
     * @param string $filter
     * @param number $offset
     */
    public function index($filter = 'all', $offset = 0) {
        $this->manage('links', $filter, $offset);
    }
    
    /**
     * Manage Links/Menus
     * @param string $module
     * @param string $filter
     * @param number $offset
     */
    public function manage($module = 'links', $filter = 'all', $offset = 0) {
        $extra = array(
            'model' => 'links',
            'context' => 'developer',
            'fields' => $module,
            'prefix' => 'link'
        );
        if($module == 'links') {
            unset($extra['prefix']);
            $extra['advanced_filter'] = array('menu_id');
        }
        $this->gc->module_view($module, $filter, $offset, 'admin', $extra);
    }
    
    /**
     * Add/Edit Links Data
     *
     * @return void
     */
    public function form($module = 'links', $action = 'create') {
        $params = array(
            'selectize' => true,
            'model' => 'links',
            'context' => 'developer',
            'module_name' => 'links',
            'fields_method' => $module,
            'prefix' => 'link'
        );
        if($module == 'links') {
            unset($params['prefix']);
            $id = $this->uri->segment(7) ? $this->uri->segment(7) : 0;
            Template::set('links', $this->get_parent_links($id));
            $theme_layouts = $this->db->select('file_folder AS id, file_folder AS title')->get_where('themes', array('type_id' => 2, 'is_default' => 1))->result();
            Template::set('theme_layouts', $theme_layouts);
        }
        $this->gcrud->form($module, $action, 'admin', $params);
    }
    
    /**
     * Get all parent links
     * @return multitype:string NULL
     */
    public function get_parent_links($id = 0) {
        $where = array('parent_id' => 0);
        if($id != 0) {
            $where['id != '] = $id;
        }
        $links = $this->links_model->select('id, name, name_lang')->find_all_by($where);
        
        // Return key => value pair array
        $result = array(
            array(
            'id' => 0,
            'name' => lang('label_no_parent')
        ));
        if ($links) {
            foreach ($links as $key => $link) {
                $result[] = array(
                    'id' => $link->id,
                    'name' => $link->name_lang == 1 ? lang($link->name) : $link->name
                );
            }
        }
        return $result;
    }
    
    /**
     * Allow admin to change order of fields
     */
    public function order () {
    	//LOAD NESTED SORTABLE JAVASCRIPT AND CSS FILE
    	Assets::add_css('nestedSortable/admin.css');
    	Assets::add_js('nestedSortable/jquery-ui.min.js');
    	Assets::add_js('nestedSortable/jquery.mjs.nestedSortable.js');
    
    	Assets::add_js($this->load->view('developer/order_links_js', null, true), 'inline');
    	$groups =  $this->gc->get_module_relations('groups', 'links');
    	Template::set('groups', $groups);
    	Template::set('sortable', TRUE);
    	Template::set('maxLevels', 5);
    	Template::set('toolbar_title', lang('label_order') . ' ' . lang('label_links'));
    	Template::render();
    }
    
    /**
     * Order links using AJAX nestedSortable
     */
    public function order_ajax () {
        $menu_id = (int) $this->input->post('menu_id');
    	// Save order from ajax call
    	if (isset($_POST['sortable'])) {
    	    $this->links_model->save_order($_POST['sortable'], $menu_id);
    	}
    	// Load view
    	$links = $this->links_model->get_links_order($menu_id);
    	Template::set('result', $links);
    	Template::set('maxLevels', 5);
    	Template::set('toolbar_title', lang('label_order') . ' ' . lang('label_links'));
    	Template::set_view('common/order_ajax');
    	Template::render();
    }
    
    /**
     * Selectize search - Users/Listings
     * @param string $module
     */
    public function selectizeSearch($module_id, $q) {
    	header('Content-Type: application/x-json; charset=utf-8');
    	$q = escape(urldecode($q));
    	$module = gc_module_name($module_id);
    	switch($module) {
    		case 'categories':
    		case 'cities':
    			$field = 'name';
    			break;
    		default:
    			$field = 'title';
    	}
    	if(!empty($q)) {
    		$this->db->or_like($field, $q);
    	}
    	$result = $this->db->select('id, slug AS name')->get($module)->result();
    	echo json_encode($result);
    }
    
    /**
     * Prepare module links
     * A user can add a job without a listing
     */
    public function module_links($update_status = 0) {        
        $gc_modules = gc_modules();
        unset($gc_modules[0]);
        $link_urls = array('view', 'members/view');
        foreach($link_urls as $link_str) {
            $links = $this->db->where('link_url', $link_str)->get('links')->result();
            if($links) {
                $this->load->library('crypt');
                $data = array();
                foreach($links as $link) {
                    if(in_array($link->name, $gc_modules)) {
                        $link_data = array(
                            'id' => $link->id,
                            'link_url' => 'members/view',
                            'gc_url' => 1,
                            'extra' => 'get:md=' . $this->crypt->encode($link->name)
                        );
                        if($update_status == 1) {
                            $link_data['status'] = 1;
                        }
                        $data[] = $link_data;
                    }
                }
                if(!empty($data)) {
                    $this->db->update_batch('links', $data, 'id');
                }
            }
        }
        $new_modules_links = array();
        // get modules whose links does not exists
        foreach($gc_modules as $gc_module) {
            $where = "name='" . $gc_module . "' AND link_url = 'members/view'";
            $link_exist = $this->db->where($where)->count_all_results('links');
            if(empty($link_exist)) {
                $new_modules_links[] = $gc_module;
            }
        }
       
        
        // Add new module links in the database
        if(!empty($new_modules_links)) {
            // get any link
            $link_data = (array) $this->db->where('link_url', 'members/view')->get('links')->row();
            if($link_data) {
                unset($link_data['id']);
                $link_data['created_on'] = date('Y-m-d H:i:s');
                $link_data['modified_on'] = date('Y-m-d H:i:s');                
                if($update_status == 1) {
                    $link_data['status'] = 1;
                }
            }
            if(!empty($link_data)) {
                foreach($new_modules_links as $module_name) {
                    // get module information from the modules table
                    $module_info = $this->db->where('name', $module_name)->get('modules')->row();
                    if($module_info) {
                        $module_link_data = $link_data;
                        $module_link_data['name'] = $module_info->name;
                        $module_link_data['icon'] = 'fas fa-' . $module_info->icon;
                        $module_link_data['extra'] = 'get:md=' . $this->crypt->encode($module_info->name);
                        $this->db->insert('links', $module_link_data);
                    }
                }
            }
        }
        Template::set_message(lang('msg_success'), 'success');
        Template::redirect(SITE_AREA . '/developer/links');
    }
}