<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Migration_Install_links extends Migration
{
	/**
	 * The name of the database table
	 *
	 * @var String
	 */
	private $table_name = 'links';
	private $table_link_locations = 'link_locations';

	/**
	 * The table's fields
	 *
	 * @var Array
	 */
	private $fields = array(
		'id' => array(
			'type' => 'SMALLINT',
			'constraint' => 5,
			'auto_increment' => TRUE,
		),
		'type_id' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default' => 0
		),
		'group_id' => array(
			'type' => 'SMALLINT',
			'constraint' => 5,
			'null' => FALSE,
			'default' => 0
		),
		'parent_id' => array(
			'type' => 'SMALLINT',
			'constraint' => 5,
			'null' => FALSE,
			'default' => 0
		),
		'module_id' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default' => 0
		),
		'row_id' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'null' => FALSE,
			'default' => 0
		),
		'url' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => FALSE
		),
		'gc_url' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default' => 0
		),
		'target' => array(
			'type' => 'VARCHAR',
			'constraint' => 20,
			'null' => TRUE
		),
		'icon' => array(
			'type' => 'VARCHAR',
			'constraint' => 100,
			'null' => FALSE
		),
		'label' => array(
			'type' => 'VARCHAR',
			'constraint' => 100,
			'null' => FALSE
		),
		'label_lang' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default' => 0
		),
		'counts_param' => array(
			'type' => 'VARCHAR',
			'constraint' => 50,
			'null' => TRUE
		),
		'counts_condition' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default' => 0
		),
		'permission' => array(
			'type' => 'VARCHAR',
			'constraint' => 100,
			'null' => TRUE,
		),
		'link_condition' => array(
			'type' => 'VARCHAR',
			'constraint' => 450,
			'null' => TRUE
		),
		'extra' => array(
			'type' => 'VARCHAR',
			'constraint' => 50,
			'null' => TRUE
		),
		'priority' => array(
			'type' => 'SMALLINT',
			'constraint' => 5,
			'null' => FALSE,
			'default' => 0
		),
		'status' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default' => 1
		),
		'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'modified_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP'
	);
	
	/**
	 * The link locations table's fields
	 *
	 * @var Array
	 */
	private $fields_link_locations = array(
		'link_id' => array(
			'type' => 'SMALLINT',
			'constraint' => 5,
			'null' => FALSE,
			'default' => 0
		),
		'location_id' => array(
			'type' => 'TINYINT',
			'constraint' => 2,
			'null' => FALSE,
			'default' => 0
		)
	);
	
	/**
	 * Install this migration
	 *
	 * @return void
	 */
	public function up() {
		$this->dbforge->add_field($this->fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('parent_id', false);
		$this->dbforge->create_table($this->table_name);
		
		// Link Locations Table
		$this->dbforge->add_field($this->fields_link_locations);
		$this->dbforge->add_key('link_id', false);
		$this->dbforge->add_key('location_id', false);
		$this->dbforge->create_table($this->table_link_locations);
		
		// Unique Link Locations table
		$unique_link = "ALTER TABLE " . $this->db->dbprefix . $this->table_link_locations . " ADD UNIQUE INDEX `unique_link` (`link_id`, `location_id`)";
		$this->db->query ($unique_link);
		
		// link settings row
		$this->db->insert('settings', array('name' => 'link.settings', 'module' => 'link', 'value' => '{"display_group":1}'));
	}

	//--------------------------------------------------------------------

	/**
	 * Uninstall this migration
	 *
	 * @return void
	 */
	public function down() {
		$this->dbforge->drop_table($this->table_name);
		$this->dbforge->drop_table($this->table_link_locations);
		// remove settings row
		$settings = $this->db->select('name')->get_where('settings', array('name' => 'link.settings'));
		if($settings) {
			$this->db->delete('settings', array('name' => 'link.settings'));
		}
	}

	//--------------------------------------------------------------------

}