<?php defined('BASEPATH') || exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # eComsta - Online eCommerce Solution							   # ||
|| # Copyright (C) 2021 eComsta. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by eComsta.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.ecomsta.com | https://www.ecomsta.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class Links_model extends BF_Model
{
    public $table_name	= 'links';
	protected $key			= 'id';
	protected $date_format	= 'datetime';

	protected $log_user 	= false;
	protected $set_created	= true;
	protected $set_modified = true;
	protected $soft_deletes	= false;

	protected $created_field     = 'created_on';
	protected $modified_field    = 'modified_on';

	// Customize the operations of the model without recreating the insert,
    // update, etc. methods by adding the method names to act as callbacks here.
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 	    = array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	// For performance reasons, you may require your model to NOT return the id
	// of the last inserted row as it is a bit of a slow method. This is
    // primarily helpful when running big loops over data.
	protected $return_insert_id = true;

	// The default type for returned row data.
	protected $return_type = 'object';

	// Items that are always removed from data prior to inserts or updates.
	protected $protected_attributes = array();

	// You may need to move certain rules (like required) into the
	// $insert_validation_rules array and out of the standard validation array.
	// That way it is only required during inserts, not updates which may only
	// be updating a portion of the data.
	protected $validation_rules 		= array();
	protected $insert_validation_rules  = array();
	protected $skip_validation 			= false;
	protected $link_html;
	protected $processed_links = array();
    protected $z_cache;

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();        
        $this->load->library('gzf_cache', array('lifetime' => 604800));
        $this->load->library('slug');
        $this->z_cache = $this->gzf_cache->get_instance();
        $this->load->library('crypt');
    }
    
    /**
     * Links Fields
     *
     * @return array
     */
    public function links() {        
        $custom_fields = array(
            array(
                'name' => 'key',
                'type' => 'text',
                'label' => lang('label_key')
            ),
            array(
                'name' => 'value',
                'type' => 'text',
                'label' => lang('label_value')
            )
        );
        Template::set('custom_fields', $custom_fields);
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'dropdown',
                'name' => 'menu_id',
                'label' => 'singular_link_menus',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:link_menus;value:database',
                'options_detailed' => 1,
                'field_extra' => 'selectpicker:1;multiple:1;max_options:10',
                'class' => 4,
                'rules' => 'trim',
            ),
            array(
                'type' => 'dropdown',
                'name' => 'parent_id',
                'label' => 'label_parent',
                'frontend' => false,
                'admin_only' => false,
                'frontend_index' => false,
                'admin_index' => true,
                'options' => 'name:links;value:database',
                'options_detailed' => 1,
                'options_lang' => 1,
                'class' => 4,
                'rules' => 'trim|required',
            ),
            array(
                'type' => 'text',
                'name' => 'name',
                'label' => 'label_label',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'tooltip' => lang('help_label'),
                'class' => 4,
                'edit' => 1,
                'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
                'rules' => 'trim|max_length[100]|required',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'name_lang',
                'label' => 'label_lang',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'textarea',
                'name' => 'description',
                'label' => 'label_description',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'field_extra' => 'rows:3',
                'tooltip' => lang('help_link_description'),
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'link_url',
                'label' => 'label_url',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'tooltip' => lang('help_url'),
                'rules' => 'trim|max_length[255]',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'gc_url',
                'label' => 'label_gc_url',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'dropdown',
                'name' => 'target',
                'label' => 'label_target',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'options' => 'name:option_same_window;value:_self|name:option_new_window;value:_blank|name:option_top_window;value:_top|name:option_parent_window;value:_parent',
                'options_detailed' => 1,
                'options_lang' => 1,
                'class' => 2,
                'rules' => 'trim',
            ),
            array(
                'type' => 'iconpicker',
                'name' => 'icon',
                'label' => 'label_icon',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'class' => 3,
                'rules' => 'trim|max_length[100]',
            ),
            array(
                'type' => 'text',
                'name' => 'permission',
                'label' => 'label_permission',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'tooltip' => lang('help_permission'),
                'rules' => 'trim|max_length[100]',
            ),
            array(
                'type' => 'text',
                'name' => 'link_condition',
                'label' => 'label_link_condition',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'tooltip' => lang('help_condition'),
                'rules' => 'trim|max_length[450]',
            ),
            array(
                'type' => 'text',
                'name' => 'extra',
                'label' => 'label_extra',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'tooltip' => lang('help_extra'),
                'rules' => 'trim|max_length[255]',
            ),
            array(
                'type' => 'custom',
                'name' => 'params',
                'label' => 'label_options',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'tooltip' => lang('help_extra_options'),
                'class' => 9,
                'rules' => 'trim',
            ),
            array(
                'type' => 'dropdown',
                'name' => 'layout_id',
                'label' => 'label_layout',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'tooltip' => lang('help_condition'),
                'options' => 'name:theme_layouts;value:database',
                'options_detailed' => 1,
                'class' => 3,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'priority',
                'label' => 'label_priority',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'tooltip' => lang('help_priority'),
                'value' => 0,
                'class' => 2,
                'rules' => 'trim|is_numeric|required',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'status',
                'label' => 'label_status',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
        );
        return $fields;
    }
    
    /**
     * Menu Fields
     *
     * @return array
     */
    public function menus() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'text',
                'name' => 'name',
                'label' => 'label_name',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'edit' => 1,
                'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
                'rules' => 'trim|required|max_length[150]'
            ),
            array(
                'type' => 'checkbox',
                'name' => 'name_lang',
                'label' => 'name_lang',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'iconpicker',
                'name' => 'icon',
                'label' => 'label_icon',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'class' => 3,
                'rules' => 'trim'
            ),
            array(
                'type' => 'text',
                'name' => 'priority',
                'label' => 'label_priority',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'value' => 0,
                'class' => 2,
                'rules' => 'trim|required|is_numeric',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'status',
                'label' => 'label_status',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            )
        );
        return $fields;
    }
    
    /**
     * Process links by passing the Member Queries results
     * @param string $location
     * @param array $member_queries - Member Area Only - Comments, Business Queries etc. Counts
     * Usage: $this->links_model->process_links(6, $link_queries, array('tpl' => 0, 'with_icon' => true));
     * $this->links_model->process_links(6, array(), array('tpl' => 1, 'parent_class' => 'nav-list', 'with_icon' => true)))
     * @param string parent_class - used to assign a class to the parent ul.
     * @param string with_heading - display heading name
     * @param boolean $is_seo_crawler - The request is from the SEO crawler page
     */
    public function process_links($menu_id = 0, $member_queries = array(), $extra = array(), $is_seo_crawler = false) { 
    	// Always Cache Links
        $cacheID = str_replace('-', '_', $this->slug->create_slug('links_' . $menu_id . '_' . $this->session->userdata('language'))); // Cache giving errors when invalid url with spaces and special characters
    	if(!$result = $this->z_cache->load($cacheID)) {    
    	    if(settings_item('site.minimal') == 1) {
    	        $result = $this->gc->site_cache(array('cache_id' => $cacheID));
    	        if(!$result) {
    	            $links = $this->_get_links((int) $menu_id);
    	            $heading = '';
    	            $menu = $this->db->where('id', $menu_id)->get('link_menus')->row();
    	            if($menu) {
    	                $heading = !empty($menu->name_lang) ? lang($menu->name) : $menu->name;
    	            }
    	            $result = array('links' => $links, 'heading' => $heading);
    	            $this->gc->site_cache(array('cache_id' => $cacheID, 'value' => $result, 'tag' => 'links'));
    	        }
    	    } else {    	    
        	    $links = $this->_get_links((int) $menu_id);    	    
        	    $heading = '';
        	    $menu = $this->db->where('id', $menu_id)->get('link_menus')->row();
        	    if($menu) {
        	        $heading = !empty($menu->name_lang) ? lang($menu->name) : $menu->name;
        	    }
        	    $result = array('links' => $links, 'heading' => $heading);
    	    }
    	    $this->z_cache->save($result, $cacheID, array('links'));
    	}
    	$links = $result['links'];
    	if(!empty($links)) {
    		// Convert links into Member Area Array format
    		foreach($links as $key => $link) {
    		    $links[$key]['has_permission'] = 1; // set link default to active
    		    if(!empty($link['permission'])) {
    		        switch(strtolower(trim($link['permission']))) {
    		            case 'site.loggedin.user': // only registered users can view this link
    		                if(!$this->auth->is_logged_in()) {
    		                    $links[$key]['has_permission'] = 0;
    		                }
    		                break;
    		            case 'site.notloggedin.user': // only not logged in users can view this link e.g. signup/login
    		                if($this->auth->is_logged_in()) {
    		                    $links[$key]['has_permission'] = 0;
    		                }
    		                break;
    		            default:
    		                $links[$key]['has_permission'] = $this->auth->has_permission($link['permission']) ? 1 : 0;
    		        }    		       
    		    }
    		    
    			// Set Link Active/Inactive based on the LINK CONDITIONS
    			if(!empty($link['link_condition'])) {
    				$link_conditions = $link['link_condition'];
    				$links[$key]['active'] = $this->check_link_condition($link_conditions) ? 1 : 0;
    			}

    			$links[$key]['badge'] = NULL;
    			// Show Counts in Badge
    			if(!empty($link['counts_param']) && !empty($member_queries)) {
    			    $counts = isset($member_queries[$link['counts_param']]) ? $member_queries[$link['counts_param']] : 0;
    			    $links[$key]['badge'] = $counts;
    			    if(array_key_exists($link['counts_param'], $member_queries) && empty($counts) && $link['counts_condition'] == 1) {
    			        // Do not display the link
    			        unset($links[$key]);
    			    }
    			}
    			// unset($links[$key]['permission']); // required in seo crawler
    			unset($links[$key]['link_condition']);
    			unset($links[$key]['counts_param']);
    			unset($links[$key]['counts_condition']);
    		}    		
    	}
    	if($is_seo_crawler) {
    	    return $links;
    	} else {
        	if(!empty($links)) {
        	    $this->link_html = '';
        	    $tpl = isset($extra['tpl']) ? (int) $extra['tpl'] : 0;
        	    switch($tpl) {
        	        case 4:
        	            $this->print_tree($links, false, null, $tpl, $extra);
        	            break;
        	        case 3:
        	            $this->link_html .= '<div class="navbar-nav' . (isset($extra['parent_class']) ? $extra['parent_class'] : '') . '">';
        	            $this->print_tree($links, false, null, $tpl, $extra);
        	            $this->link_html .= '</div>';
        	            break;
        	        case 2:
        	            $this->link_html .= '<div id="dropdownMenu' . $tpl . $menu_id . '">';
        	            $this->link_html .= '<div class="list-group panel">';
        	            $this->print_tree($links, false, null, $tpl, $extra);
        	            $this->link_html .= '</div>';
        	            $this->link_html .= '</div>';
        	            break;
        	        case 1:
        	            if(!empty($extra['with_heading'])) {
        	               $this->link_html .= '<h5>' . $result['heading'] . '</h5>';
        	            }
        	            if(empty($extra['list_only'])) {
        	                $this->link_html .= '<ul' . (isset($extra['parent_class']) ? ' class="' . $extra['parent_class'] . '"' : '') . '>';
        	            }
        	            $this->print_tree($links, false, null, $tpl, $extra);
        	            if(empty($extra['list_only'])) {
        	                $this->link_html .= '</ul>';
        	            }
        	            break;
        	        default:
        	            // $this->link_html .= '<ul class="navbar-nav ' . (isset($extra['parent_class']) ? $extra['parent_class'] : 'ml-auto') . '">';
        	            $this->print_tree($links, false, null, $tpl, $extra);
        	            // $this->link_html .= '</ul>';    	            
        	    }
        	}
        	return $this->link_html;
    	}
    	// return $links;
    }
    
    /**
     * Member Area - Set link active/inactive based on the link_conditions
     * @param array $link_conditions
     * @return boolean
     * in_array Multiple Values - https://stackoverflow.com/questions/7542694/in-array-multiple-values
     */
    private function check_link_condition($link_conditions = array()) {
    	$operator = false;
    	$uri_segments = array();
    	$matches = array();
    	// To Apply AND operator, the first condition must have Operator defined
    	foreach($link_conditions as $link_condition) {
    		$uri_segment = $this->uri->segment(intval($link_condition['id']));
    		if (count($_GET) > 0) {
    			$uri_segment = $uri_segment . '?' . http_build_query($_GET, '', "&");
    		}
    		$uri_segments[intval($link_condition['id'])] = $uri_segment;
    		$match = isset($link_condition['string']) && $link_condition['string'] == 1 ? gc_uri_string($link_condition['name']) : $link_condition['name'];
    		$matches[] = $match;
    		// If the Operator is defined, this is an AND condition
    		// Return TRUE only if ALL statements are TRUE
    		if(isset($link_condition['operator']) && $link_condition['operator'] == '&') {
    			$operator = true;
    		}
    	}
    	// Execute all conditions with AND Operator - All must be TRUE
    	if($operator) {
    	    // reset the $uri_segments array keys to 0
    	    $uri_segments = array_values($uri_segments);
    	    if(count(array_intersect_assoc($matches, $uri_segments)) == count($uri_segments)) {
    			// all of $uri_segments is in $matches
    			return true;
    		}
    	} else {
	    	// OR condition - Return TRUE if any of the link condition matches
	    	if(count(array_intersect($matches, $uri_segments)) > 0) {
	    		// at least one of $uri_segments is in $matches
	    		return true;
	    	}
    	}
    	return false;
    }
    
    /**
     * Save order only when user clicks on save button
     * @param array $links
     */
    public function save_order($links, $menu_id = 0) {
    	if (count($links)) {
    		foreach ($links as $order => $link) {    		    
    		    $where = 'id = ' . (!empty($link['id']) ? $link['id'] : 0);
    		    if(!empty($menu_id)) {
    		        $where .= ' AND FIND_IN_SET(' . $menu_id . ', menu_id)';
    		    }
    			if (!empty($link['id'])) {    			    
    			    $parent_id = (int) $link['parent_id'];
    			    $data = array('priority' => $order, 'parent_id' => $parent_id);
    				$this->db->where($where)->update('links', $data);
    				
    				/**
    				 * Save Link Relations in the link_relations table for the multilevel menus
    				 * @var array $record
    				 */
    				$id = $link['id'];
    				$record = $this->db->where('id', $id)->get($this->table_name)->row();
    				if($record) {
    				    $parent_id = $record->parent_id;
    				    $this->load->library('goclixy/gclosure');
    				    $gclosure = new Gclosure('links', 'link_relations');
    				    $gclosure->delete($id, false); // do not delete from the links table
    				    // save hierarchy
    				    $gclosure->add($id, ($parent_id == 0 ? $id : $parent_id));
    				}
    			}
    		}
    	}
    }
   
   /**
    * Get Links
    * @param number $menu_id
    * @return string|boolean
    */
   private function _get_links($menu_id = 0) {       
       $this->load->library('goclixy/gclosure');
       $gclosure = new Gclosure('links', 'link_relations');
       $extra = array(
           'where' => 't.status = 1'
       );
       if(!empty($menu_id)) {
           $extra['where'] .= " AND FIND_IN_SET(" . $menu_id . ", t.menu_id)";
       }
       $extra['sort_by'] = 't.priority ASC';
       $records = $gclosure->tree(null, $extra, true, false, true);
       return $records;
   }
   
   /**
    * Link has children hierarchy
    * The first element of the array is not a multidimensional array
    * @param array $tree
    */
   private function print_tree($records, $has_children = false, $ul_extra = '', $tpl = 0, $extra = array()) {
       if($has_children) {
           switch($tpl) {
               case 1:
                   $this->link_html .= PHP_EOL . '<ul' . $ul_extra . '>';
                   break;
               case 2:
                   $this->link_html .= '<div class="collapse"' . $ul_extra . '>';
                   break;
               case 3:
                   break;
               default:
                   $this->link_html .= PHP_EOL . '<ul ' . $ul_extra . 'class="dropdown-menu border-0 shadow">';
           }           
       }       
       foreach($records as $record) {
           $this->link_html($record, $has_children, $tpl, $extra);
       }
       if($has_children) {
           switch($tpl) {
               case 2:
                   $this->link_html .= '</div>';
                   break;
               case 3:
                   break;
               default:
                   $this->link_html .= '</ul>';
           } 
       }
   }
   
   /**
    * Display Tree View
    * @param array $affiliate
    */
   private function link_html($record, $has_children = false, $tpl = 0, $extra = array()) {       
       if(!isset($record['has_permission']) || (isset($record['has_permission']) && $record['has_permission'] == 1)) {
           $this->link_html .= PHP_EOL;
           $icon = !empty($extra['with_icon']) ? '<i class="' . (!empty($record['icon']) ? $record['icon'] : 'fas fa-check-circle') . ' fa-fw mr-1"></i>' : '';
           $target = !empty($record['target']) && $record['target'] == '_blank' ? ' target="' . $record['target'] . '"' : '';
           $menu_id = $record['menu_id'];
           $label = !empty($record['icon_only']) ? '<i class="' . (!empty($record['icon']) ? $record['icon'] : 'fas fa-check-circle') . ' fa-fw mr-1"></i>' : $record['label'];
           switch($tpl) {
               case 1: // Member Area - Accordion
                   $link_badge = !empty($record['badge']) ? '<span class="nva01__label">' . $record['badge'] . '</span>' : '';
                   $child_class = !empty($extra['child_class']) ? $extra['child_class'] : 'link-list';
                   $this->link_html .= '<li class="' . (!empty($record['active']) ? 'active ' : '') . $child_class . '">';               
                   if(!empty($record['children'])) {
                       $this->link_html .= '<a href="javascript:void(0);" title="'. $record['label'] .'"' . $target . $record['link_extra'] . '>' . $icon . $label . '</a>' . $link_badge;
                       $ul_class = ' class="submenu"';
                       $this->print_tree($record['children'], true, $ul_class, $tpl, $extra);
                   } else {
                       $this->link_html .= '<a href="' . $record['url'] . '" title="'. $record['label'] .'"' . $target . $record['link_extra'] . '>' . $icon . $label . '</a>' . $link_badge;
                   }
                   $this->link_html .= '</li>';
                   break;
               case 2: // Vertical Accordion Menu
                   $active = (!empty($record['active']) ? ' active' : '');
                   $badge = !empty($record['badge']) ? '<span class="badge badge-danger badge-pill float-right">' . $record['badge'] . '</span>' : '';
                   $link_class = isset($extra['link_class']) ? $extra['link_class'] : 'list-group-item-success';
                   if(empty($record['children'])) {
                       if($record['parent_id'] == 0) {
                           $this->link_html .= '<a href="' . $record['url'] . '" class="list-group-item ' . $link_class . $active . '" data-parent="#dropdownMenu' . $tpl . $menu_id . '">' . $icon . $label . $badge . '</a>' . PHP_EOL;
                       } else {
                           $this->link_html .= '<a href="' . $record['url'] . '" class="list-group-item ' . $link_class . $active . '" data-parent="#dropdownMenu' . $tpl . $menu_id . $record['parent_id'] . '">' . $icon . $label . $badge . '</a>' . PHP_EOL;
                       }
                   } else {
                       if($record['parent_id'] == 0) {
                           $this->link_html .= '<a href="#dropdownMenu' . $tpl . $menu_id . $record['id'] . '" class="list-group-item ' . $link_class . $active . '" data-toggle="collapse" data-parent="#dropdownMenu' . $tpl . $menu_id . '">' . $icon . $label . $badge . ' <i class="fas fa-caret-down"></i></a>' . PHP_EOL;
                       } else {
                           $this->link_html .= '<a href="#dropdownMenu' . $tpl . $menu_id . $record['id'] . '" class="list-group-item" data-toggle="collapse" data-parent="#dropdownMenu' . $tpl . $menu_id . $record['parent_id'] . '">' . $icon . $label . $badge . ' <i class="fas fa-caret-down"></i></a>' . PHP_EOL;
                       }
                       $ul_class = ' class="collapse list-group-submenu" id="dropdownMenu' . $tpl . $menu_id . $record['id'] . '"';
                       $this->print_tree($record['children'], true, $ul_class, $tpl, $extra);
                   }
                   break;
               case 3:
                   $active = (!empty($record['active']) ? 'btn-theme-primary active' : 'btn-theme-secondary');
                   $this->link_html .= '<a class="nav-item nav-link ' . $active . '" href="' . $record['url'] . '" title="'. $record['label'] .'"' . $target . '>' . $icon . $label . '</a>';
                   break;
               case 4:
                   $link_class = isset($extra['link_class']) ? $extra['link_class'] : 'link-list';
                   $active = (!empty($record['active']) ? ' active' : '');
                   $this->link_html .= '<a class="' . $link_class . $active . '" href="' . $record['url'] . '" title="'. $record['label'] .'"' . $target . '>' . $icon . $label . '</a>';
                   break;
               default: // Navbar Dropdown Menu 
                   // $link_active = (current_url() == $record['url'] ? ' active' : '');
                   $active = current_url() == $record['url'] ? ' active' : '';
                   if(!empty($record['children'])) {
                       if($record['parent_id'] == 0) {
                           $this->link_html .= '<li class="nav-item dropdown">' . PHP_EOL;
                           $this->link_html .= '<a id="dropdownMenu' . $tpl . $menu_id . $record['id'] . '" href="#" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false" class="nav-link dropdown-toggle" title="'. $record['label'] .'"' . $target . '>' . $icon . $label . '</a>' . PHP_EOL;
                           $ul_id = '';
                       } else {
                           $this->link_html .= '<li class="dropdown-submenu">' . PHP_EOL;
                           $this->link_html .= '<a id="dropdownMenu' . $tpl . $menu_id . $record['id'] . '" href="#" role="button" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false" class="dropdown-item dropdown-toggle" title="'. $record['label'] .'"' . $target . '>' . $icon . $label . '</a>' . PHP_EOL;
                           $ul_id = 'aria-labelledby="dropdownMenu' . $tpl . $menu_id . $record['id'] . '" ';
                       }
                       $this->print_tree($record['children'], true, $ul_id, $tpl, $extra);
                   } else {
                       if($record['parent_id'] == 0) {
                           $this->link_html .= '<li class="nav-item' . $active . '">' . PHP_EOL;
                           $this->link_html .= '<a href="' . $record['url'] . '" class="nav-link" title="'. $record['label'] .'"' . $target . $record['link_extra'] . '>' . $icon . $label . '</a>' . PHP_EOL;
                       } else {
                           $this->link_html .= '<li>' . PHP_EOL;
                           $this->link_html .= '<a href="' . $record['url'] . '" class="dropdown-item" title="'. $record['label'] .'"' . $target . $record['link_extra'] . '>' . $icon . $label . '</a>' . PHP_EOL;
                       }
                   }
                   $this->link_html .= '</li>';
           }
       }
   }
   
   /**
    * Process Link - This method is called in the Gclosure table to prepare links
    * @param array $record
    * @return mixed[]
    */
   public function process_link($record) {
       $row = (object) $record;
       $show_link = true;
       $options = array();
       $link_extra = '';
       if(!empty($row->extra)) {
           // NOTE: Module link conditions are handled via Extra Column instead of the Link Condition column
           // Module links have a different format of condition with additional parameters.
           // Do not use the link_condition column to store the module extra paramters
           // Else multiple lines must be modified for this small change.
           // table:pages;row_id:1
           $extra_options = array_map('trim', explode(';', $row->extra ?? ''));
           if(!empty($extra_options)) {
               foreach($extra_options as $extra_option) {
                   $extra = array_map('trim', explode(':', $extra_option ?? ''));
                   // segment: / is the separator for segments e.g. segments:slug/id
                   // get: no separator use them directly e.g. popular=all_time&status=1
                   if($extra[0] == 'segment') {
                       $options[$extra[0]] = '/' . $extra[1];
                   } else {
                       $options[$extra[0]] = $extra[1];
                   }
               }
           }
       }
       $link = $options; // To store link information
       switch($row->link_url) {
           case 'home':
               $url = site_url();
               break;
           case 'separator':
               $url = '';
               break;
           default:
               if(substr($row->link_url, 0, 1) == '#') { // modal link
                   $url = $row->link_url;
                   $link_extra = ' data-toggle="modal" data-target="' . $url . '"';
               } else if(empty($row->link_url)) {
                   $url = 'javascript:void(0);';
               } else {
                    $url = ($row->gc_url == 1 ? gc_url_format($row->link_url) : (valid_url($row->link_url) ? $row->link_url : site_url($row->link_url)));
               }
       }
       if($row->gc_url == 1 && !empty($options['table']) && !empty($options['row_id'])) {
           $row_info = $this->db->where('id', $options['row_id'])->get($options['table'])->row();
           if($row_info) {
               switch($options['table']) {
                   case 'pages':
                       $url = gc_url_format($row->link_url, $row_info->slug);
                       // Active Link Condition
                       $link_condition[] = array(
                           'id' => 3,
                           'name' => $row_info->slug
                       );
                       break;
                   case 'products':
                   case 'product_categories':
                   case 'blog_categories':
                       $url = gc_url_format($row->link_url, $row_info->slug, $row_info->id);
                       break;
               }
           }
           unset($options['table']);
           unset($options['row_id']);
       }
       // GET parameters
       if(!empty($link['get'])) {
           if(strpos($link['get'], 'md=') !== false) {
                $get_data = explode('md=', $link['get']);
                $module_encoded_value = $this->crypt->encode($get_data[1]);
                $link['get'] = str_replace($link['get'], 'md=' . $get_data[1], 'md=' . $module_encoded_value);
           }
           $url .= '?' . $link['get'];
       }
       // segment parameters
       if(!empty($link['segment'])) {
           $url .= $link['segment'];
       }
       // prepare link conditions format
       $link_condition = array();
       if(!empty($row->link_condition)) {
           // get all options
           $options_array = array_map('trim', explode('|', $row->link_condition));
           if($options_array) {
               foreach($options_array as $option_array) {
                   $options_fields = array_map('trim', explode(';', $option_array ?? ''));
                   if($options_fields) {
                       $option_result = array();
                       foreach($options_fields as $option_field) {
                           $option_value = array_map('trim', explode(':', $option_field ?? ''));
                           $option_result[$option_value[0]] = (isset($option_value[1]) ? $option_value[1] : '');
                       }
                       // $option[$option_result['id']] = isset($option_result['string']) &&  $option_result['string'] == 1 && !empty($option_result['name']) ? gc_uri_string($option_result['name']) : $option_result['name'];
                       $link_condition[] = $option_result;
                   }
               }
           }
       }
       
       // counts_param and counts_condition are handled using params
       $params = !empty($row->params) ? json_decode($row->params, true) : array();
       if(!empty($params)) {
           foreach($params as $param) {
               if(!empty($param['key'])) {
                   $link[$param['key']] = $param['value'];
                   /*switch($param['key']) {
                       case 'counts_param':
                       case 'counts_condition':
                           $link[$param['key']] = $param['value'];
                           break;
                   }*/
               }
           }
       }
       
       $link['id'] = $row->id;
       $link['menu_id'] = $row->menu_id;
       $link['menu'] = !empty($row->menu_name) ? ($row->menu_lang == 1 ? lang($row->menu_name) : $row->menu_name) : lang('label_none');
       $link['parent_id'] = $row->parent_id;
       $link['permission'] = $row->permission;
       $link['link_condition'] = $link_condition;
       $link['label'] = ($row->name_lang == 1 ? lang($row->name) : $row->name);
       $link['url'] = $url;
       $link['target'] = $row->target;
       $link['icon'] = $row->icon;
       $link['counts_param'] = !isset($link['counts_param']) ? '' : $link['counts_param'];
       $link['counts_condition'] = !isset($link['counts_condition']) ? '' : $link['counts_condition'];
       $link['created_on'] = $row->created_on; // Sitemap
       $link['modified_on'] = $row->modified_on; // Sitemap
       if(isset($row->children)) {
           $link['children'] = $row->children;
       }
       $link['parent'] = isset($row->parent) ? $row->parent : '';
       $link['level'] = isset($row->level) ? $row->level : 0;
       $link['link_extra'] = $link_extra; // modal parameters
       return $link;
   }
   
   /**
    * To display all links to the admin
    * @return multitype:multitype:unknown
    */
   public function get_links_order($menu_id = 0) {
       $where = array();
       if($menu_id != 0) {
           $where['L.menu_id'] = $menu_id;
       }
       $this->db->from('links L');
       $result = $this->db->select('L.id, L.parent_id, L.name, L.name_lang')->where($where)->group_by('L.id')->order_by('L.priority')->get()->result_array();
       return $this->prepareList($result);
   }
   
   /**
    * Prepare list to get parents and its children
    * @param array $items
    * @param number $pid
    * @return multitype:multitype:unknown
    */
   private function prepareList(array $items, $pid = 0) {
       $output = array();
       
       if(!empty($items)) {
           # loop through the items
           foreach ($items as $item) {
               $item['name'] = $item['name_lang'] == 1 ? lang($item['name']) : $item['name'];
               unset($item['name_lang']);
               # Whether the parent_id of the item matches the current $pid
               if((int) $item['parent_id'] == $pid) {
                   
                   # Call the function recursively, use the item's id as the parent's id
                   # The function returns the list of children or an empty array()
                   if ($children = $this->prepareList($items, $item['id'])) {
                       # Store all children of the current item
                       $item['children'] = $children;
                   }
                   
                   # Fill the output
                   $output[] = $item;
               }
           }
       }
       return $output;
   }
   
   /**
    * After Link Save
    * @param number $id
    * @param string $action_type
    * @param array $record
    */
   public function after_link_save($id, $action_type = 'insert', $record = array()) {
       $record = $this->db->where('id', $id)->get($this->table_name)->row();
       if($record) {
           $parent_id = $record->parent_id;
           $this->load->library('goclixy/gclosure');
           $gclosure = new Gclosure('links', 'link_relations');
           $gclosure->delete($id, false); // do not delete from the links table
           // save hierarchy
           $gclosure->add($id, ($parent_id == 0 ? $id : $parent_id));
       }
   }
   
   /**
    * Find Layout ID on layout load
    * @param string $url
    * @return array|NULL[]|NULL[][]
    */
   public function get_all_links($url = null) {
       $records = $this->db->get('links')->result();
       $links = array();
       if($records) {
           foreach($records as $record) {
               $params = array();
               if(!empty($record->params)) {
                   $saved_params = json_decode($record->params, true);
                   if(!empty($saved_params)) {
                       foreach($saved_params as $param) {
                           switch($param['key']) {
                               case 'background_image':
                                   $params[$param['key']] = valid_url($param['value']) ? $param['value'] : base_url('assets/images/' . $param['value']);
                                   break;
                               default:
                                   $params[$param['key']] = $param['value'];
                           }
                       }
                   }
               }
               $links[$this->process_link($record)['url']] = array('id' => $record->id, 'layout_id' => $record->layout_id, 'description' => $record->description, 'params' => $params);
           }
       }
       if(!empty($url)) {
           return isset($links[$url]) ? $links[$url] : array();
       }
       return $links;
   }
}