<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/

/**
 * Reports controller
 */
class Reports extends Admin_Controller {
	protected $permissionDelete = 'Listing_Views.Reports.Delete';
	protected $permissionView   = 'Listing_Views.Reports.View';
	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct() {
		parent::__construct();

		$this->auth->restrict($this->permissionView);
		$this->load->model('listing_views_model', null, true);
		$this->lang->load('listing_views');
		
		Template::set_block('sub_nav', 'reports/_sub_nav');

		Assets::add_module_js('listing_views', 'listing_views.js');
	}
	
	/**
	 * Displays a list of form data.
	 *
	 * @return void
	 */
	public function index($offset = 0) {		
		// Deleting anything?
		if (isset ( $_POST ['delete'] )) {
			$checked = $this->input->post ( 'checked' );			
			if (is_array ( $checked ) && count ( $checked )) {
				$result = FALSE;
				foreach ( $checked as $pid ) {
					$result = $this->listing_views_model->delete ( $pid );
				}
				
				if ($result) {
					Template::set_message ( count ( $checked ) . ' ' . sprintf(lang('msg_action_success'), lang('listing_views'), lang('label_deleted')), 'success' );
				} else {
					Template::set_message ( sprintf(lang ( 'msg_delete_failure' ), lang('listing_views')) . $this->listing_views_model->error, 'danger' );
				}
			}
		}
		
		$where = "";
		$searchterm = $this->gc->search('lvm', $this->uri->segment(6), '/reports/listing_views');
		if(!empty($searchterm)) {
			$where .= " LV.page LIKE '%" . $searchterm . "%' OR LV.keyword LIKE '%" . $searchterm . "%' OR LV.listing_id = '$searchterm'";
		}
		
		$sort_by = "LV.id DESC";
		$order_by = $this->gc->sort('lvm');
		if(!empty($order_by)) {
			$sort_by = $order_by;
		}
		
		$records = $this->listing_views_model->get_listing_views($offset, $this->limit, $where, $sort_by);
		
		$total_records = $this->listing_views_model->get_listing_views($offset, $this->limit, $where, $sort_by, 1);
		
		$this->load->library('pagination');
		$pager['base_url'] = site_url(SITE_AREA ."/reports/listing_views/index/");
		$pager['total_rows'] = $total_records;
		$pager['per_page'] = $this->limit;
		$pager['uri_segment']	= 5;
		
		$this->pagination->initialize($pager);

		Template::set('records', $records);
		Template::set('toolbar_title', lang('listing_views_manage'));
		Template::render();
	}
	
	/**
	 * Advanced Search
	 * Useful to find out clicks on a particular listing between a specified date from a specified page.
	 * @param number $offset
	 */
	public function advanced_search($offset = 0) {
		// DATE TIME PICKER
		Assets::add_js('moment.js');
		Assets::add_css ( 'bootstrap-datetimepicker.min.css' );
		Assets::add_js ( 'bootstrap-datetimepicker.min.js' );
		if(isset($_POST['search']) || ($this->session->userdata('lvs_from') && $this->session->userdata('lvs_to'))) {
		    if (isset($_POST['search'])) {
		        $this->gcrud->stats(array('module' => 'listing_views'));
				$this->form_validation->set_rules('listing_id', 'lang:label_listing_id', 'trim');
				$this->form_validation->set_rules('page', 'lang:label_page', 'trim');
				$this->form_validation->set_rules('keyword', 'lang:label_keyword', 'trim');			
				$this->form_validation->set_rules('date_from', 'lang:label_from', 'trim|valid_date|required');
				$this->form_validation->set_rules('date_to', 'lang:label_to', 'trim|valid_date|compare_dates[date_from]|required');
				
				if ($this->form_validation->run() === true) {
					$listing_id = $this->input->post('listing_id');
					$page = $this->input->post('page');
					$keyword = $this->input->post('keyword');				
					$from = $this->input->post('date_from');
					$to = $this->input->post('date_to');
					// Save values in Session for Pagination
					$this->session->set_userdata('lvs_listing_id', $listing_id);
					$this->session->set_userdata('lvs_page', $page);
					$this->session->set_userdata('lvs_keyword', $keyword);
					$this->session->set_userdata('lvs_from', $from);
					$this->session->set_userdata('lvs_to', $to);
					// IF USER PERFORMED SEARCH FROM PAGINATION PAGE
					if($this->uri->segment(5)) {
						Template::redirect(site_url(SITE_AREA . "/reports/listing_views/advanced_search"));
					}
				}	
			} else if($this->session->userdata('lvs_from') && $this->session->userdata('lvs_to')) {
				$listing_id = $this->session->userdata('lvs_listing_id');
				$page = $this->session->userdata('lvs_page');
				$keyword = $this->session->userdata('lvs_keyword');
				$from = $this->session->userdata('lvs_from');
				$to = $this->session->userdata('lvs_to');
			}
			
			if(!empty($from) && !empty($to)) {
				$records = $this->listing_views_model->search_listing_views($offset, $this->limit, $listing_id, $keyword, $page, $from, $to);
				
				$total_records = $this->listing_views_model->search_listing_views($offset, $this->limit, $listing_id, $keyword, $page, $from, $to, 1);
				
				if($total_records == 0) {
					Template::set_message(lang('error_no_record_found'), 'error');
				}
				$this->load->library('pagination');
				$pager['base_url'] = site_url(SITE_AREA ."/reports/listing_views/advanced_search/");
				$pager['total_rows'] = $total_records;
				$pager['per_page'] = $this->limit;
				$pager['uri_segment']	= 5;
				
				$this->pagination->initialize($pager);
				Template::set(array(
					'listing_id' => $listing_id,
					'page' => $page,
					'keyword' => $keyword,
					'from' => $from,
					'to' => $to				
				));
				
				Template::set('records', $records);
			}
		}
		Template::set('toolbar_title', lang('advanced_search'));
		Template::render();
	}
	
	/**
	 * Delete Records from Listing Views Table
	 */
	public function delete_records() {
		// DATE TIME PICKER
		Assets::add_js('moment.js');
		Assets::add_css ( 'bootstrap-datetimepicker.min.css' );
		Assets::add_js ( 'bootstrap-datetimepicker.min.js' );
		$this->auth->restrict($this->permissionDelete);
		if (isset($_POST['submit'])) {
		    $this->gcrud->stats(array('module' => 'listing_views'));
			$this->form_validation->set_rules('date_from', 'lang:label_from', 'trim|valid_date|required');
			if ($this->form_validation->run() === true) {
				// DELETE FROM `listing_views` WHERE date(modified_on) < '2015-11-19'				
				$result = $this->db->where('date(modified_on) < ', $this->input->post('date_from'))->count_all_results('listing_views');
				if($result == 0) {
					Template::set_message(lang('error_no_record_found'), 'error');
				} else {
					$this->db->delete('listing_views', array('date(modified_on) < ' => $this->input->post('date_from')));
					Template::set_message ( $result . ' ' . lang ( 'delete_success' ), 'success' );
				}				
			}			
		}
		Template::set('toolbar_title', lang('listing_views_delete'));
		Template::render();
	}
}