<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Migration_Install_listing_views extends Migration
{
	/**
	 * The name of the database table
	 *
	 * @var String
	 */
	private $table_name = 'listing_views';

	/**
	 * The table's fields
	 *
	 * @var Array
	 */
	private $fields = array(
		'id' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'auto_increment' => TRUE
		),
		'user_id' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'null' => FALSE,
			'default' => 0
		),
		'listing_id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => FALSE
		),
		'page' => array(
			'type' => 'CHAR',
			'constraint' => 20,
			'null' => FALSE
		),
		'keyword' => array(
			'type' => 'VARCHAR',
			'constraint' => 100,
			'null' => FALSE
		),
		'times' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => FALSE,
			'default' => 1
		),
		'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'modified_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP'
	);

	/**
	 * Install this migration
	 *
	 * @return void
	 */
	public function up()
	{
		$this->dbforge->add_field($this->fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('listing_id', false);
		$this->dbforge->add_key('modified_on', false);
		$this->dbforge->create_table($this->table_name);
		// CHANGE ENGINE FROM INNODB TO MYISAM
		$change_engine_sql = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " ENGINE = MyISAM";
		$this->db->query ($change_engine_sql);
		// ADD UNIQUE INDEX
		$sql = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " ADD UNIQUE INDEX `listing_page_keyword` (`listing_id`, `page`, `keyword`)";
		$this->db->query ($sql);
	}

	//--------------------------------------------------------------------

	/**
	 * Uninstall this migration
	 *
	 * @return void
	 */
	public function down()
	{
		$this->dbforge->drop_table($this->table_name);
	}

	//--------------------------------------------------------------------

}