<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * content controller
 */
class Content extends Admin_Controller
{
	protected $permissionCreate = 'Listings.Content.Create';
	protected $permissionDelete = 'Listings.Content.Delete';
	protected $permissionEdit   = 'Listings.Content.Edit';
	protected $permissionView   = 'Listings.Content.View';
	
	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct()
	{
		parent::__construct();
		$this->auth->restrict($this->permissionView);
		$this->load->model('listings_model', null, true);
		$this->lang->load('listings');
		$this->load->library('gc');
		
		//CREATE SLUG
		$config = array(
				'field' => 'slug',
				'title' => 'name',
				'table' => $this->db->dbprefix .'listings',
				'id' => 'id',
		);
		$this->load->library('slug', $config);
		
		//LOAD ALL MODELS FOR FILTERING AND OTHER OPERATIONS
		$this->load->model('business_hours_model');		
		$this->load->model('roles/role_model');
		$this->load->model('categories/categories_model');
		$this->load->model('locations/countries_model');
		$this->load->model('tags/tags_model');
		
		/*
		Assets::add_js(array(
			'js/editors/ckeditor/ckeditor.js',
		));
		$this->lang->load('datatable');
		
		//SEARCH LISTING
		//Assets::add_js(Template::theme_url('js/bootstrap.js'));
		
		Assets::add_js( array ( Template::theme_url('js/jquery.dataTables.min.js' )) );
		Assets::add_js( array ( Template::theme_url('js/bootstrap-dataTables.js' )) );
		Assets::add_css( array ( Template::theme_url('css/datatable.css') ) ) ;
		Assets::add_css( array ( Template::theme_url('css/bootstrap-dataTables.css') ) );
		*/

		// Common JS file which holds Business Hours and other modules JS code
		Assets::add_module_js('listings', 'listings.js');
		
		// Show New Claims Count on Toolbar Title
		$this->load->model('listings/claims_model');
		$new_claims = $this->claims_model->count_by(array('status' => 0));
		Template::set('new_claims', $new_claims);
		
		/* --------------------------------- */		
		Template::set_block('sub_nav', 'content/_sub_nav');
	}
	
	/**
	 * Module Specific Summary
	 * @param string $module
	 */
	public function summary($module = 'listings') {
	    // DATE TIME PICKER
	    Assets::add_js('moment.js');
	    Assets::add_css('bootstrap-datetimepicker.min.css' );
	    Assets::add_js('bootstrap-datetimepicker.min.js' );
	    Assets::add_css('chart.css');
	    Assets::add_js('Chart.min.js');
	    $this->session->unset_userdata('chart_query_params');
	    $record = $this->listings_model->summary($module);
	    $table = $module;
	    $where = '';
	    
	    // tabular blocks
	    $limit = 10;
	    $tabular_blocks = array();
	    $tabular_blocks['top_visited'] = $this->listings_model->_reports($module, 0, $limit, null, 'hits DESC', 'M');
	    $tabular_blocks['recently_added'] = $this->listings_model->_reports($module, 0, $limit, null, 'created_on DESC', 'M');
	    $tabular_blocks['recently_viewed'] = $this->listings_model->_reports($module, 0, $limit, null, 'last_viewed_on DESC', 'M');
	    
	    $column = 'counter';
	    if($module == 'listings' && $this->auth->role_id() != 1) { // display franchise specific listinsg
	        $where = "franchise_id = " . $this->auth->user_id();
	    }
	    $row_by_months = $this->gcrud->chart('months', array('status' => 1, 'where' => $where,'prefix' => 'M', 'table' => $table, 'field' => $column));
	    Assets::add_js($this->load->view('common/_chart_js', array('field_id' => 'sixMonthChart', 'labels' => $row_by_months['labels'], 'data' => $row_by_months['data'], 'label' => lang('total_rows'), 'total' => $row_by_months['total'], 'with_dates' => false, 'table' => $table), true), 'inline');
	    $result = $this->gcrud->chart('current_month', array('status' => 1, 'where' => $where, 'prefix' => 'M', 'table' => $table, 'field' => $column));
	    Assets::add_js($this->load->view('common/_chart_js', array('field_id' => 'monthlyChart', 'labels' => $result['labels'], 'data' => $result['data'], 'total' => $result['total'], 'label' => lang('total_rows'), 'with_dates' => true, 'table' => $table), true), 'inline');
	    // $record->tabular_blocks = $tabular_blocks;
	    //$record->recently_viewed = $this->forums_model->manage(0, null, 'M.last_viewed DESC', 20, 0, array('module' => 'views'));
	    Template::set(array(
	        'module' => $module,
	        'tabular_blocks' => $tabular_blocks,
	        'toolbar_title' => lang($module) . ' - ' . lang('gc_menu_summary'),
	        'record' => $record
	    ));
	    Template::render();
	}

	/* ------------------------ LISTING MODULE ---------------------------------------- */
	/**
	 * Displays a list of form data.
	 * Status: 0 -> Inactive, 1 -> Active, 2 -> Expired, 3 -> Spammed
	 * @return void
	 */
	
	public function index($filter='all', $offset=0) {		
	    if(!empty($_POST['pdf'])) {
	        $this->gcrud->stats(array('module' => 'listings'));
	        // one listing pdf
	        $this->gemailer->sms_email_pdf('pdf', array('template_table' => 'module_email_templates', 'module_table' => 'listings', 'action_type' => 'share', 'with_default' => 1, 'id' => $_POST['pdf']));
	    }
	    
	    // download in PDF and XLS
	    $this->gcrud->pdf_excel_download('listings', $offset, $this->limit);
	    
	    // redirect back and forth between pages
	    $this->session->set_userdata('listings_last_visited_page', current_url());
		// Advanced Search Load CSS and JS files
		Assets::add_css('selectize.bootstrap3.css');
		Assets::add_js('selectize.min.js');
		Assets::add_js('selectize-search.js');
		Assets::add_js('tinymce/tinymce.min.js');
		
		// Send Custom/Bulk SMS Code
		Assets::add_js('validator.min.js');
		Assets::add_js('gc-admin.js');
		
		// category fields url redirection
		$this->session->set_userdata('cancel_url', current_url());
		
		// Check whether any SMS gateway is active or not
		$this->load->model('sms_gateways/sms_gateways_model');
		$this->load->model('emailer/emailer_model');
		$send_sms = $this->sms_gateways_model->count_by(array('active' => 1));
		$available_variables =  $this->emailer_model->share_variables('listings', false, ', ');
		Template::set(array(
			'send_sms' => ($send_sms == 0 ? false : true),
			'module' => 'listings',
			'subject_variables' => $available_variables['subject'],
			'message_variables' => $available_variables['message']
		));
		
		//Assets::add_js($this->load->view('content/listings_js', null, true), 'inline');
		// Fetch having we might want to filter on
		$roles = $this->role_model->select('role_id, role_name')->where('deleted', 0)->find_all();
		$ordered_roles = array();
		foreach ($roles as $role) {
			$ordered_roles[$role->role_id] = $role;
		}
		Template::set('roles', $ordered_roles);	
		
		// Package wise filtering
		$this->load->model('packages/packages_model');
		$packages = $this->packages_model->select('id, title')->find_all_by(array('deleted' => 0));
		$ordered_packages = array();
		foreach ($packages as $package) {
			$ordered_packages[$package->id] = $package;
		}
		Template::set('packages', $ordered_packages);
		
		// Allow Admin to Clone A Row
		$this->clone_row('listings');
		
		// Category Filteration
		$category_filter = $this->categories_model->filter(1);
		
		// Change Status
		if (isset($_POST['change_status']))	 {
		    $this->gcrud->stats(array('module' => 'listings'));
			$business_id = $_POST['change_status'];
			$new_status = (int) $_POST['listing_status_' . $business_id];
			$notify_customer = !empty($_POST['notify_customer_' . $business_id]) ? (int) $_POST['notify_customer_' . $business_id] : 0;
			
			$sql = "UPDATE " . $this->db->dbprefix . "listings SET status = " . $new_status . " WHERE id = '" . $business_id ."'";
			$this->db->query($sql);
				
			// Notify Business Owner
			if($notify_customer == 1) {
			    $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'module_email_templates', 'module_table' => 'listings', 'action_type' => 'status_update', 'with_default' => 1, 'id' => $business_id));
			}

			Template::set_message(sprintf(lang('msg_action_success'), lang('label_listing'), lang('label_updated')), 'success');
			Template::redirect(current_url());
		}			
				
		// Do we have any actions?
		if (isset($_POST['activate']))    $action = '_activate';
		if (isset($_POST['deactivate']))  $action = '_deactivate';
		if (isset($_POST['spam']))        $action = '_spam';
		if (isset($_POST['delete']))      $action = '_delete';
		if (isset($_POST['purge']))       $action = '_purge';
		if (isset($_POST['restore']))     $action = '_restore';		
	
		if (isset($action)) {
			$checked = $this->input->post('checked');
	
			if (!empty($checked)) {
				foreach($checked as $listing_id) {
					$this->$action($listing_id);
				}
			} else {
				Template::set_message(lang('ls_empty_id'), 'error');
			}
		}
	
		// Actions done, now display the view
		$table_prefix = 'M.';
		$where = $table_prefix . 'deleted = 0';
		
		// advanced filter
		$advanced_filter = array('date_from', 'date_to', 'user_id', 'package_id', 'location', 'category_id', 'tag_id');
		// location filter
		Assets::add_css('selectize.bootstrap3.css');
		Assets::add_js('selectize.min.js');
		Assets::add_js('selectize-search.js');
		Assets::add_js('selectize-location.js');
		$extra = $this->gcrud->advanced_filter(array('advanced_filter' => $advanced_filter, 'advanced_filter_date' => true), 'listings');
		if(!empty($extra['query_where'])) {
		    $where .= " AND " . $extra['query_where'];
		}
	
		// Filters
		if (preg_match('{first_letter-([A-Z])}', $filter, $matches)) {
			$filter_type = 'first_letter';
			$first_letter = $matches[1];
		} elseif (preg_match('{role_id-([0-9]*)}', $filter, $matches)) {
			$filter_type = 'role_id';
			$role_id = (int) $matches[1];
		} elseif (preg_match('{package_id-([0-9]*)}', $filter, $matches)) {
			$filter_type = 'package_id';
			$package_id = (int) $matches[1];
		} else {
			$filter_type = $filter;
		}
		switch($filter_type) {
			case 'claimed':
			    $where .= " AND " . $table_prefix . "claimed = 1";
				break;
				
			case 'unclaimed':
			    $where .= " AND " . $table_prefix . "claimed = 0";
				break;
			
			case 'inactive':
			    $where .= " AND " . $table_prefix . "status = 0";
				break;
		
			case 'expired':
			    $where .= " AND " . $table_prefix . "status = 2";
				break;
	
			case 'spammed':
			    $where .= " AND " . $table_prefix . "status = 3";
				break;
				
			case 'featured':
			    $where .= " AND " . $table_prefix . "featured = 1";
				break;
	
			case 'deleted':
			    $where = $table_prefix . "deleted = 1";
				break;
				
			case 'logo':
				// $where = "logo_url IS NOT NULL AND logo_url <> ''"; //LOGO IS NOT NULL
				Template::set('filter_having', lang('logo')); //DISPLAY HAVING IN SELECTION
				break;				
	
			case 'role_id':
			    $where .= " AND " . $table_prefix . "user_id = ". $role_id;	
				foreach ($roles as $role) {
					if ($role->role_id == $role_id) {
						Template::set('filter_role', $role->role_name);
						break;
					}
				}
				break;
			
			case 'package_id':
			    $where .= " AND " . $table_prefix . "package_id = ". $package_id;
					foreach ($packages as $package) {
						if ($package->id == $package_id) {
							Template::set('filter_packages', $package->title);
							break;
						}
					}
					break;
	
			case 'first_letter':
				$where = "SUBSTRING( LOWER(" . $table_prefix . "title), 1, 1)= '". $first_letter ."'";
				break;
	
			case 'all':
				// Nothing to do
				break;
	
			default:
				$having_filter = $this->listings_model->having_filter($filter_type);
				if(!empty($having_filter['where'])) {
				    $where = $having_filter['where'];
				}
				if(!empty($having_filter['having'])) {
				    Template::set('filter_having', lang($having_filter['having']));
				}
		}
		
		$searchterm = $this->gc->search('listings', $this->uri->segment(5), '/content/listings/');
		if(!empty($searchterm)) {
			$where .= " AND (" . $table_prefix . "id LIKE '%" .$searchterm . "%' OR " . $table_prefix . "user_id LIKE '%" .$searchterm . "%' OR " . $table_prefix . "title LIKE '%" .$searchterm . "%' OR " . $table_prefix . "email LIKE '%" .$searchterm . "%' OR " . $table_prefix . "created_on LIKE '%" .$searchterm . "%') ";
		}
		
		// Perform the Advanced Search - Country, State and City Filter
		$search_sql = $this->listings_model->advanced_search($table_prefix);		
		if(!empty($search_sql)) {
			$where .= " AND " . implode(' AND ', $search_sql);
		}
		
		// Sorting
		$order_by = 'id DESC';
		$sort_by = $this->gc->sort('listings');
		if(!empty($sort_by)) {
			$order_by = $sort_by;
		}
		
		if($this->auth->role_id() == 7) { // franchise can view only their own listings
		    $where .= " AND " . $this->session->userdata('franchise_where');
		}
		$records = $this->listings_model->get_admin_listings(0, $offset, $this->limit, $where, $order_by, $category_filter);
		
		// Used for PDF/XLS Reporting
		$this->session->set_userdata('listings_pdf_xls_params', array('where' => $where, 'order_by' => $order_by, 'query_extra' => array('module' => 'listings')));
			
		// Pagination
		$this->load->library('pagination');
		$total_listings = $this->listings_model->get_admin_listings(1, $offset, $this->limit, $where, $order_by, $category_filter); //$this->listings_model->count_all();
	
		$pager['base_url']    = site_url(SITE_AREA ."/content/listings/index/$filter/");
		$pager['total_rows']  = $total_listings;
		$pager['per_page']    = $this->limit;
		$pager['uri_segment'] = 6;
	
		$this->pagination->initialize($pager);
		
		Template::set(array(
			'filtered_results' => $total_listings
		));
		Template::set('records', $records);
		Template::set('index_url', site_url(SITE_AREA .'/content/listings/index') .'/');
		Template::set('filter_type', $filter_type);
	
		Template::set('toolbar_title', lang('manage_listings'));
		Template::render();
	
	}//end index()
	
	/**
	 * Allow Admin to Clone A Row
	 * @param string $module
	 */
	private function clone_row($module) {
		// CREATE LISTING CLONE
		if (!empty ( $_POST ['clone'] )) {
		    $this->gcrud->stats(array('module' => $module));
			$clone_id = $_POST ['clone'];
			$el = $this->db->get_where($module, array('id' => $clone_id))->row();
			if($el) {
				$clone_data = array();
				foreach($el as $key => $value) {
					switch($key) {
						case 'id':
						case 'created_by':
						case 'modified_by':
						case 'modified_on':
						case 'modified_ip':
							break;
						case 'created_on':
							$clone_data[$key] = date('Y-m-d H:i:s');
							break;
						case 'created_ip':
							$clone_data[$key] = $this->input->ip_address();
							break;
						case 'title':
							$clone_data[$key] = $value .' '.lang('label_copy');
							break;
						default:
							$clone_data[$key] = $value;
					}
				}
				$this->db->insert($module, $clone_data);
				$inserted_id = $this->db->insert_id();
				if (is_numeric ( $inserted_id )) {
					if($module == 'listings') {
						$this->gc->generate_qrcode($inserted_id); // Generate NEW QRCODE						
					}
					// also clone categories, logo, cover_image, custom_fields and tags
					$this->listings_model->copy_row($module, $clone_id, $inserted_id);
					Template::redirect(current_url());
				} else {
					Template::set_message(lang('msg_clone_error'), 'error');
				}
			}
		}
	}

	/**
	 * Allow admin to add new listing
	 */
	public function create() {		
		$this->auth->restrict($this->permissionCreate);
		$this->listings_model->business('admin', 0, $this->auth->user_id());	
	}
	
	/**
	 * Allow admin to update listing
	 */
	public function edit() {	    
	    // $id = (int) $this->input->get('id');
		$id = $this->uri->segment(5);
		if (empty($id)) {
			Template::set_message(lang('invalid_id'), 'error');
			redirect(SITE_AREA .'/content/listings');
		}
		$this->listings_model->business('admin', 0, $this->auth->user_id(), $id);
	}

	/**
	 * Show google map on listing create and update
	 * @param string $latitude
	 * @param string $longitude
	 */
	public function google_map($latitude = NULL, $longitude = NULL){
		$this->load->library('googlemaps');
		$config = array();
		//SET LATITUDE AND LONGITUDE IF NOT EDIT
		if(($latitude == NULL) && ($longitude == NULL)) {
			if((settings_item('lst.latitude') != '') && (settings_item('lst.longitude') != '')) {
				$latitude = settings_item('lst.latitude');
				$longitude = settings_item('lst.longitude');
			} else {
				$ip = $_SERVER['REMOTE_ADDR'];
				//MADE USE OF FREE PLUGIN
				$geopluginURL='http://www.geoplugin.net/php.gp?ip=49.249.196.21'; //.$ip;
				$unarr= file_get_contents($geopluginURL);     // Get File Contents
				$AddArr = unserialize($unarr);    // Get PHP values from file contents			
				$latitude = $AddArr['geoplugin_latitude'];  // City Name
				$longitude = $AddArr['geoplugin_longitude'];
			}
		}
		$config['center'] = "$latitude, $longitude";
		$config['zoom'] = 12;
		$config['places'] = TRUE;
		$this->googlemaps->initialize($config);
	
		$marker = array();
		$marker['position'] = "$latitude, $longitude";
		$marker['draggable'] = true;
		$marker['ondragend'] = '$("#latitude").val(event.latLng.lat()), $("#longitude").val(event.latLng.lng())';
		$this->googlemaps->add_marker($marker);	
		return $this->googlemaps->create_map();
	}
	
	/**
	 * Get Subcategories using AJAX
	 * @param number $parent_id
	 */
	public function get_sub_categories() {
		$parent_id = $this->input->post('id');
		if(!empty($parent_id)) {
			$suboptions = $this->categories_model->get_sub_categories($parent_id);
		} else {
			$suboptions = '';
		}
		echo json_encode($suboptions);
	}
	
	/**
	 * Get States and Cities using AJAX
	 * @return string - location information in HTML format
	 */
	public function loadData()
	{
		$loadType=$_POST['loadType'];
		$loadId=$_POST['loadId'];
	
		$result=$this->listings_model->getData($loadType,$loadId);
		$HTML="";
	
		if($result){
			foreach($result as $list){
				$HTML.="<option value='".$list->id."'>".$list->name."</option>";
			}
		}
		echo $HTML;
	}
	

	/**
	 * Spam a listing
	 * @param int    $listing_id     Listing to ban
	 * @return void
	 */
	private function _spam($listing_id)
	{
		$this->listings_model->update_listing($listing_id, array('status'=>3));	
		Template::set_message(sprintf(lang('msg_action_success'), lang('listing_s'), lang('label_spammed')), 'success');
	}//end _spam()
	
	/**
	 * Delete listings
	 * @access private
	 * @param int $id Listing to delete
	 * @return void
	 */
	private function _delete($id)
	{
		$checked = $this->input->post('checked');
		
		$result = $this->listings_model->delete($id);
		if ($result) {		    
		    // Also set the Business Queries, Favorites and Comments status to deleted
		    $this->gc->module_soft_delete('listings', $id);
			Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), lang('listing_s'), lang('label_deleted')), 'success');
		} else {
			Template::set_message(sprintf(lang('msg_delete_failure'), lang('listing_s'))  . $this->listings_model->error, 'danger');
		}
		
		/*if (is_array($checked) && count($checked)) {
			$result = FALSE;
			foreach ($checked as $pid) {
				
			}
		
			
		}*/
		
	}//end _delete()
	
	/**
	 * Purge the selected listings which are already marked as deleted
	 * @access private
	 * @param int $id Listing to purge
	 * @return void
	 */
	private function _purge($id) {		
		$this->listings_model->purge_records($id);
		// Template::set_message(lang('action_purged'), 'success');
		Template::set_message(sprintf(lang('msg_action_success'), lang('listing_s'), lang('label_deleted')), 'success');
	}//end _purge()
	
	/**
	 * Restore the deleted listing	 *
	 * @return void
	 */
	private function _restore($id)
	{
		if ($this->listings_model->update_listing($id, array('deleted'=>0))) {		    
		    // Also set the Business Queries, Favorites and Comments status to restored
		    $this->gc->module_soft_delete('listings', $id, 0);
			Template::set_message(sprintf(lang('msg_action_success'), lang('listing_s'), lang('label_restored')), 'success');
		} else {
			Template::set_message(lang('restored_error'). $this->listings_model->error, 'danger');
		}
	
	}//end restore()

	//--------------------------------------------------------------------
	// ACTIVATION METHODS
	//--------------------------------------------------------------------
	/**
	 * Activates selected listings.
	 * @access private
	 * @param int $listing_id
	 * @return void
	 */
	private function _activate($listing_id)
	{
		$this->listing_status($listing_id,1,0);
		Template::set_message(sprintf(lang('msg_action_success'), lang('listing_s'), lang('label_activated')), 'success');
	}//end _activate()

	/**
	 * Deactivates selected listing.
	 * @access private
	 * @param int $listing_id
	 * @return void
	 */
	private function _deactivate($listing_id)
	{
		$this->listing_status($listing_id,0,0);
		Template::set_message(sprintf(lang('msg_action_success'), lang('listing_s'), lang('label_deactivated')), 'success');
	
	}//end _deactivate()
	
	
	/**
	 * Activates or deavtivates listing from the listing dashboard.	 *
	 * @access private
	 *
	 * @param int $listing_id       Listing ID int
	 * @param int $status        1 = Activate, -1 = Deactivate
	 * @param int $supress_email 1 = Supress, All others = send email
	 *
	 * @return void
	 */
	private function listing_status($listing_id = false, $status = 1, $supress_email = 0)
	{
		$supress_email = (isset($supress_email) && $supress_email == 1 ? true : false);
	
		if ($listing_id !== false && $listing_id != -1)
		{
			$result = false;
			$type = '';
			if ($status == 1) {
				$result = $this->listings_model->listing_activation($listing_id);
				$type = lang('gc_action_activate');
			} else {
				$result = $this->listings_model->listing_deactivation($listing_id);
				$type = lang('gc_action_deactivate');
			}
	
			$listing = $this->listings_model->find($listing_id);
			//$log_name = $this->settings_lib->item('auth.use_own_names') ? $this->current_user->username : ($this->settings_lib->item('auth.use_usernames') ? $listing->username : $listing->email);
	
			//log_activity($this->current_user->id, lang('us_log_status_change') . ': '.$log_name . ' : '.$type."ed", 'users');
	/*
			if ($result)
			{
				$message = lang('us_active_status_changed');
				if ($status == 1 && !$supress_email)
				{
					// Now send the email
					$this->load->library('emailer/emailer');
	
					$site_title = $this->settings_lib->item('site.title');
	
					$data = array
					(
							'to'		=> $this->listing_model->find($listing_id)->email,
							'subject'	=> lang('us_account_active'),
							'message'	=> $this->load->view('_emails/activated', array('link'=>site_url(),'title'=>$site_title), true)
					);
	
					if ($this->emailer->send($data))
					{
						$message = lang('us_active_email_sent');
					}
					else
					{
						$message=lang('us_err_no_email'). $this->emailer->error;
					}
				}
				Template::set_message($message, 'success');
			}
			else
			{
				Template::set_message(lang('us_err_status_error').$this->listing_model->error,'error');
			}//end if*/
		}
		else
		{
			Template::set_message(lang('ls_err_no_id'),'error');
		}//end if
	
	}//end listing_status()
	
	
	/*----------------------------------------------------*/
	/*	Upload Logo File
	/*----------------------------------------------------*/
	public function logo($id, $module = 'listings') {
		$id = intval($id); //Get listing ID
		$this->gc->upload_logo_cover('admin', $id, $module, 'logo');
	}
	
	
	/*----------------------------------------------------*/
	/*	Upload a Cover Image
	 /*----------------------------------------------------*/
	public function cover($id, $module = 'listings') {
		$id = intval($id); //Get listing ID
		$this->gc->upload_logo_cover('admin', $id, $module, 'cover');
	}
	
	/*----------------------------------------------------*/
	/*	Upload Photos Module
	/*----------------------------------------------------*/
	
	/**
	 * Image_CRUD 0.6 is used to upload images
	 */
	public function photos($module, $row_id) {	
		$this->load->model('images/images_model');
		if($module == 'listings') {
			$redirect_url = site_url(SITE_AREA . '/content/listings');
		} else {
			$redirect_url = site_url(SITE_AREA . '/content/listings/view/' . $module);
			if($this->session->userdata('navigation_listing_id')) {
				$redirect_url .= '/' . $this->session->userdata('navigation_listing_id');
			}
		}		
		Template::set('cancel_url', $redirect_url);
		$this->images_model->upload_photos_admin($module, $row_id, $redirect_url);
	}
	
	/*----------------------------------------------------*/
	/*	Business Hours Module
	/*----------------------------------------------------*/
	
	/**
	 * Display business hours form to the user
	 * @param number $listing_id
	 */
	public function business_hours($listing_id = NULL) {	
		if(!isset($listing_id) || $listing_id == NULL) {
			Template::redirect(SITE_AREA .'/content/listings');
		}
		//BUSINESS HOURS TIME PICKER
		Assets::add_js('moment.js');
    	Assets::add_css ( 'bootstrap-datetimepicker.min.css' );
    	Assets::add_js ( 'bootstrap-datetimepicker.min.js' );
    	
		$this->load->model('business_hours_model');
		if (isset ( $_POST ['save'] )) {
			if ($this->business_hours_model->save_business_hours('insert', $listing_id)) {					
				Template::set_message ( sprintf(lang('msg_save_success'), lang('label_business_hours')), 'success' );
				// redirect ( SITE_AREA . '/content/listings/business_hours/' . $listing_id );
				Template::redirect($this->session->userdata('listings_last_visited_page'));
			} else {
				Template::set_message ( sprintf(lang('msg_create_failure'), lang('label_business_hours')) . $this->business_hours_model->error, 'danger' );
			}
		}

		$records = $this->business_hours_model->where ( 'listing_id', $listing_id )->find_all ();
		$listing_info = $this->db->where('id', $listing_id)->select('id, title, slug, city')->get('listings')->row();
		Template::set (array(
		    'listing_id' => $listing_id,
		    'records' => $records,
		    'toolbar_title' => (!empty($listing_info) ? $listing_info->title . ' - ' : '') . lang('business_hours')
		));
		Template::set_view('listings/business_hours');
		Template::render ();
	}

	/**
	 * Change status of page from index page
	 * @param int $id
	 */
	public function change_status($type, $listing_id, $id)
	{
		//Return old status
		$this->db->select ( 'active' );
		$this->db->where('id', $id);
		$query = $this->db->get ( $this->db->dbprefix .$type );
	
		$value = 0; //declare variable to store returned result
	
		//Run loop and store integer value in variable
		foreach($query->result() as $row) {
			$value = $row->active;
		}
	
		//Compare values
		if($value == 0) {
			$data['active'] = 1;
		} else {
			$data['active'] = 0;
		}
		//Update status
		$this->db->where('id', $id);
		$this->db->update($this->db->dbprefix .$type, $data);
		redirect(site_url(SITE_AREA .'/content/listings/'.$type.'/'.$listing_id));
	}
	
	/**
	 * Bulk Generate QRCodes
	 */
	public function generate_qrcode () {		
		$this->load->library('ciqrcode');
		$listing_from = $this->input->post('listing_id_from');
		$listing_to = $this->input->post('listing_id_to');
		if (isset($_POST['save'])) {
			$this->form_validation->set_rules('listing_id_from', lang('label_between'), 'required|integer|min_val[1]');
			$this->form_validation->set_rules('listing_id_to', lang('label_and'), 'required|integer|max_val[10000000]|compare_fields['.$this->input->post('listing_id_from').']|diff_between['.$this->input->post('listing_id_from').',1000]');
			if($this->form_validation->run() == TRUE) {
				if($this->input->post('insert_or_update') == 1) {
					$sql = "SELECT L.id FROM ".$this->db->dbprefix ."listings L WHERE (L.id BETWEEN ".$listing_from." AND ".$listing_to.") AND (qrfile IS NULL OR qrfile = '') LIMIT 50000";
				} else {
					$sql = "SELECT L.id FROM ".$this->db->dbprefix ."listings L WHERE (L.id BETWEEN ".$listing_from." AND ".$listing_to.") LIMIT 50000";
				}
				$result = $this->db->query ( $sql )->result_array();
				if(!$result) {
					Template::set_message(lang('msg_nothing_to_update'), 'error');
					Template::redirect(current_url());
				}
				foreach ( $result as $listing ) {
					$this->gc->generate_qrcode($listing['id'], 'update');
				}
				// remove empty directories recursively
				$this->load->model('images/images_model');
				$path = 'assets/images/qrcodes';
				$this->images_model->remove_empty_dir($path);
				Template::set_message(lang('msg_generate_qrcode'),'success');
			}
		}
		Template::set('toolbar_title', lang('listings_generate_qrcode'));
		Template::set_view('qrcodes');
		Template::render();
	}
	
	/**
	 * Update categories level in order to achieve multilevel categories functionality
	 */
	public function update_categories_level () {
		$listing_from = $this->input->post('listing_id_from');
		$listing_to = $this->input->post('listing_id_to');
		if (isset($_POST['save'])) {
		$this->form_validation->set_rules('listing_id_from', lang('label_between'), 'required|integer|min_val[1]');
		$this->form_validation->set_rules('listing_id_to', lang('label_and'), 'required|integer|max_val[10000000]|compare_fields['.$this->input->post('listing_id_from').']|diff_between['.$this->input->post('listing_id_from').',50000]');
		if($this->form_validation->run() == TRUE) {
			// STEP 1: Get records from rs0d_categories table in hierarichal order and fill rs0d_categories_level table
			$fill_categories_level_sql = "INSERT INTO ".$this->db->dbprefix ."categories_level (SELECT PARENT.id AS parent_id, CASE WHEN SUB.sub_id IS NULL THEN 0 ELSE SUB.sub_id END, CASE WHEN SUBSUB.subsub_id IS NULL THEN 0 ELSE SUBSUB.subsub_id END FROM ".$this->db->dbprefix ."categories AS PARENT";
			$fill_categories_level_sql .= " LEFT JOIN (SELECT id AS sub_id, parent_id FROM ".$this->db->dbprefix ."categories) SUB ON PARENT.id = SUB.parent_id AND SUB.parent_id != 0";
			$fill_categories_level_sql .= " LEFT JOIN (SELECT id AS subsub_id, parent_id FROM ".$this->db->dbprefix ."categories) SUBSUB ON SUB.sub_id = SUBSUB.parent_id WHERE PARENT.parent_id = 0)";
			$this->db->simple_query($fill_categories_level_sql);
			
			// STEP 2: Fill rs0d_categories_relation table
			$sql = "SELECT ".$this->db->dbprefix ."listings.id, ".$this->db->dbprefix ."listings.category_id FROM ".$this->db->dbprefix ."listings WHERE (".$this->db->dbprefix ."listings.id BETWEEN ".$listing_from." AND ".$listing_to.") AND ".$this->db->dbprefix ."listings.category_id != 0 AND ".$this->db->dbprefix ."listings.id NOT IN (SELECT listing_id FROM ".$this->db->dbprefix ."listings_categories WHERE (listing_id BETWEEN ".$listing_from." AND ".$listing_to.")) LIMIT 50000";
			$query = $this->db->query ( $sql );
			if($query->result_array()) {
				foreach ( $query->result_array() as $listing ) {
					$category_sql = "SELECT * FROM ".$this->db->dbprefix ."categories_level WHERE parent_id = ".$listing['category_id']." OR sub_id = ".$listing['category_id']." OR subsub_id = ".$listing['category_id'];
					$category_query = $this->db->query ( $category_sql );
					if($category_query->result_array()) {
						foreach ( $category_query->result_array() as $category ) {
							if($category['parent_id'] == $listing['category_id']) {
								$parent_id = $listing['category_id'];
								$sub_id = 0;
								$subsub_id = 0;
							} else if($category['sub_id'] == $listing['category_id']) {
								$parent_id = $category['parent_id'];
								$sub_id = $category['sub_id'];
								$subsub_id = 0;
							} else if($category['subsub_id'] == $listing['category_id']) {
								$parent_id = $category['parent_id'];
								$sub_id = $category['sub_id'];
								$subsub_id = $category['subsub_id'];
							}
						}
						$data['module_id'] = gc_module_id('listings');
						$data['row_id'] =  $listing['id'];
						$data['parent_id'] = $parent_id;
						$data['sub_id'] = $sub_id;
						$data['subsub_id'] = $subsub_id;
					}
					if(isset($data)) {
						$this->db->insert('categories_relation', $data);
					}
					unset($data);
					}
				}
			}
		}		
	}
	
	/**
	 * Update rs0d_categories_relation manually via form
	 * @param integer $maxid
	 */
	public function update_categories() {
		$listing_from = $this->input->post('listing_id_from');
		$listing_to = $this->input->post('listing_id_to');
		if (isset($_POST['save'])) {
			$this->form_validation->set_rules('listing_id_from', lang('label_between'), 'required|integer|min_val[1]');
			$this->form_validation->set_rules('listing_id_to', lang('label_and'), 'required|integer|max_val[10000000]|compare_fields['.$this->input->post('listing_id_from').']|diff_between['.$this->input->post('listing_id_from').',1000000]');
			if($this->form_validation->run() == TRUE) {
				$where = "AND (L.id BETWEEN ".$listing_from." AND ".$listing_to.")";
				Template::set_message(lang('msg_listing_categories_update'),'success');
			}			
		}
		Template::set('toolbar_title', lang('update_categories'));
		Template::render();
	}
	
	/**
	 * Display categories limit to the user with message on listing_categories_js.php file 
	 * @param number $package_id
	 */
	public function retrieve_package($package_id) {
		$this->load->model('packages/packages_model');
		$this->load->helper('array');
		$categories_limit = $this->packages_model->select('module_categories_limit')->find($package_id);
		$msg = sprintf(lang('msg_categories_limit'), gc_field_value('listings', $categories_limit->module_categories_limit));
		$msg_categories_limit['msg'] = explode(',',($msg ?? ''));
		$result = array_merge(object_to_array($categories_limit), $msg_categories_limit);
		header('Content-Type: application/x-json; charset=utf-8');
		echo json_encode($result);
	}
	
	/**
	 * Reported Claims
	 */
	public function claims($filter='all', $offset=0) {
		$this->load->model('orders/orders_model');
		$this->auth->restrict($this->permissionEdit);
		// Assets::add_js($this->load->view('content/claims_js', null, true), 'inline');
		if (isset($_POST['save'])) {
		    $this->gcrud->stats(array('module' => 'listing_claims'));
			$claim_id = (int) $_POST['save'];
			$status = $_POST['status_' . $claim_id];
			$notify_customer = !empty($_POST['notify_customer_'. $claim_id]) ? 1 : 0;
			$result = $this->claims_model->update_claim_status($claim_id, $status, $notify_customer);
			if(is_bool($result)) {
				Template::set_message(sprintf(lang('msg_save_success'), lang('label_claim')), 'success');
			} else {
				Template::set_message($result, 'error');
			}
			Template::redirect(current_url());
		}
		
		// Deleting claim
		if (isset($_POST['delete'])) {
		    $this->gcrud->stats(array('module' => 'listing_claims'));
			$checked = $this->input->post('checked');		
			if (is_array($checked) && count($checked)) {
				$result = FALSE;
				foreach ($checked as $pid) {
					$result = $this->claims_model->delete($pid);
				}
		
				if ($result) {
					// set auto increment value to the last id
					$alter_autoinc = "ALTER TABLE ". $this->db->dbprefix . "listing_claims AUTO_INCREMENT = 1"; // IMPORTANT
					$this->db->simple_query($alter_autoinc);
					Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), lang('label_' . (count($checked) == 1 ? 'claim' : 'claims')), lang('label_deleted')), 'success');
				} else {
					Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_claims')) . $this->claims_model->error, 'danger');
				}
			}
		}
		
		// Filters
		$filter_type = $filter;
		
		switch($filter_type) {
			case 'active':
				$where = "status = 0";
				break;
		
			case 'approved':
				$where = "status = 1";
				break;
					
			case 'rejected':
				$where = "status = 2";
				break;
		
			case 'forged':
				$where = "status = 3";
				break;
					
			case 'cancelled':
				$where = "status = 4";
				break;
		
			case 'first_letter':
				$where = "SUBSTRING( LOWER(title), 1, 1)= '". $first_letter ."'";
				break;
					
			case 'all':
				$where = "";
				// Nothing to do
				break;
					
			default:
				show_404("listings/claims/$filter/");
		}
		
		$order_by = 'id';
		// Fetch the users to display
		$claims = $this->claims_model->get_listing_claims($offset, $this->limit, $where, $order_by);
		// Pagination
		$this->load->library('pagination');
		
		$this->claims_model->where($where);
		$total_claims = $this->claims_model->count_all();
		
		$pager['base_url'] = site_url(SITE_AREA ."/content/listings/claims/$filter/");
		$pager['total_rows'] = $total_claims;
		$pager['per_page'] = $this->limit;
		$pager['uri_segment']	= 6;
		
		$this->pagination->initialize($pager);
		
		Template::set('index_url', site_url(SITE_AREA .'/content/listings/claims') .'/');
		Template::set('filter_type', $filter_type);
		
		Template::set('claims', $claims);
		Template::set('toolbar_title', lang('manage_claims'));
		Template::render();
	}
	
	/**
	 * Bulk Link Validator - To check if the user entered links are working or broken
	 */
	public function link_validator() {		
		$listing_from = $this->input->post('listing_id_from');
		$listing_to = $this->input->post('listing_id_to');
		if (isset($_POST['save'])) {
			$this->form_validation->set_rules('listing_id_from', lang('label_between'), 'required|trim|integer|min_val[1]');
			$this->form_validation->set_rules('listing_id_to', lang('label_and'), 'required|trim|integer|max_val[10000000]|compare_fields['.$this->input->post('listing_id_from').']|diff_between['.$this->input->post('listing_id_from').',10000]');
			if($this->form_validation->run() == TRUE) {
				$this->db->where('id BETWEEN "'. $listing_from . '" AND "'. $listing_to .'"');
				$records = $this->listings_model->select('id, website')->find_all();
				
				if($records) {
					$data = array();
					foreach ( $records as $listing ) {												
						if(!empty($listing->website) && $this->isValidUrl($listing->website)) {
								$data[] = array ( 'id' => $listing->id, 'website' => '' );
						}								
					}
					if(!empty($data)) {
						$this->db->update_batch($this->db->dbprefix . 'listings', $data, 'id');
					}
					Template::set_message(lang('msg_success'), 'success');
				} else {
					Template::set_message(lang('msg_nothing_to_update'), 'error');
				}
			}
		}
		Template::set('toolbar_title', lang('link_validator'));
		Template::set_view('link_validator');
		Template::render();		
	}
	
	/**
	 * Purge Categories and Listings in Bulk
	 */
	public function bulk_purge() {	    
	    $id_from = $this->input->post('id_from');
	    $id_to = $this->input->post('id_to');
	    if (isset($_POST['save'])) {
	        $this->form_validation->set_rules('id_from', lang('label_between'), 'required|trim|integer|min_val[1]');
	        $this->form_validation->set_rules('id_to', lang('label_and'), 'required|trim|integer|max_val[10000000]|compare_fields['.$this->input->post('id_from').']|diff_between['.$this->input->post('id_from').',10000]');
	        if($this->form_validation->run() == TRUE) {
	            $this->load->model('categories/categories_model');
	            $this->load->model('listings/listings_model');
	            $module_id = $this->input->post('module_id');
	            $module = gc_module_name($module_id);
	            $this->db->where('id BETWEEN "'. $id_from . '" AND "'. $id_to .'"');
	            $records = $this->db->select('id')->get($module)->result();	            
	            if($records) {
	                $data = array();
	                foreach ( $records as $record ) {
	                    switch($module) {
	                        case 'categories':
	                            $this->categories_model->purge_category($record->id);
	                            break;
	                        case 'listings':
	                            $this->listings_model->purge_records($record->id);
	                            break;
	                    }
	                }
	                Template::set_message(lang('msg_success'), 'success');
	            } else {
	                Template::set_message(lang('msg_nothing_to_update'), 'error');
	            }
	        }
	    }
	    Template::set('toolbar_title', lang('bulk_purge'));
	    Template::set_view('bulk_purge');
	    Template::render();	
	}
	
	/**
	 * Convert string to valid URL before checking it online
	 * @param string $url
	 * @return boolean
	 */
	private function isValidUrl($url) {				
		if( ! preg_match('/^http(s)?:\/\/[a-z0-9-]+(.[a-z0-9-]+)*(:[0-9]+)?(\/.*)?$/i', $url) ) {			
			$url_http = 'http://' .$url; // convert string to valid url
			$parsed = parse_url($url_http); 
			if(strpos($parsed['host'], "www.") === 0) { // url is with www
				$url_without_www = $url_http;
			} else { // add www to url
				$url_with_www = 'http://www.' .$url;
			}
		} else {
			$parsed = parse_url($url); 
			if(strpos($parsed['host'], "www.") === 0) { // url is with www
				$url_without_www = $url;
			} else { // add www to url
				$url_with_www = 'http://www.' .$parsed['host'];
			}			
		}
		
		if((isset($url_without_www) && $this->urlExists($url_without_www)) || (isset($url_with_www) && $this->urlExists($url_with_www))) {
			return false;
		} else {
			return true;
		}
	}
	
	/**
	 * Check whether URL exists or not using HTTP GetHeader - SLOW
	 * @param string $url
	 * @return boolean
	 */
	private function urlExist_using_getheader($url) {
		$headers = @get_headers($url);
		if(strpos($headers[0],'200') === false) {
			return false;
		} else {
			return true;
		}
	}
	
	/**
	 * Check whether URL exists or not using CURL
	 * Faster than GetHeader
	 * @param string $url
	 * @return boolean
	 */
	private function urlExists($url){
		$agent = "Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0)";$ch=curl_init();
		curl_setopt ($ch, CURLOPT_URL,$url );
		curl_setopt($ch, CURLOPT_USERAGENT, $agent);
		curl_setopt ($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt ($ch,CURLOPT_VERBOSE,false);
		curl_setopt($ch, CURLOPT_TIMEOUT, 5);
		curl_setopt($ch,CURLOPT_SSL_VERIFYPEER, FALSE);
		curl_setopt($ch,CURLOPT_SSLVERSION,3);
		curl_setopt($ch,CURLOPT_SSL_VERIFYHOST, FALSE);
		$page=curl_exec($ch);
		//echo curl_error($ch);
		$httpcode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
		curl_close($ch);
		// 0 = URL Not Exists, 404 = Page Not Found
		if($httpcode == 0 || $httpcode == 404) return false;
		else return true;
	}
	
	/**
	 * Insert/Update slug for all listings
	 */
	public function slugify() {
		$this->load->library('slug');
		$row_difference = 50000;
		if (isset($_POST['save'])) {
			$this->form_validation->set_rules('module', lang('label_module'), 'trim');
			$this->form_validation->set_rules('id_from', lang('label_ids'), 'required|integer|min_val[1]');
			$this->form_validation->set_rules('id_to', lang('label_ids'), 'required|integer|max_val[100000000]|compare_fields['.$this->input->post('id_from').']|diff_between['.$this->input->post('id_from').','. $row_difference .']');
			if($this->form_validation->run() == TRUE) {
				// Change this value to true if you want to have a unique slug for each row
				$unique_slug = false;
				$table_name = $this->input->post('module');
				$update = $this->input->post('insert_or_update') ? $this->input->post('insert_or_update') : 0;
				$field = 'title';
				switch($table_name) {
				    case 'categories':
				    case 'states':
				    case 'cities':
						$field = 'name';
						break;
					case 'users':
						$field = 'email';
						break;
				}				
				// Insert only those slugs which are empty
				if($update == 1) { 
					$this->db->or_where('slug', NULL);
					$this->db->or_where('slug', '');
				}
				$records = $this->db->select('id, ' . $field)->get_where($this->db->dbprefix . $table_name, array('id >= ' => $this->input->post('id_from'), 'id <= ' => $this->input->post('id_to')))->result();
				if($records) {
					foreach($records as $record) {
						$data[] = array(
								'id' => $record->id,
								'slug' => $unique_slug ? $this->gc->slugify($table_name, $record->$field, 0, $field) : $this->slug->create_slug($record->$field)
						);
					}
					$this->db->update_batch($table_name, $data, 'id');
					Template::set_message(sprintf(lang('update_slug_success'), count($records), $table_name), 'success');
				} else {
					Template::set_message(lang('update_slug_error'), 'error');
				}
			}
		}
		Template::set( array(
			'toolbar_title' => lang('gc_menu_update_slug'),
			'row_difference' => $row_difference
		));
		Template::set_view('content/update_slug');
		Template::render();
	}
	
	/**
	 * Tweak Fields
	 */
	public function tweak_fields() {
		$row_difference = 50000;
		if (isset($_POST['save'])) {
			$this->form_validation->set_rules('module', lang('label_module'), 'trim');
			$this->form_validation->set_rules('id_from', lang('label_between'), 'required|trim|integer|min_val[1]');
			$this->form_validation->set_rules('id_to', lang('label_and'), 'required|trim|integer|max_val[' . ((int) $this->input->post('id_from') + $row_difference) . ']|compare_fields[' . $this->input->post('id_from') . ']|diff_between[' . $this->input->post('id_from') . ', ' . $row_difference . ']');
			if($this->form_validation->run() == TRUE) {
				$module = $this->input->post('module');				
				$from = $this->input->post('id_from');
				$to = $this->input->post('id_to');
				$result = $this->listings_model->bulk_tweak($module, $from, $to);	
				if($result) {
					Template::set_message(lang('msg_success'), 'success');
				} else {
					Template::set_message(lang('msg_nothing_to_update'), 'error');
				}
			}
		}
		Template::set(array(
			'toolbar_title' => lang('gc_menu_tweak_fields'),
			'row_difference' => $row_difference
		));
		Template::set_view('tweak_fields');
		Template::render();
	}
	
	/**
	 * Update Country, State and City information of all listings after State and City Change (Bulk Import)
	 * @param string $country
	 * @param int $state
	 * @param int $city
	 */
	public function ucsc() {
		$config_modules = gc_modules();
		foreach($config_modules as $config_module) {
			if(gc_module_settings($config_module, 'locations') == 1) {
			    $sql = "UPDATE " . $this->db->dbprefix . $config_module . " SET country_iso = '" . settings_item('adv.default_country') ."', state_id = " . settings_item('adv.default_state') . ", city_id = "  . settings_item('adv.default_city');
				$this->db->simple_query($sql);
				
				// Tweak Locations Information
				$this->gc->_tweak_locations_query($config_module); // Update Locations Information
			}
		}
		Template::set_message(sprintf(lang('msg_action_success'), lang('gc_menu_listings'), lang('label_updated')), 'success');
		Template::redirect(site_url(SITE_AREA .'/content/listings'));
	}
	
	/**
	 * Send Email to Listing Email Address
	 * @return array
	 */
	public function sendEmail() {
		$this->form_validation->set_rules('listing_id', 'lang:label_id', 'required|xss_clean');
		$this->form_validation->set_rules('email_subject', 'lang:label_subject', 'required|xss_clean');
		$this->form_validation->set_rules('email_message', 'lang:label_message', 'required|xss_clean');
	
		if ($this->form_validation->run() === FALSE) {
			$validation_errors = validation_errors();
			if ($validation_errors) {
				$validation_error = $validation_errors;
			}
			echo json_encode(array('result' => 0, 'message' => $validation_error, 'type' => 'error'));
		} else {
			$listing_id = escape($this->input->post('listing_id'));
			// Retrieve user email address from database
			$listing = $this->listings_model->select('email')->find($listing_id);
			if(!$listing) {
				echo json_encode(array('result' => 0, 'message' => lang('listing_email_not_exists'), 'type' => 'error'));
			}
	
			// Send Email
			$this->load->library('emailer/emailer');
			$data = array(
					'from'		=> settings_item('sender_email'),
					'to'		=> $listing->email,
					'subject'	=> escape($this->input->post('email_subject')),
					'message'	=> $this->input->post('email_message')
			);
			$success = $this->emailer->send($data, FALSE);
			if($success) {
				echo json_encode(array('result' => 1, 'message' => lang('success_message'), 'type' => 'success'));
			} else {
				echo json_encode(array('result' => 0, 'message' => lang('email_error'), 'type' => 'error'));
			}
		}
	}
	
	/**
	 * Change Categories Limit on Package Dropdown change
	 * @return array
	 */
	public function categories_limit() {
		//add the header here
		header('Content-Type: application/json');
		$package_id = $this->input->post('package_id');
		$this->load->model('packages/packages_model');
		$result = $this->packages_model->select('module_categories_limit')->find($package_id);
		$categories_limit = gc_field_value('listings', $result->module_categories_limit);
		echo json_encode(array('categories_limit' => $categories_limit));
	}
	
	/**
	 * Update Listings Table State_ID and Country_ISO based on City_ID after bulk import
	 */
	public function update_state_country() {
		$this->load->model('locations/cities_model');
		$result = $this->listings_model->select('id, city_id')->find_all_by(array('id > ' => 10, 'id <= ' => 10000));
		if($result) {
			$not_found = array();
			foreach($result as $row) {
				$location = $this->cities_model->get_city_info($row->city_id);
				if($location) {
					$data[] = array(
							'id' => $row->id,
							'state_id' => $location->state_id,
							'country_iso' => $location->country_iso
					);
				} else {
					$not_found[] = array(
							'id' => $row->id,
							'city_id' => $row->city_id
					);
				}
			}
			$this->db->update_batch('listings', $data, 'id');
		}
	}
	
	/*******************************************************/
	/* START OF MODULES
	/*******************************************************/
	/**
	 * Displays a list of form data.
	 * @return void
	 */
	public function view($module, $listing_id = 0, $filter = 'all', $offset = 0) {
	    // download in PDF and XLS
	    $this->gcrud->pdf_excel_download($module, $offset, $this->limit);
	    
	    // Advanced Search Load CSS and JS files
	    Assets::add_css('selectize.bootstrap3.css');
	    Assets::add_js('selectize.min.js');
	    Assets::add_js('selectize-search.js');
		// Allow user to stay to all rows of a module
	    // if the listing_id is set then set it
	    
	    // redirect back and forth between pages
	    $this->session->set_userdata($module . '_last_visited_page', current_url());
	    
	    // category fields url redirection
	    $this->session->set_userdata('cancel_url', current_url());
	    
		if($listing_id != 0) {
			$this->session->set_userdata('navigation_listing_id', $listing_id);
		} else {
			$this->session->unset_userdata('navigation_listing_id');
		}
		// Allow Admin to Clone A Row
		$this->clone_row($module);
		$this->gc->view_admin($module, $listing_id, $filter, null, 0, $offset, $this->limit);
	}
	
	/**
	 * Creates a module row object.
	 * @return void
	 */
	public function add($module, $row_id = 0, $toggle_module = NULL) {
		$redirect_url = site_url(SITE_AREA . '/content/listings/view');
		if($module == 'videos') {
			// $this->session->set_userdata('row_id', $row_id);
			// $this->session->set_userdata('toggle_module', $toggle_module);
			//$redirect_url = site_url(SITE_AREA . '/content/videos/index');
			Template::set(array(
				'toggle_row_id' => $row_id,
				'toggle_module' => $toggle_module
			));
			$redirect_url = site_url(SITE_AREA . '/content/listings/videos');
		}
		$this->gc->module('admin', $module, $this->auth->user_id(), $redirect_url, $row_id, 0, $toggle_module, $row_id);
		Template::set(array(
			'toolbar_title' => lang('add') . ' ' . lang($module)
		));
		Template::set_view('fields/form');
		Template::render();
	}
	
	/**
	 * Allows editing of module data.
	 * @return void
	 */
	public function modify($module) {
		$id = $this->uri->segment(6);
		$redirect_url = site_url(SITE_AREA . '/content/listings/view');
		if (empty($id)) {
			Template::set_message(lang('invalid_id'), 'error');
			redirect(SITE_AREA .'/content/listings');
		}
		if($module == 'videos') {
			$redirect_url = site_url(SITE_AREA . '/content/listings/videos');
		}
		$this->gc->module('admin', $module, $this->auth->user_id(), $redirect_url, 0, $id, null, 0);
		Template::set(array(
			'toolbar_title' => lang('edit') . ' ' . lang($module)
		));
		Template::set_view('fields/form');
		Template::render();
	}
	
	/**
	 * Display Videos to the Admin
	 * @param number $row_id
	 * @param string $toggle_module
	 * @param string $filter
	 * @param number $offset
	 */
	public function videos1($toggle_module = 'all', $row_id = 0, $filter='all', $offset = 0) {		
		$this->session->set_userdata('row_id', $row_id);		
		$redirect_url = SITE_AREA .'/content/listings/videos/' . $toggle_module . '/' . $row_id;
		$limit = $this->limit;
		$this->gc->view_module_row_admin('videos', $toggle_module, $row_id, $filter, $offset, $limit, $redirect_url);
	}
	
	/**
	 * Display Videos to the Admin
	 * 
	 * We can have maximum 4 parameters in the URI Segment of a URL
	 * i.e. a new function/method is used instead of using the existing VIEW method
	 *  
	 * @param string $filter
	 * @param string $toggle_module
	 * @param number $toggle_row_id
	 * @param number $offset
	 */
	public function videos($toggle_module = null, $toggle_row_id = 0, $filter = 'all', $offset = 0) {
		//LOAD MODAL FOR YOUTUBE VIDEO PLAYER
		Assets::add_js('js/bootstrap.youtubepopup.js');
		Assets::add_js('js/bootstrap.youtubepopup.min.js');
		if($toggle_module == 'all') {
			$this->session->unset_userdata('toggle_module', $toggle_module);
			$this->session->unset_userdata('navigation_listing_id');
		} else {
			$this->session->set_userdata('toggle_module', $toggle_module);
			$this->session->set_userdata('navigation_listing_id', $toggle_row_id);
		}
		Template::set('toggle_row_id', $toggle_row_id);
		Template::set('toggle_module', $toggle_module); // Sub Navigation Bar
		// Allow Admin to Clone A Row
		$this->clone_row('videos');
		$this->gc->view_admin('videos', 0, $filter, $toggle_module, $toggle_row_id, $offset, $this->limit);
	}
	/*******************************************************/
	/* END OF MODULES
	/*******************************************************/
	/**
	 * Bulk Insert Latitude and Longitude Information
	 */
	public function bulk_latlng() {
		$module = 'listings';
		$listing_from = (int) ($_GET['from'] ? $_GET['from'] : 1);
		$listing_to = (int) ($_GET['to'] ? $_GET['to'] : 50000);
		$where = ' (id BETWEEN ' . $listing_from . ' AND ' . $listing_to . ') AND (latitude = "" OR latitude IS NULL) AND (longitude = "" OR longitude IS NULL)';
		$records = $this->listings_model->where($where)->find_all();
		if($records) {
			foreach($records as $result) {
				$this->listings_model->addLatLng($module, $result->id, $result);
			}
			Template::set_message(lang('create_success'), 'success');			
		} else {
			Template::set_message(lang('msg_nothing_to_update'), 'danger');
		}
		Template::redirect(SITE_AREA .'/content/listings');
	}
	
	/**
	 * Allow users to add the Category Fields information
	 */
	public function category($row_id, $module = 'listings') {
	    $user_id = $this->auth->user_id();
	    $this->load->model('fields/fields_model');
	    $this->fields_model->save_categories_meta('admin', $module, $row_id, $user_id);
	}
	
	/**
	 * Directory Printed Version PDF
	 */
	public function printed_version() {
	    Assets::add_css('selectize.bootstrap3.css');
	    Assets::add_js('selectize.min.js');
	    Assets::add_js('selectize-search.js');
	    $this->load->model('fields/fields_model');
	    if(isset($_POST['save'])) {	
	        $this->form_validation->set_rules('package_id', 'lang:label_package', 'trim|required');
	        $this->form_validation->set_rules('offset', 'lang:label_offset', 'trim|is_numeric|required');
	        $this->form_validation->set_rules('limit', 'lang:label_limit', 'trim|is_numeric|required');
	        if($this->form_validation->run() == TRUE) {
	            $package_id = (int) $this->input->post('package_id');
	            $category_id = (int) $this->input->post('category_id');
	            $city_id = (int) $this->input->post('city_id');
	            $limit = (int) $this->input->post('limit');
	            $offset = (int) $this->input->post('offset');
	            $where = array(
	                'package_id' => $package_id
	            );
	            if(!empty($city_id)) {
	                $where['city_id'] = $city_id;
	            }
	            if(!empty($category_id)) {
	                $this->db->where('FIND_IN_SET(' . $category_id . ', categories_set)');
	            }
	            $this->db->where($where);
	            $this->db->limit($limit, $offset);
	            $records = $this->db->where(array('status' => 1, 'deleted' => 0))->get('listings')->result();
    	        if(!$records) {
    	            Template::set_message(lang('label_no_record_found'), 'danger');    	            
    	        } else {
    	            $file_header = '';
    	            $package = $this->db->where('id', $package_id)->get('packages')->row();
    	            if($package) {
    	                $file_header .= $package->title;
    	            }
    	            if(!empty($category_id)) {    	                
    	                $category = $this->db->where('id', $category_id)->get('categories')->row();
    	                if($category) {
    	                    $file_header .= ' / ' . $category->name;
    	                }
    	            }
    	            if(!empty($city_id)) {
    	                $city = $this->db->where('id', $city_id)->get('cities')->row();
    	                if($city) {
    	                    $file_header .= ' / ' . $city->name;
    	                }
    	            }
    	            $filename = 'printed_version_' . date('Y_m_d_H_i_s');
    	            $this->load->library('format');
    	            $html_content = $this->load->view('content/tpl_pdf', array('records' => $records, 'file_header' => $file_header), true);
        	        $this->format->to_dompdf(array('filename' => $filename, 'content' => $html_content, 'download' => true));	
    	        }
	        }
	    }
	    Template::set(array(
	        'toolbar_title' => lang('gc_menu_printed_version')	        
	    ));
	    Template::render();
	}
	
	
	/**
	 * Bulk Generate users from listings
	 * The user can bulk import listings with a user_id 1 and then generate them automatically using this method
	 * The user role_id will be checked first and if it is not admin, the user will be generated
	 */
	public function bulk_users_generate() {
	    $this->auth->restrict('Listings.Bulk.Users');
	    $row_difference = 5000;
	    $this->load->model('users/user_model');
	    $filename = 'listings';
	    if (isset($_POST['save'])) {
	        $this->form_validation->set_rules('id_from', lang('label_between'), 'required|integer|min_val[1]');
	        $this->form_validation->set_rules('id_to', lang('label_and'), 'required|integer|max_val[10000000]|compare_fields['.$this->input->post('id_from').']|diff_between['.$this->input->post('id_from').',' . $row_difference . ']');
	        if($this->form_validation->run() == TRUE) {
	            $id_from = $this->input->post('id_from');
	            $id_to = $this->input->post('id_to');
	            $this->db->where('id BETWEEN "'. $id_from . '" AND "'. $id_to .'"');
	            $records = $this->db->get('listings')->result();
	            $users = array();
	            $bulk_update_data = array();
	            $filename .= '_' . $id_from . '_' . $id_to;
	            $excel_data = array(); // provice password excel file to the admin
	            if($records) {
	                foreach($records as $record) {
	                    // get user role
	                    $generate_user = false;
	                    if(!in_array($record->user_id, $users)) {
	                        $users[$record->user_id] = (object) array('role_id' => 0, 'email' => $record->email);
	                        $user = $this->db->select('role_id, email')->where('id', $record->user_id)->get('users')->row();
	                        if($user) {
	                            $users[$record->user_id] = $user;
	                        }
	                    }
	                    $user = $users[$record->user_id];
	                    if($user->role_id != 2) {
	                        // generate a new user
	                        if(!empty($record->email)) {
	                            // check user
	                            $users_count = $this->db->where('email', $record->email)->count_all_results('users');
	                            if($users_count == 0) {
	                                $password = $this->user_model->random_password(); // 'password_' . $record->id;
	                                $display_name = !empty($record->contact_person) ? $record->contact_person : $record->title;
	                                $users_data = array(
	                                    'display_name' => $display_name,
	                                    'username' => $record->email,
	                                    'role_id' => 2,
	                                    'email' => $record->email,
	                                    'password' => $password,
	                                    'language' => settings_item('site.default_language'),
	                                    'active' => 1 // set the user active
	                                );
	                                $user_id = $this->user_model->insert($users_data);
	                                if($user_id > 0) {
	                                    $bulk_update_data[] = array(
	                                        'id' => $record->id,
	                                        'user_id' => $user_id
	                                    );
	                                    
	                                    // asign default package to the user on creation
	                                    $user = $this->db->where('id', $user_id)->get('users')->row();
	                                    if($user && ($user->role_id == 2 || $user->role_id == 4)) {
	                                        $this->load->library('users/user_model');
	                                        $this->user_model->assign_package($user_id, $user);
	                                    }
	                                    
	                                    // Emailer Data
	                                    $emailer_record = $record;
	                                    $emailer_record->display_name = $display_name;
	                                    $emailer_record->user_email = $record->email;
	                                    $emailer_record->role_id = 2;
	                                    $emailer_record->password = $password;
	                                    $emailer_record = $this->listings_model->bulk_notify($record->id, $emailer_record);
	                                    
	                                    // notify users only
	                                    $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'module_email_templates', 'module_table' => 'listings', 'action_type' => 'bulk', 'id' => $record->id, 'record' => $emailer_record, 'queue_override' => true, 'user_type' => 'user'));
	                                    
	                                    // provide excel file to the user
	                                    $excel_data[] = array(
	                                        'id' => $record->id,
	                                        'title' => $record->title,
	                                        'username' => $record->email,
	                                        'password' => $password,
	                                        'created_on' => gc_date_format(date('Y-m-d H:i:s'))
	                                    );
	                                }
	                            }
	                        }
	                    }
	                }
	            }
	            if(!empty($bulk_update_data)) {
	                // update listings
	                $this->db->update_batch('listings', $bulk_update_data, 'id');
	                Template::set_message(lang('msg_success'), 'success');
	            } else {
	                Template::set_message(lang('msg_nothing_to_update'), 'danger');
	            }
	            // generate excel file for admin record
	            if(!empty($excel_data)) {
	                // notify admin on users download
	                $record = new stdClass();
	                $record->users_list = $this->load->view('content/_tpl_users_list', array('records' => $excel_data), true);
	                $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'module_email_templates', 'module_table' => 'listings', 'action_type' => 'bulk', 'id' => 1, 'record' => $record, 'queue_override' => true, 'user_type' => 'admin'));
	                $download = $this->input->post('download');
	                if($download) {
	                    $field_names = array(lang('label_id'), lang('label_title'), lang('bf_username'), lang('bf_password'), lang('created_on'));
	                    $data = array(
	                        'file_name' => $filename,
	                        'file_type' => 'Excel2007',
	                        'field_names' => $field_names,
	                        'rows' => $excel_data,
	                        'download' => true
	                    );
	                    $this->format->to_excel($data);
	                }
	            }
	        }
	    }
	    Template::set(array(
	        'row_difference' => $row_difference,
	        'toolbar_title' => lang('gc_menu_bulk_users_generate')
	    ));
	    Template::render();
	}
	
	/**
	 * Allow user to upload documents
	 * @param string $module
	 * @param integer $row_id
	 */
	public function documents($module, $row_id) {
	    $this->load->library('goclixy/gfile');
	    $this->gfile->upload_files($module, $row_id);
	    Template::render();
	}
	
	/**
     * Add Business
	 */
	public function add_business() {
	    if(isset($_POST['save'])) {	        
	        $this->form_validation->set_rules('title', lang('label_title'), 'trim|required');
	        $this->form_validation->set_rules('contact_person', lang('label_contact_person'), 'trim|required');
	        $this->form_validation->set_rules('email', lang('label_email'), 'trim|required|valid_email');
	        $this->form_validation->set_rules('phone_number', lang('label_phone_number'), 'trim|required|phone_check');
	        $this->form_validation->set_rules('address', lang('label_address'), 'trim');
	        $this->form_validation->set_rules('postcode', lang('label_postcode'), 'trim');
	        $this->form_validation->set_rules('city_id', lang('label_city'), 'trim|required');
	        $this->form_validation->set_rules('categories_set', lang('label_category'), 'trim|required');
	        if($this->form_validation->run() == TRUE) {
	            $this->load->library('slug');
	            $module = 'listings';
	            $this->load->model('users/user_model');
	            $title = $this->input->post('title');
	            $description = $this->input->post('description');
	            $contact_person = $this->input->post('contact_person');
	            $email = $this->input->post('email');
	            $phone_number = $this->input->post('phone_number');
	            $address = $this->input->post('address');
	            $postcode = $this->input->post('postcode');
	            $city_id = $this->input->post('city_id');
	            $categories_set = $this->input->post('categories_set');
	            $data = array(
	                'package_id' => 1,
	                'title' => $title,
	                'slug' => $this->slug->create_slug($title),
	                'description' => $description,
	                'contact_person' => $contact_person,
	                'email' => $email,
	                'phone_number' => $phone_number,
	                'address' => $address,
	                'postcode' => $postcode,
	                'city_id' => $city_id,
	                'status' => 1
	            );
	            if($city_id) {
	                $this->load->model('locations/cities_model');
	                $city_info = $this->cities_model->get_city_info($city_id);
	                $data['state_id'] = $city_info->state_id;
	                $data['country_iso'] = $city_info->country_iso;
	                // Save City, State and Country Information in the Modules Table
	                $data['city'] = $city_info->city;
	                $data['state'] = $city_info->state;
	                $data['country'] = $city_info->country;
	            }
	            $this->db->insert('listings', $data);
	            $insert_id = $this->db->insert_id();
	            if($insert_id) {
	                $this->load->model('categories/categories_model');
	                $_POST['categories'] = $this->categories_model->format_category($categories_set);
    	            $this->categories_model->save_categories_relation($insert_id, $module);
    	            
    	            $user = $this->db->select('role_id, email')->where('id', $email)->get('users')->row();
    	            if($user) {
    	                $user_id = $user->id;
    	                if($user->role_id != 2) {
    	                    $this->db->where('id', $user_id)->update('users', array('role_id' => 2));
    	                }
    	            } else {	                
    	                $password = $this->random_password();
    	                $users_data = array(
    	                    'display_name' => $contact_person,
    	                    'username' => $email,
    	                    'role_id' => 2,
    	                    'email' => $email,
    	                    'password' => $password,
    	                    'language' => settings_item('site.default_language'),
    	                    'active' => 1 // set the user active
    	                );
    	                $user_id = $this->user_model->insert($users_data);
    	            }
    	            
    	            // asign default package to the user on creation
    	            $user = $this->db->where('id', $user_id)->get('users')->row();
    	            if($user && ($user->role_id == 2 || $user->role_id == 4)) {
    	                $this->load->library('users/user_model');
    	                $this->user_model->assign_package($user_id, $user);
    	            }
    	            
    	            // Generate QRCode
    	            $this->gc->generate_qrcode($insert_id);
    	            
    	            // Add Latitude and Longitude information if kept blank by the user
    	            $this->listings_model->addLatLng('listings', $insert_id);
    	            
    	            $this->db->where('id', $insert_id)->update('listings', array('user_id' => $user_id));
    	            $emailer_record = $this->notify_add_business($insert_id, array());
    	            if(isset($password)) {
    	                $emailer_record->password = $password;
    	            }
    	            // get package information
    	            $package = $this->db->where('id', 1)->get('packages')->row();
    	            if($package) {
    	                $emailer_record->package = $package->title;
    	            }
    	            $login_url = $emailer_record->login_url; // anchor(site_url($data['login_url']), $data['login_url']);
    	            $emailer_record->login_information = '<strong>' . lang('label_sign_in') . ' ' . lang('label_url') . ':</strong> ' . $login_url . '<br /><strong>' . lang('bf_username') . ':</strong> ' . $emailer_record->email . '<br /><strong>' . lang('bf_password') . ':</strong> ' . $emailer_record->password;
    	            $emailer_record->author_name = $emailer_record->contact_person;
    	            
    	            // Notify Owner via Email
    	            $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'module_email_templates', 'module_table' => $module, 'action_type' => 'add', 'user_type' => 'user', 'id' => $insert_id, 'record' => $emailer_record));
    	            
    	            $success_message = sprintf(lang('msg_save_success'), lang('label_listing'));
    	            Template::set_message($success_message, 'success');
    	            Template::redirect(SITE_AREA . '/content/listings');
	            } else {
	                $error_message = sprintf(lang('msg_create_failure'), lang('label_listing'));
	                Template::set_message($error_message, 'danger');
	            }
	        }
	    }
	    Assets::add_css('selectize.bootstrap3.css');
	    Assets::add_js('selectize.min.js');
	    Assets::add_js('selectize-search.js');
	    if(settings_item('lst.google_places_api') == 0) {
	        $this->gc->ontype_city_selection();
	    } else {
	        $this->gc->location_google_places();
	    }
	    Template::set(array(
	        'toolbar_title' => lang('label_submit_listing'),
	    ));
	    Template::set_view('add_business');
	    Template::render();
	}
	
	/**
	 * Notify Users on Listing Add
	 * @param number $listing_id
	 * @param array $record
	 * @param string $user_type - admin/user
	 * @return object
	 */
	private function notify_add_business($listing_id, $record = array(), $user_type = null) {
	    $this->load->library('crypt');
	    if(empty($record)) {
	        $this->db->from('listings L');
	        $this->db->join('users U', 'U.id = L.user_id', 'INNER');
	        $record = $this->db->select('L.id, L.title, L.slug, L.description, L.address, L.postcode, L.city, L.state, L.country, L.country_iso, L.contact_person, L.categories, L.email, L.phone_number, L.status, L.created_on, L.website, U.display_name, U.email AS user_email, U.role_id')->where('L.id', $listing_id)->get()->row();
	    }
	    if($record) {
	        $record->edit_url = gc_url_format('members/edit_business') . '?id=' . $this->crypt->encode($record->id);
	        $record->live_url = gc_url_format('listings', $record->slug, $record->id, $this->slug->create_slug($record->city));
	        $record->edit_url = anchor($record->edit_url, $record->edit_url);
	        $record->url = anchor($record->live_url, $record->live_url);
	        $record->login_url = anchor(gc_url_format('login'), gc_url_format('login'));
	        $record->status = $this->listings_model->listing_status($record->status, true);	        
	    }
	    return $record;
	}
	
	/**
	 * Generate Random Password
	 * @param number $_len
	 * @return string
	 */
	private function random_password($_len = 12) {
	    $_alphaSmall = 'abcdefghijklmnopqrstuvwxyz';            // small letters
	    $_alphaCaps  = strtoupper($_alphaSmall);                // CAPITAL LETTERS
	    $_numerics   = '1234567890';                            // numerics
	    $_specialChars = '~!@#%^*()-_=+]}[{/?|';   // Special Characters
	    
	    $_container = $_alphaSmall.$_alphaCaps.$_numerics.$_specialChars;   // Contains all characters
	    $password = '';         // will contain the desired pass
	    
	    for($i = 0; $i < $_len; $i++) {                                 // Loop till the length mentioned
	        $_rand = rand(0, strlen($_container) - 1);                  // Get Randomized Length
	        $password .= substr($_container, $_rand, 1);                // returns part of the string [ high tensile strength ;) ]
	    }
	    return $password;       // Returns the generated Pass
	}
	
	
	/**
	 * Ajax Method to show business completion percentage in a Modal on progress-completion click
	 */
	public function percent_completion() {
	    $module = 'listings';
	    $id = $this->input->post('id');
	    $this->load->model('groups/groups_model');
	    $data = $this->groups_model->percent_completion($module, $id);
	    echo json_encode(array('result' => 1, 'heading' => $data['heading'], 'data' => $data['output']));
	    return false;
	}	
	
	/**
	 * Manage Contact Queries Email Templates
	 * @param string $module
	 * @param string $filter
	 * @param number $offset
	 */
	public function manage($module = 'email_templates', $filter = 'all', $offset = 0) {
	    $extra = array(
	        'model' => 'listings',
	        'context' => 'content',
	        'module_name' => 'listings',
	        'fields' => $module,
	        'prefix' => 'module'
	    );
	    $this->gc->module_view($module, $filter, $offset, 'admin', $extra);
	}
	
	/**
	 * Add/Edit Email Templates Data
	 *
	 * @return void
	 */
	public function form($module = 'email_templates', $action = 'create') {
	    Assets::add_js(array(
	        'tinymce/tinymce.min.js',
	    ));
	    $params = array(
	        'selectize' => true,
	        'model' => 'listings',
	        'module_name' => 'listings',
	        'context' => 'content',
	        'fields_method' => $module,
	        'prefix' => 'module'
	    );
	    $this->gcrud->form($module, $action, 'admin', $params);
	}
}