<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class Listings extends Front_Controller {
	public function __construct()
	{
		parent::__construct();

		$this->load->library('form_validation');
		$this->load->model('listings_model', null, true);
		$this->load->model('images/images_model');
		$this->load->model('videos/videos_model');
		$this->load->model('business_hours_model');
		$this->load->model('comments/comments_model');
		$this->load->model('categories/categories_model');
		$this->load->model('ratings/ratings_model');
		$this->load->model('business_queries/business_queries_model');
		$this->load->model('erroneous/erroneous_model');
		$this->load->model('users/user_model');
		$this->load->model('tags/tags_model');
		$this->load->model('pages/pages_model');
		$this->load->library('parser');
				
		$this->load->library('users/auth');	
		$this->lang->load('listings');	

		//rating star
		Assets::add_js('star-rating.min.js');
		Assets::add_css('star-rating.min.css');
		
		//Assets::add_js('main.js');
		//Assets::add_js('plugins.js');
		$this->load->library('slug');
	}
	
	/**
	 * Show Listing Detail to the User
	 * @param string $slug
	 */
	public function detail($slug) {
		$this->load->model('branches/branches_model');
		$this->load->helper('array');
		$user_id = (int) $this->auth->user_id();
		// Multilanguage Ratings
		Assets::add_js( $this->load->view('common/ratings_js', null, true ), 'inline' );
		// get all characters from right to left till first dash to get listing id
		$id = (int)substr( $slug, strrpos( $slug, '-' ) + 1);
		
		// restrict users to module_views_limit set in packages
		if(settings_item('site.minimal') == 0 && !$this->gc->check_module_views_limit('listings', $id)) {
		    Template::redirect(gc_url_format('members/membership'));
		}
		
		if(!empty($_GET['g']) && settings_item('lst.google_business') == 1) { // Google Listing
		    $sql = "SELECT place_id FROM " . $this->db->dbprefix . "google_listings WHERE slug = '" . $slug . "'";
		    $place = $this->db->query($sql)->row();
		    if(!$place) {
		        Template::redirect(site_url());
		    }
		    $place_id = $place->place_id;
		    $this->load->library('Google_listings');
		    $google_business = $this->google_listings->detail($place_id);
			$record = $google_business['listing'];
		} else {
		    $slug_url = settings_item('lst.slug_url') == 1 ? $slug : null;
    		if(settings_item('site.enable_cache') == 1) { 
    			$cacheID = str_replace('-', '_', $slug); // Cache giving errors when invalid url with spaces and special characters
    			$cacheID = str_replace('-', '_', $this->slug->create_slug($cacheID));
    			if(!$result = $this->gc_cache->load($cacheID)) {
    			    $result = $this->verify_existence($id, $slug_url);
    			    $this->gc_cache->save($result, $cacheID, array('listing_detail'));
    			}
    		} else {
    		    $result = $this->verify_existence($id, $slug_url);
    		}
    		if(!empty($_GET['s']) && $this->auth->role_id() == 1) { // set listing to spam through URL, Admin Only
		        $this->db->where('id', $id)->update('listings', array('status' => 3));
		        Template::redirect(site_url());
    		}
    		
    		// Load CSS and JS files
    		Assets::add_module_css('comments', 'comments.css');
    		Assets::add_module_js('comments', 'comments.js');
    		
    		// show/hide fields from the detail page
    		$module = 'listings';
    		$show_hide_fields = $this->listings_model->show_hide_fields($result['package'], $result['record'], $module, $user_id);
    		$result['social_sharing'] = gc_social_sharing($result['record']);
    		$result['record'] = $show_hide_fields['record'];
    		$result['package_display_fields'] = $show_hide_fields['package_display_fields'];
    		// do not control module fields via field position method, instead use the display_package fields to avoid unnecessary clicks
    		$result['module_display_fields'] = $show_hide_fields['module_display_fields'];
    		
    		// Variable to Hold Different Values
    		$module = 'listings';
    		$module_id = gc_module_id('listings');
    		
    		$record = $result['record'];
    		$package = $result['package'];
    		$breadcrumb = $result['breadcrumb'];
    		$meta_categories = $result['meta_categories'];
    		
    		// Save Last Visited URL to redirect user back to the Listing page after Claim and Booking Form save
    		// Use the same session variable for redirection from the user's detail and other pages.
    		$this->session->set_userdata('last_visited_url', current_url());
    		
    		// set the login redirect url in case if a user logged in from the listing detail page.
    		$this->session->set_userdata('login_redirect_url', current_url());
    		
    		// Update visits and today's counter
    		$this->load->model('visitors/visitors_model');
    		$this->visitors_model->visitor_tracking(2, array($record->slug, $record->id, $record->city));
    		
    		// REPORTS - LISTING VIEWS
    		if($this->session->userdata('lvs_page') != '') {
    			$lvs_page = $this->session->userdata('lvs_page');
    		} else {
    			$lvs_page = 'Description';
    		}
    		$this->db->simple_query("INSERT INTO " . $this->db->dbprefix . "listing_views (user_id, listing_id, page, keyword, created_on, modified_on) VALUES (" . ($this->session->userdata('user_id') ? $this->session->userdata('user_id') : 0) .", $id, '" . $lvs_page ."', '" . $this->session->userdata('lvs_keyword') ."', '" . date('Y-m-d H:i:s') ."', '" . date('Y-m-d H:i:s') ."') ON DUPLICATE KEY UPDATE times = times + 1, modified_on = '" .date('Y-m-d H:i:s') ."'");		
    		
    		/* ------------------- GOOGLE MAP -----------------------*/
    		if((!empty($record->latitude)) && (!empty($record->longitude))) {
    			$this->load->library('googlemaps');
    			
    			$config['center'] = $record->latitude .', ' .$record->longitude;
    			$config['zoom'] = 12;
    			$this->googlemaps->initialize($config);
    			
    			$marker = array();
    			$marker['position'] =  $record->latitude .', ' .$record->longitude;
    			$this->googlemaps->add_marker($marker);
    			$map = $this->googlemaps->create_map();
    			Template::set('map', $map);
    		}
    		/* ------------------- END GOOGLE MAP --------------------*/    		
    		$id = $record->id;
    		
    		// update listing hits
    		$this->gc->updateHits($id, 'listings');
    		
    		if(settings_item('site.minimal') == 0) {
        		// add/update recently viewed status
        		$this->gc->add_recently_viewed($id, 'listings');
    		}
    		
    		if (!empty($user_id)) {
    			// Check whether listing is user favorite or not
    			$this->load->model('favorites/favorites_model');
    			$favorites = $this->favorites_model->count_by(array('row_id' => $id, 'module_id' => $module_id, 'user_id' => $user_id));
    			if($favorites > 0) {
    			    $result['record']->isFavorite = 1;
    			}
    		}
    		
    		// Do not show claim if the logged in user is the owner of the listing
    		$hide_claim = ($record->user_id == $user_id ? true : false);
    		if(!empty($record->author_slug)) {
    			$record->author_url = gc_url_format('users', $record->author_slug, $record->author_id, $record->author_city);
    		}
    		
    		// Display Banners
    		$searchKeyword = $this->session->userdata('searchKeyword') ? $this->session->userdata('searchKeyword') : '';
    		$searchLocation = '';
    		if($this->session->userdata('location_keyword')) {
    		    $location_session = explode(', ', $this->session->userdata('location_keyword'));
    		    if(!empty($location_session[0])) {
    		        $searchLocation = $location_session[0];
    		    } else {
    		        // (settings_item('adv.search_blocks') == 1 ? $record->city_id : $record->city)
    		        $tl_location = json_decode($this->session->userdata('tl_location') ?? '');
    		        if(!empty($tl_location->name)) {
    		            $searchLocation = $tl_location->name;
    		        }
    		    }
    		}
    		
    		// Open or Closed
    		$record->open_status = 0; // not defined
    		if(!empty($result['business_hours'])) {
    		    $current_day = date('N');
    		    $current_time = date('H:i');
    		    foreach($result['business_hours'] as $business_hour) {
    		        if($business_hour->day_of_week == $current_day) {
    		            if($business_hour->open_time < $current_time && $business_hour->close_time > $current_time) {
    		                $record->open_status = 1; // open
    		            } else {
    		                $record->open_status = 2; // closed
    		            }
    		        }
    		    }
    		}
    		
    		Template::set(array(
    			'listing_id' => $id, // Leads Form
    		    'searchCategory' => isset($breadcrumb->parent_id) ? $breadcrumb->parent_id : 0,
    		    'searchLocation' => $searchLocation,
    		    'searchKeyword' => $searchKeyword,
    			'searchLocality' => $record->locality_id,
    			'price'	=> $package->claim_price,
    			'hide_claim' => $hide_claim,
    			'module' => $module,
    		    'google_business' => false
    		));   
    		
    		// sanitize text
    		if(settings_item('site.minimal') == 1) {
    		    $record->description = gc_sanitize_text($record->description);
    		}
    		
    		// Hardcoded Description
    		if(!empty(settings_item('seo.listing_description'))) {
    		    $meta_data = array(
    		        'page' => 'listing',
    		        'listing_title' => $record->title,
    		        'listing_city' => $record->city,
    		        'listing_state' => $record->state,
    		        'listing_country' => ucwords($record->country)
    		    );
    		    $listing_description = $record->description;
    		    if(empty($listing_description) || strlen($listing_description) < 150) {
    		        $extra_info = '';
    		        $description_meta = $meta_data;
    		        $description_meta['listing_categories'] = $record->categories;
    		        if(!empty($record->phone_number)) {
    		            $extra_info .= '<br /><br />For any queries, call <strong>' . $record->title . '</strong> on <strong>' . $record->phone_number . '</strong>.';
    		        }
    		        if(!empty($record->email)) {
    		            $extra_info .= '<br />You can also send an email to <strong>' . $record->title . '</strong> at <a href="mailto:' . $record->email . '"><strong>' . $record->email . '</strong></a>';
    		        }
    		        $description_meta['extra'] = $extra_info;
    		        $description_meta['type'] = 'seo.listing_description';
    		        if(!empty($listing_description)) {
    		            $result['record']->description = '<strong>' . $listing_description . '</strong><br /><br />' . $this->gc->parse_meta('listing', $description_meta, $description_meta['type']);
    		        } else {
    		            $result['record']->description = $this->gc->parse_meta('listing', $description_meta, $description_meta['type']);
    		        }
    		    }
    		}
    		if(!empty($result['record']->trusted_seal)) {
    		    $result['record']->trusted_seal = base_url('assets/images/' . $result['record']->trusted_seal);
    		}
    		$result['record']->bg_cover = !empty($result['record']->cover) ? base_url($result['record']->cover) : base_url('assets/images/backgrounds/covers/' . rand(1, 5) . '.jpg');
  
    		// Set Variables
    		Template::set($result);
    		
    		// Allow user to add 3 images with a review
    		if(settings_item('lst.review_images') == 1) {
    		    $this->load->library('image_handling');
    		    $this->image_handling->upload_photos('comments');
    		}
    		
    		// Display related listings based on parent category
    		$related_links = $this->listings_model->get_fprlv($module, 'related', 3, $record->city_id, $id);
    		Template::set('related_links', $related_links); // $this->listings_model->related_listings($id, $breadcrumb->parent_id)
		}
		
		// for review and email form
		if($this->auth->is_logged_in()) {
			$user_info = (object) $this->session->userdata('user_info'); //$this->db->select('email, display_name, phone_number')->get_where('users', array('id' => $this->auth->user_id()))->row();
			Template::set('user_info', $user_info);
		}
		
		// categories metas
		if(settings_item('site.categories_modules') == 1 && !empty($record->category_id) && !empty($record->metas)) {
		    $metas = $this->fields_model->category_metas_format($module, $record, 'frontend_detail');
		    Template::set('metas', $metas);
		}
		
		// banner specific session variables to display categories/cities specific banners
		$this->load->model('banners/banners_model');
		$this->banners_model->set_banner_session($record->categories_set);
		$this->banners_model->set_banner_session($record->city_id, 'cities');
		$this->gc->load_query_fields($module); // load business query fields
		$layout = $this->gc->load_assets('modules/listings/detail');
		Template::set_view($layout['view']);
		Template::render($layout['layout']);
	}
	
	/**
	 * Verify listing existence
	 * @param integer $id
	 * @return array|boolean
	 */
	private function verify_existence($id, $slug = null) {
	    if(!empty($slug) || (int)$id) {
	        $module = 'listings';
	        $module_id = gc_module_id($module);
	        $result = array(
	            'package' => new stdClass(),
	            'meta_fields' => array(),
	            'breadcrumb' => new stdClass(),
	            'categories' => array(),
	            'images' => new stdClass(),
	            'gallery' => array(),
	            'videos' => new stdClass(),
	            'business_hours' => new stdClass(),
	            'tags' => new stdClass(),
	            'ratings' => new stdClass(),
	            'branch_offices' => new stdClass(),
	            'comments_count' => 0,
	            'meta_categories' => array(),
	            'module_info' => array(),
	            'booking_allotments' => array(),
	            'page_title' => '',
	            'page_description' => '',
	            'page_keywords' => '',
	            'fields' => array(),
	            'record' => array()
	        );
	        $record = $this->listings_model->get_listing($id, false, $slug);
	        if(!$record || empty($record->slug)) {
	            Template::redirect(site_url());
	        }
	        $id = $record->id;
	        $actual_slug = gc_listing_url($record->slug, $record->id, $record->city);
	        if($record && ($actual_slug != $this->uri->segment(settings_item('route.listings') == 1 ? 1 : 2))) {
	            Template::redirect(gc_url_format('listings', $record->slug, $record->id, $record->city));
	        }
	        
	        // get translations if exists
	        $record = $this->translations->get_translations('listings', $record->id, $record, 1);
	        
	        $package = $this->gcache->get('packages', array('where' => 'id = ' . $record->package_id));
	        if(!$package) { // To fix Listings Import Issue, if the package_id is not entered
	            Template::redirect(site_url());
	        }
	        $result['package'] = $package;
	        
	        // avoid extra queries on each page load
	        if(empty(settings_item('site.minimal'))) {
	            $extra_data = $this->_extra_data($module_id, $id, $record);
	            // $extra_data = json_decode($record->additional_data, true);
	        } else {
    	        if(empty($record->additional_data)) {
    	            $extra_data = $this->_extra_data($module_id, $id, $record);
    	            $this->db->where('id', $id)->update('listings', array('additional_data' => json_encode($extra_data)));
    	        } else {
    	            $extra_data = array();
    	            $data_extra = json_decode($record->additional_data ?? '');
    	            if(!empty($data_extra)) {
    	                foreach($data_extra as $dkey => $dvalue) {
    	                    $extra_data[$dkey] = $dvalue;
    	                }
    	            }    	            
    	            if(!empty($extra_data['module_info'])) {
    	                $extra_data['module_info'] = (array) $extra_data['module_info'];
    	            }
    	        }
	        }
	        if(!empty($extra_data['files'])) {
	            $record->files = (array) $extra_data['files'];
	            unset($extra_data['files']);
	        }
	        $meta_categories = $result['meta_categories'];
	        $result = array_merge($result, $extra_data);
	        
	        // Meta Information
	        if(!empty($record->meta_title)) {
	            $page_title = $record->meta_title;
	        } else if(settings_item('seo.listing_meta_title') != '') {
	            $meta_title = array('site_title' => settings_item('site.title'), 'listing_title' => $record->title, 'listing_city' => $record->city, 'listing_state' => $record->state, 'listing_country' => ucwords(strtolower($record->country)), 'listing_categories' => $meta_categories);
	            $page_title = $this->parser->parse_string(settings_item('seo.listing_meta_title'), $meta_title, TRUE);
	        } else {
	            $page_title = $record->title;
	        }
	        
	        if(!empty($record->meta_description)) {
	            $page_description = $record->meta_description;
	        } else if(settings_item('seo.listing_meta_description') != '') {
	            $meta_description = array('site_title' => settings_item('site.title'), 'listing_title' => $record->title, 'listing_city' => $record->city, 'listing_state' => $record->state, 'listing_country' => ucwords(strtolower($record->country)), 'listing_categories' => $meta_categories);
	            $page_description = $this->parser->parse_string(settings_item('seo.listing_meta_description'), $meta_description, TRUE);
	        } else {
	            $last_space = strrpos( substr( $record->description, 0, 200 ), ' ' ); // find the last space within 35 characters
	            $page_description = strip_tags(substr( $record->description, 0, $last_space ) . '...');
	        }
	        
	        if(!empty($record->meta_keywords)) {
	            $page_keywords = $record->meta_keywords;
	        } else if(settings_item('seo.listing_meta_keywords') != '') {
	            $meta_keywords = array('site_title' => settings_item('site.title'), 'listing_title' => $record->title, 'listing_city' => $record->city, 'listing_state' => $record->state, 'listing_country' => ucwords(strtolower($record->country)), 'listing_categories' => $meta_categories);
	            $page_keywords = $this->parser->parse_string(settings_item('seo.listing_meta_keywords'), $meta_keywords, TRUE);
	        } else {
	            $page_keywords = strip_tags($record->tags);
	        }
	        $result['page_title'] = $page_title;
	        $result['page_description'] = $page_description;
	        $result['page_keywords'] = $page_keywords;
	        
	        $result['record'] = $record;
	        $result['fields'] = $this->fields_model->get_fields($module, 'frontend_index');
	        
	        // Display comments to the User
	        $this->load->model('comments/comments_model');
	        $result['comments_count'] = $this->comments_model->count_by(array('module_id' => $module_id, 'row_id' => $id, 'deleted' => 0, 'status' => 1));	        
	        return $result;
	    } else {
	        Template::redirect(site_url());
	    }	
	    return false;
	}
	
	/**
	 * Avoid Extra Queries on each request 
	 * @param number $module_id
	 * @param number $id
	 */
	private function _extra_data($module_id, $id, $record) {
	    $result = array();
	    // Display meta information
	    $meta_fields = $this->fields_model->find_meta_for_detail('listings', $id);
	    $result['meta_fields'] = $meta_fields;
	    
	    // Display Categories Breadcrumb
	    $breadcrumb = $this->categories_model->get_breadcrumb($id);
	    $result['breadcrumb'] = $breadcrumb;
	    if(!empty($breadcrumb)) {
	        $last_item = end($breadcrumb);
	        $categories = $this->categories_model->get_categories($id, 'listings', $last_item->id);
	        $result['categories'] = $categories;
	    }
	    
	    // Display Images to the User
	    $images = $this->images_model->order_by('priority')->find_all_by(array('module_id' => $module_id, 'row_id' => $id, 'type_id' => 0, 'status' => 1, 'deleted' => 0));
	    $result['images'] = $images;
	    
	    // Merge images and videos for gallery with a new field of type: image | video
	    $gallery = array();
	    if($images) {
	        $gallery_images = array();
	        foreach($images as $image) {
	            $gallery_images[] = array(
	                'id' => $image->id,
	                'title' => $image->title,
	                'url' => $image->url,
	                'type' => 'image'
	            );
	        }
	        $gallery = $gallery_images;
	    };
	    if(gc_module_settings('videos') == 1) {
	        $videos = $this->videos_model->select('id,title,youtube_id,url')->find_all_by(array('row_id' => $id, 'module_id' => $module_id, 'status' => 1, 'deleted' => 0));
	        if($videos) {
	            $gallery_videos = array();
	            foreach($videos as $video) {
	                $gallery_videos[] = array (
	                    'id' => $video->id,
	                    'title' => $video->title,
	                    'url' => $video->youtube_id,
	                    'type' => 'video'
	                );
	            }
	            $gallery = $gallery_videos;
	        }
	        $result['videos'] = $videos;
	        if($images && $videos) {
	            $gallery = array_merge((array) $gallery_images, (array) $gallery_videos);
	        }
	    }
	    $result['gallery'] = $gallery;
	    
	    // Display Business Hours to the User
	    $result['business_hours'] = $this->business_hours_model->select('day_of_week, open_time, close_time')->find_all_by('listing_id', $id);
	    
	    // Display tags to the user
	    $result['tags'] = $this->tags_model->get_tags('listings', $id, TRUE);
	    
	    // Display Ratings to the user
	    $result['ratings'] = $this->ratings_model->select('average_rating, total_ratings, total_users')->find_by(array('row_id' => $id, 'module_id' => $module_id));
	    
	    // Get Branch Office Addresses
	    $result['branch_offices'] = $this->branches_model->get_branches(0, $id, 0, true);
	    
	    // Display comments to the User
	    $result['comments_count'] = $this->comments_model->count_by(array('module_id' => $module_id, 'row_id' => $id, 'deleted' => 0, 'status' => 1));
	    
	    // Meta Categories
	    $meta_categories = $this->categories_model->meta_categories($id);
	    $meta_categories = implode(', ', $meta_categories);
	    $result['meta_categories'] = $meta_categories;
	    
	    // Display listing related module information
	    $result['module_info'] = $this->listings_model->module_info($id, 0, $record);
	    
	    // Display Booking Forms to the User
	    $this->load->model('bookings/bookings_model');
	    $result['booking_allotments'] = $this->bookings_model->get_allotment_ids($module_id, $id);
	    
	    // additional documents added by user
	    $this->load->library('goclixy/gfile');
	    $result['files'] = $this->gfile->saved_files($record->files);
	    
	    return $result;
	}
	
	/**
	 * SAVE PAYMENT INFORMATION IN TRANSACTION TABLE STATUS AS PENDING AND UPDATE IT ON SUCCESSFUL PAYMENT
	 * @param int $listing_id
	 * @param int $package_id
	 * @param string $payment_method
	 * @param float $amount
	 */
	public function insert_transaction($listing_id, $package_id, $payment_method, $amount) {
		$this->load->model('transactions/transactions_model');
		$this->transactions_model->skip_validation(true);
		// get last invoice value
		$invoice_value = $this->transactions_model->select('invoice')->order_by('id', 'desc')->limit(1)->find_all();
		$data = array();
		$data['invoice']        = $invoice_value[0]->invoice + 1; // last invoice plus 1
		$data['package_id']     = $package_id;
		$data['user_id']        = $this->auth->user_id();
		$data['listing_id']     = $listing_id;
		$data['payment_method'] = $payment_method;
		$data['currency']     	= settings_item('site.currency_symbol');
		$data['amount']         = $amount;
		$data['received_on']    = date('Y-m-d H:i:s');
		$data['ip_address']		= $this->input->ip_address();
		return $id = $this->transactions_model->insert($data);
	}
	
	/**
	 * Deal with AJAX error responses 
	 * @param string $message
	 * 
	 */
	function errorResponse ($messsage) {
		header('HTTP/1.1 500 Internal Server Error');
		die(json_encode(array('message' => $messsage)));
	}

	/**
	 * Display captcha to the user
	 * securimage plugin is used to implement below given function
	 * 
	 */
	public function securimage() {
		header("Content-type: image/jpeg");
		$this->load->config ( 'csecurimage' );
		$active = $this->config->item ( 'si_active' );
		$allsettings = array_merge ( $this->config->item ( $active ), $this->config->item ( 'si_general' ) );
		$this->load->library ( 'securimage/securimage' );
		$img = new Securimage ( $allsettings );
		$img->show ( APPPATH . 'libraries/securimage/backgrounds/bg4.png' );
	}
	
	/**
	 * Check user entered captcha against provided image
	 * @param string $fieldname ($this->input->post())
	 * @return boolean
	 */
	public function check_captcha($fieldname) {
		$this->load->library ( 'securimage/securimage' );
		$securimage = new Securimage ();
		if (! $securimage->check ( $this->input->post ( $fieldname ) )) {
			return FALSE;
		} else {
			return TRUE;
		}
	}
	
	/**
	 * KAPSYSTEM SEND SMS FUNCTION 
	 * PHP code provided by KapSystem
	 * @param string $url
	 * 
	 */
	public function kapsystem_openurl($url) {
	    if(is_callable("curl_exec"))  {
		    // Use CURL if its available
		    $ch = @curl_init($url);
		    curl_setopt($ch, CURLOPT_POST, 1);
		    curl_setopt($ch, CURLOPT_TIMEOUT, 60);
		    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		    $result = curl_exec($ch);
	    } else {
	    // Use fopen instead
	    if($fp = @fopen($url, "rb")) {
		    $result = "";
		    while(!feof($fp)) {
		    	$result .= fgets($fp, 4096);
		    }
	    	@fclose($fp);
	    }
	   }
	}
	
		
	/**
	 * CREATE PDF FILE
	 * Creates a Business Listing PDF document using TCPDF
	 * @package com.tecnick.tcpdf
	 * @abstract TCPDF - Example: Default Header and Footer
	 */	
	public function create_pdf($slug = NULL, $module = 'listings') {
		// get all characters from right to left till first dash to get listing id
	    $row_id = substr( $slug, strrpos( $slug, '-' ) + 1);
	    if((int) $row_id == NULL) {
			redirect(site_url());	
		}
		$this->gemailer->sms_email_pdf('pdf', array('template_table' => 'module_email_templates', 'module_table' => $module, 'action_type' => 'share', 'with_default' => 1, 'id' => $row_id));
	}
	
	/**
	 * Category Browsing
	 * @param string $slug
	 * @param int $offset for pagination
	 */
	public function category($c1, $c2 = '', $c3 = '', $c4 = '', $c5 = '', $c6 = '', $c7 = '') {
	    $slug_uris = '';
	    if(!empty($c7)) {
	        $slug_uris = $c1 . '/' . $c2 . '/' . $c3 . '/' . $c4 . '/' . $c5 . '/' . $c6 . '/' . $c7;
	    } else if(!empty($c6)) {
	        $slug_uris = $c1 . '/' . $c2 . '/' . $c3 . '/' . $c4 . '/' . $c5 . '/' . $c6;
	    } else if(!empty($c5)) {
	        $slug_uris = $c1 . '/' . $c2 . '/' . $c3 . '/' . $c4 . '/' . $c5;
	    } else if(!empty($c4)) {
	        $slug_uris = $c1 . '/' . $c2 . '/' . $c3 . '/' . $c4;
	    } else if(!empty($c3)) {
	        $slug_uris = $c1 . '/' . $c2 . '/' . $c3;
	    } else if(!empty($c2)) {
	        $slug_uris = $c1 . '/' . $c2;
	    } else {
	        $slug_uris = $c1;
	    }
	    $slug = settings_item('site.category_slug_uris') == 1 ? $slug_uris : $c1;
	    $this->listings_model->category_location_tag_search(false, 'listings', 'categories', $this->session->userdata('search_country'), $slug, $this->limit);
	}
	
	/**
	 * Tag Browsing
	 * @param string $slug
	 * @param int $offset for pagination
	 */
	public function tag($slug) {
		$this->listings_model->category_location_tag_search(false, 'listings', 'tags', $this->session->userdata('search_country'), $slug, $this->limit);
	}
	
	/**
	 * Allow user to browse by categories
	 * @param $_GET['md'] - ID of the module whose categories are to be loaded
	 */
	public function categories() {
		$module_id = 1;
		// check whether the passed value is a module with categories
		if(!empty($_GET['md']) && gc_module_settings(gc_module_name(intval($_GET['md']))) == 1) {
			$module_id = intval($_GET['md']);
		}
		$categories = $this->categories_model->load_categories($module_id);
		Template::set(array(
		    'toolbar_title' => lang('home_category_browse'),
		    'categories' => $categories,
		    'module' => gc_module_name($module_id)
		));
		$layout = $this->gc->load_assets('categories');
		Template::set_view($layout['view']);
		Template::render($layout['layout']);
	}
	
	/**
	 * Location Browsing
	 * @param string $slug
	 * @param int $offset for pagination
	 */
	public function location($slug) {
		$this->listings_model->category_location_tag_search(false, 'listings', 'cities', $this->session->userdata('search_country'), $slug, $this->limit);
	}
	
	/**
	 * Locality Browsing
	 * @param string $slug
	 */
	public function locality($city, $slug) {
	    $this->listings_model->category_location_tag_search(false, 'listings', 'localities', $this->session->userdata('search_country'), $slug, $this->limit);
	}
	
	/**
	 * Allow user to browse by locations
	 */
	public function locations() {
		$module_id = 1;
		$table = 'states';
		// check whether the passed value is a module with categories
		if(!empty($_GET['md'])) {
			if(gc_module_settings(gc_module_name(intval($_GET['md'])), 'locations') == 1) {
				$module_id = intval($_GET['md']);
			} else if(intval($_GET['md']) != 1) {
				Template::redirect(site_url());
			}
		}
		// display cities on the locations page
		if(!empty($_GET['type']) && $_GET['type'] == 1) {
		    $table = 'cities';
		}
		$where = array();
		$where[] = 'active = 1';
		if(settings_item('lst.country_filter') == 1) {
		  $where[] = "country_iso = '" . $this->session->userdata('search_country') . "'";
		} else {
			// get active countries
			$ids = array();
			$countries = $this->gcache->get('countries', array('where' => 'active = 1', 'result' => 1));
			if($countries) {
				$country_isos = array();
				foreach($countries as $country) {
					$ids[] = $country->id;
					$country_isos[] = "country_iso = '" . $country->iso . "'";
				}
				$where[] = '(' . implode(' OR ', $country_isos) . ')';
				// $where[] = 'country_id IN (' . implode(',', $ids) . ')';
			}
		}
		$this->load->model('locations/states_model');
		if(!empty(settings_item('lst.hide_counts_zero_locations'))) {
		    $where[] = 'counts > 0';
		}
		$records = $this->db->where(implode(' AND ', $where))->order_by('name', 'asc')->get($table)->result();
		Template::set( array (
		    'toolbar_title' => lang('home_location_browse'),
		    'type' => $table,
		    'records' => $records,
			'module' => gc_module_name($module_id)
		));		
		$layout = $this->gc->load_assets('locations');
		Template::set_view($layout['view']);
		Template::render($layout['layout']);
	}
	
	/**
	 * Search or browse module using Tag name or ID
	 * Pass md as the GET parameter to convert search from listings to a module
	 * e.g. &md=2 - Search Products
	 * 
	 * @param string $city
	 * @param string @search_string actual keyword
	 * @param int @offset for pagination
	 * 
	 */
	public function search() {
		if(empty($_GET)) {
			redirect(site_url());
		}
		$module_id = 1;
		// check whether the passed value is a module
		if(!empty($_GET['md']) && gc_module_settings(gc_module_name(intval($_GET['md']))) == 1) {
			$module_id = intval($_GET['md']);
		}
		$module = gc_module_name($module_id);
		
		// allow user to perform search within a specific country except than the default one
		$country = $this->session->userdata('search_country');
		if(!empty($_GET['iso']) && strlen($_GET['iso']) == 2) {
			$country = $_GET['iso'];
		}
		/**
		 * The first page results will be loaded without AJAX
		 * This requires not extra parameter ajax pass in the GET parameters
		 * 
		 * ALSO HELPS IN
		 * 
		 * if the module has no filter fields, the result will be displayed normally
		 * If the search is done with ajax, it will be difficult to load filter fields, we also have to load filter fields dynamically using AJAX
		 * We need to add a mechanism to clear the result if the module is changed.
		 */
		$this->listings_model->category_location_tag_search(false, $module, 'search', $country, NULL, $this->limit);
	}
	
	/**
	 * Create slug from $text string
	 * @param string $text
	 * @return string|mixed
	 */
	public function slugify($text) {
		// replace non letter or digits by -
		$text = preg_replace ( '~[^\\pL\d]+~u', '-', $text );		
		// trim
		$text = trim ( $text, '-' );		
		// transliterate
		$text = iconv ( 'utf-8', 'us-ascii//TRANSLIT', $text );		
		// lowercase
		$text = strtolower ( $text );		
		// remove unwanted characters
		$text = preg_replace ( '~[^-\w]+~', '', $text );		
		if (empty ( $text )) {
			return 'n-a';
		}		
		return $text;
	}
		
	/**
	 * Get random values from an associative array for featured listing
	 * @param array $arr
	 * @param number $num
	 * @return multitype:array
	 */
	function array_random_assoc($arr, $num = 1) {
		$keys = array_keys($arr);
		shuffle($keys);
		$r = array();
		for ($i = 0; $i < $num; $i++) {
			$r[$keys[$i]] = $arr[$keys[$i]];
		}
		return $r;
	}
	
	/**
	 * Sort array based on value
	 */
	function sortByPrice($x, $y) {
		return $y['price'] - $x['price'];
	}
	
	/**
	 * Storing search term for pagination
	 * @param string $searchterm
	 * @return string searchterm
	 */
	public function searchterm_handler($searchterm) {
		if($searchterm) {
			$this->session->set_userdata('search_term', $searchterm);
			return $searchterm;
		} elseif($this->session->userdata('search_term')) {
			$searchterm = $this->session->userdata('search_term');			
			return $searchterm;
			$this->session->unset_userdata('search_term');
		} else {
			$searchterm ="";
			return $searchterm;
		}
	}
	
	/**
	 * Storing locality information
	 * @param string $search_locality
	 * @return string search_locality
	 */
	public function locality_handler($search_locality) {
		if($search_locality) {
			$this->session->set_userdata('search_locality', $search_locality);
			return $search_locality;
		} elseif($this->session->userdata('search_locality')) {
			$search_locality = $this->session->userdata('search_locality');
			return $search_locality;
			$this->session->unset_userdata('search_locality');
		} else {
			$search_locality ="";
			return $search_locality;
		}
	}
	
	/**
	 * GET STATES AND CITIES (AJAX)
	 * @param string $country_iso
	 * @return AJAX
	 */
	public function loadData($country_iso)
	{
		header('Content-Type: application/x-json; charset=utf-8'); //important to include because of json
		$result = $this->states_model->find_all_by(array('country_iso' => $country_iso, 'active' => 1));
		$output = json_decode(json_encode($result),true);
		echo json_encode($output);
	}
	
	/**
	 * Retrieve localities using city id
	 * @param int $city_id
	 * @return AJAX
	 */
	public function loadLocalities()
	{
		$city_id = escape($this->input->get('ci'));
		$this->session->set_userdata('search_city', $city_id);
		header('Content-Type: application/x-json; charset=utf-8'); //important to include because of json
		$result = $this->localities_model->find_all_by(array('city_id' => $city_id, 'active' => 1));
		$output = json_decode(json_encode($result),true);
		echo json_encode($output);
	}
	
	/**
	 * Get thumbnail file by adding _thumb
	 * @param $filename
	 */
	
	public function get_thumb($filename) {
		preg_match('/(?<extension>\.\w+)$/im', $filename, $matches);
		$extension = $matches['extension'];
		return preg_replace('/(\.\w+)$/im', '', $filename) . '_thumb' . $extension;
	}
	
	/**
	 * Generate Module PDF
	 */
	public function generate_pdf($module, $id) {
	    $this->gemailer->sms_email_pdf('pdf', array('template_table' => 'module_email_templates', 'module_table' => $module, 'action_type' => 'share', 'with_default' => 1, 'id' => $id));
	}
	
	public function getCity() {
		//header('Content-Type: application/x-json; charset=utf-8'); //important to include because of json
		//$result = $this->cities_model->find_all_by(array('name' => $this->input->get('q'), 'active' => 1));
		$this->db->like('name', escape($this->input->get('qr')));
		$result = $this->db->select('id, name')->get($this->db->dbprefix .'cities')->result();
		//$output = json_decode(json_encode($result),true);
		echo json_encode($result);
	}
	
	public function getCityState($zip, $blnUSA = true) {
		$url = "https://maps.googleapis.com/maps/api/geocode/json?address=" . $zip . "&sensor=true";
	
		$address_info = file_get_contents($url);
		$json = json_decode($address_info ?? '');
		$city = "";
		$state = "";
		$country = "";
		if (count($json->results) > 0) {
			//break up the components
			$arrComponents = $json->results[0]->address_components;
	
			foreach($arrComponents as $index=>$component) {
				$type = $component->types[0];
	
				if ($city == "" && ($type == "sublocality_level_1" || $type == "locality") ) {
					$city = trim($component->short_name);
				}
				if ($state == "" && $type=="administrative_area_level_1") {
					$state = trim($component->short_name);
				}
				if ($country == "" && $type=="country") {
					$country = trim($component->short_name);
	
					if ($blnUSA && $country!="US") {
						$city = "";
						$state = "";
						break;
					}
				}
				if ($city != "" && $state != "" && $country != "") {
					//we're done
					break;
				}
			}
		}
		$arrReturn = array("city"=>$city, "state"=>$state, "country"=>$country);
	
		die(json_encode($arrReturn));
	}
	
	/*******************************************************/
	/* START OF MODULES
	/*******************************************************/
	/**
	 * Set routes for all modules using the below given method
	 * e.g. <MODULE NAME> = 'listings/view/<MODULE NAME>/search'
	 * e.g. <MODULE NAME>/categories/slug = 'listings/view/<MODULE NAME>/categories/$1'
	 * e.g. <MODULE NAME>/cities/slug = 'listings/view/<MODULE NAME>/cities/$1'
	 * e.g. <MODULE NAME>/localities/slug = 'listings/view/<MODULE NAME>/localities/$1'
	 * @param string $module
	 * @param string $type
	 * @param string $slug
	 */	
	public function view($module, $type, $slug = null) {
	    if(!in_array($module, gc_modules()) && $module != 'videos') {
	        Template::redirect(site_url());
	    }
	    $this->listings_model->category_location_tag_search(false, $module, ($type == 'location' ? 'cities' : $type), $this->session->userdata('search_country'), $slug, $this->limit);
	}
	
	/**
	 * Display module description to the visitor
	 * Set routes as follows:
	 * e.g. <MODULE NAME>/detail/$1 = 'listings/describe/<MODULE NAME>/$1'
	 * @param string $slug
	 */
	public function describe($module, $slug = NULL) {
	    if(!in_array($module, gc_modules()) && $module != 'videos') {
	        Template::redirect(site_url());
	    }
		if($slug == NULL) {
			show_404();
		}

		//rating star
		Assets::add_js('star-rating.min.js');
		Assets::add_css('star-rating.min.css');
		
		// Multilanguage Ratings
		Assets::add_js( $this->load->view('common/ratings_js', null, true ), 'inline' );
		
		$this->gc->module_detail($module, $slug, intval($this->auth->user_id()));
	}
	/*******************************************************/
	/* END OF MODULES
	/*******************************************************/
	public function filter() {
		$module = 'properties';
		$field_type = 'admin_index';
		$form_fields = $this->fields_model->get_fields($module, $field_type);
		$this->fields_model->field_names($module, $field_type, $form_fields);
	}
	
	
	/**
	 * Set Non Zero Dates in all tables
	 */
	private function non_zero_dates() {
	    $tables = $this->db->list_tables();
	    $fields = array('created_on', 'modified_on');
	    foreach ($tables as $table) {
	        foreach($fields as $field) {
	            if ($this->db->field_exists($field, $table)) {
	                $this->db->query("ALTER TABLE `" . $table . "` CHANGE `" . $field . "` `" . $field . "` DATETIME NULL DEFAULT CURRENT_TIMESTAMP;");
	            }
	        }
	    }
	}
	
	/**
	 * Displays the homepage of the Bonfire app
	 *
	 * @return void
	 */
	public function form() {
	    $module = 'listings';
	    $row_id = 0;
	    if(isset($_POST['save'])) {
	        dump($_POST);
	    }
	    // image uploading
	    $this->load->library('goclixy/gfile');
	    $this->gfile->tmp_id();
	    $field_attributes = array(
	        'logo-1' => array(
	            'limit' => 1,
	            'types' => 'jpg|jpeg|png',
	            'template_id' => 1,
	            'with_thumbnail' => 1
	        ),
	        'cover-1' => array(
	            'limit' => 1,
	            'types' => 'jpg|jpeg|png',
	            'template_id' => 1,
	            'with_thumbnail' => 1
	        ),
	        'gallery-1' => array(
	            'limit' => 2,
	            'types' => 'jpg|jpeg|png',
	            'template_id' => 1,
	            'with_thumbnail' => 1
	        )
	    );
	    $this->session->set_userdata('gfile', $field_attributes);
	    
	    $files = $this->gfile->get_files($module, $row_id, array('template' => 1, 'with_thumbnail' => 1));
	    $this->session->set_flashdata('uploaded_files', $files);
	    Assets::add_css('selectize.bootstrap3.css');
	    Assets::add_js('selectize.min.js');
	    Assets::add_js('selectize-search.js');
	    Assets::add_js('location-selection.js');
	    Assets::add_js('google-map.js');
	    Assets::add_js('https://unpkg.com/esri-leaflet@2.5.1/dist/esri-leaflet.js');
	    Assets::add_css('https://cdnjs.cloudflare.com/ajax/libs/esri-leaflet-geocoder/2.3.3/esri-leaflet-geocoder.css');
	    Assets::add_js('https://cdnjs.cloudflare.com/ajax/libs/esri-leaflet-geocoder/2.3.3/esri-leaflet-geocoder.js');
	    Assets::add_css('jquery.dm-uploader.min.css');
	    Assets::add_js('jquery.dm-uploader.min.js');
	    Assets::add_js('jquery.dm-functions.js');
	    Assets::add_js('moment.js');
	    Assets::add_css('bootstrap-datetimepicker.min.css');
	    Assets::add_js('bootstrap-datetimepicker.min.js');
	    Assets::add_module_js('listings', 'listings.js');
	    $latitude = null;
	    $longitude = null;
	    $map = $this->gc->google_map($latitude, $longitude);
	    $country_options = array(
	        0 => lang('placeholder_choose')
	    );
	    $this->session->set_userdata('files_limit_listings_0', 50);
	    $countries = $this->db->where(array('active' => 1))->get('countries')->result();
	    if($countries) {
	        foreach($countries as $country) {
	            $country_options[$country->iso] = $country->name;
	        }
	    }
	    // Template::set('map', $map);
	    Template::set(array(
	        'country_options' => $country_options,
	        'countries' => $countries,
	        'map' => $map,
	        'fileuploader_langs' => true
	    ));
	    $layout = $this->gc->load_assets('members/forms');
	    Template::set_view($layout['view']);
	    Template::render($layout['layout']);
	}//end index()
	
	/**
	 * Change Country
	 */
	public function change_country() {
	    if(!empty($_GET['iso']) && strlen($_GET['iso']) == 2) {
	        $site_country_iso = 'IN';
	        $country_iso = $_GET['iso'];
	        $parsed_url = parse_url(site_url());
	        if(!empty($parsed_url['host'])) {
	           $host_array = explode('.', $parsed_url['host']);
    	        if(strlen($host_array[0]) == 2) {
    	           $site_country_iso = strtoupper($host_array[0]);
    	        }
	        }
	        if($site_country_iso == $country_iso) {
	            Template::redirect(current_url());
	        } else {
	            if($country_iso == 'IN') {
	                Template::redirect('https://www.ibizexpert.com');
	            } else {
	                Template::redirect('https://' . strtolower($country_iso) . '.ibizexpert.com');
	            }
	        }
	    } else {
	        Template::redirect(site_url());
	    }
	}
	
	/**
	 * Display countries list to the user
	 * @param string $filter - Letter Filter
	 */
	public function countries_list($filter = null) {
	    $uri_string = 'countries';
	    $query_params = array();
	    $where = 'is_online = 1';
	    if (preg_match('{first_letter-([A-Z])}', $filter, $matches)) {
	        $filter = 'first_letter';
	        $first_letter = $matches[1];
	        $query_params[] = 'letter=' . $first_letter;
	        $where .= " AND SUBSTRING( LOWER(name), 1, 1) = '". $first_letter ."'";
	    }
	    if(!empty($query_params)) {
	        $uri_string .= '?' . implode('&', $query_params);
	    }
	    // $records = $this->gcache->curl_data($uri_string);
	    $records = $this->gcache->get('countries', array('where' => $where, 'order_by' => 'priority ASC', 'result' => 1)); 
	    $toolbar_title = lang('live_sites');
	    Template::set(array(
	        'toolbar_title' => $toolbar_title,
	        'page_title' => $toolbar_title,
	        'records' => $records,
	        'filter_type' => $filter,
	        'index_url' => site_url('countries') . '/'
	    ));
	    $this->gc->demo_custom_css_js('index');
	    Template::render();
	}
}