<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
 || ##################################################################### ||
 || # GoClixy - Business Directory Script							   # ||
 || # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
 || # ----------------------------------------------------------------  # ||
 || # This file is owned and copyrighted by GoClixy.com. Your license   # ||
 || # confers  no  title  or ownership in the Script and is not a sale  # ||
 || # of any rights to the Software. This file may not be redistributed # ||
 || # in whole or significant part.									   # ||
 || # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
 || # http://www.goclixy.com | http://www.goclixy.com/license.html 	   # ||
 || ##################################################################### ||
 \*=======================================================================*/
require_once APPPATH . "/third_party/google-places/googlePlaces.php";
class Google_listings {
    /**
     * Stores the CodeIgniter core object.
     *
     * @access public
     *
     * @var object
     */
    public $CI;
    public $z_cache;
    protected $table_name = 'google_listings';
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct($config = array()) {
        $this->CI =& get_instance();
        $this->CI->load->library('Gzf_cache', array('lifetime' => 900));
        $this->CI->load->library('slug');
        $this->z_cache = $this->CI->gzf_cache->get_instance();
    }

    /**
     * Google Places Search
     * @param string $token
     */
    public function google_search($keyword, $location = null) {
        $pagination   = false;
        $apiKey       = settings_item('site.google_server_api_key');
        $googlePlaces = new googlePlaces($apiKey);
        $nextPage     = false;
        $token = $this->CI->session->userdata('next_page_token');
        $listings = array('listings' => array(), 'markers' => array());
        if(!empty($location)) {
			if(!empty($keyword)) {
				$query = $keyword . ' in ' . $location;
			} else {
				$query = '';
			}
        } else {
            $query = $keyword;
            $location = '';
        }
        $per_page = isset($_GET['per_page']) ? (int) $_GET['per_page'] : 1000;
        if(!isset($_GET['search']) && !empty($token)) { // Pagination
            $cache_id = 'google_places_' . str_replace('-', '_', $this->CI->slug->create_slug($query . '_' . $per_page));
            // $cache_id = 'google_places_' . str_replace('-', '_', $this->CI->slug->create_slug($query . '_' . $token)); // token caching issue
            if(!$listings = $this->z_cache->load($cache_id)) { // store results in session
                $results = $googlePlaces->repeat($token);
                $listings = $this->listing_rows($results, $keyword, $location);
                $this->z_cache->save($listings, $cache_id);
            }
        } else if(!empty($query) || !empty($location)) {
            $cache_id = 'google_places_' . str_replace('-', '_', $this->CI->slug->create_slug((!empty($query) ? $query : $location) . '_' . $per_page));
            // $cache_id = 'google_places_' . str_replace('-', '_', $this->CI->slug->create_slug($query));
            if(!$listings = $this->z_cache->load($cache_id)) { // store results in session
                $googlePlaces->setQuery(!empty($query) ? $query : 'in ' . $location);
                $results = $googlePlaces->textSearch();
                $listings = $this->listing_rows($results, $keyword, $location);
                $this->z_cache->save($listings, $cache_id);
            }
        }        
        
        if(!empty($location)) {
            // Save place_id searches for SEO purposes
            $city = null;
            $state = null;
            $country = null;
            $get_location = explode(', ',$location);
            if(!empty($get_location[0])) {
                $city = $get_location[0];
            }
            if(!empty($get_location[1])) {
                $state = $get_location[1];
            }
            if(!empty($get_location[2])) {
                $country = $get_location[2];
            }
        }
        
        /**
         * Pagination handling with the help of a session
         */
        if (!empty($listings['next_page_token'])) {
            $pagination = true;
            $this->CI->session->set_userdata('next_page_token', $listings['next_page_token']);
        } else {
            $this->CI->session->unset_userdata('next_page_token');
        }
        
        /**
         * Display Google Map with multiple markers
         * In order to avoide duplicate positions, a new array $markers is declared
         */
        /*if(settings_item('lst.view_map') == 1) {
            $this->CI->load->library('listings/googlemaps');
            $config['center'] = '' . settings_item('lst.latitude') . ', ' . settings_item('lst.longitude') . '';;
            $config['zoom'] = 'auto';
            $config['places'] = TRUE; // Customer Needs Form
            $this->CI->googlemaps->initialize($config);
            $markers = $listings['markers'];
            if($markers) {
                foreach($markers as $marker) {
                    $this->CI->googlemaps->add_marker($marker);
                }
            }
            $map = $this->CI->googlemaps->create_map();
            Template::set('map', $map);
        }*/
        return $listings;
    }
    
    /**
     * Display result to the user by using the existing listing views
     * @param array $results - Google Results
     * @return multitype:multitype:NULL string array Ambigous <number, array> Ambigous <string, array>
     */
    private function listing_rows ($results, $keyword = null, $location = null) {
        $this->CI->load->library('slug');
        $listings = array();
        $markers = array();
        $i = 1;
        $next_page_token = !empty($results['next_page_token']) ? $results['next_page_token'] : '';
        if(!empty($results['result'])) {
            foreach($results['result'] as $result) {
                if(!empty($result['name'])) {
                    $listing_id = 0;
                    $address = $this->formatted_address($result['formatted_address']);
                    
                    // Create Unique Slug and save for SEO purposes
                    $slug_existence = $this->CI->db->select('id, slug')->get_where($this->table_name, array('place_id' => $result['place_id']))->row();
                    if($slug_existence) {
                        $slug = $slug_existence->slug;
                        $this->CI->db->where('id', $slug_existence->id)->update($this->table_name, array('modified_on' => date('Y-m-d H:i:s'), 'modified_ip' => $this->CI->input->ip_address()));
                    } else {
                        $slug = $this->CI->gc->slugify($this->table_name, $result['name'] . ' ' . $address['city'], 0);
                        $places_slug = array(
                            'place_id' => $result['place_id'],
                            'slug' => $slug,
                            'created_ip' => $this->CI->input->ip_address(),
                            'created_on' => date('Y-m-d H:i:s')
                        );
                        $insert_query = $this->CI->db->insert_string($this->table_name, $places_slug);
                        $insert_query = str_replace('INSERT INTO','INSERT IGNORE INTO',$insert_query);
                        $this->CI->db->query($insert_query);
                    }
                    $tags = '';
                    $categories = !empty($result['types']) ? $this->get_categories($result['types'], $keyword, $location) : '';
                    if(!empty($categories)) {
                        $tags = implode(', ', $categories);
                    }
                    if(!empty($result['photos'])) {
                        $listing_image = $this->get_photos($result['photos'])[0]->url;
                    }
                    /*if(empty($listing_image)) {
                        $listing_image = gc_site_image('no-image');
                    }*/
                    $trusted_seal = !empty($result['icon']) ? $result['icon'] : '';
                    $object = (object) array(
                        'redirect_url' => site_url(gc_uri_string('detail') . '/' . $slug . '?g=1'),
                        'tags' => $tags,
                        'id' => $i, // Temporary ID
                        'image' => empty($listing_image) ? $trusted_seal : $listing_image,
                        'title' => $result['name'],
                        'slug' => $slug,
                        'description' => '',
                        'address' => $result['formatted_address'],
                        'city_id' => 0,
                        'city' => $address['city'],
                        'state' => $address['state'],
                        'country' => $address['country'],
                        'postcode' => !empty($address['postcode']) ? $address['postcode'] : null,
                        'latitude' => $result['geometry']['location']['lat'],
                        'longitude' => $result['geometry']['location']['lng'],
                        'email' => '',
                        'color_scheme' => '',
                        'border_color' => '',
                        'trusted_seal' => '', //$trusted_seal, // Icon changed to logo_url to reduce the number of hits of URL
                        'phone_number' => '',
                        'mobile_number' => '',
                        'hits' => 0,
                        'average_rating' => !empty($result['rating']) ? $result['rating'] : 0,
                        'place_id' => $result['place_id'],
                        'open_now' => !empty($result['opening_hours']['open_now']) ? $result['opening_hours']['open_now'] : '',                     
                        'created_on' => date('Y-m-d H:i:s'),
                        'author_name' => 'Google',
                        'author_url' => '',
                        'author_image' => '',
                        'locality' => '', // API does not return locality
                        'categories' => $categories,
                        'google_business' => true // Google Listing
                    );
                    
                    $markers[] = array(
                        'position' => $result['geometry']['location']['lat'] . ', ' . $result['geometry']['location']['lng'],
                        'infowindow_content' => '<a href="'. $this->detail_url($slug). '" title="' . $result['name'] . '"><strong>' . $result['name'] . '</strong></a><br />' . $address['address'] .', ' . $address['city'] .', '. $address['state'] . ' ' . $address['postcode'] . ', ' . $address['country'],
                        'animation' => 'DROP'
                    );
                    $listings[] = $object;
                    $i++;
                }
            }
            return array('listings' => $listings, 'markers' => $markers, 'next_page_token' => $next_page_token);
        } else {
            return false;
        }
    }
    
    
    /**
     * Split address for listing view page
     * @param string $address
     * @return multitype:
     */
    private function formatted_address($formatted_address) {
        // $address = 'MDR108, ITI Chowk,, Model Town, Hisar, Haryana 125005, India';
        $result = array();
        $address = array();
        $address_array = explode(",", $formatted_address ?? '');
        foreach($address_array as $value) {
            if(!empty($value)) {
                $address[] = trim($value);
            }
        }
        $length = count($address);
        if($length >= 3) {
            $result['city'] = $address[$length - 3];
        } else {
            $result['city'] = '';
        }
        if($length >= 2) {
            $state = $address[$length - 2];
            $state = explode(" ", $state ?? '');
            $postcode = end($state);
            $result['postcode'] = $postcode;
            array_pop($state); // Remove Postcode from state
            $result['state'] = implode(" ", $state);
        } else {
            $result['postcode'] = '';
            $result['state'] = '';
        }
        $result['country'] = $address[$length - 1];
        array_pop($address); // Remove Country
        array_pop($address); // Remove State and Postcode
        array_pop($address); // Remove city information
        $result['address'] = implode(", ", $address); // Convert to String
        return $result;
    }
    
    /**
     * Get categories returned from the Google API
     * @param array $types
     * @return <string, multitype:unknown >
     */
    private function get_categories($types, $keyword = null, $location = null) {
        $categories = array();
        foreach($types as $category) {
            if($category != 'point_of_interest' && $category != 'establishment') {
                $url = str_replace('_', ' ', $category);
                if(!empty($location)) {
                    if($keyword == $url) {
                        $url = '<strong>' . $url . '</strong>';
                    } else {
                        $url = anchor(gc_url_format('search') . '?q=' . urlencode($url) . '&l=' .urlencode($location), $url, 'title="'. $url .'"');
                    }
                }
                $categories[] = $url;
            }
        }
        // keyword will only be returned in case of search, but detail page will never return the keyword
        return !empty($categories) ? $categories: '';
    }
    
    /**
     * Get all photos of this particular listing
     * @param array $photos
     * @return multitype:multitype:string
     */
    private function get_photos($photos, $listing_id = 0) {
        $images = array();
        $i = 1;
        foreach($photos as $photo) {
            $url = 'maxwidth=' . $photo['width'] .  '&maxheight=' . $photo['height'] . '&photoreference=' . $photo['photo_reference'];
            if($listing_id == 0) {
                $images[] = (object) array(
                    'id' => $i,
                    'title' => '',
                    'url' => 'https://maps.googleapis.com/maps/api/place/photo?' . $url . '&key=' . settings_item('site.google_server_api_key'),
                    'type' => 'image'
                );
            } else {
                $images[] = array(
                    'id' => $i,
                    'title' => '',
                    'listing_id' => $listing_id,
                    'url' => $url,
                    'google' => 1, // process google images once a day using cronjob and set this field value to 0 after image download,
                    'type' => 'image'
                );
            }
            $i++;
        }
        return $images;
    }
    
    /**
     * Check whether any listing exists with this place_id in the database
     * if not create a new listing and redirect user to the detail page
     * @param string $place_id
     */
    public function detail($place_id) {
        // Cache records
        $cache_id = str_replace('-', '_', $this->CI->slug->create_slug($place_id));
        if(!$data = $this->z_cache->load($cache_id)) {
            $data = $this->get_detail($place_id);
            $this->z_cache->save($data, $cache_id);
        }
        if(!$data) {
            Template::redirect(site_url());
        }
        Template::set($data);
        return $data;
    }
    
    /**
     * Prepare listing information
     * @param string $place_id
     * @return boolean
     */
    private function get_detail($place_id) {
        if($place_id == null) {
            Template::redirect(site_url());
        }
        
        $keyword = $this->CI->session->userdata('searchKeyword');
        $location = $this->CI->session->userdata('searchLocation');
        /**
         * Create a new table for extra fields Google
         * Store only upto 3 levels of categories
         * Must enable security so that request from single IP should not exceed 100 views
         * If the API limit is over, display no record found instead of any error message.
         * If the request call is from API, display open_now field else, do not show
         * Store view results also in database
         * if category is empty, use search keyword as the category name
         */
        require_once APPPATH . "/third_party/google-places/googlePlaces.php";
        $this->CI->load->library('slug');
        $this->CI->load->library('parser');
        $googlePlaces = new googlePlaces(settings_item('site.google_server_api_key'));
        $googlePlaces->setPlaceId($place_id);
        $result = $googlePlaces->details();
        if(empty($result['result'])) {
            Template::redirect(site_url());
            return false;
        }        
        $result = $result['result'];
        $phone_number = '';
        $mobile_number = '';
        if(!empty($result['international_phone_number'])) {
            $unique_numbers = $this->compare_numbers($result['international_phone_number'], $result['formatted_phone_number']);
            if($unique_numbers) {
                $phone_number = $result['international_phone_number'];
            } else {
                $phone_number = $result['international_phone_number'];
                $mobile_number = $result['formatted_phone_number'];
            }
        }
        $address = $this->get_address($result['address_components']);
        
        $ratings = array(            
            'average_rating' => !empty($result['rating']) ? $result['rating'] : 0,
            'total_users' => !empty($result['user_ratings_total']) ? $result['user_ratings_total'] : 0,
            'total_ratings' => !empty($result['user_ratings_total']) ? $result['user_ratings_total'] : 0
        );
        
        // Display images to the user
        $images = !empty($result['photos']) ? $this->get_photos($result['photos']) : '';        
        
        // Display logo image to the user
        $logo_url = !empty($result['icon']) ? $result['icon'] : '';
        if(!empty($images)) {
            $logo_url = $images[0]->url; // Display first image as the logo
        }
        
        $view = array (
            'id' => 0,
            'place_id' => $result['place_id'],
            'title' => $result['name'],
            'slug' => $this->CI->slug->create_slug($result['name']),
            'description' => '',
            'phone_number' => $phone_number,
            'mobile_number' => $mobile_number,
            'address' => $this->return_address($result['formatted_address']),
            'city' => $address['city'],
            'state' => $address['state'],
            'postcode' => !empty($address['postcode']) ? $address['postcode'] : '',
            'country' => $address['country'],
            'latitude' => $result['geometry']['location']['lat'],
            'longitude' => $result['geometry']['location']['lng'],
            'website' => !empty($result['website']) ? $result['website'] : '',
            'google_url' => !empty($result['url']) ? $result['url'] : '',
            'logo_url' => $logo_url,
            'show_address' => 1,
            'show_phone' => 1,
            'show_email' => 1,
            'show_claim' => 0,
            'contact_person' => null,
            'tollfree' => null,
            'fax' => null,
            'email' => null,
            'expires_on' => null,
            'categories_set' => null,
            'city_id' => 0,
            'created_on' => null,
			'hits' => mt_rand(1000, 9999),
            // 'images' => !empty($result['photos']) ? $this->get_photos($result['photos']) : '',
            // 'reviews' => !empty($result['reviews']) ? $this->get_reviews($result['reviews']) : '',
            // 'categories' =>  !empty($result['types']) ? $this->get_categories($result['types']) : '',
            // 'business_hours' => !empty($result['opening_hours']['periods']) ? $this->business_hours($result['opening_hours']['periods']) : '',
            'open_now' => !empty($result['opening_hours']['open_now']) ? $result['opening_hours']['open_now'] : ''
        );
        $random_package = $this->CI->db->get('packages')->row();
        $package = array(
            'price' => 0,
            'claim_price' => 0,
            'address' => 1,
            'email' => 1,
            'website' => 1,
            'map' => 1, 
            'qrcode' => 1, 
            'logo' => 1, 
            'phone' => 1, 
            'fax' => 1, 
            'social' => 1, 
            'person' => 1, 
            'trusted_seal' => '', 
            'border_color' => '',
            'display_fields' => $random_package->display_fields
        );
        $package = (object) $package;
        $record = (object) $view;
        
        
        // Get Categories
        $categories = !empty($result['types']) ? $this->get_categories($result['types']) : '';
        
        // prepare breadcrumb
        $breadcrumb = array();
        $j = 1;
        $br = array(1 => 'parent', 2 => 'sub', 3 => 'subsub');
        if(!empty($categories)) {
            foreach($categories as $category) {
                $breadcrumb[] = (object) array(
					'id' => $j,
					'name' => ucwords($category),
					'slug' => $this->CI->slug->create_slug($category),
					'redirect_url' => gc_url_format('search') . '?q=' . urlencode($category) . '&l=' . urlencode($record->city)
				);
                $j++;
            }
        }
        
        /**
         * Meta Information
         */
        $meta_categories = !empty($categories) ? implode(", ", $categories) : '';
        if(settings_item('seo.listing_meta_title') != '') {
            $meta_title = array('site_title' => settings_item('site.title'), 'listing_title' => $record->title, 'listing_city' => $record->city, 'listing_state' => $record->state, 'listing_country' => ucwords(strtolower($record->country)), 'listing_categories' => $meta_categories);
            $page_title = $this->CI->parser->parse_string(settings_item('seo.listing_meta_title'), $meta_title, TRUE);
        } else {
            $page_title = $record->title;
        }
        
        if(settings_item('seo.listing_meta_description') != '') {
            $meta_description = array('site_title' => settings_item('site.title'), 'listing_title' => $record->title, 'listing_city' => $record->city, 'listing_state' => $record->state, 'listing_country' => ucwords(strtolower($record->country)), 'listing_categories' => $meta_categories);
            $page_description = $this->CI->parser->parse_string(settings_item('seo.listing_meta_description'), $meta_description, TRUE);
        } else {
            $last_space = strrpos( substr( $record->description, 0, 200 ), ' ' ); // find the last space within 35 characters
            $page_description = strip_tags(substr( $record->description, 0, $last_space ) . '...');
        }
        
        if(settings_item('seo.listing_meta_keywords') != '') {
            $meta_keywords = array('site_title' => settings_item('site.title'), 'listing_title' => $record->title, 'listing_city' => $record->city, 'listing_state' => $record->state, 'listing_country' => ucwords(strtolower($record->country)), 'listing_categories' => $meta_categories);
            $page_keywords = $this->CI->parser->parse_string(settings_item('seo.listing_meta_keywords'), $meta_keywords, TRUE);
        } else {
            $page_keywords = $record->title;
        }
        
        // Load Google Map
        $this->CI->load->library('googlemaps');
        $config['center'] = $record->latitude . ', ' . $record->longitude;
        $config['zoom'] = 12;
        $this->CI->googlemaps->initialize($config);
        $marker = array();
        $marker['position'] =  $record->latitude . ', ' . $record->longitude;
        $this->CI->googlemaps->add_marker($marker);
        $map = $this->CI->googlemaps->create_map();
        
        // Get Comments
        $comments_count = 0;
        $comments_html = '';
        if(!empty($result['reviews'])) {
            $comments = $this->get_reviews($result['reviews']);
            $comments_count = $comments['count'];
            $comments_html = $comments['html'];
        }        
        
        $return = array(
            'page_title' => $page_title,
            'page_description' => $page_description,
            'page_keywords' => $page_keywords,
            'listing_id' => 0, // Leads Form
            'listing'=> $record,
			'record'=> (object) $record,
            'ratings' => (object) $ratings,
            'map' => $map,
            'branch_offices' => array(),
            'package' => (object) $package,
            'breadcrumb' => (object) $breadcrumb,
            //'categories' => $categories, // No Other Categories
            'images' => $images,
            'gallery' => $images,
            'business_hours' => !empty($result['opening_hours']['periods']) ? $this->business_hours($result['opening_hours']['periods']) : '',
            'searchCategory' => 0,
            'searchLocation' => $record->city,
            'searchKeyword' => $record->title,
            'searchLocality' => 0,
            'price'	=> $package->price,
            'hide_claim' => true,
            'module' => 'listings',
            'comments_html' => $comments_html,
            'comments_count' => $comments_count,
            'google_business' => true, // Google Listing
            'package_display_fields' => (object) array('website' => 1, 'address' => 1, 'map' => 1, 'phone_number' => 1),
            'module_display_fields' => (object) array(array('name' => 'address', 'icon' => 'fas fa-location-dot', 'label' => lang('label_address')), array('name' => 'website', 'icon' => 'fas fa-link', 'label' => lang('label_website'))),
        );
        return $return;
    }
    
    /**
     * Compare last 10 digits of both numbers
     * @param string $phone
     * @param string $mobile
     * @return boolean
     */
    private function compare_numbers($phone, $mobile) {
        $phone_number = substr($phone, -10);
        $mobile_number = substr($mobile, -10);
        if($phone_number == $mobile_number) {
            return true;
        } else {
            return false;
        }
    }
    
    /**
     * Get address components and update them
     * @param array $address_components
     * @return multitype:unknown Ambigous <string, unknown>
     */
    private function get_address($address_components) {
        $address = array();
        $formatted_address = '';
        foreach($address_components as $address_component) {
            switch($address_component['types'][0]) {
                case 'administrative_area_level_2':
                    $address['city'] = $address_component['long_name'];
                    break;
                case 'administrative_area_level_1':
                    $address['state'] = $address_component['long_name'];
                    $address['state_abbrev'] = $address_component['short_name'];
                    break;
                case 'country':
                    $address['country'] = $address_component['long_name'];
                    $address['country_iso'] = $address_component['short_name'];
                    break;
                case 'postal_code':
                    $address['postcode'] = $address_component['long_name'];
                    break;
                default:
                    $formatted_address .= $address_component['long_name'];
            }
        }
        $address['address'] = $formatted_address;
        return $address;
    }
    
    /**
     * Remove the city name from the vincity address
     * @param string $address
     * @param string $city
     * @return string
     */
    private function return_address($address) {
        $address_array = explode(", ", $address ?? ''); // Convert to Array
        array_pop($address_array); // Remove Country
        array_pop($address_array); // Remove State and Postcode
        array_pop($address_array); // Remove city information
        $new_address = implode(", ", $address_array); // Convert to String
        return $new_address;
    }
    
    /**
     * Get all reviews posted by different users
     * Comments Table Changes (Field Added): rating, language, author_url, profile_image - To store user profile image
     * @param array $reviews
     * @return multitype:multitype:string unknown
     */
    private function get_reviews($reviews, $listing_id = 0) {
        $comments = '';
        $id = 1; // Comments Counter
        foreach($reviews as $review) {
            $data = array(
                'id' => $id,
                'parent_id' => 0,
                'module_id' => gc_module_id('listings'),
                'row_id' => $listing_id,
                'username' => $review['author_name'],
                'title' => '',
                'comment' => $review['text'],
                'url' => !empty($review['author_url']) ? $review['author_url'] : '',
                'rating' => $review['rating'],
                'ip' => $this->CI->input->ip_address(),
                'language' => isset($review['language']) ? $review['language'] : '',
                'profile_image' => (isset($review['profile_photo_url']) ? $review['profile_photo_url'] : ''),
                'status' => 1,
                'created_on' => date('Y-m-d H:i:s', $review['time'])
            );
            $comments .= $this->print_comment_html($data, false);
            $id++;
        }
        return array('count' => $id, 'html' => $comments);
    }
    
    /**
     * Business Hours conversion as per GoClixy format
     * @param array $opening_hours
     * @return multitype:multitype:NULL
     */
    private function business_hours($opening_hours, $listing_id = 0) {
        $business_hours = array();
        foreach($opening_hours as $opening_hour) {
            $business_hours[] = (object) array(
                'listing_id' => $listing_id,
                'day_of_week' => $this->day_of_week($opening_hour['open']['day']),
                'open_time' => !empty($opening_hour['open']['time']) ? $this->time_in_12_hour_format($opening_hour['open']['time']) : '',
                'close_time' => !empty($opening_hour['close']['time'])? $this->time_in_12_hour_format($opening_hour['close']['time']) : '',
            );
        }
        return $business_hours;
    }
    
    /**
     * Google return time in a format of integer e.g. 0600 or 2000
     * This function split time to get hours and minutes
     * Based on the returned hours and minutes, the time is returned in 12 hours format
     * @param int $time
     * @return string
     */
    private function time_in_12_hour_format($time) {
        $hour = substr($time, 0, 2);
        $minutes = substr($time, 2, 4);
        $time_in_12_hour_format  = date("g:i A", strtotime($hour .':' . $minutes));
        return $time_in_12_hour_format;
    }
    
    /**
     * Google Day of Week starts from 0, however, GoClixy Week starts from 1
     * The below function is used to do this conversion
     * @param int $day
     * @return number
     */
    private function day_of_week($day) {
        $day_of_week = 1;
        switch($day) {
            case 0:
                $day_of_week = 1;
                break;
            case 1:
                $day_of_week = 2;
                break;
            case 2:
                $day_of_week = 3;
                break;
            case 3:
                $day_of_week = 4;
                break;
            case 4:
                $day_of_week = 5;
                break;
            case 5:
                $day_of_week = 6;
                break;
            case 6:
                $day_of_week = 7;
                break;
        }
        return $day_of_week;
    }   
    
    /**
     * Google Search URL
     * @param string $keyword
     * @param string $location
     * @param string $token - used to handle pagination
     */
    public function search_url($keyword, $location, $token = 0) {
        if($token == 0) {
            return site_url(gc_uri_string('search') . '?q=' . urlencode($keyword) .'&l=' . urlencode($location));
        } else { // the user clicked the next button
            return site_url(gc_uri_string('search') . '?q=' . urlencode($keyword) .'&l=' . urlencode($location) . (!empty($token) ? '&per_page=' . ($token + 1) : ''));
        }
    }
    
    /**
     * Business Detail Page URL
     * @param string $keyword
     * @param string $location
     */
    private function detail_url($slug) {
        // return site_url('view?place_id=' . $place_id);
        return site_url('view/' . $slug);
    }
    
    /**
     * Listing Detail Page - Comments Tree
     * @param array $comment
     * @param boolean $isChildren
     */
    private function print_comment_html($comment, $isChildren) {
        $comment_html = '';
        if($isChildren) {
            $comment_html .= '<ul class="children list-unstyled">';
        } else {
            $comment_html .= '<ol class="commentlist list-unstyled">';
        }
        $comment_html .= '<li>';
        $comment_html .= '<div class="comment comment-wrap">';
        $comment_html .= '<div class="comment-avatar">';
        if(empty($comment['profile_image']) && empty($comment['user_image'])) {
            $comment_html .= '<img alt=""	src="http://1.gravatar.com/avatar/a7a35e2f9bf8b2a24b79c9f6ccbf78be?s=65&#038;d=mm&#038;r=g" srcset="http://1.gravatar.com/avatar/a7a35e2f9bf8b2a24b79c9f6ccbf78be?s=130&amp;d=mm&amp;r=g 2x" class="avatar photo" height="65" width="65" />';
        } else if(!empty($comment['user_image'])) {
            $comment_html .= '<img alt=""	src="' .base_url() . "assets/images/users/" . $comment['user_image']. '" class="avatar photo" height="65" width="65" />';
        } else {
            $comment_html .= '<img alt=""	src="' . $comment['profile_image'] . '" class="avatar photo" height="65" width="65" />';
        }
        $comment_html .= '</div>';
        $comment_html .= '<div class="comment-content"><div class="author-comment">';
        $comment_html .= '<cite class="fn">';
        if(empty($comment['url'])) {
            $comment_html .= ' ' . $comment['username'];
        } else {
            $comment_html .= ' <a href="'. $comment['url'] .'" target="_blank" title="'. $comment['title'] . '-' . settings_item('site.title') .'" rel="nofollow">' . $comment['username'] . '</a> ';
        }
        $comment_html .= '</cite>';
        $comment_html .= '<input id="comment_rating" type="number" value="' . round($comment['rating'], 2) .'" class="rating" min=0 max=5 step=0.5 data-disabled="true" data-show-clear="false" data-size="xs"' . (settings_item('site.direction') == 1 ? ' data-rtl="true"': '') . '/>';
        $comment_html .= '<div class="comment-meta commentmetadata">';
        $comment_html .= convert_date_time($comment['created_on']);
        $comment_html .= '</div><div class="clear"></div></div>';
        $comment_html .= '<div class="comment-title">' . $comment['title'] . '</div>';
        $comment_html .= '<p>' . $comment['comment'] . '</p>';
        $comment_html .= '</div>';
        $comment_html .= '<div class="reply">';
        $comment_html .= '<a data-toggle="modal" id="' . $comment['id'] . '" role="button" href="#review" class="comment-reply">' . lang('label_reply') . '</a>';
        //$comment_html .= '<a class="comment-reply" id="' . $comment['id'] . '" role="button" aria-expanded="false">'. lang('label_reply') .'</a>';
        $comment_html .= '</div></div>';
        if(isset($comment['children'])) {
            $this->print_comment($comment['children'], true);
        }
        $comment_html .= '</li>';
        if($isChildren) {
            $comment_html .= '</ul>';
        } else {
            $comment_html .= '</ol>';
        }
        return $comment_html;
    }
}