<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Migration_Install_listings extends Migration
{
	/**
	 * The name of the database table
	 *
	 * @var String
	 */
	private $table_name = 'listings';
	private $business_hours_table = 'business_hours';
	private $listing_search_table = 'listing_search';
	private $listing_views_table = 'listing_views';
	private $listing_claims_table = 'listing_claims';
	private $table_google_listings = 'google_listings';
	
	// Common Tables
	private $table_categories_relation = 'categories_relation';
	private $table_meta = 'metas';
	private $table_tag_relations = 'tag_relations';
	private $table_module_relations = 'module_relations';

	/**
	 * The Listing table's fields
	 *
	 * @var Array
	 */
	private $fields = array(
		'id' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'auto_increment' => TRUE,
		),
		'user_id' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'null' => FALSE,
			'default'=> 1,
		),
		'package_id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => FALSE,
			'default'=> 1,
		),
		'title' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => FALSE,
		),
		'slug' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => FALSE,
		),
		'category_id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => FALSE,
			'default'=> 0,
		),
		'country_iso' => array(
			'type' => 'CHAR',
			'constraint' => 2,
			'null' => FALSE,
			'default' => 'US'
		),
		'state_id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => FALSE,
			'default' => 0
		),
		'city_id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => FALSE,
			'default' => 0
		),
		'locality_id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => TRUE,
		),
		'postcode' => array(
			'type' => 'VARCHAR',
			'constraint' => 10,
			'null' => TRUE,
		),
		'address' => array(
			'type' => 'VARCHAR',
			'constraint' => 250,
			'null' => TRUE,
		),
		'location' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE,
		),
		'latitude' => array(
			'type' => 'VARCHAR',
			'constraint' => 30,
			'null' => TRUE,
		),
		'longitude' => array(
			'type' => 'VARCHAR',
			'constraint' => 30,
			'null' => TRUE,
		),
		'contact_person' => array(
			'type' => 'VARCHAR',
			'constraint' => 50,
			'null' => TRUE,
		),
		'phone_number' => array(
			'type' => 'VARCHAR',
			'constraint' => 20,
			'null' => TRUE,
		),
		'mobile_number' => array(
			'type' => 'VARCHAR',
			'constraint' => 30,
			'null' => TRUE,
		),
		'tollfree' => array(
			'type' => 'VARCHAR',
			'constraint' => 30,
			'null' => TRUE,
		),
		'fax' => array(
			'type' => 'VARCHAR',
			'constraint' => 30,
			'null' => TRUE,
		),
		'email' => array(
			'type' => 'VARCHAR',
			'constraint' => 100,
			'null' => TRUE,
		),
		'website' => array(
			'type' => 'VARCHAR',
			'constraint' => 200,
			'null' => TRUE,
		),
		'linkedin_url' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE,
		),
		'facebook_url' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE,
		),
		'twitter_url' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE,
		),
		'googleplus_url' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE,
		),
		'youtube_url' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE,
		),
		'pinterest_url' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE,
		),
		'instagram_url' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE,
		),
		'description' => array(
			'type' => 'MEDIUMTEXT',
			'null' => TRUE,
		),
		'keywords' => array(
			'type' => 'MEDIUMTEXT',
			'null' => TRUE,
		),
		'categories' => array(
			'type' => 'MEDIUMTEXT',
			'null' => TRUE,
		),
		'meta_title' => array(
			'type' => 'VARCHAR',
			'constraint' => 200,
			'null' => TRUE,
		),
		'meta_description' => array(
			'type' => 'TEXT',
			'null' => TRUE,
		),
		'meta_keywords' => array(
			'type' => 'TEXT',
			'null' => TRUE,
		),
		'logo_url' => array(
			'type' => 'VARCHAR',
			'constraint' => 250,
			'null' => TRUE,
		),
		'qrfile' => array(
			'type' => 'VARCHAR',
			'constraint' => 150,
			'null' => TRUE,
		),
		'priority' => array(
			'type' => 'TINYINT',
			'constraint' => 2,
			'null'=> FALSE,
			'default' => '0',
		),
		'show_address' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null'=> FALSE,
			'default' => '1',
		),
		'show_phone' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null'=> FALSE,
			'default' => '1',
		),
		'show_email' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null'=> FALSE,
			'default' => '1',
		),
		'featured' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null'=> FALSE,
			'default' => '0',
		),
		'claimed' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null'=> FALSE,
			'default' => '0',
		),		
		'expires_on' => array(
			'type' => 'DATE',
			'null'=> TRUE
		),
		'hits' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null'=> FALSE,
			'default' => '0',
		),
		'status' => array(
			'type' => 'TINYINT',
			'constraint' => 2,
			'null'=> FALSE,
			'default' => '0',
		),
		'deleted' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null'=> FALSE,
			'default' => '0',
		),
		'deleted_by' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'null'=> FALSE,
			'default' => '0'
		),	
		'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'created_by' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'null'=> FALSE,
			'default' => '0'
		),
		'created_ip' => array(
			'type' => 'VARCHAR',
			'constraint' => 50,
			'null'=> TRUE
		),
		'modified_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'modified_by' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'null'=> FALSE,
			'default' => '0'
		),
		'modified_ip' => array(
			'type' => 'VARCHAR',
			'constraint' => 50,
			'null'=> TRUE
		),
	    'logo' => array(
	        'type' => 'VARCHAR',
	        'constraint' => 255,
	        'null' => TRUE,
	    ),
	    'cover' => array(
	        'type' => 'VARCHAR',
	        'constraint' => 255,
	        'null' => TRUE,
	    ),
	    'locality' => array(
	        'type' => 'VARCHAR',
	        'constraint' => 40,
	        'null' => TRUE,
	    ),
	    'city' => array(
	        'type' => 'VARCHAR',
	        'constraint' => 40,
	        'null' => TRUE,
	    ),
	    'state' => array(
	        'type' => 'VARCHAR',
	        'constraint' => 40,
	        'null' => TRUE,
	    ),
	    'country' => array(
	        'type' => 'VARCHAR',
	        'constraint' => 80,
	        'null' => TRUE,
	    ),
	    'keywords' => array(
	        'type' => 'TEXT',
	        'null' => TRUE,
	    ),
	    'categories' => array(
	        'type' => 'TEXT',
	        'null' => TRUE,
	    ),
	    'ratings' => array(
	        'type' => 'FLOAT',
	        'null' => FALSE,
	        'default' => 0
	    ),
	    'rating_users' => array(
	        'type' => 'INT',
	        'constraint' => 11,
	        'null' => FALSE,
	        'default' => 0
	    )
	);

	/**
	 * The Business Hours table's fields
	 *
	 * @var Array
	 */
	private $business_hours_fields = array(
			'id' => array(
					'type' => 'INT',
					'constraint' => 11,
					'auto_increment' => TRUE,
			),
			'listing_id' => array(
					'type' => 'BIGINT',
					'constraint' => 20,
					'null' => FALSE,
					'default' => 0,
			),
			'day_of_week' => array(
					'type' => 'TINYINT',
					'constraint' => 1,
					'null' => FALSE,
			),
			'open_time' => array(
					'type' => 'VARCHAR',
					'constraint' => 20,
					'null' => FALSE,
			),
			'close_time' => array(
					'type' => 'VARCHAR',
					'constraint' => 20,
					'null' => FALSE,
			),
			'status' => array( //  added only to have a symmetry with all modules
					'type' => 'TINYINT',
					'constraint' => 1,
					'null' => FALSE,
					'default' => 1,
			),
			'deleted' => array( //  added only to have a symmetry with all modules
					'type' => 'TINYINT',
					'constraint' => 1,
					'null' => FALSE,
					'default' => 0,
			),
	);
	
	/**
	 * Categories Relation table's fields
	 *
	 * @var Array
	 */
	private $categories_relation_fields = array(
			'id' => array(
				'type' => 'BIGINT',
				'constraint' => 20,
				'auto_increment' => TRUE,
			),
			'module_id' => array(
				'type' => 'TINYINT',
				'constraint' => 1,
				'null' => FALSE,
				'default'=> 0,
			),
			'row_id' => array(
				'type' => 'BIGINT',
				'constraint' => 20,
				'null' => FALSE,
				'default'=> 0,
			),
			'parent_id' => array(
				'type' => 'INT',
				'constraint' => 11,
				'null' => FALSE,
			),
			'sub_id' => array(
				'type' => 'INT',
				'constraint' => 11,
				'null' => FALSE,
				'default' => 0,
			),
			'subsub_id' => array(
				'type' => 'INT',
				'constraint' => 11,
				'null' => FALSE,
				'default' => 0,
			),
			'active' => array(
				'type' => 'TINYINT',
				'constraint' => 1,
				'null' => FALSE,
				'default' => 0,
			),
	);
	
	/**
	 * Module Meta table's fields
	 *
	 * @var Array
	 */
	private $meta_fields = array(
			'meta_id' => array(
				'type' => 'BIGINT',
				'constraint' => 20,
				'auto_increment' => TRUE,
			),
			'module_id' => array(
				'type' => 'TINYINT',
				'constraint' => 1,
				'null' => FALSE,
				'default'=> 0,
			),
			'row_id' => array(
				'type' => 'BIGINT',
				'constraint' => 20,
				'null' => FALSE,
				'default'=> 0,
			),
			'category_id' => array(
				'type' => 'INT',
				'constraint' => 11,
				'null' => FALSE,
				'default'=> 0,
			),
			'meta_key' => array(
				'type' => 'VARCHAR',
				'constraint' => 255,
				'null' => FALSE,
			),
			'meta_value' => array(
				'type' => 'MEDIUMTEXT',
				'null' => TRUE,
			),
	);
	
	/**
	 * The Listing Search table's fields
	 *
	 * @var Array
	 */
	private $listing_search_fields = array(
			'id' => array(
					'type' => 'BIGINT',
					'constraint' => 20
			),
			'title' => array(
					'type' => 'VARCHAR',
					'constraint' => 255,
					'null' => TRUE,
			),
			'description' => array(
					'type' => 'MEDIUMTEXT',
					'null' => TRUE,
			),
			'country_iso' => array(
					'type' => 'CHAR',
					'constraint' => 2,
					'null' => FALSE,
			),
			'country' => array(
					'type' => 'VARCHAR',
					'constraint' => 80,
					'null' => FALSE,
			),
			'state_id' => array(
					'type' => 'INT',
					'constraint' => 11,
					'null' => FALSE,
			),
			'state' => array(
					'type' => 'CHAR',
					'constraint' => 40,
					'null' => FALSE,
			),
			'city_id' => array(
					'type' => 'INT',
					'constraint' => 11,
					'null' => FALSE,
			),
			'city' => array(
					'type' => 'CHAR',
					'constraint' => 40,
					'null' => FALSE,
			),
			'postcode' => array(
					'type' => 'VARCHAR',
					'constraint' => 10,
					'null' => TRUE,
			),
			'average_rating' => array(
					'type' => 'FLOAT',
					'null' => TRUE,
			),
			'tags' => array(
					'type' => 'MEDIUMTEXT',
					'null' => TRUE,
			),
			'parent_categories' => array(
					'type' => 'MEDIUMTEXT',
					'null' => TRUE,
			),
			'sub_categories' => array(
					'type' => 'MEDIUMTEXT',
					'null' => TRUE,
			),
			'subsub_categories' => array(
					'type' => 'MEDIUMTEXT',
					'null' => TRUE,
			),
			'videos_count' => array(
					'type' => 'SMALLINT',
					'constraint' => 5,
					'null' => FALSE,
					'default' => 0,
			),
			'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
			'featured' => array(
					'type' => 'TINYINT',
					'constraint' => 1,
					'null' => FALSE,
					'default' => 0,
			),
			'status' => array(
					'type' => 'TINYINT',
					'constraint' => 1,
					'null' => FALSE,
					'default' => 0,
			),
			'deleted' => array(
					'type' => 'TINYINT',
					'constraint' => 1,
					'null' => FALSE,
					'default' => 0,
			),
		);
	
	/**
	 * The Listing Views table's fields
	 *
	 * @var Array
	 */
	private $listing_views_fields = array(
			'id' => array(
					'type' => 'BIGINT',
					'constraint' => 20,
					'auto_increment' => TRUE,
			),
			'user_id' => array(
					'type' => 'BIGINT',
					'constraint' => 20,
					'null' => FALSE,
					'default'=> 1,
			),
			'listing_id' => array(
					'type' => 'BIGINT',
					'constraint' => 20,
					'null' => FALSE,
					'default'=> 1,
			),
			'page' => array(
					'type' => 'CHAR',
					'constraint' => 20,
					'null' => FALSE,
			),
			'keyword' => array(
					'type' => 'VARCHAR',
					'constraint' => 100,
					'null' => FALSE,
			),
			'times' => array(
					'type' => 'INT',
					'constraint' => 11,
					'null' => FALSE,
					'default' => 1
			),
			'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
			'modified_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
	);
	
	/**
	 * The Listing claim's fields
	 *
	 * @var Array
	 */
	private $listing_claims_fields = array(
			'id' => array(
					'type' => 'INT',
					'constraint' => 11,
					'auto_increment' => TRUE,
			),
			'order_id' => array(
					'type' => 'VARCHAR',
					'constraint' => 20,
					'null' => TRUE,
			),
			'listing_id' => array(
					'type' => 'BIGINT',
					'constraint' => 20,
					'null' => FALSE,
					'default'=> 1,
			),
			'listing_expiry' => array(
					'type' => 'DATE',
					'null' => TRUE
			),
			'listing_featured' => array(
					'type' => 'TINYINT',
					'constraint' => 1,
					'null' => FALSE,
					'default' => 0
			),
			'claimed_by' => array(
					'type' => 'BIGINT',
					'constraint' => 20,
					'null' => FALSE
			),
			'owned_by' => array(
					'type' => 'BIGINT',
					'constraint' => 20,
					'null' => FALSE
			),
			'reported_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
			'modified_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
			'ip' => array(
					'type' => 'VARCHAR',
					'constraint' => 40,
					'null' => FALSE
			),
			'status' => array(
					'type' => 'TINYINT',
					'constraint' => 2,
					'null' => FALSE,
					'default' => 0
			),
			'deleted' => array(
					'type' => 'TINYINT',
					'constraint' => 1,
					'null' => FALSE,
					'default' => 0
			)
	);
	
	/**
	 * The Module Tags table's fields
	 *
	 * @var Array
	 */
	private $tag_relations_fields = array(
			'module_id' => array(
				'type' => 'TINYINT',
				'constraint' => 1,
				'null' => FALSE,
			),
			'row_id' => array(
				'type' => 'BIGINT',
				'constraint' => 20,
				'null' => FALSE,
			),
			'tag_id' => array(
				'type' => 'BIGINT',
				'constraint' => 20,
				'null' => FALSE,
			)
	);
	
	/**
	 * The Module Relations table's fields
	 *
	 * @var Array
	 */
	private $module_relations_fields = array(
			'row_id' => array(
				'type' => 'BIGINT',
				'constraint' => 20,
				'null' => FALSE,
			),
			'module_id' => array(
				'type' => 'TINYINT',
				'constraint' => 1,
				'null' => FALSE,
			),
			'toggle_module_id' => array(
				'type' => 'TINYINT',
				'constraint' => 1,
				'null' => FALSE,
			),
	);
	
	/**
	 * The Google Listings table's fields
	 *
	 * @var Array
	 */
	private $google_listings_fields = array(
	    'id' => array(
	        'type' => 'INT',
	        'constraint' => 11,
	        'auto_increment' => TRUE
	    ),
	    'place_id' => array(
	        'type' => 'VARCHAR',
	        'constraint' => 150,
	        'null' => FALSE,
	    ),
	    'slug' => array(
	        'type' => 'VARCHAR',
	        'constraint' => 255,
	        'null' => FALSE,
	    ),
	    'created_on' => array(
	        'type' => 'DATETIME',
	        'null' => TRUE
	    ),
	    'created_ip' => array(
	        'type' => 'VARCHAR',
	        'constraint' => 50,
	        'null' => TRUE,
	    ),
	    'modified_on' => array(
	        'type' => 'DATETIME',
	        'null' => TRUE
	    ),
	    'modified_ip' => array(
	        'type' => 'VARCHAR',
	        'constraint' => 50,
	        'null' => TRUE,
	    )
	);
	
	/**
	 * Install this migration
	 *
	 * @return void
	 */
	public function up()
	{		
		// LISTINGS TABLE
		$this->dbforge->add_field($this->fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('user_id',false);
		$this->dbforge->add_key('package_id',false);
		$this->dbforge->add_key('category_id',false);
		$this->dbforge->add_key('country_iso',false);
		$this->dbforge->add_key('state_id',false);
		$this->dbforge->add_key('locality_id',false);
		$this->dbforge->add_key('created_on',false);
		$this->dbforge->add_key('hits',false);
		$this->dbforge->add_key('city_id',false);
		$this->dbforge->add_key('featured',false);
		$this->dbforge->add_key('postcode',false);
		$this->dbforge->create_table($this->table_name);
		$change_engine_sql = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " ENGINE = MyISAM";
		$this->db->query ($change_engine_sql);
		$index_sql = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " ADD INDEX `deleted_status` (`deleted`, `status`)";
		$this->db->query ($index_sql);
		$listing_fulltext_sql = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " ADD FULLTEXT INDEX `title_description` (`title`, `description`)";
		$this->db->query ($listing_fulltext_sql);
		// SET SLUG FIELD COLLATION TO utf8_bin
		$collation_sql = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " CHANGE `slug` `slug` VARCHAR( 255 ) CHARACTER SET utf8 COLLATE utf8_bin NOT NULL";
		$this->db->query ($collation_sql);
		
		
		// BUSINESS HOURS TABLE
		$this->dbforge->add_field($this->business_hours_fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('listing_id', false);
		$this->dbforge->create_table($this->business_hours_table);
				
		// LISTING SEARCH TABLE
		$this->dbforge->add_field($this->listing_search_fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('created_on', false);
		$this->dbforge->add_key('featured', false);
		$this->dbforge->add_key('state_id', false);
		$this->dbforge->add_key('city_id', false);
		$this->dbforge->add_key('country_iso', false);
		$this->dbforge->add_key('average_rating', false);
		$this->dbforge->add_key('videos_count', false);
		$this->dbforge->add_key('status', false);
		$this->dbforge->add_key('deleted', false);
		$this->dbforge->add_key('postcode', false);
		$this->dbforge->create_table($this->listing_search_table);
		$change_search_engine_sql = "ALTER TABLE " . $this->db->dbprefix . $this->listing_search_table . " ENGINE = MyISAM";
		$this->db->query ($change_search_engine_sql);
		$listing_fulltext_sql = "ALTER TABLE " . $this->db->dbprefix . $this->listing_search_table . " ADD FULLTEXT INDEX `listing` (`title`, `description`,`tags`,`parent_categories`,`sub_categories`,`subsub_categories`)";
		$this->db->query ($listing_fulltext_sql);
		$listing_fulltext_sql = "ALTER TABLE " . $this->db->dbprefix . $this->listing_search_table . " ADD FULLTEXT INDEX `location` (`state`, `city`)";
		$this->db->query ($listing_fulltext_sql);
		
		// LISTING VIEWS TABLE
		$this->dbforge->add_field($this->listing_views_fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('listing_id', false);
		$this->dbforge->add_key('user_id', false);
		$this->dbforge->add_key('modified_on', false);
		$this->dbforge->create_table($this->listing_views_table);
		$listing_views_index_sql = "ALTER TABLE " . $this->db->dbprefix . $this->listing_views_table . " ADD UNIQUE INDEX `listing_page_keyword` (`listing_id`, `page`, `keyword`)";
		$this->db->query ($listing_views_index_sql);
		
		// LISTING CLAIMS TABLE
		$this->dbforge->add_field($this->listing_claims_fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('listing_id', false);
		$this->dbforge->create_table($this->listing_claims_table);
		
		// CATEGORIES RELATION TABLE
		$this->dbforge->add_field($this->categories_relation_fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('module_id', false);
		$this->dbforge->add_key('row_id', false);
		$this->dbforge->add_key('parent_id', false);
		$this->dbforge->add_key('sub_id', false);
		$this->dbforge->add_key('subsub_id', false);
		$this->dbforge->create_table($this->table_categories_relation);
		$categories_engine_sql = "ALTER TABLE " . $this->db->dbprefix . $this->table_categories_relation . " ENGINE = MyISAM";
		$this->db->query ($categories_engine_sql);
		$categories_unique_row = "ALTER TABLE " . $this->db->dbprefix . $this->table_categories_relation . " ADD UNIQUE INDEX `unique_row` (`module_id`, `row_id`,`parent_id`,`sub_id`,`subsub_id`)";
		$this->db->query ($categories_unique_row);
		$module_index_sql = "ALTER TABLE " . $this->db->dbprefix . $this->table_categories_relation . " ADD INDEX `module` (`module_id`, `row_id`)";
		$this->db->query ($module_index_sql);
		
		// MODULE META TABLE
		$this->dbforge->add_field($this->meta_fields);
		$this->dbforge->add_key('meta_id', true);
		$this->dbforge->add_key('meta_key', false);
		$this->dbforge->create_table($this->table_meta);
		$meta_index_sql = "ALTER TABLE " . $this->db->dbprefix . $this->table_meta . " ADD INDEX `module` (`module_id`, `row_id`)";
		$this->db->query ($meta_index_sql);
		
		// MODULE TAGS TABLE
		$this->dbforge->add_field($this->tag_relations_fields);
		$this->dbforge->add_key('module_id', false);
		$this->dbforge->add_key('row_id', false);
		$this->dbforge->add_key('tag_id', false);
		$this->dbforge->create_table($this->table_tag_relations);
		$tag_relations_unique = "ALTER TABLE " . $this->db->dbprefix . $this->table_tag_relations . " ADD UNIQUE INDEX `unique_tag` (`row_id`, `module_id`,`tag_id`)";		
		$this->db->query($tag_relations_unique);
		
		// MODULE RELATIONS TABLE
		$this->dbforge->add_field($this->module_relations_fields);
		$this->dbforge->add_key('row_id', false);
		$this->dbforge->add_key('module_id', false);
		$this->dbforge->create_table($this->table_module_relations);
		$module_relations_unique = "ALTER TABLE " . $this->db->dbprefix . $this->table_module_relations . " ADD UNIQUE INDEX `unique_relation` (`row_id`, `module_id`,`toggle_module_id`)";		
		$this->db->query($module_relations_unique);
		
		// GOOGLE LISTINGS TABLE
		$this->dbforge->add_field($this->google_listings_fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('place_id', false);
		$this->dbforge->add_key('slug', false);
		$this->dbforge->create_table($this->table_google_listings);
	}

	//--------------------------------------------------------------------

	/**
	 * Uninstall this migration
	 *
	 * @return void
	 */
	public function down()
	{	
		if ($this->db->table_exists($this->table_name)) {
			$this->dbforge->drop_table($this->table_name);
		}
		if ($this->db->table_exists($this->business_hours_table)) {
			$this->dbforge->drop_table($this->business_hours_table);
		}
		if ($this->db->table_exists($this->listing_search_table)) {
			$this->dbforge->drop_table($this->listing_search_table);
		}
		if ($this->db->table_exists($this->listing_views_table)) {
			$this->dbforge->drop_table($this->listing_views_table);
		}
		if ($this->db->table_exists($this->listing_claims_table)) {
			$this->dbforge->drop_table($this->listing_claims_table);
		}		
		// module related tables - categories, meta, tags and relations
		if ($this->db->table_exists($this->table_categories_relation)) {
			$this->dbforge->drop_table($this->table_categories_relation);
		}
		if ($this->db->table_exists($this->table_meta)) {
			$this->dbforge->drop_table($this->table_meta);
		}
		if ($this->db->table_exists($this->table_tag_relations)) {
			$this->dbforge->drop_table($this->table_tag_relations);
		}
		if ($this->db->table_exists($this->table_module_relations)) {
			$this->dbforge->drop_table($this->table_module_relations);
		}
		if ($this->db->table_exists($this->table_google_listings)) {
		    $this->dbforge->drop_table($this->table_google_listings);
		}
	}

	//--------------------------------------------------------------------

}