<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Migration_Install_classifieds extends Migration
{
	/**
	 * The name of the database table
	 *
	 * @var String
	 */
	private $table_name = 'classifieds';

	/**
	 * The table's fields
	 *
	 * @var Array
	 */
	private $fields = array(
		'id' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'auto_increment' => TRUE,
		),
		'listing_id' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'null' => FALSE,
			'default' => 0
		),
		'user_id' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'null' => FALSE,
			'default' => 0
		),
		'package_id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => FALSE,
			'default' => 0
		),
		'title' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => FALSE,
		),
		'slug' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE,
		),
		'price' => array(
			'type' => 'VARCHAR',
			'constraint' => 100,
			'null' => TRUE,
		),
		'country_iso' => array(
			'type' => 'CHAR',
			'constraint' => 2,
			'null' => FALSE,
			'default'=> 'US',
		),
		'state_id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => FALSE,
		),
		'city_id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => FALSE,
		),
		'locality_id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => TRUE,
		),
		'postcode' => array(
			'type' => 'VARCHAR',
			'constraint' => 10,
			'null' => TRUE,
		),
		'address' => array(
			'type' => 'TEXT',
		    'null' => TRUE,
		),
		'latitude' => array(
			'type' => 'VARCHAR',
			'constraint' => 30,
			'null' => TRUE,
		),
		'longitude' => array(
			'type' => 'VARCHAR',
			'constraint' => 30,
			'null' => TRUE,
		),
		'external_url' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE,
		),
		'description' => array(
			'type' => 'TEXT',
			'null' => TRUE,
		),
		'hits' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => FALSE,
			'default'=> 0,
		),
		'featured' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default'=> 0,
		),
		'status' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default'=> 0,
		),
		'meta_title' => array(
			'type' => 'VARCHAR',
			'constraint' => 200,
			'null' => TRUE
		),
		'meta_description' => array(
			'type' => 'TEXT',
			'null' => TRUE,
		),
		'meta_keywords' => array(
			'type' => 'TEXT',
			'null' => TRUE,
		),
		'deleted' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default'=> 0,
		),
		'deleted_by' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'null'=> FALSE,
			'default' => '0'
		),
		'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',		
		'created_by' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'null'=> FALSE,
			'default' => '0'
		),
		'created_ip' => array(
			'type' => 'VARCHAR',
			'constraint' => 50,
			'null'=> TRUE
		),
		'modified_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'modified_by' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'null'=> FALSE,
			'default' => '0'
		),
		'modified_ip' => array(
			'type' => 'VARCHAR',
			'constraint' => 50,
			'null'=> TRUE
		),
		'cover' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE,
		),
		'locality' => array(
			'type' => 'VARCHAR',
			'constraint' => 40,
			'null' => TRUE,
		),
		'city' => array(
			'type' => 'VARCHAR',
			'constraint' => 40,
			'null' => TRUE,
		),
		'state' => array(
			'type' => 'VARCHAR',
			'constraint' => 40,
			'null' => TRUE,
		),
		'country' => array(
			'type' => 'VARCHAR',
			'constraint' => 80,
			'null' => TRUE,
		),
		'keywords' => array(
			'type' => 'TEXT',
			'null' => TRUE,
		),
		'categories' => array(
			'type' => 'TEXT',
			'null' => TRUE,
		),
	    'ratings' => array(
	        'type' => 'FLOAT',
	        'null' => FALSE,
	        'default' => 0
	    ),
	    'rating_users' => array(
	        'type' => 'INT',
	        'constraint' => 11,
	        'null' => FALSE,
	        'default' => 0
	    )
	);

	/**
	 * Install this migration
	 *
	 * @return void
	 */
	public function up()
	{
		$this->dbforge->add_field($this->fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('listing_id', false);
		$this->dbforge->add_key('user_id', false);
		$this->dbforge->add_key('package_id', false);
		$this->dbforge->create_table($this->table_name);
		// CHANGE ENGINE FROM INNODB TO MYISAM
		$change_engine_sql = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " ENGINE = MyISAM";
		$this->db->query ($change_engine_sql);
		// ADD LOCATION INDEX
		$index_location = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " ADD FULLTEXT INDEX `search_location` (`postcode`, `locality`, `city`, `state`, `country`)";
		$this->db->query ($index_location);
		// ADD FULLTEXT INDEX
		$fulltext_sql = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " ADD FULLTEXT INDEX `search_query` (`title`, `description`, `keywords`, `categories`)";
		$this->db->query ($fulltext_sql);
		// SET SLUG FIELD COLLATION TO utf8_bin
		$collation_sql = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " CHANGE `slug` `slug` VARCHAR( 255 ) CHARACTER SET utf8 COLLATE utf8_bin NULL DEFAULT NULL";
		$this->db->query ($collation_sql);
		// ADD DELETED_STATUS INDEX
		$idx_deleted_status = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " ADD INDEX `deleted_status` ( `deleted`, `status`)";
		$this->db->query ($idx_deleted_status);
	}

	//--------------------------------------------------------------------

	/**
	 * Uninstall this migration
	 *
	 * @return void
	 */
	public function down() {
		$this->dbforge->drop_table($this->table_name);
	}

	//--------------------------------------------------------------------

}