<?php

if (! defined('BASEPATH'))
    exit('No direct script access allowed');

class Migration_Install_products extends Migration
{

    /**
     * The name of the database table
     *
     * @var String
     */
    private $table_name = 'products';

    /**
     * The table's fields
     *
     * @var Array
     */
    private $fields = array(
        'id' => array(
            'type' => 'BIGINT',
            'constraint' => 20,
            'auto_increment' => TRUE
        ),
        'listing_id' => array(
            'type' => 'BIGINT',
            'constraint' => 20,
            'null' => FALSE,
            'default' => 0
        ),
        'user_id' => array(
            'type' => 'BIGINT',
            'constraint' => 20,
            'null' => FALSE,
            'default' => 0
        ),
        'package_id' => array(
            'type' => 'INT',
            'constraint' => 11,
            'null' => FALSE,
            'default' => 0
        ),
        'title' => array(
            'type' => 'VARCHAR',
            'constraint' => 255,
            'null' => FALSE
        ),
        'slug' => array(
            'type' => 'VARCHAR',
            'constraint' => 255,
            'null' => TRUE
        ),
        'price' => array(
            'type' => 'VARCHAR',
            'constraint' => 100,
            'null' => TRUE
        ),
        'offer_price' => array(
            'type' => 'FLOAT',
            'null' => FALSE,
            'default' => 0
        ),
        'stock' => array(
            'type' => 'INT',
            'constraint' => 11,
            'null' => FALSE,
            'default' => 0
        ),
        'stock_threshold' => array(
            'type' => 'INT',
            'constraint' => 11,
            'null' => FALSE,
            'default' => 0
        ),
        'tax_percent' => array(
            'type' => 'FLOAT',
            'null' => FALSE,
            'default' => 0
        ),
        'tax_inclusive' => array(
            'type' => 'TINYINT',
            'constraint' => 1,
            'null' => FALSE,
            'default' => 0
        ),
        'variants' => array(
            'type' => 'LONGTEXT',
            'null' => TRUE
        ),
        'commission' => array(
            'type' => 'FLOAT',
            'null' => FALSE,
            'default' => 0
        ),
        'refund_duration' => array(
            'type' => 'SMALLINT',
            'constraint' => 5,
            'null' => FALSE,
            'default' => 0
        ),
        'refund_unit' => array(
            'type' => 'TINYINT',
            'constraint' => 1,
            'null' => FALSE,
            'default' => 1
        ),
        'description' => array(
            'type' => 'TEXT',
            'null' => FALSE
        ),
        'external_url' => array(
            'type' => 'VARCHAR',
            'constraint' => 255,
            'null' => TRUE
        ),
        'hits' => array(
            'type' => 'INT',
            'constraint' => 11,
            'null' => FALSE,
            'default' => 0
        ),
        'featured' => array(
            'type' => 'TINYINT',
            'constraint' => 1,
            'null' => FALSE,
            'default' => 0
        ),
        'status' => array(
            'type' => 'TINYINT',
            'constraint' => 1,
            'null' => FALSE,
            'default' => 0
        ),
        'meta_title' => array(
            'type' => 'VARCHAR',
            'constraint' => 200,
            'null' => TRUE
        ),
        'meta_description' => array(
            'type' => 'TEXT',
            'null' => TRUE
        ),
        'meta_keywords' => array(
            'type' => 'TEXT',
            'null' => TRUE
        ),
        'deleted' => array(
            'type' => 'TINYINT',
            'constraint' => 1,
            'null' => FALSE,
            'default' => 0
        ),
        'deleted_by' => array(
            'type' => 'BIGINT',
            'constraint' => 20,
            'null' => FALSE,
            'default' => '0'
        ),
        'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
        'created_by' => array(
            'type' => 'BIGINT',
            'constraint' => 20,
            'null' => FALSE,
            'default' => '0'
        ),
        'created_ip' => array(
            'type' => 'VARCHAR',
            'constraint' => 50,
            'null' => TRUE
        ),
        'modified_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
        'modified_by' => array(
            'type' => 'BIGINT',
            'constraint' => 20,
            'null' => FALSE,
            'default' => '0'
        ),
        'modified_ip' => array(
            'type' => 'VARCHAR',
            'constraint' => 50,
            'null' => TRUE
        ),
        'cover' => array(
            'type' => 'VARCHAR',
            'constraint' => 255,
            'null' => TRUE
        ),
        'keywords' => array(
            'type' => 'TEXT',
            'null' => TRUE
        ),
        'categories' => array(
            'type' => 'TEXT',
            'null' => TRUE
        ),
        'ratings' => array(
            'type' => 'FLOAT',
            'null' => FALSE,
            'default' => 0
        ),
        'rating_users' => array(
            'type' => 'INT',
            'constraint' => 11,
            'null' => FALSE,
            'default' => 0
        )
    );

    /**
     * Install this migration
     *
     * @return void
     */
    public function up()
    {
        $this->dbforge->add_field($this->fields);
        $this->dbforge->add_key('id', true);
        $this->dbforge->add_key('listing_id', false);
        $this->dbforge->add_key('user_id', false);
        $this->dbforge->add_key('package_id', false);
        $this->dbforge->create_table($this->table_name);
        // CHANGE ENGINE FROM INNODB TO MYISAM
        $change_engine_sql = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " ENGINE = MyISAM";
        $this->db->query($change_engine_sql);
        // ADD FULLTEXT INDEXT
        $fulltext_sql = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " ADD FULLTEXT INDEX `search_query` (`title`, `description`, `keywords`, `categories`)";
        $this->db->query($fulltext_sql);
        // SET SLUG FIELD COLLATION TO utf8_bin
        $collation_sql = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " CHANGE `slug` `slug` VARCHAR( 255 ) CHARACTER SET utf8 COLLATE utf8_bin NULL DEFAULT NULL";
        $this->db->query($collation_sql);
        // ADD DELETED_STATUS INDEX
        $idx_deleted_status = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " ADD INDEX `deleted_status` ( `deleted`, `status`)";
        $this->db->query($idx_deleted_status);
    }

    // --------------------------------------------------------------------
    
    /**
     * Uninstall this migration
     *
     * @return void
     */
    public function down()
    {
        $this->dbforge->drop_table($this->table_name);
    }
    
    // --------------------------------------------------------------------
}