<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class Ajax_model extends BF_Model {
	protected $z_cache;
	public function __construct() {
		if(settings_item('site.enable_cache') == 1) {
			$this->load->library('gzf_cache', array('lifetime' => 900));
			$this->load->library('slug');
			$this->z_cache = $this->gzf_cache->get_instance();
		}
	}
	
		/**
	 * Send Business Query
	 * @return boolean[]|string[]
	 */
	public function send_query($user_id, $mobile = false) {
	    $error_message = lang('failure_message');
	    $module = escape($this->input->post('module'));
	    $form_fields = $this->gc->load_query_fields($module, 'frontend', true, $mobile);
	    if(!$form_fields) {
	        if($mobile) {
	            return array('success' => false, 'message' => $error_message);
	        } else {
	            echo json_encode(array('result' => false, 'message' => $error_message, 'type' => 'danger'));
	            return false;
	        }
	    }
	    $module_id = gc_module_id($module);
	    $row_id = (int) $this->input->post('row_id');	    
	    // Member Area - Business Queries Count display to the user
	    // A new field owner_id is introduced, this will store the module row user_id
	    // With the help of this field, Multi JOIN of all modules table will be avoided
	    // We cannot join all modules table for a simple count operation as this will slow down the website
	    $owner_id = 0;
	    if($module_id == 0 || empty($row_id)) {
	        return false;
	    }
	    $summary = array();
        $module_info = $this->db->where('id', $row_id)->get($module)->row();
        if($module_info && !empty($module_info->user_id)) {
           $owner_id = $module_info->user_id;
        }
        // to avoid unnecessary table join
        $summary = array(
           'id' => $module_info->id,
           'title' => $module_info->title,
           'slug' => $module_info->slug,
           'city' => !empty($module_info->city) ? $module_info->city : '',
        );
	    $user_id = empty($user_id) ? (int) $this->auth->user_id() : $user_id;
	    $role = 'frontend';
	    $extra_fields = array(
	        'module_id' => $module_id,
	        'row_id' => $row_id,
			'listing_id' => $module == 'listings' ? $module_info->id : $module_info->listing_id, // to update business queries on listing claim status update
	        'user_id' => $user_id,
	        'owner_id' => $owner_id,
	        'summary' => json_encode($summary),
	        'name' => $this->input->post('name'),
	        'email' => $this->input->post('email'),
	        'phone_number' => $this->input->post('phone_number'),
	        'status_id' => settings_item('lst.business_query_status') == 1 ? 2 : 1 // auto approve business queries
	    );
	    if ($insert_id = $this->fields_model->save_module($form_fields, 'business_queries', $user_id, $role, 0, 'insert', $extra_fields)) {
	        // if no module template is defined, use listings email template to notify user
	        $record = $this->db->where('module_table', $module)->get('query_email_templates')->row();
	        if(!$record) {
	            $module = 'listings';
	        }
	        $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'query_email_templates', 'module_table' => $module, 'action_type' => 'add', 'id' => $insert_id));
	        
	        $result = 1;
	        $type = 'success';
	        $message = lang('success_message');
	        if($mobile) {
	            $message = ['success' => ($result == 1 ? true : false), 'message' => $message];
	            return $message;
	        } else {
	            echo json_encode(array('result' => $result, 'message' => $message, 'type' => $type));
	        }
	    } else {
	        $error_message = sprintf(lang('msg_create_failure'), lang('singular_business_queries'));
	        $validation_errors = validation_errors();
	        if ($mobile) {
	            if ($validation_errors) {
	                $error_message = $validation_errors;
	            }
	            return array(
	                'success' => false,
	                'message' => $error_message
	            );
	        } else {
	            $alert_type = 'danger';
	            if ($validation_errors) {
	                $message = ['success' => false, 'message' => $validation_errors, 'type' => $alert_type];
	            } else {
	                $message = ['success' => false, 'message' => lang('msg_failure'), 'type' => $alert_type];
	            }
	            echo json_encode($message);
	            return false;
	        }
	    }
	}
	
	/**
	 * Business Owner Reply to the User
	 * @return string
	 */
	public function reply_query($user_id, $mobile = false) {
	    $this->lang->load('members/members');
	    $this->form_validation->set_rules('query_id', 'lang:members_business_queries', 'required|trim');
	    $this->form_validation->set_rules('reply_subject', 'lang:label_subject', 'required|trim');
	    $this->form_validation->set_rules('reply_message', 'lang:label_message', 'required|trim');
	    if(!$mobile && $this->auth->is_logged_in() === false):
	    $this->form_validation->set_rules('captcha_code', 'lang:captcha_code', 'trim|required|valid_captcha');
	    endif;
	    if ($this->form_validation->run() === FALSE) {
	        if($mobile) {
	            return array('success' => false, 'message' => validation_errors());
	        } else {
	            echo json_encode(array('result' => 0, 'message' => validation_errors(), 'type' => 'error'));
	        }
	    } else {
	        $user_id = empty($user_id) ? $this->auth->user_id() : $user_id;
	        $query_id = escape($this->input->post('query_id'));
	        // Retrieve user email address from database
	        $save_query = escape($this->input->post('save_query'));
	        $this->load->model('business_queries/business_queries_model');
	        $query = $this->business_queries_model->select('user_id, email, module_id, row_id')->find($query_id);
	        if($query) {
	            // Retrieve user email address from database
	            $email = $this->business_queries_model->get_owner_email($query);
	            if($email) { // notify user
	                $subject = escape($this->input->post('reply_subject'));
	                // email is sent to the business owner from user via posted_queries page
	                $message = '';
	                if($save_query && $save_query != 0) {
	                    $email_to = $email;
	                    $message .= sprintf(lang('reply_email'), $query->email) . "<br /><br />";
	                } else { // business owner replied to the user
	                    $email_to = $query->email;
	                }
	                $message .= escape($this->input->post('reply_message')) .'<hr><br />' .settings_item('site.title') .'<br />' .site_url();
	                // Send Email
	                $this->load->library('emailer/emailer');
	                $data = array(
	                    // 'from'		=> $query->email_from,
	                    'to'		=> $email_to,
	                    'subject'	=> $subject,
	                    'message'	=> $message
	                );
	                $this->emailer->send($data, FALSE);
	            }
	            if(!empty($query->user_id)) {
	                // save query reply for future reference
	                $reply = array(
	                    'query_id' => $query_id,
	                    'user_id' => $query->user_id,
	                    'replied_by' => $user_id,
	                    'subject' => $subject,
	                    'message' => escape($this->input->post('reply_message')),
	                    'created_on' => date('Y-m-d H:i:s'),
	                    'created_ip' => $this->input->ip_address()
	                );
	                $this->db->insert('query_replies', $reply);
	            }
	            // Update only if the business owner replies
	            if(!$save_query) {
	                $this->db->update('business_queries', array('replied' => 1), array('id' => $query_id));
	            } else {
	                $this->db->update('business_queries', array('replied' => 0), array('id' => $query_id));
	            }
	            if($mobile) {
	                return array('success' => true, 'message' => lang('success_message'));
	            } else {
	                echo json_encode(array('result' => 1, 'message' => lang('success_message'), 'type' => 'success'));
	            }
	        } else {
	            if($mobile) {
	                return array('success' => false, 'message' => lang('error_message'));
	            } else {
	                echo json_encode(array('result' => 0, 'message' => lang('error_message'), 'type' => 'error'));
	            }
	        }
	    }
	}
	
	/**
	 * Allow users to send SMS to themselves or friend
	 * @return array
	 */
	public function send_sms($mobile = false) {
	    $this->form_validation->set_rules('sms_mobile', 'lang:label_phone', 'required|phone_check|trim');
	    $this->form_validation->set_rules('sms_captcha_code', 'lang:captcha_code', 'trim|required|valid_captcha');
	    
	    if ($this->form_validation->run() === FALSE) {
	        $validation_errors = validation_errors();
	        if(!$mobile) {
	            echo json_encode(array('result' => 0, 'message' => $validation_errors, 'type' => 'error'));
	        } else {
	            if ($validation_errors) {
	                $message = ['success' => false, 'message' => $validation_errors];
	            } else {
	                $message = ['success' => false, 'message' => lang('msg_failure')];
	            }
	            return $message;
	        }
	    } else {
	        $id = escape($this->input->post('row_id'));
	        $module = escape($this->input->post('module'));
	        $owner_detail = array();
	        if(empty($module)) {
	            // Get Listing info to be sent
	            $result = $this->listings_model->find($id);
	            $owner_detail['name'] = $result->contact_person;
	            $owner_detail['number'] = !empty($result->mobile_number) ? $result->mobile_number : $result->phone_number;
	            $message = $result->title . ', ' .$result->address .', ' .$result->mobile_number .', ' .lang('sms_info_source') .' ' . site_url();
	            $this->gc->send_sms($message, $this->input->post('sms_mobile'));
	        } else {
	            $record = $this->gc->get_module_row_detail($module, $id);
	            if(!empty($record)) {
	                $author = $this->gc->get_module_owner($record);
	                if(!empty($author)) {
	                    $owner_detail['name'] = $author->primary_contact;
	                    $owner_detail['number'] = !empty($author->author_mobile_number) ? $author->author_mobile_number : $author->author_phone_number;
	                    $message = $record->title . ', ' . $author->primary_contact . ', ' . $author->author_phone_number . ', ' . lang('sms_info_source') . ' ' . site_url();
	                    $this->gc->send_sms($message, $this->input->post('sms_mobile'));
	                }
	            }
	        }
	        
	        // Notify Listing Owner about SMS Enquiry
	        if(settings_item('lst.sms_notify_owner') == 1 && !empty($owner_detail['number'])) {
	            $owner_message = '';
	            if(!empty($owner_detail['name'])) {
	                $owner_message = sprintf(lang('sms_owner_name'), $owner_detail['name']);
	            }
	            $owner_message .= sprintf(lang('sms_owner_message'), $this->input->post('sms_mobile'), settings_item('site.title'));
	            $this->gc->send_sms($owner_message, $owner_detail['number']);
	        }
	        
	        if($mobile) {
	            $message = ['success' => true, 'message' => lang('sms_success')];
	            return $message;
	        } else {
	            echo json_encode(array('result' => 1, 'message' => lang('sms_success'), 'type' => 'success'));
	        }
	    }
	}
	
	/**
	 * Allow users to send Email
	 * @return array
	 */
	public function send_email($mobile = false) {
	    $this->form_validation->set_rules('email_to', 'lang:label_friends_email', 'required|valid_email|trim');
	    $this->form_validation->set_rules('email_from', 'lang:label_your_email', 'required|valid_email|trim');
	    $this->form_validation->set_rules('name', 'lang:label_name', 'required|trim');
	    if(!$mobile) {
	        $this->form_validation->set_rules('captcha_code', 'lang:captcha_code', 'trim|required|valid_captcha');
	    }
	    if ($this->form_validation->run() === FALSE) {
	        $validation_errors = validation_errors();
	        if(!$mobile) {
	            echo json_encode(array('result' => 0, 'message' => $validation_errors, 'type' => 'error'));
	        } else {
	            if ($validation_errors) {
	                $message = ['success' => false, 'message' => $validation_errors];
	            } else {
	                $message = ['success' => false, 'message' => lang('msg_failure')];
	            }
	            return $message;
	        }
	    } else {
	        // Send Module Email
	        $from = escape($this->input->post('name'));
	        $email_to = escape($this->input->post('email_to'));
	        $email_from = escape($this->input->post('email_from'));
	        $module = escape($this->input->post('module'));
	        $id = int($this->input->post('row_id'));
	        $module = gc_module_id($module) == 0 ? 'listings' : $module;
	        $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'module_email_templates', 'module_table' => $module, 'action_type' => 'share', 'with_default' => 1, 'id' => $id, 'email_to' => $email_to));
	        if($mobile) {
	            $message = ['success' => true, 'message' => lang('success_message')];
	            return $message;
	        } else {
	            echo json_encode(array('result' => 1, 'message' => lang('success_message'), 'type' => 'success'));
	        }
	    }
	}
	
	/**
	 * Save Feedbacks/Callbacks
	 * @param string $module - callbacks/feedbacks
	 */
	public function feedbacks_callbacks($module = 'callbacks', $is_mobile = false, $rest_extra = array()) {
	    $this->lang->load('emailer/emailer');
	    $prefix = $is_mobile ? '' : $module . '_';
	    $this->form_validation->set_rules($prefix . 'name', 'lang:label_name', 'required|trim|max_length[255]|sanitize');
	    $this->form_validation->set_rules($prefix . 'email', 'lang:bf_email', 'trim|valid_email');
	    $this->form_validation->set_rules($prefix . 'phone_number', 'lang:label_phone', 'phone_check|trim|required');
	    if($module == 'callbacks') {
	        $this->form_validation->set_rules($prefix . 'country_code', 'lang:label_code', 'required|trim|sanitize');
	        $this->form_validation->set_rules($prefix . 'country_iso', 'lang:label_location', 'required|trim|sanitize');
	    } else {
	        $this->form_validation->set_rules($prefix . 'subject', 'lang:label_subject', 'trim|required|max_length[255]|sanitize');
	        $this->form_validation->set_rules($prefix . 'message', 'lang:label_message', 'trim|required|sanitize');
	    }
	    if(!$is_mobile) {
	        $this->form_validation->set_rules($prefix . 'captcha_code', 'lang:captcha_code', 'trim|required|valid_captcha');
	    }
	    if ($this->form_validation->run() === FALSE) {
	        $validation_errors = validation_errors();
	        if ($validation_errors) {
	            $validation_error = $validation_errors;
	        }
	        $request_result = array('result' => 0, 'message' => $validation_error, 'type' => 'danger');
	        if($is_mobile) {
	            $request_result['success'] = false;
	            return $request_result;
	        } else {
	            echo json_encode($request_result);
	            return false;
	        }
	    } else {
	        $email_template_id = $module == 'callbacks' ? 38 : 37;
	        $data = array(
	            'name' => escape($this->input->post ( $prefix . 'name' )),
	            'email' => $this->input->post($prefix . 'email'),
	            'phone_number' => escape($this->input->post ( $prefix . 'phone_number' )),
	        );
	        if($module == 'callbacks') {
	            $data['country_iso'] = escape($this->input->post ( $prefix . 'country_iso' ));
	            $data['country_code'] = escape($this->input->post ( $prefix . 'country_code' ));
	        } else {
	            $data['subject'] = escape($this->input->post ( $prefix . 'subject' ));
	            $data['message'] = escape($this->input->post ( $prefix . 'message' ));
	        }
	        $data['created_on'] = date('Y-m-d H:m:s');
	        $data['created_ip'] = $this->input->ip_address();
	        $data['user_id'] = !empty($rest_extra['user_id']) ? (int) $rest_extra['user_id'] : (int) $this->session->userdata('user_id');
	        
	        // Insert Data
	        $this->db->insert($module, $data );
	        $insert_id = $this->db->insert_id();
	        if ($insert_id > 0) {
	            $message = lang('success_message');
	            $type = 'success';
	            $result = 1;
	            
	            // Customers and Admin Email
	            $table_prefix = $module == 'feedbacks' ? 'feedback' : 'callback';
	            $this->gemailer->sms_email_pdf('sms_email', array('template_table' => $table_prefix . '_email_templates', 'module_table' => $module, 'action_type' => 'add', 'id' => $insert_id));
	        } else {
	            $message = lang('failure_message');
	            $type = 'error';
	            $result = 0;
	        }
	        
	        $request_result = array('result' => $result, 'message' => $message, 'type' => $type);
	        if($is_mobile) {
	            $request_result['success'] = true;
	            return $request_result;
	        } else {
	            echo json_encode($request_result);
	            return true;
	        }
	    }
	}
	
	/**
	 * Rate Business
	 * @return <multitype:, multitype:string >
	 */
	public function rate_business($user_id = 0, $mobile = false) {
	    if (settings_item('lst.loggedin_rating_only') == 1 && $user_id == 0) { // Only Loggedin Users can Rate a Business
	        $message = array(
	            'success' => false,
	            'label' => 'danger',
	            'result' => lang('error_loggedin_rating_only') . (!$mobile ? ' <a class="white-color" href="' . gc_url_format('login') .'">'.lang('msg_click_here_login').'</a>.' : '')
	        );
	        return $message;
	    }
	    $module = escape($this->input->post('module'));
	    $row_id = escape($this->input->post('row_id'));
	    $ip_address = $this->input->ip_address();
	    $new_rating = $this->input->post('ratings', true);
	    
	    $data = array (
	        'row_id' => $row_id,
	        'module_id' => gc_module_id($module),
	        'ip_address' => $ip_address,
	        'new_rating' => $new_rating
	    );
	    $result = $this->gc->star_rating($data, $this->auth->user_id());
	    
	    $message = array();
	    switch($result) {
	        case 1:
	        case 4:
	            $message = array(
	            'success' => true,
	            'label' => 'success',
	            'result' => lang('success_rating'),
	            'message' => lang('success_rating') // for mobile
	            );
	            break;
	        case 0:
	        case 2:
	        case 5:
	            $message = array(
	            'success' => false,
	            'label' => 'danger',
	            'result' => sprintf(lang('error_rating'), lang('singular_' . $module)),
	            'message' => sprintf(lang('error_rating'), lang('singular_' . $module))
	            );
	            break;
	        case 3:
	            $message = array(
	            'success' => false,
	            'label' => 'danger',
	            'result' => sprintf(lang('failure_rating'), lang('singular_' . $module)),
	            'message' => sprintf(lang('failure_rating'), lang('singular_' . $module))
	            );
	            break;
	    }
	    if($mobile && $message['success']) {
	        $this->load->model('ratings/ratings_model');
	        $message['rating_info'] = $this->ratings_model->get_rating($module, $row_id);
	    }
	    return $message;
	}
}