<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class Claims_model extends BF_Model {

	protected $table_name	= "listing_claims";
	protected $key			= "id";
	protected $soft_deletes	= false;
	protected $date_format	= "datetime";

	protected $log_user 	= FALSE;

	protected $set_created	= true;
	protected $set_modified = true;
	protected $created_field = "reported_on";
	protected $modified_field = "modified_on";

	/*
		Customize the operations of the model without recreating the insert, update,
		etc methods by adding the method names to act as callbacks here.
	 */
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 		= array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	/*
		For performance reasons, you may require your model to NOT return the
		id of the last inserted row as it is a bit of a slow method. This is
		primarily helpful when running big loops over data.
	 */
	protected $return_insert_id 	= TRUE;

	// The default type of element data is returned as.
	protected $return_type 			= "object";

	// Items that are always removed from data arrays prior to
	// any inserts or updates.
	protected $protected_attributes = array();

	/*
		You may need to move certain rules (like required) into the
		$insert_validation_rules array and out of the standard validation array.
		That way it is only required during inserts, not updates which may only
		be updating a portion of the data.
	 */
	protected $insert_validation_rules 	= array();
	protected $skip_validation 			= FALSE;

	//--------------------------------------------------------------------
	
	/**
	 * Get claims for index page
	 */
	public function get_listing_claims($offset, $limit, $where, $order_by) {
		$sql = "SELECT LC.id, LC.order_id, LC.listing_id, LC.claimed_by as claimed_user, LC.owned_by as owned_user, L.title AS listing_title, U.display_name AS claimed_by, OB.display_name AS owned_by,"; 
  		$sql .= " LC.reported_on, LC.modified_on, LC.ip, LC.status FROM (SELECT * FROM " . $this->db->dbprefix . "listing_claims ";
  		if(!empty($where) && (strpos($where,'SUBSTRING') !== false)) {
  			$sql .= " WHERE ".$where;
  		} else if(!empty($where)) {
  			$sql .= " WHERE ".$where;
  		}
		$sql .= " ORDER BY reported_on desc LIMIT " . $offset . ", " . $limit .") LC";
  		$sql .= " LEFT JOIN " . $this->db->dbprefix . "listings L ON L.id = LC.listing_id";
  		$sql .= " LEFT JOIN " . $this->db->dbprefix . "users U ON U.id = LC.claimed_by";
  		$sql .= " LEFT JOIN " . $this->db->dbprefix . "users OB ON OB.id = LC.owned_by";
		$query = $this->db->query ( $sql );
		$result = $query->result_array ();
		return $result;
	}
	
	/**
	 * Send Claim Email
	 * @param int $claim_id
	 * @param int $claimed_by
	 */
	public function email_claim($claim_id) {
		$this->db->select($this->db->dbprefix . 'listing_claims.id as claim_id, ' . $this->db->dbprefix . 'listing_claims.order_id, ' . $this->db->dbprefix . 'listing_claims.reported_on, ' . $this->db->dbprefix . 'listing_claims.status, L.title as listing_title, U.email, U.display_name, U.phone_number');
		$this->db->join($this->db->dbprefix . 'listings L','L.id = ' . $this->db->dbprefix . 'listing_claims.listing_id', 'LEFT');
		$this->db->join($this->db->dbprefix . 'users U','U.id = ' . $this->db->dbprefix . 'listing_claims.claimed_by', 'LEFT');
		return $this->claims_model->find_by(array($this->db->dbprefix . 'listing_claims.id' => $claim_id));		
	}
	
	/**
	 * Take action on claim status change
	 * Also used by Orders module
	 * @param int $claim_id
	 * @param int $status
	 * @param int $notify_customer
	 */
	public function update_claim_status($claim_id, $status, $notify_customer = 0) {
		$message = sprintf(lang('msg_edit_failure'), lang('label_claim'));
		$claim_info = $this->claims_model->find($claim_id);
		if(!$claim_info) {
			return $message;
		}
		if($claim_info->status == $status) {
			$message = lang('msg_nothing_to_update');
			return $message;
		}
		$claimed_by = $claim_info->claimed_by;
		$owned_by = $claim_info->owned_by;
		$claimed_listing_id = $claim_info->listing_id;
		$this->db->where('id', $claim_id)->update('listing_claims', array('status' => $status));
		if($status == 1) {
			/**
			 * get listing existing expires_on and featured status
			 * and save them in the listing_claims table for revert
			 */
			$listing_info = $this->listings_model->select('featured, expires_on')->find($claimed_listing_id);
			if(empty($listing_info)) {
				$message = lang('msg_listing_not_exist');
				return $message;
			}
			$this->db->where('order_id', $claim_info->order_id)->update('listing_claims', array('listing_featured' => $listing_info->featured, 'listing_expiry' => $listing_info->expires_on));
		
			/**
			 * Get subscribed package from orders table for expires_on
			 * Get featured information from packages table
			 * Update orders table to set status of order to claimed
			*/
			// $sql = "SELECT package_id, order_id, expires_on FROM " . $this->db->dbprefix . "orders O WHERE O.user_id = " . $claimed_by . " AND O.order_type = 1 AND O.status = 1 AND (O.expires_on IS NULL OR O.expires_on = '00-00-0000' OR O.expires_on >= '". date('Y-m-d') ."')";
			$sql = "SELECT row_id AS package_id, order_id, expires_on FROM " . $this->db->dbprefix . "order_items OT WHERE OT.user_id = " . $claimed_by . " AND OT.module_id = " . gc_module_id('packages') . " AND OT.status_id = 2 AND (OT.expires_on IS NULL OR OT.expires_on = '00-00-0000' OR OT.expires_on >= '". date('Y-m-d') ."')";
			$active_membership = $this->db->query($sql)->row();
			if(!empty($active_membership)) {
				// get package information
				$this->load->model('packages/packages_model');
				$package_info = $this->packages_model->find($active_membership->package_id);
				if(!$package_info) {
					return $message;
				}
				$listing_data['user_id'] = $claimed_by;
				$listing_data['claimed'] = 1;
				$listing_data['expires_on'] = $active_membership->expires_on;
				$listing_data['featured'] = gc_field_value('listings', $package_info->module_featured);
				// update listing table
				$this->db->where('id', $claimed_listing_id)->update('listings', $listing_data);
				$this->listings_model->update_module($claimed_listing_id, 'user_id', $claimed_by);
				// update business queries
				$this->db->where('listing_id', $claimed_listing_id)->update('business_queries', array('owner_id' => $claimed_by, 'modified_on' => date('Y-m-d H:i:s'), 'modified_ip' => $this->input->ip_address()));
			} else {
				$message = lang('msg_no_active_membership');
				return $message;
			}
		} else {
			$listing_info = $this->claims_model->select('listing_expiry, listing_featured')->find_by('order_id', $claim_info->order_id);
			if(!$listing_info) {
				return $message;
			}
			$listing_data['user_id'] = $owned_by;
			$listing_data['claimed'] = 0;
			$listing_data['expires_on'] = $listing_info->listing_expiry;
			$listing_data['featured'] = $listing_info->listing_featured;
			$this->db->where('id', $claimed_listing_id)->update('listings', $listing_data);
			$this->listings_model->update_module($claimed_listing_id, 'user_id', $owned_by);
			// update business queries
			$this->db->where('listing_id', $claimed_listing_id)->update('business_queries', array('owner_id' => $owned_by, 'modified_on' => date('Y-m-d H:i:s'), 'modified_ip' => $this->input->ip_address()));
		}
		if($notify_customer == 1) {
			$this->claim_email($claim_id);
		}
		return true;
	}
	
	/**
	 * Send claim email
	 * @param int $claimed_by
	 */
	public function claim_email($claim_id) {
	    //Retrieve user information from database
	    $claimed_info = $this->claims_model->email_claim($claim_id);
	    if($claimed_info) {
	        $status ='';
	        switch ($claimed_info->status) {
	            case 0:
	                $status = 'action_processed';
	                break;
	            case 1:
	                $status = 'tab_approved';
	                break;
	            case 2:
	                $status = 'tab_rejected';
	                break;
	            case 3:
	                $status = 'tab_forged';
	                break;
	            case 4:
	                $status = 'tab_cancelled';
	                break;
	        }
	        $claimed_info->status = $status;
	        
	        // send email
	        $this->gemailer->sms_email_pdf('email', array('template_table' => 'module_email_templates', 'module_table' => 'listing_claims', 'action_type' => 'status_update', 'id' => $claim_id, 'record' => $claimed_info));
	        
	        return $claimed_info;
	    } else {
	        Template::set_message(lang('msg_unable_to_notify'), 'error');
	    }
	}
	
	/**
	 * Allow User to claim a listing
	 * Get amount from the package_id (members/checkout method)
	 */
	public function request() {
	    $this->form_validation->set_rules('listing_id', 'lang:label_listing_id', 'trim|required');
	    $this->form_validation->set_rules('price', 'lang:label_price', 'trim|is_decimal');
	    if ($this->form_validation->run() === FALSE) {
	        $validation_errors = validation_errors();
	        if ($validation_errors) {
	            $validation_error = $validation_errors;
	        }
	        return array('success' => 0, 'result' => 0, 'message' => $validation_error, 'type' => 'danger');
	    } else {
	        $this->load->model('orders/orders_model');
	        $user_id = $this->auth->user_id();
	        $result = $this->orders_model->get_user_orders(1, $user_id);
	        if($result->count > 0) {
	            $listing_id = (int) $this->input->post('listing_id');
	            return $this->orders_model->free_paid_checkout($listing_id, 'claims', $user_id);
	        } else if($result->count == 0) {
	            return array('success' => 0, 'result' => 0, 'message' => lang('msg_active_membership_claim'), 'type' => 'danger');
	        }
	    }
	}
}
