<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Locations controller
 */
class Settings extends Admin_Controller
{
	protected $permissionView   = 'Locations.Settings.View';
	protected $permissionCreate = 'Locations.Settings.Create';
	protected $permissionDelete = 'Locations.Settings.Delete';
	protected $permissionEdit   = 'Locations.Settings.Edit';
	
	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct()
	{
		parent::__construct();

		$this->auth->restrict($this->permissionView);
		$this->load->model('countries_model', null, true);
		$this->load->model('states_model');
		$this->load->model('cities_model');
		$this->load->model('localities_model');
		$this->lang->load('locations');
		$this->lang->load('categories/categories'); // Counts
				
		Template::set_block('sub_nav', 'settings/_sub_nav');
	}
	
	/*----------------------------------------------------*/
	/*	MANAGE COUNTRIES
	/*----------------------------------------------------*/
	/**
	 * Display all countries to the user
	 * @param string $filter for alphabetical display
	 * @param number $offset for pagination
	 */
	public function index($filter='active', $offset=0)
	{	
	    // redirect back and forth between pages
	    $this->session->set_userdata('countries_last_visited_page', current_url());
	    
		// Status Change
	    if (isset($_POST['status_change'])) {
	        gc_demo_mode();
	        $this->gcrud->stats(array('module' => 'locations'));
			$this->gc->status_change('countries', $_POST['status_change'], 'active');
		}
		
		// Do we have any actions?
		$this->actions('countries');

		$where = array();
		// Filters
		if (preg_match('{first_letter-([A-Z])}', $filter, $matches)) {
			$filter_type = 'first_letter';
			$first_letter = $matches[1];
		} else {
			$filter_type = $filter;
		}
		
		switch($filter_type) {
			case 'active':
				$where['countries.active'] = 1;
				break;
			
			case 'inactive':
				$where['countries.active'] = 0;
				break;
				
			case 'first_letter':
				$where['SUBSTRING( LOWER(name), 1, 1)='] = $first_letter;
				break;
					
			case 'all':
				// Nothing to do
				break;
					
			default:
				show_404("locations/index/$filter/");
		}
		
		// Sort By
		$like_array = array();
		$searchterm = $this->gc->search('countries', $this->uri->segment(6), '/settings/locations/index');
		if(!empty($searchterm)) {
			$like_array = array(
				'iso' => $searchterm,
				'name' => $searchterm
			);
		}
		$order_by = $this->gc->sort('countries');
		if(!empty($order_by)) {
			$this->countries_model->order_by($this->session->userdata('countries_sort')['column'], $this->session->userdata('countries_sort')['order']);
		} else {
			$this->countries_model->order_by('iso', 'ASC');
		}
		if(!empty($like_array)) {
			$this->db->or_like($like_array);
		}
		$this->countries_model->limit($this->limit, $offset)->where($where);
		$records = $this->countries_model->find_all();		
		Template::set('records', $records);
		
		// Pagination
		$this->load->library('pagination');
		
		if(!empty($like_array)) {
			$this->db->or_like($like_array);
		}
		$this->countries_model->where($where);
		$total_countries = $this->countries_model->count_all();
		
		$pager['base_url'] = site_url(SITE_AREA ."/settings/locations/index/$filter/");
		$pager['total_rows'] = $total_countries;
		$pager['per_page'] = $this->limit;
		$pager['uri_segment']	= 6;
		
		$this->pagination->initialize($pager);
		
		Template::set('index_url', site_url(SITE_AREA .'/settings/locations/index/'));
		Template::set('filter_type', $filter_type);
		// Counts redirect - Last visited URL
		$this->session->set_userdata('last_visited_url', current_url());
		Template::set('toolbar_title', lang('manage_countries'));
		Template::render();
		
	}

	/**
	 * Allow admin to add new country
	 */
	public function add_country() {
	    Assets::add_css('selectize.bootstrap3.css');
	    Assets::add_js('selectize.min.js');
	    Assets::add_js('selectize-search.js');
		$this->auth->restrict($this->permissionCreate);
		$redirect_url = $this->session->userdata('countries_last_visited_page') ? $this->session->userdata('countries_last_visited_page') : SITE_AREA .'/settings/locations';
		if (isset($_POST['save'])) {
		    gc_demo_mode();
		    $this->gcrud->stats(array('module' => 'locations'));
			$check_existence = $this->countries_model->count_by(array('iso' => escape($this->input->post('iso'))));
			if($check_existence > 0) {
				Template::set_message(sprintf(lang('bf_form_unique'), lang('label_iso')), 'error');
			} else {
				if ($insert_id = $this->save_country()) {
					// Log the activity
					log_activity($this->auth->user_id(), lang('act_create_record') .': '. $insert_id .' : '. $this->input->ip_address(), 'locations');

					Template::set_message(sprintf(lang('msg_save_success'), lang('label_country')), 'success');
					redirect($redirect_url);
				} else {
					Template::set_message(sprintf(lang('msg_create_failure'), lang('label_country')) . $this->countries_model->error, 'danger');
				}
			}
		}
		
		Template::set('toolbar_title', lang('label_country_new'));
		Template::set_view('settings/countries_form');
		Template::render();
	}

	/**
	 * Allow admin to change country information
	 */
	public function edit_country() {	    
	    Assets::add_css('selectize.bootstrap3.css');
	    Assets::add_js('selectize.min.js');
	    Assets::add_js('selectize-search.js');
		$id = $this->uri->segment(5);
		$redirect_url = $this->session->userdata('countries_last_visited_page') ? $this->session->userdata('countries_last_visited_page') : SITE_AREA .'/settings/locations';
		if (empty($id)) {
			Template::set_message(lang('invalid_id'), 'error');
			redirect($redirect_url);
		}
		$record = $this->countries_model->find($id);
		if (isset($_POST['save'])) {
		    gc_demo_mode();
		    $this->gcrud->stats(array('module' => 'locations'));
			$this->auth->restrict($this->permissionEdit);

			if ($this->save_country('update', $id)) {
				// Log the activity
				log_activity($this->auth->user_id(), lang('act_edit_record') .': '. $id .' : '. $this->input->ip_address(), 'locations');

				Template::set_message(sprintf(lang('msg_save_success'), lang('label_country')), 'success');
				// Tweak Field Update
				$this->gc->update_tweak_fields('countries', $id, $record->name);
				Template::redirect($redirect_url);
			} else {
				Template::set_message(sprintf(lang('msg_edit_failure'), lang('label_country')) . $this->countries_model->error, 'danger');
			}
		} else if (isset($_POST['delete'])) {
		    gc_demo_mode();
		    $this->gcrud->stats(array('module' => 'locations'));
			$this->auth->restrict($this->permissionDelete);

			if ($this->countries_model->delete($id)) {
				// Log the activity
				log_activity($this->auth->user_id(), lang('act_delete_record') .': '. $id .' : '. $this->input->ip_address(), 'locations');

				Template::set_message(sprintf(lang('msg_delete_success'), lang('label_country')), 'success');

				redirect($redirect_url);
			} else {
				Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_country')) . $this->countries_model->error, 'danger');
			}
		}
		Template::set('countries', $record);
		Template::set('toolbar_title', lang('label_country_edit'));
		Template::set_view('settings/countries_form');
		Template::render();
	}

	/**
	 * Save Country information
	 * @param string $type
	 * @param number $id
	 * @return number
	 */
	private function save_country($type='insert', $id=0) {
		if ($type == 'update') {
			$_POST['iso'] = $id;
		}
		// make sure we only pass in the fields we want		
		$data = array();
		$data['iso']        		= $this->input->post('iso');
		$data['name']        		= $this->input->post('name');		
		// Check whether slug is entered or not
		$this->load->library('slug');
		if ($this->input->post('slug')) {
		    $data['slug'] 		= $this->slug->create_slug($this->input->post('slug'));
		} else {
		    $data['slug'] 		= $this->slug->create_slug($data['name']);
		}
		$data['image']        		= $this->input->post('image');		
		$data['printable_name'] 	= $this->input->post('printable_name');
		$data['iso3']       		= $this->input->post('iso3');
		$data['numcode']        	= (int) $this->input->post('numcode');
		$data['phone_code']        	= $this->input->post('phone_code');
		$data['payment_methods']   = $this->input->post('payment_methods');
		$data['currency_code']      = $this->input->post('currency_code');
		$data['active']        	    = $this->input->post('active') ? 1 : 0;
		if ($type == 'insert') {
			$id = $this->countries_model->insert($data);
			if (is_numeric($id)) {
				$return = $this->countries_model->count_all() + 1;
			} else {
				$return = FALSE;
			}
		} elseif ($type == 'update') {
			$return = $this->countries_model->update($id, $data);
		}

		return $return;
	}
	
	/*----------------------------------------------------*/
	/*	MANAGE STATES
	/*----------------------------------------------------*/
	
	/**
	 * Display states to the user
	 * @param string $country_iso
	 * @param string $filter
	 * @param number $offset
	 */
	public function states($country_iso = NULL, $filter='active', $offset=0, $id = NULL) {
		if($country_iso == NULL) {
			Template::set_message(lang('selection_failure'), 'error');
			redirect(site_url(SITE_AREA .'/settings/locations/'));
		}
		
		// Status Change
		if (isset($_POST['status_change'])) {
		    gc_demo_mode();
		    $this->gcrud->stats(array('module' => 'locations'));
			$this->gc->status_change('states', $_POST['status_change'], 'active');
		}
		
		// Do we have any actions?
		$this->actions('states');
		
		$where = array();
		// Filters
		if (preg_match('{first_letter-([A-Z])}', $filter, $matches)) {
			$filter_type = 'first_letter';
			$first_letter = $matches[1];
		} else {
			$filter_type = $filter;
		}
		
		switch($filter_type) {
			case 'active':
				$where['states.active'] = 1;
				break;
					
			case 'inactive':
				$where['states.active'] = 0;
				break;
				
			case 'first_letter':
				$where['SUBSTRING( LOWER(name), 1, 1)='] = $first_letter;
				break;
					
			case 'all':
				// Nothing to do
				break;
					
			default:
				show_404("locations/states/$country_iso/$filter/");
		}
		
		//FOR NAVIGATION
		$this->session->set_userdata('country_iso', $country_iso);
		
		// Sort By
		$like_array = array();
		$searchterm = $this->gc->search('states', $this->uri->segment(6), '/settings/locations/states/' . $country_iso);
		if(!empty($searchterm)) {
			$like_array = array(
				'states.id' => $searchterm,
				'states.name' => $searchterm
			);
		}
		$order_by = $this->gc->sort('states');
		if(!empty($order_by)) {
			$this->states_model->order_by($this->session->userdata('states_sort')['column'], $this->session->userdata('states_sort')['order']);
		} else {
			$this->states_model->order_by('name', 'ASC');
		}
		if(!empty($like_array)) {
			$this->db->or_like($like_array);
		}
		
		$this->states_model->limit($this->limit, $offset)->where($where);
		if($country_iso != NULL) {
			$records = $this->states_model->where('country_iso', $country_iso)->find_all();
		} else {
			$records = $this->states_model->find_all();
		}
		Template::set('records', $records);
		$record = array();
		if(!empty($id)) {
			$record = $this->states_model->find($id);
		}
		// Add and update state		
		if (isset($_POST['save'])) {
		    gc_demo_mode();
		    $this->gcrud->stats(array('module' => 'locations'));
			if($id == NULL) {
				$this->auth->restrict($this->permissionCreate);
				if ($insert_id = $this->save_state()) {
					// Log the activity
					log_activity($this->auth->user_id(), lang('act_create_record') .': '. $insert_id .' : '. $this->input->ip_address(), 'locations');
			
					Template::set_message(sprintf(lang('msg_save_success'), lang('label_state')), 'success');
					if($this->uri->segment(5) && ($this->uri->segment(6)) && ($this->uri->segment(7))) {
						redirect(site_url(SITE_AREA .'/settings/locations/states/'.$this->uri->segment(5).'/'.$this->uri->segment(6).'/'.$this->uri->segment(7).'/'));
					} elseif($this->uri->segment(5) && ($this->uri->segment(6))) {
						redirect(site_url(SITE_AREA .'/settings/locations/states/'.$this->uri->segment(5).'/'.$this->uri->segment(6).'/'));
					} else {
						redirect(site_url(SITE_AREA . '/settings/locations/states/'.$this->uri->segment(5)));
					} 
				} else {
					Template::set_message(sprintf(lang('msg_create_failure'), lang('label_state')) . $this->states_model->error, 'danger');
				}
			} else {			
				$this->auth->restrict($this->permissionEdit);
				if ($this->save_state('update', $id)) {
					// Log the activity
					log_activity($this->auth->user_id(), lang('act_edit_record') .': '. $id .' : '. $this->input->ip_address(), 'locations');
					// Tweak Field Update
					$this->gc->update_tweak_fields('states', $id, $record->name);
					Template::set_message(sprintf(lang('msg_save_success'), lang('label_state')), 'success');
					if($this->uri->segment(5) && ($this->uri->segment(6)) && ($this->uri->segment(7))) {
						redirect(site_url(SITE_AREA .'/settings/locations/states/'.$this->uri->segment(5).'/'.$this->uri->segment(6).'/'.$this->uri->segment(7).'/'));						
					} elseif($this->uri->segment(5) && ($this->uri->segment(6))) {
						redirect(site_url(SITE_AREA .'/settings/locations/states/'.$this->uri->segment(5).'/'.$this->uri->segment(6).'/'));
					} else {
						redirect(site_url(SITE_AREA . '/settings/locations/states/'.$this->uri->segment(5)));
					}
				} else {
					Template::set_message(sprintf(lang('msg_edit_failure'), lang('label_state')) . $this->states_model->error, 'danger');
				}
			}
		}
	
		// Pagination
		$this->load->library('pagination');
		
		if(!empty($like_array)) {
			$this->db->or_like($like_array);
		}
		$this->states_model->where($where);
		$total_states = $this->states_model->where('country_iso', $country_iso)->count_all();
	
		$pager['base_url'] = site_url(SITE_AREA ."/settings/locations/states/$country_iso/$filter/");
		$pager['total_rows'] = $total_states;
		$pager['per_page'] = $this->limit;
		$pager['uri_segment']	= 7;
	
		$this->pagination->initialize($pager);
	
		Template::set('states', $record);
		
		Template::set('states_url', site_url(SITE_AREA .'/settings/locations/states/'. $country_iso .'/' . $filter . '/') .'/');
		Template::set('index_url', site_url(SITE_AREA .'/settings/locations/states/' . $country_iso . '/') .'/');
		Template::set('filter_type', $filter_type);
		
		// Counts redirect - Last visited URL
		$this->session->set_userdata('last_visited_url', current_url());
		Template::set('toolbar_title', lang('label_manage_states'));
		Template::render();
	}
	
	/**
	 * SAVE STATE INFORMATION
	 * @param string $type
	 * @param number $id
	 * @return number
	 */
	private function save_state($type='insert', $id=0) {
		if ($type == 'update') {
			$_POST['id'] = $id;
		}
	
		// make sure we only pass in the fields we want
	
		$data = array();
		$data['name']       = $this->input->post('name');
		$data['slug']		= $this->gc->slugify('states', $data['name'], $id, 'name');
		$data['image']    	= $this->input->post('image');	
		$data['abbrev']     = $this->input->post('abbrev');
		$data['country_iso']= $this->session->userdata('country_iso');		
		if(!empty($data['country_iso'])) {
		    $country = $this->db->where('iso', $data['country_iso'])->get('countries')->row();
		    if($country) {
		        $data['country_id'] = $country->id;
		        // $data['country'] = $country->printable_name;
		    }
		}
		if ($type == 'insert') {
			$id = $this->states_model->insert($data);
			if (is_numeric($id)) {
				$return = $id;
			} else {
				$return = FALSE;
			}
		}
		elseif ($type == 'update') {
			$return = $this->states_model->update($id, $data);
		}
		return $return;
	}	

	/*----------------------------------------------------*/
	/*	MANAGE CITIES
	/*----------------------------------------------------*/
	
	/**
	 * Allow admin to view, add and update cities information
	 * @param number $state_id
	 * @param string $filter
	 * @param number $offset
	 */
	public function cities($state_id = 0, $filter = 'active', $offset = 0, $city_id = NULL)
	{
		if($state_id == 0) {
			Template::set_message(lang('selection_failure'), 'error');
			redirect(site_url(SITE_AREA .'/settings/locations/'));
		}
		//CHECK WHETHER CITY IS ALPHA OR NUMERIC, IF ALPHA CONVERT IT TO INTEGER
		$state_id = (int)$state_id;
		
		// Status Change
		if (isset($_POST['status_change'])) {
		    gc_demo_mode();
		    $this->gcrud->stats(array('module' => 'locations'));
			$this->gc->status_change('cities', $_POST['status_change'], 'active');
		}
		$record = array();
		if(!empty($city_id)) {
			$record = $this->cities_model->find($city_id);
		}
		if (isset($_POST['save'])) {
		    gc_demo_mode();
		    $this->gcrud->stats(array('module' => 'locations'));
			if($city_id == NULL) {
				if ($insert_id = $this->save_city()) {
					// Log the activity
					log_activity($this->auth->user_id(), lang('act_create_record') .': '. $insert_id .' : '. $this->input->ip_address(), 'locations');
					
					Template::set_message(sprintf(lang('msg_save_success'), lang('label_city')), 'success');
					if($this->uri->segment(5) && ($this->uri->segment(6)) && ($this->uri->segment(7))) {
						redirect(site_url(SITE_AREA .'/settings/locations/cities/'.$this->uri->segment(5).'/'.$this->uri->segment(6).'/'.$this->uri->segment(7).'/'));
					} else if($this->uri->segment(5) && ($this->uri->segment(6))) {
						redirect(site_url(SITE_AREA .'/settings/locations/cities/'.$this->uri->segment(5).'/'.$this->uri->segment(6).'/'));
					} else {
						redirect(site_url(SITE_AREA . '/settings/locations/cities/'.$this->uri->segment(5)));
					} 
				}
				else {
					Template::set_message(sprintf(lang('msg_create_failure'), lang('label_city')) . $this->cities_model->error, 'danger');
				}
			} else {
				$this->auth->restrict($this->permissionEdit);				
				if ($this->save_city('update', $city_id)) {
					// Log the activity
					log_activity($this->auth->user_id(), lang('act_edit_record') .': '. $city_id .' : '. $this->input->ip_address(), 'locations');
					// Tweak Field Update
					$this->gc->update_tweak_fields('cities', $city_id, $record->name);
					Template::set_message(sprintf(lang('msg_save_success'), lang('label_city')), 'success');
					if($this->uri->segment(5) && ($this->uri->segment(6)) && ($this->uri->segment(7))) {
						redirect(site_url(SITE_AREA .'/settings/locations/cities/'.$this->uri->segment(5).'/'.$this->uri->segment(6).'/'.$this->uri->segment(7).'/'));
					} else if($this->uri->segment(5) && ($this->uri->segment(6))) {
						redirect(site_url(SITE_AREA .'/settings/locations/cities/'.$this->uri->segment(5).'/'.$this->uri->segment(6).'/0/'));
					} else {
						redirect(site_url(SITE_AREA . '/settings/locations/cities/'.$this->uri->segment(5)));
					} 
				} else {
					Template::set_message(sprintf(lang('msg_edit_failure'), lang('label_city')) . $this->cities_model->error, 'danger');
				}
			}
		}
		
		// Do we have any actions?
		$this->actions('cities');
		
		$where = array();
		// Filters
		if (preg_match('{first_letter-([A-Z])}', $filter, $matches)) {
			$filter_type = 'first_letter';
			$first_letter = $matches[1];
		} else {
			$filter_type = $filter;
		}
		
		switch($filter_type) {
			case 'active':
				$where['cities.active'] = 1;
				break;
					
			case 'inactive':
				$where['cities.active'] = 0;
				break;
			
			case 'first_letter':
				$where['SUBSTRING( LOWER(' . $this->db->dbprefix . 'cities.name), 1, 1)='] = $first_letter;
				break;					
			case 'all':
				// Nothing to do
				break;					
			default:
				show_404("locations/cities/$state_id/$filter/");
		}
		
		
		//FOR NAVIGATION
		$this->session->set_userdata('state_id', $state_id);
		
		// Sort By
		$like_array = array();
		$searchterm = $this->gc->search('cities', $this->uri->segment(6), '/settings/locations/cities/' . $state_id);
		if(!empty($searchterm)) {
			$like_array = array(
					'cities.id' => $searchterm,
					'cities.name' => $searchterm
			);
		}
		$order_by = $this->gc->sort('cities');
		if(!empty($order_by)) {
			$this->cities_model->order_by($this->session->userdata('cities_sort')['column'], $this->session->userdata('cities_sort')['order']);
		} else {
			$this->cities_model->order_by('name', 'ASC');
		}
		if(!empty($like_array)) {
			$this->db->or_like($like_array);
		}
			
		$this->cities_model->limit($this->limit, $offset)->where($where);
	
		if($state_id == 0) {	//USER ENTERED URL WITHOUT CITY ID
			$records = $this->cities_model->get_cities_state(0);
		}
		else {
			$records = $this->cities_model->get_cities_state($state_id);
		}
	
		Template::set('records', $records);
	
		// Pagination
		$this->load->library('pagination');
		
		if(!empty($like_array)) {
			$this->db->or_like($like_array);
		}
		$this->cities_model->where($where);
		if($state_id == 0) {
			$total_cities = $this->cities_model->count_all();
		}
		else {
			$total_cities = $this->cities_model->where('state_id', $state_id)->count_all();
		}
		$pager['base_url'] = site_url(SITE_AREA ."/settings/locations/cities/$state_id/$filter/");
		$pager['total_rows'] = $total_cities;
		$pager['per_page'] = $this->limit;
		$pager['uri_segment']	= 7;
	
		$this->pagination->initialize($pager);
		Template::set('index_url', site_url(SITE_AREA .'/settings/locations/cities/' . $state_id . '/') .'/');
		Template::set('filter_type', $filter_type);
		Template::set('cities', $record);
		// Counts redirect - Last visited URL
		$this->session->set_userdata('last_visited_url', current_url());
		Template::set('toolbar_title', lang('label_manage_cities'));
		Template::render();
	}
	
	/**
	 * Save City Information
	 * @param string $type
	 * @param number $id
	 * @return number
	 */
	private function save_city($type='insert', $id=0)
	{
		$this->load->library('gc');
		if ($type == 'update') {
			$_POST['id'] = $id;
		}
		$data = array();
		$data['name']       = $this->input->post('name');
		$data['slug']		= $this->gc->slugify('cities', $data['name'], $id, 'name');
		$data['abbrev']     = $this->input->post('abbrev');
		$data['state_id']	= $this->session->userdata('state_id');
		$data['image']      = $this->input->post('image');
		$data['description']= $this->input->post('description');
		$data['hits']       = $this->input->post('hits');
		// get country_iso and save in the cities table for Export/Import
		$state = $this->db->where('id', $data['state_id'])->get('states')->row();
		if($state) {
		    $data['country_id'] = $state->country_id;
		    $data['country_iso'] = $state->country_iso;
		    $data['state'] = $state->name;
		    if(empty($state->country)) {
		        $country = $this->db->where('iso', $data['country_iso'])->get('countries')->row();
		        $data['country_id'] = $country->id;
		        $data['country'] = $country->printable_name;
		    } else { 
		        $data['country'] = $state->country;
		    }
		}
			
		if ($type == 'insert') {
			$data['created_ip'] = $this->input->ip_address();
			$id = $this->cities_model->insert($data);
			if (is_numeric($id)) {
				$return = $id;
			} else {
				$return = FALSE;
			}
		} elseif ($type == 'update') {
			$data['modified_ip'] = $this->input->ip_address();
			$return = $this->cities_model->update($id, $data);
		}
		return $return;
	}
	
	/*----------------------------------------------------*/
	/*	MANAGE LOCALITIES
	/*----------------------------------------------------*/
	
	/**
	 * Allow admin to view, add and update localities
	 * @param number $city_id
	 * @param string $filter
	 * @param number $offset
	 */
	public function localities($city_id = 0, $filter='active', $offset=0, $locality_id = NULL)
	{		
		//CHECK WHETHER CITY IS ALPHA OR NUMERIC, IF ALPHA CONVERT IT TO INTEGER
		$city_id = (int)$city_id;
		if($city_id == 0) {
			Template::set_message(lang('selection_failure'), 'error');
			redirect(site_url(SITE_AREA .'/settings/locations/'));
		}
		
		// Status Change
		if (isset($_POST['status_change'])) {
		    gc_demo_mode();
		    $this->gcrud->stats(array('module' => 'locations'));
			$this->gc->status_change('localities', $_POST['status_change'], 'active');
		}
		
		$record = array();
		if(!empty($locality_id)) {
			$record = $this->localities_model->find($locality_id);
		}
		if (isset($_POST['save'])) {
		    gc_demo_mode();
		    $this->gcrud->stats(array('module' => 'locations'));
			if($locality_id == NULL) {
				$this->auth->restrict($this->permissionCreate);
				if ($insert_id = $this->save_locality()) {
					// Log the activity
					log_activity($this->auth->user_id(), lang('act_create_record') .': '. $insert_id .' : '. $this->input->ip_address(), 'locations');
			
					Template::set_message(sprintf(lang('msg_save_success'), lang('label_locality')), 'success');
					if($this->uri->segment(5) && ($this->uri->segment(6)) && ($this->uri->segment(7))) {
						redirect(site_url(SITE_AREA .'/settings/locations/localities/'.$this->uri->segment(5).'/'.$this->uri->segment(6).'/'.$this->uri->segment(7).'/'));
					} else if($this->uri->segment(5) && ($this->uri->segment(6))) {
						redirect(site_url(SITE_AREA .'/settings/locations/localities/'.$this->uri->segment(5).'/'.$this->uri->segment(6)));
					} else {
						redirect(site_url(SITE_AREA . '/settings/locations/localities/'.$this->uri->segment(5)));
					} 
				} else {
					Template::set_message(sprintf(lang('msg_create_failure'), lang('label_locality')) . $this->localities_model->error, 'danger');
				}
			} else {
				$this->auth->restrict($this->permissionEdit);
				if ($this->save_locality('update', $locality_id)) {
					// Log the activity
					log_activity($this->auth->user_id(), lang('act_edit_record') .': '. $locality_id .' : '. $this->input->ip_address(), 'locations');
					// Tweak Field Update
					$this->gc->update_tweak_fields('localities', $locality_id, $record->name);
					Template::set_message(sprintf(lang('msg_save_success'), lang('label_locality')), 'success');
					if($this->uri->segment(5) && ($this->uri->segment(6)) && ($this->uri->segment(7))) {
						redirect(site_url(SITE_AREA .'/settings/locations/localities/'.$this->uri->segment(5).'/'.$this->uri->segment(6).'/'.$this->uri->segment(7).'/'));
					} else if($this->uri->segment(5) && ($this->uri->segment(6))) {
						redirect(site_url(SITE_AREA .'/settings/locations/localities/'.$this->uri->segment(5).'/'.$this->uri->segment(6)));
					} else {
						redirect(site_url(SITE_AREA . '/settings/locations/localities/'.$this->uri->segment(5)));
					} 
				} else {
					Template::set_message(sprintf(lang('msg_edit_failure'), lang('label_locality')) . $this->localities_model->error, 'danger');
				}
			}
		}
		
		// Do we have any actions?
		$this->actions('localities');
		
		$where = array();
		// FILTER DATA
		if (preg_match('{first_letter-([A-Z])}', $filter, $matches)) {
			$filter_type = 'first_letter';
			$first_letter = $matches[1];
		} else {
			$filter_type = $filter;
		}
	
		switch($filter_type) {
			case 'active':
				$where['localities.active'] = 1;
				break;
					
			case 'inactive':
				$where['localities.active'] = 0;
				break;
			
			case 'first_letter':
				$where['SUBSTRING( LOWER(`' . $this->db->dbprefix . 'localities`.`name`), 1, 1)='] = $first_letter;
				break;
					
			case 'all':
				// Nothing to do
				break;
					
			default:
				show_404("cities/localities/$filter/");
		}
		//ADD NEW CITY
		$this->session->set_userdata('city_id', $city_id);
		
		// Sort By
		$like_array = array();
		$searchterm = $this->gc->search('localities', $this->uri->segment(6), '/settings/locations/localities/' . $city_id);
		if(!empty($searchterm)) {
			$like_array = array(
					'localities.id' => $searchterm,
					'localities.name' => $searchterm
			);
		}
		$order_by = $this->gc->sort('localities');
		if(!empty($order_by)) {
			$this->localities_model->order_by($this->session->userdata('localities_sort')['column'], $this->session->userdata('localities_sort')['order']);
		} else {
			$this->localities_model->order_by('name', 'ASC');
		}
		if(!empty($like_array)) {
			$this->db->or_like($like_array);
		}
		$this->localities_model->limit($this->limit, $offset)->where($where)->like('localities.name', $searchterm);
	
		if($city_id == 0) {	//USER ENTERED URL WITHOUT CITY ID
			$records = $this->localities_model->get_localities_city();
		}
		else {
			$records = $this->localities_model->get_localities_city($city_id);
		}
	
		Template::set('records', $records);
	
		// Pagination
		$this->load->library('pagination');
	
		if(!empty($like_array)) {
			$this->db->or_like($like_array);
		}
		$this->localities_model->where($where)->like('localities.name', $searchterm);
	
		if($city_id == 0) {
			$total_cities = $this->localities_model->count_all();
		}
		else {
			$total_cities = $this->localities_model->where('city_id', $city_id)->count_all();
		}
		$pager['base_url'] = site_url(SITE_AREA ."/settings/locations/localities/$city_id/$filter/");
		$pager['total_rows'] = $total_cities;
		$pager['per_page'] = $this->limit;
		$pager['uri_segment']	= 7;
	
		$this->pagination->initialize($pager);
		Template::set('localities_url', site_url(SITE_AREA .'/settings/locations/localities/'. $city_id .'/') .'/');
		Template::set('index_url', site_url(SITE_AREA .'/settings/locations/localities/' . $city_id . '/') .'/');
		Template::set('filter_type', $filter_type);
		Template::set('localities', $record);
			
		Template::set('toolbar_title', lang('label_manage_localities'));
		Template::render();
	}
		
	/**
	 * Save Locality Information
	 * @param string $type
	 * @param number $id
	 * @return number
	 */
	private function save_locality($type='insert', $id=0)
	{
		if ($type == 'update') {
			$_POST['id'] = $id;
		}
	
		$data = array();
		$data['name']        = $this->input->post('name');
		$data['city_id']	= $this->session->userdata('city_id');
			
		if ($type == 'insert') {
		    // get country_iso and state_id and save in the localities table for Export/Import
		    $city = $this->db->where('id', $data['city_id'])->get('cities')->row();
		    if($city) {
		        $data['country_iso'] = $city->country_iso;
		        $data['state_id'] = $city->state_id;
		    }
			$id = $this->localities_model->insert($data);
			if (is_numeric($id)) {
				$return = $id;
			} else {
				$return = FALSE;
			}
		} elseif ($type == 'update') {
			$return = $this->localities_model->update($id, $data);
		}
		return $return;
	}
	
	/**
	 * Change status of location from index page
	 * @param int $id
	 */
	public function update_status($type, $id, $filter = 'all', $offset = 0, $city_id = 0)
	{
		$this->auth->restrict($this->permissionEdit);
		if($type == 'countries') {
			$result = $this->db->select('active')->get_where($type, array('iso' => $id))->row();
		} else {
			$result = $this->db->select('active')->get_where($type, array('id' => $id))->row();
		}
		if($result) {
			$status = $result->active == 1 ? 0 : 1;
			$this->recursive($id, $type, $status);
		}
		if($type == 'countries') {
			if($offset != 0){
				redirect(site_url(SITE_AREA .'/settings/locations/index/'.$filter.'/'.$offset));
			} elseif ($filter != 'all') {
				redirect(site_url(SITE_AREA .'/settings/locations/index/' .$filter));
			} else {
				redirect(site_url(SITE_AREA .'/settings/locations'));
			}
		} elseif(($type == 'states') || ($type == 'cities')) {
			if($offset != 0){
				redirect(site_url(SITE_AREA .'/settings/locations/' . $type .'/'. $filter.'/'. $offset));
			} elseif ($filter != 'all') {
				redirect(site_url(SITE_AREA .'/settings/locations/' . $type .'/' . $filter));
			} else {
				redirect(site_url(SITE_AREA .'/settings/locations/' . $type ));
			}
		} elseif($type == 'localities') {
			if($offset != 0){
				redirect(site_url(SITE_AREA .'/settings/locations/' . $type .'/'. $city_id .'/'. $filter.'/'. $offset));
			} elseif ($filter != 'all') {
				redirect(site_url(SITE_AREA .'/settings/locations/' . $type .'/' . $city_id . '/'. $filter));
			} else {
				redirect(site_url(SITE_AREA .'/settings/locations/' . $type .'/' . $city_id ));
			}
		}
	}
	
	/**
	 * Update state status
	 */
	public function update_state($id, $country_iso, $filter = 'all', $offset = 0) {
		$result = $this->db->select('active')->get_where('states', array('id' => $id))->row();
		if($result) {
			$status = $result->active == 1 ? 0 : 1;
			$this->recursive($id, 'states', $status);
		}
		if($offset != 0){
			redirect(site_url(SITE_AREA .'/settings/locations/states/'. $country_iso .'/'. $filter.'/'. $offset));
		} elseif ($filter != 'all') {
			redirect(site_url(SITE_AREA .'/settings/locations/states/' . $country_iso .'/' . $filter));
		} else {
			redirect(site_url(SITE_AREA .'/settings/locations/states/' . $country_iso ));
		}		
	}
	
	/**
	 * Update city status
	 */
	public function update_city($id, $state_id, $filter = 'all', $offset = 0) {
		$result = $this->db->select('active')->get_where('cities', array('id' => $id))->row();
		if($result) {
			$status = $result->active == 1 ? 0 : 1;
			$this->recursive($id, 'cities', $status);
		}
		if($offset != 0){
			redirect(site_url(SITE_AREA .'/settings/locations/cities/'. $state_id .'/'. $filter.'/'. $offset));
		} elseif ($filter != 'all') {
			redirect(site_url(SITE_AREA .'/settings/locations/cities/' . $state_id .'/' . $filter));
		} else {
			redirect(site_url(SITE_AREA .'/settings/locations/cities/' . $state_id));
		}	
	}
	
	/**
	 * Update city status
	 */
	public function update_locality($id, $city_id, $filter = 'all', $offset = 0) {
		$result = $this->db->select('active')->get_where('localities', array('id' => $id))->row();
		if($result) {
			$status = $result->active == 1 ? 0 : 1;
			$this->recursive($id, 'localities', $status);
		}
		if($offset != 0){
			redirect(site_url(SITE_AREA .'/settings/locations/localities/'. $city_id .'/'. $filter.'/'. $offset));
		} elseif ($filter != 'all') {
			redirect(site_url(SITE_AREA .'/settings/locations/localities/' . $city_id .'/' . $filter));
		} else {
			redirect(site_url(SITE_AREA .'/settings/locations/localities/' . $city_id));
		}
	}
	
	/**
	 * Create Slug
	 */
	public function cities_slug($table_name = 'cities') {
		$this->load->library('gc');
		$this->db->or_where('slug', NULL);
		$this->db->or_where('slug', '');
		$records = $this->db->select('id, name')->get_where($this->db->dbprefix . $table_name)->result();
		if($records) {
			foreach($records as $record) {
				$data[] = array(
						'id' => $record->id,
						'slug' => $this->gc->slugify($table_name, $record->name, 0, 'name')
				);
			}
			$this->db->update_batch($table_name, $data, 'id');
		}
	}
	
	/**
	 * Common function to perform Delete/Activate/Deactivate operations
	 * @param string $location
	 */
	private function actions($location) {
		// Do we have any actions?
		if (isset($_POST['delete']))      $action = '_delete';
		if (isset($_POST['activate']))    $action = '_activate';
		if (isset($_POST['deactivate']))  $action = '_deactivate';
		
		if (isset($action)) {
			$checked = $this->input->post('checked');
			if (!empty($checked)) {
				if($location == 'cities') {
					// To delete information from the Ads module
					$this->load->model('ads/ads_model');
				}
				foreach($checked as $id) {
					if($location == 'cities') {						
						$this->ads_model->purge_from_ad($id, 'cities');
					}
					$this->$action($location, $id);
				}
				$singular = 'country';
				switch($location) {
					case 'countries':
						$singular = 'country';
						break;
					case 'states':
						$singular = 'state';
						break;
					case 'cities':
						$singular = 'city';
						break;
					case 'localities':
						$singular = 'locality';
						break;
				}
				switch($action) {
					case '_delete':
						Template::set_message(count($checked) .' '. sprintf(lang('msg_delete_success'), lang('label_' . (count($checked) == 1 ? $singular : $location))), 'success');
						break;
					case '_activate':
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), lang('label_' . (count($checked) == 1 ? $singular : $location)), lang('label_activated')), 'success');
						break;
					case '_deactivate':
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), lang('label_' . (count($checked) == 1 ? $singular : $location)), lang('label_deactivated')), 'success');
						break;
				}
				$this->redirect();
			} else {
				Template::set_message(lang('ls_empty_id'), 'error');
			}
		}
	}

	/**
	 * After status update redirect user to the appropriate page
	 */
	private function redirect() {
		if($this->uri->segment(4) && $this->uri->segment(5) && ($this->uri->segment(6)) && ($this->uri->segment(7))) {
			redirect(site_url(SITE_AREA . '/settings/locations/' . $this->uri->segment(4) . '/' . $this->uri->segment(5) . '/' .$this->uri->segment(6) . '/' . $this->uri->segment(7) . '/'));
		} else if($this->uri->segment(4) && $this->uri->segment(5) && ($this->uri->segment(6))) {
			redirect(site_url(SITE_AREA . '/settings/locations/' . $this->uri->segment(4) . '/' . $this->uri->segment(5) . '/' . $this->uri->segment(6) . '/'));
		} else if($this->uri->segment(4) && $this->uri->segment(5)) {
			redirect(site_url(SITE_AREA . '/settings/locations/' . $this->uri->segment(4) . '/'.$this->uri->segment(5)));
		} else {
			redirect(site_url(SITE_AREA . '/settings/locations/' . $this->uri->segment(4)));
		}
	}
	
	/**
	 * Delete selected location
	 */
	private function _delete($location, $id) {
		$this->recursive($id, $location, 0, true);
		//Template::set_message(sprintf(lang('msg_delete_success'), lang('label_' . $location)), 'success');
	}
	
	/**
	 * Activates selected location
	 */
	private function _activate($location, $id) {
		$this->recursive($id, $location, 1);
		//Template::set_message(sprintf(lang('msg_action_success'), lang('label_' . $location), lang('label_activated')), 'success');
	}
	
	/**
	 * Deactivates selected location.
	 */
	private function _deactivate($location, $id) {
		$this->recursive($id, $location, 0);
		//Template::set_message(sprintf(lang('msg_action_success'), lang('label_' . $location), lang('label_deactivated')), 'success');
	}
	
	/**
	 * Perform recursive operation
	 * @param array $checked
	 * @param int $status
	 * @param boolean $delete
	 */
	public function recursive($id, $location, $status, $delete = false) {
		if($location == 'countries') {
			// get all states under this country
			$states = $this->db->select('id')->get_where('states', array('country_iso' => $id))->result();
			if($states) {
				foreach($states as $state){
					// get all cities under this state
					$cities = $this->db->select('id')->get_where('cities', array('state_id' => $state->id))->result();
					// update all cities first
					if($cities) {
						// update localities first
						foreach($cities as $city) {
							if($delete) {
								$this->db->where('city_id', $city->id)->delete('localities');
							} else {
								$this->db->where('city_id', $city->id)->update('localities', array('active' => $status));
							}
						}
						// update cities
						if($delete) {
							$this->db->where('state_id', $state->id)->delete('cities');
						} else {
							$this->db->where('state_id', $state->id)->update('cities', array('active' => $status));
						}
					}
				}
				// update states
				if($delete) {
					$this->db->where('country_iso', $id)->delete('states');
				} else {
					$this->db->where('country_iso', $id)->update('states', array('active' => $status));
				}
			}
			if($delete) {
				$this->db->where('iso', $id)->delete('countries');
			} else {
				$this->db->where('iso', $id)->update('countries', array('active' => $status));
			}
		} else if($location == 'states') {
			// get all cities under this state
			$cities = $this->db->select('id')->get_where('cities', array('state_id' => $id))->result();
			// update all cities first
			if($cities) {
				// update localities first
				foreach($cities as $city) {
					if($delete) {
						$this->db->where('city_id', $city->id)->delete('localities');
					} else {
						$this->db->where('city_id', $city->id)->update('localities', array('active' => $status));
					}
				}
				// update cities
				if($delete) {
					$this->db->where('state_id', $id)->delete('cities');
				} else {
					$this->db->where('state_id', $id)->update('cities', array('active' => $status));
				}
			}
			// update states
			if($delete) {
				$this->db->where('id', $id)->delete('states');
			} else {
				$this->db->where('id', $id)->update('states', array('active' => $status));
			}
		} else if($location == 'cities') {
			if($delete) {
				// delete localities
				$this->db->where('city_id', $id)->delete('localities');
				// delete city
				$this->db->where('id', $id)->delete('cities');
			} else {
				// update localities
				$this->db->where('city_id', $id)->update('localities', array('active' => $status));
				// update city status
				$this->db->where('id', $id)->update('cities', array('active' => $status));
			}
		} else if($location == 'localities') {
			if($delete) {
				// delete locality
				$this->db->where('id', $id)->delete('localities');
			} else {
				// update locality status
				$this->db->where('id', $id)->update('localities', array('active' => $status));
			}
		}
	}
	
	/**
	 * Replace city id in case of city replacement
	 * $from - OLD CITY
	 * $new - NEW CITY
	 * Note:- country_iso is not included
	 */
	public function replace_cities($from, $to) {
		$existing_city = $this->db->select('state_id')->get_where('cities', array('id' => $from))->row();
		if($existing_city) {
			$new_city = $this->db->select('state_id')->get_where('cities', array('id' => $to))->row();
			if($new_city) {
				$data = array('state_id' => $new_city->state_id, 'city_id' => $to);
				// get all modules having locations
				$modules_having_cities = gc_module_key('location');
				if(!empty($modules_having_cities)) {
					foreach($modules_having_cities as $module => $value) {
						if($value == 1) {
							$this->db->where('city_id', $from)->update($module, $data);
						}
					}
				}
				$this->db->where('city_id', $from)->update('order_customers', array('city_id' => $to));
				$this->db->where('city_id', $from)->update('users', array('city_id' => $to));
				// Delete old city
				$this->db->where('id', $from)->delete('cities');
			}
		}		
	}
	
	/**
	 * Update Images in the location table
	 * @param string $table
	 */
	public function update_images($table = 'countries', $field = 'iso') {
	    $result = $this->db->get($table)->result();
	    $images_path = FCPATH . 'assets/images/locations/' . $table . '/';
	    if($result) {
    	    foreach($result as $row) {
    	        $file_name = $row->{$field} . '.jpg';
    	        // check file existence
    	        if(file_exists($images_path . $file_name)) {
    	            $this->db->where('id', $row->id)->update($table, array('image' => 'locations/' . $table . '/' . $file_name));
    	        }
    	    }
	    }
	}
	
	/**
	 * Export Locations database in MySQL format
	 * @param string $table_name - Name of the table to be exported
	 * @param string $field_name - Name of the column to be used in where condition
	 * @param string $field_value - Value of the column to be matched
	 *
	 * URL: http://localhost/mkt/public_html/ajax/locations?table=countries&field=iso&value=AF
	 */
	public function export($field_value = 'US', $table_name = 'countries', $field_name = 'iso', $download = true) {	    
	    // all parameters must be passed in GET
	    if(empty($table_name) && empty($field_name) && empty($field_value)) {
	        return false;
	    }
	    
	    // export mysql rows in a tmp_table
	    // this table will be used during import
	    $tmp_table_states = $this->db->dbprefix . "tmp_states";
	    $tmp_table_cities = $this->db->dbprefix . "tmp_cities";
	    $tmp_table_localities = $this->db->dbprefix . "tmp_localities";
	    
	    $this->load->dbutil();
	    $tables = array(
	        'tmp_states' => array('id', 'country_iso', 'name', 'slug', 'image', 'abbrev', 'latitude', 'longitude', 'active'),
	        'tmp_cities' => array('id', 'state_id', 'name', 'slug', 'image', 'description', 'abbrev', 'latitude', 'longitude', 'postcode', 'active'),
	        'tmp_localities' => array('id', 'city_id', 'name', 'slug', 'image', 'abbrev', 'latitude', 'longitude', 'postcode', 'active')
	    );
	    
	    // Drop temporary table
	    $this->drop_tmp_tables($tables);
	    
	    // NOTE: The TEMPORARY keyword is removed, the table_exists method of codeigniter is unable to work on temporary tables.
	    $where = '';
	    switch($table_name) {
	        case 'countries':
	            $column_name = 'country_iso';
	            break;
	        case 'states':
	            $column_name = 'state_id';
	            unset($tables['tmp_states']);
	            break;
	        case 'cities':
	            $column_name = 'city_id';
	            unset($tables['tmp_states']);
	            unset($tables['tmp_cities']);
	            break;
	    }
	    
	    $table_name = $table_name;
	    // find row id using the field name and field value e.g. country_iso = IN
	    $result = $this->db->where($field_name, $field_value)->get($table_name)->row();
	    if($result) {
	        // check whether we have corresponding record in the database tables
	        $where = $column_name . ' = ' . ($table_name = 'countries' ? '\'' . $result->iso . '\'' : $result->id);
	    }
	    
	    // Create Temporary Tables
	    $export_tables = array();
	    if(!empty($where)) {
	        foreach($tables as $tmp_table => $table_fields) {
	            $columns = implode(', ', $table_fields);
	            $db_table = str_replace('tmp_', '', $tmp_table);
	            $records_count = $this->db->where($where)->count_all_results($db_table);
	            if($records_count > 0) {
	                $this->db->query("CREATE TABLE " . $this->db->dbprefix . $tmp_table . " AS (SELECT " . $columns . " FROM " . $this->db->dbprefix . $db_table . " WHERE " . $where . ")");
	                // dump("CREATE TABLE " . $this->db->dbprefix . $tmp_table . " AS (SELECT " . $columns . " FROM " . $this->db->dbprefix . $db_table . " WHERE " . $where . ")");
	                $export_tables[] = $this->db->dbprefix . $tmp_table;
	            }
	        }
	    }
	    if(empty($export_tables)) {
	        return false;
	    }
	    $tmp_table_name = 'locations';
	    $prefs = array(
	        'tables'        => $export_tables,      // Array of tables to backup.
	        'ignore'        => array(),             // List of tables to omit from the backup
	        'format'        => 'txt',               // gzip, zip, txt
	        'filename'      => $tmp_table_name,         // File name - NEEDED ONLY WITH ZIP FILES
	        'add_drop'      => TRUE,                // Whether to add DROP TABLE statements to backup file
	        'add_insert'    => TRUE,                // Whether to add INSERT data to backup file
	        'newline'       => "\n"                 // Newline character used in backup file
	    );
	    
	    // Backup temporary table and assign it to a variable
	    $backup = $this->dbutil->backup($prefs);
	    
	    // Drop temporary table
	    $this->load->dbforge();
	    $this->dbforge->drop_table($tmp_table_name, TRUE); // DROP TABLE IF EXISTS tmp_table
	    
	    // Load the file helper and write the file to your server
	    $zip_path = FCPATH . 'uploads/exports/' ;
	    $destination_file = $zip_path . $field_value . '/public_html/uploads/imports/_locations.sql';
	    
	    // Check whether the destination path exist or not
	    // Copying locations must also create folders
	    $dir_path = dirname($destination_file);
	    // if path does not exist, create folders recursively
	    if (!file_exists($dir_path)) {
	        mkdir($dir_path, 0777, true);
	    }
	    
	    $this->load->helper('file');
	    write_file($destination_file, $backup);
	    
	    // Create an Archieve
	    $file_name = $field_value . '.zip';
	    create_zip($zip_path . $field_value, $zip_path . $file_name);
	    
	    // Drop temporary table
	    $this->drop_tmp_tables($tables);
	    
	    //  remove assets directory contents to avoid future references
	    unlink($destination_file);
	    if($download) {
	        $filename = $zip_path . '/' . $file_name;
	        if(file_exists($filename) && is_readable($filename) && file_exists($filename)){
	            header("Content-Disposition: attachment; filename=".basename(str_replace(' ', '_', $filename)));
	            header("Content-Type: application/force-download");
	            header("Content-Type: application/octet-stream");
	            header("Content-Type: application/download");
	            header("Content-Description: File Transfer");
	            header("Content-Length: " . filesize($filename));
	            flush(); // this doesn't really matter.
	            
	            $fp = fopen($filename, "r");
	            while (!feof($fp))
	            {
	                echo fread($fp, 65536);
	                flush(); // this is essential for large downloads
	            }
	            fclose($fp);
	            
	            // Unlink Backup File
	            unlink($filename);
	            
	            exit;
	        }
	        
	        // Unlink Backup File
	        unlink($filename);
	    }
	}
	
	/**
	 * Import Locations from GoClixy Server
	 */
	public function import() {
	    $this->lang->load('translate/translate');
	    $this->lang->load('migrate/migrate');
	    Assets::add_module_js('locations', 'locations.js');
	    //$table_name = 'countries'; // name of the table to be imported
	    //$field_name = 'iso'; // name of the column to be used in where condition
	    //$field_value = 'IN';  // value of the column to be matched
	    if(isset($_POST['truncate'])) { // Completely purge locations information before import
	        $this->db->truncate('states');
	        $this->db->truncate('cities');
	        $this->db->truncate('localities');
	        Template::set_message(lang('action_purged'), 'success');
	        Template::redirect(current_url());
	    }
	    if(isset($_POST['import'])) {
	        $this->form_validation->set_rules('import_from', 'lang:translate_import_from', 'trim|required');
	        if($this->input->post('import_from') == 0) {
	            $this->form_validation->set_rules('import_file_uploaded_file', 'lang:label_file', 'trim|required');
	            // Check whether a file is uploaded or not
	            if(isset($_FILES['import_file']) && $_FILES['import_file']['size'] > 0) {
	                $settings = array(
	                    'allowed_types' => 'zip',
	                    'max_size' => 10240,
	                    'upload_path' => realpath(FCPATH . '/uploads/')
	                );
	                $file_data = $this->gc->upload_file('import_file', $settings);
	                if(!$file_data) {
	                    $_POST['import_file_upload'] = 'NONE';
	                    Template::set_message($this->session->flashdata('upload_file_error'), 'danger');
	                    //$this->form_validation->set_rules('import_file_upload', 'label:label_file', 'valid_file_upload');
	                }
	                // Avoid file upload process, if the form has validation issues with other fields
	                $_POST['import_file_uploaded_file'] = isset($file_data['upload_data']['file_name']) ? $file_data['upload_data']['file_name'] : '';
	            }
	        }
	        if ($this->form_validation->run() === TRUE) {
	            $filename = $this->input->post('country_iso');
	            $file_name = $filename . '.zip';
	            if($this->input->post('import_from') == 0) {
	                $file_url = site_url('uploads/' . $_POST['import_file_uploaded_file']);
	            } else {
	                $file_url = 'https://www.goclixy.com/updates/locations/' . $file_name;
	            }
	            $zipFile = FCPATH . 'uploads/imports/' . $file_name;
	            $headers = getHeaders($file_url);
	            if ($headers['http_code'] === 200) {
	                if (curl_download($file_url, $zipFile)) {
	                    $delimiter = $filename;
	                    // Extract Zip File
	                    extract_zip($zipFile, $delimiter);
	                    
	                    // Tables to be imported
	                    $tables = array(
	                        'tmp_states' => array('id', 'country_iso', 'name', 'slug', 'image', 'abbrev', 'latitude', 'longitude', 'active'),
	                        'tmp_cities' => array('id', 'state_id', 'name', 'slug', 'image', 'description', 'abbrev', 'latitude', 'longitude', 'postcode', 'active'),
	                        'tmp_localities' => array('id', 'city_id', 'name', 'slug', 'image', 'abbrev', 'latitude', 'longitude', 'postcode', 'active')
	                    );
	                    
	                    // Make DB Changes
	                    // Drop temporary table
	                    $this->drop_tmp_tables($tables);
	                    
	                    // Hardcoded - File will be replaced on zip file extraction
	                    $file = FCPATH . 'uploads/imports/_locations.sql';
	                    
	                    // Insert Only
	                    $insert_only = $this->input->post('insert_or_update') == 1 ? true : false;
	                    
	                    // Check file existence
	                    if(!file_exists($file)) {
	                        return false;
	                    }
	                    
	                    // import large dump file
	                    // shell_exec('mysql -u ' . $this->db->username . ' -p' . $this->db->password . ' ' . $this->db->database . ' < "' . $file . '"');
	                    
	                    // Set line to collect lines that wrap
	                    $templine = '';
	                    
	                    // Read in entire file
	                    $lines = file($file);
	                    
	                    // Loop through each line
	                    foreach ($lines as $line) {
	                        
	                        // Skip it if it's a comment
	                        if (substr($line, 0, 2) == '--' || $line == '')
	                            continue;
	                            
	                            // Add this line to the current templine we are creating
	                            $templine .= $line;
	                            
	                            // If it has a semicolon at the end, it's the end of the query so can process this templine
	                            if (substr(trim($line), -1, 1) == ';') {
	                                // Perform the query
	                                $this->db->query($templine);
	                                
	                                // Reset temp variable to empty
	                                $templine = '';
	                            }
	                       }
	                    
	                    // Check table existence
	                    $this->load->library('themes/widgets');
	                    
	                    // For a very long list of columns you may see a truncated output, you may use this statement before the query below
	                    $this->db->query("SET SESSION group_concat_max_len = 1000000;");
	                    foreach($tables as $table => $table_fields) {
	                        $columns = implode(', ', $table_fields);
	                        if($this->widgets->is_table_exists($table)) {
	                            $sql = "INSERT";
	                            if($insert_only) {
	                                $sql .= " IGNORE";
	                            }
	                            $sql .= " INTO " . $this->db->dbprefix . str_replace('tmp_', '', $table) . " (" . $columns . ")";
	                            $sql .= " SELECT " . $columns . " FROM " . $this->db->dbprefix . $table;
	                            if(!$insert_only) {
	                                $duplicate_columns = array();
	                                foreach($table_fields as $table_column) {
	                                    $duplicate_columns[] = $table_column . '= VALUES(' . $table_column . ')';
	                                }
	                                $sql .= " ON DUPLICATE KEY UPDATE " . implode(',', $duplicate_columns);
	                            }
	                            $this->db->query($sql);
	                        }
	                    }
	                    
	                    // drop temporary created tables
	                    $this->drop_tmp_tables($tables);
	                    
	                    Template::set_message(sprintf(lang('msg_migrate_success'), lang('label_import')), 'success');
	                } else {
	                    Template::set_message(lang('msg_nothing_to_import'), 'danger');
	                }
	            } else {
	                Template::set_message(lang('msg_nothing_to_import'), 'danger');
	            }
	        }
	    }
	    Template::set(array(
	        'countries' => $this->db->select('iso, name, printable_name')->order_by('name')->get('countries')->result(),
	        'toolbar_title' => lang('bulk_import')
	    ));
	    Template::render();
	}
	
	/**
	 * Drop temporary table
	 * @param array $tables
	 */
	private function drop_tmp_tables($tables) {
	    // Drop temporary table
	    $this->load->dbforge();
	    foreach($tables as $table => $table_fields) {
	        $this->dbforge->drop_table($table, TRUE); // TRUE - DROP TABLE IF EXISTS
	    }
	}
	
	/**
	 * Update Cities, States and Countries counts
	 */
	public function counts() {
	    $this->load->model('listings/listings_model');
	    $this->listings_model->update_counts('localities');
	    $this->listings_model->update_counts('cities');
	    $this->listings_model->update_counts('states');
	    $this->listings_model->update_counts('countries');
	    Template::set_message(lang('message_count_success'), 'success');
	    Template::redirect(SITE_AREA .'/settings/locations');
	}
	
	/**
	 * Display module counts
	 */
	public function display_counts($row_id, $module = 'cities') {
	    $this->load->model('listings/listings_model');
	    $this->listings_model->display_counts($row_id, $module);
	}
	
	/**
	 * Update Country ISO in the cities table
	 * Update Country ISO and state_id in the localities table
	 * @param string $table - Name of the table to be updated
	 */
	public function update_locations($table = 'cities') {
	    $countries = $this->db->get('countries')->result();
	    $countries_iso = array();
	    if($countries) {
	        foreach($countries as $country) {
	            $countries_iso[$country->iso] = $country->iso;
	        }
	    }
	    
	    // update country_iso based on the states table
	    $states_iso = array();
	    if($table == 'cities') {
	        // get all states
	        $states = $this->db->get('states')->result();
	        if($states) {
	            foreach($states as $state) {
	                // $states_iso[$state->id] = $countries_iso[$state->country_iso];
	                // update cities table
	                $this->db->where(array('state_id' => $state->id))->update('cities', array('country_iso' => $state->country_iso));
	            }
	        }	        
	    } else if($table == 'localities') { // get all localities city ids
	        // search localities as they have small data
	        $city_ids = array();
	        $localities = $this->db->get('localities')->result();
	        if($localities) {
	            foreach($localities as $locality) {
	                $city_ids[] = $locality->city_id;
	            }
	            array_unique($city_ids);
	            // get cities information
	            $cities = $this->db->where('id IN (' . implode(',', $city_ids) . ')')->get('cities')->result();
	            if($cities) {
	                foreach($cities as $city) {
	                    // update cities table
	                    $this->db->where(array('city_id' => $city->id))->update('localities', array('country_iso' => $city->country_iso, 'state_id' => $city->state_id));
	                }
	            }	
	        }
	    }
	    // $this->load->library('slug');
	    /*$record = $this->db->where('slug IS NULL')->limit(10000)->get('cities')->result();
	    if($record) {
	        foreach($record as $row) {
	            $data = array(
	                // 'slug' => $this->slug->create_slug($row->name),
	                'country_iso' => $countries_iso[$row->country_id]
	            );
	            $this->db->where('id', $row->id)->update('cities_db', $data);
	        }
	    }*/
	}
	
	/**
	 * Import MaxMind Locations DB
	 */
	public function import_db() {
	    $this->lang->load('migrate/migrate');
	    if(isset($_POST['submit'])) {
	        gc_demo_mode();
	        $this->gcrud->stats(array('module' => 'locations'));
			$filename = 'locations';
	        $file_name = 'locations.zip';
	        $file_url = 'https://api.goclixy.com/uploads/exports/' . $file_name;
	        $zipFile = FCPATH . 'uploads/imports/' . $file_name;	
	        $headers = getHeaders($file_url);
	        if ($headers['http_code'] === 200) {
	            if (curl_download($file_url, $zipFile)) {
	                $delimiter = $filename;
					
	                // Extract Zip File
					$extractPath = FCPATH . 'uploads/imports/';
					$zip = new ZipArchive;
					/* Extract Zip File */
					if($zip->open($zipFile) != "true") {
						Template::set_message(lang('uvf_error_zip'), 'danger');
						Template::redirect(current_url());
					}
					$zip->extractTo($extractPath);
					$zip->close();
		
	                // extract_zip($zipFile, $delimiter);
					
	                // Hardcoded - File will be replaced on zip file extraction
	                $file = FCPATH . 'uploads/imports/locations.sql';
	                
	                // Check file existence
	                if(!file_exists($file)) {
	                    Template::set_message(sprintf(lang('msg_migrate_failure'), lang('label_import')), 'danger');
						Template::redirect(current_url());
	                }
	                
        	        // truncate table first
        	        $this->db->truncate('countries', true);
        	        $this->db->truncate('states', true);
        	        $this->db->truncate('cities', true);
        	        
        	        // import database
        	        $query = 'mysql -u ' . $this->db->username . ' -p' . $this->db->password . ' ' . $this->db->database . ' < ' . APPPATH . 'db/backups/locations.sql';
        	        $output = shell_exec($query);
					
        	        // Unlink zip file
					unlink($zipFile);
		
        	        // remove the extracted file
        	        if(file_exists($file)) {
        	            unlink($file);
        	        }
        	        Template::set_message(sprintf(lang('msg_migrate_success'), lang('label_import')), 'success');
        	        Template::redirect(current_url());
	            } else {					
					Template::set_message(sprintf(lang('msg_migrate_failure'), lang('label_import')), 'danger');
					Template::redirect(current_url());
				}
	        } else {
				Template::set_message(sprintf(lang('msg_migrate_failure'), lang('label_import')), 'danger');
				Template::redirect(current_url());
			}
	    }
	    Template::set(array(
	        'toolbar_title' => lang('bulk_import')
	    ));
	    Template::render();
	}
	
	public function change_status($module = 'cities', $value = 1) {
	    $this->db->update($module, array('status' => $value));
	    Template::set_message(lang('msg_success'), 'success');
	    Template::redirect(site_url());
	}
	
	/**
	 * Update State Name and Country ID in the cities table
	 */
	public function update_state_country() {
	    $sql = "UPDATE " . $this->db->dbprefix . "cities SET country_id = (SELECT id FROM " . $this->db->dbprefix . "countries WHERE iso = " . $this->db->dbprefix . "cities.country_iso), country = (SELECT printable_name FROM " . $this->db->dbprefix . "countries WHERE iso = " . $this->db->dbprefix . "cities.country_iso), state = (SELECT name FROM " . $this->db->dbprefix . "states WHERE id = " . $this->db->dbprefix . "cities.state_id)";
	    $this->db->query($sql);
	}
	
	/**
	 * Update State Names in the Cities Table
	 */
	public function update_states() {
	    $sql = "UPDATE " . $this->db->dbprefix . "cities INNER JOIN " . $this->db->dbprefix . "states ON " . $this->db->dbprefix . "states.id = " . $this->db->dbprefix . "cities.state_id SET " . $this->db->dbprefix . "cities.state = " . $this->db->dbprefix . "states.name";
	    $this->db->query($sql);
	}
	
	/**
	 * Update Latitude and Longitude
	 */
	public function update_lat_lng() {
	    $cities = $this->db->where('is_processed', 0)->limit(100)->get('cities')->result();
	    if($cities) {
	        foreach($cities as $city) {
	            $query = urlencode($city->name . ',' . $city->state . ',' . $city->country);
	            $url = 'https://nominatim.openstreetmap.org/search?q=' . $query . '&format=json&polygon_geojson=0&addressdetails=0';
	            $ch = curl_init();
	            $config['useragent'] = 'Mozilla/5.0 (Windows NT 6.2; WOW64; rv:17.0) Gecko/20100101 Firefox/17.0';
	            curl_setopt($ch, CURLOPT_USERAGENT, $config['useragent']);
	            curl_setopt($ch, CURLOPT_URL,$url);
	            curl_setopt($ch, CURLOPT_RETURNTRANSFER,1);
	            // curl_setopt($ch, CURLOPT_POSTFIELDS, $postfields);
	            $result = curl_exec($ch);
	            curl_close($ch);
	            if($result) {
	                $result = json_decode($result, true);
	                if(!empty($result[0]['lat'])) {
	                    $this->db->where('id', $city->id)->update('cities', array('latitude' => $result[0]['lat'], 'longitude' => $result[0]['lon'], 'is_processed' => 1));
	                } else {
	                    $this->db->where('id', $city->id)->update('cities', array('is_processed' => 1));
	                }
	            }
	        }
	    }
	}
	
	/**
	 * Get latitude and longitude from city name or a location
	 * @param string $location
	 * @return string[]
	 */
	public function get_latlng($location = 'Brussels, Belgium') {
	    $this->load->library('goclixy/gcache');
	    $url = 'https://geocode.xyz/' . urlencode($location) . '?json=1&auth=363355910172691265565x81267';
	    $result = $this->gcache->curl_data(null, null, $url, true);
	    $output = array();
	    if(!empty($result['latt']) && !empty($result['longt'])) {
	        $output['longitude'] = $result['longt'];
	        $output['latitude'] = $result['latt'];
	        $output['result'] = $result['latt'] . ',' . $result['longt'];
	    }
	    return $output;
	}
	
	/**
	 * Manage Locations
	 * @param string $module
	 * @param string $filter
	 * @param number $offset
	 */
	public function manage($module = 'countries', $filter = 'all', $offset = 0) {
	    $extra = array(
	        'model' => $module,
	        'context' => 'settings',
	        'fields' => null,
	        'search_fields' => array('name')
	    );
	    $extra['actions'] = array( // Extra buttons
	        array('type' => 'link', 'link' => site_url(SITE_AREA . '/settings/locations/form/' . $module . '/edit/%s'), 'label' => '<i class="fas fa-edit fa-fw mr-1"></i>' . lang('edit'), 'class' => 'btn btn-xs btn-primary'),
	    );
	    $this->gc->module_view($module, $filter, $offset, 'admin', $extra);
	}
	
	/**
	 * Add/Edit Locations Data
	 *
	 * @param string $module
	 * @param string $action
	 *
	 * @return void
	 */
	public function form($module = 'countries', $action = 'create') {
	    $params = array(
	        'model' => $module,
	        'context' => 'settings',
	        'redirect_url' => site_url(SITE_AREA . '/settings/locations/manage/' . $module)
	    );
	    $this->gcrud->form($module, $action, 'admin', $params);
	}
}