<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Migration_Install_locations extends Migration
{
	/**
	 * The name of the database table
	 *
	 * @var String
	 */	
	private $country_table = 'countries';
	private $state_table = 'states';
	private $city_table = 'cities';
	private $locality_table = 'localities';

    /**
     * The Country table's fields
     *
     * @var Array
     */
    private $country_fields = array(
        'id' => array(
            'type' => 'SMALLINT',
            'constraint' => 5,
            'auto_increment' => TRUE
        ),
        'iso' => array(
            'type' => 'CHAR',
            'constraint' => 2,
            'null' => FALSE,
            'default' => 'US',
            'unique' => TRUE
        ),
        'name' => array(
            'type' => 'VARCHAR',
            'constraint' => 80,
            'null' => FALSE
        ),
        'slug' => array(
            'type' => 'VARCHAR',
            'constraint' => 255,
            'null' => TRUE
        ),
        'image' => array(
            'type' => 'VARCHAR',
            'constraint' => 255,
            'null' => TRUE
        ),
        'printable_name' => array(
            'type' => 'VARCHAR',
            'constraint' => 80,
            'null' => FALSE
        ),
        'iso3' => array(
            'type' => 'CHAR',
            'constraint' => 3,
            'null' => TRUE
        ),
        'numcode' => array(
            'type' => 'SMALLINT',
            'constraint' => 6,
            'null' => TRUE
        ),
        'latitude' => array(
            'type' => 'VARCHAR',
            'constraint' => 30,
            'null' => TRUE
        ),
        'longitude' => array(
            'type' => 'VARCHAR',
            'constraint' => 30,
            'null' => TRUE
        ),
        'active' => array(
            'type' => 'TINYINT',
            'constraint' => 1,
            'null' => FALSE,
            'default' => 1
        )
    );

    /**
     * The State table's fields
     *
     * @var Array
     */
    private $state_fields = array(
        'id' => array(
            'type' => 'INT',
            'constraint' => 11,
            'auto_increment' => TRUE
        ),
        'country_iso' => array(
            'type' => 'VARCHAR',
            'constraint' => 2,
            'null' => FALSE
        ),
        'name' => array(
            'type' => 'CHAR',
            'constraint' => 40,
            'null' => FALSE
        ),
        'slug' => array(
            'type' => 'VARCHAR',
            'constraint' => 255,
            'null' => TRUE
        ),
        'image' => array(
            'type' => 'VARCHAR',
            'constraint' => 255,
            'null' => FALSE
        ),
        'abbrev' => array(
            'type' => 'CHAR',
            'constraint' => 3,
            'null' => FALSE
        ),
        'latitude' => array(
            'type' => 'VARCHAR',
            'constraint' => 30,
            'null' => TRUE
        ),
        'longitude' => array(
            'type' => 'VARCHAR',
            'constraint' => 30,
            'null' => TRUE
        ),
        'active' => array(
            'type' => 'TINYINT',
            'constraint' => 1,
            'null' => FALSE,
            'default' => 1
        )
    );

    /**
     * The City table's fields
     *
     * @var Array
     */
    private $city_fields = array(
        'id' => array(
            'type' => 'INT',
            'constraint' => 11,
            'auto_increment' => TRUE
        ),
        'country_iso' => array(
            'type' => 'CHAR',
            'constraint' => 2,
            'null' => FALSE
        ),
        'state_id' => array(
            'type' => 'INT',
            'constraint' => 11,
            'null' => FALSE
        ),
        'name' => array(
            'type' => 'VARCHAR',
            'constraint' => 40,
            'null' => FALSE
        ),
        'slug' => array(
            'type' => 'VARCHAR',
            'constraint' => 255,
            'null' => TRUE
        ),
        'image' => array(
            'type' => 'VARCHAR',
            'constraint' => 255,
            'null' => TRUE
        ),
        'description' => array(
            'type' => 'TEXT',
            'null' => TRUE
        ),
        'abbrev' => array(
            'type' => 'CHAR',
            'constraint' => 3,
            'null' => FALSE
        ),
        'latitude' => array(
            'type' => 'VARCHAR',
            'constraint' => 30,
            'null' => TRUE
        ),
        'longitude' => array(
            'type' => 'VARCHAR',
            'constraint' => 30,
            'null' => TRUE
        ),
        'postcode' => array(
            'type' => 'VARCHAR',
            'constraint' => 10,
            'null' => TRUE
        ),
        'hits' => array(
            'type' => 'INT',
            'constraint' => 11,
            'null' => FALSE,
            'default' => 0
        ),
        'active' => array(
            'type' => 'TINYINT',
            'constraint' => 1,
            'null' => FALSE,
            'default' => 1
        ),
        'create_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
        'created_ip' => array(
            'type' => 'VARCHAR',
            'constraint' => 50,
            'null' => TRUE
        ),
        'modified_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
        'modified_ip' => array(
            'type' => 'VARCHAR',
            'constraint' => 50,
            'null' => TRUE
        )
    );

    /**
     * The Locality table's fields
     *
     * @var Array
     */
    private $locality_fields = array(
        'id' => array(
            'type' => 'INT',
            'constraint' => 11,
            'auto_increment' => TRUE
        ),
        'country_iso' => array(
            'type' => 'CHAR',
            'constraint' => 2,
            'null' => FALSE
        ),
        'state_id' => array(
            'type' => 'INT',
            'constraint' => 11,
            'null' => FALSE
        ),
        'city_id' => array(
            'type' => 'INT',
            'constraint' => 11,
            'null' => FALSE
        ),
        'name' => array(
            'type' => 'VARCHAR',
            'constraint' => 40,
            'null' => FALSE
        ),
        'slug' => array(
            'type' => 'VARCHAR',
            'constraint' => 255,
            'null' => FALSE
        ),
        'image' => array(
            'type' => 'VARCHAR',
            'constraint' => 255,
            'null' => TRUE
        ),
        'latitude' => array(
            'type' => 'VARCHAR',
            'constraint' => 30,
            'null' => TRUE
        ),
        'longitude' => array(
            'type' => 'VARCHAR',
            'constraint' => 30,
            'null' => TRUE
        ),
        'postcode' => array(
            'type' => 'VARCHAR',
            'constraint' => 10,
            'null' => TRUE
        ),
        'active' => array(
            'type' => 'TINYINT',
            'constraint' => 1,
            'null' => FALSE,
            'default' => 1
        )
    );



	/**
	 * Install this migration
	 *
	 * @return void
	*/
	public function up()
	{
		// COUNTRIES TABLE
		$this->dbforge->add_field($this->country_fields);
		$this->dbforge->add_key('id', true);
		// $this->dbforge->add_key('iso', false);
		$this->dbforge->create_table($this->country_table);
		// CHANGE ENGINE FROM INNODB TO MYISAM
		$change_country_engine_sql = "ALTER TABLE " . $this->db->dbprefix . $this->country_table . " ENGINE = MyISAM";
		$this->db->query ($change_country_engine_sql);
		
		// STATES TABLE
		$this->dbforge->add_field($this->state_fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('country_iso',false);
		$this->dbforge->create_table($this->state_table);
		$change_state_engine_sql = "ALTER TABLE " . $this->db->dbprefix . $this->state_table . " ENGINE = MyISAM";
		$this->db->query ($change_state_engine_sql);
		// ADD FULLTEXT INDEX
		$state_fulltext_sql = "ALTER TABLE " . $this->db->dbprefix . $this->state_table . " ADD FULLTEXT INDEX `name` (`name`)";
		$this->db->query ($state_fulltext_sql);
		// SET SLUG FIELD COLLATION TO utf8_bin
		$state_collation_sql = "ALTER TABLE " . $this->db->dbprefix . $this->state_table . " CHANGE `slug` `slug` VARCHAR( 255 ) CHARACTER SET utf8 COLLATE utf8_bin NULL DEFAULT NULL";
		$this->db->query ($state_collation_sql);

		// CITIES TABLE
		$this->dbforge->add_field($this->city_fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('state_id',false);
		$this->dbforge->create_table($this->city_table);
		$change_city_engine_sql = "ALTER TABLE " . $this->db->dbprefix . $this->city_table . " ENGINE = MyISAM";
		$this->db->query ($change_city_engine_sql);
		$city_fulltext_sql = "ALTER TABLE " . $this->db->dbprefix . $this->city_table . " ADD FULLTEXT INDEX `name` (`name`)";
		$this->db->query ($city_fulltext_sql);
		$city_collation_sql = "ALTER TABLE " . $this->db->dbprefix . $this->city_table . " CHANGE `slug` `slug` VARCHAR( 255 ) CHARACTER SET utf8 COLLATE utf8_bin NULL DEFAULT NULL";
		$this->db->query ($city_collation_sql);
		
		// LOCALTIES TABLE
		$this->dbforge->add_field($this->locality_fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('city_id',false);
		$this->dbforge->create_table($this->locality_table);
		$change_locality_engine_sql = "ALTER TABLE " . $this->db->dbprefix . $this->locality_table . " ENGINE = MyISAM";
		$this->db->query ($change_locality_engine_sql);
		$locality_fulltext_sql = "ALTER TABLE " . $this->db->dbprefix . $this->locality_table . " ADD FULLTEXT INDEX `name` (`name`)";
		$this->db->query ($locality_fulltext_sql);
	}

	//--------------------------------------------------------------------

	/**
	 * Uninstall this migration
	 *
	 * @return void
	 */
	public function down()
	{
		$this->dbforge->drop_table($this->country_table);
		$this->dbforge->drop_table($this->state_table);
		$this->dbforge->drop_table($this->city_table);
		$this->dbforge->drop_table($this->locality_table);
	}

	//--------------------------------------------------------------------

}