<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
 || ##################################################################### ||
 || # GoClixy - Business Directory Script							   # ||
 || # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
 || # ----------------------------------------------------------------  # ||
 || # This file is owned and copyrighted by GoClixy.com. Your license   # ||
 || # confers  no  title  or ownership in the Script and is not a sale  # ||
 || # of any rights to the Software. This file may not be redistributed # ||
 || # in whole or significant part.									   # ||
 || # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
 || # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
 || ##################################################################### ||
 \*=======================================================================*/
class Cities_model extends BF_Model {
    
    protected $table_name	= "cities";
    protected $key			= "id";
    protected $soft_deletes	= false;
    protected $date_format	= "datetime";
    
    protected $log_user 	= FALSE;
    
    protected $set_created	= TRUE;
    protected $set_modified = TRUE;
    protected $created_field     = 'created_on';
    protected $modified_field    = 'modified_on';
    
    /*
     Customize the operations of the model without recreating the insert, update,
     etc methods by adding the method names to act as callbacks here.
     */
    protected $before_insert 	= array();
    protected $after_insert 	= array();
    protected $before_update 	= array();
    protected $after_update 	= array();
    protected $before_find 		= array();
    protected $after_find 		= array();
    protected $before_delete 	= array();
    protected $after_delete 	= array();
    
    /*
     For performance reasons, you may require your model to NOT return the
     id of the last inserted row as it is a bit of a slow method. This is
     primarily helpful when running big loops over data.
     */
    protected $return_insert_id 	= TRUE;
    
    // The default type of element data is returned as.
    protected $return_type 			= "object";
    
    // Items that are always removed from data arrays prior to
    // any inserts or updates.
    protected $protected_attributes = array();
    
    /*
     You may need to move certain rules (like required) into the
     $insert_validation_rules array and out of the standard validation array.
     That way it is only required during inserts, not updates which may only
     be updating a portion of the data.
     */
    protected $validation_rules 		= array(
        array(
            "field"		=> "name",
            "label"		=> "lang:label_name",
            "rules"		=> "required|max_length[40]"
        ),
        array(
            "field"		=> "hits",
            "label"		=> "lang:label_hits",
            "rules"		=> "trim|required|is_numeric"
        ),
        array(
            "field"		=> "abbrev",
            "label"		=> "lang:label_code",
            "rules"		=> "max_length[3]"
        ),
    );
    protected $insert_validation_rules 	= array();
    protected $skip_validation 			= FALSE;
    protected $gc_cities_cache;
    
    //--------------------------------------------------------------------
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
        // Load Zend Caching
        $this->load->library('slug');
        $this->load->library('gzf_cache', array('lifetime' => 900));
        if(method_exists($this->gzf_cache, 'get_instance')) {
            $this->gc_cities_cache = $this->gzf_cache->get_instance();
        }
    }
    
    /**
     * Cities Fields
     *
     * @return array
     */
    public function fields() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'text',
                'name' => 'country_id',
                'label' => 'label_country_id',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'country_iso',
                'label' => 'label_country_iso',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'country',
                'label' => 'label_country',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim|max_length[100]',
            ),
            array(
                'type' => 'text',
                'name' => 'state_id',
                'label' => 'label_state_id',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'state',
                'label' => 'label_state',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|max_length[100]',
            ),
            array(
                'type' => 'text',
                'name' => 'name',
                'label' => 'label_name',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|max_length[80]',
            ),
            array(
                'type' => 'text',
                'name' => 'slug',
                'label' => 'label_slug',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim|max_length[255]',
            ),
            array(
                'type' => 'text',
                'name' => 'image',
                'label' => 'label_image',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|max_length[255]',
            ),
            array(
                'type' => 'text',
                'name' => 'description',
                'label' => 'label_description',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'abbrev',
                'label' => 'label_abbrev',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'latitude',
                'label' => 'label_latitude',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim|max_length[30]',
            ),
            array(
                'type' => 'text',
                'name' => 'longitude',
                'label' => 'label_longitude',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim|max_length[30]',
            ),
            array(
                'type' => 'text',
                'name' => 'postcode',
                'label' => 'label_postcode',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim|max_length[10]',
            ),
            array(
                'type' => 'text',
                'name' => 'counts',
                'label' => 'label_counts',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'hits',
                'label' => 'label_hits',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'active',
                'label' => 'label_status',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_ip',
                'label' => 'created_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_ip',
                'label' => 'modified_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'is_processed',
                'label' => 'label_is_processed',
                'frontend' => false,
                'admin_only' => false,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            
        );
        return $fields;
    }
    
    /**
     * Get all states
     * @return Object
     */
    public function get_states(){
        $this->db->select('id, name');
        return $this->db->get($this->db->dbprefix . 'states')->result();
    }
    
    /**
     * Display state code on city page
     * @param number $state_id
     */
    public function get_cities_state($state_id = 0) {
        $this->db->select('cities.*, states.abbrev as states_abbrev');
        $this->db->join($this->db->dbprefix . 'states',$this->db->dbprefix . 'states.id = ' . $this->db->dbprefix . 'cities.state_id', 'left');
        //USER ENTERED URL WITHOUT STATE ID
        if($state_id != 0) {
            $this->db->where('states.id', $state_id);
        }
        $this->db->order_by('cities.name', 'asc');
        return $this->cities_model->find_all();
    }
    
    /**
     * Frontend search function to get city and locality ID
     * @param string $city
     * @param number $locality_id
     */
    public function get_city_locality($city, $locality_id) {
        $this->db->select($this->db->dbprefix . 'cities.id as city_id, ' . $this->db->dbprefix . 'cities.name as city_name, ' . $this->db->dbprefix . 'localities.id as locality_id');
        $this->db->join($this->db->dbprefix . 'localities', $this->db->dbprefix . 'cities.id = ' . $this->db->dbprefix . 'localities.city_id', 'INNER');
        return $this->cities_model->find_by(array($this->db->dbprefix . 'cities.name' => $city, $this->db->dbprefix . 'localities.id' => $locality_id));
    }
    
    /**
     * Get all cities when admin disables Country and State selection from settings
     */
    public function get_all_cities($country_iso) {
        $this->db->select($this->db->dbprefix . 'cities.id, ' . $this->db->dbprefix . 'cities.name');
        $this->db->join($this->db->dbprefix . 'states', $this->db->dbprefix . 'states.id = ' . $this->db->dbprefix . 'cities.state_id', 'INNER');
        $this->db->where(array($this->db->dbprefix . 'cities.active' => 1, $this->db->dbprefix . 'states.country_iso' => $country_iso));
        return $this->cities_model->find_all();
    }
    
    /**
     * Cities Selection
     */
    public function ajax_cities_search($query, $limit = 10) {
        // affiliates search
        $state = null;
        $country = null;
        if(strpos($query, ',') !== false) {
            $query = array_map('trim', explode(',', $query));
            if(!empty($query[1])) {
                $state = escape($query[1]);
            }
            if(!empty($query[2])) {
                $country = escape($query[2]);
            }
            $query = escape($query[0]);
        }
        $sql = "SELECT CT.id, CONCAT(CT.name, ', ', CT.state, ', ', CT.country_iso) AS name, COUNT(L.id) hasLocalities FROM " . $this->db->dbprefix . "cities CT";
        // $sql .= " INNER JOIN " . $this->db->dbprefix . "states S ON CT.state_id = S.id AND S.active = 1";
        // $sql .= " INNER JOIN " . $this->db->dbprefix . "countries C ON C.iso = S.country_iso AND C.active = 1";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "localities L ON L.city_id = CT.id AND L.active = 1";
        $sql .= " WHERE CT.active = 1";
        if(!empty($state)) {
            $sql .= " AND CT.state = '" . $state . "'";
        }
        if(!empty($country)) {
            $sql .= " AND CT.country_iso = '" . $country . "'";
        }
        $sql .= " AND CT.name LIKE '%". $query ."%'";
        if(settings_item('lst.allow_country_selection') == 0) {
            $sql .= " AND S.country_iso = '" . settings_item('adv.default_country') . "'";
        }
        $sql .= " GROUP BY CT.id";
        $sql .= " ORDER BY CT.name ASC LIMIT " . $limit;
        return $this->db->query($sql)->result();
    }
    
    /**
     * Get City Info
     * @param number $city_id
     * @return boolean
     */
    public function city_info($city_id) {
        $cacheID = 'cities_' . $city_id;
        if(!$record = $this->gc_cities_cache->load($cacheID)) {
            $record = $this->get_city_info($city_id);
            $this->gc_cities_cache->save($record, $cacheID, array('cities'));
        }
        return $record;
    }
    
    /**
     * OnTypeCitySelect - Get State ID and Country ISO from City ID
     * Common function for Save and Edit
     * @param $city_id
     */
    public function get_city_info($city_id) {
        if($city_id != 0) {
            $sql = "SELECT CT.id, CONCAT(CT.name, ', ', S.name, ', ', S.country_iso) AS name, CT.name AS city, S.id AS state_id, S.name AS state, S.country_iso AS country_iso, C.name as country, COUNT(L.id) hasLocalities, CT.latitude, CT.longitude FROM " . $this->db->dbprefix . "cities CT";
            $sql .= " INNER JOIN " . $this->db->dbprefix . "states S ON CT.state_id = S.id";
            $sql .= " INNER JOIN " . $this->db->dbprefix . "countries C ON C.iso = S.country_iso";
            $sql .= " LEFT JOIN " . $this->db->dbprefix . "localities L ON L.city_id = CT.id AND L.active = 1";
            $sql .= " WHERE CT.id = $city_id";
            return $this->db->query($sql)->row();
        } else {
            return false;
        }
    }
    
    /**
     * Update Country, State and City information in the modules table to fasten search queries
     * @param string $module
     * @param integer $row_id
     * @param boolean $is_update - TRUE - Update instantly, FALSE - Return value for Bulk Tweaking of Multiple Rows
     */
    public function _tweak_location_row($module, $row_id, $is_update = true) {
        $record = $this->db->get_where($module, array('id' => $row_id))->row();
        if($record) {
            if(property_exists ($record, 'city') && !empty($record->city_id)) {
                $location = $this->get_city_info($record->city_id);
                if($location) {
                    $data = array();
                    $data['city'] = $location->city;
                    if(property_exists ($record, 'state')) {
                        $data['state'] = $location->state;
                    }
                    if(property_exists ($record, 'country')) {
                        $data['country'] = $location->country;
                    }
                    if(property_exists ($record, 'locality') && !empty($record->locality_id)) {
                        // Get locality information
                        $locality = $this->db->get_where('localities', array('id' => $record->locality_id))->row();
                        if($locality) {
                            $data['locality'] = $locality->name;
                        }
                    }
                    if($is_update) {
                        // $location = $record->address . ', ' . (!empty($data['locality']) ? $data['locality'] . ', ' : '') . $data['city'] . ', ' . $data['state'] . ' - ' . $record->postcode . ', ' . $data['country']
                        $this->db->where(array('id' => $row_id))->update($module, $data);
                        $this->gc->_tweak_location_search($module, 'id = ' . $row_id); // update location column
                    } else {
                        return $data;
                    }
                }
            }
        }
    }
    
    /**
     * Get Country, State and City information for listing_form and checkout process
     * if Google Auto Places API is Turned ON
     * @param string $city
     * @param string $state_abbrev
     * @param string $state
     * @param string $country_iso
     * @param string $country
     * @return multitype:unknown Ambigous <unknown, unknown> |boolean
     */
    public function getCityStateCountry($city_name, $state_abbrev, $state_name, $country_iso, $country_name) {
        // Country having no states e.g. Bangkok, Thailand
        // Use City Name as the State Name
        if(!empty($city_name) && empty($state_name) && (!empty($country_name) || !empty($country_iso))) {
            $state_name = $city_name;
        }
        if(empty($city_name) && !empty($state_name) && (!empty($country_name) || !empty($country_iso))) {
            $city_name = $state_name;
        }
        if(!empty($city_name) && !empty($state_name) && (!empty($country_name) || !empty($country_iso))) {
            $this->load->model('locations/cities_model');
            $this->load->model('locations/states_model');
            $this->load->model('locations/countries_model');
            $city_id = 0;
            $state_id = 0;
            $country_iso = $country_iso;
            $country = $this->countries_model->find_by('iso', $country_iso);
            if($country) {
                $country_iso = $country->iso;
                $state = $this->states_model->find_by(array('name' => $state_name, 'country_iso' => $country->iso));
                if($state) {
                    $state_id = $state->id;
                    $city = $this->cities_model->select('id')->find_by(array('name' => $city_name, 'state_id' => $state_id));
                    if($city) {
                        $city_id = $city->id;
                    } else { // new city
                        $city_id = $this->add_city($country, $state, $city_name);
                    }
                } else { // new state
                    $state = $this->add_state($country, $state_abbrev, $state_name, $city_name);
                    $state_id = $state['state_id'];
                    $city_id = $state['city_id'];
                }
            } else { // new country
                $location = $this->add_country($country_iso, $country_name, $state_abbrev, $state_name, $city_name);
                $country_iso = $location['country_iso'];
                $state_id = $location['state_id'];
                $city_id = $location['city_id'];
            }
            $result = array('country_iso' => $country_iso, 'state_id' => $state_id, 'city_id' => $city_id);
            return $result;
        } else {
            return false;
        }
    }
    
    /**
     * Add New City information
     * @param int $state_id
     * @param string $city
     * @return int city_id|number
     */
    private function add_city($country, $state, $city) {
        $this->load->library('slug');
        $city_data = array(
            'country_id' => $country->id,
            'country_iso' => $country->iso,
            'country' => $country->printable_name,
            'state_id' => $state->id,
            'state' => $state->name,
            'name' => $city,
            'slug' => $this->slug->create_slug($city),
            'abbrev' => strtoupper(substr($city, 0, 3)), // Get first three characters from the City name and use them as as abbreviation
            'hits' => 0,
            'created_on' => date('Y-m-d H:i:s'),
            'created_ip' => $this->input->ip_address()
        );
        $this->db->insert('cities', $city_data);
        $city_id = $this->db->insert_id();
        if (is_numeric($city_id)) {
            return $city_id;
        } else {
            return 0;
        }
    }
    
    /**
     * Add State and City information
     * @param string $country_iso
     * @param string $state_abbrev
     * @param string $state
     * @param string $city
     * @return multitype:Ambigous <city_id, number> unknown |multitype:number
     */
    private function add_state($country, $state_abbrev, $state, $city) {
        $this->load->library('slug');
        $state_data = array(
            'country_id' => $country->id,
            'country_iso' => $country->iso,
            // 'country' => $country->name,
            'name' => $state,
            'slug' => $this->slug->create_slug($state),
            'abbrev' => strlen($state_abbrev) <= 3 ? $state_abbrev : strtoupper(substr($state, 0, 3))
        );
        $this->db->insert('states', $state_data);
        $state_id = $this->db->insert_id();
        if (is_numeric($state_id)) {
            $state = $this->gc->get('states', array('where' => 'id = ' . $state_id));
            $city_id = $this->add_city($country, $state, $city);
            return array('state_id' => $state_id, 'city_id' => $city_id);
        } else {
            return array('state_id' => 0, 'city_id' => 0);
        }
    }
    
    /**
     * Add Country, State and City information
     * @param string $country_iso
     * @param string $country
     * @param string $state_abbrev
     * @param string $state
     * @param string $city
     * @return multitype:unknown Ambigous <Ambigous> |multitype:number unknown
     */
    private function add_country($country_iso, $country, $state_abbrev, $state, $city) {
        $this->load->library('slug');
        $country_data = array(
            'iso' => $country_iso,
            'name' => $country,
            'printable_name' => $country
        );
        $this->db->insert('countries', $country_data);
        // insert_id returns 0 but it should be a char value,
        // therefore, we need to check whether a country is inserted or not
        $this->load->model('locations/countries_model');
        $country_result = $this->countries_model->select('iso')->find_by('iso', $country_iso);
        if ($country_result && $country_result->iso) {
            $location = $this->add_state($country_result, $state_abbrev, $state, $city);
            if($location['state_id'] != 0 && $location['city_id'] != 0) {
                return array('country_iso' => $country_iso, 'state_id' => $location['state_id'], 'city_id' => $location['city_id']);
            }
        } else {
            return array('country_iso' => $country_iso, 'state_id' => 0, 'city_id' => 0);
        }
    }
    
    /**
     * Ajax Load Locations - Country, State and City dropdown
     * @param string $table - Name of the Table
     * @param string $name - Name of the column
     */
    public function load($table, $parent_id = null) {
        switch($table) {
            case 'states':
            case 'cities':
            case 'localities':
                $column = $table == 'states' ? 'country_iso' : ($table == 'cities' ? 'state_id' : 'city_id');
                return $this->db->select('id, name')->where($column, $parent_id)->get($table)->result();
                break;
        }
        return false;
    }
    
    /**
     * Display countries to the user
     *
     * @return array countries iso and name
     */
    public function get_countries() {
        return $this->db->select('iso, name')
        ->order_by('name', 'asc')
        ->get('countries')->result();
    }
    
    /**
     * Display states information to the user based on country_iso
     *
     * @param string $country_iso
     */
    public function get_country_states($country_iso = NULL) {
        if ($country_iso != NULL) {
            return $this->db->select('id, name')
            ->where('country_iso', $country_iso)
            ->order_by('name', 'asc')
            ->get('states')->result();
        } else {
            return;
        }
    }
    
    /**
     * Display cities to the user based on State ID
     *
     * @param int $state_id
     */
    public function get_state_cities($state_id = NULL) {
        if ($state_id != NULL) {
            return $this->db->select('id, name')
            ->where('state_id', $state_id)
            ->order_by('name', 'asc')
            ->get('cities')->result();
        } else {
            return;
        }
    }
    
    /**
     * Get localities information based on City ID
     *
     * @param int $city_id
     */
    public function get_localities($city_id = NULL) {
        if ($city_id != NULL) {
            return $this->db->select('id, name')
            ->where('city_id', $city_id)
            ->order_by('name', 'asc')
            ->get('localities')->result();
        } else {
            return;
        }
    }
    
    /**
     * Import Cities retrieved from api.goclixy.com
     * @param string $cities - Comma separated values returned from the selectize input field
     */
    public function gc_cities($cities) {
        if(!empty($cities)) {
            $cities = $this->db->where('id IN (' . $cities . ')')->get('gc_cities')->result();
            // $cities = explode(',', $cities);
            $data = array();
            if(!empty($cities)) {
                foreach($cities as $city) {
                    $data[] = array(
                        'id' => $city->id,
                        'country_iso' => $city->country_iso,
                        'state_id' => $city->state_id,
                        'name' => $city->name,
                        'slug' => $city->slug,
                        'latitude' => $city->latitude,
                        'longitude' => $city->longitude,
                        'active' => 1
                    );
                }
                if(!empty($data)) {
                    $this->db->on_duplicate('cities', $data);
                }
            }
        }
    }
}