<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class Countries_model extends BF_Model {

	protected $table_name	= "countries";
	protected $key			= "iso";
	protected $soft_deletes	= false;
	protected $date_format	= "datetime";

	protected $log_user 	= FALSE;

	protected $set_created	= false;
	protected $set_modified = false;

	/*
		Customize the operations of the model without recreating the insert, update,
		etc methods by adding the method names to act as callbacks here.
	 */
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 		= array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	/*
		For performance reasons, you may require your model to NOT return the
		id of the last inserted row as it is a bit of a slow method. This is
		primarily helpful when running big loops over data.
	 */
	protected $return_insert_id 	= TRUE;

	// The default type of element data is returned as.
	protected $return_type 			= "object";

	// Items that are always removed from data arrays prior to
	// any inserts or updates.
	protected $protected_attributes = array();

	/*
		You may need to move certain rules (like required) into the
		$insert_validation_rules array and out of the standard validation array.
		That way it is only required during inserts, not updates which may only
		be updating a portion of the data.
	 */
	protected $validation_rules 		= array(
		array(
			"field"		=> "iso",
			"label"		=> "lang:label_iso",
			"rules"		=> "required|min_length[2]|max_length[2]|alpha"
		),
		array(
			"field"		=> "name",
			"label"		=> "lang:label_name",
			"rules"		=> "trim|required|unique[countries.name,countries.iso]|max_length[80]|sanitize"
		),
		array(
			"field"		=> "printable_name",
			"label"		=> "lang:label_printable_name",
			"rules"		=> "trim|required|alpha_extra"
		),
		array(
			"field"		=> "iso3",
			"label"		=> "lang:label_iso_3",
			"rules"		=> "trim|alpha|max_length[3]"
		),
		array(
			"field"		=> "numcode",
			"label"		=> "lang:label_numcode",
			"rules"		=> "trim|integer|required"
		),
	);
	protected $insert_validation_rules 	= array();
	protected $skip_validation 			= FALSE;

	//--------------------------------------------------------------------
	protected $ctz_cache;
	
	/**
	 * Constructor
	 */
	public function __construct() {
	    $this->load->library('gzf_cache', array('lifetime' => 43200)); // 12 hours storage
	    $this->load->library('slug');
	    if(method_exists($this->gzf_cache, 'get_instance')) {
	        $this->ctz_cache = $this->gzf_cache->get_instance();
	    }
	}
	
	/**
	 * Countries Fields
	 *
	 * @return array
	 */
	public function fields() {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'iso',
	            'label' => 'label_iso',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'name',
	            'label' => 'label_name',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim|max_length[80]',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'slug',
	            'label' => 'label_slug',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim|max_length[255]',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'image',
	            'label' => 'label_image',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim|max_length[255]',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'printable_name',
	            'label' => 'label_printable_name',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim|max_length[80]',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'iso3',
	            'label' => 'label_iso_3',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'numcode',
	            'label' => 'label_numcode',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'phone_code',
	            'label' => 'label_phone_code',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'latitude',
	            'label' => 'label_latitude',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim|max_length[30]',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'longitude',
	            'label' => 'label_longitude',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim|max_length[30]',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'payment_methods',
	            'label' => 'label_payment_methods',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim|max_length[150]',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'currency_code',
	            'label' => 'label_currency_code',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'counts',
	            'label' => 'label_counts',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'hits',
	            'label' => 'label_hits',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'active',
	            'label' => 'label_status',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'priority',
	            'label' => 'label_priority',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'is_online',
	            'label' => 'label_is_online',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        
	    );
	    return $fields;
	}
	
	/**
	 * Function to backup countries in GoClixy Format
	 */
	public function backup_countries() {
	    $countries = $this->db->get('countries')->result();
	    if($countries) {
	        foreach($countries as $country) {
	            $this->create_locations_backup('countries', 'iso', $country->iso);
	        }
	    }
	}
	
	/**
	 * Create Locations Backup for Import
	 * @param string $table_name
	 * @param string $field_name
	 * @param string $field_value
	 * @return boolean
	 */
	private function create_locations_backup($table_name, $field_name, $field_value) {
	    // all parameters must be passed in GET
	    if(empty($table_name) && empty($field_name) && empty($field_value)) {
	        return false;
	    }
	    
	    // export mysql rows in a tmp_table
	    // this table will be used during import
	    $tmp_table_states = $this->db->dbprefix . "tmp_states";
	    $tmp_table_cities = $this->db->dbprefix . "tmp_cities";
	    $tmp_table_localities = $this->db->dbprefix . "tmp_localities";
	    
	    $this->load->dbutil();
	    $tables = array(
	        'tmp_states' => array('id', 'country_iso', 'name', 'slug', 'image', 'abbrev', 'latitude', 'longitude', 'active'),
	        'tmp_cities' => array('id', 'state_id', 'name', 'slug', 'image', 'description', 'abbrev', 'latitude', 'longitude', 'postcode', 'active'),
	        'tmp_localities' => array('id', 'city_id', 'name', 'slug', 'image', 'abbrev', 'latitude', 'longitude', 'postcode', 'active')
	    );
	    
	    // Drop temporary table
	    $this->drop_tmp_tables($tables);
	    
	    // NOTE: The TEMPORARY keyword is removed, the table_exists method of codeigniter is unable to work on temporary tables.
	    $where = '';
	    switch($table_name) {
	        case 'countries':
	            $column_name = 'country_id';
	            break;
	        case 'states':
	            $column_name = 'state_id';
	            unset($tables['tmp_states']);
	            break;
	        case 'cities':
	            $column_name = 'city_id';
	            unset($tables['tmp_states']);
	            unset($tables['tmp_cities']);
	            break;
	    }
	    
	    $table_name = $table_name . '_db';
	    // find row id using the field name and field value e.g. country_iso = IN
	    $result = $this->db->where($field_name, $field_value)->get($table_name)->row();
	    if($result) {
	        // check whether we have corresponding record in the database tables
	        $where = $column_name . ' = ' . $result->id;
	    }
	    
	    // Create Temporary Tables
	    $export_tables = array();
	    if(!empty($where)) {
	        foreach($tables as $tmp_table => $table_fields) {
	            $columns = implode(', ', $table_fields);
	            $db_table = str_replace('tmp_', '', $tmp_table) . '_db';
	            $where .= ' AND active = 1';
	            $records_count = $this->db->where($where)->count_all_results($db_table);
	            if($records_count > 0) {
	                $this->db->query("CREATE TABLE " . $this->db->dbprefix . $tmp_table . " AS (SELECT " . $columns . " FROM " . $this->db->dbprefix . $db_table . " WHERE " . $where . ")");
	                // dump("CREATE TABLE " . $this->db->dbprefix . $tmp_table . " AS (SELECT " . $columns . " FROM " . $this->db->dbprefix . $db_table . " WHERE " . $where . ")");
	                $export_tables[] = $this->db->dbprefix . $tmp_table;
	            }
	        }
	    }
	    if(empty($export_tables)) {
	        return false;
	    }
	    $prefs = array(
	        'tables'        => $export_tables,      // Array of tables to backup.
	        'ignore'        => array(),             // List of tables to omit from the backup
	        'format'        => 'txt',               // gzip, zip, txt
	        'filename'      => 'locations',         // File name - NEEDED ONLY WITH ZIP FILES
	        'add_drop'      => TRUE,                // Whether to add DROP TABLE statements to backup file
	        'add_insert'    => TRUE,                // Whether to add INSERT data to backup file
	        'newline'       => "\n"                 // Newline character used in backup file
	    );
	    
	    // Backup temporary table and assign it to a variable
	    $backup = $this->dbutil->backup($prefs);
	    
	    // Drop temporary table
	    $this->load->dbforge();
	    //$this->dbforge->drop_table($tmp_table, TRUE); // DROP TABLE IF EXISTS tmp_table
	    
	    // Load the file helper and write the file to your server
	    $zip_path = FCPATH . 'uploads/exports/' ;
	    $destination_file = $zip_path . $field_value . '/public_html/uploads/imports/_locations.sql';
	    
	    // Check whether the destination path exist or not
	    // Copying locations must also create folders
	    $dir_path = dirname($destination_file);
	    // if path does not exist, create folders recursively
	    if (!file_exists($dir_path)) {
	        mkdir($dir_path, 0777, true);
	    }
	    
	    $this->load->helper('file');
	    write_file($destination_file, $backup);
	    
	    // Create an Archieve
	    $file_name = $field_value . '.zip';
	    create_zip($zip_path . $field_value, $zip_path . $file_name);
	    
	    // Drop temporary table
	    //$this->drop_tmp_tables($tables);
	    $download = false;
	    //  remove assets directory contents to avoid future references
	    // unlink($destination_file);
	    if($download) {
	        $filename = $zip_path . '/' . $file_name;
	        if(file_exists($filename) && is_readable($filename) && file_exists($filename)){
	            header("Content-Disposition: attachment; filename=".basename(str_replace(' ', '_', $filename)));
	            header("Content-Type: application/force-download");
	            header("Content-Type: application/octet-stream");
	            header("Content-Type: application/download");
	            header("Content-Description: File Transfer");
	            header("Content-Length: " . filesize($filename));
	            flush(); // this doesn't really matter.
	            
	            $fp = fopen($filename, "r");
	            while (!feof($fp))
	            {
	                echo fread($fp, 65536);
	                flush(); // this is essential for large downloads
	            }
	            fclose($fp);
	            
	            // Unlink Backup File
	            unlink($filename);
	            
	            exit;
	        }
	        
	        // Unlink Backup File
	        unlink($filename);
	    }
	}
	
	/**
	 * Get and set country specific payment methods and currency
	 * @string $iso
	 */
	public function payment_methods_currency($iso) {
	    if(settings_item('lst.allow_country_selection') == 1) {
    	    $record = $this->db->where('iso', $iso)->get($this->table_name)->row();
    	    if($record) {
    	        if(!empty($record->currency_code)) { // Display all prices in country specific currency
    	            $this->load->model('currencies/currencies_model');
    	            $this->currencies_model->currency_info($record->currency_code, 'country_currency', 'id');
    	        } else {
    	            $this->session->unset_userdata('country_currency');
    	        }
    	        if(!empty($record->payment_methods)) { // Display country specific payment methods only	            
    	            $this->session->set_userdata('country_payment_methods', $record->payment_methods);
    	        } else {
    	            $this->session->unset_userdata('country_payment_methods');
    	        }
    	    }
	    }
	}
	
	/**
	 * Get Country Payment Methods
	 * @param string $country_iso - Country ISO
	 * @parm boolean $return_all - if no country specific gateway found, return all active gateways
	 * @return string
	 */
	public function _get_payment_methods($country_iso = null, $return_all = false, $return_form_fields = false) {
	    if(empty($country_iso)) {
	        $country_iso = $this->session->userdata('visitor_country_iso');
	    }
	    $cacheID = 'payment_gateways_' . $country_iso . '_' . $return_all;
	    if(!$record = $this->ctz_cache->load($cacheID)) {
	        $payment_methods = array();
	        $country = $this->db->where(array('iso' => $country_iso, 'payment_methods != ' => null))->get($this->table_name)->row();
	        if($country && !empty($country->payment_methods)) {
	            $this->db->where('id IN (' . $country->payment_methods . ') AND status = 1');
	            $payment_methods = $this->db->select('id, name, display_name, is_default')->get('payment_gateways')->result();
	        }
	        if($return_all && empty($payment_methods)) {
	            $payment_methods = $this->db->select('id, name, display_name, is_default')->where(array('status' => 1, 'deleted' => 0))->get('payment_gateways')->result();
	        }
	        $record = array('payment_methods' => $payment_methods);
	        $this->ctz_cache->save($record, $cacheID, array('countries'));
	    }
	    $payment_methods = $record['payment_methods'];
	    if($return_form_fields) {
	        if(!empty($payment_methods)) {
	            $i = 1; // the first payment gateway will be selected automatically
	            foreach($payment_methods as $payment_method) {
	                $radio_option = array(
	                    'name' => 'payment_method',
	                    'label' => $payment_method->display_name,
	                    'default_value' => $payment_method->id,
	                    'value' => ((!isset($_POST['payment_method']) && $payment_method->is_default == 1) || isset($_POST['payment_method']) && $_POST['payment_method'] == $payment_method->id) || (empty($_POST['payment_method']) && $i == 1)
	                );
	                $radio_options[] = $radio_option;
	                $i++;
	            }
	            $payment_methods = gc_form_element('radio', array('name' => 'payment_method', 'field_only' => true, 'options' => $radio_options));
	        }
	    }
	    return $payment_methods;
	}
	
	/**
	 * Get Country Information
	 * @return string
	 */
	public function _get($country_iso) {
	    $cacheID = $country_iso;
	    if(!$country = $this->ctz_cache->load($cacheID)) {
	        $country = $this->db->where('iso', $country_iso)->get($this->table_name)->row();
	        $this->ctz_cache->save($country, $cacheID, array('countries'));
	    }
	    return $country;
	}
	
	/**
	 * Get Country Currency
	 * @return string
	 */
	public function _get_currency($country_iso) {
	    $cacheID = 'currency_' . $country_iso;
	    if(!$record = $this->ctz_cache->load($cacheID)) {
	        $this->db->from($this->table_name . ' C');
	        $this->db->join('currencies CR', 'CR.id = C.currency_code', 'INNER');
	        $record = $this->db->select('CR.*')->where('iso', $country_iso)->get()->row();
	        $record = array('currency' => $record);
	        $this->ctz_cache->save($record, $cacheID, array('countries'));
	    }
	    return $record['currency'];
	}
}
