<?php if (! defined ( 'BASEPATH' ))exit ( 'No direct script access allowed' );
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class Help extends Admin_Controller
{
	protected $permissionView   = 'Marketplace.Help.View';
	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct() {
		parent::__construct();
		$this->auth->restrict($this->permissionView);
		$this->lang->load('marketplace');
		Template::set_block('sub_nav', 'help/_sub_nav');
	}
	
	/**
	 * Submenu Home Redirect
	 * @param number $offset
	 */
	public function home($offset = 0) {
	    $this->session->unset_userdata('category_filter_marketplace');
	    Template::redirect(site_url(SITE_AREA . '/help/marketplace'));
	}
	
	/**
	 * Displays a list of themes from marketplace
	 *
	 * @return void
	 */
	public function index($offset = 0) {
	    $this->lang->load('themes/themes');
	    $this->lang->load('migrate/migrate');
	    $navbar_category_id = 3; // navbar category id
	    Assets::add_module_css('marketplace', 'metisMenu.min.css');
	    Assets::add_module_js('marketplace', 'metisMenu.min.js');
	    Assets::add_module_css('marketplace', 'marketplace.css');
	    Assets::add_module_js('marketplace', 'marketplace.js');
	    
	    // Category filter
	    if(isset($_POST['category_filter'])) {
	        $this->session->set_userdata('category_filter_marketplace', $_POST['category_filter']);
	    }
	    if(isset($_POST['clear_category_filter'])) {
	        $this->session->unset_userdata('category_filter_marketplace');
	    }
	    $category_id = $this->session->userdata('category_filter_marketplace') ? $this->session->userdata('category_filter_marketplace') : $navbar_category_id; // product category id
	    
	    // Actions done, now display the view
	    $where = "";
	    
	    // default sort
	    if(!$this->session->userdata('sort_by_marketplace')) {
	        $this->session->set_userdata('sort_by_marketplace', 'id DESC');
	    }
	    if(isset($_POST['sort_by'])) {
	        $this->session->set_userdata('sort_by_marketplace', $_POST['sort_by']);
	    }
	    
	    // Search Handling
	    $searchterm = $this->gc->search('marketplace', $this->uri->segment(5), '/help/marketplace');
	    
	    // Sorting
	    $sort_by = $this->session->userdata('sort_by_marketplace');
	    
	    // Display rows to the user
	    $uri_string = 'products';
	    $post_fields = array(
	        'domain' => $this->sanitizeDomain(site_url()),
	        'download_id' => settings_item('site.download_id'), // get the files purchased by the user
	        'navbar_category_id' => $navbar_category_id,
	        'category_id' => $category_id,
	        'currency_code' => settings_item('site.currency_code'),
	        'sort_by' => 'P.' . $sort_by,
	        'limit' => $this->limit,
	        'q' => escape($searchterm),
	        'offset' => $offset
	    );
	    $result = curl_marketplace($uri_string, $post_fields);
	    $records = isset($result['records']) ? $result['records'] : array();
	    $total_rows = isset($result['total_rows']) ? $result['total_rows'] : 0;
	    $ordered_items = isset($result['ordered_items']) ? $result['ordered_items'] : array();
	    $installed_items = array();
	    $installed_products = $this->db->get('marketplace')->result();
	    if($installed_products) {
	        foreach($installed_products as $installed_product) {
	            $installed_items[$installed_product->product_id] = $installed_product->release_id;
	        }
	    }
	    
	    /**
	     * Install the file
	     */
	    if(isset($_POST['install'])) {
	        $this->load->library('themes/widgets');
	        $product_key = (int) $_POST['install'];
	        if(!empty($records[$product_key])) {
	            $item = $records[$product_key];
	            if(empty($item['download_urls'])) {
	                Template::set_message(lang('error_install'), 'danger');
	            }
	            $file_url = reset($item['download_urls']); // [0]; // get first item first item cannot be zero
	            $headers = getHeaders($file_url);
	            if ($headers['http_code'] === 200) {
	                $file = 'marketplace_' . date('Y_m_d_H_i_s') . '.zip';
	                $filepath = FCPATH . '/uploads/imports/' . $file; // substr( $file_url, strrpos( $file_url, '/' ) + 1); // dynamic URL, i.e. substr will not work
	                if (curl_download($file_url, $filepath)) {
	                    $extension = substr( $file, strrpos( $file, '.' ) + 1);
	                    $success = false;
	                    if($extension == 'zip') {
	                        // Extract Zip file
	                        // 'themes' is the delimiter, it is the folder structure themes/public_html/uploads/import
	                        extract_zip($filepath, 'themes');
	                        $_POST['set_default'] = 1;
	                        $_POST['insert_or_update'] = 0;
	                        $_POST['write_files'] = 1;
	                        $success = $this->widgets->import_dump();
	                    }
	                    if($success) {
	                        $datetime = date('Y-m-d H:i:s');
	                        $marketplace_data = array(
	                            'product_id' => $item['id'],
	                            'release_id' => $item['release_id'],
	                            'version' => !empty($item['version']) ? $item['version'] : 0,
	                            'revision' => !empty($item['revision']) ? $item['revision'] : 0
	                        );
	                        $item_info = $this->db->where('product_id', $item['id'])->get('marketplace')->row();
	                        if($item_info) { // update
	                            $marketplace_data['modified_on'] = $datetime;
	                            $this->db->where('id', $item_info->id)->update('marketplace', $marketplace_data);
	                        } else { // insert
	                            $marketplace_data['created_on'] = $datetime;
	                            $this->db->insert('marketplace', $marketplace_data);
	                        }
	                        Template::set_message(sprintf(lang('msg_migrate_success'), lang('label_import')), 'success');
	                        Template::redirect(current_url());
	                    }
	                }
	            }
	            Template::set_message(lang('error_install'), 'danger');
	        }
	    }
	    
	    // Pagination
	    $this->load->library('pagination');
	    
	    $pager['base_url'] = site_url(SITE_AREA ."/help/marketplace/");
	    $pager['total_rows'] = $total_rows;
	    $pager['per_page'] = $this->limit;
	    $pager['uri_segment']	= 5;
	    
	    $this->pagination->initialize($pager);
	    Template::set(array(
	        'toolbar_title' => lang('gc_menu_marketplace'),
	        'records' => $records,
	        'total_rows' => $total_rows,
	        'ordered_items' => $ordered_items,
	        'installed_items' => $installed_items,
	        'sort_by' => $this->session->userdata('sort_by_marketplace'),
	        'navbar' => isset($result['navbar']) ? $result['navbar'] : '',
	        'category' => isset($result['category']) ? $result['category'] : array(),
	        'navbar_category_id' => $navbar_category_id,
	        'category_id' => $category_id
	    ));
	    Template::render();
	}
	
	// Needs properly implementing - validates the user-specified domain
	private function sanitizeDomain($domainName) {
	    // remove www from the url
	    return str_replace('www.', '', $this->remove_http($domainName));
	}
	
	// Remove http and https from the URL
	private function remove_http($url) {
	    $disallowed = array('http://', 'https://');
	    foreach($disallowed as $d) {
	        if(strpos($url, $d) === 0) {
	            return str_replace($d, '', $url);
	        }
	    }
	    return $url;
	}
}