<?php if (! defined ( 'BASEPATH' ))exit ( 'No direct script access allowed' );
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class Members extends Front_Controller {	
	/**
	 * Constructor
	 *
	 * @return void
	 */
	//public $listing_permission;
	public function __construct() {
		parent::__construct ();
	
		$this->load->library ( 'form_validation' );
		$this->load->library('gc');
		// already loaded in the member_message_count
		// in order to have translations in other modules
		// $this->lang->load('members');
		$this->lang->load('listings/listings');
		//CREATE SLUG
		$config = array(
				'field' => 'slug',
				'title' => 'name',
				'table' => $this->db->dbprefix . 'listings',
				'id' => 'id',
		);
		$this->load->library('slug', $config);
		$this->load->library('crypt');
		//LOAD ALL MODELS FOR FILTERING AND OTHER OPERATIONS
		$this->load->model('listings/listings_model');
		$this->load->library('users/auth');
		$this->load->model('videos/videos_model');
		$this->load->model('listings/business_hours_model');
		$this->load->model('roles/role_model');
		$this->load->model('categories/categories_model');
		$this->load->model('locations/countries_model');
		$this->load->model('tags/tags_model');
		$this->load->model('business_queries/business_queries_model');
		$this->load->model('comments/comments_model');
		$this->load->model('orders/orders_model');
		$this->load->model('members/members_model');
		$this->load->model('orders/invoices_model');
		$this->load->model('packages/packages_model');
		$this->load->model('emailer/emailer_model');
		$this->load->model('links/links_model');
		
		
		//user authentication library
		Assets::add_js(array(
			'tinymce/tinymce.min.js',
		));
		
		Assets::add_module_js ( 'members', 'members.js' );
	}
	

	/**
	 * Displays dashboard to the member
	 *
	 * @return void
	 */
	public function index($offset = 0) {
	    // clear additional data from the listings table
	    if(settings_item('site.minimal') == 1) {
	        $this->db->where('user_id', $this->auth->user_id())->update('listings', array('additional_data' => null));
	    }
	    $this->auth->restrict();
	    $this->load->model('groups/groups_model');
	    $this->lang->load('groups/groups');
		// redirect back and forth between pages
		$this->session->set_userdata('listings_last_visited_page', current_url());
		// Deleting anything?
		if (isset ( $_POST ['delete'] )) {
			$checked = $this->input->post ( 'checked' );
			
			if (is_array ( $checked ) && count ( $checked )) {
				$result = FALSE;
				foreach ( $checked as $pid ) {
					$result = $this->listings_model->delete($pid);
					// Also set the Business Queries, Favorites and Comments status to deleted
					$this->gc->module_soft_delete('listings', $pid);
				}
				
				if ($result) {
					Template::set_message ( count ( $checked ) . ' ' . sprintf(lang ( 'msg_delete_success' ), lang('label_listing')), 'success' );
				} else {
					Template::set_message ( sprintf(lang ( 'msg_delete_failure' ), lang('label_listing')) . $this->listings_model->error, 'danger' );
				}
			}
		}
		$records = $this->listings_model->members(0, $this->auth->user_id(), $this->limit, $offset);
		$listing_count = $this->listings_model->members(1, $this->auth->user_id(), $this->limit, $offset);
		
		// Booking Forms
		$this->load->model('bookings/bookings_model');
		$this->bookings_model->booking_link();
		if(isset($_POST['booking'])) {
		    $this->bookings_model->view_forms('listings', $_POST['booking']);
		}
		
		// Pagination
		$this->load->library('pagination');
		$this->config->load("pagination");
		$config['base_url'] = gc_url_format('members');
		$config['total_rows'] = $listing_count;
		$config['per_page'] = $this->limit;
		$this->pagination->initialize($config);
		
		Template::set (array(
			'toolbar_title'	=> lang('members_dashboard'),	
			'records' => $records) 
		);
		$this->load->model('members/members_model');	
		
		$message = $this->members_model->member_dashboard($this->auth->user_id());
		Template::set(array('membership_info' => $message, 'page_title' => lang('members_dashboard')));
		// save current URL in a session variable for back and forth redirection
		$this->gc->cancel_url();
		$layout = $this->gc->load_assets('members/index');
		Template::set_view($layout['view']);
		Template::render($layout['layout']);
	}
	
	/**
	 * Allow member to add new business
	 * @param int $package_id 
	 */
	public function add_business($role = 'member', $package_id = NULL) {	
		$this->auth->restrict('Site.Business.Owner');
		$package_id = intval($this->crypt->decode($this->input->get('id')));
		$this->listings_model->business('members', $package_id, $this->auth->user_id());
	}
	
	/**
	 * Edit business information
	 * @param string $id
	 */
	public function edit_business() {
	    $this->auth->restrict('Site.Business.Owner');
	    $id = intval($this->crypt->decode($this->input->get('id')));
	    $this->listings_model->business('members', 0, $this->auth->user_id(), $id);
	}
	
	/**
	 * Ajax function to load sub categories
	 * @param int $parent_id
	 * @return string
	 */
	public function loadSubCategories($parent_id)
	{
		header('Content-Type: application/x-json; charset=utf-8');
		$result = $this->categories_model->find_all_by('parent_id', $parent_id);
		$output = json_decode(json_encode($result),true);
		echo json_encode($output);
	}
	
	/**
	 * Ajax function to load locations
	 * @param string $type
	 * @param int $id
	 * @return string
	 */
	public function loadLocations($type, $id)
	{
		header('Content-Type: application/x-json; charset=utf-8');
		$result = $this->listings_model->getData($type, $id);
		$output = json_decode(json_encode($result),true);
		echo json_encode($output);
	}
	
	/**
	 * Display Google Map to the member
	 * @param float $latitude
	 * @param float $longitude
	 */
	public function google_map($latitude = NULL, $longitude = NULL) {
		$this->load->library ( 'listings/googlemaps' );
		//Assets::add_module_js('members', 'listing.js');
		$config = array ();
		// SET LATITUDE AND LONGITUDE IF NOT EDIT
		if (($latitude == NULL) && ($longitude == NULL)) {
			if((settings_item('lst.latitude') != '') && (settings_item('lst.longitude') != '')) {
				$latitude = settings_item('lst.latitude');
				$longitude = settings_item('lst.longitude');
			} else {
				$ip = $_SERVER ['REMOTE_ADDR'];
				// MADE USE OF FREE PLUGIN
				$geopluginURL = 'http://www.geoplugin.net/php.gp?ip=49.249.196.21'; // .$ip;
				$unarr = file_get_contents ( $geopluginURL ); // Get File Contents
				$AddArr = unserialize ( $unarr ); // Get PHP values from file contents
				$latitude = $AddArr ['geoplugin_latitude']; // City Name
				$longitude = $AddArr ['geoplugin_longitude'];
			}
		}
		$config ['center'] = "$latitude, $longitude";
		$config ['zoom'] = 14;
		$config ['places'] = TRUE;
		$this->googlemaps->initialize ( $config );
		
		$marker = array ();
		$marker ['position'] = "$latitude, $longitude";
		$marker ['draggable'] = true;
		$marker ['ondragend'] = '$("#latitude").val(event.latLng.lat()), $("#longitude").val(event.latLng.lng())';
		$this->googlemaps->add_marker ( $marker );
		return $this->googlemaps->create_map ();
	}
	
	/**
	 * Allow member to upload listing logo
	 */
	public function logo() {
		$this->auth->restrict('Site.Business.Owner');
		$id = intval($this->crypt->decode($this->input->get('id')));
		$this->gc->upload_logo_cover('member', $id, 'listings', 'logo');
	}
	
	/**
	 * Allow member to upload a cover image
	 */
	public function cover() {
		$this->auth->restrict('Site.Business.Owner');
		$id = intval($this->crypt->decode($this->input->get('id')));
		$module = $this->crypt->decode($this->input->get('md'), true);
		$module = (empty($module) || gc_module_id($module) == 0 ? 'listings' : $module);
		$this->gc->upload_logo_cover('member', $id, $module, 'cover');
	}
	
	/**
	 * Create thumbs of uploaded image
	 * @param string $path for saving image
	 * @param string $file for file name
	 */
	public function resize($path,$file) {
		$config['image_library']='gd2';
		$config['source_image']=$path;
		$config['create_thumb']=TRUE; // This will create new file
		$config['maintain_ratio']=TRUE;
		$config['width']=settings_item('lst.thumbnail_width');
		$config['height']=settings_item('lst.thumbnail_height');
		$config['thumb_marker'] = false;
		$config['new_image']='./assets/images/logo/thumbs/'.$file; //CHANGE THIS LINE FOR PATH
	
		$this->load->library('image_lib',$config);
		$this->image_lib->resize();
	}
	
	/**
	 * Upload image using CRUD
	 * Image_CRUD 0.6 is used to upload images
	 */
	public function photos($module, $row_id = 0) {
		$this->auth->restrict('Site.Business.Owner');
		if (!is_numeric($row_id)) {
			Template::set_message ( lang ( 'invalid_id' ), 'error' );
			Template::redirect( gc_url_format( 'members' ));
		}
		$this->load->model('images/images_model');
		$this->images_model->upload_photos($module, $row_id);
	}
	
	/**
	 * Allow member to add business hours
	 * @param string $listing_id
	 */
	public function business_hours() {
		$this->auth->restrict('Site.Business.Owner');
		$listing_id = intval($this->crypt->decode($this->input->get('id')));
		$listing_user = $this->listings_model->select('user_id')->find_by(array('id' => $listing_id, 'deleted' => 0)); // find user_id of specified listing
		if( ($listing_id != NULL) && $listing_user && ($listing_user->user_id == $this->auth->user_id())) {		
			//BUSINESS HOURS TIME PICKER
			Assets::add_js('moment.js');
			Assets::add_css ( 'bootstrap-datetimepicker.min.css' );
			Assets::add_js ( 'bootstrap-datetimepicker.min.js' );
			Assets::add_module_js ( 'listings', 'listings.js' );
			
			$this->load->model('listings/business_hours_model');
			if (isset ( $_POST ['save'] )) {
				if ($this->business_hours_model->save_business_hours('insert', $listing_id)) {					
					Template::set_message ( sprintf(lang('msg_save_success'), lang('label_business_hours')), 'success' );
					Template::redirect ( gc_url_format('members/business_hours') . '?id=' . $this->crypt->encode($listing_id));
				} else {
					Template::set_message ( sprintf(lang('msg_create_failure'), lang('label_business_hours')) . $this->business_hours_model->error, 'danger' );
				}
			}
			$records = $this->business_hours_model->where ( 'listing_id', $listing_id )->find_all();
			$listing_info = $this->db->where('id', $listing_id)->select('id, title, slug, city')->get('listings')->row();
			$toolbar_title = (!empty($listing_info) ? $listing_info->title . ' - ' : '') . lang('business_hours');
			Template::set (array(
				'member_area' => TRUE, // set this variable to TRUE for QUERY STRING FORM else the SAVE button will redirect without saving.
				'elements_container' => 'div.card.card-body', // wrap form elements within a card container
			    'page_title'	=> $toolbar_title,
			    'toolbar_title'	=> $toolbar_title,
				'listing_id' => $listing_id,
				'records' => $records
			));
			Template::set_view('listings/business_hours');
			// This method has no View in the Themes Module, i.e. load layout files manually
			$this->gc->demo_custom_css_js('member');
			Template::render ('member');
		} else {
			redirect(gc_url_format('members'));
		}
	}
	
	/**
	 * Allow member to view comments posted by users on their listings
	 * @param number $offset for pagination
	 */
	public function view_comments($module = 'listings', $offset = 0) {
		$this->auth->restrict('Site.Business.Owner');
	    $this->lang->load('comments/comments');
		$this->member_view_module('view_comments', $module, $offset);
	}
	
	/**
	 * Allow member to view comments posted by users on others listings
	 * @param number $offset for pagination
	 */
	public function my_comments($module='listings', $offset = 0) {
	    $this->lang->load('comments/comments');
		$this->member_view_module('my_comments', $module, $offset);
	}
	
	/**
	 * Allow member to view business_queries posted by users on their listings
	 * @param number $offset for pagination
	 */
	public function view_business_queries($module='listings', $offset = 0) {
		$this->auth->restrict('Site.Business.Owner');
		Assets::add_module_js ( 'members', 'business_queries.js' );
		Assets::add_module_css ( 'business_queries', 'business_queries.css' );
		$this->member_view_module('view_business_queries', $module, $offset);
	}
	
	/**
	 * Allow member to view posted queries asked by user on others listings
	 * @param number $offset for pagination
	 */
	public function posted_queries($module='listings', $offset = 0) {
		$this->lang->load('business_queries/business_queries');
		Assets::add_module_css ( 'business_queries', 'business_queries.css' );
		Assets::add_module_js ( 'members', 'business_queries.js' );
		$this->member_view_module('posted_queries', $module, $offset);
	}
	
	/**
	 * Display MyFavorites to the User
	 * @param string $filter
	 * @param number $offset
	 */
	public function favorites($module='listings', $offset = 0) {
		$this->member_view_module('favorites', $module, $offset);
	}
	
	
	
	/**
	 * Allow member to view module specific rows
	 * @param number $offset for pagination
	 */
	public function member_view_module($method, $module, $offset) {
		$this->auth->restrict();
		$user_id = $this->auth->user_id();
		$module_id = gc_module_id($module);
		// redirect user to the member area if the filter type is not a module
		if($module_id == 0) {
			Template::redirect(gc_url_format('members'));
		}
		switch($method) {
			case 'view_comments':
				$title = lang('members_comments');
				$view = 'members/view_comments';
				$table = 'comments';
				$label = lang('label_comment');
				break;
			case 'my_comments':
				$title = lang('my_comments');
				$view = 'members/view_comments';
				$table = 'comments';
				$label = lang('label_comment');
				break;
			case 'view_business_queries':
				$title = lang('members_business_queries');
				$view = 'members/view_business_queries';
				$table = 'business_queries';
				$label = lang('label_business_query');
				break;
			case 'posted_queries':
				$title = lang('my_queries');
				$view = 'members/view_business_queries';
				$table = 'business_queries';
				$label = lang('my_queries');
				break;
			case 'favorites':
			    $this->load->model('favorites/favorites_model');
				$title = lang('my_favorites');
				$view = 'members/favorites';
				$table = 'favorites';
				$label = lang('label_favorite');
				break;
		}
		
		// Return Module Counts
		$counts = $this->members_model->fcb_counts($user_id, $method);
		
		// default loading module
        if(empty($counts[$module]) && !empty($counts)) {
            $module = null;
            foreach($counts as $cmod => $cval) {
                if(!empty($cval) && empty($module)) {
                    $module = $cmod;
                }
            }
        }
		
		// Deleting anything?
		if (isset ( $_POST ['delete'] )) {
			$checked = $this->input->post ( 'checked' );
			if (is_array ( $checked ) && count ( $checked )) {
				$result = FALSE;
				foreach ( $checked as $pid ) {
					switch($method) {
						case 'view_comments':
						case 'my_comments':
							$result = $this->comments_model->delete_hierarchy($pid);
							break;
						case 'view_business_queries':
						case 'posted_queries':
							$result = $this->business_queries_model->delete($pid);
							break;
						case 'favorites':
							$result = $this->favorites_model->delete ( $pid );
							break;
					}					
				}
	
				if ($result) {
					Template::set_message ( count ( $checked ) . ' ' . sprintf(lang ( 'msg_delete_success' ), $label), 'success' );
					Template::redirect(gc_url_format('members/' . $method));
				} else {
					Template::set_message ( sprintf(lang ( 'msg_delete_failure' ), $label), 'error' );
				}
			}
		}
	
		$fcb = $this->members_model->fcb_records($user_id, $method, $table, $module, 0, $offset, $this->limit);
		$records = $fcb['records'];
		$total_records = $fcb['total_records'];
		
		$pagerUriSegment = 4;
		$pagerBaseUrl = gc_url_format('members/' . $method) . '/' . $module .'/';
	
		$this->load->library('pagination');
		$pager['base_url']    = $pagerBaseUrl;
		$pager['total_rows']  = $total_records;
		$pager['per_page']    = $this->limit;
		$pager['uri_segment'] = $pagerUriSegment;
	
		$this->pagination->initialize($pager);
		Template::set(array(
		    'method' => $method,
			'counts' => $counts,
			'module' => $module,
			'index_url' => gc_url_format('members/' . $method) .'/',
			'filter_type' => $module,
			'records' => $records,
			'page_title' => $title,
			'toolbar_title' => $title
		));
		$layout = $this->gc->load_assets($view);
		Template::set_view($layout['view']);
		Template::render($layout['layout']);
	}
	
	/**
	 * Allow member to select package for their listing
	 * @param number $offset
	 */
	public function packages() {
		$this->auth->restrict();
		$this->load->model('packages/packages_model');
		$this->load->model('members/members_model');
		
		// change package
		$listing_id = intval($this->crypt->decode($this->input->get('id')));
		if($listing_id) {
			$listing_package_price = $this->members_model->listing_package_price($listing_id);
			Template::set('listing_package_price', $listing_package_price['price']);
			Template::set('listing_id', $listing_id);
			$this->session->set_userdata('lst_package_id', $listing_package_price['package_id']); // for change_package
			$this->session->set_userdata('lst_package_price', $listing_package_price['price']); // for change_package
			$this->session->set_userdata('lst_id', $listing_id); // for change package
		}
		
		/**
		 * User is a subscribed member i.e. display only those packages where user is member of
		 * If User is new or all previous subcriptions are expired or user has reached listing limit display new packages
		 */
		$order_detail = $this->members_model->get_order_details($this->auth->user_id());
		if(!empty($order_detail)) {
		    // get existing listing package
		    $listing = $this->db->where('id', $listing_id)->get('listings')->row();
			$ordered_packages = array();
			$purchased_packages = array(); // do not display the same package twice
			foreach($order_detail as $value) {
			    // do not allow user to downgrade to a low price package // $listing_package_price < $value['price']
			    if((!$listing || ($listing && $listing->package_id != $value['package_id'])) && !empty($listing_package_price) && $listing_package_price < $value['price']) {
    			    if(!in_array($value['package_id'], $purchased_packages)) { // do not display the same package twice
    			        $purchased_packages[] = $value['package_id'];
        				$package_info = array();
        				foreach($value as $key => $ordered_package) {
        					switch($key) {
        						case 'id':
        							break;
        						case 'package_id':
        							$package_info['id'] = $ordered_package;
        							break;
        						case 'package_title':
        							$package_info['title'] = $ordered_package;
        							break;
        						case 'package_description':
        							$package_info['description'] = $ordered_package;
        							break;
        						default:
        							$package_info[$key] = $ordered_package;
        					}
        				}
        				$ordered_packages[] = (object) $package_info;
    			    }
			    }
			}
			// the user has purchased only one package
			// redirect user to the membership page for another package purchase
			if(empty($ordered_packages)) {
			    Template::redirect(gc_url_format('members/membership'));
			}
			Template::set('records', $ordered_packages);
			if($listing_id) {
				Template::set('url', 'change_package');
			} else {
				Template::set('url', 'add_business');
			}
			Template::set('btn_text', lang('header_add_business'));
		} else {
			// GET PACKAGES WHICH ARE NOT EXPIRED
			//$where = "((duration = 0) || (duration >= DATEDIFF( '" .$now ."', created_on )) + duration) AND active = 1";
			/*$all_packages = $this->members_model->get_packages();
			if($all_packages) {// && (count($all_packages)> 1)) { // when multiple packages are set
				$this->session->set_userdata('membership_type', 'new');
				Template::set ( 'records', $all_packages );
				Template::set('url', 'checkout');
				Template::set('btn_text', lang('btn_buy_now'));
			} */
			Template::redirect(gc_url_format('members/membership'));
		}
		$subscription = $this->members_model->get_order_details_for_package($this->auth->user_id());
		Template::set ('subscription', $subscription);
		Template::set ('page_title', lang('label_select_package'));
		Template::set ('toolbar_title', lang('label_select_package'));

		$layout = $this->gc->load_assets('members/packages');
		Template::set_view($layout['view']);
		Template::render($layout['layout']);
	}
	
	/**
	 * Display Packages to the User
	 */
	public function membership() {
		$this->auth->restrict ();
		$this->load->model('members_model');
		$all_packages = $this->members_model->get_packages();
		if($all_packages) {
			$this->session->set_userdata('order_type', 'packages');
			$this->session->set_userdata('subscription_type', 1);
			Template::set ( 'packages', $all_packages );
			Template::set('url', 'checkout');
			Template::set('btn_text', lang('btn_buy_now'));
		} else {
		    // no more packages to purchase
		    Template::redirect(gc_url_format('members'));
		}
		Template::set ('page_title', lang('label_select_package'));
		Template::set ('toolbar_title', lang('label_select_package'));

		$layout = $this->gc->load_assets('members/membership');
		Template::set_view($layout['view']);
		Template::render($layout['layout']);
	}
	
	/**
	 * Display upgrade packages to the member
	 */
	public function upgrade() {
		$this->auth->restrict('Site.Business.Financial');
		$this->load->model('packages/packages_model');
		$this->load->model('members/members_model');
		$this->session->set_userdata('membership_type', 'upgrade');
		$package_id = intval($this->crypt->decode($this->input->get('id')));
		
		if(!(int)$package_id) {
			redirect(gc_url_format('members'));
		}
			
		/**
		 * Old Order ID is retrieved through GET Request
		 * If Order ID is valid save orderid in a session variable
		 * so that it can be accessed from payment_gateways transaction successful method
		 */
		$order_id = $this->crypt->decode($this->input->get('order_id'), true);
		if(!ctype_alnum($order_id) && strlen($order_id) > 20) {
			redirect(gc_url_format('members'));
		}
		
		// check for the upgrades
		$user_id = $this->auth->user_id();
		$upgrade_packages = $this->packages_model->get_upgrade_packages($user_id, true);
		if(empty($upgrade_packages)) {
		    Template::set_message(lang('msg_no_package_available'),'danger');
		    Template::redirect(gc_url_format('members'));
		}
		
		// $this->session->set_userdata('order_type', 'packages');
		// $this->session->set_userdata('subscription_type', 2);
		// $this->session->set_userdata('upgrade_order_id', $order_id);
		$this->session->set_userdata('gc_order_type', array(
		    'subscription_type' => 2,
		    'order_id' => $order_id
		));
		
		if(!empty($_GET['page']) && $_GET['page'] == 'change_package') {
		    // change package page
		    $upgrade_packages = $this->members_model->get_packages('upgrade', $package_id);
		    if(!$upgrade_packages) {
		        Template::set_message(lang('msg_no_package_available'),'danger');
		        Template::redirect(gc_url_format('members'));
		    }
		    $package_info = reset($upgrade_packages);
		    $package_id = $package_info['id'];
		}
		
		Template::redirect(gc_url_format('members/checkout') . '?id=' . $this->crypt->encode($package_id));
		
		/*$all_packages = $this->members_model->get_packages('upgrade', $package_id);
		if($all_packages) {
			Template::set ( 'packages', $all_packages );
		} else {
			Template::set_message(lang('msg_no_package_available'),'error');
			redirect(gc_url_format('members'));
		}
		Template::set ('page_title', lang('label_select_package'));
		Template::set ('toolbar_title', lang('label_select_package'));
		$layout = $this->gc->load_assets('members/membership');
		Template::set_view($layout['view']);
		Template::render($layout['layout']);*/
	}
	
	/**
	 * Renew Package if it is renewable
	 * Display upgrade packages to the member
	 */
	public function renew() {
		$this->auth->restrict('Site.Business.Financial');
		$this->load->model('packages/packages_model');
		$package_id = intval($this->crypt->decode($this->input->get('id')));
		if(!(int)$package_id) {
			redirect(gc_url_format('members'));
		}
		$order_id = escape($this->crypt->decode($this->input->get('order_id'), true));
		if(!ctype_alnum($order_id) && strlen($order_id) > 20) {
			redirect(gc_url_format('members'));
		}
		
		$order_items = $this->db->where(array('order_id' => $order_id))->get('order_items')->result();
		if(!$order_items) {
		    redirect(gc_url_format('members'));
		}
		$todays_date = date('Y-m-d');
		foreach($order_items as $order_item) {
		    $item_reminders = json_decode($order_item->reminders ?? '', true);
		    if(is_string($item_reminders[0])) {
		        $item_reminders = json_decode($item_reminders[0], true);
		    }
		    if(!empty($item_reminders['invoice_expiry']) && $item_reminders['invoice_expiry'] < $todays_date) {
		        // invoice expired, the user cannot renew the package
		        $message = sprintf(lang('order_invoice_expired'), gc_date_format($item_reminders['invoice_expiry'], false));
		        Template::set_message($message, 'danger');
		        Template::redirect(gc_url_format('members'));
		    }
		}
		
		$package = $this->packages_model->find_by(array('id' => $package_id, 'renewable' => 1));
		if($package) {
			/**
			 * Send old order id to the checkout page for expiry date calculation
			 * Required if renewed before expiry date
			 */
			// $this->session->set_userdata('order_type', 'packages');
			// $this->session->set_userdata('subscription_type', 3);
			// $this->session->set_userdata('upgrade_order_id', $order_id);		
			// if the invoicing package is different, renew a different package
		    if(!empty($package->invoicing_package)) {
		        $package_id = $package->invoicing_package;
		    }
		    $this->session->set_userdata('gc_order_type', array(
		        'subscription_type' => 3,
		        'order_id' => $order_id
		    ));
		
			redirect(gc_url_format('members/checkout') . '?id=' . $this->crypt->encode($package_id));
		} else {
			Template::set_message(lang('msg_package_not_renewable'),'error');
			redirect(gc_url_format('members/membership'));
		}
	}
	
	/**
	 * Change listing package
	 */
	public function change_package() {
		$this->auth->restrict('Site.Business.Owner');
		$this->load->model('listings/listings_model');
		$this->load->model('members/members_model');
		$this->load->model('packages/packages_model');
		$package_id = intval($this->crypt->decode($this->input->get('id')));
		$count = $this->listings_model->count_by(array('user_id' => $this->auth->user_id(), 'package_id' => $package_id));
		$order_info = $this->members_model->get_package_info($this->auth->user_id(), $package_id);
		if($order_info && $package_id != $this->session->userdata('lst_package_id')) { // old package id and new are not same
			$listings_limit = gc_field_value('listings', $order_info->module_limit);
			$listings_featured = gc_field_value('listings', $order_info->module_featured);
			if(($listings_limit == 0) || ($count < $listings_limit)) {
				$data = array ('package_id' => $package_id, 'featured' => $listings_featured, 'expires_on' => $order_info->expires_on);
				$this->db->where('id', $this->session->userdata('lst_id'));
				$this->db->update('listings', $data);
				// update module package also
				$this->listings_model->update_module($this->session->userdata('lst_id'), 'package_id', $package_id);
				
				// update status as per the new package
				$new_status = 0;
				$module_status = json_decode($order_info->module_status ?? '', true);
				if(!empty($module_status['listings'])) {
				    $new_status = 1;
				}
				// update listing associated images, videos and other modules status
				$this->listing_update($this->session->userdata('lst_id'), $new_status, $module_status);
				
				$this->session->unset_userdata('lst_id');
				$this->session->unset_userdata('lst_package_price');
				Template::set_message(sprintf(lang('msg_package_updated'), $order_info->package_title), 'success');
				redirect(gc_url_format('members'));
			} else {
				Template::set_message(sprintf(lang('user_limit'), $listings_limit, 'listing(s)'), 'error');
				redirect(gc_url_format('members'));
			}
		} else {
			Template::set_message(lang('msg_same_package'), 'error');
			redirect(gc_url_format('members'));
		}
	}
	
	/**
	 * Allow member to view all orders
	 * The only difference between the order and invoice is the invoice id
	 * i.e. a common method is used to simplify things
	 * @param number $offset for pagination
	 */
	public function orders($offset = 0) {
	    $this->auth->restrict('Site.Business.Financial');
	    $this->session->set_userdata('orders_last_visited_page', current_url());
	    $this->load->model('orders/orders_model');
	    if (isset($_POST['print'])) {
	        $this->orders_model->email_print($_POST['print']);
	    }
	    $user_id = $this->auth->user_id();
	    $records = $this->gcrud->get_all_crud(0, "M.user_id = " . $user_id, 'M.id DESC', $this->limit, $offset, array('module' => 'orders', 'member' => true));
	    $total_rows = $this->gcrud->get_all_crud(1, "M.user_id = " . $user_id, 'M.id DESC', $this->limit, $offset, array('module' => 'orders', 'member' => true));
	    
	    // Pagination
	    $this->load->library('pagination');
	    $this->config->load('pagination');
	    $config['base_url'] = gc_url_format('members/orders');
	    $config['total_rows'] = $total_rows;
	    $config['per_page'] = $this->limit;
	    $this->pagination->initialize($config);
	    Template::set(array(
	        'records' => $records,
	        'page_title' => lang('orders_history'),
	        'toolbar_title' => lang('orders_history')
	    ));
	    $layout = $this->gc->load_assets('members/orders');
	    Template::set_view($layout['view']);
	    Template::render($layout['layout']);
	}
	
	/**
	 * Allow member to view their invoice in HTML format
	 * @param int $transaction_id to store transaction information
	 */
	public function detail($id, $module = 'orders') {
	    if($module != 'orders') {
	        Template::redirect(gc_url_format('members'));
	    }
	    $order_id = $id;
	    $this->load->model('orders/orders_model');
	    $order = $this->orders_model->get_order($order_id, 'order_id');
	    if(!$order || $order->user_id != $this->auth->user_id()) {
	        Template::redirect(gc_url_format('members'));
	    }
	    $content = $this->orders_model->email_print($order->order_id, 'view', array('order' => $order, 'role' => 'frontend', 'action_type' => 'status_update'));
	    Template::set(array(
	        'toolbar_title'	=> lang('label_order') . ' #' . $order->order_id,
	        'order_info'=> $content
	    ));
	    Template::set_view('view_order');
	    // This method has no View in the Themes Module, i.e. load layout files manually
	    $render = $this->gc->demo_custom_css_js('member');
	    Template::render($render);
	}
	
	/**
	 * Pay Pending Order
	 * @param string $order_id
	 */
	public function pay($order_id) {
	    $order_id = escape($order_id);
	    $user_id = $this->auth->user_id();
	    $where = "order_id = '" . $order_id . "' AND user_id = '" . $user_id . "' AND status_id != 2 AND status_id != 6 AND status_id != 7";
	    $order_items = $this->db->where($where)->get('order_items')->result();
	    if(!$order_items) {
	        Template::redirect(gc_url_format('members'));
	    }
	    $this->load->model('orders/orders_model');
	    // clear cart if any other items are in the list
	    $this->orders_model->checkout_params($user_id, false, array(), true);
	    foreach($order_items as $order_item) {
	        // add items to the cart and redirect user to the checkout page
	        $this->session->unset_userdata('gc_order_type');
	        $extra = array();
	        if(!empty($order_item->ru_order_id)) {
	            $extra['gc_order_type'] = array(
	                'subscription_type' => $order_item->subscription_type,
	                'order_id' => $order_item->ru_order_id
	            );
	            $this->session->set_userdata('gc_order_type', $extra['gc_order_type']);
	        }
	        $this->orders_model->cart($order_item->row_id, 0, $order_item->qty, 0, gc_module_name($order_item->module_id), true, $extra, $user_id);
	    }
	    Template::redirect(gc_url_format('members/checkout'));
	}
	
	/**
	 * Allow user to pay invoice
	 * @param string $order_id
	 */
	public function pay_invoice() {
		$this->auth->restrict('Site.Business.Financial');	
		$this->gc->demo_custom_css_js('member');
		Assets::add_css ( 'checkout.css' );
		Assets::add_js(array('jquery.validate.min.js', 'jquery.payment.min.js'));
		Assets::add_js('checkout.js');
		$this->load->model('locations/states_model');
		$this->load->model('locations/cities_model');
		$order_info = $this->invoice_permission(FALSE);
		$order_id = $order_info->order_id;
		$order_type = $order_info->order_type;
		$package_id = $order_info->package_id;
		$module_row_id = $package_id;
		$params = array();
		switch($order_type) {
			case 1:
				$item = json_decode($order_info->item ?? '');
				$params['upgrade_order_id'] = $order_info->renew_upgrade_order_id;
				$params['renewed_order_id'] = $order_info->renew_upgrade_order_id;
				$params['subscription_type'] = $order_info->subscription_type;
				break;
			case 2:
				$params['listing_id'] = $order_info->row_id;
				break;
			case 3:
				break;
			case 4:
				$params['module'] = gc_module_name($order_info->module_id);
				$params['renewed_order_id'] = $order_info->renew_upgrade_order_id;
				$params['subscription_type'] = $order_info->subscription_type;
				break;
			case 6:
			case 7:
			case 8:
			case 9:
				$module_row_id = $order_info->row_id;
				break;
			default:
			    $params['amount'] = $order_info->order_total;
		}
		$params['is_invoice'] = true;
		$params['order_id'] = $order_id;
		$params['payment_method'] = $order_info->payment_method;
		$params['status'] = $order_info->status;
		$order_type = $this->orders_model->order_type_info($order_info->order_type);
		$result = $this->orders_model->checkout($this->auth->user_id(), $module_row_id, $order_type, $params);//$this->orders_model->pay_invoice(, $order_id);
		if($result['display'] == true) {
			if($result['success'] == true) {
				Template::set_message ( $result['message'], 'success' );
			} else {
				Template::set_message ( $result['message'], 'error' );
			}
		}
		if(isset($result['url'])) {
			redirect($result['url']);
		}
	}
	
	/**
	 * Allow member to view their invoice in HTML format
	 * @param int $transaction_id to store transaction information
	 */
	public function view_invoice() {
	    $this->auth->restrict('Site.Business.Financial');
	    if(isset($_GET['order_id'])) {
	        $order_id = escape($_GET['order_id']);
	    } else {
	        $order_id = $this->invoice_permission();
	    }
		$this->load->model('orders/invoices_model');
		$invoice_info = $this->invoices_model->select('id, prefix, user_id')->find_by('order_id', $order_id);
		if(!$invoice_info || $invoice_info->user_id != $this->auth->user_id()) { // restrict user to view his/her invoice only
		    Template::redirect(gc_url_format('members'));
		}
		$content = '';
		$order = $this->orders_model->get_order($order_id, 'order_id');
		if($order) {
		    $content = $this->orders_model->email_print($order->order_id, 'view', array('order' => $order));
		}
		Template::set(array(
			'toolbar_title'	=> lang('invoice_title') . ' #' . $invoice_info->prefix . $invoice_info->id,
			'invoice'=> $content
		));

		$layout = $this->gc->load_assets('members/view_invoice');
		Template::set_view($layout['view']);
		Template::render($layout['layout']);
	}
	
	/**
	 * Verify permission
	 * @return array
	 */
	private function invoice_permission($return_order_id = TRUE) {
		$id = intval($this->crypt->decode($this->input->get('id')));
		if($id == 0) {
			redirect(gc_url_format('members'));
		}
		
		// Allow user to view only their own invoices
		$role_id = $this->auth->role_id();
		$user_id = $this->auth->user_id();
		$where = array();
		$where['id'] = $id;
		// User having admin role can view all invoices
		if($role_id != 1) {
		//	$where['user_id'] = $user_id;
		}
		
		$order_info = $this->invoices_model->find_by($where);
		if(!$order_info) {
			redirect(gc_url_format('members'));
		}
		if($return_order_id) {
			return $order_info->order_id;
		} else {			
			return $this->orders_model->find_by(array('order_id' => $order_info->order_id));
		}
	}
	
	/**
	 * CREATE PDF FILE
	 * Creates a Business Listing PDF document using TCPDF
	 * @package com.tecnick.tcpdf
	 * @abstract TCPDF - Example: Default Header and Footer
	 */
	public function download_invoice() {
		$this->auth->restrict('Site.Business.Financial');
		$order_id = $this->invoice_permission();
		$this->gc->invoice_pdf($order_id);
	}	
	
	/**
	 * To display message to the member on email reply failure
	 * @param string $messsage
	 */
	function errorResponse ($messsage) {
		header('HTTP/1.1 500 Internal Server Error');
		die(json_encode(array('message' => $messsage)));
	}
	
	/**
	 * Verify captcha before proceeding
	 * @param string $fieldname for POST
	 * @return boolean
	 */
	public function check_captcha($fieldname) {
		$this->load->library ( 'securimage/securimage' );
		$securimage = new Securimage ();
		if (! $securimage->check ( $this->input->post ( $fieldname ) )) {
			return FALSE;
		} else {
			return TRUE;
		}
	}
	
	/**
	 * Send email to the user
	 */
	public function sendEmail() {
		$query_id = $this->input->post ('query_id' );
		if(!($query_id)) {
			$this->errorResponse('Unable to send email, please try again later');
		}
		else {
			$config = Array(
					'protocol' => settings_item('protocol'),
					'smtp_host' => settings_item('smtp_host'),
					'smtp_port' => settings_item('smtp_port'),
					'smtp_user' => settings_item('smtp_user'), // change it to yours
					'smtp_pass' => settings_item('smtp_pass'), // change it to yours
					'mailtype' => 'html',
					'charset' => 'iso-8859-1',
					'wordwrap' => TRUE);
			$emails = $this->business_queries_model->get_email_addresses($query_id);
			header('Content-Type: application/x-json; charset=utf-8');
	
			//do Captcha check, make sure the submitter is not a robot:)...
			if(!$this->check_captcha('email_captcha_code')) {
				$this->errorResponse('Invalid Security Code');
			} else {
				// Send Email
				$this->load->library('email', $config);
				$this->email->from($emails->email_from, $emails->title); // change it to yours
				$this->email->to($emails->email_to, $emails->name);// change it to yours
				$this->email->subject($this->input->post('email_subject'));
				$this->email->message($this->input->post('email_message') .'<hr><br />' .settings_item('site.title') .'<br />' .site_url());
				$success = $this->email->send();
				if($success) {
					echo json_encode ( array (
							'message' => 'Your message has been sent successfully'
					) );
				} else {
					echo json_encode ( array (
							'message' => $this->email->print_debugger()
					) );
				}
			}
		}
	}
	
	/**
	 * Proceed to checkout based on session variables
	 * @param number $user_id
	 * Admin can also create an order or claim a listing on behalf of a user
	 */
	public function checkout() {
	    // $this->auth->restrict('Site.Business.Financial');
	    $user_id = $this->auth->user_id();
	    $package_id = $this->input->get('id');
	    if(!empty($package_id)) {
	        $this->load->library('crypt');
	        $package_id = intval($this->crypt->decode($package_id));
	        $this->load->model('orders/orders_model');
	        // renewal and upgrade
	        $extra['gc_order_type'] = $this->session->userdata('gc_order_type');
	        $this->session->unset_userdata('gc_order_type');
	        $this->orders_model->cart($package_id, 0, 1, 1, 'packages', true, $extra);
	        /*$result = $this->orders_model->free_paid_checkout($package_id, 'packages', $user_id);
	        if(!empty($result['url']) && empty($result['free_checkout'])) {
	            // Template::redirect($result['url']);
	        }*/
	    }	    
	    $this->load->model('orders/orders_model');
	    $this->gc->demo_custom_css_js('member');
	    $this->orders_model->checkout($user_id);
	}
	
	/**
	 * Update listings, images, videos and other modules status
	 * @param int $listing_id
	 * @param int $status
	 */
	private function listing_update($listing_id, $status, $module_statuses = array()) {
		$this->auth->restrict('Site.Business.Owner');
		
		// Update listing status
		$this->db->where('id', $listing_id)->update($this->db->dbprefix . 'listings', array ('status' => $status));
		
		// Update image status
		$this->db->where(array('module_id' => gc_module_id('listings'), 'row_id' => $listing_id))->update('images', array ('status' => $status));
		
		// Update Other Module Status
		$this->listings_model->update_module($listing_id, 'status', $status, $module_statuses);

		// Update listing_categories status
		$this->db->where(array('module_id' => gc_module_id('listings'), 'row_id' => $listing_id))->update('categories_relation', array ('active' => $status));
		
		return true;
	}
	
	/**
	 * Display wallets information to the User
	 * @param number $offset
	 */
	public function wallets($offset = 0) {
		$this->auth->restrict('Site.Business.Financial');
		$this->load->model('wallets/wallets_points_model');
		$this->wallets_points_model->member('wallets', $offset, $this->limit);
	}
	
	/**
	 * Display points information to the User
	 * @param number $offset
	 */
	public function points($offset = 0) {
		$this->auth->restrict('Site.Business.Financial');
		$this->load->model('wallets/wallets_points_model');
		$this->wallets_points_model->member('points', $offset, $this->limit);
	}
	
	/**
	 * Wallets/Points receipt
	 * @param string $module - Wallets/Points
	 * @param number $id - Transaction ID
	 */
	public function receipt($module, $id) {
	    $this->auth->restrict('Site.Business.Financial');
	    $this->load->model('wallets/wallets_points_model');
	    $this->wallets_points_model->email_print($module, $id, 'view');
	    $this->gc->demo_custom_css_js('member');
	    Template::render('member');
	}
	
	/**
	 * Remove member_message_count cache id on deletion of a row
	 * This method is also called on index page in order to delete with unexpected add rows anomalies
	 */
	private function remove_count_cache_id() {
		$this->load->library('gzf_cache', array('lifetime' => 900));
		//$z_cache = $this->gzf_cache->get_instance();
		$this->gzf_cache->remove('member_area_'.$this->auth->user_id());
	}
	
	/**********************************************************/
	/* START OF MODULES
	 **********************************************************/
	// View Module
	public function view() {
		$this->auth->restrict('Site.Business.Owner');
		$this->load->library('crypt');
		$this->load->library('users/auth');
		$redirect_url = gc_url_format('members');
		
		$offset = intval($this->input->get('per_page'));
		$id = intval($this->crypt->decode($this->input->get('id')));
		$md = $this->input->get('md');
		$module = !in_array($md, gc_modules()) ? $this->crypt->decode($md, true) : $md;
		$toggle_module = $this->crypt->decode($this->input->get('tm'), true);
		
		// redirect back and forth between pages
		$current_url = current_url() . (!empty($_SERVER['QUERY_STRING']) ? '?' . $_SERVER['QUERY_STRING'] : $_SERVER['QUERY_STRING']);
		$this->session->set_userdata($module . '_last_visited_page', $current_url);
		
		// Member Area Only - Allow user to delete a record
		if (isset ( $_POST ['delete'] )) {
			$checked = $this->input->post ( 'checked' );
			if (is_array ( $checked ) && count ( $checked )) {
				$result = FALSE;
				foreach ( $checked as $pid ) {
					$result = $this->db->where('id', $pid)->update($module, array('deleted' => 1));
					// Also set the Business Queries, Favorites and Comments status to deleted
					$this->gc->module_soft_delete($module, $pid);
				}
				if ($result) {
					$counts = count ( $checked );
					if($counts > 1) {
						$message_string = lang('singular_' . $module);
					} else {
						$message_string = lang($module);
					}
					Template::set_message ( count ( $checked ) . ' ' . sprintf(lang ( 'msg_delete_success' ), $message_string), 'success' );
					Template::redirect(current_url().'?'.http_build_query($_GET));
				} else {
					Template::set_message ( sprintf(lang ( 'msg_delete_failure' ), $message_string), 'error' );
				}
			}
		}
		// Booking Forms
		$this->load->model('bookings/bookings_model');
		$this->bookings_model->booking_link();
		if(isset($_POST['booking'])) {
		    $this->bookings_model->view_forms($module, $_POST['booking']);
		}
		$this->gc->view('members', $module, $this->auth->user_id(), $redirect_url, $id, $toggle_module, $id, $offset, $this->limit);
	}
	
	/**
	 * Add Module Information
	 */
	public function add() {
		$this->auth->restrict('Site.Business.Owner');
		$this->load->library('crypt');
		$all_modules = gc_modules();
		$md = $this->input->get('md');
		$module = !in_array($md, $all_modules) ? $this->crypt->decode($md, true) : $md;
		$module_info = $this->gcache->get('modules', array('where' => "name = '" . $module . "'"));
		if(!$module_info) {
		    Template::redirect(gc_url_format('members'));
		}
		if(!empty($module_info->group_form) && $module_info->group_form == 1) {
		    // add module in group format
		    $this->load->library('fields/group_fields');
		    $this->group_fields->group_form('frontend', $module, array());
		} else {
    		$id = intval($this->crypt->decode($this->input->get('id')));
    		$tm = $this->input->get('tm');
    		$toggle_module = !in_array($tm, $all_modules) ? $this->crypt->decode($tm, true) : $tm;
    		if(gc_module_id($module) == 0) {
    			Template::redirect(gc_url_format('members'));
    		}
    		$redirect_url = $this->session->userdata($module . '_last_visited_page'); // gc_url_format('members/view');
    		// The $id variable holds the listing_id/row_id value, based on the type of the module
    		// e.g. in case of Videos $id will be treated as a row_id whereas in other modules it will work as a listing_id
    		$this->gc->module('members', $module, $this->auth->user_id(), $redirect_url, $id, 0, $toggle_module, $id);
    		$toolbar_title = lang('add') . ' ' . lang('singular_' . $module);
    		Template::set(array(
    			'elements_container' => 'div.card.card-body', // wrap form elements within a card container
    			'member_area' => TRUE, // set this variable to TRUE for QUERY STRING FORM else the SAVE button will redirect without saving.
    			'page_title' => $toolbar_title,
    			'toolbar_title' => $toolbar_title
    		));
    		Template::set_view('fields/form');
    		Template::render('member');
		}
	}
	
	/**
	 * Edit Module
	 */
	public function edit() {
		$this->auth->restrict('Site.Business.Owner');
		$this->load->library('crypt');
		$all_modules = gc_modules();
		$md = $this->input->get('md');
		$module = !in_array($md, $all_modules) ? $this->crypt->decode($md, true) : $md;
		$module_info = $this->gcache->get('modules', array('where' => "name = '" . $module . "'"));
		if(!$module_info) {
		    Template::redirect(gc_url_format('members'));
		}
		if(!empty($module_info->group_form) && $module_info->group_form == 1) {
		    // edit module in group format
		    $this->load->library('fields/group_fields');
		    $this->group_fields->group_form('frontend', $module, array());
		} else {    		
    		$id = intval($this->crypt->decode($this->input->get('id')));				
    		$redirect_url = gc_url_format('members/view');
    		$this->gc->module('members', $module, $this->auth->user_id(), $redirect_url, 0, $id);
    		$toolbar_title = lang('edit') . ' ' . lang('singular_' . $module);
    		Template::set(array(
    			'elements_container' => 'div.card.card-body', // wrap form elements within a card container
    			'member_area' => TRUE, // set this variable to TRUE for QUERY STRING FORM else the SAVE button will redirect without saving.
    			'page_title' => $toolbar_title,
    			'toolbar_title' => $toolbar_title
    		));
    		Template::set_view('fields/form');
    		Template::render('member');
		}
	}
	
	// Videos
	public function videos($offset = 0) {
		$this->auth->restrict('Site.Business.Owner');
		Assets::add_module_js('videos', 'videos.js');
		Assets::add_js('js/bootstrap.youtubepopup.min.js');
		$this->gc->view($this->limit);
	}
	/**********************************************************/
	/* END OF MODULES
	 **********************************************************/	
	/**
	 * Display custom pages to the User in the Member Area
	 * @param string $slug
	 */
	public function pages($slug) {
		$this->auth->restrict('Site.Member.Pages');
		$this->load->model('pages/pages_model');
		$this->pages_model->detail($slug, 'member');	
	}
	
	/**
	 * Allow User to Edit Profile
	 */
	public function profile() {
		$this->load->model('users/user_model');
		$this->user_model->add_edit_user('members', $this->auth->user_id(), $this->auth->user_id());
	}
	
	/**
	 * Allow user to change the ROLE from the USER to BUSINESS OWNER
	 */
	public function role() {
	    if($this->auth->role_id() != 4) {
	        Template::redirect(gc_url_format('members'));
		}
		if(isset($_POST['save'])) {		    
		    $this->db->where('id', $this->auth->user_id())->update('users', array('role_id' => 2));
		    Template::set_message(lang('role_update_success'), 'success');
		    Template::redirect(gc_url_format('members'));
		}
		Template::set(array(
		    'toolbar_title' => lang('update_role_text')
		));
		$this->gc->demo_custom_css_js('member');
		Template::render('member');
	}
	
	/**
	 * Allow users to add the Category Fields information
	 */
	public function category() {
	    $id = intval($this->crypt->decode($this->input->get('id')));
	    $module = $this->crypt->decode($this->input->get('md'), true);
	    $user_id = $this->auth->user_id();
	    $this->load->model('fields/fields_model');
	    $this->fields_model->save_categories_meta('member', $module, $id, $user_id);
	}
	
	/*************************
	 * START OF ECOMMERCE
	 *************************/
	/**
	 * Allow member to add/edit/view variants
	 * @param number $action - 0 -> VIEW, 1 -> ADD, 2 - > EDIT
	 * @param number $offset
	 */
	public function variants($action = 0, $id = 0, $offset = 0) {
	    $id = (int) $id;
	    if(!$this->session->userdata('variant_product_id')) {
	        Template::redirect(gc_url_format('members'));
	    }
	    
	    // check permissions
	    $product = $this->db->select('user_id, listing_id')->get_where('products', array('id' => $this->session->userdata('variant_product_id')))->row();
	    if(!$product && $product->user_id != $this->auth->user_id()) {
	        Template::redirect(gc_url_format('members'));
	    }
	    $this->lang->load('ecommerce/ecommerce');
	    $this->load->model('ecommerce/ecommerce_model');
	    
	    // Allow user to redirect back to products
	    $module_links = '<li class="nav-item">' . anchor(gc_url_format('members/view') . '?id=' . $this->crypt->encode($product->listing_id) . '&md=' . $this->crypt->encode('products'), lang('back_to_products'), array('title' => lang('back_to_products'), 'class' => 'nav-link')) . '</li>';
	    switch($action) {
	        case 0:
	            $module_links .= '<li class="nav-item">' . anchor(gc_url_format('members/variants') . '/1/' . $this->session->userdata('variant_product_id'), lang('add_variant'), array('title' => lang('add_variant'), 'class' => 'nav-link active')) . '</li>';
	            $this->ecommerce_model->view('member', $offset, $this->limit);
	            break;
	        case 1:
	            if($id == 0) {
	                Template::redirect(gc_url_format('members'));
	            }
	            $create_anchor_class = 'nav-link active';
	            $this->ecommerce_model->variant($id, 'insert', 'member');
	            break;
	        case 2:
	            if($id == 0) {
	                Template::redirect(gc_url_format('members'));
	            }
	            $edit_anchor_class = 'nav-link active';
	            $this->ecommerce_model->variant($id, 'update', 'member');
	            break;
	        default:
	            Template::redirect(gc_url_format('members'));
	            
	    }
	    Template::set('module_links', $module_links);
	    $this->gc->demo_custom_css_js('member');
	    Template::render('member');
	}
	
	/**
	 * Display Ordered Items to the Member
	 * @param number $offset
	 */
	public function ordered_items($is_seller = 1, $offset = 0) {
	    // This method has no View in the Themes Module, i.e. load layout files manually
	    $this->session->set_userdata('order_items_last_visited_page', current_url()); // cancellation and track redirect
	    $this->gc->demo_custom_css_js('member');
	    $this->load->model('orders/ordered_items_model');
	    $this->ordered_items_model->view('member', $this->auth->user_id(), $offset, $this->limit, $is_seller);
	    Template::render('member');
	}
	
	/**
	 * Allow member to view and change Shipments
	 * @param string $order_item_id
	 */
	public function shipments($order_item_id) {
	    // This method has no View in the Themes Module, i.e. load layout files manually
	    $this->gc->demo_custom_css_js('member');
	    $this->load->model('orders/ordered_items_model');
	    $this->ordered_items_model->shipments('member', $order_item_id, $this->session->userdata('order_items_last_visited_page'));
	    Template::render('member');
	}
	
	/**
	 * Allow user to track the ordered items using GET parameters
	 * @param integer $item_id - Ordered Item ID
	 * @param string $order_id
	 */
	public function track() {
	    $order_item_id = null;
	    $order_id = null;
	    if(isset($_GET['item_id'])) {
	        $order_item_id = $_GET['item_id'];
	    }
	    if(isset($_GET['order_id'])) {
	        $order_id = $_GET['order_id'];
	    }
	    // This method has no View in the Themes Module, i.e. load layout files manually
	    $this->gc->demo_custom_css_js('member');
	    $this->load->model('orders/ordered_items_model');
	    $this->ordered_items_model->track($order_item_id, $order_id, $this->session->userdata('order_items_last_visited_page'), 'frontend');
	    Template::render('member');
	}
	
	
	/**
	 * Order Cancellations - Products only
	 * @param integer $item_id
	 */
	public function cancellation($item_id) {
	    $this->load->model('orders/cancellations_model');
	    $this->cancellations_model->form('order_items', $item_id, 0, 'member');
	    Template::set('cancel_url', $this->session->userdata('orders_last_visited_page'));
	    $this->gc->demo_custom_css_js('member');
	    Template::render('member');
	}
	
	/**
	 * Display wishlists to the user
	 * @param number $offset
	 */
	public function wishlists($offset = 0) {
	    $this->lang->load('ecommerce/ecommerce');
	    $redirect_url = gc_url_format('members/wishlists');
	    // This method has no View in the Themes Module, i.e. load layout files manually
	    $this->gc->demo_custom_css_js('member');
	    $this->load->model('ecommerce/ecommerce_model');
	    $this->ecommerce_model->wishlist_notify_me('wishlists', $redirect_url, $offset, $this->limit);
	}
	
	/**
	 * Display notifications to the user
	 * @param number $offset
	 */
	public function notifications($offset = 0) {
	    $this->lang->load('ecommerce/ecommerce');
	    $redirect_url = gc_url_format('members/notifications');
	    // This method has no View in the Themes Module, i.e. load layout files manually
	    $this->gc->demo_custom_css_js('member');
	    $this->load->model('ecommerce/ecommerce_model');
	    $this->ecommerce_model->wishlist_notify_me('notifications', $redirect_url, $offset, $this->limit);
	}
	
	/**
	 * Display Payouts to the user
	 * @param number $offset
	 */
	public function payouts($offset = 0) {
	    $this->session->set_userdata('member_payouts_last_visited_page', current_url());
	    $filter_type = 'all';
	    $this->auth->restrict('Site.Business.Financial');
	    $this->load->model('payouts/payouts_model');
	    $this->lang->load('payouts/payouts');
	    
	    Assets::add_js( array(
	        'jquery.dataTables.min',
	        'dataTables.bootstrap.min',
	    ));
	    Assets::add_css('dataTables.bootstrap.min');
	    Assets::add_js("$('#payouts_history').DataTable();", 'inline');
	    Assets::add_js("$('#payouts_earning').DataTable();", 'inline');
	    
	    $user_id = $this->auth->user_id();
	    $user_payouts = $this->payouts_model->get_user_payouts($user_id, 0);
	    if (isset($_POST['request'])) {
	        $module_id = $_POST['request'];
	        $payout = new stdClass();
	        if($user_payouts) {
	            foreach($user_payouts as $user_payout) {
	                if($user_payout->module_id == $module_id) {
	                    $payout = $user_payout;
	                }
	            }
	        }
	        if($payout) {
	            $reference_no = $this->payouts_model->payout_request($user_id, $module_id, $payout->pending_payout);
	            if($reference_no) {
	                Template::set_message(sprintf(lang('payout_request_success'), $reference_no), 'success');
	            } else {
	                if($payout->pending_payout < settings_item('fin.minimum_payout')) {
	                    Template::set_message(sprintf(lang('minimum_payout_error'), currency_format(decimal_convert(settings_item('fin.minimum_payout'))), currency_format(decimal_convert((floatval(settings_item('fin.minimum_payout')) - floatval($payout->pending_payout))))), 'error');
	                } else {
	                    Template::set_message(lang('payout_already_requested'), 'error');
	                }
	            }
	            Template::redirect(current_url());
	        }
	    }
	    $where = array('user_id' => $user_id);
	    $this->load->model('affiliates/affiliates_model');
	    $this->affiliates_model->logs(10, $user_id, 'member', array(), false, false);
	    Template::set(array(
	        'filter_type' => $filter_type,
	        'records' => $user_payouts,
	        'toolbar_title' => lang('my_payouts')
	    ));
	    // Affiliate View has no View in the Themes Module, i.e. load layout files manually
	    Assets::add_js('member.js');
	    Assets::add_css('member.css');
	    Template::set_view('payouts/view');
	    Template::render('member');
	}
	
	/**
	 * View/PDF Invoice
	 */
	public function invoice() {
	    $this->lang->load('payouts/payouts');
	    if(empty($_GET['id'])) {
	        die(lang('payout_invoice_error'));
	    }
	    $payout_id = $_GET['id'];
	    $view = isset($_GET['view']) ? $_GET['view'] : 0;
	    $this->load->library('crypt');
	    $payout_id = (int) $this->crypt->decode($payout_id);
	    if(empty($payout_id)) {
	        die(lang('payout_invoice_error'));
	    }
	    $this->lang->load('orders/orders');
	    $this->load->model('payouts/payouts_model');
	    $this->load->model('commissions/commissions_model');
	    Assets::add_css('member.css');
	    Assets::add_css('_member.css');
	    Assets::add_js('member.js');
	    Assets::add_js('_member.js');
	    $this->payouts_model->email_print($payout_id, $view == 1 ? 'view' : 'pdf', array(), 'member');
	}
	/*************************
	 * END OF ECOMMERCE
	 *************************/
	/**
	 * Display Invitation Code to the User
	 */
	public function invitation_coupons() {
	    $this->lang->load('coupons/coupons');
	    Assets::add_js('clipboard.min.js');
	    Assets::add_js('new ClipboardJS(".copy-text");', 'inline');
	    Assets::add_css('jssocials.css');
	    Assets::add_js('jssocials.min.js');
	    Assets::add_js(
	        '$(".sharing").each(function() {
			$(this).jsSocials({
				url : $(this).data("url"),
				text: $(this).data("txt"),
				showLabel : false,
				shares: ["email", "twitter", "facebook", "googleplus", "linkedin", "pinterest", "stumbleupon", "whatsapp"]
			});
		});', 'inline'
	        );
	    $affiliate_id = $this->auth->user_id();
	    $records = $this->db->get_where('coupons', array('invitation_user_id' => $affiliate_id))->result();
	    Template::set(array(
	        'records' => $records,
	        'toolbar_title' => lang('my_invitations')
	    ));
	    // This View has no View in the Themes Module, i.e. load layout files manually
	    $this->gc->demo_custom_css_js('member');
	    Template::render('member');
	}
	
	/**
	 * Display posted requirements to the user
	 */
	public function my_requirements($offset = 0) {
	    $user_id = $this->auth->user_id();
	    if(isset($_POST['update_status'])) {
	        $lead_id = (int) $_POST['update_status'];
	        $this->db->where('id', $lead_id)->update('leads', array('status_id' => 1, 'modified_on' => date('Y-m-d H:i:s'), 'modified_ip' => $this->input->ip_address()));
	        Template::redirect(current_url());
	    }
	    $this->load->model('leads/leads_model');
	    $limit = $this->limit;
	    $records = $this->leads_model->get_leads($offset, $limit, null, 'CN.deleted = 0 AND CN.user_id = ' . $user_id, "CN.id DESC");
	    $total_rows = $this->leads_model->get_leads($offset, $limit, null, 'CN.deleted = 0 AND CN.user_id = ' . $user_id, "CN.id DESC", 1);
	    /*Assets::add_js( array(
	        'jquery.dataTables.min',
	        'dataTables.bootstrap.min',
	    ));
	    Assets::add_css('dataTables.bootstrap.min');
	    Assets::add_js("$('#myRequirements').DataTable();", 'inline');*/
	    $pagerUriSegment = 3;
	    $pagerBaseUrl = gc_url_format('members/my_requirements');
	    
	    $this->load->library('pagination');
	    $pager['base_url']    = $pagerBaseUrl;
	    $pager['total_rows']  = $total_rows;
	    $pager['per_page']    = $this->limit;
	    $pager['uri_segment'] = $pagerUriSegment;
	    
	    $this->pagination->initialize($pager);
	    Template::set(array(
	        'records' => $records,
	        'toolbar_title' => lang('my_requirements')
	    ));
	    // This View has no View in the Themes Module, i.e. load layout files manually
	    $this->gc->demo_custom_css_js('member');
	    Template::render('member');
	}
	
	/**
	 * Allow user to upload documents
	 * @param string $module
	 * @param integer $row_id
	 */
	public function documents($module, $row_id) {
	    $this->load->library('goclixy/gfile');
	    $this->gfile->upload_files($module, $row_id);
	    $this->gc->demo_custom_css_js('member');
	    Template::render('member');
	}
}