<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class Members_model extends BF_Model {
    protected $gc_members_cache;
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
        // Load Zend Caching
        $this->load->library('slug');
        $this->load->library('gzf_cache', array('lifetime' => 900));
        if(method_exists($this->gzf_cache, 'get_instance')) {
            $this->gc_members_cache = $this->gzf_cache->get_instance();
        }
    }
    
    /**
     * Get order details
     * @return array
     */
    public function get_order_details($user_id) {
        $where = "OT.module_id = 20 AND OT.user_id = ". $user_id ." AND OT.status_id = 2 AND (OT.expires_on >= '" .date('Y-m-d') ."' OR OT.expires_on IS NULL)"; //  OR OT.expires_on = '' // not working on the latest versions
        $select = "O.id, O.order_id, OT.expires_on, OT.row_id AS package_id, OT.item, P.title AS package_title, P.price, P.duration, P.description AS package_description, P.is_default, P.trusted_seal, P.icon, P.highlight, P.commissions,";
        $select .= " P.module_limit, P.module_images_limit, P.module_videos_limit, P.module_categories_limit, P.module_status, P.keywords_limit, P.pyg, P.days_before_next_invoice, P.module_images_status, P.module_videos_status, P.module_featured";
        return $this->orders_model->get_user_orders(0, $user_id, array('where' => $where, 'select' => $select, 'return' => 'result_array'));
    }
    
    /**
     * Get expired order details for a particular user
     * @return array
     */
    public function get_expired_order_details($user_id) {
        $select = "O.id, O.order_id, OT.expires_on, OT.row_id AS package_id, OT.item, P.price, P.duration";
        $where = " OT.module_id = 20 AND OT.user_id = ". $user_id ." AND (OT.status_id = 8 OR (OT.expires_on IS NOT NULL AND OT.expires_on < '" .date('Y-m-d') ."'))"; // AND OT.expires_on != '' 
        return $this->orders_model->get_user_orders(0, $user_id, array('where' => $where, 'select' => $select, 'return' => 'result_array'));
    }
	
    /**
     * Display messages on dashboard to the user after login
     * @param integer $user_id
     */
    public function member_dashboard($user_id, $mobile = false) {
        $this->load->model('listings/listings_model');
        $result = $this->get_order_details($user_id);
        $message = array();
        $redirect_url = NULL;
        $purchased_packages = array(); // do not display the same package twice
        if($result) {
            $i = 1;
            foreach($result as $package_info) {
                if(!in_array($package_info['package_id'], $purchased_packages) && $i == 1) { // do not display the same package twice
                    $purchased_packages[] = $package_info['package_id'];
                    $item_title = ordered_item_info($package_info['item']);
                    // display package expiration to the member
                    $today_date=new DateTime(date("Y-m-d"));
                    $expires_on = new DateTime($package_info['expires_on']);
                    $diff = $expires_on->diff($today_date)->format("%a"); // calculate date difference in days
                    $count = $this->listings_model->count_by(array ('user_id' => $user_id, 'package_id' => $package_info['package_id'], 'deleted' => 0));
                    if(!empty($package_info['expires_on']) && $diff < $package_info['days_before_next_invoice'] ) {
                        if(!$mobile) {
                            $redirect_url = '<a href="'. gc_url_format('members/renew') . '?id='.$this->crypt->encode($package_info['package_id']).'&order_id='.$this->crypt->encode($package_info['order_id']).'" class="text-white"><strong>'.lang('click_here').'</strong></a>';
                        }
                        $expiring_date = !empty($package_info['expires_on']) ? DateTime::createFromFormat('Y-m-d H:i:s', $package_info['expires_on'])->format('F j, Y') : lang('label_never');
                        $message[] = array(sprintf(lang('msg_package_expiring'), $item_title, $expiring_date, $redirect_url), 'danger');
                    } else {
                        $pyg = json_decode($package_info['pyg'] ?? '');
                        $listings_limit = gc_field_value('listings', $package_info['module_limit']);
                        if($listings_limit == 0) {
                            $listings_limit = lang('no_limit');
                        } else if($listings_limit == 999) {
                            $listings_limit = 0;
                        }
                        $images_limit = gc_field_value('listings', $package_info['module_images_limit']);
                        $images_limit = $images_limit == 999 ? 0 : $images_limit;
                        $categories_limit = gc_field_value('listings', $package_info['module_categories_limit']);
                        $categories_limit = $categories_limit == 999 ? 0 : $categories_limit;
                        $keywords_limit = $package_info['keywords_limit'] == 0 ? lang('no_limit') : $package_info['keywords_limit'];
                        $plimit = json_decode($package_info['module_limit'] ?? '');
                        $package_limits = array();
                        foreach($plimit as $package_info => $key ) {
                            if(gc_module_id($package_info) != 0) {
                                $key_label = $key;
                                switch($key) {
                                    case 0:
                                        $key_label = lang('no_limit');
                                        break;
                                    case 999;
                                    $key_label = 0;
                                    break;
                                }
                                $package_limits[] = $key_label  . '&nbsp' . lang($package_info);
                            }
                        }
                        $limit_message = (!empty($package_limits) ? implode(', ' , $package_limits) . ', ' : '') . $categories_limit .  ' ' . lang('label_categories') . ', ' . $keywords_limit .  ' ' . lang('label_keywords') . ', ' . $images_limit . ' ' . lang('images');                        
                        if($listings_limit == 0 || $pyg->listings > 0) {
                            $message[] = array ( sprintf(lang('msg_dashboard_no_limit'), $count, $item_title, $limit_message, $item_title), 'success');
                        } else {
                            $message[] = array ( sprintf(lang('msg_paid_customer'), $count, $listings_limit, $item_title, $limit_message, $item_title) , ($count < $listings_limit ? 'success' : 'danger'));
                        }                        
                        /*if($listings_limit == 0 || $pyg->listings > 0) {
                            $message[] = array ( sprintf(lang('msg_dashboard_no_limit'), $count, $item_title, $listings_limit, $categories_limit, $keywords_limit, $images_limit, $item_title), 'success');
                        } else {
                            $message[] = array ( sprintf(lang('msg_paid_customer'), $count, $listings_limit, $item_title, $categories_limit, $keywords_limit, $images_limit, $item_title), ($count < $listings_limit ? 'success' : 'error'));
                        }*/
                    }
                }
                $i++;
            }
        }
        
        /**
         * Display expired orders to the user if not paid within a specific time
         */
        if(empty($message)) {
            $expired_orders = $this->members_model->get_expired_order_details($user_id);
            if($expired_orders) {
                $i = 1;
                foreach($expired_orders as $order_renew) {
                    if($i == 1 && !in_array($order_renew['package_id'], $purchased_packages)) { // display one order only
                        $ordered_item = json_decode($order_renew['item'] ?? '', true);
                        $item_title = !empty($ordered_item[0]) ? ordered_item_info($order_renew['item'], (!empty($order_renew['order_type']) ? $order_renew['order_type'] : '')) : '';
                        // $item_title = ordered_item_info($order_renew['item'], $order_renew['order_type']);
                        if(!$mobile) {
                            $redirect_url = '<a href="'. gc_url_format('members/renew') . '?id='.$this->crypt->encode($order_renew['package_id']).'&order_id='.$this->crypt->encode($order_renew['order_id']).'"><strong>'.lang('click_here').'</strong></a>';
                        }
                        $expires_on = explode(' ', $order_renew['expires_on'] ?? '');
                        if(!empty($expires_on[0])) {
                            $datetime = DateTime::createFromFormat('Y-m-d', $expires_on[0]);
                            $datetime = $datetime ? $datetime->format('F j, Y') : null;
                        } else {
                            $datetime = null;
                        }
                        $message[] = array(sprintf(lang('msg_package_expired'), $item_title, $datetime, $redirect_url), 'danger');
                    }
                    $i++;
                }
            }
        }
        /**
         * Display success, warning and error messages in order
         */
        if (!empty($message)) {
            $messages = array(
                'success' => array(),
                'warning' => array(),
                'danger' => array()
            );
            foreach($message as $expdate ) {
                $messages[$expdate[1]][] = $expdate;
            }
            $message = array_merge($messages['success'], $messages['warning'], $messages['danger']);
        }
        return $message;
    }
    
    /**
     * Display messages on dashboard to the user after login
     * @param integer $user_id
     */
    public function member_dashboard_updated($user_id, $mobile = false) {
        $this->load->model('listings/listings_model');
        $result = $this->get_order_details($user_id);
        $message = array();
        $redirect_url = NULL;
        $purchased_packages = array(); // do not display the same package twice
        if($result) {
            foreach($result as $package_info) {
                if(!in_array($package_info['package_id'], $purchased_packages)) { // do not display the same package twice
                    $purchased_packages[] = $package_info['package_id'];
                    $item_title = ordered_item_info($package_info['item']);
                    // display package expiration to the member
                    $today_date=new DateTime(date("Y-m-d"));
                    $expires_on = new DateTime($package_info['expires_on']);
                    $diff = $expires_on->diff($today_date)->format("%a"); // calculate date difference in days
                    $count = $this->listings_model->count_by(array ('user_id' => $user_id, 'package_id' => $package_info['package_id'], 'deleted' => 0));
                    if($diff < $package_info['days_before_next_invoice'] ) {
                        if(!$mobile) {
                            $redirect_url = '<a href="'. gc_url_format('members/renew') . '?id='.$this->crypt->encode($package_info['package_id']).'&order_id='.$this->crypt->encode($package_info['order_id']).'" class="text-black"><strong>'.lang('click_here').'</strong></a>';
                        }
                        $expiring_date = !empty($package_info['expires_on']) ? DateTime::createFromFormat('Y-m-d H:i:s', $package_info['expires_on'])->format('F j, Y') : lang('label_never');
                        $message[] = array(sprintf(lang('msg_package_expiring'), $item_title, $expiring_date, $redirect_url), 'warning');
                        
                    } else {
                        $pyg = json_decode($package_info['pyg'] ?? '');
                        $listings_limit = gc_field_value('listings', $package_info['module_limit']);
                        //dump($package_info);
                        if($listings_limit == 0) {
                            $listings_limit = lang('no_limit');
                        } else if($listings_limit == 999) {
                            $listings_limit = 0;
                        }
                        $images_limit = gc_field_value('listings', $package_info['module_images_limit']);
                        $images_limit = $images_limit == 999 ? 0 : $images_limit;
                        $categories_limit = gc_field_value('listings', $package_info['module_categories_limit']);
                        $categories_limit = $categories_limit == 999 ? 0 : $categories_limit;
                        $keywords_limit = $package_info['keywords_limit'] == 0 ? lang('no_limit') : $package_info['keywords_limit'];
                        $plimit = json_decode($package_info['module_limit'] ?? '');
                        $package_limits = array();
                        foreach($plimit as $package_info => $key ) {
                            $key_label = $key;
                            switch($key) {
                                case 0:
                                    $key_label = lang('no_limit');
                                    break;
                                case 999;
                                $key_label = 0;
                                break;
                            }
                            $package_limits[] = $key_label  . '&nbsp' . lang($package_info);
                        }
                        $limit_message = implode(', ' , $package_limits) . ', ' . $categories_limit .  ' ' . lang('label_categories') . ', ' . $keywords_limit .  ' ' . lang('label_keywords') . ', ' . $images_limit . ' ' . lang('images');
                        if($listings_limit == 0 || $pyg->listings > 0) {
                            $message[] = array ( sprintf(lang('msg_dashboard_no_limit'), $count, $item_title, $limit_message, $item_title), 'success');
                        } else {
                            $message[] = array ( sprintf(lang('msg_paid_customer'), $count, $listings_limit, $item_title, $limit_message, $item_title) , ($count < $listings_limit ? 'success' : 'error'));
                        }
                    }
                }
            }
        }
        
        /**
         * Display expired orders to the user if not paid within a specific time
         */
        $expired_orders = $this->members_model->get_expired_order_details($user_id);
        if($expired_orders) {
            foreach($expired_orders as $order_renew) {
                if(!in_array($order_renew['package_id'], $purchased_packages)) {
                    $ordered_item = json_decode($order_renew['item'] ?? '', true);
                    $item_title = !empty($ordered_item[0]) ? ordered_item_info($order_renew['item'], $order_renew['order_type']) : '';
                    // $item_title = ordered_item_info($order_renew['item'], $order_renew['order_type']);
                    if(!$mobile) {
                        $redirect_url = '<a href="'. gc_url_format('members/renew') . '?id='.$this->crypt->encode($order_renew['package_id']).'&order_id='.$this->crypt->encode($order_renew['order_id']).'"><strong>'.lang('click_here').'</strong></a>';
                    }
                    if (date('Y-m-d',strtotime('-1 month')) <= $order_renew['expires_on']) {
                        $message[] = array(sprintf(lang('msg_package_expired'), $item_title, DateTime::createFromFormat('Y-m-d H:i:s', $order_renew['expires_on'])->format('F j, Y'), $redirect_url), 'error');
                    }
                }
            }
        }
        if (!empty($message)) {
            $messages = array(
                'success' => array(),
                'warning' => array(),
                'error' => array()
            );
            foreach($message as $key => $expdate ){
                $messages[$expdate[1]][] = $expdate;
            }
            $message = array_merge($messages['success'], $messages['warning'], $messages['error']);
        }
        return $message;
    }
	
	/**
	 * Display packages to the user based on new and upgrade
	 */
	public function get_packages($plan_type = 'new', $package_id = NULL, $user_id = 0) {
	    $user_packages_only = settings_item('fin.user_packages_only');
	    $user_id = empty($user_id) ? (int) $this->auth->user_id() : $user_id;
	    $cacheID = 'member_packages_' . $plan_type . '_' . (int) $package_id . '_' . $user_id . '_' . $user_packages_only;
	    $cacheID = str_replace('-', '_', $this->slug->create_slug($cacheID));
	    if(!$records = $this->gc_members_cache->load($cacheID)) {
	        // get existing package
	        $not_renewable_packages = array();
	        if(!empty($user_id) && $user_packages_only == 1) {
	            $order_where = "O.user_id = " . $user_id . " AND (O.status_id = 2 OR O.status_id = 8 OR O.status_id = 6 OR O.status_id = 7)"; // do not include upgraded and renewed packages
	            $orders = $this->orders_model->get_user_orders(0, $user_id, array('where' => $order_where, 'return' => 'result'));
	            if($orders) { // show only renewable packages to the user
	                foreach($orders as $order) {
	                    /* if(empty($order->multisale)) { // the package is not renewable
	                     // renewable removed, the multisale is taken as renewable because of hardcode in different files
	                     $not_renewable_packages[$order->package_id] = $order->package_id;
	                     }*/
	                    // once a package is purchased, do not display it in the buy membership page
	                    $not_renewable_packages[$order->package_id] = $order->package_id;
	                }
	                $package_id = $orders[0]->package_id; // user's active package, do not display lowest price packages
	            }
	            $not_renewable_packages = array_unique($not_renewable_packages);
	        }
	        
	        // $sql = "SELECT id, title, description, icon, price, duration, highlight, is_default FROM " . $this->db->dbprefix . "packages";
	        // $sql = " WHERE ((duration = 0) || (duration >= DATEDIFF( '" .$now ."', created_on )) + duration)";
	        // $table_fields = array('id', 'title', 'description', 'icon', 'price', 'duration', 'highlight', 'is_default');
	        $query = $this->translations->build_query('packages'); // , $table_fields
	        $now = date('Y-m-d H:m:s');
	        $sql = $query['select'];
	        $sql .= " FROM " . $this->db->dbprefix . "packages M";
	        if(!empty($query['join'])) {
	            $sql .= $query['join'] . PHP_EOL;
	        }
	        $sql .= " WHERE M.active = 1 AND M.deleted = 0 AND M.plan_type LIKE '". $plan_type ."' AND M.is_user_default = 0";
	        if($plan_type == 'upgrade' && $package_id != NULL) {
	            $sql .= " AND (M.upgrade_valid_for = " .$package_id ." || M.upgrade_valid_for = 0)";
	        }
	        if(!empty($package_id)) {
	            // get only those packages where price is higher than existing package
	            $sql .= " AND (M.price > (SELECT price FROM " . $this->db->dbprefix . "packages WHERE id = ". $package_id ."))";
	        }
	        $sql .= " ORDER BY M.display_order";
	        $query = $this->db->query ( $sql );
	        $records = $query->result_array ();
	        if($records) {
	            foreach($records as $key => $record) {
	                if(in_array($record['id'], $not_renewable_packages)) {
	                    unset($records[$key]);
	                }
	            }
	        }
	        $this->gc_members_cache->save($records, $cacheID, array('packages', 'orders_' . $user_id));
	    }
	    return $records;
	}
	
	/**
	 * Get package detail for add listing and edit listing page to restrict number of keywords, description etc.
	 * @return int $user_id
	 * @return int $package_id
	 */
	public function get_package_info($user_id, $package_id, $listing_id = NULL, $module = 'packages') {
	    $package_id = empty($package_id) ? 0 : $package_id;
	    $common_fields = " P.module_limit, P.module_status, P.module_featured, P.module_categories_limit, P.module_images_limit, P.module_images_status, P.module_videos_limit, P.module_videos_status,";
	    $common_fields .= " P.keywords_limit, P.keywords_length, P.description_limit, P.form_fields, P.title AS package_title, P.description AS package_description";
	    $extra = array(
	        'select' => 'O.id, O.order_id, OT.expires_on, OT.item, OT.price, P.pyg,' . $common_fields,
	        'where' => "OT.user_id = ". $user_id ." AND OT.row_id = ". $package_id ." AND OT.status_id = 2 AND (OT.expires_on >= '" .date('Y-m-d') ."' OR OT.expires_on IS NULL) AND OT.module_id = " . gc_module_id($module) // OT.expires_on = '' OR 
	    );
	    $result = $this->orders_model->get_user_orders(0, $user_id, $extra);
	    if(empty($result) && $listing_id != NULL) {
	        $package_sql = "SELECT P.title AS package_title, P.pyg, '' AS order_id,";
	        $package_sql .= $common_fields;
	        $package_sql .= ", L.expires_on FROM " . $this->db->dbprefix . "packages P";
	        $package_sql .= " LEFT JOIN " . $this->db->dbprefix . "listings L ON L.package_id = P.id AND L.id = ".$listing_id;
	        $package_sql .= " WHERE P.id = " .$package_id;
	        $package_query = $this->db->query ( $package_sql );
	        $result = $package_query->row();
	    }
	    return $result;
	}
	
	/**
	 * Replace Buy Now with Add Business if user is already a member of existing package
	 * @param integer $user_id
	 */
	public function get_order_details_for_package($user_id) {
	    $sql = "SELECT OT.order_id, OT.row_id AS package_id, P.price FROM " . $this->db->dbprefix . "order_items OT";
	    $sql .= " INNER JOIN " . $this->db->dbprefix . "packages P ON P.id = OT.row_id";
	    $sql .= " WHERE OT.module_id = " . gc_module_id('packages') . " AND OT.user_id = " . $user_id . " AND OT.status_id = 2 AND ((OT.expires_on >= '" .date('Y-m-d') ."') OR (OT.expires_on = '0000-00-00' OR OT.expires_on IS NULL))";
	    //		$sql .= " AND ((OP.listings_limit = 0) OR (OP.listings_limit > (SELECT COUNT(" . $this->db->dbprefix . "listings.id) FROM " . $this->db->dbprefix . "listings WHERE " . $this->db->dbprefix . "listings.user_id = ". $user_id ." AND " . $this->db->dbprefix . "listings.package_id = OP.package_id)))";
	    $query = $this->db->query ( $sql );
	    return $query->result_array ();
	}
	
	/**
	 * Listing package price for price comparision when package is upgraded or degraded
	 */
	public function listing_package_price($listing_id) {
		$sql = "SELECT P.id as package_id, P.price FROM (SELECT package_id FROM " . $this->db->dbprefix . "listings WHERE id = ". $listing_id .") L";
		$sql .= " LEFT JOIN " . $this->db->dbprefix . "packages P ON P.id = L.package_id";
		$query = $this->db->query ( $sql );
		foreach($query->result_array () as $row) {
			return $row;
		}		
	}
	
	/**
	 * Pay As You Go - Verification
	 * @params $module
	 * @params $package_id
	 */
	public function pyg_check($module, $user_id, $package_id, $package_info = array(), $row_id = 0, $toggle_module = 'listings', $mobile = false) {
		$this->load->model('packages/packages_model');	
		if(empty($package_info)) {
			$package_info = $this->packages_model->select('module_limit, module_images_limit, module_videos_limit, pyg')->find($package_id);
		}
		if(!$package_info) {
			if($mobile) {
				return array('success' => false);
			} else {
				redirect(gc_url_format('members'));
			}
		}
		
		// in case of a video, a module can have multiple videos
		switch($module) {
			case 'images':
				$user_limit = gc_field_value($toggle_module, $package_info->module_images_limit);
				break;
			case 'videos':
				$user_limit = gc_field_value($toggle_module, $package_info->module_videos_limit);
				break;
			default: // in case of a module, a listing can have multiple modules
				$user_limit = gc_field_value($module, $package_info->module_limit);
		}
		
		// the user is allowed to add unlimited listings or other modules
		if($user_limit == 0) {
			if($mobile) {
				return array('success' => true);
			} else {
				return true;
			}			
		} else {
			// Get module count
			$module_count = $this->module_count($module, $user_id, $package_id, $toggle_module, $row_id);
			
			// the user has can add listings or other modules without any cost
			if($user_limit != 999 && $user_limit > $module_count) {
				if($mobile) {
					return array('success' => true);
				} else {
					return true;
				}
			}
				
			// The user has an option to add the listing or module row by paying the Pay As You Go price
			$pyg = json_decode($package_info->pyg ?? '');
			if($pyg->$module > 0) {
				$module_id = gc_module_id($module);
				// the user already paid the Pay As You Go price
				// check the quantity
				$pyg_ordered = $this->db->select('SUM(qty) AS qty')->where(array('status_id' => 2,  'deleted' => 0, 'module_id' => 53, 'user_id' => $user_id, 'row_id' => $package_id, 'variant_id' => $module_id))->get('order_items')->row();
				if(!empty($pyg_ordered->qty) && ($pyg_ordered->qty + ($user_limit != 999 ? $user_limit : 0)) > $module_count) {
					$limit = $pyg_ordered->qty + ($user_limit != 999 ? $user_limit : 0);
					if($mobile) {
						return array('success' => true);
					} else {
						if($module == 'images') {
					        return $limit;
					    } else {
						    return true;
					    }
					}
				} else {
					if(!$mobile) {
						// do not redirect user, because of image_crud library
					    $redirect_url = gc_url_format('members/checkout'); // $this->gcrud->update_version('v' . $version)
						$this->load->model('orders/orders_model');
						$this->orders_model->cart($package_id, 0, 1, 1, 'pyg', false, array('params' => array('package_id' => $package_id, 'module' => $module, 'qty' => 1)));
						if($module == 'images') {
							return array('price' => $pyg->$module, 'redirect_url' => $redirect_url); // package id is required for redirect
						}
						Template::redirect($redirect_url);
					} else {
					    $result = array(
					        'success' => false,
					        'pyg_module' => $module,
					        'message' => sprintf(lang('msg_module_error'), $package_info->package_title, gc_module_translation($module))
					    );
					    return $result;
					}
				}
			} else {
				if($user_limit == 999) {
					$message = sprintf(lang('msg_module_disabled'), gc_module_translation($module));
					$type = 'error';
				} else {
					$message = sprintf(lang('user_limit'), $user_limit, gc_module_translation($module));
					$type = 'error';
				}
				if(!$mobile) {					
					if($module == 'images') {
						return false;
					} 
					Template::set_message($message, $type);
					if($module == 'listings') {
						redirect(gc_url_format('members'));
					} else {
						$this->load->library('crypt');
						$redirect_url = gc_url_format('members/view') . '?';
						if($row_id != 0) {
							$redirect_url .= 'id='.$this->crypt->encode($row_id) . '&';
						}
						$redirect_url .= 'md=' . $this->crypt->encode($module, true) . ($module == 'videos' ? '&tm=' . $this->crypt->encode($toggle_module, true) : '');
						Template::redirect($redirect_url);
					}					
				} else {
						$result = array(
							'success' => false,
							'message' => $message,
							'type' => $type,
							'url' => ($module == 'listings' ? 'app.dashboard' : 'app.listingMenu'),
							'row_id' => $row_id
						);
						return $result;
				}
			}
		}
		if($mobile) {
			return array('success' => false);
		} else {
			return false;
		}
	}
	
	/**
	 * Display module row count in the success message
	 * @param string $module
	 * @param integer $user_id
	 * @param integer $package_id
	 * @param string $module
	 * @param string $toggle_module
	 * @return number
	 */
	public function module_count($module, $user_id, $package_id, $toggle_module, $row_id) {
		$module_count = 0;
		switch($module) {
			case 'videos':
			case 'images':
				$toggle_module_id = gc_module_id($toggle_module);
				$module_count = $this->db->where(array('deleted' => 0, 'row_id' => $row_id, 'module_id' => $toggle_module_id))->count_all_results($module);
				break;
			case 'listings':
				$module_count = $this->listings_model->count_by(array('deleted' => 0, 'user_id' => $user_id, 'package_id' => $package_id));
				break;
			default:
			    $module_count = $this->db->where(array('deleted' => 0, 'user_id' => $user_id, 'package_id' => $package_id))->count_all_results($module);
				break;
		}
		return $module_count;
	}
	
	/**
	 * PYG View
	 * @param string $module
	 * @param number $user_id
	 * @param number $package_id
	 * @param number $user_limit
	 * @return number
	 */
	public function pyg_view($module, $user_id, $package_id, $user_limit) {
	    $module_id = gc_module_id($module);
	    // the user already paid the Pay As You Go price
	    // check the quantity
	    // $pyg_ordered = $this->orders_model->select('SUM(qty) AS qty')->find_by(array('status' => 1,  'deleted' => 0, 'order_type' => 4, 'user_id' => $user_id, 'package_id' => $package_id, 'module_id' => $module_id));
	    // SELECT SUM(qty) AS qty FROM `rs0d_orders` WHERE `status` = 1 AND `deleted` =0 AND `order_type` = 4 AND `user_id` = 1 AND `package_id` = '3' AND `module_id` = '2'
	    $this->load->model('orders/orders_model');
	    $where = "OT.status_id = 2 AND OT.module_id = 53 AND OT.row_id = " . $package_id . " AND OT.user_id = " . $user_id . " AND OT.params LIKE '%" . $module . "%'";
	    $pyg_ordered = $this->orders_model->get_user_orders(0, $user_id, array('where' => $where, 'select' => 'SUM(qty) AS qty'));
	    if($pyg_ordered) {
	        return $pyg_ordered->qty + ($user_limit != 999 ? $user_limit : 0);
	    } else {
	        return 0;
	    }
	}
	
	/**
	 * Member Area/Mobile - Return Counts 
	 * view_comments, my_comments, view_business_queries, posted_queries and favorites 
	 * @param integer $user_id
	 * @return multitype:NULL
	 */
	public function fcb_counts($user_id, $method) {
		$modules = gc_modules();
		$modules[] = 'videos';
		$counts = array();
		if(!empty($modules)) {
			foreach($modules as $config_module) {
				switch($method) {
					case 'my_comments':
						$counts[$config_module] = intval($this->comments_model->count_by(array('user_id' => $user_id, 'module_id' => gc_module_id($config_module), 'deleted' => 0)));
						break;
					case 'view_comments' :
						$counts[$config_module] = intval($this->comments_model->count_view_comments($user_id, $config_module));
						break;
					case 'view_business_queries':
						$counts[$config_module] = intval($this->business_queries_model->count_business_queries($user_id, $config_module));
						break;
					case 'posted_queries':
						$counts[$config_module] = intval($this->business_queries_model->count_posted_queries($user_id, $config_module));
						break;
					case 'favorites':
						$counts[$config_module] = $this->favorites_model->count_by(array('user_id' => $user_id, 'module_id' => gc_module_id($config_module), 'deleted' => 0));
						break;
				}		
			}
		}
		return $counts;
	}
	
	/**
	 * Return Records
	 * view_comments, my_comments, view_business_queries, posted_queries and favorites 
	 * @param integer $user_id
	 * @param string $method
	 * @param string $table
	 * @return multitype:unknown
	 */
	public function fcb_records($user_id, $method, $table, $module, $row_id, $offset, $limit, $mobile = false) {
	    // Actions done, now display the view
	    // T.user_id = " . $user_id . " AND
	    // if($table != 'favorites') {
	    $where = "T.deleted = 0";
	    // }
	    switch($method) {
	        case 'my_comments':
	        case 'posted_queries':
	        case 'favorites':
	            $where .= " AND T.user_id = " . $user_id;
	            break;
	        case 'view_business_queries':
	        case 'view_comments':
	            if(!empty($row_id)) {
	                $where .= " AND T.row_id = " . $row_id;
	            }
	            break;
	    }
	    
	    $sort_by = "T.id DESC";
	    $select = array();
	    if($table != 'favorites') {
	        if($method != 'my_comments' && $method != 'posted_queries') {
	            /*if($module == 'listings') {
	             $where .= " AND M.user_id = $user_id";
	             } else {
	             $where .= " AND L.user_id = $user_id";
	             }*/
	            $where .= " AND M.user_id = $user_id";
	        }
	        $select[] = "U.display_name AS user, U.profile_image";
	        if($table == 'business_queries') {
	            $select[] = " (SELECT COUNT(*) FROM " . $this->db->dbprefix . "query_replies QR WHERE QR.query_id = T.id) AS replies";
	            $select[] = " (SELECT GROUP_CONCAT(DISTINCT(CONCAT(meta_id, '!~!', meta_key, '!~!', meta_value)) SEPARATOR '!:::!') FROM " . $this->db->dbprefix . "metas WHERE module_id = " . gc_module_id('business_queries') . " AND row_id = T.id) AS metas";
	        }
	    }
	    // if($mobile) { // Mobile App Specific
	        if($table == 'favorites') {
	            $select_fields = "M.title, M.slug, M.description, M.ratings AS average_rating, M.cover";
	            // save tweak location fields
	            if(gc_module_settings($module, 'locations') == 1) {
	                $select_fields .= ", M.address, M.city, M.state, M.country, M.postcode";
	            }
	            if($module == 'listings') {
	                $select_fields .= ", M.contact_person, M.phone_number";
	            }
	            $select[] = $select_fields;
	        } else if($table == 'comments') {
	            $select[] = "M.title AS module_title";
	        } else if($table == 'business_queries') {
	            $select[] = "M.title";
	        }
	    // }
	    $join = array();
	    if($table == 'comments') {
	        $join = array("INNER JOIN ". $this->db->dbprefix ."users U ON U.id = T.user_id");
	    } else if($table == 'business_queries') {
	        $join = array("LEFT JOIN ". $this->db->dbprefix ."users U ON U.id = T.user_id");
	    }
	    
	    $records = $this->gc->get_module_rows(0, $table, $module, $where, $sort_by, $offset, $limit, NULL, array(), $select, $join);
	    // dump($this->db->last_query());
	    $total_records = $this->gc->get_module_rows(1, $table, $module, $where, $sort_by, $offset, $limit, NULL, array(), $select, $join);
	    
	    return array('records' => $records, 'total_records' => $total_records);
	}
}