<?php defined('BASEPATH') || exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Online Software Solutions							   # ||
|| # Copyright (C) 2021 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Components controller
 */
class Components extends Admin_Controller
{
    protected $permissionCreate = 'Messages.Components.Create';
    protected $permissionDelete = 'Messages.Components.Delete';
    protected $permissionEdit   = 'Messages.Components.Edit';
    protected $permissionView   = 'Messages.Components.View';
    protected $context			        = 'components';
    protected $moduleName               = 'messages';
    protected $navUrl = '/components/messages';
    protected $table_subscribers = 'subscriber_users';

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
        
        $this->auth->restrict($this->permissionView);
        $this->load->model('messages/messages_model');
        $this->lang->load('messages');
        
      	$this->form_validation->set_error_delimiters("<span class='error'>", "</span>");
        
        Template::set_block('sub_nav', 'components/_sub_nav');
        Assets::add_module_js('messages', 'messages.js');
    }

    /**
     * Display a list of messages data.
     *
     * @return void
     */
    public function index($filter = 'all', $offset = 0) {
        $this->messages_model->view('queue', $filter, $offset, 'admin', array('redirect_uri' => '/components/messages/index'));
    }
	
	/**
	 * Send SMS
	 */
	public function send_sms($from = 0, $to = 0, $template_id = 2) {
		$records = $this->db->limit(450)->get_where('message_subscribers', array('status' => 1, 'sent_flag' => 0))->result();
		if($records) {
			$template = $this->db->where('id', $template_id)->get('message_templates')->row();
			if($template) {
				$message = $template->message;
				$data = array();
				$flag_data = array();
				$replace_strings = array('+91-', '91 ', '91-', '+');
				foreach($records as $record) {
					$flag_data[] = array('id' => $record->id, 'sent_flag' => 1);
					$phone_number = $record->phone_number;
					foreach($replace_strings as $replace_string) {
						$phone_number = str_replace($replace_string, '', $phone_number);
					}
					if(strlen($phone_number) == 10) {
						$data[] = array(
							'template_id' => $template_id,
							'subscriber_id' => $record->id,
							'message' => $message,
							'phone_number' => $phone_number,
							'created_on' => date('Y-m-d H:i:s')
						);
					}
				}
				$this->db->insert_batch('message_sent', $data);
				$this->db->update_batch('message_subscribers', $flag_data, 'id');
			}
		}
	}	
	
	public function test_queue() {
		$template_id = 1;
		$phone_number = '9999999999';
		$template = $this->db->where('id', $template_id)->get('message_templates')->row();
		if($template) {
			$message = $template->message;
			$data = array(
				'template_id' => $template_id,
				'subscriber_id' => 0,
				'message' => $message,
				'phone_number' => $phone_number,
				'status' => 0,
				'created_on' => date('Y-m-d H:i:s')
			);
			$this->db->insert('message_sent', $data);
			$this->process_queue();
		}
	}
	
	/**
	 * Display a list of Messages data
	 * @param string $module
	 * @param string $filter
	 * @param number $offset
	 */
	public function manage($module = 'templates', $filter = 'all', $offset = 0) {
	    $this->messages_model->view($module, $filter, $offset);
	}
	
	/**
	 * Add/Edit SEO Data
	 *
	 * @return void
	 */
	public function form($module = 'templates', $action = 'create') {
	    $params = array(
	        'model' => 'messages',
	        'context' => $this->context,
	        'fields_method' => $module,
	        'set_table' => true,
	        'uri_segment' => 7,
	        'redirect_url' => site_url(SITE_AREA . '/' . $this->context . '/' . $this->moduleName . '/manage/' . $module)
	    );
	    if($module == 'links') {
	        // save link relations
	        $params['method'] = 'after_link_save';
	    }
	    $this->gcrud->form('message_' . $module, $action, 'admin', $params);
	}
	
	/**
	 * Send Messages
	 */
	public function send($module = 'subscriber_users', $id = 0, $offset = 0) {
	    $this->load->model('subscribers/subscribers_model');
	    $this->subscribers_model->send('message', $id, $this->limit, $offset, $module);
	}
}