<?php defined('BASEPATH') || exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Online Software Solutions							   # ||
|| # Copyright (C) 2021 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class Messages_model extends BF_Model
{
    public $table_name	         = 'message_queue';
    protected $table_templates   = 'message_templates';
    protected $table_sent        = 'message_sent';
	protected $key			= 'id';
	protected $date_format	= 'datetime';

	protected $log_user 	= false;
	protected $set_created	= true;
	protected $set_modified = true;
	protected $soft_deletes	= false;

	protected $created_field     = 'created_on';
	protected $modified_field    = 'modified_on';

	// Customize the operations of the model without recreating the insert,
    // update, etc. methods by adding the method names to act as callbacks here.
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 	    = array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	// For performance reasons, you may require your model to NOT return the id
	// of the last inserted row as it is a bit of a slow method. This is
    // primarily helpful when running big loops over data.
	protected $return_insert_id = true;

	// The default type for returned row data.
	protected $return_type = 'object';

	// Items that are always removed from data prior to inserts or updates.
	protected $protected_attributes = array();

	// You may need to move certain rules (like required) into the
	// $insert_validation_rules array and out of the standard validation array.
	// That way it is only required during inserts, not updates which may only
	// be updating a portion of the data.
	protected $validation_rules 		= array(
		array(
			'field' => 'subscriber_id',
			'label' => 'lang:label_subscriber_id',
			'rules' => 'trim|max_length[11]',
		),
		array(
			'field' => 'message',
			'label' => 'lang:label_message',
			'rules' => 'required|trim|max_length[450]',
		),
	);
	protected $insert_validation_rules  = array();
	protected $skip_validation 			= false;
	protected $context			        = 'components';
	protected $moduleName               = 'messages';

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
    }
    
    /**
     * Templates Form Fields
     *
     * @return array[]
     */
    public function queue() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => false,
                'admin_index' => true
            ),
            array(
                'type' => 'text',
                'name' => 'name',
                'label' => 'label_name',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'edit' => 1,
                'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
                'rules' => 'required|trim|max_length[255]',
            ),
            array(
                'type' => 'textarea',
                'name' => 'message',
                'label' => 'label_message',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'required|trim|max_length[255]',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim'
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim'
            )
        );
        return $fields;
    }
	
	/**
     * Templates Form Fields
     *
     * @return array[]
     */
    public function templates() {
		$fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => false,
                'admin_index' => true
            ),
		    array(
		        'type' => 'text',
		        'name' => 'name',
		        'label' => 'label_name',
		        'frontend' => true,
		        'admin_only' => true,
		        'frontend_index' => true,
		        'admin_index' => true,
		        'edit' => 1,
		        'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
		        'rules' => 'required|trim|max_length[255]',
		    ),
			array(
				'type' => 'textarea',
				'name' => 'message',
				'label' => 'label_message',
				'frontend' => true,
				'admin_only' => true,
				'frontend_index' => true,
				'admin_index' => true,
				'rules' => 'required|trim|max_length[255]',
			),
			array(
				'type' => 'none',
				'name' => 'created_on',
				'label' => 'created_on',
				'frontend' => true,
				'admin_only' => true,
				'frontend_index' => true,
				'admin_index' => true,
				'rules' => 'trim'
			),
			array(
				'type' => 'none',
				'name' => 'modified_on',
				'label' => 'modified_on',
				'frontend' => true,
				'admin_only' => true,
				'frontend_index' => true,
				'admin_index' => true,
				'rules' => 'trim'
			)
		);
        return $fields;
    }
    
    /**
     * Message Sent Form Fields
     *
     * @return array[]
     */
    public function sent() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => false,
                'admin_index' => true
            )
        );
        return $fields;
    }
	
	/**
	 * Get All Subscribers
	 */
	public function get_subscribers($count, $where, $sort_by, $limit, $offset) {
		if($count == 1) {
			return $this->db->count_all_results('message_subscribers');
		} else {
			$this->db->limit($limit, $offset);
			return $this->db->order_by($sort_by)->get('message_subscribers')->result();
		}
	}
	
	/**
	 * Get a specific Subscribers
	 */
	public function get_subscriber($id) {
		return $this->db->where('id', $id)->get('message_subscribers')->row();
	}
	
	/**
	 * Get All Templates
	 */
	public function get_templates($count, $where, $sort_by, $limit, $offset) {
		if($count == 1) {
			return $this->db->count_all_results('message_templates');
		} else {
			$this->db->limit($limit, $offset);
			return $this->db->order_by($sort_by)->get('message_templates')->result();
		}
	}
	
	/**
	 * Get a specific Template
	 */
	public function get_template($id) {
		return $this->db->where('id', $id)->get('message_templates')->row();
	}
	
	/**
	 * Get All Remarks
	 */
	public function get_remarks($count, $where, $sort_by, $limit, $offset) {
		if(!empty($where)) {
			$this->db->where($where);
		}
		if($count == 1) {
			return $this->db->count_all_results('message_remarks');
		} else {
			$this->db->limit($limit, $offset);
			$this->db->join('message_subscribers MS', 'MS.id = message_remarks.subscriber_id', 'LEFT');
			return $this->db->select('message_remarks.*, MS.name AS subscriber_id')->order_by($sort_by)->get('message_remarks')->result();
		}
	}
	
	/**
	 * Get Product Topics/Products/Groups etc.
	 * @param number $count
	 * @param string $where
	 * @param string $sort_by
	 * @param number $limit
	 * @param number $offset
	 * @return array|number
	 */
	public function manage($count = 0, $where = null, $sort_by = 'M.id DESC', $limit = 15, $offset = 0, $extra = array()) {
	    if(!isset($extra['module'])) {
	        return false;
	    }
	    $module = $extra['module'];
	    if($count == 1) {
	        $sql = "SELECT COUNT(*) AS total_count";
	    } else {
	        switch($module) {
	            case 'sent':
	                $sql = "SELECT M.*, MT.name AS template";
	                break;
	            default:
	                $sql = "SELECT M.*";
	        }
	    }
	    $table = 'message_' . $module;
	    if($module == 'sent') {
	        $table = 'subscriber_messages';
	        $where .= (!empty($where) ? ' AND ' : '') . "M.type = 'message'";
	    }
	    $sql .= " FROM " . $this->db->dbprefix . $table . " M";
	    if($count == 0) {
	        switch($module) {
	            case 'sent':
	                $sql .= " LEFT JOIN " . $this->db->dbprefix . "message_templates MT ON MT.id = M.template_id";	                
	                break;
	            default:
	        }
	    }
	    if(isset($extra['filter']) && $extra['filter'] == 'deleted') {
	        $sql .= " WHERE M.deleted = 1";
	    } else {
	        $sql .= " WHERE M.deleted = 0";
	    }
	    if(!empty($where)) {
	        $sql .= ' AND '. $where;
	    }
	    if ($count == 0) {
	        $sql .= " GROUP BY M.id ORDER BY " . $sort_by;
	        if ($limit != 0) {
	            $sql .= " LIMIT " . $offset . ", " . $limit;
	        } else {
	            $sql .= " LIMIT $limit";
	        }
	        $query = $this->db->query($sql);
	        return $query->result();
	    } else {
	        $result = $this->db->query($sql)->row();
	        return $result ? $result->total_count : 0;
	    }
	}
	
	/**
	 * View Message Queue
	 * @param number $count
	 * @param string $where
	 * @param string $sort_by
	 * @param number $limit
	 * @param number $offset
	 * @return array|number
	 */
	public function view_queue($count = 0, $where = null, $sort_by = 'M.id DESC', $limit = 15, $offset = 0, $extra = array()) {
	    if(!isset($extra['module'])) {
	        return false;
	    }
	    $module = $extra['module'];	    
	    $table = 'message_' . $module;
	    if($count == 1) {
	        $sql = "SELECT COUNT(*) AS total_count";
	        $sql .= " FROM " . $this->db->dbprefix . $table . " M";
	        if(isset($extra['filter']) && $extra['filter'] == 'deleted') {
	            $sql .= " WHERE M.deleted = 1";
	        } else {
	            $sql .= " WHERE M.deleted = 0";
	        }
	        if(!empty($where)) {
	            $sql .= ' AND '. $where;
	        }
	    } else {
	        $sql = "SELECT M.*, U.name AS subscriber, MT.name AS template";
	        $sql .= " FROM (SELECT *";
	        $sql .= " FROM " . $this->db->dbprefix . $table . " M";
	        if(isset($extra['filter']) && $extra['filter'] == 'deleted') {
	            $sql .= " WHERE M.deleted = 1";
	        } else {
	            $sql .= " WHERE M.deleted = 0";
	        }
	        if(!empty($where)) {
	            $sql .= ' AND '. $where;
	        }
	        $sql .= " ORDER BY " . $sort_by;
	        if ($limit != 0) {
	            $sql .= " LIMIT " . $offset . ", " . $limit;
	        } else {
	            $sql .= " LIMIT $limit";
	        }
	        $sql .= ") M";
	        $sql .= " LEFT JOIN " . $this->db->dbprefix . "subscriber_users U ON U.id = M.row_id AND M.module_id = " . gc_module_id('subscriber_users');
	        $sql .= " LEFT JOIN " . $this->db->dbprefix . "message_templates MT ON MT.id = M.template_id";
	    }
	    if ($count == 0) {
	        $query = $this->db->query($sql);
	        return $query->result();
	    } else {
	        $result = $this->db->query($sql)->row();
	        return $result ? $result->total_count : 0;
	    }
	}
	
	/**
	 * Display a list of Templates/Subscribers/Statuses data.
	 *
	 * @return void
	 */
	public function view($module = 'queue', $filter = 'all', $offset = 0, $role = 'admin', $extra = array()) {
	    $has_status = false;
	    // Status Change
	    if (isset($_POST['status_change'])) {
	        $this->gc->status_change('manual_' . $module, $_POST['status_change'], 'status');
	    }
	    // Set Default
	    if (isset($_POST['set_default'])) {
	        $this->set_default($_POST['set_default']);
	        Template::redirect(current_url());
	    }
	    // Allow Admin to change status
	    if(isset($_POST['change_status'])) {
	        $id = $_POST['change_status'];
	        $new_status = $_POST['status_' . $id];
	        $this->update_status($module, $id, $new_status);
	    }
	    // Process Queue
	    if(isset($_POST['process_queue'])) {
	        gc_demo_mode();
	        $this->gcrud->stats(array('module' => 'messages'));
	        $this->process_queue();
	    }
	    if(isset($_POST['process_selected'])) {
	        gc_demo_mode();
	        $this->gcrud->stats(array('module' => 'messages'));
	        if(!empty($_POST['checked'])) {
	           $records = $this->db->where('id IN (' . implode(',', $_POST['checked']) .')')->get($this->table_name)->result();
	           if($records) {
	               foreach($records as $record) {
	                   $this->gc->send_sms($record->message, $record->phone_number);
	                   // update modified date
	                   $this->db->where('id', $record->id)->update($this->table_name, array('status' => 1, 'sent_on' => date('Y-m-d H:i:s'), 'modified_on' => date('Y-m-d H:i:s')));
	               }
	               Template::set_message(sprintf(lang('message_queue_processed'), count($records)), 'success');
	               Template::redirect(current_url());
	           }
	        } else {
	            Template::set_message(lang('nothing_to_prcess'), 'danger');
	        }
	    }
	    // Clear Queue
	    if(isset($_POST['clear_queue'])) {
	        $this->db->query('TRUNCATE TABLE ' . $this->db->dbprefix . $this->table_name);
	        Template::set_message(lang('msg_success'), 'success');
	        Template::redirect(current_url());
	    }
	    
	    // Clear Sent
	    if(isset($_POST['clear_sent'])) {
	        if($this->db->query('DELETE FROM ' . $this->db->dbprefix . $this->table_name . " WHERE status = 1")) {
    	        Template::set_message(lang('msg_success'), 'success');
    	        Template::redirect(current_url());
	        } else {
	            Template::set_message(lang('msg_operation_failure'), 'danger');
	            Template::redirect(current_url());
	        }
	    }
	    
	    // Clear Pending
	    if(isset($_POST['clear_pending'])) {
	        if($this->db->query('DELETE FROM ' . $this->db->dbprefix . $this->table_name . " WHERE status = 0")) {
	            Template::set_message(lang('msg_success'), 'success');
	            Template::redirect(current_url());
	        } else {
	            Template::set_message(lang('msg_operation_failure'), 'danger');
	            Template::redirect(current_url());
	        }
	    }
	    
	    // Send SMS
	    if(isset($_POST['send_sms'])) {
	        gc_demo_mode();
	        $this->gcrud->stats(array('module' => 'messages'));
	        $this->send_sms($_POST['send_sms']);
	    }
	    $data = array(
	        'context' => $this->context,
	        'model' => 'messages',
	        'fields_method' => $module,
	        'set_table' => true,
	        'query_method' => 'manage', // Extra Method in case of different columns
	        'query_extra' => array('module' => $module, 'filter' => $filter),
	        'table_alias' => 'M.', // Table Alias for table joins
	        'prefix' => 'M',
	        'filter' => $filter,
	        'view' => 'components/manage',
	        'uri_segment' => 6,
	        'redirect_uri' => '/components/messages/manage/' . $module,
	        'edit_url' => site_url(SITE_AREA . '/' . $this->context . '/' . $this->moduleName . '/form/' . $module . '/edit/')
	    );
	    if(isset($extra['redirect_uri'])) {
	        $data['redirect_uri'] = $extra['redirect_uri'];
	    }
	    switch($module) {
	        case 'queue':
	            $data['query_method'] = 'view_queue';
	            $data['search_fields'] = array('phone_number');
	            break;
	        case 'templates':
	            $data['search_fields'] = array('name', 'message');
	            break;
	        case 'subscribers':
	            $data['search_fields'] = array('name');
	            $has_status = true;
	            break;
	        case 'statuses':
	            $data['search_fields'] = array('statuses');
	            break;
	        case 'sent':
	            $data['search_fields'] = array('name');
	            break;
	    }
	    if($has_status) {
	        $data['redirect_uri'] = '/components/messages/manage/' . $module;
	        $data['filter'] = $filter;
	        $data['filter_counter'] = true;
	        $filter_status = $this->get_filter('statuses', 0, null, $module);
	        Template::set(array(
	            'filter_status' => $filter_status
	        ));
	    }
	    $this->gcrud->index('message_' . $module, $offset, 'admin', $data);
	}
	
	/**
	 * Get Filter
	 * @param string $module
	 * @return NULL[]
	 */
	public function get_filter($module, $user_id = 0, $field_name = 'user_id', $table = null) {
	    $column_name = '';
	    switch($module) {
	        case 'statuses':
	            $column_name = 'status_id';
	            break;
	    }
	    $module = 'message_' . $module;
	    $result = array();
	    $records = $this->db->get($module)->result();
	    if($records) {
	        $where = array();
	        if(!empty($user_id)) {
	            $where[$field_name] = $user_id;
	        }
	        $count_table = empty($table) ? $this->table_name : 'message_' . $table;
	        foreach($records as $record) {
	            $where[$column_name] = $record->id;
	            $result[$record->id] = array(
	                'id' => $record->id,
	                'name' => !empty($record->name_lang) ? lang($record->name) : $record->name,
	                'counts' => $this->db->where($where)->count_all_results($count_table)
	            );
	        }
	    }
	    return $result;
	}
	
	/**
	 * Process Messages Queue
	 * @param number $limit - Number of items to be processed
	 * @param boolean $redirect - TRUE - Redirect User, FALSE - Do not redirect user in case of CronJob
	 */
	public function process_queue($limit = 10000, $process_all = false, $redirect = true) {
	    if($process_all) {
	        $where = "status = 0";
	    } else {
	        $where = "status = 0 AND (scheduled_on IS NULL OR scheduled_on <= '" . date('Y-m-d H:i:s') . "')";
	    }
	    $records = $this->db->where($where)->order_by('priority', 'DESC')->limit($limit)->get($this->table_name)->result();
	    if($records) {
	        foreach($records as $record) {
	            $this->gc->send_sms($record->message, $record->phone_number);
	            // update modified date
	            $this->db->where('id', $record->id)->update($this->table_name, array('status' => 1, 'sent_on' => date('Y-m-d H:i:s'), 'modified_on' => date('Y-m-d H:i:s')));
	        }
	        if($redirect) {
    	        Template::set_message(sprintf(lang('message_queue_processed'), count($records)), 'success');
    	        Template::redirect(current_url());
	        }
	    } else {
	        if($redirect) {
    	        Template::set_message(lang('message_empty_queue'), 'danger');
    	        Template::redirect(current_url());
	        }
	    }
	}
	
	/**
	 * Send SMS
	 * @param number $id
	 */
	public function send_sms($id, $is_redirect = true) {
	    $record = $this->db->where('id', $id)->get($this->table_name)->row();
	    if($record) {
	        $this->gc->send_sms($record->message, $record->phone_number, false, $record->id);
	        Template::set_message(lang('message_processed'), 'success');
	    } else {
	        Template::set_message(lang('msg_operation_failure'), 'danger');
	    }
	    if($is_redirect) {
	        Template::redirect(current_url());
	    }
	}
}