<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Import/Exports controller
 */
class Settings extends Admin_Controller
{
	protected $permissionView   = 'Migrate.Settings.View';
	protected $permissionCreate = 'Migrate.Settings.Create';
	protected $permissionDelete = 'Migrate.Settings.Delete';
	protected $permissionEdit   = 'Migrate.Settings.Edit';	

	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct()
	{
		parent::__construct();

		$this->auth->restrict($this->permissionView);
		$this->load->model('migrate_model', null, true);
		$this->lang->load('migrate');
		
		Template::set_block('sub_nav', 'settings/_sub_nav');
	}

	/*----------------------------------------------------*/
	/*	Import Data
	/*----------------------------------------------------*/ 
	/**
	 * Displays a list of all imports to the admin.
	 *
	 * @return void
	 */
	public function all_imports($offset = 0)
	{
		// Deleting anything?
		if (isset($_POST['delete'])) {
		    gc_demo_mode();
		    $this->gcrud->stats(array('module' => 'migrate'));
			$this->auth->restrict($this->permissionDelete);
			$checked = $this->input->post('checked');
	
			if (is_array($checked) && count($checked)) {
				$result = FALSE;
				foreach ($checked as $pid) {					
					$file = $this->migrate_model->select('file')->find($pid);
					if(file_exists(realpath(FCPATH.'/uploads/imports/' .$file->file))) {
						unlink(FCPATH .'uploads/imports/' .$file->file);
					}
					$result = $this->migrate_model->delete($pid);
				}
	
				if ($result) {
					Template::set_message(count($checked) .' '. lang('label_import') .' ' . lang('delete_success'), 'success');
				} else {
					Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_import')) . $this->migrate_model->error, 'danger');
				}
			}
		}
	
		// Sort By
		$order_by = $this->gc->sort('importer');
		if(!empty($order_by)) {
			$this->migrate_model->order_by($this->session->userdata('importer_sort')['column'], $this->session->userdata('importer_sort')['order']);
		} else {
			$this->migrate_model->order_by('id', 'desc');
		}
		$this->migrate_model->limit($this->limit, $offset);
		$records = $this->migrate_model->where('operation_type', 'Import')->find_all();
		
		// Pagination
		$this->load->library('pagination');
		
		$total_imports = $this->migrate_model->where('operation_type', 'Import')->order_by('performed_on', 'desc')->count_all();;
		
		$pager['base_url'] = site_url(SITE_AREA ."/settings/migrate/all_imports/");
		$pager['total_rows'] = $total_imports;
		$pager['per_page'] = $this->limit;
		$pager['uri_segment']	= 5;
		
		$this->pagination->initialize($pager);
		
		Template::set('records', $records);
		Template::set('toolbar_title', lang('label_import_list'));
		Template::render();
	}
	
	/**
	 * Allow admin to import data
	 */
	public function importer() {
		$this->auth->restrict($this->permissionCreate);
	
		if (isset($_POST['save'])) {
		    gc_demo_mode();
		    $this->gcrud->stats(array('module' => 'migrate'));
			$rules = $this->migrate_model->validation_rules_import;
			$this->form_validation->set_rules($rules);
			if (($this->form_validation->run() == TRUE) && ($insert_id = $this->save_migrate())) {
				// Get the Row ID before insert
				$module = $this->input->post('type');				
				
				// GET INSERTED FILE NAME
				$file = $this->migrate_model->select('file')->find($insert_id);
				/*if($this->input->post('file_type') == 'csv') {
					// Not for shared hostings
					// $this->import_csv($file->file, $this->input->post('type'), $this->input->post('fields_separator'), $this->input->post('lines_separator'), $insert_id);
				} else {
					// $this->import_excel($this->input->post('file_type'), $file->file, $insert_id, $this->input->post('type'));
				}*/
				
				// Log the activity
				// log_activity($this->auth->user_id(), lang('act_create_record') .': '. $insert_id .' : '. $this->input->ip_address(), 'migrate');
	
				// Template::set_message(sprintf(lang('msg_migrate_success'), lang('label_import')), 'success');
				// redirect(SITE_AREA .'/settings/migrate/importer');
				
				// PERFORM IMPORT OPERATION
				$this->session->set_userdata('migrate_import', array(
					'file_type' => $this->input->post('file_type'),
					'migrate_id' => $insert_id,
					'module' => $this->input->post('type'),
					'file' => $file->file,
					'fields_separator' => $this->input->post('fields_separator'),
					'lines_separator' => $this->input->post('lines_separator')
				));
				Template::redirect(site_url(SITE_AREA . '/settings/migrate/mapping'));
			} else {
				Template::set_message(sprintf(lang('msg_migrate_failure'), lang('label_import')) . $this->migrate_model->error, 'danger');
			}
		}
	
		Assets::add_module_js('migrate', 'migrate.js');
		Template::set('toolbar_title', lang('label_import_new'));
		Template::render();
	}
	
	/**
	 * Summary
	 *
	 * @param String $type Either "insert" or "update"
	 * @param Int	 $id	The ID of the record to update, ignored on inserts
	 *
	 * @return Mixed    An INT id for successful inserts, TRUE for successful updates, else FALSE
	 */
	private function save_migrate($type='insert', $id=0)
	{
		if ($type == 'update') {
			$_POST['id'] = $id;
		}
	
		$data = array();
		$data['type']       		= $this->input->post('type');
		$data['fields_separator']  	= $this->input->post('fields_separator');
		$data['lines_separator']  	= $this->input->post('lines_separator');
		if($_POST['online_file_path']) {
			$this->form_validation->set_rules('online_file_path', lang('label_import') .' '.lang('label_file'), 'trim|required|check_file_existence');
			if ($this->form_validation->run() === false) {
				return false;
			}
			$data['file']			=	$this->input->post('online_file_path');
		} else {
			/* $file_data 			= $this->upload_file();
			$data['file'] 			= isset($file_data['upload_data']['file_name']) ? $file_data['upload_data']['file_name'] : '';*/
			$this->form_validation->set_rules('uploaded_file', lang('label_import') .' '.lang('label_file'), 'trim|required');
			//Check whether file is uploaded or not
			if(isset($_FILES['migrate_file']) && $_FILES['migrate_file']['size'] > 0){
				$file_data 			= $this->upload_file();
				if(!$file_data) {
					$_POST['file_upload'] = 'NONE';
					$this->form_validation->set_rules('file_upload', lang('label_import') .' '.lang('label_file'), 'valid_file_upload');
				}
				// Avoid logo upload process, if the form has validation issues with other fields
				$_POST['uploaded_file'] = isset($file_data['upload_data']['file_name']) ? $file_data['upload_data']['file_name'] : '';
			}
			if ($this->form_validation->run() === false) {
				return false;
			}
			$data['file'] 		= $this->input->post('uploaded_file');
		}
		$data['status']     	= $this->input->post('migrate_status') ? 1 : 0;
		$data['operation_type']	= 'Import';
		
		if ($type == 'insert') {
			$id = $this->migrate_model->insert($data);
	
			if (is_numeric($id)) {
				$return = $id;
			} else {
				$return = FALSE;
			}
		} elseif ($type == 'update') {
			$return = $this->migrate_model->update($id, $data);
		}
	
		return $return;
	}
	
	/**
	 * Allow admin to upload excel file for import
	 * @return multitype:NULL
	 */
	private function upload_file() {
	    if(!gc_demo_mode(array('no_message' => true, 'no_redirect' => true))) {
	        return false;
	    }
		$config['upload_path'] = realpath(FCPATH.'uploads/imports/'); //Make SURE that you chmod this directory to 777!
		$config['allowed_types'] = '*';//'csv|xls|xlsx';
		//$config['max_size']='4096';
		$config['remove_spaces']=TRUE; //Remove spaces from the file name
		$this->load->library('upload', $config);
		if (!$this->upload->do_upload('migrate_file')) {
			//Template::set_message($this->upload->display_errors(), 'error'); // Show errors to the user
			$this->session->set_flashdata('upload_file_error', $this->upload->display_errors());
			return false;
			//redirect(SITE_AREA .'/settings/migrate/importer'); //Request is from create page
		}
		else {
			$data = array('upload_data' => $this->upload->data());
		}
		return $data;
	}
	
	/**
	 * Import CSV file
	 * $this->db->simple_query($query) command is used instead of $this-db->query($query)
	 * as query command was generating errors.
	 */
	public function import_csv($file_name, $table_name, $fields_terminated_by, $lines_terminated_by, $id) {
		$file = realpath(FCPATH.'uploads/imports/'. $file_name);//base_url() ."uploads/" .$file_name;
		// $file = str_replace('\\', '/', $file); FOR WINDOWS HOSTING
		if (!file_exists($file)) {
			Template::set_message(lang('msg_file_not_exists'), 'error');
			Template::redirect(site_url(SITE_AREA .'/settings/migrate/importer'));
		}
		
		// If the file extension is not CSV, display error to the user
		$path_info = pathinfo($file); // GET File Extension
		if($path_info['extension'] != 'csv') {
			Template::set_message(lang('error_file_format'), 'error');
			Template::redirect(site_url(SITE_AREA .'/settings/migrate/importer'));
		}
		
		$fields = array();
		$field_to_be_updated = array();
		$fields_without_id = array();
	
		// get first row of csv file
		$first_row = fopen($file, 'r');
		$data = fgetcsv($first_row, "r", "$fields_terminated_by");
		foreach($data as $field_to) {
			$fields[] = strtolower($field_to);
			if($field_to != 'id') {
				$field_to_be_updated[] = $field_to .' = VALUES('.$field_to.')';
				$fields_without_id[] = $field_to;
			}
		}
		fclose($first_row);
	
		// compare header columns once only, before the first row import
		// get fields name from database table
		$db_columns = $this->db->field_data($table_name);
		if(!$valid_columns) {
			$valid_columns = $this->valid_columns($db_columns, $fields);
		}
		
		if($this->input->post('insert_or_update') == 1) {
			// fill table with data
			$load_data = "LOAD DATA LOCAL INFILE '".$file."' INTO TABLE ". $this->db->dbprefix . $table_name;
			$load_data .= " FIELDS TERMINATED BY '".$fields_terminated_by."' ENCLOSED BY '". $this->input->post('enclosed_by') ."' LINES TERMINATED BY '".$lines_terminated_by."' IGNORE 1 LINES (".implode(',', $fields).")";
			$this->db->simple_query($load_data);
			// set auto increment value to the last id
			$alter_autoinc = "ALTER TABLE " . $this->db->dbprefix . $table_name." AUTO_INCREMENT = 1"; // IMPORTANT
			$this->db->simple_query($alter_autoinc);
		} else {
			// create temporary table to store all data
			$create_temp_table = "CREATE TEMPORARY TABLE IF NOT EXISTS temp LIKE "  . $this->db->dbprefix . $table_name;
			$this->db->simple_query($create_temp_table);
		
			// fill temp table with data
			$load_data = "LOAD DATA LOCAL INFILE '".$file."' INTO TABLE temp";
			$load_data .= " FIELDS TERMINATED BY '".$fields_terminated_by."' LINES TERMINATED BY '".$lines_terminated_by."' IGNORE 1 LINES (".implode(',', $fields).")";
			$this->db->simple_query($load_data);
				
			if(($this->input->post('insert_or_update') == 0) && ($this->input->post('use_id_for_update') == 1)) {
				// copy data from temporary table to target table
				$update_on_duplicate = "INSERT INTO "  . $this->db->dbprefix . $table_name ."(".implode(',', $fields).")";
				$update_on_duplicate .= " SELECT ".implode(',', $fields)." FROM temp";
				$update_on_duplicate .= " ON DUPLICATE KEY UPDATE " .implode(',', $field_to_be_updated);
				$this->db->simple_query($update_on_duplicate);
			} else {
				$update_on_duplicate = "INSERT INTO "  . $this->db->dbprefix . $table_name ."(".implode(',', $fields_without_id).")";
				$update_on_duplicate .= " SELECT ".implode(',', $fields_without_id)." FROM temp";
				$update_on_duplicate .= " ON DUPLICATE KEY UPDATE id = LAST_INSERT_ID(" . $this->db->dbprefix . $table_name.".id), " .implode(',', $field_to_be_updated);
				$this->db->simple_query($update_on_duplicate);
			}
		
			// set auto increment value to the last id
			$alter_autoinc = "ALTER TABLE ". $this->db->dbprefix . $table_name." AUTO_INCREMENT = 1"; // IMPORTANT
			$this->db->simple_query($alter_autoinc);
				
			// after import and update drop temporary table
			$drop_table = "DROP TEMPORARY TABLE temp";
			$this->db->simple_query($drop_table);
		}
		
		// update the migrate status and perform other operations
		$this->common_import($id, $table_name);
	}
	
	
	/**
	 * Read and import data using Excel Library
	 * @param string $fileType
	 * @param string $file
	 * @param integer $id
	 * @param string $table_name - name of the table in which data to be imported
	 */
	private function import_excel($fileType, $file, $id, $table_name) {
	    $this->load->library('format');
	    $excel_params = array(
	        'file_name' => $file
	    );
	    if($fileType != 'Excel2007') {
	        $excel_params['file_type'] = $fileType;
	    }
	    $excel_data = $this->format->from_excel($excel_params);
	    if($excel_data['success'] === false) {
	        Template::set_message($excel_data['message'], 'error');
	        Template::redirect(site_url(SITE_AREA .'/settings/migrate/importer'));
	    } else {
	        $header = $excel_data['header'];
	        $arr_data = $excel_data['data'];
	    }
		// Perform the import operation
		$this->import_data($id, $table_name, $header, $arr_data);
	}
	
	/**
	 * Update Location Slug
	 * @param string $table_name
	 * @return boolean
	 */
	private function update_location_slug($table_name) {
		$this->load->library('gc');
		$this->db->or_where('slug', NULL);
		$this->db->or_where('slug', '');
		$records = $this->db->select('id, name')->get_where($this->db->dbprefix . $table_name)->result();
		if($records) {
			foreach($records as $record) {
				$data[] = array(
						'id' => $record->id,
						'slug' => $this->gc->slugify($table_name, $record->name, 0, 'name')
				);
			}
			$this->db->update_batch($table_name, $data, 'id');
		}
		return true;
	}
	
	/**
	 * File Type and File Extension Validation Function
	 * @return boolean
	 */
	public function _valid_file() {
		//Get uploaded file extension
		$path_info = pathinfo($_FILES['migrate_file']['name']);
		$file_extension = $path_info['extension'];
	
		if(($this->input->post('file_type') == 'Excel5') && $file_extension != 'xls') {
			$this->form_validation->set_message('_valid_file', lang('error_wrong_file'));
			return FALSE;
		}
		elseif(($this->input->post('file_type') == 'Excel2007') && $file_extension != 'xlsx') {
			$this->form_validation->set_message('_valid_file', lang('error_wrong_file'));
			return FALSE;
		} else {
			return TRUE;
		}
	}
	
	/*----------------------------------------------------*/
	/*	Export Data
	/*----------------------------------------------------*/
	/**
	 * Displays a list of all exported data.
	 *
	 * @return void
	 */
	public function all_exports($offset = 0)
	{
		// Deleting anything?
	    if (isset($_POST['delete'])) {
	        gc_demo_mode();
	        $this->gcrud->stats(array('module' => 'migrate'));
			$this->auth->restrict($this->permissionDelete);
			$checked = $this->input->post('checked');
	
			if (is_array($checked) && count($checked)) {
				$result = FALSE;
				foreach ($checked as $pid) {
					$file = $this->migrate_model->select('file')->find($pid);
					if($file && file_exists(FCPATH. 'uploads/exports/' . $file->file)) {
						unlink(FCPATH .'uploads/exports/' .$file->file);
					}
					$result = $this->migrate_model->delete($pid);
				}
	
				if ($result) {
					Template::set_message(count($checked) .' '. lang('label_export') .' ' . lang('delete_success'), 'success');
				} else {
					Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_export')) . $this->migrate_model->error, 'danger');
				}
			}
		}
	
		// Sort By
		$order_by = $this->gc->sort('exporter');
		if(!empty($order_by)) {
			$this->migrate_model->order_by($this->session->userdata('exporter_sort')['column'], $this->session->userdata('exporter_sort')['order']);
		} else {
			$this->migrate_model->order_by('id', 'desc');
		}
		$this->migrate_model->limit($this->limit, $offset);
		$records = $this->migrate_model->where('operation_type', 'Export')->order_by('performed_on', 'desc')->find_all();
		
		// Pagination
		$this->load->library('pagination');
		
		$total_exports = $this->migrate_model->where('operation_type', 'Export')->count_all();;
		
		$pager['base_url'] = site_url(SITE_AREA ."/settings/migrate/all_exports/");
		$pager['total_rows'] = $total_exports;
		$pager['per_page'] = $this->limit;
		$pager['uri_segment']	= 5;
		
		$this->pagination->initialize($pager);
	
		Template::set('records', $records);
		Template::set('toolbar_title', lang('label_export_list'));
		Template::render();
	}
	
	/**
	 * Display Exporter form to the user
	 */
	public function exporter()
	{
		$this->auth->restrict($this->permissionCreate);
	
		if (isset($_POST['save'])) {
		    gc_demo_mode();
		    $this->gcrud->stats(array('module' => 'migrate'));
			//Validation Rules
			$rules = $this->migrate_model->validation_rules_export;
			$this->form_validation->set_rules($rules);
			if (($this->form_validation->run() == TRUE) && ($insert_id = $this->save_exports())) {
				if($this->input->post('file_type') == 'csv') {
					$file_name = $this->export_csv($this->input->post('type'), $this->input->post('fields_separator'), $this->input->post('lines_separator'), $this->input->post('enclosed_by'), $this->input->post('start_id'), $this->input->post('end_id'), $insert_id);
				} else {
					$this->export_data($insert_id, $this->input->post('file_type'), $this->input->post('type'), $this->input->post('archieve'));
				}
				// Log the activity
				log_activity($this->auth->user_id(), lang('act_create_record') .': '. $insert_id .' : '. $this->input->ip_address(), 'migrate');
	
				Template::set_message(sprintf(lang('msg_migrate_success'), lang('label_export')), 'success');
				// redirect(SITE_AREA .'/settings/migrate/exporter');
			}
			else {
				Template::set_message(sprintf(lang('msg_migrate_failure'), lang('label_export')) . $this->migrate_model->error, 'danger');
			}
		}
		Assets::add_module_js('migrate', 'migrate.js');
		
		Template::set('toolbar_title', lang('label_export_new'));
		Template::render();
	}
	
	/**
	 * Export data in CSV format for VPS/Dedicated Server
	 * Requires special permissions on a server
	 * Make sure you have granted FILE to user 'user'@'localhost'. > GRANT FILE ON *.* TO 'user'@'localhost';
	 * > grant all privileges 
		  on YOUR_DATABASE.* 
		  to 'asdfsdf'@'localhost' 
		  identified by 'your_password';
		> flush privileges; 
	 *  If it's your system (you're admin), and you know how to secure it, this is how you enable those permissions.
			USE mysql;
			UPDATE user SET File_priv = 'Y' WHERE User = 'db_user';
			FLUSH PRIVILEGES;
	 * @param string $table_name
	 * @param string $fields_terminated_by
	 * @param string $lines_terminated_by
	 * @param number $start_id
	 * @param number $end_id
	 * @return string
	 */
	public function export_csv_vps($table_name, $fields_terminated_by, $lines_terminated_by, $fields_enclosed_by, $start_id, $end_id, $id) {
		$file = ucfirst($table_name). '_'. date('dMy') . '_' . date('H-i-s');
	
		// get exact file name and replace back slashes with forward slashes otherwise file will not be created
		$file_name = FCPATH. 'uploads/exports/' .$file.'.csv'; // change the filename here
		//$file_name = str_replace('\\', '/', $exact_file_name);
	
		/** Add header to csv file
		 * first of all get fields name from database table
		*/
	
		$db_fields = $this->db->field_data($table_name);
		$db_fields_name = array(); //ARRAY TO STORE FIELD NAME
		foreach($db_fields as $db_field) {
			$db_fields_name[] = strtolower($db_field->name);
		}
	
		$header_fields = "'" .implode("','", $db_fields_name) ."'";
		//  OPTIONALLY ENCLOSED BY '' is necessary otherwise format will be different
		$sql = "SELECT ".$header_fields." UNION ALL"; // add header fields to csv file
		$sql .= " SELECT * INTO OUTFILE '". $file_name ."' FIELDS TERMINATED BY '". $fields_terminated_by ."' OPTIONALLY ENCLOSED BY '". $fields_enclosed_by ."' LINES TERMINATED BY '". $lines_terminated_by ."'";
		$sql .= "  FROM " . $this->db->dbprefix . $table_name . " WHERE id BETWEEN ". $start_id ." AND " . $end_id;
		$this->db->query($sql);
	
		if($this->input->post('archieve')) {
			// create a zip archive of existing file
			$this->load->library('zip');
			$this->zip->read_file($file_name);
				
			// write file to the server
			$this->zip->archive(FCPATH.'uploads/exports/'.$file.'.zip');
				
			// delete csv file if user selected zip file
			unlink(FCPATH . "uploads/exports/" .$file .'.csv');
			
			// insert file name in database
			$this->migrate_model->update($id, array('file' => $file .'.zip'));
			
			// Download the file to your desktop. Name it "my_backup.zip"
			$this->zip->download($file.'.zip');
		} else {
			// allow user to download csv file
			$this->load->helper('download');
			$data = file_get_contents(FCPATH . "uploads/exports/" .$file .'.csv'); // Read the file's contents
			$name = $file .'.csv';			
			// insert file name in database
			$this->migrate_model->update($id, array('file' => $name));				
			force_download($name, $data);			
		}
	}
	
	/**
	 * Export Data in CSV Format
	 * @param string $table_name
	 * @param string $fields_terminated_by
	 * @param string $lines_terminated_by
	 * @param number $start_id
	 * @param number $end_id
	 * @return string
	 */
	public function export_csv ($table_name, $fields_terminated_by, $lines_terminated_by, $fields_enclosed_by, $start_id, $end_id, $id) {
		$file = ucfirst($table_name). '_'. date('dMy') . '_' . date('H-i-s');
	
		// get exact file name and replace back slashes with forward slashes otherwise file will not be created
		$file_name = FCPATH. 'uploads/exports/' .$file.'.csv'; // change the filename here
	
		$this->load->dbutil();
		$this->load->helper('file');
		$result = $this->db->query("SELECT * FROM " . $this->db->dbprefix . $table_name . " WHERE id BETWEEN ". $start_id ." AND " . $end_id);
		$newline = str_replace(array("\r\n","\r","\n","\\r","\\n","\\r\\n"),PHP_EOL,$lines_terminated_by);
		//$newline = str_replace("\\n",PHP_EOL,$lines_terminated_by);
		$csv = $this->dbutil->csv_from_result($result, $fields_terminated_by, $newline);
		write_file( $file_name, $csv);
	
		if($this->input->post('archieve')) {
			// create a zip archive of existing file
			$this->load->library('zip');
			$this->zip->read_file($file_name);
	
			// write file to the server
			$this->zip->archive(FCPATH.'uploads/exports/'.$file.'.zip');
	
			// delete csv file if user selected zip file
			unlink(FCPATH . "uploads/exports/" .$file .'.csv');
				
			// insert file name in database
			$this->migrate_model->update($id, array('file' => $file .'.zip'));
				
			// Download the file to your desktop. Name it "my_backup.zip"
			$this->zip->download($file.'.zip');
		} else {
			// allow user to download csv file
			$this->load->helper('download');
			$data = file_get_contents(FCPATH . "uploads/exports/" .$file .'.csv'); // Read the file's contents
			$name = $file .'.csv';
			// insert file name in database
			$this->migrate_model->update($id, array('file' => $name));
			force_download($name, $data);
		}
	}
	
	/**
	 * Export Data
	 * @param string $table_name
	 * @return boolean
	 */
	public function export_data($id, $file_type, $table_name, $archieve = 0) {
	    $this->load->library('format');
		if($table_name != 'countries') {
			$this->db->where(array ('id >= ' => $this->input->post('start_id'), 'id <=' => $this->input->post('end_id')));	
		}			
		$query = $this->db->get($table_name);		
		if(!$query) {
			return false;
		}

		// prepare filename
		$filename = ucfirst($table_name). '_'. date('dMy') . '_' . date('H.i.s');
		$this->load->library('slug');
		$filename = $this->slug->create_slug($filename);
		
		// excel data parameters
		$excel_data = array(
		    'file_name' => $filename,
		    'file_type' => $file_type,
		    'download' => true,
		    'archieve' => $archieve == 0 ? false : true
		);
		$data = $this->fields_model->position_field_values(2, $table_name, $query->result());
		if(!empty($data['rows'])) {
    		$excel_data['field_names'] = $data['columns'];
    		$excel_data['rows'] = $data['rows'];
		} else {
		    $excel_data['query'] = $query;
		}
		$this->format->to_excel($excel_data);
		
		// update migrate table
		if($file_type == 'Excel2007') {
		    $ext = '.xlsx';
		} else {
		    $ext = '.xls';
		}		
		// insert file name in database
		$this->migrate_model->update($id, array('file' => $filename . $ext));
	}
	
	/**
	 * Summary
	 *
	 * @param String $type Either "insert" or "update"
	 * @param Int	 $id	The ID of the record to update, ignored on inserts
	 *
	 * @return Mixed    An INT id for successful inserts, TRUE for successful updates, else FALSE
	 */
	private function save_exports($type='insert', $id = 0)
	{
		$data = array();
		$data['type']       = $this->input->post('type');
		//$data['file']		= NULL;
		$data['fields_separator']  = $this->input->post('fields_separator');
		$data['lines_separator']  = $this->input->post('lines_separator');
		$offset = $this->input->post('offset_value');
		$limit_rows = $this->input->post('limit_rows');
		if (($offset != 0) || ($limit_rows != 0)) {
			$data['start_id'] = $offset;
			$data['end_id']	= $limit_rows;
		} else {
			$data['start_id']  = $this->input->post('start_id');
			$data['end_id']  = $this->input->post('end_id');
		}
		$data['status']     = 1; //ALWAYS SUCCESSFUL
		$data['operation_type']	= 'Export';
	
		if ($type == 'insert') {
			$id = $this->migrate_model->insert($data);
	
			if (is_numeric($id)) {
				$return = $id;
			} else {
				$return = FALSE;
			}
		}
		
		return $return;
	}
	
	private function get_mysqli() {
		$db = (array)get_instance()->db;
		return mysqli_connect($db['hostname'], $db['username'], $db['password'], $db['database']);
	}
	
	public function custom_field_import () {
		$file = realpath(FCPATH.'uploads/imports/test.csv');
		
		$fields = array();
		$field_to_be_updated = array();
		$fields_without_id = array();
	
		// get first row of csv file
		$first_row = fopen($file, 'r');
		$data = fgetcsv($first_row, "r", ",");
		foreach($data as $field_to) {
			if($field_to != 'listing_id') {
				$field_to_be_updated[$field_to] = $field_to .' = VALUES('.$field_to.')';
			}
		}
		fclose($first_row);
	}
	
	/**
	 * Prepare Data
	 * @param array $data
	 * @return multitype:Ambigous <Ambigous>
	 */
	private function import_data($migrate_id, $module, $header, $data) {
		$result = array();
		// get fields name from database table
		$db_columns = $this->db->field_data($module);
		// compare columns on first row insert only
		// do not proceed with import if there is a column mismatch
		$valid_columns = FALSE;
		// create slug
		$this->load->library('slug');
		$type = ($this->input->post('insert_or_update') == 0 ? 'insert_update' : 'insert_only');
		if(!empty($data)) {
			$this->db->trans_start();
			foreach ($data as $rows_to_insert) {			    
			    $info = $this->process_data($rows_to_insert);
			    $rows_to_insert = $info['row'];
				if(!empty($rows_to_insert)) {		
					// compare header columns once only, before the first row import
					if(!$valid_columns) {
						$valid_columns = $this->valid_columns($db_columns, array_keys($rows_to_insert));
					}
					
					// the import file must have the column slug to automatically generate slug
					// the field can be set empty
					// if the slug field has value, do not overwrite it
					if(in_array('slug', $header) && empty($rows_to_insert['slug'])) {
					    if($module == 'users') {
					        $slug_source = $rows_to_insert['email'];
					    } else {
					        $slug_source = (isset($rows_to_insert['title']) ? $rows_to_insert['title'] : $rows_to_insert['name']);
					    }
						$rows_to_insert['slug'] = $this->slug->create_slug($slug_source);
					}
					
					// insert/update row and get id
					if($type == 'insert_update') {
					    if(isset($rows_to_insert['id'])) {
					        $row_id = $rows_to_insert['id'];
					        // chec row existence
					        $count_check = $this->db->where('id', $row_id)->count_all_results($module);
					        if($count_check > 0) {
					            unset($rows_to_insert['id']);
					            $this->db->where('id', $row_id)->update($module, $rows_to_insert);
					        } else {
					            $this->db->on_duplicate($module, $rows_to_insert);
					        }
					    } else {
					        $this->db->on_duplicate($module, $rows_to_insert);
					    }
						// $this->db->on_duplicate($module, $rows_to_insert);
					} else {
						$insert_query = $this->db->insert_string($module, $rows_to_insert);
						$insert_query = str_replace('INSERT INTO', 'INSERT IGNORE INTO', $insert_query);
						$this->db->query($insert_query);
					}
					$row_id = $this->db->insert_id();
					
					// in case of insert ignore, no row will be affected and mysql returns 0 as the id
					// do not process additional information in such cases
					if($row_id > 0 && !empty($info['result'])) {
					    // import location information
					    if(!empty($info['result']['country']['gc_country']) && !empty($info['result']['state']['gc_state']) && !empty($info['result']['city']['gc_city'])) {
					        $locality = !empty($info['result']['locality']) ? $info['result']['locality']['gc_locality'] : NULL;
					        $this->migrate_model->update_locations($module, $row_id, $locality, $info['result']['city']['gc_city'], $info['result']['state']['gc_state'], $info['result']['country']['gc_country']);
					    }
						
						// In case of categories, also save the relations with module
						// a category can belong to one or more modules
						if($module == 'categories') {
							$config_modules = gc_modules();
							$config_modules[] = 'videos';
							$toggle_modules = array();
							foreach($config_modules as $config_module) {
								if(!empty($info['result'][$config_module]['relation_' . $config_module])) {									
									$toggle_modules[] = $config_module;
								}
							}
							if(!empty($toggle_modules)) {
								$toggle_modules = array_flip($toggle_modules);
								$this->gc->save_module_relations($row_id, $toggle_modules, $module);
							}
						}
						
						// import additional information
						foreach($info['result'] as $key => $value) {
							if(!empty($info['result'][$key])) {
								switch($key) {
									case 'categories':
										$this->migrate_model->bulk_import_categories($module, $row_id, $value);
										break;
									case 'images':
										$this->migrate_model->bulk_import_images($module, $row_id, $value);
										break;
									case 'logo':
										$this->migrate_model->bulk_import_images($module, $row_id, $value, 2);
										break;
									case 'cover':
										$this->migrate_model->bulk_import_images($module, $row_id, $value, 1);
										break;
									case 'custom_fields':
										$this->migrate_model->bulk_import_custom_fields($module, $row_id, $value);
										break;
									case 'translations': // Multi Language Import
									    $this->translations->bulk_import($module, $row_id, $value);
									    break;
									case 'keywords':
										$this->migrate_model->bulk_import_tags($module, $row_id, $value);
										break;
									case 'assign_package': // Assign package to a user on Bulk Import
									    if($value['assign_package'] == 1) { // Assign package only if the value is 1
									        // count if an order exists or not with the same user
									        $orders_count = $this->db->where(array('user_id' => $row_id))->count_all_results('orders');
									        if($orders_count == 0) {
									            // Assign default package to the user
									            $this->load->model('users/user_model');
									            $this->user_model->assign_package($row_id);
									        }
									    }
									    break;
								}
							}
						}
					}
				}
			}
			$this->db->trans_complete();
		}
		$this->common_import($migrate_id, $module);
	}
	
	/**
	 * Common Import Function
	 * @param integer $migrate_id - Import Request ID
	 * @param string $module - Table Name
	 */
	private function common_import($migrate_id, $module) {
		// Core Module e.g. Listings, Videos etc.
		$key = array_search($module, gc_modules());
		if (false !== $key) {
			// get maxid for listing categories and start insertion from this id
			$maxid = 0;
			$max_row = $this->db->query('SELECT MAX(id) AS `maxid` FROM `' . $this->db->dbprefix . $module .'`')->row()->maxid;
			if ($max_row) {
				$maxid = $max_row;
			}
			
			// Update Tweak Fields
			$this->load->model('listings/listings_model');
			$this->listings_model->bulk_tweak($module, $maxid, 50000);
		}
		if(($module == 'cities') || ($module == 'states')) {
			$this->update_location_slug($module);
		}		
		// Unset Bulk Import Session
		$this->session->unset_userdata('bulk_import');
		$this->migrate_model->update($migrate_id, array('status' => 1));
	}
	
	/**
	 * Group data according to the specified fields
	 * 
	 * Assumptions - First category will be the default category
	 * All header columns must be in small letters
	 * 
	 * @param array $rows_to_insert
	 * @return multitype:Ambigous <multitype:, unknown> unknown
	 */
	private function process_data($rows_to_insert) {
		$result = array();
		// all find strings must be of 4 characters
		$fields = array(
			'images' => 'image_', 
			'categories' => 'categories_',
			'logo' =>'logo',
			'cover' => 'cover',
		    'custom_fields' => 'ctf_', // also change prefix in the migrate_model
		    'translations' => 'lang_', // also change prefix in the translations library
			// The country_iso, state_id, city_id and locality_id were skipped by the Import Process
			// because of name match, i.e. gc prefix is added with country, state and city column names
			'country' => 'gc_country', 
			'state' => 'gc_state',
			'city' => 'gc_city',
			'locality' => 'gc_locality',
			'keywords' => 'keywords',
		    'assign_package' => 'assign_package' // Assign package to a user on Bulk Import
		);		
		// In case of categories, also save the relations with module 
		// a category can belong to one or more modules
		// The Excel file must have columns starting with a field called relation_<NAME OF THE MODULE>
		$config_modules = gc_modules();
		$config_modules[] = 'videos';
		foreach($config_modules as $config_module) {
			$fields[$config_module] = 'relation_' . $config_module;
		}		
		foreach($fields as $key => $find) {
			$info = $this->additional_info($rows_to_insert, $find);
			$rows_to_insert = $info['row'];
			$result[$key] = $info['result'];
		}
		return array('row' => $rows_to_insert, 'result' => $result);
	}
	
	/**
	 * Get data specific to the find string
	 * e.g. img_, cat_
	 * 
	 * @param array $rows_to_insert
	 * @param string $find
	 * @return multitype:unknown multitype:unknown
	 */
	private function additional_info($rows_to_insert, $find) {
		$result = array();
		foreach($rows_to_insert as $key => $value) {
			$key = strtolower($key);
			//if(substr( $key, 0, 4 ) === $find) {
			// if(0 === strpos($key, $find)) { // NOT DELIVERING EXACT MATCH
			/**
			 * Given the following input: foo foobar
			 * \bfoo\b -  This regular expression will match only the first foo.
			 * \bfoo - This regular expression will match the first and also the foo in foobar:
			 */
			if(preg_match("/\b" . $find . "/i", $key)) {
				// $key is the column name
				$result[$key] = $value;
				// unset key from the rows_to_insert
				unset($rows_to_insert[$key]);
			}
		}
		return array('row' => $rows_to_insert, 'result' => $result);
	}
	
	/**
	 * Compare db columns with header columns to avoid any error on import
	 * @param array $db_columns
	 * @param array $header_columns
	 * @return boolean
	 */
	private function valid_columns($db_columns, $header_columns) {
		// db columns
		$fields_name = array();
		if(!empty($db_columns)) {
			foreach($db_columns as $db_column) {
				$fields_name[] = strtolower($db_column->name);
			}
		}
		
		//covert header fields to small letters for array difference
		$header_fields = array();
		foreach($header_columns as $header_column) {
		    if(!empty($header_column)) {
			    $header_fields_name[] = strtolower($header_column);
		    }
		}
		
		// difference between header columns
		$fields_difference = array_diff($header_fields_name, $fields_name);
		// perform insert operation only if there is no field difference
		if(empty($fields_difference)) {
			return true;
		} else {
			// db transaction is started above
			// it must be completed manually to dispaly the error message with redirect
			$this->db->trans_complete();
			// CONVERT FIRST LETTER TO UPPERCASE
			$mismatched_fields = array();
			foreach($fields_difference as $fields) {
				$mismatched_fields[] = $fields;
			}
			$columns_error = implode(", ", $mismatched_fields); //ADD COMMAS WITH MULTIPLE VALUES
			Template::set_message(lang('upload_failure') .'. ' . sprintf(lang('msg_mismatch_columns'), $columns_error), 'error'); // Display errors to the user
			Template::redirect(site_url(SITE_AREA .'/settings/migrate/importer'));
		}
	}
	
	/**
	 * Migrate PMD database to GoClixy
	 */
	public function migrate() {
	    $this->lang->load('leads/leads');
	    // Deleting anything?
	    if (isset($_POST['submit'])) {
	        gc_demo_mode();
	        $this->gcrud->stats(array('module' => 'migrate'));
	        $this->form_validation->set_rules('country_iso', lang('label_country'), 'trim|required');
	        $this->form_validation->set_rules('website', lang('label_website'), 'trim|required');
	        $this->form_validation->set_rules('id_limit', lang('label_id') . ' ' . lang('label_limit'), 'trim|required|is_numeric');
	        $this->form_validation->set_rules('id_from', lang('label_id') . ' ' . lang('label_from'), 'trim|required|is_numeric');
	        $this->form_validation->set_rules('id_to', lang('label_id') . ' ' . lang('label_to'), 'trim|required|is_numeric');
	        if ($this->form_validation->run() === true) {
	            $country_iso = $this->input->post('country_iso');
	            $website = $this->input->post('website');
	            $id_limit = $this->input->post('id_limit');
	            $id_from = $this->input->post('id_from');
	            $id_to = $this->input->post('id_to');
	            if(!empty($website)) {
	                $website = str_replace('http://', '', $website);
	                $website = str_replace('https://', '', $website);
	                $website = str_replace('www.', '', $website);
	                $website = str_replace('/', '', $website);
	            }
	            $this->load->library('migrate');
	            $data = array('country_iso' => $country_iso, 'website' => $website, 'id_limit' => $id_limit, 'id_from' => $id_from, 'id_to' => $id_to);
	            $success = $this->migrate->pmd($data);
	            if($success) {
	                Template::set_message(sprintf(lang('msg_migrate_success'), lang('label_import')), 'success');
	                Template::redirect(current_url());
	            }
	        }    	    
	    }
	    $countries = $this->db->order_by('printable_name', 'ASC')->get('countries')->result();
	    $country_options = array();
	    if($countries) {
	        foreach($countries as $country) {
	            $country_options[$country->iso] = $country->printable_name;
	        }
	    }
	    Template::set(array(
	        'toolbar_title' => lang('label_import_new'),
	        'country_options' => $country_options
	    ));
	    Template::render();
	}
	
	/**
	 * Fields Mapping
	 * CSV import is done using the MS-DOS CSV file
	 */
	public function mapping() {
	    $redirect_url = site_url(SITE_AREA .'/settings/migrate/all_imports');
	    $import_info = $this->session->userdata('migrate_import');
	    if(empty($import_info['module']) && empty($import_info['file'])) {
	        Template::redirect($redirect_url);
	    }
	    // database fields
	    $module = $import_info['module'];
		$module_id = gc_module_id($module);
		$file_type = $import_info['file_type'];
		$file_name = $import_info['file'];
	    $header_fields = array();
	    $record = array();
	    
	    // Read file header fields
		if($file_type == 'csv') {			
			$fields_terminated_by = $import_info['fields_separator'];
			$file = realpath(FCPATH.'uploads/imports/'. $file_name);
			if (!file_exists($file)) {
				Template::set_message(lang('msg_file_not_exists'), 'error');
				Template::redirect(site_url(SITE_AREA .'/settings/migrate/importer'));
			}
			
			// If the file extension is not CSV, display error to the user
			$path_info = pathinfo($file); // GET File Extension
			if($path_info['extension'] != 'csv') {
				Template::set_message(lang('error_file_format'), 'error');
				Template::redirect(site_url(SITE_AREA .'/settings/migrate/importer'));
			}
			
			$header_columns = array();	
			// get first row of csv file
			$handle = fopen($file, 'r');
			$header_data = fgetcsv($handle, 0, "$fields_terminated_by");
			foreach($header_data as $header_field) {
				// $header_columns[] = strtolower($header_field);
				$header_columns[] = $header_field;
				$header_fields[$header_field] = $header_field;
				$record[$header_field] = $header_field;
			}
			// fclose($handle);
		} else {			
			$this->load->library('format');
			$excel_params = array(
				'file_name' => $file_name
			);
			$excel_data = $this->format->from_excel($excel_params);
			if(empty($excel_data['success'])) {
				Template::redirect($redirect_url);
			}
			if(!empty($excel_data['success'])) {
				foreach($excel_data['header'] as $header_field) {
					$header_fields[$header_field] = $header_field;
					$record[$header_field] = $header_field;
				}
			}
		}
	    // database fields
	    $fields = $this->db->field_data($module);
	    $columns = array();
	    foreach($fields as $field) {
	        $field = (array) $field;
	        $columns[$field['name']] = $field['name'];
			ksort($columns);
	    }
	    if(isset($_POST['map_fields'])) {
	        $mapped_data = array();
			$mapped_fields = array();
			$unique_fields = array(); // add unique entries only
	        $post_data = $_POST;
	        unset($post_data['map_fields']);
	        foreach($post_data as $key => $data) {
	            $mapped_data[$data[1]] = $data[2];
				$this->field_mappings($module_id, $data[1], $data[2], (!empty($data[3]) ? 1 : 0));
				$mapped_fields[] = $data[2];
				if(!empty($data[3])) {
					$unique_fields[] = $data[2];
				}
	        }
			$required_fields = array();
			switch($module) {
				case 'listings':
				    $required_fields = array('package_id', 'user_id', 'title', 'country_iso'); // , 'country_iso', 'state', 'city', 'categories'
					break;
			}
			
			// difference between required and mapped fields columns	
			$fields_difference = array_diff($required_fields, $mapped_fields);
			// perform insert operation only if there is no field difference
			if(!empty($required_fields) && !empty($fields_difference)) {
				// convert first letter to lowercase
				$mismatched_fields = array();
				foreach($fields_difference as $fields) {
					$mismatched_fields[] = $fields;
				}
				$columns_error = implode(", ", $mismatched_fields); // ADD COMMAS WITH MULTIPLE VALUES
				Template::set_message(lang('upload_failure') .'. ' . sprintf(lang('msg_required_columns'), $columns_error), 'danger'); // Display errors to the user
			} else {				
				$this->load->model('categories/categories_model');
				$unique_data = array();
				if($file_type == 'csv') { // csv import
					// do not close header handle above, this will include the header column here
					$import_data = array();
					while (($rows = fgetcsv($handle, 0, "$fields_terminated_by")) !== false) {
						if($rows[1] && array(null) !== $rows) { // ignore blank lines
							$row_data = array();
							$unique_key = array();
							foreach($header_columns as $key => $column) {
								$db_column = $mapped_data[$column];
								$db_value = escape($rows[$key]);
								$row_data[$db_column] = $db_value;
								if(!empty($unique_fields) && in_array($db_column, $unique_fields)) {
									$unique_key[] = $db_value;
								}
							}
							if(!empty($unique_fields)) { // filter duplicate rows
								$unique_key = implode('___', $unique_key);
								if(!in_array($unique_key, $unique_data)) {
									$import_data[] = $row_data;
									$unique_data[] = $unique_key;
								}
							} else {
								$import_data[] = $row_data;
							}
							$this->_import($module, $row_data, $fields, $unique_fields);
						}
					}
					fclose($handle);
					/**
					 * FASTEST
					 * import all data into a temporary table
					 * After import, process categories, states, cities, tags and country information using the unique values
					 * Update the processed data into the temp table itself
					 * Once tmp_table is ready, move the information to the main modules table
					 * Remove duplicates from the main table using the methods mentioned at
					 * https://www.mysqltutorial.org/mysql-delete-duplicate-rows/
					 */
					/*$tmp_table = 'tmp_' . $module;
					$this->db->query("DROP TABLE " . $this->db->dbprefix . $tmp_table . ";");
					$sql = "CREATE TABLE " . $this->db->dbprefix . $tmp_table .  " LIKE " . $this->db->dbprefix . $module . ";";
					$this->db->query($sql);
					$this->db->query("SET NAMES utf8mb4;");
					$this->db->insert_batch($tmp_table, $import_data);*/
				} else {
					$i = 0;
					$ids = array();
					$excel_data = $excel_data['data'];
					foreach($excel_data as $rows_to_insert) { // excel import
						// prepare row with new columns
						$row_data = array();
						foreach($rows_to_insert as $rkey => $rvalue) {
							$row_data[$mapped_data[$rkey]] = $rvalue;
						}	
						$ids[$i] = $this->_import($module, $row_data, $fields, $unique_fields);
						$i++;		
					}
					// update categories, tags, images, metas and other information
					$i = $i > 0 ? $i - 1 : 0;
					if(!empty($ids) && $ids[0] != $ids[$i]) {
						$this->_bulk_tweak($module, $ids[0], $ids[$i]);
					}
				}		
				$this->session->unset_userdata('migrate_import');
				// update status information in the migrations table
				$this->migrate_model->update($import_info['migrate_id'], array('status' => 1));
				Template::set_message(sprintf(lang('msg_migrate_success'), lang('label_import')), 'success');
				Template::redirect($redirect_url);
			}
	    }
		
		// avoid mapping of the same fields again and again, save them in database and retrieve for future
		$where_header = "'" . implode ( "', '", $header_fields ) . "'";
		$map_records = $this->db->where("module_id = " . $module_id . " AND field_name IN (" . $where_header . ")")->get('migration_mappings')->result();
		if($map_records) {
			foreach($map_records as $map_record) {
				$record[$map_record->field_name] = array('mapped_with' => $map_record->mapped_with, 'is_unique' => $map_record->is_unique);
			}
		} else {
		    foreach($record as $fkey => $fvalue) {
		        $record[$fkey] = array('mapped_with' => $fkey);
		    }
		}
	    Template::set(array(
	        'toolbar_title' => lang('migrate_mapping'),
	        'record' => $record,
	        'header_fields' => $header_fields,
	        'columns' => $columns,
	    ));
	    Template::set_view('mapping');
	    Template::render();
	}
	
	/**
	 * Import Data
	 * @param string $module
	 * @param array $row
	 * @param array $fields
	 * @param array $unique_fields
	 */
	private function _import($module, $row, $fields, $unique_fields) {
	    $table_row_data = array();
	    foreach($fields as $field) {
	        $field = (array) $field;
	        if(empty($field['type_id'])) { // core field
	            if(array_key_exists($field['name'], $row)) {
	                $table_row_data[$field['name']] = $row[$field['name']];
	            }
	        }
	    }
	    if(in_array($module, gc_modules())) {						
			/*$format_fields = array('categories' => '=>', 'tags' => ',', 'state' => '_____', 'city' => '_____');
			foreach($format_fields as $field_name => $separator) {
				$table_row_data = $this->format_data($table_row_data, $row, $field_name, $separator);
			}*/
			
		} else if(array_key_exists('id', $table_row_data)) {
			unset($table_row_data['id']);
		}		
		// prepare slug field if it exists in the database
		$db_fields = $this->gc->get_db_fields($module);
		if(in_array('slug', $db_fields) && empty($table_row_data['slug'])) {
		    if(!empty($table_row_data['title'])) {
		        $table_row_data['slug'] = $this->slug->create_slug($table_row_data['title']);
		    } else if(!empty($table_row_data['name'])) {
		        $table_row_data['slug'] = $this->slug->create_slug($table_row_data['name']);
		    }
		}
		if(!empty($unique_fields)) {
			$where_unique = array();
			foreach($unique_fields as $unique_field) {
				if(!empty($table_row_data[$unique_field])) {
					$where_unique[$unique_field] = $table_row_data[$unique_field];
				}
			}
			$record = $this->db->where($where_unique)->get($module)->row();											
			if($record) {	                 
				$row_id = $record->id;
				$this->db->where('id', $record->id)->update($module, $table_row_data);
			} else { // insert data
			    $this->db->insert($module, $table_row_data);
				$row_id = $this->db->insert_id();
			}
		} else {
			$this->db->insert($module, $table_row_data);
			$row_id = $this->db->insert_id();
		}
		
		// tweak categories and tags
		/*if(!empty($row_id)) {
			if(in_array($module, gc_modules())) {
				$this->migrate_model->tweak_tags_categories($module, $row_id);
				$module_id = gc_module_id($module);
				// tweak categories and tags
				$this->categories_model->_tweak_categories($module_id, $row_id);
			}
		}*/
		return $row_id;
	}
	
	/**
	 * Format Data before insert
	 */
	private function format_data($table_row_data, $row, $field_name, $separator = '=>') {							
		/**
		 * COUNTY -  A county is an area of a state that is larger than a city, but not a state or city
		 * Therefore, a separate field county is created in the modules (e.g. listings) table to deal with counties
		 */	
		if(array_key_exists($field_name, $table_row_data)) {
			switch($field_name) {
				case 'state':
					$module = 'states';
					break;
				case 'city':
					$module = 'cities';
					break;
				default:
					$module = $field_name;
			}
			$table_rows = explode($separator, $row[$field_name] ?? '');
			$row_ids = array();
			if(!empty($table_rows[0])) {
				$i = 0;
				foreach($table_rows as $table_row) {
					$params = array();
					if($module == 'categories') {
						$params['fields_extra'] = array('parent_id' => $i > 0 ? $i - 1 : $i);
					}
					if($module == 'cities') {
						if(empty($row['state_id']) && !empty($row['state'])) {
							$result = $this->migrate_model->get_row('states', array('name' => $row['state']), $params);
							if($result) {
								$params['fields_extra'] = array('state_id' => $result->id, 'country_iso' => $result->country_iso, 'country_id' => $result->country_id);
							}
						}
					}					
					if($module == 'states') {
						if(!empty($row['country_iso'])) {
							$result = $this->migrate_model->get_row('countries', array('iso' => $row['country_iso']), $params);
							if($result) {
								$params['fields_extra'] = array('country_iso' => $result->iso, 'country_id' => $result->id);
							}
						}
					}
					$result = $this->migrate_model->get_row($module, array('name' => $table_row), $params);
					if($result) {
						switch($module) {
							case 'cities':
								$table_row_data['city_id'] = $result->id;		
								$table_row_data['city'] = $result->name;
								break;
							case 'states':
								$table_row_data['state_id'] = $result->id;		
								$table_row_data['state'] = $result->name;
								break;
							case 'countries':
								$table_row_data['country_iso'] = $result->iso;		
								$table_row_data['country'] = $result->name;
								break;
							default:								
								$row_ids[] = $result->id;
						}
					}
					$i++;
				}
			}
			switch($field_name) {
				case 'categories':
				case 'tags':
					$table_row_data[$field_name . '_set'] = implode(',', $row_ids);
					unset($table_row_data[$field_name]);
					break;
			}
		}
		return $table_row_data;
	}
	
	/**
	 * Save field mappings in database to avoid selection on each file
	 * @param string $field_name
	 * @param string $mapped_with
	 */
	private function field_mappings($module_id, $field_name, $mapped_with, $is_unique) {
		$sql = "INSERT INTO " . $this->db->dbprefix . "migration_mappings (module_id, field_name, mapped_with, is_unique) VALUES(" . $module_id . ", '" . $field_name . "', '" . $mapped_with . "', " . $is_unique . ") ON DUPLICATE KEY UPDATE module_id=" . $module_id . ", field_name='" . $field_name . "', mapped_with='" . $mapped_with . "', is_unique=" . $is_unique;
		$this->db->query($sql);
	}
	
	
	
	/**
	 * Data added in bulk, tweak locations, categories, tags and images
	 */
	public function bulk_import_tweak() {	
	    $this->load->library('slug');
	    $row_difference = 50000;
	    if (isset($_POST['save'])) {
	        $this->form_validation->set_rules('module', lang('label_module'), 'trim|required');
	        $this->form_validation->set_rules('id_from', lang('label_between'), 'trim|required|integer|min_val[1]');
	        $this->form_validation->set_rules('id_to', lang('label_and'), 'trim|required|integer|max_val[10000000]|compare_fields['.$this->input->post('id_from').']|diff_between['.$this->input->post('id_from').',' . $row_difference . ']');
	        $this->form_validation->set_rules('is_update', lang('migrate_update'), 'trim');
	        if($this->form_validation->run() == TRUE) {
	            $id_from = $this->input->post('id_from');
	            $id_to = $this->input->post('id_to');
	            $is_update = $this->input->post('is_update') ? true : false;	            
	            $module = $this->input->post('module');
				if($this->_bulk_tweak($module, $id_from, $id_to, $is_update)) {					
					Template::set_message(lang('msg_success'),'success');
					Template::redirect(current_url());
				} else {
					Template::set_message(lang('msg_nothing_to_update'), 'error');
					Template::redirect(current_url());
				}
	        }
	    }
	    Template::set(array(
	        'toolbar_title' => lang('migrate_bulk_tweak'),
	        'row_difference' => $row_difference
	    ));
	    Template::set_view('tweak');
	    Template::render();
	}

	/**
	 * Bulk Tweak fields like categories, tags, images and metas
	 */
	private function _bulk_tweak($module, $id_from, $id_to, $is_update = true) {
		$params = array(
			'id_from' => $id_from,
			'id_to' => $id_to,
		);
		$sql = "SELECT M.id FROM " . $this->db->dbprefix . $module . " M WHERE (M.id BETWEEN " . $id_from . " AND " . $id_to . ")";
		$result = $this->db->query ( $sql )->result();
		if($result) {
			// delete existing rows first if update
			if($is_update) {
				$this->delete_module('categories_relation', $module, $params);
				$this->delete_module('tag_relations', $module, $params);
				$this->delete_module('images', $module, $params);
				$this->delete_module('metas', $module, $params);
			}
			
			// perform the tweak operation
			$this->locations_module($module, $params);
			$this->categories_module($module, $params);
			$this->tags_module($module, $params);
			$this->images_module($module, 'images', $params);
			$this->images_module($module, 'cover', $params);
			$this->images_module($module, 'logo', $params);
			$this->metas_module($module, $params);
			return true;	      
		}
		return false;
	}
	
	/**
	 * Delete data first in case of update
	 * @param string $table
	 * @param string $module
	 * @param array $params
	 */
	private function delete_module($table, $module, $params = array()) {
	    $module_id = gc_module_id($module);
	    $sql = "DELETE FROM " . $this->db->dbprefix . $table . " WHERE module_id = " . $module_id;
	    $sql .= " AND (row_id BETWEEN " . $params['id_from'] . " AND " . $params['id_to'] . ")";
	    $this->db->query($sql);
	    // set auto increment value to the last id
	    $alter_autoinc = "ALTER TABLE " . $this->db->dbprefix . $table." AUTO_INCREMENT = 1"; // IMPORTANT
	    $this->db->simple_query($alter_autoinc);
	}
	
	/**
	 * Handle Module Locations Import
	 * @param string $module
	 */
	private function locations_module($module, $params = array()) {	    
	    // tweak locations first
	    // NOTE: country_iso is must
	    $this->load->model('locations/cities_model');	    
	    $sql = "SELECT * FROM " . $this->db->dbprefix . $module . " M WHERE M.city_id = 0 AND M.city != ''";
	    $sql .= " AND (M.id BETWEEN " . $params['id_from'] . " AND " . $params['id_to'] . ")";
	    $records = $this->db->query($sql)->result();
	    if($records) {
	        $location_data = array();
	        foreach($records as $row) {
	            if(!empty($row->city) && !empty($row->state) && !empty($row->country_iso)) {
	                $location = $this->gc->get('cities', array('where' => "name = '" . $row->city . "' AND state = '" . $row->state . "' AND country = '" . $row->country . "'"));
	                if($location) {
	                    $location_data[] = array(
	                        'id' => $row->id,
	                        'city_id' => $location->id,
	                        'state_id' => $location->state_id,
	                        'country_iso' => $location->country_iso,
	                    );
	                } else {
	                    $new_location = $this->cities_model->getCityStateCountry($row->city, null, $row->state, $row->country_iso, $row->country);
	                    if(!empty($new_location['city_id'])) {
	                        $new_location['id'] = $row->id;
	                        $location_data[] = $new_location;
	                    }
	                }
	            }
	        }
	        if(!empty($location_data)) {
	            $this->db->update_batch($module, $location_data, 'id');
	        }
	    }
	}
	
	/**
	 * Handle Module Categories Import
	 * E.g. 'Beauty and Spa=>Skin Care=>Cosmetics###Beauty and Spa=>Body Care=>Perfumes###Beauty'
	 * @param string $module
	 */
	private function categories_module($module, $params = array()) {
	    $sql = "SELECT M.* FROM " . $this->db->dbprefix . $module . " M WHERE categories LIKE '%=>%' AND id NOT IN (SELECT row_id FROM " . $this->db->dbprefix . "categories_relation WHERE module_id = " . gc_module_id($module) . " AND row_id = M.id)";
	    $sql .= " AND (M.id BETWEEN " . $params['id_from'] . " AND " . $params['id_to'] . ")";
	    $records = $this->db->query($sql)->result();
	    if($records) {
	        $this->load->model('categories/categories_model');
	        $this->load->library('goclixy/gclosure');
	        $gclosure = new Gclosure('categories', 'categories_hierarchy');
	        $module_id = gc_module_id($module);
	        $categories_relation_data = array();
	        $default_category_data = array();
	        $separator = '###';
	        $relation_separator = '=>';
	        foreach($records as $record) {
				if(strpos($record->categories, $relation_separator) === false) {
	                continue;
	            }
	            $row_id = $record->id;
	            $categories_string = $record->categories;
	            $categories = array_map('trim', explode($separator, $categories_string ?? ''));
	            if(!empty($categories)) {
	                $result = array();
	                $i = 1;
	                foreach($categories as $category) {
	                    $category = array_map('trim', explode($relation_separator, $category ?? ''));
	                    // $result = $this->categories_model->import_categories($category);
	                    $output = $this->categories_model->bulk_import($category, $module_id, $gclosure);
	                    if($i == 1) {
	                        $default_category_data[] = array(
	                            'id' => $row_id,
	                            'category_id' => end($output)
	                        );
	                    }
	                    $result = array_merge($result, $output);
	                    $i++;
	                }
	                $result = array_unique($result);
	                if(!empty($result)) {
	                    foreach($result as $category_id) {
	                        $categories_relation_data[] = array(
	                            'module_id' => $module_id,
	                            'row_id' => $row_id,
	                            'category_id' => $category_id
	                        );
	                    }
	                }
	            }
	        }
	        // insert categories data
	        if(!empty($categories_relation_data)) {
	            $this->db->insert_batch('categories_relation', $categories_relation_data);
				foreach($categories_relation_data as $relation_data) {
					$this->categories_model->_tweak_categories($relation_data['module_id'], $relation_data['row_id']);
				}
	        }
	        // set default category
	        if(!empty($default_category_data)) {
	            $this->db->update_batch($module, $default_category_data, 'id');
	        }
	    }
	}
	
	/**
	 * Handle Module Tags Import
	 * @param string $module
	 * @param number $limit
	 */
	private function tags_module($module, $params = array()) {
	    $sql = "SELECT M.* FROM " . $this->db->dbprefix . $module . " M WHERE keywords != '' AND id NOT IN (SELECT row_id FROM " . $this->db->dbprefix . "tag_relations WHERE module_id = " . gc_module_id($module) . " AND row_id = M.id)";
	    $sql .= " AND (M.id BETWEEN " . $params['id_from'] . " AND " . $params['id_to'] . ")";
	    $records = $this->db->query($sql)->result();
	    if($records) {
    	    $this->load->library('form_validation');
    	    $this->load->model('tags/tags_model');
    	    foreach($records as $record) {
    	        $this->tags_model->save_module_tags($record->keywords, $record->id, $module, 1);
    	    }
	    }
	}
	
	/**
	 * Modules Image Import
	 * Note: image name must be without comma
	 * @param string $module
	 * @param string $type
	 * @param number $limit
	 */
	private function images_module($module, $type, $params = array()) {
	    $type_id = 0;
	    if($type == 'cover') {
	        $type_id = 1;
	    } else if($type == 'logo') {
	        $type_id = 2;
	    }
	    $sql = "SELECT M.* FROM " . $this->db->dbprefix . $module . " M WHERE " . $type . " != '' AND id NOT IN (SELECT row_id FROM " . $this->db->dbprefix . "images WHERE module_id = " . gc_module_id($module) . " AND row_id = M.id AND type_id = " . $type_id . ")";
	    $sql .= " AND (M.id BETWEEN " . $params['id_from'] . " AND " . $params['id_to'] . ")";
	    $records = $this->db->query($sql)->result();
	    if($records) {
	        $this->load->model('images/images_model');
	        $separator = ',';
	        foreach($records as $record) {
	        $images = array_map('trim', explode($separator, $record->{$type} ?? ''));
    	        if(!empty($images)) {
    	            if($type_id == 1 || $type_id == 2) {
    	                $this->images_model->save_image_db($module, $record->id, $type_id, null, $images[0], null, 1, 1, TRUE);
    	            } else {
    	                foreach($images as $image) {
    	                    $this->images_model->save_image_db($module, $record->id, $type_id, null, $image, null, 1, 1, TRUE);
    	                }
    	            }
    	        }
	        }
	    }
	}
	
	/**
	 * Handle Module Custom Fields
	 * E.g. 'company_name=>Skin Care###license_no=>DEMO'
	 * @param string $module
	 * @param number $limit
	 */
	private function metas_module($module, $params = array()) {
	    $module_id = gc_module_id($module);
	    $sql = "SELECT * FROM " . $this->db->dbprefix . $module . " M WHERE M.metas != ''";
	    $sql .= " AND (M.id BETWEEN " . $params['id_from'] . " AND " . $params['id_to'] . ")";
	    $records = $this->db->query($sql)->result();
	    if($records) {
    	    $separator = '###';
    	    $relation_separator = '=>';
    	    $metas_data = array();
    	    foreach($records as $record) {
    	        $row_id = $record->id;
    	        $metas = array_map('trim', explode($separator, $record->metas ?? ''));	
    	        foreach($metas as $meta) {	            
    	            $meta = array_map('trim', explode($relation_separator, $meta ?? ''));
    	            $metas_data[] = array(
    	                'module_id' => $module_id,
    	                'row_id' => $row_id,
    	                'meta_key' => str_replace('-', '_', $this->slug->create_slug($meta[0])),
    	                'meta_value' => $meta[1]
    	            );
    	        }
    	    }
    	    if(!empty($metas_data)) {
    	        $this->db->on_duplicate('metas', $metas_data);
    	    }
	    }
	}

	/**
	 * Import Data from a table which is already filled with CSV data
	 * https://www.thebigbarhop.com/
	 */
	public function _import_csv() {
		$records = $this->db->limit(1500)->where('processed', 0)->get('listings_import')->result();
		$this->load->model('migrate/migrate_model');
		$this->load->library('form_validation');
		/*$truncate_tables = array('business_hours', 'listings', 'categories_relation', 'images', 'tags', 'tag_relations', 'tag_locations');
		foreach($truncate_tables as $truncate_table) {
			$this->db->truncate($truncate_table);
			$alter_autoinc = "ALTER TABLE " . $this->db->dbprefix . $truncate_table." AUTO_INCREMENT = 1"; // IMPORTANT
			$this->db->simple_query($truncate_table);
		}*/
		$this->load->library('slug');
		if($records) {
			foreach($records as $record) {
				$data = array(
					'user_id' => (int) $record->user_id,
					'package_id' => (int) $record->package_id,
					'title' => $record->title,
					'slug' => $this->slug->create_slug($record->title),
					'locality' => $record->locality,
					'postcode' => $record->postcode,
					'address' => $record->address,
					'latitude' => $record->latitude,
					'longitude' => $record->longitude,		
					'contact_person' => $record->contact_person,
					'phone_number' => $record->phone_number,	
					'mobile_number' => $record->mobile_number,
					'tollfree' => $record->tollfree,		
					'fax' => $record->fax,
					'website' => $record->website,
					'logo_url' => $record->logo_url,
					'google_rating' => $record->google_rating,
					'status' => 1,
				);
				$socials = array('linkedin_url', 'facebook_url', 'twitter_url', 'googleplus_url', 'youtube_url', 'pinterest_url', 'instagram_url', 'description');
				foreach($socials as $skey => $social) {
					$data[$social] = $record->{$social};
				}
				if(!empty($record->email)) {
					// has multiple emails
					$emails = array_map('trim', explode(',', $record->email));
					if(count($emails) > 1) { // save remaining emails in another column
						$data['email'] = $emails[0];
						unset($emails[0]);
						$data['emails'] = implode(', ', $emails);
					}
				}
				if(!empty($record->state)) {
					// get state based on abbrev
					$state = $this->gc->get('states', array('where' => "abbrev = '" . $record->state . "'"));
					if($state) {
						$data['state_id'] = $state->id;
						$data['state'] = $state->name;
					}
				}
				if(!empty($record->city)) {
					$city = $this->gc->get('cities', array('where' => 'name = "' . $record->city . '"'));
					if($city) {
						$data['city_id'] = $city->id;
						$data['city'] = $city->name;
					}
				}
				$data['country'] = 'United States';
				$this->db->insert('listings', $data);
				$listing_id = $this->db->insert_id();
				if(!empty($listing_id)) {
					if(!empty($record->hours)) {
						$this->save_business_hours($record->hours, $listing_id);
					}
					if(!empty($record->categories) && strtolower($record->categories) == 'all') {
						$this->save_categories($listing_id);
					}
					// save tags
					if(!empty($record->keywords)) {
						$this->migrate_model->bulk_import_tags('listings', $listing_id, strtolower($record->keywords));
					}
					if(!empty($record->logo_url)) {
						$this->migrate_model->bulk_import_images('listings', $listing_id, array('logo' => $record->logo_url), 2);
					}
				}
				$this->db->where('id', $record->id)->update('listings_import', array('processed' => 1));
			}
		}
	}

	/**
	 * Save Business Hours
	 */
	public function save_business_hours($str, $listing_id) {
		$days = array('monday' => 1, 'tuesday' => 2, 'wednesday' => 3, 'thursday' => 4, 'friday' => 5, 'saturday' => 6, 'sunday' => 7);
		// $str = "Monday: [11‚  AM-2‚  AM], Tuesday: [11‚  AM-2‚  AM], Wednesday: [11‚  AM-2‚  AM], Thursday: [11‚  AM-2‚  AM], Friday: [11‚  AM-2‚  AM], Saturday: [11‚  AM-2‚  AM], Sunday: [11‚  AM-2‚  AM]";
		// $str = "Tuesday: [11‚  AM-2‚  PM, 4-9‚  PM], Wednesday: [11‚  AM-2‚  PM, 4-9‚  PM], Thursday: [11‚  AM-2‚  PM, 4-10‚  PM], Friday: [11‚  AM-2‚  PM, 4-10‚  PM], Saturday: [11‚  AM-2‚  PM, 4-10‚  PM], Sunday: [10‚  AM-2‚  PM], Monday: [11‚  AM-2‚  PM, 4-9‚  PM]";
		$str_array = explode(']', $str);
		$business_hours = array();
		foreach($str_array as $arr) {
			if(!empty($arr)) {
				$hours_data = array(
					'listing_id' => $listing_id
				);
				$arr = str_replace(', ', '' ,$arr);
				$day_array = explode(':', $arr);
				$day_lower = strtolower($day_array[0]);
				if(array_key_exists($day_lower, $days)) {
					$hours_data['day_of_week'] = $days[$day_lower];
				}
				if(!empty($day_array[1])) {
					$day_str = str_replace('[', '', $day_array[1]);
					$day_explode = explode('-', $day_str);
					$first = reset($day_explode);
					$last = end($day_explode);
					$start_time = trim(str_replace('‚ ', '', $first));
					$end_time = trim(str_replace('‚ ', '', $last));

					// convert to 24 hours format
					$open_time = date("H:i", strtotime($start_time));
					$close_time = date("H:i", strtotime($end_time));
					$hours_data['open_time'] = $open_time;
					$hours_data['close_time'] = $close_time;
					$hours_data['status'] = 1;
				}
				$business_hours[] = $hours_data;
			}
		}
		if(!empty($business_hours)) {
			$this->db->insert_batch('business_hours', $business_hours);
		}
	}
	
	/**
	 * Save Categories Relations
	 */
	private function save_categories($listing_id) {
		// get all categories
		$categories = $this->gc->get('categories', array('result' => 1));
		if($categories) {
			$category_data = array();
			foreach($categories as $category) {
				$category_data[] = array(
					'module_id' => 1,
					'row_id' => $listing_id,
					'category_id' => $category->id
				);
			}
			$this->db->insert_batch('categories_relation', $category_data);
		}
	}
}