<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
 || ##################################################################### ||
 || # GoClixy - Business Directory Script							   # ||
 || # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
 || # ----------------------------------------------------------------  # ||
 || # This file is owned and copyrighted by GoClixy.com. Your license   # ||
 || # confers  no  title  or ownership in the Script and is not a sale  # ||
 || # of any rights to the Software. This file may not be redistributed # ||
 || # in whole or significant part.									   # ||
 || # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
 || # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
 || ##################################################################### ||
 \*=======================================================================*/
class Migrate {
    
    /**
     * Constructor
     *
     * @return void
     */
    public $CI;
    protected $default_country = 'GB'; // Set default country
    protected $domain_name = 'example.com'; // String to be appended with email ids
    protected $pmd;
    protected $table_migrate = 'migrate_pmd';
    
    // process records in chunks for larger database
    protected $process_modules = array('locations', 'categories', 'listings', 'pages', 'blogs', 'classifieds', 'users', 'packages', 'orders', 'order_customers', 'invoices');
    protected $id_limit = 20000; // number of records to be processed at a time
    protected $id_from = 0; // start from 0
    protected $id_to = 20000; // process 20000 records
    
    /**
     * The migrate_db table's fields
     *
     * @var Array
     */
    private $field_migrate_db = array(
        'id' => array(
            'type' => 'INT',
            'constraint' => 11,
            'auto_increment' => TRUE,
        ),
        'module' => array(
            'type' => 'VARCHAR',
            'constraint' => 100,
            'unique' => TRUE,
            'null' => FALSE
        ),
        'id_from' => array(
            'type' => 'INT',
            'constraint' => 11,
            'null' => FALSE,
            'default'=> '0'
        ),
        'id_to' => array(
            'type' => 'INT',
            'constraint' => 11,
            'null' => FALSE,
            'default' => 1
        ),
        'processed' => array(
            'type' => 'TINYINT',
            'constraint' => 1,
            'null' => FALSE,
            'default' => 0
        )
    );
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        $this->CI =& get_instance();
        $this->CI->load->helper('application');
        $this->CI->load->library('form_validation');
        $this->CI->load->library('session');
        $this->CI->load->helper('gc');
        $this->CI->load->library('gc');
        $this->CI->load->library('settings/settings_lib');
        $this->CI->load->dbforge();
        $this->pmd = $this->CI->load->database('migrate', TRUE);
        
        // Disable hooks, since they may rely on an installed environment.
        get_instance()->hooks->enabled = false;
    }
    
    /**
     * Migrate DB to GoClixy
     * @param array $data
     * @return boolean
     */
    public function pmd($data = array()) {
        // $default_country = null, $domain_name = null, $limit = 0
        if(!empty($data['country_iso'])) {
            $this->default_country = $data['country_iso'];
        }
        if(!empty($data['website'])) {
            $this->domain_name = $data['website'];
        }
        if(!empty($data['id_limit'])) {
            $this->id_limit = $data['id_limit'];
        }
        if(!empty($data['id_from'])) {
            $this->id_from = $data['id_from'];
        }
        if(!empty($data['id_to'])) {
            $this->id_to = $data['id_to'];
        }
        // create migrate table to process records in chunks
        if(!$this->CI->db->table_exists($this->table_migrate)) {
            $this->CI->dbforge->add_field($this->field_migrate_db);
            $this->CI->dbforge->add_key('id', true);
            $this->CI->dbforge->create_table($this->table_migrate);
            
            // drop tables
            $this->drop_tables();
            
            // add fields
            $this->add_fields();
        }
        
        // process everything in chunks
        $this->check_processed();
        
        $data_migrated = false;
        for($m = 1; $m <= 50; $m++) {
            if($m != 1) { // increment ids on each iteration
                $this->id_from = $this->id_to + 1;
                $this->id_to = $this->id_to + $this->id_limit;
            }
            if(!empty($this->process_modules)) { // process everything until the process_modules array is not empty
                
                // migrate only if module exists in the process_modules array
                if(in_array('locations', $this->process_modules)) {
                    $this->migrate_locations();
                }
                if(in_array('categories', $this->process_modules)) {
                    $this->migrate_categories();
                }
                if(in_array('listings', $this->process_modules)) {
                    $this->migrate_listings();
                }
                if(in_array('pages', $this->process_modules)) {
                    $this->migrate_pages();
                }
                if(in_array('blogs', $this->process_modules)) {
                    $this->migrate_blogs();
                }
                if(in_array('classifieds', $this->process_modules)) {
                    $this->migrate_classifieds();
                }
                if(in_array('users', $this->process_modules)) {
                    $this->migrate_users();
                }
                if(in_array('packages', $this->process_modules)) {
                    $this->migrate_packages();
                }
                if(in_array('orders', $this->process_modules)) {
                    $this->migrate_orders();
                }
                if(in_array('order_customers', $this->process_modules)) {
                    $this->migrate_order_customers();
                }
                if(in_array('invoices', $this->process_modules)) {
                    $this->migrate_invoices();
                }
            } else {
                $data_migrated = true;
                break;
            }
        }
        if($data_migrated) {
            // All items are processed, do not proceed further
            $pmd_migrated = $this->CI->db->where(array('module' => 'pmd_migrated', 'processed' => 1))->count_all_results($this->table_migrate);
            if($pmd_migrated == 0) {
                
                // Migrate Order Item information
                $this->migrate_order_item(); // Run this only once
                
                // Copy images
                $oldpath = FCPATH . 'assets/pmd/files/logo/';
                $newpath = FCPATH . 'assets/images/';
                $this->copy_files('logo', $oldpath, $newpath);
                
                // Copy Images
                $oldpath = FCPATH . 'assets/pmd/files/images/';
                $newpath = FCPATH . 'assets/images/';
                $this->copy_files('logo', $oldpath, $newpath);
                
                // Copy Classified Images
                $oldpath = FCPATH . 'assets/pmd/files/classifieds/';
                $newpath = FCPATH . 'assets/images/';
                $this->copy_files('classifieds', $oldpath, $newpath);
                
                // Copy Banner Images
                $oldpath = FCPATH . 'assets/pmd/files/banner/';
                $newpath = FCPATH . 'assets/images/';
                $this->copy_files('banners', $oldpath, $newpath);
                
                // Regenerate thumbnails for each module
                $this->CI->load->model('images/images_model');
                $this->CI->images_model->thumbnail_generate('listings');
                
                // set active country
                $this->CI->db->where('iso', $this->default_country)->update('countries', array('active' => 1));
                
                // Set Administrator - Set in the migrate query to avoid issues on timeout login
                // $this->CI->db->where('id', 1)->update('users', array('role_id' => 1));
                
                $this->CI->db->on_duplicate($this->table_migrate, array('module' => 'pmd_migrated', 'id_from' => $this->id_from, 'id_to' => $this->id_to, 'processed' => 1));
            }
            return true;
        }
        return false;
    }
    
    /**
     * Migrate Locations
     * 
     * INSERT IGNORE INTO rs0d_states (id, country_iso, name, slug, active) SELECT 
id, "US" AS country_iso, title as name, friendly_url AS slug, 1 AS active FROM myusatto_main23.pmd_locations WHERE level = 1
     * 
     * INSERT IGNORE INTO rs0d_cities (id, country_iso, state_id, name, slug, hits, active) SELECT 
id, "US" AS country_iso, parent_id AS state_id, title as name, friendly_url AS slug, impressions AS hits, 1 AS active FROM myusatto_main23.pmd_locations WHERE level = 2

     */
    private function migrate_locations() {
        // States Table
        $states_destination_columns = array('id', 'country_iso', 'name', 'slug', 'active');
        $states_source_columns = array('id', '"' . $this->default_country . '" AS country_iso', 'title AS name', 'friendly_url AS slug', '1 AS active');
        $this->copy_tables('locations', 'states', $states_source_columns, $states_destination_columns, 'level = 1');
        
        // Cities Table - Level 2
        $cities_destination_columns = array('id', 'country_iso', 'state_id', 'name', 'slug', 'hits', 'active');
        $cities_source_columns = array('id', '"' . $this->default_country . '" AS country_iso', 'parent_id AS state_id', 'title AS name', 'friendly_url AS slug', 'impressions AS hits', '1 AS active');
        $this->copy_tables('locations', 'cities', $cities_source_columns, $cities_destination_columns, 'level = 2');
        
        // Localities Table - Level 3
        $localities_destination_columns = array('id', 'country_iso', 'state_id', 'city_id', 'name', 'slug', 'hits', 'active');
        $localities_source_columns = array('id', '"' . $this->default_country . '" AS country_iso', '0 AS state_id', 'parent_id AS city_id', 'title AS name', 'friendly_url AS slug', 'impressions AS hits', '1 AS active');
        $this->copy_tables('locations', 'localities', $localities_source_columns, $localities_destination_columns, 'level = 3');
    }
    
    /**
     * Migrate Categories
     */
    private function migrate_categories() {
        $destination_columns = array('id', 'parent_id', 'name', 'slug', 'description', 'meta_title', 'meta_description', 'meta_keywords', 'counts', 'hits', 'active');
        $source_columns = array('id', '(CASE WHEN parent_id = 1 THEN 0 ELSE parent_id END) AS parent_id', 'title AS name', 'friendly_url AS slug', 'description', 'meta_title', 'meta_description', 'meta_keywords', 'count AS counts', 'impressions AS hits', '1 AS active');
        $this->copy_tables('categories', 'categories', $source_columns, $destination_columns, 'parent_id IS NOT NULL');
        $this->migrate_module_relations(); // Move categories module relations
    }
    
    /**
     * Migrate Module Relations
     */
    private function migrate_module_relations() {
        $module_category_id = gc_module_id('categories');
        $module_listing_id = gc_module_id('listings');
        $destination_columns = array('row_id', 'module_id', 'toggle_module_id');
        $source_columns = array('id', $module_category_id . ' AS module_id', $module_listing_id . ' AS toggle_module_id');
        $this->copy_tables('categories', 'module_relations', $source_columns, $destination_columns, 'parent_id IS NOT NULL');
    }
    
    /**
     * Migrate Pages
     */
    private function migrate_pages() {
        $destination_columns = array('id', 'title', 'slug', 'body', 'meta_title', 'meta_description', 'meta_keywords', 'status');
        $source_columns = array('id', 'title', 'friendly_url AS slug', 'content AS body', 'meta_title', 'meta_description', 'meta_keywords', '(CASE WHEN active = 1 THEN 1 ELSE 0 END) AS status');
        $this->copy_tables('pages', 'pages', $source_columns, $destination_columns);
    }
    
    /**
     * Migrate Blogs
     */
    private function migrate_blogs() {
        $destination_columns = array('id', 'user_id', 'package_id', 'title', 'slug', 'description', 'hits', 'meta_title', 'meta_description', 'meta_keywords', 'status', 'created_on');
        $source_columns = array('id', 'user_id', '1 AS package_id', 'title', 'friendly_url AS slug', 'content AS description', 'impressions AS hits', 'meta_title', 'meta_description', 'meta_keywords', '(CASE WHEN status = "active" THEN 1 ELSE 0 END) AS status', 'date AS created_on');
        $this->copy_tables('blog', 'blogs', $source_columns, $destination_columns);
    }    
    
    /**
     * Migrate Classifieds
     */
    private function migrate_classifieds() {
        $destination_columns = array('id', 'listing_id', 'user_id', 'package_id', 'title', 'slug', 'description', 'external_url', 'hits', 'meta_title', 'meta_description', 'meta_keywords', 'status', 'created_on');
        $source_columns = array('id', 'listing_id', '1 AS user_id', '1 AS package_id', 'title', 'friendly_url AS slug', 'description', 'www AS external_url', '0 AS hits', 'meta_title', 'meta_description', 'meta_keywords', '1 AS status', 'date AS created_on');
        $this->copy_tables('classifieds', 'classifieds', $source_columns, $destination_columns);
    }
    
    /**
     * Migrate Users
     */
    private function migrate_users() {
        $destination_columns = array(
            'id', 'role_id', 'username', 'pass', 'password_salt', 'password_hash', 'email', 'display_name', 'address', 'country_iso', 'city', 'state', 'country',
            'postcode', 'phone_number', 'fax', 'organization', 'active', 'last_login', 'last_ip', 'created_on', 'timezone'
        );
        $source_columns = array(
            'id', 'CASE WHEN id = 1 THEN 1 ELSE 2 END AS role_id', 'login AS username', 'pass', 'password_salt', 'password_hash',
            'CASE WHEN user_email != "" THEN user_email ELSE CONCAT(login, "@' . $this->domain_name . '") END AS email',
            'CASE WHEN user_first_name != "" THEN CONCAT(user_first_name, " " , user_last_name) END AS display_name',
            'CASE WHEN user_address1 != "" THEN CONCAT(user_address1, ", ", user_address2) END AS address', '"' . $this->default_country . '" AS country_iso',
            'user_city AS city', 'user_state AS state', 'user_country AS country', 'user_zip AS postcode',
            'user_phone AS phone_number', 'user_fax AS fax', 'user_organization AS organization', '1 AS active', 'logged_last AS last_login',
            'logged_ip AS last_ip', 'created AS created_on', 'timezone'
        );
        $this->copy_tables('users', 'users', $source_columns, $destination_columns);
    }
    
    /**
     * Migrate Packages
     */
    private function migrate_packages() {
        $destination_columns = array(
            'id', 'title', 'description', 'active', 'price', 'order_status'
        );
        $source_columns = array(
            'P.id', 'P.name AS title', 'P.description', 'PP.active', 'PP.price', '1 AS order_status'
        );
        $this->copy_tables('products', 'packages', $source_columns, $destination_columns);
    }
    
    /**
     * Migrate Orders
     */
    private function migrate_orders() {
        $destination_columns = array(
            'id', 'order_id', 'invoice_id', 'order_type', 'user_id', 'package_id',
            'order_subtotal', 'order_tax', 'order_total', 'order_discount',
            'module_id', 'row_id', 'payment_method', 'created_on', 'status', 'ip_address'
        );
        $source_columns = array(
            'O.id', 'CONCAT("OD", O.order_id) AS order_id', 'I.id AS invoice_id', '1 AS order_type', 'O.user_id', 'O.pricing_id AS package_id',
            'CASE WHEN I.subtotal IS NOT NULL THEN I.subtotal ELSE "0.00" END AS order_subtotal',
            'CASE WHEN I.tax IS NOT NULL THEN I.tax ELSE "0.00" END AS order_tax',
            'CASE WHEN I.total IS NOT NULL THEN I.total ELSE "0.00" END AS order_total',
            'CASE WHEN I.discount_code_value IS NOT NULL THEN I.discount_code_value ELSE "0.00" END AS order_discount',
            '1 AS module_id', 'O.type_id AS row_id', 'O.gateway_id AS payment_method', 'O.date AS created_on',
            'CASE WHEN O.status = "active" THEN 1 ELSE 0 END AS status', 'O.ip_address'
        );
        $this->copy_tables('orders', 'orders', $source_columns, $destination_columns);
    }
    
    /**
     * Migrate Order Customers
     */
    private function migrate_order_customers() {
        $destination_columns = array(
            'order_id', 'user_id', 'name', 'email', 'phone_number', 'address', 'postcode', 'city_id', 'city'
        );
        $source_columns = array(
            'CONCAT("OD", O.order_id) AS order_id', 'O.user_id',
            'CASE WHEN U.user_first_name != "" THEN CONCAT(U.user_first_name, " ", U.user_last_name) END AS name',
            'CASE WHEN U.user_email != "" THEN U.user_email ELSE CONCAT(U.login, "@' . $this->domain_name . '") END AS email',
            'U.user_phone AS phone_number',
            'CASE WHEN U.user_address1 != "" THEN CONCAT(U.user_address1, ", ", U.user_address2) END AS address',
            'U.user_zip AS postcode', '0 AS city_id', 'U.user_city AS city'
        );
        $this->copy_tables('orders', 'order_customers', $source_columns, $destination_columns);
    }
    
    /**
     * Migrate Order Items
     */
    private function migrate_order_item() {
        // Pacakges table form fields in an array
        $packages = $this->CI->db->select('id, title, description')->get('packages')->result();
        if($packages) {
            foreach($packages as $package) {
                $item = array(
                    array(
                        'title' => $package->title,
                        'description' => $package->description
                    )
                );
                $this->CI->db->where('package_id', $package->id)->update('orders', array('item' => json_encode($item)));
            }
        }
    }
    
    /**
     * Migrate Invoices
     */
    private function migrate_invoices() {
        $destination_columns = array('prefix', 'order_id', 'user_id', 'date_paid', 'created_on', 'due_date', 'next_invoice_date', 'status');
        $source_columns = array(
            '"INV" AS prefix', 'CONCAT("OD", O.order_id) AS order_id', 'I.user_id',
            'I.date_paid', 'O.date AS created_on',
            'CASE WHEN O.next_due_date = "0000-00-00" THEN NULL ELSE O.next_due_date END AS due_date',
            'CASE WHEN O.next_invoice_date = "0000-00-00" THEN NULL ELSE O.next_invoice_date END AS next_invoice_date',
            'CASE WHEN I.status = "paid" THEN 1 ELSE 0 END AS status'
        );
        $this->copy_tables('invoices', 'invoices', $source_columns, $destination_columns, 'I.order_id IS NOT NULL');
    }
    
    /**
     * Migrate Images
     */
    private function migrate_images() {
        // SELECT 1 AS module_id, listing_id AS row_id, 2 AS type_id, title, CONCAT('assets/images/listings/logos/', listing_id, '.', extension) AS url, 'assets/images/listings/logos/' AS path, CONCAT(listing_id, '.', extension) AS filename FROM `pmd_images`
        $source_columns = array('1 AS module_id', 'listing_id AS row_id', '2 AS type_id', 'title',
            'CONCAT("assets/images/listings/logos/", listing_id, ".", extension) AS url', '"assets/images/listings/logos/" AS path',
            'CONCAT(listing_id, ".", extension) AS filename', '1 AS status');
        $destination_columns = array('module_id', 'row_id', 'type_id', 'title', 'url', 'path', 'filename', 'status');
        $this->copy_tables('images', 'images', $source_columns, $destination_columns);
    }
    
    
    /**
     * Generate Thumbnails
     * @param string $module
     */
    private function copy_files($module, $oldpath, $newpath) {
        // $oldpath = $oldpath . $module . '/';
        if($module == 'logo') {
            $newpath .= 'listings/logos/';
        } else {
            $newpath .= ($module == 'photos' ? 'listings' : $module) . '/';
        }
        
        /**
         * Sub-folders contained within the directory will be mapped as well.
         * If you wish to control the recursion depth, you can do so using the second parameter (integer).
         * A depth of 1 will only map the top level directory
         * By default, hidden files will not be included in the returned array.
         * To override this behavior, you may set a third parameter to true (boolean):
         */
        //$files = directory_map('./assets/images/' . $module, 1);
        $files = directory_map($oldpath);
        if(!$files) {
            return false;
        }
        asort($files);
        
        // copy files recursively - inside folders and subfolders
        $this->recursive_copy($files, $oldpath, $newpath);
        
        // Regenerate thumbnails for each module
        // $this->CI->images_model->thumbnail_generate($module);
    }
    
    /**
     * Recursively copy files inside a subdirectory
     * @param array $files
     * @param string $oldpath
     * @param string $newpath
     */
    private function recursive_copy($files, $oldpath, $newpath) {
        if (!file_exists($newpath)) { // if newpath does not exist, create it
            mkdir($newpath, 0777, true);
        }
        foreach($files as $file_folder => $file) {
            // get only image files based on their extensions
            if(is_string($file)) {
                $info = pathinfo($file);
                if(isset($info['extension'])) {
                    switch(strtolower($info['extension'])) {
                        case "jpg":
                        case "jpeg":
                        case "png":
                        case "gif":
                            if(file_exists($oldpath . $info['basename'])) {
                                copy($oldpath . $info['basename'], $newpath . $info['basename']);
                            }
                            break;
                    }
                }
            } else {
                $this->recursive_copy($file, $oldpath . $file_folder, $newpath . $file_folder);
            }
        }
    }
    
    /**
     * Migrate Listings
     */
    private function migrate_listings() {
        /*
         * SELECT L.id, L.user_id, O.pricing_id AS package_id, L.primary_category_id AS category_id, L.title,
         L.friendly_url AS slug, L.description, L.meta_title, L.meta_description,
         L.meta_keywords, L.keywords, L.phone AS phone_number, L.fax, L.mail AS email,
         CONCAT(L.listing_address1, ', ', L.listing_address2) AS address, L.listing_zip AS postalcode, L.location_id AS city_id,
         L.latitude, L.longitude, L.www AS website,
         (CASE WHEN
         L.facebook_page_id IS NOT NULL AND L.facebook_page_id != '' THEN
         CONCAT('https://www.facebook.com/', L.facebook_page_id) ELSE NULL END) AS facebook_url,
         L.ip AS created_ip, L.date AS created_on, L.date_update AS modified_on, L.ip_update AS modified_ip,
         L.impressions AS hits, L.featured, L.address_allow AS show_address, L.phone_allow AS show_phone,
         L.email_allow AS show_email, L.claim_allow AS show_claim
         FROM pmd_listings L
         LEFT JOIN pmd_orders O ON O.type_id = L.id
         */
        $destination_columns = array('id', 'user_id', 'package_id', 'category_id', 'title', 'slug',
            'description', 'meta_title', 'meta_description', 'meta_keywords', 'keywords',
            'phone_number', 'fax', 'email', 'address', 'postcode', 'city_id', 'latitude', 'longitude',
            'website', 'created_ip', 'created_on', 'modified_on', 'modified_ip', 'hits', 'featured',
            'show_address', 'show_phone', 'show_email', 'show_claim', 'status');
        $source_columns = array('L.id', 'L.user_id', 'O.pricing_id AS package_id', 'L.primary_category_id AS category_id', 'L.title', 'L.friendly_url AS slug',
            'L.description', 'L.meta_title', 'L.meta_description', 'L.meta_keywords', 'L.keywords',
            'L.phone AS phone_number', 'L.fax', 'L.mail AS email', 'CONCAT(L.listing_address1, ", ", L.listing_address2) AS address', 'L.listing_zip AS postcode', 'L.location_id AS city_id',
            'L.latitude', 'L.longitude', 'L.www AS website', 'L.ip AS created_ip', 'L.date AS created_on', 'L.date_update AS modified_on', 'L.ip_update AS modified_ip',
            'L.impressions AS hits', 'L.featured', 'L.address_allow AS show_address', 'L.phone_allow AS show_phone', 'L.email_allow AS show_email', 'L.claim_allow AS show_claim',
            '(CASE WHEN L.status = "active" THEN 1 ELSE 0 END) AS status'
        );
        $this->copy_tables('listings', 'listings', $source_columns, $destination_columns);
        $this->migrate_categories_relation();
        $this->migrate_keywords();
        $this->update_locations();
        $this->migrate_business_hours();
        $this->migrate_listings_social();
        $this->migrate_images();
    }
    
    /**
     * Migrate keywords in the tags table
     */
    private function migrate_keywords() {
        $sql = "SELECT * FROM " . $this->CI->db->dbprefix . "listings WHERE keywords IS NOT NULL AND keywords != ''";
        $sql .= " AND (id BETWEEN " . $this->id_from . " AND " . $this->id_to . ")";
        $records = $this->CI->db->query($sql)->result();
        if(!empty($records)) {
            $data = array();
            $this->CI->load->model('tags/tags_model');
            foreach($records as $record) {
                $this->CI->tags_model->save_module_tags($record->keywords, $record->id, 'listings', 1);
            }
        }
    }
    
    /**
     * Migrate Categories
     */
    private function migrate_categories_relation() {
        $destination_columns = array('module_id', 'row_id', 'parent_id', 'sub_id', 'subsub_id');
        $source_columns = array('1 AS module_id', 'LC.list_id AS row_id',
            'CASE WHEN C.level = 1 THEN LC.cat_id ELSE C.parent_id END AS parent_id',
            'CASE WHEN C.level = 2 THEN C.id ELSE C.parent_id END AS sub_id',
            'CASE WHEN C.level = 3 THEN C.id ELSE 0 END AS subsub_id');
        $this->copy_tables('listings_categories', 'categories_relation', $source_columns, $destination_columns, 'parent_id IS NOT NULL');
    }
    
    /**
     * Update country_iso and state_id in the listings table
     */
    private function update_locations() {
        $sql = "SELECT DISTINCT(city_id) AS city_id FROM " . $this->CI->db->dbprefix . "listings";
        $sql .= " WHERE (id BETWEEN " . $this->id_from . " AND " . $this->id_to . ")";
        $records = $this->CI->db->query($sql)->result();
        if(!empty($records)) {
            // get city information from the cities table
            foreach($records as $record) {
                $data = array();
                // search localities table first                
                $locality = $this->CI->db->get_where('localities', array('id' => $record->city_id))->row();
                if($locality) {
                    if($locality->state_id == 0) {
                        // get state_id from the cities table and update the localities table
                        $city = $this->CI->db->get_where('cities', array('id' => $locality->city_id))->row();
                        if($city) {
                            $this->CI->db->where('id', $locality->id)->update('localities', array('state_id' => $city->state_id));
                            $locality->state_id = $city->state_id;
                        }
                    }
                    $data = array(
                        'country_iso' => $city->country_iso,
                        'state_id' => $locality->state_id,
                        'city_id' => $locality->city_id,
                        'locality_id' => $locality->id
                    );
                    $this->CI->db->where(array('state_id' => 0, 'city_id' => $record->city_id))->update('listings', $data);
                } else {
                    $city = $this->CI->db->get_where('cities', array('id' => $record->city_id))->row();
                    if($city) {
                        $data = array(
                            'country_iso' => $city->country_iso,
                            'state_id' => $city->state_id
                        );
                        $this->CI->db->where(array('state_id' => 0, 'city_id' => $record->city_id))->update('listings', $data);
                    } else { // city does not exist
                        // get data from the states table
                        $state = $this->CI->db->get_where('states', array('id' => $record->city_id))->row();
                        if($state) {
                            $data = array(
                                'country_iso' => $state->country_iso,
                                'state_id' => $state->id
                            );
                            $this->CI->db->where(array('state_id' => 0, 'city_id' => $record->city_id))->update('listings', $data);
                            
                            // also insert information in the city table
                            $city_data = $data;
                            $city_data['id'] = $record->city_id;
                            $city_data['name'] = $state->name;
                            $city_data['slug'] = $state->slug;
                            $this->CI->db->on_duplicate('cities', $city_data);
                        }
                    }
                }
            }
        }
    }
    
    /**
     * Migrate listings social
     */
    private function migrate_listings_social() {
        $socials = array(
            'facebook_page_id' => array('https://www.facebook.com/', 'facebook_url'),
            'google_page_id' => array('https://plus.google.com/', 'googleplus_url'),
            'linkedin_id' => array('https://www.linkedin.com/company/', 'linkedin_url'),
            'linkedin_company_id' => array('https://www.linkedin.com/company/', 'linkedin_url'),
            'twitter_id' => array('https://twitter.com/', 'twitter_url'),
            'pinterest_id' => array('https://www.pinterest.com/', 'pinterest_url'),
            'youtube_id' => array('https://www.youtube.com/channel/', 'youtube_url'),
            'foursquare_id' => array('https://foursquare.com/', 'foursquare_url'),
            'instagram_id' => array('https://www.instagram.com/', 'instagram_url'),
        );
        foreach($socials as $source => $params) {
            $sql = "SELECT * FROM " . $this->pmd->dbprefix . "listings";
            $sql .= " WHERE (id BETWEEN " . $this->id_from . " AND " . $this->id_to . ")";
            $sql .= " AND " . $source . " IS NOT NULL AND " . $source . " != ''";
            $records = $this->pmd->query($sql)->result();
            if(!empty($records)) {
                $data = array();
                $regexp =  "/((?:https?\:\/\/|www\.)(?:[-a-z0-9]+\.)*[-a-z0-9]+.*)/i";
                foreach($records as $record) {
                    if(preg_match($regexp, $record->{$source})) {
                        // url does not start with http or https protocol
                        if(substr($record->{$source}, 0, 4) == "www.") {
                            // append http to the image url
                            $url = 'https://'. $record->{$source};
                        }
                    } else {
                        $url = $params[0] . $record->{$source};
                    }
                    if(!empty($url)) {
                        $data[] = array(
                            'id' => $record->id,
                            $params[1] => $url
                        );
                    }
                }
                if(!empty($data)) {
                    $this->CI->db->update_batch('listings', $data, 'id'); // update batch based on id
                }
            }
        }
    }
    
    /**
     * Migrate business hours
     */
    private function migrate_business_hours() {
        $sql = "SELECT id, hours FROM " . $this->pmd->dbprefix . "listings";
        $sql .= " WHERE hours IS NOT NULL AND hours != '' AND hours != 'N;'";
        $sql .= " AND (id BETWEEN " . $this->id_from . " AND " . $this->id_to . ")";
        $records = $this->pmd->query($sql)->result();
        if($records) {
            foreach($records as $record) {
                $business_hours = unserialize($record->hours);
                if(!empty($business_hours)) {
                    $data = array();
                    foreach($business_hours as $business_hour) {
                        $business_hour = explode(' ', $business_hour ?? '');
                        if(!empty($business_hour[0]) && !empty($business_hour[1]) && !empty($business_hour[2])) {
                            $data[] = array(
                                'listing_id' => $record->id,
                                'day_of_week' => $business_hour[0],
                                'open_time' => $business_hour[1],
                                'close_time' => $business_hour[2]
                            );
                        }
                    }
                    if(!empty($data)) {
                        $this->CI->db->on_duplicate('business_hours', $data);
                    }
                }
            }
        }
    }
    
    /**
     * Import Locations programmatically
     */
    private function import_locations() {
        $records = $this->pmd->get('locations');
        if($records) {
            $states = array();
            $cities = array();
            foreach($records as $record) {
                $data = array(
                    'id' => $record->id,
                    'country_iso' => $this->default_country,
                    'name' => $record->title,
                    'slug' => $record->friendly_url
                );
                if($record->level == 1) {
                    $states[] = $data;
                }
                if($record->level == 2) {
                    $data['state_id'] = $record->parent_id;
                    $data['hits'] = $record->impressions;
                    $cities[] = $data;
                }
            }
            if(!empty($states)) {
                $this->CI->db->truncate('states');
                $this->CI->db->insert('states', $states);
            }
            if(!empty($cities)) {
                $this->CI->db->truncate('cities');
                $this->CI->db->insert('cities', $cities);
            }
        }
    }
    
    /**
     * Copy Tables using MySQL command - Links and Fields
     * @param string $table_source - Source Table Name
     * @param string $table_bak - Backup/Old Table Name
     * @param boolean $insert_only - TRUE - Perform insert only do not update
     */
    private function copy_tables($table_source, $table_destination, $source_columns = null, $destination_columns = null, $where = null, $insert_only = true) {
        if(empty($source_columns) && empty($destination_columns)) {
            $existing_column_names = $this->CI->db->list_fields($table_destination);
            $new_column_names = $this->pmd->list_fields($table_source);
            $columns = array_intersect($existing_column_names, $new_column_names);
            if(!empty($columns)) {
                $columns_comma_separated = implode(',', $columns);
                $source_columns = $columns_comma_separated;
                $destination_columns = $columns_comma_separated;
            }
        } else {
            $source_columns = implode(',', $source_columns);
            $destination_columns = implode(',', $destination_columns);
        }
        if(!empty($source_columns) && !empty($destination_columns)) {
            $sql = "INSERT";
            if($insert_only) {
                $sql .= " IGNORE";
            }
            $table_source_prefix = '';
            $sql .= " INTO " . $this->CI->db->dbprefix . $table_destination . " (" . $destination_columns . ")";
            $sql .= " SELECT " . $source_columns . " FROM " . $this->pmd->database . '.' . $this->pmd->dbprefix . $table_source;
            switch($table_source) {
                case 'listings':
                    $table_source_prefix = 'L.';
                    $sql .= " L";
                    $sql .= " LEFT JOIN " . $this->pmd->database . '.' . $this->pmd->dbprefix ."orders O ON O.type_id = L.id";
                    break;
                case 'listings_categories':
                    $table_source_prefix = 'LC.list_';
                    $sql .= " LC";
                    $sql .= " LEFT JOIN " . $this->pmd->database . '.' . $this->pmd->dbprefix ."categories C ON LC.cat_id = C.id";
                    break;
                case 'products':
                    $table_source_prefix = 'P.';
                    $sql .= " P";
                    $sql .= " LEFT JOIN " . $this->pmd->database . '.' . $this->pmd->dbprefix ."products_pricing PP ON PP.product_id = P.id";
                    break;
                case 'orders':
                    $table_source_prefix = 'O.';
                    $sql .= " O";
                    if($table_destination == 'order_customers') {
                        $sql .= " LEFT JOIN " . $this->pmd->database . '.' . $this->pmd->dbprefix ."users U ON U.id = O.user_id";
                    } else {
                        $sql .= " LEFT JOIN " . $this->pmd->database . '.' . $this->pmd->dbprefix ."invoices I ON I.order_id = O.id";
                    }
                    break;
                case 'invoices':
                    $table_source_prefix = 'I.';
                    $sql .= " I";
                    $sql .= " LEFT JOIN " . $this->pmd->database . '.' . $this->pmd->dbprefix ."orders O ON I.order_id = O.id";
                    break;
            }
            if(!empty($where)) {
                $sql .= " WHERE " . $where;
                $sql .= " AND (" . $table_source_prefix . "id BETWEEN " . $this->id_from . " AND " . $this->id_to . ")";
            } else {
                $sql .= " WHERE (" . $table_source_prefix . "id BETWEEN " . $this->id_from . " AND " . $this->id_to . ")";
            }
            
            if(!$insert_only) {
                $duplicate_columns = array();
                foreach($source_columns as $column) {
                    $duplicate_columns[] = $column . '= VALUES(' . $column . ')';
                }
                $sql .= " ON DUPLICATE KEY UPDATE " . implode(',', $duplicate_columns);
            }
            $this->CI->db->query($sql);
            
            // set processed
            $this->set_processed($table_source, $table_destination);
        }
    }
    
    /**
     * Drop Tables
     */
    private function drop_tables() {
        $tables = array('states', 'cities', 'listings', 'business_hours', 'categories', 'pages', 'blogs', 'classifieds', 'categories_level', 'tags', 'tag_relations', 'users', 'packages', 'orders', 'order_customers', 'invoices');
        foreach($tables as $table) { // drop tables before import
            $this->CI->db->truncate($table);
        }
        
        // delete information from the module relations table
        $this->CI->db->where('module_id', gc_module_id('categories'))->delete('module_relations');
        
        // delete information from the module relations table
        $this->CI->db->where('module_id', gc_module_id('listings'))->delete('images');
    }
    
    // add social links
    // process business hours
    // update state_id and country_iso in the listings table
    // pages
    // move categories and tags using php
    // users
    // roles to be set in the users table
    // packages
    // orders
    // invoices
    // order customers to be added
    // order items to be udpated
    
    /**
     * Add extra fields
     */
    private function add_fields() {
        $newly_fields = array();
        
        // Add Fields to the listings Table
        // description_short, foursquare_url in the listings table
        $newly_fields['listings'] = array(
            'foursquare_url' => array('type' => 'VARCHAR', 'constraint' => 255, 'null' => TRUE, 'after' => 'instagram_url'),
            'description_short' => array('type' => 'TEXT', 'null' => TRUE, 'after' => 'foursquare_url')
        );
        
        // Add Fields to the listings Table
        // pass, password_salt, fax and organization in the users table
        $newly_fields['users'] = array(
            'pass' => array('type' => 'VARCHAR', 'constraint' => 64, 'null' => TRUE, 'after' => 'username'),
            'password_salt' => array('type' => 'VARCHAR', 'constraint' => 32, 'null' => TRUE, 'after' => 'pass'),
            'fax' => array('type' => 'VARCHAR', 'constraint' => 20, 'null' => TRUE, 'after' => 'phone_number'),
            'organization' => array('type' => 'VARCHAR', 'constraint' => 255, 'null' => TRUE, 'after' => 'fax')
        );
        
        // Add Fields to the order_customers Table
        // city in the order_customers table
        $newly_fields['order_customers'] = array(
            'city' => array('type' => 'VARCHAR', 'constraint' => 255, 'null' => TRUE, 'after' => 'city_id')
        );
        
        // Add fields
        foreach($newly_fields as $field_table => $newly_field) {
            $field_count[$field_table] = 0;
            foreach($newly_field as $field_name => $field_structure) {
                if (!$this->CI->db->field_exists($field_name, $field_table)) {
                    $fields = array($field_name => $field_structure);
                    $this->CI->dbforge->add_column($field_table, $fields);
                }
                $field_count[$field_table] += 1;
            }
        }
    }
    
    /**
     * Check processed
     */
    private function check_processed() {
        $migrate_history = $this->CI->db->order_by('id_to', 'DESC')->get($this->table_migrate)->result();
        if($migrate_history) { // process everything
            $i = 1;
            foreach($migrate_history as $row) {
                if($row->processed == 1 && in_array($row->module, $this->process_modules)) {
                    // do not process task further
                    unset($this->process_modules[array_search($row->module, $this->process_modules)]);
                }
                if($i == 1) {
                    $this->id_from = $row->id_from;
                    $this->id_to = $row->id_to;
                }
                $i++;
            }
        }
    }
    
    /**
     * Set processed status in the migrate table
     * @param string $table_source
     */
    private function set_processed($table_source, $table_destination = null) {
        $field = 'id';
        if($table_source == 'listings_categories') {
            $field = 'list_id';
        }
        $count = $this->pmd->where($field . ' >', $this->id_to)->count_all_results($table_source);
        if($count == 0) {
            // unset the module from the processed_modules array
            // do not process with this module further
            switch($table_source) {
                case 'blog':
                    $table_source = 'blogs';
                    break;
                case 'products':
                    $this->CI->db->on_duplicate($this->table_migrate, array('module' => $table_source, 'id_from' => $this->id_from, 'id_to' => $this->id_to, 'processed' => 1));
                    $table_source = 'packages';
                    break;
                case 'orders':
                    if($table_destination == 'order_customers') {
                        $this->CI->db->on_duplicate($this->table_migrate, array('module' => $table_source, 'id_from' => $this->id_from, 'id_to' => $this->id_to, 'processed' => 1));
                        $table_source = 'order_customers';
                    }
                    break;
            }
            $this->CI->db->on_duplicate($this->table_migrate, array('module' => $table_source, 'id_from' => $this->id_from, 'id_to' => $this->id_to, 'processed' => 1));
            // unset the module from the processed_modules array
            unset($this->process_modules[array_search($table_source, $this->process_modules)]);
        } else {
            $this->CI->db->on_duplicate($this->table_migrate, array('module' => $table_source, 'id_from' => $this->id_from, 'id_to' => $this->id_to));
        }
    }
    
    /**
     * Drop the migrate table
     */
    private function drop_processed() {
        $this->CI->dbforge->drop_table($this->table_migrate);
    }
    
    /**
     * Check image existence, if not exist, use a different extension and find the file
     */
    private function image_existence() {
        $records = $this->CI->db->get_where('images', array('type_id' => 2))->result();
        if($records) {
            foreach($records as $record) {
                if(!file_exists($record->url)) {
                    // check with png extension
                    $png_file = str_replace('.png', '.jpg', $record->url);
                    if(file_exists($png_file)) {
                        $data = array(
                            'url' => $png_file,
                            'filename' => str_replace('.png', '.jpg', $record->filename)
                        );
                        $this->CI->db->where('id', $record->id)->update('images', $data);
                    }
                }
            }
        }
    }
}