<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Migrate_model extends BF_Model {

	protected $table_name	= "migrations";
	protected $key			= "id";
	protected $soft_deletes	= false;
	protected $date_format	= "datetime";

	protected $log_user 	= FALSE;

	protected $set_created	= true;
	protected $set_modified = false;
	protected $created_field = "performed_on";

	/*
		Customize the operations of the model without recreating the insert, update,
		etc methods by adding the method names to act as callbacks here.
	 */
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 		= array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	/*
		For performance reasons, you may require your model to NOT return the
		id of the last inserted row as it is a bit of a slow method. This is
		primarily helpful when running big loops over data.
	 */
	protected $return_insert_id 	= TRUE;

	// The default type of element data is returned as.
	protected $return_type 			= "object";

	// Items that are always removed from data arrays prior to
	// any inserts or updates.
	protected $protected_attributes = array();

	/*
		You may need to move certain rules (like required) into the
		$insert_validation_rules array and out of the standard validation array.
		That way it is only required during inserts, not updates which may only
		be updating a portion of the data.
	 */
	public $validation_rules_import 		= array(
		array(
			"field"		=> "type",
			"label"		=> "lang:label_import",
			"rules"		=> "required"
		),
		array(
			"field"		=> "fields_separator",
			"label"		=> "lang:label_fields_separator",
			"rules"		=> "required|xss_clean"
		),
		array(
			"field"		=> "lines_separator",
			"label"		=> "lang:label_lines_separator",
			"rules"		=> "required|xss_clean"
		)
	);
	public $validation_rules_export = array(
		array(
			"field"		=> "type",
			"label"		=> "lang:label_import",
			"rules"		=> "required"
		),
	    array(
	        "field"		=> "file_type",
	        "label"		=> "lang:label_file_type",
	        "rules"		=> "required|xss_clean"
	    ),
		array(
			"field"		=> "fields_separator",
			"label"		=> "lang:label_fields_separator",
			"rules"		=> "required|xss_clean"
		),
		array(
			"field"		=> "lines_separator",
			"label"		=> "lang:label_lines_separator",
			"rules"		=> "required|xss_clean"
		), 
		array(
			"field"		=> "start_id",
			"label"		=> "lang:label_start_id",
			"rules"		=> "required|intval"
		),
		array(
			"field"		=> "end_id",
			"label"		=> "lang:label_end_id",
			"rules"		=> "required|intval"
		)
	);
	protected $insert_validation_rules 	= array();
	protected $skip_validation 			= TRUE;
	protected $migrate_fields_cache;
	
	/**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
        // Load Zend Caching
        $this->load->library('slug');
        $this->load->library('gzf_cache', array('lifetime' => 900));
        if(method_exists($this->gzf_cache, 'get_instance')) {
            $this->migrate_fields_cache = $this->gzf_cache->get_instance();
        }
    }

	//--------------------------------------------------------------------

	/**
	 * Bulk import categories information
	 * @param int $id
	 * @param string $module
	 * @param string $categories - comma separated values
	 * First category will be used as the default category
	 */
	public function bulk_import_categories($module, $row_id, $categories) {
		// $categories = array('A=>B=>C', 'D=>E=>F', 'G=>H=>I');
		$this->load->model('categories/categories_model');
		$tree = array();
		foreach($categories as $category) {
			$session_value = $category;
			$result = $this->get_import_session('categories', $session_value);
			if(!$result) {
				// check if the category name has sub/subsub separator in it
				if (strpos($category, '=>') !== false) {
					$category = explode('=>', $category);
				} else { // convert string into array
					$category = array($category);
				}
				$result = $this->categories_model->import_categories($category);
				$tree[] = $result['tree'];
				$this->save_import_session('categories', $session_value, $result);
			} else {
				$tree[] = $result['tree'];
			}
		}
		$_POST['categories'] = implode(',',$tree);
		if($module == 'listings') {
			// First category will be used as the default category
			$_POST['default_category'] = $tree[0];
		}
		$this->categories_model->save_categories_relation($row_id, $module);
	}
	
	/**
	 * Bulk import tags
	 * @param int $id
	 * @param string $module
	 * @param string $tags - comma separated values
	 */
	public function bulk_import_tags($module, $row_id, $tags) {
		$this->load->model('tags/tags_model');
		if(isset($tags['keywords'])) {
			$tags = $tags['keywords'];
		}
		$this->tags_model->save_module_tags($tags, $row_id, $module, 1);
	}
	
	/**
	 * Bulk Import Images
	 * @param string $module
	 * @param int $row_id
	 * @param string $path
	 * @param string $url
	 * @param string $filename
	 */
	public function bulk_import_images($module, $row_id, $url, $type_id = 0) {
		$this->load->model('images/images_model');
		$path = NULL;
		$filename = NULL;
		if(!empty($url)) {
			switch($type_id) {
				case 0:
					// other images
					foreach($url as $image) {
						$this->images_model->save_image_db($module, $row_id, 0, $path, $image, $filename, 1, 1, TRUE);
					}
					break;
				case 1:
					// cover image
					$this->images_model->save_image_db($module, $row_id, 1, $path, $url['cover'], $filename, 1, 1, TRUE);
					break;
				case 2:
					// logo image
					$this->images_model->save_image_db($module, $row_id, 2, $path, $url['logo'], $filename, 1, 1, TRUE);
					break;
			}
		}
	}
	
	/**
	 * Custom Fields
	 * @param string $module
	 * @param integer $row_id
	 * @param array $fields
	 */
	public function bulk_import_custom_fields($module, $row_id, $fields) {
		$module_id = gc_module_id($module);
		if(!empty($fields)) {
			$data = array();
			foreach($fields as $key => $value) {
				$data[] = array(
					'module_id' => $module_id,
					'row_id' => $row_id,
					'meta_key' => str_replace('ctf_', '', $key),
					'meta_value' => $value
				);
			}
			$this->db->on_duplicate('metas', $data);
		}
	}
	
	/**
	 * Update Locations - Country, State, City and Locality
	 * 
	 * @param string $module
	 * @param integer $row_id
	 * @param string $locality
	 * @param string $city
	 * @param string $state
	 * @param string $country_iso
	 */
	public function update_locations($module, $row_id, $locality, $city, $state, $country_iso) {
		$this->load->model('locations/cities_model');
		$result = $this->cities_model->getCityStateCountry($city, NULL, $state, $country_iso, NULL);
		if($result) {
			$locality_id = 0;
			if(!empty($locality)) {
				$this->load->model('locations/localities_model');
				$locality_id = $this->localities_model->get_locality($result['city_id'], $locality);
			}
			$this->db->where('id', $row_id)->update($module, array('locality_id' => $locality_id, 'city_id' => $result['city_id'], 'state_id' => $result['state_id'], 'country_iso' => $result['country_iso']));
		}
	}

	/**
	 * Avoid running of multiple MySQL Queries during import using a session variable
	 *
	 * Unset this variable after completion of the import process.
	 *
	 * @param $key - categories/locations/keywords etc.
	 * @param $value - e.g. keyword name/ category name etc.
	 *
	 * @return
	 */
	public function get_import_session($key, $value) {
		$bulk_import_session = $this->session->userdata('bulk_import');
		if(!empty($bulk_import_session[$key]) && array_key_exists($value, $bulk_import_session[$key])) {
			return $bulk_import_session[$key][$value];
		}
		return false;
	}
	
	/**
	 * Avoid running of multiple MySQL Queries during import using a session variable
	 *
	 * Unset this variable after completion of the import process.
	 *
	 * @param $key - categories/locations/keywords etc.
	 * @param $value - e.g. keyword name/ category name etc.
	 * @param $data - data to be stored in session variable
	 *
	 * @return
	 */
	public function save_import_session($key, $value, $data) {
		$session_data = array();
		$session_data[$key] = array(
				$value => $data
		);
		$this->session->set_userdata('bulk_import', $session_data);
	}
	
	/**
	 * Get Row
	 * @param string $module
	 * @param array $where
	 * @param array $params
	 * Usage: $this->migrate_model->get_row('categories', array('name' => 'Plumbing, Heating, and Air-Conditioning'), array('fields_extra' => array('parent_id' => 0)));
	 */
	public function get_row($module, $where, $params = array()) {
		$cacheID = $module . json_encode($where) . json_encode($params);
		$cacheID = str_replace('-', '_', $this->slug->create_slug($cacheID));
		if(!$record = $this->migrate_fields_cache->load($cacheID)) {
			$record = $this->db->where($where)->get($module)->row();
			if(!$record) {
				$data = array();
				if(!empty($where['name'])) {
					$data['name'] = $where['name'];
					$data['slug'] = $this->slug->create_slug($data['name']);
				}
				switch($module) {
					case 'categories':
					case 'tags':
						$applicable_modules = gc_modules();
						$module_relations = array();
						foreach($applicable_modules as $applicable_module) {
							$module_relations[] = gc_module_id($applicable_module);
						}
						$data['module_relations'] = implode(',', $module_relations);
						$data['created_on'] = date('Y-m-d H:i:s');
						break;
					case 'cities':
						$data['created_on'] = date('Y-m-d H:i:s');
						$data['created_ip'] = $this->input->ip_address();
						break;
				}
				if(!empty($data)) {
					if(!empty($params['fields_extra'])) {
						$data = array_merge($data, $params['fields_extra']);
					}
					$this->db->insert($module, $data);
					$insert_id = (int) $this->db->insert_id();
					if($insert_id > 0) {
						$record = $this->db->where('id', $insert_id)->get($module)->row();
					}
				}
			}
			$this->migrate_fields_cache->save($record, $cacheID, array($module));
		}
		return $record;
	}
	
	/**
	 * Tweak Categories/Tags Relation
	 * @param string $module
	 * @param number $row_id
	 */
	public function tweak_tags_categories($module, $row_id) {	
		$record = $this->db->where('id', $row_id)->get($module)->row();
		if($record) {
			$module_id = gc_module_id($module);
			$tables = array('categories', 'tags');
			foreach($tables as $table) {
				$explode_data = $record->{$table . '_set'} ?? '';
			    if($table == 'categories') {
			        if(!empty($record->{$table . '_set'})) {
			            $explode_data = $record->{$table . '_set'};
			        } else if(!empty($record->{$table})) {
			            $explode_data = $record->{$table};
			        }
			    }
				$rows = explode(',', $explode_data);
				if(!empty($rows[0])) {
					if($table == 'categories') {
						$table_relation = 'categories_relation';
						$field_id = 'category_id';
					} else {
						$table_relation =  'tag_relations';
						$field_id = 'tag_id';
					}					
					// delete existing categories in case user removed one
					$this->db->where(array('row_id' => $row_id, 'module_id' => $module_id))->delete($table_relation);
					// save categories
					$data = array();
					foreach($rows as $row) {
						$data[] = array(
							'module_id' => $module_id,
							'row_id' => $row_id,
							$field_id => $row
						);
					}
					$this->db->insert_batch($table_relation, $data);
				}
			}
		}
	}
}
