<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Mobiles controller
 */
class Components extends Admin_Controller
{
	protected $permissionView = 'Mobiles.Components.View';
	protected $permissionEdit   = 'Mobiles.Components.Edit';
	protected $permissionDelete   = 'Mobiles.Components.Delete';
	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct()
	{
		parent::__construct();

		$this->auth->restrict($this->permissionView);
		$this->load->model('mobiles_model', null, true);
		$this->lang->load('mobiles');

		// DATE TIME PICKER
		Assets::add_js('moment.js');
		Assets::add_css ( 'bootstrap-datetimepicker.min.css' );
		Assets::add_js ( 'bootstrap-datetimepicker.min.js' );
		
		Assets::add_module_js('mobiles', 'mobiles.js');
		
		Template::set_block('sub_nav', 'components/_sub_nav');

	}

	/**
	 * Displays a list of all mobile devices.
	 *
	 * @return void
	 */
	public function index_old($filter='all', $offset = 0) {
		
		// Push Status Change
		if (isset($_POST['push_status_change'])) {
		    gc_demo_mode();
		    $this->gcrud->stats(array('module' => 'mobiles'));
			$this->gc->status_change('mobile_devices', $_POST['push_status_change'], 'push');
		}
		
		// Status Change
		if (isset($_POST['status_change'])) {
		    $this->gcrud->stats(array('module' => 'mobiles'));
			$this->gc->status_change('mobile_devices', $_POST['status_change'], 'status');
		}
		
		// Do we have any actions?
		if (isset($_POST['submit'])) {
		    $this->gcrud->stats(array('module' => 'mobiles'));
			$action = $this->input->post('submit');
			$checked = $this->input->post('checked');
			if($checked) {
				$module_name = lang('label_devices');
				switch(strtolower($action)) {
					case 'activate':
						$this->gc->update_status('mobile_devices', $checked, 'status', 1);
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_updated')), 'success');
						break;
	
					case 'deactivate':
						$this->gc->update_status('mobile_devices', $checked, 'status', 0);
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_updated')), 'success');
						break;
	
					case 'epush':
						$this->gc->update_status('mobile_devices', $checked, 'push', 1);
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_updated')), 'success');
						break;
	
					case 'dpush':
						$this->gc->update_status('mobile_devices', $checked, 'push', 0);
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_updated')), 'success');
						break;
	
					case 'delete':
						$this->gc->purge_records('mobile_devices', $checked);
						foreach($checked as $pid) { // Delete Notifications also
							$this->db->delete('mobile_notifications', array('device_id' => $pid)); 
						}						
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_deleted')), 'success');
						break;
				}
			} else {
				Template::set_message(lang('msg_no_row_selected'), 'error');
			}
		}
		
		// Actions done, now display the view
		$where = "";
		// Filters
		$filter_type = $filter;
		 
		switch($filter_type) {
			case 'android':
				$where .= "platform = 'Android'";
				break;
	
			case 'ios':
				$where .= "platform = 'iOS'";
				break;
				
			case 'active':
				$where .= "status = 1";
				break;
	
			case 'inactive':
				$where .= "status = 0";
				break;
			
			case 'epush':
				$where .= "push = 1";
				break;
	
			case 'dpush':
				$where .= "push = 0";
				break;
				 
			case 'all':
				// Nothing to do
				break;
				 
			default:
				show_404("mobiles/index/$filter/");
		}
		
		if (isset($_POST['push'])) {
			Template::redirect(site_url(SITE_AREA . '/components/mobiles/push/' . $_POST['push']));
		}
		
		$searchterm = $this->gc->search('mobile_devices', $this->uri->segment(6), '/components/mobiles');
		if(!empty($searchterm)) {
			if(!empty($where)) {
				$where .= " AND";
			}
			$where .= " (id LIKE '%" . $searchterm . "%' OR platform LIKE '%" . $searchterm . "%' OR uuid LIKE '%" . $searchterm . "%' OR model LIKE '%" . $searchterm . "%' OR token LIKE '%" . $searchterm . "%')";			
		}
		
		$sort_by = "id DESC";
		$order_by = $this->gc->sort('mobile_devices');
		if(!empty($order_by)) {
			$sort_by = $order_by;
		}		
		$records = $this->mobiles_model->get_devices(0, $where, $sort_by, $this->limit, $offset);
		$total_rows = $this->mobiles_model->get_devices(1, $where, $sort_by);
		
		// Pagination
		$this->load->library('pagination');
		
		$pager['base_url'] = site_url(SITE_AREA . '/components/mobiles/index/' . $filter) . '/';
		$pager['total_rows'] = $total_rows;
		$pager['per_page'] = $this->limit;
		$pager['uri_segment']	= 5;
		
		$this->pagination->initialize($pager);
		
		Template::set('index_url', site_url(SITE_AREA .'/components/mobiles/index/'));
		Template::set('filter_type', $filter_type);
		Template::set('total_rows', $total_rows);
		Template::set('records', $records);
		Template::set('toolbar_title', lang('view_mobile_devices'));
		Template::render();
	}
	
	/**
     * Display Player IDs
	 */
	public function index($filter = 'all', $offset = 0) {
	    // send test notification
	    if(isset($_POST['test_notification'])) {
	        $id = $_POST['test_notification'];
	        $record = $this->db->where('id', $id)->get('mobile_player_ids')->row();
	        if($record) {
	            $api_string = 'tabs/member/info-pages/purchases/pending/0'; // http://localhost:8100/tabs/member/info-pages/purchases/pending/0
	            $this->mobiles_model->test_notification($record->player_id, $api_string);
	        }
	    }
	    $this->manage('player_ids', $filter, $offset);
	}
	
	/**
	 * Allows editing of Comments data.
	 *
	 * @return void
	 */
	public function view() {
		$id = $this->uri->segment(5);
		$this->load->model('mobiles/devices_model');

		if (empty($id)) {
			Template::set_message(lang('invalid_id'), 'error');
			redirect(SITE_AREA .'/components/mobiles');
		}

		if (isset($_POST['save'])) {
		    $this->gcrud->stats(array('module' => 'mobiles'));
			$this->auth->restrict('Mobiles.Components.Edit');

			if ($this->save_device('update', $id)) {
				// Log the activity
				log_activity($this->current_user->id, lang('act_edit_record') .': '. $id .' : '. $this->input->ip_address(), 'mobiles');

				Template::set_message(sprintf(lang('msg_save_success'), lang('device')), 'success');
				redirect(site_url(SITE_AREA .'/components/mobiles/'));
			} else {
				Template::set_message(sprintf(lang('msg_edit_failure'), lang('device')) . $this->devices_model->error, 'danger');
			}
		} else if (isset($_POST['delete'])) {
		    $this->gcrud->stats(array('module' => 'mobiles'));
			$this->auth->restrict($this->permissionDelete);

			if ($this->devices_model->delete($id)) {
				$this->db->delete('mobile_notifications', array('device_id' => $id)); // // Delete Notifications also
				// Log the activity
				log_activity($this->current_user->id, lang('act_delete_record') .': '. $id .' : '. $this->input->ip_address(), 'mobiles');

				Template::set_message(sprintf(lang('msg_delete_success'), lang('label_devices')), 'success');

				redirect(SITE_AREA .'/components/mobiles');
			} else {
				Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_devices')) . $this->devices_model->error, 'danger');
			}
		}
		
		Template::set('device', $this->db->get_where('mobile_devices', array('id' => $id))->row());
		Template::set('toolbar_title', lang('view_device'));
		Template::render();
	}
	
	/**
	 * Push Notification
	 */
	public function push() {
		$id = $this->uri->segment(5);
		
		if (empty($id)) {
			Template::set_message(lang('invalid_id'), 'error');
			redirect(SITE_AREA .'/components/mobiles');
		}
		
		if (isset($_POST['send'])) {
		    gc_demo_mode();
		    $this->gcrud->stats(array('module' => 'mobiles'));
			$this->form_validation->set_rules('title', 'lang:label_title', 'required|max_length[120]');			
			$this->form_validation->set_rules('message', 'lang:label_message', 'trim|max_length[255]|required');
			$this->form_validation->set_rules('type', 'lang:message_type', 'trim');
			$this->form_validation->set_rules('date_sent', 'lang:date_sent', 'trim');
			if($_POST['date_sent'] == 2) {
				$this->form_validation->set_rules('scheduled_on', 'lang:scheduled_on', 'trim|required|valid_date[TRUE]');
			}
			if ($this->form_validation->run() !== FALSE) {
				$current_time = date('Y-m-d H:i:s');
				$data = array(
					'device_id' => $id,
					'title' => $this->input->post('title'),
					'message' => $this->input->post('message'),
					'type' => $this->input->post('type'),
					'broadcast' => 0,
				    'scheduled_on' => $this->input->post('scheduled_on') ? $this->input->post('scheduled_on') : NULL,
					'created_on' => $current_time,
					'ip_address' => $this->input->ip_address()
				);
				
				$this->load->model('mobiles/devices_model');
				$device = $this->devices_model->select('token')->find_by(array('id' => $id, 'push' => 1, 'status' => 1)); // Get Token from the database
				if($device) {
					if($this->input->post('date_sent') == 1) {
						$data['date_sent'] = $current_time;
						$data['status'] = 1;					
						
						$tokens = array($device->token);
						$response = $this->mobiles_model->push_notification($tokens); // Push Notification
						$data['response'] = serialize($response);
						$success_message = lang('push_success');
						$alert_type = 'success';					
					} else {
						$success_message = sprintf(lang('push_schedule_success'), gc_date_format($this->input->post('scheduled_on'), true));
						$alert_type = 'warning';
					}
					Template::set_message($success_message, $alert_type);
					$this->db->insert('mobile_notifications', $data);
					Template::redirect(site_url(SITE_AREA . "/components/mobiles/push/" . $id));
				} else {
					Template::set_message(lang('push_device_error'), 'error');
				}
			}
		}
		Template::set('toolbar_title', lang('push_notification'));
		Template::set('broadcast', false); // Hide Platform Field
		Template::render();
	}
	
	/**
	 * Broadcast Notification
	 */
	public function broadcast() {		
		if (isset($_POST['send'])) {
		    gc_demo_mode();
		    $this->gcrud->stats(array('module' => 'mobiles'));
			$this->form_validation->set_rules('title', 'lang:label_title', 'required|max_length[120]');			
			$this->form_validation->set_rules('message', 'lang:label_message', 'trim|max_length[255]|required');
			$this->form_validation->set_rules('type', 'lang:message_type', 'trim');
			$this->form_validation->set_rules('date_sent', 'lang:date_sent', 'trim');
			if($_POST['date_sent'] == 2) {
				$this->form_validation->set_rules('scheduled_on', 'lang:scheduled_on', 'trim|required|valid_date[TRUE]');
			}
			if ($this->form_validation->run() !== FALSE) {
				$current_time = date('Y-m-d H:i:s');
				$data = array(
					'title' => $this->input->post('title'),
					'message' => $this->input->post('message'),
					'type' => $this->input->post('type'),
					'broadcast' => 1,
				    'scheduled_on' => $this->input->post('scheduled_on') ? $this->input->post('scheduled_on') : NULL,
					'created_on' => $current_time,
					'ip_address' => $this->input->ip_address()
				);
				
				/**
				 * GET DEVICES INFORMATION
				 * id - For mobile_notifications Table
				 * token - For Push Notification If User Selected Instantly
				 */
				$this->load->model('mobiles/devices_model');
				$search = array(
					'status' => 1, 
					'push' => 1						
				);
				if($this->input->post('platform') != 'All') {
					$search['platform'] = $this->input->post('platform');
				}
				$devices = $this->devices_model->select('id, token')->find_all_by($search); // Get Token and ID from the database
				
				if($devices) {
					// Push Notification Immediately
					if($this->input->post('date_sent') == 1) {
						$data['date_sent'] = $current_time;
						$data['status'] = 1;					
						
						// Create Token Array
						$android_tokens = array();
						$ios_tokens = array();
						foreach($devices as $mobile) {
							if(!empty($mobile->platform) && $mobile->platform == 'Android') {
								$android_tokens[] = $mobile->token;
							} else {
								$ios_tokens[] = $mobile->token;
							}							
						}
						if(!empty($android_tokens)) {
							$response = $this->mobiles_model->push_notification($android_tokens); // Push Notification
						}
						if(!empty($ios_tokens)) {
							$response = $this->mobiles_model->push_notification($ios_tokens); // Push Notification
						}
						
						$data['response'] = serialize($response);
						
						$success_message = lang('push_success');
						$alert_type = 'success';	
					} else {
						$success_message = sprintf(lang('push_schedule_success'), gc_date_format($this->input->post('scheduled_on'), true));
						$alert_type = 'warning';
					}			
					
					// Create Notification Logs
					$notification_data = array();
					foreach($devices as $device) {
						$data['device_id'] = $device->id;
						$notification_data[] = $data;
					}					
					$this->db->insert_batch('mobile_notifications', $notification_data);
					
					// Display Message to the User
					Template::set_message($success_message, $alert_type);
					Template::redirect(site_url(SITE_AREA . "/components/mobiles/broadcast"));
				} else {
					Template::set_message(lang('broadcast_device_error'), 'error');
				}
			}
		}
		Template::set('toolbar_title', lang('broadcast_notification'));
		Template::set('broadcast', true); // Display Platform Field
		Template::set_view('push');
		Template::render();
	}
	
	/**
	 * Display Notification Logs to the Admin.
	 *
	 * @return void
	 */
	public function logs($filter='all', $offset = 0) {
		
		// Do we have any actions?
		if (isset($_POST['submit'])) {
		    $this->gcrud->stats(array('module' => 'mobiles'));
			$action = $this->input->post('submit');
			$checked = $this->input->post('checked');
			if($checked) {
				$module_name = lang('label_devices');
				switch(strtolower($action)) {
					case 'activate':
						$this->gc->update_status('mobile_notifications', $checked, 'status', 1);
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_updated')), 'success');
						break;
	
					case 'deactivate':
						$this->gc->update_status('mobile_notifications', $checked, 'status', 0);
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_updated')), 'success');
						break;
	
					case 'disabled':
						$this->gc->update_status('mobile_notifications', $checked, 'status', 2);
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_updated')), 'success');
						break;
	
					case 'delete':
						$this->gc->purge_records('mobile_notifications', $checked);
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_deleted')), 'success');
						break;
				}
			} else {
				Template::set_message(lang('msg_no_row_selected'), 'error');
			}
		}
		
		// Actions done, now display the view
		$where = "";
		$count_where = "";
		// Filters
		$filter_type = $filter;
		 
		switch($filter_type) {
			case 'android':
				$where .= " MD.platform = 'Android'";
				break;
	
			case 'ios':
				$where .= " MD.platform = 'iOS'";
				break;
				
			case 'active':
				$where .= " MN.status = 1";
				break;
	
			case 'inactive':
				$where .= " MN.status = 0";
				break;
			
			case 'disabled':
				$where .= " MN.status = 2";
				break;
				 
			case 'all':
				// Nothing to do
				break;
				 
			default:
				show_404("mobiles/logs/$filter/");
		}
		
		// Search
		$searchterm = $this->gc->search('notification_logs', $this->uri->segment(6), '/components/mobiles/logs');
		if(!empty($searchterm)) {
			if(!empty($where)) {
				$where .= " AND";
			}
			$where .= " (MN.id LIKE '%" . $searchterm . "%' OR MD.platform LIKE '%" . $searchterm . "%' OR MD.uuid LIKE '%" . $searchterm . "%' OR MD.token LIKE '%" . $searchterm . "%' OR MN.title LIKE '%" . $searchterm . "%' OR MN.message LIKE '%" . $searchterm . "%' OR MN.scheduled_on LIKE '%" . $searchterm . "%' OR MN.date_sent LIKE '%" . $searchterm . "%')";
		}
		
		// Order By
		$sort_by = "id DESC";
		$order_by = $this->gc->sort('notification_logs');
		if(!empty($order_by)) {
			$sort_by = $order_by;
		}
		
		$records = $this->mobiles_model->get_notifications(0, $where, $sort_by, $this->limit, $offset);
		$total_rows = $this->mobiles_model->get_notifications(1, $where, $sort_by);
		
		// Pagination
		$this->load->library('pagination');
		
		$pager['base_url'] = site_url(SITE_AREA . '/components/mobiles/logs/' . $filter) . '/';
		$pager['total_rows'] = $total_rows;
		$pager['per_page'] = $this->limit;
		$pager['uri_segment']	= 6;
		
		$this->pagination->initialize($pager);
		
		Template::set('index_url', site_url(SITE_AREA .'/components/mobiles/logs/') .'/');
		Template::set('filter_type', $filter_type);
		Template::set('total_rows', $total_rows);
		Template::set('records', $records);
		Template::set('toolbar_title', lang('view_mobile_devices'));
		Template::render();
	}
	
	/**
	 * Allows users to view notification.
	 *
	 * @return void
	 */
	public function notification() {
		$id = $this->uri->segment(5);
		$this->load->model('mobiles/notifications_model');
		if (empty($id)) {
			Template::set_message(lang('invalid_id'), 'error');
			redirect(SITE_AREA .'/components/mobiles/logs');
		}

		if (isset($_POST['delete'])) {
		    $this->gcrud->stats(array('module' => 'mobiles'));
			$this->auth->restrict($this->permissionDelete);

			if ($this->notifications_model->delete($id)) {
				// Log the activity
				log_activity($this->current_user->id, lang('act_delete_record') .': '. $id .' : '. $this->input->ip_address(), 'mobiles');

				Template::set_message(sprintf(lang('msg_delete_success'), lang('label_notification')), 'success');

				redirect(SITE_AREA .'/components/mobiles/logs');
			} else {
				Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_notification')) . $this->notifications_model->error, 'danger');
			}
		}
		
		Template::set('notification', $this->mobiles_model->get_push_detail($id));
		Template::set('toolbar_title', lang('view_notification'));
		Template::render();
	}
	
	/**
	 * Summary
	 *
	 * @param String $type Either "insert" or "update"
	 * @param Int	 $id	The ID of the record to update, ignored on inserts
	 *
	 * @return Mixed    An INT id for successful inserts, TRUE for successful updates, else FALSE
	 */
	private function save_device($type='update', $id=0)
	{
		$_POST['id'] = $id;
		$data = array();
		$data['platform']        	= $this->input->post('platform') ? $this->input->post('platform') : '';
		$data['model']        		= $this->input->post('model') ? $this->input->post('model') : '';
		$data['version']        	= $this->input->post('version') ? $this->input->post('version') : '';
		$data['manufacturer']      	= $this->input->post('manufacturer') ? $this->input->post('manufacturer') : '';
		$data['token']        		= $this->input->post('token') ? $this->input->post('token') : '';
		$data['push']        		= $this->input->post('push') ? 1 : 0;
		$data['status']        		= $this->input->post('status') ? 1 : 0;

		$return = $this->devices_model->update($id, $data);
		return $return;
	}
	
	/**
	 * API Key and other settings
	 */
	public function settings() {
	    $setting_name = 'mobile.settings';
		clear_cache('site_settings'); // Clear Site Settings Cache
		// save the settings to the DB
		$this->load->model('settings/settings_model');
		if (isset($_POST['save'])) {
		    $this->gcrud->stats(array('module' => 'mobiles'));
		    $this->form_validation->set_rules('mobile', 'lang:gc_menu_settings', 'trim');
		    $this->form_validation->set_rules('hide_payment', 'lang:mobile_hide_payment', 'trim');
		    $this->form_validation->set_rules('ad_view_cost', 'lang:ad_view_cost', 'trim|required|is_decimal');
		    $this->form_validation->set_rules('ad_view_process_limit', 'lang:ad_view_process_limit', 'trim|required|is_numeric');
			if($this->form_validation->run() == TRUE) {
			    $settings = $this->input->post('mobile');
			    $settings['hide_payment'] = (int) $this->input->post('hide_payment');
			    $settings['ad_view_cost'] = $this->input->post('ad_view_cost');
			    $settings['ad_view_process_limit'] = $this->input->post('ad_view_process_limit');
				$data = array(
				    array('name' => $setting_name, 'value' => json_encode($settings))
				);
				$updated = $this->settings_model->update_batch($data, 'name');				
				if ($updated) {
					Template::set_message(lang('settings_saved_success'), 'success');
					Template::redirect(SITE_AREA .'/components/mobiles/settings');
				} else {
					Template::set_message(lang('settings_error_success'), 'error');
				}
			}
		}
    	$result = $this->db->select('value')->get_where('settings', array('name' => $setting_name))->row();
    	if($result) {
    	    $record = json_decode($result->value ?? '', true);
    	} else {
    	    $record = json_decode(settings_item($setting_name), true);
    	}		
		Template::set('record', $record);		
		Template::set('toolbar_title', lang('mobile_settings'));
		Template::render();
	}
	
	/**
	 * Create thumbs of uploaded image
	 */
	public function resize($path,$file){
		$config['image_library']='gd2';
		$config['source_image']=$path;
		//$config['create_thumb']=TRUE; // This will create new file
		$config['maintain_ratio']=TRUE;
		$config['width']=200;
		$config['height']=200;
		$config['new_image']='./assets/images/banners/'.$file; //CHANGE THIS LINE FOR PATH
	
		$this->load->library('image_lib',$config);
		$this->image_lib->resize();
	}
	
	public function push_notificationss() {
		/*$authorization = "Authorization: Bearer eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJqdGkiOiJlMTczYWY4Yy0xY2M5LTRjZDAtYjY4ZC1kODRhMTM2NThiNmIifQ.UU_nBxZSsiVpRR1aWytJieNOpCJCWRCyI4HpnwkfyTw";
		curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: application/json' , $authorization ));
		curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_POSTFIELDS,$post);
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
		$result = curl_exec($ch);
		curl_close($ch);
		return json_decode($result);*/
		$msg = $_GET['msg'];
		$user = $_GET['user'];

		$device_token="TOKEN";

		$url = 'https://push.ionic.io/api/v1/push';

		$data = array(
		  'tokens' => array($device_token), 
		  'notification' => array('alert' => ''.$user.': '.$msg),      
		);
			  
		$content = json_encode($data);

		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_POST, TRUE);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $content);
		curl_setopt($ch, CURLOPT_USERPWD, "Private Api key" . ":" );  
		curl_setopt($ch, CURLOPT_HTTPHEADER, array(
			'Content-Type: application/json',
			'X-Ionic-Application-Id: App-ID' 
		));
		$result = curl_exec($ch);
		curl_close($ch);
	}
	
	public function push_cron() {
		$this->load->model('mobiles/mobiles_model');
		$this->mobiles_model->push_cron();
	}
	
	/**
	 * Display a list of menus data.
	 *
	 * @return void
	 */
	public function manage_old($module = 'mobile_links', $filter = 'all', $offset = 0) {
	    if($module == 'player_ids' || $module == 'templates') {
	        $extra = array(
	            'model' => 'mobiles',
	            'context' => 'components',
	            'prefix' => 'mobile',
	            'fields' => $module
	        );
	        $this->gc->module_view($module, $filter, $offset, 'admin', $extra);
	    } else {
    	    // Assets::add_css('https://unpkg.com/ionicons@4.5.10-0/dist/css/ionicons.min.css', 'external');
    	    Assets::add_js('https://unpkg.com/ionicons@4.5.10-0/dist/ionicons.js', 'external');
    	    $this->lang->load('links/links');
    	    if($module == 'mobile_components' || $module == 'mobile_links') {
    	        clear_cache($module);
    	    }
    	    if($module == 'mobile_links' || $module == 'mobile_sliders') {
    	        $extra = array(
    	            'model' => 'mobiles',
    	            'module_name' => 'mobiles',
    	            'context' => 'components',
    	            'fields' => $module,
    	            'prefix' => ''
    	        );
    	        $this->gc->module_view($module, $filter, $offset, 'admin', $extra);
    	    } else {
        	    $params = array(
        	        'context' => 'components',
        	        'model' => 'mobiles',
        	        'fields_method' => $module,
        	        'query_method' => 'get_all_crud',
        	        'table_alias' => 'M.',
        	        'prefix' => 'M',
        	        'search_fields' => array('label', 'link_url'),
        	        'uri_segment' => 6,
        	        'query_extra' => array('module' => $module),
        	        'redirect_url' => SITE_AREA . '/components/mobiles/manage/' . $module,
        	        'edit_url' => SITE_AREA . '/components/mobiles/form/' . $module . '/edit/',
        	        'can_create' => true,
        	        'create_url' => site_url(SITE_AREA . '/components/mobiles/form/' . $module)
        	    );
        	    if($module == 'mobile_popups') {
        	        $params['search_fields'] = array('name');
        	    } else if($module == 'mobile_components') {
        	        $params['search_fields'] = array('label', 'extra_params');
        	    }
        	    $this->gcrud->index($module, $offset, 'admin', $params);
    	    }
	    }
	}
	
	/**
	 * Add/Edit Object.
	 *
	 * @return void
	 */
	public function form_old($module = 'mobile_links', $action = 'create') {
	    $this->lang->load('links/links');
	    $extra = array(
	        'context' => 'components',
	        'model' => 'mobiles',
	        'fields_method' => $module,
	        'redirect_url' => SITE_AREA . '/components/mobiles/manage/' . $module,
	        //'cancel_btn' => true
	    );
	    if($action == 'edit') {
	        $extra['id'] = $this->uri->segment(7);
	        // $extra['edit_url'] = SITE_AREA . '/developer/themes/form/' . $module . '/edit/%s';
	    }
	    if($module == 'mobile_components' || $module == 'mobile_links') {
	        clear_cache($module);
	    }
	    if($module == 'mobile_sliders') {
	        $extra['method'] = 'after_ad_save';
	    }
	    $this->gcrud->form($module, $action, 'admin', $extra);
	}
	
	/**
	 * Display a list of menus data.
	 *
	 * @return void
	 */
	public function manage($module = 'links', $filter = 'all', $offset = 0) {
	    // Assets::add_css('https://unpkg.com/ionicons@4.5.10-0/dist/css/ionicons.min.css', 'external');
	    Assets::add_js('https://unpkg.com/ionicons@5.1.2/dist/ionicons.js', 'external');
	    $this->lang->load('links/links');
	    $extra = array(
	        'model' => 'mobiles',
	        'module_name' => 'mobiles',
	        'context' => 'components',
	        'fields' => $module,
	        'prefix' => 'mobile'
	    );
	    $this->gc->module_view($module, $filter, $offset, 'admin', $extra);
	}
	
	/**
	 * Add/Edit Object.
	 *
	 * @return void
	 */
	public function form($module = 'mobile_links', $action = 'create') {
	    $this->lang->load('links/links');
	    $extra = array(
	        'context' => 'components',
	        'model' => 'mobiles',
	        'fields_method' => $module,
	        'redirect_url' => SITE_AREA . '/components/mobiles/manage/' . $module,
	        'cancel_btn' => true,
	        'prefix' => 'mobile'
	    );
	    if($action == 'edit') {
	        $extra['id'] = $this->uri->segment(7);
	        // $extra['edit_url'] = SITE_AREA . '/developer/themes/form/' . $module . '/edit/%s';
	    }
	    $this->gcrud->form($module, $action, 'admin', $extra);
	}
	
	/**
	 * Selectize search - Users/Listings
	 * @param string $module
	 */
	public function selectizeSearch($module_id, $q) {
	    header('Content-Type: application/x-json; charset=utf-8');
	    $q = escape(urldecode($q));
	    $module = gc_module_name($module_id);
	    $field = 'title';
	    if(!empty($q)) {
	        $this->db->or_like($field, $q);
	    }
	    $result = $this->db->select('id, ' . $field . ' AS name')->get($module)->result();
	    echo json_encode($result);
	}
}