<?php defined('BASEPATH') OR exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
// This can be removed if you use __autoload() in config.php OR use Modular Extensions
require APPPATH . '/libraries/REST_Controller.php';
/**
 * Mobiles Controller
 */
class Mobiles extends REST_Controller {

	private $siteSettings;
	private $limit;
	protected $gc_mobiles_cache;
	protected $language = 'english';
    function __construct() {
        // HTTP OPTIONS error when basic authorization is enabled
        /*header('Access-Control-Allow-Origin: *');
        header("Access-Control-Allow-Headers: X-API-KEY, Origin, X-Requested-With, Content-Type, Accept, Access-Control-Request-Method, Access-Control-Request-Headers, Authorization, Token");
        header("Access-Control-Allow-Methods: GET, POST, OPTIONS, PUT, DELETE");
        $method = $_SERVER['REQUEST_METHOD'];
        if($method == "OPTIONS") {
            die();
        }*/
        
        // Construct the parent class
        parent::__construct();
		$this->load->model('mobiles_model');
		// Disable hooks, since they may rely on an installed environment.
        get_instance()->hooks->enabled = false;
        // Configure limits on our controller methods
        // Ensure you have created the 'limits' table and enabled 'limits' within application/config/rest.php
        $this->methods['user_get']['limit'] = 500; // 500 requests per hour per user/key
        $this->methods['user_post']['limit'] = 100; // 100 requests per hour per user/key
        $this->methods['user_delete']['limit'] = 50; // 50 requests per hour per user/key
		$this->load->library('settings/Settings_lib');
		$this->load->library('translations');
		// if(settings_item('site.enable_cache') == 1) {
			$this->load->library('gzf_cache', array('lifetime' => 900));
			$this->load->library('slug');
			$this->gc_mobiles_cache = $this->gzf_cache->get_instance();
		// }
		
		/**
		 * Get User selected Language
		 * The user selected language is loaded by using a session variable
		 * This variable is updated onSplashLoading, Device Token Registration and Settings language change
		 */
		$this->load->library('session');
		// No language is selected by the user, get the default language from settings table
		if(!$this->session->userdata('device_language')) {
			$this->session->set_userdata('device_language', settings_item('site.default_language'));
		}
		$this->language = $this->session->userdata('device_language');
		
		$this->load->helper('gc');
		$this->load->helper('application');
		$this->load->library('gc');
		$this->load->library('form_validation');
		$this->load->helper('security'); // XSS_Clean Validation Error
		$this->load->library('gcrud');
		$this->lang->load('mobiles', $this->language);
		$this->lang->load('routes', 'english');	
		$this->lang->load('application', $this->language);
		$this->lang->load('goclixy', $this->language);
		$this->load->helper('form_helper');
		$this->load->library('users/auth');
		$this->load->library('layouts');
		$this->load->library('goclixy/gemailer');
		$this->limit = settings_item('site.front_list_limit');
		$this->siteSettings = $this->settings_lib->find_all();
    }
    
    /**
     * Common Function for all View Pages
     * @param string $mobile
     * @param number $limit
     * @param number $page
     */
    public function load_options() {
        $this->header_options();
    }
    public function load_get() {
        $this->header_params();
        $module = $this->get('module') ? escape($this->get('module')) : 'cities';
        $user_id = $this->user_id();
        $limit = (int) $this->get('limit');
        $limit = $limit == 0 ? $this->limit : $limit;
        $page = (int) $this->get('page');
        if($page <= 0) {
            $this->response(NULL, REST_Controller::HTTP_BAD_REQUEST);
        }
        if($page == 1) { // First page set offset to 0
            $offset = 0;
        } else {
            $offset = $this->limit * ($page - 1); // -1 to retrieve correct records
        }
        $q = escape($this->get('q'));
        $records = array();
        $total_rows = 0;
        $extra_data = array('toolbar_title' => lang('search_results'));
        switch($module) {
            case 'suggestions': // search suggestions
                $md = escape($this->get('md')); // pass module here
                // $md = gc_module_name($md);
                if(!in_array($md, gc_modules())) {
                    $this->response(NULL, REST_Controller::HTTP_BAD_REQUEST); // BAD_REQUEST (400) being the HTTP response code
                }
                $fields = $this->get();
                unset($fields['page']);
                unset($fields['limit']);
                unset($fields['module']);
                unset($fields['md']);
                unset($fields['q']);
                if(!empty($fields['lat'])) {
                    unset($fields['lat']);
                }
                if(!empty($fields['lng'])) {
                    unset($fields['lng']);
                }
                $records = $this->mobiles_model->search_suggestions($md, $q, $fields);
                $total_rows = $limit;
                $extra_data['module'] = lang('singular_' . $md);
                break;
            case 'search':
                $this->load->model('locations/cities_model');
                $id = (int) $this->get('id'); // Category/Location Browse
                $edit = (int) $this->get('edit'); // The request is from the edit page
                $filter = escape($this->get('filter')); // all, featured, popular, recently_added and recently viewed
                $offset = 0;
                $md = escape($this->get('md')); // pass module here
                $type = escape($this->get('type'));
                if(gc_module_id($md) == 0) {
                    $this->response(NULL, REST_Controller::HTTP_BAD_REQUEST); // BAD_REQUEST (400) being the HTTP response code
                }
                if($edit == 1) { // the request is from the member area
                    if(empty($user_id)) {
                        $this->response(NULL, REST_Controller::HTTP_BAD_REQUEST); // BAD_REQUEST (400) being the HTTP response code
                    }
                    // display listing specific products, blogs, etc.
                    if($md != 'listings' && !empty($id)) {
                        $where = 'M.listing_id = ' . $id;
                    } else {
                        $where = null;
                    }
                    $sort_by = 'M.id DESC';
                    switch($filter) {
                        case 'featured':
                            $where .= (!empty($where) ? ' AND ' : '') . 'M.featured = 1';
                            break;
                        case 'popular':
                            $sort_by = 'M.hits DESC';
                            break;
                        case 'last_viewed':
                            $sort_by = 'M.last_viewed_on DESC';
                            break;
                    }
                    $result = array(
                        'success' => true,
                        'toolbar_title' => lang($md),
                        'total_rows' => $this->mobiles_model->member_edit($md, $user_id, 1, $where, $limit, $offset),
                        'records' => $this->mobiles_model->member_edit($md, $user_id, 0, $where, $limit, $offset) // $this->mobile_data($module, $records)
                    );
                    $records = $result['records'];
                    $total_rows = $result['total_rows'];       
                    $extra_data['toolbar_title'] = lang($md); 
                } else {
                    $country = escape($this->get('iso'));
                    $user_id = intval($this->get('user_id'));
                    $member_area = intval($this->get('member_area'));
                    
                    // Videos and Images Module Specific
                    $toggle_module = $type;
                    $toggle_row_id = $id;
                    
                    if($type == 'search')	{
                        $_GET['search'] = true;
                        $_GET['q'] = escape($this->get('q'));
                        $_GET['l'] = $this->get('l') ? escape($this->get('l')) : "";
                        $_GET['cg'] = $this->get('c') ? $this->get('c') : 0;
                        $_GET['lt'] = $this->get('lt') ? $this->get('lt') : 0;
                        $_GET['tl'] = $this->get('tl') ? $this->get('tl') : 0;
                        $_GET['t'] = $this->get('t') ? $this->get('t') : 0;
                        $_GET['ct'] = $this->get('ct') ? $this->get('ct') : NULL;
                        
                        // display listing specific products, blogs, etc.
                        if($md != 'listings' && !empty($id)) {
                            $_GET['lstid'] = $id;
                        }
                    } else {
                        // Display only specific location listings
                        if($type == 'categories' || $type == 'tags') {
                            $_GET['l'] = $this->get('l') ? escape($this->get('l')) : "";
                        }
                        // Validate the id.
                        if ($id <= 0) {
                            // Invalid id, set the response and exit.
                            $this->response(NULL, REST_Controller::HTTP_BAD_REQUEST); // BAD_REQUEST (400) being the HTTP response code
                        }
                    }
                    /*
                    if(!empty($_GET['lat']) && !empty($_GET['lng'])) {
                        $_GET['latitude'] = $_GET['lat'];
                        $_GET['longitude'] = $_GET['lng'];
                        unset($_GET['lat']);
                        unset($_GET['lng']);
                    }
                    */
                    // Location Filter - Allow user to view selected country/state results
                    $location_filter = $this->get('lfltr');
                    if(!empty($location_filter) && !empty($_GET['l'])) {
                        $city_info = $this->cities_model->get_city_info($_GET['l']);
                        if($city_info) {
                            if($location_filter == 'country') {
                                $_GET['l'] = 0;
                                $_GET['tl'] = 0;
                            } else if($location_filter == 'state') {
                                $_GET['l'] = $city_info->state_id;
                                $_GET['tl'] = 2;
                            }
                        }
                    }
                    
                    $_GET['selectRows'] = $this->get('selectRows') ? $this->get('selectRows') : $this->limit;
                    $_GET['sort_by'] = $this->get('sort_by') ? $this->get('sort_by') : 'alphabetically';
                    $_GET['having'] = $this->get('having');
                    
                    // filter results
                    if($this->get('f')) {
                        $_GET['f'] = $this->get('f');
                    }
                    
                    if($page === NULL) {
                        $page = 1;
                    } else {
                        if($page == 1) { // First page set offset to 0
                            $offset = 0;
                        } else {
                            $offset = $this->limit * ($page - 1); // -1 to retrieve correct records
                        }
                    }
                    $this->load->model('listings/listings_model');
                    $id = ($type == 'search' ? null : $id);
                    
                    // featured, popular, recently_added and last_viewed filter
                    switch($filter) {
                        case 'featured':
                            $_GET['featured'] = 1;
                            break;
                        case 'popular':
                            $_GET['sort_by'] = 'popular';
                            break;
                        case 'recently_added':
                            $_GET['sort_by'] = 'relevance';
                            break;
                        case 'last_viewed':
                            $_GET['sort_by'] = 'last_viewed';
                            break;
                    }
                    $cacheID = str_replace('-', '_', $this->slug->create_slug(implode('_', $_GET) . '_' . $id));
                    // if(!$url_results = $this->gc_mobiles_cache->load($cacheID)) { // cache results
                        switch($type) {
                            case 'categories':
                            case 'tags':
                            case 'search':
                            case 'cities':
                                // $_GET['latitude']) && !empty($_GET['longitude'])) -  Businesses near to users location
                                // $this->session->set_userdata('search_country', $country); // load country specific results
                                // unset($_GET['limit']);
                                // $limit = 6;
                                $result = $this->listings_model->category_location_tag_search(true, $md, $type, $country, $id, $limit, $offset, $user_id);
                                break;
                            default:
                                // Listings/Users Detail Page - Display module specific rows on the module name click
                                if($member_area == 0) {
                                    $result = $this->listings_model->frontend_modules($md, $type, $id, $limit, $offset);
                                } else { // Request is from the Member Area
                                    $result = $this->gc->view('mobile', $md, $user_id, NULL, $id, $toggle_module, $toggle_row_id, $offset, $limit);
                                }
                        }
                        if(isset($result)) {
                            // Check if the result data store contains listings (in case the database result returns NULL)
                            if (!empty($result['records'])) {
                                $records = $result['records'];
                                // Append site URL to the image path
                                foreach($records as $key => $row) {
                                    $row = (object) $row;
                                    if(!empty($row->image)) {
                                        $result['records'][$key]->image = gc_image_url($row->image);
                                    } else if(!empty($row->cover)) {
                                        $result['records'][$key]->image = gc_image_url($row->cover);
                                    } else if(!empty($row->youtube_id)) { // Videos Module
                                        $result['records'][$key]->image = 'http://img.youtube.com/vi/' . $row->youtube_id . '/0.jpg';
                                    } else {
                                        $result['records'][$key]->image = gc_site_image('no-image');
                                    }
                                }
                            }
                            $records = $result['records'];
                            $total_rows = $result['total_rows'];
                            $extra_data['toolbar_title'] = $result['toolbar_title'];
                            if(!empty($_GET['city_id'])) { // get latitude and longitude for map central location
                                $city_info = $this->cities_model->city_info($_GET['city_id']);
                                if($city_info) {
                                    $extra_data['city_info'] = array('name' => $city_info->name,'geo' => !empty($city_info->latitude) ? array($city_info->latitude, $city_info->longitude) : array());
                                }
                            }
                        }
                        // filter segments
                        $segments = array();
                        $segment_filters = array('featured', 'popular', 'recently_added', 'last_viewed');
                        $segments[] = array(
                            'value' => 'all',
                            'label' => lang('tab_all')
                        );
                        foreach($segment_filters as $segment_filter) {
                            $segments[] = array(
                                'value' => $segment_filter,
                                'label' => lang($segment_filter)
                            );
                        }
                        $extra_data['segments'] = $segments;
                        // cache variable
                        $url_results = array(
                            'records' => $records,
                            'total_rows' => $total_rows,
                            'extra_data' => $extra_data
                        );
                        $this->gc_mobiles_cache->save($url_results, $cacheID, array('search'));
                    // }
                    $records = $url_results['records'];
                    $total_rows = $url_results['total_rows'];
                    $extra_data = $url_results['extra_data'];
                    // display category specific slides or banners
                    // $extra_data['slides'] = $this->mobiles_model->get_sliders(1);
                }
                break;
            case 'categories':
                $is_form_select = $this->get('form') ? true : true;
                if($is_form_select) {
                    $toggle_module = $this->get('tm');
                    $this->load->library('goclixy/gclosure');
                    $gclosure = new Gclosure('categories', 'categories_hierarchy');
                    // $records = $gclosure->tree(array(), array('select' => 't.id, t.name, t.slug, t.parent_id', 'sort_by' => 't.name'));	
                    $this->db->limit($limit, $offset);
                    $result = array();
                    $query_where = '';
                    if(!empty($q)) {
                        $query_where = "name LIKE '%" . escape($q) . "%'";
                    }
                    if(!empty($toggle_module)) {
                        $toggle_module_id = gc_module_id($toggle_module);
                        $query_where .= (!empty($query_where) ? ' AND ' . $query_where : '') . 'FIND_IN_SET(' . $toggle_module_id . ', module_relations)';
                    }
                    if(!empty($query_where)) {
                        $this->db->where($query_where);
                    }
                    $records = $this->db->select('id, name')->get('categories')->result();                    
                    if(!empty($query_where)) {
                        $this->db->where($query_where);
                    }
                    $total_rows = $this->db->count_all_results('categories');
                } else {
                    $parent_id = $this->get('parent_id') ? (int) $this->get('parent_id') : 0;
                    $md = escape($this->get('md'));
                    $where = "id = " . $parent_id;
                    $this->load->model('categories/categories_model');
                    $parent_category = '';
                    if($parent_id != 0) {
                        $parent_name = null;
                        $sub_category = null;
                        $parent_result = $this->categories_model->get_translated($where);
                        if($parent_result) {
                            $sub_category = $parent_result->name;
                            if($parent_result->parent_id != 0) {
                                $where = "id = " . $parent_result->parent_id;
                                // $where['id'] = $parent_result->parent_id;
                                $parent = $this->categories_model->get_translated($where);
                                if($parent) {
                                    $parent_name = $parent->name;
                                }
                            }
                        }
                        $parent_category = $parent_name != null ? $parent_name . ' -> ' . $sub_category : $sub_category;
                    }
                    $result = array();
                    $query_where = array();
                    if(!empty($q)) {
                        $query_where = $q;
                    }
                    $categories = $this->categories_model->categories(0, $parent_id, (!empty($md) ? $md : 'listings'), $query_where);
                    $total_rows = $this->categories_model->categories(1, $parent_id, (!empty($md) ? $md : 'listings'), $query_where);
                    if(!empty($categories)) {
                        foreach($categories as $category) {
                            $result[] = array(
                                'parent' => $parent_category,
                                'id' => $category->id,
                                'name' => $category->name,
                                'description' => !empty($category->description) ? $category->description : null,
                                'image' => !empty($category->image) ? gc_image_url($category->image, false, false, 'categories') : gc_site_image('no-image'),
                                'hasChildren' => $category->has_children
                            );
                        }
                    }
                    $records = $result;
                }
                break;
            case 'comments': // view module specific comments
                $md = $this->get('tm');
                $id = $this->get('row_id') != null ? $this->get('row_id') : 0;
                $module_info = $this->mobiles_model->get_info($md, $id);
                if(!$module_info) {
                    $this->set_response(false, REST_Controller::HTTP_NOT_FOUND);
                    return false;
                }
                $this->load->model('comments/comments_model');
                $records = $this->comments_model->get_comments(0, $md, $id, $q, $limit, $offset, $user_id, true);
                $total_rows = $this->comments_model->get_comments(1, $md, $id, $q, $limit, $offset, $user_id, true);
                $extra_data['toolbar_title'] = lang('gc_menu_comments') . ' - ' . $module_info->title; 
                break;
            case 'locations':
                $md = $this->get('md');
                $id = $this->get('id') != null ? $this->get('id') : 0;
                $records = $this->mobiles_model->load_locations(0, $md, $id, $q, $limit, $offset);
                $total_rows = $this->mobiles_model->load_locations(1, $md, $id, $q);
                break;
            case 'cities':
                $country = $this->get('country');
                $records = $this->mobiles_model->get_cities(0, $q, $country, $limit, $offset);
                $total_rows = $this->mobiles_model->get_cities(1, $q, $country);
                break;
            case 'localities':
                $city_id = (int) $this->get('city_id');
                $records = $this->mobiles_model->load_areas(0, $city_id, $q, $limit, $offset);
                $total_rows = $this->mobiles_model->load_areas(1, $city_id, $q);
                break;
            case 'postcodes': // postal/zip code search
                $city_id = (int) $this->get('city_id');
                $records = $this->mobiles_model->load_zip(0, $city_id, $q, $limit, $offset);
                $total_rows = $this->mobiles_model->load_zip(1, $city_id, $q);
                break;
            case 'tags':                
                $records = $this->mobiles_model->load_tags(0, $q, $limit, $offset);
                $total_rows = $this->mobiles_model->load_tags(1, $q);
                break;
            case 'orders':
                $this->load->model('orders/orders_model');
                $records = array();
                $rows = $this->orders_model->retrieve_orders($user_id, $offset, $this->limit);
                if($rows) {
                    foreach($rows as $key => $record) {
                        $row = array(
                            'order_id' => $record['order_id'],
                            'created_on' => gc_date_format($record['created_on'], TRUE),
                            'amount' => $record['order_total'] == 0 ? lang('label_free') : currency_format($record['order_total']),
                            'status' => !empty($record['status_lang']) ? lang($record['status']) : $record['status'],
                            'status_color' => $record['status_color'],
                            'redirect_url' => '/member/info-detail/orders/' . $record['id']
                        );
                        $records[] = $row;
                    }
                }
                $total_rows = $this->orders_model->count_by(array('user_id' => $user_id, 'deleted' => 0));
                $extra_data['toolbar_title'] = lang('members_orders'); 
                $extra_data['created_on'] = lang('label_placed_on');
                $extra_data['amount'] = lang('label_amount');
                $extra_data['status'] = lang('label_status');
                $extra_data['view'] = lang('label_view');
                break;                
            case 'faq_groups':
            case 'faqs':
                $group_id = (int) $this->get('group_id');
                $cacheID = $module . '_' . $group_id . '_' . $offset . '_' . $limit;
                if(!$result = $this->gc_mobiles_cache->load($cacheID)) { // cache results
                    $this->lang->load('faqs/faqs');
                    $this->load->model('faqs/faqs_model');
                    if($module == 'faqs') {
                        $result = $this->faqs_model->get_info('faqs', $group_id, $offset, $limit);
                        $extra_data['toolbar_title'] = lang('gc_menu_' . $module);
                    } else {
                        $result = $this->faqs_model->get_info('groups', 0, $offset, $limit);
                        $extra_data['toolbar_title'] = lang('faq_groups');
                    }
                    $records = $result['records'];
                    $total_rows = $result['total_rows'];
                    $result = array('records' => $records, 'total_rows' => $total_rows, 'extra_data' => $extra_data);
                    $this->gc_mobiles_cache->save($result, $cacheID, array($module));
                }
                $records = $result['records'];
                $total_rows = $result['total_rows'];
                $extra_data = $result['extra_data'];
                break;
			case 'pages':
			case 'news':
			    $year = $this->get('year'); // success stories year filter
			    $this->db->limit($limit, $offset);
				if(!empty($q)) {
                    $this->db->or_like('title', $q);
                }
                $records = $this->db->where(array('status' => 1))->order_by('id', 'DESC')->get($module)->result();
                $rows = array();
				if($records) {
					foreach($records as $record) {
						$rows[] = $this->mobiles_model->format_others($module, $record);
					}
				}
				$records = $rows;
				if(!empty($q)) {
                    $this->db->or_like('name', $q);
                }
                $total_rows = $this->db->where(array('status' => 1))->count_all_results($module);
                $extra_data['toolbar_title'] = lang('gc_menu_' . $module);
				if($module == 'success_stories') { // year specific data
				    $years = array(
				        array(
				            'name' => lang('gc_menu_all'),
				            'value' => 'all'
				        )
				    );
				    $date = date("Y"); //current year				    
				    $date_range = range($date, $date-4);//get year range
				    foreach($date_range as $range_year) {
				        $years[] = array(
				            'name' => $range_year,
				            'value' => $range_year
				        );
				    }
				    $extra_data['years'] = $years;
				    $extra_data['read_more'] = lang('label_read_more');
				}
				break;
			case 'notes':
			    $this->db->limit($limit, $offset);
			    if(!empty($q)) {
			        $this->db->or_like('title', $q);
			    }
                $records = $this->db->select('id, title, description, created_on')->get($module)->result();
                if(!empty($q)) {
                    $this->db->or_like('title', $q);
                }
                if($records) {
                    foreach($records as $key => $record) {
                        $records[$key]->created_on = gc_date_format($record->created_on, true);
                    }
                }
                $total_rows = $this->db->count_all_results($module);
                $extra_data['toolbar_title'] = lang('gc_menu_notes'); 
				break;				
			case 'view_comments':
			case 'my_comments':
			case 'view_business_queries':
			case 'posted_queries':
			case 'favorites':
			    if($user_id > 0) {
			        $toggle_module = escape($this->get('tm'));
			        $row_id = (int) $this->get('row_id');
			        $output = $this->mobiles_model->member_view_module($user_id, $module, $toggle_module, $row_id, $offset, $limit);
			        $records = $output['records'];
			        $total_rows = $output['total_rows'];
			        $segment_modules = array();
			        foreach(gc_modules() as $gc_module) {
			            $segment_modules[] = array(
			                'name' => $gc_module,
			                'label' => lang($gc_module)
			            );
			        }
			        $extra_data['modules'] = $segment_modules;
			        $extra_data['toolbar_title'] = $output['toolbar_title']; 
			    } else {
			        $result['success'] = false; 
			        $records = array();
			        $total_rows = array();
			    }
    			break;
			case 'wallets':
			case 'points':
			    $this->lang->load('members/members');
			    $this->lang->load('wallets/wallets');
			    $this->load->model('wallets/wallets_points_model');
			    $user = $this->db->select('wallets, points')->where('id', $user_id)->get('users')->row();
			    if(!$user) {
			        $this->set_response(false, REST_Controller::HTTP_NOT_FOUND);
			    }
			    $where = "user_id = " . $user_id; // module_id = " . gc_module_id($module) . " AND 
			    $sort_by = "WP.id DESC";
			    $total_rows = $this->wallets_points_model->get_history(1, $module, $where);
			    $rows = $this->wallets_points_model->get_history(0, $module, $where, $sort_by, $offset, $limit);
			    $records = array();
			    if($rows) {
			        if($module == 'points') {
    			        foreach($rows as $row) {
    			            $records[] = array(
    			                'id' => $row->id,
    			                'title' => $this->wallets_points_model->transaction($row->module_id, $module),
    			                /*'debit' => $row->debit,
    			                'credit' => $row->credit,
    			                'opening_balance' => $row->opening_balance,
    			                'closing_balance' => $row->closing_balance,*/
    			                'amount' => !empty($row->debit) ? $row->debit : $row->credit,
    			                'is_credit' => !empty($row->debit) ? 0 : 1,
    			                'description' => $this->wallets_points_model->transaction_description($row),
    			                'created_on' => gc_date_format($row->created_on)
    			            );
    			        }
			        } else {			            
			            foreach($rows as $row) {
			                if($row->module_id == 21) {
			                    $transaction_module = lang('wallet_transfer');
			                } else {
			                    $transaction_module = $this->wallets_points_model->transaction($row->module_id, $module);
			                }
			                $records[] = array(
			                    'id' => $row->id,
			                    'title' => $transaction_module,
			                    'amount' => currency_format(!empty($row->debit) ? $row->debit : $row->credit),
			                    'is_credit' => !empty($row->debit) ? 0 : 1,
			                    'description' => $this->wallets_points_model->transaction_description($row),
			                    'created_on' => gc_date_format($row->created_on)
			                );
			            }
			        }
			    }
			    if($module == 'wallets') {
    			    $links = array(
    			        array(
        			        'label' => lang('gc_action_transfer'),
        			        'url' => '/crud/wallet_transfer/1/wallets/0',
        			        'color' => 'primary',
        			        // 'icon' => 'repeat-outline' // spacing issue
    			        ),
    			        array(
    			            'label' => lang('gc_action_withdraw'),
    			            'url' => '/crud/wallet_withdraw/1/wallets/0',
    			            'color' => 'secondary',
    			            // 'icon' => 'cloud-download-outline' // spacing issue
    			        )
    			    );
    			    $extra_data['links'] = $links;
			    }
			    $extra_data['toolbar_title'] = lang('my_' . $module);
			    $extra_data['credit'] = lang('wallet_credit');
			    $extra_data['debit'] = lang('wallet_debit');
			    $extra_data['amount'] = $module == 'wallets' ? currency_format($user->wallets) : $user->points;
			    if($module == 'wallets') {
			        $extra_data['toolbar_title'] = lang('label_wallet');
			        $extra_data['redirect_url'] = '/crud/checkout_wallet/0';
			    }
			    break;
			case 'my_requirements':
			    if($module == 'my_requirements' && empty($user_id)) {
			        $this->set_response(false, REST_Controller::HTTP_NOT_FOUND);
			        return false;
			    }
			    $this->lang->load('members/members');
			    $this->load->model('leads/leads_model');
			    $where = 'CN.deleted = 0 AND CN.user_id = ' . $user_id;			    
			    $filter = escape($this->get('tm'));
			    switch($filter) {
			        case 'today':
			            $where .= " AND DATE(CN.created_on) = '" . date('Y-m-d') . "'";
			            break;
			        case 'week':
			            $where .= " AND DATE(CN.created_on) BETWEEN '" . date('Y-m-d') . "' AND '" . date('Y-m-d', strtotime('+1 week')) . "'";
			            break;
			        case 'month':
			            $where .= " AND DATE(CN.created_on) BETWEEN '" . date('Y-m-d') . "' AND '" . date('Y-m-d', strtotime('+1 month')) . "'";
			            break;
			        case 'forwarded':
			            $where .= " AND (LE.user_id = " . $user_id ." OR CN.assigned_to = " . $user_id . ")"; // Specific to this user
			            $where .= " AND (CN.status_id = 2 OR CN.status_id = 3)"; // Paid and
			            break;
			    }			    
			    $records = array();
			    $rows = $this->leads_model->get_leads($offset, $limit, $q, $where, "CN.id DESC", 0, $filter, $user_id);
			    if($rows) {
			        foreach($rows as $row) {	
			            $links = array();
			            $links[] = array(
			                'label' => lang('detail'),
			                'url' => '/tabs/member/info-detail/lead/' . $row->id,
			                'color' => 'secondary',
			                'icon' => 'information-circle-outline'
			            );
			            $summary = json_decode($row->summary ?? '');
			            if(!empty($summary->name)) {
    			            $records[] = array(
    			                'id' => $row->id,
    			                'title' => $row->category,
    			                'name' => $summary->name,
    			                'address' => !empty($summary->location) ? $summary->location : $summary->address,
    			                'price' => currency_format($row->price),
    			                'created_on' => gc_date_format($row->created_on),
    			                'links' => $links
    			            );
			            }
			        }
			    }
			    $total_rows = $this->leads_model->get_leads($offset, $limit, $q, $where, "CN.id DESC", 1, $filter);
			    if($module == 'leads') {
    			    $segment_modules = array(
    			        array(
        			        'name' => 'all',
        			        'label' => lang('tab_all')
    			        ),
    			        array(
    			            'name' => 'today',
    			            'label' => lang('lead_today')
    			        ),
    			        array(
    			            'name' => 'week',
    			            'label' => lang('lead_week')
    			        ),
    			        array(
    			            'name' => 'month',
    			            'label' => lang('lead_month')
    			        ),
    			        array(
    			            'name' => 'forwarded',
    			            'label' => lang('lead_forwarded')
    			        ),
    			    );
    			    $extra_data['modules'] = $segment_modules;
    			    $extra_data['searchbar'] = true;
			    }
			    $extra_data['toolbar_title'] = $module == 'leads' ? lang('gc_menu_leads') : lang('my_requirements');
			    break;
			case 'leads': // get information from the broadcasts table
			    $this->lang->load('leads/leads');
			    $where = null;
			    $toggle_module = 'broadcasts';
			    $prefix = 'lead';
			    $sort_by = 'M.id DESC';
			    $where = "M.seller_id = " . $user_id;
			    $records = array();
			    $rows = $this->gcrud->get_all_crud(0, $where, $sort_by, $limit, $offset, array('module' => $toggle_module, 'prefix' => $prefix));
			    if($rows) {
			        foreach($rows as $record) {
			            $links = array();
			            $links[] = array(
			                'label' => lang('detail'),
			                'url' => '/tabs/member/info-detail/lead/' . $record->lead_id,
			                'color' => 'secondary',
			                'icon' => 'information-circle-outline'
			            );
			            if($record->status_id == 1) { // change status to available
    			            $links[] = array(
    			                'label' => lang('status_available'),
    			                'url' => '/crud/lead_form/' . $record->id,
    			                'color' => 'success',
    			                'icon' => 'checkmark-circle-outline'
    			            );
			            }
			            $business_summary = json_decode($record->business_summary ?? '');
			            $row = json_decode($record->summary ?? ''); // user summary
			            if(!empty($row)) {
			                $records[] = array(
			                    'id' => $record->id,
			                    'title' => $row->category,
			                    'name' => $row->name,
			                    'address' => $row->location,
			                    // 'image' => gc_image_url($business_summary->cover),
			                    'user_distance' => $business_summary->distance . ' ' . lang('label_km'),
			                    'phone_number' => $row->phone_number,
			                    'created_on' => gc_date_format($record->forwarded_on),
			                    // 'description' => $record->description,
			                    'status' => !empty($record->status_lang) ? lang($record->status) : $record->status,
			                    'status_color' => $record->status_color,
			                    'links' => $links
			                );
			            }
			        }
			    }
			    $total_rows = $this->gcrud->get_all_crud(1, $where, $sort_by, $limit, $offset, array('module' => $toggle_module, 'prefix' => $prefix));
			    $extra_data['toolbar_title'] = lang('gc_menu_leads');
			    break;
			case 'view_orders':
			    $this->load->model('orders/orders_model');
			    $records = $this->orders_model->retrieve_orders($user_id, $offset, $limit);
			    $total_rows = $this->orders_model->count_by(array('user_id' => $user_id, 'deleted' => 0));
			    $extra_data['toolbar_title'] = lang('members_orders');
			    break;
			case 'ads':
			    $toggle_module = escape($this->get('tm'));
			    $toggle_row_id = (int) $this->get('row_id');
			    // check whether the user is the owner of the module or not
			    // $this->valid_owner($user_id, $toggle_row_id, $toggle_module);
			    $module_table = 'mobile_sliders';
			    $sort_by = 'id DESC';
			    $where = 'M.module_id = ' . gc_module_id($toggle_module) . ' AND M.ad_row_id = ' . $toggle_row_id;
			    $records = array();
			    $rows = $this->gcrud->get_all_crud(0, $where, $sort_by, $limit, $offset, array('module' => $module_table, 'prefix' => ''));
			    if($rows) {
			        foreach($rows as $row) {
			            if(!empty($row->start_date)) {
			                $row->start_date = gc_date_format($row->start_date, true);
			            }
			            if(!empty($row->end_date)) {
			                $row->end_date = gc_date_format($row->end_date, true);
			            }
			            $daily_budget = empty($row->daily_budget)? lang('no_limit') : currency_format($row->daily_budget);
			            $records[] = array(
			                'id' => $row->id, // update status
			                'title' => $row->title,
			                'image' => !empty($row->image) ? gc_image_url($row->image, false, false, array('module' => 'mobile_sliders')) : '', //base_url('assets/images/' . $row->image) : '',
			                'start_date' => $row->start_date,
			                'end_date' => $row->end_date,
			                'todays_spent' => currency_format($row->todays_spent),
			                'daily_budget' => $daily_budget,
			                'ad_views' => $row->total_ad_views,
			                'distance' => sprintf(lang('ad_radius_coverage'), $row->max_distance),
			                'created_on' => gc_date_format($row->created_on),
			                'status' => (int) $row->status,//empty($row->status) ? lang('us_inactive') : lang('us_active')			                
			                'redirect_url' => '/crud/' . $module . '/' . $row->id . '/' . $toggle_module . '/' . $toggle_row_id
			            );
			        }
			    }
			    $total_rows = $this->gcrud->get_all_crud(1, $where, $sort_by, $limit, $offset, array('module' => $module_table, 'prefix' => ''));
			    $extra_data['toolbar_title'] = lang('gc_menu_ads');
			    $extra_data['redirect_url'] = '/crud/ads/0/' . $toggle_module . '/' . $toggle_row_id;
			    $extra_data['ad_views'] = lang('mobile_ad_views');
			    $extra_data['ad_radius'] = lang('ad_radius');
			    $extra_data['ad_todays_spent'] = lang('ad_todays_spent');
			    $extra_data['ad_daily_budget'] = lang('ad_daily_budget');
			    break;
			case 'training_groups':
			case 'training_videos':
			    $group_id = (int) $this->get('row_id');
			    $cacheID = $module . '_' . $group_id;
			    // if(!$result = $this->gc_mobiles_cache->load($cacheID)) { // cache results
    			    $this->lang->load('videos/videos');
    			    $this->load->model('videos/videos_model');
    			    if($module == 'training_videos') {
    			        $result = $this->videos_model->training('videos', $group_id);
    			        $extra_data['toolbar_title'] = lang('training_videos');
    			    } else {
    			        $result = $this->videos_model->training('groups');
    			        $extra_data['toolbar_title'] = lang('training_groups');
    			    }
    			    $records = $result['records'];
    			    $total_rows = $result['total_rows'];
    			    $result = array('records' => $records, 'total_rows' => $total_rows, 'extra_data' => $extra_data);
    			    $this->gc_mobiles_cache->save($result, $cacheID, array($module));
			    // }
			    $records = $result['records'];
			    $total_rows = $result['total_rows'];
			    $extra_data = $result['extra_data'];
			    break;	    
			case 'variants':
			    if(empty($user_id)) {
			        $this->set_response(false, REST_Controller::HTTP_NOT_FOUND);
			        return false;
			    }
			    $product_id = (int) $this->get('row_id');
			    $product = $this->db->where('id', $product_id)->select('id, user_id, title, slug')->get('products')->row();
			    if(!$product || $product->user_id != $user_id) {
			        $this->set_response(false, REST_Controller::HTTP_NOT_FOUND);
			        return false;			        
			    }
			    $this->db->limit($limit, $offset);
			    $records = array();
			    $rows = $this->db->where('product_id', $product_id)->get('product_variants')->result();
			    if($rows) {
			        foreach($rows as $record) {
			            $row = array(
			                'id' => $record->id,
			                'name' => $record->name,
			                'price' => currency_format($record->price),
			                'redirect_url' => 'crud/variant/' . $record->product_id . '/variants/' . $record->id// edit url
			            );
			            if(!empty($record->offer_price)) {
			                $extra = calculate_savings($record->price, $record->offer_price, 'variant-savings');
			                $row['savings'] = $extra['html'];
			                $row['price'] = currency_format($record->offer_price);
			            } else if(!empty($record->price)) {
			                $row['price'] = currency_format($record->price);
			            }
			            $records[] = $row;
			        }
			    }
			    $total_rows = $this->db->where('product_id', $product_id)->count_all_results('product_variants');
			    $extra_data['toolbar_title'] = lang('gc_menu_variants') . ' - ' . $product->title;			    
			    $extra_data['redirect_url'] = '/crud/variant/' . $product_id . '/variants/0';
			    break;
			default:
			    if(in_array($module, gc_modules())) {
			        $field = 'title';
			    } else {
			        $field = 'name';
			    }
			    $this->db->limit($limit, $offset);
			    if(!empty($q)) {
			        $this->db->or_like($field, $q);
			    }
			    $records = $this->db->select('id, ' . $field . ' AS name')->get($module)->result();
			    if(!empty($q)) {
			        $this->db->or_like($field, $q);
			    }
			    $total_rows = $this->db->count_all_results($module);
        }        
        $total_pages = 0;
        // IF THERE IS PAGINATION, ONLY THEN ADD REL LINKS
        if($total_rows > 0 && $total_rows > $limit) {
            $total_pages = ceil($total_rows/$this->limit);
            // $this->mobiles_model->canonical_links($total_pages, $page);
        }
        $result = array(
            'records' => $records,
            'total_rows' => $total_rows,
            'total_pages' => $total_pages == 0 ? 1 : $total_pages,
			'extra' => $extra_data
        );
		if(empty($result['records'])) {
		    $result['extra']['message'] = lang('error_no_record_found');
		}
		/*if(empty($result['records'])) {
			$result['extra'] = array('message' => lang('error_no_record_found'));
			$this->set_response($result, REST_Controller::HTTP_NOT_FOUND);
		} else {
			// Set the response and exit
			$this->response($result, REST_Controller::HTTP_OK); // OK (200) being the HTTP response code
		}*/
		// Set the response and exit
		$this->response($result, REST_Controller::HTTP_OK); // OK (200) being the HTTP response code
	}
    
    /**
     * Fetch Record - Common Function for Signup, Pages and News
     * @param string $module
     * @param number $id
     */
    public function fetch_options() {
        $this->header_options();
    }
    public function fetch_get() {
        $this->header_params();
        $record = array();
        $module = escape($this->get('module'));
        $id = (int) $this->get('id');
        $user_id = $this->user_id();
		switch($module) {
		    case 'detail':
		        $md = $this->get('md');
		        $row_id = (int) $this->get('row_id');
		        // $record = $this->gc->module_detail($md, $row_id, $user_id, true);
		        $record = $this->mobiles_model->module_detail($md, $row_id, $user_id, false);
		        break;
		    case 'comments':
		        $md = $this->get('md');
		        $row_id = (int) $this->get('row_id');
		        $module_id = gc_module_id($md);
		        // Display comments to the User
		        $this->load->model('comments/comments_model');
		        $result['comments_count'] = $this->comments_model->count_by(array('module_id' => $module_id, 'row_id' => $row_id, 'deleted' => 0, 'status' => 1));
		        $result['comments_html'] = $this->comments_model->comment_tree($module_id, $row_id);
		        $result['success'] = true;
		        $record = $result;
		        break;
		    case 'checkout_address':
		        $this->lang->load('orders/orders');
		        $user_profile = $this->mobiles_model->get_user_profile($user_id, false);
		        if(!$user_profile) {
		            $this->response(false, REST_Controller::HTTP_NOT_FOUND);
		            return false;
		        }
		        $record = array(
		            'name' => $user_profile->name,
		            'address' => $user_profile->address,
		            'email' => $user_profile->email,
		            'phone_number' => $user_profile->phone_number,
		            'city_id' => $user_profile->city_id,
		            'location' => $user_profile->city_id,
		            'postcode' => $user_profile->postcode
		        );
		        $this->load->model('fields/fields_model');
		        $form_fields = $this->fields_model->get_fields('orders', 'frontend'); // custom fields
		        $fields = $this->mobiles_model->format_fields($form_fields);	
		        $record = $this->mobiles_model->format_data($fields, $record);
		        $record = array(
		            'success' => true,
		            'fields' => $fields,
		            'record' => $record,
		            'extra' => array(
		                'name' => lang('checkout_billing_address'),
		                'btn_text' => lang('label_checkout'),
		                'icon' => 'location-outline',
		                'toolbar_title' => lang('label_checkout'),
		                'help' => lang('form_heading_title')
		            ),
		        );
		        break;
		    case 'orders':
		    case 'checkout_status':
				// $order_id = $this->get('row_id');
				$this->load->model('orders/orders_model');
		        $order = $this->orders_model->get_order($id, 'id');
		        if(!$order || $order->user_id != $user_id) { //  || $order->user_id != $user_id
		            $this->response(false, REST_Controller::HTTP_NOT_FOUND);
		            return false;
		        }
		        $actions = array(
		            array('label' => lang('bf_action_download'), 'color' => 'primary', 'icon' => 'download-outline', 'url' => 'download'),
		            array('label' => lang('label_email'), 'color' => 'tertiary', 'icon' => 'mail-outline', 'url' => '/tabs/crud/order_email/' . $id . '/users/0'),
		        );
		        $content = '';
		        if($module == 'checkout_status') {
		            $this->lang->load('payment_gateways/payment_gateways');
		            if($order->status_id == 2) { // successful payment
		                $content = $this->load->view('orders/thankyou', array('order_id' => $order->order_id, 'email_id' => $order->user_email), true);
		            } else if($order->status_id == 4) { // the user cancelled the order
		                $content = $this->load->view('orders/cancelled', array('order_id' => $order->order_id, 'email_id' => $order->user_email), true);
		            } else if($order->payment_method == 'BankTransfer') { // the user selected bank transfer as the payment method
		                $payment_method = $this->db->where('name', 'BankTransfer')->get('payment_gateways')->row();
		                if($payment_method) {
		                    $payment_settings = unserialize($payment_method->settings);
		                    $content = $payment_settings['payment_instructions'] . '<br />';
		                    $actions[] = array(
		                        'label' => lang('transaction_update'),
		                        'icon' => 'create-outline',
		                        'color' => 'danger',
		                        'url' => '/tabs/member/crud/transaction_update/' . $id . '/users/0'
		                    );
		                }
		                $content .= $this->orders_model->email_print($order->order_id, 'view', array('order' => $order, 'role' => 'frontend'));
		            } else if($order->status_id == 9) { // pending for admin confirmation		                
		                $content = $this->load->view('orders/pending', array('order_id' => $order->order_id, 'email_id' => $order->user_email), true);
		            } else { // payment not processed // if(empty($order->payment_details) && $order->status_id == 1) 
		                $content = '<div class="bg-theme-primary p-1">' . lang('msg_payment_processing_error') . '</div>';
		                $content .= $this->load->view('orders/cancelled', array('order_id' => $order->order_id, 'email_id' => $order->user_email), true);
		            }		         
		            // allow user to go back to a specific page
		            $order_items = $this->orders_model->get_order_items($order->order_id);
		            $back_url = '/tabs/member/info-pages/orders/all';
		            if($order_items && $order_items[0]->module_id == gc_module_id('wallets')) {
		                $back_url = '/tabs/member/info-pages/wallets/all';
		            }
		            $actions[] = array(
		                'label' => lang('bf_go_back'),
		                'icon' => 'arrow-back-outline',
		                'color' => 'primary',
		                'url' => $back_url
		            );
		        } else {
		            $content = '';
		            if($order->payment_method == 'BankTransfer' && ($order->status_id == 1 || $order->status_id == 9)) { // the user selected bank transfer as the payment method
		                $payment_method = $this->db->where('name', 'BankTransfer')->get('payment_gateways')->row();
		                if($payment_method) {
		                    $payment_settings = unserialize($payment_method->settings);
		                    $content = $payment_settings['payment_instructions'] . '<br />';
		                    $actions[] = array(
		                        'label' => lang('transaction_update'),
		                        'icon' => 'create-outline',
		                        'color' => 'danger',
		                        'url' => '/tabs/member/crud/transaction_update/' . $id . '/users/0'
		                    );
		                }
		            }
		            $content .= $this->orders_model->email_print($order->order_id, 'view', array('order' => $order, 'role' => 'frontend'));
		        }
		        $record = array(
		            'success' => true,
		            'record' => $content,
		            'toolbar_title'	=> lang('label_order_id') . ': ' . $order->order_id,
		            'extra' => array(
		                'toolbar_title'	=> lang('label_order_id') . ': ' . $order->order_id,
		                'actions' => $actions,
		                'order_id' => $order->order_id,
		                'download_url' => site_url('orders/download?order_id=' . $order->order_id)
		            )
		        );
		        break;
		    case 'checkout_wallet':
		        $this->lang->load('wallets/wallets');
		        $this->lang->load('orders/orders');
		        $this->load->model('payment_gateways/payment_gateways_model');
		        $user_profile = array();
		        if($user_id > 0) {
		            $user_profile = $this->mobiles_model->get_user_profile($user_id, false);
		            // $user_profile = $this->mobiles_model->format_data($fields, $user_profile);
		        }
		        $fields = array();
		        $payment_options = $this->payment_gateways_model->get_payment_gateways();
		        $payment_methods = $payment_options['payment_methods'];
		        $fields[] = array('name' => 'amount', 'label' => lang('label_amount'), 'type' => 'text', 'rules' => array('required' => 'required'));
		        $fields[] = array('name' => 'payment_method', 'label' => lang('label_payment_method'), 'type' => 'select', 'options' => $payment_methods, 'rules' => array('required' => 'required'));
		        $fields = $this->mobiles_model->rest_fields($fields);
		        $record = array(
		            'success' => true,
		            'fields' => $fields,
		            'record' => array('amount' => 500, 'payment_method' => $payment_options['default']),
		            'extra' => array(
		                'name' => lang('wallet_checkout'),
		                'btn_text' => lang('label_checkout'),
		                'icon' => 'card-outline',
		                'help' => sprintf(lang('wallet_balance'), currency_format($user_profile->wallets)),
		                'toolbar_title' => lang('item_description_wallet')
		            )
		        );
		        break;
		    case 'transaction_update': // order transaction id udpate
		    case 'order_email': // order email
		        if($user_id == 0) {
		            $this->response(array('success' => false), REST_Controller::HTTP_NOT_FOUND);
		        }
		        $this->load->model('orders/orders_model');
		        $order_id = (int) $this->get('group_id');
		        if($module == 'transaction_update') {
		          $record = $this->orders_model->transaction_update($order_id, $user_id, 'mobile');
		        } else {
		            $record = $this->orders_model->order_email($order_id, $user_id, 'mobile');		            
		        }
		        break;
		    case 'pages':	
		    case 'news':
				$where = array('id' => $id);
				$record = $this->gcrud->frontend_detail($module, array('where' => $where), true);
				break;
		    case 'notes':
		        $this->load->model('notes/notes_model');
		        $record = $this->notes_model->get_note($id);
		        if($record) {
		            $record = array(
		                'success' => true,
		                'record' => $record
		            );
		        } else {
		            $this->response(array('success' => false), REST_Controller::HTTP_NOT_FOUND);
		        }
		        break;
		    case 'testimonials':	
		        $this->lang->load('testimonials/testimonials');
		        $this->load->model('testimonials/testimonials_model');
		        $record = $this->testimonials_model->get_testimonial($id);
		        break;
		    case 'gallery': // allow user to upload module images
		        $tm = escape($this->get('tm'));
		        $row_id = (int) $this->get('row_id');
		        $row = $this->db->where('id', $row_id)->get($tm)->row();
		        if(!$row) {
		            $this->response(array('success' => false), REST_Controller::HTTP_NOT_FOUND);
		        }
		        // get images
		        $images = array();
		        $records = $this->db->where(array('module_id' => gc_module_id($tm), 'row_id' => $row_id, 'type_id' => 0))->get('images')->result();
		        if($records) {
		            foreach($records as $record) {
		                $images[] = array(
		                    'id' => $record->id,
		                    // 'thumb' => gc_image_url($record->url), // base_url($record->path . '/thumbs/thumb__' . $record->filename),
		                    'original' => gc_image_url($record->url, false)
		                );
		            }
		        }
		        // get package
		        $this->load->model('packages/packages_model');
		        $package = $this->packages_model->get_package($row->package_id);
		        $limit = 1;
		        if($package) {
		            $limit = json_decode($package->module_images_limit ?? '', true);
		            $limit = !empty($limit[$tm]) ? $limit[$tm] : 1;
		        }
		        $fields = array();
		        $fields[] = array('name' => 'images', 'label' => lang('label_image_uploader'), 'type' => 'image', 'rules' => array('required' => 'required'), 'params' => array('files_limit' => $limit));
		        $fields = $this->mobiles_model->rest_fields($fields);
		        $toolbar_title = lang('bf_action_update') . ' ' . lang('gc_menu_images');
		        $record = array(
		            'success' => true,
		            'fields' => $fields,
		            'record' => array('images' => $images),
		            'extra' => array(
		                'name' => $toolbar_title,
		                'icon' => 'camera-outline',
		                'help' => lang('help_images_upload'),
		                'toolbar_title' => $row->title
		            )
		        );
		        break;
			case 'profile_image': // allow user to change profile_image
			    $user_profile = array();
			    if($user_id > 0) {
			        $user_profile = $this->mobiles_model->get_user_profile($user_id, false);
			    }
			    $fields = array();
			    $fields[] = array('name' => 'profile_image', 'label' => lang('label_image_uploader'), 'type' => 'image', 'rules' => array('required' => 'required'), 'params' => array('files_limit' => 1));
			    $fields = $this->mobiles_model->rest_fields($fields);
			    $toolbar_title = lang('bf_action_update') . ' ' . lang('label_profile_image');
			    $record = array(
			        'success' => true,
			        'fields' => $fields,
			        'record' => array('profile_image' => $user_profile->profile_image),
			        'extra' => array(
			            'name' => lang('label_profile_image'),
			            'icon' => 'camera-outline',
			            'help' => $toolbar_title,
			            'toolbar_title' => $toolbar_title
			        )
			    );
			    break;
		    case 'add':			    
			    if($user_id == 0) {
			        $this->response(array('success' => false), REST_Controller::HTTP_NOT_FOUND);
			    }
			    $group_id = (int) $this->get('group_id');
			    $listing_id = (int) $this->get('row_id'); // required in case of modules e.g. Jobs, Products, Properties etc.
			    // $group_id = $group_id == 0 ? 99999 : $group_id;
			    $toggle_module = escape($this->get('tm'));
			    $record = $this->mobiles_model->add_module($group_id, $toggle_module, $user_id, $listing_id);
			    if(!$record['success']) {
			        $this->response($record, REST_Controller::HTTP_NOT_FOUND);
			    }
			    break;
			case 'edit': // Member Area - Display Saved Information
			    if($user_id == 0) {
			        $this->response(array('success' => false), REST_Controller::HTTP_NOT_FOUND);
			    }
			    $toggle_module = escape($this->get('tm'));
			    if($toggle_module == 'users') { // the user can edit his/her own profile only
			         $row_id = $user_id;   
			    } else {
			         $row_id = (int) $this->get('row_id');
			    }
			    $record = $this->mobiles_model->module_detail($toggle_module, $row_id, $user_id, true);
			    break;
			case 'groups':		
			    if($user_id == 0) {
			        $this->response(array('success' => false), REST_Controller::HTTP_NOT_FOUND);
			    }
			    $this->lang->load('listings/listings');
			    $group_id = (int) $this->get('group_id');
			    $toggle_module = escape($this->get('tm'));
			    if($toggle_module == 'users') {
			        $row_id = $user_id;   
			    } else {
			        $row_id = (int) $this->get('row_id');
			    }
			    // check whether the user is the owner of the listing or not
			    $row = $this->db->where('id', $row_id)->get($toggle_module)->row();
			    if($row) {
			        $record = $this->mobiles_model->group_form($toggle_module, $row_id, $group_id, 'mobile', $user_id);
			        // $record = $record['record'];
			        // $record['record'] = $this->mobiles_model->format_data($record['fields'], $record['record']);
			    } else {
			        $record = array();
			    }
			    break;
			case 'verify_email':
			    // not required as user email is already verified during registration
			    $user = $this->user_model->get_user($user_id);
			    if($user) {
			        $fields = $this->mobiles_model->rest_fields(array(
			            array('name' => 'verification_code', 'label' => lang('label_code'), 'type' => 'text', 'rules' => array('required' => 'required', 'minLength' => 4, 'maxLength' => 6)),
			        ));
			        $record = array(
			            'success' => true,
			            'fields' => $fields,
			            'extra' => array(
			                'name' => lang('us_change_password'),
			                'icon' => 'envelope',
			                'toolbar_title' => lang('us_change_password'),
			                'help' => $this->user_model->password_hints(true)
			            ),
			        );
			    } else {
			        $record = array();			        
			    }
			    break;
			case 'otp':			    
			    if(empty($user_id)) {
			        $user_id = $this->user_id($this->get('token'));
			    }
			    $user = $this->user_model->get_user($user_id);
			    if($user) {
			        $fields = $this->mobiles_model->rest_fields(array(
			            array('name' => 'otp', 'label' => lang('otp_placeholder'), 'type' => 'text', 'rules' => array('required' => 'required', 'minLength' => 5, 'maxLength' => 6)),
			        ));
			        $otp_limit = $this->user_model->otp_limit($user_id, $user, false, true);
			        $extra = array(
			            'name' => lang('otp_verify_heading'),
			            'icon' => 'call',
			            'toolbar_title' => lang('otp_verify_heading'),
			            'limit' => $otp_limit,
			            'resend_limit' => sprintf(lang('otp_resend_limit'), $otp_limit),
			            'resend_btn' => lang('otp_resend_btn'),
			            'resend_limit_exceeded' => lang('otp_resend_limit_exceeded'),
			            'retry_limit_exceeded' => lang('otp_retry_limit_exceeded'),
			            'help' => lang('otp_sent') . ' ' . $user->phone_number . '<br />' . lang('otp_help'),
			            'btn_text' => lang('otp_verify')
			        );
			        $record = array('fields' => $fields, 'extra' => $extra);
			    } else {
			        $record = array();
			    }			        
			    break;
			case 'notes_form':
			    $id = (int) $this->get('group_id');
			    $this->load->model('notes/notes_model');
			    $record = $this->notes_model->form($user_id, $id);
			    break;
			case 'post_requirements':
			    $lat = (float) $this->get('lat');
			    $lng = (float) $this->get('lng');
			    $user_profile = array();
			    $this->load->model('leads/leads_model');
			    $group_id = (int) $this->get('group_id');
			    $group = $this->leads_model->valid_group($group_id);
			    if(!$group) {
			        // not a valid group, redirect the user to the home page
			        $this->response(array('success' => false), REST_Controller::HTTP_NOT_FOUND);
			    }
			    $group_id = $group->id;
			    $form_fields = $this->leads_model->group_fields($group_id);
			    $fields = $this->mobiles_model->format_fields($form_fields);
			    if($user_id > 0) {
			        $profile = $this->mobiles_model->get_user_profile($user_id);
			        $user_profile = $this->mobiles_model->format_data($fields, $profile);
			    }
			    $record = array(
			        'success' => true,
			        'fields' => $fields,
			        'record' => $user_profile,
			        'extra' => array(
			            'name' => lang('post_your_requirements'),
			            'icon' => 'create',
			            'help' => sprintf(lang('cn_form_help'),''),
			            'toolbar_title' => lang('post_your_requirements')
			        )
			    );
			    break;
			case 'lead':
			    $user = $this->mobiles_model->get_user_profile($user_id, false);
			    $id = (int) $this->get('id');
			    $this->load->model('leads/leads_model');
			    $row = $this->leads_model->detail($id, false, array('role_id' => $user->role_id, 'mobile' => true));
			    if(!$row) {
			        $this->response(false, REST_Controller::HTTP_NOT_FOUND);
			        return false;
			    }
			    $record = $row;
			    $record['success'] = true;
			    break;
			case 'erroneous':
			case 'review':
			case 'business_query':
			    $toggle_module = escape($this->get('tm'));
			    $row_id = (int) $this->get('row_id');
			    $existing_row_id = (int) $this->get('group_id'); // edit comment
			    $is_edit = (int) $this->get('edit'); // a comment can be edited if edit 1 is passed to the crud method
			    $module_info = $this->mobiles_model->get_info($toggle_module, $row_id);
			    if(!$module_info) {
			        $this->response(array('success' => false), REST_Controller::HTTP_NOT_FOUND);
			    }
			    $user_profile = array();
			    if($module == 'business_query') {
			         $fields = $this->gc->load_query_fields($toggle_module, 'frontend', true, true); // load business query fields
			         $fields = $this->mobiles_model->format_fields($fields);
			    } else {
			         $fields = $this->mobiles_model->feedback_callback_query($module);
			    }
			    if($user_id > 0) {
			        if($module == 'review' && !empty($existing_row_id) && $is_edit == 1) {
			            $user_profile = $this->db->select('username AS name, title, comment AS message, rating, url')->where('id', $existing_row_id)->get('comments')->row();
			        } else {
    			        $profile = $this->mobiles_model->get_user_profile($user_id);
    			        $user_profile = $this->mobiles_model->format_data($fields, $profile);
			        }
			    }
			    $name = lang('label_report');
			    if($module == 'review') {
			        $name = lang('label_review');
			    } else if($module == 'business_query') {
			        $name = lang('business_query');
			    }
			    $record = array(
			        'success' => true,
			        'fields' => $fields,
			        'record' => $user_profile,
			        'extra' => array(
			            'name' => $name,
			            'icon' => 'create',
			            'help' => lang('callback_heading_form'),
			            'toolbar_title' => $module_info->title
			        )
			    );
			    break;
			case 'query_reply': // Business Query Reply
			    $query_id = (int) $this->get('group_id');
			    $record = $this->db->where('id', $query_id)->get('business_queries')->row();
			    if(!$record) { //  || $record->owner_id != $user_id || $record->user_id != $user_id
			        $this->response(array('success' => false), REST_Controller::HTTP_NOT_FOUND);
			    }
			    $prefix = 'reply_';
			    $fields[] = array('name' => $prefix . 'subject', 'label' => lang('label_subject'), 'type' => 'text', 'rules' => array('required' => 'required'));
			    $fields[] = array('name' => $prefix . 'message', 'label' => lang('label_message'), 'type' => 'textarea', 'rules' => array('required' => 'required'));
			    $fields = $this->mobiles_model->rest_fields($fields);
			    $record = array(
			        'success' => true,
			        'fields' => $fields,
			        'record' => array(), //array('reply_subject' => 'Re:' . $record->subject),
			        'extra' => array(
			            'name' => lang('label_dated') . '::' . gc_date_format($record->created_on),
			            'icon' => 'create',
			            'help' => lang('form_heading_title'),
			            'toolbar_title' => lang('query_reply')
			        )
			    );
			    break;			    
			case 'query_replies':
			    $this->load->model('business_queries/business_queries_model');
			    $id = (int) $this->get('id');
			    $query = $this->business_queries_model->get_query($id);
			    if(!$query) { //  || $record->owner_id != $user_id || $record->user_id != $user_id
			        $this->response($record, REST_Controller::HTTP_NOT_FOUND);
			    }
			    $replies = array(array(
			        'subject' => $query->user . '::' . gc_date_format($query->created_on), //$row->subject,
			        'message' => $query->message
			    ));
			    $records = $this->business_queries_model->query_replies($id, 0, 50);
			    if($records) {
			        foreach($records as $row) {
			            $replies[] = array(
			                'subject' => $row->user . '::' . gc_date_format($row->created_on), //$row->subject,
			                'message' => $row->message
			            );
			        }
			    }
			    $record = array(
			        'success' => true,
			        'toolbar_title' => lang('view_replies') . ' # ' . $query->id,
			        'query' => array(
			            'message' => $query->message,
			            'created_on' => gc_date_format($query->created_on)
			        ),
			        'replies' => $replies
			    );
			    break;
			case 'contact_query':
			    $user_profile = array();
			    $fields = $this->mobiles_model->feedback_callback_query($module);
			    if($user_id > 0) {
			        $profile = $this->mobiles_model->get_user_profile($user_id);
			        $user_profile = $this->mobiles_model->format_data($fields, $profile);
			    }
			    $record = array(
			        'success' => true,
			        'fields' => $fields,
			        'record' => $user_profile,
			        'extra' => array(
			            'name' => lang('label_contact_us'),
			            'icon' => 'create',
			            'help' => lang('callback_heading_form'),
			            'toolbar_title' => lang('label_contact_us')
			        )
			    );
			    break;
			case 'feedbacks':			
			case 'callbacks':
			    $user_profile = array();
			    $fields = $this->mobiles_model->feedback_callback_query($module);
			    if($user_id > 0) {
			        $profile = $this->mobiles_model->get_user_profile($user_id);
			        $profile->country_iso = 'IN';
			        $user_profile = $this->mobiles_model->format_data($fields, $profile);
			    }
			    $record = array(
    			    'success' => true,
			        'fields' => $fields,
			        'record' => $user_profile,
    			    'extra' => array(
    			        'name' => $module == 'callbacks' ? lang('callback_form_btn') : lang('feedback_message'),
        			    'icon' => 'create',
        			    'help' => $module == 'callbacks' ? lang('callback_heading_form') : lang('feedback_heading_sub'),
        			    'toolbar_title' => $module == 'callbacks' ? lang('callback_form_btn') : lang('feedback_message')
    			    )
			    );
			    break;
			case 'ads': // banner ads
			    $toggle_module = $this->get('tm');
			    $toggle_row_id = (int) $this->get('row_id');
			    
			    // check whether the user is the owner of the module or not
			    // $this->valid_owner($user_id, $row_id, $module);
			    $record = $this->mobiles_model->get_info($toggle_module, $toggle_row_id);
			    if(!$record) {
			        $this->response(array('success' => false), REST_Controller::HTTP_NOT_FOUND);
			    }
			    
			    $id = (int) $this->get('group_id');
			    $row = array();
			    if(!empty($id)) {
			        $row = (array) $this->db->where('id', $id)->get($module == 'ads' ? 'mobile_sliders' : $module)->row();
			    } else {			        
			        $slider_groups = $this->db->select('id, name')->where('status', 1)->get('mobile_slider_groups')->result();
			        $row = array( // default values
			            'slider_group_id' => $slider_groups,
			            'max_distance' => 2,
			            'daily_budget' => 50,
			            'max_budget' => 500,
			            'start_date' => date('Y-m-d H:i:s')
			        );
			    }
			    $fields = $this->gcrud->get_module_fields($module, array('page' => 'form', 'model_folder' => 'mobiles', 'model' => 'mobiles', 'fields' => 'mobile_sliders'));
			    $extra_data = array(
			        'name' => $record->title,
			        'icon' => 'create',
			        'help' => lang('ad_heading_form'),
			        'toolbar_title' => lang('label_promote')
			    );
			    $fields = $this->mobiles_model->format_fields($fields);
			    if($id && $row) {
			        $row = $this->mobiles_model->format_data($fields, $row);
			    }
			    if(!empty($row['image'])) {
			        $row['image'] = gc_image_url($row['image'], true, true, array('module' => 'mobile_sliders'));
			    }
			    $record = array(
			        'success' => true,
			        'fields' => $fields,
			        'record' => $row,
			        'extra' => $extra_data
			    );
			    break;			    
			case 'files':
			    $module = $this->get('tm');
			    $row_id = (int) $this->get('row_id');
			    $fields = array();
			    $fields[] = array('name' => 'file', 'label' => lang('label_file'), 'type' => 'file', 'rules' => array(), 'params' => array('limit' => 2));
			    $record = array(
			        'success' => true,
			        'fields' => $this->mobiles_model->rest_fields($fields),
			        'record' => array(),
			        'extra' => array(
			            'name' => lang('file_documents'),
			            'icon' => 'create',
			            'help' => lang('file_click_to_add'),
			            // 'hasUploader' => true, // the form has a uploader field and it can only be handled using formdata
			            'toolbar_title' => lang('file_documents')
			        )
			    );
			    break;
			case 'business_hours': // Business Hours
			    if($user_id == 0) {
			        $this->response(array('success' => false), REST_Controller::HTTP_NOT_FOUND);
			    }
			    $row_id = (int) $this->get('row_id');
			    $record = $this->mobiles_model->business_hours($row_id, $user_id);
			    break;
			case 'packages':
			    if($user_id == 0) {
			        $this->response(array('success' => false), REST_Controller::HTTP_NOT_FOUND);
			    }
				// hide purchased packages from the user
			    $this->load->model('members/members_model');
			    // $packages = $this->members_model->get_packages('new', 0, $user_id);
			    $this->load->library('layouts');
			    $packages = $this->layouts->widget_packages(null, 0, null, null, array('mobile' => true, 'user_id' => $user_id, 'plan_type' => 'new')); 
			    $extra = array(
			        'toolbar_title' => lang('gc_menu_packages'),
			        'btn_text' => lang('btn_buy_now')
			    );
			    if(empty($packages)) {
			        $this->response(array('success' => false), REST_Controller::HTTP_NOT_FOUND);
			    }
			    $record = array('packages' => $packages, 'extra' => $extra);
			    break;
			case 'member':
			    if($user_id == 0) {
			        $this->response(array('success' => false), REST_Controller::HTTP_NOT_FOUND);
			    }
			    $record = $this->mobiles_model->member_area($user_id);
			    break;
			case 'dashboard':
			    if($user_id == 0) {
			        $this->response(array('success' => false), REST_Controller::HTTP_NOT_FOUND);
			    }
			    $toggle_module = escape($this->get('tm'));
			    $toggle_module = empty($toggle_module) ? 'listings' : $toggle_module;
			    $row_id = (int) $this->get('row_id');
			    $record = $this->mobiles_model->member_module($toggle_module, $row_id, $user_id);
			    break;
			case 'home':
			    $record = $this->mobiles_model->home($user_id);
			    break;
			case 'variant':
			    if($user_id == 0) {
			        $this->response(array('success' => false), REST_Controller::HTTP_NOT_FOUND);
			    }
			    $product_id = (int) $this->get('group_id');
			    $row_id = (int) $this->get('row_id');
			    $this->load->model('ecommerce/ecommerce_model');
			    $record = $this->ecommerce_model->variant_form($user_id, $product_id, $row_id, 'mobile');
			    break;
			default:
				$record = $this->db->where('id', $id)->get($module)->row();			
		}        
		
		// get popup to be displayed on different pages
		$record = (array) $record;
        $result = array(
            'record' => $record
        );
        $this->response($result, REST_Controller::HTTP_OK); // OK (200) being the HTTP response code
    }
    
    /**
     * Authenticate Username/Password
     */
    private function signin() {
        $this->load->library('users/auth');
        $this->load->library('events');
        $this->load->library('crypt');
        $_POST['login'] = $this->post('username');
        $_POST['password'] = $this->post('password');
        $_POST['log-me-in'] = true;
        $this->load->model('users/user_model');
        $result = $this->user_model->login(false, true);
        if($result['success']) {
            // Generate Token using base64_encode
            $user = $this->db->select('id, role_id, display_name, email, profile_image, phone_number')->where('id', $result['user_id'])->get('users')->row();
            // Send OTP to the user
            $verify_otp = (isset($result['verify_otp']) ? true : false);
            if($verify_otp) {
                $this->user_model->send_otp($user, false, true);
                $result = array(
                    'success' => false,
                    'verifyOtp' => true,
                    'otpParams' => array( // OTP Parameters to be used during verify_otp
                        'type' => 'login',
                        'token' => $this->crypt->encode($result['user_id']),
                        'hash' => $this->crypt->encode(json_encode(array($_POST['login'], $_POST['password'])))
                    )
                );
            } else {
                $result = $this->authenticate_response($result['user_id'], $user, false);
            }
        }
        return $result;
    }
    
    /**
     * Login Authentication
     * First username and password are verified,
     * if they are correct, the user is redirected to the OTP screen
     * If OTP is correct, again credentials are verified
     */
    private function authenticate() {
        $this->load->library('users/auth');
        $this->load->library('events');
        $this->load->library('crypt');
        $_POST['login'] = $this->post('username');
        $_POST['password'] = $this->post('password');
        $_POST['log-me-in'] = true;
        $this->load->model('users/user_model');
        $result = $this->user_model->login(false, true);
        if($result['success']) {
            // Generate Token using base64_encode
            $user = $this->db->select('id, role_id, display_name, email, profile_image, phone_number')->where('id', $result['user_id'])->get('users')->row();
            // Send OTP to the user
            $verify_otp = (isset($result['verify_otp']) ? true : false);
            if($verify_otp) {
                $this->user_model->send_otp($user, false, true);
                $result = array(
                    'success' => false,
                    'verifyOtp' => true,
                    'otpParams' => array( // OTP Parameters to be used during verify_otp
                        'type' => 'login',
                        'token' => $this->crypt->encode($result['user_id']),
                        'hash' => $this->crypt->encode(json_encode(array($_POST['login'], $_POST['password'])))
                    )
                );
            } else {
                $result = $this->authenticate_response($result['user_id'], $user, false);
            }
        }
        return $result;
    }
    
    /**
     * Verity OTP
     * @param array $result
     * @return array|boolean[]|string[][]
     */
    private function verify_otp($result, $post_data = array()) {
        $output = array();
        $user = $this->db->select('id, display_name, email, profile_image, phone_number')->where('id', $result['user_id'])->get('users')->row();
        // Send OTP to the user
        $verify_otp = (isset($result['verify_otp']) ? true : false);
        if($verify_otp) {
            $this->user_model->send_otp($user, false, true);
            $this->load->library('crypt');
            $token = $this->crypt->encode($user->id);
            $output = array(
                'success' => false,
                'verifyOtp' => true,
                'otpParams' => array( // OTP Parameters to be used during verify_otp
                    'type' => 'login',
                    'token' => $token
                )
            );
            
            // a new column, rest_hash is created in the users table to avoid passing user information between different API methods
            // this column stores the user entered login and password information in encrypted format
            // this parameter is used in the verify_otp method
            $rest_hash = array(
                'login' => $post_data['login'],
                'password' => $post_data['password']
            );
            $this->load->library('crypt');
            $rest_hash = $this->crypt->encode(json_encode($rest_hash));
            $this->db->where('id', $user->id)->update('users', array('rest_hash' => $rest_hash));
        }
        return $output;
    }
    
    /**
     * Common Authenticate Response for Login and Signup
     * @param number $user_id
     * @param string $email
     * @param boolean $verify_otp
     * @return array
     */
    private function authenticate_response($user_id, $user, $verify_otp) {
        $this->load->library('crypt');
        $token = $this->crypt->encode($user_id);
        // tabs menu
        $this->lang->load('members/members');
        $tabs_menu = array();
        switch($user->role_id) {
            case 2: // business owner
                // get the number of the listings user have
                $listings = $this->db->select('id')->where(array('user_id' => $user_id, 'deleted' => 0))->limit(2)->get('listings')->result_array();
                $tabs_menu = array(
                    array(
                        'label' => lang('bf_home'),
                        'url' => 'home',
                        'icon' => 'home'
                    ),
                    array(
                        'label' => lang('my_businesses'),
                        'url' => count($listings) == 1 ? 'member/dashboard/listings/' . $listings[0]['id'] : 'modules/listings/search/0/all/1',
                        'icon' => 'list-outline'
                    ),
                    array(
                        'label' => lang('more'),
                        'url' => 'panel',
                        'icon' => 'ellipsis-horizontal-outline'
                    )
                );
                break;
            case 4: // customer
                $tabs_menu = array(
                    array(
                        'label' => lang('bf_home'),
                        'url' => 'home',
                        'icon' => 'home'
                    ),
                    array(
                        'label' => lang('gc_menu_favorites'),
                        'url' => 'member/info-pages/favorites/listings',
                        'icon' => 'heart-outline'
                    ),
                    array(
                        'label' => lang('more'),
                        'url' => 'panel',
                        'icon' => 'ellipsis-horizontal-outline'
                    )
                );
                break;
        }
        $message = array(
            'success' => true,
            'localId' => (int) $user_id,
            'idToken' => $token,
            'name' => $user->display_name,
            'profileImage' => base_url('assets/images/' . (!empty($user->profile_image) ?  'users/' . $user->profile_image : 'dp.jpg')),
            'email' => $user->email,
            'phoneNumber' => $user->phone_number,
            'expiresIn' => 2147483, // https://stackoverflow.com/questions/3468607/why-does-settimeout-break-for-large-millisecond-delay-values
            'registered' => true,
            'verifyOtp' => $verify_otp,
            'roleId' => (int) $user->role_id,
            'tabsMenu' => $tabs_menu
        );
        return $message;
    }
    
    /**
     * Signup in Groups
     */
    private function signup() {
        $this->load->model('users/user_model');
        $this->load->model('fields/fields_model');
        $field_results = $this->fields_model->registration_fields_by_groups();
        $last_group_id = key(array_slice($field_results, -1, 1, true)); // last group
        $group_id = (int) $this->post('group');
        $form_fields = $field_results[$group_id]['fields'];
        $role = 'frontend';
        $module = 'registrations';
        $result = array();
        
        // Current/Previous/Next Group IDs
        $keys = array_keys($field_results);
        sort($keys); // sort array by value
        $next_group_id = $group_id != $last_group_id ? $keys[array_search($group_id, $keys) + 1] : $group_id;
        $previous_group_id = $group_id != 0 ? $keys[array_search($group_id, $keys) - 1] : 0;
        
        $_POST = $this->post();
        $_POST['save'] = true;
        
        // format selectable values
        foreach($_POST as $key => $value) {
            if($key == 'country_iso' && is_array($value)) { // country_iso cannot be integer
                $country_id = $value['id'];
                $country = $this->db->where('id', $country_id)->get('countries')->row();
                if($country) {
                    $_POST['country_iso'] = $country->iso;
                }
            } else if(is_array($value)) {
                $_POST[$key] = $value['id'];
            }
        }
        if(isset($_POST['save'])) {
            if($group_id == $last_group_id) {
                // in the last step validate all fields
                $form_fields = $this->fields_model->get_fields($module, $role);
            }
            if(empty($form_fields)) {
                return array('success' => false, 'message' => lang('us_register_disabled'));
            }
            foreach($form_fields as $fkey => $fvalue) {
                switch($fvalue['type']) {
                    case 'captcha':
                    case 'google_captcha':
                        unset($form_fields[$fkey]);
                        break;
                }
            }
            $validation_result = $this->fields_model->fields_validation_rules($form_fields, $module, $role);
            if($validation_result) {
                if($group_id != $last_group_id) {
                    $result = array('success' => false, 'next_group_id' => $next_group_id, 'previous_group_id' => $previous_group_id, 'last_group_id' => $last_group_id);
                } else {
                    $this->load->library('goclixy/gemailer');
                    $_POST['pass_confirm'] = $this->post('password');
                    $result = $this->user_model->register_mobile($form_fields);
                }
            } else {
                $validation_errors = validation_errors();
                $result = array('success' => false, 'message' => $validation_errors);
            }
        }
        return $result;
    }
    
	/**
	 * Load Fields for the Form
	 */
    public function fields_options() {
        $this->header_options();
    }
    public function fields_get() {
        $this->header_params();
        $this->load->model('fields/fields_model');
        $fields = array();
        $extra = array();
        $module = !empty($_GET['module']) ? escape($_GET['module']) : 'registrations';
        $user_id = $this->user_id();
        switch($module) {
            case 'otp':                
                $token = $this->get('token');
                $this->load->library('crypt');
                $phone_number = $this->crypt->decode($token);
                if(empty($phone_number)) {
                    $this->response(array('success' => false), REST_Controller::HTTP_NOT_FOUND);
                }
                $fields = $this->mobiles_model->rest_fields(array(
                    array('name' => 'otp', 'label' => lang('otp_placeholder'), 'type' => 'text', 'rules' => array('required' => 'required', 'minLength' => 6, 'maxLength' => 6), 'params' => array('type' => 'number')),
                ));
                $otp_limit = $this->user_model->otp_limit(0, array(), false, true, $phone_number);
                $extra = array(
                    'limit' => $otp_limit,
                    'resend_limit' => sprintf(lang('otp_resend_limit'), $otp_limit),
                    'resend_btn' => lang('otp_resend_btn'),
                    'resend_limit_exceeded' => lang('otp_resend_limit_exceeded'),
                    'retry_limit_exceeded' => lang('otp_retry_limit_exceeded'),
                    'otp_sent' => lang('otp_sent') . ' ' . $phone_number . '<br />' . lang('otp_help')
                );
                $fields = array('fields' => $fields, 'name' => lang('otp_verify_heading'), 'extra' => $extra);
                break;
            case 'login':
                $fields = $this->mobiles_model->rest_fields(array(
                    array('name' => 'username', 'label' => lang('bf_login_type_both'), 'type' => 'text', 'rules' => array('required' => 'required'), 'params' => array('type' => 'text')),
                    array('name' => 'password', 'label' => lang('bf_password'), 'type' => 'password', 'rules' => array('required' => 'required')),
                ));
				$extra = array('btn_text' => lang('login_now'), 'text_forgot_password' => lang('us_forgot_your_password'), 'text_signup' => lang('bf_action_register'));
                $fields = array('fields' => $fields, 'name' => lang('us_login'), 'extra' => $extra);
                break;
            case 'signup':
                $group_id = (int) $this->get('group');
                $fields = $this->fields_model->registration_fields_by_groups();
                $last_group_id = key(array_slice($fields, -1, 1, true)); // last group
                
				// Current/Previous/Next Group IDs
				$keys = array_keys($fields);
				$next_group_id = $group_id != $last_group_id ? $keys[array_search($group_id, $keys) + 1] : $group_id;
				$previous_group_id = $group_id != 0 ? $keys[array_search($group_id, $keys) - 1] : 0;
				
				$fields[0]['id'] = 0;
				$fields[0]['next_group_id'] = $next_group_id;
				$fields[0]['previous_group_id'] = $previous_group_id;
				$fields[0]['name'] = lang('us_sign_up');
				$fields[0]['extra'] = array('btn_text' => lang('gc_action_next'), 'text_login' => lang('bf_action_login'));
                foreach($fields as $key => $field_result) {					
					$next_group_id = $field_result['id'] != $last_group_id ? $keys[array_search($field_result['id'], $keys) + 1] : $field_result['id'];
					$previous_group_id = $field_result['id'] != 0 ? $keys[array_search($field_result['id'], $keys) - 1] : 0;
					$fields[$key]['next_group_id'] = $next_group_id;
					$fields[$key]['previous_group_id'] = $previous_group_id;
                    $btn_text = $field_result['id'] != $last_group_id ? lang('gc_action_next') : lang('bf_action_register');
					$fields[$key]['extra'] = array('btn_text' => $btn_text, 'text_login' => lang('bf_action_login'));
                    $fields[$key]['fields'] = $this->mobiles_model->format_fields($field_result['fields']);
                }
                $fields = !empty($group_id) ? $fields[$group_id] : $fields; // get group name also to be displayed in the title bar
                // $fields = $fields[0];//array('fields' => $fields[0], 'name' => lang('us_login'), 'extra' => $extra);
				break;
            case 'forgotPassword':
                $fields = $this->mobiles_model->rest_fields(array(
                    array('name' => 'email_phone', 'label' => lang('label_email'), 'type' => 'text', 'rules' => array('required' => 'required')),
                ));
                $fields = array('fields' => $fields, 'name' => lang('us_forgot_your_password'), 'extra' => array('reset_note' => lang('us_reset_note'), 'text_login' => lang('bf_action_login')));
                break;
            case 'resetPassword':
                $fields = $this->mobiles_model->rest_fields(array(
                    array('name' => 'password', 'label' => lang('bf_password'), 'type' => 'password', 'rules' => array('required' => 'required')),
                    array('name' => 'pass_confirm', 'label' => lang('bf_password_confirm'), 'type' => 'password', 'rules' => array('required' => 'required')),
                ));
                $fields = array('fields' => $fields, 'name' => lang('us_reset_password'), 'extra' => array('password_hints' => $this->user_model->password_hints(true)));
                break;
            case 'signupfields': // can be deleted
                $step = (int) $this->get('step');
                $step_id = $step == 2 ? 2 : 1;
                $field_results = $this->fields_model->registration_fields_by_steps();
                $records = $field_results['fields'][$step_id]['fields'];
                
                // display group name on a single page logic
                $field_groups = $field_results['groups'];
                $result_fields = array();
                $group_sequence = 100001;
                foreach($field_groups as $field_group) {
                    $result_fields[] = array(
                        'id' => $group_sequence + 1,
                        'type_id' => 0,
                        'name' => $field_group->name,
                        'type' => 'group',
                        'rules' => '',
                        'options' => '',
                        'label' => $field_group->name,
                        'label_lang' => $field_group->name_lang,
                        'required' => false
                    );
                    $fields_by_group = gc_form_fields($records, 'group', $field_group->id);
                    foreach($fields_by_group as $field_by_group) {
                        $result_fields[] = $field_by_group;
                    }
                }
                $records = $result_fields;
                // usort($records, array("mobiles", "order_by_group"));
                break;
            /*default:
                $records = $this->fields_model->get_fields($module, 'frontend');  */              
        }
        if(empty($fields) && empty($records)) {
            $this->response(array('success' => false, 'message' => lang('error_no_record_found')), REST_Controller::HTTP_NOT_FOUND);
        } else {
            $fields = empty($fields) && !empty($records) ? $this->mobiles_model->format_fields($records) : $fields;
            $this->response($fields, REST_Controller::HTTP_OK);
        }
    }    
    
    /**
     * Delete Data
     */
    public function purge_options() {
        $this->header_options();
    }
    public function purge_get() {
        $this->header_params();
        $module = escape($this->get('module'));
        $id = (int) $this->get('id');
        $user_id = $this->user_id();
        if(empty($id)) {
            $this->response(array('success' => false, 'message' => lang('msg_error')), REST_Controller::HTTP_NOT_FOUND);
        }
        $result = array();
        switch($module) {
            case 'notes':
                $this->lang->load('notes/notes');
                $profile = $this->mobiles_model->get_user_profile($user_id);
                if($profile) {
                    if($this->db->where(array('id' => $id, 'user_id' => $user_id))->delete($module)) {
                        $result = array('success' => true, 'message' => sprintf(lang('msg_action_success'), lang('singular_notes'), lang('label_deleted')));
                    }
                }
                break;
        }
        if(empty($result['success'])) {
            if(empty($result['message'])) {
                $result['message'] = lang('msg_error');
            }
            $this->response($result, REST_Controller::HTTP_NOT_FOUND);
        } else {
            $this->response($result, REST_Controller::HTTP_OK);
        }
    }
    
    /**
     * Order By Priority
     * @param string $a
     * @param string $b
     * @return number
     */
    public function order_by_group($a, $b) {
        return ($a["group"]<=$b["group"])?-1:1;
    }
    
    /**
     * Common Function - Form Data Post
     */
    public function data_options() {
        $this->header_options();
    }
    
    public function data_post() {
        $this->header_params();
        $module = escape($this->post('module'));
        $id = escape($this->post('id'));
        $user_id = (int) $this->user_id();
        $result = array();
        switch($module) {
            case 'push_token': // save push notification token in database
                $_POST = $this->post();
                $result = $this->mobiles_model->save_push_token($user_id);
                break;
            case 'player_id': // save one signal player id to send push notifications
                $_POST = $this->post();
                $result = $this->mobiles_model->save_player_id($user_id);
                break;
            case 'loading':
                $_POST = $this->post();
                $result = $this->mobiles_model->loading($user_id);
                break;
            case 'login':
                $result = $this->authenticate();
                break;
            case 'signup':
                $result = $this->signup();
                break;
            case 'forgot_password':
                $result = $this->forgot_password();
                break;
            case 'reset_password':
                $result = $this->reset_password();
                break;
            case 'sms_hash':
                $hash = escape($this->post('hash'));
                $this->db->where('name', 'site.sms_hash')->update('settings', array('value' => $hash));
                $result['success'] = true;
                break;
            case 'home':
                if($this->post('lat') && $this->post('lng') && is_float($this->post('lat'))) {
                    $_GET['lat'] = $this->post('lat');
                    $_GET['lng'] = $this->post('lng');
                }
                $result = $this->mobiles_model->home($user_id);
                break;                
            case 'profile_image':
                if(empty($user_id)) {
                    $this->response(array('success' => false), REST_Controller::HTTP_NOT_FOUND);
                }
                $profile_image = $this->post('profile_image');
                if(empty($profile_image)) {
                    $this->response(array('success' => false, 'message' => sprintf(lang('required'), lang('label_image'))), REST_Controller::HTTP_NOT_FOUND);
                }
                $image_name = $this->mobiles_model->save_image($profile_image, $user_id);
                if(!$image_name) {
                    $this->response(array('success' => false, 'message' => lang('msg_error')), REST_Controller::HTTP_NOT_FOUND);
                }
                $this->load->model('users/user_model');
                $this->load->model('images/images_model');  
                $user_profile = $this->user_model->get_user($user_id);
                $path = 'assets/images/users';
                if(!empty($user_profile->profile_image)) {
                    // delete existing image before uploading new one
                    $this->images_model->purge_image($user_profile->profile_image, $path);
                }
                // move image from temporary folder to users folder
                $this->images_model->move_resize($image_name, $path);
                // update image path
                $this->db->where('id', $user_id)->update('users', array('profile_image' => $image_name));
                $result = array('success' => true, 'message' => sprintf(lang('msg_action_success'), lang('label_profile_image'), lang('label_updated')), 'profile_image' => gc_image_url($image_name, true, true, array('module' => 'users'))); // base_url($path . '/' . $image_name)
                // Mobile App - update profile_image in the users session
                break;
            case 'change_password':
                $this->load->model('users/user_model');
                $_POST = $this->post();
                $_POST['save'] = 1;
                $result = $this->user_model->change_password($user_id, 'mobile');
                break;
            case 'notes_form':
                $this->load->model('notes/notes_model');
                $_POST = $this->post();
                $group_id = (int) $_POST['group_id'];
                $_POST['save'] = 1;
                $result = $this->notes_model->form($user_id, $group_id, 'mobile');
                break;
            case 'feedbacks':
            case 'callbacks':
                if(empty($user_id)) {
                    $this->response(array('success' => false), REST_Controller::HTTP_NOT_FOUND);
                }
                $this->load->model('listings/ajax_model');
                $fields = $this->mobiles_model->feedback_callback_query($module);
                $_POST = $this->mobiles_model->format_post_data($this->post(), $fields);
                $user_profile = $this->mobiles_model->get_user_profile($user_id, false);
                if($user_profile) {
                    $_POST['name'] = $user_profile->name;
                    $_POST['email'] = $user_profile->email;
                    if($module == 'feedbacks') {
                        $_POST['phone_number'] = $user_profile->phone_number;
                    }
                }
                if(!empty($_POST['country_iso'])) {
                    $country = $this->db->where('iso', $_POST['country_iso'])->get('countries')->row();
                    if($country) {
                        $_POST['country_code'] = '+' . $country->phone_code;
                        $_POST['country_iso'] = $country->iso;
                    }
                }
                $result = $this->ajax_model->feedbacks_callbacks($module, true, array('user_id' => $user_id));
                break;
            case 'contact_query':
                if(empty($user_id)) {
                    $this->response(array('success' => false), REST_Controller::HTTP_NOT_FOUND);
                }
                $_POST = $this->post();
                $user_profile = $this->mobiles_model->get_user_profile($user_id, false);
                if($user_profile) {
                    $_POST['name'] = $user_profile->name;
                    $_POST['email'] = $user_profile->email;
                    $_POST['phone_number'] = $user_profile->phone_number;
                }
                $this->load->model('contact_queries/contact_queries_model');
                $_POST['phone'] = $_POST['phone_number'];
                $result = $this->contact_queries_model->contact_query($user_id, true);
                break;
            case 'erroneous':
                if(empty($user_id)) {
                    $this->response(array('success' => false), REST_Controller::HTTP_NOT_FOUND);
                }
                $_POST = $this->post();            
                $user_profile = $this->mobiles_model->get_user_profile($user_id, false);
                if($user_profile) {
                    $_POST['name'] = $user_profile->name;
                    $_POST['email'] = $user_profile->email;
                    $_POST['phone_number'] = $user_profile->phone_number;
                }
                $this->load->model('erroneous/erroneous_model');    
                $toggle_module = escape($this->get('tm'));
                $row_id = (int) $this->get('row_id');
                $_POST['listing_id'] = (int) $_POST['row_id'];
                $_POST['type'] = $_POST['type_id'];
                $_POST['phone'] = $_POST['phone_number'];
                $result = $this->erroneous_model->report($user_id, true);
                break;
            case 'review':                
                if(empty($user_id)) {
                    $this->response(array('success' => false), REST_Controller::HTTP_NOT_FOUND);
                }
                $_POST = $this->post();
                $user_profile = $this->mobiles_model->get_user_profile($user_id, false);
                if($user_profile) {
                    $_POST['name'] = $user_profile->name;
                }
                $this->load->model('comments/comments_model');
                $is_edit = (int) $_POST['edit'];
                $id = 0;
                if($is_edit == 1) {
                    $id = (int) $_POST['group_id']; // group_id is used as the comment id
                } else {
                    $_POST['parent_id'] = (int) $_POST['group_id']; // group_id is used for the review parent id
                }
                $_POST['module'] = $_POST['tm'];
                $_POST['row_id'] = (int) $_POST['row_id'];
                $_POST['user_name'] = $_POST['name'];
                $result = $this->comments_model->save_review($user_id, true, $id);
                break;
            case 'rating':             
                // requred parameters: module, row_id and ratings
                $this->load->model('listings/ajax_model');
                $_POST = $this->post();
                $_POST['module'] = $_POST['tm'];
                $_POST['row_id'] = (int) $_POST['row_id'];
                $result = $this->ajax_model->rate_business($user_id, true);
                break;
            case 'favorite':
                // requred parameters: module, row_id and isFavorite
                $this->load->model('favorites/favorites_model');
                $_POST = $this->post();
                $_POST['module'] = $_POST['tm'];
                $_POST['row_id'] = (int) $_POST['row_id'];
                $result = $this->favorites_model->save_favorite($user_id, true);
                break;
            case 'click_to_call':
                switch($this->post('device')) {
                    case 'iPad':
                        $device_id = 2;
                        break;
                    case 'iPhone':
                        $device_id = 3;
                        break;
                    case 'Android':
                        $device_id = 4;
                        break;
                    case 'BlackBerry':
                        $device_id = 5;
                        break;
                    case 'webOS':
                        $device_id = 6;
                        break;
                    default:    
                        if($this->post('os') == 'android') {
                            $device_id = 4;
                        } else {
                            $device_id = 1;
                        }
                }
                $data['device'] = $device_id;
                $data['listing_id'] = $this->post('row_id');
                $data['ip_address'] = $this->input->ip_address();
                $data['created_on'] = date('Y-m-d H:i:s');
                $data['user_id'] = $user_id;
                $this->load->model('click_to_call/click_to_call_model');
                $sql = $this->db->insert_string($this->db->dbprefix .'click_to_call', $data) . ' ON DUPLICATE KEY UPDATE counts = counts + 1';
                $this->db->query($sql);
                $result['success'] = true;
                break;
            case 'business_query':
                // dynamic fields              
                if(empty($user_id)) {
                    $this->response(array('success' => false), REST_Controller::HTTP_NOT_FOUND);
                }
                $this->load->model('listings/ajax_model'); 
                $_POST = $this->post();
                $user_profile = $this->mobiles_model->get_user_profile($user_id, false);
                if($user_profile) {
                    $_POST['name'] = $user_profile->name;
                    $_POST['email'] = $user_profile->email;
                    $_POST['phone_number'] = $user_profile->phone_number;
                }
                $_POST['module'] = $_POST['tm'];
                $_POST['row_id'] = (int) $_POST['row_id'];
                $result = $this->ajax_model->send_query($user_id, true);
                break;
            case 'query_reply':
                $this->load->model('listings/ajax_model');
                $_POST = $this->post();
                $_POST['query_id'] = (int) $_POST['group_id'];
                $result = $this->ajax_model->reply_query($user_id, true);
                break;
            case 'post_requirements':
                if(empty($user_id)) {
                    $this->response(array('success' => false), REST_Controller::HTTP_NOT_FOUND);
                }
                // dynamic fields
				$this->lang->load('leads/leads');
                $this->load->model('leads/leads_model');
                $group_id = (int) $this->post('group_id');
                $_POST = $this->post();
                $group = $this->leads_model->valid_group($group_id);
                if(!$group) {
                    // not a valid group, redirect the user to the home page
                    $this->response(array('success' => false), REST_Controller::HTTP_NOT_FOUND);
                }
                $_POST['group_id'] = $group->id;
                $result = $this->leads_model->save_leads($user_id, true);
                break;
            case 'lead_form': // lead available form
                if(empty($user_id)) {
                    $this->response(array('success' => false), REST_Controller::HTTP_NOT_FOUND);
                }
                $this->load->model('leads/leads_model');
                $_POST = $this->post();
                $_POST['save'] = 1;
                $result = $this->leads_model->broadcast_update((int) $this->post('group_id'), $user_id, 'mobile', 2);
                break;
            case 'business_hours':
                $this->load->model('listings/business_hours_model');
                $_POST = $this->post();
                $_POST['module'] = $_POST['tm'];
                $_POST['listing_id'] = (int) $_POST['row_id'];
                $result = $this->business_hours_model->save_business_hours('insert', $_POST['listing_id'], true);
                break;
			case 'add':
			    $group_id = (int) $this->post('group_id');
			    // $group_id = $group_id == 0 ? 99999 : $group_id;
                $toggle_module = escape($this->post('tm'));
                $_POST = $this->post();
                $listing_id = (int) $this->post('row_id');
                $_POST['save'] = 1;
			    $result = $this->mobiles_model->add_module($group_id, $toggle_module, $user_id, $listing_id);
			    break;
            case 'groups':
                $group_id = (int) $this->post('group_id');
                $toggle_module = escape($this->post('tm'));
                if($toggle_module == 'users') {
                    $row_id = $user_id;
                } else {
                    $row_id = (int) $this->post('row_id');
                }
                $_POST = $this->post();
                $_POST['save'] = 1;
                // check whether the user is the owner of the listing or not
                $row = $this->db->where('id', $row_id)->get($toggle_module)->row();
                if($row) {
                    $result = $this->mobiles_model->group_form($toggle_module, $row_id, $group_id, 'mobile', $user_id);
                    // the user updated the profile, update the display name in the global web service to display new name during chat and in the side menu
                    if($toggle_module == 'users' && !empty($_POST['display_name'])) { 
                        $result['name'] = $_POST['display_name'];
                    }
                } else {
                    $result['success'] = false;
                    $result['extra'] = array('toolbar_title' => lang('edit') . ' ' . lang('singular_' . $module));
                }                
                break;
            case 'checkout_wallet':
                if(!$user_id) {
                    $this->response(false, REST_Controller::HTTP_NOT_FOUND);
                }
                $_POST = $this->post();
                $this->load->model('wallets/wallets_points_model');
                $result = $this->wallets_points_model->wallet_checkout('wallets', $user_id, true);
                break;                
            case 'checkout_address':
                $this->lang->load('orders/orders');
                $_POST = $this->post();                
                $form_fields = $this->fields_model->get_fields('orders', 'frontend'); // custom fields
                $fields = $this->mobiles_model->format_fields($form_fields);
                $data = $this->mobiles_model->format_post_data($_POST, $fields);
                $data['display_name'] = $data['name'];
                unset($data['name']);
                if(!empty($data['location'])) {
                    $city_id = $data['location'];
                    unset($data['location']);
                } else if(!empty($data['city_id'])) {
                    $city_id = $data['city_id'];
                }
                $this->load->model('locations/cities_model');                
                $city = $this->cities_model->get_city_info($city_id);
                if($city) {
                    $data['city_id'] = $city->id;
                    $data['state_id'] = $city->state_id;
                    $data['country_iso'] = $city->country_iso;
                    $data['city'] = $city->city;
                    $data['state'] = $city->state;
                    $data['country'] = $city->country;
                }
                $this->db->where('id', $user_id)->update('users', $data);
                return array('success' => true);
                break;
            case 'checkout':
                if(!$user_id) {
                    $this->response(false, REST_Controller::HTTP_NOT_FOUND);
                }
                // Retrieve package, if the price is zero, checkout the user in PHP itself.
                // if checkout is not successful, display the appropriate message to the user.
                $this->load->model('orders/orders_model');
                $is_save = !empty($this->post('save')) ? true : false;
                $checkout_module = $this->post('tm');  
                if(!$is_save) {
                    // only one item can be ordered at a time
                    $this->db->where('user_id', $user_id)->update('order_checkouts', array('params' => ''));
                    $row_id = (int) $this->post('row_id');                  
                    // $this->orders_model->cart(0, 0, 1, 1, 'wallets', true, array('price' => $this->post('amount')));
                    if($checkout_module == 'wallets') { // wallet checkout
                        $this->orders_model->cart($row_id, 0, 1, 0, $checkout_module, true, array('price' => $this->post('amount')), $user_id);
                    } else if($checkout_module == 'pyg') {
                        $params = $this->post('params');
                        $this->orders_model->cart($params['package_id'], 0, 1, 1, 'pyg', false, array('params' => $params), $user_id);
                    } else {
                        $this->orders_model->cart($row_id, 0, 1, 0, $checkout_module, true, array('zero_checkout' => true), $user_id);
                    }
                    $result = $this->orders_model->checkout_output($user_id, true);
                    // free package checkout, change users tabs menu on role upgrade
                    if($checkout_module == 'packages' && !empty($result['order_id'])) {
                        $user = $this->mobiles_model->get_user_profile($user_id, false);
                        $result['auth'] = $this->authenticate_response($user_id, $user, false);
                        clear_cache('links_' . $user_id);
                    }
                } else {
                    $payment_method = (int) $this->post('payment_method');
                    $_POST = $this->post();
                    $pay_using_wallet = !empty($_POST['pay_using_wallet']) ? 1 : false;
                    $user = $this->user_model->users_checkout($user_id);
                    $_POST = $user;
                    $_POST['pay_using_wallet'] = $pay_using_wallet;
                    $_POST['save'] = true;
                    $_POST['chk_terms'] = true;
                    $_POST['payment_method'] = $payment_method;
                    $result = $this->orders_model->checkout($user_id, 'mobile');
                    // change users tabs menu on role upgrade
                    if(!empty($result['auth'])) {
                        $user = $this->mobiles_model->get_user_profile($user_id, false);
                        $result['auth'] = $this->authenticate_response($user_id, $user, false);
                        clear_cache('links_' . $user_id);
                    }
                }
                break;
            case 'apply_coupon': // apply coupon
                $coupon = $this->post('coupon');                
                $this->load->model('coupons/coupons_model');
                $data = $this->coupons_model->apply_coupon($coupon, $user_id, true);
                if(empty($data['success'])) {
                    $result = $data;
                } else {
                    $result = $this->orders_model->checkout_output($user_id, true);
                }
                break;
            case 'clear_coupon':
                $this->load->model('orders/orders_model');
                $this->orders_model->checkout_params($user_id, false, array('clear' => 'coupon'));
                $this->orders_model->update_cart_items($user_id, true);                
                $result = $this->orders_model->checkout_output($user_id, true);
                break;
            case 'transaction_update': // order transaction id udpate
            case 'order_email': // email invoice to the user
                if($user_id == 0) {
                    $this->response(array('success' => false), REST_Controller::HTTP_NOT_FOUND);
                }
                $this->load->model('orders/orders_model');
                $order_id = (int) $this->post('group_id');
                $_POST = $this->post();
                $_POST['save'] = 1;
                if($module == 'transaction_update') {
                    $result = $this->orders_model->transaction_update($order_id, $user_id, 'mobile');
                } else {
                    $result = $this->orders_model->order_email($order_id, $user_id, 'mobile');                    
                }
                break;
            case 'authenticate':                
                $this->load->model('users/user_model');
                $phone_number = $this->post('username');
                $result = $this->signin(); // user_model->otp_phone_number($phone_number, 'mobile');  
                break;
            case 'otp':
                $this->load->model('users/user_model');
                // get phone number from the token
                $token = $this->post('token');                
                $this->load->library('crypt');
                $phone_number = $this->crypt->decode($token);
                $otp = (int) $this->post('otp');
                $success = $this->user_model->verify_phone_number($phone_number, $otp);
                if($success) {
                    $user = $this->user_model->phone_login_signup($phone_number);
                    $result = $this->authenticate_response($user->id, $user, false);
                } else {
                    $result = array('success' => false, 'message' => lang('otp_invalid'));
                }        
                break;
            case 'resend_otp':
                $this->load->model('users/user_model');
                // get phone number from the token
                $token = $this->post('token');
                $this->load->library('crypt');
                $phone_number = $this->crypt->decode($token);
                if(empty($phone_number)) {
                    $this->response(NULL, REST_Controller::HTTP_BAD_REQUEST);
                } else {
                    $result = $this->user_model->resend_otp($user_id, false, true, $phone_number);
                    if(!empty($result['limit'])) {
                        $result['resend_limit'] = sprintf(lang('otp_resend_limit'), $result['limit']);
                    } else {
                        $result['resend_limit'] = lang('otp_retry_limit_exceeded');
                    }
                }
                break;
            case 'gallery': // allow user to upload module images
                $tm = escape($this->post('tm'));
                $row_id = (int) $this->post('row_id');
                $images = $this->post('images');
                $row = $this->db->where('id', $row_id)->get($tm)->row();
                if(!$row) {
                    $this->response(array('success' => false), REST_Controller::HTTP_NOT_FOUND);
                }
                
                // get package
                $this->load->model('packages/packages_model');
                $package = $this->packages_model->get_package($row->package_id);
                $limit = 1;
                $status = 0;
                if($package) {
                    $limit = json_decode($package->module_images_limit ?? '', true);
                    $limit = !empty($limit[$tm]) ? $limit[$tm] : 1;
                    $images_status = json_decode($package->module_images_status ?? '', true);
                    $status = !empty($images_status[$tm]) ? 1 : 0;
                }
                
                $module_id = gc_module_id($tm);
                $where = array('module_id' => $module_id, 'row_id' => $row_id);
                $this->load->model('images/images_model');
                if(empty($images)) { // the user deleted all the images
                    $delete_where = $where;
                    $delete_where['type_id'] = 0;
                    $this->db->where($delete_where)->delete('images');
                } else {
                    $saved_images = array();
                    foreach($images as $image) {
                        if(!empty($image['id'])) {
                            $saved_images[] = $image['id'];
                        } else if(!empty($image['original'])) {
                            // save image
                            $filename = $this->mobiles_model->save_image($image['original'], $user_id);
                            if(!empty($filename)) {
                                $insert_id = $this->images_model->save_image($tm, $row_id, $filename, $status, 0, $limit);
                                $saved_images[] = $insert_id;
                            }
                        } 
                    }
                }
                // delete all other images except saved ones
                if(!empty($saved_images)) {
                    $where = 'module_id = ' . $module_id . ' AND row_id = ' . $row_id . ' AND type_id = 0 AND id NOT IN (' . implode(',', $saved_images) . ')';
                    $deleted_images = $this->db->where($where)->get('images')->result();
                    if($deleted_images) {
                        foreach($deleted_images as $deleted_image) {
                            $this->images_model->purge_image($deleted_image->filename, $deleted_image->path);
                            $this->db->where('id', $deleted_image->id)->delete('images');
                        }
                    }
                }
                $this->images_model->_tweak_images_row($tm, $row_id);
                clear_cache($tm . '_' . $row_id); // clear cache
                $result = array('success' => true, 'message' => sprintf(lang('msg_save_success'), lang('gc_menu_images')));
                break;
            case 'image':
                $this->load->model('images/images_model');
                $list_images = true;
                $action_id = (int) $this->post('action_id'); // 0 - Save, 1 - List, 2 - Purge
                $type_id = (int) $this->post('type_id'); // 0 - images, 1 - cover image, 2 - logo, 3 - documents
                if($action_id == 2) {
                    if($type_id == 1 || $type_id == 2) {
                        // get image id
                        $row_id = (int) $this->post('row_id');                        
                        $toggle_module = $this->post('tm');
                        $toggle_module_id = gc_module_id($toggle_module);
                    } else {
                        $image_id = (int) $this->post('image');
                        // get module and row_id details from the images table
                        $image = $this->db->where('id', $image_id)->get('images')->row();
                        if(!$image) {
                            $this->response(array('success' => false), REST_Controller::HTTP_NOT_FOUND);
                            return false;
                        }
                        $toggle_module = gc_module_name($image->module_id);
                        $toggle_module_id = $image->module_id;
                        $row_id = $image->row_id;
                    }
                } else {
                    $toggle_module = $this->post('tm');
                    $toggle_module_id = gc_module_id($toggle_module);
                    $row_id = (int) $this->post('row_id');                    
                }                
                $record = $this->db->where('id', $row_id)->get($toggle_module)->row();
                if(!$record && $record->user_id != $user_id) { //  || $record->user_id != $user_id
                    // don't have permission to view images
                    $this->response(array('success' => false), REST_Controller::HTTP_NOT_FOUND);
                    return false;
                }
                if($type_id > 2) {
                    // don't have permission to view images
                    $this->response(array('success' => false), REST_Controller::HTTP_NOT_FOUND);
                    return false;
                }
                $db_table = $type_id == 3 ? 'files' : 'images';
                // note: in mobile app, the user can upload documents in the image format only
                if($action_id != 2) {                    
                    // get limit from packages
                    $package = $this->db->where('id', $record->package_id)->get('packages')->row();     
                    if($type_id == 0) {
                        $limit = json_decode($package->module_images_limit ?? '', true);
                        $limit = !empty($limit[$toggle_module]) ? $limit[$toggle_module] : 1;
                    }
                }
                switch($action_id) {
                    case 0:                        
                        // save image in the temp folder
                        $image_data = $this->post('image');
                        $filename = $this->mobiles_model->save_image($image_data, $user_id);
                        if($filename) {
                            $images_status = json_decode($package->module_images_status ?? '', true);
                            $status = !empty($images_status[$toggle_module]) ? 1 : 0;
                            if($type_id == 1 || $type_id == 2) { // logo and cover images
                                $limit = 1;
                                $this->images_model->purge_images($toggle_module, $row_id, $type_id);
                            }
                            $this->images_model->save_image($toggle_module, $row_id, $filename, $status, $type_id, $limit, true);
                        }
                        $result = array('success' => true, 'message' => sprintf(lang('msg_save_success'), ($type_id == 1 ? lang('label_cover_image') : lang('logo'))));
                        break;
                    case 2: // purge image
                        switch($type_id) {
                            case 1: // cover
                            case 2: // logo
                                $this->load->model('images/images_model');
                                $this->images_model->purge_images($toggle_module, $row_id, $type_id);
                                $result['success'] = true;
                                break;
                            default:
                                $image_id = $this->post('image_id'); // image to be purged
                                // pass id in case of gallery/album
                                $result['success'] = $this->images_model->purge_image_by_id($image_id);
                                break;
                        }
                        break;
                    default: // list images
                        $result['success'] = true;
                        
                }
                $toolbar_title = lang('gc_menu_images');
                switch($type_id) {
                    case 1:
                        $toolbar_title = lang('label_cover_image');
                        break;
                    case 2:
                        $toolbar_title = lang('label_logo');
                        break;
                    case 3:
                        $toolbar_title = lang('label_documents');
                        break;
                }
                if($list_images) {
                    // List Images
                    // the images will be displayed after save and delete
                    $where = array('module_id' => $toggle_module_id, 'row_id' => $row_id);
                    if($db_table == 'images') {
                        $where['type_id'] = $type_id;
                    }
                    $rows = $this->db->where($where)->get($db_table)->result();
                    if($rows) {
                        foreach($rows as $row) {
                            $file_url = array(
                                'original' => gc_image_url($row->path . '/' . $row->filename, false),
                                'thumb' => gc_image_url($row->path . '/' . $row->filename),
                                'title' => $row->title,
                                'id' => $row->id,
                                'status' => $row->status
                            );
                            $result['images'][] = $file_url;
                        }
                    }
                }
                $result['extra'] = array('toolbar_title' => $toolbar_title);
                if($action_id != 2 && $type_id == 0) {
                    // $total_images = $result['images'];
                    $result['extra']['limit_message'] = sprintf(lang('images_limit'), $limit);
                }
                break;
            case 'update_status':
                if(!$user_id) {
                    // not a valid group, redirect the user to the home page
                    $this->response(array('success' => false), REST_Controller::HTTP_NOT_FOUND);
                }
                $toggle_module = $this->post('tm');
                switch($toggle_module) {
                    case 'ads':
                        $toggle_module = 'mobile_sliders';
                        break;
                    case 'variants':
                        $toggle_module = 'product_variants';
                        break;
                }
                $row_id = (int) $this->post('row_id');
                $status = (int) $this->post('status');
                if($toggle_module == 'mobile_sliders' || $toggle_module == 'ads') {
                    $result = $this->mobiles_model->update_ad_status($row_id, $user_id);
                    if(empty($result['success'])) {
                        $this->db->where('id', $row_id)->update($toggle_module, array('status' => 0));
                        $this->response($result, REST_Controller::HTTP_NOT_FOUND);
                        return false;
                    }
                    $status = $result['status'];
                }
                $this->db->where('id', $row_id)->update($toggle_module, array('status' => $status));
                $result['success'] = true;
                break;
            case 'delete':
                $_POST = $this->post();
                $module = $this->input->post('tm');
                $row_id = $this->input->post('row_id');
                switch($module) {
                    case 'view_comments':
                    case 'my_comments':
                        $label = lang('label_comment');
                        $this->load->model('comments/comments_model');
                        $success = $this->comments_model->delete_hierarchy($row_id);
                        break;
                    case 'view_business_queries':
                    case 'posted_queries':
                        $label = lang('label_business_query');
                        $this->load->model('business_queries/business_queries_model');
                        $success = $this->business_queries_model->delete($row_id);
                        break;
                    case 'favorites':
                        $label = lang('label_favorite');
                        $this->load->model('favorites/favorites_model');
                        $success = $this->favorites_model->delete($row_id);
                        break;
                    case 'variants':
                        $this->lang->load('ecommerce/ecommerce');
                        $label = lang('label_variant');
                        $this->load->model('ecommerce/ecommerce_model');
                        $this->ecommerce_model->_purge_variants(array($row_id));
                        $success = true;
                        break;
                    case 'my_requirements':
                        $this->lang->load('leads/leads');
                        $label = lang('lead');
                        $this->load->model('leads/leads_model');
                        $this->leads_model->_purge(array($row_id));
                        $success = true;
                        break;
                    case 'leads': // delete broadcast
                        $this->lang->load('leads/leads');
                        $label = lang('lead');
                        $this->load->model('leads/leads_model');
                        $this->leads_model->_purge_broadcast(array($row_id));
                        $success = true;
                        break;
                    default:
                        $this->lang->load('listings/listings');
                        if(in_array($module, gc_modules())) {
                            $result = $this->listings_model->delete_soft($module, $row_id, 'mobile');
                            $success = $result['success'];
                            $label = lang('singular_' . $module);
                        } else {
                            // check whether the user is the owner of the module or not
                            // $this->valid_owner($user_id, $row_id, $module);
                            $label = $module == 'ads' ? lang('label_ad') : lang('singular_' . $module);
                            $toggle_module = $module == 'ads' ? 'mobile_sliders' : $module;
                            $success = $this->db->where('id', $row_id)->delete($toggle_module);
                        }
                        break;
                }                
                $result['success'] = true;
                if ($success) {
                    $result['message'] = sprintf(lang ( 'msg_delete_success' ), $label);
                } else {
                    $result['message'] = sprintf(lang ( 'msg_delete_failure' ), $label);
                }
                break;
            case 'device':                
                $platform = $this->post('platform');
                $uuid = $this->post('uuid');
                $model = $this->post('model');
                $version = $this->post('version');
                $manufacturer = $this->post('manufacturer');
                $push = $this->post('push');
                $token = $this->post('token');
                
                if($uuid == NULL) {
                    $this->response(NULL, REST_Controller::HTTP_BAD_REQUEST);
                }
                $data = array(
                    'platform' => $platform,
                    'model' => $model,
                    'version' => $version,
                    'manufacturer' => $manufacturer,
                    'token' => $token
                );
                // $language = settings_item('site.default_language');
                // $result = $this->db->select('language, push')->get_where('mobile_devices', array('uuid' => $uuid))->row();
                if($result) {
                    // The user might have no account on website, but he/she has selected a default language on mobile device
                    // $language = $result->language;
                    // $this->session->set_userdata('device_language', $language);
                    $data['modified_on'] = date('Y-m-d H:i:s');
                    $data['modified_ip'] = $this->input->ip_address();
                    $this->db->where('uuid', $uuid)->update('mobile_devices', $data);
                } else {
                    $data['uuid'] = $uuid;
                    // $data['language'] = $language; // Set the Default Language
                    $data['created_on'] = date('Y-m-d H:i:s');
                    $data['created_ip'] = $this->input->ip_address();
                    $this->db->insert('mobile_devices', $data);
                }
                $result['success'] = true;
                break;
            case 'ads':
                $_POST = $this->post();
                $toggle_module = escape($this->input->post('tm'));
                $row_id = (int) $this->input->post('row_id');
                $id = $this->input->post('group_id');
                $this->valid_owner($user_id, $row_id, $toggle_module);
                $extra_fields = array();                
                if($id == 0) {
                    $column_id = $module == 'ads' ? 'ad_row_id' : 'offer_row_id';
                    $extra_fields = array(
                        'module_id' => gc_module_id($toggle_module),
                        $column_id => $row_id
                    );
                }
                switch($module) {
                    case 'ads':
                        $params = array('page' => 'form', 'model_folder' => 'mobiles', 'model' => 'mobiles', 'fields' => 'mobile_sliders', 'extra_fields' => $extra_fields, 'method' => 'after_ad_save');
                        break;
                }
                $result = $this->gcrud->save_form('mobile_sliders', $user_id, $params, $id, $id == 0 ? 'insert' : 'update');
                break;
          case 'paytm_generate_hash': // Cordova Plugin - Generate Hash
                $order_id = $this->post('order_id'); // "015897134666882"
                $type = $this->post('type');
                $this->load->model('payment_gateways/payment_gateways_model');
                $result = $this->payment_gateways_model->paytm_generate_checksumhash($order_id, $type);
                break;
            case 'cashfree_generate_hash': // Cordova Plugin - Generate Hash
                $order_id = $this->post('order_id'); // "015897134666882"
                $type = $this->post('type');
                $this->load->model('payment_gateways/payment_gateways_model');
                $result = $this->payment_gateways_model->cashfree_checksum($order_id, $type);
                break;
            case 'transaction_status': // Cordova Plugin - Checkout Transaction Status
                $_POST = $this->post();
                $order_id = escape($this->post('order_id'));
                $response = $this->post('response'); // transaction response
                $type = $this->post('type');
                $payer_id = null;
                $transaction_id = !empty($response['TXNID']) ? $response['TXNID'] : null;
                if(is_numeric($order_id)) {
                    $order_info = $this->db->where('id', $order_id)->get('orders')->row();
                } else {
                    $order_info = $this->db->where('order_id', $order_id)->get('orders')->row();
                }
                if(!$order_info) {
                    $this->response(array('success' => false), REST_Controller::HTTP_NOT_FOUND);
                }
                $this->load->model('payment_gateways/payment_gateways_model');
                if(!empty($response['STATUS']) && $response['STATUS'] == 'TXN_SUCCESS') {
                    $result = $this->payment_gateways_model->transaction_confirmed($order_info, $transaction_id, $payer_id, $response, true);
                } else {
                    $result = $this->payment_gateways_model->transaction_cancelled($order_info, $transaction_id, $payer_id, $response, true);
                }
                break;
            case 'paytm_validate_checksum':
                $_POST = $this->post();
                $this->load->model('payment_gateways/payment_gateways_model');
                $result = $this->payment_gateways_model->paytm_validate_checksum();                
                break;
            case 'removal':
                if($user_id == 0) {
                    $this->response(array('success' => false), REST_Controller::HTTP_NOT_FOUND);
                }
                $this->load->model('listings/listings_model');
                $row_id = (int) $this->post('row_id');                
                $_POST = $this->post();
                $_POST['save'] = 1;
                $result = $this->listings_model->removal_form('listings', $row_id, $user_id, 'mobile');
                break;
            case 'variant':
                if($user_id == 0) {
                    $this->response(array('success' => false), REST_Controller::HTTP_NOT_FOUND);
                }
                $product_id = (int) $this->post('group_id');
                $row_id = (int) $this->post('row_id');
                $_POST = $this->post();
                $_POST['save'] = 1;
                $this->load->model('ecommerce/ecommerce_model');
                $result = $this->ecommerce_model->variant_form($user_id, $product_id, $row_id, 'mobile');
                break;
        }
        // common extra data
        switch($module) {
            case 'checkout':
            case 'apply_coupon':
            case 'clear_coupon':
                $extra = array(
                    'subtotal' => lang('invoice_subtotal'),
                    'discount' => lang('label_discount'),
                    'tax' => lang('invoice_tax'),
                    'total' => lang('invoice_total'),
                    'submit' => lang('gc_action_submit'),
                    'apply_coupon' => lang('label_apply_coupon'),
                    'clear_coupon' => lang('label_clear_coupon'),
                    'btn_text' => lang('label_checkout'),
                    'toolbar_title' => lang('label_checkout')
                );
                $result['extra'] = !empty($result['extra']) ? array_merge($extra, $result['extra']) : $extra;
                break;
        }
        if(empty($result['success'])) {
            $this->response($result, REST_Controller::HTTP_NOT_FOUND);
        } else {   
            if(empty($result['extra'])) {
                $result['extra'] = array();
            }
            $this->response($result, REST_Controller::HTTP_OK);    
        }
    }
    
    /**
     * Allows a user to start the process of resetting their password.
     * An email is allowed with a special temporary link that is only valid
     * for 24 hours. This link takes them to reset_password().
     *
     * @access public
     *
     * @return void
     */
    private function forgot_password() {
        $this->lang->load('users/users', $this->language);
        $this->load->model('users/user_model');
        $this->load->library('users/auth');
        $this->load->library('events');
        $this->load->library('crypt');
        $this->load->library('goclixy/gemailer');
        $_POST['email_phone'] = $this->post('email_phone');
        $_POST['send'] = true;
        $result = $this->user_model->forgot_password(false, true);
        // Send OTP to the user
        $verify_otp = isset($result['verify_otp']) ? true : false;
        // Send OTP to the user
        if($result && $verify_otp) {
            unset($result['message']);
            $this->user_model->send_otp($result['user'], false, true); // send otp
            $user = (object) $result['user']; //$this->user_model->get_user($user_registered->user_id);
            // Send OTP to the User, it is already sent in the users module
            // $this->user_model->send_otp($user, false, true);
            // OTP Parameters to be used during verify_otp
            // Save these parameters in local memory and pass to the verify_otp method
            $result['otpParams'] = array(
                'type' => 'reset_password',
                'token' => $this->crypt->encode($user->id),
                'hash' => $result['hash'], // required in case of OTP verification
            );
            $result['success'] = false;
        }
        if($result['success']) {
            $token = $this->crypt->encode($result['user']->id);
            $result['otpParams'] = array(
                'token' => $token,
                'hash' => $result['hash'], // required in case of OTP verification
                'verifyOtp' => $verify_otp,
                'message' => isset($result['message']) ? $result['message'] : '',
                'success' => true
            );
        }
        $result['type'] = 'reset_password';
        unset($result['user']);
        return $result;
    }
    
    /**
     * Get User ID
     * @return number
     */
    private function user_id($token = null) {
        $user_id = 0;
        if(empty($token)) {
            $headers = $this->input->request_headers();
            if(!empty($headers['Token'])) {
                $token = $headers['Token'];
            }
        }
        if(!empty($token)) {
            $this->load->library('crypt');
            $user_id = (int) $this->crypt->decode($token);
        }
        return $user_id;
    }
    
    /**
     * Check whether the user is the owner of a specific business or not
     * @param number $listing_id
     */
    private function valid_owner($user_id, $row_id, $module = 'listings') {
        $record = $this->db->where('id', $row_id)->get($module)->row();
        if(!$record || $record->user_id != $user_id) {
            $this->response(array('success' => false), REST_Controller::HTTP_NOT_FOUND);
        }
        return true;
    }
	
	public function header_options() {
		$this->output->set_header('Access-Control-Allow-Origin: *');
		$this->output->set_header('Access-Control-Allow-Headers: Content-Type, Authorization, X-Requested-With, Token');
		$this->output->set_header('Access-Control-Allow-Methods: GET, POST, OPTIONS, PUT, DELETE');
        $this->set_response(REST_Controller::HTTP_CREATED);
	}
	
	public function header_params() {
		$this->output->set_header('Access-Control-Allow-Origin: *');
		$this->output->set_header('Access-Control-Allow-Headers: Content-Type, Authorization, X-Requested-With, Token');
		$this->output->set_header('Access-Control-Allow-Methods: GET, POST, OPTIONS, PUT');
        $this->set_response(REST_Controller::HTTP_CREATED);
	}
	
	public function validate_options() {
		$this->output->set_header('Access-Control-Allow-Origin: *');
		$this->output->set_header('Access-Control-Allow-Headers: Content-Type, Authorization, X-Requested-With, Token');
        $this->set_response(REST_Controller::HTTP_CREATED);
	}
	
	public function validate_get() {
		$this->load->helpers(array('string', 'security'));
		$challenge = do_hash(random_string('alnum', 40) . time());
		// save this challenge in a session for authrization
		$this->session->set_userdata('api_challenge', $challenge);
		// send challenge to the application
		$this->output->set_header('Access-Control-Allow-Origin: *');
		$this->output->set_header('Access-Control-Allow-Headers: Content-Type, Authorization, X-Requested-With, Token');
		$this->output->set_header('Access-Control-Expose-Headers: Content-Type, Token');
		$this->output->set_header('Token: ' . $challenge);
        $this->set_response(REST_Controller::HTTP_CREATED);
		$this->response(null, REST_Controller::HTTP_OK);
	}
	
	/**
	 * BASIC Authentication
	 */
	private function basic_authentication() {
		$reqHeaders = $this->input->request_headers();
		// if the session exists return true
		if($this->session->userdata('api_token') && !empty($reqHeaders['Token']) && $reqHeaders['Token'] == $this->session->userdata('api_token')) {
			return true;
		} else if(!empty($reqHeaders['Token'])) {	
			// Decode Token
			$authorization = base64_decode($reqHeaders['Token']); 
			// compare challenge and password with server credentials
			$settings = json_decode(settings_item('mobile.settings'));
			if($authorization == $settings->api_password) {
				// create a new session for this connection
				// compare this session for the future connections
				$this->session->set_userdata('api_token', $reqHeaders['Token']);
				return true;
			} else {
				return false;
			}
		} else {
			return false;
		}
	}
	
	/**
	 * Advanced Authentication Mechanism
	 */
	private function advanced_authentication() {
		// ISSUE WITH THIS Method
		// $this->session->userdata('api_challenge') is being deleted on each call
		// Token must be stored in a file or database instead of session
		$reqHeaders = $this->input->request_headers();
		// if the session exists return true
		if($this->session->userdata('api_token') && !empty($reqHeaders['Token']) && $reqHeaders['Token'] == $this->session->userdata('api_token')) {
			return true;
		} else if(!empty($reqHeaders['Token'])) {				
			// Decode Token
			$authorization = base64_decode($reqHeaders['Token']); 
			// Explode token to get the token and password string
			$authdata = explode(':' , $authorization ?? '');
			// compare challenge and password with server credentials
			$settings = json_decode(settings_item('mobile.settings'));
			$api_password = base64_decode($authdata[1]);
			if($this->session->userdata('api_challenge') == $authdata[0] && $api_password == $settings->api_password) {
				// create a new session for this connection
				// compare this session for the future connections
				$this->session->set_userdata('api_token', $reqHeaders['Token']);
				return true;
			} else {
				return false;
			}
		} else {
			return false;
		}
	}
}