<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Migration_Install_mobiles extends Migration
{

	/**
	 * Permissions to Migrate
	 *
	 * @var Array
	 */
	private $permission_values = array(
		array(
			'name' => 'Mobiles.Components.View',
			'description' => 'Allow Users to View Mobile Settings',
			'status' => 'active',
		),
		array(
			'name' => 'Mobiles.Components.Edit',
			'description' => 'Allow Users to Edit Existing Settings',
			'status' => 'active',
		),
		array(
			'name' => 'Mobiles.Components.Delete',
			'description' => 'Allow Users to Delete Existing Push Devices Information',
			'status' => 'active',
		)
	);

	/**
	 * The name of the permissions table
	 *
	 * @var String
	 */
	private $table_name = 'permissions';

	/**
	 * The name of the role/permissions ref table
	 *
	 * @var String
	 */
	private $roles_table = 'role_permissions';
	
	/**
	 * The mobile devices table's fields
	 *
	 * @var Array
	 */
	private $mobile_devices_fields = array(
		'id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'auto_increment' => TRUE
		),
		'platform' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE
		),
		'uuid' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => FALSE
		),
		'model' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE
		),
		'version' => array(
			'type' => 'VARCHAR',
			'constraint' => 100,
			'null' => TRUE
		),
		'manufacturer' => array(
			'type' => 'VARCHAR',
			'constraint' => 100,
			'null' => TRUE
		),
		'token' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => FALSE
		),
		'push' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'default' => 1
		),
		'language' => array(
			'type' => 'VARCHAR',
			'constraint' => 50,
			'null' => TRUE
		),
		'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'modified_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'created_ip' => array(
			'type' => 'VARCHAR',
			'constraint' => 50,
			'null' => TRUE
		),
		'modified_ip' => array(
			'type' => 'VARCHAR',
			'constraint' => 50,
			'null' => TRUE
		),
		'status' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'default' => 1
		),
	);
	
	/**
	 * The mobile notifications table's fields
	 *
	 * @var Array
	 */
	private $mobile_notifications_fields = array(
		'id' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'auto_increment' => TRUE
		),
		'device_id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'default' => 0
		),
		'title' => array(
			'type' => 'VARCHAR',
			'constraint' => 100,
			'null' => FALSE
		),
		'message' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => FALSE,
		),
		'type' => array(
			'type' => 'CHAR',
			'constraint' => 50,
			'null' => FALSE,
		),
		'response' => array(
			'type' => 'TEXT',
			'null' => TRUE,
		),
		'scheduled_on' => array(
			'type' => 'DATETIME',
			'null' => TRUE
		),
		'date_sent' => array(
			'type' => 'DATETIME',
			'null' => TRUE
		),
		'status' => array(
			'type' => 'TINYINT',
			'constraint' => 2,
			'default' => 0
		),
		'broadcast' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'default' => 0
		),
		'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'modified_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'ip_address' => array(
			'type' => 'VARCHAR',
			'constraint' => 50,
			'null' => TRUE
		),
	);
	
	/**
	 * The mobile slider table's fields
	 *
	 * @var Array
	 */
	private $mobile_sliders_fields = array(
		'id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'auto_increment' => TRUE
		),
		'title' => array(
			'type' => 'VARCHAR',
			'constraint' => 100,
			'null' => FALSE
		),
		'image' => array(
			'type' => 'VARCHAR',
			'constraint' => 100,
			'null' => TRUE
		),
		'module_id' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default' => 0
		),
		'row_id' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'null' => FALSE,
			'default' => 0
		),
		'row_title' => array(
			'type' => 'VARCHAR',
			'constraint' => 150,
			'null' => TRUE
		),
		'external_url' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE
		),
		'start_date' => array(
			'type' => 'DATE',
			'null' => TRUE
		),
		'end_date' => array(
			'type' => 'DATE',
			'null' => TRUE
		),
		'max_clicks' => array(
			'type' => 'SMALLINT',
			'constraint' => 5,
			'null' => FALSE,
			'default'=> 0
		),
		'clicks' => array(
			'type' => 'SMALLINT',
			'constraint' => 5,
			'null' => FALSE,
			'default'=> 0
		),
		'status' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default'=> 0
		),
		'priority' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => FALSE,
			'default'=> 0
		),
		'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'modified_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
	);
	//--------------------------------------------------------------------

	/**
	 * Install this migration
	 *
	 * @return void
	 */
	public function up()
	{
		// Install Permissions
		$role_permissions_data = array();
		foreach ($this->permission_values as $permission_value)
		{
			$this->db->insert($this->table_name, $permission_value);

			$role_permissions_data[] = array(
				'role_id' => '1',
				'permission_id' => $this->db->insert_id(),
			);
		}

		$this->db->insert_batch($this->roles_table, $role_permissions_data);
		
		// Install Settings Information
		$this->db->insert('settings', array('name' => 'mobile.components', 'module' => 'mobile', 'value' => '{"search":{"postcode":1,"locality":1}}'));
		$this->db->insert('settings', array('name' => 'mobile.api_access_key', 'module' => 'mobile', 'value' => 'demo'));
		
		// Create Mobile Devices Table
		$this->dbforge->add_field($this->mobile_devices_fields);
		$this->dbforge->add_key('id', true);
		// $this->dbforge->add_key('uuid', false);
		$this->dbforge->add_key('status', false);
		$this->dbforge->create_table('mobile_devices');
		// ADD UNIQUE INDEX TO MOBILE DEVICES TABLE ON DEVICE ID
		$unique_device_sql = "ALTER TABLE " . $this->db->dbprefix . "mobile_devices ADD UNIQUE INDEX `device` (`uuid`)";
		$this->db->query ($unique_device_sql);
		
		// Create Mobile Notifications Table
		$this->dbforge->add_field($this->mobile_notifications_fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('device_id', false);
		$this->dbforge->add_key('message', false);
		$this->dbforge->add_key('scheduled_on', false);
		$this->dbforge->add_key('status', false);
		$this->dbforge->create_table('mobile_notifications');	
		
		// Create Mobile Slider Table
		$this->dbforge->add_field($this->mobile_sliders_fields);
		$this->dbforge->add_key('id', true);
		//$this->dbforge->add_key('module_id', false);
		$this->dbforge->create_table('mobile_sliders');	
		//$unique_index_sql = "ALTER TABLE " . $this->db->dbprefix . "mobile_sliders ADD UNIQUE INDEX `module` (`module_id`, `row_id`)";
		//$this->db->query ($unique_index_sql);
	}

	//--------------------------------------------------------------------

	/**
	 * Uninstall this migration
	 *
	 * @return void
	 */
	public function down()
	{
		// Remove Permissions
		foreach ($this->permission_values as $permission_value)
		{
			$query = $this->db->select('permission_id')
				->get_where($this->table_name, array('name' => $permission_value['name'],));

			foreach ($query->result() as $row)
			{
				$this->db->delete($this->roles_table, array('permission_id' => $row->permission_id));
			}

			$this->db->delete($this->table_name, array('name' => $permission_value['name']));
		}
		
		// Remove Settings Information
		$settings = $this->db->select('name')->get_where('settings', array('name' => 'mobile.components'));
		if($settings) {
			$this->db->delete('settings', array('name' => 'mobile.components'));
		}
		$keys = $this->db->select('name')->get_where('settings', array('name' => 'mobile.api_access_key'));
		if($keys) {
			$this->db->delete('settings', array('name' => 'mobile.api_access_key'));
		}
		
		// Drop Mobile Devices Table
		$this->dbforge->drop_table('mobile_devices');
		
		// Drop Mobile Notifications Table
		$this->dbforge->drop_table('mobile_notifications');		
		
		// Drop Mobile Slider Table
		$this->dbforge->drop_table('mobile_sliders');
	}

	//--------------------------------------------------------------------

}