<?php defined('BASEPATH') || exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Settings controller
 */
class Settings extends Admin_Controller {
    protected $permissionCreate = 'Options.Settings.Create';
    protected $permissionDelete = 'Options.Settings.Delete';
    protected $permissionEdit   = 'Options.Settings.Edit';
    protected $permissionView   = 'Options.Settings.View';
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
        
        $this->auth->restrict($this->permissionView);
        $this->load->model('options/options_model');
        $this->lang->load('options');        
      	$this->form_validation->set_error_delimiters("<span class='error'>", "</span>");        
        Template::set_block('sub_nav', 'settings/_sub_nav');        
        Assets::add_module_js('options', 'options.js');
        // clear options cache
        clear_cache('options');
    }
    
    /**
     * Display a list of options data.
     *
     * @return void
     */
    public function index($filter = 'all', $offset = 0) {
       previousUrl(true);
       $this->gcrud->index('options', $offset, 'admin', array(
       		'context' => 'settings',
       		'filter' => $filter,
       		'filter_letters' => true,
       		'filter_tabs' => $this->options_model->option_types(0),
       		'filter_field' => 'type_id',
       		'filter_dropdown' => true,
       		'footer_actions' => array('activate', 'deactivate'),
            'search_fields' => array('M.id', 'M.name'),
			'query_method' => 'get_all_crud',
			'query_extra' => array('module' => 'options'),
            'uri_segment' => 6,
            'table_shorthand' => 'M',
            'soft_delete' => true,
       		'actions' => array(
       			array('type' => 'button', 'class' => 'btn-warning', 'name' => 'copy', 'label' => lang('label_copy')),
       		)
       ));
    }
    
    /**
     * Create a options object.
     *
     * @return void
     */
    public function create() {
		$params = array('context' => 'settings', 'redirect_url' => previousUrl());
		$params['method'] = 'after_option_save';
        $this->gcrud->form('options', 'create', 'admin', $params);
    }
    /**
     * Allows editing of options data.
     *
     * @return void
     */
    public function edit() {
		$params = array('context' => 'settings', 'uri_segment' => 5, 'redirect_url' => previousUrl());
		$params['method'] = 'after_option_save';
        $this->gcrud->form('options', 'edit', 'admin', $params);
    }
    
    /**
     * Allow admin to change order of options
     */
    public function order () {
        $this->lang->load('fields/fields');
        
        //LOAD NESTED SORTABLE JAVASCRIPT AND CSS FILE
        Assets::add_css('nestedSortable/admin.css');
        Assets::add_js('nestedSortable/jquery-ui.min.js');
        Assets::add_js('nestedSortable/jquery.mjs.nestedSortable.js');
        
        Assets::add_js($this->load->view('settings/order_options_js', null, true), 'inline');
        Template::set('sortable', TRUE);
        Template::set('toolbar_title', lang('order_options'));
        Template::render();
    }
    
    /**
     * Order options using AJAX nestedSortable
     */
    public function order_ajax () {
        $type_id = (int) $this->input->post('type_id');
        // Save order from ajax call
        if (isset($_POST['sortable'])) {
            $this->options_model->save_order($_POST['sortable'], $type_id);
        }
        if($type_id != 0) {
            // Load view
            $records = $this->db->select('id, name, name_lang')->where(array('type_id' => $type_id))->order_by('priority')->get('options')->result_array();
            if($records) {
                foreach($records as $key => $record) {
                    $records[$key]['name'] = !empty($record['name_lang']) ? lang($record['name']) : $record['name'];
                }
            }
            Template::set('result', $records);
        }
        Template::set('toolbar_title', lang('order_options'));
        Template::render();
    }
	
	/**
	 * Manage Option Types
	 * @param string $module
	 * @param string $filter
	 * @param number $offset
	 */
	public function manage($module = 'types', $filter = 'all', $offset = 0) {
	    $extra = array(
	        'model' => 'options',
	        'context' => 'settings',
			'prefix' => 'option',
	        'soft_delete' => true,
	        'fields' => $module,
	    );
	    $extra['actions'] = array( // Extra buttons
	        array('type' => 'link', 'link' => site_url(SITE_AREA . '/settings/options/form/' . $module . '/edit/%s'), 'label' => '<i class="fas fa-edit fa-fw mr-1"></i>' . lang('edit'), 'class' => 'btn btn-xs btn-primary'),
	    );
	    $this->gc->module_view($module, $filter, $offset, 'admin', $extra);
	}
	
	/**
	 * Add/Edit Options Data
	 *
	 * @param string $module
	 * @param string $action
	 *
	 * @return void
	 */
	public function form($module = 'types', $action = 'create') {
	    // clear_cache('types');
	    /*Assets::add_js(array(
	        'tinymce/tinymce.min.js',
	    ));*/
	    $params = array(
	        'model' => 'options',
	        'context' => 'settings',
			'prefix' => 'option',
	        'fields_method' => $module,
	        'redirect_url' => site_url(SITE_AREA . '/settings/options/manage/' . $module)
	    );
	    if($module == 'types') {
	        $params['method'] = 'after_option_type_save';
	    }
	    $this->gcrud->form($module, $action, 'admin', $params);
	}
}