<?php defined('BASEPATH') || exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class Options_model extends BF_Model
{
    public $table_name	= 'options';
    protected $table_types	= 'option_types';
	protected $key			= 'id';
	protected $date_format	= 'datetime';

	protected $log_user 	= false;
	protected $set_created	= false;
	protected $set_modified = false;
	public $soft_deletes	= false;


	// Customize the operations of the model without recreating the insert,
    // update, etc. methods by adding the method names to act as callbacks here.
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 	    = array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	// For performance reasons, you may require your model to NOT return the id
	// of the last inserted row as it is a bit of a slow method. This is
    // primarily helpful when running big loops over data.
	protected $return_insert_id = true;

	// The default type for returned row data.
	protected $return_type = 'object';

	// Items that are always removed from data prior to inserts or updates.
	protected $protected_attributes = array();

	// You may need to move certain rules (like required) into the
	// $insert_validation_rules array and out of the standard validation array.
	// That way it is only required during inserts, not updates which may only
	// be updating a portion of the data.
	protected $validation_rules 		= array();
	protected $insert_validation_rules  = array();
	protected $skip_validation 			= false;
	
	protected $gc_options_cache;

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
        // Load Zend Caching
        $this->load->library('slug');
        $this->load->library('gzf_cache', array('lifetime' => 900));
        if(method_exists($this->gzf_cache, 'get_instance')) {
            $this->gc_options_cache = $this->gzf_cache->get_instance();
        }
    }
	
	/**
	 * Get all option types
	 * @param boolean $lang_string
	 */
	public function option_types($lang_string = 1) {
		$cacheID = 'gc_option_types_' . $lang_string;
	    if (! $data = $this->gc_options_cache->load($cacheID)) {
    		$records = $this->db->get_where($this->table_types, array('status' => 1))->result();
    		$data = array();
        	if($records) {
        		foreach($records as $record) {
        			$data[$record->id] = !empty($record->name_lang) && $lang_string == 1 ? lang($record->name) : $record->name;
        		}
        	}
        	$this->gc_options_cache->save($data, $cacheID, array('option_types'));
	    }
        return $data;
	}
    
    /**
     * Get Fields
     * @return string[][]|boolean[][]|number[][]
     */
    public function fields() {
    	$nav_options = array();
    	foreach($this->option_types(0) as $key => $nav_tab) {
    		$nav_options[] = 'name:' . lang('tab_' . $nav_tab) . ';value:' . $key;
    	}
    	/*$religion_options[] = 'name:none;value:0';
    	$religions = $this->db->where('type_id', 1)->get('options')->result();
    	if($religions) {
    		foreach($religions as $religion) {
    			$religion_options[] = 'name:' . ($religion->name_lang == 1 ? lang($religion->name) : $religion->name) . ';value:' . $religion->id;
    		}
    	}*/
    	$custom_fields = array(
    	    array(
    	        'name' => 'key',
    	        'type' => 'text',
    	        'label' => lang('label_key')
    	    ),
    	    array(
    	        'name' => 'value',
    	        'type' => 'text',
    	        'label' => lang('label_value')
    	    )
    	);
    	Template::set('custom_fields', $custom_fields);
    	$fields = array(
    			array(
    				'type' => 'id',
    				'name' => 'id',
    				'frontend_index' => true,
    				'admin_index' => true
    			),
    			array(
    				'type' => 'dropdown',
    				'name' => 'type_id',
    				'label' => 'label_type_id',
    				'admin_only' => true,
    				'admin_index' => true,
    				'options' => implode('|', $nav_options),
    				'options_detailed' => 1,
    				'options_lang' => 0,
    				'rules' => 'required|trim',
    				'class' => '3'
    			),
    			array(
    				'type' => 'dropdown',
    				'name' => 'parent_id',
    				'label' => 'label_parent',
    				'admin_only' => true,
					'options' => 'name:options;value:database',
					'options_detailed' => 1,
    				'options_lang' => 0,
    				'rules' => 'trim|is_numeric',
    				'class' => '3'
    			),
    			array(
    				'type' => 'text',
    				'name' => 'name',
    				'label' => 'label_name',
    				'admin_only' => true,
    				'admin_index' => true,
    				'field_extra' => 'index_edit_admin:1',
    				'rules' => 'required|trim|max_length[100]',
    			),
        	    array(
        	        'type' => 'iconpicker',
        	        'name' => 'icon',
        	        'label' => 'label_icon',
        	        'frontend' => true,
        	        'admin_only' => true,
        	        'frontend_index' => false,
        	        'admin_index' => false,
        	        'rules' => 'trim'
        	    ),
				array(
					'type' => 'filemanager',
					'name' => 'image',
					'label' => 'label_image',
					'frontend' => true,
					'admin_only' => true,
					'frontend_index' => true,
					'admin_index' => true,
					'rules' => 'trim'
				),
				array(
					'type' => 'textarea',
					'name' => 'description',
					'label' => 'label_description',
					'frontend' => true,
					'admin_only' => true,
					'frontend_index' => true,
					'admin_index' => true,
					'custom_rules' => 'editor',
					'field_class' => 'editor',
					'class' => 9,
					'rules' => 'trim'
				),
        	    array(
        	        'type' => 'custom',
        	        'name' => 'params',
        	        'label' => 'label_params',
        	        'frontend' => true,
        	        'admin_only' => true,
        	        'frontend_index' => false,
        	        'admin_index' => false,
        	        'field_extra' => 'options:custom_fields',
        	        'class' => 9,
        	        'rules' => 'trim',
        	    ),
        	    array(
        	        'type' => 'text',
        	        'name' => 'priority',
        	        'label' => 'label_priority',
        	        'admin_only' => true,
        	        'admin_index' => true,
        	        'rules' => 'required|trim',
        	        'value' => 0,
        	        'class' => '2'
        	    ),
    			array(
    			    'type' => 'checkbox',
    				'name' => 'status',
    				'label' => 'label_status',
    				'admin_only' => true,
    				'admin_index' => true,
    				'options' => 'name_lang,status',
    				'options_lang' => 1,
    				'field_extra' => 'index_action_admin:1',
    				'rules' => 'trim|max_length[1]',
    			),
    	);
    	return $fields;
    }
	
	/**
     * Option Types Fields
     *
     * @return array
     */
    public function types() {
		$fields = array(
            array(
                'type' => 'id',
            	'name' => 'id',
            	'frontend_index' => true,
            	'admin_index' => true
            ),
			array(
				'type' => 'dropdown',
				'name' => 'parent_id',
				'label' => 'label_parent',
				'admin_only' => true,
				'options' => 'name:option_types;value:database',
				'options_detailed' => 1,
				'options_lang' => 0,
				'rules' => 'trim|is_numeric',
				'class' => '3'
			),
            array(
                'type' => 'text',
                'name' => 'name',
                'label' => 'label_name',
    			'admin_only' => true,
    			'admin_index' => true,
    			'field_extra' => 'index_edit_admin:1',
    			'rules' => 'required|trim|max_length[100]',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'name_lang',
                'label' => 'label_name_lang',
                'frontend' => false,
                'admin_only' => false,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'priority',
                'label' => 'label_priority',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|required|is_numeric',
				'value' => 0,
				'class' => 2
            ),
            array(
                'type' => 'checkbox',
                'name' => 'status',
                'label' => 'label_status',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            )            
         );
         return $fields;
    }
    
    /**
     * Build Options Tree for Profile Form
     * @param boolean $is_translate - TRUE - return the translated value, FALSE -  return actual name (Bulk Import Options)
     * @param string $where
     * @return mixed
     */
    public function options_tree($data = array()) {
        $key = !empty($data['key']) ? $data['key'] : 'type_id';
        $cacheID = 'options_tree_' . json_encode($data) . '_' . $key;
        $cacheID = str_replace('-', '_', $this->slug->create_slug($cacheID));
        if (! $result = $this->gc_options_cache->load($cacheID)) {
            if(!empty($data['where'])) {
                $where = $data['where'];
            }
            if(empty($data['all'])) {
                $where = (!empty($where) ? $where . ' AND ' : '') . 'status = 1 AND deleted = 0';
            }
            $records = $this->db->order_by(empty($data['order_by']) ? 'priority' : $data['order_by'])->get_where($this->table_name, $where)->result();
            $result = array();
            if($records) {
                // Build options tree
                foreach($records as $record) {
                    $option_data = array(
                        'id' => $record->id
                    );
                    if(empty($data['no_translate'])) {
                        $option_name = $record->name_lang == 1 ? lang($record->name) : $record->name;
                    } else {
                        $option_name = $record->name;
                    }
                    $option_data['name'] = $option_name;
                    $option_data['icon'] = !empty($record->icon) ? $record->icon : '';
                    $option_data['image'] = array(
                        'thumb' => gc_image_url(!empty($record->image) ? 'assets/images/' . $record->image : $record->image),
                        'original' => gc_image_url(!empty($record->image) ? 'assets/images/' . $record->image : $record->image, false)
                    );
                    $option_data['description'] = $record->description;
                    if(!empty($record->params)) {
                        $option_data['params'] = json_decode($record->params ?? '', true);
                    }
                    if($key == 'type_id') {
                        $result[$record->type_id][] = $option_data;
                    } else {
                        $result[$record->id] = $option_data;
                    }
                    //$result[$record->type_id][$record->id] = $record->name_lang == 1 ? lang($record->name) : $record->name;
                }
            }
            $this->gc_options_cache->save($result, $cacheID, array('options'));
        }
        return $result;
    }
    
    /**
     * Save multi options in the user_options table. e.g. checkboxes, multiselect
     * @param number $user_id
     */
    public function save_user_options($user_id) {
    	// delete all old values first
    	$this->db->where('user_id', $user_id)->delete('user_options');
    	$data = array();
    	foreach($this->option_types() as $type_id => $nav_tab) {
    		$options = $this->input->post($nav_tab);
    		if(!empty($options) && is_array($options)) {
    			foreach($options as $option) {
	    			$data[] = array(
	    				'user_id' => $user_id,
	    				'type_id' => $type_id,
	    				'option_id' => $option
	    			);
    			}
    		}
    	}
    	if(!empty($data)) {
    		$this->db->insert_batch('user_options', $data);
    	}
    }
	
	/**
	 * Get all options to display values on the Profile Detail Page
	 */
	public function get_options() {
	    $cacheID = 'options';
	    if (! $data = $this->gc_options_cache->load($cacheID)) {
    		$records = $this->db->get('options')->result();
    		$data = array();
        	if($records) {
        		// Build options tree
        		foreach($records as $record) {
        			$data[$record->id] = array($record->name, $record->name_lang);
    			// do not save hardcoded language strings in database, else multi-language will not work
    			// $record->name_lang == 1 ? lang($record->name) : $record->name;
        		}
        	}
        	$this->gc_options_cache->save($data, $cacheID, array('options'));
	    }
        return $data;
	}
	
	/**
	 * Save order only when user clicks on save button
	 * @param array $options
	 * @param number $type_id - Optional Option Type
	 */
	public function save_order($options, $type_id = 0) {
	    if (count($options)) {
	        foreach ($options as $order => $option) {
	            if (!empty($option['id'])) {
	                $data = array('priority' => $order);
	                $this->db->where(array('id' => $option['id']))->update($this->table_name, $data);
	            }
	        }
	    }
	}
	
	/**
	 * Height Filter
	 * @param number $min_height
	 * @param number $max_height
	 * @param array $options
	 */
	public function height_filter($min_height, $max_height, $option_values = array()) {
	    $cacheID = 'height_filter_' . $min_height . '_' . $max_height;
	    if (! $ids = $this->gc_options_cache->load($cacheID)) {
	        if(empty($option_values)) {
    	       $option_values = $this->get_options();	
	        }
    	    $min_height = $option_values[$min_height][0];
    	    $max_height = $option_values[$max_height][0];
    	    
    	    // get options
    	    $sql = "SELECT GROUP_CONCAT(id) AS ids FROM " . $this->db->dbprefix . "options WHERE type_id = 4 AND (name BETWEEN " . $min_height . " AND " . $max_height . ")";
    	    $result = $this->db->query($sql)->row();
    	    if($result) {
    	        $ids = $result->ids;
    	    }
    	    $ids = array('ids' => $ids);
    	    $this->gc_options_cache->save($ids, $cacheID, array('options'));
	    }
	    return $ids['ids'];
	}
	
	/**
	 * Method called in Gcrud library after option insert/update
	 * @param number $id
	 * @param string $action_type
	 * @param string $record
	 */
	public function after_option_save($id, $action_type = 'insert', $record = array()) {	    
	    $this->load->library('goclixy/gclosure');
		$gclosure = new Gclosure('options', 'option_hierarchies');
		$record = $this->db->where('id', $id)->get('options')->row();
		if($record) {
			$parent_id = $record->parent_id;
			$gclosure->delete($id, false); // do not delete from the categories table		        
			$gclosure->add($id, ($parent_id == 0 ? $id : $parent_id)); // save hierarchy
			$label = !empty($record->name_lang) ? lang($record->name) : $record->name;
			$data = array('summary' => $label);
			if(!empty($record->name_lang)) {
			    $result = $this->insert_update_language_string($record, 'options');
			    $data['name'] = $result;
			}
			$this->db->where('id', $id)->update($this->table_name, $data);
		}
	}
	
	/**
	 * Method called in Gcrud library after option insert/update
	 * @param number $id
	 * @param string $action_type
	 * @param string $record
	 */
	public function after_option_type_save($id, $action_type = 'insert', $record = array()) {
	    $record = $this->db->where('id', $id)->get('option_types')->row();
	    if($record) {
	        $result = $this->insert_update_language_string($record, 'option_types');
	        if(!empty($result)) {
	            $this->db->where('id', $id)->update('option_types', array('name' => $result));
	        }	        
	        clear_cache('option_types');
	    }
	}
	
	/**
	 * Get option types
	 * @param string $key
	 * @param string $value
	 */
	public function type_options($key, $value) {
		$result = array();
		$row = $this->db->where($key, $value)->get($this->table_types)->row();
		if($row) {
			$where = array('type_id' => $row->type_id, 'status' => 1);
			$records = $this->db->order_by('priority')->get_where($this->table_name, $where)->result();
			$options = array();
			if($records) {
				foreach($records as $record) {
					$options[$record->id] = !empty($record->name_lang) ? lang($record->name) : $record->name;
				}
			}
			$result = array(
				'id' => $row->id,
				'name' => !empty($row->name_lang) ? lang($row->name) : $row->name,
				'options' => $options
			);
		}
		return $result;
	}
	
	/**
	 * Save new language string in the options_lang.php file
	 * @param string $record
	 * @return mixed
	 */
	public function insert_update_language_string($record, $table = 'option_types') {
	    if($table == 'options_types' || !empty($record->name_lang)) {
    	    $this->load->model('languages/languages_model');    	    
    	    $name = $record->name;
    	    $name = str_replace('-', '_', $this->slug->create_slug($name));
    	    $value = ucwords(str_replace('_', ' ', $name));
    	    if($table == 'option_types') {
    	        $keys = array($name, 'tab_' . $name);
    	    } else {
    	        $key = strpos($name, 'option_') !== false ? $name : 'option_' . $name;
    	        $name = $key;
    	        $keys = array($key);
    	    }
    	    foreach($keys as $key) {
    	       $this->languages_model->insert_lang_string($key, $value, 'english', array('options_lang.php'));
    	    }
    	    return $name;
	    }
	}
}