<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/

/**
 * Financial controller
 */
class Financial extends Admin_Controller {
	protected $permissionCreate = 'Orders.Financial.Create';
	protected $permissionDelete = 'Orders.Financial.Delete';
	protected $permissionEdit   = 'Orders.Financial.Edit';
	protected $permissionView   = 'Orders.Financial.View';
	protected $permissionManage = 'Orders.Financial.Manage';

	//--------------------------------------------------------------------


	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct() {
		parent::__construct();

		$this->auth->restrict($this->permissionView);
		$this->load->model('orders_model', null, true);
		$this->lang->load('orders');		
		Template::set_block('sub_nav', 'financial/_sub_nav');
		
		//DATE TIME PICKER
		Assets::add_js('moment.js');
		Assets::add_css ( 'bootstrap-datetimepicker.min.css' );
		Assets::add_js ( 'bootstrap-datetimepicker.min.js' );

		//Assets::add_module_js('orders', 'orders.js');
	}

	//--------------------------------------------------------------------

	/**
	 * Display Orders Summary to the User
	 */
	public function summary() {
	    $this->auth->restrict('Orders.Financial.Summary');
	    // DATE TIME PICKER
	    Assets::add_js('moment.js');
	    Assets::add_css('bootstrap-datetimepicker.min.css' );
	    Assets::add_js('bootstrap-datetimepicker.min.js' );    
	    Assets::add_css('chart.css');
	    Assets::add_js('Chart.min.js');
	    $this->session->unset_userdata('chart_query_params');
	    $record = $this->orders_model->summary();
	    $tabular_blocks = array();
	    if($this->auth->role_id() == 1) {
	        $column = 'order_total';
	        $table = 'orders';
	        $where = '';
	    } else { // display items specific sales to the vendor
	        $column = 'total';
	        $table = 'order_items';
	        $where = "seller_id = " . $this->auth->user_id();
	    }
	    $sales_by_months = $this->gcrud->chart('months', array('status_id' => 2, 'where' => $where,'prefix' => 'M', 'table' => $table, 'field' => $column));
	    Assets::add_js($this->load->view('common/_chart_js', array('field_id' => 'sixMonthSales', 'labels' => $sales_by_months['labels'], 'data' => $sales_by_months['data'], 'total' => $sales_by_months['total'], 'label' => lang('total_sales'), 'with_dates' => false, 'table' => $table), true), 'inline');
	    $result = $this->gcrud->chart('current_month', array('status_id' => 2, 'where' => $where, 'prefix' => 'M', 'table' => $table, 'field' => $column));
	    Assets::add_js($this->load->view('common/_chart_js', array('field_id' => 'monthlyChart', 'labels' => $result['labels'], 'data' => $result['data'], 'total' => $result['total'], 'label' => lang('total_sales'), 'with_dates' => true, 'table' => $table), true), 'inline');
	    $record->tabular_blocks = $tabular_blocks;
	    //$record->recently_viewed = $this->forums_model->manage(0, null, 'M.last_viewed DESC', 20, 0, array('module' => 'views'));
	    Template::set(array(
	        'toolbar_title' => lang('gc_menu_orders') . ' - ' . lang('gc_menu_summary'),
	        'record' => $record
	    ));
	    Template::render();
	}
	
	/**
	 * View Orders
	 * @param string $filter
	 * @param number $offset
	 */
	public function index($filter = 'all', $offset = 0) {
	    $this->auth->restrict('Orders.Financial.Manage');
	    $this->manage('orders', $filter, $offset);
	}
	
	/**
	 * Display a list of Orders data
	 * @param string $module
	 * @param string $filter
	 * @param number $offset
	 */
	public function manage($module = 'orders', $filter = 'all', $offset = 0) {
	    if($module == 'orders') {	        
	        $this->auth->restrict('Orders.Financial.Manage');
	    }
	    // Regenerate Invoice
	    if(isset($_POST['regenerate_invoice'])) {
	        $this->gcrud->stats(array('module' => 'orders'));
	        $order = $this->db->where('id', $_POST['regenerate_invoice'])->get('orders')->row();
	        if($order) {
	            if($this->orders_model->generate_invoice($order->order_id, $order)) {
	                Template::set_message(sprintf(lang('generate_invoice_success'), $order->order_id), 'success');
	            } else {
	                Template::set_message(lang('generate_invoice_error'), 'danger');
	            }
	        }
	    }
	    
	    // Generate Order PDF
	    if(isset($_POST['order_pdf'])) {
	        $this->gcrud->stats(array('module' => 'orders'));
	        $order = $this->orders_model->get_order($_POST['order_pdf'], 'id');
	        if($order) {
	            $this->orders_model->email_print($order->order_id, 'pdf', array('order' => $order));
	        }
	    }
	    
	    // Refund wallet payment
		if (isset($_POST['refund_wallet'])) {
		    $this->gcrud->stats(array('module' => 'orders'));
		    $order_item_id = $_POST['refund_wallet'];
		    $order_item_info = $this->db->where('id', $order_item_id)->get('order_items')->row();
		    if($order_item_info && empty($order_item_info->wallets_refund)) {
		        // get order information
		        $order_where = "order_id = '" . $order_item_info->order_id . "' AND wallets != 0";
		        $order = $this->db->where($order_where)->get('orders')->row();
		        if($order) {
        			// get wallet balance
        			$wallet_balance = $this->db->select('wallets AS cash')->get_where('users', array('id' => $order->user_id))->row();
        			if($wallet_balance) {
        			    // update order_items
        			    $this->db->where('id', $order_item_id)->update('order_items', array('wallets_refund' => date('Y-m-d H:i:s'), 'status_id' => 5));
        			    
						// update order
        			    $this->db->where('id', $order->id)->update('orders', array('wallets' => ($order->wallets - $order_item_info->total)));
        				
        			    // update order status based on the current status of all ordered items
        			    $this->load->model('orders/ordered_items_model');
        			    $this->ordered_items_model->update_order_item_status($order_item_info->id, $order_item_info);

						// deduct the amount from the users wallet	        
        				$this->load->model('wallets/wallets_points_model');
        				$this->wallets_points_model->update_cash('wallets', $order_item_info->user_id, 'orders', $order_item_info->total, 'credit', null, $order_item_info->id, array('order_id' => $order_item_info->order_id));
        				Template::set_message(lang('refund_success'), 'success');
        				Template::redirect(current_url());
        			} else {
        				Template::set_message(lang('refund_error'), 'danger');
        			}
		        }
		    }
		}

	    $extra = array(
	        'model' => 'orders',
	        'context' => 'financial',
	        'fields' => $module
	    );
	    if($module == 'orders') {
	        // order_statuses table
	        $extra['table_prefix'] = 'order';
	    } else {
	        if($module == 'invoices' || $module == 'customs') {
	            // order_statuses table
	            $extra['table_prefix'] = 'order';	            
	        }
	        $extra['prefix'] = 'order';
	        if($module == 'hierarchies' || $module == 'coupons') {
	            $extra['actions'] = array( // Extra buttons
	                array('type' => 'link', 'link' => site_url(SITE_AREA . '/financial/commissions/form/' . $module . '/edit/%s'), 'label' => '<i class="fas fa-edit fa-fw mr-1"></i>' . lang('edit'), 'class' => 'btn btn-xs btn-primary'),
	            );
	        }
	        if($module == 'items' || $module == 'cancellations') {
	            $this->load->model('orders/ordered_items_model');
	            $extra['view'] = 'orders/manage';
	        }
	    }
	    if($module == 'items') {
	        $extra['table_prefix'] = 'order';
	    }	    
	    switch($module) {
	        case 'items':
	        case 'cancellations':
	            if($this->auth->role_id() != 1) {
	                $extra['query_where'] = 'M.seller_id = ' . $this->auth->user_id();
	            }
	            if($module == 'cancellations' && $this->auth->role_id() == 1) {
	               Template::set('extra_action', '<a href="' . site_url(SITE_AREA . '/financial/orders/manage/cancellation_reasons') . '" class="btn btn-primary btn-sm ml-2" data-toggle="tooltip" data-title="' . lang('order_cancellation_reasons') . '"><i class="fas fa-book-reader fa-fw mr-1"></i>' . lang('singular_cancellation_reasons') . '</a>');
	            }
	            break;
	        case 'cancellation_reasons':
	            Template::set('extra_action', '<a href="' . site_url(SITE_AREA . '/financial/orders/manage/cancellations') . '" class="btn btn-primary btn-sm ml-2" data-toggle="tooltip" data-title="' . lang('order_cancellations') . '"><i class="fas fa-home fa-fw mr-1"></i></a>');
	            break;
	    }
	    $this->gc->module_view($module, $filter, $offset, 'admin', $extra);
	}
	
	/**
	 * Add/Edit Order Data
	 *
	 * @return void
	 */
	public function form($module = 'statuses', $action = 'create') {
	    if($module == 'orders') {
	        $this->create();
	    } else {
    	    if($module == 'cancellations') {
    	        Template::redirect(SITE_AREA . '/financial/orders/manage/items');
    	    }
    	    $extra = array(
    	        'selectize' => true,
    	        'model' => 'orders',
    	        'context' => 'financial',
    	        'fields_method' => $module
    	    );
    	    if($module != 'orders') {
    	        $extra['prefix'] = 'order';
    	        $extra['module_name'] = 'orders';
    	    }
    	    if($module == 'items') {
    	        $extra['method'] = 'after_item_save';
    	    }
    	    $this->gcrud->form($module, $action, 'admin', $extra);
	    }
	}
	
	/**
	 * Add a new order
	 */
	public function create() {
	    if(isset($_POST['create'])) {
	        $this->form_validation->set_rules('user_id', 'lang:bf_user', 'trim|required');
	        $this->form_validation->set_rules('product_ids', 'lang:gc_menu_products', 'trim');
	        $this->form_validation->set_rules('package_id', 'lang:singular_packages', 'trim|is_numeric|required');
	        $this->form_validation->set_rules('ad_id', 'lang:singular_ads', 'trim');
	        $this->form_validation->set_rules('wallet_amount', 'lang:item_title_wallet', 'trim|is_decimal');
	        if(empty($_POST['product_ids']) && empty($_POST['order_custom_id']) && empty($_POST['ad_id']) && empty($_POST['wallet_amount']) && empty($_POST['package_id'])) {
	            $this->form_validation->set_rules('order_service', 'lang:order_service', 'trim|required');	            
	        }
	        if ($this->form_validation->run() === true) {
    	        $post_data = $_POST;
    	        unset($post_data['create']);
    	        // save data in a session variable to restore it on validation error
    	        $this->session->set_userdata('order_create_admin', $post_data);
	        }
	    }
	    // clear cart items
		$user_id = 0;
	    if(isset($_POST['clear_cart'])) {
	        $this->orders_model->checkout_params($user_id, false, array(), true);
	    }
	    if($this->session->userdata('order_create_admin')) {	   
	        // Let user select optional items before checkout
	        $order_optional_step = gc_json_settings('order.settings', 'optional_items_selection');
	        $step = !empty($_GET['step']) ? (int) $_GET['step'] : 1;
	        $step = $step == 1 ? 1 : 2;
	        if($order_optional_step && $step == 1) {
	            $this->orders_model->optional_item_steps();
	            Template::set(array(
	                'toolbar_title' => lang('optional_items')
	            ));
	            Template::set_view('members/checkout');
	            Template::render();
	        } else {
    	        $order_data = $this->session->userdata('order_create_admin');
    	        $user_id = (int) $order_data['user_id'];
    	        if(!isset($order_data['item_processed'])) {
        	        $product_ids = explode(',', $order_data['product_ids'] ?? '');
        	        if(!empty($product_ids)) {
        	            foreach($product_ids as $product_id) {
        	                if(!empty($product_id)) {
        	                   $this->orders_model->cart((int) $product_id, 0, 1, 0, 'products', false);
        	                }
        	            }
        	        }
        	        if(!empty($order_data['package_id'])) {
        	            $this->orders_model->cart((int) $order_data['package_id'], 1, 1, 0, 'packages', false, array(), $user_id);
        	        }
        	        if(!empty($order_data['ad_id'])) { // Ad Package
        	            $this->session->set_userdata('checkout_ad_id', $order_data['ad_id']);
        	        }
        	        if(!empty($order_data['wallet_amount'])) { // Wallet Amount
        	            $this->session->set_userdata('checkout_wallet_amount', $order_data['wallet_amount']);
        	        }
        	        $order_data['item_processed'] = true;
        	        $this->session->set_userdata('order_create_admin', $order_data);
    	        }
    	        $this->session->set_userdata('checkout_redirect', current_url());
    	        $this->load->library('crypt');
    	        $this->load->library('layouts');
    	        $this->orders_model->checkout($user_id, false, 'admin');
	        }
	    } else {
	        Assets::add_css('selectize.bootstrap3.css');
	        Assets::add_js('selectize.min.js');
	        Assets::add_js('selectize-search.js');
	        $user_id = (int) $this->input->post('user_id');
	        Assets::add_js("
$('#user_id').change(function () {
	var selected_value = $(this).val();
	var options = $(\"#order_custom_id\");
	$.getJSON('" . site_url(SITE_AREA . '/financial/orders/order_customs/') . "' + selected_value, function(response) {
        options.empty();        
        $.each(response, function(key, value) {
	        options.append($('<option />').val(key).text(value));
	    });
	});
});
            ", 'inline');
	        Template::set(array(
	            'toolbar_title' => lang('title_create_order')
	        ));
	        Template::set_view('financial/create');
	        Template::render();
	    }
	}
	
	/**
	 * Get user specific order items
	 * @param number $user_id
	 */
	public function order_customs($user_id = 0) {
	    echo json_encode($this->orders_model->get_user_order_customs($user_id));
	}
	
	/**
	 * Get Customer Detail
	 */
	public function get_customer_detail() {
		$user_info = $this->orders_model->get_customer_info($this->input->get('id'));
		$user_info->google_places_api = settings_item('site.google_places_api');
		header('Content-Type: application/x-json; charset=utf-8');
		echo json_encode($user_info);
	}
	
	/**
	 * Get States and Cities using AJAX
	 * @return string - location information in HTML format
	 */
	public function loadData() {
		$loadType=$_POST['loadType'];
		$loadId=$_POST['loadId'];
		$this->load->model('users/user_model');
		
		// Display meta information
		$this->load->model('fields/fields_model');
		$user_info = $this->fields_model->find_meta_for('users', $_POST['user_id']);
		
		$result=$this->orders_model->getData($loadType,$loadId);
		$HTML="";
		if($result){
			if($loadType == 'state') {
				foreach($result as $list){
						$HTML.="<option value='".$list->id."' " . (!empty($user_info->state_id) && ($list->id == $user_info->state_id) ? 'selected' :'').">".$list->name."</option>";

				}
			} elseif($loadType == 'city') {
				foreach($result as $list){				
					$HTML.="<option value='".$list->id."' ".(!empty($user_info->city_id) && ($list->id == $user_info->city_id) ? 'selected' :'').">".$list->name."</option>";	
				}
			}
		}
		echo $HTML;
	}
	

	/**
	 * Allows editing of Orders data.
	 * @param number $id
	 * @param number $is_invoice - 1 - The request is from the invoice page
	 * @param number $is_order_id - 1 - The passed ID value is an order_id
	 * @return void
	 */
	public function view($id, $is_invoice = 0, $is_order_id = 0) {
	    $this->auth->restrict('Orders.Financial.Detail');
	    $this->load->model('users/user_model');
	    
	    // redirect user to the orders page after order item edit
	    $this->session->set_userdata('order_items_last_visited_page', current_url());
	    
		if (empty($id)) {
			Template::set_message(lang('error_no_record_found'), 'danger');
			redirect(SITE_AREA .'/financial/orders');
		}
		$this->load->model('currencies/currencies_model');
		$this->load->model('payouts/payouts_model');
		$order_info = (array) $this->orders_model->get_order($id, $is_invoice == 1 ? 'invoice_id' : ($is_order_id == 1 ? 'order_id' : 'id'));
		if(empty($order_info)) {
		    Template::set_message(lang('error_no_record_found'), 'danger');
		    redirect(SITE_AREA .'/financial/orders');
		}
		// download PDF file
		if (isset($_POST['print'])) {
		    $this->gcrud->stats(array('module' => 'orders'));
		    $this->orders_model->email_print($order_info['order_id'], 'pdf', 3, array('order' => (object) $order_info, 'role' => 'frontend'));
		}
		
		// Refund wallet payment
		if (isset($_POST['refund_wallet'])) {
		    $this->gcrud->stats(array('module' => 'orders'));
			// get wallet balance
			$wallet_balance = $this->db->select('wallets AS cash')->get_where('users', array('id' => $order_info['user_id']))->row();
			if($wallet_balance) {
				$this->orders_model->update_status($order_info['id'], null, null, 5, 1, true); // change status to refunded
			    $this->db->where('order_id', $order_info['order_id'])->update('orders', array('wallets_refund' => date('Y-m-d H:i:s')));
				// calculate new balance
				// $new_balance = $wallet_balance->cash + $order_info['wallets'];
				// refund wallet balance
				// $this->db->where('id', $order_info['user_id'])->update('users', array('wallets' => $new_balance));
				// deduct the amount from the users wallet
				$notes = null;
			    $this->load->model('wallets/wallets_points_model');
			    $this->wallets_points_model->update_cash('wallets', $order_info['user_id'], 'orders', $order_info['wallets'], 'credit', $notes, $order_info['id'], array('order_id' => $order_info['order_id']));
				Template::set_message(lang('refund_success'), 'success');
				Template::redirect(current_url());
			} else {
				Template::set_message(lang('refund_error'), 'danger');
			}
		}
		if (isset($_POST['save'])) {	
		    $this->gcrud->stats(array('module' => 'orders'));
			$this->auth->restrict($this->permissionEdit);
			$new_status_id = $this->input->post('new_status_id') ? $this->input->post('new_status_id') : 0;
			$notify_customer = $this->input->post('notify_customer') ? 1 : 0;
			$customer_notes = $this->input->post('order_notes');
			$reminder_notifications = (int) $this->input->post('reminder_notifications');
			
		    // update order status
		    $reverse_row = $this->input->post('reverse_row') ? 1 : 0;
		    $this->orders_model->update_status($order_info['id'], $order_info['order_id'], array(), $new_status_id, $notify_customer, $reverse_row);
		    
		    // Allow user to update payment method and transaction id
		    $order_update_data = array(
		        'payment_method' => $this->input->post('payment_method'),
		        'transaction_id' => $this->input->post('transaction_id'),
		        'reminder_notifications' => $reminder_notifications,
		        'notes' => $customer_notes
		    );
		    $this->db->where('order_id', $order_info['order_id'])->update('orders', $order_update_data);
		    // update the reminder_notifications column in the order_items table
		    if($order_info['reminder_notifications'] != $reminder_notifications) {
		        $this->db->where('order_id', $order_info['order_id'])->update('order_items', array('reminder_notifications' => $reminder_notifications));
		    }
		    Template::set_message(sprintf(lang('msg_save_success'), lang('label_order')), 'success');
			// Template::redirect(current_url());
		} 
		
		// Display meta information
		$this->load->model('fields/fields_model');
		
		$user = array();
		$role = 'frontend';
		$form_fields = $this->fields_model->get_fields('orders', $role); // custom fields
		$field_groups = $this->fields_model->field_groups($form_fields, array());
		$field_type = 'frontend';
		$form_fields = $this->fields_model->load_fields('orders', array(), $field_type, $user, 0, false, $form_fields);
		$customer_info = $this->db->where('order_id', $order_info['order_id'])->get('order_customers')->row();
		Template::set(array(
		    'record' => $customer_info,
		    'form_fields' => $form_fields,
		    'field_groups' => $field_groups
		));
		
		// update customer information
		if (isset($_POST['save_customer_info'])) {
		    $this->gcrud->stats(array('module' => 'orders'));
		    $validated = $this->fields_model->fields_validation_rules($form_fields, 'order_customers', $role);
		    if($validated) {
		        // save order customers
		        if($this->fields_model->save_module($form_fields, 'order_customers', $order_info['user_id'], $role, $customer_info->id, 'update', array('order_id' => $order_info['order_id'], 'user_id' => $order_info['user_id'], 'email' => $order_info['user_email']))) {
		            Template::set_message(sprintf(lang('msg_save_success'), lang('label_customer_information')), 'success');
		            redirect(site_url(SITE_AREA .'/financial/orders/view/' .$id));
		        }
		    }
		}
		
		// Display shipment information to the Admin
		$shipments = $this->db->get_where('order_shipments', array('order_id' => $order_info['order_id']))->result();
		$ordered_items = $this->orders_model->get_order_items($order_info['order_id']);

		// Display Affiliate User Email and Display Name to the admin
		if(!empty($order_info['affiliate_id'])) {
		    $affiliate_user = $this->user_model->get_user($order_info['affiliate_id']);
		    Template::set('affiliate_user', $affiliate_user);
		    $coupon_commission = $this->db->where(array('user_id' => $order_info['affiliate_id'], 'module_id' => 22, 'order_id' => $order_info['order_id']))->get('commissions')->row();
		    if($coupon_commission) {
		        $order_info['affiliate_commission'] = $coupon_commission->amount;
		    } else {
		        $order_info['affiliate_commission'] = 0;
		    }
		}
		
		// Display Invitation User Email and Display Name to the admin
		if(!empty($order_info['invitation_user_id'])) {
		    $invitation_user = $this->user_model->get_user($order_info['invitation_user_id']);
		    Template::set('invitation_user', $invitation_user);
		}
		
		// Display payment gateways list
		$payment_gateways = array('' => lang('placeholder_select'));
		$gateways_result = $this->db->select('name, display_name')->get('payment_gateways')->result();
		if($gateways_result) {
		    foreach($gateways_result as $gateway_result) {
		        $payment_gateways[$gateway_result->name] = $gateway_result->display_name;
		    }
		}
		
		Template::set(array(
		    'toolbar_title' => lang('title_view_order'),
		    'order' => $order_info,		    
		    'payment_gateways' => $payment_gateways,
		    'shipments' => $shipments,
		    'order_items' => $ordered_items,
		    'order_histories' => $this->orders_model->order_history_comments($order_info['order_id']),
		    'order_currency' => $this->currencies_model->get_currency($order_info['currency_code'])
		));
		Template::render();
	}
	
	/**
	 * Generate Order Reports
	 */
	public function reports($offset = 0) {
	    $this->auth->restrict('Orders.Financial.Reports');
		// DATE TIME PICKER
		Assets::add_js('moment.js');
		Assets::add_css ( 'bootstrap-datetimepicker.min.css' );
		Assets::add_js ( 'bootstrap-datetimepicker.min.js' );
		Assets::add_js('order-reports.js');		

		// user search js and css dependencies
		Assets::add_css('selectize.bootstrap3.css');
		Assets::add_js('selectize.min.js');
        Assets::add_js('selectize-search.js');
		Assets::add_js('location-selection.js');
		
		$this->load->model('locations/countries_model');
		$this->load->model('locations/states_model');
		$this->load->model('locations/cities_model');
		
		$this->load->library('format');
		$action = 'summary';
		if (isset($_POST['submit'])) {
		    $this->gcrud->stats(array('module' => 'orders'));
			$action = $_POST['submit'];
			$limit = $action == 'view' ? 20000 : 50000;
			$this->form_validation->set_rules('date_from', 'lang:label_from', 'trim|valid_date');
			$this->form_validation->set_rules('date_to', 'lang:label_to', 'trim|valid_date|compare_dates[date_from]');
			$this->form_validation->set_rules('status', 'lang:label_status', 'trim');
			$this->form_validation->set_rules('country_iso', 'lang:label_country', 'trim');
			$this->form_validation->set_rules('state_id', 'lang:label_state', 'trim');
			$this->form_validation->set_rules('city_id', 'lang:label_city', 'trim');
			
			if ($this->form_validation->run() === true) {				
				$records = $this->_report($action, 0, $limit, $offset);
				if($action != 'summary') {
					$amount_total = 0;
					$serial_number = 0;
					$formatted_records = array();
					$field_names = array(lang('label_serial_no'), lang('label_id'), lang('label_name'), lang('label_email'), lang('label_city'), lang('label_postcode'),  lang('created_on'), lang('label_invoice_no'), lang('label_amount'));
					if(!empty($records)) {
						foreach($records as $record) {
						    $item_title = lang('not_applicable');
							$invoice_no = (!empty($record->invoice_id) ? $record->invoice_prefix . $record->invoice_id : lang('not_applicable'));
							$current_package = ($record->status_id == 1 && $record->created_on <= date('Y-m-d H:i:s') ? $item_title : '');
							$lfcr = chr(10) . chr(13);
							if(!array_key_exists($record->user_id, $formatted_records)) {
								$serial_number += 1;					
								$formatted_records[$record->user_id] = array(
									'serial_number' => $serial_number,
									'user_id' => $record->user_id,
									'name' => $record->display_name,
									'email' => $record->email,
									'city' => $record->city,
									'postcode' => $record->postcode,
									// 'reference_code' => $record->reference_code,
									'created_on' => $record->created_on,
									'invoice_no' => $invoice_no,
								    'amount' => currency_format($record->order_subtotal)
								);
							} else {
								$formatted_records[$record->user_id]['created_on'] .= "\n" . $record->created_on;
								$formatted_records[$record->user_id]['invoice_no'] .= "\n" . $invoice_no;
								$formatted_records[$record->user_id]['amount'] .= "\n" . currency_format($record->order_subtotal);
							}
							$amount_total += $record->order_subtotal;
						}
					}
				}
				switch($action) {
					case 'excel':
						// first row is the header, so add 1
						$rows_count = count($formatted_records) + 2;
						// array(column, row, value),
						$extra_cells = array(				
							array(9, $rows_count, lang('label_order_total')),
							array(10, $rows_count, $amount_total),
						);
						$this->format->to_excel(array(
							'field_names' => $field_names,
							'rows' => $formatted_records,
							'file_name' => 'users',
							'extra_cells' => $extra_cells,
							'wrap_cells' => 'G2:K' . $rows_count, // Wrap Text Fields
							'autosize_columns' => array('G', 'H','I','J','K')
						));
					break;
					case 'pdf':
						$content = $this->load->view('orders/financial/report_tpl', 
						array(
							'field_names' => $field_names,
							'records' => $formatted_records,
							'amount_total' => $amount_total
						), TRUE);
						$this->gc->create_pdf('orders_' . time(), $content);
					break;
					case 'view':
						Assets::add_js( array(
							'jquery.dataTables.min',
							'dataTables.bootstrap.min',
						));
						Assets::add_css('dataTables.bootstrap.min');
						Assets::add_js("$('.orders-report').DataTable({responsive: true});", 'inline');
						Template::set(array(
							'field_names' => $field_names,
							'records' => $formatted_records,
							'amount_total' => $amount_total
						));
					break;
					case 'summary':
						// Pagination
						$this->load->library('pagination');
						// $total_orders = count($records);
						$total_orders = !empty($records) ? count((array) $records) : 0;
						$pager['base_url'] = site_url(SITE_AREA ."/financial/orders/reports/");
						$pager['total_rows'] = $total_orders;
						$pager['per_page'] = $this->limit;
						$pager['uri_segment']	= 5;						
						$this->pagination->initialize($pager);
						Template::set('records', $records);
					break;
				}
			}
		}
		$countries = $this->countries_model->find_all_by('active', 1);
		if(!empty($_POST['country_iso'])) {
			$states = $this->states_model->find_all_by(array('active' => 1, 'country_iso' => $_POST['country_iso']));
			Template::set('states', $states);
		}		
		if(!empty($_POST['state_id'])) {
			$cities = $this->cities_model->find_all_by(array('active' => 1, 'state_id' => $_POST['state_id']));
			Template::set('cities', $cities);
		}
		Template::set(array(
			'countries' => $countries,
			'action' => $action,
			'toolbar_title' => lang('advanced_search')
		));
		Template::render();		
	}
	
	/**
	 * Query to display report to the user
	 */
    private function _report($action, $count = 0, $limit = 20, $offset = 0) {
		$package_id = (int) $this->input->post('package_id');
		//$reference_code = $this->input->post('reference_code');
		$country_iso = $this->input->post('country_iso');
		$state_id = (int) $this->input->post('state_id');
		$city_id = (int) $this->input->post('city_id');
		$user_id = (int) $this->input->post('user_id');
		$date_from = $this->input->post('date_from');
		$date_to = $this->input->post('date_to');
		$status = (int) $this->input->post('status');		
				
		$where = " WHERE O.deleted = 0";
		if(!empty($user_id)) {
			$where .= " AND O.user_id = " . $user_id;
		}
		if(!empty($package_id)) {
			$where .= " AND O.package_id = " . $package_id;
		}
		/*if(!empty($reference_code)) {
			$where .= " AND U.reference_code = '" . $reference_code . "'";
		}*/		
		if($status != 99) {
			$where .= " AND O.status_id = " . $status;
		}
		if(!empty($date_from) && !empty($date_to)) {
			$where .= " AND (date(O.created_on) BETWEEN '" . $date_from . "' AND '" . $date_to . "')";
		} else if(!empty($date_from)) {
			$where .= " AND date(O.created_on) >= '" . $date_from . "'";
		} else if(!empty($date_to)) {
			$where .= " AND date(O.created_on) >= '" . $date_to . "'";
		}
		
		if($action == 'summary') {
			$group_by = " GROUP BY C.iso";
			if(!empty($city_id)) {
				$where .= " AND CT.id = $city_id";
				$group_by = " GROUP BY OC.city_id";
			} else if(!empty($state_id)) {
				$where .= " AND S.id = $state_id";
				$group_by = " GROUP BY S.id";
			} else if(!empty($country_iso)) {
				$where .= " AND C.iso = '$country_iso'";
				$group_by = " GROUP BY C.iso";
			}
		} else {
			if(!empty($country_iso)) {
				$where .= " AND U.country_iso = '" . $country_iso . "'";
			}
			if(!empty($state_id)) {
				$where .= " AND U.state_id = " . $state_id;
			}
			if(!empty($city_id)) {
				$where .= " AND U.city_id = " . $city_id;
			}
		}
		
		if($action == 'summary') {
			if($count == 0) {
				$sql = "SELECT COUNT(O.order_id) AS TotalSales, COUNT(DISTINCT(OC.user_id)) AS TotalUsers, SUM(O.order_discount) AS TotalDiscount, SUM(O.order_tax) AS TotalTax, SUM(O.order_subtotal) AS SubTotal, SUM(O.order_total) AS GrossAmount,";
				if(!empty($city)) {
					$sql .= " CT.name AS Location";
				} else if(!empty($state)) {
					$sql .= " S.name AS Location";
				} else {
					$sql .= " C.name AS Location";
				}
			} else {
				$sql = "SELECT COUNT(O.order_id) AS counts";
			}
			$sql .= " FROM " . $this->db->dbprefix . "orders O";
			$sql .= " LEFT JOIN " . $this->db->dbprefix . "users U ON U.id = O.user_id";
			$sql .= " INNER JOIN " . $this->db->dbprefix . "order_customers OC ON OC.order_id = O.order_id";
			$sql .= " LEFT JOIN " . $this->db->dbprefix . "cities CT ON CT.id = OC.city_id";
			$sql .= " LEFT JOIN " . $this->db->dbprefix . "states S ON S.id = CT.state_id";
			$sql .= " INNER JOIN " . $this->db->dbprefix . "countries C ON C.iso = S.country_iso";
			$sql .= $where;
			$sql .= $group_by;
			if($count == 0) {
				$result = $this->db->query ( $sql )->result();
				return $result ? $result : null;
			} else {
				$result = $this->db->query ( $sql )->row();
				return $result ? $result->counts : 0;
			}
		} else {
			$sql = "SELECT U.id AS user_id, U.display_name, U.email, U.city, U.postcode, O.currency_code, O.currency_value,";
			$sql .= " O.order_subtotal, INV.prefix AS invoice_prefix, INV.id as invoice_id, O.created_on,";
			$sql .= " O.items, O.status_id";
			$sql .= " FROM " . $this->db->dbprefix . "orders O";
			$sql .= " INNER JOIN " . $this->db->dbprefix . "users U ON U.id = O.user_id";
			$sql .= " LEFT JOIN " . $this->db->dbprefix . "order_invoices INV ON INV.order_id = O.order_id";
			$sql .= " LEFT JOIN " . $this->db->dbprefix . "order_customers OC ON OC.order_id = O.order_id";
			$sql .= $where;
			if($count == 0) {
				$sql .=  PHP_EOL . " GROUP BY O.order_id" . PHP_EOL;
				$sql .= " ORDER BY U.id";
				if ($limit != 0) {
					$sql .= " LIMIT " . $offset . ", " . $limit;
				} else {
					$sql .= " LIMIT $limit";
				}
				return $this->db->query($sql)->result();		
			} else {
				$result = $this->db->query ( $sql )->row();
				return $result ? $result->total_count : 0;
			}
		}
    }
    
    /**
     * Allow user to track the ordered items using GET parameters
     * @param integer $item_id - Ordered Item ID
     * @param string $order_id
     */
    public function track() {
        $this->auth->restrict('Orders.Financial.Track');
        $order_item_id = null;
        $order_id = null;
        if(isset($_GET['item_id'])) {
            $order_item_id = $_GET['item_id'];
        }
        if(isset($_GET['order_id'])) {
            $order_id = $_GET['order_id'];
        }
        // This method has no View in the Themes Module, i.e. load layout files manually
        $this->load->model('orders/ordered_items_model');
        $this->ordered_items_model->track($order_item_id, $order_id, SITE_AREA .'/financial/orders/manage/items', 'admin');
        Template::render();
    }
    
    /**
     * Allow Admin to view all shipment information
     */
    public function shipments() {
        $redirect_url = SITE_AREA .'/financial/orders/manage/items';
        $this->load->model('orders/ordered_items_model');
        $this->ordered_items_model->shipments('admin', $this->uri->segment(5), $redirect_url);
        Template::render();
    }
    
    /**
     * Cancel Order/Order ITEMs
     */
    public function cancel($item_id = 0, $order_id = null) {
        $this->load->model('orders/ordered_items_model');
        $this->ordered_items_model->cancel($item_id, $order_id, 'admin');
    }
    
    /**
     * Save settings
     */
    public function settings() {
        clear_cache('site_settings'); // Clear Site Settings Cache
        $setting_name = 'order.settings';
        $this->load->model('settings/settings_model');
        if (isset($_POST['save'])) {
            $this->gcrud->stats(array('module' => 'orders'));
            $this->form_validation->set_rules('item_id_invoice', 'lang:item_id_invoice', 'trim');
            $this->form_validation->set_rules('order_prefix', 'lang:order_prefix', 'trim|required|max_length[10]');
            $this->form_validation->set_rules('invoice_prefix', 'lang:invoice_prefix', 'trim|required|max_length[10]');
            $this->form_validation->set_rules('order_status', 'lang:invoice_prefix', 'trim');
            $this->form_validation->set_rules('terms_conditions', 'lang:terms_conditions_page', 'trim');
            $this->form_validation->set_rules('checkout_cart', 'lang:order_checkout_cart', 'trim');
            if($this->form_validation->run() == TRUE) {
                $settings = array(
                    'checkout_cart' => $this->input->post('checkout_cart') ? 1 : 0,
                    'item_id_invoice' => (int) $this->input->post('item_id_invoice'),
                    'order_prefix' => $this->input->post('order_prefix'),
                    'invoice_prefix' => $this->input->post('invoice_prefix'),
                    'order_status' => (int) $this->input->post('order_status'),
                    'terms_conditions' => $this->input->post('terms_conditions')
                );
                $data = array (array('name' => $setting_name, 'value' => json_encode($settings)));
                $updated = $this->settings_model->update_batch($data, 'name');
                if ($updated) {
                    Template::set_message(sprintf(lang('msg_save_success'), lang('gc_menu_settings')), 'success');
                    Template::redirect(site_url(SITE_AREA . '/financial/orders/settings'));
                } else {
                    Template::set_message(sprintf(lang('msg_edit_failure'), lang('gc_menu_settings')), 'danger');
                }
            }
        }
        $result = $this->db->select('value')->get_where('settings', array('name' => $setting_name))->row();
        if($result) {
            $settings = json_decode($result->value ?? '');
        } else {
            $settings = json_decode(settings_item($setting_name) ?? '');
        }
        Template::set('settings', $settings);
        Template::set('toolbar_title', lang('gc_menu_settings'));
        Template::render();
    }
    
    /**
     * Send invoice to the user and accept payments directly
     */
    public function send_invoice() {
        $user_id = 13;
        $product_ids = '1,3';
        $custom_payment_ids = '10,11';
        $this->load->model('users/user_model');
        $this->load->model('fields/fields_model');
        $user = $this->user_model->get_user($user_id);
        
        // get fields
        $field_role = 'frontend';
        $form_fields = $this->fields_model->get_fields('orders', $field_role); // custom fields
        $field_type = 'frontend';
        $form_fields = $this->fields_model->load_fields('orders', array(), $field_type, $user, 0, false, $form_fields);
        
        // products
        if(!empty($product_ids)) {
            $product_ids = explode(',', $product_ids);
            foreach($product_ids as $id) {
                $this->orders_model->cart($id, 0, 1, 0, 'products', false);
            }
        }
        
        // custom payments
        $this->session->set_userdata('checkout_payment_id', explode(',', $custom_payment_ids ?? ''));
        
        // prepare cart items
        $cart = $this->orders_model->cart_items($user_id, $user);
        
        // save order in different tables
        // also use the prepare_order method in the checkout
        // not yet tested
        $this->orders_model->prepare_order($user_id, $user, $form_fields, $field_role, false, 'admin');
    }
    
    /**
     * Assign Free Package to a user
     */
    public function assign_package() {
        Assets::add_css('selectize.bootstrap3.css');
        Assets::add_js('selectize.min.js');
        Assets::add_js('selectize-search.js');
        if(isset($_POST['save'])) {
            $this->gcrud->stats(array('module' => 'orders'));
            $this->form_validation->set_rules('user_id', 'lang:bf_user', 'trim|required');
            if ($this->form_validation->run() === true) {
                $user_id = (int) $this->input->post('user_id');
                $order_item = $this->db->where(array('user_id' => $user_id, 'module_id' => gc_module_id('packages')))->get('order_items')->row();
                if(!$order_item) { // assign a package
                    $this->load->model('users/user_model');
                    $this->user_model->assign_package($user_id);
                    Template::set_message(lang('msg_success'), 'success');
                    Template::redirect(SITE_AREA . '/financial/orders');                    
                } else {
                    Template::set_message(sprintf(lang('assign_package_error'), gc_date_format($order_item->created_on, true), (!empty($order_item->expires_on) ? gc_date_format($order_item->expires_on) : lang('label_never'))) . ' ', 'danger');
                }
            }
        }
        Template::set(array(
            'toolbar_title' => lang('assign_package')
        ));
        Template::render();
    }
}