<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Migration_Install_orders extends Migration
{
	/**
	 * The name of the database table
	 *
	 * @var String
	 */
	private $order_table = 'orders';
	private $order_coupons_table = 'order_coupons';
	private $order_customers_table = 'order_customers';
	private $order_histories_table = 'order_histories';

	/**
	 * The orders table's fields
	 *
	 * @var Array
	 */
	private $fields = array(
			'id' => array(
					'type' => 'INT',
					'constraint' => 11,
					'auto_increment' => TRUE
			),
			'order_id' => array(
					'type' => 'VARCHAR',
					'constraint' => 20,
					'null' => FALSE
			),
			'order_type' => array(
					'type' => 'TINYINT',
					'constraint' => 2,
					'null' => FALSE,
					'default' => 0
			),
			'subscription_type' => array(
					'type' => 'TINYINT',
					'constraint' => 2,
					'null' => FALSE,
					'default' => 1
			),
			'renew_upgrade_order_id' => array(
					'type' => 'VARCHAR',
					'constraint' => 20,
					'null' => TRUE
			),
			'user_id' => array(
					'type' => 'BIGINT',
					'constraint' => 20,
					'null' => FALSE
			),
    	    'seller_id' => array( // Bookings Payout
    	        'type' => 'BIGINT',
    	        'constraint' => 20,
    	        'null' => FALSE,
    	        'default' => 0
    	    ),
			'package_id' => array(
					'type' => 'INT',
					'constraint' => 11,
					'null' => FALSE,
					'default' => 0
			),
			'module_id' => array(
					'type' => 'TINYINT',
					'constraint' => 1,
					'null' => FALSE,
					'default' => 0
			),
			'row_id' => array(
					'type' => 'BIGINT',
					'constraint' => 20,
					'null' => FALSE,
					'default' => 0
			),
			'user_email' => array(
					'type' => 'VARCHAR',
					'constraint' => 100,
					'null' => FALSE
			),
			'item_price' => array(
					'type' => 'DECIMAL',
					'constraint' => '12,2',
					'null' => FALSE
			),
			'order_subtotal' => array(
					'type' => 'DECIMAL',
					'constraint' => '12,2',
					'null' => FALSE
			),
			'order_tax' => array(
					'type' => 'DECIMAL',
					'constraint' => '12,2',
					'null' => FALSE,
					'default'=> '0.00'
			),
			'order_discount' => array(
					'type' => 'DECIMAL',
					'constraint' => '12,2',
					'null' => FALSE,
					'default'=> '0.00'
			),
			'order_total' => array(
					'type' => 'DECIMAL',
					'constraint' => '12,2',
					'null' => FALSE
			),
    	    'order_commission' => array(
        	        'type' => 'FLOAT',
        	        'null' => FALSE,
        	        'default' => 0
    	    ),
			'tax_percent' => array(
					'type' => 'DECIMAL',
					'constraint' => '11,2',
					'null' => TRUE
			),
			'including_tax' => array(
					'type' => 'TINYINT',
					'constraint' => 1,
					'null' => FALSE,
					'default'=> 0
			),
			'currency_code' => array(
					'type' => 'CHAR',
					'constraint' => 3,
					'null' => FALSE
			),
			'currency_value' => array(
					'type' => 'DECIMAL',
					'constraint' => '12,2',
					'null' => FALSE
			),
			'payment_method' => array(
					'type' => 'VARCHAR',
					'constraint' => 200,
					'null' => TRUE
			),
			'payment_status' => array(
					'type' => 'TINYINT',
					'constraint' => 1,
					'null' => FALSE,
					'default' => 0
			),		
			'wallets' => array(
					'type' => 'FLOAT',
					'null' => FALSE,
					'default' => 0
			),			
			'wallets_refund' => array(
					'type' => 'DATETIME',
					'null' => TRUE
			),		
			'transaction_id' => array(
					'type' => 'VARCHAR',
					'constraint' => 100,
					'null' => TRUE
			),
			'payer_id' => array(
					'type' => 'VARCHAR',
					'constraint' => 100,
					'null' => TRUE
			),
			'payment_details' => array(
					'type' => 'MEDIUMTEXT',
					'null' => TRUE
			),
			'item' => array(
					'type' => 'TEXT',
					'null' => TRUE
			),
			'qty' => array(
					'type' => 'INT',
					'constraint' => 11,
					'null' => FALSE,
					'default' => 0
			),
			'comments' => array(
					'type' => 'MEDIUMTEXT',
					'null' => TRUE
			),
			'notes' => array(
					'type' => 'MEDIUMTEXT',
					'null' => TRUE
			),
			'is_invoiced' => array(
					'type' => 'TINYINT',
					'constraint' => 1,
					'null' => FALSE,
					'default'=> 0
			),
			'date_paid' => array(
					'type' => 'DATE',
					'null' => TRUE
			),
			'next_invoice_date' => array(
					'type' => 'DATE',
					'null' => TRUE
			),
			'expires_on' => array(
					'type' => 'DATE',
					'null' => TRUE
			),
			'affiliate_id' => array(
					'type' => 'BIGINT',
					'constraint' => 20,
					'null' => FALSE,
					'default' => 0
			),
			'affiliate_commission' => array(
					'type' => 'float',
					'default' => 0
			),
			'commission_module' => array(
					'type' => 'TINYINT',
					'constraint' => 1,
					'null' => FALSE,
					'default' => 0
			),
			'cronrun_on' => array(
					'type' => 'DATETIME',
					'null' => TRUE
			),
			'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
			'modified_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
			'ip_address' => array(
					'type' => 'VARCHAR',
					'constraint' => 50,
					'null' => TRUE
			),
			'status' => array(
					'type' => 'TINYINT',
					'constraint' => 1,
					'null' => FALSE,
					'default'=> 0
			),
			'deleted' => array(
					'type' => 'TINYINT',
					'constraint' => 1,
					'null' => FALSE,
					'default'=> 0
			),
	);
	/**
	 * The Order Coupons table's fields
	 *
	 * @var Array
	 */
	private $order_coupons_fields = array(
			'id' => array(
					'type' => 'INT',
					'constraint' => 11,
					'auto_increment' => TRUE
			),
			'order_id' => array(
					'type' => 'VARCHAR',
					'constraint' => 20,
					'null' => FALSE
			),
			'user_id' => array(
					'type' => 'BIGINT',
					'constraint' => 20,
					'null' => FALSE
			),
			'user_email' => array(
					'type' => 'VARCHAR',
					'constraint' => 30,
					'null' => FALSE
			),
			'coupon_id' => array(
					'type' => 'INT',
					'constraint' => 11,
					'null' => FALSE
			),
			'coupon_code' => array(
					'type' => 'VARCHAR',
					'constraint' => 150,
					'null' => FALSE
			),
			'value' => array(
					'type' => 'DECIMAL',
					'constraint' => '12,2',
					'null' => FALSE
			),
			'value_type' => array(
					'type' => 'CHAR',
					'constraint' => 1,
					'null' => FALSE
			),
			'amount' => array(
					'type' => 'DECIMAL',
					'constraint' => '12,2',
					'null' => FALSE
			),
			'created_date' => array(
					'type' => 'DATETIME',
					'null' => FALSE
			),
	);
	/**
	 * The Order Customers table's fields
	 *
	 * @var Array
	 */
	private $order_customers_fields = array(
			'id' => array(
				'type' => 'INT',
				'constraint' => 11,
				'auto_increment' => TRUE
			),
			'order_id' => array(
				'type' => 'VARCHAR',
				'constraint' => 20,
				'null' => FALSE
			),
			'user_id' => array(
				'type' => 'BIGINT',
				'constraint' => 20,
				'null' => FALSE
			),
			'name' => array(
				'type' => 'VARCHAR',
				'constraint' => 100,
				'null' => FALSE
			),
			'email' => array(
				'type' => 'VARCHAR',
				'constraint' => 100,
				'null' => FALSE
			),
			'phone_number' => array(
				'type' => 'VARCHAR',
				'constraint' => 20,
				'null' => TRUE
			),
			'address' => array(
				'type' => 'TEXT',
				'null' => TRUE
			),
			'postcode' => array(
				'type' => 'VARCHAR',
				'constraint' => 10,
				'null' => TRUE
			),
			'city_id' => array(
				'type' => 'INT',
				'constraint' => 11,
				'null' => FALSE,
				'default'=> 0
			)
	);
	/**
	 * The Order Histories table's fields
	 *
	 * @var Array
	 */
	private $order_histories_fields = array(
			'id' => array(
					'type' => 'INT',
					'constraint' => 11,
					'auto_increment' => TRUE,
			),
			'order_id' => array(
					'type' => 'VARCHAR',
					'constraint' => 20,
					'null' => FALSE,
			),
			'order_status' => array(
					'type' => 'TINYINT',
					'constraint' => 2,
					'null' => FALSE,
			),
			'customer_notified' => array(
					'type' => 'TINYINT',
					'constraint' => 1,
					'null' => FALSE,
					'default'=> 0,
			),
			'comments' => array(
					'type' => 'TEXT',
					'null' => FALSE,
			),
			'created_on' => array(
					'type' => 'DATETIME',
					'null' => FALSE,
			),
	);
	
	/**
	 * Install this migration
	 *
	 * @return void
	 */
	public function up()
	{
		// ORDER TABLE
		$this->dbforge->add_field($this->fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('order_id', false);
		$this->dbforge->add_key('user_id', false);
		$this->dbforge->create_table($this->order_table);
		
		// ORDER COUPONS TABLE
		$this->dbforge->add_field($this->order_coupons_fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('order_id', false);
		$this->dbforge->add_key('user_id', false);
		$this->dbforge->create_table($this->order_coupons_table);
		
		// ORDER CUSTOMERS TABLE
		$this->dbforge->add_field($this->order_customers_fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('order_id', false);
		$this->dbforge->add_key('user_id', false);
		$this->dbforge->create_table($this->order_customers_table);
		
		// ORDER HISTORIES TABLE
		$this->dbforge->add_field($this->order_histories_fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('order_id', false);
		$this->dbforge->create_table($this->order_histories_table);
	}
	
	//--------------------------------------------------------------------
	
	/**
	 * Uninstall this migration
	 *
	 * @return void
	 */
	public function down()
	{
		$this->dbforge->drop_table($this->order_table);
		$this->dbforge->drop_table($this->order_coupons_table);
		$this->dbforge->drop_table($this->order_customers_table);
		$this->dbforge->drop_table($this->order_histories_table);
	}
	
	//--------------------------------------------------------------------
	
	}
