<?php defined('BASEPATH') || exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class Cancellations_model extends BF_Model
{
    protected $table_name	= 'order_cancellations';
	protected $key			= 'id';
	protected $date_format	= 'datetime';

	protected $log_user 	= true;
	protected $set_created	= true;
	protected $set_modified = true;
	protected $soft_deletes	= false;

	protected $created_field     = 'created_on';
    protected $created_by_field  = 'created_by';
	protected $modified_field    = 'modified_on';
    protected $modified_by_field = 'modified_by';

	// Customize the operations of the model without recreating the insert,
    // update, etc. methods by adding the method names to act as callbacks here.
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 	    = array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	// For performance reasons, you may require your model to NOT return the id
	// of the last inserted row as it is a bit of a slow method. This is
    // primarily helpful when running big loops over data.
	protected $return_insert_id = true;

	// The default type for returned row data.
	protected $return_type = 'object';

	// Items that are always removed from data prior to inserts or updates.
	protected $protected_attributes = array();

	// You may need to move certain rules (like required) into the
	// $insert_validation_rules array and out of the standard validation array.
	// That way it is only required during inserts, not updates which may only
	// be updating a portion of the data.
	protected $validation_rules 		= array(
		array(
			'field' => 'order_id',
			'label' => 'lang:label_order_id',
			'rules' => 'required|trim|max_length[20]',
		),
		array(
			'field' => 'user_id',
			'label' => 'lang:label_user_id',
			'rules' => 'required|trim|max_length[20]',
		),
		array(
			'field' => 'module_id',
			'label' => 'lang:label_module_id',
			'rules' => 'required|trim|max_length[1]',
		),
		array(
			'field' => 'row_id',
			'label' => 'lang:label_row_id',
			'rules' => 'required|trim|max_length[20]',
		),
		array(
			'field' => 'reason',
			'label' => 'lang:label_reason',
			'rules' => 'required|trim|max_length[1]',
		),
		array(
			'field' => 'comments',
			'label' => 'lang:label_comments',
			'rules' => 'trim',
		),
		array(
			'field' => 'status',
			'label' => 'lang:label_status',
			'rules' => 'trim|max_length[1]',
		),
	);
	protected $insert_validation_rules  = array();
	protected $skip_validation 			= false;

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
    }
    
    /**
     * Display a list of cancellations data to the User, Admin and the Business Owner
     *
     * @return void
     */
    public function view($user_id, $type = 0, $limit = 25, $offset = 0, $role = 'member') {
        $module = 'order_cancellations';
        if($role == 'admin') {
            $uri_segment = 5; // offset URI segment
            $redirect_uri = '/financial/orders/cancellations';
            $base_url = site_url(SITE_AREA . $redirect_uri);
        } else {
            $uri_segment = 5;
            $base_url = gc_url_format('cancellations/view') . '/' . $type . '/' . $allotment_id;
            $redirect_uri = $base_url;
        }
        $user_field = $type == 1 ? 'seller_id' : 'user_id';
        $where = '';
        $searchterm = $this->gc->search($module, $this->uri->segment($uri_segment), $redirect_uri);
        if (! empty($searchterm)) {
            $where = "T.id LIKE '%" . $searchterm . "%' OR T.order_id LIKE '%" . $searchterm . "%' OR T.created_on LIKE '%" . $searchterm . "%'";
        }
        $pagerUriSegment = $uri_segment;
        $pagerBaseUrl = $base_url . '/';
        
        // Order By
        $sort_by = "T.id DESC";
        $order_by = $this->gc->sort($module);
        if(!empty($order_by)) {
            $sort_by = $order_by;
        }
        $total_rows = $this->get_cancellations(1, $user_id, $user_field, $where, $sort_by, $limit, $offset);
        $this->load->library('pagination');
        $pager['base_url'] = $pagerBaseUrl;
        $pager['total_rows'] = $total_rows;
        $pager['per_page'] = $limit;
        $pager['uri_segment'] = $pagerUriSegment;
        
        $this->pagination->initialize($pager);
        $records = $this->get_cancellations(0, $user_id, $user_field, $where, $sort_by, $limit, $offset);
        if($role != 'admin' && $type == 1) {
            Template::set('module_links', '<li class="nav-item mt-1">' . anchor(gc_url_format($module . '/forms'), lang('cancel'), 'class="btn btn-warning btn-xs"') . '</li>');
        }
        Template::set(array(
            'type' => $type,
            'role' => $role,
            'records' => $records,
            'toolbar_title' => lang('order_cancellations')
        ));
        Template::set_view('orders/view');
    }
    
    /**
     * Get Cancellations Query
     * @param number $count
     * @param number $user_id
     * @param string $user_field - user_id/seller_id
     * @param array $where
     * @param string $sort_by
     * @param number $limit
     * @param number $offset
     * @return number
     */
    public function get_cancellations($count = 0, $user_id = 0, $user_field = 'user_id', $where = array(), $sort_by = 'T.id DESC', $limit = 25, $offset = 0) {
        if($count == 0) {            
            $sql = "SELECT T.*, U.display_name, U.email AS user_email, U.slug AS user_slug, U.city AS user_city,";
            $sql .= $this->gc->module_query(array('bookings'), 'T', false);
        } else {
            $sql = "SELECT COUNT(T.id) AS total_count";
        }
        $sql .= " FROM " . $this->db->dbprefix . "order_cancellations T";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "orders O ON O.order_id = T.order_id";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "users U ON U.id = T.user_id";
        if(!empty($user_id) && !empty($where)) {
            $sql .= " WHERE T." . $user_field . " = " . $user_id;
            $sql .= " AND " . $where;
        } else if(!empty($user_id)) {
            $sql .= " WHERE T.user_id = " . $user_id;
        }
        if($count == 0) {
            $sql .= " ORDER BY " . $sort_by;
            if ($limit != 0) {
                $sql .= " LIMIT " . $offset . ", " . $limit;
            } else if(!empty($limit)) {
                $sql .= " LIMIT $limit";
            }
            $result = $this->db->query ( $sql )->result();
            return $result;
        }  else {
            $result = $this->db->query ( $sql )->row();
            return $result ? $result->total_count : 0;
        }
    }
    
    /**
     * Display Cancellation Form to the User
     * 
     * Note: Order Items table is queried in case of a single product cancellation.
     * 
     * The user cannot cancel the entire order because each item will have their own refund policy.
     * 
     * @param string $module - bookings/order_items/orders
     * @param number $id
     * @param string $role
     * @param number $type - 0 -> User, 1 -> Member/Admin - Load Reasons
     */
    public function form($module, $id, $type = 0, $role = 'admin', $redirect_url = null) {
        $this->lang->load('bookings/bookings');
        if(empty($redirect_url)) {
            $redirect_url = $this->session->userdata('track_last_visited_page') ? $this->session->userdata('track_last_visited_page') : gc_url_format('members');
        }
        // check existence
        $id = (int) $id;
        $seller_id = 0;
        $item_id = $id;
        switch($module) {
            case 'bookings': // bookings can be free
                $module_id = gc_module_id($module);
                $where = array('module_id' => $module_id, 'row_id' => $id, 'status_id' => 2, 'deleted' => 0);
                $order_info = $this->db->where($where)->get('order_items')->row();
                break;
            case 'order_items':
                $order_info = $this->db->where('id', $id)->get($module)->row();
                break;
            case 'orders':
                $order_info = $this->db->where('order_id', $id)->get($module)->row();
                break;
        }
        if(!$order_info) {
            Template::redirect($redirect_url);
        }
        $loggedin_user_id = $this->auth->user_id();
        if(!$order_info && $order_info->user_id != $loggedin_user_id && $order_info->seller_id != $loggedin_user_id) { // A user can cancel his/her own order only.
            Template::redirect($redirect_url);
        }
        if($module == 'order_items' && $order_info->seller_id == $loggedin_user_id) {
            $type = 1;
        }
        
        // check for the refund duration
        if($module == 'order_items' && $order_info->user_id == $loggedin_user_id) {
            $this->load->model('orders/orders_model');
            if(!$this->orders_model->has_refund_policy($order_info->created_on, $order_info->refund_duration, 1)) {
                Template::redirect($redirect_url);
            }
        }
        
        // check cancellations request, if already submitted, redirect user to the previous page
        $order_cancellation = $this->db->where(array('order_id' => $order_info->order_id, 'order_item_id' => $order_info->id))->get('order_cancellations')->row();
        if($order_cancellation) {
            Template::set_message(lang('cancellation_process'), 'danger');
            Template::redirect($redirect_url);
        }
        if($module == 'bookings') {
            // get seller id from the allotments_table
            $allotment = $this->db->where('id', $order_info->row_id)->get('booking_allotments')->row();
            if($allotment) {
                $seller_id = $allotment->user_id;
            }
        } else if($module == 'order_items') { // Products - Order Items
            $seller_id = $order_info->seller_id;
        } else { // The user cancelled the entire order
            // The user cannot cancel the entire order because each item will have their own refund policy.
            $seller_id = 0;
        }
        if(isset($_POST['save'])) {
            $result = $this->save_request($module, $item_id, $order_info->order_id, $order_info, $seller_id, $role);
            if($result) {
                Template::set_message($result['message'], $result['badge']);
                if($result['badge'] == 'success') { 
                    if($module == 'bookings') {
                        $redirect_url = ($role == 'admin' ? $redirect_url : gc_url_format('bookings/view'));
                    } else {
                        if($this->session->userdata('last_visited_url')) {
                            $redirect_url = $this->session->userdata('last_visited_url') . '?order_id=' . $order_info->order_id;
                            $this->session->unset_userdata('last_visited_url');
                        } else if($redirect_url) {
                            $redirect_url = current_url();
                        }
                    }
                    Template::redirect($redirect_url);
                }
            }
        }
        $item = isset($order_info->form_data) ? $order_info->form_data : $order_info->item;
        Template::set(array(
            'module' => $module,
            'type' => $type == 0 ? 'buyer' : 'seller',
            'record' => $order_info,
            'item' => json_decode($item ?? ''),
            'toolbar_title' => lang('cancellation_form') . ' - ' . lang('label_order_id') . ': '. $order_info->order_id      
        ));
        Template::set_view('orders/form');
    }
    
    /**
     * Save request in the cancellations table
     * @param number $module_id
     * @param number $row_id
     */
    private function save_request($module, $id, $order_id, $order_info, $seller_id = 0, $role = 'member') {
        $this->load->library('form_validation');
        $this->form_validation->set_rules('reason', lang('label_reason'), 'trim|required');
        $this->form_validation->set_rules('comments', lang('label_comments'), 'trim');
        if($this->form_validation->run() === FALSE) {
            return false;
        }
        if(empty($order_info)) {
            $order_info = $this->db->where(array('order_id' => $order_id, 'status' => 1, 'deleted' => 0))->get('order_items')->row();
        }
        if(!empty($order_info)) {
            $user_id = $this->auth->user_id();
            if($role != 'admin' && $order_info->user_id != $user_id && $order_info->seller_id != $user_id) { //  || (!empty($order_info->seller_id) && $order_info->seller_id != $user_id))
                Template::redirect(site_url());
            }
            $refund_duration = 0;
            $refund_unit = 0;
            $row_id = $order_info->row_id;
            if($order_info->module_id == 29) { // Booking Order
                $sql = "SELECT BA.refund_duration, BA.refund_unit FROM " . $this->db->dbprefix . "bookings B";
                $sql .= " INNER JOIN " . $this->db->dbprefix . "booking_allotments BA ON BA.id = B.allotment_id";
                $sql .= " WHERE B.id = " . $order_info->row_id;
                $refund_info = $this->db->query($sql)->row();
                if($refund_info) {
                    $refund_duration = $refund_info->refund_duration;
                    $refund_unit = $refund_info->refund_unit;
                }
            } else {                
                // get refund duration and unit from the items row
                $item = json_decode($order_info->item ?? '', true);
                if(!empty($item['refund_duration'])) {
                    $refund_duration = $item['refund_duration'];
                    $refund_unit = isset($item['refund_unit']) ? $item['refund_unit'] : 1;
                }
                $row_id = $order_info->row_id; // Item ID generated in the ordered items table. 
            }
            $accept_request = true;
            if(!empty($refund_duration)) {
                $today = new DateTime(date('Y-m-d H:i:s'));
                $order_date = new DateTime($order_info->created_on);
                $interval= $order_date->diff($today);
                if($refund_unit == 0) { // Hours
                    $duration = $interval->h;
                    $duration_text = lang('label_hours');
                } else { // Days
                    $duration = $interval->days;
                    $duration_text = lang('label_days');
                }
                if($refund_duration > $duration) { // Process Refund
                    $accept_request = true;
                } else { // Refund cannot be processed, duration is over
                    // Display message to the user
                    return array('badge' => 'danger', 'message' => sprintf(lang('cancellation_success'), $refund_duration, $duration_text));
                }
            } else {
                $accept_request = true;
                // return array('badge' => 'danger', 'message' => lang('msg_no_refund_policy'));
            }
            if($accept_request) {
                // save information in the cancellations table
                $data = array(
                    'order_id' => $order_info->order_id,
                    'order_item_id' => $order_info->id,
                    'user_id' => $user_id,
                    'seller_id' => $seller_id,
                    'module_id' => $order_info->module_id,
                    'row_id' => $row_id,
                    'reason_id' => (int) $this->input->post('reason'),
                    'comments' => $this->input->post('comments'),
                    'status' => 0, // Under Process
                    'created_on' => date('Y-m-d H:i:s'),
                    'created_ip' => $this->input->ip_address(),
                    'created_by' => $user_id
                );
                $this->db->insert('order_cancellations', $data);
                
                // Cancel the order and refund the amount automatically
                $this->orders_model->update_status(0, $order_id, null, 4, 1, true, false, $order_info->id);
                // $this->orders_model->update_status(0, $order_id, null, 4, 1, true);
                
                // Give admin an option to manually approve a cancellation
                /*// set order status to cancelled to avoid further cancellation request of the same item
                 $this->db->where('id', $order_info->id)->update('orders', array('status' => 4));
                 if($order_info->order_type == 9) { // Bookings
                 // set the booking status to inactive
                 // order payouts and other amounts will be refunded on order_cancellation approval
                 $this->db->where('id', $order_info->row_id)->update('bookings', array('status' => 0));
                 }*/
                
                return array('badge' => 'success', 'message' => lang('cancellation_success'));
            }
        }
    }
    
    /**
     * Return Cancellation Reasons
     * @param string $module
     */
    public function reasons($key = 0, $module = 'bookings', $type = 'seller') {
        $reasons = array(
            '' => lang('label_select'),
            1 => lang('reason_01'),
            2 => lang('reason_02'),
            3 => lang('reason_03'),
            4 => lang('reason_04'),
            5 => lang('reason_05'),
            6 => lang('reason_06'),
            0 => lang('reason_other')
        );
        $bookings_buyer_reasons = array(
            '' => lang('label_select'),
            7 => lang('reason_07'),
            2 => lang('reason_02'),
            0 => lang('reason_other')
        );
        if(!empty($key)) {
            return $reasons[$key];
        } else {
            $return = array(
                'bookings' => array(
                    'seller' => $bookings_buyer_reasons,
                    'buyer' => $bookings_buyer_reasons
                ),
                'order_items' => array(
                    'seller' => $reasons,
                    'buyer' => $reasons
                )
            );
            return $return[$module][$type];
        }
    }
}