<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Invoices_model extends BF_Model {

	protected $table_name	= "order_invoices";
	protected $key			= "id";
	protected $soft_deletes	= TRUE;
	protected $date_format	= "datetime";

	protected $log_user 	= FALSE;

	protected $set_created	= true;
	protected $set_modified = true;
	protected $created_field = "created_on";
	protected $modified_field = "modified_on";

	/*
		Customize the operations of the model without recreating the insert, update,
		etc methods by adding the method names to act as callbacks here.
	 */
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 		= array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	/*
		For performance reasons, you may require your model to NOT return the
		id of the last inserted row as it is a bit of a slow method. This is
		primarily helpful when running big loops over data.
	 */
	protected $return_insert_id 	= TRUE;

	// The default type of element data is returned as.
	protected $return_type 			= "object";

	// Items that are always removed from data arrays prior to
	// any inserts or updates.
	protected $protected_attributes = array();

	/*
		You may need to move certain rules (like required) into the
		$insert_validation_rules array and out of the standard validation array.
		That way it is only required during inserts, not updates which may only
		be updating a portion of the data.
	 */
	protected $validation_rules 		= array(
		array(
			"field"		=> "order_id",
			"label"		=> "lang:label_order_id",
			"rules"		=> "trim|required"
		),
		array(
			"field"		=> "user_id",
			"label"		=> "lang:label_user",
			"rules"		=> "trim|required"
		),
		array(
			"field"		=> "due_date",
			"label"		=> "lang:label_due_date",
			"rules"		=> "trim|valid_date"
		),
		array(
			"field"		=> "date_paid",
			"label"		=> "lang:label_date_paid",
			"rules"		=> "trim|valid_date[TRUE]"
		),
		array(
			"field"		=> "next_invoice_date",
			"label"		=> "lang:label_next_invoice_date",
			"rules"		=> "trim|valid_date"
		),
		array(
			"field"		=> "first_reminder",
			"label"		=> "lang:label_first_reminder",
			"rules"		=> "trim|valid_date"
		),
		array(
			"field"		=> "second_reminder",
			"label"		=> "lang:label_second_reminder",
			"rules"		=> "trim|valid_date"
		),
		array(
			"field"		=> "third_reminder",
			"label"		=> "lang:label_third_reminder",
			"rules"		=> "trim|valid_date"
		),
		array(
			"field"		=> "status",
			"label"		=> "lang:label_status",
			"rules"		=> "trim|required"
		)
	);
	protected $insert_validation_rules 	= array();
	protected $skip_validation 			= FALSE;
	
	/**
	 * Get particular user invoices for member area
	 * @param integer $user_id
	 */
	public function get_invoices($user_id, $offset = 0, $limit = 25) {
		$sql = "SELECT INV.prefix, INV.id, O.currency_code, O.currency_value, O.items, ";
		$sql .= " INV.order_id, O.order_total, O.comments, INV.date_paid, INV.created_on, INV.status_id";
		$sql .= " FROM " . $this->db->dbprefix . "order_invoices INV";
		$sql .= " INNER JOIN " . $this->db->dbprefix . "orders O ON O.order_id = INV.order_id";
		$sql .= " WHERE INV.user_id = $user_id AND INV.deleted = 0 ORDER BY INV.id DESC";
		$sql .= " LIMIT " . $offset . ", " . $limit;
		$query = $this->db->query ( $sql );
		return $query->result_array ();
	}
	
	/**
	 * Update Status, Restore and Delete Records
	 * @param string $table - Table Name
	 * @param array $records - Checked Records
	 * @param string $field - Field to be Updated
	 * @param int $value - Value to be Set
	 */
	public function update_status($table, $records, $field, $value) {
		foreach($records as $record) {
			$data = array( array ( 'id' => $record, $field => $value ));
		}
		$this->db->update_batch($this->db->dbprefix . $table, $data, 'id');
	}
	
	/**
	 * PURGE - Permanently Delete Records
	 * @param string $table
	 * @param array $records
	 */
	public function purge($table, $records) {
		$records = implode(',',$records);
		$this->db->where_in('id', $records);
		$this->db->delete($this->db->dbprefix .$table);
	}

	//--------------------------------------------------------------------

}
