<?php defined('BASEPATH') || exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class Ordered_items_model extends BF_Model
{
    protected $table_name	= 'order_items';
    protected $table_items	= 'order_items';
    protected $table_shipments	= 'order_shipments';
	protected $key			= 'id';
	protected $date_format	= 'datetime';

	protected $log_user 	= false;
	protected $set_created	= false;
	protected $set_modified = false;
	protected $soft_deletes	= false;


	// Customize the operations of the model without recreating the insert,
    // update, etc. methods by adding the method names to act as callbacks here.
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 	    = array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	// For performance reasons, you may require your model to NOT return the id
	// of the last inserted row as it is a bit of a slow method. This is
    // primarily helpful when running big loops over data.
	protected $return_insert_id = true;

	// The default type for returned row data.
	protected $return_type = 'object';

	// Items that are always removed from data prior to inserts or updates.
	protected $protected_attributes = array();

	// You may need to move certain rules (like required) into the
	// $insert_validation_rules array and out of the standard validation array.
	// That way it is only required during inserts, not updates which may only
	// be updating a portion of the data.
	protected $validation_rules 		= array();
	protected $insert_validation_rules  = array();
	protected $skip_validation 			= false;

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
        $this->lang->load('orders/orders');
    }
    
    /**
     * Display a list of Ordered Items data.
     * @param string $role
     * @param number $user_id
     * @param number $offset
     * @param number $limit
     * @param boolean $is_seller - True Seller Items
     *
     * @return void
     */
    public function view($role = 'admin', $user_id = 0, $offset = 0, $limit = 25, $is_seller = true) {
        if($role == 'admin') {            
            $redirect_url = '/financial/orders/items';
            $pagerUriSegment = 5;
            $pagerBaseUrl = site_url(SITE_AREA . '/financial/orders/items') . '/';
        } else {            
            $redirect_url = gc_url_format('members');
            $pagerUriSegment = 4;
            $pagerBaseUrl = gc_url_format('members') . '/ordered_items/' . ($is_seller ? 1 : 0);
        }
        $like_array = array();
        $searchterm = $this->gc->search('ordered_items', $this->uri->segment($pagerUriSegment), $redirect_url);
        if(!empty($searchterm)) {
            // This is the basic search filter, you could add more fields to the search
            $like_array = array(
                'order_items.item' => $searchterm
            );
        }
        
        if(!empty($like_array)) {
            $this->ordered_items_model->or_like($like_array);
        }
        if(!empty($user_id)) {
            $this->ordered_items_model->where($is_seller ? 'seller_id' : 'user_id', $user_id);
        }
        // display products only
        $where = array('module_id' => 2);
        if(!empty($is_seller)) {
            // $where['status_id'] = 2; // display all items to the seller
        }
        $total_rows = $this->ordered_items_model->where($where)->count_all();
        $this->load->library('pagination');
        $pager['base_url']    = $pagerBaseUrl;
        $pager['total_rows']  = $total_rows;
        $pager['per_page']    = $limit;
        $pager['uri_segment'] = $pagerUriSegment;
        
        $this->pagination->initialize($pager);
        $this->ordered_items_model->limit($limit, $offset);
        
        $order_by = $this->gc->sort('ordered_items');
        if(!empty($order_by)) {
            $this->ordered_items_model->order_by($this->session->userdata('ordered_items_sort')['column'], $this->session->userdata('ordered_items_sort')['order']);
        } else {
            $this->ordered_items_model->order_by('id', 'DESC');
        }
        
        if(!empty($like_array)) {
            $this->ordered_items_model->or_like($like_array);
        }
        $this->ordered_items_model->join('orders O', 'O.order_id = order_items.order_id', 'LEFT');
        $this->ordered_items_model->join('order_customers OC', 'OC.order_id = order_items.order_id', 'LEFT');
        $this->ordered_items_model->join('users U', 'U.id = order_items.seller_id', 'LEFT');
        if(!empty($is_seller)) {
            // $where = array('order_items.status_id' => 2);
            $where = array();
        } else {
            $where = array();
        }
        if(!empty($user_id)) {
            $where[$is_seller ? 'order_items.seller_id' : 'order_items.user_id'] = $user_id;
        }
        $this->db->where($where);
        // display products only
        $this->db->join('order_statuses S', 'S.id = order_items.status_id', 'LEFT');
        $records = $this->ordered_items_model->select('O.created_on, U.display_name, U.email AS user_email, order_items.*, S.name AS status, S.name_lang AS status_lang, S.color AS status_color, OC.name AS customer_name, OC.address AS customer_address, OC.postcode AS customer_postcode, OC.city AS customer_city, OC.state AS customer_state, OC.country AS customer_country, OC.phone_number AS customer_phone_number')->find_all_by(array('module_id' => 2));
        
        Template::set(array(
            'is_seller' => $is_seller,
            'records' => $records,
            'toolbar_title' => lang('manage_ordered_items'),
            'role' => $role
        ));
        Template::set_view('orders/items');
    }    
    
    /******************************************************************
     * START OF ORDERED ITEMS
     ******************************************************************/
    /**
     * Shipment Status
     * @param number $id
     * @return string|string[]
     */
    public function shipment_status($id = 0) {
        $shipment_status = array(
            1 => lang('label_packed'),
            2 => lang('label_shipped'),
            3 => lang('label_in_delivery'),
            4 => lang('label_delivered'),
        );
        if(!empty($id)) {
            return $shipment_status[$id];
        } else {
            return $shipment_status;
        }
    }
    
    /**
     * Order Shipments - Member and Admin Area
     * @param string $order_id
     */
    public function shipments($role, $order_item_id, $redirect_url) {
        $this->lang->load('orders/orders');
        $this->load->helper('text');
        $order_item_id = (int) $order_item_id;
        if (empty($order_item_id)) {
            // Template::set_message(lang('invalid_id'), 'danger');
            // redirect($redirect_url);
        }
        $role_id = $this->auth->role_id();
        $user_id = $this->auth->user_id();
        $order = $this->db->get_where($this->table_items, array('id' => $order_item_id, 'module_id' => 2, 'status_id' => 2, 'deleted' => 0))->row();
        if(!$order) {
            Template::redirect($redirect_url);
        }
        // Only Product Owner or Site Admin can set shipment status
        if(!$order || ($order->seller_id != $user_id && $role_id != 1)) { // Invalid Order ID
            // Template::redirect($redirect_url);
        }
        if(isset($_POST['save'])) {
            $shipment_status = $this->input->post('status');
            $data = array(
                'order_item_id' => $order_item_id,
                'order_id' => $order->order_id,
                'description' => $this->input->post('description'),
                'status' => $shipment_status,
                'created_on' => date('Y-m-d H:i:s'),
                'created_by' => $user_id,
                'created_ip' => $this->input->ip_address()
            );
            $this->db->insert($this->table_shipments, $data);
            // Also update shipment status in the ordered_items table
            $this->db->where('id', $order_item_id)->update($this->table_items, array('shipment_status' => $shipment_status));
            Template::redirect(current_url());
        }
        
        if(isset($_POST['delete'])) {
            $checked = $this->input->post('checked');
            if (is_array($checked) && count($checked)) {
                foreach ($checked as $pid) {
                    // delete information from the shipments table
                    $result = $this->db->where('id', $pid)->delete($this->table_shipments);
                }
                // Also update shipment status in the ordered_items table
                // get last status saved for this item id
                $shipment_status = 0;
                $row = $this->db->select('status')->order_by('id', 'DESC')->get_where($this->table_shipments, array('order_item_id' => $order_item_id))->row();
                if($row) {
                    $shipment_status = $row->status;
                }
                $this->db->where('id', $order_item_id)->update($this->table_items, array('shipment_status' => $shipment_status));
                $message_module = count($checked) == 1 ? lang('label_shipment') : lang('label_shipments');
                Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $message_module, lang('label_deleted')), 'success');
                Template::redirect(current_url());
            }
        }
        $records = $this->db->get_where($this->table_shipments, array('order_item_id' => $order_item_id))->result();
        Template::set(array(
            'records' => $records,
            'cancel_url' => $redirect_url,
            'toolbar_title' => lang('manage_shipments'),
            'role' => $role
        ));
        Template::set_view('orders/shipments');
    }
    
    /**
     * Allow user to Track an Order
     * @param number $order_item_id
     * @param string $order_id
     * @param string $redirect_url
     */
    public function track($order_item_id = 0, $order_id = null, $redirect_url = null, $role = 'admin') {
        if(empty($_POST)) {
            $cancellation_redirect_url = current_url() . (isset($_SERVER['QUERY_STRING']) ? '?' . $_SERVER['QUERY_STRING'] : '');
            $this->session->set_userdata('track_last_visited_page', $cancellation_redirect_url); // cancellation redirect url
        }
        if(isset($_POST['cancel_item'])) {
            $this->session->set_userdata('last_visited_url', current_url());
            Template::redirect(gc_url_format('members/cancellation') . '/' . $_POST['cancel_item']);
        }
        // allow user to search a specific item
        if(empty($order_item_id) && empty($order_id)) {
            if(isset($_POST['submit'])) {
                $this->form_validation->set_rules('order_item_id', 'lang:track_order_item_id', 'trim|required');
                if ($this->form_validation->run() === true) {
                    $track_order_item_id = escape($_POST['order_item_id']);
                    if(is_numeric($track_order_item_id)) {
                        $orders_item_count = $this->db->where('id', $track_order_item_id)->count_all_results('order_items');
                        if($orders_item_count > 0) {
                            $order_item_id = $track_order_item_id;
                        }
                    } else {
                        // check from the orders table first
                        $orders_count = $this->db->where('order_id', $track_order_item_id)->count_all_results('orders');
                        if($orders_count > 0) {
                            $order_id = $track_order_item_id;
                        }
                    }
                    if(empty($order_item_id) && empty($order_id)) {
                        Template::set_message(lang('error_no_record_found'), 'danger');
                    }
                }
            }
            Template::set(array(
                'toolbar_title' => lang('track_shipment')
            ));
        }
        if(!empty($order_item_id) || !empty($order_id)) {
            $module_id = gc_module_id('products');
            if(!empty($order_item_id)) { // Single Item Info
                // Get items from the order_items table based on the Ordered Item ID.
                $ordered_items = $this->get_order_items('M.id = ' . $order_item_id . ' AND M.module_id = 2');
                if(!$ordered_items) {
                    Template::redirect(site_url());
                }
                $order_id = $ordered_items[0]->order_id;
                $shipments = $this->db->get_where($this->table_shipments, array('order_item_id' => $order_item_id))->result();
            } else {
                // Get items from the order_items table based on the Order ID.
                $ordered_items = $this->get_order_items("M.order_id = '" . $order_id . "'");
                if(!$ordered_items) {
                    Template::redirect($redirect_url);
                }
                $shipments = $this->db->get_where($this->table_shipments, array('order_id' => $order_id))->result();
            }
            $order = (object) $this->orders_model->get_order($order_id);
            // Only Product Owner or Site Admin can set shipment status
            //if(!$order || $order->module_id != $module_id) { // Invalid Order ID
            if(!$order || ($this->auth->role_id() != 1 && ($order->user_id != $this->auth->user_id()))) { // restrict user to track only his/her order_item_id
                Template::redirect($redirect_url);
            }
            
            $order = (object) $order;
            $this->load->model('locations/cities_model');
            $location = $this->cities_model->get_city_info($order->city_id);
            Template::set(array(
                'redirect_url' => $role == 'admin' ? site_url(SITE_AREA . '/financial/orders/view/' . $order->order_id . '/1') : gc_url_format('members/detail') . '/' . $order->order_id,
                'items' => $ordered_items,
                'shipments' => $shipments,
                'order' => $order,
                'location' => $location,
                'toolbar_title' => lang('track_shipment') . ' - ' . $order_id
            ));
        }
        Template::set('role', $role);
        Template::set_view('orders/track');
    }
    
    /**
     * Allow user to cancel the ordered items using GET parameters
     * @param integer $item_id - Ordered Item ID
     * @param string $order_id
     */
    public function cancel($order_item_id, $order_id, $role = 'admin') {
        $redirect_url = $role == 'admin' ? site_url(SITE_AREA . '/financial/orders/manage/items') : site_url('members/manage/items');
        $user_id = $this->auth->user_id();
        $item_record = array();
        if($role == 'admin') {
            $this->auth->restrict('Orders.Financial.Cancel');
        }
        if(!empty($order_item_id)) {
            $records = $this->get_order_items('M.id = ' . $order_item_id);
            $item_record = $this->db->where('order_item_id', $order_item_id)->get('order_cancellations')->row();
        } else if(!empty($order_id)) {
            $records = $this->get_order_items("M.order_id = '" . $order_id . "'");
        }
        if(!$records) {
            Template::redirect($redirect_url);
        }
        if($role != 'admin' && ($records[0]->user_id != $user_id || $records[0]->seller_id != $user_id)) {
            Template::redirect($redirect_url);
        }
        $order_id = $records[0]->order_id;
        if(isset($_POST['save'])) {
            $this->form_validation->set_rules('reason_id', 'lang:singular_order_cancellation_reasons', 'trim|required');
            if ($this->form_validation->run() === true) {
                $datetime = date('Y-m-d H:i:s');
                $insert_data = array();
                $update_data = array();
                foreach($records as $record) {
                    $where = array(
                        'order_id' => $record->order_id,
                        'order_item_id' => $record->id,
                        'seller_id' => $record->seller_id,
                        'user_id' => $record->user_id
                    );
                    // check whether the cancellation request already exists or not
                    $row = $this->db->where($where)->get('order_cancellations')->row();
                    if(!$row) {
                        $insert_data[] = array(
                            'order_id' => $record->order_id,
                            'order_item_id' => $record->id,
                            'seller_id' => $record->seller_id,
                            'user_id' => $record->user_id,
                            'module_id' => $record->module_id,
                            'row_id' => $record->row_id,
                            'reason_id' => $this->input->post('reason_id'),
                            'comments' => $this->input->post('comments'),
                            'status' => $role == 'admin' ? 1 : 0,
                            'created_on' => $datetime,
                            'created_by' => $this->auth->user_id(),
                            'created_ip' => $this->input->ip_address(),
                            'modified_on' => $datetime,
                            'modified_ip' => $this->input->ip_address()
                        );
                    } else {
                        $update_data[] = array(
                            'id' => $row->id,
                            'reason_id' => $this->input->post('reason_id'),
                            'comments' => $this->input->post('comments'),
                            'modified_on' => $datetime,
                            'modified_by' => $this->auth->user_id(),
                            'modified_ip' => $this->input->ip_address()
                        );
                    }
                }
                if(!empty($insert_data)) {
                    $this->db->insert_batch('order_cancellations', $insert_data);
                }
                if(!empty($update_data)) {
                    $this->db->update_batch('order_cancellations', $update_data, 'id');
                }
                
                // update item status to cancelled
                $this->db->where('id', $record->id)->update('order_items', array('status_id' => 4, 'modified_on' => date('Y-m-d H:i:s'), 'modified_ip' => $this->input->ip_address()));
                
                Template::set_message(sprintf(lang('msg_save_success'), lang('order_cancellations')), 'success');
                Template::redirect($redirect_url);
            }
        }
        $reasons = array();
        $cancellation_reasons = $this->db->where(array('status' => 1, 'deleted' => 0))->get('order_cancellation_reasons')->result();
        if($cancellation_reasons) {
            foreach($cancellation_reasons as $cancellation_reason) {
                $reasons[$cancellation_reason->id] = !empty($cancellation_reason->name_lang) ? lang($cancellation_reason->name) : $cancellation_reason->name;
            }
        }
        $this->db->from('orders O');
        $this->db->join('order_customers OC', 'OC.order_id = O.order_id', 'LEFT');
        $order = $this->db->select('O.id, O.order_id, O.created_on, O.order_total, O.currency_code, OC.name AS customer_name, OC.address, OC.city, OC.state, OC.postcode, OC.country, OC.email, OC.phone_number')->where('O.order_id', $order_id)->get()->row();
        Template::set(array(
            'toolbar_title' => lang('order_cancellations'),
            'items' => $records,
            'reasons' => $reasons,
            'order' => $order,
            'redirect_url' => $redirect_url,
            'record' => $item_record,
            'role' => $role
        ));
        Template::set_view('orders/cancel');
        Template::render();
    }
        
    /**
     * Get Order Items
     * @param string $where
     * @return object
     */
    private function get_order_items($where) {
        $sql = "SELECT M.*, S.name AS status, S.name_lang AS status_lang, S.color AS status_color";
        $sql .= " FROM " . $this->db->dbprefix . "order_items M";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "order_statuses S ON S.id = M.status_id";
        $sql .= " WHERE " . $where;
        $records = $this->db->query($sql)->result();
        return $records;
    }
    
    /**
     * Update Order status on item status update
     * Called on order cancellation and wallet refund
     * @param number $item_id
     * @param array $order_item_info
     */ 
    public function update_order_item_status($item_id, $order_item_info = array()) {
        if(empty($order_item_info)) {
            $order_item_info = $this->db->where('id', $item_id)->get('order_items')->row();
        }
        if($order_item_info) {
            // get number of items related to this order
            $status = array();
            $ordered_items = $this->db->where('order_id', $order_item_info->order_id)->get('order_items')->result();
            if($ordered_items) {
                foreach($ordered_items as $ordered_item) {
                    $status[$ordered_item->status_id] = $ordered_item->status_id;
                }
            }
            if(count($status) == 1) {
                $order_status_id = reset($status);
                $order_info = $this->db->where('order_id', $order_item_info->order_id)->get('orders')->row();
                $this->db->where('order_id', $order_item_info->order_id)->update('orders', array('status_id' => $order_status_id));
                // update order history
                $this->load->model('orders/orders_model');
                $this->orders_model->save_order_history($order_item_info->order_id, $order_info->status_id, $order_status_id, 0);
            }
        }
    }
    /******************************************************************
     * END OF ORDERED ITEMS
     ******************************************************************/
}