<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Packages controller
 */
class Financial extends Admin_Controller
{
	protected $permissionCreate = 'Packages.Financial.Create';
	protected $permissionDelete = 'Packages.Financial.Delete';
	protected $permissionEdit   = 'Packages.Financial.Edit';
	protected $permissionView   = 'Packages.Financial.View';
	protected $modules 			= array();

	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct()
	{
		parent::__construct();
        clear_cache('packages');
		$this->auth->restrict($this->permissionView);
		$this->load->model('packages_model', null, true);
		$this->lang->load('packages');
		$this->lang->load('listings/listings');
		
		// Date Time Picker
		Assets::add_js('moment.js');
		Assets::add_css ( 'bootstrap-datetimepicker.min.css' );
		Assets::add_js ( 'bootstrap-datetimepicker.min.js' );
		
		Template::set_block('sub_nav', 'financial/_sub_nav');
		
		$this->modules = gc_modules('module', false);
	}

	/**
	 * Displays a list of packages to the admin
	 *
	 * @return void
	 */
	public function index($offset = 0)
	{
	    $this->session->set_userdata('packages_last_visited_page', current_url());
		// Deleting anything?
		if (isset($_POST['delete'])) {
			$checked = $this->input->post('checked');

			if (is_array($checked) && count($checked)) {
				$result = FALSE;
				foreach ($checked as $pid) {
				    $result = $this->db->where('id', $pid)->update('packages', array('deleted' => 1)); // $this->gc->purge_with_translations('packages', $pid);
				}

				if ($result) {
					Template::set_message(count($checked) .' '. sprintf(lang('msg_delete_success'), lang('label_packages')), 'success');
				} else {
					Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_packages')) . $this->packages_model->error, 'danger');
				}
			}
		}
		
		// Highlight Package
		if (isset($_POST['highlight'])) {
			$this->packages_model->highlight($_POST['highlight'], 'packages', 1);
		}
		
		// Status Change
		if (isset($_POST['status_change'])) {
			$this->gc->status_change('packages', $_POST['status_change'], 'active');
		}
		
		// Set Default
		if (isset($_POST['set_default'])) {
		    $this->set_default($_POST['set_default']);
		}
		
		// Sort By
		$order_by = $this->gc->sort('packages');
		if(!empty($order_by)) {
			$this->packages_model->order_by($this->session->userdata('packages_sort')['column'], $this->session->userdata('packages_sort')['order']);
		} else {
			$this->packages_model->order_by('id', 'desc');
		}
		$this->packages_model->limit($this->limit, $offset);
		$records = $this->packages_model->find_all_by(array('deleted' => 0));
		
		// Pagination
		$this->load->library('pagination');
		
		$total_packages = $this->packages_model->count_all();;
		
		$pager['base_url'] = site_url(SITE_AREA ."/financial/packages/index/");
		$pager['total_rows'] = $total_packages;
		$pager['per_page'] = $this->limit;
		$pager['uri_segment']	= 5;
		
		$this->pagination->initialize($pager);

		Template::set('records', $records);
		Template::set('toolbar_title', lang('manage_packages'));
		Template::render();
	}

	/**
	 * Allow admin to create a new package for member
	 *
	 * @return void
	 */
	public function create()
	{
	    // Color Picker
	    Assets::add_css('bootstrap-colorpicker.min.css');
	    Assets::add_js('bootstrap-colorpicker.min.js');
	    Assets::add_js('$(".gc-colorpicker").colorpicker();', 'inline');
	    
	    // FontAwesome Icon Picker
		Assets::add_css('fontawesome-iconpicker.min.css');
		Assets::add_js('fontawesome-iconpicker.min.js');
		
		// user search js and css dependencies
		Assets::add_css('selectize.bootstrap3.css');
		Assets::add_js('selectize.min.js');
		Assets::add_js('selectize-search.js');
		$this->auth->restrict($this->permissionCreate);
		
		// create and edit form common
		$common_fields = $this->package_form();
		
		if (isset($_POST['save']) || isset($_POST['save_only'])) {
		    $this->gcrud->stats(array('module' => 'packages'));
		    if ($insert_id = $this->save_packages('insert', 0, $common_fields['module_form_fields'], $common_fields['module_display_fields'])) {
				// Log the activity
				log_activity($this->auth->user_id(), lang('act_create_record') .': '. $insert_id .' : '. $this->input->ip_address(), 'packages');

				Template::set_message(sprintf(lang('msg_save_success'), lang('label_package')), 'success');
				if(isset($_POST['save'])) {
				    $redirect_url = $this->session->userdata('packages_last_visited_page') ? $this->session->userdata('packages_last_visited_page') : SITE_AREA . '/financial/packages';
				    Template::redirect($redirect_url);
				} else {
				    Template::redirect(current_url());
				}
			} else {
				Template::set_message(sprintf(lang('msg_create_failure'), lang('label_package')) . $this->packages_model->error, 'danger');
			}
		}
		Assets::add_module_js('packages', 'packages.js');
		$upgrade_valid_for =	$this->packages_model->select('id,title')->find_all_by(array('deleted' => 0));
		$invoicing_packages = $this->packages_model->select('id,title')->find_all_by(array('price !=' => 0, 'deleted' => 0));
		// PLAN TYPE POSTBACK HANDLING
		/*if($this->input->post('plan_type')) {
			Template::set('plan_type', $this->input->post('plan_type'));
			if($this->input->post('plan_type') == 'upgrade') {
				Template::set('upgrade_plan', $this->input->post('upgrade_valid_for'));
			}
		}*/
		$this->applicable_categories();
		Template::set(array(
		    'modules' => $this->modules,
			'order_status' => ($this->input->post('order_status') ? $this->input->post('order_status') : 0),
			'invoicing_packages' => $invoicing_packages,
			'upgrade_valid_for' => $upgrade_valid_for,
			'toolbar_title' => lang('create_package'),
		));
		Template::set_view('financial/packages_form');
		Template::render();
	}

	/**
	 * Allows editing of Packages data.
	 *
	 * @return void
	 */
	public function edit() {
	    // Color Picker
	    Assets::add_css('bootstrap-colorpicker.min.css');
	    Assets::add_js('bootstrap-colorpicker.min.js');
	    Assets::add_js('$(".gc-colorpicker").colorpicker();', 'inline');
	    // Fontawesome Icons Picker
		Assets::add_css('fontawesome-iconpicker.min.css');
		Assets::add_js('fontawesome-iconpicker.min.js');
		// user search js and css dependencies
		Assets::add_css('selectize.bootstrap3.css');
		Assets::add_js('selectize.min.js');
		Assets::add_js('selectize-search.js');
		Assets::add_module_js('packages', 'packages.js');
		
		$id = $this->uri->segment(5);

		if (empty($id)) {
			Template::set_message(lang('packages_invalid_id'), 'error');
			redirect(SITE_AREA .'/financial/packages');
		}
		$package = $this->packages_model->find($id);
		
		// create and edit form common
		$common_fields = $this->package_form();
		
		if (isset($_POST['save']) || isset($_POST['save_only'])) {
		    $this->gcrud->stats(array('module' => 'packages'));
			$this->auth->restrict($this->permissionEdit);

			if ($this->save_packages('update', $id, $common_fields['module_form_fields'], $common_fields['module_display_fields'])) {
				// Clear trusted seal
				$this->packages_model->clear_trusted_seal($id, 'packages');		
				// Log the activity
				log_activity($this->auth->user_id(), lang('act_edit_record') .': '. $id .' : '. $this->input->ip_address(), 'packages');

				Template::set_message(sprintf(lang('msg_save_success'), lang('label_package')), 'success');
				// Set package default
				$package_updated = $this->packages_model->select('is_default, priority')->find($id);				
				if($package_updated->is_default == 1) {
				    $this->set_default($id);
				}
				
				// Tweak module's package_priority column
				$this->packages_model->_tweak_changed_priority($id, $package->priority, $package_updated->priority);				
				if(isset($_POST['save'])) {
				    $redirect_url = $this->session->userdata('packages_last_visited_page') ? $this->session->userdata('packages_last_visited_page') : SITE_AREA . '/financial/packages';
				    Template::redirect($redirect_url);
				} else {
				    Template::redirect(current_url());
				}
				//redirect(SITE_AREA .'/financial/packages/edit/' .$id);
			} else {
			    $form_validation = $this->session->flashdata('invalid_commission');
			    if($form_validation == 1) {
			        Template::set_message(lang('invalid_commission'), 'danger');
			    } else {
				    Template::set_message(sprintf(lang('msg_edit_failure'), lang('label_package')) . $this->packages_model->error, 'danger');
			    }
			}
		} else if (isset($_POST['delete'])) {
			$this->auth->restrict($this->permissionDelete);

			if ($this->db->where('id', $id)->update('packages', array('deleted' => 1))) { // $this->gc->purge_with_translations('packages', $id)
				// Log the activity
				log_activity($this->auth->user_id(), lang('act_delete_record') .': '. $id .' : '. $this->input->ip_address(), 'packages');
				
				Template::set_message(sprintf(lang('msg_delete_success'), lang('label_package')), 'success');

				redirect(SITE_AREA .'/financial/packages');
			} else {
				Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_package')) . $this->packages_model->error, 'danger');
			}
		}
		$this->db->where_not_in('id', $id);
		$upgrade_valid_for =	$this->packages_model->select('id,title')->find_all();
		$invoicing_packages = $this->packages_model->select('id,title')->where_not_in('id', $id)->find_all_by(array('price !=' => 0, 'deleted' => 0));
		
		// PLAN TYPE POSTBACK HANDLING
		/*if($this->input->post('plan_type')) {
			Template::set('plan_type', $this->input->post('plan_type'));
			if($this->input->post('plan_type') == 'upgrade') {
				Template::set('upgrade_plan', $this->input->post('upgrade_valid_for'));
			}
		}*/
		
		// Multi Langauge Fields - Get translations if exists
		$package = $this->translations->get_translations('packages', $id, $package);
		
		$downgrade_packages = $this->packages_model->select('id, title')->where_not_in('id', $id)->find_all_by(array('price <' => $package->price, 'deleted' => 0));
		$this->applicable_categories($package->applicable_categories);
		Template::set(array(
		    'modules' => $this->modules,
			'module_limit' => json_decode($package->module_limit ?? ''),
			'module_status' => json_decode($package->module_status ?? ''),
			'module_featured' => json_decode($package->module_featured ?? ''),
		    'module_categories_limit' => json_decode($package->module_categories_limit ?? ''),
		    'module_documents_limit' => json_decode($package->module_documents_limit ?? ''),
			'module_images_limit' => json_decode($package->module_images_limit ?? ''),
			'module_images_status' => json_decode($package->module_images_status ?? ''),
			'module_videos_limit' => json_decode($package->module_videos_limit ?? ''),
		    'module_videos_status' => json_decode($package->module_videos_status ?? ''),
		    'module_views_limit' => json_decode($package->module_views_limit ?? ''),
		    'commissions' => (array) json_decode($package->commissions ?? ''),
			'pyg' => json_decode($package->pyg ?? ''),
			'order_status' => ($this->input->post('order_status') ? $this->input->post('order_status') : $package->order_status),
			'invoicing_packages' => $invoicing_packages,
			'downgrade_packages' => $downgrade_packages,
			'upgrade_valid_for' => $upgrade_valid_for,
			'package' => $package,
		    'form_fields' => json_decode($package->form_fields ?? ''),
		    'display_fields' => json_decode($package->display_fields ?? ''),
		    'loggedin_display_fields' => json_decode($package->loggedin_display_fields ?? ''),
		    'special_display_fields' => json_decode($package->special_display_fields ?? ''),
			'toolbar_title' => lang('edit_package')
		));
		Template::set_view('financial/packages_form');
		Template::render();
	}
	
	/**
	 * Package Form common fields
	 * @return array[]
	 */
	private function package_form() {
	    $site_modules = gc_modules(); 
	    // Module Form Fields
	    $this->load->model('fields/fields_model');
	    foreach($site_modules as $site_module) {
	        $module_form_fields[$site_module] = $this->fields_model->package_form_fields($site_module, 'field_package', 1);
	        $module_display_fields[$site_module] = $this->fields_model->package_form_fields($site_module, 'display_package', 1);
	    }
	    $data = array(
	        'site_modules' => $site_modules,
	        'module_form_fields' => $module_form_fields,
	        'module_display_fields' => $module_display_fields
	    );
	    Template::set($data);
	    return $data;
	}
	
	/**
	 * Display selected categories to the user on POST validation and Package Edit
	 * @param array $selected - Selected Categories
	 */
	private function applicable_categories($selected = array()) {
		$selected_categories = $this->input->post('applicable_categories') ? $this->input->post('applicable_categories') : (!empty($selected) ? $selected : array());
		// REMOVE quotes at the front and beginning of $selected text string using trim($selected_categories, '"');
		if(is_string($selected_categories) && $selected_categories != '"\"\""') {
			$selected_categories = trim($selected_categories, '"');
			if(!empty($selected_categories)) {			
				$applicable_categories = $this->db->select('id, name')->where('id IN (' . trim($selected_categories, '"') . ')')->get('categories')->result();
				Template::set('applicable_categories', $applicable_categories);
			}
		}
	}

	//--------------------------------------------------------------------
	// !PRIVATE METHODS
	//--------------------------------------------------------------------

	/**
	 * Summary
	 *
	 * @param String $type Either "insert" or "update"
	 * @param Int	 $id	The ID of the record to update, ignored on inserts
	 *
	 * @return Mixed    An INT id for successful inserts, TRUE for successful updates, else FALSE
	 */
	private function save_packages($type='insert', $id=0, $module_form_fields = array(), $module_display_fields = array())
	{
		if ($type == 'update') {
			$_POST['id'] = $id;
		}
		// validate translation fields
		$this->translations->validate_prepare_fields();
		
		$this->form_validation->set_rules('uploaded_image', 'lang:label_trusted_seal', 'trim');
		$this->form_validation->set_rules('price', 'lang:label_price', 'trim|is_greater_than[offer_price]');
		//Check whether the image is uploaded or not
		if(isset($_FILES['trusted_seal']) && $_FILES['trusted_seal']['size'] > 0) {
			$file_data = $this->packages_model->upload_trusted_seal();
			if(!$file_data) {
				$_POST['file_upload'] = 'NONE';
				$this->form_validation->set_rules('file_upload', 'label:label_trusted_seal', 'valid_file_upload');
			}
			// Avoid image upload process, if the form has validation issues with other fields
			$_POST['uploaded_image'] = isset($file_data['upload_data']['file_name']) ? $file_data['upload_data']['file_name'] : '';
		}
		if ($this->form_validation->run() === false) {
			return false;
		}
		
		// unset non existing translation POST fields
		$this->translations->translate_unset();
		
		// make sure we only pass in the fields we want
		$data = array();
		
		// Basic Settings Tab
		$data['title']        			= $this->input->post('title');
		$data['description']        	= $this->input->post('description');
		$data['icon']        			= $this->input->post('icon');
		$data['is_default']        		= $this->input->post('is_default') ? 1 : 0;
		$data['plan_type']        		= 'new'; //$this->input->post('plan_type');
		$data['upgrade_valid_for'] 		= $this->input->post('upgrade_valid_for') ? $this->input->post('upgrade_valid_for') : 0;
		$data['claim_status']        	= $this->input->post('claim_status') ? $this->input->post('claim_status') : 0;
		$data['order_status']        	= $this->input->post('order_status') ? $this->input->post('order_status') : 0;
		$data['active']					= $this->input->post('active') ? $this->input->post('active') : 0;
		
		// Price & Durations Tab
		$data['duration']        		= $this->input->post('duration') ? $this->input->post('duration') : '0';
		$data['recurring']        		= $this->input->post('recurring') ? 1 : 0;
		$data['renewable']        		= $this->input->post('renewable') ? 1 : 0;
		$data['days_before_next_invoice']= $this->input->post('days_before_next_invoice');
		$data['invoice_expiry']			= $this->input->post('invoice_expiry');
		$data['invoice_first_reminder'] = $this->input->post('invoice_first_reminder');
		$data['invoice_second_reminder']= $this->input->post('invoice_second_reminder');
		$data['invoice_third_reminder'] = $this->input->post('invoice_third_reminder');
		$data['fixed_date']   			= $this->input->post('fixed_date') ? $this->input->post('fixed_date') : NULL;
		$data['forever']        		= $this->input->post('forever') ? 1 : 0;
		$data['price']        			= $this->input->post('price') ? $this->input->post('price') : '0';
		$data['offer_price']        	= $this->input->post('offer_price') ? $this->input->post('offer_price') : '0';
		$data['claim_price']        	= $this->input->post('claim_price') ? $this->input->post('claim_price') : '0';
		$data['downgrade_package']		= $this->input->post('downgrade_package');
		
		// Commissions Fields
		// if($this->input->post('commissions')) {
		    $commissions = $this->input->post('commissions');
		    if($commissions['products']['value'] != 999 && $commissions['products']['value'] > 100) {
		        $this->session->set_flashdata('invalid_commission', 1);
		        return false;
		    }
		    $data['commissions'] = $this->input->post('commissions') ? json_encode($this->input->post('commissions')) : null;
		//}		
		
		// Limit Tab
		$data['keywords_limit']			= $this->input->post('keywords_limit') ? $this->input->post('keywords_limit') : 0;
		$data['keywords_length']		= $this->input->post('keywords_length') ? $this->input->post('keywords_length') : 0;
		$data['description_limit']		= $this->input->post('description_limit') ? $this->input->post('description_limit') : 0;
		$data['module_limit']			= $this->input->post('module_limit') ? json_encode($this->input->post('module_limit')) : null;
		
		// Images Limit
		$data['module_images_limit']	= $this->input->post('module_images_limit') ? json_encode($this->input->post('module_images_limit')) : null;
		
		// Videos Limit
		$data['module_videos_limit']	= $this->input->post('module_videos_limit') ? json_encode($this->input->post('module_videos_limit')) : null;
		
		// Categories Limit
		$data['module_categories_limit'] = $this->input->post('module_categories_limit') ? json_encode($this->input->post('module_categories_limit')) : null;
		
		// Documents Limit
		$data['module_documents_limit'] = $this->input->post('module_documents_limit') ? json_encode($this->input->post('module_documents_limit')) : null;
		
		// Featured Tab
		$data['module_featured']		= $this->input->post('module_featured') ? json_encode($this->input->post('module_featured')) : null;	

		// Auto Approve Tab
		$data['module_status']			= $this->input->post('module_status') ? json_encode($this->input->post('module_status')) : null;
		$data['module_images_status']	= $this->input->post('module_images_status') ? json_encode($this->input->post('module_images_status')) : null;
		$data['module_videos_status']	= $this->input->post('module_videos_status') ? json_encode($this->input->post('module_videos_status')) : null;
		
		// Module Views Limit
		$data['module_views_limit'] = $this->input->post('module_views_limit') ? json_encode($this->input->post('module_views_limit')) : null;
		
		// Applicable Categories
		$data['applicable_categories']	= $this->input->post('applicable_categories') ? json_encode($this->input->post('applicable_categories')) : null;		
		
		// PYG Fields
		$data['pyg'] = $this->input->post('pyg') ? json_encode($this->input->post('pyg')) : null;
		
		// Form Fields
		$form_fields = array();
		if(!empty($module_form_fields)) {
    		foreach($module_form_fields as $field_module_name => $field_module_values) {
    		    foreach($field_module_values as $field_module_value) {
    		        $form_fields[$field_module_name][$field_module_value['name']] = gc_field_value($field_module_value['name'], json_encode(isset($_POST['form_' . $field_module_name]) ? $_POST['form_' . $field_module_name] : array()));
    		    }
    		}
		}
		$data['form_fields'] 			= !empty($form_fields) ? json_encode($form_fields) : null;
		
		// Display Fields
		$display_fields = array();
		$loggedin_display_fields = array();
		$special_display_fields = array();
		if(!empty($module_display_fields)) {
		    foreach($module_display_fields as $display_module_name => $display_module_values) {
		        foreach($display_module_values as $display_module_value) {
		            $display_fields[$display_module_name][$display_module_value['name']] = gc_field_value($display_module_value['name'], json_encode(isset($_POST['display_' . $display_module_name]) ? $_POST['display_' . $display_module_name] : array()));
		            $loggedin_display_fields[$display_module_name][$display_module_value['name']] = gc_field_value($display_module_value['name'], json_encode(isset($_POST['loggedin_display_' . $display_module_name]) ? $_POST['loggedin_display_' . $display_module_name] : array()));
		            $special_display_fields[$display_module_name][$display_module_value['name']] = gc_field_value($display_module_value['name'], json_encode(isset($_POST['special_display_' . $display_module_name]) ? $_POST['special_display_' . $display_module_name] : array()));
		        }
		    }
		}
		$data['display_fields'] 		    = !empty($display_fields) ? json_encode($display_fields) : null;
		$data['loggedin_display_fields'] 	= !empty($loggedin_display_fields) ? json_encode($loggedin_display_fields) : null;
		$data['special_display_fields'] 	= !empty($special_display_fields) ? json_encode($special_display_fields) : null;
		
		// Layout Tab
		$data['color_scheme']        	= $this->input->post('color_scheme');
		$data['border_color']        	= $this->input->post('border_color');
		$data['trusted_seal'] 			= $this->input->post('uploaded_image');
		$data['display_order']        	= $this->input->post('display_order');
		$data['priority']        		= $this->input->post('priority');
		
		// upload logo
		/*if (isset($_FILES['trusted_seal']) && is_uploaded_file($_FILES['trusted_seal']['tmp_name'])) {
			$file_data = $this->packages_model->upload_trusted_seal(); // upload logo file
			$data['trusted_seal'] = isset($file_data['upload_data']['file_name']) ? $file_data['upload_data']['file_name'] : ''; // retrieve file name of uploaded file
		}*/
		
		if ($type == 'insert') {
			$id = $this->packages_model->insert($data);

			if (is_numeric($id)) {
				$package_id = $this->input->post('invoicing_package') ? $this->input->post('invoicing_package') : $id;
				$this->db->where('id', $id)->update('packages', array('invoicing_package' => $package_id));
				$return = $id;
			} else {
				$return = FALSE;
			}
		} elseif ($type == 'update') {			
			$data['invoicing_package']        = $this->input->post('invoicing_package') ? $this->input->post('invoicing_package') : $id;
			$return = $this->packages_model->update($id, $data);
		}
		
		// Assign a package to the user on registration handling
		if($return) {
			$value = $this->input->post('is_user_default') ? 1 : 0;
			$this->is_user_default($id, $value);
			// Highlight package
			$this->packages_model->highlight($id, 'packages', ($this->input->post('highlight') ? 1 : 0));
		}
		
		// save translations
		$this->translations->save_translations('packages', $id, $return);
		
		return $return;
	}

	/**
	 * Allow admin to change status
	 * @param int $id
	 */
	public function change_status($id)
	{
		//Return old status
		$this->db->select ( 'active' );
		$this->db->where('id', $id);
		$query = $this->db->get ( $this->db->dbprefix . 'packages' );
	
		$value = 0; //declare variable to store returned result
	
		//Run loop and store integer value in variable
		foreach($query->result() as $row) {
			$value = $row->active;
		}
	
		//Compare values
		if($value == 0) {
			$data['active'] = 1;
		} else {
			$data['active'] = 0;
		}
		//Update status
		$this->db->where('id', $id);
		$this->db->update($this->db->dbprefix . 'packages', $data);
		redirect(SITE_AREA .'/financial/packages/');
	}
	
	/**
	 * Set default
	 * @param int $id
	 */
	public function set_default_old($id) {
		$rows_to_update = array();
		
		if(($this->uri->segment(4) == 'edit' || $this->uri->segment(4) == 'create') && $this->input->post('default')) {
		//CHECK WHETHER REQUEST IS FROM INDEX PAGE OR FORM
			//SET DEFAULT 1 FOR SELECTED ROW
			// $this->packages_model->update($id, array('is_default' => 1));
			$this->db->update('packages', array('is_default' => 1), array('id' => $id));
			
			//GET IDS OF ALL ROWS EXCEPT SPECIFIED ROW
			$result = $this->packages_model->select('id')->where('id NOT IN (' . $id .')')->find_all();
			if($result) {
				//STORE VALUE IN AN ARRAY
				foreach ($result as $row) {
					$rows_to_update[] = $row->id;
				}
				
				$rows = implode(',',$rows_to_update);
				$this->db->where('id IN (' . $rows . ')');
				$this->db->update($this->db->dbprefix . 'packages', array('is_default' => 0, 'modified_on' => date('Y-m-d H:m:s')));
			}
		} else if($this->uri->segment(4) != 'edit' && $this->uri->segment(4) != 'create') {
			//SET DEFAULT 1 FOR SELECTED ROW
			// $this->packages_model->update($id, array('is_default' => 1));
			$this->db->update('packages', array('is_default' => 1), array('id' => $id));
			
			//GET IDS OF ALL ROWS EXCEPT SPECIFIED ROW
			$result = $this->packages_model->select('id')->where('id NOT IN (' . $id .')')->find_all();
			if($result) {
				//STORE VALUE IN AN ARRAY
				foreach ($result as $row) {
					$rows_to_update[] = $row->id;
				}
					
				$rows = implode(',',$rows_to_update);
				$this->db->where('id IN (' . $rows . ')');
				$this->db->update($this->db->dbprefix . 'packages', array('is_default' => 0, 'modified_on' => date('Y-m-d H:m:s')));
			}
		}		
		Template::redirect(current_url());
		// redirect(SITE_AREA .'/financial/packages/');		
	}
	
	/**
	 * Set Package Default
	 * @param integer $id
	 * @return string
	 */
	private function set_default($id, $table = 'packages') {
	    // set is_default to 0 for all packages
	    $this->db->update($table, array('is_default' => 0, 'modified_on' => date('Y-m-d H:m:s')));
	    // set is_default to 1 of this package
	    return $this->db->where(array('id' => $id))->update($table, array('is_default' => 1, 'modified_on' => date('Y-m-d H:m:s')));
	}
	
	/**
	 * Assign package to the user on registration through is_user_default field
	 * Update this column information based on the user selection
	 * @param int $id
	 * @param number $value
	 */
	private function is_user_default($id, $value = 1) {
		if($value == 1) {
			// only one package can be user default
			// set the is_user_default field to 0 for all packages
			$this->db->where('id != ', $id)->update('packages', array('is_user_default' => 0));
		}
		$this->db->where('id', $id)->update('packages', array('is_user_default' => $value));
	}
}