<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Migration_Install_packages extends Migration
{
	/**
	 * The name of the database table
	 *
	 * @var String
	 */
	private $table_name = 'packages';

	/**
	 * The table's fields
	 *
	 * @var Array
	 */
	private $fields = array(
			'id' => array(
				'type' => 'INT',
				'constraint' => 11,
				'auto_increment' => TRUE
			),
			'title' => array(
				'type' => 'VARCHAR',
				'constraint' => 100,
				'null' => FALSE
			),
			'description' => array(
				'type' => 'TEXT',
				'null' => FALSE
			),
			'icon' => array(
				'type' => 'VARCHAR',
				'constraint' => 30,
				'null' => TRUE
			),
			'plan_type' => array(
				'type' => 'VARCHAR',
				'constraint' => 20,
				'null' => FALSE
			),
			'upgrade_valid_for' => array(
				'type' => 'INT',
				'constraint' => 11,
				'null' => FALSE,
				'default'=> 0
			),
			'claim_status' => array(
				'type' => 'TINYINT',
				'constraint' => 2,
				'null' => FALSE,
				'default'=> 0
			),
			'order_status' => array(
				'type' => 'TINYINT',
				'constraint' => 2,
				'null' => FALSE,
				'default'=> 0
			),
			'priority' => array(
				'type' => 'SMALLINT',
				'constraint' => 5,
				'null' => FALSE,
				'default'=> 0
			),
			'is_default' => array(
				'type' => 'TINYINT',
				'constraint' => 1,
				'null' => FALSE,
				'default'=> 0
			),
			'is_user_default' => array(
				'type' => 'TINYINT',
				'constraint' => 1,
				'null' => FALSE,
				'default'=> 0
			),
			'price' => array(
				'type' => 'FLOAT',
				'null' => FALSE
			),
			'claim_price' => array(
				'type' => 'FLOAT',
				'null' => FALSE,
				'default'=> 0
			),
			'duration' => array(
				'type' => 'INT',
				'constraint' => 5,
				'null' => FALSE
			),
			'fixed_date' => array(
				'type' => 'DATE',
				'null' => TRUE
			),
			'forever' => array(
				'type' => 'TINYINT',
				'constraint' => 1,
				'null' => FALSE,
				'default'=> 0
			),
			'multisale' => array(
				'type' => 'TINYINT',
				'constraint' => 1,
				'null' => FALSE,
				'default'=> 1
			),
			'module_limit' => array(
				'type' => 'VARCHAR',
				'constraint' => 255,
				'null' => FALSE
			),
			'module_status' => array(
				'type' => 'VARCHAR',
				'constraint' => 255,
				'null' => FALSE
			),
			'module_featured' => array(
				'type' => 'VARCHAR',
				'constraint' => 255,
				'null' => FALSE
			),
			'module_categories_limit' => array(
				'type' => 'VARCHAR',
				'constraint' => 255,
				'null' => FALSE
			),
			'module_images_limit' => array(
				'type' => 'VARCHAR',
				'constraint' => 255,
				'null' => FALSE
			),
			'module_images_status' => array(
				'type' => 'VARCHAR',
				'constraint' => 255,
				'null' => FALSE
			),
			'keywords_limit' => array(
				'type' => 'SMALLINT',
				'constraint' => 5,
				'null' => FALSE,
				'default'=> 0
			),
			'keywords_length' => array(
				'type' => 'SMALLINT',
				'constraint' => 5,
				'null' => FALSE,
				'default'=> 0
			),
			'description_limit' => array(
				'type' => 'MEDIUMINT',
				'constraint' => 8,
				'null' => FALSE,
				'default'=> 0
			),	
			'applicable_categories' => array(
				'type' => 'TEXT',
				'null' => TRUE
			),	
			'address' => array(
				'type' => 'TINYINT',
				'constraint' => 1,
				'null' => FALSE
			),
			'email' => array(
				'type' => 'TINYINT',
				'constraint' => 1,
				'null' => FALSE
			),
			'website' => array(
				'type' => 'TINYINT',
				'constraint' => 1,
				'null' => FALSE
			),
			'map' => array(
				'type' => 'TINYINT',
				'constraint' => 1,
				'null' => FALSE
			),
			'qrcode' => array(
				'type' => 'TINYINT',
				'constraint' => 1,
				'null' => FALSE
			),
			'logo' => array(
				'type' => 'TINYINT',
				'constraint' => 1,
				'null' => FALSE
			),
			'phone' => array(
				'type' => 'TINYINT',
				'constraint' => 1,
				'null' => FALSE
			),
			'fax' => array(
				'type' => 'TINYINT',
				'constraint' => 1,
				'null' => FALSE
			),
			'social' => array(
				'type' => 'TINYINT',
				'constraint' => 1,
				'null' => FALSE
			),
			'person' => array(
				'type' => 'TINYINT',
				'constraint' => 1,
				'null' => FALSE
			),
			'form_fields' => array(
				'type' => 'TEXT',
				'null' => TRUE
			),
			'recurring' => array(
				'type' => 'TINYINT',
				'constraint' => 1,
				'null' => FALSE,
				'default'=> 0
			),
			'invoicing_package' => array(
				'type' => 'INT',
				'constraint' => 11,
				'null' => FALSE,
				'default'=> 0
			),
			'downgrade_package' => array(
				'type' => 'INT',
				'constraint' => 11,
				'null' => FALSE,
				'default'=> 0
			),
			'days_before_next_invoice' => array(
				'type' => 'SMALLINT',
				'constraint' => 5,
				'null' => FALSE,
				'default'=> 0
			),
			'invoice_expiry' => array(
				'type' => 'SMALLINT',
				'constraint' => 5,
				'null' => FALSE,
				'default'=> 0
			),
			'invoice_first_reminder' => array(
				'type' => 'SMALLINT',
				'constraint' => 5,
				'null' => FALSE,
				'default'=> 0
			),
			'invoice_second_reminder' => array(
				'type' => 'SMALLINT',
				'constraint' => 5,
				'null' => FALSE,
				'default'=> 0
			),
			'invoice_third_reminder' => array(
				'type' => 'SMALLINT',
				'constraint' => 5,
				'null' => FALSE,
				'default'=> 0
			),
			'color_scheme' => array(
				'type' => 'VARCHAR',
				'constraint' => 10,
				'null' => TRUE
			),
			'border_color' => array(
				'type' => 'VARCHAR',
				'constraint' => 10,
				'null' => TRUE
			),
			'trusted_seal' => array(
				'type' => 'VARCHAR',
				'constraint' => 50,
				'null' => TRUE
			),
			'display_order' => array(
				'type' => 'INT',
				'constraint' => 11,
				'null' => FALSE,
				'default'=> 0
			),
			'highlight' => array(
				'type' => 'TINYINT',
				'constraint' => 1,
				'null' => FALSE,
				'default'=> 0
			),
			'active' => array(
				'type' => 'TINYINT',
				'constraint' => 1,
				'null' => FALSE,
				'default'=> 1
			),
			'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
			'modified_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
	);
	/**
	 * Install this migration
	 *
	 * @return void
	 */
	public function up()
	{
		$this->dbforge->add_field($this->fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('priority', false);
		$this->dbforge->create_table($this->table_name);
	}

	//--------------------------------------------------------------------

	/**
	 * Uninstall this migration
	 *
	 * @return void
	 */
	public function down()
	{
		$this->dbforge->drop_table($this->table_name);
	}

	//--------------------------------------------------------------------
	
}