<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
############################################################## ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class Packages_model extends BF_Model {

	protected $table_name	= "packages";
	protected $key			= "id";
	protected $soft_deletes	= false;
	protected $date_format	= "datetime";

	protected $log_user 	= FALSE;

	protected $set_created	= true;
	protected $set_modified = true;
	protected $created_field = "created_on";
	protected $modified_field = "modified_on";

	/*
		Customize the operations of the model without recreating the insert, update,
		etc methods by adding the method names to act as callbacks here.
	 */
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 		= array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	/*
		For performance reasons, you may require your model to NOT return the
		id of the last inserted row as it is a bit of a slow method. This is
		primarily helpful when running big loops over data.
	 */
	protected $return_insert_id 	= TRUE;

	// The default type of element data is returned as.
	protected $return_type 			= "object";

	// Items that are always removed from data arrays prior to
	// any inserts or updates.
	protected $protected_attributes = array();

	/*
		You may need to move certain rules (like required) into the
		$insert_validation_rules array and out of the standard validation array.
		That way it is only required during inserts, not updates which may only
		be updating a portion of the data.
	 */
	protected $validation_rules 		= array(
		array(
			"field"		=> "title",
			"label"		=> "lang:label_title",
			"rules"		=> "trim|required|xss_clean"
		),
		array(
			"field"		=> "description",
			"label"		=> "lang:label_description",
			"rules"		=> "trim|xss_clean"
		),
		array(
			"field"		=> "icon",
			"label"		=> "lang:label_icon",
			"rules"		=> "trim"
		),/*
		array(
			"field"		=> "plan_type",
			"label"		=> "lang:label_plan_type",
			"rules"		=> "trim|required|xss_clean"
		),*/
		array(
			"field"		=> "upgrade_valid_for",
			"label"		=> "lang:help_upgrade_valid_for",
			"rules"		=> "trim|xss_clean"
		),
		array(
			"field"		=> "claim_status",
			"label"		=> "lang:label_claim_status",
			"rules"		=> "trim"
		),
		array(
			"field"		=> "order_status",
			"label"		=> "lang:label_order_status",
			"rules"		=> "trim"
		),
		array(
			"field"		=> "display_order",
			"label"		=> "lang:label_order",
			"rules"		=> "trim|required|is_numeric"
		),
		array(
			"field"		=> "priority",
			"label"		=> "lang:label_priority",
			"rules"		=> "trim|required|max_length[5]|integer"
		),
		array(
			"field"		=> "default",
			"label"		=> "lang:label_set_default",
			"rules"		=> "trim"
		),
		array(
			"field"		=> "is_user_default",
			"label"		=> "lang:is_user_default",
			"rules"		=> "trim"
		),
		array(
			"field"		=> "active",
			"label"		=> "lang:label_status",
			"rules"		=> "trim"
		),		
		array(
			"field"		=> "price",
			"label"		=> "lang:label_price",
			"rules"		=> "trim|max_length[6]|is_decimal"
		),
	    array(
	        "field"		=> "offer_price",
	        "label"		=> "lang:label_offer_price",
	        "rules"		=> "trim|max_length[6]|is_decimal"
	    ),
		array(
			"field"		=> "claim_price",
			"label"		=> "lang:label_claim_price",
			"rules"		=> "trim|max_length[6]|is_decimal"
		),
		array(
			"field"		=> "duration",
			"label"		=> "lang:label_duration",
			"rules"		=> "trim|max_length[5]|is_numeric"
		),
		array(
			"field"		=> "fixed_date",
			"label"		=> "lang:label_fixed_date",
			"rules"		=> "trim|xss_clean"
		),
		array(
			"field"		=> "forever",
			"label"		=> "lang:label_forever",
			"rules"		=> "trim"
		),
		array(
			"field"		=> "multisale",
			"label"		=> "lang:label_multisale",
			"rules"		=> "trim"
		),		
		array(
			"field"		=> "module_limit",
			"label"		=> "lang:label_module_limit",
			"rules"		=> "trim|is_array"
		),
		array(
			"field"		=> "module_status",
			"label"		=> "lang:label_module_status",
			"rules"		=> "trim|is_array"
		),
		array(
			"field"		=> "module_featured",
			"label"		=> "lang:label_module_featured",
			"rules"		=> "trim|is_array"
		),
		array(
			"field"		=> "module_categories_limit",
			"label"		=> "lang:module_categories_limit",
			"rules"		=> "trim|is_array"
		),
		array(
			"field"		=> "module_images_limit",
			"label"		=> "lang:module_images_limit",
			"rules"		=> "trim|is_array"
		),
		array(
			"field"		=> "module_images_status",
			"label"		=> "lang:module_images_status",
			"rules"		=> "trim|is_array"
		),
		array(
			"field"		=> "module_videos_limit",
			"label"		=> "lang:module_videos_limit",
			"rules"		=> "trim|is_array"
		),
		array(
			"field"		=> "module_videos_status",
			"label"		=> "lang:module_videos_status",
			"rules"		=> "trim|is_array"
		),
		array(
			"field"		=> "keywords_limit",
			"label"		=> "lang:label_keywords_limit",
			"rules"		=> "trim"
		),
		array(
			"field"		=> "keywords_length",
			"label"		=> "lang:label_keywords_length",
			"rules"		=> "trim|max_length[8]|is_numeric"
		),
		array(
			"field"		=> "description_limit",
			"label"		=> "lang:label_description_length",
			"rules"		=> "trim|max_length[8]|is_numeric"
		),
		array(
			"field"		=> "applicable_categories",
			"label"		=> "lang:applicable_categories",
			"rules"		=> "trim"
		),
		array(
			"field"		=> "package_address",
			"label"		=> "lang:label_address",
			"rules"		=> "trim"
		),
		array(
			"field"		=> "package_email",
			"label"		=> "lang:label_email",
			"rules"		=> "trim"
		),
		array(
			"field"		=> "package_website",
			"label"		=> "lang:label_website",
			"rules"		=> "trim"
		),
		array(
			"field"		=> "package_map",
			"label"		=> "lang:label_map",
			"rules"		=> "trim"
		),
		array(
			"field"		=> "package_qrcode",
			"label"		=> "lang:label_qrcode",
			"rules"		=> "trim"
		),
		array(
			"field"		=> "package_logo",
			"label"		=> "lang:label_logo",
			"rules"		=> "trim"
		),
		array(
			"field"		=> "package_phone",
			"label"		=> "lang:label_phone",
			"rules"		=> "trim"
		),
		array(
			"field"		=> "package_fax",
			"label"		=> "lang:label_fax",
			"rules"		=> "trim"
		),
		array(
			"field"		=> "package_social",
			"label"		=> "lang:label_social",
			"rules"		=> "trim"
		),
		array(
			"field"		=> "package_person",
			"label"		=> "lang:label_contact_person",
			"rules"		=> "trim"
		),
		array(
			"field"		=> "package_location",
			"label"		=> "lang:locations",
			"rules"		=> "trim"
		),
		array(
			"field"		=> "recurring",
			"label"		=> "lang:label_recurring",
			"rules"		=> "trim"
		),
		array(
			"field"		=> "invoicing_package",
			"label"		=> "lang:label_invoicing_package",
			"rules"		=> "trim"
		),
		array(
			"field"		=> "downgrade_package",
			"label"		=> "lang:label_downgrade_package",
			"rules"		=> "trim"
		),
		array(
			"field"		=> "days_before_next_invoice",
			"label"		=> "lang:days_before_next_invoice",
			"rules"		=> "trim|is_numeric"
		),
		array(
			"field"		=> "invoice_expiry",
			"label"		=> "lang:invoice_expiry",
			"rules"		=> "trim|is_numeric"
		),			
		array(
			"field"		=> "invoice_first_reminder",
			"label"		=> "lang:invoice_first_reminder",
			"rules"		=> "trim|is_numeric"
		),
		array(
			"field"		=> "invoice_second_reminder",
			"label"		=> "lang:invoice_second_reminder",
			"rules"		=> "trim|is_numeric"
		),
		array(
			"field"		=> "invoice_third_reminder",
			"label"		=> "lang:invoice_third_reminder",
			"rules"		=> "trim|is_numeric"
		),				
		array(
			"field"		=> "color_scheme",
			"label"		=> "lang:label_color_scheme",
			"rules"		=> "trim"
		),
		array(
			"field"		=> "border_color",
			"label"		=> "lang:label_border_color",
			"rules"		=> "trim"
		),
		array(
			"field"		=> "clear_trusted_seal",
			"label"		=> "lang:clear_trusted_seal",
			"rules"		=> "trim"
		)
	);
	protected $insert_validation_rules 	= array();
	protected $skip_validation 			= FALSE;
	public $listings_form_fields = array('show_address', 'show_phone', 'show_email', 'tollfree', 'fax', 'website', 'map', 'linkedin_url', 'facebook_url', 'twitter_url', 'googleplus_url', 'youtube_url', 'pinterest_url', 'instagram_url');
	
	protected $gc_packages_cache;
	
	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct() {
	    parent::__construct();
	    // Load Zend Caching
	    $this->load->library('slug');
	    $this->load->library('gzf_cache', array('lifetime' => 900));
	    if(method_exists($this->gzf_cache, 'get_instance')) {
	        $this->gc_packages_cache = $this->gzf_cache->get_instance();
	    }
	}


	/**
	 * MEMBER AREA
	 * Get package information for member add business
	 * @param number $listing_id
	 */
	public function get_package_info($listing_id) {
		$this->db->select($this->db->dbprefix . 'packages.*');
		$this->db->join($this->db->dbprefix . 'listings L',$this->db->dbprefix . 'packages.id = L.package_id', 'inner');
		return $this->packages_model->find_by(array('L.id' => $listing_id));
	}
	
	/**
	 * MOBILE
	 * Check whether renewal option to be shown with package or not
	 * @param integer $package_id
	 * @param integer $expires_on
	 */
	public function renewable($package_id, $expires_on) {
		if(!empty($expires_on) && $expires_on != '0000-00-00') {
			$today_date=new DateTime(date("Y-m-d"));
			$expires_on = new DateTime($expires_on);
			$diff = $expires_on->diff($today_date)->format("%a"); // calculate date difference in days
			$package = $this->db->select('id, days_before_next_invoice, multisale, invoicing_package')->get_where('packages', array('id' => $package_id))->row();
			if($diff < $package->days_before_next_invoice) {
			    if($package->multisale == 1) {
					return $package->invoicing_package;
				} else {
					return 0;
				}
			} else {
				return 0; // Next invoice date has not yet come
			}
		} else {
			return 0; // Package is for lifetime
		}
	}
	
	/**
	 * Save uploaded image
	 * @return multitype:NULL
	 */
	public function upload_trusted_seal() {	
	    if(!gc_demo_mode(array('no_message' => true, 'no_redirect' => true))) {
	        return false;
	    }
		$config['upload_path'] = realpath(FCPATH.'assets/images/'); //Make SURE that you chmod this directory to 777!
		$config['allowed_types'] = 'gif|jpg|png';
		$config['max_size']= '100';
		$config['max_width']= '85';
		$config['max_height']= '80';
		$config['remove_spaces']=TRUE; //Remove spaces from the file name
	
		$this->load->library('upload', $config);	
		if (!$this->upload->do_upload('trusted_seal')) { // Image name is necessary
			$this->session->set_flashdata('upload_image_error', $this->upload->display_errors());
			return false;
		} else {
			$data = array('upload_data' => $this->upload->data());
			//	$this->resize($data['upload_data']['full_path'],$data['upload_data']['file_name']);
		}
		return $data;
	}
	
	/**
	 * Create thumbs of uploaded image
	 */
	public function resize($path,$file){
		$config['image_library']='gd2';
		$config['source_image']=$path;
		//$config['create_thumb']=TRUE; // This will create new file
		$config['maintain_ratio']=TRUE;
		$config['width']= '85';
		$config['height']= '80';
		$config['new_image']='./assets/images/'.$file; //CHANGE THIS LINE FOR PATH
	
		$this->load->library('image_lib',$config);
		$this->image_lib->resize();
	}
	
	/**
	 * Highlight package
	 * @param int $id
	 * @param string $table packages/ad_packages
	 * @param int $highlight
	 */
	public function highlight($id, $table, $highlight = 0) {
		// set highlight status
		$this->db->update($table, array('highlight' => $highlight), array('id' => $id));
		// set the highlight to 0 for other packages
		if($highlight == 1) {
			$this->db->where(array('id != ' =>  $id))->update($table, array('highlight' => 0));
		}
	}
	
	/**
	 * Clear Trusted Seal
	 * @param int $id
	 * @param string $table
	 */
	public function clear_trusted_seal($id, $table) {
		// Clear Trusted Seal image
		if($this->input->post('no_image')) {
			$get_trusted = $this->db->get_where($table, array('id' => $id))->row();
			if($get_trusted && !empty($get_trusted->trusted_seal) && file_exists(realpath(FCPATH.'/assets/images/' . $get_trusted->trusted_seal))) {
				unlink(realpath(FCPATH.'/assets/images/' . $get_trusted->trusted_seal));
			}
			$this->db->where('id', $id)->update($table, array('trusted_seal' => NULL));
		}
	}
	
	/**
	 * Tweak Packages' priority in bulk
	 */
	public function _tweak_bulk($module = null, $where = null) {
	    $packages = $this->packages_model->find_all();
	    if($packages) {
	        if(empty($module)) {
    	        $site_modules = gc_modules();
    	        $site_modules[] = 'videos';
	        } else {
	            $site_modules = array($module);
	        }
	        foreach($packages as $package) {
        	    foreach($site_modules as $site_module) {
        	        if(!empty($where)) {
        	            $this->db->where($where);
        	        }
        	        $this->db->where('package_id', $package->id)->update($site_module, array('package_priority' => $package->priority));
        	    }
	        }
	    }
	}
	
	/**
	 * Tweak module's package priority on package priority change
	 * @param number $package_id
	 * @param number $old_priority
	 * @param number $new_priority
	 */
	public function _tweak_changed_priority($package_id, $old_priority, $new_priority) {
	    if($old_priority != $new_priority) {
	        $site_modules = gc_modules();
	        $site_modules[] = 'videos';
	        foreach($site_modules as $site_module) {
	            $this->db->where('package_id', $package_id)->update($site_module, array('package_priority' => $new_priority));
	        }
	    }
	}
	
	/**
	 * Tweak Packages Priority
	 * @param number $module
	 * @param number $row_id
	 * @param boolean $is_update - TRUE - Update instantly, FALSE - Return value for Bulk Tweaking of Multiple Rows
	 */
	public function _tweak_package_priority($module, $row_id, $is_update = true) {
	    $record = $this->db->get_where($module, array('id' => $row_id))->row();
	    if($record && !empty($record->package_id)) {
	        if(property_exists ($record, 'package_priority') && !empty($record->package_priority)) {
	            $package = $this->db->where('id', $record->package_id)->get_where($this->table_name)->row();
	            if($package) {
	                $this->db->where('id', $row_id)->update($module, array('package_priority' => $record->package_priority));
	            }
	        }
	    }
	}
	
	/**
	 * Auto fill package description
	 * @param string $package_id
	 * @param array $package
	 */
	public function description($package_id, $package = array()) {
	    $this->lang->load('packages/packages');
	    $cacheID = 'packages_description_' . $package_id;
	    if(!$description = $this->gc_packages_cache->load($cacheID)) {
	        $description = '';
	        if(empty($package)) {
	           $package = $this->db->where('id', $package_id)->get('packages')->row();
	        }
	        if(!empty($package)) {
	            if(!empty($package->description)) {
	                $description = $package->description;
	            } else {
	                $tag_start = '<li class="list-group-item p-1">';
	                $tag_end = '</li>';
	                $outer_start = '<ul class="list-group">';
	                $outer_end = '</ul>';
	                $outer_start = null;
	                $outer_end = null;
	                $html_output = array();
	                $html_output[] = $this->process_json($package, 'module_featured', $tag_start, $tag_end, $outer_start, $outer_end);
	                $html_output[] = $this->process_json($package, 'module_limit', $tag_start, $tag_end, $outer_start, $outer_end);
	                $html_output[] = $this->process_json($package, 'module_status', $tag_start, $tag_end, $outer_start, $outer_end);
	                $html_output[] = $this->process_json($package, 'module_images_limit', $tag_start, $tag_end, $outer_start, $outer_end);
	                $html_output[] = $this->process_json($package, 'module_images_status', $tag_start, $tag_end, $outer_start, $outer_end);
	                $html_output[] = $this->process_json($package, 'module_videos_limit', $tag_start, $tag_end, $outer_start, $outer_end);
	                $html_output[] = $this->process_json($package, 'module_videos_status', $tag_start, $tag_end, $outer_start, $outer_end);
	                $html_output[] = $this->process_json($package, 'pyg', $tag_start, $tag_end, $outer_start, $outer_end);
	                if(!empty($package->trusted_seal)) {
	                   $html_output[] = lang('trusted_seal');
	                }
	                
	                $description = implode(PHP_EOL, $html_output);
	            }
	        }
	        $this->gc_packages_cache->save($description, $cacheID, array('packages'));
	    }
	    return $description;
	}
	
	/**
	 * Process JSON field
	 * @param object $record
	 * @param string $field
	 * @param string $tag_start
	 * @param string $tag_end
	 * @return string
	 */
	public function process_json($record, $field = 'module_limit', $tag_start = '<li>', $tag_end = '</li>', $outer_start = '<ul class="list-unstyled">', $outer_end = '</ul>') {
	    $records = json_decode($record->{$field} ?? '', true);
	    if($records) {
	        $result = array();
    	    foreach($records as $module => $value) {
    	        $limit = '';
    	        switch($value) {
    	            case 0:
    	                $limit = lang('no_limit');
    	                break;
    	            case 999:
    	                $limit = lang('label_disabled');
    	                break;
    	            default:
    	                $limit = $value;
    	        }
    	        switch($field) {
    	            case 'pyg':
    	                if($value != 0) {
    	                    $result[] = $tag_start . currency_format($value) . '/' .  lang('singular_' . $module) . $tag_end;
    	                }
    	                break;
    	            case 'module_status':
    	            case 'module_featured':
    	            case 'module_images_status':
    	            case 'module_videos_status':
    	                $result[] = $tag_start . '<i class="fas fa-check fa-fw"></i>' .  lang($module) . $tag_end;
    	                break;
    	            case 'module_images_limit':
    	            case 'module_videos_limit':
    	                $result[] = $tag_start . $limit . '/' . lang('singular_' . $module) . $tag_end;
    	                break;
    	            default:
    	                $result[] = $tag_start . $limit . ' ' . ($limit == 1 ? lang('singular_' . $module) : lang($module)) . $tag_end;
    	        }
    	    }
    	    if(!empty($outer_start) && !empty($outer_end)) {
    	        $output = '<h4 class="text-uppercase">' . lang('pkg_' . $field)  . '</h4>';
    	        $output .= $outer_start . implode(PHP_EOL, $result) . $outer_end;
    	        return $output; 
    	    } else {
    	        $output = str_replace('list-group-item', 'list-group-item list-group-item-secondary', $tag_start) . lang('pkg_' . $field) .  $tag_end . implode(PHP_EOL, $result);
    	        return $output; 
    	    }
	    }
	    return null;
	}
	
	/**
	 * Get upgrade packages to display them to the user to promote sales
	 * Store them in a session variable during the user login session setup and avoid unnecessary queries
	 * @param number $user_id
	 * @return array[][]
	 */
	public function get_upgrade_packages($user_id, $session_update = false) {
	    $upgrade_packages = array();
	    // get order information from the orders table
	    $this->load->model('orders/orders_model');
	    $order = $this->orders_model->get_user_orders(0, $user_id);
	    if($order && !empty($order->package_id)) {
	        // get package price information
	        $package = $this->db->where('id', $order->package_id)->get('packages')->row();
	        if($package) {
	            // next packages with higher pricing, mysql is ignoring the price > here
	            // $packages = $this->db->where("active = 1 AND deleted = 0 AND plan_type = 'upgrade' AND upgrade_valid_for = " . $order->package_id)->get('packages')->result();
	            $packages = $this->db->where('price > ' . $package->price . ' AND active = 1 AND id !=' . $order->package_id)->get('packages')->result();
	            if($packages) {
	                foreach($packages as $package) {
	                    $upgrade_package = array(
	                        'id' => $package->id,
	                        'title' => $package->title,
	                        'description' => $this->description($package->id, $package),
	                        'trusted_seal' => $package->trusted_seal
	                    );
	                    // dump($this->packages_model->process_json($package, 'module_categories_limit'));
	                    if(!empty($order->order_id)) {
	                        $upgrade_package['order_id'] = $order->order_id;
	                    }
	                    $upgrade_packages[] = $upgrade_package;
	                }
	            }
	        }
	    }
	    if($session_update) {
	        $this->session->set_userdata('upgrade_packages', $upgrade_packages);
	    }
	    return $upgrade_packages;
	}
	
	/**
	 * Get Package Information
	 * @param number $id
	 */
	public function get_package($id) {
	    $cacheID = 'packages_' . $id;
	    if(!$record = $this->gc_packages_cache->load($cacheID)) {
	        $record = $this->db->where('id', $id)->get('packages')->row();
	        $this->gc_packages_cache->save($record, $cacheID, array('packages'));
	    }
	    return $record;
	}
	
	/**
	 * Member Area - Display Existing Package Information to the User
	 * @param boolean $modal - TRUE return modal code
	 * @param boolean $mobile - TRUE mobile specific code only
	 * @param number $id
	 */
	public function package_info($id, $modal = false, $record = array(), $module = 'listings', $mobile = false) {
	    if(empty($record)) {
	        $record = $this->db->where('id', $id)->get('packages')->row();
	    }
	    $package_info = array();
	    $output = '';
	    if($record) {
	        $package_info = array(
	            'trusted_seal' => array('value' => base_url('assets/images/' . $record->trusted_seal)),
	            'title' => array('value' => $record->title, 'label' => lang('label_title')),
	            'description' => array('value' => $record->description, 'label' => lang('label_description')),
	        );
	        $label_title = lang('label_title');
	        $title = $record->title;
	        $description = $record->description;
	        $trusted_seal = empty($record->trusted_seal) ? gc_site_image('no-image') : gc_image_url($record->trusted_seal, false, false);
	        $records = $this->_module_specific($record);
	        $limit_info = $this->_tabular_modules($records);
	        if($mobile) {
	            $mobile_records = array();
	            foreach($records as $key_module => $values) {
	                $row = array(
	                    'label' => lang($key_module)
	                );
	                foreach($values as $key => $value) {
	                    switch($key) {
	                        case 'module_status':
	                        case 'module_featured':
	                        case 'module_images_status':
	                        case 'module_videos_status':
	                            $value = empty($value) ? 'close' : 'check';
	                            break;
	                        case 'pyg':
	                            $value = !empty($value) ? currency_format($value) . '/' . lang('singular_' . $module) : lang('not_applicable');
	                            break;
	                    }
	                    $row['values'][] = array(
	                        'label' => lang('pkgm_' . $key),
	                        'value' => $value
	                    );
	                }
	                $mobile_records[] = $row;
	            }
	            return array('title' => $title, 'description' => $description, 'trusted_seal' => $trusted_seal, 'modules_info' => $mobile_records);
	        }
	        $content = <<<EOL
<div class="card">
   <div class="card-body">
    <div class="row">
        <div class="col-sm-2 mb-2">
            <img class="thumb-100 img-fluid" src="{$trusted_seal}" alt="{$title}">
        </div>
        <div class="col-sm-10">
           <h4 class="card-title">{$title}</h4>
           <ul class="list-unstyled">{$description}</ul>
        </div>
    </div>
   {$limit_info}
   </div>
</div>
EOL;
	    }
	    if(!$modal) {
	        return $content;
	    } else {
	        $heading = lang('package_info');
	        $close = lang('form_close');
	        $output = <<<EOL
<div class="modal" id="previewModal">
  <div class="modal-dialog mw-100 w-75">
    <div class="modal-content">
      <div class="modal-header bg-theme-secondary pt-2 pb-2">
        <h4 class="modal-title">{$heading}</h4>
        <button type="button" class="close text-white" data-dismiss="modal">&times;</button>
      </div>
      <div class="modal-body">
        {$content}
      </div>
      <div class="modal-footer">
        <button type="button" class="btn btn-danger" data-dismiss="modal">{$close}</button>
      </div>
    </div>
  </div>
</div>
EOL;
        return $output;
	    }
	}
	
	/**
	 * Get Modules Limit and Other information
	 * @param array $record
	 * @return array|string
	 */
	private function _module_specific($record) {
	    $modules = gc_modules();
	    // $modules[] = 'branches';
	    // $modules[] = 'videos';
	    $display_columns = array('module_limit', 'module_categories_limit', 'module_documents_limit', 'module_images_limit', 'module_videos_limit', 'pyg', 'module_status', 'module_featured', 'module_images_status', 'module_videos_status');
	    $result = array();
	    foreach($modules as $module) {
	        foreach($display_columns as $display_column) {
	            if(isset($record->{$display_column})) {
	                $values = json_decode($record->{$display_column}, true);
	                if(!empty($values)) {
	                    switch($display_column) {
	                        case 'module_status':
	                        case 'module_featured':
	                        case 'module_images_status':
	                        case 'module_videos_status':
	                        case 'pyg':
	                            if(!empty($values[$module])) {
	                                $value = $values[$module];
	                            } else {
	                                $value = 0;
	                            }
	                            $result[$module][$display_column] = $value;
	                            break;
	                        default:
	                            if(!empty($values[$module]) && $values[$module] == 999) {
	                                $value = lang('label_disabled');
	                            } else if(!empty($values[$module])) {
	                                $value = $values[$module];
	                            } else {
	                                $value = lang('no_limit');
	                            }
	                            $result[$module][$display_column] = $value;
	                    }
	                } else {
	                    $result[$module][$display_column] = 9999;
	                }
	            } else { // null value
	                $result[$module][$display_column] = 9999;
	            }
	        }
	    }
	    return $result;
	}
	
	/**
	 * Tabular View
	 * @param array $records
	 */
	private function _tabular_modules($records) {
	    $rows = array();
	    $headings = array('<th></th>');
	    $table_rows = array();
	    foreach($records['listings'] as $key => $value) {
	        $tr = '<tr>';
	        $tr .= '<th>' . lang('pkgm_' . $key) . '</th>' . PHP_EOL;
	        foreach(gc_modules() as $module) {
	            $value = $records[$module][$key];
	            switch($key) {
	                case 'module_status':
	                case 'module_featured':
	                case 'module_images_status':
	                case 'module_videos_status':
	                    $value = $value == 9999 ? '<i class="fas fa-times"></i>' : '<i class="fas fa-check"></i>';
	                    break;
	                case 'pyg':
	                    $value = !empty($value) ? currency_format($value) . '/' . lang('singular_' . $module) : lang('not_applicable');
	                    break;
	            }
	            $tr .= '<td>' . $value . '</td>' . PHP_EOL;
	        }
	        $tr .= '</tr>';
	        $table_rows[] = $tr;
	    }
	    foreach($records as $module => $record) {
	        $headings[] = '<th>' . lang($module) . '</th>' . PHP_EOL;
	    }
	    $headings = '<tr>' . implode('', $headings) . '</tr>';
	    $rows = implode('', $rows);
	    $table_rows = implode('', $table_rows);
	    $output = <<<EOL
<div class="table-responsive small">
<table class="table table-sm table-striped">
    {$headings}
    {$table_rows}
</table>
</div>
EOL;
    return $output;
	}
	
	/**
	 * Get current package information 
	 * @param number $user_id
	 * @return array
	 */
	public function get_current_package($user_id) {
	    // get order information from the orders table
	    $this->load->model('orders/orders_model');
	    $order = $this->orders_model->get_user_orders(0, $user_id);
	    $package = array();
	    if($order && !empty($order->package_id)) {
	        // get package price information
	        $package = $this->db->select('id, title, description, priority, loggedin_display_fields, module_limit, module_views_limit')->where('id', $order->package_id)->get('packages')->row();
	    }
	    return $package;
	}
}
