<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * pages controller
 */
class Content extends Admin_Controller {
	protected $permissionCreate = 'Pages.Content.Create';
	protected $permissionDelete = 'Pages.Content.Delete';
	protected $permissionEdit   = 'Pages.Content.Edit';
	protected $permissionView   = 'Pages.Content.View';
	
	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct() {
		parent::__construct();
		$this->auth->restrict($this->permissionView);
		clear_cache('pages'); // Popup Pages Content Change
		$this->load->model('pages/pages_model');
		$this->lang->load('pages');
		Template::set_block('sub_nav', 'content/_sub_nav');		
		Assets::add_js(array(
			'tinymce/tinymce.min.js',
		));		
		Assets::add_module_js('pages', 'pages.js');		
	}	
	
	/**
	 * Manage Pages
	 * @param string $filter
	 * @param number $offset
	 */
	public function index($filter = 'all', $offset = 0) {
	    $this->manage('pages', $filter = 'all', $offset = 0);
	}
	
	/**
	 * Manage Pages
	 * @param string $module
	 * @param string $filter
	 * @param number $offset
	 */
	public function manage($module = 'pages', $filter = 'all', $offset = 0) {
	    // enable contact form
	    if (isset($_POST['contact_form'])) {
	        $this->gc->status_change($module, $_POST['contact_form'], 'contact_form');
	    }
	    $extra = array(
	        'model' => 'pages',
	        'context' => 'content',
	        'fields' => null
	    );
	    $extra['actions'] = array( // Extra buttons
	        array('type' => 'link', 'link' => site_url(SITE_AREA . '/content/pages/form/' . $module . '/edit/%s'), 'label' => '<i class="fas fa-edit fa-fw mr-1"></i>' . lang('edit'), 'class' => 'btn btn-xs btn-primary'),
	    );
	    if($module != 'pages') {
	        $extra['fields'] = $module;
	        $extra['prefix'] = 'page';
	        if($module != 'groups' && $module != 'popups') {
	            $this->filter_counter($module);
	            $extra['actions'] = array( // Extra buttons
	                array('type' => 'link', 'link' => site_url(SITE_AREA . '/content/pages/form/' . $module . '/edit/%s'), 'label' => '<i class="fas fa-edit fa-fw mr-1"></i>' . lang('edit'), 'class' => 'btn btn-xs btn-primary'),
	            );
	            $extra['filter_tabs'] = $this->pages_model->other_stuff;
	        }
	    }
	    $this->gc->module_view($module, $filter, $offset, 'admin', $extra);
	}
	
	/**
	 * Add/Edit Pages Data
	 *
	 * @param string $module
	 * @param string $action
	 *
	 * @return void
	 */
	public function form($module = 'pages', $action = 'create') {
	    clear_cache('pages');
	    if($module == 'stuff') {
	        $this->pages_model->stuff_form($module, $action);
	    } else {
    	    Assets::add_js(array(
    	        'tinymce/tinymce.min.js',
    	    ));
    	    $params = array(
    	        'model' => 'pages',
    	        'module_name' => 'pages',
    	        'context' => 'content',
    	        'redirect_url' => site_url(SITE_AREA . '/content/pages/manage/' . $module)
    	    );
    	    if($module != 'pages') {
    	        $params['fields_method'] = $module;
    	        $params['prefix'] = 'page';
    	        $this->gcrud->form($module, $action, 'admin', $params);
    	    } else {
    	        $params['extra_fields'] = array('multi_language' => 1);
    	        $this->gcrud->form($module, $action, 'admin', $params);    	        
    	    }
	    }
	}
	
	/**
	 * Validation Callback Function - Either select a screenshot image OR 
	 * enter a valid URL of the image, both are not allowed.
	 * @return boolean
	 */
	public function one_field_only() {
	    return false;
	}
	
	/**
	 * Check whether the file is valid or not
	 * @param string $filename
	 * @param string $valid_ext
	 * @return boolean
	 */
	public function valid_file($filename, $field) {
	    if($field == 'pdf') {
	        $valid_extensions = array('pdf');
	    } else {
	        $valid_extensions = array('gif', 'jpg', 'png');
	    }
	    $file_url = FCPATH . 'assets/images/' . $filename;
	    if(file_exists($file_url)) {
	        $ext = pathinfo($filename, PATHINFO_EXTENSION);
	        if(in_array($ext, $valid_extensions)) {
	            return true;
	        }
	    }
	    $this->form_validation->set_message('valid_file', sprintf(lang('form_validation_valid_file'), $field, implode(', ', $valid_extensions)));
	    return false;
	}
	
	/**
	 * Filter Counter - Materials/Tutorials module
	 * @param string $module - materials/tutorials
	 */
	public function filter_counter($module) {
	    $table = 'page_' . $module;
	    $sql = "SELECT COUNT(*) AS total";
	    foreach($this->pages_model->other_stuff as $key => $value) {
	        $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $table . " WHERE group_id = '" . $key . "') as " . $value;
	    }
	    $sql .= " FROM " . $this->db->dbprefix . $table;
	    $counter = (array) $this->db->query($sql)->row();
	    Template::set('counter', $counter);
	}
	
	/**
	 * Allow admin to change order of pages
	 */
	public function order () {
		//LOAD NESTED SORTABLE JAVASCRIPT AND CSS FILE
		Assets::add_css('nestedSortable/admin.css');
		Assets::add_js('nestedSortable/jquery-ui.min.js');
		Assets::add_js('nestedSortable/jquery.mjs.nestedSortable.js');
		
		Assets::add_js($this->load->view('content/pages_js', null, true), 'inline');
		Template::set('sortable', TRUE);
		Template::set('toolbar_title', lang('order_pages'));
		Template::render();
	}
	
	/**
	 * Order pages using AJAX nestedSortable
	 */
	public function order_ajax () {
		// Save order from ajax call
		if (isset($_POST['sortable'])) {
			$this->pages_model->save_order($_POST['sortable']);
		}
	
		// Load view
		$order_pages = $this->pages_model->get_nested();
		Template::set('pages', $order_pages);
		Template::set('toolbar_title', lang('order_pages'));
		Template::render();
	}
}