<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Migration_Install_pages extends Migration
{
	/**
	 * The name of the database table
	 *
	 * @var String
	 */
	private $table_name = 'pages';

	/**
	 * The table's fields
	 *
	 * @var Array
	 */
	private $fields = array(
		'id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'auto_increment' => TRUE
		),
		'title' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => FALSE
		),
		'slug' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => FALSE
		),
		'body' => array(
			'type' => 'TEXT',
			'null' => FALSE
		),
		'meta_title' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE
		),
		'meta_keywords' => array(
			'type' => 'TEXT',
			'null' => TRUE
		),
		'meta_description' => array(
			'type' => 'TEXT',
			'null' => TRUE
		),
		'contact_form' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'default' => 0,
			'null' => FALSE
		),
		'hits' => array(
			'type' => 'INT',
			'constraint' => 11,
			'default' => 0,
			'null' => FALSE
		),
		'status' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE
		),
		'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'modified_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP'
	);

	/**
	 * Install this migration
	 *
	 * @return void
	 */
	public function up()
	{
		$this->dbforge->add_field($this->fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->create_table($this->table_name);
		// SET SLUG FIELD COLLATION TO utf8_bin
		$collation_sql = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " CHANGE `slug` `slug` VARCHAR( 255 ) CHARACTER SET utf8 COLLATE utf8_bin NULL DEFAULT NULL";
		$this->db->query ($collation_sql);
	}

	//--------------------------------------------------------------------

	/**
	 * Uninstall this migration
	 *
	 * @return void
	 */
	public function down()
	{
		$this->dbforge->drop_table($this->table_name);
	}

	//--------------------------------------------------------------------

}