<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class Pages_model extends BF_Model {

	public $table_name	= "pages";
	protected $key			= "id";
	protected $soft_deletes	= false;
	protected $date_format	= "datetime";

	protected $log_user 	= FALSE;

	protected $set_created	= TRUE;
	protected $set_modified = TRUE;
	protected $created_field = "created_on";
	protected $modified_field = "modified_on";

	/*
		Customize the operations of the model without recreating the insert, update,
		etc methods by adding the method names to act as callbacks here.
	 */
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 		= array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	/*
		For performance reasons, you may require your model to NOT return the
		id of the last inserted row as it is a bit of a slow method. This is
		primarily helpful when running big loops over data.
	 */
	protected $return_insert_id 	= TRUE;

	// The default type of element data is returned as.
	protected $return_type 			= "object";

	// Items that are always removed from data arrays prior to
	// any inserts or updates.
	protected $protected_attributes = array();

	/*
		You may need to move certain rules (like required) into the
		$insert_validation_rules array and out of the standard validation array.
		That way it is only required during inserts, not updates which may only
		be updating a portion of the data.
	 */
	protected $validation_rules = array();
	protected $insert_validation_rules 	= array();
	protected $skip_validation 			= FALSE;
	protected $stuff_fields = array(
	    'screenshot_title', 'screenshot_description', 'screenshot_image', 'screenshot_url',
	    'video_title', 'video_description', 'video_url',
	    'feature_title', 'feature_description'
	);
	public $other_stuff = array('screenshot' => 'stuff_type__screenshot', 'video' => 'stuff_type__video', 'feature' => 'stuff_type__feature');
	public $gc_pages_cache;
	
	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct() {
	    parent::__construct();
	    // Load Zend Caching
	    $this->load->library('slug');
	    $this->load->library('gzf_cache', array('lifetime' => 900));
	    if(method_exists($this->gzf_cache, 'get_instance')) {
	        $this->gc_pages_cache = $this->gzf_cache->get_instance();
	    }
	}
	
	/**
	 * Pages Fields
	 *
	 * @return array
	 */
	public function fields() {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'title',
	            'label' => 'label_title',
	            'placeholder' => 'placeholder_title',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'edit' => 1,
	            'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
	            'multi_language' => 1,
	            'rules' => 'trim|required|max_length[255]',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'slug',
	            'label' => 'label_slug',
	            'placeholder' => 'placeholder_slug',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim|max_length[255]',
	        ),
	        array(
	            'type' => 'filemanager',
	            'name' => 'image',
	            'label' => 'label_cover',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim|max_length[255]'
	        ),
	        array(
	            'type' => 'textarea',
	            'name' => 'body',
	            'label' => 'label_body',
	            'placeholder' => 'placeholder_description',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'class' => 9,
	            'field_class' => 'editor',
	            'multi_language' => 1,
	            'rules' => 'trim'
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'meta_title',
	            'label' => 'label_meta_title',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'multi_language' => 1,
	            'rules' => 'trim|max_length[255]',
	        ),
	        array(
	            'type' => 'textarea',
	            'name' => 'meta_description',
	            'label' => 'label_meta_description',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'multi_language' => 1,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'textarea',
	            'name' => 'meta_keywords',
	            'label' => 'label_meta_keywords',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'multi_language' => 1,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'hits',
	            'label' => 'label_hits',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'class' => 2,
	            'value' => 0,
	            'rules' => 'trim|required',
	        )/*,
	        array(
	            'type' => 'checkbox',
	            'name' => 'contact_form',
	            'label' => 'include_contact_form',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'help' => lang('help_contact_form'),
	            'rules' => 'trim',
	        )*/,
	        array(
	            'type' => 'checkbox',
	            'name' => 'hide_title',
	            'label' => 'hide_title',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'status',
	            'label' => 'label_status',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_on',
	            'label' => 'created_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_on',
	            'label' => 'modified_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        
	    );
	    return $fields;
	}   
	
	/**
	 * Page Popups Fields
	 *
	 * @return array
	 */
	public function popups() {
	    $custom_fields = array(
	        array(
	            'name' => 'url',
	            'type' => 'text',
	            'label' => lang('label_url')
	        )
	    );
	    Template::set('custom_fields', $custom_fields);
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'name',
	            'label' => 'label_name',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'edit' => 1,
	            'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
	            'rules' => 'trim|required|max_length[255]',
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'page_id',
	            'label' => 'label_page',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'name:pages;value:database',
	            'options_detailed' => 1,
	            'class' => 4,
	            'rules' => 'trim|required'
	        ),
	        array(
	            'type' => 'custom',
	            'name' => 'link_url',
	            'label' => 'label_link_url',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => false,
	            'tooltip' => lang('popup_help_url'),
	            'class' => 9,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'timeout',
	            'label' => 'popup_timeout',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'value' => 0,
	            'class' => 2,
	            'rules' => 'trim|required|numeric',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'frequency',
	            'label' => 'popup_frequency',
	            'help' => 'popup_frequency_help',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'value' => 0,
	            'class' => 2,
	            'tooltip' => lang('popup_frequency_help'),
	            'rules' => 'trim|required|numeric',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'frequency_duration',
	            'label' => 'popup_frequency_duration',
	            'help' => 'popup_frequency_duration_help',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'value' => 0,
	            'class' => 2,
	            'tooltip' => lang('popup_frequency_duration_help'),
	            'rules' => 'trim|required|numeric',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'date_start',
	            'label' => 'popup_start_date',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'tooltip' => lang('popup_help_start_date'),
	            'class' => 3,
	            'rules' => 'trim|valid_date',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'date_end',
	            'label' => 'popup_end_date',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'tooltip' => lang('popup_help_end_date'),
	            'class' => 3,
	            'rules' => 'trim|valid_date|compare_dates[date_start]',
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'status',
	            'label' => 'label_status',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_on',
	            'label' => 'created_on',
	            'frontend' => false,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_on',
	            'label' => 'modified_on',
	            'frontend' => false,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        
	    );
	    return $fields;
	}
	
	/**
	* Page Stuff Fields
	*
	* @return array
	*/
	public function stuff() {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'group_id',
	            'label' => 'singular_groups',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'name:page_groups;value:database',
	            'options_detailed' => 1,
	            'rules' => 'trim|required|max_length[255]'
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'stuff_type',
	            'label' => 'label_type',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim|max_length[10]',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'params',
	            'label' => 'label_page',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            // 'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'priority',
	            'label' => 'label_priority',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'status',
	            'label' => 'label_status',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_on',
	            'label' => 'created_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_on',
	            'label' => 'modified_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        
	    );
	    return $fields;
	}
	
	/**
	* Groups Fields
	*
	* @return array
	*/
	public function groups($role = 'admin') {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'name',
	            'label' => 'label_name',
	            'placeholder' => 'placeholder_name',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
	            'rules' => 'required|trim|max_length[255]',
	        ),
	        array(
	            'type' => 'textarea',
	            'name' => 'description',
	            'label' => 'label_description',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim|max_length[255]',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'priority',
	            'label' => 'label_priority',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'value' => 0,
	            'class' => 2,
	            'rules' => 'trim|max_length[255]'
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'status',
	            'label' => 'label_status',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	            'field_extra' => 'index_action_admin:options|icon_primary#fas fa-thumbs-up|icon_secondary#fas fa-thumbs-down'
	        )
	    );
	    return $fields;
	}

	//--------------------------------------------------------------------
	/**
	 * Get all parent pages
	 * @return multitype:string NULL
	 */
	public function get_no_parents () {
		// Fetch pages without parents
		$this->db->select('id, title');
		$this->db->where('parent_id', 0);
		$pages = $this->db->get($this->db->dbprefix . 'pages')->result();
	
		// Return key => value pair array
		$array = array(
				0 => lang('label_no_parent')
		);
		if (count($pages)) {
			foreach ($pages as $page) {
				$array[$page->id] = $page->title;
			}
		}
	
		return $array;
	}
	
	/*----------------------------------------------------*/
	/*	Order Pages using Ajax NestedSortable
	/*----------------------------------------------------*/
	
	/**
	 * Save order only when user clicks on save button
	 * @param array $pages
	 */
	public function save_order ($pages)
	{
		if (count($pages)) {
			foreach ($pages as $order => $page) {
				if (isset($page['id']) && $page['id'] != '') {
					$data = array('parent_id' => (int) $page['parent_id'], 'order' => $order);
					$this->db->set($data)->where('id', $page['id'])->update($this->table_name);
				}
			}
		}
	}
	
	/**
	 * Display all pages to the user
	 * @return multitype:array
	 */
	public function get_nested () {
		$pages = $this->db->order_by('order')->get('pages')->result_array();
		return $this->prepareList($pages);
	}
	
	/**
	 * Prepare list to get parents and its children
	 * @param array $items
	 * @param number $pid
	 * @return multitype:multitype:unknown
	 */
	private function prepareList(array $items, $pid = 0) {
			$output = array();
		
			# loop through the items
			foreach ($items as $item) {
		
				# Whether the parent_id of the item matches the current $pid
				if ((int) $item['parent_id'] == $pid) {
			
				# Call the function recursively, use the item's id as the parent's id
					# The function returns the list of children or an empty array()
					if ($children = $this->prepareList($items, $item['id'])) {
			
					# Store all children of the current item
					$item['children'] = $children;
					}
						# Fill the output
						$output[] = $item;
				}
			}	
			return $output;
	}
	
	/**
	 * Display pages information to the user
	 * @param string $slug
	 */
	/**
	 * Display pages information to the user
	 * @param string $slug
	 */
	public function detail($slug = NULL, $render = 'pages') {
	    if($slug == NULL)	{
	        redirect(site_url());
	    }
	    $page_detail = $this->pages_model->select('id, slug, title, image, body, meta_title, meta_keywords, contact_form, hide_title, meta_description')->find_by(array('slug' => $slug, 'status' => 1));
	    if(!$page_detail) {
	        Template::redirect(site_url());
	    }
	    // get translations if exists
	    $page_detail = $this->translations->get_translations('pages', $page_detail->id, $page_detail, 1);
	    // slug_redirect('pages', $page_detail); // must pass ID with slug for multi language redirect
	    /**
	     * Meta Information
	     */
	    if(!empty($page_detail->meta_title)) {
	        $page_title = $page_detail->meta_title;
	    } else if(settings_item('seo.page_meta_title') != '') {
	        $meta_title = array('site_title' => settings_item('site.title'), 'page_title' => $page_detail->title);
	        $page_title = $this->parser->parse_string(settings_item('seo.page_meta_title'), $meta_title, TRUE);
	    } else {
	        $page_title = $page_detail->title;
	    }
	    
	    if(!empty($page_detail->meta_description)) {
	        $page_description = $page_detail->meta_description;
	    } else if(settings_item('seo.page_meta_description') != '') {
	        $meta_description = array('site_title' => settings_item('site.title'), 'page_title' => $page_detail->title);
	        $page_description = $this->parser->parse_string(settings_item('seo.page_meta_description'), $meta_description, TRUE);
	    } else {
	        $last_space = strrpos( substr( $page_detail->body, 0, 200 ), ' ' ); // find the last space within 35 characters
	        $page_description = strip_tags(substr( $page_detail->body, 0, $last_space ) . '...');
	    }
	    
	    if(!empty($page_detail->meta_keywords)) {
	        $page_keywords = $page_detail->meta_keywords;
	    } else if(settings_item('seo.page_meta_keywords') != '') {
	        $meta_keywords = array('site_title' => settings_item('site.title'), 'page_title' => $page_detail->title);
	        $page_keywords = $this->parser->parse_string(settings_item('seo.page_meta_keywords'), $meta_keywords, TRUE);
	    }
	    if(isset($page_keywords)) {
	        Template::set('page_keywords', $page_keywords);
	    }
	    
	    // update module hits
	    $this->gc->update_hits($page_detail->id, 'pages');
	    
	    // add/update recently viewed status
	    $this->gc->add_recently_viewed($page_detail->id, 'pages');
	    
	    if($page_detail) {
	        if($page_detail->contact_form == 1) {
	            Assets::add_js('intlTelInput.min.js');
	            Assets::add_css('intlTelInput.css');
	        }
	        
	        // Process Widgets if any
	        // FORMAT : {widget}id:79|type_id:0|cols:2{/widget}
	        $this->load->library('themes/widgets');
	        $page_detail->body = $this->widgets->process_widgets($page_detail->body, $page_detail->slug);
	        
	        //footer pages link
	        Template::set(array(
	            'toolbar_title' => $page_detail->title,
	            'title' => $page_detail->title,
	            'page_detail'=> $page_detail,
	            'page_title' => $page_title,
	            'page_description' => $page_description
	        ));
	        if($render == 'pages') {
	            $layout = $this->gc->load_assets('pages');
	        } else {
	            $layout = $this->gc->load_assets('members/pages');
	        }
	        Template::set_view($layout['view']);
	        Template::render($layout['layout']);
	    } else {
	        redirect(site_url());
	    }
	}
	
	/**
	 * Allows Adding/Editing of Stuff Data (Screenshots/Videos/Features).
	 *
	 * @return void
	 */
	public function stuff_form($module, $type = 'create') {
	    $table = 'page_' . $module;
	    $redirect_url = SITE_AREA .'/content/pages/manage/' . $module;
	    if($type == 'edit') {
	        $id = (int) $this->uri->segment(7);
	        if (empty($id)) {
	            Template::set_message(lang('invalid_id'), 'danger');
	            redirect($redirect_url);
	        }
	        $record = $this->db->where('id', $id)->get($table)->row();
	        $action = 'update';
	    } else {
	        $record = array();
	        $id = 0;
	        $action = 'insert';
	    }
	    $module_label = lang('singular_page_' . $module);
	    if (isset($_POST['save'])) {
	        $this->auth->restrict('Page_' . ucwords($module) . '.Content.' . ucwords($type));
	        if ($this->save_stuff($table, $action, $id)) {
	            // Log the activity
	            log_activity($this->auth->user_id(), lang('act_' . $type . '_record') .': '. $id .' : '. $this->input->ip_address(), $table);
	            Template::set_message(sprintf(lang('msg_save_success'), $module_label), 'success');
	            // redirect($redirect_url);
	        } else {
	            Template::set_message(sprintf(lang('msg_' . $type . '_failure'), $module_label), 'danger');
	        }
	    } else if (isset($_POST['delete'])) {
	        $this->auth->restrict('Page_' . ucwords($module) . '.Content.Delete');
	        
	        if ($this->db->where('id', $id)->delete($table)) {
	            log_activity($this->auth->user_id(), lang('act_delete_record') .': '. $id .' : '. $this->input->ip_address(), $table);
	            Template::set_message(sprintf(lang('msg_action_success'), $module_label, lang('label_deleted')), 'success');
	            redirect($redirect_url);
	        } else {
	            Template::set_message(sprintf(lang('msg_delete_failure'), $module_label), 'danger');
	        }
	    }
	    Template::set(array(
	        'toolbar_title' => lang($type . '_' . $table),
	        'record' => $record
	    ));
	    Template::set_view($module);
	    Template::render();
	}
	
	//--------------------------------------------------------------------
	// !PRIVATE METHODS
	//--------------------------------------------------------------------
	
	/**
	 * Summary
	 *
	 * @param String $type Either "insert" or "update"
	 * @param Int	 $id	The ID of the record to update, ignored on inserts
	 *
	 * @return Mixed    An INT id for successful inserts, TRUE for successful updates, else FALSE
	 */
	private function save_stuff($table, $type='insert', $id = 0) {
	    if ($type == 'update') {
	        $_POST['id'] = $id;
	    }
	    $this->form_validation->set_rules('group_id', 'lang:singular_groups', 'trim');
	    $this->form_validation->set_rules('stuff_type', 'lang:label_type', 'trim|required');
	    $stuff_type = $this->input->post('stuff_type');
	    switch($stuff_type) {
	        case 'screenshot':
	            if(empty($_POST['screenshot_url']) && empty($_POST['screenshot_image'])) {
	                $this->form_validation->set_rules('screenshot_image', 'lang:label_image', 'trim|required');
	            } else if(!empty($_POST['screenshot_url']) && !empty($_POST['screenshot_image'])) {
	                $this->form_validation->set_rules('screenshot_image', 'lang:label_image', 'trim|required|callback_one_field_only');
	            } else if(!empty($_POST['screenshot_url']) && empty($_POST['screenshot_image'])) {
	                $this->form_validation->set_rules('screenshot_url', 'lang:label_image_url', 'trim|required|valid_url');
	            } else if(!empty($_POST['screenshot_image'])) {
	                $this->form_validation->set_rules('screenshot_image', 'lang:label_image', 'trim|required|callback_valid_file[image]');
	            }
	            break;
	        case 'video':
	            $this->form_validation->set_rules('video_title', 'lang:params_video_title', 'trim');
	            $this->form_validation->set_rules('video_description', 'lang:label_description', 'trim');
	            $this->form_validation->set_rules('video_url', 'lang:placeholder_video_url', 'trim|valid_youtube_url|required');
	            break;
	        case 'feature':
	            $this->form_validation->set_rules('feature_title', 'lang:label_title', 'trim|required');
	            $this->form_validation->set_rules('feature_description', 'lang:label_description', 'trim|required');
	            break;
	    }
	    if ($this->form_validation->run() === false) {
	        return false;
	    }
	    // make sure we only pass in the fields we want
	    $data = array();
	    $data['group_id'] = $this->input->post('group_id');
	    $data['stuff_type']       = $this->input->post('stuff_type');
	    $params = array();
	    foreach($_POST as $key => $param_data) {
	        if(!empty($param_data) && in_array($key, $this->stuff_fields) && strpos($key, $stuff_type) !== false) {
	            $params[$key] = $param_data;
	        }
	    }
	    if($data['stuff_type'] == 'video') {
	        $params['video_id'] = get_video_id($params['video_url']);
	        if(!empty($params['video_id'])) {
	            $params['video_duration'] = get_video_duration($params['video_id']);
	        }
	        $params['video_title'] = !empty($params['video_title']) ? $params['video_title'] : get_video_title($params['video_id']);
	        $params['video_image'] = !empty($params['video_id']) ? 'https://img.youtube.com/vi/' . $params['video_id'] . '/0.jpg' : base_url() . 'assets/images/video.png';
	    }
	    $data['params']     = json_encode($params);
	    $data['priority']   = (int) $this->input->post('priority');
	    $data['status']     = (int) $this->input->post('status');
	    if ($type == 'insert') {
	        $data['created_on'] = date('Y-m-d H:i:s');
	        $this->db->insert($table, $data);
	        $id = $this->db->insert_id();
	        if (is_numeric($id)) {
	            $return = $id;
	        } else {
	            $return = FALSE;
	        }
	    } elseif ($type == 'update') {
	        $data['modified_on'] = date('Y-m-d H:i:s');
	        $return = $this->db->where('id', $id)->update($table, $data);
	    }
	    return $return;
	}
}