<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
 || ##################################################################### ||
 || # GoClixy - Business Directory Script							   # ||
 || # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
 || # ----------------------------------------------------------------  # ||
 || # This file is owned and copyrighted by GoClixy.com. Your license   # ||
 || # confers  no  title  or ownership in the Script and is not a sale  # ||
 || # of any rights to the Software. This file may not be redistributed # ||
 || # in whole or significant part.									   # ||
 || # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
 || # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
 || ##################################################################### ||
 \*=======================================================================*/
use Omnipay\Omnipay;

class Payment_gateways extends Front_Controller {
    
    private $user_info = array();
    private $order_info = array();
    private $item_title = '';
    private $item_description = '';
    private $amount = 0.00;
    private $order_id = '';
    private $renew_upgrade_order_id = '';
    private $payment_method = array();
    private $payment_currency = 'USD';
    private $mobile = false;
    private $transaction_cancelled;
    private $transaction_confirmed;
    
    //--------------------------------------------------------------------
    public function __construct() {
        parent::__construct();
        $this->load->library('form_validation');
        $this->load->library('gc');
        $this->load->model('payment_gateways_model', null, true);
        $this->load->model('packages/packages_model');
        $this->load->model('orders/orders_model');
        $this->load->library('users/auth');
        $this->lang->load('payment_gateways');
        $this->lang->load('orders/orders');
        if($this->input->get('order_id')) {
            $this->order_id = $this->input->get('order_id');
            $this->transaction_cancelled = site_url('payment_gateways/transaction_cancelled?order_id=' . $this->order_id);
            $this->transaction_confirmed = site_url('payment_gateways/transaction_confirmed?order_id=' . $this->order_id);
            $this->user_info = $this->payment_gateways_model->get_user_info($this->order_id);
            $this->order_id = $this->input->get('order_id');
            $this->db->join('users U', 'U.id = orders.user_id');
            $result = $this->orders_model->select('orders.*, U.checkout_params')->find_by(array('orders.order_id' => $this->order_id, 'orders.status_id != ' => 2, 'orders.deleted' => 0));
            if(!$result) {
                $this->_redirect();
            }
            $this->session->set_userdata('order_id', $this->input->get('order_id'));
            $checkout_params = !empty($result->checkout_params) ? json_decode($result->checkout_params, true) : array();
            $this->mobile = !empty($checkout_params['mobile']) ? true : false;
            $this->order_info = $result;
            $this->item_title = $this->order_id;
            $this->item_description = implode(PHP_EOL, (!empty($result->items) ? json_decode($result->items) : ''));
            
            // get payment gateway information from database
            $payment_gateway = $result->payment_method;
            $method_name = $this->payment_method(strtolower($payment_gateway));
            $gateway_settings = $this->payment_gateways_model->select('settings')->find_by('name', $method_name);
            if(!$gateway_settings) { // No Payment Gateway Found
                $this->transaction_cancelled();
            }
            //Retrieve settings from database
            $this->payment_method = unserialize($gateway_settings->settings);
            $this->amount = $result->order_total;
            //$this->session->set_userdata('amount', $result->order_total);
            if(isset($this->payment_method['currency'])) {
                $this->payment_currency = $this->payment_method['currency'];
                $this->amount = floatval($this->currencyConversion($this->order_id, $this->payment_currency, $result->order_total, $result->currency_code));
                /*if($paying_currency_method) {
                 $this->amount = $result->order_total;
                 } else {
                 $this->amount = floatval($this->currencyConversion($this->order_id, $this->payment_currency, $result->order_total));
                 }*/
            }
            // INR payment gateways
            switch($payment_gateway) {
                case 'ebs':
                case 'paytm':
                case 'payumoney':
                case 'ccavenue':
                case 'razorpay':
                    $this->amount = floatval($this->currencyConversion($this->order_id, 'INR', $this->order_info->order_total, $this->order_info->currency_code));
                    break;
            }
        } else {
            if($this->order_info->mobile_checkout == 1) {
                $this->_redirect();
            } else {
                Template::redirect(gc_url_format('members'));
            }
        }
    }
    
    /**
     * Get Payment Method Name
     * @param string $payment_gateway
     * @return NULL|string
     */
    private function payment_method($payment_gateway) {
        $method_name = null;
        switch($payment_gateway) {
            case 'banktransfer': // Individual
                $method_name = 'BankTransfer';
                break;
            case 'paypal': // Omnipay 3
                $method_name = 'PayPal';
                break;
            case 'mollie': // Omnipay 3
                $method_name = 'Mollie';
                break;
            case 'payfast': // Omnipay 3
                $method_name = 'PayFast';
                break;
            case 'ebs':
                $method_name = 'EBS';
                break;
            case 'payumoney':
                $method_name = 'PayUMoney';
                break;
            case 'twocheckout':
                $method_name = 'TwoCheckout';
                break;
            case 'authorizenet': // Omnipay 3
                $method_name = 'AuthorizeNet';
                break;
            case 'stripe': // Omnipay 3
                $method_name = 'Stripe';
                break;
            case 'interswitch':
                $method_name = 'Interswitch';
                break;
            case 'paystack': // Individual Package
                $method_name = 'PayStack';
                break;
            case 'ccavenue':
                $method_name = 'CCAvenue';
                break;
            case 'paytm': // Omnipay 3
                $method_name = 'Paytm';
                break;
            case 'wecashup': // Individual Package
                $method_name = 'WeCashUp';
                break;
            case 'square': // Omnipay 3
                $method_name = 'Square';
                break;
            case 'razorpay': // Individual Package
                $method_name = 'Razorpay';
                break;
        }
        return $method_name;
    }
    
    /*----------------------------------------------------*/
    /*	Bank Transfer
     /*----------------------------------------------------*/
    /**
     * Bank Transfer
     */
    public function banktransfer() {
        // do not delete bank transfer payment methods on order change
        $this->db->where('order_id', $this->order_id)->update('orders', array('status_id' => 9));
        $this->db->where('order_id', $this->order_id)->update('order_items', array('status_id' => 9));
        Template::set_message ( $this->payment_method['payment_instructions'], 'success' );
        $this->load->model('orders/orders_model');
        $this->orders_model->email_print($this->order_id, 'email'); // send email
        if($this->order_info->mobile_checkout) {
            $this->_redirect();
        } else {
            // Admin checkout
            // $checkout_redirect = gc_url_format('members/view_invoice') . '?order_id=' . $this->order_id;
            $checkout_redirect = gc_url_format('members/detail') . '/' . $this->order_id;
            if($this->session->userdata('checkout_redirect')) {
                $checkout_redirect = $this->session->userdata('checkout_redirect');
            }
            $this->payment_gateways_model->unset_session($this->order_info->user_id);
            Template::redirect($checkout_redirect);
        }
    }
    
    /*----------------------------------------------------*/
    /*	PayPal Payment Gateway
     /*----------------------------------------------------*/
    public function paypal() {
        $testmode = false;
        if (!empty($this->payment_method['testmode'])) {
            $testmode = true;
        }
        $gateway = Omnipay::create('PayPal_Express');
        $gateway->setUsername($this->payment_method['api_username']);
        $gateway->setPassword($this->payment_method['api_password']);
        $gateway->setSignature($this->payment_method['api_signature']);
        $gateway->setTestMode($testmode);
        try {
            $response = $gateway->purchase(
                ['cancelUrl'=> $this->transaction_cancelled,
                    'returnUrl'=> site_url('payment_gateways/paypalCompletePurchase?order_id=' . $this->order_id),
                    'amount' => $this->amount,
                    'description' => lang('label_order_id') . ': ' . $this->order_id . ' ' . $this->item_description,
                    'currency' => $this->payment_currency]
                )->send();
                
                if ($response->isSuccessful()) {
                    // payment was successful: update database
                    $reference = $response->getTransactionReference();  // TODO; Check the reference/id with your database
                    // echo "Transaction '" . $response->getTransactionId() . "' succeeded!";
                } elseif ($response->isRedirect()) {
                    // redirect to offsite payment gateway
                    $response->redirect();
                    //return "Error " .$response->getCode() . ': ' . $response->getMessage();
                }
                else {
                    // payment failed: display message to customer
                    echo $response->getMessage();
                }
        } catch (Exception $e) {
            $this->_redirect(true, $e->getMessage()); // internal error, log exception and display a generic message to the customer
        }
    }
    
    /**
     * PayPal Complete Purchase Method
     * @return boolean
     */
    public function paypalCompletePurchase() {
        /**
         * completePurchase: completePurchase() part is required when Payment Method returns to your returnUrl.
         * Else payment is not considered successful.
         * Retrieve settings from database
         */
        $testmode = false;
        if (!empty($this->payment_method['testmode'])) {
            $testmode = true;
        }
        $gateway = Omnipay::create('PayPal_Express');
        $gateway->setUsername($this->payment_method['api_username']);
        $gateway->setPassword($this->payment_method['api_password']);
        $gateway->setSignature($this->payment_method['api_signature']);
        $gateway->setTestMode($testmode);
        $response = $gateway->completePurchase(array(
            'token' => $this->input->get('token'),
            'PayerID' => $this->input->get('PayerID'),
            'amount' => $this->amount,
            'description' => lang('label_order_id') . ': ' . $this->order_id . ' ' . $this->item_description,
            'currency' => $this->payment_currency,
        ))->send();
        
        $paypalResponse = $response->getData(); // this is the raw response object
        
        if ( ! $response->isSuccessful()) {
            Template::set_message ( $response->getMessage(), 'error' );
            $this->transaction_cancelled($response->getTransactionReference(), $this->input->get('PayerID'), $paypalResponse);
        }
        
        if(isset($paypalResponse['PAYMENTINFO_0_ACK']) && $paypalResponse['PAYMENTINFO_0_ACK'] === 'Success') {
            // Response
            $this->transaction_confirmed($response->getTransactionReference(), $this->input->get('PayerID'), $paypalResponse);
        } else {
            Template::set_message ( lang('transaction_cancelled'), 'error' );
            $this->transaction_cancelled($response->getTransactionReference(), $this->input->get('PayerID'), $paypalResponse);
        }
    }
    
    /**
     * PayPal Payment Gateway
     */
    public function paypal_recurring() {
        $order_id = $this->order_id;
        // get all ordered items of this order
        $all_order_items = $this->db->where('order_id', $order_id)->get('order_items')->result_array();
        
        // the recurring payment option is only available for packages
        $sql = "SELECT P.*, OT.module_id, OT.order_id";
        $sql .= " FROM " . $this->db->dbprefix . "packages P";
        $sql .= " INNER JOIN " . $this->db->dbprefix . "order_items OT ON OT.row_id = P.id AND OT.module_id = " . gc_module_id('packages');
        $sql .= " WHERE OT.order_id = '" . $order_id . "' AND P.recurring = 1";
        $order_item = $this->db->query($sql)->result_array();
        if(!empty($order_item) && $order_item[0]) {
            if(count($all_order_items) == count($order_item)) {
                $this->load->library('payment_gateways/paypal');
                $this->paypal->pay((object) $order_item[0]);
            } else {
                // remove other items from the cart and keep this only to proceed further
                $message = sprintf(lang('order_subscription_message'), $order_item[0]->title);
                Template::set_message($message, 'danger');
                Template::redirect(gc_url_format('members/checkout'));
            }
        } else {
            $testmode = false;
            if ($this->payment_method['testmode'] == 1) {
                $testmode = true;
            }
            $gateway = Omnipay::create('PayPal_Rest');
            // $gateway->setUsername($this->payment_method['api_username']);
            // $gateway->setPassword($this->payment_method['api_password']);
            // $gateway->setSignature($this->payment_method['api_signature']);
            // $gateway->setTestMode($testmode);
            // $gateway->setUsername($this->payment_method['client_id']);
            // $gateway->setUsername($this->payment_method['secret']);
            $gateway->initialize(array(
                'clientId' => $this->payment_method['client_id'],
                'secret'   => $this->payment_method['secret'],
                'testMode' => $testmode, // Or false when you are ready for live transactions
            ));
            try {
                $response = $gateway->purchase(
                    ['cancelUrl'=> $this->transaction_cancelled,
                        'returnUrl'=> site_url('payment_gateways/paypalCompletePurchase?order_id=' . $this->order_id),
                        'amount' => $this->amount,
                        'description' => lang('label_order_id') . ': ' . $this->order_id . ' ' . $this->item_description,
                        'currency' => $this->payment_currency]
                    )->send();
                    
                    if ($response->isSuccessful()) {
                        // payment was successful: update database
                        $reference = $response->getTransactionReference();  // TODO; Check the reference/id with your database
                        // echo "Transaction '" . $response->getTransactionId() . "' succeeded!";
                    } elseif ($response->isRedirect()) {
                        // redirect to offsite payment gateway
                        $response->redirect();
                        //return "Error " .$response->getCode() . ': ' . $response->getMessage();
                    }
                    else {
                        // payment failed: display message to customer
                        echo $response->getMessage();
                    }
            } catch (Exception $e) {
                $this->_redirect(true, $e->getMessage()); // internal error, log exception and display a generic message to the customer
            }
        }
    }
    
    /**
     * PayPal Complete Purchase Method
     * @return boolean
     */
    public function paypalCompletePurchase_recurring() {
        if(!empty($_GET['subscriptionID'])) { // recurring payment option
            $data = array();
            $data['subscriptionID'] = $_GET['subscriptionID'];
            $data['access_token'] = $_GET['access_token'];
            $this->load->library('payment_gateways/paypal');
            $result = $this->paypal->verify($data);
            if(!empty($result['success'])) {
                $this->transaction_confirmed($result['transaction_id'], $result['payer_id'], $result['payment_details']);
            } else {
                $this->transaction_cancelled($result['transaction_id'], $result['payer_id'], $result['payment_details']);
            }
        } else {
            /**
             * completePurchase: completePurchase() part is required when Payment Method returns to your returnUrl.
             * Else payment is not considered successful.
             * Retrieve settings from database
             */
            $testmode = false;
            if ($this->payment_method['testmode'] == 1) {
                $testmode = true;
            }
            /*$gateway = Omnipay::create('PayPal_Express');
             $gateway->setUsername($this->payment_method['api_username']);
             $gateway->setPassword($this->payment_method['api_password']);
             $gateway->setSignature($this->payment_method['api_signature']);
             $gateway->setTestMode($testmode);*/
            $gateway = Omnipay::create('PayPal_Rest');
            $gateway->initialize(array(
                'clientId' => $this->payment_method['client_id'],
                'secret'   => $this->payment_method['secret'],
                'testMode' => $testmode, // Or false when you are ready for live transactions
            ));
            $response = $gateway->completePurchase(array(
                'token' => $this->input->get('token'),
                'PayerID' => $this->input->get('PayerID'),
                'amount' => $this->amount,
                'description' => lang('label_order_id') . ': ' . $this->order_id . ' ' . $this->item_description,
                'currency' => $this->payment_currency,
            ))->send();
            
            $paypalResponse = $response->getData(); // this is the raw response object
            
            if ( ! $response->isSuccessful()) {
                Template::set_message ( $response->getMessage(), 'error' );
                $this->transaction_cancelled($response->getTransactionReference(), $this->input->get('PayerID'), $paypalResponse);
            }
            
            if(isset($paypalResponse['PAYMENTINFO_0_ACK']) && $paypalResponse['PAYMENTINFO_0_ACK'] === 'Success') {
                // Response
                $this->transaction_confirmed($response->getTransactionReference(), $this->input->get('PayerID'), $paypalResponse);
            } else {
                Template::set_message ( lang('transaction_cancelled'), 'error' );
                $this->transaction_cancelled($response->getTransactionReference(), $this->input->get('PayerID'), $paypalResponse);
            }
        }
    }
    
    /*----------------------------------------------------*/
    /*	Mollie Payment Gateway
     /*----------------------------------------------------*/
    public function mollie() {
        $testmode = false;
        if ($this->payment_method['testmode'] == 1) {
            $testmode = true;
        }
        $gateway = Omnipay::create('Mollie');
        $gateway->setApiKey($this->payment_method['apiKey']);
        $gateway->setTestMode($testmode);
        try {
            $response = $gateway->purchase(
                array(
                    'amount' => $this->amount,
                    'description' => lang('label_order_id') . ': ' . $this->order_id . ' ' . $this->item_description,
                    'returnUrl'=> site_url('payment_gateways/mollieCompletePurchase?order_id=' . $this->order_id),
                    'redirectUrl'=> site_url('payment_gateways/mollieCompletePurchase?order_id=' . $this->order_id),
                    'metadata'=> array('order_id' => $this->order_id)
                )
                )->send();
                
                if($response->isRedirect()){
                    /* IMPORTANT */
                    // SAVE TRANSACTIONREFERENCE IN SESSION TEMPORARILY AND ACCESS IT IN CONFIRMED TRANSACTION METHOD
                    $this->session->set_flashdata('reference', $response->getTransactionReference());
                    $response->redirect();
                } elseif($response->isPending()) {
                    return "Pending, Reference: ". $response->getTransactionReference();
                } else {
                    return "Error " .$response->getCode() . ': ' .$response->getMessage();
                }
        } catch (Exception $e) {
            $this->_redirect(true, $e->getMessage()); // internal error, log exception and display a generic message to the customer
        }
    }
    
    /**
     * Mollie Complete Purchase Method
     * @return boolean
     */
    public function mollieCompletePurchase() {
        $testmode = false;
        if ($this->payment_method['testmode'] == 1) {
            $testmode = true;
        }
        $gateway = Omnipay::create('Mollie');
        $gateway->setApiKey($this->payment_method['apiKey']);
        $gateway->setTestMode($testmode);
        $response = $gateway->completePurchase(
            array(
                'amount' => $this->amount,
                'transactionReference' => $this->session->flashdata('reference')
            )
            )->send();
            $data = $response->getData();
            if ($response->isSuccessful()) {
                $this->transaction_confirmed($response->getTransactionReference(), null, $data);
            } else {
                Template::set_message ( $response->getMessage(), 'error' );
                $this->transaction_cancelled($response->getTransactionReference(), null, $data);
            }
    }
    
    /*----------------------------------------------------*/
    /*	PayFast Payment Gateway
     /*----------------------------------------------------*/
    public function payfast() {
        $testmode = false;
        if ($this->payment_method['testmode'] == 1) {
            $testmode = true;
        }
        $gateway = Omnipay::create('PayFast');
        $gateway->setMerchantId($this->payment_method['merchantId']);
        $gateway->setMerchantKey($this->payment_method['merchantKey']);
        $gateway->setPdtKey($this->payment_method['pdtKey']);
        $gateway->setPassphrase($this->payment_method['passphrase']);
        $gateway->setTestMode($testmode);
        try {
            $response = $gateway->purchase(
                array(
                    'amount' => $this->amount,
                    'transactionId' => $this->order_id,
                    'description' => lang('label_order_id') . ': ' . $this->order_id, // do not add more description, it will not work
                    'returnUrl'=> site_url('payment_gateways/payfastCompletePurchase?order_id=' . $this->order_id),
                    'cancelUrl'=> site_url('payment_gateways/payfastCompletePurchase?order_id=' . $this->order_id),
                    'notifyUrl'=> gc_url_format('members'),
                    'metadata'=> array('order_id' => $this->order_id)
                )
                )->send();
                
                if($response->isRedirect()){
                    /* IMPORTANT */
                    // SAVE TRANSACTIONREFERENCE IN SESSION TEMPORARILY AND ACCESS IT IN CONFIRMED TRANSACTION METHOD
                    $this->session->set_flashdata('reference', $response->getTransactionReference());
                    $response->redirect();
                } elseif($reponse->isPending()) {
                    return "Pending, Reference: ". $response->getTransactionReference();
                } else {
                    return "Error " .$response->getCode() . ': ' .$response->getMessage();
                }
        } catch (Exception $e) {
            $this->_redirect(true); // internal error, log exception and display a generic message to the customer
        }
    }
    
    /**
     * PayFast Complete Purchase Method
     * @return boolean
     */
    public function payfastCompletePurchase() {
        $testmode = false;
        if ($this->payment_method['testmode'] == 1) {
            $testmode = true;
        }
        $gateway = Omnipay::create('PayFast');
        
        $gateway->setMerchantId($this->payment_method['merchantId']);
        $gateway->setMerchantKey($this->payment_method['merchantKey']);
        $gateway->setPdtKey($this->payment_method['pdtKey']);
        $gateway->setPassphrase($this->payment_method['passphrase']);
        
        $gateway->setTestMode($testmode);
        
        $response = $gateway->completePurchase(
            array(
                'amount' => $this->amount,
                'transactionId' => $this->order_id,
                'description' => lang('label_order_id') . ': ' . $this->order_id . ' ' . $this->item_description,
                'returnUrl'=> site_url('payment_gateways/payfastCompletePurchase?order_id=' . $this->order_id),
                'cancelUrl'=> site_url('payment_gateways/payfastCompletePurchase?order_id=' . $this->order_id),
                'redirectUrl'=> site_url('payment_gateways/payfastCompletePurchase?order_id=' . $this->order_id),
                'metadata'=> array('order_id' => $this->order_id)
            )
            )->send();
            $data = $response->getData();
            if ($response->isSuccessful()) {
                $this->transaction_confirmed($response->getTransactionReference(), null, $data);
            } else {
                Template::set_message ( $response->getMessage(), 'error' );
                $this->transaction_cancelled($response->getTransactionReference(), null, $data);
            }
    }
    
    /*----------------------------------------------------*/
    /*	EBS Payment Gateway
     /*----------------------------------------------------*/
    public function ebs() {
        $testmode = 'LIVE';
        if ($this->payment_method['testmode'] == 1) {
            $testmode = 'TEST';
        }
        $gateway = Omnipay::create('EBS');
        $gateway->setAccountId($this->payment_method['accountId']);
        $gateway->setSecretKey($this->payment_method['secretKey']);
        $gateway->setReferenceNo($this->order_id);
        $gateway->setTestMode($testmode);
        try {
            $response = $gateway->purchase(
                array(
                    'amount' => $this->amount,
                    'description' => $this->item_description,
                    'returnUrl'=> site_url('payment_gateways/ebsCompletePurchase?order_id=' . $this->order_id),
                    'redirectUrl'=> site_url('payment_gateways/ebsCompletePurchase?order_id=' . $this->order_id),
                    'userInfo' => $this->user_info
                )
                )->send();
                
                if($response->isRedirect()){
                    $response->redirect();
                } elseif($reponse->isPending()) {
                    return "Pending, Reference: ". $response->getTransactionReference();
                } else {
                    return "Error " .$response->getCode() . ': ' .$response->getMessage();
                }
        } catch (Exception $e) {
            $this->_redirect(true); // internal error, log exception and display a generic message to the customer
        }
    }
    
    /**
     * EBS Complete Purchase Method
     * @return boolean
     */
    public function ebsCompletePurchase() {
        if($_GET) {
            if($_GET['ResponseCode'] == 0) {
                $this->transaction_confirmed($_GET['TransactionID'], null, $_GET);
            } else {
                Template::set_message ( $response->getMessage(), 'error' );
                $this->transaction_cancelled($_GET['TransactionID'], null, $_GET);
            }
        } else {
            Template::set_message ( $response->getMessage(), 'error' );
        }
    }
    
    /*----------------------------------------------------*/
    /*	PayUMoney Payment Gateway
     /*----------------------------------------------------*/
    public function payumoney() {
        $testmode = false;
        if ($this->payment_method['testmode'] == 1) {
            $testmode = true;
        }
        $gateway = Omnipay::create('PayUMoney');
        $gateway->setMerchantKey($this->payment_method['merchantKey']);
        $gateway->setSalt($this->payment_method['salt']);
        $gateway->setTxnId($this->order_id);
        $gateway->setTestMode($testmode);
        try {
            $response = $gateway->purchase(
                array(
                    'amount' => (float) $this->amount,
                    'description' => $this->item_title,
                    'returnUrl'=> site_url('payment_gateways/payumoneyCompletePurchase?order_id=' . $this->order_id),
                    'redirectUrl'=> site_url('payment_gateways/payumoneyCompletePurchase?order_id=' . $this->order_id),
                    'userInfo' => $this->user_info
                )
                )->send();
                
                if($response->isRedirect()){
                    $response->redirect();
                } elseif($reponse->isPending()) {
                    return "Pending, Reference: ". $response->getTransactionReference();
                } else {
                    return "Error " .$response->getCode() . ': ' .$response->getMessage();
                }
        } catch (Exception $e) {
            $this->_redirect(true); // internal error, log exception and display a generic message to the customer
        }
    }
    
    /**
     * PayUMoney Complete Purchase Method
     * @return boolean
     */
    public function payumoneyCompletePurchase() {
        if($_POST) {
            $status	= $_POST['status'];
            $firstname = $_POST['firstname'];
            $amount = $_POST['amount'];
            $txnid = $_POST['txnid'];
            $posted_hash = $_POST['hash'];
            $key = $_POST['key'];
            $productinfo = $_POST['productinfo'];
            $email = $_POST['email'];
            $salt = $this->payment_method['salt'];
            if($_POST['status'] == 'success') {
                $retHashSeq = $salt.'|'.$status.'|||||||||||'.$email.'|'.$firstname.'|'.$productinfo.'|'.$amount.'|'.$txnid.'|'.$key;
                $hash = strtolower(hash("sha512", $retHashSeq));
                if ($hash == $posted_hash) {
                    $this->transaction_confirmed($_POST['mihpayid'], null, $_POST);
                } else {
                    Template::set_message ( $response->getMessage(), 'error' );
                    $this->transaction_cancelled($_POST['mihpayid'], null, $_POST);
                }
            } else {
                Template::set_message (lang('msg_payment_processing_error'), 'error' );
                $this->transaction_cancelled($_POST['mihpayid'], null, $_POST);
            }
        } else {
            $this->_redirect(true); // internal error, log exception and display a generic message to the customer
        }
    }
    
    /*----------------------------------------------------*/
    /*	2Checkout Payment Gateway
     /*----------------------------------------------------*/
    /**
     * 2CHECKOUT Payment Gateway
     */
    public function twocheckout() {
        $testmode = false;
        if ($this->payment_method['testmode'] == 1) {
            $testmode = true;
        }
        $gateway = Omnipay::create('TwoCheckout');
        $gateway->setAccountNumber($this->payment_method['api_account_number']);
        $gateway->setSecretWord($this->payment_method['api_secret_word']);
        $gateway->setTestMode($testmode);
        try {
            $response = $gateway->purchase(
                ['transactionId' => $this->order_id,
                    'cancelUrl'	=> site_url('payment_gateways/twocheckoutCompletePurchase?order_id=' . $this->order_id),
                    'returnUrl' => site_url('payment_gateways/twocheckoutCompletePurchase?order_id=' . $this->order_id),
                    'amount' => $this->amount,
                    'currency' => $this->payment_currency]
                )->send();
                if ($response->isSuccessful()) {
                } elseif ($response->isRedirect()) {
                    // redirect to offsite payment gateway
                    $response->redirect();
                } else {
                    echo $response->getMessage();
                }
        } catch (Exception $e) {
            $this->_redirect(true, $e->getMessage()); // internal error, log exception and display a generic message to the customer
        }
    }
    
    /**
     * 2Checkout Complete Purchase Method
     * @return boolean
     */
    public function twocheckoutCompletePurchase() {
        $testmode = false;
        if ($this->payment_method['testmode'] == 1) {
            $testmode = true;
        }
        $gateway = Omnipay::create('TwoCheckout');
        $gateway->setAccountNumber($this->payment_method['api_account_number']);
        $gateway->setSecretWord($this->payment_method['api_secret_word']);
        $gateway->setTestMode($testmode);
        try {
            $response = $gateway->completePurchase(
                ['transactionId' => $this->order_id,
                    'cancelUrl'	=> site_url('payment_gateways/twocheckoutCompletePurchase?order_id=' . $this->order_id),
                    'returnUrl' => site_url('payment_gateways/twocheckoutCompletePurchase?order_id=' . $this->order_id),
                    'amount' => $this->amount,
                    'currency' => $this->payment_currency]
                )->send();
                $twocheckoutResponse = $response->getData();
                if ( ! $response->isSuccessful()) {
                    Template::set_message ( lang('msg_payment_processing_error'), 'error' );
                    $this->transaction_cancelled($response->getTransactionReference(), $this->input->post('sid'), $twocheckoutResponse);
                } else {
                    $this->transaction_confirmed($response->getTransactionReference(), $this->input->post('sid'), $twocheckoutResponse);
                }
        } catch (Exception $e) {
            $this->_redirect(true, $e->getMessage()); // internal error, log exception and display a generic message to the customer
        }
    }
    
    /*----------------------------------------------------*/
    /*	AuthorizeNet SIM Payment Gateway
     /*----------------------------------------------------*/
    /**
     * AuthorizeNet SIM Payment Gateway
     */
    public function authorizenet() {
        /**
         * AuthorizeNET uses x_relay_url as the Return URL
         * This URL must be valid and accessible else AuthorizeNET will send an Error Message
         * CSRF protection must be disabled for AuthorizeNET Complete Purchase URL via config.php
         * Registered User Restriction ($this->auth->restrict()) must be removed  from the controller constructor.
         * Payment Gateway model must be loaded as $this->load->model('payment_gateways/payment_gateways_model'); instead of $this->load->model('payment_gateways_model', null, true);
         * Complete Purchase URL must be Publically Accessible.
         */
        $testmode = false;
        if ($this->payment_method['testmode'] == 1) {
            $testmode = true;
        }
        $gateway = Omnipay::create('AuthorizeNet_SIM');
        $gateway->setApiLoginId($this->payment_method['login_id']);
        $gateway->setTransactionKey($this->payment_method['transaction_key']);
        $gateway->setHashSecret($this->payment_method['hash']);
        $gateway->setTestMode($testmode);
        $gateway->setDeveloperMode($testmode);
        try{
            $response = $gateway->purchase(
                ['transactionId' => $this->order_id,
                    'cancelUrl'=> site_url('payment_gateways/authorizenetCompletePurchase?order_id=' . $this->order_id),
                    'returnUrl'=> site_url('payment_gateways/authorizenetCompletePurchase?order_id=' . $this->order_id),
                    'amount' => $this->amount,
                    'x_currency_code' => $this->payment_currency]
                )->send();
                
                if ($response->isSuccessful()) {
                } elseif ($response->isRedirect()) {
                    $response->redirect();
                }
        } catch (Exception $e) {
            $this->_redirect(true, $e->getMessage()); // internal error, log exception and display a generic message to the customer
        }
    }
    
    /**
     * AuthorizeNET Complete Purchase Method
     * @return boolean
     */
    public function authorizenetCompletePurchase() {
        $testmode = false;
        if ($this->payment_method['testmode'] == 1) {
            $testmode = true;
        }
        $gateway = Omnipay::create('AuthorizeNet_SIM');
        $gateway->setApiLoginId($this->payment_method['login_id']);
        $gateway->setTransactionKey($this->payment_method['transaction_key']);
        $gateway->setHashSecret($this->payment_method['hash']);
        $gateway->setTestMode($testmode);
        $gateway->setDeveloperMode($testmode);
        try{
            $response = $gateway->completePurchase(
                ['transactionId' => $this->order_id,
                    'cancelUrl'=> site_url('payment_gateways/authorizenetCompletePurchase?order_id=' . $this->order_id),
                    'returnUrl'=> site_url('payment_gateways/authorizenetCompletePurchase?order_id=' . $this->order_id),
                    'amount' => $this->amount,
                    'x_currency_code' => $this->payment_currency]
                )->send();
                
                $authorizenetResponse = $response->getData();
                if ( ! $response->isSuccessful() && ($this->input->post('x_response_code') != 1)) {
                    Template::set_message ( lang('msg_payment_processing_error'), 'error' );
                    $this->transaction_cancelled($response->getTransactionReference(), $_POST['x_invoice_num'], $authorizenetResponse);
                } else {
                    $this->transaction_confirmed($response->getTransactionReference(), $_POST['x_invoice_num'], $authorizenetResponse);
                }
        } catch (Exception $e) {
            $this->_redirect(true, $e->getMessage()); // internal error, log exception and display a generic message to the customer
        }
    }
    
    /*----------------------------------------------------*/
    /*	AuthorizeNet AIM Payment Gateway
     /*----------------------------------------------------*/
    /**
     * AuthorizeNet AIM Payment Gateway
     */
    public function authorizenet_aim() {
        $testmode = false;
        if ($this->payment_method['testmode'] == 1) {
            $testmode = true;
        }
        $gateway = Omnipay::create('AuthorizeNet_AIM');
        $gateway->setApiLoginId($this->payment_method['login_id']);
        $gateway->setTransactionKey($this->payment_method['transaction_key']);
        $gateway->setTestMode($testmode);
        $gateway->setDeveloperMode(true);
        try {
            $response = $gateway->purchase(
                [
                    'cancelUrl'=> $this->transaction_cancelled,
                    'returnUrl'=> $this->transaction_confirmed,
                    'amount' => floatval($this->session->flashdata('amount')), //convert amount to float value before passing
                    'x_currency_code' => $this->payment_currency,
                    'card' => array('number' => '4007000000027', 'expiryMonth' => 01, 'expiryYear' => 22),
                    'transactionReference' => $this->session->userdata('transaction_id'),
                    'transactionId' => $this->session->userdata('transaction_id')
                ]
                )->send();
                
                if ($response->isSuccessful()) {
                    $this->transaction_confirmed($transaction_id, null, $data, true);
                } elseif ($response->isRedirect()) {
                    // redirect to offsite payment gateway
                    $response->redirect();
                }
        } catch (Exception $e) {
            $this->_redirect(true); // internal error, log exception and display a generic message to the customer
        }
    }
    
    /*----------------------------------------------------*/
    /*	Stripe Payment Gateway
     /*----------------------------------------------------*/
    
    /**
     * Stripe Payments using Stripe-PHP
     * https://github.com/stripe/stripe-php
     */
    public function stripe() {
        $card = unserialize($this->session->userdata('cc_info'));
        if(isset($card['expiryMonth'])) {
            $card['exp_month'] = (int) $card['expiryMonth'];
            unset($card['expiryMonth']);
        }
        if(isset($card['expiryYear'])) {
            $card['exp_year'] = (int) $card['expiryYear'];
            unset($card['expiryYear']);
        }
        if(isset($card['cvv'])) {
            $card['cvc'] = $card['cvv'];
            unset($card['cvv']);
        }
        $this->session->unset_userdata('cc_info');
        \Stripe\Stripe::setApiKey($this->payment_method['secret_key']);
        try {
            $token = \Stripe\Token::create([
                'card' => $card
            ]);
            $item_description = lang('label_order_id') . ': ' . $this->order_id . ' ' . $this->item_description;
            $charge = \Stripe\Charge::create(['amount' => (float) $this->amount * 100, 'currency' => $this->payment_currency, 'source' => $token, 'description' => $item_description]);
            $data = json_encode($charge);
            if($charge->status == 'succeeded') {
                $transaction_id = $charge->id;
                $this->transaction_confirmed($transaction_id, null, $data, true);
            } else {
                $transaction_id = $charge->id;
                $this->transaction_cancelled($transaction_id, null, $data, true);
            }
            //catch exception
        } catch(Exception $e) {
            $this->_redirect(true, $e->getMessage()); // internal error, log exception and display a generic message to the customer
        }
    }
    /**
     * Stripe Payment Gateway using Omnipay - Not Working
     */
    public function stripe_omnipay() {
        $this->payment_gateway = 'stripe';
        $testmode = false;
        if ($this->payment_method['testmode'] == 1) {
            $testmode = true;
        }
        // Example form data
        /*$formData = [
         'number' => '4242424242424242',
         'expiryMonth' => '6',
         'expiryYear' => '2016',
         'cvv' => '123'
         ];*/
        
        $gateway = Omnipay::create('Stripe');
        $gateway->setApiKey($this->payment_method['secret_key']);
        $gateway->setTestMode($testmode);
        try {
            // generate token first
            $response = $gateway->createToken(array('card' => unserialize($this->session->userdata('cc_info'))))->send();
            $data = $response->getData();
            if ($response->isSuccessful()) {
                $token = $data['id'];
                $response = $gateway->purchase(
                    [
                        'cancelUrl'=> site_url('payment_gateways/transaction_cancelled?order_id=' . $this->order_id),
                        'amount' => decimal_convert($this->amount),
                        'token' => $token,
                        'description' => lang('label_order_id') . ': ' . $this->order_id . ' ' . $this->item_description,
                        'currency' => $this->payment_currency
                    ]
                    )->send();
                    
                    $data = $response->getData();
                    if ($response->isSuccessful()) {
                        // payment was successful: update database
                        $this->session->unset_userdata('cc_info');
                        $this->transaction_confirmed($response->getTransactionReference(), null, $data);
                    } elseif ($response->isRedirect()) {
                        // redirect to offsite payment gateway
                        Template::set_message ( $response->getMessage(), 'error' );
                        $this->session->unset_userdata('cc_info');
                        $this->transaction_cancelled($response->getTransactionReference(), null, $data);
                    } else {
                        // payment failed: display message to customer
                        Template::set_message ( $response->getMessage(), 'error' );
                        $this->session->unset_userdata('cc_info');
                        $this->transaction_cancelled($response->getTransactionReference(), null, $data);
                    }
            }
        } catch (Exception $e) {
            // internal error, log exception and display a generic message to the customer
            Template::set_message(lang('msg_payment_processing_error'), 'danger');
            $this->session->unset_userdata('cc_info');
            Template::redirect(gc_url_format('members'));
        }
    }
    
    /*----------------------------------------------------*/
    /*	Interswitch Payment Gateway
     /*----------------------------------------------------*/
    public function interswitch() {
        $testmode = false;
        if ($this->payment_method['testmode'] == 1) {
            $testmode = true;
        }
        $gateway = Omnipay::create('Interswitch');
        $gateway->setMacKey($this->payment_method['mac_key']);
        $gateway->setPayItemId($this->payment_method['pay_item_id']);
        $gateway->setProductId($this->payment_method['product_id']);
        $gateway->setCurrency($this->payment_currency);
        $gateway->setTestMode($testmode);
        try {
            $response = $gateway->purchase(
                array(
                    'amount' => $this->amount,
                    'transactionId' => $this->order_id,
                    'returnUrl'=> site_url('payment_gateways/interswitchCompletePurchase?order_id=' . $this->order_id)
                )
                )->send();
                
                if($response->isRedirect()){
                    /* IMPORTANT */
                    // SAVE TRANSACTIONREFERENCE IN SESSION TEMPORARILY AND ACCESS IT IN CONFIRMED TRANSACTION METHOD
                    //$this->session->set_flashdata('reference', $order_id);
                    $response->redirect();
                } elseif($response->isPending()) {
                    return "Pending, Reference: ". $response->getTransactionReference();
                } else {
                    return "Error " .$response->getCode() . ': ' .$response->getMessage();
                }
        } catch (Exception $e) {
            $this->_redirect(true); // internal error, log exception and display a generic message to the customer
        }
    }
    
    /**
     * Interswitch Complete Purchase Method
     * @return boolean
     */
    public function interswitchCompletePurchase() {
        $testmode = false;
        if ($this->payment_method['testmode'] == 1) {
            $testmode = true;
        }
        $gateway = Omnipay::create('Interswitch');
        $gateway->setMacKey($this->payment_method['mac_key']);
        $gateway->setPayItemId($this->payment_method['pay_item_id']);
        $gateway->setProductId($this->payment_method['product_id']);
        $gateway->setCurrency($this->payment_currency);
        $gateway->setTestMode($testmode);
        $response = $gateway->completePurchase(
            array(
                'amount' => $_POST['amount'],
                'txnref' => $_POST['txnref']
            )
            )->send();
            $data = $response->getData();
            if ($response->isSuccessful()) {
                $this->transaction_confirmed($response->getTransactionReference(), null, $data);
            } else {
                Template::set_message ( $response->getMessage(), 'error' );
                $this->transaction_cancelled($response->getTransactionReference(), null, $data);
            }
    }
    
    /*--------------------------------------------------------------------*/
    /*	PayStack Payment Gateway - https://github.com/yabacon/paystack-php
     /*--------------------------------------------------------------------*/
    public function paystack() {
        $email = $this->user_info['email'];
        $reference = $this->order_id;
        $paystack = new Yabacon\Paystack($this->payment_method['secret_key']);
        $amount = $this->amount; // Multipy the amount by 100 if their is any price difference, as faced while testing
        try {
            $tranx = $paystack->transaction->initialize([
                'amount'=>$amount,       // in kobo
                'email'=>$email,         // unique to customers
                'reference'=>$reference, // unique to transactions
                'callback_url'=> site_url('payment_gateways/paystackCompletePurchase?order_id=' . $this->order_id)
            ]);
        } catch(\Yabacon\Paystack\Exception\ApiException $e) {
            Template::set_message ( $e->getMessage(), 'error' );
            $this->transaction_cancelled($reference, null, $e->getResponseObject());
            //print_r($e->getResponseObject());
            //die($e->getMessage());
        }
        
        // paystack issue fix
        /**
         * Modify the /vendor/yabacon/paystack-php/src/Paystack/Helpers/Router.php line number 100 try block
         * return !empty($paystackObj->custom_routes) ? $paystackObj->custom_routes[$this->route] : 'Yabacon\\Paystack\\Routes\\' . ucwords($this->route);
         */
        
        // store transaction reference so we can query in case user never comes back
        // perhaps due to network issue
        // save_last_transaction_reference($tranx->data->reference);
        $this->db->where(array('order_id' => $this->order_id))->update('orders', array('transaction_id' => $tranx->data->reference));
        // redirect to page so User can pay
        Template::redirect($tranx->data->authorization_url);
    }
    
    /**
     * PayStack Complete Purchase Method
     * @return boolean
     */
    public function paystackCompletePurchase() {
        $reference = isset($_GET['reference']) ? $_GET['reference'] : '';
        if(!$reference){
            die('No reference supplied');
        }
        // initiate the Library's Paystack Object
        $paystack = new Yabacon\Paystack($this->payment_method['secret_key']);
        try {
            // verify using the library
            $tranx = $paystack->transaction->verify([
                'reference'=>$reference, // unique to transactions
            ]);
        } catch(\Yabacon\Paystack\Exception\ApiException $e) {
            Template::set_message ( $e->getMessage(), 'error' );
            $this->transaction_cancelled($reference, null, $e->getResponseObject());
        }
        
        if ('success' === $tranx->data->status) {
            // transaction was successful...
            // please check other things like whether you already gave value for this ref
            // if the email matches the customer who owns the product etc
            // Give value
            $this->transaction_confirmed($reference, null, $tranx->data);
        } else {
            // Template::set_message ( $response->getMessage(), 'error' );
            $this->transaction_cancelled($reference, null, $tranx->data);
        }
    }
    
    /*----------------------------------------------------*/
    /*	CCAvenue Payment Gateway
    /*----------------------------------------------------*/
    public function ccavenue() {
        $this->load->library('payment_gateways/ccavenue');
        $this->order_info->item_description = $this->item_description;
        $this->order_info->amount = $this->amount;
        $this->ccavenue->pay($this->order_info, $this->user_info);
    }
    
    /**
     * CCavenue Complete Purchase
     */
    public function ccavenueCompletePurchase() {
        $this->load->library('payment_gateways/ccavenue');
        $result = $this->ccavenue->verify();
        if(!empty($result['success'])) {
            $this->transaction_confirmed($result['transaction_id'], $result['payer_id'], $result['payment_details']);
        } else {
            $this->transaction_cancelled($result['transaction_id'], $result['payer_id'], $result['payment_details']);
        }
    }
    
    /*----------------------------------------------------*/
    /*	Paytm Payment Gateway
     /*----------------------------------------------------*/
    public function paytm() {
        $testmode = false;
        if (!empty($this->payment_method['testmode'])) {
            $testmode = true;
        }
        $gateway = Omnipay::create('Paytm_Express');
        $gateway->setMID($this->payment_method['mid']);
        $gateway->setKey($this->payment_method['merchant_key']);
        $gateway->setIndustryTypeId($this->payment_method['industry_type_id']);
        $gateway->setChannelId($this->payment_method['channel_id']);
        $gateway->setWebSite($this->payment_method['website']);
        $gateway->setEnvironment($testmode ? 'staging' : 'production');
        $params = [
            'ChannelID' => $this->payment_method['channel_id'],
            'IndustryTypeID' => $this->payment_method['industry_type_id'],
            'orderId' => $this->order_id,
            'CustID' => $this->order_info->user_id . '_' . mt_rand(10,1000),
            'CallbackURL' => site_url('payment_gateways/paytmCompletePurchase?order_id=' . $this->order_id),
            'amount' => $this->amount
        ];
        try {
            $response = $gateway->purchase($params)->send();
            
            if($response->isRedirect()) {
                $response->redirect();
            } elseif($reponse->isPending()) {
                return "Pending, Reference: ". $response->getTransactionReference();
            } else {
                return "Error " .$response->getCode() . ': ' .$response->getMessage();
            }
        } catch (Exception $e) {
            $this->_redirect(true);
        }
    }
    
    /**
     * Paytm Complete Purchase Method
     * @return boolean
     */
    public function paytmCompletePurchase() {
        $testmode = false;
        if (!empty($this->payment_method['testmode'])) {
            $testmode = true;
        }
        $gateway = Omnipay::create('Paytm_Express');
        $gateway->setMID($this->payment_method['mid']);
        $gateway->setKey($this->payment_method['merchant_key']);
        
        $response = $gateway->completePurchase(['request_params' => $_REQUEST])->send();
        $data = $response->getData();
        if ($data['STATUS'] == 'TXN_SUCCESS') {
            $this->transaction_confirmed($response->getTransactionReference(), null, $data);
        } else {
            Template::set_message ( $response->getMessage(), 'danger' );
            $this->transaction_cancelled($response->getTransactionReference(), null, $data);
        }
    }
    
    /*----------------------------------------------------*/
    /*	WeCashUp Payment Gateway
     /*----------------------------------------------------*/
    public function wecashup() {
        if(isset($_POST['cancel'])) {
            // redirect user to the transaction cancelled page
            Template::set_message(lang('msg_payment_processing_error'), 'error');
            $this->transaction_cancelled();
        }
        $merchant_uid = $this->payment_method['merchant_uid'];
        $merchant_public_key = $this->payment_method['merchant_public_key'];
        $merchant_secret = $this->payment_method['merchant_secret'];
        if(!empty($_POST)) {
            header('Access-Control-Allow-Origin: *');
            header('Access-Control-Allow-Methods: GET, POST');
            $transaction_uid = '';// create an empty transaction_uid
            $transaction_token  = '';// create an empty transaction_token
            $transaction_provider_name  = ''; // create an empty transaction_provider_name
            $transaction_confirmation_code  = ''; // create an empty confirmation code
            if(isset($_POST['transaction_uid'])){
                $transaction_uid = $_POST['transaction_uid']; // Get the transaction_uid posted by the payment box
            }
            if(isset($_POST['transaction_token'])){
                $transaction_token  = $_POST['transaction_token']; // Get the transaction_token posted by the payment box
            }
            if(isset($_POST['transaction_provider_name'])){
                $transaction_provider_name  = $_POST['transaction_provider_name']; // Get the transaction_provider_name posted by the payment box
            }
            if(isset($_POST['transaction_confirmation_code'])){
                $transaction_confirmation_code  = $_POST['transaction_confirmation_code']; // Get the transaction_confirmation_code posted by the payment box
            }
            $url = 'https://www.wecashup.com/api/v2.0/merchants/'.$merchant_uid.'/transactions/'.$transaction_uid.'?merchant_public_key='.$merchant_public_key;
            //Steps 7 : You must complete this script at this to save the current transaction in your database.
            /* Provide a table with at least 5 columns in your database capturing the following
             /  transaction_uid | transaction_confirmation_code| transaction_token| transaction_provider_name | transaction_status */
            //Step 8 : Sending data to the WeCashUp Server
            
            $fields = array(
                'merchant_secret' => urlencode($merchant_secret),
                'transaction_token' => urlencode($transaction_token),
                'transaction_uid' => urlencode($transaction_uid),
                'transaction_confirmation_code' => urlencode($transaction_confirmation_code),
                'transaction_provider_name' => urlencode($transaction_provider_name),
                '_method' => urlencode('PATCH')
            );
            $fields_string = '';
            foreach($fields as $key => $value) {
                $fields_string .= $key.'='.$value.'&';
            }
            rtrim($fields_string, '&');
            $ch = curl_init();
            
            curl_setopt($ch, CURLOPT_URL,$url);
            curl_setopt($ch, CURLOPT_POST, count($fields));
            curl_setopt($ch, CURLOPT_POSTFIELDS, $fields_string);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            
            //Step 9  : Retrieving the WeCashUp Response
            $server_output = curl_exec ($ch);
            curl_close ($ch);
            $data = !empty($server_output) ? json_decode($server_output, true) : array();
            $is_paid = false;
            $is_pending = false;
            if($data['response_status'] =="success") {
                echo lang('order_processed') . '<br />' . lang('msg_member_redirection');
                $is_paid = $this->wecashupCompletePurchase($url, $merchant_uid, $merchant_secret, $transaction_uid, $transaction_token);
                // redirect user to the transaction confirmed page
                $location = $this->transaction_confirmed($transaction_uid, null, $data, false);
            } else {
                // redirect user to the transaction cancelled page
                echo lang('msg_payment_processing_error') . '<br />' . lang('msg_member_redirection');
                Template::set_message(lang('msg_payment_processing_error'), 'error');
                $location = $this->transaction_cancelled($transaction_uid, null, $data, false);
            }
            echo '<script>top.window.location = "' . $location . '"</script>';
        } else {
            $record = new stdClass();
            $record->order_id = $this->order_id;
            $record->merchant_uid = $this->payment_method['merchant_uid'];
            $record->merchant_public_key = $this->payment_method['merchant_public_key'];
            $record->testmode = $this->payment_method['testmode'];
            $record->amount = $this->amount;
            $record->name = $this->user_info['name'];
            $record->phone_code = $this->user_info['phone_code'];
            $record->phone_number = $this->user_info['phone_number'];
            $record->item_title = strip_tags(ordered_item_info($this->order_info->item, $this->order_info->order_type, $return = 'title'));
            $record->item_description = $this->item_description;
            $record->item_price = $this->order_info->item_price;
            $record->item_qty = $this->order_info->qty;
            $this->gc->demo_custom_css_js('index');
            Template::set('record', $record);
            Template::set_view('wecashup');
            Template::render('blank');
        }
    }
    
    /**
     * Retrieve the transaction status after payment
     * It will not come to PAID immediately, we have to wait for few minutes
     * Use cronjob to find the status at regular interval
     * @param string $url
     * @param string $merchant_id
     * @param string $merchant_secret
     * @param string $transaction_id
     * @param string $transaction_token
     */
    public function wecashupCompletePurchase($url, $merchant_uid, $merchant_secret, $transaction_uid, $transaction_token) {
        $is_paid = false;
        $url .= '&merchant_secret=' . $merchant_secret;
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL,$url);
        curl_setopt( $ch, CURLOPT_CUSTOMREQUEST, 'GET' );
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        
        // Retrieving the WeCashUp Response
        $response = curl_exec ($ch);
        curl_close ($ch);
        $response_data = !empty($response) ? json_decode($response, true) : array();
        if(!empty($response_data['response_content']['transactions'][0])) {
            $response_content = $response_data['response_content']['transactions'][0];
            if( !empty($response_content['transaction_status']) && $response_content['transaction_status'] == 'PAID' &&
                !empty($response_content['transaction_uid']) && $response_content['transaction_uid'] == $transaction_uid &
                !empty($response_content['transaction_token']) && $response_content['transaction_token'] == $transaction_token) {
                    $is_paid = true;
                }
        }
        return $is_paid;
    }
    
    /*----------------------------------------------------*/
    /*	PayGateGlobal Gateway
     /*----------------------------------------------------*/
    public function paygateglobal() {
        $testmode = false;
        if($this->payment_method['method'] == 'POST') {
            // set post fields
            $post = [
                'auth_token' => $this->payment_method['api_key'],
                'phone_number' => $this->user_info['phone_number'],
                'amount'   => $this->amount,
                'description'   => urlencode(lang('label_order_id') . ': ' . $this->order_id),
                'identifier'   => $this->order_id,
            ];
            
            $ch = curl_init('https://paygateglobal.com/api/v1/pay');
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $post);
            
            // execute!
            $response = curl_exec($ch);
            
            // close the connection, release resources used
            curl_close($ch);
            
            // do anything you want with your response
            $response = !empty($response) ? json_decode($response) : array();
            if($response->status == 0) {
                
                $post = [
                    'auth_token' => $this->payment_method['api_key'],
                    'tx_reference' => $response->tx_reference
                ];
                
                // Check the status of a Payment
                $ch = curl_init('https://paygateglobal.com/api/v1/status');
                curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                curl_setopt($ch, CURLOPT_POSTFIELDS, $post);
                $response = curl_exec($ch);
                
                // close the connection, release resources used
                curl_close($ch);
                
                $response = !empty($response) ? json_decode($response) : null;
                $data = json_encode($response);
                if($response->status == 0) {
                    // transaction confirmed
                    $this->transaction_confirmed($response->payment_reference, null, $data);
                } else if($response->status == 2) {
                    // in progress
                    $this->transaction_cancelled($response->payment_reference, null, $data);
                    $this->db->where('order_id', $this->order_id)->update('orders', array('status' => 2));
                    Template::set_message(lang('msg_payment_in_progress'), 'info');
                } else if($response->status == 4) {
                    // expired
                    $this->transaction_cancelled($response->payment_reference, null, $data);
                    Template::set_message(lang('msg_payment_expired'), 'danger');
                } else if($response->status == 6) {
                    // cancelled
                    $this->transaction_cancelled($response->payment_reference, null, $data);
                    Template::set_message(lang('msg_payment_cancelled'), 'danger');
                }
                Template::redirect(gc_url_format('members/view_invoice') . '?order_id=' . $this->order_id);
            } else {
                $this->transaction_cancelled(null, null, null, true);
            }
        } else {
            $url = 'https://paygateglobal.com/v1/page?';
            $url .= 'token=' . $this->payment_method['api_key'];
            $url .= '&amount=' . $this->amount;
            $url .= '&description=' . urlencode(lang('label_order_id') . ': ' . $this->order_id);
            $url .= '&identifier=' . $this->order_id;
            $url .= '&url=' . urlencode(site_url('payment_gateways/payGateGlobalCompletePurchase?order_id=' . $this->order_id));
            Template::redirect($url);
        }
    }
    
    /**
     * PayGateGlobal Complete Purchase Method
     * @return boolean
     */
    public function payGateGlobalCompletePurchase() {
        if(empty($_POST)) {
            Template::set_message(lang('msg_payment_processing_error'), 'danger');
            $this->transaction_cancelled(null, null, array());
            Template::redirect(gc_url_format('members'));
        }
        $post = [
            'auth_token' => $this->payment_method['api_key'],
            'tx_reference' => $this->input->post('tx_reference')
        ];
        
        // Check the status of a Payment
        $ch = curl_init('https://paygateglobal.com/api/v1/status');
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $post);
        $response = curl_exec($ch);
        
        // close the connection, release resources used
        curl_close($ch);
        
        $response = !empty($response) ? json_decode($response) : null;
        $data = json_encode($response);
        if($response->status == 0) {
            $this->transaction_confirmed($response->payment_reference, null, $data);
        } else {
            Template::set_message(lang('msg_payment_processing_error'), 'danger');
            $this->transaction_cancelled($response->payment_reference, null, $data);
        }
    }
    
    /*----------------------------------------------------*/
    /*	Square Payment Gateway
     *
     * Note: Do not forget to modify the WebPaymentRequest.php file line number 57
     *
     * 'order' => array(
     'idempotency_key' => uniqid(),
     'order' => new SquareConnect\Model\Order(array(
     'location_id' => $this->getLocationId(),
     // 'reference_id' => $this->getTransactionReference(),
     'line_items' => $items_list
     )
     )),
     */
    /*----------------------------------------------------*/
    public function square() {
        $testmode = false;
        if (!empty($this->payment_method['testmode'])) {
            $testmode = true;
        }
        $gateway = Omnipay::create('Square');
        $gateway->setAccessToken($this->payment_method['access_token']);
        $gateway->setLocationId($this->payment_method['location_id']);
        $gateway->setTestMode($testmode);
        try {
            $parameters = [
                'transactionReference' => $this->session->userdata('order_id'),
                'currency' => $this->payment_currency,
                'items' => array(
                    array(
                        'name' => $this->item_description,
                        'price' => $this->amount,
                        'quantity' => 1
                    )
                )
            ];
            $response = $gateway->purchase($parameters)->send();
            
            if ($response->isSuccessful()) {
                // payment was successful: update database
                /*$reference = $response->getTransactionReference();  // TODO; Check the reference/id with your database
                 echo "Transaction '" . $response->getTransactionId() . "' succeeded!";
                 */
                $data = $response->getRedirectData();
                header("Location: ".$data['checkout_url']);
                exit;
            } elseif ($response->isRedirect()) {
                // redirect to offsite payment gateway
                $response->redirect();
                //return "Error " .$response->getCode() . ': ' . $response->getMessage();
            }
            else {
                // payment failed: display message to customer
                echo $response->getMessage();
            }
        } catch (Exception $e) {
            // internal error, log exception and display a generic message to the customer
            $this->payment_gateways_model->unset_session();
            Template::set_message(lang('msg_payment_processing_error'), 'error');
            Template::redirect(gc_url_format('members'));
        }
        
    }
    
    /**
     * Square Complete Purchase Method
     * @return boolean
     */
    public function squareCompletePurchase() {
        $testmode = false;
        if ($this->payment_method['testmode'] == 1) {
            $testmode = true;
        }
        $gateway = Omnipay::create('Square');
        $gateway->setAccessToken($this->payment_method['access_token']);
        $gateway->setLocationId($this->payment_method['location_id']);
        $gateway->setTestMode($testmode);
        $parameters = [
            'transactionReference' => $this->session->userdata('order_id'),
            'currency' => $this->payment_currency,
            'items' => array(
                array(
                    'name' => $this->item_description,
                    'price' => $this->amount,
                    'quantity' => 1
                )
            )
        ];
        $response = $gateway->completePurchase($parameters)->send();
        
        $responseData = $response->getData(); // this is the raw response object
        
        if ( ! $response->isSuccessful()) {
            Template::set_message ( $response->getMessage(), 'error' );
            $this->transaction_cancelled($response->getTransactionReference(), $this->input->get('PayerID'), $paypalResponse);
        }
        
        if(isset($responseData['PAYMENTINFO_0_ACK']) && 	$responseData['PAYMENTINFO_0_ACK'] === 'Success') {
            // Response
            $this->transaction_confirmed($response->getTransactionReference(), $this->input->get('PayerID'), $responseData);
        } else {
            Template::set_message ( lang('transaction_cancelled'), 'error' );
            $this->transaction_cancelled($response->getTransactionReference(), $this->input->get('PayerID'), $responseData);
        }
    }
    
    /**
     * Pay using Razorpay
     */
    public function razorpay() {
        $this->load->library('payment_gateways/razorpay');
        $this->order_info->item_description = $this->item_description;
        $this->order_info->amount = $this->amount;
        $this->razorpay->pay($this->order_info, $this->user_info);
    }
    
    /**
     * Razorpay Complete Purchase
     */
    public function razorpayCompletePurchase() {
        $this->load->library('payment_gateways/razorpay');
        $result = $this->razorpay->verify();
        if(!empty($result['success'])) {
            $this->transaction_confirmed($result['transaction_id'], $result['payer_id'], $result['payment_details']);
        } else {
            $this->transaction_cancelled($result['transaction_id'], $result['payer_id'], $result['payment_details']);
        }
    }
    
    /**
     * Update transaction table after successful payment
     */
    private function transaction_confirmed($transaction_id = null, $payer_id = null, $payment_details = array(), $is_redirect = true) {
        return $this->payment_gateways_model->transaction_confirmed($this->order_info, $transaction_id, $payer_id, $payment_details, $is_redirect);
    }
    
    /**
     * Update transaction table after payment cancelled
     */
    public function transaction_cancelled($transaction_id = null, $payer_id = null, $payment_details = array()) {
        $this->payment_gateways_model->transaction_cancelled($this->order_info, $transaction_id, $payer_id, $payment_details);
    }
    
    /**
     * Convert Currency
     * @param string $to - Converted To
     * @param float $amount - Amount on which conversion will be applied
     * Yahoo suddenly terminated its finance download API on November 17, 2017.
     */
    private function currencyConversion($order_id, $to, $amount, $from = null) {
        if(empty($from)) {
            $from = settings_item('site.currency_code');
        }
        if($from != $to) {
            $this->load->library('CurrencyConverter');
            $result = $this->currencyconverter->amount_conversion($to, $amount, $from, true);
            $amount =  $result['amount'];
            $this->db->where(array('order_id' => $order_id))->update('orders', array('currency_code' => $to, 'currency_value' => floatval($result['rate'])));
            return floatval($amount);
        } else {
            return floatval($amount);
        }
    }
    
    /**
     * Mobile App redirect
     * @param boolean $exception - true - exception happened
     * @param string $error_message - display passed error message to the user
     */
    private function _redirect($exception = false, $error_message = null) {
        $this->payment_gateways_model->_redirect($this->order_info, $exception, $error_message);
    }
}