<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
 || ##################################################################### ||
 || # GoClixy - Online Software Solutions							   # ||
 || # Copyright (C) 2021 GoClixy. All Rights Reserved.				   # ||
 || # ----------------------------------------------------------------  # ||
 || # This file is owned and copyrighted by GoClixy.com. Your license   # ||
 || # confers  no  title  or ownership in the Script and is not a sale  # ||
 || # of any rights to the Software. This file may not be redistributed # ||
 || # in whole or significant part.									   # ||
 || # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
 || # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
 || ##################################################################### ||
 \*=======================================================================*/
/**
 * Settings controller
 */
class Settings extends Admin_Controller
{
    protected $permissionView   = 'Payment_Gateways.Settings.View';
    protected $permissionEdit   = 'Payment_Gateways.Settings.Edit';
    
    private $file_size = 100; // set maximum size
    private $max_width = 150; // set maximum width
    private $max_height = 30; // set maximum height
    private $payment_gateway_fields = array(        
        0 => array('settings'),
        1 => array('payment_instructions'),
        2 => array('api_username','api_password','api_signature','client_id','secret','currency','testmode'),
        3 => array('login_id','transaction_key','hash','currency','testmode'),
        4 => array('api_account_number','api_secret_word','currency','testmode'),
        5 => array('secret_key','publishable_key','currency','testmode'),
        6 => array('merchant_id','merchant_key','testmode','shop_id'),
        7 => array('merchant_id','secret_key','testmode'),
        8 => array('mid','vpnPswd','remotePswd','usageType','confirmEmail','additionalOptions','mailCustomer','testmode'),
        9 => array('accountNumber','storeId','storePassword','testmode'),
        10 => array('username','sharedSecret','testmode','paymentRoutingNumber'),
        11 => array('merchantId','merchantPassword','merchantAccountNumber','testmode'),
        12 => array('merchantId','transactionPassword','testmode'),
        13 => array('installationId','accountId','secretWord','callbackPassword','testmode'),
        14 => array('secretKey','testmode','currency'),
        15 => array('merchantId','password','testmode'),
        16 => array('apiKey','secret','accountId','testmode'),
        17 => array('profileId','secretKey','accessKey','testmode'),
        18 => array('merchantId','transactionKey','username','password','testmode'),
        19 => array('merchantId','password'),
        20 => array('username','password','apiKey','accountNumber'),
        21 => array('websiteKey','secretKey','testmode'),
        22 => array('apiKey','password','testmode'),
        23 => array('storeId','sharedSecret','testmode'),
        24 => array('email','password','notifyUrl','currency','testmode'),
        25 => array('merchantId','account','secret','3dSecure','testmode'),
        26 => array('username','password','signature','testmode'),
        27 => array('vendor','referrerId','testmode'),
        28 => array('merchantId','password','testmode'),
        29 => array('merchantId','merchantKey','pdtKey','testmode'),
        30 => array('setSubAccountId','testmode'),
        31 => array('accountId','siteId','siteCode','testmode'),
        32 => array('setMerchantAccessCode','merchant','testmode'),
        33 => array('installation','merchant','password','testmode'),
        34 => array('apiKey','testmode'),
        35 => array('merchantId','password','testmode'),
        36 => array('serverKey','environment','testmode'),
        37 => array('merchantId','merchantKey','testmode'),
        38 => array('username','token','testmode'),
        39 => array('merNo','gatewayNo','signKey','testmode'),
        40 => array('appId','appSecret','merchantId','accessToken','testmode'),
        41 => array('merchantId','gatewayToken','testmode'),
        42 => array('username','password','testmode'),
        43 => array('partner','key','testmode'),
        44 => array('accountId','secretKey','testmode'),
        45 => array('merchantKey','salt','currency','testmode'),
        46 => array('mac_key','pay_item_id','product_id','currency','testmode'),
        47 => array('secret_key'),
        48 => array('merchant_id','access_code','working_key','currency','testmode'),
        49 => array('mid','merchant_key','industry_type_id', 'channel_id','website','currency','testmode'),
        50 => array('merchant_uid','merchant_public_key','merchant_secret','currency','testmode'),
        51 => array('api','method','currency'),
        52 => array('access_token','location_id','currency','testmode'),
    );
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
        
        $this->auth->restrict($this->permissionView);
        $this->load->model('payment_gateways_model', null, true);
        $this->lang->load('payment_gateways');
        
        Template::set_block('sub_nav', 'settings/_sub_nav');
        // settings used in help option
        $file_upload_settings = array (
            'file_size' => $this->file_size,
            'max_width'	=> $this->max_width,
            'max_height' => $this->max_height
        );
        Template::set('file_upload_settings',$file_upload_settings);
    }
    
    /**
     * Manage Payment Gateways
     * @param string $filter
     * @param number $offset
     */
    public function index($filter = 'all', $offset = 0) {
        $this->manage('payment_gateways', $filter = 'all', $offset = 0);
    }
    
    /**
     * Manage Payment Gateways
     * @param string $module
     * @param string $filter
     * @param number $offset
     */
    public function manage($module = 'payment_gateways', $filter = 'all', $offset = 0) {
        $extra = array(
            'model' => 'payment_gateways',
            'context' => 'settings',
            'fields' => null
        );
        $extra['actions'] = array( // Extra buttons
            array('type' => 'link', 'link' => site_url(SITE_AREA . '/settings/payment_gateways/form/' . $module . '/edit/%s'), 'label' => '<i class="fas fa-edit fa-fw mr-1"></i>' . lang('edit'), 'class' => 'btn btn-xs btn-primary'),
        );
        $this->gc->module_view($module, $filter, $offset, 'admin', $extra);
    }
    
    /**
     * Add/Edit Payment Gateways Data
     *
     * @param string $module
     * @param string $action
     *
     * @return void
     */
    public function form($module = 'payment_gateways', $action = 'create') {
        Assets::add_js(array(
            'tinymce/tinymce.min.js',
        ));
        $id = (int) $this->uri->segment(7);
        if (empty($id) && $action != 'create') {
            Template::set_message(lang('invalid_id'), 'danger');
            redirect(SITE_AREA .'/settings/payment_gateways');
        }
        if (isset($_POST['save']) || isset($_POST['save_only'])) {
            $this->gcrud->stats(array('module' => 'payment_gateways'));
            $this->auth->restrict($this->permissionEdit);
            if ($this->save_gateway(empty($id) ? 'insert' : 'update', $id)) {
                // Log the activity
                log_activity($this->auth->user_id(), lang('act_edit_record') .': '. $id .' : '. $this->input->ip_address(), 'payment_gateways');
                Template::set_message(sprintf(lang('msg_save_success'), lang('payment_gateway')), 'success');
                if(isset($_POST['save'])) {
                    Template::redirect($this->session->userdata('payment_gateways_last_visited_page'));
                }
            } else {
                Template::set_message(sprintf(lang('msg_edit_failure'), lang('payment_gateway')) . $this->payment_gateways_model->error, 'danger');
            }
        }
        $record = array();
        $settings = array();
        if(!empty($id)) {
            $record = $this->payment_gateways_model->find($id);
            if(!$record) {
                Template::set_message(lang('invalid_id'), 'danger');
                redirect(SITE_AREA .'/settings/payment_gateways');
            }
            
            // Retrieve settings from database
            $settings = unserialize($record->settings);
        }
        $fields = isset($this->payment_gateway_fields[$id]) ? $this->payment_gateway_fields[$id] : (!empty($this->payment_gateway_fields[0]) ? $this->payment_gateway_fields[0] : null);
        Template::set(array(
            'fields' => $fields,
            'record' => $record,
            'settings' => $settings,
            'toolbar_title' => $record ? lang('edit') . ' ' . $record->name : lang('create_payment_gateways')
        ));
        Template::render();
    }
    
    /**
     * Save Gateway Information
     * @param int $id	The ID of the record to update
     */
    private function save_gateway($type, $id = 0) {
        $_POST['id'] = $id;
        $this->form_validation->set_rules('display_name', 'lang:label_display_name', 'required|trim|max_length[100]|sanitize');
        $this->form_validation->set_rules('description', 'lang:label_description', 'trim');
        $this->form_validation->set_rules('priority', 'lang:label_order', 'trim|integer|required');
        $this->form_validation->set_rules('custom', 'lang:label_settings', 'trim');
        $this->form_validation->set_rules('status', 'lang:label_status', 'trim');
        $this->form_validation->set_rules('image', 'lang:label_image', 'trim');
        foreach($_POST as $key => $value) {
            switch($key) {
                case 'display_name':
                case 'description':
                case 'priority':
                case 'status':
                case 'image':
                case 'previous_image':
                case 'settings':
                    break;
                default:
                    if($type == 'update') {
                        $this->form_validation->set_rules($key, 'lang:label_' . $key, 'trim|required');
                    }
            }
        }
        if ($this->form_validation->run() === false) {
            return false;
        }
        $data = array();
        $settings = array();
        foreach($_POST as $key => $value) {
            switch($key) {
                case 'display_name':
                case 'description':
                case 'image':
                    $data[$key] = $this->input->post($key);
                    break;
                case 'priority':
                case 'status':
                    $data[$key] = (int) $this->input->post($key);
                    break;
                case 'settings':
                    $settings = $this->input->post($key);
                    break;
                default:
                    switch($key) {
                        case 'testmode':
                            $settings[$key] = $this->input->post('testmode') ? 1 : 0;
                            break;
                        default:
                            $settings[$key] = $this->input->post($key);
                    }
            }
        }
        if(isset($settings['save'])) {
            unset($settings['save']);
        }
        if(array_key_exists('id', $settings)) {
            unset($settings['id']);
        }
        $data['settings'] = serialize($settings);
        if($type == 'insert') {
            $this->load->library('slug');
            $data['name'] = ucwords(str_replace('-', '', $this->slug->create_slug($data['display_name'])));
            return $this->payment_gateways_model->insert($data);
        } else {
            return $this->payment_gateways_model->update($id, $data);
        }
    }
}