<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
 || ##################################################################### ||
 || # GoClixy - Online Software Solutions							   # ||
 || # Copyright (C) 2021 GoClixy. All Rights Reserved.				   # ||
 || # ----------------------------------------------------------------  # ||
 || # This file is owned and copyrighted by GoClixy.com. Your license   # ||
 || # confers  no  title  or ownership in the Script and is not a sale  # ||
 || # of any rights to the Software. This file may not be redistributed # ||
 || # in whole or significant part.									   # ||
 || # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
 || # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
 || ##################################################################### ||
 \*=======================================================================*/
class Ccavenue {
    protected $CI;
    protected $payment_method;
    protected $merchant_id;
    protected $access_code;
    protected $working_key;
    protected $currency;

    /**
     * Constructor
     */
    public function __construct() {
        $this->CI =& get_instance();
        $this->CI->lang->load('payment_gateways/payment_gateways');
        $this->CI->load->model('payment_gateways/payment_gateways_model');
        $this->payment_method = $this->CI->payment_gateways_model->get_payment_method(48);
        if(empty($this->payment_method['merchant_id']) && $this->payment_method['access_code'] && $this->payment_method['working_key']) {
            Template::redirect(gc_url_format('members'));
        }
        $this->merchant_id = $this->payment_method['merchant_id'];
        $this->access_code = $this->payment_method['access_code'];
        $this->working_key = $this->payment_method['working_key'];
        $this->currency = !empty($this->payment_method['currency']) ? $this->payment_method['currency'] : 'INR';
    }

    /**
     * Allow user to pay using Ccavenue
     * Load this method just before payment gateway redirect
     */
    public function pay($order_info, $user_info) {
        $verification_url = site_url('payment_gateways/ccavenueCompletePurchase?order_id=' . $order_info->order_id);
        $merchant_data='';      
        $params = array(
            'tid' => time().rand(111,999),
            'merchant_id' => $this->merchant_id,
            'order_id' => $order_info->order_id,
            'amount' => $order_info->amount,
            'currency' => $this->currency,
            'redirect_url' => $verification_url,
            'cancel_url' => $verification_url,
            'language' => 'EN',
            'billing_name' => $user_info['name'],
            'billing_address' => $user_info['address'],
            'billing_city' => $user_info['city'],
            'billing_state' => $user_info['state'],
            'billing_zip' => $user_info['postcode'],
            'billing_country' => $user_info['country'],
            'billing_email' => $user_info['email'],
            'billing_tel' => $user_info['phone_number'],
            'integration_type' => 'iframe_normal'
        );
        foreach ($params as $key => $value){
            $merchant_data.=$key.'='.$value.'&';
        }
        $encrypted_data = $this->encrypt($merchant_data,$this->working_key); // Method for encrypting the data.
        $production_url='https://secure.ccavenue.com/transaction/transaction.do?command=initiateTransaction&encRequest='.$encrypted_data.'&access_code='.$this->access_code;
        Template::set(array(
            'production_url' => $production_url
        ));
        Template::set_view('payment_gateways/ccavenue');
        Template::render('blank');
    }

    /**
     * Ccavenue Complete Purchase Method
     * @return boolean
     */
    public function verify() {
        if(!empty($_POST["encResp"])) {
            $encResponse = $_POST["encResp"];			// This is the response sent by the CCAvenue Server
            $rcvdString = $this->decrypt($encResponse,$this->working_key);		//Crypto Decryption used as per the specified working key.
            $order_status = "";
            $payment_details = array();
            $decryptValues = explode('&', $rcvdString);
            $dataSize = sizeof($decryptValues);
            for($i = 0; $i < $dataSize; $i++) {
                $information = explode('=',$decryptValues[$i]);
                if($i == 3) {
                    $order_status=$information[1];
                }
                $payment_details[$information[0]] = $information[1];
            }
            $transaction_id = !empty($payment_details['tracking_id']) ? $payment_details['tracking_id'] : '';
            $signature = !empty($payment_details['bank_ref_no']) ? $payment_details['bank_ref_no'] : '';
            if($order_status === "Success") {
                return array('success' => true, 'transaction_id' => $transaction_id, 'payer_id' => $signature, 'payment_details' => $payment_details); 
            } else {
                return array('success' => false, 'transaction_id' => $transaction_id, 'payer_id' => $signature, 'payment_details' => $payment_details);
            }
        } else {
            return array('success' => false, 'transaction_id' => null, 'payer_id' => null, 'payment_details' => null);
        }
    }

    /*
     * @param1 : Plain String
     * @param2 : Working key provided by CCAvenue
     * @return : Decrypted String
     */
    private function encrypt($plainText,$key) {
        $key = $this->hextobin(md5($key));
        $initVector = pack("C*", 0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f);
        $openMode = openssl_encrypt($plainText, 'AES-128-CBC', $key, OPENSSL_RAW_DATA, $initVector);
        $encryptedText = bin2hex($openMode);
        return $encryptedText;
    }

    /*
     * @param1 : Encrypted String
     * @param2 : Working key provided by CCAvenue
     * @return : Plain String
     */
    private function decrypt($encryptedText,$key) {
        $key = $this->hextobin(md5($key));
        $initVector = pack("C*", 0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f);
        $encryptedText = $this->hextobin($encryptedText);
        $decryptedText = openssl_decrypt($encryptedText, 'AES-128-CBC', $key, OPENSSL_RAW_DATA, $initVector);
        return $decryptedText;
    }

    private function hextobin($hexString) { 
        $length = strlen($hexString); 
        $binString="";   
        $count=0; 
        while($count<$length) {       
            $subString =substr($hexString,$count,2);           
            $packedString = pack("H*",$subString); 
            if ($count==0) {
                $binString=$packedString;
            } else {
                $binString.=$packedString;
            } 
            
            $count+=2; 
        } 
        return $binString; 
    } 
}