<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
 || ##################################################################### ||
 || # GoClixy - Online Software Solutions							    # ||
 || # Copyright (C) 2021 GoClixy. All Rights Reserved.				    # ||
 || # ----------------------------------------------------------------  # ||
 || # This file is owned and copyrighted by GoClixy.com. Your license   # ||
 || # confers  no  title  or ownership in the Script and is not a sale  # ||
 || # of any rights to the Software. This file may not be redistributed # ||
 || # in whole or significant part.									    # ||
 || # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
 || # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
 || ##################################################################### ||
 \*=======================================================================*/
class PayPal {
    protected $CI;
    protected $payment_method;
    protected $client_id;
    protected $secret;
    protected $live_url = 'https://api-m.paypal.com/';
    protected $url = 'https://api-m.sandbox.paypal.com/';
    
    /**
     * Constructor
     */
    public function __construct() {
        $this->CI =& get_instance();
        $this->CI->lang->load('payment_gateways/payment_gateways');
        $this->CI->load->model('payment_gateways/payment_gateways_model');
        $this->payment_method = $this->CI->payment_gateways_model->get_payment_method(2);
        // $this->payment_method['client_id'] = 'AdrmfQRmKbTgpDbEOFXr6ORax3K7j0eaaNQRPPNSY0DJL4uyzfvwN-bmy00H-FIM6jVUK2i2kGLQUbNZ';
        // $this->payment_method['secret'] = 'EETg9fBXEP8f7SbG5MZhnqv21lcVPPJCgyFaYwaW6hJNbktvKkT5hXaIUEG9QS95WzDXzKxS-NxAYJZj';
        if(empty($this->payment_method['client_id']) && $this->payment_method['secret']) {
            Template::redirect(gc_url_format('members'));
        }
        if ($this->payment_method['testmode'] == 0) {
            $this->url = $this->live_url;
        }
        $this->client_id = $this->payment_method['client_id'];
        $this->secret = $this->payment_method['secret'];
        $this->currency = !empty($this->payment_method['currency']) ? $this->payment_method['currency'] : 'USD';
    }
    
    /**
     * Send user to the paypal gateway
     */
    public function pay($order_item) {
        $result = $this->create_plan($order_item);
        $data = array(
            'order_id' => $order_item->order_id,
            'client_id' => $this->client_id,
            'plan_id' => !empty($result['plan_id']) ? $result['plan_id'] : '',
            'access_token' => !empty($result['access_token']) ? $result['access_token'] : '',
        );
        if(!empty($result['message'])) {
            $data['message'] = $result['message'];
        }
        Template::set($data);
        Template::set_view('payment_gateways/paypal');
        Template::render('blank');
    }
    
    /**
     * After payment, save subscription details in database
     */
    public function verify($data = array()) {
        if(empty($data['subscriptionID'])) {
            return array('success' => false, 'access_token' => null, 'message' => null);
        }
        if(empty($data['access_token'])) {
            $token = $this->get_token();
            if(!empty($token['access_token'])) {
                $data['access_token'] = $token['access_token'];
            } else {
                return $token;
            }
        }
        $subscription_id = $data['subscriptionID'];
        $token = $data['access_token'];
        // Generated by curl-to-PHP: http://incarnate.github.io/curl-to-php/
        $ch = curl_init();
        
        curl_setopt($ch, CURLOPT_URL, $this->url . 'billing/subscriptions/' . $subscription_id);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'GET');
        
        
        $headers = array();
        $headers[] = 'Content-Type: application/json';
        $headers[] = 'Authorization: Bearer ' . rawurlencode($token);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        
        $result = curl_exec($ch);
        $err = curl_error($ch);
        curl_close($ch);
        if($err) { // subscription failed
            return array('success' => false, 'access_token' => null, 'message' => $err);
        } else {
            $response = json_decode($result ?? '');
            $subscriber_id = $response->id;
            $transaction_id = $subscriber_id;
            $transaction_data = array(
                'name' => $response->subscriber->name->given_name . ' ' . $response->subscriber->name->surname,
                'email_address' => $response->subscriber->email_address,
                'plan_id' => $response->plan_id,
                'start_time' => $response->start_time,
                'status_update_time' => $response->status_update_time,
                'status' => $response->status               
            );
            if($response->status == 'ACTIVE' && $subscription_id == $subscriber_id) { // subscription success
                return array('success' => true, 'access_token' => $token, 'transaction_id' => $transaction_id, 'payer_id' => $subscriber_id, 'payment_details' => $transaction_data, 'message' => null);
            } else { // subscription failed
                return array('success' => false, 'access_token' => null, 'message' => $err);
            }
        }
    }
    
    /**
     * Make payment online - Order Checkout without subscription
     * @return boolean[]|NULL[]|string[]|boolean[]|unknown[]|NULL[]
     */
    public function payment() {
        $token = $this->get_token();
        if($token['success']) {
            $token = $token['access_token'];
            // $post_data = '{"intent": "CAPTURE","purchase_units":[{"amount": {"currency_code": "USD","value": "100.00"}}]}';
            $post_data = array (
                'intent' => 'CAPTURE',
                'purchase_units' => array (
                    array (
                        'amount' => array (
                            'currency_code' => 'USD',
                            'value' => '10.00',
                        ),
                    ),
                ),
            );
            
            // Generated by curl-to-PHP: http://incarnate.github.io/curl-to-php/
            $ch = curl_init();
            
            curl_setopt($ch, CURLOPT_URL, $this->url . 'v2/checkout/orders');
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($post_data));
            
            $headers = array();
            $headers[] = 'Content-Type: application/json';
            $headers[] = 'Authorization: Bearer ' . rawurlencode($token);
            curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
            
            $result = curl_exec($ch);
            $err = curl_error($ch);
            curl_close($ch);
            if($err) {
                return array('success' => false, 'access_token' => null, 'message' => $err);
            } else {
                $response = json_decode($result);
                $order_id = $response->id;
                if($order_id) {
                    return array('success' => true, 'access_token' => $token, 'order_id' => $order_id, 'message' => null);
                }
            }
        } else {
            return $token;
        }
    }
    
    /**
     * Create Plan
     * @param array $data
     */
    private function create_plan($item) {
        $product = $this->create_product($item);
        if($product['success']) {
            $token = $product['access_token'];
            $product_id = $product['product_id'];
            $plan_id = 'PLAN' . '-' . $item->module_id . '-' . $item->id; // e.g. PLAN-18062020-001
            $price = !empty($item->offer_price) && $item->offer_price < $item->price ? $item->offer_price : $item->price;
            $currency = 'USD';
            $post_data = array(
                'product_id' => $product_id,
                'name' => $item->title,
                'description' => $item->description,
                'billing_cycles' => array(
                    array (
                        'frequency' => array (
                            'interval_unit' => 'DAY',
                            'interval_count' => $item->duration,
                        ),
                        'tenure_type' => 'REGULAR',
                        'sequence' => 1,
                        'total_cycles' => 0,
                        'pricing_scheme' => array (
                            'fixed_price' => array (
                                'value' => $price,
                                'currency_code' => $currency,
                            ),
                        ),
                    )
                ),
                'payment_preferences' => array (
                    'auto_bill_outstanding' => true,
                    'setup_fee' => array (
                        'value' => $price,
                        'currency_code' => $currency,
                    ),
                    'setup_fee_failure_action' => 'CONTINUE',
                    'payment_failure_threshold' => 3,
                ),
                'taxes' => array (
                    'percentage' => '0',
                    'inclusive' => true,
                )
            );
            
            // Generated by curl-to-PHP: http://incarnate.github.io/curl-to-php/
            $ch = curl_init();
            
            curl_setopt($ch, CURLOPT_URL, $this->url . 'v1/billing/plans');
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($post_data));
            
            $headers = array();
            $headers[] = 'Accept: application/json';
            $headers[] = 'Authorization: Bearer ' . rawurlencode($token);
            $headers[] = 'Content-Type: application/json';
            $headers[] = 'Paypal-Request-Id: ' . $plan_id;
            curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
            
            $result = curl_exec($ch);
            $err = curl_error($ch);
            curl_close($ch);
            if($err) {
                return array('success' => false, 'access_token' => null, 'message' => $err);
            } else {
                $response = json_decode($result ?? '');
                $plan_id = $response->id;
                if($plan_id) {
                    return array('success' => true, 'access_token' => $token, 'product_id' => $product_id, 'plan_id' => $plan_id, 'message' => null);
                }
            }
        } else {
            return $product;
        }
    }
    
    /**
     * Create PayPal product for subscription
     * @return boolean[]|NULL[]|string[]
     */
    private function create_product($item) {
        $token = $this->get_token();
        if($token['success']) {
            $token = $token['access_token'];
            $category = 'SOFTWARE'; // strtoupper(gc_module_name($item->module_id));
            $image_url = 'https://www.goclixy.com/assets/images/emails/email-header.jpg';
            $product_id = 'ITEM' . '-' . $item->module_id . '-' . $item->id; // e.g. PRODUCT-18062020-001
            $item_description = $item->description;
            $home_url = 'https://www.goclixy.com/pricing'; gc_url_format('pages', 'pricing');
            $post_data = array(
                'name' => $item->title,
                'description' => $item_description,
                'type' => 'SERVICE',
                'category' => $category,
                'image_url' => $image_url,
                'home_url' => $home_url
            );
            
            // Generated by curl-to-PHP: http://incarnate.github.io/curl-to-php/
            $ch = curl_init();
            
            curl_setopt($ch, CURLOPT_URL, $this->url . 'v1/catalogs/products');
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($post_data));
            $headers = array();
            $headers[] = 'Content-Type: application/json';
            $headers[] = 'Authorization: Bearer ' . rawurlencode($token);
            $headers[] = 'Paypal-Request-Id: ' . rawurlencode($product_id);
            curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
            
            $result = curl_exec($ch);
            $err = curl_error($ch);
            curl_close($ch);
            if($err) {
                return array('success' => false, 'access_token' => null, 'message' => $err);
            } else {
                $response = json_decode($result ?? '');
                if(empty($response->id)) {                    
                    if(!empty($response->message)) {
                        $err = $response->message;
                    }
                    return array('success' => false, 'access_token' => null, 'message' => $err);
                } else {
                    $product_id = $response->id;
                    if($product_id) {
                        return array('success' => true, 'access_token' => $token, 'product_id' => $product_id, 'message' => null);
                    }
                }
            }
        } else {
            return $token;
        }
    }
    
    /**
     * Get access token from the paypal website
     * @param string $role
     */
    private function get_token() {
        // Generated by curl-to-PHP: http://incarnate.github.io/curl-to-php/
        $ch = curl_init();
        
        curl_setopt($ch, CURLOPT_URL, $this->url . 'v1/oauth2/token');
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, "grant_type=client_credentials");
        curl_setopt($ch, CURLOPT_USERPWD, $this->client_id . ':' . $this->secret);
        
        $headers = array();
        $headers[] = 'Accept: application/json';
        $headers[] = 'Accept-Language: en_US';
        $headers[] = 'Content-Type: application/x-www-form-urlencoded';
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        
        $result = curl_exec($ch);
        $err = curl_error($ch);
        curl_close($ch);
        if($err) {
            return array('success' => false, 'access_token' => null, 'message' => $err);
        } else {
            $response = json_decode($result ?? '');
            $access_token = $response->access_token;
            if($access_token) {
                return array('success' => true, 'access_token' => $access_token, 'message' => null);
            }
        }
    }
}