<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
 || ##################################################################### ||
 || # GoClixy - Online Software Solutions							   # ||
 || # Copyright (C) 2021 GoClixy. All Rights Reserved.				   # ||
 || # ----------------------------------------------------------------  # ||
 || # This file is owned and copyrighted by GoClixy.com. Your license   # ||
 || # confers  no  title  or ownership in the Script and is not a sale  # ||
 || # of any rights to the Software. This file may not be redistributed # ||
 || # in whole or significant part.									   # ||
 || # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
 || # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
 || ##################################################################### ||
 \*=======================================================================*/
use Razorpay\Api\Api;
use Razorpay\Api\Errors\SignatureVerificationError;
class Razorpay {
    /**
     * Razorpay official php kit installed via composer
     * 
     * View file must have button with mentioned id, change it runtime during checkout
     * Also add the form with hidden variables as follows
     * 
     * <button id="rzp-button1">Pay with Razorpay</button>

<?php echo form_open(site_url('payment_gateways/razorpayCompletePurchase?order_id=' . $order_id), 'name="razorpayform"');?>
    <input type="hidden" name="razorpay_payment_id" id="razorpay_payment_id">
    <input type="hidden" name="razorpay_signature"  id="razorpay_signature" >
<?php echo form_close(); ?>
     */
    protected $CI;
    protected $payment_method;
    protected $key_id;
    protected $key_secret;
    protected $currency;
    
    /**
     * Constructor
     */
    public function __construct() {
        $this->CI =& get_instance();
        $this->CI->lang->load('payment_gateways/payment_gateways');
        $this->CI->load->model('payment_gateways/payment_gateways_model');
        $this->payment_method = $this->CI->payment_gateways_model->get_payment_method(51);
        if(empty($this->payment_method['key_id']) && $this->payment_method['key_secret']) {
            Template::redirect(gc_url_format('members'));
        }
        $this->key_id = $this->payment_method['key_id'];
        $this->key_secret = $this->payment_method['key_secret'];
        $this->currency = !empty($this->payment_method['currency']) ? $this->payment_method['currency'] : 'INR';
    }
    
    /**
     * Allow user to pay using razorpay
     * Load this method just before payment gateway redirect
     */
    public function pay($order_info, $user_info) {
        $order_id = $order_info->order_id;
        $displayCurrency = $this->currency;
        $amount = $order_info->amount;
        $api = new Api($this->key_id, $this->key_secret);
        
        //
        // We create an razorpay order using orders api
        // Docs: https://docs.razorpay.com/docs/orders
        //
        $orderData = [
            'receipt'         => $order_info->order_id,
            'amount'          => $amount * 100, // 2000 rupees in paise
            'currency'        => 'INR',
            'payment_capture' => 1 // auto capture
        ];
        
        $razorpayOrder = $api->order->create($orderData);
        $razorpayOrderId = $razorpayOrder['id'];
        $_SESSION['razorpay_order_id'] = $razorpayOrderId;
        $displayAmount = $amount = $orderData['amount'];
        if ($displayCurrency !== 'INR') {
            $url = "https://api.fixer.io/latest?symbols=$displayCurrency&base=INR";
            $exchange = json_decode(file_get_contents($url), true);
            $displayAmount = $exchange['rates'][$displayCurrency] * $amount / 100;
        }
        $data = [
            "key"               => $this->key_id,
            "amount"            => $amount,
            "name"              => $order_info->order_id,
            "description"       => $order_info->order_id, // $order_info->item_description,
            "image"             => gc_image_url(settings_item('site.logo'), false),
            "prefill"           => [
                "name"              => $user_info['name'],
                "email"             => $user_info['email'],
                "contact"           => $user_info['phone_number'],
            ],
            "notes"             => [
                "address"           => $user_info['address'],
                "merchant_order_id" => $order_info->order_id,
            ],
            "theme"             => [
                "color"             => "#F37254"
            ],
            "order_id"          => $razorpayOrderId,
        ];
        
        if ($displayCurrency !== 'INR') {
            $data['display_currency']  = $displayCurrency;
            $data['display_amount']    = $displayAmount;
        }
        $data = array('json' => json_encode($data));
        $this->load_js($data, $order_info->order_id);
        Template::set('order_id', $order_info->order_id);
        Template::set_view('payment_gateways/razorpay');
        Template::render('blank');
    }
    
    /**
     * Load JS files and other javascript related code
     * @param array $data
     */
    private function load_js($data, $order_id) {
        
        Assets::add_js('https://checkout.razorpay.com/v1/checkout.js');
        Assets::add_js(
            '
// automatically click on the pay button
window.onload = function() {
  document.getElementById(\'rzp-button1\').click();
}
// Checkout details as a json
var options = ' . $data['json'] . '
            
/**
 * The entire list of Checkout fields is available at
 * https://docs.razorpay.com/docs/checkout-form#checkout-fields
 */
options.handler = function (response){
    document.getElementById(\'razorpay_payment_id\').value = response.razorpay_payment_id;
    document.getElementById(\'razorpay_signature\').value = response.razorpay_signature;
    document.razorpayform.submit();
};
            
// Boolean whether to show image inside a white frame. (default: true)
options.theme.image_padding = false;
            
options.modal = {
    ondismiss: function() {
    	window.location.href = "' . site_url('payment_gateways/razorpayCompletePurchase?order_id=' . $order_id) . '";
        // console.log("This code runs when the popup is closed");
    },
    // Boolean indicating whether pressing escape key
    // should close the checkout form. (default: true)
    escape: true,
    // Boolean indicating whether clicking translucent blank
    // space outside checkout form should close the form. (default: false)
    backdropclose: false
};
            
var rzp = new Razorpay(options);
            
document.getElementById(\'rzp-button1\').onclick = function(e){
    rzp.open();
    e.preventDefault();
}
',
            'inline'     );
    }
    
    /**
     * Razorpay Complete Purchase Method
     * @return boolean
     */
    public function verify() {
        if (empty($_POST['razorpay_payment_id']) === false) {
            $api = new Api($this->key_id, $this->key_secret);
            try {
                // Please note that the razorpay order ID must
                // come from a trusted source (session here, but
                // could be database or something else)
                $razorpay_order_id = $_SESSION['razorpay_order_id'];
                $transaction_id = $_POST['razorpay_payment_id'];
                $signature = $_POST['razorpay_signature'];
                $attributes = array(
                    'razorpay_order_id' => $razorpay_order_id,
                    'razorpay_payment_id' => $transaction_id,
                    'razorpay_signature' => $signature
                );
                $api->utility->verifyPaymentSignature($attributes);
                return array('success' => true, 'transaction_id' => $transaction_id, 'payer_id' => $signature, 'payment_details' => $attributes);
            } catch(SignatureVerificationError $e) {
                return array('success' => false, 'transaction_id' => null, 'payer_id' => null, 'payment_details' => $e->getMessage());
            }
        }
    }
}