<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Migration_Install_payment_gateways extends Migration
{
	/**
	 * The name of the database table
	 *
	 * @var String
	 */
	private $table_name = 'payment_gateways';

	/**
	 * The table's fields
	 *
	 * @var Array
	 */
	private $fields = array(
		'id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'auto_increment' => TRUE
		),
		'name' => array(
			'type' => 'VARCHAR',
			'constraint' => 100,
			'null' => FALSE
		),
		'display_name' => array(
			'type' => 'VARCHAR',
			'constraint' => 100,
			'null' => FALSE
		),
		'image' => array(
			'type' => 'VARCHAR',
			'constraint' => 100,
			'null' => TRUE
		),
		'settings' => array(
			'type' => 'text',
			'null' => TRUE
		),
		'order' => array(
			'type' => 'SMALLINT',
			'constraint' => 4,
			'null' => FALSE,
			'default' => 0
		),
		'active' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default' => 0
		),
	);
	
	/**
	 * @var array Data to insert into the Settings table
	 */
	private $data = array(
			array(
					'id' => 1,
					'name' => 'BankTransfer',
					'display_name' => 'Bank Transfer',
					'image' => 'bank-transfer.png',
					'settings' => 'a:1:{s:20:"payment_instructions";s:132:"Please transfer funds to our below given Bank Account:
									<br /><br />
									A/C: 11111111111<br />
									IFC: ABC123<br />
									BANK: TESTING<br />";}',
					'order' => 0,
					'active' => 1,
			),
			array(
					'id' => 2,
					'name' => 'PayPal',
					'display_name' => 'PayPal Checkout',
					'image' => 'paypal.png',
					'settings' => 'a:5:{s:12:"api_username";s:34:"infor-facilitator_api1.goclixy.com";s:12:"api_password";s:16:"7R89TPXC4WWNE29W";s:13:"api_signature";s:56:"AiPC9BjkCyDFQXbSkoZcgqH3hpacANmMzP14C1gSHUfDHZREMCdaMq16";s:8:"currency";s:3:"USD";s:8:"testmode";s:1:"1";}',
					'order' => 1,
					'active' => 1,
			),		
			array(
					'id' => 3,
					'name' => 'AuthorizeNet',
					'display_name' => 'AuthorizeNet',
					'image' => 'authorize.png',
					'settings' => 'a:7:{s:8:"login_id";s:10:"7tsE56xfMt";s:15:"transaction_key";s:16:"7X6Eyj2f64V4Vf7d";s:4:"hash";s:13:"5YEe96YC.yiPY";s:8:"currency";s:3:"USD";s:21:"alternate_gateway_url";b:0;s:8:"logo_url";b:0;s:8:"testmode";s:1:"1";}',
					'order' => 2,
					'active' => 1,
			),
			array(
					'id' => 4,
					'name' => 'TwoCheckout',
					'display_name' => '2Checkout.com',
					'image' => '2checkout.png',
					'settings' => 'a:4:{s:18:"api_account_number";s:9:"901297812";s:15:"api_secret_word";s:48:"YjQyMjM4ZWItNWNiOC00M2Y4LWE0NTMtOWRiNDAwNTE4OTlk";s:8:"currency";s:3:"USD";s:8:"testmode";s:1:"1";}',
					'order' => 3 ,
					'active' => 0 ,
			),
			array(
					'id' => 5,
					'name' => 'Stripe',
					'display_name' => 'Stripe',
					'image' => 'stripe.png',
					'settings' => 'a:4:{s:10:"secret_key";s:7:"testing";s:15:"publishable_key";s:7:"testing";s:8:"currency";s:3:"USD";s:8:"testmode";s:1:"1";}',
					'order' => 4, 
					'active' => 0,
			),
			array(
					'id' => 6,
					'name' => 'Sisow',
					'display_name' => 'Sisow',
					'image' => 'sisow.png',
					'settings' => 'a:4:{s:11:"merchant_id";s:4:"2312";s:12:"merchant_key";s:4:"4123";s:8:"testmode";s:1:"1";s:7:"shop_id";s:5:"31232";}',
					'order' => 5,
					'active' => 0,
			),
			array(
					'id' => 7,
					'name' => 'PayU',
					'display_name' => 'PayU',
					'image' => 'payu.png',
					'settings' => 'a:3:{s:11:"merchant_id";s:8:"asdfasdf";s:10:"secret_key";s:8:"adsfsadf";s:8:"testmode";s:1:"1";}',
					'order' => 6,
					'active' => 0,
			),
			array(
					'id' => 8,
					'name' => 'SecPay',
					'display_name' => 'SecPay',
					'image' => 'null',
					'settings' => 'a:8:{s:3:"mid";s:7:"testing";s:7:"vpnPswd";s:7:"testing";s:10:"remotePswd";s:7:"testing";s:9:"usageType";s:7:"testing";s:12:"confirmEmail";s:7:"testing";s:17:"additionalOptions";s:7:"testing";s:12:"mailCustomer";s:1:"1";s:8:"testmode";s:1:"1";}',
					'order' => 7,
					'active' => 0,
			),
			array(
					'id' => 9,
					'name' => 'NetBanx',
					'display_name' => 'NetBanx',
					'image' => 'netbanx.png',
					'settings' => 'a:4:{s:13:"accountNumber";s:7:"testing";s:7:"storeId";s:7:"testing";s:13:"storePassword";s:7:"testing";s:8:"testmode";s:1:"1";}',
					'order' => 8,
					'active' => 0,
			),
			array(
					'id' => 10,
					'name' => 'Pacnet',
					'display_name' => 'Pacnet',
					'image' => 'pacnet.png',
					'settings' => 'a:4:{s:8:"username";s:4:"asdf";s:12:"sharedSecret";s:3:"fds";s:8:"testmode";s:1:"1";s:20:"paymentRoutingNumber";s:2:"ew";}',
					'order' => 9,
					'active' => 0,
			),
			array(
					'id' => 11,
					'name' => 'Ecopayz',
					'display_name' => 'Ecopayz',
					'image' => 'ecopayz.png',
					'settings' => 'a:4:{s:10:"merchantId";s:4:"1232";s:16:"merchantPassword";s:8:"12312343";s:21:"merchantAccountNumber";s:6:"123213";s:8:"testmode";s:1:"1";}',
					'order' => 10,
					'active' => 0,
			),
			array(
					'id' => 12,
					'name' => 'SecurePay',
					'display_name' => 'SecurePay',
					'image' => 'secure-pay.png',
					'settings' => 'a:3:{s:10:"merchantId";s:3:"123";s:19:"transactionPassword";s:3:"321";s:8:"testmode";s:1:"1";}',
					'order' => 11,
					'active' => 0,
			),
			array(
					'id' => 13,
					'name' => 'WorldPay',
					'display_name' => 'WorldPay',
					'image' => 'worldpay.png',
					'settings' => 'a:5:{s:14:"installationId";s:4:"2131";s:9:"accountId";s:8:"12312312";s:10:"secretWord";s:8:"12312312";s:16:"callbackPassword";s:8:"12312312";s:8:"testmode";s:1:"1";}',
					'order' => 12,
					'active' => 0,
			),
			array(
					'id' => 14,
					'name' => 'Pin',
					'display_name' => 'Pin',
					'image' => 'pin-payments.png',
					'settings' => 'a:3:{s:9:"secretKey";s:5:"12312";s:8:"testmode";s:1:"1";s:8:"currency";s:3:"AUD";}',
					'order' => 13,
					'active' => 0,
			),
			array(
					'id' => 15,
					'name' => 'CardSave',
					'display_name' => 'CardSave',
					'image' => 'cardsave.png',
					'settings' => 'a:3:{s:10:"merchantId";s:5:"12321";s:8:"password";s:5:"ffsad";s:8:"testmode";s:1:"1";}',
					'order' => 14,
					'active' => 0,
			),
			array(
					'id' => 16,
					'name' => 'Coinbase',
					'display_name' => 'Coinbase',
					'image' => 'coinbase.png',
					'settings' => 'a:4:{s:6:"apiKey";s:7:"Testing";s:6:"secret";s:7:"Testing";s:9:"accountId";s:4:"Test";s:8:"testmode";s:1:"1";}',
					'order' => 15,
					'active' => 0,
			),
			array(
					'id' => 17,
					'name' => 'Cybersource',
					'display_name' => 'Cybersource',
					'image' => 'cybersource.png',
					'settings' => 'a:4:{s:9:"profileId";s:4:"asdf";s:9:"secretKey";s:4:"asdf";s:9:"accessKey";s:4:"sadf";s:8:"testmode";s:1:"1";}',
					'order' => 16,
					'active' => 0,
			),
			array(
					'id' => 18,
					'name' => 'CybersourceSOAP',
					'display_name' => 'Cybersource SOAP',
					'image' => 'cybersource.png',
					'settings' => 'a:5:{s:10:"merchantId";s:5:"12312";s:14:"transactionKey";s:6:"123123";s:8:"username";s:6:"123123";s:8:"password";s:6:"123123";s:8:"testmode";s:1:"1";}',
					'order' => 17,
					'active' => 0,
			),
			array(
					'id' => 19,
					'name' => 'DataCash',
					'display_name' => 'DataCash',
					'image' => 'datacash.png',
					'settings' => 'a:2:{s:10:"merchantId";s:7:"testing";s:8:"password";s:7:"testing";}',
					'order' => 18,
					'active' => 0,
			),
			array(
					'id' => 20,
					'name' => 'Agms',
					'display_name' => 'Agms',
					'image' => 'null',
					'settings' => 'a:4:{s:8:"username";s:6:"213123";s:8:"password";s:6:"312312";s:6:"apiKey";s:6:"123123";s:13:"accountNumber";s:8:"12312332";}',
					'order' => 19,
					'active' => 0,
			),
			array(
					'id' => 21,
					'name' => 'Buckaroo',
					'display_name' => 'Buckaroo Credit Card',
					'image' => 'buckaroo.png',
					'settings' => 'a:3:{s:10:"websiteKey";s:5:"12312";s:9:"secretKey";s:6:"321231";s:8:"testmode";s:1:"1";}',
					'order' => 20,
					'active' => 0,
			),
			array(
					'id' => 22,
					'name' => 'Eway',
					'display_name' => 'Eway',
					'image' => 'eway.png',
					'settings' => 'a:3:{s:6:"apiKey";s:6:"123123";s:8:"password";s:6:"312321";s:8:"testmode";s:1:"1";}',
					'order' => 21,
					'active' => 0,
			),
			array(
					'id' => 23,
					'name' => 'FirstData',
					'display_name' => 'FirstData',
					'image' => 'first-data.png',
					'settings' => 'a:3:{s:7:"storeId";s:4:"asdf";s:12:"sharedSecret";s:4:"fdsa";s:8:"testmode";s:1:"1";}',
					'order' => 20,
					'active' => 0,
			),
			array(
					'id' => 24,
					'name' => 'Skrill',
					'display_name' => 'Skrill',
					'image' => 'skrill.png',
					'settings' => 'a:5:{s:5:"email";s:16:"info@goclixy.com";s:8:"password";s:7:"goclixy";s:9:"notifyUrl";s:22:"https://www.goclixy.com";s:8:"currency";s:3:"USD";s:8:"testmode";s:1:"1";}',
					'order' => 23,
					'active' => 0,
			),
			array(
					'id' => 25,
					'name' => 'Realex',
					'display_name' => 'Realex',
					'image' => 'realex.png',
					'settings' => 'a:5:{s:10:"merchantId";s:5:"12321";s:7:"account";s:6:"312312";s:6:"secret";s:6:"123123";s:8:"3dSecure";s:1:"4";s:8:"testmode";b:0;}',
					'order' => 24,
					'active' => 0,
			),
			array(
					'id' => 26,
					'name' => 'Wirecard',
					'display_name' => 'Wirecard',
					'image' => 'wirecard.png',
					'settings' => 'a:4:{s:8:"username";s:4:"asdf";s:8:"password";s:4:"asdf";s:9:"signature";s:4:"afsd";s:8:"testmode";s:1:"1";}',
					'order' => 25,
					'active' => 0,
			),
			array(
					'id' => 27,
					'name' => 'SagePay',
					'display_name' => 'Sage Pay Direct',
					'image' => 'sage-pay.png',
					'settings' => 'a:3:{s:6:"vendor";s:4:"sadf";s:10:"referrerId";s:4:"asdf";s:8:"testmode";s:1:"1";}',
					'order' => 26,
					'active' => 0,
			),
			array(
					'id' => 28,
					'name' => 'PaymentSense',
					'display_name' => 'PaymentSense',
					'image' => 'payment-sense.png',
					'settings' => 'a:3:{s:10:"merchantId";s:4:"asdf";s:8:"password";s:4:"asdf";s:8:"testmode";b:0;}',
					'order' => 27,
					'active' => 0,
			),
			array(
					'id' => 29,
					'name' => 'PayFast',
					'display_name' => 'PayFast',
					'image' => 'payfast.png',
					'settings' => 'a:4:{s:10:"merchantId";s:3:"123";s:11:"merchantKey";s:3:"321";s:6:"pdtKey";s:3:"123";s:8:"testmode";s:1:"1";}',
					'order' => 28,
					'active' => 0,
			),
			array(
					'id' => 30,
					'name' => 'TargetPay',
					'display_name' => 'Target Pay',
					'image' => 'target-pay.png',
					'settings' => 'a:2:{s:15:"setSubAccountId";s:10:"1234567890";s:8:"testmode";s:1:"1";}',
					'order' => 29,
					'active' => 0,
			),
			array(
					'id' => 31,
					'name' => 'MultiSafepay',
					'display_name' => 'MultiSafepay',
					'image' => 'multisafepay.png',
					'settings' => 'a:4:{s:9:"accountId";s:7:"Testing";s:6:"siteId";s:10:"1234567890";s:8:"siteCode";s:10:"1234567890";s:8:"testmode";s:1:"1";}',
					'order' => 30,
					'active' => 0,
			),
			array(
					'id' => 32,
					'name' => 'Migs',
					'display_name' => 'Migs',
					'image' => 'migs.png',
					'settings' => 'a:3:{s:21:"setMerchantAccessCode";s:10:"1234567890";s:8:"merchant";s:10:"1234567890";s:8:"testmode";s:1:"1";}',
					'order' => 31,
					'active' => 0,
			),
			array(
					'id' => 33,
					'name' => 'WorldPayXML',
					'display_name' => 'WorldPay XML Direct',
					'image' => 'worldpay.png',
					'settings' => 'a:4:{s:12:"installation";s:10:"1234567890";s:8:"merchant";s:10:"1234567890";s:8:"password";s:10:"1234567890";s:8:"testmode";s:1:"1";}',
					'order' => 32,
					'active' => 0,
			),
			array(
					'id' => 34,
					'name' => 'Mollie',
					'display_name' => 'Mollie',
					'image' => NULL,
					'settings' => 'a:2:{s:6:"apiKey";s:7:"testing";s:8:"testmode";b:0;}',
					'order' => 33,
					'active' => 0,
			),
			array(
					'id' => 35,
					'name' => 'Netaxept',
					'display_name' => 'Netaxept (BBS)',
					'image' => NULL,
					'settings' => 'a:3:{s:10:"merchantId";s:10:"1234567890";s:8:"password";s:10:"1234567890";s:8:"testmode";s:1:"1";}',
					'order' => 34,
					'active' => 0,
			),
			array(
					'id' => 36,
					'name' => 'Veritrans',
					'display_name' => 'Veritrans',
					'image' => 'veritrans.png',
					'settings' => 'a:3:{s:9:"serverKey";s:10:"1234567890";s:11:"environment";s:11:"Development";s:8:"testmode";s:1:"1";}',
					'order' => 35,
					'active' => 0,
			),
			array(
					'id' => 37,
					'name' => 'Neteller',
					'display_name' => 'Neteller',
					'image' => 'neteller.png',
					'settings' => 'a:3:{s:10:"merchantId";s:10:"1234567890";s:11:"merchantKey";s:10:"1234567890";s:8:"testmode";s:1:"1";}',
					'order' => 36,
					'active' => 0,
			),
			array(
					'id' => 38,
					'name' => 'FatZebra',
					'display_name' => 'Fat Zebra',
					'image' => NULL,
					'settings' => 'a:3:{s:8:"username";s:10:"1234567890";s:5:"token";s:10:"1234567890";s:8:"testmode";s:1:"1";}',
					'order' => 37,
					'active' => 0,
			),
			array(
					'id' => 39,
					'name' => 'Globalcloudpay',
					'display_name' => 'Globalcloudpay',
					'image' => 'globalcloudpay.png',
					'settings' => 'a:4:{s:5:"merNo";s:10:"1234567890";s:9:"gatewayNo";s:10:"1234567890";s:7:"signKey";s:10:"1234567890";s:8:"testmode";s:1:"1";}',
					'order' => 38,
					'active' => 0,
			),
			array(
					'id' => 40,
					'name' => 'GoCardless',
					'display_name' => 'GoCardless',
					'image' => 'gocardless.png',
					'settings' => 'a:5:{s:5:"appId";s:10:"1234567890";s:9:"appSecret";s:7:"Testing";s:10:"merchantId";s:10:"1234567890";s:11:"accessToken";s:10:"1234567890";s:8:"testmode";s:1:"1";}',
					'order' => 39,
					'active' => 0,
			),
			array(
					'id' => 41,
					'name' => 'Helcim',
					'display_name' => 'Helcim',
					'image' => 'helcim.png',
					'settings' => 'a:3:{s:10:"merchantId";s:10:"1234567890";s:12:"gatewayToken";s:7:"Testing";s:8:"testmode";s:1:"1";}',
					'order' => 40,
					'active' => 0,
			),
			array(
					'id' => 42,
					'name' => 'NMI',
					'display_name' => 'Network Merchants Inc. (NMI)',
					'image' => 'nmi.png',
					'settings' => 'a:3:{s:8:"username";s:7:"Testing";s:8:"password";s:7:"Testing";s:8:"testmode";s:1:"1";}',
					'order' => 41,
					'active' => 0,
			),
			array(
					'id' => 43,
					'name' => 'Alipay',
					'display_name' => 'Alipay',
					'image' => 'alipay.png',
					'settings' => 'a:3:{s:7:"partner";s:10:"1234567890";s:3:"key";s:10:"1234567890";s:8:"testmode";b:0;}',
					'order' => 42,
					'active' => 0,
			),
			array(
					'id' => 44,
					'name' => 'EBS',
					'display_name' => 'EBS',
					'image' => 'ebs.png',
					'settings' => 'a:3:{s:9:"accountId";s:7:"testing";s:9:"secretKey";s:38:"fd8bdrf402eb3c0f39edds321bf9951799c16f";s:8:"testmode";s:1:"1";}',
					'order' => 43,
					'active' => 0,
			),
			array(
					'id' => 45,
					'name' => 'PayUMoney',
					'display_name' => 'PayUMoney',
					'image' => 'payumoney.png',
					'settings' => 'a:3:{s:11:"merchantKey";s:6:"1JRs9X";s:4:"salt";s:8:"rF3dKEyx";s:8:"testmode";s:1:"1";}',
					'order' => 44,
					'active' => 0,
			),
	);

	/**
	 * Install this migration
	 *
	 * @return void
	 */
	public function up()
	{
		$this->dbforge->add_field($this->fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('name', false);
		$this->dbforge->create_table($this->table_name);
		// INSERT PAYMENT GATEWAYS
		foreach ($this->data as $payment_gateway) {
			$this->db->insert($this->table_name, $payment_gateway);
		}
	}

	//--------------------------------------------------------------------

	/**
	 * Uninstall this migration
	 *
	 * @return void
	 */
	public function down()
	{
		$this->dbforge->drop_table($this->table_name);
	}

	//--------------------------------------------------------------------

}