<?php defined('BASEPATH') || exit('No direct script access allowed');

/**
 * Financial controller
 */
class Financial extends Admin_Controller
{
    protected $permissionCreate = 'Payouts.Financial.Create';
    protected $permissionDelete = 'Payouts.Financial.Delete';
    protected $permissionEdit   = 'Payouts.Financial.Edit';
    protected $permissionView   = 'Payouts.Financial.View';

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
        
        $this->auth->restrict($this->permissionView);
        $this->load->model('payouts/payouts_model');
        $this->load->model('commissions/commissions_model');
        $this->lang->load('payouts');
        
        $this->form_validation->set_error_delimiters("<span class='error'>", "</span>");
        
        Template::set_block('sub_nav', 'financial/_sub_nav');
        Assets::add_module_js('payouts', 'payouts.js');
    }
    
    /**
     * Summary
     */
    public function index() {
        // Display Chart
        Assets::add_js('moment.js');
        Assets::add_css('bootstrap-datetimepicker.min.css' );
        Assets::add_js('bootstrap-datetimepicker.min.js' );
        Assets::add_css('flags.css');
        Assets::add_css('chart.css');
        Assets::add_js('Chart.min.js');
        $this->session->unset_userdata('chart_query_params');
        
        // Six Months Filter
        $six_month_chart = $this->gcrud->chart('months', array('table' => 'payouts', 'field' => 'amount', 'table' => 'payouts'));
        Assets::add_js($this->load->view('common/_chart_js', array('field_id' => 'sixMonthChart', 'labels' => $six_month_chart['labels'], 'data' => $six_month_chart['data'], 'total' => $six_month_chart['total'], 'label' => lang('affiliate_total_referrals'), 'with_dates' => false, 'table' => 'payouts'), true), 'inline');
        
        // Monthly Filter
        $result = $this->gcrud->chart('current_month', array('table' => 'payouts', 'field' => 'amount', 'table' => 'payouts'));
        Assets::add_js($this->load->view('common/_chart_js', array('field_id' => 'monthlyChart', 'labels' => $result['labels'], 'data' => $result['data'], 'total' => $result['total'], 'label' => lang('affiliate_total_referrals'), 'with_dates' => true, 'table' => 'payouts'), true), 'inline');
        
        $record = $this->payouts_model->summary();
        Template::set(array(
            'toolbar_title' => lang('gc_menu_payouts') . ' - ' . lang('gc_menu_summary'),
            'record' => $record
        ));
        Template::render();
    }
    
    /**
     * View Payout
     * @param string $filter
     * @param number $offset
     */
    public function info($module = 'commissions') {
        // DATE TIME PICKER
        Assets::add_js('moment.js');
        Assets::add_css('bootstrap-datetimepicker.min.css' );
        Assets::add_js('bootstrap-datetimepicker.min.js' );
        Assets::add_css('selectize.bootstrap3.css');
        Assets::add_js('selectize.min.js');
        Assets::add_js('selectize-search.js');
        Assets::add_js("
        $('#date_from').datetimepicker({
            format: 'YYYY-MM-DD',
            widgetPositioning: {
                 horizontal: 'right',
                 vertical: 'bottom'
            }
        });
        $('#date_to').datetimepicker({
            format: 'YYYY-MM-DD',
            widgetPositioning: {
                horizontal: 'right',
                vertical: 'bottom'
           }
        });", 'inline');
		$payout_module = $this->input->get('md') && $this->input->get('md') == 'order_items' ? 'order_items' : 'commissions';
		$offset = intval($this->input->get('per_page'));
        if(isset($_POST['update'])) {
            $this->form_validation->set_rules('user_id', 'lang:bf_user', 'trim|is_numeric');
            $this->form_validation->set_rules('module_id', 'lang:label_module', 'trim|is_numeric');
            $this->form_validation->set_rules('date_from', 'lang:label_from', 'trim|required|valid_date');
            $this->form_validation->set_rules('date_to', 'lang:label_to', 'trim|required|valid_date|is_greater_than[date_from]');
            if($this->form_validation->run() == TRUE) {
                $this->session->set_userdata('payout_filter', 
                    array(
                        'user_id' => $this->input->post('user_id'),
                        'module_id' => $this->input->post('module_id'),
                        'from' => $this->input->post('date_from'), 
                        'to' => $this->input->post('date_to')
                    ));
            }
        }
        
        if(!$this->session->userdata('payout_filter')) {
            $this->session->set_userdata('payout_filter', array('from' => date('Y')  . '-' . date('m') . '-01', 'to' => date('Y-m-d')));
        }
		$prefix = 'PM.';
		$payout_user_field = $module == 'order_items' ? 'seller_id' : 'user_id';
        $payout_filter = $this->session->userdata('payout_filter');
        $where = "(DATE(" . $prefix . "created_on) BETWEEN '" . $payout_filter['from'] . "' AND '" . $payout_filter['to'] . "')";
        if(!empty($payout_filter['user_id'])) {
            $where .= " AND " . $prefix . $payout_user_field . " = " . $payout_filter['user_id'];
        }
        if(!empty($payout_filter['module_id'])) {
            $where .= " AND " . $prefix . "module_id = " . $payout_filter['module_id'];
        }
        $settings = gc_json_settings('payout.settings'); 
        $sort_by = $prefix . 'created_on DESC';
        $limit = $this->limit;
        // search
        $searchterm = $this->gc->search('payout_' . $payout_module, 0, '/financial/payouts/info/' . $module);
        if(!empty($searchterm)) {
            $where .= " AND " . $prefix . "user_id LIKE '%" .$searchterm . "%'";
        }
        // sort filter
        $order_by = $this->gc->sort('payout_' . $payout_module, 'PM');
        if(!empty($order_by)) {
            $sort_by = $order_by;
        }

        // set minimum payout
        $minimum_payout = $settings['minimum_payout'];
        if(!empty($payout_filter['module_id']) && $module == 'order_items') {
            if($payout_filter['module_id'] == 2) {
                $minimum_payout = $settings['ecom_min_payout'];
            } else if($payout_filter['module_id'] == 29) {
                $minimum_payout = $settings['bookings_min_payout'];
            }
        }
		$records = $this->payouts_model->payout_info(0, $where, $sort_by, $limit, $offset, array('minimum_payout' => $minimum_payout, 'module' => $payout_module));
		$total_rows = $this->payouts_model->payout_info(1, $where, $sort_by, $limit, $offset, array('minimum_payout' => $minimum_payout, 'module' => $payout_module));

        // Pagination
        $this->load->library('pagination');
        $pager['base_url'] = site_url(SITE_AREA ."/financial/payouts/info/" . $module);		
		$pager['total_rows'] = $total_rows;
		$pager['per_page'] = $limit;
		// Pagination using GET Method
		$pager['reuse_query_string'] = TRUE;
		$pager['enable_query_strings'] = TRUE;
		$pager['page_query_string'] = TRUE;
        
        $this->pagination->initialize($pager);
        Template::set(array(
            'minimum_payout' => $minimum_payout,
            'toolbar_title' => lang($payout_module == 'commissions' ? 'payout_pay_commissions' : 'payout_orders'),
            'records' => $records,
            'total_rows' => $total_rows,
            'payout_filter' => $payout_filter,
            'payout_module' => $payout_module
        ));
        // Template::set_view('index');
        Template::render();
    }
    
    /**
     * Pay Affiliates
     * @param number $id
     * @param string $field
     */
    public function pay() {        
        // DATE TIME PICKER
        Assets::add_js('moment.js');
        Assets::add_css('bootstrap-datetimepicker.min.css' );
        Assets::add_js('bootstrap-datetimepicker.min.js' );
        Assets::add_js("
        $('#paid_on').datetimepicker({
            format: 'YYYY-MM-DD HH:mm:ss',
            sideBySide: true,
            widgetPositioning: {
                 horizontal: 'right',
                 vertical: 'bottom'
            }
        });", 'inline');
		$id = $this->input->get('id'); 
		$payout_module = $this->input->get('md') && $this->input->get('md') == 'order_items' ? 'order_items' : 'commissions';
		$field = $payout_module == 'commissions' ? 'user_id' : 'seller_id';
		$amount_field = $payout_module == 'commissions' ? 'amount' : 'payout';
		$prefix = $payout_module == 'commissions' ? 'C.' : 'PM.';
        $this->load->model('affiliates/affiliates_model');
        $this->load->model('commissions/commissions_model');
		if($payout_module == 'commissions') {
        	$record = $this->affiliates_model->get_affiliate_user($id, true, $field);
			if(!$record) {
        	    $record = $this->db->select('*, display_name AS name, id AS user_id')->where('id', $id)->get('users')->row();
        	}
		} else {
			$record = $this->db->select('*, display_name AS name, id AS user_id')->where('id', $id)->get('users')->row();
		}
		if(!$record) { // || empty($record->commission_amount) - creating issues with users
            Template::redirect(site_url(SITE_AREA . '/financial/payouts'));
        }
        
        // get passbook information
        $passbook = $this->db->where('user_id', $record->user_id)->order_by('id', 'DESC')->get('passbooks')->row();
        $record->closing_balance = $passbook ? $passbook->closing_balance : 0;

        $affiliate = $record;
        $settings = gc_json_settings('payout.settings');        
        if(!$this->session->userdata('payout_filter')) {
            $this->session->set_userdata('payout_filter', array('from' => date('Y')  . '-' . date('m') . '-01', 'to' => date('Y-m-d')));
        }
        $payout_filter = $this->session->userdata('payout_filter');
        $where = "(DATE(" . $prefix . "created_on) BETWEEN '" . $payout_filter['from'] . "' AND '" . $payout_filter['to'] . "')";
        if(!empty($payout_filter['module_id'])) {
            $where .= " AND " . $prefix . "module_id = " . $payout_filter['module_id'];
        }
        $where .= " AND " . $prefix . "payout_id = 0";

        // get amount only for the checked items - needed because of datatables pagination
        // $this->session->unset_userdata('payout_checked_items');
        $checked_key = $payout_module . '_' . $record->user_id;
        $payout_checked_items = (array) $this->session->userdata('payout_checked_items');
        if(!empty($payout_checked_items[$checked_key])) {
            $where .= " AND " . $prefix . "id IN (" . implode(',', $payout_checked_items[$checked_key]) . ")";
        }

		if($payout_module == 'commissions') {
			// Display earned commissions using datatables
			$this->affiliates_model->logs(9, $record->user_id, 'admin', $record, false);
			
			// Commission amount earned till date
			$commission_amount = $this->commissions_model->payout_user_commission_amount($record->user_id, 2, $where);
			$record->payout_amount = $commission_amount->commission_amount;
		} else {
			// Display earned commissions using datatables
			$this->affiliates_model->logs(10, $record->user_id, 'admin', $record, false);
			$where .= " AND PM.status_id = 2 AND PM.deleted = 0 AND PM." . $field . " = " . $record->user_id;
			$this->db->from('order_items PM');
			$result_payout = $this->db->select('SUM(PM.payout) AS payout_amount')->where($where)->get()->row();
			$record->payout_amount = $result_payout->payout_amount;
		}

        // get all items and save them in the payout_checked_items variable for datatables pagination
        // needed because of datatables pagination
		if(empty($payout_checked_items[$checked_key])) {
		    if($payout_module == 'commissions') {
    		    $records_sql = $this->commissions_model->_summary_commission($record->user_id, 2, $where, 'C.id');
    		    $items_record = $this->db->query($records_sql)->result();
		    } else {
		        $this->db->from('order_items PM');
			    $items_record = $this->db->select('PM.id')->where($where)->get()->result();
		    }
		    if($items_record) {
		        $payout_items = array();
		        foreach($items_record as $item_record) {
		            $payout_items[] = $item_record->id;
		        }
		        $this->session->set_userdata('payout_checked_items', array($checked_key => $payout_items));
		    }
		}
		
        // only pay the selected items
        Assets::add_js("
        $(document).on('click', '.chk-status', function() {
            $.post('" . site_url('ajax/payout_amount') . "', {'ci_csrf_token': '" . $this->security->get_csrf_hash() . "', 'user_id': " . $record->user_id. ", 'item_id': $(this).val(), 'status': $(this).is(':checked') ? 1 : 0}, function( result ) {
                $('#payout_amount').val(result);
            });
        });
		$('.btn-payout-tax').click(function() {
            var payoutAmount = parseFloat($('.payout-amount').html());
            if($(this).val() == 1) {
                var payoutTax = ((payoutAmount * 18) / 100).toFixed(2);
                var payoutAmountWithTax = (parseFloat(payoutAmount) + parseFloat(payoutTax)).toFixed(2);
                $('#payout_amount').val(payoutAmountWithTax);
                $('.amount-with-tax').html(payoutAmount + ' + ' + payoutTax + ' = ' + payoutAmountWithTax);
                $('.amount-with-tax').show();
                $(this).val(0);
                $(this).html($(this).data('with'));
            } else {
                $('#payout_amount').val(payoutAmount);
                $('.amount-with-tax').html('');
                $('.amount-with-tax').hide();
                $(this).val(1);
                $(this).html($(this).data('without'));
            }
        });
        ", 'inline');
		
		// Ajax Price Update
		$payouts_user = array();
		$payouts_user[$record->user_id] = array(
		    'user_id' => $record->user_id,
		    'module' => $payout_module,
		    'where' => $where
		);
		$this->session->set_userdata('payouts_user', $payouts_user);
		
        // Save Payouts Information
        if (isset($_POST['save'])) {
            $this->auth->restrict($this->permissionEdit);
			$this->form_validation->set_rules('payout_amount', lang('payout_amount'), 'trim|required|is_decimal');
            $this->form_validation->set_rules('paid_on', lang('paid_on'), 'trim');
            $this->form_validation->set_rules('payment_method', lang('payment_method'), 'trim');
            $this->form_validation->set_rules('transaction_id', lang('transaction_id'), 'trim');
            $this->form_validation->set_rules('comments', lang('label_comments'), 'trim');
            $this->form_validation->set_rules('notes', lang('label_notes'), 'trim');
            if($this->form_validation->run() === TRUE) {
				$payout_checked_items = (array) $this->session->userdata('payout_checked_items');
				$checked = array();
                if(!empty($payout_checked_items[$checked_key])) {
                    $checked = $payout_checked_items[$checked_key];
                }
                // Get user commissions information to update in the payouts table for reporting purposes
                $payouts_where = $where . ' AND ' . $prefix . $field . ' = ' . $record->user_id . ' AND ' . $prefix . 'status_id = 2 AND ' . $prefix . 'id IN (' . implode(',', $checked) . ')';
                $sql = "SELECT " . $prefix . "id, " . $prefix . "module_id, " . $prefix . "order_id, " . $prefix . $amount_field . " AS amount, " . $prefix . "created_on FROM " . $this->db->dbprefix . $payout_module . ' ' . str_replace('.', '', $prefix) . " WHERE " . $payouts_where;
                $payouts = $this->db->query($sql)->result();
                $wallet_transfer = $this->input->post('wallet_transfer') ? true : false;
                $payout_amount = $this->input->post('payout_amount');
                $payment_method = $this->input->post('payment_method');
                $data = array(
                    'user_id' => $record->user_id,
					'module_id' => gc_module_id($payout_module),
                    'toggle_module_id' => !empty($payout_filter['module_id']) ? $payout_filter['module_id'] : 0, // affiliate_users id
                    'amount' => $payout_amount,
                    'paid_on' => $this->input->post('paid_on'),
                    'payment_method' => empty($payment_method) && $wallet_transfer ? lang('wallet_transfer') : $payment_method,
                    'transaction_id' => $this->input->post('transaction_id'),
                    'comments' => $this->input->post('comments'),
                    'notes' => $this->input->post('notes'),
                    'created_on' => date('Y-m-d H:i:s'),
                    'created_by' => $this->auth->user_id(),
                    'created_ip' => $this->input->ip_address(),
					'status' => 1
                );
                $data['params'] = json_encode(array(
                    'affiliate' => $affiliate,
                    'payouts' => $payouts
                ));
                $this->db->insert('payouts', $data);
                $payout_id = $this->db->insert_id();
                if($payout_id > 0) {
                    // make wallet transaction
                    if($wallet_transfer) {
                        $this->load->model('wallets/wallets_points_model');
                        $this->wallets_points_model->update_cash('wallets', $record->user_id, 'payouts', $record->payout_amount, 'credit', $data['notes'], 0, array('payout_id' => $payout_id));
                    }
                    foreach($payouts as $payout) {
                        // update payout_id in the commissions table to be paid
                        $this->db->where('id', $payout->id)->update($payout_module, array('payout_id' => $payout_id, 'modified_on' => date('Y-m-d H:i:s'), 'modified_ip' => $this->input->ip_address()));
                    }
                    if($payout_module == 'order_items') {
						$order_item = new stdClass();
						$order_item->id = 0;						
						$order_item->user_id = $record->user_id;	
						$order_item->module_id = !empty($payout_filter['module_id']) ? $payout_filter['module_id'] : 0;
						$order_item->order_id = 0;
						$order_item->status_id = 1;
						$this->payouts_model->save_history($order_item, $record->payout_amount, array('payout_id' => $payout_id));
					}
					
					// update passbook
					$this->gc->update_passbook($record->user_id, 'payouts', $payout_id, $payout_amount, 'debit');

                    // notify user via email/sms
                    if($this->input->post('notify_user') == 1) {
                        $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'payout_email_templates', 'module_table' => 'payouts', 'action_type' => 'add', 'id' => $payout_id));
                    }
                    log_activity($this->auth->user_id(), lang('act_edit_record') . ': ' . $id . ' : ' . $this->input->ip_address(), 'payouts');
                    // clear checked items session - needed because of datatable
                    $this->session->unset_userdata('payout_checked_items');
					Template::set_message(sprintf(lang('msg_save_success'), lang('label_payout')), 'success');
                    // redirect user to the affiliate receipt page
                    Template::redirect(site_url(SITE_AREA . '/financial/payouts/invoice/' . $payout_id . '/1'));
                }
            }
        }

        // set minimum payout
        $minimum_payout = $settings['minimum_payout'];
        if(!empty($payout_filter['module_id']) && $payout_module == 'order_items') {
            if($payout_filter['module_id'] == 2) {
                $minimum_payout = $settings['ecom_min_payout'];
            } else if($payout_filter['module_id'] == 29) {
                $minimum_payout = $settings['bookings_min_payout'];
            }
        }
        
        Template::set(array(
            'toolbar_title' => $record->name,
            'record' => $record,
            'minimum_payout' => $minimum_payout,
            'payout_filter' => $payout_filter,
			'payout_module' => $payout_module
        ));
        Template::render();
    }
    
    /**
     * Display a list of Payouts data
     * @param string $module
     * @param string $filter
     * @param number $offset
     */
    public function manage($module = 'payouts', $filter = 'all', $offset = 0) {
        $extra = array(
            'model' => 'payouts',
            'context' => 'financial',
            'fields' => $module
        );
        if($module == 'payouts') {
            // affiliate_statuses table
            $extra['table_prefix'] = 'payout';
        } else {
            $extra['prefix'] = 'payout';
            if($module == 'hierarchies' || $module == 'coupons') {
                $extra['actions'] = array( // Extra buttons
                    array('type' => 'link', 'link' => site_url(SITE_AREA . '/financial/commissions/form/' . $module . '/edit/%s'), 'label' => '<i class="fas fa-edit fa-fw mr-1"></i>' . lang('edit'), 'class' => 'btn btn-xs btn-primary'),
                );
            }
			if($module == 'passbook' && !empty($_GET)) {
				$query_where = array();
				if($this->input->get('user_id')) {
					$query_where[] = 'M.user_id = ' . $this->input->get('user_id');
				}
				if($this->input->get('module_id')) {
					$query_where[] = 'M.module_id = ' . $this->input->get('module_id');
				}
				$extra['query_where'] = implode(' AND ', $query_where);
			}
        }
        $this->gc->module_view($module, $filter, $offset, 'admin', $extra);
    }
    
    /**
     * Add/Edit Payouts Data
     *
     * @return void
     */
    public function form($module = 'statuses', $action = 'create') {
        $extra = array(
            'selectize' => true,
            'model' => 'payouts',
            'context' => 'financial',
            'fields_method' => $module
        );
        if($module != 'payouts') {
            $extra['prefix'] = 'payout';
            $extra['module_name'] = 'payouts';
        }
        $this->gcrud->form($module, $action, 'admin', $extra);
    }
    
    
    
    public function fields() {
        $this->load->library('fields/fields');
        //$this->fields->prepare_fields('commission_levels');
        // $this->fields->permissions('Commission_Products', 'Financial');
    }
    
    /**
     * Displays a list of queries data.
     *
     * @return void
     */
    public function index__($filter='all', $offset=0) {
    	$this->payouts($filter, $offset, 'index');
    }
    
    public function affiliates($filter='all', $offset=0) {
    	$this->payouts($filter, $offset, 'affiliates');
    }
    
    public function products($filter='all', $offset=0) {
    	$this->payouts($filter, $offset, 'products');
    }

    private function payouts($filter, $offset, $module) {
		// Do we have any actions?
		if (isset($_POST['submit'])) {
			$action = $this->input->post('submit');
			$checked = $this->input->post('checked');
			if($checked) {
				$module_name = count($checked) == 1 ? lang('label_payout') : lang('label_payouts') ;
				switch(strtolower($action)) {
					case 'paid':
						foreach($checked as $pid) {
							$paid_id = $this->payouts_model->_request_paid($pid, 1);
						}
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_activated')), 'success');
						break;
							
					case 'request':
						foreach($checked as $pid) {
							$this->payouts_model->_request_paid($pid, 0);
						}
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_deactivated')), 'success');
						break;
							
					case 'delete':
						$this->gc->update_status('payouts', $checked, 'deleted', 1);
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_deleted')), 'success');
						break;
							
					case 'purge':
						$this->gc->purge_records('payouts', $checked);
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_deleted')), 'success');
						break;
							
					case 'restore':
						$this->gc->update_status('payouts', $checked, 'deleted', 0);
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_restored')), 'success');
						break;
				}
			}
		}
		
		// Change Status
		if (isset($_POST['change_status']))	 {
			if(!empty($_POST['status'])) {
				$new_status = $_POST['status'] == 99 ? 0 : $_POST['status'];
				$this->payouts_model->_request_paid($_POST['change_status'], $new_status);
				Template::set_message(sprintf(lang('msg_action_success'), lang('label_payout'), lang('label_updated')), 'success');
			}
		}
		
		// Actions done, now display the view
		$where = "P.deleted = 0";
		// Filters
		$filter_type = $filter;
		
		switch($filter_type) {
			case 'active': // New Requests
				$where .= " AND P.status = 0";
				break;
					
			case 'paid':
				$where .= " AND P.status = 1";
				break;
		
			case 'deleted':
				$where = "P.deleted = 1";
				break;
		
			case 'all':
				// Nothing to do
				break;
		
			default:
				show_404("payouts/index/$filter/");
		}
		
		$searchterm = $this->gc->search('payouts', $this->uri->segment(6), '/financial/payouts/');
		if(!empty($searchterm)) {
			$where .= " AND (P.id LIKE '%". $searchterm ."%' OR U.display_name LIKE '%". $searchterm ."%' OR U.email LIKE '%". $searchterm ."%' OR P.requested_on LIKE '%". $searchterm ."%' OR P.paid_on LIKE '%". $searchterm ."%')";
		}
		
		$sort_by = "P.id DESC";
		if(isset($_POST['sort']) && $_POST['sort'] == 'display_name') {
			$order_by = $this->gc->sort('payouts', 'U');
		} else {
			$order_by = $this->gc->sort('payouts', 'P');
		}
		if(!empty($order_by)) {
			$sort_by = $order_by;
		}
		
		if($module == 'index') {
			$records = $this->payouts_model->get_all_payouts(0, $where, $sort_by, $offset, $this->limit);
		} else {
			$records = $this->payouts_model->get_payouts(0, 0, $module, $where, $sort_by, $offset, $this->limit);
		}
		
		Template::set('records', $records);
			
		// Pagination
		$this->load->library('pagination');
		
		if($module == 'index') {
			$total_payouts = $this->payouts_model->get_all_payouts(1, $where, $sort_by, $offset, $this->limit);
		} else {
			$total_payouts = $this->payouts_model->get_payouts(1, 0, $module, $where, $sort_by, $offset, $this->limit);
		}
		
		$pager['base_url'] = site_url(SITE_AREA ."/financial/payouts/$module/$filter/");
		$pager['total_rows'] = $total_payouts;
		$pager['per_page'] = $this->limit;
		$pager['uri_segment']	= 6;
		
		$this->pagination->initialize($pager);
		
		Template::set('index_url', site_url(SITE_AREA .'/financial/payouts/' . $module .'/'));
		Template::set('filter_type', $filter_type);
		Template::set('module', $module);
		Template::set('toolbar_title', lang('manage_payouts'));
		Template::set_view('payouts/financial/view');
		Template::render();
	}
	
	/**
	 * Mark Payout Status as Paid
	 */
	public function edit() {

		//BUSINESS HOURS TIME PICKER
		Assets::add_js('moment.js');
		Assets::add_css ( 'bootstrap-datetimepicker.min.css' );
		Assets::add_js ( 'bootstrap-datetimepicker.min.js' );
		
		$id = $this->uri->segment(5);
		if (empty($id)) {
			Template::set_message(lang('invalid_id'), 'danger');
		
			redirect(SITE_AREA . '/financial/payouts');
		}
		$payout = $this->payouts_model->get_payout($id);
		
		if (isset($_POST['save'])) {
			$this->auth->restrict($this->permissionEdit);
		
			if ($this->save_payout('update', $id)) {
				
				// if the user updated the status field
				if($payout->status != $this->input->post('status')) {
					$this->payouts_model->update_payout_user($payout, $this->input->post('status'));					
				}
				// notify user via email
				if($this->input->post('notify_user') == 1) {					
					$this->payouts_model->notify_payout_success($id);
				}
				log_activity($this->auth->user_id(), lang('act_edit_record') . ': ' . $id . ' : ' . $this->input->ip_address(), 'payouts');
				Template::set_message(sprintf(lang('msg_save_success'), lang('label_payout')), 'success');
				redirect(SITE_AREA . '/financial/payouts');
			}
		
			// Not validation error
			if ( ! empty($this->payouts_model->error)) {
				Template::set_message(sprintf(lang('msg_edit_failure'), lang('label_payout')) . $this->payouts_model->error, 'danger');
			}
		}
		Template::set('payout', $payout);
		
		Template::set('toolbar_title', lang('mark_paid'));
		Template::render();
	}
	
	/**
	 * Manage All Payout Users
	 * @param string $filter
	 * @param number $offset
	 */
	public function payout_users($filter = 'all', $offset = 0) {
		//LOAD TinyMCE EDITOR
		Assets::add_js(array('tinymce/tinymce.min.js'));
		Assets::add_js('admin-reply-email.js');
		
		// Status Change
		if (isset($_POST['status_change'])) {
			$this->gc->status_change('payout_users', $_POST['status_change'], 'status');
		}
	
		// Transfer amount form payout to wallet
		if(isset($_POST['wallet_transfer'])) {
		    $this->payouts_model->wallet_transfer($_POST['wallet_transfer'], 0, 'credit');
		    Template::redirect(current_url());
		}
		
		// Do we have any actions?
		if (isset($_POST['submit'])) {
			$action = $this->input->post('submit');
			$checked = $this->input->post('checked');
			if($checked) {
				$module_name = count($checked) == 1 ? lang('label_payout_user') : lang('label_payout_users') ;
				switch(strtolower($action)) {
					case 'activate':
						$this->gc->update_status('payout_users', $checked, 'status', 1);
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_activated')), 'success');
						break;
							
					case 'deactivate':
						$this->gc->update_status('payout_users', $checked, 'status', 0);
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_deactivated')), 'success');
						break;					
							
					case 'delete':
						$this->gc->update_status('payout_users', $checked, 'deleted', 1);
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_deleted')), 'success');
						break;
							
					case 'purge':
						// Delete all affiliate coupons
						foreach($checked as $checked_id) {
							$user_record = $this->db->select('user_id, type')->get_where('payout_users', array('id' => $checked_id))->row();
							if($user_record && $user_record->type == gc_module_id('affiliates')) {
								$this->db->delete('affiliate_coupons', array('user_id' => $user_record->user_id));
							}
						}
						$this->gc->purge_records('payout_users', $checked);
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_deleted')), 'success');
						break;
							
					case 'restore':
						$this->gc->update_status('payout_users', $checked, 'deleted', 0);
						Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_restored')), 'success');
						break;
				}
			}
		}
		
		// Payout Request Handling
		if (isset($_POST['request'])) {
			$payout = $this->db->select('pending_payout, user_id, module_id')->get_where('payout_users', array('id' => $_POST['request']))->row();
			if($payout) {
			    $reference_no = $this->payouts_model->payout_request($payout->user_id, $payout->module_id);
				if($reference_no) {
					Template::set_message(sprintf(lang('payout_request_success'), $reference_no), 'success');
					// notify user via email
					$this->payouts_model->notify_payout_request($payout->user_id, $reference_no);
				} else {
					if($payout->pending_payout < settings_item('fin.minimum_payout')) {
						Template::set_message(sprintf(lang('minimum_payout_error'), currency_format(decimal_convert(settings_item('fin.minimum_payout'))), currency_format(decimal_convert((floatval(settings_item('fin.minimum_payout')) - floatval($payout->pending_payout))))), 'danger');
					} else {
						Template::set_message(lang('payout_already_requested'), 'danger');
					}
				}
			} else {
				Template::set_message(lang('payout_request_error'), 'danger');
			}
		}
		
		// Actions done, now display the view
		$where = "PU.deleted = 0";
		// Filters
		$filter_type = $filter;
		
		switch($filter_type) {
			case 'active': // New Requests
				$where .= " AND PU.status = 1";
				break;
					
			case 'inactive':
				$where .= " AND PU.status = 0";
				break;
			
			case 'affiliates':
				$where .= " AND PU.module_id = " . gc_module_id('affiliates');
				break;
				
			case 'products':
				$where .= " AND PU.module_id = " . gc_module_id('products');
				break;
				
			case 'bookings':
			    $where .= " AND PU.module_id = " . gc_module_id('bookings');
			    break;
			    
			case 'invitations':
			    $where .= " AND PU.module_id = " . gc_module_id('invitations');
			    break;
		
			case 'deleted':
				$where = "PU.deleted = 1";
				break;
		
			case 'all':
				// Nothing to do
				break;
		
			default:
				show_404("payouts/index/$filter/");
		}
		
		$searchterm = $this->gc->search('payout_users', $this->uri->segment(6), '/financial/payouts/payout_users');
		if(!empty($searchterm)) {
			$where .= " AND (PU.id LIKE '%". $searchterm ."%' OR U.display_name LIKE '%". $searchterm ."%' OR U.email LIKE '%". $searchterm ."%' OR PU.created_on LIKE '%". $searchterm ."%')";
		}
		
		$sort_by = "PU.id DESC";
		if(isset($_POST['sort']) && $_POST['sort'] == 'display_name') {
			$order_by = $this->gc->sort('payout_users', 'U');
		} else {
			$order_by = $this->gc->sort('payout_users', 'PU');
		}
		if(!empty($order_by)) {
			$sort_by = $order_by;
		}
		
		$records = $this->payouts_model->get_all_payout_users(0, $where, $sort_by, $offset, $this->limit);		
		Template::set('records', $records);
			
		// Pagination
		$this->load->library('pagination');
		
		$total_users = $this->payouts_model->get_all_payout_users(1, $where, $sort_by, $offset, $this->limit);
		
		$pager['base_url'] = site_url(SITE_AREA ."/financial/payouts/payout_users/$filter/");
		$pager['total_rows'] = $total_users;
		$pager['per_page'] = $this->limit;
		$pager['uri_segment']	= 6;
		
		$this->pagination->initialize($pager);
		
		Template::set('index_url', site_url(SITE_AREA .'/financial/payouts/payout_users') .'/');
		Template::set('filter_type', $filter_type);
		Template::set('toolbar_title', lang('manage_payouts'));
		Template::set_view('financial/payout_users');
		Template::render();
	}
	
	//--------------------------------------------------------------------------
	// !PRIVATE METHODS
	//--------------------------------------------------------------------------
	
	/**
	 * Save the data.
	 *
	 * @param string $type Either 'insert' or 'update'.
	 * @param int    $id   The ID of the record to update, ignored on inserts.
	 *
	 * @return boolean|integer An ID for successful inserts, true for successful
	 * updates, else false.
	 */
	private function save_payout($type = 'insert', $id = 0) {
		if ($type == 'update') {
			$_POST['id'] = $id;
		}
	
		$data = array();
		// Validate the data
		$this->form_validation->set_rules($this->payouts_model->get_validation_rules());
		if ($this->form_validation->run() === false) {
			return false;
		}
	
		// Make sure we only pass in the fields we want
		//$data = $this->payouts_model->prep_data($this->input->post());
		$data['paid_on'] 		= $this->input->post('paid_on');
		$data['payment_method'] = $this->input->post('payment_method');
		$data['transaction_id'] = $this->input->post('transaction_id');
		$data['comments'] 		= $this->input->post('comments');
		$data['notes'] 			= $this->input->post('notes');
		$data['status'] 		= $this->input->post('status');
		
		// Additional handling for default values should be added below,
		// or in the model's prep_data() method
	
		$return = false;
		if ($type == 'insert') {
			$id = $this->payouts_model->insert($data);
	
			if (is_numeric($id)) {
				$return = $id;
			}
		} elseif ($type == 'update') {
			$return = $this->payouts_model->update($id, $data);
		}	
		return $return;
	}
	
	/**
	 * Set Minimum Payout Amount
	 */
	public function settings() {
	    clear_cache('site_settings'); // Clear Site Settings Cache
	    $setting_name = 'payout.settings';
	    if (isset($_POST['save'])) {
	        $this->form_validation->set_rules('minimum_payout', 'lang:affiliates_min_payout', 'trim|required|is_decimal');
	        $this->form_validation->set_rules('ecom_min_payout', 'lang:ecom_min_payout', 'trim|required|is_decimal');
	        $this->form_validation->set_rules('bookings_min_payout', 'lang:bookings_min_payout', 'trim|required|is_decimal');
	        if($this->form_validation->run() == TRUE) {
	            $settings = array(
	                'minimum_payout' => $this->input->post('minimum_payout'),
	                'ecom_min_payout' => $this->input->post('ecom_min_payout'),
	                'bookings_min_payout' => $this->input->post('bookings_min_payout')
	            );
	            $data = array (array('name' => $setting_name, 'value' => json_encode($settings)));
	            $updated = $this->settings_model->update_batch($data, 'name');
	            if ($updated) {
	                Template::set_message(sprintf(lang('msg_save_success'), lang('gc_menu_settings')), 'success');
	                Template::redirect(site_url(SITE_AREA . '/financial/payouts/settings'));
	            } else {
	                Template::set_message(sprintf(lang('msg_edit_failure'), lang('gc_menu_settings')), 'danger');
	            }
	        }
	    }
	    $result = $this->db->select('value')->get_where('settings', array('name' => $setting_name))->row();
	    if($result) {
	        $settings = json_decode($result->value ?? '');
	    } else {
	        $settings = json_decode(settings_item($setting_name) ?? '');
	    }
	    Template::set('settings', $settings);
	    Template::set('toolbar_title', lang('gc_menu_settings'));
	    Template::render();
	}
	
	/**
	 * Generate PDF Invoice
	 */
	public function invoice($payout_id, $view = 0) {
	    $this->lang->load('orders/orders');
	    $this->load->model('payouts/payouts_model');
	    $this->load->model('commissions/commissions_model');
	    $this->payouts_model->email_print($payout_id, $view == 1 ? 'view' : 'pdf', array(), 'admin');
	}
}