<?php defined('BASEPATH') || exit('No direct script access allowed');

class Migration_Install_payouts extends Migration
{
	/**
	 * @var string The name of the database table
	 */
	private $table_name = 'payouts';
	private $table_payout_users = 'payout_users';
	private $table_payout_histories = 'payout_histories';

	/**
	 * @var array The table's fields
	 */
	private $fields = array(
		'id' => array(
			'type'       => 'INT',
			'constraint' => 11,
			'auto_increment' => true,
		),
        'user_id' => array(
            'type'       => 'BIGINT',
            'constraint' => 20,
            'null'       => false,
        ),
        'module_id' => array(
            'type'       => 'TINYINT',
            'constraint' => 1,
            'default'       => 0,
        ),
        'payment' => array(
            'type'       => 'FLOAT',
            'default'       => 0,
        ),
		'request_number' => array(
			'type'       => 'VARCHAR',
			'constraint' => 30,
			'null'       => true,
		),
        'requested_on' => array(
            'type'       => 'DATETIME',
            'null'       => TRUE
        ),
        'paid_on' => array(
            'type'       => 'DATETIME',
            'null'       => TRUE
        ),
		'payment_method' => array(
			'type'       => 'VARCHAR',
			'constraint' => 150,
			'null'       => true,
		),
		'transaction_id' => array(
			'type'       => 'VARCHAR',
			'constraint' => 255,
			'null'       => true,
		),
		'comments' => array(
			'type'       => 'TEXT',
			'null'       => true,
		),
		'notes' => array(
			'type'       => 'TEXT',
			'null'       => true,
		),
        'status' => array(
            'type'       => 'TINYINT',
            'constraint' => 2,
            'default'    => 0,
        ),
        'deleted' => array(
            'type'       => 'TINYINT',
            'constraint' => 1,
            'default'    => '0',
        ),
        'deleted_by' => array(
            'type'       => 'BIGINT',
            'constraint' => 20,
            'null'       => true,
        ),
		'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
        'created_by' => array(
            'type'       => 'BIGINT',
            'constraint' => 20,
            'null'       => false,
            'default'    => 0
        ),
        'created_ip' => array(
            'type'       => 'VARCHAR',
            'constraint' => 50,
            'null'       => true,
        ),
		'modified_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
        'modified_by' => array(
            'type'       => 'BIGINT',
            'constraint' => 20,
            'null'       => false,
            'default'    => 0
        ),
        'modified_ip' => array(
            'type'       => 'VARCHAR',
            'constraint' => 50,
            'null'       => true,
        ),
	);
	
	/**
	 * @var array The table's fields
	 */
	private $payout_users_fields = array(
		'id' => array(
			'type'       => 'INT',
			'constraint' => 11,
			'auto_increment' => true,
		),
		'user_id' => array(
			'type'       => 'BIGINT',
			'constraint' => 20,
			'null'       => false,
		),
		'module_id' => array(
			'type'       => 'TINYINT',
			'constraint' => 1,
			'default'    => 0,
		),
		'gross_earning' => array(
			'type'      => 'FLOAT',
			'default'   => 0,
		),
		'pending_payout' => array(
			'type'      => 'FLOAT',
			'default'   => 0,
		),
		'is_requested' => array(
			'type'       => 'TINYINT',
			'constraint' => 1,
			'default'   => 0,
		),
		'requested_on' => array(
			'type'       => 'DATETIME',
			'null'       => true
		),
		'status' => array(
			'type'       => 'TINYINT',
			'constraint' => 2,
			'default'    => 1,
		),
		'deleted' => array(
			'type'       => 'TINYINT',
			'constraint' => 1,
			'default'    => '0',
		),
		'deleted_by' => array(
			'type'       => 'BIGINT',
			'constraint' => 20,
			'null'       => true,
		),
		'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'created_by' => array(
			'type'       => 'BIGINT',
			'constraint' => 20,
			'null'       => false,
		),
		'created_ip' => array(
			'type'       => 'VARCHAR',
			'constraint' => 50,
			'null'       => true,
		),
		'modified_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'modified_by' => array(
			'type'       => 'BIGINT',
			'constraint' => 20,
			'null'       => true,
		),
		'modified_ip' => array(
			'type'       => 'VARCHAR',
			'constraint' => 50,
			'null'       => true,
		),
	);
	
	/**
	 * @var array The table's fields
	 * For reference purposes
	 */
	private $payout_histories_fields = array(
	    'id' => array(
	        'type'       => 'INT',
	        'constraint' => 11,
	        'auto_increment' => true,
	    ),
	    'user_id' => array(
	        'type'       => 'BIGINT',
	        'constraint' => 20,
	        'null'       => false,
	    ),
	    'module_id' => array(
	        'type'       => 'TINYINT',
	        'constraint' => 1,
	        'default'    => 0,
	    ),
	    'row_id' => array(
	        'type'       => 'BIGINT',
	        'constraint' => 20,
	        'null'       => false,
	    ),
	    'order_id' => array(
	        'type'       => 'VARCHAR',
	        'constraint' => 20,
	        'null'       => true
	    ),
	    'order_status' => array(
	        'type'       => 'TINYINT',
	        'constraint' => 1,
	        'default'    => 1,
	    ),
	    'amount' => array(
	        'type'      => 'FLOAT',
	        'default'   => 0,
	    ),
	    'action' => array(
	        'type'       => 'TINYINT',
	        'constraint' => 1,
	        'default'    => 1, // 1 - SUM, 0 - MINUS
	    ),
		'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
	    'created_ip' => array(
	        'type'       => 'VARCHAR',
	        'constraint' => 50,
	        'null'       => true,
	    )
	);

	/**
	 * Install this version
	 *
	 * @return void
	 */
	public function up()
	{
		// Payout Table
		$this->dbforge->add_field($this->fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->create_table($this->table_name);
		
		// Payout Users Table
		$this->dbforge->add_field($this->payout_users_fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->create_table($this->table_payout_users);
		$unqiue_payout_user = "ALTER TABLE " . $this->db->dbprefix . $this->table_payout_users . " ADD UNIQUE INDEX `unique_payout_user` (`user_id`,`type`)";
		$this->db->query($unqiue_payout_user);
		
		// Install Settings Information
		$this->db->insert('settings', array('name' => 'fin.minimum_payout', 'module' => 'finance', 'value' => '20'));
		
		// Payout Histories Table
		$this->dbforge->add_field($this->payout_histories_fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->create_table($this->table_payout_histories);
	}

	/**
	 * Uninstall this version
	 *
	 * @return void
	 */
	public function down() {
		$this->dbforge->drop_table($this->table_name);
		$this->dbforge->drop_table($this->table_payout_users);
		$this->dbforge->drop_table($this->table_payout_histories);
		$minimum_payout = $this->db->select('name')->get_where('settings', array('name' => 'fin.minimum_payout'));
		if($minimum_payout) {
			$this->db->delete('settings', array('name' => 'fin.minimum_payout'));
		}
	}
}