<?php defined('BASEPATH') || exit('No direct script access allowed');

class Payouts_model extends BF_Model {
    public $table_name	= 'payouts';
    protected $table_users  = 'payout_users';
	protected $key			= 'id';
	protected $date_format	= 'datetime';

	protected $log_user 	= true;
	protected $set_created	= true;
	protected $set_modified = true;
	protected $soft_deletes	= true;

	protected $created_field     = 'created_on';
    protected $created_by_field  = 'created_by';
	protected $modified_field    = 'modified_on';
    protected $modified_by_field = 'modified_by';
    protected $deleted_field     = 'deleted';
    protected $deleted_by_field  = 'deleted_by';

	// Customize the operations of the model without recreating the insert,
    // update, etc. methods by adding the method names to act as callbacks here.
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 	    = array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	// For performance reasons, you may require your model to NOT return the id
	// of the last inserted row as it is a bit of a slow method. This is
    // primarily helpful when running big loops over data.
	protected $return_insert_id = true;

	// The default type for returned row data.
	protected $return_type = 'object';

	// Items that are always removed from data prior to inserts or updates.
	protected $protected_attributes = array();

	// You may need to move certain rules (like required) into the
	// $insert_validation_rules array and out of the standard validation array.
	// That way it is only required during inserts, not updates which may only
	// be updating a portion of the data.
	protected $validation_rules 		= array(
		array(
			"field"		=> "paid_on",
			"label"		=> "lang:paid_on",
			"rules"		=> "trim|required"
		),
		array(
			"field"		=> "transaction_id",
			"label"		=> "lang:transaction_id",
			"rules"		=> "trim|required"
		),
		array(
			"field"		=> "comments",
			"label"		=> "lang:label_comments",
			"rules"		=> "trim"
		)
	);
	protected $insert_validation_rules  = array();
	protected $skip_validation 			= false;
    public $applicable_modules          = array('products', 'bookings');

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
    }
    
    /**
     * Payouts Fields
     *
     * @return array
     */
    public function payouts() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'selectize',
                'name' => 'user_id',
                'label' => 'bf_user',
                'placeholder' => 'placeholder_selectize',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => true,
                'options' => 'load:users;module:users;max_items:1',
                'options_detailed' => 1,
                'rules' => 'trim|required',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'module_id',
                'label' => 'label_module',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'amount',
                'label' => 'label_amount',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'request_number',
                'label' => 'label_request_number',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim|max_length[30]',
            ),
            array(
                'type' => 'text',
                'name' => 'requested_on',
                'label' => 'label_requested_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'paid_on',
                'label' => 'paid_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'payment_method',
                'label' => 'payment_method',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|max_length[150]',
            ),
            array(
                'type' => 'text',
                'name' => 'transaction_id',
                'label' => 'transaction_id',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|max_length[255]',
            ),
            array(
                'type' => 'text',
                'name' => 'comments',
                'label' => 'label_comments',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'notes',
                'label' => 'label_notes',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'status',
                'label' => 'label_status',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_by',
                'label' => 'created_by',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_ip',
                'label' => 'created_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_by',
                'label' => 'modified_by',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_ip',
                'label' => 'modified_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'deleted_by',
                'label' => 'deleted_by',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            
        );
        return $fields;
    }
    
    /**
     * Payout Users Fields
     *
     * @return array
     */
    public function users() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'selectize',
                'name' => 'user_id',
                'label' => 'bf_user',
                'placeholder' => 'placeholder_selectize',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => true,
                'options' => 'load:users;module:users;max_items:1',
                'options_detailed' => 1,
                'rules' => 'trim|required',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'module_id',
                'label' => 'label_module',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'gross_earning',
                'label' => 'label_gross_earning',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'pending_payout',
                'label' => 'payout_pending',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'requested_on',
                'label' => 'payout_requested_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'status',
                'label' => 'label_status',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_by',
                'label' => 'created_by',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_ip',
                'label' => 'created_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_by',
                'label' => 'modified_by',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_ip',
                'label' => 'modified_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'deleted_by',
                'label' => 'label_deleted_by',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            
        );
        return $fields;
    }
    
    /**
     * Payout Passbook Fields
     *
     * @return array
     */
    public function passbook() {
	    $fields = array(
            array(
                'type' => 'id',
            	'name' => 'id',
            	'frontend_index' => true,
            	'admin_index' => true
            ),
            array(
                'type' => 'selectize',
                'name' => 'user_id',
                'label' => 'bf_user',
                'placeholder' => 'placeholder_selectize',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => true,
                'options' => 'load:users;module:users;max_items:1',
                'options_detailed' => 1,
                'rules' => 'trim|required',
            ),
            array(
                'type' => 'text',
                'name' => 'order_id',
                'label' => 'label_order_id',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|max_length[20]',
            ),
            array(
                'type' => 'text',
                'name' => 'order_item_id',
                'label' => 'label_order_item_id',
                'frontend' => false,
                'admin_only' => false,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'module_id',
                'label' => 'label_module',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'opening_balance',
                'label' => 'passbook_opening_balance',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'debit',
                'label' => 'label_debit',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
             array(
                'type' => 'text',
                'name' => 'credit',
                'label' => 'label_credit',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'closing_balance',
                'label' => 'passbook_closing_balance',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'notes',
                'label' => 'label_notes',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'additional_info',
                'label' => 'label_additional_info',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim|max_length[255]',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_ip',
                'label' => 'created_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            )            
         );
         return $fields;
    }
    
    /**
     * Payout Histories Fields
     *
     * @return array
     */
    public function histories() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'selectize',
                'name' => 'user_id',
                'label' => 'bf_user',
                'placeholder' => 'placeholder_selectize',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => true,
                'options' => 'load:users;module:users;max_items:1',
                'options_detailed' => 1,
                'rules' => 'trim|required',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'module_id',
                'label' => 'label_module_id',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'row_id',
                'label' => 'label_row_id',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'order_id',
                'label' => 'label_order_id',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|max_length[20]',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'order_status',
                'label' => 'label_order_status',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'amount',
                'label' => 'label_amount',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'action',
                'label' => 'label_action',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_ip',
                'label' => 'created_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            
        );
        return $fields;
    }
    
    /**
     * Payout Requests Fields
     *
     * @return array
     */
    public function requests() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'selectize',
                'name' => 'user_id',
                'label' => 'bf_user',
                'placeholder' => 'placeholder_selectize',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => true,
                'options' => 'load:users;module:users;max_items:1',
                'options_detailed' => 1,
                'rules' => 'trim|required',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'module_id',
                'label' => 'label_module',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'request_number',
                'label' => 'payout_request_number',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|max_length[50]',
            ),
            array(
                'type' => 'text',
                'name' => 'amount',
                'label' => 'label_amount',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'requested_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'status',
                'label' => 'label_status',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_ip',
                'label' => 'created_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_by',
                'label' => 'created_by',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_ip',
                'label' => 'modified_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_by',
                'label' => 'modified_by',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            
        );
        return $fields;
    }
    
	/**
	 * Email Template Fields
	 *
	 * @return array
	 */
	public function email_templates($role = 'admin') {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'module_table',
	            'label' => 'label_module',
	            'frontend' => false,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'name:gc_menu_payouts;value:payouts',
	            'options_detailed' => 1,
	            'options_lang' => 1,
	            'class' => 3,
	            'rules' => 'trim|unique_email_template[payout]',
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'user_type',
	            'label' => 'bf_user',
	            'frontend' => false,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'name:placeholder_select;value:|name:us_admin;value:admin|name:bf_user;value:user',
	            'options_detailed' => 1,
	            'options_lang' => 1,
	            'class' => 3,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'action_type',
	            'label' => 'label_action',
	            'frontend' => false,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'name:placeholder_select;value:|name:add;value:add|name:edit;value:edit|name:delete;value:delete|name:singular_payout_requests;value:request',
	            'options_detailed' => 1,
	            'options_lang' => 1,
	            'class' => 3,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'name',
	            'label' => 'label_name',
	            'placeholder' => 'placeholder_name',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
	            'rules' => 'required|trim|max_length[50]'
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'subject',
	            'label' => 'label_subject',
	            'placeholder' => 'placeholder_subject',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'required|trim|max_length[255]'
	        ),
	        array(
	            'type' => 'textarea',
	            'name' => 'message',
	            'label' => 'label_message',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'custom_rules' => 'editor',
	            'field_class' => 'editor',
	            'class' => 9,
	            'rules' => 'trim|required',
	        ),
	        array(
	            'type' => 'textarea',
	            'name' => 'sms_text',
	            'label' => 'sms_text',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'class' => 9,
	            'field_extra' => 'rows:3',
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'textarea',
	            'name' => 'params',
	            'label' => 'em_params',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'class' => 9,
	            'tooltip' => lang('em_params_help'),
	            'field_extra' => 'rows:3',
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'status',
	            'label' => 'label_status',
	            'frontend' => false,
	            'admin_only' => false,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_on',
	            'label' => 'created_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_on',
	            'label' => 'modified_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        )
	    );
	    return $fields;
	}
    
    /*----------------------------------------------------*/
    /*	Admin Area Index - All Payouts
     /*----------------------------------------------------*/
    public function get_all_payouts($count, $where, $sort_by, $offset = 0, $limit = 25) {
    	if($count == 0) {
    		$sql = "SELECT P.*, CONCAT(U.display_name, ' (', U.email , ')') AS user";
    	} else {
    		$sql = "SELECT COUNT(DISTINCT(P.id)) AS TotalCount";
    	}
    	$sql .= " FROM ". $this->db->dbprefix ."payouts P";
    	$sql .= " INNER JOIN ". $this->db->dbprefix ."users U ON U.id = P.user_id";
    	$sql .= " LEFT JOIN ". $this->db->dbprefix ."payout_users PU ON PU.user_id = U.id";
    
    	if(!empty($where)) {
    		$sql .= " WHERE $where";
    	}
    		
    	if($count == 0) {
    	    $sql .= " GROUP BY P.id";
    		$sql .= " ORDER BY " . $sort_by;
    		$sql .= " LIMIT " . $offset . ", " . $limit;
    		return $this->db->query ( $sql )->result();
    	} else {
    		$result = $this->db->query ( $sql )->row();
    		return $result ? $result->TotalCount : 0;
    	}
    }
    
    /*----------------------------------------------------*/
    /*	Admin Area Index - All Payout Users
     /*----------------------------------------------------*/
    public function get_all_payout_users($count, $where, $sort_by, $offset = 0, $limit = 25) {
    	if($count == 0) {
    		$sql = "SELECT PU.*, CONCAT(U.display_name, ' (', U.email , ')') AS user";
    	} else {
    		$sql = "SELECT COUNT(DISTINCT(PU.id)) AS TotalCount";
    	}
    	$sql .= " FROM ". $this->db->dbprefix ."payout_users PU";
    	$sql .= " INNER JOIN ". $this->db->dbprefix ."users U ON U.id = PU.user_id";
    
    	if(!empty($where)) {
    		$sql .= " WHERE $where";
    	}
    
    	if($count == 0) {
    	    $sql .= " GROUP BY PU.id";
    		$sql .= " ORDER BY " . $sort_by;
    		$sql .= " LIMIT " . $offset . ", " . $limit;
    		return $this->db->query ( $sql )->result();
    	} else {
    		$result = $this->db->query ( $sql )->row();
    		return $result ? $result->TotalCount : 0;
    	}
    }
    
    /*----------------------------------------------------*/
    /*	Member And Admin Area
     /*----------------------------------------------------*/
    /**
     * listing_id is an important field, user_id is retrieved using this field
     * @param number $count
     * @param number $user_id
     * @param string $module
     * @param string $where
     * @param number $offset
     * @param number $limit
     * @return number
     */
    public function get_payouts($count, $user_id, $module, $where, $sort_by, $offset = 0, $limit = 25) {
    	if($count == 0) {
    		$sql = "SELECT P.*, CONCAT(U.display_name, ' (', U.email , ')') AS user";
    	} else {
    		$sql = "SELECT COUNT(P.id) AS TotalCount";
    	}
    	$sql .= " FROM ". $this->db->dbprefix ."payouts P";
    	$sql .= " INNER JOIN ". $this->db->dbprefix ."users U ON U.id = P.user_id";
    	$sql .= " LEFT JOIN ". $this->db->dbprefix ."payout_users PU ON PU.user_id = P.user_id";
    
    	$sql .= " WHERE " . $where;
    
    	// Admin Area Only
    	if($user_id != 0) {
    		$sql .= " AND P.user_id = " . $user_id;
    		$sql .= " AND P.deleted = 0";
    	}
    		
    	if($count == 0) {
    		$sql .= " ORDER BY " . $sort_by;
    		$sql .= " LIMIT " . $offset . ", " . $limit;
    		return $this->db->query ( $sql )->result();
    	} else {
    		$result = $this->db->query ( $sql )->row();
    		return $result ? $result->TotalCount : 0;
    	}
    }
    
    /**
     * Edit Payout
     * @param int $id
     * @return object
     */
    public function get_payout($id) {
    	$sql = "SELECT P.*, CONCAT(U.display_name, ' (', U.email , ')') AS user";
    	$sql .= " FROM ". $this->db->dbprefix ."payouts P";
    	$sql .= " INNER JOIN ". $this->db->dbprefix ."users U ON U.id = P.user_id";
    	$sql .= " WHERE P.id = " . $id;
    	$result = $this->db->query ( $sql )->row();
    	return $result;    	
    }
    
    /**
     * Change Status
     * @param int $id
     * @param int $status
     * @return boolean
     */
    public function _request_paid($id, $status) {
    	// set pending payout
    	$user = $this->select('user_id, module_id, payment, status')->payouts_model->find($id);
    	if($user && $user->status != $status) {
	    	$this->db->update($this->table_name, array('status' => $status), array('id' => $id));
	    	
	    	if($status == 0) { // Status changed from Paid to Request
				$this->db->where('id', $id)->update($this->table_name, array('paid_on' => NULL));
				if($user) {
					// get pending payout from the payout_users table				
					// pending payout must be merged with existing amount
					$this->db->query("UPDATE ". $this->db->dbprefix . "payout_users SET pending_payout = pending_payout + ". floatval($user->payment) ." WHERE module_id = " . $user->module_id . " AND user_id = " . $user->user_id);
				}
	    	} else { // Status changed from Request to Paid
	    		$this->db->where('id', $id)->update($this->table_name, array('paid_on' => date('Y-m-d H:i:s')));    		
	    		if($user) {
	    			// subtract the amount from the pending payout
	    			$this->db->query("UPDATE ". $this->db->dbprefix . "payout_users SET pending_payout = pending_payout - ". floatval($user->payment) ." WHERE module_id = " . $user->module_id . " AND user_id = " . $user->user_id);
	    		}
	    	}
	    	if ($this->db->affected_rows() != 1) {
	    		return FALSE;
	    	}
	    	return TRUE;
    	} else {
    		return $id;
    	}
    }
    
    /**
     * Update Payout User's Pending Payout - UPDATE STATUS
     * @param object $payout
     * @param int $status
     */
    public function update_payout_user($payout, $status) {
        $payout_user = $this->db->get_where('payout_users', array('module_id' => $payout->module_id, 'user_id' => $payout->user_id))->row();
        if($payout_user) {
            $pending_payout = $payout_user->pending_payout;
            $payout_amount = floatval($payout->payment);
            if($status == 0) {
                $pending_payout = $pending_payout + $payout_amount;
                // Status changed from Paid to Request
                // get pending payout from the payout_users table
                // pending payout must be merged with existing amount
                $this->db->query("UPDATE ". $this->db->dbprefix . "payout_users SET is_requested = 1, pending_payout = ". $pending_payout ." WHERE id = " . $payout_user->id);
            } else { // Status changed from Request to Paid
                // subtract the amount from the pending payout
                if($pending_payout >= $payout_amount) { // The admin has already transferred the amount to wallet
                    $pending_payout = $pending_payout - $payout_amount;
                }
                $this->db->query("UPDATE ". $this->db->dbprefix . "payout_users SET is_requested = 0, pending_payout = ". $pending_payout ." WHERE id = " . $payout_user->id);
            }
        }
    }
    
    /**
     * Payout Request by the User/Admin
     * @param int $user_id
     * @param int $module_id - COUPONS/PRODUCTS/BOOKINGS
     * @return boolean
     */
    public function payout_request($user_id, $module_id, $amount = 0) {
        $payout_request = $this->db->where(array('user_id' => $user_id, 'module_id' => $module_id, 'status' => 0))->count_all_results('payout_requests');
        $min_field_name = 'minimum_payout';
        if($module_id == 2) {
            $min_field_name = 'ecom_min_payout';
        } else if($module_id == 29) {
            $min_field_name = 'bookings_min_payout';
        }
        $min_payout = gc_json_settings('payout.settings', $min_field_name);
        if(!$payout_request && $amount >= $min_payout) {
    		$requested_on = date('Y-m-d H:i:s');
    		$data = array();
    		$data['user_id'] 		= $user_id;
    		$data['module_id'] 		= $module_id;
    		$data['request_number'] = 'PR' . $user_id . '-' . time();
    		$data['amount']		    = $amount;	
    		$data['created_on']		= $requested_on;
    		$data['created_by']		= $this->auth->user_id();
    		$data['created_ip']		= $this->input->ip_address();
    		$this->db->insert('payout_requests', $data);
    		$insert_id = $this->db->insert_id();
            if($insert_id > 0) {
                $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'payout_email_templates', 'module_table' => 'payout_requests', 'action_type' => 'request', 'record' => (object) $data, 'id' => $insert_id));
                return $data['request_number'];
            }
        }
    	return false;
    }
    
    /**
     * Get Payout User Detail
     * @param int $id - 1 -> COUPONS
     * @param int $module_id
     * @return object
     */
    public function get_user_payouts($user_id, $module_id = 0) {
        $db_prefix = $this->db->dbprefix;
        $sql = " SELECT M.*, PR.created_on AS requested_on, PR.request_number, CONCAT(U.display_name, ' (', U.email , ')') AS user, U.profile_image FROM";
        $sql .= " (SELECT OT.seller_id AS user_id, OT.module_id, SUM(payout) AS gross_earning,";
        $sql .= " (SELECT SUM(payout) FROM " . $db_prefix . "order_items WHERE seller_id = OT.seller_id AND module_id = OT.module_id AND payout_id = 0 AND status_id = 2) AS pending_payout";
        $sql .= " FROM " . $db_prefix . "order_items OT";
        $sql .= " WHERE OT.seller_id = " . $user_id . " AND OT.payout != 0 AND OT.status_id = 2";
        if($module_id != 0) {
            $sql .= " AND OT.module_id = ". $module_id;
        }
        $sql .= " GROUP BY OT.module_id) M";
        $sql .= " INNER JOIN ". $this->db->dbprefix ."users U ON U.id = M.user_id";
        $sql .= " LEFT JOIN ". $this->db->dbprefix ."payout_requests PR ON PR.user_id = M.user_id AND PR.module_id = M.module_id AND status = 0";
    	if($module_id != 0) {
    	    $result = $this->db->query($sql)->row();
    	} else {    	    
    	    $result = $this->db->query($sql)->result();
    	}
    	return $result;
    }

	/**
	 * Update User's Payout
	 * @param number $user_id
	 * @param number $module_id
	 * @param number $amount
	 * @param number $order_status
	 * @param boolean $add_payout_user - Add Payout User if not exist
	 */
    public function update_payout($order_item) {
	    $user_id = $order_item->seller_id;
	    $module_id = $order_item->module_id;
	    $module = gc_module_name($module_id);
	    if($module != 'bookings' && $module != 'products' && $module != 'ecart') {
	        return false;
	    }
	    $calculation_amount = $order_item->subtotal - $order_item->discount;
	    
	    // deduct all commissions on this item
	    $order_item_commission = $this->db->select('SUM(amount) AS commission')->where('order_item_id', $order_item->id)->get('commissions')->row();
	    if($order_item_commission) {
	        $calculation_amount = $calculation_amount - $order_item_commission->commission;
	        if($calculation_amount <= 0) {
	            $calculation_amount = 0;
	        }
	    }
	    $amount = $calculation_amount;
	    $transaction_type = 'debit';
	    if($order_item->status_id == 2) {
	        $transaction_type = 'credit';
	    }
	    
	    // update passbook
	    $this->gc->update_passbook($user_id, 'order_items', $order_item->id, $amount, $transaction_type);

	    // update payouts in the order_items table
	    $this->db->where('id', $order_item->id)->update('order_items', array('payout' => ($transaction_type == 'debit' ? 0 : $amount), 'modified_on' => date('Y-m-d'), 'modified_ip' => $this->input->ip_address(), 'modified_by' => $this->auth->user_id()));
	    
        // if the order_item is being processed for the first time with not confirmed status,
        // do not save history to avoid negative payout
        $save_history = true; 
        if($order_item->status_id != 2) {
            $history = $this->db->where('order_item_id', $order_item->id)->get('payout_passbook')->row();
            if(!$history) {
                $save_history = false;
            }
        }

	    // save payouts history
        if($save_history) {
	        $this->save_history($order_item, $amount);
        }
	}
	
	/**
	 * Save Wallets/Points History
	 * @param int $module_id
	 * @param int $user_id
	 * @param float $amount
	 * @param string $type
	 * @return boolean
	 */
	public function save_history($order_item, $amount, $extra = array()) {
	    $table = 'payout_passbook';
        $user_id = $order_item->user_id;
        $module_id = $order_item->module_id;
        $type = $order_item->status_id == 2 ? 'credit' : 'debit';
	    $history = $this->db->where('module_id', $module_id)->order_by('id', 'desc')->get($table)->row();
	    $cash = 0;
	    if($history) {
	        $cash = $history->closing_balance;
	    }
	    $data = array();
	    $data['user_id'] = $user_id;
	    $data['order_id'] = $order_item->order_id;
	    $data['order_item_id'] = $order_item->id;
	    $data['module_id'] = $module_id;
	    $data['opening_balance'] = $cash;
	    if($type == 'debit') {
	        $data['debit'] = $amount;
	        $data['credit'] = 0;
	        $data['closing_balance'] = $cash - $amount;
	    } else if($type == 'credit') {
	        $data['debit'] = 0;
	        $data['credit'] = $amount;
	        $data['closing_balance'] = $cash + $amount;
	    }
	    if(!empty($extra['notes'])) {
	        $data['notes'] = $extra['notes'];
	        unset($extra['notes']);
	    }
	    $data['created_on'] = date('Y-m-d H:i:s');
	    $data['created_ip'] = $this->input->ip_address();
	    $data['additional_info'] = json_encode($extra);
	    $this->db->insert($table, $data);
	    return $this->db->insert_id();
	}
	
	/**
	 * Notify Payout Users on Payout Request
	 * @param int $user_id
	 */
	public function notify_payout_request($user_id, $reference_number) {
		// get email id from the users table
		$user = $this->db->select('display_name,email')->get_where('users', array('id' => $user_id))->row();
		if($user) {
			// Send Email
			$this->load->library('emailer/emailer');
			$data = array(
					'to'		=> $user->email,
					'subject'	=> sprintf(lang('subject_payout_request'), $reference_number),
					'message'	=> sprintf(lang('mail_payout_request'), $user->display_name, $reference_number, settings_item('site.title'), site_url())
			);
			$this->emailer->send($data, FALSE);
		}
	}
	
	/**
	 * Notify Payout Users on Payout Success
	 * @param int $user_id
	 */
	public function notify_payout_success($id) {
		$payout = $this->payouts_model->select('user_id,request_number,requested_on,payment_method,transaction_id,paid_on')->find($id);
		if($payout) {
			// get email id from the users table
			$user = $this->db->select('display_name, email')->get_where('users', array('id' => $payout->user_id))->row();
			if($user) {
				// Send Email
				$this->load->library('emailer/emailer');
				$data = array(
						'to'		=> $user->email,
						'subject'	=> sprintf(lang('subject_payout_success'), $payout->request_number),
						'message'	=> sprintf(lang('mail_payout_success'), $user->display_name, $payout->request_number, $payout->requested_on, $payout->payment_method, $payout->transaction_id, $payout->paid_on, settings_item('site.title'), site_url())
				);
				$this->emailer->send($data, FALSE);
			}
		}
	}
	
	/**
	 * Allow Admin to send an email to the user from the Payout Users page
	 * @param $id - integer
	 */
	public function get_user($id) {
		$sql = "SELECT U.email";
		$sql .= " FROM ". $this->db->dbprefix ."payout_users PU";
		$sql .= " INNER JOIN ". $this->db->dbprefix ."users U ON U.id = PU.user_id";
		$sql .= " WHERE PU.id = " . $id;
		$result = $this->db->query ( $sql )->row();
		return $result;
	}
	
	/**
	 * Transfer pending payout to wallet
	 * 
	 * Note: A user can have multiple payouts (products, bookings, commissions etc.), 
	 * therefore, vice-verca (wallet to payouts) cannot be performed.
	 * 
	 * @param number $id
	 * @param float $amount
	 * @param string $action
	 */
	public function wallet_transfer($id, $amount = 0, $action = 'credit') {
	    $this->lang->load('wallets/wallets');
	    $record = $this->db->where('id', $id)->get('payout_users')->row();
	    if($record) {
	        $this->load->model('wallets/wallets_points_model');
	        if($amount == 0) {
	           $amount = $record->pending_payout; 
	        }
	        $this->wallets_points_model->update_cash('wallets', $record->user_id, 'payouts', $amount, $action);
	        // set pending_payout to zero
	        $this->db->where('id', $id)->update('payout_users', array('pending_payout' => ($record->pending_payout - $amount)));
	        Template::set_message(sprintf(lang('msg_user_wallet'), currency_format($record->pending_payout), lang('label_' . $action . 'ed'), lang('bf_user')), 'success');
	    } else {
	        Template::set_message(lang('msg_error'), 'danger');
	    }
	}
	
	/**
	 * Email/PDF
	 * @param number $payout_id
	 * @param array $record
	 * @return boolean
	 */
	public function email_print($payout_id, $type = 'email', $record = array(), $role = 'admin') {	    
	    $data = $this->invoice_template_params($payout_id);
	    if(!empty($data)) { // affiliate commissions payout
    	    if(isset($_POST['invoice_pdf'])) {    	       
	            $this->gemailer->sms_email_pdf($type, array('template_table' => 'payout_email_templates', 'module_table' => 'payouts', 'action_type' => '', 'record' => $data, 'processed' => true, 'filename' => $filename));
    	    } 
    	    switch($type) {
    	        case 'sms_email':
    	        case 'email':
    	        case 'pdf':
    	            $filename = 'invoice_' . $payout_id . '_' . time();
    	            $this->gemailer->sms_email_pdf($type, array('template_table' => 'payout_email_templates', 'module_table' => 'payouts', 'action_type' => '', 'record' => $data, 'processed' => true, 'filename' => $filename));
    	            break;
    	        case 'view';    	        
    	            $content = $this->gemailer->sms_email_pdf($type, array('template_table' => 'payout_email_templates', 'module_table' => 'payouts', 'action_type' => '', 'record' => $data, 'processed' => true));
        	        $message = $content['message'];
        	        $this->load->library('crypt');
        	        $invoice_payout_id = $this->crypt->encode($data['value_id']);
        	        Template::set(array(
        	            'toolbar_title' => lang('singular_invoices') . ' - ' . $payout_id,
        	            'invoice_content' => $message,
        	            'invoice_payout_id' => $invoice_payout_id,
                        'role' => $role,
        	            'page' => 'invoice'
        	        ));
        	        Template::set_view('payouts/invoice');
        	        if($role == 'admin') {
        	            Template::render();
        	        } else {
                        if($role == 'member') {
                            Template::render('member');
                        } else {
                            // display form
                            Assets::add_js('_affiliates.js');
                            Assets::add_css('_affiliates.css');
                            Template::render($this->affiliates_model->layout);
                        }
        	        }
        	        break;
    	    }
	   }
	}
	
	/**
	 * Payouts Invoice Template Data
	 * @param number $payout_id
	 * @return boolean|array|boolean[]|string[]|NULL[]
	 */
	public function invoice_template_params($payout_id) {
	    $this->lang->load('affiliates/affiliates');
	    $this->lang->load('payouts/payouts');
	    $this->lang->load('orders/orders');
	    $this->lang->load('commissions/commissions');
	    $this->load->model('commissions/commissions_model');
	    $this->load->model('affiliates/affiliates_model');
	    $this->load->model('users/user_model');
	    $data = array();
	    if(empty($record)) {
	        $record = $this->db->where('id', $payout_id)->get('payouts')->row();
	    }
	    if(empty($record)) {
	        return false;
	    }
        $user = null;
        $commission_module_id = gc_module_id('commissions');
        if($record->module_id == $commission_module_id) {
	        $user = $this->affiliates_model->get_affiliate_user($record->user_id, true, 'user_id');
        }
        if(!$user) {
	        // $user = $this->user_model->get_user($record->user_id);
            $user = $this->db->select('*, display_name AS name, id AS user_id')->where('id', $record->user_id)->get('users')->row();
	    }
	    if(!$user) {
	        return false;
	    }	    
	    // payout is not affiliate specific
        $params = json_decode($record->params ?? '');
        $payouts = $this->load->view('affiliates/tpl/_commissions', array('commissions' => (!empty($params->payouts) ? $params->payouts : array())), true);
        $data = array(
            'label_user' => $record->module_id == $commission_module_id ? lang('singular_affiliates') : lang('bf_user'),
            'label_total' => lang('payout_total_amount'),
            'label_paid_on' => lang('paid_on'),
            'value_paid_on' => gc_date_format($record->paid_on),
            'label_amount' => lang('label_amount'),
            'value_amount' => currency_format($record->amount),
            'label_payment_method' => lang('payment_method'),
            'value_payment_method' => $record->payment_method,
            'label_transaction_id' => lang('transaction_id'),
            'value_transaction_id' => $record->transaction_id,
            'label_comments' => lang('label_comments'),
            'value_comments' => $record->comments,
            'value_payouts' => $payouts
        );
        foreach($user as $key => $value) {
            switch($key) {
                case 'affiliate_id':
                case 'profile_user':
                case 'deleted':
                case 'created_by':
                case 'modified_by':
                case 'state_id':
                case 'city_id':
                case 'country_iso':
                case 'locality_id':
                case 'status_id':
                case 'created_ip':
                case 'modified_ip':
                case 'status_color':
                case 'status_lang':
                case 'blogs_count':
                case 'forum_posts':
                case 'is_online':
                case 'referred_by':
                case 'tracking_duration':
                case 'referred_counter':
                case 'locality':
                case 'level_id':
                case 'ref_by':
                case 'payment_method':
                    break;
                case 'meta_data':
                    $meta_data = json_decode($value ?? '');
                    foreach($meta_data as $meta_key => $meta_value) {
                        if(!empty($meta_value)) {
                            $data['label_' . $meta_key] = lang('label_' . $meta_key);
                            $data['value_' . $meta_key] = $meta_value;
                        }
                    }
                    break;
                case 'created_on':
                case 'modified_on':
                    $data['label_' . $key] = lang($key);
                    $data['value_' . $key] = gc_date_format($value);
                    break;
                case 'last_login':
                    $data['label_' . $key] = lang('us_last_login');
                    $data['value_' . $key] = gc_date_format($value);
                    break;
                case 'join_date':
                    $data['label_' . $key] = lang('joined_on');
                    $data['value_' . $key] = gc_date_format($value);
                    break;
                case 'level_id':
                    $data['label_level'] = lang('label_level');
                    $data['value_level_id'] = $value;
                    break;
                case 'payment_info':
                    $data['label_payment_info'] = lang('payment_info');
                    $data['value_level_id'] = $value;
                    break;
                case 'payment_method':
                    $data['label_payment_method'] = lang('label_payment_method');
                    $data['value_payment_method'] = lang($value);
                    break;
                case 'user_id':
                    $data['label_user_id'] = lang('label_id');
                    $data['value_user_id'] = $value;
                    break;
                case 'address':
                    $data['label_address'] = lang('label_address');
                    $data['value_address'] = $value;
                    break;
                case 'user_email':
                    $data['label_email'] = lang('label_email');
                    $data['value_email'] = $value;
                    break;
                case 'username':
                    $data['label_username'] = lang('bf_username');
                    $data['value_username'] = $value;
                    break;
                case 'display_name':
                    $data['label_name'] = lang('bf_username');
                    $data['value_name'] = $value;
                    break;
                case 'status':
                    $data['label_status'] = lang('label_status');
                    $data['value_status'] = empty($user->status_lang) ? lang($user->status) : $user->status;
                    break;
                case 'id':
                    $data['label_' . $key] = lang('label_' . $key);
                    $data['value_' . $key] = $record->id;	                    
                    break;
                default:
                    $data['label_' . $key] = lang('label_' . $key);
                    $data['value_' . $key] = $value;
                    break;
            }
        }
	    return $data;
	}
	
	/**
	 * Display Summary
	 * @return array
	 */
	public function summary() {
	    $sql = "SELECT COUNT(*) AS total_transactions";
	    $sql .= ", (SELECT COUNT(DISTINCT(id)) FROM " . $this->db->dbprefix . $this->table_name . ") AS total_users";
	    
	    // this month payouts
	    $sql .= ", (SELECT SUM(amount) FROM " . $this->db->dbprefix . $this->table_name;
	    $sql .= " WHERE MONTH(created_on) = MONTH(CURDATE()) AND YEAR(created_on)  =  YEAR(CURDATE())) AS month_payouts";
	    
	    // six month payouts
	    // $sql .= ", (SELECT SUM(amount) FROM " . $this->db->dbprefix . $this->table_name;
	    // $sql .= " WHERE MONTH(created_on) = MONTH(CURDATE()) AND YEAR(created_on)  =  YEAR(CURDATE())) AS month_payouts";
	        
	    $sql .= ", (SELECT SUM(amount) FROM " . $this->db->dbprefix . $this->table_name . ") AS total_payouts";
	    $sql .= " FROM " . $this->db->dbprefix . $this->table_name;
	    $result = $this->db->query($sql)->row();
	    foreach($result as $key => $value) {
	        switch($key) {
	            case 'month_payouts':
	            case 'total_payouts':
	                $result->{$key} = currency_format(empty($value) ? 0 : $value);
	                break;
	        }
	    }
	    return $result;
	}
	
	/**
	 * Get the Purchase Info for Email and PDF
	 * @param number $group_id
	 * @param number $purchase_id
	 * @param array $record
	 * @return array
	 */
	public function info($module, $id, $record = array()) {
	    $this->lang->load('payouts/payouts');
	    if(empty($record)) {
	        $this->db->from($module . ' M');
	        $this->db->join('users U', 'U.id = M.user_id', 'INNER');
	        $record = $this->db->select('M.*, U.display_name AS name, U.phone_number, U.email, U.address, U.city')->where('M.id', $id)->get()->row();
	    }
	    if(!$record) {
	        return false;
	    }
	    $record = (array) $record;
	    $data = array(
	        'label_user' => lang('bf_user'),
	        'label_total' => lang('label_total'),
	        'value_payouts' => ''
	    );
	    foreach($record as $key => $value) {
	        switch($key) {
	            case 'created_by':
	            case 'created_ip':
	            case 'modified_ip':
	            case 'modified_by':
	                unset($record[$key]);
	                break;
	            case 'paid_on':
	            case 'created_on':
	            case 'modified_on':
	                $data['label_' . $key] = lang($key);
	                $data['value_' . $key] = gc_date_format($value);
	                break;
	            case 'amount':
	            case 'credit':
	            case 'debit':
	                $data['label_' . $key] = lang('label_' . $key);
	                $data['value_' . $key] = currency_format($value);
	                break;
	            case 'user_id':
	                $data['label_' . $key] = lang('label_id');
	                $data['value_' . $key] = $value;
	                break;
	            default:
	                $data['label_' . $key] = lang('label_' . $key);
	                $data['value_' . $key] = $value;
	        }
	    }
	    return $data;
	}
    
    /**
     * Get Payout users qualifies minimum payout balance
     * 
     * @param number $count
     * @param string $where
     * @param string $sort_by
     * @param number $limit
     * @param number $offset
     * @param array $extra
     * @return array
     */
    public function payout_info($count = 0, $where = null, $sort_by = 'PM.created_on DESC', $limit = 15, $offset = 0, $extra = array()) {
        $module = 'commissions';
        if(!empty($extra['module'])) {
            $module = $extra['module'];
        }
        $payout_user_field = $module == 'commissions' ? 'user_id' : 'seller_id';
        $payout_amount_field = $module == 'commissions' ? 'amount' : 'payout';
        if($count == 1) {
            $sql = "SELECT COUNT(*) AS total_count";
        } else {
            $sql = " SELECT M.*, " . user_fields();
            if($module == 'order_items') {
                $sql .= ", PR.request_number, PR.created_on AS requested_on";
            }
        }
        $sql .= " FROM (SELECT COUNT(PM.id) AS payout_total, PM.created_on, PM." . $payout_user_field . " AS user_id, SUM(PM." . $payout_amount_field . ") AS payout_amount";
        if($module == 'order_items') {
            $sql .= ", PM.module_id";
        }
        $sql .= " FROM " . $this->db->dbprefix . $module . " PM";
        $sql .= " WHERE PM.status_id = 2"; // Confirmed Payments Only
        $sql .= " AND PM.payout_id = 0"; // not yet paid
        if(!empty($where)) {
            $sql .= " AND " . $where;
        }
        $sql .= " GROUP BY PM." . $payout_user_field;
        $order_sort_by = $sort_by;
        if($module == 'order_items' && strpos($sort_by, 'PM.amount') !== false) {
            $order_sort_by = str_replace('PM.amount', 'PM.payout', $sort_by);
        }
        $sql .= " ORDER BY " . $order_sort_by;
        $sql .= " ) M";
        $sql .= " INNER JOIN " . $this->db->dbprefix . "users U ON U.id = M.user_id";       
        if($module == 'order_items') {
            $sql .= " LEFT JOIN " . $this->db->dbprefix . "payout_requests PR ON PR.user_id = M.user_id AND PR.module_id = M.module_id AND PR.status = 0";
        }
        $sql .= " WHERE payout_amount > " . $extra['minimum_payout'];
        if ($count == 0) {
            $sql .= " GROUP BY M.user_id";
            if(strpos($sort_by, 'PM.') !== false) {
                if(strpos($sort_by, 'PM.amount') !== false) {
                    $sort_by = str_replace('PM.amount', 'M.payout_amount', $sort_by);
                }
                $sort_by = str_replace('PM.', 'M.', $sort_by);
            }
            $sql .= " ORDER BY " . $sort_by;
            if ($limit != 0) {
                $sql .= " LIMIT " . $offset . ", " . $limit;
            } else {
                $sql .= " LIMIT $limit";
            }
            $query = $this->db->query($sql);
            return $query->result();
        } else {
            $result = $this->db->query($sql)->row();
            return $result ? $result->total_count : 0;
        }
        $result = $this->db->query($sql)->result();
        return $result;
    }
}