<?php defined('BASEPATH') || exit('No direct script access allowed');

/**
 * Financial controller
 * View files are in the application/view/wallets_points folder
 */
class Financial extends Admin_Controller {

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
        
        $this->auth->restrict('Points.Financial.View');
        $this->load->model('wallets/wallets_points_model');
        $this->lang->load('wallets/wallets');
        $this->form_validation->set_error_delimiters("<span class='error'>", "</span>");        
        Template::set_block('sub_nav', 'financial/_sub_nav');
    }
    /**
     * Index Method
     * @param number $filter
     */
    public function index($filter = 0) {
    	$this->wallets_points_model->index('points', $filter);
    }
    
    /**
     * Add Wallet
     */
    public function create() {
    	$this->wallets_points_model->create('points');
    }
    
    /**
     * Edit Wallet
     */
    public function edit() {
    	$id = $this->uri->segment(5);
    	$this->wallets_points_model->edit($id, 'points');
    }
    
    /**
     * View Wallet History
     * @param number $filter
     */
    public function history($filter = 0) {
    	$this->wallets_points_model->history('points', $filter);
    }
    
    /**
     * Save settings
     */
    public function settings() {
		clear_cache('site_settings'); // Clear Site Settings Cache
    	$this->load->model('settings/settings_model');
    	if (isset($_POST['save'])) {
    	    $this->gcrud->stats(array('module' => 'points'));
    		$this->form_validation->set_rules('points_conversion', 'lang:points_conversion', 'trim|required|is_decimal');
    		$this->form_validation->set_rules('minimum_points', 'lang:minimum_points', 'trim|required|is_numeric');
    		$this->form_validation->set_rules('modules[]', 'lang:applicable_modules', 'trim|required|is_numeric|is_array');
    		if($this->form_validation->run() == TRUE) {
    			$settings = array(
    				'points_conversion' => $this->input->post('points_conversion'),
    				'minimum_points' => $this->input->post('minimum_points'),
    				'modules' => $this->input->post('modules')					
    			);
    			$data = array (array('name' => 'user.points', 'value' => json_encode($settings)));
    			$updated = $this->settings_model->update_batch($data, 'name');
    			if ($updated) {
    				Template::set_message(sprintf(lang('msg_save_success'), lang('gc_menu_settings')), 'success');
    				Template::redirect(site_url(SITE_AREA . '/financial/points/settings'));
    			} else {
    				Template::set_message(sprintf(lang('msg_edit_failure'), lang('gc_menu_settings')), 'error');
    			}
    		}
    	}
    	$setting_name = 'user.points';
    	$result = $this->db->select('value')->get_where('settings', array('name' => $setting_name))->row();
    	if($result) {
    	    $settings = json_decode($result->value ?? '');
    	} else {
    	    $settings = json_decode(settings_item($setting_name) ?? '');
    	}
    	Template::set('settings', $settings);
    	Template::set('toolbar_title', lang('gc_menu_settings'));
    	Template::render();
    }
    
    /**
     * Deduct amount from users wallets and convert in points
     */
    public function wallet_to_points() {
    	Assets::add_css('selectize.bootstrap3.css');
    	Assets::add_js('selectize.min.js');
    	Assets::add_js('selectize-search.js');
    	if (isset($_POST['save'])) {
    	    $this->gcrud->stats(array('module' => 'points'));
    		$this->form_validation->set_rules('user_id', 'lang:label_user', 'trim|required');
    		$this->form_validation->set_rules('cash', 'lang:label_cash', 'trim|required|is_decimal');
    		if($this->form_validation->run() == TRUE) {
    			$user_id = $this->input->post('user_id');
    			$amount = $this->input->post('cash');
    			if($this->wallets_points_model->wallet_to_points($amount, $user_id)) {
    				Template::set_message(sprintf(lang('msg_action_success'), lang('label_points'), lang('label_updated')), 'success');
    				Template::redirect(site_url(SITE_AREA . '/financial/points/wallet_to_points'));
    			} else {
    				Template::set_message(lang('msg_error_conversion'), 'error');
    			}
    		}
    	}
    	Template::set('module', 'points');
    	Template::set('toolbar_title', lang('wallet_to_points'));
    	Template::set_view('wallets_points/form');
    	Template::render();
    }
}