<?php defined('BASEPATH') || exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Online Software Solutions							   # ||
|| # Copyright (C) 2021 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Content controller
 */
class Content extends Admin_Controller {
    
    protected $permissionCreate = 'Posts.Content.Create';
    protected $permissionDelete = 'Posts.Content.Delete';
    protected $permissionEdit   = 'Posts.Content.Edit';
    protected $permissionView   = 'Posts.Content.View';

    protected $context			 = 'content';
    protected $module			 = 'posts';
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
        $this->load->model('posts/posts_model');
        $this->auth->restrict($this->permissionView);
        $this->lang->load('posts');
        $this->lang->load('fields/fields');
        
      	$this->form_validation->set_error_delimiters("<span class='error'>", "</span>");
        
        Template::set_block('sub_nav', 'content/_sub_nav');        
        Assets::add_module_js('posts', 'posts.js');
    }
    
    /**
     * Display a list of Posts data.
     *
     * @return void
     */
    public function index($offset = 0) {
        $this->manage($this->module, 'all', $offset);
    }    
    
	/**
     * Display a list of Posts data.
     *
     * @return void
     */
	public function manage($module, $filter = 'all', $offset = 0) {
        $params = array(
            'model' => $this->module,
            'context' => $this->context,
            'fields' => $module
        );
        if($module != 'posts') {
            $params['prefix'] = 'post';
        } else {
            $params['table_prefix'] = 'post';
		}
        $this->gc->module_view($module, $filter, $offset, 'admin', $params);
	}
    
	/**
     * Add/Edit Posts Data
     *
     * @return void
     */
    public function form($module, $action = 'create') {
		if($module == 'posts') {
			$params = array(
				'cancel_url' => $this->session->userdata($module . '_last_visited_page')
			);
			$this->posts_model->form($module, (int) $this->uri->segment(7), 'admin', $params);
		} else {
			$params = array(
				'model' => $this->module,
				'context' => $this->context,
				'module_name' => $this->module,
				'fields_method' => $module
			);
			if($module != 'posts') {
				$params['prefix'] = 'post';
			}
			$this->gcrud->form($module, $action, 'admin', $params);
		}
    }
	
	/**
	 * Assign fields to a Category
	 * @param number $id
	 * @param string $module
	 */
	public function assign($id) {
	    $this->load->model('fields/fields_model');
	    $this->fields_model->assign($id, 'post_types');
	}
}