<?php defined('BASEPATH') || exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Online Software Solutions							   # ||
|| # Copyright (C) 2021 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Posts controller
 */
class Posts extends Front_Controller {
    
    protected $context			 = 'posts';
    protected $module			 = 'posts';
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
        $this->lang->load('posts');
        $this->load->model('posts/posts_model');
    }
    
    /**
     * Display Posts
     * @param string $module
     */
    public function view() {
        $this->lang->load('posts/posts');
        $parent_id = (int) $this->input->get('parent_id');
        $user_id = (int) $this->input->get('user_id');
        $post_type = (int) $this->input->get('post_type');
        $offset = (int) $this->input->get('per_page');
        $limit = $this->limit;
        $module = 'posts';
        $where = 'M.status_id = 2 AND M.deleted = 0';
        if(!empty($parent_id)) {
            $where .= ' AND FIND_IN_SET(' . $parent_id . ', M.parent_ids)';
        }
        if(!empty($user_id)) {
            $where .= ' AND FIND_IN_SET(' . $user_id . ', M.user_id)';
        }
        if(!empty($post_type)) {
            $where .= " AND M.post_type = '" . $post_type . "'";
        }
        $total_rows = $this->gcrud->get_all_crud(1, $where, 'M.id DESC', $limit, $offset, array('module' => $module, 'prefix' => ''));
        $records = $this->gcrud->get_all_crud(0, $where, 'M.id DESC', $limit, $offset, array('module' => $module, 'prefix' => ''));
        
        // Pagination
        $this->load->library('pagination');
        $this->config->load('pagination');
        $config['base_url'] = gc_url_format('posts');
        $config['total_rows'] = $total_rows;
        $config['per_page'] = $limit;
        // Pagination using GET Method
        $config['reuse_query_string'] = TRUE;
        $config['enable_query_strings'] = TRUE;
        $config['page_query_string'] = TRUE;
        
        $this->pagination->initialize($config);
        $pagination = $this->pagination->create_links();
        Template::set(array(
            'module' => $module,
            'pagination' => $pagination,
            'records' => $records
        ));
        $layout = $this->gc->load_assets('modules/posts/view');
        Template::set_view($layout['view']);
        Template::render($layout['layout']);
    }    
    
    /**
     * Show Post to the User
     * @param string $slug
     */
    public function detail($slug) {
        $slug = escape($slug);
        $record = $this->db->where('slug', $slug)->get('posts')->row();
        if(!$record || empty($record->post_type)) {
            Template::redirect(site_url());
        }
        $metas = json_decode($record->metas ?? '', true);
        if(!empty($metas)) {
            foreach($metas as $mkey => $mvalue) {
                $record->{'meta_' . str_replace('-', '_', $mkey)} = $mvalue;
            }
        }
        Template::set(array(
            'record' => $record,
            'module' => 'posts'
        ));
        $layout = $this->gc->load_assets('modules/posts/detail');
        Template::set_view($layout['view']);
        Template::render($layout['layout']);
    }
}