<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
 || ##################################################################### ||
 || # GoClixy - Online Software Solutions							   # ||
 || # Copyright (C) 2021 GoClixy. All Rights Reserved.				   # ||
 || # ----------------------------------------------------------------  # ||
 || # This file is owned and copyrighted by GoClixy.com. Your license   # ||
 || # confers  no  title  or ownership in the Script and is not a sale  # ||
 || # of any rights to the Software. This file may not be redistributed # ||
 || # in whole or significant part.									   # ||
 || # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
 || # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
 || ##################################################################### ||
 \*=======================================================================*/
class Posts_model extends BF_Model {
	/**
	 * E.g. Module - Question and Answers
	 * Questions will have no parent
	 * Answers will have question as the parent id
     * Question Answers belongs to which tables relations
     * Question Answer fields through the Admin Panel via Post Types
	 * Frontend View and Detail View
	 * Testimonials will have group_id
	 * Video Trainings etc.
	 * Grab all fields including metas
	 * The module will be like subscribers where groups has their own fields.
	 **/
    
    public $table_name	    = "posts";
    protected $table_metas  = "post_metas";
    protected $key			= "id";
    protected $soft_deletes	= false;
    protected $date_format	= "datetime";
    
    protected $log_user 	= FALSE;
    
    protected $set_created	= TRUE;
    protected $set_modified = TRUE;
    protected $created_field = "created_on";
    protected $modified_field = "modified_on";
    
    /*
     Customize the operations of the model without recreating the insert, update,
     etc methods by adding the method names to act as callbacks here.
     */
    protected $before_insert 	= array();
    protected $after_insert 	= array();
    protected $before_update 	= array();
    protected $after_update 	= array();
    protected $before_find 		= array();
    protected $after_find 		= array();
    protected $before_delete 	= array();
    protected $after_delete 	= array();
    
    /*
     For performance reasons, you may require your model to NOT return the
     id of the last inserted row as it is a bit of a slow method. This is
     primarily helpful when running big loops over data.
     */
    protected $return_insert_id 	= TRUE;
    
    // The default type of element data is returned as.
    protected $return_type 			= "object";
    
    // Items that are always removed from data arrays prior to
    // any inserts or updates.
    protected $protected_attributes = array();
    
    /*
     You may need to move certain rules (like required) into the
     $insert_validation_rules array and out of the standard validation array.
     That way it is only required during inserts, not updates which may only
     be updating a portion of the data.
     */
    protected $validation_rules = array();
    protected $insert_validation_rules 	= array();
    protected $skip_validation 			= FALSE;
    protected $gc_posts_cache;
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
        // Load Zend Caching
        $this->load->library('slug');
        $this->load->library('gzf_cache', array('lifetime' => 900));
        if(method_exists($this->gzf_cache, 'get_instance')) {
            $this->gc_posts_cache = $this->gzf_cache->get_instance();
        }
        $this->load->model('fields/fields_model');
    }
    
    /**
     * Posts Fields
     *
     * @return array
     */
    public function posts() {
	    $fields = array(
            array(
                'type' => 'id',
            	'name' => 'id',
            	'frontend_index' => true,
            	'admin_index' => true
            ),
            array(
                'type' => 'selectize',
                'name' => 'parent_id',
                'label' => 'label_parent',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'load:posts;module:posts;max_items:10',
                'options_detailed' => 1,
                'rules' => 'trim|max_length[455]',
            ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'type_id',
	            'label' => 'post_type',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'name:post_types;value:database',
	            'options_detailed' => 1,
	            'class' => 3,
	            'rules' => 'trim|max_length[255]',
	        ),
            array(
                'type' => 'text',
                'name' => 'title',
                'label' => 'label_title',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|required|max_length[100]',
            ),
            array(
                'type' => 'text',
                'name' => 'slug',
                'label' => 'label_slug',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim|max_length[100]',
            ),
            array(
                'type' => 'textarea',
                'name' => 'summary',
                'label' => 'label_summary',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'class' => 6,
                'rules' => 'trim',
            ),
            array(
                'type' => 'textarea',
                'name' => 'description',
                'label' => 'label_description',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'field_class' => 'editor',
                'class' => 9,
                'rules' => 'trim',
            ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'status_id',
	            'label' => 'label_status',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'options' => 'name:post_statuses;value:database',
	            'options_detailed' => 1,
	            'class' => 3,
	            'rules' => 'trim|required',
	        ),
            array(
                'type' => 'none',
                'name' => 'metas',
                'label' => 'label_description',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'priority',
                'label' => 'label_priority',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),

         );
         return $fields;
    }
    
	/**
     * Post Type Fields
     *
     * @return array
     */
    public function types() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'text',
                'name' => 'name',
                'label' => 'label_name',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'edit' => 1,
                'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
                'rules' => 'trim|required|max_length[150]'
            ),
            array(
                'type' => 'checkbox',
                'name' => 'name_lang',
                'label' => 'name_lang',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'slug',
                'label' => 'label_slug',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|max_length[255]'
            ),
            array(
                'type' => 'text',
                'name' => 'message',
                'label' => 'subscriber_success_message',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'tooltip' => lang('subscriber_help_success_message'),
                'edit' => 1,
                'rules' => 'trim|max_length[450]'
            ),
            array(
                'type' => 'checkbox',
                'name' => 'message_lang',
                'label' => 'message_lang',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'iconpicker',
                'name' => 'icon',
                'label' => 'label_icon',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'class' => 3,
                'rules' => 'trim|max_length[255]',
            ),
            array(
                'type' => 'text',
                'name' => 'priority',
                'label' => 'label_priority',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'value' => 0,
                'class' => 2,
                'rules' => 'trim|required|is_numeric',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'status',
                'label' => 'label_status',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            )
        );
        return $fields;
    }
    
    /**
     * Status Fields
     *
     * @return array
     */
    public function statuses() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'text',
                'name' => 'name',
                'label' => 'label_name',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'edit' => 1,
                'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
                'rules' => 'trim|required|max_length[150]'
            ),
            array(
                'type' => 'checkbox',
                'name' => 'name_lang',
                'label' => 'name_lang',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'colorpicker',
                'name' => 'color',
                'label' => 'label_color',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'class' => 2,
                'rules' => 'trim|max_length[255]',
            ),
            array(
                'type' => 'text',
                'name' => 'priority',
                'label' => 'label_priority',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'value' => 0,
                'class' => 2,
                'rules' => 'trim|required|is_numeric',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'status',
                'label' => 'label_status',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            )
        );
        return $fields;
    }
    
    /**
     * Display form using post_types fields form
     */
    public function form($module, $id = 0, $role = 'admin', $params = array()) {
        if($id == 0) {
            $id = (int) $this->input->get('id');
        }
        $form_type = !empty($_GET['form_type']) ? (int) $_GET['form_type'] : 0;
        if(empty($id) && empty($form_type)) {
            if(isset($_POST['save'])) {
                $form_type = (int) $this->input->post('form_type');
				Template::redirect(current_url() . '?form_type=' . $form_type);
            }
			Template::set(array(
				'fields_module' => !empty($params['fields_module']) ? $params['fields_module'] : 'post_types'
			));
            Template::set_view('posts/form');
            if($role != 'admin') {
                $this->gc->demo_custom_css_js('member');
                Template::render('member');
            } else {
                Template::render();
            }
        } else {
            if($id > 0) {
                $row = $this->db->where('id', $id)->get($module)->row();
                if(!$row) {
                    Template::redirect(gc_url_format('members'));
                }
                $form_type = $row->type_id;
            }
            $user_id = $this->auth->user_id();
			$params['form_type'] = $form_type;
            $this->save_meta($module, $role, $id, $user_id, $params);
        }
    }
	
	/**
     * Save posts information in the database
     *
     * @param string $role 		 - admin/member/mobile - For posts validations by setting the session variable
	 * @param string $module 	 - Name of the module, it can be posts/terms and others
     * @param string $row_id 	 - ID of the Post/Term
     * @param string $user_id 	 - Not required, only added to fulfil the fields_validation_rules parameters requirements
     */
    public function save_meta($module, $role, $row_id, $user_id = 0, $params = array()) {
		$fields_module = !empty($params['fields_module']) ? $params['fields_module'] : 'post_types'; // this table has fields assigned
		$table_metas = !empty($params['table_metas']) ? $params['table_metas'] : $this->table_metas;
		$field_relation = !empty($params['field_relation']) ? $params['field_relation'] : 'post_id';
		
    	$module_id = gc_module_id($module);
    	$error_message = sprintf(lang('msg_no_additional_fields'), lang($module));
    	$cancel_url = !empty($params['cancel_url']) ? $params['cancel_url'] : $this->session->userdata('cancel_url');
    	$field_type = ($role == 'admin' ? 'admin_only' : 'frontend');
    	if(!empty($params['form_type'])) {
    	    $form_type = $params['form_type'];
    	    $form_data = $this->gc->get($fields_module, array('where' => 'id = ' . $form_type));
    	    if(!$form_data) {
    	        Template::set_message($error_message, 'danger');
    	        Template::redirect($cancel_url);
    	    }
    	}
		$form_title = !empty($form_data->name_lang) ? lang($form_data->name) : $form_data->name;
    	
    	// Display saved values to the user
    	$record = array();
    	if($row_id > 0) {
        	$records = $this->db->select('meta_key, meta_value')->get_where($table_metas, array($field_relation => $row_id))->result();   	
        	if($records) {
        		foreach($records as $row) {
        		    $record[$row->meta_key] = $row->meta_value;
        		}
        	}
    	}
    	
    	// Load fields    	
    	$fields_form = $this->get_fields($fields_module, $field_type, $form_type);
		$form_fields = $fields_form['fields'];
		
    	$form_fields = $this->fields_model->load_fields($module, array(), $field_type, $record, $row_id, false, $form_fields, array());    	
    	if(empty($form_fields)) {
    		Template::set_message($error_message, 'danger');
    		Template::redirect($cancel_url);
    	}    	
    	$transaction_type = $row_id == 0 ? 'insert' : 'update';
    	$frontend = FALSE;
    	$mobile = FALSE;
    	if ($role == 'members' || $role == 'mobile') {
    	    $frontend = TRUE;
    	    if ($role == 'mobile') {
    	        $mobile = TRUE;
    	    }
    	}    	
    	$module_info = new stdClass();
		$title = lang('add') . ' - ' . $form_title;
    	if($row_id > 0) {
    	    $module_info = $this->db->select('title, metas')->where('id', $row_id)->get($module)->row();
    	    $title = (!empty($module_info->title) ? $module_info->title . ' - ' : '') . lang('edit');
    	}
    	if (isset($_POST['save']) || isset($_POST['save_only'])) {
    		$result = $this->fields_model->fields_validation_rules($form_fields, $module, $role);
    		if($result) {
    			$meta_data = $this->fields_model->field_post_values($form_fields, $user_id, 0, $role);
    			// save meta information
    			if($meta_data) {
    			    $meta_params = array('fields' => $form_fields, 'module_info' => $module_info, 'table_metas' => $table_metas, 'field_relation' => $field_relation, 'form_type' => $form_type);
    			    $row_id = $this->save_meta_for($module, $row_id, $meta_data, $meta_params);
    			}
    			if($role == 'mobile') {
    			    // return data for mobile view
    			} else {
    			    if($row_id && $row_id > 0) {
						$success_message = !empty($form_data->success_message) ? (!empty($form_data->success_message_lang) ? lang($form_data->success_message) : $form_data->success_message) : sprintf(lang ( 'msg_save_success' ), $form_title);
    			        Template::set_message ($success_message, 'success' );
    			        Template::redirect($cancel_url);
    			    } else {
    			        Template::set_message ( sprintf(lang ( 'msg_edit_failure' ), lang('singular_' . $module)), 'danger' );
    			    }
    			}
    		} else {
    		    Template::set_message ( sprintf(lang ( 'msg_edit_failure' ), lang('singular_' . $module)), 'danger' );
    		}
    	}
    	if($role != 'mobile') {
	    	Template::set(array(
	    		'module' => $module,
	    		'cancel_url' => $cancel_url,
	    		'record' => (object) $record,
	    		'toolbar_title' => $title,
	    		'page_title' => $title,
	    	    'clear_image' => lang('bf_clear')
	    	));
	    	Template::set_view('fields/form');
	    	if($role == 'admin') {
	    		Template::render();
	    	} else {
	    		// Append query string to the form
	    		Template::set(array(
	    			'member_area' => true,
	    			'elements_container' => 'div.card.card-body'
	    		));
	    		// This method has no View in the Themes Module, i.e. load layout files manually
	    		$this->gc->demo_custom_css_js('member');
	    		Template::render('member');
	    	}
    	}
    }
    
    /**
     * Format metas in a format that can be parsed in the view files
	 * Especially for the details page
     * @param string $module - posts/terms
	 * @param string $fields_module - post_types/term_types/taxonomy_types
     * @param object $record
     * @param string $field_type
     * @return array
     */
    public function metas_format($module, $fields_module, $record, $field_type = 'frontend_index') {
        $metas = array();
        $form_data = $this->get_fields($fields_module, $field_type, $record->type_id);
        if($form_data['fields']) {
            $form_groups_info = $form_data['groups'];
            $form_fields_info = $form_data['fields'];
            // get groups
            $metas_record = array();
            $metas_record_by_groups = array();
            $record_metas = new stdClass();
            $user_saved_metas = json_decode($record->metas ?? '');
            foreach($user_saved_metas as $user_saved_meta) {
                $record_metas->{$user_saved_meta->meta_key} = $user_saved_meta->meta_value;
            }
            foreach($form_fields_info as $form_field_info) {
                $original_value = '';
                $original_value = !empty($record_metas->{$form_field_info['name']}) ? $record_metas->{$form_field_info['name']} : '';  
                $meta_group_id = !empty($form_field_info['group']) ? $form_field_info['group'] : 0;
                $meta_formatted = array(
                    'group' => !empty($form_field_info['group']) ? $form_field_info['group'] : 0,
                    'priority' => !empty($form_field_info['priority']) ? $form_field_info['priority'] : 0,
                    'field_id' => $form_field_info['id'],
                    'original_value' => $original_value,
                    'name' => $form_field_info['name'],
                    'icon' => $form_field_info['icon'],
                    'label' => !empty($form_field_info['label_lang']) ? lang($form_field_info['label']) : $form_field_info['label'],
                    'value' => $this->fields_index($form_field_info, $record_metas, $module, 'member')
                );
                $metas_record[$form_field_info['name']] = $meta_formatted;
                if(!array_key_exists($meta_group_id, $metas_record_by_groups)) {
                    $metas_record_by_groups[$meta_group_id] = $form_groups_info[$meta_group_id];
                }
                $metas_record_by_groups[$meta_group_id]['fields'][$form_field_info['name']] = $meta_formatted;
            }
            $metas = array('default' => $metas_record, 'groups' => $metas_record_by_groups);
        }
        return $metas;
    }
    
    /**
     * Fields for the form, view and detail page
     * @param string $module
     * @param object $package
     * @param string $field_type
     * @param array $category_ids
     * @return number|multitype:multitype:boolean|array
     */
    private function get_fields($fields_module, $field_type, $form_type_id) {
		$cacheID = $fields_module . '_fields_' . gc_cache_id(array($field_type, $form_type_id));
		$cacheID = str_replace('-', '_', $this->slug->create_slug($cacheID));
		if(!$result = $this->gc_posts_cache->load($cacheID)) {
			$this->lang->load('groups/groups');
			// get fields
			$fields_module_id = gc_module_id($fields_module);
			$form_fields = $this->fields_model->get_fields($fields_module, $field_type, array(), 0, array('fm_module_id' => $fields_module_id, 'fm_row_id' => $form_type_id));
			// get category_fields
			$params = array();
			$fields_module_data = $this->db->where('module_id = ' . $fields_module_id . ' AND row_id = ' . $form_type_id)->get('field_modules')->result();
			if($fields_module_data) {
				foreach($fields_module_data as $field_module_data) {
					$params[$field_module_data->field_id] = json_decode($field_module_data->params ?? '', true);
				}
			}
			foreach($form_fields as $fkey => $fvalue) {
				if(!empty($params[$fvalue['id']][$field_type])) {
					$form_fields[$fkey] = array_merge($fvalue, $params[$fvalue['id']][$field_type]);
				}
				// change fields priority
				if(!array_key_exists('priority', $form_fields[$fkey])) {
					$form_fields[$fkey]['priority'] = 0;
				}
				// remove category id from the group
				if($form_fields[$fkey]['group'] == $form_type_id) {
					$form_fields[$fkey]['group'] = 0;
				}
			}
			uasort($form_fields, "gc_sort_array");
			
			// get groups
			$group_names = array();
			$field_groups = $this->fields_model->field_groups($form_fields, null);
			$group_names[0] = array(
				'id' => 0,
				'icon' => 'fas fa-list',
				'label' => lang('group_main'),
				'priority' => 0,
			);
			if(!empty($field_groups)) {
				foreach($field_groups as $field_group) {
					$group_names[$field_group->id] = array(
						'id' => $field_group->id,
						'icon' => $field_group->icon,
						'label' => $field_group->name_lang == 1 ?  lang('group_' . $field_group->name) : $field_group->name,
						'priority' => $field_group->priority,
					);
				}
			}
			$result = array('groups' => $group_names, 'fields' => $form_fields);
			$this->gc_posts_cache->save($result, $cacheID, array('fields'));
		 }
		 return $result;
    }
	
	/**
     * Saves one or more key/value pairs of additional meta information for a module.
     *
     * @access public
     * @example
     * $data = array(
     *    'location'	=> 'That City, Katmandu',
     *    'interests'	=> 'My interests'
     * );
     * $this->fields_model->save_meta_for('posts', 1, $data);
     *
     * @param string $module 		The name of the module to save the meta for.
     * @param int   $row_id 		The ID of the row to save the meta for.
     * @param array $data    		An array of key/value pairs to save.
     * @param boolean $is_category	To deal with categories meta fields 
     *
     * @return boolean true/false
     */
    public function save_meta_for($module, $row_id = 0, $data = array(), $params = array()) {
		$table_metas = !empty($params['table_metas']) ? $params['table_metas'] : $this->table_metas;
		$field_relation = !empty($params['field_relation']) ? $params['field_relation'] : 'post_id';
		$form_type = !empty($params['form_type']) ? $params['form_type'] : 0;
		
        $db_fields = $this->gc->get_db_fields($module);
    
    	// While this won't give us the results of each
    	// insert, it does give some sort of indiciation about
    	// the success/failure since if one fails, it's likely
    	// others will also.
    	$result = false;
    	$module_id = gc_module_id($module);
    	$insert_data = array();
    	$update_data = array();
    	$insert_module_data = array();
    	foreach ($data as $key => $value) {	
    		// get field id from the fields table to save in the metas table
			$field = $this->fields_model->select('id')->find_by(array('name' => $key));
			if(!$field) {
				return false;
			}		
			if(in_array($key, $db_fields)) {
				$insert_module_data[$key] = $value; 
			}
			$field_id = $field->id;
    		if($row_id > 0) {
        		$where = array($field_relation => $row_id, 'meta_key' => $key);
        		$row = $this->db->where($where)->get($table_metas)->row();
    		} else {
    		    $row = false;
    		}
    
    		$obj = array(
    				$field_relation	=> $row_id,
    				'meta_id'		=> $field_id,
    				'meta_key'		=> $key,
    				'meta_value'	=> $value
    		);
    		if(!$row) {
    		    $insert_data[] = $obj;
    		} else {
    		    $update_data[] = array(
    		        'id' => $row->id,
    		        'meta_value' => $value
    		    );
    		}
    	}//end foreach
    	$is_update = $row_id != 0 ? true : false;
    	if(!empty($insert_data)) {
    	    if($row_id == 0) {
    	        if(!empty($insert_module_data)) {
					$insert_module_data['type_id'] = $form_type;
    	            $this->db->insert($module, $insert_module_data);
    	            $row_id = $this->db->insert_id();
    	            if($row_id > 0) {
    	                foreach($insert_data as $ikey => $ivalue) {
    	                    $insert_data[$ikey][$field_relation] = $row_id;
    	                }
    	            }
    	        }
    	    }
    	    if($row_id > 0) {
    	       $result = $this->db->insert_batch($table_metas, $insert_data);
    	    } else {
    	        $result = false;
    	    }
    	}
    	if(!empty($update_data)) {
    	    if($is_update && !empty($insert_module_data)) {
    	        $update_module_data = array();
    	        foreach($insert_module_data as $mkey => $mvalue) {
    	            $update_module_data[] = array(
    	                'id' => $row_id,
    	                $mkey => $mvalue
    	            );
    	        }
    	        $this->db->update_batch($module, $update_module_data, 'id');
    	    }
    	    $result = $this->db->update_batch($table_metas, $update_data, 'id');
    	    $result = true;
    	}
    	if(!empty($row_id)) {
    	    // handle images
    	    if(!empty($params['fields'])) {
    	        $module_info = $params['module_info'];
    	        $form_fields = $params['fields'];
    	        $meta_data = $data;
        	    // move images to the listings folder
        	    // if an image is removed, unlink it from the images folder
        	    $this->load->model('images/images_model');
        	    if(!empty($module_info->metas)) {     
        	        $saved_metas = array();
        	        $metas_saved = json_decode($module_info->metas ?? '', true);
        	        if(!empty($metas_saved)) {
        	            foreach($metas_saved as $meta_saved) {
        	                $saved_metas[$meta_saved['meta_key']] = $meta_saved['meta_value'];
        	            }
        	        }
        	    }
        	    foreach($form_fields as $field_form) {
        	        if($field_form['type'] == 'image') {
        	            $name_field = $field_form['name'];
        	            if(empty($meta_data[$field_form['name']]) && !empty($module_info->metas)) {
        	                if(!empty($saved_metas[$name_field])) {
        	                    $this->images_model->purge_image(str_replace($module . '/', '', $saved_metas[$name_field]), 'assets/images/' . $module);
        	                }
        	            } else if(!empty($meta_data[$field_form['name']])) {
        	                // new image move it from the temp to the appropriate folder
        	                if(strpos($meta_data[$field_form['name']], 'temp/') !== FALSE) {
        	                    $image_name = str_replace('temp/', '', $meta_data[$field_form['name']]);
        	                    $this->images_model->move_resize($image_name, 'assets/images/' . $module);
        	                    $update_where = array(
        	                        $field_relation => $row_id,
        	                        'meta_key' => $name_field
        	                    );
        	                    $this->db->where($update_where)->update($table_metas, array('meta_value' => $module . '/' . $image_name));
        	                }
        	            }
        	        }
        	    }
    	    }
    	    $metas = $this->db->where(array($field_relation => $row_id))->get($table_metas)->result();
    	    $this->db->where('id', $row_id)->update($module, array('metas' => !empty($metas) ? json_encode($metas) : null));
    	    $result = $row_id;
			clear_cache($module);
    	}
    	return $result;
    }//end save_meta_for()
		
	/**
	 * Get information from a specific table
	 * e.g. $records = $this->posts_model->get($module, array('where' => 'type_id = 1 AND status_id = 2 AND deleted = 0', 'result' => 1));
	 *
	 * @param array $params
	 * @return object
	 */
	public function get($table, $params = array(), $role = 'admin') {
	    $cacheID = $table . '_' . json_encode($params);
	    $cacheID = str_replace('-', '_', $this->slug->create_slug($cacheID));
	    if(!$result = $this->gc_posts_cache->load($cacheID)) {
	        if(!empty($params['select'])) {
	            $this->db->select($params['select']);
	        }
	        if(!empty($params['where'])) {
	            $this->db->where($params['where']);
	        }
	        if(!empty($params['order_by'])) {
	            $this->db->order_by($params['order_by']);
	        }
	        if(!empty($params['result'])) {
	            if(!empty($params['limit'])) {
	                $this->db->limit($params['limit'], !empty($params['offset']) ? $params['offset'] : 0);
	            }
	            $result = $this->db->get($table)->result();
				if($result) {
					if(!empty($params['field_type']) && !empty($params['form_type'])) {
					    $fields = $this->get_fields((!empty($params['fields_module']) ? $params['fields_module'] : $table), $params['field_type'], $params['form_type']);
						foreach($result as $key => $record) {
							$processed_metas = $this->process_metas($fields, $record, $table, $role);
							foreach($processed_metas as $rkey => $rvalue) {
								$result[$key]->{$rkey} = $rvalue;
							}
						}
					}
				}
	        } else {
	            $result = $this->db->get($table)->row();
				if($result) {
					if(!empty($params['field_type']) && !empty($params['form_type'])) {
					    $fields = $this->get_fields((!empty($params['fields_module']) ? $params['fields_module'] : $table), $params['field_type'], $params['form_type']);
						$processed_metas = $this->process_metas($fields, $result, $module, $role);
						foreach($processed_metas as $rkey => $rvalue) {
							$result->{$rkey} = $rvalue;
						}
					}
				}
	        }	        
	        $this->gc_posts_cache->save($result, $cacheID, array($table));
	    }
	    return $result;
	}
	
	/**
	 * Process Meta Information
	 * @param array $fields
	 * @param object $record
	 * @param string $module
	 * @param string $role
	 */
	private function process_metas($fields, $record, $module, $role) {
		$result = new stdClass();
		$metas = json_decode($record->metas ?? '', true);
		$meta_info = new stdClass();
		foreach($metas as $meta) {
			switch($meta['meta_key']) {
				case 'parent_id':
				case 'title':
					break;
				default:
					$meta_info->{$meta['meta_key']} = $meta['meta_value'];
			}
		}
		$fields = $fields['fields'];
		foreach($fields as $field) {
			if (isset($meta_info->{$field['name']})) {
				$result->{$field['name']} = $this->fields_model->fields_index($field, $meta_info, $module, $role);
			}
		}
		return $result;
	}
}