<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * projects controller
 */
class Content extends Admin_Controller {
	protected $permissionView   = 'Projects.Content.View';
	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct() {
		parent::__construct();

		$this->auth->restrict($this->permissionView);
		$this->load->model('projects_model', null, true);
		$this->lang->load('projects');
		$this->lang->load('listings/listings');
		Template::set_block('sub_nav', 'content/_sub_nav');
	}
	
	/**
	 * Manage Projects
	 * @param string $filter
	 * @param number $offset
	 */
	public function index($filter = 'all', $offset = 0) {
	    $this->manage('projects', $filter = 'all', $offset = 0);
	}
	
	/**
	 * Manage Projects
	 * @param string $module
	 * @param string $filter
	 * @param number $offset
	 */
	public function manage($module = 'projects', $filter = 'all', $offset = 0) {
	    if(isset($_POST['details_regenerate'])) {
	        $this->projects_model->random_description((int) $_POST['details_regenerate']);
	        Template::set_message(lang('msg_success'), 'success');
	    }
	    if(isset($_POST['regenerate_details_all'])) {
	        $this->projects_model->regenerate_details_all();
	        Template::set_message(lang('msg_success'), 'success');	        
	    }
	    if($module == 'details') {
	        Template::set('extra_action', '<button type="submit" name="regenerate_details_all" class="btn btn-danger btn-sm ml-1"><i class="fas fa-sync fa-fw mr-1"></i>' . lang('project_details_regenerate_all') . '</button>');
	    }
	    $extra = array(
	        'model' => 'projects',
	        'context' => 'content',
	        'fields' => null
	    );
	    if($module != 'projects') {
	        $extra['prefix'] = 'project';
	        $extra['fields'] = $module;
	        $extra['search_fields'] = array('name');
	    }
	    $extra['actions'] = array( // Extra buttons
	        array('type' => 'link', 'link' => site_url(SITE_AREA . '/content/projects/form/' . $module . '/edit/%s'), 'label' => '<i class="fas fa-edit fa-fw mr-1"></i>' . lang('edit'), 'class' => 'btn btn-xs btn-primary'),
	    );
	    $this->gc->module_view($module, $filter, $offset, 'admin', $extra);
	}
	
	/**
	 * Add/Edit Projects Data
	 *
	 * @param string $module
	 * @param string $action
	 *
	 * @return void
	 */
	public function form($module = 'projects', $action = 'create') {
	    $params = array(
	        'model' => 'projects',
	        'context' => 'content',
	        'redirect_url' => site_url(SITE_AREA . '/content/projects/manage/' . $module),
	        'dynamic_fields' => true
	    );
	    if($module != 'projects') {
	        $params['prefix'] = 'project';
	        $params['fields_method'] = $module;
	        unset($params['dynamic_fields']);
	        if($module == 'names') {	            
	            Assets::add_js(array(
	                'tinymce/tinymce.min.js',
	            ));
	        }
	    } else {	        
	        clear_cache('projects');
	        Assets::add_js(array(
	            'tinymce/tinymce.min.js',
	        ));
	        $params['extra_fields'] = array('multi_language' => 1);
	        $params['method'] = 'after_project_save';	        
	        $custom_fields = array(
	            array(
	                'name' => 'key',
	                'type' => 'text',
	                'label' => lang('label_key')
	            ),
	            array(
	                'name' => 'value',
	                'type' => 'text',
	                'label' => lang('label_value')
	            )
	        );
	        Template::set('custom_fields', $custom_fields);
	    }
	    $this->gcrud->form($module, $action, 'admin', $params);
	}
	
	/*----------------------------------------------------*/
	/*	Upload Logo File
	 /*----------------------------------------------------*/
	public function logo($id, $module = 'projects') {
	    $id = intval($id);
	    $this->gc->upload_logo_cover('admin', $id, $module, 'logo');
	}
	
	
	/*----------------------------------------------------*/
	/*	Upload a Cover Image
	 /*----------------------------------------------------*/
	public function cover($id, $module = 'projects') {
	    $id = intval($id);
	    $this->gc->upload_logo_cover('admin', $id, $module, 'cover');
	}
	
	/*----------------------------------------------------*/
	/*	Upload Photos Module
	 /*----------------------------------------------------*/
	
	/**
	 * Image_CRUD 0.6 is used to upload images
	 */
	public function photos($module, $row_id) {
	    $this->load->model('images/images_model');
	    if($module == 'projects') {
	        $redirect_url = site_url(SITE_AREA . '/content/projects');
	    } else {
	        $redirect_url = site_url(SITE_AREA . '/content/projects/view/' . $module);
	        if($this->session->userdata('navigation_project_id')) {
	            $redirect_url .= '/' . $this->session->userdata('navigation_project_id');
	        }
	    }
	    Template::set('cancel_url', $redirect_url);
	    $this->images_model->upload_photos_admin($module, $row_id, $redirect_url);
	}
	
	/**
	 * Display Videos to the Admin
	 *
	 * We can have maximum 4 parameters in the URI Segment of a URL
	 * i.e. a new function/method is used instead of using the existing VIEW method
	 *
	 * @param string $filter
	 * @param string $toggle_module
	 * @param number $toggle_row_id
	 * @param number $offset
	 */
	public function videos($toggle_module = null, $toggle_row_id = 0, $filter = 'all', $offset = 0) {
	    //LOAD MODAL FOR YOUTUBE VIDEO PLAYER
	    Assets::add_js('js/bootstrap.youtubepopup.js');
	    Assets::add_js('js/bootstrap.youtubepopup.min.js');
	    if($toggle_module == 'all') {
	        $this->session->unset_userdata('toggle_module', $toggle_module);
	        $this->session->unset_userdata('navigation_property_id');
	    } else {
	        $this->session->set_userdata('toggle_module', $toggle_module);
	        $this->session->set_userdata('navigation_property_id', $toggle_row_id);
	    }
	    Template::set('toggle_row_id', $toggle_row_id);
	    Template::set('toggle_module', $toggle_module); // Sub Navigation Bar
	    // Allow Admin to Clone A Row
	    $this->clone_row('videos');
	    $this->gc->view_admin('videos', 0, $filter, $toggle_module, $toggle_row_id, $offset, $this->limit);
	}
	
	/**
	 * Allow user to upload documents
	 * @param string $module
	 * @param integer $row_id
	 */
	public function documents($module, $row_id) {
	    $this->load->library('goclixy/gfile');
	    $this->gfile->upload_files($module, $row_id);
	    Template::render();
	}
}