<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class Projects extends Front_Controller {

	public function __construct() {
		parent::__construct();

		$this->load->library('form_validation');
		$this->load->model('projects_model', null, true);
		$this->load->library('parser');
		$this->lang->load('projects');
		//Assets::add_module_js('projects', 'projects.js');
	}
	
	/**
	 * Displays a list of all projects.
	 *
	 * @return void
	 */
	public function index() {
	    $module = 'projects';
	    // $this->projects_model->limit($this->limit, $offset);
	    // $records = $this->projects_model->find_all();
	    $module = 'projects';
	    $where = array();
	    $sort_by = 'M.id DESC';
	    $limit = $this->limit;
	    $offset = intval($this->input->get('per_page'));
	    $search_form = $this->projects_model->search_form();
	    $params = $search_form['params'];
	    if(!empty($params)) {
	        $table_fields = $this->gc->get_db_fields($module);
	        foreach($params as $field => $value) {
	            if(!empty($value)) {
    	            $value = escape($value);
    	            if(in_array($field, $table_fields)) {
    	                if(is_array($value)) {
    	                    if(count($value) > 1) {
    	                        $where[] = "(M." . $field . " BETWEEN '" . $value[0] . "' AND '" . $value[1] . "')";
    	                    }
    	                } else {
    	                    if(is_numeric($value)) {
    	                        $where[] = "M." . $field . " = " . $value;
    	                    } else {
    	                        $where[] = "M." . $field . " = '" . $value . "'";
    	                    }
    	                }
    	            }
	            }
	        }
	    }
	    $where = implode(' AND ', $where);
	    $records = $this->gc->get_all_crud(0, $where, $sort_by, $limit, $offset, array('module' => $module, 'prefix' => '', 'cache' => $module));
	    $records = $this->layouts->widget_modules_view($module, $records, array());
	    $total_rows = $this->gc->get_all_crud(1, $where, $sort_by, $limit, $offset, array('module' => $module, 'prefix' => '', 'cache' => $module));
	    
	    // Pagination
	    $this->load->library('pagination');
	    $pager['base_url']   = gc_url_format('projects');
	    $pager['total_rows'] = $total_rows;
	    $pager['per_page'] = $limit;
	    // Pagination using GET Method
	    $pager['reuse_query_string'] = TRUE;
	    $pager['enable_query_strings'] = TRUE;
	    $pager['page_query_string'] = TRUE;
	    
	    $this->pagination->initialize($pager);
	    $pagination = $this->pagination->create_links();
	    
	    Template::set(array(
	        'module' => $module,
	        'pagination' => $pagination,
	        'records' => $records	        
	    ));
	    Template::set('toolbar_title', lang('manage_projects'));
	    $layout = $this->gc->load_assets('modules/projects/view');
	    Template::set_view($layout['view']);
	    Template::render($layout['layout']);
	}

	/**
	 * Display projects information to the user
	 * @param string $slug
	 */
	public function detail($location = null, $slug = null) {	
		if($slug == NULL)	{
			redirect(site_url());
		} else {
		    $slug = escape($slug);
		    $records = $this->gc->get_all_crud(0, "M.slug = '" . $slug ."'" , 'M.id ASC', 1, 0, array('module' => 'projects', 'prefix' => '', 'cache' => 'projects'));
		    if(!$records) {
		        redirect(site_url());
		    }
		    $record = $records[0];
			
			// get translations if exists
			$record = $this->translations->get_translations('projects', $record->id, $record, 1);
			$project_calculations = $this->projects_model->calculate_total($record);		
			$record->project_costs = $project_calculations['project_costs'];
			$record->project_total = $project_calculations['total'];
			$colors = $project_calculations['colors'];
			$costs = $project_calculations['costs'];
			Assets::add_js('https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.5.0/Chart.js');
			Assets::add_js("
var colors = " . json_encode($colors) . ";
var projectCostsOptions = {
  cutoutPercentage: 85, 
  legend: {display: false} // position:'bottom', padding:5, labels: {pointStyle:'circle', usePointStyle:true}
};
var projectCostsData = {
    labels: " . json_encode($costs['names']) . ",
    datasets: [
      {
        backgroundColor: colors.slice(0," . count($costs['names']) . "),
        borderWidth: 0,
        data: " . json_encode($costs['values']) . "
      }
    ]
};
var projectCosts = document.getElementById('projectCosts');
if (projectCosts) {
  new Chart(projectCosts, {
      type: 'pie',
      data: projectCostsData,
      options: projectCostsOptions
  });
}
", 'inline');
			
			// get and load all images			
			Assets::add_js('xzoom.min.js');
			Assets::add_css('xzoom.css');
			Assets::add_js("$('.xzoom, .xzoom-gallery').xzoom({
        	zoomWidth: 800, // creating issue on bootstrap 4
        	zoomHeight: 600,
        	title: true,
        	tint: false
        });
        if ($(window).width() < 600) {
            $('.zoom-thumb').css('flex-direction', 'row !important');
            $('.zoom-thumb').css('display', 'block !important');
            $('.zoomer').css('display', 'block !important');
        }
    ", 'inline');
			$images = $this->gc->get('images', array('where' => 'module_id = ' . gc_module_id('projects') . ' AND row_id = ' . $record->id . ' AND status = 1 AND deleted = 0', 'result' => 1));
			if($images) {
			    $project_images = array();
			    foreach($images as $image) {
			        if($image->type_id == 1) {
			            $record->cover = array(
			                'title' => empty($image->title) ? $record->title : $image->title,
			                'thumb' => gc_image_url($image->url),
			                'original' => gc_image_url($image->url, false)
			            );
			        } else if($image->type_id == 2) {
			            $record->logo = array(
			                'title' => empty($image->title) ? $record->title : $image->title,
			                'thumb' => gc_image_url($image->url),
			                'original' => gc_image_url($image->url, false)
			            );
			        } else {
			            $project_images[] = array(
			                'title' => empty($image->title) ? $record->title : $image->title,
			                'thumb' => gc_image_url($image->url),
			                'original' => gc_image_url($image->url, false)
			            );
			        }
			    }
			    $record->images = $project_images;
			}
			$tenant = $this->gc->get('project_tenants', array('where' => 'project_id = ' . $record->id, 'order_by' => 'id DESC'));
			if(!$tenant) {
			    $tenant = $this->gc->get('project_tenants', array('where' => 'name_id = ' . $record->builder_project_id . ' AND project_id = 0', 'order_by' => 'id DESC'));
			}
			$record->tenant = $tenant;

			// yearly return
			$yearly_yield = ($record->bsp_amount * $record->yield) / 100;
			$yearly_maintenance = $yearly_yield * 20 / 100;
			$yearly_tax = ($yearly_yield - $yearly_maintenance) * 10 / 100;
			$monthly_yield = amount_inr_format(round($yearly_yield / 12));
			$monthly_maintenance = amount_inr_format(round($yearly_maintenance / 12));
			$monthly_tax = amount_inr_format(round($yearly_tax / 12));			
			
			Assets::add_css('nice-select.css');
			Assets::add_js('jquery.nice-select.min.js');
			Assets::add_js("
$('#property_returns').niceSelect();
$('#property_returns').change(function() {
    if($(this).val() == 'monthly') {
        $('.project-rent').text('" . $monthly_yield . "');
        $('.project-rent-maintenance').text('" . $monthly_yield . "');
        $('.project-rent-tax').text('" . $monthly_tax . "');
    } else {
        $('.project-rent').text('" . amount_inr_format($yearly_yield) . "');
        $('.project-rent-maintenance').text('" . amount_inr_format($yearly_maintenance) . "');
        $('.project-rent-tax').text('" . amount_inr_format($yearly_tax) . "');
    }
});
", 'inline');
			
			Template::set(array(
				'record' => $record,
			    'yearly_yield' => $yearly_yield,
			    'yearly_maintenance' => $yearly_maintenance,
			    'yearly_tax' => $yearly_tax,
			    'monthly_yield' => $monthly_yield,
			    'monthly_maintenance' => $monthly_maintenance,
			    'monthly_tax' => $monthly_tax
			));
			
			// update module hits
			$this->gc->updateHits($record->id, 'projects');
				
			// add/update recently viewed status
			$this->gc->add_recently_viewed($record->id, 'projects');
			$layout = $this->gc->load_assets('modules/projects/detail');
			Template::set_view($layout['view']);
			Template::render($layout['layout']);
		}
	}
}