<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class Projects_model extends BF_Model {
	public $table_name	= "projects";
	protected $key			= "id";
	protected $soft_deletes	= false;
	protected $date_format	= "datetime";

	protected $log_user 	= FALSE;

	protected $set_created	= TRUE;
	protected $set_modified = TRUE;
	protected $created_field = "created_on";
	protected $modified_field = "modified_on";

	/*
		Customize the operations of the model without recreating the insert, update,
		etc methods by adding the method names to act as callbacks here.
	 */
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 		= array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	/*
		For performance reasons, you may require your model to NOT return the
		id of the last inserted row as it is a bit of a slow method. This is
		primarily helpful when running big loops over data.
	 */
	protected $return_insert_id 	= TRUE;

	// The default type of element data is returned as.
	protected $return_type 			= "object";

	// Items that are always removed from data arrays prior to
	// any inserts or updates.
	protected $protected_attributes = array();

	/*
		You may need to move certain rules (like required) into the
		$insert_validation_rules array and out of the standard validation array.
		That way it is only required during inserts, not updates which may only
		be updating a portion of the data.
	 */
	protected $validation_rules = array();
	protected $insert_validation_rules 	= array();
	protected $skip_validation 			= FALSE;
	
	/**
	 * Project Tenants Fields
	 *
	 * @return array
	 */
	public function tenants() {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'name_id',
	            'label' => 'singular_project_names',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'name:project_names;value:database',
	            'options_detailed' => 1,
	            'options_lang' => 1,
	            'class' => 4,
	            'rules' => 'trim|is_numeric',
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'project_id',
	            'label' => 'singular_projects',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'name:projects;value:database',
	            'options_detailed' => 1,
	            'options_lang' => 1,
	            'class' => 4,
	            'rules' => 'trim|is_numeric',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'name',
	            'label' => 'label_name',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'edit' => 1,
	            'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
	            'rules' => 'trim|required|max_length[255]',
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'country',
	            'label' => 'label_country',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'options' => 'name:countries;value:database',
	            'options_detailed' => 1,
	            'options_lang' => 1,
	            'class' => 4,
	            'rules' => 'trim|required|max_length[80]',
	        ),
	        array(
	            'type' => 'filemanager',
	            'name' => 'logo',
	            'label' => 'label_logo',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim|max_length[255]',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'lease_start_date',
	            'label' => 'project_lease_start_date',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'class' => 3,
	            'rules' => 'trim|required|valid_datetime',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'lease_end_date',
	            'label' => 'project_lease_end_date',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'class' => 3,
	            'rules' => 'trim|required|valid_datetime',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'lease_lockin_period',
	            'label' => 'project_lease_lockin_period',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'value' => 0,
	            'class' => 2,
	            'rules' => 'trim|required|is_decimal',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'rent_squarefeet',
	            'label' => 'project_rent_squarefeet',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'value' => 0,
	            'class' => 2,
	            'rules' => 'trim|required|is_decimal',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'escalation',
	            'label' => 'project_escalation',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'value' => 0,
	            'class' => 2,
	            'rules' => 'trim|required|is_decimal',
	        ),
	        array(
	            'type' => 'textarea',
	            'name' => 'description',
	            'label' => 'label_description',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_on',
	            'label' => 'created_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_by',
	            'label' => 'created_by',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_ip',
	            'label' => 'created_ip',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_on',
	            'label' => 'modified_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_by',
	            'label' => 'modified_by',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_ip',
	            'label' => 'modified_ip',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        )	        
	    );
	    return $fields;
	}
	
	/**
	 * Project Builders Fields
	 *
	 * @return array
	 */
	public function builders() {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'name',
	            'label' => 'label_name',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'edit' => 1,
	            'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
	            'rules' => 'trim|required|max_length[100]',
	        ),
	        array(
	            'type' => 'filemanager',
	            'name' => 'image',
	            'label' => 'label_image',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim|max_length[255]',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_on',
	            'label' => 'created_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_on',
	            'label' => 'modified_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        )	        
	    );
	    return $fields;
	}
	
	/**
	 * Project Names Fields
	 *
	 * @return array
	 */
	public function names() {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'builder_id',
	            'label' => 'singular_project_builders',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'name:project_builders;value:database',
	            'options_detailed' => 1,
	            'options_lang' => 1,
	            'class' => 3,
	            'rules' => 'trim|required',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'name',
	            'label' => 'label_name',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'edit' => 1,
	            'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
	            'rules' => 'trim|required|max_length[100]',
	        ),
	        array(
	            'type' => 'filemanager',
	            'name' => 'image',
	            'label' => 'label_image',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim|max_length[255]',
	        ),
	        array(
	            'type' => 'textarea',
	            'name' => 'payment_terms',
	            'label' => 'label_payment_terms',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'custom_rules' => 'editor',
	            'field_class' => 'editor',
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_on',
	            'label' => 'created_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_on',
	            'label' => 'modified_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        )	        
	    );
	    return $fields;
	}
	
	/**
	 * Project Details Fields
	 *
	 * @return array
	 */
	public function details() {
	    $groups_text = array();
	    $text_groups = $this->text_groups();
	    foreach($text_groups as $tkey => $tvalue) {
	        $groups_text[] = 'name:' . $tvalue . ";value:" . $tkey;
	    }
	    $groups_text = implode('|', $groups_text);
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'text_group',
	            'label' => 'label_group',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => $groups_text,
	            'options_detailed' => 1,
	            'class' => 3,
	            'rules' => 'trim|required|max_length[50]',
	        ),
	        array(
	            'type' => 'textarea',
	            'name' => 'description',
	            'label' => 'label_description',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim|required',
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'status',
	            'label' => 'label_status',
	            'frontend' => false,
	            'admin_only' => false,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        
	    );
	    return $fields;
	}
	
	/**
	 * Calculate Project Total
	 * @param object $record
	 * @return array
	 */
	public function calculate_total($record) {
	    $colors = array('#007bff','#28a745','#333333','#c3e6cb','#dc3545','#6c757d', '#F28878', '#E97E00', '#FB21BA', '#A3706F', '#7457EF', '#351267', '#DCA9E2', '#B39A88', '#8BD80A');
	    $costs = array();
	    $project_costs = array();
	    $i = 0;
	    $project_total = 0;
	    foreach($record as $key => $value) {
	        if(strpos($key, '_amount') !== false && !empty($value)) {
	            switch($key) {
	                case 'ifms_amount':
	                case 'ff_amount':
	                case 'eec_amount':
	                case 'edc_amount':
	                case 'pb_amount':
	                    $value = $record->squarefeet * $value;
	                    break;
	                case 'gst_amount':
	                    $value = ($record->bsp_amount * $value) / 100;
	                    break;
	            }
	            $costs['names'][] = lang('project_' . $key);
	            $costs['values'][] = $value;
	            $project_costs[] = array(
	                'name' => $key,
	                'label' => lang('project_' . $key),
	                'value' => $value,
	                'color' => $colors[$i]
	            );
	            $i++;
	            $project_total += $value;
	        }
	    }
	    return array('colors' => $colors, 'total' => $project_total, 'costs' => $costs, 'project_costs' => $project_costs);
	}
	
	/**
	 * Method called in Gcrud library after project insert/update
	 * @param number $id
	 * @param string $action_type
	 * @param string $record
	 */
	public function after_project_save($id, $action_type = 'insert', $record = array()) {
	    $record = $this->db->where('id', $id)->get($this->table_name)->row();
	    if($record) {
	        $project_calculations = $this->calculate_total($record);
	        $price_squarefeet = round(currency_format($project_calculations['total'] / $record->squarefeet, array('price_only' => true)));
	        // get builder_id
	        $data = array('price_squarefeet' => $price_squarefeet);
	        $data['project_total'] = $project_calculations['total'];
	        $data['min_investment'] = round($record->project_total/$record->max_investors);
	        $builder = $this->db->where('id', $record->name_id)->get('project_names')->row();
	        if($builder) {
	            $data['builder_id'] = $builder->builder_id;
	        }
	        $this->db->where('id', $id)->update($this->table_name, $data);
	    }
	}
	
	/**
	 * Search Form
	 */
	public function search_form() {
	    $params = $_GET;
	    if(!empty($params['min_investment'])) {
	        $params['min_investment'] = explode(',', $params['min_investment'] ?? '');
	    }
	    // investment range selector
	    $min = $this->gc->get('projects', array('where' => 'status = 1 AND deleted = 0', 'select' => 'min_investment AS min', 'order_by' => 'min_investment ASC'));
	    $max = $this->gc->get('projects', array('where' => 'status = 1 AND deleted = 0', 'select' => 'min_investment AS max', 'order_by' => 'min_investment DESC'));
	    $investment_min = empty($min->min) || $min->min == $max->max ? 0 : $min->min;
	    $investment_max = empty($max->max) ? 0 : $max->max;
	    $min_investment_value = !empty($params['min_investment'][0]) ? (int) $params['min_investment'][0] : $investment_min;
	    $max_investment_value = !empty($params['min_investment'][1]) ? (int) $params['min_investment'][1] : $investment_max;
	    $step = 5;
	    Assets::add_css('nice-select.css');
	    Assets::add_js('jquery.nice-select.min.js');
	    Assets::add_js("$('#builder_id').selectize();", 'inline');
	    Assets::add_css('https://cdnjs.cloudflare.com/ajax/libs/jqueryui/1.12.1/themes/start/jquery-ui.min.css');
	    Assets::add_css('https://cdnjs.cloudflare.com/ajax/libs/jqueryui/1.12.1/themes/start/theme.min.css');
	    Assets::add_js('https://cdnjs.cloudflare.com/ajax/libs/jqueryui/1.12.1/jquery-ui.min.js');
	    Assets::add_js('https://cdnjs.cloudflare.com/ajax/libs/jqueryui-touch-punch/0.2.3/jquery.ui.touch-punch.min.js');
	    Assets::add_js("
var fieldName = 'investment_range';
var currency = 'Rs ';
$('.slider-range').slider({
    range:true,
    min: ". $investment_min .",
    max: ". $investment_max .",
    values: [". $min_investment_value .", ". $max_investment_value ."],
    step: ". $step .",
    stop:function(event, ui) {
		$('#' + fieldName).html(currency + ui.values[0] + ' - ' + currency + ui.values[1]);
        // $('.slider-range > span:first-of-type::after').style.content = ui.values[0];
        // $('.slider-range > span:first-of-type::after').setAttribute('style', \"content: red;\");
        // $('.slider-range > span:first-of-type::after').css('content', 'RED');
        // $('.slider-range > span:first-of-type::after').attr('style','content: 300px');
        // $(this).find('ui-state-focus::after').css('content', 'RED');
        // $('#' + fieldName + '_min').val(ui.values[0]);
        $('#min_investment').val([ui.values[0], ui.values[1]]);
    }
});
", 'inline');
	    $result = array('params' => $params);
	    Template::set(array(	        
	        'investment_min' => $investment_min,
	        'investment_max' => $investment_max,
	        'min_investment_value' => $min_investment_value,
	        'max_investment_value' => $max_investment_value
	    ));
	    return $result;
	}
	
	/**
	 * Regenerate description for all projects
	 */
	public function regenerate_details_all() {
	    $records = $this->gc->get_all_crud(0, null, 'M.id ASC', 50000, 0, array('module' => 'projects', 'prefix' => ''));
	    if($records) {
	        foreach($records as $row) {
	            $this->random_description($row->id, $row);
	        }
	    }
	}
	
	/**
	 * Generate random description
	 * @param number $row_id
	 * @param array $row
	 */
	public function random_description($row_id = 0, $row = array()) {
	    if(empty($row)) {	        
	        $records = $this->gc->get_all_crud(0, "M.id = '" . $row_id ."'" , 'M.id ASC', 1, 0, array('module' => 'projects', 'prefix' => ''));
	        if(!$records) {
	            redirect(site_url());
	        }
	        $row = $records[0];
	    }
	    $records = $this->db->order_by('RAND()')->group_by('text_group')->get('project_details')->result();
	    if($records && !empty($row)) {
	        $row = (array) $row;
	        $tenant = $this->gc->get('project_tenants', array('where' => 'project_id = ' . $row['id'], 'order_by' => 'id DESC'));
	        if(!$tenant) {
	            $tenant = $this->gc->get('project_tenants', array('where' => 'name_id = ' . $row['builder_project_id'] . ' AND project_id = 0', 'order_by' => 'id DESC'));
	        }
	        if($tenant) {
	            foreach($tenant as $key => $value) {
	                switch($key) {
	                    case 'project_id':
	                    case 'name_id':
	                    case 'created_on':
	                    case 'created_by':
	                    case 'created_ip':
	                    case 'modified_on':
	                    case 'modified_by':
	                    case 'modified_ip':
	                    case 'deleted':
	                        break;
	                    case 'lease_start_date':
	                    case 'lease_end_date':
	                        $row['tenant_' . $key] = gc_date_format($value);
	                        break;
	                    default:
	                        $row['tenant_' . $key] = $value;
	                }
	            }
	        }
	        $row['project_total'] = 'Rs ' . amount_inr_format($row['project_total']);
	        $row['price_squarefeet'] = 'Rs ' . amount_inr_format($row['price_squarefeet']);
	        $this->load->library('parser');
	        $output = array();
	        /*if(!empty($row['floors'])) { // text field, input value simply
	            $floor_options = array();
	            $floors = array_map('trim', explode(',', $row['floors'] ?? ''));
	            $last_floor = end($floors);
	            foreach($floors as $floor) {
	                switch($floor) {
        	            case '-1':
        	                $floor_option = lang('project_floor_basement');
        	                break;
        	            case 0:
        	                $floor_option = lang('project_floor_ground');
        	                break;
        	            default:
        	                $floor_option = $this->ordinal($floor);
        	        }
        	        if(count($floors) > 1 && $last_floor == $floor) {
        	            $floor_options[] = ' and ' . $floor_option;
        	        } else {
        	           $floor_options[] = $floor_option;
        	        }
	            }
	            $row['floors'] = implode(', ', $floor_options);
	        }*/
	        foreach($records as $record) {
	            $output[$record->text_group] = $this->parser->parse_string($record->description, $row, true);
	        }
	        $text_groups = $this->text_groups();
	        $details = array();
	        foreach($text_groups as $tkey => $tvalue) {
	            if(!empty($output[$tkey])) {
	                $details[] = $output[$tkey];
	            }
	        }
	        $description = implode('<br /><br />', $details);
	        $this->db->where('id', $row['id'])->update($this->table_name, array('description' => $description));
	        return $description;
	    }
	}
	
	/**
	 * Word Groups
	 * @return string[]
	 */
	public function text_groups() {
	    $groups = array(
	        'heading' => 'Heading',
	        'first_paragraph' => 'First Paragraph',
	        'second_paragraph' => 'Second Paragraph',
	        'other' => 'Other'
	    );
	    return $groups;
	}
	
	/**
	 * Add ordinal to the floor number
	 * @param number $number
	 * @return string
	 */
	public function ordinal($number) {
	    $ends = array('th','st','nd','rd','th','th','th','th','th','th');
	    if ((($number % 100) >= 11) && (($number%100) <= 13))
	        return $number. 'th';
	        else
	            return $number. $ends[$number % 10];
	}
}
