<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Ratings controller
 */
class Content extends Admin_Controller
{
	protected $permissionCreate = 'Listings.Content.Create';
	protected $permissionDelete = 'Listings.Content.Delete';
	protected $permissionEdit   = 'Listings.Content.Edit';
	protected $permissionView   = 'Listings.Content.View';
	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct() {
		parent::__construct();

		$this->auth->restrict($this->permissionView);
		$this->load->model('ratings_model', null, true);
		$this->lang->load('ratings');
		
		Template::set_block('sub_nav', 'content/_sub_nav');
	}
	
	/**
	 * Display Ratings to admin
	 */
	public function index($filter='all', $offset = 0) {
		// Do we have any actions?
		if ($action = $this->input->post('submit')) {
			$checked = $this->input->post('checked');
			if (is_array($checked) && count($checked)) {
				switch(strtolower($action)) {
					case 'delete':
						$result = FALSE;
						foreach ($checked as $pid) {
							$result = $this->ratings_model->delete($pid);
						}
						if ($result) {
							Template::set_message(count($checked) .' '. sprintf(lang('msg_delete_success'), lang('label_ratings')), 'success');
						} else {
							Template::set_message(lang('delete_failure') . $this->ratings_model->error, 'danger');
						}
						break;
					case 'save':
						$result = FALSE;
						foreach ($checked as $pid) {
							$result = $this->save_rating($pid);
						}
						if ($result) {
							Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), lang('label_ratings'), lang('label_updated')), 'success');
						} else {
							Template::set_message(sprintf(lang('msg_edit_failure'), lang('label_ratings')), 'error');
						}
						break;
				}
			}					
		}
		
		// To add new rating, the Admin need to do it via Frontend
		// Admin Panel is only to update the existing ratings
		if(!empty($_POST['save'])) {
			$field_id = $_POST['save'];
			$this->save_rating($field_id);
			Template::set_message(sprintf(lang('msg_action_success'), lang('label_ratings'), lang('label_updated')), 'success');
			// Template::redirect(current_url());
		}
		
		// Actions done, now display the view
		$where = NULL;
	
		// Filters
		$filter_type = $filter;
	
		
		switch($filter_type) {
			case 'all':
				break;
	
			default:
				$module_id = gc_module_id($filter_type);
				if($module_id == 0) {
					Template::redirect(site_url(SITE_AREA ."/content/ratings/index"));
				}
				$where = "module_id = " . $module_id;
		}
	
		// Pagination
		$this->load->library('pagination');
		
		// Search
		$searchterm = $this->gc->search('ratings', $this->uri->segment(6), '/content/ratings');
		if(!empty($searchterm)) {
		    if(!empty($where)) {
		        $where .= " AND ";
		    }
		    $where .= " (T.id LIKE '%". $searchterm ."%' OR T.row_id LIKE '%". $searchterm ."%' OR T.average_rating LIKE '%". $searchterm ."%' OR T.total_users LIKE '%". $searchterm ."%' OR T.total_ratings LIKE '%". $searchterm ."%')";
		}
	
		// Sort By
		$sort_by = "id DESC";
		$order_by = $this->gc->sort('images');
		if(!empty($order_by)) {
			$sort_by = $order_by;
		}
		$select = array("(SELECT COUNT(RU.id) FROM " . $this->db->dbprefix . "rating_users RU WHERE RU.rating_id = T.id) AS rating_users");
		$records = $this->gc->get_all_row(0, 'ratings', $where, $sort_by, $offset, $this->limit, null, array(), $select);
		$total_rows = $this->gc->get_all_row(1, 'ratings', $where, $sort_by, $offset, $this->limit);
		$pager['base_url'] = site_url(SITE_AREA ."/content/ratings/index/$filter");
		$pager['total_rows'] = $total_rows;
		$pager['per_page'] = $this->limit;
		$pager['uri_segment']	= 6;
		
		$this->pagination->initialize($pager);
	
		Template::set('index_url', site_url(SITE_AREA .'/content/ratings/index/'));
		Template::set('filter_type', $filter_type);
	
		Template::set('records', $records);
		Template::set('toolbar_title', lang('manage_ratings'));
		Template::render();
	}
	
   /**
     * Admin - Display Rating History to the Admin
     * @param int $offset
     * @return void
     */
    public function history($id, $offset = 0) {
    	if (empty($id)) {
    		Template::set_message(lang('invalid_id'), 'error');    	
    		redirect(SITE_AREA . '/content/ratings');
    	}
    	// Deleting anything?
    	if (isset($_POST['delete'])) {
    		$this->auth->restrict('Ratings.Content.Delete');
    		$checked = $this->input->post('checked');
    		if (is_array($checked) && count($checked)) {
    
    			// If any of the deletions fail, set the result to false, so
    			// failure message is set if any of the attempts fail, not just
    			// the last attempt
    
    			$result = false;
    			foreach ($checked as $pid) {
    				$result = $this->db->delete('rating_users', array('id' => $pid));
    			}
    			if ($result) {
    				Template::set_message(count($checked) . ' ' . sprintf(lang('msg_delete_success'), lang('rating_history')), 'success');
    			} else {
    				Template::set_message(sprintf(lang('msg_delete_failure'), lang('rating_history')) . $this->ratings_model->error, 'danger');
    			}
    		}
    	}
    
    	// Order By
    	$sort_by = "id desc";
    	$order_by = $this->gc->sort('rating_users');
    	if(!empty($order_by)) {
    		$sort_by = $order_by;
    	}
    
    	$pagerUriSegment = 6;
    	$pagerBaseUrl = site_url(SITE_AREA . '/content/ratings/history/' . $id) . '/';
    
    	$limit  = $this->settings_lib->item('site.list_limit') ?: 15;
    	$count = $this->ratings_model->get_history(1, $id, null);
    
    	$this->load->library('pagination');
    	$pager['base_url']    = $pagerBaseUrl;
    	$pager['total_rows']  = $count;
    	$pager['per_page']    = $limit;
    	$pager['uri_segment'] = $pagerUriSegment;
    
    	$this->pagination->initialize($pager);
    
    	$records = $this->ratings_model->get_history(0, $id, null, $sort_by, $offset, $limit);
    
    	Template::set('records', $records);
    	Template::set('toolbar_title', lang('label_history'));
    	Template::render();
    }
	
	/**
	 * Save Rating
	 * @param number $field_id - Rating ID
	 */
	private function save_rating($field_id) {
		$average_rating = 'average_rating_' . $field_id;
		$total_ratings = 'total_ratings_' . $field_id;
		$total_users = 'total_users_' . $field_id;
		$this->form_validation->set_rules($average_rating, lang('average_rating'), 'trim|required|is_decimal');
		$this->form_validation->set_rules($total_ratings, lang('total_ratings'), 'trim|required|is_decimal');
		$this->form_validation->set_rules($total_users, lang('total_users'), 'trim|required|is_numeric');
		if ($this->form_validation->run() == TRUE) {
			$data = array();
			$data['average_rating'] = $this->input->post($average_rating);
			$data['total_ratings'] = $this->input->post($total_ratings);
			$data['total_users'] = $this->input->post($total_users);
			$rating = $this->ratings_model->find($field_id);
			if($rating) {
			    // update ratings in the module table
			    $rating_module = gc_module_name($rating->module_id);
			    $this->ratings_model->update_rating($rating_module, $rating->row_id);
			}
			return $this->ratings_model->update($field_id, $data);
		}
	}
	
	/**
	 * Allow admin to bulk generate ratings for all listings
	 */
	public function random() {
		$this->lang->load('listings/listings');
		$this->load->model('ratings/ratings_model');
		if (isset($_POST['save'])) {
			$this->form_validation->set_rules('module', lang('label_module'), 'trim');
			$this->form_validation->set_rules('avg_min_value', lang('label_average_rating'), 'required|integer|min_val[1]');
			$this->form_validation->set_rules('avg_max_value', lang('label_average_rating'), 'required|integer|max_val[5]|compare_fields['.$this->input->post('avg_min_value').']');
			$this->form_validation->set_rules('usr_min_value', lang('label_total_users'), 'required|integer|min_val[1]');
			$this->form_validation->set_rules('usr_max_value', lang('label_total_users'), 'required|integer|max_val[20]|compare_fields['.$this->input->post('usr_min_value').']');
			$this->form_validation->set_rules('id_from', lang('label_ids'), 'required|integer|min_val[1]');
			$this->form_validation->set_rules('id_to', lang('label_ids'), 'required|integer|max_val[10000000]|compare_fields['.$this->input->post('id_from').']|diff_between['.$this->input->post('id_from').',50000]');
			if($this->form_validation->run() == TRUE) {
				$module = $this->input->post('module');
				$avg_min = $this->input->post('avg_min_value');
				$avg_max = $this->input->post('avg_max_value');
				$user_min = $this->input->post('usr_min_value');
				$user_max = $this->input->post('usr_max_value');
				if($this->input->post('insert_or_update') == 1) {
					// INSERT ONLY
					$sql = "SELECT id FROM " . $this->db->dbprefix . $module ." WHERE id BETWEEN ".$this->input->post('id_from')." AND ".$this->input->post('id_to')." AND id NOT IN (SELECT row_id FROM " . $this->db->dbprefix . "ratings R WHERE R.module_id = ". gc_module_id($module) .") LIMIT 50000";
					$query = $this->db->query ( $sql );
					$get_ids = $query->result_array ();
					if ($get_ids) {
						$data = array ();
						foreach ( $get_ids as $id ) {
							$avergate_rating = mt_rand ( $avg_min * 5, $avg_max * 5 ) / 5;
							$total_users = mt_rand ( $user_min, $user_max );
							$data [] = array (
									'row_id' => $id ['id'],
									'module_id' => gc_module_id($module),
									'average_rating' => $avergate_rating,
									'total_ratings' => $avergate_rating * $total_users,
									'total_users' => $total_users
							);
							
							// update ratings in the module table
							$this->ratings_model->update_rating($module, $id['id']);
						}
						$this->ratings_model->insert_batch( $data );
						Template::set_message(lang('msg_success'), 'success');
					} else {
						Template::set_message(sprintf(lang('msg_nothing_to_update'), gc_module_translation($module)), 'error');
					}
				} else {
					// INSERT AND UPDATE
					$update_sql = "SELECT id FROM " . $this->db->dbprefix . $module . " WHERE id BETWEEN ".$this->input->post('id_from')." AND ".$this->input->post('id_to')." LIMIT 50000";
					$update_query = $this->db->query ( $update_sql );
					$update_ids = $update_query->result_array ();
					if ($update_ids) {
						foreach ( $update_ids as $id ) {
							$avg_rating = mt_rand ( $avg_min * 5, $avg_max * 5 ) / 5;
							$tot_users = mt_rand ( $user_min, $user_max );
	
							$insert_data = array (
									'row_id' => $id ['id'],
									'module_id' => gc_module_id($module),
									'average_rating' => $avg_rating,
									'total_ratings' => $avg_rating * $tot_users,
									'total_users' => $tot_users,
									'created_on' => date ( "Y-m-d H:i:s" )
							);
							
							// update ratings in the module table
							$this->ratings_model->update_rating($module, $id['id']);
							
							$update_data = "average_rating = " . $avg_rating . ", total_ratings = " . $avg_rating * $tot_users . ", total_users = " . $tot_users . ", modified_on = '" . date ( "Y-m-d H:i:s" ) . "'";
	
							$insert_update_query = $this->db->insert_string ( $this->db->dbprefix . 'ratings', $insert_data ) . ' ON DUPLICATE KEY UPDATE ' . $update_data;
							$this->db->query ( $insert_update_query );
							Template::set_message(lang('msg_success'), 'success');
						}
					} else {
						Template::set_message(sprintf(lang('msg_insert_success'), gc_module_translation($module)), 'success');
					}
				}
			} else {
				Template::set_message(lang('create_failure') . $this->ratings_model->error, 'danger');
			}
		}
		Template::set('toolbar_title', lang('gc_menu_random_ratings'));
		Template::render();
	}
	
	/**
	 * Update ratings and rating_users column in the modules table
	 * Done to prevent Extra JOIN
	 * 
	 */
	public function update_modules() {
	    $modules = gc_modules_only();
	    $modules[] = 'listings';
	    $modules[] = 'videos';
	    if(isset($_POST['save'])) {
	        $rating_modules = array();
	        $site_modules = $this->input->post('modules');
	        if($site_modules != 0) {
	            $rating_modules[] = gc_module_name($site_modules);
	        } else {
	            $rating_modules = $modules;
	        }
	        if(!empty($rating_modules)) {
	            foreach($rating_modules as $rating_module) {
    	            $module_id = gc_module_id($rating_module);
    	            $sql = "SELECT id, module_id, row_id, average_rating, total_users FROM " . $this->db->dbprefix . "ratings WHERE module_id = " . $module_id;
    	            $records = $this->db->query($sql)->result();
    	            if($records) {
    	                $data = array();
    	                foreach($records as $record) {
							$data[] = array(
								'id' => $record->row_id,
								'ratings' => $record->average_rating,
								'rating_users' => $record->total_users
							);
    	                }
    	            }
    	            $this->db->update_batch($rating_module, $data, 'id');
    	        }
    	        Template::set_message(lang('msg_success'), 'success');
    	        Template::redirect(current_url());
    	    } else {
    	        Template::set_message(lang('msg_error'), 'danger');
    	    }	
	    }
	    $view_modules = array(	        
	        0 => lang('tab_all')
	    );
	    foreach($modules as $module) {
	        $view_modules[gc_module_id($module)] = lang($module);
	    }
	    asort($view_modules);
	    Template::set(array(
	        'toolbar_title' => lang('gc_menu_module_ratings'),
	        'modules' => $view_modules
	    ));
	    Template::render();
	}
	
	
	/**
	 * Display a list of options data.
	 *
	 * @return void
	 */
	public function options($filter='all', $offset = 0) {
	    // Status Change
	    if (isset($_POST['status_change'])) {
	        $this->gc->status_change('rating_options', $_POST['status_change'], 'status');
	    }
	    // Deleting anything?
	    if (isset($_POST['delete'])) {
	        $this->auth->restrict($this->permissionDelete);
	        $checked = $this->input->post('checked');
	        if (is_array($checked) && count($checked)) {
	            
	            // If any of the deletions fail, set the result to false, so
	            // failure message is set if any of the attempts fail, not just
	            // the last attempt
	            
	            $result = true;
	            foreach ($checked as $pid) {
	                $deleted = $this->ratings_model->purge_option($pid);
	                if ($deleted == false) {
	                    $result = false;
	                }
	            }
	            if ($result) {
	                Template::set_message(count($checked) . ' ' . sprintf(lang('msg_action_success'), lang('label_options'), lang('label_deleted')), 'success');
	            } else {
	                Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_options')) . $this->ratings_model->error, 'danger');
	            }
	        }
	    }
	    $where = '';
	    // Filters
	    $filter_type = $filter;
	    
	    switch($filter_type) {
	        case 'ratings':
	            $where = "type_id = 0";
	            break;
	            
	        case 'groups':
	            $where = "type_id = 1";
	            break;
	            
	        case 'all':
	            // Nothing to do
	            break;
	            
	        default:
	            show_404("ratings/options/$filter/");
	    }
	    
	    $like_array = array();
	    $searchterm = $this->gc->search('rating_options', $this->uri->segment(5), '/content/ratings/options');
	    if(!empty($searchterm)) {
	        // This is the basic search filter, you could add more fields to the search
	        $like_array = array(
	            'id' => $searchterm,
	            'name' => $searchterm
	        );
	    }
	    $pagerUriSegment = 6;
	    $pagerBaseUrl = site_url(SITE_AREA . '/content/ratings/options') . '/';
	    
	    $limit  = $this->settings_lib->item('site.list_limit') ?: 15;// Sort By
	    if(!empty($where)) {
	        $this->db->where($where);
	    }
	    if(!empty($like_array)) {
	        $this->db->or_like($like_array);
	    }
	    $total_rows = $this->db->count_all_results('rating_options');
	    $this->load->library('pagination');
	    $pager['base_url']    = $pagerBaseUrl;
	    $pager['total_rows']  = $total_rows;
	    $pager['per_page']    = $limit;
	    $pager['uri_segment'] = $pagerUriSegment;
	    
	    $this->pagination->initialize($pager);
	    $this->db->limit($limit, $offset);
	    
	    $order_by = $this->gc->sort('rating_options');
	    if(!empty($order_by)) {
	        $this->db->order_by($this->session->userdata('options_sort')['column'], $this->session->userdata('options_sort')['order']);
	    } else {
	        $this->db->order_by('id', 'DESC');
	    }
	    // Sort By
	    if(!empty($where)) {
	        $this->db->where($where);
	    }
	    if(!empty($like_array)) {
	        $this->db->or_like($like_array);
	    }
	    $records = $this->db->get('rating_options')->result();
	    Template::set('records', $records);
	    Template::set('index_url', site_url(SITE_AREA .'/content/ratings/options/'));
	    Template::set('filter_type', $filter_type);
	    Template::set('toolbar_title', lang('manage_options'));
	    Template::render();
	}
	
	/**
	 * Create a options object.
	 *
	 * @return void
	 */
	public function add_option() {
	    $this->auth->restrict($this->permissionCreate);
	    
	    if (isset($_POST['save'])) {
	        if ($insert_id = $this->save_options()) {
	            log_activity($this->auth->user_id(), lang('act_create_record') . ': ' . $insert_id . ' : ' . $this->input->ip_address(), 'rating_options');
	            Template::set_message(sprintf(lang('msg_save_success'), lang('label_option')), 'success');
	            
	            redirect(SITE_AREA . '/content/ratings/options');
	        }
	        
	        // Not validation error
	        if ( ! empty($this->ratings_model->error)) {
	            Template::set_message(sprintf(lang('msg_create_failure'), lang('label_option')) . $this->ratings_model->error, 'danger');
	        }
	    }
	    
	    Template::set('toolbar_title', lang('add_option'));
	    Template::set_view('content/options_form');
	    Template::render();
	}
	/**
	 * Allows editing of options data.
	 *
	 * @return void
	 */
	public function edit_option() {
	    $id = $this->uri->segment(5);
	    if (empty($id)) {
	        Template::set_message(lang('invalid_id'), 'error');
	        redirect(SITE_AREA . '/content/ratings/options');
	    }
	    
	    if (isset($_POST['save'])) {
	        $this->auth->restrict($this->permissionEdit);
	        
	        if ($this->save_options('update', $id)) {
	            log_activity($this->auth->user_id(), lang('act_edit_record') . ': ' . $id . ' : ' . $this->input->ip_address(), 'rating_options');
	            Template::set_message(sprintf(lang('msg_save_success'), lang('label_option')), 'success');
	            redirect(SITE_AREA . '/content/ratings/options');
	        }
	        
	        // Not validation error
	        if ( ! empty($this->ratings_model->error)) {
	            Template::set_message(sprintf(lang('msg_edit_failure'), lang('label_option')) . $this->ratings_model->error, 'danger');
	        }
	    } elseif (isset($_POST['delete'])) {
	        $this->auth->restrict($this->permissionDelete);
	        
	        if ($this->ratings_model->purge_option($id)) {
	            log_activity($this->auth->user_id(), lang('act_delete_record') . ': ' . $id . ' : ' . $this->input->ip_address(), 'rating_options');
	            Template::set_message(sprintf(lang('msg_delete_success'), lang('label_option')), 'success');
	            redirect(SITE_AREA . '/content/ratings/options');
	        }
	        Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_option')) . $this->ratings_model->error, 'danger');
	    }
	    $record = $this->db->where('id', $id)->get('rating_options')->row();
	    Template::set('record', $record);
	    
	    Template::set('toolbar_title', lang('edit_option'));
	    Template::set_view('content/options_form');
	    Template::render();
	}
	
	//--------------------------------------------------------------------------
	// !PRIVATE METHODS
	//--------------------------------------------------------------------------
	
	/**
	 * Save the data.
	 *
	 * @param string $type Either 'insert' or 'update'.
	 * @param int    $id   The ID of the record to update, ignored on inserts.
	 *
	 * @return boolean|integer An ID for successful inserts, true for successful
	 * updates, else false.
	 */
	private function save_options($type = 'insert', $id = 0) {
	    if ($type == 'update') {
	        $_POST['id'] = $id;
	    }
	    
	    // Validate the data
	    $this->form_validation->set_rules('type_id', 'lang:label_type', 'trim|is_numeric');
	    $this->form_validation->set_rules('name', 'lang:label_name', 'trim|required');
	    $this->form_validation->set_rules('status', 'lang:label_status', 'trim');
	    if ($this->form_validation->run() === false) {
	        return false;
	    }
	    $data = array(
	        'type_id' => $this->input->post('type_id'),
	        'name' => $this->input->post('name'),
	        'status' => $this->input->post('status') ? 1 : 0
	    );
	    $return = false;
	    if ($type == 'insert') {
	        $data['created_on'] = date('Y-m-d H:i:s');
	        $data['created_by'] = $this->auth->user_id();
	        $data['created_ip'] = $this->input->ip_address();
	        $this->db->insert('rating_options', $data);
	        $id = $this->db->insert_id();
	        
	        if (is_numeric($id)) {
	            $return = $id;
	        }
	    } elseif ($type == 'update') {
	        $data['modified_on'] = date('Y-m-d H:i:s');
	        $data['modified_by'] = $this->auth->user_id();
	        $data['modified_ip'] = $this->input->ip_address();
	        $return = $this->db->where('id', $id)->update('rating_options', $data);
	    }
	    
	    return $return;
	}
	
	public function test() {
	    $this->ratings_model->selectize_options(1, 0, '1', 1);
	}
}