<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Migration_Install_ratings extends Migration
{
	/**
	 * The name of the database table
	 *
	 * @var String
	 */
	private $table_name = 'ratings';
	private $table_rating_users = 'rating_users';

	/**
	 * The table's fields
	 *
	 * @var Array
	 */
	private $fields = array(
		'id' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'auto_increment' => TRUE
		),
		'module_id' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default'=> 0
		),
		'row_id' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'null' => FALSE,
			'default'=> 0
		),
		'average_rating' => array(
			'type' => 'FLOAT',
			'null' => FALSE,
			'default'=> 0
		),
		'total_ratings' => array(
			'type' => 'FLOAT',
			'null' => FALSE,
			'default'=> 0
		),
		'total_users' => array(
			'type' => 'INT',
			'constraint' => 5,
			'null' => FALSE,
			'default'=> 0
		),
		'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'modified_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
	);
	
	/**
	 * The Rating users table's fields
	 *
	 * @var Array
	 */
	private $rating_users_fields = array(
		'rating_id' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'auto_increment' => TRUE
		),
		'user_id' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'null' => FALSE,
			'default'=> 0
		),
	    'module_id' => array( // for reporting
	        'type' => 'TINYINT',
	        'constraint' => 1,
	        'null' => FALSE,
	        'default'=> 0
	    ),
	    'row_id' => array( // for reporting
	        'type' => 'BIGINT',
	        'constraint' => 20,
	        'null' => FALSE,
	        'default'=> 0
	    ),
	    'rating' => array( // for reporting
	        'type' => 'FLOAT',
	        'null' => FALSE,
	        'default'=> 0
	    ),
		'rated_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'ip_address' => array(
			'type' => 'VARCHAR',
			'constraint' => 50,
			'null' => FALSE
		)
	);
	
	/**
	 * Install this migration
	 *
	 * @return void
	 */
	public function up() {
		// RATINGS TABLE
		$this->dbforge->add_field($this->fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('module_id', false);
		$this->dbforge->create_table($this->table_name);
		$ratings_index_sql = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " ADD UNIQUE INDEX `module` (`module_id`, `row_id`)";
		$this->db->query ($ratings_index_sql);
		
		// RATING USERS TABLE
		$this->dbforge->add_field($this->rating_users_fields);
		$this->dbforge->create_table($this->table_rating_users);
		$index_sql = "ALTER TABLE " . $this->db->dbprefix . $this->table_rating_users . " ADD INDEX `module` (`module_id`, `row_id`)";
		$this->db->query ($index_sql);
		$rating_users_index_sql = "ALTER TABLE " . $this->db->dbprefix . $this->table_rating_users . " ADD UNIQUE INDEX `unique_rating` (`rating_id`, `user_id`, `ip_address`)";
		$this->db->query ($rating_users_index_sql);
	}

	//--------------------------------------------------------------------

	/**
	 * Uninstall this migration
	 *
	 * @return void
	 */
	public function down() {		
		if ($this->db->table_exists($this->table_name)) {
			$this->dbforge->drop_table($this->table_name);
		}
		if ($this->db->table_exists($this->table_rating_users)) {
			$this->dbforge->drop_table($this->table_rating_users);
		}
	}

	//--------------------------------------------------------------------
}