<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class Ratings_model extends BF_Model {

	protected $table_name	= "ratings";
	protected $table_options	= 'rating_options';
	protected $table_relation	= 'rating_relations';
	protected $key			= "id";
	protected $soft_deletes	= FALSE;
	protected $date_format	= "datetime";

	protected $log_user 	= FALSE;

	protected $set_created	= true;
	protected $set_modified = true;
	protected $created_field = "created_on";
	protected $modified_field = "modified_on";

	/*
		Customize the operations of the model without recreating the insert, update,
		etc methods by adding the method names to act as callbacks here.
	 */
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 		= array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	/*
		For performance reasons, you may require your model to NOT return the
		id of the last inserted row as it is a bit of a slow method. This is
		primarily helpful when running big loops over data.
	 */
	protected $return_insert_id 	= TRUE;

	// The default type of element data is returned as.
	protected $return_type 			= "object";

	// Items that are always removed from data arrays prior to
	// any inserts or updates.
	protected $protected_attributes = array();

	/*
		You may need to move certain rules (like required) into the
		$insert_validation_rules array and out of the standard validation array.
		That way it is only required during inserts, not updates which may only
		be updating a portion of the data.
	 */
	protected $insert_validation_rules 	= array();
	protected $skip_validation 			= TRUE;

	//--------------------------------------------------------------------

	/**
	 * Display Rating history to the user
	 * @param number $count
	 * @param string $where
	 * @param string $sort_by
	 * @param number $offset
	 * @param number $limit
	 * @return number
	 */
	public function get_history($count, $rating_id, $where = null, $sort_by = 'id DESC', $offset = 0, $limit = 25) {
		if($count == 0) {
			$sql = "SELECT RU.*, U.display_name, U.email";
		} else {
			$sql = "SELECT COUNT(RU.id) AS TotalCount";
		}
		$sql .= " FROM " . $this->db->dbprefix . "rating_users RU";
		$sql .= " LEFT JOIN " . $this->db->dbprefix . "users U ON U.id = RU.user_id";
		$sql .= " WHERE rating_id = " . $rating_id;  
		if(!empty($where)) {
			$sql .= " AND $where";
		}
			
		if($count == 0) {
			$sql .= " ORDER BY " . $sort_by;
			$sql .= " LIMIT " . $offset . ", " . $limit;
			return $this->db->query ( $sql )->result();
		} else {
			$result = $this->db->query ( $sql )->row();
			return $result ? $result->TotalCount : 0;
		}
	}
	
	/**
	 * Count ratings based on rating value.
	 * @param number $module_id
	 * @param number $row_id
	 */
	public function ratings_count($module_id, $row_id) {
	    // Logic
	    // Available Ratings = Total Ratings - Higher Rating (5)
	    // Rating * 100 / Available Ratings
	    $total_ratings = $this->db->where(array('module_id' => $module_id, 'row_id' => $row_id))->count_all_results('rating_users');;
	    $rating_users = 0;
	    if($total_ratings) {
    	    $values = array(1,2,3,4,5);
    	    $available_ratings = $total_ratings;
    	    foreach($values as $value) {
    	        $gt = $value - 1;	
    	        $where = "module_id = $module_id AND row_id = $row_id AND rating";
    	        if($gt == 0) {
    	            $where .= " < " . $value;
    	        } else {
    	            $where .= " BETWEEN $gt AND " . $value;
    	        }
    	        $rating_bar = 0;
    	        $rating_count = $this->db->where($where)->count_all_results('rating_users');
    	        $rating_users = $rating_users + $rating_count;
    	        if($value == 5) {
    	            $rating_bar = 100;
    	            $available_ratings = $total_ratings - $rating_count;
    	        } else if($available_ratings != 0) {
    	            $rating_bar = ($rating_count * 100) / $available_ratings;
    	        }
    	        $rating_counts[$value] = array('bar' => $rating_bar, 'count' => $rating_count);
    	    }
    	    return array('users' => $rating_users, 'counts' => $rating_counts);
	    } else {
	        return array();
	    }
	}
	
	/**
	 * Update ratings
	 * @param string $module
	 * @param number $row_id
	 */
	public function update_rating($module, $row_id, $result = array()) {
	    $module_id = gc_module_id($module);
	    if(empty($result)) {
	       $result = $this->db->where(array('module_id' => $module_id, 'row_id' => $row_id))->get('ratings')->row();
	    }
	    if($result) {
	        $this->db->where('id', $row_id)->update($module, array('ratings' => $result->average_rating, 'rating_users' => $result->total_users));
	    }
	}
	
	/**
	 * Save Options Relation
	 * When Selectize input is used to input option
	 * Input Format: 1,4,5
	 *
	 * Use this method to allow a user to select existing options activated by the Admin and also allow them to create a new one.
	 *
	 * @param string $module
	 * @param integer $row_id
	 * @param integer $type_id - 0: Ratings, 1: Groups
	 * @param string $options
	 */
	public function selectize_options($module, $row_id, $type_id, $options, $status = 0) {
	    $module_id = gc_module_id($module);
	    // get existing options to compare them with the post submitted value
	    $saved_options = $this->options_comma_separated($module, $row_id, $type_id);
	    // skip below given code if nothing changed
	    if($saved_options && $saved_options == $options) {
	        return false;
	    }
	    $option_ids = array();
	    
	    // get existing options id to update count column
	    $existing_options = $this->get_option_ids($module, $row_id, $type_id);
	    if($existing_options) {
	        foreach($existing_options as $existing_option) {
	            $option_ids[] = $existing_option->option_id;
	        }
	    }
	    
	    // save options information
	    $data = array();
	    if(!empty($options)) {
	        // convert string to an array and trim the whitespace
	        $options = array_map('trim', explode(',', $options));
	        // save only unique keywords
	        $options = array_unique($options);
	        foreach($options as $option) {
	            $option = trim($option);
	            // check whether the tag already exists in the Tag table
	            if(is_numeric($option)) { // The user selected the existing tag or typed a numeric value
	                $option_info = $this->db->select('id')->where('id', $option)->get($this->table_options)->row();
	                if(!$option_info) {
	                    $option_info = $this->db->select('id')->where('name', $option)->get($this->table_options)->row();
	                }
	            } else { // This is a new option added by the user
	                $option = $this->form_validation->rating_option_handling($option); // Remove unwanted characters from a Option
	                $option_info = $this->db->select('id')->where('name', $option)->get($this->table_options)->row();
	                $option = strtoupper($option);
	            }
	            // Get Keyword info if an existing tag, if a new tag, save the info first and then retrieve
	            $result = $this->get_option_info($option_info, $option, $module, $row_id, $type_id, $status);
	            if(!empty($result['data'])) {
	                $data[] = $result['data'];
	            }
	        }
	        if(!empty($data)) {
	            // delete existing options in case user removed one
	            $this->db->where(array('module_id' => $module_id, 'row_id' => $row_id, 'type_id' => $type_id))->delete($this->table_relation);
	            // save options information
	            $this->db->insert_batch($this->table_relation, $data);
	        }
	    } else {
	        // The user may have unselected all options
	        // delete existing options in case user removed all
	        $this->db->where(array('module_id' => $module_id, 'row_id' => $row_id))->delete($this->table_relation);
	    }
	}
	
	/**
	 * Get Keyword Information
	 * @param array $keyword
	 * @param integer/string $tag
	 * @param integer $row_id
	 * @param integer $module_id
	 * @param integer $status - Status of the newly added keyword, can be used in bulk import
	 */
	private function get_option_info($option_info, $option, $module, $row_id, $type_id, $status = 0) {
	    $data = array();
	    $module_id = gc_module_id($module);
	    if($option_info) {
	        $data = array(
	            'module_id' => $module_id,
	            'row_id' => $row_id,
	            'option_id' => $option_info->id,
	            'type_id' => $type_id
	        );
	    } else {
	        // insert option in the options table
	        $option_data = array();
	        $option_data['type_id'] = $type_id;
	        $option_data['name'] = $option;
	        $option_data['status'] = $status;
	        $option_data['created_on'] = date('Y-m-d H:i:s');
	        $option_data['created_by'] = $this->auth->user_id();
	        $option_data['created_ip'] = $this->input->ip_address();
	        $this->db->insert('options', $option_data);
	        $option_id = $this->db->insert_id();
	        if($option_id) {
	            $data = array(
	                'module_id' => $module_id,
	                'row_id' => $row_id,
	                'option_id' => $option_id,
	                'type_id' => $type_id
	            );
	        }
	    }
	    return array('data' => $data);
	}
	
	/**
	 * Tags in comma separated format
	 * Edit Module - Display both active and inactive tags
	 * @param string $module
	 * @param number $row_id
	 * @return string|boolean
	 */
	public function options_comma_separated($module, $row_id, $type_id, $active_only = FALSE) {
	    $saved_options = $this->get_option_relations ($module, $row_id, $type_id, $active_only);
	    $options = array();
	    if($saved_options) {
	        foreach ( $saved_options as $option ) {
	            $options[] = $option->name;
	        }
	        // store them as a string in comma separated version
	        $options = implode( ', ', $options);
	        return $options;
	    }
	    return false;
	}
	
	/**
	 * Get option from the relations table
	 * @param string $module
	 * @param number $row_id
	 */
	public function get_option_relations($module, $row_id, $type_id, $active_only = FALSE) {
	    $module_id = gc_module_id($module);
	    $sql = "SELECT O.id, O.name FROM " . $this->db->dbprefix . $this->table_options . " O";
	    $sql .= " INNER JOIN " . $this->db->dbprefix . $this->table_relation . " ORL ON ORL.option_id = O.id AND ORL.module_id = " . $module_id . " AND ORL.row_id = " . $row_id . " AND ORL.type_id = " . $type_id;
	    if($active_only) {
	        $sql .= " WHERE O.status = 1";
	    }
	    $sql .= " ORDER BY O.id";
	    return $this->db->query($sql)->result();
	}
	
	/**
	 * Get option ids from the relations association table
	 * @param string $module
	 * @param number $row_id
	 */
	public function get_option_ids($module, $row_id, $type_id) {
	    $module_id = gc_module_id($module);
	    $option_ids = $this->db->select('option_id')->group_by('option_id')->get_where($this->table_relation, array('module_id' => $module_id, 'row_id' => $row_id, 'type_id' => $type_id))->result();
	    return $option_ids;
	}
	
	/**
	 * Display saved options on edit page
	 * @param string $module
	 * @param integer $row_id
	 * @param integer $type_id
	 */
	public function saved_options($module, $row_id, $type_id = 0) {
	    $module_id = gc_module_id($module);
	    // Display Existing Tags to the User
	    $result = $this->db->select('GROUP_CONCAT(option_id) AS options')->get_where($this->table_relation, array('module_id' => $module_id, 'row_id' => $row_id, 'type_id' => $type_id))->row();
	    if($result) {
	        return $result->options;
	    }
	    return null;
	}
	
	/**
	 * Purge Option
	 * @param integer $id
	 */
	public function purge_option($id) {
	    $this->db->where('option_id', $id)->delete($this->table_relation);
	    $this->db->where('option_id', $id)->delete($this->table_name);
	    return $this->db->where('id', $id)->delete($this->table_options);
	}
	
	/**
	 * Get Ratings to display on the modules detail page
	 * @param string $module - Name of the module
	 * @param number $row_id - ID of the row
	 */
	public function get_rating($module, $row_id) {
	    $module_id = gc_module_id($module);
	    $result = $this->ratings_model->select('average_rating, total_ratings, total_users')->find_by(array('row_id' => $row_id, 'module_id' => $module_id));
	    if(!empty($result->average_rating)) {
	        $result->message = round($result->average_rating, 2) . ' ' . lang('label_ratings') . ' ' . $result->total_users . ' ' . lang('label_users_rated');
	    }
	    return $result;
	}
}
