<?php defined('BASEPATH') || exit('No direct script access allowed');

/**
 * Reports controller
 */
class Reports extends Admin_Controller
{
    protected $permissionDelete = 'Recently_Viewed.Reports.Delete';
    protected $permissionView   = 'Recently_Viewed.Reports.View';

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
        
        $this->auth->restrict($this->permissionView);
        $this->load->model('recently_viewed/recently_viewed_model');
        $this->lang->load('recently_viewed');
        
            $this->form_validation->set_error_delimiters("<span class='error'>", "</span>");
        
        Template::set_block('sub_nav', 'reports/_sub_nav');

        Assets::add_module_js('recently_viewed', 'recently_viewed.js');
    }

    /**
     * Display a list of Recently Viewed data.
     *
     * @return void
     */
    public function index($filter='listings', $offset = 0)
    {
        // Deleting anything?
        if (isset($_POST['delete'])) {
            $this->gcrud->stats(array('module' => 'recently_viewed'));
            $this->auth->restrict($this->permissionDelete);
            $checked = $this->input->post('checked');
            if (is_array($checked) && count($checked)) {

                // If any of the deletions fail, set the result to false, so
                // failure message is set if any of the attempts fail, not just
                // the last attempt

                $result = true;
                foreach ($checked as $pid) {
                    $deleted = $this->recently_viewed_model->delete($pid);
                    if ($deleted == false) {
                        $result = false;
                    }
                }
                if ($result) {
                    Template::set_message(count($checked) . ' ' . sprintf(lang('msg_action_success'), lang('recently_viewed'), lang('label_deleted')), 'success');
                } else {
                    Template::set_message(sprintf(lang('msg_delete_failure'), lang('recently_viewed')) . $this->recently_viewed_model->error, 'danger');
                }
            }
        }
        // Actions done, now display the view
        $where = "";
        // Filters
        $filter_type = $filter;
        
        $module = $filter_type;
        // must be a valid module
        if(gc_module_id($module) == 0) {
        	Template::redirect(site_url(SITE_AREA . '/reports/recently_viewed'));
        }
        $where .= "RV.module_id = " . gc_module_id($module);
        
        $searchterm = $this->gc->search('recently_viewed', $this->uri->segment(6), '/reports/recently_viewed');
        if(!empty($searchterm)) {
        	$where .= " AND (RV.id LIKE '%". $searchterm ."%' OR U.display_name LIKE '%". $searchterm ."%' OR RV.row_id LIKE '%". $searchterm ."%' OR RV.last_viewed LIKE '%". $searchterm ."%')";
        }
        
        $sort_by = "RV.id DESC";
        $order_by = $this->gc->sort('recently_viewed');
        if(!empty($order_by)) {
        	$sort_by = $order_by;
        }
        
        $records = $this->recently_viewed_model->get_recently_viewed(null, $offset, $this->limit, $module, $where, $sort_by);         
        $total_records = $this->recently_viewed_model->get_recently_viewed(null, $offset, $this->limit, $module, $where, $sort_by, 1);
        
        $pagerUriSegment = 6;
        $pagerBaseUrl = site_url(SITE_AREA . '/reports/recently_viewed/index/' . $filter) . '/';
        
        $limit  = $this->limit;

        $this->load->library('pagination');
        $pager['base_url']    = $pagerBaseUrl;
        $pager['total_rows']  = $total_records;
        $pager['per_page']    = $limit;
        $pager['uri_segment'] = $pagerUriSegment;

        $this->pagination->initialize($pager);
        Template::set('module', $module);
        Template::set('index_url', site_url(SITE_AREA .'/reports/recently_viewed/index/'));
        Template::set('filter_type', $filter_type);

        Template::set('records', $records);
        
    	Template::set('toolbar_title', lang('recently_viewed_manage'));

        Template::render();
    }
    
    /**
     * Create a Recently Viewed object.
     *
     * @return void
     */
    public function create() {
        $this->auth->restrict($this->permissionCreate);        
        if (isset($_POST['save'])) {
            $this->gcrud->stats(array('module' => 'recently_viewed'));
            if ($insert_id = $this->save_recently_viewed()) {
                log_activity($this->auth->user_id(), lang('recently_viewed_act_create_record') . ': ' . $insert_id . ' : ' . $this->input->ip_address(), 'recently_viewed');
                Template::set_message(lang('recently_viewed_create_success'), 'success');
                redirect(SITE_AREA . '/reports/recently_viewed');
            }

            // Not validation error
            if ( ! empty($this->recently_viewed_model->error)) {
                Template::set_message(lang('recently_viewed_create_failure') . $this->recently_viewed_model->error, 'danger');
            }
        }
        Template::set('toolbar_title', lang('recently_viewed_action_create'));
        Template::render();
    }
    /**
     * Allows editing of Recently Viewed data.
     *
     * @return void
     */
    public function edit()
    {
        $id = $this->uri->segment(5);
        if (empty($id)) {
            Template::set_message(lang('recently_viewed_invalid_id'), 'error');

            redirect(SITE_AREA . '/reports/recently_viewed');
        }
        
        if (isset($_POST['save'])) {
            $this->gcrud->stats(array('module' => 'recently_viewed'));
            $this->auth->restrict($this->permissionEdit);

            if ($this->save_recently_viewed('update', $id)) {
                log_activity($this->auth->user_id(), lang('recently_viewed_act_edit_record') . ': ' . $id . ' : ' . $this->input->ip_address(), 'recently_viewed');
                Template::set_message(lang('recently_viewed_edit_success'), 'success');
                redirect(SITE_AREA . '/reports/recently_viewed');
            }

            // Not validation error
            if ( ! empty($this->recently_viewed_model->error)) {
                Template::set_message(lang('recently_viewed_edit_failure') . $this->recently_viewed_model->error, 'danger');
            }
        } elseif (isset($_POST['delete'])) {
            $this->gcrud->stats(array('module' => 'recently_viewed'));
            $this->auth->restrict($this->permissionDelete);

            if ($this->recently_viewed_model->delete($id)) {
                log_activity($this->auth->user_id(), lang('recently_viewed_act_delete_record') . ': ' . $id . ' : ' . $this->input->ip_address(), 'recently_viewed');
                Template::set_message(lang('recently_viewed_delete_success'), 'success');

                redirect(SITE_AREA . '/reports/recently_viewed');
            }

            Template::set_message(lang('recently_viewed_delete_failure') . $this->recently_viewed_model->error, 'danger');
        }
        
        Template::set('recently_viewed', $this->recently_viewed_model->find($id));

        Template::set('toolbar_title', lang('recently_viewed_edit_heading'));
        Template::render();
    }

    //--------------------------------------------------------------------------
    // !PRIVATE METHODS
    //--------------------------------------------------------------------------

    /**
     * Save the data.
     *
     * @param string $type Either 'insert' or 'update'.
     * @param int    $id   The ID of the record to update, ignored on inserts.
     *
     * @return boolean|integer An ID for successful inserts, true for successful
     * updates, else false.
     */
    private function save_recently_viewed($type = 'insert', $id = 0)
    {
        if ($type == 'update') {
            $_POST['id'] = $id;
        }

        // Validate the data
        $this->form_validation->set_rules($this->recently_viewed_model->get_validation_rules());
        if ($this->form_validation->run() === false) {
            return false;
        }

        // Make sure we only pass in the fields we want
        
        $data = $this->recently_viewed_model->prep_data($this->input->post());

        // Additional handling for default values should be added below,
        // or in the model's prep_data() method
        

        $return = false;
        if ($type == 'insert') {
            $id = $this->recently_viewed_model->insert($data);

            if (is_numeric($id)) {
                $return = $id;
            }
        } elseif ($type == 'update') {
            $return = $this->recently_viewed_model->update($id, $data);
        }

        return $return;
    }
    
    /**
     * Save recently viewed settings
     */
    public function settings() {
		clear_cache('site_settings'); // Clear Site Settings Cache
    	$this->load->model('settings/settings_model');
    	if (isset($_POST['save'])) {
    	    $this->gcrud->stats(array('module' => 'recently_viewed'));
    		$this->form_validation->set_rules('user_id', 'lang:label_user', 'trim');
    		$this->form_validation->set_rules('ip_address', 'lang:label_ip_address', 'trim');
    		if($this->form_validation->run() == TRUE) {
    			$setting = array(
    				'user_id' => $this->input->post('user_id') ? 1 : 0,
    				'ip_address' => $this->input->post('ip_address') ? 1 : 0
    			);
    			$data = array (array('name' => 'site.recently_viewed', 'value' => json_encode($settings)));
    			$updated = $this->settings_model->update_batch($data, 'name');    
    			if ($updated) {
    				Template::set_message(lang('settings_saved_success'), 'success');
    			} else {
    				Template::set_message(lang('settings_error_success'), 'error');
    			}
    		}
    	}
		$setting_name = 'site.recently_viewed';
    	$result = $this->db->select('value')->get_where('settings', array('name' => $setting_name))->row();
    	if($result) {
    	    $settings = json_decode($result->value ?? '');
    	} else {
    	    $settings = json_decode(settings_item($setting_name) ?? '');
    	}
    	Template::set('record', $settings);
    	Template::set('toolbar_title', lang('recently_viewed_settings'));
    	Template::render();
    }
    
    /**
     * Delete Records from Listing Views Table
     */
    public function delete_records() {
    	// DATE TIME PICKER
    	Assets::add_js('moment.js');
    	Assets::add_css ( 'bootstrap-datetimepicker.min.css' );
    	Assets::add_js ( 'bootstrap-datetimepicker.min.js' );
    	$this->auth->restrict($this->permissionDelete);
    	if (isset($_POST['submit'])) {
    		$this->form_validation->set_rules('date_from', 'lang:label_from', 'trim|valid_date|required');
    		if ($this->form_validation->run() === true) {
    			// DELETE FROM `listing_views` WHERE date(modified_on) < '2015-11-19'
    			$result = $this->db->where('date(last_viewed) < ', $this->input->post('date_from'))->count_all_results('recently_viewed');
    			if($result == 0) {
    				Template::set_message(lang('error_no_record_found'), 'error');
    			} else {
    				$this->db->delete('recently_viewed', array('date(last_viewed) < ' => $this->input->post('date_from')));
    				Template::set_message ( $result . ' ' . lang ( 'delete_success' ), 'success' );
    			}
    		}
    	}
    	Template::set('toolbar_title', lang('recently_viewed_delete'));
    	Template::render();
    }
}