<?php  if (!defined('BASEPATH')) exit('No direct script access allowed');

class Migration_Install_recently_viewed extends Migration
{
	/**
	 * The name of the database table
	 *
	 * @var String
	 */
	private $table_name = 'recently_viewed';
	
	/**
	 * The Recently Viewed table's fields
	 *
	 * @var Array
	 */
	private $fields = array(
			'id' => array(
					'type' => 'BIGINT',
					'constraint' => 20,
					'auto_increment' => TRUE,
			),
			'user_id' => array(
					'type' => 'BIGINT',
					'constraint' => 20,
					'null' => FALSE,
					'default' => 0
			),
			'module_id' => array(
					'type' => 'TINYINT',
					'constraint' => 1,
					'null' => FALSE,
					'default' => 0
			),
			'row_id' => array(
					'type' => 'BIGINT',
					'constraint' => 20,
					'null' => FALSE,
			),
			'counts' => array(
					'type' => 'INT',
					'constraint' => 11,
					'null' => FALSE,
					'default' => 0
			),		
			'last_viewed DATETIME NULL DEFAULT CURRENT_TIMESTAMP',		
			'ip_address' => array(
					'type' => 'VARCHAR',
					'constraint' => 50,
					'null' => FALSE,
			),
	);
	
	/**
	 * Install this migration
	 *
	 * @return void
	 */
	public function up() {
		$this->dbforge->add_field($this->fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('user_id',false);
		$this->dbforge->add_key('module_id',false);
		$this->dbforge->add_key('row_id',false);
		$this->dbforge->add_key('ip_address',false);
		$this->dbforge->add_key('last_viewed',false);	
		$this->dbforge->create_table($this->table_name);
		
		// Add Unique Index
		$index_sql = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " ADD UNIQUE `unique_view` (`user_id`, `module_id`, `row_id`, `ip_address`)";
		$this->db->query ($index_sql);
	}
	
	/**
	 * Uninstall this migration
	 *
	 * @return void
	 */
	public function down() {
		if ($this->db->table_exists($this->table_name)) {
			$this->dbforge->drop_table($this->table_name);
		}
	}
}