<?php defined('BASEPATH') || exit('No direct script access allowed');

class Recently_viewed_model extends BF_Model
{
    protected $table_name	= 'recently_viewed';
	protected $key			= 'id';
	protected $date_format	= 'datetime';

	protected $log_user 	= false;
	protected $set_created	= true;
	protected $set_modified = false;
	protected $soft_deletes	= false;
	
	protected $created_field = "created_on";


	// Customize the operations of the model without recreating the insert,
    // update, etc. methods by adding the method names to act as callbacks here.
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 	    = array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	// For performance reasons, you may require your model to NOT return the id
	// of the last inserted row as it is a bit of a slow method. This is
    // primarily helpful when running big loops over data.
	protected $return_insert_id = true;

	// The default type for returned row data.
	protected $return_type = 'object';

	// Items that are always removed from data prior to inserts or updates.
	protected $protected_attributes = array();

	// You may need to move certain rules (like required) into the
	// $insert_validation_rules array and out of the standard validation array.
	// That way it is only required during inserts, not updates which may only
	// be updating a portion of the data.
	protected $validation_rules 		= array(
		array(
			'field' => 'user_id',
			'label' => 'lang:recently_viewed_field_user_id',
			'rules' => 'max_length[20]',
		),
		array(
			'field' => 'row_id',
			'label' => 'lang:recently_viewed_field_row_id',
			'rules' => 'max_length[11]',
		),
		array(
			'field' => 'module_id',
			'label' => 'lang:recently_viewed_field_module_id',
			'rules' => 'max_length[40]',
		),
	);
	protected $insert_validation_rules  = array();
	protected $skip_validation 			= false;

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }
    
    /**
     * Display Recently Viewed to the user
     * @param int $user_id
     * @param int $offset
     * @param int $limit
     * @param int $count
     * @return object
     */
    public function get_recently_viewed($user_id, $offset, $limit, $module, $where, $sort_by, $count = 0) {
    	if($module == 'listings' && $count == 0) {
    		$sql = "SELECT RV.*, U.display_name AS user, L.title, L.slug, L.address, L.postcode, C.name AS city, S.name AS state, CS.name AS country, L.description, L.modified_on, L.status, L.hits";
    	} else if($module != 'listings' && $count == 0) {
    		$sql = "SELECT RV.*, U.display_name AS user,";
    		switch($module) {
    			case 'pages':
    				$sql .= " M.title, M.body AS description, M.slug, M.hits, M.status";
    				break;
    			case 'categories':
    				$sql .= " M.name AS title, M.description, M.slug, M.counts AS hits, M.active AS status";
    				break;
    			case 'cities':
    				$sql .= " M.name AS title, M.description, M.slug, M.hits, M.active AS status";
    				break;
    			default:
    				$sql .= " M.title, M.description, M.slug, M.hits, M.status";
    		}
    		
    	} else {
    		$sql = "SELECT COUNT(RV.id) AS TotalCount";
    	}
    	$sql .= " FROM " . $this->db->dbprefix . "recently_viewed RV";
    	$sql .= " LEFT JOIN " . $this->db->dbprefix . "users U ON U.id = RV.user_id";
    	if($module == 'listings' && $count == 0) {
	    	$sql .= " INNER JOIN " . $this->db->dbprefix . "listings L ON L.id = RV.row_id AND L.deleted = 0";
	    	$sql .= " INNER JOIN " . $this->db->dbprefix . "cities C ON C.id = L.city_id";
	    	$sql .= " INNER JOIN " . $this->db->dbprefix . "states S ON S.id = L.state_id";
	    	$sql .= " INNER JOIN " . $this->db->dbprefix . "countries CS ON CS.iso = L.country_iso";
    	} else if($count == 0)  {
    		$sql .= " INNER JOIN " . $this->db->dbprefix . "$module M ON M.id = RV.row_id";
    	}
    	
    	$sql .= " WHERE RV.module_id = " . gc_module_id($module); 
    	if(!empty($where)) {
    		$sql .= " AND $where";
    	}
    	
    	if($user_id != null) {
    		$sql .= " AND RV.user_id = $user_id";
    	}
    	
    	if($count == 0) {
    		$sql .= " ORDER BY " . $sort_by;
    		$sql .= " LIMIT " . $offset . ", " . $limit;
    		return $this->db->query ( $sql )->result();
    	} else {
    		$result = $this->db->query ( $sql )->row();
    		return $result ? $result->TotalCount : 0;
    	}
    }
}