<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Reports controller
 */
class Reports extends Admin_Controller {
	protected $permissionDelete = 'Search_Queries.Reports.Delete';
	protected $permissionView   = 'Search_Queries.Reports.View';
	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct()
	{
		parent::__construct();

		$this->auth->restrict($this->permissionView);
		$this->load->model('search_queries_model', null, true);
		$this->lang->load('search_queries');
		
		Template::set_block('sub_nav', 'reports/_sub_nav');

		Assets::add_module_js('search_queries', 'search_queries.js');
	}
	
	/**
	 * Displays a list of form data.
	 *
	 * @return void
	 */
	public function index($offset = 0) {
		// Deleting anything?
	    if (isset($_POST['delete'])) {
	        $this->gcrud->stats(array('module' => 'search_queries'));
			$checked = $this->input->post('checked');
			if (is_array ( $checked ) && count ( $checked )) {
				$result = FALSE;
				foreach ( $checked as $pid ) {
					$result = $this->search_queries_model->delete ( $pid );
				}
				
				if ($result) {
					Template::set_message ( count ( $checked ) . ' ' . sprintf(lang('msg_delete_success'), lang('search_query')), 'success' );
				} else {
					Template::set_message ( sprintf(lang('msg_delete_failure'), lang('search_query')) . $this->search_queries_model->error, 'danger' );
				}
			}
		}
		$where = "";
		$searchterm = $this->gc->search('search_queries', $this->uri->segment(6), '/reports/search_queries');
		if(!empty($searchterm)) {
			$where .= "location LIKE '%" . $searchterm . "%' OR keyword LIKE '%" . $searchterm . "%' OR times LIKE '%" . $searchterm . "%' OR created_on LIKE '%" . $searchterm . "%' OR modified_on LIKE '%" . $searchterm . "%'";
		}
		
		$sort_by = "modified_on DESC";
		$order_by = $this->gc->sort('search_queries');
		if(!empty($order_by)) {
			$sort_by = $order_by;
		}
		
		$records = $this->search_queries_model->get_search_queries($offset, $this->limit, $where, $sort_by);		
		$total_records = $this->search_queries_model->get_search_queries($offset, $this->limit, $where, $sort_by, 1);
		
		$this->load->library('pagination');
		$pager['base_url'] = site_url(SITE_AREA ."/reports/search_queries/index/");
		$pager['total_rows'] = $total_records;
		$pager['per_page'] = $this->limit;
		$pager['uri_segment']	= 5;
		
		$this->pagination->initialize($pager);

		Template::set('records', $records);
		Template::set('toolbar_title', lang('sq_manage'));
		Template::render();
	}
	
	/**
	 * Advanced Search
	 * Useful to find out the location and keyword searches within a specified date range.
	 * @param number $offset
	 */
	public function advanced_search($offset = 0) {
		// DATE TIME PICKER
		Assets::add_js('moment.js');
		Assets::add_css ( 'bootstrap-datetimepicker.min.css' );
		Assets::add_js ( 'bootstrap-datetimepicker.min.js' );
		if(isset($_POST['search']) || ($this->session->userdata('sq_from') && $this->session->userdata('sq_to'))) {
		    if (isset($_POST['search'])) {
		        $this->gcrud->stats(array('module' => 'search_queries'));
				$this->form_validation->set_rules('location', 'lang:label_location', 'trim');
				$this->form_validation->set_rules('keyword', 'lang:label_keyword', 'trim');
				$this->form_validation->set_rules('date_from', 'lang:label_from', 'trim|valid_date|required');
				$this->form_validation->set_rules('date_to', 'lang:label_to', 'trim|valid_date|compare_dates[date_from]|required');
					
				if ($this->form_validation->run() === true) {
					$location = $this->input->post('location');
					$keyword = $this->input->post('keyword');
					$from = $this->input->post('date_from');
					$to = $this->input->post('date_to');
					// Save values in Session for Pagination
					$this->session->set_userdata('sq_location', $location);
					$this->session->set_userdata('sq_keyword', $keyword);
					$this->session->set_userdata('sq_from', $from);
					$this->session->set_userdata('sq_to', $to);
					// IF USER PERFORMED SEARCH FROM PAGINATION PAGE
					if($this->uri->segment(5)) {
						Template::redirect(site_url(SITE_AREA . "/reports/search_queries/advanced_search"));
					}
				}
			} else if($this->session->userdata('sq_from') && $this->session->userdata('sq_to')) {
				$location = $this->session->userdata('sq_location');
				$keyword = $this->session->userdata('sq_keyword');
				$from = $this->session->userdata('sq_from');
				$to = $this->session->userdata('sq_to');
			}
			if(!empty($from) && !empty($to)) {
				$records = $this->search_queries_model->advanced_search_queries($offset, $this->limit, $keyword, $location, $from, $to);
					
				$total_records = $this->search_queries_model->advanced_search_queries($offset, $this->limit, $keyword, $location, $from, $to, 1);
				
				if($total_records == 0) {
					Template::set_message(lang('error_no_record_found'), 'error');
				}
				
				$this->load->library('pagination');
				$pager['base_url'] = site_url(SITE_AREA ."/reports/search_queries/advanced_search/");
				$pager['total_rows'] = $total_records;
				$pager['per_page'] = $this->limit;
				$pager['uri_segment']	= 5;
					
				$this->pagination->initialize($pager);
				Template::set(array(
					'location' => $location,
					'keyword' => $keyword,
					'from' => $from,
					'to' => $to
				));
					
				Template::set('records', $records);
			}
		}
		Template::set('toolbar_title', lang('advanced_search'));
		Template::render();
	}
}