<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Search_queries_model extends BF_Model {

	protected $table_name	= "search_queries";
	protected $key			= "id";
	protected $soft_deletes	= false;
	protected $date_format	= "datetime";

	protected $log_user 	= FALSE;

	protected $set_created	= true;
	protected $set_modified = true;
	protected $created_field = "created_on";
	protected $modified_field = "modified_on";

	/*
		Customize the operations of the model without recreating the insert, update,
		etc methods by adding the method names to act as callbacks here.
	 */
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 		= array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	/*
		For performance reasons, you may require your model to NOT return the
		id of the last inserted row as it is a bit of a slow method. This is
		primarily helpful when running big loops over data.
	 */
	protected $return_insert_id 	= TRUE;

	// The default type of element data is returned as.
	protected $return_type 			= "object";

	// Items that are always removed from data arrays prior to
	// any inserts or updates.
	protected $protected_attributes = array();
	protected $insert_validation_rules 	= array();
	protected $skip_validation 			= TRUE;

	//--------------------------------------------------------------------
	/**
	 * Get search queries using date in descending order
	 * @param int $limit
	 * @return object
	 */
	public function get_search_queries($offset, $limit, $where, $sort_by, $count = 0) {
		if($count == 0) {
			$sql = "SELECT *";
		} else {
			$sql = "SELECT COUNT(id) AS TotalCount";
		}
		$sql .= " FROM " . $this->db->dbprefix . "search_queries";
		if(!empty($where)) {
			$sql .= " WHERE " . $where;
		}
		if($count == 0) {
			$sql .= " ORDER BY " . $sort_by;
			$sql .= " LIMIT " . $offset . ", " . $limit;
			return $this->db->query ( $sql )->result();
		} else {
			$result = $this->db->query ( $sql )->row();
			return $result ? $result->TotalCount : 0;
		}
	}
	
	/**
	 * Advanced Search
	 * @param number $offset
	 * @param number $limit
	 * @param number $listing_id
	 * @param string $keyword
	 * @param string $page
	 * @param date $from
	 * @param date $to
	 * @param number $count
	 * @return number
	 */
	public function advanced_search_queries($offset, $limit, $keyword, $location, $from, $to, $count = 0) {
		if($count == 0) {
			$sql = "SELECT *";
		} else {
			$sql = "SELECT COUNT(id) AS TotalCount";
		}
		$sql .= " FROM " . $this->db->dbprefix . "search_queries";
		$sql .= " WHERE ((date(modified_on) BETWEEN '$from' AND '$to') OR modified_on = '0000-00-00 00:00:00' OR modified_on IS NULL)";
		if(!empty($keyword)) {
			$sql .= " AND keyword LIKE '%" . $keyword . "%'";
		}
		if(!empty($location)) {
			$sql .= " AND location LIKE '%" . $location . "%'";
		}
		if($count == 0) {
			$sql .= " ORDER BY modified_on desc";
			$sql .= " LIMIT " . $offset . ", " . $limit;
			return $this->db->query ( $sql )->result();
		} else {
			$result = $this->db->query ( $sql )->row();
			return $result ? $result->TotalCount : 0;
		}
	}

}
