<?php defined('BASEPATH') || exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Online Software Solutions							   # ||
|| # Copyright (C) 2021 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Components controller
 */
class Components extends Admin_Controller {
    protected $permissionCreate = 'SEO.Components.Create';
    protected $permissionDelete = 'SEO.Components.Delete';
    protected $permissionEdit   = 'SEO.Components.Edit';
    protected $permissionView   = 'SEO.Components.View';
    protected $context			= 'components';
    protected $moduleName       = 'seo';
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
        
        $this->auth->restrict($this->permissionView);
        clear_cache('seo_links');
        $this->load->model('seo/seo_model');
        $this->lang->load('seo');
        
      	$this->form_validation->set_error_delimiters("<span class='error'>", "</span>");        
        Template::set_block('sub_nav', 'components/_sub_nav');        
        Assets::add_module_js('seo', 'seo.js');
    }
    
    /**
     * Display a list of SEO data.
     *
     * @return void
     */
    public function index($filter = 'all', $offset = 0) {
        $this->auth->restrict('SEO.Components.Summary');
        $record = $this->seo_model->summary();        
        $link = $this->db->where('is_default', 1)->get('seo_analysis')->row();
        if($link) {
            $url = $link->domain; // '//www.' . 
        } else {
            $url = site_url(); //'https://www.goclixy.com';
        }
        $site_url = $url; // $this->seo_model->get_domain($url);
        $stats = null;
        if($site_url) {
            $row = $this->db->where('domain', $site_url)->get('seo_analysis')->row();
            if(!$row) {            
                $this->load->library('authenticate');
                $stats = $this->authenticate->seo_stats($site_url);
                if(!empty($stats)) {
                    $this->db->insert('seo_analysis', array('domain' => $site_url, 'stats' => json_encode($stats), 'created_on' => date('Y-m-d H:i:s')));
                }
            } else {
                $stats = json_decode($row->stats ?? '');
            }
        }
        $record->stats = $stats;
        $record->top_visited_urls = $this->db->order_by('hits', 'DESC')->limit(20)->get('seo_links')->result();
        $record->last_modified_urls = $this->db->order_by('modified_on', 'DESC')->limit(20)->get('seo_links')->result();
        Template::set(array(
            'toolbar_title' => lang('gc_menu_summary'),
            'record' => $record,
            'domain' => $site_url
        ));
        Template::render();
    }
    
    /**
     * Display a list of SEO data
     * @param string $module
     * @param string $filter
     * @param number $offset
     */
    public function manage($module = 'backlinks', $filter = 'all', $offset = 0) {
        switch($module) {
            case 'keywords':
            case 'links':
            case 'analysis':
            case 'error_links':
            case 'error_link_referrers':
            case 'text':
                if(isset($_POST['refresh'])) { // Recrawl the link
                    $record = $this->db->where('id', (int) $_POST['refresh'])->get('seo_links')->row();
                    if($record) {
                        $this->seo_model->update_link($record);                        
                        Template::set_message(lang('msg_success'), 'success');
                        Template::redirect(current_url());
                    } else {
                        Template::set_message(lang('msg_operation_failure'), 'danger');
                    }
                }                
                // Change the Sitemap Status
                if (isset($_POST['sitemap'])) {
                    $this->gc->status_change('seo_links', $_POST['sitemap'], 'sitemap');
                }
                if($module == 'error_links' || $module == 'error_link_referrers') {
                    $this->seo_model->process_error_links();
                }
                $extra = array(
                    'model' => 'seo',
                    'context' => 'components',
                    'fields' => $module,
                    'prefix' => 'seo'
                );
                $this->gc->module_view($module, $filter, $offset, 'admin', $extra);
                break;
            default:
                $this->seo_model->view($module, $filter, $offset);
                break;
        }      
    }
    
    /**
     * Add/Edit SEO module Data
     *
     * @return void
     */
    public function form($module = 'backlinks', $action = 'create') {
        if($module == 'backlink_emails') {            
            // user search js and css dependencies
            Assets::add_css('selectize.bootstrap3.css');
            Assets::add_js('selectize.min.js');
            Assets::add_js('selectize-search.js');
            Assets::add_js('location-selection.js');
        }
        if($module == 'text') {
            Assets::add_js(array(
                'tinymce/tinymce.min.js',
            ));	
        }
        $params = array(
            'model' => 'seo',
            'context' => $this->context,
            'fields_method' => $module,
            'set_table' => true,
            'uri_segment' => 7,
            'redirect_url' => site_url(SITE_AREA . '/' . $this->context . '/' . $this->moduleName . '/manage/' . $module)
        );
        if($module == 'backlinks') {
            // save domain name
            $params['method'] = 'after_backlink_save';
        }
        if($module == 'backlink_emails') {
            // save password salt in an encrypted string for security reasons
            $params['method'] = 'after_backlink_email_save';
        }        
        $this->gcrud->form('seo_' . $module, $action, 'admin', $params);
    }
    
    /**
     * Save settings
     */
    public function settings() {
        clear_cache('site_settings'); // Clear Site Settings Cache
        $this->load->model('settings/settings_model');
        $setting_name = 'seo.settings';
        $result = $this->db->select('value')->get_where('settings', array('name' => $setting_name))->row();
        if($result) {
            $settings = json_decode($result->value ?? '');
        } else {
            $settings = json_decode(settings_item($setting_name) ?? '');
        }
        if (isset($_POST['save'])) {
            $this->gcrud->stats(array('module' => 'seo'));
            $this->form_validation->set_rules('display_site_title', 'lang:seo_display_site_title', 'trim');
            $this->form_validation->set_rules('h1_logo', 'lang:seo_h1_logo', 'trim');
            $this->form_validation->set_rules('h1_use_page_meta', 'lang:seo_use_page_meta', 'trim');
            $this->form_validation->set_rules('h1_title', 'lang:seo_h1_title', 'trim');
            $this->form_validation->set_rules('h1_keywords', 'lang:seo_h1_keywords', 'trim');
            $this->form_validation->set_rules('status', 'lang:label_status', 'trim');
            $this->form_validation->set_rules('client_id', 'lang:seo_client_id', 'trim');
            $this->form_validation->set_rules('service_account', 'lang:seo_service_account', 'trim');
            $this->form_validation->set_rules('key_file', 'lang:seo_key_file', 'trim');
            $this->form_validation->set_rules('application_name', 'lang:seo_application_name', 'trim');
            $this->form_validation->set_rules('profile_id', 'lang:seo_profile_id', 'trim');
            $this->form_validation->set_rules('rows_limit', 'lang:seo_rows_limit', 'trim|is_numeric|required');
            if($this->form_validation->run() == TRUE) {                
                // upload analytics key file
                if (isset($_FILES['key_file']) && is_uploaded_file($_FILES['key_file']['tmp_name'])) {
                    $file_data = $this->upload_key_file(); // upload key file
                    $key_file = isset($file_data['upload_data']['file_name']) ? $file_data['upload_data']['file_name'] : ''; // retrieve file name of uploaded file
                } else {
                    $key_file = $settings->analytics->key_file; // retrieve old file name and update database
                }
                $settings = array(
                    'display_site_title' => $this->input->post('display_site_title') ? 1 : 0,
                    'h1' => array(
                        'logo' => $this->input->post('h1_logo') ? 1 : 0,
                        'use_page_meta' => $this->input->post('h1_use_page_meta') ? 1 : 0,
                        'title' => $this->input->post('h1_title'),
                        'keywords' => $this->input->post('h1_keywords')
                    ),
                    'site' => array(
                        'meta_title' => $this->input->post('meta_title'),
                        'meta_description' => $this->input->post('meta_description'),
                        'meta_keywords' => $this->input->post('meta_keywords'),
                    ),
                    'analytics' => array(
                        'status' => $this->input->post('status') ? 1 : 0,
                        'client_id' => $this->input->post('client_id'),
                        'service_account' => $this->input->post('service_account'),
                        'key_file' => $key_file,
                        'application_name' => $this->input->post('application_name'),
                        'profile_id' => $this->input->post('profile_id'),
                        'rows_limit' => $this->input->post('rows_limit'),
                    )
                );
                $data = array (array('name' => $setting_name, 'value' => json_encode($settings)));
                $updated = $this->settings_model->update_batch($data, 'name');
                if ($updated) {
                    Template::set_message(sprintf(lang('msg_save_success'), lang('gc_menu_settings')), 'success');
                    // Template::redirect(current_url());
                } else {
                    Template::set_message(sprintf(lang('msg_edit_failure'), lang('gc_menu_settings')), 'danger');
                }
            }
        }
        Template::set(array(
            'toolbar_title' => lang('gc_menu_settings'),
            'settings' => $settings
        ));
        Template::render();
    }
    
    /**
     * Save analytics key
     * @return multitype:NULL
     */
    private function upload_key_file() {    
		if(!gc_demo_mode(array('no_message' => true, 'no_redirect' => true))) {
            return false;
        }	
        $data = array();
        $config['upload_path'] = realpath(APPPATH.'./third_party/Google/'); //Make SURE that you chmod this directory to 777!
        $config['allowed_types'] = 'p12';
        $config['max_size']= 100;        
        $this->load->library('upload', $config);        
        if (!$this->upload->do_upload('key_file')) { // Image name is necessary
            Template::set_message($this->upload->display_errors(), 'danger'); // Show errors to the user
            // $this->session->set_flashdata('upload_file_error', $this->upload->display_errors());
            // return false;
            Template::redirect(current_url());
        } else {
            $data = array('upload_data' => $this->upload->data());
        }
        return $data;
    }
    
    /**
     * Generate Sitemap
     */
    public function sitemap() {
        $this->seo_model->generate_sitemap();
        Template::set_message(lang('seo_sitemap_success') . ' ' . lang('label_url') . ': ' . anchor(site_url('sitemap.xml'), site_url('sitemap.xml'), 'target="_blank"'), 'success');
        Template::redirect(site_url(SITE_AREA . '/components/seo'));
    }
    
    /**
     * View Domain Stats
     * @param number $id
     */
    public function stats($id) {
        $record = $this->db->where('id', (int) $id)->get('seo_analysis')->row();
        if(!$record) {
            Template::set_message(lang('invalid_id'), 'danger');
            Template::redirect(site_url(SITE_AREA . '/components/seo'));
        }
        if(isset($_POST['stats_pdf'])) {
            $this->gcrud->stats(array('module' => 'seo'));
            $this->load->library('slug');
            $content = json_decode($record->stats ?? '', true);
            $content = $this->load->view('components/stats', array('stats' => $content, 'pdf' => true), true);
            $this->gc->create_pdf($this->slug->create_slug($this->seo_model->get_domain($record->domain)), $content);
        }
        if(isset($_POST['refresh'])) { // Refresh the stats
            $this->gcrud->stats(array('module' => 'seo'));
            $this->load->library('authenticate');
            $stats = $this->authenticate->seo_stats($record->domain);
            if(!empty($stats)) {
                $this->db->where('id', $id)->update('seo_analysis', array('stats' => json_encode($stats), 'modified_on' => date('Y-m-d H:i:s')));
            }
            Template::redirect(current_url());
        }
        $domain = $record->domain;
        if(empty($record->stats)) {
            $this->gcrud->stats(array('module' => 'seo'));
            $this->load->library('authenticate');
            $stats = $this->authenticate->seo_stats($record->domain);
            if(!empty($stats)) {
                $this->db->where('id', $id)->update('seo_analysis', array('stats' => json_encode($stats), 'modified_on' => date('Y-m-d H:i:s')));
            }
        } else {
            $stats = json_decode($record->stats ?? '', true);
        }
        Template::set(array(
            'stats' => $stats,
            'domain' => $domain,
            'pdf' => false,
            'toolbar_title' => lang('seo_stats') . ' - ' . $domain
        ));
        Template::render();
    }
    
    /**
     * Edit SEO Link
     * @param number $id
     */
    public function edit($id) {
        clear_cache('seo_links');        
        $record = $this->db->where('id', (int) $id)->get('seo_links')->row();
        // $site_url = 'https://www.goclixy.com';
        if(!$record) {
            Template::set_message(lang('invalid_id'), 'danger');
            Template::redirect(site_url(SITE_AREA . '/components/seo/manage/links'));
        }
        if(isset($_POST['refresh'])) { // Recrawl the link
            $this->gcrud->stats(array('module' => 'seo'));
            $record = $this->seo_model->update_link($record);
            Template::set_message(lang('msg_success'), 'success');
        }  
        if(isset($_POST['pagespeed'])) {  // Get Pagespeed information from Google      
            $this->gcrud->stats(array('module' => 'seo'));
            $url = site_url($record->uri_string); // e.g. 'https://www.goclixy.com';
            $data = $this->seo_model->pagespeed(urlencode($url));
            if(isset($data->score)) {
                $this->db->where('id', $record->id)->update('seo_links', array('pagespeed_score' => $data->score, 'pagespeed' => json_encode($data)));
                Template::set_message(lang('msg_success'), 'success');
                Template::redirect(current_url());
            } else {
                Template::set_message(lang('msg_operation_failure'), 'danger');
            }
        }
        if(isset($_POST['save'])) {
            $this->gcrud->stats(array('module' => 'seo'));
            $_POST['id'] = $id;
            $this->form_validation->set_rules('uri_string', 'lang:seo_uri_string', 'trim|unique[seo_links.uri_string,seo_links.id]');
            $this->form_validation->set_rules('title', 'lang:label_title', 'trim');
            $this->form_validation->set_rules('description', 'lang:label_description', 'trim');
            $this->form_validation->set_rules('keywords', 'lang:label_keywords', 'trim');
            $this->form_validation->set_rules('sitemap_freq', 'lang:seo_sitemap_freq', 'trim');
            $this->form_validation->set_rules('sitemap_priority', 'lang:seo_sitemap_priority', 'trim');
            if ($this->form_validation->run() === true) {
                $data = array(
                    'uri_string' => $this->input->post('uri_string'),
                    'title' => $this->input->post('title'),
                    'description' => $this->input->post('description'),
                    'keywords' => $this->input->post('keywords'),
                    'sitemap_freq' => $this->input->post('sitemap_freq'),
                    'sitemap_priority' => $this->input->post('sitemap_priority'),
                    'modified_on' => date('Y-m-d H:i:s'),
                    'modified_ip' => $this->input->ip_address()
                );
                if($this->db->where('id', $id)->update('seo_links', $data)) {
                    Template::set_message(sprintf(lang('msg_save_success'), lang('singular_seo_links')), 'success');
                    Template::redirect(current_url());
                } else {
                    Template::set_message(sprintf(lang('msg_create_failure'), lang('singular_seo_links')), 'danger');
                }
            }
        }
        if(isset($_POST['refresh'])) {
            $this->gcrud->stats(array('module' => 'seo'));
            $record = $this->seo_model->update_link($record);
        }
        if(empty($record->params)) {
            $this->gcrud->stats(array('module' => 'seo'));
            $record = $this->seo_model->update_link($record);
        }
        $site_url = site_url($record->uri_string);
        Template::set(array(
            'toolbar_title' => lang('edit') . ' - ' . $record->uri_string,// anchor($site_url, $record->uri_string, 'target="_blank"'),
            'record' => $record
        ));
        Template::render();
    }
    
    /**
     * Run the crawler to crawl new links
     */
    public function crawler() {
        if(isset($_POST['save'])) {
            $this->gcrud->stats(array('module' => 'seo'));
            $links_count = $this->seo_model->crawler(true);
            Template::set_message(sprintf(lang('seo_crawl_success'), $links_count), 'success');
        }
        Template::set('toolbar_title', lang('seo_crawler'));
        Template::render();
    }
    
    /**
     * Allow user to edit static files like _variables.scss, _utilties.scss etc
     * @param string	$filename	The name of the file to update
     */
    public function robots() {
        $this->lang->load('themes/themes');
        $filename = 'robots.txt';
        $file = FCPATH . '/' . $filename;
        // Save file
        if (isset ( $_POST ['save_write'] )) {
            $this->gcrud->stats(array('module' => 'seo'));
            if(isset($_POST['file_content'])) {
                $content = $_POST['file_content'];
                // Write the changes out...
                if (! function_exists('write_file')) {
                    $this->load->helper('file');
                }
                
                // save file
                if (write_file($file, $content)) {
                    Template::set_message ( sprintf(lang('msg_save_success'), lang('label_file')), 'success' );
                } else {
                    Template::set_message(lang('msg_operation_failure'), 'danger');
                }
            }
        }
        // display file contents within the editor
        Assets::add_js('edit_area/edit_area_full.js');
        Assets::add_js('editAreaLoader.init({
		id: "file_content"	// id of the textarea to transform
		,start_highlight: true	// if start with highlight
		,allow_resize: "both"
		,allow_toggle: false
		,word_wrap: false
		,language: "en"
		,syntax: "css"
	});', 'inline');
        
        $file_content = '';
        if (file_exists($file)) {
            $file_content = file_get_contents($file);
        }
        Template::set(array(
            'toolbar_title' => lang('label_file') . ' - ' . $filename,
            'file_content' => $file_content,
            'filename' => $filename
        ));
        Template::render();
    }
}