<?php defined('BASEPATH') || exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Online Software Solutions							   # ||
|| # Copyright (C) 2021 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class Seo_model extends BF_Model
{
    public $table_name	                     = 'seo_links';
    protected $table_error_links             = 'seo_error_links';
    protected $table_error_link_referrers    = 'seo_error_link_referrers';
    protected $table_competitors             = 'seo_competitors';
    protected $table_backlinks	             = 'seo_backlinks';
    protected $table_backlink_emails	     = 'seo_backlink_emails';
    protected $table_backlink_products	     = 'seo_backlink_products';
    protected $table_backlink_types	         = 'seo_backlink_types';
	protected $key			= 'id';
	protected $date_format	= 'datetime';

	protected $log_user 	= true;
	protected $set_created	= true;
	protected $set_modified = true;
	public $soft_deletes	= false;

	protected $created_field     = 'created_on';
    protected $created_by_field  = 'created_by';
	protected $modified_field    = 'modified_on';
    protected $modified_by_field = 'modified_by';

	// Customize the operations of the model without recreating the insert,
    // update, etc. methods by adding the method names to act as callbacks here.
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 	    = array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	// For performance reasons, you may require your model to NOT return the id
	// of the last inserted row as it is a bit of a slow method. This is
    // primarily helpful when running big loops over data.
	protected $return_insert_id = true;

	// The default type for returned row data.
	protected $return_type = 'object';

	// Items that are always removed from data prior to inserts or updates.
	protected $protected_attributes = array();

	// You may need to move certain rules (like required) into the
	// $insert_validation_rules array and out of the standard validation array.
	// That way it is only required during inserts, not updates which may only
	// be updating a portion of the data.
	protected $validation_rules 		= array();
	protected $insert_validation_rules  = array();
	protected $skip_validation 			= false;
	protected $context			        = 'components';
	protected $moduleName               = 'seo';
	public $recommended_limits = array(
	    'title' => 60,
	    'description' => 320,
	    'keywords' => 255,
	    'headings' => 0,
	    'images' => 10,
	    'internal' => 100,
	    'external' => 100
	);

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
    }
    
    /**
     * Seo Links Fields
     *
     * @return array
     */
    public function links() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'text',
                'name' => 'uri_string',
                'label' => 'label_url',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|max_length[255]',
            ),
            array(
                'type' => 'text',
                'name' => 'title',
                'label' => 'label_title',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'description',
                'label' => 'label_description',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'keywords',
                'label' => 'label_keywords',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'params',
                'label' => 'label_params',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'hits',
                'label' => 'label_hits',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'sitemap',
                'label' => 'seo_sitemap',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'status',
                'label' => 'label_status',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_ip',
                'label' => 'created_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_ip',
                'label' => 'modified_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            
        );
        return $fields;
    }
    
    /**
     * Module Fields
     *
     * @return array
     */
    public function backlinks() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'dropdown',
                'name' => 'product_id',
                'label' => 'singular_products',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:seo_backlink_products;value:database',
                'options_detailed' => 1,
                'class' => 4,
                'rules' => 'trim|is_numeric',
            ),
            array(
                'type' => 'dropdown',
                'name' => 'type_id',
                'label' => 'label_type',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:seo_backlink_types;value:database',
                'options_detailed' => 1,
                'class' => 3,
                'rules' => 'trim|is_numeric',
            ),
            array(
                'type' => 'text',
                'name' => 'link_url',
                'label' => 'label_link_url',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'edit' => 1,
                'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
                'rules' => 'trim|max_length[255]',
            ),
            array(
                'type' => 'text',
                'name' => 'submitted_url',
                'label' => 'label_submitted_url',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|max_length[255]',
            ),
            array(
                'type' => 'text',
                'name' => 'submitted_on',
                'label' => 'label_submitted_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'class' => 3,
                'rules' => 'trim|valid_datetime',
            ),
            array(
                'type' => 'none',
                'name' => 'domain',
                'label' => 'label_domain',
                'frontend' => false,
                'admin_only' => false,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim|max_length[255]',
            ),
            array(
                'type' => 'dropdown',
                'name' => 'page_rank',
                'label' => 'label_page_rank',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:0;value:0|name:1;value:1|name:2;value:2|name:3;value:3|name:4;value:4|name:5;value:5|name:6;value:6|name:7;value:7|name:8;value:8|name:9;value:9|name:10;value:10',
                'options_detailed' => 1,
                'class' => 1,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'source',
                'label' => 'label_source',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'class' => 4,
                'rules' => 'trim|max_length[40]',
            ),
            array(
                'type' => 'text',
                'name' => 'anchor_text',
                'label' => 'label_anchor_text',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim|max_length[100]',
            ),
            array(
                'type' => 'text',
                'name' => 'name',
                'label' => 'label_name',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'email',
                'label' => 'label_email',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim|valid_email',
            ),
            array(
                'type' => 'text',
                'name' => 'username',
                'label' => 'bf_username',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'password_salt',
                'label' => 'bf_password',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'textarea',
                'name' => 'comments',
                'label' => 'label_submitted_text',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'textarea',
                'name' => 'remarks',
                'label' => 'label_remarks',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'status',
                'label' => 'label_status',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_by',
                'label' => 'created_by',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => $this->auth->role_id() == 1 ? true : false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_ip',
                'label' => 'created_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim|max_length[255]',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_by',
                'label' => 'modified_by',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_ip',
                'label' => 'modified_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            )
        );
        return $fields;
    }
    
	/**
	 * SEO Analysis Fields
	 * @return array
	 */
	public function analysis($role = 'admin') {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'domain',
	            'label' => 'seo_domain',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'edit' => 1,
	            'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
	            'rules' => 'trim|required|max_length[255]|valid_url'
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'is_default',
	            'label' => 'label_default',
	            'frontend' => false,
	            'admin_only' => false,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_on',
	            'label' => 'created_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim|max_length[255]',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_on',
	            'label' => 'modified_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim|max_length[255]',
	        )
	    );
	    return $fields;	    
	}
	
	/**
	 * Seo Error Links Fields
	 *
	 * @return array
	 */
	public function error_links() {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'request_uri',
	            'label' => 'label_url',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim|max_length[255]',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'code',
	            'label' => 'label_code',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim|max_length[10]',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'counts',
	            'label' => 'label_counts',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_on',
	            'label' => 'created_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_ip',
	            'label' => 'created_ip',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_on',
	            'label' => 'last_accessed',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_ip',
	            'label' => 'ip_address',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        
	    );
	    return $fields;
	}
	
	/**
	 * Seo Error Link Referrers Fields
	 *
	 * @return array
	 */
	public function error_link_referrers() {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'error_url',
	            'label' => 'label_url',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'referrer',
	            'label' => 'seo_referrer',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim|max_length[255]',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'counts',
	            'label' => 'label_counts',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_on',
	            'label' => 'created_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_ip',
	            'label' => 'created_ip',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_on',
	            'label' => 'last_accessed',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_ip',
	            'label' => 'ip_address',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        
	    );
	    return $fields;
	}
	
	/**
	 * Get Product Topics/Products/Groups etc.
	 * @param number $count
	 * @param string $where
	 * @param string $sort_by
	 * @param number $limit
	 * @param number $offset
	 * @return array|number
	 */
	public function manage($count = 0, $where = null, $sort_by = 'M.id DESC', $limit = 15, $offset = 0, $extra = array()) {
	    if(!isset($extra['module'])) {
	        return false;
	    }
	    $module = $extra['module'];
	    if($count == 1) {
	        $sql = "SELECT COUNT(*) AS total_count";
	    } else {
	        switch($module) {
	            case 'backlinks':
	                $sql = "SELECT M.*, P.name AS row_id, T.name AS type_id, M.created_by AS user_id, " . user_fields() . PHP_EOL;
	                break;
	            default:
	                $sql = "SELECT M.*";
	        }
	    }
	    $table = 'seo_' . $module;
	    $sql .= " FROM " . $this->db->dbprefix . $table . " M";
	    if($count == 0) {
	        switch($module) {
	            case 'backlinks':
	                $sql .= " LEFT JOIN " . $this->db->dbprefix . "users U On U.id = M.created_by";
	                $sql .= " LEFT JOIN " . $this->db->dbprefix . $this->table_backlink_products . " P ON P.id = M.product_id" . PHP_EOL;
	                $sql .= " LEFT JOIN " . $this->db->dbprefix . $this->table_backlink_types . " T ON T.id = M.type_id" . PHP_EOL;
	                break;
	            default:
	        }
	    }
	    if(isset($extra['filter']) && $extra['filter'] == 'deleted') {
	        $sql .= " WHERE M.deleted = 1";
	    } else {
	        $sql .= " WHERE M.deleted = 0";
	    }
	    if(!empty($where)) {
	        $sql .= ' AND '. $where;
	    }
	    if ($count == 0) {
	        $sql .= " GROUP BY M.id ORDER BY " . $sort_by;
	        if ($limit != 0) {
	            $sql .= " LIMIT " . $offset . ", " . $limit;
	        } else {
	            $sql .= " LIMIT $limit";
	        }
	        $query = $this->db->query($sql);
	        return $query->result();
	    } else {
	        $result = $this->db->query($sql)->row();
	        return $result ? $result->total_count : 0;
	    }
	}
	
	/**
	 * Display a list of URL/Backlinks data.
	 *
	 * @return void
	 */
	public function view($module = 'urls', $filter = 'all', $offset = 0, $role = 'admin', $extra = array()) {
	    $has_status = false;
	    // Status Change
	    if (isset($_POST['status_change'])) {
	        $this->gc->status_change('seo_' . $module, $_POST['status_change'], 'status');
	    }
	    // Set Default
	    if (isset($_POST['set_default'])) {
	        $this->set_default($_POST['set_default']);
	        Template::redirect(current_url());
	    }
	    // Allow Admin to change status
	    if(isset($_POST['change_status'])) {
	        $id = $_POST['change_status'];
	        $new_status = $_POST['status_' . $id];
	        $this->update_status($module, $id, $new_status);
	    }
	    $data = array(
	        'context' => $this->context,
	        'model' => 'seo',
	        'fields_method' => $module,
	        'set_table' => true,
	        'query_method' => 'manage', // Extra Method in case of different columns
	        'query_extra' => array('module' => $module, 'filter' => $filter),
	        'table_alias' => 'M.', // Table Alias for table joins
	        'prefix' => 'M',
	        'filter' => $filter,
	        // 'view' => 'components/manage',
	        'edit_url' => site_url(SITE_AREA . '/' . $this->context . '/' . $this->moduleName . '/form/' . $module . '/edit/')
	    );
	    if(isset($extra['redirect_uri'])) {
	        $data['redirect_uri'] = $extra['redirect_uri'];
	    }
	    $create_url = site_url(SITE_AREA . '/' . $this->context . '/' . $this->moduleName . '/form/' . $module);
	    switch($module) {
	        case 'backlinks':
	            $data['create_url'] = $create_url;
	            if($this->auth->role_id() != 1) { // staff and other users can view only their own submit links
	               $data['query_where'] = "M.created_by = " . $this->auth->user_id();
	            }
	            $data['search_fields'] = array('submitted_url', 'description');
	            $data['filter_tabs'] = array('active', 'inactive');
	            $data['footer_actions'] = array('activate', 'deactivate');
	            $data['redirect_uri'] = '/components/seo/manage/' . $module;
	            $data['filter_counter'] = true;
	            $data['filter_status_counter'] = true;
	            break;
	        case 'backlink_products':
	        case 'backlink_types':
	            $data['create_url'] = $create_url;
	            $data['search_fields'] = array('name');
	            break;
	        case 'backlink_emails':
	            $data['create_url'] = $create_url;
	            $data['search_fields'] = array('first_name', 'last_name', 'email');
	            break;
	    }
	    if($has_status) {
	        $data['redirect_uri'] = '/components/seo/manage/' . $module;
	        $data['filter'] = $filter;
	        $data['filter_counter'] = true;
	        $filter_status = $this->get_filter('statuses', 0, null, $module);
	        Template::set(array(
	            'filter_status' => $filter_status
	        ));
	    }
	    $this->gcrud->index('seo_' . $module, $offset, 'admin', $data);
	}
	
	/**
	 * Get Filter
	 * @param string $module
	 * @return NULL[]
	 */
	public function get_filter($module, $user_id = 0, $field_name = 'user_id', $table = null) {
	    $column_name = '';
	    switch($module) {
	        case 'statuses':
	            $column_name = 'status_id';
	            break;
	    }
	    $module = 'seo_' . $module;
	    $result = array();
	    $records = $this->db->get($module)->result();
	    if($records) {
	        $where = array();
	        if(!empty($user_id)) {
	            $where[$field_name] = $user_id;
	        }
	        $count_table = empty($table) ? $this->table_name : 'seo_' . $table;
	        foreach($records as $record) {
	            $where[$column_name] = $record->id;
	            $result[$record->id] = array(
	                'id' => $record->id,
	                'name' => !empty($record->name_lang) ? lang($record->name) : $record->name,
	                'counts' => $this->db->where($where)->count_all_results($count_table)
	            );
	        }
	    }
	    return $result;
	}
	
	/**
	 * Update Domain Name
	 */
	private function update_domain() {
	    $result = $this->db->select('id, link_url')->get($this->table_backlinks)->result();
	    foreach($result as $link) {
	        $data = array(
	            'domain' => $this->get_domain($link->link_url)
	        );
	        $this->db->where('id', $link->id);
	        $this->db->update('rs0d_links', $data);
	    }
	}
	
	/**
	 * Get Domain Name From URL
	 * @param string $url
	 * @return string|boolean
	 */
	public function get_domain($url) {
	    $pieces = parse_url($url);
	    $domain = isset($pieces['host']) ? $pieces['host'] : '';
	    if (preg_match('/(?P<domain>[a-z0-9][a-z0-9\-]{1,63}\.[a-z\.]{2,6})$/i', $domain, $regs)) {
	        return $regs['domain'];
	    }
	    return false;
	}
	
	/**
	 * Generate Sitemap using the seo_links Table
	 */
	public function generate_sitemap() {
	    $this->load->library ('sitemaps');
	    
	    // Empty the sitemap folder first
	    $this->empty_sitemap_folder();
	    
	    $this->load->library('sitemaps');
	    $module = $this->table_name;
	    $where = array('sitemap' => 1, 'status' => 1, 'deleted' => 0);
	    $total_rows = $this->db->where($where)->count_all_results($module);
	    if($total_rows > 0) {
	        $querylimit = 49999;
	        $urls = array ();
	        // Get lowest id from the listing table and start sitemap generation from that ID
	        $min_query = "SELECT min( id ) as id, max(id) as max_id FROM " . $this->db->dbprefix . $module;
	        $min = $this->db->query($min_query)->row();
	        for($i = $min->id; $i <= $min->max_id; $i ++) {
	            $limit = $i + $querylimit;
	            $this->db->where("id BETWEEN $i AND $limit");
	            $records = $this->db->where($where)->get($module)->result();
	            if(!empty($records)) {
	                /**
	                 * Changefreq Guidelines
	                 * NEVER: Old news stories, press releases, etc
	                 * YEARLY: Contact, About Us, login, registration pages
	                 * MONTHLY: FAQs, instructions, occasionally updated articles
	                 * WEEKLY: Product info pages, website directories
	                 * DAILY: Blog entry index, modules, small message board
	                 * HOURLY: Major news site, weather information, forum
	                 * ALWAYS: Stock market data, social bookmarking categories
	                 */
	                /**
	                 * Priority XML Guidelines
	                 * 0.8-1.0: Homepage, subdomains, product info, major features
	                 * 0.4-0.7: Articles and blog entries, category pages, FAQs
	                 * 0.0-0.3: Outdated news, info that has become irrelevant
	                 */
	                foreach($records as $record) {
	                    if(!empty($record->modified_on)) {
	                        $lastmod = $record->modified_on;
	                    } else if (!empty($record->created_on)) {
	                        $lastmod = $record->created_on;
	                    } else {
	                        $lastmod = date ( 'Y-m-d H:i:s' );
	                    }
	                    switch($module) {
	                        default:
	                            $item = array(
    	                            "loc" => site_url($record->uri_string),
    	                            "lastmod" => date ( "c", strtotime ( $lastmod ) ),
    	                            "changefreq" => $record->sitemap_freq,
    	                            "priority" => $record->sitemap_priority
	                            );
	                    }
	                    
	                    $this->sitemaps->add_item( $item );
	                }
	                $file_name = $this->sitemaps->build ( "sitemap/links-$i-" . $min->max_id .".xml" );
	                $urls [] = array (
	                    "loc" => site_url ( $file_name ),
	                    "lastmod" => date ( "c", strtotime ( date ( 'Y-m-d H:i:s' ) ) )
	                );
	                $i = $i + $querylimit;
	            }
	        }
	    }	    
	    $this->sitemaps->build_index ( $urls, "sitemap.xml" );
	}
	
	/**
	 * Generate sitemap
	 */
	public function generate_sitemap_standard() {
	    $this->load->library ('sitemaps');
	    
	    // Empty the sitemap folder first
	    $this->empty_sitemap_folder();
	    
	    // generate the static links sitemap first e.g. Home, Blogs, Forums etc.
	    /*$urls = array(site_url(), gc_url_format('forums'), gc_url_format('blogs'));
	    foreach($urls as $url) {
	        $item = array(
	            "loc" => $url,
	            "lastmod" => date ( "c", strtotime ( date('Y-m-d H:i:s') ) ),
	            "changefreq" => "weekly",
	            "priority" => "0.8"
	        );
	        $this->sitemaps->add_item( $item );
	    }
	    $file_name = $this->sitemaps->build ( "sitemap/site.xml" );
	    $file_urls[] = array (
	        "loc" => site_url ( $file_name ),
	        "lastmod" => date ( "c", strtotime ( date ( 'Y-m-d H:i:s' ) ) )
	    );*/
	    $file_urls[] = (array) $this->links_sitemap();
	    // generate module sitemaps
	    // 'pages' - Pages are already there in links
	    $modules = array('news', 'blogs', 'products', 'forum_categories', 'forum_threads', 'forum_posts');	    
	    foreach($modules as $module) {
	        $file_urls = array_merge($file_urls, (array) $this->module_sitemap($module));
	    }
	    $this->sitemaps->build_index ( $file_urls, "sitemap.xml" );
	}
	
	/**
	 * Links Sitemap
	 * @return string[]|NULL[]
	 */
	public function links_sitemap() {
	    $this->load->library('sitemaps');
	    $this->load->model('links/links_model');
	    $links = $this->links_model->process_links();
	    // Home Page URL Link
	    /*$item = array(
	        "loc" => site_url(),
	        "lastmod" => date ( "c", strtotime ( date('Y-m-d H:i:s') ) ),
	        "changefreq" => "weekly",
	        "priority" => "0.8"
	    );
	    $this->sitemaps->add_item( $item );
	    */
	    foreach($links as $link) {
	        if($link['location_id'] == 1 || $link['location_id'] == 2) { // Header and Footer Links only	            
	            if(!empty($link['modified_on'])) {
	                $lastmod = $link['modified_on'];
	            } else if (!empty($link['created_on'])) {
	                $lastmod = $link['created_on'];
	            } else {
	                $lastmod = date ( 'Y-m-d H:i:s' );
	            }
	            $item = array(
	                "loc" => $link['url'],
	                "lastmod" => date ( "c", strtotime ( $lastmod ) ),
	                "changefreq" => "weekly",
	                "priority" => "0.8"
	            );
	            $this->sitemaps->add_item( $item );
	        }
	    }
	    
	    $file_name = $this->sitemaps->build ( "sitemap/links.xml" );
	    $url = array (
	        "loc" => site_url ( $file_name ),
	        "lastmod" => date ( "c", strtotime ( date ( 'Y-m-d H:i:s' ) ) )
	    );
	    return $url;
	}
	
	/**
	 * Build Sitemap for Modules
	 * @param string $module - Name of the Module
	 */
	public function module_sitemap($module, $where = array()) {
	    $this->load->library('sitemaps');
	    switch($module) {
	        case 'news':
	        case 'pages':
	            $where = array('status' => 1, 'deleted' => 0);
	            break;
	        default:
	            $where = array('status_id' => 2, 'deleted' => 0);
	            break;
	    }
	    $total_rows = $this->db->where($where)->count_all_results($module);
	    if($total_rows > 0) {
	        $querylimit = 49999;
	        $urls = array ();
	        // Get lowest id from the listing table and start sitemap generation from that ID
	        $min_query = "SELECT min( id ) as id, max(id) as max_id FROM " . $this->db->dbprefix . $module;
	        $min = $this->db->query($min_query)->row();
	        for($i = $min->id; $i <= $min->max_id; $i ++) {
	            $limit = $i + $querylimit;
	            $this->db->where("id BETWEEN $i AND $limit");
	            $records = $this->db->where($where)->get($module)->result();
	            if(!empty($records)) {
	                /**
	                 * Changefreq Guidelines
	                 * NEVER: Old news stories, press releases, etc
	                 * YEARLY: Contact, About Us, login, registration pages
	                 * MONTHLY: FAQs, instructions, occasionally updated articles
	                 * WEEKLY: Product info pages, website directories
	                 * DAILY: Blog entry index, modules, small message board
	                 * HOURLY: Major news site, weather information, forum
	                 * ALWAYS: Stock market data, social bookmarking categories
	                 */
	                /**
	                 * Priority XML Guidelines
	                 * 0.8-1.0: Homepage, subdomains, product info, major features
	                 * 0.4-0.7: Articles and blog entries, category pages, FAQs
	                 * 0.0-0.3: Outdated news, info that has become irrelevant
	                 */
	                foreach($records as $record) {
	                    if(!empty($record->modified_on)) {
	                        $lastmod = $record->modified_on;
	                    } else if (!empty($record->created_on)) {
	                        $lastmod = $record->created_on;
	                    } else {
	                        $lastmod = date ( 'Y-m-d H:i:s' );
	                    }
	                    switch($module) {
	                        default:
	                            $item = array(
	                               "loc" => gc_url_format($module, $record->slug, $record->id),
	                               "lastmod" => date ( "c", strtotime ( $lastmod ) ),
	                               "changefreq" => "weekly",
	                               "priority" => "0.8"
	                             );
	                    }
	                    
	                    $this->sitemaps->add_item( $item );
	                }
	                $file_name = $this->sitemaps->build ( "sitemap/" .$module. "-$i-" . $min->max_id .".xml" );
	                $urls [] = array (
	                    "loc" => site_url ( $file_name ),
	                    "lastmod" => date ( "c", strtotime ( date ( 'Y-m-d H:i:s' ) ) )
	                );
	                $i = $i + $querylimit;
	            }
	            return $urls;
	        }
	    }
	}
	
	/**
	 * Empty the sitemap folder first before generating the new sitemap
	 * IMPORTANT - Else duplicate files will be created each time on ID changes
	 * @return boolean
	 */
	private function empty_sitemap_folder() {
	    $dir = FCPATH . '/sitemap';
	    $di = new RecursiveDirectoryIterator($dir, FilesystemIterator::SKIP_DOTS);
	    $ri = new RecursiveIteratorIterator($di, RecursiveIteratorIterator::CHILD_FIRST);
	    foreach ( $ri as $file ) {
	        if($file->isDir()) {
	            rmdir($file);
	        } else if(file_exists(realpath(FCPATH . $file))) {
	            unlink($file);
	        }
	    }
	    return true;
	}
	
	/**
	 * Process Error Links
	 * There is no way to access CI instance in the 404 file. 
	 * Neither through controller and neither through other helper function, so a file is created in the application/logs/error_links.txt file. 
	 * This file is read when the error_links is clicked. It will be set to empty after processing.
	 */
	public function process_error_links() {
	    $filename = APPPATH . 'logs/error_links.txt';
	    if(file_exists($filename)) {
	        $lines = file($filename);
	        if(!empty($lines)) {
	            $lines = array_unique($lines);
	            $data = array();
	            foreach($lines as $line) {
	                if(!empty($line)) {
	                    $link = explode('!~~!', $line);
	                    if(strlen($link[0]) <= 255 && !empty($link[1]) && is_string($link[1])) {
	                        $datetime = $link[2];
	                        $ip_address = $link[3];
	                        $request_uri = $link[0];
	                        $referrer = $link[1];
	                        $link_id = 0;
	                        $type = 'insert';
	                        // check link existence
	                        $record = $this->db->where('request_uri', $request_uri)->get($this->table_error_links)->row();
	                        if($record) { // existing link update counts, modified_on and modified_ip
	                            $update_data = array(
	                                'counts' => $record->counts + 1,
	                                'modified_on' => $datetime,
	                                'modified_ip' => $ip_address
	                            );
	                            $this->db->where('id', $record->id)->update($this->table_error_links, $update_data);
	                            $link_id = $record->id;
	                            $type = 'update';
	                        } else { // new link
	                            $insert_data = array(
	                                'request_uri' => $request_uri,
	                                'counts' => 1,
	                                'created_on' => $datetime,
	                                'created_ip' => $ip_address,
	                                'modified_on' => $datetime,
	                                'modified_ip' => $ip_address
	                            );
	                            $this->db->insert($this->table_error_links, $insert_data);
	                            $link_id = $this->db->insert_id();
	                        }
	                        // Save Referrer
	                        if($link_id > 0 && $referrer != 'NA') {
	                            if($type == 'update') {
	                                $row = $this->db->where(array('error_link_id' => $link_id, 'referrer' => $referrer))->get($this->table_error_link_referrers)->row();
	                                if($row) {
	                                    $referrer_update_data = array(
	                                        'counts' => $row->counts + 1,
	                                        'modified_on' => $datetime,
	                                        'modified_ip' => $ip_address
	                                    );
	                                    $this->db->where('id', $row->id)->update($this->table_error_link_referrers, $referrer_update_data);
	                                } else {
	                                    $type = 'insert';
	                                }
	                            }
	                            if($type == 'insert') {
	                                $referrer_insert_data = array(
	                                    'error_link_id' => $link_id,
	                                    'error_url' => $request_uri,
	                                    'referrer' => $referrer,
	                                    'counts' => 1,
	                                    'created_on' => $datetime,
	                                    'created_ip' => $ip_address,
	                                    'modified_on' => $datetime,
	                                    'modified_ip' => $ip_address
	                                );
	                                $this->db->insert($this->table_error_link_referrers, $referrer_insert_data);
	                                $link_id = $this->db->insert_id();
	                            }
	                        }
	                    }
	                }
	            }
	            // empty the error_links.txt file.
	            $fh = fopen( $filename, 'w' );
	            fclose($fh);
	        }
	    }
	}
	
	/**
	 * Purge Error Links
	 * @param array $ids
	 */
	public function purge_error_links($ids = array()) {
	    foreach($ids as $id) {
	        // purge referrers first
	        $this->db->where('error_link_id', $id)->delete($this->table_error_link_referrers);
	        
	        // purge error link
	        $this->db->where('id', $id)->delete($this->table_error_links);
	    }
	}
	
	/**
	 * Link Crawler
	 * @param boolean $is_recrawl - Purge all links before adding new
	 */
	public function crawler($is_recrawl = false) {	  
	    $uri_strings = array();
	    $modules = array('links', 'pages', 'news', 'blogs', 'products', 'forum_categories', 'forum_threads', 'forum_posts');	
	    foreach($modules as $module) {
	        $uri_strings = array_merge($uri_strings, (array) $this->crawl_links($module));
	    }
	    $data = array();
	    $uri_strings = array_unique($uri_strings);
	    $datetime = date('Y-m-d H:i:s');
	    $ip_address = $this->input->ip_address();
	    if($is_recrawl) {
	        // purge all
	        // do not truncate links, this will remove all seo title, meta_title and other settings
	        // if a page does not exist, just make it inactive
	        // $this->db->truncate($this->table_name);
	    }
	    foreach($uri_strings as $uri_string) {
	        $sql = "INSERT INTO " . $this->db->dbprefix . $this->table_name . " (uri_string, created_on, created_ip) VALUES ('" . $uri_string . "', '" . $datetime . "', '" . $ip_address . "') ON DUPLICATE KEY UPDATE modified_on = '" . $datetime . "', modified_ip = '" . $ip_address . "'";
	        $this->db->query($sql);
	    }
	    return count($uri_strings);
	}
	
	/**
	 * Crawl Module Links
	 * @param string $module
	 * @return mixed[]
	 */
	public function crawl_links($module) {
	    $uri_strings = array();
	    if($module == 'links') {	        
	        $this->load->model('links/links_model');
	        $links = $this->links_model->process_links(0, array(), array(), true);
	        foreach($links as $link) {
	            $link['permission'] = strtolower($link['permission']);
	            if(empty($link['permission']) || $link['permission'] == 'site.notloggedin.user') {
	                $uri_string = str_replace(site_url(), '', $link['url']);
	                if(strpos($uri_string, 'FIXME') === false && strpos($uri_string, 'javascript:void(0);') === false) {
	                    $uri_strings[] = $uri_string;
	                }
	            }
	        }
	        $uri_strings = array_unique($uri_strings);
	    } else {
    	    switch($module) {
    	        case 'news':
    	        case 'pages':
    	            $where = array('status' => 1, 'deleted' => 0);
    	            break;
    	        default:
    	            $where = array('status_id' => 2, 'deleted' => 0);
    	            break;
    	    }
    	    $records = $this->db->where($where)->get($module)->result();
    	    if($records) {
    	        foreach($records as $record) {
    	            switch($module) {
    	                case 'forum_posts':
    	                    $link = gc_url_format('forums/post', $record->slug);
    	                    break;
    	                case 'forum_threads':
    	                    $link = gc_url_format('forums/thread', $record->slug);
    	                    break;
    	                case 'forum_categories':
    	                    $link = gc_url_format('forums/category', $record->slug);
    	                    break;
    	                case 'blogs':
    	                    $link = gc_url_format('blogs/detail', $record->slug);
    	                    break;
    	                default:
    	                    $link = gc_url_format($module, $record->slug, $record->id);
    	            }
    	            $uri_string = str_replace(site_url(), '', $link);
    	            if(strpos($uri_string, 'FIXME') === false) {
    	                $uri_strings[] = $uri_string;
    	            }
    	        }
    	    }
	    }
	    return $uri_strings;
	}
	
	/**
	 * The link crawl is a slow process, so it will be called on link edit.
	 * @param array $record
	 */
	public function update_link($record) {
	    $site_url = site_url($record->uri_string);
	    $this->load->library('crawler');
	    $result = $this->crawler->crawl($site_url);
	    $datetime = date('Y-m-d H:i:s');
	    $ip_address = $this->input->ip_address();
	    if(!empty($result)) {
	        $data = array(
	            'params' => json_encode($result['params']),
	            'modified_on' => $datetime,
	            'modified_ip' => $ip_address
	        );
	        if(empty($record->title)) {
	            $data['title'] = $result['title'];
	        }
	        if(empty($record->description)) {
	            $data['description'] = $result['description'];
	        }
	        if(empty($record->keywords)) {
	            $data['keywords'] = $result['keywords'];
	        }
	        if(!empty($data)) {
	            $this->db->where('id', $record->id)->update($this->table_name, $data);
	            $record->params = $data['params'];
	            $record->modified_on = $datetime;
	            $record->modified_ip = $ip_address;
	        }
	    }
	    return $record;
	}
	
	/**
	 * Get Page Speed information using Google
	 * @param string $url
	 */
	public function pagespeed($url) {
	    // View https://developers.google.com/speed/docs/insights/v1/getting_started#before_starting to get a key
	    $key = settings_item('site.google_api_key'); // do not forget to enable the pagespeed API at your https://console.developers.google.com
	    $url = "https://www.googleapis.com/pagespeedonline/v1/runPagespeed?url=$url&key=$key";
	    $ch = curl_init();
	    curl_setopt($ch, CURLOPT_URL,$url);
	    curl_setopt($ch, CURLOPT_RETURNTRANSFER,1);
	    $result = curl_exec($ch);
	    curl_close($ch);
	    $data = json_decode($result ?? '');
	    return $data;
	}
	
	/**
	 * Replace Placeholder
	 * @param string $format
	 * @param string $args
	 * @return mixed
	 */
	public function replace_placeholders($format, $args) {
	    $i = 1;
	    foreach ($args as $arg) {
	        $arg_value = $arg->type == 'URL' ? anchor($arg->value, $arg->value, 'target="_blank"') : $arg->value;
	        $format = str_replace("\$" . $i, "$arg_value", $format);
	        $i++;
	    }
	    return $format;
	}
	
	/**
	 * Display Summary on the Home Page
	 * @return array
	 */
	public function summary() {	    
	    $sql = "SELECT COUNT(*) AS total_links";
	    $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->table_name . " WHERE title IS NULL) AS missing_title";
	    $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->table_name . " WHERE description IS NULL) missing_description";
	    $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->table_name . " WHERE keywords IS NULL) missing_keywords";
	    $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->table_backlinks . ") total_backlinks";
	    $sql .= " FROM " . $this->db->dbprefix . $this->table_name;
	    $query = $this->db->query($sql);
	    return $query->row();
	}
}