<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/**
 * Bonfire
 *
 * An open source project to allow developers get a jumpstart their development of CodeIgniter applications
 *
 * @package   Bonfire
 * @author    Bonfire Dev Team
 * @copyright Copyright (c) 2011 - 2013, Bonfire Dev Team
 * @license   http://guides.cibonfire.com/license.html
 * @link      http://cibonfire.com
 * @since     Version 1.0
 * @filesource
 */

// ------------------------------------------------------------------------

/**
 * Settings Module
 *
 * Allows the user to management the preferences for the site.
 *
 * @package    Bonfire
 * @subpackage Modules_Settings
 * @category   Controllers
 * @author     Bonfire Dev Team
 * @link       http://guides.cibonfire.com/helpers/file_helpers.html
 *
 */
class Settings extends Admin_Controller
{

	/**
	 * Private variable declaration for file upload
	 * Used in help option for consistency
	 * @var number
	 */
	private $file_size = 70; // set maximum size
	private $max_width = 163; // set maximum width
	private $max_height = 45; // set maximum height
	private $resize_width = 163; // set resize width
	private $resize_height = 45; // set resize height
	
	/**
	 * Sets up the require permissions and loads required classes
	 *
	 * @return void
	 */
	public function __construct()
	{
		parent::__construct();

		// restrict access - View and Manage
		$this->auth->restrict('Bonfire.Settings.View');
		$this->auth->restrict('Bonfire.Settings.Manage');

		Template::set('toolbar_title', 'Site Settings');
		
		// settings used in help option
		$file_upload_settings = array (
				'file_size' => $this->file_size,
				'max_width'	=> $this->max_width,
				'max_height' => $this->max_height,
				'resize_width' => $this->resize_width,
				'resize_height' => $this->resize_height
		);
		Template::set('file_upload_settings',$file_upload_settings);

		$this->load->helper('config_file');
		$this->lang->load('settings');

	}//end __construct()

	//--------------------------------------------------------------------

	/**
	 * Displays a form with various site setings including site name and
	 * registration settings
	 *
	 * @access public
	 *
	 * @return void
	 */
	public function index()
	{
		$this->load->model('locations/countries_model');
		$this->load->model('listings/listings_model');
		$this->load->model('currencies/currencies_model');
		$this->load->model('pages/pages_model');
		$this->load->config('extended_settings');
		$this->load->config('general_settings');
		$extended_settings = config_item('extended_settings_fields');
		$general_settings = config_item('general_settings_fields');

		if (isset($_POST['save'])) {
		    gc_demo_mode();		    
		    $this->gcrud->stats(array('module' => 'settings'));
			if ($this->save_settings($extended_settings, $general_settings)) {
				Template::set_message(lang('settings_saved_success'), 'success');
				redirect(SITE_AREA .'/settings/settings');
			} else {
				Template::set_message(lang('settings_error_success'), 'error');
			}
		}

		// Read our current settings
		$settings = $this->settings_lib->find_all();
		
		// Multi Langauge Fields - Get translations if exists
		$settings = $this->translations->get_translations('settings', 1, (object) $settings);
		$settings = (array) $settings;
		
		// Load currencies information
		$currencies = $this->currencies_model->find_all();
		// Country, State and City Selectize Dropdown
		$this->gc->location_dropdown(settings_item('adv.default_country'), settings_item('adv.default_state'), settings_item('adv.default_city'), true, true);
	
		Template::set(array('settings' => $settings,
							'extended_settings'=> $extended_settings,
							'general_settings'=> $general_settings,
							'currencies'	=> $currencies,
							'pages'	=> $this->pages_model->select('id, slug, title')->find_all()
							));
		
		// Get the possible languages
		$this->load->helper('translate/languages');
		Template::set('languages', list_languages());
		Template::set(array(
			'selected_languages' => $this->input->post('languages') ? $this->input->post('languages') : unserialize($settings['site.languages'])
		));
		Assets::add_module_js('settings', 'js/settings.js');
		
		/*Assets::add_css('selectize.bootstrap3.css');
		Assets::add_js('selectize.min.js');
		Assets::add_js("
$('.selectize-fields').selectize({
    maxItems: 3
});", "inline");*/
		Template::set_view('settings/settings/index');
		Template::render();

	}//end index()
	
	//GET STATES AND CITIES (AJAX)
	public function loadData() {
		$loadType = $_POST['loadType'];
		$loadId = $_POST['loadId'];
		if($_POST['loadType'] == 'state') {
			$this->load->model('locations/states_model');
			$result = $this->states_model->select('id, name')->where('country_iso', $loadId)->order_by('name', 'asc')->find_all();		
		} else {
			$this->load->model('locations/cities_model');
			$result = $this->cities_model->select('id, name')->where('state_id', $loadId)->order_by('name', 'asc')->find_all();	
		}
		
		$HTML="";
	
		if($result) {
			foreach($result as $list){
				$HTML.="<option value='".$list->id."'>".$list->name."</option>";
			}
		}
		echo $HTML;
	}
	

	//--------------------------------------------------------------------

	//--------------------------------------------------------------------
	// !PRIVATE METHODS
	//--------------------------------------------------------------------

	/**
	 * Performs the form validation and saves the settings to the database
	 *
	 * @access private
	 *
	 * @param array	$extended_settings	An optional array of settings from the extended_settings config file
	 *
	 * @return bool
	 */
	private function save_settings($extended_settings=array(), $general_settings=array())
	{
	    // validate translation fields
	    $this->translations->validate_prepare_fields();
	    
	    // Advanced
	    /*foreach(gc_modules() as $gc_module):
	    $this->form_validation->set_rules('search_fields[' . $gc_module . ']', lang($gc_module) . ' ' . lang('search_fields'), 'trim|required');
	    endforeach;*/
		$this->form_validation->set_rules('title', 'lang:bf_site_name', 'required|trim');
		$this->form_validation->set_rules('site_logo', 'lang:site_logo', 'required|trim');
		$this->form_validation->set_rules('site_logo_contrast', 'lang:site_logo_contrast', 'trim|required');
		$this->form_validation->set_rules('site_logo_pdf', 'lang:site_logo_pdf', 'trim|required');
		$this->form_validation->set_rules('system_email', 'lang:bf_site_email', 'required|trim|valid_email');
		$this->form_validation->set_rules('list_limit','Items <em>p.p.</em>', 'required|trim|numeric');
		$this->form_validation->set_rules('front_list_limit','Items <em>p.p.</em>', 'required|trim|numeric');
		$this->form_validation->set_rules('site_currency_code', 'lang:singular_currencies', 'trim');
		$this->form_validation->set_rules('site_currency_restrict', 'lang:site_currency_restrict', 'trim');
		$this->form_validation->set_rules('multi_language', 'lang:multi_language', 'trim');
		$this->form_validation->set_rules('theme_switcher', 'lang:theme_switcher', 'trim');
		$this->form_validation->set_rules('theme_scheme', 'lang:theme_scheme', 'trim');
		$this->form_validation->set_rules('display_top_menu', 'lang:display_top_menu', 'trim');
		$this->form_validation->set_rules('display_email', 'lang:display_email', 'trim');
		$this->form_validation->set_rules('site_address', 'lang:label_address', 'trim');
		$this->form_validation->set_rules('phone_number', 'lang:site_call_us', 'trim');
		$this->form_validation->set_rules('search_blocks', 'lang:search_blocks', 'trim');
		$this->form_validation->set_rules('search_location', 'lang:search_location', 'trim');
		$this->form_validation->set_rules('country_iso', 'lang:default_country', 'trim|required');
		$this->form_validation->set_rules('state_id', 'lang:default_state', 'trim|required');
		$this->form_validation->set_rules('city_id', 'lang:default_city', 'trim|required');
		$this->form_validation->set_rules('password_min_length','lang:bf_password_length', 'required|trim|numeric');
		$this->form_validation->set_rules('password_force_numbers', 'lang:bf_password_force_numbers', 'trim|numeric');
		$this->form_validation->set_rules('password_force_symbols', 'lang:bf_password_force_symbols', 'trim|numeric');
		$this->form_validation->set_rules('password_force_mixed_case', 'lang:bf_password_force_mixed_case', 'trim|numeric');
		$this->form_validation->set_rules('password_show_labels', 'lang:bf_password_show_labels', 'trim|numeric');
		$this->form_validation->set_rules('languages[]', 'lang:bf_language', 'trim|required|is_array');
		$this->form_validation->set_rules('date_format', 'lang:date_time_format', 'required|trim');
		$this->form_validation->set_rules('time_format', 'lang:date_time_format', 'required|trim');
		$this->form_validation->set_rules('category_listings_count', 'lang:category_listings_count', 'trim');
		$this->form_validation->set_rules('hide_counts_zero_categories', 'lang:hide_counts_zero_categories', 'trim');
		$this->form_validation->set_rules('hide_counts_zero_locations', 'lang:hide_counts_zero_locations', 'trim');
		$this->form_validation->set_rules('view_map', 'lang:view_map', 'trim');
		$this->form_validation->set_rules('map_leaflet', 'lang:map_leaflet', 'trim');
		$this->form_validation->set_rules('geoip', 'lang:label_geoip', 'trim');		
		$this->form_validation->set_rules('invoice_prefix', 'lang:invoice_prefix', 'trim|required|max_length[3]');
		$this->form_validation->set_rules('order_prefix', 'lang:order_prefix', 'trim|required|max_length[3]');
		$this->form_validation->set_rules('otp_login', 'lang:otp_login', 'trim');
		$this->form_validation->set_rules('otp_timeout', 'lang:otp_timeout', 'trim|required|numeric|max_length[5]');
		$this->form_validation->set_rules('max_otp_attempts', 'lang:max_otp_attempts', 'trim|required|numeric|max_length[5]|greater_than[0]');
		$this->form_validation->set_rules('loggedin_review_only', 'lang:loggedin_review_only', 'trim');
		$this->form_validation->set_rules('loggedin_rating_only', 'lang:loggedin_rating_only', 'trim');
		$this->form_validation->set_rules('google_api_key', 'lang:google_api_key', 'trim');
		$this->form_validation->set_rules('google_server_api_key', 'lang:google_server_api_key', 'trim');
		$this->form_validation->set_rules('whitelist', 'lang:whitelist', 'trim');
		
		// Ecommerce
		$this->form_validation->set_rules('site_ecommerce', 'lang:site_ecommerce', 'trim');
		// $this->form_validation->set_rules('sms_notify_stock', 'lang:sms_notify_stock', 'trim');
		// $this->form_validation->set_rules('email_notify_seller', 'lang:email_notify_seller', 'trim');
		// $this->form_validation->set_rules('sms_notify_seller', 'lang:sms_notify_seller', 'trim');
		
		// Listings
		$this->form_validation->set_rules('business_hours_format', 'lang:business_hours_format', 'trim');
		$this->form_validation->set_rules('business_query', 'lang:label_business_query', 'trim');
		$this->form_validation->set_rules('business_query_status', 'lang:business_query_status', 'trim');
		$this->form_validation->set_rules('query_email_notification', 'lang:query_email_notification', 'trim');
		$this->form_validation->set_rules('query_sms_notification', 'lang:query_sms_notification', 'trim');
		$this->form_validation->set_rules('sms_notify_owner', 'lang:sms_notify_owner', 'trim');
		$this->form_validation->set_rules('allow_country_selection', 'lang:allow_country_selection', 'trim');
		$this->form_validation->set_rules('country_filter', 'lang:country_filter', 'trim');
		$this->form_validation->set_rules('category_city_filter', 'lang:category_city_filter', 'trim');
		$this->form_validation->set_rules('google_places_api', 'lang:google_places_api', 'trim');
		$this->form_validation->set_rules('google_business', 'lang:google_business', 'trim');
		$this->form_validation->set_rules('logo_file_size', 'lang:logo_file_size', 'trim|required|is_numeric|greater_than[0]');
		$this->form_validation->set_rules('logo_width', 'lang:logo_pixels', 'trim|required|is_numeric|greater_than[0]');
		$this->form_validation->set_rules('logo_height', 'lang:logo_pixels', 'trim|required|is_numeric|greater_than[0]');
		$this->form_validation->set_rules('cover_file_size', 'lang:cover_file_size', 'trim|required|is_numeric|greater_than[0]');
		$this->form_validation->set_rules('cover_width', 'lang:cover_pixels', 'trim|required|is_numeric|greater_than[0]');
		$this->form_validation->set_rules('cover_height', 'lang:cover_pixels', 'trim|required|is_numeric|greater_than[0]');
		$this->form_validation->set_rules('image_file_size', 'lang:image_file_size', 'trim|required|is_numeric|greater_than[0]');
		$this->form_validation->set_rules('image_width', 'lang:image_pixels', 'trim|required|is_numeric|greater_than[0]');
		$this->form_validation->set_rules('image_height', 'lang:image_pixels', 'trim|required|is_numeric|greater_than[0]');
		$this->form_validation->set_rules('thumbnail_width', 'lang:thumbnail_pixels', 'trim|required|is_numeric|greater_than[0]');
		$this->form_validation->set_rules('thumbnail_height', 'lang:thumbnail_pixels', 'trim|required|is_numeric|greater_than[0]');
		$this->form_validation->set_rules('allow_review', 'lang:allow_review', 'trim');
		$this->form_validation->set_rules('review_images', 'lang:review_images', 'trim');
		$this->form_validation->set_rules('auto_approve_review', 'lang:auto_approve_review', 'trim');
		$this->form_validation->set_rules('auto_approve_tag', 'lang:auto_approve_tag', 'trim');
		$this->form_validation->set_rules('slug_url', 'lang:slug_url', 'trim');
		$this->form_validation->set_rules('city_url', 'lang:city_url', 'trim');
		/*$this->form_validation->set_rules('show_address', 'lang:show_address', 'trim');
		$this->form_validation->set_rules('show_phone', 'lang:show_phone', 'trim');
		$this->form_validation->set_rules('show_email', 'lang:show_email', 'trim');
		$this->form_validation->set_rules('allow_linkedin_url', 'lang:allow_linkedin_url', 'trim');
		$this->form_validation->set_rules('allow_facebook_url', 'lang:allow_facebook_url', 'trim');
		$this->form_validation->set_rules('allow_googleplus_url', 'lang:allow_googleplus_url', 'trim');
		$this->form_validation->set_rules('allow_twitter_url', 'lang:allow_twitter_url', 'trim');*/
		
		// SEO
		$this->form_validation->set_rules('enable_site_title', 'lang:enable_site_title', 'trim');
		$this->form_validation->set_rules('use_h1', 'lang:use_h1', 'trim');
		
		// Developer Rules
		$this->form_validation->set_rules('download_id', 'lang:download_id', 'trim');
		$this->form_validation->set_rules('hourly_emails_limit','lang:hourly_emails_limit', 'required|trim|numeric');
		$this->form_validation->set_rules('enable_cache', 'lang:enable_cache', 'trim');
		$this->form_validation->set_rules('full_text_search', 'lang:full_text_search', 'trim');
		$this->form_validation->set_rules('show_profiler', 'lang:bf_show_profiler', 'trim');
		$this->form_validation->set_rules('show_front_profiler', 'lang:bf_show_front_profiler', 'trim');
		
		$this->form_validation->set_rules('home_featured_listings', 'lang:home_featured_listings', 'trim|ids_limit[50]|required');
		
		// Site Logo
		$this->form_validation->set_rules('uploaded_image', 'lang:label_logo', 'trim');
		if(isset($_FILES['site_logo']) && $_FILES['site_logo']['size'] > 0){
			$file_data = $this->upload_logo_file();
			if(!$file_data) {
				$_POST['file_upload'] = 'NONE';
				$this->form_validation->set_rules('file_upload', 'label:label_image', 'valid_file_upload');
			}
			// Avoid logo upload process, if the form has validation issues with other fields
			$_POST['uploaded_image'] = isset($file_data['upload_data']['file_name']) ? $file_data['upload_data']['file_name'] : '';
		}

		// setup the validation rules for any extended settings
		$extended_data = array();
		foreach ($extended_settings as $field)
		{
			if ( empty($field['permission'])
				|| $field['permission'] === FALSE
				|| ( ! empty($field['permission']) && has_permission($field['permission']) )
				)
			{
				$this->form_validation->set_rules($field['name'], $field['label'], $field['rules']);
				$extended_data['ext.' . $field['name']] = $this->input->post($field['name']);
			}
		}
		
		// setup the validation rules for any general settings
		$general_data = array();
		foreach ($general_settings as $field) {
			if ( empty($field['permission'])
					|| $field['permission'] === FALSE
					|| ( ! empty($field['permission']) && has_permission($field['permission']) )
			)
			{
				$this->form_validation->set_rules($field['name'], $field['label'], $field['rules']);
				$general_data['site.' . $field['name']] = $this->input->post($field['name']);
			}
		}

		if ($this->form_validation->run() === FALSE) {
			return FALSE;
		}
		
		// unset non existing translation POST fields
		$this->translations->translate_unset();
		
		// upload analytics key file
		if (isset($_FILES['analytics_key_file']) && is_uploaded_file($_FILES['analytics_key_file']['tmp_name'])) {
			$file_data = $this->upload_key_file(); // upload logo file
			$key_file = isset($file_data['upload_data']['file_name']) ? $file_data['upload_data']['file_name'] : ''; // retrieve file name of uploaded file
		} else {
			$key_file = settings_item('seo.analytics_key_file'); // retrieve old file name and update database
		}
		$currency = explode('|', $this->input->post('currency') ?? '');
		
		// Get the Language code for translations
		$language_code = 'en';
		$code = $this->db->select('code')->get_where('languages', array('label' => $this->input->post('default_language')))->row();
		if($code) {
		    $language_code = $code->code;
		}
		
		$data = array(
			array('name' => 'site.title', 'value' => $this->input->post('title')),
		    array('name' => 'site.logo', 'value' => $this->input->post('site_logo')),
		    array('name' => 'site.logo_contrast', 'value' => $this->input->post('site_logo_contrast')),
		    array('name' => 'site.logo_pdf', 'value' => $this->input->post('site_logo_pdf')),
			array('name' => 'site.system_email', 'value' => $this->input->post('system_email')),
			array('name' => 'site.status', 'value' => $this->input->post('status')),
			array('name' => 'site.list_limit', 'value' => $this->input->post('list_limit')),
			array('name' => 'site.front_list_limit', 'value' => $this->input->post('front_list_limit')),
			array('name' => 'site.currency_symbol', 'value' => $currency[0]),
		    array('name' => 'site.currency_code', 'value' => $currency[1]),
		    array('name' => 'site.currency_restrict', 'value' => $this->input->post('site_currency_restrict') ? 1 : 0),
			array('name' => 'site.default_language', 'value' => $this->input->post('default_language')),
		    array('name' => 'site.language_code', 'value' => $language_code),
		    array('name' => 'site.multi_language', 'value' => isset($_POST['multi_language']) ? 1 : 0),
			array('name' => 'site.date_format', 'value' => $this->input->post('date_format')),
			array('name' => 'site.time_format', 'value' => $this->input->post('time_format')),
		    array('name' => 'site.google_api_key', 'value' => $this->input->post('google_api_key')),
		    array('name' => 'site.google_server_api_key', 'value' => $this->input->post('google_server_api_key') ? $this->input->post('google_server_api_key') : 'demo'),
			array('name' => 'site.geoip', 'value' => $this->input->post('geoip')),
			array('name' => 'site.whitelist', 'value' => $this->input->post('whitelist')),
		    
		    // Ecommerce
		    array('name' => 'site.ecommerce', 'value' => $this->input->post('site_ecommerce') ? 1 : 0),
		    // array('name' => 'ecom.sms_notify_stock', 'value' => $this->input->post('sms_notify_stock') ? 1 : 0),
		    // array('name' => 'ecom.email_notify_seller', 'value' => $this->input->post('email_notify_seller') ? 1 : 0),
		    // array('name' => 'ecom.sms_notify_seller', 'value' => $this->input->post('sms_notify_seller') ? 1 : 0),
		    
			array('name' => 'auth.allow_register', 'value' => isset($_POST['allow_register']) ? 1 : 0),
			array('name' => 'auth.notify_admin', 'value' => isset($_POST['new_user_notify_admin']) ? 1 : 0),
			array('name' => 'auth.otp_login', 'value' => isset($_POST['otp_login']) ? 1 : 0),
			array('name' => 'auth.content_protector', 'value' => isset($_POST['content_protector']) ? 1 : 0),
			array('name' => 'auth.restrict_visitors', 'value' => isset($_POST['restrict_visitors']) ? 1 : 0),
			array('name' => 'auth.per_day_limit', 'value' => $this->input->post('per_day_limit')),
		    array('name' => 'auth.google_recaptcha', 'value' => isset($_POST['google_recaptcha']) ? 1 : 0),
			array('name' => 'auth.recaptcha_site_key', 'value' => $this->input->post('recaptcha_site_key')),
			array('name' => 'auth.recaptcha_secret_key', 'value' => $this->input->post('recaptcha_secret_key')),
			array('name' => 'auth.user_activation_method', 'value' => isset($_POST['user_activation_method']) ? $_POST['user_activation_method'] : 0),
			array('name' => 'auth.login_type', 'value' => $this->input->post('login_type')),
			array('name' => 'auth.use_usernames', 'value' => isset($_POST['use_usernames']) ? $this->input->post('use_usernames') : 0),
			array('name' => 'auth.allow_remember', 'value' => isset($_POST['allow_remember']) ? 1 : 0),
			array('name' => 'auth.remember_length', 'value' => (int)$this->input->post('remember_length')),
			array('name' => 'auth.use_extended_profile', 'value' => isset($_POST['use_ext_profile']) ? 1 : 0),
			array('name' => 'auth.allow_name_change', 'value' => $this->input->post('allow_name_change') ? 1 : 0),
			array('name' => 'auth.name_change_frequency', 'value' => $this->input->post('name_change_frequency')),
			array('name' => 'auth.name_change_limit', 'value' => $this->input->post('name_change_limit')),
			array('name' => 'auth.password_min_length', 'value' => $this->input->post('password_min_length')),
			array('name' => 'auth.password_force_numbers', 'value' => $this->input->post('password_force_numbers') ? 1 : 0),
			array('name' => 'auth.password_force_symbols', 'value' => $this->input->post('password_force_symbols') ? 1 : 0),
			array('name' => 'auth.password_force_mixed_case', 'value' => $this->input->post('password_force_mixed_case') ? 1 : 0),
			array('name' => 'auth.password_show_labels', 'value' => $this->input->post('password_show_labels') ? 1 : 0),
			array('name' => 'auth.otp_timeout', 'value' => $this->input->post('otp_timeout')),
		    array('name' => 'auth.max_otp_attempts', 'value' => $this->input->post('max_otp_attempts')),
		    array('name' => 'site.package_views_limit', 'value' => isset($_POST['package_views_limit']) ? 1 : 0),
			
			// Developer Settings
			array('name' => 'site.download_id', 'value' => $this->input->post('download_id')),
			array('name' => 'site.hourly_emails_limit', 'value' => $this->input->post('hourly_emails_limit')),
			array('name' => 'updates.do_check', 'value' => isset($_POST['do_check']) ? 1 : 0),
			array('name' => 'updates.bleeding_edge', 'value' => isset($_POST['bleeding_edge']) ? 1 : 0),
			array('name' => 'site.show_profiler', 'value' => isset($_POST['show_profiler']) ? 1 : 0),
			array('name' => 'site.enable_cache', 'value' => isset($_POST['enable_cache']) ? 1 : 0),
			array('name' => 'site.full_text_search', 'value' => isset($_POST['full_text_search']) ? 1 : 0),
			array('name' => 'site.show_front_profiler', 'value' => isset($_POST['show_front_profiler']) ? 1 : 0),
			array('name' => 'site.languages', 'value' => $this->input->post('languages') != '' ? serialize($this->input->post('languages')) : ''),
			array('name' => 'password_iterations', 'value' => $this->input->post('password_iterations')),
				
			
			// Advanced Settings
			// array('name' => 'site.display_top_menu', 'value' => $this->input->post('display_top_menu') ? 1 : 0),
		    array('name' => 'site.display_email', 'value' => $this->input->post('display_email') ? 1 : 0),
		    array('name' => 'site.address', 'value' => $this->input->post('site_address')),		 
		    array('name' => 'site.phone_number', 'value' => $this->input->post('phone_number')),		    
		    array('name' => 'site.map_location', 'value' => $this->input->post('map_location')),
		    // array('name' => 'adv.search_fields', 'value' => json_encode($this->input->post('search_fields'))),
			// array('name' => 'adv.search_blocks', 'value' => $this->input->post('search_blocks')),
			// array('name' => 'adv.search_location', 'value' => $this->input->post('search_location')),
			array('name' => 'adv.default_country', 'value' => $this->input->post('country_iso')),
			array('name' => 'adv.default_state', 'value' => $this->input->post('state_id')),
			array('name' => 'adv.default_city', 'value' => $this->input->post('city_id')),
				
			// Listing Settings
		    array('name' => 'lst.notify_admin', 'value' => isset($_POST['new_listing_notify_admin']) ? 1 : 0),
		    array('name' => 'lst.business_hours_format', 'value' => isset($_POST['business_hours_format']) ? 1 : 0),
		    array('name' => 'lst.business_query', 'value' => isset($_POST['business_query']) ? 1 : 0),
			array('name' => 'lst.business_query_status', 'value' => isset($_POST['business_query_status']) ? 1 : 0),
			array('name' => 'lst.top_advertisement', 'value' => $this->input->post('top_advertisement') ? 1 : 0),
		    array('name' => 'lst.category_listings_count', 'value' => isset($_POST['category_listings_count']) ? 1 : 0),
		    array('name' => 'lst.hide_counts_zero_categories', 'value' => isset($_POST['hide_counts_zero_categories']) ? 1 : 0),
		    array('name' => 'lst.hide_counts_zero_locations', 'value' => isset($_POST['hide_counts_zero_locations']) ? 1 : 0),
			array('name' => 'lst.view_map', 'value' => isset($_POST['view_map']) ? 1 : 0),
		    array('name' => 'lst.map_leaflet', 'value' => isset($_POST['map_leaflet']) ? 1 : 0),
			array('name' => 'lst.latitude', 'value' => $this->input->post('latitude')),
			array('name' => 'lst.longitude', 'value' => $this->input->post('longitude')),
			array('name' => 'lst.advertisement_page', 'value' => $this->input->post('advertisement_page')),
			array('name' => 'lst.home_featured_listings', 'value' => $this->input->post('home_featured_listings')),
			array('name' => 'lst.top_featured_count', 'value' => $this->input->post('top_featured_count')),
			array('name' => 'lst.right_featured_count', 'value' => $this->input->post('right_featured_count')),
			array('name' => 'lst.logo_file_size', 'value' => $this->input->post('logo_file_size')),
			array('name' => 'lst.logo_width', 'value' => $this->input->post('logo_width')),
			array('name' => 'lst.logo_height', 'value' => $this->input->post('logo_height')),
			array('name' => 'lst.cover_file_size', 'value' => $this->input->post('cover_file_size')),
			array('name' => 'lst.cover_width', 'value' => $this->input->post('cover_width')),
			array('name' => 'lst.cover_height', 'value' => $this->input->post('cover_height')),
			array('name' => 'lst.image_file_size', 'value' => $this->input->post('image_file_size')),
			array('name' => 'lst.image_width', 'value' => $this->input->post('image_width')),
			array('name' => 'lst.image_height', 'value' => $this->input->post('image_height')),
			array('name' => 'lst.thumbnail_width', 'value' => $this->input->post('thumbnail_width')),
			array('name' => 'lst.thumbnail_height', 'value' => $this->input->post('thumbnail_height')),
			array('name' => 'lst.click_to_call', 'value' => $this->input->post('click_to_call') ? 1 : 0),
			array('name' => 'lst.allow_sms', 'value' => $this->input->post('allow_sms') ? 1 : 0),
			array('name' => 'lst.allow_email', 'value' => $this->input->post('allow_email') ? 1 : 0),
			array('name' => 'lst.allow_print', 'value' => $this->input->post('allow_print') ? 1 : 0),
		    array('name' => 'lst.allow_review', 'value' => $this->input->post('allow_review') ? 1 : 0),
		    array('name' => 'lst.review_images', 'value' => $this->input->post('review_images') ? 1 : 0),
		    array('name' => 'lst.auto_approve_review', 'value' => $this->input->post('auto_approve_review') ? 1 : 0),
			array('name' => 'lst.loggedin_review_only', 'value' => $this->input->post('loggedin_review_only') ? 1 : 0),
		    array('name' => 'lst.loggedin_rating_only', 'value' => $this->input->post('loggedin_rating_only') ? 1 : 0),
		    array('name' => 'lst.auto_approve_tag', 'value' => $this->input->post('auto_approve_tag') ? 1 : 0),
		    array('name' => 'lst.google_business', 'value' => $this->input->post('google_business') ? 1 : 0),
		    array('name' => 'lst.slug_url', 'value' => $this->input->post('slug_url') ? 1 : 0),
		    array('name' => 'lst.city_url', 'value' => $this->input->post('city_url') ? 1 : 0),
						
			/*array('name' => 'lst.show_address', 'value' => $this->input->post('show_address') ? 1 : 0),
			array('name' => 'lst.show_phone', 'value' => $this->input->post('show_phone') ? 1 : 0),
			array('name' => 'lst.show_email', 'value' => $this->input->post('show_email') ? 1 : 0),
			array('name' => 'lst.allow_linkedin_url', 'value' => $this->input->post('allow_linkedin_url') ? 1 : 0),
			array('name' => 'lst.allow_facebook_url', 'value' => $this->input->post('allow_facebook_url') ? 1 : 0),
			array('name' => 'lst.allow_twitter_url', 'value' => $this->input->post('allow_twitter_url') ? 1 : 0),
			array('name' => 'lst.allow_googleplus_url', 'value' => $this->input->post('allow_googleplus_url') ? 1 : 0),
			*/
			array('name' => 'lst.allow_country_selection', 'value' => $this->input->post('allow_country_selection') ? 1 : 0),
			array('name' => 'lst.google_places_api', 'value' => $this->input->post('google_places_api') ? 1 : 0),
			array('name' => 'lst.country_filter', 'value' => $this->input->post('country_filter') ? 1 : 0),
			array('name' => 'lst.category_city_filter', 'value' => $this->input->post('category_city_filter') ? 1 : 0),
			array('name' => 'lst.query_email_notification', 'value' => $this->input->post('query_email_notification') ? 1 : 0),
		    array('name' => 'lst.query_sms_notification', 'value' => $this->input->post('query_sms_notification') ? 1 : 0),
		    array('name' => 'lst.sms_notify_owner', 'value' => $this->input->post('sms_notify_owner') ? 1 : 0),
				
		    // Financial Settings
		    array('name' => 'fin.user_packages_only', 'value' => $this->input->post('user_packages_only') ? 1 : 0),
			array('name' => 'fin.allow_coupons', 'value' => $this->input->post('allow_coupons') ? 1 : 0),
			array('name' => 'fin.enable_tax', 'value' => $this->input->post('enable_tax') ? 1 : 0),
			array('name' => 'fin.tax_settings', 'value' => $this->input->post('tax_settings')),
			array('name' => 'fin.terms_and_conditions', 'value' => $this->input->post('terms_and_conditions') ? 1 : 0),
			array('name' => 'fin.invoice_prefix', 'value' => $this->input->post('invoice_prefix')),
			array('name' => 'fin.order_prefix', 'value' => $this->input->post('order_prefix')),
			array('name' => 'fin.terms_and_conditions_page', 'value' => $this->input->post('terms_and_conditions_page')),
				
			// Analytics Settings
			array('name' => 'seo.allow_analytics', 'value' => $this->input->post('allow_analytics') ? 1 : 0),
			array('name' => 'seo.analytics_key_file', 'value' => $key_file),
			array('name' => 'seo.analytics_property_id', 'value' => $this->input->post('analytics_property_id')),
			array('name' => 'seo.analytics_rows_limit', 'value' => $this->input->post('analytics_rows_limit')),
				
			// SEO Settings	
			array('name' => 'seo.site_title', 'value' => $this->input->post('enable_site_title') ? 1 : 0),
			array('name' => 'seo.use_h1', 'value' => $this->input->post('use_h1') ? 1 : 0),
			array('name' => 'seo.site_title_format', 'value' => $this->input->post('site_title_format')),
			array('name' => 'site.meta_title', 'value' => $this->input->post('meta_title')),
			array('name' => 'site.meta_description', 'value' => $this->input->post('meta_description')),
			array('name' => 'site.meta_keywords', 'value' => $this->input->post('meta_keywords')),	
			array('name' => 'seo.listing_meta_title', 'value' => $this->input->post('listing_meta_title')),
			array('name' => 'seo.listing_meta_description', 'value' => $this->input->post('listing_meta_description')),
			array('name' => 'seo.listing_meta_keywords', 'value' => $this->input->post('listing_meta_keywords')),
			array('name' => 'seo.category_meta_title', 'value' => $this->input->post('category_meta_title')),
			array('name' => 'seo.category_meta_description', 'value' => $this->input->post('category_meta_description')),
			array('name' => 'seo.category_meta_keywords', 'value' => $this->input->post('category_meta_keywords')),
			array('name' => 'seo.location_meta_title', 'value' => $this->input->post('location_meta_title')),
			array('name' => 'seo.location_meta_description', 'value' => $this->input->post('location_meta_description')),
			array('name' => 'seo.location_meta_keywords', 'value' => $this->input->post('location_meta_keywords')),
			array('name' => 'seo.search_meta_title', 'value' => $this->input->post('search_meta_title')),
			array('name' => 'seo.search_meta_description', 'value' => $this->input->post('search_meta_description')),
			array('name' => 'seo.search_meta_keywords', 'value' => $this->input->post('search_meta_keywords')),
			array('name' => 'seo.page_meta_title', 'value' => $this->input->post('page_meta_title')),
			array('name' => 'seo.page_meta_description', 'value' => $this->input->post('page_meta_description')),
			array('name' => 'seo.page_meta_keywords', 'value' => $this->input->post('page_meta_keywords'))			
		);

		//destroy the saved update message in case they changed update preferences.
		if ($this->cache->get('update_message')) {
			$this->cache->delete('update_message');
		}

		// Log the activity
		log_activity($this->auth->user_id(), lang('bf_act_settings_saved').': ' . $this->input->ip_address(), 'core');

		// Hide H1 tag in the Header Logo and make it to look like a real tag to the Search Engines
		if(settings_item('site.logo') != $this->input->post('site_logo')) {
			// Recomile the global.less file if the site logo is changed
			$this->load->library('themes/widgets');
			$this->widgets->compile_global($this->input->post('site_logo'));
		}		
		// save the settings to the DB
		$updated = $this->settings_model->update_batch($data, 'name');

		// if the update was successful and we have extended settings to save,
		if ($updated && ! empty($extended_data))
		{
			// go ahead and save them
			$updated = $this->save_extended_settings($extended_data);
		}
		
		// if the update was successful and we have general settings to save,
		if ($updated && ! empty($general_data))
		{
			/*$general_data['site.footer_subscribers'] = $general_data['site.footer_subscribers'] ? 1 : 0;
			$general_data['site.subscriber_notify_admin'] = $general_data['site.subscriber_notify_admin'] ? 1 : 0;
			$general_data['site.display_footer_popular'] = $general_data['site.display_footer_popular'] ? 1 : 0;
			$general_data['site.copyright_info'] = $general_data['site.copyright_info'] ? 1 : 0;*/
			// go ahead and save them
			$updated = $this->save_general_settings($general_data);
		}
		
		// Set ['enable_hooks'] = TRUE; in application/config.php file
		if($this->input->post('status') == 0) {
			file_put_contents('../application/config/config.php', str_replace("['enable_hooks'] = FALSE;", "['enable_hooks'] = TRUE;", file_get_contents('../application/config/config.php')));
		}

		// save translations
		$this->translations->save_translations('settings', 1, 1);
		
		// Clear multilanguage session variable defined in the translations
		clear_cache('site_settings');
		
		// clear field languages
		$this->session->unset_userdata('field_languages');
		
		// update site information
		$this->load->library('statistics');
		$this->statistics->info();
		clear_cache('gc_stats_settings');

		// check download id string - if changed remove the license file
		$file_path = APPPATH . GC_AUTHENTICATE;
		if(file_exists($file_path)) {
			$download_id = $this->input->post('download_id');
			$file_array = file($file_path);
			if(empty($file_array[0]) || $file_array[0] != $download_id) {
				unlink($file_path);
			}
		}
		
		return $updated;

	}//end save_settings()
	
	/**
	 * Save user selected validation rules
	 * @param string $field_name
	 * @return string
	 */
	private function post_validation_rules($field_name) {
		$rules = $this->input->post($field_name . '_rules');
		$valildation_radio_rule = $field_name . '_radio_rule';
		if($this->input->post($valildation_radio_rule) && $this->input->post($valildation_radio_rule) != 'none') {
			$rules[] = $this->input->post($field_name . '_radio_rule');
		}
		return serialize($rules);
	}

	//--------------------------------------------------------------------

	/**
	 * Save the extended settings
	 *
	 * @access private
	 * @param	array	$extended_data	An array of settings to save
	 * @return	mixed/bool	TRUE or an inserted id if all settings saved successfully, else FALSE
	 */
	private function save_extended_settings($extended_data)
	{
		if ( ! is_array($extended_data)
			|| empty($extended_data)
			|| ! count($extended_data)
			)
		{
			return FALSE;
		}

		$setting = FALSE;
		foreach ($extended_data as $key => $value)
		{
			$setting = $this->settings_lib->set($key, $value);
			if ($setting === FALSE)
			{
				return FALSE;
			}
		}

		return $setting;
	}// end save_extended_settings()
	
	/**
	 * Save the general settings
	 *
	 * @access private
	 * @param	array	$general_data	An array of settings to save	 *
	 * @return	mixed/bool	TRUE or an inserted id if all settings saved successfully, else FALSE
	 */
	private function save_general_settings($general_data) {
		if ( ! is_array($general_data)
				|| empty($general_data)
				|| ! count($general_data)
		) {
			return FALSE;
		}
	
		$setting = FALSE;
		foreach ($general_data as $key => $value) {
			$setting = $this->settings_lib->set($key, $value);
			if ($setting === FALSE) {
				return FALSE;
			}
		}
	
		return $setting;
	}// end save_general_settings()

		/* -------------------------UPLOAD LOGO FUNCTION --------------------------------------*/
	/**
	 * Save uploaded image
	 * @return multitype:NULL
	 */
	private function upload_logo_file() {
	    if(!gc_demo_mode(array('no_message' => true, 'no_redirect' => true))) {
	        return false;
	    }	
		$config['upload_path'] = realpath(FCPATH.'assets/images/'); //Make SURE that you chmod this directory to 777!
		$config['allowed_types'] = 'gif|jpg|png';
		$config['max_size']= $this->file_size;
		$config['max_width']= $this->max_width;
		$config['max_height']= $this->max_height;
		$config['remove_spaces']=TRUE; //Remove spaces from the file name
	
		$this->load->library('upload', $config);	
		if (!$this->upload->do_upload('site_logo')) {// Image name is necessary
			$this->session->set_flashdata('upload_image_error', $this->upload->display_errors());
			return false;
		} else {
			$data = array('upload_data' => $this->upload->data());
		//	$this->resize($data['upload_data']['full_path'],$data['upload_data']['file_name']);
		}
		return $data;
	}
	
	/**
	 * Save analytics key
	 * @return multitype:NULL
	 */
	private function upload_key_file() {
	    if(!gc_demo_mode(array('no_message' => true, 'no_redirect' => true))) {
	        return false;
	    }	
		$config['upload_path'] = realpath(APPPATH.'./third_party/Google/'); //Make SURE that you chmod this directory to 777!
		$config['allowed_types'] = 'json';
		$config['max_size']= 100;
	
		$this->load->library('upload', $config);	
		if (!$this->upload->do_upload('analytics_key_file')) // Image name is necessary
		{
			Template::set_message($this->upload->display_errors(), 'error'); // Show errors to the user
			redirect(SITE_AREA .'/settings/');
		} else {
			$data = array('upload_data' => $this->upload->data());
		}
		return $data;
	}
	
	/**
	 * Create thumbs of uploaded image
	 */
	public function resize($path,$file){
		$config['image_library']='gd2';
		$config['source_image']=$path;
		//$config['create_thumb']=TRUE; // This will create new file
		$config['maintain_ratio']=TRUE;
		$config['width']= $this->resize_width;
		$config['height']= $this->resize_height;
		$config['new_image']='./assets/images/'.$file; //CHANGE THIS LINE FOR PATH
	
		$this->load->library('image_lib',$config);
		$this->image_lib->resize();
	}
	
	/* ------------------------ END OF UPLOAD LOGO ------------------------------------- */
}//end Settings()
