<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Settings controller
 */
class Settings extends Admin_Controller
{
	protected $permissionView   = 'SMS_Gateways.Settings.View';
	protected $permissionEdit   = 'SMS_Gateways.Settings.Edit';	

	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct()
	{
		parent::__construct();

		$this->auth->restrict($this->permissionView);
		$this->lang->load('sms_gateways');
		$this->load->model('sms_gateways_model', null, true);
		
		$config['api_key'] = 'xxxxxx';
		$config['api_secret'] = 'xxxxxx';
		$this->load->library('nexmo', $config);
		
		Template::set_block('sub_nav', 'settings/_sub_nav');

		Assets::add_module_js('sms_gateways', 'sms_gateways.js');
	}


	/**
	 * Displays a list of form data.
	 *
	 * @return void
	 */
	public function index($offset = 0) {
		// Status Change
		if (isset($_POST['status_change'])) {
		    $this->gcrud->stats(array('module' => 'sms_gateways'));
			$id = $_POST['status_change'];
			// Get Old Status
			// Only One SMS Gateway can be set Active at a Time
			$gateway = $this->sms_gateways_model->find($id);
			if($gateway && $gateway->active == 0) {
				//SET DEFAULT 1 FOR SELECTED ROW
				//GET IDS OF ALL ROWS EXCEPT SPECIFIED ROW
				$result = $this->sms_gateways_model->select('id')->where('id NOT IN (' . $id .')')->find_all();
				if($result) {
					//STORE VALUE IN AN ARRAY
					foreach ($result as $row) {
						$rows_to_update[] = $row->id;
					}
					$this->db->where('id IN (' . implode(',',$rows_to_update) . ')')->update('sms_gateways', array('active' => 0));
				}
			}
			$this->gc->status_change('sms_gateways', $_POST['status_change'], 'active');			
		}
		
		// Sort By
		$order_by = $this->gc->sort('sms_gateways');
		if(!empty($order_by)) {
			$this->sms_gateways_model->order_by($this->session->userdata('sms_gateways_sort')['column'], $this->session->userdata('sms_gateways_sort')['order']);
		} else {
			$this->sms_gateways_model->order_by('name', 'ASC');
		}
		$this->sms_gateways_model->limit($this->limit, $offset);
		$records = $this->sms_gateways_model->find_all();
		
		// Pagination
		$this->load->library('pagination');
		
		$total_gateways = $this->sms_gateways_model->count_all();;
		
		$pager['base_url'] = site_url(SITE_AREA ."/settings/sms_gateways/index/");
		$pager['total_rows'] = $total_gateways;
		$pager['per_page'] = $this->limit;
		$pager['uri_segment']	= 5;
		
		$this->pagination->initialize($pager);

		Template::set('records', $records);
		Template::set('toolbar_title', lang('manage_sms_gateways'));
		Template::render();
	}
	
	/*----------------------------------------------------*/
	/*	NEXMO GATEWAY
	/*----------------------------------------------------*/
	/**
	 * Allows editing of SMSGlobal Payment Gateway data.
	 *
	 * @return void
	 */
	public function nexmo()
	{
		$this->load->model('sms_gateways_model');
		$id = $this->uri->segment(5);
	
		if (empty($id))
		{
			Template::set_message(lang('invalid_id'), 'error');
			redirect(SITE_AREA .'/settings/sms_gateways');
		}
	
		if (isset($_POST['save']))
		{
		    $this->gcrud->stats(array('module' => 'sms_gateways'));
			$this->auth->restrict($this->permissionEdit);
	
			if ($this->save_nexmo('update', $id)) {				
				// Log the activity
				log_activity($this->auth->user_id(), lang('act_edit_record') .': '. $id .' : '. $this->input->ip_address(), 'sms_gateways');
	
				Template::set_message(sprintf(lang('msg_save_success'), lang('label_nexmo')), 'success');
				$this->set_default($id);
				redirect(SITE_AREA .'/settings/sms_gateways/');
			} else {
				Template::set_message(sprintf(lang('msg_edit_failure'), lang('label_nexmo')) . $this->sms_gateways_model->error, 'danger');
			}
		}
		//Find record
		$nexmo = $this->sms_gateways_model->find($id);
		//Retrieve settings from database
		$settings = unserialize($nexmo->settings);
	
		Template::set('record', $nexmo);
		Template::set('settings', $settings);
		Template::set('toolbar_title', lang('edit') . ' ' . lang('label_nexmo'));
		Template::render();
	}
	
	/**
	 * Save SMSGlobal Information
	 * @param Int	 $id	The ID of the record to update
	 */
	private function save_nexmo($type='update', $id=0)
	{
		$this->load->model('sms_gateways_model');
		$_POST['id'] = $id;
		$data = array();
		$data['display_name']        = $this->input->post('display_name');
		$settings = array('from'=>$this->input->post('from'),'api_key'=>$this->input->post('api_key'),'api_secret'=>$this->input->post('api_secret'));
		$data['settings'] = serialize($settings);
		$data['active']        = $this->input->post('active') ? 1 : 0;
	
		//Validation Rules
		$rules = $this->sms_gateways_model->validation_rules_nexmo;
		$this->form_validation->set_rules($rules);
		if($this->form_validation->run() == TRUE) {
			return $this->sms_gateways_model->update($id, $data);
		}
		return FALSE;
	}
	/*----------------------------------------------------*/
	/*	END OF NEXMO GATEWAY
	/*----------------------------------------------------*/
	
	/*----------------------------------------------------*/
	/*	SMS GLOBAL GATEWAY
	/*----------------------------------------------------*/
	/**
	 * Allows editing of SMSGlobal Payment Gateway data.
	 *
	 * @return void
	 */
	public function smsglobal()
	{
		$this->load->model('sms_gateways_model');
		$id = $this->uri->segment(5);
	
		if (empty($id))
		{
			Template::set_message(lang('invalid_id'), 'error');
			redirect(SITE_AREA .'/settings/sms_gateways');
		}
	
		if (isset($_POST['save']))
		{
		    $this->gcrud->stats(array('module' => 'sms_gateways'));
			$this->auth->restrict($this->permissionEdit);
	
			if ($this->save_smsglobal('update', $id)) {				
				// Log the activity
				log_activity($this->auth->user_id(), lang('act_edit_record') .': '. $id .' : '. $this->input->ip_address(), 'sms_gateways');
				$this->set_default($id);
				Template::set_message(sprintf(lang('msg_save_success'), lang('label_sms_global')), 'success');
				redirect(SITE_AREA .'/settings/sms_gateways/');
			} else {
				Template::set_message(sprintf(lang('msg_edit_failure'), lang('label_sms_global')) . $this->sms_gateways_model->error, 'danger');
			}
		}
		//Find record
		$smsglobal = $this->sms_gateways_model->find($id);
		//Retrieve settings from database
		$settings = unserialize($smsglobal->settings);

		Template::set('record', $smsglobal);
		Template::set('settings', $settings);
		Template::set('toolbar_title', lang('edit') . ' ' . lang('label_sms_global'));
		Template::render();
	}
	
	/**
	 * Save SMSGlobal Information
	 * @param Int	 $id	The ID of the record to update
	 */
	private function save_smsglobal($type='update', $id=0)
	{
		$this->load->model('sms_gateways_model');
		$_POST['id'] = $id;
		$data = array();
		$data['display_name']        = $this->input->post('display_name');
		$settings = array('api_username'=>$this->input->post('api_username'),'api_password'=>$this->input->post('api_password'),'api_mobile'=>$this->input->post('api_mobile'));
		$data['settings'] = serialize($settings);
		$data['active']        = $this->input->post('active') ? 1 : 0;
		
		//Validation Rules
		$rules = $this->sms_gateways_model->validation_rules_smsglobal;
		$this->form_validation->set_rules($rules);
		if($this->form_validation->run() == TRUE) {
			return $this->sms_gateways_model->update($id, $data);
		}
		return FALSE;
	}
	
	/*----------------------------------------------------*/
	/*	END OF SMS GLOBAL GATEWAY
	 /*----------------------------------------------------*/
	
	/*----------------------------------------------------*/
	/*	LK INFOTECH
	 /*----------------------------------------------------*/
	/**
	 * Allows editing of LK INFOTECH SMS Gateway data.
	 *
	 * @return void
	 */
	public function lkinfotech()
	{
		$this->load->model('sms_gateways_model');
		$id = $this->uri->segment(5);
	
		if (empty ( $id )) {
			Template::set_message ( lang ( 'invalid_id' ), 'error' );
			redirect ( SITE_AREA . '/settings/sms_gateways' );
		}
		
		if (isset ( $_POST ['save'] )) {
		    $this->gcrud->stats(array('module' => 'sms_gateways'));
			$this->auth->restrict ( $this->permissionEdit );
			
			if ($this->save_lkinfotech ( 'update', $id )) {				
				// Log the activity
				log_activity ( $this->auth->user_id(), lang ( 'act_edit_record' ) . ': ' . $id . ' : ' . $this->input->ip_address (), 'sms_gateways' );
				$this->set_default ( $id );
				Template::set_message ( sprintf(lang('msg_save_success'), lang('label_lkinfotech')), 'success' );
				redirect ( SITE_AREA . '/settings/sms_gateways/' );
			} else {
				Template::set_message ( sprintf(lang('msg_edit_failure'), lang('label_lkinfotech')) . $this->sms_gateways_model->error, 'danger' );
			}
		}
		//Find record
		$lkinfotech = $this->sms_gateways_model->find($id);
		//Retrieve settings from database
		$settings = unserialize($lkinfotech->settings);
		Template::set('record', $lkinfotech);
		Template::set('settings', $settings);
		Template::set('toolbar_title', lang('edit') . ' ' .lang('label_lkinfotech'));
		Template::render();
	}
	
	/**
	 * Save LKInfotech Information
	 * @param Int	 $id	The ID of the record to update
	 */
	private function save_lkinfotech($type='update', $id=0)
	{
		$this->load->model('sms_gateways_model');
		$_POST['id'] = $id;
		$data = array();
		$data['display_name']        = $this->input->post('display_name');
		$settings = array('api_username'=>$this->input->post('api_username'),'api_password'=>$this->input->post('api_password'), 'sender'=>$this->input->post('sender'), 'priority'=>$this->input->post('priority'), 'smstype'=>$this->input->post('smstype'));
		$data['settings'] = serialize($settings);
		$data['active']        = $this->input->post('active') ? 1 : 0;
	
		//Validation Rules
		$rules = $this->sms_gateways_model->validation_rules_lkinfotech;
		$this->form_validation->set_rules($rules);
		if($this->form_validation->run() == TRUE) {
			return $this->sms_gateways_model->update($id, $data);
		}
		return FALSE;
	}
	
	/*----------------------------------------------------*/
	/*	END OF LKINFOTECH GATEWAY
	/*----------------------------------------------------*/
	
	/*----------------------------------------------------*/
	/*	GLOBAL SMS GATEWAY
	/*----------------------------------------------------*/
	/**
	 * Allows editing of GLOBAL SMS Gateway data.
	 *
	 * @return void
	 */
	public function global_gateway() {
		$this->load->model('sms_gateways_model');
		$id = $this->uri->segment(5);
	
		if (empty ( $id )) {
			Template::set_message ( lang ( 'invalid_id' ), 'error' );
			redirect ( SITE_AREA . '/settings/sms_gateways' );
		}
		
		//Find record
		$global_gateway = $this->sms_gateways_model->find($id);
		
		// POST DATA HANDLING
		/**
		 * Validation rules are must for field_options
		 * else you will get array to string conversion errors
		 */
		if(isset($_POST['field_options'])) {
		    $this->gcrud->stats(array('module' => 'sms_gateways'));
			$option_name = array();
			$option_value = array();
				
			foreach($_POST['field_options']['name'] as $name) {
				$option_name[] = $name;
			}
			foreach($_POST['field_options']['value'] as $value) {
				$option_value[] = $value;
			}
		
			$options = array_combine($option_value, $option_name);
			Template::set('options', $options);
		} else {
			$settings = unserialize($global_gateway->settings);
			$options = unserialize($settings['options']);
			Template::set('options', $options);
		}
	
		if (isset ( $_POST ['save'] )) {
		    $this->gcrud->stats(array('module' => 'sms_gateways'));
			$this->auth->restrict ( $this->permissionEdit );
				
			if ($this->save_global ( 'update', $id )) {
				// Log the activity
				log_activity ( $this->auth->user_id(), lang ( 'act_edit_record' ) . ': ' . $id . ' : ' . $this->input->ip_address (), 'sms_gateways' );
				$this->set_default ( $id );
				Template::set_message ( sprintf(lang('msg_save_success'), lang('label_lkinfotech')), 'success' );
				redirect ( SITE_AREA . '/settings/sms_gateways/' );
			} else {
				Template::set_message ( sprintf(lang('msg_edit_failure'), lang('label_lkinfotech')) . $this->sms_gateways_model->error, 'danger' );
			}
		}		
		
		if(!empty($options)) {
			$gateway_url = $this->input->post('url') ? $this->input->post('url') : (!empty($settings) ? $settings['url'] : '') .'?';
			
			// Checking during array iteration, if the current element is the last element
			// Get array keys
			$arrayKeys = array_keys($options);
			// Fetch last array key
			$lastArrayKey = array_pop($arrayKeys);
			
			foreach($options as $name => $value) {
				$gateway_url .= $name . '=' . urlencode($value);
				if($name != $lastArrayKey) {
					//during array iteration this condition states the last element.
					$gateway_url .= '&';
				}
			}
			Template::set('gateway_url', $gateway_url);
		}
		Template::set('record', $global_gateway);
		Template::set('settings', !empty($settings) ? $settings : '');
		Template::set('toolbar_title', lang('edit') . ' ' .lang('label_global_gateway'));
		Template::render();
	}
	
	/**
	 * Save Global Gateway Information
	 * @param Int	 $id	The ID of the record to update
	 */
	private function save_global($type='update', $id=0)
	{
		$this->load->model('sms_gateways_model');
		$_POST['id'] = $id;
		$data = array();
		$data['display_name']       = $this->input->post('display_name');
		if(isset($_POST['field_options'])) {
			$option_name = array();
			$option_value = array();			
			foreach($_POST['field_options']['name'] as $name) {
				$option_name[] = $name;
			}
			foreach($_POST['field_options']['value'] as $value) {
				$option_value[] = $value;
			}				
			$field_options = array_combine($option_name, $option_value);
			//$data['options'] = serialize($field_options);
		}
		
		$settings = array('url'=>$this->input->post('url'),'options'=> serialize($field_options));
		$data['settings'] = serialize($settings);
		
		$data['active']        		= $this->input->post('active') ? 1 : 0;
		
		//Validation Rules
		$rules = $this->sms_gateways_model->validation_rules_global;
		$this->form_validation->set_rules($rules);
		if($this->form_validation->run() == TRUE) {
			return $this->sms_gateways_model->update($id, $data);
		}
		return FALSE;
	}
	
	/*----------------------------------------------------*/
	/*	END OF GLOBAL GATEWAY
	 /*----------------------------------------------------*/
	
	/**
	 * Common function for future gateways
	 */
	public function edit() {
	    $id = $this->uri->segment(5);
	    if (empty($id)) {
	        Template::set_message(lang('invalid_id'), 'error');
	        redirect(SITE_AREA .'/settings/sms_gateways');
	    }
	    if (isset($_POST['save'])) {
	        $this->gcrud->stats(array('module' => 'sms_gateways'));
	        $this->auth->restrict($this->permissionEdit);
	        if ($this->save_gateway($id)) {
	            $this->set_default ( $id );
	            // Log the activity
	            log_activity($this->auth->user_id(), lang('act_edit_record') .': '. $id .' : '. $this->input->ip_address(), 'sms_gateways');
	            Template::set_message(sprintf(lang('msg_save_success'), lang('sms_gateway')), 'success');
	        } else {
	            Template::set_message(sprintf(lang('msg_edit_failure'), lang('sms_gateway')) . $this->sms_gateways_model->error, 'danger');
	        }
	    }
	    $record = $this->sms_gateways_model->find($id);
	    if(!$record) {
	        Template::set_message(lang('invalid_id'), 'error');
	        redirect(SITE_AREA .'/settings/sms_gateways');
	    }
	    
	    // Retrieve settings from database
	    $settings = unserialize($record->settings);
	    
	    Template::set('record', $record);
	    Template::set('settings', $settings);
	    Template::set('toolbar_title', lang('edit') . ' ' . $record->display_name);
	    Template::render();
	}
	
	/**
	 * Save Gateway Information
	 * @param int $id	The ID of the record to update
	 */
	private function save_gateway($id) {
	    $_POST['id'] = $id;
	    $this->form_validation->set_rules('display_name', 'lang:label_display_name', 'required|trim|max_length[100]|sanitize|alpha_title');
	    $this->form_validation->set_rules('active', 'lang:label_status', 'trim');
	    switch($id) {
	        case 5:
	            $this->form_validation->set_rules('api_sid', 'lang:api_sid', 'trim|required');
	            $this->form_validation->set_rules('api_token', 'lang:api_token', 'trim|required');
	            $this->form_validation->set_rules('sender', 'lang:label_sender', 'trim|required');
	            break;
	    }
	    if ($this->form_validation->run() === false) {
	        return false;
	    }
	    $data = array();
	    $data['display_name']	= $this->input->post('display_name');
	    $data['active']   		= $this->input->post('active') ? 1 : 0;
	    $settings = array();
	    switch($id) {
	        case 5:
	            $settings['api_sid'] = $this->input->post('api_sid');
	            $settings['api_token'] = $this->input->post('api_token');
	            $settings['sender'] = $this->input->post('sender');
	            break;
	    }
	    $data['settings'] 		= serialize($settings);
	    return $this->sms_gateways_model->update($id, $data);
	}
	
	/**
	 * Set default from EDIT PAGE
	 * @param int $id
	 */
	public function set_default($id) {
		$rows_to_update = array();
	
		if($this->input->post('active') == 1) {
			//SET DEFAULT 1 FOR SELECTED ROW
			//GET IDS OF ALL ROWS EXCEPT SPECIFIED ROW
			$result = $this->sms_gateways_model->select('id')->where('id NOT IN (' . $id .')')->find_all();
		
			//STORE VALUE IN AN ARRAY
			foreach ($result as $row) {
				$rows_to_update[] = $row->id;
			}
		
			$rows = implode(',',$rows_to_update);
			$this->db->where('id IN (' . $rows . ')');
			$this->db->update($this->db->dbprefix . 'sms_gateways', array('active' => 0));
			Template::set_message(sprintf(lang('msg_save_success'), lang('sms_gateway')), 'success');
			redirect(SITE_AREA .'/settings/sms_gateways/');
		}
	}
	
	/**
	 * Set default from index page
	 * @param int $id
	 */
	public function set_default_home($id) {
		$rows_to_update = array();
		//CHECK WHETHER REQUEST IS FROM INDEX PAGE OR FROM EDIT
		if(!($this->input->post('active'))) {
			//SET DEFAULT 1 FOR SELECTED ROW
			$this->sms_gateways_model->update($id, array('active' => 1));
		}
		//GET IDS OF ALL ROWS EXCEPT SPECIFIED ROW
		$result = $this->sms_gateways_model->select('id')->where('id NOT IN (' . $id .')')->find_all();
	
		//STORE VALUE IN AN ARRAY
		foreach ($result as $row) {
			$rows_to_update[] = $row->id;
		}
	
		$rows = implode(',',$rows_to_update);
		$this->db->where('id IN (' . $rows . ')');
		$this->db->update($this->db->dbprefix . 'sms_gateways', array('active' => 0));
		Template::set_message(sprintf(lang('msg_action_success'), lang('sms_gateway'), lang('label_updated')), 'success');
		redirect(SITE_AREA .'/settings/sms_gateways/');
	}
	
	/**
	 * Test SMS Gateways
	 */
	public function test() {
		if (isset($_POST['test'])) {
		    $this->gcrud->stats(array('module' => 'sms_gateways'));
			//Validation Rules
			$rules = $this->sms_gateways_model->validation_rules_test;
			$this->form_validation->set_rules($rules);
			if($this->form_validation->run() == TRUE) {
				$mobile_number = $this->input->post('mobile_number');
				$mobile_number = str_replace(' ', '', $mobile_number);
				//$mobile_number = str_replace('+', '', $mobile_number);
				$message = lang('sms_test_message');
				$success = $this->gc->send_sms($message, $mobile_number);
				if($success) {
					Template::set_message(lang('sms_test_success'), 'success');
				} else {
				    $message = $this->session->flashdata('sms_error');
				    if(empty($message)) {
				        $message = lang('sms_test_error');
				    }
				    Template::set_message($message, 'error');
				}
			}
		}
		Template::set('toolbar_title', lang('sms_test_settings'));
		Template::render();
	}
	
	/**
	 * SMS Gateways Template
	 */
	public function template() {	
		$this->lang->load('emailer/emailer');	 
		if (isset($_POST['save'])) {
		    $this->gcrud->stats(array('module' => 'sms_gateways'));
			$header = $_POST['query'];
		
			$this->load->helper('file');		
			write_file(APPPATH . 'modules/sms_gateways/views/settings/_query_tpl.php', $header, 'w+');
			Template::set_message(sprintf(lang('msg_save_success'), lang('em_template')), 'success');
			redirect(SITE_AREA . '/settings/sms_gateways/template');
		}
		$help_query = array(
			'{{module}}	= ' . lang('label_module'),
			'{{title}} = ' . lang('label_title'),
			'{{site_title}} = ' . lang('label_site_title'),
			'{{site_url}} = ' . lang('label_site_url'),
			'{{name}} = ' . lang('label_name'),	
			'{{phone}} = ' . lang('label_phone_number'),	
			'{{email}} = ' . lang('label_email'),	
			'{{message}} = ' . lang('label_message'),		
							 
		);
		Template::set(array(
			'toolbar_title' => lang('em_template'),
			'help_query' => lang('help_template') . '<br />' . implode('<br />', $help_query)
		));
		Template::render();
	}
}