<?php defined('BASEPATH') || exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Online Software Solutions							   # ||
|| # Copyright (C) 2021 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Components controller
 */
class Components extends Admin_Controller {
    protected $permissionCreate = 'Subscribers.Components.Create';
    protected $permissionDelete = 'Subscribers.Components.Delete';
    protected $permissionEdit   = 'Subscribers.Components.Edit';
    protected $permissionView   = 'Subscribers.Components.View';
    protected $context			= 'components';
    protected $moduleName       = 'subscribers';
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
        
        $this->auth->restrict($this->permissionView);
        $this->load->model('subscribers/subscribers_model');
        $this->lang->load('subscribers');        
      	$this->form_validation->set_error_delimiters("<span class='error'>", "</span>");        
        Template::set_block('sub_nav', 'components/_sub_nav');        
        Assets::add_module_js('subscribers', 'subscribers.js');
    }
    
    /**
     * Display a list of Subscriber Users.
     *
     * @return void
     */
    public function index($filter = 'all', $offset = 0) {
        $this->auth->restrict('Subscribers.Components.Summary');
        $record = $this->subscribers_model->summary();
        $tabular_blocks = array();
        $tabular_blocks['last_messages'] = $this->subscribers_model->manage(0, null, 'M.id DESC', 20, 0, array('module' => 'messages', 'prefix' => 'subscriber'));
        $tabular_blocks['recently_added'] = $this->subscribers_model->manage(0, null, 'M.id DESC', 20, 0, array('module' => 'users', 'prefix' => 'subscriber'));
        $tabular_blocks['coming_followups'] = $this->subscribers_model->manage(0, null, 'M.next_follow_up DESC', 20, 0, array('module' => 'followups', 'prefix' => 'subscriber'));
        $record->tabular_blocks = $tabular_blocks;
        Template::set(array(
            'toolbar_title' => lang('gc_menu_subscribers') . ' - ' . lang('gc_menu_summary'),
            'record' => $record
        ));
        Template::render();
    }
    
    /**
     * Display a list of subscribers data
     * @param string $module
     * @param string $filter
     * @param number $offset
     */
    public function manage($module = 'users', $filter = 'all', $offset = 0) {
        if($module == 'users' || $module == 'followups' || $module == 'templates' || $module == 'queries') {
            $extra = array(
                'model' => 'subscribers',
                'context' => $this->context,
                'fields' => $module,
                'prefix' => 'subscriber',
                'table_prefix' => 'subscriber'
            );            
            if($module == 'users') {           
                // subscriber specific followups
                if(isset($_POST['followups'])) {
                    $this->session->set_userdata('subscriber_followups_rows_filter', array('subscriber_id' => $_POST['followups']));
                    Template::redirect(SITE_AREA . '/components/subscribers/manage/followups');
                }
                $extra['advanced_filter'] = array('group_ids', 'date_from', 'date_to', 'city_id', 'created_by');
                $extra['advanced_filter_date'] = true;          
            } else if($module != 'templates' && $module != 'queries') {
                $extra['advanced_filter'] = array('subscriber_id', 'date_from', 'date_to');
                $extra['advanced_filter_date'] = true;
                // subscriber specific followups
                if($this->session->userdata('subscriber_followups_rows_filter')) {
                    $subscriber_followups_rows_filter = (array) $this->session->userdata('subscriber_followups_rows_filter');
                    if(!empty($subscriber_followups_rows_filter['subscriber_id'])) {
                        $extra['query_where'] = "M.subscriber_id = " . $subscriber_followups_rows_filter['subscriber_id'];
                    }
                }
            }
            if($module == 'queries') { // followups                
                if(isset($_POST['followups'])) {
                    $query = $this->subscribers_model->get_query($_POST['followups']);
                    if($query) {
                        $this->session->set_userdata('subscriber_followups_rows_filter', array('subscriber_id' => $query->subscriber_id));
                        Template::redirect(SITE_AREA . '/components/subscribers/manage/followups');
                    }
                }
                // view group specific queries
                if($this->session->userdata('subscriber_query_group_id')) {
                    $extra['query_where'] = (!empty($extra['query_where']) ? $extra['query_where'] . " AND " : '') . "M.group_id = " . $this->session->userdata('subscriber_query_group_id');
                }
            }
            if($module == 'users' || $module == 'queries') {                
                // email and send sms variables
                $this->load->model('emailer/emailer_model');
                $available_variables =  $this->emailer_model->share_variables('subscriber_users', false, ', ');
                Template::set(array(
                    'subject_variables' => $available_variables['subject'],
                    'message_variables' => $available_variables['message']
                ));
            }
            if($this->auth->role_id() != 1 && $module != 'templates') { // staff and other users can view only their own submit links
                $extra['query_where'] = (!empty($extra['query_where']) ? $extra['query_where'] . " AND " : '') . "M.created_by = " . $this->auth->user_id();
            }
            $this->gc->module_view($module, $filter, $offset, 'admin', $extra);
        } else {
            $this->subscribers_model->view($module, $filter, $offset);
        }
    }
    
    /**
     * Add/Edit Subscribers Data
     *
     * @return void
     */
    public function form($module = 'users', $action = 'create') {
        $params = array(
            'model' => 'subscribers',
            'context' => $this->context,
            'fields_method' => $module,
            'set_table' => true,
            'uri_segment' => 7,
            'redirect_url' => site_url(SITE_AREA . '/' . $this->context . '/' . $this->moduleName . '/manage/' . $module)
        );
        
        // set the subscribers id automatically
        $subscriber_followups_rows_filter = (array) $this->session->userdata('subscriber_followups_rows_filter');
        if($module == 'followups' && $action == 'create' && !empty($subscriber_followups_rows_filter['subscriber_id'])) {
            $record = array(
                'subscriber_id' => $subscriber_followups_rows_filter['subscriber_id']
            );
            $params['record'] = (object) $record;
        }
        
        if($module == 'users') { // && $this->session->userdata('filter_group_id')
            $params['dynamic_fields'] = false;
            $params['fields_method'] = 'form_users';
            $params['query_method'] = 'get_subscriber';
            $params['method'] = 'after_subscriber_save';
            // save link relations
            // $params['fields'] = $this->subscribers_model->group_fields($this->session->userdata('filter_group_id'));
            if($this->session->userdata('filter_group_id') && $action == 'create') {
                $record = array(
                    'group_ids' => $this->session->userdata('filter_group_id')
                );
                $params['record'] = (object) $record;
            }
        }
        if($module == 'queries') {
            $params['method'] = 'after_query_save';            
        }
        if($module == 'templates') {
            // Template::set('extra_action', $this->load->view('common/_template_fields', array('template_module' => 'subscriber_users'), true));
            if($action == 'edit') {
                Template::set('extra_actions', array(
                   '<a class="template-preview btn btn-info" href="javascript:void(0)" data-href="' . site_url(SITE_AREA . '/components/subscribers/preview/' . $this->uri->segment(7)) . '"><i class="fas fa-eye fa-fw mr-1"></i>' . lang('bf_action_preview') . '</a>'
                ));
            }
        }
        $this->gcrud->form('subscriber_' . $module, $action, 'admin', $params);
    }
    
    /**
     * Assign fields to a Group
     * @param number $id
     * @param string $module
     */
    public function assign($id = 0, $module = 'categories') {
        $cancel_url = site_url(SITE_AREA . '/components/subscribers/manage/groups');
        // Allow admin to change row
        if (isset($_POST['change']) && !empty($_POST['group_id'])) {
            Template::redirect(site_url(SITE_AREA . '/components/subscribers/assign/' . $_POST['group_id']));
        }
        // module search js and css dependencies
        Assets::add_css('selectize.bootstrap3.css');
        Assets::add_js('selectize.min.js');
        Assets::add_js('selectize-search.js');
        Assets::add_module_css('subscribers', 'subscribers.css');
        // Display fields using datatables
        Assets::add_js( array(
            'jquery.dataTables.min',
            'dataTables.bootstrap.min',
        ));
        Assets::add_css('dataTables.bootstrap.min');
        Assets::add_js( "$('#fields_table').DataTable({
		   'aoColumnDefs': [{
		        'bSortable': false,
		        'aTargets': [0] /* Disable sorting on the first column, 1st one, start by the left */
		    }],
		    order: [[5, 'asc']], /* Set default sort */
		    pageLength: 50 /* Limit number of rows */
		});", 'inline' );
        $this->lang->load('fields/fields');
        $module = 'subscriber_users';
        $module_id = gc_module_id($module);
        $fields = (array) $this->fields_model->get_fields($module, 'frontend');
        $group = $this->db->where('id', $id)->get('subscriber_groups')->row();
        if(!$group) {
            Template::redirect($cancel_url);
        }
        
        // Saved Fields
        $records = array();
        $existing_fields = array(); // fields to be deleted in case of unchecked
        $saved_result = $this->db->get_where('subscriber_group_fields', array('group_id' => $id))->result();
        if($saved_result) {
            foreach($saved_result as $row) {
                $records[$row->field_id] = $row;
                $existing_fields[$row->field_id] = $row->id;
            }
        }
        
        // Save Fields
        if (isset($_POST['save'])) {
            gc_demo_mode();
            $this->gcrud->stats(array('module' => 'subscribers'));
            clear_cache('subscriber_group_fields'); // clear cache on save
            $checked = $this->input->post('checked');
            if(is_array($checked) && count($checked)) { // Allow user to unassign all fields
                $result = FALSE;
                $data = array();
                foreach ($checked as $pid) {
                    $data[] = array(
                        'group_id' => $id,
                        'field_id' => $pid,
                        'priority' => (int) $this->input->post('priority_' . $pid),
                        'is_required' => (int) $this->input->post('required_' . $pid),
                        'is_unique' => (int) $this->input->post('unique_' . $pid),
                        // 'status' => (int) $this->input->post('status_' . $pid)
                    );
                    // fields to be deleted
                    if(array_key_exists($pid, $existing_fields)) {
                        unset($existing_fields[$pid]);
                    }
                }
                $this->db->on_duplicate('subscriber_group_fields', $data);
                if(!empty($existing_fields)) {// delete unchecked fields
                    $this->db->where('id IN (' . implode(',', $existing_fields) . ')')->delete('subscriber_group_fields');
                }
                $message_label = (count($checked) == 1 ? lang('label_field') : lang('label_fields'));
                Template::set_message(count($checked) .' '. sprintf(lang('msg_save_success'), $message_label), 'success');
                Template::redirect(current_url());
            } else {                
                if(!empty($existing_fields)) {// delete unchecked fields
                    $this->db->where('id IN (' . implode(',', $existing_fields) . ')')->delete('subscriber_group_fields');
                }
                Template::set_message(sprintf(lang('msg_action_success'), lang('label_fields'), lang('action_unassigned')), 'success');
                Template::redirect(current_url());
            }
        }
        Template::set(array(
            'module' => $module,
            'group' => $group,
            'fields' => $fields,
            'records' => $records,
            'cancel_url' => $cancel_url,
            'toolbar_title' => lang('subscriber_field_assign') . ' - ' . (!empty($group->name_lang) ? lang($group->name) : $group->name)
        ));
        Template::render();
    }
    
    /**
     * Export Subscribers
     */
    public function export() {
        $this->auth->restrict('Subscribers.Components.Export');        
        Assets::add_js('moment.js');
        Assets::add_css('bootstrap-datetimepicker.min.css' );
        Assets::add_js('bootstrap-datetimepicker.min.js' );
        Assets::add_js("
    $('#date_from').datetimepicker({
        format: 'YYYY-MM-DD',
        widgetPositioning: {
            horizontal: 'right',
            vertical: 'bottom'
        }
    });
    $('#date_to').datetimepicker({
        format: 'YYYY-MM-DD',
        widgetPositioning: {
            horizontal: 'right',
            vertical: 'bottom'
        }
    });", 'inline');
        // Set Default
        if (isset($_POST['save'])) {
            gc_demo_mode();
            $this->gcrud->stats(array('module' => 'subscribers'));
            $this->form_validation->set_rules('group_id', 'lang:singular_subscriber_groups', 'trim');
            $this->form_validation->set_rules('limit', 'lang:label_limit', 'trim|numeric|required');
            $this->form_validation->set_rules('offset', 'lang:label_offset', 'trim|numeric|required');
            $this->form_validation->set_rules('format', 'lang:label_format', 'trim'); 
            if ($this->form_validation->run() === true) {
                $group_id = (int) $this->input->post('group_id');
                $limit = (int) $this->input->post('limit');
                $offset = (int) $this->input->post('offset');
                $format = $this->input->post('format');
                $where = null;
                if($this->input->post('date_from') && $this->input->post('date_to')) {
                    $where = "(DATE(M.created_on) BETWEEN '" . $this->input->post('date_from') . "' AND '" . $this->input->post('date_from') . "')";
                } else if($this->input->post('date_from')) {
                    $where = "DATE(M.created_on) >= '" . $this->input->post('date_from') . "'";
                } else if($this->input->post('date_to')) {
                    $where = "DATE(M.created_on) <= '" . $this->input->post('date_to') . "'";
                }
                $this->subscribers_model->export($group_id, $limit, $offset, $format, 'M.id DESC', $where);
            }            
        }
        Template::set(array(
            'toolbar_title' => lang('label_export')
        ));
        Template::render();
    }
    
    /**
     * Unique Subscriber Template
     * @return boolean
     */
    public function unique_template() {
        $id = (int) $this->input->post('id');
        $group_id = (int) $this->input->post('group_id');
        $action_type = (int) $this->input->post('action_type');
        $where = array('group_id' => $group_id, 'action_type' => $action_type);
        if($id != 0) {
            $where['id != '] = $id;
        }
        $count = $this->db->where($where)->count_all_results('subscriber_templates');
        if($count > 0) {
            $this->form_validation->set_message('unique_template', lang('form_validation_unique_template'));
            return false;
        } else {
            return true;
        }
    }
    
    /**
     * Display Votes in Threads and Posts
     */
    public function get_users() {
        $module = isset($_GET['module']) ? $_GET['module'] : '';
        $prefix = $module == 'subscriber_users' ? 'subscriber' : '';
        if($module == 'users') {
            $columns = array('check', 'id', 'display_name', 'email', 'phone_number', 'active');
        } else if($module == 'listings') {
            $columns = array('check', 'id', 'title', 'email', 'phone_number', 'status');
        } else {
            $columns = array('check', 'id', 'name', 'email', 'phone_number', 'status_id');
        }
        $where = null;
        if($module == 'subscriber_users') {
            $module = 'users';
        }
        $this->gcrud->get_data($columns, $where, $module, $prefix);
    }
    
    /**
     * Template Preview
     * @param number $id
     * @return boolean
     */
    public function preview($id = 0) {
        $record = $this->db->where('id', $id)->get('subscriber_templates')->row();
        $sms_text = !empty($record->sms_text) ? '<div class="bg-secondary text-white p-2"><h3 class="h2 font-weight-bold">SMS Text</h3>' . $record->sms_text . '</div>' : '';
        if(empty($record)) {
            echo '';
            return false;
        }
        $close = lang('form_close');
        $output = <<<EOL
<div class="modal" id="previewModal">
  <div class="modal-dialog mw-100 w-75">
    <div class="modal-content">
      <div class="modal-header bg-primary text-white">
        <h4 class="modal-title">{$record->subject}</h4>
        <button type="button" class="close" data-dismiss="modal">&times;</button>
      </div>
      <div class="modal-body">        
        {$record->message}
        {$sms_text}
      </div>
      <div class="modal-footer">
        <button type="button" class="btn btn-danger" data-dismiss="modal">{$close}</button>
      </div>
    </div>
  </div>
</div>
EOL;
        echo $output;
        return false;
    }
}