<?php defined('BASEPATH') || exit('No direct script access allowed');
/*=======================================================================*\
 || ##################################################################### ||
 || # GoClixy - Online Software Solutions							   # ||
 || # Copyright (C) 2021 GoClixy. All Rights Reserved.				   # ||
 || # ----------------------------------------------------------------  # ||
 || # This file is owned and copyrighted by GoClixy.com. Your license   # ||
 || # confers  no  title  or ownership in the Script and is not a sale  # ||
 || # of any rights to the Software. This file may not be redistributed # ||
 || # in whole or significant part.									   # ||
 || # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
 || # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
 || ##################################################################### ||
 \*=======================================================================*/
class Subscribers_model extends BF_Model
{
    public $table_name	             = 'subscriber_users';
    protected $table_groups	         = 'subscriber_groups';
    protected $table_group_fields    = 'subscriber_group_fields';
    protected $table_group_relations = 'subscriber_group_relations';
    protected $table_metas           = 'subscriber_metas';
    protected $table_statuses	     = 'subscriber_statuses';
    protected $table_followups	     = 'subscriber_followups';
    protected $table_templates       = 'subscriber_templates';
    protected $table_messages        = 'subscriber_messages';
    protected $table_queries         = 'subscriber_queries';
    protected $table_queries_metas   = 'subscriber_queries_metas';
    
    protected $key			= 'id';
    protected $date_format	= 'datetime';
    
    protected $log_user 	= true;
    protected $set_created	= true;
    protected $set_modified = true;
    public $soft_deletes	= false;
    
    protected $created_field     = 'created_on';
    protected $created_by_field  = 'created_by';
    protected $modified_field    = 'modified_on';
    protected $modified_by_field = 'modified_by';
    
    // Customize the operations of the model without recreating the insert,
    // update, etc. methods by adding the method names to act as callbacks here.
    protected $before_insert 	= array();
    protected $after_insert 	= array();
    protected $before_update 	= array();
    protected $after_update 	= array();
    protected $before_find 	    = array();
    protected $after_find 		= array();
    protected $before_delete 	= array();
    protected $after_delete 	= array();
    
    // For performance reasons, you may require your model to NOT return the id
    // of the last inserted row as it is a bit of a slow method. This is
    // primarily helpful when running big loops over data.
    protected $return_insert_id = true;
    
    // The default type for returned row data.
    protected $return_type = 'object';
    
    // Items that are always removed from data prior to inserts or updates.
    protected $protected_attributes = array();
    
    // You may need to move certain rules (like required) into the
    // $insert_validation_rules array and out of the standard validation array.
    // That way it is only required during inserts, not updates which may only
    // be updating a portion of the data.
    protected $validation_rules 		= array();
    protected $insert_validation_rules  = array();
    protected $skip_validation 			= false;
    protected $context			        = 'components';
    protected $moduleName               = 'subscribers';
    protected $gc_subscribers_cache;
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
        // Load Zend Caching
        $this->load->library('slug');
        $this->load->library('gzf_cache', array('lifetime' => 900));
        if(method_exists($this->gzf_cache, 'get_instance')) {
            $this->gc_subscribers_cache = $this->gzf_cache->get_instance();
        }
    }
    
    /**
     * Users Fields - Index Page
     *
     * @return array[]
     */
    public function users($role = 'admin') {
        /*$module = 'subscriber_users';
        $field_type = $role == 'admin' ? 'admin_only' : 'frontend';
        $extra = array();
        $this->load->model('fields/fields_model');
        $fields = $this->fields_model->get_fields($module, $field_type, array(), 0, $extra);
        dump($fields);*/
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => false,
                'admin_index' => true
            ),
            array(
                'type' => 'selectize',
                'name' => 'group_ids',
                'label' => 'label_group',
                'placeholder' => 'placeholder_selectize',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => true,
                'options' => 'load:subscriber_groups;module:subscriber_groups;max_items:10',
                'options_detailed' => 1,
                'rules' => 'trim|required',
            ),
            array(
                'type' => 'text',
                'name' => 'name',
                'label' => 'label_name',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'edit' => 1,
                'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
                'rules' => 'required|trim|max_length[255]',
            ),
            array(
                'type' => 'text',
                'name' => 'email',
                'label' => 'label_email',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'edit' => 0,
                'rules' => 'trim|valid_email',
            ),
            array(
                'type' => 'text',
                'name' => 'phone_number',
                'label' => 'label_phone_number',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'edit' => 0,
                'rules' => 'trim|required|phone_check|unique[subscriber_users.phone_number,subscriber_users.id]',
            ),
            array(
                'type' => 'textarea',
                'name' => 'address',
                'label' => 'label_address',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'edit' => 0,
                'rules' => 'trim',
            ),
            array(
                'type' => 'dropdown',
                'name' => 'status_id',
                'label' => 'singular_forum_statuses',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:subscriber_statuses;value:database',
                'options_detailed' => 1,
                'class' => 3,
                'rules' => 'trim|is_numeric',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim'
            ),
            array(
                'type' => 'none',
                'name' => 'created_by',
                'label' => 'created_by',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => true,
                'rules' => 'trim'
            ),
            array(
                'type' => 'none',
                'name' => 'created_ip',
                'label' => 'created_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim'
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim'
            ),
            array(
                'type' => 'none',
                'name' => 'modified_by',
                'label' => 'modified_by',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim'
            ),
            array(
                'type' => 'none',
                'name' => 'modified_ip',
                'label' => 'modified_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim'
            )
        );
        return $fields;
    }
    
    /**
     * Users Fields - Index Page
     *
     * @return array[]
     */
    public function form_users($role = 'admin', $form_type = 'index') {
        $field_type = $role == 'admin' ? 'admin_only' : 'frontend';
        switch($form_type) {
            case 'insert':
            case 'update':
                $field_type = $role == 'admin' ? 'admin_index' : 'frontend_index';
                break;
        }
        $group_id = 0;
        if($this->session->userdata('filter_group_id')) {
            $group_id = $this->session->userdata('filter_group_id');
        }
        $cacheID = 'subscriber_group_fields_' . $role . '_' . $form_type . '_' . $group_id;
        if(!$fields = $this->gc_subscribers_cache->load($cacheID)) {
            $fields = array();
            $display_group_fields_only = false;
            
            // required/unique field validation rules
            $group_fields = array();
            if(!empty($group_id)) {
                // Display Group Fields Only
                $group_info = $this->db->where('id', $group_id)->get($this->table_groups)->row();
                if($group_info && !empty($group_info->group_fields_only)) {
                    $display_group_fields_only = true;
                }
                $records = $this->db->where('group_id', $group_id)->order_by('priority', 'ASC')->get($this->table_group_fields)->result_array();
                if($records) {
                    foreach($records as $record) {
                        $group_fields[$record['field_id']] = $record;
                    }
                }
            }
            
            $module = 'subscriber_users';
            $module_id = gc_module_id('subscriber_users');
            $extra = array();
            $this->load->model('fields/fields_model');
            if($display_group_fields_only) {
                $form_fields = $this->group_fields($group_id, 'admin_only');
            } else {
                $form_fields = $this->fields_model->get_fields($module, $field_type, array(), 0, $extra);
            }
            $field_ids = array();
            if(!empty($form_fields)) {
                foreach($form_fields as $form_field) {
                    if(empty($form_field['query'])) {
                        $field_ids[] = $form_field['id'];             
                    }
                }
                $where = 'field_id IN (' . implode(',', $field_ids) . ') AND module_id = ' . $module_id;
                $this->db->join('fields F', 'F.id = field_relations.field_id', 'LEFT');
                $field_relations = $this->db->select('field_relations.*, F.name, F.type_id')->where($where)->order_by('priority')->get('field_relations')->result_array();
                $relation_fields = array();
                if($field_relations) {
                    foreach($field_relations as $field_relation) {
                        $relation_fields[$field_relation['field_id']] = $field_relation;
                    }
                }
                foreach($form_fields as $form_field) {
                    if(empty($form_field['query'])) {
                        $field_info = $relation_fields[$form_field['id']];
                        $rules = $form_field['rules'];
                        // required/is_unique field validation rules
                        if(array_key_exists($form_field['id'], $group_fields)) {
                            if(strpos($rules, 'required') === false && !empty($group_fields[$form_field['id']]['is_required'])) {
                                $rules .= '|required';
                            }
                            if(strpos($rules, 'unique') === false && !empty($group_fields[$form_field['id']]['is_unique'])) {
                                $rules .= '|unique[subscriber_users.' . $form_field['name'] . ', subscriber_users.id]';
                            }
                        }
                        $field = array(
                            'type' => $field_info['type'],
                            'type_id' => $field_info['type_id'],
                            'name' => $field_info['name'],
                            'label' => $field_info['label'],
                            'placeholder' => $field_info['placeholder'],
                            'placeholder_lang' => $field_info['placeholder_lang'],
                            'frontend' => $field_info['frontend'],
                            'admin_only' => $field_info['admin_only'],
                            'frontend_index' => $field_info['frontend_index'],
                            'admin_index' => $field_info['admin_index'],
                            'rules' => $rules,
                            'options' => $field_info['options'],
                            'options_detailed' => $field_info['options_detailed'],
                            'options_lang' => $field_info['options_lang'],
                            'class' => $field_info['class'],
                            'field_extra' => !empty($field_info['field_extra']) ? $field_info['field_extra'] : null,
                        );
                        $fields[] = $field;
                    }
                }
            }
            $this->gc_subscribers_cache->save($fields, $cacheID, array('subscriber_group_fields'));
        }
        return $fields;
    }
    
    /**
     * Group Fields
     *
     * @return array
     */
    public function groups() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'text',
                'name' => 'name',
                'label' => 'label_name',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'edit' => 1,
                'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
                'rules' => 'trim|required|max_length[150]'
            ),
            array(
                'type' => 'checkbox',
                'name' => 'name_lang',
                'label' => 'name_lang',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'message',
                'label' => 'subscriber_success_message',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'tooltip' => lang('subscriber_help_success_message'),
                'edit' => 1,
                'rules' => 'trim|required|max_length[450]'
            ),
            array(
                'type' => 'checkbox',
                'name' => 'message_lang',
                'label' => 'message_lang',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'iconpicker',
                'name' => 'icon',
                'label' => 'label_icon',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'class' => 3,
                'rules' => 'trim|max_length[255]',
            ),
            array(
                'type' => 'text',
                'name' => 'priority',
                'label' => 'label_priority',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'value' => 0,
                'class' => 2,
                'rules' => 'trim|required|is_numeric',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'notify_duplicate',
                'label' => 'notify_duplicate',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'group_fields_only',
                'label' => 'group_fields_only',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'status',
                'label' => 'label_status',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            )
        );
        return $fields;
    }
    
    /**
     * Status Fields
     *
     * @return array
     */
    public function statuses() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'text',
                'name' => 'name',
                'label' => 'label_name',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'edit' => 1,
                'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
                'rules' => 'trim|required|max_length[150]'
            ),
            array(
                'type' => 'checkbox',
                'name' => 'name_lang',
                'label' => 'name_lang',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'colorpicker',
                'name' => 'color',
                'label' => 'label_color',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'class' => 2,
                'rules' => 'trim|max_length[255]',
            ),
            array(
                'type' => 'text',
                'name' => 'priority',
                'label' => 'label_priority',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'value' => 0,
                'class' => 2,
                'rules' => 'trim|required|is_numeric',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'status',
                'label' => 'label_status',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            )
        );
        return $fields;
    }
    
    /**
     * Followups Form Fields
     *
     * @return array[]
     */
    public function followups() {
        $subscriber_id = $this->uri->segment(5) ? $this->uri->segment(5) : 0;
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => false,
                'admin_index' => true
            ),
            array(
                'type' => 'selectize',
                'name' => 'subscriber_id',
                'label' => 'singular_subscribers',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'load:subscriber_users;module:subscriber_users;max_items:1',
                'options_detailed' => 1,
                'edit' => 0,
                'rules' => 'required|trim|is_numeric|max_length[255]',
            ),
            array(
                'type' => 'textarea',
                'name' => 'remarks',
                'label' => 'subscriber_remarks',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'custom_rules' => 'editor',
                'field_class' => 'editor',
                // 'edit' => 1,
                // 'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
                'rules' => 'required|trim|max_length[255]',
            ),
            array(
                'type' => 'text',
                'name' => 'next_follow_up',
                'label' => 'subscriber_next_follow_up',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'class' => 3,
                'rules' => 'trim|valid_datetime'
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim'
            ),
            array(
                'type' => 'none',
                'name' => 'created_ip',
                'label' => 'created_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim'
            ),
            array(
                'type' => 'none',
                'name' => 'created_by',
                'label' => 'created_by',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => $this->auth->role_id() == 1 ? false : false,
                'rules' => 'trim'
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => false,
                'admin_only' => false,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim'
            ),
            array(
                'type' => 'none',
                'name' => 'modified_by',
                'label' => 'modified_by',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim'
            ),
            array(
                'type' => 'none',
                'name' => 'modified_ip',
                'label' => 'modified_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim'
            )
        );
        return $fields;
    }
    
    /**
     * Email Template Fields
     *
     * @return array
     */
    public function templates($role = 'admin') {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'dropdown',
                'name' => 'module_table',
                'label' => 'label_module',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:gc_menu_subscribers;value:subscriber_users',
                'options_detailed' => 1,
                'options_lang' => 1,
                'class' => 3,
                'rules' => 'trim|unique_email_template[subscriber]',
            ),
            array(
                'type' => 'dropdown',
                'name' => 'group_id',
                'label' => 'singular_subscriber_groups',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:subscriber_groups;value:database',
                'options_detailed' => 1,
                'class' => 3,
                'rules' => 'trim|is_numeric|required',
            ),
            array(
                'type' => 'dropdown',
                'name' => 'user_type',
                'label' => 'bf_user',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:placeholder_select;value:|name:us_admin;value:admin|name:bf_user;value:user',
                'options_detailed' => 1,
                'options_lang' => 1,
                'class' => 3,
                'rules' => 'trim',
            ),
            array(
                'type' => 'dropdown',
                'name' => 'action_type',
                'label' => 'label_action',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:placeholder_select;value:|name:add;value:add|name:edit;value:edit|name:delete;value:delete',
                'options_detailed' => 1,
                'options_lang' => 1,
                'class' => 3,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'name',
                'label' => 'label_name',
                'placeholder' => 'placeholder_name',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
                'rules' => 'required|trim|max_length[50]'
            ),
            array(
                'type' => 'text',
                'name' => 'subject',
                'label' => 'label_subject',
                'placeholder' => 'placeholder_subject',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'required|trim|max_length[255]'
            ),
            array(
                'type' => 'textarea',
                'name' => 'message',
                'label' => 'label_message',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'custom_rules' => 'editor',
                'field_class' => 'editor editor-full-url',
                'class' => 9,
                'rules' => 'trim|required',
            ),
            array(
                'type' => 'textarea',
                'name' => 'sms_text',
                'label' => 'sms_text',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'class' => 9,
                'field_extra' => 'rows:3',
                'rules' => 'trim',
            ),
            array(
                'type' => 'textarea',
                'name' => 'params',
                'label' => 'label_params',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'class' => 9,
                'tooltip' => lang('emailer_params_help'),
                'field_extra' => 'rows:3',
                'rules' => 'trim',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'status',
                'label' => 'label_status',
                'frontend' => false,
                'admin_only' => false,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            )
        );
        return $fields;
    }
    
    /**
     * Subscribers Query Fields
     *
     * @return array[]
     */
    public function queries($role = 'admin') {
        $group_id = $this->session->userdata('subscriber_query_group_id'); // handle this using the session id
        $form_fields = $this->group_fields($group_id, $role == 'admin' ? 'admin_index' : 'frontend_index');
        $query_fields = array();
        foreach($form_fields as $fkey => $fvalue) {
            if(!empty($fvalue['query'])) {
                $query_fields[] = $fvalue;
            }
        }
        if(empty($query_fields)) {
            return false;
        }
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => false,
                'admin_index' => true
            ),
            array(
                'type' => 'selectize',
                'name' => 'subscriber_id',
                'label' => 'singular_subscribers',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'load:subscriber_users;module:subscriber_users;max_items:1',
                'options_detailed' => 1,
                'edit' => 0,
                'rules' => 'required|trim|is_numeric|max_length[255]',
            ),/*
            array(
                'type' => 'dropdown',
                'name' => 'query_module_id',
                'label' => 'label_module',
                'frontend' => false,
                'admin_only' => false,
                'frontend_index' => false,
                'admin_index' => false,
                'options' => 'name:products;value:1',
                'options_detailed' => 1,
                'rules' => 'required|trim|is_numeric',
            ),
            array(
                'type' => 'selectize',
                'name' => 'query_row_id',
                'label' => 'singular_products',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'load:products;module:products;max_items:1',
                'options_detailed' => 1,
                'rules' => 'required|trim|is_numeric',
            ),*/
      );
      $common_fields = array(
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim'
            ),
            array(
                'type' => 'none',
                'name' => 'created_ip',
                'label' => 'created_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim'
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim'
            ),
            array(
                'type' => 'none',
                'name' => 'modified_ip',
                'label' => 'modified_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim'
            )
        );
        $fields = array_merge($fields, $query_fields, $common_fields);
        return $fields;
    }
    
    /**
     * Get Product Topics/Products/Groups etc.
     * @param number $count
     * @param string $where
     * @param string $sort_by
     * @param number $limit
     * @param number $offset
     * @return array|number
     */
    public function manage($count = 0, $where = null, $sort_by = 'M.id DESC', $limit = 15, $offset = 0, $extra = array()) {
        if(!isset($extra['module'])) {
            return false;
        }
        $module = $extra['module'];
        if($count == 1) {
            $sql = "SELECT COUNT(*) AS total_count";
        } else {
            switch($module) {
                case 'users':
                    $sql = "SELECT M.*, S.color AS status_color, S.name AS status, S.name_lang AS status_lang";
                    $sql .= ", " . query_fields('subscribers');
                    break;
                case 'followups':
                    $sql = "SELECT M.*, U.name AS subscriber";
                    $sql .= ", " . query_fields('subscribers', 'U');
                    break;
                case 'templates':
                    $sql = "SELECT M.*, G.name AS group_name";
                    break;
                case 'groups':
                    $sql = "SELECT M.*";
                    $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->table_name . " WHERE FIND_IN_SET (M.id, group_ids)) AS counts";
                    $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->table_queries . " WHERE FIND_IN_SET (M.id, group_id)) AS query_counts";
                    break;
                case 'messages':
                    $sql = "SELECT M.*";
                    $sql .= ", CONCAT(EM.id, '!~!', EM.name, '!~!', M.template_id, '!~!', M.message_to, '!~!subscriber_messages') AS redirect_uri";
                    break;
                default:
                    $sql = "SELECT M.*";
            }
        }
        $table = 'subscriber_' . $module;
        $sql .= " FROM " . $this->db->dbprefix . $table . " M";
        if($count == 0) {
            switch($module) {
                case 'users':
                    $sql .= " LEFT JOIN " . $this->db->dbprefix . $this->table_statuses . " S ON S.id = M.status_id";
                    break;
                case 'followups':
                    $sql .= " LEFT JOIN " . $this->db->dbprefix . $this->table_name . " U ON U.id = M.subscriber_id";
                    break;
                case 'templates':
                    $sql .= " LEFT JOIN " . $this->db->dbprefix . $this->table_groups . " G ON G.id = M.group_id";
                    break;
                case 'messages':
                    $sql .= " LEFT JOIN " . $this->db->dbprefix . "email_templates EM ON EM.id = M.template_id";
                    break;
            }
        }
        if(isset($extra['filter']) && $extra['filter'] == 'deleted') {
            $sql .= " WHERE M.deleted = 1";
        } else {
            $sql .= " WHERE M.deleted = 0";
        }
        if(!empty($where)) {
            $sql .= ' AND '. $where;
        }
        // Subscribers Filter by Group ID
        if($module == 'users' && $this->session->userdata('filter_group_id')) {
            $filter_group_id = $this->session->userdata('filter_group_id');
            $sql .= ' AND FIND_IN_SET (' . $filter_group_id . ', M.group_ids)';
        }
        if ($count == 0) {
            $sql .= " GROUP BY M.id ORDER BY " . $sort_by;
            if ($limit != 0) {
                $sql .= " LIMIT " . $offset . ", " . $limit;
            } else {
                $sql .= " LIMIT $limit";
            }
            $query = $this->db->query($sql);
            return $query->result();
        } else {
            $result = $this->db->query($sql)->row();
            return $result ? $result->total_count : 0;
        }
    }
    
    /**
     * Get Thread/Post/Category Row
     * @param string $module
     * @param number $id
     * @param boolean $is_pdf
     * @param string $where
     * @return array|number
     */
    public function get_row($module, $id, $is_pdf = false, $where = null) {
        $sql = "SELECT M.*";
        $sql .= ", ST.color AS status_color, ST.name_lang AS status_lang, ST.name AS status";
        $sql .= " FROM " . $this->db->dbprefix . 'subscriber_' . $module . " M" . PHP_EOL;
        // $sql .= " LEFT JOIN " . $this->db->dbprefix . $this->table_groups . " G ON G.id = M.group_id" . PHP_EOL;
        $sql .= " LEFT JOIN " . $this->db->dbprefix . $this->table_statuses .  " ST ON ST.id = M.status_id" . PHP_EOL;
        $sql .= " WHERE M.deleted = 0";
        if(!empty($id)) {
            $sql .= " AND M.id = " . $id;
        }
        if(!empty($where)) {
            $sql .= " AND " . $where;
        }
        $result = $this->db->query($sql)->row();
        return $result;
    }
    
    /**
     * Display a list of Templates/Subscribers/Statuses data.
     *
     * @return void
     */
    public function view($module = 'users', $filter = 'all', $offset = 0, $role = 'admin', $extra = array()) {
        $has_status = false;
        // Status Change
        if (isset($_POST['status_change'])) {
            $this->gc->status_change('subscriber_' . $module, $_POST['status_change'], 'status');
        }
        // Groups - Export Excel
        if (isset($_POST['export_excel'])) {            
            gc_demo_mode();
            $this->gcrud->stats(array('module' => 'subscribers'));
            $this->export($_POST['export_excel']);
        }
        // Groups - Export PDF
        if (isset($_POST['export_pdf'])) {
            gc_demo_mode();
            $this->gcrud->stats(array('module' => 'subscribers'));
            $this->export($_POST['export_pdf'], 20000, 0, 'pdf');
        }
        // Set Default
        if (isset($_POST['set_default'])) {
            $this->gcrud->stats(array('module' => 'subscribers'));
            $this->set_default($_POST['set_default']);
            Template::redirect(current_url());
        }
        // Set group_id filter
        if(isset($_POST['filter_group_id'])) {
            $this->gcrud->stats(array('module' => 'subscribers'));
            $rows_filter = array('group_ids' => $_POST['filter_group_id']);
            $this->session->set_userdata('subscriber_users_rows_filter', $rows_filter);
            $this->session->set_userdata('filter_group_id', $_POST['filter_group_id']);
            Template::redirect(site_url(SITE_AREA . '/' . $this->context . '/' . $this->moduleName . '/manage/users'));
        }
        // Clear group_id filter
        if(isset($_POST['clear_group_filter'])) {
            $this->session->unset_userdata('filter_group_id');
        }
        // Allow Admin to change status
        if(isset($_POST['change_status'])) {
            $id = $_POST['change_status'];
            $new_status = $_POST['status_' . $id];
            $this->update_status($module, $id, $new_status);
        }
        // Query Group IDs
        if(isset($_POST['query_group_id'])) {
            $this->session->set_userdata('subscriber_query_group_id', $_POST['query_group_id']);
            Template::redirect(site_url(SITE_AREA . '/components/subscribers/manage/queries'));
        }
        $data = array(
            'context' => $this->context,
            'model' => 'subscribers',
            'fields_method' => $module,
            'set_table' => true,
            'query_method' => 'manage', // Extra Method in case of different columns
            'query_extra' => array('module' => $module, 'filter' => $filter),
            'table_alias' => 'M.', // Table Alias for table joins
            'prefix' => 'M',
            'filter' => $filter,
            'view' => 'components/manage',
            'redirect_uri' => '/' . $this->context . '/' . $this->moduleName . '/manage/' . $module,
            'edit_url' => site_url(SITE_AREA . '/' . $this->context . '/' . $this->moduleName . '/form/' . $module . '/edit/')
        );
        if(isset($extra['redirect_uri'])) {
            $data['redirect_uri'] = $extra['redirect_uri'];
        }
        switch($module) {
            case 'users':
                if($this->auth->role_id() != 1) { // staff and other users can view only their own submit links
                    $data['query_where'] = "M.created_by = " . $this->auth->user_id();
                }
                $data['search_fields'] = array('name', 'description');
                $data['delete_method'] = '_purge_subscriber';
                $has_status = true;
                break;
            case 'groups':
                $data['search_fields'] = array('name');
                break;
            case 'statuses':
                $data['search_fields'] = array('name');
                break;
            case 'followups':
                if($this->auth->role_id() != 1) { // staff and other users can view only their own submit links
                    $data['query_where'] = "M.created_by = " . $this->auth->user_id();
                }
                $data['search_fields'] = array('remarks', 'next_follow_up');
                break;
            case 'templates':
                $data['search_fields'] = array('subject', 'message');
                break;
        }
        if($has_status) {
            $data['redirect_uri'] = '/components/subscribers/manage/' . $module;
            $data['filter'] = $filter;
            $data['filter_counter'] = true;
            $filter_status = $this->get_filter('statuses', 0, null, $module);
            Template::set(array(
                'filter_status' => $filter_status
            ));
        }
        $this->gcrud->index('subscriber_' . $module, $offset, 'admin', $data);
    }
    
    /**
     * Get Filter
     * @param string $module
     * @return NULL[]
     */
    public function get_filter($module, $user_id = 0, $field_name = 'user_id', $table = null) {
        $column_name = '';
        switch($module) {
            case 'statuses':
                $column_name = 'status_id';
                break;
        }
        $module = 'subscriber_' . $module;
        $result = array();
        $records = $this->db->get($module)->result();
        if($records) {
            $where = array();
            if(!empty($user_id)) {
                $where[$field_name] = $user_id;
            }
            $count_table = empty($table) ? $this->table_name : 'subscriber_' . $table;
            foreach($records as $record) {
                $where[$column_name] = $record->id;
                $result[$record->id] = array(
                    'id' => $record->id,
                    'name' => !empty($record->name_lang) ? lang($record->name) : $record->name,
                    'counts' => $this->db->where($where)->count_all_results($count_table)
                );
            }
        }
        return $result;
    }
    
    /**
     * Get Group Fields
     * @param number $id
     * @param string $type
     * @return string
     */
    public function group_fields($id, $type = 'frontend') {
        $cacheID = 'subscriber_group_' . $id . '_' . $type;
        if(!$fields = $this->gc_subscribers_cache->load($cacheID)) {
            $field_ids = array();
            $records = $this->db->where('group_id', $id)->order_by('priority', 'ASC')->get($this->table_group_fields)->result_array();
            if($records) {
                foreach($records as $record) {
                    $field_ids[$record['field_id']] = $record;
                }
            }
            $module = 'subscriber_users';
            // get all fields
            $fields = $this->fields_model->get_fields($module, $type);
            if(!empty($field_ids)) {
                foreach($fields as $key => $field) {
                    if(!array_key_exists($field['id'], $field_ids)) {
                        unset($fields[$key]);
                    } else {
                        $rules = explode('|', $field['rules'] ?? '');
                        if($field_ids[$field['id']]['is_required'] == 1) {
                            $fields[$key]['required'] = 'required';
                            if(!in_array('required', $rules)) {
                                $rules[] = 'required';
                                $fields[$key]['rules'] = implode('|', $rules);
                            }
                        } else {
                            $fields[$key]['required'] = '';
                            if(in_array('required', $rules)) {
                                $rkey = array_search('required', $rules);
                                if (false !== $rkey) {
                                    unset($rules[$rkey]);
                                }
                                $fields[$key]['rules'] = implode('|', $rules);
                            }
                        }
                        $fields[$key]['priority'] = $field_ids[$field['id']]['priority'];
                    }
                }
            }
            // get the first field
            $first_field = reset($fields);
            if(array_key_exists('priority', $first_field)) { // if the first field has the priority key, sort fields by priority
                uasort($fields, "gc_sort_array"); // sort fields by priority
            }
            $this->gc_subscribers_cache->save($fields, $cacheID, array('subscriber_group_fields'));
        }
        return $fields;
    }
    
    /**
     * Common method to send email
     * @param array $data
     */
    public function send_sms_email($group_id, $id, $action_type = 1, $data = array()) {
        // 1 - Email Only, 2 - SMS Only, 3 - Both Email and SMS
        $notify_type = isset($data['notify_type']) ? $data['notify_type'] : 0;
        // Retrieve template from database
        $template = $this->db->where(array('group_id' => $group_id, 'action_type' => $action_type, 'status' => 1))->get($this->table_templates)->row();
        if(!$template) {
            return false;
        }
        $this->load->library('parser');
        $record = !empty($data['record']) ? $data['record'] : $this->get_row('users', $id);
        if(!$record) {
            Template::set_message(lang('invalid_id'), 'danger');
            return false;
        }
        if(empty($data['fields'])) {
            $form_fields = $this->subscribers_model->group_fields($group_id);
        } else {
            $form_fields = $data['fields'];
        }
        $from = !empty($data['from']) ? $data['from'] : null;
        $to = !empty($data['to']) ? $data['to'] : null;
        $record = (array) $record;
        $notification_template = array(); // admin notifications
        if(!empty($record['email']) && ($notify_type == 1 || $notify_type == 3)) {
            $meta_data = json_decode($record['meta_data'] ?? '', true);
            if(!empty($meta_data)) {
                foreach($meta_data as $meta) {
                    $record['label_' . $meta['name']] = !empty($meta['label_lang']) ? lang($meta['label']) : $meta['label'];
                    $record['value_' . $meta['name']] = $meta['value'];
                    $notification_template[$record['label_' . $meta['name']]] = $meta['value'];
                }
            }
            unset($record['meta_data']);
            foreach($form_fields as $form_field) {
                if(array_key_exists($form_field['name'], $record)) {
                    $record['label_' . $form_field['name']] = !empty($form_field['label_lang']) ? lang($form_field['label']) : $form_field['label'];
                    $record['value_' . $form_field['name']] = $record[$form_field['name']];
                    $notification_template[$record['label_' . $form_field['name']]] = $record[$form_field['name']];
                }
            }
            $record['site_title'] = settings_item('site.title');
            $record['site_url'] = site_url();
            $subject = $this->parser->parse_string($template->subject, $record, TRUE);
            $message = $this->parser->parse_string($template->message, $record, TRUE);
            $this->load->library('emailer/emailer');
            $no_queue = !empty($data['no_queue']) ? false : true;
            $module = !empty($record['module_id']) ? gc_module_name($record['module_id']) : 'subscriber_users';
            $this->load->library('crypt');
            $data = array(
                // 'from'		=> $from,
                'to'		=> $record['email'],
                'subject'	=> $subject,
                'message'	=> gc_unsubscribe($this->crypt->encode($record['id']), $module, $message),
                'headers' => array(
                    'reply_email' => settings_item('site.system_email'),
                    'reply_name' => settings_item('site.title')
                ),
            );
            
            // send message only
            $this->emailer->send($data, $no_queue); // Add to the Queue and process it manually, this will speed up the process
            if(!empty($template->notify_admin)) {
                $notification_template[lang('label_location')] = !empty($record['geo_location']) ? $record['geo_location'] : lang('not_applicable');
                $notification_template[lang('created_on')] = gc_date_format($record['created_on'], true);
                $notification_template[lang('ip_address')] = !empty($record['created_ip']) ? $record['created_ip'] : lang('not_applicable');
                // notification template is in a view file
                $notification_message = $this->load->view('subscribers/tpl/_notification', array('record' => $notification_template, 'template' => $template), true);
                $admin_data = array(
                    // 'from'		=> $from,
                    'to'		=> empty($template->notification_email) ? settings_item('site.system_email') : $template->notification_email,
                    'subject'	=> $template->name,
                    'message'	=> $notification_message,
                    'headers' => array(
                        'reply_email' => $record['email'],
                    )
                );
                $this->emailer->send($admin_data, $no_queue);
            }
        }
        // send message
        if(($notify_type == 2 || $notify_type == 3) && !empty($template->sms_text) && !empty($record['phone_number'])) {
            $sms_text = $this->parser->parse_string($template->sms_text, $record, TRUE);
            $this->gc->send_sms($sms_text, $record['phone_number'], true); // Add message  to the queue
        }
    }
    
    /**
     * Locates a single row  and joins there meta information based on a the id match.
     *
     * @access public
     *
     * @param int $row_id Integer of ID to fetch
     *
     * @return bool|object An object with the module's info and meta information, or FALSE on failure.
     */
    public function get_subscriber($id) {
        $record = $this->db->where('id', $id)->get($this->table_name)->row();
        $record = $this->merge_meta($record);
        return $record;
    }
    
    /**
     * Load Subscribers Form
     * @param number $id
     * @return string|boolean
     */
    public function load_form($id, $data = array()) {
        $fields = $this->subscribers_model->group_fields($id);
        if(!empty($fields)) {
            // get groups
            $groups = $this->gcache->get('subscriber_groups', array('result' => 1));
            $data['subscriber_fields'] = $fields;
            $data['subscriber_groups'] = $groups;
            $data['group_id'] = $id;
            $data['record'] = array();
            $content = $this->load->view('subscribers/tpl/_form', $data, true);
            return $content;
        }
        return false;
    }
    
    /**
     * Save Subscribers Form
     * @param number $group_id
     */
    public function save_subscriber($mobile = false) {
        $this->lang->load('subscribers/subscribers');
        /**
         * a single page can have multiple subscribers form
         * Remove the group_id and form_id field from the POST before sending them for save
         * @var array $post_data
         */
        $post_data = $this->input->post();
        $new_post_data = array(); // post data without prefix
        if(!empty($post_data)) {
            foreach($post_data as $pkey => $pvalue) {
                if(strpos($pkey, '___') !== false) {
                    $field = explode('___', $pkey);
                    $new_post_data[$field[1]] = $pvalue;
                } else if(!array_key_exists($pkey, $new_post_data)) { // to fix the row_id issue
                    $new_post_data[$pkey] = $pvalue;
                }
            }
            $_POST = $new_post_data;
        }
        $group_id = (int) $this->input->post('group_id');        
        $group = $this->db->where('id', $group_id)->get($this->table_groups)->row();
        if($group_id > 0) {
            $module = 'subscriber_users';
            $user_id = 0;
            $this->load->model('fields/fields_model');
            $form_fields = $this->subscribers_model->group_fields($group_id);
            
            // subscriber_queries fields
            $query_fields = array(); 
            foreach($form_fields as $fkey => $fvalue) {
                if(!empty($fvalue['query'])) {
                    $query_fields[] = $fvalue;
                    // do not save the field having query parameter in the subscribers table
                    // save them only in the subscriber_queries table
                    // to set query field, set the field extra input as query:1 while adding a subscriber's field
                    unset($form_fields[$fkey]);
                }
            }
            
            $result = $this->fields_model->fields_validation_rules($form_fields, $module, 'frontend');
            if($result) {
                $is_duplicate = false;
                $data = $this->fields_model->field_post_values($form_fields, $user_id);
                if(!empty($data['email'])) { // email is unique cannot be duplicate
                    $row = $this->db->where('email', $data['email'])->get($this->table_name)->row();
                    if($row) {                        
                        $subscriber_groups = explode(',', $row->group_ids ?? '');
                        if(in_array($group_id, $subscriber_groups)) {
                            $this->save_module_info($row->id, $new_post_data, $query_fields, $row); // save module_id and row_id information
                            $is_duplicate = true;
                            if(empty($group->notify_duplicate) || $row->status_id == 3) {
                                // unsubscribed user
                                if($row->status_id == 3) {
                                    $this->db->where('id', $row->id)->update($this->table_name, array('status_id' => 2));
                                }
                                // Notify Users even if the subscriber exists E.g. do not show duplicate message to demo users, resend SMS/Email with a success message
                                // resend sms/email to subscriber
                                $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'subscriber_templates', 'module_table' => 'subscriber_users', 'action_type' => 'add', 'group_id' => $group_id, 'id' => $row->id));
                                $message = !empty($group->message) ? (!empty($group->message_lang) ? lang($group->message) : $group->message) : lang('success_registration');
                                if($mobile) {
                                    return array('result' => 1, 'message' => $message, 'type' => 'success');
                                } else {
                                    echo json_encode(array('result' => 1, 'message' => $message, 'type' => 'success'));
                                }
                            } else {
                                $message = lang('subscriber_email_exists');
                                if($mobile) {
                                    return array('result' => 0, 'message' => $message, 'type' => 'danger');
                                } else {
                                    echo json_encode(array('result' => 0, 'message' => $message, 'type' => 'danger'));
                                }
                            }
                            return false;
                        }
                    }
                }
                if(!empty($data['phone_number'])) { // phone number is unique cannot be duplicate
                    $phone_row = $this->db->where('phone_number', $data['phone_number'])->get($this->table_name)->row();
                    if($phone_row) {
                        $subscriber_groups = explode(',', $phone_row->group_ids ?? '');
                        if(in_array($group_id, $subscriber_groups)) {
                            // email and phone number both exists but in a different row
                            if(!empty($row) && $row->id != $phone_row->id) {
                                $is_duplicate = false;
                            } else {
                                $is_duplicate = true;
                            }
                            if(!empty($group->notify_duplicate)) {
                                $this->save_module_info($phone_row->id, $new_post_data, $query_fields, $phone_row); // save module_id and row_id information
                                $message = lang('subscriber_phone_exists');
                                if($mobile) {
                                    return array('result' => 0, 'message' => $message, 'type' => 'danger');
                                } else {
                                    echo json_encode(array('result' => 0, 'message' => $message, 'type' => 'danger'));
                                }
                                return false;
                            }
                        }
                        if(empty($row) && empty($data['email'])) { // give priority to email id over phone
                            $row = $phone_row;
                        } else if(empty($row) && !empty($data['email'])) {
                            $is_duplicate = false;
                        }
                    }
                }
                if(!empty($row) && !$is_duplicate) {
                    // the subscriber exists in another group, add it to this group also
                    $subscriber_groups[] = $group_id;
                    $this->db->where('id', $row->id)->update($this->table_name, array('group_ids' => implode(',', $subscriber_groups)));
                    $message = $this->save_meta($row->id, $form_fields, $user_id, $group);
                    $this->save_module_info($row->id, $new_post_data, $query_fields, $row); // save module_id and row_id information
                    if($mobile) {
                        return array('result' => 1, 'message' => $message, 'type' => 'success');
                    } else {
                        echo json_encode(array('result' => 1, 'message' => $message, 'type' => 'success'));
                    }
                } else if(!$is_duplicate) {
                    if($this->input->post('phone_code')) {
                        $data['phone_code'] = $this->input->post('phone_code');
                    }
                    // new subscriber, save it
                    $data['group_ids'] = $group_id;
                    $data['created_on'] = date('Y-m-d H:i:s');
                    $data['created_ip'] = $this->input->ip_address();
                    $data['modified_on'] = $data['created_on'];
                    $data['modified_ip'] = $data['created_ip'];
                    if(empty($data['name'])) { // if the name field is optional, fill email or mobile number in this field.
                        $data['name'] = isset($data['email']) ? $data['email'] : (isset($data['phone_number']) ? $data['phone_number'] : '');
                    }
                    // save geo location information
                    // get information from the visitors table
                    $visitor_info = $this->session->userdata('visitor_info');
                    if(!empty($visitor_info)) {
                        $geo_location = !empty($visitor_info['city']) ? $visitor_info['city'] . ', ' : '';
                        $geo_location .= !empty($visitor_info['region']) ? $visitor_info['region'] . ', ' : '';
                        $geo_location .= $visitor_info['country'];
                        $data['geo_location'] = $geo_location;
                    }
                    if(array_key_exists('notify_user', $data)) {
                        unset($data['notify_user']);
                    }
                    if(empty($data['city_id'])) {
                        $data['city_id'] = 0;
                    }
                    if(empty($data['state_id'])) {
                        $data['state_id'] = 0;
                    }
                    if(empty($data['locality_id'])) {
                        $data['locality_id'] = 0;
                    }
                    $this->db->insert($this->table_name, $data);
                    $insert_id = $this->db->insert_id();
                    if($insert_id > 0) {
                        // save module_id and row_id information
                        $this->save_module_info($insert_id, $new_post_data, $query_fields);
                        
                        // send email to subscribers
                        $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'subscriber_templates', 'module_table' => 'subscriber_users', 'action_type' => 'add', 'group_id' => $group_id, 'id' => $insert_id));
                        
                        // save relation in the group_relations table
                        $this->db->on_duplicate($this->table_group_relations, array('subscriber_id' => $insert_id, 'group_id' => $group_id));
                        $message = $this->save_meta($insert_id, $form_fields, $user_id, $group, false, array('no_queue' => true));
                        if($mobile) {
                            return array('result' => 1, 'message' => $message, 'type' => 'success');
                        } else {
                            echo json_encode(array('result' => 1, 'message' => $message, 'type' => 'success'));
                        }
                    }
                }
            } else {
                $validation_errors = validation_errors();
                if($mobile) {
                    return array('result' => 0, 'message' => $validation_errors, 'type' => 'danger');
                } else {
                    $alert_type = 'danger';
                    if ($validation_errors) {
                        $message = ['success' => false, 'message' => $validation_errors, 'type' => $alert_type];
                    } else {
                        $message = ['success' => false, 'message' => lang('msg_failure'), 'type' => $alert_type];
                    }
                    echo json_encode($message);
                }
            }
        }
    }
    
    /**
     * Save Meta for
     * @param number $subscriber_id
     */
    public function save_meta_for($subscriber_id, $form_fields, $user_id) {
        $subscriber = $this->db->where('id', $subscriber_id)->get($this->table_name)->row();
        if($subscriber) {
            $group_ids = explode(',', $subscriber->group_ids ?? '');
            if(!empty($group_ids)) {
                foreach($group_ids as $group_id) {
                    $this->save_meta($subscriber_id, $form_fields, $user_id, 0, true);
                }
            }
        }
    }
    
    /**
     * Save Meta Information - Also called in save_module method of fields
     * @param number $subscriber_id
     * @param array $form_fields
     * @param number $user_id
     * @param object $group
     * @param boolean $is_admin_edit - TRUE - do not send email on admin edit
     * @param array $extra - e.g. no_queue - true, send email immediately instead of queueing it.
     */
    private function save_meta($subscriber_id, $form_fields, $user_id, $group, $is_admin_edit = false, $extra = array()) {
        if($subscriber_id) {
            $group_id = isset($group->id) ? $group->id : $group;
            // Meta information is also stored in the subscribers table to avoid unnecessary joins
            $meta_column = array();
            $meta_data = array();
            $metas = $this->fields_model->field_post_values($form_fields, $user_id, 1);
            foreach($form_fields as $field) {
                if(array_key_exists($field['name'], $metas)) {
                    $meta = array(
                        'subscriber_id' => $subscriber_id,
                        // 'group_id' => $group_id, Not required
                        'field_id' => $field['id'],
                        'value' => $metas[$field['name']]
                    );
                    $meta_data[] = $meta;
                    // Meta information is also stored in the subscribers table to avoid unnecessary joins
                    $meta_column_data = array(
                        'name' => $field['name'],
                        'label' => $field['label'],
                        'label_lang' => $field['label_lang'],
                        'value' => $metas[$field['name']]
                    );
                    $meta_column[] = $meta_column_data;
                }
            }
            if(!empty($meta_data)) {
                $this->db->on_duplicate($this->table_metas, $meta_data);
                $this->db->where('id', $subscriber_id)->update($this->table_name, array('meta_data' => json_encode($meta_column)));
            }
        }
        if(!$is_admin_edit) { 
            // do not send email on admin edit
            // send email to subscriber
            $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'subscriber_templates', 'module_table' => 'subscriber_users', 'action_type' => 'add', 'group_id' => $group_id, 'id' => $subscriber_id, 'user_type' => 'user'));
            $message = !empty($group->message) ? (!empty($group->message_lang) ? lang($group->message) : $group->message) : lang('success_registration');
            return $message;
        }
    }
    
    /**
     * Find whether the subscriber is duplicate or not
     * @param array $data
     * @param string $field
     * @return boolean|array
     */
    private function is_duplicate($data, $field = 'email') {
        $is_duplicate = true;
        if(!empty($data[$field])) { // unique cannot be duplicate
            $row = (array) $this->db->where($field, $data[$field])->get($this->table_name)->row();
            if($row) {
                $subscriber_groups = explode(',', $row['group_ids'] ?? '');
                if(in_array($group_id, $subscriber_groups)) {
                    return true;
                }
                return $row;
            }
        }
        return false;
    }
    
    /**
     * Products Enquiry Form
     * Save module_id and row_id information in the extras column
     * 
     * Add query:1 in the extra_fields while adding query fields to the subscriber form
     * Query fields will not be saved in the subscribers table
     * 
     * @param number $subscriber_id
     * @param number $post_data
     * @param number $query_fields - Query specific fields
     * @param array $subscriber
     */
    private function save_module_info($subscriber_id, $post_data = array(), $query_fields = array(), $subscriber = array()) {
        if(empty($subscriber)) {
            $subscriber = $this->db->where('id', $subscriber_id)->get('subscriber_users')->row();
        }
        if(!empty($subscriber)) {
            $item = array();
            $module_id = !empty($post_data['module_id']) ? (int) $post_data['module_id'] : 0;
            $row_id = !empty($post_data['row_id']) ? (int) $post_data['row_id'] : 0;
            if(!empty($module_id) && !empty($row_id)) {
                $table = gc_module_name($module_id);
                $row = $this->db->where('id', $row_id)->get($table)->row();
                if($row) {
                    $item = array(
                        'id' => $row->id,
                        'name' => $row->name,
                        'slug' => $row->slug,
                        'cover' => $row->cover,
                        'datetime' => date('Y-m-d H:i:s') // datetime on which enquired
                    );
                } 
            }
            
            // save information in the queries_metas_table to avoid creating new fields in the subscriber_queries table
            // this will be helpful in reporting also
            $metas = array();
            foreach($query_fields as $query_field) {
                if(!empty($post_data[$query_field['name']])) {
                    $item[$query_field['name']] = $post_data[$query_field['name']];
                    $metas[$query_field['name']] = $post_data[$query_field['name']];
                }
            }
            
            // save information in the subscriber_queries table
            // module_id and row_id fields are hardcoded, i.e. query_ is added as a prefix
            $queries_where = array('subscriber_id' => $subscriber_id); // , 'query_module_id' => $module_id, 'query_row_id' => $row_id
            // $record = $this->db->where($queries_where)->get($this->table_queries)->row();
            $datetime = date('Y-m-d H:i:s');
            $ip_address = $this->input->ip_address();
            
            // save information in the table itself to avoid unnecessary joins
            $subscriber_info = array( // for search operations
                'name' => !empty($post_data['name']) ? $post_data['name'] : $subscriber->name,
                'email' => !empty($post_data['email']) ? $post_data['email'] : $subscriber->email,
                'phone_number' => !empty($post_data['phone_number']) ? $post_data['phone_number'] : $subscriber->phone_number
            );
            $query_extra = array('item' => $item, 'subscriber' => $subscriber_info);
            $queries_where['extras'] = json_encode($query_extra);
            $queries_where['group_id'] = !empty($post_data['group_id']) ? $post_data['group_id'] : 0;
            $queries_where['created_on'] = $datetime;
            $queries_where['created_ip'] = $ip_address;
            $this->db->insert($this->table_queries, $queries_where);
            $query_id = $this->db->insert_id();
            if($query_id > 0 && !empty($metas)) {
                // save information in the query_metas table
                $meta_data = array();
                foreach($metas as $meta_key => $meta_value) {                    
                    $meta_data[] = array(
                        'subscriber_id' => $subscriber_id,
                        'query_id' => $query_id,
                        'meta_key' => $meta_key,
                        'meta_value' => $meta_value
                    );
                }
                $this->db->insert_batch($this->table_queries_metas, $meta_data);
            }
        }
    }
    
    /**
     * Get Row Information
     * @param string $module
     * @param number $id
     * @return array
     */
    public function _get($module, $id = 0, $field = 'name') {
        if(!empty($id)) {
            $this->db->where('id', $id);
        }
        $row = $this->db->get($module)->row();
        if($row) {
            if(!empty($field)) {
                return $row->{$field};
            } else {
                return $row;
            }
        } else {
            return '';
        }
    }
    
    /**
     * Purge Subscribers
     * @param array $ids
     */
    public function _purge_subscriber($ids = array()) {
        if(!empty($ids)) {
            foreach($ids as $id) {
                // delete subscribers meta first
                $this->db->where('subscriber_id', $id)->delete($this->table_metas);
                
                // delete subscribers followups
                $this->db->where('subscriber_id', $id)->delete($this->table_followups);
                
                // delete subscribers group_relations
                $this->db->where('subscriber_id', $id)->delete($this->table_group_relations);
                
                // delete subscribers queries
                $this->db->where('subscriber_id', $id)->delete($this->table_queries);
                
                // notify subscriber
                $record = $this->get_row('users', $id);
                if($record) {
                    $group_ids = explode(',', $record->group_ids ?? '');
                    if(!empty($group_ids)) {
                        foreach($group_ids as $group_id) {
                            $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'subscriber_templates', 'module_table' => 'subscriber_users', 'action_type' => 'delete', 'group_id' => $group_id, 'id' => $id));
                        }
                    }
                }
                
                // delete subscriber
                $this->db->where('id', $id)->delete($this->table_name);
            }
        }
    }
    
    /**
     * Meta information is saved in the subscribers table to avoid unnecessary joins
     * Get it and join with records
     * @param array $record
     */
    public function merge_meta($record) {
        if(!empty($record->meta_data)) {
            $metas = json_decode($record->meta_data, true);
            if(!empty($metas)) {
                foreach($metas as $meta) {
                    $record->{$meta['name']} = $meta['value'];
                }
            }
        }
        return $record;
    }
    
    /**
     * Export subscribers list in an Excel/PDF format
     * @param number $group_id
     */
    public function export($group_id = 0, $limit = 20000, $offset = 0, $format = 'excel', $sort_by = 'M.id DESC', $where = null) {
        $this->auth->restrict('Subscribers.Components.Export');
        $this->load->library('format');
        $module = 'users';
        $form_fields = $this->subscribers_model->group_fields($group_id);
        if(!empty($form_fields)) {
            $table = 'subscriber_' . $module;
            $sql = "SELECT M.*, S.color AS status_color, S.name AS status, S.name_lang AS status_lang";
            $sql .= ", (SELECT GROUP_CONCAT(name SEPARATOR ', ') FROM " . $this->db->dbprefix . $this->table_groups . " WHERE FIND_IN_SET(id, M.group_ids)) AS groups_info";
            $sql .= " FROM " . $this->db->dbprefix . $table . " M";
            $sql .= " LEFT JOIN " . $this->db->dbprefix . $this->table_statuses . " S ON S.id = M.status_id";
            $query_where = null;
            if(!empty($group_id)) {
                $group = $this->db->where('id', $group_id)->get($this->table_groups)->row();
                $query_where = ' WHERE FIND_IN_SET (' . $group_id . ', M.group_ids)';
            }
            if(!empty($where)) {
                $query_where .= !empty($query_where) ? ' AND ' . $where : ' WHERE ' . $where;
            }
            $sql .= $query_where;
            $sql .= " GROUP BY M.id ORDER BY " . $sort_by;
            if ($limit != 0) {
                $sql .= " LIMIT " . $offset . ", " . $limit;
            } else {
                $sql .= " LIMIT $limit";
            }
            $records = $this->db->query($sql)->result();
            if(!empty($records)) {
                $mandatory_fields = array('name', 'status', 'created_on');
                $not_specified = lang('not_specified');
                $field_names = array(
                    'name'
                );
                $field_labels = array(lang('label_name'));
                $columns = array('name' => $not_specified);
                foreach($form_fields as $form_field) {
                    if(!in_array($form_field['name'], $mandatory_fields)) {
                        $field_names[] = $form_field['name'];
                        $field_labels[] = !empty($form_field['label_lang']) ? lang($form_field['label']) : $form_field['label'];
                        $columns[$form_field['name']] = $not_specified;
                    }
                }
                if(empty($group_id)) {
                    $field_names[] = 'groups';
                    $field_labels[] = lang('subscriber_groups');
                    $columns['groups'] = $not_specified;
                }
                $field_names[] = 'status';
                $field_names[] = 'created_on';
                $field_labels[] = lang('singular_subscriber_statuses');
                $field_labels[] = lang('created_on');
                $columns['status'] = $not_specified;
                $columns['created_on'] = $not_specified;
                $file_name = 'Subscribers' . (!empty($group->name) ? '_' . $group->name : '') . '_' . date('Y-m-d H:i:s');
                $file_name = $this->slug->create_slug($file_name);
                $rows = array();
                /*if(empty($group_id)) {
                 foreach($records as $key => $record) {
                 $records[$key] = $this->merge_meta($record);
                 }
                 $rows = $records;
                 }*/
                foreach($records as $key => $record) {
                    $record = (array) $this->merge_meta($record);
                    $row = $columns;
                    $row['name'] = $record['name'];
                    foreach($record as $key => $value) {
                        if(in_array($key, $field_names)) {
                            $row[$key] = $value;
                        }
                    }
                    $row['status'] = !empty($record['status_lang']) ? lang($record['status']) : $record['status'];
                    $row['created_on'] = $record['created_on'];
                    $rows[] = $row;
                }
                if($format == 'excel') {
                    $data = array('file_name' => $file_name, 'field_names' => $field_names, 'rows' => $rows);
                    $this->format->to_excel($data);
                } else { // pdf format
                    $content = $this->load->view('subscribers/tpl/_pdf', array('field_names' => $field_labels, 'rows' => $rows), true);
                    $data = array(
                        'filename' => $file_name,
                        'content' => $content,
                        'download' => true
                    );
                    $this->gc->create_pdf($file_name, $content);
                }
            } else {
                Template::set_message(lang('subscriber_export_error'), 'danger');
            }
        } else {
            Template::set_message(lang('subscriber_export_error'), 'danger');
        }
    }
    
    /**
     * Send Messages
     *
     * @param string $type
     * @param number $id
     * @param number $offset
     */
    public function send($type = 'email', $id = 0, $limit = 25, $offset = 0, $module = 'subscriber_users') {
        $datatable = 1;
        if($datatable == 1) {
            if($module == 'users') {
                $columns = array('check' => 'check', 'id' => lang('label_id'), 'display_name' => lang('label_name'), 'email' => lang('label_email'), 'phone_number' => lang('label_phone_number'), 'active' => lang('label_status'));
            } else if($module == 'listings') {
                $columns = array('check' => 'check', 'id' => lang('label_id'), 'title' => lang('label_name'), 'email' => lang('label_email'), 'phone_number' => lang('label_phone_number'), 'status' => lang('label_status'));
            } else {
                $columns = array('check' => 'check', 'id' => lang('label_id'), 'name' => lang('label_name'), 'email' => lang('label_email'), 'phone_number' => lang('label_phone_number'), 'status_id' => lang('label_status'));
            }
            Template::set('columns', $columns);
            // Load datatables to display rating_users
            Assets::add_js('//cdn.datatables.net/1.10.19/js/jquery.dataTables.min.js');
            Assets::add_js('//cdn.datatables.net/fixedheader/3.1.6/js/dataTables.fixedHeader.min.js');
            Assets::add_js('//cdn.datatables.net/responsive/2.2.3/js/dataTables.responsive.min.js');
            Assets::add_js('//cdn.datatables.net/1.10.19/js/dataTables.bootstrap4.min.js');
            // Assets::add_css('//cdn.datatables.net/1.10.19/css/jquery.dataTables.min.css');
            Assets::add_css('//cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/4.1.3/css/bootstrap.css');
            Assets::add_css('//cdn.datatables.net/1.10.19/css/dataTables.bootstrap4.min.css');
            Assets::add_css('//cdn.datatables.net/fixedheader/3.1.6/css/fixedHeader.bootstrap.min.css');
            Assets::add_css('//cdn.datatables.net/responsive/2.2.3/css/responsive.bootstrap.min.css');
            Assets::add_js('
            // destroy datatable
            if ($.fn.DataTable.isDataTable(".table-data")) {
              $(\'.table-data\').DataTable().clear().destroy();
            }
            var table = $(".table-data").DataTable( {
                "processing": true,
                "serverSide": true,
                "responsive": true,
                columnDefs: [ { // disable sorting on first column
                    "targets": [0], /* column index [0,1,2,3]*/
                    "orderable": false, /* true or false */
                }],
                "ajax": {
                    "url": "' . site_url(SITE_AREA . '/components/subscribers/get_users') . '",
                    "data": function (d) { // pass additional parameters to the jQuery DataTable ajax call
                         d.module = "' . $module . '";
                     }
                 }
            });', 'inline');
        }
        $redirect_url = site_url(SITE_AREA . "/components/");
        $redirect_url .= $type == 'email' ? 'emailer/create' : 'messages/create';
        $redirect_url .= '/' . 0;
        // DATE TIME PICKER
        Assets::add_js('moment.js');
        Assets::add_css ( 'bootstrap-datetimepicker.min.css' );
        Assets::add_js ( 'bootstrap-datetimepicker.min.js' );
        // Assets::add_module_js('subscribers', 'subscribers.js');
        $this->load->model('subscribers/subscribers_model');
        $this->load->library('form_validation');
        $this->load->helper('date_helper');
        $this->load->library('crypt');
        $this->load->library('parser');
        Assets::add_js(array(
            'tinymce/tinymce.min.js'
        ));
        Assets::add_module_js('subscribers', 'subscribers.js');
        if(empty($_POST['message']) && $this->session->userdata('template_id')) {
            $template = $this->db->where('id', $this->session->userdata('template_id'))->get($type . '_templates')->row();
            if($template) {
                $message_data = array(
                    'message' => $template->message
                );
                if(isset($template->subject)) {
                    $message_data['subject'] = $template->subject;
                }
                Template::set($message_data);
            }
        }
        if(isset($_POST['save'])) {            
            gc_demo_mode();
            $this->gcrud->stats(array('module' => ($type == 'email' ? 'emailer' : 'messages')));
            $action = $_POST['save'];
            $message_to = isset($_POST['message_to']) ? $_POST['message_to'] : null;
            // Validate message and recipients
            $this->form_validation->set_rules('name', 'lang:label_name', 'trim|required|max_length[255]');
            $this->form_validation->set_rules('message', 'lang:label_message', 'trim|required|min_length[1]');
            if($action == 'send') {
                $message_to = 'selected';
            }
            if($message_to == 'ids') {
                // Check range in case of all button clicks except the CREATE one.
                $this->form_validation->set_rules('id_from', lang('label_from'), 'trim|required|integer');
                $this->form_validation->set_rules('id_to', lang('label_to'), 'trim|required|integer|max_val[10000]|compare_fields['.$this->input->post('id_from').']|diff_between['.$this->input->post('id_from').',10000]');
            } else if($message_to == 'users') {
                $this->form_validation->set_rules('specific_users', lang('specific_users'), 'trim|required');
            } else if($message_to == 'groups') {
                $this->form_validation->set_rules('group_id', lang('label_group'), 'trim|integer');
                $this->form_validation->set_rules('limit', lang('label_limit'), 'trim|integer|required');
                $this->form_validation->set_rules('offset', lang('label_offset'), 'trim|integer|required');
                if($type == 'message') {
                    $this->form_validation->set_rules('phone_code', lang('phone_code'), 'trim|integer');
                }
            } else if($module != 'users') {
                $this->form_validation->set_rules('checked[]', 'lang:gc_menu_subscribers', 'required');
            }
            $this->form_validation->set_rules('status_id', lang('label_status'), 'trim');
            $this->form_validation->set_rules('scheduled_on', 'lang:em_scheduled_on', 'trim|valid_datetime_future');
            $this->form_validation->set_rules('schedule_limit', lang('label_schedule_limit'), 'trim|integer');
            $this->form_validation->set_rules('schedule_interval', lang('label_schedule_interval'), 'trim|integer');
            $this->form_validation->set_rules('params', lang('label_params'), 'trim');
            $this->form_validation->set_rules('priority', lang('priority'), 'trim|integer');
            if($type == 'email') {
                $this->form_validation->set_rules('subject', 'lang:label_subject', 'trim|required|min_length[1]');
                $this->form_validation->set_rules('headers', 'lang:label_headers', 'trim|is_array');
                $this->form_validation->set_rules('switch_from', 'lang:label_switch_from', 'trim');
                $this->form_validation->set_rules('switch_template', 'lang:label_switch_template', 'trim');
            }
            
            if ($this->form_validation->run() === true) {
                $name = $this->input->post('name');
                $template_id = (int) $this->input->post('template_id');
                $subject = $this->input->post('subject');
                $message = $this->input->post('message');
                $specific_users = $this->input->post('specific_users');
                $group_id = $this->input->post('group_id');
                $limit = (int) $this->input->post('limit');
                $offset = (int) $this->input->post('offset');
                $id_from = (int) $this->input->post('id_from');
                $id_to = (int) $this->input->post('id_to');
                $status_id = (int) $this->input->post('status_id');
                $phone_code = (int) $this->input->post('phone_code');
                $scheduled_on = $this->input->post('scheduled_on') ? $this->input->post('scheduled_on') : NULL;
                $schedule_limit = (int) $this->input->post('schedule_limit');
                $schedule_interval = (int) $this->input->post('schedule_interval');
                $priority = (int) $this->input->post('priority');
                $headers = $type == 'email' ? json_encode($this->input->post('headers')) : null;
                $switch_from = (int) $this->input->post('switch_from');
                $switch_template = (int) $this->input->post('switch_template');
                $checked = $this->input->post('checked');
                $specific_ids = null;
                if(!empty($checked)) {
                    $specific_ids = implode(',', $checked);
                }
                $data = array(
                    'type' => $type,
                    'name' => $name,
                    'user_id' => (int) $this->auth->user_id(),
                    'template_id' => $template_id,
                    'module_id' => gc_module_id($module),
                    'message_to' => $message_to,
                    'specific_users' => $specific_users,
                    'specific_ids' => $specific_ids,
                    'group_id' => $group_id,
                    'rows_limit' => $limit,
                    'rows_offset' => $offset,
                    'id_from' => $id_from,
                    'id_to' => $id_to,
                    'status_id' => $status_id,
                    'phone_code' => $phone_code,
                    'subject' => $subject,
                    'message' => $message,
                    'headers' => $headers,
                    'scheduled_on' => $scheduled_on,
                    'schedule_limit' => $schedule_limit,
                    'schedule_interval' => $schedule_interval,
                    'switch_from' => $switch_from,
                    'switch_template' => $switch_template,
                    'params' => json_encode($this->input->post('params')),
                    'priority' => $priority,
                    'created_on' => date('Y-m-d H:i:s')
                );
                $this->db->insert($this->table_messages, $data);
                $insert_id = $this->db->insert_id();
                if($insert_id > 0) {
                    $result = $this->bulk_send($insert_id);
                    if (isset($result['success_count'])) {
                        Template::set_message(sprintf(lang('message_queued'), $result['success_count']), 'success');
                    } else {
                        Template::set_message(sprintf(lang('message_queued'), 0), 'danger');
                    }
                } else {
                    Template::set_message(sprintf(lang('msg_create_failure'), lang('label_message')), 'danger');
                }
            }
        }
        $records = array();
        if($datatable == 0) {
            // Normal search
            $where = 'M.status_id != 3';
            $searchterm = $this->gc->search('emailer', $this->uri->segment(6), '/messages/send');
            if(!empty($searchterm)) {
                $where .= " AND M.email LIKE '%" . $searchterm . "%'";
            }
            
            // Sort By
            $sort_by = 'M.id DESC';
            $order_by = $this->gc->sort('messages');
            if(!empty($order_by)) {
                $sort_by = $order_by;
            }
            if($module == 'users' || $module == 'listings' || $module == 'profiles') {
                // Records                
                $where = '';
                switch($module) {
                    case 'users':
                        $select = 'M.*, M.display_name AS name, "#444" AS status_color, CASE WHEN active = 1 THEN "status_active" ELSE "status_inactive" END AS status, 1 AS status_lang';
                        $where = "M.active != 3";
                        break;
                    case 'listings':
                        $select = 'M.*, M.title AS name';
                        break;
                    case 'profiles':
                        $select = 'M.*';
                        break;
                }                
                if(!empty($searchterm)) {
                    $where .= " AND M.email LIKE '%" . $searchterm . "%'";
                }
                $sql = "SELECT " . $select . " FROM " . $this->db->dbprefix . "users M WHERE " . $where;
                $records = $this->db->query($sql)->result();
                
                // Counts
                $sql = "SELECT COUNT(*) AS total_rows FROM " . $this->db->dbprefix . "users M WHERE " . $where;
                $total_rows = $this->db->query($sql)->row();
                $total_rows = isset($total_rows->total_rows) ? $total_rows->total_rows : 0;
            } else {
                $records = $this->manage(0, $where, $sort_by, $limit, $offset, array('module' => 'users'));
                $total_rows = $this->manage(1, $where, $sort_by, $limit, $offset, array('module' => 'users'));
            }
            $this->load->library('pagination');
            $pager['base_url'] = $redirect_url;
            $pager['total_rows'] = $total_rows;
            $pager['per_page'] = $limit;
            $pager['uri_segment'] = 7;
            
            $this->pagination->initialize($pager);
        }
        
        // Edit Record
        if($id == 0) {
            $record = array();
        } else {
            $record = $this->db->where('id', $id)->get($this->table_messages)->row();
        }
        $available_variables =  $this->share_variables($module);
        Template::set(array(
            'type' => $type,
            'module' => $module,
            'record' => $record,
            'records' => $records,
            'toolbar_title' => lang('subscriber_new_' . $type),
            'subject_variables' => $available_variables['subject'],
            'message_variables' => $available_variables['message']
        ));
        Template::set_view('subscribers/components/send');
        Template::render();
    }
    
    /**
     * Return next schedule date using the intervals
     * @param string $schedule_date
     * @param number $schedule_interval
     * @return string|DateTime
     */
    public function next_schedule_date($schedule_date, $schedule_interval = 7) {
        if(!empty($schedule_date)) {
            $schedule_date = new DateTime($schedule_date);
            date_add($schedule_date,date_interval_create_from_date_string($schedule_interval . " days"));
            return date_format($schedule_date,"Y-m-d H:i:s");
        } else {
            return $schedule_date;
        }
    }
    
    /**
     * Bulk Send
     * @param number $id
     */
    public function bulk_send($id) {
        $row = $this->db->where('id', $id)->get($this->table_messages)->row();
        if($row) {
            $table = gc_module_name($row->module_id);
            if($table == 'subscriber_users') {
                $status_field = 'status_id';
            } else if($table == 'listings' || $table == 'profiles') {
                $status_field = 'status';
            } else { // users table
                $status_field = 'active';
            }
            $type = $row->type;
            $module = gc_module_name($row->module_id);
            // $module = 'subcriber_users';
            $this->load->library('parser');
            if($row->type == 'email') {
                $column = 'email';
                $table_queue = 'email_queue';
            } else {
                $column = 'phone_number';
                $table_queue = 'message_queue';
            }
            switch($row->message_to) {
                case 'groups': // specific groups
                    switch($module) {
                        case 'listings':
                            $where = $column . ' != "" AND status = 1';
                            break;
                        case 'users':
                            $where = $column . ' != "" AND active = 1';
                            break;
                        default:                            
                            $where = $column . ' != "" AND status_id = 2';
                    }                   
                    if($row->group_id != 0) {
                        $where .= ' AND FIND_IN_SET(' . $row->group_id . ', group_ids)';
                    }
                    if($row->type == 'message' && !empty($row->phone_code)) { // send message to specific country numbers
                        $where .= " AND phone_code = " . $row->phone_code;
                    }
                    $this->db->limit($row->rows_limit, $row->rows_offset);
                    $records = $this->db->where($where)->get($table)->result_array();
                    break;
                case 'users': // specific users (phone numbers/email)
                    $records = array();
                    $specific_users = array_map('trim', explode(',', $row->specific_users ?? ''));
                    if(!empty($specific_users)) {
                        foreach($specific_users as $specific_user) {
                            $record = array(
                                'id' => 0,
                                $column => $specific_user,
                                'meta_data' => null
                            );
                            $records[] = $record;
                        }
                    }
                    break;
                case 'ids': // ids between a specific range
                    $where = 'id BETWEEN ' . $row->id_from . ' AND ' . $row->id_to . ' AND ' . $column . ' != ""';
                    /*if(property_exists($row, $status_field)) {
                        $where .= ' AND ' . $status_field . ' = ' . $row->{$status_field};
                    }*/
                    $where .= ' AND ' . $status_field . ' = ' . $row->status_id;
                    // $where .= ' AND status_id = ' . $row->status_id;
                    // $records = $this->db->where($where)->get($table)->result_array();
                    $sql = "SELECT * FROM " . $this->db->dbprefix . $table . " WHERE " . $where;
                    $records = $this->db->query($sql)->result_array();
                    break;
                case 'selected': // id selected using checkboxes
                    $where = 'id IN (' . $row->specific_ids . ') AND ' . $column . ' != ""';
                    // $where .= ' AND ' . $status_field . ' = ' . $row->status_id;
                    $records = $this->db->where($where)->get($table)->result_array();
                    break;
                default:
                    switch($module) {
                        case 'listings':
                        case 'profiles':
                            $where = $status_field . ' = 1';
                            break;
                        case 'users':
                            $where = $status_field . ' = 1';
                            break;
                        default:
                            $where = $status_field . ' = 2';
                    } 
                    $records = $this->db->where($where)->get($table)->result_array();
                    break;
            }
            $success_count = 0;
            $i = 1;
            $last_ip = $this->input->ip_address();
            if(!empty($records)) {
                if($type == 'email') {
                    // email is opened or not
                    $links = array();
                    $email_links = $this->db->get('email_links')->result();
                }
                $this->load->library('crypt');
                $this->lang->load('emailer/emailer');
                $this->session->unset_userdata('subscriber_scheduled_on');
                $module_id = gc_module_id($module);
                foreach($records as $record) {
                    $record = $this->merge_meta((object)$record);
                    $record = (array) $record;
                    if(!empty($record[$column])) {
                        $schedule_dates = array();
                        // process schedule dates
                        if(!empty($row->scheduled_on)) {
                            if($row->schedule_limit > 0 && $row->schedule_interval > 0 && ($i % $row->schedule_limit) == 0) { // schedule limit reached, get new date
                                $schedule_dates[] = $this->next_schedule_date($row->scheduled_on, $row->schedule_interval);
                            } else {
                                $schedule_dates[] = $row->scheduled_on;
                            }
                        } else {
                            $scheduled_on = $this->calculate_schedule_on((object) $record, $row, array('processed_counts' => $i));
                            if(!empty($scheduled_on)) {
                                $schedule_dates = $scheduled_on;
                            } else {
                                $schedule_dates[] = null;
                            }
                        }
                        ++$i;
                        foreach($schedule_dates as $schedule_date) {
                            $record['scheduled_on'] = $schedule_date;
                            $message_data = $record;
                            $data = array(
                                'user_id' => (int) $this->auth->user_id(),
                                'template_id' => (int) $row->template_id,
                                'module_id' => $module_id,
                                'row_id' => $record['id'],
                                'sent_id' => $id,
                                'scheduled_on' => isset($record['scheduled_on']) ? $record['scheduled_on'] : null,
                                'priority' => $row->priority,
                                'date_sent' => date('Y-m-d H:i:s')
                            );
                            if($type == 'email') { // email specific
                                // insert information in the email_sent table and get the sent id
                                // email sent email will have a unique sent id to track it
                                $email_sent_data = array(
                                    'module_id' => $module_id,
                                    'row_id' => $record['id'],
                                    'message_id' => $id,
                                    'last_ip' => $last_ip
                                );
                                $this->db->insert('email_sent', $email_sent_data);
                                $sent_id = $this->db->insert_id();
                                $encrypted_sent_id = $this->crypt->encode($sent_id);
                                // default parameters
                                $message_data['value_site_title'] = settings_item('site.title');;
                                $message_data['value_site_email'] = settings_item('site.system_email');
                                $message_data['value_site_url'] = anchor(site_url(), site_url());
                                $message_data['value_site_address'] = settings_item('site.address');
                                $message_data['value_site_phone'] = settings_item('site.phone_number');
                                // message data
                                $message_data['value_logo_url'] = '<img src="' . str_replace('..', '.', site_url('logo?read=' . $encrypted_sent_id)) . '" alt="' . settings_item('site.title') . '" />';
                                
                                // insert links
                                if($email_links) {
                                    foreach($email_links as $email_link) {
                                        $message_data['value_link_only_' . $email_link->id] = site_url('visit?id=' . $encrypted_sent_id . '&link=' . $this->crypt->encode($email_link->id));
                                        $message_data['value_link_' . $email_link->id] = anchor(site_url('visit?id=' . $encrypted_sent_id . '&link=' . $this->crypt->encode($email_link->id)), $email_link->url);
                                    }
                                }
                                
                                $subject = $this->parser->parse_string($row->subject, $record, true);
                                $data['to_email'] = $record['email'];
                                $data['subject'] = $subject;
                                $data['headers'] = $row->headers;
                                $data['switch_from'] = $row->switch_from;
                                $data['switch_template'] = $row->switch_template;
                            } else {
                                $phone_number = explode(',', $record['phone_number'] ?? ''); // the phone number field can have multiple values
                                $data['phone_number'] = $phone_number[0]; // $record['phone_number'];
                            }
                            $message = $this->parser->parse_string($row->message, $message_data, true);
                            if($type == 'email') {
                                $data['message'] = gc_unsubscribe($this->crypt->encode($record['id']), $module, $message);
                            } else {
                                $data['message'] = $message;
                            }
                            $this->db->insert($table_queue, $data);
                            if($this->db->insert_id() > 0) {
                                $success_count++;
                            }
                        }
                    }
                }
                return array('success_count' => $success_count);
            }
        }
    }
    
    /**
     * Display available variable names to the Admin
     * @param string $module
     * @return multitype:Ambigous <multitype:, multitype:string >
     */
    public function share_variables($module, $separator = '<br>') {
        $subject = array();
        $message = array();
        if($module == 'users') {
            $subject = array(
                'email' => lang('label_email'),
                'username' => lang('bf_username'),
                'display_name'=>lang('label_display_name')
            );
            $message = array(
                'email' => lang('label_email'),
                'username' => lang('bf_username'),
                'display_name' => lang('label_display_name'),
                'profile_image' => lang('label_profile_image'),
                'phone_number' => lang('label_phone_number'),
                'address' => lang('label_address'),
                'city' => lang('label_city'),
                'state' => lang('label_state'),
                'postcode' => lang('label_postcode'),
                'country' =>lang('label_country')
            );
        } else {
            $message = array();
            $form_fields = $this->group_fields(0);
            if($form_fields) {
                foreach($form_fields as $form_field) {
                    $message[$form_field['name']] = !empty($form_field['label_lang']) ? lang($form_field['label']) : $form_field['label'];
                }
            }
            $subject = $message;
        }
        $subject_format = array();
        if(!empty($subject)) {
            foreach($subject as $key => $value) {
                $subject_format[] = $key . ' => ' . $value;
            }
        }
        $message_format = array();
        if(!empty($message)) {
            foreach($message as $key => $value) {
                $message_format[] = $key . ' => ' . $value;
            }
        }
        $help = lang('help_variables') . '<br />';
        return array('subject' => $help . (implode($separator, $subject_format)), 'message' => $help . (implode($separator, $message_format)));
    }
    
    /**
     * Display Summary on the Home Page
     * @return array
     */
    public function summary() {
        $sql = "SELECT COUNT(*) AS total_users";
        $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->table_name . " WHERE status_id = 1) AS total_inactive";
        $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->table_name . " WHERE status_id = 2) AS total_active";
        $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->table_name . " WHERE status_id = 3) AS total_unsubscribed";
        $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->table_followups . " WHERE DATE(next_follow_up) = '" . date('Y-m-d') . "') AS todays_followups";
        // $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->table_links . ") AS total_links";
        // $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->table_links . " WHERE status_id = 1) AS inactive_links";
        $sql .= " FROM " . $this->db->dbprefix . $this->table_name;
        $query = $this->db->query($sql);
        $record = $query->row();
        if($record) {
            $data = array();
            foreach($record as $key => $value) {
                switch($key) {
                    case 'total_users':
                        $url = site_url(SITE_AREA . '/components/subscribers/manage/users');
                        break;
                    case 'total_inactive':
                        $url = site_url(SITE_AREA . '/components/subscribers/manage/users/status_id--1');
                        break;
                    case 'total_active':
                        $url = site_url(SITE_AREA . '/components/subscribers/manage/users/status_id--2');
                        break;
                    case 'total_unsubscribed':
                        $url = site_url(SITE_AREA . '/components/subscribers/manage/users/status_id--3');
                        break;
                    case 'todays_followups':
                        $url = site_url(SITE_AREA . '/components/subscribers/manage/followups');
                        break;
                    default:
                        $url = null;
                }
                $data[$key] = array(
                    'value' => $value,
                    'url' => $url
                );
            }
            return (object) $data;
        } else {
            return $record;
        }
    }
    
    /**
     * Method called after subscriber insert/update
     * @param number $id
     * @param string $action_type
     * @param string $record
     */
    public function after_subscriber_save($id, $action_type = 'insert', $record = array()) {
        $notify_user = (int) $this->input->post('notify_user');
        if(!empty($notify_user)) {
            $record = $this->get_row('users', $id);
            if($record) {
                $group_ids = explode(',', $record->group_ids ?? '');
                if(!empty($group_ids)) {
                    foreach($group_ids as $group_id) {
                        $this->gemailer->sms_email_pdf($notify_user == 1 ? 'email' : ($notify_user == 2 ? 'sms' : 'sms_email'), array('template_table' => 'subscriber_templates', 'module_table' => 'subscriber_users', 'action_type' => ($action_type == 'insert' ? 'add' : 'edit'), 'group_id' => $group_id, 'id' => $id));
                    }
                }
            }
        }
    }
    
    /**
     * Save product row information in the extras table to avoid joins
     * @param number $id
     * @param string $action_type
     * @param array $record
     */
    public function after_query_save($id, $action_type = 'insert', $record = array()) {
        $record = $this->get_query($id);
        if(!$record) {
            return false;
        }
        $module_id = $record->query_module_id;
        $row_id = $record->query_row_id;
        $table = gc_module_name($module_id);
        $row = $this->db->where('id', $row_id)->get($table)->row();
        if($row) {
            $subscriber = $this->db->where('id', $record->subscriber_id)->get($this->table_name)->row();
            if(!$subscriber) {
                return false;
            }
            $extras = (array) json_decode($subscriber->extras ?? '', true);
            $items = !empty($extras['items']) ? $extras['items'] : array();
            $item = array(
                'id' => $row->id,
                'name' => $row->name,
                'slug' => $row->slug,
                'cover' => $row->cover,
                'datetime' => date('Y-m-d H:i:s') // datetime on which enquired
            );
            $subscriber_info = array( // for search operations
                'name' => $subscriber->name,
                'email' => $subscriber->email,
                'phone_number' => $subscriber->phone_number
            );
            // save information in the subscriber_queries table
            $this->db->where('id', $id)->update($this->table_queries, array('extras' => json_encode(array('item' => $item, 'subscriber' => $subscriber_info))));
            
            // update data in the subscriber_users table to avoid joins
            $items[$module_id . '_' . $row_id] = $item;
            $extras['items'] = $items;
            $this->db->where('id', $subscriber->id)->update('subscriber_users', array('extras' => json_encode($extras)));
        }
    }
    
    /**
     * Get Query
     * @param number $value - ID of the row
     * @param string $field - Name of the field
     * @return object
     */
    public function get_query($value, $field = 'id') {
        $record = $this->db->where($field, $value)->get($this->table_queries)->row();
        return $record;
    }
    
    /**
     * Get subscribers data
     * @param number $group_id
     * @param number $id
     * @return array
     */
    public function get_info($group_id, $id) {
        $record = $this->get_row('users', $id);
        if(!$record) {
            return false;
        }
        $form_fields = $this->subscribers_model->group_fields($group_id);
        $record = (array) $record;
        $notification_template = array(); // admin notifications
        $meta_data = json_decode($record['meta_data'] ?? '', true);
        if(!empty($meta_data)) {
            foreach($meta_data as $meta) {
                $record['label_' . $meta['name']] = !empty($meta['label_lang']) ? lang($meta['label']) : $meta['label'];
                $record['value_' . $meta['name']] = $meta['value'];
            }
        }
        unset($record['meta_data']);
        /*foreach($form_fields as $form_field) {
         if(array_key_exists($form_field['name'], $record)) {
         $record['label_' . $form_field['name']] = !empty($form_field['label_lang']) ? lang($form_field['label']) : $form_field['label'];
         $record['value_' . $form_field['name']] = $record[$form_field['name']];
         }
         }*/
        $row = $record;
        foreach($record as $key => $value) {
            if(!empty($value)) {
                switch($key) {
                    case 'group_ids':
                    case 'city_id':
                    case 'state_id':
                    case 'country_iso':
                    case 'status_color':
                        break;
                    case 'created_on':
                    case 'modified_on':
                        $record['value_' . $key] = gc_date_format($value);
                        break;
                    default:
                        $record['value_' . $key] = $value;
                }
            }
            unset($record[$key]);
        }
        $record['value_status'] = !empty($row['status_lang']) ? lang($row['status']) : $row['status'];
        $record['label_ip_address'] = lang('ip_address');
        $record['value_ip_address'] = !empty($record['value_created_ip']) ? $record['value_created_ip'] : lang('not_applicable');
        $record['label_location'] = lang('label_location');
        $record['value_location'] = !empty($record['geo_location']) ? $record['geo_location'] : lang('not_applicable');
        return $record;
    }
    
    /**
     * Calculate scheduled on based on different parameters
     *
     * Condition 1: Send Date of Birth message yearly at 11:00 AM
     *
     * field: dob (send on a specific field_value)
     * frequency: yearly (weekly, monthly, quarterly, halfyearly, yearly) - Not Required to simplify things
     * timing: 11:00:00 (24 hours format), if you want to send on a specific time, keep blank to get timing from the field value
     *
     * Send on a specific time, when the user added to the database
     *
     * field: created_on (get timing from the created_on field)
     *
     * weekdays: 1,2,3,4,5,6,7 // 1 is sunday and 7 is saturday
     * days: 1,5,10 (specific days within a month)
     * months: 1,2,3, and so on 12
     * years: 2022, 2023 and so on (comma separated value)
     * period: 1/2/3/4/5 years (Advance calculations for this much months/years)
     * schedule_date: send on a specific date. E.g. format 2021-11-19
     *
     * limit: number of rows which have the same scheduled date
     * interval: add specific days to scheduled_on to calculate new expiry date
     * Note: these two also requires processed_counts parameter to be passed in $extras
     * E.g. $this->subscribers_model->calculate_schedule_on($row, $message, array('processed_counts' => 2))
     *
     * thrice or twice within a day - use timing value in a comma separated format
     * send on alternate weekdays - use weekdays in a comma separated format
     * send on alternate days - use days in a comma separated format
     * send in alternate years - use years in a comma separated format
     *
     * send on alternate weekdays e.g. second saturdays only within a specific month
     * this can be achieved by creating a separate campaign
     *
     * after demo registration, send message on next day, 15 days or 20 days etc. schedule it on one click with email templates
     * avoid sending the duplicate email within a specific day
     * if a user unsubscribed, do not send the email, check this before sending
     *
     * template matching concept
     *
     * Similarly, do this for send messages
     *
     * process - set a schedule to calculate for each year
     * once, yearly, monthly, daily
     */
    public function calculate_schedule_on($subscriber, $message_info, $extras = array()) {
        $row = $subscriber;
        $params = gc_process_params($message_info);
        /* $row = new stdClass();
         $row->dob = '1983-12-29 05:55:00';
         $params = array(
         'field' => 'dob',
         // 'frequency' => 'weekly',
         // 'timings' => '11:00', // ,05:00
         'weekdays' => '1,3', // 1 is sunday, 7 is saturday
         'days' => '2,8',
         'months' => '1,2,3',
         'years' => '2023',
         'period' => 3
         );*/
        if(empty($params['period'])) {
            $period = 1;
        } else {
            $period = $params['period'];
        }
        $years = array();
        if(!empty($params['years'])) {
            $years = array_map('trim', explode(',', $params['years']));
        }
        $days = array();
        if(!empty($params['days'])) {
            $days = array_map('trim', explode(',', $params['days']));
        }
        $months = array();
        if(!empty($params['months'])) {
            $months = array_map('trim', explode(',', $params['months']));
        }
        $weekdays = array();
        if(!empty($params['weekdays'])) {
            $weekdays = array_map('trim', explode(',', $params['weekdays']));
        }
        
        // Note: timing is an array, the user can send email multiple times within a day
        $timings = array();
        if(!empty($params['timings'])) {
            $timings = array_map('trim', explode(',', $params['timings']));
        }
        
        $schedule_dates = array();
        $now = new \DateTime('now');
        $current_year = $now->format('Y');
        $current_month = $now->format('m');
        $current_day = $now->format('d');
        $todays_date = date('Y-m-d H:i:s');
        
        $year = '';
        $month = '';
        $day = '';
        $hours = '';
        $minutes = '';
        $seconds = '';
        $field_value = '';
        if(!empty($params['field']) && !empty($row->{$params['field']})) {
            $field_value = $row->{$params['field']};
            $year = date('Y', strtotime($field_value));
            $month = date('m', strtotime($field_value));
            $day = date('d', strtotime($field_value));
            if(empty($months) && empty($params['schedule_date'])) {
                $months[] = $month;
            }
            if(empty($days) && empty($params['schedule_date'])) {
                $days[] = $day;
            }
            $hours = date('H', strtotime($field_value));
            $minutes = date('i', strtotime($field_value));
            $seconds = date('s', strtotime($field_value));
            
            // get timing from the field value
            if(empty($params['timings'])) {
                $timings[] = $hours . ':' . $minutes . ':' . $seconds;
            }
        }
        if(empty($timings)) {
            $timings[] = '00:00:00'; // send at 12
        }
        
        // weekdays or days are required
        $current_year = $current_year - 1;
        for($y = 1; $y <= $period; $y++) {
            $calculated_year = $current_year + $y;
            if(empty($years) || in_array($calculated_year, $years)) { // send in specific year only
                for($i = 1; $i <= 12; $i++) { // run for each month
                    $calculated_month = append_zero($i);
                    $calculated_date = $calculated_year . '-' . $calculated_month . '-' . '01 00:00:00';
                    if($calculated_date >= $todays_date) {
                        // send in specific months only
                        if(empty($months) || in_array($calculated_month, $months)) {
                            $calculated_weekdays = $this->get_weekdays($calculated_month, $calculated_year);
                            // weekdays or days must be set
                            if(!empty($weekdays)) {
                                foreach($weekdays as $weekday) {
                                    if(array_key_exists($weekday, $calculated_weekdays)) {
                                        foreach($calculated_weekdays[$weekday] as $calculated_weekday) {
                                            $date_calculated = $calculated_year . '-' . $calculated_month . '-' . append_zero($calculated_weekday);
                                            foreach($timings as $timing) {
                                                $schedule_dates[] = $date_calculated . ' ' . $timing;
                                            }
                                        }
                                    }
                                }
                            }
                            // send on specific days
                            if(!empty($days)) {
                                foreach($days as $specific_day) {
                                    $date_calculated = $calculated_year . '-' . $calculated_month . '-' . append_zero($specific_day);
                                    foreach($timings as $timing) {
                                        $schedule_dates[] = $date_calculated . ' ' . $timing;
                                    }
                                }
                                
                            }
                        }
                    }
                }
            }
        }
        
        /**
         *
         * Schedule the email based on the timing of the created_on field.
         *
         * Parameters
         *
         * field: created_on
         * schedule_date: 2021-11-19
         * limit: 5
         * interval: 1 (Day)
         */
        if(!empty($params['schedule_date'])) {
            foreach($timings as $timing) {
                if($this->session->userdata('subscriber_scheduled_on')) {
                    $scheduled_on = explode(' ', $this->session->userdata('subscriber_scheduled_on'));
                    $scheduled_on = $scheduled_on[0] . ' ' . $timing;
                } else {
                    $scheduled_on = $params['schedule_date'] . ' ' . $timing;
                }
                $i = !empty($extras['processed_counts']) ? (int) $extras['processed_counts'] : 1;
                $i = $i > 1 ? $i - 1 : $i;
                if(!empty($params['limit']) && !empty($params['interval']) && ($i % $params['limit']) == 0) {
                    $scheduled_on = $this->next_schedule_date($scheduled_on, $params['interval']);
                    $schedule_dates[] = $scheduled_on;
                } else {
                    $schedule_dates[] = $scheduled_on;
                }
                $this->session->set_userdata('subscriber_scheduled_on', $scheduled_on);
            }
        }
        return $schedule_dates;
    }
    
    /**
     * Notify subscriber after a few days
     */
    public function notify_subscriber($days = 3) {
        $field = 'modified_on';
        $records = $this->get_subscribers($days);
        if(!records) {
            $field = 'created_on';
            $records = $this->get_subscribers($days, $field);
        }
        if($records) {
            $this->load->library('crypt');
            $this->load->library('parser');
            $this->lang->load('emailer/emailer');
            $email_links = $this->gc->get('email_links', array('result' => 1));
            $email_template_group_id = 6;
            $subscriber_emails_data = array();
            foreach($records as $record) {
                if(empty($record->order_id) && !empty($record->email)) {
                    // the email must not exist in the subscriber_emails table
                    $where = "email_template_group_id = " . $email_template_group_id . " AND email = '" . $record->email . "'"; // send email only once
                    $subscriber_emails_counts = $this->db->where($where)->count_all_results('subscriber_emails');
                    if($subscriber_emails_counts == 0) {
                        $time = explode(' ', $record->{$field} ?? '');
                        if(!empty($days)) {
                            $date = date('Y-m-d', strtotime($time[0]. ' + ' . $days . ' days'));
                            $datetime = $date . ' ' . $time[1];
                        } else {
                            $datetime = date('Y-m-d') . ' ' . $time[1];
                        }
                        $where = 'FIND_IN_SET(' . $email_template_group_id . ', group_ids) AND status = 1';
                        $email_template = $this->db->where($where)->order_by('RAND()')->get('email_templates')->row();
                        $subscriber_emails_data[] = array(
                            'subscriber_id' => $record->id,
                            'email_template_group_id' => $email_template_group_id,
                            'email_template_id' => $email_template ? $email_template->id : 0,
                            'email' => $record->email,
                            'notified_on' => $datetime,
                            'status' => 1
                        );
                        $this->email_send($record, $datetime, $email_template, $email_links);
                    }
                }
            }
            if(!empty($subscriber_emails_data)) {
                $this->db->insert_batch('subscriber_emails', $subscriber_emails_data);
            }
        }
    }
    
    /**
     * Insert email into the queue table
     */
    private function email_send($subscriber, $schedule_date, $email_template, $email_links) {
        $email_template_id = $email_template->id;
        $last_ip = $this->input->ip_address();
        $headers = array(
            'reply_email' => 'info@goclixy.com',
            'reply_name' => 'GoClixy'
        );
        $module = 'subscriber_users';
        // email is opened or not
        $module_id = gc_module_id($module);
        foreach($subscriber as $skey => $svalue) {
            $message_data['value_' . $skey] = $svalue;
        }
        // insert information in the email_sent table and get the sent id
        // email sent email will have a unique sent id to track it
        $email_sent_data = array(
            'module_id' => $module_id,
            'row_id' => $subscriber->id,
            'message_id' => 0,
            'last_ip' => $last_ip
        );
        $this->db->insert('email_sent', $email_sent_data);
        $sent_id = $this->db->insert_id();
        $encrypted_sent_id = $this->crypt->encode($sent_id);
        // default parameters
        $message_data['value_site_title'] = settings_item('site.title');;
        $message_data['value_site_email'] = settings_item('site.system_email');
        $message_data['value_site_url'] = anchor(site_url(), site_url());
        $message_data['value_site_address'] = settings_item('site.address');
        $message_data['value_site_phone'] = settings_item('site.phone_number');
        // message data
        $message_data['value_logo_url'] = '<img src="' . str_replace('..', '.', site_url('logo?read=' . $encrypted_sent_id)) . '" alt="' . settings_item('site.title') . '" />';
        
        // insert links
        if($email_links) {
            foreach($email_links as $email_link) {
                $message_data['value_link_only_' . $email_link->id] = site_url('visit?id=' . $encrypted_sent_id . '&link=' . $this->crypt->encode($email_link->id));
                $message_data['value_link_' . $email_link->id] = anchor(site_url('visit?id=' . $encrypted_sent_id . '&link=' . $this->crypt->encode($email_link->id)), $email_link->url);
            }
        }
        $subject = $this->parser->parse_string($email_template->subject, $message_data, true);
        $data = array(
            'user_id' => (int) $this->auth->user_id(),
            'template_id' => $email_template_id,
            'module_id' => $module_id,
            'row_id' => $subscriber->id,
            'sent_id' => $sent_id,
            'scheduled_on' => $schedule_date,
            'priority' => 2
        );
        $data['to_email'] = $subscriber->email;
        $data['subject'] = $subject;
        $data['headers'] = json_encode($headers);
        $data['switch_from'] = 1;
        $message = $this->parser->parse_string($email_template->message, $message_data, true);
        $data['message'] = gc_unsubscribe($this->crypt->encode($subscriber->id), $module, $message);
        $table_queue = 'email_queue';
        $this->db->insert($table_queue, $data);
    }
    
    /**
     * Get all subscribers which are registered recently
     * The order_id must be blank
     */
    private function get_subscribers($days = 3, $field = 'modified_on', $limit = 0) {
        $sql = "SELECT M.*, O.order_id FROM " . $this->db->dbprefix . "subscriber_users M";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "orders O ON O.user_email = M.email AND O.status_id = 2";
        $sql .= " WHERE M.status_id = 2";
        if(!empty($days)) {
            $between_date = date('Y-m-d H:i:s', strtotime(date('Y-m-d H:i:s'). ' - ' . $days . ' days'));
            $sql .= " AND M." . $field . " >= '" . $between_date . "'";
            if($field == 'created_on') {
                $sql .= " AND (M.modified_on IS NULL OR M.modified_on = '')";
            }
        }
        $sql .= " GROUP BY M.email";
        if(!empty($limit)) {
            $sql .= " LIMIT " . $limit;
        }
        //  BETWEEN '" . $between_date . "' AND '" . $today_date . "'";
        $result = $this->db->query($sql)->result();
        return $result;
    }

    /**
     * Get Weekdays of a specific month and year
     * @param string $month
     * @param string $year
     * @return string
     */
    private function get_weekdays($month, $year) {
        $days = array();
        // loop through month days
        for ($i = 1; $i <= 31; $i++) {
            
            // given month timestamp
            $timestamp = mktime(0, 0, 0, $month, $i, $year);
            
            // to be sure we have not gone to the next month
            if (date("n", $timestamp) == $month) {
                
                // current day in the loop
                $day = date("N", $timestamp);
                
                // if this is between 1 to 7, weekdays, 1 = Monday, 5 = Friday
                if ($day == 1 OR $day <= 7) {
                    
                    // write it down now
                    $days[$day][] = date("j", $timestamp);
                }
            }
        }
        return $days;
    }
}