<?php if (! defined ( 'BASEPATH' )) exit ( 'No direct script access allowed' );
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Dashboard controller
 */
use Khill\Lavacharts\DataTables\Cells\DateCell;
class Dashboard extends Admin_Controller {
	
	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct() {
		parent::__construct ();
		
		$this->auth->restrict ( 'Summary.Dashboard.View' );
		$this->load->model ( 'summary_model' );
		$this->lang->load('summary', $this->session->userdata('language'));
		
		// LOAD ALL MODELS FOR FILTERING AND OTHER OPERATIONS
		$this->load->model ( 'packages/packages_model' );
		$this->load->model ( 'listings/listings_model' );
		$this->load->model ( 'videos/videos_model' );
		$this->load->model ( 'listings/business_hours_model' );
		$this->load->model ( 'roles/role_model' );
		$this->load->model ( 'categories/categories_model' );
		$this->load->model ( 'locations/countries_model' );
		$this->load->model ( 'tags/tags_model' );
		
		Template::set_block ( 'sub_nav', 'dashboard/_sub_nav' );
		
		Assets::add_module_js ( 'summary', 'summary.js' );
	}
	
	/**
	 * Displays a list of module summary to the admin.
	 *
	 * @return void
	 */
	public function index() {
		// CHECK FOR UPDATES
	    $site_version = settings_item('site.version');
		$current_version = $this->gc->live_messages('revision');
	    if(!empty($current_version->revision) && $current_version->revision > $site_version) {
	        Template::set('update', array(
	            'software' => 'GoClixy',
	            'version' => $current_version->revision,
	            'link' => site_url(SITE_AREA . '/settings/cronjobs/update'),
	            'target' => '_self',
	            'helpurl' => ''));
	    } /*else {
    		$update = parse_ini_string($this->get_content(), TRUE);
    		if(!empty($update) && $update['version'] > $site_version) {
    			Template::set('update', $update);
    		}
	    }*/
		
		$lava = new Khill\Lavacharts\Lavacharts;
		
		$key_file_location = APPPATH .'third_party/Google/' .settings_item('seo.analytics_key_file'); //key.p12
		if(settings_item('seo.allow_analytics') == 1 && !empty(settings_item('seo.analytics_key_file')) && file_exists($key_file_location) && !empty(settings_item('seo.analytics_property_id'))) {
			$this->load->library('analytics');
			$analytics_data = $this->analytics->data();
			try {
				
				/*----------------------------------------------------------*/
				/* Display visitors chart to the admin from Google Analytics
				 * ----------------------------------------------------------*/
				$visitorsTable = $lava->DataTable();
				
				$visitorsTable->addStringColumn(lang('label_last_30_days'))
				->addNumberColumn(lang('label_visits'));
				foreach($analytics_data as $analytic_data) {
					$date = DateTime::createFromFormat ( 'Y-m-d', $analytic_data['date'] )->format ( 'M d' );
					$data = array($date, $analytic_data[1]);
					$visitorsTable->addRow($data);
				}
				$lineChart = $lava->LineChart('Visitors', $visitorsTable, [
						//'datatable' => $visitorsTable,
						'title' => lang('label_visitors'),
						'pointSize' => 4,
						'legend' => [ // HIDE LEGEND
								'position' => 'none'
						],
						//'curveType' => "function",
						'hAxis' => [
							'baselineColor' => '#fc32b0',
							'format' => 'M d', // IMPORTANT TO SET DATETIME FORMAT
							'gridlines' => array(
								'color' => '#43fc72',
								'count' => 6
							),
							'minorGridlines' => array(
								'color' => '#b3c8d1',
								'count' => 3
							),
							'textPosition' => 'out',
							'maxAlternation' => 6,
							'maxTextLines' => 2,
							'title' => lang('label_date'),
						],
						'vAxis' => [
							'baseline' => 5,
							'baselineColor' => '#CF3BBB',
							'format' => '##',
							'textPosition' => 'out',
							'title' => lang('label_visitors'),
						]
					]);
				Template::set('allow_analytics', settings_item('seo.allow_analytics'));
			}  catch (Exception $e) {
				Template::set_message(lang('msg_analytics'), 'error');
			}
		}
		
		$result = $this->summary_model->this_month_sales_graph ();
	
		/* -------------------------------------------------------------*/
		/* Generate Sales Graph
		/* -------------------------------------------------------------*/
		$salesTable = $lava->DataTable(); 
		
		// Set Date Format
		$dateFormat = $lava->DateFormat();
	    $dateFormat->pattern('Y-m-d');
	    
		$salesTable->addDateColumn(lang('label_sales'), $dateFormat)->addNumberColumn(lang('label_sales'));
		
		// Setting chart options!
		foreach ( $result as $daywise ) {
			$day_date = DateTime::createFromFormat ( 'Y-m-d', $daywise ['date'] )->format ( 'Y-m-d' );
			$data = array (
					$day_date,
					$daywise ['total_sales']
			);
			$salesTable->addRow ( $data );
		}
		$back_date = date('Y-m-d', strtotime("-30 days"));
		$hAxis = [
			'baselineColor' => '#fc32b0',
			'format' => 'Y-m-d', // IMPORTANT TO SET DATETIME FORMAT
			'gridlines' => array(
					'color' => '#43fc72',
					'count' => 6
			),
			'minorGridlines' => array(
					'color' => '#b3c8d1',
					'count' => 3
			),
			'textPosition' => 'out',
			'maxAlternation' => 6,
			'maxTextLines' => 2,
			'title' => lang('label_date'),
			'minValue' => (string) DateCell::parseString($back_date, 'Y-m-d'),
			'maxValue' => (string) DateCell::parseString(date('Y-m-d'), 'Y-m-d'),
		];
		if(!$result) {
			unset($hAxis['format']);
		} else {
			unset($hAxis['minValue']);
			unset($hAxis['maxValue']);
		}
		$lineChart = $lava->LineChart('Sales', $salesTable, [
				//'datatable' => $salesTable,
				'title' => lang ( 'label_this_month_sales' ) . ' ' . lang ( 'label_sales' ),
				'pointSize' => 4,
				'legend' => [ // HIDE LEGEND
						'position' => 'none'
				],
				//'curveType' => "function",
				'hAxis' => $hAxis,
				'vAxis' => [
						'baseline' => 5,
						'baselineColor' => '#CF3BBB',
				        'format' => settings_item('site.currency_symbol') . '##',
						'textPosition' => 'out',
						'title' => lang('label_amount'),
				]
		]);
		
		Template::set('lava', $lava);
		
		/**
		 * Package wise listings
		 */
		$packages = $this->packages_model->select ( 'id, title' )->find_all_by(array('deleted' => 0));
		$package_listings = array ();
		if ($packages) {
			foreach ( $packages as $package ) {
				$package_listings [] = array (
						'id' => $package->id,
						'title' => $package->title,
						'count' => $this->listings_model->count_by ( 'package_id', $package->id ) 
				);
			}
		}
		
		$summary = $this->summary_model->get_dashboard ();
		Template::set ('total_packages', count($packages));
		Template::set ( 'today_sales', $this->summary_model->today_sales () );
		Template::set ( 'yesterday_sales', $this->summary_model->yesterday_sales () );
		Template::set ( 'this_month_sales', $this->summary_model->this_month_sales () );
		Template::set ( 'all_time_sales', $this->summary_model->all_time_sales () );
		Template::set ( 'package_wise', $package_listings );
		Template::set ( 'summary', $summary );
		Template::set ( 'toolbar_title', lang('gc_menu_summary') );
		Template::render ();
	}
	
	/**
	 * Get latest version information from GoClixy Official website
	 * @param string $URL
	 * @return mixed
	 */
	private function get_content() {
		$URL = "https://www.goclixy.com/update";
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_URL, $URL);
		$data = curl_exec($ch);
		curl_close($ch);
		return $data;
	}
	
	/**
	 * Clear All Cache Items
	 */
	public function clear_cache () {
	    clear_cache(null, true);
	    $this->lang->load('logs/logs');
	    Template::set_message(lang('logs_purge_cache_success'), 'success');
	    Template::redirect(SITE_AREA .'/dashboard/summary');
	}
}