<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * content controller
 */
class Content extends Admin_Controller
{
	protected $permissionCreate = 'Tags.Content.Create';
	protected $permissionDelete = 'Tags.Content.Delete';
	protected $permissionEdit   = 'Tags.Content.Edit';
	protected $permissionView   = 'Tags.Content.View';
	
	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct()
	{
		parent::__construct();

		$this->auth->restrict($this->permissionView);
		$this->load->model('tags_model', null, true);
		$this->lang->load('datatable');
		$this->lang->load('tags');
		
		// DATATABLES IMPLEMENTATION
		/*Assets::add_js(Template::theme_url('js/bootstrap.js'));
		Assets::add_js($this->load->view('content/tags_js', null, true), 'inline');
		
		Assets::add_js( array ( Template::theme_url('js/jquery.dataTables.min.js' )) );
		Assets::add_js( array ( Template::theme_url('js/bootstrap-dataTables.js' )) );
		Assets::add_css( array ( Template::theme_url('css/datatable.css') ) ) ;
		Assets::add_css( array ( Template::theme_url('css/bootstrap-dataTables.css') ) ) ;*/
		
		Template::set_block('sub_nav', 'content/_sub_nav');
	}

	//--------------------------------------------------------------------


	/**
	 * Displays a list of form data.
	 *
	 * @return void
	 */
	public function index($filter='all', $offset=0, $id = NULL) {
		// Status Change
		if (isset($_POST['status_change'])) {
			$this->gc->status_change('tags', $_POST['status_change'], 'active', 99, false);
			$this->gc->update_tweak_fields('tags', $_POST['status_change']);
			//Template::redirect(current_url());
		}
		// Do we have any actions?
		if (isset($_POST['activate']))    $action = '_activate';
		if (isset($_POST['deactivate']))  $action = '_deactivate';
		
		if (isset($action)) {
			$checked = $this->input->post('checked');		
			if (!empty($checked)) {
				$new_status = ($action == '_activate' ? 1 : 0);
				foreach($checked as $tag_id) {
					// Get Existing Status
					$tag = $this->tags_model->find($tag_id);
					// Change status only if it is not already Active/Inactive
					if($tag->active != $new_status) {
						// Update Status
						$this->db->where('id', $tag_id)->update('tags', array('active' => $new_status));
						// Update Tweak Field (Keywords) in the Module Table
						$this->gc->update_tweak_fields('tags', $tag_id);
					}
				}
				Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), lang('tag_s'), lang('label_updated')), 'success');
			} else {
				Template::set_message(lang('ls_empty_id'), 'error');
			}
			Template::redirect(current_url());
		}

		$where = '';
		// Filters
		$filter_type = $filter;
		
		switch($filter_type) {
			case 'active':
				$where = "active = 1";
				break;
				
			case 'inactive':
				$where= "active = 0";
				break;
				
			case 'first_letter':
				$where = "SUBSTRING( LOWER(name), 1, 1)= $first_letter";
				break;
				
			case 'all':
			// Nothing to do
				break;
				
			default:
				show_404("tags/index/$filter/");
		}
		// Deleting anything?
		if (isset($_POST['delete'])) {
			$this->auth->restrict($this->permissionDelete);
			$checked = $this->input->post('checked');
			// To delete information from the Ads module
			$this->load->model('ads/ads_model');
			if (is_array($checked) && count($checked)) {
				$result = FALSE;
				foreach ($checked as $pid) {
					$result = $this->tags_model->purge_tag($pid);
				}

				if ($result) {
					Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), lang('tag_s'), lang('label_deleted')), 'success');
				} else {
					Template::set_message(sprintf(lang('msg_delete_failure'), lang('tag_s')) . $this->tags_model->error, 'danger');
				}
			}
		}
		
		// Search
		$searchterm = $this->gc->search('tags', $this->uri->segment(6), '/content/tags');
		if(!empty($searchterm)) {
			if(!empty($where)) {
				$where .= " AND";
			}
			$where .= " (id LIKE '%". $searchterm ."%' OR name LIKE '%". $searchterm ."%' OR created_on LIKE '%". $searchterm ."%')";
		}
		
		// Sort By
		if(!empty($where)) {
			$this->db->where($where);
		}
		$order_by = $this->gc->sort('tags');
		if(!empty($order_by)) {
			$this->tags_model->order_by($this->session->userdata('tags_sort')['column'], $this->session->userdata('tags_sort')['order']);
		} else {
			$this->tags_model->order_by('id', 'desc');
		}
		$this->tags_model->limit($this->limit, $offset);
		$records = $this->tags_model->find_all();		
		
		// Pagination
		$this->load->library('pagination');
		if(!empty($where)) {
			$this->db->where($where);
		}
		$total_tags = $this->tags_model->count_all();
		
				
		$pager['base_url'] = site_url(SITE_AREA ."/content/tags/index/$filter/");
		$pager['uri_segment']	= 6;
		$pager['total_rows'] = $total_tags;
		$pager['per_page'] = $this->limit;
		
		
		$this->pagination->initialize($pager);
		
		Template::set('index_url', site_url(SITE_AREA .'/content/tags/index/'));
		Template::set('filter_type', $filter_type);	

		Template::set('records', $records);
		Template::set('toolbar_title', lang('manage_tags'));
		Template::render();
	}
	
	/**
	 * Allow Admin to add a new tag
	 * 
	 * @return void
	 */
	public function add() {
		$this->auth->restrict($this->permissionCreate);		
		if (isset($_POST['save'])) {
			if ($insert_id = $this->save_tags()) {
				// Log the activity
				log_activity($this->current_user->id, lang('act_create_record') .': '. $insert_id .' : '. $this->input->ip_address(), 'tags');				
				Template::set_message(sprintf(lang('msg_save_success'), lang('label_tag')), 'success');
			}
		}
		/*if(!empty($_POST['modules'])) {	
			Template::set('relations', $_POST['modules']);
		}*/
		Template::set('modules', gc_modules());
		Template::set('toolbar_title', lang('add') . ' ' . lang('label_tag'));
		Template::set_view('content/tag_form');
		Template::render();
	}
	
	/**
	 * Allows editing of Tags data.
	 *
	 * @return void
	 */
	public function edit() {
		$id = $this->uri->segment(5);
		if (empty($id)) {
			Template::set_message(lang('invalid_id'), 'error');
			redirect(SITE_AREA .'/content/tags');
		}

		if (isset($_POST['save'])) {
			$this->auth->restrict($this->permissionEdit);

			if ($this->save_tags('update', $id)) {
				// Log the activity
				log_activity($this->auth->user_id(), lang('act_edit_record') .': '. $id .' : '. $this->input->ip_address(), 'tags');
				// Update Tweak Field (Keywords) in the Module Table
				$this->gc->update_tweak_fields('tags', $id);
				Template::set_message(sprintf(lang('msg_save_success'), lang('label_tag')), 'success');
				redirect(SITE_AREA . '/content/tags');
			} 
			
			// Not validation error
			if ( ! empty($this->tags_model->error)) {
				Template::set_message(sprintf(lang('msg_edit_failure'), lang('label_tag')) . $this->tags_model->error, 'danger');
			}
		} else if (isset($_POST['delete'])) {
			$this->auth->restrict($this->permissionDelete);
			$this->load->model('ads/ads_model'); // Purge Ad Relation
			if ($this->tags_model->purge_tag($id)) {
				log_activity($this->auth->user_id(), lang('act_delete_record') .': '. $id .' : '. $this->input->ip_address(), 'tags');
				Template::set_message(sprintf(lang('msg_action_success'), lang('label_tag'), lang('label_deleted')), 'success');
				redirect(SITE_AREA .'/content/tags');
			} else {
				Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_tag')) . $this->tags_model->error, 'danger');
			}
		}
		$record = $this->tags_model->find($id);
		
		// Multi Langauge Fields - Get translations if exists
		$record = $this->translations->get_translations('tags', $id, $record);
		
		$relations = $this->gc->get_relations($id, 'tags');
		Template::set('relations', $relations);
		Template::set('modules', gc_modules());
		Template::set('tag', $record);
		Template::set('toolbar_title', lang('edit') .' ' .lang('label_tag'));
		Template::set_view('content/tag_form');
		Template::render();
	}

	//--------------------------------------------------------------------
	// !PRIVATE METHODS
	//--------------------------------------------------------------------

	/**
	 * Summary
	 *
	 * @param String $type Either "insert" or "update"
	 * @param Int	 $id	The ID of the record to update, ignored on inserts
	 *
	 * @return Mixed    An INT id for successful inserts, TRUE for successful updates, else FALSE
	 */
	private function save_tags($type='insert', $id=0)
	{
		if ($type == 'update') {
			$_POST['id'] = $id;
		}
		// validate translation fields
		$this->translations->validate_prepare_fields();
		$this->form_validation->set_rules('name', 'lang:label_name', 'required|unique[tags.name,tags.id]|trim|alpha_extra|max_length[255]');
		$this->form_validation->set_rules('hits', 'lang:label_hits', 'trim|required|is_numeric');
		if($this->input->post('validate_module')) { // validate module is a hidden field in the tag_form
			$this->form_validation->set_rules('modules[]', 'lang:applicable_modules', 'required|is_array');
		}
		if ($this->form_validation->run() === false) {
			return false;
		}
		// unset non existing translation POST fields
		$this->translations->translate_unset();
		
		// make sure we only pass in the fields we want		
		$data = array();
		$data['name']        	= strtolower($this->input->post('name'));		
		$data['slug']			= $this->gc->slugify('tags', $data['name'], $id, 'name');	
		$data['hits']        	= intval($this->input->post('hits'));
		$data['active'] 		= $this->input->post('active') ? 1 : 0;
		// save selected modules info
		$selected_modules           = $this->input->post('modules');
		$modules_selected = array();
		if(!empty($selected_modules)) {
		    foreach($selected_modules as $selected_module => $selected_value) {
		        $modules_selected[] = gc_module_id($selected_module);
		    }
		}
		$data['module_relations']   = implode(',', $modules_selected);
		if ($type == 'insert') {
			$id = $this->tags_model->insert($data);
			if (is_numeric($id)) {
				$return = $id;
			} else {
				$return = FALSE;
			}
		} elseif ($type == 'update') {
			$return = $this->tags_model->update($id, $data);
		}
		
		// Save Tag and Module relationship
		// A Tag can belong to multiple modules
		if($return) {
			$this->gc->save_module_relations($id, $this->input->post('modules'), 'tags');
		}
		// save translations
		$this->translations->save_translations('tags', $id, $return);
		return $return;
	}
	
	/**
	 * Make tag active or inactive
	 * @param number $id
	 */
	public function change_status($id)
	{
		//Return old status
		$this->db->select ( 'active' );
		$this->db->where('id', $id);
		$query = $this->db->get ( $this->db->dbprefix . 'tags' );
	
		$value = 0; //declare variable to store returned result
	
		//Run loop and store integer value in variable
		foreach($query->result() as $row) {
			$value = $row->active;
		}
	
		//Compare values
		if($value == 0) {
			$data['active'] = 1;
		} else {
			$data['active'] = 0;
		}
		//Update status
		$this->db->where('id', $id);
		$this->db->update($this->db->dbprefix . 'tags', $data);
		Template::set_message(sprintf(lang('msg_action_success'), lang('label_tag'), lang('label_updated')), 'success');
		redirect(SITE_AREA .'/content/tags/');
	}
	
	//--------------------------------------------------------------------
	// ACTIVATION METHODS
	//--------------------------------------------------------------------
	/**
	 * Activates selected tag
	 */
	private function _activate($tag_id)
	{
		$this->tag_status($tag_id,1,0);
		Template::set_message(sprintf(lang('msg_action_success'), lang('tag_s'), lang('label_activated')));	
	}
	/**
	 * Deactivates selected tag.
	 */
	private function _deactivate($tag_id)
	{
		$this->tag_status($tag_id,0,0);
		Template::set_message(sprintf(lang('msg_action_success'), lang('tag_s'), lang('label_deactivated')));
	
	}
	
	/**
	 * For bulk update
	 * @param string $tag_id
	 * @param number $status
	 * @param number $supress_email
	 */
	private function tag_status($tag_id = false, $status = 1, $supress_email = 0)
	{
		$supress_email = (isset ( $supress_email ) && $supress_email == 1 ? true : false);
		
		if ($tag_id !== false && $tag_id != - 1) {
			$result = false;
			$type = '';
			if ($status == 1) {
				$result = $this->tags_model->tag_activation ( $tag_id );
				$type = lang ( 'gc_action_activate' );
			} else {
				$result = $this->tags_model->tag_deactivation ( $tag_id );
				$type = lang ( 'gc_action_deactivate' );
			}
			
			// $tag = $this->tags_model->find ( $tag_id );
			// log_activity($this->current_user->id, lang('us_log_status_change') . ': '.$log_name . ' : '.$type."ed", 'users');
		} else {
			Template::set_message ( lang ( 'ls_err_no_id' ), 'error' );
		}//end if
	}//end tag_status()
	
	/**
	 * Create slug for all tag
	 */
	public function update_slug($table_name = 'tags') {
		$this->load->library('gc');
		$this->db->or_where('slug', NULL);
		$this->db->or_where('slug', '');
		$records = $this->db->select('id, name')->get_where($this->db->dbprefix . $table_name)->result();
		if($records) {
			foreach($records as $record) {
				$data[] = array(
						'id' => $record->id,
						'slug' => $this->gc->slugify($table_name, $record->name, 0, 'name')
				);
			}
			$this->db->update_batch($table_name, $data, 'id');
		}
	}
}