<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Migration_Install_tags extends Migration
{
	/**
	 * The name of the database table
	 *
	 * @var String
	 */
	private $table_name = 'tags';
	private $location_table_name = 'tag_locations';

	/**
	 * The table's fields
	 *
	 * @var Array
	 */
	private $fields = array(
		'id' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'auto_increment' => TRUE,
		),
		'name' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => FALSE,
		),
		'slug' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => FALSE,
		),
		'count' => array(
			'type' => 'INT',
			'constraint' => 11,
			'default' => 0,
		),
		'hits' => array(
				'type' => 'INT',
				'constraint' => 11,
				'default' => 0,
		),
		'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'modified_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'active' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'default' => 0,
		),
	);
	
	/**
	 * Tag Location's fields
	 *
	 * @var Array
	 */
	private $location_fields = array(
		'id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'auto_increment' => TRUE,
		),
		'tag_id' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'null' => FALSE,
		),
		'city_id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => TRUE,
		),
		'city_count' => array(
			'type' => 'INT',
			'constraint' => 5,
			'null' => FALSE,
			'default'=> 0,
		),
		'state_id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => TRUE,
		),
		'state_count' => array(
			'type' => 'INT',
			'constraint' => 5,
			'null' => FALSE,
			'default'=> 0,
		),
		'country_iso' => array(
			'type' => 'CHAR',
			'constraint' => 2,
			'null' => TRUE,
		),
		'country_count' => array(
			'type' => 'INT',
			'constraint' => 5,
			'null' => FALSE,
			'default'=> 0,
		),
	);

	/**
	 * Install this migration
	 *
	 * @return void
	 */
	public function up()
	{
		$this->dbforge->add_field($this->fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->create_table($this->table_name);
		// CHANGE ENGINE FROM INNODB TO MYISAM
		$change_engine_sql = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " ENGINE = MyISAM";
		$this->db->query ($change_engine_sql);
		// ADD FULLTEXT INDEX
		$fulltext_sql = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " ADD FULLTEXT INDEX `name` (`name`)";
		$this->db->query ($fulltext_sql);
		
		// TAG LOCATIONS TABLE
		$this->dbforge->add_field($this->location_fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->create_table($this->location_table_name);
		// ADD UNIQUE INDEX
		$fulltext_sql = "ALTER TABLE " . $this->db->dbprefix . $this->location_table_name . " ADD UNIQUE INDEX `unique_index` (`tag_id`, `city_id`)";
		$this->db->query ($fulltext_sql);
	}

	//--------------------------------------------------------------------

	/**
	 * Uninstall this migration
	 *
	 * @return void
	 */
	public function down()
	{
		$this->dbforge->drop_table($this->table_name);
		$this->dbforge->drop_table($this->location_table_name);
	}

	//--------------------------------------------------------------------

}