<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * A tag information is stored in three tables
 * 
 * tags - Assign a new id to each tag and save its name
 * 
 * module_relations - A tag can belong to multiple modules
 * e.g. the tag "business directory script" can be used in both listings and products 
 * The module_relations table stores the relationship between a tag and modules
 * e.g. Tag ID 1 belongs to Listings and Products
 * module_id				row_id		toggle_module_id
 * gc_module_id('tags')		1			gc_module_id('listings')
 * gc_module_id('tags')		1			gc_module_id('products')
 * 
 * tag_relations - This table stores the relationship between a tag and a specific row of a module
 * e.g. Tag ID 1 used in the Listing ID 20
 * module_id				row_id		tag_id
 * gc_module_id('listings')	20			1
 * 
 * @author GoClixy
 *
 */
class Tags_model extends BF_Model {

	protected $table_name	= "tags";
	protected $tag_relations = "tag_relations";
	protected $key			= "id";
	protected $soft_deletes	= false;
	protected $date_format	= "datetime";

	protected $log_user 	= FALSE;

	protected $set_created	= true;
	protected $set_modified = true;
	protected $created_field = "created_on";
	protected $modified_field = "modified_on";

	/*
		Customize the operations of the model without recreating the insert, update,
		etc methods by adding the method names to act as callbacks here.
	 */
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 		= array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	/*
		For performance reasons, you may require your model to NOT return the
		id of the last inserted row as it is a bit of a slow method. This is
		primarily helpful when running big loops over data.
	 */
	protected $return_insert_id 	= TRUE;

	// The default type of element data is returned as.
	protected $return_type 			= "object";

	// Items that are always removed from data arrays prior to
	// any inserts or updates.
	protected $protected_attributes = array();

	/*
		You may need to move certain rules (like required) into the
		$insert_validation_rules array and out of the standard validation array.
		That way it is only required during inserts, not updates which may only
		be updating a portion of the data.
	 */
	protected $validation_rules 		= array();
	protected $insert_validation_rules 	= array();
	protected $skip_validation 			= FALSE;

	//--------------------------------------------------------------------
	
	/**
	 * Count Inactive tags.
	 *
	 * @access public
	 *
	 * @return int Inactive tag count.
	 */
	public function count_inactive_tags()
	{
		$this->db->where('active',-1);
		return $this->count_all(FALSE);
	
	}//end count_inactive_tags()
	
	
	/**
	 * Activate Business tag
	 * @access public
	 *
	 * @param int    $tag_id        The tag
	 *
	 * @return int tag Id on success, FALSE on error
	 */
	public function activate($tag_id)
	{
		$this->db->update($this->table_name, array('active'=>1), array('id' => $tag_id));
	
		if ($this->db->affected_rows() != 1)
		{
			return FALSE;
		}
	
		return TRUE;
	
	}//end activate()
	
	
	/**
	 * This function is triggered during tag set up to assure tag is not active.
	 * This function can be used to deactivate tags based on public view events.
	 *
	 * @param int    $tag_id    The tag or email to match to deactivate
	 *
	 * @return mixed $activate_hash on success, FALSE on error
	 */
	public function deactivate($tag_id)
	{
		$this->db->update($this->table_name, array('active'=>0), array('id' => $tag_id));
	
		if ($this->db->affected_rows() != 1)
		{
			return FALSE;
		}
	
		return TRUE;
	
	}//end deactivate()
	
	
	/**
	 * Admin specific activation function for admin approvals or re-activation.
	 *
	 * @access public
	 *
	 * @param int $tag_id The tag ID to activate
	 *
	 * @return bool TRUE on success, FALSE on error
	 */
	public function tag_activation($tag_id = FALSE)
	{
		if ($tag_id === FALSE)
		{
			$this->error = lang('ls_err_no_id');
			return FALSE;
		}
	
		if ($this->activate($tag_id, 'id', FALSE))
		{
			return $tag_id;
		}
		else
		{
			$this->error = lang('ls_err_tag_is_active');
			return FALSE;
		}
	
	}//end tag_activation()
	
	
	/**
	 * Admin only deactivation function.
	 *
	 * @access public
	 *
	 * @param int $tag_id The tag ID to deactivate
	 *
	 * @return bool TRUE on success, FALSE on error
	 */
	public function tag_deactivation($tag_id = FALSE)
	{
		if ($tag_id === FALSE)
		{
			$this->error = lang('ls_err_no_id');
			return FALSE;
		}
	
		if ($this->deactivate($tag_id, 'id', FALSE))
		{
			return $tag_id;
		}
		else
		{
			$this->error = lang('ls_err_tag_is_inactive');
			return FALSE;
		}
	
	}//end admin_deactivation()
	
	/**
	 * Get module tags from the tag_relations association table
	 * @param string $module
	 * @param number $row_id
	 */
	public function get_tags($module, $row_id = 0, $active_only = FALSE) {
		$module_id = gc_module_id($module);
		$query = $this->translations->build_query('tags', array('id', 'name'));
		$sql = $query['select'];
		$sql .= " FROM " . $this->db->dbprefix . "tags M";
		if(!empty($query['join'])) {
		    $sql .= $query['join'];
		}
		$sql .= " INNER JOIN " . $this->db->dbprefix . "tag_relations TR ON TR.tag_id = M.id AND TR.module_id = " . $module_id . " AND TR.row_id = " . $row_id;
		if($active_only) {
		    $sql .= " WHERE M.active = 1";
		}
		$sql .= " ORDER BY M.id";
		return $this->db->query($sql)->result();
	}
	
	/**
	 * Get tag ids from the tag_relations association table
	 * @param string $module
	 * @param number $row_id
	 */
	public function get_tag_ids($module, $row_id = 0) {		
		$module_id = gc_module_id($module);
		$tag_ids = $this->db->select('tag_id')->group_by('tag_id')->get_where($this->tag_relations, array('module_id' => $module_id, 'row_id' => $row_id))->result();
		return $tag_ids;
	}
	
	/**
	 * Get Tag information
	 * @param string $string
	 */
	public function find_tags($string) {
		$where = 'name IN (' . $string . ')';
		return $this->tags_model->select('id, name')->find_all_by($where);
	}
	
	/**
	 * Add association information
	 * @param array $tags
	 */
	public function insert_tags($tags) {
		$this->db->insert_batch($this->tag_relations, $tags);
	}
	
	/**
	 * Delete association from the tag_relations table on row delete
	 * @param string $module_id
	 * @param string $row_id
	 */
	public function delete_tags($module, $row_id) {
		$module_id = gc_module_id($module);
		return $this->db->delete_where($this->tag_relations, array('module_id' => $module_id,'row_id' => $row_id));
	}
	
	/**
	 * Function to decrement tag count from tag_locations association table on insert and update
	 * @param string $existing_tags_id_string
	 * @param number $selected_city_id
	 */
	public function decrement_tag_locations_count ($tag_ids, $selected_city_id) {
		foreach($tag_ids as $tag_id) {
			$count = $this->db->where(array('tag_id' => $tag_id, 'city_id' => $selected_city_id))->count_all_results('tag_locations');
			if($count > 0) {
				$this->db->set('city_count', 'city_count - 1', FALSE);
				$this->db->set('state_count', 'state_count - 1', FALSE);
				$this->db->set('country_count', 'country_count -1', FALSE);
				$this->db->where(array('tag_id' => $tag_id, 'city_id' => $selected_city_id))->update('tag_locations');
			}
		}		
	}
	
	/**
	 * Add information in tag_locations association table
	 * Single function to handle insert and update on duplicate
	 * @param string $data
	 */
	public function insert_update_tag_locations ($tag_ids, $city_id, $state_id, $country_iso) {
		foreach($tag_ids as $tag_id) {
			$count = $this->db->where(array('tag_id' => $tag_id, 'city_id' => $city_id))->count_all_results('tag_locations');
			if($count > 0) {
				$this->db->set('city_count', 'city_count + 1', FALSE);
				$this->db->set('state_count', 'state_count + 1', FALSE);
				$this->db->set('country_count', 'country_count + 1', FALSE);
				$this->db->where(array('tag_id' => $tag_id, 'city_id' => $city_id))->update('tag_locations');
			} else {
				$this->db->insert('tag_locations', array('tag_id' => $tag_id, 'city_id' => $city_id, 'state_id' => $state_id, 'country_iso' => $country_iso));
			}
		}	
	}	
	
	/**
	 * Tags Search - Post Your Requirements
	 */
	public function ajax_tags_search($query) {
		$sql = "SELECT T.id, T.name FROM " . $this->db->dbprefix . "tags T";
		$sql .= " WHERE T.name LIKE '%". $query ."%' AND T.active = 1";
		$sql .= " ORDER BY T.name ASC LIMIT 10";
		return $this->db->query($sql)->result();
	}
	
	/**
	 * Handle tags or keywords information
	 * @param string $type: 'insert_tags'
	 * @param int $listing_id
	 * @return null
	 */
	public function deal_with_tags($type, $listing_id) {
		$this->load->model('tags/tags_model');
		if ($this->input->post ('keywords')) {
			$selected_country_iso = $this->input->post('country_iso');
			$selected_state_id = $this->input->post('state_id');
			$selected_city_id = $this->input->post('city_id');
	
			$data = array ();
	
			$user_entered_tags = array (); // STORE USER ENTERED TAGS
			$new_tags = array (); // STORE NEW_TAGS
			$all_tags = array (); // STORE ID OF ALL TAGS
			$existing_tags_id = array(); // FOR COUNT
			$existing_tags_id_string = ''; // FOR MULTIPLE COUNT UPDATE
			$tags = ''; //SET TAGS VALUE TO NULL IF NO ASSOCIATION EXIST
	
			// GET KEYWORDS FROM TAGS TABLE AND STORE THEM IN AN ARRAY
			$keywords = array ();
			$result = $this->get_tags ('listings', $listing_id );
			if ($result) {
				foreach ( $result as $keyword ) {
					$keywords [] = $keyword->name;
					$existing_tags_id [] = $keyword->id;
					$tag_locations_changed_array[] = '(' .$keyword->id. ', ' .$selected_city_id. ', 0, ' .$selected_state_id. ', 0, \'' .$selected_country_iso .'\', 0)';
				}
				$tags = implode ( ', ', $keywords ); // STORE THEM AS A STRING IN COMMA SEPARATED VERSION
				$existing_tags_id_string = implode ( ', ', $existing_tags_id );
			}
	
			// CHECK WHETHER USER UPDATED TAGS OR NOT, IF UPDATED
			if ($tags != $this->input->post ('keywords')) {
				if($result) {
					// DECREMENT EXISTING TAGS COUNT AS THEY WILL BE UPDATED LATER
					$this->db->where('id IN ('.$existing_tags_id_string.')');
					$this->db->set('count', 'count - 1', FALSE);
					$this->db->update('tags');
	
					// DECREMENT TAG COUNT IN TAG_LOCATION TABLE
					$this->decrement_tag_locations_count($existing_tags_id_string, $selected_city_id);
				}
	
				// TO REMOVE EXTRA SPACES
				$patterns = array("/\s+/", "/\s([?.!])/");
				$replacer = array(" ","$1");
	
				// TAKE VALUES FROM POST AND SEPARATE THEM
				$ids = explode ( ',', $this->input->post ( 'keywords' ) );
				foreach ( $ids as $id ) {
					$user_entered_tags [] = mysqli_real_escape_string($this->get_mysqli(), strtolower(trim(preg_replace( $patterns, $replacer, escape($id))))); //REMOVE WHITE SPACES BEFORE AND AFTER COMMAS
				}
					
				// CREATE STRING OF USER ENTERED TAGS
				$user_entered_tags_array = implode(",", $user_entered_tags);
					
				// ADD QUOTE TO EACH KEYWORD
				$string = "'" . str_replace ( ",", "','", $user_entered_tags_array) . "'";
					
				// REMOVE EXTRA SPACES WITH IN EACH TAG (e.g. PHP  TRAINING)
				$str = preg_replace( $patterns, $replacer, $string);
					
				// TAKE ALL KEYWORDS AND COMPARE THEM WITH DATABASE
				$result = $this->find_tags($str);
					
				// STORE EXISTING TAGS IN AN ARRAY FOR COMPARING
				$mysql_exist_rows = array ();
				if($result) {
					foreach ( $result as $keyword ) {
						$mysql_exist_rows [] = $keyword->name;
					}
				}
	
				// COMPARE DATABASE TAGS AND USER ENTERED TAGS
				$new_tags = array_diff ($user_entered_tags, $mysql_exist_rows ); // NEW TAGS
					
				// INSERT NEW TAGS IN DATABASE
				if ($new_tags) {
					foreach ( $new_tags as $tag ) {
						$data_tags['name'] = mysqli_real_escape_string($this->get_mysqli(), trim(strtolower($tag)));
						$data_tags['slug'] = $this->slugify('tags', $data_tags['name'], 0, 'name', false);
						$data_tags['count'] = -1; // SET COUNT TO -1 AS THEY ARE NEW ONE
						$insert_query = $this->db->insert_string($this->db->dbprefix . 'tags', $data_tags);
						$insert_query = str_replace('INSERT INTO', 'INSERT IGNORE INTO', $insert_query);
						$this->db->query($insert_query);
					}
				}
					
				// FIND TAG_ID OF ALL USER ENTERED TAGS
				$tag_ids = $this->tags_model->find_tags ( $string );
				foreach ( $tag_ids as $tag_id ) {
					$all_tags [] = $tag_id->id;
				}
					
				if ($all_tags) {
					$tags_to = implode ( ',', $all_tags );
					// INSERT TAGS IN ASSOCIATION TABLE
					$listing_tags = array ();
					$tag_locations_data_array = array(); // TO HANDLE TAG LOCATIONS
					foreach ( $all_tags as $associated_tags ) {
						$listing_tags [] = array (
								'module_id' => gc_module_id('listings'),
								'row_id' => $listing_id,
								'tag_id' => $associated_tags
						);
						$tag_locations_data_array[] = '(' .$associated_tags. ', ' .$selected_city_id. ', 0, ' .$selected_state_id. ', 0, \'' .$selected_country_iso .'\', 0)';
					}
	
					// TAG_LOCATIONS INFORMATION
					$tag_locations_data_string = implode(',', $tag_locations_data_array );
	
	
					if ($type == 'insert_listing_tags') {
						$this->tags_model->insert_tags ( $listing_tags );
						// INCREMENT KEYWORDS COUNT BY 1
						$this->db->where('id IN ('.$tags_to.')');
						$this->db->set('count', 'count + 1', FALSE);
						$this->db->update($this->db->dbprefix . 'tags');
	
						// UPDATE TAG LOCATIONS
						$this->tags_model->insert_update_tag_locations ($tag_locations_data_string);
					}
					elseif ($type == 'update_listing_tags') {
						// DELETE ALL EXISTING TAGS AND INSERT THEM
						$this->tags_model->delete_tags ('listings', $listing_id );
						$this->tags_model->insert_tags ( $listing_tags );
	
						// INCREMENT KEYWORDS COUNT BY 1
						$this->db->where('id IN ('.$tags_to.')');
						$this->db->set('count', 'count + 1', FALSE);
						$this->db->update($this->db->dbprefix . 'tags');
	
						// UPDATE TAG LOCATIONS
						$this->tags_model->insert_update_tag_locations ($tag_locations_data_string);
					}
				}
			}	else if (($tags == $this->input->post ('keywords')) && ($selected_city_id != $this->session->userdata('previous_city'))) {
				// TAG_LOCATIONS INFORMATION
				$tag_locations_changed_string = implode(',', $tag_locations_changed_array );
				$this->tags_model->insert_update_tag_locations ($tag_locations_changed_string);
			}
		}
		else {
			//USER SELECTED TO DELETE ALL EXISTING TAGS
			if ($type == 'update_listing_tags') {
				// DELETE ALL EXISTING TAGS
				$result = $this->tags_model->get_tags ('listings', $listing_id );
				if ($result) {
					foreach ( $result as $keyword ) {
						$existing_tags_id [] = $keyword->id;
					}
					$existing_tags_id_string = implode ( ', ', $existing_tags_id );
					// DECREMENT EXISTING TAGS COUNT AS THEY WILL BE UPDATED LATER
					$this->db->where('id IN ('.$existing_tags_id_string.')');
					$this->db->set('count', 'count - 1', FALSE);
					$this->db->update($this->db->dbprefix . 'tags');
	
					// DECREMENT TAG COUNT IN TAG_LOCATION TABLE
					$this->tags_model->decrement_tag_locations_count($existing_tags_id_string, $this->input->post('city_id'));
				}
				return $this->tags_model->delete_tags ('listings', $listing_id );
			}
		}
	}
	
	/**
	 * Save Tags information in the tag_relations table
	 * @param string $tags
	 * @param int $row_id
	 * @param string $module
	 * @param integer $status - Status of the newly added keyword - can be used while Bulk Import
	 */
	public function save_module_tags($tags, $row_id, $module, $status = 0) {		
		// get existing tags to compare them with the post submitted value		
		$saved_tags = $this->tags_comma_separated($module, $row_id);
		// skip below given code if nothing changed
		if($saved_tags && $saved_tags == $tags) {
			return false;
		}
		$module_id = gc_module_id($module);
		$tag_ids = array();
		$tag_location_ids = array(); // to hold all tag ids for tag_locations table update
		$selected_city_id = 0;
		$selected_state_id = 0;
		$selected_country_iso = 'US';
		
		// get existing tags id to update count column
		$existing_tags = $this->get_tag_ids($module, $row_id);
		if($existing_tags) {
			foreach($existing_tags as $existing_tag) {
				$tag_ids[] = $existing_tag->tag_id;
			}
		}
		
		// update the tag_locations table
		if($module == 'listings') {
			$selected_country_iso = $this->input->post('country_iso');
			$selected_state_id = $this->input->post('state_id');
			$selected_city_id = $this->input->post('city_id');
			// decrement tag counts in the tag_locations table
			if(!empty($tag_ids)) {
				$this->decrement_tag_locations_count($tag_ids, $selected_city_id);
			}
		}
		
		if(!empty($tags)) {
			$this->load->library('slug');			
			$data = array();
			$tag_relations = array();
			$tag_locations_data = array(); // TO HANDLE TAG LOCATIONS

			// convert string to an array and trim the whitespace
			$tags = array_map('trim', explode(',', $tags ?? ''));
			
			// save only unique keywords
			$tags = array_unique($tags);
			foreach($tags as $tag) {
				if(!empty($tag)) {
					$tag = trim($tag);
					$tag = $this->form_validation->tag_handling($tag); // Remove unwanted characters from a Tag
					// check whether the tag already exists in the Tag table
					$keyword = $this->tags_model->find_by('name', $tag);
					// Get Keyword info if an existing tag, if a new tag, save the info first and then retrieve
					$result = $this->get_keyword_info($keyword, $tag, $row_id, $module_id, $status);
					if(!empty($result['data'])) {
						$data[] = $result['data'];
						$tag_relations[] = $result['tag_relations'];
						$tag_location_ids[] = $result['tag_relation_id'];
					}
				}
			}
			if(!empty($data)) {
				// delete existing tags in case user removed one
				$this->db->where(array('row_id' => $row_id, 'module_id' => $module_id))->delete($this->tag_relations);
				// save tags information
				$this->db->insert_batch($this->tag_relations, $data);
				// save tags relation
				// A tag can belong to multiple modules like Listings, Videos, Modules etc.
				$this->db->on_duplicate('module_relations', $tag_relations);
				// update tag locations count
				// counting issue, query is run based on the tag id instead of row_id
				// enable below given code if tag_locations is required in the future
				/*if($module == 'listings') {									
					$this->tags_model->insert_update_tag_locations($tag_location_ids, $selected_city_id, $selected_state_id, $selected_country_iso);
				}*/
			}
		} else {
			// delete existing tags in case user removed all
			$this->db->where(array('row_id' => $row_id, 'module_id' => $module_id))->delete($this->tag_relations);
		}
		$this->_tweak_tags($module, $row_id);		
	}
	
	/**
	 * Update Module Keywords
	 * @param string $module
	 * @param integer $row_id
	 * @param boolean $is_update - TRUE - Update instantly, FALSE - Return value for Bulk Tweaking of Multiple Rows
	 */
	public function _tweak_tags($module, $row_id, $active_only = true, $is_update = true) {
		// update tags in the module keywords column
		$keywords = $this->tags_comma_separated($module, $row_id, $active_only, true);
		if($keywords) {
		    $data = array('keywords' => $keywords['keywords'], 'tags_set' => $keywords['tags_set'], 'tag_ids' => $keywords['tag_ids']);
			if($is_update) {
				$this->db->where('id', $row_id)->update($module, $data);
			} else {
				return $data;
			}
		} else if(!empty($row_id)) {
		    $data = array('keywords' => null, 'tags_set' => null, 'tag_ids' => null);
		    $this->db->where('id', $row_id)->update($module, $data);
		}
	}
	
	/**
	 * Tags in comma separated format
	 * Edit Module - Display both active and inactive tags
	 * @param string $module
	 * @param number $row_id
	 * @param number $active_only - Return Active Tags Only
	 * @pram boolean $return_tag_ids - Return Tag IDs for search tweak
	 * @return string|boolean
	 */
	public function tags_comma_separated($module, $row_id, $active_only = FALSE, $return_tag_ids = FALSE) {
		$saved_tags = $this->get_tags ($module, $row_id, $active_only);
		$keywords = array();
		$tag_ids = array();
		$tags_set = array();
		if($saved_tags) {
			foreach ( $saved_tags as $keyword ) {
				$keywords[] = $keyword->name;
				$tags_set[] = $keyword->id;
				$tag_ids[] = 'id' . $keyword->id . 't'; // id and t is the separator for search operations
				// Note: the string length must be greater than 4 for FULL TEXT search
			}
			// store them as a string in comma separated version
			$keywords = implode( ', ', $keywords);
			if($return_tag_ids) {
			    return array('tags_set' => implode(',', $tags_set), 'tag_ids' => implode(',', $tag_ids), 'keywords' => $keywords);
			} else {
			    return $keywords;
			}
		}
		return false;
	}
	
	/**
	 * Purge Tag Relation Information
	 * @param string $module
	 * @param number $row_id
	 */
	public function purge_tag_relation($module, $row_id) {
		$module_id = gc_module_id($module);
		$this->db->where(array('module_id' => $module_id,'row_id' => $row_id))->delete($this->tag_relations);
	}
	
	/**
	 * Purge Tag
	 * @param int $id
	 * @return boolean
	 */
	public function purge_tag($id) {
		// Update Tweak Field (Keywords) in the Module Table
		$this->gc->update_tweak_fields('tags', $id, null, 'delete');
		// Purge all relations of this tag from module_relations table
		$this->gc->purge_relation($id, 'tags');
		// Purge relationship from the tag_relations table, also.
		$this->db->where('tag_id', $id)->delete('tag_relations');
		// Purge any tags locations, also.
		$this->db->where('tag_id', $id)->delete('tag_locations');
		// Purge information from the Ads module
		$this->ads_model->purge_from_ad($id, 'tags');		
		// Purge Translations
		$this->translations->purge_tanslations('tags', $id);
		
		// Purge Tag
		$result = $this->tags_model->delete($id);
		return $result;
	}
	
	/**
	 * Save Tags Relation
	 * When Selectize input is used to input tags
	 * Input Format: 1,4,5
	 * 
	 * Use this method to restrict a user to select only existing tags activated by the dmin.
	 * 
	 * @param string $module
	 * @param integer $row_id
	 * @param string $tags
	 */
	public function selectize_tags_existing_only($module, $row_id, $tags) {
		// get existing tags to compare them with the post submitted value
		$saved_tags = $this->tags_comma_separated($module, $row_id);
		// skip below given code if nothing changed
		if($saved_tags && $saved_tags == $tags) {
			return false;
		}
		$module_id = gc_module_id($module);
		
		$tag_ids = array();
		$tag_location_ids = array(); // to hold all tag ids for tag_locations table update
		
		// get existing tags id to update count column
		$existing_tags = $this->get_tag_ids($module, $row_id);
		if($existing_tags) {
			foreach($existing_tags as $existing_tag) {
				$tag_ids[] = $existing_tag->tag_id;
			}
		}
		// delete existing tags in case user removed one
		$this->db->where(array('row_id' => $row_id, 'module_id' => $module_id))->delete($this->tag_relations);
		// save tags information
		$data = array();
		if(!empty($tags)) {
			// Convert string into array
			$tags = explode(',', $tags);
			foreach($tags as $tag) {
				$data[] = array(
					'row_id' => $row_id,
					'module_id' => $module_id,
					'tag_id' => $tag
				);
				// update count
				if(!in_array($tag, $tag_ids)) {
					$this->db->query("UPDATE " . $this->db->dbprefix . $this->table_name . " SET count = count + 1 WHERE id = " . $tag);
				}
			}
			$this->db->insert_batch($this->tag_relations, $data);
			// update tags in the module keywords column
			$this->_tweak_tags($module, $row_id);
		}
	}
	
	/**
	 * Save Tags Relation
	 * When Selectize input is used to input tags
	 * Input Format: 1,4,5
	 *
	 * Use this method to allow a user to select existing tags activated by the Admin and also allow them to create a new one.
	 *
	 * @param string $module
	 * @param integer $row_id
	 * @param string $tags
	 */
	public function selectize_tags($module, $row_id, $tags, $status = 0) {
		$this->load->library('slug');
		// get existing tags to compare them with the post submitted value
		$saved_tags = $this->tags_comma_separated($module, $row_id);
		// skip below given code if nothing changed
		if($saved_tags && $saved_tags == $tags) {
			return false;
		}
		$module_id = gc_module_id($module);
		$tag_ids = array();
		$tag_location_ids = array(); // to hold all tag ids for tag_locations table update
	
		// get existing tags id to update count column
		$existing_tags = $this->get_tag_ids($module, $row_id);
		if($existing_tags) {
			foreach($existing_tags as $existing_tag) {
				$tag_ids[] = $existing_tag->tag_id;
			}
		}
		
		// save tags information
		$data = array();
		if(!empty($tags)) {
			// convert string to an array and trim the whitespace
			$tags = array_map('trim', explode(',', $tags));
			// save only unique keywords
			$tags = array_unique($tags);
			foreach($tags as $tag) {
				$tag = trim($tag);
				// check whether the tag already exists in the Tag table
				if(is_numeric($tag)) { // The user selected the existing tag or typed a numeric value
					$keyword = $this->tags_model->find($tag);
					if(!$keyword) {
						$keyword = $this->tags_model->find_by(array('name' => $tag));
					}
				} else { // This is a new tag added by the user
					$tag = $this->form_validation->tag_handling($tag); // Remove unwanted characters from a Tag
					$keyword = $this->tags_model->find_by('name', $tag);
				}
				// Get Keyword info if an existing tag, if a new tag, save the info first and then retrieve
				$result = $this->get_keyword_info($keyword, $tag, $row_id, $module_id, $status);
				if(!empty($result['data'])) {
				    $data[] = $result['data'];
				}
				if(!empty($result['tag_relations'])) {
				    $tag_relations[] = $result['tag_relations'];
				}
				$tag_id = $result['tag_relation_id'];
				// update count
				if(!in_array($tag_id, $tag_ids)) {
					$this->db->query("UPDATE " . $this->db->dbprefix . $this->table_name . " SET count = count + 1 WHERE id = " . $tag_id);
				}
			}
			if(!empty($data)) {
				// delete existing tags in case user removed one
				$this->db->where(array('row_id' => $row_id, 'module_id' => $module_id))->delete($this->tag_relations);
				// save tags information
				$this->db->insert_batch($this->tag_relations, $data);
				// save tags relation
				// A tag can belong to multiple modules like Listings, Videos, Modules etc.
				$this->db->on_duplicate('module_relations', $tag_relations);
			}
		} else { 
			// The user may have unselected all tags
			// delete existing tags in case user removed all
			$this->db->where(array('row_id' => $row_id, 'module_id' => $module_id))->delete($this->tag_relations);
		}
		// Reset SELECT parameters
		$this->db->reset_query();
		// update tags in the module keywords column
		$this->_tweak_tags($module, $row_id);
	}
	
	/**
	 * Get Keyword Information
	 * @param array $keyword
	 * @param integer/string $tag
	 * @param integer $row_id
	 * @param integer $module_id
	 * @param integer $status - Status of the newly added keyword, can be used in bulk import
	 */
	private function get_keyword_info($keyword, $tag, $row_id, $module_id, $status = 0) {
		$data = array();
		$tag_relations = array();
		$tag_relation_id = 0;
		if($keyword) {
			$data = array(
					'row_id' => $row_id,
					'module_id' => $module_id,
					'tag_id' => $keyword->id
			);
			$tag_relations = array(
					'row_id' => $keyword->id,
					'module_id' => gc_module_id('tags'),
					'toggle_module_id' => $module_id
			);
			$module_relations = explode(',', $keyword->module_relations ?? '');
			$module_relations[] = $module_id;
			$module_relations = array_unique($module_relations);
			// update count
			$count = (!empty($tag_ids) && in_array($keyword->id, $tag_ids) ? $keyword->count : $keyword->count + 1);
			$this->db->where('id', $keyword->id)->update($this->table_name, array('count' => $count, 'module_relations' => implode(',', $module_relations)));
				
			// if the module is listings, update the tag_locations data
			// counts are set to one because they have no significance
			// if the row is duplicate they will be increased by one automatically by on_duplicate update query
			$tag_relation_id = $keyword->id;
		} else if(!empty($tag)) {
			// insert tag in the tags table
			$tag_data = array();
			$tag_data['name'] = $tag;
			$tag_data['slug'] = $this->slug->create_slug($tag);
			$tag_data['count'] = 1; // set the count to 1 to indicate that this keyword is used once
			$tag_data['active'] = ($status == 0 ? (int) settings_item('lst.auto_approve_tag') : $status);
			$tag_data['created_on'] = date('Y-m-d H:i:s');
			$tag_data['module_relations'] = $module_id;
			$tag_id = $this->tags_model->insert($tag_data);
				
			if($tag_id) {
				$data = array(
						'row_id' => $row_id,
						'module_id' => $module_id,
						'tag_id' => $tag_id
				);
				$tag_relations = array(
						'row_id' => $tag_id,
						'module_id' => gc_module_id('tags'),
						'toggle_module_id' => $module_id
				);
				// if the module is listings, update the tag_locations data
				$tag_relation_id = $tag_id;
			}
		}
		return array('data' => $data, 'tag_relations' => $tag_relations, 'tag_relation_id' => $tag_relation_id);
	}
	
	/**
	 * Display Selectize Keywords while editing a module (Listings, Videos etc.)
	 * @param string $module
	 * @param integer $row_id
	 */
	public function selectize_keywords($module, $row_id) {
		// Display Existing Tags to the User
		$result = $this->db->select('GROUP_CONCAT(tag_id) AS tags')->get_where('tag_relations', array('row_id' => $row_id, 'module_id' => gc_module_id($module)))->row();
		if($result) {
			return $result->tags;
			// return $this->fields_model->selectize_result('tags', 'keywords', implode(',', $tag_ids));
		}
		return null;
	}
}
