<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class Financial extends Admin_Controller
{
	protected $permissionCreate = 'Tax_Rates.Financial.Create';
	protected $permissionDelete = 'Tax_Rates.Financial.Delete';
	protected $permissionEdit   = 'Tax_Rates.Financial.Edit';
	protected $permissionView   = 'Tax_Rates.Financial.View';
	
	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct()
	{
		parent::__construct();

		$this->auth->restrict($this->permissionView);
		$this->load->model('tax_rates_model', null, true);
		$this->lang->load('tax_rates');
		Template::set_block('sub_nav', 'financial/_sub_nav');

		Assets::add_module_js('tax_rates', 'tax_rates.js');
	}

	//--------------------------------------------------------------------
	
	
	/**
	 * Manage Tax Rates
	 * @param string $filter
	 * @param number $offset
	 */
	public function index($filter = 'all', $offset = 0) {
	    $this->manage('tax_rates', $filter = 'all', $offset = 0);
	}
	
	/**
	 * Manage Tax Rates
	 * @param string $module
	 * @param string $filter
	 * @param number $offset
	 */
	public function manage($module = 'tax_rates', $filter = 'all', $offset = 0) {
	    // Featured - Products
	    if (isset($_POST['tax_inclusive'])) {
	        $this->gc->status_change($module, $_POST['tax_inclusive'], 'tax_inclusive');
	    }
	    $extra = array(
	        'model' => 'tax_rates',
	        'context' => 'financial',
	        'fields' => $module
	    );
	    if($module == 'tax_rates') {
	        $extra['fields'] = null;
    	    $extra['actions'] = array( // Extra buttons
    	        array('type' => 'link', 'link' => site_url(SITE_AREA . '/financial/tax_rates/edit/%s'), 'label' => '<i class="fas fa-edit fa-fw mr-1"></i>' . lang('edit'), 'class' => 'btn btn-xs btn-primary'),
    	    );
	    } else {
	        $extra['model_name'] = 'tax_rates';
	        $extra['prefix'] = 'tax';
	    }
	    $this->gc->module_view($module, $filter, $offset, 'admin', $extra);
	}
	
	/**
	 * Add New Tax Rate/Tax Slab
	 * @param string $module
	 */
	public function form($module = 'tax_rates', $action = 'create') {
	    if($module == 'tax_rates') {
	        $this->create();
	    } else {
    	    $params = array(
    	        'selectize' => true,
    	        'model' => 'tax_rates',
    	        'context' => 'financial',
    	        'module_name' => 'tax_rates',
    	        'fields_method' => $module,
    	        'prefix' => 'tax',
    	        'keep_prefix' => true
    	    );
    	    $this->gcrud->form($module, $action, 'admin', $params);
	    }	    
	}

	/**
	 * Creates a Tax Rates object.
	 *
	 * @return void
	 */
	public function create() {
		$this->auth->restrict($this->permissionCreate);

		if (isset($_POST['save'])) {
			if ($insert_id = $this->save_tax_rates()) {
				// Log the activity
				log_activity($this->auth->user_id(), lang('act_create_record') .': '. $insert_id .' : '. $this->input->ip_address(), 'tax_rates');

				Template::set_message(sprintf(lang('msg_save_success'), lang('label_taxrate')), 'success');
				redirect(SITE_AREA .'/financial/tax_rates');
			} else {
				Template::set_message(sprintf(lang('msg_create_failure'), lang('label_taxrate')) . $this->tax_rates_model->error, 'danger');
			}
		}
		Assets::add_module_js('tax_rates', 'tax_rates.js');
		$this->load->model('locations/countries_model');
		Template::set('countries', $this->countries_model->select('iso, name')->order_by('name')->find_all());		
		Template::set('toolbar_title', lang('create_tax_rates'));
		Template::set_view('taxrate_form');
		Template::render();
	}

	/**
	 * Allows editing of Tax Rates data.
	 *
	 * @return void
	 */
	public function edit() {
		$id = $this->uri->segment(5);

		if (empty($id)) {
			Template::set_message(lang('invalid_id'), 'danger');
			redirect(SITE_AREA .'/financial/tax_rates');
		}

		if (isset($_POST['save'])) {
			$this->auth->restrict($this->permissionEdit);

			if ($this->save_tax_rates('update', $id)) {
				// Log the activity
				log_activity($this->auth->user_id(), lang('act_edit_record') .': '. $id .' : '. $this->input->ip_address(), 'tax_rates');

				Template::set_message(sprintf(lang('msg_save_success'), lang('label_taxrate')), 'success');
				redirect(SITE_AREA .'/financial/tax_rates');
			} else {
				Template::set_message(sprintf(lang('msg_edit_failure'), lang('label_taxrate')) . $this->tax_rates_model->error, 'danger');
			}
		} else if (isset($_POST['delete'])) {
			$this->auth->restrict($this->permissionDelete);

			if ($this->tax_rates_model->delete($id)) {
				// Log the activity
				log_activity($this->auth->user_id(), lang('act_delete_record') .': '. $id .' : '. $this->input->ip_address(), 'tax_rates');
				Template::set_message(sprintf(lang('msg_delete_success'), lang('label_taxrate')), 'success');
				redirect(SITE_AREA .'/financial/tax_rates');
			} else {
				Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_taxrate')) . $this->tax_rates_model->error, 'danger');
			}
		}
		$record = $this->tax_rates_model->find($id);
		Template::set('tax_rates', $record);
		$this->load->model('locations/countries_model');
		Template::set('countries', $this->countries_model->select('iso, name')->order_by('name')->find_all());
		if($record->country_iso != NULL) {
			$this->load->model('locations/states_model');
			$states = $this->states_model->select('id, name')->order_by('name')->find_all_by('country_iso', $record->country_iso);
			Template::set('states', $states);
		}
		Template::set('toolbar_title', lang('edit_tax_rates'));
		Template::set_view('taxrate_form');
		Template::render();
	}

	//--------------------------------------------------------------------
	// !PRIVATE METHODS
	//--------------------------------------------------------------------

	/**
	 * Summary
	 *
	 * @param String $type Either "insert" or "update"
	 * @param Int	 $id	The ID of the record to update, ignored on inserts
	 *
	 * @return Mixed    An INT id for successful inserts, TRUE for successful updates, else FALSE
	 */
	private function save_tax_rates($type='insert', $id = 0) {
		if ($type == 'update') {
			$_POST['id'] = $id;
		}
		
		$this->form_validation->set_rules('name', 'lang:label_name', 'trim|required');
		$this->form_validation->set_rules('order_customs', 'lang:tax_order_customs', 'trim');
		$this->form_validation->set_rules('ads', 'lang:tax_ads', 'trim');
		$this->form_validation->set_rules('wallets', 'lang:tax_wallets', 'trim');
		
		// $this->form_validation->set_rules('applicable_modules[]', 'lang:applicable_modules', 'trim|is_array');
		if ($this->form_validation->run() === false) {
		    return false;
		}
		// make sure we only pass in the fields we want
		$data = array();
		$data['name']       		= $this->input->post('name');
		$data['country_iso']        = $this->input->post('country_iso');
		$data['state_id']        	= $this->input->post('state_id') ? $this->input->post('state_id') : 0;
		$data['tax_percent']        = $this->input->post('tax_percent');
		$data['tax_inclusive']      = $this->input->post('tax_inclusive') ? 1 : 0;
		$data['applicable_modules'] = json_encode($this->input->post('applicable_modules'));
		
		if ($type == 'insert') {
			$id = $this->tax_rates_model->insert($data);

			if (is_numeric($id)) {
				$return = $id;
			} else {
				$return = FALSE;
			}
		} elseif ($type == 'update') {
			$return = $this->tax_rates_model->update($id, $data);
		}

		return $return;
	}
	
	/**
	 * Save settings
	 */
	public function settings() {
	    clear_cache('site_settings'); // Clear Site Settings Cache
	    $this->load->model('settings/settings_model');
	    if (isset($_POST['save'])) {
	        $this->form_validation->set_rules('custom', 'lang:order_customs', 'trim');
	        $this->form_validation->set_rules('ad', 'lang:allow_pdf', 'trim');
	        $this->form_validation->set_rules('wallet', 'lang:allow_comments', 'trim');
	        if($this->form_validation->run() == TRUE) {
	            $settings = array(
	                'custom' => (int) $this->input->post('custom'),
	                'ad' => (int) $this->input->post('ad'),
	                'wallet' => (int) $this->input->post('wallet')
	            );
	            $data = array (array('name' => 'tax.settings', 'value' => json_encode($settings)));
	            $updated = $this->settings_model->update_batch($data, 'name');
	            if ($updated) {
	                Template::set_message(sprintf(lang('msg_save_success'), lang('gc_menu_settings')), 'success');
	                Template::redirect(current_url());
	            } else {
	                Template::set_message(sprintf(lang('msg_edit_failure'), lang('gc_menu_settings')), 'danger');
	            }
	        }
	    }
	    $setting_name = 'tax.settings';
	    $result = $this->db->select('value')->get_where('settings', array('name' => $setting_name))->row();
	    if($result) {
	        $settings = json_decode($result->value ?? '');
	    } else {
	        $settings = json_decode(settings_item($setting_name) ?? '');
	    }
	    Template::set(array(
	        'toolbar_title' => lang('gc_menu_settings'),
	        'settings' => $settings,
	        'statuses' => $statuses
	    ));
	    Template::render();
	}
	
	public function fields() {
	    $this->load->library('fields/fields');
	    $this->fields->prepare_fields('tax_rates');
	}
}