<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Online IT Solutions							           # ||
|| # Copyright (C) 2021 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Developer controller
 */
class Developer extends Admin_Controller
{
	protected $permissionView   = 'Themes.Developer.View';
	protected $permissionCreate = 'Themes.Developer.Create';
	protected $permissionEdit   = 'Themes.Developer.Edit';
	protected $permissionDelete = 'Themes.Developer.Delete';
	protected $folders = array();
	protected $views = array();
	protected $layouts = array();
	protected $write_from_db = true;

	/**
	 * scss_variables - stores complete information about each variable
	 * scss_values - stores only the updated values
	 */
	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct() {
		parent::__construct();

		$this->auth->restrict($this->permissionView);
		$this->lang->load('themes');
		$this->load->model('themes_model', null, true);
		$this->load->library('widgets');
		$this->load->library('theme_builder');
		Template::set_block('sub_nav', 'developer/_sub_nav');
		$this->folders = array(
		    '' => lang('label_root'),
		    'affiliates' => lang('gc_menu_affiliates'),
		    'blogs' => lang('gc_menu_blogs'),
		    'posts' => lang('gc_menu_posts'),
		    'forums' => lang('gc_menu_forums'),
		    'members' => lang('gc_menu_members'),
			'categories' => lang('label_categories'),
			'footers' => lang('label_footers'),
			'headers' => lang('label_headers'),
		    'modules' => lang('label_modules'),
		    'products' => lang('gc_menu_products'),
		    'blogs' => lang('gc_menu_blogs'),
			'navbars' => lang('label_navbars'),
		    'sliders' => lang('label_sliders'),
		    'faqs' => lang('label_faqs'),
		    'features' => lang('label_features'),
		    'teams' => lang('label_teams'),
			'testimonials' => lang('label_testimonials'),
			'social_icons' => lang('label_social_icons'),
			'subscribers' => lang('label_subscribers'),
		    'users' => lang('users'),
		    'packages' => lang('gc_menu_ad_packages'),
		    'home' => lang('label_home'),
		    'share' => lang('label_share'),
		    'search' => lang('label_search'),
		    'details' => lang('label_product_details'),
			'others' => lang('others'),
		);
		
		/**
		 * Views folders inside themes/default/views must be created with the same name as defined in the key
		 * View and widget folders are hard coded. In case of View, the Type field is used as the folder name
		 * 
		 * How to Create a New View?
		 * 
		 * 1. Create a new folder inside the themes/default/views folder. e.g. user_detail
		 * 2. Create a view with the same name (e.g. user_detail) through the Admin Panel via Developer->Themes->Add tab.
		 * 3. After save, click the Default Icon to set it as the Default view for Template Render method.
		 * Settings Default will automatically create a new php file with the view name e.g. user_detail.php inside the themes/default folder. 
		 * 4. Use that file in the Template::render('user_detail') class.
		 * 
		 * You can now create multiple files inside the themes/default/views/user_detail folder to have a separate design for the user detail page.
		 * Repeat steps 2 & 3 to use the new design.
		 */
		$this->views = array(
			'home' => lang('label_home'),
			'users/login' => lang('label_users_login'),
			'users/register' => lang('label_users_register'),
			'users/forgot_password' => lang('label_forgot_password'),
			'users/reset_password' => lang('label_reset_password'),
			'users/verify_otp' => lang('label_verify_otp'),
			'users/detail' => lang('label_users_detail'),
			'categories' => lang('label_categories'),
			'locations' => lang('label_locations'),
		    'leads/form' => lang('label_leads_form'),
		    'pages' => lang('label_pages'),
		    // 'customs' => lang('label_customs'),
		    'modules/browse' => lang('label_browse'),
		    'modules/affiliates' => lang('gc_menu_affiliates'),
            'affiliates/packages' => lang('gc_menu_affiliates') . '/' . lang('gc_menu_packages'),
		    'modules/forums' => lang('gc_menu_forums'),
		    'modules/releases' => lang('product_releases') . ' ' . lang('label_view'),
			'members/index' => lang('label_member_index'),
		    'members/favorites' => lang('label_member_favorites'),
		    'members/wishlists' => lang('label_member_wishlists'),
		    'members/notifications' => lang('label_member_notifications'),
			'members/orders' => lang('label_member_orders'),
			'members/checkout' => lang('label_member_checkout'),
			'members/comments' => lang('label_member_comments'),
			'members/view' => lang('label_member_view'),
			'members/ads' => lang('label_member_ads'),
			'members/pages' => lang('label_member_pages'),
		    'members/releases' => lang('label_member_releases'),
		    'members/messages' => lang('label_member_messages'), // emailer queue messages
		    'members/packages' => lang('ad_packages')
		);
		
		$config_modules = gc_modules();
		$config_modules[] = 'common';
		foreach($config_modules as $config_module) {
			$this->views['modules/' . $config_module . '/view'] = lang($config_module) . ' ' . lang('label_view');
			$this->views['modules/' . $config_module . '/detail'] = lang($config_module) . ' ' . lang('label_detail');
		}
		
		/**
		 * Layouts
		 * Note: We cannot have a LAYOUT with the same folder name as in the VIEW folder. 
		 * Creating a same folder e.g. pages view and pages layout will result in css overwritten.
		 * i.e. Home folder name is changed to _home and pages to _pages
		 */
		$this->layouts = array(
			'index' => lang('label_index'),
			'_home' => lang('bf_home'),
			'_view' => lang('label_view'),
			'_detail' => lang('label_detail'),
		    '_pages' => lang('label_pages'),
		    '_posts' => lang('gc_menu_posts'),
		    '_affiliates' => lang('gc_menu_affiliates'),
		    '_users' => lang('label_users'),
		    '_forums' => lang('gc_menu_forums'),
		    '_blogs' => lang('gc_menu_blogs'),
		    '_browse' => lang('label_browse'),
		    '_manuals' => lang('gc_menu_manuals'),
		    '_none' => lang('label_none'),
			'member' => lang('label_member'),
			'two_columns' => lang('two_columns'),
			'three_columns' => lang('three_columns'),
			'custom_01' => lang('label_custom') . '_01',
			'custom_02' => lang('label_custom') . '_02',
			'custom_03' => lang('label_custom') . '_03',
			'custom_04' => lang('label_custom') . '_04',
		    'custom_05' => lang('label_custom') . '_05',
		    'custom_06' => lang('label_custom') . '_06',
		    'custom_07' => lang('label_custom') . '_07',
		    'custom_08' => lang('label_custom') . '_08',
		    'custom_09' => lang('label_custom') . '_09',
		    'custom_10' => lang('label_custom') . '_10',
		    'custom_11' => lang('label_custom') . '_11',
		    'custom_12' => lang('label_custom') . '_12',
		    'custom_13' => lang('label_custom') . '_13',
		    'custom_14' => lang('label_custom') . '_14',
		    'custom_15' => lang('label_custom') . '_15'
		);
		
		// Get folder names from the theme_folders table
		$this->views = $this->themes_model->get_folders(0);		
		$config_modules = gc_modules();
		$config_modules[] = 'blogs';
		$config_modules[] = 'posts';
		$config_modules[] = 'common';
		foreach($config_modules as $config_module) {
		    $this->views['modules/' . $config_module . '/view'] = lang($config_module) . ' ' . lang('label_view');
		    $this->views['modules/' . $config_module . '/detail'] = lang($config_module) . ' ' . lang('label_detail');
		}
		$this->folders = $this->themes_model->get_folders(1);
		$this->layouts = $this->themes_model->get_folders(2);

		// Clear Theme Specific Cache
		// Each Controller method query the database to load Assets (CSS and JS), View and Render Layout.
		// i.e. it is necessary to clear the cache on theme changes e.g. Delete, Default Set, Edit etc.
		clear_cache('theme_layouts');
		
		Assets::add_module_css('themes', 'themes.css');
		Assets::add_module_js('themes', 'themes.js');
	}


	/**
	 * Displays a list of form data.
	 *
	 * @return void
	 */
	public function index($filter='all', $folder = 'all', $offset=0) {
		// Actions done, now display the view
		$where = "";
		// Filters
		$filter_type = $filter;
		
		switch($filter_type) {
			case 'widgets':
				$where .= " T.type_id = 1";
				break;
					
			case 'views':
				$where .= " T.type_id = 0";
				break;
					
			case 'layouts':
				$where .= " T.type_id = 2";
				break;
		
			case 'all':
				// Nothing to do
				break;
		
			default:
				show_404("themes/index/$filter/");
		}
		
		// Allow Admin to Make a Copy of the Link
		if (!empty ( $_POST ['clone'] )) {
		    gc_demo_mode();
		    $this->gcrud->stats(array('module' => 'themes'));
			$this->gc->clone_row('themes', 'theme_views', 'theme_id');
		}
		
		/*------------------------------------------------*/
		/* Set the selected view as the default
		/* Also overwrite the corresponding view file
		/*------------------------------------------------*/
		if(isset($_POST['default'])) {
		    $this->gcrud->stats(array('module' => 'themes'));
			//SET DEFAULT 1 FOR SELECTED ROW
			$id = $_POST['default'];
			$this->widgets->set_default('views', $id);
		}
		
		if(isset($_POST['fill_scss'])) {
			// Auto Fill SCSS Variables
			$id = $_POST['fill_scss'];
			$this->widgets->fill_scss(NULL, $id);
			Template::set_message(lang('msg_fill_success'), 'success');
		}
		
		if(isset($_POST['is_theme_default'])) {
		    gc_demo_mode();
		    $this->gcrud->stats(array('module' => 'themes'));
			// Make selected view as theme default
			$id = $_POST['is_theme_default'];
			$this->themes_model->is_theme_default($id, 1);
			$record = $this->themes_model->select('theme, file_folder')->find($id);
			Template::set_message(sprintf(lang('theme_default_success'), ucfirst($record->theme), ucfirst($record->file_folder)), 'success');
		}
		
		/*-------------------------------------------------*/
		/* Delete selected rows and the corresponding files
		/* PHP, HTML and JS Files
		/*-------------------------------------------------*/
		if (isset($_POST['submit'])) {
		    gc_demo_mode();
		    $this->gcrud->stats(array('module' => 'themes'));
			switch($_POST['submit']) {
				case 'delete':
					$this->auth->restrict($this->permissionDelete);
					$checked = $this->input->post('checked');
					if (is_array($checked) && count($checked)) {
						$result = FALSE;
						foreach ($checked as $pid) {
							$result = $this->_delete_widget($pid);
						}
							
						if ($result) {
							Template::set_message(count($checked) .' '. sprintf(lang('msg_delete_success'), $this->theme_type(false)), 'success');
						} else {
							Template::set_message(sprintf(lang('msg_delete_failure'), $this->theme_type(false)) . $this->themes_model->error, 'danger');
						}
					}
					break;
			}
		}
		
		// advanced filter option
		
		// Search Handling
		$searchterm = $this->gc->search('themes', $this->uri->segment(6), '/developer/themes/');
		if(!empty($searchterm)) {
			if(!empty($where)) {
				$where .= " AND";
			}
			$where .= " (T.id LIKE '%". $searchterm ."%' OR T.name LIKE '%". $searchterm ."%' OR T.file_folder LIKE '%". $searchterm ."%' OR T.file_name LIKE '%". $searchterm ."%' OR FIND_IN_SET('" . $searchterm . "', T.theme))";
		}
		
		// Sorting
		$sort_by = 'T.id, T.is_default DESC';
		if(isset($_POST['sort']) && $_POST['sort'] == 'view') {
			$order_by = $this->gc->sort('themes', 'TV');
		} else {
			$order_by = $this->gc->sort('themes', 'T');
		}
		if(!empty($order_by)) {
			$sort_by = $order_by;
		}
		
		// Display rows to the user
		if(!empty($folder) && $folder != 'all' && !empty($where)) {
		    $where .= " AND T.file_folder = '" . str_replace('-', '/', $folder) . "'";
		}
		$records = $this->themes_model->get_widgets(0, $where, $sort_by, $offset, $this->limit);
		$total_themes = $this->themes_model->get_widgets(1, $where, $sort_by, $offset, $this->limit);
		previousUrl(true);
		// Pagination
		$this->load->library('pagination');
		
		$pager['base_url'] = site_url(SITE_AREA ."/developer/themes/index/$filter/$folder/");
		$pager['total_rows'] = $total_themes;
		$pager['per_page'] = $this->limit;
		$pager['uri_segment']	= 7;
		
		$this->pagination->initialize($pager);
		
		// View Data
		$view_data = array(
		    'index_url' => site_url(SITE_AREA .'/developer/themes/index/'),
		    'filter_type' => $filter_type,
		    'records' => $records,
		    'toolbar_title' => lang('manage_themes'),
		    'total_rows' => $total_themes
		);
		if($filter !== 'all') {
		    if($filter == 'views') {
		        $filter_options = $this->views;		        
		    } else if($filter == 'layouts') {
		        $filter_options = $this->layouts;		        
		    } else {
		        $filter_options = $this->folders;
		    }
		    $option_filters = array();
		    foreach($filter_options as $fkey => $filter_option) {
		        $fkey = str_replace('/', '-', $fkey);
		        if(!empty($fkey)) {
		          $option_filters[$fkey] = $filter_option;
		        }
		    }
		    $view_data['folder_filter'] = !empty($option_filters[$folder]) ? $option_filters[$folder] : lang('tab_all'); 
		    $view_data['filter_options'] = $option_filters;
		}
		Template::set($view_data);
		Template::render();
	}
	
	/**
	 * Add New View/Widget
	 */
	public function add() {
		Assets::add_js(array(
			'jquery.fancybox.pack.js',			
		));
		Assets::add_css('selectize.bootstrap3.css');
		Assets::add_js('selectize.min.js');
		Assets::add_js('selectize-search.js');		
		
		// multiple theme select
		Assets::add_js('bootstrap-select.min.js');
		Assets::add_css('bootstrap-select.min.css');
		$maxOptions = 100;
		$field_name = 'theme';
		Assets::add_js( "$('#" . $field_name . "').selectpicker({liveSearch: true, maxOptions: " . $maxOptions . ", multipleSeparator: ', ', showTick: true})" , 'inline' );
		Assets::add_js("var savedValues = $('#" . $field_name . "').data('value'); $('#" . $field_name . "').selectpicker('val', savedValues ? savedValues.toString().split(',') : '');", 'inline');
				
		// Selectized Input Field - Display Data
		$this->selectized_views();
		
		if (isset($_POST['save'])) {
			if ($insert_id = $this->save_theme()) {
				// Log the activity
				log_activity($this->auth->user_id(), lang('act_create_record') .': '. $insert_id .' : '. $this->input->ip_address(), 'themes');
	
				Template::set_message(sprintf(lang('msg_save_success'), $this->theme_type()), 'success');
				redirect(previousUrl());
			} else {
				Template::set_message(sprintf(lang('msg_create_failure'), $this->theme_type()) . $this->themes_model->error, 'danger');
			}
		}
		
		// Widget specific parameters handling
		$this->params_handling();
		
		// CSS & JS dependencies
		$this->css_jss_dependencies('css');
		$this->css_jss_dependencies('js');
		
		// Selected View(s) POST HANDLING
		if(!empty($_POST['views'])) {
			Template::set('selected_views', $_POST['views']);
		}
		if(!empty($_POST['view'])) {
			Template::set('view_selected', $_POST['view']);
		}
		// Allow user to group a VIEW in theme
		$themes = $this->db->select('name')->get_where('theme_settings', array('type' => 'themes'))->result();
		Template::set(array(
			'folders' => $this->folders,
			'views' => $this->views,
			'layouts' => $this->layouts,
			'themes' => $themes
		));
	
		Template::set('toolbar_title', lang('add') . ' ' . lang('layout_view_widget'));
		Template::set_view('widget_form');
		Template::render();
	}
	
	/**
	 * Edit View/Widget Developer
	 */
	public function edit() {
		Assets::add_css('selectize.bootstrap3.css');
		Assets::add_js('selectize.min.js');
		Assets::add_js('selectize-search.js');		
		
		// multiple theme select
		Assets::add_js('bootstrap-select.min.js');
		Assets::add_css('bootstrap-select.min.css');
		$maxOptions = 100;
		$field_name = 'theme';
		Assets::add_js( "$('#" . $field_name . "').selectpicker({liveSearch: true, maxOptions: " . $maxOptions . ", multipleSeparator: ', ', showTick: true})" , 'inline' );
		Assets::add_js("var savedValues = $('#" . $field_name . "').data('value'); $('#" . $field_name . "').selectpicker('val', savedValues ? savedValues.toString().split(',') : '');", 'inline');
		
		$id = $this->uri->segment(5);
		
		if (empty($id)) {
			Template::set_message(lang('invalid_id'), 'danger');
			redirect(SITE_AREA .'/developer/themes');
		}
		Assets::add_js(array(
			'jquery.fancybox.pack.js'
		));
		
		if (isset($_POST['save'])) { // save view/widget
			if ($this->save_theme('update', $id)) {
				// Log the activity
				log_activity($this->auth->user_id(), lang('act_edit_record') .': '. $id .' : '. $this->input->ip_address(), 'themes');

				Template::set_message(sprintf(lang('msg_save_success'), $this->theme_type()), 'success');
				// Template::redirect(SITE_AREA .'/developer/themes/edit/' . $id);
				Template::redirect(previousUrl());
			} else {
				Template::set_message(sprintf(lang('msg_edit_failure'), $this->theme_type()) . $this->themes_model->error, 'danger');
			}
		} else if (isset($_POST['delete'])) { // delete view/widget
		    gc_demo_mode();
		    $this->gcrud->stats(array('module' => 'themes'));
			$this->auth->restrict($this->permissionDelete);

			if ($this->_delete_widget($id)) {
				// Log the activity
				log_activity($this->auth->user_id(), lang('act_delete_record') .': '. $id .' : '. $this->input->ip_address(), 'themes');
				Template::set_message(sprintf(lang('msg_delete_success'), $this->theme_type()), 'success');
				redirect(SITE_AREA .'/developer/themes');
			} else {
				Template::set_message(sprintf(lang('msg_delete_failure'), $this->theme_type()) . $this->themes_model->error, 'danger');
			}
		}
		
		// Get widget/view information
		$record = $this->themes_model->find($id);
		if(!$record) {
			Template::redirect(SITE_AREA .'/developer/themes');
		}
		
		// File URL handling - restore category and file URL fields
		/*if(!empty($record->url) && $record->type_id == 1) {
			$file_url = explode('/', $record->url ?? '');
			if(sizeof($file_url) > 2) {
				Template::set(array(
					'category' => $file_url[1],
					'file_name' => $file_url[2]
				));
			} else if(sizeof($file_url) == 2) {
				Template::set(array(
					'category' => '',
					'file_name' => $file_url[1]
				));
			}
		}*/
		
		// Show widget specific parameters to the user
		$this->params_handling(true, $record->params);
		
		// CSS and JS dependencies
		$this->css_jss_dependencies('css', true, $record->assets_css);
		$this->css_jss_dependencies('js', true, $record->assets_js);
		
		// selected views handling
		$views = $this->db->get_where('theme_views', array('theme_id' => $id))->result();
		if($views) {
			if($record->type_id == 1) {
				if(!empty($_POST['views'])) {
					Template::set('selected_views', $_POST['views']);
				} else {
					$selected_views = array();
					foreach($views as $view) {
						$selected_views[] = $view->view;
					}
					Template::set('selected_views', $selected_views);					
				}			
			} else {
				if(!empty($_POST['view'])) {
					Template::set('view_selected', $_POST['view']);
				} else {
					Template::set('view_selected', $views[0]->view);
				}
			}			
		}
		
		// Selectized Input Field - Display Data
		$this->selectized_views($views);
		
		// Allow user to group a VIEW in theme
		$themes = $this->db->select('name')->get_where('theme_settings', array('type' => 'themes'))->result();
		Template::set(array(
			'record' => $record,
			'folders' => $this->folders,
			'views' => $this->views,
			'layouts' => $this->layouts,
			'themes' => $themes
		));
		
		Template::set('toolbar_title', lang('edit') . ' ' . lang('layout_view_widget'));
		Template::set_view('widget_form');
		Template::render();
	}
	
	/**
	 * Widget Form - Select Widget Applicable Views Selectized Input Field
	 * @param object $records
	 */
	private function selectized_views($records = array()) {
		$selectized_views = array();
		// Selectized Views Input Field
		if(!empty($_POST['applicable_views'])) {
			$views = explode(',', $_POST['applicable_views']);
			if(!empty($views)) {
				foreach($views as $view) {
					$selectized_views[] = array(
						'id' => $view,
						'name' => ($view == 'all' ? lang('all_views') : $this->views[$view])
					);
				}
			}
		} else if(!empty($records)) {
			foreach($records as $view) {
				if(isset($this->views[$view->view])) {
					$selectized_views[] = array(
						'id' => $view->view,
						'name' => $this->views[$view->view]
					);
				}
			}			
		} else {
			$selectized_views[] = array(
				'id' => 'all',
				'name' => lang('all_views')
			);
		}
		Template::set('selectized_views', $selectized_views);
	}
	
	/**
	 * Widget specific parameters handling
	 * @param string $edit
	 * @param array $params
	 */
	private function params_handling($edit = false, $params = array()) {
		// POST DATA HANDLING
		/**
		 * Validation rules are must for field_options
		 * else you will get array to string conversion errors
		 */
		if(isset($_POST['field_options'])) {
			$variables = array();
			foreach($_POST['field_options']['name'] as $key => $value) {
				$variable = array();
				$variable['name'] = $_POST['field_options']['name'][$key];
				$variable['value'] = $_POST['field_options']['value'][$key];
				$variable['description'] = $_POST['field_options']['description'][$key];
				$variables[] = $variable;
			}
			Template::set('options', $variables);
		} else if($edit) {
			//$variables = $this->themes_model->get_scss_variables($id);
			$variables = json_decode($params ?? '');
			Template::set('options', $variables);
		}
	}
	
	/**
	 * Summary
	 *
	 * @param String $type Either "insert" or "update"
	 * @param Int	 $id	The ID of the record to update, ignored on inserts
	 *
	 * @return Mixed    An INT id for successful inserts, TRUE for successful updates, else FALSE
	 */
	private function save_theme($type='insert', $id = 0) {
	    gc_demo_mode();
	    $this->gcrud->stats(array('module' => 'themes'));
		if ($type == 'update') {
			$_POST['id'] = $id;
		}
		
		$type_id				= intval($this->input->post('type_id'));
		switch($type_id) {
			case 1:
				$file_folder	= $this->input->post('file_folder');
				break;
			case 2:
				$file_folder	= $this->input->post('layout');
				break;
			default:
				$file_folder	= $this->input->post('view');
		}
		$file_url			= $this->themes_model->file_url($type_id, $file_folder) . $this->input->post('file_name');
		$_POST['file_url'] 	= $file_url;
		$this->form_validation->set_rules('name', 'lang:label_name', 'trim|required|unique[themes.name, themes.id]');
		$this->form_validation->set_rules('file_name', 'lang:label_file_name', 'trim|required|alpha_slug');
		$this->form_validation->set_rules('class_name', 'lang:label_class_name', 'trim');
		$this->form_validation->set_rules('theme_block_id', 'lang:singular_theme_blocks', 'trim');
		if($type_id == 1) {
			//$this->form_validation->set_rules('views[]', 'lang:label_applicable_view', 'trim|required|is_array');
			$this->form_validation->set_rules('applicable_views', 'lang:label_applicable_view', 'trim|required');
			$this->form_validation->set_rules('file_folder', 'lang:label_file_folder', 'trim');

			// Widget specific parameters
			$this->form_validation->set_rules('field_options[name][]', 'lang:label_name', 'trim|is_array');
			$this->form_validation->set_rules('field_options[value][]', 'lang:label_value', 'trim|is_array');
			$this->form_validation->set_rules('field_options[description][]', 'lang:label_description', 'trim|is_array');
			$this->form_validation->set_rules('field_options[lang][]', 'lang:label_lang', 'trim|is_array');
		}
		
		// CSS dependencies
		$this->form_validation->set_rules('css[type][]', 'lang:label_type', 'trim|is_array');
		$this->form_validation->set_rules('css[url][]', 'lang:label_url', 'trim|is_array');
		$this->form_validation->set_rules('css[file][]', 'lang:label_file', 'trim|is_array');
		
		// JS dependencies
		$this->form_validation->set_rules('js[type][]', 'lang:label_type', 'trim|is_array');
		$this->form_validation->set_rules('js[url][]', 'lang:label_url', 'trim|is_array');
		$this->form_validation->set_rules('js[file][]', 'lang:label_file', 'trim|is_array');
		
		$this->form_validation->set_rules('file_url', 'lang:label_file_name', 'trim|unique[themes.file_url, themes.id]');
		$this->form_validation->set_rules('image_url', 'lang:label_image', 'trim');
		$this->form_validation->set_rules('notes', 'lang:label_notes', 'trim');
		$this->form_validation->set_rules('is_widget_design', 'lang:is_widget_design', 'trim');
		
		if ($this->form_validation->run() === false) {
			return false;
		}
		
		// make sure we only pass in the fields we want
		$data = array();
		$data['name']       = $this->input->post('name');
		$data['file_folder']= $file_folder;
		$data['file_name']	= $this->input->post('file_name');
		$data['file_url']	= $file_url;
		$data['class_name']	= $this->input->post('class_name');
		$data['theme_block_id']	= (int) $this->input->post('theme_block_id');
		$data['image_url'] 	= $this->input->post('image_url');
		$data['notes']    	= $this->input->post('notes');
		// the layout design widget will be displayed while adding theme blocks
		$data['is_widget_design'] = $this->input->post('is_widget_design') ? 1 : 0;
		
		$params = array();
		if($type_id == 1) {
			if(isset($_POST['field_options'])) {
				foreach($_POST['field_options']['name'] as $key => $value) {
					if(isset($value)) {
						$param = array();
						// widget/view ID will be appended with each variable to differentiate it from other views/widgets
						// this variable will be removed before processing the variable name
						$field_name = $_POST['field_options']['name'][$key];
						$field_value = $_POST['field_options']['value'][$key];
						$field_lang = intval($_POST['field_options']['lang'][$key]);
						$param['name'] = $field_name;
						$param['value'] = $field_value;
						$param['description'] = $_POST['field_options']['description'][$key];
						$param['lang'] = $field_lang != 1 ? 0 : 1;						
						$params[] = $param;
					}
				}
			}			
		} else {
		    $data['layout'] = $this->input->post('layout');
		    $selected_themes = $this->input->post('theme');
		    $data['theme'] = is_array($selected_themes) ? implode(',', $selected_themes) : $selected_themes;
		}
		$data['params'] = json_encode($params);
		$data['assets_css'] = json_encode($this->dependencies_post_handling('css', true));
		$data['assets_js'] = json_encode($this->dependencies_post_handling('js', true));
		$data['type_id'] = $type_id;
		
		if ($type == 'insert') {
			
			$id = $this->themes_model->insert($data);
	
			if (is_numeric($id)) {
				$return = $id;
			} else {
				$return = FALSE;
			}
		} elseif ($type == 'update') {
			$return = $this->themes_model->update($id, $data);
		}

		// update theme_views table
		// a widget can be used in multiple themes
		if($return && $id != 0) {
			if($type_id == 1) {
				// OLD CHECKBOX SELECTION CODE
				/*$views = $this->input->post('views');
				$view_data = array();
				if(!empty($views)) {
					foreach($views as $key => $view) {
						$view_data[] = array(
							'theme_id' => $id,
							'view' => $key
						);
					}
					// Admin may have unselected a view,
					// so delete all existing views and insert from the beginning
					$this->db->where('theme_id', $id)->delete('theme_views');
					$this->db->insert_batch('theme_views', $view_data);
				}*/
				$views = $this->input->post('applicable_views');
				$views = explode(',', $views ?? '');
				if(!empty($views)) {
					$view_data = array();
					if(in_array('all', $views)) {
						// This widget is applicable to all views i.e. set the is_widget_all column value to 1
						// Addition of any VIEW in the FUTURE will also be able to view this widget in the Editor selection list
						$this->db->where('id', $id)->update('themes', array('is_widget_all' => 1));						
						/*foreach($this->views as $key => $view) {
							$view_data[] = array(
								'theme_id' => $id,
								'view' => $key
							);
						}*/
						// Delete all existing views
						$this->db->where('theme_id', $id)->delete('theme_views');
					} else {
						// Widget is not for all View
						$this->db->where('id', $id)->update('themes', array('is_widget_all' => 0));
						foreach($views as $key) {
							$view_data[] = array(
								'theme_id' => $id,
								'view' => $key
							);
						}
						// Admin may have unselected a view,
						// so delete all existing views and insert from the beginning
						$this->db->where('theme_id', $id)->delete('theme_views');
						$this->db->insert_batch('theme_views', $view_data);
					}					
				}
			} else {
				$view_data = array(
					'theme_id' => $id,
					'view' => ($type_id == 0 ? $this->input->post('view') : $this->input->post('layout'))
				);
				if($type == 'insert') {
					$this->db->insert('theme_views', $view_data);
				} else if($type == 'update') {
				    if($this->db->where('theme_id', $id)->count_all_results('theme_views') > 0) {
					   $this->db->where('theme_id', $id)->update('theme_views', $view_data);
				    } else {
				        $this->db->insert('theme_views', $view_data);
				    }
				}
				// Handle Theme Default
				$this->themes_model->is_theme_default($id, ($this->input->post('is_theme_default') ? 1 : 0));
			}
		}
		return $return;
	}
	
	/**
	 * Return Theme Type - To display appropriate message on Theme Save
	 * @return string
	 */
	private function theme_type($singular = true) {
		$type_id = intval($this->input->post('type_id'));
		switch($type_id) {
			case 1:
				$lang = lang('label_widget');
				break;
			case 2:
				$lang = lang('label_layout');
				break;
			default:
				$lang = lang('label_view');
		}
		return $lang;
	}
	
	/**
	 * CSS and JS dependencies handling
	 * @param string $edit
	 * @param array $params
	 */
	private function css_jss_dependencies($field = 'css', $edit = false, $assets = array()) {
		/**
		 * Validation rules are must for css files
		 * else you will get array to string conversion errors
		 */
		if($edit) {			
			if(!empty($_POST[$field])) {
				Template::set($field . '_files', $this->dependencies_post_handling($field));
			} else {
				$files = json_decode($assets ?? '');
				Template::set($field . '_files', $files);
			}
		} else {
			Template::set($field . '_files', $this->dependencies_post_handling($field));
		}
	}
	
	/**
	 * CSS and JS files POST dependencies handling method
	 * @param string $field - CSS/JS
	 * @return multitype:multitype:NULL
	 */
	private function dependencies_post_handling($field, $save = false) {
		$files = array();
		$files_to_be_saved = array();
		if(isset($_POST[$field])) {
			foreach($_POST[$field]['type'] as $key => $value) {
				$file = array();
				$type = $_POST[$field]['type'][$key];
				$file_url = ($type == 'external' ? $_POST[$field]['url'][$key] : $_POST[$field]['file'][$key]);
				$file['url'] = $file_url;
				$file['type'] = $_POST[$field]['type'][$key];
				if($save && !empty($file['url'])) {	// do not save a file if the url field is blank			
					$files_to_be_saved[] = $file;
				}
				$files[] = $file;
			}
		}
		if($save) {
			return $files_to_be_saved;
		} else {
			return $files;
		}
	}
	
	/**
	 * Common upload function for css and js files
	 * @param string $upload_path
	 * @param string $field_name
	 * @return multitype:NULL
	 */
	private function upload_file_old($upload_path, $field_name){
		if(!gc_demo_mode(array('no_message' => true, 'no_redirect' => true))) {
            return false;
        }
		$config['upload_path'] = realpath(FCPATH.'/themes/default/'. $upload_path .'/'); //Make SURE that you chmod this directory to 777!
		$config['allowed_types'] = 'js|css';
		$config['max_size']= '4096';
		$config['remove_spaces']=TRUE; //Remove spaces from the file name
		$this->load->library('upload', $config);	
		if (!$this->upload->do_upload($field_name)) { // Image name is necessary
			$this->session->set_flashdata('upload_file_error', $this->upload->display_errors());
			return false;				
		} else {
			$data = array('upload_data' => $this->upload->data());
		}
		return $data;
	}
	
	/**
	 * Delete widget/view completely
	 * @param number $pid
	 */
	private function _delete_widget($pid) {
		// delete widget files first before deleting the database rows
		$file = $this->themes_model->select('type_id, file_folder, file_name, file_url')->find($pid);
		if($file && !empty($file->file_url)) { // Delete widget files only not views
			$file_folder = $this->themes_model->file_url($file->type_id, $file->file_folder);
			$this->widgets->unlink_file($file_folder . $file->file_name, 'php');
			$this->widgets->unlink_file($file_folder . 'scss/' .  $file->file_name, 'scss');
			$this->widgets->unlink_file($file_folder . 'js/' .  $file->file_name, 'js');
		}
		// delete corresponding views
		$this->db->delete('theme_views', array('theme_id' => $pid));
		return $this->themes_model->delete($pid);
	}
	
	/**
	 * Theme files editor
	 * @return boolean
	 */
	public function editor() {
		$id = $this->uri->segment(5);
		$load_file = $this->uri->segment(6) ? true : true; // Load contents from File
		
		if (empty($id)) {
			Template::set_message(lang('invalid_id'), 'danger');
			redirect(SITE_AREA .'/developer/themes');
		}
		
		Assets::add_js('edit_area/edit_area_full.js');
		
		Assets::add_js( array(
			'jquery.dataTables.min',
			'dataTables.bootstrap.min',
		));
		Assets::add_css('dataTables.bootstrap.min');
		Assets::add_js("$('#widgets').DataTable();", 'inline');

		Assets::add_css('codemirror/lib/codemirror.css');
		Assets::add_js('codemirror/lib/codemirror.js');
		Assets::add_js('codemirror/mode/javascript/javascript.js');
		
		$file = $this->themes_model->select('type_id, contents, file_folder, file_name, image_url')->find($id);
		
		if($file && !empty($file->file_name)) {
			// Get view type
			$view_type = $this->db->select('view')->get_where('theme_views', array('theme_id' => $id))->row();
			
			// Add slash after the folder name
			switch($file->type_id) {
				case 1:
					$file_folder = 'widgets/';
					break;
				case 2:
					$file_folder = 'layouts/';
					break;
				default:
					$file_folder = 'views/';					
			}
			if($file->type_id == 1) {
			} else {
				
				// Display all avaialble widgets to the user with image and help
				if($view_type) {
					$widgets = $this->themes_model->get_view_widgets($view_type->view, $id);
					Template::set('widgets', $widgets);
				} 
			}
			$file_folder .= !empty($file->file_folder) ? $file->file_folder . '/' : '';
			
			if (isset($_POST['save']) || isset($_POST['save_write'])) {
			    gc_demo_mode();
			    $this->gcrud->stats(array('module' => 'themes'));
				$this->form_validation->set_rules('php_contents', 'lang:label_php', 'trim');
				$this->form_validation->set_rules('scss_contents', 'lang:label_scss', 'trim');
				$this->form_validation->set_rules('js_contents', 'lang:label_js', 'trim');
				/*$this->form_validation->set_rules('html', 'lang:save_html', 'trim');
				$this->form_validation->set_rules('scss', 'lang:save_scss', 'trim');
				$this->form_validation->set_rules('js', 'lang:save_js', 'trim');*/
				if ($this->form_validation->run() === false) {
					return false;
				}
				
				$write_file = isset($_POST['save_write']) ? true : false;
				
				$contents = array();
				
				/*if($this->input->post('php_contents')) {
					
				}*/
				// Allow blank file creation
				if($write_file) {
					$this->widgets->save_file($file_folder . $file->file_name, 'php', $this->input->post('php_contents'));
				}
				$contents['php'] = $this->input->post('php_contents');
				
				// Allow blank file creation
				if($write_file) {
					$this->widgets->save_file($file_folder . 'scss/' . $file->file_name, 'scss', $this->input->post('scss_contents'));
				}
				$contents['scss'] = $this->input->post('scss_contents');
				/*if($this->input->post('scss_contents')) {
					
				}*/
				
				// Allow blank file creation
				if($write_file) {
					$this->widgets->save_file($file_folder . 'js/' . $file->file_name, 'js', $this->input->post('js_contents'));
				}
				$contents['js'] = $this->input->post('js_contents');
				
				/*if($this->input->post('js_contents')) {
					
				}*/
				$this->db->where('id', $id);
				$this->db->update('themes', array('contents' => json_encode($contents)));
				
				// Process the View file
				$this->widgets->process_view($id);
				Template::set_message(sprintf(lang('msg_save_success'), lang('label_widget')), 'success');
			}
			
			// Load file directly instead of loading contents from DB
			$data = array();
			if($load_file) {
				$data['php'] = $this->widgets->get_contents($file_folder . $file->file_name . '.php');
				$data['scss'] = $this->widgets->get_contents($file_folder . 'scss/' . $file->file_name. '.scss');
				$data['js'] = $this->widgets->get_contents($file_folder . 'js/' . $file->file_name. '.js');
			} else {
				$data = (array)json_decode($file->contents ?? '');
			}
			
			Template::set(array(
				'contents' => $data,
				'id' => $id,
				//'folders' => $this->folders,
				//'views' => $this->views
			));
		}
		Template::set('toolbar_title', lang('manage_widgets'));
		Template::render();		
	}
	
	/*----------------------------------------------------*/
	/*	SCSS Variable Declaration
	/*----------------------------------------------------*/
	/**
	 * Allows editing of SCSS Variables data.
	 *
	 * @return void
	 */
	public function scss_variables() {
		Assets::add_css('bootstrap-colorpicker.min.css');
		Assets::add_js('bootstrap-colorpicker.min.js');
		$id = $this->uri->segment(5);
	
		if (empty ( $id )) {
			Template::set_message ( lang ( 'invalid_id' ), 'error' );
			redirect ( SITE_AREA . '/developer/themes' );
		}		
		
		// in case of view, do not update the scss_values column
		$view = $this->uri->segment(6) ? 1 : '';
		
		// Variables information is stored in the theme_settings table
		// Update theme_settings table instead of themes
		$settings = false; 
		if(is_numeric($id)) {
			//Find record
			$record = $this->themes_model->find($id);
		} else { // Global variables or other variable file from the theme_settings table
			$record = $this->db->select('value AS scss_variables')->get_where('theme_settings', array('name' => escape($id)))->row();
			if(isset($record->value)) {
			    $record->scss_variables = $record->value;
			}
			$settings = true;			
		}

		if(!$record) {
			redirect ( SITE_AREA . '/developer/themes' );
		}
		
		
		// POST DATA HANDLING
		/**
		 * Validation rules are must for field_options
		 * else you will get array to string conversion errors
		 */
		if(isset($_POST['field_options'])) {
			$variables = array();
			foreach($_POST['field_options']['name'] as $key => $value) {
				$variable = array();
				$variable['type'] = $_POST['field_options']['type'][$key];
				$variable['name'] = $_POST['field_options']['name'][$key];
				$variable['title'] = $_POST['field_options']['title'][$key];
				$variable['description'] = $_POST['field_options']['description'][$key];
				$variable['value'] = $_POST['field_options']['value'][$key];
				if(!empty($_POST['field_options']['options'][$key]) && $variable['type'] == 'select') {
					$variable['options'] = $_POST['field_options']['options'][$key];
				}				
				$variables[] = $variable;
			}
			Template::set('options', $variables);
		} else {
			//$variables = $this->themes_model->get_scss_variables($id);
			$variables = json_decode($record->scss_variables ?? '');
			Template::set('options', $variables);
		}
	
		if (isset ( $_POST ['save'] )) {
			$this->auth->restrict ( $this->permissionEdit );
			if ($this->save_variables ($id, $view, $settings )) {
				// Log the activity
				log_activity ( $this->auth->user_id(), lang ( 'act_edit_record' ) . ': ' . $id . ' : ' . $this->input->ip_address (), 'themes' );
				Template::set_message ( sprintf(lang('msg_save_success'), lang('label_variables')), 'success' );
				redirect ( SITE_AREA . '/developer/themes/scss_variables/' . $id . '/' . $view );
			} else {
				Template::set_message ( sprintf(lang('msg_edit_failure'), lang('label_variables')) . $this->themes_model->error, 'danger' );
			}
		}
		Template::set('record', $record);
		Template::set('toolbar_title', lang('label_scss_variables'));
		Template::render();
	}
	
	/**
	 * Save SCSS variables
	 * @param Int	 $id	The ID of the record to update
	 * @param int	 $view	If set to 1 do not update scss_values
	 * @param boolean $settings Variable file is a Global file and it is stored in the theme_settings table
	 */
	private function save_variables($id, $view, $settings = false) {
	    gc_demo_mode();
	    $this->gcrud->stats(array('module' => 'themes'));
		$_POST['theme_id'] = $id; // For unique validation
		$variables = array();
		$values = array();
		if(isset($_POST['field_options'])) {
			foreach($_POST['field_options']['name'] as $key => $value) {
				if(!empty($value)) {
					$data = array();
					// widget/view ID will be appended with each variable to differentiate it from other views/widgets
					// this variable will be removed before processing the variable name
					$field_name = $_POST['field_options']['name'][$key];
					$field_value = $_POST['field_options']['value'][$key];
					
					// $data['theme_id'] = $id;
					$data['type'] = $_POST['field_options']['type'][$key];
					$data['name'] = $field_name;
					$data['title'] = $_POST['field_options']['title'][$key];
					$data['description'] = $_POST['field_options']['description'][$key];
					$data['value'] = $field_value;
					$data['options'] = NULL;
					if(!empty($_POST['field_options']['options'][$key]) && $data['type'] == 'select') {
						$data['options'] = $_POST['field_options']['options'][$key];
					}
					
					$variables[] = $data;
					$values[$field_name] = $field_value;
				}
			}
		}
		$variable_data = array();
		$variable_data['scss_variables']= json_encode($variables);
		if(empty($view)) {
			$variable_data['scss_values'] = json_encode($values);
		}
		
		if(empty($_POST['field_options'])) {
			//$this->db->delete('theme_scssvariables', array('theme_id' => $id));
			if($settings) {
				$this->db->where('name', $id)->update('theme_settings', array('value' => json_encode($variables)));
			} else {
				$this->db->where('id', $id)->update('themes', $variable_data);
			}
			return true;
		} else {
			//Validation Rules
			$rules = $this->themes_model->validation_scss_variables;
			$this->form_validation->set_rules($rules);
			if($this->form_validation->run() == TRUE) {
				// Delete all items first and reinsert them
				// Removed item must also be deleted from the variables table
				// $this->db->delete('theme_scssvariables', array('theme_id' => $id));
				// $this->db->insert_batch('theme_scssvariables', $variables);
				
				// update scss variables
				if($settings) {
					$this->db->where('name', $id)->update('theme_settings', array('value' => json_encode($variables)));
				} else {
					$this->db->where('id', $id)->update('themes', $variable_data);
				}
				return true;
			}
		}
		return FALSE;
	}
	
	/**
	 * Unique scss variable name validation
	 * @param string $str
	 * @return boolean
	 */
	function unique_variable($str) {
		$array = array();
		$names = $this->input->post('field_options[name]');
		// to deal with sentence cases names
		foreach($names as $variable) {
			$array[] = strtolower($variable);
		}
		if(count($array) !== count(array_unique($array))) {
			$this->form_validation->set_message('unique_variable', lang('bf_form_unique'));
			return FALSE;
		} else {
			return TRUE;
		}
	}
	
	/**
	 * The URI segment six determines the theme type VIEW/WIDGET
	 * Set segment to 1 for a VIEW file
	 */
	public function set_scss() {		
		Assets::add_css('bootstrap-colorpicker.min.css');
		Assets::add_js('bootstrap-colorpicker.min.js');
		$id = $this->uri->segment(5);		
		if (empty ( $id )) {
			Template::set_message ( lang ( 'invalid_id' ), 'error' );
			redirect ( SITE_AREA . '/developer/themes' );
		}
		//Find record
		$record = $this->themes_model->find($id);
		if(!$record) {
			redirect ( SITE_AREA . '/developer/themes' );
		}

		// Is View/Layout?
		$view = ($record->type_id == 0 || $record->type_id == 2 ? 1 : '');
		
		// store all widget variables
		$widget_variables = array();
		
		// widget ids variable is used while saving the SCSS values
		$widget_ids = array();
		$widget_names = array();
		
		if($view) {
			// A view can have multiple widgets
			// if a view is added for the first time, then scss_value field is blank
			// in such cases, use widgets default values
			// get all widgets used in the view file
			$widgets = $this->widgets->get_widgets($record->file_url . '.php');
			/*$widgets = array(
					'widgets/navbar',
					'widgets/featured'
			);*/
			
			// deal with stored values
			$view_scss_values = json_decode($record->scss_values ?? '');
			
			// firstly display global variables to the user
			// modify this function accordingly for another variable files stored in the theme_settings table
			// or you can also use SELECT name, value AS scss_variables FROM theme_settings WHERE type = variables
			// use foreach loop to iterate through each file 
			/*$global = $this->db->select('name, value AS scss_variables')->get_where('theme_settings', array('name' => 'global'))->row();
			if($global && !empty($global->scss_variables)) {
				$widget_ids[] = $global->name;
				
				// Global has no scss values, therefore, pass blank array to the get_les_variables method
				$global->id = $global->name;
				$global->scss_values = json_encode(array());
				
				// The user never changed the default values, get default values from the widget
				$widget_variables[$global->name] = $this->themes_model->get_scss_variables($global, $view_scss_values);
				
				// set the widget name
				$widget_names[$global->name] = lang('label_global_variables'); //ucfirst($global->name);
			}*/
			
			// then display view specific SCSS settings
			$widget_ids[] = $record->id;
			$get_widget_scss_variables = $this->themes_model->get_scss_variables($record, $view_scss_values);
			if(!empty($get_widget_scss_variables)) {
				$widget_variables[$record->id] = $get_widget_scss_variables;
			}
			$widget_names[$record->id] = $record->name;
						
			
			if(!empty($widgets)) {
				foreach($widgets as $widget) {
					// get widget id
					$widget_info = $this->themes_model->select('id, name, scss_variables, scss_values')->find_by('file_url', $widget);
					if($widget_info) {
						$widget_ids[] = $widget_info->id;
						$get_scss_variables = $this->themes_model->get_scss_variables($widget_info, $view_scss_values);
						if(!empty($get_scss_variables)) {
							$widget_variables[$widget_info->id] = $get_scss_variables;
						}
						$widget_names[$widget_info->id] = $widget_info->name;
					}
				}
			}
		} else {
			$widget_ids[] = $record->id;
			$widget_variables[$record->id] = $this->themes_model->get_scss_variables($record, json_decode($record->scss_values ?? ''));
			$widget_names[$record->id] = $record->name;
		}
		
		if (isset ( $_POST ['save'] )) {
			$this->auth->restrict ( $this->permissionEdit );
			if ($this->save_view_scss ( $id, $widget_ids )) {
				// Log the activity
				log_activity ( $this->auth->user_id(), lang ( 'act_edit_record' ) . ': ' . $id . ' : ' . $this->input->ip_address (), 'view_scss' );
				Template::set_message ( sprintf(lang('msg_save_success'), lang('label_variables')), 'success' );
				if($record->type_id == 0 || $record->type_id == 2) {
					$this->widgets->process_view($id);
				}
				redirect ( SITE_AREA . '/developer/themes/set_scss/' . $id . '/' . $view );
			} else {
				Template::set_message ( sprintf(lang('msg_edit_failure'), lang('label_variables')) . $this->themes_model->error, 'danger' );
			}
			
		}
		
		Template::set(array(
			'widget_names' => $widget_names,
			'widgets' => $widget_variables,
			'record' => $record,
			'toolbar_title' => lang('default_scss_values')
		));
		Template::render();
	}
	
	/**
	 * AJAX Function to Override SCSS Values
	 */
	public function ajax_override_scss() {
		$id = $this->input->post('theme_id');
		$global = intval($this->input->post('global'));
		if(empty($id)) {
			echo json_encode(array());
		} else {
			$result = $this->themes_model->override_scss_values($id, $global);
			echo json_encode($result);
		}
	}
	
	/**
	 * Save new set values
	 * @param Int	 $id	The ID of the record to update
	 */
	private function save_view_scss($id, $widget_ids) {
	    gc_demo_mode();
	    $this->gcrud->stats(array('module' => 'themes'));
		$data = array();
		$values = array();
		// set validation rules
		if(!empty($_POST['widget'])) {
			foreach($widget_ids as $widget) {
				if(array_key_exists($widget, $_POST['widget'])) {
					foreach($_POST['widget'][$widget] as $key => $value) {
						if($key != 'save') {
							$this->form_validation->set_rules('widget[' . $widget . '][' . $key . ']', $key, 'trim|required');
						}
					}
				}
			}
		}
		
		// save values only on validation success
		if($this->form_validation->run() == TRUE) {
			foreach($widget_ids as $widget_id) {
				if(array_key_exists($widget_id, $_POST['widget'])) {
					foreach($_POST['widget'][$widget_id] as $key => $value) {
						$values[$widget_id][$key] = $value;
					}
				}
			}
			$data['scss_values'] = json_encode($values);
			$this->db->where('id', $id)->update('themes', $data);
			return true;
		}
		return FALSE;		
	}
	
	/*----------------------------------------------------*/
	/*	Save JSON information in the theme_settings table 
	 /*----------------------------------------------------*/
	/**
	 * Common function to save array in JSON format in the settings table
	 *
	 * @return void
	 */
	public function settings_json() {
		$name = $this->uri->segment(5);
	
		if (empty ( $name )) {
			Template::set_message ( lang ( 'invalid_id' ), 'error' );
			redirect ( SITE_AREA . '/developer/themes' );
		}
		$record = $this->db->select('value')->get_where('theme_settings', array('name' => escape($name)))->row();
		if(!$record) {
			redirect ( SITE_AREA . '/developer/themes' );
		}
		// POST Handling
		$this->json_fields(true, $record->value);
		
		if (isset ( $_POST ['save'] )) {
		    gc_demo_mode();
		    $this->gcrud->stats(array('module' => 'themes'));
			$this->auth->restrict ( $this->permissionEdit );
			if ($this->save_settings_json ($name )) {
				log_activity ( $this->auth->user_id(), lang ( 'act_edit_record' ) . ': ' . $name . ' : ' . $this->input->ip_address (), 'themes' );
				Template::set_message ( sprintf(lang('msg_save_success'), lang('label_settings')), 'success' );
				redirect ( SITE_AREA . '/developer/themes/settings_json/' . $name );
			} else {
				Template::set_message ( sprintf(lang('msg_edit_failure'), lang('label_settings')) . $this->themes_model->error, 'danger' );
			}
		}
		Template::set('record', $record);
		Template::set('toolbar_title', lang('manage_fonts'));
		Template::render();
	}
	
	/**
	 * Handle JSON Settings to be shown to the user from the Settings table
	 * @param string $edit
	 * @param array $value
	 */
	private function json_fields($edit = false, $value = null) {
		if(isset($_POST['field_options'])) {
			Template::set('options', $this->json_fields_post());
		} else if($edit && !empty($value)) {
			$options = json_decode($value);
			Template::set('options', $options);
		}
	}
	
	/**
	 * JSON Field POST Handling
	 * @param string $setting - setting name like fonts, global etc.
	 * @param string $save - if save return non empty rows
	 * @return multitype:multitype:NULL  |multitype:
	 */
	private function json_fields_post($setting = null, $save = false) {
		$options = array();
		$fields_to_be_saved = array(); // Fields without blank value
		if(isset($_POST['field_options'])) {			
			foreach($_POST['field_options']['name'] as $key => $value) {
				$option = array();
				$option['name'] = $_POST['field_options']['name'][$key];
				$option['type'] = $_POST['field_options']['type'][$key];
				if($save && !empty($option['name'])) {
					$fields_to_be_saved[] = $option;
				}
				$fonts[] = $option;
			}			
		}
		if($save) {
			return $fields_to_be_saved;
		} else {
			return $fonts;
		}
	}
	
	/**
	 * Save settings JSON format
	 * @param Int	 $name	The name of the record to update
	 */
	private function save_settings_json($name) {
		$data = array();
		// Validations
		$this->form_validation->set_rules('field_options[name][]', 'lang:label_name', 'trim|required|is_array');
		if ($this->form_validation->run() === false) {
			return false;
		}		
		$data['value'] = json_encode($this->json_fields_post(null, true));
		$return = $this->db->where('name', $name)->update('theme_settings', $data);
		return $return;
	}
	
	/**
	 * Allow user to view all Static Files
	 */
	public function static_files() {
	    $files = $this->widgets->static_files();
		Template::set(
			array(
				'toolbar_title' => lang('static_files'),
				'records' => $files
			)
		);
		Template::render();
	}
	
	/**
	 * Allow user to edit static files like _variables.scss, _utilties.scss etc
	 * @param string	$filename	The name of the file to update
	 */
	public function static_file($filename) {
		// display file contents within the editor
		Assets::add_js('edit_area/edit_area_full.js');
		Assets::add_js('editAreaLoader.init({
		id: "contents"	// id of the textarea to transform		
		,start_highlight: true	// if start with highlight
		,allow_resize: "both"
		,allow_toggle: false
		,word_wrap: false
		,language: "en"
		,syntax: "css"	
	});', 'inline');
		
		$files = $this->widgets->static_files();
		$contents = '';
		$file_info = array();
		foreach($files as $file) {
			if($file['name'] == $filename) {
				$contents = $this->widgets->get_contents($file['url']);
				$file_info = array(
					'name' => $file['name'],
					'extension' => $file['extension'],
					'url' => $file['url'],
					'path' => $file['path']
				);
			}
		}
		if(empty($contents)) {
			Template::set_message ( lang ( 'invalid_file' ), 'error' );
			redirect ( SITE_AREA . '/developer/themes' );
		}
		$fileinfo = $file_info['name'] . '.' . $file_info['extension'];
		Template::set(
			array(
				'toolbar_title' => sprintf(lang('file_update'), $fileinfo),
				'contents' => $contents,
				'filename' => $fileinfo
			)
		);
		// Save file
		if (isset ( $_POST ['save_write'] )) {
		    gc_demo_mode();
		    $this->gcrud->stats(array('module' => 'themes'));
			if(isset($_POST['contents'])) {
				$this->widgets->save_file($file_info['path'] . $file_info['name'], $file_info['extension'], $_POST['contents']);
				Template::set_message ( sprintf(lang('msg_save_success'), lang('label_file')), 'success' );
			}
		}
		Template::render();
	}
	
	/**
	 * Allow Admin to Add New Settings e.g. Schemes, Themes etc.
	 */
	public function settings($type = 'schemes', $offset = 0, $name = null) {
		$this->lang->load('settings/settings');
		if($type == 'typography') {		    
		    Assets::add_css('selectize.bootstrap3.css');
		    Assets::add_js('selectize.min.js');
		}
		if($type == 'themes') { // clear themes options cache		    
		    clear_cache('themes');
		}
		// Allow Admin to Make a Copy of the Scheme
		if (($type == 'schemes' || $type == 'typography') && !empty ( $_POST ['clone'] )) {
		    $record = $this->db->get_where('theme_settings', array('name' => escape($_POST['clone']), 'type' => $type))->row();
			if($record) {
				$clone_data = array();
				foreach($record as $key => $value) {
					if($key == 'is_default') {
						$clone_data['is_default'] = 0;
					} else if($key == 'name') {
						$clone_data[$key] = $value . '_copy_' . rand(1, 100);
					} else {
						$clone_data[$key] = $value;
					}
				}
				$this->db->insert('theme_settings', $clone_data);
				Template::redirect(current_url());
			}
		}
		
		// Add/Edit Variable
		if (isset($_POST['save'])) {
		    gc_demo_mode();
		    $this->gcrud->stats(array('module' => 'themes'));
			if(!empty($name)) {
				$record_count = $this->db->where(array('name' => $name, 'type' => $type))->count_all_results('theme_settings');
				// update the existing name scheme
				if($record_count > 0) {
				    // recompile the theme if the theme is default and navbar and footer changed
				    $default_theme = theme_settings();
					if($this->save_setting('update', $type, $name)) {
					    if($type == 'themes') {
							// Compile global.scss file
						    $this->widgets->compile_global();
						    if($name == $default_theme['theme']) { // recompile the theme
						        $settings_data = $_POST;
						        if($settings_data['navbar'] != $default_theme['navbar'] || $settings_data['footer'] != $default_theme['footer']) {
						            clear_cache('default_theme');
						            $this->widgets->set_default('themes', $name, false, $settings_data);
						        }
						    }
						}
						Template::set_message ( sprintf(lang('msg_save_success'), lang($type . '_singular')), 'success' );
						Template::redirect(site_url(SITE_AREA ."/developer/themes/settings/" . $type));
					} else {
						Template::set_message(sprintf(lang('msg_edit_failure'), lang($type . '_singular')), 'danger');
					}
				} else {
					Template::redirect(site_url(SITE_AREA ."/developer/themes/settings/" . $type));
				}
			} else { // insert a new scheme				
				if($this->save_setting('insert', $type)) {
					Template::set_message ( sprintf(lang('msg_save_success'), lang($type . '_singular')), 'success' );
					Template::redirect(site_url(SITE_AREA ."/developer/themes/settings/" . $type));
				} else {
					Template::set_message(sprintf(lang('msg_create_failure'), lang($type . '_singular')), 'danger');
				}
			}
		} else if (isset($_POST['delete'])) { // delete view/widget
		    gc_demo_mode();
		    $this->gcrud->stats(array('module' => 'themes'));
			$this->auth->restrict($this->permissionDelete);
			$checked = $this->input->post('checked');
			if (is_array($checked) && count($checked)) {
				$result = FALSE;
				foreach ($checked as $pid) {
					$result = $this->db->delete('theme_settings', array('name' => $pid, 'type' => $type));
				}
				if ($result) {
					Template::set_message(count($checked) .' '. sprintf(lang('msg_delete_success'), lang($type . '_plural')), 'success');
					redirect(SITE_AREA .'/developer/themes/settings/' . $type);
				} else {
					Template::set_message(sprintf(lang('msg_delete_failure'), lang($type . '_plural')), 'danger');
				}
			}
		}
		
		// Process Theme
		if(isset($_POST['process_theme'])) {
		    gc_demo_mode();
		    $this->gcrud->stats(array('module' => 'themes'));
			$setting_name = $_POST['process_theme'];
			// Do not process theme specific files multiple times
			// i.e. the below given session variable is used
			// A default value (setting_name) is also set to check the session existence
			$this->session->set_userdata('theme_scheme_process', array($setting_name));
			switch($type) {
				case 'themes':
					// Perhaps, the older theme is using an Ajax Login box
					// i.e. unset the ajax_login session to avoid redirection to the older login and register method.
					$this->session->unset_userdata('ajax_login');
					$theme_name = $_POST['process_theme'];
					$this->widgets->set_default('themes', $theme_name);
					break;
				case 'schemes':
				    $scheme_name = $_POST['process_theme'];
				    $this->widgets->set_default('schemes', $scheme_name);	
				    break;
			}
		}
		
		// Edit color scheme
		if(!empty($name)) {
			$record = $this->db->get_where('theme_settings', array('name' => $name, 'type' => $type))->row();
			if($record) {
				Template::set('record', $record);
			}
		}
		
		$this->db->limit($this->limit, $offset);
		$records = $this->db->select('name, is_default, value')->get_where('theme_settings', array('type' => $type))->result_array();
		
		$counts = $this->db->where(array('type' => $type))->count_all_results('theme_settings');
		
		// Pagination
		$this->load->library('pagination');
		
		$pager['base_url'] = site_url(SITE_AREA ."/developer/themes/settings/" . $type ."/");
		$pager['total_rows'] = $counts;
		$pager['per_page'] = $this->limit;
		$pager['uri_segment']	= 6;
		
		$this->pagination->initialize($pager);
		
		Template::set(array(
			'records' => $records,
			'type' => $type,
			'toolbar_title' => lang($type . '_plural')			
		));
		Template::render();
	}
	
	/**
	 * On Theme process, update default status
	 * @param string $name - Name of the Theme
	 * @param string $value - New Default Value
	 */
	private function update_theme_default($theme, $name, $value) {
		$existing_value = (array)json_decode($theme->value ?? '');
		$new_values = array();
		$new_values['default'] = $value;
		$new_values['image_url'] = !empty($existing_value['image_url']) ? $existing_value['image_url'] : '';
		$this->db->where('name', $name)->update('theme_settings', array('value' => json_encode($new_values)));
	}
	
	/**
	 * Save setting e.g. scheme/theme
	 * @param string $type
	 * @param string $name
	 * @return boolean|array
	 */
	private function save_setting($query = 'insert', $type = null, $name = NULL) {
		/*if ($query == 'update') {
			$_POST['name'] = $name;
		}*/
	    // unique[theme_settings.name' . (!empty($name) ? ',theme_settings.name' : '') . ']
	    $_POST['current_name'] = $name;
		$this->form_validation->set_rules('name', 'lang:label_name', 'trim|required|alpha_dash|callback_theme_name_check');
		if($type == 'themes') {
			$this->form_validation->set_rules('logo_width', 'lang:logo_width', 'trim|required|is_numeric');
			$this->form_validation->set_rules('logo_height', 'lang:logo_height', 'trim|required|is_numeric');
			$this->form_validation->set_rules('navbar', 'lang:theme_navbar', 'trim');
			$this->form_validation->set_rules('footer', 'lang:theme_footer', 'trim');
		} else if($type == 'typography') {
		    $this->form_validation->set_rules('classes', 'lang:label_classes', 'trim|required');
		    $this->form_validation->set_rules('description', 'lang:label_description', 'trim|required');
		}
		if ($this->form_validation->run() === false) {
			return false;
		}
		$data = array();
		$data['name'] = strtolower($this->input->post('name'));
		$data['type'] = $type;
		if ($query == 'insert') {
			$value = array();
			if($type == 'themes') {
				$value['default'] = 0;
				$value['image_url'] = $this->input->post('image_url');
				$value['logo_width'] = $this->input->post('logo_width');
				$value['logo_height'] = $this->input->post('logo_height');
				$value['navbar'] = $this->input->post('navbar');
				$value['footer'] = $this->input->post('footer');
				$value['options'] = json_encode($this->input->post('options'));
				// save theme specific scss file
				$this->widgets->save_theme_scss($data['name']);
			} else if($type == 'typography') {
			    $value['classes'] = $this->input->post('classes');
			    $value['description'] = $this->input->post('description');
			}
			$data['value'] = json_encode($value);
			$this->db->insert('theme_settings', $data);
			$return = true;
		} elseif ($query == 'update') {
			if($type == 'themes') {
				$record = $this->db->get_where('theme_settings', array('name' => $name, 'type' => $type))->row();
				$values = json_decode($record->value ?? '');
				$value = array();
				if($record) {
					$value['default']	= !empty($values->default) ? $values->default : 0;
					$value['image_url'] = $this->input->post('image_url');
					$value['logo_width'] = $this->input->post('logo_width');
					$value['logo_height'] = $this->input->post('logo_height');
					$value['navbar'] = $this->input->post('navbar');
					$value['footer'] = $this->input->post('footer');
					$value['options'] = json_encode($this->input->post('options'));
				}
				$data['value'] = json_encode($value);
			} else if($type == 'typography') {
			    $value['classes'] = $this->input->post('classes');
			    $value['description'] = $this->input->post('description');
			    $data['value'] = json_encode($value);
			}
			$return = $this->db->where('name', $name)->update('theme_settings', $data);
		}
		return $return;
	}
	
	/**
	 * Callback Theme/Scheme Name Check
	 * @return boolean
	 */
	public function theme_name_check() {
	    $current_name = escape($this->input->post('current_name'));
	    $new_name = escape($this->input->post('name'));
	    $count = $this->db->where("name = '" . $new_name . "' AND name != '" . $current_name . "'")->count_all_results('theme_settings');
	    if($count == 0) {
	        return true;
	    } else {
	        $this->form_validation->set_message('theme_name_check', sprintf(lang('bf_form_unique'), lang('label_name')));
	        return false;
	    }
	}
	
	/**
	 * The URI segment six determines the theme type VIEW/WIDGET
	 * Set segment to 1 for a VIEW file
	 */
	public function settings_scss() {		
		Assets::add_css('bootstrap-colorpicker.min.css');
		Assets::add_js('bootstrap-colorpicker.min.js');
		$name = $this->uri->segment(5);
	
		if (empty ( $name )) {
			Template::set_message ( lang ( 'invalid_name' ), 'error' );
			redirect ( SITE_AREA . '/developer/themes' );
		}
		//$this->db->or_like(array('type' => 'schemes'));
		$record = $this->db->select('value')->get_where('theme_settings', array('name' => escape($name), 'type' => 'schemes'))->row();
		if(!$record) {
			redirect ( SITE_AREA . '/developer/themes' );
		}
		
		$global = $this->db->select('value AS scss_variables')->get_where('theme_settings', array('name' => 'global'))->row();
		
		// get_scss_variables format
		$variables = array(
			'id' => $name,
			'scss_values' => empty($record->value) ? json_encode(array()) : $record->value,
			'scss_variables' => $global->scss_variables
		);
		
		// store all widget variables
		$widget_variables = array();
		
		// Get all global variables
		$widget_variables = $this->themes_model->get_scss_variables((object)$variables, array());
		
		// get all default views to apply the color scheme
		$default_views = $this->themes_model->select('id, name, file_folder')->find_all_by(array('type_id !=' => 1, 'is_default' => 1));
		
		if (isset ( $_POST ['save'] )) {
		    gc_demo_mode();
		    $this->gcrud->stats(array('module' => 'themes'));
			$this->auth->restrict ( $this->permissionEdit );
			if ($this->save_settings_scss ( $name )) {				
				// Log the activity
				log_activity ( $this->auth->user_id(), lang ( 'act_edit_record' ) . ': ' . $name . ' : ' . $this->input->ip_address (), 'theme_settings' );
				Template::set_message ( sprintf(lang('msg_save_success'), lang('label_variables')), 'success' );
				Template::redirect ( SITE_AREA . '/developer/themes/settings_scss/' . $name );
			} else {
				Template::set_message ( sprintf(lang('msg_edit_failure'), lang('label_variables')) . $this->themes_model->error, 'danger' );
			}				
		}
		
		Template::set(array(
			'options' => $widget_variables,
			'name' => $name,
			'record' => $record,
			'records' => $default_views,
			'toolbar_title' => ucwords(str_replace('_', ' ', $name)) . ' ' . lang('schemes_singular')
		));
		Template::render();
	}
	
	/**
	 * Save new color schemes values
	 * @param Int	 $id	The ID of the record to update
	 */
	private function save_settings_scss($name) {
		$data = array();
		$values = array();
		$return = false;
		// set validation rules
		if(!empty($_POST)) {
			foreach($_POST['widget'][$name] as $key => $value) {
				if($key != 'save') {
					$this->form_validation->set_rules('widget[' . $name . '][' . $key . ']', $key, 'trim|required');
				}
			}
		}
	
		// save values only on validation success
		if($this->form_validation->run() == TRUE) {
			// Update Color Scheme value in the VIEW file, else the next process of the same view will use its saved default values 
			$update_scheme_scss = $this->input->post('update_saved_scss') ? 1 : 0;
			
			foreach($_POST['widget'][$name] as $key => $value) {
				$values[$key] = $value;
			}
			// settings name must not be same
			$return = $this->db->where('name', $name)->update('theme_settings', array('value' => json_encode($values)));
			
			// Apply this Color Scheme to the checked views
			if($return) {
				$checked = $this->input->post('checked');
				// Do not process theme specific files multiple times
				// i.e. the below given session variable is used
				// A default value (setting_name) is also set to check the session existence
				$this->session->set_userdata('theme_scheme_process', array($name));
				if (is_array($checked) && count($checked)) {
					foreach($checked as $view) {
						$this->widgets->process_view($view, $values, $update_scheme_scss);
						$this->widgets->set_default('views', $view);
					}
				} else {
				    // rest the color scheme
				    $this->widgets->set_default('schemes', $name);
				}
				$this->session->unset_userdata('theme_scheme_process');
			}
		}		
		return $return;
	}
	
	/**
	 * Allow Admin to create a new Grid using Grid Editor plugin
	 */
	public function grid_editor() {
		Template::render('blank');
	}
	
	/**
	 * Site Settings
	 */
	public function site_settings() {
		// save the settings to the DB
		$this->load->model('settings/settings_model');
		if (isset($_POST['save'])) {
		    gc_demo_mode();
		    $this->gcrud->stats(array('module' => 'themes'));
			$data = array();
			$updated = $this->settings_model->update_batch($data, 'name');
			if ($updated) {
				Template::set_message(sprintf(lang('msg_save_success'), lang('gc_menu_settings')), 'success');
			} else {
				Template::set_message(sprintf(lang('msg_edit_failure'), lang('gc_menu_settings')), 'danger');
			}
		}
		// Display themes dropdown to the user
		$themes = $this->db->select('name')->get_where('theme_settings', array('type' => 'themes'))->result();
		Template::set(array(
			'toolbar_title' => lang('manage_widgets'),
			'themes' => $themes
		));
		Template::render();
	}
	
	/**
	 * Replace layouts with widgets
	 */
	private function replace_layout() {
		$result = $this->themes_model->find_all();
		if($result) {
			foreach($result as $row) {
				$new_url = str_replace('layouts', 'widgets', $row->file_url);
				$this->db->where('id', $row->id)->update('themes', array('file_url' => $new_url));
			}
		}
	}
	
	/**
	 * Allow Admin to Change Links
	 * @param string $filter
	 * @param number $offset
	 */
	public function links($filter = 'all', $offset = 0) {
		// Do we have any actions?
		if ($action = $this->input->post('submit')) {
			$checked = $this->input->post('checked');
		
			switch(strtolower($action)) {
				case 'active':
					$this->change_status($checked, 1);
					break;
		
				case 'inactive':
					$this->change_status($checked, 0);
					break;
		
				case 'delete':
					$this->delete($checked);
					break;
			}
		}
		// Filters
		$filter_type = $filter;
		
		switch($filter_type) {
			case 'member':
				$where['location'] = 1;
				break;
					
			case 'header':
				$where['location']= " T.type_id = 0";
				break;
			
			case 'footer':
				$where['location']= " T.type_id = 0";
				break;
		
			case 'all':
				// Nothing to do
				break;
		
			default:
				show_404("themes/links/$filter/");
		}
		
		// Search Handling
		$searchterm = $this->gc->search('links', $this->uri->segment(6), '/developer/themes/links/');
		if(!empty($searchterm)) {
			if(!empty($where)) {
				$where .= " AND";
			}
			$where .= " (T.id LIKE '%". $searchterm ."%' OR T.name LIKE '%". $searchterm ."%')";
		}
		
		// Sorting
		$sort_by = 'T.id, T.is_default DESC';
		if(isset($_POST['sort']) && $_POST['sort'] == 'view') {
			$order_by = $this->gc->sort('links', 'TV');
		} else {
			$order_by = $this->gc->sort('links', 'T');
		}
		if(!empty($order_by)) {
			$sort_by = $order_by;
		}
		
		$records = $this->db->get('links')->result();
		$total_records = $this->db->count_all('links');
		
		// Pagination
		$this->load->library('pagination');
		
		$pager['base_url'] 		= site_url(SITE_AREA ."/developer/themes/links/$filter/");
		$pager['total_rows'] 	= $total_themes;
		$pager['per_page'] 		= $this->limit;
		$pager['uri_segment'] 	= 6;
		
		$this->pagination->initialize($pager);
		
		Template::set('index_url', site_url(SITE_AREA .'/developer/themes/links/'));
		Template::set('filter_type', $filter);
		
		Template::set(array(
			'records' => $records,
			'toolbar_title' => lang('manage_links')
		));
		
		Template::render();
	}
	
	/**
	 * Easily change a Color Scheme 
	 * Find and replace values in a Color Scheme
	 * 
	 * Format: "#ffd400:#8950404;#1d1d1d:#d34ddd;";
	 */
	public function color_switcher() {
		Assets::add_css('bootstrap-colorpicker.min.css');
		Assets::add_js('bootstrap-colorpicker.min.js');
		if (isset($_POST['submit']) && $_POST['submit'] == 'submit') {
		    gc_demo_mode();
		    $this->gcrud->stats(array('module' => 'themes'));
			$this->form_validation->set_rules('values', 'lang:label_values', 'trim|required');			
			if ($this->form_validation->run() === true) {
				$values = $this->input->post('values');
				$source_scheme = $this->input->post('source');
				$destination_scheme = $this->input->post('destination');
				$option_result = array();
				// get all options
				$options_array = array_map('trim', explode(';', $values ?? ''));
				if($options_array) {
					foreach($options_array as $option_array) {
						if(!empty($option_array)) {
							$option_value = array_map('trim', explode(':', $option_array));
		    				$option_result[strtolower($option_value[0])] = (isset($option_value[1]) ? strtolower($option_value[1]) : '');
						}
					}
				}
				if(!empty($option_result)) {
					// Get the existing color scheme values
					$result = $this->db->get_where('theme_settings', array('name' => $source_scheme, 'type' => 'schemes'))->row();
					if($result) {
						$scheme = $result->value;
						foreach($option_result as $key => $value) {
							$scheme = str_replace($key, $value, $scheme);
						}
						// Update the Destination Color Scheme
						$this->db->where(array('name' => $destination_scheme, 'type' => 'schemes'))->update('theme_settings', array('value' => $scheme));
						Template::set_message(sprintf(lang('msg_action_success'), lang('label_scheme'), lang('label_updated')), 'success');
						Template::redirect(current_url());
					}
				}
			}
		}
		$unique_colors = array();
		// Get Unique Colors Values from an Existing Scheme
		if (isset($_POST['submit']) && $_POST['submit'] != 'submit') {
		    gc_demo_mode();
		    $this->gcrud->stats(array('module' => 'themes'));
			$scheme = $this->input->post($_POST['submit']);
			$result = $this->db->get_where('theme_settings', array('name' => $scheme, 'type' => 'schemes'))->row();
			$color_values = json_decode($result->value ?? '');
			$unique_colors = array();
			foreach($color_values as $color_value) {
				if(substr( $color_value, 0, 1 ) === "#" && !in_array($color_value, $unique_colors)) {
					$unique_colors[] = $color_value;
				}
			}
		}
		
		// Display all color schemes to the User
		$schemes = array();
		$schemes_result = $this->db->get_where('theme_settings', array('type' => 'schemes'))->result();
		if(!empty($schemes_result)) {
			foreach($schemes_result as $scheme_result) {
				$schemes[$scheme_result->name] = ucwords(str_replace('_', ' ', $scheme_result->name));
			}
		}
		Template::set(array(
			'schemes' => $schemes,
			'unique_colors' => $unique_colors,
			'toolbar_title' => lang('color_switcher')
		));
		Template::render();
	}
	
	/**
	 * Export Theme Files
	 */
	public function export($id, $type = 'views', $excel = 0) {
	    if($excel == 1) {
	        $this->widgets->export_excel($id, $type);
	    } else {
	        $this->widgets->migrate($type, $id);
	    }
	}
	
	/**
	 * Import Theme Files
	 */
	public function import() {
	    $this->load->model('migrate/migrate_model');
	    if(isset($_POST['save'])) {
	        gc_demo_mode();
	        $this->gcrud->stats(array('module' => 'themes'));
	        $insert_id = $this->widgets->save_import_file();
	        $file = $this->input->post('online_file_path') ? $this->input->post('online_file_path') : $this->input->post('uploaded_file');
	        if(!$insert_id) {
	            Template::set_message(sprintf(lang('msg_migrate_failure'), lang('label_import')) . $this->migrate_model->error, 'danger');
	        } else if(!empty($file)) {
	            $extension = substr( $file, strrpos( $file, '.' ) + 1);
	            if($extension == 'zip') {
	                // Extract Zip file
	                // 'themes' is the delimiter, it is the folder structure themes/public_html/uploads/import
	                extract_zip(FCPATH . '/uploads/imports/' . $file, 'themes');
	                $success = $this->widgets->import_dump();
	            } else {
	                $success = $this->widgets->import_excel();
	            }	            
	            
	            // Update import status in the migrations table
	            if($success) {
	               // $this->migrate_model->update($insert_id, array('status' => 1));
	            
    	            // Log the activity
    	            // log_activity($this->auth->user_id(), lang('act_create_record') .': '. $insert_id .' : '. $this->input->ip_address(), 'migrate');
    	            Template::set_message(sprintf(lang('msg_migrate_success'), lang('label_import')), 'success');
    	            Template::redirect(current_url());
	            }
	        }
	    }
	    Template::set('toolbar_title', lang('label_import') . ' ' . lang('view_widget'));
	    Template::render();
	}
	
	/**
	 * Export Mobirise Theme
	 */
	public function mobirise() {
	    $fileName = FCPATH . 'uploads/imports/themes/index.html';
	    $str = 'assets/';
	    $this->widgets->read_copy_files($fileName, $str, 1);
	}
	
	/**
	 * Manage Theme Layouts
	 * @param string $module
	 * @param string $filter
	 * @param number $offset
	 */
	public function manage($module = 'layouts', $filter = 'all', $offset = 0) {
	    $extra = array(
	        'model' => 'themes',
	        'context' => 'developer',
	        'fields' => $module,
	        'prefix' => 'theme'
	    );
	    $extra['actions'] = array( // Extra buttons
	        array('type' => 'link', 'link' => site_url(SITE_AREA . '/developer/themes/form/' . $module . '/edit/%s'), 'label' => '<i class="fas fa-edit fa-fw mr-1"></i>' . lang('edit'), 'class' => 'btn btn-xs btn-primary'),
	    );
	    if($module == 'blocks') {
	        $extra['actions'][] = array('type' => 'link', 'link' => site_url(SITE_AREA . '/developer/themes/manage/demos/%s'), 'label' => '<i class="fas fa-rocket fa-fw mr-1"></i>' . lang('theme_demos'), 'class' => 'btn btn-xs btn-danger', 'condition' => array(array('name' => 'block_type_id', 'match' => 1, 'value' => 0), array('name' => 'block_type_id', 'match' => 1, 'value' => 3)));
	    }
	    if($module == 'demos') {	        
	        if(is_numeric($filter)) {
	            // theme_block_id is passed
	            $theme_block_id = $filter;
	            $filter = 'all';
	            $this->session->set_userdata('theme_demo_block_id', $theme_block_id);
	        }
	        if($this->session->userdata('theme_demo_block_id')) {
	            $extra['query_where'] = 'block_id = ' . $this->session->userdata('theme_demo_block_id');
	        }
	    }
	    $this->gc->module_view($module, $filter, $offset, 'admin', $extra);
	}
	
	/**
	 * Add/Edit Themes Data
	 *
	 * @param string $module
	 * @param string $action
	 *
	 * @return void
	 */
	public function form($module = 'layouts', $action = 'create') {
	    Assets::add_module_js('themes', 'layouts.js');
	    $params = array(
	        'model' => 'themes',
	        'context' => 'developer',
	        'redirect_url' => site_url(SITE_AREA . '/developer/themes/manage/' . $module),
	        'prefix' => 'theme',
	        'fields_method' => $module
	    );
	    $widgets_dropdown = array();
	    $widgets = $this->db->select('id, name')->where('type_id', 1)->order_by('name', 'desc')->get('themes')->result();
	    if($widgets) {
	        foreach($widgets as $widget) {
	            $widgets_dropdown[$widget->id] = $widget->name;
	        }
	    }
	    $custom_fields = array(array('name' => 'widget', 'type' => 'dropdown', 'label' => lang('label_name'), 'options' => $widgets_dropdown, 'class' => 'select-widget'), array('name' => 'priority', 'type' => 'text', 'label' => lang('label_priority'), 'default_value' => 0), array('name' => 'options', 'type' => 'button', 'label' => lang('label_options'), 'class' => 'btn btn-primary btn-sm widget-options'));
	    $themes = $this->db->select('name as id, name')->where('type', 'themes')->get('theme_settings')->result();
	    if($themes) {
	        foreach($themes as $key => $value) {
	            $themes[$key]->name = ucwords(str_replace('_', ' ', $value->name));
	        }
	    }
	    $layouts = array();
	    foreach($this->layouts as $layout => $label) {
	        $layouts[] = (object) array('id' => $layout, 'name' => $label);
	    }
	    if($module == 'folders') {
	        clear_cache('theme_folders');
	    }
	    Template::set(array(
	        'custom_fields' => $custom_fields,
	        'themes' => $themes,
	        'layouts' => $layouts,
	        'form_extra' => $this->load->view('developer/_widget_fields', array(), true)
	    ));
	    $this->gcrud->form($module, $action, 'admin', $params);
	}
	
	/**
	 * Theme Unique Layout Design
	 * @return boolean
	 */
	public function unique_layout() {
	    $id = (int) $this->input->post('id');
	    $layout = $this->input->post('layout');
	    $theme = $this->input->post('theme');
	    $where = array('theme' => $theme, 'layout' => $layout);
	    if($id > 0) {
	        $where['id !='] = $id;
	    }
	    $counts = $this->db->where($where)->count_all_results('theme_layouts');
	    if($counts == 0) {
	        return true;
	    } else {
	        return false;
	    }
	}
	
	/**
	 * Prepare Layouts
	 * @param string $layout
	 */
	public function prepare_layout($layout) {
	    $default_theme = $this->db->where(array('is_default' => 1, 'type' => 'themes'))->get('theme_settings')->row();
	    if($default_theme) {
	        // get layout
	        $layout = $this->db->where(array('theme' => $default_theme->name, 'layout' => $layout))->get('theme_layouts')->row();
	        if($layout) {
	            // get blocks
	            $blocks = array(
	                array(
	                    'widget' => 'widgets/features/_block_02',
	                    'data' => "'group_id' => 5",
	                    'position' => 'navbar'	                    
	                ),
	                array(
	                    'widget' => 'widgets/features/_block_01',
	                    'data' => "",
	                    'position' => 'navbar'
	                ),
	                array(
	                    'widget' => 'widgets/subscribers/_form_01',
	                    'data' => "",
	                    'position' => 'position_02'
	                ),
	                array(
	                    'widget' => 'widgets/subscribers/_form_01',
	                    'data' => "",
	                    'position' => 'position_03'
	                ),
	                array(
	                    'widget' => 'content',
	                    'data' => "",
	                    'position' => 'position_03'
	                ),
	                array(
	                    'widget' => 'message',
	                    'data' => "",
	                    'position' => 'position_03'
	                )
	            );
	            $positions = json_decode($layout->value, true);
	            if(!empty($positions)) {
    	            // order by priority
    	            usort($positions, array("developer", "order_by_priority"));
    	            $content = array();
    	            foreach($positions as $position) {
    	                $block_contents = array();
    	                foreach($blocks as $block) {
    	                    if($block['position'] == $position['name']) {
    	                        switch($block['widget']) {
    	                            case 'content':
    	                                $block_contents[] = "echo isset(\$content) ? \$content : Template::content();";
    	                                break;
    	                            case 'message':
    	                                $block_contents[] = "echo Template::message();";
    	                                break;
    	                            default:
    	                                $block_contents[] = "Template::block('" . $position['name'] . "', '" . $block['widget'] . "', array(" . $block['data']. "), true);";
    	                        }
    	                        
    	                    }
    	                }
    	                if($block_contents) { // write the content only if block contents are set
    	                    // in case of left, right and center blocks find the positions e.g. div.col-sm-2 - 1, div.col-sm-7 - 2, div.col-sm-3 - 3
    	                   $content[] = emmet($position['class']);
    	                }
    	            }
    	            dump($block_contents);
    	            // get widgets and write using blocks
    	            // write content in the file
    	            dump($content);
	            }
	        }
	    }
	}
	
	/**
	 * Order By Priority
	 * @param string $a
	 * @param string $b
	 * @return number
	 */
	public function order_by_priority($a, $b) {
	    return ($a["priority"]<=$b["priority"])?-1:1;
	}
	
	/**
	 * Get Widget Fields to display them in the modal window
	 * @param number $id
	 */
	public function widget_fields() {
	    $id = (int) $this->input->get('id');
	    $widget_title = lang('widget_params');
	    $show_submit = 0;
	    $widget_cover = '';
	    $widget_no_params = '<div class="alert alert-danger">' . lang('widget_no_params') . '</div>';
		$row = $this->db->select('id, name, image_url, file_folder, file_url, params')->where('id', $id)->get('themes')->row();
		if(!$row) {
		    die(json_encode(array('title' => $widget_title, 'fields' => $widget_no_params, 'submit' => $show_submit, 'cover' => $widget_cover)));
		}
	    $record = array();
	    $fields_html = array();
	    if($row) {
	        if(!empty($row->image_url)) {
	            $widget_cover = '<img src="' . gc_image_url('assets/images/' . $row->image_url, false) . '" class="img-fluid">';
	        }
	        $layout_id = $this->session->userdata('layout_design_id');
	        $saved_layout_widgets = (array) $this->session->userdata('layout_widgets');
	        $widget_params = isset($saved_layout_widgets[$layout_id][$id]) ? $saved_layout_widgets[$layout_id][$id] : array();
	        $widget_title = $row->id . ' - ' . $row->name;
	        $params = json_decode($row->params ?? '', true);
	        if(!empty($params)) {
	            if(empty($widget_params)) {
	                foreach($params as $default_param) {
	                    $widget_params[$default_param['name']] = $default_param['value'];
	                }
	            }
				$this->load->model('fields/fields_model');
	            // $fields_html[] = '<form role="form" id="widgetForm" method="post" data-toggle="validator">';//form_open('', 'class="form-horizontal" id="widgetForm"', false);
	            $fields_html[] = form_hidden('widget_id', $id);
	            foreach($params as $field) {
					$field_html = '';
					switch($field['name']) {
						case 'product_id':
						    $field_html = gc_form_element('dropdown', array('name' => $field['name'], 'label_width' => 'col-sm-4', 'help' => lang('param_desc_' . $field['name']), 'options' => $this->fields_model->dropdown_options('products', true, 'name', 'id', true)), $widget_params, lang('param_' . $field['name']) . lang('bf_form_label_required'), 'col-sm-8');
							break;
						case 'group_id':
						    if($row->file_url == 'widgets/others/_banners' || $row->file_folder == 'headers') {
						        $group_options = $this->fields_model->dropdown_options('banner_groups', true, 'name', 'id', true);
						    } else if($row->file_folder == 'subscribers') {
						        $group_options = $this->fields_model->dropdown_options('subscriber_groups', true, 'name', 'id', true);
						    } else {
						        $group_options = $this->fields_model->dropdown_options('product_content_groups', true, 'name', 'id', true);
						    }
						    $field_html = gc_form_element('dropdown', array('name' => $field['name'], 'label_width' => 'col-sm-4', 'help' => lang('param_desc_' . $field['name']), 'options' => $group_options), $widget_params, lang('param_' . $field['name']) . lang('bf_form_label_required'), 'col-sm-8');
							break;
						case 'brand_id':
						    $field_html = gc_form_element('dropdown', array('name' => $field['name'], 'label_width' => 'col-sm-4', 'help' => lang('param_desc_' . $field['name']), 'options' => $this->fields_model->dropdown_options('product_brands', true, 'name', 'id', true)), $widget_params, lang('param_' . $field['name']) . lang('bf_form_label_required'), 'col-sm-8');
							break;
						case 'ids':
						    $field_html = gc_form_element('dropdown', array('name' => $field['name'], 'label_width' => 'col-sm-4', 'help' => lang('param_desc_' . $field['name']), 'options' => $this->fields_model->dropdown_options('banner_sliders', false, 'title', 'id', true)), $widget_params, lang('param_' . $field['name']) . lang('bf_form_label_required'), 'col-sm-8');
							break;
						case 'block':
							$block_options = array(
								'featured' => lang('featured'),
								// 'home_featured' => 'home_featured',
								'pouplar' => lang('popular'),
								'recently_added' => lang('recently_added'),
								'recently_viewed' => lang('recently_viewed')
							);
							$field_html = gc_form_element('dropdown', array('name' => $field['name'], 'label_width' => 'col-sm-4', 'help' => lang('param_desc_' . $field['name']), 'options' => $block_options), $widget_params, lang('param_' . $field['name']) . lang('bf_form_label_required'), 'col-sm-4');
							break;
						case 'cols':
							for($i = 1; $i <= 12; $i++) {
								$col_options[$i] = $i; 
							}
							$field_html = gc_form_element('dropdown', array('name' => $field['name'], 'label_width' => 'col-sm-4', 'help' => lang('param_desc_' . $field['name']), 'options' => $col_options), $widget_params, lang('param_' . $field['name']) . lang('bf_form_label_required'), 'col-sm-2');
							break;
						case 'module':
							$module_options = array(
								'screenshots' => 'screenshots',
								'videos' => 'videos'
							);
							$field_html = gc_form_element('dropdown', array('name' => $field['name'], 'label_width' => 'col-sm-4', 'help' => lang('param_desc_' . $field['name']), 'options' => $module_options), $widget_params, lang('param_' . $field['name']) . lang('bf_form_label_required'), 'col-sm-8');
							break;
						case 'category_id':
						    $field_html = gc_form_element('dropdown', array('name' => $field['name'], 'label_width' => 'col-sm-4', 'help' => lang('param_desc_' . $field['name']), 'options' => $this->fields_model->dropdown_options('product_categories', true, 'name', 'id', true)), $widget_params, lang('param_' . $field['name']) . lang('bf_form_label_required'), 'col-sm-8');
							break;
						case 'limit':
						    $limit_options = array();
						    for($i = 1; $i <= 100; $i++) {
						        $limit_options[$i] = $i;
						    }						    
						    $field_html = gc_form_element('dropdown', array('name' => $field['name'], 'label_width' => 'col-sm-4', 'help' => lang('param_desc_' . $field['name']), 'options' => $limit_options), $widget_params, lang('param_' . $field['name']) . lang('bf_form_label_required'), 'col-sm-2');
							break;
						case 'without_heading':
							$heading_options = array(0 => lang('bf_yes'), 1 => lang('bf_no'));
							$field_html = gc_form_element('dropdown', array('name' => $field['name'], 'label_width' => 'col-sm-4', 'help' => lang('param_desc_' . $field['name']), 'options' => $heading_options), $widget_params, lang('param_' . $field['name']) . lang('bf_form_label_required'), 'col-sm-2');
							break;
						case 'heading_text':
						case 'heading_sub_text':
						case 'btn_label':
						case 'link_text':
						    $field_html = gc_form_element('text', array('name' => $field['name'], 'help' => lang('param_desc_' . $field['name']), 'label_width' => 'col-sm-4', 'default_value' => $field['value']), $widget_params, lang('param_' . $field['name']) . lang('bf_form_label_required'), 'col-sm-8');
						    $field_html .= gc_form_element('checkbox', array('name' => $field['name'] . '_lang', 'label_width' => 'col-sm-4', 'no_label' => true), $widget_params, lang('param_' . $field['name'] . '_lang'), 'col-sm-8');
						    break;
						default:
							$field_html = gc_form_element('text', array('name' => $field['name'], 'help' => lang('param_desc_' . $field['name']), 'label_width' => 'col-sm-4', 'default_value' => $field['value']), $widget_params, lang('param_' . $field['name']) . lang('bf_form_label_required'), 'col-sm-8');
					}          
					$fields_html[] = $field_html;     
				}
				// Do not display widget class and parallax fields in some widgets e.g. _banners.
				$widget_class = true;
				$parallax = true;
				switch($row->file_url) {
				    case 'widgets/others/_banners':
				        $widget_class = false;
				        $parallax = false;
				        break;
				}
				if($widget_class) {
				    $fields_html[] = gc_form_element('text', array('name' => 'widget_class', 'help' => lang('param_desc_widget_class'), 'label_width' => 'col-sm-4'), $widget_params, lang('param_widget_class'), 'col-sm-8');
				}
				if($parallax) {
    				switch($row->file_folder) {
    				    case 'navbars':
    				    case 'footers':
    				    case 'banners':
    				        break;
    				    default:
    				        // $fields_html[] = '<fieldset>';
    				        // $fields_html[] = '<legend class="small text-primary pt-2 pb-2 text-center">' . lang('param_parallax_legend') . '</legend>';
    				        $fields_html[] = gc_form_element('text', array('name' => 'parallax_params', 'label_width' => 'col-sm-4', 'help' => lang('param_desc_parallax_params')), $widget_params, lang('param_parallax_params'), 'col-sm-8');
    				        // $fields_html[] = gc_form_element('checkbox', array('name' => 'parallax__video', 'label_width' => 'col-sm-4', 'no_label' => TRUE), $widget_params, lang('param_parallax_video'));
    				        // $fields_html[] = gc_form_element('text', array('name' => 'parallax_video', 'label_width' => 'col-sm-4', 'help' => lang('param_desc_parallax_video')), $widget_params, lang('param_parallax_video'), 'col-sm-8');
    				        // $fields_html[] = '</fieldset>';
    				}
				}
				
	            // $fields_html[] = '</form>';//form_close();
	            $show_submit = 1;
	        } else {	            
	            $fields_html[] = $widget_no_params;
	        }
	    } else {
	        $fields_html[] = $widget_no_params;
	    }
	    die(json_encode(array('title' => $widget_title, 'fields' => implode(PHP_EOL, $fields_html), 'submit' => $show_submit, 'cover' => $widget_cover)));
	}
	
	/**
	 * Save theme settings
	 */
	public function theme_settings() {
	    clear_cache('site_settings'); // Clear Site Settings Cache
	    $this->load->model('settings/settings_model');
	    if (isset($_POST['save'])) {
	        gc_demo_mode();
	        $this->gcrud->stats(array('module' => 'themes'));
	        $this->form_validation->set_rules('navbar', 'lang:theme_navbar', 'trim');
	        $this->form_validation->set_rules('footer', 'lang:theme_footer', 'trim');
	        $this->form_validation->set_rules('options', 'lang:label_options', 'trim');
	        if($this->form_validation->run() == TRUE) {
	            $settings = array(
	                'navbar' => $this->input->post('navbar'),
	                'footer' => $this->input->post('footer'),
	                'options' => $this->input->post('options')
	            );
	            $data = array (array('name' => 'theme.settings', 'value' => json_encode($settings)));
	            $updated = $this->settings_model->update_batch($data, 'name');
	            if ($updated) {
	                $this->widgets->set_default('themes', 'default', false, $settings);
	                Template::set_message(sprintf(lang('msg_save_success'), lang('gc_menu_settings')), 'success');
	                Template::redirect(current_url());
	            } else {
	                Template::set_message(sprintf(lang('msg_edit_failure'), lang('gc_menu_settings')), 'danger');
	            }
	        }
	    }
	    $setting_name = 'theme.settings';
	    $result = $this->db->select('value')->get_where('settings', array('name' => $setting_name))->row();
	    if($result) {
	        $settings = json_decode($result->value ?? '');
	    } else {
	        $settings = json_decode(settings_item($setting_name) ?? '');
	    }
	    Template::set(array(
	        'toolbar_title' => lang('gc_menu_settings'),
	        'settings' => $settings
	    ));
	    Template::render();
	}
	
	/**
	 * Generate Random Views
	 * Common temp view for all pages with or without default navbars
	 * Temp layout testing temporarily and option to save
	 * Fields - demo or random data
	 * Common fields for optional parameters e.g. common block for featured, popular and recently added with parameters
	 * Option for random data
	 * warning this will add random data to all tables, do not run it on a live website, make the changes locally
	 */
	public function random() {
	    $this->load->library('themes/theme_builder');
	    $id = 'home';
	    if(isset($_POST['save'])) {
	        gc_demo_mode();
	        $this->gcrud->stats(array('module' => 'themes'));
	        $saved_layout_widgets = (array) $this->session->userdata('layout_widgets');
	        $saved_widgets = $_POST['layout_widgets'];
	        if(!empty($saved_widgets)) {
	            foreach($saved_widgets as $key => $saved_widget) {
	                $saved_widgets[$key]['params'] = '';
	                if(!empty($saved_layout_widgets[$id][$saved_widget['widget']])) {
	                    $saved_widgets[$key]['params'] = $saved_layout_widgets[$id][$saved_widget['widget']];
	                }
	            }
	        }
	        $this->db->where('id', $id)->update('themes', array('layout_widgets' => json_encode($saved_widgets)));
	        // prepare layout design
	        $this->theme_builder->prepare_layout_design($id);
	        // set default
	        $this->widgets->set_default('views', $id);
	        Template::set_message(sprintf(lang('msg_save_success'), lang('label_widgets')), 'success');
	        Template::redirect(current_url());
	    }
	    Template::render();
	}
	
	/**
	 * Theme options metas
	 * @param int $theme_option_id
	 */
	public function option_metas($theme_option_id) {
	    if(isset($_POST['save']) || isset($_POST['save_only'])) {
	        gc_demo_mode();
	        $this->gcrud->stats(array('module' => 'themes'));
	        $post_data = $_POST;
	        unset($post_data['save']);
	        unset($post_data['save_only']);	        
	        if(!empty($post_data)) {
	            foreach($post_data as $mkey => $mvalue) {
	                $meta_where = array(
	                    'theme_option_id' => $theme_option_id,
	                    'meta_key' => $mkey
	                );
	                $meta = $this->db->where($meta_where)->get('theme_option_metas')->row();
	                $mvalue = is_array($mvalue) ? json_encode($mvalue) : $mvalue;;
	                if($meta) {
	                    $this->db->where('id', $meta->id)->update('theme_option_metas', array('meta_value' => $mvalue));
	                } else {
	                    $meta_where['meta_value'] = $mvalue;
	                    $this->db->insert('theme_option_metas', $meta_where);	                    
	                }
	            }
	        }
	        $metas = $this->db->select('meta_key, meta_value')->where('theme_option_id', $theme_option_id)->get('theme_option_metas')->result();
	        if($metas) {
	            $meta_data = array();
	            foreach($metas as $meta) {
	                $meta_data[$meta->meta_key] = is_array($meta->meta_value) ? json_encode($meta->meta_value) : $meta->meta_value;
	            }
	            $this->db->where('id', $theme_option_id)->update('theme_options', array('metas' => json_encode($meta_data)));
	        }
	        Template::set_message(sprintf(lang('msg_save_success'), lang('theme_option_metas')), 'success');
	        if(isset($_POST['save_only'])) {
	            Template::redirect(current_url());
	        } else {
	           redirect($this->session->userdata('theme_options_last_visited_page'));
	        }
	    }
	    $theme_option_id = (int) $theme_option_id;
	    $record = $this->get_theme_options('id = ' . $theme_option_id, false);
	    if(!$record || empty($record->option_parent_ids)) {
	        Template::redirect(site_url() . SITE_AREA . '/developer/themes/manage/options');
	    }
	    $saved_metas = json_decode($record->metas ?? '');
	    // get parent field
	    // ' . $record->option_parent_ids . '
	    $option_parent_ids = (array) explode(',', $record->option_parent_ids ?? '');
	    $fields = array();
	    if(in_array(1, $option_parent_ids)) { // demo content fields	 
	        // get theme blocks
	        $theme_blocks = array();
	        $block_themes = $this->db->order_by('name')->get('theme_blocks')->result();
	        if($block_themes) {
	            foreach($block_themes as $block_theme) {
	                $theme_blocks[$block_theme->id] = $block_theme->name;
	            }
	        }
	        $theme_pages = array(
	            '' => lang('tab_all'),
	            'home' => lang('bf_home'),
	            'view' => lang('label_view'),
	            'detail' => lang('label_detail')
	        );
	        $custom_fields = array(
	            array('name' => 'block', 'type' => 'dropdown', 'label' => lang('label_id'), 'options' => $theme_blocks),
	            array('name' => 'page', 'type' => 'dropdown', 'label' => lang('label_page'), 'options' => $theme_pages),
	            array('name' => 'position', 'type' => 'text', 'label' => lang('theme_position'), 'default_value' => 0, 'class' => 'width-80'),
	            array('name' => 'params', 'type' => 'text', 'label' => lang('label_params')),
	        );
	        Template::set('custom_fields', $custom_fields);
	        $fields[] = array(
	            'name' => lang('label_options'),
	            'slug' => 'options',
	            'type' => 'custom',
	            'field_extra' => array('fields' => $custom_fields),
	            'class' => 9
	            // 'field_extra' => 'options:custom_fields'
	        );
	        if(!empty($saved_metas->options)) {
	            $saved_metas = json_decode($saved_metas->options);
	        }
	    } else if(in_array(3, $option_parent_ids)) {
	        $fields = array(
	            array(
	                'name' => 'Type',
	                'slug' => 'type',
	                'type' => 'dropdown',
	                'options' => array('text' => 'TEXT', 'textarea' => 'TEXTAREA', 'dropdown' => 'DROPDOWN', 'selectize' => 'SELECTIZE')
	            ),
	            array(
	                'name' => 'Options',
	                'slug' => 'options',
	                'type' => 'textarea',
	                'options' => array()
	            )
	        );
	    } else {
	        if(!in_array(3, $option_parent_ids)) {
	            $option_parent_ids[] = 3;
	        }
    	    // find_in_set with multiple search string
    	    $sql_where = array();
    	    foreach($option_parent_ids as $option_parent_id) {
    	        $sql_where[] = 'FIND_IN_SET (' . $option_parent_id . ', option_parent_ids)';
    	    }
    	    // FIND_IN_SET (3, option_parent_ids)
    	    // ' . implode(',', $option_parent_ids) . '
    	    // 'option_parent_ids IN (' . implode(',', $option_parent_ids) . ')'
    	    // 'CONCAT(",", `option_parent_ids`, ",") REGEXP ",(' . implode('|', $option_parent_ids) . '),"'
    	    // get fields
    	    $fields = $this->get_theme_options(implode(' AND ', $sql_where)); // check whether the option has fields or not
	    }
	    if(!empty($fields)) {
	        Assets::add_css('selectize.bootstrap3.css');
	        Assets::add_js('selectize.min.js');
	        Assets::add_js('selectize-search.js');
    	    foreach($fields as $fkey => $fvalue) {
    	        $field_metas = !empty($fvalue->metas) ? json_decode($fvalue->metas, true) : array();
    	        if(!empty($field_metas)) {
    	            foreach($field_metas as $meta_key => $meta_value) {
    	                if(!empty($meta_value) && $meta_key == 'type') {
    	                    switch($meta_value) {
    	                        case 'selectize':
    	                            $meta_value = 'text';
    	                            Assets::add_js("$('#" . $fvalue->slug . "').selectize({create: true, plugins: ['remove_button']});", "inline");
    	                            break;
    	                    }
    	                }
    	                $fields[$fkey]->{$meta_key} = $meta_value;
    	            }
    	        }
    	    }
	    }
	    Template::set(array(
	        'fields' => $fields,
	        'record' => $saved_metas,
	        'toolbar_title' => lang('theme_option_metas') . '::' . $record->name
	    ));
	    Template::render();
	}
	    
	/**
	 * Get Theme Options
	 * @param string/array $where
	 * @param boolean $multi
	 * @return array/object
	 */
	private function get_theme_options($where, $multi = true) {
	    if($multi) {
	        return $this->db->where($where)->get('theme_options')->result();
	    } else {
	       return $this->db->where($where)->get('theme_options')->row();
	    }
	}
	
	/**
	 * Prepare Theme Demo
	 * 
	 * Views are content specific, 
	 * Layouts are common
	 * CSS are color specific, they must be inside the css folder
	 * 
	 * @param number $theme_option_id
	 */
	public function prepare_demo($theme_option_id) {
	    $this->auth->restrict('Theme_Options.Developer.View');
	    $theme_option_id = (int) $theme_option_id;
	    $row = $this->db->where('id', $theme_option_id)->get('theme_options')->row();
	    if(!$row || $row->option_parent_ids != 1) { // parent must be of demo_content
	        Template::redirect($this->session->userdata('theme_options_last_visited_page'));
	    }
	    if(isset($_POST['save'])) {
	        gc_demo_mode();
	        $this->gcrud->stats(array('module' => 'themes'));
	        $theme = $this->input->post('theme');
	        $scheme = $this->input->post('scheme');
	        $is_default_scheme = (int) $this->input->post('is_default_scheme');
	        $this->load->library('theme_creator');
	        // $this->session->unset_userdata('theme_demo_content');
	        $this->theme_creator->theme_demo_content($row, $theme, $scheme, false, true);
	        
	        // set default scheme to be used during demo load
	        $extras = json_decode($row->extras ?? '', true);
	        if(!empty($is_default_scheme)) {	            
	            $extras['themes'][$theme]['scheme'] = $scheme;
	        } else if(empty($extras['themes'][$theme]['scheme'])) { // make this scheme as default
	            $extras['themes'][$theme]['scheme'] = $scheme;
	        }
	        $this->db->where('id', $row->id)->update('theme_options', array('extras' => json_encode($extras)));
	        clear_cache('theme_options');
	        Template::set_message(lang('msg_success'), 'success');
	    }
	    $records = $this->db->where("type = 'schemes' OR type = 'themes'")->get('theme_settings')->result();
	    $schemes = array();
	    $themes = array();
	    if($records) {
	        foreach($records as $record) {
	            $name = explode('_', $record->name ?? '');
	            $name = ucwords(implode(' ', $name));
	            if($record->type == 'schemes') {
	                $schemes[$record->name] = $name;
	            } else {
	                $themes[$record->name] = $name;
	            }
	        }
	    }
	    Template::set(array(
	        'toolbar_title' => lang('theme_demo'),
	        'themes' => $themes,
	        'schemes' => $schemes
	    ));
	    Template::render();
	}
	
	/**
	 * Prepare all contents demo at once for all themes and their schemes
	 */
	private function prepare_all_themes_demo() {    
	    $this->load->library('theme_creator');
	    // get all themes
	    $contents = $this->db->where(array('option_parent_ids' => 1))->get('theme_options')->result();
	    $themes = $this->db->where(array('type' => 'themes'))->get('theme_settings')->result();
	    $schemes = $this->db->where(array('type' => 'schemes'))->get('theme_settings')->result();
	    if($contents) {
	        foreach($contents as $row) {
    	        foreach($themes as $theme) {
    	            $theme = $theme->name;
    	            $i = 1;
    	            foreach($schemes as $scheme) {
    	                $this->theme_creator->theme_demo_content($row, $theme, $scheme, false, true);
    	                if($i == 1) {
        	                // set default scheme to be used during demo load
        	                $extras = json_decode($row->extras ?? '', true);
        	                if(empty($extras['themes'][$theme]['scheme'])) { // make this scheme as default
        	                    $extras['themes'][$theme]['scheme'] = $scheme;
        	                }
        	                $this->db->where('id', $row->id)->update('theme_options', array('extras' => json_encode($extras)));
    	                }
    	                $i++;
    	            }
    	        }
	        }
	        clear_cache('theme_options');
	    }	    
	    Template::set_message(lang('msg_success'), 'success');
	    Template::redirect($this->session->userdata('theme_options_last_visited_page'));
	}
	
	/**
	 * Theme options sliders/banners demo content
	 * The data is stored in the extras column of the theme_options table
	 * @param number $theme_option_id
	 * @param string $action - update/clear
	 * 
	 * sliders_banners - Refresh Sliders/Banners
	 * themes - Generate theme content
	 */
	public function demo_content($theme_option_id, $action = 'update') {
	    $this->auth->restrict('Theme_Options.Developer.View');
	    $theme_option_id = (int) $theme_option_id;
	    $row = $this->db->where('id', $theme_option_id)->get('theme_options')->row();
	    if($row && $row->option_parent_ids == 1) { // parent must be of demo_content
	        if($action == 'clear') {
	            $extras = json_decode($row->extras ?? '', true);
	            if(!empty($extras['sliders'])) {
	                unset($extras['sliders']);
	            }
	            if(!empty($extras['banners'])) {
	                unset($extras['banners']);
	            }
	            $this->db->where('id', $theme_option_id)->update('theme_options', array('extras' => json_encode($extras)));
	        } else {
	            // cronjob can also be run to automate this process
    	        $this->load->library('goclixy/gsample');
    	        $this->gsample->get_slider_banners('sliders', $row->slug);
    	        $this->gsample->get_slider_banners('banners', $row->slug);
	        }
	        clear_cache('theme_options');
	        Template::set_message(lang('msg_success'), 'success');
	    } else {
	        Template::set_message(lang('msg_operation_failure'), 'danger');
	    }
	    Template::redirect($this->session->userdata('theme_options_last_visited_page'));
	}

	/**
	 * Get non existence theme files
	 */
	public function non_existence() {
	    $records = $this->db->get('themes')->result();
	    if($records) {
	        foreach($records as $record) {
	            $path = FCPATH . 'themes/default/';
	            if(!file_exists($path. $record->file_url . '.php')) {
	                dump($record->id);
	            }
	        }
	    }
	}	
	
	/**
	 * Design Layout
	 * @param int $id
	 * @param string $design_page
	 */
	public function layout_design($id, $design_page = null) {
	    $this->theme_builder->layout_design($id, $design_page);
	}
	
	/**
	 * Preview View/Layout Content
	 * @param integer $view_id
	 * @param array $record
	 * @param boolean $view_content
	 */
	public function preview($view_id, $record = array(), $view_content = false) {
	    $this->theme_builder->preview($view_id, $record, $view_content);
	}
	
	/**
	 * Covert Demo to Theme
	 * @param integer $id
	 */
	public function demo_to_theme($id) {
	    $this->theme_builder->demo_to_theme($id);
	}
	
	/**
	 * Ajax preview reload
	 * @param int $id
	 * @param array $layout
	 */
	public function ajax_preview_load($id, $layout = array()) {
	    $this->theme_builder->preview($id, $layout);
	    echo json_encode(array('success' => 1));
	}
	
	/**
	 * Ajax - Allow user to save layout widgets in a session variable
	 * These variable will be saved in the layout_widgets column on Save button click
	 */
	public function save_layout_widgets($id = 0) {
	    die($this->theme_builder->save_layout_widgets($id));
	}
	
	/**
	 * The Theme Folder must be unique
	 * @return boolean
	 */
	public function unique_folder() {
	    $id = (int) $this->input->post('id');
	    $type_id = $this->input->post('type_id');
	    $folder = $this->input->post('folder');
	    $where = array('type_id' => $type_id, 'folder' => $folder);
	    if($id > 0) {
	        $where['id !='] = $id;
	    }
	    $counts = $this->db->where($where)->count_all_results('theme_folders');
	    if($counts == 0) {
	        return true;
	    } else {
	        return false;
	    }
	}
	
	/**
	 * Update applicable views 
	 * UPDATE rs0d_themes SET applicable_views = (SELECT GROUP_CONCAT(view) AS applicable_views FROM `rs0d_theme_views` WHERE theme_id = rs0d_themes.id GROUP BY theme_id)
	 */
	public function update_applicable_views() {
	    $sql = "UPDATE " . $this->db->dbprefix . "themes SET applicable_views = (SELECT GROUP_CONCAT(view) AS applicable_views FROM `" . $this->db->dbprefix . "theme_views` WHERE theme_id = " . $this->db->dbprefix . "themes.id GROUP BY theme_id)";
	    $this->db->query($sql);
	}
}