<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
 || ##################################################################### ||
 || # GoClixy - Online IT Solutions							            # ||
 || # Copyright (C) 2021 GoClixy. All Rights Reserved.				    # ||
 || # ----------------------------------------------------------------  # ||
 || # This file is owned and copyrighted by GoClixy.com. Your license   # ||
 || # confers  no  title  or ownership in the Script and is not a sale  # ||
 || # of any rights to the Software. This file may not be redistributed # ||
 || # in whole or significant part.									   # ||
 || # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
 || # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
 || ##################################################################### ||
 \*=======================================================================*/
/**
 * Theme Builder
 * 
 * Build theme randomly
 */
class Theme_builder {
    /**
     * Hold codeigniter instance
     * @var object
     */
    protected $CI;
    /**
     * Constructor
     * @param array $config
     */
    public function __construct($config = array()) {
        $this->CI =& get_instance();
        $this->CI->load->library('themes/widgets');
    }
    
    /**
     * Layout Widgets
     * @param number $id - Layout ID
     * @return void
     */
    public function layout_design($id, $design_page = null) {
        Assets::add_js('edit_area/edit_area_full.js'); // PHP, CSS and JS Editors
        $redirect_url = SITE_AREA . '/developer/themes';
        $toolbar_title = ''; // Toolbar Title
        $preview_page = ''; // Preview Page
        $contents = array(); // php, scss and js content
        switch($design_page) {
            case 'demos':
                $layout = $this->get_theme_demo($id);
                if(!$layout) {
                    Template::set_message(lang('invalid_id'), 'danger');
                    redirect($redirect_url);
                }
                $preview_page = $layout->file_folder;
                break;
            case 'random':
                $theme_block = $this->CI->db->where('id', $id)->get('theme_blocks')->row();
                if(!$theme_block) {
                    Template::set_message(lang('invalid_id'), 'danger');
                    redirect($redirect_url);
                }
                $preview_page = $theme_block->file_folder;
                break;
        }
        // get preview link if layout assigned to atleast one link
        $preview_url = null;
        switch($preview_page) {
            case 'home':
                $preview_url = site_url();
                break;
            case 'modules/products/view':
                $preview_url = gc_url_format('search') . (!empty($_SERVER['QUERY_STRING']) ? '?' . $_SERVER['QUERY_STRING'] : '');
                break;
            case 'modules/products/detail':
                // get random product
                $product = $this->CI->db->where(array('status_id' => 2, 'deleted' => 0))->order_by('RAND()')->get('products')->row();
                $preview_url = gc_url_format('products', $product->slug, $product->id);
                break;
        }
        $preview_html = '<div class="embed-responsive embed-responsive-16by9" style="max-height: 250px;"><iframe class="embed-responsive-item" src="' . $preview_url . '" allowfullscreen></iframe></div>';
        $reload_url = site_url(SITE_AREA . '/developer/themes/ajax_preview_load/' . $id);
        $save_widgets_url = site_url(SITE_AREA . '/developer/themes/save_layout_widgets/');
        $inline_js = <<<EOL
        $('.select-widget').change(function(e) {
            e.preventDefault();
            var widgetId = $(this).val();
            $.getJSON('{$save_widgets_url}' + widgetId + '/0/0/1', '', function(data) {
                reloadPreview();
            });
        });
        $('#reloadPreview').click(function(e) {
            e.preventDefault();
            reloadPreview();
        });
	    function reloadPreview() {
            $.getJSON('{$reload_url}', '', function(data) {
                $('.live-preview').html('{$preview_html}');
            });
        }
EOL;
        Assets::add_js($inline_js, 'inline');
        // Preview of random layout
        if(isset($_POST['preview'])) {
            $this->preview($id);
            if(!empty($preview_url)) {
                // Template::redirect($preview_url);
                // popup must be enabled for redirection
                echo '<script>window.open("' . $preview_url .'","_blank")</script>';
            }
        }
        // get preview view and layouts
        $view_layout_ids = $this->preview_view_layout();
        if(!empty($view_layout_ids[0])) {
            $preview_id = $view_layout_ids[0];
        }
        if(empty($design_page)) {
            $layout = $this->CI->db->where('id', $id)->get('themes')->row();
            if(!$layout || $layout->type_id == 1) {
                Template::set_message(lang('invalid_id'), 'danger');
                redirect($redirect_url);
            }
            
            // get widgets from the file instead of database to avoid any ambiguity
            $layout->layout_widgets = $this->get_layout_widgets_with_params($layout);
            $toolbar_title = $layout->name;
            // if(empty($layout->layout_widgets)) {}
        } else if($design_page == 'demos') {
            // write the content temporarily to the preview view file to view them
            $this->CI->db->where('id', $preview_id)->update('themes', array(
                'contents' => $layout->contents,
                'assets' => $layout->assets,
                'assets_css' => $layout->assets_css,
                'assets_js' => $layout->assets_js));
            $preview_file = $this->CI->db->where('id', $preview_id)->get('themes')->row();
            $contents = json_decode($preview_file->contents ?? '');
            if(!empty($contents)) {
                $file_folder = 'views/preview/';
                $this->CI->widgets->save_file($file_folder . $preview_file->file_name, 'php', $contents->php);
                $this->CI->widgets->save_file($file_folder . 'scss/' . $preview_file->file_name, 'scss', !empty($contents->scss) ? $contents->scss : '');
                $this->CI->widgets->save_file($file_folder . 'js/' . $preview_file->file_name, 'js', !empty($contents->js) ? $contents->js : '');
            }
            // get widgets from the preview file
            $layout->layout_widgets = $this->get_layout_widgets_with_params($preview_file);
            $toolbar_title = $layout->name;
        } else if($design_page == 'random') {
            $layout = new stdClass();
            $layout->id = $id;
            $layout = $this->random_layout($id, $layout, $redirect_url);
            $toolbar_title = $theme_block->name;
        } else {
            Template::set_message(lang('invalid_id'), 'danger');
            redirect($redirect_url);
        }
        // unable to generate layout on widget change because of POST data
        // to fix this issue, session variable is used
        $this->CI->session->set_userdata('saved_layout_widgets', json_decode($layout->layout_widgets ?? '', true));
        
        // regerate the layout with random widgets
        if(isset($_POST['regenerate'])) {
            $this->CI->session->unset_userdata('layout_widgets');
            $layout = $this->random_layout($design_page == 'demos' ? $layout->block_id : $id, $layout, $redirect_url);
            $_POST = array();
        }
        
        // Preview Contents
        $preview = $this->preview($id, $layout);
        if($preview) {
            $contents = json_decode($preview->contents ?? '', true);
        }
        
        // create a new folder name demo/preview in the css and js folders to hold temporary css and js files
        // create preview folder in the demo/layouts and demo/views folders
        
        /*$this->CI->load->model('links/links_model');
         $link_layouts = $this->links_model->get_all_links();
         foreach($link_layouts as $link => $link_layout) {
         if(empty($preview_url) && $link_layout['layout_id'] == $layout_id) {
         $preview_url = $link;
         }
         }*/
        
        Assets::add_css('selectize.bootstrap3.css');
        Assets::add_js('selectize.min.js');
        Assets::add_js('selectize-search.js');
        $this->CI->session->set_userdata('layout_design_id', $id);
        if(isset($_POST['save'])) {
            gc_demo_mode();
            $saved_layout_widgets = (array) $this->CI->session->userdata('layout_widgets');
            $saved_widgets = $_POST['layout_widgets'];
            if(!empty($saved_widgets)) {
                foreach($saved_widgets as $key => $saved_widget) {
                    $saved_widgets[$key]['params'] = '';
                    if(!empty($saved_layout_widgets[$id][$saved_widget['widget']])) {
                        $saved_widgets[$key]['params'] = $saved_layout_widgets[$id][$saved_widget['widget']];
                    }
                }
            }
            
            /**
             * Theme Demos are dependent on the preview view and layout rows in the themes table
             * In case of random designs, they will be stored in the preview rows and moved from there to the theme_demos table
             */
            $view_id = $id;
            if(!empty($design_page)) {
                $view_id = $preview_id;
                $demo_block_id = $id;
            }
            $this->CI->db->where('id', $view_id)->update('themes', array('layout_widgets' => json_encode($saved_widgets)));
            
            // prepare layout design
            $this->prepare_layout_design($view_id);
            
            // set default
            if(empty($design_page)) {
                $this->CI->widgets->set_default('views', $view_id);
            } else {
                // get content from the preview file
                $preview = $this->CI->db->where('id', $view_layout_ids[0])->get('themes')->row();
                if($preview) {
                    $datetime = date('Y-m-d H:i:s');
                    $data = array(
                        'contents' => $preview->contents,
                        'assets_css' => $preview->assets_css,
                        'assets_js' => $preview->assets_js,
                        'modified_on' => $datetime
                    );
                    // store data in the theme_demos table for future review
                    if($design_page == 'demos') {
                        // update the existing layout
                        $this->CI->db->where('id', $demo_block_id)->update('theme_demos', $data);
                    } else {
                        $data['block_id'] = $demo_block_id;
                        $data['created_on'] = $datetime;
                        $this->CI->db->insert('theme_demos', $data);
                    }
                }
            }
            $this->CI->session->unset_userdata('theme_preview');
            Template::set_message(sprintf(lang('msg_save_success'), lang('label_widgets')), 'success');
            // Template::redirect(current_url());
        }
        Assets::add_module_js('themes', 'layouts.js');
        $widgets_dropdown = array();
        $where = "type_id = 1 AND theme_block_id != 0"; // AND is_widget_design = 1
        if(empty($design_page)) {
            $where .= " AND file_folder != 'navbars' AND file_folder != 'footers'";
        }
        $widgets = $this->CI->db->select('id, name')->where($where)->order_by('name')->get('themes')->result();
        if($widgets) {
            foreach($widgets as $widget) {
                $widgets_dropdown[$widget->id] = $widget->name;
            }
        }
        // post handling/restore saved values
        $layout_widgets = json_decode($layout->layout_widgets ?? '', true);
        if(!empty($layout_widgets)) {
            usort($layout_widgets, "gc_sort_array");
            $layout_widgets_data = (array) $this->CI->session->userdata('layout_widgets');
            foreach($layout_widgets as $layout_widget) {
                if(!empty($layout_widget['params']) && empty($layout_widgets_data[$id][$layout_widget['widget']])) {
                    $layout_widgets_data[$id][$layout_widget['widget']] = $layout_widget['params'];
                }
                $this->CI->session->set_userdata('layout_widgets', $layout_widgets_data);
            }
        }
        $custom_fields = array(array('name' => 'widget', 'type' => 'dropdown', 'label' => lang('label_name'), 'options' => $widgets_dropdown, 'class' => 'select-widget'), array('name' => 'priority', 'type' => 'text', 'label' => lang('label_priority'), 'default_value' => 0), array('name' => 'options', 'type' => 'button', 'label' => lang('label_options'), 'class' => 'btn btn-primary btn-sm widget-options'));
        Template::set(array(
            'toolbar_title' => $toolbar_title,
            'custom_fields' => $custom_fields,
            'layout' => $layout,
            'layout_widgets' => $layout_widgets,
            'preview_url' => $preview_url,
            'design_page' => $design_page,
            'contents' => (array) $contents,
            'form_extra' => $this->CI->load->view('developer/_widget_fields', array(), true)
        ));
        Template::render();
    }    
    
    /**
     * To simplify preview, a preview view, and layout is created in the theme table itself
     * The slug is used instead of id to simplify things, id can be changed but not slug
     * No need to create additional folders for preview only
     * Set this preview view and layout temporarily to view the design
     * allow user to add css and js to the preview file
     * @param number $id
     *
     * e.g. http://localhost/ecommerce/public_html/admin/developer/themes/preview/136/0/1
     */
    public function preview($view_id, $record = array(), $view_content = false) {
        $theme = 'default';
        $scheme = 'orange';
        // check for the preview view and layout in the themes table
        $ids = $this->preview_view_layout();
        $id = $ids[0]; // view file
        if($view_content) {
            // get content from the saved file
            $view = $this->CI->db->where('id', $view_id)->get('themes')->row();
            if($view) {
                $this->write_preview_files($view, $ids[0]);
                // get layout from the
                $where = array('type_id' => 2, 'file_folder' => $view->layout);
                $layout = $this->CI->db->where($where)->get('themes')->row();
                if($layout) {
                    $this->write_preview_files($layout, $ids[1]);
                }
            }
        } else {
            $saved_layout_widgets = (array) $this->CI->session->userdata('layout_widgets');
            // $saved_widgets = !empty($_POST['layout_widgets']) ? $_POST['layout_widgets'] : (!empty($record->layout_widgets) ? json_decode($record->layout_widgets, true) : (array) $this->CI->session->userdata('saved_layout_widgets'));
            $saved_widgets = (array) $this->CI->session->userdata('saved_layout_widgets');
            // preview can be generated directly from the index page
            // in this case, copy the file directly
            if(!empty($saved_widgets)) {
                foreach($saved_widgets as $key => $saved_widget) {
                    $saved_widgets[$key]['params'] = '';
                    if(!empty($saved_layout_widgets[$view_id][$saved_widget['widget']])) {
                        $saved_widgets[$key]['params'] = $saved_layout_widgets[$view_id][$saved_widget['widget']];
                    }
                }
                $this->CI->db->where('id', $id)->update('themes', array('layout_widgets' => json_encode($saved_widgets)));
                // prepare layout design
                $this->prepare_layout_design($id);
                // process view temporarily using the session variable
            }
            // prepare layout file
            if(!empty($record->layout)) {
                // get layout from the
                $where = array('type_id' => 2, 'file_folder' => $view->layout);
                $layout = $this->CI->db->where($where)->get('themes')->row();
                if($layout) {
                    $this->write_preview_files($layout, $ids[1]);
                }
            }
            if(empty($layout)) {
                // use blank layout
                $content = <<<EOL
<?php
echo theme_view('widgets/others/_template_message');
echo theme_view('widgets/others/_template_content');
?>
EOL;
                
                // write the changes out...
                if (! function_exists('write_file')) {
                    $this->CI->load->helper('file');
                }
                $theme_folder = FCPATH . 'themes/default/';
                write_file($theme_folder . 'layouts/preview/_preview.php', $content); // write php file
                write_file($theme_folder . 'layouts/preview/scss/_preview.scss', ''); // // write scss file
                write_file($theme_folder . 'layouts/preview/js/_preview.js', '');  // write js file
            }
        }
        // process view and layout
        $this->CI->widgets->process_view($ids[0], $scheme);
        $this->CI->widgets->process_view($ids[1], $scheme);
        
        // set them temporarily on the page
        $layout = $this->CI->db->where('id', $ids[0])->get('themes')->row();
        $this->CI->session->set_userdata('theme_preview', $layout);
        return $layout;
    }
    
    /**
     * Demo to Theme Conversion
     * @param number $id
     */
    public function demo_to_theme($id) {
        $redirect_url = site_url(SITE_AREA . '/developer/themes/manage/demos');
        $layout = $this->get_theme_demo($id);
        if(!$layout) {
            Template::set_message(lang('invalid_id'), 'danger');
            redirect($redirect_url);
        }
        if(isset($_POST['save']) || isset($_POST['save_only'])) {
            $this->CI->form_validation->set_rules('theme', 'lang:themes_singular', 'trim');
            $this->CI->form_validation->set_rules('is_theme_default', 'lang:is_theme_default', 'trim');
            $this->CI->form_validation->set_rules('name', 'lang:label_name', 'trim|required|unique[themes.name, themes.id]');
            $this->CI->form_validation->set_rules('file_name', 'lang:label_file_name', 'trim|required|alpha_slug');
            $this->CI->form_validation->set_rules('class_name', 'lang:label_class_name', 'trim');
            $this->CI->form_validation->set_rules('image_url', 'lang:label_image', 'trim');
            if ($this->CI->form_validation->run() === true) {
                // check for the name existence first
                $name = $this->CI->input->post('name');
                $file_folder = $layout->file_folder;
                $file_name = $this->CI->input->post('file_name');
                $class_name = $this->CI->input->post('class_name');
                $image_url = $this->CI->input->post('image_url');
                $datetime = date('Y-m-d H:i:s');
                $root_folder = 'views';
                $type_id = 0;
                $layout_type = 'index';
                $page_type = lang('label_view');
                if($layout->block_type_id == 1) {
                    $root_folder = 'layouts';
                    $type_id = 2;
                    $layout_type = $file_folder;
                    $page_type = lang('label_layout');
                }
                $file_url = $root_folder . '/' . $file_folder . '/' . $file_name;
                $data = array(
                    'type_id' => $type_id,
                    'name' => $name,
                    'layout' => $layout_type,
                    'file_folder' => $file_folder,
                    'file_name' => $file_name,
                    'file_url' => $file_url,
                    'contents' => $layout->contents,
                    'assets' => $layout->assets,
                    'assets_css' => $layout->assets_css,
                    'assets_js' => $layout->assets_js,
                    'class_name' => $class_name,
                    'image_url' => $image_url,
                );
                $where = array('file_folder' => $file_folder, 'file_name' => $file_name);
                // insert and update
                $row = $this->CI->db->where($where)->get('themes')->row();
                $success = false;
                if($row) {
                    // update existing row
                    $data['modified_on'] = $datetime;
                    if($this->CI->input->post('theme_is_update')) {
                        $success = $this->CI->db->where('id', $row->id)->update('themes', $data);
                    } else {
                        Template::set_message(lang('theme_demo_convert_error'), 'danger');
                    }
                } else {
                    // insert
                    $data['created_on'] = $datetime;
                    $data['modified_on'] = $datetime;
                    $this->CI->db->insert('themes', $data);
                    $success = $this->CI->db->insert_id();
                }
                if($success) {
                    Template::set_message(sprintf(lang('msg_save_success'), $page_type), 'success');
                    if(isset($_POST['save_only'])) {
                        Template::redirect(current_url());
                    } else {
                        Template::redirect($redirect_url);
                    }
                }
            }
        }
        $preview_page = $layout->file_folder;
        $themes = array();
        $all_themes = $this->CI->db->where('type', 'themes')->get('theme_settings')->result();
        if($all_themes) {
            foreach($all_themes as $all_theme) {
                $themes[$all_theme->name] = ucwords(str_replace('_', ' ', $all_theme->name));
            }
        }
        $record = array(
            'name' => $layout->name,
            'class_name' => generateRandomString()
        );
        Template::set(array(
            'record' => $record,
            'toolbar_title' => $layout->name . ' (' . $preview_page . ')',
            'themes' => $themes
        ));
        Template::render();
    }
    
    /**
     * Ajax - Allow user to save layout widgets in a session variable
     * These variable will be saved in the layout_widgets column on Save button click
     */
    public function save_layout_widgets($id = 0) {
        $id = empty($id) ? $this->CI->session->userdata('layout_design_id') : $id;
        $widget_id = empty($id) ? (int) $this->CI->input->post('widget_id') : $id;
        unset($_POST['module']);
        unset($_POST['row_id']);
        unset($_POST['widget_id']);
        $layout_widgets = (array) $this->CI->session->userdata('layout_widgets');
        $layout_widgets[$id][$widget_id] = $_POST;
        // save data in a session variable to update it on save
        $this->CI->session->set_userdata('layout_widgets', $layout_widgets);
        $this->post_layout_widgets($widget_id); // widget changed
        return(json_encode(array('result' => 1, 'message' => lang('widget_save_success'), 'type' => 'success')));
    }
    
    /**
     * Ajax POST handling on block change
     * @param number $widget_id
     */
    private function post_layout_widgets($widget_id) {        
        // Ajax way to deal with $_POST parameters
        $saved_layout_widgets = (array) $this->CI->session->userdata('saved_layout_widgets');
        $record = $this->CI->gc->get('themes', array('where' => 'id = ' . $widget_id));
        if($record) {
            $theme_block = $this->CI->gc->get('theme_blocks', array('where' => 'id = ' . $record->theme_block_id));
            if($theme_block) {
                // if the same block already exist, remove it
                foreach($saved_layout_widgets as $key => $saved_layout_widget) {
                    if(!empty($saved_layout_widget['theme_block_id']) && $saved_layout_widget['theme_block_id'] == $record->theme_block_id) {
                        unset($saved_layout_widgets[$key]);
                    }
                }
                $params = json_decode($record->params ?? '', true);
                $saved_layout_widgets[$widget_id] = array(
                    'widget' => $record->id,
                    'theme_block_id' => $record->theme_block_id,
                    'priority' => $theme_block->priority,
                    'params' => $params
                );
                $this->CI->session->set_userdata('saved_layout_widgets', $saved_layout_widgets);
            }
        }
    }
    
    /**
     * Prepare Layout Design
     * @param number $id
     * @return boolean
     */
    private function prepare_layout_design($id) {
        $layout = $this->CI->db->where('id', $id)->get('themes')->row();
        if(!$layout || $layout->type_id == 1 || empty($layout->layout_widgets)) {
            Template::set_message(lang('invalid_id'), 'danger');
            return false;
        }
        $widgets = json_decode($layout->layout_widgets ?? '', true);
        usort($widgets, 'gc_sort_array');
        $content = array();
        $inline_css = array();
        $inline_js = array();
        $external_js_dependencies = array();
        $external_css_dependencies = array();
        if(!empty($widgets)) {
            foreach($widgets as $widget) {
                $widget_id = $widget['widget'];
                // get widget information
                $record = $this->CI->db->where('id', $widget_id)->get('themes')->row();
                if($record && $record->type_id == 1) {
                    // saved parameters
                    $widget_params = !empty($widget['params']) ? (array) $widget['params'] : array();
                    if(!empty($record->params)) {
                        if(empty($widget_params)) {
                            // load default values
                            $default_params = json_decode($record->params ?? '', true);
                            if($default_params) {
                                foreach($default_params as $default_param) {
                                    $widget_params[$default_param['name']] = $default_param['value'];
                                }
                            }
                        }
                    }
                    if(!empty($widget_params)) {
                        $params_array = array();
                        foreach($widget_params as $key => $value) {
                            switch($key) {
                                case 'parallax_params':
                                    if(!empty($value) && is_string($value)) {
                                        $parallax_params = array();
                                        $params_parallax = explode('|', $value);
                                        if(!empty($params_parallax)) {
                                            foreach($params_parallax as $param_parallax) {
                                                $param_parallax = explode('~', $param_parallax);
                                                if(!empty($param_parallax[1])) {
                                                    $parallax_params[$param_parallax[0]] = $param_parallax[1];
                                                }
                                            }
                                        }
                                        if(!empty($parallax_params['url'])) {
                                            $parallax_class = isset($parallax_params['class']) ? ' ' . $parallax_params['class'] : '';
                                            if(!empty($parallax_params['is_video'])) {
                                                $video_url = $parallax_params['url'];
                                                $inline_js[] = <<<EOL
/*----------------------------------------------------*/
/*	Parallax Video Background
/*----------------------------------------------------*/
$(".{$record->class_name}").prepend('<div class="jarallax-overlay{$parallax_class}"></div>');
$('.{$record->class_name}').jarallax({
    speed: 0.2,
    videoSrc: '{$video_url}'
});
EOL;
                                                $external_js_dependencies['jarallax-video.min.js'] = 'internal';
                                            } else {
                                                // is_local - 1 - the file is on the local system
                                                $image_url = !empty($parallax_params['is_local']) ? gc_image_url($parallax_params['url']) : $parallax_params['url'];
                                                $inline_js[] = <<<EOL
/*----------------------------------------------------*/
/*	Parallax Image Background
/*----------------------------------------------------*/
$(".{$record->class_name}").prepend('<div class="jarallax-overlay{$parallax_class}"></div>');
$('.{$record->class_name}').jarallax({
	speed: 0.2,
	imgSrc: '{$image_url}'
});
EOL;
                                                $external_js_dependencies['jarallax.min.js'] = 'internal';
                                            }
                                        }
                                    }
                                    break;
                                case 'widget_class': // Add class to the widget main class
                                    if(!empty($value)) {
                                        $inline_js[] = <<<EOL
/*----------------------------------------------------*/
/*	Additional Class e.g. btn-theme-secondary etc.
/*----------------------------------------------------*/
$('.{$record->class_name}').addClass("{$value}");
EOL;
                                    }
                                    
                                    break;
                                default:
                                    $params_array[] = "'" . $key . "' => " . (is_numeric($value) ? $value : "'" . $value . "'");
                            }
                        }
                        $content[] = "echo theme_view('" . $record->file_url . "', array(" . implode(', ', $params_array) . "));";
                    } else {
                        $content[] = "echo theme_view('" . $record->file_url . "');";
                    }
                }
            }
        }
        $content = implode(PHP_EOL, $content);
        $copyright_info_content = <<<EOL
<?php
/*=======================================================================*\
 || ##################################################################### ||
 || # GoClixy - Online IT Solutions							            # ||
 || # Copyright (C) 2021 GoClixy. All Rights Reserved.				    # ||
 || # ----------------------------------------------------------------  # ||
 || # This file is owned and copyrighted by GoClixy.com. Your license   # ||
 || # confers  no  title  or ownership in the Script and is not a sale  # ||
 || # of any rights to the Software. This file may not be redistributed # ||
 || # in whole or significant part.									    # ||
 || # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
 || # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
 || ##################################################################### ||
 \*=======================================================================*/
{$content}
?>
EOL;
        $content = $copyright_info_content;
        
        // write the changes out...
        if (! function_exists('write_file')) {
            $this->CI->load->helper('file');
        }
        
        // write_file(file_url . '.php', $content);
        // write php file with new content
        $theme_folder = FCPATH . 'themes/default/';
        write_file($theme_folder . $layout->file_url . '.php', $content);
        $contents = array(
            'php' => $content
        );
        
        // write inline js
        // if(!empty($inline_js)) { // removal of the code
        $contents['js'] = implode(PHP_EOL, $inline_js);
        $js_path = 'layouts/' . $layout->file_folder . '/js/' . $layout->file_name . '.js';
        write_file($theme_folder . $js_path, $contents['js']);
        //}
        
        // write inline css
        // if(!empty($inline_css)) { // removal of the code
        $contents['css'] = implode(PHP_EOL, $inline_css);
        $scss_path = 'layouts/' . $layout->file_folder . '/scss/' . $layout->file_name . '.scss';
        write_file($theme_folder . $scss_path, $contents['css']);
        // }
        
        // layout data
        $layout_data = array(
            'contents' => json_encode($contents)
        );
        $external_js_dependencies = $this->prepare_css_json($external_js_dependencies, $layout->assets_js);
        $external_css_dependencies = $this->prepare_css_json($external_css_dependencies, $layout->assets_css);
        // also remove files on parallax remove
        // if(!empty($external_js_dependencies)) {
        $layout_data['assets_js'] = json_encode($external_js_dependencies);
        // }
        // if(!empty($external_css_dependencies)) {
        $layout_data['assets_css'] = json_encode($external_css_dependencies);
        // }
        
        // update themes table with new data
        $this->CI->db->where('id', $layout->id)->update('themes', $layout_data);
    }
    
    /**
     * Generate a random layout
     */
    private function random_layout($id, $layout, $redirect_url) {
        // generate content randomly
        $where = 'layout_ids IN (' . $id . ')';
        $blocks = $this->CI->db->where($where)->order_by('priority')->get('theme_blocks')->result();
        if(!$blocks) {
            Template::set_message(lang('invalid_id'), 'danger');
            redirect($redirect_url);
        }
        $i = 0;
        $layout_widgets = (array) $this->CI->session->userdata('layout_widgets');
        foreach($blocks as $block) {
            $theme_block_id = $block->id;
            $where = "FIND_IN_SET(" . $theme_block_id . ", theme_block_id)";
            $record = $this->CI->db->where($where)->order_by('RAND()')->get('themes')->row();
            if($record) {
                $params = array();
                $saved_params = json_decode($record->params ?? '', true);
                if(!empty($saved_params)) {
                    foreach($saved_params as $saved_param) {
                        $params[$saved_param['name']] = $saved_param['value'];
                    }
                }
                $widget_id = $record->id;
                $layout_widgets[$id][$widget_id] = $params;
                $layout_data[$i] = array('widget' => $widget_id, 'priority' => $i, 'params' => $params);
                $i++;
            }
        }
        if(empty($layout_widgets)) {
            Template::set_message(lang('invalid_id'), 'danger');
            redirect($redirect_url);
        }
        // save data in a session variable to update it on save
        $this->CI->session->set_userdata('layout_widgets', $layout_widgets);
        $layout->layout_widgets = !empty($layout_data) ? json_encode($layout_data) : null;
        return $layout;
    }
    
    /**
     * Prepare CSS and JSON content
     * @param string $external_dependencies
     * @param string $saved_assets
     */
    private function prepare_css_json($external_dependencies, $saved_assets) {
        $assets_data = array();
        if(!empty($external_dependencies)) {
            $assets = json_decode($saved_assets ?? '', true);
            if(!empty($assets)) {
                foreach($assets as $asset) {
                    if(!isset($external_dependencies[$asset['url']])) {
                        $external_dependencies[$asset['url']] = $asset['type'];
                    }
                }
            }
            if(!empty($external_dependencies)) {
                foreach($external_dependencies as $url => $type) {
                    $assets_data[] = array(
                        'url' => $url,
                        'type' => $type
                    );
                }
            }
        }
        return $assets_data;
    }
    
    /**
     * Write preview/layout files
     * @param array $record
     */
    private function write_preview_files($record, $view_layout_id) {
        if($record) {
            $file_php = FCPATH . 'themes/default/' . $record->file_url . '.php';
            $file_css = FCPATH . 'themes/default/' . str_replace($record->file_name, 'scss/', $record->file_url) . $record->file_name . '.scss';
            $file_js = FCPATH . 'themes/default/' .  str_replace($record->file_name, 'js/', $record->file_url) . '.js';
            $files = array('php' => $file_php, 'scss' => $file_css, 'js' => $file_js);
            
            // check whether the theme_view exist in the database or not
            $theme_view = $this->CI->db->where('theme_id', $view_layout_id)->get('theme_views')->row();
            $view = $record->type_id == 0 ? 'preview' : '_preview';
            if($theme_view) {
                $this->CI->db->where('theme_id', $view_layout_id)->update('theme_views', array('view' => $view));
            } else {
                $this->CI->db->insert('theme_views', array('theme_id' => $view_layout_id, 'view' => $view));
            }
            
            // write the changes out...
            if (! function_exists('write_file')) {
                $this->CI->load->helper('file');
            }
            foreach($files as $type => $file) {
                if(file_exists($file)) {
                    $content = file_get_contents($file);
                    
                    // write_file(file_url . '.php', $content);
                    // write php file with new content
                    $theme_folder = FCPATH . 'themes/default/';
                    $file_path = $theme_folder . ($record->type_id == 0 ? 'views' : 'layouts') . '/preview/' . ($type == 'php' ? '' : $type . '/') . ($record->type_id == 0 ? '' : '_') . 'preview.' . $type;
                    write_file($file_path, $content);
                }
            }
        }
    }
    
    /**
     * Create preview folders if does not exist
     */
    private function preview_folders() {
        $folder = 'preview';
        $theme_path = FCPATH . 'themes/default/';
        $folders = array(
            'layouts/' . $folder, 'layouts/' . $folder . '/scss', 'layouts/' . $folder . '/js',
            'views/' . $folder, 'views/' . $folder . '/scss', 'views/' . $folder . '/js'
        );
        foreach($folders as $folder) {
            $folder_path = $theme_path . $folder;
            if (!is_dir($folder_path)) {
                mkdir($folder_path, 0777, TRUE);
            }
        }
    }
    
    /**
     * Get preview view and layout from the themes table
     * @return NULL[]
     */
    private function preview_view_layout() {
        $ids = array();
        $preview_files = array(0 => 'preview', 2 => '_preview');
        foreach($preview_files as $type_id => $preview_file) {
            $where = array('type_id' => $type_id, 'file_folder' => 'preview', 'file_name' => $preview_file);
            $row = $this->CI->db->where($where)->get('themes')->row();
            if($row) {
                $ids[] = $row->id;
            } else {
                // create new row
                $data = $where;
                $data['theme'] = 'default';
                $data['created_on'] = date('Y-m-d H:i:s');
                $data['modified_on'] = $data['created_on'];
                if($type_id == 0) {
                    $data['name'] = 'Preview - View';
                    $data['layout'] = '_preview';
                    $data['file_url'] = 'views/preview/' . $preview_file;
                } else {
                    $data['name'] = 'Preview - Layout';
                    $data['layout'] = 'preview';
                    $data['file_url'] = 'layouts/preview/' . $preview_file;
                }
                $this->CI->db->insert('themes', $data);
                $insert_id = $this->CI->db->insert_id();
                $ids[] = $insert_id;
                
                // update the theme_views table to process the view/layout
                $this->CI->db->insert('theme_views', array('theme_id' => $insert_id, 'view' => $preview_file));
                
                // check for the folders
                $this->preview_folders();
            }
        }
        return $ids;
    }
    
    /**
     * Get Layout Widgets with parameters to save into the layout_design column
     * @param array $record
     */
    private function get_layout_widgets_with_params($record) {
        // read file
        $folder = FCPATH . 'themes/default/';
        $file_location = $folder . $record->file_url . '.php';
        $layout_data = array();
        if(file_exists($file_location)) {
            // get all widgets
            $widgets = $this->CI->widgets->getLineWithString($file_location, 'theme_view', "theme_view('", "'");
            $lines = file($file_location);
            $i = 0;
            foreach($lines as $line) {
                $trim_line = trim($line);
                if(!empty($trim_line) && isset($widgets[$i]) && strpos($line, $widgets[$i])) {
                    // note: $i is not widget position specific, get widget theme_block_id
                    $widget_info = $this->CI->db->where('file_url', $widgets[$i])->get('themes')->row();
                    if($widget_info) {
                        $widget_params = array();
                        // get existing parameters
                        $string_params = $this->CI->widgets->getStringBetween(trim($trim_line), 'array(', ');', false);
                        if(!empty($string_params)) {
                            $parameters = array_map('trim', explode(',', $string_params));
                            if(!empty($parameters)) {
                                foreach($parameters as $parameter) {
                                    $parameter = array_map('trim', explode('=>', $parameter ?? ''));
                                    if(!empty($parameter[0]) && !empty($parameter[1])) {
                                        $parameter_0 = str_replace(array('\'', '"'), '', $parameter[0]);
                                        $parameter_1 = trim(str_replace(array('\'', '"'), '', $parameter[1]));
                                        $parameter_1 = preg_replace('/\s+/','',$parameter_1);
                                        $parameter_1 = str_replace(')', '', $parameter_1);
                                        $widget_params[$parameter_0] = $parameter_1;
                                    }
                                }
                            }
                            $layout_data[$i] = array('widget' => $widget_info->id, 'theme_block_id' => (int) $widget_info->theme_block_id, 'priority' => $i, 'params' => $widget_params);
                        }
                    }
                    $i++;
                }
            }
        }
        return json_encode($layout_data);
    }
    
    /**
     * Get Random Widgets
     */
    private function _get_random_widgets($id = 'home', $theme_block_id = 1) {
        $where = "FIND_IN_SET(" . $theme_block_id . ", theme_block_id)";
        $record = $this->CI->db->where($where)->order_by('RAND()')->get('themes')->row();
        $layout_widgets = (array) $this->CI->session->userdata('layout_widgets');
        if($record) {
            $params = json_decode($record->params ?? '', true);
            $widget_id = $record->id;
            $layout_widgets[$id][$widget_id] = $params;
            // save data in a session variable to update it on save
            /*$this->CI->session->set_userdata('layout_widgets', $layout_widgets);
             if(!empty($params)) {
             foreach($params as $param) {
             $attributes[] = "'" . $param['name'] . "' => '" . $param['value'] . "'";
             }
             }
             dump('array(' . implode(', ', $attributes) . ')');*/
            $layout_data[$i] = array('widget' => $widget_id, 'priority' => $i, 'params' => $widget_params);
        }
        // save data in a session variable to update it on save
        $this->CI->session->set_userdata('layout_widgets', $layout_widgets);
    }
    
    /**
     * Get Theme Demo
     * @param number $id
     * @return object
     */
    private function get_theme_demo($id) {        
        // get design from the demos table
        $this->CI->db->from('theme_demos AS M');
        $this->CI->db->join('theme_blocks BT', 'BT.id = M.block_id', 'INNER');
        $record = $this->CI->db->select('M.*, BT.file_folder, BT.block_type_id')->where('M.id', $id)->get()->row();
        return $record;
    }
}