<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
 || ##################################################################### ||
 || # GoClixy - Online Software Solutions							   # ||
 || # Copyright (C) 2021 GoClixy. All Rights Reserved.				   # ||
 || # ----------------------------------------------------------------  # ||
 || # This file is owned and copyrighted by GoClixy.com. Your license   # ||
 || # confers  no  title  or ownership in the Script and is not a sale  # ||
 || # of any rights to the Software. This file may not be redistributed # ||
 || # in whole or significant part.									   # ||
 || # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
 || # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
 || ##################################################################### ||
 \*=======================================================================*/
/**
 * Prepare and modify all views for demo data
 * Load these views instead of calling the final one to avoid ambiguity in different time save
 * Modify views only not widgets, create new layouts with header and footer if necessary
 * Same concept will be used for multiple theme generation and test
 * Save theme in the layout_designs folder, save widgets only
 * 
 * 
    // load demo content if requested in the load_assets function
    if($this->CI->session->userdata('theme_demo_content')) { // load demo content
        $demo_content = $this->CI->session->userdata('theme_demo_content');
        if(!empty($demo_content[$layout->file_url])) {
            // $layout->file_url = $demo_content[$layout->file_url];
        }
    }
 *
 * Assumption, all widgets are in a new line
 * Process default color scheme
 */
class Theme_creator {
    /**
     * Hold codeigniter instance
     * @var object
     */
    protected $CI;
    protected $demo_folder;
    protected $demo_files;
    protected $session_key = 'theme_demo_content'; // name of the session key, use this in load_assets function
    
    /**
     * Constructor
     * @param array $config
     */
    public function __construct($config = array()) {
        $this->CI =& get_instance();
        $this->CI->load->library('themes/widgets');
        $this->demo_folder = FCPATH . 'themes/default/demo/views/';
    }
    
    /**
     * Prepare theme demo content files
     * 
     * @param string $record - row from the theme_options table
     * @param string $theme_name
     * @param boolean $reprocess
     */
    public function theme_demo_content($record, $theme_name, $scheme = null, $check_generate_session = false, $reprocess = false) {
        if($check_generate_session && $this->CI->session->userdata($this->session_key)) {
            // do not process further if session already exist
            return true;
        }
        $theme_folder = $theme_name . '/' . $record->slug;
        $demo_dir = $this->demo_folder . $theme_folder;
        if(!file_exists($demo_dir) || $reprocess) {            
            // generate files
            $this->theme_content($theme_name, $scheme, $record); 
        }
        // create directory if not exist
        if (!file_exists($demo_dir)) {
            mkdir($demo_dir, 0777, true);
        }
        $this->read_folder($demo_dir, $theme_folder . '/');
        if($check_generate_session) {
            // set session data
            $this->CI->session->set_userdata($this->session_key, $this->demo_files);
        }
    }
    
    /**
     * Read folder and return all files
     * @param string $path
     * @return array
     */
    private function read_folder($dir, $theme_folder){
        $ffs = scandir($dir);
        
        unset($ffs[array_search('.', $ffs, true)]);
        unset($ffs[array_search('..', $ffs, true)]);
        
        // prevent empty ordered elements
        if (count($ffs) < 1)
            return;
        
        foreach($ffs as $ff) {
            if(strpos($ff, '.php') !== false) {
                $view_file = str_replace($this->demo_folder . $theme_folder, '', $dir. '/'. str_replace('.php', '', $ff));
                $key = $view_file;
                $this->demo_files[$key] = 'demo/' . str_replace($this->demo_folder, '', $dir. '/'. str_replace('.php', '', $ff));
            }
            if(is_dir($dir . '/' . $ff)) $this->read_folder($dir . '/'. $ff, $theme_folder);
        }
    }
    
    /**
     * A single theme can have multiple content, generate content specific views
     * Theme with demo content is loaded using the session variable
     * 
     * E.g. $this->session->userdata('theme_demo_content');
     * 
     * @param string $theme_name - To save new file in the demos folder
     * @param string $scheme - Generate scheme specific css files
     * @param string $demo_content_slug - Slug of the demo content to be processed
     * @param array $demo_content - demo_content row from the theme_options file
     * 
     * Schemes are theme specific not content specific, i.e. generate them as follows
     * css/demo/gocart/css/$scheme
     * js/demo/gocart/js
     */
    private function theme_content($theme_name = null, $scheme_name = null, $demo_content = array(), $demo_content_slug = 'fruits-vegetables') {
        if(empty($theme_name)) {
            // get the current theme
            $theme = $this->CI->db->where(array('is_default' => 1, 'type' => 'themes'))->get('theme_settings')->row();
            $theme_name = $theme && !empty($theme->name) ? $theme->name : 'default';
        }
        
        // get content information from the metas field
        if(empty($demo_content)) {
            $demo_content = $this->CI->db->where('slug', $demo_content_slug)->get('theme_options')->row();
        }
        if(!$demo_content && empty($demo_content->metas)) {
            return false;
        }
        $demo_data_params = json_decode($demo_content->metas ?? '');
        $demo_data_params = json_decode($demo_data_params->options ?? '', true);
        if(empty($demo_data_params)) {
            return false;
        }
        
        // generate layouts, css and js files
        $this->_layouts_css_js($theme_name, $scheme_name);
        
        // sort array by position
        // position keyword is important, 0 means the first match, 1 means the second match and so on
        array_multisort(array_column($demo_data_params, 'position'), SORT_ASC, $demo_data_params);
        
        $params_demo_data = array(); // convert $demo_data_params into a specific format for easy match
        foreach($demo_data_params as $demo_data_param) {
            // only one position is allowed
            $page_params = json_decode($demo_data_param['params'] ?? '', true);
            $pages_param = array();
            switch($demo_data_param['page']) {
                case 'home':
                    $pages_param[] = 'home';
                    break;
                case 'view':
                    $pages_param[] = 'modules/products/view';
                    break;
                case 'detail':
                    $pages_param[] = 'modules/products/detail';
                    break;
                case '':
                    $pages_param[] = 'home';
                    $pages_param[] = 'modules/products/view';
                    $pages_param[] = 'modules/products/detail';
                    break;
                    
            }
            foreach($pages_param as $page_param) {
                // home, detail, view etc. specific params
                $params_demo_data[$page_param][$demo_data_param['block']][$demo_data_param['position']] = $page_params;
            }
        }
        $records = $this->CI->db->where(array('type_id' => 0, 'theme' => $theme_name))->get('themes')->result();
        if($records) {
            $folder = FCPATH . 'themes/default/';
            foreach($records as $record) {
                switch($record->file_folder) {
                    case 'home':
                    case 'modules/products/view':
                    case 'modules/products/detail':
                        if(!empty($params_demo_data[$record->file_folder])) { // must have parameter
                            // $file_name = $record->file_folder . '.php';
                            // $file_location = $folder . $file_name;
                            $file_location = $folder . $record->file_url . '.php';
                            if(file_exists($file_location)) {
                                // get all widgets
                                $widgets = $this->CI->widgets->getLineWithString($file_location, 'theme_view', "theme_view('", "'");
                                $lines = file($file_location);
                                $i = 0;
                                $j = 0; // theme block specific position
                                $widget_positions = array(); // hold widget positions
                                foreach($lines as $lkey => $line) {
                                    $trim_line = trim($line);
                                    if(!empty($trim_line) && isset($widgets[$i]) && strpos($line, $widgets[$i])) {
                                        // note: $i is not widget position specific, get widget theme_block_id
                                        $widget_info = $this->CI->db->where('file_url', $widgets[$i])->get('themes')->row();
                                        if($widget_info) {
                                            if(array_key_exists($widget_info->theme_block_id, $widget_positions)) {
                                                $widget_positions[$widget_info->theme_block_id] += 1;
                                            } else {
                                                $widget_positions[$widget_info->theme_block_id] = 0;
                                            }
                                            if(!empty($params_demo_data[$record->file_folder][$widget_info->theme_block_id][$widget_positions[$widget_info->theme_block_id]])) { // the block has parameters
                                                // do not modify a line which is commented
                                                if(strpos($line, '// echo') === false) {
                                                    $demo_data_specific_params = $params_demo_data[$record->file_folder][$widget_info->theme_block_id][$widget_positions[$widget_info->theme_block_id]];
                                                    $widget_params = array();
                                                    // get existing parameters
                                                    $string_params = $this->CI->widgets->getStringBetween(trim($trim_line), 'array(', ');', false);
                                                    if(!empty($string_params)) {
                                                        $parameters = array_map('trim', explode(',', $string_params));
                                                        if(!empty($parameters)) {
                                                            foreach($parameters as $parameter) {
                                                                $parameter = array_map('trim', explode('=>', $parameter ?? ''));
                                                                if(!empty($parameter[0]) && !empty($parameter[1])) {
                                                                    $parameter_0 = str_replace(array('\'', '"'), '', $parameter[0]);
                                                                    $parameter_1 = trim(str_replace(array('\'', '"'), '', $parameter[1]));
                                                                    $parameter_1 = preg_replace('/\s+/','',$parameter_1);
                                                                    // check for the extra bracket at the end and replace it
                                                                    /*$last_string = substr( $parameter_1, strrpos( $parameter_1, '-' ) + 1);
                                                                     if($last_string === ')') {
                                                                     dump($parameter_1);
                                                                     
                                                                     }*/
                                                                    $parameter_1 = str_replace(')', '', $parameter_1);
                                                                    $widget_params[$parameter_0] = $parameter_1;
                                                                }
                                                            }
                                                        }
                                                    }
                                                    // prepare widget with new parameters
                                                    $params_new = array();
                                                    $param_widget = '';
                                                    $new_params = array_merge($widget_params, $demo_data_specific_params);
                                                    if(!empty($new_params)) {
                                                        foreach($new_params as $pkey => $pvalue) {
                                                            if(is_numeric($pvalue)) {
                                                                $params_new[] = "'" . $pkey . "' => " . $pvalue . "";
                                                            } else {
                                                                $params_new[] = "'" . $pkey . "' => '" . $pvalue . "'";
                                                            }
                                                        }
                                                        $param_widget = "', " . 'array(' . implode(', ', $params_new) . ')';
                                                    }
                                                    $widget_with_new_param = "<?php echo theme_view('" . $widgets[$i] . $param_widget . "); ?>";
                                                    $lines[$lkey] = str_replace($trim_line, $widget_with_new_param, $line);
                                                }
                                            }
                                        }
                                        $i++;
                                    }
                                }
                                /**
                                 * Once a demo file is processed, save it in a specific folder with theme name to avoid unnecessary processing
                                 * @var string $destination_file
                                 */
                                $destination_file = $this->demo_folder . $theme_name . '/' . $demo_content_slug . '/' . $record->file_url . '.php';
                                $demo_dir = substr($destination_file, 0,strrpos($destination_file, '/'));
                                if (!file_exists($demo_dir)) {                                    
                                    mkdir($demo_dir, 0777, true);
                                }
                                $this->CI->load->helper('file');
                                write_file($destination_file, implode('', $lines));
                            }
                        }
                        break;
                }
            }
        }
    }
    
    /**
     * Process remaining Layouts and CSS files
     * @param string $theme_name - Name of the theme
     * @param array $scheme - Scheme data from the theme_options table
     */
    private function _layouts_css_js($theme_name, $scheme_name) {
        // get the scheme
        $scheme = $this->CI->db->where(array('name' => $scheme_name, 'type' => 'schemes'))->get('theme_settings')->row();        
        clear_cache('default_theme'); // clear the session variable
        $theme_settings = theme_settings($theme_name); // get header and footer to be processed
        $where = "(theme = '" . $theme_name . "' OR type_id = 2) AND is_theme_default = 1";// array('theme' => $theme_name, 'is_theme_default' => 1);
        $views = $this->CI->themes_model->select('id, type_id, layout, file_folder')->find_all_by($where);
        if($views) {
            foreach($views as $view) {
                // Get Default Scheme
                $scheme_values = (array) json_decode($scheme->value ?? '');
                $this->CI->widgets->process_view($view->id, $scheme_values, false, $theme_name, array('theme_demo' => true, 'scheme' => $scheme_name, 'theme_settings' => $theme_settings));
            }            
            // Compile global.scss file
            $this->CI->widgets->compile_global();
        }
    }
}