<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
 || ##################################################################### ||
 || # GoClixy - Online IT Solutions							            # ||
 || # Copyright (C) 2021 GoClixy. All Rights Reserved.				    # ||
 || # ----------------------------------------------------------------  # ||
 || # This file is owned and copyrighted by GoClixy.com. Your license   # ||
 || # confers  no  title  or ownership in the Script and is not a sale  # ||
 || # of any rights to the Software. This file may not be redistributed # ||
 || # in whole or significant part.									    # ||
 || # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
 || # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
 || ##################################################################### ||
 \*=======================================================================*/
use MatthiasMullie\Minify; // Minify JS files https://github.com/matthiasmullie/minify
use ScssPhp\ScssPhp\Compiler; // SASS Compiler http://leafo.github.io/scssphp/
class Widgets {
    /**
     * // VIEW_FILE PLACEHOLDER - Hardcoded in each view file
     *
     * @access public
     *
     * @var object
     */
    protected $CI;
    public $widget_path;
    public $save_path;
    
    protected $migration_filename = 'theme_migrations'; // Migration File Name
    
    protected $fonts;				// store all used fonts in this variable
    protected $scheme = array();	// for color schemes, ignore local variables and process all SCSS files using color SCSS values
    
    protected $assets_js; 			// store links of all js scripts
    protected $assets_css; 			// store links of all css files
    
    
    protected $contents_scss; 		// get file contents of each scss file and store here after settings the variables
    protected $contents_js;			// get file contents of each javascript file and store here
    
    protected $default_scss;		// define global scss files
    protected $default_js;			// define global js files
    
    protected $view_widgets;		// store all widgets used in a view (inner also)
    protected $theme_settings;      // get theme settings from the database
    protected $z_cache;
    protected $widget_cache;
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct($config = array()) {
        $this->CI =& get_instance();
        
        // Load Zend Caching
        $this->CI->load->library('slug');
        $this->CI->load->library('gzf_cache', array('lifetime' => 900));
        $this->widget_cache = $this->CI->gzf_cache->get_instance();
        
        $this->widget_path = FCPATH . 'themes/default/';
        $this->save_path = FCPATH . 'assets/';
        
        $this->default_scss = array(
            '@import "../scss/themes/abstracts/_variables.scss";',
            // '@import "../scss/hacks.scss";'
        );
        
        $this->default_js = $this->widget_path . 'js/variables.js';
        $this->theme_settings = theme_settings();
    }
    
    /**
     * Allow users to edit static files
     * @return string[][]
     */
    public function static_files() {
        $static_files = array(	// static files like _variables.scss, _utilites.scss etc.
            array(
                'name' => 'variables', 	// file name cannot be same
                'extension' => 'scss',
                'url' => 'scss/themes/abstracts/_variables.scss',
                'path' => 'scss/'
            ),
            array(
                'name' => 'utilities',
                'extension' => 'scss',
                'url' => 'scss/themes/base/_utilities.scss',
                'path' => 'scss/'
            )
        );
        // read themes scss files
        $theme_dir = FCPATH . 'themes/default/scss/themes';
        $files = scandir($theme_dir, 0);
        for($i = 2; $i < count($files); $i++) {
            if(strpos($files[$i], '.scss') !== false) {
                $theme_file = $files[$i];
                $static_files[] = array(
                    'name' => str_replace('.scss', '', $theme_file),
                    'extension' => 'scss',
                    'url' => 'scss/themes/' . $theme_file,
                    'path' => 'scss/'
                );
            }
        }
        return $static_files;
    }
    
    /**
     * Save theme scss file
     * @param string $theme
     * @return boolean
     */
    public function save_theme_scss($theme) {
        $theme_dir = FCPATH . 'themes/default/scss/themes';
        if(!file_exists($theme_dir . '/' . $theme . '.scss')) {
            $content = <<<EOL
@import "base/_base.scss";
@import "base/_fonts.scss";
@import "base/_typography.scss";
@import "base/_utilities.scss";
@import "components/_general.scss";
.heading {
	&-theme,
	&-theme > a {
		margin-bottom: 1rem;
		border-bottom: 1px solid #eee;
    	font-size: 1.75rem;
    	font-weight: 400;
	    padding-bottom: 1rem;
	    margin-top: 0.5rem;
    	text-decoration: none;
	}
	&-secondary {
		text-align: center;
	    padding-bottom: .5rem;
	    margin-top: 0.5rem;
    	text-decoration: none;
	}
	&-sub {
		font-weight: 300;
	}
}
.heading {
	&-theme,
	&-theme > a {
    	color: \$theme-primary;
	}
	&-secondary,
	&-secondary > a {
		color: \$link-color;
	}
}
EOL;
            // Write the changes out...
            if (! function_exists('write_file')) {
                $this->CI->load->helper('file');
            }
            
            // save file
            if (write_file($theme_dir . '/' . $theme . '.scss', $content)) {
                return true;
            }
        }
    }
    
    /**
     * Process view in the theme folder
     * @param string $view_id
     * @param array $scheme - For Color Schemes, compile all scss files using global variables
     * @param boolean $update_scheme_scss - Update Color Scheme value in the VIEW file, else the next time process of the same view will use the saved default values
     * @param string $theme_name - Name of the theme
     * @param array $params - Extra parameters to be passed to this function
     */
    public function process_view($view_id, $scheme = array(), $update_scheme_scss = false, $theme_name = '', $params = array()) {
        if(!empty($params['theme_settings'])) {
            // Passed while creating theme demo files
            $this->theme_settings = $params['theme_settings'];
        }
        $this->scheme = $scheme;
        
        // get view information
        if(empty($params['record'])) {
            $record = $this->CI->themes_model->find($view_id);
        } else {
            $record = $params['record'];
        }
        if($record) {
            // Do not process previously processed VIEW css and js files
            $this->assets_css = array();
            $this->assets_js = array();
            $this->contents_scss = null;
            $this->contents_js = array();
            // get view type - home/detail/view etc.
            $view = $this->CI->db->select('view')->get_where('theme_views', array('theme_id' => $view_id))->row();
            if($view) {
                /**
                 * Update Color Scheme value in the VIEW file,
                 * else the next time process of the same view will use the saved default values
                 *
                 * Note: the Layout/View file must have widget CSS, so fill them using Auto Fill SCSS option
                 */
                if(!empty($scheme) && $update_scheme_scss) {
                    $existing_widgets = !empty($record->scss_values) ? json_decode($record->scss_values) : null;
                    if(!empty($existing_widgets)) {
                        foreach($existing_widgets as $existing_id => $existing_widget) {
                            foreach($existing_widget as $existing_key => $existing_value) {
                                if(isset($scheme[$existing_key])) {
                                    $existing_widgets->{$existing_id}->{$existing_key} = $scheme[$existing_key];
                                }
                            }
                        }
                    }
                    $new_scss_values = json_encode($existing_widgets);
                    $this->CI->db->where(array('id' => $view_id))->update('themes', array('scss_values' => $new_scss_values));
                }
                /**
                 * Settings any file inside MEMEBERS view as default (e.g. invoices, view_comments etc.) should not process default SCSS files
                 * (e.g. _utilities.scss and _fonts.scss) as they have already been processed by the Layout view file.
                 *
                 * CONDITION - EACH LINK (e.g. LEADS, ADS etc.) IN THE MEMBER AREA MUST HAVE THE VIEW FILE INSIDE THE MEMBERS FOLDER TO AVOID PROCESSING OF DEFAULT FILES.
                 *
                 * Note: we cannot use WIDGETS for the MEMBERS View because of the following reasons:
                 *
                 * 1) The Admin will loose the feature of changing view for the Member's Area.
                 * 2) Widgets cannot be set as default, therefore, the admin has to manually update the code to change the MEMBERS View.
                 * 3) In order to change a MEMBERS design, the admin need to process the MEMBER view each time on code change.
                 */
                $members_view = false;
                /*if(strpos($view->view, 'members/') !== false) {
                 $members_view = true;
                 }*/
                if($record->type_id != 2) {
                    $members_view = true;
                }
                // check file existence
                $filepath = $this->widget_path . $record->file_url;
                $php_file = $filepath . '.php';
                // Theme Type
                switch($record->type_id) {
                    case 1:
                        $folder = 'widgets/';
                        break;
                    case 2:
                        $folder = 'layouts/';
                        break;
                    default:
                        $folder = 'views/';
                }
                $scss_file = $this->widget_path . $folder . $record->file_folder . '/scss/' . $record->file_name . '.scss';
                $js_file = $this->widget_path .  $folder . $record->file_folder . '/js/' . $record->file_name . '.js';
                if($this->check_existence($php_file)) {
                    
                    // Process the View/Layout External CSS and JS Dependencies
                    if($record->type_id == 0 || $record->type_id == 2) {
                        $assets_css = !empty($record->assets_css) ? json_decode($record->assets_css) : null;
                        // The link can be either relative or absolute.
                        if(!empty($assets_css)) {
                            foreach($assets_css as $asset_css) {
                                $this->assets_css[] = $asset_css->url;
                            }
                        }
                        $assets_js = !empty($record->assets_js) ? json_decode($record->assets_js) : null;
                        if(!empty($assets_js)) {
                            foreach($assets_js as $asset_js) {
                                $this->assets_js[] = $asset_js->url;
                            }
                        }
                    }
                    
                    $scss_values = !empty($record->scss_values) ? json_decode($record->scss_values) : null;
                    // Compile default scss files first
                    // global name is stored in the theme_settings table and it can only be changed through database
                    // $this->compile_scss("../scss/themes/abstracts/_variables.scss", 'global', NULL, $scss_values);
                    // Default and Theme specific SCSS files will be compiled only if it is a Layout
                    // Note: They will not be processed in case of a VIEW and WIDGET
                    if(!$members_view) {
                        //$hacks_path = $this->widget_path . "scss/hacks.scss";
                        /*if($this->theme_scheme_process($hacks_path)) {
                         $this->theme_scheme_update($hacks_path, $this->compile_scss($hacks_path, 'hacks', NULL, $scss_values), 'scss');
                         } */
                        
                        // Each theme can have its own SCSS file
                        // Create a scss file for each theme and call it on each page to maintain consistency
                        // across all pages e.g. common title, background, heading etc.
                        $theme_name = empty($theme_name) ? $record->theme : $theme_name;
                        $theme_scss_path = $this->widget_path . "scss/themes/" . $theme_name . ".scss";
                        
                        // Do not process the hacks and theme scss files twice in case of Default Theme or Scheme set
                        if($this->theme_scheme_process($theme_scss_path)) {
                            $this->theme_scheme_update($theme_scss_path, $this->compile_scss($theme_scss_path, $record->theme, NULL, $scss_values), 'scss');
                        }
                    }
                    
                    // Compile view specific scss file before widget scss files
                    if($this->check_existence($scss_file)) {
                        $this->compile_scss($scss_file, $record->id, NULL, $scss_values);
                    }
                    
                    // Compile view specific Javascript file
                    if($this->check_existence($js_file)) {
                        $this->contents_js[] = $js_file;
                    }
                    
                    // get all widgets inside this view
                    $widgets = $this->getLineWithString($php_file, 'theme_view', "theme_view('", "'", true);
                    if($widgets) {
                        // all widgets with parameters to process _call specific dynamic widgets
                        $widgets_with_params = $widgets;
                        $param_widgets = array();
                        foreach($widgets as $widget) {
                            $param_widgets[] = $widget['view'];
                        }
                        $widgets = $param_widgets;
                        // a widget can be used multiple times in the same view
                        // process such widgets only once
                        $widgets = array_unique($widgets);
                        // process widgets depedencies (SCSS and JS files) one by one
                        foreach($widgets as $widget) {
                            // process navbar and footers using the saved settings
                            switch($widget) {
                                case 'widgets/navbars/':
                                    $widget = $widget . $this->theme_settings['navbar'];
                                    break;
                                case 'widgets/footers/':
                                    $widget = $widget . $this->theme_settings['footer'];
                                    break;
                            }
                            $widget_info = $this->CI->themes_model->find_by('file_url', $widget);
                            if($widget_info) {
                                $this->recursive_widgets($widget_info, $scss_values, $widgets_with_params);
                            }
                        }
                    }
                    
                    // After widget process, write view file in the destination file
                    $this->create_view($record->file_url, $view->view, $record->type_id, $view_id, $theme_name, $params);
                    
                    // $this->merge_files($views, 'scss', $destination);
                    // $this->merge_files($views, 'js', $destination);
                    return true;
                    
                }
            }
        }
        return false;
    }
    
    /**
     * Do not process a widget, view and layouts twice in case of Default Theme or Scheme set
     * @param string $filepath
     * @return boolean
     */
    private function theme_scheme_process($filepath) {
        // if the admin is setting a default scheme or a theme then
        // do not process the same widget, view and layouts twice
        if( $this->CI->session->userdata('theme_scheme_process')) {
            $theme_scheme_process = $this->CI->session->userdata('theme_scheme_process');
            // check the file existence in this variable
            if(array_key_exists($filepath, $theme_scheme_process)) {
                if(!empty($theme_scheme_process[$filepath]['scss'])) {
                    $this->contents_scss .= $theme_scheme_process[$filepath]['scss'];
                }
                return false;
            } else {
                $theme_scheme_process[$filepath] = $filepath;
                $this->CI->session->set_userdata('theme_scheme_process', $theme_scheme_process);
            }
        }
        return true;
    }
    
    /**
     * Update the session variable
     * @param string $filepath
     * @param string $value
     * @param string $type - scss/js
     */
    private function theme_scheme_update($filepath, $value, $type) {
        if($this->CI->session->userdata('theme_scheme_process')) {
            $theme_scheme_process = $this->CI->session->userdata('theme_scheme_process');
            $theme_scheme_process[$filepath] = array(
                $type => $value
            );
            $this->CI->session->set_userdata('theme_scheme_process', $theme_scheme_process);
        }
    }
    
    /**
     * Recursive function to process internal widgets
     * @param object $row - widget row from the themes table
     * @param array $scss_variables
     * @param array $widget_with_params
     */
    private function recursive_widgets($row, $scss_variables, $widget_with_params = array()) {
        $type_id = $row->type_id;
        $assets_js = $row->assets_js;
        $assets_css = $row->assets_css;
        $filepath = $this->widget_path . $row->file_url . '.php';
        
        // Do not process a widget, view and layouts twice in case of Default Theme or Scheme set
        // Creating issues - some widgets are not processed correctly
        /*if(!$this->theme_scheme_process($filepath)) {
         return false;
         }*/
        
        if($this->check_existence($filepath)) {
            
            // get recursive/inner widgets in the widget file
            $widgets = $this->getLineWithString($filepath, 'theme_view', "theme_view('", "'");
            
            // process widgets called using a dynamic variable - July 18, 2021
            if((strpos($row->file_url, '_call') !== false) && !empty($widget_with_params)) {
                $widgets = array();
                // get folder path where the file exist
                $path_widget_array = explode('/', $row->file_url);
                array_pop($path_widget_array);
                $path_widget = implode('/', $path_widget_array);
                foreach($widget_with_params as $wkey => $widget_with_param) {
                    if($widget_with_param['view'] == $row->file_url) {
                        if(!empty($widget_with_param['params']['load'])) {
                            $path_widget = $path_widget_array[0] . '/' . $widget_with_param['params']['load'];
                        }
                        // the name widget is the name of the file
                        if(!empty($widget_with_param['params']['widget'])) {
                            $widget_url = $path_widget . '/' . $widget_with_param['params']['widget'];
                            // same key is used to avoid duplication
                            $widgets[$widget_url] = $widget_url;
                        }
                    }
                }
            }
            
            // get external files dependency first
            $this->assets_files('css', $assets_css);
            $this->assets_files('js', $assets_js);
            
            // set SCSS file variables and compile it
            switch($type_id) {
                case 1:
                    $file_root = 'widgets/';
                    break;
                case 2:
                    $file_root = 'layouts/';
                    break;
                default:
                    $file_root = 'views/';
            }
            $scss_file = $this->widget_path . $file_root . $row->file_folder . '/scss/' . $row->file_name . '.scss';
            $this->compile_scss($scss_file, $row->id, (!empty($row->scss_values) ? json_decode($row->scss_values) : null), $scss_variables);
            
            // get Javascript code related to this particular widget
            $js_file = $this->widget_path . $file_root . $row->file_folder . '/js/' . $row->file_name . '.js';
            if($this->check_existence($js_file)) {
                $js_file_url = $this->widget_path . $file_root . $row->file_folder . '/js/' . $row->file_name . '.js';
                $this->contents_js[] = $js_file_url;
            }
            
            // inner widgets exists
            if(!empty($widgets)) {
                // a widget can be used multiple times in the same view
                // process such widgets only once
                $widgets = array_unique($widgets);
                foreach($widgets as $widget) {
                    // $folder = substr($widget, 0, strrpos( $widget, '/'));
                    // $filename = substr( $widget, strrpos( $widget, '/' ) + 1);
                    $record = $this->CI->themes_model->find_by(array('type_id' => 1, 'file_url' => $widget));
                    if($record) {
                        $this->recursive_widgets($record, $scss_variables);
                    }
                }
            }
        }
    }
    
    /**
     * Widget has dependency on external CSS & JS files
     * Process depedencies first before going further
     * @param string $type - CSS/JS
     * @param array $assets
     */
    private function assets_files($type, $assets) {
        if(!empty($assets)) {
            $files_array = !empty($assets) ? json_decode($assets) : null;
            foreach($files_array as $asset) {
                if(!empty($asset->url)) {
                    // Has dependency on external file either from CDN or from some external source
                    /*if($asset->type == 'external') {
                     if($type == 'js') {
                     $this->assets_external_js[] = $asset->url;
                     } else if($type == 'css')  {
                     $this->assets_external_css[] = $asset->url;
                     }
                     } else {
                     
                     }*/
                    if($type == 'js') {
                        $this->assets_js[] = $asset->url;
                    } else if($type == 'css')  {
                        $this->assets_css[] = $asset->url;
                    }
                }
            }
        }
    }
    
    /**
     * Set SCSS file variables through PHP and compile it
     * @param int $widget_id
     * @param array $widget_options - Default Values
     * @param array $view_options - New values
     */
    private function compile_scss($scss_file, $widget_id, $widget_options, $view_options) {
        // check the scss file existence
        if($this->check_existence($scss_file)) {
            // get file contents
            // $contents = file_get_contents($scss_file);
            $contents = file($scss_file); // Place file contents in an array
            if(!empty($contents)) {
                
                // initialize scss library
                $scss = new Compiler();
                
                // contains methods for use by other SCSS files like links, fonts settings etc.
                // Note: Append mixins to the file contents only after passing variables to avoid variable errors
                // Note: This is not included in the Theme file as it is already used by other widgets
                $mixins = file_get_contents(realpath($this->widget_path . 'scss/themes/abstracts/_mixins.scss'));
                
                // Apply selected color scheme to all Default Views
                if(!empty($this->scheme)) {
                    $variables = $this->get_variables((object)$this->scheme);
                } else {
                    // if the user has changed the widget SCSS file variables then
                    // set value of these variables through PHP
                    // find such widgets by matching the view_options retrieved id with the widget id.
                    if(!empty($view_options) && $this->_key_exists($widget_id, $view_options)) {
                        $variables = $this->get_variables($view_options->$widget_id);
                    } else if(!empty($widget_options->$widget_id)) {
                        // get default values from the widget scss_variables column
                        $variables = $this->get_variables($widget_options->$widget_id);
                    } else {
                        // the widget has no variables
                        // process file using scss/themes/abstracts/_variables.scss file else SCSS compiler will throw exceptions
                        // all variables are defined in the scss/themes/abstracts/_variables.scss file
                        $variables = $this->get_variables();
                    }
                }
                
                // compile SCSS contents in CSS
                try {
                    if(!empty($variables['img-url'])) {
                        $variables['img-url'] = "'" . $variables['img-url'] ."'";
                    }
                    // SEO Hide H1 text in the Header Logo
                    $variables['logo-url'] = "'" . gc_image_url(settings_item('site.logo'), false) ."'";
                    $scss->setVariables($variables);
                    // Append import files content to the CSS file
                    $contents = $this->get_imported_scss($contents);
                    $contents = $mixins . $contents;
                    $this->contents_scss .= $scss->compile($contents);
                    return $this->contents_scss;
                } catch (exception $e) {
                    echo $e->getMessage();
                }
            }
        }
    }
    
    /**
     * Get all imported SCSS files
     * In case of theme, 7-1 CSS Architecture is followed
     * If the file has any imported SCSS files also append their contents to the generated CSS file.
     * @param string $lines
     */
    private function get_imported_scss($lines) {
        // get all variables
        if(!empty($lines)) {
            foreach ($lines as $lineNumber => $line) {
                if (strpos($line, '@import') !== false) {
                    // get file name
                    $name = trim($this->getStringBetween($line, '@import "','"'));
                    // after retrieving filename, remove the import statement from the line
                    $lines[$lineNumber] = '';
                    if($name) {
                        // check if the filename has an extension with .scss
                        if (strpos($name, '.scss') === false) {
                            $name .= '.scss'; // Append extension with the filename
                        }
                        $filePath = realpath($this->widget_path . 'scss/themes/' . $name);
                        if(file_exists($filePath)===TRUE) { // get file contents
                            $content = file_get_contents($filePath);
                            if(!empty($content)) {
                                if(strpos($name, '_fonts.scss') !== false) {
                                    // Get Google Fonts URL
                                    $google_fonts = $this->import_fonts();
                                    if($google_fonts) {
                                        $content .=  $google_fonts;
                                    }
                                }
                                $lines[$lineNumber] = $content; // Append the content to the same line number
                            }
                        }
                    }
                }
            }
        }
        return implode('', $lines);
    }
    
    /**
     * Get all variables from a SCSS file
     * if the variable is not defined, get its value from the _variables.scss file
     * @param array $scss_values
     */
    private function get_variables($scss_values = array()) {
        $fileName = $this->widget_path . 'scss/themes/abstracts/_variables.scss';
        $lines = file($fileName);
        $variables = array();
        // Global settings save multiple information for a single attribute
        // Whereas color scheme store variable name as the key and value as the value of the attribute
        // i.e. they must be processed separately
        $global = false;
        // Try to get default values from the applied color scheme
        $result = $this->CI->db->get_where('theme_settings', array('type' => 'schemes', 'is_default' => 1))->row();
        if(!$result || ($result && empty($result->value))) {
            // get values from the global variables defined under the Global Variables tab
            $result = $this->CI->db->select('name, value')->get_where('theme_settings', array('name' => 'global'))->row();
            $global = true;
        }
        $global_variables = !empty($result->value) ? json_decode($result->value) : null;
        $global_values = array();
        if(!empty($global_variables)) {
            if($global) {
                foreach($global_variables as $var) {
                    $global_values[$var->name] = $var->value;
                    // get fonts name to process it later
                    if($var->name == 'font-custom') {
                        $this->fonts[] = $var->value;
                    }
                }
            } else {
                foreach($global_variables as $var_key => $var_value) {
                    $global_values[$var_key] = $var_value;
                    // get fonts name to process it later
                    if($var_key == 'font-custom') {
                        $this->fonts[] = $var_value;
                    }
                }
            }
        }
        // get all variables
        foreach ($lines as $lineNumber => $line) {
            if (strpos($line, '$') !== false) {
                // get variable name starting with $
                $name = trim($this->getStringBetween($line,'$',':'));
                // if the value is already defined by the user
                // either in the Widget or in the View then use that value
                if(!empty($scss_values) && $this->_key_exists($name, $scss_values)) {
                    // get fonts name to process it later
                    if($name == 'font-custom') {
                        $this->fonts[] = $scss_values->$name;
                    }
                    $variables[$name] = $scss_values->$name;
                } elseif(!empty($global_values) && $this->_key_exists($name, $global_values)) {
                    // values are not defined neither in the widget nor in the view
                    // however, global values are set via Global Variables
                    // use these values
                    $global_values = (object) $global_values;
                    $variables[$name] = $global_values->$name;
                    // get fonts name to process it later
                    if($name == 'font-custom') {
                        $this->fonts[] = $global_values->$name;
                    }
                } else { // the value is not defined get it from _variables.scss file
                    $value = trim($this->getStringBetween($line, '$' . $name .':',';'));
                    $variables[$name] = $value;
                    // get fonts name to process it later
                    if($name == 'font-custom') {
                        $this->fonts[] = $value;
                    }
                }
            }
        }
        return $variables;
    }
    
    /**
     * Check key existence in an Obejct
     * @param int $id
     * @param array $values
     * @return boolean
     */
    private function _key_exists($id, $values) {
        foreach($values as $key => $value) {
            if($id == $key) {
                return true;
            }
        }
        return false;
    }
    
    /**
     * Iterate through each folder, join and minify each type of file
     * @param array $views
     * @param string $destination - File Name
     * @param string $type - scss, css, js
     * @param number $type_id - 2 -> Layout, 0 -> View
     * @param number $view_id - To get assets of the view
     * @param string $theme_name - Name of the theme
     * @param array $params - Extra Parameters to be passed to the function
     *
     * http://stackoverflow.com/questions/11901521/replace-string-in-text-file-using-php
     *
     * e.g. if the view name is Home, $destination must be passed as home
     * file will be saved as follows home.css and home.js
     *
     */
    private function create_view($view, $destination, $type_id, $view_id, $theme_name = null, $params = array()) {
        /**
         * Settings any file inside MEMEBERS view as default (e.g. invoices, view_comments etc.) should not process default SCSS files
         * (e.g. _utilities.scss and _fonts.scss) as they have already been processed by the Layout view file.
         */
        $members_view = false;
        if($type_id != 2) {
            $members_view = true;
        }
        
        /*if(strpos($view, '/members/') !== false) {
         $members_view = true;
         }
         /*-----------------------------------------*/
        /* FONTS handling and CSS creation
         /*-----------------------------------------*/
        //$css_path = $this->widget_path . 'css/' . $destination . '.css';
        //write_file($css_path, );
        $css_contents = $this->contents_scss;
        
        // css and js folder path
        $css_folder = 'css/' . $destination;
        $js_folder = 'js/' . $destination;
        $layout_folder = $destination;
        
        // create demo folders, if they do not exist
        if(!empty($params['theme_demo'])) {
            // create demo files in the public_html/themes/default/demo/themes folder
            // load these files on demo different demo load
            $theme_folder = 'demo/';
            
            // css files must be stored in the css folder to make Assets::add_css work
            // e.g. css/demo/gocart/orange/<CSS FILES>
            $scheme = 'default';
            if(!empty($params['scheme'])) {
                $scheme = $params['scheme'];
            }
            $css_folder = 'css/demo/' . $theme_name . '/' . $scheme . '/' . $destination;
            
            // js files must be stored in the js folder to make Assets::add_js work
            // e.g. js/demo/gocart/<JS FILES>
            $js_folder = 'js/demo/' . $theme_name . '/' . $destination;
            
            $layout_folder = $theme_folder . 'layouts/' . $theme_name . '/_home';
            $this->mkdir_demo_folders($css_folder);
            $this->mkdir_demo_folders($js_folder);
            $this->mkdir_demo_folders($layout_folder);
            $layout_folder = $theme_folder . 'layouts/' . $theme_name . '/' . $destination;
        }
        
        // write css file from the compiled scss files
        $this->save_file($css_folder, 'css', $css_contents);
        /*// do not add home.css to the header as this file is already added by the Home controller
         if($destination != 'home' && !empty($this->contents_scss)) {
         $this->assets_css[] = $destination . '.css';
         }*/
        /*-----------------------------------------*/
        /* View specific inner JS files handling
         /*-----------------------------------------*/
        if(!empty($this->contents_js)) {
            $minifier = new Minify\JS($this->default_js);
            $minifier->add($this->contents_js);
            // save minified file to disk
            $minifiedPath = $this->widget_path . $js_folder . '.js';
            $minifier->minify($minifiedPath);
            /*// do not add home.js to the header as this file is already added by the Home controller
             if($destination != 'home' && !empty($this->contents_js)) {
             $this->assets_js[] = $destination . '.js';
             }*/
        }
        
        // if the view is home, remove home.css and home.js from the array
        $assets = array();
        $assets['css'] = array_flip(array_unique(array_flip($this->assets_css)));
        $assets['js'] = array_flip(array_unique(array_flip($this->assets_js)));
        // Update assets information to process it in the controller files
        $this->CI->db->where('id', $view_id)->update('themes', array('assets' => json_encode($assets)));
        
        // Finally Write the Layout file
        if($type_id == 2) { // Write Layouts Only
            $this->write_view($assets, $view, $layout_folder, $view_id);
        }
    }
    
    /**
     * Write View File
     * @param string $view - Name of the view
     * @param string $destination - File Name
     */
    private function write_view($assets, $view, $destination, $view_id) {
        $contents = '<?php
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Online IT Solutions							           # ||
|| # Copyright (C) 2021 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
?>';
        /**
         * Do not add header and footer to an internal view
         * Views like members/favorites, members/index, users/activate etc.
         * The parent (member.php) already has Header and Footer view
         */
        if (strpos($destination, 'layouts/') === false && strpos($destination, '/') !== false) {
            $contents .= "\n<?php echo theme_view('" . $view . "'); ?>";
        } else {
            // external css and js dependencies are passed as a string using json_encode
            if(!empty($assets['csss']) || !empty($assets['jss'])) {
                $contents .= "\n<?php echo theme_view('_header', array('assets' => '" . json_encode($assets) ."')); ?>";
            } else {
                $contents .= "\n<?php echo theme_view('_header'); ?>";
            }
            $contents .= "\n<?php echo theme_view('" . $view . "'); ?>";
            $contents .= "\n<?php echo theme_view('_footer'); ?>";
        }
        $this->save_file($destination, 'php', $contents);
    }
    
    /**
     * Import Google Fonts
     * @return string
     */
    private function import_fonts() {
        // get unique fonts
        $google_fonts = array();
        $selected_fonts = array();
        if(is_array($this->fonts)) {
            $selected_fonts = array_unique($this->fonts);
        }
        $fonts = array();
        $result = $this->CI->db->select('value')->get_where('theme_settings', array('name' => 'fonts'))->row();
        if($result && !empty($result->value)) {
            $fonts = !empty($result->value) ? json_decode($result->value) : null;
        }
        if(!empty($selected_fonts) && !empty($fonts)) {
            foreach($selected_fonts as $selected_font) {
                foreach($fonts as $font) {
                    if($selected_font == $font->name) {
                        // get font type
                        if($font->type == 'google') {
                            // replace space with +
                            $google_fonts[] = str_replace(' ', '+', $font->name);
                        }
                    }
                }
            }
        }
        if(!empty($google_fonts)) {
            $import_url = "@import url('https://fonts.googleapis.com/css?family=" . implode('|', $google_fonts) . "');\n";
            return $import_url;
        }
        return false;
    }
    
    /**
     * Find and replace text in a PHP file
     * @param string $filename
     * @param string $string_to_replace
     * @param string $replace_with
     */
    public function replace_string_in_file($filename, $string_to_replace, $replace_with){
        $content=file_get_contents($filename);
        $content_chunks=explode($string_to_replace, $content ?? '');
        $content=implode($replace_with, $content_chunks);
        file_put_contents($filename, $content);
    }
    
    /**
     * Replaces a string in a file with appropriate messages
     * Also the Explode, Implode are slower than a simple str_replace
     * @param string $FilePath
     * @param string $OldText text to be replaced
     * @param string $NewText new text
     * @return array $Result status (success | error) & message (file exist, file permissions)
     */
    public function replace_in_file($FilePath, $OldText, $NewText) {
        $result = array('status' => 'error', 'message' => '');
        if(file_exists($FilePath)===TRUE) {
            if(is_writeable($FilePath)) {
                try {
                    $FileContent = file_get_contents($FilePath);
                    $FileContent = str_replace($OldText, $NewText, $FileContent);
                    if(file_put_contents($FilePath, $FileContent) > 0) {
                        $result["status"] = 'success';
                    } else {
                        $result["message"] = 'Error while writing file';
                    }
                } catch(Exception $e) {
                    $Result["message"] = 'Error : '.$e;
                }
            } else {
                $Result["message"] = 'File '.$FilePath.' is not writable !';
            }
        } else {
            $Result["message"] = 'File '.$FilePath.' does not exist !';
        }
        return $Result;
    }
    
    /**
     * Read template file line by line
     * @param string $fileName
     * @param string $str
     * @param boolean $with_param - Return parameters also
     * @return multitype:string
     */
    public function getLineWithString($fileName, $str, $from, $to, $with_param = false) {
        $views = array();
        if($this->check_existence($fileName)) {
            $lines = file($fileName);
            foreach ($lines as $lineNumber => $line) {
                if (strpos($line, $str) !== false) {
                    $view = $this->getStringBetween(trim($line), $from, $to, false);
                    if($with_param) {
                        $string_params = $this->getStringBetween(trim($line), 'array(', ');', false);
                        if(!empty($string_params)) {
                            $parameters = array_map('trim', explode(',', $string_params));
                            $widget_params = array();
                            // get existing parameters
                            $string_params = $this->CI->widgets->getStringBetween(trim($line), 'array(', ');', false);
                            if(!empty($string_params)) {
                                $parameters = array_map('trim', explode(',', $string_params));
                                if(!empty($parameters)) {
                                    foreach($parameters as $parameter) {
                                        $parameter = array_map('trim', explode('=>', $parameter ?? ''));
                                        if(!empty($parameter[0]) && !empty($parameter[1])) {
                                            $parameter_0 = str_replace(array('\'', '"'), '', $parameter[0]);
                                            $parameter_1 = trim(str_replace(array('\'', '"'), '', $parameter[1]));
                                            $parameter_1 = preg_replace('/\s+/','',$parameter_1);
                                            $parameter_1 = str_replace(')', '', $parameter_1);
                                            $widget_params[$parameter_0] = $parameter_1;
                                        }
                                    }
                                }
                            }
                        }
                        $view = array('view' => $view, 'params' => $widget_params);
                    }
                    $views[] = $view;
                }
            }
        }
        return $views;
    }
    
    /**
     * Extract text between two characters from a string
     * @param string $str
     * @param string $from
     * @param string $to
     * @param string $withFromAndTo
     * @return string
     */
    public function getStringBetween($str,$from,$to, $withFromAndTo = false) {
        $sub = substr($str, strpos($str,$from)+strlen($from),strlen($str));
        if ($withFromAndTo)
            return $from . substr($sub,0, strrpos($sub,$to)) . $to;
            else
                return substr($sub, 0, stripos($sub, $to));
    }
    
    /**
     * Iterate through each folder, join and minify each type of file
     * @param array $views
     * @param string $type - scss, css, js
     * @param string $destination - File Name
     *
     * e.g. if the view name is Home, $destination must be passed as home
     * file will be saved as follows home.css and home.js
     */
    public function merge_files($views, $type, $destination) {
        if(!empty($views)) {
            switch($type) {
                case 'scss':
                    $file_contents = array(
                    '@import "'. $this->widget_path . 'scss/themes/abstracts/_variables.scss' .'";',
                    );
                    break;
                case 'js':
                    $sourcePath = $this->widget_path . 'js/variables.js';
                    $minifier = new Minify\JS($sourcePath);
                    break;
                case 'css':
                    break;
                    
            }
            
            foreach($views as $view) {
                $folder = substr($view, 0, strrpos( $view, '/'));
                $filename = substr( $view, strrpos( $view, '/' ) + 1);
                $filepath = $this->widget_path . $folder . '/' . $type . '/' . $filename . '.' . $type;
                // Important - SCSS COMPILES LOOKS IN THE FOLDER
                // i.e. PATH MUST BE SET @import '../<FOLDER>/<FILE NAME>'
                $file_url = '../' . $folder . '/' . $type . '/' . $filename . '.' . $type;
                $path = $this->check_existence($filepath);
                if(!empty($path)) {
                    switch($type) {
                        case 'scss':
                            $file_contents[] = '@import "' . $file_url . '";';
                            break;
                        case 'js':
                            $minifier->add($file_url);
                            break;
                        case 'css':
                            break;
                            
                    }
                }
            }
            if($type == 'scss') {
                // Write the changes out...
                if (! function_exists('write_file')) {
                    $this->CI->load->helper('file');
                }
                $contents = implode(PHP_EOL, $file_contents);
                $scss_path = $this->widget_path . 'scss/' . $destination . '.scss';
                $css_path = $this->widget_path . 'css/' . $destination . '.css';
                
                // save scss file
                if (write_file($scss_path, $contents)) {
                    // create css file
                    $scss = new Compiler();
                    $scss->checkedCompile($scss_path, $css_path);
                    return true;
                }
            } else {
                // save minified file to disk
                $minifiedPath = $this->widget_path . 'js/' . $destination . '.js';
                $minifier->minify($minifiedPath);
            }
        }
    }
    
    /**
     * Check file existence before reading file line by line
     * @param string $path
     * @return string
     */
    public function check_existence($path) {
        try {
            if (file_exists($path)) {
                return $path;
            }
        } catch(Exception $e) {
            echo $e->getMessage();
        }
    }
    
    /**
     * Get file contents for editing
     * @param string $file
     * @return array
     */
    public function get_contents($url) {
        /*return array(
         'html' => $this->get_file_contents(realpath($this->widget_path . $url . '.php')),
         'scss' => $this->get_file_contents(realpath($this->widget_path . $url . '.scss')),
         'js' => $this->get_file_contents(realpath($this->widget_path . $url . '.js')),
         );*/
        return $this->get_file_contents(realpath($this->widget_path . $url));
    }
    
    /**
     * Check file existence and return contents
     * @param string $file
     * @return string|boolean
     */
    private function get_file_contents($file) {
        if (file_exists($file)) {
            return file_get_contents($file);
        }
        return false;
    }
    
    /**
     * Save file
     * @param string $url
     * @param string $extension
     * @param string $contents
     * @return boolean
     */
    public function save_file($url, $extension, $contents) {
        // Write the changes out...
        if (! function_exists('write_file')) {
            $this->CI->load->helper('file');
        }
        
        // save file
        if (write_file($this->widget_path . $url . '.' . $extension, $contents)) {
            return true;
        }
        return false;
    }
    
    /**
     * Delete file
     * @param string $url
     * @param string $extension
     */
    public function unlink_file($url, $extension) {
        if(file_exists($this->widget_path . $url . '.' . $extension)) {
            unlink($this->widget_path . $url . '.' . $extension);
        }
    }
    
    /**
     * Get all widgets (recursive) in a view file to set SCSS variables
     * @param string $filepath
     */
    public function get_widgets($filepath) {
        $filepath = $this->widget_path . $filepath;
        
        // get rinner widgets in the view file
        $widgets = $this->getLineWithString($filepath, 'theme_view', "theme_view('", "'");
        $this->get_inner_widgets($widgets);
        
        return $this->view_widgets;
    }
    
    /**
     * Get all inner widgets used in a widget
     * @param array $widgets
     */
    private function get_inner_widgets($widgets) {
        if($widgets) {
            foreach($widgets as $widget) {
                // save widget in global variable
                $this->view_widgets[] = $widget;
                
                // open file and get inner widgets
                $filepath = $this->widget_path . $widget . '.php';
                if($this->check_existence($filepath)) {
                    $inner_widgets = $this->getLineWithString($filepath, 'theme_view', "theme_view('", "'");
                    if(!empty($inner_widgets)) {
                        $this->get_inner_widgets($inner_widgets);
                    }
                }
            }
        }
    }
    
    /**
     * Write PHP, SCSS and JS file from the saved content
     * @param object $record
     * @param number $id
     * @param string $fill_scss
     * @return boolean
     */
    public function write_content_files($record = array(), $id = 0, $fill_scss = false) {
        if($id != 0) {
            $record = $this->CI->themes_model->find($id);
        }
        if(!$record) {
            return false;
        }
        // Add slash after the folder name
        switch($record->type_id) {
            case 1:
                $file_folder = 'widgets/';
                break;
            case 2:
                $file_folder = 'layouts/';
                break;
            default:
                $file_folder = 'views/';
        }
        $contents = !empty($record->contents) ? json_decode($record->contents) : null;
        if(!empty($record->file_folder)) {
            $file_folder .= $record->file_folder . '/';
        }
        if(!empty($contents->php)) {
            $this->save_file($file_folder . $record->file_name, 'php', $contents->php);
        }
        if(!empty($contents->scss)) {
            $this->save_file($file_folder . 'scss/' . $record->file_name, 'scss', $contents->scss);
        }
        if(!empty($contents->js)) {
            $this->save_file($file_folder . 'js/' . $record->file_name, 'js', $contents->js);
        }
        if($fill_scss) {
            $this->fill_scss($record);
        }
    }
    
    /**
     * Fill SCSS Variables using Global Variables defined in the theme_settings table
     *
     * This method will look for all the variables starting from @ and ending at ; in the SCSS file
     * Pass variables as a paramter to the MIXINS.scss file.
     * Existing defined variables will not be updated, only new will be added.
     * Any variable that is removed from the Global Variables will also be removed.
     * This method is not recursive, it will process only the immediate VIEW/WIDGET SCSS file.
     *
     * @param object $record
     * @param int $id
     * @return array
     */
    public function fill_scss($record, $id = 0) {
        if($id != 0) {
            $record = $this->CI->themes_model->select('id, type_id, file_folder, file_name, file_url, scss_variables, scss_values')->find($id);
        }
        if(!$record) {
            return false;
        }
        switch($record->type_id) {
            case 1:
                $folder = 'widgets/';
                break;
            case 2:
                $folder = 'layouts/';
                break;
            default:
                $folder = 'views/';
        }
        $fileName = realpath($this->widget_path . $folder . $record->file_folder . '/scss/' .  $record->file_name . '.scss');
        if(!$this->check_existence($fileName)) {
            return false;
        }
        $lines = file($fileName);
        
        // store all variables name stored in the file
        // later on match these variables with the name stored in the scss_variables column for the same file
        // if name exist, do not import information from the global variables settings
        // use previously saved information
        $global_variables = array();
        $file_variables = array();
        $scss_variables = !empty($record->scss_variables) ? json_decode($record->scss_variables) : array();
        if($record->type_id == 1) {
            $scss_values = !empty($record->scss_values) ? json_decode($record->scss_values) : array(); // existing scss values
        } else { // View File Auto Fill, Get only View Specific SCSS values
            $scss_values = !empty($record->scss_values) ? json_decode($record->scss_values) : array();
            // 13/10/2017 - Remove Ambigous fields having no name e.g. [0] => #FFF
            // Correct variable must have a name e.g. [theme-primary] => #FFF
            if(!empty($scss_values)) {
                foreach($scss_values as $key_scss_parent => $scss_value) {
                    if(is_array($scss_value) && !empty($scss_value)) {
                        foreach($scss_value as $key_scss => $value_scss) {
                            if($key_scss == 0) {
                                unset($scss_values->$key_scss_parent);
                            }
                        }
                    }
                }
            }
            
            $view_existing_scss = $scss_values;
            if(!empty($scss_values) && !empty($scss_values->$id)) {
                $scss_values = $scss_values->$id;
            } else {
                $scss_values = array();
            }
        }
        $new_variables = array();
        $new_values = array();
        // global variables defined via Global Variables link
        $result = $this->CI->db->select('name, value')->get_where('theme_settings', array('name' => 'global'))->row();
        if($result && !empty($result->value)) {
            $global_variables = !empty($result->value) ? json_decode($result->value) : null;
            $global_values = array();
            if(!empty($global_variables)) {
                foreach($global_variables as $var) {
                    $global_values[$var->name] = $var->value;
                }
            }
        }
        
        // get all variables from the saved file
        foreach ($lines as $lineNumber => $line) {
            // mixins with multiple parameters
            if(strpos($line, '#') !== false && strpos($line, '$') !== false) {
                $mixins_variables = explode(';', trim($line));
                foreach($mixins_variables as $mixins_variable) {
                    $mixins_variable = str_replace(')', '', trim($mixins_variable)) .';'; // remove extra space
                    $name = trim($this->getStringBetween($mixins_variable,'$',';'));
                    if(!empty($name)) {
                        $file_variables[] = $name;
                    }
                }
            } else if (strpos($line, '@{') !== false) { // Image URL
                // get variable name starting with $
                $name = trim($this->getStringBetween($line,'@{','}'));
                $file_variables[] = $name;
            } else if (strpos($line, '$') !== false) {
                // get variable name starting with $
                $name = trim($this->getStringBetween($line,'$',';'));
                $file_variables[] = $name;
            }
            
        }
        // if scss file has variables
        if(!empty($file_variables)) {
            // get all unique variables
            $file_variables = array_unique($file_variables);
            // check whether the variable already exists in the scss_variables column of the same record
            foreach($file_variables as $file_variable) {
                if($file_variable == 'font-family-base') {
                    $file_variable = 'font-custom';
                }
                $check_existence = $this->_variable_existence($file_variable, $scss_variables, 'name');
                $global_check_existence = $this->_variable_existence($file_variable, $global_variables, 'name');
                if($check_existence && $global_check_existence) { // Variable must also exist in the global list
                    // use existing value
                    $new_variables[] = $check_existence;
                    if(!empty($scss_values->$file_variable)) {
                        $new_values[$file_variable] = $scss_values->$file_variable;
                    }
                } else if($global_check_existence) { // get variable from the global variables
                    $new_variables[] = $global_check_existence;
                    $new_values[$file_variable] = $global_check_existence->value;
                }
            }
        }
        
        if($record->type_id == 0 && !empty($new_values)) {
            // Update View Specific SCSS values only
            $view_existing_scss->$id = $new_values;
            $new_values = $view_existing_scss;
        }
        
        // update new information
        $this->CI->db->where('id', $record->id)->update('themes', array('scss_variables' => json_encode($new_variables), 'scss_values' => json_encode($new_values)));
    }
    
    /**
     * Check variable existence
     * Return variable if it exists in the array
     * @param string $name
     * @param array $variables
     * @param string $match - NAME/ID
     * @return array|boolean
     */
    private function _variable_existence($name, $variables, $match) {
        if(!empty($variables)) {
            foreach($variables as $variable) {
                if($name == $variable->$match) {
                    return $variable;
                }
            }
        }
        return false;
    }
    
    /**
     * Some of the variables like Logo Image through CSS file must be recompiled on logo change.
     * i.e. a separate file global.css is compiled separately on each time a logo is changed.
     * Other settings which are globally common and requires SCSS compilation can be placed inside the global.css file.
     *
     * @param string $logo - Settings are saved in the cache and they return the old logo file name after change
     * i.e. the post file name is passed as a parameter to this method.
     */
    public function compile_global($logo = null) {
        // $variables = $this->get_variables((object)$this->scheme);
        // Process Global SCSS file - included in the header.php file.
        $global_scss = file_get_contents(realpath($this->widget_path . 'scss/global.scss'));
        $variables = array();
        $global_contents = null;
        // initialize scss library
        $scss = new Compiler();
        // SEO Hide H1 text in the Header Logo
        if(empty($logo)) {
            $logo = settings_item('site.logo');
        }
        $variables['logo-url'] = "'" . '../../../assets/images/' . $logo ."'";
        
        // Get the Logo Width and Height from the Default Theme
        // Logo width and height are not saved in the site settings table because each theme will have their own sizes.
        // Changing theme should not push user to make changes in the settings table.
        $default_theme = $this->CI->db->get_where('theme_settings', array('type' => 'themes', 'is_default' => 1))->row();
        if($default_theme) {
            $value = !empty($default_theme->value) ? json_decode($default_theme->value) : null;
            $variables['logo-width'] = $value->logo_width . 'px';
            $variables['logo-height'] = $value->logo_height . 'px';
        } else {
            $variables['logo-width'] = '200px';
            $variables['logo-height'] = '45px';
        }
        $scss->setVariables($variables);
        try {
            $global_contents = $scss->compile($global_scss);
        } catch (exception $e) {
            echo $e->getMessage();
        }
        if(!empty($global_contents)) {
            // write css file from the compiled scss files
            $this->save_file('css/global', 'css', $global_contents);
        }
    }
    
    /**
     * Process individual widget included in the Pages
     * @param integer $id - ID of the Widget
     * @param string $filename - Name of the File CSS and JS to be saved
     *
     * Write CSS and JS files to be included in the Controller Method
     */
    public function process_widgets($str, $filename, $loadAssets = true, $widget_urls = array(), $return_content_only = true) {
        $this->CI->load->model('themes/themes_model');
        $this->assets_css = array();
        $this->assets_js = array();
        $this->contents_scss = null;
        $this->contents_js = array();
        if(!empty($widget_urls)) {
            // Load CSS and JS of widgets used in the Admin Panel VIEW files
            foreach($widget_urls as $widget_url) {
                $this->get_widget_contents(0, array('file_url' => $widget_url));
            }
            $folder = 'widgets';
            $filename = 'admin-' . $filename;
        } else {
            $folder = 'pages';
            // Process Widgets recursively
            $content = $this->recursive_widget_process($str);
        }
        
        // write css file from the compiled scss files
        if(!empty($this->contents_scss)) {
            $this->save_file('css/' . $folder . '/' . $filename, 'css', $this->contents_scss);
        }
        // write JS contents
        if(!empty($this->contents_js)) {
            $minifier = new Minify\JS($this->default_js);
            $minifier->add($this->contents_js);
            // save minified file to disk
            $minifiedPath = $this->widget_path . 'js/' . $folder . '/' . $filename . '.js';
            $minifier->minify($minifiedPath);
        }
        if($loadAssets) {
            // Load Assets
            if(!empty($this->assets_css)) {
                Assets::add_css($this->assets_css);
            }
            if(!empty($this->assets_js)) {
                Assets::add_js($this->assets_js);
            }
            
            // Load CSS and JS Specific Files
            Assets::add_css('css/' . $folder . '/' . $filename . '.css');
            Assets::add_js('js/' . $folder . '/' . $filename . '.js');
        }
        if($return_content_only) {
            return !empty($content) ? $content : '';
        } else {
            return array('content' => $content, 'css' => $this->assets_css, 'js' => $this->assets_js);
        }
    }
    
    /**
     * A page can have multiple widgets, process them recursively
     * @param string $str
     * @return mixed
     */
    private function recursive_widget_process($str) {
        $widget = getStringBetween($str, '{widget}', '{/widget}');
        $options_array = array_map('trim', explode('|', $widget ?? ''));
        $params = array();
        $id = 0;
        if(!empty($options_array)) {
            foreach($options_array as $option_array) {
                $param_array = array_map('trim', explode(':', $option_array ?? ''));
                if(isset($param_array[0]) && isset($param_array[1])) {
                    if($param_array[0] == 'id') {
                        $id = $param_array[1];
                    } else {
                        $params[$param_array[0]] = $param_array[1];
                    }
                }
            }
            // {widget}id:388|load:packages|widget:_pkg_widget_02|heading:2|type_id:0|cols:2{/widget} - CALL
            if($id > 0) {
                $file_url = $this->get_widget_contents($id);
                $widget_content = '';
                if($file_url) {
                    if(!empty($params['load'])) {
                        $wparams = array();
                        if(!empty($params['cols'])) {
                            $wparams['cols'] = $params['cols'];
                            unset($params['cols']);
                        }
                        if(!empty($params['heading'])) {
                            $wparams['heading'] = $params['heading'];
                            unset($params['heading']);
                        }
                        $widget_content = theme_view($file_url, array('load' => $params['load'], 'widget' => $params['widget'], 'wparams' => $wparams, 'params' => $params));
                    } else {
                        $widget_content = theme_view($file_url, $params, true);
                    }            
                }
                $str = str_replace('{widget}' . $widget . '{/widget}', $widget_content, $str);
                // Check for the next widget if exists on the page
                $has_widget = getStringBetween($str, '{widget}', '{/widget}');
                if($has_widget) {
                    $str = $this->recursive_widget_process($str);
                }
            }
        }
        return $str;
    }
    
    /**
     * Process individual widget included in the Pages
     * @param integer $id - ID of the Widget
     * @param string $filename - Name of the File CSS and JS to be saved
     */
    private function get_widget_contents($id, $where = array()) {
        // View and Layouts cannot be loaded in a page i.e. restrict query to widgets only
        if(empty($where)) {
            $where = array('id' => $id, 'type_id' => 1);
        }
        $record = $this->CI->themes_model->find_by($where);
        if($record) {
            $folder = 'widgets/';
            $filepath = $this->widget_path . $record->file_url;
            $php_file = $filepath . '.php';
            if($this->check_existence($php_file)) {
                $scss_file = $this->widget_path . $folder . $record->file_folder . '/scss/' . $record->file_name . '.scss';
                $js_file = $this->widget_path .  $folder . $record->file_folder . '/js/' . $record->file_name . '.js';
                // Compile view specific scss file before widget scss files
                // Widget SCSS values are used instead of SCHEME file so that user can easily change it
                // Taking SCHEME default values will make it difficult for the user to change the Color Scheme
                if($this->check_existence($scss_file)) {
                    $this->compile_scss($scss_file, $record->id, (!empty($record->scss_values) ? json_decode($record->scss_values) : null), json_decode($record->scss_values ?? ''));
                }
                // Compile view specific Javascript file
                if($this->check_existence($js_file)) {
                    $this->contents_js[] = $js_file;
                }
                
                $assets_css = !empty($record->assets_css) ? json_decode($record->assets_css) : null;
                // The link can be either relative or absolute.
                if(!empty($assets_css)) {
                    foreach($assets_css as $asset_css) {
                        $this->assets_css[] = $asset_css->url;
                    }
                }
                $assets_js = !empty($record->assets_js) ? json_decode($record->assets_js) : null;
                if(!empty($assets_js)) {
                    foreach($assets_js as $asset_js) {
                        $this->assets_js[] = $asset_js->url;
                    }
                }
                return $record->file_url;
            }
        }
    }
    
    /**************************************************************************
     * IMPORT/EXPORT
     **************************************************************************/
    /**
     * Export Theme/View/Layout/Widget
     * @param string $type
     * @param integer $id
     */
    public function migrate($type = 'themes', $id = 0, $export_slider = true) {
        // Path where files will be copied
        $zip_path = FCPATH . 'assets/tmp/themes';
        // read all blocks within a file and save them in database before export
        // save unsaved blocks in database
        // export the sql file for import
        $widgets = array();
        $views = array();
        $layouts = array();
        $files = array();
        if($type != 'themes') {
            $result = $this->CI->db->get_where('themes', array('id' => $id))->row();
            if(!$result) {
                return false;
            }
            switch($result->type_id) {
                case 0:
                    $type = 'views';
                    break;
                case 1:
                    $type = 'widgets';
                    break;
                case 2:
                    $type = 'layouts';
                    break;
            }
        }
        switch($type) {
            case 'themes':
                // Get all view and layout files of this specific themes
                $result = $this->CI->db->get_where('themes', array('theme' => $id))->result();
                if($result) {
                    foreach($result as $page) {
                        if($page->type_id == 0) {
                            $files[] = $page->file_url;
                        } else if($page->type_id == 2) {
                            $files[] = $page->file_url;
                        }
                        $widget_files = $this->get_widgets($page->file_url . '.php');
                        if(!empty($widget_files)) {
                            $files = array_merge($files, $widget_files);
                        }
                    }
                }
                break;
            case 'views':
                // also process the layout
                $layout = $this->CI->db->get_where('themes', array('type_id' => 2, 'theme' => $result->theme, 'layout' => $result->layout, 'is_default' => 1))->row();
                if($layout) {
                    $files[] = $layout->file_url;
                    // Export Layout External Dependencies
                    $this->export_external_css_js($layout, $zip_path);
                }
                $files[] = $result->file_url;
                break;
            case 'layouts':
                $files[] = $result->file_url;
                break;
            case 'widgets':
                $files[] = $result->file_url;
                break;
        }
        // Scan widget files and take backup
        if($type != 'themes') {
            $widget_files = $this->get_widgets($result->file_url . '.php');
            if(!empty($widget_files)) {
                $files = array_merge($files, $widget_files);
            }
            // Export External Dependencies
            $this->export_external_css_js($result, $zip_path);
            // Export Images
            $this->read_copy_files(FCPATH . 'themes/default/' . $result->file_url . '.php', 'assets/images/', 0, null, $zip_path);
            $this->read_copy_files(FCPATH . 'themes/default/' . $result->file_url . '.php', 'assets/plugins/images/', 0, null, $zip_path);
        } else {
            // add global scss file to the zip file
            $scss_path = 'themes/default/scss/themes/' . $id . '.scss';
            $source_file = FCPATH . $scss_path;
            if(file_exists($source_file)) {
                file_copy($source_file, $zip_path . '/' . $scss_path);
            }
        }
        $files = array_unique($files);
        $extensions = array('php' => '', 'scss' => 'scss/', 'js' => 'js/');
        if(!empty($files)) {
            foreach($files as $file_path) {
                $data = array();
                // read files (PHP, SCSS and JS) and update them in database
                $file_url = FCPATH . 'themes/default/' . $file_path;
                $file_name = substr( $file_url, strrpos( $file_url, '/' ) + 1);
                $file_url = substr($file_url, 0, strrpos( $file_url, '/') );
                foreach($extensions as $extension => $path) {
                    $file = $file_url . '/' . $path . $file_name . '.' . $extension;
                    if($this->check_existence($file)) {
                        $data[$extension] = file_get_contents($file);
                    }
                }
                if(!empty($data['php'])) {
                    // check row existence in themes table using file_url
                    $check_existence = $this->CI->db->where('file_url', $file_path)->count_all_results('themes');
                    if($check_existence == 0) { // if not exists insert row
                        $file_type = explode('/', $file_path ?? '');
                        if($file_type[0] == 'widgets') { // Save widgets only
                            $file_folder = isset($file_type[2]) ? $file_type[1] : NULL;
                            $widget_data = array(
                                'type_id' => 1,
                                'is_widget' => 1,
                                'is_widget_all' => 1,
                                'is_default' => 0,
                                'is_theme_default' => 0,
                                'theme' => NULL,
                                'layout' => NULL,
                                'name' => ucwords(str_replace('/', ' ', $file_path)) . ' - ' . rand(10,100), // Unique Name
                                'contents' => json_encode($data),
                                'file_folder' => $file_folder,
                                'file_name' => end($file_type),
                                'file_url' => $file_path,
                                'created_on' => date('Y-m-d H:i:s'),
                                'modified_on' => date('Y-m-d H:i:s')
                            );
                            // Insert new row
                            $this->CI->db->insert('themes', $widget_data);
                        }
                    } else {
                        // update database using $file_path
                        $this->CI->db->where('file_url', $file_path)->update('themes', array('contents' => json_encode($data)));
                    }
                }
            }
        }
        
        // Drop temporary table
        $this->drop_tables();
        
        // export mysql rows in a tmp_table
        // this table will be used during import
        $tmp_table = $this->CI->db->dbprefix . "tmp_themes";
        $tmp_table_views = $this->CI->db->dbprefix . "tmp_theme_views";
        $tmp_table_sliders = $this->CI->db->dbprefix . "tmp_sliders";
        $file_urls = "'" . implode ( "', '", $files ) . "'";
        $this->CI->load->dbutil();
        // NOTE: The TEMPORARY keyword is removed, the table_exists method of codeigniter is unable to work on temporary tables.
        $this->CI->db->query("CREATE TABLE " . $tmp_table_views . " AS (SELECT TV.* FROM " . $this->CI->db->dbprefix . "theme_views TV INNER JOIN " . $this->CI->db->dbprefix . "themes T ON T.id = TV.theme_id WHERE T.file_url IN (" . $file_urls . "))");
        $this->CI->db->query("CREATE TABLE " . $tmp_table . " AS (SELECT * FROM " . $this->CI->db->dbprefix . "themes WHERE file_url IN (" . $file_urls . "))");
        $tables = array($tmp_table, $tmp_table_views);
        if($type == 'themes') {
            $tmp_table_settings = $this->CI->db->dbprefix . "tmp_theme_settings";
            $tables[] = $tmp_table_settings;
            $this->CI->db->query("CREATE TABLE " . $tmp_table_settings . " AS (SELECT * FROM " . $this->CI->db->dbprefix . "theme_settings WHERE type = 'themes' AND name = '" . $id . "')");
        }
        
        // also export existing active sliders
        if((($type == 'views' && $result->file_folder == 'home') || $type == 'themes') && $export_slider) {
            $this->CI->db->query("CREATE TABLE " . $tmp_table_sliders . " AS (SELECT * FROM " . $this->CI->db->dbprefix . "banner_sliders WHERE status = 1)");
            $tables[] = $tmp_table_sliders;
            $this->export_slider_images($zip_path); // Export slider images
        }
        
        // Migrate language strings saved in the custom_themes_lang.php file
        $this->migrate_language_strings(false);
        if ($this->is_table_exists('tmp_theme_langs') ) {
            $tmp_table_langs = $this->CI->db->dbprefix . "tmp_theme_langs";
            $tables[] = $tmp_table_langs;
        }
        
        $prefs = array(
            'tables'        => $tables,             // Array of tables to backup.
            'ignore'        => array(),             // List of tables to omit from the backup
            'format'        => 'txt',               // gzip, zip, txt
            'filename'      => $tmp_table,          // File name - NEEDED ONLY WITH ZIP FILES
            'add_drop'      => TRUE,                // Whether to add DROP TABLE statements to backup file
            'add_insert'    => TRUE,                // Whether to add INSERT data to backup file
            'newline'       => "\n"                 // Newline character used in backup file
        );
        
        // Backup temporary table and assign it to a variable
        $backup = $this->CI->dbutil->backup($prefs);
        
        // Drop temporary table
        $this->CI->load->dbforge();
        $this->CI->dbforge->drop_table($tmp_table, TRUE); // DROP TABLE IF EXISTS tmp_table
        
        // Load the file helper and write the file to your server
        $destination_file = $zip_path . '/public_html/uploads/imports/' . $this->migration_filename . '.sql';
        
        $this->CI->load->helper('file');
        write_file($destination_file, $backup);
        
        // Create an Archieve
        if(!empty($result->file_folder) && !empty($result->file_name)) {
            $file_prefix = $result->file_folder . '_' . $result->file_name;
        } else {
            $file_prefix = $id . '_';
        }
        $file_name = str_replace('-', '_', $this->CI->slug->create_slug($file_prefix)) . '_' . date('Y-m-d') . '_' . time() . '.zip';
        // $file_name = $file_prefix . '_' . $this->migration_filename . '_' . date('Y-m-d') . '_' . time() . '.zip';
        create_zip($zip_path, $file_name);
        
        // Drop temporary table
        $this->drop_tables();
        
        // Load the download helper and send the file to your desktop
        /*$this->CI->load->helper('download');
         $file_name = $this->migration_filename . '_' . date('Y-m-d H:i:s') . '.zip';
         force_download($file_name, file_get_contents($zip_path . $file_name));*/
        
        //  remove assets directory contents to avoid future references
        rrmdir($zip_path . '/public_html/assets/');
        unlink($destination_file);
        
        $filename = FCPATH . '/' . $file_name;
        if(file_exists($filename) && is_readable($filename) && file_exists($filename)){
            header("Content-Disposition: attachment; filename=".basename(str_replace(' ', '_', $filename)));
            header("Content-Type: application/force-download");
            header("Content-Type: application/octet-stream");
            header("Content-Type: application/download");
            header("Content-Description: File Transfer");
            header("Content-Length: " . filesize($filename));
            flush(); // this doesn't really matter.
            
            $fp = fopen($filename, "r");
            while (!feof($fp))
            {
                echo fread($fp, 65536);
                flush(); // this is essential for large downloads
            }
            fclose($fp);
            
            // Unlink Backup File
            unlink($filename);
            
            exit;
        }
        
        // Unlink Backup File
        if(file_exists($filename)) {
            unlink($filename);
        }
    }
    
    /**
     * Export active slider images
     * @param string $destination_path
     */
    private function export_slider_images($destination_path) {
        $sliders = $this->CI->db->where('status', 1)->get('banner_sliders')->result();
        $source_path = FCPATH . 'assets/images/';
        if($sliders) {
            foreach($sliders as $slider) {
                if(!empty($slider->image)) {
                    $slider_image_url = $source_path . $slider->image;
                    if(file_exists($slider_image_url)) { // copy only if source file exists
                        file_copy($slider_image_url, $destination_path . '/public_html/assets/images/' . $slider->image);
                    }
                }
            }
        }
    }
    
    /**
     * Export External Dependencies
     * @param object $row
     * @param string $destination_path
     */
    public function export_external_css_js($row, $destination_path) {
        $source_folder = FCPATH . 'assets/';
        $dependencies = array('assets_css', 'assets_js');
        foreach($dependencies as $dependency) {
            if(!empty($row->{$dependency})) {
                $files = json_decode($row->{$dependency});
                if(!empty($files)) {
                    foreach($files as $file) {
                        // get file and paste it in the folder with path
                        if($file->type == 'internal') { // do not process URL dependencies
                            $source_file = $source_folder . $file->url;
                            if(file_exists($source_file)) { // copy only if source file exists
                                file_copy($source_folder . $file->url, $destination_path . '/public_html/assets/' . $file->url);
                                // copy entire plugin folders e.g. fonts, library with images
                                $plugin_found = $this->copy_plugin_folders($file->url, $source_folder, $destination_path . '/public_html/assets/');
                            }
                        }
                    }
                }
            }
        }
    }
    
    /**
     * Read file line by line, get and copy values
     *
     * Common function to extract images, CSS and JS files
     *
     * @param string $fileName
     * @param string $str
     * @param number $type - 0 -> Images, 1 -> CSS/JS files from header Mobiriser
     * @return multitype:string
     */
    public function read_copy_files($fileName, $str, $type = 0, $theme_view = 'home', $destination_path = null) {
        if($this->check_existence($fileName)) {
            if($type == 1) { // Prepare themes content
                $contents = array('php' => '');
                $file_contents = file_get_contents($fileName);
                if(empty($file_contents)) {
                    return false;
                }
                $assets_css = array();
                $assets_js = array();
                $random_number = time() . '_' . rand(1, 1000);
                $view_name = 'GoClixy ' . ucfirst($theme_view) . ' - ' . $random_number;
                $view_filename = '_goclixy_' . $theme_view . '_' . $random_number;
                
                $from = 'assets/';
                $to = '"';
                $source_path = FCPATH . 'uploads/imports/themes/';
                $destination_path = FCPATH . 'assets/plugins/';
                // files to be skipped
                $files_to_be_skipped = array(
                    'bootstrap/css/bootstrap.min.css',
                    'bootstrap/css/bootstrap-grid.min.css',
                    'bootstrap/css/bootstrap-reboot.min.css',
                    'mobirise/css/mbr-additional.css',
                    'theme/css/style.css',
                    'jquery.min.js',
                    'bootstrap.min.js',
                    'theme/js/script.js'
                );
                $asset_files = array(
                    'css' => 'assets/mobirise/css/mbr-additional.css',
                    'scss' => 'assets/theme/css/style.css',
                    'js' => 'assets/theme/js/script.js'
                );
            }
            $lines = file($fileName);
            foreach ($lines as $lineNumber => $line) {
                if (strpos($line, $str) !== false) {
                    // get filename using $str
                    if($type == 0) { // images
                        $from = $str;
                        $to = '"';
                        // get file name
                        $file = $this->getStringBetween(trim($line), $from, $to, false);
                        
                        // image full path
                        $file_path = $from . $file;
                        $source_file = FCPATH . $file_path;
                        if(file_exists($source_file)) {
                            file_copy($source_file, $destination_path . '/public_html/' . $file_path);
                        }
                    } else if($type == 1) { // mobilizer images and other files
                        $do_not_skip_file = true;
                        foreach($files_to_be_skipped as $file_to_be_skipped) {
                            if(strpos($line, $file_to_be_skipped) !== false) {
                                $do_not_skip_file = false;
                            }
                        }
                        
                        // SCSS/JS contents
                        foreach($asset_files as $asset_key => $asset_file) {
                            if(strpos($line, $asset_file) !== false) {
                                $contents[$asset_key] = file_get_contents($source_path . $asset_file);
                            }
                        }
                        if($do_not_skip_file) {
                            
                            // get file name
                            $file_url_without_assets = $this->getStringBetween(trim($line), $from, $to, false);
                            
                            // file full path
                            $file_path = $from . $file_url_without_assets;
                            
                            // copy mobirise themes files inside the uploads/imports/themes/ folder
                            $source_file = $source_path .  $file_path;
                            
                            // search and replace images path in php file
                            // change path from assets/images/ to assets/plugins/images/
                            if(strpos($line, 'assets/images/') !== false) {
                                $lines[$lineNumber] = str_replace('assets/images/', 'assets/plugins/images/', $line);
                            }
                            
                            // copy file to plugins folder
                            if(file_exists($source_file)) {
                                file_copy($source_file, $destination_path . $file_url_without_assets);
                                $plugin_found = $this->copy_plugin_folders($file_url_without_assets, $source_path . 'assets/', $destination_path);
                                
                                // prepare data for View save
                                // check file extension
                                $file_extension = substr($source_file, -3);
                                if($file_extension == '.js') {
                                    $assets_js[] = array(
                                        'url' => 'plugins/' . $file_url_without_assets,
                                        'type' => 'internal'
                                    );
                                } else if($file_extension == 'css') {
                                    $assets_css[] = array(
                                        'url' => 'plugins/' . $file_url_without_assets,
                                        'type' => 'internal'
                                    );
                                }
                            }
                        }
                    }
                }
            }
            if($type == 1) {
                $today_date = date('Y-m-d H:i:s');
                $updated_content = implode('', $lines);
                $contents['php'] = $this->getStringBetween($updated_content, '<body>', '</body>', false);
                $file_url = 'views/' . $theme_view . '/';
                // merge $contents['css'] and $contents['scss'] files
                if(isset($contents['css'])) {
                    if(isset($contents['scss'])) {
                        $contents['scss'] = $contents['css'] . PHP_EOL . '/* START OF PAGE STYLES */' . PHP_EOL . $contents['scss'];
                    } else {
                        $contents['scss'] = $contents['css'];
                    }
                    unset($contents['css']);
                }
                $theme_data = array(
                    'type_id' => 0,
                    'is_widget' => 0,
                    'is_widget_all' => 0,
                    'is_default' => 0,
                    'is_theme_default' => 0,
                    'theme' => 'default',
                    'layout' => 'blank',
                    'name' => $view_name,
                    'contents' => json_encode($contents),
                    'file_folder' => $theme_view,
                    'file_name' => $view_filename,
                    'file_url' => $file_url . $view_filename,
                    'assets_css' => json_encode($assets_css),
                    'assets_js' => json_encode($assets_js),
                    'created_on' => $today_date,
                    'modified_on' => $today_date
                );
                $this->CI->db->insert('themes', $theme_data);
                $view_id = $this->CI->db->insert_id();
                if(!empty($view_id)) {
                    $this->CI->db->insert('theme_views', array('theme_id' => $view_id, 'view' => $theme_view));
                    // write files
                    $this->save_file($file_url . $view_filename, 'php', $contents['php']);
                    if(!empty($contents['scss'])) {
                        $this->save_file($file_url .'scss/' . $view_filename, 'scss', $contents['scss']);
                    }
                    if(!empty($contents['js'])) {
                        $this->save_file($file_url . 'js/' . $view_filename, 'js', $contents['js']);
                    }
                }
            }
        }
    }
    
    /**
     * Copy Entire Plugins Folder
     * @param string $file_url
     * @param string $source_path
     * @param string $destination_path
     * @return boolean
     */
    public function copy_plugin_folders($file_url, $source_path, $destination_path) {
        $files = array('mobirise-icons.css');
        $string_found = false;
        foreach($files as $file) {
            if(strpos($file_url, $file) !== false) { // if string exists
                $folder = str_replace($file, '', $file_url);
                xcopy($source_path . $folder, $destination_path . $folder);
                $string_found = true;
            }
        }
        return $string_found;
    }
    
    
    /**
     * Import Dump file into database
     * https://forum.codeigniter.com/archive/index.php?thread-63410.html
     */
    public function import_dump() {
        // Drop temporary table
        $this->drop_tables();
        
        // Hardcoded - File will be replaced on zip file extraction
        $file = FCPATH . 'uploads/imports/' . $this->migration_filename . '.sql';
        
        // Insert Only
        $insert_only = $this->CI->input->post('insert_or_update') == 1 ? true : false;
        
        // Check file existence
        if(!$this->check_existence($file)) {
            return false;
        }
        // Set line to collect lines that wrap
        $templine = '';
        
        // Read in entire file
        $lines = file($file);
        
        // Loop through each line
        foreach ($lines as $line) {
            
            // Skip it if it's a comment
            if (substr($line, 0, 2) == '--' || $line == '')
                continue;
                
                // Add this line to the current templine we are creating
                $templine .= $line;
                
                // If it has a semicolon at the end, it's the end of the query so can process this templine
                if (substr(trim($line), -1, 1) == ';') {
                    // Perform the query
                    $this->CI->db->query($templine);
                    
                    // Reset temp variable to empty
                    $templine = '';
                }
        }
        
        // Read temporary table
        // note: temporary table will not work in a separate function
        // it will also not be displayed in PHPMyAdmin
        if ($this->is_table_exists('tmp_theme_views') ) {
            $tmp_table_views = $this->CI->db->dbprefix . 'tmp_theme_views'; // Theme Views
            $result_views = $this->CI->db->get($tmp_table_views)->result();
        }
        if ($this->is_table_exists('tmp_themes') ) {
            $tmp_table = $this->CI->db->dbprefix . 'tmp_themes';
            $result = $this->CI->db->get($tmp_table)->result();
            if($result) {
                $ids_to_be_set_default = array();
                foreach($result as $row) {
                    $row = (array) $row;
                    $row['modified_on'] = date('Y-m-d H:i:s');
                    $return = $this->_insert_update($row, $insert_only, $result_views);
                    if(!empty($return)) {
                        $ids_to_be_set_default[$return['id']] = $return['type'];
                    }
                }
                // set default
                if(!empty($ids_to_be_set_default)) {
                    $this->CI->load->model('themes/themes_model');
                    foreach($ids_to_be_set_default as $id_to_be_set_default => $theme_type) {
                        $this->CI->themes_model->is_theme_default($id_to_be_set_default, 1);
                        $this->set_default($theme_type, $id_to_be_set_default, false);
                    }
                }
            }
        }
        
        // Theme Settings
        // Also save theme name in the theme_settings table
        if ($this->is_table_exists('tmp_theme_settings') ) {
            $theme_settings = $this->CI->db->get('tmp_theme_settings')->result();
            if($theme_settings) {
                foreach($theme_settings as $theme_setting) {
                    $where_setting = array('name' => $theme_setting->name, 'type' => $theme_setting->type);
                    $check_existence = $this->CI->db->get_where('theme_settings', $where_setting)->row();
                    // Set Theme Default
                    if($this->CI->input->post('set_default') == 1) {
                        $theme_setting->is_default = 1;
                        // set other themes default to 0
                        $this->CI->db->where(array('type' => $theme_setting->type))->update('theme_settings', array('is_default' => 0));
                    } else {
                        $theme_setting->is_default = 0;
                    }
                    if(!$check_existence) {
                        $this->CI->db->insert('theme_settings', $theme_setting);
                    } else if(!$insert_only) {
                        $this->CI->db->where($where_setting)->update('theme_settings', $theme_setting);
                    }
                }
            }
        }
        
        // copy active sliders
        if ($this->is_table_exists('tmp_sliders') ) {
            $tmp_table_sliders = $this->CI->db->dbprefix . 'tmp_sliders'; // Theme Views
            $result_sliders = $this->CI->db->get($tmp_table_sliders)->result();
            if($result_sliders) {
                $new_slider_ids = array();
                foreach($result_sliders as $result_slider) {
                    // do not update if an image already exists in the slider
                    $slider_count = $this->CI->db->where('image', $result_slider->image)->count_all_results('sliders');
                    if($slider_count == 0) { // slider does not exist
                        // insert slider
                        $slider_data = (array) $result_slider;
                        unset($slider_data['id']);
                        $this->CI->db->insert('sliders', $slider_data);
                        $new_slider_ids[] = $this->CI->db->insert_id();
                    }
                }
            }
        }
        
        // Migrate language strings
        $this->migrate_language_strings();
        
        // Drop temporary table
        $this->drop_tables();
        
        // Unlink SQL File
        unlink($file);
        return true;
    }
    
    /**
     * Check if a table exists or not.
     * @param string $table - Name of the table without prefix
     *
     * Added because $this->CI->db->table_exists was not working
     */
    public function is_table_exists($table) {
        $table = $this->CI->db->dbprefix . $table;
        $row = $this->CI->db->query("SHOW TABLES LIKE '" . $table . "'")->row();
        if($row) {
            return true;
        } else {
            return false;
        }
    }
    
    /**
     * Drop TMP Tables
     */
    private function drop_tables() {
        // Drop temporary table
        $this->CI->load->dbforge();
        $this->CI->dbforge->drop_table('tmp_themes', TRUE); // DROP TABLE IF EXISTS tmp_themes
        $this->CI->dbforge->drop_table('tmp_theme_views', TRUE); // DROP TABLE IF EXISTS tmp_theme_views
        $this->CI->dbforge->drop_table('tmp_theme_settings', TRUE); // DROP TABLE IF EXISTS tmp_theme_settings
        $this->CI->dbforge->drop_table('tmp_sliders', TRUE); // DROP TABLE IF EXISTS tmp_sliders
        $this->CI->dbforge->drop_table('tmp_theme_langs', TRUE); // DROP TABLE IF EXISTS tmp_theme_langs
    }
    
    /**
     * Insert/Update Themes Table
     * @param array $row
     * @param boolean $insert_only
     */
    private function _insert_update($row, $insert_only, $views) {
        $return = array();
        $where = array('is_widget' => $row['is_widget'], 'name' => $row['name'], 'file_name' => $row['file_name']);
        $result = $this->CI->db->get_where('themes', $where)->row();
        // $result = $this->CI->db->get_where('themes', array('file_url' => $row['file_url']))->row();
        
        // Exported row and imported row may have a different ID.
        $tmp_id = $row['id'];
        
        // unset id variable
        unset($row['id']);
        
        // check row existence in themes table using file_url
        // $check_existence = $this->CI->db->where('file_url', $row['file_url'])->count_all_results('themes');
        if(!$result) { // if not exists insert row
            $row['created_on'] = date('Y-m-d H:i:s');
            $this->CI->db->insert('themes', $row);
            $id = $this->CI->db->insert_id();
            // Insert Views in the theme_views table with the new insert_id
            $this->_insert_update_views($id, $tmp_id, $views);
            // Write Files on Disk and Auto Fill SCSS variables
            if($this->CI->input->post('write_files') == 1) {
                $this->write_content_files(array(), $id, true);
            }
            // Set Default after import - do this only after all widget insert
            // else widget css will not be processed
            if($this->CI->input->post('set_default') == 1) {
                $result = $this->CI->db->where('id', $id)->get('themes')->row();
                if($result && ($result->type_id == 0 || $result->type_id == 2)) {
                    $return = array(
                        'type' => ($result->type_id == 0 ? 'views' : 'layouts'),
                        'id' => $id
                    );
                }
            }
        } else { // update existing rows also
            if(!$insert_only) {
                $this->CI->db->where($where)->update('themes', $row);
                // update views in the theme views table
                $this->_insert_update_views($result->id, $tmp_id, $views);
                // Write Files on Disk and Auto Fill SCSS variables
                if($this->CI->input->post('write_files') == 1) {
                    $this->write_content_files($result, 0, true);
                }
            }
            // Set Default after import
            if($this->CI->input->post('set_default') == 1 && ($result->type_id == 0 || $result->type_id == 2)) { // process only if view and layout
                $return = array(
                    'type' => ($result->type_id == 0 ? 'views' : 'layouts'),
                    'id' => $result->id
                );
            }
        }
        return $return;
    }
    
    /**
     * Check whether this VIEW/LAYOUT/WIDGET has a view in the theme_views table
     * @param integer $theme_id
     * @param integer $tmp_id
     * @param array $views
     */
    private function _insert_update_views($theme_id, $tmp_id, $views) {
        $view_data = array();
        if($views) {
            foreach($views as $view) { // insert_update all views
                if($view->theme_id == $tmp_id) {
                    $view_data[] = array(
                        'theme_id' => $theme_id,
                        'view' => $view->view
                    );
                }
            }
        }
        if(!empty($view_data)) {
            $this->CI->db->where('theme_id', $theme_id)->delete('theme_views');
            $this->CI->db->insert_batch('theme_views', $view_data);
        }
    }
    
    /**
     * Set a Theme/Layout/View as Default
     * @param string $type
     * @param integer $id
     * @param boolean $message
     */
    public function set_default($type, $id, $message = true, $theme_settings = array()) {
        if(!empty($theme_settings)) { // cache issue, while setting the theme to default on the settings page
            $this->theme_settings = $theme_settings;
        }
        clear_cache('default_theme'); // clear the session variable
        clear_cache('theme_default_set'); // clear the cache variable
        switch($type) {
            case 'themes':
                $theme_name = $id;
                $this->theme_settings = theme_settings($theme_name); // get header and footer to be processed
                $views = $this->get_views($theme_name);
                if($views) {
                    // Get Default Scheme
                    $scheme_values = array();
                    $scheme = $this->CI->db->get_where('theme_settings', array('type' => 'schemes', 'is_default' => 1))->row();
                    if($scheme) {
                        $scheme_values = !empty($scheme->value) ? (array) json_decode($scheme->value) : null;
                    }
                    
                    // process found views and layouts
                    $this->process_views($views, $theme_name, $scheme_values);
                    
                    // Unset the theme_scheme_process session data
                    $this->CI->session->unset_userdata('theme_scheme_process');
                    
                    // Compile global.scss file
                    $this->compile_global();
                    $this->CI->themes_model->set_is_default($theme_name, $type, 1);
                    
                    if($message) {
                        Template::set_message(sprintf(lang('process_theme_sucess'), ucfirst($theme_name)), 'success');
                    }
                }
                break;
            case 'schemes':
                $scheme_name = $id;
                $scheme = $this->CI->db->get_where('theme_settings', array('name' => $scheme_name, 'type' => $type))->row();
                if($scheme) {
                    $scheme_values = !empty($scheme->value) ? (array) json_decode($scheme->value) : null;
                    if(!empty($scheme_values)) {
                        // Get Default Theme
                        $default_theme = $this->CI->db->get_where('theme_settings', array('type' => 'themes', 'is_default' => 1))->row();
                        // Get All views of default theme
                        if($default_theme) {
                            // process views again with new color values
                            $theme_name = $default_theme->name;
                            $this->theme_settings = theme_settings($theme_name); // get header and footer to be processed
                            $views = $this->get_views($theme_name);
                            if($views) {
                                $this->process_views($views, $theme_name, $scheme_values);
                            }
                        }
                    }
                    
                    // Unset the theme_scheme_process session data
                    $this->CI->session->unset_userdata('theme_scheme_process');
                    
                    // Compile global.scss file
                    $this->compile_global();
                    $this->CI->themes_model->set_is_default($scheme_name, $type, 1);
                    
                    if($message) {
                        Template::set_message(sprintf(lang('process_scheme_sucess'), ucfirst(str_replace('_', ' ', $scheme_name))), 'success');
                    }
                }
                break;
            default:
                // Note: file_folder column is used instead of the view in the theme_views table
                $view = $this->CI->themes_model->find($id);
                if(!$view) {
                    return false;
                }
                if($view->type_id == 1) { // do not proceed in case of a widget
                    return false;
                }
                // Get Default Scheme
                $scheme_values = array();
                $scheme = $this->CI->db->get_where('theme_settings', array('type' => 'schemes', 'is_default' => 1))->row();
                if($scheme) {
                    $scheme_values = !empty($scheme->value) ? (array) json_decode($scheme->value) : null;
                }
                
                // Get Default Theme
                $theme_name = null;
                $view_themes = explode(',', $view->theme ?? '');
                if(!empty($view_themes[0])) {
                    $theme_name = $view_themes[0];
                } else {
                    $default_theme = $this->CI->db->get_where('theme_settings', array('type' => 'themes', 'is_default' => 1))->row();
                    // Get All views of default theme
                    if($default_theme) {
                        // process views again with new color values
                        $theme_name = $default_theme->name;
                    }
                }
                $success = $this->process_view_layouts($view, $scheme_values, $theme_name);                
                if($message) {
                    if($success) {
                        Template::set_message(lang('msg_default_success'), 'success');
                    } else {
                        Template::set_message(lang('msg_default_error'), 'error');
                    }
                }
        }
    }
    
    /**
     * Get views and layouts to set default
     */
    private function get_views($theme_name) {        
        // $where = "(theme = '" . $theme_name . "' OR type_id = 2) AND is_theme_default = 1";// array('theme' => $theme_name, 'is_theme_default' => 1);
        // $where = "(FIND_IN_SET('" . $theme_name . "', theme) AND is_theme_default = 1) OR (type_id = 2 AND (FIND_IN_SET('" . $theme_name . "', theme) OR FIND_IN_SET('default', theme)))";
        
        // get theme specific views and layouts only - give top preferences to the theme files first
        $where = "theme = '" . $theme_name . "' AND is_theme_default = 1 AND type_id != 1";
        $views_theme_only = $this->CI->db->where($where)->get('themes')->result_array();
        
        // get views and layouts where theme is also selected
        $where = "FIND_IN_SET('" . $theme_name . "', theme) AND is_theme_default = 1 AND type_id != 1";
        $views_themes = $this->CI->db->where($where)->get('themes')->result_array();
        
        // use default theme if no view or layout exist in the main theme
        $where = "FIND_IN_SET('default', theme) AND is_theme_default = 1 AND type_id != 1";
        $views_default = $this->CI->db->where($where)->get('themes')->result_array();
        
        // merge all views and layouts
        $result = array_merge($views_theme_only, $views_themes, $views_default);
        return $result;
    }
    
    /**
     * Common function to process views 
     */
    private function process_views($views, $theme_name, $scheme_values, $params = array()) {
        $views = $this->filter_duplicate_layouts($views, $theme_name);
        $views = $this->filter_duplicate_layouts($views, $theme_name, 0);
        if(empty($params)) {
            // reset all view and layouts default to 0
            $this->CI->db->where(array('type_id' => 0))->update('themes', array('is_default' => 0));
            $this->CI->db->where(array('type_id' => 2))->update('themes', array('is_default' => 0));
        }
        foreach($views as $view) {
            $view = (object) $view;
            $this->process_view_layouts($view, $scheme_values, $theme_name, $params);
        }
    }
    
    /**
     * Process all views and layouts
     * @param object $view
     * @param array $scheme_values
     * @param string $theme_name
     * @param boolean $params - Theme Demo parameters
     */
    private function process_view_layouts($view, $scheme_values, $theme_name, $params = array()) {
        $process_params = $params;
        $process_params['record'] = $view;
        // if($view->type_id == 0) {
        $this->process_view($view->id, $scheme_values, false, $theme_name, $process_params);
        //}
        if(empty($params)) {
            // reset the default
            $this->CI->db->where(array('file_folder' => $view->file_folder))->update('themes', array('is_default' => 0));
            
            // set the view as default
            $this->CI->db->where(array('id' => $view->id))->update('themes', array('is_default' => 1));
        }
        
        // set the default to 0 for the same view of other themes
        // $where_theme = "type_id = 0 AND !FIND_IN_SET('" . $theme_name . "', theme) AND file_folder = '" . $view->file_folder . "'";
        // $this->CI->db->where($where_theme)->update('themes', array('is_default' => 0, 'modified_on' => date('Y-m-d H:m:s')));
        
        // set the default to 0 for the same layout of other themes
        // $this->CI->db->where(array('type_id' => 2, 'theme != ' => $theme_name, 'file_folder' => $view->layout))->update('themes', array('is_default' => 0, 'modified_on' => date('Y-m-d H:m:s')));
        // check whether this view has a layout or not
        if($view->type_id == 0) { // do this for views only
            $theme_layout = $this->get_process_layout($theme_name, $view->layout, false); // first try to find the module layouts only
            if(!$theme_layout) {
                $theme_layout = $this->get_process_layout($theme_name, $view->layout, true); // find where a layout is assigned to multiple themes
            }
            if(!$theme_layout) { // view has no layout
                // make the default layout active
                $default_layout = $this->get_process_layout('default', $view->layout);
                if($default_layout) {
                    if(empty($params)) {
                        $this->CI->db->where('id', $default_layout->id)->update('themes', array('is_default' => 1, 'modified_on' => date('Y-m-d H:m:s')));
                    }                    
                    $process_params = $params;
                    $process_params['record'] = $default_layout;
                    $this->process_view($default_layout->id, $scheme_values, false, $theme_name, $process_params);
                }
            } else { // the view has a layout
                // make the theme layout active
                // $where_theme = "type_id = 2 AND !FIND_IN_SET('" . $theme_name . "', theme) AND file_folder = '" . $view->layout . "'";
                // $this->CI->db->where($where_theme)->update('themes', array('is_default' => 0, 'modified_on' => date('Y-m-d H:m:s')));
                if(empty($params)) {
                    // process the theme layout
                    $this->CI->db->where('id', $theme_layout->id)->update('themes', array('is_default' => 1, 'modified_on' => date('Y-m-d H:m:s')));
                }
                $process_params = $params;
                $process_params['record'] = $theme_layout;
                $this->process_view($theme_layout->id, $scheme_values, false, $theme_name, $process_params);
            }
        }
        return true;
    }
    
    /**
     * Get Layout to be processed
     * @param string $theme
     * @param string $layout
     * @return object[]
     */
    private function get_process_layout($theme, $layout, $find_set = true) {
        $cacheID = 'theme_layout_' . str_replace('-', '_', $this->CI->slug->create_slug($theme . '_' . $layout . '_' . $find_set));
        if(!$result = $this->widget_cache->load($cacheID)) {
            $where_theme = "type_id = 2 AND file_folder = '" . $layout . "'";
            if($find_set) {
                $where_theme .= " AND FIND_IN_SET('" . $theme . "', theme)";
            } else {
                $where_theme .= " AND theme = '" . $theme . "'";
            }
            $record = $this->CI->db->where($where_theme)->get('themes')->row();
            $result = array('record' => $record);
            $this->widget_cache->save($result, $cacheID, array('theme_default_set'));
        }
        return $result['record'];
    }
    
    /**
     * Get Theme Layout
     * @param string $theme
     * @param string $layout
     * @return object
     */
    private function get_theme_layout($theme, $layout) {
        $sql = "SELECT id FROM " . $this->CI->db->dbprefix . "themes WHERE FIND_IN_SET('" . $theme . "', theme) AND is_theme_default = 1 AND type_id = 2 AND layout = '" . $layout . "'";
        $view_layout = $this->CI->db->query($sql)->row();
        return $view_layout;
    }
    
    /**
     * Remove duplicate layouts
     * @param array $views
     */
    private function filter_duplicate_layouts($views, $theme_name, $match_value = 2) {
        $layouts = array();
        $layouts_theme = array();
        foreach($views as $key => $view) {
            $view = (object) $view;
            if($view->type_id == $match_value) {
                if($match_value == 0) {
                    $compare_value = $view->is_theme_default . '_' . $view->file_folder;
                } else {
                    $compare_value = $view->is_theme_default . '_' . $view->type_id . '_' . $view->layout . '_' . $view->file_folder;
                }
                $view_theme = explode(',', $view->theme ?? '');
                if(array_key_exists($compare_value, $layouts_theme)) {
                    if(!in_array($theme_name, $layouts_theme[$compare_value])) {
                        if(!in_array('default', $view_theme)) { // if no view, use default one
                            $unset_key = array_search($compare_value, $layouts);
                            unset($views[$unset_key]);
                        }
                    } else {
                        unset($views[$key]);
                    }
                }
                $layouts[$key] = $compare_value;
                $layouts_theme[$compare_value] = $view_theme;
            }
        }
        return $views;
    }
    
    /**
     * Export VIEW/WIDGET in XLS/XLSX format
     * @param string $table_name
     * @return boolean
     */
    public function export_excel($id, $type = 'themes', $archieve = 0) {
        $this->CI->load->library('format');
        if($type == 'schemes') {
            $query = $this->CI->db->query("SELECT * FROM " . $this->CI->db->dbprefix . "theme_settings WHERE name = '" . $id . "' AND type = 'schemes'");
        } else {
            $query = $this->CI->themes_model->export_data($id);
        }
        if(!$query) {
            return false;
        }
        $filename = ucwords($id) . '_' . ucfirst($type). '_'. date('dMy') . '_' . date('H.i.s');
        $excel_data = array(
            'file_name' => $filename,
            'query' => $query,
            'download' => true
        );
        $this->CI->format->to_excel($excel_data);
        // Insert into migrations table
        $this->CI->db->insert('migrations', array('type' => 'themes', 'start_id' => $id, 'end_id' => $id, 'file' => $filename, 'status' => 1, 'operation_type' => 'Export'));
    }
    
    /**
     * Read and import data using Excel
     * @param string $table_name - themes/theme_settings
     */
    public function import_excel() {
        $this->load->library('format');
        $file = $this->CI->input->post('file');
        $excel_params = array(
            'file_name' => $file
        );
        $excel_data = $this->format->from_excel($excel_params);
        if($excel_data['success'] === false) {
            Template::set_message($excel_data['message'], 'error');
            Template::redirect(site_url(SITE_AREA .'/developer/themes/import'));
        } else {
            $header = $excel_data['header'];
            $arr_data = $excel_data['data'];
        }
        
        // NOTE: escaped data will not work
        // prepare view and theme data
        $prepare_data = array(); // data without escaping
        $theme_data = array();
        $theme_data_string = array();
        $view_data = array();
        $write_files_ids = array(); // store ids of all rows to write contents on disk
        
        if(!empty($arr_data)) {
            foreach ($arr_data as $data_key => $data) {
                foreach($data as $key => $value) {
                    if(!empty($data['id'])) {
                        $write_files_ids[] = $data['id'];
                    }
                    if($key == 'views') {
                        // prepare theme_views table data
                        $views = explode(',', $data['views'] ?? '');
                        foreach($views as $view) {
                            $view_data[] = array(
                                'theme_id' => $data['id'],
                                'view' => $view
                            );
                        }
                    } else {
                        // insert and update
                        $theme_data[$data_key][$key] = $value;
                        // insert only
                        if($key == 'id') {
                            $prepare_data[$data_key][$key] = $value;
                        } else {
                            $prepare_data[$data_key][$key] = '\''.$value.'\'';
                        }
                    }
                }
                $theme_data_string[]= "(" . implode(',',$prepare_data[$data_key]) .")";
            }
        }
        
        // Remove views column from the header arra
        foreach ($header as $hkey => $hvalue){
            if ($hvalue == 'views') {
                unset($header[$hkey]);
            }
        }
        //send the data in an array format
        $data['header'] = $header;
        
        /****************** COMPARE DATABASE FIELDS WITH EXCEL FILE HEADER **************/
        //get fields name from database table
        $table_name = 'themes';
        if(in_array('type', $header) && in_array('value', $header)) {
            $table_name = 'theme_settings';
        }
        $fields = $this->CI->db->field_data($table_name);
        $fields_name = array(); //ARRAY TO STORE FIELD NAME
        if(!empty($fields)) {
            foreach($fields as $field) {
                $fields_name[] = strtolower($field->name);
            }
        }
        
        //covert header fields to small letters for array difference
        $header_fields = array();
        foreach($header as $he) {
            $header_fields_name[] = strtolower($he);
        }
        
        $fields_difference = array_diff($header_fields_name, $fields_name);
        //perform insert operation if there is no field difference
        if(empty($fields_difference)&& isset($theme_data)) {
            //Perform insert operation for multiple rows and ignore if any row has wrong values
            if($this->CI->input->post('insert_or_update') == 1) {
                // User selected insert only, ignore existing rows
                $sql = "INSERT IGNORE INTO ". $this->CI->db->dbprefix . $table_name ." (" . implode(',', $header) . ") VALUES " . implode(',', $theme_data_string);
                $this->CI->db->query($sql);
                // Insert theme_views table data
                if(!empty($view_data)) {
                    foreach($view_data as $theme_view) {
                        $insert_query = $this->CI->db->insert_string('theme_views', $theme_view);
                        $insert_query = str_replace('INSERT INTO', 'INSERT IGNORE INTO', $insert_query);
                        $this->CI->db->query($insert_query);
                    }
                }
            }
            else if($this->CI->input->post('insert_or_update') == 0) {
                // User selected insert only, ignore existing rows
                // The below query will not work as batch insert is performed
                /*$sql = "INSERT IGNORE INTO ". $this->CI->db->dbprefix . $table_name ." (" . implode(',', $header) . ") VALUES " . implode(',', $theme_data_string) ." ON DUPLICATE KEY UPDATE " . implode(',', $update_fields);
                 //$sql .= " ON DUPLICATE KEY UPDATE " .implode(', ', $update_fields);
                 $this->CI->db->query($sql);*/
                // $this->import_data($table_name, $theme_data, $view_data);
                if($table_name == 'themes') {
                    $this->import_data($table_name, $theme_data, $view_data);
                } else {
                    foreach($theme_data as $data_scheme) {
                        $data_scheme['value'] = stripslashes($data_scheme['value']);
                        $where = array('name' => $data_scheme['name'], 'type' => 'schemes');
                        $count_all_results = $this->CI->db->where($where)->count_all_results($table_name);
                        if($count_all_results == 0) {
                            $this->CI->db->where($where)->insert($table_name, $data_scheme);
                        } else {
                            $this->CI->db->where($where)->update($table_name, $data_scheme);
                        }
                    }
                }
            }
            // Write Files on Disk and Auto Fill SCSS variables
            if($this->CI->input->post('write_files') == 1 && $table_name != 'theme_settings') {
                foreach($write_files_ids as $file_id) {
                    $this->widgets->write_files(null, $file_id, true);
                }
            }
            // Set Default after import
            if($this->CI->input->post('set_default') == 1 && $table_name == 'theme_settings') {
                // only set first one as default
                $sd = 1;
                foreach($theme_data as $scheme_data) {
                    if($sd == 1) {
                        $this->set_default('schemes', $scheme_data['name'], false);
                    }
                    ++$sd;
                }
            }
            return true;
        }
        else {
            // CONVERT FIRST LETTER TO UPPERCASE
            $mismatched_fields = array();
            foreach($fields_difference as $fields) {
                $mismatched_fields[] = ucfirst($fields);
            }
            $columns_error = implode(", ", $mismatched_fields); //ADD COMMAS WITH MULTIPLE VALUES
            Template::set_message(lang('upload_failure') .'. Column name(s) ' .$columns_error .' mismatched.', 'error'); // Display errors to the user
            return false;
        }
    }
    
    /**
     * Import XLS/XLSX - INSERT AND UPDATE
     * @param string $table_name
     * @param array $theme_data
     * @param array $view_data
     */
    private function import_data($table_name, $theme_data, $view_data) {
        $this->CI->db->trans_start();
        foreach ($theme_data as $rows_to_insert) {
            if(!empty($rows_to_insert)) {
                foreach($rows_to_insert as $key => $value) {
                    if($key != 'id') {
                        $update_fields[] = $key.' = \''.$value.'\'';
                    }
                }
                $insert_query = $this->CI->db->insert_string($table_name, $rows_to_insert) . ' ON DUPLICATE KEY UPDATE ' .implode(', ', $update_fields);
                $insert_query = str_replace('INSERT INTO', 'INSERT IGNORE INTO', $insert_query);
                $this->CI->db->query($insert_query);
                unset($update_fields);
            }
        }
        $this->CI->db->trans_complete();
        $this->CI->db->on_duplicate('theme_views', $view_data);
    }
    
    /**
     * Save Upload File
     * @return boolean
     */
    public function save_import_file() {
        $this->CI->load->model('migrate/migrate_model');
        $data = array();
        $data['type'] = 'themes';
        if($_POST['online_file_path']) {
            $this->CI->form_validation->set_rules('online_file_path', lang('label_import') .' '.lang('label_file'), 'trim|required|check_file_existence');
            if ($this->CI->form_validation->run() === false) {
                return false;
            }
            $data['file'] =	$this->CI->input->post('online_file_path');
        } else {
            $this->CI->form_validation->set_rules('uploaded_file', lang('label_import') .' '.lang('label_file'), 'trim|required');
            //Check whether file is uploaded or not
            if(isset($_FILES['migrate_file']) && $_FILES['migrate_file']['size'] > 0) {
                $file_data = $this->upload_file();
                if(!$file_data) {
                    $_POST['file_upload'] = 'NONE';
                    $this->CI->form_validation->set_rules('file_upload', lang('label_import') .' '.lang('label_file'), 'valid_file_upload');
                }
                // Avoid logo upload process, if the form has validation issues with other fields
                $_POST['uploaded_file'] = $file_data && isset($file_data['upload_data']['file_name']) ? $file_data['upload_data']['file_name'] : '';
            }
            if ($this->CI->form_validation->run() === false) {
                return false;
            }
            $data['file'] = $this->CI->input->post('uploaded_file');
        }
        $data['status']     	= 0;
        $data['operation_type']	= 'Import';
        // $id = $this->CI->migrate_model->insert($data); // Do not insert data into migrations table
        
        /*if (is_numeric($id)) {
         $_POST['file'] = $data['file'];
         $return = $id;
         } else {
         $return = FALSE;
         }
         return $return;*/
        return 1;
    }
    
    /**
     * Allow admin to upload excel file for import
     * @return multitype:NULL
     */
    private function upload_file(){
        if(!gc_demo_mode(array('no_message' => true, 'no_redirect' => true))) {
            return false;
        }
        $config['upload_path'] = realpath(FCPATH . 'uploads/imports/'); //Make SURE that you chmod this directory to 777!
        $config['allowed_types'] = 'xlsx|zip';//'csv|xls|xlsx';
        $config['max_size']='409600';
        $config['remove_spaces']=TRUE; //Remove spaces from the file name
        $this->CI->load->library('upload', $config);
        if (!$this->CI->upload->do_upload('migrate_file')) {
            $this->CI->session->set_flashdata('upload_file_error', $this->CI->upload->display_errors());
            return false;
        }
        else {
            $data = array('upload_data' => $this->CI->upload->data());
        }
        return $data;
    }
    
    
    /**
     * Migrate Design Specific Language Strings
     * @param boolean $is_import - TRUE -> Import, FALSE -> Export
     */
    public function migrate_language_strings($is_import = true) {
        // To simplify, the language import/export is done using database instead of files
        // all language text must be kept in a single file called custom_lang.php inside the application folder.
        $lang_file = 'custom_lang.php';
        $table_name = 'tmp_theme_langs';
        $this->CI->load->helper('translate/languages');
        if($is_import) {
            // read table and get text
            $theme_langs = array();
            if ($this->is_table_exists($table_name) ) {
                $theme_langs = $this->CI->db->get($table_name)->result();
            }
            if(!empty($theme_langs)) {
                $custom_themes_array = load_lang_file($lang_file, 'english');
                foreach($theme_langs as $theme_lang) {
                    $find = $theme_lang->key;
                    $value = $theme_lang->value;
                    $key = null;
                    if($custom_themes_array !== FALSE) { // by default, language file is empty
                        $key = array_search ($find, $custom_themes_array);
                    }
                    // Translate only if the key is not in the file
                    if(empty($key)) {
                        $custom_themes_array[$find] = $value;
                        save_lang_file($lang_file, 'english', $custom_themes_array, false, true);
                    }
                }
            }
        } else { // Export Language File
            // read language file
            $custom_themes_array = load_lang_file($lang_file, 'english');
            $table_data = array();
            if(!empty($custom_themes_array)) {
                foreach($custom_themes_array as $key => $value) {
                    $table_data[] = array(
                        'key' => $key,
                        'value' => $value
                    );
                }
                
                // create temporary table
                $this->CI->load->dbforge();
                $this->CI->dbforge->drop_table('tmp_theme_langs', TRUE);
                $this->CI->dbforge->add_field(array(
                    'id' => array(
                        'type' => 'INT',
                        'constraint' => 5,
                        'unsigned' => TRUE,
                        'auto_increment' => TRUE
                    ),
                    'key' => array(
                        'type' => 'VARCHAR',
                        'constraint' => '255',
                        'unique' => TRUE,
                        'null' => FALSE
                    ),
                    'value' => array(
                        'type' => 'TEXT',
                        'null' => FALSE
                    )
                ));
                $this->CI->dbforge->add_key('id', TRUE); // add `id` as primary key
                $this->CI->dbforge->create_table($table_name);
                
                // insert data
                $this->CI->db->insert_batch($table_name, $table_data);
            }
        }
    }
    
    /**
     * Create multi demo folder, if they does not exist
     * @param string $folder
     * @param boolean $create_dir - create directory if it does not exist
     */
    public function mkdir_demo_folders($folder, $create_dir = true) {
        $dir = explode('/', $folder ?? '');
        $filename = end($dir);
        $folder = str_replace('/' . $filename, '', $folder);
        if($create_dir) { // create directory
            $demo_dir = $this->widget_path . $folder;
            if (!empty($demo_dir) && !file_exists($demo_dir)) {
                mkdir($demo_dir, 0777, true);
            }
        }
    }
    
    /*********************************************
     * DEMO DATA
     *********************************************/
    /**
     * Load Demo Theme Specific Views
     * @param string $theme_name
     * @return NULL[][]
     */
    public function demo_views($theme_name) {
        $this->CI->load->library('slug');
        $this->CI->load->library('gzf_cache', array('lifetime' => 172800)); // 2 days
        $this->z_cache = $this->CI->gzf_cache->get_instance();
        $cacheID = 'theme_demo_' . str_replace('-', '_', $this->CI->slug->create_slug($theme_name));
        if(!$views = $this->z_cache->load($cacheID)) {
            $views = array();
            $select = array('id', 'theme', 'is_theme_default', 'assets', 'type_id', 'layout', 'file_folder', 'file_name', 'file_url');
            $where = "type_id = 0 AND FIND_IN_SET('" . $theme_name . "', theme) AND is_theme_default = 1";
            $records_themes = $this->CI->db->select($select)->where($where)->get('themes')->result_array();
            $where = "type_id = 0 AND FIND_IN_SET('default', theme) AND is_theme_default = 1";
            $records_default = $this->CI->db->select($select)->where($where)->get('themes')->result_array();
            $records = array_merge($records_themes, $records_default);
            if($records) {
                $records = $this->filter_duplicate_layouts($records, $theme_name, 0);
                if($records) {
                    foreach($records as $record) {
                        $record = (object) $record;
                        $views[$record->file_folder] = array(
                            'assets' => $record->assets,
                            'layout' => $record->layout,
                            'file_folder' => $record->file_folder,
                            'file_url' => $record->file_url
                        );
                    }
                }
            }
            $this->z_cache->save($views, $cacheID, array('theme_options'));
        }
        return $views;
    }
    
    /**
     * Prepare Layouts, CSS and JS files
     * @param string $theme_name
     */
    public function prepare_layouts_css_js($theme_name, $scheme = 'orange', $regenerate = false) {
        $params = array('theme_demo' => true, 'theme' => $theme_name, 'scheme' => $scheme);
        // check for the file existence first
        if(!file_exists(FCPATH . 'themes/default/css/demo/' . $theme_name . '/'. $scheme . '/index.css') || $regenerate) {
            clear_cache('default_theme'); // clear the session variable
            $this->theme_settings = theme_settings($theme_name); // get header and footer to be processed
            $views = $this->get_views($theme_name);
            if($views) {
                // Get Default Scheme
                $scheme_values = array();
                $scheme = $this->CI->db->get_where('theme_settings', array('type' => 'schemes', 'name' => $params['scheme']))->row();
                if(!$scheme) {
                    $scheme = $this->CI->db->get_where('theme_settings', array('type' => 'schemes', 'is_default' => 1))->row();
                }
                $scheme_values = !empty($scheme->value) ? (array) json_decode($scheme->value) : null;                
                
                // process found views and layouts
                $this->process_views($views, $theme_name, $scheme_values, $params);
            }
        }
    }
    /*********************************************
     * END OF DEMO DATA
     *********************************************/
}