<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Migration_Install_themes extends Migration
{
	/**
	 * The name of the database table
	 *
	 * @var String
	 */
	private $table_name = 'themes';
	private $table_views = 'theme_views';
	private $table_settings = 'theme_settings';
	
	/**
	 * The table's fields
	 *
	 * @var Array
	 */
	private $fields = array(
		'id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'auto_increment' => TRUE,
		),
		'type_id' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default' => 0
		),
		'is_widget_all' => array( // Is this widget applicable to all views
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default' => 0
		),
		'is_default' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default' => 0
		),
		'is_theme_default' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default' => 0
		),
		'theme' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE,
		),
		'layout' => array(
			'type' => 'VARCHAR',
			'constraint' => 100,
			'null' => TRUE,
		),
		'name' => array(
			'type' => 'VARCHAR',
			'constraint' => 100,
			'null' => FALSE,
		),
		'contents' => array(
			'type' => 'LONGTEXT',
			'null' => TRUE
		),
		'contents_default' => array(
			'type' => 'LONGTEXT',
			'null' => TRUE
		),
		'image_url' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE,
		),
		'file_folder' => array(
			'type' => 'CHAR',
			'constraint' => 100,
			'null' => TRUE,
		),
		'file_name' => array(
			'type' => 'VARCHAR',
			'constraint' => 100,
			'null' => FALSE,
		),
		'file_url' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => FALSE,
		),
		'scss_variables' => array(
			'type' => 'LONGTEXT',
			'null' => TRUE
		),
		'scss_values' => array(
			'type' => 'LONGTEXT',
			'null' => TRUE
		),
		'assets' => array(
			'type' => 'TEXT',
			'null' => TRUE,
		),
		'assets_css' => array(
			'type' => 'TEXT',
			'null' => TRUE,
		),
		'assets_js' => array(
			'type' => 'TEXT',
			'null' => TRUE,
		),
		'params' => array(
			'type' => 'TEXT',
			'null' => TRUE,
		),
		'notes' => array(
			'type' => 'TEXT',
			'null' => TRUE,
		),
		'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'modified_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP'
	);
	
	/**
	 * The table's views fields
	 *
	 * @var Array
	 */
	private $fields_views = array(
		'theme_id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => FALSE,
		),
		'view' => array(
			'type' => 'CHAR',
			'constraint' => 50,
			'null' => FALSE,
		),
	);
	
	/**
	 * The table's settings fields
	 *
	 * @var Array
	 */
	private $fields_settings = array(
		'name' => array(
			'type' => 'VARCHAR',
			'constraint' => 100,
			'null' => FALSE,
			'unique' => TRUE,
		),
		'type' => array(
			'type' => 'CHAR',
			'constraint' => 50,
			'null' => FALSE,
		),
		'is_default' => array( // Applicable only to Theme and Schemes
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default' => 0
		),
		'value' => array(
			'type' => 'LONGTEXT',
			'null' => FALSE,
		),
	);
	
	/**
	 * Install this migration
	 *
	 * @return void
	*/
	public function up() {
		// Themes Table
		$this->dbforge->add_field($this->fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->create_table($this->table_name);
		$unique_widget = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " ADD UNIQUE INDEX `unique_widget` (`type_id`, `name`, `file_name`)";
		$this->db->query($unique_widget);
		
		// Theme Views Table
		$this->dbforge->add_field($this->fields_views);
		$this->dbforge->create_table($this->table_views);
		$unique_view = "ALTER TABLE " . $this->db->dbprefix . $this->table_views . " ADD UNIQUE INDEX `unique_view` (`theme_id`, `view`)";
		$this->db->query($unique_view);
		
		// Theme Settings Table
		$this->dbforge->add_field($this->fields_settings);
		$this->dbforge->create_table($this->table_settings);
	}
	
	//--------------------------------------------------------------------
	
	/**
	 * Uninstall this migration
	 *
	 * @return void
	 */
	public function down() {
		if ($this->db->table_exists($this->table_name)) {
			$this->dbforge->drop_table($this->table_name);
		}
		if ($this->db->table_exists($this->table_views)) {
			$this->dbforge->drop_table($this->table_views);
		}
		if ($this->db->table_exists($this->table_settings)) {
			$this->dbforge->drop_table($this->table_settings);
		}
	}
}