<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Online Software Solutions							   # ||
|| # Copyright (C) 2021 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class Themes_model extends BF_Model {

	public $table_name	= "themes";
	protected $key			= "id";
	protected $soft_deletes	= false;
	protected $date_format	= "datetime";

	protected $log_user 	= FALSE;

	protected $set_created	= TRUE;
	protected $set_modified = TRUE;
	protected $created_field = "created_on";
	protected $modified_field = "modified_on";

	/*
		Customize the operations of the model without recreating the insert, update,
		etc methods by adding the method names to act as callbacks here.
	 */
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 		= array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	/*
		For performance reasons, you may require your model to NOT return the
		id of the last inserted row as it is a bit of a slow method. This is
		primarily helpful when running big loops over data.
	 */
	protected $return_insert_id 	= TRUE;

	// The default type of element data is returned as.
	protected $return_type 			= "object";

	// Items that are always removed from data arrays prior to
	// any inserts or updates.
	protected $protected_attributes = array();

	/*
		You may need to move certain rules (like required) into the
		$insert_validation_rules array and out of the standard validation array.
		That way it is only required during inserts, not updates which may only
		be updating a portion of the data.
	 */
	protected $validation_rules 		= array();
	protected $insert_validation_rules 	= array();
	protected $skip_validation 			= FALSE;
	
	public $validation_scss_variables	= array(
		array(
				"field"		=> "field_options[name][]",
				"label"		=> "lang:label_name",
				"rules"		=> "trim|required|is_array|callback_unique_variable"
		),			
		array(
				"field"		=> "field_options[title][]",
				"label"		=> "lang:label_title",
				"rules"		=> "trim|required|is_array"
		),
		array(
				"field"		=> "field_options[description][]",
				"label"		=> "lang:label_description",
				"rules"		=> "trim|is_array"
		),
		array(
				"field"		=> "field_options[options][]",
				"label"		=> "lang:label_options",
				"rules"		=> "trim|is_array"
		),
		array(
				"field"		=> "field_options[value][]",
				"label"		=> "lang:label_value",
				"rules"		=> "trim|required|is_array"
		)
	);
	private $gc_themes_cache; // cache variable
	
	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct() {
	    parent::__construct();
	    // Load Zend Caching
	    $this->load->library('slug');
	    $this->load->library('gzf_cache', array('lifetime' => 900));
	    if(method_exists($this->gzf_cache, 'get_instance')) {
	        $this->gc_themes_cache = $this->gzf_cache->get_instance();
	    }
	}
	
	/**
	 * Theme Folders Fields
	 *
	 * @return array
	 */
	public function folders() {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'type_id',
	            'label' => 'label_type',
	            'frontend' => false,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'name:label_views;value:0|name:label_widgets;value:1|name:label_layouts;value:2',
	            'options_detailed' => 1,
	            'options_lang' => 1,
	            'class' => 3,
	            'rules' => 'trim|required|is_numeric',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'name',
	            'label' => 'label_name',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim|required|max_length[150]',
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'name_lang',
	            'label' => 'label_name_lang',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'folder',
	            'label' => 'label_file_folder',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim|required|max_length[150]|callback_unique_folder',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'priority',
	            'label' => 'label_priority',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'value' => 0,
	            'class' => 2,
	            'rules' => 'trim|required|is_numeric',
	        )
	    );
	    return $fields;
	}
	
	/**
	 * Theme Layouts Fields
	 *
	 * @return array
	 */
	public function layouts() {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'theme',
	            'label' => 'themes_singular',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'name:themes;value:database',
	            'options_detailed' => 1,
	            'class' => 3,
	            'rules' => 'trim|required',
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'layout',
	            'label' => 'label_layout',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'name:layouts;value:database',
	            'options_detailed' => 1,
	            'class' => 3,
	            'rules' => 'trim|required|callback_unique_layout',
	        ),/*
	        array(
	            'type' => 'custom',
	            'name' => 'value',
	            'label' => 'label_value',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'tooltip' => lang('help_theme_options'),
	            'class' => 9,
	            'rules' => 'trim',
	        ),*/
	        array(
	            'type' => 'custom',
	            'name' => 'widgets',
	            'label' => 'label_widgets',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'tooltip' => lang('help_theme_options'),
	            'class' => 9,
	            'rules' => 'trim',
	        ),
	        
	    );
	    return $fields;
	}
	
	/**
	 * Theme Block Fields
	 *
	 * @return array
	 */
	public function blocks($role = 'admin') {
	    $theme_layouts = array();
	    $theme_layouts = $this->db->where(array('block_type_id' => 1))->get('theme_blocks')->result();
	    /*if($layouts) {
	        foreach($layouts as $layout) {
	            $theme_layouts[$layout->id] = $layout->name;
	        }
	    }
	    dump($theme_layouts);*/
	    Template::set('theme_layouts', $theme_layouts);
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	       array(
	            'type' => 'dropdown',
	            'name' => 'layout_ids',
	            'label' => 'singular_theme_layouts',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'name:theme_layouts;value:database',
	            'options_detailed' => 1,
	            'options_lang' => 1,
	            'field_extra' => 'selectpicker:1;multiple:1;max_options:10',
	            'class' => 3,
	            'rules' => 'trim'
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'block_type_id',
	            'label' => 'label_type',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'name:label_default;value:0|name:label_layout;value:1|name:label_view;value:2|name:label_widget;value:3',
	            'options_detailed' => 1,
	            'options_lang' => 1,
	            'class' => 3,
	            'rules' => 'trim|required'
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'parent_id',
	            'label' => 'label_parent',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'options' => 'name:theme_blocks;value:database',
	            'options_detailed' => 1,
	            'class' => 3,
	            'rules' => 'trim|max_length[150]'
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'name',
	            'label' => 'label_name',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'edit' => 1,
	            'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
	            'class' => 4,
	            'rules' => 'trim|required|max_length[150]'
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'name_lang',
	            'label' => 'name_lang',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'slug',
	            'label' => 'label_slug',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim|max_length[150]',
	            'class' => 4
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'file_folder',
	            'label' => 'theme_file_folder',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'class' => 4,
	            'rules' => 'trim|max_length[255]'
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'priority',
	            'label' => 'label_priority',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'value' => 0,
	            'class' => 2,
	            'rules' => 'trim|required|is_numeric',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_on',
	            'label' => 'created_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_on',
	            'label' => 'modified_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        )
	    );
	    return $fields;
	}
	
	/**
	 * Theme Content Fields
	 * 
	 * NOT USED BECAUSE OF COMPLEXITY IN DIFFERENT MODULES
	 * E.g. Testimonials requires approvals, videos can be product specific etc.
	 * Each module has its own display fields
	 *
	 * @return array
	 */
	public function contents() {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'selectize',
	            'name' => 'user_id',
	            'label' => 'bf_user',
	            'placeholder' => 'placeholder_selectize',
	            'frontend' => false,
	            'admin_only' => false,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'options' => 'load:users;module:users;max_items:1',
	            'options_detailed' => 1,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'selectize',
	            'name' => 'brand_ids',
	            'label' => 'singular_product_brands',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'options' => 'load:product_brands;module:product_brands;max_items:100',
	            'options_detailed' => 1,
	            'rules' => 'trim|max_length[255]',
	        ),
	        array(
	            'type' => 'selectize',
	            'name' => 'product_ids',
	            'label' => 'singular_products',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'load:products;module:products;max_items:100',
	            'options_detailed' => 1,
	            'rules' => 'trim|max_length[255]',
	        ),
	        array(
	            'type' => 'selectize',
	            'name' => 'group_ids',
	            'label' => 'gc_menu_groups',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => true,
	            'options' => 'load:theme_content_groups;module:theme_content_groups;max_items:100',
	            'options_detailed' => 1,
	            'rules' => 'trim|max_length[255]',
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'content_type',
	            'label' => 'label_type',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'name:content_features;value:features|name:content_faqs;value:faqs',
	            'options_detailed' => 1,
	            'options_lang' => 1,
	            'class' => 3,
	            'rules' => 'trim|required|is_numeric',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'title',
	            'label' => 'label_title',
	            'placeholder' => 'placeholder_title',
	            'placeholder_lang' => 1,
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'edit' => 1,
	            'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
	            'rules' => 'trim',
	            'control_class' => 'content-field'
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'subtitle',
	            'label' => 'label_subtitle',
	            'placeholder' => 'placeholder_subtitle',
	            'placeholder_lang' => 1,
	            'tooltip' => 'help_subtitle',
	            'tooltip_lang' => 1,
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	            'control_class' => 'content-field content-features'
	        ),
	        array(
	            'type' => 'textarea',
	            'name' => 'description',
	            'label' => 'label_description',
	            'placeholder' => 'placeholder_description',
	            'placeholder_lang' => 1,
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	            'control_class' => 'content-field'
	        ),
	        array(
	            'type' => 'iconpicker',
	            'name' => 'icon',
	            'label' => 'label_icon',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	            'control_class' => 'content-field content-features'
	        ),
	        array(
	            'type' => 'filemanager',
	            'name' => 'image',
	            'label' => 'label_image',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	            'control_class' => 'content-field content-features content-screenshots'
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'video_url',
	            'label' => 'label_youtube_url',
	            'placeholder' => 'placeholder_youtube_url',
	            'placeholder_lang' => 1,
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim|valid_youtube_url',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'external_url',
	            'label' => 'label_external_url',
	            'placeholder' => 'placeholder_external_url',
	            'placeholder_lang' => 1,
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim|valid_url'
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'priority',
	            'label' => 'label_priority',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'value' => 0,
	            'class' => 2,
	            'rules' => 'trim|required|is_numeric',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'priority',
	            'label' => 'label_priority',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'status',
	            'label' => 'label_status',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_on',
	            'label' => 'created_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_on',
	            'label' => 'modified_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        
	    );
	    return $fields;
	}
	
	/**
	 * Theme Options Fields
	 *
	 * @return array
	 */
	public function options() {
	    /*$theme_type_options = array(
	        array('id' => '', 'name' => lang('placeholder_select'))
	    );
	    $theme_types = $this->db->select('id, name')->where('option_type = 0')->get('theme_options')->result();
	    if($theme_types) {
	        foreach($theme_types as $theme_type) {
	            $theme_type_options[] = array('id' => $theme_type->id, 'name' => $theme_type->name);
	        }
	    }
	    Template::set('theme_type_options', $theme_type_options);*/
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'selectize',
	            'name' => 'option_parent_ids',
	            'label' => 'label_parent',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'load:theme_options;module:theme_options;max_items:10',
	            'options_detailed' => 1,
	            'rules' => 'trim',
	        ),/*
	        array(
	            'type' => 'dropdown',
	            'name' => 'option_type',
	            'label' => 'label_type',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'name:theme_type_options;value:database',
	            'options_detailed' => 1,
	            'class' => 3,
	            'rules' => 'trim',
	        ),*/
	        array(
	            'type' => 'text',
	            'name' => 'name',
	            'label' => 'label_name',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'edit' => 1,
	            'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
	            'rules' => 'trim|required|max_length[100]',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'slug',
	            'label' => 'label_slug',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'textarea',
	            'name' => 'description',
	            'label' => 'label_description',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_on',
	            'label' => 'created_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_on',
	            'label' => 'modified_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        
	    );
	    return $fields;
	}
	
	/**
	 * Theme Demos Fields
	 *
	 * @return array
	 */
	public function demos() {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'block_id',
	            'label' => 'label_page',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim|required'
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'name',
	            'label' => 'label_name',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim|required|max_length[100]',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'contents',
	            'label' => 'label_contents',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim|required',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_on',
	            'label' => 'created_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_ip',
	            'label' => 'created_ip',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_on',
	            'label' => 'modified_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_ip',
	            'label' => 'modified_ip',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        
	    );
	    return $fields;
	}	
	
	/**
	 * Show all Views/Widgets/Layouts to the Admin - Admin Index Page
	 * @param number $count
	 * @param string $where
	 * @param number $offset
	 * @param number $limit
	 * @return number
	 */
	public function get_widgets($count = 0, $where = null, $sort_by = "T.id DESC", $offset = 0, $limit = 25) {
		if($count == 0) {
			$sql = "SELECT T.*";
			//$sql .= ", CONCAT(CONCAT(TV.view), (CASE WHEN  COUNT(TV.theme_id) > 1 THEN ',' ELSE '' END)) AS views";
			$sql .= ", GROUP_CONCAT(CONCAT(TV.view) SEPARATOR ',') AS views";
		} else {
			$sql = "SELECT COUNT(T.id) AS TotalCount";
		}
		$sql .= " FROM " . $this->db->dbprefix . "themes T";
		if($count == 0) {
			$sql .= " LEFT JOIN " . $this->db->dbprefix . "theme_views TV ON TV.theme_id = T.id";
		}
		if(!empty($where)) {
			$sql .= " WHERE " . $where;
		}		
		if($count == 0) {
			$sql .= " GROUP BY T.id";
			$sql .= " ORDER BY " . $sort_by;
			$sql .= " LIMIT " . $offset . ", " . $limit;
			return $this->db->query ( $sql )->result();
		} else {
			$result = $this->db->query ( $sql )->row();
			return $result ? $result->TotalCount : 0;
		}
	}
	
	public function file_url($type_id, $file_folder) {
		// Add slash after the folder name
		switch($type_id) {
			case 1:
				$file_url = 'widgets/';
				break;
			case 2:
				$file_url = 'layouts/';
				break;
			default:
				$file_url = 'views/';				
		}
		$file_url .= !empty($file_folder) ? $file_folder . '/' : '';
		return  $file_url;
	}
	
	/*public function get_scss_variables($theme_id) {
		return $this->db->get_where('theme_scssvariables', array('theme_id' => $theme_id))->result();
	}*/
	
	/**
	 * Display View specific widgets
	 * @param string $view_type
	 * @param int $theme_id
	 */
	public function get_view_widgets($view_type, $theme_id) {
		$sql = "SELECT T.id, T.name, T.image_url, T.file_url, T.params, T.notes FROM " . $this->db->dbprefix . "themes T";
		$sql .= " LEFT JOIN " . $this->db->dbprefix . "theme_views TV ON TV.theme_id = T.id";
		// Except this view, get all Widgets
		// $sql .= " WHERE T.id != " . $theme_id . " AND T.type_id = 1";
		$sql .= " WHERE T.type_id = 1";
		$sql .= " AND (TV.view = '" . $view_type . "' OR T.is_widget_all = 1)";
		$sql .= " GROUP BY T.id";
		return $this->db->query ( $sql )->result();		
	}
	
	/**
	 * Display Widget URL to the user
	 * @param string $file_url
	 * @param array $params
	 * @return string
	 */
	public function widget_url($file_url, $params = array(), $id = 0) {
	    $widget_params = array();
	    $path_array = explode('/', $file_url);
	    $widget_folder = !empty($path_array[1]) ? $path_array[1] : '';
	    switch($widget_folder) {
	        case 'testimonials':
	            break;
	        case 'packages':
	            $widget_params[] = "'packages' => ''";
	            $widget_params[] = "'type_id' => 0";
	            break;
	        case 'sliders':
	            $widget_params[] = "'module' => 'listings'";
	            $widget_params[] = "'block' => 'popular'";
	            break;
	        case 'products':
	            $widget_params[] = "'module' => 'products'";
	            $widget_params[] = "'block' => 'popular'";
	            break;
	        case 'banner_sliders':
	            $widget_params[] = "'ids' => 0";
	            $widget_params[] = "'group_id' => 0";
	            break;
	        case 'categories':
	            $widget_params[] = "'parent_id' => 0";
	            break;
	        case 'fprlvs':
	            $widget_params[] = "'module' => 'products'";
	            $widget_params[] = "'block' => 'popular'";
	            break;
	            
	    }
	    switch($widget_folder) {
	        case 'testimonials':
	        case 'packages':
	        case 'sliders':
	        case 'products':
	        case 'banner_sliders':
	        case 'categories':
	            $file_url = 'widgets/modules/_call';
	            $widget_params[] = "'load' => '" . $widget_folder . "'";
	            $widget_params[] = "'widget' => '" . $path_array[2] . "'";
	            $widget_params[] = "'heading' => 2";
	            $widget_params[] = "'limit' => 5";
	            $widget_params[] = "'cols' => 3";
	            $widget_params[] = "'records' => ''";
	            $widget_params[] = "'parent_class' => ''";
	            $widget_params[] = "'heading_block_class' => ''";
	            $widget_params[] = "'heading_class' => ''";
	            $widget_params[] = "'heading_sub_class' => ''";
	            $widget_params[] = "'heading_text' => ''";
	            $widget_params[] = "'heading_text_lang' => ''";
	            $widget_params[] = "'heading_sub_text' => ''";
	            $widget_params[] = "'heading_sub_text_lang' => ''";
	            $widget_params[] = "'content_parent_class' => ''";
	            $widget_params[] = "'content_class' => ''";
	            $widget_params[] = "'content_widget_class' => ''";
	            $widget_params[] = "'content_bg' => ''";
	            break;
	        
	    }
	    if(!empty($path_array[2]) && $path_array != '_call') {
    		$params = json_decode($params ?? '');
    		$pages_params = array( // Allow user to add a widget to a page
    			'id:' . $id
    		);
    		if(!empty($params)) {
    			foreach($params as $param) {
    				$param_value = (is_numeric($param->value) ? $param->value : "'$param->value'");
    				$widget_params[] = "'$param->name'" . ' => ' . $param_value;
    				$pages_params[] = "$param->name:" . $param_value;
    			}
    		}
    		if(!empty($widget_params)) {
    			$widget_url = '&lt;?php echo theme_view(\'' . $file_url .'\', array('. implode(', ', $widget_params) .'));?&gt;';
    		} else {
    			$widget_url = '&lt;?php echo theme_view(\'' . $file_url .'\');?&gt;';
    		}
    		if($id > 0) {
    			$widget_url .= '<br /><strong>' . lang('label_pages') . ' ' . lang('label_widget'). '</strong>: <br />{widget}' . implode('|',$pages_params) . '{/widget}';
    		}
    		return $widget_url;
	    }
	}
	
	/**
	 * Get view/widget information
	 * @param integer $id
	 */
	public function export_data($id) {
		$export_widget_ids = array();
		$record = $this->select('id, file_url')->themes_model->find($id);
		if($record) {
			$export_widget_ids[] = $record->id;
			$widgets = $this->widgets->get_widgets($record->file_url . '.php');
			if(!empty($widgets)) {
				// a widget can be used multiple times in the same view
				// process such widgets only once
				$widgets = array_unique($widgets);
				foreach($widgets as $widget) {
					// get widget id
					$widget_info = $this->themes_model->select('id, file_url')->find_by(array('file_url' => $widget, 'type_id' => 1));
					if($widget_info) {
						$export_widget_ids[] = $widget_info->id;
					}
				}
			}
		}
		$sql = "SELECT T.*, GROUP_CONCAT(TV.view separator ',') AS views FROM " . $this->db->dbprefix . "themes T";
		$sql .= " INNER JOIN " . $this->db->dbprefix . "theme_views TV ON TV.theme_id = T.id";
		$sql .= " WHERE T.id IN (" . implode(',', $export_widget_ids) . ")";
		$sql .= " GROUP BY T.id";
		return $this->db->query ( $sql );
	}
	
	/**
	 * Make a View theme default
	 * This method should be called after the View/Widget save
	 * @param string $id
	 * @param int $value
	 */
	public function is_theme_default($id, $value) {
		// Get theme and view information
		$sql = "SELECT T.theme, TV.view FROM " . $this->db->dbprefix . "themes T";
		$sql .= " INNER JOIN " . $this->db->dbprefix . "theme_views TV ON T.id = TV.theme_id";
		$sql .= " WHERE T.id = " . $id;
		$result = $this->db->query($sql)->row();
		if(!$result) {
			return false;
		}
		$theme = $result->theme;
		$view = $result->view;
		
		// Get ids of all views where view and theme name matches 
		$qsql = "SELECT T.id, T.is_theme_default FROM " . $this->db->dbprefix . "themes T";
		$qsql .= " INNER JOIN " . $this->db->dbprefix . "theme_views TV ON T.id = TV.theme_id AND TV.view = '" . $view . "'";
		$qsql .= " WHERE T.id != $id AND T.theme = '" . $theme ."'";
		$qresult = $this->db->query($qsql)->result();		
		if($value == 1) {
			// change is_theme_default variable to 0 for all rows
			if($qresult) {
				foreach($qresult as $qid) {
					$this->db->where('id', $qid->id)->update('themes', array('is_theme_default' => 0));
				}
			}
			// Make this view as default for $theme
			$this->db->where('id', $id)->update('themes', array('is_theme_default' => 1));			
		} else {
			if($qresult) { // Default already exist for $view and $theme
				$this->db->where('id', $id)->update('themes', array('is_theme_default' => 0));
			} else { // There is no default, make this view as the theme default
				$this->db->where('id', $id)->update('themes', array('is_theme_default' => 1));
			}
		}
	}
	
	/**
	 * Override SCSS values with default
	 * @param int $id
	 * @param string $type_id - 0 - Widget, 1 - Global, 2 - Scheme
	 * @return object - variables with new values
	 */
	public function override_scss_values($id, $type) {
		$record = $this->themes_model->select('id, scss_variables, scss_values')->find($id);
		switch($type) {
			case 2:
				$result = $this->db->select('name, value')->get_where('theme_settings', array('type' => 'schemes', 'is_default' => 1))->row();
				if($result) {
					// convert to a format accepted by the get_scss_variables method
					$scss_values = json_encode(array($record->id => json_decode($result->value ?? '')));
				} else {
					$scss_values = json_encode(array($record->id => array()));
				}
				break;
			case 0:				
				// convert to a format accepted by the get_scss_variables method
				$scss_values = json_encode(json_decode($record->scss_values ?? ''));
				break;
			default:
				$result = $this->db->select('name, value')->get_where('theme_settings', array('name' => 'global'))->row();
				$variables = json_decode($result->value ?? '');
				$global = array();
				if(!empty($variables)) {
					foreach($variables as $variable) {
						$global[$variable->name] = $variable->value;
					}
				}
				// convert to a format accepted by the get_scss_variables method
				$scss_values = json_encode(array($record->id => $global));
		}
		return $this->get_scss_variables($record, json_decode($scss_values ?? ''));
	}
	
	/**
	 * Return all scss variables of a particular widget
	 * @param object $widget_info
	 * @param array $view_scss_values
	 * @return object
	 */
	public function get_scss_variables($widget_info, $view_scss_values) {
		$widget_variables = array();
		$widget_id = $widget_info->id;
		// if widget exists get all SCSS variables
		//$variables = $this->themes_model->get_scss_variables($widget_info->id);
		$variables = json_decode($widget_info->scss_variables ?? '');
		if(!empty($variables)) {
				
			// The View already has SCSS variable values
			if(!empty($view_scss_values)) {
				// check if this particular widget values are already stored in this view
				if($this->check_array_2d($view_scss_values, $widget_info->id)) {
					// if the widget SCSS values exists, display stored values
					$widget_variables = $this->get_saved_values($variables, $view_scss_values->$widget_id);
				} else {
					// This widget is newly added, show user SCSS variables with default values
					$widget_variables = $this->get_saved_values($variables, json_decode($widget_info->scss_values ?? ''));
				}
			} else {
				// The user never changed the default values, get default values from the widget
				$widget_variables = $this->get_saved_values($variables, json_decode($widget_info->scss_values ?? ''));
			}
		}
		return $widget_variables;
	}
	
	/**
	 * Check whether the key exists in the scss_values array or not
	 * if key exists with the id, return its value
	 *
	 * SCSS_VALUE format - [FIELD ID] => [FIELD VALUE]
	 *
	 * @param array $scss_values - Single Dimensional Array
	 * @param string $key
	 * @return string|boolean
	 */
	private function check_array($scss_values, $key) {
	    $scss_values = (array) $scss_values;
	    if(!empty($scss_values)) {
	        if (array_key_exists($key, $scss_values)) {
	            return $scss_values[$key];
	        }
	    }
	    return false;
	}
	
	/**
	 * Get new saved values of each variable and display it to the user
	 * @param array $scss_variables - All SCSS Variables
	 * @param array $scss_values - Default/New Values Array
	 * @return multitype:
	 */
	public function get_saved_values($scss_variables, $scss_values) {
		$variables = array();
		foreach($scss_variables as $scss_variable) {
			//if(!empty($scss_values)) {
			$value = $this->check_array($scss_values, $scss_variable->name);
			$get_value_by_key = array(
					'type' => $scss_variable->type,
					'name' => $scss_variable->name,
					'title' => $scss_variable->title,
					'description' => $scss_variable->description,
					'value' => $value ? $value : $scss_variable->value
			);
			if(!empty($scss_variable->options)) {
				$get_value_by_key['options'] = $scss_variable->options;
			}
			$variables[] = $get_value_by_key;
			//}
		}
		return $variables;
	}
	
	/**
	 * Set is_default parameter - Themes/Schemes
	 * @param int $name
	 * @param string $type
	 * @param int $is_default
	 */
	public function set_is_default($name, $type = 'themes', $is_default = 0) {
	    $table = 'theme_settings';
	    // set the is_default value
	    $this->db->update($table, array('is_default' => $is_default), array('name' => $name, 'type' => $type));
	    // set the is_default to 0 for other packages
	    if($is_default == 1) {
	        $this->db->where(array('name != ' => $name, 'type' => $type))->update($table, array('is_default' => 0));
	    }
	}
	
	/**
	 * Get Theme Options to set the default cart widget or other options
	 * @param string $key - Name of the Key
	 * @param string $theme - Name of the theme whose options to be retrieved
	 * @return string|array
	 */
	public function get_theme_options($key = null, $theme = null) {
	    $cacheID = 'theme_default';
	    if(!empty($theme)) {
	        $cacheID .= '_' . $theme;
	    }
	    if(!$options = $this->gc_themes_cache->load($cacheID)) {
			$options = array();
    	    if(empty($theme)) {
    	        $record = $this->db->where(array('is_default' => 1, 'type' => 'themes'))->get('theme_settings')->row();
    	    } else {
    	        $record = $this->db->where(array('name' => $theme, 'type' => 'themes'))->get('theme_settings')->row();
    	    }
    	    if($record && !empty($record->value)) {
    	        $value = !empty($record->value) ? json_decode($record->value) : new stdClass();
    	        $options['theme'] = $record->name;
    	        $options['navbar'] = !empty($value->navbar) ? $value->navbar : '';
    	        $options['footer'] = !empty($value->footer) ? $value->footer : '';
    	        if(!empty($value->options)) {	           
    	            $options_array = json_decode($value->options, true);
    	            if(!empty($options_array)) {
    	                foreach($options_array as $option_array) {
    	                    $options[$option_array['name']] = $option_array['value'];
    	                }
    	            }
    	            
    	        }
    	    }
    	    $this->gc_themes_cache->save($options, $cacheID, array('themes', 'default_theme'));
	    }
	    if(!empty($key)) {
	        return !empty($options[$key]) ? $options[$key] : null;
	    } else {
	        return $options;
	    }
	}
	
	/**
	 * Get Widgets/Views/Layouts folders
	 * @param number $type_id
	 */
	public function get_folders($type_id = 0) {
	    $cacheID = 'theme_folders_' . $type_id;
	    if(!$folders = $this->gc_themes_cache->load($cacheID)) {
			$folders = array();
	        $records = $this->db->where('type_id', $type_id)->order_by('priority, folder')->get('theme_folders')->result();
	        if($records) {
	            foreach($records as $record) {
	                $folders[$record->folder] = !empty($record->name_lang) ? lang($record->name) : $record->name;
	            }
	        }
	        $this->gc_themes_cache->save($folders, $cacheID, array('theme_folders'));
	    }
	    return $folders;
	}
	
	/**
	 * Get all themes and display them to the User
	 */
	public function all_themes() {
	    $cacheID = 'all_themes';
	    if(!$themes = $this->gc_themes_cache->load($cacheID)) {
	        $records = $this->db->where('type', 'themes')->get('theme_settings')->result();
	        $themes = array();
	        if($records) {
	            foreach($records as $record) {
	                $themes[$record->name] = $this->_get_theme($record->name);
	            }
	        }
	        $this->gc_themes_cache->save($themes, $cacheID, array('themes'));
	    }
	    return $themes;
	}
	
	/**
	 * Get Theme by name
	 * @param string $theme
	 * @return array
	 */
	private function _get_theme($theme, $select = 'id, image_url') {
	    $sql = "SELECT " . $select;
	    $sql .=" FROM " . $this->db->dbprefix . "themes";
	    $sql .= " WHERE type_id = 0 AND is_theme_default = 1 AND file_folder = 'home' AND FIND_IN_SET('" . $theme . "', theme)";
	    $row = $this->db->query($sql)->row();
	    return $row;
	}
	
	/**
	 * Check whether the key exists in the scss_values array or not
	 * if key exists with the id, return its value
	 *
	 * SCSS_VALUE format - [FIELD ID] => [FIELD VALUE]
	 *
	 * @param array $scss_values - Single Dimensional Array
	 * @param string $key
	 * @return string|boolean
	 */
	private function check_array_2d($scss_values, $widget_id) {
	    foreach($scss_values as $key => $value) {
	        if($key == $widget_id) {
	            return true;
	        }
	    }
	    return false;
	}
}
