<?php defined('BASEPATH') || exit('No direct script access allowed');
/**
 * Bonfire
 *
 * An open source project to allow developers get a jumpstart their development of CodeIgniter applications
 *
 * @package   Bonfire
 * @author    Bonfire Dev Team
 * @copyright Copyright (c) 2011 - 2015, Bonfire Dev Team
 * @license   http://opensource.org/licenses/MIT The MIT License
 * @link      http://cibonfire.com
 * @since     Version 1.0
 * @filesource
 */

/**
 * Translate Module
 *
 * Manages the language files in Bonfire and allows an easy way for the user
 * to add language files for other languages.  The user can export current language
 * files for translation.
 *
 * @package Bonfire\Modules\Translate\Controllers\Developer
 * @author  Bonfire Dev Team
 * @link    http://cibonfire.com/docs
 */
// use Stichoza\GoogleTranslate\GoogleTranslate;

class Developer extends Admin_Controller
{
    /** @var array The names of the current languages. */
    private $langs;

    /**
     * Constructor - Load required classes
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();

        // Restrict access - View and Manage
        $this->auth->restrict('Bonfire.Translate.View');
        $this->auth->restrict('Bonfire.Translate.Manage');

        $this->lang->load('translate');
        $this->load->library('gc');

        $this->load->helper('languages');
        $this->langs = list_languages();

        Assets::add_module_js('translate', 'translate.js');

        Template::set_block('sub_nav', 'developer/_sub_nav');
    }

    /**
     * Display a list of all core language files, as well as a list of modules
     * the user can choose to edit.
     *
     * @param string $transLang The target language for translation.
     *
     * @return void
     */
    public function index($transLang = '')
    {
    	Assets::add_module_css('translate', 'translate.css');
        if (empty($transLang)) {
            $config =& get_config();
            $transLang = isset($config['language']) ? $config['language'] : 'english';
        }

        // Selecting a different language?
        if (isset($_POST['select_lang'])) {
            $this->gcrud->stats(array('module' => 'translate'));
            $transLang = $this->input->post('trans_lang') == 'other' ?
                $this->input->post('new_lang') : $this->input->post('trans_lang');
        }

        // If the selected language is not in the list of languages, add it.
        if (! in_array($transLang, $this->langs)) {
            $this->langs[] = $transLang;
        }

        // Check whether there are custom modules with lang files.
        $allLangFiles = list_lang_files();
        if (isset($allLangFiles['custom'])) {
            $moduleLangFiles = $allLangFiles['custom'];
            sort($moduleLangFiles);
            Template::set('modules', $moduleLangFiles);
        }

        $coreLangFiles = $allLangFiles['core'];
        // remove routes_lang.php from the language list
	    if (in_array('routes_lang.php', $coreLangFiles)) {
	    	unset($coreLangFiles[array_search('routes_lang.php', $coreLangFiles)]);
		}
        sort($coreLangFiles);
        Template::set('lang_files', $coreLangFiles);
        Template::set('languages', $this->langs);
        Template::set('trans_lang', $transLang);
        Template::set('toolbar_title', sprintf(lang('translate_title_index'), ucfirst($transLang)));
        Template::render();
    }

    /**
     * Allow the user to edit a language file.
     *
     * @param string $transLang The target language for translation.
     * @param string $langFile  The file to translate.
     *
     * @return void
     */
    public function edit($transLang = '', $langFile = '') {
    	Assets::add_module_css('translate', 'translate.css');
        $config =& get_config();
        $origLang = isset($config['language']) ? $config['language'] : 'english';
        $chkd = array();
        $this->load->model('languages/languages_model');

        if ($langFile) {
            // Save the file...
            if (isset($_POST['save'])) {
                gc_demo_mode();
                $this->gcrud->stats(array('module' => 'translate'));
                $transLang = strtolower($transLang);
                // If the file saves successfully, redirect to the index.
                if (save_lang_file($langFile, $transLang, $_POST['lang'])) {
                    Template::set_message(lang('translate_save_success'), 'success');
                    redirect(SITE_AREA . "/developer/translate/index/{$transLang}");
                }

                Template::set_message(lang('translate_save_fail'), 'danger');
            }
            
            // Translate using Google...
            if (isset($_POST['google'])) {
                gc_demo_mode();
                $this->gcrud->stats(array('module' => 'translate'));
            	$source = get_language_code('english');
            	$target = get_language_code($transLang);
            	if($source && $target) { // Language code should be valid
            		$english_lang = load_lang_file($langFile, 'english'); // Original Language
            		if(!empty($english_lang)) {
            			$translated_text = $this->translate_text($source, $target, $english_lang);
            			if (save_lang_file($langFile, $transLang, $translated_text)) {
            				Template::set_message(lang('translate_save_success'), 'success');
            				redirect(SITE_AREA .'/developer/translate/edit/'. $transLang .'/' .$langFile);
            			} else {
            				Template::set_message(lang('translate_save_fail'), 'danger');
            			}
            		}
            		/*$tra = load_lang_file($lang_file, $trans_lang);
            		 $new_values = array_diff_key($english_lang, $tra);
            		if(!empty($new_values)) {
            		$translated_text = $this->translate_text($source, $target, $new_values);
            		if (save_lang_file($lang_file, $trans_lang, $translated_text)) {
            		Template::set_message(lang('tr_save_success'), 'success');
            		redirect(SITE_AREA .'/developer/translate/edit/'. $trans_lang .'/' .$lang_file);
            		} else {
            		Template::set_message(lang('tr_save_fail'), 'danger');
            		}
            		}*/
            
            	}
            }

            // Get the lang file.
            $orig = load_lang_file($langFile, $origLang);
            $new = $transLang ? load_lang_file($langFile, $transLang) : $orig;

            // Translate.
            if (isset($_POST['translate']) && !empty($_POST['checked']) && is_array($_POST['checked'])) {
                gc_demo_mode();
                $this->gcrud->stats(array('module' => 'translate'));
                if (isset($_POST['lang'])) {
                    foreach ($_POST['lang'] as $key => $val) {
                        $new[$key] = $val;
                    }
                }

                if ($transLang) {
                    $codes = $this->translations->get_languages(false);
                    $transLangCode = isset($codes[$transLang]) ? $codes[$transLang] : '';
                    $origLangCode  = isset($codes[$origLang]) ? $codes[$origLang] : 'en';
                    $language_name = $transLang;
                    $errcnt = 0;
                    $cnt = 0;
                    $this->load->library('translate_lib');
                    
                    // unset the google translate session variable
                    // $this->session->unset_userdata('stichoza');
                    
                    // Attempt to translate each of the selected values.
                    foreach ($_POST['checked'] as $key) {
                        $new[$key] = $new[$key];
                        $val = '';
                        if ($transLangCode) {                            
                            /*$google_translated = $this->languages_model->file_translate($langFile, $language_name, $transLangCode);
                            if(!empty($google_translated[$language_name][$langFile])) {
                                if(array_key_exists($key, $google_translated[$language_name][$langFile])) {
                                    $val = $google_translated[$language_name][$langFile][$key];
                                }
                            } else {
                                $val = $this->translate_lib->setLang($origLangCode, $transLangCode)->translate($orig[$key]);
                            }*/
                            $val = $this->translate_lib->setLang($origLangCode, $transLangCode)->translate($orig[$key]);
                        }
                        if ($val) {
                            $new[$key] = $val;
                            $cnt++;
                        } else {
                            $errcnt++;
                            $chkd[] = $key;
                        }

                        // Give up if 5 errors occur without any successful attempts,
                        // as this may imply network or other issues.
                        if ($errcnt >= 5 && $cnt == 0) {
                            break;
                        }
                    }

                    if ($errcnt == 0) {
                        Template::set_message(sprintf(lang('translate_success'), $cnt), 'success');
                    } elseif ($cnt > 0) {
                        Template::set_message(sprintf(lang('translate_part_success'), $cnt, $errcnt), 'info');
                    } else {
                        Template::set_message(sprintf(lang('translate_failed'), PHP_VERSION), 'danger');
                    }
                }
            }

            Template::set('orig', $orig);
            Template::set('new', $new);
            Template::set('lang_file', $langFile);
        }

        Template::set('chkd', $chkd);
        Template::set('orig_lang', $origLang);
        Template::set('trans_lang', $transLang);
        Template::set('toolbar_title', sprintf(lang('translate_title_edit'), $langFile, ucfirst($transLang)));
        Template::render();
    }

    /**
     * Export a set of files for a language
     *
     * @return void
     */
    private function export_old() {
    	Assets::add_module_css('translate', 'translate.css');
        if (isset($_POST['export'])) {
            $this->do_export($this->input->post('export_lang'), $this->input->post('include_core'), $this->input->post('include_custom'));
            die();
        }

        Template::set('languages', $this->langs);
        Template::set('toolbar_title', lang('translate_export'));
        Template::render();
    }

    /**
     * Retrieve all files for a language, zip them, and send the zip file to the
     * browser for immediate download
     *
     * @param string $language      The language for which to retrieve the files
     * @param bool $includeCore     Include the core language files
     * @param bool $includeCustom   Include the custom module language files
     *
     * @return mixed false on error or void
     */
    public function do_export($language = null, $includeCore = false, $includeCustom = false) {
    	Assets::add_module_css('translate', 'translate.css');
        if (empty($language)) {
            $this->error = 'No language file chosen.';
            return false;
        }

        $all_lang_files = list_lang_files($language);
        if (! count($all_lang_files)) {
            $this->error = 'No files found to archive.';
            return false;
        }

        // Make the zip file
        $this->load->library('zip');
        foreach ($all_lang_files as $key => $file) {
            if (is_numeric($key) && $includeCore) {
                $content = load_lang_file($file, $language);
                $this->zip->add_data($file, save_lang_file($file, $language, $content, true));
            } elseif (($key == 'core' && $includeCore)
                      || ($key == 'custom' && $includeCustom)
            ) {
                foreach ($file as $f) {
                    $content = load_lang_file($f, $language);
                    $this->zip->add_data($f, save_lang_file($f, $language, $content, true));
                }
            }
        }

        $this->zip->download("bonfire_{$language}_files.zip");
        die();
    }
    
    /**
     * Translate text using Online Google Translator
     * @param string $source
     * @param string $target
     * @param string $english_lang
     * @return multitype:array
     */
    private function translate_text($source, $target, $english_lang) {
        Assets::add_module_css('translate', 'translate.css');
    	// Get keys from associate array for further merge
    	$keys = array();
    	foreach($english_lang as $k => $v) {
    		$keys[] = $k;
    	}
    	
    	// do not unset the google translate session variable
    	// $this->session->unset_userdata('stichoza');
    	
    	// Convert Array From Associative to Indexed as API uses indexed array
    	$text_to_be_translated = array();
    	foreach($english_lang as $eng) {
    		$text_to_be_translated[] = $eng;
    	}
    
    	$flip_keys = array_flip($keys); // Flip array to interchage values and keys
    
    	// Translate text and get result in the form of indexed array
    	$val = $this->translate_lib->setLang($source, $target)->translate($text_to_be_translated);
    	
    	// Flip array to interchange values and keys
    	$final = array_flip($result);
    
    	$lang = array();
    	foreach($final as $key => $value) {
    		if(array_search($value, $flip_keys)) { // Search value in flipped keys array
    			$lang[$keys[$value]]= $key; // Get key from $keys array
    		}
    	}
    	return $lang;
    }
    
    /**
     * Easy Translation to a Single Language
     */
    public function easy_translate() {
        Assets::add_module_css('translate', 'translate.css');
    	$config =& get_config();
    	$this->load->library('translate_lib');
    	$this->load->model('languages/languages_model');
    	$origLang = 'english';
    	$origLangCode = 'en';
    	// Load all language codes from the config file
    	$codes = $this->translations->get_languages(false);
    	// Save the file...
    	if (isset($_POST['save'])) {
    	    gc_demo_mode();
    	    $this->gcrud->stats(array('module' => 'translate'));
    		$this->form_validation->set_rules('transLang', 'lang:translate_translate', 'trim|required');
    		//$this->form_validation->set_rules('checked', 'lang:label_title', 'trim|is_array|required');
    		if ($this->form_validation->run() === true) {
    		    $transLang = $this->input->post('transLang');
    			// If the selected language is not in the list of languages, add it.
    			if (! in_array($transLang, $this->langs)) {
    				$this->langs[] = $transLang;
    			}
    			$language_name = $transLang;
    			$transLangCode = isset($codes[$transLang]) ? $codes[$transLang] : '';
    			$translatedFiles = array();
    			if(isset($_POST['checked']) && is_array($_POST['checked'])) {
    				$langFiles = $_POST['checked'];
    				foreach($langFiles as $langFile) {
    					// Load the original language file
    					$orig = load_lang_file($langFile, $origLang);
    
    					// Load the new language file
    					$new = load_lang_file($langFile, $transLang);
    
    					// Language file does not exists, assign original language
    					if(!$new) {
    						$new = $orig;
    					}
    
    					// Start the translation
    					$errcnt = 0;
    					$cnt = 0;
    					
    					// unset the google translate session variable
    					// $this->session->unset_userdata('stichoza');
    					
    					// Attempt to translate each of the selected values.
    					foreach ($orig as $key => $value) {
    					    if(isset($new[$key])) {
    						    $new[$key] = '* ' . $new[$key];
    					    }
    						$val = '';
    						if ($transLangCode) {
    						    /*$google_translated = $this->languages_model->file_translate($langFile, $language_name, $transLangCode);
    						    if(!empty($google_translated[$language_name][$langFile])) {
    						        if(array_key_exists(strtolower($key), $google_translated[$language_name][$langFile])) {
    						            $val = $google_translated[$language_name][$langFile][$key];
    						        }
    						    } else {
    						        $val = $this->translate_lib->setLang($origLangCode, $transLangCode)->translate($orig[$key]);
    						    }*/
    						    $val = $this->translate_lib->setLang($origLangCode, $transLangCode)->translate($orig[$key]);
    						}
    						if ($val) {
    							$new[$key] = $val;
    							$cnt++;
    						} else {
    							$errcnt++;
    							$chkd[] = $key;
    						}
    
    						// Give up if 5 errors occur without any successful attempts,
    						// as this may imply network or other issues.
    						if ($errcnt >= 5 && $cnt == 0) {
    							break;
    						}
    					}    					
    					
    					// Save language file
    					$transLang = strtolower($transLang);
    					if(save_lang_file($langFile, $transLang, $new)) {
    					    // Display converted language file names to the user
    					    $translatedFiles = $langFile;
    					}
    				}
    				if(!empty($translatedFiles)) {
    					Template::set_message(lang('translate_save_success'), 'success');
    				} else {
    					Template::set_message(lang('translate_save_fail'), 'danger');
    				}
    			} else {
    				Template::set_message(lang('msg_no_row_selected'), 'danger');
    			}
    		}
    	}
    
    	// Check whether there are custom modules with lang files.
    	$allLangFiles = list_lang_files();
    	$coreLangFiles = $allLangFiles['core'];
    	// remove routes_lang.php from the language list
    	if (in_array('routes_lang.php', $coreLangFiles)) {
    		unset($coreLangFiles[array_search('routes_lang.php', $coreLangFiles)]);
    	}
    	sort($coreLangFiles);
    	if (isset($allLangFiles['custom'])) {
    		$moduleLangFiles = $allLangFiles['custom'];
    		sort($moduleLangFiles);
    		$coreLangFiles = array_merge($coreLangFiles, $moduleLangFiles);
    	}
    	Template::set(array(
    		'codes' => $codes,
    		'records' => $coreLangFiles,
    		'toolbar_title' => sprintf(lang('single_click_translate'), ucfirst($origLang))
    	));
    	Template::render();
    }
    
    /**
     * Find and Replace Text in the Language Files
     * @return boolean
     */
    public function find_replace() {
    	$config =& get_config();
    	$codes = $this->translations->get_languages(false);
    	if (isset($_POST['find']) || isset($_POST['find_replace'])) {
    	    gc_demo_mode();
    	    $this->gcrud->stats(array('module' => 'translate'));
    		$this->form_validation->set_rules('find_text', 'lang:find_text', 'trim|required');
    		$this->form_validation->set_rules('search_language', 'lang:search_language', 'trim|required');
    		if(isset($_POST['find_replace'])) {
    			$this->form_validation->set_rules('replace_text', 'lang:replace_text', 'trim|required');
    		}
    		if ($this->form_validation->run() === true) {
	    		$find = $this->input->post('find_text');
	    		$replace = isset($_POST['find_replace']) ? true : false;
	    		$replace_text = $this->input->post('replace_text');
	    		$search_language = $this->input->post('search_language');
		    	$languages = list_lang_files();
		    	$not_found = true;
		    	foreach($languages as $modules) {
		    		foreach($modules as $language) {
			    		$lang_array = load_lang_file($language, $search_language);
			    		// search in array and return the key
			    		if($lang_array) {
			    		    foreach ($lang_array as $key => $val) {
			    		        $lang_array[$key] = strtolower($val);
			    		    }
				    		$key = array_search ($find, $lang_array);
				    		if($key) {
				    			// search and replace text
				    			if($replace) {
				    				$lang_array[$key] = $replace_text;
				    				save_lang_file($language, $search_language, $lang_array);
				    				Template::set_message(lang('replace_success'), 'success');				    				
				    			} else {
				    				Template::set_message(sprintf(lang('find_success'), $language, $key), 'success');
				    			}
				    			$not_found = false;
				    			break;
				    		}
			    		}
		    		}
		    	}
		    	if($not_found) {
	    			// No record found
	    			Template::set_message(lang('label_no_record_found'), 'danger');
		    	}
    		}
    	}
    	Template::set(array(
    		'codes' => $codes,
    		'toolbar_title' => lang('find_replace_text')
    	));
    	Template::render();
    }
    
    /**
     * Translate Routes
     * The routes_lang.php file is loaded in the Base_controller.php and other libraries
     */
    public function routes() {    	
    	$config =& get_config();
    	$lang_file = 'routes_lang.php';
    	$this->load->config('langcodes');
    	$this->load->model('settings/settings_model');
    	$this->load->library('slug');
    	if (isset($_POST['save'])) {
    	    gc_demo_mode();
    	    $this->gcrud->stats(array('module' => 'translate'));
    		$this->form_validation->set_rules('route[]', 'lang:label_routes', 'trim|callback_slugify_route|required|is_array|callback_unique_check');
    		if($this->form_validation->run() == TRUE) {
    			$contents = $this->input->post('route');
    			save_lang_file($lang_file, 'english', $contents);
    			$this->update_routes($contents);
    			Template::set_message(lang('routes_save_success'), 'success');
    		}
    	}
    	if(isset($_POST['reset'])) {
    	    gc_demo_mode();
    	    $this->gcrud->stats(array('module' => 'translate'));
    	    $contents = $this->backup_routes();// load_lang_file($lang_file, 'english');
    	    if(!empty($contents)) {
    	        save_lang_file($lang_file, 'english', $contents);
    	        $this->update_routes($contents);
    	        Template::set_message(lang('routes_save_success'), 'success');
    	        Template::redirect(SITE_AREA . "/developer/translate/routes");
    	    } else {
    	        Template::set_message(lang('translate_save_fail'), 'danger');
    	        Template::redirect(current_url());
    	    }
    	}
    	
    	$routes = load_lang_file($lang_file, 'english');    	
    	$codes = $this->translations->get_languages(false);
    	// Translate using Goolge Translate
    	if(isset($_POST['translate'])) {
    	    gc_demo_mode();
    	    $this->gcrud->stats(array('module' => 'translate'));
    		$this->load->library('translate_lib');
    		$transLang = $this->input->post('transLang');
    		$transLangCode = isset($codes[$transLang]) ? $codes[$transLang] : 'en';
    		$contents = array();
    		$counts = 0;
    		
    		// unset the google translate session variable
    		// $this->session->unset_userdata('stichoza');
    		
    		// Translate only if the language is not english
    		foreach($routes as $key => $route) {
    			$text = str_replace('-', ' ', $route);
    			$val = $this->translate_lib->setLang('en', $transLangCode)->translate($text);
    			if($val) {
    				$contents[$key] = $this->slugify_route($val);
    			} else {
    				$counts++;
    				$contents[$key] = $route;
    			}    			
	    	}
	    	
	    	// check unique values
	    	foreach($contents as $rkey => $str) {
		    	if (count(array_keys($contents, $str)) > 1) {
		    		// add random value with string
		    		$contents[$rkey] = $str . rand(1,999);
		    	}
	    	}	    	
	    	if($counts > 5) {
	    		Template::set_message(sprintf(lang('translate_failed'), PHP_VERSION), 'danger');
	    	} else {
	    		save_lang_file($lang_file, 'english', $contents);
	    		$this->update_routes($contents);
	    		Template::set_message(lang('routes_save_success'), 'success');
	    	}	    	
    		Template::redirect(SITE_AREA . "/developer/translate/routes");
    	}
    	Template::set('codes', $codes);    	
    	Template::set('routes', $routes);
    	Template::set('toolbar_title', lang('manage_routes'));
    	Template::render();
    }    
    
    /**
     * Get stored string from the backup file
     * @return string[]|array
     */
    private function backup_routes() {
        $path = APPPATH . 'db/backups/routes_lang.php'; // Path of the backup file
        if(file_exists($path)) {
            $lang_array = array();
            $lines = file($path);
            if($lines) {
                foreach($lines as $line) {
                    if (strpos($line, '$lang[') !== false) {
                        // get file name
                        $key = trim(getStringBetween($line, '$lang[\'','\']'));
                        $val = trim(getStringBetween($line, '= \'','\';'));
                        if(!empty($key) && !empty($val)) {
                            $lang_array[$key] = $val;
                        }
                    }
                }
            }
            return $lang_array;
        }
        return array();
    }

    /**
     * All routes must have unique values validation 
     * @param string $str
     * @return boolean
     */
    public function unique_check($str) {
    	if (count(array_keys($this->input->post('route'), $str)) > 1) {
    		$this->form_validation->set_message('unique_check', sprintf(lang('unique_check'), $str));
    		return FALSE;
    	} else {
    		return TRUE;    		
    	}
    }
    
    /**
     * Slugify each value before save
     * @param string $str
     */
    public function slugify_route($str) {
    	return str_replace('-', '_', $this->slug->create_slug($str));
    }
    
	/**
	 * Update Website Routes from site.routes
	 * 
	 * @param array $routes
	 */
    private function update_routes($routes) {
    	// get routes from the settings
    	//$routes = (array) json_decode(settings_item('site.routes'));
    	//$routes = load_lang_file($lang_file, 'english');
    	// get contents from the backup file
    	$file = realpath(APPPATH . 'db/config/routes.php');
    	// after replacement overwrite the route file
    	$route_file = realpath(APPPATH . 'config/routes.php');
    	if (file_exists($file)) {
	    	$lines = file($file);
	    	if($lines) {
	    		// read line one by one
	    		foreach ($lines as $lineNumber => $line) {
	    			$new_line = $this->update_line($lineNumber, $line, $routes);
	    			if($new_line) {
	    				$new_lines[$lineNumber] = $new_line;
	    			} else {
	    				$new_lines[$lineNumber] = $line;
	    			}
	    		}
	    		// Write the changes out...
	    		if (! function_exists('write_file')) {
	    			$this->load->helper('file');
	    		}
	    		$contents = implode('', $new_lines);
	    		write_file($route_file, $contents);
	    	}
    	}
    }
    
    /**
     * Update Line Text
     * @param int $lineNumber
     * @param string $line
     * @param array $routes
     * @return mixed|boolean
     */
    private function update_line($lineNumber, $line, $routes) {
    	// specify line types
    	// get all line starting with specified line types
    	$line_types = array('Route::any(\'' => '\'', '$route[\'' => '\']');
    	foreach($line_types as $start => $end) {
	    	if (strpos($line, $start) !== false) {
	    		// get content between two strings
	    		$getstring = getStringBetween(trim($line), $start, $end, false);
	    		// replace result with the new value
	    		// convert string into array	
	    		$strs = explode('/', $getstring ?? '');
	    		foreach($strs as $key => $value) {
	    			// check whether the route value exists for this value or not
	    			// e.g. module name & detail both exists in the routes
	    			if(array_key_exists('route_' . $value, $routes)) {
	    				$strs[$key] = $routes['route_' .$value];
	    			}
	    		}
	    		// convert the replaced array into string
	    		$replacement = implode('/',$strs);
	    		// replace the line content
	    		return $this->replace_between($line, $start, $end, $replacement);
	    	}	    	
    	}
    	return false;
    }
    
    /**
     * Replace string between two tags within a line
     *
     * @param string $str - Complete Line
     * @param string $needle_start - Start Tag
     * @param string $needle_end - End Tag
     * @param string $replacement - Text to be Replaced
     * @return mixed
     */
    private function replace_between($str, $needle_start, $needle_end, $replacement) {
    	$pos = strpos($str, $needle_start);
    	$start = $pos === false ? 0 : $pos + strlen($needle_start);
    
    	$pos = strpos($str, $needle_end, $start);
    	$end = $pos === false ? strlen($str) : $pos;
    	return substr_replace($str, $replacement, $start, $end - $start);
    }
    
    /**
     * Backup Language Files inside the public_html/uploads folder
     * A ziparchieve will be created with the language name e.g. english.zip
     */
    public function export() {
        if (isset($_POST['export'])) {
            $this->gcrud->stats(array('module' => 'translate'));
            $language = $this->input->post('export_lang');
            $include_core = $this->input->post('include_core') ? true : false;
            $include_custom = $this->input->post('include_mods') ? true : false;
            $success = $this->perform_export($language, $include_core, $include_custom); 
            if(!$success) {                
                $message = lang('edit_failure');
                if($include_core) {
                    $message .= '<br />' . lang('translate_no_core');
                }
                if($include_custom) {
                    $message .= '<br />' . lang('translate_no_modules');
                }
                Template::set_message($message, 'danger');
            }
        }
        Assets::add_module_css('translate', 'translate.css');
        Template::set('languages', $this->langs);
        Template::set('toolbar_title', lang('translate_export'));
        Template::set_view('export');
        Template::render();
    }
    
    /**
     * Perform Export Opertion
     * @param string $language
     * @param boolean $include_core
     * @param boolean $include_custom
     * @param boolean $force_download
     * @return boolean
     */
    private function perform_export($language, $include_core, $include_custom, $force_download = true) {
        $zip_folder = FCPATH . 'uploads/';
        $destination_folder = $zip_folder . $language . '/';
        $zip_file = $zip_folder . $language . '.zip';
        // Unline Existing Zip File if exist
        if(file_exists($zip_file)) {
            unlink(realpath($zip_file));
        }
        $this->load->helper('languages');
        $this->load->model('cronjobs/cronjob_model');
        $lang_files = list_lang_files($language);
        $language_files = array();
        if(!empty($lang_files)) {
            foreach($lang_files as $key => $lang_file) {
                if($key == 'core' && $include_core) {
                    foreach($lang_file as $core) {
                        $file_path = 'language/' . $language . '/' . $core;
                        $language_files[] = array(
                            'source' => APPPATH . $file_path,
                            'destination' => 'application/' . $file_path
                        );
                    }
                } else if($key == 'custom' && $include_custom) {
                    foreach($lang_file as $custom) {
                        // get module name
                        $module = str_replace('_lang.php', '', $custom);
                        $module_path = APPPATH;
                        $module_folder = 'application/';
                        $file_path = 'modules/' . $module . '/language/' . $language . '/' .  $custom;
                        if(!file_exists(APPPATH . $file_path)) {
                            $module_path = BFPATH;
                            $module_folder = 'goclixy/';
                        }
                        $language_files[] = array(
                            'source' => $module_path . $file_path,
                            'destination' => $module_folder . $file_path
                        );
                    }
                }
            }
            // Copy Files
            if(!empty($language_files)) {
                foreach($language_files as $language_file) {
                    $this->cronjob_model->file_copy($language_file['source'], $destination_folder . $language_file['destination']);
                }
                // create zip archieve of language folder inside the public_html/uploads folder
                create_zip($zip_folder . $language, $zip_file);
                if($force_download) {
                    // Force Zip File Download
                    header("Content-Description: File Transfer");
                    header("Content-Type: application/zip");
                    header("Content-Disposition: attachment; filename=" . basename($language . '.zip') . "");
                    readfile ($zip_file);
                    exit();
                }
            } else {
                return false;
            }
        }
    }
    
    /**
     * Restore language files from Local/GoClixy Server
     */
    public function import() {
        if (isset($_POST['import'])) {
            gc_demo_mode();
            $this->gcrud->stats(array('module' => 'translate'));
            $this->form_validation->set_rules('import_from', 'lang:translate_import_from', 'trim|required');
            if($this->input->post('import_from') == 0) {
                $this->form_validation->set_rules('import_file_uploaded_file', 'lang:label_file', 'trim|required');
                // Check whether a file is uploaded or not
                if(isset($_FILES['import_file']) && $_FILES['import_file']['size'] > 0) {
                    $settings = array(
                        'allowed_types' => 'zip',
                        'max_size' => 10240,
                        'upload_path' => realpath(FCPATH . '/uploads/')
                    );
                    $file_data = $this->gc->upload_file('import_file', $settings);
                    if(!$file_data) {
                        $_POST['import_file_upload'] = 'NONE';
                        Template::set_message($this->session->flashdata('upload_file_error'), 'danger');
                        //$this->form_validation->set_rules('import_file_upload', 'label:label_file', 'valid_file_upload');
                    }
                    // Avoid file upload process, if the form has validation issues with other fields
                    $_POST['import_file_uploaded_file'] = $file_data && isset($file_data['upload_data']['file_name']) ? $file_data['upload_data']['file_name'] : '';
                }
            }
            if ($this->form_validation->run() === TRUE) {
                $this->load->model('cronjobs/cronjob_model');
                $language = $this->input->post('import_lang');
                $language = strtolower($language);
                $file_name = $language . '.zip';
                if($this->input->post('import_from') == 0) {
                    $file_url = site_url('uploads/' . $_POST['import_file_uploaded_file']);
                } else {
                    $file_url = 'https://www.ecomsta.com/updates/languages/v' . settings_item('site.version') . '/' . $file_name;
                }
                $zipFile = FCPATH . 'uploads/' . $file_name;
                $this->cronjob_model->update_files($zipFile, $file_url, $language, false);
            }
        }        
        Assets::add_module_css('translate', 'translate.css');
        Template::set('languages', $this->translations->get_languages());
        Template::set('toolbar_title', lang('translate_import'));
        Template::set_view('import');
        Template::render();
    }
    
    /**
     * Export All Languages
     * Written specifically to export all language files for GoClixy Demos
     */
    public function export_all() {
        foreach($this->translations->get_languages() as $language) {
            $this->perform_export($language, true, true, false);
        }
    }
    
    /**
     * Import All Language Files
     * Written specifically to import all language files into eComsta Demos
     */
    public function import_all() {
        $this->load->model('cronjobs/cronjob_model');
        $languages = $this->translations->get_languages();
        foreach($languages as $language) {
            if($language != 'english') { // Do not overwrite english language
                $file_name = $language . '.zip';
                $file_url = 'https://www.ecomsta.com/updates/languages/' . $file_name;
                $zipFile = FCPATH . 'uploads/' . $file_name;
                $this->cronjob_model->update_files($zipFile, $file_url, $language, false);
            }
        }
    }
    
    /**
     * Replace encrypted language files with non-encrypted files
     * Run this method after encrypting files
     */
    public function copy_language_files() {
        $php_versions = array('5.6', '7.1');
        foreach($php_versions as $php_version) {
            $destination_path = 'E:/xampp/htdocs/encryption/encrypted/' . $php_version; // Destination Path
            $languages = $this->langs;
            $language_files = list_lang_files();
            foreach($language_files['custom'] as $language_file) {
                $module = str_replace('_lang.php', '', $language_file);
                foreach($languages as $language) {
                    $source = APPPATH . 'modules/' . $module . '/language/' . $language . '/' . $language_file;
                    $destination = $destination_path . '/modules/' . $module . '/language/' . $language . '/' . $language_file;
                    $path = pathinfo($destination);
                    if (!file_exists($path['dirname'])) {
                        mkdir($path['dirname'], 0777, true);
                    }
                    if(file_exists($source)) {
                        if (!copy($source,$destination)) {
                            echo "copy failed \n";
                        }
                    }
                }
            }
        }
    }
    
    /**
     * Remove Specific language Translations
     * @param string $language - Name of the language file to be removed
     */
    public function remove_translations($language = 'portuguese') {
        // $this->load->helper('languages');
        $lang_files = list_lang_files($language);
        if($lang_files) {
            if(!empty($lang_files['core'])) {
                // Remove core files
                $this->rrmdir(APPPATH . 'language/' . $language);
            }
            if(!empty($lang_files['custom'])) {
                $modules = $lang_files['custom'];
                if($modules) {
                    foreach($modules as $module) {
                        $module_name = str_replace('_lang.php', '', $module);
                        $module_path = APPPATH . 'modules/' . $module_name . '/language/' . $language;
                        if(is_dir($module_path)) {
                            // Remove module files
                            $this->rrmdir($module_path);
                        }
                    }
                }
            }
        }
    }
    
    /**
     * Deletes a directory and all files and folders under it
     * @return Null
     * @param $dir String Directory Path
     */
    private function rrmdir($dir) {
        if (is_dir($dir)) {
            $objects = scandir($dir);
            foreach ($objects as $object) {
                if ($object != "." && $object != "..") {
                    if (filetype($dir."/".$object) == "dir")
                        rrmdir($dir."/".$object);
                        else unlink   ($dir."/".$object);
                }
            }
            reset($objects);
            rmdir($dir);
        }
    }
    
    /**
     * Upload all new added strings to the GoClixy API server for translations
     * @param string $target
     */
    public function bulk_translate($target = 'fr') {
        $this->load->library('translate_lib');
        $this->translate_lib->bulk_translate($target);
    }
    
    /**
     * Add new string to the custom_lang.php file
     */
    public function new_string() {
        $config =& get_config();
        $codes = $this->translations->get_languages(false);
        if (isset($_POST['save'])) {
            gc_demo_mode();
            $this->form_validation->set_rules('language', 'lang:translate_language', 'trim|required');
            $this->form_validation->set_rules('key', 'lang:translate_key', 'trim|required|alpha_key');
            $this->form_validation->set_rules('value', 'lang:label_value', 'trim|required');
            if ($this->form_validation->run() === true) {
                $this->load->model('languages/languages_model');
                $key = str_replace('-', '_', $this->slug->create_slug($this->input->post('key')));
                $value = $this->input->post('value');
                $language = $this->input->post('language');
                $result = $this->languages_model->insert_lang_string($key, $value);
                if(empty($result)) {
                    // success
                    Template::set_message(lang('translate_string_success'), 'success');
                    // Template::redirect(current_url());
                } else {
                    // failure
                    Template::set_message(lang('translate_string_error'), 'danger');
                }
            }
        }        
        Template::set(array(
            'codes' => $codes,
            'toolbar_title' => lang('translate_new_string')
        ));
        Template::render();
    }

    /**
     * Restore language files from the existing backup
     * Note: the backup folder must exist in the public_html folder with the name backup
     */
    public function restore() {
		// restore language files under the application/modules and goclixy/modules folders
		$folders = array('application', 'goclixy');
		foreach($folders as $folder) {
			$backup_folder = FCPATH . 'backup/' . $folder . '/modules'; // backup folder is in the public_html folder
			$directories = glob($backup_folder . '/*' , GLOB_ONLYDIR); // get language files
			if($directories) {
				foreach($directories as $directory) {
					$languages = glob($directory . '/language/*' , GLOB_ONLYDIR);
					foreach($languages as $language) {
						$language_files = glob($language . '/*');
						if(!empty($language_files)) {
							foreach($language_files as $language_file) {
								if(strpos($language_file, 'language/english/') === false) { // do not copy english language files
									$module_language_array = explode($folder . '/modules/', $language_file);
									$destination_file = APPPATH . 'modules/' . $module_language_array[1];								
									if(!file_exists($destination_file)) { // create folder first if it does not exist
										$destination_folder = substr($destination_file, 0, strrpos($destination_file, "/"));										
										mkdir($destination_folder, 0755, true); // in case of amazon or another server, use "chmod -R 777 application" to give permission to the user
									}
									copy($language_file, $destination_file);
								}
							}
						}
					}
				}
			}
		}
		
		// restore the language files under the application/language folder
		$directory = FCPATH . 'backup/application';
		$languages = glob($directory . '/language/*' , GLOB_ONLYDIR);
		foreach($languages as $language) {
			$language_files = glob($language . '/*');
			if(!empty($language_files)) {
				foreach($language_files as $language_file) {
					if(strpos($language_file, 'language/english/') === false) { // do not copy english language files
						$module_language_array = explode('application/', $language_file);
						$destination_file = APPPATH . $module_language_array[1];
						if(!file_exists($destination_file)) { // create folder first if it does not exist
							$destination_folder = substr($destination_file, 0, strrpos($destination_file, "/"));
							if(!is_dir( $destination_folder )) {
								mkdir($destination_folder, 0755, true); // in case of amazon or another server, use "chmod -R 777 application" to give permission to the user
							}
						}
						copy($language_file, $destination_file);
					}
				}
			}
		}
	}
}