<?php defined('BASEPATH') || exit('No direct script access allowed');
/**
 * Bonfire
 *
 * An open source project to allow developers to jumpstart their development of
 * CodeIgniter applications.
 *
 * @package   Bonfire
 * @author    Bonfire Dev Team
 * @copyright Copyright (c) 2011 - 2014, Bonfire Dev Team
 * @license   http://opensource.org/licenses/MIT
 * @link      http://cibonfire.com
 * @since     Version 1.0
 * @filesource
 */

/**
 * Users Settings Controller.
 *
 * Manages the user functionality on the admin pages.
 *
 * @package    Bonfire\Modules\Users\Controllers\Settings
 * @author     Bonfire Dev Team
 * @link       http://cibonfire.com/docs/developer/roles_and_permissions
 *
 */
class Settings extends Admin_Controller
{
    private $siteSettings;

    private $permissionCreate = 'Bonfire.Users.Add';
    private $permissionManage = 'Bonfire.Users.Manage';
    private $permissionView   = 'Bonfire.Users.View';

    /**
     * Setup the required permissions.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();

        $this->auth->restrict($this->permissionView);

        $this->lang->load('users');
        $this->load->model('roles/role_model');

        $this->siteSettings = $this->settings_lib->find_all();
        if ($this->siteSettings['auth.password_show_labels'] == 1) {
            Assets::add_module_js('users', 'password_strength.js');
            Assets::add_module_js('users', 'jquery.strength.js');
        }

        Template::set_block('sub_nav', 'users/settings/_sub_nav');
    }
    
    //--------------------------------------------------------------------
    
    /**
     * Display Orders Summary to the User
     */
    public function summary() {
        $this->auth->restrict('Users.Settings.Summary');
        // DATE TIME PICKER
        Assets::add_js('moment.js');
        Assets::add_css('bootstrap-datetimepicker.min.css' );
        Assets::add_js('bootstrap-datetimepicker.min.js' );
        Assets::add_css('chart.css');
        Assets::add_js('Chart.min.js');
        $this->session->unset_userdata('chart_query_params');
        $record = $this->user_model->summary();
        $tabular_blocks = array();
        $column = 'users_total';
        $table = 'users';
        $where = '';
        $users_by_months = $this->gcrud->chart('months', array('active' => 1, 'where' => $where,'prefix' => 'M', 'table' => $table, 'field' => 'counter'));
        Assets::add_js($this->load->view('common/_chart_js', array('field_id' => 'sixMonthChart', 'labels' => $users_by_months['labels'], 'data' => $users_by_months['data'], 'total' => $users_by_months['total'], 'label' => lang('us_total_users'), 'with_dates' => false, 'table' => $table), true), 'inline');
        $result = $this->gcrud->chart('current_month', array('active' => 1, 'where' => $where, 'prefix' => 'M', 'table' => $table, 'field' => 'counter'));
        Assets::add_js($this->load->view('common/_chart_js', array('field_id' => 'monthlyChart', 'labels' => $result['labels'], 'data' => $result['data'], 'total' => $result['total'], 'label' => lang('us_total_users'), 'with_dates' => true, 'table' => $table), true), 'inline');
        $record->tabular_blocks = $tabular_blocks;
        Template::set(array(
            'toolbar_title' => lang('gc_menu_users') . ' - ' . lang('gc_menu_summary'),
            'record' => $record
        ));
        Template::render();
    }

    /*
     * Display the user list and manage the user deletions/banning/purge.
     *
     * @param string $filter The filter to apply to the list.
     * @param int    $offset The offset from which the list will start.
     *
     * @return  void
     */
    public function index($filter = 'all', $offset = 0)
    {
    	// Status Change
        if (isset($_POST['status_change'])) {
            gc_demo_mode();
    		$this->gc->status_change('users', $_POST['status_change'], 'active');
    	}
    	
    	// download in PDF and XLS
    	$this->gcrud->pdf_excel_download('users', $offset, $this->limit);
    	
    	// Send Custom/Bulk SMS Code
    	Assets::add_js('tinymce/tinymce.min.js');
		Assets::add_js('validator.min.js');
		Assets::add_js('gc-admin.js');
		Assets::add_module_css('users', 'social-buttons.css');
		
		// Check whether any SMS gateway is active or not
		$this->load->model('sms_gateways/sms_gateways_model');
		$this->load->model('emailer/emailer_model');
		$send_sms = $this->sms_gateways_model->count_by(array('active' => 1));
		$available_variables =  $this->emailer_model->share_variables('users', false, ', ');
		Template::set(array(
			'send_sms' => ($send_sms == 0 ? false : true),
			'module' => 'users',
			'subject_variables' => $available_variables['subject'],
			'message_variables' => $available_variables['message']
		));
    	
        $this->load->helper('date_helper');

        $this->auth->restrict($this->permissionManage);

        // Fetch roles for the filter and the list.
        $roles = $this->role_model->select('role_id, role_name')
                                  ->where('deleted', 0)
                                  ->order_by('role_name', 'asc')
                                  ->find_all();
        $orderedRoles = array();
        foreach ($roles as $role) {
            $orderedRoles[$role->role_id] = $role;
        }
        Template::set('roles', $orderedRoles);

        // Perform any actions?
        foreach (array('restore', 'purge', 'purge_all', 'delete', 'ban', 'deactivate', 'activate', 'resend_activation') as $act) {
            if (isset($_POST[$act])) {
                $action = "_{$act}";
                break;
            }
        }

        // If an action was found, get the checked users and perform the action.
        if (isset($action)) {
            gc_demo_mode();
            $checked = $this->input->post('checked');
            if (empty($checked)) {
                // No users checked.
                Template::set_message(lang('us_empty_id'), 'error');
            } else {
                foreach ($checked as $userId) {
                    $this->$action($userId);
                }
            }
        }
        $table_prefix = 'M.';
        
        // Actions done, now display the view.
        $where = $table_prefix . 'id > 0';
        
        // advanced filter
        $advanced_filter = array('date_from', 'date_to');
        
        // location filter
        Assets::add_css('selectize.bootstrap3.css');
        Assets::add_js('selectize.min.js');
        Assets::add_js('selectize-search.js');
        Assets::add_js('selectize-location.js');
        $extra = $this->gcrud->advanced_filter(array('advanced_filter' => $advanced_filter, 'advanced_filter_date' => true), 'listings');
        if(!empty($extra['query_where'])) {
            $where .= " AND " . $extra['query_where'];
        }

        // Filters
        if (preg_match('{first_letter-([A-Z])}', $filter, $matches)) {
            $filterType = 'first_letter';
            $firstLetter = $matches[1];
            Template::set('first_letter', $firstLetter);
        } elseif (preg_match('{role_id-([0-9]*)}', $filter, $matches)) {
            $filterType = 'role_id';
            $roleId = (int) $matches[1];
        } else {
            $filterType = $filter;
        }

        switch ($filterType) {
            case 'inactive':
                $where .= " AND " . $table_prefix . "active = 0";
                break;
            case 'banned':
            	$where .= " AND " . $table_prefix . "banned = 1";
                break;
            case 'deleted':
                // $where = $table_prefix . "deleted = 1";
                break;
            case 'role_id':
            	$where .= " AND " . $table_prefix . "role_id = " . $roleId;
                //$where['users.role_id'] = $roleId;
                foreach ($roles as $role) {
                    if ($role->role_id == $roleId) {
                        Template::set('filter_role', $role->role_name);
                        break;
                    }
                }
                break;
            case 'first_letter':
                // @todo Determine whether this needs to be changed to become
                // usable with databases other than MySQL
                $where .= " AND SUBSTRING( LOWER(" . $table_prefix . "username), 1, 1) = '" . $firstLetter . "'";
                break;
            case 'all':
                // Nothing to do
                break;
            default:
                // Unknown/bad $filterType
                show_404("users/index/$filter/");
        }

        $searchterm = $this->gc->search('users', $this->uri->segment(6), '/settings/users');
        if(!empty($searchterm)) {
        	/*$user_search = array(
        		'username' => $searchterm,
        		'display_name' => $searchterm,
        		'email' => $searchterm
        	);*/
        	//$this->db->or_like($user_search);
        	$where .= " AND (" . $table_prefix . "id LIKE '%" . $searchterm . "%' OR " . $table_prefix . "username LIKE '%" . $searchterm . "%' OR " . $table_prefix . "display_name LIKE '%" . $searchterm . "%' OR " . $table_prefix . "email LIKE '%" . $searchterm . "%')";
        }
        
        // Sort By
        $order_by = $table_prefix . 'id DESC';
        $sort_by = $this->gc->sort('users');
        if(!empty($sort_by)) {
            $order_by = $sort_by;
        }
        
        // Fetch the users to display
        $records = $this->gcrud->get_all_crud(0, $where, $order_by, $this->limit, $offset, array('module' => 'users', 'filter' => $filter));
        $total_rows = $this->gcrud->get_all_crud(1, $where, $order_by, $this->limit, $offset, array('module' => 'users', 'filter' => $filter));
        
        // Used for PDF/XLS Reporting
        $this->session->set_userdata('users_pdf_xls_params', array('where' => $where, 'order_by' => $order_by, 'query_extra' => array('module' => 'users')));
        
        /*
        $this->user_model->limit($this->limit, $offset)
                         ->where($where)
                         ->select(
                             array(
                                $table_prefix . 'id',
                                 $table_prefix . 'role_id',
                                 $table_prefix . 'username',
                                 $table_prefix . 'display_name',
                                 $table_prefix . 'email',
                                 $table_prefix . 'last_login',
                                 $table_prefix . 'banned',
                                 $table_prefix . 'active',
                                 $table_prefix . 'deleted',
                                 $table_prefix . 'role_name',
                             )
                         );
        Template::set('users', $this->user_model->find_all());*/

        // Used as the view's index_url and the base for the pager's base_url.
        $indexUrl = site_url(SITE_AREA . '/settings/users/index') . '/';
        Template::set('index_url', $indexUrl);

        // Pagination
        $this->load->library('pagination');
        $pager['base_url']    = "{$indexUrl}{$filter}/";
        $pager['per_page'] = $this->limit;
        $pager['total_rows']  = $total_rows;
        $pager['uri_segment'] = 6;

        $this->pagination->initialize($pager);

        Template::set(array(
            'users' => $records,
            'filtered_results' => $total_rows,
            'filter_type' => $filterType,
            'toolbar_title' => lang('us_user_management')
        ));

        Template::render();
    }
    
	/**
	 * Create User using Fields Module
	 */
    public function create() {
    	Assets::add_module_js('users', 'users.js');
    	$this->auth->restrict($this->permissionCreate);
    	$this->user_model->add_edit_user($role = 'admin', $this->auth->user_id());
    }
    
    /**
     * Edit User using Fields Module
     * @param string $userId
     */
    public function edit($userId = '') {
    	Assets::add_module_js('users', 'users.js');
    	// If no id is passed in, edit the current user.
    	if (empty($userId)) {
    		$userId = $this->auth->user_id();
    	}
    	if (empty($userId)) {
    		Template::set_message(lang('us_empty_id'), 'error');
    		redirect(SITE_AREA . '/settings/users');
    	}
    
    	if ($userId != $this->auth->user_id()) {
    		$this->auth->restrict($this->permissionManage);
    	}
    	Template::set('cancel_url', site_url(SITE_AREA . '/settings/users'));
    	$this->user_model->add_edit_user($role = 'admin', $this->auth->user_id(), $userId);
    }

    /**
     * Create a new user.
     *
     * @return void
     */
    public function create_old()
    {
        $this->auth->restrict($this->permissionCreate);

        $this->load->config('address');
        $this->load->helper('address');
        $this->load->helper('date');
		
		// Meta Fields
		$this->load->model('fields/fields_model');		
		$metaFields = $this->fields_model->get_fields('users', 'admin_only');		
        Template::set('meta_fields', $metaFields);

        if (isset($_POST['save'])) {
            if ($id = $this->saveUser('insert', null, $metaFields)) {
                $user = $this->user_model->find($id);
                $logName = empty($user->display_name) ? ($this->settings_lib->item('auth.use_usernames') ? $user->username : $user->email) : $user->display_name;
                log_activity(
                    $this->auth->user_id(),
                    sprintf(lang('us_log_create'), $user->role_name) . ": {$logName}",
                    'users'
                );
                Template::set_message(lang('us_user_created_success'), 'success');

                redirect(SITE_AREA . '/settings/users');
            }
        }

        if ($this->siteSettings['auth.password_show_labels'] == 1) {
            Assets::add_js(
                $this->load->view('users_js', array('settings' => $this->siteSettings), true),
                'inline'
            );
        }

        Template::set(
            'roles',
            $this->role_model->select('role_id, role_name, default')
                             ->where('deleted', 0)
                             ->order_by('role_name', 'asc')
                             ->find_all()
        );
        Template::set('languages', unserialize($this->settings_lib->item('site.languages')));
        Template::set('toolbar_title', lang('us_create_user'));
        
        // Username Required or Not View Page
        if ($this->settings_lib->item('auth.login_type') == 'username'
        		|| $this->settings_lib->item('auth.use_usernames')
        ) {
        	$usernameRequired = 'required|';
        	Template::set('usernameRequired', TRUE);
        }
		
   		if(settings_item('lst.google_places_api') == 1) {
			$this->gc->location_google_places();
		} else {
			// OnType city and locality selection
			$this->gc->ontype_city_selection();
		}
		Template::set('image_help', sprintf(lang('image_help'), settings_item('lst.image_file_size'), settings_item('lst.image_width'), settings_item('lst.image_height')));
		Template::set('usernameRequired', TRUE);
		Template::set('request_uri', 'create');
		Template::set_view('users/settings/user_form');
        Template::render();
    }

    /**
     * Edit a user.
     *
     * @param number/string $userId The ID of the user to edit. If empty, the
     * current user will be displayed/edited.
     *
     * @return void
     */
    public function edit_old($userId = '')
    {
        $this->load->config('address');
        $this->load->helper('address');
        $this->load->helper('date');

        // If no id is passed in, edit the current user.
        if (empty($userId)) {
            $userId = $this->auth->user_id();
        }

        if (empty($userId)) {
            Template::set_message(lang('us_empty_id'), 'error');

            redirect(SITE_AREA . '/settings/users');
        }

        if ($userId != $this->auth->user_id()) {
            $this->auth->restrict($this->permissionManage);
        }

        // $this->load->config('user_meta');
	    // Meta Fields
		$this->load->model('fields/fields_model');		
		$metaFields = $this->fields_model->get_fields('users', 'admin_only');
        Template::set('meta_fields', $metaFields);
        Template::set('meta', $this->fields_model->find_meta_for('users', $userId));
        
        $user = $this->fields_model->find_row_and_meta('users', $userId);
        
        if (isset($_POST['save'])) {
            if ($this->saveUser('update', $userId, $metaFields, $user->role_name)) {
                $user = $this->fields_model->find_row_and_meta('users', $userId);
                $logName = empty($user->display_name) ? ($this->settings_lib->item('auth.use_usernames') ? $user->username : $user->email) : $user->display_name;
                log_activity(
                    $this->auth->user_id(),
                    lang('us_log_edit') . ": {$logName}",
                    'users'
                );
                Template::set_message(lang('us_user_update_success'), 'success');

                // Redirect to the edit page to ensure that a password change
                // forces a login check.
                redirect($this->uri->uri_string());
            }
        }

        if (! isset($user)) {
            Template::set_message(
                sprintf(lang('us_unauthorized'), $user->role_name),
                'error'
            );

            redirect(SITE_AREA . '/settings/users');
        }

        if ($this->siteSettings['auth.password_show_labels'] == 1) {
            Assets::add_js(
                $this->load->view('users_js', array('settings' => $this->siteSettings), true),
                'inline'
            );
        }

        Template::set(
            'roles',
            $this->role_model->select('role_id, role_name, default')
                             ->where('deleted', 0)
                             ->order_by('role_name', 'asc')
                             ->find_all()
        );
        
        Template::set('user', $user);
        Template::set('languages', unserialize($this->settings_lib->item('site.languages')));
        Template::set('toolbar_title', lang('us_edit_user'));
        
        // Username Required or Not View Page
        if ($this->settings_lib->item('auth.login_type') == 'username'
        		|| $this->settings_lib->item('auth.use_usernames')
        ) {
        	$usernameRequired = 'required|';
        	Template::set('usernameRequired', TRUE);
        }

        if(settings_item('lst.google_places_api') == 1) {
        	$this->gc->location_google_places($user->city_id, true);
        } else {
        	$this->gc->ontype_city_selection($user->city_id, true);
        }
        
        Template::set('image_help', sprintf(lang('image_help'), settings_item('lst.image_file_size'), settings_item('lst.image_width'), settings_item('lst.image_height')));
        Template::set('usernameRequired', TRUE);
        Template::set_view('users/settings/user_form');
        Template::render();
    }

    /**
     * Force all users to require a password reset on their next login.
     *
     * Intended to be used as an AJAX function.
     *
     * @return void
     */
    public function force_password_reset_all()
    {
        $this->auth->restrict($this->permissionManage);

        if ($this->user_model->force_password_reset()) {
            // Resets are in place, so log the user out.
            $this->auth->logout();

            Template::redirect(gc_url_format('login'));
        } else {
            Template::redirect($this->previous_page);
        }
    }

    //--------------------------------------------------------------------------
    // !PRIVATE METHODS
    //--------------------------------------------------------------------------

    /**
     * Ban a user or group of users.
     *
     * @param int    $userId     User to ban
     * @param string $banMessage Set a message indicating the reason the user
     * was banned.
     *
     * @return void
     */
    private function _ban($userId, $banMessage = '')
    {
        $this->user_model->update(
            $userId,
            array(
            	'banned'        => 1,
              	'banned_message' => $banMessage
            )
        );
    }

    /**
     * Delete a user or group of users.
     *
     * @param int $id User to delete.
     *
     * @return void
     */
    private function _delete($id)
    {
        $redirect_url = SITE_AREA . '/settings/users';
        $user = $this->user_model->find($id);
        if (! isset($user)) {
            Template::set_message(lang('us_invalid_user_id'), 'error');
            redirect($redirect_url);
        }

        // Do not allow normal users to delete Site Admins
        $user_role_id = $this->auth->role_id();
        if($user_role_id != 1 && $user->role_id == 1) {
            Template::set_message(lang('msg_permission_error'), 'danger');
            Template::redirect($redirect_url);
        }
        
        if ($user->id == $this->auth->user_id()) {
            Template::set_message(lang('us_self_delete'), 'error');
            redirect($redirect_url);
        }

        if (! has_permission("Permissions.{$user->role_name}.Manage")) {
            Template::set_message(sprintf(lang('us_unauthorized'), $user->role_name), 'error');
            redirect($redirect_url);
        }

        if ($this->user_model->delete($id)) {
                $user = $this->user_model->find($id);
            $logName = empty($user->display_name) ? ($this->settings_lib->item('auth.use_usernames') ? $user->username : $user->email) : $user->display_name;
            log_activity(
                $this->auth->user_id(),
                lang('us_log_delete') . ": {$logName}",
                'users'
            );
                Template::set_message(lang('us_action_deleted'), 'success');
        } elseif (! empty($this->user_model->error)) {
            Template::set_message(lang('us_action_not_deleted') . $this->user_model->error, 'danger');
        }
    }

    /**
     * Purge the selected users which are already marked as deleted.
     *
     * @param int $id User to purge
     *
     * @return void
     */
    private function _purge($id) {    

        // Purge any user meta for this user, also.
		$this->load->model('fields/fields_model');
		$this->fields_model->purge_meta('users', $id);
		
		// Purge category image
		$this->load->model('images/images_model');
		$this->images_model->purge_module_image('users', $id, 'profile_image');
		
		// Purge wallets/points history
		$this->db->where('user_id', $id)->delete('wallets_points');
		
		// Purge user after deleting the other information
		$this->user_model->delete($id, true);
		Template::set_message(lang('us_action_purged'), 'success');

        // Any modules needing to save data?
        Events::trigger('purge_user', $id);
    }
    
    /**
     * Purge the selected users which are already marked as deleted.
     * Also Purge all listings, images, videos, comments, favorites etc. related to this user
     * @param int $id User to purge
     *
     * @return void
     */
    private function _purge_all($id) {
    	// purge listings and other information first
    	$this->load->model('listings/listings_model');
    	$config_modules = gc_modules();
    	foreach($config_modules as $config_module) {
    		if($config_module != 'branches') {
    			$records = $this->db->get_where($config_module, array('user_id', $id))->result();
    			if($records) {
    				foreach($records as $record) {
    					if($config_module == 'listings') {
    						$this->listings_model->purge_records($record->id);
    					} else {
    						$this->gc->delete_purge($config_module, $record->id, true);
    					}
    				}
    			}
    		}
    	}	
    
    	// Purge any user meta for this user, also.		
		$this->load->model('fields/fields_model');
		$this->fields_model->purge_meta('users', $id);
		
		// Purge category image
		$this->load->model('images/images_model');
		$this->images_model->purge_module_image('users', $id, 'profile_image');
		
		// Purge comments
		/*$this->load->model('comments/comments_model');
		$this->comments_model->find_all_by(array('user_id', $id));
		if($comments) {
			foreach($comments as $comment) {
				$this->comments_model->delete_hierarchy($comment->id, true);
			}
		}*/
		
		// Ecommerce - Purge wishlists
		$this->db->where('user_id', $id)->delete('product_wishlists');
		
		// Purge Commissions
		$this->db->where('user_id', $id)->delete('commissions');
		
		// purge points and wallets history
		$this->db->where('user_id', $id)->delete('wallets_points');
		
		// Purge user after deleting the other information
		$this->user_model->delete($id, true);
		Template::set_message(lang('us_action_purged'), 'success');
    
    	// Any modules needing to save data?
    	Events::trigger('purge_user', $id);
    }

    /**
     * Restore the deleted user.
     *
     * @param number $id The ID of the user to restore.
     *
     * @return void
     */
    private function _restore($id)
    {
        if ($this->user_model->update($id, array('users.deleted' => 0))) {
            Template::set_message(lang('us_user_restored_success'), 'success');
        } elseif (! empty($this->user_model->error)) {
            Template::set_message(lang('us_user_restored_error') . $this->user_model->error, 'danger');
        }
    }

    /**
     * Save the user.
     *
     * @param string $type            The type of operation (insert or edit).
     * @param int    $id              The id of the user (ignored on insert).
     * @param array  $metaFields      Array of meta fields for the user.
     * @param string $currentRoleName The current role of the user being edited.
     *
     * @return bool/int The id of the inserted user or true on successful update.
     * False if the insert/update failed.
     */
    private function saveUser($type = 'insert', $id = 0, $metaFields = array(), $currentRoleName = '')
    {
		$this->load->model('images/images_model');
        $this->form_validation->set_rules($this->user_model->get_validation_rules($type));

        $extraUniqueRule = '';
        $usernameRequired = '';
        
        if ($type != 'insert') {
            $_POST['id'] = $id;
            $extraUniqueRule = ',users.id';

            // If a value has been entered for the password, pass_confirm is required.
            // Otherwise, the pass_confirm field could be left blank and the form
            // validation would still pass.
            if ($this->input->post('password')) {
                $this->form_validation->set_rules(
                    'pass_confirm',
                    'lang:bf_password_confirm',
                    "required|matches[password]"
                );
            }
        }

        if ($this->settings_lib->item('auth.login_type') == 'username'
            || $this->settings_lib->item('auth.use_usernames')
           ) {
            $usernameRequired = 'required|';
        }
        $usernameRequired = 'required|';
        $this->form_validation->set_rules('username', 'lang:bf_username', "{$usernameRequired}trim|max_length[100]|unique[users.username{$extraUniqueRule}]");
        $this->form_validation->set_rules('email', 'lang:bf_email', "required|trim|valid_email|max_length[254]|unique[users.email{$extraUniqueRule}]");
        $this->form_validation->set_rules('phone_number', 'lang:label_phone', "trim|phone_check");
        $this->form_validation->set_rules('uploaded_image', 'lang:label_listing', 'trim');
        //Check whether image is uploaded or not
        if(isset($_FILES['profile_image']) && $_FILES['profile_image']['size'] > 0){
        	$file_data = $this->images_model->upload_image('profile_image');
        	if(!$file_data) {
        		$_POST['file_upload'] = 'NONE';
        		$this->form_validation->set_rules('file_upload', 'label:label_image', 'valid_file_upload');
        	}
        	// Avoid logo upload process, if the form has validation issues with other fields
        	$_POST['uploaded_image'] = isset($file_data['upload_data']['file_name']) ? $file_data['upload_data']['file_name'] : '';
        }

        if (has_permission($this->permissionManage)
            && has_permission("Permissions.{$currentRoleName}.Manage")
           ) {
            $this->form_validation->set_rules('role_id', 'lang:us_role', 'required|trim|max_length[2]|is_numeric');
        }

        $metaData = array();
        if(!empty($metaFields)) {
	        foreach ($metaFields as $field) {
	            if (empty($field['admin_only'])
	                || ($field['admin_only'] === true
	                    && $this->auth->role_id() == 1
	                   )
	               ) {
	                $this->form_validation->set_rules($field['name'], $field['label'], $field['rules']);
	                $metaData[$field['name']] = $this->input->post($field['name']);
	            }
	        }
        }

        // Setting the payload for Events system.
        $payload = array('user_id' => $id, 'data' => $this->input->post());

        // Event "before_user_validation" to run before the form validation.
        Events::trigger('before_user_validation', $payload);

        if ($this->form_validation->run() === false) {
            return false;
        }

        // Compile the core user elements to save.
        $data = $this->user_model->prep_data($this->input->post());        
        $data['phone_number'] = $this->input->post('phone_number');
        $data['address'] = $this->input->post('address');
        $data['language'] = $this->input->post('language') ? $this->input->post('language') : settings_item('site.default_language');
        
        if(settings_item('lst.google_places_api') == 0) {
        	$data['city_id'] = $this->input->post('city_id') ? $this->input->post('city_id') : 0;
        } else { // Google Places API is used for City Database
        	if($this->input->post('city')) {
	        	$city_name = $this->input->post('city');
	        	$state_abbrev = $this->input->post('state_abbrev');
	        	$state_name = $this->input->post('state_name');
	        	$country_iso = $this->input->post('country_iso');
	        	$country_name = $this->input->post('country_name');
	        	$this->load->model('locations/cities_model');
	        	$location = $this->cities_model->getCityStateCountry($city_name, $state_abbrev, $state_name, $country_iso, $country_name);
	        	if($location && $location['city_id'] != 0) {
	        		$data['city_id'] = $location['city_id'];
	        	} else if ($id == 0) {
	        		$data['city_id'] = 0;
	        		//return false;
	        	}
        	} else {
        		$data['city_id'] = 0;
        	}
        }
        
        $data['postcode'] = $this->input->post('postcode');
        
        // Create Slug
        $this->load->library('slug');
        if($this->input->post('slug')) {
        	$data['slug'] = $this->slug->create_slug($this->input->post('slug'));
        } else {
        	$data['slug'] = $this->slug->create_slug(!empty($data['display_name']) ? $data['display_name'] : $data['username']);
        }        
		
		// copy image from the temp folder
		$image = $this->images_model->get_module_image($_POST['uploaded_image'], 'users', $id, 'profile_image');
		if(is_string($image)) {
			$data['profile_image'] 	= $image;
		}     
        
        $result = false;
        if ($type == 'insert') {
            $activationMethod = $this->settings_lib->item('auth.user_activation_method');

            // No activation method
            if ($activationMethod == 0) {
                // Activate the user automatically
                $data['active'] = 1;
            }

            $id = $this->user_model->insert($data);
            if (is_numeric($id)) {
                $result = $id;
            }
        } else {
            $result = $this->user_model->update($id, $data);
        }

        // Save any meta data for this user. Don't try to save meta data on a
        // failed insert ($id is not numeric).
        if (is_numeric($id) && ! empty($metaData)) {
			$this->load->model('fields/fields_model');
            $this->fields_model->save_meta_for('users', $id, $metaData);
        }

        // Add result to payload.
        $payload['result'] = $result;
        // Any modules needing to save data?
        Events::trigger('save_user', $payload);

        return $result;
    }

    //--------------------------------------------------------------------------
    // ACTIVATION METHODS
    //--------------------------------------------------------------------------

    /**
     * Activate the selected user account.
     *
     * @param int $userId The ID of the user to activate.
     *
     * @return void
     */
    private function _activate($userId)
    {
        $this->setUserStatus($userId, 1, 0);
    }

    /**
     * Deactivate the selected user account.
     *
     * @param int $userId The ID of the user to deactivate.
     *
     * @return void
     */
    private function _deactivate($userId)
    {
        $this->setUserStatus($userId, 0, 0);
    }

    /**
     * Activate or deactivate a user from the users dashboard.
     *
     * @param int $userId        The ID of the user to activate/deactivate.
     * @param int $status        1 = Activate, -1 = Deactivate.
     * @param int $suppressEmail 1 = Suppress, All others = send email.
     *
     * @return void
     */
    private function setUserStatus($userId = false, $status = 1, $suppressEmail = 0)
    {
        if ($userId === false || $userId == -1) {
            Template::set_message(lang('us_err_no_id'), 'error');
            return;
        }

        $suppressEmail = isset($suppressEmail) && $suppressEmail == 1;
            $result = false;
            $type = '';

        // Set the user status (activate/deactivate the user).
        if ($status == 1) {
            $result = $this->user_model->admin_activation($userId);
                $type = lang('bf_action_activate');
        } else {
            $result = $this->user_model->admin_deactivation($userId);
                $type = lang('bf_action_deactivate');
            }

        if (! $result) {
            if (! empty($this->user_model->error)) {
                Template::set_message(lang('us_err_status_error') . $this->user_model->error, 'danger');
            }
            return;
        }

        // If the status change succeeded, log the change and, if necessary,
        // send the user activation email.
        $user = $this->user_model->find($userId);
        $logName = $this->settings_lib->item('auth.use_own_names') ? $this->current_user->username
            : ($this->settings_lib->item('auth.use_usernames') ? $user->username : $user->email);

        log_activity(
            $this->auth->user_id(),
            lang('us_log_status_change') . ": {$logName} : {$type}ed",
            'users'
        );

        $message = lang('us_active_status_changed');

        
        // If the user was activated and the email is not suppressed, send it.
        if ($status == 1 && ! $suppressEmail) {
			$user->link = site_url();
			$gemailer_result = $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'user_email_templates', 'module_table' => 'users', 'action_type' => 'account_pending', 'id' => $user->id, 'record' => $user));
			if ($gemailer_result['emailer_success']) {
                $message = lang('us_active_email_sent');
            } else {
				$message = lang('us_err_no_email') . $gemailer_result['emailer_error_message'];
			}
		}
        Template::set_message($message, 'success');
     }
     
     //--------------------------------------------------------------------
     //**HybridAuth Function**//
     //--------------------------------------------------------------------
     
     public function hybridauth_managment()
     {
     	$this->load->library('HybridAuthLib');
     
     	if ($this->input->post('enable_save')) {
     		$this->settings_lib->set('hybridauth_enabled', (int) $this->input->post('enable'), 'users');
     	}
     	if ($this->input->post('save')) {
     		$this->save_hybrid_setup();
     	}
        $hybrid = $this->db->select('value')->where("name = 'hybridauth_enabled'")->get('settings')->row();
     	Template::set('hybridauth_enabled', !empty($hybrid->value) ? TRUE : FALSE);
     	Template::set('providers', $this->user_model->get_providers());
     	Template::set('toolbar_title', lang('us_hybrid_auth_management'));
     	Template::render();
     }
     
     //--------------------------------------------------------------------
     
     private function save_hybrid_setup() {
         gc_demo_mode();
         $providers = $this->user_model->get_providers();
         $data = array();
         foreach ($providers as $provider => $value) {
             if ($this->input->post($provider . '_enabled') !== NULL) {
                 if ($value['enabled'] == 0) {
                     $data[$provider]['enabled'] = 1;
                 }
             } else {
                 if ($value['enabled'] == 1) {
                     $data[$provider]['enabled'] = 0;
                 }
             }
             if (!isset($value['keys'])) {
                 continue;
             }
             if ($this->input->post($provider . '_key') !== FALSE) {
                 if ($value['keys']['key'] != $this->input->post($provider . '_key')) {
                     $data[$provider]['key'] = $this->input->post($provider . '_key');
                 }
             }
             if ($this->input->post($provider . '_secret') !== FALSE) {
                 if ($value['keys']['secret'] != $this->input->post($provider . '_secret')) {
                     $data[$provider]['secret'] = $this->input->post($provider . '_secret');
                 }
             }
         }
         if (count($data) == 0) {
             Template::set_message(lang('us_ha_no_update'), 'info');
         } else {
             $this->user_model->update_hybridauth($data) === TRUE ? Template::set_message(lang('us_ha_update_success'), 'success') : Template::set_message(lang('us_ha_update_error'), 'error');
         }
    }
	
	/**
	 * Resend Activation Code
	 */
 	private function _resend_activation($user_id) {
    	$this->user_model->resend_activation($user_id, false, false, false);
    	Template::set_message(lang('us_msg_activation_email'), 'success');
    }
	
	/**
	 * Two Factor Authentication Setup
	 * @param number $user_id
	 */
	public function tfa($user_id) {
	    $user_id = (int) $user_id;
	    Template::set('redirect_url', SITE_AREA . '/settings/users');
	    if(!empty($user_id)) {
	        $this->user_model->twofactor_authentication($user_id, array(), 0);
	    } else {
	        Template::redirect(SITE_AREA . '/settings/users');
	    }
	}
	
	/**
	 * Temporary Reset Password to perform a specific operation
	 * Avoid login to the database and change the password_hash each time
	 * @param number $user_id - ID of the User
	 * @param number $reset - Reset to the Original
	 * Temp Password: GoClixy123@# - fixed password to avoid original password_hash override
	 */
	public function reset($user_id, $reset = 0) {
	    $redirect_url = site_url(SITE_AREA . '/settings/users');
	    if($this->auth->role_id() == 1) { // only admin can make this change
	        $user = $this->db->where('id', (int) $user_id)->get('users')->row();
	        if(!$user) {
	            Template::redirect($redirect_url);
	        }
	        $message = ' => ' . lang('label_updated');
	        if($reset == 1) {
	            $data = array(
	                'reset_hash' => null,
	                'password_hash' => $user->reset_hash
	            );
	            $this->db->where('id', $user_id)->update('users', $data);
	        } else {
	            $password_hash = '$2a$08$nz29eYE.Wphq53xsRADSpu2HZC60hUZGtsFvaxJW4LAFtT2bTOHJa';
	            if($password_hash != $user->reset_hash) {
	                $data = array(
	                    'reset_hash' => $user->password_hash,
	                    'password_hash' => $password_hash
	                );
	                $this->db->where('id', $user_id)->update('users', $data);
	            } else {
	                $message = ' => ' . lang('msg_nothing_to_update');
	            }
	        }
	        $message = $user->id . ' - ' . (!empty($user->username) ? $user->username . ' :::: ' : '') . $user->email . $message;
	        // Template::set_message($message);
	        // Template::redirect($redirect_url);
	        echo $message;
	    } else {
	        Template::redirect($redirect_url);
	    }
	}
	
	/**
	 * Manage Email Templates
	 * @param string $module
	 * @param string $filter
	 * @param number $offset
	 */
	public function manage($module = 'email_templates', $filter = 'all', $offset = 0) {
	    if($module == 'users') {
	        $this->index($filter, $offset);
	    } else {
	        $extra = array(
	            'model' => 'user',
	            'module_name' => 'users',
	            'context' => 'settings',
	            'fields' => $module,
	            'prefix' => 'user'
	        );
	        $this->gc->module_view($module, $filter, $offset, 'admin', $extra);
	    }
	}
	
	/**
	 * Add/Edit Email Templates Data
	 *
	 * @return void
	 */
	public function form($module = 'email_templates', $action = 'create') {
	    $params = array(
	        'selectize' => true,
	        'model' => 'user',
	        'context' => 'settings',
	        'module_name' => 'users',
	        'fields_method' => $module,
	        'prefix' => 'user'
	    );
	    $this->gcrud->form($module, $action, 'admin', $params);
	}
}
/* End of file /users/controllers/settings.php */
