<?php

if (!defined('BASEPATH'))
    exit('No direct script access allowed');
/**
 * Bonfire
 *
 * An open source project to allow developers get a jumpstart their development of CodeIgniter applications
 *
 * @package   Bonfire
 * @author    Bonfire Dev Team
 * @copyright Copyright (c) 2011 - 2013, Bonfire Dev Team
 * @license   http://guides.cibonfire.com/license.html
 * @link      http://cibonfire.com
 * @since     Version 1.0
 * @filesource
 */
// ------------------------------------------------------------------------

/**
 * Users Controller
 *
 * Provides front-end functions for users, like login and logout.
 *
 * @package    Bonfire
 * @subpackage Modules_Users
 * @category   Controllers
 * @author     Bonfire Dev Team
 * @link       http://cibonfire.com
 *
 */
class Users extends Front_Controller
{
    //--------------------------------------------------------------------
	private $login_url = "";
	private $register_url = "";
    /**
     * Setup the required libraries etc
     *
     * @retun void
     */
    public function __construct()
    {
        parent::__construct();

        $this->load->helper('form');
        $this->load->helper('cookie');
        $this->load->library('form_validation');

        $this->load->model('users/user_model');

        $this->load->library('users/auth');

        $this->lang->load('users');
        
        //phone number verification
        Assets::add_js('intlTelInput.min.js');
        Assets::add_js('isValidNumber.js');
        Assets::add_css('intlTelInput.css');
        Assets::add_module_js('users', 'facebook.js');
        Assets::add_module_js ( 'users', 'users.js' );
        
        // Set LOGIN_URL to modal if theme login is in modal
        $this->login_url = gc_url_format('login');
        $this->register_url = gc_url_format('register');
        /**
         * To set default country for mobile number in jQuery function based on cookie value
         */
        Template::set('mobile_country_iso', strtolower($this->session->userdata('search_country')));
    }

//end __construct()
    //--------------------------------------------------------------------

    /**
     * Presents the login function and allows the user to actually login.
     *
     * @access public
     *
     * @return void
     */
    public function login($role_id = NULL) {
    	Assets::add_css ( 'social-buttons.css' );
    	Assets::add_js('bootstrap-show-password.min.js');
    	delete_cookie("formbox"); // important or no message will be displayed
        // if the user is not logged in continue to show the login page
        
    	// if the user is not logged in continue to show the login page
		// Load Meta Fields
		$this->load->model('fields/fields_model');		
		$meta_fields = $this->fields_model->get_fields('users', 'frontend');
    	Template::set('meta_fields', $meta_fields);
    	
    	// Common method for login
    	$this->user_model->login();
    	
    	// Template::set_view('users/login');
    	//Hybrid_auth
    	if ($this->settings_lib->item('hybridauth_enabled') == 1) {
    		Template::set('hybrid_auth_providers', $this->user_model->get_providers());
    	}
    		
    	// Because of common login, registration and forgot password form
    	// Generate password hint messages.
    	$this->user_model->password_hints();
    		
    	Template::set('page_title', lang('bf_action_login'));
    	$layout = $this->gc->load_assets('users/login');
    	Template::set_view($layout['view']);
    	Template::render($layout['layout']);    	
    }
    
    /**
     * Allow user to login via ajax
     * @return array data
     */
    public function ajax_login() {
    	$this->user_model->login(true);
    }
    
    /**
     * Presents the admin login function and allows the admin to actually login.
     *
     * @access public
     *
     * @return void
     */
    public function restrictedZone($role_id = 'admin') {
        Assets::add_module_css('users', 'restricted-zone.css');
    	// if the user is not logged in continue to show the login page
    	if ($this->auth->is_logged_in() === FALSE) {
    		if (isset($_POST['log-me-in'])) {
    			$remember = $this->input->post('remember_me') == '1' ? TRUE : FALSE;
    			// Try to login
    			if ($this->auth->login(escape($this->input->post('login')), $this->input->post('password'), $remember, FALSE, $role_id) === TRUE) {
    
    				// Log the Activity
    				log_activity($this->auth->user_id(), lang('us_log_logged') . ': ' . $this->input->ip_address(), 'users');
    				
    				// create a temporary file on server for filemanager access    				
    				if (! function_exists('write_file')) {
    				    $this->load->helper('file');
    				}
    				$filename = str_replace(':', '', $this->get_client_ip_server());
    				write_file(FCPATH . 'assets/filemanager/' . $filename . '.txt', date('Y-m-d H:i:s'));
    
    				// Now redirect.  (If this ever changes to render something,
    				// note that auth->login() currently doesn't attempt to fix
    				// `$this->current_user` for the current page load).
    
    				/*
    				 In many cases, we will have set a destination for a
    				particular user-role to redirect to. This is helpful for
    				cases where we are presenting different information to different
    				roles that might cause the base destination to be not available.
    				*/
    				if ($this->settings_lib->item('auth.do_login_redirect') && !empty($this->auth->login_destination)) {
    					Template::redirect($this->auth->login_destination);
    				} else {
    					if (!empty($this->requested_page)) {
    						Template::redirect($this->requested_page);
    					} else {
    						Template::redirect(SITE_AREA);
    					}
    				}
    			}
    			//end if
    		}//end if
    
    		/**
    		 * FORGOT PASSWORD
    		 */
    		if (isset($_POST['send'])) {
    			$this->form_validation->set_rules('email', 'lang:bf_email', 'required|trim|valid_email');
    
    			if ($this->form_validation->run() !== FALSE) {
    				// We validated. Does the user actually exist?
    				$user = $this->user_model->find_by(array('email' => $_POST['email'], $this->db->dbprefix . 'users.role_id' => 1));
    
    				if ($user !== FALSE) {
    					// User exists, so create a temp password.
    					$this->load->helpers(array('string', 'security'));
    
    					$pass_code = random_string('alnum', 40);
    
    					$hash = do_hash($pass_code . $_POST['email']);
    
    					// Save the hash to the db so we can confirm it later.
    					$this->user_model->update_where('email', $_POST['email'], array('reset_hash' => $hash, 'reset_by' => strtotime("+24 hours")));
    
    					// Create the link to reset the password
    					$this->load->library('crypt');
    					$pass_link = gc_url_format('reset_password') . '/' . $this->crypt->encode($user->id) . '/' . $hash;
    
    					// Now send the email
    					$this->load->library('emailer/emailer');
    
    					$data = array(
    							'to' => $_POST['email'],
    							'subject' => lang('us_reset_pass_subject'),
    							'message' => $this->load->view('_emails/forgot_password', array('link' => $pass_link), TRUE)
    					);
    
    					if ($this->emailer->send($data)) {
    						Template::set_message(lang('us_reset_pass_message'), 'success');
    					} else {
    						Template::set_message(lang('us_reset_pass_error') . $this->emailer->error, 'danger');
    					}
    				} else {
    					Template::set_message(lang('us_invalid_email'), 'danger');
    				}//end if
    			}//end if
    		}//end if
    
    		Template::set('page_title', lang('bf_action_login'));
    		Template::render('blank');
    	}
    	else {
    		Template::redirect(SITE_AREA);
    	}//end if
    }
    
    // Function to get the real client ip address
    // https://www.virendrachandak.com/techtalk/getting-real-client-ip-address-in-php-2/
    private function get_client_ip_server() {
        $ipaddress = '';
        if (isset($_SERVER['HTTP_CLIENT_IP'])) {
            $ipaddress = $_SERVER['HTTP_CLIENT_IP'];
        } else if(isset($_SERVER['HTTP_X_FORWARDED_FOR'])) {
            $ipaddress = $_SERVER['HTTP_X_FORWARDED_FOR'];
        } else if(isset($_SERVER['HTTP_X_FORWARDED'])) {
            $ipaddress = $_SERVER['HTTP_X_FORWARDED'];
        } else if(isset($_SERVER['HTTP_FORWARDED_FOR'])) {
            $ipaddress = $_SERVER['HTTP_FORWARDED_FOR'];
        } else if(isset($_SERVER['HTTP_FORWARDED'])) {
            $ipaddress = $_SERVER['HTTP_FORWARDED'];
        } else if(isset($_SERVER['REMOTE_ADDR'])) {
            $ipaddress = $_SERVER['REMOTE_ADDR'];
        } else {
            $ipaddress = 'UNKNOWN';
        }                                    
        return $ipaddress;
    }    

//end login()
    //--------------------------------------------------------------------

    /**
     * Calls the auth->logout method to destroy the session and cleanup,
     * then redirects to the home page.
     *
     * @access public
     *
     * @return void
     */
    public function logout()
    {
        if (isset($this->current_user->id))
        {
            // Login session is valid.  Log the Activity
            log_activity($this->current_user->id, lang('us_log_logged_out') . ': ' . $this->input->ip_address(), 'users');
        }
        
        // Filemanager Security - Do not allow guest/users to open filemanager
        // delete this file on logout
        $filename = str_replace(':', '', $this->input->ip_address());
        $admin_logged_in = FCPATH . 'assets/filemanager/' . $filename . '.txt';
        if(file_exists($admin_logged_in)) {
            unlink($admin_logged_in);
        }
        
        // Always clear browser data (don't silently ignore user requests :).
        $this->auth->logout();

        redirect('/');
    }

	//end  logout()
    //--------------------------------------------------------------------

    /**
     * Allow user to Forgot Password
     * @access public
     * @return void
     */
    public function forgot_password() {
    	$this->user_model->forgot_password();        
    }
    
    /**
     * Ajax Forgot Password Method
     */
    public function ajax_forgot_password() {	
    	$this->user_model->forgot_password(true);
    }
    
    /**
     * Allow user to login via auth
     * @return array data
     */
    public function auth_login() {
        $this->gcrud->auth_login();
    }
    
    /**
     * Allows a user to edit their own profile information.
     *
     * @access public
     *
     * @return void
     */
    public function profile() {
    	Assets::add_module_css('users', 'users.css');
    	$this->user_model->add_edit_user('members', $this->auth->user_id(), $this->auth->user_id());
    }
    
    //--------------------------------------------------------------------

    /**
     * Allows the user to create a new password for their account. At the moment,
     * the only way to get here is to go through the forgot_password() process,
     * which creates a unique code that is only valid for 24 hours.
     *
     * Since 0.7 this method is also gotten to here by the force_password_reset
     * security features.
     *
     * @access public
     *
     * @param string $email The email address to check against.
     * @param string $code  A randomly generated alphanumeric code. (Generated by forgot_password() ).
     *
     * @return void
     */
    public function reset_password($user_id = '', $code = '') {
        // if the user is not logged in continue to show the login page
        if ($this->auth->is_logged_in()) {
        	Template::redirect(site_url());
        }
        $this->load->library('crypt');
        $user_id = intval($this->crypt->decode($user_id));
        if($user_id == 0) {
        	Template::set_message(lang('us_reset_invalid_email'), 'error');
        	Template::redirect(gc_url_format('login'));
        }
        
        $this->user_model->reset_password($user_id, $code);
        
        $settings = $this->settings_lib->find_all();
        if ($settings['auth.password_show_labels'] == 1) {
        	Assets::add_module_js('users', 'password_strength.js');
        	Assets::add_module_js('users', 'jquery.strength.js');
        	Assets::add_js($this->load->view('users_js', array('settings' => $settings), true), 'inline');
        }
        
        // Generate password hint messages.
        $this->user_model->password_hints();
        
        $layout = $this->gc->load_assets('users/reset_password');
        Template::set_view($layout['view']);
        Template::render($layout['layout']);
    }// end reset_password()
    
    //--------------------------------------------------------------------

    /**
     * Display the registration form for the user and manage the registration process.
     *
     * You can override the redirect URL's for success (Login) and failure (register)
     * by including a hidden field in your form for each, named 'register_url' and
     * 'login_url' respectively.
     *
     * @access public
     *
     * @return void
     */
    public function register() {
        // Assets::add_js('bootstrap-show-password.min.js');
        if ($this->auth->is_logged_in() === TRUE) {
            Template::redirect();
        }
        $this->user_model->register();
        Template::set('page_title', lang('us_register'));
        $layout = $this->gc->load_assets('users/register');
        Template::set_view($layout['view']);
        Template::render($layout['layout']);
    }
    
    
    /**
     * User Registration using AJAX Method
     */
    public function ajax_register() {    	
    	$this->load->model('roles/role_model');
    	$this->load->helper('date');
    
    	$this->load->config('address');
    	$this->load->helper('address');
    	$this->user_model->register(true);
	}

//--------------------------------------------------------------------

    /**
     * Save the user
     *
     * @access private
     *
     * @param int   $id          The id of the user in the case of an edit operation
     * @param array $meta_fields Array of meta fields fur the user
     *
     * @return bool
     */
    private function save_user($id = 0, $meta_fields = array()) {
		$this->load->model('images/images_model');
        if ($id == 0) {
            $id = $this->current_user->id; /* ( $this->input->post('id') > 0 ) ? $this->input->post('id') :  */
        }

        $_POST['id'] = $id;

        // Simple check to make the posted id is equal to the current user's id, minor security check
        if ($_POST['id'] != $this->current_user->id)
        {
            $this->form_validation->set_message('email', 'lang:us_invalid_userid');
            return FALSE;
        }

        // Setting the payload for Events system.
        $payload = array('user_id' => $id, 'data' => $this->input->post());


        $this->form_validation->set_rules('email', 'lang:bf_email', 'required|trim|valid_email|max_length[120]|unique[users.email,users.id]');
        $this->form_validation->set_rules('password', 'lang:bf_password', 'max_length[120]|valid_password');

        // check if a value has been entered for the password - if so then the pass_confirm is required
        // if you don't set it as "required" the pass_confirm field could be left blank and the form validation would still pass
        $extra_rules = !empty($_POST['password']) ? 'required|' : '';
        $this->form_validation->set_rules('pass_confirm', 'lang:bf_password_confirm', '' . $extra_rules . 'matches[password]');

        $username_required = '';
        if ($this->settings_lib->item('auth.login_type') == 'username' ||
                $this->settings_lib->item('auth.use_usernames'))
        {
            $username_required = 'required|';
        }
        $this->form_validation->set_rules('username', 'lang:bf_username', $username_required . 'trim|max_length[30]|unique[users.username,users.id]');

        $this->form_validation->set_rules('language', 'lang:bf_language', 'required|trim');
        $this->form_validation->set_rules('phone_number', 'lang:label_phone', 'phone_check|xss_clean');
        $this->form_validation->set_rules('timezones', 'lang:bf_timezone', 'required|trim|max_length[4]');
        $this->form_validation->set_rules('display_name', 'lang:bf_display_name', 'trim|max_length[255]');
        $this->form_validation->set_rules('uploaded_image', 'lang:label_listing', 'trim');
        //Check whether image is uploaded or not
        if(isset($_FILES['profile_image']) && $_FILES['profile_image']['size'] > 0){
        	$file_data = $this->images_model->upload_image('users', 'profile_image');
        	if(!$file_data) {
        		$_POST['file_upload'] = 'NONE';
        		$this->form_validation->set_rules('file_upload', 'label:label_image', 'valid_file_upload');
        	}
        	// Avoid logo upload process, if the form has validation issues with other fields
        	$_POST['uploaded_image'] = isset($file_data['upload_data']['file_name']) ? $file_data['upload_data']['file_name'] : '';
        }

        // Added Event "before_user_validation" to run before the form validation
        Events::trigger('before_user_validation', $payload);

		if(!empty($meta_fields)) {
	        foreach ($meta_fields as $field) {
	            if ((!isset($field['admin_only']) || $field['admin_only'] === FALSE || (isset($field['admin_only']) && $field['admin_only'] === TRUE && isset($this->current_user) && $this->current_user->role_id == 1)) && (!isset($field['frontend']) || $field['frontend'] === TRUE)) {
	                $this->form_validation->set_rules($field['name'], $field['label'], $field['rules']);
	            }
	        }
		}


        if ($this->form_validation->run() === FALSE) {
            return FALSE;
        }

        // Compile our core user elements to save.
        $data = array(
            'email' => $this->input->post('email'),
            'language' => $this->input->post('language') ? $this->input->post('language') : settings_item('site.default_language'),
            'timezone' => $this->input->post('timezones'),
        );

        // If empty, the password will be left unchanged.
        if ($this->input->post('password') !== '') {
            $data['password'] = $this->input->post('password');
        }

        if ($this->input->post('display_name') !== '') {
            $data['display_name'] = $this->input->post('display_name');
        }
        
        if ($this->input->post('phone_number') !== '') {
        	$data['phone_number'] = $this->input->post('phone_number');
        }
        
        if (isset($_POST['username'])) {
            $data['username'] = $this->input->post('username');
        }
                
        $data['address'] = $this->input->post('address');
        if(settings_item('lst.google_places_api') == 0) {
        	$data['city_id'] = $this->input->post('city_id');
        } else { // Google Places API is used for City Database
        	$city_name = $this->input->post('city');
        	$state_abbrev = $this->input->post('state_abbrev');
        	$state_name = $this->input->post('state_name');
        	$country_iso = $this->input->post('country_iso');
        	$country_name = $this->input->post('country_name');
        	$this->load->model('locations/cities_model');
        	$location = $this->cities_model->getCityStateCountry($city_name, $state_abbrev, $state_name, $country_iso, $country_name);
        	if($location && $location['city_id'] != 0) {
        		$data['city_id'] = $location['city_id'];
        	} else if ($id == 0) {
        		return false;
        	}
        }
        
        $data['postcode'] = $this->input->post('postcode');
        $image = $this->images_model->get_module_image($_POST['uploaded_image'], 'users', $id, 'profile_image');
		if(is_string($image)) {
			$data['profile_image'] 	= $image;
		}

        // Any modules needing to save data?
        // Event to run after saving a user
        Events::trigger('save_user', $payload);
        	
        return $this->user_model->update($id, $data);
    }

	//end save_user()
    //--------------------------------------------------------------------
    //--------------------------------------------------------------------
    // ACTIVATION METHODS
    //--------------------------------------------------------------------
    /*
      Activate user.

      Checks a passed activation code and if verified, enables the user
      account. If the code fails, an error is generated and returned.

     */
    public function activate($user_id = NULL, $code = NULL) {
    	$this->user_model->activate_user($user_id, $code);    	
    }

    //--------------------------------------------------------------------

    /*
      Method: resend_activation

      Allows a user to request that their activation code be resent to their
      account's email address. If a matching email is found, the code is resent.
     */
    public function resend_activation($user_id) {
    	$this->load->library('crypt');
    	$user_id = intval($this->crypt->decode($user_id));
    	if($user_id == 0) {
    		Template::redirect(gc_url_format('login'));
    	}
    	$this->user_model->resend_activation($user_id);
    }
    
    /**
     * OTP Verification
     */
    public function verify_otp($user_id = 0) {
    	$this->load->library('crypt');
    	$user_id = intval($this->crypt->decode($user_id));
    	if($user_id == 0) {
    		Template::redirect(gc_url_format('login'));
    	}
    	$this->user_model->verify_otp($user_id);
    }

	/**
	 * Users Detail Page
	 * @param string $slug
	 */
	public function detail($slug = null) {
	    if(empty($slug)) {
	        Template::redirect(site_url());
	    }
		$this->user_model->profile_detail('members', $slug);
	}	
	
	/**
	 * Common function for Social Login
	 * @param string hauth_done - GET name of the provider
	 */
	public function endpoint($provider = null) {
	    $provider = empty($provider) && !empty($_GET) ? $_GET['hauth_done'] : $provider;
	    $config = $this->user_model->get_hybrid_auth_config($provider);
	    if(!$config) {
	        Template::redirect(gc_url_format('login'));
	    }
	    try{
	        //Feed configuration array to Hybridauth
	        $hybridauth = new Hybridauth\Hybridauth($config);
	        
	        //Then we can proceed and sign in with Twitter as an example. If you want to use a diffirent provider,
	        //simply replace 'Twitter' with 'Google' or 'Facebook'.
	        
	   	        //Attempt to authenticate users with a provider by name
	        $adapter = $hybridauth->authenticate($provider);
	        
	        //Retrieve User's profile
	        $userProfile = $adapter->getUserProfile();
	        
	        //Disconnect the adapter
	        $adapter->disconnect();
	        
	        //A rare issue found with someones account with an expired email address
	        if($userProfile->email == NULL || empty($userProfile->email)) {
	            Template::set_message(lang('us_ha_no_email'), 'error');
	            Template::redirect(gc_url_format('login'));
	        }
	        
	        //Register provider and redirect to the Member Area
	        if ($this->user_model->is_unique('email', $userProfile->email) === TRUE) {
	            $this->user_model->register_provider_user($provider, $userProfile, gc_url_format('login'), gc_url_format('register'));
	        }
	        
	        // Get User
	        $user = $this->db->where(array('email' => $userProfile->email))->get('users')->row();
	        
	        //Redirect user to the desired page
	        if ($user && $this->auth->login('','',FALSE,$user) == TRUE) {
	            Template::redirect(gc_url_format('members'));
	        }
	        
	        // Redirect user to the login page
	        Template::redirect(gc_url_format('login'));
	        // Retrieve OAuth 1 / OAuth 2 Access Tokens
	        
	        // These access tokens can be stored to database and later used to restore user's session.
	        
	        // $accessToken = $adapter->getAccessToken();
	        //$this->session->set_userdata('provider_token', $accessToken);
	    } catch(\Exception $e){
	        $error = 'Unexpected error';
	        switch ($e->getCode()) {
	            case 0 : $error = lang('us_ha_error_unknown');
	            break;
	            case 1 : $error = lang('us_ha_error_configuration');
	            break;
	            case 2 : $error = lang('us_ha_error_provider_config');
	            break;
	            case 3 : $error = lang('us_ha_error_provider_unkown');
	            break;
	            case 4 : $error = lang('us_ha_error_provider_missing_credentials');
	            break;
	            case 5 : log_message('debug', lang('us_ha_error_connection_cancelled'));
	            if (isset($service))
	            {
	                log_message('debug', lang('us_ha_logout'));
	                $service->logout();
	            }
	            Template::set_message(lang('us_ha_cancelled') . $provider);
	            Template::redirect($this->register_url);
	            break;
	            case 6 : $error = lang('us_ha_error_user_profile_fail');
	            break;
	            case 7 : $error = lang('us_ha_error_user_not_connected');
	            break;
	            default:
	                $error = $e->getMessage();
	        }
	        
	        if (isset($service)) {
	            $service->logout();
	        }
	        
	        log_message('error', $error);
	        Template::set_message($error, 'danger');
	        Template::redirect(gc_url_format('register'));
	    }
	}
	
	/**
	 * Redirect user to the providers page
	 * @param string $provider
	 */
	public function hybrid_login($provider) {
	    Template::redirect(site_url('users/endpoint/' . $provider));
	}
	
	/**
	 * Two Factor Authentication Validate
	 */
	public function validate() {
	    $user_id = $this->session->userdata('tfa_user_id');
	    if(empty($user_id) || $this->auth->is_logged_in()) {
	        Template::redirect(site_url());
	    }
	    $this->user_model->twofactor_authentication($user_id);
	}
	
	/**
	 * Login with Temporary ID and Password
	 *
	 * Listing Email is used as the login id
	 * Listing ID is encrypted using a random string and provided as a password to the user
	 * The user can change the encryption_key string in the config file
	 * @return boolean
	 */
	public function signin() {
	    $this->load->library('crypt');
	    $this->load->library('auth');
	    $this->lang->load('users/users');
	    if(isset($_POST['save'])) {
	        $listing_verified = $this->session->userdata('listing_verified') ? $this->session->userdata('listing_verified') : false;
	        if(!$listing_verified) {
	            $this->form_validation->set_rules('email', 'lang:label_email', 'trim|required|valid_email');
	            $this->form_validation->set_rules('password', 'lang:bf_password', 'trim|required|listing_check');
	            if ($this->form_validation->run() === true) {
	                $email = $this->input->post('email');
	                $id = (int) $this->crypt->decode($this->input->post('password'));
	                // get listing information
	                $record = $this->db->select('id, user_id, email, contact_person, phone_number, mobile_number, city_id, state_id, country_iso, city, state, country')->get_where('listings', array('id' => $id, 'email' => $email))->row();
	                if(!$record) {
	                    // invalid username and password
	                    Template::set_message(lang('us_bad_email_pass'), 'danger');
	                    // Template::redirect(current_url());
	                } else {
	                    // check the existing user information
	                    // all such listings must have a user_id with role_id 1
	                    $user = $this->db->where('id', $record->user_id)->get('users')->row();
	                    if($user && $user->role_id != 1) { // if not admin redirect user to the login page
	                        // invalid username and password
	                        Template::set_message(lang('us_temp_login'), 'danger');
	                        Template::redirect(gc_url_format('login'));
	                    }
	                    // store value in a session variable and redirect user to the password reset page
	                    $this->session->set_userdata('listing_verified', $record);
	                }
	            }
	        } else {
	            // reset password form, create a new user and redirect user to the listing edit page
	            $this->form_validation->set_rules('password', 'lang:bf_password', 'xss_clean|required|max_length[120]|valid_password');
	            $this->form_validation->set_rules('pass_confirm', 'lang:bf_password_confirm', 'xss_clean|required|matches[password]');
	            if ($this->form_validation->run() === TRUE) {
	                $record = $listing_verified;
	                $data = array();
	                $password = $this->auth->hash_password($this->input->post('password'));
	                if (empty($password) || empty($password['hash'])) {
	                    return false;
	                }
	                // get the listing and insert a new user in database using listing's email, phone number and other details.
	                $data['role_id'] = 2;
	                $data['email'] = $record->email;
	                $data['username'] = $record->email;
	                $data['password_hash'] = $password['hash'];
	                $data['password_iterations']	= $password['iterations'];
	                $data['display_name'] = !empty($record->contact_person) ? $record->contact_person : $record->email;
	                $data['phone_number'] = !empty($record->mobile_number) ? $record->mobile_number : (!empty($record->phone_number) ? $record->phone_number : null);
	                $data['city_id'] = $record->city_id;
	                $data['state_id'] = $record->state_id;
	                $data['country_iso'] = $record->country_iso;
	                $data['city'] = $record->city;
	                $data['state'] = $record->state;
	                $data['country'] = $record->country;
	                $data['active'] = 1;
	                $data['last_login'] = date('Y-m-d H:i:s', time());
	                $data['last_ip'] = $this->input->ip_address();
	                // create user here
	                $this->db->where('id', $user_id)->insert('users', $data);
	                $user_id = (int) $this->db->insert_id();
	                if($user_id != 0) {
	                    // unset the listing verified session variable
	                    $this->session->unset_userdata('listing_verified');
	                    // assign a package to the user
	                    $package_id = $this->user_model->assign_package($user_id);
	                    
	                    // update data in the listings table
	                    $listing_data = array(
	                        'user_id' => $user_id,
	                        'package_id' => $package_id,
	                        'modified_on' => date('Y-m-d H:i:s'),
	                        'modified_ip' => $this->input->ip_address(),
	                        'modified_by' => $user_id
	                    );
	                    $this->db->where('id', $record->id)->update('listings', $listing_data);
	                    
	                    // setup session and redirect user to the listing edit page
	                    // Try to login
	                    $result = $this->auth->login($data['email'], $this->input->post('password'), true, FALSE, NULL, true);
	                    if ($result === TRUE) {
	                        $redirect_url = gc_url_format('members/edit_business') . '?id=' . $this->crypt->encode($record->id);
	                        // also show the listing delete option to the user
	                        Template::set_message(sprintf(lang('msg_save_success'), lang('bf_password')), 'success');
	                        Template::redirect($redirect_url);
	                    } else { // redirect user to the current url
	                        Template::redirect(current_url());
	                    }
	                } else {
	                    Template::set_message(sprintf(lang('msg_edit_failure'), lang('bf_password')), 'danger');
	                }
	            }
	        }
	    }
	    $record = array();
	    if($this->session->userdata('listing_verified')) {
	        $record = $this->session->userdata('listing_verified');
	        $this->user_model->password_hints();
	    }
	    Template::set(array(
	        'page_title' => lang('bf_action_login'),
	        'record' => $record
	    ));
	    $layout = $this->gc->load_assets('users/login');
	    Template::set_view('signin');
	    Template::render($layout['layout']);
	}
	
	/**
	 * Display user information to the visitor
	 */
	public function user() {
	    if(empty($_GET['id'])) {
	        Template::redirect(site_url());
	    }
	    $id = (int) $_GET['id'];
	    $module = isset($_GET['m']) ? $_GET['m'] : 'forums';
	    $this->user_model->profile_detail('members', null, $id);
	}
}

//end Users

/* Front-end Users Controller */
/* End of file users.php */

