<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Migration_Install_users extends Migration
{
	/**
	 * The name of the database table
	 *
	 * @var String
	 */
	private $table_name = 'users';

	/**
	 * The Listing table's fields
	 *
	 * @var Array
	 */
	private $fields = array(
		'id' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'auto_increment' => TRUE,
		),
		'role_id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => FALSE,
			'default'=> 4,
		),
		'email' => array(
			'type' => 'VARCHAR',
			'constraint' => 254,
			'null' => FALSE,
		),
		'username' => array(
			'type' => 'VARCHAR',
			'constraint' => 120,
			'null' => FALSE,
		),
		'password_hash' => array(
			'type' => 'CHAR',
			'constraint' => 60,
			'null' => FALSE,
		),
		'reset_hash' => array(
			'type' => 'VARCHAR',
			'constraint' => 40,
			'null' => FALSE,
		),
		'reset_by' => array(
			'type' => 'INT',
			'constraint' => 10,
			'null' => TRUE,
		),
		'display_name' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE,
		),
		'slug' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => FALSE,
		),
		'profile_image' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE,
		),
		'display_name_changed' => array(
			'type' => 'DATE',
			'null' => TRUE,
		),
		'description' => array(
			'type' => 'MEDIUMTEXT',
			'null' => TRUE,
		),
		'address' => array(
			'type' => 'VARCHAR',
			'constraint' => 250,
			'null' => TRUE,
		),
		'postcode' => array(
			'type' => 'VARCHAR',
			'constraint' => 10,
			'null' => TRUE,
		),
		'city_id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => FALSE,
			'default' => 0
		),
		'city' => array(
			'type' => 'VARCHAR',
			'constraint' => 40,
			'null' => TRUE,
		),
		'phone_number' => array(
			'type' => 'VARCHAR',
			'constraint' => 20,
			'null' => TRUE,
		),
		'website' => array(
			'type' => 'VARCHAR',
			'constraint' => 200,
			'null' => TRUE,
		),
		'linkedin_url' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE,
		),
		'facebook_url' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE,
		),
		'twitter_url' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE,
		),
		'googleplus_url' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE,
		),
		'youtube_url' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE,
		),
		'pinterest_url' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE,
		),
		'instagram_url' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE,
		),
		'timezone' => array(
			'type' => 'VARCHAR',
			'constraint' => 40,
			'null' => FALSE,
			'default' => 'UM6'
		),
		'language' => array(
			'type' => 'VARCHAR',
			'constraint' => 20,
			'null' => FALSE,
			'default' => 'english'
		),
		'wallets' => array(
			'type' => 'FLOAT',
			'null' => FALSE,
			'default' => 0
		),
		'points' => array(
			'type' => 'FLOAT',
			'null' => FALSE,
			'default' => 0
		),
		'active' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default' => 0
		),
		'activate_hash' => array(
			'type' => 'VARCHAR',
			'constraint' => 40,
			'null' => FALSE
		),
		'activate_otp' => array(
			'type' => 'INT',
			'constraint' => 6,
			'null' => FALSE,
			'default' => 0
		),
		'otp_attempts' => array(
			'type' => 'SMALLINT',
			'constraint' => 5,
			'null' => FALSE,
			'default' => 0
		),	
		'otp_expires_on' => array(
			'type' => 'DATETIME',
			'null'=> TRUE
		),
		'password_iterations' => array(
			'type' => 'INT',
			'constraint' => 4,
			'null' => FALSE,
			'default' => 0
		),
		'force_password_reset' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => TRUE,
			'default' => 0
		),
		'hauth_provider' => array(
			'type' => 'VARCHAR',
			'constraint' => 45,
			'null' => TRUE
		),
		'hits' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null'=> FALSE,
			'default' => '0',
		),
		'meta_title' => array(
			'type' => 'VARCHAR',
			'constraint' => 200,
			'null' => TRUE,
		),
		'meta_description' => array(
			'type' => 'TEXT',
			'null' => TRUE,
		),
		'meta_keywords' => array(
			'type' => 'TEXT',
			'null' => TRUE,
		),
		'banned' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null'=> FALSE,
			'default' => '0',
		),
		'banned_message' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE,
		),
		'deleted' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null'=> FALSE,
			'default' => '0',
		),
		'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'last_login' => array(
			'type' => 'DATETIME',
			'null'=> TRUE
		),
		'last_ip' => array(
			'type' => 'VARCHAR',
			'constraint' => 50,
			'null'=> TRUE
		)
	);
	
	/**
	 * Install this migration
	 *
	 * @return void
	 */
	public function up() {	
		$this->dbforge->add_field($this->fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('city_id',false);
		$this->dbforge->add_key('hits',false);
		$this->dbforge->add_key('postcode',false);
		$this->dbforge->add_key('created_on',false);
		$this->dbforge->create_table($this->table_name);
		$change_engine_sql = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " ENGINE = MyISAM";
		$this->db->query ($change_engine_sql);
		$index_sql = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " ADD INDEX `deleted_active` (`deleted`, `active`)";
		$this->db->query ($index_sql);
		$fulltext_sql = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " ADD FULLTEXT INDEX `name_description` (`display_name`, `description`)";
		$this->db->query ($fulltext_sql);
		// SET SLUG FIELD COLLATION TO utf8_bin
		$collation_sql = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " CHANGE `slug` `slug` VARCHAR( 255 ) CHARACTER SET utf8 COLLATE utf8_bin NOT NULL";
		$this->db->query ($collation_sql);
		// ADD DELETED_STATUS INDEX
		$idx_deleted_status = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " ADD INDEX `deleted_active` ( `deleted`, `active`)";
		$this->db->query ($idx_deleted_status);
	}

	//--------------------------------------------------------------------

	/**
	 * Uninstall this migration
	 *
	 * @return void
	 */
	public function down() {	
		if ($this->db->table_exists($this->table_name)) {
			$this->dbforge->drop_table($this->table_name);
		}
	}

	//--------------------------------------------------------------------

}