<?php

if (!defined('BASEPATH'))
    exit('No direct script access allowed');
/**
 * Bonfire
 *
 * An open source project to allow developers get a jumpstart their development of CodeIgniter applications
 *
 * @package   Bonfire
 * @author    Bonfire Dev Team
 * @copyright Copyright (c) 2011 - 2013, Bonfire Dev Team
 * @license   http://guides.cibonfire.com/license.html
 * @link      http://cibonfire.com
 * @since     Version 1.0
 * @filesource
 */
// ------------------------------------------------------------------------

/**
 * User Model
 *
 * The central way to access and perform CRUD on users.
 *
 * @package    Bonfire
 * @subpackage Modules_Users
 * @category   Models
 * @author     Bonfire Dev Team
 * @link       http://cibonfire.com
 */
class User_model extends BF_Model
{

    /**
     * Name of the table
     *
     * @access protected
     *
     * @var string
     */
    protected $table_name = 'users';

    /**
     * Name of the roles table
     *
     * @access protected
     *
     * @var string
     */
    protected $roles_table = 'roles';

    /**
     * Use soft deletes or not
     *
     * @access protected
     *
     * @var bool
     */
    protected $soft_deletes = TRUE;

    /**
     * The date format to use
     *
     * @access protected
     *
     * @var string
     */
    protected $date_format = 'datetime';

    /**
     * Set the created time automatically on a new record
     *
     * @access protected
     *
     * @var bool
     */
    protected $set_modified = FALSE;

    /**
     * @var Bool Optionally skip the validation.
     */
    protected $skip_validation = TRUE;

    /**
     * @var Array Validation rules
     */
    protected $validation_rules = array(
       /* array(
            'field' => 'password',
            'label' => 'lang:bf_password',
            'rules' => 'max_length[120]|valid_password|matches[pass_confirm]',
        ),
        array(
            'field' => 'pass_confirm',
            'label' => 'lang:bf_password_confirm',
            'rules' => '',
        ),*/
        array(
            'field' => 'display_name',
            'label' => 'lang:bf_display_name',
            'rules' => 'trim|max_length[255]',
        ),
        array(
            'field' => 'language',
            'label' => 'lang:bf_language',
            'rules' => 'required|trim',
        ),
        array(
            'field' => 'timezones',
            'label' => 'lang:bf_timezone',
            'rules' => 'required|trim|max_length[4]',
        ),
        array(
            'field' => 'username',
            'label' => 'lang:bf_username',
            'rules' => 'trim|max_length[30]',
        ),
        array(
            'field' => 'email',
            'label' => 'lang:bf_email',
            'rules' => 'required|trim|valid_email|max_length[120]',
        ),
        array(
            'field' => 'role_id',
            'label' => 'lang:us_role',
            'rules' => 'trim|max_length[2]|is_numeric',
        ),
    );

    /**
     * @var Array Additional Validation rules only used on insert
     */
    protected $insert_validation_rules = array(
        array(
            'field' => 'password',
            'label' => 'lang:bf_password',
            'rules' => 'required',
        ),
        array(
            'field' => 'pass_confirm',
            'label' => 'lang:bf_password_confirm',
            'rules' => 'required',
        ),
    );

    /**
     * @var array Metadata for the model's database fields
     */
    protected $field_info = array(
        array('name' => 'id', 'primary_key' => 1),
        array('name' => 'created_on'),
        array('name' => 'deleted'),
        array('name' => 'role_id'),
        array('name' => 'email'),
        array('name' => 'username'),
        array('name' => 'password_hash'),
        array('name' => 'reset_hash'),
        array('name' => 'last_login'),
        array('name' => 'last_ip'),
        array('name' => 'banned'),
        array('name' => 'ban_message'),
        array('name' => 'reset_by'),
        array('name' => 'display_name'),
        array('name' => 'display_name_changed'),
        array('name' => 'timezone'),
        array('name' => 'language'),
        array('name' => 'active'),
        array('name' => 'activate_hash'),
        array('name' => 'force_password_reset'),
    );

    //--------------------------------------------------------------------

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }//end __construct()
    
    /**
     * Email Template Fields
     *
     * @return array
     */
    public function email_templates($role = 'admin') {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'dropdown',
                'name' => 'module_table',
                'label' => 'label_module',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:gc_menu_users;value:users',
                'options_detailed' => 1,
                'options_lang' => 1,
                'class' => 3,
                'rules' => 'trim|unique_email_template[user]',
            ),
            array(
                'type' => 'dropdown',
                'name' => 'user_type',
                'label' => 'bf_user',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:placeholder_select;value:|name:label_admin;value:admin|name:bf_user;value:user',
                'options_detailed' => 1,
                'options_lang' => 1,
                'class' => 3,
                'rules' => 'trim',
            ),
            array(
                'type' => 'dropdown',
                'name' => 'action_type',
                'label' => 'label_action',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:placeholder_select;value:|name:add;value:add|name:edit;value:edit|name:delete;value:delete|name:forum_status_update;value:status_update|name:em_activation_code;value:activation_code|name:em_account_activated;value:account_activated|name:em_account_pending;value:account_pending|name:em_forgot_password;value:forgot_password|name:em_password_updated;value:password_updated',
                'options_detailed' => 1,
                'options_lang' => 1,
                'class' => 3,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'name',
                'label' => 'label_name',
                'placeholder' => 'placeholder_name',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
                'rules' => 'required|trim|max_length[50]'
            ),
            array(
                'type' => 'text',
                'name' => 'subject',
                'label' => 'label_subject',
                'placeholder' => 'placeholder_subject',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'required|trim|max_length[255]'
            ),
            array(
                'type' => 'textarea',
                'name' => 'message',
                'label' => 'label_message',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'custom_rules' => 'editor',
                'field_class' => 'editor',
                'class' => 9,
                'rules' => 'trim|required',
            ),
            array(
                'type' => 'textarea',
                'name' => 'sms_text',
                'label' => 'sms_text',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'class' => 9,
                'field_extra' => 'rows:3',
                'rules' => 'trim',
            ),
            array(
                'type' => 'textarea',
                'name' => 'params',
                'label' => 'label_params',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'class' => 9,
                'tooltip' => lang('emailer_params_help'),
                'field_extra' => 'rows:3',
                'rules' => 'trim',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'status',
                'label' => 'label_status',
                'frontend' => false,
                'admin_only' => false,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            )
        );
        return $fields;
    }
    
    //--------------------------------------------------------------------

    /**
     * Extracts the model's fields (except the key and those handled by
     * Observers) from the $post_data and returns an array of name => value pairs
     *
     * @param Array $post_data The post data, usually $this->input->post() when called from the controller
     *
     * @return Array    An array of name => value pairs containing the data for the model's fields
     */
    public function prep_data($post_data)
    {
        $data = parent::prep_data($post_data);

        if (!empty($post_data['timezones']))
        {
            $data['timezone'] = $post_data['timezones'];
        }
        if (!empty($post_data['password']))
        {
            $data['password'] = $post_data['password'];
        }
        if ($data['display_name'] === '')
        {
            unset($data['display_name']);
        }
        if (isset($post_data['restore']) && $post_data['restore'])
        {
            $data['deleted'] = 0;
        }
        if (isset($post_data['unban']) && $post_data['unban'])
        {
            $data['banned'] = 0;
        }
        if (isset($post_data['activate']) && $post_data['activate'])
        {
            $data['active'] = 1;
        }
        elseif (isset($post_data['deactivate']) && $post_data['deactivate'])
        {
            $data['active'] = 0;
        }

        return $data;
    }

    //--------------------------------------------------------------------

    /**
     * Helper Method for Generating Password Hints based on Settings library.
     *
     * Call this method in your controller and echo $password_hints in your view.
     *
     * @access public
     *
     * @return void
     */
    public function password_hints() {
        $min_length = (string) $this->settings_lib->item('auth.password_min_length');

        $message = sprintf(lang('bf_password_min_length_help'), $min_length);

        if ($this->settings_lib->item('auth.password_force_numbers') == 1) {
            $message .= '<br />' . lang('bf_password_number_required_help');
        }

        if ($this->settings_lib->item('auth.password_force_symbols') == 1) {
            $message .= '<br />' . lang('bf_password_symbols_required_help');
        }

        if ($this->settings_lib->item('auth.password_force_mixed_case') == 1) {
            $message .= '<br />' . lang('bf_password_caps_required_help');
        }

        Template::set('password_hints', $message);

        unset($min_length, $message);
    }

//end password_hints()
    //--------------------------------------------------------------------

    /**
     * Creates a new user in the database.
     *
     * Required parameters sent in the $data array:
     * * password
     * * A unique email address
     *
     * If no _role_id_ is passed in the $data array, it will assign the default role from <Roles> model.
     *
     * @access public
     *
     * @param array $data An array of user information.
     *
     * @return bool|int The ID of the new user.
     */
    public function insert($data = array())
    {
        // Display Name
        if (empty($data['display_name']))
        {
            if ($this->settings_lib->item('auth.use_usernames') == 1 && !empty($data['username']))
            {
                $data['display_name'] = $data['username'];
            }
            else
            {
                $data['display_name'] = $data['email'];
            }
        }

        $password = $this->auth->hash_password($data['password']);

        if (empty($password) || empty($password['hash']))
        {
            return false;
        }

        $data['password_hash'] = $password['hash'];
        $data['password_iterations']	= $password['iterations'];

        unset($data['password'], $password);

        // What's the default role?
        if (!isset($data['role_id']))
        {
            // We better have a guardian here
            if (!class_exists('Role_model'))
            {
                $this->load->model('roles/Role_model', 'role_model');
            }

            $data['role_id'] = $this->role_model->default_role_id();
        }

        $id = parent::insert($data);

        Events::trigger('after_create_user', $id);

        return $id;
    }

//end insert()
    //--------------------------------------------------------------------

    /**
     * Updates an existing user. Before saving, it will:
     * * generate a new password/salt combo if both password and pass_confirm are passed in.
     * * store the country code
     *
     * @access public
     *
     * @param int   $id   An INT with the user's ID.
     * @param array $data An array of key/value pairs to update for the user.
     *
     * @return bool TRUE/FALSE
     */
    public function update($id = null, $data = array())
    {
        if (empty($id))
        {
            return NULL;
        }
		
        $trigger_data = array('user_id' => $id, 'data' => $data);
        Events::trigger('before_user_update', $trigger_data);

        if (!empty($data['password']))
        {
            $password = $this->auth->hash_password($data['password']);

            if (empty($password) || empty($password['hash']))
            {
                return false;
            }

            $data['password_hash'] = $password['hash'];

            unset($data['password'], $password);
        }

        // Handle the country
        if (isset($data['iso']))
        {
            $data['country_iso'] = $data['iso'];
            unset($data['iso']);
        }

        $return = parent::update($id, $data);

        if ($return)
        {
            $trigger_data = array('user_id' => $id, 'data' => $data);
            Events::trigger('after_user_update', $trigger_data);
        }

        return $return;
    }

//end update()

    /**
     * For all users with the matching role, set them to the default role instead.
     *
     * @access public
     *
     * @return bool
     */
    public function set_to_default_role($current_role)
    {
        if (!is_int($current_role))
        {
            return FALSE;
        }

        // We better have a guardian here
        if (!class_exists('Role_model'))
        {
            $this->load->model('roles/Role_model', 'role_model');
        }

        $data = array();
        $data['role_id'] = $this->role_model->default_role_id();

        $query = $this->db->where('role_id', $current_role)
                ->update($this->table_name, $data);

        if ($query)
        {
            return TRUE;
        }

        return FALSE;
    }

//end set_to_default_role()
    //--------------------------------------------------------------------

    /**
     * Finds an individual user record. Also returns role information for the user.
     *
     * @access public
     *
     * @param int $id An INT with the user's ID.
     *
     * @return bool|object An object with the user's information.
     */
    public function find($id = null)
    {
        if (empty($this->selects))
        {
            $this->select($this->table_name . '.*, role_name');
        }

        $this->db->join($this->roles_table, $this->roles_table . '.role_id = ' . $this->table_name . '.role_id', 'left');

        return parent::find($id);
    }

//end find()
    //--------------------------------------------------------------------

    /**
     * Returns all user records, and their associated role information.
     *
     * @access public
     *
     * @return bool An array of objects with each user's information.
     */
    public function find_all()
    {
        if (empty($this->selects))
        {
            $this->select($this->table_name . '.*, role_name');
        }

        $this->db->join($this->roles_table, $this->roles_table . '.role_id = ' . $this->table_name . '.role_id', 'left');

        return parent::find_all();
    }

//end find_all()
    //--------------------------------------------------------------------

    /**
     * Locates a single user based on a field/value match, with their role information.
     * If the $field string is 'both', then it will attempt to find the user
     * where their $value field matches either the username or email on record.
     *
     * @access public
     *
     * @param string $field A string with the field to match.
     * @param string $value A string with the value to search for.
     * @param string $type  The type of where clause to create. Either 'and' or 'or'.
     *
     * @return bool|object An object with the user's info, or FALSE on failure.
     */
    public function find_by($field = null, $value = null, $type = 'and')
    {
        if (empty($this->selects))
        {
            $this->select($this->table_name . '.*, role_name');
        }

        $this->db->join($this->roles_table, $this->roles_table . '.role_id = ' . $this->table_name . '.role_id', 'left');

        return parent::find_by($field, $value, $type);
    }

//end find_by()
    //--------------------------------------------------------------------

    /**
     * Returns the number of users that belong to each role.
     *
     * @access public
     *
     * @return bool|array An array of objects representing the number in each role.
     */
    public function count_by_roles()
    {
        $this->db->select(array(
                    $this->roles_table . '.role_name',
                    'count(1) as count',
                ))
                ->from($this->table_name)
                ->join($this->roles_table, $this->roles_table . '.role_id = ' . $this->table_name . '.role_id', 'left')
                ->group_by($this->table_name . '.role_id');

        $query = $this->db->get();

        if ($query->num_rows())
        {
            return $query->result();
        }

        return FALSE;
    }

//end count_by_roles()
    //--------------------------------------------------------------------

    /**
     * Counts all users in the system.
     *
     * @access public
     *
     * @param bool $get_deleted If FALSE, will only return active users. If TRUE, will return both deleted and active users.
     *
     * @return int An INT with the number of users found.
     */
    public function count_all($get_deleted = FALSE)
    {
        if ($get_deleted)
        {
            // Get only the deleted users
            $this->db->where($this->table_name . '.deleted !=', 0);
        }
        else
        {
            $this->db->where($this->table_name . '.deleted', 0);
        }

        return $this->db->count_all_results($this->table_name);
    }

//end count_all()
    //--------------------------------------------------------------------

    /**
     * Performs a standard delete, but also allows for purging of a record.
     *
     * @access public
     *
     * @param int  $id    An INT with the record ID to delete.
     * @param bool $purge If FALSE, will perform a soft-delete. If TRUE, will permanently delete the record.
     *
     * @return bool TRUE/FALSE
     */
    public function delete($id = 0, $purge = FALSE)
    {
        if ($purge === TRUE)
        {
            // temporarily set the soft_deletes to TRUE.
            $this->soft_deletes = FALSE;
        }

        return parent::delete($id);
    }

//end delete()
    //--------------------------------------------------------------------

    /**
     * Flags one or more user accounts to require their password to
     * be reset on their next login.
     *
     * @param  int $user_id
     *
     * @return bool TRUE/FALSE
     */
    public function force_password_reset($user_id = null)
    {
        if (!empty($user_id) && is_numeric($user_id))
        {
            $this->db->where('id', $user_id);
        }

        return $this->db->set('force_password_reset', 1)->update($this->table_name);
    }

    //--------------------------------------------------------------------
    //--------------------------------------------------------------------
    // !AUTH HELPER METHODS
    //--------------------------------------------------------------------

    /**
     * Generates a new password hash for the given password.
     *
     * @access public
     *
     * @param string $old The password to hash.
     * @param int	 $iterations	The number of iterations to use in generating the hash
     *
     * @return array An array with the hashed password and the number of iterations, or false
     */
    public function hash_password($old = '', $iterations = 0)
    {
        $password = $this->auth->hash_password($old, $iterations);

        if (empty($password) || empty($password['hash']))
        {
            return false;
        }

        return array($password['hash'], $password['iterations']);
    }

//end hash_password()
    //--------------------------------------------------------------------

    /**
     * Create a salt (no longer used for passwords)
     *
     * @access private
     *
     * @return string A random string of 7 characters
     */
    private function generate_salt()
    {
        if (!function_exists('random_string'))
        {
            $this->load->helper('string');
        }

        return random_string('alnum', 7);
    }

//end generate_salt()
    //--------------------------------------------------------------------
    //--------------------------------------------------------------------
    // !HMVC METHOD HELPERS
    //--------------------------------------------------------------------

    /**
     * Returns the most recent login attempts and their description.
     *
     * @access public
     *
     * @param int $limit An INT which is the number of results to return.
     *
     * @return bool|array An array of objects with the login information.
     */
    public function get_login_attempts($limit = 15)
    {
        $this->db->limit($limit);
        $this->db->order_by('login', 'desc');
        $query = $this->db->get('login_attempts');

        if ($query->num_rows())
        {
            return $query->result();
        }

        return FALSE;
    }

//end get_login_attempts()
    //--------------------------------------------------------------------
    //--------------------------------------------------------------------
    // !ACTIVATION
    //--------------------------------------------------------------------

    /**
     * Count Inactive users.
     *
     * @access public
     *
     * @return int Inactive user count.
     */
    public function count_inactive_users()
    {
        $this->db->where('active', -1);
        return $this->count_all(FALSE);
    }

//end count_inactive_users()

    /**
     * Accepts an activation code and validates is against a matching entry int eh database.
     *
     * There are some instances where we want to remove the activation hash yet leave the user
     * inactive (Admin Activation scenario), so leave_inactive handles this use case.
     *
     * @access public
     *
     * @param int    $user_id        The user to be activated (NULL will match any)
     * @param string $code           The activation code to be verified
     * @param bool   $leave_inactive Flag whether to remove the activate hash value, but leave active = 0
     *
     * @return int User Id on success, FALSE on error
     */
    public function activate($user_id, $code, $leave_inactive = FALSE) {
        if ($user_id) {
            $this->db->where('id', $user_id);
        }        
	    $activation_method = $this->settings_lib->item('auth.user_activation_method');
		if($activation_method == 3) {
			$result = $this->db->select('id, activate_otp, otp_expires_on')->get('users')->row();
			if($result) {
				if($result->activate_otp == 0 || $result->activate_otp != $code) {
					$this->error = lang('us_err_no_matching_code') . '. ' . lang('us_err_activate_code');
					return FALSE;
				} else if($result->activate_otp == $code && $result->otp_expires_on < date('Y-m-d H:i:s')) {
					$this->error = lang('otp_expired');
					return FALSE;
				}
			} else {
				$this->error = lang('us_err_no_matching_code') . '. ' . lang('us_err_activate_code');
				return FALSE;
			}
		} else {
			$query = $this->db->select('id')
                ->where('activate_hash', $code)
                ->get($this->table_name);
			if ($query->num_rows() !== 1) {
				$this->error = lang('us_err_no_matching_code') . '. ' . lang('us_err_activate_code');
				return FALSE;
			}
			$result = $query->row();
		} 
		
        // Now we can find the $user_id, even if it was passed as NULL
        $user_id = $result->id;

        $active = ($leave_inactive === FALSE) ? 1 : 0;
        if ($this->update($user_id, array('activate_hash' => '', 'activate_otp' => 0, 'active' => $active))) {
            return $user_id;
        }
    }

//end activate()

    /**
     * This function is triggered during account set up to assure user is not active and,
     * if not supressed, generate an activation hash code. This function can be used to
     * deactivate accounts based on public view events.
     *
     * @param int    $user_id    The username or email to match to deactivate
     * @param string $login_type Login Method
     * @param bool   $make_hash  Create a hash
     *
     * @return mixed $activate_hash on success, FALSE on error
     */
    public function deactivate($user_id, $make_hash = TRUE)
    {
        // create a temp activation code.
        $activate_hash = '';
        if ($make_hash === true)
        {
            $this->load->helpers(array('string', 'security'));
            $activate_hash = do_hash(random_string('alnum', 40) . time());
        }

        $this->db->update($this->table_name, array('active' => 0, 'activate_hash' => $activate_hash), array('id' => $user_id));

        if ($this->db->affected_rows() != 1)
        {
            return FALSE;
        }

        return $make_hash ? $activate_hash : TRUE;
    }

//end deactivate()

    /**
     * Admin specific activation function for admin approvals or re-activation.
     *
     * @access public
     *
     * @param int $user_id The user ID to activate
     *
     * @return bool TRUE on success, FALSE on error
     */
    public function admin_activation($user_id = FALSE) {

        if ($user_id === FALSE) {
            $this->error = lang('us_err_no_id');
            return FALSE;
        }

        $query = $this->db->select('id')
                ->where('id', $user_id)
                ->limit(1)
                ->get($this->table_name);

        if ($query->num_rows() !== 1) {
            $this->error = lang('us_err_no_matching_id');
            return FALSE;
        }

        $result = $query->row();
        $this->update($result->id, array('activate_hash' => '', 'activate_otp' => 0, 'active' => 1));

        if ($this->db->affected_rows() > 0) {
            return $result->id;
        } else {
            $this->error = lang('us_err_user_is_active');
            return FALSE;
        }
    }

//end admin_activation()

    /**
     * Admin only deactivation function.
     *
     * @access public
     *
     * @param int $user_id The user ID to deactivate
     *
     * @return bool TRUE on success, FALSE on error
     */
    public function admin_deactivation($user_id = FALSE)
    {
        if ($user_id === FALSE)
        {
            $this->error = lang('us_err_no_id');
            return FALSE;
        }

        if ($this->deactivate($user_id, FALSE))
        {
            return $user_id;
        }
        else
        {
            $this->error = lang('us_err_user_is_inactive');
            return FALSE;
        }
    }

//end admin_deactivation()
    //--------------------------------------------------------------------

    /**
     * Configure activation for the given user based on current user_activation_method
     *
     * @param Number $user_id User's ID
     *
     * @return Array    An array containing a 'message' (String) and 'error' (Boolean, true if an error occurred sending the activation email)
     */
    public function set_activation($user_id)
    {
        // User activation method
        $activation_method = $this->settings_lib->item('auth.user_activation_method');

        // Prepare user messaging vars
        $emailMsgData = array();
        $emailView = '';
        $subject = '';
        $email_mess = '';
        $message = lang('us_email_thank_you');
        $type = 'success';
        $site_title = $this->settings_lib->item('site.title');
        $error = false;

        switch ($activation_method)
        {
            case 0:
                // No activation required. Activate the user and send confirmation email
                $subject = str_replace('[SITE_TITLE]', $this->settings_lib->item('site.title'), lang('us_account_reg_complete'));
                $emailView = '_emails/activated';
                $message .= lang('us_account_active_login');

                $emailMsgData = array(
                    'title' => $site_title,
                    'link' => site_url(),
                );
                break;

            case 1:
                // Email Activiation.
                // Run the account deactivate to assure everything is set correctly
                $activation_code = $this->deactivate($user_id);
                // Create the link to activate membership
                $activate_link = site_url(gc_uri_string('activate') . '/' . $user_id . '/' . $activation_code);
                $subject = lang('us_email_subj_activate');
                $emailView = '_emails/activate';
                $message .= lang('us_check_activate_email');

                $emailMsgData = array(
                    'title' => $site_title,
                    'code' => $activation_code,
                    'link' => $activate_link
                );
                break;

            case 2:
                // Admin Activation
                $subject = lang('us_email_subj_pending');
                $emailView = '_emails/pending';
                $message .= lang('us_admin_approval_pending');

                $emailMsgData = array(
                    'title' => $site_title,
                );
                break;
        }//end switch

        $email_mess = $this->load->view($emailView, $emailMsgData, true);

        // Now send the email
        $this->load->library('emailer/emailer');
        $data = array(
            'to' => $this->find($user_id)->email,
            'subject' => $subject,
            'message' => $email_mess,
        );

        if (!$this->emailer->send($data))
        {
            $message .= lang('us_err_no_email') . $this->emailer->error;
            $error = true;
        }

        return array('message' => $message, 'error' => $error);
    }

    //--------------------------------------------------------------------
    //**HybridAuth Methods**//
    //--------------------------------------------------------------------

    protected $table_hybridauth = 'hybrid_auth_settings';


    public function get_config() {
		$this->load->library('goclixy/gcache');
        $result = $this->gcache->get($this->table_hybridauth, array('result' => 1));
        $return_array['providers'] = array();
        foreach ($result as $provider) {
            $return_array['providers'][$provider->provider] = array(
                    'enabled' => ($provider->enabled == 1 ? TRUE : FALSE )
            );
            if($provider->key !== "NULL") {
                $return_array['providers'][$provider->provider]['keys'] = array('id' => $provider->key, 'key' => $provider->key, 'secret' => $provider->secret);
            }
            if($provider->scope !== "NULL") {
                $return_array['providers'][$provider->provider]['scope'] = $provider->scope;
            }
        }
        return $return_array;
    }
    
    /**
     * Get HybridAuth Providers ID and Secret Keys
     * @param string $provider - Name of the provider e.g. Facebook/Google/Twitter etc.
     */
    public function get_hybrid_auth_config($provider) {
        // Load Zend Caching
        $this->load->library('slug');
        $this->load->library('gzf_cache', array('lifetime' => 900));
        if(method_exists($this->gzf_cache, 'get_instance')) {
            $this->gc_users_cache = $this->gzf_cache->get_instance();
        }
        $cacheID = 'hybrid_auth_providers'; // Cache giving errors when invalid url with spaces and special characters
        if(!$result = $this->gc_users_cache->load($cacheID)) {
            $providers = $this->user_model->get_providers();
            // IMPORTANT - index.php must be there in the Callback URL, else looping will go on without success
            $config = array(
                'providers' => array()
            );
            if(!empty($providers)) {
                foreach($providers as $key => $value) {
                    if($value['enabled'] == 1) {
                        $config['providers'][$key] = array(
                            'enabled' => true,
                            'keys' => array('key' => $value['keys']['key'], 'secret' => $value['keys']['secret'])
                        );
                    }
                }
            }
            $result = array('config' => $config);
            $this->gc_users_cache->save($result, $cacheID, array('hybrid_auth_providers'));
        }
        // get configurations from the cache variable
        $config = $result['config'];
        if(empty($config)) {
            return false;
        }
        // provider does not exist or disabled
        if(empty($config['providers'][$provider])) {
            return false;
        }
        $config['callback'] = site_url('users/endpoint/' . $provider); //dump(Hybridauth\HttpClient\Util::getCurrentUrl());
        return $config;
    }
    
    /**
     * Get All Hybrid Auth Providers
     */
    public function get_providers() {
        $result = $this->db->get($this->table_hybridauth)->result();
        $return_array = array();
        foreach ($result as $provider) {
            $return_array[$provider->provider] = array('enabled' => ($provider->enabled == 1 ? TRUE : FALSE ));
            if($provider->key !== "NULL") {
                $return_array[$provider->provider]['keys'] = array('id' => $provider->key, 'key' => $provider->key, 'secret' => $provider->secret);
            }
            if($provider->scope !== "NULL") {
                $return_array[$provider->provider]['scope'] = $provider->scope;
            }
        }
        return $return_array;
    }
    
    public function update_hybridauth($data) {
        $s = TRUE;
        foreach ($data as $provider => $values) {
            $this->db->where(array('provider'=>$provider));
            $this->db->update($this->table_hybridauth, $values);
            if($s === TRUE) {
                if($this->db->affected_rows() <= 0) {
                    $s = FALSE;
                }
            }
        }
        return $s;
    }
    
    /**
     * Activate User via OTP Method
     */
    public function otp($user_id) {
    	$six_digit_random_number = mt_rand(100000, 999999); // substr(number_format(time() * rand(),0,'',''),0,6); // Alternate Method
    	$this->db->update($this->table_name, array('active' => 0, 'activate_otp' => $six_digit_random_number, 'otp_attempts' => 1, 'otp_expires_on' => $this->otp_expiry()), array('id' => $user_id));
    	return $six_digit_random_number;
    }
    
    /**
     * Function to find and print OTP expiry
     */
    public function otp_expiry() {
    	$date = date("Y-m-d H:i:s");
    	$currentDate = strtotime($date);
    	$minutes = settings_item('auth.otp_timeout');
    	$futureDate = $currentDate+(60*$minutes);
    	$formatDate = date("Y-m-d H:i:s", $futureDate);
    	return $formatDate;
    }
    
    //--------------------------------------------------------------------
    
    /**
     * Hybrid Auth User Registration
     * @param string $provider - facebook/google
     * @param string $user - user information
     * @param boolean $mobile - TRUE/FALSE
     */
    public function register_provider_user($provider, $user, $login_url = null, $register_url = null, $mobile = false) {
    	$this->load->helpers(array('email', 'string', 'security'));
    	//Checks for username and email to get the user into our system. The key is the email
    	//as the password and username can be made up as the login auth will not attempt any login
    	//if they user a provider
    	if (!isset($user->email) || !valid_email($user->email)) {
    		if($mobile) {
    			$message = array(
    				'success' => false,
    				'message' => lang('us_ha_invalid_email'),
    				'redirect' => 'signup'
    			);    			
    		} else {
	    		Template::set_message(lang('us_ha_invalid_email'), 'error');
	    		Template::redirect($register_url);
    		}
    	}
    	if (!$this->user_model->is_unique('email', $user->email)) {
    		if($mobile) {
    			$message = array(
    				'success' => false,
    				'message' => lang('us_ha_email_not_unique'),
    				'redirect' => 'login'
    			);
    		} else {
    			Template::set_message(lang('us_ha_email_not_unique'), 'error');
    			Template::redirect($login_url);
    		}
    	}
    	
    	$username = $user->displayName;
    	if (empty($username)) {
    		$username = $user->firstName;
    	}
    	if (empty($username)) {
    		$username = random_string('alpha', 8);
    	}
    	
    	if (!$this->user_model->is_unique('username', $username)) {
    		$index = 0;
    		for (; $index < 10; $index++) {
	    		$username = random_string('alpha', 8);
	    		if (!$this->user_model->is_unique('email', $user->email)) {
	    			break;
	    		}
    		}
    		if ($index >= 10) {
    			if($mobile) {
    				$message = array(
    					'success' => false,
    					'message' => lang('us_ha_username_fail')
    				);
    			} else {
    				Template::set_message(lang('us_ha_username_fail'), 'error');
    				return;
    			}    			
    		}
    	}
    	// Create Slug
    	$this->load->library('slug');
    	$data = array(
        	    'email' => $user->email,
        	    'slug' => $this->slug->create_slug($user->email),
    			'password' => random_string('alnum', 16),
	    		'username' => $user->email,
	    		'display_name' => $user->displayName,
	    		'profile_image' => $user->photoURL,
	    		'phone_number' => $user->phone,
    			'hauth_provider' => $provider,
	    		'language' => settings_item('site.default_language'),
    			'active' => 1
    	);
	    
    	if ($user_id = $this->user_model->insert($data)) {
    	    
    	    // Assign default package to the Hybrid Auth User
    	    $this->assign_package($user_id);
    	    
    		// User Activation
    		/*$activation = $this->user_model->set_activation($user_id);
    		$message = $activation['message'];
    		$error = $activation['error'];
    
    		$type = $error ? 'error' : 'success';
    		Template::set_message($message, $type);
    
    		// Log the Activity
    		log_activity($user_id, lang('us_log_register'), 'users');
    		$this->auth->login();
    		Template::redirect($login_url);*/
    	    $user_info = $this->db->where('id', $user_id)->get('users')->row();
    	    if ($this->auth->login('','',FALSE,$user_info) == TRUE) {
    			if($mobile) { // Login Success
	    			$message = array(
	    				'success' => true
	    			);
    			} else {
    				Template::redirect(gc_url_format('members'));
    			}
    		}
    	} else {
    		if($mobile) {
    			$message = array(
    					'success' => false,
    					'message' => lang('us_registration_fail'),
    					'redirect' => 'signup'
    			);
    		} else {
    			Template::set_message(lang('us_registration_fail'), 'error');
    			Template::redirect($register_url);
    		}
    	}//end if
    	if($mobile) {
    		return $message;
    	}
    }
    
    /**
     * EMAILER - Get user information for email content
     * @param number $id
     */
    public function emailer_users($id = 0) {
    	$sql = "SELECT U.id, U.email, U.username, U.display_name, U.profile_image, U.address, U.postcode, U.phone_number,";
    	$sql .= " CT.name AS city, S.name AS state, C.name AS country";
    	$sql .= " FROM " . $this->db->dbprefix . "users U";
    	$sql .= " LEFT JOIN " . $this->db->dbprefix . "cities CT ON CT.id = U.city_id";
    	$sql .= " LEFT JOIN " . $this->db->dbprefix . "states S ON S.id = CT.state_id";
    	$sql .= " LEFT JOIN " . $this->db->dbprefix . "countries C ON C.iso = S.country_iso";
    	$sql .= " WHERE U.deleted = 0 AND U.banned = 0 AND U.active = 1";
    	if($id != 0) {
    		$sql .= " AND U.id = " . $id;
    	}
    	return $this->db->query ( $sql )->result();
    }
    
	/**
	 * Select Users - 12/02/17
	 * @param string $isEdit
	 * @param array $users_array
	 * @param string $field_name
	 */
	public function users_selection($isEdit = false, $users_array = array(), $field_name = 'applicable_users') {
		$selected_users = array();
		$users = $this->db->select("id, CONCAT(display_name, ' (' , email , ')') AS email")->get('users')->result_array();
		$all_users[] = array(
				'id' => 'all',
				'email' => lang('all_users')
		);
		Template::set('users', array_merge($all_users, $users));
		if($this->input->post($field_name)) { // Post handling
			Template::set($field_name, $this->selected_users($this->input->post($field_name)));
		} else if(empty($_POST) && $isEdit) {
			Template::set($field_name, $this->selected_users($users_array));
		} else if($field_name == 'applicable_users') { // set default value to ALL USERS
			Template::set($field_name, $this->selected_users('all'));
		}
	}
	
	/**
	 * Users selected via POSTS/EDIT Page
	 * @param array $users
	 * @return <multitype:, multitype:string , unknown>
	 */
	private function selected_users($users) {
		$selected_users = array();
		$users_array = explode(',',$users ?? '');
		// User selected all users
		if(in_array('all', $users_array)) {
			$selected_users  = array(
					'id' => 'all',
					'name' => lang('all_users')
			);
		} else if(!empty($users)) { // User selected only few users
			$sql = "SELECT id, CONCAT(display_name, ' (' , email , ')') AS email FROM " . $this->db->dbprefix . "users where id in (" . $users . ")";
			$result = $this->db->query($sql)->result_array();
			if($result) {
				$selected_users = $result;
			}
		}
		return $selected_users;
	}
	
	/**
	 * Assign Default Package to the user on registration
	 * @param int $user_id
	 */
	public function assign_package($user_id) {
		$this->load->model('packages/packages_model');
		$package = $this->packages_model->select('id')->find_by(array('is_user_default' => 1, 'deleted' => 0));
		if(!$package) {
		    $package = $this->db->select('id, price, offer_price')->where(array('deleted' => 0))->order_by('price')->get('packages')->row();
		    if($package && $package->price > 0) {
		        return false;
		    }
		}
		if($package) {
		    
		    // Automatic Assign Free Package to the user
		    $this->load->model('orders/orders_model');
		    $this->orders_model->checkout_params($user_id, false, array(), true);
		    $this->orders_model->cart($package->id, 0, 1, 1, 'packages', true, array('zero_checkout' => true), $user_id); // assign package to the user
		    return $package->id;
		} else {
		    return 0;
		}
	}
	
	/**
	 * Handle User's Profile through the Fields Module
	 * To get rid from two different form changing for a single change
	 * @param string $role
	 * @param integer $user_id
	 * @param integer $id
	 */
	public function add_edit_user($role, $user_id, $id = 0) {
		$module = 'users';
		$this->load->model('fields/fields_model');
		$this->lang->load('listings/listings');
		$record = array(); // load fields 4th parameter
		$redirect_url = ($role == 'admin' ? site_url(SITE_AREA . '/settings/users') : current_url());
		// Member Area
		$frontend = FALSE;
		$mobile = FALSE;
		if($role == 'members' || $role == 'mobile') {
			$frontend = TRUE;
			if($role == 'mobile') {
				$mobile = TRUE;
			}
		}
	
		// Pass $module as a parameter to the View File
		if($role == 'members' || $role == 'admin') {
			Template::set('module', $module);
		}
	
		// Fields like user_id, package_id, module_id, row_id can be handled via Hidden Fields
		// but they will impose security risk as the external user can modify these values before submitting the form
		// therefore, such parameters are passed in the extra_fields array
		$extra_fields = array();
	
		// DB Transaction Type
		if($id == 0) {
			$action_type = 'create'; // For module links, Member Area
			$transaction_type = 'insert';
		} else {
			$action_type = 'edit'; // For module links, Member Area
			$where = array($module . '.id' => $id);
			if($frontend) {
				$where[$module . '.deleted'] = 0;
			}
			// Get existing record and pass it to the View File
			$record = $this->fields_model->find_row_and_meta($module, $id, $where);
			if(!$record) {
				if($role == 'mobile') {
					return array('success' => FALSE);
				} else {
					Template::redirect($redirect_url);
				}
			} else {
				if(!$mobile) {
					Template::set('record', $record);
				}
			}
			$transaction_type = 'update';
		}
	
		if($role == 'admin') {
		    // A normal user should not be able to assign Administrator role to another user.
		    $user_role_id = $this->auth->role_id();
		    $where_roles = array('deleted' => 0);
		    if($user_role_id != 1) {
		        if(!empty($record) && $record->role_id == 1) {
		            // do not allow normal users to edit Site Admins
		            Template::set_message(lang('msg_permission_error'), 'danger');
		            Template::redirect($redirect_url);
		        }
		        $where_roles['role_id != '] = 1;
		    }
		    if($user_role_id != 1 && $record->user_id = $this->auth->user_id()) {
		        // the user should not be able to change his/her own role except admin
		        $where_roles['role_id'] = 2;
		    }
			$this->load->model('roles/role_model');
			$roles = $this->role_model->select('role_id AS id, role_name AS title')
			->where($where_roles)
			->order_by('role_name', 'desc')
			->find_all();
			Template::set('roles', $roles);
		}
		$site_languages = unserialize(settings_item('site.languages'));
		$languages = array();
		if(!empty($site_languages)) {
			foreach($site_languages as $site_language) {
				$language = array(
						'id' => $site_language,
						'title' => ucwords($site_language)
				);
				$languages[] = (object) $language;
			}
		}
		// Do not change user_languages to languages else Member Area edit will throw errors
		Template::set('user_languages', $languages);
		
		// Allow user to select a Timezone
		$this->load->helper('date');
		$this->lang->load('date');
		$timezones = array();
		foreach (timezones() as $key => $val) {
			$timezone = array(
					'id' => $key,
					'title' => $this->lang->line($key)
			);
			$timezones[] = (object) $timezone;
		}
		if(!empty($record) && empty($record->timezone)) {
			$record->timezone = strtoupper(settings_item('site.default_user_timezone'));
		}
		Template::set('timezones', $timezones);
	
		// Username Required or Not View Page
		if (settings_item('auth.login_type') == 'username' || settings_item('auth.use_usernames')) {
			$view_params['usernameRequired'] = TRUE;
		}
	
		// Load password strength JS to display warning messages
		if (settings_item('auth.password_show_labels') == 1) {
			Assets::add_module_js('users', 'password_strength.js');
			Assets::add_module_js('users', 'jquery.strength.js');
			Assets::add_js(
				$this->load->view('users_js', array(), true),
				'inline'
			);
		}
		
		// Image help
		Template::set('help_image', sprintf(lang('image_help'), settings_item('lst.image_file_size'), settings_item('lst.image_width'), settings_item('lst.image_height')));
		
		// Load Fields
		$form_fields = $this->fields_model->load_fields($module, array(), ($role == 'admin' ? 'admin_only' : 'frontend'), $record, $id, $mobile);
		
		// Save Method
		if (isset($_POST['save']) || isset($_POST['save_only'])) {
		    $this->gcrud->stats(array('module' => 'users'));
			if ($result = $this->fields_model->save_module($form_fields, $module, $user_id, $role, $id, $transaction_type, $extra_fields)) {
				$insert = false;
				if($id == 0) { // Insert Operation
					$id = $result;
					$insert = true;
				}
				
				// Update force password reset if the user changes the password
				if(!$insert && $role != 'admin' && $this->input->post('password')) {
					$this->db->where('id', $id)->update('users', array('force_password_reset' => 0));
				}
	
				log_activity($user_id, lang('act_create_record') .': '. $id .' : '. $this->input->ip_address(), $module);
				if($insert && $frontend) {
					// Earn Points
					$this->gc->earn_points($module, $user_id, $id);
						
					$activationMethod = $this->settings_lib->item('auth.user_activation_method');
					// No activation method
					if ($activationMethod == 0) {
						// Activate the user automatically
						$data['active'] = 1;
					}
				}
				$success_message = sprintf(lang('msg_save_success'), lang('singular_' . $module));
				if($role == 'mobile') {
					return array('success' => TRUE, 'message' => $success_message);
				} else {
					Template::set_message($success_message, 'success');
					if(!empty($_POST['save_only'])) {
					    Template::redirect(current_url());
					} else {
    					if($role == 'admin' && $this->auth->role_id() != 1) {
    					    // staff and other users login
    					    Template::redirect(current_url());
    					} else {
    					    Template::redirect($redirect_url);
    					}
					}
				}
			} else {
			    $error_message = sprintf(lang($transaction_type == 'update' ? 'msg_edit_failure' : 'msg_create_failure'), lang('singular_' . $module));
				if($role == 'mobile') {
					$validation_errors = validation_errors();
					if ($validation_errors) {
						$error_message = $validation_errors;
					}
					return array('success' => FALSE, 'message' => $error_message);
				} else {
					Template::set_message($error_message, 'error');
				}
			}
		}
		$view_params = array();
		$view_params['image_help'] = sprintf(lang('image_help'), settings_item('lst.image_file_size'), settings_item('lst.image_width'), settings_item('lst.image_height'));
		if($role == 'admin') {
			$toolbar_title = $id == 0 ? lang('us_create_user') : lang('us_edit_user');
		} else {
			$toolbar_title = lang('us_edit_profile');
			$view_params['page_title'] = $toolbar_title;
			$view_params['title'] = $toolbar_title;
		}
		$view_params['toolbar_title'] = $toolbar_title;
		$view_params['image_path'] = base_url() .'assets/images/users/';
		Template::set($view_params);
		Template::set_view('fields/form');
		if($role == 'members') {
			Template::set(array(
				'member_area' => true,
			    'extra_actions' => array(anchor(gc_url_format('users', $record->slug, $record->id), lang('label_view'), 'class="btn btn-secondary btn-theme-tertiary" target="_blank"')),
			    'disable_cancel_btn' => true,
				'elements_container' => 'div.card.card-body'
			));
			
			// This method has no View in the Themes Module, i.e. load layout files manually
			$this->gc->demo_custom_css_js('member');
		}
		$render = ($role == 'members' ? 'member' : '');
		Template::render($render);
	}
	
	/**
	 * Mobile App/Member Area - Allow visitor to view user profile
	 * @param string $role: 'members'
	 * @param string $slug
	 */
	public function profile_detail($role, $slug, $id = 0) {
		$this->lang->load('listings/listings');
		$module = 'users';
		$view_data = array(); // Data to be passed to the View
		// get all characters from right to left till first dash to get listing id
		if($role == 'mobile') {
			$id = $slug;
		} else {
		    $id = (int) $id;
			$id = !empty($id) ? $id : (int)substr( $slug, strrpos( $slug, '-' ) + 1);
		}		
		if(settings_item('site.enable_cache') == 1) {
			$this->gc_cache = $this->gzf_cache->get_instance();
			$cacheID = str_replace('-', '_', $slug); // Cache giving errors when invalid url with spaces and special characters
			if(!$result = $this->gc_cache->load($cacheID)) {
			    $result = $this->user_info($id, $role);
			    $this->gc_cache->save($result, $cacheID, array('users'));				
			}
		} else {
		    $result = $this->user_info($id, $role);
		}		
		$record = $result['record'];
		if(empty($record)) {
			Template::redirect(site_url());
		}
		unset($record->password_hash);
		unset($record->reset_hash);
		$actual_slug = $record->slug . '-' . $record->id;
		if($role == 'members' && ($slug != $actual_slug)) {
			Template::redirect(gc_url_format('users', $record->slug, $record->id, $record->city));
		}
		
		// Display user related module information		
		Template::set('module_info', $result['module_info']);
		
		// update user profile hits
		$this->gc->updateHits($id, $module);
		
		// add/update recently viewed status
		$this->gc->add_recently_viewed($id, $module, $id);
		
		// Prepare Social Links
		$social_links = array();
		if(!empty($record->website)) {
			$social_links[] = array(
					'icon' => $role == 'mobile' ? 'earth' : 'globe',
					'lang' => lang('label_website'),
					'link' => $record->website
			);
		}
		if(!empty($record->linkedin_url)) {
			$social_links[] = array(
					'icon' => 'linkedin',
					'lang' => lang('label_linkedin_url'),
					'link' => $record->linkedin_url
			);
		}
		if(!empty($record->facebook_url)) {
			$social_links[] = array(
					'icon' => 'facebook',
					'lang' => lang('label_facebook_url'),
					'link' => $record->facebook_url
			);
		}
		if(!empty($record->twitter_url)) {
			$social_links[] = array(
					'icon' => 'twitter',
					'lang' => lang('label_twitter_url'),
					'link' => $record->twitter_url
			);
		}
		if(!empty($record->googleplus_url)) {
			$social_links[] = array(
					'icon' =>  $role == 'mobile' ? 'googleplus' : 'google-plus',
					'lang' => lang('label_googleplus_url'),
					'link' => $record->googleplus_url
			);
		}
		if(!empty($record->youtube_url)) {
			$social_links[] = array(
					'icon' => 'youtube',
					'lang' => lang('label_youtube_url'),
					'link' => $record->youtube_url
			);
		}
		if(!empty($record->pinterest_url)) {
			$social_links[] = array(
					'icon' => 'pinterest',
					'lang' => lang('label_pinterest_url'),
					'link' => $record->pinterest_url
			);
		}
		if(!empty($record->instagram_url)) {
			$social_links[] = array(
					'icon' => 'instagram',
					'lang' => lang('label_instagram_url'),
					'link' => $record->instagram_url
			);
		}
		if(!empty($record->profile_image)) {
			$record->profile_image = gc_image_url($record->profile_image, true, true, 'users');
		}
		$view_data['record'] = $record;
		$view_data['social_links'] = $social_links;
		if($role == 'members') {
			Template::set($view_data);
			$layout = $this->gc->load_assets('users/detail');
			Template::set_view($layout['view']);
			Template::render($layout['layout']);
		} else {
			return $view_data;
		}		
	}
	
	/**
	 * Get User Info
	 * @param number $id
	 * @param string $role
	 * @return array[]
	 */
	private function user_info($id, $role) {
	    $module = 'users';
	    $result = array(
	        'result' => new stdClass(),
	        'module_info' => array()
	    );
	    $record = $this->db->get_where('users', array('id' => $id, 'active' => 1, 'deleted' => 0))->row();
	    if(!$record) {
	        return $result;
	    }
	    
	    // Display meta information
	    $this->load->model('fields/fields_model');
	    $record->meta_fields = $this->fields_model->find_meta_for_detail($module, $id);
	    
	    if($role == 'mobile') {
	        // Display listing related module information
	        $config_modules = gc_modules();
	        foreach($config_modules as $config_module) {
	            if(gc_module_settings($config_module) == 1) {
	                $module_info[$config_module] = $this->db->where(array('user_id' => $id, 'status' => 1, 'deleted' => 0))->count_all_results($config_module);
	            }
	        }
	        $record->module_info = $module_info;
	    } else {
	        $result['module_info'] = $this->listings_model->module_info(0, $id, $record);
	    }
	    $result['record'] = $record;
	    return $result;
	}
	
	/**
	 * Commong method for login
	 * @param boolean $ajax
	 * @param boolean $mobile
	 */
	public function login($ajax = false, $mobile = false) {		
		$remember = $this->input->post('remember_me') == '1' ? true : false;
		if ($this->auth->is_logged_in() === FALSE) {
			if ($ajax || isset($_POST['log-me-in'])) {
				$username = escape($this->input->post('login'));
				$password = $this->input->post('password');
				// OTP Login
				if(settings_item('auth.otp_login') == 1) {
					$otp_login = $this->otp_login($username, $password, $remember, $ajax, $mobile);
					if($otp_login['success']) {
						$this->session->set_userdata('otp_verify', 'login');
						$login_data = $this->session->userdata('otp_login');
						$this->load->library('crypt');
						$redirect_url = gc_url_format('verify_otp') . '/' . $this->crypt->encode($login_data['user_id']);
						if($mobile) {
							return ['success' => true, 'user_id' => $login_data['user_id'], 'verify_otp' => true];
						} else if($ajax) {
							echo json_encode(array('result'=> 1, 'url' => $redirect_url));
						} else {
							Template::redirect($redirect_url);
						}						
					} else {
						$message = $otp_login['message'];
						if($mobile) {
							return $otp_login;
						} else if($ajax) {
							echo json_encode(array('result' => 0, 'message' => $message, 'type' => 'error'));
						} else {
							Template::set_message($message, 'danger');
						}
					}
				} else {
					return $this->valid_login($username, $password, $remember, false, $ajax, $mobile);	
				}			
			}
		} else if(!$mobile && !$ajax) {
			Template::redirect(gc_url_format('members'));
		} else if($mobile) {
		    //end if
		    $this->auth->logout(); // logout the user
		    return array('success' => false, 'message' => lang('us_failed_login_attempts'));
		}
	}
	
	/**
	 * The user has valid login credentials
	 * @param boolean $ajax
	 * @param boolean $mobile
	 * @return multitype:boolean
	 */
	private function valid_login($username, $password, $remember = false, $otp_login = false, $ajax = false, $mobile = false) {
		// Try to login
		$result = $this->auth->login($username, $password, $remember, FALSE, NULL, $mobile);
		if ($result === TRUE) {
			if($mobile) {
				return array('success' => true, 'user_id' => $this->auth->user_id());
			} else {
				if($otp_login) {
					// Unset OTP Login Userdata
					$this->session->unset_userdata('otp_login');
				}
				// Log the Activity
				log_activity($this->auth->user_id(), lang('us_log_logged') . ': ' . $this->input->ip_address(), 'users');
			
				/*
				 Now redirect - In many cases, we will have set a destination for a particular user-role to redirect to.
				This is helpful for cases where we are presenting different information to different roles that might cause the base destination to be not available.
				*/
				if($this->session->userdata('login_redirect_url')) {
					$login_redirect_url = $this->session->userdata('login_redirect_url');
					$this->session->unset_userdata('login_redirect_url');
					$url = $login_redirect_url;
					// only first 50 characters are being returned from the session variable
					// if string length is 50 redirect user to the home page
					if(strlen($login_redirect_url) == 50) {
						$url = gc_url_format('members');
					}
				} else if ($this->settings_lib->item('auth.do_login_redirect') && !empty($this->auth->login_destination)) {
					$url = $this->auth->login_destination;
				} else {
					if (!empty($this->requested_page)) {
						$url = gc_url_format('members'); //$this->requested_page;
					} else {
						$url = gc_url_format('members'); // site_url();
					}
				}
				if($ajax) {
					echo json_encode(array('result' => 1, 'url' => $url));
				} else {
					Template::redirect($url);
				}
			}
		} else {			
			$message = lang('us_bad_email_pass'); // Default Message
			// check if the account has been activated.
			// in case we go to a unix timestamp later, this will still work.
			$user = $this->user_model->select('id, password_hash, active')->find_by(array('username' => $this->input->post('login'), 'email' => $this->input->post('login')), null, 'or');
			if($user) {
				$result = $this->check_activation($user, $password, $ajax, $mobile);
				if($result['success'] === false) {
					$message = $result['message'];
				}
			}
			if($mobile) {
				if($result && $result['success'] === false) {
					return $result;
				} else {
					return array('success' => false, 'message' => $message);
				}
			} else if($ajax) {
				echo json_encode(array('result' => 0, 'type' => 'error', 'message' => $message));
			} else {
				Template::set($message, 'danger');
			}
		}
	}
	
	/**
	 * Dynamic Fields Registration
	 * @param boolean $mobile
	 */
	public function register($mobile = false) {
	    // Are users even allowed to register?
	    if (!$this->settings_lib->item('auth.allow_register')) {
	        if($mobile) {
	            return array('message' => lang('us_register_disabled'), 'result' => false, 'type' => 'danger');
	        } else {
    	        Template::set_message(lang('us_register_disabled'), 'danger');
    	        Template::redirect(site_url());
	        }
	    }
	    $settings = $this->settings_lib->find_all();
	    if (!$mobile && $settings['auth.password_show_labels'] == 1) {
	        Assets::add_module_js('users', 'password_strength.js');
	        Assets::add_module_js('users', 'jquery.strength.js');
	        Assets::add_js($this->load->view('users_js', array('settings' => $settings), true), 'inline');
	    }
	    
	    // Generate password hint messages.
	    $this->user_model->password_hints();
	    
	    // Template::set('languages', unserialize($this->settings_lib->item('site.languages')));
	    
	    //Hybrid_auth
	    if (!$mobile && $this->settings_lib->item('hybridauth_enabled') == 1) {
	        Template::set('hybrid_auth_providers', $this->user_model->get_providers(true));
	    }
	    $module = 'registrations';
	    $type = 'frontend';
	    $this->load->model('fields/fields_model');
	    $this->lang->load('groups/groups');
	    $fields = $this->fields_model->get_fields($module, $type);
	    $groups = $this->fields_model->field_groups($fields, array());
	    $form_fields = $this->fields_model->load_fields($module, array(), $type, array(), 0, $mobile, $fields);
	    if (isset($_POST['register'])) {
	        $user_id = $this->fields_model->save_module($fields, 'users', 0, 'frontend', 0, 'insert');
	        if(!$user_id && $mobile) {
    	        return array('result' => 0, 'message' => validation_errors(), 'type' => 'danger');
	        } else if($user_id) {
	            $result = $this->notify_registration($user_id, $mobile);
	            if($mobile) {
	                return $result;
	            }
	        }
	    }
	    if(!$mobile) {
    	    Template::set(array(
    	        'form_fields' => $fields,
    	        'field_groups' => $groups
    	    ));
	    }
	}
	
	/**
	 * Activate user.
	 * Checks a passed activation code and if verified, enables the user account. 
	 * If the code fails, an error is generated and returned.
	 */
	public function activate_user($user_id = NULL, $code = NULL, $ajax = false, $mobile = false) {
		// If the user is already active, redirect him/her to the home page
		if($user_id != NULL) {
			$user_status = $this->db->where(array('id' => $user_id, 'active' => 0))->count_all_results('users');
			if($user_status <= 0) {
				if($mobile) {
					return ['success' => false, 'url' => 'app.login'];
				} else if($ajax) {
					echo json_encode(array('result' => 0, 'url' => gc_url_format('login'), 'type' => 'error'));
				} else {
				    Template::set_message(lang('us_account_already_active'), 'danger');
					Template::redirect(gc_url_format('login'));
				}
			}
		}		 
		if (isset($_POST['activate'])) {
			$this->form_validation->set_rules('code', 'lang:us_verification_code', 'required|trim');
			if ($this->form_validation->run() == TRUE) {
				$code = $this->input->post('code');
			}
		}
		$code = mysqli_real_escape_string($this->gc->get_mysqli(), trim($code));
		if(!empty($code)) {
			$this->user_activation($user_id, $code, $ajax, $mobile);
		}
		if(!$ajax && !$mobile) {
			Template::set('page_title', lang('us_account_activation'));
			$layout = $this->gc->load_assets('users/activate');
			Template::set_view($layout['view']);
			Template::render($layout['layout']);
		}
	}
	
	/**
	 * User's Activation
	 * @param integer $user_id
	 * @param string $code
	 * @param boolean $ajax
	 * @param boolean $mobile
	 * @return multitype:boolean string
	 */
	private function user_activation($user_id, $code, $ajax, $mobile) {
		$activated = $this->user_model->activate($user_id, $code);
		if(!$ajax && !$mobile) {
			Template::set('success_msg', lang('us_account_active'));
		}
		if ($activated) {
			$user_id = $activated;
			$user = (object) $this->get_user($user_id);
			$user->link = gc_url_format('login');
			$gemailer_result = $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'user_email_templates', 'module_table' => 'users', 'action_type' => 'account_activated', 'id' => $user_id, 'record' => $user));
			if ($gemailer_result['emailer_success']) {
			    $message = sprintf(lang('us_account_active'), settings_item('site.title'));
			    if($mobile) {
			        return ['success' => true, 'message' => $message];
			    } else if($ajax) {
			        echo json_encode(array('result' => 1, 'message' => $message, 'type' => 'success', 'url' => gc_url_format('login')));
			    } else {
			        Template::set_message($message, 'success');
			        Template::redirect(gc_url_format('login'));
			    }
			} else {
			    $message = lang('us_err_no_email'). $gemailer_result['emailer_error_message'] . ' ' . lang('us_account_active_login');
			    if($mobile) {
			        return ['success' => false, 'message' => $message];
			    } else if($ajax) {
			        echo json_encode(array('result' => 0, 'message' => $message, 'type' => 'error'));
			    } else {
			        Template::set_message($message, 'danger');
			        Template::redirect(gc_url_format('login'));
			    }
			}
			//header( "refresh:5; url=" . $this->login_url );
		} else if(!$ajax && !$mobile) {
			Template::set_message($this->user_model->error, 'danger');
			Template::redirect(gc_url_format('login'));
		}
	}
	
	/**
	 * Method: resend_activation
	 * 
	 * Allows a user to request that their activation code be resent to their account's email address.
	 * If a matching email is found, the code is resent.
	*/
	public function resend_activation($user_id, $ajax = false, $mobile = false, $redirect = true) {
		$result = $this->db->get_where('users', array('id' => $user_id))->row();
		if(!empty($result->email)) {
			if(!empty($result->activate_hash)) {
				$activation_code = $result->activate_hash;
			} else {
				$activation_code = $this->user_model->deactivate($user_id);
			}
			$activate_link   = site_url(gc_uri_string('activate') . '/' . $user_id . '/' . $activation_code);
			$subject         = lang('us_email_subj_activate');		
			
			// send email
			$user = $result;
			$user->subject = $subject;
			$user->link = $activate_link;
			$user->code = $activation_code;
			$gemailer_result = $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'user_email_templates', 'module_table' => 'users', 'action_type' => 'activation_code', 'id' => $user->id, 'record' => $user));
			
			if (!$gemailer_result['emailer_success']) {
			    $message = lang('us_err_no_email') . $gemailer_result['emailer_error_message'];
			    $success = false;
			    // Template::set_message(lang('us_err_no_email').$this->emailer->error.", ".$this->emailer->debug_message, 'danger');
			} else {
			    $message = lang('us_check_activate_email');
			    $success = true;
			    // Template::set_message(lang('us_msg_activation_email'), 'success');
			}
			if($mobile) {
			    return ['success' => $success, 'message' => $message];
			} else if($ajax) {
			    echo json_encode(array('result' => ($success ? 1 : 0), 'message' => $message, 'type' => ($success ? 'success' : 'error')));
			} else {
			    Template::set_message($message, ($success ? 'success' : 'error'));
			    if($redirect) {
			        Template::redirect(gc_url_format('login'));
			    }
			}
		} else {
			$message = lang('us_invalid_email');
			if($mobile) {
				return ['success' => false, 'message' => $message];
			} else if($ajax) {
				echo json_encode(array('result' => 0, 'message' => $message, 'type' => 'error'));
			} else {
				Template::set_message($messasge, 'error');
				if($redirect) {
					Template::redirect(gc_url_format('login'));
				}
			}
		}
	}
	
	/**
	 * Allows a user to start the process of resetting their password.
	 * An email is allowed with a special temporary link that is only valid
	 * for 24 hours. This link takes them to reset_password().
	 *
	 * @access public
	 * 
	 * @param boolean $ajax - True - Request is from the Ajax Form
	 * @param boolean $mobile - True - Request is from the Mobile App
	 *
	 * @return void
	 */
	public function forgot_password($ajax = false, $mobile = false) {
		// if the user is not logged in continue to show the login page
		if ($this->auth->is_logged_in() === FALSE) {
			if ($ajax || isset($_POST['send'])) {
				$this->form_validation->set_rules('email_phone', 'lang:mobile_email', 'required|trim');
	
				if ($this->form_validation->run() === FALSE) {
					$validation_errors = validation_errors();
					if ($validation_errors) {
						$validation_error = $validation_errors;
					}
					if($mobile) {
						return ['success' => false, 'message' => $validation_error];
					} else if($ajax) {
						echo json_encode(array('result' => 0, 'message' => $validation_error, 'type' => 'error'));
					} else {
					    Template::set_message($validation_errors, 'danger');
					    Template::redirect(current_url());
					}
				} else {
					$email_phone = $this->input->post('email_phone');
					// $this->db->like('email', $email_phone);
					// $this->db->or_like('phone_number', $email_phone);
					// We validated. Does the user actually exist?
					$user = $this->db->where('(email = "' .  $email_phone . '" OR phone_number = "' . $email_phone . '")')->get('users')->row(); // $this->user_model->find_by(array('email' => $_POST['email'], $this->db->dbprefix . 'users.role_id !=' => 1));
	
					if ($user) {
						$this->load->library('crypt');
						if ($user->hauth_provider !== NULL) {
							if($mobile) {
								return ['success' => false, 'message' => lang('us_invalid_email')];
							} else if($ajax) {
								echo json_encode(array('result'=> 0, 'message' => lang ( 'us_invalid_email' ), 'type' => 'error'));
							} else {
								Template::set_message(lang('us_invalid_email'), 'error');
							}
						} else if(settings_item('auth.user_activation_method') == 3) {
							$hash = $this->reset_hash($user);
							$this->session->set_userdata('otp_verify', 'reset_password');
							$this->session->set_userdata('otp_reset_password', array('hash' => $hash));
							$redirect_url = gc_url_format('verify_otp') . '/' . $this->crypt->encode($user->id);
							if($mobile) {
								return ['success' => true, 'verify_otp' => true, 'hash' => $hash, 'user' => $user];
							} else if($ajax) {
								echo json_encode(array('result'=> 1, 'url' => $redirect_url));
							} else {
								Template::redirect($redirect_url);
							}
						} else if (!empty($user->email)) {						
							$email = $user->email;							
							$hash = $this->reset_hash($user);
							// Create the link to reset the password
							$pass_link = gc_url_format('reset_password') . '/' . $this->crypt->encode($user->id) . '/' . $hash;	
							
							$user->link = $pass_link;
							$user->link_url = anchor($pass_link, $pass_link);
							$gemailer_result = $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'user_email_templates', 'module_table' => 'users', 'action_type' => 'forgot_password', 'id' => $user->id, 'record' => $user));
							
							if($gemailer_result['emailer_success']) {
							    $message = lang ( 'us_reset_pass_message' );
							    if($mobile) {
							        return array('success' => true, 'message' => $message, 'hash' => $hash, 'user' => $user);
							    } else if($ajax) {
							        echo json_encode(array('result'=> 1, 'message' => $message, 'type' => 'success'));
							    } else {
							        Template::set_message($message, 'success');
							    }
							} else {
							    $message = lang ( 'us_reset_pass_error' )  . $gemailer_result['emailer_error_message'];
							    if($mobile) {
							        return array('success' => false, 'message' => $message);
							    } else if($ajax) {
							        echo json_encode(array('result'=> 0, 'message' => $message, 'type' => 'error'));
							    } else {
							        Template::set_message($message, 'danger');
							    }
							}
						}
					} else {
						$message = lang('invalid_email_phone');
						if($mobile) {
							return ['success' => false, 'message' => $message];
						} else if($ajax) {
							echo json_encode(array('result'=> 0, 'message' => $message, 'type' => 'error'));
						} else {
							Template::set_message($message, 'error');
						}
					}//end if
				}//end if
			}//end if
			if(!$ajax && !$mobile) {
				Template::set('page_title', lang('us_reset_password'));
				$layout = $this->gc->load_assets('users/forgot_password');
				Template::set_view($layout['view']);
				Template::render($layout['layout']);
			}
		} else {
			$message = lang('invalid_email_phone');
			if($mobile) {
				return ['success' => false, 'message' => $message];
			} else if($ajax) {
				echo json_encode(array('result'=> 0, 'message' => $message, 'type' => 'error'));
			} else {
				Template::redirect(site_url());
			}
		}//end if
	}
	
	/**
	 * Generate and update reset Hash
	 * @param integer $user
	 * @return string
	 */
	private function reset_hash($user) {
		if(!empty($user->email)) {
			$param = $user->email;
			$field = 'email';
		} else {
			$param = $user->phone_number;
			$field = 'phone_number';
		}
		// User exists, so create a temp password.
		$this->load->helpers(array('string', 'security'));
		
		$pass_code = random_string('alnum', 40);
		
		$hash = do_hash($pass_code . $param);
		
		// Save the hash to the db so we can confirm it later.
		$this->user_model->update_where($field, $param, array('reset_hash' => $hash, 'reset_by' => strtotime("+24 hours")));
		
		return $hash;
	}
	
	/**
	 * Allow user to reset password
	 * 
	 * @param string $email
	 * @param string $code
	 * @param string $ajax
	 * @param string $mobile
	 */
	public function reset_password($user_id, $code, $ajax = false, $mobile = false) {
		// If we're set here via Bonfire and not an email link
		// then we might have the email and code in the session.
		if (empty ( $code ) && $this->session->userdata ( 'pass_check' )) {
			$code = $this->session->userdata ( 'pass_check' );
		}
			
		if (empty ( $user_id ) && $this->session->userdata ( 'user_id' )) {
			$user_id = $this->session->userdata ( 'user_id' );
		}
		
		// If there is no code, then it's not a valid request.
		if (empty ( $code ) || empty ( $user_id )) {
			$message = lang ( 'us_reset_invalid_email' );
			if($mobile) {
				return ['success' => false, 'message' => $message];
			} else if($ajax) {
				echo json_encode(array('result'=> 0, 'message' => $message, 'type' => 'error'));
			} else {
				Template::set_message ( $message, 'error' );
				Template::redirect (gc_url_format('login'));
			}
		}
		
		// Check the code against the database
		$user = $this->user_model->find_by(array(
				'id' => $user_id,
				'reset_hash' => $code,
				'reset_by >=' => time()
		));
		
		// It will be an Object if a single result was returned.
		if (!is_object($user)) {
			$message = lang ( 'us_reset_invalid_email' );
			if($mobile) {
				return ['success' => false, 'message' => $message];
			} else if($ajax) {
				echo json_encode(array('result'=> 0, 'message' => $message, 'type' => 'error'));
			} else {
				Template::set_message ( $message, 'error' );
				Template::redirect (gc_url_format('login'));
			}
		}
		
		if(!$ajax && !$mobile) {
			// If we're here, then it is a valid request....
			Template::set('user', $user);
		}
		
		// Handle the form
		if (isset($_POST['set_password'])) {
			$this->form_validation->set_rules('password', 'lang:bf_password', 'required|max_length[120]|valid_password');
			$this->form_validation->set_rules('pass_confirm', 'lang:bf_password_confirm', 'required|matches[password]');
		
			if ($this->form_validation->run() !== FALSE) {
				// The user model will create the password hash for us.
				$data = array('password' => $this->input->post('password'),
						'reset_by' => 0,
						'reset_hash' => '',
						'force_password_reset' => 0);
		
				if ($this->user_model->update($user_id, $data)) {
					// Log the Activity
					log_activity($this->input->post('user_id'), lang('us_log_reset'), 'users');
					$message = lang('us_reset_password_success');
					if($mobile) {
						return ['success' => true, 'message' => $message];
					} else if($ajax) {
						echo json_encode(array('result'=> 1, 'message' => $message, 'type' => 'success'));
					} else {
						Template::set_message ( $message, 'success' );
						delete_cookie("formbox"); // important or no message will be displayed and user will be on forgot_password formbox
						Template::redirect (gc_url_format('login'));
					}
				} else {
					$message = sprintf(lang('us_reset_password_error'), $this->user_model->error);
					if($mobile) {
						return ['success' => false, 'message' => $message];
					} else if($ajax) {
						echo json_encode(array('result'=> 0, 'message' => $message, 'type' => 'error'));
					} else {
						Template::set_message ( $message, 'error' );
					}
				}
			}
		}//end if
	}

	/**
	 * Send OTP
	 * @param array $user
	 * @return multitype:boolean string
	 */
	public function send_otp($user, $ajax = false, $mobile = false) {
		// Send SMS
		$otp = mt_rand(100000, 999999);
		$this->db->update('users', array('activate_otp' => $otp, 'otp_attempts' => 1, 'otp_expires_on' => $this->user_model->otp_expiry()), array('id' => $user->id));
		if(!empty($user->phone_number)) { // User logged in with email address
			$this->gc->send_sms(sprintf(lang('otp_message'), $otp, site_url(), $this->user_model->otp_expiry()), $user->phone_number);
			$message = lang('otp_sent') . ' ' . $user->phone_number;
			if($mobile) {
				return array('success' => true, 'message' => $message);
			} else if($ajax) {
				echo json_encode(array('result' => 1, 'message' => $message, 'type' => 'success'));
			} else {
				Template::set_message($message, 'success');
			}
		}
	}

	/**
	 * Resend OTP code
	 * 
	 * @param integer $user_id
	 * @param string $ajax
	 * @param string $mobile
	 * @return multitype:boolean string
	 */
	public function resend_otp($user_id, $ajax = false, $mobile = false) {
		$success = false;
		$message = '';
		$user = $this->db->select('activate_otp, otp_attempts, phone_number')->where('id', $user_id)->get('users')->row();
		if($user) {
			$otp_limit = settings_item('auth.max_otp_attempts');
			// Resend same activation code to avoid user confusion
			// Update database and resend OTP only if the OTP limit is not exceeded
			if(!empty($user->activate_otp) && ($otp_limit > $user->otp_attempts)) {
				$otp = $user->activate_otp;
				$this->db->where('id', $user_id)->update('users', array('otp_attempts' => (int)$user->otp_attempts + 1, 'otp_expires_on' => $this->user_model->otp_expiry()));
			} else if(empty($user->activate_otp)) { // Generate new OTP if this is the first time
				$otp = $this->user_model->otp($user_id);
			} else if($otp_limit <= $user->otp_attempts) {
				$message = lang('otp_retry_limit_exceeded');
			}
		
			if($otp_limit > $user->otp_attempts && !empty($user->phone_number)) {
				$this->gc->send_sms(sprintf(lang('otp_message'), $otp, site_url(), $this->user_model->otp_expiry()), $user->phone_number);
				$message = lang('otp_success');
				$success = true;
			}
			if($mobile) {
				return array('success' => $success, 'message' => $message);
			} else if($ajax) {
				echo json_encode(array('result' => ($success ? 1 : 0), 'message' => $message, 'type' => ($success ? 'success' : 'error')));
			} else {
				Template::set_message($message, ($success ? 'success' : 'error'));
			}
		} else { // No record exists in database
			if($mobile) {
				return array('success' => $success, 'message' => lang('otp_error'));
			} else if($ajax) {
				echo json_encode(array('result' => 0, 'message' => lang('otp_error'), 'type' => 'error'));
			} else {
				Template::set_message(lang('otp_error'), 'error');
			}
		}
	}
	


	/**
	 * OTP verifiy before login using the session variable OTP Login
	 * Insert a new row (auth.otp_login) in the settings table
	 * Change it to 1 to enable OTP Login and change the login() method.
	 *
	 * Use Mobile number as the username in case of registration if the login type is OTP
	 * Get Email of the user in case of OTP and store it in Session
	 * Auth library uses Email for verification.
	 * All steps are performed to avoid the OTP generation if the account is not acctive
	 */
	public function otp_login($username, $password, $remember = FALSE, $ajax = false, $mobile = false) {
		$success = false;
		$user = $this->db->where("role_id != 1 AND (phone_number='" . $username . "' OR email='" . $username . "')")->get('users')->row();
	
		// check to see if a value of FALSE came back, meaning that the username or email or password doesn't exist.
		if ($user == false) {
			$message = lang('us_bad_phone_pass');
		}
	
		//Check to see if its a HybridAuth login account
		if (isset($user) && $user->hauth_provider != NULL) {
			$message = lang('us_bad_email_pass');
		}
	
		// check if the account has been soft deleted.
		// in case we go to a unix timestamp later, this will still work.
		if (isset($user) && $user->deleted >= 1) {
			$message = sprintf(lang('us_account_deleted'), html_escape(settings_item("site.system_email")));
		}
		
		// check if the account has been activated.
		// in case we go to a unix timestamp later, this will still work.
		$result = $this->check_activation($user, $password, $ajax, $mobile);
		if($result['success'] === false) {
			return $result;
		}
	
		// Try password
		if (isset($user) && $user->active == 1 && $this->auth->check_password($password, $user->password_hash)) {
			// check if the account has been banned.
			if ($user->banned) {
				$this->auth->increase_login_attempts($login);
				$message = $user->banned_message ? $user->banned_message : lang('us_banned_msg');
			} else {
				// Set otp_login userdata
				$login_data = array(
						'user_id' => $user->id,
						'username' => settings_item('auth.user_activation_method') == 'username' ? $user->username : $user->email,
						'password' => $password,
						'remember' => $remember
				);
				$this->session->set_userdata('otp_login', $login_data);
				return array('success' => true);
			}
		} else { // Bad password
			$this->auth->increase_login_attempts($username);
			$message = lang('us_bad_phone_pass');
		}
		return array('success' => false, 'message' => $message);
	}
	
	/**
	 * Common Method to display appropriate message to an inactive user
	 * @param array $user
	 * @param boolean $ajax
	 * @param boolean $mobile
	 * @return multitype:boolean NULL string |multitype:boolean
	 */
	public function check_activation($user, $password, $ajax, $mobile) {
		$activation_type = settings_item('auth.user_activation_method');
		if ($user && $this->auth->check_password($password, $user->password_hash) && $user->active == 0 && $activation_type > 0) {
			$this->load->library('crypt');
			if ($activation_type == 1) {
				$message = lang('us_account_not_active');
				if(!$mobile) {
					$message .= sprintf(lang('us_resend_activation'), anchor(gc_url_format('resend_activation') . '/' . $this->crypt->encode($user->id), lang('click_here')));
				} else {
					$message .= sprintf(lang('us_resend_activation'), lang('otp_mobile_message'));
					$resend_activation = true; // Mobile App - Allow user to request for activation code
				}
			} elseif ($activation_type == 2) {
				$message = lang('us_admin_approval_pending');
			} elseif ($activation_type == 3) {
				$message = lang('us_otp_account_not_active');
				if(!$mobile) {
					$message .= sprintf(lang('us_otp_activation'), anchor(gc_url_format('verify_otp') . '/' . $this->crypt->encode($user->id), lang('click_here')));
				} else {
					$message .= sprintf(lang('us_otp_activation'), lang('otp_mobile_message'));
					$verify_otp = true;
				}
			}
			$return_data = array('success' => false, 'message' => $message);
			if(isset($resend_activation)) { // Mobile App - Allow user to request for activation code
				$return_data['resend_activation'] = true;
			}
			if(isset($verify_otp)) { // Mobile App - Redirect the user to the Mobile OTP Modal
				$return_data['verify_otp'] = true;
			}
			if(isset($user->id)) {
				$return_data['user_id'] = $user->id; // To Resend Activation or to redirect user to the Verify OTP Modal
			}
			return $return_data;
		} else {
			return array('success' => true);
		}
	}
	
	/**
	 * Common Verify OTP Method
	 * @param boolean $ajax
	 * @param boolean $mobile
	 */
	public function verify_otp($user_id = 0, $ajax = false, $mobile = false) {
		// Get the method for which the OTP verification is done
		// This is handled through a session variable
		$verify_otp = $this->session->userdata('otp_verify');
		switch($verify_otp) {
			case 'login':
				$login = $this->session->userdata('otp_login');
				if ($this->auth->is_logged_in() || empty($login)) {
					return false;
				}
				break;
			default:
				if ($this->auth->is_logged_in()) {
					return false;
				}
				break;
		}
		$user = $this->db->select('id, activate_otp, otp_attempts, phone_number')->where('id', $user_id)->get('users')->row();
		if(!$user) {
			return false;
		}

		// Send OTP to the User, if not already sent
		if(!$mobile && !$this->session->userdata('otp_sent')) {
			$this->send_otp($user, $ajax, $mobile);
			$this->session->set_userdata('otp_sent', true);
		}
		
		$this->otp_limit($user_id, $user, $ajax, $mobile);
		if(isset($_POST['submit'])) {
			$this->form_validation->set_rules('otp', 'lang:us_verification_code', 'required|trim');
			if ($this->form_validation->run() == TRUE) {
				$code = $this->input->post('otp');

				if($user && $user->activate_otp == $code) {
					// Clear the OTP Sent Variable
					$this->session->unset_userdata('otp_sent');
					$this->session->unset_userdata('otp_verify');
					switch($verify_otp) {
						case 'login':
							// Create login session only if the OTP is verified
							$this->session->unset_userdata('otp_login');
							return $this->valid_login($login['username'], $login['password'], $login['remember'], true, $ajax, $mobile);
							break;
						case 'activate':
							// Activate User
							return $this->user_activation($user_id, $code, $ajax, $mobile);
							break;
						case 'reset_password':
							$reset = $this->session->userdata('otp_reset_password');
							$this->session->unset_userdata('otp_reset_password');
							if(!empty($reset['hash'])) {
								if($ajax || $mobile) {
									return array('success' => true);
								} else {									
									Template:redirect(gc_url_format('reset_password') . '/' . $this->crypt->encode($user_id) . '/' . $reset['hash']);
								}
							}
							break;
					}
				} else {
					$message = lang('otp_invalid');
					if($mobile) {
						return array('success' => false, 'message' => $message);
					} elseif($ajax) {
						return array('return' => 0, 'message' => $message, 'type' => 'error');
					} else {
						Template::set_message($message, 'error');
					}					
				}
			} else {
				$validation_errors = validation_errors();
				if ($validation_errors) {
					$validation_error = $validation_errors;
				}
				if($mobile) {
					return ['success' => false, 'message' => $validation_error];
				} else if($ajax) {
					echo json_encode(array('result' => 0, 'message' => $validation_error, 'type' => 'error'));
				}
			}
		}
		if(isset($_POST['resend_otp'])) {
			if($user_id > 0) {
				$this->resend_otp($user_id, $ajax, $mobile);
			}
		}
		if(!$ajax && !$mobile) {
			Assets::add_module_css ( 'users', 'users.css' );
			Template::set('page_title', lang('otp_verify_heading'));
			$layout = $this->gc->load_assets('users/verify_otp');
			Template::set_view($layout['view']);
			Template::render($layout['layout']);
		}
	}
	
	/**
	 * Get OTP limit to display to the user
	 * @param integer $user_id
	 */
	public function otp_limit($user_id, $user = array(), $ajax = false, $mobile = false) {
		// OTP Registration, display remaining OTP attempts to the user
		if(empty($user)) {
			$user = $this->db->select('activate_otp, otp_attempts, phone_number')->where('id', $user_id)->get('users')->row();
			if(!$user) {
				return false;
			}
		}
		$otp_limit = settings_item('auth.max_otp_attempts');
		$otp_limit = $otp_limit - (int)$user->otp_attempts;
		if($ajax || $mobile) {
			return $otp_limit;
		} else {
			Template::set('otp_limit', $otp_limit);
			Template::set('user', $user);
		}
	}
	
	/**
	 * Checkout Page - Notify Admin on User's Registration
	 */
	public function notify_admin($data = array()) {
	    // Notify Admin on New User Registration
	    if(settings_item('auth.notify_admin') == 1 && !empty($data['email'])) {
	        
	        // Retrieve template from database
	        $email_template = $this->get_email_template(16);
	        
	        // User's Information
	        $display_name = (isset($data['display_name']) ? $data['display_name'] : '');
	        $email = (isset($data['email']) ? $data['email'] : '');
	        $username = (isset($data['username']) ? $data['username'] : '');
	        $phone_number = (isset($data['phone_number']) ? $data['phone_number'] : '');
	        
	        // Message
	        $subject_data = array(
	            'subject' => $email_template->subject,
	            'email'	=> $email,
	            'username'	=> $username
	        );
	        $subject = $this->parser->parse('email_templates/email_subject', $subject_data, TRUE);
	        
	        $admin_message_data = array(
	            'message' => $email_template->message,
	            'display_name'	=> $display_name,
	            'username'	=> $username,
	            'email'	=> $email,
	            'phone_number'	=> $phone_number,
	            'site_title' => settings_item('site.title')
	        );
	        $admin_email_data = $this->parser->parse('email_templates/email_body', $admin_message_data, TRUE); //TRUE is important to return string
	        
	        // Send Email
	        $this->load->library('emailer/emailer');
	        $admin_email = array(
	            //'from'		=> settings_item('sender_email'),
	            'to'		=> settings_item('site.system_email'),
	            'subject'	=> $subject,
	            'message'	=> $admin_email_data
	        );
	        $this->emailer->send($admin_email, FALSE);
	    }
	}
	
	/**
	 * Save and Validate Two Factor Authentication
	 * @param number $user_id
	 * @param object $user
	 * @param number $page = 1; // 0 -> Regenerate Form, 1 -> Validate Form
	 */
	public function twofactor_authentication($user_id, $user = array(), $page = 1, $role = 'admin') {
	    $retry_limit = 10; // Default Retry Limit
	    if(empty($user)) {
	        $user = $this->db->where('id', $user_id)->get('users')->row();
	    }
	    if(empty($user)) {
	        Template::redirect(site_url());
	    }
	    $retry_counter = $user->tfa_retry_counter; // Retry Counter
	    // save/regenerate qrcode
	    if(isset($_POST['save']) || isset($_POST['save_only']) || isset($_POST['regenerate'])) {
	        $data = array();
	        if(isset($_POST['regenerate'])) {
	            $issuer = $user->tfa_issuer;
	            $accountName = $user->tfa_account;
	        } else {
	            $issuer = $this->input->post('tfa_issuer');
	            $accountName = $this->input->post('tfa_account');
	            $data['tfa_status'] = $this->input->post('tfa_status') ? 1 : 0;
	        }
	        if(!empty($issuer) && !empty($accountName)) {
	            $data['tfa_issuer'] = $issuer;
	            $data['tfa_account'] = $accountName;
	            $secretFactory = new \Dolondro\GoogleAuthenticator\SecretFactory();
	            $secret = $secretFactory->create($issuer, $accountName);
	            $data['tfa_secret'] = $secret->getSecretKey();
	            $qrImageGenerator = new \Dolondro\GoogleAuthenticator\QrImageGenerator\EndroidQrImageGenerator();
	            $data['tfa_qrcode'] = $qrImageGenerator->generateUri($secret);
	            $this->db->where('id', $user_id)->update('users', $data);
	            Template::set_message(sprintf(lang('msg_save_success'), lang('label_tfa')), 'success');
	            if(isset($_POST['save'])) {
	                $redirect_url = $this->session->userdata('users_last_visited_page') ? $this->session->userdata('users_last_visited_page') : SITE_AREA . '/settings/users';
	                Template::redirect($redirect_url);
	            } else {
	                Template::redirect(current_url());
	            }
	        } else {
	            Template::set_message(lang('msg_save_error'), 'danger');
	        }
	    }
	    if(isset($_POST['validate'])) {
	        $this->form_validation->set_rules('tfa_code', 'lang:label_code', 'trim|required|is_numeric');
	        if ($this->form_validation->run() === true) {
	            $secretKey = $user->tfa_secret;
	            $code = $this->input->post('tfa_code');
	            $googleAuthenticator = new \Dolondro\GoogleAuthenticator\GoogleAuthenticator();
	            $result = $googleAuthenticator->authenticate($secretKey, $code);
	            if($result) {
	                // set the tfa_retry_counter to zero
	                $this->db->where('id', $user->id)->update('users', array('tfa_retry_counter' => 0, 'tfa_retry_on' => null));
	                
	                // unset the tfa_user_id session variable
	                $this->session->unset_userdata('tfa_user_id');
	                
	                // setup session and redirect user
	                $this->auth->authenticate($user);
	                
	                // Redirect the user
	                $this->login_redirect();
	            } else {
	                // update the tfa_retry_counter
	                // check this reactivation date on next login instead of using cronjobs
	                $retry_counter = $user->tfa_retry_counter + 1;
	                $data = array('tfa_retry_counter' => $retry_counter);
	                // if the counter value exceeds the default limit, set the re-activation date
	                if($retry_counter == $retry_limit) {
	                    $date = date('Y-m-d H:i:s');
	                    $data['tfa_retry_on'] = date('Y-m-d H:i:s', strtotime($date . ' + 1 days'));
	                }
	                $this->db->where('id', $user->id)->update('users', $data);
	                Template::set_message(lang('otp_invalid'), 'danger');
	            }
	        }
	    }
	    Template::set(array(
	        'toolbar_title' => lang('label_tfa'),
	        'page' => $page,
	        'record' => $user,
	        'redirect_url' => $role == 'admin' ? SITE_AREA . '/settings/users' : gc_url_format('members'),
	        'retry_counter' => $retry_limit - $retry_counter
	    ));
	    if($page == 1) {
	        $this->gc->demo_custom_css_js('index');
	    }
	    Template::set_view('twofactor_authentication');
	    Template::render();
	}
	
	/**
	 * Redirect user to their specified area after successful login
	 * @param boolean $otp_login
	 * @param boolean $ajax
	 */
	private function login_redirect($otp_login = false, $ajax = false, $mobile = false) {
	    if($mobile) {
	        return array('success' => true, 'user_id' => $this->auth->user_id());
	    } else {
	        if($otp_login) {
	            // Unset OTP Login Userdata
	            $this->session->unset_userdata('otp_login');
	        }
	        // Log the Activity
	        log_activity($this->auth->user_id(), lang('us_log_logged') . ': ' . $this->input->ip_address(), 'users');
	        
	        /*
	         Now redirect - In many cases, we will have set a destination for a particular user-role to redirect to.
	         This is helpful for cases where we are presenting different information to different roles that might cause the base destination to be not available.
	         */
	        if($this->auth->role_id() == 1) {
	            Template::redirect(SITE_AREA);
	        }
	        if($this->session->userdata('login_redirect_url')) {
	            $login_redirect_url = $this->session->userdata('login_redirect_url');
	            $this->session->unset_userdata('login_redirect_url');
	            $url = $login_redirect_url;
	            // only first 50 characters are being returned from the session variable
	            // if string length is 50 redirect user to the home page
	            if(strlen($login_redirect_url) == 50) {
	                $url = gc_url_format('members');
	            }
	        } else if ($this->settings_lib->item('auth.do_login_redirect') && !empty($this->auth->login_destination)) {
	            $url = $this->auth->login_destination;
	        } else {
	            if (!empty($this->requested_page)) {
	                $url = gc_url_format('members'); //$this->requested_page;
	            } else {
	                $url = gc_url_format('members'); // site_url();
	            }
	        }
	        if($ajax) {
	            echo json_encode(array('result' => 1, 'url' => $url));
	        } else {
	            Template::redirect($url);
	        }
	    }
	}
	
	/**
	 * Get User Information
	 * @param number $id
	 * @param string $return_field
	 */
	public function get_user($id, $return_field = null) {
	    $record = $this->db->where('id', $id)->get('users')->row();
	    if($record) {
	        return !empty($return_field) ? $record->{$return_field} : $record;
	    }
	    return false;
	}
	
	/**
	 * Mobile App User Registration
	 * @param array $form_fields
	 * @return boolean[]|string[]
	 */
	public function register_mobile($form_fields) {
	    if ($user_id = $this->fields_model->save_module($form_fields, 'users', 0, 'frontend', 0, 'insert')) {	        
	        $result = $this->notify_registration($user_id, true);
	    } else {
	        $result = array('success' => false, 'message' => lang('msg_operation_failure'));
	    }
	    return $result;
	}
	
	/**
	 * Notify Registration to the User and the Admin
	 * @param number $user_id
	 * @return boolean[]
	 */
	private function notify_registration($user_id, $mobile = false, $ajax = false) {
	    $user = $this->db->where('id', $user_id)->get('users')->row();
	    // Earn Points
	    // $this->gc->earn_points('users', $user_id, $user_id);
	    /**
	     * NOTE: One package must be set to free
	     * in case if you want to disable all features, set the limit tab accordingly
	     * Disable listings and views
	     */
	    $this->assign_package($user_id); // assign package to the user
	    
	    $subject = '';
	    $email_mess = '';
	    $message = lang('us_email_thank_you');
	    $type = 'success';
	    $site_title = $this->settings_lib->item('site.title');
	    $error = false;
	    $login_url = gc_url_format('login');
	    $register_url = gc_url_format('register');
	    $activation_method = $this->settings_lib->item('auth.user_activation_method');
	    switch ($activation_method) {
	        case 0:
	            // No activation required. Activate the user and send confirmation email
	            $this->db->where(array('id' => $user_id))->update($this->table_name, array('active' => 1));
	            $subject       =  str_replace('[SITE_TITLE]',$this->settings_lib->item('site.title'),lang('us_account_reg_complete'));
	            $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'user_email_templates', 'module_table' => 'users', 'action_type' => 'account_activated', 'id' => $user->id, 'record' => $user));
	            $message      .= lang('us_account_active_login');
	            break;
	        case 1:
	            $activation_code = $this->user_model->deactivate($user_id);
	            $activate_link   = site_url(gc_uri_string('activate') . '/' . $user_id . '/' . $activation_code);
	            $subject         = lang('us_email_subj_activate');
	            $user->link      = $activate_link;
	            $user->link_url  = anchor($activate_link, $activate_link);
	            $user->code      = $activation_code;
	            $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'user_email_templates', 'module_table' => 'users', 'action_type' => 'activation_code', 'id' => $user->id, 'record' => $user));
	            $message        .= lang('us_check_activate_email');
	            break;
	        case 2:
	            // Admin Activation
	            // Clear hash but leave user inactive
	            $subject         =  lang('us_email_subj_pending');
	            
	            //Get email content from database
	            $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'user_email_templates', 'module_table' => 'users', 'action_type' => 'account_pending', 'id' => $user->id, 'record' => $user));
	            $message        .= lang('us_admin_approval_pending');
	            break;
	        case 3:
	            // OTP Activation
	            // Send otp to the user
	            // In case of Web Application, all messages are send
	            /*$activation_code = $this->user_model->deactivate($user_id);
	             $otp = $this->user_model->otp($user_id);
	             $otp_expiry = $this->user_model->otp_expiry();
	             $this->gc->send_sms(sprintf(lang('otp_message'), $otp, site_url(), gc_date_format($otp_expiry)), $this->input->post('phone_number'));*/
	            if($mobile) {
	                $activation_code = $this->user_model->deactivate($user_id);
	                $otp = $this->user_model->otp($user_id);
	                $otp_expiry = $this->user_model->otp_expiry();
	                $this->gc->send_sms(sprintf(lang('otp_message'), $otp, site_url(), gc_date_format($otp_expiry)), $this->input->post('phone_number'));
	            }
	            $subject =  lang('us_email_subj_pending');
	            $message .= lang('otp_account_not_active');
	            break;
	    }//end switch
	    
	    $type = $error ? 'danger': 'success';
	    
	    // Notify Admin on New User Registration
	    $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'user_email_templates', 'module_table' => 'users', 'action_type' => 'add', 'id' => $user_id, 'record' => $user));
	    
	    // Log the Activity
	    log_activity($user_id, lang('us_log_register'), 'users');
	    
	    if($mobile) {
	        $mobile_data = array('result' => 1, 'success' => true, 'message' => $message, 'user_id' => $user_id, 'type' => $type);
	        if(settings_item('auth.user_activation_method') == 3) {
	            $mobile_data['verify_otp'] = true;
	        }
	        return $mobile_data;
	    } else {
	        $url = $login_url;
	        // If the activation method is OTP, redirect user to the activation page
	        if(settings_item('auth.user_activation_method') == 3) {
	            $this->load->library('crypt');
	            $this->session->set_userdata('otp_verify', 'activate');
	            $url = site_url('verify_otp/' . $this->crypt->encode($user_id));
	        }
	        if($ajax) {
	            echo json_encode(array('result' => 1, 'message' => $message, 'type' => $type, 'url' => $url));
	        } else {
	            Template::set_message($message, $type);
	            Template::redirect($url);
	        }
	    }
	}
	
	/**
	 * Rest API - Users Checkout
	 * @param number $user_id
	 * @return array|NULL[]|string[]
	 */
	public function users_checkout($user_id) {
	    $user = $this->get_user($user_id);
	    $data = array();
	    if($user) {
	        $data ['user_id'] = $user->id;
	        $data ['name'] = $user->display_name;
	        $data ['email'] = $user->email;
	        // $data ['company_name'] = $user->company_name;
	        // $data ['company_tax_no'] = $user->company_tax_no;
	        $data ['phone_number'] = !empty($user->phone_number) ? $user->phone_number : '';
	        if(!empty($user->address)) {
	            $data ['address'] = $user->address;
	        }
	        if(!empty($user->postcode)) {
	            $data ['postcode'] = $user->postcode;
	        }
	        if(empty($user->city_id)) {
	            $this->load->model('visitors/visitors_model');
	            $location = $this->visitors_model->get_save_location($this->input->ip_address());
	            if(!empty($location)) {
	                // avoid further lookup
	                $this->db->where('id', $user->id)->update('users', $location);
	                $data = array_merge($data, $location);
	            }
	        } else {
	            $data ['country_iso'] = $user->country_iso;
	            $data ['state_id'] = $user->state_id;
	            $data ['city_id'] = $user->city_id;
	            $data ['country'] = $user->country;
	            $data ['state'] = $user->state;
	            $data ['city'] = $user->city;
	        }
	    }
	    return $data;
	}
	
	/**
	 * Forum - Display Users
	 *
	 * @return array
	 */
	public function users($role = 'admin') {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        )
	    );
	    return $fields;
	}
	
	/**
	 * Get Email Templates
	 * @param number $id
	 * @return object
	 */
	private function get_email_template($id) {
	    return $this->db->where('id', $id)->get('user_email_templates')->row();
	}
	
	/**
	 * Summary
	 * @return array
	 */
	public function summary() {
	    $table = 'users';
	    $where = '';
	    $sql = "SELECT COUNT(*) AS total_users";
	    
	    // today users
	    $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $table;
	    $sql .= " WHERE YEAR(created_on) = YEAR(NOW()) AND MONTH(created_on) = MONTH(NOW()) AND DAY(created_on) = DAY(NOW()) AND active = 1" . $where .") AS today_users";
	    
	    // this month users
	    $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $table;
	    $sql .= " WHERE MONTH(created_on) = MONTH(CURDATE()) AND YEAR(created_on)  =  YEAR(CURDATE()) AND active = 1" . $where .") AS month_users";
	    
	    // yesterday sales
	    $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $table;
	    $sql .= " WHERE created_on <= NOW() AND created_on >= DATE_ADD(Now(),interval - 1 day) AND active = 1" . $where .") AS yesterday_users";
	    
	    $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->table_name . " WHERE active = 0) AS inactive_users";
	    // $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->table_name . " WHERE active = 1) AS active_users";
	    $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->table_name . " WHERE banned = 1) AS banned_users";
	    $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->table_name . " WHERE deleted = 1) AS deleted_users";
	    $sql .= " FROM " . $this->db->dbprefix . $this->table_name;
	    $query = $this->db->query($sql);
	    return $query->row();
	}
	
	/**
	 * Generate Random Password
	 * @param number $_len
	 * @return string
	 */
	public function random_password($_len = 12) {
	    $_alphaSmall = 'abcdefghijklmnopqrstuvwxyz';            // small letters
	    $_alphaCaps  = strtoupper($_alphaSmall);                // CAPITAL LETTERS
	    $_numerics   = '1234567890';                            // numerics
	    $_specialChars = '~!@#%^*()-_=+]}[{/?|';   // Special Characters
	    
	    $_container = $_alphaSmall.$_alphaCaps.$_numerics.$_specialChars;   // Contains all characters
	    $password = '';         // will contain the desired pass
	    
	    for($i = 0; $i < $_len; $i++) {                                 // Loop till the length mentioned
	        $_rand = rand(0, strlen($_container) - 1);                  // Get Randomized Length
	        $password .= substr($_container, $_rand, 1);                // returns part of the string [ high tensile strength ;) ]
	    }
	    return $password;       // Returns the generated Pass
	}
	
	/**
	 * Orders Model - Send Activation Code to the User
	 * @param object $user
	 */
	public function send_activation_code($user) {
	    $user_id = $user->id;
	    if(!empty($user->activate_hash)) {
	        $activation_code = $user->activate_hash;
	    } else {
	        $activation_code = $this->user_model->deactivate($user_id);
	    }
	    $activate_link   = site_url(gc_uri_string('activate') . '/' . $user_id . '/' . $activation_code);
	    $user->link = $activate_link;
	    $user->link_url = anchor($activate_link, $activate_link);
	    $user->code = $activation_code;
	    $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'user_email_templates', 'module_table' => 'users', 'action_type' => 'activation_code', 'id' => $user->id, 'record' => $user));
	}
	
	/**
	 * Perform OTP specific operations
	 * @param string $phone_number
	 * @param number $user_id
	 * @param string $action
	 * @return array|boolean
	 */
	public function otp_actions($phone_number, $user_id = 0, $action = 'select', $user = array()) {
	    if(empty($phone_number)) {
	        $where = 'user_id = ' . $user_id;
	    } else {
	        $where = "phone_number = '" . $phone_number . "'";
	    }
	    $result = false;
	    switch($action) {
	        case 'select':
	            $record = $this->db->where($where)->get($this->table_otp)->row();
	            if($record) {
	                $result = $record;
	            }
	            break;
	        case 'reset':
	            $result = $this->db->where($where)->update($this->table_otp, array('attempts' => 0));
	            break;
	        case 'delete':
	            $result = $this->db->where($where)->delete($this->table_otp);
	            break;
	        case 'generate':
	            $six_digit_random_number = mt_rand(100000, 999999); // substr(number_format(time() * rand(),0,'',''),0,6); // Alternate Method
	            $data = array('phone_number' => $phone_number, 'otp' => $six_digit_random_number, 'attempts' => 1, 'expires_on' => $this->otp_expiry(), 'requested_on' => date('Y-m-d H:i:s'));
	            if(!empty($user_id)) { // get user phone number
	                if(empty($user)) {
	                    $user = $this->get_user($user_id);
	                }
	                if($user) {
	                    $data['user_id'] = $user->id;
	                    $data['phone_number'] = $user->phone_number;
	                }
	            }
	            $record = $this->db->where('phone_number', $data['phone_number'])->get($this->table_otp)->row();
	            if($record) { // update
	                $this->db->where('phone_number', $data['phone_number'])->update($this->table_otp, $data);
	            } else { // insert new otp
	                if(!empty($data['phone_number'])) {
	                    $this->db->insert($this->table_otp, $data);
	                }
	            }
	            $result = $six_digit_random_number;
	            break;
	    }
	    return $result;
	}
}

//end User_model