<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Content controller
 */
class Content extends Admin_Controller
{

	//--------------------------------------------------------------------

	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct()
	{
		parent::__construct();

		$this->auth->restrict('Videos.Content.View');
		$this->load->model('videos_model', null, true);
		$this->load->model('listings/listings_model');
		$this->lang->load('videos');
		
		Template::set_block('sub_nav', 'content/_sub_nav');
		Assets::add_module_js('videos', 'videos.js');
	}
	
	/**
	 * Display Videos to the Admin
	 * @param number $row_id
	 * @param string $toggle_module
	 * @param string $filter
	 * @param number $offset
	 */
	public function index($toggle_module = 'all', $row_id = 0, $filter='all', $offset = 0) {
		//LOAD MODAL FOR YOUTUBE VIDEO PLAYER
		Assets::add_js('js/bootstrap.youtubepopup.js');
		Assets::add_js('js/bootstrap.youtubepopup.min.js');
		$this->session->set_userdata('row_id', $row_id);
		if($toggle_module == 'all') {
			$this->session->unset_userdata('toggle_module', $toggle_module);
		} else {
			$this->session->set_userdata('toggle_module', $toggle_module);
		}
		$redirect_url = SITE_AREA .'/content/videos/index/' . $toggle_module . '/' . $row_id;
		$limit = $this->limit;
		$this->gc->view_module_row_admin('videos', $toggle_module, $row_id, $filter, $offset, $limit, $redirect_url);
	}
	
	/**
	 * Creates a Video object.
	 * @return void
	 */
	public function create($toggle_module, $row_id) {
		$toggle_module_id = gc_module_id($toggle_module);
		if($toggle_module_id == 0 || $row_id == 0) {
			Template::redirect(site_url(SITE_AREA . '/content/videos'));
		}
		$this->session->set_userdata('row_id', $row_id);
		$this->session->set_userdata('toggle_module', $toggle_module);
		$redirect_url = site_url(SITE_AREA . '/content/videos/index');
		$this->gc->module('admin', 'videos', $this->auth->user_id(), $redirect_url, 0, 0, $toggle_module, $row_id);
		Template::set(array(
			'toolbar_title' => lang('add') . ' ' . lang('videos')
		));
		Template::set_view('fields/form');
		Template::render();
	}
	
	/**
	 * Edit a Video object.
	 * @return void
	 */
	public function edit() {
		$id = $this->uri->segment(5);
		if (empty($id)) {
			Template::set_message(lang('invalid_id'), 'error');
			redirect(SITE_AREA .'/content/videos');
		}
		$redirect_url = site_url(SITE_AREA . '/content/videos');
		$this->gc->module('admin', 'videos', $this->auth->user_id(), $redirect_url, 0, $id);
		Template::set(array(
			'toolbar_title' => lang('edit') . ' ' . lang('videos')
		));
		Template::set_view('fields/form');
		Template::render();
	}
	
	/**
	 * Save video information
	 * @param string $type
	 * @param number $id
	 * @return number
	 */
	private function save_videos($type='insert', $id=0)
	{
		$this->load->model('videos_model');
		if ($type == 'update') {
			$_POST['id'] = $id;
		}
	
		$data = array();
		$data['module_id'] = $this->session->userdata('module');
		$data['row_id'] = $this->session->userdata('row_id');	
		if($this->input->post('url')) {
			$data['url']    = get_video_id($this->input->post('url')); //Return video id
		}
		if($this->input->post('title')) { //Retrieve title from youtube and save in database
			$data['title']  = $this->input->post('title');
		} else if($this->input->post('url')) {
			$data['title'] 	= '' .get_video_title($data['url']) .'';
		}
		if(!empty($data['title'])) {
			$data['slug'] 	= $this->gc->slugify('videos', $data['title'], $id);
		}
		$data['description']= $this->input->post('description');
		$data['keywords'] 	= $this->input->post('keywords');
		$data['featured'] 	= $this->input->post('featured') ? 1 : 0;
		$data['status']		= $this->input->post('status') ? 1 : 0;;		
	
		if ($type == 'insert') {
			$id = $this->videos_model->insert($data);	
			if (is_numeric($id)) {
				$return = $id;
			} else {
				$return = FALSE;
			}
		}
		elseif ($type == 'update') {
			$return = $this->videos_model->update($id, $data);
		}	
		return $return;
	}
	
	/**
	 * Manage Training Videos and Groups
	 * @param string $module
	 * @param string $filter
	 * @param number $offset
	 */
	public function manage($module = 'videos', $filter = 'all', $offset = 0) {
	    $extra = array(
	        'model' => 'videos',
	        'context' => 'content',
	        'fields' => $module,
	        'prefix' => 'training'
	    );
	    $extra['actions'] = array( // Extra buttons
	        array('type' => 'link', 'link' => site_url(SITE_AREA . '/content/videos/form/' . $module . '/edit/%s'), 'label' => '<i class="fas fa-edit fa-fw mr-1"></i>' . lang('edit'), 'class' => 'btn btn-xs btn-primary'),
	    );
	    $this->gc->module_view($module, $filter, $offset, 'admin', $extra);
	}
	
	/**
	 * Add/Edit Training Videos Data
	 *
	 * @param string $module
	 * @param string $action
	 *
	 * @return void
	 */
	public function form($module = 'videos', $action = 'create') {
	    /*
	     Assets::add_js(array(
	     'tinymce/tinymce.min.js',
	     ));*/
	    $params = array(
	        'model' => 'videos',
	        'module_name' => 'videos',
	        'context' => 'content',
	        'prefix' => 'training',
	        'fields_method' => $module,
	        'redirect_url' => site_url(SITE_AREA . '/content/videos/manage/' . $module)
	    );
	    if($module == 'videos') {
	        $params['method'] = 'after_video_save';
	    }
	    $this->gcrud->form($module, $action, 'admin', $params);
	}
}