<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Migration_Install_videos extends Migration
{
	/**
	 * The name of the database table
	 *
	 * @var String
	 */
	private $table_name = 'videos';

	/**
	 * The table's fields
	 *
	 * @var Array
	 */
	private $fields = array(
		'id' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'auto_increment' => TRUE,
		),
		'module_id' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default' => 0
		),
		'row_id' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'null' => FALSE,
			'default' => 0
		),
		'user_id' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'null' => FALSE,
			'default' => 0
		),
		'package_id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => FALSE,
			'default' => 0
		),
		'title' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => FALSE,
		),
		'slug' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => FALSE,
		),
		'description' => array(
			'type' => 'TEXT',
			'null' => TRUE,
		),
		'youtube_id' => array(
			'type' => 'VARCHAR',
			'constraint' => 100,
			'null' => TRUE,
		),
		'url' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => FALSE,
		),
		'keywords' => array(
			'type' => 'TEXT',
			'null' => TRUE,
		),
		'external_url' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE,
		),
		'hits' => array(
			'type' => 'INT',
			'constraint' => 11,
			'null' => FALSE,
			'default'=> 0,
		),
		'featured' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default'=> 0,
		),
		'status' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default'=> 0,
		),
		'meta_title' => array(
			'type' => 'VARCHAR',
			'constraint' => 200,
			'null' => TRUE,
		),
		'meta_description' => array(
			'type' => 'TEXT',
			'null' => TRUE,
		),
		'meta_keywords' => array(
			'type' => 'TEXT',
			'null' => TRUE,
		),
		'deleted' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default'=> 0,
		),
		'deleted_by' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'null'=> FALSE,
			'default' => '0'
		),
		'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',		
		'created_by' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'null'=> FALSE,
			'default' => '0'
		),
		'created_ip' => array(
			'type' => 'VARCHAR',
			'constraint' => 50,
			'null'=> TRUE
		),
		'modified_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'modified_by' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'null'=> FALSE,
			'default' => '0'
		),
		'modified_ip' => array(
			'type' => 'VARCHAR',
			'constraint' => 50,
			'null'=> TRUE
		)
	);

	/**
	 * Install this migration
	 *
	 * @return void
	 */
	public function up()
	{
		$this->dbforge->add_field($this->fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('module_id', false);
		$this->dbforge->add_key('row_id', false);
		$this->dbforge->add_key('user_id', false);
		$this->dbforge->add_key('package_id', false);
		$this->dbforge->create_table($this->table_name);
		// CHANGE ENGINE FROM INNODB TO MYISAM
		$change_engine_sql = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " ENGINE = MyISAM";
		$this->db->query ($change_engine_sql);
		// ADD FULLTEXT INDEXT
		$fulltext_sql = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " ADD FULLTEXT INDEX `title_description_keywords` (`title`, `description`, `keywords`)";
		$this->db->query ($fulltext_sql);
		// SET SLUG FIELD COLLATION TO utf8_bin
		$collation_sql = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " CHANGE `slug` `slug` VARCHAR( 255 ) CHARACTER SET utf8 COLLATE utf8_bin NULL DEFAULT NULL";
		$this->db->query ($collation_sql);
	}

	//--------------------------------------------------------------------

	/**
	 * Uninstall this migration
	 *
	 * @return void
	 */
	public function down() {
		$this->dbforge->drop_table($this->table_name);
	}

	//--------------------------------------------------------------------

}