<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class Videos_model extends BF_Model {

    protected $table_name	= "videos";
    protected $table_training = 'training_videos';
    protected $table_groups = 'training_groups';
	protected $key			= "id";
	protected $date_format	= "datetime";

	protected $log_user 	= TRUE;
	protected $soft_deletes	= TRUE;
	protected $set_created	= TRUE;
	protected $set_modified = TRUE;
	
	protected $created_field     = 'created_on';
	protected $created_by_field  = 'created_by';
	protected $modified_field    = 'modified_on';
	protected $modified_by_field = 'modified_by';
	protected $deleted_field     = 'deleted';
	protected $deleted_by_field  = 'deleted_by';

	/*
		Customize the operations of the model without recreating the insert, update,
		etc methods by adding the method names to act as callbacks here.
	 */
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 		= array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	/*
		For performance reasons, you may require your model to NOT return the
		id of the last inserted row as it is a bit of a slow method. This is
		primarily helpful when running big loops over data.
	 */
	protected $return_insert_id 	= TRUE;

	// The default type of element data is returned as.
	protected $return_type 			= "object";

	// Items that are always removed from data arrays prior to
	// any inserts or updates.
	protected $protected_attributes = array();

	/*
		You may need to move certain rules (like required) into the
		$insert_validation_rules array and out of the standard validation array.
		That way it is only required during inserts, not updates which may only
		be updating a portion of the data.
	 */
	protected $validation_rules 		= array(
		array(
			"field"		=> "title",
			"label"		=> "lang:label_title",
			"rules"		=> "max_length[100]|sanitize"
		),
		array(
			"field"		=> "url",
			"label"		=> "lang:label_video_url",
			"rules"		=> "required|xss_clean"
		)
	);
	protected $insert_validation_rules 	= array();
	protected $skip_validation 			= FALSE;
	
	/**
	 * Training Videos Fields
	 *
	 * @return array
	 */
	public function videos($role = 'admin') {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'group_ids',
	            'label' => 'gc_menu_groups',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => true,
	            //'options' => 'load:training_groups;module:training_groups;max_items:100',
	            'options' => 'name:training_groups;value:database',
	            'options_detailed' => 1,
	            'field_extra' => 'selectpicker:1;multiple:1;max_options:10',
	            'rules' => 'trim|max_length[255]',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'title',
	            'label' => 'label_title',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'video_url',
	            'label' => 'label_youtube_url',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim|required|valid_youtube_url',
	        ),
	        array(
	            'type' => 'filemanager',
	            'name' => 'image',
	            'label' => 'label_image',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim|max_length[255]',
	        ),
	        array(
	            'type' => 'textarea',
	            'name' => 'description',
	            'label' => 'label_description',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'priority',
	            'label' => 'label_priority',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'value' => 0,
	            'class' => 2,
	            'rules' => 'trim|required|is_numeric',
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'status',
	            'label' => 'label_status',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_on',
	            'label' => 'created_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_ip',
	            'label' => 'created_ip',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_by',
	            'label' => 'created_by',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_on',
	            'label' => 'modified_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_ip',
	            'label' => 'modified_ip',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_by',
	            'label' => 'modified_by',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        )
	    );
	    return $fields;
	}
	
	/**
	 * Training Group Fields
	 *
	 * @return array
	 */
	public function groups($role = 'admin') {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'name',
	            'label' => 'label_name',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'edit' => 1,
	            'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
	            'rules' => 'trim|required|max_length[150]'
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'name_lang',
	            'label' => 'name_lang',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'textarea',
	            'name' => 'description',
	            'label' => 'label_description',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim|max_length[255]',
	        ),
	        array(
	            'type' => 'filemanager',
	            'name' => 'image',
	            'label' => 'label_image',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim|max_length[255]',
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'status',
	            'label' => 'label_status',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        )
	    );
	    return $fields;
	}
	
	/**
	 * Common Function to Save Video
	 * @param string $role - Member/Admin/Mobile
	 * @param integer $user_id
	 * @param string $redirect_url
	 * @param string $toggle_module
	 * @param number $toggle_row_id
	 * @param number $id
	 * @return multitype:boolean |multitype:boolean unknown |multitype:boolean string
	 */
	public function save($role, $user_id, $redirect_url, $toggle_module = NULL, $toggle_row_id = 0, $id = 0) {
		$this->lang->load('listings/listings');
		$this->load->model('packages/packages_model');
		$this->load->model('members/members_model');
		$module = 'videos';
		$package_id = 0;
		$package = array();
		$frontend = FALSE;
		if($role == 'members' || $role == 'mobile') {
			$frontend = TRUE;
		}
		
		if($role == 'members' || $role == 'admin') {
			Template::set('module', $module);
		}
		// Fields like user_id, package_id, module_id, row_id can be handled via Hidden Fields
		// but they will impose security risk as the external user can modify these values before submitting the form
		// therefore, such parameters are passed in the extra_fields array
		$extra_fields = array();
		
		// DB Transaction Type
		if($id == 0) {
			$transaction_type = 'insert';
			$extra_fields['user_id'] = $user_id;
			$extra_fields['row_id'] = $toggle_row_id;
			$extra_fields['module_id'] = gc_module_id($toggle_module);
		} else {
			$where = array('id' => $id);
			if($frontend) {
				$where['deleted'] = 0;
			}
			// Get existing record and pass it to the View File
			$record = $this->videos_model->find_by($where);
			if(!$record) {
				if($role == 'mobile') {
					return array('success' => FALSE);
				} else {
					Template::redirect($redirect_url);
				}
			} else {
				if($role == 'mobile') {
					return array('success' => TRUE, 'record' => $record);
				} else {
					Template::set('record', $record);
				}
			}
			$toggle_row_id = $record->row_id;
			$toggle_module = gc_module_name($record->module_id);
			$transaction_type = 'update';
		}
		
		// Modify $redirect_url and append the toggle_module and the row_id to redirect user to the Videos Home Page
		if($role == 'admin') {
			$redirect_url = $redirect_url . '/' . $toggle_module . '/' . $toggle_row_id;
		}
		
		// Get package information
		$module_info = $this->gc->get_info($toggle_row_id, $toggle_module);
		if(!$module_info) {
			if($role == 'mobile') {
				return array('success' => FALSE);
			} else {
				Template::redirect($redirect_url);
			}
		}
		
		// Member Area Check Permission - Both Create and Edit
		if($frontend && $module_info->user_id != $user_id) {
			if($role == 'mobile') {
				return array('success' => FALSE, 'message' => lang('msg_permission_expired_error'));
			} else {
				Template::set_message(lang('msg_permission_expired_error'), 'error');
				Template::redirect($redirect_url);
			}
		}
		
		// Get Package information
		$package_id = $module_info->package_id;
		$package = $this->members_model->get_package_info($module_info->user_id, $package_id, $module_info->listing_id, 'packages');
		if(!$package) {
			if($role == 'mobile') {
				return array('success' => FALSE);
			} else {
				Template::redirect($redirect_url);
			}
		}
		
		// Limit Check - Member Area Only
		if($frontend) {
			$user_limit = gc_field_value($toggle_module, $package->module_videos_limit);
			$active_field_value = gc_field_value($toggle_module, $package->module_videos_status);	
			$this->members_model->pyg_check($module, $user_id, $package_id, $package, $toggle_row_id, $toggle_module);
			$extra_fields['package_id'] = $package_id;
			$extra_fields['status'] = $active_field_value;
		}
		
		// Load Fields
		$this->load->model('fields/fields_model');
		$form_fields = $this->fields_model->load_fields($module, $package, ($role == 'members' || $role == 'mobile' ? 'frontend' : 'admin_only'));
		
		// Save Method
		if (isset($_POST['save'])) {
			if ($result = $this->fields_model->save_module($form_fields, $module, $user_id, $role, $id, $transaction_type, $extra_fields)) {
				$insert = false;
				if($id == 0) { // Insert Operation
					$id = $result;
					$insert = true;
				}
				log_activity($user_id, lang('act_create_record') .': '. $id .' : '. $this->input->ip_address(), $module);				
				if($insert && $frontend) {
					// Earn Points
					$this->earn_points($module, $user_id, $id);
				}
				if($role == 'mobile') {
					return array('success' => TRUE, 'message' => sprintf(lang('msg_save_success'), lang('singular_' . $module)));
				} else {
					Template::set_message(sprintf(lang('msg_save_success'), lang('singular_' . $module)), 'success');
					Template::redirect($redirect_url);
				}
			} else {
				if($role == 'mobile') {
					return array('success' => FALSE, 'message' => sprintf(lang('msg_create_failure'), lang('singular_' . $module)));
				} else {
					Template::set_message(sprintf(lang('msg_create_failure'), lang('singular_' . $module)), 'error');
				}
			}
		} else if($role != 'mobile' && $id > 0 && isset($_POST['delete'])) { // Allow user to delete a Video from the Edit Page
			if($role == 'admin') {
				$this->auth->restrict('Listings.Content.Delete');
			}
			if ($this->db->where('id', $id)->update($module, array('deleted' => 1))) {
				log_activity($user_id, lang('act_delete_record') .': '. $id .' : '. $this->input->ip_address(), $module);
				Template::set_message(sprintf(lang('msg_delete_success'), lang('singular_' . $module)), 'success');
				Template::redirect($redirect_url);
			} else {
				Template::set_message(sprintf(lang('msg_delete_failure'), lang('singular_' . $module)), 'error');
			}
		}
	}
	
	/**
	 * Update YouTube Video ID and Title after save
	 * @param number $id
	 * @param string $action_type
	 * @param array $record
	 */
	public function after_video_save($id, $action_type = 'insert', $record = array()) {
	    $record = $this->db->where('id', $id)->get($this->table_training)->row();
	    if($record) {
	        $data = array();
	        $data['youtube_id'] = get_video_id($record->video_url); //Return video id
	        if(empty($record->title)) {
	            $data['title'] = get_video_title($record->video_url);
	        }
	        $this->db->where('id', $id)->update($this->table_training, $data);
	        clear_cache('training_videos');
	    }
	}
	
	/**
	 * Display Training Videos and Groups to the User
	 * @param string $type
	 */
	public function training($type = 'videos', $group_id = 0) {
	    $table = $type == 'videos' ? $this->table_training : $this->table_groups;
	    $records = array();
	    $where = "status = 1";
	    if(!empty($group_id)) {
	        $where .= " AND FIND_IN_SET(" . $group_id . ", group_ids)";
	    }
	    $rows = $this->db->where($where)->order_by('priority')->get($table)->result();
	    if($rows) {
	        if($type == 'groups') {
	            foreach($rows as $row) {
	                $record = array(
	                    'id' => $row->id,
	                    'title' => !empty($row->name_lang) ? lang($row->name) : $row->name,
	                    'description' => $row->description,
	                    'image' => !empty($row->image) ? base_url('assets/images/' . $row->image) : null,
	                    'url' => '/lists/training_videos/' . $row->id
	                );
	                $records[] = $record;
	            }
	        } else {
	            foreach($rows as $row) {
	                $record = array(
	                    'id' => $row->id,
	                    'title' => $row->title,
	                    'description' => $row->description,
	                    'youtube_id' => $row->youtube_id,
	                    'image' => !empty($row->image) ? base_url('assets/images/' . $row->image) : 'https://img.youtube.com/vi/' . $row->youtube_id . '/0.jpg',
	                    'url' => null
	                );
	                $records[] = $record;
	            }
	        }
	    }
	    $total_rows = $this->db->where($where)->count_all_results($table);
	    return array('records' => $records, 'total_rows' => $total_rows);
	}
}
