<?php defined('BASEPATH') || exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/

/**
 * Reports controller
 */
class Reports extends Admin_Controller
{
    protected $permissionDelete = 'Visitors.Reports.Delete';
    protected $permissionView   = 'Visitors.Reports.View';

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
        
        $this->auth->restrict($this->permissionView);
        $this->load->model('visitors/visitors_model');
        $this->lang->load('visitors');
        
        $this->form_validation->set_error_delimiters("<span class='error'>", "</span>");        
        Template::set_block('sub_nav', 'reports/_sub_nav');
    }

    /**
     * Display a list of Visitors data.
     *
     * @return void
     */
    public function index($offset = 0) {
        // Deleting anything?
        if (isset($_POST['delete'])) {
            $this->auth->restrict($this->permissionDelete);
            $checked = $this->input->post('checked');
            if (is_array($checked) && count($checked)) {

                // If any of the deletions fail, set the result to false, so
                // failure message is set if any of the attempts fail, not just
                // the last attempt

                $result = true;
                foreach ($checked as $pid) {
                    $deleted = $this->visitors_model->delete($pid);
                    if ($deleted == false) {
                        $result = false;
                    }
                }
                if ($result) {
                    Template::set_message(count($checked) . ' ' . sprintf(lang('msg_action_success'), lang('label_visitors'), lang('label_deleted')), 'success');
                } else {
                    Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_visitor')) . $this->visitors_model->error, 'danger');
                }
            }
        }
        
        // Search Term
        $like_array = array();
        $searchterm = $this->gc->search('visitors', $this->uri->segment(5), '/reports/visitors/index');
        if(!empty($searchterm)) {
            $like_array = array(
                'ip' => $searchterm,
                'city' => $searchterm,
                'region' => $searchterm,
                'country' => $searchterm,
            );
        }
        
        $pagerUriSegment = 5;
        $pagerBaseUrl = site_url(SITE_AREA . '/reports/visitors/index') . '/';
        if(!empty($like_array)) {
            $this->visitors_model->or_like($like_array);
        }
        $rows_count = $this->visitors_model->count_all();
        $limit  = $this->limit;

        $this->load->library('pagination');
        $pager['base_url']    = $pagerBaseUrl;
        $pager['total_rows']  = $rows_count;
        $pager['per_page']    = $limit;
        $pager['uri_segment'] = $pagerUriSegment;

        $this->pagination->initialize($pager);
        
        // Sort By
        $order_by = $this->gc->sort('visitors');
        if(!empty($order_by)) {
        	$this->visitors_model->order_by($this->session->userdata('visitors_sort')['column'], $this->session->userdata('visitors_sort')['order']);
        } else {
        	$this->visitors_model->order_by('id', 'desc');
        }        
        if(!empty($like_array)) {
            $this->visitors_model->or_like($like_array);
        }
        $this->visitors_model->limit($limit, $offset);
        $records = $this->visitors_model->find_all();

        Template::set('records', $records);
        Template::set('filtered_results', $rows_count);
    	Template::set('toolbar_title', lang('visitors_manage'));
        Template::render();
    }
    
    /**
     * Allows editing of Visitors data.
     * @param number $id - ID of the row
     * @param boolean $is_ip - 
     * @return void
     */
    public function view($id = 0, $is_ip = 0) {
        if(settings_item('lst.map_leaflet') == 1) {
        	Assets::add_css('leaflet.css');
        	Assets::add_js('leaflet.js');
        	Assets::add_module_css('visitors', 'visitors.css');
        }
        Assets::add_module_js('visitors', 'visitors.js');
		
        $id = $this->uri->segment(5);
        if (empty($id)) {
            Template::set_message(lang('invalid_id'), 'danger');
            redirect(SITE_AREA . '/reports/visitors');
        }     
     	if (isset($_POST['delete'])) {
            $this->auth->restrict($this->permissionDelete);
            if ($this->visitors_model->delete($id)) {
                log_activity($this->auth->user_id(), lang('act_delete_record') . ': ' . $id . ' : ' . $this->input->ip_address(), 'visitors');
                Template::set_message(sprintf(lang('msg_action_success'), lang('label_visitor'), lang('label_deleted')), 'success');

                redirect(SITE_AREA . '/reports/visitors');
            }
            Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_visitor')). $this->visitors_model->error, 'danger');
        }
        if($is_ip) {
            $record = $this->visitors_model->find_by(array('ip' => $id));
            if(!$record) { // get information from the API and store it in the database for future purpose
                $this->visitors_model->geoip(false, $id);
                $record = $this->visitors_model->find_by(array('ip' => $id));
            }
        } else {
            $record = $this->visitors_model->find($id);
        }
        if(!$record) {
            Template::set_message(lang('error_no_record_found'), 'danger');
            redirect(SITE_AREA . '/reports/visitors');
        }
        if($record->type_location == 1 && $record->location != 0) {
        	$this->load->model('locations/cities_model');
        	$search_location = $this->cities_model->get_city_info($record->location);        	        	
        } else if($record->type_location == 2 && $record->location != 0) {
        	$this->load->model('locations/states_model');
        	$search_location = $this->states_model->select('name')->find($record->location);
        }
        if($record->location != 0 && $search_location) {
        	Template::set('location', $search_location->name);
        }
        Template::set(array(
            'record' => $record,
            'toolbar_title' => lang('label_view')
        ));
        Template::render();
    }
    
    /**
     * Manage Restricted IPs
     * @param string $module
     * @param string $filter
     * @param number $offset
     */
    public function manage($module = 'restricted_ips', $filter = 'all', $offset = 0) {
        clear_cache('visitor_restricted_ips');
        $this->session->unset_userdata('auth_ip_authenticated');
        $extra = array(
            'model' => 'visitors',
            'context' => 'reports',
            'fields' => $module,
            'prefix' => 'visitor',
            'search_fields' => array('ip_range')
        );
        $extra['actions'] = array( // Extra buttons
            array('type' => 'link', 'link' => site_url(SITE_AREA . '/reports/visitors/form/' . $module . '/edit/%s'), 'label' => '<i class="fas fa-edit fa-fw mr-1"></i>' . lang('edit'), 'class' => 'btn btn-xs btn-primary'),
        );
        $this->gc->module_view($module, $filter, $offset, 'admin', $extra);
    }
    
    /**
     * Add/Edit Restricted IPs Data
     *
     * @param string $module
     * @param string $action
     *
     * @return void
     */
    public function form($module = 'restricted_ips', $action = 'create') {
        $params = array(
            'prefix' => 'visitor',
            'model' => 'visitors',
            'context' => 'reports',
            'fields_method' => $module,
            'redirect_url' => site_url(SITE_AREA . '/reports/visitors/manage/' . $module)
        );
        $this->gcrud->form($module, $action, 'admin', $params);
    }
}