<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
/**
 * Visitors controller
 */
class Visitors extends Front_Controller
{

	//--------------------------------------------------------------------
	
	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct()
	{
		parent::__construct();

		$this->load->model('visitors_model', null, true);
		$this->lang->load('visitors');
	}
	
	/**
	 * Save Visitors information in visitors table
	 */
	public function index() {		
		if(isset($_POST['lat'], $_POST['lng'])) {
			$lat = $_POST['lat'];
			$lng = $_POST['lng'];
	
			$url = sprintf("https://maps.googleapis.com/maps/api/geocode/json?latlng=%s,%s", $lat, $lng);
	
			$content = file_get_contents($url); // get json content
	
			$metadata = json_decode($content ?? '', true); //json decoder
	
			if(count($metadata['results']) > 0) {
				// for format example look at url
				// https://maps.googleapis.com/maps/api/geocode/json?latlng=40.714224,-73.961452
				$result = $metadata['results'][0];
	
				// save it in db for further use
				$location = array();
				$location['ip'] = $this->input->ip_address();
				$location['address'] = $result['formatted_address'];
				foreach ($result['address_components'] as $component) {
					switch ($component['types']) {
						case in_array('locality', $component['types']):
							$location['locality'] = $component['long_name'];
							break;
						case in_array('administrative_area_level_2', $component['types']):
							$location['city'] = $component['long_name'];
							break;
						case in_array('administrative_area_level_1', $component['types']):
							$location['region'] = $component['long_name'];
							$location['region_abbrev'] = $component['short_name'];
							break;
						case in_array('country', $component['types']):
							$location['country'] = $component['long_name'];
							$location['country_abbrev'] = $component['short_name'];
							// GET LOCALE CODE FROM LOCALES TABLE
							$locale = $this->db->select('code, name')->get_where('locales', array('country_iso' => $location['country_abbrev']))->row();
							$location['locale_code'] = $locale->code;
							$location['locale'] = $locale->name;
							break;
						case in_array('postal_code', $component['types']):
							$location['postal_code'] = $component['long_name'];
							break;
					}
				}
				$location['geo_source'] = 2;
				$location['latitude'] = $result['geometry']['location']['lat'];
				$location['longitude'] = $result['geometry']['location']['lng'];
				$location['result'] = serialize($result);
				$location['created_on'] = date('Y-m-d H:i:s');
				$insert_query = $this->db->insert_string('visitors', $location);
				$insert_query = str_replace('INSERT INTO','INSERT IGNORE INTO',$insert_query);
				$this->db->query($insert_query);
			}
		}
	}
	
	/**
	 * Restrict Visitors based on USER IP and show Captcha
	 * To prevent robots and scripts from pulling content from website
	 * @param string $slug
	 */
	public function verification($page = 1) {
	    $this->gc->demo_custom_css_js('index');
		if((int)$page <= 0 && (int)$page > 4) {
			redirect(site_url());
		}
		/**
		 * 1 - Home Page
		 * 2 - Listing Detail Page
		 */
		$url = site_url();
		switch($page) {
			case 2:
				$slug = $this->session->userdata('visitor_verification');
				if(empty($slug[0])) {
					redirect(site_url());
				}
				$url = gc_url_format('listings', $slug[0], $slug[1], $slug[2]);
				break;
		}
		$this->config->load('recaptcha');
		$this->config->set_item('recaptcha_site_key', settings_item('auth.recaptcha_site_key'));
		$this->config->set_item('recaptcha_secret_key', settings_item('auth.recaptcha_secret_key'));
		$this->load->library('recaptcha');
		Assets::add_module_css('listings', 'recaptcha.css');
		if (isset($_POST['submit'])){
			$recaptcha = $this->input->post('g-recaptcha-response');
			if (!empty($recaptcha)) {
				$response = $this->recaptcha->verifyResponse($recaptcha);
				if (isset($response['success']) && $response['success'] === true) {
					$data['counter'] = 0;
					$this->db->where('ip', $this->input->ip_address());
					$this->db->update('visitors', $data);
					$this->session->unset_userdata('visitor_verification');
					redirect($url);
				}
			}
		}
		Template::set( array (
    		'widget' => $this->recaptcha->getWidget(),
    		'script' => $this->recaptcha->getScriptTag(),
		));
		Template::render();
	}
}