<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Migration_Install_visitors extends Migration
{
	/**
	 * The name of the database table
	 *
	 * @var String
	 */
	private $table_name = 'visitors';

	/**
	 * The table's fields
	 *
	 * @var Array
	 */
	private $fields = array(
		'id' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'auto_increment' => TRUE,
		),
		'ip' => array(
			'type' => 'VARCHAR',
			'constraint' => 50,
			'null' => FALSE,
		),
		'user_id' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'null' => FALSE,
			'default'=> 0
		),
		'address' => array(
			'type' => 'VARCHAR',
			'constraint' => 255,
			'null' => TRUE,
		),
		'locality' => array(
			'type' => 'VARCHAR',
			'constraint' => 50,
			'null' => TRUE,
		),
		'city' => array(
			'type' => 'VARCHAR',
			'constraint' => 50,
			'null' => TRUE,
		),
		'region' => array(
			'type' => 'VARCHAR',
			'constraint' => 50,
			'null' => TRUE,
		),
		'region_abbrev' => array(
			'type' => 'CHAR',
			'constraint' => 4,
			'null' => TRUE,
		),
		'country' => array(
			'type' => 'VARCHAR',
			'constraint' => 80,
			'null' => TRUE,
		),
		'country_abbrev' => array(
			'type' => 'CHAR',
			'constraint' => 2,
			'null' => TRUE,
		),
		'locale_code' => array(
			'type' => 'CHAR',
			'constraint' => 3,
			'null' => TRUE,
		),
		'locale' => array(
			'type' => 'VARCHAR',
			'constraint' => 40,
			'null' => TRUE,
		),
		'postal_code' => array(
			'type' => 'VARCHAR',
			'constraint' => 30,
			'null' => TRUE,
		),
		'latitude' => array(
			'type' => 'VARCHAR',
			'constraint' => 30,
			'null' => TRUE,
		),
		'longitude' => array(
			'type' => 'VARCHAR',
			'constraint' => 30,
			'null' => TRUE,
		),
		'result' => array(
			'type' => 'TEXT',
			'null' => TRUE,
		),
		'location' => array(
			'type' => 'VARCHAR',
			'constraint' => 20,
			'null' => TRUE
		),
		'type_location' => array(
			'type' => 'TINYINT',
			'constraint' => 2,
			'null' => FALSE,
			'default'=> 0,
		),
		'language' => array(
			'type' => 'VARCHAR',
			'constraint' => 20,
			'null' => FALSE,
			'default'=> 'engllish',
		),
		'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'last_visit DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'visits' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'null' => FALSE,
			'default'=> 0,
		),
		'counter' => array(
			'type' => 'SMALLINT',
			'constraint' => 5,
			'null' => FALSE,
			'default'=> 0,
		)		
	);

	/**
	 * Install this migration
	 *
	 * @return void
	 */
	public function up()
	{
		$this->dbforge->add_field($this->fields);
		$this->dbforge->add_key('id', true);
		//$this->dbforge->add_key('ip', false);
		$this->dbforge->create_table($this->table_name);
		// CHANGE ENGINE FROM INNODB TO MYISAM
		$change_engine_sql = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " ENGINE = MyISAM";		
		$this->db->query ($change_engine_sql);
		// ADD UNIQUE IP INDEX
		$unique_ip = "ALTER TABLE " . $this->db->dbprefix . $this->table_name . " ADD UNIQUE(`ip`)";
		$this->db->query($unique_ip);
	}

	//--------------------------------------------------------------------

	/**
	 * Uninstall this migration
	 *
	 * @return void
	 */
	public function down()
	{
		$this->dbforge->drop_table($this->table_name);
	}

	//--------------------------------------------------------------------

}