<?php defined('BASEPATH') || exit('No direct script access allowed');
use GeoIp2\Database\Reader;
class Visitors_model extends BF_Model {
    public $table_name	= 'visitors';
    protected $key			= 'id';
    protected $date_format	= 'datetime';
    
    protected $log_user 	= false;
    protected $set_created	= true;
    protected $set_modified = true;
    protected $soft_deletes	= false;
    
    protected $created_field = "created_on";
    protected $modified_field = "last_visit";
    
    // Customize the operations of the model without recreating the insert,
    // update, etc. methods by adding the method names to act as callbacks here.
    protected $before_insert 	= array();
    protected $after_insert 	= array();
    protected $before_update 	= array();
    protected $after_update 	= array();
    protected $before_find 	    = array();
    protected $after_find 		= array();
    protected $before_delete 	= array();
    protected $after_delete 	= array();
    
    // For performance reasons, you may require your model to NOT return the id
    // of the last inserted row as it is a bit of a slow method. This is
    // primarily helpful when running big loops over data.
    protected $return_insert_id = true;
    
    // The default type for returned row data.
    protected $return_type = 'object';
    
    // Items that are always removed from data prior to inserts or updates.
    protected $protected_attributes = array();
    
    // You may need to move certain rules (like required) into the
    // $insert_validation_rules array and out of the standard validation array.
    // That way it is only required during inserts, not updates which may only
    // be updating a portion of the data.
    protected $validation_rules 		= array(
        array(
            'field' => 'ip',
            'label' => 'lang:visitors_field_ip',
            'rules' => 'unique[rs0d_visitors.ip,rs0d_visitors.id]|max_length[50]',
        ),
        array(
            'field' => 'address',
            'label' => 'lang:visitors_field_address',
            'rules' => 'max_length[255]',
        ),
        array(
            'field' => 'city',
            'label' => 'lang:visitors_field_city',
            'rules' => 'max_length[50]',
        ),
        array(
            'field' => 'region',
            'label' => 'lang:visitors_field_region',
            'rules' => 'max_length[50]',
        ),
        array(
            'field' => 'country',
            'label' => 'lang:visitors_field_country',
            'rules' => 'max_length[80]',
        ),
        array(
            'field' => 'postal_code',
            'label' => 'lang:visitors_field_postal_code',
            'rules' => 'max_length[30]',
        ),
        array(
            'field' => 'latitude',
            'label' => 'lang:visitors_field_latitude',
            'rules' => 'max_length[30]',
        ),
        array(
            'field' => 'longitude',
            'label' => 'lang:visitors_field_longitude',
            'rules' => 'max_length[30]',
        ),
        array(
            'field' => 'result',
            'label' => 'lang:visitors_field_result',
            'rules' => '',
        ),
        array(
            'field' => 'created_on',
            'label' => 'lang:visitors_field_created_on',
            'rules' => '',
        ),
        array(
            'field' => 'last_visit',
            'label' => 'lang:visitors_field_last_visit',
            'rules' => '',
        ),
        array(
            'field' => 'visits',
            'label' => 'lang:visitors_field_visits',
            'rules' => 'max_length[20]',
        ),
        array(
            'field' => 'counter',
            'label' => 'lang:visitors_field_counter',
            'rules' => 'max_length[5]',
        ),
    );
    protected $insert_validation_rules  = array();
    protected $skip_validation 			= false;
    
    
    /** @var	GeoIp2\Database\Reader	The MaxMind GeoLite database reader */
    private $reader = null;
    
    /** @var	array	Records for IP addresses already looked up */
    private $lookups = array();
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
    }
    
    /**
     * Visitor Restricted Ips Fields
     *
     * @return array
     */
    public function restricted_ips() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => false,
                'admin_index' => true
            ),
            array(
                'type' => 'text',
                'name' => 'ip_range',
                'label' => 'ip_address',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => true,
                'rules' => 'trim|max_length[455]|unique[visitor_restricted_ips.ip_range, visitor_restricted_ips.id]',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'status',
                'label' => 'label_status',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => true,
                'rules' => 'trim',
            )
        );
        return $fields;
    }
    
    /**
     * Get Users Location
     * @param boolean $return
     * @param string $ip_address
     * @param boolean $max_mind_always - TRUE - get from the max_mind db always
     * @return array|boolean
     */
    public function geoip($return = false, $ip_address = null, $max_mind_always = false, $session_store = false) {
        $ip = empty($ip_address) ? $this->input->ip_address() : $ip_address;
        $location = array();
        if($max_mind_always || settings_item('site.geoip') == 1) {
            $location = $this->maxMind($ip);
        } else if(settings_item('site.geoip') == 3) {
            $geo = json_decode(@file_get_contents("http://extreme-ip-lookup.com/json/$ip"));
            if(!empty($geo)) {
                $location['geo_source'] = 3;
                $location['city'] = $geo->city;
                $location['region'] = $geo->region;
                $location['country_abbrev'] = $geo->countryCode;
                $location['country'] = $geo->country;
                $location['latitude'] = $geo->lat;
                $location['longitude'] = $geo->lon;
            }
        }
        
        if(!empty($location)) {
            $location = $this->get_locale($ip, $location);
            if($session_store) {
                // display listings based on user's location
                $this->session->set_userdata('geo_visitor_info', $location);
            }
            if($return) {
                return $location;
            } else {
                $this->db->on_duplicate('visitors', $location);
            }
            /*$insert_query = $this->db->insert_string('visitors', $location);
             $insert_query = str_replace('INSERT INTO','INSERT IGNORE INTO',$insert_query);
             $this->db->query($insert_query);*/
        }
        return false;
    }
    
    /**
     * Get Information using MaxMind
     * @param string $ip - IP Address
     * @return array
     */
    public function maxMind($ip) {
        $location = array();
        $record = $this->getCountryRecord($ip);
        if($record) {
            $location['city'] = $record->city->name;
            $location['region'] = $record->mostSpecificSubdivision->name;
            $location['region_abbrev'] = $record->mostSpecificSubdivision->isoCode;
            $location['country_abbrev'] = $record->country->isoCode;
            $location['country'] = $record->country->name;
            $location['postal_code'] = $record->postal->code;
            $location['latitude'] = $record->location->latitude;
            $location['longitude'] = $record->location->longitude;
        }
        return $location;
    }
    
    /**
     * Get Locale Information
     * @param string $ip - IP Address
     * @param array $location - Data fetched from GEO services
     * @return array
     */
    private function get_locale($ip, $location) {
        $location['ip'] = $ip;
        // GET LOCALE/LANGUAGE CODE FROM LOCALES TABLE
        $locale = $this->db->select('code, name')->get_where('locales', array('country_iso' => $location['country_abbrev']))->row();
        $location['locale_code'] = !empty($locale) ? $locale->code : '';
        $location['locale'] = !empty($locale) ? $locale->name : '';
        $location['created_on'] = date('Y-m-d H:i:s');
        return $location;
    }
    
    /**
     * Gets a raw location information from an IP address
     *
     * @param   string  $ip  The IP address to look up	 *
     * @return  mixed  A \GeoIp2\Model\City record if found, false
     * if the IP address is not found, null
     * if the db can't be loaded
     */
    public function getCountryRecord($ip) {
        if (!array_key_exists($ip, $this->lookups)) {
            try {
                $reader = new Reader(realpath(FCPATH . '../vendor/geoip2/geoip2/maxmind-db/GeoLite2-City.mmdb'));
            } catch(\Exception $e) {
                $reader = null;
            }
            try {
                if(!is_null($reader)) {
                    $this->lookups[$ip] =  $reader->city($ip);
                } else {
                    $this->lookups[$ip] = null;
                }
            } catch (\GeoIp2\Exception\AddressNotFoundException $e) {
                $this->lookups[$ip] = false;
            } catch (\MaxMind\Db\Reader\InvalidDatabaseException $e) {
                $this->lookups[$ip] = null;
            } catch (Exception $e) {
                $this->lookups[$ip] = null;
            }
        }
        return $this->lookups[$ip];
    }
    
    /**
     * Record number of visitors visited
     * @param Dotted Decimal $ip
     */
    public function record_visitors($ip) {
        $sql = "INSERT INTO " . $this->db->dbprefix . "visitors (ip, user_id, created_on, last_visit, visits, counter) VALUES( '$ip', " . (int) $this->auth->user_id() . ", NOW(), NOW(), 1, 1 ) ON DUPLICATE KEY UPDATE `last_visit` = NOW(), visits=visits+1, counter=counter+1";
        $this->db->query($sql);
    }
    
    /**
     * Call this method where you want to enable the visitors tracking
     * Initially it is kept in the HOME, LISTING VIEW and LISTING DETAIL PAGE
     * @param $page - 1 -> HOME PAGE, 2 - LISTING DETAIL PAGE
     * @param $slug - URL where user to be redirected after successfuly authentication
     */
    public function visitor_tracking($page, $url = array()) {
        $ip_address = $this->input->ip_address();
        // find the last visit date
        $result = $this->visitors_model->select('counter, visits')->find_by(array('ip' => $ip_address, 'DATE(last_visit)' => date('Y-m-d')));
        // if the date is today update the counter else reset and update
        if($result) {
            /**
             * For bots and spider
             * Also protect website content from copying
             * Display captcha if the user crossed per day page browse limit
             */
            if(settings_item('auth.restrict_visitors') == 1) {
                $last_visit = date('Y-m-d H:m:s',strtotime('-24 hours'));
                if((int)$result->counter > settings_item('auth.per_day_limit')) {
                    $this->session->set_userdata('visitor_verification', $url);
                    redirect(site_url('visitors/verification/' . $page));
                }
            }
            
            $data = array(
                'last_visit' => date('Y-m-d H:i:s'),
                'visits' => ((int)$result->visits) + 1,
                'counter' => ((int)$result->counter) + 1
            );
            $this->db->update('visitors', $data,  array('ip' => $ip_address));
        } else { // reset and update
            // update location in the visitors table
            $count = $this->db->where('ip', $ip_address)->count_all_results('visitors');
            if($count == 0) { // update only if the information does not exist in the database
                $geo_location = $this->geoip(true);
                if(empty($geo_location)) {
                    $geo_location = $this->maxMind($ip_address);
                    if(!empty($geo_location)) {
                        $geo_location = $this->get_locale($ip_address, $geo_location);
                    }
                }
                if(!empty($geo_location)) {
                    // update information in the visitors table
                    $this->db->insert('visitors', $geo_location);
                }
            }
            // User ID is stored for future reference
            $sql = "INSERT INTO " . $this->db->dbprefix . "visitors (ip, user_id, created_on, last_visit, visits, counter) VALUES( '$ip_address', " . (int) $this->auth->user_id() . ", NOW(), NOW(), 1, 1 ) ON DUPLICATE KEY UPDATE `last_visit` = '" . date('Y-m-d H:i:s') ."', visits=visits+1, counter = 1";
            $this->db->query($sql);
        }
    }
    
    /**
     * Update Country ISO, State ID and City ID in the visitors table for future action
     * Saved data will be used to display the last searched location in the location field
     * @param number $type_location
     * @param number $location_id
     */
    public function update_visitor_info($type_location, $location_id) {
        if(($type_location == 1 || $type_location == 2) && $location_id != 0) {
            $visitors_data = array(
                'location' => $location_id,
                'type_location' => $type_location
            );
            $this->db->update('visitors', $visitors_data, array('ip' => $this->input->ip_address()));
        }
    }
    
    /**
     * Get and Save Location - checkout without user location
     * @param string $ip_address
     * @return string[]
     */
    public function get_save_location($ip_address = '112.196.183.152') {
        $result = $this->maxMind($ip_address);
        $location = array();
        if($result) {
            $this->load->model('locations/cities_model');
            $location = $this->cities_model->getCityStateCountry($result['city'], $result['region_abbrev'], $result['region'], $result['country_abbrev'], $result['country']);
            if($location) {
                $location['country'] = $result['country'];
                $location['state'] = $result['region'];
                $location['city'] = $result['city'];
                $location['address'] = $location['city'] . ', ' . $location['state'] . ' - ' . $result['postal_code'] . ', ' . $location['country'];
                $location['postcode'] = $result['postal_code'];
            }
        }
        return $location;
    }
    
    /**
     * Check IP for restrictions
     */
    public function check_ip_restrictions() {
        $ip_address = $this->input->ip_address();
        $records = $this->gc->get('visitor_restricted_ips', array('where' => 'status = 1 AND deleted = 0', 'result' => 1));
        if(!empty($records)) {
            foreach($records as $record) {
                if(ip_in_range($ip_address, $record->ip_range)) {
                    return true;
                }
            }
        }
        return false;
    }
}