<?php defined('BASEPATH') || exit('No direct script access allowed');

/**
 * Financial controller
 * View files are in the application/view/wallets_points folder
 */
class Financial extends Admin_Controller {
    protected $context = 'financial';
    protected $moduleName = 'wallets';

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
        
        $this->auth->restrict('Wallets.Financial.View');
        $this->load->model('wallets/wallets_points_model');
        $this->lang->load('wallets');
        $this->form_validation->set_error_delimiters("<span class='error'>", "</span>");        
        Template::set_block('sub_nav', 'financial/_sub_nav');
    }
    /**
     * Index Method
     * @param number $filter
     */
    public function index($filter = 0) {
    	$this->wallets_points_model->index('wallets', $filter);
    }
    
    /**
     * Add Wallet
     */
    public function create() {
    	$this->wallets_points_model->create('wallets');
    }
    
    /**
     * Edit Wallet
     */
    public function edit() {
    	$id = $this->uri->segment(5);
    	$this->wallets_points_model->edit($id, 'wallets');
    }
    
    /**
     * View Wallet History
     * @param number $filter
     */
    public function history($filter = 0) {
    	$this->wallets_points_model->history('wallets', $filter);
    }
    
    /**
     * Save settings
     */
    public function settings() {
		clear_cache('site_settings'); // Clear Site Settings Cache
    	$this->load->model('settings/settings_model');
    	if (isset($_POST['save'])) {
    		$this->form_validation->set_rules('minimum', 'lang:minimum_amount', 'trim|required|is_decimal');
    		$this->form_validation->set_rules('maximum', 'lang:maximum_amount', 'trim|required|is_decimal');
    		if($this->form_validation->run() == TRUE) {
    			$settings = array(
    				'amount' => array (
    					'minimum' => $this->input->post('minimum'),
    					'maximum' => $this->input->post('maximum')
    				)    					
    			);
    			$data = array (array('name' => 'user.wallets', 'value' => json_encode($settings)));
    			$updated = $this->settings_model->update_batch($data, 'name');
    			if ($updated) {
    				Template::set_message(sprintf(lang('msg_save_success'), lang('gc_menu_settings')), 'success');
    				Template::redirect(site_url(SITE_AREA . '/financial/wallets/settings'));
    			} else {
    				Template::set_message(sprintf(lang('msg_edit_failure'), lang('gc_menu_settings')), 'error');
    			}
    		}
    	}
    	$setting_name = 'user.wallets';
    	$result = $this->db->select('value')->get_where('settings', array('name' => $setting_name))->row();
    	if($result) {
    	    $settings = json_decode($result->value ?? '');
    	} else {
    	    $settings = json_decode(settings_item($setting_name) ?? '');
    	}
    	Template::set('settings', $settings);
    	Template::set('toolbar_title', lang('gc_menu_settings'));
    	Template::render();
    }
    
    /**
     * Wallet Pay
     * @param number $action_type
     * 1 - Subscription Upgrade
     * 2 - SMS Payment
     * 3 - Lead Payment
     * @param number $row_id - Lead ID can be directly passed
     * @param string $role
     * @param array $rest_extra - Mobile Parameters
     */
    public function pay($action_type = 0, $lead_id = 0, $role = 'admin', $rest_extra = array()) {
        $this->lang->load('orders/orders');
        $this->load->model('packages/packages_model');
        $this->load->model('leads/leads_model');
        $this->load->model('users/user_model');
        $redirect_url = $this->previous_page;
        $mobile = $role == 'mobile' ? true : false;
        $package = '';
        $package_price = '';
        $amount = 0;
        $user_id = $this->auth->user_id();
        $user = $this->db->where('id', $user_id)->get('users')->row();
        $franchise_params = json_decode($user->franchise_params ?? '', true);
        $wallet_balance = $user->wallets;
        switch($action_type) {
            case 1:
                $wallet_change_package = $this->session->userdata('wallet_change_package');
                if(empty($wallet_change_package)) {
                    if($mobile) {
                        return array('success' => false);
                    } else {
                        Template::redirect(SITE_AREA . '/content/profiles');
                    }
                }
                $package_id = $wallet_change_package['package_id'];
                
                // package payment
                $record_package = $this->packages_model->get_package($package_id);
                if($record_package) {
                    // the order will be created automatically on payment confirmation
                    // if order is already there
                    $package = $record_package->title . '<br />' . $record_package->description;
                    $package_price = calculate_savings($record_package->price, $record_package->offer_price); //!empty($record_package->offer_price) ? $record_package->offer_price : $record_package->price;
                    $package_price = !empty($package_price['html']) ? currency_format($record_package->offer_price) . ' ' . $package_price['html'] : currency_format($record_package->price);
                    $amount = $record_package->franchise_price;
                }
                $toolbar_title = lang('wallet_package_upgrade');
                break;
            case 2:
                $sms_settings = json_decode(settings_item('sms.settings'), true);
                $min_qty = !empty($sms_settings['min_qty']) ? $sms_settings['min_qty'] : 1000;
                $sms_price = !empty($sms_settings['sms_price']) ? $sms_settings['sms_price'] : 0.25;
                $amount = $min_qty * $sms_price;
                if(!$mobile) {
                    Assets::add_js(
                        "
        $('#qty').blur(function() {
            inputValue = $('#qty').val();
            if(inputValue < " . $min_qty . ") {
                $('#qty').val(" . $min_qty . ");
                $('.pay-amount').html('" . currency_format($amount) . "');
            } else {
                $('#qty_loader').show();
                $('#qty').prop('disabled', true); // disable the qty field
                $.getJSON('" . site_url('ajax/sms_price?qty=') . "' + $('#qty').val(), function(data) {
                    $('.pay-amount').html(data);
                     // enable the qty field
                    $('#qty').prop('disabled', false);
                    $('#qty_loader').hide();
                });
            }
        });", 'inline');
                    Template::set(array(
                        'min_qty' => $min_qty,
                        'sms_price' => $sms_price
                    ));
                }
                $toolbar_title = lang('wallet_buy_sms');
                break;
            case 3:
                $lead = $this->leads_model->get_lead((int) $lead_id);
                if(!$lead || !empty($lead->assigned_to)) {
                    if($mobile) {
                        return array('success' => false);
                    } else {
                        Template::redirect(SITE_AREA . '/financial/leads');
                    }
                }
                $amount = $lead->price;
                $toolbar_title = lang('wallet_buy_lead');
                break;
            default:
                Template::redirect(site_url(SITE_AREA . '/content/profiles'));
        }
        if(isset($_POST['save'])) {
            // if not a sufficient balance, redirect user to the checkout page
            switch($action_type) {
                case 1:
                    break;
                case 2:
                    $qty = (int) $this->input->post('qty');
                    $amount = $qty * $sms_price;
                    // credit SMS balance
                    break;
                case 3:
                    break;
            }
            // check amount in the wallet
            if($user->wallets <= $amount) {
                $remaining_amount = $amount - $user->wallets;
                $message = sprintf(lang('wallet_low_amount'), currency_format($user->wallets), currency_format($remaining_amount), lang('click_here'));
                if($mobile) {
                    return array('success' => false, 'message' => $message);
                } else {
                    Template::set_message($message, 'danger');
                }
            } else {
                switch($action_type) {
                    case 1:
                        // do not deduct full amount from the users account.
                        // record it as a discount for internal audit purpose.
                        // create a new order for payment receipt and notifications
                        $this->load->model('orders/orders_model');
                        $result = $this->orders_model->direct_checkout($package_id, $record_profile->user_id, array('payment_method' => 'franchise'));
                        $order_id = $result['order_id'];
                        // unset the parameters passed from the change_package method
                        $this->session->unset_userdata('wallet_change_package');
                        // save profiles and user_id for reporting purpose
                        $transaction = 'packages';
                        $module = 'profiles';
                        $row_id = $record_profile->user_id; // whose profile is updated
                        $summary = $record_profile->profile_id . ' ' . $record_profile->name . ' (' . $record_package->title . ')';
                        break;
                    case 2:
                        $sms_balance = empty($franchise_params['sms_balance']) ? 0 : $franchise_params['sms_balance'];
                        $franchise_params['sms_balance'] = $sms_balance + $qty;
                        $this->db->where('id', $user_id)->update('users', array('franchise_params' => json_encode($franchise_params)));
                        // print payment receipt from wallets history
                        $transaction = 'sms';
                        $module = 'sms';
                        $row_id = $user_id;
                        $summary = $qty;
                        break;
                    case 3: // leads payment
                        // update assigned_to in the leads table
                        $data = array(
                            'assigned_to' => $user_id,
                            'assigned_on' => date('Y-m-d H:i:s'),
                            'assigned_ip' => $this->input->ip_address(),
                            'modified_on' => date('Y-m-d H:i:s'),
                            'modified_ip' => $this->input->ip_address()
                        );
                        $this->db->where('id', $lead_id)->update('leads', $data);
                        $transaction = 'leads';
                        $module = 'leads';
                        $row_id = $lead_id;
                        $summary = $record_profile->profile_id . ' ' . $record_profile->name;
                        $redirect_url = site_url(SITE_AREA . '/financial/leads');
                }
                // deduct amount from the wallet
                $extra = array(
                    'user_id' => $user_id,
                    'module' => $module,
                    'row_id' => $row_id,
                    'summary' => $summary
                );
                if(!empty($order_id)) { // save order_id in the wallets table for reporting purpose
                    $extra['order_id'] = $order_id;
                }
                $this->load->model('wallets/wallets_points_model');
                $this->wallets_points_model->update_cash('wallets', $user_id, $transaction, $amount, 'debit', null, $extra);
                $message = sprintf(lang('wallet_payment_success'), currency_format($user->wallets - $amount));
                // update session variable and display the credit information
                $this->auth->profile_session($this->auth->user_id(), true);
                if($mobile) {
                    return array('success' => true, 'message' => $message);
                } else {
                    Template::set_message($message, 'success');
                    Template::redirect($redirect_url); // redirect user based on the action type
                }
            }
        }
        $data = array(
            'package' => $package,
            'package_price' => $package_price,
            'amount' => currency_format($amount),
            'wallet_balance' => currency_format($wallet_balance),
            'action_type' => $action_type,
            'toolbar_title' => $toolbar_title
        );
        if($wallet_balance < $amount) {
            $message = sprintf(lang('wallet_low_amount'), currency_format($wallet_balance), currency_format($amount - $wallet_balance), lang('click_here'));
            if($mobile) {
                $data['success'] = true;
                $data['message'] = $message;
            } else {
                Template::set_message($message, 'danger');
            }
        }
        if(!$mobile) {
            Template::set($data);
            Template::render();
        }
    }
    
    /**
     * Wallet Checkout
     */
    public function checkout() {
        $user_wallets = json_decode(settings_item('user.wallets'), true);
        $min_amount = $user_wallets['amount']['minimum'];
        $max_amount = $user_wallets['amount']['maximum'];
        Assets::add_js("
        $('#amount').blur(function() {
            inputValue = parseFloat($('#amount').val());
            if(inputValue < " . $min_amount . ") {
                $('#amount').val(" . $min_amount . ");
            } else if(inputValue > " . $max_amount . ") {
                $('#amount').val(" . $max_amount . ");
            } else if(isNaN(inputValue)) {
                $('#amount').val(" . $max_amount . ");
            }
        });", 'inline');
        if(isset($_POST['save'])) {
            $amount = floatval($this->input->post('amount'));
            if($amount < $min_amount || $amount > $max_amount) {
                $amount = $max_amount;
            }
            $payment_method = $this->input->post('payment_method');
            $this->session->set_userdata('checkout_redirect', SITE_AREA . '/content/profiles');
            $this->load->model('orders/orders_model');
            $result = $this->orders_model->direct_checkout(0, $this->auth->user_id(), array('order_type' => 5, 'amount' => $amount, 'payment_method' => $payment_method));
            if(!empty($result['url'])) {
                Template::redirect($result['url']);
            }
        }
        $this->load->model('payment_gateways/payment_gateways_model');
        $payment_gateways = $this->payment_gateways_model->select('name, display_name, is_default')->find_all_by(array('status' => 1));
        Template::set(array(
            'min_amount' => $min_amount,
            'max_amount' => $max_amount,
            'payment_gateways' => $payment_gateways,
            'toolbar_title' => lang('label_checkout')
        ));
        Template::render();
    }
    
    /**
     * Print Receipt
     * @param number $transaction_id
     */
    public function receipt($transaction_id) {
        $this->wallets_points_model->email_print('wallets', $transaction_id, 'view');
    }
    
    /**
     * Display a list of deliveries data
     * @param string $module
     * @param string $filter
     * @param number $offset
     */
    public function manage($module = 'email_templates', $filter = 'all', $offset = 0) {
        $extra = array(
            'model' => 'wallets', // $module == 'withdraws' ? 'wallets' : 'wallets_points',
            'module_name' => $this->moduleName,
            'context' => $this->context,
            'fields' => $module,
            'prefix' => 'wallets',
            'table_prefix' => 'wallets',
            'module_name' => $this->moduleName
        );
        $this->gc->module_view($module, $filter, $offset, 'admin', $extra);
    }
    
    /**
     * Add/Edit Email Templates Data
     *
     * @return void
     */
    public function form($module = 'email_templates', $action = 'create') {
        $params = array(
            'module_name' => $this->moduleName,
            'model' => 'wallets', // $module == 'withdraws' ? 'wallets' : 'wallets_points',
            'context' => $this->context,
            'fields_method' => $module,
            'set_table' => true,
            'uri_segment' => 7,
            'redirect_url' => site_url(SITE_AREA . '/' . $this->context . '/' . $this->moduleName . '/manage/' . $module)
        );
        $this->gcrud->form('wallets_' . $module, $action, 'admin', $params);
    }
}