<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

class Migration_Install_wallets extends Migration
{
	/**
	 * The name of the database table
	 *
	 * @var String
	 */
	private $table_name = 'wallets_points';
	
	/**
	 * The history's fields
	 *
	 * @var Array
	 */
	private $fields = array(
		'id' => array(
			'type' => 'INT',
			'constraint' => 11,
			'auto_increment' => TRUE,
		),
		'module_id' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default'=> 0
		),
		'row_id' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'null' => FALSE,
			'default'=> 0
		),
		'user_id' => array(
			'type' => 'BIGINT',
			'constraint' => 20,
			'null' => FALSE,
			'default'=> 0
		),
		'type_id' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default'=> 0
		),
		'opening_balance' => array(
			'type' => 'float',
			'null' => FALSE,
			'default'=> 0,
		),
		'debit' => array(
			'type' => 'float',
			'null' => FALSE,
			'default'=> 0,
		),
		'credit' => array(
			'type' => 'float',
			'null' => FALSE,
			'default'=> 0,
		),
		'closing_balance' => array(
			'type' => 'float',
			'null' => FALSE,
			'default'=> 0,
		),
		'notes' => array(
			'type' => 'text',
			'null' => TRUE,
		),
		'ip_address' => array(
			'type' => 'varchar',
			'constraint' => 50,
			'null' => TRUE,
		),
		'created_on DATETIME NULL DEFAULT CURRENT_TIMESTAMP',
		'status' => array(
			'type' => 'TINYINT',
			'constraint' => 1,
			'null' => FALSE,
			'default'=> 1
		),
	    'is_affiliate' => array(
	        'type' => 'TINYINT',
	        'constraint' => 1,
	        'null' => FALSE,
	        'default'=> 0
	    )
	);

	/**
	 * Install this migration
	 *
	 * @return void
	 */
	public function up()
	{
		
		// Wallet History Table
		$this->dbforge->add_field($this->fields);
		$this->dbforge->add_key('id', true);
		$this->dbforge->add_key('user_id', false);
		$this->dbforge->add_key('module_id', false);
		$this->dbforge->create_table($this->table_name);
		
		// Points settings row
		$this->db->insert('settings', array('name' => 'user.points', 'module' => 'user', 'value' => '{"points_conversion":"100","minimum_points":"1000","modules":{"listings":"5","blogs":"2","classifieds":"1","deals":"1","events":"0","jobs":"0","products":"0","properties":"0","videos":"0","images":"0","users":"5","comments":"12"}}'));
		
		// Wallet settings row
		$this->db->insert('settings', array('name' => 'user.wallets', 'module' => 'user', 'value' => '{"amount":{"minimum":"5","maximum":"10"}}'));
	}

	//--------------------------------------------------------------------

	/**
	 * Uninstall this migration
	 *
	 * @return void
	 */
	public function down() {
		$this->dbforge->drop_table($this->table_name);
		// Remove settings row
		$points_settings = $this->db->select('name')->get_where('settings', array('name' => 'user.points'));
		if($points_settings) {
			$this->db->delete('settings', array('name' => 'user.points'));
		}
		$wallets_settings = $this->db->select('name')->get_where('settings', array('name' => 'user.wallets'));
		if($wallet_settings) {
			$this->db->delete('settings', array('name' => 'user.wallets'));
		}
	}

	//--------------------------------------------------------------------

}