<?php defined('BASEPATH') || exit('No direct script access allowed');

class Wallets_model extends BF_Model {
    protected $table_name	= 'wallets';
    protected $key			= 'id';
    protected $date_format	= 'datetime';
    
    protected $log_user 	= false;
    protected $set_created	= false;
    protected $set_modified = false;
    protected $soft_deletes	= false;
    
    
    // Customize the operations of the model without recreating the insert,
    // update, etc. methods by adding the method names to act as callbacks here.
    protected $before_insert 	= array();
    protected $after_insert 	= array();
    protected $before_update 	= array();
    protected $after_update 	= array();
    protected $before_find 	    = array();
    protected $after_find 		= array();
    protected $before_delete 	= array();
    protected $after_delete 	= array();
    
    // For performance reasons, you may require your model to NOT return the id
    // of the last inserted row as it is a bit of a slow method. This is
    // primarily helpful when running big loops over data.
    protected $return_insert_id = true;
    
    // The default type for returned row data.
    protected $return_type = 'object';
    
    // Items that are always removed from data prior to inserts or updates.
    protected $protected_attributes = array();
    
    // You may need to move certain rules (like required) into the
    // $insert_validation_rules array and out of the standard validation array.
    // That way it is only required during inserts, not updates which may only
    // be updating a portion of the data.
    protected $validation_rules 		= array(
        array(
            'field' => 'user_id',
            'label' => 'lang:label_user',
            'rules' => 'trim|required',
        )
    );
    protected $insert_validation_rules  = array();
    protected $skip_validation 			= false;
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
    }
    
    /**
     * Email Template Fields
     *
     * @return array
     */
    public function email_templates($role = 'admin') {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'dropdown',
                'name' => 'module_table',
                'label' => 'label_module',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:gc_menu_wallets;value:wallets|name:gc_menu_points;value:points',
                'options_detailed' => 1,
                'options_lang' => 1,
                'class' => 3,
                'rules' => 'trim|unique_email_template[wallets]',
            ),
            array(
                'type' => 'dropdown',
                'name' => 'user_type',
                'label' => 'bf_user',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:placeholder_select;value:|name:us_admin;value:admin|name:bf_user;value:user',
                'options_detailed' => 1,
                'options_lang' => 1,
                'class' => 3,
                'rules' => 'trim',
            ),
            array(
                'type' => 'dropdown',
                'name' => 'action_type',
                'label' => 'label_action',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:placeholder_select;value:|name:add;value:add|name:edit;value:edit|name:delete;value:delete|name:em_status_update;value:status_update|name:wallet_receipt;value:receipt',
                'options_detailed' => 1,
                'options_lang' => 1,
                'class' => 3,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'name',
                'label' => 'label_name',
                'placeholder' => 'placeholder_name',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
                'rules' => 'required|trim|max_length[50]'
            ),
            array(
                'type' => 'text',
                'name' => 'subject',
                'label' => 'label_subject',
                'placeholder' => 'placeholder_subject',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'required|trim|max_length[255]'
            ),
            array(
                'type' => 'textarea',
                'name' => 'message',
                'label' => 'label_message',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'custom_rules' => 'editor',
                'field_class' => 'editor',
                'class' => 9,
                'rules' => 'trim|required',
            ),
            array(
                'type' => 'textarea',
                'name' => 'sms_text',
                'label' => 'sms_text',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'class' => 9,
                'field_extra' => 'rows:3',
                'rules' => 'trim',
            ),
            array(
                'type' => 'textarea',
                'name' => 'params',
                'label' => 'em_params',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'class' => 9,
                'tooltip' => lang('em_params_help'),
                'field_extra' => 'rows:3',
                'rules' => 'trim',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'status',
                'label' => 'label_status',
                'frontend' => false,
                'admin_only' => false,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            )
        );
        return $fields;
    }
    
    /**
     * Wallet Withdraw Fields
     *
     * @return array
     */
    public function withdraws($role = 'admin') {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'selectize',
                'name' => 'user_id',
                'label' => 'bf_user',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:gc_menu_wallets;value:wallets|name:gc_menu_points;value:points',
                'options_detailed' => 1,
                'class' => 3,
                'rules' => 'trim|required',
            ),
            array(
                'type' => 'text',
                'name' => 'amount',
                'label' => 'label_amount',
                'placeholder' => 'wallet_amount',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|required'
            ),
            array(
                'type' => 'checkbox',
                'name' => 'status_id',
                'label' => 'label_status',
                'frontend' => false,
                'admin_only' => false,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_by',
                'label' => 'created_by',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_ip',
                'label' => 'created_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_by',
                'label' => 'modified_by',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_ip',
                'label' => 'modified_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            )
        );
        return $fields;
    }
    
}