<?php defined('BASEPATH') || exit('No direct script access allowed');

class Wallets_points_model extends BF_Model
{
    protected $table_name	= 'wallets_history';
    protected $table_withdraws	= 'wallets_withdraws';
    protected $table_points	= 'points_history';
	protected $key			= 'id';
	protected $date_format	= 'datetime';

	protected $log_user 	= false;
	protected $set_created	= false;
	protected $set_modified = false;
	protected $soft_deletes	= false;


	// Customize the operations of the model without recreating the insert,
    // update, etc. methods by adding the method names to act as callbacks here.
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 	    = array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	// For performance reasons, you may require your model to NOT return the id
	// of the last inserted row as it is a bit of a slow method. This is
    // primarily helpful when running big loops over data.
	protected $return_insert_id = true;

	// The default type for returned row data.
	protected $return_type = 'object';

	// Items that are always removed from data prior to inserts or updates.
	protected $protected_attributes = array();

	// You may need to move certain rules (like required) into the
	// $insert_validation_rules array and out of the standard validation array.
	// That way it is only required during inserts, not updates which may only
	// be updating a portion of the data.
	protected $validation_rules 		= array(
		array(
			'field' => 'user_id',
			'label' => 'lang:label_user',
			'rules' => 'trim|required',
		)
	);
	protected $insert_validation_rules  = array();
	protected $skip_validation 			= false;

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
    }
    
    /**
     * Update User Wallets/Points information
     * @param int $table - wallets/points
     * @param int $user_id
     * @param float $amount
     * @param string $type
     * @param array $extra - Extra information to be saved in the history table
     * @return boolean
     */
    public function update_cash($table, $user_id, $module, $amount, $type = 'debit', $notes = null, $row_id = 0, $extra = array()) {
        $cash = 0;
        $amount = (float) $amount;
        $result = $this->db->select($table . ' AS cash')->get_where('users', array('id' => $user_id))->row();
        if(!$result) {
            return false;
        }        
        $cash = (float) $result->cash;
        
        // prepare closing balance
        // negative balance is allowed
        $closing_balance = ($type == 'credit' ? $cash + $amount : $cash - $amount);
        
        // update closing balance
        $this->db->where('id', $user_id)->update('users', array($table => $closing_balance));
        
        // save history
        $history_id = $this->save_history($table, $user_id, $this->type_id($module), $amount, $cash, $type, $notes, $row_id, $extra);
        return $history_id;
    }
    
    /**
     * Points conversion to Cash
     * @param number $user_id
	 * @param number $points - Number of points to be converted to cash
     * @return boolean
     */
    public function points_conversion($user_id, $points = 0) {
    	$result = $this->db->select('wallets, points')->get_where('users', array('id' => $user_id))->row();
    	if($result && !empty($result->points) && $result->points >= $points) {
    	    $points = !empty($points) ? $points : intval($result->points);
    		// get conversion info from the database
    		$conversion_value = gc_field_value('points_conversion', settings_item('user.points'));
    		$minimum_points = intval(gc_field_value('minimum_points', settings_item('user.points')));
    		// perform conversion only if the user has reached the limit
    		if($points < $minimum_points) {
    			return false;
    		}
    		$cash = decimal_convert($points/$conversion_value);
    		// update wallets/points and save history
    		$this->update_cash('points', $user_id, 'wallets', $points, 'debit'); // debit points
    		$this->update_cash('wallets', $user_id, 'points', $cash, 'credit'); // credit wallet
    		return $cash;
    	}
    	return false;
    }
    
    /**
     * Transfer Points to other user
     * @param number $user_id
     * @param number $points - Number of points to be transferred
     * @param number $receiver_id
     * @return boolean
     */
    public function transfer_points($user_id, $points, $receiver_id) {
        $user = $this->db->select('points')->get_where('users', array('id' => $user_id))->row();
        $receiver = $this->db->where('id', $receiver_id)->get('users')->row();
        if($user && $receiver && !empty($user->points) && $user->points >= $points) {
            // update points and save history
            $this->update_cash('points', $user_id, 'points', $points, 'debit'); // debit points
            $this->update_cash('points', $receiver_id, 'points', $points, 'credit'); // credit points
            return $points;
        }
        return false;
    }
    
    /**
     * Withdraw/Transfer Points
     * @param string $type
     * @param string $role
     */
    public function withdraw_transfer($type = 'withdraw', $role = 'admin') {
        if($type == 'transfer') {
            Assets::add_css('selectize.bootstrap3.css');
            Assets::add_js('selectize.min.js');
            Assets::add_js('selectize-search.js');
        }
        $user_id = $this->auth->user_id();
        $user = $this->db->where('id', $user_id)->get('users')->row();
        if(!$user) {
            Template::redirect(site_url('members'));
        }
        $this->lang->load('wallets/wallets');
        if(isset($_POST['save'])) {
            if($type == 'transfer') {
                $this->form_validation->set_rules('receiver_id', 'lang:label_user', 'trim|required');
            }
            $this->form_validation->set_rules('points', 'lang:label_points', 'trim|required|is_numeric');
            if ($this->form_validation->run() === TRUE) {
                $points = $this->input->post('points');
                if($type == 'transfer') {
                    $receiver_id = $this->input->post('receiver_id');
                    $amount = $this->transfer_points($user_id, $points, $receiver_id);
                    $message = sprintf(lang('msg_transfer_success'), $amount . ' ' . lang('label_points'));
                } else {
                    $amount = $this->points_conversion($user_id, $points);
                    $message = sprintf(lang('msg_success_conversion'), currency_format($amount));
                }
                if(!is_bool($amount)) {
                    Template::set_message($message, 'success');
                    Template::redirect(current_url());
                } else {
                    Template::set_message(lang('msg_error_conversion'), 'error');
                }
            }
        }
        $title = lang('label_' . $type);
        Template::set(array(
            'type' => $type,
            'points' => $user->points,
            'title' => $title,
            'page_title' => $title,
            'toolbar_title' => $title
        ));
        Template::set_view('wallets_points/withdraw_transfer');
        Template::render('member');
    }
    
    /**
     * Wallet to Points conversion
     * @param number $amount
     * @param number $user_id
     * @return number|boolean
     */
    public function wallet_to_points($amount = 10, $user_id = 0) {
        $user_id = ($user_id == 0 ? intval($this->auth->user_id()) : $user_id);
        $result = $this->db->select('wallets, points')->get_where('users', array('id' => $user_id))->row();
        if($result && floatval($result->wallets) > floatval($amount)) {
            // get conversion info from the database
            $conversion_value = gc_field_value('points_conversion', settings_item('user.points'));
            // calculate cash
            $new_points = intval($amount * $conversion_value);
            // update wallets/points and save history
            $this->update_cash('points', $user_id, 'wallets', $new_points, 'credit'); // credit points
            $this->update_cash('wallets', $user_id, 'points', $amount, 'debit'); // debit wallet
            return $new_points;
        }
        return false;
    }
    
    /**
     * Display Transaction Type to the user based on the type_id stored in the wallets_points table
     * @param int $id
     * @return string $module - wallets/points
     * 
     */
    public function transaction($id, $module = 'wallets') {
    	switch($id) {
    		case 100: // Amount updated by admin
    			$transaction = lang('type_admin'); 
    			break;
    		case 101:
    			$transaction = lang('type_user');
    			break;
    		case 102:
    			$transaction = lang('type_claims');
    			break;
    		case 103:
    			$transaction = lang('type_pyg');
    			break;
    		default:
    			$transaction = gc_module_translation(gc_module_name($id));
    	}
    	return $transaction;
    }
    
    /**
     * Get transaction type id based on the string
     * type_id will be stored in the wallets_points table
     * 
     * Most of the time type_id is equal to module_id
     * i.e. find module_id first, if the return value is zero then get self assigned type_id
     * 
     * @param string $string
     */
    private function type_id($string) {
    	switch($string) {
    		case 'admin':
    			$type_id = 100;
    			break;
    		case 'user':
    			$type_id = 101;
    			break;
    		case 'claims':
    			$type_id = 102;
    			break;
    		case 'pyg':
    			$type_id = 103;
    			break;
    		default:
    			$type_id = gc_module_id($string);
    	}
    	return $type_id;
    }
    
    /**
     * Save Wallets/Points History
     * @param int $module_id
     * @param int $user_id
     * @param float $amount
     * @param string $type
     * @return boolean
     */
    public function save_history($table, $user_id, $module_id, $amount, $cash, $type = 'debit', $notes = null, $row_id = 0, $extra = array()) {
        $table = $table == 'points' ? $this->table_points : $this->table_name;
        /*$result = $this->db->where(array('module_id' => $module_id, 'user_id' => $user_id))->count_all_results($table);
         if($result > 0) { // Do not assign points for the same module
         return false;
         }*/
        $data = array();
        $data['user_id'] = $user_id;
        $data['module_id'] = $module_id;
        $data['row_id'] = $row_id;
        $data['opening_balance'] = $cash;
        if($type == 'debit') {
            $data['debit'] = $amount;
            $data['credit'] = 0;
            $data['closing_balance'] = $cash - $amount;
        } else if($type == 'credit') {
            $data['debit'] = 0;
            $data['credit'] = $amount;
            $data['closing_balance'] = $cash + $amount;
        }
        if(!empty($notes)) {
            $data['notes'] = $notes;
        }
        $data['created_on'] = date('Y-m-d H:i:s');
        $data['created_ip'] = $this->input->ip_address();
        $data['additional_info'] = json_encode($extra);
        $this->db->insert($table, $data);
        $insert_id = $this->db->insert_id();
        // send email and sms on each transaction
        $this->load->library('goclixy/gemailer');
        $this->gemailer->sms_email_pdf('email', array('template_table' => 'wallets_email_templates', 'module_table' => str_replace('_history', '', $table), 'action_type' => 'receipt', 'user_type' => 'user', 'id' => $insert_id));
        return $insert_id;
    }
    
   	/**
     * Display wallets/points to the Admin
     * @param number $count
     * @param string $module - wallets/points
     * @param string $where
     * @param string $sort_by
     * @param number $offset
     * @param number $limit
     * @return number
     */
    public function get_wallets_points($count, $module, $where, $sort_by, $offset = 0, $limit = 25) {
    	if($count == 0) {
    		$sql = "SELECT id, display_name, email, phone_number, " . $module . " AS cash";
    	} else {
    		$sql = "SELECT COUNT(id) AS TotalCount";
    	}
    	$sql .= " FROM " . $this->db->dbprefix . "users";
    	// $sql .= " WHERE " . $module . " > 0";
    	if(!empty($where)) {
    		$sql .= " WHERE $where";
    	}
    	
    	if($count == 0) {
    		$sql .= " ORDER BY " . $sort_by;
    		$sql .= " LIMIT " . $offset . ", " . $limit;
    		return $this->db->query ( $sql )->result();
    	} else {
    		$result = $this->db->query ( $sql )->row();
    		return $result ? $result->TotalCount : 0;
    	}
    }
    
    /**
     * Display Wallets/Points history to the user
     * @param number $count
     * @param string $module
     * @param string $where
     * @param string $sort_by
     * @param number $offset
     * @param number $limit
     * @return number
     */
    public function get_history($count = 0, $module = 'wallets', $where = null, $sort_by = 'WP.id DESC', $offset = 0, $limit = 25) {
        $table = $module == 'wallets' ? $this->table_name : $this->table_points;
        if($count == 0) {
            $sql = "SELECT WP.*, U.display_name, U.email, U.phone_number AS user_phone_number";
        } else {
            $sql = "SELECT COUNT(WP.id) AS TotalCount";
        }
        $sql .= " FROM " . $this->db->dbprefix . $table . " WP";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "users U ON U.id = WP.user_id";
        if(!empty($where)) {
            $sql .= " WHERE $where";
        }
        // display franchise or user specific transactions only
        if($this->auth->role_id() != 1 && $this->auth->user_id()) { // web app only
            $sql .= (empty($where) ? " WHERE " : " AND ") . "user_id = " . $this->auth->user_id();
        }
        if($count == 0) {
            $sql .= " ORDER BY " . $sort_by;
            $sql .= " LIMIT " . $offset . ", " . $limit;
            return $this->db->query ( $sql )->result();
        } else {
            $result = $this->db->query ( $sql )->row();
            return $result ? $result->TotalCount : 0;
        }
    }
    
    /**
     * Set a Wallets/Points to zero and purge the history information
     * @param user_id
     * @return boolean
     */
    private function _purge($user_id, $module = NULL) {
        $data = array();
        $table = $module == 'wallets' ? $this->table_name : $this->table_points;
        
        // delete history first
        $this->db->where('user_id', $user_id)->delete($table);
        
        // update users table
        if(empty($module) || $module == NULL) {
            $data = array(
                'wallets' => 0,
                'points' => 0
            );
        } else {
            $data = array($module => 0);
        }
        
        // set auto increment value to the last id
        $alter_autoinc = "ALTER TABLE ". $this->db->dbprefix . $table . " AUTO_INCREMENT = 1";
        $this->db->simple_query($alter_autoinc);
        
        // set the user Wallet/Points to zero
        $this->db->where('id', $user_id)->update('users', $data);
        return true;
    }
    
    /***********************************************************/
    /* START OF CONTROLLERS METHOD
     /***********************************************************/
    
    /**
     * Display a list of Wallets data.
     *
     * @return void
     */
    public function index($module, $offset) {
        // Deleting anything?
        if (isset($_POST['delete'])) {
            $this->auth->restrict(ucfirst($module) . '.Financial.Delete');
            $checked = $this->input->post('checked');
            if (is_array($checked) && count($checked)) {
                $result = true;
                foreach ($checked as $pid) {
                    $result = $this->_purge($pid, $module);
                }
                if ($result) {
                    Template::set_message(count($checked) . ' ' . sprintf(lang('msg_delete_success'), lang('label_' . $module)), 'success');
                } else {
                    Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_' . $module)) . $this->wallets_points_model->error, 'danger');
                }
            }
        }
        $table = $module == 'wallets' ? $this->table_name : $this->table_points;
        
        // Search
        $where = null;
        $searchterm = $this->gc->search($table, $this->uri->segment(6), '/financial/' . $module . '/index');
        if(!empty($searchterm)) {
            $where .= " id LIKE '%". $searchterm ."%' OR $module LIKE '%". $searchterm ."%' OR email LIKE '%". $searchterm ."%' OR display_name LIKE '%". $searchterm ."%'";
        }
        
        // Order By
        $sort_by = "id desc";
        $order_by = $this->gc->sort($table);
        if(!empty($order_by)) {
            $sort_by = $order_by;
        }
        
        $pagerUriSegment = 5;
        $pagerBaseUrl = site_url(SITE_AREA . '/financial/' . $module . '/index') . '/';
        
        $limit  = $this->settings_lib->item('site.list_limit') ?: 15;
        $count = $this->wallets_points_model->get_wallets_points(1, $module, $where, $sort_by);
        
        $this->load->library('pagination');
        $pager['base_url']    = $pagerBaseUrl;
        $pager['total_rows']  = $count;
        $pager['per_page']    = $limit;
        $pager['uri_segment'] = $pagerUriSegment;
        
        $this->pagination->initialize($pager);
        
        $records = $this->wallets_points_model->get_wallets_points(0, $module, $where, $sort_by, $offset, $limit);
        
        Template::set('records', $records);
        Template::set('module', $module);
        Template::set('toolbar_title', lang($module . '_manage'));
        Template::set_view('wallets_points/index');
        Template::render();
    }
    
    /**
     * Create a Wallets/Points object.
     * @param string $module
     * @return void
     */
    public function create($module) {
        $this->auth->restrict(ucfirst($module) . '.Financial.Create');
        Assets::add_css('selectize.bootstrap3.css');
        Assets::add_js('selectize.min.js');
        Assets::add_js('selectize-search.js');
        if (isset($_POST['save'])) {
            if ($insert_id = $this->save('insert', $module)) {
                log_activity($this->auth->user_id(), lang('act_create_record') . ': ' . $insert_id . ' : ' . $this->input->ip_address(), $module);
                Template::set_message(sprintf(lang('msg_save_success'), lang('label_' . $module)), 'success');
                redirect(SITE_AREA . '/financial/' . $module);
            }
            
            // Not validation error
            if ( ! empty($this->wallets_points_model->error)) {
                Template::set_message(sprintf(lang('msg_create_failure'), lang('label_' . $module)) . $this->wallets_points_model->error, 'danger');
            }
        }
        // user post validation
        if(!empty($_POST['user_id'])) {
            $user_info = $this->db->select('id, email AS name')->where('id', $_POST['user_id'])->get('users')->row();
            if($user_info) {
                Template::set('user_info', $user_info);
            }
        }
        Template::set('module', $module);
        Template::set('toolbar_title', lang('add_' . $module));
        Template::set_view('wallets_points/form');
        Template::render();
    }
    
    /**
     * Allows editing of Wallets data.
     *
     * @return void
     */
    public function edit($id, $module) {
        Assets::add_css('selectize.bootstrap3.css');
        Assets::add_js('selectize.min.js');
        Assets::add_js('selectize-search.js');
        if (empty($id)) {
            Template::set_message(lang('invalid_id'), 'error');
            
            redirect(SITE_AREA . '/financial/' . $module);
        }
        
        if (isset($_POST['save'])) {
            $this->auth->restrict(ucfirst($module) . '.Financial.Edit');
            
            if ($this->save('update', $module, $id)) {
                log_activity($this->auth->user_id(), lang('act_edit_record') . ': ' . $id . ' : ' . $this->input->ip_address(), $module);
                Template::set_message(sprintf(lang('msg_save_success'), lang('label_' . $module)), 'success');
                redirect(SITE_AREA . '/financial/' . $module);
            }
            
            // Not validation error
            if ( ! empty($this->wallets_points_model->error)) {
                Template::set_message(sprintf(lang('msg_edit_failure'), lang('label_' . $module)) . $this->wallets_points_model->error, 'danger');
            }
        } elseif (isset($_POST['delete'])) {
            $this->auth->restrict(ucfirst($module) . '.Financial.Delete');
            if ($this->_purge($id, $module)) {
                log_activity($this->auth->user_id(), lang('act_delete_record') . ': ' . $id . ' : ' . $this->input->ip_address(), $module);
                Template::set_message(sprintf(lang('msg_delete_success'), lang('label_' . $module)), 'success');
                
                redirect(SITE_AREA . '/financial/' . $module);
            }
            Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_' . $module)) . $this->wallets_points_model->error, 'danger');
        }
        $record = $this->db->select('id, '. $module .' AS cash, email AS name')->where('id', $id)->get('users')->row();
        Template::set('record', $record);
        Template::set('module', $module);
        Template::set('toolbar_title', lang('edit_' . $module));
        Template::set_view('wallets_points/form');
        Template::render();
    }
    
    //--------------------------------------------------------------------------
    // !PRIVATE METHODS
    //--------------------------------------------------------------------------
    
    /**
     * Save the data.
     *
     * @param string $type Either 'insert' or 'update'.
     * @param int    $id   The ID of the record to update, ignored on inserts.
     *
     * @return boolean|integer An ID for successful inserts, true for successful
     * updates, else false.
     */
    private function save($type, $module, $id = 0) {
        if ($type == 'update') {
            $_POST['id'] = $id;
        }
        
        // Validate the data
        $this->form_validation->set_rules($this->wallets_points_model->get_validation_rules());
        $rule_cash = ($module == 'points' ? 'integer' : 'is_decimal');
        $this->form_validation->set_rules('cash', lang('label_cash'), 'trim|required|' . $rule_cash);
        if ($this->form_validation->run() === false) {
            return false;
        }
        
        // Make sure we only pass in the fields we want
        $user_id = $this->input->post('user_id');
        $amount = (float) $this->input->post('cash');
        $existing_balance = 0;
        $action_type = 'credit';
        $return = false;
        $balance_updated = true;
        
        // check whether the admin updated balance or not
        $existing = $this->db->select($module . ' AS cash')->get_where('users', array('id' => $user_id))->row();
        $data = array();
        if($existing) {
            $action = (int) $this->input->post('action');
            if($action == 0) { // Sum Cash
                $amount = $existing->cash + $amount;
            } else { // Subtract Cash
                $action_type = 'debit';
                $amount = $existing->cash - $amount; // do not allow in minus
                if($amount < 0) {
                    $amount = 0;
                }
            }
            $existing_balance = $existing->cash;
        }
        $return = $this->db->where('id', $user_id)->update('users', array($module => $amount));
        
        // in case of Admin, amount will always be credited
        if($return) {
            if($balance_updated) {
                $this->wallets_points_model->save_history($module, $user_id, $this->wallets_points_model->type_id('admin'), (float) $this->input->post('cash'), $existing_balance, $action_type, $this->input->post('notes'), $user_id);
            }
        }
        return $return;
    }
    
    /**
     * Admin - Display a list of Wallets/Points History.
     * @param string $module - Wallets/Points
     * @param int $offset
     * @return void
     */
    public function history($module, $offset) {
        // send receipt on email
        if(isset($_POST['email'])) {
            $this->email_print($module, $_POST['email'], 'email');
        }
        // generate pdf receipt
        if(isset($_POST['pdf'])) {
            $this->email_print($module, $_POST['pdf'], 'pdf');
        }
        $table = $module == 'wallets' ? $this->table_name : $this->table_points;
        // Deleting anything?
        if (isset($_POST['delete'])) {
            $this->auth->restrict(ucfirst($module) . '.Financial.Delete');
            $checked = $this->input->post('checked');
            if (is_array($checked) && count($checked)) {
                
                // If any of the deletions fail, set the result to false, so
                // failure message is set if any of the attempts fail, not just
                // the last attempt
                $result = false;
                foreach ($checked as $pid) {
                    $result = $this->db->delete($table, array('id' => $pid));
                }
                if ($result) {
                    Template::set_message(count($checked) . ' ' . lang('delete_success'), 'success');
                } else {
                    Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_history')) . $this->wallets_points_model->error, 'danger');
                }
            }
        }
        
        // Search
        $where = "";
        $searchterm = $this->gc->search($table . '_table', $this->uri->segment(5), '/financial/' . $module . '/history');
        if(!empty($searchterm)) {
            $where .= " WP.user_id LIKE '%". $searchterm ."%' OR WP.created_on LIKE '%". $searchterm ."%' OR U.email LIKE '%". $searchterm ."%' OR U.display_name LIKE '%". $searchterm ."%'";
        }
        
        // Order By
        $sort_by = "WP.id desc";
        $order_by = $this->gc->sort($table . '_table');
        if(!empty($order_by)) {
            $sort_by = $order_by;
        }
        
        $pagerUriSegment = 5;
        $pagerBaseUrl = site_url(SITE_AREA . '/financial/' . $module . '/history') . '/';
        
        $limit  = $this->settings_lib->item('site.list_limit') ?: 15;
        $count = $this->wallets_points_model->get_history(1, $module, $where);
        
        $this->load->library('pagination');
        $pager['base_url']    = $pagerBaseUrl;
        $pager['total_rows']  = $count;
        $pager['per_page']    = $limit;
        $pager['uri_segment'] = $pagerUriSegment;
        
        $this->pagination->initialize($pager);
        $records = $this->wallets_points_model->get_history(0, $module, $where, $sort_by, $offset, $limit);
        Template::set('records', $records);
        Template::set('module', $module);
        Template::set('toolbar_title', lang('label_history'));
        Template::set_view('wallets_points/history');
        Template::render();
    }
    
    /***********************************************************/
    /* END OF CONTROLLERS METHOD
    /***********************************************************/
    /***********************************************************/
    /* START OF MEMBER AREA METHOD
    /***********************************************************/
    public function member($module, $offset, $limit) {
        $this->auth->restrict();
        $this->lang->load('wallets/wallets');
        $user_id = $this->auth->user_id();
        if(isset($_POST['email'])) {
            $this->email_print($module, (int) $_POST['email'], 'email');
            Template::set_message(lang('wallet_email_success'), 'success');
        }
        if(isset($_POST['pdf'])) {
            $this->email_print($module, (int) $_POST['pdf'], 'pdf');
        }
        if (isset($_POST['add'])) {
            $settings = json_decode(settings_item('user.' . $module) ?? '');
            $this->load->library ( 'form_validation' );
            $this->form_validation->set_rules('amount', 'lang:label_amount', 'trim|required|is_decimal|min_val[' . $settings->amount->minimum . ']|max_val[' . $settings->amount->maximum . ']');
            if ($this->form_validation->run() === TRUE) {
                $result = $this->wallet_checkout($module);
                if(!empty($result['message'])) {
                    Template::set_message($result['message']);
                }
                // set amount in a session variable
                // $this->session->set_userdata('order_type', $module);
                // $this->session->set_userdata($module . '_amount', $this->input->post('amount'));
                // Template::redirect(gc_url_format('members/checkout'));
            }
        }
        // perform points conversion
        if ($module == 'points' && isset($_POST['points_conversion'])) {
            $amount = $this->points_conversion($user_id);
            if(!is_bool($amount)) {
                Template::set_message(sprintf(lang('msg_success_conversion'), currency_format($amount)), 'success');
                Template::redirect(gc_url_format('members/points'));
            } else {
                Template::set_message(lang('msg_error_conversion'), 'error');
            }
        }
        $pagerUriSegment = 3;
        $pagerBaseUrl = gc_url_format('members/' . $module);
        
        $where = "user_id = " . $user_id;
        $order_by = "WP.id DESC";
        $table = $module == 'wallets' ? $this->table_name : $this->table_points;
        // Sort By
        $sort_by = $this->gc->sort($table);
        if(empty($sort_by)) {
            $sort_by = $order_by;
        }
        $count = $this->get_history(1, $module, $where);
        $records = $this->get_history(0, $module, $where, $sort_by, $offset, $limit);
        
        $this->load->library('pagination');
        $this->config->load('pagination');
        $pager['base_url']    = $pagerBaseUrl;
        $pager['total_rows']  = $count;
        $pager['per_page']    = $limit;
        $pager['uri_segment'] = $pagerUriSegment;
        
        $this->pagination->initialize($pager);
        // display wallets/points to the user
        $result = $this->db->select($module . ' AS cash')->get_where('users', array('id' => $user_id))->row();
        if($result) {
            // Display point conversion button to the user
            if($module == 'points') {
                $minimum_points = intval(gc_field_value('minimum_points', settings_item('user.points')));
                if(intval($result->cash) >= $minimum_points) {
                    Template::set('point_conversion', true);
                }
            }
            Template::set('cash', $result->cash);
        }
        
        Template::set('records', $records);
        Template::set('module', $module);
        Template::set('toolbar_title', lang('my_' . $module));
        Template::set('page_title', lang('my_' . $module));
        Template::set('title', lang('my_' . $module) . ($result ? ' - ' . ($module == 'wallets' ? currency_format($result->cash) : $result->cash) : ''));
        Template::set_view('wallets_points/member');
        
        // Wallets and Points have no View in the Themes Module, i.e. load layout files manually
        $this->gc->demo_custom_css_js('member');
        Template::render('member');
    }
    
    /**
     * Wallet Checkout
     * @param string $module
     * @param boolean $mobile
     */
    public function wallet_checkout($module = 'wallets', $user_id = 0, $mobile = false) {
        $this->lang->load('orders/orders');
        $settings = json_decode(settings_item('user.' . $module) ?? '');
        $this->load->library ( 'form_validation' );
        $this->form_validation->set_rules('amount', 'lang:label_amount', 'trim|required|is_decimal|min_val[' . $settings->amount->minimum . ']|max_val[' . $settings->amount->maximum . ']');
        if($mobile) {
            $this->form_validation->set_rules('payment_method', 'lang:label_payment_method', 'trim|required');
        }
        if ($this->form_validation->run() === TRUE) {
            $amount = $this->input->post('amount');
            if($mobile) { // the cart item will be set by the 'checkout' case
                // save items in the cart
                $this->load->model('orders/orders_model');
                $this->load->model('users/user_model');
                $this->orders_model->cart(0, 0, 1, 0, 'wallets', true, array('price' => $amount), $user_id);
                
                // proceed to checkout
                $payment_method = (int) $this->input->post('payment_method');//49;
                $user = $this->user_model->users_checkout($user_id);
                $_POST = $user;
                $_POST['pay_using_wallet'] = false;
                $_POST['save'] = true;
                $_POST['chk_terms'] = true;
                $_POST['payment_method'] = $payment_method;
                $result = $this->orders_model->checkout($user_id, 'mobile');
                $result['record'] = array('amount' => $amount);
                return $result;                
                // return array('success' => true, 'amount' => $amount);
            } else {
                //set amount in a session variable
                $this->load->model('orders/orders_model');
                $this->orders_model->cart(0, 0, 1, 1, 'wallets', true, array('price' => $amount));
                Template::redirect(gc_url_format('members/checkout'));
            }
        } else {
            return array('success' => false, 'message' => validation_errors());
        }
    }
    
    /**
     * Email/PDF
     * @param number $payout_id
     * @param array $record
     * @return boolean
     */
    public function email_print($table, $transaction_id, $type = 'email', $record = array(), $role = 'admin') {
        $type = $type == 'email' ? 'sms_email' : $type;
        if(isset($_POST['invoice_pdf']) || isset($_POST['pdf'])) {
            $type = 'pdf';
        }
        $filename = '';
        if($type == 'pdf') {
            $filename = 'receipt_' . $transaction_id . '_' . time();
        }
        // send receipt on email
        if(isset($_POST['email'])) {
            $type = 'sms_email';
        }
        $content = $this->gemailer->sms_email_pdf($type, array('template_table' => 'wallets_email_templates', 'module_table' => $table, 'action_type' => 'receipt', 'user_type' => 'user', 'id' => $transaction_id, 'filename' => $filename));
        if($type == 'view') {    
            if($content) {
                Template::set(array(
                    'toolbar_title' => lang($table . '_receipt') . ' - ' . $transaction_id,
                    'row_id' => $transaction_id,
                    'receipt' => $content['message']
                ));
                Template::set_view('wallets/receipt');
                Template::render();
            } else {
                Template::set_message(lang('label_no_record_found'), 'danger');
                Template::redirect(gc_url_format('members/' . $table));
            }
        }
    }
    
    /**
     * Email Template
     * @param string $table
     * @param number $row_id
     * @param array $record
     * @return boolean
     */
    public function info($table, $row_id, $record = array()) {
        $this->lang->load('wallets/wallets');
        $receipt_module = $table;
        $table = $table == 'wallets' ? $this->table_name : $this->table_points;
        if(empty($record)) {
            $this->db->from($table . ' M');
            $this->db->join('users U', 'U.id = M.user_id', 'LEFT');
            $record = $this->db->select('M.*, U.display_name AS name, phone_number, email, address, postcode, city, state, country')->where('M.id', $row_id)->get()->row();
        }
        if(empty($record)) {
            return false;
        }
        $data = array(
            'transaction_module' => lang($receipt_module . '_transaction')
        );
        foreach($record as $key => $value) {
            switch($key) {
                case 'row_id':
                case 'user_id':
                    break;
                case 'module_id':
                    $data['label_' . $key] = lang('transaction_type');
                    $data['value_' . $key] = !empty($value) ? $this->transaction($value, $table) : lang('not_applicable');
                    break;
                case 'opening_balance':
                case 'closing_balance':
                    $data['label_' . $key] = lang($key);
                    $data['value_' . $key] = currency_format($value);
                    break;
                case 'created_on':
                case 'modified_on':
                    $data['label_' . $key] = lang($key);
                    $data['value_' . $key] = gc_date_format($value, true);
                    break;
                default:
                    $data['label_' . $key] = lang('label_' . $key);
                    $data['value_' . $key] = $value;
                    break;
            }
        }
        if(!empty($record->debit)) {
            $data['label_transaction'] = lang($receipt_module . '_from');
            $data['value_transaction'] = currency_format($record->debit);
        } else {
            $data['label_transaction'] = lang($receipt_module . '_to');
            $data['value_transaction'] = currency_format($record->credit);
        }
        // display additional information on the receipt e.g. order_id
        $data['value_order_id'] = '';
        if(!empty($record->additional_info)) {
            $additional_info = json_decode($record->additional_info, true);
            if(!empty($additional_info)) {
                foreach($additional_info as $akey => $avalue) {
                    $data['label_' . $akey] = lang('label_' . $akey);
                    $data['value_' . $akey] = $avalue;
                }
            }
        }
        return $data;
    }
    
    /**
     * Save Earned Points
     * @param number $user_id
     * @param string $type
     */
    public function _earn_points($module, $user_id, $row_id) {
        $user_id = ($user_id > 0 ? $user_id : intval($user_id));
        if($user_id > 0) {
            $settings = json_decode(settings_item('user.points') ?? '');
            $this->load->model('wallets/wallets_points_model');
            if($module == 'users') { // user registration
                $affiliate_points = (int) gc_field_value('affiliates', json_encode($settings->modules));
                if($affiliate_points > 0) {
                    // check for the affiliate link, assign affiliate points also
                    $this->load->library('crypt');
                    $ref_by = (int) $this->crypt->decode($this->input->cookie('ref_by'));
                    if($ref_by > 0) {
                        if($this->_check_existence($ref_by, 'affiliates', $user_id)) {
                            $this->load->model('users/user_model');
                            $record = $this->user_model->get_user($ref_by);
                            if($record && $ref_by != $this->auth->user_id()) {
                                // valid user, assign affiliate points
                                $this->update_cash('points', $ref_by, 'affiliates', $affiliate_points, 'credit', null, $user_id);
                            }
                        }
                    }
                }
            }
            $points = gc_field_value($module, json_encode($settings->modules));
            // check whether points already given or not
            // if already given, do not assign again
            if($points > 0) {
                if($this->_check_existence($user_id, $module, $row_id)) {
                    $this->update_cash('points', $user_id, $module, $points, 'credit', null, $row_id);
                }
            }
        }
    }
    
    /**
     * Check Points Allotment
     * @param string $module
     * @param number $user_id
     * @param number $row_id
     */
    private function _check_existence($user_id, $module, $row_id) {
        $module_id = gc_module_id($module);
        $counts = $this->db->where(array('user_id' => $user_id, 'module_id' => $module_id, 'row_id' => $row_id))->count_all_results($this->table_points);
        if($counts == 0) {
            return true;
        }
        return false;
    }
    
    /**
     * Wallet form
     * @param string $module
     * @return array|boolean[]|string[]
     */
    public function wallet_form($user_id, $step_id) {
        $next_step_id = 0;
        $field_results = $this->wallet_transfer_fields($user_id, $step_id);
        if(!$field_results) {
            return false;
        }
        // delivery not availablle in this area/city
        if(empty($field_results[$step_id])) {
            return $field_results;
        }
        $last_step_id = $field_results[$step_id]['extra']['last_group_id']; // valid last group id and has fields
        
        if(empty($field_results[$step_id]['fields'])) {
            return array('success' => false);
        }
        $step_fields = $field_results[$step_id]; // $this->_process_step_fields($step_id, $keys, $field_results, $last_step_id, $toolbar_title, $extra_params);
        $form_fields = $step_fields['fields'];
        
        // Current/Previous/Next Group IDs
        $next_step_id = $field_results[$step_id]['extra']['next_group_id'];
        $previous_step_id = $field_results[$step_id]['extra']['previous_group_id'];
        $fields = $field_results[$step_id]; // get group name also to be displayed in the title bar
        
        if(isset($_POST['save'])) {
            // format selectable values
            foreach($_POST as $key => $value) {
                if(is_array($value) && isset($value['id'])) {
                    $_POST[$key] = $value['id'];
                }
            }
            if($step_id == $last_step_id) {
                $form_fields = $this->wallet_transfer_fields($user_id, 1);
                // in the last step validate all fields
                foreach($field_results as $field_result) {
                    $form_fields = array_merge($field_result['fields'], $form_fields[1]['fields']);
                }
            }
            $_POST = $this->mobiles_model->format_post_data($_POST, $form_fields);
            foreach($form_fields as $form_field) {
                $field_rules = array();
                foreach($form_field['validators'] as $field_validator) {
                    if(!empty($field_validator)) {
                        $field_rules[] = $field_validator;
                    }
                }
                $this->form_validation->set_rules($form_field['name'], $form_field['label'], 'trim' . (!empty($field_rules) ? '|' . implode('|', $field_rules) : ''));
            }
            if ($this->form_validation->run() === TRUE) {
                if($step_id != $last_step_id) {
                    // to avoid extra api request
                    $field_results = $this->wallet_transfer_fields($user_id, $next_step_id);
                    if(!empty($field_results['message'])) {
                        $result = array('success' => false, 'message' => $field_results['message']);
                    } else {
                        $next_step_fields = $field_results[$next_step_id]; //$this->_process_step_fields($next_step_id, $keys, $field_results, $last_step_id, $toolbar_title);
                        $result = array('success' => false, 'next_group_fields' => array($next_step_id => $next_step_fields), 'next_group_id' => $next_step_id, 'previous_group_id' => $field_results[$next_step_id]['extra']['previous_group_id'], 'last_group_id' => $last_step_id);
                    }
                } else {
                    $amount = $_POST['amount'];
                    $result = $this->save_wallet_transfer($user_id, $_POST['phone_number'], $amount);
                    if($result['success']) {
                        $result['next_group_id'] = $next_step_id;
                        $result['previous_group_id'] = $previous_step_id;
                        $result['last_group_id'] = $last_step_id;
                    }
                    return $result;
                }
            } else {
                // validation errors display them to the user
                $validation_errors = validation_errors();
                $result = array('success' => false, 'message' => $validation_errors);
            }
        } else {
            $result = $fields;
            $result['success'] = true;
        }
        return $result;
    }
    
    /**
     * Save wallet transfer transaction
     * @param number $user_id - Debit User ID
     * @param string $phone_number - Credit User Phone NationalInstance
     * @param float $amount - Amount to be Credited/Debited 
     */
    public function save_wallet_transfer($user_id, $phone_number, $amount) {
        if($amount == 0) {
            return array('success' => false, 'message' => sprintf(lang('required'), lang('label_amount')));
        }
        $this->load->model('users/user_model');
        $user = $this->user_model->get_user($user_id);
        if(!$user || empty($user->wallets) || $user->wallets < $amount) {
            return array('success' => false, 'message' => lang('wallet_low_balance'));
        }        
        $credit_user = $this->db->where('phone_number', $phone_number)->get('users')->row();
        if(!$credit_user) {
            return array('success' => false, 'message' => lang('wallet_user_not_exists'));
        }        
        // debit/credit the user account
        // update_cash($table, $user_id, $module, $amount, $type = 'debit', $notes = null, $row_id = 0, $extra = array())
        $this->update_cash('wallets', $user_id, 'wallets', $amount, 'debit', null, 0, array('name' => $credit_user->display_name, 'phone_number' => $credit_user->phone_number)); // debit wallet
        $this->update_cash('wallets', $credit_user->id, 'wallets', $amount, 'credit', null, 0, array('name' => $user->display_name, 'phone_number' => $user->phone_number)); // credit wallet
        return array('success' => true);
    }
    
    /**
     * Save Wallet Withdraw Requests
     * @param number $user_id
     * @param float $amount
     * @param string $role
     */
    public function save_withdraws($user_id, $amount, $role = 'mobile') {
        if($amount == 0) {
            return array('success' => false, 'message' => sprintf(lang('required'), lang('label_amount')));
        }
        $this->load->model('users/user_model');
        $user = $this->user_model->get_user($user_id);
        if(!$user || empty($user->wallets) || $user->wallets < $amount) {
            return array('success' => false, 'message' => lang('wallet_low_balance'));
        }
        // check for the existing withdraw requests
        $row = $this->db->where(array('user_id' => $user_id, 'status_id != ' => 3))->order_by('id', 'DESC')->get($this->table_withdraws)->row();
        if($row) {
            return array('success' => false, 'message' => lang('wallet_withdraw_exists'));
        }
        // save request
        $data = array(
            'user_id' => $user_id,
            'amount' => $amount,
            'created_on' => date('Y-m-d H:i:s'),
            'created_by' => $role == 'mobile' ? $user_id : $this->auth->user_id(),
            'created_ip' => $this->input->ip_address()
        );
        $this->db->insert($this->table_withdraws, $data);
        return array('success' => true, 'message' => lang('wallet_withdraw_success'));
    }
    
    /**
     * Wallet to Wallet Transfer
     * @param number $user_id
     * @param number $step_id
     */
    public function wallet_transfer_fields($user_id, $step_id) {
        $this->lang->load('wallets/wallets');
        $this->load->model('users/user_model');
        $user = $this->user_model->get_user($user_id);
        if(!$user || empty($user->wallets)) {
            return array('success' => false, 'message' => lang('wallet_low_balance'));
        }
        $prefix = '';
        if($step_id == 1) { // the first step            
            // prepare fields for display
            $fields = array();
            $fields[] = array('name' => $prefix . 'phone_number', 'label' => lang('label_phone_number'), 'type' => 'text', 'rules' => array('required' => 'required'));
            $fields = $this->mobiles_model->rest_fields($fields);
            $extra = array(
                'id' => 1,
                'name' => lang('wallet_transfer_user'),
                'icon' => 'cash-outline',
                'help' => lang('wallet_phone_number'),
                'toolbar_title' => lang('wallet_transfer'),
                'next_group_id' => 2,
                'previous_group_id' => 0,
                'last_group_id' => 2
            );
            $steps[1] = array(
                'extra' => $extra,
                'fields' => $fields,
                'record' => array()
            );
            return $steps;
        } else {
            $phone_number = $_POST['phone_number'];
            $credit_user = $this->db->where('phone_number', $phone_number)->get('users')->row();
            if(!$credit_user) {
                return array('success' => false, 'message' => lang('wallet_user_not_exists'));
            }
            $user_info = $credit_user->display_name . ' (' . $credit_user->phone_number . ')';
            // second step
            $second_step_fields = array(
                array('name' => $prefix . 'user_info', 'label' => lang('bf_user'), 'type' => 'html', 'rules' => array()),
                array('name' => $prefix . 'amount', 'label' => lang('label_amount'), 'type' => 'text', 'rules' => array('required' => 'required', 'is_decimal' => 'is_decimal')),
            );
            $extra = array(
                'id' => 2,
                'name' => lang('wallet_transfer'),
                'mobile_icon' => 'card-outline',
                'toolbar_title' => lang('label_pay') . ' # '. $credit_user->display_name,
                'help' => lang('wallet_amount'),
                'next_group_id' => 2,
                'previous_group_id' => 1,
                'last_group_id' => 2
            );
            $steps[2] = array(
                'extra' => $extra,
                'fields' => $this->mobiles_model->rest_fields($second_step_fields),
                'record' => array('user_info' => $user_info)
            );
            return $steps;
        }
    }
    
    /**
     * Transaction Description to be displayed to the user
     * @param object $wallet
     */
    public function transaction_description($record) {
        $is_credit = !empty($record->credit) ? true : false;
        $additional_info = json_decode($record->additional_info ?? '', true);
        $description = '';
        switch($record->module_id) {
            case 21:
                $description = ($is_credit ? lang('label_from') : lang('label_to')) . '::' . $additional_info['name'] . ' (' .  $additional_info['phone_number'] . ')';
                break;
            case 17:
                $description = lang('label_ad') . '#' . $record->row_id . ' (' . $additional_info['views'] . ')';
                break;
        }
        return $description;
    }
    /***********************************************************/
    /* END OF MEMBER AREA METHOD
    /***********************************************************/
}