/*----------------------------------------------------*/
/*	Tiny MCE Editor
/*----------------------------------------------------*/
if($('.editor--').length) {
	tinymce.init({
		theme: "modern",
	    plugins: [
	        "advlist autolink lists link image charmap print preview hr anchor pagebreak",
	        "searchreplace wordcount visualblocks visualchars code fullscreen",
	        "insertdatetime media nonbreaking save table contextmenu directionality",
	        "emoticons template paste textcolor colorpicker textpattern"
	    ],
	    toolbar1: "insertfile undo redo | styleselect | bold italic | alignleft aligncenter alignright alignjustify | bullist numlist outdent indent | link image",
	    toolbar2: "print preview media | forecolor backcolor emoticons",
	    image_advtab: true,
	    entity_encoding : "raw",
		language: $('#site_url').attr('lang'),
		mode: "specific_textareas", // enable or disable TinyMCE on textareas with certain classes
		editor_selector: "editor" // enable TinyMCE only for textareas having editor class  
	});
}
if($('.editor').length && $('#sms_text').length == 0 && typeof tinymce == 'function') {
	tinymce.init({
		theme: "modern",
		plugins: [
	        "lists wordcount code"
	    ],
	    menubar: '',  // skip file
	    toolbar1: "undo redo | bold italic underline | alignleft aligncenter alignright alignjustify | bullist numlist | emoticons | code",
		mode: "specific_textareas", // enable or disable TinyMCE on textareas with certain classes
		editor_selector: "editor" // enable TinyMCE only for textareas having editor class  
	});
}

if($('.editor-').length) {
	tinymce.init({
		theme: "modern",
		plugins: [
	        "lists wordcount"
	    ],
	    toolbar1: "undo redo | bold italic underline | alignleft aligncenter alignright alignjustify | bullist numlist | code",
	    entity_encoding : "raw",
		language: $('#site_url').attr('lang'),
		mode: "specific_textareas", // enable or disable TinyMCE on textareas with certain classes
		editor_selector: "editor" // enable TinyMCE only for textareas having editor class  
	});
}

/*----------------------------------------------------*/
/*	TinyMCE with Responsive File Manager
/*----------------------------------------------------*/
if($('.editor-filemanager').length) {
	tinymce.init({
		theme: "modern",
	    plugins: [
	        "advlist autolink lists link image charmap print preview hr anchor pagebreak",
	        "searchreplace wordcount visualblocks visualchars code fullscreen",
	        "insertdatetime media nonbreaking save table contextmenu directionality",
	        "emoticons template paste textcolor colorpicker textpattern responsivefilemanager"
	    ],
	    toolbar1: "insertfile undo redo | styleselect | bold italic | alignleft aligncenter alignright alignjustify | bullist numlist outdent indent | link image",
	    toolbar2: "| responsivefilemanager | print preview media | forecolor backcolor emoticons",
	    image_advtab: true,
	    entity_encoding : "raw",
		language: $('#site_url').attr('lang'),
		mode: "specific_textareas", // enable or disable TinyMCE on textareas with certain classes
		editor_selector: "editor-filemanager", // enable TinyMCE only for textareas having editor class  
		external_filemanager_path: $('#site_url').attr('home_url') + "assets/filemanager/",
		filemanager_title: $('#site_url').attr('fmlang'),
		filemanager_access_key: $('#site_url').attr('akey'),
		external_plugins: { "filemanager" : "../filemanager/plugin.min.js"},
		convert_urls: false,
		relative_urls: false, 
	});
}

/*----------------------------------------------------*/
/*	Global Variable
/*----------------------------------------------------*/
var csrf_token = $.cookie('ci_csrf_token');
if (typeof csrf_token === "undefined") {
	csrf_token = $('.js-extra').attr('data-csrf');
}

/*----------------------------------------------------*/
/*	Toggle Translate Fields
/*----------------------------------------------------*/
function toggleTranslate(iso) {
	$('.translate-field').hide();
	$('.lang-' + iso).show();
}

/*----------------------------------------------------*/
/*	Display OpenStreet Map in Module FORMs
/*----------------------------------------------------*/
if ($('#mapLeafletForm').length) {
	// the map container size changed and updates the map if 
	// so call it after you've changed the map size dynamically
	/* $('#mapLeafletForm').on('shown.bs.collapse', function (e) {
	    map.invalidateSize(true);
	})*/
	
	// MAP TAB ISSUE
	// find the ID of the TAB having Leaflet Canvas
	var mapTabID = $("#mapLeafletForm").closest("div.tab-pane").prop("id");
	$('a[href="#' + mapTabID + '"]').on('shown.bs.tab', function(e) {
		map.invalidateSize(true);
	});
	
    // use below if you want to specify the path for leaflet's images
    //L.Icon.Default.imagePath = '@Url.Content("~/Content/img/leaflet")';

	//position we will use later
	var lat = $('#mapLeafletForm').attr('data-latitude');
	var lon = $('#mapLeafletForm').attr('data-longitude');
    var curLocation = [lat, lon];
    // use below if you have a model
    // var curLocation = [@Model.Location.Latitude, @Model.Location.Longitude];

    if (curLocation[0] == '' || curLocation[1] == '') {
    	// get current location of the user
    	var map = L.map('mapLeafletForm').locate({setView: true, maxZoom: 16});
		map.on("locationfound", e => {
			if (!marker) {
				marker = new L.marker(e.latlng).addTo(this.map);
			} else {
				marker.setLatLng(e.latlng);
			}
		}).on("locationerror", error => {
			if (marker) {
				map.removeLayer(marker);
				marker = undefined;
			}
		});
    } else {    
    	// display saved location
    	var map = L.map('mapLeafletForm').setView(curLocation, 10);
    }    

    L.tileLayer('http://{s}.tile.osm.org/{z}/{x}/{y}.png', {
        attribution: '&copy; <a href="http://osm.org/copyright">OpenStreetMap</a> contributors'
    }).addTo(map);

    map.attributionControl.setPrefix(false);

    var marker = new L.marker(curLocation, {
        draggable: 'true'
    });

    marker.on('dragend', function(event) {
        var position = marker.getLatLng();
        marker.setLatLng(position, {
            draggable: 'true'
        }).bindPopup(position).update();
        $("#latitude").val(position.lat);
        $("#longitude").val(position.lng).keyup();
    });

    $("#latitude, #longitude").change(function() {
        var position = [parseInt($("#latitude").val()), parseInt($("#longitude").val())];
        marker.setLatLng(position, {
            draggable: 'true'
        }).bindPopup(position).update();
        map.panTo(position);
    });

    map.addLayer(marker);
}

/*----------------------------------------------------*/
/*	Get Current Location Using Web Browser
/*----------------------------------------------------*/
$('.leaflet-current-location').click(function() {
	getCurrentLocation();
});
function getCurrentLocation() {
    if ("geolocation" in navigator) {
        navigator.geolocation.getCurrentPosition(function(position) {
            var latitude = position.coords.latitude;
            var longitude = position.coords.longitude;
            // var accuracy = position.coords.accuracy;
			$("#latitude").val(latitude);
			$("#longitude").val(longitude);

			// update position on the map
			var position = [parseInt($("#latitude").val()), parseInt($("#longitude").val())];
			marker.setLatLng(position, {
				draggable: 'true'
			}).bindPopup(position).update();
			map.panTo(position);
			map.setView(position, 13);

            // You can use latitude and longitude for further processing
        }, function(error) {
			var str = "An unknown error occurred.";
            switch(error.code) {
                case error.PERMISSION_DENIED:
					str = "User denied the request for Geolocation.";
                    break;
                case error.POSITION_UNAVAILABLE:
					str = "Location information is unavailable.";
                    break;
                case error.TIMEOUT:
					str = "The request to get user location timed out.";
                    break;
                case error.UNKNOWN_ERROR:
					str = "An unknown error occurred.";
                    break;
            }
			$.toast({text: str, position : 'top-center', hideAfter : 1500 });
        });
    } else {
		var str = "Geolocation is not available in this browser.";
        $.toast({text: str, position : 'top-center', hideAfter : 1500 });
    }
}

/*------------------------------------
 * Feedbacks/Callbacks Form Submit
 *-----------------------------------*/
$('#formFeedbacks').validator().on('submit', function(e) {
	    feedbackCallbacks(e, $(this), 'feedbacks');
});
$('#formCallbacks').validator().on('submit', function(e) {
	    feedbackCallbacks(e, $(this), 'callbacks');
});
function feedbackCallbacks(e, form, module) {
	if (!e.isDefaultPrevented()) { // Check whether validations are successful or not		
	    e.preventDefault();
		$("#ajax_loader_" + module).show();
	    $.ajax({
	    	type: form.attr('method'),
	      	url: form.attr('action'),
	      	data: form.serialize()
	    }).done(function(data) {
			$("#ajax_loader_" + module).hide();
	    	if(JSON.parse(data).result == 1) {
				form.trigger('reset');
				refreshCaptcha();
				msgModal(JSON.parse(data).message, JSON.parse(data).type, module); 
			} else {
				refreshCaptcha();
			    msgModal(JSON.parse(data).message, JSON.parse(data).type, module); 
			}
	    }).fail(function(data) {
			$("#ajax_loader_" + module).hide();
	      	// Optionally alert the user of an error here...
	    });
	    return false;
	} else {
		msgModal($('#error_lang').val(), 'danger', module);
	}
}

/**
 * Modal Preview
 * @param e
 * @returns
 */
$('.preview-modal').on('click',function(e) {
	e.preventDefault();
    var dataURL = $(this).attr('data-href');
	var loader = $('.loader-img');
	loader.show();	
    $('.gc-global-load').load(dataURL, function() {
		loader.hide();
    	if($('#previewModal').is(':visible') === false) {
        	$('#previewModal').modal({show:true});
        	// tooltip not working in a modal fix
        	$('.modal [data-toggle="tooltip"]').tooltip({trigger: 'hover'});
        }
    });
});

/*----------------------------------------------------*/
/*	Gallery
/*----------------------------------------------------*/
if ($(".gc-fancybox").length != 0) {
	//$(".gc-fancybox").prettyPhoto();
	// Disable social tools
	$(".gc-fancybox").fancybox({
		openEffect:	'elastic',
		closeEffect: 'elastic',
		openSpeed: 200, 
		closeSpeed: 200,
	});
}

/*----------------------------------------------------*/
/*	View Page Gallery
/*----------------------------------------------------*/
if ($(".gc-view-gallery").length != 0) {
	$('.gc-view-gallery').slick({
	  dots: false,
	  infinite: true,
	  speed: 500,
	  fade: true,
	  autoplay: false,
	  autoplaySpeed: 2000,
	  cssEase: 'linear'
	});
}

/*----------------------------------------------------*/
/*	Toggle List/Grid View
/*----------------------------------------------------*/
if ($(".grid-list-view").length != 0) {
	$( ".grid-list-view" ).each(function() {
    	$(this).on("click", function(event) {
			event.preventDefault();
			var parentDiv = $('.gc-view');
			parentDiv.toggleClass('gc-list-view');
			parentDiv.toggleClass('gc-grid-view');
			if($(this).data('type') == 'grid') {
				parentDiv.removeClass('gc-list-view');
				parentDiv.addClass('gc-grid-view');
			} else {
				parentDiv.removeClass('gc-grid-view');
				parentDiv.addClass('gc-list-view');
			}
			
			// set cookie to store the selected value
			$.cookie("gc_view_type", $(this).data('type'), { expires: 30, path: '/' });
			
			// dropdown type - display the selected value
			if($('.gc-view-type').length > 0) {
				$('.gc-view-type').html($(this).data('title'));
			}
		});
	});
}

/*----------------------------------------------------*/
/*	Display Phone Number on click of gc-number class
/*----------------------------------------------------*/
$(document).on('click','.gc-number',function() {
    $(this).find('span').show();
    $(this).find('.phone_show').hide();
    event.preventDefault();
});

/*----------------------------------------------------*/
/*	Description Read More and Less
/*----------------------------------------------------*/
if($('.readmore').length > 0) {
	var more = $('.readmore').data('more');
	var less = $('.readmore').data('less');
	more = typeof more === "undefined" ? 'More...' : more;
	less = typeof less === "undefined" ? 'Less' : less;
	$('.readmore').readmore({
	  speed: 75,
	  collapsedHeight: 90,
	  lessLink: '<a href="#" class="text-right text-decoration-none p-2">' + less + '</a>',
	  moreLink: '<a href="#" class="text-right text-decoration-none p-2">' + more + '</a>',
	  blockCSS: 'display: block; width: 100%;'
	});
}

/*----------------------------------------------------*/
/*	Readmore Plugin - This works on height
/*----------------------------------------------------*/
if($('.text-readmore').length > 0) {
	gcReadmore();
}
function gcReadmore() {
	$( '.text-readmore' ).each(function() {
		$(this).readmore({
			collapsedHeight:50,
			lessLink:'<div class="readmore-text"><a class="btn btn-sm btn-link" href="javascript:">'+gcLangString.readless+'</a></div>',
			moreLink:'<div class="readmore-text"><a class="btn btn-sm btn-link" href="javascript:">'+gcLangString.readmore+'</a></div>',
			afterToggle:function(trigger, element, expanded){
				if(!expanded){
					$('html,body').animate({
						scrollTop:element.offset().top
					},{
						duration:100
					}
				)}
			}
		});
	});
}

/*----------------------------------------------------*/
/*	Parallax scrolling effect for background images
/*----------------------------------------------------*/
if($('.jarallax').length) {
	$('.jarallax').jarallax({
		speed: 0.2
	});
}
/*--------------------------------------------*/
/* CART - PRODUCT ECOMMERCE
 *--------------------------------------------*/
// alert($.parseJSON($('.cart-string').val())['progress']);
if($('.cart-strings').length && $('.cart-strings').val() != '') {
	var cartStrings = $.parseJSON($('.cart-strings').val());
} else {
	var cartStrings = { progress: 'Adding...', btn: 'Add to Cart', success: 'Item added to your cart.', error: 'Unable to add item to your cart.'};
}
$(".form-item").submit(function(e){
	var form_data = $(this).serialize();
	var button_content = $(this).find('button[type=submit]');
	button_content.html(cartStrings.progress); // Loading button text 
	
	$.ajax({ //make ajax request to cart_process.php
		url: "cart.php",
		type: "POST",
		dataType:"json", //expect json value from server
		data: {'id': row_id, 'ci_csrf_token' : csrf_token},
	}).done(function(data){ //on Ajax success
		$(".cart-items-count").html(data.items); //total items in cart-info element
		button_content.html(cartStrings.btn); //reset button text to original text
		alert(cartStrings.success); //alert user
		if($(".shopping-cart-box").css("display") == "block"){ //if cart box is still visible
			$(".cart-box").trigger( "click" ); //trigger click to update the cart box.
		}
	})
	e.preventDefault();
});

/*-----------------------------------------------*/
/* Clear Cart on Button Click
 *-----------------------------------------------*/
$('body').on('click', '.clear-cart', function(e) {
	e.preventDefault();
	$("#ajax_loader").show();
	$.post( site_url + "ajax/clear_cart", {'ci_csrf_token': csrf_token}, function( data ) {
		$("#ajax_loader").hide();
		$(".cart-items-count").html(0);
		$(".cart-box").trigger( "click" ); //trigger click on cart-box to update the items list
		// if the request is from the checkout page, redirect the user to the home page
		if (window.location.href.indexOf(data.checkout) != -1 || window.location.href.indexOf(data.cart) != -1) {
			window.location.href = site_url;
		}
	});
});

// Add Item to Cart
$(".cart-item").click(function(e) {
	var button_content = $(this);
	var btn_text = cartStrings.btn;
	$(this).html(cartStrings.progress); // Loading button text
	var itemId = $(this).data('id');
	var module = $(this).data('module');
	$.ajax({ // make ajax request to cart_process.php
		url: site_url + "ajax/cart",
		type: "POST",
		dataType:"json", // expect json value from server
		data: {'action': 0, 'item': itemId, 'module': module, 'ci_csrf_token' : csrf_token},
	}).done(function(data){ // on Ajax success
		if(data.items_count == 0) {
			alert(cartStrings.error); //alert user
			button_content.html(btn_text); //reset button text to original text
		} else {
			$(".cart-items-count").html(data.items_count); //total items in cart-info element
			button_content.html(btn_text); //reset button text to original text
			alert(cartStrings.success); //alert user
			if($(".shopping-cart-box").css("display") == "block"){ //if cart box is still visible
				$(".cart-box").trigger( "click" ); //trigger click to update the cart box.
			}
		}
	})
	e.preventDefault();
});

// Buy Now
$(".buy-item").click(function(e) {
	var button_content = $(this);	
	var itemId = $(this).data('id');
	var module = $(this).data('module');
	$.ajax({ //make ajax request to cart_process.php
		url: site_url + "ajax/cart",
		type: "POST",
		dataType:"json", //expect json value from server
		data: {'action': 0, 'item': itemId, 'module': module, 'ci_csrf_token' : csrf_token},
	}).done(function(data){ //on Ajax success
		if(data.items_count > 0) {
			$(".cart-items-count").html(data.items); //total items in cart-info element
			if($(".shopping-cart-box").css("display") == "block"){ //if cart box is still visible
				$(".cart-box").trigger( "click" ); //trigger click to update the cart box.
			}
			// redirect user to the checkout page
			if (typeof data.checkout_url !== "undefined") {
				window.location.href = data.checkout_url;
			}
		}
	})
	e.preventDefault();
});

// Show Items in Cart
$( ".cart-box").click(function(e) { //when user clicks on cart box
	e.preventDefault(); 
	$(".shopping-cart-box").fadeIn(); //display cart box
	$(".shopping-cart-results").html('<img src="' + site_url + 'assets/images/ajax-loader.gif">'); //show loading image
	$(".shopping-cart-results" ).load( site_url + "ajax/cart", {'action': 2, 'ci_csrf_token' : csrf_token}); //Make ajax request using jQuery Load() & update results
});

// Close Cart
$( ".close-shopping-cart-box").click(function(e) { // user clicked on cart box close link
	e.preventDefault(); 
	$(".shopping-cart-box").fadeOut(); //close cart-box
});

// Remove items from cart
$(".shopping-cart-results").on('click', 'a.remove-item', function(e) {
	e.preventDefault(); 
	var pcode = $(this).attr("data-code"); //get product code
	$(this).parent().fadeOut(); //remove item element from box
	$.getJSON(site_url + "ajax/cart", {"code": pcode, 'ci_csrf_token' : csrf_token}, function(data) { //get Item count from Server
		$(".cart-items-count").html(data.items); //update Item count in cart-info
		$(".cart-box").trigger( "click" ); //trigger click on cart-box to update the items list
	});
});

// Checkout - Remove items from cart
$("body").on('click', '.shopping-cart a.item-remove', function(e) {
	e.preventDefault(); 
	var pcode = $(this).parents("tr").data('id');
	var module = $(this).parents("tr").data('module');
	$(this).parent().fadeOut(); //remove item element from box
	// $(".checkout-cart-layout").html('');
	// $(".checkout-cart-summary").html('');
	$.getJSON(site_url + "ajax/cart", {"item": pcode, 'module': module, 'ci_csrf_token' : csrf_token}, function(data) { //get Item count from Server
		if(data.items_count == 0) {
			// $(this).closest('form').trigger('submit');
			window.location.href = site_url;
		} else {
			$(".checkout-cart-layout").html(data.cart_layout);
			$(".checkout-cart-summary").html(data.cart_summary);
		}
		// $(".cart-items-count").html(data.items); //update Item count in cart-info
		// $(".cart-box").trigger( "click" ); //trigger click on cart-box to update the items list
	});
	// $(this).closest('form').trigger('submit');
	// $(this).parents("tr").remove();
});

// Products detail page - on attribute change, resubmit form 
$('.variants').on('change', function(e) {
    $(this).closest('form')
           .trigger('submit')
})

// Update Item Quantity
$("body").on('change', '.item-qty', function(e) {
// $('.item-qty').on('change', function(e) {
	e.preventDefault();
	var item = $(this).parents("tr").data('id');
	var module = $(this).parents("tr").data('module');
	var form = $(this).closest('form');
	var qty = $(this).val();
	$.post( site_url + "ajax/cart", {'ci_csrf_token': csrf_token, 'item': item, 'qty': qty, 'action': 'qty', 'update_cart': 1, 'module': module}, function( data ) {
		if(data.success == 0) {
			alert(data.message);
		} else {
			$(".checkout-cart-layout").html(data.cart_layout);
			$(".checkout-cart-summary").html(data.cart_summary);
		}
	});
});

// Add Item to Wishlist
$(document).on('click','.item-wishlist',function(e) {
	var row_id = $(this).data('id');
	var module = $(this).data('module');
	var inWishlist = $(this).data('wishlist') ? $(this).data('wishlist') : 0;
	var btnWishlist = $('#wishlist-' + module + '-' + row_id);
	var spanWishlist = $('#wishlist-text-' + module + '-' + row_id);
	var iconWishlist = $('#wishlist-icon-' + module + '-' + row_id);
	var addText = $(this).data('add');
	var removeText = $(this).data('remove');
	btnWishlist.prop('disabled', true);
	var wishListField = $(this);
	$.post( site_url + "ajax/wishlist", {'ci_csrf_token': csrf_token, 'row_id': row_id, 'module': module, 'wishlist': inWishlist}, function( data ) {
		var parsedData = JSON.parse(data);
		inWishlistValue = parsedData.success == 2 ? 0 : 1;		
		if(parsedData.success != 0) {
			wishListField.attr('data-wishlist', inWishlistValue);
			btnWishlist.data('wishlist', (inWishlist == 0 ? 1 : 0));
			btnWishlist.toggleClass('btn-wishlist');
			btnWishlist.find('i').toggleClass('far fa-heart').toggleClass('fas fa-heart');
			btnWishlist.prop('disabled', false);		
			if(spanWishlist.length > 0) {			
				btnWishlist.removeClass(inWishlist == 0 ? 'btn-outline-success' : 'btn-success');
				btnWishlist.addClass(inWishlist == 0 ? 'btn-success' : 'btn-outline-success');
				spanWishlist.html(inWishlist == 1 ? addText : removeText);
				iconWishlist.removeClass(inWishlist == 1 ? 'fa-check-square' : 'fa-heart');
				iconWishlist.addClass(inWishlist == 1 ? 'fa-heart' : 'fa-check-square');
			}
		}
		alertDialog(JSON.parse(data).message);
	});
});

// Item Out of Stock, Notify Customer
$('.item-notify').click(function(e) {
	e.preventDefault();
	var item = $(this).attr("data-code"); //get product code
	var notify_email = $('#notify_email').val();
	var btn = $(this);
	$("#ajax_loader").show();
	$.post( site_url + "ajax/notify_me", {'ci_csrf_token': csrf_token,'notify_email': notify_email, 'item': item}, function( data ) {
		$("#ajax_loader").hide();
		var type = (data.result == 1 ? 'success' : 'danger');
		msgModal(data.message, type, 'notify');
	});
});

// Return Item ID
function item_id(str) {
	return str.replace('item_', '');
}

/*----------------------------------------------------*/
/*	Update Cart on Button Click
/*----------------------------------------------------*/
$(".cart-update").on('click', '', function(e) {
	e.preventDefault();
	var data = [];
	$( ".cart-item-qty" ).each(function(index, item) {
		var itemId = $(item).attr('data-id');
		var qty = $(item).val();
		data.push({id: itemId, qty: qty});
	});
	$("#ajax_loader").show();
	$(".cd").fadeIn(500);
	var customCart = $('#custom_cart').length > 0 ? 1 : 0; // a hidden input variable on the custom checkout page
	$.post( site_url + "ajax/update_cart", {'ci_csrf_token': csrf_token, 'items' : JSON.stringify(data), coupon: $('#couponCode').val(), custom_cart: customCart }, function( data ) {
		customCartPage(data);
	});
});

/*----------------------------------------------------*/
/*	Custom Cart Page Specific
/*----------------------------------------------------*/
// Assign custom-cart class to the page
function customCartPage(data) {
	$("#ajax_loader").hide();
	$(".cd").fadeOut(500);
	if(data.result == 1) {			
		$.each(data.items, function(i, item) {
			$('.total-' + item.id).html(item.total);
		});
		$('.cart-total').html(data.total);
		$('.cart-subtotal').html(data.subtotal);
	}
	var type = (data.result == 1 ? 'success' : 'danger');
	msgModal(data.message, type, 'notify');
}
$("body").on('click', '.custom-cart a.item-remove', function(e) {
	e.preventDefault(); 
	var pcode = item_id($(this).parents("tr").data('id'));
	var module = item_id($(this).parents("tr").data('module'));
	$(this).parent().fadeOut(); //remove item element from box
	$.getJSON(site_url + "ajax/cart", {"item": pcode, 'module': module, custom_cart: customCart, 'ci_csrf_token' : csrf_token}, function(data) { //get Item count from Server
		if(data.items_count == 0) {
			window.location.href = site_url;
		} else {
			customCartPage(data);
		}
	});
});
/*--------------------------------------------*/
/* END OF CART
 *--------------------------------------------*/

/*----------------------------------------------------*/
/*	Common Form Submit with jQuery Validation
/*----------------------------------------------------*/
var site_url = $('#site_url').val();
var home_url = $('#site_url').data('home');
if(typeof home_url !== 'undefined') {
	site_url = home_url;
}
formSubmit('subscribers-form', 'ajax/subscribe');
formSubmit('contactForm', 'ajax/contact_query');
formSubmit('leadsForm', 'ajax/leads');
formSubmit('sendQueryForm', 'ajax/business_query');
formSubmit('signupForm', 'ajax/signup');
formSubmit('reviewForm', 'review');
formSubmit('erroneousReportForm', 'erroneous');
formSubmit('emailShareForm', 'sendEmail');
formSubmit('smsShareForm', 'sendSMS');
formSubmit('sendEmailForm', 'ajax/email');
formSubmit('sendSMSForm', 'ajax/sms');
// formSubmit('replyQueryForm', 'replyQuery'); // gc-admin.js already has this function for users & listings
function formSubmit(formId, url) {
	if(formId == 'subscribers-form') { // the form will be processed using classnames
		var idForm = '.' + formId;
	} else {
		var idForm = '#' + formId;
	}
	$(idForm).validate({
		submitHandler: function(form) {
			var formData = new FormData(form);
			switch(formId) {
				case 'subscribers-form': // send group id
					formData.append('group_id', $(idForm).data('group'));
					break;
				case 'sendQueryForm':
				case 'reviewForm':
				case 'erroneousReportForm':
				case 'emailShareForm':
				case 'smsShareForm':
				case 'sendSMSForm':
				case 'sendEmailForm':
					var module = $('#' + formId).attr('data-module');
					var rowId = $('#' + formId).attr('data-id');
					formData.append('module', module);
					if(formId == 'sendEmailForm' || formId == 'sendSMSForm') {
						if(formId == 'sendEmailForm') {
							// Fix - unable to post data on first click
							formData.append('email_message', tinyMCE && tinyMCE.activeEditor ? tinyMCE.activeEditor.getContent() : $('#email_message').val());
						}
						formData.append('row_ids', rowId);
					} else {
						formData.append('row_id', rowId);
					}
					break;
				default:
					formData.append('module', $("#module").val());
					formData.append('row_id', $("#row_id").val());
					if(formId == 'commentForm') {
						formData.append('parent_id', $('#comment_parent_id').val());
					}
					if(formId == 'leadsForm') {
						// Fix - unable to post data on first click
						formData.append('description', tinyMCE && tinyMCE.activeEditor ? tinyMCE.activeEditor.getContent() : $('#description').val());
					}
					if(formId == 'replyQueryForm') {
						// Fix - unable to post data on first click
						formData.append('reply_message', tinyMCE && tinyMCE.activeEditor ? tinyMCE.activeEditor.getContent() : $('#reply_message').val());
						formData.append('query_id', $('#query_id').val());
					}
			}
			var loader = $(idForm + ' .loader');
			var messageBlock = $(idForm + ' .message-block');
			loader.show();
			$('.gc-btn-ajax').prop('disabled', true); // disable the submit button click
			$.ajax({
	            url: site_url + url, 
	            type: "POST",             
	            data: formData,
				contentType: false,       		// Important: The content type used when sending data to the server.
				cache: false,             		// To unable request pages to be cached
				processData:false,        		// To send DOMDocument or non processed data file it is set to false     
	            success: function(data) {
	            	// hide the loader
	            	loader.hide();
					$('.gc-btn-ajax').prop('disabled', false);
	                if(JSON.parse(data).result == 1) {
	                	if(formId != 'widgetForm') { // modal jquery form
							form.reset(); // reset all fields
						}
						refreshCaptcha(); // reload captcha
	                	// display message to the user
		    			if(formId == 'commentForm') {
		    				$('#comment_parent_id').val(0);
		    			}
						// change the row status to replied
						if(formId == 'sendEmailForm') {
							tinyMCE.activeEditor.setContent(''); // Empty TinyMCE after email
							updateStatus();
						}
						if(formId = 'replyQueryForm') {
							var query_id = $("#query_id").val();
							// refresh all queries
							viewReplies(query_id);
							// update reply status
							updateReplyStatus(query_id);
							// show panel
							$('#collapse-' + query_id).collapse('show');
						}
		    			msgAlert(JSON.parse(data).message, JSON.parse(data).type, messageBlock); 
		           } else {
					   // reload captcha
					   refreshCaptcha(); // reload captcha
	                   // display message to the user
					   msgAlert(JSON.parse(data).message, JSON.parse(data).type, messageBlock); 
		           }
	            }
	        });
		},
		invalidHandler: function(event, validator) {
			var errors = validator.numberOfInvalids();
			if(errors) {
				$("div.error").hide();
			} else {
				$("div.error").hide();
			}
		}
	});
}

/*-----------------------------------------------*/
/* Display Success and Error Message to the User
 *-----------------------------------------------*/
function msgAlert(message, type, messageBlock) {
	messageBlock.show();
    $('.modal-dialog').addClass('shake');
    if(type == 'success') {
    	messageBlock.removeClass('alert alert-danger').html("");
    }
    messageBlock.addClass('alert alert-' + type).html(message);
    window.setTimeout( function(){ 
	   $('.modal-dialog').removeClass('shake'); 
	   if(type == 'success') {
		   messageBlock.removeClass('alert alert-success').html("");
	   }
    }, 10000 ); 
}

/*-----------------------------------------------*/
/* Refresh All Captcha's After Success
 *-----------------------------------------------*/
function refreshCaptcha() {
	$( '.refresh-captcha' ).each(function(index) {
		$('.refresh-captcha').attr('src', site_url + 'securimage?' + Math.random());
	});
}

/*----------------------------------------------------*/
/*	On Reply Success, Update Row Status to REPLIED
/*----------------------------------------------------*/
function updateStatus() {
	var rowId = $('#sendEmailForm').attr('data-id');
	var row = $('.badge-' + rowId);
	if(row.length > 0) {
		row.html($('#sendEmailForm').attr('data-replied'));
		row.removeClass("badge-danger");
		row.addClass("badge-success");
		$('#query_status_' + rowId).val(1);
	}
}

/*----------------------------------------------------*/
/*	Site Currency Change
/*----------------------------------------------------*/
if($(".dropdown-currencies").length > 0) {
	$(".dropdown-currencies a").click(function (e) {
		var currencyFieldId = $(this).parent().closest('div').attr('data-id');
		$('#site_currency' + currencyFieldId).val($(this).html());
		document.forms['currency_form' + currencyFieldId].submit();
	});
}
if($(".currencies-list").length > 0) {
	$('.currencies-list').each(function() {
		var currencyField = $(this).attr('id');
		var currencyFieldId = $(this).attr('data-id');
		$('#' + currencyField).selectize({
			onChange: function(value) {
				if(value != '') {
					var name = value.split('~');
					$('#site_currency' + currencyFieldId).val(name[0]);
					//currencyForm.closest('form').trigger('submit');
					document.forms['currency_form' + currencyFieldId].submit();
				}
		},	
		labelField: 'name',
		searchField: 'name',
		create: false
		});
	});
}

/*----------------------------------------------------*/
/*	Common Navbar Side Search For All Themes
/*----------------------------------------------------*/
function openNav() {
	document.getElementById("nav-sidebar__sidenav").style.width = window.screen.width <= 567 ? "100%" : "300px";
}
/* Set the width of the side navigation to 0 and the left margin of the page content to 0 */
function closeNav() {
	document.getElementById("nav-sidebar__sidenav").style.width = "0";
}

/*----------------------------------------------------*/
/*	Avoid several clicks in a fraction of second
/*----------------------------------------------------*/
$('.gc-form').on('submit', function() {
	$(this).find('.btn-submit').prop('hidden', 1);
	$(this).find('.btn-form-hidden').prop('hidden', 0);
});

/*----------------------------------------------------*/
/*	Form clear the uploaded image
/*----------------------------------------------------*/
$('.img-clear').on('click', function() {
	var imgId = $(this).attr('id');
	$('.img-' + imgId).empty();
	$('#img-' + imgId).val('');
	$(this).hide();
});

/*----------------------------------------------------*/
/*	Categories Module Search
/*----------------------------------------------------*/
if($('.selectize-categories-modules').length) {
	$( '.selectize-categories-modules').each(function() {
		var selectField = $('#' + $(this).attr('id'));
		categoriesModulesSearch(selectField);
	});
}
function categoriesModulesSearch(selectField) {
	selectField.selectize({
	  persist: true,
	  maxItems: 1,
	  valueField: "id",
	  labelField: "name",
	  searchField: ["name", "description"],
	  options: selectField.data('items'),
	  render: {
		item: function (item, escape) {
		  return "<div>" + (item.name ? '<span class="gc-label">' + escape(item.name) + "</span>" : "") + (item.description ? '<span class="ml-1 small">' + escape(item.description) + "</span>" : "") + "</div>";
		},
		option: function (item, escape) {
		  var label = item.name || item.description;
		  var caption = item.name ? item.description : null;
		  return "<div>" + '<span class="gc-label">' + escape(label) + "</span>" + (caption ? '<span class="caption d-block small">' + escape(caption) + "</span>" : "") + "</div>";
		},
	  },
	  onInitialize: function() { // preload data on validation error/edit coupon page
		var dataValue = this.$input.data('selectize-value');
		if(typeof dataValue != 'undefined' && dataValue != '') {
			var existingOptions = dataValue;
			var self = this;
			if(Object.prototype.toString.call( existingOptions ) === "[object Array]") {
				existingOptions.forEach( function (existingOption) {
					self.addOption(existingOption);
					self.addItem(existingOption[self.settings.valueField]);
				});
			}
			else if (typeof existingOptions === 'object') {
				self.addOption(existingOptions);
				self.addItem(existingOptions[self.settings.valueField]);
			}
		}
	  }
	});
}

/*----------------------------------------------------*/
/*	Change country using Ajax
/*----------------------------------------------------*/
$('.gc-change-country').on('click',function(e) {
	e.preventDefault();
	$('.page-loader').show();
    var dataURL = $(this).attr('data-href');
    $('.gc-global-load').load(dataURL, function() {
    	if($('#modalCountryChange').is(':visible') === false) {
        	$('#modalCountryChange').modal({show:true});
        	// tooltip not working in a modal fix
        	$('.modal [data-toggle="tooltip"]').tooltip({trigger: 'hover'});
			$('.page-loader').hide();
        }
    });
});

/*----------------------------------------------------*/
/*	Common function to copy text
/*----------------------------------------------------*/
/*var clipboard = new ClipboardJS('.clipboard');
clipboard.on('success', function(e) {
	var str = 'Copied';
	$.toast({text: str, position : 'top-center', hideAfter : 1500 });
	e.clearSelection();
	setTimeout(function() {
	// $(e.trigger).text("Copy");
	}, 2500);
});*/

/*----------------------------------------------------*/
/*	Lazyload All images with lazyload class
/*----------------------------------------------------*/
var instanceLazy = $("img.lazyload").Lazy({
	chainable: false,
    effect: "fadeIn",
    effectTime: 1000,
    threshold: 0
});

/************************************************
 * Common function to copy text only
 ************************************************/ 
if($('.copy-text-only').length > 0) {
    var clipboard = new ClipboardJS('.copy-text-only',{
      text: function(trigger) {
          var result = $(trigger.getAttribute('data-clipboard-target')).text();
          return result;
      }
    });
    clipboard.on('success', function(e) {
        var str = 'Copied';
		$.toast({text: str, position : 'top-center', hideAfter : 1500 });
        // e.clearSelection();
        return e.text;
    });
    clipboard.on('error', function(e) {
    });
}


/*-----------------------------------------------*/
/* Business Owner Reply to Visitor
 *-----------------------------------------------*/
// Update Query Reply Status
function updateReplyStatus(query_id) {
	var row = $('.badge-' + query_id);
	if(row.length > 0) {
		$('.badge-' + query_id + ' span').html($('#lang_replied').val());
		row.removeClass("badge-danger");
		row.addClass("badge-success");
		$('.badge-' + query_id + ' > i').removeClass('fa-clock');
		$('.badge-' + query_id + ' > i').addClass('fa-check');
	}
}

// Allow Business Owner to view all replies on their Dashboard
$('.view-replies').on("click", function(){
	var id = $(this).attr('id');
	if($('#collapse-' + id).length == 0) {
		viewReplies(id);
	}
});
function viewReplies(id) {	
	$('#collapse-' + id).remove(); // remove collapse or it will append data
	$('.parent-heading-' + id).after('<div id="collapse-'+ id +'" class="panel-collapse collapse" data-parent="#replyAccordion' + id + '"></div>'); 
	$.ajax({
		type: "POST",
		dataType:'json',
		url: site_url + "queryReplies",
		data: { id: id, 'ci_csrf_token': csrf_token },
		success: function(data) {
			var items = [];
			items.push('<div class="card-body"><div class="row"><div class="col-sm-12"><div class="timeline-centered">');
			$.each(data, function (id, row) {
				items.push('<article class="timeline-entry">'
						+ '<div class="timeline-entry-inner">'
						+ '<div class="timeline-icon bg-success"><i class="fas fa-feather"></i></div>'
						+ '<div class="timeline-label">'
						+ '<h2><a href="#">' + row.user + '</a> <span>' + row.created_on + '</span></h2>'
						+ (row.subject ? row.subject + '<br />' : '') + row.message
						+ '</div>'
						+ '</div>'
						+ '</article>'
				);
			});
			items.push('</div></div></div></div>');
			$("#collapse-" + id + "").html(items.join(''));
		}
	});
}
function CollapseIconChange(element) {
    $(element).children("i").toggleClass("fa-chevron-up").toggleClass("fa-chevron-down");
}