<?php

Assets::add_css(array(
    'bootstrap.min.css',
    'font-awesome.min.css',
    'famfamfam-flags.css',
    'general.css',
    'jquery.fancybox.min.css' // RESPONSIVE FILE MANAGER    
));

// RTL Admin Panel
if($this->session->userdata('is_rtl')) {
    Assets::add_css(array(
        'bootstrap-rtl.min.css',
        'bootstrap-flipped.min.css'
    ));
}

Assets::add_js(array('jquery.fancybox.min.js', 'validator.min.js', 'jquery.validate.min.js', 'jquery.lazy.min.js', 'jquery.lazy.plugins.min.js', 'global.js', 'general.js')); // RESPONSIVE FILE MANAGER

if (isset($shortcut_data) && is_array($shortcut_data['shortcut_keys'])) {
    Assets::add_js($this->load->view('ui/shortcut_keys', $shortcut_data, true), 'inline');
}

?>
<!doctype html>
<html lang="en">
<head>
	<meta charset="utf-8">
	<title><?php
        echo isset($toolbar_title) ? "{$toolbar_title} : " : '';
        e($this->settings_lib->item('site.title'));
    ?></title>
	<meta name="viewport" content="width=device-width, initial-scale=1.0">
	<meta name="robots" content="noindex" />
    <?php
    /* Modernizr is loaded before CSS so CSS can utilize its features */
    // https://cdnjs.cloudflare.com/ajax/libs/modernizr/2.8.3/modernizr.min.js
    // echo Template::theme_url('js/modernizr-2.5.3.js');
    ?>
	<script src="<?php echo Template::theme_url('js/modernizr.min.js'); ?>"></script>	
    <?php if(settings_item('lst.map_leaflet') == 1): // Google Maps is no longer free, display map using leaflet?>
    <link rel="stylesheet" href="https://unpkg.com/leaflet@1.3.1/dist/leaflet.css" integrity="sha512-Rksm5RenBEKSKFjgI3a41vrjkw4EVPlJ3+OiI65vTjIdo9brlAacEuKOiQ5OFh7cOI1bkDwLqdLw3Zg0cRJAAQ==" crossorigin=""/>
    <script src="https://unpkg.com/leaflet@1.3.1/dist/leaflet.js" integrity="sha512-/Nsx9X4HebavoBvEBuyp3I7od5tA0UzAxs+j83KgC8PU0kgB4XiK4Lfe4y4cgBtaRJQEIFCW+oC506aPT2L1zw==" crossorigin=""></script>
    <?php endif;?>
	<link href="https://fonts.googleapis.com/css?family=Roboto&display=swap" rel="stylesheet">
	<?php echo Assets::css(null, true); ?>
</head>
<body class="desktop">
    <!--[if lt IE 7]>
    <p class=chromeframe>Your browser is <em>ancient!</em> <a href="http://browsehappy.com/">Upgrade to a different browser</a> or
    <a href="http://www.google.com/chromeframe/?redirect=true">install Google Chrome Frame</a> to experience this site.</p>
    <![endif]-->
	<noscript>
		<p>Javascript is required to use Bonfire's admin.</p>
	</noscript>
    <nav class="navbar py-0 navbar-expand-lg navbar-dark bg-dark" id="topbar">
		<div class="container">
      <?php echo anchor('/', html_escape($this->settings_lib->item('site.title')), 'class="navbar-brand"'); ?>
      <button class="navbar-toggler" type="button" data-toggle="collapse" data-target="#navbarSupportedContent" aria-controls="navbarSupportedContent" aria-expanded="false" aria-label="Toggle navigation">
        <span class="navbar-toggler-icon"></span>
      </button>
    
      <div class="collapse navbar-collapse" id="navbarSupportedContent">
        <?php echo Contexts::render_menu('text', 'normal'); ?>
        <!-- User Menu -->
        <div class="navbar-nav ml-auto" id="user-menu">
            <div class="btn-group">
                <a href="<?php echo site_url(SITE_AREA . '/settings/users/edit'); ?>" id="tb_email" class="btn dark" title="<?php echo lang('bf_user_settings'); ?>">
                    <?php
                    $userDisplayName = isset($current_user->display_name) && ! empty($current_user->display_name) ? $current_user->display_name : ($this->settings_lib->item('auth.use_usernames') ? $current_user->username : $current_user->email);
                    echo $userDisplayName;
                    ?>
                </a>
                <?php
                /* Change **light** to **dark** to match colors */
                ?>
                <a class="nav-link btn dropdown-toggle light" data-toggle="dropdown" href="#"><span class="caret"></span></a>
                <ul class="dropdown-menu dropdown-menu-right toolbar-profile">
                    <li>
                        <div class="inner">
                            <div class="toolbar-profile-img">
                                <?php echo gravatar_link($current_user->email, 96, null, $userDisplayName); ?>
                            </div>

                            <div class="toolbar-profile-info">
                                <p><strong><?php echo $userDisplayName; ?></strong><br />
                                    <?php e($current_user->email); ?>
                                    <br/>
                                </p>
                                <a href="<?php echo site_url(SITE_AREA . '/settings/users/edit'); ?>"><?php echo lang('bf_user_settings'); ?></a>
                                <a href="<?php echo gc_url_format('logout'); ?>"><?php echo lang('bf_action_logout'); ?></a>
                            </div>
                        </div>
                    </li>
                </ul>
            </div>
        </div>
        <?php if (isset($shortcut_data) && is_array($shortcut_data['shortcuts']) && is_array($shortcut_data['shortcut_keys']) && count($shortcut_data['shortcut_keys'])) :
        ?>        
        <!-- Shortcut Menu -->
        <div class="navbar-nav" id="shortcuts">
            <div class="btn-group">
                <a class="nav-link dropdown-toggle light btn" data-toggle="dropdown" href="#"><img src="<?php echo Template::theme_url('images/keyboard-icon.png'); ?>" id="shortkeys_show" title="Keyboard Shortcuts" alt="Keyboard Shortcuts" /></a>
                <ul class="dropdown-menu dropdown-menu-right toolbar-keys">
                    <li>
                        <div class="inner keys">
                            <h4><?php echo lang('bf_keyboard_shortcuts'); ?></h4>
                            <ul>
                                <?php foreach ($shortcut_data['shortcut_keys'] as $key => $data) : ?>
                                <li><span><?php e($data); ?></span> : <?php echo $shortcut_data['shortcuts'][$key]['description']; ?></li>
                                <?php endforeach; ?>
                            </ul>
    	    				<?php if ( has_permission('Bonfire.UI.View') && has_permission('Bonfire.UI.Manage') ): ?>
                            <a href="<?php echo site_url(SITE_AREA . '/settings/ui'); ?>"><?php echo lang('bf_keyboard_shortcuts_edit'); ?></a>
    	    				<?php endif; ?>
                        </div>
                    </li>
    			</ul>
    		</div>
        </div>
    	<?php endif;?>
      </div>
      </div>
    </nav>    
    <nav class="navbar py-0 subnav" >
       <div class="container">
           <?php if (isset($toolbar_title)) : ?>
           <h1 class="navbar-brand"><?php echo $toolbar_title; ?></h1>
           <?php endif; ?>
           <div class="ml-auto" id="sub-menu">
               <?php Template::block('sub_nav', ''); ?>
           </div>
       </div>
    </nav>