<?php
/**
 * Change the data-value, data-duration and data-restart values to set and restart the clock automatically
 * data-value holds the remaining time
 * data-duration holds the timer restart value
 * data-restart if 1 restart the timer automatically
 * @param number $ids - ID of the slider
 * Usage: e.g. echo view('widgets/headers/_timer_01', array('ids' => 26)); 
 * 
 * @param number $duration - Duration in seconds
 * @param number $restart - 0/1
 * 
 * Demo values: {"2":{"key":"left_image","value":"sliders\/offer-01.png"},"3":{"key":"right_image","value":"sliders\/offer-02.png"},"4":{"key":"subtitle","value":"Lifetime Free Updates"},"5":{"key":"discount","value":"90% OFF"},"6":{"key":"offer_text","value":"No monthly fee"},"7":{"key":"duration","value":"2800"},"8":{"key":"restart","value":"1"}}
 */
$sliders = $this->layouts->widget_slider(array(
    'ids' => isset($ids) ? $ids : 0
));
if(!empty($sliders[0])):
$slider_id = !empty($slider_id) ? $slider_id : 1; // two instance of the same widget
$slider = $sliders[0];
$params = $slider->params;

// Timer restart logic
$duration = !empty($params['duration']) ? $params['duration'] : 1000;
$restart = !empty($params['restart']) ? (int) $params['restart'] : 0;
$current_date = strtotime(date('Y-m-d H:i:s'));
$start_date = strtotime(!empty($params['start_date']) ? $params['start_date'] : $current_date);
$seconds_remaining = $start_date - $current_date;
if($restart == 1 && $seconds_remaining < 1):
    // update the timer in the slider params for new date value
    $start_date = date("Y-m-d H:i:s", strtotime("+$duration sec")); // get new date
    $params['start_date'] = $start_date;
    // convert data to params format
    $new_params = array();
    foreach($params as $pkey => $pvalue) {
        $new_params[] = array('key' => $pkey, 'value' => $pvalue);
    }
    clear_cache('banners'); // clear the cache
    // update database
    $this->db->where('id', $ids)->update('banner_sliders', array('params' => json_encode($new_params)));
    // calculate new duration
    $seconds_remaining = strtotime($start_date) - $current_date;
endif;
?>
<div class="ecomtimer01" id="ecomtimer01<?php echo $slider_id; ?>" data-value="<?php echo $seconds_remaining; ?>" data-restart="<?php echo (int) $restart; ?>" data-duration="<?php echo (int) $duration; ?>">
    <div class="container">
        <div class="d-flex ecomtimer01_column flex-column justify-content-center text-center position-relative align-items-center">
        	<?php if(!empty($params['left_image'])):
        	$left_image = valid_url($params['left_image']) ? $params['left_image'] : base_url('assets/images/' . $params['left_image']);
        	?>
            <div class="ecomtimer01_img-left d-none d-lg-block">
                <img class="w-100" src="<?php echo $left_image; ?>">
            </div>
            <?php endif; ?>
            <?php if(!empty($params['right_image'])):
            $right_image = valid_url($params['right_image']) ? $params['right_image'] : base_url('assets/images/' . $params['right_image']);
            ?>
            <div class="ecomtimer01_img-right d-none d-lg-block">
                <img class="w-100" src="<?php echo $right_image; ?>">
            </div>
            <?php endif; ?>
            <div class="ecomtimer01_border">
                <div class="ecomtimer01_bubbles"></div>
                <div class="ecomtimer01_block">
                    <div class="ecomtimer01_description">
                        <h1 class="text-uppercase"><?php echo $slider->title; ?></h1>
                        <?php if(!empty($params['discount'])):?>
                        <div class="mb-2 mx-auto ecomtimer01_description_subtitle"><?php echo lang('label_upto'); ?></div>
                        <div class="d-flex justify-content-center ecomtimer01_description_price"><?php echo $params['discount']; ?></div>
                        <?php endif; ?>
                        <?php if(!empty($params['subtitle'])):?>
                        <h4 class="mb-10 mb-20-sm mx-auto"><?php echo $params['subtitle']; ?></h4>
                        <?php endif; ?>
                        <?php if(!empty($params['offer_text'])):?>
                        <p class="ecomtimer01_description_offer"><?php echo $params['offer_text']; ?></p>
                        <?php endif; ?>
                        <p class="ecomtimer01_description_offer mt-4 mb-3"><?php echo lang('label_deal_ends_in'); ?></p>
                        <div class="ecomtimer01_description_timer mt-4 mb-5">
                            <div class="clock"></div>
                            <div class="message"></div>
                        </div>
                        <?php echo theme_view('widgets/headers/_links', array('slider' => $slider, 'link_css' => 'ecomtimer01_description_btn btn d-flex justify-content-center align-items-center m-auto'));?>
                        <p class="ecomtimer01_description_offer mt-5"><?php echo $slider->html_text; ?></p>
                    </div>
                </div>
            </div>
        </div>
    </div>
</div>
<?php endif; ?>