<?php
/**
 * Common widget to call most of the modules
 * Important: do not forget to add this widget in the themes module through the Admin Panel.
 * 
 * Widget Parameters:
 * 
 * cols - Number of Columns to be Displayed
 * parent_bg - Parent Background Image
 * parent_class - Parent Class
 * content_parent_class - Content Parent Class
 * content_class - Content Class
 * content_widget_class - Content Widget Class
 * heading_block_class - Heading Block Class
 * heading_class - Heading Class
 * heading_sub_class - Sub Heading Class
 * heading_text - Heading Text
 * heading_sub_text - Heading Sub Text
 * content_bg - Content Background
 * module - Module
 * view_all - View All Link
 * slider - 2 // No Slider
 * heading - 2 // No Heading
 * arrows - 2 // No arrows
 * 
 * $params = array(
 * 'full_length' => $full_length, // Testimonial
 * 'parent_id' => $parent_id,
 * 'type_id' => $type_id,
 * 'product_id' => isset($product_id) ? $product_id : 0,
 * 'block' => 'recently_added',
 * 'brand_id' => isset($brand_id) ? $brand_id : 1,
 * 'group_id' => isset($group_id) ? $group_id : 4, 
 * 'limit' => isset($limit) ? $limit : 4,
 * 'packages' => $packages, 
 * 'records' => $records 
 * )
 * 
 * 
 * Usage: <?php echo view('widgets/modules/_call', array('load' => 'banner_sliders', 'widget' => '_slider_widget_26', 'wparams' => array('module' => '', 'heading' => 1, 'parent_class' => '', 'content_parent_class' => 'container-fluid', 'content_class' => 'container', 'heading_block_class' => 'heading-block', 'heading_class' => 'heading-theme-primary', 'heading_sub_class' => 'heading-theme-secondary', 'cols' => 3), 'params' => array('block' => '', 'records' => null, 'limit' => 12))); ?>
 * @var string $load
 * @var string $widget
 * @var array $wparams
 * @var array $params
 */
if(!empty($widget) && !empty($load)):

    // widget specific parameters
    $wparams = !empty($wparams) ? $wparams : array();
    $parent_bg = !empty($wparams['parent_bg']) ? $wparams['parent_bg'] : '';
    $cols = !empty($wparams['cols']) ? (int) $wparams['cols'] : 2;
    $view_all = !empty($wparams['view_all']) ? $wparams['view_all'] : false;
    $module = !empty($wparams['module']) ? $wparams['module'] : $load;
    
    // records/data specific parameters
    $params = !empty($params) ? $params : array();
    $block = !empty($params['block']) ? $params['block'] : '';
    $limit = !empty($params['limit']) ? (int) $params['limit'] : 20;

    $rkey = 'records';
    $heading_text_default = '';
    $heading_sub_text_default = '';
    $additional_params = array();
    $content_widget_default = '';
    $content_class_default = 'container';
    
	switch($load) {
		case 'banner_sliders':
		    $module = 'banner_sliders';
		    $block = 'popular';
		    // $params = array('ids' => $ids, 'group_id' => $group_id, 'limit' => $limit);
		    $records = $this->layouts->widget_slider($params);			
		    if($records && !empty($parent_bg) && $parent_bg == 'image_url'):
		        // the background image is from the slider
		        $params = $limit == 1 ? $records->params : $records[0]->params;
    		    if(!empty($wparams['background_image'])):
    		         $parent_bg = $wparams['background_image'];
    		    endif;
		    endif;
			break;
		case 'categories':
		    $module = !empty($module) ? $module : 'listings';
		    $parent_id = !empty($params['parent_id']) ? $params['parent_id'] : 0;
		    $records = $this->layouts->widget_categories($module, null, $parent_id, $limit, $params);
		    $rkey = 'categories';
		    $heading_text_default = sprintf(lang('heading_categories'), lang('gc_menu_' . $module));
		    break;
		case 'options':
		    $this->load->model('options/options_model');
		    $type_id = !empty($params['type_id']) ? $params['type_id'] : 0;
		    $records = $this->options_model->options_tree(true, 'type_id = ' . $type_id);
		    $heading_text_default = sprintf(lang('heading_options'), lang('listings'), lang('option_' . $type_id));
		    break;
		case 'packages':
		    if(GC_PRODUCT_ID == 1) {
    		    $packages = !empty($params['packages']) ? $params['packages'] : 0;
    		    $type_id = !empty($params['type_id']) ? $params['type_id'] : 0;
    		    $records = $this->layouts->widget_packages($packages, $type_id);
    		    $heading_text_default = empty($type_id) ? lang('select_pricing_plan') : lang('heading_advertise');
    		    $heading_sub_text_default = empty($type_id) ? lang('package_sub_heading') : lang('tagline_advertise');
    		    $rkey = 'packages_result';
    		    $additional_params['type_id'] = $type_id;
    			$view_all = false;
		    }
		    break;
		case 'testimonials':
		    if(GC_PRODUCT_ID == 3) {
		        $records = $this->layouts->widget_fftv($load, $params);
		    } else {
    		    $module = 'testimonials';
    		    $full_length = empty($params['full_length']) ? 0 : null;
    		    $records = $this->layouts->widget_testimonials($limit, $module, $full_length);
    		    $heading_text_default = lang('heading_tm01');
    		    $heading_sub_text_default = lang('heading_sub_testimonials');
    		    $rkey = 'testimonials';
    		    $content_widget_default = 'slider-slick';
    			$view_all = false;
		    }
			break;
		case 'videos':
		    if(GC_PRODUCT_ID == 3) {
		        $records = $this->layouts->widget_fftv($load, $params);
		    }
		    break;
		case 'features':
		case 'faqs':
		case 'references':
		case 'screenshots':
		case 'teams':
		    if(GC_PRODUCT_ID == 3) {
		        $records = $this->layouts->widget_fftv($load, $params);
		    }
		    break;
		case 'products':
		case 'sliders':
		case 'fprlvs':
		    if(GC_PRODUCT_ID == 3) {
		        $records = $this->layouts->widget_fprlv($module, $block, $limit, $params);
		    } else {
    		    $params['module'] = $module;
    		    $params['length'] = 200;
    		    $params['address_tpl'] = 2;
    		    $params['keyword_wrap'] = null;
    		    $params['thumb'] = false;
    		    $records = $this->layouts->widget_sliders($params);
		    }
		    if($load != 'fprlvs') {
		        $content_widget_default = (count($records) > $cols && (empty($wparams['slider']) || (!empty($wparams['slider']) && $wparams['slider'] != 2)) ? 'slider-slick ' : '') . 'gc-grid-view';
		    } else {
		        $content_class_default = 'fprlv-container';
		    }
		    $heading_text_default = lang($block) . ' ' . lang($module);
		    break;
	}
    if(!empty($records)):
    $clearfix = $cols;
    $block_cols = 12 / $cols;
    $parent_class = !empty($wparams['parent_class']) ? ' ' . $wparams['parent_class'] : '';
    $content_parent_class = !empty($wparams['content_parent_class']) ? $wparams['content_parent_class'] : 'container-fluid';
    $content_class = !empty($wparams['content_class']) ? $wparams['content_class'] : $content_class_default;
    $content_widget_class = !empty($wparams['content_widget_class']) ? $wparams['content_widget_class'] : $content_widget_default;
    $heading_block_class = !empty($wparams['heading_block_class']) ? $wparams['heading_block_class'] : 'heading-block';
    $heading_class = !empty($wparams['heading_class']) ? $wparams['heading_class'] : 'heading-theme-primary';
    $heading_sub_class = !empty($wparams['heading_sub_class']) ? $wparams['heading_sub_class'] : 'heading-theme-secondary';
    $widget_class_name = str_replace('_', '', $widget);
    ?>
    <section class="<?php echo $widget_class_name . '-' . $module . '-' . $block . ' ' . $load . '-' . $widget_class_name; ?><?php echo $parent_class; ?>"<?php echo !empty($parent_bg) ? ' style="background-image: url(' . gc_image_url($parent_bg, false, false, 'backgrounds') . ')"' : ''; ?>>
    <?php
    if(empty($wparams['heading']) || $wparams['heading'] == 1):
    $heading_text = !empty($wparams['heading_text']) ? (!empty($wparams['heading_text_lang']) ? lang($wparams['heading_text']) : $wparams['heading_text']) : $heading_text_default;
    $heading_sub_text = !empty($wparams['heading_sub_text']) ? (!empty($wparams['heading_sub_text_lang']) ? lang($wparams['heading_sub_text']) : $wparams['heading_sub_text']) : $heading_sub_text_default;
    ?>
    <div class="<?php echo $heading_block_class; ?> position-relative">
        <h3 class="<?php echo $heading_class; ?>"><?php echo $heading_text;?></h3>
        <?php if(!empty($heading_sub_text)): ?>
        <h5 class="<?php echo $heading_sub_class; ?>"><?php echo $heading_sub_text;?></h5>
        <?php endif; ?>
    </div>
    <?php endif;?>
    <div class="<?php echo $content_parent_class; ?>"<?php echo !empty($wparams['content_bg']) ? ' style="background-image: url(' . gc_image_url($wparams['content_bg'], false, false, 'backgrounds') . ')"' : ''; ?>>
        <div class="<?php echo $content_class; ?>">
            <div class="<?php echo $content_widget_class; ?>" id="<?php echo $widget_class_name . $module . $block . '-' . $cols; ?>" data-arrows="<?php echo !empty($wparams['arrows']) && $wparams['arrows'] == 2 ? 0 : 1; ?>">
            	<?php $additional_params = array('module' => $module, $rkey => $records, 'cols' => $block_cols, 'clearfix' => $clearfix); ?>
            	<?php echo theme_view('widgets/' . $load . '/' . $widget, array_merge($params, $additional_params)); ?>
            </div>
			<?php 
				if(!empty($view_all)): 
				switch($load):
					case 'categories':
						echo anchor(gc_url_format('categories'), lang('view_all'), 'class="btn btn-theme-tertiary mb-5 rounded-0 btn-lg"');
						break;
				endswitch;
				endif;
			?>
        </div>
    </div>
    </section>
    <?php endif; ?>
<?php endif; ?>