/*
 * Easing Compatibility v1 - http://gsgd.co.uk/sandbox/jquery/easing
 *
 * Adds compatibility for applications that use the pre 1.2 easing names
 *
 * Copyright (c) 2007 George Smith
 * Licensed under the MIT License:
 *   http://www.opensource.org/licenses/mit-license.php
 */
(function($) {
    $.extend($.easing, {
        easeIn: function(x, t, b, c, d) {
            return $.easing.easeInQuad(x, t, b, c, d);
        },
        easeOut: function(x, t, b, c, d) {
            return $.easing.easeOutQuad(x, t, b, c, d);
        },
        easeInOut: function(x, t, b, c, d) {
            return $.easing.easeInOutQuad(x, t, b, c, d);
        },
        expoin: function(x, t, b, c, d) {
            return $.easing.easeInExpo(x, t, b, c, d);
        },
        expoout: function(x, t, b, c, d) {
            return $.easing.easeOutExpo(x, t, b, c, d);
        },
        expoinout: function(x, t, b, c, d) {
            return $.easing.easeInOutExpo(x, t, b, c, d);
        },
        bouncein: function(x, t, b, c, d) {
            return $.easing.easeInBounce(x, t, b, c, d);
        },
        bounceout: function(x, t, b, c, d) {
            return $.easing.easeOutBounce(x, t, b, c, d);
        },
        bounceinout: function(x, t, b, c, d) {
            return $.easing.easeInOutBounce(x, t, b, c, d);
        },
        elasin: function(x, t, b, c, d) {
            return $.easing.easeInElastic(x, t, b, c, d);
        },
        elasout: function(x, t, b, c, d) {
            return $.easing.easeOutElastic(x, t, b, c, d);
        },
        elasinout: function(x, t, b, c, d) {
            return $.easing.easeInOutElastic(x, t, b, c, d);
        },
        backin: function(x, t, b, c, d) {
            return $.easing.easeInBack(x, t, b, c, d);
        },
        backout: function(x, t, b, c, d) {
            return $.easing.easeOutBack(x, t, b, c, d);
        },
        backinout: function(x, t, b, c, d) {
            return $.easing.easeInOutBack(x, t, b, c, d);
        }
    });
})(jQuery);

/*
author: Boostraptheme
author URL: https://boostraptheme.com
License: Creative Commons Attribution 4.0 Unported
License URL: https://creativecommons.org/licenses/by/4.0/
*/

// ====================================================
// ANIMATION
// ====================================================
$(function() {
    // animate on scroll
    new WOW().init();
});

// ====================================================
// NAVIGATION
// ====================================================
(function($) {
    "use strict"; // Start of use strict
    // Smooth scrolling using jQuery easing
    $('a.js-scroll-trigger[href*="#"]:not([href="#"])').click(function() {
        var target = $(this.hash);
        target = target.length ? target : $('[name=' + this.hash.slice(1) + ']');
        if (target.length) {
            $('html, body').animate({
                scrollTop: (target.offset().top - 0)
            }, 1000, "easeInOutExpo");
            return false;
        }
    });

    // Closes responsive menu when a scroll trigger link is clicked
    $('.js-scroll-trigger').click(function() {
        $('.navbar-collapse').collapse('hide');
    });

    $("a.smooth-scrolls").click(function(event) {

        // event.preventDefault();

        // get/return id like #about, #work, #team and etc
        var section = $(this).attr("href");

        $('html, body').animate({
            scrollTop: ($(section).offset().top - 2)
        }, 1000, "easeInOutExpo");
    });

    // Activate scrollspy to add active class to navbar items on scroll
    $('body').scrollspy({
        target: '#mainNav',
        offset: 62
    });

    // $(".navbar-collapse ul li a").on("click touch", function(){

    //     $(".navbar-toggle").click();
    // });

    // search bar
    jQuery('.search').on("click", function() {
        if (jQuery('.search-button').hasClass('fa-search')) {
            jQuery('.search-open').fadeIn(500);
            jQuery('.search-button').removeClass('fa-search');
            jQuery('.search-button').addClass('fa-times');
        } else {
            jQuery('.search-open').fadeOut(500);
            jQuery('.search-button').addClass('fa-search');
            jQuery('.search-button').removeClass('fa-times');
        }
    });

    //fixed navbar
    var toggleAffix = function(affixElement, scrollElement, wrapper) {

        var height = affixElement.outerHeight(),
            top = wrapper.offset().top;

        if (scrollElement.scrollTop() >= top) {
            wrapper.height(height);
            affixElement.addClass("affix");
        } else {
            affixElement.removeClass("affix");
            wrapper.height('auto');
        }

    };


    $('[data-toggle="affix"]').each(function() {
        var ele = $(this),
            wrapper = $('<div></div>');

        ele.before(wrapper);
        $(window).on('scroll resize', function() {
            toggleAffix(ele, $(this), wrapper);
        });

        // init
        toggleAffix(ele, $(window), wrapper);
    });

    // Hover dropdown 
    $('ul.navbar-nav li.dropdown').hover(function() {
        $(this).find('.dropdown-menu').stop(true, true).delay(200).fadeIn(500);
    }, function() {
        $(this).find('.dropdown-menu').stop(true, true).delay(200).fadeOut(500);
    });

})(jQuery); // End of use strict