<?php 
/**
 * Search Form Layouts
 * @param boolean $tpl
 * e.g. usage <?php echo view('widgets/search/_form', array('tpl' => 'horizontal', 'module' => 'listings', 'fields' => array(), 'parent_class' => '', 'child_class' => '')); ?>
 * Example Default: echo view('widgets/search/_form', array('tpl' => 'default', 'module' => 'listings', 'fields' => array('location' => array('container' => 'col-md-6 mb-2', 'field_class' => 'form-control-lg rounded-0'), 'category' => array('container' => 'col-md-6 mb-2', 'field_class' => 'form-control-lg rounded-0'), 'query' => array('container' => 'col-md-12', 'field_class' => 'form-control-lg rounded-0')), 'parent_class' => 'navbar-form form mt-2', 'child_class' => ''));
 * Example Horizontal: echo view('widgets/search/_form', array('tpl' => 'horizontal', 'module' => 'listings', 'fields' => array('category' => array(), 'location' => array(), 'query' => array(), 'submit' => array()), 'parent_class' => '', 'child_class' => ''));
 * Example Sidebar: echo view('widgets/search/_form', array('tpl' => 'default', 'module' => 'listings', 'fields' => array('category' => array('container' => 'col-md-12 mb-2'), 'location' => array('container' => 'col-md-12 mb-2'), 'query' => array('container' => 'col-md-12', 'group' => 1), 'submit' => array('container' => 'col-md-12 mt-2 mb-2')), 'parent_class' => '', 'child_class' => ''));
 * Example Oceanus: echo view('widgets/search/_form', array('tpl' => 'oceanus', 'module' => 'listings', 'module' => 'listings', 'parent_class' => 'text-left', 'fields' => array('category' => array('icon' => 'fas fa-search text-muted', 'field_class' => 'form-control ebizheader_categories-select'), 'location' => array('icon' => 'fas fa-location-arrow text-muted', 'field_class' => 'form-control-lg rounded-0'), 'query' => array('icon' => 'fas fa-pen text-muted', 'field_class' => 'form-control-lg rounded-0', 'group' => 1), 'submit' => array('field_class' => 'btn btn-theme-secondary btn-block', 'tpl' => 'text_only')), 'child_class' => ''));
 * Example Modero: echo view('widgets/search/_form', array('tpl' => 'input-group', 'module' => 'listings', 'parent_class' => 'form-inline navsearch w-100', 'fields' => array('category' => array(), 'location' => array(), 'query' => array('group' => 1), 'submit' => array()), 'child_class' => ''));
 */
$tpl = !empty($tpl) ? $tpl : '';
$action_url = gc_url_format('search');
if($module != 'listings'):
    $action_url .= '?md=' . gc_module_id($module);
endif;
$fields = !empty($fields) ? $fields : array('category' => array(), 'location' => array(), 'query' => array(), 'submit' => array()); // 'city', 'locality'
$parent_class = !empty($parent_class) ? ' ' . $parent_class : '';
$data = array(
    'country' => isset($default_country) ? $default_country : 'US', 
    'state' => isset($default_state) ? $default_state : 0, 
    'city' => (isset($default_city) ? $default_city : 0), 
    'location' => (isset($searchLocation) ? $searchLocation : ''), 
    'type_location' => (isset($type_location) ? $type_location : 1),
    'cities' => (isset($cities) ? $cities : array()),
    'localities' => (isset($localities) ? $localities : array()),
    'category' => (isset($searchCategory) ? $searchCategory : 0),
    'categories' => (!empty($header_categories) ? $header_categories : $this->categories_model->categories(0, 0, $module, null, 1000)),
    'keyword' => (isset($searchKeyword) ? $searchKeyword : ''),
    'module' => $module
);
?>
<?php echo form_open($action_url, array('method' => 'get', 'id' => 'search-form', 'class' => 'gc-search-form search-form' . $parent_class)); ?>
<?php
switch($tpl):
    case 'horizontal': 
    $field_parent_class = "form-group row";
    $field_label_class = "col-sm-3 col-form-label col-form-label-lg";
    $field_container_class = "col-sm-9";
    foreach($fields as $field => $params):
        if($field != 'submit'):
    ?>
    <div class="<?php echo $field_parent_class; ?>">
        <label for="<?php echo $field; ?>" class="<?php echo $field_label_class; ?>">
            <?php echo lang('gc_search_' . $field); ?>
        </label>
        <div class="<?php echo $field_container_class; ?>">
            <?php 
            $data['field'] = $field;
            $data['class'] = 'w-100 form-control-lg';
            echo theme_view('widgets/search/_fields', array('data' => $data));
            ?>
        </div>
    </div>
    <?php else: ?>
        <div class="<?php echo $field_parent_class; ?>">
            <div class="offset-sm-3 col-sm-9">
                <button type="submit" name="search" class="btn btn-primary btn-block"><?php echo lang('bf_action_search');?></button>
            </div>
        </div>
    <?php endif; ?>
    <?php endforeach; ?>
    <?php break; case 'oceanus':?>
        <?php
        $field_parent_class = "form-group row";
        foreach($fields as $field => $params):
        ?>
    <div class="<?php echo $field_parent_class; ?>">
        <div class="col-sm-4">
            <?php if($field != 'submit'): ?>
            <label for="<?php echo $field; ?>" class="mt-1"><?php echo lang('gc_search_' . $field); ?></label>
            <?php endif; ?>
        </div>
        <div class="col-sm-8">
            <div class="input-group">
                <?php if($field != 'submit'): ?>
                <div class="input-group-prepend">
                    <span class="input-group-text rounded-0 demo-4"><i class="<?php echo !empty($params['icon']) ? $params['icon'] : 'fas fa-pen text-muted';?>"></i></span>
                </div>
                <?php endif; ?>
                <?php 
                $data['field'] = $field;
                $data['class'] = !empty($params['field_class']) ? $params['field_class'] : 'w-100';
                if(!empty($params['tpl'])):
                    $data['tpl'] = $params['tpl'];
                endif;
                echo theme_view('widgets/search/_fields', array('data' => $data));
                ?>
            </div>
        </div>
    </div>
    <?php endforeach; ?>
    <?php break; case 'input-group':?>
        <div class="input-group has-feedback-right searchbar w-100">
            <div class="input-group-append w-100">
        <?php
        foreach($fields as $field => $params):
            $field_class = '';
            switch($field):
                case 'city':
                    $field_class = 'form-control-lg w-100 rounded-0';
                    break;
                case 'locality':
                    $field_class = 'form-control-lg w-100 rounded-0';
                    break;
                case 'category':
                    $field_class = 'search-selectize selectize-first-child form-control-lg p-0 rounded-0';
                    $data['field_id'] = !empty($params['field_id']) ? $params['field_id'] : 0;
                    break;
                case 'location':
                    $field_class = 'search-input form-control-lg w-100 rounded-0';
                    break;
                case 'query':
                    $data['group'] = empty($params['group']) ? true : false;
                    $field_class = 'search-input w-100 form-control-lg rounded-0';
                    if(!empty($params['btn_class'])):
                        $data['btn_class'] = $params['btn_class'];
                    endif;
                    break;
                case 'submit':
                    $field_class = 'search-btn' . (!empty($params['field_class']) ? ' ' . $params['field_class'] : '');
                    if(!empty($params['tpl'])):
                        $data['tpl'] = $params['tpl'];
                    endif;
                    break;
            endswitch;
            $data['field'] = $field;
            $data['class'] = $field_class;
            echo theme_view('widgets/search/_fields', array('data' => $data));
        endforeach; ?>
        </div>
    </div>
    <?php break; default:?>    
        <div class="form-row">
        <?php foreach($fields as $field => $params):?>
            <div class="<?php echo !empty($params['container']) ? $params['container'] : 'col-md-4'; ?>">
            <?php
            $field_class = '';
            switch($field):
                case 'location':
                    $field_class = 'search-input w-100';
                    break;
                case 'category':
                    $data['field_id'] = !empty($params['field_id']) ? $params['field_id'] : 0;
                    break;
                case 'query':
                    $data['group'] = empty($params['group']) ? true : false;
                    $field_class = 'search-input w-100';
                    if(!empty($params['btn_class'])):
                        $data['btn_class'] = $params['btn_class'];
                    endif;
                    break;
                case 'submit':
                    $field_class = 'search-btn';
                    break;
            endswitch;
            $data['field'] = $field;
            $data['class'] = $field_class . (!empty($params['field_class']) ? ' ' . $params['field_class'] : '');
            echo theme_view('widgets/search/_fields', array('data' => $data));
            ?>        
            </div>
        <?php endforeach; ?>
        </div>
<?php endswitch; ?>
<?php echo form_close(); ?>