<?php if (! defined('BASEPATH')) exit('No direct script access allowed');
/*
 * =======================================================================*\
 * || ##################################################################### ||
 * || # GoClixy - Business Directory Script # ||
 * || # Copyright (C) 2014 GoClixy. All Rights Reserved. # ||
 * || # ---------------------------------------------------------------- # ||
 * || # This file is owned and copyrighted by GoClixy.com. Your license # ||
 * || # confers no title or ownership in the Script and is not a sale # ||
 * || # of any rights to the Software. This file may not be redistributed # ||
 * || # in whole or significant part. # ||
 * || # --------------- COPYRIGHTED COMMERCIAL SOFTWARE --------------- # ||
 * || # https://www.goclixy.com | https://www.goclixy.com/license.html # ||
 * || ##################################################################### ||
 * \*=======================================================================
 */
class Listings_model extends BF_Model
{
    
    protected $table_name = "listings";
    protected $key = "id";
    protected $date_format = "datetime";
    protected $log_user = TRUE;
    protected $soft_deletes = TRUE;
    protected $set_created = TRUE;
    protected $set_modified = TRUE;
    protected $created_field = 'created_on';
    protected $created_by_field = 'created_by';
    protected $modified_field = 'modified_on';
    protected $modified_by_field = 'modified_by';
    protected $deleted_field = 'deleted';
    protected $deleted_by_field = 'deleted_by';
    
    /*
     * Customize the operations of the model without recreating the insert, update,
     * etc methods by adding the method names to act as callbacks here.
     */
    protected $before_insert = array();
    protected $after_insert = array();
    protected $before_update = array();
    protected $after_update = array();
    protected $before_find = array();
    protected $after_find = array();
    protected $before_delete = array();
    protected $after_delete = array();
    
    /*
     * For performance reasons, you may require your model to NOT return the
     * id of the last inserted row as it is a bit of a slow method. This is
     * primarily helpful when running big loops over data.
     */
    protected $return_insert_id = TRUE;
    
    // The default type of element data is returned as.
    protected $return_type = "object";
    
    // Items that are always removed from data arrays prior to
    // any inserts or updates.
    protected $protected_attributes = array();
    
    protected $insert_validation_rules = array();
    
    protected $skip_validation = FALSE;
    
    protected $z_cache;
    
    // --------------------------------------------------------------------
    // !ACTIVATION
    // --------------------------------------------------------------------
    public function __construct()
    {
        $this->load->library('settings/Settings_lib');
        // if (settings_item('site.enable_cache') == 1) {
            $this->load->library('gzf_cache', array(
                'lifetime' => 900
            ));
            $this->load->library('slug');
            $this->z_cache = $this->gzf_cache->get_instance();
        // }
    }
    
    /**
     * Email Template Fields
     *
     * @return array
     */
    public function email_templates($role = 'admin') {
        $module_options = array();
        $modules = gc_modules();
        foreach($modules as $module) {
            $module_options[] = 'name:' . $module . ';value:' . $module;
        }
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'dropdown',
                'name' => 'module_table',
                'label' => 'label_module',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => implode('|', $module_options),
                'options_detailed' => 1,
                'options_lang' => 1,
                'class' => 3,
                'rules' => 'trim|unique_email_template[contact_queries]',
            ),
            array(
                'type' => 'dropdown',
                'name' => 'user_type',
                'label' => 'bf_user',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:placeholder_select;value:|name:us_admin;value:admin|name:bf_user;value:user|name:us_guest;value:guest|name:us_seller;value:seller',
                'options_detailed' => 1,
                'options_lang' => 1,
                'class' => 3,
                'rules' => 'trim',
            ),
            array(
                'type' => 'dropdown',
                'name' => 'action_type',
                'label' => 'label_action',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:placeholder_select;value:|name:add;value:add|name:edit;value:edit|name:delete;value:delete|name:em_status_update;value:status_update|name:label_share;value:share|name:gc_menu_erroneous;value:erroneous|name:em_expiry_notification;value:expiry_notification|name:gc_menu_bulk_users_generate;value:bulk|name:product_back_in_stock;value:back_in_stock|name:product_low_stock;value:low_stock|name:product_orders;value:order',
                'options_detailed' => 1,
                'options_lang' => 1,
                'class' => 3,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'name',
                'label' => 'label_name',
                'placeholder' => 'placeholder_name',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
                'rules' => 'required|trim|max_length[50]'
            ),
            array(
                'type' => 'text',
                'name' => 'subject',
                'label' => 'label_subject',
                'placeholder' => 'placeholder_subject',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'required|trim|max_length[255]'
            ),
            array(
                'type' => 'textarea',
                'name' => 'message',
                'label' => 'label_message',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'custom_rules' => 'editor',
                'field_class' => 'editor',
                'class' => 9,
                'rules' => 'trim|required',
            ),
            array(
                'type' => 'textarea',
                'name' => 'sms_text',
                'label' => 'sms_text',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'class' => 9,
                'field_extra' => 'rows:3',
                'rules' => 'trim',
            ),
            array(
                'type' => 'textarea',
                'name' => 'params',
                'label' => 'em_params',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'class' => 9,
                'tooltip' => lang('em_params_help'),
                'field_extra' => 'rows:3',
                'rules' => 'trim',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'status',
                'label' => 'label_status',
                'frontend' => false,
                'admin_only' => false,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            )
        );
        return $fields;
    }
    
    /**
     * Count Inactive listings.
     *
     * @access public
     *
     * @return int Inactive listing count.
     */
    public function count_inactive_listings()
    {
        $this->db->where('active', - 1);
        return $this->count_all(FALSE);
    }
    // end count_inactive_listings()
    
    /**
     * Activate Business Listing
     *
     * @access public
     *
     * @param int $listing_id
     *            The listing
     *
     * @return int Listing Id on success, FALSE on error
     */
    public function activate($listing_id)
    {
        $this->db->update($this->table_name, array(
            'status' => 1
        ), array(
            'id' => $listing_id
        ));
        
        if ($this->db->affected_rows() != 1) {
            return FALSE;
        }
        
        return TRUE;
    }
    // end activate()
    
    /**
     * This function is triggered during listing set up to assure listing is not active.
     * This function can be used to deactivate listings based on public view events.
     *
     * @param int $listing_id
     *            The listing or email to match to deactivate
     *
     * @return mixed $activate_hash on success, FALSE on error
     */
    public function deactivate($listing_id)
    {
        $this->db->update($this->table_name, array(
            'status' => 0
        ), array(
            'id' => $listing_id
        ));
        
        if ($this->db->affected_rows() != 1) {
            return FALSE;
        }
        
        return TRUE;
    }
    // end deactivate()
    
    /**
     * Admin specific activation function for admin approvals or re-activation.
     *
     * @access public
     *
     * @param int $listing_id
     *            The listing ID to activate
     *
     * @return bool TRUE on success, FALSE on error
     */
    public function listing_activation($listing_id = FALSE)
    {
        if ($listing_id === FALSE) {
            $this->error = lang('ls_err_no_id');
            return FALSE;
        }
        
        if ($this->activate($listing_id, 'id', FALSE)) {
            return $listing_id;
        } else {
            $this->error = lang('ls_err_listing_is_active');
            return FALSE;
        }
    }
    // end listing_activation()
    
    /**
     * Admin only deactivation function.
     *
     * @access public
     *
     * @param int $listing_id
     *            The listing ID to deactivate
     *
     * @return bool TRUE on success, FALSE on error
     */
    public function listing_deactivation($listing_id = FALSE)
    {
        if ($listing_id === FALSE) {
            $this->error = lang('ls_err_no_id');
            return FALSE;
        }
        
        if ($this->deactivate($listing_id, 'id', FALSE)) {
            return $listing_id;
        } else {
            $this->error = lang('ls_err_listing_is_inactive');
            return FALSE;
        }
    }
    // end admin_deactivation()
    
    /**
     * To update listing information
     *
     * @param int $id
     * @param array $data
     */
    public function update_listing($id = NULL, $data = array())
    {
        return $this->db->update($this->table_name, $data, array(
            'id' => $id
        ));
    }
    
    /**
     * Delete all listing record permanently
     *
     * @param int $id
     * @param boolean $purge
     */
    public function delete($id = 0, $purge = FALSE)
    {
        if ($purge === TRUE) {
            // temporarily set the soft_deletes to TRUE.
            $this->soft_deletes = FALSE;
        } /*
        * else { // The user might have some other records which are deleted, restoring the listing will restore all records including the deleted one
        * $this->update_recursive($id, 'deleted', 1); // On listing delete also soft delete internal modules.
        * }
        */
        return parent::delete($id);
    }
    // end delete()
    
    // PHOTOS FILTER
    public function photos_filter()
    {
        $this->db->select('distinct(listing_id)');
        return $this->db->get($this->db->dbprefix . 'images')->result();
    }
    
    /**
     * Get parent categories using AJAX
     *
     * @return multitype:string NULL
     */
    public function get_parent_categories()
    {
        $this->table_name = 'categories';
        $result = $this->listings_model->select('id, name')
        ->where('parent_id', 0)
        ->find_all();
        
        $options = array(); // FILL OPTION ARRAY
        
        $options[''] = '-- Select Parent Category --';
        // Format for passing into form_dropdown function
        foreach ($result as $option) {
            $options[$option->id] = $option->name;
        }
        return $options;
    }
    
    /**
     * Get sub categories using AJAX based on parent_id
     *
     * @param string $parent_id
     * @return void|multitype:NULL
     */
    public function get_sub_categories($parent_id = NULL)
    {
        if (! is_null($parent_id)) {
            $this->table_name = 'categories';
            $results = $this->listings_model->select('id, name')
            ->where('parent_id', $parent_id)
            ->find_all();
            $sub_categories = array();
            // if there are sub categories
            if ($results) {
                // Format for passing into jQuery loop
                foreach ($results as $result) {
                    $sub_categories[$result->id] = $result->name;
                }
                return $sub_categories;
            }
        }
        return;
    }
    
    /**
     * Display countries to the user
     *
     * @return array countries iso and name
     */
    public function get_countries()
    {
        $this->table_name = 'countries';
        return $this->listings_model->select('iso, name')
        ->order_by('name', 'asc')
        ->find_all();
    }
    
    /**
     * Display states information to the user based on country_iso
     *
     * @param string $country_iso
     */
    public function get_states($country_iso = NULL)
    {
        $this->table_name = 'states';
        if ($country_iso != NULL) {
            return $this->listings_model->select('id, name')
            ->where('country_iso', $country_iso)
            ->order_by('name', 'asc')
            ->find_all();
        } else {
            return;
        }
    }
    
    /**
     * Display cities to the user based on State ID
     *
     * @param int $state_id
     */
    public function get_cities($state_id = NULL)
    {
        $this->table_name = 'cities';
        if ($state_id != NULL) {
            return $this->listings_model->select('id, name')
            ->where('state_id', $state_id)
            ->order_by('name', 'asc')
            ->find_all();
        } else {
            return;
        }
    }
    
    /**
     * Get localities information based on City ID
     *
     * @param int $city_id
     */
    public function get_localities($city_id = NULL)
    {
        $this->table_name = 'localities';
        if ($city_id != NULL) {
            return $this->listings_model->select('id, name')
            ->where('city_id', $city_id)
            ->order_by('name', 'asc')
            ->find_all();
        } else {
            return;
        }
    }
    
    /**
     * Populate location information using AJAX
     *
     * @param string $loadType
     *            for state or city
     * @param string/int $loadId
     */
    public function getData($loadType, $loadId)
    {
        if ($loadType == "country") {
            $this->table_name = 'countries';
            return $this->listings_model->select('iso AS id, name')
            ->order_by('name', 'asc')
            ->find_all_by('active', 1);
        } else if ($loadType == "state") {
            $this->table_name = 'states';
            return $this->listings_model->select('id, name')
            ->where('country_iso', $loadId)
            ->order_by('name', 'asc')
            ->find_all_by('active', 1);
        } elseif (($loadType == "city")) {
            $this->table_name = 'cities';
            return $this->listings_model->select('id, name')
            ->where('state_id', $loadId)
            ->order_by('name', 'asc')
            ->find_all_by('active', 1);
        } else {
            $this->table_name = 'localities';
            return $this->listings_model->select('id, name')
            ->where('city_id', $loadId)
            ->order_by('name', 'asc')
            ->find_all_by('active', 1);
        }
    }
    
    /* ---------------------------------------------------- */
    /*
     * Frontend Functions
     * /*----------------------------------------------------
     */
    
    
    /**
     * Get listing information for detail page
     *
     * @param int $id
     * @return array information
     */
    public function get_listing($id, $mobile = false, $slug = null) {
        $sql = "SELECT L.*, L.logo AS logo_url, CT.slug as city_slug,";
        $sql .= " L.user_id AS author_id, U.display_name AS author_name,";
        $sql .= " U.slug AS author_slug, U.city AS author_city, U.profile_image AS author_image, U.email AS author_email,";
        $sql .= " P.price AS package_price, P.claim_price AS package_claim_price, P.display_fields, P.loggedin_display_fields, P.trusted_seal, P.border_color";
        $sql .= " FROM " . $this->db->dbprefix . "listings L";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "cities CT ON CT.id = L.city_id";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "users U ON L.user_id = U.id";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "packages P ON L.package_id = P.id";
        $sql .= " WHERE L.deleted = 0 AND L.status = 1";
        if(empty($slug)) {
            $sql .= " AND L.id = " . $id;
        } else {
            $sql .= " AND L.slug = '" . $slug . "'";
        }
        return $this->db->query($sql)->row();
    }
    
    /**
     * Get user information for detail page
     *
     * @param int $user_id
     */
    public function get_user_info($user_id)
    {
        $this->table_name = "users";
        return $this->listings_model->find($user_id);
    }
    
    /**
     * Listing information for PDF
     *
     * @param int $listing_id
     */
    public function get_listing_detail($listing_id)
    {
        // loc.name as locality, c.name as city, s.name as state, ct.name as country,
        $this->db->select('listings.*, u.display_name, u.email AS user_email, p.title as package, p.claim_price');
        /*
         * $this->db->join($this->db->dbprefix . 'localities as loc','loc.id = ' . $this->db->dbprefix . 'listings.locality_id', 'left');
         * $this->db->join($this->db->dbprefix . 'cities as c','c.id = ' . $this->db->dbprefix . 'listings.city_id', 'left');
         * $this->db->join($this->db->dbprefix . 'states as s','s.id = ' . $this->db->dbprefix . 'listings.state_id', 'left');
         * $this->db->join($this->db->dbprefix . 'countries ct','ct.iso = ' . $this->db->dbprefix . 'listings.country_iso', 'left');
         */
        $this->db->join('users as u', 'u.id = ' . $this->db->dbprefix . 'listings.user_id', 'left');
        $this->db->join('packages as p', 'p.id = ' . $this->db->dbprefix . 'listings.package_id', 'left');
        return $this->listings_model->find($listing_id);
    }
    
    /**
     * State, City and Zip Search
     *
     * @param string $keyword
     * @return object
     */
    public function ajax_search_location($keyword, $module = null, $limit = 10)
    {
        /**
         * Selection Location Codes
         * 1 - City
         * 2 - State
         * 3 - Postcode
         * 4 - Locality
         */
        $sql = "SELECT * FROM (SELECT S.name, 2 AS type FROM " . $this->db->dbprefix . "states S WHERE S.name LIKE '%" . $keyword . "%' AND S.active = 1";
        if (settings_item('lst.country_filter') == 1) {
            $sql .= " AND S.country_iso = '" . $this->session->userdata('search_country') . "'";
        }
        // $sql .= " LIMIT " .$limit;
        $sql .= " UNION SELECT CONCAT(C.name, ', ', ST.name) as name, 1 AS type FROM " . $this->db->dbprefix . "cities C INNER JOIN " . $this->db->dbprefix . "states ST ON ST.id = C.state_id";
        $sql .= " WHERE C.name LIKE '" . $keyword . "%' AND C.active = 1";
        if (settings_item('lst.country_filter') == 1) {
            $sql .= " AND ST.country_iso = '" . $this->session->userdata('search_country') . "'";
        }
        ;
        // $sql .= " LIMIT " .$limit;
        $sql .= " UNION";
        if ($module == null) {
            $sql .= " SELECT L.postcode AS name, 3 AS type FROM " . $this->db->dbprefix . "listings L WHERE L.postcode LIKE '%" . $keyword . "%' GROUP BY L.postcode UNION";
        } else {
            $sql .= " SELECT M.postcode AS name, 3 AS type FROM " . $this->db->dbprefix . $module . " M WHERE M.postcode LIKE '%" . $keyword . "%' GROUP BY M.postcode UNION";
        }
        $sql .= " SELECT LOC.name, 4 AS type FROM " . $this->db->dbprefix . "localities LOC INNER JOIN " . $this->db->dbprefix . "cities ON " . $this->db->dbprefix . "cities.id = LOC.city_id INNER JOIN " . $this->db->dbprefix . "states ON " . $this->db->dbprefix . "states.id = " . $this->db->dbprefix . "cities.state_id";
        if (settings_item('lst.country_filter') == 1) {
            $sql .= " AND " . $this->db->dbprefix . "states.country_iso = '" . $this->session->userdata('search_country') . "'";
        }
        $sql .= " WHERE LOC.name LIKE '%" . $keyword . "%' AND LOC.active = 1) SR LIMIT " . $limit;
        $query = $this->db->query($sql);
        $result = $query->result();
        return $result;
    }
    
    /**
     * Get results based on user entered keywords
     *
     * @param number $count
     * @param string $module
     * @param string $keyword
     * @param array $where_fields: all table specific fields e.g. category, tag, city, location, featured etc. must be passed using this variable
     * extra_where must also be passed through this variable only
     *
     * @param string $sort_by
     * @param number $limit for pagination
     * @param number $offset for paginatione
     * @param array $extra: extra fields like user_id for favorites will be passed using this variable
     * Note: Filter/Additional Search Fields are also passed via $extra variable
     *
     * $type_location - the user selected a location from the typehead search result
     *            Selection Location Codes = 1 - City / 2 - State / 3 - Postcode / 4 - Locality
     * @return object
     */
    private function search_module($count, $module, $keyword, $where_fields = array(), $sort_by = 'id DESC', $limit = 0, $offset = 0, $extra = array()) {
        // $country, $location, $location_id, $postcode = 0, $type_location = 0, $category_id = 0, $locality_id = 0, $tag_id = 0, $featured = 0, $sort_by, $where = NULL, $limit = 0, $offset = 0, $fields, $user_id = 0
        // Search only fields lookup in the listings table using LIKE operator
        // therefore, the keyword must not change on full_text_search
        $sphinx = settings_item('site.sphinx') == 1 ? true : false; //  || !empty($_GET['sphinx'])
        $search_only_keyword = $keyword;
        $offset = ! empty($offset) ? $offset : 0; // set offset in case of first page as value is always 0
        $featured = !empty($where_fields['featured']) ? 1 : 0;
        $module_id = gc_module_id($module);
        if (!$sphinx && ! empty($keyword)) { // Escape strings with a Apostrophe '
            $keyword = mysqli_real_escape_string($this->gc->get_mysqli(), $keyword);
        }
        
        $has_location = ($module == 'listings' || gc_module_settings($module, 'locations') == 1 ? TRUE : FALSE);
        
        // Uncomment below given line for special characters search issue
        // $keyword = mysql_real_escape_string(utf8_encode(str_replace(" "," +", htmlentities($keyword))));
        if (settings_item('site.full_text_search') == 1 && !empty($keyword)) {
            $keyword = str_replace(" ", " +", $keyword);
        }
        
        // Filter Search added after all TABLES JOIN for desired result
        $filter_search_fields = array();
        $filter_search = array();
        if(!empty($extra['filter_search_fields'])) {
            $filter_search_fields = $extra['filter_search_fields'];
            $filter_search = $this->filter_search($filter_search_fields);
        }
        
        // sphinx is only valid for listings
        $sphinx_where = array();
        if($sphinx && $module == 'listings') {
            $sphinx_data = array(
                'filter_search' => $filter_search,
                'keyword' => $keyword,
                'where_fields' => $where_fields,
                'search_only_keyword' => $search_only_keyword,
                'filter_search_fields' => $filter_search_fields,
                'sort_by' => $sort_by,
                'featured' => $featured,
                'offset' => $offset,
                'limit' => $limit,
                'extra' => $extra,
                'has_location' => $has_location
            );
            $sphinx_result = $this->sphinx($count, $sphinx_data);
            if($sphinx_result) {
                if($count == 0) {
                    if(!empty($sphinx_result['result'])) {
                        $sphinx_where = 'M.id IN (' . $sphinx_result['result'] . ')'; 
                    } else {
                        return false;
                        // $sphinx_where = 'M.id = 0';                    
                    }
                } else {
                    return $sphinx_result['result'];
                }
            }
        }
        
        // select fields
        $select_fields = $this->select_fields($module, $count, $extra);
        $sql = $select_fields['select'] . PHP_EOL;
        
        /**
         * Keyword is not empty
         * The keyword is also not a category or tag
         * In this case search all tables for the matched keywords
         */
        if($count == 0 && settings_item('site.multi_language') == 1) { // multi-language keywords
            $sql .= ", GROUP_CONCAT(DISTINCT(CONCAT(T.name)) SEPARATOR ', ') AS keywords ";
        }
        
        // find nearest based on user's latitude and longitude
        if(!empty($extra['latitude']) && !empty($extra['longitude']) && $has_location) {
            $sql .= ", ROUND(MD.user_distance, 2) AS user_distance";
        }
        
        $sql .= " FROM (";
        $sql .= " SELECT M.*";
        
        // find nearest based on user's latitude and longitude
        if(!empty($extra['latitude']) && !empty($extra['longitude']) && $has_location) {
            $sql .= ", P.distance_unit * DEGREES(ACOS(LEAST(1.0, COS(RADIANS(P.latpoint)) * COS(RADIANS(M.latitude)) * COS(RADIANS(P.longpoint) - RADIANS(M.longitude)) + SIN(RADIANS(P.latpoint)) * SIN(RADIANS(M.latitude))))) AS user_distance";
        }
        
        $sql .= " FROM " . $this->db->dbprefix . $module . " M";        
        
        // find nearest based on user's latitude and longitude
        if(!empty($extra['latitude']) && !empty($extra['longitude']) && $has_location) {
            // a degree of latitude is 111.045 km
            // eliminate the points that are too far north or too far south to possibly be within 25000 km.
            $search_radius = !empty($extra['radius']) ? $extra['radius'] : 25000.0;
            $distance_unit = settings_item('site.distance_unit') == 'km' ? 111.045 : 69.0; // use miles instead of km, the distance_unit for miles is 69.0.
            $sql .= " JOIN (SELECT " . $extra['latitude'] . "  AS latpoint,  " . $extra['longitude'] . " AS longpoint, " . $search_radius . " AS radius, " . $distance_unit . " AS distance_unit) AS P ON 1=1";
        }
        
        if(!empty($sphinx_where)) {
            $sql .= " WHERE " . $sphinx_where;
        } else {            
            // create and force different indexes to make search faster
            // category search is slow with indexes, but the keyword search is fast
            /*if(!empty($keyword)) { // force index only in case of actual query
                $sql .= $this->force_index($module, $featured, $type_location, $sort_by);
             }*/
            if (isset($filter_search['join_metas']) && $filter_search['join_metas']) {
                $sql .= " INNER JOIN " . $this->db->dbprefix . "metas MT ON MT.row_id = M.id AND MT.module_id = " . $module_id . PHP_EOL;
            } else if(!empty($keyword) && !empty($extra['fields'])) { // Deal with search only fields
                $sql .= " LEFT JOIN " . $this->db->dbprefix . "metas MT ON MT.row_id = M.id AND MT.module_id = " . $module_id . PHP_EOL;
            }
            // Open Now Filter
            if($module == 'listings' && isset($_GET['fltr']) && $_GET['fltr'] == 'open') {
                $sql .= " LEFT JOIN " . $this->db->dbprefix . "business_hours BH ON BH.listing_id = M.id AND BH.day_of_week = " . date('N') . PHP_EOL;
                $current_time = date('H:i');
                $open_where = "((BH.open_time < '$current_time' AND BH.close_time > '$current_time') OR M.is_open = 1)";
                $where_fields['where'] = !empty($where_fields['where']) ? 'M.' . $where_fields['where'] . ' AND ' . $open_where : $open_where;
            }
            // display recently viewed items only
            if($sort_by == 'last_viewed') {
                $sql .= " INNER JOIN " . $this->db->dbprefix . "recently_viewed RV ON RV.row_id = M.id AND RV.module_id = " . $module_id;
                if(!empty($extra['user_id'])) {
                    $sql .= " AND RV.user_id = " . $extra['user_id'];
                }
            }
            $sql .= $this->where_only($module, $filter_search, $keyword, $where_fields, $search_only_keyword, $filter_search_fields, 'M.');
            // where get
            $where_get = $this->where_get($module);
            if(!empty($where_get)) {
                $sql .= " AND " . implode(" AND ", $where_get);
            }
            
            // find nearest based on user's latitude and longitude
            if(!empty($extra['latitude']) && !empty($extra['longitude']) && $has_location) {
                $sql .= " AND (M.latitude BETWEEN P.latpoint  - (P.radius / P.distance_unit) AND P.latpoint  + (P.radius / P.distance_unit)";
                $sql .= " AND M.longitude BETWEEN P.longpoint - (P.radius / (P.distance_unit * COS(RADIANS(P.latpoint)))) AND P.longpoint + (P.radius / (P.distance_unit * COS(RADIANS(P.latpoint)))))";
            }
            
            if ($count == 0) {
                $sql .= $this->limit_group_order($sort_by, $featured, $offset, $limit, 'M', $extra, $has_location);
            }
        }
        $sql .= ") MD" . PHP_EOL;
        if ($count == 0) {
            if (! empty($select_fields['join'])) {
                $sql .= $select_fields['join'];
            }
            $sql .= " LEFT JOIN " . $this->db->dbprefix . "packages RP ON RP.id = MD.package_id" . PHP_EOL;
            // If the module name is not listing, then join the listings table
            // to display the listing title with redirect link
            // Do not restrict the user to the listings specific rows, also display rows added without Business Registration
            if ($module != 'listings') {
                // Multi Language Translations
                $sql .= $this->translations->common_joins('listings') . "ML ON ML.id = MD.listing_id" . PHP_EOL;
            }
            // INNER JOIN Users Table
            $sql .= " INNER JOIN " . $this->db->dbprefix . "users U ON U.id = MD.user_id" . PHP_EOL;
            if($sort_by == 'last_viewed') { // display last viewed items only
                $sql .= " INNER JOIN " . $this->db->dbprefix . "recently_viewed RV ON RV.row_id = MD.id AND RV.module_id = " . $module_id . PHP_EOL;
            }
            
            // MULTI LANGUAGE KEYWORDS
            if(settings_item('site.multi_language') == 1) { // multi-language keywords
                $sql .= " LEFT JOIN " . $this->db->dbprefix . "tag_relations TR ON MD.id = TR.row_id AND TR.module_id = " . $module_id . PHP_EOL;
                $sql .= $this->translations->common_joins('tags') . " T ON TR.tag_id = T.id AND T.active = 1" . PHP_EOL;
                $sql .= " GROUP BY MD.id";
            } else if($sort_by == 'last_viewed') {
                $sql .= " GROUP BY MD.id";
            }
            $sql .= $this->_sort_result($sort_by, 'MD', $extra, $has_location);
            $result = $this->db->query($sql)->result();
            return $result;
        } else {
            // INNER JOIN Users Table - Invalid counts issue fix
            // $sql .= " INNER JOIN " . $this->db->dbprefix . "users U ON U.id = MD.user_id" . PHP_EOL;
            $result = $this->db->query($sql)->row();
            return $result ? $result->total_count : 0;
        }
    }
    
    /**
     * Sphinxsearch API version 3
     * 
     * Extract the sphinx in the root folder where application folder exists.
     * https://sphinxsearch.com/downloads/sphinx-3.4.1-efbcc65-linux-amd64.tar.gz/thankyou.html
     * Copy the sphinx.conf in the sphinx/bin folder.
     * 
     * if sphinx is installed on server then query local files
     * overwrite sphinx/sphinx.conf on your server from the application/db folder
     * copy sphinx/sphinx.php file in the root or public_html folder of your website
     * also add a cronjob "indexer --rotate --all" to update files twice a day
     * 
     * $res = mysqli_query($conn, "SHOW VARIABLES");
     * $res = mysqli_query($conn, "SELECT * FROM listings WHERE match('@(location) bengal')"); //  ORDER BY package_priority
     * 
     * search location and keywords both
     * $res = mysqli_query($conn, "SELECT * FROM listings WHERE match('@(title,description) L. d. bhattar + @(location) west') ORDER BY hits DESC"); //  ORDER BY package_priority
     * $res = mysqli_query($conn, "SELECT * FROM listings WHERE match('@(title,description,keywords,categories) L. d. bhattar + @(location) west') ORDER BY hits DESC"); //  ORDER BY package_priority
        
     * $res = mysqli_query($conn, "SELECT * FROM listings WHERE match('@(category_ids) id153c')"); //  ORDER BY package_priority
     * $res = mysqli_query($conn, "SELECT * FROM listings WHERE match('@(tag_ids) id1t')");
     * $res = mysqli_query($conn, "SELECT * FROM listings WHERE match('@(tag_ids) id1t + @status = 1 + @deleted = 0 | @featured = 1')");
     * $res = mysqli_query($conn, "SELECT count(*) FROM listings");
     * 
     * NOTE: table prefix does not work in sphinx query
     */
    private function sphinx($count, $data = array()) {
        $conn = mysqli_connect("127.0.0.1:9306", "", "", "");
        if (mysqli_connect_errno()) {
            // die("failed to connect to Sphinx: " . mysqli_connect_error());
            return false;
        } else {
            // "SELECT * FROM listings WHERE match('@(deleted) 0 + @(status) 1 + @(location) bengal + @(title,description,keywords,categories) bhattar')"  . $this->listings_model->test()
            $module = 'listings';
            $filter_search = $data['filter_search'];
            $keyword = $data['keyword'];
            $where_fields = $data['where_fields'];
            $search_only_keyword = $data['search_only_keyword'];
            $filter_search_fields = $data['filter_search_fields'];
            $sort_by = $data['sort_by'];
            $featured = $data['featured'];
            $offset = $data['offset'];
            $limit = $data['limit'];
            $extra = $data['extra'];
            $has_location = $data['has_location'];
            $params = array('sphinx' => true);
            if($count == 1) {
                $sql = "SELECT COUNT(*) AS total_rows FROM listings";
            } else {
                $sql = "SELECT id FROM listings";
            }
            $where = $this->where_only($module, $filter_search, $keyword, $where_fields, $search_only_keyword, $filter_search_fields, null, $params);
            // where get
            if(!empty($_GET)) {
                $where_get = $this->where_get($module, $params);
                if(!empty($where_get)) {
                    $where .= " + " . implode(" + ", $where_get) . " ";
                }
            }
            $sql .= $where;
            if($count == 0) {
                $sql .= $this->limit_group_order($sort_by, $featured, $offset, $limit, null, $extra, $has_location, $params);
            }
            $records = mysqli_query($conn, $sql);
            $data = array();
            if($records) {
                while ($row = mysqli_fetch_row($records)) {
                    $data[] = $row[0];
                }
            }
            $result = array(
                'rows' => $data,
                'result' => implode(',', $data)
            );
            return $result;
        }
    }
    
    /**
     * GET parameters filteration
     * @param string $module
     * @param array $params
     */
    private function where_get($module, $params = array()) {
        $sphinx = !empty($params['sphinx']) ? true : false;
        $where_get = array();
        // extra parameters or options fields
        if(!empty($_GET)) {
            $db_fields = $this->gc->get_db_fields($module);
            foreach($_GET as $ekey => $evalue) {
                switch($ekey) {
                    case 'c':
                    case 'q':
                    case 'l':
                    case 'md':
                    case 'listing_id':
                    case 'search':
                    case 'latitude':
                    case 'longitude':
                    case 'id':
                        // already used fields
                        break;
                    case 'days':
                        if($evalue == 'today') {
                            // note added_on is a unixstamp field in sphinx
                            $where_get[] = $sphinx ? 'added_on = ' . strtotime(date('Y-m-d')) : " (YEAR(created_on) = YEAR(NOW()) AND MONTH(created_on) = MONTH(NOW()) AND DAY(created_on) = DAY(NOW()))";
                        } else if($evalue == 'month') {
                            $where_get[] = $sphinx ? 'added_on BETWEEN ' . strtotime(date('Y-m-01')) . ' AND ' . strtotime(date('Y-m-d')) : " (MONTH(created_on) = MONTH(CURDATE()) AND YEAR(created_on)  =  YEAR(CURDATE()))";
                        } else if(!empty((int) $evalue)) {
                            $date = date('Y-m-d', strtotime('-' . $evalue . ' day', strtotime(date('Y-m-d'))));
                            $where_get[] = $sphinx ? 'added_on BETWEEN ' . strtotime($date) . ' AND ' . strtotime(date('Y-m-d')) : " (created_on <= NOW() AND created_on = DATE_ADD(Now(),interval - " . (int) $evalue . " day))";
                        }
                        break;
                    default:
                        if(in_array($ekey, $db_fields) && !empty($evalue)) {
                            $evalues = explode(',', $evalue);
                            $get_value = array();
                            foreach($evalues as $evalue) {
                                if($sphinx) {
                                    $get_value[] = '@(' . $ekey . ') ' . $evalue;
                                } else {
                                    $get_value[] = " FIND_IN_SET('" . escape($evalue) . "', " . $ekey. ")";
                                }
                            }
                            if($sphinx) {
                                $where_get[] = implode('|', $get_value);
                            } else {
                                $where_get[] = "(" . implode(" OR ", $get_value) . ")";
                            }
                        }
                }
            }
        }
        return $where_get;
    }
    
    /**
     * Force Indexes to fasten search queries
     * @param string $module
     * @param number $featured
     * @param number $type_location
     * @param string $sort_by
     * @return string
     */
    private function force_index($module, $featured, $type_location, $sort_by) {
        $sql = '';
        if($module == 'listings') {
            switch ($featured) {
                case 1:
                    switch($type_location) {
                        case 1:
                            $sql .= ' FORCE INDEX (idx_featured_city)';
                            break;
                        case 2:
                            $sql .= ' FORCE INDEX (idx_featured_state)';
                            break;
                        default:
                            $sql .= ' FORCE INDEX (idx_featured_basic)';
                            break;
                    }
                    break;
                case 0:
                    switch($type_location) {
                        case 1:
                            $sql .= ' FORCE INDEX (idx_city)';
                            break;
                        case 2:
                            $sql .= ' FORCE INDEX (idx_state)';
                            break;
                        default:
                            $sql .= ' FORCE INDEX (idx_basic)';
                            break;
                    }
                    break;
            }
        }
        return $sql;
    }
    
    /**
     * Add/Drop search indexes to fasten search queries on larger database
     * @param string $module
     * @param boolean $is_delete - remove indexes
     */
    public function search_indexes($module = 'listings', $is_delete = false) {
        $table_name = $this->db->dbprefix . $module;
        $indexes = array(
            'idx_featured_city' => array('deleted', 'status', 'featured', 'city_id', 'package_priority', 'created_on'),
            'idx_featured_state' => array('deleted', 'status', 'featured', 'state_id', 'package_priority', 'created_on'),
            'idx_featured_basic' => array('deleted', 'status', 'featured', 'package_priority', 'created_on'),
            'idx_city' => array('deleted', 'status', 'city_id', 'package_priority', 'created_on'),
            'idx_state' => array('deleted', 'status', 'state_id', 'package_priority', 'created_on'),
            'idx_basic' => array('deleted', 'status', 'package_priority', 'created_on'),
        );
        foreach($indexes as $index_name => $index_columns) {
            $check_index = "SHOW KEYS FROM " . $table_name . " WHERE Key_name='" . $index_name . "'";
            $records = $this->db->query($check_index)->result();
            if($records) {
                if($is_delete) {
                    // drop index
                    $this->db->query("DROP INDEX " . $index_name . " ON " . $table_name);
                }
            } else {
                if(!$is_delete) {
                    // add index
                    $index_sql = "ALTER TABLE " . $table_name . " ADD INDEX `". $index_name . "` (" . implode(', ', $index_columns) . ")";
                    $this->db->query ($index_sql);
                }
            }
        }
    }
    
    /**
     * Keyword/Category/Tag search query
     *
     * @param int $category_id
     * @param int $tag_id
     * @param string $keyword
     * @return string
     */
    private function keyword_category_tag($module, $category_id, $tag_id, $keyword, $search_only_keyword, $fields, $filter_search, $where, $featured, $has_location, $location, $location_id, $postcode, $type_location, $locality_id)
    {
        $module_id = gc_module_id($module);
        $prefix = 'M.';
        $sql = " SELECT " . $prefix . "* FROM " . $this->db->dbprefix . $module . " M";
		$sql .= " FIND_IN_SET(" . $category_id . ", categories_set)";
        /*
         * if($category_id != 0) {
         * // cached query, no matters if the cache is enabled or not
         * $categories_level = $this->gc->category_level_search($category_id);
         * // Filter results based on listing categories
         * $sql .= " INNER JOIN " . $this->db->dbprefix . "categories_relation CR ON CR.row_id = M.id AND CR.module_id = " . $module_id;
         * if($categories_level['subsub_id'] == $category_id) {
         * $sql .= " AND CR.subsub_id = " . $categories_level['subsub_id'];
         * } else if($categories_level['sub_id'] == $category_id) {
         * $sql .= " AND CR.sub_id = " . $categories_level['sub_id'];
         * } else if($categories_level['parent_id'] == $category_id) {
         * $sql .= " AND CR.parent_id = " . $categories_level['parent_id'];
         * }
         * }
         *
         * // Filter results based on listing tags
         * if($tag_id != 0) {
         * $sql .= " INNER JOIN " . $this->db->dbprefix . "tag_relations TR ON TR.row_id = M.id AND TR.module_id = " . $module_id . " AND TR.tag_id = " . $tag_id;
         * }
         */
        
        // keyword search
        if ($tag_id == 0) { // if the searched keyword is a tag, do not perform the Listing's Title and Description lookup
            $sql .= $this->where_only($filter_search, $where, $featured, $has_location, $location, $location_id, $postcode, $type_location, $locality_id);
            $sql .= $this->keyword_filter($module, $keyword, $search_only_keyword, $fields);
        }
        // $sql .= ")";
        
        // Tags table search
        // e.g. query title LIKE '%KEYWORD%' OR description LIKE '%KEYWORD%' OR keywords LIKE '%KEYWORD%'
        // Keywords cannot be searched this way as they can be activated or deactivated by the Admin at anytime
        /*
         * if(!empty($keyword) && $tag_id == 0) {
         * $sql .= " UNION ";
         * $sql .= $this->tags_search($module, $keyword);
         * }
         */
        return $sql;
    }
    
    /**
     * Keyword specific searchh and filteration
     *
     * @param string $keyword
     * @param string $search_only_keyword
     * @param array $fields
     * @return string
     */
    private function keyword_filter($module, $keyword, $search_only_keyword, $fields, $params = array()) {
        // perform INNER JOIN of the metas table only if the $metas variable is TRUE
        $metas = FALSE;
        $prefix = 'M.';
        $sql = '';
        $sphinx = !empty($params['sphinx']) ? true : false;
        if (! empty($keyword)) {
            if($sphinx) {
                $sql = " + @(title,description,keywords,categories,location) " . $keyword;
            } else {
                $sql = " AND (";
                if(settings_item('site.categories_modules') == 1) {
                    if (settings_item('site.full_text_search') == 1) {
                        $sql .= " MATCH (" . $prefix . "metas) AGAINST ('\"+" . $keyword . "*\"' IN BOOLEAN MODE)";
                    } else {
                        $sql .= " " . $prefix . "metas LIKE '%" . $keyword . "%'";
                    }
                } else {
                    if (settings_item('site.full_text_search') == 1) {
                        $sql .= " MATCH (" . $prefix . "title," . $prefix . "description," . $prefix . "keywords," . $prefix . "categories) AGAINST ('\"+" . $keyword . "*\"' IN BOOLEAN MODE)";
                    } else {
                        $sql .= " " . $prefix . "title LIKE '%" . $keyword . "%' OR " . $prefix . "description LIKE '%" . $keyword . "%' OR " . $prefix . "keywords LIKE '%" . $keyword . "%' OR " . $prefix . "categories LIKE '%" . $keyword . "%'";
                    }
                }
            }
            /*
             * if(settings_item('site.full_text_search') == 1) {
             * $sql .= " MATCH (" . $prefix . "title," . $prefix . "description) AGAINST ('\"+". $keyword ."*\"' IN BOOLEAN MODE)";
             * } else {
             * $sql .= " " . $prefix . "title LIKE '%" . $keyword . "%' OR " . $prefix . "description LIKE '%" . $keyword. "%'";
             * }
             */
            
            // Search only fields lookup in the module table using OR operator
            // Additional Core field search enabled by the Admin
            $search_only_fields = $fields['search'];
            if (! empty($search_only_fields['core'])) {
                // $search_only_keyword
                $core_fields = $search_only_fields['core'];
                foreach ($core_fields as $core_field) {
                    if($sphinx) {
                        $search_core_sql[] = "@(" . $core_field . ") " . $search_only_keyword;
                    } else {
                        $search_core_sql[] = $prefix . $core_field . " LIKE '%" . $search_only_keyword . "%'";
                    }
                }
                if (isset($search_core_sql)) {
                    if($sphinx) {
                        $sql .= " |" . implode(' |', $search_core_sql);
                    } else {
                        $sql .= " OR (" . implode(' OR ', $search_core_sql) . ")";
                    }
                }
            }
            // Additional Custom field search enabled by the Admin
            if (! empty($search_only_fields['custom'])) {
                $metas = TRUE;
                $custom_fields = $search_only_fields['custom'];
                foreach ($custom_fields as $custom_field) {
                    if($sphinx) {
                        // metas are stored in the listings table in json format
                        $search_custom_sql[] = '@(metas) "' . $custom_field . '":"' . $search_only_keyword . '"';
                    } else {
                        $search_custom_sql[] = "(MT.meta_key = '" . $custom_field . "' AND MT.meta_value LIKE '%" . $search_only_keyword . "%')";
                    }
                }
                if (isset($search_custom_sql)) {
                    if($sphinx) {
                        $sql .= " |" . implode(' |', $search_custom_sql);
                    } else {
                        $sql .= " OR (" . implode(' OR ', $search_custom_sql) . ")";
                    }
                }
            }
            if(!$sphinx) {
                // Search only fields must also be enclosed in the same parantheses for correct results
                $sql .= ")";
            }
        }
        /*if ($metas) {
         // perform LEFT JOIN instead of INNER as filter fields are added in the WHERE clause using AND operation
         $sql = " LEFT JOIN " . $this->db->dbprefix . "metas MT ON MT.row_id = " . $prefix . "id AND MT.module_id = " . gc_module_id($module) . $sql;
         }*/
        return $sql;
    }
    
    /**
     * Filter Search
     * Adding this to keyword filter was resulting in incorrect results when category_id and tag_id is 0
     * Add the filter search query after all JOIN operations for correct results
     *
     * @param array $fields
     * @return array
     */
    private function filter_search($fields)
    {
        $join_metas = FALSE;
        $prefix = "M.";
        $sql = '';
        // All filter fields value will be retrieved using POST e.g. search['contact_person']
        // it might happen that the user not typed a keyword in the main search input but filled the filter field
        // in such cases perform the search operation as normal
        $filter_keywords = (isset($_GET['f']) ? $_GET['f'] : array());
        // restore filter fields value on pagination or page refresh until the user manually remove them
        $filter_search = array();
        // Filter fields also lookup in the module table, but using AND operator
        // They also have their own keyword
        if (! empty($filter_keywords)) {
            $filter_fields = $fields['filter'];
            if (! empty($filter_fields['core'])) {
                // $search_only_keyword
                $filter_core_fields = $filter_fields['core'];
                foreach ($filter_core_fields as $filter_core_field) {
                    // check whether the filter input field is filled or not
                    if (! empty($filter_keywords[$filter_core_field])) { // if filled
                        // sanitize the keyword for security
                        $filter_keyword = escape(trim($filter_keywords[$filter_core_field]));
                        $filter_search[$filter_core_field] = $filter_keyword;
                        $filter_sql[] = $prefix . $filter_core_field . " LIKE '%" . $filter_keyword . "%'";
                    }
                }
                if (isset($filter_sql)) {
                    $sql .= " AND (" . implode(' AND ', $filter_sql) . ")";
                }
            }
            // Filter Additional Custom field enabled by the Admin
            if (! empty($filter_fields['custom'])) {
                $join_metas = TRUE; // to perform metas table LEFT JOIN
                $filter_custom_fields = $filter_fields['custom'];
                foreach ($filter_custom_fields as $filter_custom_field) {
                    // check whether the filter input field is filled or not
                    if (! empty($filter_keywords[$filter_custom_field])) { // if filled
                        // sanitize the keyword for security
                        $filter_keyword = escape($filter_keywords[$filter_custom_field]);
                        $filter_search[$filter_custom_field] = $filter_keyword;
                        $filter_custom_sql[] = "(MT.meta_key = '" . $filter_custom_field . "' AND MT.meta_value LIKE '%" . $filter_keyword . "%')";
                    }
                }
                if (isset($filter_custom_sql)) {
                    $sql .= " AND (" . implode(' AND ', $filter_custom_sql) . ")";
                }
            }
            if (! empty($filter_search)) {
                $this->session->set_userdata('filter_search', $filter_search);
            }
        }
        return array(
            'join_metas' => $join_metas,
            'sql' => $sql
        );
    }
    
    /**
     * Prepare category search SQL
     *
     * @param string $keyword
     * @param string $module
     * @return string
     */
    private function categories_search($module, $keyword)
    {
        $module_id = gc_module_id($module);
        $sql = " (SELECT CR.row_id FROM " . $this->db->dbprefix . "categories C LEFT JOIN " . $this->db->dbprefix . "categories_relation CR ON CR.parent_id = C.id AND CR.module_id = " . $module_id . " WHERE C.active =1";
        if (settings_item('site.full_text_search') == 1) {
            $sql .= " AND ( MATCH ( C.name ) AGAINST ( '\"+" . $keyword . "*\"' IN BOOLEAN MODE ))";
        } else {
            $sql .= " AND C.name LIKE '%" . $keyword . "%'";
        }
        $sql .= ") UNION" . PHP_EOL;
        $sql .= " (SELECT CR.row_id FROM " . $this->db->dbprefix . "categories C LEFT JOIN " . $this->db->dbprefix . "categories_relation CR ON CR.sub_id = C.id AND CR.module_id = " . $module_id . " WHERE C.active =1";
        if (settings_item('site.full_text_search') == 1) {
            $sql .= " AND ( MATCH ( C.name ) AGAINST ( '\"+" . $keyword . "*\"' IN BOOLEAN MODE ))";
        } else {
            $sql .= " AND C.name LIKE '%" . $keyword . "%'";
        }
        $sql .= ") UNION" . PHP_EOL;
        $sql .= " (SELECT CR.row_id FROM " . $this->db->dbprefix . "categories C LEFT JOIN " . $this->db->dbprefix . "categories_relation CR ON CR.subsub_id = C.id AND CR.module_id = " . $module_id . " WHERE C.active =1";
        if (settings_item('site.full_text_search') == 1) {
            $sql .= " AND ( MATCH ( C.name ) AGAINST ( '\"+" . $keyword . "*\"' IN BOOLEAN MODE ))";
        } else {
            $sql .= " AND C.name LIKE '%" . $keyword . "%'";
        }
        $sql .= ")";
        return $sql;
    }
    
    /**
     * Tag table search
     *
     * @param string $module
     * @param string $keyword
     * @return string
     */
    private function tags_search($module, $keyword)
    {
        $sql = "(SELECT TR.row_id FROM " . $this->db->dbprefix . "tags T";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "tag_relations TR ON TR.tag_id = T.id AND TR.module_id = " . gc_module_id($module);
        $sql .= " WHERE T.active = 1";
        if (settings_item('site.full_text_search') == 1) {
            $sql .= " AND ( MATCH ( T.name ) AGAINST ( '\"+" . $keyword . "*\"' IN BOOLEAN MODE ))";
        } else {
            $sql .= " AND T.name LIKE '%" . $keyword . "%'";
        }
        $sql .= ")";
        return $sql;
    }
    
    /**
     * Display featured listings
     *
     * @param string $ids
     *            - pass them using explode - for advertisement module
     * @param int $tag_id
     */
    public function ads_featured($module, $ids, $tag_id, $user_id = 0)
    {
        $module_id = gc_module_id($module);
        $select_fields = $this->select_fields($module, 0, 0, $user_id, TRUE);
        $select_fields_data = str_replace('RP.', 'AP.', $select_fields['select']);
        $select_fields_data = str_replace('AP.display_fields', 'RP.display_fields', $select_fields_data);
        $select_fields_data = str_replace('AP.loggedin_display_fields', 'RP.loggedin_display_fields', $select_fields_data);
        $sql = $select_fields_data;
        $sql .= " FROM " . $this->db->dbprefix . $module . " MD";
        if (! empty($select_fields['join'])) {
            $sql .= $select_fields['join'];
        }
        if ($module != 'listings') {
            $sql .= " LEFT JOIN " . $this->db->dbprefix . "listings ML ON ML.id = MD.listing_id" . PHP_EOL;
            // $sql .= " LEFT JOIN " . $this->db->dbprefix . "cities LCT ON LCT.id = ML.city_id" . PHP_EOL; // listing city
        }
        // Ads and ad_packages tables are JOINED to have color schemes in the Featured block
        $sql .= " INNER JOIN " . $this->db->dbprefix . "ads A ON A.row_id = MD.id AND A.module_id = " . $module_id;
        $sql .= " INNER JOIN " . $this->db->dbprefix . "ad_packages AP ON AP.id = A.package_id" . PHP_EOL;
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "packages RP ON RP.id = MD.package_id" . PHP_EOL;
        // $sql .= " LEFT JOIN " . $this->db->dbprefix . "ratings R ON R.row_id = MD.id AND R.module_id = " . $module_id;
        // $sql .= " LEFT JOIN " . $this->db->dbprefix . "tag_relations TR ON MD.id = TR.row_id AND TR.module_id = " . $module_id;
        // $sql .= $this->translations->common_joins('tags') . " T ON TR.tag_id = T.id AND T.active = 1";
        if ($module == 'listings' || gc_module_settings($module, 'locations') == 1) {
            // $sql .= " INNER JOIN " . $this->db->dbprefix . "countries CO ON CO.iso = MD.country_iso";
            $country = $this->session->userdata('search_country');
            if (settings_item('lst.country_filter') == 1 && ! empty($country)) { // The country will be null in case of Member Area
                $sql .= " AND MD.country_iso = '" . $country . "'";
            }
            // $sql .= " INNER JOIN " . $this->db->dbprefix . "states S ON S.id = MD.state_id" . PHP_EOL;
            // $sql .= " INNER JOIN " . $this->db->dbprefix . "cities CT ON CT.id = MD.city_id" . PHP_EOL;
        }
        // LEFT JOIN Users Table
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "users U ON U.id = MD.user_id" . PHP_EOL;
        // $sql .= " LEFT JOIN " . $this->db->dbprefix . "favorites F ON F.row_id = MD.id AND F.module_id = " . $module_id;
        // $sql .= " LEFT JOIN " . $this->db->dbprefix . "localities LOC ON MD.locality_id = LOC.id" . PHP_EOL;
        // $sql .= " LEFT JOIN " . $this->db->dbprefix . "images IMG ON IMG.row_id = MD.id AND IMG.module_id = " . $module_id;
        $sql .= " WHERE MD.deleted = 0 AND MD.status = 1";
        $sql .= " AND MD.id IN ($ids)" . PHP_EOL;
        $sql .= " GROUP BY MD.id";
        $result = $this->db->query($sql)->result();
        return $result;
    }
    
    /**
     * Common fields for view page results
     *
     * @param string $module
     *            - name of the module
     * @param int $count
     * @param int $user_id
     *            - For Favorites Count
     * @param boolean $ad
     *            - Display Ad specific color scheme
     * @return string
     */
    private function select_fields($module, $count, $extra = array(), $ad = FALSE) {
        if ($count == 0) {
            $table_fields = array();
            if ($module == 'listings') {
                $table_fields = array(
                    'id',
                    'title',
                    'slug',
                    'description'
                );
            }
            $query = $this->translations->build_query($module, $table_fields, array(), 'MD.');
        }
        if ($module == 'listings') {
            if ($count == 0) {
                $sql = $query['select'] . ", MD.contact_person, MD.address, MD.user_id, MD.ratings AS average_rating, MD.rating_users AS total_users, MD.logo, MD.latitude, MD.longitude," . PHP_EOL;
                $sql .= " MD.postcode, MD.phone_number, MD.mobile_number, MD.show_address, MD.show_phone, MD.show_email, RP.display_fields, RP.loggedin_display_fields," . PHP_EOL;
                $sql .= " MD.email, MD.website, MD.priority, MD.featured, MD.created_on, RP.price AS package_price," . PHP_EOL;
                $sql .= " MD.country, MD.state, MD.city, MD.city_id, MD.locality_id, MD.locality, MD.hits, MD.cover, MD.package_id, MD.category_id, MD.metas, MD.categories_set,";
                $sql .= " BH.open_time, BH.close_time, "; // Open Now
                $sql .= "MD.keywords, MD.tags_set AS tags, U.slug AS user_slug";
            } else {
                // $sql = "SELECT COUNT(DISTINCT(MD.id)) as total_count";
                $sql = "SELECT COUNT(*) as total_count"; // DISTINCT(MD.id)
            }
            $join = " LEFT JOIN " . $this->db->dbprefix . "business_hours BH ON BH.listing_id = MD.id AND BH.day_of_week = " . date('N');
            $query['join'] = !empty($query['join']) ? $query['join'] . ' ' . $join : $join;
        } else {
            if ($count == 0) {
                $sql = $query['select'] . ", MD.cover AS image, ML.logo AS listing_logo, ML.title AS listing_title, ML.slug AS listing_slug, ML.city AS listing_city, MD.ratings AS average_rating, MD.rating_users AS total_users," . PHP_EOL;
                $sql .= " MD.categories_set, MD.keywords, MD.tags_set AS tags, U.slug AS user_slug";
            } else {
                $sql = "SELECT COUNT(MD.id) AS total_count";
            }
        }
        // Display Ad specific color scheme
        if ($count == 0) {
            if (!empty($extra['user_id'])) {
                $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . "favorites F WHERE F.module_id = " . gc_module_id($module) . " AND F.row_id = MD.id AND F.user_id = " . $extra['user_id'] . ") AS isFavorite";
            }
            $sql .= ", MD.images, U.id AS author_id, U.display_name AS author_name, U.slug AS author_slug, U.profile_image AS author_image, U.city AS author_city";
            // making query slow, saved_categories added to the modules table iteself
            // $sql .= ", (SELECT GROUP_CONCAT(CONCAT(id, '!~!', name, '!~!', COALESCE(slug,''), '!~!', COALESCE(icon,'')) SEPARATOR '~~~') FROM " . $this->db->dbprefix . "categories WHERE FIND_IN_SET(id, MD.categories_set) AND active = 1) AS saved_categories";
            if ($ad) {
                $sql .= ", AP.color_scheme, AP.border_color, AP.trusted_seal";
            } else {
                $sql .= ", RP.color_scheme, RP.border_color, RP.trusted_seal";
            }
        }
        return array(
            'select' => $sql,
            'join' => (isset($query) ? $query['join'] : array())
        );
    }
    
    /**
     * Where Filter Only
     * @param string $module - Name of the module
     * @param array $filter_search
     * @param
     * @param string $where
     * @param number $featured
     * @return string
     */
    private function where_only($module, $filter_search, $keyword, $where_fields, $search_only_keyword, $filter_search_fields, $table_prefix = 'MD.', $params = array()) {
        $has_location = ($module == 'listings' || gc_module_settings($module, 'locations') == 1 ? TRUE : FALSE);
        $sphinx = !empty($params['sphinx']) ? true : false;
        if($sphinx) {
            $sql = " WHERE match('@(deleted) 0 + @(status) 1";
        } else {
            $sql = " WHERE " . $table_prefix . "deleted = 0 AND " . $table_prefix . "status = 1";
        }
        if (! empty($filter_search['sql'])) {
            $sql .= $filter_search['sql'];
        }
        foreach($where_fields as $field_name => $value) {
            if(!empty($value)) {
                switch($field_name) {
                    case 'category_id': // Note: the string length must be greater than 4 for FULL TEXT search
                        $field_value = 'id' . $value . 'c';
                        if($sphinx) {
                            $sql .= ' + @(category_ids) ' . $field_value;
                        } else {
                            if (settings_item('site.full_text_search') == 1) {
                                $sql .= " AND MATCH (" . $table_prefix . "category_ids) AGAINST ('\"id" . $value . "c\"' IN BOOLEAN MODE)";
                            } else {
                                $sql .= " AND FIND_IN_SET(" . $value . ", categories_set)";
                            }
                        }                        
                        break;
                    case 'tag_id': // Note: the string length must be greater than 4 for FULL TEXT search
                        $field_value = 'id' . $value . 't';
                        if($sphinx) {
                            $sql .= ' + @(tag_ids) ' . $field_value;
                        } else {
                            if (settings_item('site.full_text_search') == 1) {
                                $sql .= " AND MATCH (" . $table_prefix . "tag_ids) AGAINST ('\"id" . $value . "t\"' IN BOOLEAN MODE)";
                            } else {
                                $sql .= " AND FIND_IN_SET(" . $value . ", tags_set)";
                            }
                        } 
                        // $sql .= $sphinx ? ' + @(tags_set) ' . $value : " AND FIND_IN_SET(" . $value . ", tags_set)";
                        break;
                    case 'location':
                        if($has_location) {
                            if($this->uri->segment(1) != gc_uri_string('location')) {
                                if($sphinx) {
                                    $sql .= ' + @(location) ' . $value;
                                } else {
                                    $value = mysqli_real_escape_string($this->gc->get_mysqli(), $value);
                                    if (settings_item('site.full_text_search') == 1) {
                                        if(strlen($value) > 3) {
                                            $sql .= " AND MATCH (" . $table_prefix . "location) AGAINST ('\"+" . $value . "*\"' IN BOOLEAN MODE)";
                                        }
                                    } else {
                                        $sql .= " AND " . $table_prefix . "location LIKE '%" . $value . "%'";
                                    }
                                }
                            }
                        }
                        break;
                    case 'where':
                        $sql .= " AND " . (strpos($value, 'BH.') === false ? $table_prefix . $value : $value);
                        break;
                    case 'country_iso':
                    case 'postcode':
                        if($has_location) {
                            if($sphinx) {
                                $sql .= ' + @(' . $field_name . ') ' . $value;
                            } else {
                                $sql .= " AND " . $table_prefix . $field_name . " = '" . $value . "'";
                            }
                        }
                        break;
                    case 'state_id':
                    case 'city_id':
                    case 'locality_id':
                        if($has_location) {
                            if($sphinx) {
                                $sql .= ' + @(' . $field_name . ') ' . $value;
                            } else {
                                $sql .= " AND " . $table_prefix . $field_name . " = " . $value;
                            }
                        }
                        break;
                    default:
                        if($sphinx) {
                            $sql .= ' + @(' . $field_name . ') ' . $value;
                        } else {
                            $sql .= " AND " . $table_prefix . $field_name . " = " . $value;
                        }
                }
            }
        }
        if (!empty($keyword) && empty($where_fields['tag_id'])) {
            $keyword_filter = $this->keyword_filter($module, $keyword, $search_only_keyword, $filter_search_fields, $params);
            $sql .= $keyword_filter;
        }
        if($sphinx) {
            $sql .= "')";
        }
        return $sql;
    }
    
    /**
     * Limit records before additional join
     *
     * @param string $sort_by
     * @param number $featured
     * @param number $offset
     * @param number $limit
     * @param string $table_prefix
     *            - Prefix of the module table
     * @return string
     */
    private function limit_group_order($sort_by, $featured, $offset, $limit, $table_prefix = 'M', $extra = array(), $has_location = false) {
        $sql = '';
        if(!empty($table_prefix)) {
            $sql = PHP_EOL . " GROUP BY " . $table_prefix . ".id" . PHP_EOL; // Not required
        }
        $sql .= $this->_sort_result($sort_by, $table_prefix, $extra, $has_location);
        // if ($featured == 0) { // the mobile app has a featured tab/segment, that is displaying duplicate listings
        if ($limit != 0) {
            $sql .= " LIMIT " . $offset . ", " . $limit;
        } else {
            $sql .= " LIMIT $limit";
        }
        // }
        return $sql;
    }
    
    /**
     * Sort  By Filter
     * @param string $table_prefix
     */
    private function _sort_result($sort_by, $table_prefix = 'M', $extra = array(), $has_location = false) {
        if(!empty($table_prefix)) {
            $table_prefix = $table_prefix . '.';
        }
        $tbl_prefix = $table_prefix;
        switch($sort_by) {
            case 'top_rated':
                $field = 'ratings DESC';
                break;
            case 'alphabetically':
                $field = 'title ASC';
                break;
            case 'popular':
                $field = 'hits DESC';
                break;
            case 'featured':
                $field = 'featured DESC';
                break;
            case 'last_viewed':
                $tbl_prefix = 'RV.';
                $field = 'last_viewed DESC';
                break;
            default:
                $field = 'id DESC';
        }
        if($sort_by == 'rand') {
            $order_by = 'RAND()'; // slowest
            //$sql .= " AND id >= (SELECT FLOOR( MAX(id) * RAND()) FROM ". $this->dbprefix . $module .")"; // display rows on random basis
        } else {
            $order_by = $tbl_prefix . $field . ", " . $table_prefix . "package_priority DESC";
            // find nearest based on user's latitude and longitude
            if(!empty($extra['latitude']) && !empty($extra['longitude']) && $has_location) {
                // $order_by = $table_prefix . "user_distance, " . $order_by;
            }
        }
        return " ORDER BY " . $order_by;
    }
    
    /**
     * Activate this method if you have Million of listings
     * A new table listing_search must be created in database with appropriate indexes
     * This method of search has few drawbacks which must be worked on
     * NOTE: NOT TESTED WITH DEFAULT AND OTHER SEARCH BLOCKS
     */
    public function listing_search() {}
    
    /**
     * Common Method for Mobile and Web App to display on type ajax result
     *
     * @param string $module
     *            - Name of the module
     * @param string $keyword
     * @param string $country_iso
     * @param string $search_location
     * @param int $locality_id
     * @param int $category_id
     * @param string $mobile
     * @return multitype:multitype:unknown string
     */
    public function ajax_results($module, $keyword, $country_iso, $search_location, $locality_id, $category_id, $mobile = FALSE)
    {
        $this->load->library('slug');
        $result = array();
        if (settings_item('site.enable_cache') == 1) {
            $cacheID = str_replace('-', '_', $this->slug->create_slug($module . '_' . $keyword)) . '_' . $country_iso; // Cache giving errors when invalid url with spaces and special characters
            if (! $records = $this->z_cache->load($cacheID)) {
                $records = $this->ajax_search($module, $keyword, $country_iso, $search_location, $locality_id, $category_id);
                $this->z_cache->save($records, $cacheID);
            }
        } else {
            $records = $this->ajax_search($module, $keyword, $country_iso, $search_location, $locality_id, $category_id);
        }
        if ($records) {
            foreach ($records as $record) {
                $data = array(
                    'id' => $record['id'],
                    'title' => $record['title'],
                    'type' => $record['type']
                );
                if (! $mobile) {
                    switch ($record['type']) {
                        case 'M':
                            // City name is concatenated in the slug to avoid multiple changes in the ajax_search method
                            $slug = explode('/', $record['slug'] ?? '');
                            $url = gc_url_format($module, $slug[0], $record['id'], (isset($slug[1]) ? $slug[1] : NULL));
                            break;
                        default:
                            $url = ($module == 'listings' ? gc_uri_string('search') : gc_uri_string($module));
                            break;
                    }
                    $data['url'] = $url;
                }
                $result[] = $data;
            }
        }
        return $result;
    }
    
    /**
     * Get rows as user type keyword using AJAX
     *
     * @param string $country_iso
     *            - filter result based on country
     * @param string $keyword
     * @param number $locality
     * @return object listings
     */
    public function ajax_search($module, $keyword, $country_iso, $search_city, $locality_id = 0, $category_id = 0, $postcode = 0, $mobile = false)
    {
        $module_id = gc_module_id($module);
        if (! $mobile && settings_item('adv.search_blocks') == 1) {
            $search_city = $this->session->userdata('search_city');
        }
        if (settings_item('site.full_text_search') == 1) {
            $keyword = str_replace(" ", " +", $keyword);
            $search_city = str_replace(" ", " +", $search_city);
        }
        
        if ($locality_id == - 1) {
            $locality_id = 0;
        }
        
        if ($category_id == - 1) {
            $category_id = 0;
        }
        
        $hasLocation = ($module == 'listings' || gc_module_settings($module, 'locations') == 1 ? TRUE : FALSE);
        $limit = 20;
        /*
         * M - Module
         * PC - Parent Category
         * SC - Sub Category
         * SSC - Sub Sub Category
         * T - Tag
         * CASE WHEN id <> 0 THEN 'M' END AS type
         */
        $sql = "SELECT M.id, M.slug, M.name as title, M.type FROM";
        /**
         * **************************************************************
         */
        /* LISTING SEARCH */
        /**
         * **************************************************************
         */
        $sql .= " ((SELECT MD.id,";
        if ($hasLocation) { // pass city name as a parameter for city enabled modules
            $sql .= " CONCAT(MD.slug, '/', MD.city) AS slug,";
        } else {
            $sql .= " MD.slug,";
        }
        $sql .= " MD.title as name, 'M' AS type FROM " . $this->db->dbprefix . $module . " MD";
        $sql .= " WHERE MD.deleted = 0 AND MD.status = 1";
        
        $table_prefix = 'MD.';
        if ($hasLocation) {
            // Apply country filter
            if (settings_item('lst.country_filter') == 1) {
                $sql .= " AND MD.country_iso = '" . $country_iso . "'";
            }
            if (! empty($postcode)) {
                $sql .= " AND MD.postcode = '" . $postcode . "'";
            } else if (! empty($search_city)) {
                if (is_numeric($search_city)) {
                    $sql .= " AND MD.city_id = " . $search_city;
                } else {
                    if (settings_item('site.full_text_search') == 1) {
                        $sql .= " AND MATCH (" . $table_prefix . "address," . $table_prefix . "postcode," . $table_prefix . "locality," . $table_prefix . "city," . $table_prefix . "state) AGAINST ('\"+" . $search_city . "*\"' IN BOOLEAN MODE)";
                    } else {
                        $sql .= " AND (" . $table_prefix . "address LIKE '%" . $search_city . "%' OR " . $table_prefix . "postcode LIKE '%" . $search_city . "%' OR " . $table_prefix . "locality LIKE '%" . $search_city . "%' OR " . $table_prefix . "city LIKE '%" . $search_city . "%' OR " . $table_prefix . "state LIKE '%" . $search_city . "%')";
                    }
                }
            }
        }
        if (! empty($category_id)) {
            // Note: the string length must be greater than 4 for FULL TEXT search
			$sql .= " AND FIND_IN_SET(" . $category_id . ", categories_set)";
        }
        if ($locality_id != 0) {
            $sql .= " AND MD.locality_id = " . $locality_id;
        }
        
        if (settings_item('site.full_text_search') == 1) {
            $sql .= " AND MATCH (" . $table_prefix . "title," . $table_prefix . "description," . $table_prefix . "keywords," . $table_prefix . "categories) AGAINST ('\"+" . $keyword . "*\"' IN BOOLEAN MODE)";
        } else {
            $sql .= " AND (" . $table_prefix . "title LIKE '" . $keyword . "%' OR " . $table_prefix . "description LIKE '%" . $keyword . "%' OR " . $table_prefix . "keywords LIKE '%" . $keyword . "%' OR " . $table_prefix . "categories LIKE '%" . $keyword . "%')";
        }
        $sql .= " LIMIT " . $limit;
        $sql .= " ) UNION";
        /**
         * **************************************************************
         */
        /* CATEGORY SEARCH - NO NEED TO FILTER BY CITIES */
        /**
         * **************************************************************
         */
        // Display all active categories
        $sql .= "(SELECT C.id, C.slug, C.name, 'C' AS type FROM " . $this->db->dbprefix . "categories C";
        $sql .= " INNER JOIN " . $this->db->dbprefix . "module_relations MR ON MR.row_id = C.id AND MR.module_id = " . gc_module_id('categories') . " AND MR.toggle_module_id = " . $module_id;
        $sql .= " WHERE C.active = 1";
        
        if (settings_item('site.full_text_search') == 1) {
            $sql .= " AND ( MATCH ( C.name ) AGAINST ( '\"+" . $keyword . "*\"' IN BOOLEAN MODE ))";
        } else {
            $sql .= " AND C.name LIKE '%" . $keyword . "%'";
        }
        
        $sql .= " LIMIT " . $limit;
        $sql .= " ) UNION";
        /**
         * **************************************************************
         */
        /* TAG SEARCH - NO NEED TO FILTER BY CITIES */
        /**
         * **************************************************************
         */
        // Display all active tags
        $sql .= " (SELECT T.id, T.slug, T.name, 'T' AS type FROM " . $this->db->dbprefix . "tags T ";
        $sql .= " INNER JOIN " . $this->db->dbprefix . "module_relations MR ON MR.row_id = T.id AND MR.module_id = " . gc_module_id('tags') . " AND MR.toggle_module_id = " . $module_id;
        $sql .= " WHERE T.active = 1";
        if (settings_item('site.full_text_search') == 1) {
            $sql .= " AND ( MATCH ( T.name ) AGAINST ( '\"+" . $keyword . "*\"' IN BOOLEAN MODE ))";
        } else {
            $sql .= " AND T.name LIKE '%" . $keyword . "%'";
        }
        $sql .= " LIMIT " . $limit;
        $sql .= " )) M ORDER BY M.name ASC LIMIT " . $limit;
        $query = $this->db->query($sql);
        $result = $query->result_array();
        return $result;
    }
    
    /**
     * Display listings to the Admin alongwith module count
     * so that admin can come to know which of the module rows are not activated
     *
     * @param number $offset
     * @param number $limit
     * @param string $where
     * @param string $order_by
     * @return array|object
     */
    public function get_admin_listings($count, $offset, $limit, $where, $order_by, $category_filter = array()) {
        if ($count == 0) {
            $modules = gc_modules_only();
            $modules[] = 'images';
            $modules[] = 'videos';
            $modules[] = 'branches';
            $select = array();
            $join = array();
            foreach ($modules as $module) {
                $select[] = " COUNT(DISTINCT $module.id) as inactive_" . $module . PHP_EOL;
                if ($module == 'images' || $module == 'videos') {
                    $join[] = " LEFT JOIN " . $this->db->dbprefix . $module . " AS $module ON $module.row_id = L.id AND $module.module_id = " . gc_module_id('listings') . " AND $module.status = 0 AND $module.deleted = 0" . ($module == 'images' ? " AND $module.type_id = 0" : '') . PHP_EOL;
                } else {
                    $join[] = " LEFT JOIN " . $this->db->dbprefix . $module . " AS $module ON $module.listing_id = L.id AND $module.status = 0 AND $module.deleted = 0" . PHP_EOL;
                }
            }
            $sql = "SELECT L.id, L.package_id, pkg.title as package, L.title, L.slug, IMG.url AS logo_url, L.email, L.hits, L.created_on, L.created_ip, L.modified_on, L.modified_ip, L.status, ct.name AS city, L.user_id, U.display_name AS user, L.phone_number, L.mobile_number, L.percent_completion," . PHP_EOL;
            $sql .= implode(',', $select);            
            $table_prefix = 'LS.';
            $sql .= " FROM (SELECT * FROM " . $this->db->dbprefix . "listings M" . PHP_EOL;
            // Category Filter
            if (! empty($category_filter)) {
                $sql .= " INNER JOIN " . $this->db->dbprefix . "categories_relation CR ON CR.row_id = M.id AND CR.module_id = " . $category_filter['module_id'] . " AND CR." . $category_filter['level'] . " = " . $category_filter['category_id'] . PHP_EOL;
            }
            if (! empty($where) && (strpos($where, 'SUBSTRING') !== false)) {
                $sql .= " WHERE " . $where;
            } else if (! empty($where)) {
                $sql .= " WHERE " . $where;
            }
            $sql .= " ORDER BY M." . $order_by . " LIMIT " . $offset . ", " . $limit . ") L" . PHP_EOL;
            $sql .= " LEFT JOIN `" . $this->db->dbprefix . "packages` as pkg ON `pkg`.`id` = L.package_id" . PHP_EOL;
            $sql .= " LEFT JOIN " . $this->db->dbprefix . "images as IMG ON IMG.row_id = L.id AND IMG.module_id = " . gc_module_id('listings') . " AND IMG.type_id = 2" . PHP_EOL;
            $sql .= " LEFT JOIN " . $this->db->dbprefix . "users as U ON U.id = L.user_id" . PHP_EOL;
            $sql .= " LEFT JOIN " . $this->db->dbprefix . "cities as ct ON ct.id = L.city_id" . PHP_EOL;
            $sql .= implode(' ', $join);
            $sql .= " GROUP BY L.id ORDER BY L." . $order_by;
            $query = $this->db->query($sql);
            $result = $query->result_array();
        } else {
            $sql = "SELECT COUNT(DISTINCT(M.id)) AS total_count";
            $sql .= " FROM " . $this->db->dbprefix . "listings M";
            // Category Filter
            if (! empty($category_filter)) {
                $sql .= " INNER JOIN " . $this->db->dbprefix . "categories_relation CR ON CR.row_id = M.id AND CR.module_id = " . $category_filter['module_id'] . " AND CR." . $category_filter['level'] . " = " . $category_filter['category_id'];
            }
            if (! empty($where) && (strpos($where, 'SUBSTRING') !== false)) {
                $sql .= " WHERE " . $where;
            } else if (! empty($where)) {
                $sql .= " WHERE " . $where;
            }
            $result = $this->db->query($sql)->row();
            $result = ! empty($result->total_count) ? $result->total_count : 0;
        }
        return $result;
    }
    
    /**
     * Function to retrieve/generate QRCode
     *
     * @param integer $id
     */
    public function get_listing_qrcode($id)
    {
        $this->db->select($this->db->dbprefix . 'listings.slug,' . $this->db->dbprefix . 'listings.qrfile, C.name as city');
        $this->db->join($this->db->dbprefix . 'cities as C', 'C.id = ' . $this->db->dbprefix . 'listings.city_id', 'left');
        return $this->listings_model->find($id);
    }
    
    /*
     * ------------------------------------------------------------------------------------------------------
     * FRONTEND CATEGORY, LOCATION AND SEARCH
     * ------------------------------------------------------------------------------------------------------
     */
    /**
     * Search rows in a module
     *
     * @param boolean $mobile
     *            - If TRUE request if from Mobile APP
     * @param string $module
     *            @pram string $type - Category/Location/Search
     * @param string $slug
     *            - Category and Location slug to get ID
     * @param int $limit
     * @param int $offset
     *            - offset is must because of Mobile App
     * @param int $user_id
     *            - Recently Viewed Items - Mobile App
     */
    public function category_location_tag_search($mobile, $module, $type, $country, $slug, $limit, $offset = 0, $user_id = 0) {
        if(!$mobile && empty($user_id)) {
            $user_id = $this->auth->user_id();
        }
        // Fields Search and Filteration
        $this->load->model('locations/cities_model');
        $this->load->model('fields/fields_model');
        $this->load->library('layouts');
        $fields = $this->fields_model->filter($module, $mobile);
        
        // In case of listing, the category/city slug is at segment 2
        // whereas for other modules it is at segment 3
        // required for slug comparison, if slug is different, redirect to the correct URL
        if ($module == 'listings') {
            $uri_segment = $this->uri->segment(2);
        } else {
            $uri_segment = $this->uri->segment(3);
        }
        
        // Passing Parameters
        if (! $mobile) {
            $offset = intval($this->input->get('per_page'));
        }
        $id = 0;
        $category_id = 0;
        $tag_id = 0;
        $location_id = 0;
        $type_location = 0;
        $postcode = 0;
        $locality_id = 0;
        $sort_by = 'created_on';
        $keyword = "";
        $location = "";
        $searched_category = 0;
        $markers = array(); // To Avoid duplicates
        
        // Allow user to select number of rows
        $limit = isset($_GET['limit']) ? $_GET['limit'] : $limit;
        $row_limit = $limit;
        
        // Sort By Relevance (Created On), Ratings and Title Feature
        $sortBy = isset($_GET['sort_by']) ? $_GET['sort_by'] : 'relevance';
        
        $where = NULL;
        // avoid unnecessary categories search and filters
        if(settings_item('site.minimal') == 1) {
            $_GET['cg'] = 0;
            $_GET['having'] = '';
            $_GET['c'] = 0;
        }
        if ($module == 'listings' && isset($_GET['having'])) { // Records having images, videos and other modules
            $having_filter = $this->having_filter($_GET['having']);
            $where = $having_filter['where'];
            $having = $having_filter['having'];
        }
        
        // nested categories dropdown filter
        if(!$mobile) {
            Assets::add_js('categories-selection.js');
            if(!empty($_GET['cg'])) {
                Template::set('filter_category', $this->gc->closure(array('where' => 'id = ' . $_GET['cg'])));
            }
        }
        if(isset($_POST['category_filter'])) {
            // Selected Category
            if($this->input->post('subsub_id')) {
                $category_id = escape($this->input->post('subsub_id'));
            } else if($this->input->post('sub_id')) {
                $category_id = escape($this->input->post('sub_id'));
            } else {
                $category_id = escape($this->input->post('parent_id'));
            }
            $current_url = current_url() . (!empty($_SERVER['QUERY_STRING']) ? '?' . $_SERVER['QUERY_STRING'] : '');
            $current_url = replace_querystring(array('cg' => $category_id));
            Template::redirect($current_url);
        }
        
        // categories, cities & localities module
        if ($type == 'categories' || $type == 'cities' || $type == 'localities' || $type == 'tags') {
            // unset filter_search session
            $this->session->unset_userdata('filter_search');
            /**
             * Find redirect URL based on module
             * get all characters from right to left till first dash to get id
             * Redirect visitor to home page if id is not an integer value
             * Find record with given id
             * Redirect visitor to home page if id is not found in database
             * Create or get slug from database
             * Get Banners to display on category and location page
             * Count total results for pagination
             */
            if ($mobile) {
                $id = $slug;
            } else {
                if($type != 'categories') {
                    $id = substr($slug, strrpos($slug, '-') + 1);
                    if (! intval($id)) {
                        redirect(site_url());
                    }
                }
            }
            
            switch ($type) {
                case 'categories':
                    $this->load->model('categories/categories_model');
                    if(settings_item('site.category_slug_uris') == 1) {
                        $db_column = 'slug_uris';
                        $db_value = escape($slug);
                    } else {
                        $id = substr($slug, strrpos($slug, '-') + 1);
                        if (! intval($id)) {
                            redirect(site_url());
                        }
                        $db_column = 'id';
                        $db_value = $id;
                    }
                    $record = $this->gc->get('categories', array('where' => $db_column . " = '" . $db_value . "'"));
                    if (! $record) {
                        if ($mobile) {
                            $mobile_response = array(
                                'success' => false,
                                'message' => lang('label_no_record_found')
                            );
                            return $mobile_response;
                        } else {
                            Template::redirect(site_url());
                        }
                    }
                    
                    // get translations if exists
                    $record = $this->translations->get_translations('categories', $id, $record, 1);
                    
                    $redirect_url = gc_category_url($module, $record->slug, $record->id);
                    if (settings_item('lst.category_city_filter') == 1) {
                        // The module has location, display only the selected city results to the user
                        if ($module == 'listings' || gc_module_settings($module, 'locations' == 1)) {
                            $category_location = json_decode($this->session->userdata('tl_location') ?? '');
                            if (isset($category_location->name)) {
                                $location_id = $category_location->id;
                                $location = $category_location->name;
                                $type_location = $category_location->type;
                            }
                        }
                        // slug for comparison
                        $record->slug = gc_slug_city($module, $record->slug);
                    }
                    // Generate random description by parsing keywords in a string
                    if(empty($record->description)) {
                        $record->description = $this->gc->parse_meta('category', array('category' => $record->name, 'page' => 'category'), 'seo.category_description');
                    }
                    // for accordion menu
                    Template::set('parent_id', $record->parent_id);
                    break;
                case 'cities':
                    $type_model = $this->load->model('locations/cities_model');
                    $record = $this->cities_model->find($id);
                    if (! $record) {
                        if ($mobile) {
                            $mobile_response = array(
                                'success' => false,
                                'message' => lang('label_no_record_found')
                            );
                            return $mobile_response;
                        } else {
                            Template::redirect(site_url());
                        }
                    }
                    $redirect_url = gc_location_url($module, $record->slug, $record->id);
                    $city_info = $this->cities_model->get_city_info($record->id);
                    $place = $record->name . ', ' . $city_info->state . ', ' . ucwords(strtolower($city_info->country));
                    // Generate random description by parsing keywords in a string
                    if(empty($record->description)) {
                        $record->description = $this->gc->parse_meta('location', array('place' => $place, 'location' => $record->name, 'page' => 'location'), 'seo.location_description');
                    }
                    break;
                case 'localities':
                    $type_model = $this->load->model('locations/localities_model');
                    // $record = $this->localities_model->find($id);
                    $sql = "SELECT M.id, M.slug, M.name, M.city_id, C.name AS city, C.slug AS city_slug";
                    $sql .= " FROM " . $this->db->dbprefix . "localities M";
                    $sql .= " INNER JOIN " . $this->db->dbprefix. "cities C ON C.id = M.city_id";
                    $sql .= " WHERE M.id = " . (int) $id;
                    $record = $this->db->query($sql)->row();
                    break;
                case 'tags':
                    $type_model = $this->load->model('tags/tags_model');
                    $record = $this->tags_model->find($id);
                    break;
            }
            
            if (! $mobile && ! $record) {
                redirect(site_url());
            } else if ($mobile && ! $record) {
                $mobile_response = array(
                    'success' => false,
                    'message' => lang('label_no_record_found')
                );
                return $mobile_response;
            }
            
            if (! $mobile) {
                $title_page = $record->name;
                if ($type == 'localities') {
                    $slug = $record->slug . '-' . $record->id;
                    $redirect_url = site_url(gc_uri_string('location') . '/' . $record->city_slug . '/' . $slug);
                    $uri_segment = $this->uri->segment(2) . '/' . $this->uri->segment(3);
                    $slug = $record->city_slug . '/' . $slug;
                    $record->description = null;
                    $locality = $record;
                    if(!empty($_GET['cg'])) {
                        $category_info = $this->gc->get('categories', array('where' => 'id = ' . (int) $_GET['cg']));
                        if(!empty($category_info)) {
                            $title_page = $category_info->name . ' in ' . $record->name;
                        }
                    }
                } else {
                    $slug = $record->slug . '-' . $record->id;
                }
                if($type == 'categories' && settings_item('site.category_slug_uris') == 1) {
                    // do nothing
                } else if($slug != $uri_segment) {
                    redirect($redirect_url);
                }
            }
            
            if (! $mobile) {
                // Display category/city title and description to the user
                Template::set(array(
                    'title' => $title_page,
                    'description' => (($module == 'tags' || $module == 'localities') ? '' : $record->description),
                    'row_info' => $record, // Display Category/Location image to the user if it exists
                    'row_type' => $type // Display image based on the Type
                ));
                
                // REPORTS - LISTING VIEWS
                switch ($module) {
                    case 'categories':
                        $this->session->set_userdata('lvs_page', 'Category');
                        break;
                    case 'cities':
                        $this->session->set_userdata('lvs_page', 'Location');
                        break;
                    case 'tags':
                        $this->session->set_userdata('lvs_page', 'Tag');
                        break;
                }
                $this->session->set_userdata('lvs_keyword', $record->name);
            }
            
            // Pass parameters to the search function
            switch ($type) {
                case 'categories':
                    $searched_category = $id;
                    $category_id = $id;
                    break;
                case 'cities':
                    $location = $record->name;
                    $location_id = $id;
                    $type_location = 1; // City Search
                    break;
                case 'localities':
                    $location_id = $locality->city_id;
                    $locality_id = $id;
                    break;
            }
            
            /*
             * if($type == 'categories') {
             * $category_id = $id;
             * } else if($type == 'cities') {
             * $location_id = $id;
             * if(!$mobile) {
             * Template::set('pre_location', $record->name);
             * Template::set('type_location', 1);
             * }
             * } else if($type == 'localities') {
             *
             * if(!$mobile) {
             * Template::set('pre_location', $record->name);
             * Template::set('type_location', 2);
             * }
             * }
             */
        }        
        $cg = (int) $this->input->get('cg');
        if(!empty($cg)) {
            $searched_category = $cg;
            $category_id = $searched_category;
        }
        if (isset($_GET['search']) || isset($_GET['q']) || isset($_GET['l']) || (isset($_GET['cg']) && isset($_GET['search']))) {
            $keyword = escape($this->input->get('q'));
            $location = escape($this->input->get('l'));
            $category_id = (int) escape($this->input->get('cg'));
            $locality_id = $this->input->get('lt') ? (int) escape($this->input->get('lt')) : 0;
            $type_location = $this->input->get('tl') ? (int) escape($this->input->get('tl')) : 0;
            $tag_id = $this->input->get('t') ? (int) escape($this->input->get('t')) : 0;
            
            // save last search parameters to avoid user to type city on each module search page
            $last_search_params = $this->input->get();
            unset($last_search_params['search']);
            if(!empty($last_search_params)) {
                $this->session->set_userdata('last_search_params', $last_search_params);
            }
            
            // check whether the keyword is a Tag or not - 23/04/17
            // if the keyword is a Tag find its ID and pass it to the search_listings method
            // done to handle tags manually using PHP
            if ($tag_id == 0) {
                $this->load->model('tags/tags_model');
                $get_tag_id = $this->gc->get('tags', array('where' => "name = '" . $keyword . "'", 'row' => 1));
                if ($get_tag_id) {
                    $tag_id = (int) $get_tag_id->id;
                    $id = $tag_id; // for Ads module
                }
            }
            if(empty($category_id) && !empty($keyword)) { // banners
                $category_info = $this->gc->get('categories', array('where' => "name = '" . $keyword . "'", 'row' => 1));
                if($category_info) {
                    $category_id = $category_info->id;
                }
            }
            // redirect user to the home page if the location type is not city, state and postcode
            if ($type_location > 4) {
                if ($mobile) {
                    return array(
                        'success' => false,
                        'total_rows' => 0
                    );
                } else {
                    Template::redirect(site_url());
                }
            }
            // To deal with a category name typed in the keyword field
            $keyword_category = escape($this->input->get('ca') ? (int) $this->input->get('ca') : 0);
            
            /**
             * tl - To find out whether the user searched by a city, state, postcode or locality
             * The user typed a location and search for a keyword
             * Again click on the search icon for the same location without changing the keyword value
             * was resulting in unset of tl query strings
             * To pass them again to the url, a session variable is declared.
             * All parameters are stored in this session variable
             */
            // Do not perform below operations if default and only search is enabled
            if (settings_item('adv.search_blocks') != 1 || settings_item('adv.search_blocks') != 4) {
                if ($type_location == 3) { // search location is a postcode
                    $postcode = $location;
                } else if (! $mobile && ! empty($location)) { // search location is either a city/state or locality
                    $json_array = (array) json_decode($this->session->userdata('tl_location') ?? '');
                    if (!empty($json_array['name']) && $json_array['name'] == $location) {
                        if ($json_array['type'] == 4) { // user searched by a locality
                            $locality_id = $json_array['id'];
                        } else { // user searched by a state/city
                            $location_id = $json_array['id'];
                        }
                        $type_location = $json_array['type'];
                    } else {
                        /* COMMENTED ON 11/04/19 BECAUSE of the LOCATION column in the modules table
                         * $db_location_table = 'cities';
                         switch ($type_location) {
                         case 2:
                         $db_location_table = 'states';
                         break;
                         case 4:
                         $db_location_table = 'localities';
                         break;
                         }
                         
                         $result_location = $this->get_location_id($location, $db_location_table);
                         if ($result_location) {
                         if ($result_location->type == 4) {
                         $locality_id = $result_location->id;
                         } else {
                         $location_id = $result_location->id;
                         }
                         $type_location = $result_location->type;
                         $this->session->set_userdata('tl_location', json_encode($result_location));
                         }
                         */
                        $type_location = 4; // Added on 11/04/19
                    }
                } else if ($mobile && ! empty($location)) {
                    $location_id = $location;
                    // $type_location = 1;
                }
                
                if (! $mobile) {
                    // For navbar search fields
                    Template::set('type_location', $type_location);
                    // _footer.php file
                    Template::set('pre_location', $location);
                }
            }
            
            // update visitors table
            if (settings_item('adv.search_blocks') != 4 && !empty($location_id)) {
                // Default search block store location id in location query string
                $this->load->model('visitors/visitors_model');
                if (settings_item('adv.search_blocks') == 1) {
                    $this->visitors_model->update_visitor_info(1, $location);
                } else {
                    $this->visitors_model->update_visitor_info($type_location, $location_id);
                }
            } /*else { // there is already a table search_queries
            $ip_address = $this->input->ip_address();
            $created_date = date('Y-m-d H:i:s');
            $module_id = gc_module_id($module);
            $this->db->query("INSERT IGNORE INTO " . $this->db->dbprefix . "visitor_searches (user_id, module_id, location, category_id, keyword, ip_address, created_on) VALUES ($user_id, $module_id, '$location', $category_id, '$keyword', '$ip_address', '$created_date')");
            }*/
            
            /**
             * $keyword_category/ca - To deal with a category name typed in the keyword field
             * $searched_category - Category_id passed to the search function
             */
            if ($keyword_category != 0) {
                // When the searched keyword category id is equals to the parent category id
                if ($keyword_category == $category_id) {
                    $searched_category = $keyword_category;
                    $category_id = $keyword_category;
                } else if (! $mobile) {
                    // when the searched category is not a parent
                    // To avoid database query repeatation in case of pagination
                    $searched_category = $keyword_category; // set the category id
                    $s_category = (array) json_decode($this->session->userdata('s_category') ?? '');
                    // set the category id to display in the category dropdown field
                    if ($this->session->userdata('s_category') && ($keyword_category == $s_category['subsub_id'] || $keyword_category == $s_category['sub_id'])) {
                        if ($keyword_category == $s_category['subsub_id']) {
                            $category_id = $s_category['parent_id'];
                        } else if ($keyword_category == $s_category['sub_id']) {
                            $category_id = $s_category['parent_id'];
                        }
                    } else {
                        $categories_level = $this->gc->category_level_search($keyword_category);
                        $cat_array = array(
                            'name' => $keyword,
                            'parent_id' => $categories_level['parent_id'],
                            'sub_id' => $categories_level['sub_id'],
                            'subsub_id' => $categories_level['subsub_id']
                        );
                        // To avoid database query repeatation in case of pagination
                        $this->session->set_userdata('s_category', json_encode($cat_array));
                        $searched_category = $keyword_category;
                        $category_id = $categories_level['parent_id'];
                    }
                    // The user clicked a category name in the ajax search result
                    // i.e. search using the category id, no need to search the entire table
                    // set the keyword to null
                    $keyword = NULL;
                }
                /**
                 * User clicked on the search button without changing the keyword (category name) and location
                 * Get category name from the session variable and match with the keyword if both are same
                 * append searched category id field (ca) with the query string for category search
                 */
            } else if (! $mobile && ! empty($category_id) && $this->session->userdata('s_category')) {
                $s_category = (array) json_decode($this->session->userdata('s_category') ?? '');
                // Session saved category name must match the keyword
                // Saved category parent id must also match the $category_id
                // sometimes user change the parent category from the category list
                // in such cases, ca parameters must be removed from the query string
                // as the searched keyword is from a different category
                if ($keyword == $s_category['name'] && ($s_category['parent_id'] == $category_id)) {
                    if (! empty($s_category['subsub_id'])) {
                        $keyword_category = $s_category['subsub_id'];
                    } else if (! empty($s_category['sub_id'])) {
                        $keyword_category = $s_category['sub_id'];
                    } else if (! empty($s_category['parent_id'])) {
                        $keyword_category = $s_category['parent_id'];
                    }
                }
            } else if (! $mobile && ! empty($category_id)) { // Category search issue fixed on April 22, 2017
                $searched_category = $category_id;
            }
            
            /**
             * Reset offset is necessary if user searches from the last page of pagination
             * without redirect, if no result is found, user will be moved to the specified pagination page
             */
            if (! $mobile && isset($_GET['search'])) {
                /**
                 * Postcode is only applicable in case of Mobile App and ajax search
                 * If postcode is entered, ignore city search
                 */
                /**
                 * the user selected a location from the typehead search result
                 * Selection Location Codes
                 * 1 - City
                 * 2 - State
                 * 3 - Postcode
                 * 4 - Locality
                 * tl - Type of Location
                 */
                // NOT REQUIRED BECAUSE OF PAGINATION CHANGE FROM POST TO QUERY STRING
                /*
                 * if(!empty($keyword) && !empty($location)) {
                 * redirect(site_url(gc_uri_string($module) ."?q=".urlencode($keyword) ."&l=".urlencode($location) ."&mtl=" . $type_location));
                 * } elseif(!empty($keyword)) {
                 * redirect(site_url(gc_uri_string($module) ."?q=".urlencode($keyword)));
                 * } elseif(!empty($location)) {
                 * redirect(site_url(gc_uri_string($module) ."?l=".urlencode($location)."&mtl=" . $type_location));
                 * } else {
                 * redirect(site_url(gc_uri_string($module)));
                 * }
                 */
                // unset filter search session variable
                $this->session->unset_userdata('filter_search');
                
                // REPORTS - SEARCH QUERIES
                $search_loc = $location;
                if (settings_item('adv.search_blocks') == 1) {
                    $city_name = $this->cities_model->select('name')->find($location);
                    if ($city_name) {
                        $search_loc = $city_name->name;
                    }
                }
                
                // Update the search_city, search_state and search_country session variables
                // Session must be updated if the search_city is changed to set the default country for future visits
                if (!empty($location_id) && (int) $location_id != (int) $this->session->userdata('search_city')) {
                    $city_info = $this->cities_model->get_city_info($location_id);
                    if ($city_info) {
                        $this->session->set_userdata('search_city', $location_id);
                        $this->session->set_userdata('pre_city', $city_info->city);
                        $this->session->set_userdata('search_state', $city_info->state_id);
                        $this->session->set_userdata('pre_state', $city_info->state);
                        $this->session->set_userdata('search_country', $city_info->country_iso);
                    }
                }
                
                // REPORTS - LISTING VIEWS
                $this->session->set_userdata('lvs_page', 'Search');
                $this->session->set_userdata('lvs_keyword', $keyword);
            }
            
            /**
             * Set visitor selected keyword, location, category, locality information to the specified block
             * after get request for pagination
             */
            $meta_search_city = ucwords($location);
            
            if (settings_item('adv.search_blocks') == 1) {
                if (! $mobile) {
                    Template::set(array(
                        'searchLocality' => $locality_id
                    ));
                }
                
                // Set city name for meta title, description and keyword
                $meta_search_city = $this->cities_model->select('name')->find($location);
                if ($meta_search_city) {
                    $meta_search_city = $meta_search_city->name;
                }
            }
        }
        
        // Location and Category Specific Banners - August 20, 2019
        if(empty($location_id) && empty($search_loc)) {
            $this->session->unset_userdata('banner_location_id');
        } else if(empty($location_id) && !empty($search_loc)) {
            $cacheID = 'search_location_id_' . str_replace('-', '_', $this->slug->create_slug($search_loc));
            if (! $location_id = $this->z_cache->load($cacheID)) {
                $location_id = 0;
                $location_array = explode(', ', $search_loc ?? '');
                if(isset($location_array[0])) {
                    $this->db->from('cities C');
                    if(!empty($location_array[1])) {
                        $this->db->join('states S', 'S.id = C.state_id', 'INNER');
                        $this->db->where('S.name', $location_array[1]);
                    }
                    $city = $this->db->select('C.id, C.state_id')->where('C.name', $location_array[0])->get()->row();
                    if($city) {
                        $location_id = $city->id;
                    }
                }
                $this->z_cache->save($location_id, $cacheID, array('locations'));
            }
        }
        if(!empty($location_id)) {
            $this->session->set_userdata('banner_location_id', $location_id);
        }
        if(empty($category_id)) {
            $this->session->unset_userdata('banner_category_id');
        } else {
            $this->session->set_userdata('banner_category_id', $category_id);
        }
        
        /*
         * FEATURED SECTION
         * /******************************************************
         */
        // extra parameters to be passed
        $fields_extra = array(
            'filter_search_fields' => $fields,
            'user_id' => $user_id
        );
        
        // Order businesses based on users current location
        if(!empty($_GET['lat']) && !empty($_GET['lng'])) {
            $fields_extra['latitude'] = $_GET['lat'];
            $fields_extra['longitude'] = $_GET['lng'];
            $fields_extra['radius'] = !empty($_GET['radius']) ? $_GET['radius'] : 25000; // search within 50 kilometers of radius
        } else if(($type == 'categories' || $type == 'tags') && !empty(settings_item('site.geo_radius')) && $this->session->userdata('geo_visitor_info')) {
            $geo_visitor_info = $this->session->userdata('geo_visitor_info');
            if(!empty($geo_visitor_info['latitude']) && !empty($geo_visitor_info['longitude'])) {
                $fields_extra['latitude'] = $geo_visitor_info['latitude'];
                $fields_extra['longitude'] = $geo_visitor_info['longitude'];
                $fields_extra['radius'] = !empty($geo_visitor_info['radius']) ? $geo_visitor_info['radius'] : floatval(settings_item('site.geo_radius')); // display within a range of 50 kms
            }
        }
        
        // Advertisement Module Changes
        $this->load->model('ads/ads_model');
        $ad_featured_rows = $this->ads_model->ads($module, ($tag_id != 0 ? $tag_id : $id), ($type == 'search' ? 'tags' : $type), array(), $fields_extra);
        
        // Set Limit Tab
        $this->load->helper('array');
        $limit = $limit;
        $row_limit = $limit;
        
        /**
         * Get all featured listings from database
         * Count total featured listings and declare array for top, right and random featured.
         * Random_featured array is to store featured listings which are randomly selected.
         * Top_featured array will store top featured listings.
         * Right_featured array is for right hand side featured listings.
         * Compare total featured count to user specified top featured counts.
         * Get total number of listings specified by the admin.
         * If total number of featured listings is greater than total_random, get random values equal to total_random or else equal to featured_listings_count
         */
        $where_fields = array(
            'where' => $where,
            'tag_id' => $tag_id,
            'postcode' => $postcode,
            'locality_id' => $locality_id,
            'category_id' => $searched_category
        );
        if (settings_item('lst.country_filter') == 1 && !empty($country)) {
            $where_fields['country_iso'] = $country;
        }
        
        // display listings specific modules e.g. products, blogs, properties etc.
        if(!empty($_GET['listing_id']) && $module != 'listings') {
            $where_fields['listing_id'] = (int) $_GET['listing_id'];
        }
        
        // Package specific modules
        if(!empty($_GET['package_id'])) {
            $where_fields['package_id'] = (int) $_GET['package_id'];
        }
        
        // User specific rows
        if(!empty($_GET['user_id'])) {
            $where_fields['user_id'] = (int) $_GET['user_id'];
        }
        
        $where_fields['location'] = $location;
        switch($type_location) {
            case 1:
                $where_fields['city_id'] = $location_id;
                break;
            case 2:
                $where_fields['state_id'] = $location_id;
                break;
        }
        if(!empty($_GET['featured'])) { // view featured listings only
            $where_fields['featured'] = 1;
        }
        if(!empty($_GET['type_id'])) { // products - veg/egg/non-veg filter
            $where_fields['type_id'] = $_GET['type_id'];
        }
        $featured_where_fields = $where_fields;
        $featured_where_fields['featured'] = 1;
        
        /**
         * Get all featured listings from database
         * Count total featured listings and declare array for top, right and random featured.
         * Random_featured array is to store featured listings which are randomly selected.
         * Top_featured array will store top featured listings.
         * Right_featured array is for right hand side featured listings.
         * Compare total featured count to user specified top featured counts.
         * Get total number of listings specified by the admin.
         * If total number of featured listings is greater than total_random, get random values equal to total_random or else equal to featured_listings_count
         */
        $featured_rows = $this->get_rows(0, $module, $keyword, $featured_where_fields, $sortBy, $limit, $offset, $fields_extra);
        $featured_rows = array_merge($ad_featured_rows, $featured_rows);
        if (! empty($ad_featured_rows)) {
            $featured_rows = $this->unique_multidim_array($featured_rows, 'id');
        }
        
        $random_featured = array();
        $top_featured = array();
        $right_featured = array();
        
        if ($featured_rows) {
            $featured_rows_count = count($featured_rows);
            if ($featured_rows_count > (int) settings_item('lst.top_featured_count')) {
                $total_random = (int) settings_item('lst.top_featured_count') + (int) settings_item('lst.right_featured_count');
                if ($featured_rows_count >= $total_random) {
                    $random_featured = $this->array_random_assoc($featured_rows, $total_random); // get random values from array
                } else {
                    $random_featured = $this->array_random_assoc($featured_rows, $featured_rows_count);
                }
                // Sort an array by values using a user-defined comparison function.
                // usort($random_featured, array($this, "sortByPrice"));
                $top_featured = array_slice($random_featured, 0, (int) settings_item('lst.top_featured_count')); // split array starting from 0
                $right_featured = array_slice($random_featured, (int) settings_item('lst.top_featured_count')); // split array starting from 2
            } else {
                $top_featured = $this->array_random_assoc($featured_rows, $featured_rows_count);
            }
        }
        
        // Ads Impression Counter - Array merge top and right featured for ad impression counter
        $this->ads_model->impression_counter(array_merge($top_featured, $right_featured));
        /**
         * ****************************************************
         */
        /*
         * END OF FEATURED SECTION
         * /******************************************************
         */
        
        /**
         * ****************************************************
         */
        /*
         * GET NON FEATURED ROWS
         * /******************************************************
         */
        /**
         * Apply limit and offset and get remaining rows
         * Get page banners and footer pages link
         */
        $records = $this->get_rows(0, $module, $keyword, $where_fields, $sortBy, $limit, $offset, $fields_extra);
        $totalrows = $this->get_rows(1, $module, $keyword, $where_fields, $sortBy, $limit, $offset, $fields_extra);
        
        // Update Categories and Cities Hits
        // Add Recently Viewed
        if ($type != 'search') {
            if(empty(settings_item('site.minimal'))) {
                $this->gc->updateHits($id, $type);
                $this->gc->add_recently_viewed($id, $type, ($mobile ? $user_id : $this->auth->user_id()));
            }
        } else {            
            $ip_address = $this->input->ip_address();
            $query_keyword = str_replace('+', ' ', $keyword);
            $this->db->query("INSERT INTO " . $this->db->dbprefix . "search_queries (keyword, location, results, created_ip, created_on) VALUES ('$query_keyword', '$location', $totalrows, '$ip_address', '" . date('Y-m-d H:i:s') . "') ON DUPLICATE KEY UPDATE times = times + 1, results = " . $totalrows. ", modified_on = '" . date('Y-m-d H:i:s') . "', modified_ip = '" . $ip_address . "'");            
        }
        
        $google_listings = false;
        // Theme/View Specific Fields
        $tag_url_params = array(
            'tag_id' => $tag_id,
            'location' => $location,
            'location_id' => $location_id,
            'category_id' => $category_id,
            'locality_id' => $locality_id,
            'type_location' => $type_location
        );
        if($mobile) {
            // the mobile has different fields, avoid page loading time by passing only specific fields
            $this->load->model('mobiles/mobiles_model');
            $records = $this->mobiles_model->module_view_fields($module, $records);
        } else {
            $records = $this->layouts->widget_modules_view($module, $records, $tag_url_params, null);
        }
        
        // No Records found or records count is less than the limit
        // Display results from Google
        if ($module == 'listings' && count($records) < $limit && settings_item('lst.google_business') == 1 && ! $mobile) {
            $this->load->library('listings/Google_listings');
            $google_keyword = $keyword;
            $google_location = $location;
            // Category/Tags search
            if (empty($google_keyword) && ($type == 'categories' || $type == 'tags') && isset($record->name)) {
                $google_keyword = $record->name;
            }
            if (empty($google_location)) {
                if (($type == 'cities' || $type == 'localities') && isset($record->name)) {
                    $google_location = $record->name;
                } else { // Get City information from the session variable
                    $google_location = $this->session->userdata('pre_city');
                }
            }
            $google_result = $this->google_listings->google_search($google_keyword, $google_location);
            if($google_result) {
				$listings_result = $google_result['listings'];
				if (empty($records) || isset($_GET['token'])) {
					$records = $google_result['listings'];
				} else if (! empty($listings_result)) {
					foreach ($listings_result as $listing_result) {
						$records[] = $listing_result;
					}
				}
				$markers = $google_result['markers'];
			}
            $google_listings = true;
        }
        
        if($mobile) {
            // the mobile has different fields, avoid page loading time by passing only specific fields
            $top_featured = $this->mobiles_model->module_view_fields($module, $top_featured);
            $right_featured = $this->mobiles_model->module_view_fields($module, $right_featured);
        } else {
            $top_featured = $this->layouts->widget_modules_view($module, $top_featured, $tag_url_params, null);
            $right_featured = $this->layouts->widget_modules_view($module, $right_featured, $tag_url_params, null);
        }
        
        /**
         * ****************************************************
         */
        /*
         * END OF NON FEATURED ROWS
         * /******************************************************
         */
        if (! $mobile) {
            // Pagination
            $this->load->library('pagination');
            $this->config->load('pagination');
            
            /**
             * if you go back from another pagination number to click on the first(1), CI does not care anymore of your suffix config
             * therefore, below given code retrieve query string in case of pagination links are clicked
             */
            if ($module == 'listings') {
                if ($type == 'search') {
                    $config['base_url'] = gc_url_format('search');
                } else {
                    if($type == 'localities') {
                        $config['base_url'] = $redirect_url;
                    } else {
                        $config['base_url'] = gc_url_format($type, $record->slug, $id);
                    }
                }
            } else {
                $config['base_url'] = gc_url_format($module);
            }
            $config['total_rows'] = $totalrows;
            $config['per_page'] = $limit;
            // Pagination using GET Method
            $config['reuse_query_string'] = TRUE;
            $config['enable_query_strings'] = TRUE;
            $config['page_query_string'] = TRUE;
            
            $this->pagination->initialize($config);
            $pagination = $this->pagination->create_links();
            
            // Showing total rows count
            if ($pagination != '') {
                $sLocation = null;
                if ($type == 'cities') {
                    $sLocation = $record->name;
                } else if($type == 'localities') {
                    $sLocation = $record->name . ', ' . $record->city;
                } else if ($module == 'listings' && $type == 'search') {
                    $sLocation = $location;
                }
                $this->pagination_summary($module, $totalrows, $sLocation, $keyword);
            }
            
            /**
             * Meta Information
             */
            if ($module == 'listings') {
                $saved_meta_title = ! empty($record->meta_title) ? $record->meta_title : NULL;
                $saved_meta_description = ! empty($record->meta_description) ? $record->meta_description : NULL;
                $saved_meta_keywords = ! empty($record->meta_keywords) ? $record->meta_keywords : NULL;
                $meta_name = $type == 'search' || $type == 'tags' ? ($type == 'tags' ? $record->name : $keyword) : $record->name;
                $meta_search_location = ! empty($meta_search_city) ? $meta_search_city : NULL;
                $place = isset($place) ? $place : null;
                if(empty($meta_name)) {
                    $meta_name = 'list of companies';
                    $meta_search_location = 'different cities of';
                    $this->session->set_userdata('meta_country', 'India');
                }
                $page_title = $this->_meta($type, $saved_meta_title, $meta_name, null, $meta_search_location, 'title', $place);
                $page_description = $this->_meta($type, $saved_meta_description, $meta_name, isset($record->description) ? $record->description : NULL, $meta_search_location, 'description', $place);
                $page_keywords = $this->_meta($type, $saved_meta_keywords, $meta_name, ! empty($record->meta_keywords) ? $record->meta_keywords : NULL, $meta_search_location, 'keywords', $place);
                if (! empty($page_keywords)) {
                    Template::set('page_keywords', $page_keywords);
                }
            } else {
                // Module Titles - As Meta Keywords and Meta Description
                $country = $this->db->query("SELECT iso, name FROM " . $this->db->dbprefix . "countries WHERE iso = '" . $country . "'")->row();
                // NOT WORKING INVALID ID ISSUE - $country = $this->db->select('iso, name')->get_where('countries', array('iso' => $country))->row();
                // NOT WORKING INVALID ID ISSUE - $country = $this->countries_model->select('name')->find($country);
                $page_title = ! empty($keyword) ? ucwords($keyword) : sprintf(lang('module_page_title'), gc_module_translation($module));
                $page_description = sprintf(lang('module_page_description'), gc_module_translation($module), $country->name);
                $module_meta_keywords = array();
                if (! empty($records)) {
                    $j = 1; // Display only first 5 rows title in the meta information
                    foreach ($records as $m_record) {
                        if ($j <= 5) {
                            $module_meta_keywords[] = strtolower(escape($m_record->title));
                            $j ++;
                        } else {
                            break;
                        }
                    }
                    $meta_info = implode(", ", $module_meta_keywords);
                    Template::set('page_keywords', $meta_info);
                }
            }
            
            /**
             * The state name is added with the city name in the search bar
             * Multiple states can have a city with the same name
             * Display state name with the city
             */
            if (! empty($location)) {
                $city_info = $this->cities_model->get_city_info($location_id);
                if (!empty($city_info->city)) {
                    $search_location = $city_info->city . ', ' . $city_info->state;
                    $this->session->set_userdata('location_keyword', $search_location);
                }
            }
            // display selected location on the category page
            $location_keyword = '';
            if(! empty($location) && $this->session->userdata('location_keyword')) {
                $location_session = explode(', ', $this->session->userdata('location_keyword'));
                if(!empty($location_session[0])) {
                    $location_keyword = $location_session[0];
                } else {
                    $tl_location = json_decode($this->session->userdata('tl_location') ?? '');
                    if(!empty($tl_location->name)) {
                        $location_keyword = $tl_location->name;
                    }
                }
            }
            // display keyword on listing detail page
            if(!empty($keyword)) {
                $this->session->set_userdata('searchKeyword', $keyword);
            } else {
                $this->session->unset_userdata('searchKeyword'); // unset keyword on browse by location and category
            }
            
            // banner specific session variables to display categories/cities specific banners
            $this->load->model('banners/banners_model');
            $this->banners_model->set_banner_session($category_id);
            $this->banners_model->set_banner_session($tag_id, 'tags');
            $this->banners_model->set_banner_session($location_id, 'cities');
            
            // default title and description in case of search
            if($type == 'search' && !empty(settings_item('seo.search_description')) && (!empty($keyword) || !empty($location))) {
                $search_title = '';
                $search_description = '';
                $keyword_search = '';
                $location_search = '';
                if(!empty($keyword)) {
                    $search_title .= $keyword . ' ';
                    $keyword_search = ucwords($keyword) . ' ';
                }
                if(!empty($location)) {
                    $location_search = ' ' . ucwords(lang('label_in') . ' ' . $location);
                    if(empty($keyword)) {
                        $search_title .= 'List of Companies' . $location_search;
                        $location_search = 'Products or Services' . $location_search;
                    } else {
                        $search_title .= $location_search;
                    }
                    $location_search .= ' ';
                }
                
                $search_description = $this->gc->parse_meta('search_keyword', array('keyword_search' => $keyword_search, 'location_search' => $location_search, 'page' => 'search'), 'seo.search_description');
                Template::set(array(
                    'title' => $search_title,
                    'description' => $search_description
                ));
            }            
            
            // child categories selection
            $category_parent_id = 0;
            if(!empty($category_id) && $type != 'categories') {
                $category_info = $this->gc->get('categories', array('where' => 'id = ' . $category_id));
                if(!empty($category_info) && $category_info->parent_id != 0) {
                    $category_parent_id = $category_info->parent_id;
                }
            }
            
            Template::set(array(
                'pagination' => $pagination,
                'page_title' => $page_title,
                'page_description' => $page_description,
                'records' => $records,
				'total_rows' => $totalrows,
                'top_featured' => $top_featured,
                'featured' => $right_featured,
                'module' => $module,
                // 'searchKeyword' => ! empty($keyword) ? $keyword : (! empty($record) && ($type == 'categories') ? $record->name : ''),
                'searchKeyword' => ! empty($keyword) ? $keyword : '', // do not put category name in the keyword field, category is already there in the category field
                'searchLocation' => !empty($location) ? $location : (! empty($record) && $type == 'cities' ? $record->name : $location_keyword),
                'pagination' => $pagination,
                'limit' => $row_limit, // set dropdown field value
                'sortBy' => $sortBy,
                'tag_url_params' => $tag_url_params, // Manual Tag Handling
                'searchCategory' => ($type == 'categories' ? $record->id : (!empty($category_parent_id) ? $category_parent_id : (isset($category_id) ? $category_id : 0))), // Listings Only
                'location_id' => $location_id, // Listings Only
                'google_listings' => $google_listings
            ));
            // categories/locations page sharing on social sites
            if(!empty($record)) {
                Template::set('social_sharing', gc_social_sharing($record));
            }
            // Listing specific
            // left side categories and right side map block
            if ($type == 'categories' || !empty($category_id)) {
                // right hand categories
                /*$child_categories = $this->categories_model->select('id, name, slug, icon, counts')
                 ->order_by('name', 'asc')
                 ->find_all_by(array(
                 'active' => 1,
                 'parent_id' => $id
                 ));*/
                $where_category_id = !empty($category_id) ? $category_id : $id;
                // get parent
                $category_tree = $this->gc->closure(array('where' => $where_category_id, 'parent' => 1, 'self' => true));
                $categories_where = 'active = 1 AND parent_id = ' . $where_category_id;
                $categories_where .= settings_item('lst.hide_counts_zero_categories') == 1 ? ' AND counts != 0' : '';
                $has_sub_categories = $this->gc->get('categories', array('where' => $categories_where));
                if(!$has_sub_categories && !empty($record)) {
                    $categories_where = 'active = 1 AND parent_id = ' . $record->parent_id;
                    $categories_where .= settings_item('lst.hide_counts_zero_categories') == 1 ? ' AND counts != 0' : '';
                }
                $child_categories = $this->translations->get_translated_rows('categories', array('id', 'name', (!empty(settings_item('site.category_slug_uris')) ? 'slug_uris as slug' : 'slug'), 'icon', 'counts', 'module_counts'), $categories_where, 'name ASC', 10000);
                $category_heading = '';
                if($type == 'categories') {
                    $category_heading = $record->name;
                } else if(!empty($category_info)) {
                    $category_heading = $category_info->name;
                }
                if ($child_categories) {
                    Template::set(array(
                        'category_tree' => $category_tree,
                        'child_categories' => $child_categories,
                        'category_heading' => $category_heading
                    ));
                }
            }
            if(empty($child_categories)) {
                $categories = $this->categories_model->categories(0, $category_parent_id, $module);
                Template::set('categories', $categories);
            }
            
            /*
             * Template::set_view('view');
             * Template::render('view');
             * else {
             * Template::render('module_view');
             * }
             */
            if (gc_module_settings($module, 'locations') == 1) {
                /**
                 * Display Google Map with multiple markers
                 * In order to avoide duplicate positions, a new array $markers is declared
                 */
                if (settings_item('lst.view_map') == 1) {
                    $map_leaflet = settings_item('lst.map_leaflet') == 1 ? true : false;
                    if (! $map_leaflet) { // must be initialized before adding markers
                        $this->load->library('listings/googlemaps');
                        $config['center'] = '' . settings_item('lst.latitude') . ', ' . settings_item('lst.longitude') . '';
                        $config['zoom'] = 'auto';
                        $config['places'] = TRUE; // Leads Form
                        $this->googlemaps->initialize($config);
                    }
                    $marker_top_featured = $this->right_map($top_featured, array(), true, $map_leaflet);
                    if (! empty($marker_top_featured)) {
                        $markers[] = $marker_top_featured;
                    }
                    $marker_right_featured = $this->right_map($right_featured, array(), true, $map_leaflet);
                    if (! empty($marker_right_featured)) {
                        $markers[] = $marker_right_featured;
                    }
                    if (! empty($markers)) {
                        if ($map_leaflet) { // Display OpenStreetMap
                            $markers = array_merge((array) $marker_top_featured, (array) $marker_right_featured);
                            // Data is stored in a session variable and sent using jquery POST method
                            $this->session->set_userdata('map_markers', $markers);
                        } else { // Display Google Map
                            if ($google_listings) {
                                $this->right_map($records, $markers, false);
                            } else {
                                foreach ($markers as $marker) {
                                    $this->googlemaps->add_marker($marker);
                                }
                            }
                            $map = $this->googlemaps->create_map();
                            Template::set('map', $map);
                        }
                    } else {
                        // if no markers, unset the session variable to avoid any ambiguity
                        $this->session->unset_userdata('map_markers');
                    }
                }
                Template::set('hasLocation', true);
            }
            $this->gc->load_query_fields($module); // load business query fields
            $layout = $this->gc->load_assets('modules/' . $module . '/view');
            Template::set_view($layout['view']);
            Template::render($layout['layout']);
        } else {
            $mobile_response = array(
                'success' => true,
                'toolbar_title' => !empty($record->name) ? $record->name : lang('search_results'),
                'total_rows' => $totalrows,
                'top_featured' => $top_featured, // $this->mobile_data($module, $top_featured),
                'records' => $records // $this->mobile_data($module, $records)
            );
            return $mobile_response;
        }
    }
    
    /**
     * Return Unique Multidimensional array for any single key index.
     *
     * @param array $array
     * @param string $key
     * @return multitype:StdClass
     */
    private function unique_multidim_array($array, $key)
    {
        $temp_array = array();
        $i = 0;
        $key_array = array();
        
        foreach ($array as $val) {
            $val = (array) $val;
            if (! in_array($val[$key], $key_array)) {
                $key_array[$i] = $val[$key];
                $temp_array[$i] = (object) $val; // return result as an object
            }
            $i ++;
        }
        return $temp_array;
    }
    
    /**
     * Having Filter
     *
     * @param string $module
     * @return multitype:Ambigous <NULL, string> Ambigous <NULL, unknown>
     */
    public function having_filter($module)
    {
        $modules = gc_modules_only();
        $modules[] = 'videos';
        $modules[] = 'images';
        $modules[] = 'branches';
        $modules[] = 'business_hours';
        $where = NULL;
        $having = NULL;
        if (in_array($module, $modules)) {
            $listings = array();
            if ($module == 'images' || $module == 'videos') {
                $where_search = array();
                $where_search['module_id'] = gc_module_id('listings');
                $where_search['status'] = 1;
                $where_search['deleted'] = 0;
                $result = $this->db->select('distinct(row_id) AS listing_id')
                ->get_where($module, $where_search)
                ->result();
            } else {
                $result = $this->db->select('distinct(listing_id)')
                ->get_where($module, array(
                    'status' => 1,
                    'deleted' => 0
                ))
                ->result();
            }
            if ($result) {
                foreach ($result as $row) {
                    $listings[] = $row->listing_id;
                }
            }
            
            if ($listings) {
                $where = "id IN (" . implode(",", $listings) . ")";
            }
            $having = $module;
        }
        return array(
            'where' => $where,
            'having' => $having
        );
    }
    
    /**
     * Display pagination summary to the visitor
     *
     * @param string $module
     * @param string $totalrows
     */
    public function pagination_summary($module, $totalrows, $location = null, $keyword = null)
    {
        $result_start = ($this->pagination->cur_page - 1) * $this->pagination->per_page + 1; // Result row for a page starts
        if ($result_start == 0)
            $result_start = 1; // *it happens only for the first run*
            $result_end = $result_start + $this->pagination->per_page - 1; // Result row for a page ends
            switch ($module) {
                case 'cities':
                case 'categories':
                case 'search':
                    $searchKeyword = !empty($keyword) ? ucwords($keyword) : lang('listings');
                    break;
                default:
                    $searchKeyword = !empty($keyword) ? ucwords($keyword) : lang($module);
                    break;
            }
            $searchLocation = ! empty($location) ? ucwords(strtolower($location)) : lang('msg_location_all');
            
            if ($result_end < $this->pagination->per_page) { // happens when records less than per page
                $result_end = $this->pagination->per_page;
            } else if ($result_end > $totalrows) { // happens when result end is greater than total records
                $result_end = $totalrows;
            }
            $pagination_summary = sprintf(lang('msg_pagination_results'), $result_start, $result_end, $totalrows, $searchKeyword, $searchLocation);
            if ($result_end == $totalrows) {
                $pagination_summary = sprintf(lang('msg_pagination_without_end'), $result_start, $totalrows, $searchKeyword, $searchLocation);
            }
            Template::set('pagination_summary', $pagination_summary);
    }
    
    /**
     * Prepare Right Map for the VIEW page
     *
     * @param array $listings
     * @param array $markers
     * @param boolean $return
     * @param boolean $map_leaflet
     *            - True - Display OpenStreetMap
     * @return string
     */
    private function right_map($listings, $markers = array(), $return = true, $map_leaflet = false)
    {
        if ($listings) {
            foreach ($listings as $listing) {
                if (! empty($listing->latitude) && ! empty($listing->longitude)) {
                    if (empty($markers)) {
                        if ($map_leaflet) {
                            $markers[] = $this->marker($listing, false, $map_leaflet);
                        } else {
                            $markers[] = $listing->latitude . $listing->longitude;
                            $this->marker($listing, false, $map_leaflet);
                        }
                    } else {
                        if (! in_array(($listing->latitude . $listing->longitude), $markers)) {
                            if ($map_leaflet) {
                                $markers[] = $this->marker($listing, false, $map_leaflet);
                            } else {
                                $this->marker($listing, false, $map_leaflet);
                            }
                        }
                    }
                } /*
                * else { // Do not use Address to display the User's location, loading the map from address is slowing down the page browsing.
                * $this->marker($listing, true);
                * }
                */
            }
        }
        if ($return) {
            return $markers;
        }
    }
    
    /**
     * Add marker to the Google Map
     *
     * @param array $listing
     * @param
     *            boolean - if TRUE use listing address instead of latitude and longitude
     * @param boolean $map_leaflet
     */
    private function marker($listing, $address = false, $map_leaflet = false)
    {
        // $this->load->library('slug');
        $marker = array();
        if (isset($listing->address) || (isset($listing->latitude) && isset($listing->longitude))) {
            $infowindow = '<a href="' . $listing->redirect_url . '" title="' . $listing->title . '"><strong>' . $listing->title . '</strong></a><br />';
            if (! empty($listing->address)) {
                // The Google Map does not recognise special characters (:, ()) in an address which results in a blank map.
                // i.e. the address must be json encoded, this will add a double qoute around the address
                $infowindow .= json_encode($listing->address);
            }
            /*
             * $infowindow .= $listing->city .', '. $listing->state;
             * if(isset($listing->postcode)) {
             * $infowindow .= ' ' . $listing->postcode;
             * }
             */
            if ($address && isset($listing->address)) {
                $marker['position'] = urlencode($infowindow);
            } else {
                $marker['position'] = "$listing->latitude, $listing->longitude";
            }
            $marker['infowindow_content'] = $infowindow; // . ', ' . ucwords(strtolower($listing->country));
            // $marker['icon'] = 'http://chart.apis.google.com/chart?chst=d_map_pin_letter&chld=A|9999FF|000000';
            if ($map_leaflet) {
                if ($map_leaflet) {
                    $marker['lat'] = $listing->latitude;
                    $marker['lng'] = $listing->longitude;
                }
                return $marker;
            } else {
                $marker['animation'] = 'DROP';
                $this->googlemaps->add_marker($marker);
            }
        }
    }
    
    /**
     * Clean Address for Google Map Marker
     *
     * Marker will not work if the address has special characters like dot, hyphen etc.
     *
     * @param string $str
     * @return string
     */
    private function clean_address($str)
    {
        $array = explode(' ', $str ?? '');
        $result = array();
        foreach ($array as $strg) {
            $slugify = ucwords($this->slug->create_slug($strg));
            $result[] = $slugify;
        }
        return implode(' ', $result);
    }
    
    /**
     * Get results based on user entered keywords
     *
     * @param number $count
     * @param string $module
     * @param string $keyword
     * @param array $where_fields: all table specific fields e.g. category, tag, city, location, featured etc. must be passed using this variable
     * extra_where must also be passed through this variable only
     *
     * @param string $sort_by
     * @param number $limit for pagination
     * @param number $offset for paginatione
     * @param array $extra: extra fields like user_id for favorites will be passed using this variable
     * Note: Filter/Additional Search Fields are also passed via $extra variable
     *
     * $type_location - the user selected a location from the typehead search result
     *            Selection Location Codes = 1 - City / 2 - State / 3 - Postcode / 4 - Locality
     * @return object
     */
    private function get_rows($count, $module, $keyword, $where_fields, $sort_by, $limit, $offset, $extra) {
        if (settings_item('site.enable_cache') == 1) {
            // Filter search term is also added in the cache id to deal with cached query issues.
            $cache_array = array(
                'count' => $count,
                'module' => $module,
                'keyword' => $keyword,
                'where_fields' => $where_fields,
                'sort_by' => $sort_by,
                'limit' => $limit,
                'offset' => $offset,
                'extra' => $extra,
                'params' => $this->input->get()
            );
            $rowsCacheID = gc_cache_id($cache_array);
            $rowsCacheID = 'search_' . str_replace('-', '_', $this->slug->create_slug($rowsCacheID));
            if (! $rows = $this->z_cache->load($rowsCacheID)) {
                $rows = array(
                    'rows' => $this->search_module($count, $module, $keyword, $where_fields, $sort_by, $limit, $offset, $extra)
                );
                $this->z_cache->save($rows, $rowsCacheID);
            }
        } else {
            $rows = array(
                'rows' => $this->search_module($count, $module, $keyword, $where_fields, $sort_by, $limit, $offset, $extra)
            );
        }
        return $rows['rows'];
    }
    
    /**
     * Pass search data to mobile API
     *
     * @param array $listings
     * @return multitype:multitype:string number boolean NULL
     */
    public function mobile_data($module, $records, $user_id = 0) {
        $result = array();
        if (! empty($records)) {
            if ($module == 'listings') {
                foreach ($records as $record) {
					if($user_id > 0) {
						$package_display_fields = !empty($record->loggedin_display_fields) ? json_decode($record->loggedin_display_fields)->listings : null;
					} else {
						$package_display_fields = !empty($record->display_fields) ? json_decode($record->display_fields)->listings : null;
					}
                    $result[] = array(
                        'id' => $record->id,
                        'title' => $record->title,
                        'image' => gc_image_url($record->logo, false, false),
                        'address' => $record->show_address == 1 && !empty($package_display_fields->address) ? $record->address . ', ' . $record->city . ' - ' . $record->postcode . ', ' . $record->state . ', ' . ucwords(strtolower($record->country)) : '',
                        'description' => strip_tags(substr($record->description, 0, 100)), // IONIC ERROR - The sanitizer was unable to parse the following block of html
                        'rating' => $record->average_rating,
                        'total_users' => $record->total_users,
                        'featured' => $record->featured,
                        'latitude' => $record->latitude,
                        'longitude' => $record->longitude,
                        'email' => !empty($package_display_fields->email) && $record->show_email == 1 ? $record->email : '',
                        'phone_number' => ! empty($record->mobile_number) ? $record->mobile_number : $record->phone_number,
                        'show_phone' => $record->show_phone,
                        'isPhone' => !empty($package_display_fields->phone_number) && $record->show_phone == 1 ? 1 : 0,
                        'isFavorite' => $record->isFavorite ? true : false,
                        'trusted_seal' => ! empty($record->trusted_seal) ? base_url('assets/images/' . $record->trusted_seal) : '',
                        'hits' => $record->hits
                    );
                }
                return $result;
            } else {
                // Unset unwanted keys
                $remove = array(
                    'slug',
                    'keywords',
                    'created_by',
                    'modified_by',
                    'created_ip',
                    'modified_ip',
                    'modified_on',
                    'meta_title',
                    'meta_keywords',
                    'meta_description',
                    'deleted',
                    'deleted_by',
                    'color_scheme',
                    'border_color'
                );
                $data = array();
                foreach ($records as $record) {
                    $row = array_diff_key((array) $record, array_flip($remove));
                    if (! empty($row['image'])) {
                        $row['image'] = gc_image_url($row['image']);
                    }
                    if (! empty($row['youtube_id'])) {
                        $row['image'] = 'https://img.youtube.com/vi/' . $row['youtube_id'] . '/0.jpg';
                    }
                    if (! empty($row['trusted_seal'])) {
                        $row['trusted_seal'] = base_url() . 'assets/images/' . $row['trusted_seal'];
                    }
                    $data[] = (object) $row;
                }
                return $this->fields_model->process_records($module, 'frontend', $data, array(), true);
            }
        }
    }
    
    /**
     * Get Location Information
     *
     * @param string $keyword
     * @param string $table
     */
    public function get_location_id($keyword, $table)
    {
        $sql = "SELECT L.id, L.name";
        switch ($table) {
            case 'cities':
                // Deal with the same name cities
                // where state is added after comma e.g. Aurangabad, Maharashtra
                $keyword = explode(', ', $keyword ?? '');
                if (isset($keyword[1])) {
                    $state = $keyword[1];
                }
                $keyword = $keyword[0];
                $sql .= ", 1 AS type";
                break;
            case 'states':
                $sql .= ", 2 AS type";
                break;
            case 'localities':
                $sql .= ", 4 AS type";
                break;
        }
        $sql .= " FROM " . $this->db->dbprefix . $table . " L";
        if (isset($state)) {
            $sql .= " INNER JOIN " . $this->db->dbprefix . "states S ON S.id = L.state_id AND MATCH(S.name) AGAINST ('" . $state . "')";
        }
        // $sql .= " WHERE MATCH(L.name) AGAINST ('". $keyword ."' IN Boolean MODE)";
        $sql .= " WHERE MATCH(L.name) AGAINST ('" . $keyword . "')";
        return $this->db->query($sql)->row();
    }
    
    /**
     * Get random values from an associative array for featured listing
     *
     * @param array $arr
     * @param number $num
     * @return multitype:array
     */
    private function array_random_assoc($arr, $num = 1)
    {
        $keys = array_keys($arr);
        shuffle($keys);
        $r = array();
        for ($i = 0; $i < $num; $i ++) {
            $r[$keys[$i]] = $arr[$keys[$i]];
        }
        return $r;
    }
    
    /**
     * Sort array based on value
     */
    function sortByPrice($x, $y) {
        return $y->price - $x->price;
    }
    
    /**
     * Pages Meta Data - Meta Title, Meta Keywords, Meta Description
     *
     * @param string $module
     * @param string $saved_value - Value Hardcoded in the DB. E.g. Category and pages meta title, meta keywords and meta description
     * @param string $name - Category/Tag Name/Search Keyword
     * @param string $default_value - Hardcoded Default Value, in case of description, category description is used
     * @param string $searchLocation
     * @param string $type
     * @param string $place - City, State and Country -> Passed in case of location or city page
     * @return string
     */
    private function _meta($module, $saved_value, $name, $default_value, $searchLocation = NULL, $type = 'title', $place = null) {
        // $name is used $keyword and $searchLocation is used as $search_city for search page meta title, description and keywords
        $page_uri = $this->get_page_uri($module);
        if(settings_item('site.default_country')) {
            $country_data = settings_item('site.default_country');
            $country_info = json_decode($country_data, true);
            if(!empty($country_info)) {
                $country = $country_info;
            } else {
                $country = $country_data;
            }
        } else {
            $country = $this->session->userdata('meta_country');
        }
        if (! empty($saved_value) && $saved_value != NULL) {
            $meta_data = $saved_value;
        } else if (settings_item('seo.' . $page_uri . '_meta_' . $type) != '') {
            $parse_data = array(
                'site_title' => settings_item('site.title'),
                'search_string' => ucwords($name),
                $page_uri . '_name' => $name,
                'search_city' => $searchLocation,
                'place' => $place
            );
            if(is_array($country)) {
                $parse_data = array_merge($parse_data, $country);
            } else {
                $parse_data['country'] = $country;
            }
            $meta_data = $this->parser->parse_string(settings_item('seo.' . $page_uri . '_meta_' . $type), $parse_data, TRUE);
        } else {
            if($type == 'description') {
                $last_space = strrpos(substr($default_value, 0, 200), ' '); // find the last space within 35 characters
                $meta_data = strip_tags(substr($default_value, 0, $last_space) . '...');
            } else {
                $meta_data = $default_value;
            }
        }
        return $meta_data;
    }
    
    /**
     * Get Page URI
     *
     * @param string $module
     * @return string
     */
    private function get_page_uri($module) {
        switch ($module) {
            case 'categories':
                $page_uri = 'category';
                break;
            case 'cities':
            case 'localities':
                $page_uri = 'location';
                break;
            case 'search':
            case 'tags':
                $page_uri = 'search';
                break;
        }
        return $page_uri;
    }
    
    /*
     * ------------------------------------------------------------------------------------------------------
     * END OF FRONTEND CATEGORY, LOCATION AND SEARCH
     * ------------------------------------------------------------------------------------------------------
     */
    
    /**
     * Insert latitude and longitude information, if kept blank by the user
     *
     * @param string $module
     * @param integer $row_id
     */
    public function addLatLng($module, $row_id, $result = array())
    {
        if (gc_module_settings($module, 'locations') == 0) {
            return false;
        }
        if (empty($result)) {
            // Get address information from the module table
            $sql = "SELECT address, latitude, longitude, city, state, country FROM " . $this->db->dbprefix . $module . " WHERE id = " . $row_id;
            $result = $this->db->query($sql)->row();
        }
        if ($result && empty($result->latitude) && empty($result->latitude)) {
            $latitude = '';
            $longitude = '';
            /**
             * Get latitude and longitude from the address
             * if not available get from city, state and country
             */
            // We define our address
            $address = $result->address . ', ' . $result->city . ', ' . $result->state . ', ' . $result->country;
            // We get the JSON results from this request
            $geo = @file_get_contents('https://maps.googleapis.com/maps/api/geocode/json?address=' . urlencode($address) . '&sensor=false');
            // We convert the JSON to an array
            $geo = json_decode($geo ?? '', true);
            // If everything is cool
            if ($geo['status'] = 'OK' && ! empty($geo['results'])) {
                // We set our values
                $latitude = $geo['results'][0]['geometry']['location']['lat'];
                $longitude = $geo['results'][0]['geometry']['location']['lng'];
            } else if ($geo['status'] = 'OK') {
                $city_state_country = $result->city . ', ' . $result->state . ', ' . $result->country;
                $city_geo = @file_get_contents('https://maps.googleapis.com/maps/api/geocode/json?address=' . urlencode($city_state_country) . '&sensor=false');
                $city_geo = json_decode($city_geo ?? '', true);
                if (! empty($city_geo) && $city_geo['status'] = 'OK' && ! empty($city_geo['results'])) {
                    // We set our values
                    $latitude = $city_geo['results'][0]['geometry']['location']['lat'];
                    $longitude = $city_geo['results'][0]['geometry']['location']['lng'];
                }
            }
            // Update module information
            if ($latitude != '' && $longitude != '') {
                $this->db->where('id', $row_id)->update($module, array(
                    'latitude' => $latitude,
                    'longitude' => $longitude
                ));
            }
        }
    }
    
    /**
     * Member module counting for listing index page
     * so that admin can come to know which of the above are not activated
     */
    public function get_member_listings($user_id, $limit = 25, $offset = 0)
    {
        $sql = "SELECT LS.id, LS.title, LS.description, LS.address, LS.postcode, LS.featured, LS.locality, LS.city, LS.state, LS.country,";
        $sql .= " IMG.url AS logo_url, LS.hits, LS.created_on, LS.status, R.average_rating, R.total_users";
        $sql .= " FROM " . $this->db->dbprefix . "listings LS";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "images as IMG ON IMG.row_id = LS.id AND IMG.module_id = " . gc_module_id('listings') . " AND IMG.type_id = 2";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "ratings R ON R.row_id = LS.id AND R.module_id = " . gc_module_id('listings');
        $sql .= " INNER JOIN " . $this->db->dbprefix . "packages P ON P.id = LS.package_id";
        /*
         * $sql .= " INNER JOIN " . $this->db->dbprefix . "countries CT ON LS.country_iso = CT.iso";
         * $sql .= " INNER JOIN " . $this->db->dbprefix . "states S ON LS.state_id = S.id";
         * $sql .= " INNER JOIN " . $this->db->dbprefix . "cities C ON LS.city_id = C.id";
         */
        $sql .= " WHERE LS.deleted = 0 AND LS.status != 3 AND LS.user_id = $user_id";
        $sql .= " GROUP BY LS.id ORDER BY LS.id DESC LIMIT " . $offset . ", " . $limit . "";
        $query = $this->db->query($sql);
        $result = $query->result();
        return $result;
    }
    
    /**
     * Mobile and Web App - Display Listing Menu in a separate page
     *
     * @param integer $listing_id
     * @param integer $user_id
     * @return array
     */
    public function listing_menu($listing_id, $user_id)
    {
        $listing = $this->listings_model->get_member_listing($listing_id, $user_id);
        $result = array();
        if ($listing) {
            $listing_detail = array(
                'id' => $listing->id,
                'title' => $listing->title,
                'logo' => gc_image_url($listing->logo_url, false, false),
                'hits' => $listing->hits,
                'created_on' => gc_date_format($listing->created_on, true),
                'status' => $listing->status
            );
            $config_modules = gc_modules_only();
            $config_modules[] = 'videos';
            $config_modules[] = 'images';
            $config_modules[] = 'branches';
            $module_count = array();
            if (! empty($config_modules)) {
                foreach ($config_modules as $config_module) {
                    $active_module = 'active_' . $config_module;
                    $inactive_module = 'inactive_' . $config_module;
                    $module_count['active_' . $config_module] = $listing->{$active_module};
                    $module_count['inactive_' . $config_module] = $listing->{$inactive_module};
                }
            }
            $module_limit = (array) json_decode($listing->module_limit ?? '');
            $result = array(
                'listing_detail' => $listing_detail,
                'module_count' => $module_count,
                'module_limit' => $module_limit,
                'pyg' => $listing->pyg
            );
        }
        return $result;
    }
    
    /**
     * Mobile App - Display module specific rows on the module name click from the Listings/Users Detail Page
     *
     * @param string $module
     * @param number $listing_id
     * @param number $limit
     * @param number $offset
     * @return multitype:boolean multitype: multitype:multitype:string listings
     */
    public function frontend_modules($module, $type, $id, $limit, $offset) {
        $this->load->model('fields/fields_model');
        if ($type == 'users') {
            $user_id = $id;
            $where = 'user_id = ' . $id;
        } else {
            $user_id = 0;
            $where = 'listing_id = ' . $id;
        }
        if ($module == 'branches') {
            $this->load->model('branches/branches_model');
            $records = $this->branches_model->get_branches(0, $id, $user_id, false, 'BR.id DESC', $limit, $offset);
            $totalrows = $this->branches_model->get_branches(1, $id, $user_id);
        } else {
            $records = $this->get_rows(0, $module, null, array('where' => $where), $_GET['sort_by'], $limit, $offset, array('user_id' => $user_id));
            $totalrows = $this->get_rows(1, $module, null, array('where' => $where), $_GET['sort_by'], $limit, $offset, array('user_id' => $user_id));
        }
        $mobile_response = array(
            'success' => true,
            'total_rows' => $totalrows,
            'top_featured' => array(),
            'records' => $this->mobile_data($module, $records)
        );
        return $mobile_response;
    }
    
    /**
     * Admin module counting for the listing index page
     * so that admin can come to know which of the above are not activated
     */
    public function get_member_listing($listing_id, $user_id) {
        $modules = gc_modules_only();
        $modules[] = 'branches';
        $sql = "SELECT L.id, L.title, IMG.url AS logo_url, L.email, L.hits, L.created_on, L.status, P.module_limit, P.module_images_limit, P.pyg,";
        foreach ($modules as $module) {
            $sql .= " (SELECT COUNT(*) FROM " . $this->db->dbprefix . $module . " M WHERE M.status = 1 AND M.deleted = 0 AND M.listing_id = $listing_id) as active_" . $module . ",";
            $sql .= " (SELECT COUNT(*) FROM " . $this->db->dbprefix . $module . " M WHERE M.status = 0 AND M.deleted = 0 AND M.listing_id = $listing_id) as inactive_" . $module . ",";
        }
        $sql .= " (SELECT COUNT(*) FROM " . $this->db->dbprefix . "videos V WHERE V.status = 1 AND V.deleted = 0 AND V.module_id = " . gc_module_id('listings') . " AND V.row_id = $listing_id) as active_videos,";
        $sql .= " (SELECT COUNT(*) FROM " . $this->db->dbprefix . "videos V WHERE V.status = 0 AND V.deleted = 0 AND V.module_id = " . gc_module_id('listings') . " AND V.row_id = $listing_id) as inactive_videos,";
        $sql .= " (SELECT COUNT(*) FROM " . $this->db->dbprefix . "images I WHERE I.status = 1 AND I.deleted = 0 AND I.type_id = 0 AND I.module_id = " . gc_module_id('listings') . " AND I.row_id = $listing_id) as active_images,";
        $sql .= " (SELECT COUNT(*) FROM " . $this->db->dbprefix . "images I WHERE I.status = 0 AND I.deleted = 0 AND I.type_id = 0 AND I.module_id = " . gc_module_id('listings') . " AND I.row_id = $listing_id) as inactive_images";
        $sql .= " FROM " . $this->db->dbprefix . "listings L";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "images as IMG ON IMG.row_id = L.id AND IMG.module_id = " . gc_module_id('listings') . " AND IMG.type_id = 2";
        $sql .= " INNER JOIN " . $this->db->dbprefix . "packages P ON P.id = L.package_id";
        $sql .= " WHERE L.id = $listing_id AND L.user_id = $user_id";
        $query = $this->db->query($sql);
        $result = $query->row();
        return $result;
    }
    
    /**
     * Display Listings to the Member Area
     *
     * @param int $count
     * @param int $user_id
     *            Integer of User ID to fetch
     * @param int $limit
     * @param int $offset
     */
    public function members($count, $user_id, $limit = 0, $offset = 0) {
        if ($count == 0) {
            $sql = "SELECT L.*, IMG.url AS logo_url, C.name AS city, SUM(CTC.counts) AS ctcsum, COUNT(CTC.id) as uniquectc";
            $sql .= ", P.title AS package_title, P.module_images_limit, P.pyg, P.module_limit";
        } else {
            $sql = "SELECT COUNT(L.id) AS total_count";
            $sql .= " FROM " . $this->db->dbprefix . "listings L";
            $sql .= " WHERE L.user_id = " . $user_id . " AND L.deleted = 0"; //  AND L.status != 3
        }
        if ($count == 0) {
            $sql .= " FROM (SELECT * FROM " . $this->db->dbprefix . "listings L";
            $sql .= " WHERE L.user_id = " . $user_id . " AND L.deleted = 0"; //  AND L.status != 3
            $sql .= ") L";
            $sql .= " LEFT JOIN " . $this->db->dbprefix . "images as IMG ON IMG.row_id = L.id AND IMG.module_id = " . gc_module_id('listings') . " AND IMG.type_id = 2";
            $sql .= " LEFT JOIN " . $this->db->dbprefix . "click_to_call CTC ON CTC.listing_id = L.id";
            $sql .= " LEFT JOIN " . $this->db->dbprefix . "packages P ON P.id = L.package_id";
            $sql .= " LEFT JOIN " . $this->db->dbprefix . "cities C ON C.id = L.city_id";
        }
        if ($count == 0) {
            $sql .= " GROUP BY L.id ORDER BY L.id DESC";
            if ($limit != 0) {
                $sql .= " LIMIT " . $offset . ", " . $limit;
            } else {
                $sql .= " LIMIT $limit";
            }
            $query = $this->db->query($sql);
            return $query->result();
        } else {
            $result = $this->db->query($sql)->row();
            return $result ? $result->total_count : 0;
        }
    }
    
    /**
     * Update User/Package ID on listing update
     * This function is also called for listing claims
     * Member Area - Package Change
     *
     * @param int $listing_id
     * @param string $field
     *            - user_id/package_id
     * @param int $value
     */
    public function update_module($listing_id, $field, $value, $module_status = array()) {
        $data = array(
            $field => $value
        );
        $modules = gc_modules_only();
        foreach ($modules as $module) {
            if($field == 'status') {
                $data['status'] = !empty($module_status[$module]) ? 1 : 0;
            }
            $this->db->where('listing_id', $listing_id)->update($module, $data);
        }
    }
    
    /**
     * Change listings and module row status on listing activation/deactivation/expiration
     *
     * Mainly added for cronjob
     *
     * @param number $listing_id
     * @param string $field
     *            - status/package_id
     * @param number $value
     *            - new value
     */
    public function update_recursive($listing_id, $field, $value)
    {
        $this->db->query("UPDATE " . $this->db->dbprefix . "listings SET " . $field . " = " . $value . " WHERE id = " . $listing_id);
        if ($field == 'status') {
            $this->db->query("UPDATE " . $this->db->dbprefix . "images SET " . $field . " = " . $value . " WHERE row_id =" . $listing_id . " AND module_id = " . gc_module_id('listings'));
        }
        $this->db->query("UPDATE " . $this->db->dbprefix . "videos SET " . $field . " = " . $value . " WHERE row_id =" . $listing_id . " AND module_id = " . gc_module_id('listings'));
        $config_modules = gc_modules_only();
        foreach ($config_modules as $config_module) {
            // get ids of all rows
            $module_rows = $this->db->select('id')
            ->get_where($config_module, array(
                'listing_id' => $listing_id
            ))
            ->result();
            if ($module_rows) {
                // modify images and videos fields accordingly
                foreach ($module_rows as $module_row) {
                    if ($field == 'status') {
                        $this->db->query("UPDATE " . $this->db->dbprefix . "images SET " . $field . " = " . $value . " WHERE row_id =" . $module_row->id . " AND module_id = " . gc_module_id($config_module));
                    }
                    $this->db->query("UPDATE " . $this->db->dbprefix . "videos SET " . $field . " = " . $value . " WHERE row_id =" . $module_row->id . " AND module_id = " . gc_module_id($config_module));
                }
            }
            // change module row_status
            $this->db->query("UPDATE " . $this->db->dbprefix . $config_module . " SET " . $field . " = " . $value . " WHERE listing_id =" . $listing_id);
        }
    }
    
    /**
     * Purge all the information related to a listing
     *
     * This method is also called on user purge
     *
     * @param int $id
     */
    public function purge_records($id)
    {
        // Delete Logo and QRFile
        $record = $this->listings_model->select('qrfile')->find($id);
        
        if (! empty($record->qrfile) && file_exists(realpath(FCPATH . $record->qrfile))) {
            unlink(realpath(FCPATH . $record->qrfile));
        }
        
        // Purge all modules related information
        $modules = gc_modules_only();
        foreach ($modules as $module) {
            $ids = array();
            $rows = $this->db->select('id')
            ->get_where($module, array(
                'listing_id' => $id
            ))
            ->result();
            if ($rows) {
                foreach ($rows as $row) {
                    $ids[] = $row->id;
                }
            }
            $this->gc->delete_purge($module, $ids, TRUE);
        }
        
        // Purge any videos for this listing, also.
        $this->gc->purge_module('videos', $id, 'listings');
        
        // Purge any images for this listing, also.
        $this->load->model('images/images_model');
        
        // Purge other images
        $this->images_model->purge_images('listings', $id, 99);
        
        // Purge any business hours for this listing, also.
        $this->db->where('listing_id', $id)->delete('business_hours');
        
        // Purge any business queries for this listing, also.
        $this->db->where(array('module_id' => 1, 'row_id' => $id))->delete('business_queries');
        
        // Purge any click to call for this listing, also.
        $this->db->where('listing_id', $id)->delete('click_to_call');
        
        // Purge any claims for this listing, also.
        $this->db->where('listing_id', $id)->delete('listing_claims');
        
        // Purge any incorrect reports for this listing, also.
        $this->db->where('listing_id', $id)->delete('listing_erroneous');
        
        // Purge any meta information for this listing, also.
        $this->load->model('fields/fields_model');
        $this->fields_model->purge_meta('listings', $id);
        
        // Purge any tags association for this listing, also.
        $this->load->model('tags/tags_model');
        $this->tags_model->purge_tag_relation('listings', $id);
        
        // Purge any categories for this listing, also.
        $this->load->model('categories/categories_model');
        $this->categories_model->purge_categories_relation('listings', $id);
        
        // Purge any listing views for this listing, also.
        $this->db->where('listing_id', $id)->delete('listing_views');
        
        // Purge any listing locations for this listing, also.
        $this->db->where('listing_id', $id)->delete('branches');
        
        // Purge any ratings associated with listing, also.
        $this->gc->purge_module('ratings', $id, 'listings');
        
        // Purge any favorites associated with listing, also
        $this->gc->purge_module('favorites', $id, 'listings');
        
        // Purge any comments associated with listing, also.
        $this->load->model('comments/comments_model');
        $this->comments_model->purge_comment('listings', $id);
        
        // Purge Translations
        $this->translations->purge_tanslations('listings', $id);
        
        // Purge listing in the end
        $this->listings_model->delete($id, TRUE);
        
        // set auto increment value to the last id
        // Making delete slow in case of thousand rows
        /* $alter_autoinc = "ALTER TABLE " . $this->db->dbprefix . "listings AUTO_INCREMENT = 1";
         $this->db->simple_query($alter_autoinc); */
        
        // Any modules needing to save data?
        Events::trigger('purge_listing', $id);
    }
    
    /**
     * Create PDF File
     *
     * @param int $listing_id
     */
    public function create_pdf($listing_id) {
        $this->gemailer->sms_email_pdf('pdf', array('template_table' => 'module_email_templates', 'module_table' => 'listings', 'action_type' => 'share', 'id' => $listing_id));
    }
    
    /**
     * Listing Status
     *
     * @param int $status
     * @return string
     */
    public function listing_status($status)
    {
        switch ($status) {
            case '1':
                $status = lang('tab_approved');
                break;
            case '2':
                $status = lang('status_expired');
                break;
            case '3':
                $status = lang('label_spammed');
                break;
            default:
                $status = lang('label_deactivated');
                break;
        }
        return $status;
    }
    
    /**
     * Common function to clone categories, images (includes logo and cover_image),
     * custom_fields and tags with new IDs
     *
     * @param string $table
     *            - Name of the table e.g. categories_relation/images/meta/tag_relations
     * @param string $module
     * @param integer $row_id
     *            - Old Row ID
     * @param integer $new_row_id
     *            - Newly Inserted Row ID
     */
    public function copy_row($module, $row_id, $new_row_id)
    {
        $tables = array(
            'categories_relation',
            'metas',
            'images',
            'tag_relations'
        );
        foreach ($tables as $table) {
            $this->clone_row($table, $module, $row_id, $new_row_id);
        }
    }
    
    private function clone_row($table, $module, $row_id, $new_row_id)
    {
        $module_id = gc_module_id($module);
        $result = $this->db->get_where($table, array(
            'module_id' => $module_id,
            'row_id' => $row_id
        ))->result();
        if ($result) {
            $data = array();
            foreach ($result as $rows) {
                $prepare_data = array();
                foreach ($rows as $key => $value) {
                    switch ($key) {
                        case 'id':
                        case 'meta_id':
                            break;
                        case 'row_id':
                            $prepare_data[$key] = $new_row_id;
                            break;
                        default:
                            $prepare_data[$key] = $value;
                    }
                }
                $data[] = $prepare_data;
            }
            $this->db->insert_batch($table, $data);
        }
    }
    
    /**
     * Admin/Member/Mobile - Common method to Add/Edit Business
     *
     * @param string $role: 'members'
     * @param number $package_id
     * @param number $user_id
     * @param number $id
     * @return multitype:boolean |multitype:boolean unknown |multitype:boolean string
     */
    public function business($role, $package_id = 0, $user_id = 0, $id = 0) {
        $module = 'listings';
        if(settings_item('site.categories_modules') == 1) {
            $this->fields_model->categories_module_form($module, $id, $role);
        } else {
            if ($role == 'members') {
                // This method has no View in the Themes Module, i.e. load layout files manually
                $this->gc->demo_custom_css_js('member');
                $this->auth->restrict('Site.Business.Owner');
            }
            
            // Member Area
            $record = array();
            $frontend = FALSE;
            $mobile = FALSE;
            if ($role == 'members' || $role == 'mobile') {
                $frontend = TRUE;
                if ($role == 'mobile') {
                    $mobile = TRUE;
                }
            }
            $redirect_url = $role == 'admin' ? site_url(SITE_AREA . '/content/listings') : gc_url_format('members');
            $this->load->model('members/members_model');
            $this->load->model('packages/packages_model');
            $this->load->model('fields/fields_model');
            // Fields like user_id, package_id, module_id, row_id can be handled via Hidden Fields
            // but they will impose security risk as the external user can modify these values before submitting the form
            // therefore, such parameters are passed in the extra_fields array
            $extra_fields = array();
            
            // Type Create/Edit
            if ($id == 0) {
                $toolbar_title = lang('header_add_business');
                $transaction_type = 'insert';
                if ($frontend) {
                    $extra_fields['user_id'] = $user_id;
                    // load information from the session variable
                    $record = (object) $this->session->userdata('user_info');
                    $record->contact_person = $record->display_name;
                    if(gc_module_settings($module, 'locations') == 1 && !empty($record->city_id)) {
                        $this->gc->location_google_places($record->city_id, true);
                    }
                    Template::set('record', $record);
                }
            } else {
                $transaction_type = 'update';
                $where = array();
                $where[] = 'id = ' . $id;
                if ($frontend) {
                    $where[] = 'deleted = 0';
                }            
                if($role == 'admin' && $this->auth->role_id() == 7) {
                    $where[] = (array) $this->session->userdata('franchise_where');
                }
                // Get existing record and pass it to the View File
                $record = $this->fields_model->find_row_and_meta($module, $id, implode(' AND ', $where));
                if (! $record) {
                    if ($mobile) {
                        return array(
                            'success' => FALSE
                        );
                    } else {
                        Template::set_message(lang('msg_permission_error'), 'error');
                        Template::redirect($redirect_url);
                    }
                }
                
                // Member Area - check permission
                if (($frontend && (int) $record->user_id != $user_id)) {
                    if ($mobile) {
                        return array(
                            'success' => FALSE
                        );
                    } else {
                        Template::set_message(lang('unauthorized_deleted'), 'error');
                        Template::redirect(gc_url_format('members'));
                    }
                }
                
                // Get Applicable Categories using package_id
                $package_id = $record->package_id;
                
                // Get Active and Inactive tags from the Tags table
                // Keyword column only has active keywords
                $this->load->model('tags/tags_model');
                if ($mobile) {
                    $record->keywords = $this->tags_model->tags_comma_separated($module, $id);
                } else {
                    $record->keywords = $this->tags_model->selectize_keywords($module, $id); // Selectize Keywords
                    
                    // Multi Langauge Fields - Get translations if exists
                    $record = $this->translations->get_translations($module, $id, $record);
                }
                
                // Set previous city session variable for tags location handling
                $this->session->set_userdata('previous_city', $record->city_id);
                
                /**
                 * Edit Page - Set the default category variable (IMPORTANT)
                 * Else page will show error on submission (click on save button twice on edit page)
                 */
                if (! $mobile) {
                    $category = $this->categories_model->get_breadcrumb($id);
                    Template::set('record', $record);
                } else { // Edit Listing - Mobile Specific
                    $record = $this->mobile_edit($module, $id, $record);
                }
                // Set page title
                $toolbar_title = lang('bf_action_edit') . ' - ' . $record->title;
            }
            
            // Add Business - Check the existing order details
            if ($frontend && $id == 0 && $package_id == 0) {
                $package_id = $this->last_ordered_package_id($user_id, $mobile);
            }
            if ($role == 'admin') { // Admin Specific
                if ($id == 0) {
                    $redirect_url = SITE_AREA . '/content/listings';
                    $toolbar_title = lang('add') . ' ' . lang('singular_' . $module);
                    // load categories tree using the first package id
                    $package = $this->db->where('deleted', 0)->get('packages')->row();
                } else {
                    $redirect_url = SITE_AREA . '/content/listings/edit/' . $id;
                    $package = $this->packages_model->find($record->package_id);
                }
                
                $cancel_url = site_url(SITE_AREA . '/content/listings');
                $packages = $this->packages_model->select('id, title')->where('deleted', 0)->find_all();
                
                // Change Categories limit on Package Change
                if ($this->input->post('package_id')) {
                    $package_info = $this->packages_model->select('form_fields, module_categories_limit')->where('deleted', 0)->find($this->input->post('package_id'));
                }
                Template::set('packages', $packages);
            } else {
                $redirect_url = gc_url_format('members');
                $cancel_url = $redirect_url;
                $package = $this->members_model->get_package_info($user_id, $package_id, $id, 'packages');
                // Display package information to the user while adding a new listing as it was before.
                if ($package) {
                    $package_title = $package->package_title . ($package->expires_on == NULL || $package->expires_on == '0000-00-00' ? ' (' . lang('label_expires_on') . ': ' . lang('label_never') . ')' : ' (' . lang('label_valid_upto') . ': ' . gc_date_format($package->expires_on, false) . ')');
                    $packages = json_decode(json_encode(array(
                        array(
                            'id' => $package_id,
                            'title' => $package_title
                        )
                    )));
                    Template::set('packages', $packages);
                }
            }
            
            // Go Back/Cancel Button Click URL
            Template::set('cancel_url', $cancel_url);
            
            if (empty($package)) { // count result if package exists
                if ($mobile) {
                    return array(
                        'status' => false,
                        'order_type' => 'packages',
                        'url' => 'app.checkout'
                    );
                } else {
                    Template::redirect($role == 'admin' ? SITE_AREA . '/content/listings' : gc_url_format('members/packages'));
                }
            }
            
            // Limit Check - Member Area Only
            // Do PYG Check only in case of Insert
            if ($frontend && $id == 0) {
                // check user permissions
                $pyg_check = $this->members_model->pyg_check($module, $user_id, $package_id, $package, 0, $module, $mobile);
                if ($mobile && ! $pyg_check['success']) {
                    return $pyg_check;
                }
                
                // Get status field information
                // In case of Admin, it will be selected by the Admin
                $extra_fields['package_id'] = $package_id;
                $extra_fields['featured'] = gc_field_value($module, $package->module_featured);
                $extra_fields['status'] = gc_field_value($module, $package->module_status);
            }
            
            // calculate expires on
            if(empty($_POST['expires_on'])) {
                if(empty($package->expires_on)) {
                    $this->load->model('orders/orders_model');
                    $extra_fields['expires_on'] = $this->orders_model->calculate_expires_on($package, 1, null);
                } else {
                    $extra_fields['expires_on'] = $package->expires_on;
                }
            }
            
            $form_fields = $this->fields_model->load_fields($module, $package, ($frontend ? 'frontend' : 'admin_only'), $record, $id, $mobile);
            
            // Perform save operation
            if (isset($_POST['save']) || isset($_POST['save_only'])) {
                $this->gcrud->stats(array('module' => $module));
                // the admin can change the listing expiration date
                if($role == 'admin' && !empty($_POST['expires_on']) && isset($extra_fields['expires_on'])) {
                    unset($extra_fields['expires_on']);
                }
                // title and description not displaying in the Admin Panel if the english translations are not filled
                /*if(settings_item('site.multi_language') == 1) {
                    $translate_fields = array('title', 'description');
                    foreach($translate_fields as $translate_field) {
                        if(empty($_POST[$translate_field . '_trlng_en'])) {
                            // get the text from the other post variable
                        }
                    }
                }*/
                if ($insert_id = $this->fields_model->save_module($form_fields, $module, $user_id, $role, $id, $transaction_type, $extra_fields)) {
                    if($id == 0) { // Insert Operation
                        $id = $insert_id;
                    }
                    $this->save_module_info($role, $id, $user_id, $transaction_type, $module, ($frontend ? $redirect_url : SITE_AREA . '/content/listings'));                
                } else {
                    $error_message = sprintf(lang('msg_create_failure'), lang('label_listing')) . $this->listings_model->error;
                    if ($mobile) {
                        $validation_errors = validation_errors();
                        if ($validation_errors) {
                            $error_message = $validation_errors;
                        }
                        return array(
                            'success' => false,
                            'message' => $error_message
                        );
                    } else {
                        Template::set_message($error_message, 'error');
                    }
                }
            } else if ($id > 0 && isset($_POST['delete'])) {
                if ($this->listings_model->delete($id)) {
                    // Log the activity
                    log_activity($user_id, sprintf(lang('msg_act_delete_record'), lang('label_listing')) . ': ' . $id . ' : ' . $this->input->ip_address(), $module);
                    
                    Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_listing')), 'success');
                    Template::redirect($frontend ? gc_url_format('members') : SITE_AREA . '/content/listings');
                } else {
                    Template::set_message(sprintf(lang('msg_delete_failure'), lang('label_listing')) . $this->listings_model->error, 'danger');
                }
            }
            if ($mobile) {
                return array(
                    'success' => true,
                    'form_fields' => $form_fields,
                    'record' => $record,
                    'categories_limit' => gc_field_value($module, $package->module_categories_limit) // categories limit
                );
            } else {
                Template::set(array(
                    'page_title' => $toolbar_title,
                    'package_id' => $package_id,
                    'module' => $module,
                    'package' => $package,
                    'toolbar_title' => $toolbar_title
                ));
                if ($role == 'members') {
                    Template::set(array(
                        'elements_container' => 'div.card.card-body', // wrap form elements within a container
                        'member_area' => TRUE // set this variable to TRUE for QUERY STRING FORM else the SAVE button will redirect without saving.
                    ));
                }
                Template::set_view('fields/form');
                Template::render($role == 'admin' ? '' : 'member');
            }
        }
    }
    
    /**
     * Get the last ordered package ID
     * All new listings will be added using this Package ID
     *
     * @param string $user_id
     * @return multitype:boolean string
     */
    public function last_ordered_package_id($user_id, $mobile = FALSE)
    {
        $order_detail = $this->members_model->get_order_details($user_id);
        // no need to display packages, if user subscribed only for one package
        if (! empty($order_detail)) {
            $package_id = $order_detail[0]['package_id'];
            return $package_id;
        } else {
            if ($mobile) {
                return 0;
            } else {
                Template::redirect(gc_url_format('members/packages'));
            }
        }
    }
    
    /**
     * Edit Module - Listing Specific
     *
     * @param string $module
     * @param integer $id
     * @param array $record
     * @return object
     */
    public function mobile_edit($module, $id, $record)
    {
        // Load Categories
        $this->load->model('categories/categories_model');
        $record->categories = $this->categories_model->get_module_categories($id, $module);
        
        if ($module == 'listings' || $module == 'branches' || gc_module_settings($module, 'locations') == 1) {
            $this->load->model('locations/countries_model');
            $this->load->model('locations/states_model');
            $this->load->model('locations/cities_model');
            $country = $this->countries_model->select('iso, name')->find($record->country_iso);
            $state = $this->states_model->select('id, name')->find($record->state_id);
            $city = $this->cities_model->select('id, name')->find($record->city_id);
            $record->country = array(
                'iso' => $country->iso,
                'name' => $country->name
            );
            $record->state = array(
                'id' => $state->id,
                'name' => $state->name
            );
            $record->city = array(
                'id' => $city->id,
                'name' => $city->name
            );
        }
        return $record;
    }
    
    /**
     * Display Listing/User related module rows to the user
     *
     * @param number $listing_id
     * @param number $user_id
     * @param object $record
     */
    public function module_info($listing_id = 0, $user_id = 0, $record = array()) {
        $tag_url_params = array(
            'tag_id' => 0,
            'location' => (isset($record->city) ? $record->city : ''),
            'location_id' => $record->city_id,
            'category_id' => (isset($record->category_id) ? $record->category_id : 0),
            'locality_id' => (isset($record->locality_id) ? $record->locality_id : 0),
            'type_location' => 1
        );
        if ($user_id != 0) {
            $config_modules = gc_modules();
        } else {
            $config_modules = gc_modules_only();
        }
        $module_info = array();
        foreach ($config_modules as $config_module) {
            if (gc_module_settings($config_module) == 1) {
                $records = $this->get_fprlv($config_module, 'recently', 50, 0, 0, $listing_id, $user_id);
                if(!empty($records)) {
                    $module_info[$config_module] = $records;
                }
                // $module_info[$config_module] = $this->layouts->widget_modules_view($config_module, $records, $tag_url_params, 250);
            }
        }
        return $module_info;
    }
    
    /**
     * Common function to add tweak fields in a specific module for specific rows
     * Can also be used in the Migrate module after Bulk Import
     *
     * @param string $module
     * @param integer $from
     * @param integer $to
     * @return boolean - For messages
     */
    public function bulk_tweak($module, $from, $to) {
        $module_id = gc_module_id($module);
        if ($module_id == 0) {
            return false;
        }
        $where = 'id BETWEEN "' . $from . '" AND "' . $to . '"';
        $records = $this->db->where($where)
        ->select('id')
        ->get($module)
        ->result();
        if ($records) {
            $this->gc->_tweak_fields($module, 'M.id BETWEEN "' . $from . '" AND "' . $to . '"');
            return true;
        }
        return false;
    }
    
    /**
     * Advanced Search
     */
    public function advanced_search($prefix = 'LS.') {
        // Clear Advanced Search
        if (isset($_POST['clear_advanced_search'])) {
            $this->session->unset_userdata('listings_advanced_search');
            Template::redirect(current_url());
        }
        
        $this->load->model('locations/countries_model');
        $this->load->model('locations/states_model');
        $this->load->model('locations/cities_model');
        $this->load->model('locations/localities_model');
        $this->load->model('packages/packages_model');
        Assets::add_js('location-selection.js');
        
        $search_session = $this->session->userdata('listings_advanced_search');
        $country_iso = $this->input->post('country_iso') ? $this->input->post('country_iso') : (! empty($search_session['country_iso']) ? $search_session['country_iso'] : '');
        $state_id = $this->input->post('state_id') ? $this->input->post('state_id') : (! empty($search_session['state_id']) ? $search_session['state_id'] : '');
        $city_id = $this->input->post('city_id') ? $this->input->post('city_id') : (! empty($search_session['city_id']) ? $search_session['city_id'] : '');
        $locality_id = $this->input->post('locality_id') ? $this->input->post('locality_id') : (! empty($search_session['locality_id']) ? $search_session['locality_id'] : '');
        
        $search_session = array();
        $search_sql = array();
        $countries = $this->countries_model->order_by('name')->find_all();
        Template::set('countries', $countries);
        if (! empty($country_iso)) {
            Template::set('selected_country', $country_iso);
            $states = $this->states_model->select('id, name')->find_all_by('country_iso', $country_iso);
            Template::set('states', $states);
            $search_session['country_iso'] = $country_iso;
            $search_sql[] = $prefix . "country_iso = '" . $country_iso . "'";
        }
        
        if (! empty($state_id)) {
            Template::set('selected_state', $state_id);
            $cities = $this->cities_model->select('id, name')->find_all_by('state_id', $state_id);
            Template::set('cities', $cities);
            $search_session['state_id'] = $state_id;
            $search_sql[] = $prefix . "state_id = " . $state_id;
        }
        
        if (! empty($city_id)) {
            Template::set('selected_city', $city_id);
            $localities = $this->localities_model->select('id, name')->find_all_by('city_id', $city_id);
            Template::set('localities', $localities);
            $search_session['city_id'] = $city_id;
            $search_sql[] = $prefix . "city_id = " . $city_id;
        }
        
        if (! empty($locality_id)) {
            $search_session['locality_id'] = $locality_id;
            $search_sql[] = $prefix . "locality_id = " . $locality_id;
        }
        
        if (! empty($search_session)) {
            $this->session->set_userdata('listings_advanced_search', $search_session);
            return $search_sql;
        }
    }
    
    /**
     * Update Cities, States, Countries and Categories counts
     * This method is called via CronJob
     * @param string $module - NULL process all types
     */
    public function update_counts($module = NULL) {
        // store listing counts only in the categories, cities, states and countries counts column
        // other module including listing counts will be stored in the module_counts table
        $this->load->model('categories/categories_model');
        if(empty($module)) {
            $this->db->truncate('module_counts'); // truncate module_counts table each time to avoid any ambiguity
        } else {
            $module_id = gc_module_id($module);
            $this->db->where('module_id', $module_id)->delete('module_counts');
        }
        $modules = empty($module) ? array('categories', 'localities', 'cities', 'states', 'countries') : array($module);
        $data = array();
        $unique_rows = array();
        foreach($modules as $module) {
            $module_id = gc_module_id($module);
            $toggle_modules = gc_site_modules();
            foreach($toggle_modules as $toggle_module) {
                $toggle_module = $toggle_module['module'];
                $toggle_module_id = gc_module_id($toggle_module);
                if($module == 'categories') {
                    if($toggle_module != 'branches') { // Branches has no categories
                        $records = $this->categories_model->get_counts($toggle_module_id);
                        if($records) {
                            $listing_data = array();
                            foreach($records as $record) {
                                if($record->TotalCounts != 0) {
                                    $unique_row_id = $module_id . '-' . $record->id . '-' . $toggle_module_id;                                    
                                    if(!in_array($unique_row_id, $unique_rows)) {
                                        $data[$unique_row_id] = array(
                                            'module_id' => $module_id,
                                            'row_id' => $record->id,
                                            'toggle_module_id' => $toggle_module_id,
                                            'counts' => $record->TotalCounts
                                        );
                                        $unique_rows[$unique_row_id] = $unique_row_id;
                                    }
                                    $listing_data[] = array(
                                        'id' => $record->id,
                                        'counts' => $record->TotalCounts
                                    );
                                }
                            }
                            if($toggle_module == 'listings') { // update counts information in the listings table
                                // reset hits to 0 to avoid any ambiquity
                                $this->db->update($module, array('counts' => 0));
                                if(!empty($listing_data)) {
                                    // update counter
                                    $this->db->update_batch($module, $listing_data, 'id');
                                }
                            }
                        }
                    }
                } else {
                    if (gc_module_settings($toggle_module, 'locations') == 1) {
                        // do not process counts if the location is not enabled for the module
                        $field_name = 'city_id';
                        $table_field_name = 'id';
                        switch($module) {
                            case 'countries':
                                $field_name = 'country_iso';
                                $table_field_name = 'iso';
                                break;
                            case 'states':
                                $field_name = 'state_id';
                                break;
                            case 'localities':
                                $field_name = 'locality_id';
                                break;
                        }
                        // get only unique ids
                        $records = $this->db->select('DISTINCT(' . $field_name . ') AS id')->get($toggle_module)->result();
                        if($records) {
                            $listing_data = array();
                            foreach($records as $record) {
                                if(!empty($record->id)) { // Locality can be null or 0
                                    $counts = $this->db->where(array($field_name => $record->id))->count_all_results($toggle_module);
                                    $row_id = $record->id;
                                    if($module == 'countries') {
                                        // get country id to save it as the row_id
                                        $row = $this->db->where('iso', $record->id)->get($module)->row();
                                        if($row) {
                                            $row_id = $row->id;
                                        }
                                    }
                                    if(!empty($row_id)) {
                                        $unique_row_id = $module_id . '-' . $row_id . '-' . $toggle_module_id;
                                        if(!in_array($unique_row_id, $unique_rows)) {
                                            $data[$unique_row_id] = array(
                                                'module_id' => $module_id,
                                                'row_id' => $row_id,
                                                'toggle_module_id' => $toggle_module_id,
                                                'counts' => $counts
                                            );
                                            $unique_rows[$unique_row_id] = $unique_row_id;
                                        }                                        
                                        $listing_data[] = array(
                                            $table_field_name => $record->id,
                                            'counts' => $counts
                                        );
                                    }
                                }
                            }
                            if($toggle_module == 'listings') { // update counts information in the listings table
                                // reset hits to 0 to avoid any ambiquity
                                $this->db->update($module, array('counts' => 0));
                                // update counter
                                if(!empty($listing_data)) {
                                    $this->db->update_batch($module, $listing_data, $table_field_name);
                                }
                            }
                            if(!empty($data)) { // update module counts table
                                // $this->db->on_duplicate('module_counts', $data);
                            }
                        }
                    }
                }
            }
        }
        if(!empty($data)) { // update module counts table
            $this->db->truncate('module_counts'); // Once again truncate module_counts table before insert
            $this->db->insert_batch('module_counts', $data);
        }
    }
    
    /**
     * Display counts to the user
     * @param int $row_id
     * @param string $module
     * @return number[]|NULL[]
     */
    public function display_counts($row_id, $module = 'categories') {
        $info = $this->db->where('id', $row_id)->get($module)->row();
        if(!$info) {
            return false;
        }
        $result = array();
        $site_modules = gc_site_modules();
        foreach($site_modules as $site_module) {
            $result[$site_module['id']] = 0;
        }
        if($module == 'categories') {
            unset($result[gc_module_id('branches')]);
        }
        $module_id = gc_module_id($module);
        if($module == 'categories') {
            $module_counts = json_decode($info->module_counts ?? '', true);
            if(!empty($module_counts)) {
                foreach($module_counts as $k => $v) {
                    $result[gc_module_id($k)] = $v;
                }
            }
        } else {
            $records = $this->db->where(array('module_id' => $module_id, 'row_id' => $row_id))->get('module_counts')->result();
            if($records) {
                foreach($records as $record) {
                    if(array_key_exists($record->toggle_module_id, $result)) {
                        $result[$record->toggle_module_id] = $record->counts;
                    }
                }
            }
        }
        Template::set(array(
            'module' => $module,
            'redirect_url' => $this->session->userdata('last_visited_url'),
            'toolbar_title' => $info->name . ' - ' . lang('label_total'),
            'result' => $result
        ));
        Template::set_view('listings/counts');
        Template::render();
    }
    
    /**
     * Member Area- Preview Printed Version PDF File
     */
    public function preview_printed() {
        $this->lang->load('listings/listings');
        $records = $this->db->where('user_id', $this->auth->user_id())->get('listings')->result();
        $file_header = 'Sample Printed Preview of Your Businesses';
        $filename = 'printed_version_' . date('Y_m_d_H_i_s');
        $this->load->library('format');
        $html_content = $this->load->view('content/tpl_pdf', array('records' => $records, 'file_header' => $file_header), true);
        $this->format->to_dompdf(array('filename' => $filename, 'content' => $html_content));	
    }
    
    /**
     * Distance specific latitude and longitude information
     * @param string $module
     * @param string $block
     * @param number $limit
     * @return array
     */
    public function get_fprlv($module, $block, $limit, $city_id = 0, $id = 0, $listing_id = 0, $user_id = 0) {
        $sortBy = 'id'; // recently added        
        $offset = 0;
        $where_fields = array();
        switch($block) {
            case 'popular':
                $sortBy = 'popular';
                break;
            case 'home_featured':
                if(settings_item('lst.home_featured_listings') != 0) {
                    // To deal with typing mistakes made by user
                    $ids = array_map('trim', explode(',', settings_item('lst.home_featured_listings')));
                    $ids = array_filter($ids, 'strlen'); // Remove Empty values
                    $ids = implode(',', $ids);
                    
                    $where_fields['where'] = "id IN (" . $ids . ")";
                    $sortBy ='rand';
                    //$sql .= " AND id >= (SELECT FLOOR( MAX(id) * RAND()) FROM ". $this->dbprefix . $module ." WHERE id IN (" . $ids . "))"; // display rows on random basis
                    //$sql .= " ORDER BY FIELD(id, " . $ids . ")";
                } else {
                    $where_fields['featured'] = 1;
                    // uncomment the line below for smaller database, do not forget to comment the OFFSET parameter below
                    // $sql .= " AND id >= (SELECT FLOOR( MAX(id) * RAND()) FROM ". $this->dbprefix . $module .")"; // display rows on random basis
                }
                break;
            case 'featured':
                $sortBy ='rand'; // Slowest
                // The below given query is not returning the correct numbers - sometimes it returns only one record
                //$sql .= " AND id >= (SELECT FLOOR( MAX(id) * RAND()) FROM ". $this->dbprefix . $module .")"; // display rows on random basis
                break;
            case 'recently':
                // Records displayed on the Listing and User's Detail page are also retrieved through this method
                // i.e. $user_id and $listing_id parameters are passed
                // if they are not empty then display only specific information
                // Note: The Block param value must be set to recently for Listing and User's specific records.
                if($user_id != 0) {
                    $where_fields['where'] = "user_id = " . $user_id;
                }
                if($listing_id != 0 && $module != 'listings') {
                    $where_fields['where'] = (!empty($where_fields['where']) ? " AND M." : "") . "listing_id = " . $listing_id;
                }
                break;                
            case 'related':
                if($id != 0) {
                    $where_fields['where'] = "id != " . $id;
                    $hasLocation = gc_module_settings($module, 'locations') == 1 ? true : false;
                    if($hasLocation && $city_id != 0) {
                        $where_fields['where'] .= " AND M.city_id = " . $city_id;
                    }
                    if($listing_id > 0) {
                        $where_fields['where'] .= " AND M.listing_id = " . $listing_id;
                    }
                    // uncomment the line below for smaller database, do not forget to comment the OFFSET parameter below
                    // $sql .= " AND id >= (SELECT FLOOR( MAX(id) * RAND()) FROM ". $this->dbprefix . $module .")"; // display rows on random basis
                }
                break;
        }
        $fields_extra = array(
            'filter_search_fields' => array()
        );
        if(!empty($_GET['featured']) || $block == 'featured') { // view featured listings only
            $where_fields['featured'] = 1;
        }
        // Order businesses based on users current location
        if(!empty($_GET['lat']) && !empty($_GET['lng'])) {
            $fields_extra['latitude'] = $_GET['lat'];
            $fields_extra['longitude'] = $_GET['lng'];
            $fields_extra['radius'] = 50; // display within a range of 50 kms
        } else if(!empty(settings_item('site.geo_radius')) && $this->session->userdata('geo_visitor_info')) {
            $geo_visitor_info = $this->session->userdata('geo_visitor_info');
            if(!empty($geo_visitor_info['latitude']) && !empty($geo_visitor_info['longitude'])) {
                $fields_extra['latitude'] = $geo_visitor_info['latitude'];
                $fields_extra['longitude'] = $geo_visitor_info['longitude'];
                $fields_extra['radius'] = !empty($geo_visitor_info['radius']) ? $geo_visitor_info['radius'] : floatval(settings_item('site.geo_radius')); // display within a range of 50 kms
            }
        }
        // get_rows($count, $module, $keyword, $country, $location, $location_id, $postcode, $type_location, $searched_category, $locality_id, $tag_id, $featured, $sortBy, $where, $limit, $offset, $fields, $user_id)
        $records = $this->get_rows(0, $module, null, $where_fields, $sortBy, $limit, $offset, $fields_extra);
        return $records;
    }
    
    /**
     * Listing Statuses
     * @param number $status
     * @return string
     */
    public function listing_statuses() {
        $statuses = array(
            0 => lang('tab_inactive'),
            1 => lang('tab_approved'),
            2 => lang('status_expired'),
            3 => lang('label_spammed')
        );
        return $statuses;
    }
    
    /**
     * Module Save - Common function for all modules, mobiles, frontend and Admin Area
     * @param string $role
     * @param number $id
     * @param number $user_id
     * @param string $transaction_type
     * @param string $module
     * @param string $redirect_url
     * @return boolean[]|string[]
     */
    public function save_module_info($role, $id, $user_id, $transaction_type, $module, $redirect_url) {
        $frontend = FALSE;
        $mobile = FALSE;
        if ($role == 'members' || $role == 'mobile') {
            $frontend = TRUE;
            if ($role == 'mobile') {
                $mobile = TRUE;
            }
        }
        
        // prevent spam
        $this->prevent_spam($id, $module);
        
        // Perform below given operations only on listing create
        if($transaction_type == 'insert') {
            if ($role == 'admin' && $module == 'listings') {
                // Save statistics
                $this->load->library('statistics');
                $this->statistics->info();
            } else {
                // Earn Points
                $this->gc->earn_points($module, $user_id, $id);
            }
            if($module == 'listings') { // Generate QRCode
                $this->gc->generate_qrcode($id);
            }
            
            // Send email to Admin
            if ($frontend) { // Notify Owner
                $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'module_email_templates', 'module_table' => $module, 'action_type' => 'add', 'id' => $id)); // , 'with_default' => 1
            }
            
            // Add Latitude and Longitude information if kept blank by the user
            $this->addLatLng($module, $id);
        } else {
            if($transaction_type == 'update' && $module == 'products') { // Back in Stock Notification
                $this->load->model('orders/orders_model');
                $this->orders_model->back_in_stock($id, $module);
            }
            if($module == 'listings') { // Update QRCode
                $this->gc->generate_qrcode($id, $transaction_type);
                if ($role == 'admin') {
                    // Update modules User/Package ID
                    // Used in Member Area for count operations
                    $updated_listing = $this->listings_model->select('user_id, package_id')->find($id);
                    $this->listings_model->update_module($id, 'user_id', $updated_listing->user_id);
                    $this->listings_model->update_module($id, 'package_id', $updated_listing->package_id);
                }
                // update completion percentage
                $this->load->model('groups/groups_model');
                $this->groups_model->percent_completion($module, $id, $role, false);
            }
        }
        
        // Allow admin to Notify Business Owner
        if ($role == 'admin' && $this->input->post('notify_user')) {
            $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'module_email_templates', 'module_table' => $module, 'action_type' => 'edit', 'with_default' => 1, 'id' => $id));
        }
        
        // check for the objectionable words
        if(settings_item('site.international') == 1) {
            $this->gc->is_objectionable($module, $id);
        }
        
        // Log the activity
        log_activity($user_id, sprintf(lang('msg_act_create_record'), lang('singular_' . $module)) . ': ' . $id . ' : ' . $this->input->ip_address(), $module);
        $success_message = sprintf(lang('msg_save_success'), lang('singular_' . $module));
        if ($mobile) {
            return array(
                'success' => true,
                'message' => $success_message
            );
        } else {
            // save translations - done in fields_model.php
            // $this->translations->save_translations($module, $id, $id);
            
            Template::set_message($success_message, 'success');
            if(isset($_POST['save'])) {
                $redirect_url = $this->session->userdata($module . '_last_visited_page') ? $this->session->userdata($module . '_last_visited_page') : $redirect_url;
                Template::redirect($redirect_url);
            } else {
                Template::redirect(current_url());
            }
        }
    }
    
    /**
     * Check whether a specific busines is currently open or not
     * @param number $id - ID of the Row
     * @param string $column - Name of the column
     * @return object
     */
    public function is_open($id, $column = 'user_id') {
        $days = array('monday' => 1, 'tuesday' => 2, 'wednesday' => 3, 'thursday' => 4, 'friday' => 5, 'saturday' => 6, 'sunday' => 7);
        $current_day = $days[strtolower(date('l'))];
        $sql = "SELECT M.id, M.is_open, ";
        $sql .= " (SELECT COUNT(*) FROM " . $this->db->dbprefix . "business_hours WHERE listing_id = M.id AND day_of_week = " . $current_day . " AND (CURTIME() >= open_time OR CURTIME() <= close_time)) AS open_status";
        $sql .= " FROM " . $this->db->dbprefix . "listings M WHERE M." . $column . " = " . $id;
        return $this->db->query($sql)->row();
    }
    
    /**
     * Get Module Address from the Business Page
     * @param string $module
     * @param number $listing_id
     * @param array $extra_fields
     * @return array
     */
    public function module_address_info($module, $listing_id, $extra_fields = array()) {
        switch($module) { // get address detail from the listing id
            case 'jobs':
            case 'properties':
            case 'deals':
            case 'products':
                $this->load->model('listings/listings_model');
                $listing_info = $this->listings_model->get_listing_info($listing_id);
                if($listing_info) {
                    $extra_fields['country_iso'] = $listing_info->country_iso;
                    $extra_fields['state_id'] = $listing_info->state_id;
                    $extra_fields['city_id'] = $listing_info->city_id;
                    $extra_fields['locality_id'] = $listing_info->locality_id;
                    $extra_fields['country'] = $listing_info->country;
                    $extra_fields['state'] = $listing_info->state;
                    $extra_fields['city'] = $listing_info->city;
                    $extra_fields['locality'] = $listing_info->locality;
                    $extra_fields['address'] = $listing_info->address;
                    $extra_fields['postcode'] = $listing_info->postcode;
                    $extra_fields['latitude'] = $listing_info->latitude;
                    $extra_fields['longitude'] = $listing_info->longitude;
                    $extra_fields['location'] = $listing_info->address . ', ' . $listing_info->city . ', ' . $listing_info->state . ' - ' . $listing_info->postcode . ', ' . $listing_info->country . ' - #' . $listing_info->city_id . 'CT,#' . $listing_info->state_id . 'ST,#' . $listing_info->country_iso . 'CO';
                    $extra_fields['package_priority'] = $listing_info->package_priority;
                    if($module != 'jobs') {
                        $extra_fields['categories'] = $listing_info->categories;
                        $extra_fields['categories_set'] = $listing_info->categories_set;
                    }
                }
                break;
        }
        return $extra_fields;
    }
    
    
    /**
     * Set default values if not set by the user e.g. price, offer_price cannot be null in case of products
     * @param string $module
     */
    public function module_default_values($module) {
        // default values
        switch($module) {
            case 'products':
                if(empty($_POST['price'])) {
                    $_POST['price'] = 0;
                }
                if(empty($_POST['offer_price'])) {
                    $_POST['offer_price'] = 0;
                }
                break;
            case 'deals':
                if(empty($_POST['sort_order'])) {
                    $_POST['sort_order'] = 0;
                }
                break;
        }
    }
    
    /**
     * Get Listing Info for Jobs and other module pages
     * @param number $listing_id
     * @return object
     */
    public function get_listing_info($listing_id, $column = 'id') {
        return $this->db->where($column, $listing_id)->get($this->table_name)->row();
    }
    
    /**
     * Ajax Rerieve Module Information
     */
    public function ajax() {
        $this->load->library('template');
        $this->load->library('layouts');
        $filter_where = array('deleted = 0');
        $filter_data = array();
        $filter = $this->input->get('f');
        $searchterm = $this->input->get('q');
        $category_id = $this->input->get('cg');
        $tag_id = $this->input->get('t');
        $location = $this->input->get('l');
        $location_type = $this->input->get('tl');
        $offset = intval($this->input->get('per_page'));
        $page = (int) $this->input->get('page'); // infinite scroll
        $limit  = $this->settings_lib->item('site.front_list_limit') ?: 15;
        $module_id = (int) $this->input->get('md');
        // search_module is set in the themes\default\widgets\search\_fields.php
        $module = empty($module_id) ? ($this->session->userdata('search_module') ? gc_module_name($this->session->userdata('search_module')) : 'listings') : gc_module_name($module_id);
        if(empty($offset) && !empty($page) && $page > 1) {
            $offset = $limit * ($page - 1); // -1 to retrieve correct records
        }
        // get filter fields from the database
        $filter_fields = $this->get_filter_fields($category_id);
        if(!empty($filter)) {
            foreach($filter as $field => $value) {
                if(strpos($field, '_min') !== false || strpos($field, '_max') !== false) {
                    $column = explode('_', $field);
                    if($column[0] == 'price') {
                        // convert price back to the original value
                        $value = currency_format($value, array('reverse_conversion' => 1));
                    }
                    $filter_data[$column[0]][$column[1]] = (int) $value;
                } else {
                    $filter_data[$field] = $value;
                }
            }
            foreach($filter_data as $key => $value) {
                if(!empty($filter_fields[$key])) { // the field must exist in the $filter_fields array
                    if(!empty($filter_fields[$key]['category_id'])) { // meta table search
                        $column = 'MT.meta_value';
                        $extra_where = " AND MT.meta_key = '" . $key . "'";
                    } else {
                        $column = 'M.' . $key;
                        $extra_where = '';
                    }
                    $where_filter = '';
                    if(is_array($value)) {
                        if(!empty($value['max'])) { // !empty($value['min']) && - min value can be zero
                            $where_filter = '(' . $column . ' BETWEEN ' . $value['min'] . ' AND '. $value['max']. ')';
                        } else if(isset($value[0]) && is_numeric($value[0])) {
                            $where_filter = $column . ' IN (' . implode(',', $value) . ')';
                        } else {
                            if(count($value) == 1) {
                                // can be a string value
                                $where_filter = "FIND_IN_SET ('" . implode(',', $value) . "', " . $column . ")";
                            } else {
                                $where_filter = "FIND_IN_SET (" . implode(',', $value) . ", " . $column . ")";
                            }
                        }
                    } else if(!empty($value)) {
                        if(is_numeric($value)) {
                            $where_filter = $column . ' IN (' . $value . ')';
                        } else {
                            $where_filter = "FIND_IN_SET ('" . $value . "', " . $column . ")";
                        }
                    }
                    if(!empty($where_filter)) {
                        $filter_where[] = $where_filter . $extra_where;
                    }
                }
            }
        }
        // get filter fields
        $fields = $this->fields_model->filter($module, false);
        $user_id = $this->auth->user_id();
        
        // extra parameters to be passed
        $fields_extra = array(
            'filter_search_fields' => $fields,
            'user_id' => $user_id
        );
        
        // Order businesses based on users current location
        if(!empty($_GET['lat']) && !empty($_GET['lng'])) {
            $fields_extra['latitude'] = $_GET['lat'];
            $fields_extra['longitude'] = $_GET['lng'];
            $fields_extra['radius'] = !empty($_GET['radius']) ? $_GET['radius'] : 25000; // search within 50 kilometers of radius
        }
        $where_fields = array();
        if(!empty($category_id)) {
            $where_fields['category_id'] = $category_id;
        }
        if(!empty($tag_id)) {
            $where_fields['tag_id'] = $tag_id;
        }
        if(!empty($location)) {
            if(is_numeric($location) && $location_type == 1) {
                $where_fields['city_id'] = $location;
            } else {
                $where_fields['location'] = escape($location);
            }
        }
        $keyword = $searchterm;
        $sortBy = 'M.id DESC';
        if(!empty($filter_where)) {
            $where_fields['where'] = implode(' AND ', $filter_where);
        }
        // load demo content only
        if(settings_item('site.demo_mode') == 1 && $this->session->userdata('theme_demo')) {
            $theme_demo = $this->session->userdata('theme_demo');
            if(!empty($theme_demo['content'])) {
                $where_fields['where'] .= "FIND_IN_SET ('" . $theme_demo['content'] . "', M.meta_keywords)";
            }
        }
        
        /**
         * Apply limit and offset and get remaining rows
         * Get page banners and footer pages link
         */
        $records = $this->get_rows(0, $module, $keyword, $where_fields, $sortBy, $limit, $offset, $fields_extra);
        $total_rows = $this->get_rows(1, $module, $keyword, $where_fields, $sortBy, $limit, $offset, $fields_extra);
        //$img_class = !empty($img_class) ? $img_class : 'img-crop-center thumb-h300';
        // $records = $this->gcrud->get_all_crud(0, $query_where, 'M.id DESC', $limit, $offset, array('module' => $module, 'where' => 'deleted = 0', 'view' => true));
        // $total_rows = $this->gcrud->get_all_crud(1, $query_where, 'M.id DESC', $limit, $offset, array('module' => $module, 'where' => 'deleted = 0', 'view' => true));
        $result['html'] = '';
        if($records) {
            $records = $this->layouts->widget_modules_view($module, $records, null);
            /*foreach($records as $key => $record) {
                $records[$key] = $this->layouts->widget_modules_view($module, $record, null);
            }*/
            /**
             * Ajax layout file id _grid_list_02 (2) is passed using a variable data-id in the main-shop-page div
             * Theme view is also used using comments to process the js and css content
             * Note: The file name must be _grid_list_<number> format
             *
             * echo theme_view('widgets/sliders/_slider_widget_02'); // to process css only
             */
            $view_id = (int) $this->input->get('view_id');
            $view_id = empty($view_id) ? '01' : append_zero($view_id);
            $result['html'] = theme_view('widgets/sliders/_slider_widget_' . $view_id, array('module' => $module, 'records' => $records), true);
        } else if(empty($offset)) { //display no record found message
            $html = theme_view('views/modules/common/view/tpl/_no_record_found', array('tpl' => 2), true);
            $result['html'] = $html;
        } else {
            // infinitescroll - return a 404 error on no more pages
            $this->output->set_status_header('404');
        }
        $result['total_rows'] = $total_rows;
        // Assets::add_js("$('#input-5').rating({clearCaption: 'No stars yet'});", 'inline');
        // header('Content-Type: application/x-json; charset=utf-8');
        echo json_encode($result);
        return true;
    }
    
    /**
     * Get module filter fields to display it on the view page
     * @param number $category_id - Display category specific filter fields
     * @param string $module
     * @return array
     */
    public function get_filter_fields($category_id = 0, $module = 'listings') {
        $filter_fields = array();
        // get common filter fields
        /**
         * Filter field extra
         * Brands: filter_type:checkbox;filter_priority:0
         * Slider Field: filter_type:slider;slider_step:1;filter_priority:0
         * Ratings Field: filter_type:slider;slider_step:1;slider_min:0;slider_max:5;filter_priority:0
         *
         * The field_priority key sorts all fields by priority
         */
        $fields_filter = $this->fields_model->get_fields($module, 'filter', array(), 0);
        if($fields_filter) {
            foreach($fields_filter as $field_filter) {
                $field_filter['type'] = !empty($field_filter['filter_type']) ? $field_filter['filter_type'] : $field_filter['type'];
                switch($field_filter['name']) {
                    /*case 'ratings': // slider is used because of min and max logic
                     $options_filter = array();
                     if(!empty($product_filter_field['options'])) {
                     foreach($product_filter_field['options'] as $option_filter) {
                     $options_filter[$option_filter['value']] = !empty($product_filter_field['options_lang']) ? lang($option_filter['name']) : $option_filter['name'];
                     }
                     }
                     $filter_options = $options_filter;
                     break;*/
                    default:
                        $filter_options = $this->filter_options($field_filter, false, $module);
                }
                $filter_fields[$field_filter['name']] = array('type' => $field_filter['type'], 'name' => $field_filter['name'], 'label' => $field_filter['label'], 'label_lang' => $field_filter['label_lang'], 'field_id' => $field_filter['id'], 'category_id' => 0, 'options' => $filter_options, 'search_filter' => 2, 'priority' => !empty($field_filter['filter_priority']) ? $field_filter['filter_priority'] : 0);
            }
        }
        if(!empty($category_id)) {
            // get category specific filter fields
            $categories_filter_fields = $this->fields_model->get_fields($module, 'frontend', array(), 0, array('categories' => $category_id, 'toggle_module' => $module, 'filter_search' => true));
            if($categories_filter_fields) {
                foreach($categories_filter_fields as $categories_filter_field) {
                    // set different field types and display filters based on the options field value
                    // e.g. {"search_filter":1,"filter_type":"checkbox"}
                    if(!empty($categories_filter_field['category_options'])) {
                        $category_field_options = json_decode($categories_filter_field['category_options'], true);
                        if(!empty($category_field_options['search_filter'])) {
                            // get field name from the categories field
                            $category_field = explode('CF', $categories_filter_field['name'] ?? '');
                            $categories_field_name = $category_field[2];
                            if(!array_key_exists($categories_field_name, $filter_fields)) {
                                // set filter_type in the categories_options field
                                $field_type = 'checkbox';
                                if(!empty($category_field_options['filter_type'])) {
                                    $field_type = $category_field_options['filter_type'];
                                }
                                $categories_filter_field['name'] = $categories_field_name;
                                $categories_filter_field['type'] = $field_type;
                                $filter_options = $this->filter_options($categories_filter_field, true, 'product_metas');
                                $filter_fields[$categories_field_name] = array('type' => $field_type, 'name' => $categories_field_name, 'label' => $categories_filter_field['label'], 'label_lang' => $categories_filter_field['label_lang'], 'field_id' => $category_field[1], 'category_id' => $category_field[0], 'options' => $filter_options, 'search_filter' => $category_field_options['search_filter'], 'priority' => !empty($category_field_options['filter_priority']) ? $category_field_options['filter_priority'] : 0);
                            }
                        }
                    }
                }
            }
        }
        if(!empty($filter_fields)) {
            // sort array by priority field and maintain the keys
            uasort($filter_fields, function($a, $b) {
                return $a['priority'] <=> $b['priority'];
            });
        }
        return $filter_fields;
    }
    
    /**
     * Get Filter Options
     * @param array $filter_field
     * @param boolean $is_category
     * @return array[]
     */
    private function filter_options($filter_field, $is_category = false, $module = 'listings') {
        $dbprefix = $this->db->dbprefix;
        $field_filter_options = array();
        // get filter options on the products page
        if($filter_field['type'] == 'slider') {
            if(isset($filter_field['slider_min']) || isset($filter_field['slider_max'])) {
                // e.g. filter_type:slider;slider_step:1;slider_min:0;slider_max:5;filter_priority:0
                $field_filter_options['min'] = !empty($filter_field['slider_min']) ? $filter_field['slider_min'] : 0;
                $field_filter_options['max'] = !empty($filter_field['slider_max']) ? $filter_field['slider_max'] : 100;
            } else {
                // get minimum and maximum values
                if($is_category) {
                    $sql = "SELECT MIN(meta_value) AS min, MAX(meta_value) AS max FROM " . $dbprefix . $module . " WHERE meta_key = '" . $filter_field['name'] . "'";
                    $record = $this->db->query($sql)->row();
                } else {
                    $sql = "SELECT MIN(" . $filter_field['name'] . ") AS min, MAX(" . $filter_field['name'] . ") AS max FROM " . $dbprefix . $module;
                    $record = $this->db->query($sql)->row();
                }
                // slider options
                // set field_extra field value to ::::: filter_type:slider;slider_step:1
                $field_filter_options['min'] = $record->min;
                $field_filter_options['max'] = $record->max;
            }
            if(isset($filter_field['slider_step'])) {
                $field_filter_options['step'] = $filter_field['slider_step'];
            } else {
                $field_filter_options['step'] = 1;
            }
        } else { // similarly colors and other format types can also be specified
            // get unique values from the products table to avoid further lookup
            // add indexes to fasten the search queries
            if($is_category) {
                $sql = "SELECT DISTINCT(meta_value) AS filter_option FROM " . $dbprefix . $module . " WHERE meta_key = '" . $filter_field['name'] . "'";
                $records = $this->db->query($sql)->result();
            } else {
                $sql = 'SELECT DISTINCT(' . $filter_field['name'] . ') AS filter_option FROM ' . $dbprefix . $module;
                if($filter_field['name'] == 'brand_id') {
                    $ids = $this->gc->demo_filter_ids('product_brands', 'extras');
                    if(!empty($ids)) {
                        $sql .= " WHERE id IN (" . $ids . ")";
                    }
                }
                $records = $this->db->query($sql)->result();
            }
            // get all available options       
            $field_dropdown_options = $this->fields_model->field_dropdown_options($filter_field['options'], $filter_field['options_lang']);
            if($records) {
                $field_filter_options = array();
                foreach($records as $record) {
                    $filter_options = explode(',', $record->filter_option ?? ''); // field can be a comma separated value in case of multi selection field
                    foreach($filter_options as $filter_option) {
                        // get unique values
                        if(!empty($field_dropdown_options) && array_key_exists($filter_option, $field_dropdown_options)) {
                            $field_filter_options[$filter_option] = $field_dropdown_options[$filter_option]; // name of the option
                        } else {
                            $field_filter_options[$filter_option] = $filter_option;
                        }
                    }
                }
            }            
            if(!empty($field_filter_options)) {
                $field_filter_options = array_merge(array('' => lang('placeholder_select')), $field_filter_options);
            }
        }
        return $field_filter_options;
    }
    
    /**
     * Common function to soft delete module rows
     * @param string $module
     * @param number $id
     * @param string $role
     */
    public function delete_soft($module, $id, $role = 'mobile') {
        $result = $this->db->where('id', $id)->update($module, array('deleted' => 1, 'modified_on' => date('Y-m-d H:i:s'), 'modified_ip' => $this->input->ip_address()));
        if ($result) {
            // Also set the Business Queries, Favorites and Comments status to deleted
            $this->gc->module_soft_delete($module, $id);
            $message = sprintf(lang('msg_action_success'), lang('singular_' . $module), lang('label_deleted'));
            if($role == 'mobile') {
                return array('success' => true, 'message' => $message);
            } else {
                Template::set_message($message, 'success');
            }
        } else {
            $message = sprintf(lang('msg_delete_failure'), lang('singular_' . $module));
            if($role == 'mobile') {
                return array('success' => false, 'message' => $message);
            } else {
                Template::set_message($message, 'danger');
            }
        }
    }
    
    /**
     * Display Summary
     * @return array
     */
    public function summary($module) {
        $table = $module;
        $where = '';
        if($module == 'listings' && $this->auth->role_id() != 1) {
            $where = " AND franchise_id = " . $this->auth->user_id();
        }
        $sql = "SELECT COUNT(*) AS total_active";
        
        // today rows
        $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $table;
        $sql .= " WHERE YEAR(created_on) = YEAR(NOW()) AND MONTH(created_on) = MONTH(NOW()) AND DAY(created_on) = DAY(NOW())" . $where .") AS today_rows";
        
        // this month rows
        $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $table;
        $sql .= " WHERE MONTH(created_on) = MONTH(CURDATE()) AND YEAR(created_on)  =  YEAR(CURDATE())" . $where .") AS month_rows";
        
        // yesterday rows
        $sql .= ", (SELECT COUNT(*) FROM " . $this->db->dbprefix . $table;
        $sql .= " WHERE created_on <= NOW() AND created_on = DATE_ADD(Now(),interval - 1 day)" . $where .") AS yesterday_rows";
        $sql .= " FROM " . $this->db->dbprefix . $table;
        $sql .= " WHERE status = 1" . $where;
        $query = $this->db->query($sql);
        $result = $query->row();
        foreach($result as $key => $value) {
            $result->{$key} = $value;
        }
        return $result;
    }
    
    /**
     * Get Records For Reporting
     * @param string $module
     * @param number $offset
     * @param number $limit
     * @param string $where
     * @param string $order_by
     * @param string $prefix
     */
    public function _reports($module, $offset, $limit, $where, $order_by, $prefix = 'LS') {
        $sql = "SELECT M.*, P.title AS package, CONCAT(U.id, ' - ', U.display_name) AS user";
        $sql .= " FROM (SELECT " . $prefix . ".* FROM " . $this->db->dbprefix . $module . " " . $prefix;
        if (! empty($where) && (strpos($where, 'SUBSTRING') !== false)) {
            $sql .= " WHERE " . $where;
        } else if (! empty($where)) {
            $sql .= " WHERE " . $where;
        }
        if($module == 'listings' && empty($where) && $this->auth->role_id() == 7) { // franchise can view only their own listings
            $sql .= " WHERE " . $prefix . ".franchise_id = " . $this->auth->user_id();
        }
        $sql .= " GROUP BY " . $prefix .".id ORDER BY " . $prefix . "." . $order_by;
        $sql .= " LIMIT " . $offset . ", " . $limit;
        $sql .= " ) M";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "packages P ON P.id = M.package_id";
        $sql .= " LEFT JOIN " . $this->db->dbprefix . "users U ON U.id = M.user_id";
        $sql .= " GROUP BY M.id ORDER BY M." . $order_by;
        $query = $this->db->query($sql);
        $result = $query->result();
        return $result;
    }
    
    /**
     * Display/Hide fields from the listings page based on package value
     * NOTE: Do not control module fields via field position method, instead use the display_package to avoid unnecessary clicks
     * 
     * @param array $package
     * @param object $result
     * @return array
     */
    public function show_hide_fields($package, $record, $module = 'listings', $user_id = 0, $role = 'admin') {        
        // check field display
		$user_id = empty($user_id) ? (int) $this->auth->user_id() : $user_id;
		
		// set the address, phone and email to null if disabled by the user
		if($module == 'listings') {
    		if($record->show_address == 0) {
    		    $record->address = null;
    		}
    		if($record->show_phone == 0) {
    		    $record->phone_number = null;
    		    $record->mobile_number = null;
    		}
    		if($record->show_email == 0) {
    		    $record->email = null;
    		}
		}
		
		// display fields information
		/**
		 * if a user is logged in and the package of the user (e.g. platinum) is not the same of the listing's package (e.g. platinum)
		 * the loggedin_display_fields come into the picture, they will be displayed to the user
		 * the special fields will only be displayed if the user has the subscription of the same package
		 */
		$visitor_display_fields = (array) json_decode($package->display_fields ?? '', true);
		$loggedin_display_fields = (array) json_decode($package->loggedin_display_fields ?? '', true);
		$special_display_fields = (array) json_decode($package->special_display_fields ?? '', true);
		
		// loggedin user fields override the default values
		/*if($this->session->userdata('current_package')) {
		    $current_package = (object) $this->session->userdata('current_package');
		    $loggedin_display_fields = (array) json_decode($current_package->loggedin_display_fields ?? '', true);
		} else if($user_id != 0) {
		    $this->load->model('packages/packages_model');  
		    $user_package = $this->packages_model->get_current_package($user_id);
		    if($user_package) {
		        $loggedin_display_fields = (array) json_decode($user_package->loggedin_display_fields ?? '', true);
		    } else { // the order status is not confirmed
		        $loggedin_display_fields = array(); // (array) json_decode($package->loggedin_display_fields ?? '', true);
		    }
		}*/
		
		// the user has the subscription of this package, the loggedin and special display fields will be the same for both
		if($this->session->userdata('active_packages')) {
		    $active_packages = (array) $this->session->userdata('active_packages');
		    if(in_array($package->id, $active_packages)) {
		        $loggedin_display_fields = $special_display_fields;
		    }
		}
		
		$visitor_display_fields = (array) $this->filter_display_fields($visitor_display_fields, $module);
		$loggedin_display_fields = (array) $this->filter_display_fields($loggedin_display_fields, $module);
		$special_display_fields = (array) $this->filter_display_fields($special_display_fields, $module);
		
		// get package display fields from the field_relations table
		$module_display_fields = $this->fields_model->package_form_fields($module, 'display_package', 1);
		if(!empty($module_display_fields)) {
		    foreach($module_display_fields as $module_display_field) {
		        $field_value = 0;
		        if($user_id == 0) { // the user is not logged in
		            if(!array_key_exists($module_display_field['name'], $visitor_display_fields)) {
		                if(array_key_exists($module_display_field['name'], $loggedin_display_fields)) {
		                    $field_value = 2;
		                } else if(array_key_exists($module_display_field['name'], $special_display_fields)) {
		                    $field_value = 3;
		                }
		            } else {
		                $field_value = 4; // display value if checked in the package and the user is not logged in
		            }
		        } else {
		            if(!array_key_exists($module_display_field['name'], $loggedin_display_fields)) {
		                if(array_key_exists($module_display_field['name'], $special_display_fields)) {
		                    $field_value = 3;
		                }
		            } else {
		                $field_value = 4; // display value if checked in the package and the user is not logged in
		            }
		        }
		        if($field_value == 0) {
		            $record->{$module_display_field['name']} = null;
		        } else {
		            if($field_value != 4) {
    		            $record->{$module_display_field['name']} = array('value' => $field_value); // display lock symbol
		            }
    		        /*if($role != 'mobile') { issue with URL fields
    		            $record->{$module_display_field['name']} = theme_view('widgets/modules/tpl/_display_field', array('record' => $record, 'field' => $module_display_field['name'], 'package' => $package), true);
    		        }*/
		        }
		    }
		}
		
		/**
		 * Display only selected fields in the package
		 */
		if(settings_item('site.display_package_fields_only') == 1) {
		    if($this->session->userdata('active_packages')) {
		        $merged_fields = $special_display_fields;
		    } else {
		        $merged_fields = $visitor_display_fields;
		    }
		    // $merged_fields = array_merge($visitor_display_fields, $special_display_fields);
		    foreach($module_display_fields as $mkey => $mvalue) {
		        if(!array_key_exists($mvalue['name'], $merged_fields)) {
		            unset($module_display_fields[$mkey]);
		        }
		    }
		}
        return array('record' => $record, 'package_display_fields' => $visitor_display_fields, 'module_display_fields' => $module_display_fields);
    }
    
    /**
     * Get filtered fields
     * @param array $fields
     * @param string $module
     * @return array|boolean
     */
    private function filter_display_fields($fields, $module) {
        if(!empty($fields[$module])) {
            foreach($fields[$module] as $key => $value) {
                if(empty($value)) {
                    unset($fields[$module][$key]);
                }
            }
            $fields = $fields[$module];
            return $fields;
        }
        return false;
    }
     
    /**
     * Notify Users on Bulk Generation
     * @param number $listing_id
     * @param array $record
     * @param string $user_type - admin/user
     * @return object
     */
    public function bulk_notify($listing_id, $record = array(), $user_type = null) {
        if($user_type == 'admin' && empty($record)) {
            $this->load->model('users/user_model');
            $record = new stdClass();
            $this->db->from('listings L');
            $this->db->join('users U', 'U.id = L.user_id', 'INNER');
            $users_data = $this->db->select('L.id, L.title, U.email AS username, "' . $this->user_model->random_password() . '" AS password, L.created_on')->order_by('rand()')->limit(5)->get()->result_array();
            $record->users_list = $this->load->view('listings/content/_tpl_users_list', array('records' => $users_data), true);
        } else {
            $this->load->library('crypt');
            if(empty($record)) {
                $this->db->from('listings L');
                $this->db->join('users U', 'U.id = L.user_id', 'INNER');
                $record = $this->db->select('L.id, L.title, L.slug, L.description, L.address, L.postcode, L.city, L.state, L.country, L.country_iso, L.contact_person, L.categories, L.email, L.phone_number, L.status, U.display_name, U.email AS user_email, U.role_id')->where('L.id', $listing_id)->get()->row();
            }
            if($record) {
                $record->edit_url = gc_url_format('members/edit_business') . '?id=' . $this->crypt->encode($record->id);
                $record->live_url = gc_url_format('listings', $record->slug, $record->id, $this->slug->create_slug($record->city));
                $record->edit_url = anchor($record->edit_url, $record->edit_url);
                $record->live_url = anchor($record->live_url, $record->live_url);
                $record->login_url = anchor(gc_url_format('login'), gc_url_format('login'));
                $record->status = $this->listing_status($record->status, true);
                if(empty($record->password)) {
                    $this->load->model('users/user_model');
                    $record->password = $this->user_model->random_password();
                }
            }
        }
        return $record;
    }
    
    /**
     * Prevent spam by checking specific strings in specific fields
     * Some of the visitors submitting china listings in Indian Business Directory, this function is created to prevent such submissions
     * @param number $id - ID of the row
     * @param string $module - Name of the table
     */
    public function prevent_spam($id, $module) {
        if(!empty(settings_item('site.prevent_spam'))) {
            // $matches = '{"description":["+861","+852"],"address":["china"]}';
            $matches = json_decode(settings_item('site.prevent_spam'));
            if(!empty($matches)) {
                $record = $this->db->where('id', $id)->get($module)->row();
                if($record && $record->status == 1) { // != 1 && $record->status != 3
                    $is_spam = false;
                    foreach($matches as $field => $match_strings) {
                        if(!$is_spam && !empty($record->{$field})) {
                            $match_value = $field != 'description' ? strtolower($record->{$field}) : $record->{$field};
                            foreach($match_strings as $match) {
                                $match = strtolower($match);
                                if(strpos($match_value, $match) !== false) {
                                    $is_spam = true;
                                }
                            }
                        }
                    }
                    if($is_spam) {
                        $this->db->where('id', $id)->update($module, array('status' => 0));
                    }
                }
            }
        }
    }
}