<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
 || ##################################################################### ||
 || # GoClixy - Business Directory Script							   # ||
 || # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
 || # ----------------------------------------------------------------  # ||
 || # This file is owned and copyrighted by GoClixy.com. Your license   # ||
 || # confers  no  title  or ownership in the Script and is not a sale  # ||
 || # of any rights to the Software. This file may not be redistributed # ||
 || # in whole or significant part.									   # ||
 || # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
 || # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
 || ##################################################################### ||
 \*=======================================================================*/
class Mobiles_model extends BF_Model {
    public $mb_pages_cache;
    protected $table_ads = 'mobile_sliders';
    protected $table_ad_views = 'mobile_slider_views';
    protected $table_player_ids = 'mobile_player_ids';
    protected $table_devices = 'mobile_devices';
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();
        // Load Zend Caching
        $this->load->library('slug');
        $this->load->library('gzf_cache', array('lifetime' => 900));
        if(method_exists($this->gzf_cache, 'get_instance')) {
            $this->mb_pages_cache = $this->gzf_cache->get_instance();
        }
    }    
    
    /**
     * Links Fields
     *
     * @return array
     */
    public function links() {
        $custom_fields = array(
            array(
                'name' => 'key',
                'type' => 'text',
                'label' => lang('label_key')
            ),
            array(
                'name' => 'value',
                'type' => 'text',
                'label' => lang('label_value')
            )
        );
        Template::set('custom_fields', $custom_fields);
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'text',
                'name' => 'label',
                'label' => 'label_label',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'tooltip' => lang('help_label'),
                'class' => 4,
                'edit' => 1,
                'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
                'rules' => 'trim|max_length[100]|required',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'label_lang',
                'label' => 'label_lang',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'dropdown',
                'name' => 'link_url',
                'label' => 'label_url',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:label_none;value:|name:label_detail;value:detail|name:label_view;value:view',
                'options_detailed' => 1,
                'options_lang' => 1,
                'class' => 2,
                'rules' => 'trim',
            ),/*
            array(
                'type' => 'dropdown',
                'name' => 'mobile_icon',
                'label' => 'label_icon',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:mobile_icons;value:database',
                'options_detailed' => 1,
                // 'field_extra' => 'selectpicker:1;multiple:1;max_options:1',
                'class' => 4,
                'rules' => 'trim|max_length[100]|required',
                ),*/
            array(
                'type' => 'text',
                'name' => 'mobile_icon',
                'label' => 'label_icon',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'value' => 'checkmark',
                'tooltip' => sprintf(lang('help_mobile_icon'), anchor('https://ionicons.com/', 'https://ionicons.com/', 'target="_blank"')),
                'tooltip_position' => 'help-block',
                'class' => 6,
                'rules' => 'trim|max_length[100]|required',
            ),
            array(
                'type' => 'custom',
                'name' => 'params',
                'label' => 'label_options',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'field_extra' => 'options:custom_fields',
                'tooltip' => lang('help_slider_extra'),
                'class' => 9,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'priority',
                'label' => 'label_priority',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'tooltip' => lang('help_priority'),
                'value' => 0,
                'class' => 2,
                'rules' => 'trim|is_numeric|required',
            ),
            array(
                'type' => 'dropdown',
                'name' => 'access_type',
                'label' => 'label_access_type',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'options' => 'name:tab_all;value:0|name:loggedin_only;value:1|name:not_loggedin_only;value:2',
                'options_lang' => 1,
                'options_detailed' => 1,
                'class' => 3,
                'tooltip' => lang('help_access_type'),
                'rules' => 'trim',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'status',
                'label' => 'label_status',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
        );
        return $fields;
    }
    
    /**
     * Dashboard Components
     *
     * @return array
     */
    public function components() {
        $custom_fields = array(
            array(
                'name' => 'key',
                'type' => 'text',
                'label' => lang('label_key')
            ),
            array(
                'name' => 'value',
                'type' => 'text',
                'label' => lang('label_value')
            )
        );
        Template::set('custom_fields', $custom_fields);
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'dropdown',
                'name' => 'component',
                'label' => 'label_component',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => true,
                'options' => 'name:label_slider;value:sliders|name:label_banner;value:banner|name:gc_menu_testimonials;value:testimonials|name:featured;value:featured|name:popular;value:popular|name:recently_added;value:recently_added|name:categories;value:categories',
                'options_detailed' => 1,
                'options_lang' => 1,
                'class' => 2,
                'edit' => 1,
                'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
                'rules' => 'trim|max_length[255]',
            ),
            array(
                'type' => 'dropdown',
                'name' => 'view_type',
                'label' => 'view_type',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => true,
                'options' => 'name:label_none;value:none|name:card_view;value:card_view|name:list_view;value:list_view',
                'options_detailed' => 1,
                'options_lang' => 1,
                'class' => 2,
                'control_class' => 'view-type',
                'rules' => 'trim|max_length[255]',
            ),
            array(
                'type' => 'text',
                'name' => 'label',
                'label' => 'label_label',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'tooltip' => lang('help_label'),
                'class' => 6,
                'rules' => 'trim|max_length[100]|required',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'label_lang',
                'label' => 'label_lang',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'dropdown',
                'name' => 'mobile_icon',
                'label' => 'label_icon',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:mobile_icons;value:database',
                'options_detailed' => 1,
                'field_extra' => 'selectpicker:1;multiple:1;max_options:1',
                'class' => 4,
                'rules' => 'trim|max_length[100]',
            ),
            array(
                'type' => 'custom',
                'name' => 'params',
                'label' => 'label_options',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'field_extra' => 'options:custom_fields',
                'tooltip' => lang('help_slider_extra'),
                'class' => 9,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'priority',
                'label' => 'label_priority',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'tooltip' => lang('help_priority'),
                'value' => 0,
                'class' => 2,
                'rules' => 'trim|is_numeric|required',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'status',
                'label' => 'label_status',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
        );
        return $fields;
    }
    
    /**
     * Display Mobile Popups
     *
     * @return array
     */
    public function popups() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'text',
                'name' => 'name',
                'label' => 'label_name',
                'placeholder' => 'placeholder_name',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'edit' => 1,
                'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
                'class' => 6,
                'rules' => 'trim|max_length[100]|required',
            ),
            array(
                'type' => 'filemanager',
                'name' => 'image_url',
                'label' => 'label_image',
                'placeholder' => 'placeholder_image',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => true,
                'rules' => 'trim|required|max_length[255]',
            ),
            array(
                'type' => 'text',
                'name' => 'link_url',
                'label' => 'popup_url',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'tooltip' => lang('popup_help_url'),
                'rules' => 'trim|max_length[255]',
            ),
            array(
                'type' => 'text',
                'name' => 'start_date',
                'label' => 'label_start_date',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'tooltip' => lang('popup_help_start_date'),
                'class' => 3,
                'rules' => 'trim|valid_datetime',
            ),
            array(
                'type' => 'text',
                'name' => 'end_date',
                'label' => 'label_end_date',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'tooltip' => lang('popup_help_end_date'),
                'class' => 3,
                'rules' => 'trim|valid_datetime',
            ),
            array(
                'type' => 'text',
                'name' => 'timeout',
                'label' => 'popup_timeout',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'value' => 0,
                'tooltip' => lang('popup_help_timeout'),
                'class' => 2,
                'rules' => 'trim|required|is_numeric',
            ),
            array(
                'type' => 'text',
                'name' => 'frequency',
                'label' => 'popup_frequency',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'value' => 1,
                'tooltip' => lang('popup_help_frequency'),
                'class' => 2,
                'rules' => 'trim|required|is_numeric',
            ),
            array(
                'type' => 'text',
                'name' => 'params',
                'label' => 'popup_params',
                'placeholder' => 'popup_placeholder_params',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'tooltip' => lang('popup_help_params'),
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'priority',
                'label' => 'label_priority',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'tooltip' => lang('help_priority'),
                'value' => 0,
                'class' => 2,
                'rules' => 'trim|is_numeric|required',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'status',
                'label' => 'label_status',
                'frontend' => false,
                'admin_only' => false,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_by',
                'label' => 'created_by',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_by',
                'label' => 'modified_by',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
        );
        return $fields;
    }
    
    /**
     * Mobile Slider Fields
     *
     * @return array
     */
    public function sliders() {
        $custom_fields = array(
            array(
                'name' => 'key',
                'type' => 'text',
                'label' => lang('label_key')
            ),
            array(
                'name' => 'value',
                'type' => 'text',
                'label' => lang('label_value')
            )
        );
        Template::set('custom_fields', $custom_fields);
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'dropdown',
                'name' => 'slider_group_id',
                'label' => 'label_page',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:mobile_slider_groups;value:database',
                'options_detailed' => 1,
                'field_extra' => 'selectpicker:1;multiple:1;max_options:10',
                'class' => 4,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'title',
                'label' => 'label_title',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'edit' => 1,
                'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
                'rules' => 'trim|max_length[100]',
            ),
            array(
                'type' => 'textarea',
                'name' => 'description',
                'label' => 'label_description',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'field_extra' => 'rows:2',
                'rules' => 'trim',
            ),
            array(
                'type' => 'filemanager',
                'name' => 'image',
                'label' => 'label_image',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'field_extra' => 'files_limit:1',
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'start_date',
                'label' => 'label_start_date',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'class' => 3,
                'rules' => 'trim|valid_datetime',
            ),
            array(
                'type' => 'text',
                'name' => 'end_date',
                'label' => 'label_end_date',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'class' => 3,
                'rules' => 'trim|valid_datetime',
            ),
            array(
                'type' => 'text',
                'name' => 'max_clicks',
                'label' => 'label_max_clicks',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'class' => 2,
                'value' => 0,
                'rules' => 'trim|required|is_numeric',
            ),
            array(
                'type' => 'text',
                'name' => 'clicks',
                'label' => 'label_clicks',
                'frontend' => false,
                'admin_only' => false,
                'frontend_index' => true,
                'admin_index' => true,
                'class' => 2,
                'value' => 0,
                'rules' => 'trim|required|is_numeric',
            ),
            array(
                'type' => 'dropdown',
                'name' => 'link_url',
                'label' => 'label_url',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'options' => 'name:label_none;value:|name:label_detail;value:detail|name:label_view;value:view',
                'options_detailed' => 1,
                'options_lang' => 1,
                'class' => 2,
                'rules' => 'trim',
            ),
            array(
                'type' => 'custom',
                'name' => 'params',
                'label' => 'label_options',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'field_extra' => 'options:custom_fields',
                'tooltip' => lang('help_slider_extra'),
                'class' => 9,
                'rules' => 'trim',
            ),
            array(
                'type' => 'text',
                'name' => 'priority',
                'label' => 'label_priority',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'class' => 2,
                'value' => 0,
                'rules' => 'trim|required|is_numeric',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'status',
                'label' => 'label_status',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_by',
                'label' => 'created_by',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'created_ip',
                'label' => 'created_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_by',
                'label' => 'modified_by',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_ip',
                'label' => 'modified_ip',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            )
            
        );
        return $fields;
    }
    
    /**
     * Mobile Icons Fields
     *
     * @return array
     */
    public function icons() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'text',
                'name' => 'name',
                'label' => 'label_name',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'edit' => 1,
                'class' => 4,
                'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
                'rules' => 'trim|required|max_length[100]',
            ),
            array(
                'type' => 'text',
                'name' => 'code_point',
                'label' => 'label_code_point',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'class' => 2,
                'rules' => 'trim|required|is_numeric',
            ),
            array(
                'type' => 'checkbox',
                'name' => 'status',
                'label' => 'label_status',
                'frontend' => false,
                'admin_only' => false,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            
        );
        return $fields;
    }
    
    /**
     * Mobile Player Ids Fields
     *
     * @return array
     */
    public function player_ids() {
        $fields = array(
            array(
                'type' => 'id',
                'name' => 'id',
                'frontend_index' => true,
                'admin_index' => true
            ),
            array(
                'type' => 'selectize',
                'name' => 'user_id',
                'label' => 'bf_user',
                'placeholder' => 'placeholder_selectize',
                'frontend' => false,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => true,
                'options' => 'load:users;module:users;max_items:1',
                'options_detailed' => 1,
                'rules' => 'trim|required',
            ),
            array(
                'type' => 'text',
                'name' => 'player_id',
                'label' => 'label_player_id',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => true,
                'admin_index' => true,
                'rules' => 'trim|max_length[50]',
            ),
            array(
                'type' => 'none',
                'name' => 'created_on',
                'label' => 'created_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => true,
                'rules' => 'trim',
            ),
            array(
                'type' => 'none',
                'name' => 'modified_on',
                'label' => 'modified_on',
                'frontend' => true,
                'admin_only' => true,
                'frontend_index' => false,
                'admin_index' => false,
                'rules' => 'trim',
            ),
            
        );
        return $fields;
    }
    
    /**
     * Display All Mobile Devices
     */
    public function get_devices($count, $where, $sort_by, $limit = 0, $offset = 0) {
        $sql = "SELECT";
        if($count == 0) {
            $sql .= " *";
        } else {
            $sql .= " COUNT(id) AS total_count";
        }
        $sql .= " FROM ". $this->db->dbprefix ."mobile_devices";
        if(!empty($where)) {
            $sql .= " WHERE " . $where;
        }
        if($count == 0) {
            $sql .= " ORDER BY " . $sort_by;
            if ($limit != 0) {
                $sql .= " LIMIT " . $offset . ", " . $limit;
            } else {
                $sql .= " LIMIT $limit";
            }
            return $this->db->query ( $sql )->result();
        } else {
            $result = $this->db->query ( $sql )->row();
            return $result ? $result->total_count : 0;
        }
    }
    
    /**
     * Cities Search and List Method
     */
    public function get_cities($count, $keyword, $country, $limit = 0, $offset = 0) {
        $sql = "SELECT";
        if($count == 0) {
            $sql .= " id, CONCAT(C.name, S.name, CT.iso) AS name";
        } else {
            $sql .= " COUNT(C.id) AS total_count";
        }
        $sql .= " FROM " . $this->db->dbprefix . "cities C";
        $sql .= " INNER JOIN " . $this->db->dbprefix . "states S ON S.id = C.state_id";
        $sql .= " INNER JOIN " . $this->db->dbprefix . "countries CT ON CT.iso = S.country_iso AND CT.name = '$country'";
        
        if($keyword != NULL) {
            $sql .= " WHERE C.name LIKE '" . $keyword . "%'";
        }
        
        if($count == 0) {
            $sql .= " ORDER BY C.name";
            if ($limit != 0) {
                $sql .= " LIMIT " . $offset . ", " . $limit;
            } else {
                $sql .= " LIMIT $limit";
            }
            return $this->db->query ( $sql )->result();
        } else {
            $result = $this->db->query ( $sql )->row();
            return $result ? $result->total_count : 0;
        }
    }
    
    /**
     * Common function to load locations
     */
    public function load_locations($count, $type, $id, $searchterm, $limit = 0, $offset = 0) {
        $sql = "SELECT";
        
        if($count == 0) {
            if($type == 'countries') {
                $sql .= " L.iso AS id, L.name";
            } else if($type == 'cities') {
                $sql .= " L.id, CONCAT(L.name, ', ', L.state, ', ', L.country_iso) AS name";
            } else {
                $sql .= " L.id, L.name";
            }
        } else {
            if($type == 'countries') {
                $sql .= " COUNT(L.iso) AS total_count";
            } else {
                $sql .= " COUNT(L.id) AS total_count";
            }
        }
        $sql .= " FROM ". $this->db->dbprefix . $type ." L";
        $sql .= " WHERE L.active = 1";
        
        //if($id != 0) {
        if($type == 'states' && is_string($id)) {
            $sql .= " AND L.country_iso = '" . $id ."'";
        } else if($type == 'cities' && $id != 0) {
            $sql .= " AND L.state_id = " . $id;
        }
        //}
        if(!empty($searchterm)) {
            $sql .= " AND L.name LIKE '" . $searchterm . "%'";
        }
        if($count == 0) {
            $sql .= " ORDER BY L.name ASC";
            if ($limit != 0) {
                $sql .= " LIMIT " . $offset . ", " . $limit;
            } else {
                $sql .= " LIMIT $limit";
            }
            return $this->db->query ( $sql )->result();
        } else {
            $result = $this->db->query ( $sql )->row();
            return $result ? $result->total_count : 0;
        }
    }
    
    /**
     * Common function to load Areas/Localities
     */
    public function load_zip($count, $city_id, $searchterm, $limit = 0, $offset = 0) {
        $sql = "SELECT";
        if($count == 0) {
            $sql .= " DISTINCT(L.postcode) AS name";
        } else {
            $sql .= " COUNT(DISTINCT(L.postcode)) AS total_count";
        }
        $sql .= " FROM ". $this->db->dbprefix ."listings L";
        $sql .= " WHERE L.status = 1";
        
        if($city_id != 0) {
            $sql .= " AND L.city_id = " . $city_id;
        }
        
        if(!empty($searchterm)) {
            $sql .= " AND L.postcode LIKE '%" . $searchterm . "%'";
        }
        if($count == 0) {
            if ($limit != 0) {
                $sql .= " LIMIT " . $offset . ", " . $limit;
            } else {
                $sql .= " LIMIT $limit";
            }
            return $this->db->query ( $sql )->result();
        } else {
            $result = $this->db->query ( $sql )->row();
            return $result ? $result->total_count : 0;
        }
    }
    
    /**
     * Common function to load Areas/Localities
     */
    public function load_areas($count = 0, $city_id = 0, $searchterm = null, $limit = 0, $offset = 0) {
        $sql = "SELECT";
        if($count == 0) {
            $sql .= " L.id, L.name";
        } else {
            $sql .= " COUNT(L.id) AS total_count";
        }
        $sql .= " FROM ". $this->db->dbprefix ."localities L";
        $sql .= " WHERE L.active = 1";
        $sql .= " AND L.city_id = " . $city_id;
        
        if(!empty($searchterm)) {
            $sql .= " AND L.name LIKE '%" . $searchterm . "%'";
        }
        if($count == 0) {
            $sql .= " ORDER BY L.name ASC";
            if ($limit != 0) {
                $sql .= " LIMIT " . $offset . ", " . $limit;
            } else {
                $sql .= " LIMIT $limit";
            }
            return $this->db->query ( $sql )->result();
        } else {
            $result = $this->db->query ( $sql )->row();
            return $result ? $result->total_count : 0;
        }
    }
    
    /**
     * Common function to load Tags
     */
    public function load_tags($count, $searchterm, $limit = 0, $offset = 0) {
        $sql = "SELECT";
        if($count == 0) {
            $sql .= " T.id, T.name";
        } else {
            $sql .= " COUNT(T.id) AS total_count";
        }
        $sql .= " FROM ". $this->db->dbprefix ."tags T";
        $sql .= " WHERE T.active = 1";
        
        if(!empty($searchterm)) {
            $sql .= " AND T.name LIKE '%" . $searchterm . "%'";
        }
        if($count == 0) {
            $sql .= " ORDER BY T.name ASC";
            if ($limit != 0) {
                $sql .= " LIMIT " . $offset . ", " . $limit;
            } else {
                $sql .= " LIMIT $limit";
            }
            return $this->db->query ( $sql )->result();
        } else {
            $result = $this->db->query ( $sql )->row();
            return $result ? $result->total_count : 0;
        }
    }
    
    /**
     * TotalPages = Ceil(TotalCount/Limit)
     *
     */
    public function canonical_links($totalPages, $curPageIndex) {
        // determine relative flag
        $nextLink = "";
        $lastLink = "";
        $firstLink = "";
        $prevLink = "";
        $relAttrWithURL = "";
        
        // this pageNr is the current page
        if( $curPageIndex != 1 ) { $firstLink = '<' . $this->modQuery(array('page' => 1)) . '>; rel="first"'; }
        
        for ($i = 1; $i <= $totalPages; $i++) {
            
            // this pageNr is one above the current page
            if( $i == $curPageIndex + 1 ) { $nextLink = '<' . $this->modQuery(array('page' => $i)) . '>; rel="next"'; }
            
            // this pageNr is one below the current page
            if( $i == ($curPageIndex - 1) ) { $prevLink = '<' . $this->modQuery(array('page' => $i)) . '>; rel="prev"'; }
            
        }
        
        $lastLink= '<' . $this->modQuery(array('page' => $totalPages)) . '>; rel="last"';
        
        // Arrange links in order nextLink, lastLink, firstLink, prevLink
        if(!empty($nextLink)) { $relAttrWithURL = $nextLink .', '; }
        if(!empty($firstLink) || !empty($prevLink)) {
            $relAttrWithURL .= $lastLink .', ';
            if(!empty($firstLink) && !empty($prevLink)) {
                $relAttrWithURL .= $firstLink .', ' . $prevLink;
            } else if(empty($prevLink)) {
                $relAttrWithURL	.= $firstLink;
            }
        } else {
            $relAttrWithURL .= $lastLink;
        }
        
        // Write Header
        $this->output->set_header('Access-Control-Expose-Headers: link');
        $this->output->set_header('Access-Control-Allow-Origin: *'); // IMPORTANT - NECESSARY FOR IONIC TO ENABLE CORS SUPPORT
        $this->output->set_header('Link: ' . $relAttrWithURL);
    }
    
    function curPageURL() {
        $pageURL = 'http';
        if(isset($_SERVER["HTTPS"]))
            if ($_SERVER["HTTPS"] == "on") {
                $pageURL .= "s";
            }
        $pageURL .= "://";
        if ($_SERVER["SERVER_PORT"] != "80") {
            $pageURL .= $_SERVER["SERVER_NAME"] . ":" . $_SERVER["SERVER_PORT"] . $_SERVER["REQUEST_URI"];
        } else {
            $pageURL .= $_SERVER["SERVER_NAME"] . $_SERVER["REQUEST_URI"];
        }
        return $pageURL;
    }
    
    function current_url() {
        $CI =& get_instance();
        
        $url = $CI->config->site_url($CI->uri->uri_string());
        return $_SERVER['QUERY_STRING'] ? $url.'?'.$_SERVER['QUERY_STRING'] : $url;
    }
    
    
    /**
     * Add and modify a query string parameter
     * Return URL
     */
    public function modQuery($add_to, $rem_from = array(), $clear_all = false) {
        if ($clear_all){
            $query_string = array();
        } else {
            parse_str($_SERVER['QUERY_STRING'], $query_string);
        }
        if (!is_array($add_to)){ $add_to = array(); }
        $query_string = array_merge($query_string, $add_to);
        if (!is_array($rem_from)){ $rem_from = array($rem_from); }
        foreach($rem_from as $key){
            unset($query_string[$key]);
        }
        $new_query_string = http_build_query($query_string);
        
        // APPEND NEW GENERATED QUERY STRING TO THE URL
        $CI =& get_instance();
        $url = $CI->config->site_url($CI->uri->uri_string());
        return $_SERVER['QUERY_STRING'] ? $url.'?'.$new_query_string : $url;
    }
    
    /*--------------------------------- ADMIN PANEL --------------------------------*/
  
    /**
     * Common function to load cities
     */
    public function load_cities($count, $type, $id, $searchterm, $country_iso, $limit = 0, $offset = 0) {
        $limit = 50;
        $sql = "SELECT";
        
        if($type == 'countries') {
            $id = 1;
            if($count == 0) {
                $sql .= " L.iso AS id, L.name, L.iso AS state_id, L.name AS state, L.iso, L.name AS country";
            } else {
                $sql .= " COUNT(L.iso) AS total_count";
            }
            $sql .= " FROM ". $this->db->dbprefix . "countries L";
        } else {
            if($count == 0) {
                // $sql .= " L.id, CONCAT(L.name, ', ' , S.name) AS name, S.id AS state_id, S.name AS state, C.iso AS country_iso, C.name AS country";
                $sql .= " L.id, L.name, S.id AS state_id, S.name AS state, C.iso AS country_iso, C.name AS country";
            } else {
                $sql .= " COUNT(L.id) AS total_count";
            }
            $sql .= " FROM ". $this->db->dbprefix . "cities L";
            $sql .= " INNER JOIN " . $this->db->dbprefix . "states S ON S.id = L.state_id";
            $sql .= " INNER JOIN " . $this->db->dbprefix . "countries C ON C.iso = S.country_iso";
        }
        $sql .= " WHERE L.active = 1";
        if($type == 'states' && !empty($id)) {
            $sql .= " AND S.country_iso = '" . $id ."'";
        } else if($type == 'cities' && $id != 0) {
            $sql .= " AND L.state_id = " . $id;
        }
        
        if(!empty($searchterm)) {
            $sql .= " AND L.name LIKE '%" . $searchterm . "%'";
        }
        if(empty($id)) {
            $sql .= " AND C.iso = '" . $country_iso . "'";
        }
        if($count == 0) {
            $sql .= " ORDER BY L.name ASC";
            if ($limit != 0) {
                $sql .= " LIMIT " . $offset . ", " . $limit;
            } else {
                $sql .= " LIMIT $limit";
            }
            return $this->db->query ( $sql )->result();
        } else {
            $result = $this->db->query ( $sql )->row();
            return $result ? $result->total_count : 0;
        }
    }
    
    /**
     * Form Fields
     * @param array $data
     * @return array
     */
    public function rest_fields($records = array()) {
        $fields = array();
        foreach($records as $record) {
            $field = array(
                'name' => $record['name'],
                'type' => $record['type'],
                'label' => isset($record['label']) && empty($record['label_lang']) ? $record['label'] : lang('label_' . $record['name'])  . (!empty($record['rules']['required'])  ? '*' : ''),
                'required' => !empty($record['rules']['required']) ? true: false,
                'validators' => !empty($record['rules']) ? $record['rules'] : array(),
                'display' => 'selected',
                'options' => !empty($record['options']) ? $record['options'] : array(),
                'params' => !empty($record['params']) ? $record['params'] : array(),
            );
            if(!empty($record['options'])) {
                $field['options'] = $record['options'];
            }
            $fields[] = $field;
        }
        return $fields;
    }
    
    /**
     * Format Fields
     * @param array $records
     */
    public function format_fields($records, $extra_params = array()) {
        $fields = array();
        if(!empty($records)) {
            foreach($records as $record) {
                if($record['name'] == 'latitude_longitude') {
                    $record['type'] = 'none';
                }
                if(!isset($record['rules'])) {
                    $record['rules'] = '';
                }
                $params = array();
                $field_required = !empty($record['required']) ? TRUE : FALSE;
                switch($record['type']) {
                    case 'text':
                    case 'password':
                    case 'textarea':
                    case 'checkbox':
                    case 'radio':
                    case 'dropdown':
                    case 'location':
                    case 'city':
                    case 'categories':
                    case 'category':
                    case 'selectize':
                    case 'filemanager':
                    case 'image':
                    case 'fileuploader':
                    case 'position':
                    case 'map':
                    case 'group':
                    case 'cart':
                        // display tooltip to the user
                        /*if(!empty($record['tooltip'])) { // not working
                            $params = array(
                                'help' => $record['tooltip']
                            );
                        }*/
                        $max_items = 0;
                        $process_field = true;
                        if(!empty($record['options']) && !empty($record['options'][0]['max_items'])) {
                            $max_items = $record['options'][0]['max_items'];
                            $params['max_items'] = $max_items;
                        }
                        $field_type = $record['type'];
                        if(!empty($record['rules']) && strpos($record['rules'], 'is_array') !== false || $max_items > 1) {
                            $isMultiple = true;
                        } else {
                            $isMultiple = false;
                        }
                        switch($record['type']) {
                            case 'dropdown':
                                $field_type = 'select';
                                break;
                            case 'checkbox':
                            case 'radio':
                                // multiselect field
                                //if(!empty($record['options'][0]['value']) && $record['options'][0]['value'] == 'database') {
                                if($record['name'] == 'address_phone_email') {
                                    foreach($record['options'] as $field_option) {
                                        $fields[] = array(
                                            'name' => $field_option,
                                            'type' => 'checkbox',
                                            'label' => lang('label_' . $field_option),
                                            'required' => false,
                                            'display' => 'selected',
                                            'params' => array(),
                                            'validators' => array() 
                                        );
                                    }
                                    unset($record['options']);
                                    $process_field = false;
                                } else {
                                    if(!empty($record['options']) && $isMultiple) {
                                        $field_type = 'selectable';
                                    } else if(!$isMultiple) {
                                        $field_type = 'select';
                                    }
                                }
                                break;
                            case 'selectize':
                            case 'city':
							case 'location':
							case 'category':
                                $field_type = 'selectable';
                                break;
							case 'categories':
							    $field_type = 'selectable';
							    $record['max_items'] = !empty($record['categories_limit']) ? $record['categories_limit'] : 1;
							    /*$field_type = 'treeview';			
							    $params = array(
							        'hasAllCheckBox' => true,
							        'hasFilter' => true,
							        'hasCollapseExpand' => true,
							        'decoupleChildFromParent' => false,
							        'maxHeight' => 500
							    );*/
							    break;
							case 'filemanager':
							case 'fileuploader':
							case 'image':
							    $field_type = 'image';
							    if(!empty($record['files_limit'])) {
							        // multiple images upload
							        $params['files_limit'] = (int) $record['files_limit'];
							    }
							    break;
							case 'map':
							case 'position':
							    $field_type = 'map';
							    break;
							case 'text':
							    if($record['name'] == 'distances') { // properties module
							        foreach($record['options'] as $field_option) {
							            $fields[] = array(
							                'name' => 'distances[' . $field_option . ']',
							                'type' => 'text',
							                'label' => lang('label_' . $field_option),
							                'required' => false,
							                'display' => 'selected',
							                'params' => array(),
							                'validators' => array('numeric')
							            );
							        }
							        unset($record['options']);
							        $process_field = false;
							    }
							    break;
                        }
                        // tinymce editor
                        if(!empty($record['editor'])) {
                            $field_type = 'editor';
                        }
                        if($field_type == 'selectable') {
                            $params['onChange'] = false;
                            $params['canSearch'] = false;
                            $params['onSearch'] = false;
                            $params['hasInfiniteScroll'] = false;
                            $params['onInfiniteScroll'] = false;
                            $params['urlParams'] = false;							
							if($record['type'] == 'city' || $record['type'] == 'location') {
								$params['urlParams'] = 'module=locations&md=cities';
							} else if($record['type'] == 'categories' || $record['type'] == 'category') {
							    if(!empty($extra_params['module'])) { // add module form
							        $params['urlParams'] = 'module=categories&tm=' . $extra_params['module'];
							    } else {
							        $params['urlParams'] = 'module=categories';
							    }								
							}
							// limit the number of times a user can select
							if(isset($record['max_items']) && $record['max_items'] > 1) {
							    $isMultiple = true;
							    $params['min'] = 1;
							    $params['max'] = $record['max_items'];
							    $params['max_items'] = $record['max_items'];
							    $params['limitMessage'] = sprintf(lang('selectable_limit'), 1, $record['max_items']);
							}
							if($record['name'] == 'address_id') { // the user can add a new address
							    $params['addRedirectUrl'] = '/crud/addresses/0';
							    $params['canAdd'] = true;
							}
                        }
                        $field = array(
                            'name' => $record['name'],
                            'type' => $field_type,
                            'label' => (!empty($record['label_lang']) ? lang($record['label']) : $record['label']) . (!empty($record['required'])  ? '*' : ''),
                            'required' => !empty($record['required']) ? true : false,
                            'display' => 'selected'
                        );
                        if(!empty($record['options'])) {     
                            $dropdown_options = array();
                            $name_field = $field_type == 'selectable' ? 'name' : 'name';
                            $value_field = $field_type == 'selectable' ? 'id' : 'value';
                            foreach($record['options'] as $option_key => $option) {
                                if(!empty($option['value']) && $option['value'] == 'counter') {
                                    $counter = explode('!~', $option['name'] ?? '');
                                    if($field['name'] == 'weight') {
                                        for($x = $counter[0]; $x <= $counter[1]; $x++) {
                                            $dropdown_options[] = array(
                                                $name_field => weight_conversion($x),
                                                $value_field => (int) $x
                                            );
                                        }
                                    } else {
                                        for($x = $counter[0]; $x <= $counter[1]; $x++) {
                                            $dropdown_options[] = array(
                                                $name_field => isset($counter[3]) && $counter[3] == 1 ? $x . ' ' . lang($counter[2]) : (isset($counter[2]) ? $x . $counter[2] : $x),
                                                $value_field => (int) $x
                                            );
                                        }
                                    }
                                } else if(!empty($option['value']) && $option['value'] == 'database' && $record['type'] != 'selectize') {
                                    $dropdown_field = $this->fields_model->database_options($option);
                                    if(!$field_required){
                                        $dropdown_options[] = array(
                                            $name_field => lang('label_none'),
                                            $value_field => 0
                                        );
                                    }
                                    if(!empty($dropdown_field)) {
                                        foreach ( $dropdown_field as $dropdown ) {
                                            $dropdown_options[] = array(
                                                $name_field => !empty($dropdown->title) ? (!(empty($dropdown->title_lang)) ? lang($dropdown->title) : $dropdown->title) : (!empty($dropdown->name_lang) ? lang($dropdown->name) : $dropdown->name),
                                                $value_field => $dropdown->id
                                            );
                                        }
                                    }
                                } else if($record['type'] == 'selectize') {
                                    $option_module = isset($option['module']) ? $option['module'] : (!empty($option['load']) ? $option['load'] : '');
                                    $url_params = 'module=' . $option_module;
                                    if(!empty($option['relation_field'])) {
                                        $url_params .= '&' . $option['relation_field'] . '=' . $option['relation_value'];
                                    }
                                    if(!empty($option['params'])) {
                                        $option_params = explode(',', $option['params']);
                                        foreach($option_params as $option_param) {
                                            $option_param_values = explode('#', $option_param ?? '');
                                            $params[$option_param_values[0]] = $option_param_values[1];
                                        }
                                    }
                                    $params['urlParams'] = $url_params;
                                } else {
                                    $option_name = !empty($option['name']) ? $option['name'] : $option;
                                    $option_value = (isset($option['value']) ? $option['value'] : '');
                                    $dropdown_options[] = array(
                                        $name_field => $field['name'] == 'facilities' ? lang('label_' . $option_name) : (!empty($record['options_lang']) ? lang($option_name) : $option_name),
                                        $value_field => $field['name'] == 'facilities' ? $option_name : (is_numeric($option_value) ? (int) $option_value : $option_value)
                                    );
                                }
                            }
                            $field['options'] = $dropdown_options;
                        }
                        
                        if($field_type == 'selectable') {
                            $params['isMultiple'] = $isMultiple; // strpos($record['rules'], 'is_array') !== false ? true : false;
                            $params['canSearch'] = isset($dropdown_options) && count($dropdown_options) > 20 ? true : false;
                            if(!empty($params['urlParams'])) {
                                $params['onChange'] = true;
                                $params['canSearch'] = true;
                                $params['onSearch'] = true;
                                $params['hasInfiniteScroll'] = true;
                                $params['onInfiniteScroll'] = true;
                            }
                        }
                        if(strpos($record['rules'], 'valid_date_dob') !== false) {
                            $field['type'] = 'datetime';
                            $min_age = (int) settings_item('site.min_age');
                            $max_age = (int) settings_item('site.max_age');
                            $min_age_dob = (date("Y")-$min_age).date("-m-d");
                            $max_age_dob = (date("Y")-$max_age).date("-m-d");
                            $params['max'] = $min_age_dob;
                            $params['min'] = $max_age_dob;
                        } else if(strpos($record['rules'], 'valid_datetime') !== false) {
                            $field['type'] = 'datetime';
                            $params['max'] = date('Y-m-d', strtotime("+12 month", strtotime(date('Y-m-d'))));
                            $params['min'] = '2020-01-01';
                            $params['format'] = 'YY-MM-DD HH:mm';
                        } else if(strpos($record['rules'], 'valid_date') !== false) {
                            $field['type'] = 'datetime';
                            $params['max'] = date('Y-m-d', strtotime("+12 month", strtotime(date('Y-m-d'))));
                            $params['min'] = '2020-01-01';
                        } else if(strpos($record['rules'], 'valid_time') !== false) {
                            $field['type'] = 'datetime';
                            $params['max'] = date('Y-m-d');
                            $params['min'] = '1901-01-01';
                            $params['format'] = 'HH:mm';
                        }
                        $field['params'] = $params;
                        $validators = explode('|', $record['rules'] ?? '');
                        $validation_rules = array();
                        if(!empty($validators)) {
                            foreach($validators as $key => $validator) {
                                switch($validator) {
                                    case 'trim':
                                    case 'is_array':
                                        unset($validators[$key]);
                                        break;
                                    case 'valid_email':
                                        $validation_rules['email'] = 'email';
                                        break;
                                    case 'is_decimal':
                                        $validation_rules['pattern'] = '^[0-9]+(.[0-9]{0,2})?$';
                                        break;
                                    case 'phone_check':
                                        $validation_rules['pattern'] = '^(\+?(\d{1}|\d{2}|\d{3})[- ]?)?\d{3}[- ]?\d{3}[- ]?\d{4}$';//$this->form_validation->regex($validator);
                                        break;
                                    case 'alpha_dash':
                                        $validation_rules['pattern'] = '^[a-z0-9_-]+$';//$this->form_validation->regex($validator);
                                        break;
                                    case 'valid_date_dob':
                                        break;
                                    case 'valid_gst':
                                        $validation_rules['pattern'] = '^[0-9]{2}[A-Z]{5}[0-9]{4}[A-Z]{1}[1-9A-Z]{1}Z[0-9A-Z]{1}$';
                                        break;
                                    default:
                                        if(strpos($validator, '[') !== false) {
                                            $validator_name = explode('[', $validator);
                                            $name_validator = $validator_name[0];
                                            $value_validator = str_replace(']', '', $validator_name);
                                            if(is_string($value_validator)) {
                                                switch($name_validator) {
                                                    case 'min_length':
                                                        $validation_rules['pattern'] = '^.{' . $value_validator . ',}$';
                                                        break;
                                                    case 'max_length':
                                                        $validation_rules['pattern'] = '^.{,' . $value_validator . '}$';
                                                        break;
                                                }
                                            }
                                        } else {
                                            $validation_rules[$validator] = $validator;
                                        }
                                }
                            }
                        }
                        $field['validators'] = $validation_rules;
                        
                        // module add/edit
                        // display user specific packages and listings only
                        switch($record['name']) {
                            case 'package_id':
                                if(!empty($extra_params['packages'])) {
                                    $field['options'] = $extra_params['packages'];
                                }
                                // hide packages field
                                if(!empty($extra_params['hide_packages'])) {
                                    $process_field = false;
                                }
                                break;
                            case 'listing_id':
                                break;
                        }
                        if($field['type'] == 'treeview') {                            
                            $this->load->library('goclixy/gclosure');
                            $gclosure = new Gclosure('categories', 'categories_hierarchy');
                            $records = $gclosure->tree(array(), array('select' => 't.id, t.name, t.id AS value, t.name AS text, t.slug, t.parent_id', 'sort_by' => 't.name'));
                            // $field['options'] = $records;
                            $field['options'] = array(array('value' => 3, 'text' => 'test', 'children' => array(array('value' => 4, 'text' => '4te', 'children' => array(array('value' => 5, 'text' => '5te'), array('value' => 6, 'text' => '6te'), array('value' => 7, 'text' => '7te'))))));
                        }
                        if($process_field) {
                            $fields[] = $field;
                        }
                        break;
                    case 'group': // display group name
                        $fields[] = array(
                            'name' => $record['name'],
                            'type' => $record['type'],
                            'label' => !empty($record['label_lang']) ? lang($record['label']) : $record['label'],
                            'required' => false,
                            'display' => 'selected',
                            'options' => array(),
                            'params' => array(),
                            'validators' => array()
                        );
                        break;
                }
            }
        }
        // dump($fields);
        return $fields;
    }
    
    /**
     * Format POST Data in PHP Save Format
     * @param array $fields
     * @param array $record
     * @return array
     */
    public function format_post_data($record, $fields) {
        $record = (array) $record;
        $result = array();
        // $this->load->model('options/options_model');
        // $option_values = $this->options_model->get_options();
        // properties distances
        foreach($record as $rkey => $rvalue) {
            if(strpos($rkey, 'distances[') !== false) {
                $distance_option = str_replace('distances[', '', $rkey);
                $distance_option = str_replace(']', '', $distance_option);
                $distance_values[$distance_option] = !empty($rvalue) ? (int) $rvalue : 0;
                foreach($fields as $fdkey => $fdvalue) {
                    if($fdvalue['name'] == $rkey) {
                        unset($fields[$fdkey]);
                    }
                }
                $result['distances'] = $distance_values;
            }
        }
        if(!empty($fields)) {
            foreach($fields as $fkey => $field) { 
                switch($field['name']) {
                    case 'show_address':
                    case 'show_phone':
                    case 'show_email':
                        $result[$field['name']] = (int) $record[$field['name']];
                        break;
                        
                }
                if(!empty($record[$field['name']])) {
                    $value = $record[$field['name']];
                    switch($field['type']) {
                        case 'categories':
                        case 'category':
                        case 'city':
                        // case 'location':
                        case 'selectable':
                            // get country iso value from the id field
                            if($field['name'] == 'country_iso') {
                                if(!empty($field['params']['isMultiple'])) {
                                    $saved_values = array();
                                    if(isset($record[$field['name']]['id'])) {
                                        $saved_values[] = $record[$field['name']]['id'];
                                    } else {
                                        foreach($record[$field['name']] as $value_posted) {
                                            $saved_values[] = $value_posted['id'];
                                        }
                                    }
                                    $saved_values = implode(',', $saved_values);
                                } else {
                                    $saved_values = $record[$field['name']]['id'];
                                }
                                $rows = $this->db->where('id IN (' . $saved_values . ')')->select('iso AS id, name')->get('countries')->result_array();
                                if($rows) {
                                    if(!empty($field['params']['isMultiple'])) {
                                        $selected_values = array();
                                        foreach($rows as $row) {
                                            $selected_values[] = strval($row['id']);
                                        }
                                        $result[$field['name']] = implode(',', $selected_values);
                                    } else {
                                        $result[$field['name']] = strval($rows[0]['id']);
                                    }
                                }
                            } else if($field['name'] == 'facilities') {
                                if(!empty($record[$field['name']])) {
                                    foreach($record[$field['name']] as $facility) {
                                        $saved_values[$facility['id']] = 1;
                                    }
                                    $result[$field['name']] = $saved_values;
                                }
                            } else {
                                $selected_values = array();
                                if(!empty($field['params']['isMultiple'])) {
                                    if(!empty($value['id'])) {                                        
                                        $selected_values[] = $value['id'];
                                    } else if(is_array($value)) {
                                        foreach($value as $selected) {
                                            if(!empty($selected['id'])) {
                                                $selected_values[] = $selected['id'];
                                            }
                                        }
                                    }
                                    if(!empty($field['params']['max_items'])) {
                                        $result[$field['name']] = implode(',', $selected_values);
                                    } else {
                                        $result[$field['name']] = $selected_values; // implode(',', $selected_values); //
                                    }
                                } else if(!empty($value['id'])) {
                                    $result[$field['name']] = $value['id'];
                                } else {
									$result[$field['name']] = $value;
								}
								if($field['name'] == 'city') { // city_id is must
								    $result['city_id'] = $result[$field['name']];
								}
                            }
                            if($field['type'] == 'selectable' && $field['name'] == 'city' && !empty($result['city_id'])) { // also save country, state and city information
                                $this->load->model('locations/cities_model');
                                $city_info = $this->cities_model->city_info($result['city_id']);
                                if($city_info) {
                                    $result['country_iso'] = $city_info->country_iso;
                                    $result['state_id'] = $city_info->state_id;
                                }
                            }
                            break;
                        case 'map':
                            if(!empty($value['lat'])) {
                                if($field['name'] == 'map') {
                                    $result['latitude'] = $value['lat'];
                                    $result['longitude'] = $value['lng'];                                    
                                } else {
                                    $result[$field['name'] . '_latitude'] = $value['lat'];
                                    $result[$field['name'] . '_longitude'] = $value['lng'];
                                }
                            }
                            break;
                        default:
                            if(!empty($value) && in_array('valid_datetime', $field['validators']) !== false) {
                                $result[$field['name']] = date( "Y-m-d H:i:s", strtotime($value));
                            } elseif(!empty($value) && in_array('valid_date', $field['validators']) !== false) {
                                $result[$field['name']] = date( "Y-m-d", strtotime($value));
                                // $result[$field['name']] = date( "Y-m-d");
                            } else {
                                $result[$field['name']] = $value;
                            }
                    }
                }
            }
        }
        return $result;
    }
    
    /**
     * Format Saved Data in IONIC Dynamic Fields Format
     * @param array $fields
     * @param array $record
     * @return array
     */
    public function format_data($fields, $record) {
        $record = (array) $record;
        $result = array();
        foreach($fields as $field) {
            switch($field['name']) {
                case 'map':
                case 'latitude_longitude':
                    if(!empty($record['latitude'])) {
                        $result[$field['name']] = array('lat' => (float) $record['latitude'], 'lng' => (float) $record['longitude']);
                    }
                    break;
                case 'address_phone_email':
                    foreach($field['options'] as $field_option) {
                        if(!empty($record[$field_option])) {
                            $result[$field_option] =  (int) $record[$field_option];
                        }
                    }
                    break;
                case 'distances':
                    if(empty($record[$field['name']])) {
                        foreach($field['options'] as $field_option) {
                            $result['distances[' . $field_option . ']'] = 0;
                        }
                    }
                    break;
            }
            if(!empty($record[$field['name']])) {
                $value = $record[$field['name']];
                if(is_numeric($value)) {
                    // ion-select has an issue of numeric and string values
                    // always correct the type before passing it to the ion-select component
                    $value = (int) $value;
                }
                switch($field['type']) {
                    case 'map':
                    case 'latitude_longitude':
                        if(!empty($record['latitude'])) {
                            $result[$field['name']] = array('lat' => (float) $record['latitude'], 'lng' => (float) $record['longitude']);
                        }
                        break;
                    case 'city':
                    case 'categories':
                    case 'selectable':
                        $is_multiple = false;
                        if(is_string($value) && strpos($value, '{"') !== false) { // preferences
                            $saved_values = json_decode($value, true);
                            $is_multiple = true;
                        } else {
                            if(is_array($value)) {
                                $saved_values = $value;
                            } else {
                                $saved_values = array($value);
                            }
                        }
                        switch($field['name']) {
                            case 'country_iso':
                            case 'state_id':
                            case 'city_id':
                            case 'city':
                            case 'location':
                            case 'categories':
                            case 'slider_group_id':
                                if($field['name'] == 'country_iso') {
                                    $rows = $this->db->where("FIND_IN_SET(iso, '". implode(',', $saved_values) ."')")->select('id, name')->get('countries')->result_array();
                                } else {
                                    switch($field['name']) {
                                        case 'state_id':
                                            $table_name = 'states';
                                            break;
                                        case 'city_id':
                                        case 'city':
                                        case 'location':
                                            $table_name = 'cities';
                                            break;
                                        case 'slider_group_id':
                                            $table_name = 'mobile_slider_groups';
                                            break;
                                        case 'categories':
                                            $table_name = 'categories';
                                            $saved_values = !empty($record['categories_set']) ? explode(',', $record['categories_set']) : '';
                                            break;
                                        default:                                            
                                            $table_name = $field['name'];
                                    }
                                    if($field['name'] == 'city') { // users table
                                        if(!empty($record['city_id'])) {
                                            $saved_values = array($record['city_id']);
                                        } else {
                                            $saved_values = array();
                                        }
                                    }
                                    $rows = empty($saved_values) ? array() : $this->db->where('id IN (' . implode(',', $saved_values) . ')')->select('id, name')->get($table_name)->result_array();
                                }
                                if($rows) {
                                    if(!empty($field['params']['isMultiple'])) {
                                        foreach($rows as $row) {
                                            $result[$field['name']][] = $row;
                                        }
                                    } else {
                                        $result[$field['name']] = $rows[0];
                                    }
                                }
                                break;
                            default:
                                foreach($saved_values as $key => $saved_value) {
                                    if($is_multiple) {
                                        $saved_value = $key;
                                    }
                                    if(!empty($saved_value) && $saved_value != 'null') {
                                        $name = !empty($option_values[$saved_value]) ? (!empty($option_values[$saved_value][1]) ? lang($option_values[$saved_value][0]) : $option_values[$saved_value][0]) : $saved_value;
                                        if(!empty($field['params']['isMultiple'])) {
                                            $result[$field['name']][] = array('id' => $saved_value, 'name' => $name);
                                        } else {
                                            $result[$field['name']] = array('id' => $saved_value, 'name' => $name);
                                        }
                                    }
                                }
                        }
                        break;
                    default:
                        switch($field['name']) {
                            case 'facilities':
                                if(!empty($value) && $value != 'null') {
                                    $value = json_decode($value);
                                    $keys = array();
                                    foreach($value as $key => $fvalue) {
                                        $keys[] = array('name' => lang('label_' . $key), 'id' => $key);
                                    }
                                    $result[$field['name']] = $keys;
                                } else {
                                    $result[$field['name']] = '';
                                }
                                break;
                            case 'distances': // properties module
                                if(!empty($value) && $value != 'null') {
                                    $value = json_decode($value);
                                    $keys = array();
                                    foreach($value as $key => $fvalue) {
                                        $result['distances[' . $key . ']'] = empty($fvalue) ? 0 : $fvalue;
                                    }                                    
                                } else {
                                    foreach($field['options'] as $distance) {
                                        $result['distances[' . $distance . ']'] = 0;
                                    }
                                }
                                break;
                            default:
                                $result[$field['name']] = $value;
                        }
                }
            }
        }
        return $result;
    }
    
    /***************************************************************************
     * START OF HOME AND COMMON MODULES
     ***************************************************************************/
    
    /**
     * Variables required throughout the application
     */
    public function loading($user_id) {
        $this->lang->load('mobiles/mobiles');
        $user = array();
        if(!empty($user_id)) {
            // save player id
            $this->save_player_id($user_id);
            $this->save_push_token($user_id);
            $user_info = $this->get_user_profile($user_id, false);
            if($user_info) {
                $user['image'] = $user_info->profile_image;
                $user['name'] = $user_info->display_name;
                $user['role_id'] = $user_info->role_id;
                if(!empty($user_info->upgrade_packages)) {
                    $user['premium_package_id'] = $user_info->upgrade_packages[0];
                }
            }
        }
        // $this->mb_pages_cache
        $cacheID = 'loading';
        if(!$result = $this->mb_pages_cache->load($cacheID)) { // cache results
            // module languages
            $modules_languages = array();
            foreach(gc_modules() as $module) {
                $modules_languages[$module] = array('singular' => lang('singular_' . $module), 'plural' => lang($module));
            }
            $languages = array(
                'loading' => lang('loading'),
                'add' => lang('add'),
                'saving' => lang('saving'),
                'btn_ok' => lang('btn_ok'),
                'success_header' => lang('success_header'),
                'alert_header' => lang('alert_header'),
                'alert_message' => lang('error_no_record_found'),
                'upgrade_membership' => lang('upgrade_membership'),
                'view_all' => lang('view_all'),
                'logout' => lang('bf_action_logout'),
                'network_error' => lang('network_error'),
                'msg_success' => lang('msg_success'),
                'msg_error' => lang('msg_error'),
                'submit' => lang('gc_action_submit'),
                'home' => lang('bf_home'),
                'matches' => lang('matches'),
                'activities' => lang('activities'),
                'search' => lang('bf_action_search'),
                'chat' => lang('label_chat'),
                'read_more' => lang('label_read_more'),
                'more_info' => lang('more_info'),
                'less_info' => lang('less_info'),
                'keywords' => lang('label_keywords'),
                'clear' => lang('bf_clear'),
                'confirm' => lang('confirm'),
                'close' => lang('form_close'),
                'open_time' => lang('open_time'),
                'close_time' => lang('close_time'),
                'category' => lang('label_category'),
                'keyword' => lang('label_keyword'),
                'search_suggestions' => lang('search_suggestions'),
                'search_recent' => lang('search_recent'),
                'clear_all' => lang('clear_all'),
                'open' => lang('label_open'),
                'open_now' => lang('open_now'),
                'closed' => lang('label_closed'),
                'filter' => lang('gc_action_filter'),
                'search_results' => lang('search_results'),
                'search_tips' => lang('search_tips'),
                'spelling_mistake' => lang('spelling_mistake'),
                'alternate_keyword' => lang('alternate_keyword'),
                'business_hours' => lang('label_business_hours'),
                'take_picture' => lang('take_picture'),
                'menu' => lang('members_menu'),
                'warning' => lang('label_warning'),
                'delete_confirm' => lang('purge_del_confirm'),
                'checkout' => lang('label_checkout'),
                'cart_empty' => lang('cart_empty'),
                'cart_empty_text' => lang('cart_empty_text'),
                'cart_warning' => lang('cart_warning'),
                'cancel' => lang('bf_action_cancel'),
                'product_save' => lang('product_save'),
                'modules' => $modules_languages
            );            
            
            // navbar links
            $where = 'FIND_IN_SET(1, group_ids) AND status = 1';
            $nav_links = $this->db->where($where)->order_by('priority')->get('mobile_links')->result();
            $settings = json_decode(settings_item('mobile.settings'), true);
            $settings['google_api_key'] = settings_item('site.google_api_key');
            $settings['youtube_api_key'] = 'AIzaSyCO_Df6WDKmGJE4ZB0DrY8kDurXBp-zaWA';
            $settings['limit'] = (int) settings_item('site.front_list_limit');
            $settings['logo'] = base_url('assets/images/logo.png');        
            $settings['currency'] = settings_item('site.currency_symbol') . ' ';
            $settings['sms_hash'] = settings_item('site.sms_hash');
            
            // search fields and default values
            $search_fields = $this->search_fields();
            $this->load->model('locations/cities_model');
            $city_info = $this->cities_model->get_city_info((int) settings_item('adv.default_city'));
            $address = $city_info ? $city_info->name : 'Yamunanagar';
            $settings['coords'] = array('lat' => '30.1290', 'lng' => '77.2674', 'address' => $address);
            $default_values = array(
                'module' => array('id' => 'listings', 'name' => lang('listings')),
                'city_id' => $city_info ? array('id' => $city_info->id, 'name' => $city_info->name) : '',
                'category_id' => array('id' => 0, 'name' => lang('header_select_category')),
                'sort_by' => array('id' => 'relevance', 'name' => lang('label_relevance'))
            );
            $result = array(
                'success' => true,
                'languages' => $languages,
                'nav_links' => $nav_links,
                'settings' => $settings,
                'fields' => array('search' => $search_fields, 'values' => $default_values, 'filter' => $this->search_fields(true))
            );
            $this->mb_pages_cache->save($result, $cacheID, array('loading'));
        }
        $result['user'] = $user;
        
        // prepare links
        $links = array();
        $nav_links = $result['nav_links'];
        unset($result['nav_links']);
        if($nav_links) {
            foreach($nav_links as $nav_link) {
                if((empty($nav_link->permission) && $nav_link->access_type == 0) || ($nav_link->access_type == 2 && $user_id == 0) || (!empty($user_info) && $this->auth->has_permission($nav_link->permission, $user_info->role_id))) {
                    $links[] = array(
                        'label' => !empty($nav_link->label_lang) ? lang($nav_link->label) : $nav_link->label,
                        'icon' => $nav_link->mobile_icon,
                        'url' => $nav_link->link_url
                    );
                }
            }
        }
        $result['links'] = $links;
        return $result;
    }
    
    /**
     * Get Loggedin User Profile
     * @param number $user_id
     * @return array
     */
    public function get_user_profile($user_id, $with_package = true) {
        $this->load->model('users/user_model');
        // frequently used information stored in a session variable to avoid unnecessary queries
        $user = $this->db->select('role_id, display_name AS name, display_name, email, phone_number, address, postcode, city_id, profile_image, wallets')->where('id', $user_id)->get('users')->row();
        if($user) {
            $user->profile_image = gc_image_url($user->profile_image, true, true, array('module' => 'users')); // base_url('assets/images/' . (!empty($user->profile_image) ?  'users/' . $user->profile_image : 'dp.jpg'));
            if($with_package) {
                // Display upgrade packages to the user and promote sales
                $this->load->model('packages/packages_model');
                $user->upgrade_package = $this->packages_model->get_upgrade_packages($user_id);
            }
        }
        return $user;
        // return $this->auth->profile_session($user_id, false);
    }
    
    /**
     * Home Modules
     * @param number $user_id
     * @return array
     */
    public function home($user_id) {
        $this->load->library('layouts');
        $result = array();
        $marquee = array();
        $cacheID = 'mobile_components';
        if(!$records = $this->mb_pages_cache->load($cacheID)) {
            $records = $this->db->where(array('deleted' => 0, 'status' => 1))->order_by('priority')->get('mobile_components')->result();
            $this->mb_pages_cache->save($records, $cacheID, array('mobile_components', 'components'));
        }
        if($records) {
            foreach($records as $record) {                
                $label = !empty($record->label_lang) ? lang($record->label) : $record->label;
                $params = array();
                $saved_params = json_decode($record->params ?? '', true);
                if(!empty($saved_params) && is_array($saved_params)) {
                    foreach($saved_params as $key => $saved_param) {
                        if(array_key_exists('key', $saved_param)) {
                            if(!empty($saved_param['key'])) {
                                $params[$saved_param['key']] = $saved_param['value'];
                            }
                        } else {
                            $params[$key] = $saved_param;
                        }
                    }
                }
                switch($record->component) {
                    case 'sliders':
                        // sliders
                        $banners = $this->get_sliders(1);
                        if($banners) {
                            $result['blocks'][] = array(
                                'type' => $record->component,
                                'label' => $label,
                                'icon' => $record->mobile_icon,
                                'records' => $banners
                            );
                        }
                        break;
                    case 'categories':
                        $module = !empty($params['module']) ? $params['module'] : 'listings';                        
                        $limit = !empty($params['limit']) ? $params['limit'] : 50;
                        $cacheID = 'categories_' . $module . '_' . $limit; // . '_' . $record->extra_params;
                        $cacheID = str_replace('-', '_', $this->slug->create_slug($cacheID));
                        if(!$component_categories = $this->mb_pages_cache->load($cacheID)) {
                            $colors = array('primary', 'secondary', 'tertiary', 'success', 'warning', 'danger', 'medium', 'dark');
                            $this->load->model('categories/categories_model');
                            $categories = $this->categories_model->categories(0, 0, $module, null, $limit);
                            $i = 0;
                            $categories_data = array();
                            // slider categories
                            if($categories) {
                                $j = 0;
                                $m = 0;
                                foreach($categories as $category) {
                                    if(!isset($colors[$m])) {
                                        $m = 0;
                                    }
                                    $color = $colors[$m];
                                    $categories_data[$i][] = array(
                                        'name' => $category->name,
                                        'icon' => $category->mobile_icon,
                                        'image' => gc_image_url($category->image, true, true, array('module' => 'categories')),//base_url('assets/images/thumbs/thumb__' . $category->image),
                                        'color' => $color,
                                        'url' => '/tabs/modules/' . $module . '/categories/' . $category->id . '/all/0'
                                    );
                                    $j++;
                                    $m++;
                                    if($j%6 == 0) {
                                        $i++;
                                        $j = 0;
                                    }
                                }
                                $component_categories = array(
                                    'type' => $record->component,
                                    'label' => $label,
                                    'icon' => $record->mobile_icon,
                                    'records' => $categories_data
                                ); 
                            }
                            // $categories = $this->layouts->widget_categories('listings', array(), 0, 50, true);
                            $this->mb_pages_cache->save($component_categories, $cacheID, array('categories'));
                        }
                        $result['blocks'][] = $component_categories;
                        break;
                    case 'featured':
                    case 'popular':
                    case 'recently_added':
                    case 'last_viewed':                        
                        $module = !empty($params['module']) ? $params['module'] : 'listings';
                        $limit = !empty($params['limit']) ? $params['limit'] : 50;
                        if($module != 'last_viewed') {
                            $lat = !empty($_GET['lat']) ? $_GET['lat'] : '';
                            $lng = !empty($_GET['lng']) ? $_GET['lng'] : '';
                            $cacheID = 'fprlv' . '_' . $record->component . '_' . $module . '_' . $limit . '_' . $lat . '_' . $lng; // . '_' . $record->extra_params;
                            $cacheID = str_replace('-', '_', $this->slug->create_slug($cacheID));
                            if(!$component_fprlv = $this->mb_pages_cache->load($cacheID)) {
                                // home_fprlv($module, $limit, $record, $label, $user_id)
                                unset($_GET['lat']); // remove to display location specific listings
                                unset($_GET['lng']); // remove to display location specific listings
                                $component_fprlv = $this->home_fprlv($module, $limit, $record, $label, $user_id); // $user_id is only required only in case of last_viewed items
                                $this->mb_pages_cache->save($component_fprlv, $cacheID, array($module));
                            }
                        } else {
                            $component_fprlv = $this->home_fprlv($module, $limit, $record, $label, $user_id);
                        }
                        $result['blocks'][] = $component_fprlv;
                        break;
                    case 'testimonials':
                        $limit = !empty($params['limit']) ? $params['limit'] : 10;
                        $cacheID = 'testimonials_' . $limit;
                        if(!$component_testimonials = $this->mb_pages_cache->load($cacheID)) {
                            $testimonials = $this->layouts->widget_testimonials($limit, 'testimonials', 90);
                            $component_testimonials = array(
                                'type' => $record->component,
                                'label' => $label,
                                'icon' => $record->mobile_icon,
                                'records' => $testimonials
                            );
                            $this->mb_pages_cache->save($component_testimonials, $cacheID, array($record->component));
                        }
                        $result['blocks'][] = $component_testimonials;
                        break;
                    case 'marquee':
                        $speed = !empty($params['speed']) ? $params['speed'] : 30;
                        $class = !empty($params['class']) ? $params['class'] : 'bg-theme-secondary';
                        $text = !empty($params['text']) ? $params['text'] : '';
                        $text_lang = !empty($params['text_lang']) ? (int) $params['text_lang'] : 0;
                        $timeout = !empty($params['timeout']) ? (int) $params['timeout'] : 1000;
                        $marquee = array('text' => !empty($text_lang) ? lang($text) : $text, 'timeout' => $timeout);
                        $result['blocks'][] = array(
                            'type' => $record->component,
                            'label' => $label,
                            'icon' => $record->mobile_icon,
                            'speed' => $speed,
                            'class' => $class
                        );
                        break;
                    case 'videos':
                    case 'images':
                        $records = array();
                        foreach($params as $pkey => $pvalue) {
                            switch($pkey) {
                                case 'image':
                                    $file_data['image'] = base_url('assets/images/' . $pvalue);
                                    break;
                                case 'youtube_id':
                                    $file_data['youtube_id'] = $pvalue;
                                    break;
                                case 'url':
                                    $file_data['url'] = $pvalue;
                                    break;
                            }
                        }
                        $result['blocks'][] = array(
                            'type' => $record->component,
                            'label' => $label,
                            'icon' => $record->mobile_icon,
                            'records' => $records
                        );
                        break;
                }
            }
        }
        if(!empty($marquee)) {
            $result['marquee'] = $marquee;
        }
        $result['success'] = true;
        return $result;
    }
    
    /**
     * Format Featured, Popular, Recently Viewed
     * @param object $record
     */
    public function home_fprlv($module, $limit, $record, $label, $user_id) {     
        $data = $this->format_fprlv($module, $record->component, $limit, $user_id);
        $fprlv = array(
            'type' => $record->view_type, // type of component e.g. app-slider-overlay, app-slider-card, app-view
            'label' => $label . ' ' . lang($module),
            'icon' => $record->mobile_icon,
            'records' => $data
        );
        if(!empty($data)) {            
            $fprlv['extra'] = array(
                'view_all' => lang('view_all'),
                'image' => isset($data[0]['image']) ? $data[0]['image'] : gc_site_image('no-image'),
                'url' => '/modules/' . $module . '/search/0/' . $record->component . '/0'
            );
        }
        return $fprlv;
    }
    
    /**
     * Format FPRLV rows
     * @param string $module
     * @param string $block
     * @param number $limit
     * @return string[][]|NULL[][]|number[][]
     */
    public function format_fprlv($module, $block, $limit, $user_id, $records = array()) {
        if(empty($records)) {
            if($block == 'last_viewed') {
                $records = $this->gc->get_recently_viewed($module, $limit, $user_id);
            } else {
                $this->load->model('listings/listings_model');
                $records = $this->listings_model->get_fprlv($module, $block, $limit);
            }
        }
        $data = array();
        if(!empty($records)) {
            $thumb = true;
            $truncate_length = 200;
            foreach($records as $row) {
                if(!empty($row->cover)) {
                    $image = gc_image_url($row->cover, $thumb);
                } else if(!empty($row->logo)) {
                    $image = gc_image_url($row->logo, $thumb);
                } else if(!empty($row->youtube_id)) {
                    $image = 'https://img.youtube.com/vi/' . $row->youtube_id . '/0.jpg';
                } else if(!empty($row->image)) {
                    $image = gc_image_url($row->image, $thumb);
                } else {
                    $image = gc_site_image('no-image');
                }
                $description = truncate_text(strip_tags($row->description), intval($truncate_length));
                $row_data = array(
                    'id' => $row->id,
                    'title' => $row->title,
                    // 'description' => $description,
                    'image' => $image,
                    'average_rating' => !empty($row->average_rating) ? $row->average_rating : 0,
                    'url' => '/tabs/modules/detail/' . $module . '/' . $row->id
                );
                if(isset($row->address)) {
                    // $row_data['address'] = gc_address_format($row->address, (isset($row->locality) ? $row->locality : ''), $row->city, $row->state, (isset($row->postcode) ? $row->postcode : ''), $row->country, true);
                    // $row_data['address'] = $row->address;
                }
                if(isset($row->user_distance)) {                    
                    $row_data['user_distance'] = $row->user_distance . ' ' . lang('label_km');
                }
                if(!empty($row->discount)) {
                    if(!empty($row->type) && $row->type == 'F') {
                        $row_data['discount'] = currency_format($row->discount) . ' ' . lang('label_off');   
                    } else {
                        $row_data['discount'] = $row->discount . '% ' . lang('label_off');
                    }
                }
                if(!empty($row->offer_price)) {
                    $savings = calculate_savings($row->price, $row->offer_price, '');
                    if(!empty($savings['savings_in_percentage'])) {
                        $row_data['discount'] = $savings['savings_in_percentage'] . '%'  . ' ' . lang('label_off');  
                        $row_data['price_with_currency'] = currency_format($row->offer_price);
                        $row_data['html'] = '<del class="ml-1 mr-1 small">' . $savings['price'] . '</del>';
                        $row_data['savings'] = $savings['savings'];
                    }
                }
                $data[] = $row_data;
            }
        }
        return $data;
    }
    
    /**
     * Format News, Success Stories and Pages
     * @param array $record
     * @return string[]|NULL[]
     */
    public function format_others($module, $record, $extra = array()) {
        $truncate_limit = isset($extra['truncate_limit']) ? $extra['truncate_limit'] : 70;
        $row = array(
            'id' => $record->id,
            'title' => $record->title,
            'description' => $module == 'faqs' ? $record->description: truncate_text($record->description, $truncate_limit, true),
            'hits' => $record->hits,
            'created_on' => gc_date_format($record->created_on, true),
            'modified_on' => gc_date_format($record->modified_on, true),
        );
        if($module == 'success_stories') {
            if(!empty($record->images)) {
                $images = $this->gc->process_images($record->images);
                $image = $images[0];
            } else {
                $image['original'] = gc_site_image('no-image');
            }
            $row['image'] = $image;
        } else if($module == 'news') {
            if(!empty($record->image)) {
                $row['image'] = gc_image_url($record->image, false);
            } else {
                $row['image'] = gc_site_image('no-image');
            }
        }
        return $row;
    }
    
    /**
     * Feedback/Callback/Contact Query Fields
     * @param string $module
     * @return array
     */
    public function feedback_callback_query($module = 'feedbacks', $extra = array()) {
        $prefix = '';
        $phone_check = '^(\+?(\d{1}|\d{2}|\d{3})[- ]?)?\d{3}[- ]?\d{3}[- ]?\d{4}$'; // phone check
        $fields = array();
        /*$fields = array(
            array('name' => $prefix . 'name', 'label' => lang('label_name'), 'type' => 'text', 'rules' => array('required' => 'required')),
            array('name' => $prefix . 'email', 'label' => lang('label_email'), 'type' => 'text', 'rules' => array('required' => 'required', 'email' => 'email')),
            array('name' => $prefix . 'phone_number', 'label' => lang('label_phone_number'), 'type' => 'text', 'rules' => array('required' => 'required', 'pattern' => $phone_check)),
        );*/
        switch($module) {
            case 'feedbacks':
                $fields[] = array('name' => $prefix . 'subject', 'label' => lang('label_title'), 'type' => 'text', 'rules' => array('required' => 'required'));
                $fields[] = array('name' => $prefix . 'message', 'label' => lang('label_message'), 'type' => 'textarea', 'rules' => array('required' => 'required'));
                break;
            case 'callbacks':
                $fields[] = array('name' => $prefix . 'phone_number', 'label' => lang('label_phone_number'), 'type' => 'text', 'rules' => array('required' => 'required', 'pattern' => $phone_check));
                $fields[] = array(
                    'name' => $prefix . 'country_iso',
                    'label' => lang('label_country'),
                    'type' => 'selectable',
                    'options' => array(),
                    'params' => array(
                        'max_items' => 1,
                        'onChange' => true,
                        'canSearch' => true,
                        'onSearch' => true,
                        'hasInfiniteScroll' => true,
                        'onInfiniteScroll' => true,
                        'urlParams' => 'module=countries',
                        'isMultiple' => false
                    ),
                    'rules' => array('required' => 'required')
                );
                break;
            case 'contact_query':
                $fields = array(
                    array('name' => 'message', 'label' => lang('label_message'), 'type' => 'textarea', 'rules' => array('required' => 'required')),
                    // array('name' => 'name', 'label' => lang('label_name'), 'type' => 'text', 'rules' => array('required' => 'required')),
                    // array('name' => 'email', 'label' => lang('label_email'), 'type' => 'text', 'rules' => array('required' => 'required', 'email' => 'email')),
                    // array('name' => 'phone_number', 'label' => lang('label_phone_number'), 'type' => 'text', 'rules' => array('required' => 'required', 'pattern' => $phone_check)),
                );
                break;
            case 'erroneous':
                $this->load->model('erroneous/erroneous_model');
                $incorrect_options = array();
                $incorrect_types = $this->erroneous_model->incorrect_options(0, true);
                foreach($incorrect_types as $key => $incorrect_type) {
                    $incorrect_options[] = array(
                        'id' => $key,
                        'name' => $incorrect_type
                    );
                }
                $fields[] = array('name' => $prefix . 'type_id', 'label' => lang('label_type'), 'type' => 'select', 'options' => $incorrect_options, 'rules' => array('required' => 'required'));
                $fields[] = array('name' => $prefix . 'description', 'label' => lang('label_description'), 'type' => 'textarea', 'rules' => array('required' => 'required'));
                break;
            case 'review':
                $fields = array(
                    // array('name' => 'name', 'label' => lang('label_name'), 'type' => 'text', 'rules' => array('required' => 'required')),
                    array('name' => 'title', 'label' => lang('label_title'), 'type' => 'text', 'rules' => array('required' => 'required')),
                    array('name' => 'rating', 'label' => lang('gc_menu_ratings'), 'type' => 'rating', 'rules' => array()),
                    array('name' => 'message', 'label' => lang('label_comment'), 'type' => 'textarea', 'rules' => array('required' => 'required')),
                    // array('name' => 'url', 'label' => lang('label_website'), 'type' => 'text', 'rules' => array()),
                );
                break;
        }
        return $this->rest_fields($fields);
    }
    
    /**
     * Search Fields to be displayed to the user
     * @return array
     */
    public function search_fields($filter_fields = false) {
        $prefix = '';
        if(!$filter_fields) {
            $module_options = array();
            foreach(gc_modules() as $module) {
                $module_options[] = array('id' => $module, 'name' => lang($module));
            }
            $fields = array();
            $fields[] = array(
                'name' => $prefix . 'module',
                'label' => lang('label_module'),
                'type' => 'selectable',
                'options' => $module_options
            );
            /*$fields[] = array(
                'name' => $prefix . 'location',
                'label' => lang('label_location'),
                'type' => 'map',
                'options' => array(),
                'params' => array(
                    'searchControl' => true
                ),
                'rules' => array()
            );*/
            $fields[] = array(
                'name' => $prefix . 'city_id',
                'label' => lang('label_city'),
                'type' => 'selectable',
                'options' => array(),
                'params' => array(
                    'max_items' => 1,
                    'onChange' => true,
                    'canSearch' => true,
                    'canClear' => true,
                    'onSearch' => true,
                    'hasInfiniteScroll' => true,
                    'onInfiniteScroll' => true,
                    'urlParams' => 'module=locations&md=cities',
                    'isMultiple' => false
                ),
                'rules' => array()
            );
            $fields[] = array(
                'name' => $prefix . 'category_id',
                'label' => lang('label_category'),
                'type' => 'selectable',
                'placeholder' => 'All Categ',
                'options' => array(),
                'params' => array(
                    'max_items' => 1,
                    'onChange' => true,
                    'canSearch' => true,
                    'canClear' => true,
                    'onSearch' => true,
                    'hasInfiniteScroll' => true,
                    'onInfiniteScroll' => true,
                    'urlParams' => 'module=categories',
                    'isMultiple' => false,
                    // 'placeholder' => 'All Categories'
                ),
                'rules' => array()
            );
            // $fields[] = array('name' => $prefix . 'q', 'label' => lang('label_search'), 'type' => 'searchbar', 'rules' => array('required' => 'required'));
        } else {
            $fields = array();
            $sort_by_options = array(
                array(
                    'id' => 'relevance',
                    'name' => lang('label_relevance')
                ),
                array(
                    'id' => 'top_rated',
                    'name' => lang('label_top_rated')
                ),
                array(
                    'id' => 'alphabetically',
                    'name' => lang('label_alphabetically')
                )
            );
            $fields[] = array(
                'name' => $prefix . 'sort_by',
                'label' => lang('sorted_by'),
                'type' => 'selectable',
                'options' => $sort_by_options
            );
        }
        return $this->rest_fields($fields);
    }
    
    /**
     * Allow users to save business hours
     * @param number $listing_id
     * @return array|string[][]|boolean[][]|array[][]
     */
    public function business_hours($listing_id, $logged_in_user_id = 0) {
        $module_info = $this->get_info('listings', $listing_id, $logged_in_user_id, true);
        if(!$module_info) {
            return array('success' => false);
        }
        $this->lang->load('listings/listings');
        $record = new stdClass();
        $records = $this->db->where('listing_id', $listing_id)->get('business_hours')->result();
        if($records) {
            foreach($records as $row) {
                $record->{'open_time_' . $row->day_of_week} = $row->open_time;
                $record->{'close_time_' . $row->day_of_week} = $row->close_time;
            }
        } else {
            for($i = 1; $i <= 6; $i++) {
                $record->{'open_time_' . $i} = '09:00';
                $record->{'close_time_' . $i} = '19:00';
            }
        }
        $days = array('monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday', 'sunday'); 
        $fields = array();
        $i = 1;
        $params = array(
            'max' => date('Y-m-d'),
            'min' => '1901-01-01',
            'format' => 'HH:mm'
        );
        foreach($days as $day) {
            $fields[] = array('name' => $i, 'label' => lang('label_' . $day), 'type' => 'businesshours', 'params' => $params);
            $fields[] = array('name' => 'open_time_' . $i, 'label' => lang('label_' . $day) . ' ' . lang('label_open'), 'type' => 'hidden', 'params' => $params);
            $fields[] = array('name' => 'close_time_' . $i, 'label' => lang('label_' . $day) . ' ' . lang('label_close'), 'type' => 'hidden', 'params' => $params);
            $i++;
        }
        $result = array(
            'success' => true,
            'fields' => $this->rest_fields($fields),
            'record' => $record,
            'extra' => array(
                'name' => lang('business_hours'),
                'icon' => 'alarm-outline',
                'help' => lang('message_business_hours'),
                'toolbar_title' => $module_info->title
            )
        );
        return $result;
    }
    
    /***************************************************************************
     * END OF HOME AND COMMON MODULES
     ***************************************************************************/
    
    /***************************************************************************
     * START OF MEMBER AREA
     ***************************************************************************/
    /**
     * Member Area - Display Links on My Account Link click
     */
    public function member_area($user_id) {
        $lat = !empty($_GET['lat']) && $_GET['lat'] != 'undefined' ? $_GET['lat'] : 0;
        $lng = !empty($_GET['lng']) && $_GET['lng'] != 'undefined' ? $_GET['lng'] : 0;
        $this->lang->load('members/members');
        $result = array();
        $user_info = $this->get_user_profile($user_id, false);
        // user information
        $result['user'] = array(
            'welcome' => lang('welcome'),
            'view_all' => lang('view_all'),
            'name' => $user_info ? $user_info->display_name : '',
            'image' => $user_info ? $user_info->profile_image : ''
        );
        // navbar links
        $cacheID = 'links_' . $user_id;
        if(!$links = $this->mb_pages_cache->load($cacheID)) {
            $links = array();
            $where = 'FIND_IN_SET(2, group_ids) AND status = 1';
            $nav_links = $this->db->where($where)->order_by('priority')->get('mobile_links')->result();
            if($nav_links) {
                foreach($nav_links as $nav_link) {
                    $color = gc_process_params($nav_link, 'color');
                    $color = empty($color) ? 'secondary' : $color;
                    if(empty($nav_link->permission) || (!empty($user_info) && $this->auth->has_permission($nav_link->permission, $user_info->role_id))) {
                            $url = $nav_link->link_url;
                            $process_link = true;
                            if($url == '/tabs/packages') {                           
                                $this->load->model('members/members_model');
                                $packages = $this->members_model->get_packages('new', 0, $user_id);
                                if(empty($packages)) {
                                    $process_link = false;
                                }
                            } else {
                                if(strpos($nav_link->link_url, 'edit-module') !== false) {
                                    $url_params = explode('/', $nav_link->link_url);
                                    $link_toggle_module = $url_params[3];
                                    if(!empty($link_toggle_module)) {
                                        if(in_array($link_toggle_module, gc_modules())) {
                                            $module = $link_toggle_module;
                                            $rows = $this->db->select('id')->where(array('deleted' => 0, 'user_id' => $user_id))->get($module)->result_array();
                                            $rows_count = count($rows);
                                            switch($rows_count) {
                                                case 0: // redirect user to the add module page
                                                    $url = '/tabs/crud/add/0/' . $module . '/0';
                                                    break;
                                                case 1: // redirect user to the edit module page
                                                    $url = '/tabs/member/dashboard/' . $module . '/' . $rows[0]['id'];
                                                    break;
                                                default: // redirect user to the view page where a user can select a row
                                                    $url = 'tabs/modules/' . $module . '/search/0/all/1';
                                            }
                                        } /*else if($link_toggle_module == 'users') {
                                            $url .= '/' . $user_id;
                                        }*/
                                    }
                                }
                            }
                            if($process_link) {
                                $links[] = array(
                                    'label' => !empty($nav_link->label_lang) ? lang($nav_link->label) : $nav_link->label,
                                    'icon' => $nav_link->mobile_icon,
                                    'icon_color' => $color,
                                    'url' => $url
                                );
                            }
                        }
                }
            }
            $this->mb_pages_cache->save($links, $cacheID, array('mobile_links', 'links_' . $user_id));
        }
        $result['links'] = $links;
        $result['success'] = true;
        $banners = $this->get_sliders(2);
        $result['slides'] = $banners;
        // last viewed items
        $module = 'listings';
        $limit = 10;        
        // $last_viewed_records = $this->layouts->widget_sliders($module, 'last_viewed', $limit, 200, 2, array(), null, true, array('user_id' => $user_id));
        $result['module_rows'] = array(
            'type' => 'card', // type of component e.g. app-slider-overlay, app-slider-card, app-view
            'label' => lang('last_viewed') . ' ' . lang($module),
            'extra' => array(),
            'records' => $this->format_fprlv('listings', 'last_viewed', $limit, $user_id, array())
        );
        /*
        $result['marquee'] = array(
            'speed' => 30,
            'timeout' => 1000,
            // 'class' => 'bg-theme-secondary',
            'text' => 'Testing Member Area Testing Member Area Testing Member Area Testing Member Area Testing Member Area Testing Member Area Testing Member Area'
        );*/
        return $result;
    }
    
    /**
     * Members Area - Index Page with Links and Packages Information
     * @param number $user_id
     * @return array[]
     */
    public function member_module($module, $id, $user_id) {
        $this->lang->load('members/members');
        $this->lang->load('listings/listings');
        $record = $this->db->where(array('id' => $id, 'deleted' => 0))->get($module)->row();
        if(!$record || $record->user_id != $user_id) {
            return array('success' => false);
        }
        $package = $this->db->where('id', $record->package_id)->get('packages')->row();
        if($package) {
            $record->pyg = $package->pyg;
            $record->module_limit = $package->module_limit;
        }
        $links = array();
        $extra_links = array(); // package_info link
        $member_links = $this->layouts->widget_member_links($module, $record, array('with_edit' => true, 'with_view' => true, 'with_delete' => true, 'mobile' => true));
        foreach($member_links as $link) {
            switch($link['name']) {
                case 'change_package':
                    break;
                /*case 'package_info':
                    $extra_links[] = array(
                        'name' => $link['label'],
                        'icon' => $link['mobile_icon'],
                        'url' => 'displayPackageInfo',
                    );
                    break;*/
                default:
                    $links[] = array(
                        'name' => $link['label'],
                        'icon' => $link['mobile_icon'],
                        'url' => $link['mobile_url'],
                    );
            }
        }
        $package_info = array();
        if(!empty($package)) {
            if(!empty($package->expires_on)) {
                $expires_on = $package->expires_on;
            } else if(!empty($package->forever) || empty($package->duration)) {
                $expires_on = lang('label_never');
            } else if(!empty($package->expires_on)) {
                $expires_on = $package->expires_on;
            }
            // Add package duration days to profile create date
            $expires_on = date('Y-m-d',strtotime('+' . $package->duration . ' days', strtotime($record->created_on)));
            $package_info['languages'] = array(
                'heading' => lang('package_info'),
                'created_on' => lang('created_on'),
                'expires_on' => lang('label_expires_on')
            );
            $package_info['title'] = $package->title;
            $package_info['created_on'] = gc_date_format($record->created_on);
            $package_info['expires_on'] = lang('label_never'); // gc_date_format($expires_on, false);
            $this->load->model('packages/packages_model');
            $this->lang->load('packages/packages');
            $processed_info = $this->packages_model->package_info($package->id, false, $package, $module, true);
            if(!empty($processed_info)) {
                $package_info['description'] = $processed_info['description'];
                $package_info['trusted_seal'] = $processed_info['trusted_seal'];
                $package_info['modules_info'] = $processed_info['modules_info'];
            }
        }
        $this->lang->load('groups/groups');
        $rating_text = '';
        if(!empty($record->rating_users)) {
            $rating_text = decimal_convert($record->ratings) . ' ' . lang('label_ratings') . ' ' . $record->rating_users . ' ' . lang('label_users_rated');
        }
        $result = array(
            'success' => true,
            'title' => $record->title,
            'trusted_seal' => !empty($package->trusted_seal) ? site_url('assets/images/' . $package->trusted_seal) : '',
            'description' => truncate_text($record->description, 200),
            'cover' => gc_image_url($record->cover, false),
            'percent_completion' => !empty($record->percent_completion) ? $record->percent_completion : 0,
            'ratings' => isset($record->ratings) ? $record->ratings : '',
            'rating_users' => isset($record->rating_users) ? $record->rating_users : '',
            'rating_text' => $rating_text,
            'links' => $links,
            'extra_links' => $extra_links, // package info link
            'package' => $package_info,
            'extra' => array('toolbar_title' => lang('singular_' . $module) . ' - ' . $record->title, 'completed' => lang('label_completed'))
        );
        if(gc_module_settings($module, 'locations') == 1) {	
            $result['address'] = $record->address;
        }
        if($module == 'listings') { // display percent tree to the user
            $this->load->model('groups/groups_model');
            $result['percent_tree'] = $this->groups_model->percentage_tree('listings');
        }
        return $result;
    }
    
    /**
     * Group Form
     * @param integer $id - User ID whose proflie to be edited
     * @param integer $group_id - ID of the Group
     * @param string $role - Role
     * @param number $user_id - ID of the user who is editing the profile
     * @param array $rest_extra
     */
    public function group_form($module, $id, $group_id, $role= 'admin', $user_id = 0, $rest_extra = array()) {
        $mobile = $role == 'mobile' ? true : false;
        $this->lang->load('groups/groups');
        $this->lang->load('listings/listings');
        $this->load->model('fields/fields_model');
        $this->load->model('links/links_model');
        $this->load->model('users/user_model');
        $extra_params = array();
        if(!$mobile) {
            $role_id = $this->auth->role_id();
            // group edit request page
            $pg = (int) $this->input->get('pg');
            if($pg == 1 && $this->session->userdata('last_visited_url')) {
                // if($role != 'admin' && $this->session->userdata('last_visited_url')) {
                // request is made from the profile detail page, redirect user to the profile page.
                $redirect_url = $this->session->userdata('last_visited_url');
            } else {
                $redirect_url = $role == 'admin' ? ($this->session->userdata($module . '_last_visited_page') ? $this->session->userdata($module . '_last_visited_page') : site_url(SITE_AREA . '/content/listings')) : gc_url_format('members/edit');
            }
        } else {
            $role_id = isset($rest_extra['role_id']) ? $rest_extra['role_id'] : 4;
            $this->load->model('mobiles/mobiles_model');
            $rest_result = array('success' => false, 'message' => lang('error_no_record_found'));
        }
        if(empty($id)) {
            if($mobile) {
                return $rest_result;
            } else {
                Template::redirect($redirect_url);
            }
        }
        if($group_id > 0) {
            $group = $this->db->where('id', $group_id)->get('groups')->row();
            if(!$group) {
                if($mobile) {
                    return $rest_result;
                } else {
                    Template::redirect($redirect_url);
                }
            }
        } else {
            $group = new stdClass();
            $group->id = 0;
            $group->name = 'main';
            $group->name_lang = 1;
            $group->mobile_icon = 'chevron-forward-circle-outline';
        }
        $user_id = empty($user_id) ? $this->auth->user_id() : $user_id;
        // Updated on July 04, 2019
        $record = $this->fields_model->find_row_and_meta($module, $id); // $this->db->where('id', $id)->get($module)->row();
        if(!$record) {
            if($mobile) {
                return $rest_result;
            } else {
                Template::redirect($redirect_url);
            }
        }
        $row = $record;
        if($module != 'branches' && $module != 'users') {
            // get user information
            $user = $this->user_model->get_user($record->user_id); //$this->db->where('id', $record->user_id)->get('users')->row();
            
            // check permissions
            if($role != 'admin' && $role_id != 1 && $record->user_id != $user_id) {
                if($mobile) {
                    return $rest_result;
                } else {
                    Template::redirect($redirect_url);
                }
            }
        }
        
        // get package information to apply limits
        $package = array();
        if(in_array($module, gc_modules()) && !empty($record->package_id)) {
            $this->load->model('packages/packages_model');
            $package = $this->packages_model->get_package($record->package_id);
        }
        // Load Fields
        $fields_form = $this->fields_model->get_fields($module, ($role == 'admin' ? 'admin_only' : 'frontend'), $package, 0, array('group_id' => empty($group_id) ? 9999 : $group_id));
        if($group_id == 7 && $module == 'listings') { // hide packages field
            // $extra_params = array('hide_packages' => true); // display user specific package and listings
            $this->load->model('members/members_model'); // display user purchased package only
            $package_id = $this->listings_model->last_ordered_package_id($user_id, $mobile);
            $package = $this->members_model->get_package_info($user_id, $package_id, 0, 'packages');
            if($package) { // display user specific packages only
                $package_title = $package->package_title . ($package->expires_on == NULL || $package->expires_on == '' ? ' (' . lang('label_expires_on') . ': ' . lang('label_never') . ')' : ' (' . lang('label_valid_upto') . ': ' . gc_date_format($package->expires_on, false) . ')');
                $packages = array(
                    array(
                        'id' => (int) $package_id,
                        'value' => (int) $package_id,
                        'name' => $package_title
                    )
                );
                $extra_params['packages'] = $packages;
            }
        }
        // hide fields based on permissions
        $fields_form = $this->fields_model->filter_fields_by_type($fields_form, 'edit_only');
        
        // group without fields
        $has_fields = false;
        if($fields_form) {
            foreach($fields_form as $field_info) {
                if($field_info['type'] != 'none') {
                    $has_fields = true;
                    break;
                }
            }
        }
        if(!$has_fields) {
            if($mobile) {
                $rest_result['message'] = lang('group_without_fields');
                return $rest_result;
            } else {
                Template::set_message(lang('group_without_fields'), 'danger');
                Template::redirect($redirect_url);
            }
        }
        
        $group_name = !empty($group->name_lang) ? lang('group_' . $group->name) : $group->name;
        if(!$mobile) {
            $form_fields = $this->fields_model->load_fields($module, array(), ($role == 'admin' ? 'admin_only' : 'frontend'), $record, $id, $mobile, $fields_form);
        } else {
            $form_fields = $fields_form;
            // format fields
            $fields_form = $this->mobiles_model->format_fields($fields_form, $extra_params);
            if(!empty($record)) {
                // format record before displaying it to users
                // this will convert ids to name
                $record = $this->mobiles_model->format_data($fields_form, $record);
            }
            if(!empty($_POST['save'])) {
                // format post values
                $_POST = $this->mobiles_model->format_post_data($_POST, $fields_form);
                $_POST['save'] = 1;
            }
        }
        
        // Save Method
        if (isset($_POST['save'])) {
            $extra_fields = array();
            switch($module) {                
                case 'jobs':
                case 'properties':
                case 'deals':
                case 'products':
                    if(!empty($record['listing_id'])) {
                        $_POST['listing_id'] = $record['listing_id'];
                        $extra_fields = $this->listings_model->module_address_info($module, $record['listing_id'], $extra_fields);
                    }
                    $extra_fields['package_id'] = $row->package_id;
                    break;
            }
            
            // convert categories in fancytree format
            if(!empty($_POST['categories'])) {
                $converted_categories = $this->convert_categories(explode(',', $_POST['categories'] ?? ''));
                $converted_categories = implode(',', $converted_categories);
                $_POST['categories'] = $converted_categories;
            }
            
            // Set default values if not set by the user e.g. price, offer_price cannot be null in case of products
            $this->listings_model->module_default_values($module);
            if ($result = $this->fields_model->save_module($form_fields, $module, $user_id, $role == 'mobile' ? 'frontend' : $role, $id, 'update', $extra_fields)) {
                if($module == 'listings') {
                    // update profile completion percent
                    $this->load->model('groups/groups_model');
                    $this->groups_model->row_completion($module, $id);
                }
                // clear cache to display the updated information
                clear_cache($module . '_' . $id);
                clear_cache('fields_' . $module . '_' . $id);
                $success_message = sprintf(lang('msg_save_success'), $group_name);
                // update user_profile session variable
                //$this->detailed_profile($id, array(), $mobile ? false : true);
                if($role == 'mobile') {
                    return array('success' => TRUE, 'message' => $success_message, 'extra' => array('id' => $group->id, 'icon' => $group->mobile_icon, 'name' => $group_name, 'toolbar_title' => lang('edit')));
                } else {
                    Template::set_message($success_message, 'success');
                    Template::redirect(current_url());
                }
            } else {
                if($mobile) {
                    $validation_errors = validation_errors();
                    return array('success' => FALSE, 'message' => $validation_errors, 'extra' => array('id' => $group->id, 'icon' => $group->mobile_icon, 'name' => $group_name, 'toolbar_title' => lang('edit_profile')));
                } else {
                    $error_message = sprintf(lang('msg_create_failure'), lang('singular_' . $module));
                    Template::set_message($error_message, 'error');
                }
            }
        }
        if($mobile) {
            return array('success' => true, 'record' => $record, 'fields' => $fields_form, 'extra' => array('id' => $group->id, 'icon' => $group->mobile_icon, 'name' => $group_name, 'toolbar_title' => lang('edit') . ' ' . lang('singular_' . $module)));
        } else {
            $toolbar_title = $row->user_id != $user_id ? $row->id . ' - ' . $row->title . ' - ' . $group_name : $group_name;
            Template::set(array(
                'layout' => 'none',
                'toolbar_title' => $toolbar_title,
                'module' => $module,
                'record' => $record
            ));
            Template::set_view('fields/form');
            if($role == 'members') {
                Template::set(array(
                    'member_area' => true,
                    'cancel_url' => gc_url_format('members/edit'),
                    // 'extra_actions' => array(anchor(gc_url_format('profiles', $record->slug, $record->id), lang('label_view'), 'class="btn btn-flip" target="_blank"')),
                    'elements_container' => 'div.card.card-body'
                ));
                
                // This method has no View in the Themes Module, i.e. load layout files manually
                $this->gc->demo_custom_css_js('member');
            } else if($role == 'admin') {
                Template::set(array(
                    'cancel_url' => $redirect_url
                ));
            }
            $render = ($role == 'members' ? 'member' : '');
            Template::render($render);
        }
    }    
    
    /**
     * Add New Module
     * @param string $module
     * @return array|boolean[]|string[]
     */
    public function add_module($group_id, $module, $user_id, $listing_id = 0) {
        $this->lang->load('listings/listings');
        $this->load->model('users/user_model');
        $this->load->model('fields/fields_model');
        $this->load->model('listings/listings_model');
        $this->load->model('members/members_model');
        $role = 'frontend';
        $mobile = true;
        $extra_fields = array();
        $result = array();
        $fields = array();
        // $extra_params = array('hide_packages' => true, 'module' => $module); // display user specific package and listings
        $package_id = $this->listings_model->last_ordered_package_id($user_id, $mobile);
        $package = $this->members_model->get_package_info($user_id, $package_id, 0, 'packages');
        if(!$package) {            
            $this->lang->load('payment_gateways/payment_gateways');
            return array('success' => false, 'message' => lang('order_pending_line_2') . ' ' . sprintf(lang('thankyou_line_3'), settings_item('site.system_email')));
        }
        
        // limit and pyg check
        $this->load->model('members/members_model');
        $limit_check = $this->members_model->pyg_check($module, $user_id, $package_id, $package, 0, 'listings', true);
        if(!$limit_check['success']) {
            $result = array('success' => false, 'message' => $limit_check['message']);
            if(!empty($limit_check['pyg_module'])) { // pyg allowed, send user to the checkout screen
                $result['pyg'] = array('package_id' => $package_id, 'module' => $module, 'qty' => 1);
            }
            return $result;
        }
        
        // display user specific packages only
        $package_title = $package->package_title . ($package->expires_on == NULL || $package->expires_on == '' ? ' (' . lang('label_expires_on') . ': ' . lang('label_never') . ')' : ' (' . lang('label_valid_upto') . ': ' . gc_date_format($package->expires_on, false) . ')');
        $packages = array(
            array(
                'id' => $package_id,
                'value' => $package_id,
                'name' => $package_title
            )
        );
        $extra_params['packages'] = $packages;
        // $extra_params['hide_packages'] = true; // uncomment to hide packages
        $field_results = $this->fields_model->registration_fields_by_groups('frontend', $module, $package);
        $first_group_id = array_key_first($field_results); // first group id
        if(!isset($field_results[$group_id]['fields'])) {
            // the group_id is invalid
            // take user to the first group form
            $group_id = $first_group_id;
        }
        
        $last_group_id = key(array_slice($field_results, -1, 1, true)); // last group
        $last_group_id = $this->_valid_last_group_id($last_group_id, $field_results); // valid last group id and has fields
        
        if(empty($field_results[$group_id]['fields'])) {
            return array('success' => false);
        }
        $form_fields = $field_results[$group_id]['fields'];
        
        // Current/Previous/Next Group IDs
        $keys = array_keys($field_results);
        $next_group_id = $group_id != $last_group_id ? $keys[array_search($group_id, $keys) + 1] : $group_id;
        $previous_group_id = $group_id != 0 ? (!empty($keys[array_search($group_id, $keys) - 1]) ? $keys[array_search($group_id, $keys) - 1] : 0) : 0;
        $toolbar_title = lang('add') . ' ' . lang('singular_' . $module);
        
        // Format Fields
        $return_all_fields = false;
        if($return_all_fields) {
            $field_results[0]['id'] = 0;
            $field_results[0]['name'] = lang('group_main');
            $field_results[0]['mobile_icon'] = 'chevron-forward-circle-outline';
            $field_results[0]['step'] = 0;
            $field_results[0]['last'] = 0;
            foreach($field_results as $key => $field_result) {
                if(!empty($field_result['fields'])) {
                    $next_group_id = $field_result['id'] != $last_group_id ? $keys[array_search($field_result['id'], $keys) + 1] : $field_result['id'];
                    $previous_group_id = $field_result['id'] != 0 ? (!empty($keys[array_search($field_result['id'], $keys) - 1]) ? $keys[array_search($field_result['id'], $keys) - 1] : 0) : 0;
                    $btn_text = $field_result['id'] != $last_group_id ? lang('gc_action_next') : $toolbar_title;
                    $field_results[$key]['extra'] = array('btn_text' => $btn_text, 'text_login' => $toolbar_title);
                    $field_results[$key]['extra']['id'] = $field_result['id'];
                    $field_results[$key]['extra']['name'] = $field_result['name'];
                    $field_results[$key]['extra']['icon'] = $field_result['mobile_icon'];
                    $field_results[$key]['extra']['toolbar_title'] = $toolbar_title . ' - ' . $field_result['name'];
                    $field_results[$key]['extra']['next_group_id'] = $next_group_id;
                    $field_results[$key]['extra']['previous_group_id'] = $previous_group_id;
                    $field_results[$key]['fields'] = $this->format_fields($field_result['fields']);
                }
                unset($field_results[$key]['id']);
                unset($field_results[$key]['name']);
                unset($field_results[$key]['mobile_icon']);
                unset($field_results[$key]['step']);
                unset($field_results[$key]['last']);
            }
        } else {
            $field_results = $this->_process_step_fields($group_id, $keys, $field_results, $last_group_id, $toolbar_title, $extra_params);
        }
        $fields = $field_results[$group_id]; // get group name also to be displayed in the title bar
        
        // Add Business - Check the existing order details
        if ($module == 'listings' && ($group_id == $first_group_id || $group_id == $last_group_id)) {
            $this->load->model('orders/orders_model');
            if (empty($package)) { // count result if package exists
                return array(
                    'status' => false,
                    'order_type' => 'packages',
                    'url' => 'app.checkout'
                );
            }
            // Check user permissions
            // Limit Check - Member Area Only
            // Do PYG Check only in case of Insert
            // $_POST['package_id'] = $package_id;
            if($group_id == $first_group_id) {
                $pyg_check = $this->members_model->pyg_check($module, $user_id, $package_id, $package, 0, $module, $mobile);
                if ($mobile && ! $pyg_check['success']) {
                    return $pyg_check;
                }
            }
            if($group_id == $last_group_id) {
                // Get status field information
                // In case of Admin, it will be selected by the Admin
                $extra_fields['user_id'] = $user_id;
                $extra_fields['package_id'] = $package_id;
                $extra_fields['featured'] = gc_field_value($module, $package->module_featured);
                $extra_fields['status'] = gc_field_value($module, $package->module_status);
                if(!empty($package->order_id)) {
                    $extra_fields['order_id'] = $package->order_id;
                }
                
                // calculate expires on
                if(empty($package->expires_on)) {
                    $this->load->model('orders/orders_model');
                    $extra_fields['expires_on'] = $this->orders_model->calculate_expires_on($package, 1, null);
                } else {
                    $extra_fields['expires_on'] = $package->expires_on;
                }
            }
        } else if($module != 'listings' && !empty($listing_id) && $group_id == $last_group_id) {
            $extra_fields['listing_id'] = $listing_id;
            $extra_fields['user_id'] = $user_id;
            $extra_fields['package_id'] = $package_id;
            /*if(!empty($package->order_id)) {
                $extra_fields['order_id'] = $package->order_id;
            }*/
            $extra_fields['featured'] = gc_field_value($module, $package->module_featured);
            $extra_fields['status'] = gc_field_value($module, $package->module_status);
        }
        if(isset($_POST['save'])) {
            // convert dob from ISO to date format
            if(!empty($_POST['dob'])) {
                $_POST['dob'] = date( "Y-m-d", strtotime($_POST['dob']));
            }
            // format selectable values
            foreach($_POST as $key => $value) {
                if(is_array($value) && isset($value['id'])) {
                    $_POST[$key] = $value['id'];
                }
            }
            if($group_id == $last_group_id) {
                // in the last step validate all fields
                $form_fields = $this->fields_model->get_fields($module, $role, $package); 
            }
            $_POST = $this->mobiles_model->format_post_data($_POST, $this->format_fields($form_fields));
            if(!empty($_POST['city'])) {
                $_POST['city_id'] = $_POST['city'];
            }       
            
            // convert categories in fancytree format
            if(!empty($_POST['categories'])) {
                $converted_categories = $this->convert_categories(explode(',', $_POST['categories'] ?? ''));
                $converted_categories = implode(',', $converted_categories);
                $_POST['categories'] = $converted_categories;
            }
            $validation_result = $this->fields_model->fields_validation_rules($form_fields, $module, $role);
            if($validation_result) {
                if($group_id != $last_group_id) {
                    // to avoid extra api request
                    $next_group_fields = $this->_process_step_fields($next_group_id, $keys, $field_results, $last_group_id, $toolbar_title);
                    $record = $this->default_values($module, array(), $listing_id, $user_id);
                    $next_group_info = $next_group_fields[$next_group_id];
                    $next_group_info['record'] = $record;
                    $result = array('success' => false, 'next_group_fields' => array($next_group_id => $next_group_info), 'next_group_id' => $next_group_id, 'previous_group_id' => $previous_group_id, 'last_group_id' => $last_group_id);
                } else {
                    $this->listings_model->module_default_values($module);
                    // $extra_fields = $this->listings_model->module_address_info($module, $listing_id, $extra_fields);
                    // $data = $this->fields_model->field_post_values($form_fields, 0, 0, $role);
                    // register user and save profile information
                    $this->load->library('template');
                    $this->load->library('layouts');
                    $id = 0;
                    $transaction_type = 'insert';
                    /*if(gc_module_settings($module, 'locations') == 1 && empty($_POST['latitude'])) {	
                        dump($_POST);
                    }*/
                    if ($id = $this->fields_model->save_module($form_fields, $module, $user_id, $role, $id, $transaction_type, $extra_fields)) {
                        $result = $this->listings_model->save_module_info('mobile', $id, $user_id, $transaction_type, $module, null);
                        $result['next_group_id'] = $next_group_id;
                        $result['previous_group_id'] = $previous_group_id;
                        $result['last_group_id'] = $last_group_id;
                        if($module == 'listings') {
                            clear_cache('links_' . $user_id);
                        }
                    } else {
                        $result = array('success' => false, 'message' => lang('msg_error'));
                    }
                }
            } else {
                $validation_errors = validation_errors();
                $result = array('success' => false, 'message' => $validation_errors);
            }
        } else {
            $result = $fields;
            $result['success'] = true;
        }
        // set default package
        $user_info = $this->db->select('display_name AS contact_person, email, phone_number, address, postcode, city_id, city_id AS city')->where('id', $user_id)->get('users')->row();
        $user_info->package_id = !empty($packages[0]) ? $packages[0]['id'] : 0;
        
        // get data in one go for all fields
        $all_fields = array();
        foreach($field_results as $field_result) {
            $all_fields = array_merge($field_result['fields'], $all_fields);
        }
        $record = $this->format_data($all_fields, $user_info);
        $record = $this->default_values($module, $record, $listing_id, $user_id);
        $result['record'] = $record;
        return $result;
    }
    
    /**
     * Get Default Values
     * @param string $module
     * @param array $record
     * @param number $listing_id
     * @return array
     */
    private function default_values($module, $record, $listing_id, $user_id = 0) {
        switch($module) { // default values
            case 'jobs':
                $record['type'] = 1;
                $record['min_experience'] = 0;
                $record['max_experience'] = 0;
                $record['min_qualification'] = 0;
                $record['salary'] = 15000;
                break;
            case 'properties':
                $record['purpose'] = 2;
                $record['type'] = 2;
                $record['ownership'] = 2;
                $record['bedrooms'] = 2;
                $record['bathrooms'] = 2;
                $record['floors'] = 1;
                $record['rooms'] = 2;
                $record['squarefeet'] = 1000;
                $record['price'] = 10000;
                $record['rent_price'] = 10000;
                break;
            case 'products':
                $record['price'] = 0;
                $record['offer_price'] = 0;
        }
        if($module == 'listings') { // get default address
            $this->load->model('users/user_model');
            $user = $this->user_model->get_user($user_id);
        } else {
            $listing_info = $this->listings_model->get_listing_info($listing_id);
            if($listing_info) {
                $record['city'] = array('id' => $listing_info->city_id, 'name' => $listing_info->city . ', ' . $listing_info->state . ', ' . ucwords(strtolower($listing_info->country)));
                $record['company'] = $listing_info->title;
                $record['latitude'] = $listing_info->latitude;
                $record['longitude'] = $listing_info->longitude;
                $record['address'] = $listing_info->address;
                $record['postcode'] = $listing_info->postcode;
            }
        }
        return $record;
    }
    
    /**
     * Return valid last group_id
     * @param array $field_results
     * @return mixed
     */
    private function _valid_last_group_id($last_group_id, $field_results) {
        $keys = array_keys($field_results);
        if(empty($field_results[$last_group_id]['fields'])) { // last group id has fields
            // get previous group id
            $now_index = array_search($last_group_id, $keys);
            if(isset($keys[$now_index - 1])) {
                $last_group_id = $keys[$now_index -1 ];
            } else { // do not loop further
                return 0;
            }
            // check whether the new last group id has fields or not
            $this->_valid_last_group_id($last_group_id, $field_results);
        }
        $result = $this->format_fields($field_results[$last_group_id]['fields']);
        if(empty($result)) {
            $now_index = array_search($last_group_id, $keys);
            if(isset($keys[$now_index - 1])) {
                $last_group_id = $keys[$now_index -1 ];
                // the group must have fields
                $this->_valid_last_group_id($last_group_id, $field_results);
            } else { // do not loop further
                return 0;
            }
        }
        return $last_group_id;
    }
    
    /**
     * Add Module - Process Step Fields
     * @param number $group_id
     * @param array $field_results
     * @param array $field_result
     * @param integer $last_group_id
     * @return array
     */
    private function _process_step_fields($group_id, $keys, $field_results, $last_group_id, $toolbar_title, $extra_params = array()) {
        $key = $group_id;
        if($group_id == 0) {
            $field_results[0]['id'] = 0;
            $field_results[0]['name'] = lang('group_main');
            $field_results[0]['mobile_icon'] = 'chevron-forward-circle-outline';
            $field_results[0]['step'] = 0;
            $field_results[0]['last'] = 0;
        }
        $field_result = $field_results[$key];
        if(!empty($field_result['fields'])) {
            $next_group_id = $field_result['id'] != $last_group_id ? $keys[array_search($field_result['id'], $keys) + 1] : $field_result['id'];
            $previous_group_id = $field_result['id'] != 0 ? (!empty($keys[array_search($field_result['id'], $keys) - 1]) ? $keys[array_search($field_result['id'], $keys) - 1] : 0) : 0;
            $btn_text = $field_result['id'] != $last_group_id ? lang('gc_action_next') : $toolbar_title;
            $field_results[$key]['extra'] = array('btn_text' => $btn_text, 'text_login' => $toolbar_title);
            $field_results[$key]['extra']['id'] = $field_result['id'];
            $field_results[$key]['extra']['name'] = $field_result['name'];
            $field_results[$key]['extra']['icon'] = $field_result['mobile_icon'];
            $field_results[$key]['extra']['toolbar_title'] = $toolbar_title . ' - ' . $field_result['name'];
            $field_results[$key]['extra']['next_group_id'] = $next_group_id;
            $field_results[$key]['extra']['previous_group_id'] = $previous_group_id;
            $field_results[$key]['fields'] = $this->format_fields($field_result['fields'], $extra_params);
        }
        unset($field_results[$key]['id']);
        unset($field_results[$key]['name']);
        unset($field_results[$key]['mobile_icon']);
        unset($field_results[$key]['step']);
        unset($field_results[$key]['last']);
        return $field_results;
    }
    
    /**
     * Allow member to view module specific rows
     * @param number $user_id - Display user specific records only
     * @param string $method - Name of the method
     * @param string $module - Name of the Module
     * @param number $row_id - ID of the row, e.g. pass row_id 22 to view 22 listing specific business queries
     * @param number $offset for pagination
     * @param number $limit for pagination
     */
    public function member_view_module($user_id, $method, $module, $row_id, $offset, $limit) {
        switch($method) {
            case 'view_comments':
                $this->load->model('comments/comments_model');
                $title = lang('members_comments');
                $table = 'comments';
                $label = lang('label_comment');
                break;
            case 'my_comments':
                $this->load->model('comments/comments_model');
                $title = lang('my_comments');
                $table = 'comments';
                $label = lang('label_comment');
                break;
            case 'view_business_queries':
                $this->load->model('business_queries/business_queries_model');
                $title = lang('members_business_queries');
                $table = 'business_queries';
                $label = lang('label_business_query');
                break;
            case 'posted_queries':
                $this->load->model('business_queries/business_queries_model');
                $title = lang('my_queries');
                $table = 'business_queries';
                $label = lang('my_queries');
                break;
            case 'favorites':
                $this->load->model('favorites/favorites_model');
                $title = lang('my_favorites');
                $table = 'favorites';
                $label = lang('label_favorite');
                break;
        }        
        // Return Module Counts
        $records = array();
        $this->load->model('members/members_model');
        $counts = $this->members_model->fcb_counts($user_id, $method);
        $fcb = $this->members_model->fcb_records($user_id, $method, $table, $module, $row_id, $offset, $limit, true);
        $rows = $fcb['records'];
        if($rows) {
            switch($method) {
                case 'view_comments':
                    foreach($rows as $row) {
                        $records[] = array(
                            'id' => $row->id,
                            'title' => $row->module_title,
                            'comment_title' => $row->title,
                            'name' => $row->user,
                            'profile_image' => !empty($row->profile_image) ? base_url('assets/images/' . $row->profile_image) : '',
                            'message' => $row->comment,
                            'created_on' => gc_date_format($row->created_on),
                            'status' => $row->status == 1 ? lang('us_active') : lang('us_inactive'),
                            'status_color' => $row->status == 0 ? '#FF0000' : '#006400',
                            'rating' => $row->rating
                        );
                    }
                    break;
                case 'my_comments':
                    foreach($rows as $row) {
                        $records[] = array(
                            'id' => $row->id, // for delete
                            'title' => $row->module_title,
                            'comment_title' => $row->title,
                            'message' => $row->comment,
                            'created_on' => gc_date_format($row->created_on),
                            'status' => $row->status == 1 ? lang('us_active') : lang('us_inactive'),
                            'status_color' => $row->status == 0 ? '#FF0000' : '#006400',
                            'rating' => $row->rating,
                            'redirect_url' => 'tabs/member/modules/detail/' . gc_module_name($row->module_id) . '/' . $row->row_id
                        );
                    }
                    break;
                case 'view_business_queries':
                case 'posted_queries':
                    foreach($rows as $row) {                        
                        $links = array(
                            array(
                                'label' => lang('label_replies'),
                                'url' => '/member/info-detail/query_replies/' . $row->id,
                                'color' => 'primary',
                                'icon' => 'reader-outline'
                            ),
                            array(
                                'label' => lang('label_reply'),
                                'url' => '/crud/query_reply/' . $row->id . '/business_queries/0',
                                'color' => 'secondary',
                                'icon' => 'arrow-redo-outline'
                            ),
                            
                        );
                        if($method == 'posted_queries') {
                            $links[] = array(
                                'label' => lang('label_view'),
                                'url' => '/tabs/member/modules/detail/' . gc_module_name($row->module_id) . '/' . $row->row_id,
                                'color' => 'tertiary',
                                'icon' => 'book-outline'
                            );
                        }
                        $data = array(
                            'title' => $row->title,
                            'message' => $row->message,
                            'created_on' => gc_date_format($row->created_on),
                            'links' => $links,
                            'replied' => $row->replied == 1 ? lang('label_replied') : lang('label_not_replied'),
                            'replied_color' => $row->replied == 1 ? 'success' : 'warning'
                        );
                        if($method == 'view_business_queries') {
                            $data['name'] = $row->name;
                            $data['profile_image'] = !empty($row->profile_image) ? base_url('assets/images/' . $row->profile_image) : '';
                            $data['email'] = $row->email;
                            $data['phone_number'] = $row->phone_number;
                        }                        
                        $records[] = $data;
                    }
                    break;
                case 'favorites':
                    foreach($rows as $row) {
                        $records[] = array(
                            'id' => $row->id,
                            'row_id' => $row->row_id,
                            'title' => $row->title,
                            'description' => truncate_text($row->description, 100),
                            'contact_person' => isset($row->contact_person) ? $row->contact_person : '',
                            'phone_number' => isset($row->phone_number) ? $row->phone_number : '',
                            'address' => isset($row->address) ? $row->address : '',
                            'location' => !empty($row->city) ? $row->city . ', ' . $row->state . (!empty($row->postcode) ? ' - ' . $row->postcode : '') . ', ' . ucwords(strtolower($row->country)) : '',
                            'created_on' => gc_date_format($row->created_on),
                            'url' => !empty($row->row_id) ? '/tabs/member/modules/detail/' . gc_module_name($row->module_id) . '/' . $row->row_id : '',
                            'image' => gc_image_url($row->cover),
                            'average_rating' => $row->average_rating
                        );
                    }
                    break;
            }
        }
        $records = !empty($rows) ? $records : $records;
        return array('records' => $records, 'total_rows' => $fcb['total_records'], 'counts' => $counts, 'toolbar_title' => $title);
    }
    
    /**
     * Display Modules - Member Area
     * @param string $module
     * @param int $user_id - Integer of User ID to fetch
     * @param int $count
     * @param string $where           
     * @param int $limit
     * @param int $offset
     */
    public function member_edit($module, $user_id, $count = 0, $where = null, $limit = 15, $offset = 0, $sort_by = 'M.id DESC') {
        if ($count == 0) {
            $sql = "SELECT M.*";
        } else {
            $sql = "SELECT COUNT(M.id) AS total_count";
        }
        $sql .= " FROM " . $this->db->dbprefix . $module . " M";
        $sql .= " WHERE M.deleted = 0 AND M.user_id = " . $user_id;
        if(!empty($where)) {
            $sql .= " AND " . $where;
        }
        if ($count == 0) {
            $sql .= " GROUP BY M.id";
            $sql .= " ORDER BY " . $sort_by;
            if ($limit != 0) {
                $sql .= " LIMIT " . $offset . ", " . $limit;
            } else {
                $sql .= " LIMIT $limit";
            }
            $query = $this->db->query($sql);
            $records = $query->result();
            $records = $this->module_view_fields($module, $records, true);
            return $records;
        } else {
            $result = $this->db->query($sql)->row();
            return $result ? $result->total_count : 0;
        }
    }
    
    /**
     * Fields to be displayed on the modules view/category/search pages
     * @param object $records
     * @param string $module
     * @return array
     */
    public function module_view_fields($module, $records, $is_edit = false) {
        $this->lang->load('listings/listings');
        $rows = array();
        if(!empty($records)) {
            if($is_edit) {
                foreach($records as $record) {
                    if($module == 'branches') {
                        $row = array(
                            'id' => $record->id,
                            'title' => $record->contact_person
                        );
                    } else {
                        $row = array(
                            'id' => $record->id,
                            'title' => $record->title,
                            'image' => empty($record->cover) ? gc_image_url($record->cover, true, true) : gc_image_url($record->cover)
                        );
                    }
                    if(gc_module_settings($module, 'locations') == 1) {	
                        $row['address'] = gc_address_format($record->address, (isset($record->locality) ? $record->locality : ''), $record->city, $record->state, $record->postcode, $record->country, true);
                    } else {
                        $row['description'] = truncate_text($record->description, 100, true);
                    }
                    if($module == 'branches') {
                        $row['url'] = '/tabs/member/edit-module/branches/' . $record->id;
                    } else {
                        $row['url'] = '/tabs/member/dashboard/' . $module . '/' . $record->id;
                    }
                    $rows[] = (object) $row;
                }
            } else {
                $this->load->model('fields/fields_model');
                $address_template = 1;
                $records = $this->fields_model->process_records($module, 'frontend', $records, array(), true);
                foreach($records as $record) {
                    $package_display_fields = $module == 'listings' && !empty($record->display_fields) ? json_decode($record->display_fields)->listings : array();
                    $row = array( // common fields
                        'id' => $record->id,
                        'title' => $record->title,
                        'image' => $record->cover
                    );
                    $display_fields = array();
                    // get address
                    if(gc_module_settings($module, 'locations') == 1) {
                        if(isset($record->address)) {
                            if(!empty($package_display_fields->address) && !empty($record->show_address)) { // Listings Specific
                                $row['address'] = $record->address;
                            } else if($module != 'listings') {
                                $row['address'] = $record->address;
                            }
                        }
                        if(!empty($row['address'])) {
                            $row['address'] = gc_address_format($record->address, (isset($record->locality) ? $record->locality : ''), $record->city, $record->state, $record->postcode, $record->country, true);
                        }
                        if(!empty($record->latitude)) {
                            $row['latitude'] = $record->latitude;
                        }
                        if(!empty($record->longitude)) {
                            $row['longitude'] = $record->longitude;
                        }
                    } else {
                        $row['description'] = truncate_text($record->description, 100);
                    }
                    if($module == 'listings') { // Open/Closed Now                        
                        $row['open_status'] = (int) 2; // Business Hours not specified
                        if(!empty($record->open_time) && !empty($record->close_time)) {
                            $current_time = date('H:i');
                            if(empty($row['open_status']) && $record->open_time < $current_time && $record->close_time > $current_time) {
                                $row['open_status'] = 1;
                            } else if(empty($row['open_status'])) {
                                $row['open_status'] = 2;
                            }
                        }
                    }
                    $row['url'] = '/tabs/modules/detail/' . $module . '/' . $record->id;
                    // display email to the user
                    if(!empty($record->email)) {
                        if($module == 'listings') {
                            if(isset($record->show_email) && $record->show_email == 1 && !empty($package_display_fields->email)) {
                                $row['email'] = $record->email;
                            }
                        } else {
                            $row['email'] = $record->email;
                        }
                    }
                    // trusted seal
                    if(!empty($record->trusted_seal)) {
                        $row['trusted_seal'] = site_url('assets/images/' . $record->trusted_seal);
                    }
                    if(!empty($record->phone_number) || !empty($record->mobile_number)) {
                        if(!empty($package_display_fields->phone_number) && isset($record->show_phone) && $record->show_phone == 1) {
                            $row['phone_number'] = $record->phone_number;
                            $row['mobile_number'] = $record->mobile_number;
                        }
                    }
                    $row['hits'] = $record->hits;
                    if(isset($record->isFavorite)) {
                        $row['isFavorite'] = $record->isFavorite > 0 ? true : false;
                    }
                    $row['average_rating'] = $record->average_rating;
                    if(!empty($record->start_date)) {
                        $row['start_date'] = gc_date_format($record->start_date, true);
                        $split_date = split_datetime($record->start_date);
                        $row['start_day'] = $split_date['day'];
                        $row['start_month'] = $split_date['month'];
                    }
                    // Module Specific - Additional Fields
                    if(isset($record->salary)) {
                        $row['salary'] = $record->salary;
                    }
                    if(isset($record->code)) { // Coupon Expires On
                        $row['code'] = $record->code;
                    }
                    if(isset($record->expires_on) && $record->expires_on > date('Y-m-d')) {
                        $row['expires_on'] = gc_date_format($record->expires_on, false);
                    }
                    if(isset($record->squarefeet)) {
                        $row['squarefeet'] = $record->squarefeet;
                    }
                    if(isset($record->bedrooms)) {
                        $row['bedrooms'] = $record->bedrooms;
                    }
                    if(isset($record->bathrooms)) {
                        $row['bathrooms'] = $record->bathrooms;
                    }
                    if(isset($record->floors)) {
                        $row['floors'] = $record->floors;
                    }
                    if(isset($record->rooms)) {
                        $row['rooms'] = $record->rooms;
                    }
                    if(isset($record->rent_price)) {
                        $row['rent_price'] = $record->rent_price;
                    }
                    if(!empty($record->price)) {
                        $price = (float) $record->price;
                        $offer_price = !empty($record->offer_price) ? (float) $record->offer_price : 0;
                        if(!empty($offer_price) && $record->price > $offer_price) {
                            $savings = calculate_savings($price, $offer_price, 'product-savings');
                            $row['savings_in_percentage'] = $savings['savings_in_percentage'] . '%';
                            $row['price'] = $offer_price;
                            $row['price_with_currency'] = currency_format($offer_price);
                            $row['original_price_with_currency'] = currency_format($price);
                            $row['savings'] = $savings['savings'];
                            $row['html'] = $savings['html'];
                            $row['original_price'] = $price;
                            $row['discount'] = $savings['savings_in_percentage'] . '%'  . ' ' . lang('label_off');
                        } else {
                            $row['price'] = $price;
                            $row['price_with_currency'] = currency_format($price);
                            $row['html'] = '';
                            $row['original_price'] = $price;
                        }
                    }
                    if(isset($record->type)) {
                        $row['type'] = $record->type;
                    }
                    if(isset($record->venue)) { // Coupon Expires On
                        $row['venue'] = $record->venue;
                    }
                    
                    // display ratings to the user
                    if(!empty($record->ratings)) {
                        $row['ratings'] = number_format($record->ratings, 1, '.', '');
                    }
                    if(!empty($record->rating_users)) {
                        $row['rating_users'] = $record->rating_users;
                    }
                    
                    // distance between the user and the business
                    if(!empty($record->user_distance)) {
                        $row['user_distance'] = $record->user_distance . ' ' . (settings_item('site.distance_unit') == 'km' ? lang('label_km') : lang('label_miles'));
                    }
                    
                    // ECommerce
                    if(!empty($record->offer_price) && $record->price > $record->offer_price && is_float($record->offer_price)) {    
                        $row['savings_in_percentage'] = decimal_convert((($record->price - $record->offer_price) * 100) / $record->price) . '%';
                        $row['price'] = $record->offer_price;
                        $row['price_with_currency'] = currency_format($record->offer_price);
                        $row['offer_price'] = currency_format($record->offer_price);
                    }
                    if($module != 'products') {
                        // Author Information
                        if(!empty($record->listing_slug) && $module != 'listings') { // Display listing detail as the Author Information
                            $row['author_name'] = $record->listing_title;
                            $row['author_url'] = gc_url_format('listings', $record->listing_slug, $record->listing_id, $record->listing_city);
                        } else if(!empty($record->author_slug)) { // Display User Detail as the Author Information
                            $row['author_name'] = $record->author_name;
                            $row['author_url'] = gc_url_format('users', $record->author_slug, $record->author_id, $record->author_city);
                        }
                        if(isset($record->author_image) && !empty($record->listing_logo)) {
                            $row['author_image'] = gc_image_url($record->listing_logo);
                        } else if(isset($record->author_image)) {
                            $row['author_image'] = gc_image_url($record->author_image, true, true, array('module' => 'users'));
                        }
                    } else {
                        // cart specific changes
                        $product_variants = array();
                        $variants = json_decode($record->variants ?? '', true);
                        if(!empty($variants)) {
                            foreach($variants as $key => $variant) {
                                $product_variants[] = array(
                                    'id' => $record->id,
                                    'item_id' => $record->id . '-' . $variant['id'],
                                    'listing_id' => $record->listing_id,
                                    'title' => $variant['title'] . ' (' . $variant['price_with_currency'] . ')',
                                    'price' => !empty($variant['offer_price']) ? $variant['offer_price'] : $variant['price'],
                                    'qty' => 0,
                                    'original_price' => $variant['price']
                                );
                            }
                        }
                        $row['qty'] = 0; // required in case of cart
                        $row['variants'] = $product_variants;
                        $row['item_id'] = $record->id . '-0'; // 0 is the variant id
                    }
                    if($module != 'listings' && !empty($record->listing_id)) {
                        $row['listing_id'] = $record->listing_id;
                    }
                    $rows[] = (object) $row;
                }
            }
        }
        return $rows;
    }    
    
    /**
     * Convert categories to fancytree format - ADD/EDIT Listing
     * e.g. 22 to /1/21/22
     */
    private function convert_categories($selected_categories) {
        $categories = array();
        foreach($selected_categories as $category) {
            $result = $this->db->select('id, parent_id')->where('id', $category)->get('categories')->row();
            if($result) {
                if($result->parent_id == 0) {
                    $categories[] = '/' . $category; // Parent Category
                } else {
                    $sub_category = $this->db->select('id, parent_id')->where('id', $result->parent_id)->get('categories')->row();
                    if($sub_category) {
                        if($sub_category->parent_id == 0) {
                            $categories[] = '/' . $result->parent_id .'/' . $result->id;
                        } else {
                            $categories[] = '/' . $sub_category->parent_id .'/' . $result->parent_id . '/' . $result->id;
                        }
                    }
                }
            }
        }
        $_POST['default_category'] = $selected_categories[0];
        return $categories;
    }
    
    /**
     * Mobile App - Process Cover Image
     * @param string $cover_image
     * @param boolean $no_image - True: Display No Image
     * @return array
     */
    public function process_images($cover_image, $no_image = true, $active_only = true) {
        $cover = array();
        $cover_image = json_decode($cover_image ?? '', true);
        if(!$active_only) {
            if(!empty($cover_image['url']) && !empty($cover_image['status'])) {
                $cover = array(
                    'original' => gc_image_url($cover_image['url'], false),
                    'thumb' => gc_image_url($cover_image['url']),
                    'title' => $cover_image['title'],
                    'id' => isset($cover_image['id']) ? $cover_image['id'] : 0, // Mobile App - Delete Image
                    'status' => isset($cover_image['status']) ? $cover_image['status'] : 0
                );
            } else if($no_image) {
                $no_image_url = gc_site_image('no-image');
                $cover = array(
                    'original' => $no_image_url,
                    'thumb' => $no_image_url,
                    'title' => '',
                    'id' => 0,
                    'status' => 1
                );
            }
        } else if($no_image) {
            $no_image_url = gc_site_image('no-image');
            $cover = array(
                'original' => $no_image_url,
                'thumb' => $no_image_url,
                'title' => '',
                'id' => 0,
                'status' => 1
            );
        }
        return $cover;
    }
    /***************************************************************************
     * END OF MEMBER AREA
     ***************************************************************************/
    
    /***************************************************************************
     * START OF FRONTEND
     ***************************************************************************/
    
    /**
     * Module Details
     * @param object $module - Name of the Module
     * @param number $id - ID of the row
     * @param number $loggedin_user_id - Loggedin User ID
     * @param boolean $is_edit - TRUE: the request is from the edit page
     * @param boolean $mobile
     * @param array $extra - Mobile App (user_profile, user_preferences)
     */
    public function module_detail($module, $id, $loggedin_user_id = 0, $is_edit = false, $extra = array()) {
        $this->lang->load('groups/groups');
        $this->lang->load('listings/listings');
        $mobile = false;
        if(empty($extra['mobile'])) {
            $mobile = true;
        }        
        $extra_data = array(); // extra data to be displayed to the user e.g. categories
        $hidden_fields = array();
        $package = array();
        $module_id = gc_module_id($module);
        $result = array();
        $cacheID = $module . '_' . $id . '_' . $is_edit;
        if(!$record = $this->mb_pages_cache->load($cacheID)) {
            $record = $this->get_module_data($module, $id, $is_edit);
            if(!$record) {
                $this->mb_pages_cache->save($record, $cacheID, array($module . '_' . $id));
                return false;
            }
            if($record && !$is_edit && $module != 'branches' && $module != 'users') {
                // get translations if exists
                $record = $this->translations->get_translations($module, $id, $record, 1);
                
                // Display Categories and Breadcrumb information
                $this->load->model('categories/categories_model');
                $breadcrumb = $this->categories_model->get_breadcrumb($id, $module);
                $record->categories_breadcrumb = null;
                if(!empty($breadcrumb)) {
                    $categories = $this->categories_model->get_categories($id, $module, !empty($breadcrumb[0]->id) ? $breadcrumb[0]->id : $breadcrumb->id);
                    $record->categories_breadcrumb = $categories;
                }
                $record->breadcrumb = $breadcrumb;                
                
                // Additional documents added by user
                $this->load->library('goclixy/gfile');
                $record->extra_files = $this->gfile->saved_files($record->files);
                
                // Display tags to the user
                $this->load->model('tags/tags_model');
                $record->extra_tags = $this->tags_model->get_tags($module, $id, TRUE);
                
                // Display Ratings to the user
                $this->load->model('ratings/ratings_model');
                $record->extra_ratings = $this->ratings_model->get_rating($module, $id);
                
                if($module == 'listings') {
                    // Get Branch Office Addresses
                    $this->load->model('branches/branches_model');
                    $record->branch_offices = $this->branches_model->get_branches(0, $record->id, 0, true);
                }
                
                // Display Images to the User
                $this->load->model('images/images_model');
                $images = $this->images_model->order_by('priority')->find_all_by(array('module_id' => $module_id, 'row_id' => $id, 'status' => 1, 'deleted' => 0));
                
                // Merge images and videos for gallery with a new field of type: image | video
                $gallery_images = array();
                if($images) {
                    foreach($images as $image) {
                        if($image->type_id == 2) { // logo
                            $record->extra_logo = array(
                                'thumb' => gc_image_url($image->url),
                                'original' => gc_image_url($image->url, false)
                            );
                        } else if($module != 'listings' || $image->type_id == 0) {
                            $gallery_images[] = array(
                                'id' => $image->id,
                                'title' => $image->title,
                                'original' => gc_image_url($image->url, false),
                                'thumb' => gc_image_url($image->url),
                                'type' => 'image'
                            );
                        } else if($module == 'listings' && $image->type_id == 1) { // cover image
                            $record->extra_cover = array(
                                'thumb' => gc_image_url($image->url),
                                'original' => gc_image_url($image->url, false)
                            );
                        }
                    }
                }
                $record->extra_images = $gallery_images;
                $record->extra_videos = null;
                if(gc_module_settings('videos') == 1) {
                    $this->load->model('videos/videos_model');
                    $videos = $this->videos_model->select('id,title,youtube_id,url')->find_all_by(array('row_id' => $id, 'module_id' => $module_id, 'status' => 1));
                    if($videos) {
                        $gallery_videos = array();
                        foreach($videos as $video) {
                            $gallery_videos[] = array (
                                'id' => $video->id,
                                'title' => $video->title,
                                'url' => $video->youtube_id,
                                'type' => 'video'
                            );
                        }
                        $record->extra_videos = $gallery_videos;
                    }
                }                
                
                // modules count - redirect user to the listing specific modules
                if($module == 'listings') {
                    $gc_modules = gc_modules();
                    $sql = "SELECT";
                    foreach($gc_modules as $gc_module) {
                        if($gc_module != 'listings') {
                            $sql .= "(SELECT COUNT(*) FROM " . $this->db->dbprefix . $gc_module . " WHERE listing_id = " . $record->id . " AND status = 1 AND deleted = 0) AS total_" . $gc_module . ", ";
                            
                        }
                    }
                    $sql .= " COUNT(*) AS total_modules";
                    $sql .= " FROM " . $this->db->dbprefix . "modules";
                    $modules_count_result = (array) $this->db->query($sql)->row();
                    $modules_count = array();
                    foreach($gc_modules as $gc_module) {
                        if($gc_module != 'listings' && !empty($modules_count_result['total_' . $gc_module])) {
                            $modules_count[] = array(
                                'module' => $gc_module,
                                'icon' => gc_module_icon($gc_module, true),
                                'counts' => $modules_count_result['total_' . $gc_module]
                            );
                        }
                    }
                    $record->modules_count = $modules_count;
                    // $extra_data['modules_count'] = $modules_count;
                }
            }            
            
            if($module != 'listings' && $module != 'users') { // !$is_edit && 
                if(empty($record->listing_title)) {
                    // get listing information
                    $listing = $this->db->where('id', $record->id)->get('listings')->row();
                    if($listing) {
                        $record->listing_title = $listing->title;
                    } else {
                        $record->listing_title = '';
                    }
                }
            }
            
            // Display meta information
            if(!$is_edit && $module != 'branches') {
                $record->meta_fields = $this->fields_model->find_meta_for_detail($module, $id); 
            }
            
            // find and join meta
            $record = $this->fields_model->find_join_meta($module, $id, $record);
            $this->mb_pages_cache->save($record, $cacheID, array($module . '_' . $id));
        }
        if(empty($record->id)) {
            return false;
        }
        if(!$is_edit && $module != 'branches' && $module != 'users') {
            $extra_data['breadcrumb'] = $record->breadcrumb;
            $extra_data['categories'] = $record->categories_breadcrumb;
            if($module == 'listings') {
                $extra_data['branch_offices'] = $record->branch_offices;
            }
            $extra_data['files'] = $record->extra_files;
            $extra_data['tags'] = $record->extra_tags;
            $extra_data['ratings'] = $record->extra_ratings;  
            if(!empty($record->extra_logo)) {
                $extra_data['logo'] = $record->extra_logo;
            }
            if(!empty($record->extra_cover)) {
                $extra_data['cover'] = $record->extra_cover;                 
            }           
            $extra_data['gallery_images'] = $record->extra_images;            
            $extra_data['gallery_videos'] = $record->extra_videos;
            
            // Check whether listing is user favorite or not
            $this->load->model('favorites/favorites_model');
            $favorites = $this->favorites_model->count_by(array('row_id' => $id, 'module_id' => $module_id, 'user_id' => $loggedin_user_id));
            $extra_data['isFavorite'] = 0;
            if($favorites > 0) {
                $extra_data['isFavorite'] = 1;
            }
            // get listing title to display it on other modules
            if($module != 'users' && $module != 'listings') {
                $extra_data['listing_id'] = $record->listing_id;
                $extra_data['listing_title'] = $record->listing_title;
            }
        }        
        if(!$is_edit && $module != 'branches') {            
            $extra_data['meta_fields'] = $record->meta_fields;
        }
        if(!$is_edit && $module != 'branches' && $module != 'users') {            
            $this->load->model('packages/packages_model');
            $package = $this->packages_model->get_package($record->package_id);
            if(!$package) { // To fix Profiles Import Issue, if the package_id is not entered
                if(!$mobile) {
                    Template::redirect(site_url());
                } else {
                    return false;
                }
            }
            $result['package'] = $package;
            $record->package_id = $package->id;
            $record->package_title = $package->title;
            $hidden_fields = $this->deal_with_package_fields($module, $package, 'display_fields', 0); // display fields           
            
            // add/update recently viewed status
            $this->gc->add_recently_viewed($id, $module, $loggedin_user_id);
            
            // update listing hits
            $this->gc->update_hits($id, $module, true);
        }
        
        // Set Toolbar Title
        if($is_edit) {
            // get package to display it to the use
            if(!empty($record->package_id)) {
                $info_package = $this->gc->get('packages', array('where' => 'id = ' . $record->package_id));
                if($info_package) {
                    $record->package_title = $info_package->title;
                }
            }
            $toolbar_title = lang('edit') . ' ' . lang('singular_' . $module);
        } else {
            $toolbar_title = $record->title;
        }
        
        // Fields Specific
        $fieldsCacheID = 'fields_' . $module;
        if(!$fields_data = $this->mb_pages_cache->load($fieldsCacheID)) {
            // Display Field name and other information to the User
            $this->load->model('fields/fields_model');
            $form_fields = $this->fields_model->get_fields($module, 'frontend');
            
            // Get field groups
            $field_groups_all = array();
            $field_groups = $this->fields_model->field_groups($form_fields, array());
            $field_groups_all[] = array(
                'id' => 0,
                'name' => 'main',
                'name_lang' => 1,
                'mobile_icon' => 'chevron-forward-circle-outline'
            );
            if($field_groups) {
                foreach($field_groups as $field_group) {
                    $field_groups_all[] = array(
                        'id' => $field_group->id,
                        'name' => $field_group->name,
                        'name_lang' => $field_group->name_lang,
                        'mobile_icon' => $field_group->mobile_icon
                    );
                }
            }
            $fields_data = array('groups' => $field_groups_all, 'fields' => $form_fields);
            $this->mb_pages_cache->save($fields_data, $fieldsCacheID, array('fields'));
        }
        $form_fields = $fields_data['fields'];
        $field_groups_all = $fields_data['groups'];
        
        // Other data        
        $recordFieldsCacheID = 'fields_' . $module . '_' . $id . '_' . $loggedin_user_id . '_' . $is_edit;
        if(!$record_fields = $this->mb_pages_cache->load($recordFieldsCacheID)) {
            $record_fields = $this->field_record($module, $loggedin_user_id, $is_edit, $field_groups_all, $form_fields, $hidden_fields, $record, $result);
            $this->mb_pages_cache->save($record_fields, $recordFieldsCacheID, array('fields_' . $module . '_' . $id));            
        }
        $display_fields = $record_fields['display_fields'];
        $common_fields = $record_fields['common_fields'];
        
        $common_fields['id'] = array('value' => $record->id);
        // Map Data
        if(gc_module_settings($module, 'locations') == 1) {	
            $common_fields['latitude'] = array('value' => $record->latitude);
            $common_fields['longitude'] = array('value' => $record->longitude);
            $common_fields['latLng'] = array('value' => array($record->latitude, $record->longitude));
        }
        // Ratings Data
        // $common_fields['ratings'] = array('value' => $record->ratings);
        // $common_fields['rating_users'] = array('value' => $record->rating_users);
        // dump($extra_data);
        // Listing - Business Hours
        $record->open_status = !empty($record->is_open) ? (int) $record->is_open : 0; // not defined
        if(!empty($record->business_hours)) {
            // Open or Closed
            $current_day = date('N');
            $current_time = date('H:i');
            $business_hours = array();
            $hours_business = json_decode($record->business_hours, true);
            $days = array('monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday', 'sunday');
            $j = 0;
            foreach($days as $day) {
                $business_hours[$j] = array('label' => lang('label_' . $day), 'open_time' => null, 'close_time' => null, 'status' => 0);
                $j++;
            }
            $format_24 = settings_item('lst.business_hours_format') == 1 ? true : false;
            foreach($hours_business as $hour_business) {
                $business_hours[$hour_business['day_of_week'] - 1]['open_time'] = $format_24 ? $hour_business['open_time'] : date("h:i A", strtotime($hour_business['open_time']));
                $business_hours[$hour_business['day_of_week'] - 1]['close_time'] = $format_24 ? $hour_business['close_time'] : date("h:i A", strtotime($hour_business['close_time']));
                if($hour_business['day_of_week'] == $current_day) {
                    if($hour_business['open_time'] < $current_time && $hour_business['close_time'] > $current_time) {
                        $business_hours[$hour_business['day_of_week'] - 1]['status'] = 1;
                        if(empty($record->open_status)) {
                            $record->open_status = 1; // open
                        }
                    } else {
                        if(empty($record->open_status)) {
                            $record->open_status = 2; // closed
                        }
                    }
                }
            }
            $extra_data['business_hours'] = $business_hours;
            $extra_data['open_status'] = $record->open_status;
        }
        // Display Social Links
        $social_links = array();
        $social_fields = array('facebook_url', 'twitter_url', 'googleplus_url', 'youtube_url', 'linkedin_url', 'instagram_url', 'pinterest_url', 'website');
        foreach($social_fields as $social_field) {
            if(array_key_exists($social_field, $common_fields)) {
                if(!empty($common_fields[$social_field]['value'])) {
                    switch($social_field) {
                        case 'googleplus_url':
                            $social_icon = 'logo-google';
                            break;
                        case 'website':
                            $social_icon = 'globe-outline';
                            break;
                        default:
                            $social_icon = 'logo-' . str_replace('_url', '', $social_field);
                    }
                    $social_links[] = array(
                        'icon' => $social_icon,
                        'label' => $common_fields[$social_field]['label'],
                        'value' => $common_fields[$social_field]['value']
                    );
                }
                unset($common_fields[$social_field]);
            }
        }
        $extra_data['social'] = $social_links;
        if(in_array($module, gc_modules())) {            
            // the listing owner cannot place order, chat and report incorrect on his/her listing
            if(!$is_edit && $loggedin_user_id != $record->user_id) {
                // Actions - e.g. business_query, comments etc
                $actions = array();
                if($module == 'listings') {  
                    /*$actions[] = array(
                        'label' => lang('place_order'),
                        'icon' => 'cart-outline',
                        'url' => '/tabs/modules/detail/crud/place_order/1/' . $module . '/' . $id
                    );*/
                    if(!empty($record->modules_count)) {
                        foreach($record->modules_count as $module_count) {
                            $actions[] = array(
                                'label' => lang($module_count['module']),
                                'icon' => $module_count['icon'],
                                'url' => '/tabs/modules/' . $module_count['module'] . '/search/' . $id . '/all/0'
                            );
                        }
                    }
                    /*$actions[] = array(
                        'label' => lang('products'),
                        'icon' => 'cube-outline',
                        'url' => '/modules/products/search/' . $id
                    );*/
                }
                if($module == 'products') {
                    $actions[] = array(
                        'label' => lang('btn_buy_now'),
                        'icon' => 'cart-outline',
                        'url' => '/tabs/modules/products/search/' . $record->listing_id . '/all/0'
                    );
                }
                /*if(!empty($package->allow_chat)) {
                    $actions[] = array(
                        'label' => lang('gc_menu_chat'),
                        'icon' => 'chatbubbles-outline',
                        'url' => '/tabs/modules/detail/chat/chat-box/listings/' . $id
                    );
                }*/
                if(!empty($package->allow_query)) {
                    $actions[] = array(
                        'label' => lang('business_query'),
                        'icon' => 'mail-outline',
                        'url' => '/tabs/modules/detail/crud/business_query/0/' . $module . '/' . $id
                    );
                }
                $actions[] =  array(
                    'label' => lang('comments'),
                    'icon' => 'chatbox-outline',
                    'url' => '/tabs/modules/detail/info/comments/' . $module . '/' . $id
                );
                if($module == 'listings') {                
                    $actions[] = array(
                        'label' => lang('label_report'),
                        'icon' => 'reader-outline',
                        'url' => '/tabs/modules/detail/crud/erroneous/0/' . $module . '/' . $id
                    );
                }
                if(gc_module_settings($module, 'locations') == 1 && !empty($record->latitude) && !empty($record->longitude)) {
                    $actions[] = array(
                        'label' => lang('label_directions'),
                        'icon' => 'navigate-outline',
                        'url' => 'directions'
                    );
                }
                $extra_data['actions'] = $actions;
            }
            
            // display trusted seal with module
            if(!empty($package->trusted_seal)) {
                $extra_data['trusted_seal'] = array('value' => base_url('assets/images/' . $package->trusted_seal));
            }
        }
        
        /*$extra_data['marquee'] = array(
            'speed' => 30,
            'timeout' => 1000,
            // 'class' => 'bg-theme-secondary',
            'text' => 'Testing Member Area Testing Member Area Testing Member Area Testing Member Area Testing Member Area Testing Member Area Testing Member Area'
        );*/
        
        if(!empty($record->discount)) { // deals
            if(!empty($record->type) && $record->type == 'F') {
                $common_fields['discount'] = array('value' => currency_format($record->discount) . ' ' . lang('label_off'));
            } else {
                $common_fields['discount'] = array('value' => $record->discount . '% ' . lang('label_off'));
            }
        }
        if(!empty($record->offer_price)) { // products
            $savings = calculate_savings($record->price, $record->offer_price, '');
            $common_fields['discount'] = array('value' => $savings['savings_in_percentage'] . '%'  . ' ' . lang('label_off'));
            $common_fields['html'] = array('value' => $savings['html']);            
            $common_fields['price_with_currency'] = array('value' => currency_format($record->offer_price));
        }
        
        $data = array(
			'common' => $common_fields,
            'fields' => $display_fields,
            'extra' => $extra_data
        );
        $result = array('success' => true, 'data' => $data, 'extra' => array('toolbar_title' => $toolbar_title));
        return $result;
    }
    
    /**
     * Get Module Data
     * @param string $module
     * @param number $id
     * @param boolean $is_edit
     */
    private function get_module_data($module, $id, $is_edit) {        
        if($is_edit) {
            $record = $this->db->where('id', $id)->get($module)->row();
        } else {
            if($module == 'listing') {
                $this->load->model('listings/listings_model');
                $record = $this->listings_model->get_listing($id);
            } else {
                $record = $this->gc->get_module_row_detail($module, $id);
            }
        }
        return $record;
    }
    
    /**
     * Caching - Display module specific fields information
     * Other Data
     */
    private function field_record($module, $loggedin_user_id, $is_edit, $field_groups_all, $form_fields, $hidden_fields, $record, $result) {      
        $group_field_values = $this->_group_field_values($field_groups_all, $form_fields, $hidden_fields, $record, $module, $is_edit);
        $group_wise_fields = $group_field_values['group_wise_fields'];
        $common_fields = $group_field_values['common_fields'];
        $result['group_wise_fields'] = $group_wise_fields;
        $display_fields = $this->_display_fields($module, $loggedin_user_id, $record, $result, $is_edit);
        return array('common_fields' => $common_fields, 'display_fields' => $display_fields);
    }
	
	/**
	 * Group Field Values
	 * @param array $group_fields
	 * @param array $hidden_fields
	 * @param array $record
	 * @param array $module
	 */
	private function _group_field_values($field_groups, $form_fields, $hidden_fields, $record, $module, $is_edit) {
		$group_wise_fields = array();
		$common_fields = array(); // fields hard-coded in template files
		if(!empty($field_groups)) {
            foreach($field_groups as $field_group) {
				$field_group = (object) $field_group;
				$group_fields = gc_form_fields($form_fields, 'group', $field_group->id);
				if(!empty($group_fields)) {
					$group_field_values = array();
					foreach($group_fields as $group_field) {						
						switch($group_field['name']) {
							case 'created_by':
							case 'modified_by':
							case 'created_ip':
							case 'modified_ip':
							case 'created_on':
							case 'modified_on':
							case 'payment_method': // users edit profile hide field
							case 'timezone': // users edit profile hide field
							case 'price':
							case 'offer_price':
							case 'image':
								break;
							default:
								if(!in_array($group_field['name'], $hidden_fields)) { // do not display a field if it is hidden through package
									$is_common_field = false;
									$skip_field = false;
									switch($group_field['name']) {
									    case 'keywords':
									    case 'package_id':
										case 'title':
										case 'description':
										case 'listing_id':
										case 'facebook_url':
										case 'twitter_url':
										case 'googleplus_url':
										case 'youtube_url':
										case 'linkedin_url':
										case 'instagram_url':
										case 'pinterest_url':
										case 'website':
										case 'phone_number':
										case 'mobile_number':
										case 'email':
											$is_common_field = true;
											break;
										case 'type':
										    // fields to be skipped on the modules detail page.
										    // e.g. do not show type on the deals page
										    if($module == 'deals') {
										        $skip_field = true;
										    }
										    break;
										case 'facilities':
										    if(!empty($record->facilities)) { // properties module
										        $property_facilities = json_decode($record->facilities, true);
										        $facilities_formatted = array();
										        foreach($property_facilities as $pdkey => $property_facility) {
										            $facilities_formatted[] = array(
										                'label' => lang('label_' . $pdkey),
										                // 'value' => empty($property_facility) ? lang('not_specified') : $property_facility
										            );
										        }
										        $record->facilities = json_encode($facilities_formatted);
										    }
										    break;
										case 'distances':
										    if(!empty($record->distances)) { // properties module
										        $property_distances = json_decode($record->distances, true);
										        $distances_formatted = array();
										        foreach($property_distances as $pdkey => $property_distance) {
										            $distances_formatted[] = array(
										                'label' => lang('label_' . $pdkey),
										                'value' => empty($property_distance) ? lang('not_specified') : $property_distance
										            );
										        }
										        $record->distances = json_encode($distances_formatted);
										    }
										    break;
									}
									if(!$skip_field) {	
									    // dump($record->distances);
									    // die();
    									if(!$is_edit && $is_common_field) {
    										$common_fields[$group_field['name']] = array(
    											'icon' => 'chevron-forward-circle-outline',
    											'label' => !empty($group_field['label_lang']) ? lang($group_field['label']) : $group_field['label'],
    											'value' => $group_field['name'] == 'description' ? $record->description : $this->fields_model->fields_index($group_field, $record, $module),
    											'rules' => $group_field['rules'],
    											'options' => $group_field['options']
    										);
    									} else {
    										$group_field_values[] = array(
    											'id' => $group_field['id'],
    											'name' => $group_field['name'],
    											'label' => !empty($group_field['label_lang']) ? lang($group_field['label']) : $group_field['label'],
    											'value' => $this->fields_model->fields_index($group_field, $record, $module),
    											'rules' => $group_field['rules'],
    											'options' => $group_field['options'],
    											'type' => $group_field['type']
    										);
    									}
									}
								}
						}
					}
					$group_wise_fields[] = array(
						'id' => $field_group->id, // For edit
                        'name' => ($field_group->name_lang == 1 ?  lang('group_' . $field_group->name) : $field_group->name),
                        'mobile_icon' => $field_group->mobile_icon,
                        'fields' => $group_field_values
					);
				}
			}
		}
		return array('group_wise_fields' => $group_wise_fields, 'common_fields' => $common_fields);
	}
    
    /**
     * Deal with display and permission fields enabled/disabled through packages
     * @param string $module
     * @param object $package
     * @param string $type
     * @param number $expression_value
     *
     * @return array
     */
    public function deal_with_package_fields($module, $package, $type = 'display_fields', $expression_value = 1) {
        // Deal with fields hidden through package
        $fields = json_decode($package->{$type} ?? '', true);
        $result = array();
        if(!empty($fields[$module])) {
            $fields = $fields[$module];
            if(!empty($fields)) {
                foreach($fields as $field_name => $field_value) {
                    if($field_value == $expression_value) {
                        $result[] = $field_name;
                    }
                }
            }
        }
        return $result;
    }    
    
    /**
     * Module Display Fields Detail and Edit Page
     * @param array $data
     */
    private function _display_fields($module, $user_id, $record, $data, $is_edit = false) {
        $package = !empty($data['package']) ? $data['package'] : new stdClass();
        $display_fields = array();
        $group_fields = $data['group_wise_fields'];
        $extra_info = array();
        if(!empty($group_fields)) {
            foreach($group_fields as $group_key => $group_field) {
                $group_wise_fields = $group_field['fields'];
                $field_values = array();
                foreach($group_wise_fields as $group_field_key => $group_wise_field) {
                    $value_field = $group_wise_field['value'];
					switch($group_wise_field['name']) {
                        case 'description':
                            if($is_edit) {
                                $value_field = truncate_text($group_wise_field['value'], 100);
                                $value_field = array('value' => $value_field, 'extra' => array());
                            } else {
                                $value_field = array('value' => $value_field, 'extra' => array());
                            }
                            break;
                        case 'is_online':
                            $value_field = array('value' => $record->is_online == 1 ? lang('bf_online') : lang('bf_offline'), 'extra' => array());
                            break;
                        case 'latitude_longitude':
                            if($is_edit) {
                                $value_field = array('value' => $record->latitude . ', ' . $record->longitude);
                            }
                            break;
                        case 'last_login':
                            $value_field = array('value' => convert_date_time($record->last_login), 'extra' => array());
                            break;
                        default:
                            $value_field = array('value' => $group_wise_field['value'], 'extra' => array());
                            if(!empty($group_wise_field['options']) && strpos($group_wise_field['rules'], 'is_array') !== FALSE) {
                                $value_field['extra']['is_array'] = true;
								if($group_wise_field['type'] == 'text') {
									$value_field['extra']['text_array'] = true;
								}
                            }
                            // $value_field = $this->fields_model->fields_index($group_wise_field, $record, $module);
                            // $value_field = array('value' => $value_field, 'extra' => array());
                    }
                    if(!empty($value_field['value']) || !empty($value_field['extra'])) {
                        $value_field['extra']['group_index'] = $group_key; // update information on an action perform e.g. change status to awaiting when user requests an info
                        $value_field['extra']['field_index'] = $group_field_key; // update information on an action perform e.g. change status to awaiting when user requests an info
                        $field_values[] = array(
                            'id' => $group_wise_field['id'],
                            'name' => $group_wise_field['name'],
                            'label' => $group_wise_field['label'],
                            'value' => $value_field
                        );
                    }
                }
                $display_fields[$group_key] = array(
                    'id' => $group_field['id'],
                    'name' => $group_field['name'],
                    'mobile_icon' => $group_field['mobile_icon'],
                    'fields' => $field_values
                );
            }
        }
        return $display_fields;
    }   
    
    /**
     * Get rows as user type keyword using AJAX
     *
     * @param string $module - name of the module to be searched
     * @param string $keyword
     * @param number $fields
     * @param boolean $mobile
     * @return object listings
     */
    public function search_suggestions($module, $keyword, $fields = array(), $mobile = true) {
        $module_id = gc_module_id($module);
        if (settings_item('site.full_text_search') == 1) {
            $keyword = str_replace(" ", " +", $keyword);
        }        
        $hasLocation = ($module == 'listings' || gc_module_settings($module, 'locations') == 1 ? TRUE : FALSE);
        $limit = 10;
        
        /*
         * M - Module
         * PC - Parent Category
         * SC - Sub Category
         * SSC - Sub Sub Category
         * T - Tag
         * CASE WHEN id <> 0 THEN 'M' END AS type
         */
        $sql = "SELECT M.id, M.slug, M.name as title, M.type FROM";
        
        // Modules Search
        $sql .= " ((SELECT MD.id, MD.slug, MD.title as name, '" . $module . "' AS type FROM " . $this->db->dbprefix . $module . " MD";
        $sql .= " WHERE MD.deleted = 0 AND MD.status = 1";
        $sql .= $this->fields_sql($fields, $hasLocation);       
        $table_prefix = "MD.";
        if (settings_item('site.full_text_search') == 1) {
            $sql .= " AND MATCH (" . $table_prefix . "title," . $table_prefix . "description," . $table_prefix . "keywords," . $table_prefix . "categories) AGAINST ('\"+" . $keyword . "*\"' IN BOOLEAN MODE)";
        } else {
            $sql .= " AND (" . $table_prefix . "title LIKE '" . $keyword . "%' OR " . $table_prefix . "description LIKE '%" . $keyword . "%' OR " . $table_prefix . "keywords LIKE '%" . $keyword . "%' OR " . $table_prefix . "categories LIKE '%" . $keyword . "%')";
        }
        $sql .= " LIMIT " . $limit;
        $sql .= " ) UNION";
        
        // Categories Search
        if(!array_key_exists('category_id', $fields) && empty($fields['category_id'])) {
            $sql .= "(SELECT C.id, C.slug, C.name, 'C' AS type FROM " . $this->db->dbprefix . "categories C";
            $sql .= " WHERE C.active = 1 AND FIND_IN_SET(" . $module_id . ", C.module_relations)";            
            if (settings_item('site.full_text_search') == 1) {
                $sql .= " AND ( MATCH ( C.name ) AGAINST ( '\"+" . $keyword . "*\"' IN BOOLEAN MODE ))";
            } else {
                $sql .= " AND C.name LIKE '%" . $keyword . "%'";
            }            
            $sql .= " LIMIT " . $limit;
            $sql .= " ) UNION";
        }
        
        // Tags Search
        if(!array_key_exists('tag_id', $fields)) {
            $sql .= " (SELECT T.id, T.slug, T.name, 'T' AS type FROM " . $this->db->dbprefix . "tags T ";
            $sql .= " WHERE T.active = 1 AND FIND_IN_SET(" . $module_id . ", T.module_relations)";
            if (settings_item('site.full_text_search') == 1) {
                $sql .= " AND ( MATCH ( T.name ) AGAINST ( '\"+" . $keyword . "*\"' IN BOOLEAN MODE ))";
            } else {
                $sql .= " AND T.name LIKE '%" . $keyword . "%'";
            }
            $sql .= " LIMIT " . $limit . ")";
        }
        
        $sql .= " ) M ORDER BY M.name ASC LIMIT " . $limit;
        $query = $this->db->query($sql);
        $result = $query->result_array();
        return $result;
    }
    
    /**
     * Search Fields Query
     * @param array $fields
     */
    private function fields_sql($fields, $hasLocation = false) {
        $sql = '';
        if(!empty($fields)) {
            foreach($fields as $field => $value) {
                switch($field) {
                    case 'category_id':
                    case 'tag_id':
                        if(!empty($value)) {
                            $sql .= " AND FIND_IN_SET(" . $value . ", MD.categories_set)";
                        }
                        break;
                    case 'location':
                        if($hasLocation) {
                            $sql .= " AND MD.location LIKE '%" . $value . "%'";
                        }
                        break;
                    case 'city_id':
                        if($hasLocation) {
                            $sql .= " AND MD.city_id = " . $value;
                        }
                        break;
                    default:
                        $sql .= " AND MD." . $field . " = " . $value;
                }
            } 
        }
        return $sql;
    }
    
    /**
     * Listings, Properties and Other Modules information for Report Incorrect, Business Queries and other modules
     * @param string $module
     * @param number $row_id
     * @param number $user_id - Logged in user id
     * @param boolean $check_permission - Check Permission for edit module
     * @return string
     */
    public function get_info($module, $row_id, $user_id = 0, $check_permission = false) {
        $result = $this->db->where('id', $row_id)->get($module)->row();
        if($check_permission && !empty($user_id) && (!$result || $result->user_id != $user_id)) {
            $result = false;
        }
        return $result;
    }    
    
    /**
     * Get all slider banners for home screen
     * @return array[]
     */
    public function get_sliders($group_id = 0, $format = true, $id = 0) {
        $sql = "SELECT SB.id, SB.title, SB.image, SB.heading, SB.description, SB.link_url, SB.params"; //  SB.module_id, SB.row_id, SB.row_title, SB.external_url
        $sql .= " FROM " . $this->db->dbprefix . "mobile_sliders SB";
        $sql .= " WHERE SB.status = 1";
        if(empty($id)) {
            $sql .= " AND (SB.start_date <= '".date("Y/m/d") ."' OR SB.start_date IS NULL) AND (SB.end_date >= '".date("Y/m/d") ."' OR SB.end_date IS NULL)";
            $sql .= " AND ( SB.clicks <= SB.max_clicks OR SB.max_clicks =0 )";
        }
        if($group_id != 0) {
            $sql .= " AND FIND_IN_SET(" . $group_id . ", SB.slider_group_id)";
        }
        if(!empty($id)) {
            $ids = explode(',', $id);
            $where = array();
            foreach($ids as $id) {
                $where[] = "FIND_IN_SET(" . $id . ", SB.id)";
            }
            $sql .= " AND (" . implode(' OR ', $where) . ")";
        }
        $sql .= " GROUP BY SB.id ORDER BY SB.priority";
        $query = $this->db->query($sql);
        $records = $query->result();
        $banners = array();
        if($format && $records) {
            foreach($records as $banner) {
                $slider = array (
                    'title' => $banner->title,
                    'image' => valid_url($banner->image) ? $banner->image : base_url('assets/images/' . $banner->image),
                    // 'heading' => $banner->heading,
                    'description' => $banner->description,
                    'url' => $banner->link_url, // redirect page
                    'params' => gc_process_params($banner)
                );
                // if the link_url is none, module must be set for page redirect e.g. /login, /signup etc.
                // the redirect parameter is used to redirect user to a specific screen or page
                /*$saved_params = json_decode($banner->params ?? '', true);
                 if(!empty($saved_params)) {
                 foreach($saved_params as $saved_param) {
                 $slider[$saved_param['key']] = $saved_param['value'];
                 }
                 }*/
                $banners[] = $slider;
            }
        } else {
            $banners = $records;
        }
        return $banners;
    }
    
    /**
     * Get all slider banners for home screen
     * @return array[]
     */
    public function get_sliders_old($group_id = 0, $format = true) {
        $sql = "SELECT SB.id, SB.title, SB.image, SB.heading, SB.description, SB.redirect_url";
        $sql .= " FROM " . $this->db->dbprefix . "mobile_sliders SB";
        $sql .= " WHERE ((SB.start_date <= '".date("Y/m/d") ."') || (SB.start_date = '' OR SB.start_date IS NULL)) AND ((SB.end_date >= '".date("Y/m/d") ."') || (SB.end_date = '' OR SB.end_date IS NULL))";
        $sql .= " AND ( SB.clicks <= SB.max_clicks OR SB.max_clicks = 0 )";
        if($group_id != 0) {
            $sql .= " AND FIND_IN_SET(" . $group_id . ", SB.slider_group_id)";
        }
        $sql .= " AND SB.status = 1 GROUP BY SB.id ORDER BY SB.priority";
        $query = $this->db->query($sql);
        $records = $query->result();
        $banners = array();
        if($format && $records) {
            foreach($records as $banner) {
                $banners[] = array (
                    // 'title' => $banner->title,
                    'image' => gc_image_url($banner->image, false, false, array('module' => 'mobile_sliders')), // base_url('assets/images/' .$banner->image),
                    'heading' => $banner->heading,
                    'description' => $banner->description,
                    'url' => $banner->redirect_url
                );
            }
        } else {
            $banners = $records;
        }
        return $banners;
    }
    
    /**
     * Save image to the temp folder
     * @param string $file - Name of the file
     * @param string $concat_string - String to be concated with filename
     * 
     * @return string
     */
    public function save_image($file, $concat_string = null, $path = null, $resize = false) {
        if(strpos($file, 'data:image/jpeg;base64') !== false) {
            // All images will be saved in the TEMP folder
            // Give a new name to the file using Timestamp
            $filename = (!empty($concat_string) ? $concat_string : '') . '_' . time() . rand(1000, 9999) . '.jpg';
            if(settings_item('site.cloudinary') == 0) { // the cloudinary is off, images are stored locally
                $image_blob = str_replace('data:image/jpeg;base64,', '', $file);
                $full_path = $path;
                if(empty($path)) {
                    $full_path = $this->images_model->temp_folder_full_path;
                    $path = $this->images_model->temp_folder;
                }
                if(file_exists(realpath($full_path . $filename))) {
                    $filename = rand(1,100) .$filename;
                }
                // $temp_file = FCPATH . $path . $filename;
                $temp_file = $full_path . $filename;
                file_put_contents($temp_file, base64_decode($image_blob));
                if($resize) {
                    $this->load->model('images/images_model');
                    $this->images_model->resize_image($path, $filename);
                }
            } else { // upload image on the cloudinary server
                $this->load->model('images/images_model');
                $filename = $this->images_model->cloudinary($filename, $file);
            }
            return $filename;
        } else {
            return false;
        }
    }
    
    /***************************************************************************
     * END OF FRONTEND
     ***************************************************************************/
    
    
    /***************************************************************************
     * START OF ONESIGNAL API
     ***************************************************************************/
    
    /**
     * Get socket data from the mobile_templates table and send to the user
     * @param number $user_id
     */
    public function prepare_notification($user_id, $data, $params = array()) {
        $templates = $this->db->where($data)->get('mobile_templates')->result();
        if($templates) {
            $this->load->library('parser');
            foreach($templates as $template) {
                $subject = $this->parser->parse_string($template->subject, $params, true);
                $message = $this->parser->parse_string($template->message, $params, true);
                $template_data = array(
                    'title' => $subject,
                    'message' => $message
                );
                if(!empty($template->params)) {
                    $template_data['alert'] = $this->email_param_values($template->params, $params);
                }
                $notification_type = !empty($template->notification_type) ? $template->notification_type : 'onesignal';
                $is_socket = false;
                $is_onesignal = false;
                $is_fcm = false;
                switch($notification_type) {
                    case 'socket':
                        $is_socket = true;
                        break;
                    case 'both':
                        $is_socket = true;
                        $is_onesignal = true;
                        break;
                    default:
                        $is_onesignal = true;
                }
                if($is_socket) {
                    // send socket notification in background
                    $this->socket_notification($user_id, $template_data);
                    if($is_onesignal) {
                        // run notification in background
                    }
                } else {
                    // one signal notification
                    if($is_fcm) {
                        $tokens = $this->db->select('token')->where('user_id', $user_id)->limit(2)->order_by('modified_on', 'DESC')->get($this->table_devices)->result();
                    } else if($is_onesignal) {
                        $tokens = $this->db->select('player_id')->where('user_id', $user_id)->limit(2)->order_by('modified_on', 'DESC')->get($this->table_player_ids)->result();
                    }
                    $template_data['tokens'] = $tokens;
                    // $this->onesignal_background($user_id, $data, $params);
                    // $this->fcm_notification($template_data);
                    // $this->onesignal_notification($template_data);
                    $this->send_notification($template_data, $is_onesignal ? 'onesignal' : 'fcm');
                }
            }
        }
    }
    /**
     * Send Socket Notification to the user
     * @return boolean
     */
    public function socket_notification($user_id, $data = array()) {
        // send notification to the user
        /*$this->load->library('socketio');
        $data['user_id'] = $user_id;
        $this->socketio->send($data);
        */
        // run task in background
        $data['user_id'] = $user_id;
        $this->gc->background_task('socket', json_encode($data));
        exec_background('socket', 'php', true);
    }
    
    /**
     * Send One Signal Notification in Background Mode
     */
    public function send_notification($data = array(), $method = 'onesignal') {
        $this->gc->background_task($method, json_encode($data));
        // /usr/bin/nohup php: spawn a background process to perform firebase calls asynchronously.
        exec_background($method, 'php', true);
        // $this->onesignal_notification(json_decode(json_encode($data), true));
    }
    
    /**
     * Send FCM notification
     * @param array $template_data
     */
    public function fcm_notification($template_data) {
        $url = "https://fcm.googleapis.com/fcm/send";
        $serverKey = 'AAAAOEEGR9U:APA91bHxcY7VJinhbIf_srx4lBEdoCNhNH72zLK4PlkwsvlwxL_GHZJh59nqLlZXxO07f7cbnEfnOfaT0yATwGCGBKim36zZjU2iV-wastTnKMvEpjmzymTvBOXArZRjcewJHI4LlrfX';
        $notification = array(
            'title' =>$template_data['title'] ,
            'text' => $template_data['message'],
            'sound' => !empty($template_data['alert']['sound']) ? $template_data['alert']['sound'] : null,
            'badge' => '1',            
// "click_action" => "FCM_PLUGIN_ACTIVITY"
        );
        $data = array('alert' => !empty($template_data['alert']) ? $template_data['alert'] : array());
        $tokens = $template_data['tokens'];
        unset($template_data['tokens']);
        if(!empty($tokens)) {
            foreach($tokens as $user) {                
                $arrayToSend = array('to' => $user['token'], 'data' => $data, 'notification' => $notification, 'priority'=>'high');
                $json = json_encode($arrayToSend);
                $headers = array();
                $headers[] = 'Content-Type: application/json';
                $headers[] = 'Authorization: key='. $serverKey;
                $ch = curl_init();
                curl_setopt($ch, CURLOPT_URL, $url);
                curl_setopt($ch, CURLOPT_CUSTOMREQUEST,"POST");
                curl_setopt($ch, CURLOPT_POSTFIELDS, $json);
                curl_setopt($ch, CURLOPT_HTTPHEADER,$headers);
                //Send the request
                $response = curl_exec($ch);
                //Close request
                if ($response === FALSE) {
                    die('FCM Send Error: ' . curl_error($ch));
                }
                curl_close($ch);
            }
        }
    }
    
    /**
     * Send Onesignal notification only
     * @param array $template_data
     */
    public function onesignal_notification($template_data) {
        $mobile_settings = json_decode(settings_item('mobile.settings'), true);
        $app_id = $mobile_settings['onesignal_app_id'];
        $rest_api = $mobile_settings['onesignal_rest_api'];
        $url = 'https://onesignal.com/api/v1/notifications';
        $tokens = $template_data['tokens'];
        unset($template_data['tokens']);
        if(!empty($tokens)) {
            $player_ids = array();
            foreach($tokens as $user) {
                $player_ids[] = $user['player_id'];
            }
            $data['app_id'] = $app_id;
            $data['include_player_ids'] = $player_ids;
            $data['data'] = array('alert' => !empty($template_data['alert']) ? $template_data['alert'] : array());
            $data['headings'] = array(
                'en' => $template_data['title']
            );
            $data['contents'] = array(
                'en' => $template_data['message']
            );
            $json = json_encode($data);
            $headers = array();
            $headers[] = 'Content-Type: application/json';
            $headers[] = 'Authorization: Basic='. $rest_api;
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $url);
            curl_setopt($ch, CURLOPT_CUSTOMREQUEST,"POST");
            curl_setopt($ch, CURLOPT_POSTFIELDS, $json);
            curl_setopt($ch, CURLOPT_HTTPHEADER,$headers);
            //Send the request
            $response = curl_exec($ch);
            //Close request
            if ($response === FALSE) {
                die('Onesignal Send Error: ' . curl_error($ch));
            }
            curl_close($ch);
        }
    }
    
    /**
     * Send Notification
     * @param number $user_id
     * @param array $data - array('module_table' => 'orders', 'user_type' => 'seller', 'action_type' => 'add');
     * @param array $params - array('order_id' => 10);
     */
    public function onesignal_background($user_id, $data = array(), $params = array()) {
        // get player_ids
        $players = $this->db->select('player_id')->where('user_id', $user_id)->limit(2)->order_by('modified_on', 'DESC')->get($this->table_player_ids)->result();
        if($players) {
            $this->load->library('parser');
            $player_ids = array();
            foreach($players as $player) {
                $player_ids[] = $player->player_id;
            }
            $templates = $this->db->where($data)->get('mobile_templates')->result();
            if($templates) {
                $this->load->library('parser');
                foreach($templates as $template) {
                    $template_data = array();
                    $template_data['include_player_ids'] = $player_ids; // ad006690-e993-4857-9bb8-cfa8b385b611
                    // redirect user on notification open and click
                    // if(!empty($params['data'])) {
                    if(!empty($template->params)) {
                        $template_data['data']['alert'] = $this->email_param_values($template->params, $params);
                    }
                    if(!empty($template->app_url)) { // application launch url
                        // $template_data['app_url'] = 'https://www.fyond.com/' . $this->parser->parse_string($template->app_url, $params, true);
                    }
                    
                    $subject = $this->parser->parse_string($template->subject, $params, true);
                    $message = $this->parser->parse_string($template->message, $params, true);
                    // $data['included_segments'] = array('Active Users');
                    $template_data['headings'] = array(
                        'en' => $subject
                    );
                    $template_data['contents'] = array(
                        'en' => $message
                    );
                    $template_data['android_sound'] = 'message.mp3';                    
                    $this->call_onesignal('notification', 'POST', $template_data);
                }
            }
        }
    }
    
    /**
     * Notification template
     * @param string $params - 'sound:notification;header:test;message:new message;url:/tabs/home;cancel:Close';
     * @params array $data - Template Parameters
     * @return string[]|mixed[]|NULL
     */
    private function email_param_values($params, $data) {
        $this->load->library('parser');
        $params = $this->parser->parse_string($params, $data, true);
        if(!empty($params)) {
            $param_values = array();
            // get all params
            $params_array = array_map('trim', explode(';', $params ?? ''));
            if($params_array) {
                foreach($params_array as $param_field) {
                    $param_value = array_map('trim', explode(':', $param_field ?? ''));
                    $param_values[$param_value[0]] = (isset($param_value[1]) ? $param_value[1] : '');
                }
            }
            return $param_values;
        }
        return null;
    }
    
    /**
    * Save push token for notifications
    * @param number $user_id
    */
    public function save_push_token($user_id) {
        $device = $this->input->post('device');
        if(!empty($_POST['push_token']) && !empty($device['uuid'])) {
            $datetime = date('Y-m-d H:i:s');
            $token = $_POST['push_token'];
            $data = array();
            $record = $this->db->where('uuid', $device['uuid'])->get($this->table_devices)->row();
            if(!$record) {                
                $data = array(
                    'user_id' => $user_id,
                    'token' => $token,
                    'uuid' => $device['uuid'],
                    'platform' => !empty($device['platform']) ? $device['platform'] : null,
                    'model' => !empty($device['model']) ? $device['model'] : null,
                    'version' => !empty($device['osVersion']) ? $device['osVersion'] : null,
                    'manufacturer' => !empty($device['manufacturer']) ? $device['manufacturer'] : null,
                    'created_on' => $datetime,
                    'created_ip' => $this->input->ip_address()
                );
                $this->db->insert($this->table_devices, $data);
            } else {
                $data['user_id'] = $user_id;
                $data['token'] = $_POST['push_token'];
                $data['modified_on'] = $datetime;
                $data['modified_ip'] = $this->input->ip_address();
                $this->db->where('id', $record->id)->update($this->table_devices, $data);                
            }
            // $sql = "INSERT INTO " . $this->db->dbprefix . $this->table_devices . " (user_id, uuid, token, platform, model, version, manufacturer, created_on, modified_on) VALUES(" . $user_id . ", '" . $token ."', '" . $datetime . "', '" . $datetime . "')";
            // $sql .= " ON DUPLICATE KEY UPDATE user_id = " . $user_id . ", modified_on = '" . $datetime . "'";
            // $this->db->query($sql);
            return array('success' => true);
        } else {
            return array('success' => false);
        }
    }
    
    /**
     * Save one signal player id to send notifications
     * @param number $user_id
     */
    public function save_player_id($user_id) {
        if(!empty($_POST['player_id'])) {
            $player_id = $_POST['player_id'];
            $datetime = date('Y-m-d H:i:s');
            $sql = "INSERT INTO " . $this->db->dbprefix . $this->table_player_ids . " (user_id, player_id, created_on, modified_on) VALUES(" . $user_id . ", '" . $player_id ."', '" . $datetime . "', '" . $datetime . "')";
            $sql .= " ON DUPLICATE KEY UPDATE user_id = " . $user_id . ", modified_on = '" . $datetime . "'";
            $this->db->query($sql);
            return array('success' => true);
        } else {
            return array('success' => false);
        }
    }
    
    /**
     * Send Test Notification
     * @param string $player_id
     * @param string $app_string
     * @param array $params
     */
    public function test_notification($player_id, $app_string = null, $params = array()) {
        $template_data = array();
        $template_data['include_player_ids'] = array($player_id); // ad006690-e993-4857-9bb8-cfa8b385b611
        if(!empty($app_string)) { // application launch url
            $template_data['app_url'] = 'https://www.fyond.com/' . $this->parser->parse_string($app_string, $params, true);
        }
        // $data['included_segments'] = array('Active Users');
        $template_data['headings'] = array(
            'en' => lang('notification_test_heading')
        );
        $template_data['contents'] = array(
            'en' => lang('notification_test_message')
        );
        $template_data['data'] = array('alert' => array('sound' => 'notification', 'header' => 'Test', 'message' => 'New Message', 'url' => '/tabs/member', 'cancel' => 'Close'));
        $template_data['big_picture'] = gc_site_image('bookings');
        // ios - https://documentation.onesignal.com/reference/create-notification
        // $template_data['ios_attachments'] = '{"id1": "https://domain.com/image.jpg"}';
        // dump($template_data);
        $this->call_onesignal('notification', 'POST', $template_data);
    }
    
    /**
     * One Signal API
     * @param string $method
     * @param string $url
     * @param array $data
     * @return mixed
     */
    public function call_onesignal($action = 'notification', $method = "POST", $data = array()) {
        $mobile_settings = json_decode(settings_item('mobile.settings'), true);
        if(empty($mobile_settings['onesignal_rest_api'])
            || empty($mobile_settings['onesignal_app_id'])
            || empty($mobile_settings['android_app_id'])) {
                return false;
            }
            $app_id = $mobile_settings['onesignal_app_id'];
            $rest_api = $mobile_settings['onesignal_rest_api'];
            if($action == 'notification') {
                $url = 'https://onesignal.com/api/v1/notifications';
                $data['app_id'] = $app_id;
            } else {
                $url = 'https://onesignal.com/api/v1/players/csv_export?app_id=' . $app_id;
            }
            $curl = curl_init();
            switch ($method){
                case "POST":
                    curl_setopt($curl, CURLOPT_POST, TRUE);
                    if (!empty($data))
                        curl_setopt($curl, CURLOPT_POSTFIELDS, json_encode($data));
                        break;
                case "PUT":
                    curl_setopt($curl, CURLOPT_CUSTOMREQUEST, "PUT");
                    if ($data)
                        curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
                        break;
                default:
                    if ($data)
                        $url = sprintf("%s?%s", $url, http_build_query($data));
            }
            // OPTIONS:
            curl_setopt($curl, CURLOPT_URL, $url);
            curl_setopt($curl, CURLOPT_HTTPHEADER, array(
                'Content-Type: application/json;' . 
                'Authorization: Basic ' . $rest_api
            ));
            curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($curl, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
            // EXECUTE:
            $result = curl_exec($curl);
            if(!$result){die("Connection Failure");}
            curl_close($curl);
            $result = json_decode($result, true);
            if($action != 'notification') {
                // download the CSV file
                if(!empty($result['csv_file_url'])) {
                    // download the file
                    $this->download_onesignal_users($result['csv_file_url'], FCPATH . 'assets/tmp/' . 'onesignal_users.csv.gz');
                    
                    // exract the gz file
                    $this->extract_gz();
                    
                    // import csv file into database
                    $this->import_csv();
                }
            }
            return $result;
    }
    
    /**
     * Download CSV File from OneSignal and import it in local database
     * @param string $file_source
     * @param string $file_target
     * @return boolean
     */
    private function download_onesignal_users($file_source, $file_target) {
        $rh = fopen($file_source, 'rb');
        $wh = fopen($file_target, 'w+b');
        if (!$rh || !$wh) {
            return false;
        }
        while (!feof($rh)) {
            if (fwrite($wh, fread($rh, 4096)) === FALSE) {
                return false;
            }
            echo ' ';
            flush();
        }
        fclose($rh);
        fclose($wh);
        return true;
    }
    
    /**
     * Extract GZ file
     */
    private function extract_gz() {
        // This input should be from somewhere else, hard-coded in this example
        $file_name = FCPATH . 'assets/tmp/' . 'onesignal_users.csv.gz';
        
        // Raising this value may increase performance
        $buffer_size = 4096; // read 4kb at a time
        $out_file_name = str_replace('.gz', '', $file_name);
        
        // Open our files (in binary mode)
        $file = gzopen($file_name, 'rb');
        $out_file = fopen($out_file_name, 'wb');
        
        // Keep repeating until the end of the input file
        while(!gzeof($file)) {
            // Read buffer-size bytes
            // Both fwrite and gzread and binary-safe
            fwrite($out_file, gzread($file, $buffer_size));
        }
        
        // Files are done, close files
        fclose($out_file);
        gzclose($file);
    }
    
    /**
     * Import CSV file
     * $this->db->simple_query($query) command is used instead of $this-db->query($query)
     * as query command was generating errors.
     */
    private function import_csv() {
        $fields_terminated_by = ',';
        $file = FCPATH . 'assets/tmp/onesignal_users.csv';
        if(file_exists($file)) {
            $table = 'mobile_push_users';
            $this->load->dbforge();
            $fields = array();
            // delete table
            if ($this->db->table_exists($table)) {
                $this->dbforge->drop_table($table);
            }
            
            // get first row of csv file
            $file_read = fopen($file, 'r');
            $csv_fields = fgetcsv($file_read, "r", "$fields_terminated_by");
            foreach($csv_fields as $csv_field) {
                $csv_field = strtolower($csv_field);
                $fields[$csv_field] = array(
                    'type' => 'VARCHAR',
                    'constraint' => 255,
                    'null' => $csv_field == 'id' ? false : true
                );
            }
            
            // recreate table
            $this->dbforge->add_field($fields);
            $this->dbforge->add_key('id', true);
            $this->dbforge->create_table($table);
            
            // $file_content = fgetcsv($file_read, "r", ",");
            // dump($file_content);
            $data = array();
            while (($column = fgetcsv($file_read, 10000, ",")) !== FALSE) {
                $data[] = array_combine($csv_fields, $column);
            }
            if(!empty($data)) {
                $this->db->insert_batch($table, $data);
            }
            fclose($file_read);
        }
    }
    /***************************************************************************
     * END OF ONESIGNAL API
     ***************************************************************************/
    
    /***************************************************************************
     * LOCATION AND INTEREST SPECIFIC BANNER SLIDERS
     ***************************************************************************/
    
    /**
     * Prepare Ads for display
     * @param object $records
     * @return array
     */
    public function prepare_ads($records, $latitude = null, $longitude = null) {
        $banners = array();
        if($records) {
            $settings = json_decode(settings_item('mobile.settings'), true);
            $ad_view_cost = $settings['ad_view_cost'];
            $ad_views_data = array();
            $update_data = array();
            $disabled_banners = array();
            foreach($records as $banner) {
                // get today views
                $where = "ad_id = " . $banner->id . " AND DATE(created_on) = '" . date('Y-m-d') . "'";
                $today_views = (int) $this->db->where($where)->count_all_results($this->table_ad_views);
                $ad_views = $today_views + 1;
                $todays_spent = decimal_convert($ad_views * $ad_view_cost);
                if($banner->wallets > $todays_spent) { // the user wallet has balance
                    $module_name = gc_module_name($banner->module_id);
                    $redirect_url = $banner->redirect_url;
                    if(empty($banner->redirect_url) && !empty($module_name) && in_array($module_name, gc_modules())) {
                        $redirect_url = '/tabs/modules/detail/' . $module_name . '/' . $banner->ad_row_id;
                    }
                    $banners[] = array (
                        'image' => gc_image_url($banner->image, false, false, array('module' => 'mobile_sliders')),
                        'heading' => $banner->heading,
                        'description' => $banner->description,
                        'url' => $redirect_url
                    );
                    $ad_views_data[] = array(
                        'ad_id' => $banner->id,
                        'latitude' => $latitude,
                        'longitude' => $longitude,
                        'distance' => $banner->distance,
                        'todays_spent' => $todays_spent,
                        'created_on' => date('Y-m-d H:i:s')
                    );
                    $update_data[] = array(
                        'id' => $banner->id,
                        'ad_views' => $ad_views,
                        'todays_spent' => $todays_spent
                    );
                } else {
                    // disable the banner to avoid further lookup
                    $disabled_banners[] = array(
                        'id' => $banner->id,
                        'status' => 0
                    );
                }
            }
            if(!empty($ad_views_data)) {
                // update information in the ad views table            
                $this->db->insert_batch($this->table_ad_views, $ad_views_data);
                
                // update information in the ads table
                $this->db->update_batch($this->table_ads, $update_data, 'id');
            }
            if(!empty($disabled_banners)) {
                $this->db->update_batch($this->table_ads, $disabled_banners, 'id');
            }
        } else {
            $banners = $records;
        }
        return $banners;
    }
    
    /**
     * Process Ad View via Cronjob
     * Deduct the wallet amount and max_budget amount
     */
    public function cron_ad_view_process() {
        $settings = json_decode(settings_item('mobile.settings'), true);
        $ad_views = $settings['ad_view_process_limit'];
        $ad_view_cost = $settings['ad_view_cost'];
        $cost = decimal_convert($ad_views * $ad_view_cost);
        $where = 'user_id != 0  AND ad_views >= ' . $settings['ad_view_process_limit'];
        $records = $this->db->where($where)->get($this->table_ads)->result();
        $data = array();
        if($records) {
            $this->load->model('wallets/wallets_points_model');
            foreach($records as $record) {
                $user_id = $record->user_id;
                // amount
                $row = array(
                    'id' => $record->id,
                    'ad_views' => $record->ad_views - $ad_views,
                );
                if($record->max_budget != 0) {
                    $row['max_budget'] = $record->max_budget - $cost;
                }
                $data[] = $row;
                
                // deduct the amount from the wallet
                $this->wallets_points_model->update_cash('wallets', $user_id, 'ads', $cost, 'debit', null, $record->id, array('views' => $ad_views));
            }
            // update mobile_sliders table
            $this->db->update_batch($this->table_ads, $data, 'id');
        }
    }
    
    /**
     * Update the latitude, longitude, city_id, user_id and other information in the ads table
     */
    public function after_ad_save($id, $action_type = 'insert', $record = array()) {
        $record = $this->db->where('id', $id)->get($this->table_ads)->row();
        if($record) {
            $data = array();
            $title = lang('label_ad') . '-' . $id;
            $module = gc_module_name($record->module_id);
            if(in_array($module, gc_modules())) {
                $this->db->from($module . ' M');
                if(gc_module_settings($module, 'locations') != 1) {
                    $this->db->join('listings L', 'L.id = M.listing_id', 'INNER');
                    $select = 'M.title, L.user_id, L.latitude, L.longitude, L.city_id';
                } else {
                    $select = 'M.title, M.user_id, M.latitude, M.longitude, M.city_id';
                }
                $result = $this->db->select($select)->where('M.id', $record->ad_row_id)->get()->row();
                if($result) {
                    $data = array(
                        'user_id' => $result->user_id,
                        'latitude' => $result->latitude,
                        'longitude' => $result->longitude,
                        'city_id' => $result->city_id
                    );
                    $title .= '::' . $result->title;
                }
            }
            if(empty($record->title)) {
                $data['title'] = $title;
            }
            if(!empty($data)) {
                $this->db->where('id', $id)->update($this->table_ads, $data);
            }
        }
    }
    
    /**
     * The user can start an Ad only if the user has amount in his/her wallet and the max_budget is not not exhausted
     * @param number $ad_id
     */
    public function update_ad_status($ad_id, $user_id = 0) {
        $this->db->from($this->table_ads . ' M');
        $this->db->join('users U', 'U.id = M.user_id', 'INNER');
        $record = $this->db->select('M.user_id, M.max_budget, M.status, U.wallets')->where('M.id', $ad_id)->get()->row();
        $new_status = $record->status == 1 ? 0 : 1;
        if($record) {
            if(!empty($user_id) && $record->user_id != $user_id) { // not the owner of the ad
                return array('success' => false, 'message' => lang('msg_permission_error'));
            } else if($new_status == 1 && empty($record->max_budget) && $record->wallets > 0) {
                // the max_budget is exhausted
                return array('success' => false, 'message' => lang('ad_budget_exhausted'));
            } else if($new_status == 1 && $record->wallets <= 0) {
                $this->lang->load('wallets/wallets');
                // add amount to your wallet
                return array('success' => false, 'message' => lang('wallet_low_balance'));
            } else {
                return array('success' => true, 'status' => $new_status);
            }
        }
        return array('success' => false, 'message' => lang('msg_error'));
    }
    /***************************************************************************
     * END OF LOCATION AND INTEREST SPECIFIC BANNER SLIDERS
     ***************************************************************************/
    /***************************************************************************
     * DISPLAY PAGE SPECIFIC POPUP
     ***************************************************************************/
    
    /**
     * Get Page specific popups
     * @param string $params
     * @param string $module - to clear Cache
     * @param number $row_id - to clear cache
     */
    public function get_popups($table = 'offer_popups', $where = null, $params = null, $module = '', $row_id = 0) {
        $params_cache = str_replace('-', '_', $this->slug->create_slug($params . '_' . $where));
        $cacheID = 'popups_' . $table . '_' . $params_cache;
        if(!$result = $this->mb_pages_cache->load($cacheID)) {
            $popup = '';
            $where = (!empty($where) ? $where . ' AND ' : '') . 'status = 1 AND deleted = 0';
            $records = $this->db->where($where)->order_by('priority')->limit(1)->get($table)->result();
            if($records) {
                $today_date = date('Y-m-d H:i:s');
                foreach($records as $record) {
                    if(empty($popup) && (empty($record->start_date) || $today_date >= $record->start_date)
                        && (empty($record->end_date) || $today_date <= $record->end_date)
                        && (empty($record->params) || $params == $record->params)) {
                            $popup = array(
                                'id' => $record->id,
                                'cache_id' => $params_cache . $record->id, // mobile app local storage id
                                'link_url' => $record->link_url,
                                'is_external' => $this->validate_url($record->link_url) ? 1 : 0,
                                'frequency' => $record->frequency,
                                'image_url' => gc_image_url($record->image, false, true, array('module' => 'offer_popups')),
                                'timeout' => $record->timeout
                            );
                        }
                }
            }
            $result = array('popup' => $popup);
            $this->mb_pages_cache->save($result, $cacheID, array($table, 'popups_' . $module . '_' . $row_id));
        }
        return $result['popup'];
    }
    
    /**
     * Validate URL
     * @param string $url
     * @return boolean
     */
    private function validate_url($url) {
        $path = parse_url($url, PHP_URL_PATH);
        $encoded_path = array_map('urlencode', explode('/', $path ?? ''));
        $url = str_replace($path, implode('/', $encoded_path), $url);
        
        return filter_var($url, FILTER_VALIDATE_URL) ? true : false;
    }    
    /***************************************************************************
     * END OF POPUPS
     ***************************************************************************/  
    
    /**
     * Display All Push Notifications
     */
    public function get_notifications($count, $where, $sort_by, $limit = 0, $offset = 0) {
        $sql = "SELECT";
        if($count == 0) {
            $sql .= " MN.id, MD.uuid, MD.platform, MN.device_id, MN.title, MN.message, MN.type, MN.scheduled_on, MN.date_sent, MN.date_sent, MN.status";
        } else {
            $sql .= " COUNT(MN.id) AS total_count";
        }
        $sql .= " FROM ". $this->db->dbprefix ."mobile_notifications MN";
        $sql .= " INNER JOIN " . $this->db->dbprefix . "mobile_devices MD ON MD.id = MN.device_id";
        if(!empty($where)) {
            $sql .= " WHERE " . $where;
        }
        if($count == 0) {
            $sql .= " ORDER BY " . $sort_by;
            if ($limit != 0) {
                $sql .= " LIMIT " . $offset . ", " . $limit;
            } else {
                $sql .= " LIMIT $limit";
            }
            return $this->db->query ( $sql )->result();
        } else {
            $result = $this->db->query ( $sql )->row();
            return $result ? $result->total_count : 0;
        }
    }
}