<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/*=======================================================================*\
|| ##################################################################### ||
|| # GoClixy - Business Directory Script							   # ||
|| # Copyright (C) 2014 GoClixy. All Rights Reserved.				   # ||
|| # ----------------------------------------------------------------  # ||
|| # This file is owned and copyrighted by GoClixy.com. Your license   # ||
|| # confers  no  title  or ownership in the Script and is not a sale  # ||
|| # of any rights to the Software. This file may not be redistributed # ||
|| # in whole or significant part.									   # ||
|| # --------------- COPYRIGHTED COMMERCIAL SOFTWARE ---------------   # ||
|| # https://www.goclixy.com | https://www.goclixy.com/license.html    # ||
|| ##################################################################### ||
\*=======================================================================*/
class Orders_model extends BF_Model {

    protected $table_name	= "orders";
    protected $table_invoices = 'order_invoices';
    protected $table_items = 'order_items';
    protected $table_shipments = 'order_shipments';
    protected $table_commissions = 'commissions';
    protected $table_variants = 'product_variants';
    protected $table_product_units = 'product_variants';
    protected $table_coupons = 'order_coupons';
    protected $table_customers = 'order_customers';
    
	protected $key			= "id";
	protected $soft_deletes	= true;
	protected $date_format	= "datetime";

	protected $log_user 	= FALSE;

	protected $set_created	= true;
	protected $set_modified = true;
	protected $created_field = "created_on";
	protected $modified_field = "modified_on";

	/*
		Customize the operations of the model without recreating the insert, update,
		etc methods by adding the method names to act as callbacks here.
	 */
	protected $before_insert 	= array();
	protected $after_insert 	= array();
	protected $before_update 	= array();
	protected $after_update 	= array();
	protected $before_find 		= array();
	protected $after_find 		= array();
	protected $before_delete 	= array();
	protected $after_delete 	= array();

	/*
		For performance reasons, you may require your model to NOT return the
		id of the last inserted row as it is a bit of a slow method. This is
		primarily helpful when running big loops over data.
	 */
	protected $return_insert_id 	= TRUE;

	// The default type of element data is returned as.
	protected $return_type 			= "object";

	// Items that are always removed from data arrays prior to
	// any inserts or updates.
	protected $protected_attributes = array();

	/*
		You may need to move certain rules (like required) into the
		$insert_validation_rules array and out of the standard validation array.
		That way it is only required during inserts, not updates which may only
		be updating a portion of the data.
	 */
	protected $validation_rules 		= array();
	protected $insert_validation_rules 	= array();
	protected $skip_validation 			= TRUE;
	
	/**
	 * Constructor
	 *
	 * @return void
	 */
	public function __construct() {
	    parent::__construct();
	    $this->lang->load('orders/orders', $this->session->userdata('language'));
	}
	
	/**
	 * Orders Fields
	 *
	 * @return array
	 */
	public function orders() {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'order_id',
	            'label' => 'label_order_id',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim|max_length[20]',
	        ),
	        array(
	            'type' => 'selectize',
	            'name' => 'user_id',
	            'label' => 'bf_user',
	            'placeholder' => 'placeholder_selectize',
	            'frontend' => false,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => true,
	            'options' => 'load:users;module:users;max_items:1',
	            'options_detailed' => 1,
	            'rules' => 'trim|required',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'user_email',
	            'label' => 'label_email',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim|max_length[100]',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'order_subtotal',
	            'label' => 'label_order_subtotal',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'order_tax',
	            'label' => 'label_order_tax',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'order_discount',
	            'label' => 'label_order_discount',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'order_total',
	            'label' => 'label_order_total',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'site_currency_code',
	            'label' => 'label_currency_code',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'currency_code',
	            'label' => 'label_currency_code',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'currency_symbol',
	            'label' => 'label_currency_symbol',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim|max_length[40]',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'currency_value',
	            'label' => 'label_currency_value',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'payment_method',
	            'label' => 'label_payment_method',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim|max_length[200]',
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'payment_status',
	            'label' => 'label_payment_status',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'wallets',
	            'label' => 'label_wallets',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'wallets_refund',
	            'label' => 'label_wallets_refund',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'transaction_id',
	            'label' => 'label_transaction_id',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim|max_length[100]',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'payer_id',
	            'label' => 'label_payer_id',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim|max_length[100]',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'payment_details',
	            'label' => 'label_payment_details',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'comments',
	            'label' => 'label_comments',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'notes',
	            'label' => 'label_notes',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'is_invoiced',
	            'label' => 'label_is_invoiced',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'date_paid',
	            'label' => 'label_date_paid',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'next_invoice_date',
	            'label' => 'label_next_invoice_date',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'cronrun_on',
	            'label' => 'label_cronrun_on',
	            'frontend' => false,
	            'admin_only' => false,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'status_id',
	            'label' => 'label_status',
	            'frontend' => false,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'options' => 'name:order_statuses;value:database',
	            'options_detailed' => 1,
	            'options_lang' => 1,
	            'class' => 3,
	            'rules' => 'trim|is_numeric|required',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_on',
	            'label' => 'created_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'created_ip',
	            'label' => 'created_ip',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim|max_length[50]',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'created_by',
	            'label' => 'created_by',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim|max_length[50]',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_on',
	            'label' => 'modified_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'modified_ip',
	            'label' => 'modified_ip',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim|max_length[50]',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'modified_by',
	            'label' => 'modified_by',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim|max_length[50]',
	        )
	    );
	    return $fields;
	}
	
	/**
	 * Invoices Fields
	 *
	 * @return array
	 */
	public function invoices() {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'prefix',
	            'label' => 'label_prefix',
	            'frontend' => false,
	            'admin_only' => false,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'order_id',
	            'label' => 'label_order_id',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim|max_length[20]',
	        ),
	        array(
	            'type' => 'selectize',
	            'name' => 'user_id',
	            'label' => 'bf_user',
	            'placeholder' => 'placeholder_selectize',
	            'frontend' => false,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => true,
	            'options' => 'load:users;module:users;max_items:1',
	            'options_detailed' => 1,
	            'rules' => 'trim|required',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'date_paid',
	            'label' => 'label_date_paid',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'status_id',
	            'label' => 'label_status',
	            'frontend' => false,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'options' => 'name:order_statuses;value:database',
	            'options_detailed' => 1,
	            'options_lang' => 1,
	            'class' => 3,
	            'rules' => 'trim|is_numeric|required',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_on',
	            'label' => 'created_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_on',
	            'label' => 'modified_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        
	    );
	    return $fields;
	}
	
	/**
	 * Order Items Fields
	 *
	 * @return array
	 */
	public function items($role = 'admin') {
	    $module_orders = array();
	    $order_modules = array('products', 'packages', 'pyg', 'ads', 'leads', 'wallets', 'bookings', 'claims', 'commission_levels');
	    foreach($order_modules as $order_module) {
	        $module_orders[] = 'name:' . $order_module . ';value:' . gc_module_id($order_module);
	    }
	    $module_orders = implode('|', $module_orders);
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'order_id',
	            'label' => 'label_order_id',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'class' => 3,
	            'rules' => 'trim|max_length[20]',
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'subscription_type',
	            'label' => 'label_subscription_type',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'name:placeholder_select;value:0|name:new;value:1|name:status_upgraded;value:2|name:status_renewed;value:3',
	            'options_detailed' => 1,
	            'options_lang' => 1,
	            'class' => 2,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'ru_order_id',
	            'label' => 'order_ru_id',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'class' => 3,
	            'rules' => 'trim|max_length[20]',
	        ),
	        array(
	            'type' => 'selectize',
	            'name' => 'seller_id',
	            'label' => 'label_seller',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'load:users;module:users;max_items:1',
	            'options_detailed' => 1,
	            'value' => -1,
	            'class' => 4,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'module_id',
	            'label' => 'label_module',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => $module_orders,
	            'options_detailed' => 1,
	            'options_lang' => 1,
	            'class' => 2,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'row_id',
	            'label' => 'order_row_id',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'class' => 2,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'variant_id',
	            'label' => 'singular_product_variants',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'class' => 2,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'selectize',
	            'name' => 'relation_item_id',
	            'label' => 'order_relation_product',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'load:products;module:products;max_items:1',
	            'options_detailed' => 1,
	            'value' => -1,
	            'rules' => 'trim|is_numeric',
	        ),
	        /*
	         array(
	         'type' => 'text',
	         'name' => 'item',
	         'label' => 'label_item',
	         'frontend' => true,
	         'admin_only' => true,
	         'frontend_index' => true,
	         'admin_index' => true,
	         'rules' => 'trim',
	         ),*/
	        array(
	            'type' => 'text',
	            'name' => 'price',
	            'label' => 'label_price',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'class' => 2,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'offer_price',
	            'label' => 'label_offer_price',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'class' => 2,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'qty',
	            'label' => 'label_qty',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'class' => 2,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'tax_percent',
	            'label' => 'label_tax_percent',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'class' => 2,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'tax_inclusive',
	            'label' => 'label_tax_inclusive',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'total',
	            'label' => 'label_total',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'class' => 2,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'subtotal',
	            'label' => 'label_subtotal',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'class' => 2,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'tax',
	            'label' => 'label_tax',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'class' => 2,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'discount',
	            'label' => 'label_discount',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'class' => 2,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'discount_value',
	            'label' => 'label_discount_value',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'class' => 2,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'commission',
	            'label' => 'label_commission',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'class' => 2,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'refund_duration',
	            'label' => 'label_refund_duration',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'class' => 2,
	            'rules' => 'trim',
	        ),/*
	        array(
	        'type' => 'checkbox',
	        'name' => 'shipment_status',
	        'label' => 'label_shipment_status',
	        'frontend' => true,
	        'admin_only' => true,
	        'frontend_index' => true,
	        'admin_index' => true,
	        'rules' => 'trim',
	        ),*/
	        array(
	            'type' => 'text',
	            'name' => 'expires_on',
	            'label' => 'label_expires_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'class' => 3,
	            'rules' => 'trim|valid_datetime',
	        ),
	        array(
	            'type' => 'textarea',
	            'name' => 'reminders',
	            'label' => 'label_reminders',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'textarea',
	            'name' => 'params',
	            'label' => 'label_params',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),/*
	        array(
	        'type' => 'checkbox',
	        'name' => 'status',
	        'label' => 'label_status',
	        'frontend' => true,
	        'admin_only' => true,
	        'frontend_index' => true,
	        'admin_index' => true,
	        'rules' => 'trim',
	        ),*/
	        
	    );
	    return $fields;
	}
	
	/**
	 * Order Cancellations Fields
	 *
	 * @return array
	 */
	public function cancellations($role = 'admin') {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'order_id',
	            'label' => 'label_order_id',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim|max_length[20]',
	        ),
	        array(
	            'type' => 'selectize',
	            'name' => 'user_id',
	            'label' => 'bf_user',
	            'placeholder' => 'placeholder_selectize',
	            'frontend' => false,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => true,
	            'options' => 'load:users;module:users;max_items:1',
	            'options_detailed' => 1,
	            'rules' => 'trim|required',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'seller_id',
	            'label' => 'label_seller_id',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'module_id',
	            'label' => 'label_module_id',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'row_id',
	            'label' => 'label_row_id',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'reason_id',
	            'label' => 'label_reason',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'comments',
	            'label' => 'label_comments',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'status',
	            'label' => 'label_status',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_on',
	            'label' => 'created_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_ip',
	            'label' => 'created_ip',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_by',
	            'label' => 'created_by',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_on',
	            'label' => 'modified_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_ip',
	            'label' => 'modified_ip',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_by',
	            'label' => 'modified_by',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        
	    );
	    return $fields;
	}
	
	/**
	 * Order Cancellation Reasons Fields
	 *
	 * @return array
	 */
	public function cancellation_reasons() {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'name',
	            'label' => 'label_name',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'edit' => 1,
	            'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
	            'rules' => 'trim|required|max_length[150]'
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'name_lang',
	            'label' => 'name_lang',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'colorpicker',
	            'name' => 'color',
	            'label' => 'label_color',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'class' => 2,
	            'rules' => 'trim|max_length[255]',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'priority',
	            'label' => 'label_priority',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'value' => 0,
	            'class' => 2,
	            'rules' => 'trim|required|is_numeric',
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'status',
	            'label' => 'label_status',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        )
	    );
	    return $fields;
	}
	
	/**
	 * Email Template Fields
	 *
	 * @return array
	 */
	public function email_templates($role = 'admin') {
	    $fields = array(
	        array(
	            'type' => 'id',
	            'name' => 'id',
	            'frontend_index' => true,
	            'admin_index' => true
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'module_table',
	            'label' => 'label_module',
	            'frontend' => false,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'name:gc_menu_orders;value:orders|name:order_items;value:order_items' . (GC_PRODUCT_ID == 2 ? '' : '|name:gc_menu_ecommerce;value:ecommerce'),
	            'options_detailed' => 1,
	            'options_lang' => 1,
	            'class' => 3,
	            'rules' => 'trim|unique_email_template[order]',
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'user_type',
	            'label' => 'bf_user',
	            'frontend' => false,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'name:placeholder_select;value:|name:us_admin;value:admin|name:bf_user;value:user',
	            'options_detailed' => 1,
	            'options_lang' => 1,
	            'class' => 3,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'dropdown',
	            'name' => 'action_type',
	            'label' => 'label_action',
	            'frontend' => false,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'options' => 'name:placeholder_select;value:|name:add;value:add|name:edit;value:edit|name:delete;value:delete|name:em_status_update;value:status_update|name:order_reminders;value:reminder|name:order_invoices;value:invoice',
	            'options_detailed' => 1,
	            'options_lang' => 1,
	            'class' => 3,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'name',
	            'label' => 'label_name',
	            'placeholder' => 'placeholder_name',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'field_extra' => 'index_edit_admin:1;index_edit_frontend:1',
	            'rules' => 'required|trim|max_length[50]'
	        ),
	        array(
	            'type' => 'text',
	            'name' => 'subject',
	            'label' => 'label_subject',
	            'placeholder' => 'placeholder_subject',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'required|trim|max_length[255]'
	        ),
	        array(
	            'type' => 'textarea',
	            'name' => 'message',
	            'label' => 'label_message',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'custom_rules' => 'editor',
	            'field_class' => 'editor',
	            'class' => 9,
	            'rules' => 'trim|required',
	        ),
	        array(
	            'type' => 'textarea',
	            'name' => 'sms_text',
	            'label' => 'sms_text',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'class' => 9,
	            'field_extra' => 'rows:3',
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'textarea',
	            'name' => 'params',
	            'label' => 'em_params',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'class' => 9,
	            'tooltip' => lang('em_params_help'),
	            'field_extra' => 'rows:3',
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'checkbox',
	            'name' => 'status',
	            'label' => 'label_status',
	            'frontend' => false,
	            'admin_only' => false,
	            'frontend_index' => true,
	            'admin_index' => true,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'created_on',
	            'label' => 'created_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        ),
	        array(
	            'type' => 'none',
	            'name' => 'modified_on',
	            'label' => 'modified_on',
	            'frontend' => true,
	            'admin_only' => true,
	            'frontend_index' => false,
	            'admin_index' => false,
	            'rules' => 'trim',
	        )
	    );
	    return $fields;
	}
	
	/**
	 * Get information for index page
	 */
	public function get_order_detail($count, $offset, $limit, $where, $order_by) {
		if($count == 0) {
			$sql = "SELECT O.id, O.order_id, O.user_id, O.order_type, INV.prefix as invoice_prefix, O.currency_code, O.currency_value,";
			$sql .= " INV.id as invoice_id, O.created_on, O.expires_on, O.currency_symbol, O.order_total, O.payment_method,";
			$sql .= " U.name as display_name, O.user_email, O.status";
		} else {
			$sql = "SELECT COUNT(O.id) AS TotalCount";
		}
		/*$sql .= " FROM (SELECT id, order_id, created_on, order_total, payment_method, currency_code, user_id, user_email, status FROM " . $this->db->dbprefix . "orders";
		$sql .= ") O";*/
		if($count == 1) {
		  $sql .= " FROM " . $this->db->dbprefix . "orders O";		  
		  if(!empty($where) && (strpos($where,'SUBSTRING') !== false)) {
		      $sql .= " WHERE " . $where;
		  } else if(!empty($where)) {
		      $sql .= " WHERE " . $where;
		  }
		  $result = $this->db->query ( $sql )->row();
		  return $result ? $result->TotalCount : 0;
		}
		if($count == 0) {
		    $sql .= " FROM (SELECT * FROM " . $this->db->dbprefix . "orders O";
		    if(!empty($where) && (strpos($where,'SUBSTRING') !== false)) {
		        $sql .= " WHERE " . $where;
		    } else if(!empty($where)) {
		        $sql .= " WHERE " . $where;
		    }
		    $sql .= " GROUP BY O.id ORDER BY " . $order_by ." LIMIT " . $offset . ", " . $limit;
		    $sql .= " ) O";
    		$sql .= " LEFT JOIN `" . $this->db->dbprefix . "order_invoices` as INV ON `INV`.`order_id` = O.order_id";
    		$sql .= " LEFT JOIN `" . $this->db->dbprefix . "order_customers` as U ON `U`.`order_id` = O.order_id";    		
    		$result = $this->db->query ( $sql )->result_array ();
    		return $result;
		}
	}
	
	/**
	 * Update order information
	 * @param string $order_id
	 * @param boolean $data
	 */
	public function update_order($order_id, $data) {
		//SINGLE QUERY FOR MULTIPLE ITEMS
		$where = "id IN (" . implode(",", $order_id) . ")";
		$this->db->where($where);
		return $this->db->update('orders', $data);
	}
	
	/**
	 * Performs a standard delete, but also allows for purging of a record.
	 *
	 * @access public
	 *
	 * @param int  $id    An INT with the record ID to delete.
	 * @param bool $purge If FALSE, will perform a soft-delete. If TRUE, will permanently delete the record.
	 *
	 * @return bool TRUE/FALSE
	 */
	public function delete_order($order_id = 0, $purge = FALSE) {
		if ($purge === TRUE) {
			// temporarily set the soft_deletes to TRUE.
			$this->soft_deletes = FALSE;
		}
	
		//SINGLE QUERY FOR MULTIPLE ITEMS
		$where = "id IN (" . implode(",", $order_id) . ")";
		return $this->orders_model->delete_where($where);
	}//end delete()
	
	/**
	 * Invoice Email and PDF
	 * @param string $order_id
	 * @return array
	 */
	public function email_pdf_order($order_id) {
		$sql = "SELECT O.id, O.order_id, INV.prefix as invoice_prefix, INV.id as invoice_id, INV.date_paid, INV.status as invoice_status, INV.created_on AS invoice_created_on, O.created_on, O.order_subtotal, O.order_discount, O.order_tax, O.order_total, O.payment_method, O.transaction_id, O.payer_id, O.payment_details, O.user_id, O.user_email, C.symbol, PG.display_name as payment_method, O.comments, O.notes, O.status,";
		$sql .= " O.currency_code, O.currency_value, O.item, OC.name, OC.phone_number, OC.address, OC.postcode, OC.city_id, OCO.coupon_code, OCO.value, OCO.value_type, O.wallets, O.wallets_refund, O.module_id, O.item_price, O.qty";
		$sql .= " FROM (SELECT id, order_id, date_paid, created_on, order_subtotal, order_discount, order_tax, order_total, payment_method, wallets, wallets_refund, transaction_id, payer_id, payment_details, currency_code, currency_value, user_id, user_email, comments, notes, status_id, module_id, row_id FROM " . $this->db->dbprefix . "orders";
		$sql .= " WHERE order_id = '".$order_id."') O";
		$sql .= " LEFT JOIN `" . $this->db->dbprefix . "payment_gateways` as PG ON `PG`.`name` = O.payment_method";
		$sql .= " LEFT JOIN `" . $this->db->dbprefix . "currencies` as C ON `C`.`code` = O.currency_code";
		$sql .= " LEFT JOIN `" . $this->db->dbprefix . "order_invoices` as INV ON `INV`.`order_id` = O.order_id";
		$sql .= " LEFT JOIN `" . $this->db->dbprefix . "order_customers` as OC ON `OC`.`order_id` = O.order_id";
		$sql .= " LEFT JOIN `" . $this->db->dbprefix . "order_coupons` as OCO ON `OCO`.`order_id` = O.order_id";
		return (array) $this->db->query ( $sql )->row();
		/*foreach($query->result_array () as $row) {
			return $row;
		}*/		
	}
	
	/**
	 * Populate location information using AJAX
	 * @param string $loadType for state or city
	 * @param string/int $loadId
	 */
	public function getData($loadType,$loadId) {
		$this->load->model('locations/states_model');
		$this->load->model('locations/cities_model');
		if($loadType=="state"){
			return $this->states_model->select('id, name')->where('country_iso', $loadId)->order_by('name', 'asc')->find_all();
				
		} elseif(($loadType=="city")) {
			return $this->cities_model->select('id, name')->where('state_id', $loadId)->order_by('name', 'asc')->find_all();
		}
	}
	
	/**
	 * Common function to get user order details
	 * Check whether same package is purchased by the same user or not
	 * if purchased, check whether it is for multisale
	 * if yes allow user to buy that package again
	 * @param integer $user_id
	 * @param integer $package_id
	 */
	public function get_user_orders($count, $user_id, $extra = array()) {
	    $package_id = !empty($extra['package_id']) ? $extra['package_id'] : 0;
	    $order_by = !empty($extra['order_by']) ? $extra['order_by'] : 'O.id DESC';
	    $return = !empty($extra['return']) ? $extra['return'] : 'row';
	    if($count == 1) {
		   $sql = "SELECT COUNT(O.order_id) AS count";
        } else {
            if(!empty($extra['select'])) {
                $sql = "SELECT " . $extra['select'];
            } else {
                $sql = "SELECT O.order_id, OT.expires_on, P.*, P.id AS package_id";                
            }
        }
        $sql .= " FROM " . $this->db->dbprefix . "orders O";
        $sql .= " INNER JOIN " . $this->db->dbprefix . "order_items OT ON OT.order_id = O.order_id";
        $sql .= " INNER JOIN " . $this->db->dbprefix . "packages P ON P.id = OT.row_id";
        $sql .= " WHERE O.deleted = 0";
        if(empty($extra['where'])) {
		    $sql .= " AND O.status_id = 2 AND O.user_id = $user_id AND OT.module_id = 20";
		    if(!empty($package_id)) {
		        $sql .= " AND OT.row_id = $package_id";
		    }
        } else {
            $sql .= " AND " . $extra['where'];
        }
        $sql .= " ORDER BY " . $order_by;
		$query = $this->db->query ($sql);
		switch($return) {
		    case 'result':
		        return $query->result();
		    case 'result_array':
		        return $query->result_array();
		    default:
		        return $query->row();
		}
	}
	
	/**
	 * Listings detail page, get current packages to display special fields information to the user
	 * This function must be called in the auth library to avoid unnecessary lookup on the view and detail page
	 * @param number $user_id
	 * @return NULL[]
	 */
	public function get_active_packages($user_id) {
	    $active_packages = array();
	    $where = "status_id = 2 AND user_id = $user_id AND module_id = " . gc_module_id('packages') . " AND expires_on IS NULL OR expires_on >= '" . date('Y-m-d H:i:s') . "'";
	    $order_items = $this->db->where($where)->select('id, order_id, row_id')->get('order_items')->result();
	    if($order_items) {
	        foreach($order_items as $order_item) {
	            $active_packages[$order_item->row_id] = $order_item->row_id;
	        }
	    }
	    return $active_packages;
	}
	
	/**
	 * Display all orders to member on member area
	 * @param integer $user_id
	 */
	public function retrieve_orders($user_id, $offset = 0, $limit = 25) {
		$sql = "SELECT O.*, S.color AS status_color, S.name AS status, S.name_lang AS status_lang";
		$sql .= " FROM " . $this->db->dbprefix . "orders O";
		$sql .= " INNER JOIN " . $this->db->dbprefix . "order_statuses S ON S.id = O.status_id";
		$sql .= " WHERE O.user_id = $user_id AND O.deleted = 0 ORDER BY O.id DESC";
		$sql .= " LIMIT " . $offset . ", " . $limit;
		$query = $this->db->query ( $sql );
		return $query->result_array ();
	}
	
	/**
	 * Display Country Wise Reports to the Admin 
	 * @param string $date_from
	 * @param string $date_to
	 * @param int $status
	 * @param string $country
	 * @param string $state
	 * @param string $city
	 * @param int $count - FOR PAGINATION
	 * @return OBJECT
	 */
	public function reports($date_from, $date_to, $status, $country, $state, $city, $count = 0) {
		if($count == 0) {
			$sql = "SELECT COUNT(O.order_id) AS TotalSales, COUNT(DISTINCT(OC.user_id)) AS TotalUsers, SUM(O.order_discount) AS TotalDiscount, SUM(O.order_tax) AS TotalTax, SUM(O.order_subtotal) AS SubTotal, SUM(O.order_total) AS GrossAmount,";
			if(!empty($city)) {
				$sql .= " CT.name AS Location";
			} else if(!empty($state)) {
				$sql .= " S.name AS Location";
			} else {
				$sql .= " C.name AS Location";
			}
		} else {
			$sql = "SELECT COUNT(O.order_id) AS counts";
		}
		$sql .= " FROM " . $this->db->dbprefix . "orders O";
		$sql .= " INNER JOIN " . $this->db->dbprefix . "order_customers OC ON OC.order_id = O.order_id";
		$sql .= " INNER JOIN " . $this->db->dbprefix . "cities CT ON CT.id = OC.city_id";
		$sql .= " INNER JOIN " . $this->db->dbprefix . "states S ON S.id = CT.state_id";
		$sql .= " INNER JOIN " . $this->db->dbprefix . "countries C ON C.iso = S.country_iso";
		$sql .= " WHERE O.status = $status AND (DATE(O.created_on) BETWEEN '$date_from' AND '$date_to')";
		if(!empty($city)) {
			$sql .= " AND CT.id = $city";
			$sql .= " GROUP BY OC.city_id";
		} else if(!empty($state)) {
			$sql .= " AND S.id = $state";
			$sql .= " GROUP BY S.id";
		} else if(!empty($country)) {
			$sql .= " AND C.iso = '$country'";
			$sql .= " GROUP BY C.iso";
		} else {
			$sql .= " GROUP BY C.iso";
		}
		if($count == 0) {
			$result = $this->db->query ( $sql )->result();
			return $result ? $result : null;
		} else {
			$result = $this->db->query ( $sql )->row();
			return $result ? $result->counts : 0;
		}		
	}
	
	/**
	 * Allow the user to checkout without registration
	 * Save User information 
	 */
	public function save_user() {
	    $this->load->model('users/user_model');
		$email = $this->input->post('email');
		$password = $this->input->post('password');
		// check user existence
		$user = $this->db->get_where('users', array('email' => $email, 'deleted' => 0))->row();
		if($user) {
			$user_id = $user->id;			
			if($user->active == 0) {
			    // send activation code
			    $this->user_model->send_activation_code($user);
			}			
			// setup a login session temporarily
			$this->auth->login_session($user_id, $user);
		} else {
			$data = array();
			$data['display_name'] = $this->input->post('name');	
			$data['username'] = $email;	
			$data['email'] = $email;
			$data['slug'] = $this->slug->create_slug($email);	
			$data['address'] = $this->input->post ( 'address' );
			$data['postcode'] = $this->input->post ( 'postcode' );
			$data['phone_code'] = (int) $this->input->post ( 'phone_code' );
			$data['phone_number'] = $this->input->post ( 'phone_number' );
			$data['password'] = $password;
			$this->load->model('locations/cities_model');
			if(settings_item('lst.google_places_api') == 0) {
			    $data['city_id'] = $this->input->post('city_id') ? $this->input->post('city_id') : 0;
			    $location = $this->cities_model->get_city_info($data['city_id']);
			    if($location) {
			        $data['country_iso'] = $location->country_iso;
			        $data['state_id'] = $location->state_id;
			        $data['city_id'] = $data['city_id'];
			        $data['city'] = $location->city;
			        $data['state'] = $location->state;
			        $data['country'] = $location->country;
			    }
			} else {
			    $city_name = $this->input->post('city');
			    $state_abbrev = $this->input->post('state_abbrev');
			    $state_name = $this->input->post('state_name');
			    $country_iso = $this->input->post('country_iso');
			    $country_name = $this->input->post('country_name');
			    $location = $this->cities_model->getCityStateCountry($city_name, $state_abbrev, $state_name, $country_iso, $country_name);
			    if($location && $location['city_id'] != 0) {
			        $data['country_iso'] = $location['country_iso'];
			        $data['state_id'] = $location['state_id'];
			        $data['city_id'] = $location['city_id'];;
			        $data['city'] = $city_name;
			        $data['state'] = $state_name;
			        $data['country'] = $country_name;
			    }
			}
			$data['last_ip'] = $this->input->ip_address();	
			$user_id = $this->user_model->insert($data);
			if($user_id > 0) {
			    // send activation email
			    $user = $this->user_model->get_user($user_id);
			    
			    // setup a login session temporarily
			    $this->auth->login_session($user_id, $user);
			    
			    $this->user_model->send_activation_code($user);
			    // Notify Admin/User on Registration
			    $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'user_email_templates', 'module_table' => 'users', 'action_type' => 'add', 'id' => $user_id, 'record' => $user));
			}
		}		
		return $user_id;
	}
	
	/**
	 * Get User Information
	 * @param int $user_id
	 * @return boolean|StdClass
	 */
	public function get_user($user_id) {
		$user_info = $this->db->get_where('users', array('id' => (int) $user_id))->row();
		if(!$user_info) {
			return false;
		}
		return $user_info;
	}
	
	/**
	 * Calculate expiry date
	 * @param array $package
	 * @param string $renewed_order_id
	 */
	public function calculate_expires_on($package, $subscription_type, $renewed_order_id, $renew_module = 'packages') {	
		$expires_on = NULL; // For claim set non expiry to avoid reminders
		$expired_on = new DateTime(date("Y-m-d"));
		if(!empty($package->forever) && $package->forever == 1) {
			$expires_on = NULL;
			return $expires_on;
		} 
		if(!empty($package->fixed_date)) {
			$expires_on = $package->fixed_date;
			return $expires_on;
		}
		
		if(!empty($package->duration) && empty($package->fixed_date)) {
			$package_duration = $package->duration;
			if($subscription_type == 3) {
				/**
				 * Renew before order expiry date
				 * Calculate days between today and expiry date
				 * Add them to package duration before expires_on calculation 
				 * Calculate expiry from package duration
				 */
			    $get_order_detail = $this->db->select('expires_on')->where(array('order_id' => $renewed_order_id, 'module_id' => gc_module_id($renew_module)))->get('order_items')->row();
				$today = date('Y-m-d');
				if($get_order_detail && !empty($get_order_detail->expires_on)) {
    				$package_expiry_date = $get_order_detail->expires_on;
    				if($package_expiry_date > $today) {
    					$today_date = date_create($today);
    					$expiry_date = date_create($package_expiry_date);
    					$interval = date_diff($today_date, $expiry_date);
    					$interval = $interval->days;
    					$package_duration = $package_duration + $interval;
    				}
				}
			}
			$expired_on->modify($package_duration . " day");
			$expires_on = $expired_on->format("Y-m-d");
			return $expires_on;
		}
		return $expires_on;
	}
	
	/**
	 * Save customer information
	 * @param string $order_id
	 */
	public function save_order_customers ($order_id, $user_id, $email, $type='insert', $mobile = false) {
		$data = array ();
		$data ['user_id'] = $user_id;
		$data ['name'] = $this->input->post ( 'full_name' );
		$data ['email'] = $email;
		$data ['phone_code'] = (int) $this->input->post ( 'phone_code' );
		$data ['phone_number'] = $this->input->post ( 'phone_number' );
		$data ['address'] = $this->input->post('address');
		$data ['postcode'] = $this->input->post ( 'postcode' );
		if($mobile || settings_item('lst.google_places_api') == 0) {
			$data ['city_id'] = $this->input->post('city_id') ? $this->input->post('city_id') : 0;
		} else {
			$city_name = $this->input->post('city');
			$state_abbrev = $this->input->post('state_abbrev');
			$state_name = $this->input->post('state_name');
			$country_iso = $this->input->post('country_iso');
			$country_name = $this->input->post('country_name');
	        $this->load->model('locations/cities_model');
			$location = $this->cities_model->getCityStateCountry($city_name, $state_abbrev, $state_name, $country_iso, $country_name);
			if($location && $location['city_id'] != 0) {
				$data['city_id'] = $location['city_id'];
			} else {
				return false;
			}
		}
		if ($type == 'insert') {
			$data ['order_id'] = $order_id;
			$this->db->insert('order_customers', $data );
		} else if ($type == 'update') {
			$this->db->where('order_id', $order_id);
			$this->db->update('order_customers', $data );
		}
	}
	
	/**
	 * Save order information when price is zero
	 * @param string $order_id
	 * @param string $expires_on
	 * @param string $user_email
	 */
	public function save_order_zero_price($order_data) {
		$data = array ();
		$data ['order_id'] = $order_data['order_id'];
		$data ['order_type'] = $order_data['order_type'];
		$data ['subscription_type'] = $order_data['subscription_type'];
		if(($order_data['subscription_type'] == 3) || ($order_data['subscription_type'] == 2)) {
			$data ['renew_upgrade_order_id'] = !empty($order_data['upgrade_order_id']) ? $order_data['upgrade_order_id']: $order_data['renewed_order_id'];
		}
		$data ['user_id'] = $order_data['user_id'];
		$data ['user_email'] = $order_data['user_email'];
		$data ['item_price'] = 0;
		$data ['order_subtotal'] = 0;
		$data ['order_total'] = 0;
		$data ['currency_code'] = settings_item('site.currency_code');
		$data ['currency_symbol'] = settings_item('site.currency_symbol');
		$data ['currency_value'] = 1.00;
		
		$data ['item'] = $order_data['item'];
		$data ['package_id'] = $order_data ['package_id'];
		$data ['module_id'] = $order_data ['module_id'];
		$data ['row_id'] = $order_data ['row_id'];
		$data ['qty'] = $order_data ['qty'];
		
		$data ['expires_on'] = !empty($order_data['expires_on']) ? $order_data['expires_on'] : NULL;
		$data ['ip_address'] = $this->input->ip_address();
		$data ['status'] = $order_data['status'];
		$this->orders_model->insert ( $data );
	}
	
	/**
	 * Generate Invoice
	 * @param string $order_id
	 * @param array $order
	 */
	public function generate_invoice($order_id, $order = array()) {
	    if(empty($order)) {
	        $order = $this->db->where('order_id', $order_id)->get($this->table_name)->row();
	    }
	    if(!empty($order)) {
	        $result = $this->save_invoices($order_id, $order);
	        if($result === true || $result > 0) {
	            return true;
	        }
	    }
	    return false;
	}

	/**
	 * Update Affiliate Coupon Hits
	 * @param int $coupon_id
	 */
	private function update_affiliate_coupon_counts_hits($coupon_id, $type = 'hits') {
		$affiliate_id = (int) $this->input->cookie('ref');
		if($affiliate_id > 0) {
			$affiliate = $this->db->select('id, commission')->get_where('affiliate_coupons', array('user_id' => $affiliate_id, 'coupon_id' => $coupon_id, 'status' => 1))->row();
			if($affiliate) {
				$this->db->where('id', $affiliate->id);
				if($type == 'hits') {
					$this->db->set('hits', 'hits + 1', FALSE);
				} else {
					$this->db->set('counts', 'counts + 1', FALSE);
				}
				$this->db->update('affiliate_coupons');
				return true;
			}
		}
		return false;
	}
	
	/**
	 * Order create page
	 * @param int $user_id
	 */
	public function get_customer_info($user_id) {
		$sql = "SELECT U.display_name AS name, U.address, U.postcode, U.phone_number, U.city_id, C.name AS city, S.name AS state, S.country_iso";
		$sql .= " FROM " . $this->db->dbprefix . "users U";
		$sql .= " LEFT JOIN " . $this->db->dbprefix . "cities C ON C.id = U.city_id";
		$sql .= " LEFT JOIN " . $this->db->dbprefix . "states S ON S.id = C.state_id";
		$sql .= " WHERE U.id = " . $user_id;
		return $this->db->query($sql)->row();
	}
	
	/**
	 * Apply Coupon - Price Calculation
	 * @params $user_id - Will be used on apply coupon code
	 * @params $code - Used in all cases, it can be null in case of quantity update
	 * @params $action - Required for quantity update, to find whether the request is by quantity update or apply coupon code
	 */
	public function order_update($user_id, $code, $action = null) {
	    $user = $this->db->where('user_id', $user_id)->get('order_checkouts')->row();
	    if(empty($user->params) || empty($code)) {
	        $message = array(
	            'success' => false,
	            'message' => lang('msg_valid_coupon')
	        );
	        return array('success' => false, 'message' => lang('msg_valid_coupon'));
	    }
	    $success = false;
	    $this->load->model('coupons/coupons_model');
	    $this->lang->load('coupons/coupons');
	    
	    $data = array(
	        'success' => false
	    );
	    $coupon_discounts = json_decode($user->params ?? '', true);
	    // Coupons are user specific, the user must login to use a Coupon code.
	    if(empty($coupon_discounts) || !empty($coupon_discounts['allow_coupon'])) {
	        $message = array(
	            'success' => $success,
	            'message' => lang('msg_coupon_user_specific')
	        );
	        return $message;
	    }
	    $package_id = !empty($coupon_discounts['package_id']) ? $coupon_discounts['package_id'] : 0;
	    $toggle_module_id = isset($coupon_discounts['module_id']) ? $coupon_discounts['module_id'] : 20;
	    $order_type = $coupon_discounts['order_type'];
	    $qty = $coupon_discounts['qty'];
	    $mobile = !empty($coupon_discounts['mobile']) ? true : false;
	    
	    // On Qty change, update the Order Subtotal, Total, Tax and Discount
	    // Do this only for PYG and Bookings module.
	    // Order will be updated if a coupon is not applied.
	    $order_cart_subtotal = $coupon_discounts['subtotal'];
	    $order_cart_total = $coupon_discounts['total'];
	    $order_cart_tax = $coupon_discounts['tax'];
	    if(($order_type == 4 || $order_type == 9) && !empty($qty)) {
	        // Qty will be processed on Apply Coupon
	        foreach($coupon_discounts['cart_items'] as $order_cart_key => $order_cart_item) {
	            $coupon_discounts['cart_items'][$order_cart_key]['subtotal'] = $order_cart_item['subtotal'] * $qty;
	        }
	        if(empty($code)) {
	            $order_cart_subtotal = $order_cart_subtotal * $qty;
	            if(!empty($coupon_discounts['tax_percent'])) {
	                $order_cart_tax = decimal_convert(($order_cart_subtotal * $coupon_discounts['tax_percent']) / 100);
	                $order_cart_total = ($order_cart_subtotal + $order_cart_tax);
	            } else {
	                $order_cart_total = $order_cart_subtotal;
	            }
	            $data['order_subtotal'] = append_zero(decimal_convert($order_cart_subtotal));
	        }
	    }
	    
	    if($code != null) {
	        if($action == null) {
	            $coupon_count = $this->coupons_model->valid_coupon_count($code, $toggle_module_id);
	            if(!$coupon_count) {
	                $message = array(
	                    'success' => $success,
	                    'message' => lang('msg_valid_coupon')
	                );
	                return $message;
	            }
	        }
	        $result = $this->coupons_model->get_coupon($code, $toggle_module_id);
	        // No need to perform below operation on qty update and they are already performed
	        if($result) {
	            if($action == null) {
	                $this->gc->update_hits($result->id, 'coupons'); // Update Hits
	                $this->update_affiliate_coupon_counts_hits($result->id); // Update Affiliate Coupon Hits
	                $applicable_users = explode(',', $result->applicable_users ?? '');
	                $today = date("Y-m-d H:i:s");
	                $user_limit = $this->coupons_model->user_limit_count($code, $user_id);
	                if(($result->max_customer_uses != 0) && ($user_limit >= $result->max_customer_uses)) {
	                    $message = array(
	                        'success' => $success,
	                        'message' => lang('msg_coupon_already_used')
	                    );
	                    return $message;
	                } else if(floatval($result->min_order) > floatval($order_cart_subtotal)) {
	                    $message = array(
	                        'success' => $success,
	                        'message' => sprintf(lang('msg_min_order_amount'), currency_format($result->min_order))
	                    );
	                    return $message;
	                } else if(!empty($result->applicable_users) && $result->applicable_users != 'all' && !in_array($user_id, $applicable_users)) {
	                    $message = array(
	                        'success' => $success,
	                        'message' => lang('msg_not_applicable')
	                    );
	                    return $message;
	                } else if($package_id != 0 && $result->applicable_packages != '0' && $result->applicable_packages != $package_id) {
	                    $message = array(
	                        'success' => $success,
	                        'message' => lang('msg_package_not_valid')
	                    );
	                    return $message;
	                }
	            }
	            $data = $this->apply_coupon($result, $coupon_discounts['cart_items'], $order_cart_subtotal, $mobile);
	        }
	    } else {
	        $data['success'] = true;
	        $data['discount_type'] = '';
	        $data['discount_value'] = '';
	        $data['discounted_amount'] = 0;
	        $data['order_total'] = append_zero(decimal_convert($order_cart_total));
	        $data['tax'] = append_zero(decimal_convert($order_cart_tax));
	        if(!$mobile) {
	            // update cart session with old values
	            $this->session->set_userdata('cart_checkout', $coupon_discounts['cart_items']);
	            
	            // Important - unset applied coupon session values
	            $this->session->unset_userdata('coupon_checkout');
	            $this->session->unset_userdata('coupon_applied');
	        } else {
	            // update checkout params in the users table
	            $unset_fields = array('order_total', 'order_subtotal', 'order_tax', 'order_discount', 'couponCode');
	            foreach($unset_fields as $unset_field) {
	                if(isset($coupon_discounts[$unset_field])) {
	                    unset($coupon_discounts[$unset_field]);
	                }
	            }
	            $this->db->where('id', $user_id)->update('users', array('checkout_params' => json_encode($coupon_discounts)));
	        }
	    }
	    if($action == null) {
	        $data['order_subtotal'] = append_zero(decimal_convert($coupon_discounts['subtotal']));
	        $data['message'] = lang('msg_coupon_applied');
	    }
	    
	    // Display currency symbol with subtotal and final_price SPAN
	    if(isset($data['order_total'])) {
	        $data['html_final_price'] = $data['order_total'];
	    }
	    if(isset($data['order_subtotal'])) {
	        $data['html_subtotal'] =  $data['order_subtotal'];
	    }
	    if($data['success']) {
	        // update checkout params in the users table
	        $coupon_discounts['order_total'] = $data['order_total'];
	        $coupon_discounts['order_subtotal'] = $data['order_subtotal'];
	        $coupon_discounts['order_tax'] = $data['tax'];
	        $coupon_discounts['order_discount'] = $data['discounted_amount'];
	        $coupon_discounts['couponCode'] = $code;
	        $this->db->where('id', $user_id)->update('users', array('checkout_params' => json_encode($coupon_discounts)));
	    }
	    return $data;
	}
	
	/**
	 * Apply Coupon Code
	 * @param object $coupon
	 * @param float $order_total
	 */
	public function apply_coupon($coupon, $cart_items, $order_total, $mobile = false) {
	    $percentage = 0;
	    // The final discount must not be greater than the fixed/max_discount amount.
	    // if the max_discount is not null, get the amount in percentage.
	    /*if(!empty($coupon->max_discount) && $coupon->max_discount != 0.00) {
	     $percentage = $this->amount_percentage($coupon->max_discount, $order_total);
	     } */
	    if($coupon->type == 'F' && !empty($coupon->value)) {
	        // if the discount type is fixed, get the amount in percentage. (FIXED AMOUNT * 100) / ORDER TOTAL
	        $percentage = $this->amount_percentage($coupon->value, $order_total);
	    } else if($coupon->type == 'P') { // Discount type is in percentage
	        $percentage = $coupon->value;
	    }
	    $success = false;
	    $total_discount = 0;
	    $total_tax = 0;
	    $subtotal = 0;
	    $total = 0;
	    if(!empty($percentage) && !empty($cart_items)) {
	        $success = true;
	        // calculate percentage using item subtotal (without tax)
	        foreach($cart_items as $item_id => $cart_item) {
	            $item_subtotal = $cart_item['subtotal'];
	            
	            // discount calculation
	            $item_discount = ($item_subtotal * $percentage) / 100;
	            // The final discount must not be greater than the fixed/max_discount amount.
	            // if the max_discount is not null, get the amount in percentage.
	            if(!empty($coupon->max_discount) && $coupon->max_discount != 0.00 && $item_discount > $coupon->max_discount ) {
	                $item_discount = $coupon->max_discount;
	            }
	            $total_discount = $total_discount + $item_discount;
	            
	            // subtract the discount amount from the order subtotal
	            $item_subtotal = $item_subtotal - $total_discount;
	            $subtotal = $subtotal + $item_subtotal;
	            
	            // tax calculation
	            $item_tax = 0;
	            if(!empty($cart_item['tax_percent'])) {
	                $item_tax = decimal_convert(($item_subtotal * $cart_item['tax_percent']) / 100);
	                $total_tax = $total_tax + $item_tax;
	                /*if($cart_item['tax_inclusive'] == 1) { // Tax Inclusive
	                 $total = $total + ($item_subtotal + $item_tax);
	                 } else { // Tax Exclusive
	                 $total = $total + ($item_subtotal + $item_tax);
	                 }*/
	                $item_total = ($item_subtotal + $item_tax);
	                $total = $total + $item_total;
	            } else {
	                $item_total = $item_subtotal;
	                $total = $total + $item_subtotal;
	            }
	            
	            // Store item total, subtotal and tax to avoid unnecessary calculations again and again
	            $cart_items[$item_id]['total'] = $item_total;
	            // $cart_items[$item_id]['subtotal'] = $item_subtotal;
	            $cart_items[$item_id]['tax'] = $item_tax;
	            $cart_items[$item_id]['discount'] = $item_discount;
	        }
	        
	        if(!$mobile) {
	            // update cart session with the new values
	            $this->session->set_userdata('coupon_checkout', $cart_items);
	            
	            // on qty update, also update the discounted values to avoid unnecessary database query
	            $this->session->set_userdata('coupon_applied', $coupon);
	        }
	    }
	    $data['success'] = $success;
	    $data['discount_type'] = $coupon->type;
	    $data['discount_value'] = $coupon->value;
	    $data['discounted_amount'] = append_zero(decimal_convert(floatval($total_discount)));
	    // $data['order_subtotal'] = append_zero(decimal_convert(floatval($subtotal)));
	    $data['order_total'] = append_zero(decimal_convert(floatval($total)));
	    $data['tax'] = append_zero(decimal_convert(floatval($total_tax)));
	    return $data;
	}
	
	/**
	 * Convert the amount in percentage
	 * @param float $discount
	 * @param float $order_total
	 * @return number
	 */
	public function amount_percentage($discount, $order_total) {
	    return decimal_convert(($discount * 100) / $order_total);
	}
	
	/**
	 * Index and View Order Page - Common function to update order status
	 *
	 * This function is also called on order cancellation
	 *
	 * @param number $id
	 * @param string $order_id
	 * @param array $order_info
	 * @param number $new_status
	 * @param boolean $notify_customer
	 * @param boolean $reverse_row - Reverse all commissions and Payouts
	 * @param boolean $transaction_confirmed - Recent Checkout
	 * @param number $item_order_id - ID of the Ordered Item, Single ITEM cancellation
	 */
	public function update_status($id, $order_id, $order_info, $new_status_id, $notify_customer, $reverse_row = false, $transaction_confirmed = false, $item_order_id = 0) {
	    if(empty($order_info) && !empty($order_id)) {
	        $order_info = $this->orders_model->find_by('order_id', $order_id);
	    } else if(empty($order_info) && !empty($id)) {
	        $order_info = $this->orders_model->find($id);
	    }
	    if(!$order_info) {
	        Template::set_message(sprintf(lang('msg_edit_failure'), lang('label_order')), 'danger');
	        Template::redirect(current_url());
	    }
	    $update_notes = false;
	    $order_id = $order_info->order_id;
	    $old_status_id = $order_info->status_id;
	    // update only if user changed the status
	    if ($old_status_id != $new_status_id) {
	        $update_data = array('status_id' => $new_status_id);
	        if($new_status_id == 2 && empty($order_info->date_paid)) {
	            // update the date paid
	            $update_data['date_paid'] = date('Y-m-d H:i:s');
	        }
	        if(!empty($this->input->post('order_notes'))) {
	            $update_data['notes'] = $this->input->post('order_notes');
	        }
	        $this->db->where('order_id', $order_id)->update($this->table_name, $update_data);
	        $this->orders_model->save_order_history($order_id, $old_status_id, $new_status_id, NULL, $notify_customer);
	        $process_reversal = true; // refund amount only once not everytime on order status change.
	        if($old_status_id != 1 && $new_status_id == 1) {
	            $process_reversal = true;
	        } else if($old_status_id == 1 && $new_status_id != 1) {
	            $process_reversal = true;
	        }
	        if($process_reversal) {	            
	            $this->load->model('payouts/payouts_model');
	            $this->load->model('commissions/commissions_model');
	            $order_info->status_id = $new_status_id;
				
	            // Change module row as it was before
	            if($reverse_row) {
	                // single item cancellation
	                if(!empty($item_order_id)) {
	                    $order_items = $this->db->where('id', $item_order_id)->get('order_items')->result();
	                } else { // Update all order Items
	                	$order_items = $this->db->where('order_id', $order_id)->get('order_items')->result();
					}
	                if($order_items) {
	                    foreach($order_items as $order_item) {
	                        // set the old order_item status to upgrade or renewed
	                        if(!empty($order_item->ru_order_id)) {
	                            $old_order_item = $this->db->where(array('order_id' => $order_item->ru_order_id, 'module_id' => $order_item->module_id))->get('order_items')->row();
	                            if($old_order_item) {
	                                if($order_item->subscription_type == 2) {
	                                    $order_item->old_row_id = $old_order_item->row_id;
	                                }
	                                $this->db->where('id', $old_order_item->id)->update('order_items', array('status_id' => $order_item->subscription_type == 2 ? 6 : 7));
	                            }
	                        }
	                        $module_name = gc_module_name($order_item->module_id);
	                        switch($module_name) {
	                            case 'products': // Products
	                                $this->update_order_item($order_item->id, $order_info, $order_item, $notify_customer, $new_status_id);
	                                break;
	                            case 'ads': // Ads
	                                $this->load->model('ads/ads_model');
	                                $ad_status = ($new_status_id == 2 ? 1 : 0);
	                                if($transaction_confirmed) { // recent checkout
	                                    $ad_info = $this->ads_model->get_ad($order_item->row_id, false, false); // Ad package default status to be assiged to the AD
	                                    if($ad_info) {
	                                        $ad_status = $ad_info->order_status;
	                                    }
	                                }
	                                $this->ads_model->update_ad_status($order_item->row_id, $ad_status);
	                                break;
	                            case 'leads': // Leads
	                                // Update Assigned to and Lead Status to Converted
	                                /*if($new_status_id == 2) { // order confirmed
	                                    $lead_data = array('assigned_to' => $order_item->user_id, 'status_id' => 3);
	                                } else {
	                                    $lead_data = array('assigned_to' => 0, 'status_id' => 2);
	                                }
	                                $this->db->where(array('id' => $order_item->row_id))->update('leads', $lead_data);
	                                */
	                                // Assign a lead to multiple users
	                                $lead_allotment = $this->db->where(array('lead_id' => $order_item->row_id, 'user_id' => $order_item->user_id))->get('lead_allotments')->row();	                                
	                                if($new_status_id == 2) {
	                                    $allotment_status = 1;
	                                } else {
	                                    $allotment_status = 0;
	                                }
	                                if($lead_allotment) {
	                                    $data = array('status' => $allotment_status, 'modified_on' => date('Y-m-d H:i:s'), 'modified_ip' => $this->input->ip_address());
	                                    $this->db->where('id', $lead_allotment->id)->update('lead_allotments', $data);
	                                } else {
	                                    $data = array(
	                                        'lead_id' => $order_item->row_id,
	                                        'user_id' => $order_item->user_id,
	                                        'order_id' => $order_id,
	                                        'status' => $allotment_status,
	                                        'created_on' => date('Y-m-d H:i:s'),
	                                        'created_ip' => $this->input->ip_address()
	                                    );
	                                    $this->db->insert('lead_allotments', $data);
	                                }
	                                break;
	                            case 'packages': // Package Purchase
	                                if($transaction_confirmed) {
	                                   $this->update_package($order_item);
	                                } else {
	                                    // get the last purchased package and reverse the transaction
	                                    $last_ordered_item = $this->db->where("order_id != '" . $order_item->order_id . "' AND user_id = " . $order_item->user_id . " AND status_id = 2")->get($this->table_items)->row();
	                                    if($last_ordered_item || ($order_item->subscription_type == 2 && !empty($order_item->ru_order_id))) {
	                                        $this->update_package($order_item);
	                                    }
	                                }
	                                
	                                // set all pending orders status to expired on order confirmation
	                                if(!empty($order_item->ru_order_id) && $new_status_id == 2) {
	                                    $where = "ru_order_id = '" . $order_item->ru_order_id . "' AND status_id = 1 AND order_id != '" . $order_item->order_id . "'";
	                                    $pending_orders = $this->db->where($where)->get('order_items')->result();
	                                    if($pending_orders) {
	                                        foreach($pending_orders as $pending_order) {
	                                            $sql = "UPDATE " . $this->db->dbprefix . "order_items SET status_id = 8 WHERE id = " . $pending_order->id;
	                                            $this->db->query($sql);
	                                            $sql = "UPDATE " . $this->db->dbprefix . "orders SET status_id = 8 WHERE order_id = '" . $pending_order->order_id . "'";
	                                            $this->db->query($sql);
	                                        }
	                                    }
	                                }
	                                
	                                if($order_item->subscription_type == 3) { // the package renewed recently
	                                    $module_status = array();
	                                    if(GC_PRODUCT_ID == 2) { // matrimonial script - update profile
	                                        $site_modules = array('profiles');
	                                    } else {
	                                        $site_modules = gc_modules();
	                                    }
	                                    foreach($site_modules as $gc_module) {
	                                        $module_status[$gc_module] = 2; // expired
	                                    }
	                                    // update the renewal package
	                                    $ordered_item_last = $this->db->where(array('order_id' => $order_item->ru_order_id, 'module_id' => gc_module_id('packages')))->get('order_items')->row();
	                                    $update_module_data = array();
	                                    if($ordered_item_last) {
	                                        if($new_status_id == 2) {
	                                            $order_item_package_id = $ordered_item_last->row_id;
	                                            $update_module_data['package_id'] = $order_item->row_id;
	                                            $update_module_data['expires_on'] = $order_item->expires_on;
	                                        } else {
	                                            $order_item_package_id = $order_item->row_id;
	                                            $update_module_data['package_id'] = $ordered_item_last->row_id;
	                                            $update_module_data['expires_on'] = $ordered_item_last->expires_on;
	                                        }
	                                    } else {
	                                        $order_item_package_id = $order_item->row_id;
	                                        $update_module_data['package_id'] = $order_item_package_id;
	                                        $update_module_data['expires_on'] = $order_item->expires_on;
	                                    }
	                                    $info_package = $this->db->where('id', $order_item->row_id)->get('packages')->row();
	                                    if($new_status_id == 2) {
	                                        if($info_package) {
	                                            if(GC_PRODUCT_ID == 2) {
	                                                $module_status['profiles'] = !empty($info_package->profiles_status) ? 1 : 0;
	                                            } else {
	                                                $module_status = json_decode($info_package->module_status ?? '', true);
	                                            }
	                                        }
	                                    }
	                                    foreach($site_modules as $site_module) {
	                                        $update_module_data['status'] = !empty($module_status[$site_module]) ? $module_status[$site_module] : 0;
	                                        $this->db->where(array('user_id' => $order_item->user_id, 'package_id' => $order_item_package_id))->update($site_module, $update_module_data);
	                                    }
	                                }
	                                break;
	                            case 'commission_levels':
	                                if($new_status_id == 2) {
	                                    $this->db->where('user_id', $order_item->user_id)->update('affiliate_users', array('level_id' => $order_item->row_id));
	                                } else {
	                                    // get default level, and update it in the affiliate_users table
	                                    $level = $this->db->where(array('is_default' => 1, 'status' => 1))->get('commission_levels')->row();
	                                    $level_id = $level ? $level->id : 0;
	                                    $this->db->where('user_id', $order_item->user_id)->update('affiliate_users', array('level_id' => $level_id));
	                                }
	                                break;
	                            case 'wallets': // Wallet Payments
	                                $this->load->model('wallets/wallets_points_model');
	                                $this->wallets_points_model->update_cash('wallets', $order_info->user_id, 'user', $order_item->price, ($new_status_id == 2 ? 'credit' : 'debit'));
	                                break;
	                            case 'bookings': // Bookings	      
	                                $booking_order = $order_item;
	                                $booking_order->status_id = $new_status_id;                         
	                                $this->load->model('bookings/bookings_model');
	                                $this->bookings_model->confirm_order($order_id, $booking_order, true, $notify_customer);
	                                break;
	                            case 'pyg': // PYG
	                                if($transaction_confirmed) {
	                                    // no action is required, the user purchased a new module item, no need to update the module
    	                                // $this->update_module($order_item, 'listings', $order_item->qty);
	                                }
	                                break;
	                            case 'claims': // Listing Claims
	                                $this->load->model('listings/claims_model');
	                                $claims = $this->claims_model->select('id')->find_all_by(array('order_id' => $order_id));
	                                if(!$claims) {
	                                    // save listing claim
	                                    $this->save_listing_claims($order_id, $order_item->row_id, $order_item->user_id);
	                                    $claims = $this->claims_model->select('id')->find_all_by(array('order_id' => $order_id));
	                                }
	                                if($claims) {
	                                    foreach($claims as $claim) {
                        	                $this->db->from('listings L');
                        	                $this->db->join('packages P', 'P.id = L.package_id', 'INNER');
                        	                $result = $this->db->select('P.claim_status')->where('L.id', $claim->id)->get()->row();
                        	                $claim_status_id = $result && $result->claim_status == 2 ? 2 : 1;
											if($new_status_id == 2 && $claim_status_id == 2) {
                        	                    $claim_status = 1;
                        	                } else {
                        	                    $claim_status = $new_status_id == 2 ? 0 : 2;
                        	                }
	                                        $this->claims_model->update_claim_status($claim->id, $claim_status);
	                                    }
	                                }
	                                break;
	                        }
	                        $order_item->status_id = $new_status_id;
	                        $order_item_module_id = $order_item->module_id;
	                        $order_item_row_id = $order_item->row_id;
	                        
	                        // Affiliate Commissions and Payouts are item specific, so calculate them here
	                        // MEMORY ISSUE - module_id changing automatically here without assigning
	                        $this->commissions_model->calculate($order_info, $order_item);
	                        $order_item->module_id = $order_item_module_id;
	                        $order_item->row_id = $order_item_row_id;
	                        
	                        // Update Payouts
	                        $this->payouts_model->update_payout($order_item);
	                        
	                        // set upgraded and renewed statuses
	                        if(!empty($order_item->ru_order_id)) {
	                            $subscription_status_id = $order_item->subscription_type == 2 ? 6 : 7;
	                            if($subscription_status_id == 7 && !empty($order_item->expires_on)) {
	                                // the renewed package must be applicable from the expiry only
	                                // get the information of the old order
	                                $renewed_order_item = $this->db->where('order_id', $order_item->ru_order_id)->get('order_items')->row();
	                                if(!empty($renewed_order_item->expires_on) && ($renewed_order_item->module_id == 20 || $renewed_order_item->module_id == 34)) {
	                                    // get expires on date e.g. 2022-05-16 00:00:00
	                                    // order_item expires on e.g. 2022-05-22 00:00:00
	                                    $package_info = $this->db->where('id', $renewed_order_item->row_id)->get(gc_module_name($renewed_order_item->module_id))->row();
	                                    $new_expires_on = $this->calculate_expires_on($package_info, 3, $order_item->ru_order_id, gc_module_name($renewed_order_item->module_id));
	                                    if(!empty($new_expires_on)) {
	                                        $this->db->where('order_id', $order_item->order_id)->update('order_items', array('expires_on' => $new_expires_on));
	                                    }
	                                }
	                            }
	                            $this->db->where('order_id', $order_item->ru_order_id)->update('orders', array('status_id' => $subscription_status_id));
	                            $this->db->where('order_id', $order_item->ru_order_id)->update('order_items', array('status_id' => $subscription_status_id));
	                        }
	                    }
	                }
	            }	            
	            
	            // calculate and save hierarchy commissions
	            $this->commissions_model->hierarchy_commissions($order_info);
	            
	            // update renewed/upgraded items to hide the duplicate items
	            $this->renewed_upgraded($order_id, $new_status_id);
	            
	            // Affiliate Commissions and Payouts are item specific, so calculate it here
	            // $this->load->model('commissions/commissions_model');
	            // $this->commissions_model->calculate_commissions($order_id, $order_info);
	            // Update Invoice also
	            $this->db->query("UPDATE " . $this->db->dbprefix . $this->table_invoices . " SET status_id = " . $new_status_id . " WHERE order_id = '" . $order_id . "'");
	            if(!empty($item_order_id)) {
	                $this->db->query("UPDATE " . $this->db->dbprefix . $this->table_items . " SET status_id = " . $new_status_id . " WHERE id = '" . $item_order_id . "'");
	            } else { // Update order items also					
					$this->db->query("UPDATE " . $this->db->dbprefix . $this->table_items . " SET status_id = " . $new_status_id . " WHERE order_id = '" . $order_id . "'");
				}
	        }
	        
	        Template::set_message(sprintf(lang('msg_status_changed'), lang('label_order')), 'success');
	    } else if($notify_customer == 1) {
	        $update_notes = true;
	        $this->orders_model->save_order_history($order_id, $old_status_id, $new_status_id, NULL, $notify_customer);
	        Template::set_message(lang('label_customer_notified'), 'success');
	    } else {
	        $update_notes = true;
	        Template::set_message(lang('msg_nothing_to_update'), 'danger');
	    }
	    
	    // Update Customer Notes
	    if($update_notes && !empty($this->input->post('order_notes'))) {
	        $update_data['notes'] = $this->input->post('order_notes');
	        $this->db->where('order_id', $order_id)->update($this->table_name, $update_data);
	    }
	    
	    // generate invoice before sending email
	    if($new_status_id == 2 && $order_info->invoice_id == 0) {
	        // generate new invoice
	        $this->generate_invoice($order_id, $order_info);
	    }
	    
	    // send notification email to customer
	    if($notify_customer == 1) {
	        if($order_info->status_id == 2) {
	            // send invoice as attachment
	            $this->email_print($order_id, 'email', array('path' => FCPATH . 'assets/files/invoices', 'attach_invoice' => true));
	        } else {
	            // email invoice to the user
	            $this->email_print($order_id, 'email');
	        }
	        
	        // notify admin and seller via SMS
	        // $this->sms_order_notifications($order_id);
	        // send mobile socket notification
	        if($new_status_id == 2 && $order_info->order_total > 0) {
    	        $notification_title = lang('order_notification_title');
    	        $notification_message = sprintf(lang('order_notification_message'), $order_id);
    	        $notification_data = array(
    	            'id' => $id,
    	            'title' => $notification_title,
    	            'message' => $notification_message,
    	            'alert' => array('header' => $notification_title, 'message' => $notification_message, 'sound' => 'order-payment-received')
    	        );
    	        $this->load->model('mobiles/mobiles_model');
    	        $this->mobiles_model->socket_notification($order_info->user_id, $notification_data);
	        }
	    }
	    // Member area clear orders cache to display Upgrade To Business button
	    clear_cache('orders_' . $order_info->user_id);
	    // save ordered items information in the orders table itself to avoid looping
	    $this->ordered_items_info($order_id, isset($order_items) ? $order_items : array());
	}
	
	/**
	 * Ordered Items Info
	 */
	private function ordered_items_info($order_id, $order_items) {
	    if(!$order_items) {
	        $order_items = $this->db->where('order_id', $order_id)->get('order_items')->result();
	    }
	    if($order_items) {
	        $this->db->where('order_id', $order_id)->update('orders', array('ordered_items' => json_encode($order_items)));
	    }
	}
	
	/**
	 * Update modules on pacakage update/renew
	 * @param string $order_id
	 */
	private function update_package($order_item) {
	    $this->load->model('packages/packages_model');
	    $package = $this->packages_model->get_package($order_item->row_id);
	    if(!$package) {
	        return false;
	    }
	    $order_info = $order_item;
	    $order_info->package_id = $order_item->row_id;
	    if(GC_PRODUCT_ID == 2) { // matrimonial script - update profile
	        $data = array(
	            'package_id' => $package->id,
	            'featured' => $package->profiles_featured,
	            'expires_on' => $order_info->expires_on,
	            'modified_on' => date('Y-m-d H:i:s'),
	            'modified_ip' => $this->input->ip_address()
	        );
	        $this->db->where('user_id', $order_item->user_id)->update('profiles', $data);
	    } else {
	        if(!empty($order_item->ru_order_id)) {
	            $old_package = $this->db->where(array('order_id' => $order_item->ru_order_id, 'module_id' => $order_item->module_id))->get('order_items')->row();
	            if($old_package) {
	                if($order_item->subscription_type == 2) {
	                    $order_info->old_package_id = $old_package->row_id;
	                    
	                    // also migrate the pyg purchases to the new package
	                    $this->db->where(array('user_id' => $order_item->user_id, 'module_id' => gc_module_id('pyg'), 'row_id' => $old_package->row_id))->update('order_items', array('row_id' => $order_item->row_id));
	                }
	                $this->db->where('id', $old_package->id)->update('order_items', array('status_id' => $order_item->subscription_type == 2 ? 6 : 7));
	            }
	        }
	        $order_info->module_featured = $package->module_featured;
	        $order_info->module_limit = $package->module_limit;
    	    $config_modules = gc_modules();
    	    if(!empty($config_modules)) {
        	    foreach($config_modules as $config_module) {
        	        // upgrade package limited rows only
        	        $limit = (int) gc_field_value($config_module, $order_info->module_limit);
        	        $this->update_module($order_info, $config_module, $limit);
        	    }
    	    }
	    }
	}
	
	/**
	 * Package/PYG Renewal
	 * @param object $order_info
	 * @param string $module
	 * @param int $qty
	 */
	private function update_module($order_info, $module, $qty) {
	    $sql = "SELECT M.id, M.status FROM " . $this->db->dbprefix . $module ." M";
	    if($module != 'listings') {
	        $sql .= " INNER JOIN " . $this->db->dbprefix . "listings L ON L.id = M.listing_id";
	    }
	    $sql .= " WHERE M.deleted = 0";
	    if($module == 'listings') {
	        $sql .= " AND M.user_id = " . $order_info->user_id;
	        if(!empty($order_info->old_row_id)) {
	            $sql .= " AND M.package_id = " . $order_info->old_row_id;
	        }
	    } else {
	        $sql .= " AND L.deleted = 0 AND L.user_id = " . $order_info->user_id;
	        if(!empty($order_info->old_row_id)) {
	            $sql .=" AND (L.package_id = " . $order_info->old_row_id . " OR M.package_id = " . $order_info->old_row_id . ")";
	        }
	    }
	    $sql .= " ORDER BY M.id DESC";
	    if($qty > 0) {
	        $sql .= " LIMIT " . $qty;
	    }
	    $rows = $this->db->query($sql)->result();
	    if($rows) {
	        $data = array();
	        $featured = gc_field_value($module, $order_info->module_featured);
	        foreach($rows as $row) {
	            if($module == 'listings') {
	                $data[] = array(
	                    'id' => $row->id,
	                    'package_id' => $order_info->package_id,
	                    'featured' => $featured,
	                    'expires_on' => $order_info->expires_on,
	                    'status' => 1
	                );
	            } else {
	                $data[] = array(
	                    'id' => $row->id,
	                    'package_id' => !empty($params_data->package_id) ? $params_data->package_id : $order_info->package_id,
	                    'featured' => $featured,
	                    'status' => 1
	                );
	            }
	        }
	        if(!empty($data)) {
	            $this->db->update_batch($module, $data, 'id');
	        }
	    }
	}
	
	/**
	 * Change the status of renewed and upgraded ordered items to hide them from the member dashboard
	 * this will not display the already renewed or upgraded subscriptions
	 * @param string $order_id
	 * @param number $new_status_id
	 */
	private function renewed_upgraded($order_id, $new_status_id) {
	    $where = "order_id = '" . $order_id . "' AND ru_order_id IS NOT NULL";
	    $order_items = $this->db->select('ru_order_id, subscription_type, module_id, row_id, variant_id')->where($where)->get('order_items')->result();
	    if($order_items) {
	        foreach($order_items as $order_item) {
	            $item_where = array(
	                'order_id' => $order_item->ru_order_id,
	                'module_id' => $order_item->module_id,
	                'row_id' => $order_item->row_id,
	                'variant_id' => $order_item->variant_id
	            );
	            if($new_status_id == 2) {
	                if($order_item->subscription_type == 2) { // upgraded items
	                    $this->db->where($item_where)->update($this->table_items, array('status_id' => 6));
	                }
	                if($order_item->subscription_type == 3) { // renewed items
	                    $this->db->where($item_where)->update($this->table_items, array('status_id' => 7));
	                }
	            } else {
	                $this->db->where($item_where)->update($this->table_items, array('status_id' => $new_status_id));
	            }
	        }
	    }
	}
	
	/**
	 * Common Function for Email Print
	 * @param string $order_id
	 * @param string $type
	 * @param number $template_id
	 * @param array $params
	 * @return boolean
	 */
	public function email_print($order_id, $type = 'pdf', $params = array()) {
	    if($type == 'email') {
	        $type = 'sms_email';
	    }
	    if(empty($params['order'])) {
	        $order = $this->orders_model->get_order($order_id);
	    } else {
	        $order = $params['order'];
	        unset($params['order']);
	    }
	    if($order) {
	        $email_action_type = empty($params['action_type']) ? 'status_update' : $params['action_type']; // email template to be sent to the user
	        $order->date_paid = empty($order->date_paid) ? lang('label_never') : gc_date_format($order->date_paid);
	        $order->created_on = gc_date_format($order->created_on);
	        $order->order_status = !empty($order->status_lang) ? lang($order->status) : $order->status;
	        $order->invoice_no = empty($order->invoice_no) ? lang('not_applicable') : $order->invoice_no;
	        $order->payment_method = !empty($order->payment_method_name) ? $order->payment_method_name : (!empty($order->payment_method) ? lang('label_' . strtolower($order->payment_method)) : lang('not_applicable')); // wallet can be used as a payment method
	        // $order->site_url = site_url();
	        if(!empty($params['sequence_no'])) {
	            $order->sequence_no = $params['sequence_no'];
	            unset($params['sequence_no']);
	        }
	        $order_items = $this->get_order_items($order_id);
	        $cart_data = array(
	            'items' => $order_items,
	            'role' => isset($params['role']) ? $params['role'] : 'email',
	            'coupon_value' => isset($order->coupon_value) ? $order->coupon_value : 0,
	            'inline_style' => true
	        );
	        $order->cart_items = cart_layout($cart_data);
	        $order->cart_summary = cart_summary($cart_data);
	        $order->order_total = currency_format($order->order_total, $cart_data);
	        $order = (array) $order;
	        $attachments = array();
	        if(isset($params['attach_invoice'])) {
	            // $this->email_print($order_id, 'pdf', array('path' => $path, 'download' => 2));
	            // generate invoice and send it in attachment
	            $path = FCPATH . 'assets/files/invoices';
	            $result = $this->gemailer->sms_email_pdf('view', array('template_table' => 'order_email_templates', 'module_table' => 'orders', 'action_type' => $email_action_type, 'id' => $order['id'], 'record' => $order, 'filename' => $order_id));
	            $this->gc->create_pdf($order_id, $this->load->view('orders/_tpl_pdf', array('content' => $result['message']), true), true, array('path' => $path, 'download' => 2));
	            $attachments = array($path . '/' . $order_id . '.pdf');
	        }
	        if($type == 'preview') { // Email Templates Preview
	            return $order;
	        } else {
    	        $result = $this->gemailer->sms_email_pdf($type, array('template_table' => 'order_email_templates', 'module_table' => 'orders', 'action_type' => $email_action_type, 'id' => $order['id'], 'record' => $order, 'filename' => $order_id, 'attachments' => $attachments));
    	        if($type == 'view' && !empty($result['message'])) {
    	            return $result['message'];
    	        } else {
    	            return $result;
    	        }
	        }
	    }
	}	
	
	/**
	 * Get Order Information
	 * @param string $order_id
	 */
	public function get_order($order_id, $column = 'order_id') {
	    $sql = "SELECT M.*, OC.name AS customer_name, OC.email, OC.phone_number, OC.address, OC.city_id, OC.state_id, OC.country_iso, OC.city, OC.state, OC.country, OC.postcode, CONCAT(INV.prefix, INV.id) AS invoice_no, M.payment_method AS payment_method_name, M.transaction_id, M.reminder_notifications,";
	    $sql .= " S.color AS status_color, S.name AS status, S.name_lang AS status_lang, PM.display_name AS payment_method_name, OCS.coupon_code, OCS.value AS coupon_value, OCS.value_type AS coupon_value_type, OCS.amount AS coupon_discount";
	    $sql .= " FROM " . $this->db->dbprefix . "orders M";
	    $sql .= " LEFT JOIN " . $this->db->dbprefix . "order_invoices INV On INV.order_id = M.order_id";
	    $sql .= " LEFT JOIN " . $this->db->dbprefix . "order_customers OC On OC.order_id = M.order_id";
	    $sql .= " LEFT JOIN " . $this->db->dbprefix . "payment_gateways PM On PM.name = M.payment_method";
	    $sql .= " LEFT JOIN " . $this->db->dbprefix . "order_statuses S On S.id = M.status_id";
	    $sql .= " LEFT JOIN " . $this->db->dbprefix . "order_coupons OCS On OCS.order_id = M.order_id";
	    if(empty($order_id)) {
	       $sql .= " ORDER BY RAND()";
	    } else {
	       $sql .= " WHERE M." . $column . " = '" . $order_id . "'";
	    }
	    $row = $this->db->query($sql)->row();
	    return $row;
	}
	
	/**
	 * Get Order Items
	 * @param string $order_id
	 * @return array
	 */
	public function get_order_items($value, $field = 'order_id') {
	    return $this->db->get_where('order_items', array($field => $value))->result();
	}
	
	/**
	 * Products - Update Order Items on Order Status Change
	 *
	 * @param number $item_id
	 * @param array $order_info
	 * @param array $item_info
	 * @param boolean $notify_customer
	 */
	public function update_order_item($item_id, $order_info = array(), $item_info = array(), $notify_customer = 0, $new_status_id = 2) {
	    if(empty($item_info)) {
	        $item_info = $this->db->where(array('id' => $item_id))->get('order_items')->row();
	    }
	    if(!empty($item_info)) {
	        $order_id = $item_info->order_id;
	        if(empty($order_info)) { // Get order detail
	            $order_info = (array) $this->db->where('order_id', $order_id)->get('orders')->row();
	        }
	        $order_info = (array) $order_info;
	        if(!empty($order_info)) {
	            // get the row info
	            $module = 'products';
	            $product = $this->db->where('id', $item_info->row_id)->get('products')->row();
	            if($product) {
	                $row_id = $product->id;
	                $stock = $product->stock;
	                // check if the purchased item is a variant
	                if(!empty($item_info->variant_id)) {
	                    // get variant stock info
	                    $product_variant = $this->db->where('id', $item_info->variant_id)->get($this->table_variants)->row();
	                    if($product_variant) {
	                        $row_id = $product_variant->id;
	                        $module = 'product_variants';
	                        $stock = $product_variant->stock;
	                    }
	                }
	                if($new_status_id == 2) {
	                    $stock = $stock - $item_info->qty;
	                    
	                    // Decrement value but not negative
	                    $sql = "UPDATE " . $this->db->dbprefix . $module . " SET stock = stock - " . $item_info->qty . " WHERE id = " . $row_id . " AND stock > 0";
	                    $this->db->query($sql);
	                    
	                    // if the item is in the user's wishlist, delete it
	                    $this->db->where(array('user_id' => $item_info->user_id, 'product_id' => $item_info->row_id, 'variant_id' => $item_info->variant_id))->delete('product_wishlists');
	                } else {
	                    $stock = $stock + $item_info->qty;
	                    
	                    // Increment the stock value
	                    $sql = "UPDATE " . $this->db->dbprefix . $module . " SET stock = stock + " . $item_info->qty . " WHERE id = " . $row_id;
	                    $this->db->query($sql);
	                }
	                
	                $seller = $this->db->where('id', $product->user_id)->get('users')->row();
	                if($seller) {
	                    $record = new stdClass;
	                    $record->status = lang('not_specified');
	                    $order_status = $this->gc->get('order_statuses', array('where' => 'id = ' . $new_status_id));
	                    if($order_status) {
	                        $record->status = $order_status->name;
	                    }
	                    $record->name = $seller->display_name;
	                    $record->seller_email = $seller->email;
	                    $record->seller_phone_number = $seller->phone_number;
	                    $record->title = $product->title;
	                    $product_url = gc_url_format($module, $product->slug, $product->id, (!empty($product->city) ? $product->city : null));
    	                // low stock notification
    	                $stock_threshold = $product->stock_threshold;
    	                if($stock_threshold >= $stock && $stock_threshold != 0) {
    	                    $product_list = '<li>' . anchor($product_url, $product->title, 'title="' . $product->title . '"') . ' (' . $stock . ')</li>';;
    	                    $record->product = '<ul>' . $product_list . '</ul>';
    	                    // low stock emails
    	                    $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'module_email_templates', 'module_table' => $module, 'action_type' => 'low_stock', 'id' => $seller->id, 'record' => $record, 'email_to' => $seller->email, 'message_to' => $seller->phone_number));
    	                }
    	                $product_list = '<li>' . anchor($product_url, $product->title, 'title="' . $product->title . '"') . ' (' . $item_info->qty . ')</li>';;
    	                $record->product = '<ul>' . $product_list . '</ul>';
    	                
    	                // orders email to seller
    	                $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'module_email_templates', 'module_table' => $module, 'action_type' => 'order', 'id' => $seller->id, 'record' => $record, 'email_to' => $seller->email, 'message_to' => $seller->phone_number));
	                }
	                
	                // update the item stock
	                $this->db->where('id', $row_id)->update($module, array('stock' => $stock));
	            }
	            
	            // set the order item status to cancelled.
	            // do not process cancelled items again.
	            $this->db->where('id', $item_id)->update('order_items', array('status_id' => $order_info['status_id']));
	            
	            if($notify_customer == 1) {
	                // notify both seller and user - Common Function
	                $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'order_email_templates', 'module_table' => 'order_items', 'action_type' => 'status_update', 'id' => $item_info->id));
	            }
	        }
	    }
	}
	
	/**
	 * Convert Currency
	 * @param string $to - Converted To
	 * @param float $amount - Amount on which conversion will be applied
	 * Yahoo suddenly terminated its finance download API on November 17, 2017.
	 */
	public function currencyConversion($to, $amount) {
	    if(settings_item('site.currency_code') != $to) {
	        $this->load->library('CurrencyConverter');
	        $this->CurrencyConverter = new CurrencyConverter();
	        $amount =  $this->CurrencyConverter->convert(settings_item('site.currency_code'), $to, $amount, 1, 1);
	        return floatval($amount);
	    } else {
	        return floatval($amount);
	    }
	}	
	
	/**
	 * Display packages to the user based on new and upgrade
	 */
	public function get_package($package_id, $table = 'packages') {
	    $query = $this->translations->build_query($table);
	    $sql = $query['select'];
	    $sql .= " FROM " . $this->db->dbprefix . $table . " M";
	    if(!empty($query['join'])) {
	        $sql .= $query['join'] . PHP_EOL;
	    }
	    $sql .= " WHERE id = " . $package_id;
	    $result = $this->db->query ( $sql )->row();
	    return $result;
	}
	
	/**************************************
	 * START OF ECOMMERCE SECTION
	 **************************************/
	
	/**
	 * Add/Remove Item to/from Cart
	 * if an item is already in the cart, increase the quantity
	 *
	 * the cart is the main session variable, its value will be used on order save.
	 * Modify the cart on qty increase or decrease
	 *
	 * The user can buy only on Products Detail page, this is how most of the top sites implement eCommerce feature.
	 * 
	 * Usage E.g.
	 * $this->orders_model->cart(4, 0, 1, 1, 'leads', false); // Leads
	 * $this->orders_model->cart(5, 0, 1, 1, 'leads', false); // Leads
	 * $this->orders_model->cart(4, 0, 1, 1, 'ads'); // Ads
	 * $this->orders_model->cart(0, 0, 1, 1, 'wallets', true, array('price' => 600)); // Wallets
	 * $this->orders_model->cart(19, 0, 1, 1, 'claims', true); // Claims
	 * $this->orders_model->cart(2, 0, 1, 1, 'packages'); // Subscription
	 * $this->orders_model->cart(1, 0, 1, 1, 'pyg', false, array('params' => array('package_id' => 1, 'module' => 'listings', 'qty' => 1))); // PYG	    
	 * $this->orders_model->cart(1, 0, 1, 1, 'bookings'); // Bookings
	 * $this->orders_model->cart(8, 0, 1, 1, 'products'); // Products
	 * $this->orders_model->cart(8, 0, 1, 1, 'products'); // Variants: product_variant session must be set for this
	    
	 * @param integer $id
	 * @param integer $action - 0 -> Add Item, 1 -> Remove Item
	 * @param number $qty - The number of items to be added to cart
	 * @param string $module
	 * @param string $is_single_item - FALSE - The user can add multiple quantity to the cart
	 * @param array $extra - Optional Items - e.g. pass relation_item_id from the checkout page
	 * @return boolean
	 */
	public function cart($id, $action = 0, $qty = 1, $action_qty = 0, $module = 'products', $is_single_item = true, $extra = array(), $user_id = 0) {
	    if($action == 0) {
	        $checkout_cart_items = $this->checkout_params($user_id);
	        $module_id = gc_module_id($module);
	        // only one item can be added to the cart except wallet payments
	        // single item is not applicable for wallets
	        if($is_single_item && $module != 'wallets' && $module != 'products') {
	            // check for the item
	            $has_item = false;
	            if(!empty($checkout_cart_items)) {
	                foreach($checkout_cart_items as $checkout_cart_item) {
	                    if(!empty($checkout_cart_item['row_id']) && $checkout_cart_item['module_id'] == $module_id && $id == $checkout_cart_item['row_id']) {
	                        $has_item = true;
	                    }
	                }
	            }
	            if($has_item) {
	                return false;
	                // die(json_encode(array('items' => count($checkout_cart_items), 'checkout_url' => gc_url_format('members/checkout')))); // count total items and output json
	            }
	        }
	        switch($module) {
	            case 'products': // products
	                $this->load->library('ecommerce');
    	            $id = (int) $id;
    	            $product = $this->db->get_where($module, array('id' => $id))->row();
    	            if(!$product) {
    	                die(json_encode(array('items' => 0)));
    	                return false;
    	            }
    	            $stock = $product->stock;
    	            // The user selected a Product Variant
    	            $attributes = array();
    	            $variant_id = 0;
    	            $product_unit = (array) $this->session->userdata('product_variant');
    	            if(empty($product_unit) && !empty($extra['variant_id'])) {
    	                // add/remove variant on radio change - checkout page
    	                $product_unit = $this->ecommerce->update_item($id, array(), $extra['variant_id']);
    	            }
    	            if(!empty($product_unit)) {
    	                $product->id = $product->id . '-' . $product_unit['variant_id'];
    	                $attributes = !empty($product_unit['attributes']) ? $product_unit['attributes'] : array();
    	                if(!empty($product_unit['cover'])) {
    	                    $product->cover = $product_unit['cover'];
    	                }
    	                $variant_id = $product_unit['variant_id'];
    	                $stock = $product_unit['stock'];
    	            } else {
    	                $product->id .= '-0';
    	            }
    	            $product->id .= '-' . (isset($extra['relation_item_id']) ? $extra['relation_item_id'] : 0) . '-' . gc_module_id($module); // append module id also
    	            
    	            // if an item is already in the cart, increase the quantity
    	            if($stock > 0) {
    	                if(array_key_exists($product->id, $checkout_cart_items)) {
    	                    $product->max_qty = 50000;
    	                    /*$item_qty = $checkout_cart_items[$product->id]['qty'];
    	                    if($action_qty == 0) { // increase quantity
    	                        $checkout_cart_items[$product->id]['qty'] = $item_qty >= $product->max_qty ? $product->max_qty : $item_qty + $qty;
    	                    } else if($item_qty > 0) { // decrease quantity
    	                        $checkout_cart_items[$product->id]['qty'] = $item_qty - $qty;
    	                    }*/
    	                    $checkout_cart_items[$product->id]['qty'] = $qty;
    	                } else {
    	                    // display relation item information on the checkout/view order page
    	                    $relation_item_info = '';
    	                    if(!empty($extra['relation_item_id'])) {
    	                        $relation_item_variant_id = !empty($extra['relation_item_variant_id']) ? $extra['relation_item_variant_id'] : 0;
    	                        $relation_item = $this->ecommerce->get_product_or_variant($extra['relation_item_id'], $relation_item_variant_id);
    	                        if($relation_item && !empty($relation_item->name)) {
    	                            $relation_item_info = $relation_item->name;
    	                            if(!empty($relation_item->variant)) {
    	                                $relation_item_info .= ' (' . $relation_item->variant . ' ###' . $relation_item->sku . ')';
    	                            }
    	                        }
    	                    }
    	                    $checkout_cart_items[$product->id] = array(
    	                        'id' => $product->id,
    	                        'module_id' => gc_module_id($module),
    	                        'url' => gc_url_format('products', $product->slug, $id) . (!empty($variant_id) ? '?sku=' . $variant_id : ''), // Redirect URL
    	                        'attributes' => $attributes,
    	                        'cover' => $product->cover,
    	                        // 'optional_items' => $product->optional_items, // allow users to select optional services on the checkout page
    	                        'relation_item_id' => !empty($extra['relation_item_id']) ? $extra['relation_item_id'] : 0,
    	                        'relation_item_variant_id' => 0, // a software can have variants e.g. Standard, Business and Developer Editions
    	                        'relation_item_info' => $relation_item_info,
    	                        'qty' => 1
    	                    );
    	                }
    	            } else { // Quantity cannot be increased, not sufficient items in stock
    	                if($is_single_item) {
    	                    die(json_encode(array('items' => 0)));
    	                }
    	                return false;
    	            }
	                break;
	            case 'wallets':
	                if(isset($extra['price'])) {
    	                $item_id = '0-0-0-' . $module_id;
    	                $checkout_cart_items[$item_id] = array(
    	                    'row_id' => 0,
    	                    'module_id' => $module_id,
    	                    'price' => $extra['price']
    	                );
	                }
	                break;
	            case 'pyg':
	                if(isset($extra['params'])) {
	                    $item_id = $id . '-0-0-' . $module_id;
	                    $checkout_cart_items[$item_id] = array(
	                        'row_id' => $id,
	                        'module_id' => $module_id,
	                        'params' => $extra['params']
	                    );
	                }
	                break;
	            case 'bookings':
					// only one items can be added to the bookings checkout to resolve seats ambiguities
	                if(!empty($checkout_cart_items)) {
	                    foreach($checkout_cart_items as $ctkey => $ctvalue) {
	                        if($ctvalue['module_id'] == gc_module_id('bookings')) {
	                            unset($checkout_cart_items[$ctkey]);
	                        }
	                    }
	                }
	                $item_id = $id . '-0-0-' . $module_id;
	                $checkout_cart_items[$item_id] = array(
	                    'row_id' => $id,
	                    'module_id' => $module_id,
	                    'qty' => $qty
	                );
	                break;
	            default:
	                $item_id = $id . '-0-0-' . $module_id;
	                $checkout_cart_items[$item_id] = array(
	                    'row_id' => $id,
	                    'module_id' => $module_id
	                );
	           }	           
	           switch($module) {
	               case 'products':
	               case 'ecart':
	               case 'packages':
	               case 'ads':
	                   if(!empty($extra['gc_order_type'])) {
	                       $gc_order_type = $extra['gc_order_type'];
	                       if(!empty($gc_order_type['subscription_type'])) {
	                           $checkout_cart_items[$item_id]['subscription_type'] = $gc_order_type['subscription_type'];
	                       }
	                       if(!empty($gc_order_type['order_id'])) {
	                           $checkout_cart_items[$item_id]['order_id'] = $gc_order_type['order_id'];
	                       }
	                       if(!empty($gc_order_type['order_item_id'])) {
	                           $checkout_cart_items[$item_id]['order_item_id'] = $gc_order_type['order_item_id'];
	                       }
	                   }
	                   break;
	           }    
	    } else {	       
	        // $id = $id . '-0-0-' . gc_module_id($module); // the id already has the module id
	        // also remove item from the checkout cart
	        $checkout_cart_items = $this->checkout_params($user_id);
	        if(!empty($checkout_cart_items)) {
	            unset($checkout_cart_items[$id]);
	            // unset all optional items also
	            foreach($checkout_cart_items as $checkout_cart_item) {
	                if(!empty($checkout_cart_item['relation_item_id']) && $checkout_cart_item['relation_item_id'] == $id) {
	                    unset($checkout_cart_items[$checkout_cart_item['id']]);
	                }
	            }
	        }
	    }
	    
	    // Update Cart Items
	    if(empty($checkout_cart_items)) { // all items removed from the cart, clear the cart
	        $this->checkout_params($user_id, false, array('items' => $checkout_cart_items), true);
	    } else {
	        $this->checkout_params($user_id, false, array('items' => $checkout_cart_items));
	    }
	    
	    // prepare cart items for display
	    $this->cart_items($user_id, array(), false, !empty($extra['zero_checkout']) ? true : false);
	    
	    // unset the selected product unit - SKU
	    $this->session->unset_userdata('product_variant');
	}
	
	/**
	 * Load Cart Items
	 */
	public function cart_load($user_id = 0) {	    
	    $this->cart_items();
	    $cart = $this->checkout_params($user_id);
	    if(!empty($cart)) {
	        $cart_box = '<ul class="cart-products-loaded">';
	        $total = 0;
	        $total_savings = 0;
	        foreach($cart as $id => $item) {
	            $item = (array) $item;
	            if(!empty($item['item']['name'])) {
    	            $module_name = gc_module_name($item['module_id']);
    	            switch($module_name) {
    	                case 'products':
    	                    $product_unit = '';
    	                    if(!empty($item['attributes'])) {
    	                        $product_unit = '(<small>' . implode('|', $item['attributes']) . '</small>)';
    	                    }
    	                    $cart_box .= "<li class='cart-list-item'><a href='" . $item['url'] . "' title='" . $item['item']['name'] . "'>" . $item['item']['name'] . $product_unit . "(" . lang('label_qty') . ': ' . $item['qty'] . ")</a>";
    	                    break;
    	                default:
    	                    $cart_box .= "<li class='cart-list-item'>" . $item['item']['name'];
    	                    break;
    	            }
    	            $item_price = $item['subtotal']; // $item['qty'] * (!empty($item['offer_price']) ? $item['offer_price'] : $item['price']);
    	            $cart_box .= " &mdash; " . currency_format($item_price) . " <a href=\"#\" class=\"remove-item item-remove\" data-code=\"$id\">&times;</a></li>";
    	            $subtotal = $item['subtotal'];
    	            $total += $item['subtotal'];
    	            if(!empty($item['offer_price'])) {
    	                $total_savings = $item['savings']; // ($total_savings + ($item['qty'] * ($item['price'] - $item['offer_price'])));
    	            }
	            }
	        }
	        $cart_box .= "</ul>";
	        $cart_box .= '<div class="cart-products-total">' . lang('label_order_subtotal') . ' : '. currency_format($total, null, true).' <u><button type="button" class="btn btn-danger btn-xs clear-cart mr-1"><i class="fas fa-times fa-fw"></i>' . lang('bf_clear') . '</button><a href="' . gc_url_format('members/checkout') . '" class="btn btn-xs btn-theme-secondary" title="' . lang('cart_title') . '"><i class="fas fa-shopping-cart fa-fw mr-1"></i>' . lang('label_checkout') . '</a></u></div>';
	        if(!empty($total_savings)) {
	            $cart_box .= '<div class="cart-saving">' . lang('cart_saving') . ' ' . currency_format($total_savings, null, true) . '</div>';
	        }
	        die($cart_box); //exit and output content
	    } else {
	        die(lang('cart_empty')); //we have empty cart
	    }
	}
	
	/**
	 * Allow user to view his/her own items cart info
	 *
	 * @param string $order_id
	 * @param array $order_info
	 * @param number $seller_id
	 * @param array $items
	 * @return object
	 */
	public function seller_items($order_id, $order_info = array(), $seller_id = 0, $items = array()) {
	    if(empty($order_info)) {
	        //Retrieve listing information from database
	        $order_info = $this->orders_model->email_pdf_order($order_id);
	    }
	    if(!empty($order_info)) {
	        $summary = array(
	            'order_total' => 'total',
	            'order_subtotal' => 'subtotal',
	            'order_commission' => 'commission',
	            'order_tax' => 'tax',
	            'order_discount' => 'discount'
	        );
	        // Reset all values to zero
	        foreach($summary as $summary_key => $summary_value) {
	            $order_info[$summary_key] = 0;
	        }
	        if(empty($items) && !empty($seller_id)) {
	            // Ge all items from the order_items table
	            $items_info = $this->get_where($this->table_items, array('order_id' => $order_id, 'seller_id' => $seller_id))->result();
	            foreach($items_info as $item_info) {
	                // Get owner specific items for Order Templates - Email/SMS Notifications
	                $item_info = json_decode($item_info->item ?? '');
	                if(!empty($item_info->key)) {
	                    $items[$item_info->key] = $item_info;
	                }
	            }
	        }
	        if(!empty($items)) {
	            // Sum Order Total, Order Subtotal, Commissions, Tax and Discounts
	            // Also set the items
	            $order_items = array();
	            foreach($items as $item) {
	                $item = (array) $item;
	                foreach($summary as $key => $value) {
	                    if(isset($item[$value])) {
	                        $order_info[$key] = $order_info[$key] + $item[$value];
	                    }
	                }
	                $order_items[$item['key']] = $item;
	            }
	            $order_info['item'] = json_encode($order_items);
	        }
	    }
	    return $order_info;
	}
	
	/**
	 * Send low stock/order confirm/product stock notification (SMS/Email) to the business owner
	 *
	 * @param object $user
	 * @param object $product
	 * @param object $template
	 * @param string $type
	 * @param string $message_type -  0 -> Notify Customer, 1 -> Low Stock, 2 -> Owner Order Notification
	 * @param array $extra - Extra subject and message parameters - Order Updated Paramters
	 */
	public function notify($user, $product, $template = array(), $type = 'email', $message_type = 0, $extra = array()) {
	    if(!empty($template) && !empty($user) && !empty($product)) {
	        $product_url = gc_url_format('products', $product->slug, $product->id);
	        $customer_name = (!empty($user->display_name) ? $user->display_name : lang('label_customer'));
	        $product_title = $product->title;
	        $email_to = $user->email;
	        if($type == 'email') {
	            $subject_data = array(
	                'subject' => $template->subject,
	                'customer_name'	=> $customer_name,
	                'title' => $product_title
	            );
	            if(!empty($extra['subject'])) {
	                $subject_data = array_merge($subject_data, $extra['subject']);
	            }
	            $subject = $this->parser->parse('email_templates/email_subject', $subject_data, TRUE);
	            $message_data = array(
	                'message' => $template->message,
	                'customer_name'	=> $customer_name,
	                'title' => $product_title,
	                'url' => $product_url,
	                'site_url' => site_url(),
	                'site_title' => settings_item('site.title')
	            );
	            if(!empty($extra['message'])) {
	                $message_data = array_merge($message_data, $extra['message']);
	            }
	            $message = $this->parser->parse('email_templates/email_body', $message_data, TRUE); //TRUE is important to return string
	            // Send Email
	            $this->load->library('emailer/emailer');
	            $data = array(
	                'to'		=> $email_to,
	                'subject'	=> $subject,
	                'message'	=> $message
	            );
	            $this->emailer->send($data, ($message_type == 0 ? TRUE : FALSE)); // Send back in stock notifications to email queue
	        } else if($type == 'sms' && !empty($user->phone_number)) { // Send SMS
	            // in case of multiple items (owner notification), the product variable will be passed as a string
	            $item = isset($product->title) ? $product->title : $product;
	            $message_to = $user->phone_number;
	            switch($message_type) {
	                case 1:
	                    $message = sprintf(lang('sms_low_stock'), $user->display_name, $item);
	                    break;
	                case 2:
	                    $message = sprintf(lang('sms_new_order'), $user->display_name, $item);
	                    break;
	                default:
	                    $message = sprintf(lang('sms_back_in_stock'), $user->display_name, $item);
	            }
	            $this->gc->send_sms($message, $message_to);
	       }
	    }
	}
	
	/**
	 * Update Stock Level
	 * @param number $id
	 * @param number $qty
	 * @param number $action
	 * @param string $module
	 * @return boolean
	 */
	public function stock($id, $qty = 1, $action = 1, $module = 'products') {
	    if($module != 'products') {
	        return false;
	    }
	    $item = $this->db->get_where($module, array('id' => $id))->row();
	    if($item) {
	        if($action == 0) {
	            $stock = $item->stock + $qty;
	        } else {
	            $stock = $item->stock - $qty;
	        }
	        $this->db->where('id', $id)->update($module, array('stock' => $stock));
	    }
	}
	
	/**
	 * Check validity of Product/Variant ID - Used in Ajax Method
	 * @param string $item
	 * @return boolean|object
	 */
	public function item($item) {
	    if(empty($item)) {
	        return false;
	    }
	    $item = explode('-', $item);
	    $product_id = (int) $item[0];
	    $variant_id = 0;
	    if(isset($item[1])) {
	        $variant_id = (int) $item[1];
	    }
	    if(!empty($variant_id)) {
	        $result = $this->db->where('id', $variant_id)->get($this->table_variants)->row();
	    } else if(!empty($product_id)) {
	        $result = $this->db->where('id', $product_id)->get('products')->row();
	    }
	    return $result;
	}
	
	/**
	 * Back in Stock Notification
	 * @param integer $id
	 * @param string $table
	 * @param array $info
	 */
	public function back_in_stock($id, $table = 'products', $info = array()) {
	    $this->load->model('users/user_model');
	    // Check whether this item is added in the Notify Me list
	    $where = array('deleted' => 0);
	    if($table == 'products') {
	        $where['product_id'] = $id;
	    } else if($table == 'product_variants') {
	        $where['variant_id'] = $id;
	    }
	    $users = $this->db->where($where)->get('product_notifications')->result();
	    if($users) {
	        // check the quantity level
	        if(empty($info)) {
	            $info = $this->db->where('id', $id)->get($table)->row();
	        }
	        if(!empty($info) && $info->stock > 0) {
	            $deleted_users = array();
	            // Get product information to display it to the user
	            if($table == 'products') {
	                $product = $info;
	            } else {
	                $product = $this->db->where('id', $info->product_id)->get('products')->row();
	            }
	            // send notification
	            foreach($users as $user) {	
	                $deleted_users[] = $user->id;
	                // send back in stock notification
	                // Email Notification only
	                if(empty($user->user_id)) {
    	                $user->display_name = null;
    	                $user->phone_number = null; 
	                } else { // logged in user id requested for notification 
	                    $notify_user = $this->user_model->get_user($user->user_id);
	                    if($notify_user) {
	                        $notify_user->email = $user->email;
	                        $user = $notify_user;
	                    }
	                }
	                // $this->load->library('ecommerce');
	                // $record = $this->ecommerce->template_fields(null, (object) $product, null, $user);
	                $product->customer_name = !empty($user->display_name) ? $user->display_name : lang('label_customer');
	                $product_url = gc_url_format('products', $product->slug, $product->id);
	                $product->url = anchor($product_url, $product_url, 'title="' . $product->title . '"');
	                $this->gemailer->sms_email_pdf('sms_email', array('template_table' => 'module_email_templates', 'module_table' => 'products', 'action_type' => 'back_in_stock', 'id' => $product->id, 'record' => $product, 'email_to' => $user->email));	                
	            }
	            // set deleted status to 0 to avoid future emails
	            if(!empty($deleted_users)) {
	                // changing deleted column status giving error, i.e. notifications are deleted permanently 
	                // $this->db->where('id IN (' . implode(',', $deleted_users). ')')->update('product_notifications', array('deleted' => 1));
	                $this->db->where('id IN (' . implode(',', $deleted_users). ')')->delete('product_notifications');
	            }
	        }
	    }
	}
	
	/**
	 * Convert Order Item to a multi-dimensional array
	 */
	public function order_item_to_array() {
	    $result = $this->orders_model->find_all();
	    if($result) {
	        foreach($result as $order) {
	            if(!empty($order->item)) {
	                $order_item = (array) json_decode($order->item ?? '');
	                if(isset($order_item['title']) && $order_item['description']) {
    	                $order_item = json_encode(array(
    	                    array(
    	                        'title' => $order_item['title'],
    	                        'description' => $order_item['description'],
    	                    )
    	                ));
    	                $this->db->where('id', $order->id)->update('orders', array('item' => $order_item));
	                }
	            }
	        }
	    }
	}
	
	/**
	 * Check if the item has an active refund policy or not
	 * @param string $order_date
	 * @param number $refund_duration
	 * @param number $refund_unit
	 * @return boolean
	 */
	public function has_refund_policy($order_date, $refund_duration = 0, $refund_unit = 0) {
	    if($refund_duration > 0) {
	        $today_date = date('Y-m-d H:i:s');
	        // get difference between order and booking date
	        $d1= new DateTime($order_date);
	        $d2= new DateTime($today_date);
	        $interval= $d1->diff($d2);
	        if(!empty($interval)) {
	            if($refund_unit == 0) { // Hours
	                $hours = ($interval->days * 24) + $interval->h;
	                if($refund_duration > $hours) {
	                    return true;
	                }
	            } else if($refund_duration > $interval->days) { // Days
	                return true;
	            }
	        }
	    }
	    return false;
	}	
	/**************************************
	 * END OF ECOMMERCE SECTION
	 **************************************/
	
	/**
	 * Display Summary
	 * @return array
	 */
	public function summary() {
	    if($this->auth->role_id() == 1) {
	        $column = 'order_total';
	        $table = $this->table_name;
	        $where = '';
	    } else {
	        $column = 'total';
	        $table = $this->table_items;
	        $where = " AND seller_id = " . $this->auth->user_id();
	    }
	    $sql = "SELECT SUM(" . $column . ") AS total_sales";
	    
	    // today sales
	    $sql .= ", (SELECT SUM(" . $column . ") FROM " . $this->db->dbprefix . $table;
	    $sql .= " WHERE YEAR(created_on) = YEAR(NOW()) AND MONTH(created_on) = MONTH(NOW()) AND DAY(created_on) = DAY(NOW()) AND status_id = 2" . $where .") AS today_sales";
	    
	    // this month sales
	    $sql .= ", (SELECT SUM(" . $column . ") FROM " . $this->db->dbprefix . $table;
	    $sql .= " WHERE MONTH(created_on) = MONTH(CURDATE()) AND YEAR(created_on)  =  YEAR(CURDATE()) AND status_id = 2" . $where .") AS month_sales";
	    
	    // yesterday sales
	    $sql .= ", (SELECT SUM(" . $column . ") FROM " . $this->db->dbprefix . $table;
	    $sql .= " WHERE created_on <= NOW() AND created_on >= DATE_ADD(Now(),interval - 1 day) AND status_id = 2" . $where .") AS yesterday_sales";
	    $sql .= " FROM " . $this->db->dbprefix . $table;
	    $sql .= " WHERE status_id = 2" . $where;
	    $query = $this->db->query($sql);
	    $result = $query->row();
	    foreach($result as $key => $value) {
	        $result->{$key} =  currency_format(empty($value) ? 0 : $value);
	    }
	    return $result;
	}
	
	
	/**
	 * Last Six Month Sales
	 * @param number $duration
	 * @return object
	 */
	public function sales_by_months($duration = 5) {
	    if($this->auth->role_id() == 1) {
	        $column = 'order_total';
	        $table = $this->table_name;
	        $where = '';
	    } else { // display items specific sales to the vendor
	        $column = 'total';
	        $table = $this->table_items;
	        $where = " AND seller_id = " . $this->auth->user_id();
	    }
	    $months = array();
	    // SELECT SUM(order_total) FROM `rs0d_orders` WHERE MONTH(created_on) = 08
	    $sql = "SELECT SUM(" . $column . ") AS total_sales";
	    for ($j = 0; $j <= $duration; $j++) {
	        $month = date("m", strtotime(" -$j month"));
	        $months['month_' . $month] = date("F Y", strtotime(" -$j month"));
	        $sql .= ", (SELECT SUM(" . $column . ") FROM " . $this->db->dbprefix . $table . " WHERE MONTH(created_on) = " . $month . " AND status_id = 2) AS month_" . $month;
	    }
	    $sql .= " FROM " . $this->db->dbprefix . $table;
	    $sql .= " WHERE status_id = 2" . $where;
	    $query = $this->db->query($sql);
	    $result = (array) $query->row();
	    unset($result['total_sales']);
	    $sales = array();
	    foreach($result as $key => $value) {
	        $sales[$months[$key]] = empty($value) ? 0 : $value;
	    }
	    // ksort($sales);
	    return $sales;
	}
	
	/**
	 * Total sales this month
	 * @return object
	 */
	public function this_month_sales_graph() {
	    $sql = "SELECT DATE(O.created_on) AS date, SUM(O.order_total) AS total_sales FROM " . $this->db->dbprefix . "orders O";
	    // $sql .= " INNER JOIN " . $this->db->dbprefix . "orders O ON O.order_id = INV.order_id";
	    $sql .= " WHERE MONTH(O.created_on) = MONTH(CURDATE()) AND YEAR(O.created_on)  =  YEAR(CURDATE())";
	    $sql .= " AND O.status_id = 2";
	    $sql .= " GROUP BY date";
	    $query = $this->db->query ( $sql );
	    $result = $query->result_array();
	    return $result;
	}
	
	/**************************************
	 * START OF CHECKOUT SECTION
	 **************************************/
	/**
	 * Common checkout function
	 * Checkout all variables are passed through a session variable named checkout_session
	 * @param number $user_id - ID of the User
	 * @param string $role - frontend/admin/mobile
	 */
    public function checkout($user_id = 0, $role = 'frontend') {
        $mobile = $role == 'mobile' ? true : false;
	    // $this->checkout_params($user_id, false, array(), true);
	    $cart = $this->checkout_params($user_id);
	    $checkout_items = $cart; // to avoid session for mobile app   
	    
	    // $cart = $this->cart_items($user_id, $user);
	    if(empty($cart)) {
	        if($mobile) {
	            return array('success' => false);
	        } else {
	            // redirect user to the home page
	            Template::set_message(lang('msg_cart_empty'), 'danger');
	            $previous_page = $this->session->userdata('previous_page');
	            $redirect_page = gc_url_format('members');
	            if(!empty($previous_page) && strpos($previous_page, 'ajax/') === false && strpos($previous_page, 'citySearch') === false) {
	                $redirect_page = $previous_page;
	            }
	            Template::redirect($redirect_page);
	        }
	    }
	    $order_total = 0;
	    // the cart has items, proceed further
	    $this->load->helper('array');
	    $this->load->model('users/user_model');
	    $this->lang->load('members/members');
	    $this->lang->load('fields/fields');
	    $this->load->model('locations/countries_model');
	    $this->load->model('locations/states_model');
	    $this->load->model('locations/cities_model');
	    $this->load->model('currencies/currencies_model');
	    $this->load->model('payment_gateways/payment_gateways_model');
	    $this->load->model('pages/pages_model');
	    $this->load->model('users/user_model');
	    $this->load->model('tax_rates/tax_rates_model');
	    $this->load->model('fields/fields_model');
	    if(!$mobile) {
	        Assets::add_css ( 'checkout.css' );
	        Assets::add_module_js('products', 'products.js');
	        Assets::add_js(array('jquery.payment.min.js'));
	        Assets::add_js ( 'checkout.js' );
	    }
	    $user_id = $user_id == 0 ? (int) $this->auth->user_id() : $user_id;
	    if(!empty($user_id)) { // the request might be generated through the admin panel
	        $user = $this->db->select('*, display_name AS name')->where('id', $user_id)->get('users')->row();
	        if($user && empty($user->city_id)) {
	            $user->country_iso = settings_item('adv.default_country');
	            $user->state_id = settings_item('adv.default_state');
	            $user->city_id = settings_item('adv.default_city');
	        }
	    } else {
	        $user = array(); // Allow user to checkout without registration
	    }
	    // Load Fields
	    $is_invoice = false;
	    $record = $user;
	    $field_role = 'frontend';
	    $form_fields = $this->fields_model->get_fields('orders', $field_role); // custom fields
	    $form_fields = $this->fields_model->filter_fields_by_type($form_fields, 'create_only');
	    $field_groups = $this->fields_model->field_groups($form_fields, array());
	    if(!$mobile) {
	       $form_fields = $this->fields_model->load_fields('orders', array(), $field_role, $user, 0, $mobile, $form_fields);
	    }
	    $view_data = array(
	        'wallet' => $user ? $user->wallets : 0, // checkout without login is also possible
	        'record' => $record,
	        'form_fields' => $form_fields,
	        'field_groups' => $field_groups
	    );
	    
	    $success = false;
	    
	    // Mobile - Redirect user to the order's page if the package price is zero
	    $price_zero = false;
	    
	    // Store ordered package, claim, product, advertisement and PYG (Pay As You Go) information in this field for Admin View
	    // Delete of listing_claims should not affect the view orders information
	    $item = '';
	    
	    // Subscription type is only valid in case of a ad package/product purchase, upgrade and renewal. This variable has three values
	    // 1 - NEW, 2 - UPGRADE & 3 - RENEWAL
	    // This variable is introduced for reports as adding it to the description was making query hard.
	    // The subscription type and the renew/upgrade order id is stored in a session variable, it is also product specific
	    // e.g. $ru_subscription = array($product_id => array('subscription_type' => 2, 'ru_order_id' => 'OD9921939394'));
	    /*
	     $ru_subscription = array(12 => array('subscription_type' => 2, 'ru_order_id' => 'OD9921939394'));
	     $this->session->set_userdata('checkout_ru_subscription', $ru_subscription);
	     */
	    // renew, upgrade order id
	    // $ru_order_id = !empty($params['ru_order_id']) ? $params['ru_order_id'] : 0;
	    $order_status = 0; // Default order status
	    
	    // Ecommerce - Products module
	    $allow_coupon = true;    
	    
	    // do not display wallet payment option, if wallet payment is in the checkout items list
	    if(array_key_exists('0-0-0-' . gc_module_id('wallets'), $checkout_items)) {
	        unset($view_data['wallet']);
	    }	    
	    Template::set($view_data);
	    // save method
	    if (!empty($_POST['save'])) {
	        if($user_id == 0) {
	            $this->form_validation->set_rules('email', 'lang:label_email', 'valid_email|required|trim|check_user_existence');
	            $this->form_validation->set_rules('password', 'lang:label_password', 'required|trim');
	        }
	        if($this->input->post('pay_using_wallet') && isset($user->wallets) && $user->wallets > $this->input->post('order_total')) {
	            $this->form_validation->set_rules('payment_method', 'lang:label_payment_methods', 'trim');
	        } else {
	            $this->form_validation->set_rules('payment_method', 'lang:label_payment_methods', 'trim|required');
	        }
	        $this->form_validation->set_rules('comments', 'lang:label_comments', 'trim|sanitize');
	        if(!$mobile) {
	            $this->form_validation->set_rules('chk_terms', 'lang:label_terms_conditions', 'required|trim');
	        }
	        $this->form_validation->set_rules('pay_using_wallet', 'lang:pay_using_wallet', 'trim');
	        $this->fields_model->fields_validation_rules($form_fields, 'order_customers', $field_role);
	        
	        // shipping fields
	        if(!empty($_POST['has_shipping_address'])) {
	            $this->form_validation->set_rules('shipping_name', 'lang:label_name', 'trim|required');
	            $this->form_validation->set_rules('shipping_phone_number', 'lang:label_phone_number', 'trim|required');
	            $this->form_validation->set_rules('shipping_address', 'lang:label_address', 'trim|required');
	            $this->form_validation->set_rules('shipping_postcode', 'lang:label_postcode', 'trim|required');
	            $this->form_validation->set_rules('shipping_country_iso', 'lang:label_country', 'trim|required');
	            $this->form_validation->set_rules('shipping_state_id', 'lang:label_state', 'trim|is_numeric|required');
	            $this->form_validation->set_rules('shipping_city_id', 'lang:label_city', 'trim|is_numeric|required');
	        }

	        // check validation rules
	        if ($this->form_validation->run() === TRUE) {
	            // checkout without login
	            // update the order_checkouts table with the user_id
	            // create login session also to avoid any confusion further
	            if($user_id == 0) { 
	                $user_id = $this->save_user();
	                $user = $this->get_user($user_id);
	                if(!$user) {
	                    if(!$mobile) {
	                        Template::redirect(site_url());
	                    } else {
	                       return array('success' => false);
	                    }
	                }	            
	            }
	            $item_names = array();
	            $order_summary = array(
	                'order_subtotal' => 0,
	                'order_discount' => 0,
	                'order_tax' => 0,
	                'order_total' => 0
	            );
	            $has_package = false; // Mobile App load menus based on users role
	            foreach($checkout_items as $checkout_item) {
	                if($checkout_item['module_id'] == 20) {
	                    $has_package = true;
	                }
	                $order_summary['order_subtotal'] += $checkout_item['subtotal'];
	                $order_summary['order_discount'] += $checkout_item['discount'];
	                $order_summary['order_tax'] += $checkout_item['tax'];
	                $order_summary['order_total'] += $checkout_item['total'];
	                $item_names[] = $checkout_item['item']['name'];
	            }
	            $order_summary['items'] = json_encode($item_names);
	            
	            // Delete inactive existing order if any unpaid by the same user for the same package
	            // $order_id is passed in case of user is paying an existing invoice, generate a new invoice
	            $this->delete_existing_order($user_id);
	            
	            // Generate a new order ID
	            // During bulk import (User assign package) same order numbers were being generated.
	            // To fix this issue, a four digit random number is added with the time.
	            $order_id = gc_json_settings('order.settings', 'order_prefix') . time() . rand(1000,9999);
	            
	            // save order customers
	            if(empty($_POST['has_shipping_address']) && !empty($_POST['shipping_address'])) {
	                // unset all shipping fields
	                foreach($_POST as $post_key => $post_value) {
	                    if(strpos($post_key, 'shipping_') !== false) {
	                        unset($_POST[$post_key]);
	                    }
	                }
	            }
	            $this->fields_model->save_module($form_fields, 'order_customers', $user_id, $field_role, 0, 'insert', array('order_id' => $order_id, 'user_id' => $user_id, 'email' => $user->email));
	            
	            // update information in the users table, so that it can be used later
	            $user_data = array(
	                'phone_number' => $this->input->post('phone_number'),
	                'address' => $this->input->post('address'),
	                'country_iso' => $this->input->post('country_iso'),
	                'state_id' => $this->input->post('state_id'),
	                'city_id' => $this->input->post('city_id'),
	                'postcode' => $this->input->post('postcode')
	            );
	            $this->db->where('id', $user_id)->update('users', $user_data);
	            
	            // Pay Using Wallet
	            // Save the wallet amount in the orders table for reporting and reversal purposes
	            $payment_method = $this->input->post('payment_method');
	            $checkout_amount = $order_summary['order_total'];
	            $wallets = 0;
	            $payment_method_name = null;
	            if(!empty($user->wallets) && $this->input->post('pay_using_wallet')) {
	                $wallet_cash = $user->wallets;
	                if($wallet_cash > $checkout_amount) {
	                    $wallets = $checkout_amount; // Amount spent from the Wallet
	                    $payment_method_name = 'Wallet';
	                    // Deduct the wallet amount from the user's balance
	                    $this->load->model('wallets/wallets_points_model');
	                    $this->wallets_points_model->update_cash('wallets', $user_id, 'orders', $checkout_amount, 'debit', null, 0, array('order_id' => $order_id));
	                    $checkout_amount = 0.00; // Make the order total zero
	                    $payment_method_name = 'wallet_transfer';
	                } else {
	                    $checkout_amount = $checkout_amount - $wallet_cash;
	                    $wallets = $wallet_cash;
	                }
	                $order_summary['wallets'] = $wallets;
	            }
	            
	            // Update invoice first, second and third expiration date as specified in settings
	            // generate invoice on order confirmation only instead of order placement
	            // $order_data['invoice_id'] = $this->save_invoices($order_id, $order_type, $user_id, $expires_on, $package);
	            
	            // The user is paying through an existing invoice
	            if($is_invoice) {
	                // Add order history as customer tried to pay invoice
	                $this->save_order_history($order_id, $params['status'], $params['status'], 5);
	            }
	            // Get payment method
	            if($checkout_amount > 0) {
    	            $payment_method = $this->db->where('id', (int) $this->input->post('payment_method'))->get('payment_gateways')->row();
    	            $payment_url = site_url('payment_gateways/' . strtolower($payment_method->name) . '?order_id=' . $order_id . ($mobile ? '&mobile=true' : ''));
    	            $payment_method_name = $payment_method->name;
	            }
	            
	            // save order and order history
	            $id = $this->save_orders($order_id, $user, $payment_method_name, 1, 'insert', $order_summary, $mobile);
	            
	            // Save items in the ordered_items table
	            // save them before order coupons as individual item commission will be calculated - Affiliate/Invitation
	            $this->save_order_items($order_id, $user->id, $checkout_items);
	            
	            // save order coupons
	            $this->save_order_coupons($order_id, $user, $order_summary);
	            
	            /**
	             * save invoice and redirect user to the payment gateways
	             * if price is greater than zero
	             */
	            if($checkout_amount > 0) {
	                // STRIPE Payment Gateway
	                // if credit card information is entered, pass it to the payment gateway using session
	                if($this->input->post('cardNumber')) {
	                    $cardExpiry = explode('/', $this->input->post('cardExpiry') ?? '');
	                    $cc_info = array(
	                        'number' => $this->input->post('cardNumber'),
	                        'expiryMonth' => (!empty($cardExpiry[0]) ? $cardExpiry[0] : ''),
	                        'expiryYear' => (!empty($cardExpiry[1]) ? $cardExpiry[1] : ''),
	                        'cvv' => $this->input->post('cardCVC')
	                    );
	                    if(!$mobile) {
	                       $this->session->set_userdata('cc_info', serialize($cc_info));
	                    }
	                }
	                
	                if(!$is_invoice) {
	                    $this->save_order_history($order_id, 1, 1, 4);
	                }
	                if(!$mobile) {
	                   Template::redirect($payment_url);
	                } else {
	                    $payment_method_name = strtolower($payment_method_name);
	                    $result = $this->orders_model->checkout_output($user_id, true);
	                    $result['success'] = true;
	                    $result['url'] = $payment_url;
	                    $result['order_id'] = $order_id; // Cordova Plugin Redirect
	                    $result['payment_method'] = strtolower($payment_method_name); // Cordova Plugin Redirect
	                    if($payment_method_name == 'banktransfer') {
	                        if($user->role_id == 4 && $has_package) {
    	                        // update the users role to the business owner
								if(GC_PRODUCT_ID == 1) {
	                            	$this->db->where('id', $user_id)->update('users', array('role_id' => 2));
								}
	                            $result['auth'] = true;
	                        }
	                        // wallet checkout
	                        $result['order_id'] = $id;
	                    }
	                    return $result;
	                }
	            } else { // Applied coupon/Wallet transaction made the price zero
	                // if($is_invoice) { } // Invoice Payment
	                // Save order history
	                $this->save_order_history($order_id, 1, 2, 4);
	                $order_info = $this->orders_model->find_by('order_id', $order_id);
	                // Confirm the order without redirecting the user to the checkout page
	                $this->payment_gateways_model->transaction_confirmed($order_info);
	            }
	        } else if($mobile) {
	            return array('success' => false, 'message' => validation_errors(), 'type' => 'danger');
	        }
	    }
	    $payment_gateways = $this->countries_model->_get_payment_methods((!empty($user->country_iso) ? $user->country_iso : null), true, true);
	    if(!$mobile) {
	        Template::set (array (
	            'allow_coupon' => $allow_coupon,
	            'user_id' => $user_id,
	            'user' => $user,
	            'role' => $role,
	            'items' => $cart,
	            'coupon' => $this->checkout_params($user_id, 'coupon'),
	            'currency_info' => $this->session->userdata('site_currency'),
	            'payment_gateways' => $payment_gateways,
	            'title' => lang('label_checkout'),
	            'toolbar_title' => lang('label_checkout')
	        ));	        
	        $layout = $this->gc->load_assets('members/checkout');
	        Template::set_view($layout['view']);
	        Template::render($this->auth->is_logged_in() ? 'member' : $layout['layout']);
	    } else {
	        return $this->checkout_output($user_id);
	    }
	}
	
	/**
	 * Member Checkout Page/Send Invoice and Quotation through the Admin Panel
	 * @param number $user_id
	 * @param object $user
	 * @param array $form_fields
	 * @param string $field_role
	 * @param boolean $is_invoice
	 * @param string $role
	 */
	private function prepare_order($user_id, $user, $form_fields, $field_role, $is_invoice = false, $role = 'admin') {
	    $item_names = array();
	    $order_summary = array(
	        'order_subtotal' => 0,
	        'order_discount' => 0,
	        'order_tax' => 0,
	        'order_total' => 0,
	    );
	    $checkout_items = $this->checkout_params($user_id);
	    foreach($checkout_items as $checkout_item) {
	        $order_summary['order_subtotal'] += $checkout_item['subtotal'];
	        $order_summary['order_discount'] += $checkout_item['discount'];
	        $order_summary['order_tax'] += $checkout_item['tax'];
	        $order_summary['order_total'] += $checkout_item['total'];
	        $item_names[] = $checkout_item['item']['name'];
	    }
	    $order_summary['items'] = json_encode($item_names);
	    
	    // Delete inactive existing order if any unpaid by the same user for the same package
	    // $order_id is passed in case of user is paying an existing invoice, generate a new invoice
	    if($role != 'admin') {
	        $this->delete_existing_order($user_id);
	    }
	    
	    // Generate a new order ID
	    // During bulk import (User assign package) same order numbers were being generated.
	    // To fix this issue, a four digit random number is added with the time.
	    $order_id = gc_json_settings('order.settings', 'order_prefix') . time() . rand(1000,9999);
	    
	    // save order customers
	    $this->fields_model->save_module($form_fields, 'order_customers', $user_id, $field_role, 0, 'insert', array('order_id' => $order_id, 'user_id' => $user_id, 'email' => $user->email));
	    
	    // update information in the users table, so that it can be used later
	    if($role != 'admin') {
	        $user_data = array(
	            'phone_number' => $this->input->post('phone_number'),
	            'address' => $this->input->post('address'),
	            'country_iso' => $this->input->post('country_iso'),
	            'state_id' => $this->input->post('state_id'),
	            'city_id' => $this->input->post('city_id'),
	            'postcode' => $this->input->post('postcode')
	        );
	        $this->db->where('id', $user_id)->update('users', $user_data);
	    }
	    
	    // Pay Using Wallet
	    // Save the wallet amount in the orders table for reporting and reversal purposes
	    $payment_method = $this->input->post('payment_method');
	    $checkout_amount = $order_summary['order_total'];
	    if($role != 'admin') {
	        $wallets = 0;
	        if(!empty($user->wallets) && $this->input->post('pay_using_wallet')) {
	            $wallet_cash = $user->wallets;
	            if($wallet_cash > $checkout_amount) {
	                $wallets = $checkout_amount; // Amount spent from the Wallet
	                $checkout_amount = 0.00; // Make the order total zero
	                $payment_gateway = 'Wallet';
	            } else {
	                $checkout_amount = $checkout_amount - $wallet_cash;
	                $wallets = $wallet_cash;
	            }
	        }
	    }
	    
	    // Update invoice first, second and third expiration date as specified in settings
	    // generate invoice on order confirmation only instead of order placement
	    // $order_data['invoice_id'] = $this->save_invoices($order_id, $order_type, $user_id, $expires_on, $package);
	    
	    // The user is paying through an existing invoice
	    if($is_invoice) {
	        // Add order history as customer tried to pay invoice
	        $this->save_order_history($order_id, $params['status'], $params['status'], 5);
	    }
	    
	    // Get payment method
	    $payment_method = $this->db->where('id', (int) $this->input->post('payment_method'))->get('payment_gateways')->row();
	    $payment_url = site_url('payment_gateways/' . strtolower($payment_method->name) . '?order_id=' . $order_id . ($mobile ? '&mobile=true' : ''));
	    
	    // save order and order history
	    $id = $this->save_orders($order_id, $user, $payment_method->name, 1, 'insert', $order_summary);
	    
	    // Save items in the ordered_items table
	    // save them before order coupons as individual item commission will be calculated - Affiliate/Invitation
	    $this->save_order_items($order_id, $user->id);
	    
	    // save order coupons
	    $this->save_order_coupons($order_id, $user, $order_summary);
	    
	    if($role != 'admin') {
	        /**
	         * save invoice and redirect user to the payment gateways
	         * if price is greater than zero
	         */
	        if($checkout_amount > 0) {
	            // STRIPE Payment Gateway
	            // if credit card information is entered, pass it to the payment gateway using session
	            if($this->input->post('cardNumber')) {
	                $cardExpiry = explode('/', $this->input->post('cardExpiry') ?? '');
	                $cc_info = array(
	                    'number' => $this->input->post('cardNumber'),
	                    'expiryMonth' => (!empty($cardExpiry[0]) ? $cardExpiry[0] : ''),
	                    'expiryYear' => (!empty($cardExpiry[1]) ? $cardExpiry[1] : ''),
	                    'cvv' => $this->input->post('cardCVC')
	                );
	                $this->session->set_userdata('cc_info', serialize($cc_info));
	            }
	            
	            if(!$is_invoice) {
	                $this->save_order_history($order_id, 1, 1, 4);
	            }
	            Template::redirect($payment_url);
	        } else { // Applied coupon/Wallet transaction made the price zero
	            // if($is_invoice) { } // Invoice Payment
	            // Save order history
	            $this->save_order_history($order_id, 1, 2, 4);
	            $order_info = $this->orders_model->find_by('order_id', $order_id);
	            // Confirm the order without redirecting the user to the checkout page
	            $this->payment_gateways_model->transaction_confirmed($order_info);
	        }
	    }
	}
	
	/**
	 * Cart Items to be displayed on the checkout page
	 * @param number $user_id
	 * @param array $user
	 * @return array|number[]|string[]|string[][]
	 */
	public function cart_items($user_id = 0, $user = array(), $mobile = false, $zero_checkout = false) {
	    $this->load->model('tax_rates/tax_rates_model');
	    $this->load->model('users/user_model');
	    $this->load->model('packages/packages_model');
	    
	    // Default Values
	    $subscription_type = 1;
	    $ru_order_id = null;
	    
	    if(empty($user_id)) {
	        $user_id = (int) $this->auth->user_id();
	    }
	    if(empty($user)) {	        
	        $user = $this->user_model->get_user($user_id);
	    }
	    
	    // product items are in the cart
	    $cart = $this->checkout_params($user_id);
	    $checkout_type = (array) $this->session->userdata('checkout_type');
	    
	    /**
	     * Tax rates are calculated based on the user selected country.
	     * For non logged_in users, where country is not selected, the default tax_rates will be applied.
	     * Wallet will have no tax rate.
	     * Custom payments will have through settings.
	     */
	    $tax_country_iso = $this->checkout_params($user_id, 'country');
	    $tax_country_iso = empty($tax_country_iso) ? (!empty($user->country_iso) ? $user->country_iso : null) : $tax_country_iso;
	    if(!empty($_POST['country_iso']) && $_POST['country_iso'] != $tax_country_iso) {
	        $tax_country_iso = $_POST['country_iso'];
	        $this->checkout_params($user_id, false, array('country' => $tax_country_iso));
	        if(!$mobile) {	            
	           $this->session->set_userdata('visitor_country_iso', $tax_country_iso);
	        }
	    }
	    if(!empty($tax_country_iso)) {
	        $tax_rate = $this->tax_rates_model->get_tax_rate($tax_country_iso);
	    }
	    if(empty($tax_rate)) {
	        $tax_rate = $this->tax_rates_model->get_tax_rate(1, 'is_default');
	    }
	    $tax_applicable_modules = array();
	    $tax_percent = 0;
	    $tax_inclusive = 1;
	    if(!empty($tax_rate)) {
	        $tax_applicable_modules = !empty($tax_rate->applicable_modules) ? json_decode($tax_rate->applicable_modules, true) : array();
	        $tax_percent = $tax_rate->tax_percent;
	        $tax_inclusive = $tax_rate->tax_inclusive == 1 ? true : false;
	    }
	    if(!empty($cart)) {
	        $this->load->library('ecommerce');
	        // combos and upgrade discounts
	        // also save combo and upgrade discount information in the order items table params
	        $additional_product_discounts = array();//$this->additional_product_discounts($cart);
	        $total_savings = 0;
	        $amount = 0;
	        $product_module_id = gc_module_id('products');
	        // verify stock level of each item
	        foreach($cart as $item_id => $cart_item) {
	            $new_status_id = 1; // zero checkout new status id
	            $no_coupons = false; // do not allow coupons on specific items
	            $tax_settings = array();
	            if(empty($cart_item['module_id']) || $cart_item['module_id'] == 0) {
	                $module_name = '';
	                unset($cart[$item_id]);
	                continue;
	            } else {
	                $module_name = gc_module_name($cart_item['module_id']);
	            }
	            $cart_item_module = gc_module_name($cart_item['module_id']);
	            if($cart_item['module_id'] != 2 && !empty($tax_applicable_modules[$module_name])) { // get tax settings of each module
	                // get module specific tax settings	                
	                $tax_settings = array('tax_percent' => $tax_percent, 'tax_inclusive' => $tax_inclusive);
	            }
	            switch($cart_item_module) {
	                case 'products':	            
        	            $id_item = explode('-', $item_id ?? '');
        	            $product_id = (int) $id_item[0];
        	            $variant_id = 0;
        	            $expires_on = null;
        	            $rules_discount = 0;
        	            $additional_discount = 0; // Combo/Upgrade Discount
        	            $additional_discount_type = 0; // 1 - Combo, 2 - Upgrade, 3 - Rules Discount
        	            if(isset($id_item[1])) {
        	                $variant_id = (int) $id_item[1];
        	            }
        	            $item_info = $this->ecommerce->get_product($product_id, false, true, true);
        	            // product does not exist, remove item from cart
        	            if(!$item_info) {
        	                unset($cart[$item_id]);
        	                break;
        	            }
        	            
        	            // check max_qty here to avoid any ambiguity
        	            if(!empty($item_info->max_qty) && $cart_item['qty'] > $item_info->max_qty) {
        	                $cart_item['qty'] = $item_info->max_qty;
        	                $cart[$item_id]['qty'] = $item_info->max_qty;
        	            }
        	            
        	            // Optional Services
        	            // Display optional services on the checkout page. As an admin, you must know which product this optional service is purchased.
        	            // An optional item will be displayed in the same way, but in order to save relation, a new field relation_item_id is added in the order_items table.
        	            // This field will indicate which product this optional service is associated with.
        	            // note: the item must have a stock value and must be active
        	            $optional_items = array(); // $this->products_model->checkout_optional_items($item_info->optional_items);
        	            
        	            $item_stock = $item_info->stock;
        	            $item_offer_price = $item_info->offer_price;
        	            
        	            // the product is either for renewal or upgrade
        	            $checkout_type_key = 1 . '_' . $product_id;
        	            $cart[$item_id]['subscription_type'] = 1;
        	            $cart[$item_id]['ru_order_id'] = null;
        	            $cart[$item_id]['ru_order_item_id'] = 0;
        	            if(array_key_exists($checkout_type_key, $checkout_type)) {
        	                $checkout_type_value = $checkout_type[$checkout_type_key];
        	                if(isset($checkout_type_value[2])) {
        	                    if($checkout_type_value[2] == 'renew') {
        	                        // use the renewal price as the offer price
        	                        $item_info->offer_price = !empty($item_info->renewal_price) ? $item_info->renewal_price : $item_info->offer_price;
        	                        // The renewal price is applicable to one item only
        	                        $item_info->max_qty = 1;
        	                        $cart[$item_id]['ru_order_item_id'] = $checkout_type_value[0];
        	                        $cart[$item_id]['ru_order_id'] = $checkout_type_value[1];
        	                        $cart[$item_id]['subscription_type'] = 3;
        	                    } else if($checkout_type_value[2] == 'upgrade') {
        	                        if(!empty($checkout_type_value[3])) { // product id which to be upgraded
        	                            $older_product = $this->ecommerce->get_product($checkout_type_value[3], false);
        	                            $product_upgrade = $this->ecommerce->get_product_upgrade(0, $checkout_type_value[3], $item_info->id);
        	                            if($older_product) {
        	                                $product_upgrade = $this->ecommerce->upgrade_price($item_info, $product_upgrade);
        	                                if($product_upgrade->price > 0) {
        	                                    $item_info->offer_price = $product_upgrade->price;
        	                                    // $additional_discount = $product_upgrade->value;
        	                                    // $additional_discount_type = 2;
        	                                    // The upgrade price is applicable to one item only
        	                                    $item_info->max_qty = 1;
        	                                }
        	                                $cart[$item_id]['ru_order_item_id'] = $checkout_type_value[0];
        	                                $cart[$item_id]['ru_order_id'] = $checkout_type_value[1];
        	                                $cart[$item_id]['subscription_type'] = 2;
        	                            }
        	                        }
        	                    }
        	                }
        	            }
        	            
        	            // the amount used for further calculation
        	            $calculation_price = !empty($item_info->offer_price) ? $item_info->offer_price : $item_info->price;
        	            
        	            // commissions calculations
        	            $commissions = json_decode($item_info->commissions ?? '', true);
        	            
        	            // product commission in the package, used during order status change
        	            $commission = !empty($commissions['products']['value']) ? (int) $commissions['products']['value'] : 0;
        	            $commission_type = !empty($commissions['products']) ? (!empty($commissions['products']['type']) ? $commissions['products']['type'] : $commissions['products']['type']) : 'P';
        	            
        	            $item_cover = !empty($item_info->cover) ? $item_info->cover : '';
        	            if(!empty($variant_id)) {
        	                $variant_info = $this->ecommerce->get_variant($variant_id);
        	                
        	                $item_cover = $cart[$item_id]['cover'];
        	                // Variant does not exist, remove item from cart
        	                if(!$variant_info) {
        	                    unset($cart[$item_id]);
        	                    break;
        	                }
        	                $item_stock = $variant_info->stock;
        	                if(!empty($variant_info->price)) {
        	                    $calculation_price = $variant_info->price;
        	                }
        	            }
        	            
        	            // upgrade and combo discounts
        	            if(array_key_exists($product_id, $additional_product_discounts)) {
        	                $discount_value_type = $additional_product_discounts[$product_id]['value_type'];
        	                $discount_value = $additional_product_discounts[$product_id]['value'];
        	                if(!empty($discount_value)) {
        	                    $additional_discount_type = $additional_product_discounts[$product_id]['type'];
        	                    if($discount_value_type == 'F') {
        	                        $calculation_price = $discount_value;
        	                    } else {
        	                        $additional_discount = (($calculation_price * (float) $discount_value) / 100);
        	                        $calculation_price = $calculation_price - $additional_discount;
        	                    }
        	                    // the combo has a discount value that can be applied to one product only
        	                    $cart_item['qty'] = 1;
        	                    $cart[$item_id]['qty'] = 1;
        	                    $item_info->max_qty = 1;
        	                }
        	            }
        	            
        	            $item_offer_price = $calculation_price;
        	            // Not enough stock, remove item from cart
        	            if($item_stock == 0) {
        	                unset($cart[$item_id]);
        	                break;
        	            }
        	            
        	            // Check if the item is in Wishlist or not
        	            $cart[$item_id]['wishlist'] = 0;
        	            $wishlist_count = $this->db->where(array('user_id' => $user_id, 'product_id' => $product_id, 'variant_id' => $variant_id))->count_all_results('product_wishlists');
        	            if($wishlist_count == 1) {
        	                $cart[$item_id]['wishlist'] = 1;
        	            }
        	            
        	            // Decrease quantity, if not enough stock
        	            if($item_stock < $cart_item['qty']) {
        	                $cart[$item_id]['qty'] = $cart_item['qty'] - $item_stock;
        	            }
        	            $cart[$item_id]['stock'] = $item_stock;
        	            
        	            // Savings on Item Purchase
        	            $price = $item_info->price;
        	            $savings = 0;
        	            if(!empty($calculation_price)) {
        	                if($price > $calculation_price) {
        	                    $savings = $cart_item['qty'] * ($price - $calculation_price);
        	                    $total_savings = $total_savings + $savings;
        	                }
        	                $price = $calculation_price;
        	            }
        	            // original amount
        	            $item_amount = $cart_item['qty'] * $price;
        	            $amount = $amount + $item_amount;
        	            
        	            // Rules Discount
        	            /*
        	            $rule_discount = $this->ecommerce->check_pricing_rules($product_id, $user_id, $cart[$item_id]['qty']);
        	            if(!empty($rule_discount)) {
        	                $rules_discount = (($item_amount * (float) $rule_discount) / 100);
        	                $item_amount = $item_amount - $rules_discount;
        	                $calculation_price = $calculation_price - $rules_discount;
        	                // $savings += $rules_discount;
        	            }*/
        	            
        	            // Tax calculation
        	            $subtotal_order = $item_amount;
        	            $total_order = $item_amount;
        	            $item_tax = 0;
        	            $item_info->tax_settings = 1;
        	            $item_tax_percent = 0;
        	            $inclusive_tax = false;
        	            if(empty($item_info->tax_settings)) {
        	                // get default tax rates
        	                $default_tax_rate = $this->gc->get('tax_rates', array('where' => 'is_default = 1 AND status = 1'));
        	                if($default_tax_rate) {
        	                    $applicable_modules_tax = json_decode($default_tax_rate->applicable_modules ?? '', true);
        	                    if(!empty($applicable_modules_tax[$cart_item_module])) {
        	                        $item_tax_percent = $default_tax_rate->tax_percent;
        	                        $inclusive_tax = !empty($default_tax_rate->tax_inclusive) ? true : false;
        	                    }
        	                }
        	            } else if($item_info->tax_settings == 3) { // tax not applicable
        	                $item_tax_percent = 0;
        	                $inclusive_tax = false;
        	            } else {
        	                $item_tax_percent = $tax_percent;
        	                if($item_info->tax_settings == 0) {
        	                    $inclusive_tax = $tax_inclusive;
        	                } else if($item_info->tax_settings == 1) {
        	                    $inclusive_tax = true;
        	                } else {
        	                    $inclusive_tax = false;
        	                }
        	                // The Tax Slab is enabled on this item
        	                if(!empty($item_info->tax_slab)) {
        	                    $tax_slab = $this->tax_rates_model->get_tax_slab($item_info->tax_slab);
        	                    if($tax_slab) {
        	                        $item_tax_percent = $tax_slab->value;
        	                    }
        	                }
        	            }
        	            if(!empty($item_tax_percent)) {
        	                // Item Tax Calculation on Quantity
        	                if($inclusive_tax) {	// Tax Inclusive
        	                    $tax_item = (100 + $item_tax_percent) / 100;
        	                    $subtotal_order = decimal_convert($item_amount/$tax_item);
        	                    $item_tax = $item_amount - $subtotal_order;
        	                } else {
        	                    $item_tax = decimal_convert(($item_amount * $item_tax_percent)/100);
        	                    $total_order = $item_amount + $item_tax;
        	                }
        	            }
        	            // $item_cover = gc_image_url($item_cover);
        	            // Store item total, subtotal and tax to avoid unnecessary calculations again and again
        	            $cart[$item_id]['price'] = $item_info->price;
        	            $cart[$item_id]['offer_price'] = $item_offer_price;
        	            $cart[$item_id]['item_qty_price'] = $item_info->price * $cart_item['qty'];
        	            $cart[$item_id]['savings'] = $savings;
        	            $cart[$item_id]['module_id'] = $product_module_id;
        	            $cart[$item_id]['row_id'] = $product_id;
        	            $cart[$item_id]['variant_id'] = $variant_id;
        	            $cart[$item_id]['seller_id'] = $item_info->user_id;
        	            $cart[$item_id]['total'] = $total_order;
        	            $cart[$item_id]['subtotal'] = $subtotal_order;
        	            $cart[$item_id]['tax'] = $item_tax;
        	            // $cart[$item_id]['tax_slab'] = $item_info->tax_slab;
        	            $cart[$item_id]['tax_percent'] = $item_tax_percent;
        	            $cart[$item_id]['tax_inclusive'] = $inclusive_tax;
        	            $cart[$item_id]['tax_country_iso'] = $tax_country_iso;
        	            $cart[$item_id]['tax_settings'] = $item_info->tax_settings;
        	            $cart[$item_id]['allow_coupon'] = true;
        	            $cart[$item_id]['max_qty'] = 50000;
        	            $cart[$item_id]['discount'] = 0;
        	            $cart[$item_id]['discount_value'] = 0;
        	            $cart[$item_id]['refund_duration'] = $item_info->refund_duration;
        	            $cart[$item_id]['cover'] = $item_cover;
        	            $cart[$item_id]['commission'] = $commission;
        	            $cart[$item_id]['commission_type'] = $commission_type;
        	            $cart[$item_id]['item'] = array(
        	                'id' => $item_info->id,
        	                'name' => $item_info->title,
        	                'slug' => $item_info->slug,
        	                'cover' => $item_cover,
        	                'attributes' => isset($cart_item['attributes']) ? $cart_item['attributes'] : '',
        	                'variant_id' => $variant_id,
        	                'description' => $item_info->description,
        	                'rules_discount' => $rules_discount,
        	                'additional_discount' => $additional_discount,
        	                'additional_discount_type' => $additional_discount_type,
        	                'optional_items' => $optional_items,
        	                'relation_item_info' => !empty($cart_item['relation_item_info']) ? $cart_item['relation_item_info'] : ''
        	            );
        	            // subscription
        	            if(!empty($item_info->duration) || !empty($item_info->fixed_date)) {
        	                $expires_on = $this->calculate_expires_on($item_info, $cart[$item_id]['subscription_type'], $cart[$item_id]['ru_order_item_id'], $cart_item_module);
        	                $cart[$item_id]['expires_on'] = $expires_on;
        	            }
        	            // invoice reminders are saved in the order_items table itself to avoid unnecessary MySQL JOINS
        	            $cart[$item_id]['reminders'] = $this->save_order_reminders($item_info, $expires_on);
        	            // $cart[$item_id]['reminder_notifications'] = $item_info->reminder_notifications;
        	            $cart[$item_id]['user_id'] = $user_id; // pricing rules discount
        	        break;
	            case 'ads': // Ad Purchase           
	               $ad_id = $cart_item['row_id'];
	               if(!empty($ad_id)) {
	                   $tax_settings = array();
	                   if(!empty($tax_applicable_modules['ads'])) {
	                       $tax_settings = array('tax_percent' => $tax_percent, 'tax_inclusive' => $tax_inclusive);
	                   }
	                   $this->load->model('ads/ads_model');
	                   $order_module_id = gc_module_id('ads');
	                   $ad = $this->ads_model->get_ad($ad_id, false, false);
	                   if($ad) {
	                       if(isset($ru_subscription) && array_key_exists($ad->id, $ru_subscription)) {
	                           $subscription_type = $ru_subscription[$ad->id]['subscription_type'];
	                           $ru_order_id = $ru_subscription[$ad->id]['ru_order_id'];
	                       }
	                       $ad_extra = $tax_settings;
	                       $ad_title = $ad->package_title;
	                       $cover = '';
	                       if($ad->module_id == gc_module_id('banners') && !empty($ad->image)) { // get banner image
	                           if(!empty($ad->url)) {
	                               $ad_title .= '<br />(<small>' . anchor($ad->url, $ad->url, 'target="_blank"') . '</small>)';
	                           }
	                           $cover = gc_image_url($ad->image, false, false, 'banners');
	                       } else { // get module image
	                           $module_name = gc_module_name($ad->module_id);
	                           if(in_array($module_name, gc_modules())) {
	                               $ad_module_info = $this->db->where('id', $ad->row_id)->get($module_name)->row();
	                               if($ad_module_info) {
	                                   $ad_title .= '<br />(<small>' . $ad_module_info->title . '</small>)';
	                                   $cover = $ad_module_info->cover;
	                               }
	                           }
	                       }
	                       if(!empty($cover)) {
	                           $ad_extra['cover'] = $cover;
	                       }
	                       // invoice reminders are saved in the order_items table itself to avoid unnecessary MySQL JOINS
	                       $cart[$item_id] = $this->prepare_cart_item($order_module_id, $ad->id, $ad->price, $ad_title, $ad->package_description, 0, $subscription_type, $ad->expires_on, $ru_order_id, $ad_extra);
	                       $cart[$item_id]['reminders'] = $this->save_order_reminders($ad, $ad->expires_on);
	                       if(empty($cart[$item_id]['params']) || count($cart[$item_id]['params']) == 0) {
	                           $cart[$item_id]['params'] = array('package_id' => $ad->package_id);
	                       } else {
	                           $cart[$item_id]['params']['package_id'] = $ad->package_id;
	                       }
	                       $cart[$item_id]['params'] = $cart[$item_id]['params'];
	                       $cart[$item_id]['reminder_notifications'] = 1;
	                       $new_status_id = $ad->order_status == 1 ? 2 : 1;
	                   }
	               }
	               break;
	            case 'leads': // Lead Purchase
	                $lead_id = $cart_item['row_id'];
	                if(!empty($lead_id)) {
	                    $this->lang->load('leads/leads');
	                    $this->load->model('leads/leads_model');
	                    $order_module_id = gc_module_id('leads');
	                    $lead = $this->leads_model->get_leads_view($lead_id);
	                    if($lead) {
	                        $lead_title = lang('lead') . ':: ' . $lead->id . ' - ' . $lead->name;
	                        $lead_description = "<ul>";
	                        if(!empty($lead->category)) {
	                           $lead_description .= "<li>" . lang('label_category') . ' - ' . $lead->category ."</li>";
	                        }
	                        $lead_description .= "<li>" . lang('label_city') . ' - ' . $lead->city ."</li>";
	                        $lead_description .= "<li>" . truncate_text($lead->description, 750) ."</li>";
	                        $lead_description .= "</ul>";
	                        // Lead Extra
	                        $lead_extra = $tax_settings;
	                        $lead_extra['cover'] = gc_site_image('leads', false);
	                        // invoice reminders are saved in the order_items table itself to avoid unnecessary MySQL JOINS
	                        $cart[$item_id] = $this->prepare_cart_item($order_module_id, $lead->id, $lead->price, $lead_title, $lead_description, $lead->price, $subscription_type, null, $ru_order_id, $lead_extra);
	                    }
	                }
	                break;
	            case 'packages':	           
	                // Package Purchase
	                $checkout_package_id = $cart_item['row_id'];
	                if(!empty($checkout_package_id)) {
	                    /**
	                     * Check whether the same package is purchased by the same user or not
	                     * if purchased, check whether it is for multisale
	                     * if yes allow user to buy this package again
	                     * if no check whether request is for renewal or not
	                     * if yes allow package to renew
	                     */
	                    $package_info = $this->packages_model->get_package($checkout_package_id);
	                    if($package_info) {
	                        if(!empty($cart_item['subscription_type'])) {
	                            $subscription_type = $cart_item['subscription_type'];
	                        }
	                        $order_module_id = gc_module_id('packages');
	                        $multisale = $this->get_user_orders(1, $user_id, array('package_id' => $checkout_package_id));
	                        if(($multisale->count >= 1) && ($package_info->multisale == 0) && ($subscription_type != 3)) {
	                            // the user cannot purchase a package twice
	                            unset($cart[$item_id]);
	                        } else {
	                            $package_extra = $tax_settings;
	                            $package_extra['cover'] = !empty($package_info->trusted_seal) ? gc_image_url($package_info->trusted_seal, false) : '';
	                            $amount = !empty($package_info->offer_price) ? $package_info->offer_price : $package_info->price;
	                            
	                            // package renewal and upgrades
	                            if(!empty($cart_item['order_id'])) {
	                                $ru_order_id = $cart_item['order_id'];
	                            }
	                            if(!empty($cart_item['order_item_id'])) {
	                                $package_extra['order_item_id'] = $cart_item['order_item_id'];
	                            }
	                            
	                            // calculate expires on
	                            $expires_on = null;
	                            if(!empty($package_info->duration) || !empty($package_info->fixed_date)) {
	                                $expires_on = $this->calculate_expires_on($package_info, $subscription_type, $ru_order_id, $cart_item_module);
	                            }
	                            
	                            $cart[$item_id] = $this->prepare_cart_item($order_module_id, $checkout_package_id, $package_info->price, $package_info->title, $package_info->description, $package_info->offer_price, $subscription_type, $expires_on, $ru_order_id, $package_extra);
	                            // invoice reminders are saved in the order_items table itself to avoid unnecessary MySQL JOINS
	                            $cart[$item_id]['reminders'] = (array) $this->save_order_reminders($package_info, $expires_on);
	                            $package_params = array();
	                            $package_params['savings'] = calculate_savings($package_info->price, $package_info->offer_price);
	                            $cart[$item_id]['params'] =  $package_params;
	                            $new_status_id = $package_info->order_status;
	                        }
	                    }
	                }
	                break;
	            case 'wallets':
	                // Wallets
	                // Add Money To Wallet
	                // Pass amount as a parameter in case of Wallet
	                $wallet_amount = $cart_item['price'];
	                if(!empty($wallet_amount)) {
	                    $tax_settings = array();
	                    if(!empty($tax_applicable_modules['wallets'])) {
	                        $tax_settings = array('tax_percent' => $tax_percent, 'tax_inclusive' => $tax_inclusive);
	                    }
	                    $item_title = lang('item_title_wallet'); // lang('item_description_wallet') . 
	                    $item_description = (!empty($user) ? $user_id . ' - ' . $user->display_name . ' (' . (!empty($user->email) ? $user->email : $user->phone_number) . ')' : '');
	                    $order_module_id = gc_module_id('wallets');
	                    $wallet_extra = $tax_settings;
	                    $wallet_extra['cover'] = gc_site_image('wallet-payment');
	                    $cart[$item_id] = $this->prepare_cart_item($order_module_id, $user_id, $wallet_amount, $item_title, $item_description, 0, 1, null, null, $wallet_extra);
	                }
	                break;
	            case 'bookings': // Bookings
	                $booking_id = $cart_item['row_id'];
	                if(!empty($booking_id)) {
	                    // get information using the booking id
	                    $this->db->join('booking_forms BF', 'BF.id = bookings.form_id', 'INNER');
	                    $this->db->join('booking_allotments BA', 'BA.id = bookings.allotment_id', 'INNER');
	                    $booking = $this->db->select('bookings.*, BF.title AS form_title, BF.commission, BA.allow_coupon, BA.tax_percent, BA.tax_inclusive, BA.refund_duration, BA.refund_unit, BA.seats_limit, BA.user_id AS seller_id, BA.seats, BA.seats_unit, BA.module_id AS parent_module_id, BA.row_id AS parent_row_id')->where('bookings.id', $booking_id)->get('bookings')->row();
	                    if($booking) {
	                        $qty = empty($cart[$item_id]['qty']) ? 1 : $cart[$item_id]['qty'];
	                        $order_module_id = gc_module_id('bookings');
	                        if($booking->allow_coupon == 0) {
	                            $no_coupons = true; // Do not allow coupon
	                        }
	                        $item_amount = 0;
	                        $form_data = json_decode($booking->form_data ?? '');
	                        if(!empty($form_data)) {
	                            $booking_description = '<ul class="list-inline">';
	                            $parent_module_name = gc_module_name($booking->parent_module_id);
    	                        $module_row_info = $this->db->where('id', $booking->parent_row_id)->get($parent_module_name)->row();
    	                        if($module_row_info) {
    	                            $booking_description .= '<li>' .  lang($parent_module_name) . ': ' . anchor(gc_url_format($parent_module_name, $module_row_info->slug, $module_row_info->id, (!empty($module_row_info->city) ? $module_row_info->city : '')), $module_row_info->id . ' - ' . $module_row_info->title, 'target="_blank"') . '</li>';
    	                        }
	                            foreach($form_data as $booking_data) {
	                                if($booking_data->name == 'pricing') {
	                                    $item_amount = $booking_data->value;
	                                }
	                                $booking_description .= '<li>' . $booking_data->label . ': ' . $booking_data->value .  '</li>';
	                            }
	                            $booking_description .= '</ul>';
	                        }
	                        $this->load->model('bookings/bookings_model');
							if(!empty($booking->seats_unit) && !empty($booking->booking_date)) { // daily limit is set
	                            // get booking date seats
	                            $booking_date = explode(' ', $booking->booking_date ?? '');
	                            $total_booked_seats = (int) $this->bookings_model->get_booked_seats($booking->allotment_id, 2, 0, "B.booking_date='" . $booking_date[0] . "'");
	                        } else { // check all time limit
	                            $total_booked_seats = (int) $this->bookings_model->get_booked_seats($booking->allotment_id); // get total booked seats
	                        }
	                        $stock = 0;
	                        if($booking->seats > $total_booked_seats) {
	                            $available_seats = $booking->seats - $total_booked_seats;
	                            if($booking->seats_limit == 0) {
	                                $stock = $available_seats > 10 ? 10 : $available_seats; // a user can book max 10 seats at a time
	                            } else {
	                                // get booked seats by the same user
	                                if(!empty($booking->seats_unit) && !empty($booking->booking_date)) { // daily limit is set
        	                            // get booking date seats
        	                            $booking_date = explode(' ', $booking->booking_date ?? '');
        	                            $booked_seats = (int) $this->bookings_model->get_booked_seats($booking->allotment_id, 2, $user_id, "B.booking_date='" . $booking_date[0] . "'");
    	                            } else { // check all time limit
        	                            $booked_seats = (int) $this->bookings_model->get_booked_seats($booking->allotment_id, 2, $user_id); // get total booked seats
        	                        }
	                                $seats_limit = $available_seats > $booked_seats ? $booking->seats_limit : $available_seats;
	                                if($booked_seats == 0) {
	                                    $stock = $seats_limit;
	                                } else if($seats_limit > $booked_seats) {
	                                    $stock = $seats_limit - $booked_seats;
	                                }
	                            }
	                        }
	                        if($stock > 0) {
    	                        $booking_tax_settings = array(
    								'seller_id' => $booking->seller_id,
    	                            'tax_percent' => $booking->tax_percent,
    	                            'tax_inclusive' => $booking->tax_inclusive == 1 ? true : false,
    	                            'max_qty' => $stock,
    	                            'no_coupons' => $no_coupons,
    	                            'cover' => gc_site_image('bookings'),
    	                            'commission' => $booking->commission,
    	                            'commission_type' => 'P'
    	                        );
    	                        $item_amount = $qty * $item_amount;
    	                        $cart[$item_id] = $this->prepare_cart_item($order_module_id, $booking->id, $item_amount, $booking->form_title, $booking_description, $item_amount, $subscription_type, null, $ru_order_id, $booking_tax_settings);
    	                        $booking_params = array(
    	                            'id' => $booking->id,
    	                            'refund_duration' => $booking->refund_duration, // to deal with order after refunds
    	                            'refund_unit' => $booking->refund_unit // refund calculation unit
    	                        );
    	                        $cart[$item_id]['params'] = $booking_params;
    	                        $cart[$item_id]['qty'] = $qty;
								$cart[$item_id]['max_qty'] = $available_seats;
    	                        $cart[$item_id]['stock'] = empty($booking->seats) ? 10000 : $booking->seats;
	                        } else {
	                            unset($cart[$item_id]);
	                        }
	                    }
	                }
	                break;
	            case 'pyg': // PYG Module
	                $pyg_checkout = $cart_item['params'];
	                if(!empty($pyg_checkout)) {
	                    $pyg_package_id = $pyg_checkout['package_id'];
	                    $pyg_module = $pyg_checkout['module']; // will be passed as a variable
	                    $order_module_id = gc_module_id('pyg');
	                    $package_info = $this->packages_model->get_package($pyg_package_id);
	                    if($package_info) {
	                        $pyg_title = lang('pyg');
	                        $pyg_description = sprintf(lang('pyg_description'), $package_info->title, lang('singular_' . $pyg_module));
	                        $pyg = json_decode($package_info->pyg ?? '');
	                        $pyg_amount = $pyg->{$pyg_module};
	                        $pyg_module_id = gc_module_id($pyg_module);
	                        $pyg_settings = $tax_settings;
	                        $pyg_settings['variant_id'] = $pyg_module_id; // Module ID relation
	                        // $pyg_settings['cover'] = gc_site_image('pyg');
	                        $cart[$item_id] = $this->prepare_cart_item($order_module_id, $pyg_package_id, $pyg_amount, $pyg_title, $pyg_description, 0, $subscription_type, null, $ru_order_id, $pyg_settings);
	                        $cart[$item_id]['params'] = $cart_item['params'];
	                    }
	                }
	                break;
	            case 'claims': // Listing Claims
	                $claim_listing_id = $cart_item['row_id'];
	                if(!empty($claim_listing_id)) {
	                    $claimed_listing = $this->listings_model->get_listing_detail($claim_listing_id);
	                    if($claimed_listing) {
	                        $package_info = $this->packages_model->get_package($claimed_listing->package_id);
	                        if($package_info) {
	                            $claim_price = $package_info->claim_price;
	                            $order_module_id = gc_module_id('claims');
	                            $claim_title = lang('label_claim') . ': ' .$claimed_listing->title;
	                            $claim_description = $claimed_listing->address . '<br />' . $claimed_listing->city . ' '  . $claimed_listing->postcode .'<br />' .$claimed_listing->state .', '. ucwords(strtolower($claimed_listing->country));                           
	                            $claim_extra = $tax_settings;
	                            if(!empty($claimed_listing->cover)) {
	                                $claim_extra['cover'] = $claimed_listing->cover;
	                            }
	                            $cart[$item_id] = $this->prepare_cart_item($order_module_id, $claimed_listing->id, $claim_price, $claim_title, $claim_description, 0, $subscription_type, null, $ru_order_id, $claim_extra);
	                            $new_status_id = $package_info->claim_status;
	                        }
	                    }
	                }
	                break;
	            case 'commission_levels':
	                $checkout_level_id = $cart_item['row_id'];
	                if(!empty($checkout_level_id)) {
	                    $level_info = $this->gc->get('commission_levels', array('where' => "id = " . $checkout_level_id));
	                    if($level_info) {
	                        $order_module_id = gc_module_id('commission_levels');
	                        $level_extra = $tax_settings;
	                        $level_extra['cover'] = !empty($level_info->cover) ? gc_image_url($level_info->cover, false) : '';
	                        $amount = !empty($level_info->offer_price) ? $level_info->offer_price : $level_info->price;
	                        
	                        // calculate expires on
	                        $expires_on = null;
	                        if(!empty($level_info->duration)) {
	                            $expires_on = $this->calculate_expires_on($level_extra, $subscription_type, null);
	                        }
	                        
	                        $cart[$item_id] = $this->prepare_cart_item($order_module_id, $checkout_level_id, $level_info->price, $level_info->name, $level_info->description, $level_info->offer_price, $subscription_type, $expires_on, $ru_order_id, $level_extra);
	                        // invoice reminders are saved in the order_items table itself to avoid unnecessary MySQL JOINS
	                        $cart[$item_id]['reminders'] = (array) $this->save_order_reminders($level_info, $expires_on);
	                        $level_params = array();
	                        $level_params['savings'] = calculate_savings($level_info->price, $level_info->offer_price);
	                        $cart[$item_id]['params'] =  $level_params;
	                        $new_status_id = empty($level_info->order_status) ? 2 : $level_info->order_status;
	                    }
	                }
	                break;
	               default:
	                   unset($cart[$item_id]);
	            }	            
	            // zero checkout
	            if($zero_checkout && empty($cart[$item_id]['discount']) && empty($cart[$item_id]['total']) && !empty($cart[$item_id]['item']) && !empty($user_id)) {
	                
	                // perform checkout
	                $this->zero_checkout($user, $cart[$item_id], $new_status_id);
	                
	                // remove item from the cart and do checkout
	                unset($cart[$item_id]);
	                // $this->cart($item_id, 1);
	            }
	        }
	    }
	    if(empty($cart)) { // clear the cart
	        $this->checkout_params($user_id, false, array('items' => $cart), true);
	    } else {
	        $this->checkout_params($user_id, false, array('items' => $cart));
	    }	    
	    return $cart;
	}
	
	/**
	 * Common Method for Upgrade and Combo Discounts
	 * @param array $cart
	 * @return NULL[][]
	 */
	private function additional_product_discounts($cart) {
	    // common method for upgrade and combo discounts
	    // e.g. 1 => array('value_type' => 'F', 'value' => 10)
	    $additional_product_discounts = array();
	    // check combo items
	    $checkout_combos = $this->session->userdata('checkout_combos');
	    if($checkout_combos) {
	        // if all combo items are in the checkout cart, apply the additional discount if applicable
	        foreach($checkout_combos as $checkout_combo) {
	            // record combo discounts and items for reporting purpose
	            // this information will be stored in the params field
	            $combo_products = explode(',', $checkout_combo->product_ids ?? '');
	            $non_existing_products = $combo_products;
	            foreach($combo_products as $combo_product) {
	                if(array_key_exists($combo_product, $cart)) {
	                    $key = array_search($combo_product, $non_existing_products);
	                    if (false !== $key) {
	                        unset($non_existing_products[$key]);
	                    }
	                }
	            }
	            // all product exists in the cart
	            if(empty($non_existing_products)) {
	                foreach($combo_products as $combo_product) {
	                    // combo discount will be applied only once
	                    if(!array_key_exists($combo_product, $additional_product_discounts)) {
	                        $additional_product_discounts[$combo_product] = array(
	                            'value_type' => $checkout_combo->value_type,
	                            'value' => $checkout_combo->value,
	                            'type' => 1 // Custom Field - 1: Combo Discount, 2: Upgrade Discount
	                        );
	                    }
	                }
	            }
	        }
	    }
	    return $additional_product_discounts;
	}
	
	/**
	 * Prepare Cart Item to be saved in the orders table
	 * @param number $module_id
	 * @param number $row_id
	 * @param number $item_price
	 * @param string $item_name
	 * @param string $item_description
	 * @param number $offer_price
	 * @param number $subscription_type
	 * @param string $expires_on
	 * @param string $ru_order_id
	 * @return array
	 */
	private function prepare_cart_item($module_id, $row_id, $item_price, $item_name, $item_description, $offer_price = 0, $subscription_type = 1, $expires_on = null, $ru_order_id = null, $extra = array()) {
	    $item_amount = !empty($offer_price) ? $offer_price : $item_price;
	    $item = array(
	        'subscription_type' => $subscription_type,
	        'ru_order_id' => $ru_order_id,
	        'ru_order_item_id' => !empty($extra['order_item_id']) ? $extra['order_item_id'] : 0,
	        'module_id' => $module_id,
	        'row_id' => $row_id,
	        'variant_id' => !empty($extra['variant_id']) ? $extra['variant_id'] : 0, // PYG - Store Module Name in the variants field
	        'seller_id' => !empty($extra['seller_id']) ? $extra['seller_id'] : 1,
	        'price' => $item_price,
	        'offer_price' => $offer_price,
	        'item_qty_price' => $item_amount,
	        'savings' => !empty($offer_price) ? $item_price - $offer_price : 0,
	        'subtotal' => !empty($offer_price) ? $offer_price : $item_price,
	        'total' => !empty($offer_price) ? $offer_price : $item_price,
	        'discount' => 0,
	        'discount_value' => 0,
	        'qty' => 1,
	        'allow_coupon' => isset($extra['no_coupons']) ? $extra['no_coupons'] : true, // do not allow coupons on specific products
	        'max_qty' => isset($extra['max_qty']) ? $extra['max_qty'] : 1, // only applicable to specific items
	        'item' => array(
	            'name' => $item_name,
	            'description' => $item_description,
	            'cover' => isset($extra['cover']) ? $extra['cover'] : ($module_id == 22 ? gc_site_image('wallet-payment') : '')
	        ),
	        'expires_on' => $expires_on,
	        'params' => null,
	        'reminder_notifications' => 0,
	        'commission' => !empty($extra['commission']) ? $extra['commission'] : 0,
	        'commission_type' => !empty($extra['commission_type']) ? $extra['commission_type'] : 'P',
	    );
	    $item['tax'] = 0;
	    $item['tax_percent'] = 0;
	    $item['tax_inclusive'] = true;
	    if(!empty($extra['tax_percent']) && $item_amount > 0) {
	        $item['tax_percent'] = $extra['tax_percent'];
	        $item['tax_inclusive'] = $extra['tax_inclusive'];
	        $item_tax_percent = $item['tax_percent'];
	        $inclusive_tax = $item['tax_inclusive'];
	        // Item Tax Calculation on Quantity
	        $item_tax = decimal_convert(($item_amount * $item_tax_percent)/100);
	        $subtotal = $item['subtotal'];
	        $total = $item['total'];
	        if($inclusive_tax) {	// Tax Inclusive
	            $subtotal = $item_amount - $item_tax;
	        } else {
	            $total = $item_amount + $item_tax;
	        }
	        $item['tax'] = $item_tax;
	        $item['subtotal'] = $subtotal;
	        $item['total'] = $total;
	    }
	    return $item;
	}
	
	/**
	 * Process Cart Item
	 * @param array $cart_item
	 * @param float $coupon_value
	 * @return array
	 */
	public function process_cart_item($cart_item, $coupon_value = null) {
	    // firstly check whether the item is applicable for discount or not
	    // calculate discount on offer price
	    if(!empty($cart_item['offer_price'])) {
	        $item_price = $cart_item['offer_price'];
	    } else {
	        $item_price = $cart_item['price'];
	    }
	    $item_qty_price = $item_price * $cart_item['qty'];
	    $item_tax = $cart_item['tax'];
	    // the tax is inclusive
	    if($cart_item['tax_inclusive'] && !empty($cart_item['tax_percent'])) {
	        // subtract the tax amount from the item price
	        $item_tax = ($item_price * $cart_item['tax_percent']) / 100;
	        $item_price = $item_price - $item_tax;
	    }
	    
	    // multiply the price with selected quantity
	    $item_price = $item_price * $cart_item['qty'];
	    $cart_item['savings'] = $cart_item['qty'] * ($cart_item['price'] - $cart_item['offer_price']);
	    $cart_item['item']['rules_discount'] = 0;
	    
	    // Rules Discount
	    if($cart_item['module_id'] == gc_module_id('products')) {
	        /*
	        $this->load->library('ecommerce');
	        $rule_discount = $this->ecommerce->check_pricing_rules($cart_item['row_id'], $cart_item['user_id'], $cart_item['qty']);
	        if(!empty($rule_discount)) {
	            $additional_discount = (($item_price * (float) $rule_discount) / 100);
	            $item_price = $item_price - $additional_discount;
	            // $cart_item['savings'] += $additional_discount; // must for rules discount
	            // $cart_item['item']['savings'] += $additional_discount; // must for rules discount
	            $cart_item['item']['rules_discount'] = $additional_discount;
	        }
	        */
	    }
	    
	    // discount calculation
	    if(!empty($coupon_value)) {
	        $item_discount = decimal_convert(($item_price * $coupon_value) / 100);
	        $cart_item['discount_value'] = $coupon_value;
	    } else {
	        $item_discount = 0;
	        $cart_item['discount_value'] = 0;
	    }
	    
	    // tax calculation
	    $item_tax = 0;
	    $item_total = $item_price - $item_discount;
	    $item_qty_price = $item_qty_price - $item_discount;
	    
	    // visitor_country_iso
	    if($cart_item['module_id'] != gc_module_id('wallets') && empty($cart_item['tax_slab'])) { // not a wallet payment
	        $proceed_with_country_tax = true;
	        if($cart_item['module_id'] == 1 && !empty($cart_item['tax_settings']) && $cart_item['tax_settings'] == 3) {
	            $proceed_with_country_tax = false; // the tax is disabled for this product
	        }
	        if($proceed_with_country_tax && !empty($cart_item['user_id'])) {
	            $this->load->model('tax_rates/tax_rates_model');
	            $tax_country = $this->checkout_params($cart_item['user_id'], 'country');
	            $tax_rate = $this->tax_rates_model->get_tax_rate(empty($tax_country) ? '' : $tax_country);
	            if(!$tax_rate) {
	                $tax_rate = $this->tax_rates_model->get_tax_rate(1, 'is_default');
	            }
	            if($tax_rate) {
	                $tax_applicable_modules = !empty($tax_rate->applicable_modules) ? json_decode($tax_rate->applicable_modules, true) : array();
	                if($cart_item['module_id'] != gc_module_id('products') && !empty($tax_applicable_modules[gc_module_name($cart_item['module_id'])])) { // get tax settings of each module
	                    // get module specific tax settings
	                    $cart_item['tax_inclusive'] = $tax_rate->tax_inclusive;
	                    $cart_item['tax_percent'] = $tax_rate->tax_percent;
	                }
	            }
	        }
	    }
	    if(!empty($cart_item['tax_percent'])) {
	        $item_tax = decimal_convert(($item_qty_price * $cart_item['tax_percent']) / 100);
	        $item_total += $item_tax;
	    }
	    $cart_item['tax'] = $item_tax;
	    /*if(!empty($item_discount)) { // additional discount savings
	     $cart_item['savings'] += $item_discount;
	     }*/
	    $cart_item['item_qty_price'] = $cart_item['price'] * $cart_item['qty']; // original price to be shown in line through
	    $cart_item['discount'] = $item_discount;
	    $cart_item['subtotal'] = decimal_convert($item_price);
	    $cart_item['total'] = decimal_convert($item_total);
	    
	    return $cart_item;
	}
	
	/**
	 * Update Cart Items on Apply Coupon, Remove Coupon, Tax Country Change and Quantity Update
	 * The checkout_cart_items session variable is updated with new values
	 */
	public function update_cart_items($user_id = 0, $mobile = false, $custom_cart = false) {
	    $checkout_coupon = $this->checkout_params($user_id, 'coupon');
	    // coupon is saved in a session variable to call it again
	    // on coupon removal, this session is cleared and default values are stored
	    // country change will not have any affect on discount amount
	    // but change in the discount amount will affect tax rates
	    if(!empty($checkout_coupon)) { // coupon used
	        // calculate coupons discount
	        $this->load->model('coupons/coupons_model');
	        $this->coupons_model->calculate_discount($checkout_coupon);
	    } else { // without coupon and tax calculations
	        $this->cart_items($user_id);
	    }
	    return $this->cart_ajax_response($user_id, $mobile, $custom_cart);
	}
	
	/**
	 * Prepare data for AJAX response
	 */
	public function cart_ajax_response($user_id = 0, $mobile = false, $custom_cart = false) {
	    $this->load->library('template');
	    $checkout_params = $this->checkout_params($user_id, 'all');
	    $cart_items = !empty($checkout_params['items']) ? $checkout_params['items'] : array();
	    $checkout_coupon = !empty($checkout_params['coupon']) ? $checkout_params['coupon'] : array(); // $this->checkout_params($user_id, 'coupon');
	    $cart_data = array(
	        'items' => $cart_items,
	        'coupon_value' => isset($checkout_coupon['value']) ? $checkout_coupon['value'] : 0
	    );
	    if($custom_cart) {
	        return $cart_data;
	    } else {
    	    if(!$mobile) {
    	        $order_settings = json_decode(settings_item('order.settings'), true);
        	    $data = array(
        	        'success' => true,
        	        'items_count' => count($cart_items),
        	        'cart_layout' => cart_layout($cart_data),
        	        'cart_summary' => cart_summary($cart_data),
        	        'redirect_url' => empty($cart_items) ? site_url() : '', // last item removed from the checkout page, redirect user to the home page
        	        'checkout_url' => !empty($order_settings['checkout_cart']) ? gc_url_format('orders/cart') : gc_url_format('members/checkout')
        	    );
    	    } else {
    	        $data = array(
    	            'success' => true
    	        );
    	    }
    	    return $data;
	    }
	}
	
	/**
	 * Mobile Rest API - Checkout Output
	 * @param number $user_id
	 * @return mixed
	 */
	public function checkout_output($user_id, $with_fields = false) {
	    $wallet_payments = true;
	    $order_summary = array(
	        'order_subtotal' => 0,
	        'order_discount' => 0,
	        'order_tax' => 0,
	        'order_total' => 0,
	    );
	    $items = array();
	    $checkout_params = $this->checkout_params($user_id, 'all');
	    if(empty($checkout_params['items'])) { // [0]['total']the user selected a free package
	        // get order id to redirect user to the success page
	        $order = $this->db->select('id, order_id')->where('user_id', $user_id)->order_by('id', 'DESC')->get('orders')->row();
	        if($order) {
	            return array('success' => true, 'order_id' => (int) $order->id, 'zero_checkout' => true);
	        }
	    }
	    if(!empty($checkout_params['items'])) {
	        foreach($checkout_params['items'] as $item_id => $checkout_item) {
	            $checkout_item = (array) $checkout_item;
    	        $order_summary['order_subtotal'] += $checkout_item['subtotal'];
    	        $order_summary['order_discount'] += $checkout_item['discount'];
    	        $order_summary['order_tax'] += $checkout_item['tax'];
    	        $order_summary['order_total'] += $checkout_item['total'];
    	        
    	        $item_price = !empty($checkout_item['offer_price']) ? $checkout_item['offer_price'] : $checkout_item['price'];
    	        $savings = '';
    	        if(!empty($checkout_item['offer_price'])) {
    	            $savings = calculate_savings($checkout_item['price'], $checkout_item['offer_price'], 'checkout-savings');
    	            $savings = $savings['html'];
    	        }
    	        // item information to be displayed
    	        $items[] = array(
    	            'name' => $checkout_item['item']['name'],
    	            'cover' => !empty($checkout_item['item']['cover']) ? gc_image_url($checkout_item['item']['cover'], false) : gc_site_image('no-image'),
    	            'description' => $checkout_item['item']['description'],
    	            'savings' => $savings,
    	            'currency_subtotal' => currency_format($checkout_item['subtotal']),
    	            'currency_discount' => currency_format($checkout_item['discount']),
    	            'currency_tax' => currency_format($checkout_item['tax']),
    	            'currency_total' => currency_format($checkout_item['total']),
    	            'currency_savings' => currency_format($checkout_item['savings']),
    	            'currency_price' => currency_format($item_price),
    	            'price' => $item_price,
    	            'subtotal' => $checkout_item['subtotal'],
    	            'discount' => $checkout_item['discount'],
    	            'tax' => $checkout_item['tax'],
    	            'total' => $checkout_item['total'],
    	            'qty' => $checkout_item['qty'],
    	            'max_qty' => $checkout_item['max_qty']
    	        );
    	        // do not display wallet option in case of wallet payments
    	        if($checkout_item['module_id'] == gc_module_id('wallets')) {
    	            $wallet_payments = false;
    	        }
    	    }
    	    
    	    $order_summary['currency_order_total'] = currency_format($order_summary['order_total']);
    	    $order_summary['currency_order_subtotal'] = currency_format($order_summary['order_subtotal']);
    	    $order_summary['currency_order_tax'] = currency_format($order_summary['order_tax']);
    	    if(!empty($order_summary['order_discount'])) {
    	        $order_summary['coupon_code'] = !empty($checkout_params['coupon']['name']) ? $checkout_params['coupon']['name'] : null;
    	        $order_summary['currency_order_discount'] = currency_format($order_summary['order_discount']);
    	    }
    	    $order_summary['items'] = $items;
	        $order_summary['success'] = true;
	    }
	    if($with_fields) {
	        $this->lang->load('members/members');
	        $this->load->model('payment_gateways/payment_gateways_model');
	        $payment_gateways = $this->payment_gateways_model->select('id, name, display_name, is_default')->find_all_by(array('status' => 1));
	        // payment gateway field
	        if(!empty($payment_gateways)) {
	            $payment_options = array();
	            // get users wallet amount
	            $this->load->model('users/user_model');
	            $user = $this->user_model->get_user($user_id);
	            //$order_summary['order_total']
	            foreach($payment_gateways as $payment_gateway) {
	                $payment_options[] = array(
	                    'value' => $payment_gateway->id,
	                    'name' => $payment_gateway->display_name,
	                );
	                if($payment_gateway->is_default == 1) {
	                    $default_gateway = $payment_gateway->id;
	                }
	            }
	            $fields = array();
	            if($wallet_payments && !empty($user->wallets)) { // if not a wallet checkout, the user can pay using wallet
	               $fields[] = array('name' => 'pay_using_wallet', 'label' => lang('pay_using_wallet') . ' (' . currency_format($user->wallets) . ')', 'type' => 'checkbox', 'rules' => array());
	            }
	            $fields[] = array('name' => 'payment_method', 'label' => lang('gc_menu_payment_gateways'), 'type' => 'radio', 'options' => $payment_options, 'rules' => array('required' => 'required'));
	            $payment_gateway_field = $this->mobiles_model->rest_fields($fields);
	            $order_summary['fields'] = array('payment_gateways' => $payment_gateway_field);
	            $order_summary['record'] = array('pay_using_wallet' => $order_summary['order_total'] <= $user->wallets ? 1 : 0, 'payment_method' => !empty($default_gateway) ? $default_gateway : $payment_options[0]['value']);
	            // wallet does not have sufficient balance	   
	            if($user) {
    	            $order_summary['extra']['wallet'] = !empty($user->wallets) ? $user->wallets : 0;
    	            if($order_summary['order_total'] > $user->wallets) {
    	                $order_summary['extra']['insufficient_wallet'] = $order_summary['order_total'] - $user->wallets;
    	            }
	            }
	        }
	        
	        // coupon field
	        $coupon_field = array('name' => 'coupon', 'label' => lang('label_coupon_code'), 'type' => 'text', 'rules' => array('required' => 'required'));
	        $order_summary['fields']['coupon'] = $this->mobiles_model->rest_fields(array($coupon_field));
	        $order_summary['record']['coupon'] = !empty($checkout_params['coupon']) ? $checkout_params['coupon']['code'] : null;
	    }
	    return $order_summary;
	}
	
	/**
	 * Save Order Reminders
	 *
	 * There will be one invoice only but each item may have different reminder dates.
	 * They will be saved in the order_items table itself to avoid unnecessary JOINS and lookups.
	 *
	 * @param array $item
	 * @param string $expires_on
	 * @return array[]|string[]
	 */
	private function save_order_reminders($item, $expires_on) {
	    $data = array();
	    /**
	     * Calculate reminders from the expiry date because
	     * if the user does not pay then reminder should be sent
	     * set reminders to default value 0000-00-00 if the transaction is successful to avoid reminders
	     */
	    if(!empty($expires_on)) {
	        // Reminder date should be saved only if the item has a subscription
	        // Invoice table stores reminder and next invoice date which are not required in case of a product purchase
	        $next_invoice = (int) $item->days_before_next_invoice;
	        $reminder_first = (int) $item->invoice_first_reminder;
	        $reminder_second = (int) $item->invoice_second_reminder;
	        $reminder_third = (int) $item->invoice_third_reminder;
	        
	        if(empty($next_invoice)) {
	            $data['expires_on'] = $expires_on;
	            return json_encode($data);
	        }
	        $expiry_date = new DateTime($expires_on);
	        $expiry_date->sub(new DateInterval('P' . $next_invoice.'D'));
	        $next_invoice_date = $expiry_date->format("Y-m-d");
	        $data['next_invoice_date'] = $next_invoice_date;
	        if(!empty($reminder_first)) {
	            $first_reminder = new DateTime($next_invoice_date);
	            $first_reminder->add(new DateInterval('P' . $reminder_first.'D'));
	            $first_reminder_date = $first_reminder->format("Y-m-d");
	            $data ['first_reminder'] = $first_reminder_date;
	        }
	        
	        if(!empty($reminder_second)) {
	            $second_reminder = new DateTime(empty($first_reminder_date) ? $next_invoice_date : $first_reminder_date);
	            $second_reminder->add(new DateInterval('P' . $reminder_second.'D'));
	            $second_reminder_date = $second_reminder->format("Y-m-d");
	            $data ['second_reminder'] = $second_reminder_date;
	        }
	        
	        if(!empty($reminder_third)) {
	            if(!empty($second_reminder_date)) {
	                $third_reminder = new DateTime($second_reminder_date);
	            } else if(!empty($first_reminder_date)) {
	                $third_reminder = new DateTime($second_reminder_date);
	            } else {
	                $third_reminder = new DateTime($next_invoice_date);
	            }	                
	            $third_reminder->add(new DateInterval('P' . $reminder_third.'D'));
	            $third_reminder_date = $third_reminder->format("Y-m-d");
	            $data ['third_reminder'] = $third_reminder_date;
	        }
	        $data ['due_date'] = $expires_on;
	        
	        // Calculate and save Invoice expiry date 23/02/17
	        if($item->invoice_expiry > 0 && !empty($expires_on)) {
	            $invoice_expiry = new DateTime($expires_on);
	            $invoice_expiry->add(new DateInterval('P' . $item->invoice_expiry .'D'));
	            $invoice_expiry_date = $invoice_expiry->format("Y-m-d");
	            $data['invoice_expiry'] = $invoice_expiry_date;
	        }
	        $data['expires_on'] = $expires_on;
	    }
	    if(!empty($data)) {
	       return json_encode($data);
	    } else {
	       return $data;
	    }
	}
	
	/**
	 * Save order
	 * @param string $order_id
	 * @param string $currency_code
	 * @param float $currency_value
	 * @param string $expires_on
	 * @param integer $status - If price is 0 then set order status to 1
	 * // $currency_code, $currency_value, $expires_on, $order_data, $is_invoiced = 0, $status = 0,
	 */
	public function save_orders($order_id, $user, $payment_method, $status_id = 1, $type = 'insert', $order_summary = array(), $mobile = false) {
	    $this->load->library('crypt');
	    $this->load->model('currencies/currencies_model');
	    $datetime = date('Y-m-d H:i:s');
	    $ip_address = $this->input->ip_address();
	    $user_currency = $this->session->userdata('site_currency');
	    if(empty($user_currency)) { // Mobile API
	        // get currency via POST - Mobile API
	        $payment_currency = settings_item('site.currency_code');
	        if($this->input->post('currency')) {
	            $payment_currency = $this->input->post('currency');
	        }
	        $this->load->model('currencies/currencies_model');
	        $user_currency = $this->currencies_model->get_currency($payment_currency);
	    }
	    $user_currency = (object) $user_currency;
	    $data = array(
	        'order_id' => $order_id,
	        'user_id' => $user->id,
	        'status_id' => $status_id,
	        'user_email' => $user->email,
	        'items' => $order_summary['items'], // Payment Gateway - Display Item Names as description
	        'order_subtotal' => $order_summary['order_subtotal'],
	        'order_discount' => $order_summary['order_discount'],
	        'order_tax' => $order_summary['order_tax'],
	        'order_total' => $order_summary['order_total'],
	        'currency_code' => $user_currency->code,
	        'currency_value' => $user_currency->rate_usd,
	        'payment_method' => $payment_method,
	        'comments' => $this->input->post('comments'),
	        'affiliate_id' => (int) $this->crypt->decode($this->input->cookie('ref')),
	        'mobile_checkout' => $mobile ? 1 : 0 // 1 - Mobile Checkout, 0 - Website Checkout
	    );	    
	    if(!empty($order_summary['wallets'])) {
	        $data['wallets'] = $order_summary['order_discount'];
	    }
	    if($type == 'insert') {
	        $data ['created_on'] = $datetime;
	        $data ['created_ip'] = $ip_address;
	        $data ['created_by'] = (int) $this->auth->user_id();
	        $id = $this->orders_model->insert ( $data );
	        return $id;
	    } elseif ($type == 'update') {
	        $data ['modified_on'] = $datetime;
	        $data ['modified_ip'] = $ip_address;
	        $data ['modified_by'] = (int) $this->auth->user_id();
	        $this->db->where('order_id', $order_id)->update('orders', $data);
	    }
	}
	
	/**
	 * Save Invoices
	 * @param int $order_id
	 */
	public function save_invoices($order_id, $order_info = array()) {
	    $invoice_id = 0;
	    if(empty($order_info)) {
	        $order_info = $this->db->get_where('orders', array('order_id' => $order_id))->row();
	    }
	    $type = $order_info->invoice_id == 0 ? 'insert' : 'update';
	    if(!empty($order_info)) {
	        $datetime = date('Y-m-d H:i:s');
	        $ip_address = $this->input->ip_address();
	        $data = array();
	        $data['status_id'] = $order_info->status_id;
	        $data['date_paid'] = $datetime;
	        if($type=='insert') {
	            // Get the ITEM price using the ORDER ID
	            // If the item price is zero set the invoice status to 1 PAID
	            // Free invoices are also shown to the user, do not allow user to pay if the status is set to inactive
	            $data ['prefix'] = gc_json_settings('order.settings', 'invoice_prefix');
	            $data ['order_id'] = $order_id;
	            $data ['user_id'] = $order_info->user_id;
	            $data ['created_on'] = $datetime;
	            $data ['created_ip'] = $ip_address;
	            $data ['created_by'] = $this->auth->user_id();
	            $this->db->insert ($this->table_invoices, $data );
	            $invoice_id = (int) $this->db->insert_id();
	            // update invoice id in the orders table for Admin Search
	            $this->db->where('order_id', $order_id)->update('orders', array('invoice_id' => $invoice_id));
	        } else if(!empty($order_info->invoice_id) && $type=='update') {
	            $data ['modified_on'] = $datetime;
	            $data ['modified_ip'] = $ip_address;
	            $data ['modified_by'] = $this->auth->user_id();
	            $this->db->where('order_id', $order_id)->update ($this->table_invoices, $data );
	            $invoice_id = $order_info->invoice_id;
	        }
	    }
	    return $invoice_id;
	}
	
	/**
	 * Save Coupons Used during Order
	 * Note: Affiliate commission will always be calculated on order confirmation
	 * @param string $order_id
	 * @param array $user
	 */
	public function save_order_coupons($order_id, $user, $order_summary) {
	    $user_id = $user->id;
	    $checkout_coupon = $this->checkout_params($user_id, 'coupon');
	    if(!empty($checkout_coupon)) {
	        $checkout_coupon = (object) $checkout_coupon;
	        $data = array(
	            'order_id' => $order_id,
	            'user_id' => $user->id,
	            'user_email' => $user->email,
	            'coupon_id' => $checkout_coupon->id,
	            'coupon_code' => $checkout_coupon->code,
	            'value' => $checkout_coupon->value,
	            'value_type' => isset($checkout_coupon->value_type) ? $checkout_coupon->value_type : 'P',
	            'amount' => $order_summary['order_discount']
	        );
	        
	        
	        // Calculate Affiliate commission and update orders table
	        // $this->save_order_affiliate($order_id, $coupon->id);
	        /*$this->load->model('commissions/commissions_model');
	        $affiliate_id = (int) $this->input->cookie('ref');
	        $this->commissions_model->coupon_commissions($order_id, 'affiliates', $checkout_coupon->id, $affiliate_id);
	        $this->commissions_model->coupon_commissions($order_id, 'invitations', $checkout_coupon->id, $affiliate_id);
	        */
	        
	        // Find whether the user has already applied this coupon or not
	        $result = $this->db->where('order_id', $order_id)->count_all_results($this->table_coupons);
	        if($result > 0) { // If coupon applied previously, update coupon information
	            $this->db->where('order_id', $order_id)->update($this->table_coupons, $data);
	        } else {
	            $this->db->insert ($this->table_coupons, $data );
	            // Update Coupon Count
	            // Coupon count will only be updated on success of Coupon
	            // Count means how many times this coupon has been applied with success
	            // If an order is not confirmed, this count is still applicable
	            $this->db->where('id', $checkout_coupon->id)->set('counts', 'counts + 1', FALSE)->update('coupons');
	            
	            // Update Affiliate Coupon Counts
	            $this->update_affiliate_coupon_counts_hits($checkout_coupon->id, 'counts'); // Update Affiliate Coupon Hits
	        }
	    }
	}
	
	/**
	 * Save multiple order items
	 * @param string $order_id
	 */
	public function save_order_items($order_id, $user_id = 0, $cart_items = array()) {
	    if(!empty($cart_items)) {
    	    $data = array();
    	    foreach($cart_items as $cart_item) {
    	        $item = array(
    	            'order_id' => $order_id,
    	            'subscription_type' => isset($cart_item['subscription_type']) ? $cart_item['subscription_type'] : 1,
    	            'ru_order_id' => isset($cart_item['ru_order_id']) ? $cart_item['ru_order_id'] : null,
    	            'ru_order_item_id' => isset($cart_item['ru_order_item_id']) ? $cart_item['ru_order_item_id'] : 0,
    	            'user_id' => $user_id,
    	            'seller_id' => $cart_item['seller_id'],
    	            'module_id' => $cart_item['module_id'],
    	            'row_id' => $cart_item['row_id'],
    	            'variant_id' => isset($cart_item['variant_id']) ? $cart_item['variant_id'] : 0,
    	            'relation_item_id' => !empty($cart_item['relation_item_id']) ? $cart_item['relation_item_id'] : 0, // Product Optional Item
    	            'item' => json_encode($cart_item['item']),
    	            'price' => $cart_item['price'],
    	            'offer_price' => $cart_item['offer_price'],
    	            'qty' => $cart_item['qty'],
    	            'tax_percent' => $cart_item['tax_percent'],
    	            'tax_inclusive' => $cart_item['tax_inclusive'],
    	            'total' => $cart_item['total'],
    	            'subtotal' => $cart_item['subtotal'],
    	            'tax' => $cart_item['tax'],
    	            'savings' => !empty($cart_item['savings']) ? $cart_item['savings'] : 0,
    	            'discount' => $cart_item['discount'],
    	            'discount_value' => $cart_item['discount_value'],
    	            'commission' => $cart_item['commission'],
    	            'commission_type' => $cart_item['commission_type'],
    	            'refund_duration' => isset($cart_item['refund_duration']) ? $cart_item['refund_duration'] : 0,
    	            'expires_on' => isset($cart_item['expires_on']) ? $cart_item['expires_on'] : null,
    	            'reminders' => isset($cart_item['reminders']) ? (is_array($cart_item['reminders']) ? json_encode($cart_item['reminders']) : $cart_item['reminders']) : null,
    	            'params' => isset($cart_item['params']) ? (is_string($cart_item['params']) ? $cart_item['params'] : json_encode($cart_item['params'])) : '',
    	            'reminder_notifications' => isset($cart_item['reminder_notifications']) ? $cart_item['reminder_notifications'] : 0,
    	            'status_id' => 1,
    	            'created_on' => date('Y-m-d H:i:s') // product specific sales
    	        );
    	        $data[] = $item;
    	    }
    	    // Save Items in the order_items table
    	    $this->db->where('order_id', $order_id)->delete($this->table_items);
    	    $this->db->insert_batch($this->table_items, $data);
	    }
	}	
	
	/**
	 * Save order modified and customer notified events history
	 * @param string $order_id
	 * @param int $old_status
	 * @param int $new_status
	 */
	public function save_order_history($order_id, $old_status_id, $new_status_id, $comment_id = 0, $customer_notified = 0) {
	    $data = array();
	    $data['order_id']        = $order_id;
	    $data['old_order_status_id'] = $old_status_id;
	    $data['new_order_status_id'] = $new_status_id;
	    if(empty($comment_id)) {
	        if($old_status_id != $new_status_id && $customer_notified == 1) {
	            $data['comment_id'] = 1;
	        } else if($old_status_id != $new_status_id) {
	            $data['comment_id'] = 2;
	        } else if($customer_notified == 1) {
	            $data['comment_id'] = 3;
	        }
	    } else {
	        $data['comment_id'] = $comment_id;
	    }
	    $data['customer_notified']	 = $customer_notified;
	    $data['created_on']	= date('Y-m-d H:i:s');
	    $this->db->insert('order_histories', $data);
	}
	
	/**
	 * Order History Comments
	 * @param number $comment_id
	 * @return mixed
	 */
	public function order_history_comments($order_id) {
	    $status = array();
	    $order_statuses = $this->db->get('order_statuses')->result();
	    $order_histories = $this->db->where('order_id', $order_id)->get('order_histories')->result_array();
	    if($order_statuses && $order_histories) {
	        foreach($order_statuses as $order_status) {
	            $status[$order_status->id] = '<span style="color:' . $order_status->color . ';">' . (!empty($order_status->name_lang) ? lang($order_status->name) : $order_status->name) . '</span>';
	        }
	        foreach($order_histories as $key => $order_history) {
	            if($order_history['comment_id'] != 0) {
	                switch($order_history['comment_id']) {
	                    case 1: // Status Changed and Customer Notified
	                        $order_histories[$key]['comment'] = sprintf(lang('order_comment_' . $order_history['comment_id']), $status[$order_history['old_order_status_id']], $status[$order_history['new_order_status_id']]);
	                        break;
	                    case 2: // Status changed but customer is not notified
	                        $order_histories[$key]['comment'] = sprintf(lang('order_comment_' . $order_history['comment_id']), $status[$order_history['old_order_status_id']], $status[$order_history['new_order_status_id']]);
	                        break;
	                    case 3: // Customer Notified
	                    case 4: // Order Created and Customer Notified
	                    case 5: // Order Information Updated
	                    case 6: // First Reminder Sent
	                    case 7: // Second Reminder Sent
	                    case 8: // Third Reminder Sent
	                        $order_histories[$key]['comment'] = lang('order_comment_' . $order_history['comment_id']);
	                        break;
	                }
	                $order_histories[$key]['new_status'] = $status[$order_history['new_order_status_id']];
	                $order_histories[$key]['old_status'] = $status[$order_history['old_order_status_id']];
	            }
	        }
	    }
	    return $order_histories;
	}
	
	/**
	 * Find existing inactive order placed by the same user for the same package/claim
	 * If exists delete order related information and reset the id
	 */
	public function delete_existing_order($user_id, $order_id = null) {
	    // If user is paying an existing invoice, delete existing order and invoice
	    // before regeneration
	    if(!empty($order_id)) {
	        $this->purge_order($order_id);
	    } else {
	        // Find Order
	        /**
	         * payment_status will be changed on transaction confirmed from 0 to 1.
	         * 1 means payment is received. This variable will be used to deal with order deletion.
	         * Do not delete an order where the payment has received but the status is active because of some other reasons.
	         */
	        $sql = "SELECT O.order_id FROM ". $this->db->dbprefix . "orders O";
	        $sql .= " WHERE O.user_id = $user_id AND O.payment_status = 0 AND O.status_id = 1";
	        $result = $this->db->query($sql)->result();
	        if($result){
	            foreach($result as $order) {
	                $this->purge_order($order->order_id, $order);
	            }
	        }
	    }
	}
	
	/**
	 * Purge existing order
	 *
	 * @param string $order_id
	 * @param object $order_info
	 * @param boolean $auto_increment
	 * @return boolean
	 */
	public function purge_order($order_id, $order_info = array(), $auto_increment = true) {
	    if(empty($order_info)) {
	        $order_info = $this->db->where(array('order_id' => $order_id))->get_where('orders')->row();
	    }
	    if(empty($order_info)) {
	        return false;
	    }
	    $order_tables = array('order_cancellations', 'order_coupons', 'order_customers', 'order_histories', 'order_invoices', 'order_items', 'order_shipments', 'listing_claims', $this->table_commissions, 'orders');
	    foreach($order_tables as $table) {
	        $this->db->where('order_id', $order_id)->delete($table);
	    }
	    if($auto_increment) {
	        // Set auto increment value to the last id
	        foreach($order_tables as $table) {
	            $this->db->simple_query("ALTER TABLE ". $this->db->dbprefix . $table . " AUTO_INCREMENT = 1");
	        }
	    }
	}
	
	/**
	 * Purge Orders
	 * @param array $order_ids
	 */
	public function _purge_orders($order_ids = array()) {
	    if(!empty($order_ids)) {
	        foreach($order_ids as $order_id) {
	            $order_info = $this->db->where('id', $order_id)->get('orders')->row();
	            $this->purge_order($order_info->order_id, $order_info);
	        }
	    }
	}
	
	/**
	 * Amount Zero Checkout
	 * @param string $module
	 * @param number $user_id
	 * @param number $package_id
	 * @param array $user
	 * @param array $extra
	 * @return boolean|boolean[]|string[]
	 */
	public function zero_checkout($user, $cart_item, $order_status_id = 2) {
	    $user_id = $user->id;
	    
	    // Generate Order ID from Time
	    // During bulk import (User assign package) same order numbers were being generated.
	    // To fix this issue, a four digit random number is added with the time.
	    $order_id = gc_json_settings('order.settings', 'order_prefix') . time() . rand(1000,9999);
	    
	    // save order first
	    $order_summary = array(
	        'order_subtotal' => 0,
	        'order_discount' => 0,
	        'order_tax' => 0,
	        'order_total' => 0,
	        'items' => json_encode(array($cart_item['item']))
	    );
	    $this->save_orders($order_id, $user, null, 1, 'insert', $order_summary);
	    
	    // save order items
	    $this->save_order_items($order_id, $user_id, array($cart_item));
	    
	    // Save customer's information
	    $this->save_order_customers_via_users_table($order_id, $user);
	    
	    // Save Order History Informatation
	    $this->save_order_history($order_id, 1, 1, NULL);
	    
	    // update the users role to the business owner
	    if($user->role_id == 4 && GC_PRODUCT_ID == 1) {
	        $this->db->where('id', $user_id)->update('users', array('role_id' => 2));
	    }
	    // update ordered items and affiliate commission
	    $order_info = $this->db->where('order_id', $order_id)->get('orders')->row();
	    if($order_info) {
    	    $this->update_status($order_info->id, $order_id, $order_info, $order_status_id, true, true);
	    }
	    return $order_id;
	}
	
	/**
	 * Save customer information
	 * @param string $order_id
	 */
	public function save_order_customers_via_users_table($order_id, $user) {
	    $data = array ();
	    $data ['order_id'] = $order_id;
	    $data ['user_id'] = $user->id;
	    // $data ['company_name'] = $user->company_name;
	    // $data ['company_tax_no'] = $user->company_tax_no;
	    $data ['name'] = $user->display_name;
	    $data ['email'] = $user->email;
	    $data ['phone_number'] = !empty($user->phone_number) ? $user->phone_number : '';
	    if(!empty($user->address)) {
	        $data ['address'] = $user->address;
	    }
	    if(!empty($user->postcode)) {
	        $data ['postcode'] = $user->postcode;
	    }
	    if(empty($user->city_id)) {
	        // get location information from the maxmind database
	        $this->load->model('visitors/visitors_model');
	        $location = $this->visitors_model->get_save_location($this->input->ip_address());
	        // avoid further lookup
	        if(!empty($location)) {
    	        $this->db->where('id', $user->id)->update('users', $location);
    	        $data = array_merge($data, $location);
	        }
	    } else {
	        $data ['country_iso'] = $user->country_iso;
	        $data ['state_id'] = $user->state_id;
	        $data ['city_id'] = $user->city_id;
	        $data ['country'] = $user->country;
	        $data ['state'] = $user->state;
	        $data ['city'] = $user->city;
	    }
	    $this->db->insert('order_customers', $data );
	}
	
	/**
	 * New Order Generation - Reminders and Renewals
	 * @param object $user
	 * @param array $cart_item
	 * @param number $old_order_id - to copy customers information from existing order
	 */
	public function new_order_generation($user_id, $checkout_item, $old_order_id, $user = array()) {
	    $this->load->model('users/user_model');
	    if(empty($user)) {
	        $user = $this->user_model->get_user($user_id);
	    }
	    if($user) {
    	    $zero_checkout = false;
    	    if($checkout_item['total'] == 0) { // the checkout item is a free product
    	        // zero checkout
    	        $order_id = $this->zero_checkout($user, $checkout_item);
    	        $zero_checkout = true;
    	    } else {
    	        $order_status_id = 1;
        	    $user_id = $user->id;
        	    $order_id = gc_json_settings('order.settings', 'order_prefix') . time() . rand(1000,9999);
        	    
        	    $item_names = array();	    
        	    $order_summary = array();
        	    $order_summary['order_subtotal'] = $checkout_item['subtotal'];
        	    $order_summary['order_discount'] = $checkout_item['discount'];
        	    $order_summary['order_tax'] = $checkout_item['tax'];
        	    $order_summary['order_total'] = $checkout_item['total'];
        	    $item_names[] = $checkout_item['item']['name'];	    
        	    $order_summary['items'] = json_encode($item_names);
        	    
        	    // save orders
        	    $this->save_orders($order_id, $user, null, $order_status_id, 'insert', $order_summary);
        	    
        	    // save order items
        	    $this->save_order_items($order_id, $user_id, array($checkout_item));
        	    
        	    // save customer's information
        	    $this->_order_customer_info($old_order_id, $order_id);
        	    
        	    // save order history informatation
        	    $this->save_order_history($order_id, 1, $order_status_id, 4);
        	    
        	    // email invoice to the user
        	    // $this->email_print($order_id, 'email');
    	    }	    
    	    return array('order_id' => $order_id, 'zero_checkout' => $zero_checkout);
	    }
	}
	
	/**
	 * New Invoice Generation - copy order customers row from the same table with a new id
	 * @param string $order_id
	 * @param string $new_order_id
	 */
	private function _order_customer_info($old_order_id, $order_id) {
	    $order_customer = $this->db->where('order_id', $old_order_id)->get('order_customers')->row();
	    $data = array();
	    if($order_customer) {
	        foreach($order_customer as $field => $value) {
	            switch($field) {
	                case 'id':
	                    break;
	                case 'order_id':
	                    $data['order_id'] = $order_id;
	                    break;
	                default:
	                    $data[$field] = $value;
	            }
	        }
	        $this->db->insert($this->table_customers, $data );
	    }
	}
	
	/**
	 * Session is avoided during checkout because of Mobile API
	 * The session is not applicable in case of REST services
	 * i.e. the data is stored in a central location to use it during checkout
	 * 
	 * Table Name: rs0d_order_checkouts
	 * 
	 * @param number $user_id - It can also be a temporary id to hold the information 
	 * @param string $return - The data to be retured
	 * @param array $params - The data to be stored in the Table
	 * @param boolean $is_clear - Clear the user specific records from the order_checkouts table
	 */
	public function checkout_params($user_id, $return = 'items', $params = array(), $is_clear = null) {
	    $user_id = empty($user_id) ? $this->auth->user_id() : $user_id;
	    if(empty($user_id)) {
	        // checkout without login
	        // generate a temporary id
	        if(!$this->session->userdata('tmp_user_id')) {
    	        $tmp_user_id = $this->_tmp_user_id();
    	        $this->session->set_userdata('tmp_user_id', $tmp_user_id);
	        }
	        $user_id = $this->session->userdata('tmp_user_id');
	        if($is_clear) {
	            // unset the temporary user id
	            $this->session->unset_userdata('tmp_user_id');
	        }
	    }
	    if($is_clear) {
	        $this->db->where('user_id', $user_id)->delete('order_checkouts');
	    } else {
    	    $record = $this->db->where('user_id', $user_id)->get('order_checkouts')->row();
    	    if($return) {
    	        if(empty($record)) {
        	        return array();
        	    } else {
        	        $checkout_params = json_decode($record->params ?? '', true);
        	        if($return == 'all') {
        	            return $checkout_params;
        	        } else {
            	        if(!empty($checkout_params[$return])) {
            	            return $checkout_params[$return];
            	        } else {
            	            return array();
            	        }
        	        }
        	    }
    	    } else if(!empty($user_id)) {
    	        $data_params = array(); // do not remove coupon and items vice versa if only one is passed via params variable
    	        // save in the order checkouts table
    	        // items - Checkout items
    	        // coupon - Coupon used during checkout
    	        if($record && !empty($record->params)) {
    	            $checkout_params = json_decode($record->params ?? '', true);
    	            $data_params = $checkout_params;
    	            if(!empty($params['items'])) {
    	                $data_params['items'] = $params['items'];
    	            }
    	            if(!empty($params['coupon'])) {
    	                $data_params['coupon'] = $params['coupon'];
    	            }
    	            if(!empty($params['country'])) {
    	                $data_params['country'] = $params['country'];
    	            }
    	        } else {
    	            $data_params = $params;
    	        }
    	        // clear a specific key e.g. coupon from the order_checkouts table
    	        if(!empty($params['clear'])) {
    	            if(isset($data_params[$params['clear']])) {
    	                unset($data_params[$params['clear']]);
    	            }
    	        }
    	        // save country for tax calculations
    	        if(empty($data_params['country'])) {
    	            $data_params['country'] = $this->session->userdata('visitor_country_iso');
    	        }
    	        if($record) {
    	            $this->db->where('user_id', $user_id)->update('order_checkouts', array('params' => json_encode($data_params)));
    	        } else {
    	            $this->db->insert('order_checkouts', array('params' => json_encode($data_params), 'user_id' => $user_id));
    	        }
    	    }
	    }
	}
	
	/**
	 * Free/Paid Checkout
	 * @param number $id
	 * @param string $module
	 * @param number $user_id
	 * @return number[]|string[]|number[]|string[]|NULL[]
	 */
	public function free_paid_checkout($id, $module, $user_id) {
	    $module_id = gc_module_id($module);
	    $item_id = $id . '-0-0-' . $module_id;
	    $where = array('user_id' => $user_id, 'module_id' => $module_id, 'row_id' => $id);
	    switch($module) {
	        case 'claims':
	            $message = lang('claim_request_exists');
	            break;
	        default:
	            // the user cannot buy the same package again
	            $where['status_id'] = 2;
	            $message = lang('msg_package_not_for_multisale');
	    }
	    // check whether a claim request for this listing or this package already exists in the system or not
	    $items_count = $this->db->where($where)->count_all_results('order_items');
	    if($items_count > 1) {
	        return array('success' => 0, 'result' => 0, 'message' => $message, 'type' => 'danger');
	    }
	    // remove item first from the cart, added to resolve free claims cart issue
	    $this->orders_model->cart($item_id, 1, 1, 1, $module, true, array('zero_checkout' => true), $user_id);
	    $this->orders_model->cart($id, 0, 1, 1, $module, true, array('zero_checkout' => true), $user_id); // only one listing can be claimed at a time
	    if($this->orders_model->is_zero_checkout($id, $module, $user_id)) {
	        switch($module) {
	            case 'claims':
	                // display message
	                $this->db->from('listings L');
	                $this->db->join('packages P', 'P.id = L.package_id', 'INNER');
	                $result = $this->db->select('P.claim_status')->where('L.id', $id)->get()->row();
	                $order_status_id = $result && $result->claim_status == 2 ? 2 : 1;
	                $success_message = $order_status_id == 2 ? lang('msg_claim_approved') : lang('msg_claim_request');
	                break;
	            case 'packages':
	                $result = $this->db->where('id', $id)->get($module)->row();
	                $order_status_id = $result && $result->order_status == 2 ? 2 : 1;
	                $success_message = $order_status_id == 2 ? lang('msg_packages_approved') : lang('msg_packages_request');
	                break;
	            case 'ads':
	                $result = $this->db->where('id', $id)->get($module)->row();
	                $order_status_id = $result && $result->order_status == 2 ? 2 : 1;
	                $success_message = $order_status_id == 2 ? lang('msg_ads_approved') : lang('msg_ads_request');
	                break;	
	        }
	        $result = array('success' => 1, 'result' => 1, 'message' => $success_message, 'type' => 'success', 'free_checkout' => true);
	        if($order_status_id == 2) {
	           $result['url'] = gc_url_format('members');
	        }
	        return $result;
	    } else {
	        return array('success' => 1, 'result' => 1, 'message' => lang('msg_checkout_redirection'), 'type' => 'success', 'url' => gc_url_format('members/checkout'), 'free_checkout' => false);
	    }
	}
	
	/**
     * Checkout Without Login - Generate a Temporary User ID
	 */
	private function _tmp_user_id() {
	    $tmp_id = time() . rand(10000,99999);
	    // check existence in the users table
	    $users_count = $this->db->where('id', $tmp_id)->count_all_results('users');
	    if($users_count > 1) {
	        $this->_tmp_user_id();
	    } else {
	        return $tmp_id;
	    }
	}
	
	/**
	 * Check whether the item has a zero checkout
	 * @param number $id
	 * @param string $module
	 * @param number $user_id
	 * @return boolean
	 */
	public function is_zero_checkout($id, $module, $user_id) {	    
	    $checkout_cart_items = $this->checkout_params($user_id);
	    $module_id = gc_module_id($module);
	    $item_id = $id . '-0-0-' . $module_id;
	    if(empty($checkout_cart_items[$item_id])) {
	        // zero checkout
	        return true;
	    } else {
	        return false;
	    }
	}
	/**************************************
	 * END OF CHECKOUT SECTION
	 **************************************/
	
	/**
	 * Save user sales
	 * @param int $user_id - Sales Executive ID
	 * @param array $data
	 */
	public function update_sales_executive($user_id, $data = array()) {
	    if(empty($data['order_id'])) {
	        return array('success' => false);
	    }
	    // get order information
	    $order_id = escape($data['order_id']);
	    $order_info = $this->db->where('order_id', strtolower($order_id))->get('orders')->row();
	    if(empty($order_info)) {
	        return array('success' => false, 'message' => lang('sale_invalid_order_id'));
	    }
	    // do not allow user to submit the same order twice
	    if(!empty($order_info->sales_executive_id)) {
	        return array('success' => false, 'message' => lang('sale_order_id_exists'));
	    }
	    
	    // get order item
	    $where = "order_id = '" . $order_id . "' AND module_id = 20";
	    $order_item = $this->db->where($where)->get('order_items')->row();
	    if(!$order_item) { // only package payments are allowed, not wallets and others
	        return array('success' => false, 'message' => lang('sale_invalid_order_id'));	        
	    }
	    
	    // get user info
	    $this->load->model('users/user_model');
	    $user = $this->user_model->get_user($user_id);
	    if(!$user && $user->role_id != 9) {
	        return array('success' => false, 'message' => lang('sale_invalid_user_id'));
	    }
	    
	    // if order_status is not confirmed, check amount in the sales executive wallet
	    if($order_info->status_id != 2 && !empty($order_info->order_total) && $user->wallets < $order_info->order_total) {
	        $this->lang->load('wallets/wallets');
	        return array('success' => false, 'message' => lang('wallet_low_balance'));	        
	    }
	    
	    $order_table_data = array(
	        'sales_executive_id' => $user_id,
	        'payment_method' => escape($data['payment_method']),
	        'transaction_id' => !empty($data['transaction_id']) ? $data['transaction_id'] : null,
	        'modified_on' => date('Y-m-d H:i:s'),
	        'status_id' => 2,
	        'modified_ip' => $this->input->ip_address()
	    );
	    
	    // transaction_id and payment method are must for paid transactions
	    if($order_info->status_id != 2 && $order_info->order_total > 0 && empty($data['payment_method'])) { // && empty($data['transaction_id'])
	        return array('success' => false, 'message' => lang('sale_update_error'));	        
	    }
	    
	    if(!empty($data['payment_method'])) {
	        $order_table_data['payment_method'] = escape($data['payment_method']);
	    }
	    if(!empty($data['transaction_id'])) {
	        $order_table_data['transaction_id'] = escape($data['transaction_id']);
	    }
	    
	    $order_items_table_data = array();
	    if(!empty($data['amount'])) {
	        $amount = escape($data['amount']);
	        // recalculate order total, order subtotal, tax and discount
	        // get information from the order_items table
	        if(!empty($order_item->price)) {
	            $item_price = !empty($order_item->offer_price) ? $order_item->offer_price : $order_item->price;
	            $total_amount = $order_item->total;
	            if($amount < $total_amount) { // change in price detected
	                
	                // calculate discount
	                $order_items_table_data['discount'] = $total_amount - $amount;
	                
	                // calculate tax
	                $tax_percent = 18;
	                $tax_item = (100 + $tax_percent) / 100;
	                $subtotal_order = decimal_convert($amount/$tax_item);
	                $order_items_table_data['tax'] = $amount - $subtotal_order;
	                $order_items_table_data['subtotal'] = $subtotal_order;
	                
	                // update the item offer_price and do the recalculations
	                $order_items_table_data['offer_price'] = $subtotal_order;
	                $order_items_table_data['total'] = $subtotal_order + $order_items_table_data['tax'];
	                $order_items_table_data['status_id'] = 2;
	                
	                // order items table update
	                $this->db->where('id', $order_item->id)->update('order_items', $order_items_table_data);
	                
	                // get all order items
	                $order_items = $this->db->where('order_id', $order_id)->get('order_items')->result();
	                if($order_items) {
	                    // update the orders table
	                    $order_subtotal = 0;
	                    $order_discount = 0;
	                    $order_tax = 0;
	                    $order_total = 0;
	                    foreach($order_items as $item_order) {
	                        $item_order = (array) $item_order;
	                        $order_subtotal += $item_order['subtotal'];
	                        $order_discount += $item_order['discount'];
	                        $order_tax += $item_order['tax'];
	                        $order_total += $item_order['total'];
	                    }
	                    $order_table_data['order_subtotal'] = $order_subtotal;
	                    $order_table_data['order_tax'] = $order_tax;
	                    $order_table_data['order_discount'] = $order_discount;
	                    $order_table_data['order_total'] = $order_total;
	                }
	            }
	        }
	    }
	    $this->db->where('order_id', $order_id)->update($this->table_name, $order_table_data);
	    $order_status_id = $order_info->status_id;
	    $this->update_status($order_info->id, $order_id, $order_info, 2, true, false, true);
	    if($order_status_id != 2 && !empty($order_item->total)) {
	        $payment = !empty($data['amount']) ? $data['amount'] : $order_item->total;
	        // deduct the amount from the users wallet	        
	        $this->load->model('wallets/wallets_points_model');
	        $this->wallets_points_model->update_cash('wallets', $user_id, 'orders', $payment, 'debit', null, $order_info->id, array('order_id' => $order_id));
	        // send confirmation message to the user
	        // $order_info->phone_number = $user->phone_number;
	        // $this->email_print($order_info->order_id, 'sms', array('order' => $order_info));
	    }
	    // calculate sales earning and update in the passbook table
	    $this->sales_earning($user_id, $order_item->row_id, $order_info->id);
	    return array('success' => true, 'message' => sprintf(lang('msg_save_success'), lang('label_order')));
	}
	
	/**
	 * Calculate Sales Earning and update in the passbook table
	 * @param number $user_id
	 * @param number $package_id
	 */
	public function sales_earning($user_id, $package_id, $order_id) {
	    $this->load->model('packages/packages_model');
	    $package_info = $this->packages_model->get_package($package_id);
	    if($package_info) {
	        $price = !empty($package_info->offer_price) ? $package_info->offer_price : $package_info->price;
	        // if($price > 0) { // not a free package, can be a free package
	            $sale_commissions = json_decode($package_info->sales ?? '', true);
	            if(!empty($sale_commissions['commission']['value']) && $price > $sale_commissions['commission']['value']) {
	                $earned_amount = $sale_commissions['commission']['value']; // commission amount
	                if(!empty($sale_commissions['incentive']['value'])) {
        	            $where = "O.status_id = 2 AND O.deleted = 0 AND O.sales_executive_id = " . $user_id . " AND M.module_id = 20 AND M.row_id = " . $package_id . " AND DATE(M.created_on) = '" . date('Y-m-d') . "'";
        	            $sql = "SELECT COUNT(*) AS total_counts FROM " . $this->db->dbprefix . "order_items M";
        	            $sql .= " INNER JOIN " . $this->db->dbprefix . "orders O ON O.order_id = M.order_id";
        	            $sql .= " WHERE " . $where;
        	            $result = $this->db->query($sql)->row();
        	            if($result && $result->total_counts > $sale_commissions['min_incentive_sales']) {
        	                // earning with incentive
        	                $earned_amount = $sale_commissions['incentive']['value'];        	                
        	            }
    	            }
    	            // update passbook
    	            $this->gc->update_passbook($user_id, 'orders', $order_id, $earned_amount, 'credit');
	            }
	        // }
	    }
	}
	
	/**
	 * Get User Sales information
	 * @param number $user_id
	 */
	public function get_sales($user_id, $offset, $limit, $q, $filter, $extra = array()) {
	    $where = 'M.deleted = 0 AND M.sales_executive_id = ' . $user_id;
	    switch($filter) {
	        case 'today':
	            $where .= " AND DATE(M.created_on) = '" . date('Y-m-d') . "'";
	            break;
	        case 'week':
	            $where .= " AND (DATE(M.created_on) BETWEEN '" . date('Y-m-d', strtotime('-1 week')) . "' AND '" . date('Y-m-d') . "')";
	            break;
	        case 'month':
	            $where .= " AND (DATE(M.created_on) BETWEEN '" . date('Y-m-d', strtotime('-1 month')) . "' AND '" . date('Y-m-d') . "')";
	            break;
	    }	    
	    $module = 'orders';
	    $prefix = 'sales';
	    $sort_by = 'M.id DESC';
	    $records = array();
	    $rows = $this->gcrud->get_all_crud(0, $where, $sort_by, $limit, $offset, array('module' => $module, 'prefix' => $prefix));
	    if($rows) {
	        foreach($rows as $row) {
	            $item = json_decode($row->item ?? '', true);
	            $record = array(
	                'order_id' => $row->order_id,
	                'contact_person' => $row->display_name,
	                'amount' => currency_format($row->order_total),
	                'package_title' => $item['name'],
	                'listing_title' => $row->listing_title,
	                'status' => !empty($row->status_lang) ? lang($row->status) : $row->status,
	                'status_color' => $row->status_color,
	                'created_on' => gc_date_format($row->created_on, true)
	            );
	            $records[] = $record;
	        }
	    }
	    $total_rows = $this->gcrud->get_all_crud(1, $where, $sort_by, $limit, $offset, array('module' => $module, 'prefix' => null));	    
	    $segment_modules = array(
	        array(
	            'name' => 'all',
	            'label' => lang('tab_all')
	        ),
	        array(
	            'name' => 'today',
	            'label' => lang('tab_today')
	        ),
	        array(
	            'name' => 'week',
	            'label' => lang('tab_week')
	        ),
	        array(
	            'name' => 'month',
	            'label' => lang('tab_month')
	        ),
	    );
	    $toolbar_title = lang('gc_menu_sales');
	    return array('records' => $records, 'total_rows' => $total_rows, 'extra' => array('modules' => $segment_modules, 'searchbar' => false, 'toolbar_title' => $toolbar_title, 'redirect_url' => '/crud/update_sale/0'));
	}
	
	/**
	 * Update Transaction ID in case of Bank Transfer/PayTM/Amazon Pay/UPI
	 * @param number $id
	 * @param string $role
	 * @return boolean[]|string[]
	 */
	public function transaction_update($id, $user_id, $role = 'mobile') {
	    $this->lang->load('orders/orders');
	    $order = $this->db->select('order_id, user_id, transaction_id')->where('id', $id)->get($this->table_name)->row();
	    if(!$order || $order->user_id != $user_id) {
	        return array('success' => false);
	    }
	    if(isset($_POST['save'])) {
	        $this->form_validation->set_rules('transaction_id', 'lang:label_transaction_id', 'trim|required');
	        if ($this->form_validation->run() === TRUE) {
    	        $data = array(
    	            'transaction_id' => escape($this->input->post('transaction_id')),
    	            'status_id' => 9,
    	            'modified_on' => date('Y-m-d H:i:s'),
    	            'modified_ip' => $this->input->ip_address()
    	        );
    	        $this->db->where('id', $id)->update('orders', $data);
    	        if($role == 'mobile') {
    	            return array('success' => true, 'message' => lang('transaction_update_success'));
    	        }
	        } else if($role == 'mobile') {
	            return array('success' => false, 'message' => validation_errors());
	        }
	    }
	    if($role == 'mobile') {
	        $this->load->model('mobiles/mobiles_model');
	        $fields = array();
	        $fields[] = array('name' => 'transaction_id', 'label' => lang('label_transaction_id'), 'type' => 'text', 'rules' => array('required' => 'required', 'pattern' => '^.{4,}$'));
	        $fields = $this->mobiles_model->rest_fields($fields);
	        $record = array(
	            'success' => true,
	            'fields' => $fields,
	            'record' => array('transaction_id' => $order->transaction_id),
	            'extra' => array(
	                'name' => $order->order_id,
	                'icon' => 'create-outline',
	                'toolbar_title' => lang('transaction_update'),
	                'help' => lang('form_heading_title')
	            ),
	        );
	        return $record;
	    }
	}
	
	/**
	 * Send Invoice via Email to the User
	 * @param number $id
	 * @param string $role
	 * @return boolean[]|string[]
	 */
	public function order_email($id, $user_id, $role = 'mobile') {
	    $this->lang->load('orders/orders');
	    $user = $this->db->where('id', $user_id)->get('users')->row();
	    if(!$user) {
	        return array('success' => false);
	    }
	    $order = $this->get_order($id, 'id');
	    if(!$order || $order->user_id != $user_id) {
	        return array('success' => false);
	    }
	    if(isset($_POST['save'])) {
	        $this->form_validation->set_rules('email', 'lang:label_email', 'trim|required|valid_email');
	        if ($this->form_validation->run() === TRUE) {
	            $email = $this->input->post('email');
	            $this->db->where('id', $user_id)->update('users', array('email' => $email));
	            // send invoice as attachment
	            $order->email = $email;
	            $this->email_print($order->order_id, 'email', array('path' => FCPATH . 'assets/files/invoices', 'attach_invoice' => true, 'order' => $order));
	            if($role == 'mobile') {
	                return array('success' => true, 'message' => lang('order_email_success'));
	            }
	        } else if($role == 'mobile') {
	            return array('success' => false, 'message' => validation_errors());
	        }
	    }
	    if($role == 'mobile') {
	        $this->load->model('mobiles/mobiles_model');
	        $fields = array();
	        $fields[] = array('name' => 'email', 'label' => lang('label_email'), 'type' => 'text', 'rules' => array('required' => 'required', 'valid_email' => 'valid_email'));
	        $fields = $this->mobiles_model->rest_fields($fields);
	        $record = array(
	            'success' => true,
	            'fields' => $fields,
	            'record' => array('email' => $user->email),
	            'extra' => array(
	                'name' => $order->order_id,
	                'icon' => 'mail-outline',
	                'toolbar_title' => lang('order_email'),
	                'help' => lang('form_heading_title')
	            ),
	        );
	        return $record;
	    }
	}
	
	
	/**
	 * Save Listing Claims
	 */
	public function save_listing_claims($order_id, $listing_id, $user_id, $free = false) {
	    $this->load->model('listings/claims_model');
	    $data = array();
	    $claimed_listing = $this->listings_model->select('user_id, package_id, expires_on, featured')->find($listing_id);
	    $data['order_id'] = $order_id;
	    $data['listing_id'] = $listing_id;
	    $data['listing_expiry'] = $claimed_listing->expires_on;
	    $data['listing_featured'] = $claimed_listing->featured;
	    $data['claimed_by'] = $user_id;
	    $data['owned_by'] = $claimed_listing->user_id;
	    $data['ip'] = $this->input->ip_address();
	    $data['status'] = 0;
	    $this->claims_model->insert($data);
	}
	
	/**
	 * Auto Approve Claim if set in the Packages
	 * @param int $listing_id
	 * @param string $order_id
	 * @param int $claimed_by
	 */
	public function auto_approve_claim($package_id, $listing_id, $order_id, $claimed_by) {
	    // Check listing package claim status
	    // if 1, auto approve the claim option
	    $this->load->model('packages/packages_model');
	    $package = $this->packages_model->select('claim_status')->find($package_id);
	    if($package->claim_status == 1) {
	        /**
	         * get listing existing expires_on and featured for revert
	         */
	        $listing_info = $this->listings_model->select('featured, expires_on')->find($listing_id);
	        if(!empty($listing_info)) {
	            $this->db->where('order_id', $order_id);
	            $this->db->update($this->db->dbprefix .'listing_claims', array('listing_featured' => $listing_info->featured, 'listing_expiry' => $listing_info->expires_on, 'status' => 1));
	            
	            /**
	             * Get subscribed package from orders table for expires_on
	             * Get featured information from packages table
	             * Update orders table to set status of order to claimed
	             */
	            $sql = "SELECT * FROM " . $this->db->dbprefix . "orders O WHERE O.user_id = " . $claimed_by . " AND O.status = 1 AND (O.expires_on IS NULL OR O.expires_on = '00-00-0000' OR O.expires_on >= '". date('Y-m-d') ."')";
	            $active_membership = $this->db->query($sql)->row();
	            if(!empty($active_membership)) {
	                // get package information
	                $membership = $this->db->select('package_id')->where('order_id', $active_membership->order_id)->get('orders')->row();
	                $package_info = $this->packages_model->select('module_featured')->find($membership->package_id);
	                if($package_info) {
	                    $listing_data['user_id'] = $claimed_by;
	                    $listing_data['claimed'] = 1;
	                    $listing_data['expires_on'] = !empty($active_membership->expires_on) ? $active_membership->expires_on : NULL;
	                    $listing_data['featured'] = gc_field_value('listings', $package_info->module_featured);
	                    $this->db->where('id', $listing_id)->update('listings', $listing_data);
	                }
	            }
	        }
	    }
	}
	
	/**
	 * Update the item information after item save
	 * @param number $id
	 * @param string $action_type
	 * @param array $record
	 */
	public function after_item_save($id, $action_type = 'insert', $record = array()) {
	    $record = $this->db->where('id', $id)->get('order_items')->row();
	    if(gc_module_name($record->module_id) == 'products') {
	        $this->load->library('ecommerce');
	        $item_info = $this->ecommerce->get_product($record->row_id, false, false);
	        if($item_info) {
	            $relation_item_info = '';
	            $item_cover = !empty($item_info->cover) ? $item_info->cover : $item_info->cover;
	            if(!empty($record->relation_item_id)) {
	                $relation_item = $this->products_model->get_product_or_variant($record->relation_item_id, $record->relation_item_variant_id);
	                if($relation_item) {
	                    $relation_item_info = $relation_item->name;
	                    if(!empty($relation_item->variant)) {
	                        $relation_item_info .= ' (' . $relation_item->variant . ' ###' . $relation_item->sku . ')';
	                    }
	                }
	            }
	            $item = json_decode($record->item ?? '');
	            $item->id = $item_info->id;
	            $item->name = $item_info->name;
	            $item->slug = $item_info->slug;
	            $item->description = $item_info->description;
	            $item->relation_item_info = $relation_item_info;
	            // update the item information
	            $this->db->where('id', $id)->update('order_items', array('item' => json_encode($item)));
	        }
	    }	    
	    
	    // update the orders total, subtotal, tax and discount values in the main orders table
	    $order_items = $this->db->where('order_id', $record->order_id)->get('order_items')->result();
	    if($order_items) {
	        $order_total = 0;
	        $order_subtotal = 0;
	        $order_tax = 0;
	        $order_discount = 0;
	        foreach($order_items as $order_item) {
	            $order_total += $order_item->total;
	            $order_subtotal += $order_item->subtotal;
	            $order_tax += $order_item->tax;
	            $order_discount += $order_item->discount;
	        }
	        $data = array(
	            'order_total' => $order_total,
	            'order_subtotal' => $order_subtotal,
	            'order_tax' => $order_tax,
	            'order_discount' => $order_discount,
	        );
	        $this->db->where('order_id', $record->order_id)->update('orders', $data);
	    }
	}
	
	/**
	 * Cronjob - Order and Invoice Generation
	 * 
	 * @param number $user_id
	 * @param string $module
	 * @param number $row_id
	 * @param array $old_order_id
	 */
	public function generate_order($user_id, $module, $row_id, $old_order_id) {
	    $user = $this->gc->get('users', array('where' => 'id = ' . $user_id, 'row' => 1));
	    if(!$user) {
	        return false;
	    }
	    // get customer information from the order customers table
	    $customer_info = $this->db->where('order_id', $old_order_id)->get('order_customers')->row();
	    if(!$customer_info) {
	        return false;
	    }
	    
	    $mobile = false;
	    $payment_method_name = null;
	    
	    // clear existing cart items
	    $this->checkout_params($user_id, 'items', array(), true);
	    
	    // add item to users cart
	    $cart_extra = array('gc_order_type' => array('subscription_type' => 3, 'order_id' => $old_order_id));
	    $this->cart($row_id, 0, 1, 0, $module, true, $cart_extra, $user_id);
	    
	    // $this->checkout_params($user_id, false, array(), true);
	    $cart = $this->checkout_params($user_id);
	    
	    $checkout_items = $cart; 
	    if(!empty($checkout_items)) {
    	    $item_names = array();
    	    $order_summary = array(
    	        'order_subtotal' => 0,
    	        'order_discount' => 0,
    	        'order_tax' => 0,
    	        'order_total' => 0
    	    );
    	    foreach($checkout_items as $checkout_item) {
    	        $order_summary['order_subtotal'] += $checkout_item['subtotal'];
    	        $order_summary['order_discount'] += $checkout_item['discount'];
    	        $order_summary['order_tax'] += $checkout_item['tax'];
    	        $order_summary['order_total'] += $checkout_item['total'];
    	        $item_names[] = $checkout_item['item']['name'];
    	    }
    	    $order_summary['items'] = json_encode($item_names);
    	    
    	    // Delete inactive existing order if any unpaid by the same user for the same package
    	    // $order_id is passed in case of user is paying an existing invoice, generate a new invoice
    	    $this->delete_existing_order($user_id);
    	    
    	    // Generate a new order ID
    	    // During bulk import (User assign package) same order numbers were being generated.
    	    // To fix this issue, a four digit random number is added with the time.
    	    $order_id = gc_json_settings('order.settings', 'order_prefix') . time() . rand(1000,9999);
    	    
    	    // save order_customers
    	    $customer_data = (array) $customer_info;
    	    unset($customer_data['id']);
    	    $customer_data['order_id'] = $order_id;
    	    $this->db->insert('order_customers', $customer_data);
    	    
    	    // save order and order history
    	    $this->save_orders($order_id, $user, $payment_method_name, 1, 'insert', $order_summary, $mobile);
    	    
    	    // Save items in the ordered_items table
    	    // save them before order coupons as individual item commission will be calculated - Affiliate/Invitation
    	    $this->save_order_items($order_id, $user->id, $checkout_items);
    	    
    	    // save order coupons
    	    $this->save_order_coupons($order_id, $user, $order_summary);
    	    
    	    // save order history
    	    $this->save_order_history($order_id, 1, 1, 4);
    	    
    	    // send order generation email to the user
    	    $this->email_print($order_id, 'email', array('action_type' => 'invoice'));
	    }
	}
}